//////////////////////////////////////////////////////////////////////////////
//
// Asura_PS3_CharacterRenderer.cpp
//
// Copyright (c) 2006 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header.
#include "Rendering/Asura_PS3_CharacterRenderer.h"

// Engine independent.
#include "Animation/Asura_Animation.h"
#include "Animation/Asura_Hierarchy.h"
#include "Animation/Asura_TextureAnimation.h"
#include "Camera/Asura_Camera.h"
#include "Core/Asura_MaterialResponse_System.h"
#include "Interface/Asura_CommandConsole.h"
#include "Rendering/Asura_SortedRender.h"

// Engine platform-specific.
#include "ChunkSystem/Asura_Chunk_ResourceFile_PS3.h"
#include "Rendering/Shaders/Asura_PS3_GCM_Shader.h"
#include "Rendering/Asura_PS3_Fog.h"
#include "Rendering/Asura_PS3_FrameBuffer.h"
#include "Rendering/Asura_PS3_Graphics.h"
#include "Rendering/Asura_PS3_MaterialResponse_System.h"
#include "Rendering/Asura_PS3_RenderingStates.h"
#include "Rendering/Asura_PS3_RSXProfiling.h"
#include "Rendering/Asura_PS3_TextureManagement.h"
#include "Rendering/Asura_PS3_SPU_Skinning.h"

//Shaders definition
#include "Rendering/ShadersDefinition/VD/SmoothSkin_VD.h"
#include "Rendering/ShadersDefinition/VD/SmoothSkinShadow_VD.h"
#include "Rendering/ShadersDefinition/VD/SmoothSkin_StraightThrough_VD.h"
#include "Rendering/ShadersDefinition/VP/SmoothSkin_VP.h"
#include "Rendering/ShadersDefinition/VP/SmoothSkin_StraightThrough_VP.h"
#include "Rendering/ShadersDefinition/FP/SmoothSkin_FP.h"
#include "Rendering/ShadersDefinition/FP/SmoothSkin_Forward_FP.h"
#include "Rendering/ShadersDefinition/VP/SmoothSkinShadow_VP.h"
#include "Rendering/ShadersDefinition/FP/ClearAlpha_FP.h"


// D E B U G V A R S /////////////////////////////////////////////////////////
DEBUGVAR bool dbg_bTestDamageMaps = false;

/******************************************************************************/
/* Global Variables                                                           */
/******************************************************************************/

static Asura_TemplateHashTree< Asura_PS3_CharacterRenderer_Skin > g_xCharacterTree;

#if defined( ASURA_SUPPORT_COMMAND_CONSOLE )
static bool s_bDrawSmoothSkin	= true;
static float s_fDamageMapTest	= 0.0f; //if this is <= 0 the real damage map values will be used
#else
static const float s_fDamageMapTest	= 0.0f; //if this is <= 0 the real damage map values will be used
#endif
/******************************************************************************/
/* Asura_PS3_CharacterRenderer_Skin Class Functions                       */
/******************************************************************************/

////////////////////////////////////////////////////////////////////////////////
//
// Asura_PS3_CharacterRenderer_Skin::Asura_PS3_CharacterRenderer_Skin()
//
// Constructor
//
// Parameters:		None
// Return value:	None
//
////////////////////////////////////////////////////////////////////////////////

Asura_PS3_CharacterRenderer_Skin::Asura_PS3_CharacterRenderer_Skin()
	: m_pxStrips( NULL )
	, m_uNumStrips(0)
{
}

////////////////////////////////////////////////////////////////////////////////
//
// Asura_PS3_CharacterRenderer_Skin::~Asura_PS3_CharacterRenderer_Skin()
//
// Destructor
//
// Parameters:		None
// Return value:	None
//
////////////////////////////////////////////////////////////////////////////////

Asura_PS3_CharacterRenderer_Skin::~Asura_PS3_CharacterRenderer_Skin()
{
	m_xIndexBuffer.Shutdown();
	m_xVertexBuffer.Shutdown();

	delete[] m_pxStrips;
}

static Asura_TemplateHashTree<Asura_PS3_CharacterRenderer_Skin*> g_xSkinHashTree;

static Asura_PS3_GCM_VertexShader*			g_pxVertexShader			= NULL;
static Asura_PS3_GCM_VertexShader*			g_pxVertexShaderDM			= NULL;
static Asura_PS3_GCM_VertexShader*			g_pxStraightThroughVS		= NULL;
static Asura_PS3_GCM_VertexShader*			g_pxStraightThroughShadowVS	= NULL;
static Asura_PS3_GCM_VertexShader*			g_pxInUseVS					= NULL;

static Asura_PS3_GCM_VertexShader*			g_pxShadowVertexShader		= NULL;
static Asura_PS3_GCM_VertexShader*			g_pxZPassVertexShader		= NULL;
static Asura_PS3_FragmentShader*			g_pxFragmentShaderZ			= NULL;
Asura_PS3_MaterialResponse_Handles*			g_xMaterialResponseHandles;

static 	Asura_PS3_SPU_SkinningConstants		g_xShaderConstants;


void Asura_PS3_CharacterRenderer::Initialise(void)
{
#if defined( ASURA_SUPPORT_COMMAND_CONSOLE )
	Asura_CommandConsole::AddVar( L"Render.Enable.SmoothSkin",	&s_bDrawSmoothSkin,	Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::AddVar( L"Render.SpuSkinning.DamageMapTest",	&s_fDamageMapTest, -1.0f, 1.0f,	Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
#endif
	g_pxVertexShader			= new Asura_PS3_VP_SmoothSkin();
	g_pxVertexShaderDM			= new Asura_PS3_VP_SmoothSkinDM();
	g_pxShadowVertexShader		= new Asura_PS3_VP_SmoothSkinShadow();
	g_pxZPassVertexShader		= new Asura_PS3_VP_SmoothSkinZPass();
	g_pxStraightThroughVS		= new Asura_PS3_VP_SmoothSkin_StraightThrough();
	g_pxStraightThroughShadowVS	= new Asura_PS3_VP_SmoothSkin_StraightThroughShadow();
	g_pxFragmentShaderZ			= new Asura_PS3_FragmentShader( (unsigned char *)&_binary_PS3_DeferredPhoton_P0_FP_bin_start, (unsigned char *)&_binary_PS3_DeferredPhoton_P0_FP_bin_end, NULL );

	g_pxVertexShader->InitVertexBufferDeclaration( g_axVertexDeclaration, VD_ELEMENTS );
	g_pxShadowVertexShader->InitVertexBufferDeclaration( g_axVertexDeclarationShadow, VDS_ELEMENTS );
	g_pxStraightThroughVS->InitVertexBufferDeclaration( g_axStraightThroughVertexDeclaration, VD_STRAIGHTTHROUGH_ELEMENTS );
	g_pxStraightThroughShadowVS->InitVertexBufferDeclaration( g_axStraightThroughVertexDeclarationShadow, VDS_STRAIGHTTHROUGH_ELEMENTS );

	//BW some sanity checks for the SPU skinning code
	Asura_Assertf( sizeof(Asura_PS3_SPU_Skinning_InputVertex) == CHARACTER_VERTEX_SIZE, ("Vertex definition is wrong! SPU code depends on this!\n %d != %d", sizeof(Asura_PS3_SPU_Skinning_InputVertex), CHARACTER_VERTEX_SIZE) );
	Asura_Assertf( sizeof(Asura_PS3_SPU_Skinning_OutputVertex) == STRAIGHTTHROUGH_VERTEX_SIZE, ("SPU-skinning: output vertex size does not match declaration, this could be fatal!\n %d != %d", sizeof(Asura_PS3_SPU_Skinning_OutputVertex), STRAIGHTTHROUGH_VERTEX_SIZE) );
	Asura_Assertf( sizeof(Asura_PS3_SPU_Skinning_OutputVertex_PosOnly) == STRAIGHTTHROUGH_SHADOW_VERTEX_SIZE, ("SPU-skinning: output vertex size does not match declaration, this could be fatal!\n %d != %d", sizeof(Asura_PS3_SPU_Skinning_OutputVertex_PosOnly), STRAIGHTTHROUGH_SHADOW_VERTEX_SIZE) );
}

int Asura_PS3_CharacterRenderer::GetVertexSize( void )
{
	return CHARACTER_VERTEX_SIZE;
}


void Asura_PS3_CharacterRenderer::SetVPData( const Asura_Animation& xAnimation,
											const Asura_Hierarchy_Skin& xSkin,
											Asura_AMatrix4x4_InParam xTransform,
											Asura_AMatrix4x4_InParam xInverseTransform,
											const Asura_AMatrix4x4* const pxBones,
											bool bTransparentPass, bool bSpuVS )
{
	Asura_PS3_GCM_VertexShader*	pxVertexShader = NULL;
#ifdef ASURA_USE_DAMAGE_MAPS	
	bool bUseDamageMaps = xAnimation.IsUsingDamageMap() || s_fDamageMapTest > 0.0f;
#else
	const bool bUseDamageMaps = false;
#endif

	// Render States

	Asura_PS3_RenderingStates::RequestCullMode( ASURA_CULLMODE_CW );
	Asura_PS3_RenderingStates::RequestZBufferEnabled( true );

	if( bTransparentPass )
	{
		Asura_PS3_DeferredRendering::GetFrameBuffer()->BindAsSingleRT();
		Asura_PS3_RenderingStates::RequestAlphaTest( false );
	}
	else
	{
		Asura_PS3_DeferredRendering::GetFrameBuffer()->RestoreBind();
		Asura_PS3_RenderingStates::RequestAlphaTest( false );
	}
	
	
	// Send constants to shader or SPU
	
	if ( bSpuVS )
	{
		//SPU's are doing the vertex shading, just use the do-nothing shader
		g_pxInUseVS = g_pxStraightThroughVS;
		g_pxStraightThroughVS->Bind();
 	}
	else
	{
		//BW if SPU isn't working, or has run out of space, or something
	
		if( bUseDamageMaps )
		{
			pxVertexShader = g_pxVertexShaderDM;
		}
		else
		{
			pxVertexShader = g_pxVertexShader;
		}
		g_pxInUseVS = pxVertexShader;
		pxVertexShader->Bind();
	}
	

	if ( bSpuVS )
	{
		g_xShaderConstants.m_xWorldMatrix = xTransform;
	}
	else
	{
		pxVertexShader->SetUniformParameter( SK_WorldMatrix,  xTransform.GetElements() );
	}
	

	const Asura_AMatrix4x4& xWorldProjectionView = Asura_PS3_Graphics::GetWorldViewProjectionMatrix( &xTransform );

	if ( bSpuVS )
	{
		g_xShaderConstants.m_xWorldViewProjectionMatrix = xWorldProjectionView;
	}
	else
	{
		pxVertexShader->SetUniformParameter( SK_WorldViewProj,  xWorldProjectionView.GetElements() );
	}

	// Set bone matrices
    if( bUseDamageMaps )
    {
	    Asura_AMatrix4x4 xBones[ ASURA_MAX_NO_OF_BONES_PER_MODEL ];

	    for( int i = 0; i < xSkin.NumberOfBones; ++i )
	    {
			float fDamage;
			if ( s_fDamageMapTest > 0.0f )
			{
				fDamage = s_fDamageMapTest; //override the damage map for testing
			}
			else
			{
				fDamage = xAnimation.GetBoneDamageMapValue( i );
			}
			
		    xBones[ i ] = pxBones[ i ];
		    
		    if ( bSpuVS )
		    {
				xBones[ i ].SetElement( 3, 0, fDamage );
			}
			else
			{
				xBones[ i ].SetElement( 2, 2, fDamage );
			}

            if (dbg_bTestDamageMaps)
		    {
			    Asura_OutputToDebugger(( "Damage: %f\n", xAnimation.GetBoneDamageMapValue( i ) ));
		    }
	    }

		if ( bSpuVS )
			g_xShaderConstants.m_uBoneArrayAddr = reinterpret_cast<uint64_t>(xAnimation.GetRenderCache()->UpdateCachedBones( xBones, xSkin.NumberOfBones ));
		else
			pxVertexShader->SetUniformParameter( SK_BoneMatrices, xBones[0].GetElements(), xSkin.NumberOfBones );
    }
    else
    {
		if ( bSpuVS )
			g_xShaderConstants.m_uBoneArrayAddr = reinterpret_cast<uint64_t>(xAnimation.GetRenderCache()->UpdateCachedBones( pxBones, xSkin.NumberOfBones ));
		else
		    pxVertexShader->SetUniformParameter( SK_BoneMatrices, pxBones[0].GetElements(), xSkin.NumberOfBones );
	}

	if ( bSpuVS ) g_xShaderConstants.m_uNumBones = xSkin.NumberOfBones;


	// Skin shader needs object-space camera position
	{
		Asura_AVector4	xObjectSpaceCamPos(*Asura_Camera::GetPosition(), 1.0f);

		xObjectSpaceCamPos	= xObjectSpaceCamPos * xInverseTransform;
	
		if ( bSpuVS )
			g_xShaderConstants.m_xObjectSpaceCameraPosition = xObjectSpaceCamPos;
		else
			pxVertexShader->SetUniformParameter( SK_ObjectSpaceCamPos, xObjectSpaceCamPos.GetElements() );
	}
}



#if 0
void Asura_PS3_CharacterRenderer::RenderCharacterTransparentStripZPass(const Asura_Animation& xAnimation,
												  const Asura_Hierarchy_Skin& xSkin,
												  Asura_AMatrix4x4_InParam xTransform,
												  Asura_AMatrix4x4_InParam xInverseTransform,
												  const Asura_AMatrix4x4* const pxBones)
{
	Asura_PS3_CharacterRenderer_Skin* const pxCharacter = GetCharacter( xSkin.m_uNameHashID );
	if( !pxCharacter || pxCharacter->m_aiTransparentStrips[ 0 ] < 0 ) return;
	
	return; //BW catalin's optimisation for SS2 doesn't work on AVP, we change materials so strips can become transparent without being in m_aiTransparentStrips

//Need to finish implementation of the zPrepass not in use in this moment
#ifdef ASURA_USE_GCM_REPLAY
	cellGcmSetPerfMonPushMarker( Asura_PS3_Graphics::GetGcmCurrentContext(), "Transparent ZPrepass" );
#endif
	Asura_PS3_RSXProfiling::BeginSection( Asura_PS3_RSXProfiling::PI_SMOOTHSKIN );
	bool bFirstDrawCall = true;

	// Do transparent strips only!
	for( u_int i = 0; pxCharacter->m_aiTransparentStrips[ i ] >= 0; ++i )
	{
		u_int u = pxCharacter->m_aiTransparentStrips[ i ];
//		Asura_PS3_FragmentShaderCopy* pxShader = NULL;
#ifdef ASURA_SUPPORT_SKIN_ELEMENTS
		if (xAnimation.GetSkinElementVisibility(pxCharacter->m_pxStrips[u].m_iElementID))
#endif
		{			
			const Asura_Hash_ID uMaterialHash1 = pxCharacter->m_pxStrips[u].m_auMaterialResponseHashIDs[0];
			const Asura_MaterialResponse* const pxMaterial1 = Asura_MaterialResponse_System::GetMaterialByHashID( uMaterialHash1 );

			if( pxMaterial1 )
			{
				if( pxMaterial1->GetDontRender() ) continue;
			}

			if( bFirstDrawCall )
			{
				Asura_PS3_DeferredRendering::GetDeferredFrameBuffer()->BindAsZPassOnly();
				Asura_PS3_RenderingStates::SetColorMaskMrtState( ASURA_PS3_MRT_NONE );

				Asura_PS3_RenderingStates::SetDepthBias( -3.0f, 0.0f );
				Asura_PS3_RenderingStates::SetUseDepthBias( true );
				Asura_PS3_RenderingStates::RequestAlphaTest( false );
				Asura_PS3_RenderingStates::RequestCullMode( ASURA_CULLMODE_CW );
				Asura_PS3_RenderingStates::RequestZBufferCompareMode( ASURA_ZBUFFERCMP_LESSEQUAL );
				Asura_PS3_RenderingStates::RequestZBufferEnabled( true );
				Asura_PS3_RenderingStates::RequestZBufferWriteEnabled( true );
				Asura_PS3_RenderingStates::SetStencilTestEnabled( false );
				Asura_PS3_RenderingStates::SetStencilFunction( ASURA_PS3_STENCIL_NEVER, 0, 0 );

				g_pxZPassVertexShader->Bind();

				const Asura_AMatrix4x4& xWorldProjectionView = Asura_PS3_Graphics::GetWorldViewProjectionMatrix( &xTransform );

				g_pxZPassVertexShader->SetUniformParameter( SKZ_WorldViewProj, xWorldProjectionView.GetElements() );

				g_pxZPassVertexShader->SetUniformParameter( SKZ_BoneMatrices, pxBones[0].GetElements(), xSkin.NumberOfBones );
				pxCharacter->m_xVertexBuffer.Bind( g_axVertexDeclarationShadow, VDS_ELEMENTS );
				g_pxFragmentShaderZ->Bind();
				bFirstDrawCall = false;
			}

			pxCharacter->m_xIndexBuffer.RenderTriangles( pxCharacter->m_pxStrips[ u ].m_uNumIndices, pxCharacter->m_pxStrips[ u ].m_uStartIndex );
			Asura_PS3_Graphics::Flush();
		}
	}
	Asura_PS3_RSXProfiling::EndSection( Asura_PS3_RSXProfiling::PI_SMOOTHSKIN );
#ifdef ASURA_USE_GCM_REPLAY
	cellGcmSetPerfMonPopMarker( Asura_PS3_Graphics::GetGcmCurrentContext() );
#endif
}

void Asura_PS3_CharacterRenderer::RenderCharacterZPassOnly(const Asura_Animation& xAnimation,
												  const Asura_Hierarchy_Skin& xSkin,
												  Asura_AMatrix4x4_InParam xTransform,
												  Asura_AMatrix4x4_InParam xInverseTransform,
												  const Asura_AMatrix4x4* const pxBones)
{
//Need to finish implementation of the zPrepass not in use in this moment
#ifdef ASURA_USE_GCM_REPLAY
	cellGcmSetPerfMonPushMarker( Asura_PS3_Graphics::GetGcmCurrentContext(), "ZPrepass" );
#endif
	Asura_PS3_RSXProfiling::BeginSection( Asura_PS3_RSXProfiling::PI_SMOOTHSKIN );
	Asura_PS3_CharacterRenderer_Skin* const pxCharacter = GetCharacter( xSkin.m_uNameHashID );
	if( !pxCharacter || pxCharacter->m_aiSolidStrips[ 0 ] < 0 ) return;

	bool bFirstDrawCall = true;

	for( u_int i = 0; pxCharacter->m_aiSolidStrips[ i ] >= 0; ++i )
	{
		u_int u = pxCharacter->m_aiSolidStrips[ i ];
//		Asura_PS3_FragmentShaderCopy* pxShader = NULL;
#ifdef ASURA_SUPPORT_SKIN_ELEMENTS
		if (xAnimation.GetSkinElementVisibility(pxCharacter->m_pxStrips[u].m_iElementID))
#endif
		{			
			const Asura_Hash_ID uMaterialHash1 = pxCharacter->m_pxStrips[u].m_auMaterialResponseHashIDs[0];
			const Asura_MaterialResponse* const pxMaterial1 = Asura_MaterialResponse_System::GetMaterialByHashID( uMaterialHash1 );

			if( pxMaterial1 )
			{
				if( pxMaterial1->GetDontRender() ) continue;
			}

			if( bFirstDrawCall )
			{
				Asura_PS3_DeferredRendering::GetDeferredFrameBuffer()->BindAsZPassOnly();
				Asura_PS3_RenderingStates::SetColorMaskMrtState( ASURA_PS3_MRT_NONE );

				Asura_PS3_RenderingStates::RequestAlphaTest( false );
				Asura_PS3_RenderingStates::RequestCullMode( ASURA_CULLMODE_CW );
				Asura_PS3_RenderingStates::RequestZBufferCompareMode( ASURA_ZBUFFERCMP_LESSEQUAL );
				Asura_PS3_RenderingStates::RequestZBufferEnabled( true );
				Asura_PS3_RenderingStates::RequestZBufferWriteEnabled( true );
				Asura_PS3_RenderingStates::SetStencilTestEnabled( false );
				Asura_PS3_RenderingStates::SetStencilFunction( ASURA_PS3_STENCIL_NEVER, 0, 0 );

				g_pxZPassVertexShader->Bind();

				const Asura_AMatrix4x4& xWorldProjectionView = Asura_PS3_Graphics::GetWorldViewProjectionMatrix( &xTransform );

				g_pxZPassVertexShader->SetUniformParameter( SKZ_WorldViewProj, xWorldProjectionView.GetElements() );

				g_pxZPassVertexShader->SetUniformParameter( SKZ_BoneMatrices, pxBones[0].GetElements(), xSkin.NumberOfBones );
				pxCharacter->m_xVertexBuffer.Bind( g_axVertexDeclarationShadow, VDS_ELEMENTS );
				g_pxFragmentShaderZ->Bind();
				bFirstDrawCall = false;
			}

			pxCharacter->m_xIndexBuffer.RenderTriangles( pxCharacter->m_pxStrips[ u ].m_uNumIndices, pxCharacter->m_pxStrips[ u ].m_uStartIndex );
			Asura_PS3_Graphics::Flush();
		}
	}
#ifdef ASURA_USE_GCM_REPLAY
	cellGcmSetPerfMonPopMarker( Asura_PS3_Graphics::GetGcmCurrentContext() );
#endif
	Asura_PS3_RSXProfiling::EndSection( Asura_PS3_RSXProfiling::PI_SMOOTHSKIN );
}
#endif

void Asura_PS3_CharacterRenderer::RenderCharacterUsingZPrepass(const Asura_Animation& xAnimation,
												  const Asura_Hierarchy_Skin& xSkin,
												  Asura_AMatrix4x4_InParam xTransform,
												  Asura_AMatrix4x4_InParam xInverseTransform,
												  const Asura_AMatrix4x4* const pxBones)
{
	Asura_PS3_CharacterRenderer_Skin* const pxCharacterStrips = GetCharacter( xSkin.m_uNameHashID );
	if( !pxCharacterStrips ) return;

	Asura_PS3_RSXProfiling::BeginSection( Asura_PS3_RSXProfiling::PI_SMOOTHSKIN );

	const bool bTransparentPass = Asura_SortedRender::IsTransparentPass();

	Asura_PS3_FP_List_Tree* fpTree = Asura_PS3_GCM_ShadersLibrary::GetResettableFPTree( ASURA_SHADER_HASH_CHARACTER );

	if( bTransparentPass )
	{
#ifdef ASURA_USE_GCM_REPLAY
		cellGcmSetPerfMonPushMarker( Asura_PS3_Graphics::GetGcmCurrentContext(), "Transparent Pass" );
#endif
		bool bFirstDrawCall = true;
// 		for( u_int i = 0; pxCharacterStrips->m_aiTransparentStrips[ i ] >= 0; ++i )
// 		{
// 			u_int u = pxCharacterStrips->m_aiTransparentStrips[ i ];
		for( u_int u = 0; u < pxCharacterStrips->m_uNumStrips; ++u )
		{
			Asura_PS3_FragmentShaderCopy* pxShader = NULL;
#ifdef ASURA_SUPPORT_SKIN_ELEMENTS
			if (xAnimation.GetSkinElementVisibility(pxCharacterStrips->m_pxStrips[u].m_iElementID))
#endif
			{			
				const Asura_Hash_ID uMaterialHash1 = pxCharacterStrips->m_pxStrips[u].m_auMaterialResponseHashIDs[0];
				const Asura_MaterialResponse* const pxMaterial1 = Asura_MaterialResponse_System::GetMaterialByHashID( uMaterialHash1 );

				if( pxMaterial1 )
				{
					if( pxMaterial1->GetDontRender() )
					{
						continue;
					}
				}

				const Asura_Hash_ID uMaterialHash2 = pxCharacterStrips->m_pxStrips[u].m_auMaterialResponseHashIDs[1];
				const Asura_MaterialResponse* const pxMaterial2 = Asura_MaterialResponse_System::GetMaterialByHashID( uMaterialHash2 );
				const Asura_TextureAnimationController* pxTextureAnimationController = xAnimation.GetTextureAnimController();

				if ( pxMaterial1->IsTransparent() ) //BW catalin's optimisation for SS2 doesn't work on AVP, we have to handle transparent strips here
				{
					Asura_Hash_ID uHashID = pxCharacterStrips->m_pxStrips[ u ].m_uFPHashID;

					if( Asura_MaterialResponse_System::GetCurrentOverrideMapHash() != ASURA_HASH_ID_UNSET )
					{
						uHashID = Asura_PS3_MaterialResponse_System::CreateShaderHashID( pxMaterial1, pxMaterial2 );
					}

					if( Asura_PS3_Fog::GetFogParameters().m_bFogEnabled )
					{
						uHashID += ASURA_PS3_SHADER_FOG;
					}

					pxShader = fpTree->GetShader( uHashID );
					if( !pxShader )
					{
						Asura_Assert( false, "Error: No shader set for this strip" );
						continue;
					}


					if( bFirstDrawCall )
					{
						Asura_PS3_RenderingStates::RequestZBufferCompareMode( ASURA_ZBUFFERCMP_LESSEQUAL );
						Asura_PS3_RenderingStates::RequestZBufferWriteEnabled( false );
						SetVPData( xAnimation, xSkin, xTransform, xInverseTransform, pxBones, bTransparentPass, false );
						Asura_PS3_RenderingStates::SetColorMaskMrtState( ASURA_PS3_MRT_MRT0_ALL );
						Asura_PS3_RenderingStates::RequestCullMode( ASURA_CULLMODE_CW );
						bFirstDrawCall = false;
					}
					Asura_TextureAnimationOutput xTexAnim1, xTexAnim2;

					Asura_TextureAnimation_System::GetAnimationState( pxMaterial1->GetTextureAnim(), pxTextureAnimationController, &xTexAnim1 );
					Asura_TextureAnimation_System::GetAnimationState( pxMaterial2 ? pxMaterial2->GetTextureAnim() : ASURA_HASH_ID_UNSET, pxTextureAnimationController, &xTexAnim2 );

					const float afTexCoordOffset[4] = { xTexAnim1.m_fUOffset, xTexAnim1.m_fVOffset, xTexAnim2.m_fUOffset, xTexAnim2.m_fVOffset };
					g_pxInUseVS->SetUniformParameter( SK_UV_Offsets, afTexCoordOffset );

					Asura_PS3_MaterialResponse_System::SetBlendStates( pxMaterial1 );
					Asura_PS3_MaterialResponse_System::SetEffectParameters( pxMaterial1, pxMaterial2, pxTextureAnimationController, pxShader );
					Asura_PS3_MaterialResponse_System::SetSHParameters( pxMaterial1, pxShader );
					Asura_PS3_MaterialResponse_System::SetProjectContext( pxMaterial1, pxMaterial2 );
					
					pxShader->Bind();

					pxCharacterStrips->m_xVertexBuffer.Bind( g_axVertexDeclaration, VD_ELEMENTS );
					pxCharacterStrips->m_xIndexBuffer.RenderTriangles( pxCharacterStrips->m_pxStrips[ u ].m_uNumIndices, pxCharacterStrips->m_pxStrips[ u ].m_uStartIndex );
					Asura_PS3_Graphics::Flush();
				
				}
			}
		}
#ifdef ASURA_USE_GCM_REPLAY
		cellGcmSetPerfMonPopMarker( Asura_PS3_Graphics::GetGcmCurrentContext() );
#endif
	}
	else
	{
#ifdef ASURA_USE_GCM_REPLAY
		cellGcmSetPerfMonPushMarker( Asura_PS3_Graphics::GetGcmCurrentContext(), "Solid Pass" );
#endif
		bool bFirstDrawCall = true;
		for( u_int u = 0; u < pxCharacterStrips->m_uNumStrips; ++u )
		{
			Asura_PS3_FragmentShaderCopy* pxShader = NULL;
#ifdef ASURA_SUPPORT_SKIN_ELEMENTS
			if (xAnimation.GetSkinElementVisibility(pxCharacterStrips->m_pxStrips[u].m_iElementID))
#endif
			{			
				const Asura_Hash_ID uMaterialHash1 = pxCharacterStrips->m_pxStrips[u].m_auMaterialResponseHashIDs[0];
				const Asura_MaterialResponse* const pxMaterial1 = Asura_MaterialResponse_System::GetMaterialByHashID( uMaterialHash1 );

				if( pxMaterial1 )
				{
					const bool bTransparentStrip = pxMaterial1->IsTransparent();
					if( ( pxMaterial1->GetDontRender() ) || ( bTransparentStrip != bTransparentPass ) )
					{
						continue;
					}
				}

				const Asura_Hash_ID uMaterialHash2 = pxCharacterStrips->m_pxStrips[u].m_auMaterialResponseHashIDs[1];
				const Asura_MaterialResponse* const pxMaterial2 = Asura_MaterialResponse_System::GetMaterialByHashID( uMaterialHash2 );
				const Asura_TextureAnimationController* pxTextureAnimationController = xAnimation.GetTextureAnimController();

				Asura_Hash_ID uHashID = pxCharacterStrips->m_pxStrips[ u ].m_uFPHashID;
				if( Asura_MaterialResponse_System::GetCurrentOverrideMapHash() != ASURA_HASH_ID_UNSET )
				{
					uHashID = Asura_PS3_MaterialResponse_System::CreateShaderHashID( pxMaterial1, pxMaterial2 );
				}
				pxShader = fpTree->GetShader( uHashID );
				if( !pxShader )
				{
					Asura_Assert( false, "Error: No shader set for this strip" );
					continue;
				}

				if( bFirstDrawCall )
				{
					Asura_PS3_RenderingStates::RequestZBufferCompareMode(ASURA_ZBUFFERCMP_LESSEQUAL);
					SetVPData( xAnimation, xSkin, xTransform, xInverseTransform, pxBones, bTransparentPass, false );
					Asura_PS3_RenderingStates::SetColorMaskMrtState( ASURA_PS3_MRT_MRT0123_ALL );
					Asura_PS3_RenderingStates::RequestCullMode( ASURA_CULLMODE_CW );
					bFirstDrawCall = false;
					pxCharacterStrips->m_xVertexBuffer.Bind( g_axVertexDeclaration, VD_ELEMENTS );
				}
				Asura_TextureAnimationOutput xTexAnim1, xTexAnim2;

				Asura_TextureAnimation_System::GetAnimationState( pxMaterial1->GetTextureAnim(), pxTextureAnimationController, &xTexAnim1 );
				Asura_TextureAnimation_System::GetAnimationState( pxMaterial2 ? pxMaterial2->GetTextureAnim() : ASURA_HASH_ID_UNSET, pxTextureAnimationController, &xTexAnim2 );

				const float afTexCoordOffset[4] = { xTexAnim1.m_fUOffset, xTexAnim1.m_fVOffset, xTexAnim2.m_fUOffset, xTexAnim2.m_fVOffset };
				g_pxInUseVS->SetUniformParameter( SK_UV_Offsets, afTexCoordOffset );

				if( pxMaterial1 && pxMaterial1->GetTexture( Asura_MaterialResponse::TRANSPARENCY_MAP ) != ASURA_TEXTURE_NONE )
				{
					Asura_PS3_RenderingStates::RequestZBufferWriteEnabled( true );
				}
				else
				{
					Asura_PS3_RenderingStates::RequestZBufferWriteEnabled( false );
				}

				Asura_PS3_MaterialResponse_System::SetBlendStates( pxMaterial1 );
				Asura_PS3_MaterialResponse_System::SetEffectParameters( pxMaterial1, pxMaterial2, pxTextureAnimationController, pxShader );
				Asura_PS3_MaterialResponse_System::SetSHParameters( pxMaterial1, pxShader );
				Asura_PS3_MaterialResponse_System::SetProjectContext( pxMaterial1, pxMaterial2 );
				pxShader->Bind();

				pxCharacterStrips->m_xIndexBuffer.RenderTriangles( pxCharacterStrips->m_pxStrips[ u ].m_uNumIndices, pxCharacterStrips->m_pxStrips[ u ].m_uStartIndex );
				Asura_PS3_Graphics::Flush();
			}
		}
#ifdef ASURA_USE_GCM_REPLAY
		cellGcmSetPerfMonPopMarker( Asura_PS3_Graphics::GetGcmCurrentContext() );
#endif
	}
	Asura_PS3_RenderingStates::SetUseDepthBias( false );

	Asura_PS3_RSXProfiling::EndSection( Asura_PS3_RSXProfiling::PI_SMOOTHSKIN );
}


const Asura_Animation* g_pTempDebugLastAnim = NULL;
const Asura_PS3_CharacterRenderer_Skin* g_pTempDebugLastCharSkin = NULL;
void Asura_PS3_CharacterRenderer::TempDebugCallback() //BWTEMP
{
	// Dump Cached Indices
	Asura_PS3_GCM_IndexBuffer *pIndexBuffer = g_pTempDebugLastAnim->GetRenderCache()->ObtainIndexBuffer(0);
	u_int uNumVerts = g_pTempDebugLastAnim->GetRenderCache()->ObtainVertexBuffer(0,0)->GetNumVerts();

	Asura_OutputToDebugger_Retail(("Checking Character's Indices\n"));
	
	u_int uErrors = 0, uffffs = 0;
	const u_short* pIndices = (const u_short*)pIndexBuffer->GetReadPtr();
	const u_short* pRefIndices = (const u_short*)g_pTempDebugLastCharSkin->GetIndexBuffer()->GetReadPtr();
	for ( int i=0; i < (int)pIndexBuffer->GetIndexBufferSize()/2; i++ )
	{
		if ( pIndices[i] == 0xffff )
		{
			//Asura_OutputToDebugger_Retail(("pIndices[%d] is 0xffff\n", i));
			uffffs++;
		}
		else
		{
			if ( pIndices[i] >= uNumVerts )
			{
				Asura_OutputToDebugger_Retail(("pIndices[%d/%d] is out of range %04hx >= %04x\n", i, pIndexBuffer->GetIndexBufferSize()/2, pIndices[i], uNumVerts ));
				uErrors++;
			}

// now we're doing culling this is supposed to change!
// 			if ( pRefIndices[i] != pIndices[i] )
// 			{
// 				Asura_OutputToDebugger_Retail(("Mismatch pRefIndices[%d](%04hx) != pIndices[%d](%04hx)\n", i, pRefIndices[i], i, pIndices[i] ));
// 				uErrors+=1000000;
// 			}
		}
	}

	Asura_OutputToDebugger_Retail(("Check Finished (%d errors found, %.2f%% 0xffff)\n", uErrors, (100.0f*(float)uffffs)/((float)pIndexBuffer->GetIndexBufferSize()/2)));
	Asura_Assert( uErrors==0, "errors\n" );
}

void Asura_PS3_CharacterRenderer::RenderCharacter(const Asura_Animation& xAnimation,
													  const Asura_Hierarchy_Skin& xSkin,
													  Asura_AMatrix4x4_InParam xTransform,
													  Asura_AMatrix4x4_InParam xInverseTransform,
													  const Asura_AMatrix4x4* const pxBones,
													  u_int uSkinIdx)
{
#if defined( ASURA_SUPPORT_COMMAND_CONSOLE )
	if( !s_bDrawSmoothSkin ) return;
#endif
	Asura_PS3_CharacterRenderer_Skin* const pxCharacterStrips = GetCharacter( xSkin.m_uNameHashID );
	if( !pxCharacterStrips ) return;
#if 0//def ASURA_USE_ZPREPASS
	if( Asura_Render::IsZPrepass() )
	{
		RenderCharacterZPassOnly( xAnimation, xSkin, xTransform, xInverseTransform, pxBones );
	}
	else
	{
		RenderCharacterUsingZPrepass( xAnimation, xSkin, xTransform, xInverseTransform, pxBones );
	}
#else //#ifdef ASURA_USE_ZPREPASS
	const bool bTransparentPass = Asura_SortedRender::IsTransparentPass();
	if ( bTransparentPass )
	{
//		RenderCharacterTransparentStripZPass( xAnimation, xSkin, xTransform, xInverseTransform, pxBones );
#ifdef ASURA_USE_ZPREPASS
		RenderCharacterUsingZPrepass( xAnimation, xSkin, xTransform, xInverseTransform, pxBones );
#endif
#ifdef ASURA_USE_GCM_REPLAY
		cellGcmSetPerfMonPopMarker( Asura_PS3_Graphics::GetGcmCurrentContext() );
#endif
		return;
	}
#if 0 //def ASURA_USE_ZPREPASS
	if( !Asura_Render::IsZPrepass() ) return;
#endif
    Asura_PS3_RSXProfiling::BeginSection( Asura_PS3_RSXProfiling::PI_SMOOTHSKIN );


	// Decide if we're going to use the vertex shader or SPU for rendering
	bool bSpuVS = Asura_PS3_SPU_Skinning::IsSpuable();

	// fill vertex program constants
	Asura_PS3_RenderingStates::RequestZBufferCompareMode(ASURA_ZBUFFERCMP_LESSEQUAL);		
	SetVPData( xAnimation, xSkin, xTransform, xInverseTransform, pxBones, bTransparentPass, bSpuVS );

	Asura_PS3_GCM_VertexBuffer *pProcessedVerts = NULL;
	if ( bSpuVS )
	{
		u_int uNumVerts = pxCharacterStrips->m_xVertexBuffer.GetNumVerts();

		//make the cache big enough for the highest LOD
		//BW otherwise there will be a lot of allocing, though that would give better memory usage if it weren't for the fragmentation.
		u_int uMaxVerts = uNumVerts; //BW todo GetCharacter( topskin.hash )->m_xVertexBuffer.GetNumVerts();
		pProcessedVerts = xAnimation.GetRenderCache()->ObtainVertexBuffer( 0, uSkinIdx, uMaxVerts, sizeof(Asura_PS3_SPU_Skinning_OutputVertex) );

		//
		// SPU vertex processing
		//
		if ( !xAnimation.GetRenderCache()->IsVertexCacheCurrent( 0, uSkinIdx ) ) //BW at present we only do this once per frame so this will always go (n.b. at present SPU skinning depends on us only doing this once per character per frame)
		{
			bool bRet = Asura_PS3_SPU_Skinning::ProcessSkin( pxCharacterStrips, &g_xShaderConstants, xAnimation.GetRenderCache(), 0, uSkinIdx, uNumVerts, SPU_SKINNING_FULL_CULL );

			//BW conceivably this could be handled, e.g. if we run out of memory, but that's probably fatal anyway.
			Asura_Assert( bRet, "RenderCharacter: ProcessSkin() != IsSpuable(), this is probably fatal" );

			xAnimation.GetRenderCache()->SetVertexCacheCurrent( 0, uSkinIdx );
		}
	}


	Asura_PS3_GCM_IndexBuffer *pIndexBuffer = NULL;

	if ( bSpuVS )
	{
		//use the verts processed by SPU
		pProcessedVerts->Bind( g_axStraightThroughVertexDeclaration, VD_STRAIGHTTHROUGH_ELEMENTS );
		
		pIndexBuffer = xAnimation.GetRenderCache()->ObtainIndexBuffer(uSkinIdx);

		Asura_Assert( pIndexBuffer, "RenderCharacter: pIndexBuffer is NULL" );
		Asura_Assert( pIndexBuffer->GetIndexBufferSize() >= pxCharacterStrips->m_xIndexBuffer.GetIndexBufferSize(), "Cached buffer num indices too small" );

		g_pTempDebugLastAnim = &xAnimation;
		g_pTempDebugLastCharSkin = pxCharacterStrips;
	}
	else
	{
		//failed to setup SPU processing, do it using a vertex shader
		pxCharacterStrips->m_xVertexBuffer.Bind( g_axVertexDeclaration, VD_ELEMENTS );

		pIndexBuffer = &pxCharacterStrips->m_xIndexBuffer;
	}


	Asura_PS3_FP_List_Tree* fpTree = Asura_PS3_GCM_ShadersLibrary::GetResettableFPTree( ASURA_SHADER_HASH_CHARACTER );
	if( !fpTree ) return;

	for( u_int u = 0; u < pxCharacterStrips->m_uNumStrips; ++u )
	{
		Asura_PS3_FragmentShaderCopy* pxShader = NULL;
#ifdef ASURA_SUPPORT_SKIN_ELEMENTS
		if (xAnimation.GetSkinElementVisibility(pxCharacterStrips->m_pxStrips[u].m_iElementID))
#endif
		{			
			const Asura_Hash_ID uMaterialHash1 = pxCharacterStrips->m_pxStrips[u].m_auMaterialResponseHashIDs[0];
			const Asura_MaterialResponse* const pxMaterial1 = Asura_MaterialResponse_System::GetMaterialByHashID( uMaterialHash1 );

			if( pxMaterial1 )
			{
				const bool bTransparentStrip = pxMaterial1->IsTransparent();
				if( ( pxMaterial1->GetDontRender() ) || ( bTransparentStrip != bTransparentPass ) )
				{
					continue;
				}
			}

			const Asura_Hash_ID uMaterialHash2 = pxCharacterStrips->m_pxStrips[u].m_auMaterialResponseHashIDs[1];
			const Asura_MaterialResponse* const pxMaterial2 = Asura_MaterialResponse_System::GetMaterialByHashID( uMaterialHash2 );
			const Asura_TextureAnimationController* pxTextureAnimationController = xAnimation.GetTextureAnimController();

			Asura_Hash_ID uHashID = pxCharacterStrips->m_pxStrips[ u ].m_uFPHashID;
			if( Asura_MaterialResponse_System::GetCurrentOverrideMapHash() != ASURA_HASH_ID_UNSET )
			{
				uHashID = Asura_PS3_MaterialResponse_System::CreateShaderHashID( pxMaterial1, pxMaterial2 );
			}
			if( bTransparentPass && Asura_PS3_Fog::GetFogParameters().m_bFogEnabled )
			{
				uHashID += ASURA_PS3_SHADER_FOG;
			}

			pxShader = fpTree->GetShader( uHashID );
			if( !pxShader )
			{
				Asura_Assert( false, "Error: No shader set for this strip" );
				continue;
			}

			Asura_PS3_MaterialResponse_System::SetBlendStates( pxMaterial1 );

			Asura_PS3_MaterialResponse_System::SetEffectParameters( pxMaterial1, pxMaterial2, pxTextureAnimationController, pxShader );
			Asura_PS3_MaterialResponse_System::SetSHParameters( pxMaterial1, pxShader );
			Asura_PS3_MaterialResponse_System::SetProjectContext( pxMaterial1, pxMaterial2 );

			Asura_TextureAnimationOutput xTexAnim1, xTexAnim2;

			Asura_TextureAnimation_System::GetAnimationState( pxMaterial1->GetTextureAnim(), pxTextureAnimationController, &xTexAnim1 );
			Asura_TextureAnimation_System::GetAnimationState( pxMaterial2 ? pxMaterial2->GetTextureAnim() : ASURA_HASH_ID_UNSET, pxTextureAnimationController, &xTexAnim2 );

			const float afTexCoordOffset[4] = { xTexAnim1.m_fUOffset, xTexAnim1.m_fVOffset, xTexAnim2.m_fUOffset, xTexAnim2.m_fVOffset };
			if ( !bSpuVS ) g_pxInUseVS->SetUniformParameter( SK_UV_Offsets, afTexCoordOffset );
			pxShader->Bind();

			pIndexBuffer->RenderTriangles( pxCharacterStrips->m_pxStrips[ u ].m_uNumIndices, pxCharacterStrips->m_pxStrips[ u ].m_uStartIndex );
			Asura_PS3_Graphics::Flush();
		}
	}

	if ( bSpuVS )
	{
		Asura_PS3_SPU_Skinning::NotifyBufferUsed();
	}

	Asura_PS3_RSXProfiling::EndSection( Asura_PS3_RSXProfiling::PI_SMOOTHSKIN );
#endif //#ifdef ASURA_USE_ZPREPASS
}

void Asura_PS3_CharacterRenderer::RenderShadow(const Asura_Animation& xAnimation,
												   const Asura_Hierarchy_Skin& xSkin,
												   const Asura_Shadow& xShadow,
												   Asura_AMatrix4x4_InParam xTransform,
												   const Asura_AMatrix4x4* const pxBones,
												   u_int uSkinIdx )
{
	Asura_PS3_CharacterRenderer_Skin* const pxCharacterStrips = GetCharacter( xSkin.m_uNameHashID );
	if( !pxCharacterStrips ) return;

	// Decide if we're going to use the vertex shader or SPU for rendering
	bool bSpuVS = Asura_PS3_SPU_Skinning::IsSpuable();

	Asura_PS3_RenderingStates::RequestZBufferWriteEnabled( true );
	Asura_PS3_RenderingStates::RequestZBufferEnabled( true );
	Asura_PS3_RenderingStates::RequestZBufferCompareMode(ASURA_ZBUFFERCMP_LESSEQUAL);		
	Asura_PS3_RenderingStates::RequestTranslucencyMode( ASURA_TRANSLUCENCY_OFF );
	Asura_PS3_RenderingStates::SetBlendEnableMrt( false, false, false );
	Asura_PS3_RenderingStates::SetColorMaskMrtState( ASURA_PS3_MRT_MRT0_ALL );
	Asura_PS3_RenderingStates::SetUseDepthBias( false );
	Asura_PS3_RenderingStates::RequestCullMode( ASURA_CULLMODE_CW );

	Asura_AMatrix4x4 xWorldProjectionView;
	Asura_AMatrix4x4 xObjectMatrix;

	xObjectMatrix = xTransform;

	Asura_Matrix_3x3 xLightOri;
	xShadow.GetLightOrientation(xLightOri);
	const Asura_Vector_3& xLightPos = xShadow.GetLightPosition();

	Asura_AMatrix4x4 xPerspective;
	Asura_AMatrix4x4 xLight( xLightOri, xLightPos );
	xLight.InvertOrthogonal();

	xPerspective.SetPerspectiveForLH( Asura_Maths::DegToRad( xShadow.GetFrustum().GetVerticalFOVInDegrees() ), 1.0f, xShadow.GetFrustum().GetNearPlane(), xShadow.GetFrustum().GetFarPlane() );

	xWorldProjectionView = xObjectMatrix * xLight * xPerspective;

	if ( bSpuVS )
	{
		g_pxStraightThroughShadowVS->Bind();

//BW moved worldviewproj to vertex shader, so can handle multiple shadows with 1 buffer
// 		g_xShaderConstants.m_xWorldViewProjectionMatrix = xWorldProjectionView;
		g_pxStraightThroughShadowVS->SetUniformParameter( VS_TRANSFORM_FLOAT4_Transform, xWorldProjectionView.GetElements() );

		if ( !xAnimation.GetRenderCache()->IsVertexCacheCurrent(1,uSkinIdx) ) // SPU has already skinned this character for shadows
		{
			if ( xAnimation.GetRenderCache()->IsBoneCacheCurrent() ) // was the main character rendered this frame? we want to reuse the cached bones.
			{
				// the bones have already been cached when the character was drawn
				// n.b. it's important that we don't overwrite the cached bones, since the SPUs read them asyncronously and the character render stores some extra data in there
				Asura_Assert( xAnimation.GetRenderCache()->GetNumCachedBones() == static_cast<u_int>(xSkin.NumberOfBones), "Number of bones for shadow is different to cached number" );
			}
			else
			{
				// cache the bones now
				xAnimation.GetRenderCache()->UpdateCachedBones( pxBones, xSkin.NumberOfBones );
			}

			g_xShaderConstants.m_uBoneArrayAddr = reinterpret_cast<uint64_t>(xAnimation.GetRenderCache()->GetCachedBones());
			g_xShaderConstants.m_uNumBones = xSkin.NumberOfBones;
		}
	}
	else
	{
		g_pxShadowVertexShader->Bind();

		g_pxShadowVertexShader->SetUniformParameter( SKS_WorldViewProj, xWorldProjectionView.GetElements() );

		// Set bone matrices
		g_pxShadowVertexShader->SetUniformParameter( SKS_BoneMatrices, pxBones[0].GetElements(), xSkin.NumberOfBones );
	}


//BW unused?
//     // Frustum parameters
// 
// 	const float fShadowFrustumNearPlane = xShadow.GetFrustum().GetNearPlane();
// 	const float fOneOverShadowFrustumDepth = 1.0f / (xShadow.GetFrustum().GetFarPlane() - fShadowFrustumNearPlane);


	Asura_PS3_GCM_VertexBuffer *pProcessedVerts = NULL;
	if ( bSpuVS )
	{
		u_int uNumVerts = pxCharacterStrips->m_xVertexBuffer.GetNumVerts();

		//make the cache big enough for the highest LOD
		//BW otherwise there will be a lot of allocing, though that would give better memory usage if it weren't for the fragmentation.
		u_int uMaxVerts = uNumVerts; //BW todo
		pProcessedVerts = xAnimation.GetRenderCache()->ObtainVertexBuffer( 1, uSkinIdx, uMaxVerts, sizeof(Asura_PS3_SPU_Skinning_OutputVertex_PosOnly) );

		//
		// SPU vertex processing
		//
		if ( !xAnimation.GetRenderCache()->IsVertexCacheCurrent( 1, uSkinIdx ) )
		{
			bool bRet = Asura_PS3_SPU_Skinning::ProcessSkin( pxCharacterStrips, &g_xShaderConstants, xAnimation.GetRenderCache(), 1, uSkinIdx, uNumVerts, SPU_SKINNING_POS_ONLY_NO_CULL );

			//BW conceivably this could be handled, e.g. if we run out of memory, but that's probably fatal anyway.
			Asura_Assert( bRet, "RenderCharacter: ProcessSkin() != IsSpuable(), this is probably fatal" );

			xAnimation.GetRenderCache()->SetVertexCacheCurrent( 1, uSkinIdx );
		}

		//use the verts processed by SPU
		pProcessedVerts->Bind( g_axStraightThroughVertexDeclarationShadow, VDS_STRAIGHTTHROUGH_ELEMENTS );
	}
	else
	{
		//failed to setup SPU processing, do it using a vertex shader
 		pxCharacterStrips->m_xVertexBuffer.Bind( g_axVertexDeclarationShadow, VDS_ELEMENTS );
	}

	u_int uStartIndex = 0;
	for( u_int u = 0; u < pxCharacterStrips->m_uNumStrips; ++u )
	{
#ifdef ASURA_SUPPORT_SKIN_ELEMENTS
		if (xAnimation.GetSkinElementVisibility(pxCharacterStrips->m_pxStrips[u].m_iElementID))
#endif
		{
			const Asura_Hash_ID uHash = pxCharacterStrips->m_pxStrips[u].m_auMaterialResponseHashIDs[0];
			const Asura_MaterialResponse* const pxMaterial = Asura_MaterialResponse_System::GetMaterialByHashID( uHash );

			if( pxMaterial && !pxMaterial->IsTransparent() && (pxMaterial->GetRenderType() != Asura_MaterialResponse::RENDER_TYPE_RENDER_BUT_DONT_CAST_SHADOW) )
			{
				pxCharacterStrips->m_xIndexBuffer.RenderTriangles( pxCharacterStrips->m_pxStrips[ u ].m_uNumIndices, uStartIndex );
				
				Asura_PS3_Graphics::Flush();
			}
		}
		uStartIndex += pxCharacterStrips->m_pxStrips[u].m_uNumIndices;
	}

	if ( bSpuVS )
	{
		Asura_PS3_SPU_Skinning::NotifyBufferUsed();
	}
}


Asura_PS3_CharacterRenderer_Skin* Asura_PS3_CharacterRenderer::GetCharacter( const Asura_Hash_ID uHash )
{
	Asura_PS3_CharacterRenderer_Skin* const pxSkin = g_xCharacterTree.GetDataByHash( uHash );
	Asura_Assert( pxSkin, "Failed to get character data." );

	return pxSkin;
}
////////////////////////////////////////////////////////////////////////////////
//
// Asura_PS3_CharacterRenderer::IsCharacterOpaque()
//
// Queries whether a character is entirely opaque
//
// Parameters:		uHash - Hash ID of character name
// Return value:	true if all strips are opaque
//
////////////////////////////////////////////////////////////////////////////////

bool Asura_PS3_CharacterRenderer::IsCharacterOpaque( const Asura_Hash_ID uHash )
{
	Asura_PS3_CharacterRenderer_Skin* const pxSkin = GetCharacter( uHash );
	if( !pxSkin ) return false;

	for( u_int u = 0; u < pxSkin->m_uNumStrips; ++u )
	{
		const Asura_Hash_ID uMaterialResponseHashID = pxSkin->m_pxStrips[u].m_auMaterialResponseHashIDs[0];
		const Asura_MaterialResponse* const pxMaterial = Asura_MaterialResponse_System::GetMaterialByHashID( uMaterialResponseHashID );

		// Decal-type materials count as opaque
		if( pxMaterial && pxMaterial->IsTransparent() )
		{
			return false;
		}
	}

	return true;
}

void Asura_PS3_CharacterRenderer::Reset()
{
	g_xCharacterTree.Reset();
}

void Asura_PS3_CharacterRenderer::Shutdown(void)
{
    Reset();

	delete g_pxVertexShader;
	delete g_pxVertexShaderDM;
	delete g_pxShadowVertexShader;
	delete g_pxZPassVertexShader;
	delete g_pxStraightThroughVS;
	delete g_pxFragmentShaderZ;

	g_pxVertexShader			= NULL;
	g_pxVertexShaderDM			= NULL;
	g_pxShadowVertexShader		= NULL;
	g_pxZPassVertexShader		= NULL;
	g_pxStraightThroughVS		= NULL;
	g_pxFragmentShaderZ			= NULL;
}

////////////////////////////////////////////////////////////////////////////////
//
// Asura_PS3_CharacterRenderer::CreateCharacter()
//
// Creates a new PS3 character. Called by the chunk loader
//
// Parameters:		None
// Return value:	None
//
////////////////////////////////////////////////////////////////////////////////

Asura_PS3_CharacterRenderer_Skin* Asura_PS3_CharacterRenderer::CreateCharacter( const Asura_Hash_ID uHash, const u_int uNumStrips )
{
	#if defined ASURA_ASSERT
		const Asura_PS3_CharacterRenderer_Skin* const pxExistingCharacter = g_xCharacterTree.GetDataByHash( uHash );
		Asura_Assertf( !pxExistingCharacter, ("A character with this hash (0x%X) has already been created.", uHash) );
	#endif

	Asura_PS3_CharacterRenderer_Skin* const pxNewCharacter = g_xCharacterTree.AddNode( uHash );
	if( !pxNewCharacter )
	{
		Asura_Assert( false, "Unable to create new character." );
		return NULL;
	}

	pxNewCharacter->m_pxStrips = new Asura_PS3_CharacterRenderer_Strip[ uNumStrips ];
	if( !pxNewCharacter->m_pxStrips )
	{
		Asura_Assert( pxNewCharacter->m_pxStrips, "Unable to allocate character materials." );
		return NULL;
	}

	pxNewCharacter->m_uNumStrips = uNumStrips;

	return pxNewCharacter;
}
