//
// Asura_PS3_SPU_SkinningTypes.h
//
// Author: Ben Weston 30/5/2008
//
// Types shared by SPU and PPU code, be careful with these definitions they need to be consistent between both processors & correspond to the data from the tools
//

#if defined ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __ASURA_PS3_SPU_SKINNINGTYPES_H__
#define __ASURA_PS3_SPU_SKINNINGTYPES_H__

#include "Core/Asura_Maths.h"

typedef u_int vec11_11_10;
typedef uint64_t half4;

//from SmoothSkin_VD.h
//BW todo: compress the input vertices & possibly interlace so do 4 at once, SPUs are quicker that way.
struct Asura_PS3_SPU_Skinning_InputVertex
{
	Asura_AVector4 m_xPosition;
	u_short m_xUV[4];
	u_char m_aiBoneIndices[4];
	u_short m_afBoneWeights[4];
	vec11_11_10 m_xNormal;
	vec11_11_10 m_xTangent;
	vec11_11_10 m_xBinormal;
};

struct Asura_PS3_SPU_Skinning_OutputVertex_PosOnly
{
	Asura_AVector4 m_xPosition;
};

//from SmoothSkin_StraightThrough_VD.h
struct Asura_PS3_SPU_Skinning_OutputVertex
{
	float m_xPosition[4];
	u_short m_xUVs[4];
	float m_xCameraDirDistance[4];
	float m_xTangentSpaceCameraDir[4];
	vec11_11_10 m_xNormal;
	vec11_11_10 m_xTangent;
	vec11_11_10 m_xBinormal;
};

//BW todo: use something like this small vertex:
//notes:
// read UV from input vertex
struct Asura_PS3_SPU_Skinning_OutputVertex_Small
{
	u_short m_xPosition[4];	 //only accurate enough for screen position, not world pos
	u_short m_xCameraDirDistance[4];
	u_short m_xTangentSpaceCameraDir[4];
	vec11_11_10 m_xNormal;
	vec11_11_10 m_xTangent;
	vec11_11_10 m_xBinormal;
};

// "Constants" - values that are used for all vertices being processed, equivalent to the constants used by a vertex program
struct Asura_PS3_SPU_SkinningConstants
{
	Asura_AMatrix4x4	m_xWorldViewProjectionMatrix;
	Asura_AMatrix4x4	m_xWorldMatrix;
	Asura_AVector4		m_xObjectSpaceCameraPosition;
	Asura_AVector4		m_xUV_Offsets;
	uint64_t			m_uBoneArrayAddr; // pointer to bone array that's *guaranteed not to change for 1 frame*
	u_int				m_uNumBones;
	u_int				m_auPad[1];	 // containing stucture must be a multiple of 16 bytes, not sure how to concatenate these pads
};

//rather than having multiple SPU executables, just use an enum to say what type of shading we need.
enum SPU_SKINNING_MODE {
	SPU_SKINNING_FULL_CULL,
	SPU_SKINNING_POS_ONLY_NO_CULL	//for shadows
};

// SPU batches, a strip's indices & corresponding verts. Caller must ensure batches fit within SPU RAM (at tools time)
class Asura_PS3_SPU_Skinning_Batch
{
public:
	uint64_t	m_uVertAddr;
	uint64_t	m_uStartIndexAddr;
	uint64_t	m_uCommand; // write the new num indices to the command buffer
	u_int		m_uNumIndices;
	u_int		m_uNumVerts;
};

// SPU task info
class Asura_PS3_SPU_SkinningJob
{
public:
	Asura_PS3_SPU_SkinningJob( u_int uNumVerts=0 ):m_uInputAddr(0),
	m_uNumVerts(uNumVerts),m_eMode(SPU_SKINNING_FULL_CULL){}

	Asura_PS3_SPU_SkinningConstants	m_xShaderConstants;
	uint64_t m_uInputAddr; // address of either the input vertex data (if we're not doing culling) or the list of input batches
	uint64_t m_uVertsOutAddr;
	uint64_t m_uIndicesOutAddr; // for culling
	u_int m_uNumVerts;
	SPU_SKINNING_MODE m_eMode;
	//u_int m_auPad[2];	 // must be a multiple of 16 bytes
};

#define uSPU_SKINNING_RSX_FLAG	1

#endif //#ifndef __ASURA_PS3_SPU_SKINNINGTYPES_H__



