//////////////////////////////////////////////////////////////////////////////
//
// Asura_AI_Navigation.h
//
// Copyright (c) 2004 Rebellion. All rights reserved.
//
// Created by Kevin Lea - 24/07/2001 12:22:52
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif

#ifndef ___ASURA_AI_NAVIGATION_H___
#define ___ASURA_AI_NAVIGATION_H___ 1

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// asura
#include "AI/Asura_Nav_Types.h"
#include "Core/Asura_HashTree.h"
#include "Core/Asura_Maths.h"
#include "Core/Asura_LinkList.h"

// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////////

class Asura_AABB_Tree;

//////////////////////////////////////////////////////////////////////////////
//
// Various helper classes.
//
//////////////////////////////////////////////////////////////////////////////

enum ASURA_AI_PATHFINDER_FLAGS
{
    ASURA_AI_PATHFINDER_FLAGS_CANFALL     = 0x00000001,
    ASURA_AI_PATHFINDER_FLAGS_CANFALLFAR  = 0x00000002,
    ASURA_AI_PATHFINDER_FLAGS_CANJUMP	  = 0x00000004,
    ASURA_AI_PATHFINDER_FLAGS_IGNOREZONES = 0x00000008,

    ASURA_AI_PATHFINDER_FLAGS_BEGIN_PROJECT_SPECIFIC = 0x00010000
};

enum ASURA_AI_CONNECTION_FLAGS
{
	ASURA_AI_CONNECTION_FLAGS_BLOCKED     = 0x0001,
	ASURA_AI_CONNECTION_FLAGS_FALL        = 0x0002,
	ASURA_AI_CONNECTION_FLAGS_LONGFALL    = 0x0004,
	ASURA_AI_CONNECTION_FLAGS_RESTRICTIVE = 0x0008,
	ASURA_AI_CONNECTION_FLAGS_JUMP		  = 0x0010,
	ASURA_AI_CONNECTION_FLAGS_CROUCH	  = 0x0020,
	ASURA_AI_CONNECTION_FLAGS_PRONE		  = 0x0040,

	ASURA_AI_CONNECTION_FLAGS_BEGIN_PROJECT_SPECIFIC = 0x0100
};

//-----------------------------------------------------------------------------------------------------------
// Old route finding interface that is due for removal.
class Asura_AStar_OpenList_Elem
{
public:
	float m_fTotalCost;
	int   m_iIndex;
};

class Asura_AStar
{
public:
	float        m_fCostToHere;
	float        m_fCostToGoal;
	int          m_iIndex;
	Asura_AStar* m_pxParent;
};

class Asura_OpenClosed
{
public:
	bool m_bOpen;
	bool m_bClosed;
};
// Old route finding interface that is due for removal.
//-----------------------------------------------------------------------------------------------------------

#define ASURA_AI_MAX_LINK_LENGTH	 ( 650.0f )
#define ASURA_AI_MAX_WAYPOINT_RADIUS ( 650.0f )
#define ASURA_AI_INVALID_LINK_INDEX	 ( 0xff )
#define ASURA_AI_MAX_LINK_INDEX		 ( ASURA_AI_INVALID_LINK_INDEX - 1 )

struct Asura_AI_NavConnection
{
public:
	Asura_AI_NavConnection()
	: m_usIndex(0)
	, m_usFlags(0)
	, m_usBlockCount(0)
	, m_usLength(0)
	{}

	u_int GetIndex()	  const { return m_usIndex; }
	u_int GetFlags()	  const { return m_usFlags; }
	u_int GetBlockCount() const { return m_usBlockCount; }
	float GetLength()	  const { return static_cast< float >( m_usLength ) * 0.01f; }

	bool IsBlocked()     const { return ( m_usFlags & ASURA_AI_CONNECTION_FLAGS_BLOCKED ) ? true : false; }
	bool IsFall()        const { return ( m_usFlags & ASURA_AI_CONNECTION_FLAGS_FALL ) ? true : false; }
	bool IsLongFall()    const { return ( m_usFlags & ASURA_AI_CONNECTION_FLAGS_LONGFALL ) ? true : false; }
	bool IsRestrictive() const { return ( m_usFlags & ASURA_AI_CONNECTION_FLAGS_RESTRICTIVE ) ? true : false; }
	bool IsJump()        const { return ( m_usFlags & ASURA_AI_CONNECTION_FLAGS_JUMP ) ? true : false; }
	bool IsCrouch()      const { return ( m_usFlags & ASURA_AI_CONNECTION_FLAGS_CROUCH ) ? true : false; }
	bool IsProne()       const { return ( m_usFlags & ASURA_AI_CONNECTION_FLAGS_PRONE ) ? true : false; }

	void SetIndex( const u_int uIndex )			  { m_usIndex	   = static_cast< u_short >( uIndex );		}
	void SetFlags( const u_int uFlags )			  { m_usFlags	   = static_cast< u_short >( uFlags );		}
	void SetBlockCount( const u_int uBlockCount ) { m_usBlockCount = static_cast< u_short >( uBlockCount ); }
	void SetLength( const float fLength )
	{
		// Convert 32 bit float to 16 bit unsigned short.
		Asura_Assert( fLength <= ASURA_AI_MAX_LINK_LENGTH, "Asura_AI_NavConnection::SetLength - link length is too long" );
		m_usLength = static_cast< unsigned short >( fLength * 100.0f );
	}

	unsigned short m_usIndex;
	unsigned short m_usFlags;
	unsigned short m_usBlockCount;
private:
	unsigned short m_usLength;
};

class Asura_AI_NavConnection_Index
{
public:
	inline bool operator!=( const Asura_AI_NavConnection_Index& xCon ) const
	{
		return ( m_usWaypointIndex != xCon.m_usWaypointIndex || m_usConnectionIndex != xCon.m_usConnectionIndex );
	}

public:
	unsigned short m_usWaypointIndex;
	unsigned short m_usConnectionIndex;
};

#define ASURA_AI_COVERPOINT_FULL        0x0001
#define ASURA_AI_COVERPOINT_SEMI        0x0002
#define ASURA_AI_COVERPOINT_LOW         0x0004
#define ASURA_AI_COVERPOINT_HIGH        0x0008
#define ASURA_AI_COVERPOINT_IN_USE      0x0010
#define ASURA_AI_COVERPOINT_INVALID		0x0020

#define ASURA_AI_COVERPOINT_SW          0x01
#define ASURA_AI_COVERPOINT_S           0x02
#define ASURA_AI_COVERPOINT_SE          0x04
#define ASURA_AI_COVERPOINT_W           0x08
#define ASURA_AI_COVERPOINT_E           0x10
#define ASURA_AI_COVERPOINT_NW          0x20
#define ASURA_AI_COVERPOINT_N           0x40
#define ASURA_AI_COVERPOINT_NE          0x80

class Asura_AI_Coverpoint
{
public:
	Asura_AI_Coverpoint()
	: m_xCentre( Asura_Maths::ZeroVector3 ),
	m_usFlags( 0 ),
	m_ucShortExits( 0 ),
	m_ucLongExits( 0 )
	{}
	void Lock()
	{
		Asura_Assert(!IsLocked(), "Already locked");
		m_usFlags |= ASURA_AI_COVERPOINT_IN_USE;
	}
	void Unlock()
	{
		Asura_Assert(IsLocked(), "Not locked");
		m_usFlags &= ~ASURA_AI_COVERPOINT_IN_USE;
	}
	bool IsLocked() const
	{
		return (m_usFlags & ASURA_AI_COVERPOINT_IN_USE) != 0;
	}
    void SetValid()
    {
        m_usFlags &= ~ASURA_AI_COVERPOINT_INVALID;
    }
    void SetInvalid()
    {
        m_usFlags |= ASURA_AI_COVERPOINT_INVALID;
    }
    bool IsValid() const
    {
        return (m_usFlags & ASURA_AI_COVERPOINT_INVALID) == 0;
    }

public:
	Asura_Vector_3 m_xCentre;
	unsigned short m_usFlags;
	// compass point exits from coverpoint with short range
	unsigned char  m_ucShortExits;
	// compass point exits from coverpoint with long range
	unsigned char  m_ucLongExits;
};

enum ASURA_AI_NAVVERTEX_FLAG
{
    ASURA_AI_NAVVERTEX_FLAG_SLOPE             = 0x0001,
    ASURA_AI_NAVVERTEX_FLAG_STEPS             = 0x0002,
	ASURA_AI_NAVVERTEX_FLAG_STREAMINGZONEEXIT = 0x0004, //For streaming system... marks waypoints that lead into adjacent zones
    ASURA_AI_NAVVERTEX_FLAG_CEILING			  = 0x0008,
    ASURA_AI_NAVVERTEX_FLAG_WALL			  = 0x0010,

	ASURA_AI_NAVVERTEX_FLAG_BEGIN_PROJECT_SPECIFIC = 0x0100
};

#define ASURA_NAVVERTEX_MAX_ZONE_ID ( 0xfffc )
#define ASURA_INVALID_WAYPOINT_ID	( 0xffff )
#define ASURA_AI_MAX_WAYPOINT_ID	( ASURA_INVALID_WAYPOINT_ID - 1 )
#define ASURA_INVALID_CONNECTION_OFFSET ( 0xffffffff )

// instead of the nav vertices containing coverpoint data this is kept in a separate array
// to save space for projects that don't require coverpoints. this is accessed as before,
// by using the index of the associated nav vertex.
class Asura_AI_NavVertex
{
public:
	Asura_AI_NavVertex()
	: m_xPosition( Asura_Maths::ZeroVector3 )
	, m_pxConnections( NULL )
	, m_usZone( 0 )
	, m_usFlags( 0 )
	, m_ucNumConnections( 0 )
    , m_ucNormalIndex( 0 )
	, m_usRadius( 0 )
	{}

	u_int				  GetZone()		const { return m_usZone; }
	u_int				  GetFlags()	const { return m_usFlags; }
	const Asura_Vector_3& GetNormal()	const;
	const Asura_Vector_3& GetPosition()	const { return m_xPosition; }
	float				  GetRadius()	const { return static_cast< float >( m_usRadius ) * 0.01f; }
	u_int				  GetNumLinks() const { return m_ucNumConnections; }
	const Asura_AI_NavConnection& GetLink( const u_int uIndex ) const { return m_pxConnections[ uIndex ]; }
	Asura_AI_NavConnection&       GetLink( const u_int uIndex )       { return m_pxConnections[ uIndex ]; }
	const Asura_AI_NavConnection* GetLinkTo( const u_int uWPIndex ) const
	{
		for ( u_char uc = 0; uc < m_ucNumConnections; ++uc )
		{
			if ( m_pxConnections[ uc ].GetIndex() == uWPIndex )
			{
				return &m_pxConnections[ uc ];
			}
		}
		return NULL;
	}
	Asura_AI_NavConnection* GetLinkTo( const u_int uWPIndex )
	{
		for ( u_char uc = 0; uc < m_ucNumConnections; ++uc )
		{
			if ( m_pxConnections[ uc ].GetIndex() == uWPIndex )
			{
				return &m_pxConnections[ uc ];
			}
		}
		return NULL;
	}
	Asura_AI_NavVertex* GetLinkedWaypoint( const u_int uLinkIndex ) const;

	void SetZone( const u_int uZone )		  { m_usZone = static_cast< u_short >( uZone ); }
	void SetFlags( const u_int uFlags )		  { m_usFlags = static_cast< u_short >( uFlags ); }
	void SetNormal( const Asura_Vector_3& xNormal );
	void SetPosition( const Asura_Vector_3& xPosition ) { m_xPosition = xPosition; }
	void SetRadius( const float fRadius )
	{
		// Convert 32 bit float to 16 bit unsigned short.
		Asura_Assert( fRadius <= ASURA_AI_MAX_WAYPOINT_RADIUS, "Asura_AI_NavVertex::SetRadius - radius is too large" );
		m_usRadius = static_cast< unsigned short >( fRadius * 100.0f );
	}
	void SetNumLinks( const u_int uNumLinks ) { m_ucNumConnections = static_cast< u_char >( uNumLinks ); }

    bool IsSlopeWaypoint() const   { return ( ( m_usFlags & ASURA_AI_NAVVERTEX_FLAG_SLOPE   ) != 0 ); }
    bool IsStepsWaypoint() const   { return ( ( m_usFlags & ASURA_AI_NAVVERTEX_FLAG_STEPS   ) != 0 ); }
    bool IsCeilingWaypoint() const { return ( ( m_usFlags & ASURA_AI_NAVVERTEX_FLAG_CEILING ) != 0 ); }
    bool IsWallWaypoint() const    { return ( ( m_usFlags & ASURA_AI_NAVVERTEX_FLAG_WALL    ) != 0 ); }
#ifdef ASURA_USE_STREAMING
    bool IsStreamingZoneExit() const {return (m_usFlags & ASURA_AI_NAVVERTEX_FLAG_STREAMINGZONEEXIT) != 0;}
#endif

	// Optionally fills in the distance from the waypoint.
    bool Contains( const Asura_Vector_3& xPosition, float* pfDistanceSq = NULL ) const;

public:
	Asura_Vector_3          m_xPosition;
	Asura_AI_NavConnection* m_pxConnections;
	u_short					m_usZone;
	// see ASURA_AI_NAVVERTEX_FLAG enumeration above
	u_short					m_usFlags;
	u_char					m_ucNumConnections;
private:
	// This is an index into a table of normals.
	u_char					m_ucNormalIndex;
	u_short					m_usRadius;

    friend class Asura_Chunk_Navigation;
};

// this provides a look-up into the array of coverpoint indices for every nav vertex
class Asura_AI_PerVertexCoverpointData
{
public:
	Asura_AI_PerVertexCoverpointData() : m_pusCoverpoints( NULL ), m_usNumCoverpoints( 0 ) {}

public:
	unsigned short* m_pusCoverpoints;
	unsigned short  m_usNumCoverpoints;
};

#ifdef ASURA_TOOLS
//stores details of a subset of the waypoints to write out for an export
class Asura_AI_WaypointSubsetForExport
{
public :
	Asura_AI_WaypointSubsetForExport();
	~Asura_AI_WaypointSubsetForExport();

	u_short m_usWaypointConversionSize;//conversion array size. should match number of waypoints in handler
	int* m_piWaypointConv;//conversion from full set of waypoints to subset index. index of -1 used for waypoints that are excluded

	//class Asura_AI_Navigation_Handler* m_pxHandler;
};
#endif

//-----------------------------------------------------------------------------------------------------------
// Old route finding interface that is due for removal.
// returns the estimated cost to the goal from the waypoint
typedef float( AANS_CostToGoalEstFn( const int iWaypointIndex, void* pUserData ) );
// returns the cost associated with travelling this link
typedef float( AANS_TraversalCostFn( const int iWaypointIndex, const int iLinkIndex, const u_int uPathfinderFlags, void* pUserData ) );
// returns whether this waypoint is the goal or not
typedef bool ( AANS_GoalAchievedFn( const int iWaypointIndex, void* pUserData ) );
// returns whether to ignore the link or not
typedef bool ( AANS_LinkFlagsTestFn( const Asura_AI_NavConnection* pxLink, const u_int uPathfinderFlags, void* pUserData ) );
// Old route finding interface that is due for removal.
//-----------------------------------------------------------------------------------------------------------

//////////////////////////////////////////////////////////////////////////////
//
// Asura_AI_Navigation_System
//
//////////////////////////////////////////////////////////////////////////////

class Asura_AI_Navigation_System
{
	friend class Asura_Chunk_Navigation;
	friend class Asura_Chunk_WaypointingSettings;
	friend class Tools_Mode_Waypointing;
    friend class Tools_Waypoint_Analysis;
	friend class Tools_Waypoint_Editing;

public:
	static void Initialise();
	static void Shutdown();
	static void Reset();
	static void Update();

	static Asura_AI_Navigation_System* GetInstance() { return s_pxThis; }

	virtual void ReadUDWDataFromChunkStream( Asura_Chunk_Stream& xStream );
	#ifdef ASURA_TOOLS
	virtual void WriteUDWDataToChunkStream( Asura_Chunk_Stream& xStream ) const;
	#endif

	#ifndef ASURA_NO_DEBUG_OUTPUT
	static void RenderWaypoints( const u_int uColour = 0xffffffff, const u_int uNumSegments = 4, const u_int uNumSlices = 4 );
	static void RenderCoverpoints( const u_int uColour = 0x6f00ffff, const u_int uNumSegments = 4, const u_int uNumSlices = 4 );
	// Renders links within s_fRenderLinkDistanceSq of camera one.
	static void RenderLinks( const u_int uColour = 0xffffffff );
	static void RenderBlockedLinks( const u_int uColour = 0xffff0000 );
	#endif

	static bool    IsInitialised()          { return s_bWaypointsValid; }
	static u_short GetNumberOfWaypoints()   { return s_usNumWaypoints; }
	static int     GetNumberOfCoverpoints() { return s_iNumCoverpoints; }
	static u_int   GetNumberOfConnections() { return s_uNumConnections; }

	#ifdef ASURA_USE_NEW_NAV_STUFF
	// Finds the nearest valid waypoint ID to the position given the parameters.
	// Returns ASURA_INVALID_WAYPOINT_ID if the lookup fails.
	static u_int GetNearestWaypointID( const Asura_Vector_3& xPosition, const Asura_Navigation::TestParams& xParams = Asura_Navigation::s_xStandardTParams );
	// Finds the nearest valid waypoint ID to the position given the parameters, using the current waypoint ID as a starting point.
	// This is faster than doing a search from nothing in most cases.
	// Sets usCurrentWaypointID to ASURA_INVALID_WAYPOINT_ID if the lookup fails.
	static void UpdateNearestWaypointID( const Asura_Vector_3& xPosition, u_int& uCurrentWPID, const Asura_Navigation::TestParams& xParams = Asura_Navigation::s_xStandardTParams );
	// Resolves a given position to one inside the waypoint graph and as close as possible to the nearest surface.
	static bool ResolveToValidPosition( Asura_Vector_3& xPosition, const Asura_Navigation::TestParams& xParams = Asura_Navigation::s_xStandardTParams );
	static bool PositionIsInWaypointSpace( const Asura_Vector_3& xPosition, const Asura_Navigation::TestParams& xParams = Asura_Navigation::s_xStandardTParams );
	static bool PositionIsInsideWaypoint( const Asura_Vector_3& xPosition, const u_int uWPID );
	// Returns whether the current position in near enough the desired position.
	// Standard parameters allow for a simple spherical test, or a co-planar and a perpendicular offset to be specified.
	static bool AtPosition( const Asura_Vector_3& xCurrentPosition, const Asura_Vector_3& xDesiredPosition, const Asura_Navigation::APParams& xParams = Asura_Navigation::s_xStandardAPParams );
	// Casts a ray in waypoint space. Validity of the waypoints is checked using the test parameters.
	static bool LineSegmentIsInWaypointSpace( const Asura_Vector_3& xStart, const Asura_Vector_3& xFinish, const Asura_Navigation::TestParams& xParams = Asura_Navigation::s_xStandardTParams );
	#endif

	// Various functions to search the waypoint graph with functors, rays and bounding boxes.
	static void SearchWaypoints( const Asura_Vector_3& xRayOrigin, const Asura_Vector_3& xRayDirection,
								 float* pfDistance, bool( *pfnCallback )( const u_int uIndex ) );
	static void SearchWaypoints( const Asura_Bounding_Box& xBB, bool ( *pfnCallback )( u_short usWaypointIdx ) );

	static const Asura_AI_NavVertex* GetWaypoints() { return s_pxWaypointStorage; }
	static Asura_AI_NavVertex* GetWaypoint( const u_int uWaypointIndex )
	{
		#ifdef ASURA_ASSERT
		Asura_Assert( uWaypointIndex == ASURA_INVALID_WAYPOINT_ID || uWaypointIndex < s_usNumWaypoints, "Asura_AI_Navigation_System::GetWaypoint - waypoint ID out of range" );
		#endif
		return( uWaypointIndex < s_usNumWaypoints ? &s_pxWaypointStorage[uWaypointIndex] : NULL );
	}
	static float GetMaxWaypointRadius() { return s_fMaxWaypointRadius; }
	static void  SetWaypointWeightFactor( float fWaypointWeightFactor ) { s_fWaypointWeightFactor = fWaypointWeightFactor; }
	static float GetWaypointWeightFactor() { return s_fWaypointWeightFactor; }
	static void AddWaypointWeight( const unsigned short usWaypointIndex, const unsigned char ucWaypointWeight );
	static void RemoveWaypointWeight( const unsigned short usWaypointIndex, const unsigned char ucWaypointWeight );
	static u_char GetWaypointWeight( const unsigned short usWaypointIndex );

	static void MarkConnectionAsBlocked(const Asura_AI_NavConnection_Index xConnectionIndex, const bool bBlock);
	static bool HaveConnectionsChangedLastFrame() { return s_bConnectionChangedLastFrame; }
	static u_int GetConnectionGeneration() { return s_uConnectionGeneration; }
	static u_int GetConnectionOffset( const Asura_AI_NavConnection* pxConnection );
	static void SetBlockCallback(void (*pfnCallback)(u_int uWaypointIndex, u_int uConnectionIndex, const bool bBlock));

	//-----------------------------------------------------------------------------------------------------------
	// Old route finding interface that is due for removal.
	static void FindRoute( unsigned short* pusRoute, int& iLengthOfRoute, const int iBufferLength,
						   const unsigned short usStart, const unsigned short usTarget,
						   const unsigned uPathfinderFlags, void* pUserData );
	// use this if you have any special project requirements like a different traversal cost calculation,
	// or project specific link flags
	static void FindRouteCustom( unsigned short* pusRoute, int& iLengthOfRoute, const int iBufferLength,
								 const unsigned short usStart,
								 const unsigned uPathfinderFlags,
								 void* pUserData,
								 AANS_CostToGoalEstFn* pfnCostToGoalEstFn,
								 AANS_TraversalCostFn* pfnTraversalCostFn,
								 AANS_GoalAchievedFn*  pfnGoalAchievedFn,
								 AANS_LinkFlagsTestFn* pfnLinkFlagsTestFn );
	static void FindRouteCustom( unsigned short* pusRoute, int& iLengthOfRoute, const int iBufferLength,
								 const unsigned short usStart,
								 const unsigned short usTarget,
								 const unsigned uPathfinderFlags,
								 void* pUserData,
								 AANS_CostToGoalEstFn* pfnCostToGoalEstFn,
								 AANS_TraversalCostFn* pfnTraversalCostFn,
								 AANS_GoalAchievedFn*  pfnGoalAchievedFn,
								 AANS_LinkFlagsTestFn* pfnLinkFlagsTestFn );

	// For doing synchronous A* searches. Thread safe.
	static void AStarSearch( unsigned short* pusRoute, int& iLengthOfRoute, const int iBufferLength,
							 const unsigned short usStart,
							 const unsigned uPathfinderFlags,
							 void* pUserData,
							 AANS_CostToGoalEstFn* pfnCostToGoalEstFn,
							 AANS_TraversalCostFn* pfnTraversalCostFn,
							 AANS_GoalAchievedFn*  pfnGoalAchievedFn,
							 AANS_LinkFlagsTestFn* pfnLinkFlagsTestFn );

	// For doing asynchronous A* searches. Not thread safe.
	static void PreAStarSearch( const unsigned short usStart, const unsigned short usFinish, AANS_CostToGoalEstFn* pfnGoalCostEstFn, void* pUserData );
	static void PreAStarSearch( const unsigned short usStart, AANS_CostToGoalEstFn* pfnGoalCostEstFn, void* pUserData );
	static bool AStarSearchIncremental( unsigned short* pusRoute, int& iLengthOfRoute, const int iBufferLength,
										const unsigned uPathfinderFlags,
										void* pUserData,
										AANS_CostToGoalEstFn* pfnCostToGoalEstFn,
										AANS_TraversalCostFn* pfnTraversalCostFn,
										AANS_GoalAchievedFn*  pfnGoalAchievedFn,
										AANS_LinkFlagsTestFn* pfnLinkFlagsTestFn );

	// these are public so they can be used in FindRouteCustom when an existing implementation for a
	// particular function will suffice
	// standard pathfinding A* functions
	static float DefaultGoalCostEstimateFunction( const int iCurrentIndex, void* pUserData );
	static float DefaultTraversalCostFunction( const int iWaypointIndex, const int iConnectionIndex, const u_int uPathfinderFlags, void* pUserData );
	static bool  DefaultGoalAchievedFunction( const int iCurrentIndex, void* pUserData );
	static bool  DefaultLinkFlagsTestFunction( const Asura_AI_NavConnection* pxLink, const u_int uPathfinderFlags, void* pUserData );
	// Old route finding interface that is due for removal.
	//-----------------------------------------------------------------------------------------------------------

	//-----------------------------------------------------------------------------------------------------------
	// Old nav data query interface that is due for removal.
	static u_short FindWaypointNearestTo( const Asura_Vector_3& position,
                                          const u_short usSearchInWaypointZone = ASURA_NAVVERTEX_MAX_ZONE_ID,
                                          const bool bWaypointMustContainCoord = false,
										  const bool bAllowLinearSearch = true,
                                          const bool bSkipConnectionlessWaypoints = true );
    static bool IsPositionInWaypoint( const Asura_Vector_3& xPosition, const int iWaypointID );
	static bool IsPositionInWaypoint( const Asura_Vector_3& xPosition );
	// Old nav data query interface that is due for removal.
	//-----------------------------------------------------------------------------------------------------------

	// coverpoint accessor functions
	static int GetNumCoverpointsForWaypoint( const int iWaypointID );
	// returns NULL if there are no coverpoints
	static Asura_AI_PerVertexCoverpointData* GetCoverpointData( const int iWaypointID );
	static bool GetNearestCover( const Asura_Vector_3& xThreatPosition, const Asura_Vector_3& xCurrentPosition, Asura_Vector_3& xCoverPosition );
	static int  GetNearestCoverIndex( const Asura_Vector_3& xThreatPosition, const Asura_Vector_3& xCurrentPosition, bool bRequireExits = false );
	static Asura_AI_Coverpoint* GetCoverpoint( const int iCoverpointID );
	static bool GetAllCoverpoints( const Asura_Vector_3& xThreatPosition, unsigned short** ppiCoverpoints, int* piNumberPoints );
	static int  GetCoverpointNearestTo( const Asura_Vector_3& xPosition );

	Asura_AI_Navigation_System();
	virtual ~Asura_AI_Navigation_System();

	struct UserDefinedWaypoint
	{
		Asura_Vector_3 m_xPosition;
		Asura_Quat	   m_xOrientQ;
		u_int		   m_uFlags;

		UserDefinedWaypoint();
		virtual ~UserDefinedWaypoint() {}

		virtual void ReadFromChunkStream( Asura_Chunk_Stream& xStream );
		virtual void WriteToChunkStream( Asura_Chunk_Stream& xStream ) const;

		void AquireID() { if ( m_uID == uINVALID_ID ) m_uID = ++s_uCurrentID; }
		u_int GetID() const { return m_uID; }

		static const u_int uINVALID_ID = 0;

	private:
		u_int m_uID;

		void UpdateCurrentID( const u_int uID ) { s_uCurrentID = Asura_Maths::Max( uID, s_uCurrentID ); }
		static u_int s_uCurrentID;
		static const u_int uCURRENT_CHUNK_VERSION = 0;
	};

	virtual UserDefinedWaypoint* CreateUserDefinedWaypoint();
	virtual void DestroyUserDefinedWaypoint( UserDefinedWaypoint* pxUDW );

	#ifdef ASURA_TOOLS
	// in tools only we keep track of this information for use in the waypoint generation
	static void CreateWaypoint( const Asura_Vector_3& xPos, const float fRadius );

	// Similar to Reset, except it preserves the user defined waypoints.
	static void ResetForWaypointGeneration();

	static Asura_LinkList< UserDefinedWaypoint* >& GetUserDefinedWaypointList() { return s_xUDWPList; }
	static UserDefinedWaypoint* GetUserDefinedWaypoint( const u_int uIndex ) { return s_xUDWPList[ uIndex ]->Data(); }
	static u_int GetNumUserDefinedWaypoints() { return s_xUDWPList.Size(); }
	static UserDefinedWaypoint* AddUserDefinedWaypoint();
	static void DeleteUserDefinedWaypoint( UserDefinedWaypoint* pxUDW );

	//## Functionality for this part of the interface has been removed until it's been established whether it is still needed or not.
	static bool HasSubsetForExport() { return false; }
	static void GenerateSubsetForExport();
	static void DeleteSubsetForExport();
	static bool ConvertNavConnectionIndexForExport(const Asura_AI_NavConnection_Index& xInput, Asura_AI_NavConnection_Index& xOutput);
	#endif

private:
	static void InitialiseAStarStorage();
	static void CreateWaypointTree();

	static bool WaypointTreeCreateCallback( u_int uNumWaypoints, Asura_Bounding_Box* pxBBs, Asura_Vector_3* );
	static bool WaypointTreeCallback( unsigned short usIndex );

	//-----------------------------------------------------------------------------------------------------------
	// Old nav data query interface that is due for removal.
	//Find nearest waypoint in this (streaming) zone. The tree search may fail to find a waypoint (returns -1) if
	//the point isn't close enough to any waypoint.
    static int FindNearestWaypoint_TreeSearch( const Asura_Vector_3& xPosition,
                                               float* pfIOMinDistance = NULL,
                                               const u_short usCurrentWaypointZone = ASURA_NAVVERTEX_MAX_ZONE_ID,
                                               const bool bWaypointMustContainCoord = false,
                                               const bool bSkipConnectionlessWaypoints = true );
    static int FindNearestWaypoint_LinearSearch( const Asura_Vector_3& xPosition,
                                                 float* pfIOMinDistance = NULL,
                                                 const u_short usCurrentWaypointZone = ASURA_NAVVERTEX_MAX_ZONE_ID,
                                                 const bool bWaypointMustContainCoord = false,
                                                 const bool bSkipConnectionlessWaypoints = true );
	// Old nav data query interface that is due for removal.
	//-----------------------------------------------------------------------------------------------------------

	//-----------------------------------------------------------------------------------------------------------
	// Old route finding interface that is due for removal.
	static void         AddNodeToOpenList( const Asura_AStar* pxNode );
	static Asura_AStar* GetOpenListNode();
	// Old route finding interface that is due for removal.
	//-----------------------------------------------------------------------------------------------------------

	static Asura_AI_Navigation_System* Project_CreateSystem();

private:
	static Asura_AI_Navigation_System*		 s_pxThis;
	static bool                              s_bWaypointsValid;
	// array of nav vertices
	static unsigned short                    s_usNumWaypoints;
	static Asura_AI_NavVertex*               s_pxWaypointStorage;
	// array of connections
	static unsigned int                      s_uNumConnections;
	static Asura_AI_NavConnection*           s_pxNavConnectionStorage;
	// array of coverpoint data for each nav vertex
	static Asura_AI_PerVertexCoverpointData* s_pxPerVertexCoverpointData;
	// array of coverpoint indices
	static unsigned int                      s_uNumCoverpointIndices;
	static unsigned short*                   s_pusCoverpointIndexStorage;
	// array of coverpoints
	static int                               s_iNumCoverpoints;
	static Asura_AI_Coverpoint*              s_pxCoverpointStorage;
	static Asura_AABB_Tree*                  s_pxWaypointTree;
	// array of waypoint weights for affecting A* route generation
	static unsigned char*                    s_pucWaypointWeights;
	static float                             s_fWaypointWeightFactor;

	// Needed for callbacks
	static const Asura_Vector_3*		s_pxWaypointCallbackTestPosition;
	static float						s_fWaypointCallbackMinDistance;
	static u_short		 			    s_usWaypointCallbackWaypointZone;
	static bool                         s_bWaypointCallbackShouldContainPosition;
	static bool                         s_bWaypointCallbackShouldSkipConnectionlessWaypoints;
	static int							s_iWaypointCallbackWaypointID;

	//-----------------------------------------------------------------------------------------------------------
	// Old route finding interface that is due for removal.
	// A* algorithm storage
	static Asura_AStar*                      s_pxAStarList;
	static Asura_OpenClosed*                 s_pxAStarOpenClosedVector;
	static Asura_AStar_OpenList_Elem*        s_pxAStarOpenList;
	static int                               s_iNumElemsInOpenList;

	// data for the A* functions
	static bool                              s_bRouteTargetValid;
	static unsigned short                    s_usRouteTarget;
	static float                             s_fPathfindRange;
	static Asura_Vector_3                    s_xPathfindPosition;
	static bool                              s_bMidAStarIncremental;
	// Old route finding interface that is due for removal.
	//-----------------------------------------------------------------------------------------------------------

	static bool                              s_bConnectionChangedThisFrame;
	static bool                              s_bConnectionChangedLastFrame;

	// call back to call when blocking waypoints
	static void (*s_pfnBlockCallback)(u_int uWaypointIndex, u_int uConnectionIndex, const bool bBlock);

	enum { CONNECTION_GENERATION_INVALID = 0, CONNECTION_GENERATION_FIRST = 1 };
	static u_int							  s_uConnectionGeneration;

	static float							  s_fMaxWaypointRadius;

	#ifdef ASURA_TOOLS
	static Asura_LinkList< UserDefinedWaypoint* > s_xUDWPList;
	#endif

	static const u_int uCURRENT_CHUNK_VERSION = 0;
};

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

#endif  //___ASURA_AI_NAVIGATION_H___
