//
// ASURA_AdEngine.cpp
//
// Created by DWW 27/06/06
// 
// Copyright (C) 2006 Strangelite, All Rights Reserved.
//

#include "Core/Asura.h" 
#include "Core/Asura_TextureManagement.h"

#ifdef ASURA_INGAMEADENGINE
#include "AdEngine/Asura_InGameAdEngine.h"
#include "AdEngine/Asura_AdObject.h"

// Static 
Asura_LinkList<Asura_AdObject*> Asura_InGameAdEngine::m_xListOfAdObjects;
bool Asura_InGameAdEngine::m_bInitialised = true;

Asura_InGameAdEngine::Asura_InGameAdEngine(void)
{

}

Asura_InGameAdEngine::~Asura_InGameAdEngine(void)
{
}

// void Asura_InGameAdEngine::Initialise(void)
//
// Creation Date:		27/06/2006
// Author:				DWW 
//
// Description: 
//
//
void Asura_InGameAdEngine::Initialise(void)
{
	Platform_Initialise();

	PlatformProject_Initialise();
}

// void Asura_InGameAdEngine::Shutdown(void)
//
// Creation Date:		27/06/2006
// Author:				DWW 
//
// Description: 
//
//
void Asura_InGameAdEngine::Shutdown(void)
{
	// destroy all ad objects that are still in memory
	DestroyAllAdObjects();

	Platform_Shutdown();

	PlatformProject_Shutdown();
}

// void Asura_InGameAdEngine::Update(const float fFrameTime)
//
// Creation Date:		27/06/2006
// Author:				DWW 
//
// Description: Call this per frame, cycles through add objects and updates them
//
//
void Asura_InGameAdEngine::Update(const float fFrameTime)
{
	Platform_Update(fFrameTime);

	PlatformProject_Update(fFrameTime);

	if (GetIsInitialised())
	{
		// iterate through an internal processing list and update the ad objects
		Asura_ListElt<Asura_AdObject*>* pxElement = m_xListOfAdObjects.Front();

		while (pxElement)
		{
			pxElement->Data()->Update(fFrameTime);
			pxElement = pxElement->Next();
		}
	}
}

// void Asura_InGameAdEngine::RegisterObject(Asura_AdObject* pxObjectToAdd)
//
// Creation Date:		27/06/2006
// Author:				DWW 
//
// Description: 
//
//
void Asura_InGameAdEngine::RegisterObject(Asura_AdObject* pxObjectToAdd)
{
	if (GetIsInitialised())
	{
		m_xListOfAdObjects.Push_Front(pxObjectToAdd);
		pxObjectToAdd->SetObjectID(m_xListOfAdObjects.Size());
	}
}

// void Asura_InGameAdEngine::UnregisterObject(Asura_AdObject* pxDFEnabledObject)
//
// Creation Date:		27/06/2006
// Author:				DWW 
//
// Description: 
//
//
void Asura_InGameAdEngine::UnregisterObject(Asura_AdObject* pxAdObject)
{
	if (GetIsInitialised())
	{
		Asura_ListElt<Asura_AdObject*>* pxElement = m_xListOfAdObjects.Front();

		while (pxElement)
		{
			if (pxElement->Data()->GetObjectID() == pxAdObject->GetObjectID())
			{
				pxElement->Data()->Destroy(); // destroy the object
				m_xListOfAdObjects.Remove(pxElement);
				return;
			}

			pxElement = pxElement->Next();
		}
	}
}

// void Asura_InGameAdEngine::ReleaseAdObjects(void)
//
// Creation Date:		27/06/2006
// Author:				DWW 
//
// Description:  removes all ad objects from memory (make sure all vendor specific class handle the destroy method properly)
//
//
void Asura_InGameAdEngine::ReleaseAdObjects(void)
{
	DestroyAllAdObjects();
}

// void Asura_InGameAdEngine::DestroyAllAdObjects(void)
//
// Creation Date:		27/06/2006
// Author:				DWW 
//
// Description:  removes all ad objects from memory (make sure all vendor specific class handle the destroy method properly)
//
//
void Asura_InGameAdEngine::DestroyAllAdObjects(void)
{
	if (GetIsInitialised())
	{
		Asura_ListElt<Asura_AdObject*>* pxElement = m_xListOfAdObjects.Front();

		while(pxElement)
		{
			pxElement->Data()->Destroy();
			pxElement = pxElement->Next();
		}
	}

	m_xListOfAdObjects.Clear();
}

// const bool Asura_InGameAdEngine::GetIsInitialised()
//
// Creation Date:		27/06/2006
// Author:				DWW 
//
// Description: 
//
//
const bool Asura_InGameAdEngine::GetIsInitialised()
{
	return m_bInitialised;
}

// void Asura_InGameAdEngine::CreateAdObjects(void)
//
// Creation Date:		27/06/2006
// Author:				DWW 
//
// Description: 
//
//
void Asura_InGameAdEngine::CreateAdObjects(void)
{
	Platform_CreateAdObjects();

	PlatformProject_CreateAdObjects();
}

void Asura_InGameAdEngine::StartZone(const Asura_UnicodeChar* szLevelName)
{
	Platform_StartZone(szLevelName);

	PlatformProject_StartZone(szLevelName);
}

void Asura_InGameAdEngine::LeaveZone()
{
	Platform_LeaveZone();

	PlatformProject_LeaveZone();
}

#endif
