
///////////////////////////////////////////////////////////////////////////////
//
// File   : Asura_Affector_Global.h
// Author : Andy Tate
// Date   : 8-Dec-2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __ASURA_AFFECTOR_GLOBAL_H__
#define __ASURA_AFFECTOR_GLOBAL_H__

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Forward class declarations...
////////////////////////////////////////////////////////////////////////////////

class Asura_Chunk_Stream;


////////////////////////////////////////////////////////////////////////////////
// Class declaration...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Class        : Asura_Affector_Global
// Author       : Andy Tate
// Date         : 8-Dec-2006
//
// Description  : Class used to interface to global affectors.
////////////////////////////////////////////////////////////////////////////////

class Asura_Affector_Global
{
	/// Kevin Lea - 25/09/2002 15:25:13 - Asura_Affector's are created/destroyed from a 
	/// memory pool, and so don't want (or need) a constructor or destructor

public:
	// Public functions...
	void Reset (void)
	{
		m_uFlags			= 0;
		m_bActive			= true;
		m_fStrength			= 1.0f;
		m_fResponseScale	= 1.0f;
	}

	void ReadFromChunkStream(Asura_Chunk_Stream& xStream);
	void WriteToChunkStream(Asura_Chunk_Stream& xStream) const;

	Asura_Affector_Global& operator= (const Asura_Affector_Global& xOther)
	{
		m_fStrength			= xOther.m_fStrength;
		m_fResponseScale	= xOther.m_fResponseScale;
		m_uFlags	= xOther.m_uFlags;
		return(*this);
	}

	// Accessor functions...
	void	SetAsActive(const bool bActive)	{ m_bActive = bActive; }
	bool	IsActive(void) const			{ return(m_bActive); }

	void	SetStrength(const float fScale)	{ m_fStrength = fScale; }
	float	GetStrength(void) const			{ return(m_fStrength); }

	void	SetResponseScale(const float fScale)	{ m_fResponseScale = fScale; }
	float	GetResponseScale(void) const			{ return(m_fResponseScale); }


protected:
	// Protected variables...
	union
	{
		u_int		m_uFlags;			// Single-access point for the global affector flags.

		struct
		{
			u_int	m_bActive	: 1,	// Flag to say whether this affector is currently active.
								: 0;	// 31 spare-bits
		};
	};

	float	m_fStrength;		// The strength-value to apply to the Perlin-noise calculate (0.0f if not to be applied).
	float	m_fResponseScale;	// The scale-value to apply to the time for the Perlin-noise calculation (0.0f if not to be applied).

	// Protected functions...


private:
	// Private variables...

	// Private functions...

};   // End of class 'Asura_Affector_Global'.


#endif   // ifndef __ASURA_AFFECTOR_GLOBAL_H__


/// End of file. ///
