//
// Asura_Animation.h
//
// Created by Kevin Lea - 12/06/2001 11:13:33
// 
// Copyright (C) 2001 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef __ASURA_ANIMATION_H__
#define __ASURA_ANIMATION_H__

#include "Animation/Asura_AnimationAttachment.h"
#include "Animation/Asura_AnimationController_Group.h"
#include "Animation/Asura_Animation_PrimaryWrapper.h"
#include "Animation/Asura_Animation_BoneDataModifier.h"
#include "Animation/Asura_Hierarchy.h"
#include "Animation/Asura_Hierarchy_Tween.h"
#if defined(ASURA_USE_ANIMATION_PLATFORM_RENDER_CACHE)
#include "Animation/Asura_Platform_Animation_RenderCache.h"
#endif
#include "Animation/Asura_TextureAnimation.h"
#include "Core/Asura_HandleTreeTemplate.h"
#include "Core/Asura_HashTree.h"
#include "Core/Asura_List.h"
#include "Core/Asura_Maths.h"
#include "Core/Asura_Timers.h"
#include "Core/Asura_FixedArray.h"
#include "Rendering/Asura_Colour.h"
#include "Network/Asura_Guid.h"
#if defined(ASURA_USE_STREAMING)
#include "Streaming/Asura_Streaming_ModuleList.h"
#endif //defined(ASURA_USE_STREAMING)
#if defined( ASURA_USE_NEW_VISIBILITY_SYSTEM )
#include "Visibility/Asura_Visibility_System.h"
#endif

#ifndef ASURA_ANIMATION_SKIN_COUNT
#define ASURA_ANIMATION_SKIN_COUNT 1
#endif

// NB: Higher limits in tools builds because we need to deal with exporting
// all characters, etc. You can override these limits in your project's /
// platform's Asura_EngineConfiguration.h.
#ifndef ASURA_MAX_NO_OF_HIERARCHY_ANIMS
    #ifdef ASURA_TOOLS
        #define ASURA_MAX_NO_OF_HIERARCHY_ANIMS (2048)
    #else
        #define ASURA_MAX_NO_OF_HIERARCHY_ANIMS (1024)
    #endif
#endif

#ifndef ASURA_MAX_NO_OF_HIERARCHY_SKINS
    #ifdef ASURA_TOOLS
        #define ASURA_MAX_NO_OF_HIERARCHY_SKINS (256)
    #else
        #define ASURA_MAX_NO_OF_HIERARCHY_SKINS (64)
    #endif
#endif

#if defined(ASURA_USE_INDEPENDENT_SHADOW_SYSTEM)
#include "Shadows/Asura_Shadow_System.h"
#endif

//////////////////////////////////////////////////////////////////////////
// Forward declarations...
//////////////////////////////////////////////////////////////////////////
class Asura_AnimationController_Animation;
class Asura_ResizingMempool;
class Asura_Animation_SoundInstance;


#ifdef ASURA_USE_ANIMATION_IK_SOLVER
class Asura_Animation_IK_Solver;
#endif // ASURA_USE_ANIMATION_IK_SOLVER

//////////////////////////////////////////////////////////////////////////
// Class declarations...
//////////////////////////////////////////////////////////////////////////
class Asura_Animation_Handler
{
	friend class Asura_Animation_System;

public:
	// Public functions...
	Asura_Animation_Handler(void);
	~Asura_Animation_Handler();

	void					Reset					(void);

	// KJL 02/12/00 17:28:29 - returns true if this is a new animation
	// anim gets set to the animation's storage space
	bool					NewAnimation			(const char* szAnimName, Asura_Hierarchy_Anim*& pxAnim);

	Asura_Hierarchy_Anim*	GetAnim					(const char* szAnimName)	{ return(m_xAnimTree.GetDataByName(szAnimName)); }
	Asura_Hierarchy_Anim*	GetAnimByHash			(Asura_Hash_ID uHash)		{ return(m_xAnimTree.GetDataByHash(uHash)); }
	#ifdef ASURA_TOOLS
	Asura_Hierarchy_Anim*	GetAnim					(int iIndex)				{ return(m_xAnimTree.GetDataByIndex(iIndex)); }
	#endif

	int						GetNumberOfAnims		(void)						{ return(m_xAnimTree.GetNumberOfNodes()); }

	// Kevin Lea - 10/08/2001 15:10:55 - returns true if this is a new skin
	bool					NewSkin					(const char* szSkinName, Asura_Hierarchy_Skin*& pxSkin);

	Asura_Hierarchy_Skin*	GetSkin					(const char* szSkinName);
	Asura_Hierarchy_Skin*	GetSkinByHash			(Asura_Hash_ID uHash);	
	#ifdef ASURA_TOOLS
	Asura_Hierarchy_Skin*	GetSkin					(int iIndex);			
	#endif

	int						GetNumberOfSkins		(void)						{ return(m_xSkinTree.GetNumberOfNodes()); }

private:
	// Private variables...
	Asura_TemplateHashTree<Asura_Hierarchy_Anim>	m_xAnimTree;
	Asura_TemplateHashTree<Asura_Hierarchy_Skin*>	m_xSkinTree;

	// Private functions...
	void					ReleaseAll				(void);
	void					ReleaseSkins			(void);
};


class Asura_Animation_System
{
	friend class Asura_Animation_Handler;

public:
	// Public functions...
	static 	void						Initialise				(void);
	static 	void						Shutdown				(void);
	static 	void						Reset					(void);

	static Asura_Animation_SoundInstance* AllocateSoundInstance(void);
	static void DeallocateSoundInstance(Asura_Animation_SoundInstance* pxSoundInstance);


	// KJL 02/12/00 17:28:29 - returns true if this is a new animation
	// anim gets set to the animation's storage space
	static	bool						NewAnimation			(const char* animName, Asura_Hierarchy_Anim*& anim)		{ return(GetCurrentHandler().NewAnimation(animName, anim)); }

	static 	Asura_Hierarchy_Anim* 		GetAnim					(const char* animName)									{ return(GetCurrentHandler().GetAnim(animName)); }
	static 	Asura_Hierarchy_Anim* 		GetAnimByHash			(Asura_Hash_ID uHash)									{ return(GetCurrentHandler().GetAnimByHash(uHash)); }
	#ifdef ASURA_TOOLS
	static Asura_Hierarchy_Anim*		GetAnim					(int iIndex)											{ return(GetCurrentHandler().GetAnim(iIndex)); }
	#endif

	static	int							GetNumberOfAnims		(void)													{ return(GetCurrentHandler().GetNumberOfAnims()); }

	// Kevin Lea - 10/08/2001 15:10:55 - returns true if this is a new skin
	static	bool						NewSkin					(const char* szSkinName, Asura_Hierarchy_Skin*& skin)	{ return(GetCurrentHandler().NewSkin(szSkinName, skin)); }

	static 	Asura_Hierarchy_Skin*		GetSkin					(const char* szSkinName)								{ return(GetCurrentHandler().GetSkin(szSkinName)); }
	static 	Asura_Hierarchy_Skin*		GetSkinByHash			(Asura_Hash_ID uHash)									{ return(GetCurrentHandler().GetSkinByHash(uHash)); }
	#ifdef ASURA_TOOLS
	static Asura_Hierarchy_Skin*		GetSkin					(int iIndex)											{ return(GetCurrentHandler().GetSkin(iIndex)); }
	#endif

	static	int							GetNumberOfSkins		(void)													{ return(GetCurrentHandler().GetNumberOfSkins()); }

#if defined(ASURA_USE_MULTIPLE_HANDLERS)
	// Handler functions...
	static	void						ReparentHandler			(const Asura_Hash_ID uSourceHash, const Asura_Hash_ID uNewParentHash);
	static	void						ReparentHandler			(Asura_Animation_Handler& xSourceHandler, const Asura_Hash_ID uNewParentHash);

	static	Asura_Animation_Handler*	CreateHandler			(const Asura_Hash_ID uHash, const Asura_Hash_ID uParentHandlerHash);
	static	bool						DestroyHandler			(const Asura_Hash_ID uHash);
	static	void						WriteHandlerReport		(u_int uHandler, Asura_Stream& xLog);

	static	void						CreateAndSetHandler		(const Asura_Hash_ID uHash, const Asura_Hash_ID uParentHandlerHash);
	static	void						SetCurrentHandler		(const Asura_Hash_ID uHash);
	static	Asura_Animation_Handler&	GetCurrentHandler		(void)													{ return(*s_pxCurrentHandler); }
	static	Asura_Animation_Handler*	GetHandler				(const Asura_Hash_ID uHash)								{ return(s_xHandlerTree.GetDataByHash(uHash)); }
#else
	// Private functions...
	static	Asura_Animation_Handler&	GetCurrentHandler		(void)	{ return(s_xHandler); }
#endif	// defined(ASURA_USE_MULTIPLE_HANDLERS)


private:
#if defined(ASURA_USE_MULTIPLE_HANDLERS)
	// Private variables...
	static	Asura_Animation_Handler*						s_pxCurrentHandler;		// Currently active handler.
	static	Asura_TemplateHashTree<Asura_Animation_Handler>	s_xHandlerTree;			// Tree of all of the handlers available.
#else
	// Private variables...
	static	Asura_Animation_Handler							s_xHandler;				// Single static-instance of a zone.
#endif

	static	Asura_ResizingMempool							s_xSoundInstanceMempool;

};

#ifdef ASURA_USE_KEYFRAME_CACHING
// KOR - 29/11/2005 - cache of all the information needed for a bone's animation pos & ori.
class Asura_Cached_KeyFrame
{
public:
	// Constructor - fill out with invalid data
	Asura_Cached_KeyFrame()
	: m_fTime1(-1.f)
	, m_fTime2(-1.f)
	, m_fTime3(-1.f)
	, m_iIndex1(-1)
	{}
	// Members
	Asura_Quat		m_xQuat1;// keyframe ori before
	Asura_Quat		m_xQuat2;// keyframe ori after
	Asura_Vector_3	m_xPos1; // keyframe pos before
	Asura_Vector_3	m_xPos2; // keyframe pos before
	float			m_fTime1;// time of keyframe before
	float			m_fTime2;// time of keyframe after
	float			m_fTime3;// time of keyframe after that, so we can maybe avoid the search
	int				m_iIndex1;// index of keyframe before
};
#endif//def ASURA_USE_KEYFRAME_CACHING

#define ANIMTIME_USEGAMETIME -1.0f

//flags for StartAnim2 and AddOneShotAnim2 - RAR
#define AsuraAnimationFlag_Loop 			   0x00000001 
#define AsuraAnimationFlag_Tween			   0x00000002 
#define AsuraAnimationFlag_Reversed			   0x00000004 
#define AsuraAnimationFlag_AutoRemove		   0x00000008 	//Secondary only - Remove animation when finished
#define AsuraAnimationFlag_SyncWithPrimary	   0x00000010 	//Secondary only - Use timer from primary animation (loop and reverse flags are irrelevant if this is used)
#define AsuraAnimationFlag_TweenOut			   0x00000020 	//Secondary only - tween weight to 0 when finished
#define AsuraAnimationFlag_DontTweenRoot	   0x00000040 	//Don't do tweening on the root section of the hierarchy (for animations that move away from the centre 'compass' point)
#define AsuraAnimationFlag_AnimatedTween	   0x00000080 	//Primary only - new and old animation continue playing while this gets tweened in.
															//This flag will take precedence over the normal tween flag.
#define AsuraAnimationFlag_AnimatedTween_SyncTimers 0x00000100	//Variation on animated tween that will start the new animation using the previous animation's timer
																//and force the timers to stay in sync while the tweening happens
#define AsuraAnimationFlag_UseMovementOffset   0x00000200	//Indicates that this animation should be included when getting the offsets via GetBlendedMovementOffset().
															//Note that this is ignored when using the old GetMovementOffset() function , which just gets the values from the primary animation.
#define AsuraAnimationFlag_Mirror              0x00000400   //X-Flips animation. Note that this requires project implementations of a couple of virtual Asura_Hierarchy_Skin functions
															//(GetMirroredBone and ApplyMirroring)
#define AsuraAnimationFlag_DontTweenRootPos	   0x00000800   //Same as DontTweenRoot but just does the position
#define AsuraAnimationFlag_DontTweenRootOrient 0x00001000	//Same as DontTweenRoot but just does the orient

//RAR 25/08/2006 Cutscene events and anim change messages make use of some of the flag space for other purposes 
#define AsuraAnimationFlag_Reserved            0xf0000000   

const float fASURA_ANIMATION_DEFAULT_MAX_GROUND_HEIGHT = 10000.0f;

class Asura_Animation
{
	friend class Asura_AnimationController_Group;
	friend class Asura_AnimationController_Animation;

	friend class Asura_Hierarchy_System;

	friend class Asura_AnimationAttachment;
	friend class Asura_AnimationAttachment_Animation;
	friend class Asura_AnimationAttachment_BoneRelative;
	friend class Asura_AnimationAttachment_BoneRelativeAnimation;
	
	friend class Asura_ClientEntity_Ragdoll;
    friend class Asura_Physics_Ragdoll;
	
	public:

        enum { uINITIAL_FRAMESTAMP = 0xFFFFFFFFu };

		Asura_Animation(const char* szSkinName);
		Asura_Animation(Asura_Hash_ID uHashSkin);
		// Minimal copy constructor for ragdoll *only*
		explicit Asura_Animation( const Asura_Animation* pxAnim );

		virtual ~Asura_Animation();

        virtual void Copy( const Asura_Animation* pxSrcAnim, bool bCopySkin, bool bCopyOneShotAnims, bool bCopyAttachments );
		void CopyAttachments( const Asura_Animation* pxSrcAnim );

		//change the skin used by this animation
		void SetSkin(const char* szSkinName, u_int uSkinIndex = 0);
		void SetSkin( Asura_Hash_ID uHashSkin, bool bTemporary = false, u_int uSkinIndex = 0);

        void SetParentEntity( Asura_Guid uGUID ) { ParentEntityGuid = uGUID; }

		void UpdateBoneData (const u_int uFirstBone, const u_int uLastBone);
	
#ifdef ASURA_USE_DEBUG_TEXT
		void PrintDebugInfo( u_int& uCol, u_int& uRow, const bool bRenderControllerInfo = false ) const;
#endif
		// WGP - 25/02/2009
		// Makes sure the coarse bounding sphere is up-to-date - this will involve updating the root bone
		// to make sure any movement offset of the root bone in the anim gets applied to the bounding sphere.
		// We will not update the IK solver or any bone data mofidifiers and we will not udpate any bone other that the root bone.
		// That will all get done later in the render call.
		void UpdateCoarseBoundingSphere( const Asura_Vector_3& xPosition, const Asura_Matrix_3x3& xOrientation );

		// WGP - 26/02/2009
		// This function re calculates the bounding box to snuggly fit around the bone section bounding boxes for the current animation.
		// It requires that the bone data has been updated before it can do this.
		void UpdateTightBoundingSphere( const Asura_Vector_3& xPosition, const Asura_Matrix_3x3& xOrientation );
		
		// KJFL - 16 November 2007  
		// Accessors for the bounding sphere radius and centre - this data may be a frame out of sync,
		// and you should call UpdateBoundingSphere() if you want to trigger a lazy evaluation.
		float GetTightBoundingSphereRadius( void ) const { return m_fTightBSRadius; }
		float GetCoarseBoundingSphereRadius( void ) const { return m_fCoarseBSRadius; }
		float GetCoarseBoundingSphereIncludingEffectsRadius( void ) const { return m_fCoarseBSIncEffectsRadius; }
		const Asura_Vector_3& GetBoundingSphereCentre( void ) const { return m_xBSCentre; }

		void SetIgnoreBoneForLOS (const u_int uBone, bool bIgnore)
		{
			m_xLOSFlags[uBone] = bIgnore;
		}
		bool GetIgnoreBoneForLOS (const u_int uBone)
		{
			return m_xLOSFlags[uBone];
		}


		// KJFL - 15 November 2007  
		// This function sets the first parameter to be a coarse world space bounding box covering the
		// animation given the input world space position and orientation at which it it being played. 
		// This is done by using the bounding sphere radius and centre members of the anim.
		//
		// This routine will intentionally *only* update the bone/anim data if the bounding data stored
		// in the anim is uninitialised (e.g. it's the first frame and the anim has never been updated
		// or rendered), or if the "bForceUpdate" parameter is set to true. Otherwise it will not update
		// the anim for that frame and instead will use whatever bounding data is already present.
		//
		// Although that means it can potentially be a frame out of sync wrt the animation, the bounding
		// volume is by no means tight on the object and so this should be ok for coarse trivial rejects.
		void GetCoarseBoundingBox( Asura_Bounding_Box& xAABB, const Asura_Vector_3& xPosition, const Asura_Matrix_3x3& xOrientation, bool bForceUpdate=false );

		//////////////////////////////////////////////////////////////////////////
		// New animation system, it replaces old concepts of primary and
		// secondary animations being blended together - ART 15/09/06.
		//////////////////////////////////////////////////////////////////////////
		typedef Asura_TemplateHashTree<Asura_AnimationController_Animation*> ControllerAnimationContainer;
		typedef Asura_TemplateHashTree<Asura_AnimationController_Group*> ControllerGroupContainer;

		// Animation controller manipulation functions...
		Asura_AnimationController_Group*		CreateControllerGroup		(const char* szControllerName, const u_int uInitialFlags, const u_int uPriority, Asura_AnimationBlend* pxBlend, Asura_AnimationController_Group* pxParentGroup = NULL);
		Asura_AnimationController_Group*		CreateControllerGroup		(const Asura_Hash_ID uControllerHash, const u_int uInitialFlags, const u_int uPriority, Asura_AnimationBlend* pxBlend, Asura_AnimationController_Group* pxParentGroup = NULL);
		Asura_AnimationController_Animation*	CreateControllerAnimation	(const char* szControllerName, const u_int uInitialFlags, const u_int uPriority, const Asura_Hash_ID uAnimationHash, Asura_AnimationBlend* pxBlend, Asura_AnimationController_Group* pxParentGroup = NULL);
		Asura_AnimationController_Animation*	CreateControllerAnimation	(const Asura_Hash_ID uControllerHash, const u_int uInitialFlags, const u_int uPriority, const Asura_Hash_ID uAnimationHash, Asura_AnimationBlend* pxBlend, Asura_AnimationController_Group* pxParentGroup = NULL);
		bool									DestroyController			(const char* szControllerNam);
		bool									DestroyController			(const Asura_Hash_ID uControllerHash);
		void									DestroyAllControllers		(void);
		bool									SetControllerAnimationID	(const Asura_Hash_ID uOldID, const Asura_Hash_ID uNewID);

		// Special creation function for one-shot animations...
		Asura_AnimationController_Animation* CreateControllerAnimation (const u_int uInitialFlags,
																		const u_int uPriority,
																		const Asura_Hash_ID uAnimationHash,
																		Asura_AnimationBlend* pxBlend,
																		Asura_AnimationController_Group* pxParentGroup = NULL)
		{
			const u_int	uControllerHash	= (uAnimationHash == ASURA_HASH_ID_UNSET) ? 0xC1FF0752 : uAnimationHash;	// Asura_GetHashID("NoAnim") : uAnimationHash;
			const u_int	uModifiedFlags	= uInitialFlags | AAC_FLAG_ENSURE_UNIQUE_HASH;

			return(CreateControllerAnimation(uControllerHash, uModifiedFlags, uPriority, uAnimationHash, pxBlend, pxParentGroup));
		}

		// Update functions...
		void UpdateBoundingBox (void)
		{
			if(m_uNumTimerUpdatesAtLastBoundingBoxUpdate != m_uNumTimerUpdates)
			{
				m_uNumTimerUpdatesAtLastBoundingBoxUpdate	= m_uNumTimerUpdates;
				if(m_pxRootControllerGroup->IsActive())
				{
					Asura_Assert( m_apxSkins[0], "UpdateBoundingBox() failed - NULL Skin pointer" );
					if ( m_apxSkins[0] )
					{
						if ( !m_pxRootControllerGroup->GetTweenStore().IsBoneDataValid( 0 ) )
						{
							m_pxRootControllerGroup->TweenBoneData(*m_apxSkins[0], 0, 0, 1.0f, false, NULL);
						}

						Asura_Vector_3 xRestPoseBoneZeroPos = Asura_Maths::ZeroVector3;
						if ( m_apxSkins[0]->BoneRestPose )
						{
							xRestPoseBoneZeroPos = m_apxSkins[0]->BoneRestPose[0].Position;
						}
						Asura_Vector_3 xBoneZeroPos = Asura_Maths::ZeroVector3;
						GetTweenStore()->GetBoneData(0).GetPosition( xBoneZeroPos );
						const Asura_Vector_3 xBoneZeroDisp = xBoneZeroPos - xRestPoseBoneZeroPos;
						
						m_xObjectSpaceBB.Reset();

						// Take account of skin BBs
						for ( u_int uSkinItr = 0; uSkinItr < m_uSkinCount && uSkinItr < ASURA_ANIMATION_SKIN_COUNT && m_apxSkins[ uSkinItr ] != NULL; ++uSkinItr )
						{
							Asura_Bounding_Box xSkinBB = m_apxSkins[ uSkinItr ]->GetBB();
							xSkinBB.Displace( xBoneZeroDisp );
							m_xObjectSpaceBB.Set( xSkinBB );
						}

						// Take account of hierarchy anim BBs. This will also calculate the velocities and movement offset in the tween store.
						m_pxRootControllerGroup->TweenBoundingBox(m_xObjectSpaceBB, *m_apxSkins[0], 1.0f, false, false, NULL);
					}
				}
			}
		}

		// Gets the current velocity from the primary animation (relative to the model)
		// If no movement data exists , then it will just be based on the Speed value.
		bool GetVelocity (Asura_Vector_3* pxOutput) const
		{
			Asura_Animation*	pxAnimation	= const_cast<Asura_Animation*>(this);

			pxAnimation->UpdateBoundingBox();
			return(m_pxRootControllerGroup->GetVelocity(*pxOutput));
		}

		// Gets the current angular velocity from the primary animation (relative to the model)
		// If no movement data exists , then the angle will be zero.
		bool GetAngularVelocity	(Asura_Vector_3* pxOutputAxis, Asura_Angle* pfOutputAngle) const
		{
			Asura_Animation*	pxAnimation	= const_cast<Asura_Animation*>(this);

			pxAnimation->UpdateBoundingBox();
			return(m_pxRootControllerGroup->GetAngularVelocity(*pxOutputAxis, *pfOutputAngle));
		}

        // Gets the offset from the start position of the primary animation from the movement data , if it exists.
        bool GetMovementOffset( Asura_Vector_3* pxOutputPos, Asura_Quat* pxOutputQuat = NULL ) const
        {
			Asura_Animation*	pxAnimation	= const_cast<Asura_Animation*>(this);

			pxAnimation->UpdateBoundingBox();
            return m_pxRootControllerGroup->GetMovementOffset( pxOutputPos, pxOutputQuat );
        }

		// Goes through all active animations and works out which flags are set...
		int	GetFlagsForAllAnimsLastFrame	(void) const;

		// Traversal functions...
		void	TraverseControllerAnimations	(ControllerAnimationContainer::TraverseWithUserDataFunction* pfnTraverse, void* pUserData = NULL)	{ m_xControllerAnimationContainer.Traverse(pfnTraverse, pUserData); }
		void	TraverseControllerGroups		(ControllerGroupContainer::TraverseWithUserDataFunction* pfnTraverse, void* pUserData = NULL)		{ m_xControllerGroupContainer.Traverse(pfnTraverse, pUserData); }

		// Accessor functions...
		Asura_AnimationController_Group&			GetRootControllerGroup	(void)			{ return(*m_pxRootControllerGroup); }
		const Asura_AnimationController_Group&		GetRootControllerGroup	(void) const	{ return(*m_pxRootControllerGroup); }

		Asura_AnimationController_Group*			GetControllerGroupFromHash		(const Asura_Hash_ID uHash)			{ Asura_AnimationController_Group** ppxGroup = m_xControllerGroupContainer.GetDataByHash(uHash); return((ppxGroup == NULL) ? NULL : *ppxGroup); }
		const Asura_AnimationController_Group*		GetControllerGroupFromHash		(const Asura_Hash_ID uHash) const	{ Asura_AnimationController_Group** ppxGroup = m_xControllerGroupContainer.GetDataByHash(uHash); return((ppxGroup == NULL) ? NULL : *ppxGroup); }

		Asura_AnimationController_Animation*		GetControllerAnimationFromHash	(const Asura_Hash_ID uHash)			{ Asura_AnimationController_Animation** ppxAnimation = m_xControllerAnimationContainer.GetDataByHash(uHash); return((ppxAnimation == NULL) ? NULL : *ppxAnimation); }
		const Asura_AnimationController_Animation*	GetControllerAnimationFromHash	(const Asura_Hash_ID uHash) const	{ Asura_AnimationController_Animation** ppxAnimation = m_xControllerAnimationContainer.GetDataByHash(uHash); return((ppxAnimation == NULL) ? NULL : *ppxAnimation); }

		Asura_Hierarchy_Tween*			GetTweenStore	(void)			{ return(&(m_pxRootControllerGroup->GetTweenStore())); }
		const Asura_Hierarchy_Tween*	GetTweenStore	(void) const	{ return(&(m_pxRootControllerGroup->GetTweenStore())); }

		void	InvalidateBonesUpdateFrameStamp(void);
		void	IncrementNumTimerUpdates(void)					{ m_uNumTimerUpdates++; }
		u_int	GetNumTimerUpdates(void) const					{ return m_uNumTimerUpdates; }
		u_int	GetNumTimerUpdatesAtLastBoneUpdate(void) const	{ return(m_uNumTimerUpdatesAtLastBoneUpdate); }

		// DEPRECATED - NEEDED FOR BACKWARDS COMPATABILITY FOR THE TIME BEING...
		Asura_Animation_PrimaryWrapper	m_xPrimaryWrapper;

		//////////////////////////////////////////////////////////////////////////
		// New animation system, it replaces old concepts of primary and 
		// secondary animations being blended together - ART 15/09/06.
		//////////////////////////////////////////////////////////////////////////

		//Plays sounds triggered during the last frame... dealt with during Update()
		void PlaySoundsForLastFrame() const;
		void PlaySound(const int iSoundID, const Asura_Hierarchy_Anim& xAnimation, const Asura_Vector_3& xPosition, u_int uSoundMarkerFlags) const;
		//Used by the animation system to tell the Asura_Aniamtion that the specified sound
		//should be playing
		void PlayLoopedSound(int iAnimSoundID, const Asura_Hierarchy_Anim& xAnimation) const;
		#ifdef ASURA_USE_DIALOGUE_SYSTEM
		//And for triggering dialogue events in the same way
		void PlayDialogue(int iAnimSoundID, const Asura_Hierarchy_Anim& xAnimation,Asura_Hash_ID uDialogueEvent) const;
		#endif


		//clears out all looping sounds on the anim
		void StopLoopedSounds();


		//gets the curent position and/or orientation of a marker point relative to the model (or in worldspace if requested)
		//pass null pointers for position/orientation if not required
		bool GetMarkerLocation( int markerIndex, int iSkinIndex, Asura_Vector_3* outputPosition, Asura_Matrix_3x3* outputMatrix, bool bWorldSpace=false ) const;
		bool GetMarkerLocation( const char* szMarkerName, Asura_Vector_3* pxOutputPosition, Asura_Matrix_3x3* pxOutputMatrix, bool bWorldSpace=false, int iAttachID=Asura_AnimationAttachment::INVALID_ID ) const;
		bool GetMarkerLocationFromHash( Asura_Hash_ID uMarkerHash, Asura_Vector_3* pxOutputPosition, Asura_Matrix_3x3* pxOutputMatrix, bool bWorldSpace=false, int iAttachID=Asura_AnimationAttachment::INVALID_ID ) const;

		// Gets the current position and/or orientation of a specific bone relative to the model.
		bool GetBoneLocation(int iBoneIndex, Asura_Vector_3 *pxOutputPosition, Asura_Matrix_3x3 *pxOutputMatrix, bool bWorldSpace = false) const;
        // Gets the specified bone's modelspace BB.
        const Asura_Bounding_Box* GetBoneBB( int iBoneIdx ) const;

		// Gets the closest bone to the specified position...
		int GetClosestBone(const Asura_Vector_3& vPosition);

		//Gets the bounding box from SectionBoundingBoxData.
		//This allows a manually set bounding box to be retrieved even if there is a collision mesh.
		//RAR 01/02/2007
		const Asura_Bounding_Box* GetManualBoneBB( int iBoneIdx ) const;

        // Similar to GetMarkerLocation() except these just check that this
        // anim or attached ones have the marker point in question. The
        // methods above can't be used for this purpose since they will
        // return false if the anim hasn't been rendered yet.
        bool HasMarker( const char* szMarkerName ) const;
        bool HasMarker( Asura_Hash_ID uMarkerNameHash ) const;

		//Link in an animation at a specified marker point. If deleteWithParent is true then the child animation will
		//be deleted when this animation gets deleted (assuming it is still linked in at that point).
		//iID is a user set ID passed to the attachment. Can be used for identifying it when hit by LOS tests.
		bool LinkAnimation(Asura_Animation* pxChildAnimation,const char* pszMarkerName,bool bDeleteWithParent,int iID = 0);
		// hash id based linking		
		bool LinkAnimation(Asura_Animation* pxChildAnimation,Asura_Hash_ID uMarkerHash,bool bDeleteWithParent,int iID = 0);
		//Bone relative linking.
		bool LinkAnimation_BoneRelativeObjectSpace(Asura_Animation* pxChildAnimation, const Asura_Vector_3& xObjectSpacePos, const Asura_Quat& xObjectSpaceOri, int iBoneID, bool bDeleteWithParent, int iID = 0);
		bool LinkAnimation_BoneRelativeBoneSpace(Asura_Animation* pxChildAnimation, const Asura_Vector_3& xBoneRelativePos, const Asura_Quat& xBoneRelativeOri, int iBoneID, bool bDeleteWithParent, int iID = 0);

		//remove a linked animation (Richard May - 22/04/2004 16:27:45 - now returns true if successfully unlinked)
		bool UnlinkAnimation(Asura_Animation* childAnimation);
		
		// removes all linked animations
		void UnlinkAllAnimations( bool bDeleteOnUnlink=true );
		
		//if false (default) timers are updated after the rendering to ensure that the first frame is seen.
		//if true the timers are updated before rendering(for my own devious purposes in cutscenes).
		bool TimersUpdatedBeforeRender;

#ifdef ASURA_TOOLS
		Asura_Hierarchy_Skin* GetSkin( u_int uSkin = 0 )				{ return m_apxSkins[uSkin]; }
#endif
		const Asura_Hierarchy_Skin* GetSkin( u_int uSkin = 0 ) const	{ return m_apxSkins[uSkin]; }
		Asura_Hash_ID GetSkinHashID( u_int uSkin = 0 ) const			{ return m_apxSkins[uSkin] ? m_apxSkins[uSkin]->GetHashID() : ASURA_HASH_ID_UNSET; }


		/// Richard May - 16/09/2002 11:01:42 - posable anim stuff
		bool m_bPosable; 
		bool IsPosableAnim(void) const { return m_bPosable; };

        
		// LOS test (per poly or BB only, depending on flag).

		// AJG - 18/04/08 - Up until now, if an animation had an invalid tween store (i.e. had never been updated) we'd just test against a bounding sphere, which may have caused some oddities but seemingly no major issues.
		//					However, on AVP we're now seeing some very strange IK behaviour which has been traced to the IK LOS hitting a particularly large object that needs to be initially active, but that doesn't get
		//					rendered immediately. To solve this, we can update the animation if we hit that situation, giving it a valid tween store. However, doing that requires making the los test function non-const.
		bool LOSTest(const Asura_Vector_3& xPosition, 
                     const Asura_Matrix_3x3& xOrient,
					 const class Asura_LOS_InputData* pxInput,
					 class Asura_LOS_OutputData* pxOutput);

        
        // Get the position of the zeroth bone, i.e. the offset from the compass point.
        // Given in model space so you need to offset by model pos and orientation in worldspace.
        // Returns false if the data is not available.
        bool GetOffsetFromCompassPoint(Asura_Vector_3& xPos) const;

		// Functions to get and set the animations alpha value.
		// When Alpha = 0.0 the animation is completly transparent.		When Alpha = 1.0 the animation is completly opaque
		void SetAlpha( const float fAlpha, bool bSetAttachments=false );
		float GetAlpha( void ) const {	return m_xRGBA.m_fA;	}
        void SetRGB( float fR, float fG, float fB, bool bPropogateToAttachments = true );

        void SetAnimTexID( int iTexID ) { m_iAnimTexID = iTexID; }
        int GetAnimTexID( void ) const { return m_iAnimTexID; }

		// Project-specific functions...
		typedef void ProjectInitialiseCallback(Asura_Animation *pAnim);
		typedef void ProjectShutdownCallback(Asura_Animation *pAnim);

		static void SetProjectInitialiseCallback( ProjectInitialiseCallback* pfnCallback )	{ s_pfnProjectInitialiseCallback = pfnCallback; }
		static void SetProjectShutdownCallback( ProjectShutdownCallback* pfnCallback )		{ s_pfnProjectShutdownCallback = pfnCallback; }

		void		SetProjectData(void* pProjectData)	{ m_pProjectData = pProjectData; }
		void*		GetProjectData(void)				{ return (m_pProjectData); }
		const void*	GetProjectData(void) const			{ return (m_pProjectData); }

#if defined( ASURA_USE_NEW_VISIBILITY_SYSTEM )
		bool IsVisible( bool bNoFrustumTest = false );
#else
        bool IsInModuleVisibleToCurCam( const Asura_Vector_3& xPos, const Asura_Matrix_3x3& xOrient ) const;

#if defined(ASURA_USE_STREAMING)
		Asura_Hash_ID GetContainingZoneFromModuleTest() const { return m_xModuleList.GetContainingZoneHash(); }
		// MGH 6/6/2006: The containing zone is normally updated in IsInModuleVisibleToCurCam, but if the
		// module visibility test is disabled then we can call this function instead.
		void UpdateContainingZone(const Asura_Vector_3& xPos, const Asura_Matrix_3x3& xOrient) const;
#else
        bool IsIntersectingModuleListValid( void ) const { return( USHRT_MAX != m_ausIntersectingModules[0] ); }
        const u_short* GetIntersectingModuleList( void ) const { return m_ausIntersectingModules; }
#endif //defined(ASURA_USE_STREAMING)
#endif //!defined( ASURA_USE_NEW_VISIBILITY_SYSTEM )
		
		void Render(const Asura_Vector_3& p, const Asura_Matrix_3x3& o,unsigned uRenderFlags);
		void RenderShadow( const Asura_Vector_3& xPos, const Asura_Matrix_3x3& xOri, const u_int uRenderFlags );

#if defined(ASURA_USE_INDEPENDENT_SHADOW_SYSTEM)
		static void AddShadowCallback(const Asura_ShadowInstance_CreationParams& xParams, const void* pxUserData);
		static void CalculateShadowBBCallback(const Asura_ShadowInstance_CreationParams& xParams, const void *pxUserData);
		void AddShadowBSToCoarseBS( float fShadowBSRad, const Asura_Vector_3& xShadowBSCentre );
#endif

        Asura_Guid GetParentEntityGUID( void ) const { return this->ParentEntityGuid; }
		Asura_Guid GetRootParentEntityGUID( void ) const { return m_pxParent ? m_pxParent->GetRootParentEntityGUID() : ParentEntityGuid; }

		//Texture animation controller stuff.

		//Changes the current texture anim sequence. If the sequence is different it will also reset the timer
		void SetTextureAnimSequence(u_int uSeq){m_xTextureAnimController.SetSequence(uSeq);}
		u_int GetTextureAnimSequence() const {return m_xTextureAnimController.GetSequence();}

		//Sets a timescale/speed multiplier on the texture animation.
		//A speed of zero will pause the anim.
		//Negative values will reverse it.
		void SetTextureAnimSpeed(float fSpeed){m_xTextureAnimController.SetSpeed(fSpeed);}
		float GetTextureAnimSpeed() const{return m_xTextureAnimController.GetSpeed();}

		const Asura_TextureAnimationController* GetTextureAnimController() const {return &m_xTextureAnimController;}

		// Set and enable the self-illumination.
		void SetSelfIllumination(float fContribution, float fColourise, const Asura_RGBA& xColour)
		{
			 m_xTextureAnimController.SetSelfIllumination(fContribution, fColourise, xColour);
		}
		// Enable/disable the self-illumination (used by strobing container lights).
		void SetSelfIlluminationEnable(bool bEnable) { m_xTextureAnimController.SetSelfIlluminationEnable(bEnable); }

		//updates cached positions/orientations for all attachments
		void UpdateAttachmentLocations() const;

		// Gets the Y coordinate of the ground under the animation. If no
        // environment is hit then the height reported will be
        // fASURA_ANIMATION_DEFAULT_MAX_GROUND_HEIGHT regardless of fMaxHeight.
		float GetGroundYPosition( const Asura_Vector_3* pxAnimPos, float* pfHeightAboveGround = NULL, float fMaxHeight = fASURA_ANIMATION_DEFAULT_MAX_GROUND_HEIGHT ) const;

        // returns the material ID of the ground under the animation
#if defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM
		Asura_Hash_ID GetGroundMaterialResponseHashID( const Asura_Vector_3* pxAnimPos ) const;

		Asura_Hash_ID m_uMaterialOverrideMapID;
		Asura_Hash_ID GetMaterialOverrideMapID() const { return m_uMaterialOverrideMapID; }
		void SetMaterialOverrideMapID(const Asura_Hash_ID uOverrideMapID) { m_uMaterialOverrideMapID = uOverrideMapID; }
#endif
        short GetGroundMaterialID( const Asura_Vector_3* pxAnimPos ) const;
		const Asura_Vector_3& GetGroundNormal( const Asura_Vector_3* pxAnimPos ) const;
		void  SetGroundFlagsForLOSTest( u_int uTestFor, u_int uIgnore ) { m_uGroundFlagsToTestFor = uTestFor; m_uGroundFlagsToIgnore = uIgnore; }

		//Gets the most recent position used
		const Asura_Vector_3& GetPosition() const {return m_xPosition;}
		//Gets the most recent orient used
		const Asura_Quat& GetOrientQuat() const {return m_xOrient;}

		//uRenderFlags taken from enum AsuraHierarchyRenderFlags in Asura_Hierarchy_System
		void Update(const Asura_Vector_3& p, const Asura_Matrix_3x3& o,float time = ANIMTIME_USEGAMETIME,unsigned uRenderFlags = 0);

		//Update the anim timers , without doing any rendering , or bone calculations
		//returns true if the timers were updated for this Asura_Animation (ignoring any attachments)
		bool UpdateTimers(float fTime = ANIMTIME_USEGAMETIME,bool bIgnoreTimerFrameStamp = false);

		//calculate/update the local photon list in Asura_Photon_System
		void UpdateLighting(u_int uRenderFlags = 0) const;
		//update for use by ragdoll code
		void UpdateLighting(const Asura_Vector_3& xLightingPosition) const;
		//invalidates the dynamic part of the stored lighting
		void ResetDynamicLighting() const;

		bool GetBoneRelativePositionAndOrientation(const Asura_Vector_3& xObjectSpacePos, const Asura_Quat& xObjectSpaceOri, int iBoneID, Asura_Vector_3& xBoneRelativePos, Asura_Quat& xBoneRelativeOri) const;

		const Asura_Animation* GetParent() const {return m_pxParent;}
		Asura_Animation*	   GetParent()		 {return m_pxParent;}

		//Allows the parent animation to be set , even if this isn't part of an animation attachment.
		//This is only relevant at the moment if a shape instance attachment that is also an animation
		//is used.
		void SetParent(Asura_Animation* pxParentAnimation) {m_pxParent = pxParentAnimation;} 
        
		Asura_AnimationAttachment* GetAttachmentFromMarkerIndex(int iMarkerIndex);
		const Asura_AnimationAttachment* GetAttachmentFromMarkerIndex( int iMarkerIndex ) const;

		Asura_AnimationAttachment* GetAttachmentByID(int iID);
		const Asura_AnimationAttachment* GetAttachmentByID( int iID ) const;

		Asura_AnimationAttachment* GetFirstAttachment(void) { return m_pxFirstAttachment; }
		const Asura_AnimationAttachment* GetFirstAttachment(void) const { return m_pxFirstAttachment; }

		void DestroyAllAttachments(void);

		void UpdateStoredPosition( const Asura_Vector_3& xCurPos ) const;

        bool GetNeverHasShadow( void ) const { return m_bNeverHasShadow; }
		void SetNeverHasShadow(bool bNeverHasShadow) {m_bNeverHasShadow = bNeverHasShadow;}

		bool GetNeverUseFrustumTest() const { return m_bNeverUseFrustumTest; }
		void SetNeverUseFrustumTest( bool bNeverUseFrustumTest ) { m_bNeverUseFrustumTest = bNeverUseFrustumTest; }

        void SetJustLightOnce( void ) { m_bJustLightOnce = true; }

		//Search this skin , and recursively through attachments , for one with the sound ID defined
		const class Asura_Hierarchy_Sound* GetSoundPropertiesFromAnimSoundID(int iAnimSoundID) const;
		//Same as above , but checks the animation if nothing is found in the skin and attachments
		const class Asura_Hierarchy_Sound* GetSoundPropertiesFromAnimSoundID(int iAnimSoundID, const Asura_Hierarchy_Anim& xAnimation) const;

		#ifdef ASURA_TOOLS
		//Sets colour for wireframe outline for showing selection in tools
		void SetOutlineColour(u_int uColour){m_uOutlineColour = uColour;}
		u_int GetOutlineColour() const {return m_uOutlineColour;}
		#endif

		// Functions to handle bounding-sphere calculation methods...
		void	SetCalculateBSUsingBones	(const bool bSet)	{ m_bCalculateBSUsingBones = bSet; }
		bool	CanCalculateBSUsingBones	(void) const		{ return(m_bCalculateBSUsingBones); }

#ifdef ASURA_USE_ANIMATION_IK_SOLVER
		void	SetIKSolver( Asura_Animation_IK_Solver* pxIKSolver ) { m_pxIKSolver = pxIKSolver; }
		void	SetIKUpdates( bool bDoUpdates ) { m_bDoIKUpdates = bDoUpdates; }
		void	InvalidateNumTimerUpdatesAtLastIKUpdate() { m_uNumTimerUpdatesAtLastIKUpdate = 0; }
#endif // ASURA_USE_ANIMATION_IK_SOLVER

		void	AddToUpdateTimersList( Asura_Animation_BoneDataModifier* pxModifier );
		void	AddToUpdateBoneDataList( Asura_Animation_BoneDataModifier* pxModifier );

		void	RemoveFromUpdateTimersList( Asura_Animation_BoneDataModifier* pxModifier );
		void	RemoveFromUpdateBoneDataList( Asura_Animation_BoneDataModifier* pxModifier );

	protected:
		void Initialise();

		// Function used by LOS/BB calculation functions to cache the world-space bone data...
		void UpdateBoneWorkspace (const Asura_Hierarchy_Tween& xTweenStore, const Asura_Vector_3& xPosition, const Asura_Matrix_3x3& xOrientation) const;

#ifdef ASURA_USE_ANIMATION_IK_SOLVER
		Asura_Animation_IK_Solver*	m_pxIKSolver;
		u_int						m_uNumTimerUpdatesAtLastIKUpdate;
		bool						m_bDoIKUpdates;
#endif // ASURA_USE_ANIMATION_IK_SOLVER

		Asura_List< Asura_Animation_BoneDataModifier > m_xUpdateTimersList;
		Asura_List< Asura_Animation_BoneDataModifier > m_xUpdateBoneDataList;

		u_int														m_uSkinCount;
		Asura_Hierarchy_Skin*										m_apxSkins[ASURA_ANIMATION_SKIN_COUNT];
		Asura_Bounding_Box	*										m_pxMergedSectionBB;
		Asura_FixedArrayOfBools<ASURA_MAX_NO_OF_BONES_PER_MODEL>	m_xMergedBBValidFlags;
		Asura_FixedArrayOfBools<ASURA_MAX_NO_OF_BONES_PER_MODEL>	m_xLOSFlags;

		u_int						FrameNumberStamp;
		u_int						m_uSoundFrameStamp;

		//////////////////////////////////////////////////////////////////////////
		// New animation system, it replaces old concepts of primary and 
		// secondary animations being blended together - ART 15/09/06.
		//////////////////////////////////////////////////////////////////////////
		ControllerGroupContainer					m_xControllerGroupContainer;				// Tree of group-controllers only, stored by their unique hash.
		ControllerAnimationContainer				m_xControllerAnimationContainer;			// Tree of animation-controllers only, stored by their hash.

		mutable	Asura_AnimationController_Group*	m_pxRootControllerGroup;					// We must always have at least one group otherwise we cannot play any animations.

		mutable	u_int								m_uNumTimerUpdates;							// The number of times we've updated our animation's timers.
		mutable	u_int								m_uNumTimerUpdatesAtLastBoundingBoxUpdate;	// The number of timer updates when we last updated our BB (and hence root-bone).
		mutable	u_int								m_uNumTimerUpdatesAtLastBoneUpdate;			// The number of timer updates when we last updated our bone-data.
		mutable	u_int								m_uNumTimerUpdatesAtLastTightBoundingBoxUpdate;

		// Static functions...
		static void	PlaySoundsForLastFrame_Traverse		(Asura_Hash_ID uHash, Asura_AnimationController_Animation* const* ppxControllerAnimation);

		static bool	GetFlagsForLastFrame_Traverse		(Asura_Hash_ID uHash, Asura_AnimationController_Animation* const* ppxControllerAnimation, void* pvCulmulativeFlags);
		//////////////////////////////////////////////////////////////////////////
		// New animation system, it replaces old concepts of primary and 
		// secondary animations being blended together - ART 15/09/06.
		//////////////////////////////////////////////////////////////////////////

		Asura_AnimationAttachment*  m_pxFirstAttachment;
		Asura_Guid					ParentEntityGuid;
		Asura_RGBA                  m_xRGBA; // RGB currently just used for ambient where appropriate.
        int                         m_iAnimTexID; // Optional override for texture ID to be used by skin. Handled in project-specific code.

		// Optional callback for project specific initialisation, called at the end if Initialise() if set
		static ProjectInitialiseCallback*	s_pfnProjectInitialiseCallback;
		static ProjectShutdownCallback*		s_pfnProjectShutdownCallback;

		// Store for project-specific data...
		void*	m_pProjectData;

		Asura_TextureAnimationController m_xTextureAnimController;

		mutable Asura_Vector_3		m_xPosition;//position used for last render/update
		Asura_Quat					m_xOrient;//and orientation
		mutable float				m_fHeightAboveGround; //cached difference between y coordinate of anim and ground underneath

#if defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM
		mutable Asura_Hash_ID		m_uGroundMaterialResponseHashID;
#endif
        mutable short               m_sGroundMaterialID;     // cached material id for where we're standing
        mutable Asura_Vector_3      m_xGroundNormal;         // cached normal                "
        u_int                       m_uGroundFlagsToTestFor;
        u_int                       m_uGroundFlagsToIgnore;

        union
        {
            u_short m_usAllBitFlags;

            struct 
            {
                mutable u_short m_bHeightAboveGroundValid : 1;
                u_short         m_bNeverHasShadow         : 1;
                u_short         m_bNeverUseFrustumTest    : 1;
                u_short         m_bJustLightOnce          : 1; // Anim will be lit when first rendered, using only static lighting, and it will never be updated.
				u_short			m_bCalculateBSUsingBones  : 1;	// Forces the bounding-sphere calculation to done by evaluating the bone data every frame instead of using the skin BB.
            };
        };

		Asura_Animation*			m_pxParent;//parent animation , if this is an attachment

#if !defined(ASURA_USE_STREAMING)
        static void IntersectingModuleCallback( const u_int uIdx );
#endif //!defined(ASURA_USE_STREAMING)

        enum { MAX_INTERSECTING_MODULES_TO_STORE = 8,
			   MAX_INTERSECTING_ZONES_TO_STORE   = 3 };

#if defined( ASURA_USE_NEW_VISIBILITY_SYSTEM )
		Asura_Visibility_Query	m_xVisibilityQuery;
#else

#if defined(ASURA_USE_STREAMING)
		mutable Asura_Streaming_ModuleList<MAX_INTERSECTING_MODULES_TO_STORE, MAX_INTERSECTING_ZONES_TO_STORE> m_xModuleList;
#else
        mutable u_short        m_ausIntersectingModules[MAX_INTERSECTING_MODULES_TO_STORE];
#endif //defined(ASURA_USE_STREAMING)

        mutable Asura_Vector_3 m_xCachedPos;
        mutable u_int          m_uNextFrameForFullTest;
#endif //# !defined( ASURA_USE_NEW_VISIBILITY_SYSTEM )

		//for caching lighting calculations for attachments
		mutable class Asura_Lighting_Solution* m_pxLightingSolution;

		#ifdef ASURA_TOOLS
		u_int						m_uOutlineColour;
		#endif

#ifdef ASURA_SUPPORT_SKIN_ELEMENTS
		int							m_iNumSkinElements;
		bool					   *m_pabSkinElementVisible;
	public:		
		bool						GetSkinElementVisibility(int iElement) const;
		void						SetSkinElementVisibility(int iElement, bool bVisible);
		int							GetNumSkinElements() const {return m_iNumSkinElements;}
#endif

#ifdef ASURA_USE_DAMAGE_MAPS
	protected:
		float					   *m_pafBoneDamage;
		int							m_iDamageMapBoneCount;
		bool						m_bIsUsingDamageMap;		
	public:
		bool						IsUsingDamageMap(void) const {return m_bIsUsingDamageMap;}
		float						GetBoneDamageMapValue(int iBone) const;
		void						SetBoneDamageMapValue(int iBone, float fValue);
		void						ResetDamageMapValues(void);
#endif

#if defined(ASURA_USE_ANIMATION_PLATFORM_RENDER_CACHE)
	public:
		Asura_Platform_Animation_RenderCache* GetRenderCache() const { return m_pxRenderCache; }
#endif

	protected:
		// In order to save space we're going to use a common centre for the bounding spheres, and adjust the radii if necessary.
		Asura_Vector_3				m_xBSCentre;

		// tight bounding sphere for shadow casting etc. will only be calculated (from bone data) if we're going to be rendered
		float						m_fTightBSRadius;

		// coarse bounding box used for L-O-S etc., 
		float						m_fCoarseBSRadius;

		// coarse bounding box including things like projector attachment and shadow frusta used for culling in the visibility system 
		float						m_fCoarseBSIncEffectsRadius;
		float						m_fPrevCoarseBSIncEffectsRadius;

		// BB used to calculate the above, better to have it here than in the tween store;
		Asura_Bounding_Box			m_xObjectSpaceBB;

		//linked list of looping animation sounds
		mutable Asura_Animation_SoundInstance*	m_pxFirstSoundInstance;

#if defined(ASURA_USE_ANIMATION_PLATFORM_RENDER_CACHE)
		//Cached data for rendering animation - currently only used on PS3
		mutable Asura_Platform_Animation_RenderCache*	m_pxRenderCache;
#endif
};


//used for storing the sound handles of looping sounds on animations
class Asura_Animation_SoundInstance
{
public :
	void Initialise()
	{
		m_iHandle = ASURA_INVALID_HANDLE;
		m_iAnimSoundID = -1;
		m_uEventHash = 0;
		m_uAnimHash = 0;
		m_bUpdatedThisFrame = false;
		m_bDialogue = false;
		m_bLoopedSound = false;
		m_pxNext = 0;
	}
	void StopSound();


	Asura_Handle m_iHandle;
	int m_iAnimSoundID;//refers to the sound IDs stored in the skins/animations
	Asura_Hash_ID m_uEventHash;//hash ID of sound/dialogue event to play
	Asura_Hash_ID m_uAnimHash;//hash ID of animation that triggered the sound


	u_int m_bUpdatedThisFrame : 1;//set each frame to determine which sounds are still needed
	u_int m_bDialogue : 1;//true if dialogue event , otherwise sound event
	u_int m_bLoopedSound : 1;//This is a looping sound event that should continue until a frame with happens with no update

	Asura_Animation_SoundInstance* m_pxNext;
};





#endif//__ASURA_ANIMATION_H__
