
///////////////////////////////////////////////////////////////////////////////
//
// File   : Asura_Animation_PrimaryWrapper.h
// Author : Andy Tate
// Date   : 19-Sep-2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __ASURA_ANIMATION_PRIMARYWRAPPER_H__
#define __ASURA_ANIMATION_PRIMARYWRAPPER_H__

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura_Handle.h"
#include "Core/Asura_Hash.h"
#include "Core/Asura_Maths.h"


////////////////////////////////////////////////////////////////////////////////
// Forward class declarations...
////////////////////////////////////////////////////////////////////////////////

class Asura_Animation;
class Asura_AnimationBlend;
class Asura_Hierarchy_Anim;
class Asura_AnimationController_Animation;


////////////////////////////////////////////////////////////////////////////////
// Defines...
////////////////////////////////////////////////////////////////////////////////

#define DEFAULT_TWEEN_TIME				0.2f
#define ASURA_PRIMARY_ANIMATION_HANDLE	0


////////////////////////////////////////////////////////////////////////////////
// Class declaration...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Class        : Asura_Animation_PrimaryWrapper
// Author       : Andy Tate
// Date         : 19-Sep-2006
//
// Description  : This class is effectively deprecated from the very start. It
//				  has been created in order to get the existing Tools/Asura code
//				  to compile whilst using the new animation system. Once the code
//				  is running successfully, the old animation system should be 
//				  removed along with this class and the code fixed to directly
//				  interface to the new animation system.
////////////////////////////////////////////////////////////////////////////////

class Asura_Animation_PrimaryWrapper
{
public:
	// Public functions...
	Asura_Animation_PrimaryWrapper  (void) : m_pxAnimation(NULL), m_uPrimaryControllerHash(ASURA_HASH_ID_UNSET)	{}
//	~Asura_Animation_PrimaryWrapper ();

	//////////////////////////////////////////////////////////////////////////
	// These functions all come from the old version of "Asura_Animation"...
	//////////////////////////////////////////////////////////////////////////
	Asura_Handle StartAnim(const Asura_Hash_ID uAnimHash, const bool bLooped, const float fTweenDuration, const bool bReversed = false, const float fTimeDelay = 0.0f);

	//new anim play function taking flags in an attempt to slow down the buildup of paramaters - RAR 17/06/02
	Asura_Handle StartAnim2(const Asura_Hash_ID uAnimHash, const int iAnimFlags, const float fTweenDuration = DEFAULT_TWEEN_TIME, const float fTimeDelay = 0.0f);

	//plays an animation between startTimer (which should be between 0 and 1) , and endTimer which can be any number according
	//to the number of loops desired. The reversed flag is set automatically if endTimer<startTimer
	//If the loop flag is set , then the end time is ignored , and direction is determined by the reversed flag
	Asura_Handle PlayAnimToPos(const Asura_Hash_ID uAnimHash, float fStartTimer, float fEndTimer, const int iAnimFlags, const float fTweenDuration = DEFAULT_TWEEN_TIME);

	//new anim play function taking flags in an attempt to slow down the buildup of parameters - RAR 17/06/02
	Asura_Handle AddOneShotAnim2(const Asura_Hash_ID uAnimHash, const float fWeight, const int iAnimFlags, const float fTweenDuration = DEFAULT_TWEEN_TIME ,const float fTimeDelay = 0.0f);

	//equivalent to PlanyAnimToPos for one shot anims
	Asura_Handle PlayOneShotAnimToPos(const Asura_Hash_ID uAnimHash, const float fWeight, float fStartTimer, float fEndTimer, const int iAnimFlags);

	void ChangeOneShotAnimWeighting(const Asura_Handle iHandle, const float fWeight);

	// One-shot removal functions...
	void RemoveOneShotAnim(const Asura_Handle iHandle);
	void RemoveAllOneShotAnims(void);

	// Accessor functions...
	void	SetLooped		(const bool bLooped, const Asura_Handle iHandle = ASURA_PRIMARY_ANIMATION_HANDLE);
	bool	GetLooped		(const Asura_Handle iHandle = ASURA_PRIMARY_ANIMATION_HANDLE) const;

	void	SetPaused		(const bool bPaused, const Asura_Handle iHandle = ASURA_PRIMARY_ANIMATION_HANDLE);
	bool	GetPaused		(const Asura_Handle iHandle = ASURA_PRIMARY_ANIMATION_HANDLE) const;

	void	SetMirrored		(const bool bMirrored, const Asura_Handle iHandle = ASURA_PRIMARY_ANIMATION_HANDLE);
	bool	GetMirrored		(const Asura_Handle iHandle = ASURA_PRIMARY_ANIMATION_HANDLE ) const;

	void	SetAnimDuration	(const float fDuration, const Asura_Handle iHandle = ASURA_PRIMARY_ANIMATION_HANDLE);
	float	GetAnimDuration	(const Asura_Handle iHandle = ASURA_PRIMARY_ANIMATION_HANDLE) const;

	void	SetTimer		(const float fTimer, const Asura_Handle iHandle = ASURA_PRIMARY_ANIMATION_HANDLE);
	float	GetTimer		(const Asura_Handle iHandle = ASURA_PRIMARY_ANIMATION_HANDLE) const;

	void	SetEndTime		(const float fEndTime, const Asura_Handle iHandle = ASURA_PRIMARY_ANIMATION_HANDLE);
	float	GetEndTime		(const Asura_Handle iHandle = ASURA_PRIMARY_ANIMATION_HANDLE) const;

	void	SetTimeDelay	(const float fTimeDelay, const Asura_Handle iHandle = ASURA_PRIMARY_ANIMATION_HANDLE);
	float	GetTimeDelay	(const Asura_Handle iHandle = ASURA_PRIMARY_ANIMATION_HANDLE) const;

	bool	HasFinished		(const Asura_Handle iHandle = ASURA_PRIMARY_ANIMATION_HANDLE) const;

	// Override the animation duration by comparing the desired speed with the speed stored in tha animation
	void SetAnimDurationBySpeed(const float fSpeed, const Asura_Handle iHandle = ASURA_PRIMARY_ANIMATION_HANDLE);

	// Change the time taken to tween. Won't make any difference if an animation has 
	// already finished tweening or was never flagged to tween in the first place.
	void SetTweenDuration(const float fTweenDuration, const Asura_Handle iHandle = ASURA_PRIMARY_ANIMATION_HANDLE);

	// Shortcut to the primary timer...
	float GetPrimaryTimer(void) const	{ return(GetTimer(ASURA_PRIMARY_ANIMATION_HANDLE)); }
	Asura_Hash_ID GetPrimaryAnimHashID(void) const;

	//Gets the current velocity from the primary animation (relative to the model)
	//If no movement data exists , then it will just be based on the Speed value.
	bool GetVelocity(Asura_Vector_3* pxOutput) const;

	//Gets the current angular velocity from the primary animation (relative to the model)
	//If no movement data exists , then the angle will be zero.
	bool GetAngularVelocity(Asura_Vector_3* pxOutputAxis,Asura_Angle* pfOutputAngle) const;

	//Gets the offset from the start position of the primary animation from the movement data , if it exists.
	bool GetMovementOffset(Asura_Vector_3* pxOutputPos,Asura_Quat* pxOutputQuat = 0) const;

	// There are no blended functions in the new animation system - everything is automatically
	// blended for you, so these will simply call the non-blended versions...
	bool GetBlendedVelocity(Asura_Vector_3* pxOutput) const											{ return(GetVelocity(pxOutput)); }
	bool GetBlendedAngularVelocity(Asura_Vector_3* pxOutputAxis,Asura_Angle* pfOutputAngle) const	{ return(GetAngularVelocity(pxOutputAxis, pfOutputAngle)); }
	bool GetBlendedMovementOffset(Asura_Vector_3* pxOutputPos,Asura_Quat* pxOutputQuat = 0) const	{ return(GetMovementOffset(pxOutputPos, pxOutputQuat)); }

	// Gets the flags the animation flags that were 'triggered' during the previous timer update
	int GetFlagsForLastFrame(const Asura_Handle iHandle = ASURA_PRIMARY_ANIMATION_HANDLE) const;
	int GetFlagsForAllAnimsLastFrame(void) const;


	// DEPRECATED...
	void StartAnim(Asura_Hash_ID uHashAnim, bool loop, bool tween,bool reversed = false);

	//plays a specified distance into an animation (0 to 1). If reversed , distance is taken from the end
	void PlayAnimToPos(Asura_Hash_ID uHashAnim, float distance, float tweenDuration,bool reversed = false);

	Asura_Handle AddOneShotAnim2(const Asura_Hierarchy_Anim* pxHierAnim, float weight, int animFlags ,float timeDelay = 0);

	void ChangeOneShotAnimPriority(Asura_Handle iHandle, int iPriority);		

	bool HasLoopedThisFrame(Asura_Handle iHandle = ASURA_PRIMARY_ANIMATION_HANDLE) const;

	float GetOneShotAnimWeighting( Asura_Handle iHandle );

	//change the use movement offset flag on an animation that is already playing
	void SetUseMovementOffset( bool bUseMoveOffset, Asura_Handle iHandle = ASURA_PRIMARY_ANIMATION_HANDLE );
	//get the use movement offset flag for an anim
	bool GetUseMovementOffset( Asura_Handle iHandle = ASURA_PRIMARY_ANIMATION_HANDLE ) const;

	// change the auto-remove flag for a secondary anim that's already playing
	void SetAutoRemove( bool bAutoRemove, Asura_Handle iHandle );

	// change the sync-with-primary flag for a secondary anim that's already playing 
	void SetSyncWithPrimary( bool bSyncWithPrimary, Asura_Handle iHandle );

	// change the tween out flag for a secondary anim that's already playing
	void SetTweenOut( bool bTweenOut, Asura_Handle iHandle );

	// Starts a secondary aninimation tweening out , while continuing to play as normal.
	// Once the tweening ends ,the animation will be flagged as finished (and will
	// be removed at that stage if autoremove has been set).
	// Does nothing if the animation is already tweening out.
	void StartAnimatedTweenOut(float fTweenDuration, Asura_Handle iHandle);
	// As above, but keeps the anim where it is.
	void StartTweenOut(float fTweenDuration, Asura_Handle iHandle);

	Asura_Hash_ID GetAnimHashID( Asura_Handle iAnimHandle ) const;

	// Animation set function...
	void	SetAnimation	(Asura_Animation* pxAnimation)	{ m_pxAnimation = pxAnimation; }


protected:
	// Protected variables...


	// Protected functions...


private:
	// Private variables...
	Asura_Animation*		m_pxAnimation;				// The animation to use - all operations are performed via this.
	Asura_Hash_ID			m_uPrimaryControllerHash;	// The hash-ID of the current primary animation-controller.

	static Asura_Hash_ID	s_uUniqueAnimationHash;		// We just keep incrementing this.

	// Private functions...

	Asura_Animation_PrimaryWrapper  (const Asura_Animation_PrimaryWrapper& xOther);

	Asura_Animation_PrimaryWrapper&	operator=	(const Asura_Animation_PrimaryWrapper& xOther);

	// Internal functions...
	Asura_AnimationController_Animation*		GetAnimationFromHandle	(const Asura_Handle iHandle);
	const Asura_AnimationController_Animation*	GetAnimationFromHandle	(const Asura_Handle iHandle) const;

	// Static functions...
	static	u_int					ConvertAnimFlagsToControllerFlags	(const int iAnimFlags);
	static	Asura_AnimationBlend*	ConvertAnimFlagsToBlend				(const int iAnimFlags, const float fTweenDuration);

	static	bool					DestroyOneShotAnimation_Traverse	(Asura_Hash_ID uHash, Asura_AnimationController_Animation** ppxAnimation, void* pAnimation);

};   // End of class 'Asura_Animation_PrimaryWrapper'.


#endif   // ifndef __ASURA_ANIMATION_PRIMARYWRAPPER_H__


/// End of file. ///
