//
// Asura_Hierarchy_BodyPart_System.h
//
// Created by Richard Rice - 9/10/02
// 
// Copyright (C) 2002 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef __ASURA_HIERARCHY_BODYPART_SYSTEM_H__
#define __ASURA_HIERARCHY_BODYPART_SYSTEM_H__

#include "Animation/Asura_Hierarchy_BodyPart_ProjectEnumeration.h"
#include "Core/Asura_Hash.h"

#ifdef ASURA_TOOLS
#include "Core/Asura_String.h"
#include "Misc/list_tem.hpp"
#endif


class Asura_Hierarchy_BodyType;
class Asura_Hierarchy_BodyPart;
class Asura_Hierarchy_DeathType;

#define ASURA_BODYPART_INVALID -1


class Asura_Hierarchy_BodyPart_System
{
public :
	static void Initialise();
	static void Shutdown();
	static void Reset();

	static void LoadAnimations();

	static Asura_Hash_ID GetDeathAnimation(const class Asura_Hierarchy_Skin* pxSkin,const char* pszSectionName,enum AsuraDeathType eDeathType);
	static Asura_Hash_ID GetDeathAnimation(Asura_Hash_ID uHashSkin,const char* pszSectionName,enum AsuraDeathType eDeathType);

	//get the body part associated with a section, or ASURA_BODYPART_INVALID if not found
	static int GetBodyPartID(const class Asura_Hierarchy_Skin* pxSkin,const char* pszSectionName);
	static int GetBodyPartID(Asura_Hash_ID uHashSkin,const char* pszSectionName);

	//loading/saving from chunks
	static int GetChunkBufferSize();
	static void LoadFromChunk(const class Asura_Chunk_Header* pxChunk,const char*& pcChunkBuffer);
	static void FillInChunkBuffer(char*& pcChunkBuffer);

	//and from a text file
	static void LoadFromIniFile(struct Tools_IniFile* pxIniFile);

	#ifdef ASURA_TOOLS
	static void ListAnimationsForSkins(const List<Asura_Hash_ID>& xHashList,List<Asura_String>& xNameList);
	#endif

private :
	static const Asura_Hierarchy_BodyType* GetBodyTypeForSkin(Asura_Hash_ID uHashSkin);
    static void LoadAnimation( const class Asura_String& xName, int iIndex );

	static int s_iNumberOfBodyTypes;
	static Asura_Hierarchy_BodyType* s_pxBodyTypes;

	static class Asura_NameTree* s_pxAnimationTree;//list of all animations used by the system

	friend class Asura_Chunk_DeathAnimFilenames;
	friend class Tools_Hierarchy_BodyPart_System;

};

//Each BodyType contains a list of skins that use the same assignment of bones to body parts
class Asura_Hierarchy_BodyType
{
public :
	Asura_Hierarchy_BodyType();
	~Asura_Hierarchy_BodyType();

	const Asura_Hierarchy_BodyPart* GetBodyPart(Asura_Hash_ID uHashSection) const;
	bool UsedBySkin(Asura_Hash_ID uHashSkin) const;

	//loading/saving from chunks
	int GetChunkBufferSize();
	void LoadFromChunk(const class Asura_Chunk_Header* pxChunk,const char*& pcChunkBuffer);
	void FillInChunkBuffer(char*& pcChunkBuffer);

	//and from a text file
	void LoadFromIniFile(struct Tools_IniFile* pxIniFile);

	#ifdef ASURA_TOOLS
	void ListAnimations(List<Asura_Hash_ID>& xAnimHashList) const;
	#endif

private :
	int m_iNumberOfSkins;
	Asura_Hash_ID* m_puSkins;

	int m_iNumberOfBodyParts;
	Asura_Hierarchy_BodyPart* m_pxBodyParts;
};

//Each BodyPart contains a list of sections that belongs to the part , and a list of death types
//with animations
class Asura_Hierarchy_BodyPart
{
public :
	Asura_Hierarchy_BodyPart();
	~Asura_Hierarchy_BodyPart();

	const Asura_Hierarchy_DeathType* GetDeathType(enum AsuraDeathType eDeathType) const;
	int GetBodyPartType() const {return m_iBodyPartType;}
	bool ContainsSection(Asura_Hash_ID uHashSection) const;


	//loading/saving from chunks
	int GetChunkBufferSize();
	void LoadFromChunk(const class Asura_Chunk_Header* pxChunk,const char*& pcChunkBuffer);
	void FillInChunkBuffer(char*& pcChunkBuffer);

	//and from a text file
	void LoadFromIniFile(struct Tools_IniFile* pxIniFile);

	#ifdef ASURA_TOOLS
	void ListAnimations(List<Asura_Hash_ID>& xAnimHashList) const;
	#endif

private :
	int m_iBodyPartType;

	int m_iNumberOfSections;
	Asura_Hash_ID* m_puSections;

	int m_iNumberOfDeathTypes;
	Asura_Hierarchy_DeathType* m_pxDeathTypes;	
};

//Each DeathType contains a list of animations that can be played for that type of death
class Asura_Hierarchy_DeathType
{
public :
	Asura_Hierarchy_DeathType();
	~Asura_Hierarchy_DeathType();

	Asura_Hash_ID PickAnimation() const;
	int GetDeathType() const {return m_iDeathType;}

	//loading/saving from chunks
	int GetChunkBufferSize();
	void LoadFromChunk(const class Asura_Chunk_Header* pxChunk,const char*& pcChunkBuffer);
	void FillInChunkBuffer(char*& pcChunkBuffer);

	//and from a text file
	void LoadFromIniFile(struct Tools_IniFile* pxIniFile);

	#ifdef ASURA_TOOLS
	void ListAnimations(List<Asura_Hash_ID>& xAnimHashList) const;
	#endif


private :
	int m_iDeathType;

	int m_iNumberOfAnimations;
	Asura_Hash_ID* m_puAnimations;
};









#endif //__ASURA_HIERARCHY_BODYPART_SYSTEM_H__
