//
// Asura_Attractor_Compiled.h
//
// Created by JGP 10/05/06
//
// Copyright (C) 2006 Rebellion (Derby), All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __ASURA_ATTRACTOR_COMPILED_H__
#define __ASURA_ATTRACTOR_COMPILED_H__


// required
#include "Asura_Attractor_Data.h"
#include "Asura_Attractor_Node_Data.h"


class Asura_Vector_3;
/*
	Asura_Attractor_Compiled

	Storage for a single attractor
	compiled. Contains two end nodes. 
*/
class Asura_Attractor_Compiled : public Asura_Attractor_Data
{
	friend class Asura_Chunk_Attractors;

public:


	Asura_Attractor_Node_Data* GetNode( const int iIndex ) { Asura_Assert( ( iIndex >= 0 ) && ( iIndex < 2 ), "GetNode index out of range 0-1" ); return m_axNodes + iIndex; }
	const Asura_Attractor_Node_Data* GetNode( const int iIndex ) const { Asura_Assert( ( iIndex >= 0 ) && ( iIndex < 2 ), "GetNode index out of range 0-1" ); return m_axNodes + iIndex; }

	void SetLinks(u_int uNumLinks,u_int* puLinks){m_uNumLinks = uNumLinks;m_puLinks = puLinks;}

	// Returns true if xPos is inside the cylinder.
	// Cylinder dimensions can be adjusted using:
	// fEndAdjustment - increases the length at either end by this many metres. Can be negative to decrease the length instead.
	// fRadiusOverride - uses this radius instead of the attractor's radius. Ignored if less than or equal to zero.
	bool Contains( const Asura_Vector_3& xPos, float fEndAdjustment = 0.f, float fRadiusOverride = 0.f) const;
	
	// Gives information about the xPos relative to the attractor's cylinder:
	// *pfDistanceAlongAxis - distance along the axis from node 0 to node 1
	// *pxClosestPointOnAxis - closest point on axis to xPos
	// *pfDistanceFromAxis - distance between closest point on axis and xPos (i.e. magnitude of *pxVectorFromAxis)
	// *pxVectorFromAxis - vector from closest point on axis to xPos
	// Use fEndAdjustment to increase (or decrease, using a negative value) the length of the cylinder at either end (same as its use in Contains)
	// Use bClampClosestPosToEnds to clamp the closest point to the cylinder caps.
	void GetRelativePosition(	const Asura_Vector_3& xPos,
								float fEndAdjustment = 0.f,
								float* pfDistanceAlongAxis = NULL,
								Asura_Vector_3* pxClosestPointOnAxis = NULL,
								float* pfDistanceFromAxis = NULL,
								Asura_Vector_3* pxVectorFromAxis = NULL,
								bool bClampClosestPosToEnds = true ) const;
	float GetAxisLength() const;
	// Returns normalised direction from node 0 to node 1.
	void GetAxisDirection(Asura_Vector_3& xDirection) const;
	// Returns true if the ray segment intersects the cylinder.
	bool LOSTest( const Asura_Vector_3& xPos, const Asura_Vector_3& xDir, float fDistance) const;
	// Returns true if the attractor could intersect the world-space bounding box
	// This will do lazy evaluation which forms a bounding box around the attractor and tests against that.
	// It is reasonably quick, and is expected to be used as a first pass collecting attractors before
	// paring down the results in the callback. Doing an exact cylinder vs. BB check would be much more complex.
	bool CouldIntersect(const Asura_Bounding_Box& xBB) const;

	void SetNormal(const Asura_Vector_3& xNormal) { m_xNormal = xNormal; }
	virtual void GetNormal(Asura_Vector_3& xNormal) const { xNormal = m_xNormal; }

private:

	Asura_Attractor_Node_Data m_axNodes[ 2 ];

	// A normal for the attractor, which many interactions are going to need
	Asura_Vector_3 m_xNormal;
};


#endif // __ASURA_ATTRACTOR_COMPILED_H__
