//
// Asura_Attractor_System.h
//
// Created by JGP 03/05/06
//
// Copyright (C) 2006 Rebellion (Derby), All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __ASURA_ATTRACTOR_SYSTEM_H__
#define __ASURA_ATTRACTOR_SYSTEM_H__

#ifdef ASURA_USE_ATTRACTORS

#include "Debugging/Asura_Assert.h"
#include "Core/Asura_LBAABB_Tree.h"
#include "Core/Asura_HashedList.h"
#include "Core/Asura_HashTree.h"
#include "Asura_Attractor_Compiled.h"
#include "Asura_Attractor_EntityRelative.h"

class Asura_Message;


// callback
// The streaming version also provides the hash ID of the handler that owns the attractor
#ifdef ASURA_USE_STREAMING

typedef void (*AttractorCallback)( void* pData, const Asura_Attractor_Compiled* const pxAttractor, Asura_Hash_ID uHandlerID );
#define ATTRACTOR_CALLBACK( _class_, _fn_ ) \
	static void _ ## _fn_( void* pData, const class Asura_Attractor_Compiled* const pxAttractor, Asura_Hash_ID uHandlerID ) { ( static_cast<_class_*>( pData ) )->_fn_( pxAttractor, uHandlerID ); } \
	void _fn_( const class Asura_Attractor_Compiled* const pxAttractor, Asura_Hash_ID uHandlerID ) \


#else

typedef void (*AttractorCallback)( void* pData, const Asura_Attractor_Compiled* const pxAttractor );
#define ATTRACTOR_CALLBACK( _class_, _fn_ ) \
	static void _ ## _fn_( void* pData, const class Asura_Attractor_Compiled* const pxAttractor ) { ( static_cast<_class_*>( pData ) )->_fn_( pxAttractor ); } \
	void _fn_( const class Asura_Attractor_Compiled* const pxAttractor ) \

#endif


/////////////////////////////////////////////////////////////////////////////////////////////////////
/*-------------------------**
** Asura_Attractor_Handler **
**-------------------------*/
class Asura_Attractor_Handler
{

	friend class Asura_Chunk_Attractors;
	friend class Asura_Chunk_Attractors_InGame;
	friend class Tools_Attractor_System;
	friend class Asura_Attractor_System;

public:

	void Reset();


	// construct / destruct

	Asura_Attractor_Handler();
	~Asura_Attractor_Handler();

	// Collect Attractors

	//Get an attractor by index
	const Asura_Attractor_Compiled* GetAttractor( const int iIndex ) const { return m_pxCompiledAttractors + iIndex; }
	int GetNumAttractors() const { return m_iNumCompiledAttractors; }

	//Get an attractor by ID 
	Asura_Attractor_Compiled* GetAttractorByID(u_int uID)  {return m_xAttractorList.FindObject(uID);}
	const Asura_Attractor_Compiled* GetAttractorByID(u_int uID) const {return m_xAttractorList.FindObject(uID);}

	void CollectAttractors( const Asura_Vector_3& xPos, const Asura_Vector_3& xDir, const float fMinDistance, const float fMaxDistance, AttractorCallback pfnCallback, void* pCallbackData = NULL );
	void CollectAttractors( const Asura_Bounding_Box& xBB, AttractorCallback pfnCallback, void* pCallbackData = NULL );	

	#ifdef ASURA_USE_STREAMING
	Asura_Hash_ID GetHandlerID() const {return m_uHashID;}
	#endif

private:



	// functions

	void DestroyTree();

	static void CollectAttractorsCallback( const unsigned int uIndex );
	static void CollectAttractorsBBCallback( const unsigned int uIndex );
	static Asura_Attractor_Handler* s_pxHandlerForCallback;

	// static callback functions

	AttractorCallback m_pfnCallback;
	void* m_pCallbackData;

	// vars

	//fixed attractors
	int m_iNumCompiledAttractors;
	Asura_Attractor_Compiled* m_pxCompiledAttractors;

	//movable attractors
	int m_iNumRelativeAttractors;
	Asura_Attractor_EntityRelative* m_pxRelativeAttractors;



	Asura_LBAABB_Tree m_xAttractorTree;

	Asura_Hashed_List<Asura_Attractor_Compiled> m_xAttractorList;

	//storage for all links used by attractors
	u_int m_uNumLinks;
	u_int* m_puLinks;

	#ifdef ASURA_USE_STREAMING
	//this handler's ID
	Asura_Hash_ID m_uHashID;
	#endif

};


/////////////////////////////////////////////////////////////////////////////////////////////////////
/*------------------------**
** Asura_Attractor_System **
**------------------------*/

/*
	Asura_Attractor_System

	Storage class for attractors
	and attractor nodes.
*/
class Asura_Attractor_System
{

	friend class Asura_Chunk_Attractors;
	friend class Tools_Attractor_System;

public:

	static Asura_Attractor_System* Initialise();
	static void Shutdown();
	static void Reset();

	static Asura_Attractor_System* GetInstance() { Asura_Assert( s_pxInstance != NULL, "Asura_Attractor_System not created" ); return s_pxInstance; }


	// Collect Attractors

	//get attractor by index
	const Asura_Attractor_Compiled* GetAttractor( const int iIndex ) { return GetCurrentHandler().GetAttractor(iIndex); }
	int GetNumAttractors() { return GetCurrentHandler().GetNumAttractors(); }

	//Get an attractor by ID  (from the current zone/handler)
	const Asura_Attractor_Compiled* GetAttractorByID(u_int uID){return GetCurrentHandler().GetAttractorByID(uID);}

	#ifdef ASURA_USE_STREAMING
	//Get an attractor from a specified handler
	const Asura_Attractor_Compiled* const GetAttractorByID(u_int uID, Asura_Hash_ID uHandlerID);
	#endif

	//Searches for attractors across all handlers
	void CollectAttractors( const Asura_Vector_3& xPos, const Asura_Matrix_3x3& xOrient, const float fMinDistance, const float fMaxDistance, AttractorCallback pfnCallback, void* pCallbackData = NULL );
	void CollectAttractors( const Asura_Vector_3& xPos, const Asura_Vector_3& xDir, const float fMinDistance, const float fMaxDistance, AttractorCallback pfnCallback, void* pCallbackData = NULL );
	// Takes a world-space axis-aligned bounding box. This is a fairly loose test for intersection;
	// it is expected that the results will be filtered down in the callback.
	void CollectAttractors( const Asura_Bounding_Box& xBB, AttractorCallback pfnCallback, void* pCallbackData = NULL );	

	void HandleMessage(const Asura_Message* pxMessage);

	#if defined(ASURA_USE_STREAMING)
	// Handler functions...
	Asura_Attractor_Handler*	CreateHandler			(const Asura_Hash_ID uHash);
	bool						DestroyHandler			(const Asura_Hash_ID uHash);

	void						CreateAndSetHandler		(const Asura_Hash_ID uHash)	{ m_pxCurrentHandler = CreateHandler(uHash); }
	void						SetCurrentHandler		(Asura_Attractor_Handler* pxHandler);
	void						SetCurrentHandler		(const Asura_Hash_ID uHash)	{ m_pxCurrentHandler = m_xHandlerTree.GetDataByHash(uHash); }
	Asura_Attractor_Handler&	GetCurrentHandler		(void)						{ return(*m_pxCurrentHandler); }
	Asura_Attractor_Handler*	GetHandler				(const Asura_Hash_ID uHash)	{ return(m_xHandlerTree.GetDataByHash(uHash)); }
	#else
	Asura_Attractor_Handler&	GetCurrentHandler		(void)						{ return(m_xHandler); }
	#endif	// defined(ASURA_USE_STREAMING)

private:

	// static vars
	static Asura_Attractor_System* s_pxInstance;

	#ifdef ASURA_USE_STREAMING
	static bool CollectAttractorsCallback(Asura_Hash_ID uHash, Asura_Attractor_Handler* pxHandler, void* pData);
	static bool CollectAttractorsBBCallback(Asura_Hash_ID uHash, Asura_Attractor_Handler* pxHandler, void* pData);
	#endif
	
	//streaming handlers
	#if defined(ASURA_USE_STREAMING)
	Asura_Attractor_Handler*						m_pxCurrentHandler;		// Currently active handler.
	Asura_TemplateHashTree<Asura_Attractor_Handler>	m_xHandlerTree;			// Tree of all of the handlers available.
	#else
	Asura_Attractor_Handler							m_xHandler;				// Single instance of a zone.
	#endif


};

#endif // #ifdef ASURA_USE_ATTRACTORS

#endif // __ASURA_ATTRACTOR_SYSTEM_H__
