//*******************************************************************
//
// File: Asura_Botanical_Game_Render_Type.h  
//
// Created: 30 Nov 2006
//
// Description: 
//
//*******************************************************************

#ifndef _ASURA_BOTANICAL_GAME_RENDER_TYPE_H_
#define _ASURA_BOTANICAL_GAME_RENDER_TYPE_H_

// other libraries
#include "Botanicals/Asura_Botanical_Game_Render_Constants.h"
#include "Botanicals/Asura_Botanical_Game_Render_Dynamic_Memory_Layout.h"
#include "Botanicals/Asura_Botanical_Game_Render_StateTable.h"
#include "Botanicals/Asura_Botanical_Game_Render_TypeID.h"
#include "Botanicals/Asura_Botanical_Game_Render_Type_BasicInteractive.h"
#include "Botanicals/Asura_Botanicals_Array.h"

#include "Core/Asura_HashTree.h"
#include "Core/Asura_Maths.h"

#if defined( ASURA_TOOLS)
	#include "Core/Asura_String.h"
#endif
// internal dependencies

// forward declarations
class Asura_BGR_Instances_Header;
class Asura_BGR_StateTable;
class Asura_BGR_Type_BasicInteractive;
class Asura_Chunk_Stream;

class Tools_BotanicalShape;

//*******************************************************************
// Constants
//*******************************************************************


//*******************************************************************
// Class Definitions
//*******************************************************************

class Asura_BGR_Type
{
public:
	Asura_BGR_Type();
	virtual ~Asura_BGR_Type() = 0;
	
	Asura_BGR_TypeID GetType() const;
	void SetType( const Asura_BGR_TypeID xType);

	bool Process( Asura_Chunk_Stream& xStream);
#if defined( ASURA_TOOLS)
	bool Export( Asura_Chunk_Stream& xStream) const;
#endif
	
	const Asura_Bounding_Box& GetBoundingBox() const;
	void SetBoundingBox( Asura_Bounding_Box& xBoundingBox);
	
	float GetBoundingRadius() const;
	void SetBoundingRadius( const float fBoundingRadius);
	
#if defined( ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA) && 1 == ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA
	const Asura_BGR_DynamicMemoryLayout& GetDynamicDataLayout() const;
	Asura_BGR_DynamicMemoryLayout& GetDynamicDataLayout();
#endif
	
#if defined( ASURA_BOTANICAL_SUPPORT_INTERACTIVE)
	const Asura_BGR_Type_BasicInteractive* GetBasicInteractive() const;
	Asura_BGR_Type_BasicInteractive* CreateBasicInteractive();
	
	void ClearBasicInteractive();
#endif

#if defined( ASURA_BOTANICAL_SUPPORT_STATES) && 1 == ASURA_BOTANICAL_SUPPORT_STATES
	u_int GetStateCount() const;
	Asura_BGR_State_ID GetStateID( const Asura_Hash_ID& xHashID) const;
	#if defined( ASURA_TOOLS)
		Asura_BGR_State_ID GetStateID( const char* pszStateName) const;
	#endif

	const Asura_BGR_StateTable& GetStateTable(const Asura_BGR_State_ID xStateID) const	{ return(m_xArray_xStateTables[xStateID]); }

	void SetSectionVersionFromState( const Asura_BGR_State_ID xStateID, const Asura_BGR_State_Value xStateValue, Asura_BGR_Instances_Header* pxInstanceHeader, Asura_BGR_DynamicInstance* pxDyanmicInstance) const;
	#if defined( ASURA_TOOLS)
		bool CreateStates( const Tools_BotanicalShape& xBotanicalToolShape);
		bool CreateStateTables( const Tools_BotanicalShape& xBotanicalToolShape);
	#endif
#endif

#if defined( ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS) && 1 == ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS
	virtual u_int GetSectionCount() const { return( 0);}
	virtual Asura_BGR_Section_ID GetSectionID( const Asura_Hash_ID& xHashID) const { Asura_Assert( false, ""); return( ASURA_BGR_SECTION_ID_INVALID);}
	virtual Asura_BGR_Version_ID GetVersionID( const Asura_Hash_ID& xHashID) const { Asura_Assert( false, ""); return( ASURA_BGR_VERSION_ID_OFF);}
	
	#if defined( ASURA_TOOLS)
		virtual Asura_BGR_Section_ID GetSectionID( const char* pszSectionName) const { Asura_Assert( false, ""); return( ASURA_BGR_SECTION_ID_INVALID);}
		virtual Asura_BGR_Version_ID GetVersionID( const char* pszVersionName) const { Asura_Assert( false, ""); return( ASURA_BGR_VERSION_ID_OFF);}
	#endif
#endif

private:
#if defined( ASURA_BOTANICAL_SUPPORT_STATES) && 1 == ASURA_BOTANICAL_SUPPORT_STATES
	void SetSectionVersionFromStateTableSection( const Asura_BGR_StateTable_Section& xStateTableSection, const Asura_BGR_State_Value xPrevStateValue, const Asura_BGR_State_Value xStateValue, Asura_BGR_Instances_Header* pxInstanceHeader, Asura_BGR_DynamicInstance* pxDyanmicInstance) const;
#endif
	
protected:
	Asura_BGR_Type( const Asura_BGR_Type& xRHS);
	Asura_BGR_Type& operator=( const Asura_BGR_Type& xRHS);
	

protected:
	
	// DLH - 23 January 2007 09:47:16 ---------------------------------------------------
	// DLH - 23 January 2007 09:47:16 - Remeber to add any new members to the operator=()
	// DLH - 23 January 2007 09:47:16 ---------------------------------------------------

	Asura_BGR_TypeID m_xTypeID;
	
	Asura_Bounding_Box m_xBoundingBox;
	float m_fBoundingRadius;	

private:
#if defined( ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA) && 1 == ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA
	Asura_BGR_DynamicMemoryLayout m_xDynamicDataLayout;
#endif

#if defined( ASURA_BOTANICAL_SUPPORT_INTERACTIVE)
	Asura_BGR_Type_BasicInteractive* m_pxBasicInteractive;
#endif

#if defined( ASURA_BOTANICAL_SUPPORT_STATES) && 1 == ASURA_BOTANICAL_SUPPORT_STATES
	Asura_TemplateHashTree< Asura_BGR_State_ID> m_xHashTree_xStateID;
#endif
	
#if defined( ASURA_BOTANICAL_SUPPORT_STATES) && 1 == ASURA_BOTANICAL_SUPPORT_STATES
	// DLH - 05 December 2006 11:57:43 - Array of 1D Tables in order of StateID
	Asura_Botanical_Array< Asura_BGR_StateTable> m_xArray_xStateTables;
#endif

#if defined( ASURA_BOTANICAL_SUPPORT_STATES) && 1 == ASURA_BOTANICAL_SUPPORT_STATES
	static const Asura_BGR_StateTable_Version *s_pxPreviousCollidableVersion;
#endif

public:

#if defined( ASURA_BOTANICAL_SUPPORT_STATES) && 1 == ASURA_BOTANICAL_SUPPORT_STATES
	const Asura_BGR_StateTable_Version *GetPreviousCollidableVersion(void) const
	{
		const Asura_BGR_StateTable_Version *pxVersion = s_pxPreviousCollidableVersion;
		s_pxPreviousCollidableVersion = NULL;
		return pxVersion;
	}
#endif

#if defined( ASURA_TOOLS)
	Asura_String m_xName;
	const Tools_BotanicalShape* m_pxBotanicalToolShape;	
#endif	

};

#if defined( ASURA_TOOLS)
class Tools_BGR_Type : public Asura_BGR_Type
{
public:
	Tools_BGR_Type( const Tools_BotanicalShape& xCurrShape);
	
#if defined( ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS) && 1 == ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS
	virtual u_int GetSectionCount() const;
	virtual Asura_BGR_Section_ID GetSectionID( const Asura_Hash_ID& xHashID) const;
	virtual Asura_BGR_Version_ID GetVersionID( const Asura_Hash_ID& xHashID) const;
#endif

private:
#if defined( ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS) && 1 == ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS
	bool CreateSectionAndVersionData( const Tools_BotanicalShape& xBotanicalToolShape);
#endif
	
protected:
#if defined( ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS) && 1 == ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS
	Asura_TemplateHashTree< Asura_BGR_State_ID> m_xHashTree_xSectionID;
	Asura_TemplateHashTree< Asura_BGR_Version_ID> m_xHashTree_xVersionID;
#endif

};
#endif

#if defined( ASURA_TOOLS) && defined( ASURA_BOTANICAL_SUPPORT_INTERACTIVE) && 1 == ASURA_BOTANICAL_SUPPORT_INTERACTIVE
class Tools_BGR_Type_BoneWeightCalc
{
public:
	Tools_BGR_Type_BoneWeightCalc();
	
	void SetupBoneWeight( const Asura_BGR_Type_BasicInteractive& xTypeBasicInteractive);
	float CalculateBoneWeight( const Asura_AVector3& xPosition) const;
	
private:
	Asura_AVector4 m_xBoneWeightPlane;
	float m_fBoneWeightGamma;
};
#endif

//*******************************************************************
// Inline Function Definitions
//*******************************************************************
inline Asura_BGR_Type::Asura_BGR_Type()
{
#if defined( ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA) && 1 == ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA
	this->m_xDynamicDataLayout.Clear();
#endif

#if defined( ASURA_BOTANICAL_SUPPORT_INTERACTIVE)
	this->m_pxBasicInteractive = NULL;
#endif

#if defined( ASURA_TOOLS)
	this->m_pxBotanicalToolShape = NULL;
#endif
}

inline Asura_BGR_Type::~Asura_BGR_Type()
{
#if defined( ASURA_BOTANICAL_SUPPORT_INTERACTIVE)
	delete this->m_pxBasicInteractive;
	this->m_pxBasicInteractive = NULL;
#endif
}

inline Asura_BGR_Type& Asura_BGR_Type::operator=( const Asura_BGR_Type& xRHS)
{
	this->m_xTypeID = xRHS.m_xTypeID;
	this->m_xBoundingBox = xRHS.m_xBoundingBox;
	this->m_fBoundingRadius = xRHS.m_fBoundingRadius;

#if defined( ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA) && 1 == ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA
	this->m_xDynamicDataLayout = xRHS.m_xDynamicDataLayout;
#endif

#if defined( ASURA_BOTANICAL_SUPPORT_INTERACTIVE)
	this->ClearBasicInteractive();
	if( NULL != xRHS.m_pxBasicInteractive)
	{
		this->m_pxBasicInteractive = this->CreateBasicInteractive();
		*(this->m_pxBasicInteractive) = *(xRHS.m_pxBasicInteractive);
	}
#endif

#if defined( ASURA_BOTANICAL_SUPPORT_STATES) && 1 == ASURA_BOTANICAL_SUPPORT_STATES
	this->m_xHashTree_xStateID = xRHS.m_xHashTree_xStateID;
	this->m_xArray_xStateTables = xRHS.m_xArray_xStateTables;
#endif
	
#if defined( ASURA_TOOLS)
	this->m_xName = xRHS.m_xName;
	this->m_pxBotanicalToolShape = xRHS.m_pxBotanicalToolShape;
#endif

	return( *this);
}

inline Asura_BGR_TypeID Asura_BGR_Type::GetType() const
{
	return( this->m_xTypeID);
}

inline void Asura_BGR_Type::SetType( const Asura_BGR_TypeID xType)
{
	this->m_xTypeID = xType;
}

inline const Asura_Bounding_Box& Asura_BGR_Type::GetBoundingBox() const
{
	return( this->m_xBoundingBox);
}

inline void Asura_BGR_Type::SetBoundingBox( Asura_Bounding_Box& xBoundingBox)
{
	this->m_xBoundingBox = xBoundingBox;
}

inline float Asura_BGR_Type::GetBoundingRadius() const
{
	return( m_fBoundingRadius);
}

inline void Asura_BGR_Type::SetBoundingRadius( const float fBoundingRadius)
{
	this->m_fBoundingRadius = fBoundingRadius;
}

#if defined( ASURA_BOTANICAL_SUPPORT_INTERACTIVE)
inline const Asura_BGR_Type_BasicInteractive* Asura_BGR_Type::GetBasicInteractive() const
{
	return( this->m_pxBasicInteractive);
}
#endif

#if defined( ASURA_BOTANICAL_SUPPORT_INTERACTIVE)
inline Asura_BGR_Type_BasicInteractive* Asura_BGR_Type::CreateBasicInteractive()
{
	delete this->m_pxBasicInteractive;
	this->m_pxBasicInteractive = new Asura_BGR_Type_BasicInteractive;
	
	return( this->m_pxBasicInteractive);
}
#endif

#if defined( ASURA_BOTANICAL_SUPPORT_INTERACTIVE)
inline void Asura_BGR_Type::ClearBasicInteractive()
{
	delete this->m_pxBasicInteractive;
	this->m_pxBasicInteractive = NULL;
}
#endif

#if defined( ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA) && 1 == ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA
inline const Asura_BGR_DynamicMemoryLayout& Asura_BGR_Type::GetDynamicDataLayout() const
{
	return( this->m_xDynamicDataLayout);
}
#endif

#if defined( ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA) && 1 == ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA
inline Asura_BGR_DynamicMemoryLayout& Asura_BGR_Type::GetDynamicDataLayout()
{
	return( this->m_xDynamicDataLayout);
}
#endif

#if defined( ASURA_BOTANICAL_SUPPORT_STATES) && 1 == ASURA_BOTANICAL_SUPPORT_STATES
inline Asura_BGR_State_ID Asura_BGR_Type::GetStateID( const Asura_Hash_ID& xHashID) const
{
	Asura_BGR_State_ID* pxStateID = this->m_xHashTree_xStateID.GetDataByHash( xHashID);
	if( NULL == pxStateID)
	{
		return( ASURA_BGR_STATE_ID_INVALID);
	}
	
	return( *pxStateID);
}
#endif

#if (defined( ASURA_BOTANICAL_SUPPORT_STATES) && 1 == ASURA_BOTANICAL_SUPPORT_STATES) && defined( ASURA_TOOLS)
inline Asura_BGR_State_ID Asura_BGR_Type::GetStateID( const char* pszStateName) const
{
	Asura_Hash_ID xStateHashID = Asura_GetHashID( pszStateName);
	
	return( this->GetStateID( xStateHashID));
}
#endif

#if defined( ASURA_BOTANICAL_SUPPORT_STATES) && 1 == ASURA_BOTANICAL_SUPPORT_STATES
inline u_int Asura_BGR_Type::GetStateCount() const
{
	return( this->m_xArray_xStateTables.Size());
}
#endif

//*******************************************************************
//
// (c) 2006 Rebellion
//
// Warning: This computer program is protected by copyright law and 
// international treaties. Unauthorized reproduction or distribution
// of this program, or any portion of it, may result in severe civil and
// criminal penalties, and will be prosecuted to the maximum extent 
// possible under the law.
//
//*******************************************************************

#endif // _ASURA_BOTANICAL_GAME_RENDER_TYPE_H_
