
/**
*
* \file Asura_Botanicals_ShapeManager.cpp
*
* Created by Dave Reed - 09/01/2005
*
* Copyright (C) 2006 Rebellion (Derby), All Rights Reserved.
*
* This is the factory-type thing that manages Asura_BotanicalShapes.
*
* Note that due to the way chunk files work, it's possible that we'll have a BotanicalShape
* created with no renderable data - for example, if the definition file (.bdf) has been
* loaded, but the matching shape doesn't exist... So be extra careful checking pointers
* aren't null...
*
**/

#include "Core/Asura.h"

#ifdef ASURA_USE_BOTANICALS

#ifdef ASURA_TOOLS

#include "Core/Asura_Hash.h"
#include "Botanicals/Asura_Botanicals_ShapeManager.h"

// Static members
Asura_LinkList<Asura_BotanicalShape *> Asura_Botanicals_ShapeManager::m_xShapeList;


void Asura_Botanicals_ShapeManager::Initialise(void)
{

}


void Asura_Botanicals_ShapeManager::Shutdown(void)
{
	// Shutdown is the same as a reset - go through the list removing all shapes
	Reset();
}


void Asura_Botanicals_ShapeManager::Reset(void)
{
	for(Asura_ListElt<Asura_BotanicalShape *> *pxElt=m_xShapeList.Front();pxElt;pxElt=pxElt->Next())
	{
		Asura_BotanicalShape *pxShape=pxElt->Data();
		
		if(pxShape)
		{
			delete pxShape;
		}
	}

	m_xShapeList.Clear();
}



void Asura_Botanicals_ShapeManager::RemoveShape(Asura_BotanicalShape *pxShape)
{
	Asura_ListElt<Asura_BotanicalShape *> *pxEltToRemove=NULL;

	for(Asura_ListElt<Asura_BotanicalShape *> *pxElt=m_xShapeList.Front();pxElt;pxElt=pxElt->Next())
	{
		if (pxShape==pxElt->Data())
		{
			pxEltToRemove=pxElt;
		}
	}

	if (pxEltToRemove)
	{
		m_xShapeList.Remove(pxEltToRemove);
	}
}


Asura_BotanicalShape* Asura_Botanicals_ShapeManager::AddShape(Asura_Hash_ID uNameHash)
{
	Asura_BotanicalShape*	pxShape=new Asura_BotanicalShape();
	pxShape->SetHashID(uNameHash);
	m_xShapeList.Push_Back(pxShape);
	return pxShape;
}


Asura_BotanicalShape* Asura_Botanicals_ShapeManager::GetShapeByHash(Asura_Hash_ID uHashID)
{	
	for(Asura_ListElt<Asura_BotanicalShape *> *pxElt=m_xShapeList.Front();pxElt;pxElt=pxElt->Next())
	{
		Asura_BotanicalShape *pxShape=pxElt->Data();

		if (pxShape->GetHashID()==uHashID)
		{
			return pxShape;
		}
	}
	return NULL;
}

// Need to be careful here to init everything to NULL (or sensible values)
Asura_BotanicalShape::Asura_BotanicalShape()
{
#ifdef ASURA_TOOLS
	m_pToolsShape=NULL;
#endif
}



Asura_BotanicalShape::~Asura_BotanicalShape()
{	

#ifdef ASURA_TOOLS
	if (m_pToolsShape)
	{	
		delete m_pToolsShape;
	}
#endif
}

#endif // #ifdef ASURA_TOOLS

#endif // ASURA_USE_BOTANICALS
