
/**
*
* \file Asura_Botanicals_TemplateManager.cpp
*
* Created by Dave Reed - 08/11/2005
*
* Copyright (C) 2005 Rebellion (Derby), All Rights Reserved.
*
**/


#include "Core/Asura.h"

#ifdef ASURA_USE_BOTANICALS

#ifdef ASURA_TOOLS

#include "Core/Asura_Hash.h"
#include "Botanicals/Asura_Botanicals_TemplateManager.h"
#include "Core/Asura_Material_System.h"
#include "Core/Asura_TextureManagement.h"
#include "Debugging/Asura_Assert.h"
#include "ChunkSystem/Asura_Chunk_Botanicals_Templates.h"
#include "Shapes/Asura_Shape_System.h"

#ifdef ASURA_TOOLS
#include "Misc/Tools_MessageLog.h"
#include "Shapes/Tools_Shapes.h"
#include "PC/FileSystem/Tools_FileSystem.h"
#include "BotanicalCreation/Tools_Mode_BotanicalCreation.h"
#include "FileAccess/Asura_FileAccess.h"
#endif

#if defined( ASURA_BOTANICALS_BGR_IN_EDIT)
#include "Botanical/Asura_Direct3D_Botanicals_Game_Render_Manager.h"
#include "Botanical/Asura_Direct3D_Botanicals_Game_Render.h"
#endif

int						   Asura_Botanicals_TemplateManager::s_iNumTemplates=0;
Asura_Botanicals_Template *Asura_Botanicals_TemplateManager::s_pxTemplates=NULL;
int						   Asura_Botanicals_TemplateManager::s_iNextUID=0;


void Asura_Botanicals_TemplateManager::Initialise()
{
	Asura_Botanicals_TemplateManager::Shutdown();
}



void Asura_Botanicals_TemplateManager::Reset()
{
	Shutdown();
	Initialise();
}


void Asura_Botanicals_TemplateManager::Shutdown()
{
	if (s_pxTemplates)
	{
		delete [] s_pxTemplates;
		s_pxTemplates=0;
		s_iNumTemplates=0;
	}
}

	
int	 Asura_Botanicals_TemplateManager::GetNumTemplates(void)
{
	return s_iNumTemplates;
}


Asura_Botanicals_Template *Asura_Botanicals_TemplateManager::GetTemplate(int iIdx)
{
	if (iIdx>=0 && iIdx<s_iNumTemplates)
	{
		return &s_pxTemplates[iIdx];
	}
	else
	{
		Asura_Assert(false,"Requested invalid botanicals template!");
		return NULL;
	}
}


void Asura_Botanicals_TemplateManager::SetTemplateName(Asura_Botanicals_Template* pTemplate,char* szName)
{
	if (pTemplate)
	{
		pTemplate->xTemplateName=szName;		
	}
}


char *Asura_Botanicals_TemplateManager::GetTemplateName(int iIdx)
{
	if (iIdx>=0 && iIdx<s_iNumTemplates)
	{
		return (char *)s_pxTemplates[iIdx].xTemplateName.GetString();
	}
	else
	{
		Asura_Assert(false,"Requested invalid botanicals template!");
		return "<invalid!>";
	}
}



// (Re)allocate space for the templates when they're being loaded by the chunk system
void Asura_Botanicals_TemplateManager::AllocateTemplates(int iNumTemplates)
{
	Shutdown();
	s_iNumTemplates=iNumTemplates;
	s_pxTemplates=new Asura_Botanicals_Template[s_iNumTemplates+1];
}



int Asura_Botanicals_TemplateManager::GetTemplateIndexFromUID(int iUID)
{
	for(int iIdx=0;iIdx<s_iNumTemplates;iIdx++)
	{
		if (s_pxTemplates[iIdx].iUID==iUID)
		{
			return iIdx;
		}
	}
 
	// Not found
	return -1;
}


// Tools-only functions beyond this point!
// ------------------------------------------------------------------------------------------------

#ifdef ASURA_TOOLS
bool Asura_Botanicals_TemplateManager::Save(char* szTemplatesFileName, char* szStylesFileName)											
{
	Asura_Chunk_Botanicals_Styles::Create(szStylesFileName);
	return Asura_Chunk_Botanicals_Templates::Create(szTemplatesFileName);	
}
#endif

// This should be tools-only
void Asura_Botanicals_TemplateManager::LoadResources(void)
{
#ifdef ASURA_TOOLS
	for (int i=0;i<s_iNumTemplates;i++)
	{
		Asura_Botanicals_Template *pxTemplate=Asura_Botanicals_TemplateManager::GetTemplate(i);		

		{
			if (pxTemplate->eType==ASURA_BOTANICALS_TEMPLATE_SHAPE)
			{			
				if (true == pxTemplate->xShapeName.IsBlank())
				{
					pxTemplate->uShapeID = ASURA_HASH_ID_UNSET;
					continue;
				}
				
				char* pszShapeName = pxTemplate->xShapeName.GetString();
				
				#ifdef ASURA_PC_ALLOW_LOAD_FROM_NETWORK
					//Determine if the network version should be used instead
					char szFilename[260];
					Asura_FileAccess::PlatformGetNetworkFilenameOrGetLocalCopy( pszShapeName, szFilename, 260);
					pszShapeName = szFilename;
				#endif //ASURA_PC_ALLOW_LOAD_FROM_NETWORK
				
				if(Tools_FileSystem::FileExists(pszShapeName))
				{					
					pxTemplate->uShapeID = Tools_ShapeTable::AddShapeByFilename( pszShapeName, false);

					if( pxTemplate->uShapeID == ASURA_HASH_ID_UNSET )
					{
						Tools_MessageLog::AddMessage( "Botanicals",
													  "Missing Shapes",
													  "Unable to load shape %s needed by botanical template %s.",
													  pxTemplate->xShapeName.GetString(),
													  pxTemplate->xTemplateName.GetString() );

						Tools_MessageLog::ShowOverlay();
					}
				}
			}

			if (pxTemplate->eType==ASURA_BOTANICALS_TEMPLATE_BOTANICALSHAPE)
			{
				if (true == pxTemplate->xShapeName.IsBlank())
				{
					pxTemplate->pxBotanicalShape = NULL;
				}
				else
				{
					pxTemplate->pxBotanicalShape = Tools_Mode_BotanicalCreation::CreateBotanicalShape(pxTemplate->xShapeName.GetString(),false);
				}

				if (0 == pxTemplate->xSelectionShapeName.GetLength())
				{
					pxTemplate->uSelectionShapeID = ASURA_HASH_ID_UNSET;
				}
				else
				{				
					char* pszSelectionShapeName = pxTemplate->xSelectionShapeName.GetString();
					
					#ifdef ASURA_PC_ALLOW_LOAD_FROM_NETWORK
						//Determine if the network version should be used instead
						char szFilename[260];
						Asura_FileAccess::PlatformGetNetworkFilenameOrGetLocalCopy( pszSelectionShapeName, szFilename, 260);
						pszSelectionShapeName = szFilename;
					#endif //ASURA_PC_ALLOW_LOAD_FROM_NETWORK
					
					if(Tools_FileSystem::FileExists( pszSelectionShapeName))
					{					
						pxTemplate->uSelectionShapeID = Tools_ShapeTable::AddShapeByFilename( pszSelectionShapeName,false);						
					}
					else
					{
						Asura_Assertf( false == pxTemplate->bMergeCollisionMeshWithEnv, ("Did not find shape '%s' for Botanical Template '%s'. The Render mesh will be merged into the collision data!", pxTemplate->xSelectionShapeName.GetString(), pxTemplate->xTemplateName.GetString()));
						Tools_MessageLog::AddMessage( "Botanicals",
													  "Missing Shapes",
													  "Unable to load selection shape %s needed by botanical template %s.",
													  pxTemplate->xSelectionShapeName.GetString(),
													  pxTemplate->xTemplateName.GetString() );

						Tools_MessageLog::ShowOverlay();
					}
				}
			}			
		}
	}

	Asura_Shape_System::LoadAllShapes();
#endif
}



int	 Asura_Botanicals_TemplateManager::CreateTemplate(ASURA_BOTANICALS_TEMPLATE_TYPE eType)
{
	int iIdx=s_iNumTemplates;

	Asura_Botanicals_Template *pxNewTemplates=new Asura_Botanicals_Template[s_iNumTemplates+1];

	if (s_iNumTemplates>0)
	{	
		for(int i=0;i<s_iNumTemplates;i++)
		{
			// Copy them into the new buffer
			pxNewTemplates[i]=s_pxTemplates[i];
		}		
	}
	// Delete the old buffer
	delete [] s_pxTemplates;

	s_iNumTemplates++;
	s_pxTemplates=pxNewTemplates;

	// Set the defaults for the template
	memset(&s_pxTemplates[iIdx],0,sizeof(Asura_Botanicals_Template));
	s_pxTemplates[iIdx].eType=eType;
	s_pxTemplates[iIdx].iUID=s_iNextUID++;
	s_pxTemplates[iIdx].xTemplateName="unnamed";

	// Defaults for the editor params
	s_pxTemplates[iIdx].bVisible=true;
	s_pxTemplates[iIdx].bLocked=false;
	s_pxTemplates[iIdx].bMergeRenderMeshWithEnv=false;
	s_pxTemplates[iIdx].bMergeCollisionMeshWithEnv=false;
	s_pxTemplates[iIdx].bPermanentlyMergeMesh=false;
	s_pxTemplates[iIdx].bLocked=false;
	s_pxTemplates[iIdx].fMinScale=1.0f;
	s_pxTemplates[iIdx].fMaxScale=1.0f;
	s_pxTemplates[iIdx].fMinYRotation=0;
	s_pxTemplates[iIdx].fMaxYRotation=360.0f;
	s_pxTemplates[iIdx].fCollisionRadius=0.0f;
	s_pxTemplates[iIdx].fCollisionHeight=0.0f;
	s_pxTemplates[iIdx].pxBotanicalShape=NULL;
	s_pxTemplates[iIdx].uShapeID=ASURA_HASH_ID_UNSET;
	s_pxTemplates[iIdx].uSelectionShapeID=ASURA_HASH_ID_UNSET;

	return iIdx;
}



void Asura_Botanicals_TemplateManager::ToggleTemplateVisible(int iIdx)
{
	Asura_Botanicals_Template& xBotanicalTemplate = s_pxTemplates[iIdx];
	
	xBotanicalTemplate.bVisible = !xBotanicalTemplate.bVisible;
	
#if defined( ASURA_BOTANICALS_BGR_IN_EDIT)
	do
	{
		Asura_Direct3D_BGR_Manager& xBGRManager = Asura_Direct3D_Botanicals_Game_Render::GetDirect3DBGRManager();
		
		// DLH - 17 October 2006 15:12:30 - Find if the Shape already exists
		Asura_BGR_TypeID xTypeID( xBotanicalTemplate.iUID);
		if( xTypeID == Asura_BGR_TypeID::INVALID_ID)
		{
			continue;
		}
		
		Asura_Direct3D_BGR_Type* pxBGRType = xBGRManager.GetDirect3DType( xTypeID);
		if( NULL == pxBGRType)
		{
			continue;
		}
		
		pxBGRType->m_bHidden = (false == xBotanicalTemplate.bVisible);
	}
	while( false);
#endif
}


bool Asura_Botanicals_TemplateManager::GetTemplateVisible(int iIdx)
{
	return s_pxTemplates[iIdx].bVisible;
}


// Caution... After calling this all template indexes in the instances will need remapping!
void Asura_Botanicals_TemplateManager::RemoveTemplate(int iIdx)
{
#if defined( ASURA_BOTANICALS_BGR_IN_EDIT)
	do
	{
		Asura_Botanicals_Template* pxBotanicalTemplate = Asura_Botanicals_TemplateManager::GetTemplate( iIdx);
		if( NULL == pxBotanicalTemplate)
		{
			continue;
		}
		
		if (pxBotanicalTemplate->eType != ASURA_BOTANICALS_TEMPLATE_BOTANICALSHAPE)
		{
			continue;
		}
		
		Asura_BotanicalShape* pxBotanicalShape = pxBotanicalTemplate->pxBotanicalShape;
		if ( NULL == pxBotanicalShape)
		{
			continue;		
		}
		
		Tools_BotanicalShape* pxBotanicalToolShape = pxBotanicalShape->GetToolsShape();
		if( NULL == pxBotanicalToolShape)
		{
			continue;
		}
		
		Asura_Direct3D_BGR_Manager& xBGRManager = Asura_Direct3D_Botanicals_Game_Render::GetDirect3DBGRManager();
		
		// DLH - 17 October 2006 15:12:30 - Find if the Shape already exists
		Asura_BGR_TypeID xTypeID( pxBotanicalTemplate->iUID);
		if( xTypeID == Asura_BGR_TypeID::INVALID_ID)
		{
			continue;
		}
		
		xBGRManager.DeleteTypeID( xTypeID);
	}
	while( false);
#endif

	if (s_iNumTemplates>1)
	{
		s_iNumTemplates--;
		Asura_Botanicals_Template *pxNewTemplates=new Asura_Botanicals_Template[s_iNumTemplates];

		if (s_iNumTemplates>0)
		{	
			for(int i=0;i<s_iNumTemplates;i++)
			{
				// Copy them into the new buffer
				if (i<iIdx)
				{
					pxNewTemplates[i]=s_pxTemplates[i];
				}
				else
				{
					pxNewTemplates[i]=s_pxTemplates[i+1];
				}
			}
		}
	
		// Delete the old buffer
		delete[]s_pxTemplates;
		
		if (s_iNumTemplates>0)
		{
			s_pxTemplates=pxNewTemplates;
		}
	}
	else
	{
		// Delete the old buffer
		delete [] s_pxTemplates;
		s_pxTemplates=0;
		s_iNumTemplates=0;
	}

}

#endif // #ifdef ASURA_TOOLS

#endif //ASURA_USE_BOTANICALS

