//////////////////////////////////////////////////////////////////////////////
//
// Asura_FlyCam.h
//
// Copyright (c) 2006 Rebellion. All rights reserved.
//
// Description:
//
//     Basic camera controlled by analogue sticks or buttons. Intended only
//     for debugging and prototyping purposes. 
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef ___ASURA_FLYCAM_H___
#define ___ASURA_FLYCAM_H___ 1

#ifdef ASURA_SUPPORT_FLYCAM

#include "Input/Asura_ControlMap.h"
#include "Input/Asura_ControlMapping_Key.h"
#include "Core/Asura_Maths.h"

class Asura_FlyCam
{
public:
	enum Mappings
	{
		MOVE_FORWARDS_BACKWARDS,
		MOVE_LEFT_RIGHT,
		MOVE_UP_DOWN,

		ROLL,		// Rotation about Z axis
		PITCH,		// Rotation about X axis
		YAW,		// Rotation about Y axis

		FAST,
		EXTRA_FAST,
		SLOW,

		RESET_ROLL,

		INVERT_Y,

		MAX_MAPPINGS
	};

	static void	Initialise(void);
	static void	Shutdown(void);

	// update moves the Asura_Camera according to the controls.
	static void	Update(void);

	// Accessor functions...
	static void	SetInvertY(const bool bSet)	{ s_bInvertY = bSet; }


protected:
	// Protected functions...
	static void AddMoveForwardsBackwardsMapping(const int iAxis, const float fScale = 1.0f);
	static void AddMoveForwardsBackwardsMapping(const int iForwardsKey, const int iBackwardsKey);
	static void AddMoveLeftRightMapping(const int iAxis, const float fScale = 1.0f);
	static void AddMoveLeftRightMapping(const int iLeftKey, const int iRightKey);
	static void AddMoveUpDownMapping(const int iAxis, const float fScale = 1.0f);
	static void AddMoveUpDownMapping(const int iUpKey, const int iDownKey);

	static void AddRollMapping(const int iAxis, const float fScale = 1.0f);
	static void AddRollMapping(const int iAntiClockwiseKey, const int iClockwiseKey);
	static void AddPitchMapping(const int iAxis, const float fScale = 1.0f);
	static void AddPitchMapping(const int iAntiClockwiseKey, const int iClockwiseKey);
	static void AddYawMapping(const int iAxis, const float fScale = 1.0f);
	static void AddYawMapping(const int iAntiClockwiseKey, const int iClockwiseKey);

	static void	AddFastMapping(const int iKey1, const int iKey2);
	static void	AddFastMapping(const int iKey);
	static void	AddSlowMapping(const int iKey);

	//FR This one is only needed because some 360 pad driver only have triggers associated to an axis
	//rather than to buttons
	static void AddSlowMapping(const int iAxis, const float fScale);

	static void	AddInvertYMapping(const int iKey);
	static void	AddResetRollMapping(const int iKey);


private:
	// static data
	static Asura_ControlMap	s_xControlMap;

	static bool				s_bInvertY;
	static Asura_Angle		s_fCameraTurnSpeed;
	static float			s_fCameraMoveSpeedSlow;
	static float			s_fCameraMoveSpeedNormal;
	static float			s_fCameraMoveSpeedFast;

	// Private functions...
	static void	Platform_Initialise(void);
	static void	Platform_Shutdown(void);
};

#endif // #ifdef ASURA_SUPPORT_FLYCAM

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

#endif // #ifndef ___ASURA_FLYCAM_H___
