
/**
*
* \file Asura_Carpet_MeshManager.h
*
* Created by Dave Reed - 1/12/2005
*
* Copyright (C) 2005 Rebellion (Derby), All Rights Reserved.
*
**/


#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifdef ASURA_USE_CARPET

#ifndef __ASURA_CARPET_MESHMANAGER_H__
#define __ASURA_CARPET_MESHMANAGER_H__

#include "Core/Asura_Maths.h"
#include "Core/Asura_String.h"
#include "Core/Asura_Hash.h"
#include "Core/Asura_LinkList.h"
#include "Core/Asura_Handle.h"
#include "ChunkSystem/Asura_SaveBlock.h"

// JAT 13/02/06 this needs to be a power of 2
#define ASURA_CARPET_MESH_DATA_FRAME_SIZE_WAVEX 16



// Note - These vertex/triangle structs are used by the tools, as well as this temporary
//        system. When this module is replaced with something efficient, using proper 'cooked' data,
//        these structs will need to remain, for the tools

struct Asura_Carpet_Vertex
{
	Asura_Vector_3 xPos;
	float          fDensity;

#if defined( ASURA_DIRECT3D_BGR_SUPPORT_SM30)
	float          afColour[4];
#else
	uint32         uColour;
#endif

#if 1 && defined( ASURA_TOOLS)
	Asura_Vector_3 m_xNormal;
	u_int m_uModule;
	u_int m_uPoly;
	u_int m_uVtx;
#endif
};

struct Asura_Carpet_Triangle
{
	Asura_Carpet_Vertex	   xVerts[3];	
	float				   fArea;			// Area of triangle, in metres squared
	int					   iStyle;			// Grass style ID
	int					   iStyleUID;

	uint32				   uSeed;			// Random seed

	Asura_Vector_3		   xCentre;			// Centre/Radius for sphere-based culling
	float				   fRadius;	
};


class Asura_Carpet_MeshManager
{
	friend class Tools_Mode_Carpet;

public:
	static void Initialise();	
	static void Shutdown();
	static void Reset(); 		
	static void Render();
	static void SetFade( const float fFade );
	static float GetFade( void );

#ifdef ASURA_TOOLS
	static void RemapStyles();					// Updates indices when the style list has changed

	static void ConvertFromToolData();			// When leaving carpet edit mode
	static void ConvertToToolData(bool bUseNewFindScenePolyBlock = true);			// When entering carpet edit mode

	static void Save();
	
	// Functions called from the chunk system to load/save the mesh
	static void Deserialize(int iVertsion, Asura_Handle iFileHandle);	
	static int  GetCurrentChunkVersion(void);
	static void Serialize(Asura_Chunk_SaveBlock_Stream &xStream);

	static int GetNumTriangles( void ) { return s_iNumTriangles; }
	static Asura_LinkList<Asura_Carpet_Triangle>& GetTriangleList()
	{
		return s_xTriangleList;
	}
	
	static void UpdateLighting();
	
#endif // #ifdef ASURA_TOOLS

	static float GetMovementSpeed()				{return s_fMovementSpeed;}
	static void  SetMovementSpeed(float fSpeed) {s_fMovementSpeed = fSpeed;}

#if defined(ASURA_SUPPORT_COMMAND_CONSOLE)
	static void AddConsoleCommands( void );
	static bool ConsoleCmd_SetFade( float fFade, bool bInterrogate, bool bHelp );
#endif // #if defined(ASURA_SUPPORT_COMMAND_CONSOLE)

private:
	static void Platform_Render( const float fFade );
	static void Platform_Initialise( void );
	static void Platform_Shutdown( void );
	static void Platform_Reset( void );

#ifdef ASURA_TOOLS
	static void Platform_MakeDataSet( Asura_LinkList<Asura_Carpet_Triangle>& xTriangleList );

	static Asura_LinkList<Asura_Carpet_Triangle> s_xTriangleList;
	static int								     s_iNumTriangles;
#endif // #ifdef ASURA_TOOLS

	static float s_fFade;
	static float s_fMovementSpeed;
};

inline void Asura_Carpet_MeshManager::SetFade( const float fFade )
{
	s_fFade = fFade;
}

inline float Asura_Carpet_MeshManager::GetFade( void )
{
	return s_fFade;
}

#endif // __ASURA_CARPET_MESHMANAGER_H__

#endif // ASURA_USE_CARPET
