
/**
*
* \file Asura_Carpet_StyleManager.cpp
*
* Created by Dave Reed - 23/11/2005
*
* Copyright (C) 2005 Rebellion (Derby), All Rights Reserved.
*
**/

#include "Core/Asura.h"

#ifdef ASURA_USE_CARPET

#ifdef ASURA_TOOLS

#include "Core/Asura_Hash.h"
#include "Core/Asura_Material_System.h"
#include "Core/Asura_TextureManagement.h"
#include "Carpet/Asura_Carpet_StyleManager.h"
#include "ChunkSystem/Asura_Chunk_Carpet_Styles.h"
#include "Debugging/Asura_Assert.h"
#include "Rendering/Asura_Render.h"

Asura_Carpet_Style* Asura_Carpet_StyleManager::s_pxStyles = NULL;
int				    Asura_Carpet_StyleManager::s_iNumStyles = 0;
int				    Asura_Carpet_StyleManager::s_iNextUID = 0;

void Asura_Carpet_StyleManager::Initialise()
{
	// JAT 13/02/06 : ASURA_CARPET_BILLBOARD_SEQUENCE_LENGTH must be a power of 2
	ASURA_STATIC_ASSERT( ( ASURA_CARPET_BILLBOARD_SEQUENCE_LENGTH & ( ASURA_CARPET_BILLBOARD_SEQUENCE_LENGTH - 1 ) ) == 0 );

	Shutdown();	
	s_iNextUID=0;
}


void Asura_Carpet_StyleManager::Shutdown()
{
	if (s_pxStyles)
	{
		delete [] s_pxStyles;
		s_pxStyles = NULL;
	}

	s_iNumStyles=0;
}


void Asura_Carpet_StyleManager::Reset()
{	
	Shutdown();
}



int Asura_Carpet_StyleManager::GetNumStyles()
{
	return s_iNumStyles;
}


Asura_Carpet_Style *Asura_Carpet_StyleManager::GetStyle(int iStyle)
{
	if (iStyle<0 || iStyle>=s_iNumStyles)
	{
		return NULL;
	}
	else
	{
		return &s_pxStyles[iStyle];
	}
}


char *Asura_Carpet_StyleManager::GetStyleName(int iStyle)
{
	if (iStyle<0 || iStyle>=s_iNumStyles)
	{
		return "<Invalid>";
	}
	else
	{
		return s_pxStyles[iStyle].xName.GetString();
	}
}


int Asura_Carpet_StyleManager::CreateStyle()
{
	// Add a style
	// - give it a UID!	
	Asura_Carpet_Style *pxNewStyles=new Asura_Carpet_Style[s_iNumStyles+1];
	
	for(int i=0;i<s_iNumStyles;i++)
	{
		pxNewStyles[i]=s_pxStyles[i];
	}

	if (s_pxStyles)
	{
		delete [] s_pxStyles;
	}


	// Init the new one
	Asura_Carpet_Style *pxNewStyle=&pxNewStyles[s_iNumStyles];

	pxNewStyle->iTextureID=ASURA_TEXTURE_NONE;
	pxNewStyle->xTextureName=Asura_String("");
	pxNewStyle->xName=Asura_String("<Unnamed>");
	pxNewStyle->iUID=s_iNextUID++;
	pxNewStyle->fDensity=2.5f;
	pxNewStyle->fLodFadeNear=30.0f;
	pxNewStyle->fLodFadeFar=50.0f;
	pxNewStyle->fMovementScale=1.0f;

	for(int i=0;i<ASURA_CARPET_BILLBOARD_SEQUENCE_LENGTH;i++)
	{
		pxNewStyle->aiSequence[i]=0;
	}

	for(int i=0;i<ASURA_CARPET_BILLBOARDS_PER_STYLE;i++)
	{
		pxNewStyle->axBillboards[i].fHeight   = 1.0f;
		pxNewStyle->axBillboards[i].fWidth    = 1.0f;
		pxNewStyle->axBillboards[i].fMinScale = 0.5f;
		
		pxNewStyle->axBillboards[i].fU[0]     = 0.0f;
		pxNewStyle->axBillboards[i].fU[1]     = 1.0f;

		pxNewStyle->axBillboards[i].fV[0]     = 0.0f;
		pxNewStyle->axBillboards[i].fV[1]     = 1.0f;
	}

	
	// Swap the buffers over
	s_pxStyles=pxNewStyles;
	s_iNumStyles++;

	return (s_iNumStyles-1);
}



// Remove a style
void Asura_Carpet_StyleManager::RemoveStyle(int iStyle)
{
	Asura_Carpet_Style *pxNewStyles=0;

	if (s_iNumStyles>1)
	{
		pxNewStyles=new Asura_Carpet_Style[s_iNumStyles-1];
	
		for(int i=0;i<s_iNumStyles-1;i++)
		{	
			if (i<iStyle)
				pxNewStyles[i]=s_pxStyles[i];
			else
				pxNewStyles[i]=s_pxStyles[i+1];
		}
	}

	if (s_pxStyles)
	{
		delete [] s_pxStyles;
	}

	s_pxStyles=pxNewStyles;
	s_iNumStyles--;
}



#ifdef ASURA_TOOLS
bool Asura_Carpet_StyleManager::Save(char *szFileName)
{
	return Asura_Chunk_Carpet_Styles::Create(szFileName);	
}
#endif


void Asura_Carpet_StyleManager::LoadResources()
{
#ifdef ASURA_TOOLS
	bool bLoadedNewTexture = false;
	for (int i=0;i<s_iNumStyles;i++)
	{
		bool bWasNew = false;
		int iTextureIndex = Asura_Texture_Management::AddTexture(s_pxStyles[i].xTextureName.GetString(), &bWasNew);
		s_pxStyles[i].iTextureID = iTextureIndex;

		// Create material for this texture
		if (Asura_Material_System::GetMaterialIDForTexture(iTextureIndex, false) == ASURA_MATERIAL_NONE)
		{
			// Mark it as transparent too.
			int iMaterialID = Asura_Material_System::CreateMaterial();
			if(iMaterialID != ASURA_MATERIAL_NONE)
			{
				Asura_Material* pxMaterial = Asura_Material_System::GetMaterial(iMaterialID);
				Asura_Assert(pxMaterial,"");
				pxMaterial->m_iTextureID[0] = iTextureIndex;
				pxMaterial->m_uMaterialFlags = AsuraMaterialFlag_Transparent;
			}
		}
		if (bWasNew)
			bLoadedNewTexture = true;
	}
	if (bLoadedNewTexture)
		Asura_Texture_Management::LoadTextures();
#endif
}


int Asura_Carpet_StyleManager::FindStyleFromUID(int iUID)
{
	for (int i=0;i<s_iNumStyles;i++)
	{
		if (s_pxStyles[i].iUID==iUID)
		{
			return i;
		}
	}

	return -1;
}

#endif // #ifdef ASURA_TOOLS

#endif
