
///////////////////////////////////////////////////////////////////////////////
//
// File   : Asura_BackgroundChunkProcessing_System.h
// Author : Andy Tate
// Date   : 25-Jul-2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __ASURA_BACKGROUNDCHUNKPROCESSING_SYSTEM_H__
#define __ASURA_BACKGROUNDCHUNKPROCESSING_SYSTEM_H__

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

// Temporary...
#if !defined(ASURA_TOOLS) && !defined(ASURA_USE_BACKGROUND_CHUNK_PROCESSING)
//#define ASURA_USE_BACKGROUND_CHUNK_PROCESSING	1
#endif	// !defined(ASURA_TOOLS)


// ART - 26/07/06 - Some builds don't have multithread support yet, so I'll
// remove the whole class for now.

#if defined(ASURA_USE_BACKGROUND_CHUNK_PROCESSING)

#include "Core/Asura_Multithread.h"
#include "Core/Asura_Timers.h"


////////////////////////////////////////////////////////////////////////////////
// Forward class declarations...
////////////////////////////////////////////////////////////////////////////////

class Asura_Chunk_File_Stream;


////////////////////////////////////////////////////////////////////////////////
// Class declaration...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Class        : Asura_BackgroundChunkProcessing_System
// Author       : Andy Tate
// Date         : 25-Jul-2006
//
// Description  : Class to handle the processing of chunk-files in the background.
////////////////////////////////////////////////////////////////////////////////

class Asura_BackgroundChunkProcessing_System
{
public:
	// Public functions...
	static	void	Initialise			(void);
	static	void	Shutdown			(void);

	static	bool	BeginProcessing		(Asura_Chunk_File_Stream& xStream);
	static	bool	WaitForProcessing	(const float fTimeAllowed);

	static	void	YieldProcessing		(void);


protected:
	// Protected variables...


	// Protected functions...


private:
	// Private variables...
	static 	bool						s_bInitialised;

	static 	Asura_SyncEvent				s_xReadyEvent;
	static 	Asura_SyncEvent				s_xCompleteEvent;

	static 	Asura_TickCount				s_xStartTicks;
	static 	Asura_TickCount				s_xTicksAllowed;

	static 	Asura_Chunk_File_Stream*	s_pxFileStream;


	// Private functions...

	Asura_BackgroundChunkProcessing_System  (void);
	Asura_BackgroundChunkProcessing_System  (const Asura_BackgroundChunkProcessing_System& xOther);
	~Asura_BackgroundChunkProcessing_System ();

	Asura_BackgroundChunkProcessing_System&	operator=	(const Asura_BackgroundChunkProcessing_System& xOther);

	// Internal background chunk-processing functions...
	static	void	UpdateThread		(const void* pUserData);

};   // End of class 'Asura_BackgroundChunkProcessing_System'.


#endif	// defined(ASURA_USE_BACKGROUND_CHUNK_PROCESSING)

#endif   // ifndef __ASURA_BACKGROUNDCHUNKPROCESSING_SYSTEM_H__


/// End of file. ///
