/**
*
* \file Asura_Chunk_BillboardPlant_Template.cpp
*
* Created by Andrew Weinkove - 06/12/2002
*
* Copyright (C) 2002 Rebellion, All Rights Reserved.
*
**/

#include "Core/Asura.h"
#include "ChunkSystem/Asura_Chunk_BillboardPlant_Template.h"

#include "ChunkSystem/Asura_Chunk_Loaders.h"
#include "ChunkSystem/Asura_Chunk_Macros.h"
#include "ChunkSystem/Asura_SaveBlock.h"
#include "Core/Asura_Handle.h"
#include "Core/Asura_String.h"
#include "Core/Asura_TextureFlags.h"
#include "Debugging/Asura_Assert.h"
#include "FileAccess/Asura_FileAccess.h"
#include "SpecialFX/Asura_BillboardPlant_TemplateStore.h"

#ifdef ASURA_USE_BILLBOARD_PLANTS
#ifdef ASURA_TOOLS
void Asura_Chunk_BillboardPlant_Template::Create()
{
	int iNumberOfTemplates = Asura_BillboardPlant_TemplateStore::GetNumberOfTemplates();

	//Calculate size of chunk: Header + templates
	int iChunkSize = sizeof(Asura_Chunk_BillboardPlant_Template);
	iChunkSize += sizeof(int);//iNumberOfTemplates
	for (int i=0;i<iNumberOfTemplates;i++)
	{
		Asura_BillboardPlant_Template* pxTemplate = Asura_BillboardPlant_TemplateStore::GetTemplate(i);
		//char *m_szTemplateName; 
		Asura_String xName(pxTemplate->m_szTemplateName);
		if (xName.IsNull())
			xName.MakeBlank();
		iChunkSize += xName.GetSizeInInts() * 4;
		///char *m_szTextureName;
		Asura_String xTextureName(pxTemplate->m_szTextureName);
		if (xTextureName.IsNull())
			xTextureName.MakeBlank();
		iChunkSize += xTextureName.GetSizeInInts() * 4;
		iChunkSize += sizeof(int);//int m_iTextureIndex;
		//float m_fTopLeftU;
		//float m_fTopLeftV;
		//float m_fBottomRightU;
		//float m_fBottomRightV;
		//float m_fHeight;
		//float m_fWidth;
		//float m_fMaxDistortion;
		//float m_fStiffness;
		iChunkSize += sizeof(float) * 8;
		iChunkSize += sizeof(unsigned);//unsigned m_uUniqueID;
		iChunkSize += sizeof(unsigned);//m_uFlags.
	}
	//Create saveblock and chunk
	Asura_SaveBlock* pxSaveBlock = new Asura_SaveBlock();
	Asura_Assert(pxSaveBlock, "Unable to create saveblock");
	if(pxSaveBlock)
	{
		Asura_Chunk_BillboardPlant_Template* pxChunk = (Asura_Chunk_BillboardPlant_Template*)pxSaveBlock->GetAccessPointer(iChunkSize);

		//Fill chunk header
		pxChunk->ID = ASURA_CHUNK_BILLBOARDPLANTS_TEMPLATES;
		pxChunk->Size = iChunkSize;
		pxChunk->Version = 2;
		pxChunk->Flags = 0;

		char *pcChunkBuffer = (char*)(pxChunk+1);
		// Fill template data.
		ASURA_CHUNK_FILL(pcChunkBuffer, iNumberOfTemplates, int);
		for (int i=0;i<iNumberOfTemplates;i++)
		{
			Asura_BillboardPlant_Template* pxTemplate = Asura_BillboardPlant_TemplateStore::GetTemplate(i);
			Asura_String xName(pxTemplate->m_szTemplateName);
			if (xName.IsNull())
				xName.MakeBlank();
			ASURA_CHUNK_FILL_STRING(pcChunkBuffer, xName);
			Asura_String xTextureName(pxTemplate->m_szTextureName);
			if (xTextureName.IsNull())
				xTextureName.MakeBlank();
			ASURA_CHUNK_FILL_STRING(pcChunkBuffer, xTextureName);
			ASURA_CHUNK_FILL(pcChunkBuffer, pxTemplate->m_iTextureIndex, int);
			ASURA_CHUNK_FILL(pcChunkBuffer, pxTemplate->m_fTopLeftU, float);
			ASURA_CHUNK_FILL(pcChunkBuffer, pxTemplate->m_fTopLeftV, float);
			ASURA_CHUNK_FILL(pcChunkBuffer, pxTemplate->m_fBottomRightU, float);
			ASURA_CHUNK_FILL(pcChunkBuffer, pxTemplate->m_fBottomRightV, float);
			ASURA_CHUNK_FILL(pcChunkBuffer, pxTemplate->m_fHeight, float);
			ASURA_CHUNK_FILL(pcChunkBuffer, pxTemplate->m_fWidth, float);
			ASURA_CHUNK_FILL(pcChunkBuffer, pxTemplate->m_fMaxDistortion, float);
			ASURA_CHUNK_FILL(pcChunkBuffer, pxTemplate->m_fStiffness, float);
			ASURA_CHUNK_FILL(pcChunkBuffer, pxTemplate->m_uUniqueID, unsigned);
			ASURA_CHUNK_FILL(pcChunkBuffer, pxTemplate->m_uFlags, unsigned);
		}
		//Terminate and save chunk
		pxSaveBlock->AddAsuraTerminator();
		pxSaveBlock->SaveFile( "Misc\\PlntTmps.asr", false );
		delete pxSaveBlock;
	}
}
#endif // ASURA_TOOLS

bool Asura_Chunk_BillboardPlant_Template::Process(Asura_Handle fileHandle) const
{
	//Call shutdown first in case any templates are already loaded.
	Asura_BillboardPlant_TemplateStore::Shutdown();

	//Create storeage space for templates
	int iNumberOfTemplates;
	ASURA_CHUNK_FILEEXTRACT(fileHandle, iNumberOfTemplates, int);
	Asura_BillboardPlant_TemplateStore::AllocateTemplates(iNumberOfTemplates);
	//Load data into template store
	for (int i=0;i<iNumberOfTemplates;i++)
	{
		Asura_BillboardPlant_Template* pxTemplate = Asura_BillboardPlant_TemplateStore::GetTemplate(i);
		Asura_String xName;
		ASURA_CHUNK_FILEEXTRACT_STRING(fileHandle, xName);
#ifdef ASURA_TOOLS
		if (xName.IsBlank())
			pxTemplate->m_szTemplateName = NULL;
		else
		{
			pxTemplate->m_szTemplateName = new char[xName.GetLength() + 1];
			strcpy(pxTemplate->m_szTemplateName, xName.GetString());
		}
#else
		pxTemplate->m_szTemplateName = NULL;
#endif
		Asura_String xTextureName;
		ASURA_CHUNK_FILEEXTRACT_STRING(fileHandle, xTextureName);
		if (xTextureName.IsBlank())
			pxTemplate->m_szTextureName = NULL;
		else
		{
			pxTemplate->m_szTextureName = new char[xTextureName.GetLength() + 1];
			strcpy(pxTemplate->m_szTextureName, xTextureName.GetString());
		}
		ASURA_CHUNK_FILEEXTRACT(fileHandle, pxTemplate->m_iTextureIndex, int);
		ASURA_CHUNK_FILEEXTRACT(fileHandle, pxTemplate->m_fTopLeftU, float);
		ASURA_CHUNK_FILEEXTRACT(fileHandle, pxTemplate->m_fTopLeftV, float);
		ASURA_CHUNK_FILEEXTRACT(fileHandle, pxTemplate->m_fBottomRightU, float);
		ASURA_CHUNK_FILEEXTRACT(fileHandle, pxTemplate->m_fBottomRightV, float);
		ASURA_CHUNK_FILEEXTRACT(fileHandle, pxTemplate->m_fHeight, float);
		ASURA_CHUNK_FILEEXTRACT(fileHandle, pxTemplate->m_fWidth, float);
		ASURA_CHUNK_FILEEXTRACT(fileHandle, pxTemplate->m_fMaxDistortion, float);
		ASURA_CHUNK_FILEEXTRACT(fileHandle, pxTemplate->m_fStiffness, float);
		ASURA_CHUNK_FILEEXTRACT(fileHandle, pxTemplate->m_uUniqueID, unsigned);
		if (Version > 1)
		{
			ASURA_CHUNK_FILEEXTRACT(fileHandle, pxTemplate->m_uFlags, unsigned);
		}
		else
		{
			pxTemplate->m_uFlags = 0;
		}
		pxTemplate->m_iTextureIndex = ASURA_TEXTURE_NONE;
#ifdef ASURA_TOOLS
		Asura_BillboardPlant_TemplateStore::RegisterUniqueID(pxTemplate->m_uUniqueID);
#endif
	}
#ifdef ASURA_TOOLS
	Asura_BillboardPlant_TemplateStore::LoadTextures();
#endif
	return true;
}

#endif //ASURA_USE_BILLBOARD_PLANTS
