//////////////////////////////////////////////////////////////////////////////
//
// Asura_Chunk_Blueprints.cpp
//
// Created by NJD - 06 September 2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"

// header
#include "ChunkSystem/Asura_Chunk_Blueprints.h"

// asura
#include "Blueprint/Asura_Blueprint_System.h"
#include "ChunkSystem/Asura_Chunk_Stream.h"
#ifdef ASURA_TOOLS
#include "ChunkSystem/Asura_SaveBlock.h"
#endif
#include "Debugging/Asura_Assert.h"
#include "FileAccess/Asura_FileAccess.h"

//////////////////////////////////////////////////////////////////////////////
//
// Asura_Chunk_Blueprints
//
//////////////////////////////////////////////////////////////////////////////

Asura_Handle Asura_Chunk_Blueprints::s_iCurrentFileHandle = ASURA_INVALID_HANDLE;

bool Asura_Chunk_Blueprints::Process( Asura_Handle hFile ) const
{
    Asura_Chunk_File_Stream	xStream( hFile );
    return Process( xStream );
}

bool Asura_Chunk_Blueprints::Process( Asura_Chunk_File_Stream& xStream ) const
{
    if ( Version == CURRENT_CHUNK_VERSION )
    {
		s_iCurrentFileHandle = xStream.GetFileHandle();
        Asura_Blueprint_System::ReadFromChunkStream( xStream );
		s_iCurrentFileHandle = ASURA_INVALID_HANDLE;
        return true;
    }
    else
    {
        Asura_Assertf( false, ( "Asura_Chunk_Blueprints::Process - unhandled chunk version (%d)", Version ) );
        return false;
    }
}

void Asura_Chunk_Blueprints::GetCurrentBlueprintFileName( Asura_String& xFileName )
{
	if ( s_iCurrentFileHandle == ASURA_INVALID_HANDLE )
	{
		xFileName.MakeBlank();
	}
	else
	{
		const Asura_String xRawFileName = Asura_FileAccess::GetFilename( s_iCurrentFileHandle );
		xFileName = Asura_FileAccess::PlatformRemoveNetworkOrExecutableDir( xRawFileName.GetString() );
	}
}

#ifdef ASURA_TOOLS
bool Asura_Chunk_Blueprints::SaveBlueprints( const char* szFilename, const Asura_Hash_ID uBlueprintTypeHashID /*=ASURA_HASH_ID_UNSET*/ )
{
    Asura_SaveBlock xSaveBlock;
    if ( !Asura_Chunk_Blueprints::Create( xSaveBlock, uBlueprintTypeHashID ) ) { return false; }
    if ( !xSaveBlock.AddAsuraTerminator() ) { return false; }
    
	return xSaveBlock.SaveFile( szFilename, false /*true*/ );
}

bool Asura_Chunk_Blueprints::Create( Asura_SaveBlock& xSaveBlock, const Asura_Hash_ID uBlueprintTypeHashID )
{
    Asura_Chunk_SaveBlock_Stream xStream( &xSaveBlock );
    return Create( xStream, uBlueprintTypeHashID );
}

bool Asura_Chunk_Blueprints::Create( Asura_Chunk_Stream& xStream, const Asura_Hash_ID uBlueprintTypeHashID )
{
    xStream.WriteChunk_Start( ASURA_CHUNK_BLUEPRINTS, CURRENT_CHUNK_VERSION, 0 );
    Asura_Blueprint_System::WriteToChunkStream( xStream, uBlueprintTypeHashID );
    xStream.WriteChunk_End();
    return true;
}
#endif

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////
