//*******************************************************************
// File: Asura_Chunk_Botanical_Game_Render.cpp
//
// Created: 18 December 2006 DLH
//
// Description: 
//
//*******************************************************************

// system libraries
#include "Core/Asura.h"

#if defined( ASURA_USE_BOTANICALS)

// main header
#include "Botanicals/Asura_Botanical_Game_Render_Dynamic_Memory_Layout.h"
#include "Botanicals/Asura_Botanical_Game_Render_GameState.h"
#include "Botanicals/Asura_Botanical_Game_Render_Instance_Header.h"
#include "Botanicals/Asura_Botanical_Game_Render_Manager.h"
#include "Botanicals/Asura_Botanical_Game_Render_Type.h"
#include "Botanicals/Asura_Botanical_Game_Render_Type_BasicInteractive.h"

// other libraries
#include "Debugging/Asura_Debug_Timers.h"
#include "ChunkSystem/Asura_Chunk_Stream.h"

// internal dependencies

//*******************************************************************
// Constants
//*******************************************************************

const u_int Asura_BGR_Manager::s_uChunkVersion =
	// DLH - Version
	// 1 - Original version
	// 2 - Project-specific flags added to "Asura_BGR_StateTable_Version" (ART - 26/01/07)
	// 3 - DMR Added collision meshes
	3

	// DLH - Feature flags
#if defined( ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA) && 1 == ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA
	| 0x80000000
#endif
#if defined( ASURA_BOTANICAL_SUPPORT_INTERACTIVE)
	| 0x40000000
#endif
#if defined( ASURA_BOTANICAL_SUPPORT_STATES)
	| 0x20000000
	| (0x04000000 * (sizeof( Asura_BGR_State_Value) - 1))
#endif
#if defined( ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS)
	| 0x10000000
#endif
;

static const u_int g_uInGameChunkVersion = 0;


//*******************************************************************
// Local Classes
//*******************************************************************


//*******************************************************************
// Local Data
//*******************************************************************


//*******************************************************************
// Function Definitions
//*******************************************************************

template< class Tdata>
bool Asura_BGR_Type_Process( Asura_Chunk_Stream &xStream, Asura_TemplateHashTree<Tdata>& xHashTree)
{
	uint32 uCount;
	xStream >> uCount;
	
	for( u_int uIndex = 0; uIndex < uCount; uIndex += 1)
	{
		uint32 uHashID;
		Tdata tData;
		
		xStream >> uHashID;
		xStream >> tData;
		
		Asura_Hash_ID xHashID( uHashID);
		xHashTree.AddNode( xHashID, tData);
	}
	return( true);
}

#if defined( ASURA_TOOLS) 
template< class Tdata>
bool Asura_BGR_Type_Export( Asura_Chunk_Stream &xStream, const Asura_TemplateHashTree< Tdata>& xHashTree)
{
	const u_int uCount = xHashTree.GetNumberOfNodes();
	xStream << static_cast< uint32>( uCount);
	for( u_int uIndex = 0; uIndex < uCount; uIndex += 1)
	{
		const Asura_Hash_ID xHashId = xHashTree.GetHashByIndex( uIndex);
		const Tdata* ptData = xHashTree.GetDataByIndex( uIndex);
		
		xStream << static_cast< uint32>( xHashId);
		xStream << (*ptData);
	}
	return( true);
}
#endif

#if defined( ASURA_BOTANICAL_SUPPORT_STATES) && 1 == ASURA_BOTANICAL_SUPPORT_STATES
bool Asura_BGR_StateTable_Version::Process( Asura_Chunk_Stream &xStream)
{
	Asura_BGR_State_Value xLoadedData;
	uint16 usLoadedData;
	
	xStream >> m_uProjectFlags;
	xStream >> xLoadedData;
	this->m_xRangeStart = xLoadedData;
	xStream >> xLoadedData; 
	this->m_xRangeStop = xLoadedData;
	xStream >> usLoadedData;
	this->m_xVersionID = usLoadedData;

	xStream >> m_xCollisionMeshHash;			// Collision mesh shape hash
	xStream >> m_fCollisionMeshDensity;	  		// Collision mesh density value (0-1) - for AI LoS	
	xStream >> m_bCollisionUseBoundingBox;
	
	return( true);
}
#endif

#if defined( ASURA_TOOLS) && defined( ASURA_BOTANICAL_SUPPORT_STATES) && 1 == ASURA_BOTANICAL_SUPPORT_STATES
bool Asura_BGR_StateTable_Version::Export( Asura_Chunk_Stream &xStream) const
{
	xStream << m_uProjectFlags;
	xStream << static_cast< Asura_BGR_State_Value>( this->m_xRangeStart);
	xStream << static_cast< Asura_BGR_State_Value>( this->m_xRangeStop);
	xStream << static_cast< uint16>( this->m_xVersionID);

	xStream << m_xCollisionMeshHash;			// Collision mesh shape hash
	xStream << m_fCollisionMeshDensity;	  		// Collision mesh density value (0-1) - for AI LoS
	xStream << m_bCollisionUseBoundingBox;

	return( true);
}
#endif

#if defined( ASURA_BOTANICAL_SUPPORT_STATES) && 1 == ASURA_BOTANICAL_SUPPORT_STATES
bool Asura_BGR_StateTable_Section::Process( Asura_Chunk_Stream &xStream)
{
	uint32 uLoadedData;
	
	xStream >> uLoadedData;
	this->m_xSectionID = uLoadedData;
	
	this->m_xArray_xVersions.Process( xStream);
	return( true);
}
#endif

#if defined( ASURA_TOOLS) && defined( ASURA_BOTANICAL_SUPPORT_STATES) && 1 == ASURA_BOTANICAL_SUPPORT_STATES
bool Asura_BGR_StateTable_Section::Export( Asura_Chunk_Stream &xStream) const
{
	xStream << static_cast< uint32>( this->m_xSectionID);
	
	this->m_xArray_xVersions.Export( xStream);
	return( true);
}
#endif

#if defined( ASURA_BOTANICAL_SUPPORT_STATES) && 1 == ASURA_BOTANICAL_SUPPORT_STATES
bool Asura_BGR_StateTable::Process( Asura_Chunk_Stream &xStream)
{
	this->m_xArray_xSections.Process( xStream);
	return( true);
}
#endif

#if defined( ASURA_TOOLS) && defined( ASURA_BOTANICAL_SUPPORT_STATES) && 1 == ASURA_BOTANICAL_SUPPORT_STATES
bool Asura_BGR_StateTable::Export( Asura_Chunk_Stream &xStream) const
{
	this->m_xArray_xSections.Export( xStream);
	return( true);
}
#endif

#if defined( ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA) && 1 == ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA
bool Asura_BGR_DynamicMemoryLayout::Process( Asura_Chunk_Stream &xStream)
{
	uint32 uDataLoaded;
	
	xStream >> uDataLoaded;
	this->m_uDynamicSizeInstance = uDataLoaded;
	xStream >> uDataLoaded;
	this->m_uDynamicOffsetInteractiveData = uDataLoaded;
	xStream >> uDataLoaded;
	this->m_uDynamicOffsetSectionData = uDataLoaded;
	xStream >> uDataLoaded;
	this->m_uDynamicOffsetStateData = uDataLoaded;
	xStream >> uDataLoaded;
	this->m_uDynamicOffsetUserData = uDataLoaded;
	
	return( true);
}
#endif

#if defined( ASURA_TOOLS) && defined( ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA) && 1 == ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA
bool Asura_BGR_DynamicMemoryLayout::Export( Asura_Chunk_Stream &xStream) const
{
	xStream << static_cast< uint32>( this->m_uDynamicSizeInstance);
	xStream << static_cast< uint32>( this->m_uDynamicOffsetInteractiveData);
	xStream << static_cast< uint32>( this->m_uDynamicOffsetSectionData);
	xStream << static_cast< uint32>( this->m_uDynamicOffsetStateData);
	xStream << static_cast< uint32>( this->m_uDynamicOffsetUserData);
	return( true);
}
#endif

#if defined( ASURA_BOTANICAL_SUPPORT_INTERACTIVE)
bool Asura_BGR_Type_BasicInteractive::Process( Asura_Chunk_Stream& xStream)
{
	xStream >> this->m_xBonePosition0;
	xStream >> this->m_xBonePosition1;
	xStream >> this->m_fMass;
	xStream >> this->m_fDrag;
	xStream >> this->m_fSpringStrength;
	xStream >> this->m_fSpringLength;
	xStream >> this->m_fBoneWeightGamma;
	xStream >> this->m_fBoneLength;
	
	return( true);
}
#endif

#if defined( ASURA_TOOLS) && defined( ASURA_BOTANICAL_SUPPORT_INTERACTIVE)
bool Asura_BGR_Type_BasicInteractive::Export( Asura_Chunk_Stream& xStream)
{
	xStream << static_cast< Asura_Vector_3>( this->m_xBonePosition0);
	xStream << static_cast< Asura_Vector_3>( this->m_xBonePosition1);
	xStream << static_cast< float>( this->m_fMass);
	xStream << static_cast< float>( this->m_fDrag);
	xStream << static_cast< float>( this->m_fSpringStrength);
	xStream << static_cast< float>( this->m_fSpringLength);
	xStream << static_cast< float>( this->m_fBoneWeightGamma);
	xStream << static_cast< float>( this->m_fBoneLength);

	return( true);
}
#endif

bool Asura_BGR_Type::Process( Asura_Chunk_Stream& xStream)
{
	xStream >> this->m_xBoundingBox;
	xStream >> this->m_fBoundingRadius;
	
#if defined( ASURA_BOTANICAL_SUPPORT_INTERACTIVE)
	{
		bool bLoadBasicInteractive;
		
		xStream >> bLoadBasicInteractive;
		if( true == bLoadBasicInteractive)
		{
			this->CreateBasicInteractive();
			this->m_pxBasicInteractive->Process( xStream);
		}
	}
#endif
	
#if defined( ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA) && 1 == ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA
	this->m_xDynamicDataLayout.Process( xStream);
#endif

#if defined( ASURA_BOTANICAL_SUPPORT_STATES) && 1 == ASURA_BOTANICAL_SUPPORT_STATES
	Asura_BGR_Type_Process( xStream, this->m_xHashTree_xStateID);
	this->m_xArray_xStateTables.Process( xStream);
#endif
	return( true);
}

#if defined( ASURA_TOOLS)
bool Asura_BGR_Type::Export( Asura_Chunk_Stream& xStream) const 
{
	Asura_Assert( this->m_xBoundingBox.GetMin().GetX() <= this->m_xBoundingBox.GetMax().GetX(), "Bounding Box not Setup");
	Asura_Assert( this->m_xBoundingBox.GetMin().GetY() <= this->m_xBoundingBox.GetMax().GetY(), "Bounding Box not Setup");
	Asura_Assert( this->m_xBoundingBox.GetMin().GetZ() <= this->m_xBoundingBox.GetMax().GetZ(), "Bounding Box not Setup");
	Asura_Assert( 0 <= this->m_fBoundingRadius, "Bounding Radius not Setup");
	
	xStream << this->m_xBoundingBox;
	xStream << this->m_fBoundingRadius;

#if defined( ASURA_BOTANICAL_SUPPORT_INTERACTIVE)
	const bool bSaveBasicInteractive = (NULL != this->m_pxBasicInteractive);
	xStream << static_cast< bool>( bSaveBasicInteractive);
	if( true == bSaveBasicInteractive)
	{
		this->m_pxBasicInteractive->Export( xStream);
	}
#endif

//	Asura_BGR_TypeID m_xTypeID;

#if defined( ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA) && 1 == ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA
	this->m_xDynamicDataLayout.Export( xStream);
#endif

#if defined( ASURA_BOTANICAL_SUPPORT_STATES) && 1 == ASURA_BOTANICAL_SUPPORT_STATES
	Asura_BGR_Type_Export( xStream, m_xHashTree_xStateID);
	this->m_xArray_xStateTables.Export( xStream);
#endif

	return( true);
}
#endif

bool Asura_BGR_Chunk_GameState::Process(Asura_Chunk_Stream& xStream) const
{
#if defined( ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA) && 1 == ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA
	if( g_uInGameChunkVersion != static_cast<u_int>( this->Version ) )
	{
		return( false);
	}
	if( Asura_BGR_Manager::s_uChunkVersion != static_cast<u_int>( this->Flags ) )
	{
		return( false);
	}
	
	ASURA_CPU_PROFILE_SCOPED_TIMER( "Asura_BGR_Chunk_GameState::Process" );

	Asura_BGR_Manager& xBGRManager = Asura_BGR_Manager::GetBGRManager();
	
	{
		// DLH - 16 April 2007 12:24:04 - This Check Sum is a CRC to check that the instance data has not been moved or edited since the data block was saved.
		const u_int uBGRStaticCheckSum = xBGRManager.GetStaticCheckSum();
		u_int uStreamStaticCheckSum;
		
		xStream >> uStreamStaticCheckSum;
		
		if( uBGRStaticCheckSum != uStreamStaticCheckSum)
		{
			return( false);
		}
	}
	
	{
		u_int uStreamInstanceHeaderCount = 0;
		xStream >> uStreamInstanceHeaderCount;
		
		const u_int uBGRInstanceHeaderCount = xBGRManager.GetInstanceHeaderCount();
		if( uBGRInstanceHeaderCount != uStreamInstanceHeaderCount)
		{
			return( false);
		}
		
		for( u_int uBGRInstanceHeaderIndex = 0; uBGRInstanceHeaderIndex < uBGRInstanceHeaderCount; uBGRInstanceHeaderIndex += 1)
		{
			Asura_BGR_Instances_Header* pxBGRInstanceHeader = xBGRManager.GetInstanceHeader( uBGRInstanceHeaderIndex);
			if( NULL == pxBGRInstanceHeader)
			{
				continue;
			}
			
			const Asura_BGR_TypeID xBGRTypeID = pxBGRInstanceHeader->GetType();
			
			const Asura_BGR_Type* pxBGRType = xBGRManager.GetType( xBGRTypeID);
			if( NULL == pxBGRType)
			{
				continue;
			}
			
#if defined( ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS) && 1 == ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS
			u_int uStreamSectionCount;
			xStream >> uStreamSectionCount;
			const u_int uBGRSectionCount = pxBGRType->GetSectionCount();
			if( uBGRSectionCount != uStreamSectionCount)
			{
				return( false);
			}
#endif
			
#if defined( ASURA_BOTANICAL_SUPPORT_STATES) && 1 == ASURA_BOTANICAL_SUPPORT_STATES
			u_int uStreamStateCount;
			xStream >> uStreamStateCount;
			const u_int uBGRStateCount = pxBGRType->GetStateCount();
			if( uBGRStateCount != uStreamStateCount)
			{
				return( false);
			}
#endif
			
			const u_int uBGRDynamicInstanceCount = pxBGRInstanceHeader->GetDynamicInstanceCount();
			for( u_int uBGRDynamicInstanceIndex = 0; uBGRDynamicInstanceIndex < uBGRDynamicInstanceCount; uBGRDynamicInstanceIndex += 1)
			{
				Asura_BGR_DynamicInstance* pxBGRDynamicInstance = pxBGRInstanceHeader->GetDynamicInstanceData( uBGRDynamicInstanceIndex);
				if( NULL == pxBGRDynamicInstance)
				{
					continue;
				}
				
#if defined( ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS) && 1 == ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS
				if( 0 != uBGRSectionCount)
				{				
					Asura_BGR_Version_ID* paxSections = pxBGRInstanceHeader->GetDynamicSectionVersions( pxBGRDynamicInstance);
					for( u_int uBGRSectionIndex = 0; uBGRSectionIndex < uBGRSectionCount; uBGRSectionIndex += 1)
					{
						xStream >> (paxSections[ uBGRSectionIndex]);
					}
				}
#endif
				
#if defined( ASURA_BOTANICAL_SUPPORT_STATES) && 1 == ASURA_BOTANICAL_SUPPORT_STATES
				if( 0 != uBGRStateCount)
				{
					Asura_BGR_State_Value* paxStates = pxBGRInstanceHeader->GetStates( pxBGRDynamicInstance);
					for( u_int uBGRStateIndex = 0; uBGRStateIndex < uBGRStateCount; uBGRStateIndex += 1)
					{
						xStream >> (paxStates[ uBGRStateIndex]);
					}
				}
#endif
			}
		}
	}
#endif

	return true;
}

void Asura_BGR_Chunk_GameState::Create(Asura_SaveBlock_Base& xSaveBlock)
{
#if defined( ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA) && 1 == ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA
	ASURA_CPU_PROFILE_SCOPED_TIMER( "Asura_BGR_Chunk_GameState::Create" );

	Asura_Chunk_SaveBlock_Stream	xStream(&xSaveBlock);
	Asura_BGR_Manager& xBGRManager = Asura_BGR_Manager::GetBGRManager();

	xStream.WriteChunk_Start( ASURA_CHUNK_BOTANICALS_INGAME, g_uInGameChunkVersion, Asura_BGR_Manager::s_uChunkVersion);
	
	{
		// DLH - 16 April 2007 12:24:04 - This Check Sum is a CRC to check that the instance data has not been moved or edited since the data block was saved.
		const u_int uBGRStaticCheckSum = xBGRManager.GetStaticCheckSum();
		
		xStream << uBGRStaticCheckSum;
	}
	
	{
		const u_int uBGRInstanceHeaderCount = xBGRManager.GetInstanceHeaderCount();
		
		xStream << uBGRInstanceHeaderCount;
		
		for( u_int uBGRInstanceHeaderIndex = 0; uBGRInstanceHeaderIndex < uBGRInstanceHeaderCount; uBGRInstanceHeaderIndex += 1)
		{
			const Asura_BGR_Instances_Header* pxBGRInstanceHeader = xBGRManager.GetInstanceHeader( uBGRInstanceHeaderIndex);
			if( NULL == pxBGRInstanceHeader)
			{
				continue;
			}
			
			const Asura_BGR_TypeID xBGRTypeID = pxBGRInstanceHeader->GetType();
			
			const Asura_BGR_Type* pxBGRType = xBGRManager.GetType( xBGRTypeID);
			if( NULL == pxBGRType)
			{
				continue;
			}
			
#if defined( ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS) && 1 == ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS
			const u_int uBGRSectionCount = pxBGRType->GetSectionCount();
			xStream << uBGRSectionCount;
#endif
			
#if defined( ASURA_BOTANICAL_SUPPORT_STATES) && 1 == ASURA_BOTANICAL_SUPPORT_STATES
			const u_int uBGRStateCount = pxBGRType->GetStateCount();
			xStream << uBGRStateCount;
#endif
			
			const u_int uBGRDynamicInstanceCount = pxBGRInstanceHeader->GetDynamicInstanceCount();
			for( u_int uBGRDynamicInstanceIndex = 0; uBGRDynamicInstanceIndex < uBGRDynamicInstanceCount; uBGRDynamicInstanceIndex += 1)
			{
				const Asura_BGR_DynamicInstance* pxBGRDynamicInstance = pxBGRInstanceHeader->GetDynamicInstanceData( uBGRDynamicInstanceIndex);
				if( NULL == pxBGRDynamicInstance)
				{
					continue;
				}
				
#if defined( ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS) && 1 == ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS
				if( 0 != uBGRSectionCount)
				{
					const Asura_BGR_Version_ID* paxSections = pxBGRInstanceHeader->GetDynamicSectionVersions( pxBGRDynamicInstance);
					for( u_int uBGRSectionIndex = 0; uBGRSectionIndex < uBGRSectionCount; uBGRSectionIndex += 1)
					{
						xStream << (paxSections[ uBGRSectionIndex]);
					}
				}
#endif
				
#if defined( ASURA_BOTANICAL_SUPPORT_STATES) && 1 == ASURA_BOTANICAL_SUPPORT_STATES
				if( 0 != uBGRStateCount)
				{
					const Asura_BGR_State_Value* paxStates = pxBGRInstanceHeader->GetStates( pxBGRDynamicInstance);
					for( u_int uBGRStateIndex = 0; uBGRStateIndex < uBGRStateCount; uBGRStateIndex += 1)
					{
						xStream << (paxStates[ uBGRStateIndex]);
					}
				}
#endif
			}
		}
	}

	xStream.WriteChunk_End	();
#endif
}


//*******************************************************************
//
// (c) 2006 Rebellion
//
// Warning: This computer program is protected by copyright law and 
// international treaties. Unauthorized reproduction or distribution
// of this program, or any portion of it, may result in severe civil and
// criminal penalties, and will be prosecuted to the maximum extent 
// possible under the law.
//
//*******************************************************************

#endif //defined( ASURA_USE_BOTANICALS)
