///////////////////////////////////////////////////////////////////////////////
//
// File: Asura_Chunk_Container.h
//
// Created by Martin G.Harris (13/5/2008)
// 
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef __ASURA_CHUNK_CONTAINER_H__
#define __ASURA_CHUNK_CONTAINER_H__


#ifdef ASURA_USE_CONTAINER_SYSTEM

// Includes
#include "ChunkSystem/Asura_Chunk_Loaders.h"
#include "Core/Asura_Hash.h"


// Forward declarations
class Asura_Chunk_Stream;
class Asura_SaveBlock;
class Asura_ProtoSaveBlock;
class Asura_Container;
class Tools_Container;
class Tools_ContainerFile;
class Tools_ContainerGroup;


///////////////////////////
// Asura_Chunk_Container //
///////////////////////////
// Chunk for Asura_Container classes. Saved in exports.
class Asura_Chunk_Container : public Asura_Chunk_Header
{
public :
	bool Process(Asura_Chunk_Stream& xStream) const;
	bool Process(Asura_Handle iFileHandle) const;

	static void Create(const Asura_Container* pxContainer, Asura_Chunk_Stream& xStream);
	static void Create(const Asura_Container* pxContainer, Asura_SaveBlock* pxSaveBlock);
};


////////////////////////////////////
// Asura_Chunk_ToolsContainerFile //
////////////////////////////////////
// Chunk for Tools_ContainerFile classes.
#ifdef ASURA_TOOLS
class Asura_Chunk_ToolsContainerFile : public Asura_Chunk_Header
{
public :
	bool Process(Asura_Chunk_Stream& xStream) const;
	bool Process(Asura_Handle iFileHandle) const;

	static void Create(const Tools_ContainerFile* pxFile, Asura_Chunk_Stream& xStream);
	static void Create(const Tools_ContainerFile* pxFile, Asura_SaveBlock* pxSaveBlock);
};
#endif //#def ASURA_TOOLS


/////////////////////////////////////
// Asura_Chunk_ToolsContainerGroup //
/////////////////////////////////////
// Chunk for Tools_ContainerGroup classes.
#ifdef ASURA_TOOLS
class Asura_Chunk_ToolsContainerGroup : public Asura_Chunk_Header
{
public :
	bool Process(Asura_Chunk_Stream& xStream) const;
	bool Process(Asura_Handle iFileHandle) const;

	static void Create(const Tools_ContainerGroup* pxGroup, Asura_Chunk_Stream& xStream);
	static void Create(const Tools_ContainerGroup* pxGroup, Asura_SaveBlock* pxSaveBlock);
};
#endif //#def ASURA_TOOLS


////////////////////////////////
// Asura_Chunk_ToolsContainer //
////////////////////////////////
// Chunk for Tools_Container classes.
#ifdef ASURA_TOOLS
class Asura_Chunk_ToolsContainer : public Asura_Chunk_Header
{
public :
	bool Process(Asura_Chunk_Stream& xStream) const;
	bool Process(Asura_Handle iFileHandle) const;

	static void Create(const Tools_Container* pxContainer, Asura_Chunk_Stream& xStream);
	static void Create(const Tools_Container* pxContainer, Asura_SaveBlock* pxSaveBlock);
};
#endif //#def ASURA_TOOLS


#endif //#ifdef ASURA_USE_CONTAINER_SYSTEM

#endif //__ASURA_CHUNK_CONTAINER_H__

