#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef _ASURA_CHUNK_CUTSCENE_H_
#define _ASURA_CHUNK_CUTSCENE_H_

#include "ChunkSystem/Asura_Chunk_Loaders.h"
#include "Core/Asura_Maths.h"

class Asura_String;
class Asura_Cutscene;
class Asura_Cutscene_Track;
class Asura_Cutscene_Actor;
class Asura_Cutscene_Actor_Hierarchy;
class Asura_ClientCutscene_Actor_Hierarchy;
class Asura_ClientCutscene_Actor_CameraTarget;
class Asura_ClientCutscene_Actor_Camera;
class Asura_Cutscene_Attachment;

class Asura_Cutscene_Event;
class Asura_ClientCutscene_Event_AnimationChange;
class Asura_ClientCutscene_Event_SecondaryAnimation;
class Asura_ClientCutscene_Event_TimeScale;
class Asura_ClientCutscene_Event_FieldOfView;

class Asura_Chunk_Stream;
class Asura_SaveBlock;

class Asura_Chunk_Cutscene : public Asura_Chunk_Header
{
	public:
		bool Process(Asura_Handle fileHandle);
#ifdef ASURA_TOOLS
		static void Create(Asura_Cutscene* cutscene,Asura_SaveBlock& saveBlock);
#endif

		int NumberOfActors;
		int NumberOfEvents;
		Asura_Vector_3 Position;

		
		//static members for use during loading
		static Asura_Cutscene_Actor*& GetNextActor();
		static Asura_Cutscene_Event*& GetNextEvent();
		static Asura_Cutscene_Actor* GetActor(const Asura_String& name);
		
		static Asura_Cutscene* CurrentCutscene;
		static int NextActorIndex;
		static int NextEventIndex;

#ifdef ASURA_TOOLS
		static bool s_bCutscenesLoadedFromEntityFile;
#endif

};



////////////////////////////////////////////////////////////////////////////////////////
class Asura_Chunk_Actor : public Asura_Chunk_Header
{
	public :
		bool Process(Asura_Handle fileHandle);
#ifdef ASURA_TOOLS
		static void Create(Asura_Cutscene_Actor* actor,Asura_Chunk_Stream& xStream);
		static void Create(Asura_Cutscene_Actor* cutscene,Asura_SaveBlock& saveBlock);
#endif
		
		int Type;
		int NumberOfTracks;

		//static members for use during loading
		static Asura_Cutscene_Track*& GetNextTrack();
		static Asura_Cutscene_Attachment*& GetNextAttachment();
		
		static Asura_Cutscene_Actor* CurrentActor;
		static int NextTrackIndex;
};

////////////////////////////////////////////////////////////////////////////////////////

static const int ASURA_CHUNK_CUTSCENETRACK_VERSION = 7;
class Asura_Chunk_CutsceneTrack : public Asura_Chunk_Header
{
	public :
		bool Process(Asura_Handle fileHandle);
#ifdef ASURA_TOOLS
		static void Create(Asura_Cutscene_Track* track,Asura_Chunk_Stream& xStream);
		static void Create(Asura_Cutscene_Track* cutscene,Asura_SaveBlock& saveBlock);
#endif

		int NumberOfPoints;

		enum ChunkFlags
		{
			ChunkFlag_HasDurationArray = 0x00000001,
		};
};

////////////////////////////////////////////////////////////////////////////////////////
class Asura_Chunk_CutsceneAttachment : public Asura_Chunk_Header
{
	public :
		bool Process(Asura_Handle iFileHandle);
#ifdef ASURA_TOOLS
		static void Create(Asura_Cutscene_Attachment* pxAttachment,Asura_Chunk_Stream& xStream);
		static void Create(Asura_Cutscene_Attachment* cutscene,Asura_SaveBlock& saveBlock);
#endif
		
		int m_iType;
};

////////////////////////////////////////////////////////////////////////////////////////
class Asura_Chunk_CutsceneEvent : public Asura_Chunk_Header
{
	public :

		bool Process(Asura_Handle fileHandle);
#ifdef ASURA_TOOLS
		static void Create(Asura_Cutscene_Event* event,Asura_Chunk_Stream& xStream);
		static void Create(Asura_Cutscene_Event* cutscene,Asura_SaveBlock& saveBlock);
#endif

		int Type;
		float TriggerTime;//time into the cutscene that this event occurs

};


#endif //_ASURA_CHUNK_CUTSCENE_H_
