
///////////////////////////////////////////////////////////////////////////////
//
// File: Asura_Chunk_Decals.cpp
//
// Created by Dave Reed - 8/2/07
//
// Copyright (C) 2006 Rebellion (Derby), All Rights Reserved.
//
// Chunk code for decal definitions
//
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

#ifdef ASURA_USE_DECAL_MANAGER

#include "ChunkSystem/Asura_Chunk_Decals.h"
#include "ChunkSystem/Asura_SaveBlock.h"
#include "ChunkSystem/Asura_Chunk_Macros.h"
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "FileAccess/Asura_FileAccess.h"
#include "ChunkSystem/Asura_Chunk_Loaders.h"

#include "SpecialFX/Asura_Decal_Manager.h"

////////////
// Create //
////////////
#ifdef ASURA_TOOLS


bool Asura_Chunk_Decals::Save( const char* const szFileName )
{
	Asura_SaveBlock* pxSaveBlock = new Asura_SaveBlock();
	Asura_Assert(pxSaveBlock, "Unable to create saveblock");
	Asura_Chunk_Decals::Create(*pxSaveBlock);
	bool bSuccess=pxSaveBlock->SaveFile(szFileName, false );
	delete pxSaveBlock;
	return bSuccess;
}


void Asura_Chunk_Decals::Create(Asura_SaveBlock& xSaveBlock)
{
	Asura_Chunk_SaveBlock_Stream	xStream (&xSaveBlock);
	xStream.WriteChunk_Start	(ASURA_CHUNK_DECALS, Asura_Chunk_Decals::CURRENT_VERSION, 0);
	
	// First, output the decal materials
	int iNumMaterials = Asura_Decal_Manager::GetNumDecalMaterials();
	xStream << iNumMaterials;		

	for(int i=0;i<iNumMaterials;i++)
	{
		Asura_DecalManager_Material *pxMtl = Asura_Decal_Manager::GetDecalMaterialByIndex(i);				
		xStream << pxMtl->m_xMtlName;	
#if !defined(ASURA_TARGET_SM3)
		xStream << pxMtl->m_xTextureName;	
#else
		xStream << Asura_String("");	
#endif
		xStream << pxMtl->m_uMtlNameHash;
#if defined(ASURA_TARGET_SM3)
		xStream << pxMtl->m_uMaterialResponseHash;
#else
		xStream << 0;
#endif

		xStream << pxMtl->m_iMaxDecals;
#if defined(ASURA_TARGET_SM3)
		xStream << 0;
#else
		xStream << pxMtl->m_iBlendMode;
#endif
		xStream << pxMtl->m_bEnableFades;
		xStream << pxMtl->m_fFadeInLength;
		xStream << pxMtl->m_fFadeOutLength;
		xStream << pxMtl->m_fLifeTime;	

		xStream << pxMtl->m_bStaticDecalBackFaceCull;
		xStream << pxMtl->m_bStaticDecalBackFaceProject;
		xStream << pxMtl->m_bExportAsStaticDecal;
		
		xStream << pxMtl->m_fUScrollSpeed;
		xStream << pxMtl->m_fVScrollSpeed;
		xStream << pxMtl->m_fUScrollRange;
		xStream << pxMtl->m_fVScrollRange;		
				

	}

	// Next, output the decal sets
	int iNumSets = Asura_Decal_Manager::GetNumDecalSets();
	xStream << iNumSets;		

	for(int i=0;i<iNumSets;i++)
	{
		Asura_DecalManager_DecalSet *pxSet = Asura_Decal_Manager::GetDecalSetByIndex(i);

		xStream << pxSet->m_xSetName;
		xStream << pxSet->m_uMtlNameHash;
		xStream << pxSet->m_uSetNameHash;
		xStream << pxSet->m_bRandomOrientation;
		xStream << pxSet->m_iNumDecals;

		for(int j=0; j< pxSet->m_iNumDecals; j++)
		{
			xStream << pxSet->m_pxDecals[j].m_fU;
			xStream << pxSet->m_pxDecals[j].m_fV;
			xStream << pxSet->m_pxDecals[j].m_fDeltaU;
			xStream << pxSet->m_pxDecals[j].m_fDeltaV;
			xStream << pxSet->m_pxDecals[j].m_iFlags;
			xStream << pxSet->m_pxDecals[j].m_fWidth;
			xStream << pxSet->m_pxDecals[j].m_fHeight;
		}
	}

	xStream.WriteChunk_End();
	xStream.WriteAsuraTerminator();
}

#endif // ASURA_TOOLS

/////////////
// Process //
/////////////
bool Asura_Chunk_Decals::Process(Asura_Handle iFileHandle) const
{
	Asura_Chunk_File_Stream	xStream(iFileHandle);
	
	// First, load the decal materials
	int iNumMaterials = 0;
	xStream >> iNumMaterials;		

	Asura_Decal_Manager::s_pxMaterials = new Asura_DecalManager_Material[iNumMaterials];
	Asura_Decal_Manager::s_iNumMaterials=iNumMaterials;
	
	for(int i=0;i<iNumMaterials;i++)
	{
		Asura_DecalManager_Material *pxMtl = Asura_Decal_Manager::GetDecalMaterialByIndex(i);				
		xStream >> pxMtl->m_xMtlName;	
#if !defined(ASURA_TARGET_SM3)
		xStream >> pxMtl->m_xTextureName;	
		pxMtl->m_xTextureHash     = Asura_GetHashID_IgnoreExtension(pxMtl->m_xTextureName.GetString());
#else
		Asura_String xDummy;
		xStream >> xDummy;	
#endif

		Asura_String	xTempString;

		if (Version < 4)
        {
			// Need to be read here...
			xStream >> xTempString;
			xStream >> xTempString;
        } 
#if defined(ASURA_TARGET_SM3)       
		if (Version>1 && Version < 4)
		{		
			xStream >> xTempString;	
		}
#endif

		xStream >> pxMtl->m_uMtlNameHash;
		if (Version > 3)
		{
#if defined(ASURA_TARGET_SM3)
			xStream >> pxMtl->m_uMaterialResponseHash;
#else
			int idummy;
			xStream >> idummy;
#endif
		}

		xStream >> pxMtl->m_iMaxDecals;
#if defined(ASURA_TARGET_SM3)
		int idummy;
		xStream >> idummy;
#else
		xStream >> pxMtl->m_iBlendMode;
#endif
		xStream >> pxMtl->m_bEnableFades;
		xStream >> pxMtl->m_fFadeInLength;
		xStream >> pxMtl->m_fFadeOutLength;
		xStream >> pxMtl->m_fLifeTime;					

		if (Version>0)
		{
			xStream >> pxMtl->m_bStaticDecalBackFaceCull;
			xStream >> pxMtl->m_bStaticDecalBackFaceProject;
			xStream >> pxMtl->m_bExportAsStaticDecal;
		}

		if (Version>2)
		{
			xStream >> pxMtl->m_fUScrollSpeed;
			xStream >> pxMtl->m_fVScrollSpeed;
			xStream >> pxMtl->m_fUScrollRange;
			xStream >> pxMtl->m_fVScrollRange;
		}
	}


	// Next, output the decal sets
	int iNumSets;
	xStream >> iNumSets;		

	Asura_Decal_Manager::s_pxSets = new Asura_DecalManager_DecalSet[iNumSets];
	Asura_Decal_Manager::s_iNumSets = iNumSets;

	for(int i=0;i<iNumSets;i++)
	{
		Asura_DecalManager_DecalSet *pxSet = Asura_Decal_Manager::GetDecalSetByIndex(i);

		xStream >> pxSet->m_xSetName;
		xStream >> pxSet->m_uMtlNameHash;
		xStream >> pxSet->m_uSetNameHash;
		xStream >> pxSet->m_bRandomOrientation;
		xStream >> pxSet->m_iNumDecals;

		if (pxSet->m_iNumDecals==0)
		{
			pxSet->m_pxDecals=NULL;
		}
		else
		{
			pxSet->m_pxDecals = new Asura_DecalManager_Decal[pxSet->m_iNumDecals];

			for(int j=0; j< pxSet->m_iNumDecals; j++)
			{
				xStream >> pxSet->m_pxDecals[j].m_fU;
				xStream >> pxSet->m_pxDecals[j].m_fV;
				xStream >> pxSet->m_pxDecals[j].m_fDeltaU;
				xStream >> pxSet->m_pxDecals[j].m_fDeltaV;
				xStream >> pxSet->m_pxDecals[j].m_iFlags;
				xStream >> pxSet->m_pxDecals[j].m_fWidth;
				xStream >> pxSet->m_pxDecals[j].m_fHeight;
			}
		}
	}

	Asura_Decal_Manager::RebuildTemplates();

	return true;
}


#endif // ASURA_USE_DECAL_MANAGER
