//
// Asura_Chunk_FMVSubtitles.cpp
//
// Created by Richard Rice - 15/07/2003
// 
// Copyright (C) 2003 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"
#include "ChunkSystem/Asura_Chunk_FMVSubtitles.h"

#include "ChunkSystem/Asura_SaveBlock.h"
#include "FileAccess/Asura_FileAccess.h"

#include "ChunkSystem/Asura_Chunk_Loaders.h"
#include "ChunkSystem/Asura_Chunk_Macros.h"
#include "Cutscene/Asura_FMVSubtitles.h"

bool Asura_Chunk_FMVSubtitles::Process(Asura_Handle iFileHandle) const
{
	if( Version > CHUNK_VERSION )
	{
		Asura_Assert( Version <= CHUNK_VERSION, "Unsupported chunk version." );
		return false;
	}

	//clear out the old subtitles
	Asura_FMVSubtitles::Shutdown();


	//and read in the new ones
	ASURA_CHUNK_FILEEXTRACT(iFileHandle,Asura_FMVSubtitles::s_iNumSubtitles,int);
	if(!Asura_FMVSubtitles::s_iNumSubtitles) return true;
	
	Asura_FMVSubtitles::s_pxSubtitles = new Asura_FMVSubtitle[Asura_FMVSubtitles::s_iNumSubtitles];
	Asura_FileAccess::ReadData( iFileHandle, Asura_FMVSubtitles::s_pxSubtitles, sizeof(Asura_FMVSubtitle)*Asura_FMVSubtitles::s_iNumSubtitles);

	return true;
}

#ifdef ASURA_TOOLS
void Asura_Chunk_FMVSubtitles::Create(Asura_SaveBlock* pxSaveBlock,int iNumSubtitles,const struct Asura_FMVSubtitle* pxSubtitles)
{
	//determine chunk size
	int iChunkSize = sizeof(Asura_Chunk_FMVSubtitles);
	iChunkSize += sizeof(int) + iNumSubtitles*sizeof(Asura_FMVSubtitle);

	//allocate chunk
	Asura_Chunk_FMVSubtitles* pxChunk = (Asura_Chunk_FMVSubtitles*) pxSaveBlock->GetAccessPointer(iChunkSize);

	//fill in chunk header
	pxChunk->ID = (ASURA_CHUNKID)ASURA_CHUNK_FMVSUBTITLES;
	ASURA_CHUNK_VERIFY(pxChunk->ID, int);
	pxChunk->Size = iChunkSize;
	ASURA_CHUNK_VERIFY(pxChunk->Size, int);
	pxChunk->Version = CHUNK_VERSION;
	ASURA_CHUNK_VERIFY(pxChunk->Version, int);
	pxChunk->Flags = 0;
	ASURA_CHUNK_VERIFY(pxChunk->Flags, int);

	char* pcChunkBuffer = (char*)(pxChunk+1);

	ASURA_CHUNK_FILLANDVERIFY(pcChunkBuffer,iNumSubtitles,int);
	for(int i=0;i<iNumSubtitles;i++)
	{
		Asura_FMVSubtitle xTemp = pxSubtitles[i];
		ASURA_CHUNK_VERIFY(xTemp.m_fStartTime,float);
		ASURA_CHUNK_VERIFY(xTemp.m_fEndTime,float);
		ASURA_CHUNK_VERIFY(xTemp.m_uTextID,Asura_Hash_ID);

		ASURA_CHUNK_FILL(pcChunkBuffer,xTemp,Asura_FMVSubtitle);
	}

	Asura_Assert((int)pcChunkBuffer-(int)pxChunk == iChunkSize,"Incorrect chunk size");
	
}
#endif // ASURA_TOOLS
