///////////////////////////////////////////////////////////////////////////////
//
// File   : Asura_Chunk_FSFXOverrides.cpp
// Author : Andy Southgate
// Date   : 22-Apr-2009
//
// Copyright (C) 2009 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "ChunkSystem/Asura_Chunk_FSFXOverrides.h"

// Asura independent header-files...
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "ChunkSystem/Asura_SaveBlock.h"
#include "FSFX/Asura_FSFX_System.h"

#if defined(ASURA_TOOLS)
// Tools header-files...
#include "FSFX/Tools_Mode_FSFX.h"
#endif	// defined(ASURA_TOOLS)


////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////

static const int	g_iChunkVersion	= 0;


////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Private functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Protected functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Public functions...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Chunk_FSFXOverrides::Process
// Author       : Andy Southgate
// Date         : 22-Apr-2009
////////////////////////////////////////////////////////////////////////////////

template <typename T> static void LoadOverridesFromStream(Asura_ResourceSetWithOverride<T>& xResourceSet, Asura_Chunk_Stream &xStream)
{
	u_int	uNumOverrideMaps;

	// Read each override map in-turn...
	xStream >> uNumOverrideMaps;

	for(u_int u = 0; u < uNumOverrideMaps; u++)
	{
		u_int								uNumOverrides;
		Asura_Hash_ID						uOverrideMapHash;
		Asura_ResourceSetOverrideMap<T> *pxOverrideMap;

		// Read the name of this override-map...
		xStream >> uOverrideMapHash;

		// Ensure we have such an override-map...
		pxOverrideMap		= xResourceSet.GetOverrideMapByHash(uOverrideMapHash);

		if(pxOverrideMap == NULL)
		{
			pxOverrideMap	= xResourceSet.AllocateOverrideMap(uOverrideMapHash);
		}

		// Read each override in-turn...
		xStream >> uNumOverrides;

		for(u_int v = 0; v < uNumOverrides; v++)
		{
			Asura_Hash_ID	uTemplateHash;
			Asura_Hash_ID	uOverrideHash;
			Asura_Hash_ID	uOverrideNewHash;

			// Read the name of this override...
			xStream >> uTemplateHash;
			xStream >> uOverrideHash;

			//Skip any blank entries that were saved as a result of missing materials.RAR 21/11/2007
			if(!uTemplateHash)	continue;

			// Ensure we have such an override...
			uOverrideNewHash	= pxOverrideMap->GetOverride(uTemplateHash);

			if(uOverrideNewHash == uTemplateHash)
			{
				pxOverrideMap->CreateOverride(uTemplateHash, uOverrideHash);
			}
		}
	}
}


bool Asura_Chunk_FSFXOverrides::Process(const Asura_Handle iFileHandle) const
{
	if(Version > g_iChunkVersion)
	{
		Asura_Assert(false, "Unexpected chunk version found.");
		return(false);
	}
	else
	{
		Asura_Chunk_File_Stream	xStream(iFileHandle);

		LoadOverridesFromStream(Asura_FSFX_System::GetResourceSet(), xStream);

		return(true);
	}
}

/// End of function 'Asura_Chunk_FSFXOverrides::Process'. ///


#if defined(ASURA_TOOLS)

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Chunk_FSFXOverrides::Create
// Author       : Andy Southgate
// Date         : 22-Apr-2009
////////////////////////////////////////////////////////////////////////////////

template <typename T> static void SaveOverridesToStream(Asura_ResourceSetWithOverride<T>& xResourceSet, Asura_Chunk_Stream &xStream)
{
	u_int	uNumOverrideMaps = xResourceSet.GetNumOverrideMaps();

	// Read each override map in-turn...
	xStream << uNumOverrideMaps;

	for(Asura_ResourceSetWithOverride_ConstOverrideMapIterator<T> xIter(xResourceSet); !xIter.Done(); xIter.Next())
	{
		const Asura_ResourceSetOverrideMap<T> *pxOverrideMap = xIter.GetCurrent();
		u_int								uNumOverrides = pxOverrideMap->GetNumEntries();

		// Read the name of this override-map...
		xStream << Asura_String(pxOverrideMap->GetName()).GetHash();

		// Read each override in-turn...
		xStream << uNumOverrides;


		for(Asura_ResourceSetOverrideMap_ConstIterator<T> xIter(*pxOverrideMap); !xIter.Done(); xIter.Next())
		{
			xStream << xIter.GetCurrentHash();
			xStream << xIter.GetCurrent()->GetHash();
		}
	}

}


void Asura_Chunk_FSFXOverrides::Create(Asura_Chunk_Stream& xStream)
{
	// Begin writing...
	xStream.WriteChunk_Start(ASURA_CHUNK_FSFX_OVERRIDES, g_iChunkVersion, 0);

	SaveOverridesToStream(Asura_FSFX_System::GetResourceSet(), xStream);

	xStream.WriteChunk_End();
}

/// End of function 'Asura_Chunk_FSFXOverrides::Create'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Chunk_FSFXOverrides::Save
// Author       : Andy Southgate
// Date         : 22-Apr-2009
////////////////////////////////////////////////////////////////////////////////

bool Asura_Chunk_FSFXOverrides::Save(const char* szFilename)
{
	Asura_SaveBlock					xSaveBlock;
	Asura_Chunk_SaveBlock_Stream	xStream(&xSaveBlock);

	// Fill-in the save-block...
	Create(xStream);

	// Terminate the Asura-file...
	xStream.WriteAsuraTerminator();

	// ...before saving the file to disc...
	return(xSaveBlock.SaveFile(szFilename, false));
}

/// End of function 'Asura_Chunk_FSFXOverrides::Save'. ///

#endif	// defined(ASURA_TOOLS)


/// End of file. ///
