//
// Asura_Chunk_GUIMenu.cpp
//
// Created by Steven Keen - 5/12/2006
// 
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
#include "Core/Asura.h"

#ifdef ASURA_USE_GUI_MENU_SYSTEM
#include "Asura_Chunk_GUIMenu.h"

#ifdef ASURA_TOOLS
#include "Core/Asura_HashTree.h"
#endif
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "ChunkSystem/Asura_SaveBlock.h"

#include "GUIMenu/Asura_GUIMenu_Allocators.h"
#include "GUIMenu/Asura_GUIMenu_Animator.h"
#include "GUIMenu/Asura_GUIMenu_Root.h"
#include "GUIMenu/Asura_GUIMenu_Menu_Page.h"
#include "GUIMenu/Asura_GUIMenu_Audio.h"
#include "GUIMenu/Asura_GUIMenu_System.h"

#ifdef ASURA_TOOLS
#include "GUIMenu/Tools_GUIMenu_System.h"
#include "Misc/list_tem.hpp"
#endif

/////////////////////////////////////////////////////////////////////////////////////////////////////////////

#define ASURA_CHUNK_GUIMENU_VERSION 0

#ifdef ASURA_TOOLS

void Asura_Chunk_GUIMenu::Create(const Asura_GUIMenu_Menu_Page* pxMenuPage,Asura_SaveBlock* pxSaveBlock)
{
	Asura_Chunk_SaveBlock_Stream xStream(pxSaveBlock);
	Create	( pxMenuPage, xStream);
}

// --------------------------------------------------------------------------------------------

void Asura_Chunk_GUIMenu::Create(const Asura_GUIMenu_Menu_Page* pxMenuPage, Asura_Chunk_Stream& xStream)
{
	u_int uChunkFlags = 0;

	xStream.WriteChunk_Start( ASURA_CHUNK_GUIMENU_MENUPAGE, ASURA_CHUNK_GUIMENU_VERSION, uChunkFlags);

	pxMenuPage->WriteToChunkStream( xStream);

	xStream.WriteChunk_End();

}
#endif
// ------------------------------------------------------------------------------------------------------

bool Asura_Chunk_GUIMenu::Process(Asura_Handle iFileHandle) const
{
	Asura_Chunk_File_Stream	xStream(iFileHandle);
	return Process(xStream);
}

bool Asura_Chunk_GUIMenu::Process(Asura_Chunk_Stream& xStream) const
{
	bool bRet = false;

	if (Version > ASURA_CHUNK_GUIMENU_VERSION)
	{
		Asura_Assertf(false, ("Unknown chunk version %d for Asura_Chunk_GUIMenu", Version));
		return false;
	}

	bRet = ReadChunkData( xStream);

	return bRet;
}

// ------------------------------------------------------------------------------------------------------

bool Asura_Chunk_GUIMenu::ReadChunkData( Asura_Chunk_Stream& xStream) const
{
	u_int uType = Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_NONE;
	u_int uChunkVersion;
	bool bSuccess = false;

	xStream >> uChunkVersion;
	xStream >> uType;

	switch( uType)
	{
		case Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_MENUPAGE:
		{
			Asura_GUIMenu_Menu_Page* pxMenuPage = new Asura_GUIMenu_Menu_Page();
			pxMenuPage->ReadFromChunkStream( xStream, NULL, uChunkVersion);
			bSuccess = true;
		}
		break;

		default:
			Asura_Assert( 0, "Asura_Chunk_GUIMenu::ReadChunkData - This isn't a menu_page chunk.");
			break;
	}

	return bSuccess;
}

// ------------------------------------------------------------------------------------------------------
/////////////////////////////////////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_TOOLS
void Asura_Chunk_GUIMenu_Audio::Create( Asura_SaveBlock* pxSaveBlock)
{
	Asura_Chunk_SaveBlock_Stream xStream(pxSaveBlock);
	Create( xStream);
}

// --------------------------------------------------------------------------------------------

void Asura_Chunk_GUIMenu_Audio::Create( Asura_Chunk_Stream& xStream)
{
	u_int uChunkFlags = 0;

	xStream.WriteChunk_Start( ASURA_CHUNK_GUIMENU_AUDIO, ASURA_CHUNK_GUIMENU_AUDIO_VERSION, uChunkFlags);

	Asura_GUIMenu_Audio::WriteToChunkStream( xStream);
//	pxMenuPage->WriteToChunkStream( xStream);

	xStream.WriteChunk_End();

}
#endif
// ------------------------------------------------------------------------------------------------------

bool Asura_Chunk_GUIMenu_Audio::Process(Asura_Handle iFileHandle) const
{
	Asura_Chunk_File_Stream	xStream(iFileHandle);
	return Process(xStream);
}

bool Asura_Chunk_GUIMenu_Audio::Process(Asura_Chunk_Stream& xStream) const
{
	bool bRet = false;

	if (Version > ASURA_CHUNK_GUIMENU_AUDIO_VERSION)
	{
		Asura_Assertf(false, ("Unknown chunk version %d for Asura_Chunk_GUIMenu", Version));
		return false;
	}

	Asura_GUIMenu_Audio::ReadFromChunkStream( xStream);

	return bRet;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////
/*--------------------------**
** Asura_Chunk_GUIMenu_File **
**--------------------------*/
#define ASURA_CHUNK_GUIMENU_FILE_VERSION 0

#ifdef ASURA_TOOLS

bool Asura_Chunk_GUIMenu_File::Process(Asura_Chunk_Stream& xStream) const
{
	// Is this the correct version?

	if (Version > ASURA_CHUNK_GUIMENU_FILE_VERSION)
	{
		Asura_Assertf(false, ("Unknown chunk version %d for GUImenu file settings",Version));
		return false;
	}

	char szFilename[MAX_PATH];
	xStream.ReadString(szFilename,MAX_PATH);

	int iSortKey;
	xStream >> iSortKey;

	Tools_GUIMenu_System::SetFileSortKey(szFilename, iSortKey);	

	return true;
}

bool Asura_Chunk_GUIMenu_File::Process(Asura_Handle iFileHandle) const
{
	Asura_Chunk_File_Stream	xStream(iFileHandle);
	return Process(xStream);
}

void Asura_Chunk_GUIMenu_File::Create(const char* szFile,Asura_Chunk_Stream& xStream)
{
	u_int uChunkFlags = 0;
	xStream.WriteChunk_Start(ASURA_CHUNK_GUIMENU_FILE, ASURA_CHUNK_GUIMENU_FILE_VERSION, uChunkFlags);

	Asura_String xTemp = szFile;

	xStream << xTemp;
	xStream << Tools_GUIMenu_System::GetFileSortKey(szFile);


	xStream.WriteChunk_End();
}

void Asura_Chunk_GUIMenu_File::Create(const char* szFile,Asura_SaveBlock* pxSaveBlock)
{
	Asura_Chunk_SaveBlock_Stream xStream(pxSaveBlock);
	Create(szFile, xStream);
}


#endif // ASURA_TOOLS


//////////////////////////////////////////////////////////////////////////////
// Asura_Chunk_GUIMenu_AnimatorSequences
//////////////////////////////////////////////////////////////////////////////
#define ASURA_CHUNK_GUIMENU_ANIMATORSEQUENCES_VERSION 0

bool Asura_Chunk_GUIMenu_AnimatorSequences::Process(Asura_Chunk_Stream& xStream) const
{
	// Is this the correct version?
	if (Version > ASURA_CHUNK_GUIMENU_ANIMATORSEQUENCES_VERSION)
	{
		Asura_Assertf(false, ("Unknown chunk version %d for Asura_Chunk_GUIMenu_AnimatorSequences",Version));
		return false;
	}

	u_int uNumSequences;
	xStream >> uNumSequences;

	char acName[256];

	for (u_int u = 0; u < uNumSequences; ++u)
	{
		xStream.ReadString(acName, 256);
		if (Asura_GUIMenu_Animator_Storage::FindSequence( Asura_GetHashID(acName) ))
		{
			// already exists - skip the data
			if (!Asura_GUIMenu_Animator_Sequence::SkipChunkData( xStream ))
			{
				return false;
			}
		}
		else
		{
			Asura_GUIMenu_Animator_Sequence* pxNewSeq = new Asura_GUIMenu_Animator_Sequence(acName);
			if (!pxNewSeq->ReadFromChunkStream( xStream ))
			{
				return false;
			}
		}
	}

	return true;
}

bool Asura_Chunk_GUIMenu_AnimatorSequences::Process(Asura_Handle iFileHandle) const
{
	Asura_Chunk_File_Stream	xStream(iFileHandle);
	return Process(xStream);
}

#ifdef ASURA_TOOLS
void Asura_Chunk_GUIMenu_AnimatorSequences::Create(Asura_Chunk_Stream& xStream)
{
	u_int uChunkFlags = 0;
	xStream.WriteChunk_Start( ASURA_CHUNK_GUIMENU_ANIMSEQUENCE, ASURA_CHUNK_GUIMENU_ANIMATORSEQUENCES_VERSION, uChunkFlags );

	u_int uNumSequences = Asura_GUIMenu_Animator_Storage::GetNumberOfSequences();

	xStream << uNumSequences;

	List<const Asura_GUIMenu_Animator_Sequence*> xSequenceList;
	if(uNumSequences)
	{
		for(Asura_TemplateHashTree_SaveIterator xIt(uNumSequences);!xIt.Done();xIt.Next())
		{
			const Asura_GUIMenu_Animator_Sequence* pxSequence = Asura_GUIMenu_Animator_Storage::GetSequenceByIndex(xIt.GetCurrent());
			Asura_Assert(pxSequence, "");
			xStream << pxSequence->GetNameAsString();
			pxSequence->WriteToChunkStream(xStream);
		}
	}

	xStream.WriteChunk_End();
}

void Asura_Chunk_GUIMenu_AnimatorSequences::Create(Asura_SaveBlock* pxSaveBlock)
{
	Asura_Chunk_SaveBlock_Stream xStream(pxSaveBlock);
	Create(xStream);
}
#endif // ASURA_TOOLS


//////////////////////////////////////////////////////////////////////////////////////////////////////////
/*------------------------------**
** Asura_Chunk_GUIMenu_TextPage **
**------------------------------*/
#define ASURA_CHUNK_GUIMENU_TEXTPAGE_VERSION 0


bool Asura_Chunk_GUIMenu_TextPage::Process(Asura_Chunk_Stream& xStream) const
{
	// Is this the correct version?

	if (Version > ASURA_CHUNK_GUIMENU_TEXTPAGE_VERSION)
	{
		Asura_Assertf(false, ("Unknown chunk version %d for GUImenu text page chunk",Version));
		return false;
	}

	char szName[200];
	xStream.ReadString(szName,200);

	Asura_Hash_ID uPageID;
	xStream >> uPageID;

	Asura_GUIMenu_System::LoadMenuText(szName, uPageID);

	return true;
}

bool Asura_Chunk_GUIMenu_TextPage::Process(Asura_Handle iFileHandle) const
{
	Asura_Chunk_File_Stream	xStream(iFileHandle);
	return Process(xStream);
}

#ifdef ASURA_TOOLS
void Asura_Chunk_GUIMenu_TextPage::Create( Asura_Hash_ID uPageID, class Asura_Chunk_Stream& xStream)
{
	const Asura_TemplateHashTree<Asura_String>& xFilenameTree = Asura_GUIMenu_System::GetMenuTextFilenameTree();
	const Asura_String* const pxFilename = xFilenameTree.GetDataByHash(uPageID); 

	if(!pxFilename)
	{
		Asura_Assertf(0, ("Failed to find name of text page 0x%x in GUI menu system", uPageID));
		return ;
	}

	u_int uChunkFlags = 0;
	xStream.WriteChunk_Start(ASURA_CHUNK_GUIMENU_TEXTPAGE, ASURA_CHUNK_GUIMENU_TEXTPAGE_VERSION, uChunkFlags);


	xStream << (*pxFilename);
	xStream << uPageID;

	xStream.WriteChunk_End();
}

void Asura_Chunk_GUIMenu_TextPage::Create( Asura_Hash_ID uPageID, Asura_SaveBlock* pxSaveBlock)
{
	Asura_Chunk_SaveBlock_Stream xStream(pxSaveBlock);
	Create( uPageID, xStream);
}

#endif // ASURA_TOOLS




#endif //ASURA_USE_GUI_MENU_SYSTEM

