//
// Asura_Chunk_GlobalIllumination.h
//
// Created by KJFL - 2004-06-09 12:34:09
//
// Copyright (C) 2004 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif

#ifndef __ASURA_CHUNK_GLOBALILLUMINATION_H__
#define __ASURA_CHUNK_GLOBALILLUMINATION_H__

// KJFL - 2004-04-26 - this chunk is only to be used in Tools builds.
#ifdef ASURA_TOOLS

#include "ChunkSystem/Asura_Chunk_Loaders.h"
#include "Core/Asura_Handle.h"

// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////////

class Asura_SaveBlock;

// C L A S S E S /////////////////////////////////////////////////////////////

class Asura_Chunk_GlobalIllumination : public Asura_Chunk_Header
{

	public:
        
		bool Process( Asura_Handle hFile ) const;

		//Extracts lights from a chunkbuffer , assinging them new IDs.
		//If uSavedID is specified then only that light will be created , and its handle will
		//be returned.
		Asura_Handle ImportFromChunkBuffer(const char* pcChunkBuffer,u_int uSavedID,u_int uNewID) const;


		//If bTemplateLights is true then this chunk is being saved out into template file , and 
		//so should just contain the lights that belong to it. Otherwise the 'environment' lights
		//should be saved
		static bool Create( Asura_Chunk_Stream& xStream,bool bTemplateLights,bool bMissionLights );
		static bool Create( Asura_SaveBlock& xSaveBlock,bool bTemplateLights,bool bMissionLights );

		static bool SaveFile( const char* szFilename, bool bMissionFile );

		u_int GetNumberOfLights() const { return m_uNumberOfLights;}

	private:

		enum { CHUNK_VERSION = 14 };
		//Version 3 - Added m_bHasPhotonMap to the light source
		//		(RAR 06/09/2004)
		//Version 4 - Added angle to light source
		//		(RAR 08/09/2004)
		//Version 5 - Added short range flag
		//		(RAR 02/11/2004)
		//Version 6 - Added global ambience
		//		(RAR 25/11/2004)
		//Version 7 - Changed format of samples
		//		(RAR 09/12/2004)
		//Version 8 - Changed PhotonMapValid flag to a PhotonMapValidity
		//		(RAR 21/01/2005)
		//Version 9 -Added inner/outer range
		//		(RAR 28/01/2005)
		//Version 10-Light names
		//		(RAR 08/07/2005)
		//Version 11-Reflection Chance
		//		(RAR 21/10/2005)
		//Version 12-Lighting Sets
		//		(RAR 24/03/2006)
		//Version 13 - Added the SH resample values (2 floats at the end)
		//		(KJFL 4/12/2007)
		//Version 14-Lighting set changes to allow for the 'groups' of sets
		//		(RAR 06/03/2008)

		u_int	m_uNumberOfLights;
		u_int	m_uNumberOfModules;

};

class Asura_Chunk_PolygonLights : public Asura_Chunk_Header
{
	public:
        
		bool Process( Asura_Handle hFile ) const;



		//If bTemplateLights is true then this chunk is being saved out into template file , and 
		//so should just contain the lights that belong to it. Otherwise the 'environment' lights
		//should be saved
		static bool Create( Asura_Chunk_Stream& xSaveBlock,bool bTemplateLights );


	private:

		enum { CHUNK_VERSION = 6 };
		//Version 2 - Changed format of samples
		//		(RAR 09/12/2004)
		//Version 3 - Changed PhotonMapValid flag to a PhotonMapValidity
		//		(RAR 21/01/2005)
		//Version 4 -Added inner/outer range
		//		(RAR 28/01/2005)
		//Version 5-Reflection Chance
		//		(RAR 21/10/2005)
		//Version 6-Lighting Sets
		//		(RAR 24/03/2006)

		u_int	m_uNumberOfLights;
		u_int	m_uNumberOfModules;
};


#endif //#ifdef ASURA_TOOLS

#endif//__ASURA_CHUNK_GLOBALILLUMINATION_H__
