//
// Asura_Chunk_HashedLocalisedText.cpp
//
// Created by Richard Rice - 09/12/2002
// 
// Copyright (C) 2002 Rebellion, All Rights Reserved.
//


#include "Core/Asura.h"

#include "ChunkSystem/Asura_Chunk_HashedLocalisedText.h"

#include "ChunkSystem/Asura_Chunk_Macros.h"
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "ChunkSystem/Asura_SaveBlock.h"
#include "Core/Asura_HashedLocalisedText.h"
#include "Core/Asura_String.h"
#include "Debugging/Asura_Assert.h"
#include "FileAccess/Asura_FileAccess.h"


#define ASURA_CHUNK_HASHEDLOCALISEDTEXT_VERSION 2

bool Asura_Chunk_HashedLocalisedText::Process(Asura_Handle iFileHandle) const
{
	Asura_Assert(Version<=ASURA_CHUNK_HASHEDLOCALISEDTEXT_VERSION,"");

	if(!m_iNumberOfStrings) return false;

	Asura_Hash_ID uPageID = uASURA_HASHEDTEXTPAGE_CUTSCENE;
	u_int uSizeForStrings;
	if(Version >= 1)
	{
		Asura_FileAccess::ReadData(iFileHandle,&uPageID,sizeof(Asura_Hash_ID));
		Asura_FileAccess::ReadData(iFileHandle,&uSizeForStrings,sizeof(u_int));
	}
	else
	{
		// (MH - 22.04.2004) instead of doing a new for each string, allocate enough space
		// to hold all the strings and copy the string data to that. Each string is preceded
		// by an int indicating length and a hash ID.
		uSizeForStrings = this->Size - sizeof(Asura_Chunk_HashedLocalisedText);
		uSizeForStrings -= m_iNumberOfStrings * (sizeof(Asura_Hash_ID) + sizeof(int));
	}


	Asura_HashedLocalisedText_Page* pxPage = Asura_HashedLocalisedText::AllocateStorage(uPageID,m_iNumberOfStrings, uSizeForStrings);
	if(!pxPage) return false;

	Asura_UnicodeChar* wszString = reinterpret_cast<Asura_UnicodeChar*>(pxPage->m_pcStringBuffer);
	
	for(int i=0;i<m_iNumberOfStrings;i++)
	{
		Asura_Hash_ID uHash;
		Asura_FileAccess::ReadData(iFileHandle,&uHash,sizeof(Asura_Hash_ID));

		int iLength;
		Asura_FileAccess::ReadData(iFileHandle,&iLength,sizeof(int));

		Asura_FileAccess::ReadData(iFileHandle,wszString,iLength*2);

		pxPage->AddString(uHash,wszString);

		wszString += iLength;
	}
	Asura_Assert(wszString == reinterpret_cast<Asura_UnicodeChar*>(pxPage->m_pcStringBuffer + uSizeForStrings), "alloc error in hashed strings");

	if(Version >= 2)
	{
		Asura_String xPageName = Asura_FileAccess::ReadString( iFileHandle );

	#ifdef ASURA_TOOLS
		pxPage->m_xName = xPageName;
	#endif // ASURA_TOOLS
	}

	if(Version >= 1)
	{
		u_int uSizeForIDs;
		Asura_FileAccess::ReadData(iFileHandle,&uSizeForIDs,sizeof(u_int));

	#ifdef ASURA_TOOLS
		pxPage->m_pcTextIDBuffer = new char[uSizeForIDs];
		pxPage->m_pszTextIDs = new char*[m_iNumberOfStrings];

		//read in all the strings at once
		Asura_FileAccess::ReadData(iFileHandle,pxPage->m_pcTextIDBuffer,uSizeForIDs);

		//fix up the pointers to the strings
		char* pcPos = pxPage->m_pcTextIDBuffer;
		for(int i=0;i<m_iNumberOfStrings;i++)
		{
			pxPage->m_pszTextIDs[i] = pcPos;
			pcPos += strlen(pcPos)+1;
		}
	#else
		//don't need these outside of tools
		Asura_FileAccess::SkipData(iFileHandle,uSizeForIDs);
	#endif
	}

	return true;
}

#ifdef ASURA_TOOLS
Asura_Hash_ID Asura_Chunk_HashedLocalisedText::GetPageID(Asura_Chunk_Stream& xStream) const
{	
	Asura_Assert(Version<=ASURA_CHUNK_HASHEDLOCALISEDTEXT_VERSION,"");

	Asura_Hash_ID uPageID = uASURA_HASHEDTEXTPAGE_CUTSCENE;
	if(Version >= 1)
	{
		xStream >> uPageID;
	}
	return uPageID;
}
#endif //ASURA_TOOLS

#if defined(HASHED_TEXT_FILE_UTILITY) || defined(ASURA_TOOLS)

void Asura_Chunk_HashedLocalisedText::Create(const char* szPageName, List<Asura_Hash_ID>& xHashList, List<Asura_String>& xTextIDList, List<Asura_UnicodeChar*>& xStringList,class Asura_SaveBlock* pxSaveBlock)
{
	Asura_String xPageName = szPageName;

	if(!xHashList.size()) return;
	Asura_Assert(xHashList.size() == xStringList.size(),"");
	Asura_Assert(xHashList.size() == xTextIDList.size(),"");

	int iChunkSize = sizeof(Asura_Chunk_HashedLocalisedText) + sizeof(Asura_Hash_ID);
	//add space needed for each string
	int iNumStrings = xHashList.size();


	u_int uStringSize = 0;//total size for localised text
	u_int uIDSize = 0;//total size for text version of IDs


	for(int i=0;i<iNumStrings;i++)
	{
		const Asura_UnicodeChar* wszString = xStringList[i];
		int iLength = (wcslen(wszString)+2)&~1;
		uStringSize += iLength*2;

		uIDSize += xTextIDList[i].GetLength()+1;		
	}

	iChunkSize += uStringSize + uIDSize;
	iChunkSize += (sizeof(int) + sizeof(Asura_Hash_ID))*iNumStrings;//length and id of string
	iChunkSize += sizeof(u_int)*2;//for uStringsize and uIDSize;
	iChunkSize += xPageName.GetSizeInInts() * 4;//for tools name

	Asura_Chunk_HashedLocalisedText* pxChunk = (Asura_Chunk_HashedLocalisedText*) pxSaveBlock->GetAccessPointer(iChunkSize);
	pxChunk->ID = ASURA_CHUNK_HASHEDLOCALISEDTEXT;
	ASURA_CHUNK_VERIFY(pxChunk->ID,int);
	pxChunk->Size = iChunkSize;
	ASURA_CHUNK_VERIFY(pxChunk->Size,int);
	pxChunk->Version = ASURA_CHUNK_HASHEDLOCALISEDTEXT_VERSION;
	ASURA_CHUNK_VERIFY(pxChunk->Version,int);
	pxChunk->Flags = 0;
	ASURA_CHUNK_VERIFY(pxChunk->Flags,int);
	pxChunk->m_iNumberOfStrings = iNumStrings;
	ASURA_CHUNK_VERIFY(pxChunk->m_iNumberOfStrings,int);


	char* pcBuffer = (char*)(pxChunk+1);

	ASURA_CHUNK_FILLANDVERIFY(pcBuffer,Asura_GetHashID(szPageName),Asura_Hash_ID);

	ASURA_CHUNK_FILLANDVERIFY(pcBuffer,uStringSize,u_int);

	for(int i=0;i<iNumStrings;i++)
	{
		Asura_Hash_ID uHash = xHashList[i];
		ASURA_CHUNK_FILLANDVERIFY(pcBuffer,uHash,Asura_Hash_ID);

		const Asura_UnicodeChar* wszString = xStringList[i];

		//get length , including null terminator , and possible padding for 4 byte alignment
		int iLength = (wcslen(wszString)+2)&~1;
		ASURA_CHUNK_FILLANDVERIFY(pcBuffer,iLength,int);

		wcscpy((Asura_UnicodeChar*)pcBuffer,wszString);

		//make sure pad character (if it exists) won't be random junk
		((Asura_UnicodeChar*)pcBuffer)[iLength-1] = 0;
		for(int j=0;j<iLength;j++)
		{
			ASURA_CHUNK_VERIFY(pcBuffer[j*2],Asura_UnicodeChar);
		}

		pcBuffer += iLength*2;
	}

	//write out the un-hashed page name for tools use
	ASURA_CHUNK_FILL_STRING(pcBuffer,xPageName);

	//write out the un-hashed IDs for tools use
	ASURA_CHUNK_FILLANDVERIFY(pcBuffer,uIDSize,u_int);
	for(int i=0;i<iNumStrings;i++)
	{
		strcpy(pcBuffer,xTextIDList[i].GetString());
		pcBuffer += strlen(pcBuffer)+1;
	}
}

#endif // defined(HASHED_TEXT_FILE_UTILITY) || defined(ASURA_TOOLS)
