//
// Asura_Chunk_IK_Template.cpp
//
// Created by Richard Rice - 16/05/2008
// 
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"
#include "ChunkSystem/Asura_Chunk_IK_Template.h"

#ifdef ASURA_USE_ANIMATION_IK_SOLVER

#include "Animation/IK/Asura_Animation_IK_Template.h"
#include "Animation/IK/Asura_Animation_IK_Template_System.h"
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "ChunkSystem/Asura_SaveBlock.h"



#define ASURA_CHUNK_IK_TEMPLATE_VERSION 0


bool Asura_Chunk_IK_Template::Process(Asura_Chunk_Stream& xStream) const
{
	// Is this the correct version?

	if (Version > ASURA_CHUNK_IK_TEMPLATE_VERSION)
	{
		Asura_Assertf(false, ("Unknown chunk version %d for IK Template",Version));
		return false;
	}

	char acName[128];
	xStream.ReadString(acName,sizeof(acName));

	//make sure the template doesn't already exist
	if(Asura_Animation_IK_Template_System::FindTemplate(Asura_GetHashID(acName)))
	{
		#ifdef ASURA_TOOLS
		Asura_Assertf(0,("IK Template %s already exists",acName));
		#endif //#ifdef ASURA_TOOLS
		return false;
	}

	//allocate the template and read the rest in
	Asura_Animation_IK_Template* pxTemplate = new Asura_Animation_IK_Template(acName);
	bool bReadOkay = pxTemplate->ReadFromChunkStream(xStream);

	if(!bReadOkay)
	{
		delete pxTemplate;
	}
	return bReadOkay;
}

bool Asura_Chunk_IK_Template::Process(Asura_Handle iFileHandle) const
{
	Asura_Chunk_File_Stream	xStream(iFileHandle);
	return Process(xStream);
}

#ifdef ASURA_TOOLS
void Asura_Chunk_IK_Template::Create(const class Asura_Animation_IK_Template* pxTemplate,Asura_Chunk_Stream& xStream)
{
	u_int uChunkFlags = 0;

	xStream.WriteChunk_Start(ASURA_CHUNK_IK_TEMPLATE, ASURA_CHUNK_IK_TEMPLATE_VERSION, uChunkFlags);

	xStream << pxTemplate->m_xName;
	pxTemplate->WriteToChunkStream(xStream);

	xStream.WriteChunk_End();
}

void Asura_Chunk_IK_Template::Create(const class Asura_Animation_IK_Template* pxTemplate,Asura_SaveBlock* pxSaveBlock)
{
	Asura_Chunk_SaveBlock_Stream xStream(pxSaveBlock);
	Create	(pxTemplate,xStream);
}
#endif // ASURA_TOOLS

#endif //ASURA_USE_ANIMATION_IK_SOLVER
