//
// Asura_Chunk_Macros.h
//
// Created by Richard Rice - 10/07/2002
// 
// Copyright (C) 2002 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef __ASURA_CHUNK_MACROS_H__
#define __ASURA_CHUNK_MACROS_H_

// (AW - 12.07.2004) - Include Asura_FileAccess because some of the below macros refer to it.
#include "FileAccess/Asura_FileAccess.h"

/*
The aim here is to have a set of extract/size/fill macros for various basic types to simplify
writing the loading/saving code.
The arguments in each set of macros are similar , making copying and pasting bits between the
loading and saving sections easier. (well , thats the theory anyway).

CHUNK_FILEEXTRACT : read a type from a file
CHUNK_EXTRACT : read a type from a memory buffer , and advance the buffer pointer
CHUNK_PEEK : read a type from a memory buffer but leave the buffer pointer alone
CHUNK_FILL : write a type to a memory buffer ,and advance the buffer pointer
CHUNK_SIZE : add the size required for this type

-RAR
*/


//For simple structures :
#define ASURA_CHUNK_FILEEXTRACT( __fileHandle, __var, __type ) \
{ \
	Asura_FileAccess::ReadData(__fileHandle,&__var,sizeof(__type));\
}


#ifdef ASURA_CHUNK_USE_PER_BYTE_MACROS

#define ASURA_CHUNK_EXTRACT( __buffer, __var, __type ) \
{ \
	unsigned char* pxVar = (unsigned char*)&(__var); \
	for( unsigned int u=0; u<sizeof(__type); u++ ) \
	{ \
		*pxVar = *(const unsigned char*)(__buffer); \
		(__buffer)++; \
		pxVar++; \
	} \
}

#define ASURA_CHUNK_PEEK( __buffer, __var, __type ) \
{ \
	unsigned char* pxVar = (unsigned char*)&(__var); \
	for( unsigned int u=0; u<sizeof(__type); u++ ) \
	{ \
		*pxVar = *(const unsigned char*)(__buffer+u); \
		pxVar++; \
	} \
}

#define ASURA_CHUNK_FILL( __buffer, __var, __type ) \
{ \
	const __type xVar = (__var); \
	const unsigned char* pxVar = (const unsigned char*)&xVar; \
	for( unsigned int u=0; u<sizeof(__type); u++ ) \
	{ \
		*(unsigned char*)(__buffer) = *pxVar; \
		(__buffer)++; \
		pxVar++; \
	} \
}

#else 

#define ASURA_CHUNK_EXTRACT( __buffer, __var, __type ) \
{ \
	(__var) = *(const __type*)(__buffer); \
	(__buffer) += sizeof(__type); \
}

#define ASURA_CHUNK_PEEK( __buffer, __var, __type ) \
{ \
	(__var) = *(const __type*)(__buffer); \
}

#define ASURA_CHUNK_FILL( __buffer, __var, __type ) \
{ \
	*(__type*)(__buffer) = (__var); \
	(__buffer) += sizeof(__type); \
}

#endif


#define ASURA_CHUNK_SIZE( __chunksize, __var, __type) \
{ \
	(__chunksize) += sizeof(__type); \
}


//For Asura_String
#define ASURA_CHUNK_FILEEXTRACT_STRING(__fileHandle, __string)\
{\
	__string = Asura_FileAccess::ReadString(__fileHandle);\
}

#define ASURA_CHUNK_EXTRACT_STRING(__buffer, __string) \
{ \
	if(*(const int*)(__buffer)==Asura_String::NULL_STRING_CHUNK) \
		(__string).MakeNull(); \
	else \
		(__string) = (__buffer); \
	(__buffer) += (__string).GetSizeInInts()*4; \
}

#define ASURA_CHUNK_FILL_STRING(__buffer, __string) \
{ \
	(__buffer) = (char*)(__string).IntCopy((int*)__buffer); \
}
 

#define ASURA_CHUNK_SIZE_STRING(__chunksize, __string) \
{ \
	(__chunksize) += (__string.GetSizeInInts())*4; \
}

//Added check for ASURA_MAIN to allow the max plugins to compile. RAR 14/03/03
#if (defined(ASURA_TOOLS) && defined(ASURA_MAIN)) || defined(ASURA_RESOURCEPACKER) || defined(HASHED_TEXT_FILE_UTILITY)

#include "Load&Save/LevelExport/Tools_Level_Export.h"

/// Kim Randell - 11/02/2003 16:33:54 - endian swapper for Gamecube & PS3 exporting
#define ASURA_CHUNK_VERIFY(__a, __type)									\
Asura_Assert(sizeof(__type) <= 4, "Endian mangler cannot deal with types larger than 4 bytes.");	\
if(Tools_Level_Export::ShouldExportBigEndian())							\
{																		\
	switch(sizeof(__type))												\
	{																	\
		case 1:	break;													\
		case 2:															\
		{																\
			union														\
			{															\
				short twobytes;											\
				char bytes[2];											\
			};															\
			twobytes = *(short*)(&(__a));								\
			bytes[0] = (char)(bytes[0] ^ bytes[1]);						\
			bytes[1] = (char)(bytes[1] ^ bytes[0]);						\
			bytes[0] = (char)(bytes[0] ^ bytes[1]);						\
			*(short*)(&(__a)) = twobytes;								\
		}																\
		break;															\
		case 4:															\
		{																\
			union														\
			{															\
				int integer;											\
				char bytes[4];											\
			};															\
			integer = *(int*)(&(__a));									\
			bytes[0] = (char)(bytes[0] ^ bytes[3]);						\
			bytes[3] = (char)(bytes[3] ^ bytes[0]);						\
			bytes[0] = (char)(bytes[0] ^ bytes[3]);						\
			bytes[1] = (char)(bytes[1] ^ bytes[2]);						\
			bytes[2] = (char)(bytes[2] ^ bytes[1]);						\
			bytes[1] = (char)(bytes[1] ^ bytes[2]);						\
			*(int*)(&(__a)) = integer;									\
		}																\
		break;															\
		default:														\
			break;														\
	}																	\
}

// Useful for switching simple homogeneous structures like
// Asura_Vector_<n>, Asura_Matrix_3x3, Asura_Bounding_Box, etc.
#define ASURA_CHUNK_ARRAY_VERIFY(__a, __type, __elements)					\
Asura_Assert(sizeof(__type) <= 4, "Endian mangler cannot deal with types larger than 4 bytes.");	\
if(Tools_Level_Export::ShouldExportBigEndian() )							\
{																			\
	for( int i = 0; i < __elements; i++)									\
	{																		\
		switch(sizeof(__type))												\
		{																	\
			case 1:	break;													\
			case 2:															\
			{																\
				union														\
				{															\
					short twobytes;											\
					char bytes[2];											\
				};															\
				twobytes = *((short*)(&(__a))+i);							\
				bytes[0] = (char)(bytes[0] ^ bytes[1]);						\
				bytes[1] = (char)(bytes[1] ^ bytes[0]);						\
				bytes[0] = (char)(bytes[0] ^ bytes[1]);						\
				*((short*)(&(__a))+i) = twobytes;							\
			}																\
			break;															\
			case 4:															\
			{																\
				union														\
				{															\
					int integer;											\
					char bytes[4];											\
				};															\
				integer = *((int*)(&(__a))+i);								\
				bytes[0] = (char)(bytes[0] ^ bytes[3]);						\
				bytes[3] = (char)(bytes[3] ^ bytes[0]);						\
				bytes[0] = (char)(bytes[0] ^ bytes[3]);						\
				bytes[1] = (char)(bytes[1] ^ bytes[2]);						\
				bytes[2] = (char)(bytes[2] ^ bytes[1]);						\
				bytes[1] = (char)(bytes[1] ^ bytes[2]);						\
				*((int*)(&(__a))+i) = integer;								\
			}																\
			break;															\
			default:														\
				break;														\
		}																	\
	}																		\
}

// KOR - 30/04/2003 15:01:54 - Here's a fun one.
// Bitfields on little-endian architectures are stored working their way up in size,
// so for instance an unsigned int with 4 bitfields all set to 1 will be
// 0x0000000F ( == 00...001111)
// This is stored as 0F 00 00 00
// Bitfields on the Gamecube are stored working their way up in memory address, so the
// same flags will be stored as
// F0 00 00 00 ( == 111100...00)
// In other words, we have to swap the *bits*. not the bytes.
//
// _Hacker's Delight_ (Henry S. Warren Jr.), gives a fairly elegant way of doing this
// Adjacent bits are swapped, then adjacent two-bit pairs, then adjacent nibbles, and so on.
// The nice thing about this method is that it's trivially scaleable.
// Bear in mind, though, that we are only swapping bits in bytes and no further.
#define ASURA_CHUNK_BITFIELD_VERIFY(__var, __type) \
{ \
if(Tools_Level_Export::ShouldExportBigEndian() )	\
{																		\
	char* pcTemp = (char*)&__var; \
	for(unsigned int __uBytePos = 0; __uBytePos < sizeof(__type); __uBytePos++) \
	{ \
		*pcTemp = (char)((*pcTemp & 0x55) << 1 | (*pcTemp & 0xAA) >> 1); \
		*pcTemp = (char)((*pcTemp & 0x33) << 2 | (*pcTemp & 0xCC) >> 2); \
		*pcTemp = (char)((*pcTemp & 0x0F) << 4 | (*pcTemp & 0xF0) >> 4); \
		pcTemp++; \
	} \
} \
}

#else // defined ASURA_TOOLS && ASURA_MAIN

#ifndef ASURA_CHUNK_VERIFY
#define ASURA_CHUNK_VERIFY(a, type)	{}						
#endif
#ifndef ASURA_CHUNK_ARRAY_VERIFY
#define ASURA_CHUNK_ARRAY_VERIFY(a, type, elements)	{}
#endif
#ifndef ASURA_CHUNK_BITFIELD_VERIFY
#define ASURA_CHUNK_BITFIELD_VERIFY(__a, __type)	{}
#endif

#endif // defined ASURA_TOOLS && ASURA_MAIN

//Combination of the fill and verify macros. RAR 24/02/03
#define ASURA_CHUNK_FILLANDVERIFY(__buffer,__var,__type) \
{ \
	ASURA_CHUNK_FILL(__buffer,__var,__type); \
	ASURA_CHUNK_VERIFY(*(__buffer-sizeof(__type)),__type); \
}

/// Kim Randell - 21/03/2003 14:47:27 - combination of fill and array verify.
#define ASURA_CHUNK_ARRAY_FILLANDVERIFY(__buffer, __var, __type, __element_type, __elements) \
{ \
	ASURA_CHUNK_FILL(__buffer, __var, __type); \
	ASURA_CHUNK_ARRAY_VERIFY(*(__buffer-sizeof(__type)), __element_type, __elements); \
}

#define ASURA_CHUNK_BITFIELD_FILLANDVERIFY(__buffer, __var, __type) \
{ \
	ASURA_CHUNK_FILL(__buffer, __var, __type); \
	ASURA_CHUNK_BITFIELD_VERIFY(*(__buffer-sizeof(__type)), __type); \
}

#endif // Asura_Chunk_Macros.h
