//
// Asura_Chunk_PSP_Filter.cpp
//
// Created by Richard Rice - 11/01/2006
// 
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"
#include "ChunkSystem/Asura_Chunk_PSP_Filter.h"

#ifdef ASURA_USE_PSP_FILTER

#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "ChunkSystem/Asura_SaveBlock.h"
#include "FileAccess/Asura_FileAccess.h"
#include "Rendering/Asura_Colour.h"


#ifdef ASURA_TOOLS
#include "PSP/Tools_PSP_Filter.h"
#else
#include "Rendering/Asura_PSP_Filter.h"
#endif


#define ASURA_CHUNK_PSP_FILTER_VERSION 0

bool Asura_Chunk_PSP_Filter::Process (Asura_Chunk_Stream& xStream) const
{
	// Is this the correct version?

	if (Version > ASURA_CHUNK_PSP_FILTER_VERSION)
	{
		Asura_Assertf	(false, ("Unknown chunk version %d for psp filter",Version));
		return			(false);
	}

	
	u_int uEnabled;
	Asura_RGBA xColour;

	xStream >> uEnabled;
	xStream >> xColour.m_fR;
	xStream >> xColour.m_fG;
	xStream >> xColour.m_fB;
	xStream >> xColour.m_fA;

	//copy into tools , or psp settings as required
	#ifdef ASURA_TOOLS
	Tools_PSP_Filter::s_bEnabled = uEnabled ? true : false;
	Tools_PSP_Filter::s_xColour = xColour;
	#else 

	Asura_PSP_Filter::SetFilter(
		uEnabled ? true : false,
		Asura_Maths::FloatToInt(xColour.m_fR*255.0f),
		Asura_Maths::FloatToInt(xColour.m_fG*255.0f),
		Asura_Maths::FloatToInt(xColour.m_fB*255.0f),
		Asura_Maths::FloatToInt(xColour.m_fA*255.0f)
	);

	#endif

	return								(true);
}

bool Asura_Chunk_PSP_Filter::Process(Asura_Handle iFileHandle) const
{
	Asura_Chunk_File_Stream	xStream (iFileHandle);

	// Call the other process function...

	return	(Process(xStream));
}

#ifdef ASURA_TOOLS
void Asura_Chunk_PSP_Filter::Create (Asura_Chunk_Stream& xStream)
{

	// Write the data...

	xStream.WriteChunk_Start	(ASURA_CHUNK_PSP_FILTER, ASURA_CHUNK_PSP_FILTER_VERSION, 0);

	u_int uEnabled = Tools_PSP_Filter::s_bEnabled ? 1 : 0;

	xStream << uEnabled;
	xStream << Tools_PSP_Filter::s_xColour.m_fR;
	xStream << Tools_PSP_Filter::s_xColour.m_fG;
	xStream << Tools_PSP_Filter::s_xColour.m_fB;
	xStream << Tools_PSP_Filter::s_xColour.m_fA;

	xStream.WriteChunk_End	();
}

void Asura_Chunk_PSP_Filter::Create(Asura_SaveBlock* pxSaveBlock)
{
	Asura_Chunk_SaveBlock_Stream	xStream (pxSaveBlock);


	// Call the other version...

	Create	(xStream);
}
#endif // ASURA_TOOLS



#endif //ASURA_USE_PSP_FILTER
