//
// Asura_Chunk_Phonons.cpp
//
// Created by Kevin Lea - 12/06/2001 11:19:56
// 
// Copyright (C) 2001 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

#ifndef ASURA_USE_AUDIO_SYSTEM

#include "ChunkSystem/Asura_Chunk_Phonons.h"

#include "ChunkSystem/Asura_Chunk_Loaders.h"
#ifdef ASURA_TOOLS
#include "ChunkSystem/Asura_Chunk_Macros.h"
#include "ChunkSystem/Asura_Chunk_Stream.h"
#endif
#include "ChunkSystem/Asura_SaveBlock.h"
#include "Core/Asura_NameTree.h"     // for sound names chunk
#include "Core/Asura_PlatformBasics.h"
#include "Core/Memory/Asura_Mem.h"
#include "FileAccess/Asura_FileAccess.h"
#include "Sound/Asura_Phonon.h"
#include "Sound/Asura_Sound.h"        // for loading sounds
#if defined(ASURA_USE_STREAMING)
#include "Streaming/Asura_Streaming_System.h"
#endif	// defined(ASURA_USE_STREAMING)

#ifdef ASURA_TOOLS
#include "Load&Save/Tools_SaveAsura.h"
#endif

///////////////////////////////////////////////////////////////////////////////////////////////
/*---------------------**
** Asura_Chunk_Phonons **
**---------------------*/

bool Asura_Chunk_Phonons::Process(Asura_Handle fileHandle) const
{
	Asura_Assert(false, "Phonons chunk is obsolete");
	return false;
}

#ifdef ASURA_TOOLS
Asura_Phonon* Asura_Chunk_Phonons::ProcessSinglePhonon(const char* pcChunkBuffer,Asura_Guid uOldGuid,Asura_Guid uNewGuid, bool bMissionFile) const
{
	Asura_Assert(false, "Phonons chunk is obsolete");
	return 0;
}
#endif //ASURA_TOOLS

#ifdef ASURA_TOOLS
void Asura_Chunk_Phonons::Create(Asura_SaveBlock& saveBlock)
{
	Asura_Assert(false, "Phonons chunk is obsolete");
}

#endif // #ifdef ASURA_TOOLS
///////////////////////////////////////////////////////////////////////////////////////////////
/*----------------------------------**
** Asura_Chunk_Phonon_TemplateGuids **
**----------------------------------*/
#ifdef ASURA_TOOLS

#define ASURA_CHUNK_PHONON_TEMPLATEGUIDS_VERSION 0

bool Asura_Chunk_Phonon_TemplateGuids::Process(Asura_Handle iFileHandle) const
{
	Asura_Assert(false, "Phonon Template GUID chunk is obsolete");
	return false;
}


void Asura_Chunk_Phonon_TemplateGuids::Create(Asura_SaveBlock& xSaveBlock)
{
	Asura_Assert(false, "Phonon Template GUID chunk is obsolete");
}

#endif //ASURA_TOOLS
///////////////////////////////////////////////////////////////////////////////////////////////
/*------------------------**
** Asura_Chunk_SoundNames **
**------------------------*/

#ifdef ASURA_TOOLS

void Asura_Chunk_SoundNames::Create(const Asura_NameTree* nameTree,Asura_Sound_ID eMinID, Asura_Sound_ID eMaxID, Asura_SaveBlock* saveBlock)
{
	int number_of_sounds = nameTree->GetNumberOfNodes();
	int chunkSize = sizeof(Asura_Chunk_SoundNames);

	chunkSize += nameTree->GetSizeOfNamesInIntChunks();
	chunkSize += sizeof(int) * number_of_sounds;  // one int per sound for soundID

	Asura_Chunk_SoundNames* chunk = (Asura_Chunk_SoundNames*)saveBlock->GetAccessPointer(chunkSize);

	chunk->ID = ASURA_CHUNK_SOUNDNAMES;
	ASURA_CHUNK_VERIFY(chunk->ID, int);
	chunk->Size = chunkSize;
	ASURA_CHUNK_VERIFY(chunk->Size, int);
	chunk->Version = 1;
	ASURA_CHUNK_VERIFY(chunk->Version, int);
	chunk->Flags = 0;
	ASURA_CHUNK_VERIFY(chunk->Flags, int);

	chunk->NumberOfSounds = number_of_sounds;
	ASURA_CHUNK_VERIFY(chunk->NumberOfSounds, int);

	int* dest = (int*)(chunk+1);

	const Asura_String** nameList = new const Asura_String*[number_of_sounds];
	nameTree->MakeNameList(nameList);

	for (int i = 0; i < number_of_sounds; i++)
	{
		const Asura_String* name = nameList[i];
		*dest = Asura_Sound_System::GetSoundIDFromFileName(name->GetString(),eMinID,eMaxID);
		ASURA_CHUNK_VERIFY(*dest, int);
		dest++;
		dest = name->IntCopy(dest);
	}

	delete [] nameList;
}

bool Asura_Chunk_SoundNames::Process(Asura_Handle fileHandle) const
{
	for(int index = 0; index < NumberOfSounds; index++)
	{
		Asura_Sound_ID soundID;
		Asura_FileAccess::ReadData(fileHandle, (void*)&soundID, sizeof(int));
		const Asura_Char* filename = Asura_FileAccess::ReadString(fileHandle);
		Asura_Sound_System::LoadSound(filename, soundID);
	}
	return true;
}

const char* Asura_Chunk_SoundNames::GetFilename(const char* pcChunkBuffer,int iSoundID) const
{
	if(iSoundID == ASURA_SID_NO_SOUND) return 0;

	for(int i=0;i<NumberOfSounds;i++)
	{
		int iID;
		ASURA_CHUNK_EXTRACT(pcChunkBuffer,iID,int);
		if(iID == iSoundID)
		{
			//found the desired sound id.
			//the chunk buffer will be at the name
			return pcChunkBuffer;
		}
		//need to skip the name
		int iLength = strlen(pcChunkBuffer);
		pcChunkBuffer += (iLength+4)&~3;
	}
	//failed to find sound
	return 0;
}

#endif // #ifdef ASURA_TOOLS

#endif // #ifndef ASURA_USE_AUDIO_SYSTEM

//eof
