/////////////////////////////////////////////////////////////////////////////////////////
//
//	FILE:			Asura_Chunk_ShapeData.cpp
//
//	AUTHOR:			Andrew Weinkove - 07/04/03
//
// Copyright (C) 2003 Rebellion, All Rights Reserved.
//
/////////////////////////////////////////////////////////////////////////////////////////


#include "Core/Asura.h"
#include "ChunkSystem/Asura_Chunk_ShapeData.h"

#include "ChunkSystem/Asura_SaveBlock.h"
#include "ChunkSystem/Asura_Chunk_Macros.h"
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "FileAccess/Asura_FileAccess.h"
#include "Shapes/Asura_Shapes.h"
#include "Shapes/Asura_Shape_System.h"
#ifdef ASURA_TOOLS
#include "Load&Save/Tools_SaveAsura.h"
#include "Shapes/Tools_Mode_ShapeData.h"
#endif 

bool Asura_Chunk_ShapeData::Process( Asura_Handle iFileHandle ) const
{
	unsigned uFlags;
	Asura_Bounding_Box *pxBB = NULL;
	Asura_Hash_ID uHashID;
	ASURA_CHUNK_FILEEXTRACT(iFileHandle, uHashID, Asura_Hash_ID);
	ASURA_CHUNK_FILEEXTRACT(iFileHandle, uFlags, unsigned);
	if (uFlags & ASURA_SHAPE_DATA_FLAGS_HAS_CUSTOM_BB)
	{
		pxBB = new Asura_Bounding_Box;
		ASURA_CHUNK_FILEEXTRACT(iFileHandle, *pxBB, Asura_Bounding_Box);
	}
	else
	{
		pxBB = NULL;
	}
#ifdef ASURA_TOOLS
	Tools_Mode_ShapeData::SetData(uHashID, uFlags, pxBB);
#endif //ASURA_TOOLS
	Asura_Shape_System::SetShapeData(uHashID, uFlags, pxBB);
	if (pxBB)
	{
		delete pxBB;
	}
	return true;
}

#ifdef ASURA_TOOLS
void Asura_Chunk_ShapeData::Create( unsigned uFlags, Asura_Bounding_Box *pxBB, Asura_Hash_ID uHashID, Asura_SaveBlock& xSaveBlock )
{
	Asura_Chunk_SaveBlock_Stream xStream(&xSaveBlock);
	Create(uFlags, pxBB, uHashID, xStream);
}

void Asura_Chunk_ShapeData::Create( unsigned uFlags, Asura_Bounding_Box *pxBB, Asura_Hash_ID uHashID, Asura_Chunk_Stream& xStream)
{
	u_int uChunkVersion = 0;
	u_int uChunkFlags = 0;
	xStream.WriteChunk_Start(ASURA_CHUNK_SHAPEDATA, uChunkVersion, uChunkFlags);

	xStream << uHashID;
	xStream << uFlags;
	if (pxBB)
	{
		xStream << (*pxBB);
	}
	xStream.WriteChunk_End();
}
#endif //ASURA_TOOLS
