//
// Asura_Chunk_StartPosition.cpp
//
// Created by Kevin Lea - 12/06/2001 11:20:12
// 
// Copyright (C) 2001 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

#include "ChunkSystem/Asura_Chunk_StartPosition.h"

#include "ChunkSystem/Asura_Chunk_Loaders.h"
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "ChunkSystem/Asura_SaveBlock.h"
#include "Debugging/Asura_Assert.h"
#include "FileAccess/Asura_FileAccess.h"
#include "Mesh/Asura_Env.h"
#include "Mesh/Asura_Env_System.h"

#ifdef ASURA_TOOLS
#include "Load&Save/LevelExport/Tools_Level_Export.h"
#endif

#ifdef ASURA_TOOLS
Asura_Chunk_StartPosition::StartPositionFile Asura_Chunk_StartPosition::s_eStartPositionFile;
#endif

bool Asura_Chunk_StartPosition::Process(Asura_Handle fileHandle) const
{
	Asura_Environment* env = Asura_Environment_System::GetEnvironment();

	#ifdef ASURA_TOOLS
	//Find out which file these are being loaded from. RAR 24/1/2005
	const char* pszFilename = Asura_FileAccess::GetFilename(fileHandle);
	const char* pszExtension = strrchr(pszFilename,'.');
	if(pszExtension)
	{
		StartPositionFile eFile = SPF_NONE;
		if(!stricmp(pszExtension,".ent"))
		{
			eFile = SPF_ENTITY;
		}
		else if(!stricmp(pszExtension,".asr"))
		{
			eFile = SPF_ASURA;
		}

		if(env->NumberOfStartPositions)
		{
			if(eFile==SPF_ENTITY && s_eStartPositionFile == SPF_ASURA)
			{
				//Was loading start positions from asura file , but have now found some in the
				//entity file.
				//Delete the existing start positions and use the ones in the
				//entity file instead. 
				delete [] env->StartPositions;
				env->StartPositions = 0;
				env->NumberOfStartPositions = 0;
			}
			else if(eFile==SPF_ASURA && s_eStartPositionFile == SPF_ENTITY)
			{
				//Was loading start positions from the entity file , but have now found some
				//in the asura file.
				//We want to keep the ones from the entity file ,so just abort loading this chunk
				return false;
			}
		}

		if(eFile>s_eStartPositionFile)
		{
			s_eStartPositionFile = eFile;
		}

	}
	#endif //ASURA_TOOLS



	//get the index for this start position
	int index = 0;
	if(Version>=2)
	{
		Asura_FileAccess::ReadData(fileHandle,&index,sizeof(int));
	}
	if(index>=env->NumberOfStartPositions)
	{
		//start positions are written in reverse order , so should only need to enlard the array once
		Asura_Assert(env->NumberOfStartPositions==0,"Not getting the start positions in the expected order");
		//allocate start position array , and initialise
		env->NumberOfStartPositions = index+1;
		delete [] env->StartPositions;
		env->StartPositions = new Asura_Environment_StartPosition[env->NumberOfStartPositions];
		for(int i=0;i<env->NumberOfStartPositions;i++)
		{
			env->StartPositions[i].Orientation.Identity();
			env->StartPositions[i].Position.Zero();
			env->StartPositions[i].Position.y -= 0.2f;
			env->StartPositions[i].TeamNumber = 0;
		}
	}
	
	env->StartPositions[index].Orientation = Orientation;
	env->StartPositions[index].Position = Position;

	if(Version >= 3)
	{
		int iTemp;
		Asura_FileAccess::ReadData(fileHandle, &iTemp, sizeof(int));
		env->StartPositions[index].TeamNumber = iTemp;
	}

//	env->StartPosition.Position.y -= 0.1f;
//	StartPosition = Position;
//	StartOrient = Orientation;
//	StartPositionValid = true;
	return true;

}

#ifdef ASURA_TOOLS
void Asura_Chunk_StartPosition::Create( const Asura_Environment_StartPosition* pxStart,int iIndex, Asura_SaveBlock* pxSaveBlock)
{
	Asura_Chunk_SaveBlock_Stream xStream(pxSaveBlock);

	Create(pxStart, iIndex, xStream);
}

void Asura_Chunk_StartPosition::Create( const Asura_Environment_StartPosition* pxStart,int iIndex, Asura_Chunk_Stream& xStream)
{
	xStream.WriteChunk_Start(ASURA_CHUNK_STARTPOSITION, 3, 0);

	xStream << pxStart->Orientation;
	xStream << pxStart->Position - Tools_Level_Export::GetExportCentre();

	xStream << iIndex;
	xStream << pxStart->TeamNumber;

	xStream.WriteChunk_End();
}

#endif
