//
// Asura_Chunk_Texture_Streaming.cpp
//
// Created by KJFL - 14 March 2008
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

#if defined(ASURA_USE_TEXTURE_STREAMING)

// This file's header.
#include "ChunkSystem/Asura_Chunk_Texture_Streaming.h"

#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "Core/Asura_Texture_Streaming.h"

bool Asura_Chunk_Texture_Streaming::Process( Asura_Handle iFileHandle ) const
{
	Asura_Chunk_File_Stream xStream( iFileHandle );

	u_int uNumberOfTextures;
	
	xStream >> uNumberOfTextures;

	xStream >> Asura_Texture_Streaming_System::s_xTextureStreamingFilename;

	Asura_Texture_Streaming_System::s_uNumberOfStreamingTextures = uNumberOfTextures;

	for ( u_int u=0; u<uNumberOfTextures; ++u )
	{
		Asura_Streaming_Texture* const pxTexture = Asura_Texture_Streaming_System::GetStreamingTexture( u );

		Asura_Hash_ID uHashID;
		u_int uFilePosition;
		u_int uFileSize;
		u_int uChunkFlags;

		xStream >> uHashID;
		xStream >> uFilePosition;
		xStream >> uFileSize;
		if (Version > 0)
		{
			xStream >> uChunkFlags;
		}
		else
		{
			uChunkFlags = 0;
		}

		pxTexture->Initialise( uHashID, uFilePosition, uFileSize, uChunkFlags );
	}

	// tell the platform to open the streaming file or whatever else it needs to do
	Asura_Texture_Streaming_System::Platform_StartStreaming();

	return true;
}

#if defined( ASURA_TOOLS )

bool Asura_Chunk_Texture_Streaming::Create( class Asura_Chunk_Stream& xStream )
{
	xStream.WriteChunk_Start( ASURA_CHUNK_TEXTURE_STREAMING, CHUNK_VERSION, 0 );

	u_int uNumberOfTextures = Asura_Texture_Streaming_System::GetNumberOfStreamingTextures();

	//Don't want to save details for textures that were missing during the export process , so check
	//for ones that have been marked as having a zero file size.
	//RAR 10/10/2008
	u_int uNumberOfTexturesToSave = 0;
	for ( u_int u=0; u<uNumberOfTextures; ++u )
	{
		const Asura_Streaming_Texture* const pxTexture = Asura_Texture_Streaming_System::GetStreamingTexture( u );
		if(pxTexture->GetFileSize())
		{
			uNumberOfTexturesToSave++;
		}
	}


	xStream << uNumberOfTexturesToSave;

	xStream << Asura_Texture_Streaming_System::s_xTextureStreamingFilename;

	for ( u_int u=0; u<uNumberOfTextures; ++u )
	{
		const Asura_Streaming_Texture* const pxTexture = Asura_Texture_Streaming_System::GetStreamingTexture( u );

		if(pxTexture->GetFileSize())
		{
			xStream << pxTexture->GetHashID();
			xStream << pxTexture->GetPositionInFile();
			xStream << pxTexture->GetFileSize();
			xStream << pxTexture->GetChunkFlags();
		}
	}

	xStream.WriteChunk_End();

	return true;
}
#endif // ASURA_TOOLS

#endif // #if defined(ASURA_USE_TEXTURE_STREAMING)
