//////////////////////////////////////////////////////////////////////////////
//
// Asura_Huffman.h
//
// Copyright (c) 2005 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef ___ASURA_HUFFMAN_H___
#define ___ASURA_HUFFMAN_H___ 1

// I N C L U D E S ///////////////////////////////////////////////////////////

// Engine independent.
#include "ChunkSystem/Asura_Chunk_Loaders.h"

// M A C R O S ///////////////////////////////////////////////////////////////

#define ASURA_HUFFMAN_MAX_DEPTH  (11)
#define ASURA_HUFFMAN_TABLE_SIZE (1 << ASURA_HUFFMAN_MAX_DEPTH) // In words.

// C L A S S E S /////////////////////////////////////////////////////////////

class Asura_Chunk_Memory_Stream;

// The basic Huffman-encoded data header.
class Asura_Huffman_Package
{

public:

    u_int  m_uCompressedDataSize;
    u_int  m_uUncompressedDataSize;
    int    m_aiCodelengthCount[ASURA_HUFFMAN_MAX_DEPTH];
    u_char m_aucByteAssignment[256];

};

// An extended version of the above, used in ASR files.
class Asura_Huffman_CompressedFileHdr
{

public:

    char                  m_acASRFileID[ASURA_FILE_ID_NUM_CHARS];
    Asura_Huffman_Package m_xHuffmanPackage;

};

class Asura_Huffman_Encoder
{

public:

    static bool AllocateAndCompressForASR( const void* pvSrc, u_int uNumBytes,
                                           Asura_Huffman_CompressedFileHdr** ppxHdr, u_int* puTotalSize,
                                           bool bSwapEndianness = false );

	static u_int GetReqdBfrSize( u_int uNumBytes ) { return( sizeof(Asura_Huffman_Package) + uNumBytes ); }
    static bool CompressToBfr( const void* pvSrc, u_int uNumBytes, u_int* puTotalPackageSize, Asura_Chunk_Memory_Stream& xStream );

};

class Asura_Huffman_Decoder 
{

public:

    static u_int AllocateAndDecompressAllFromASR( const Asura_Huffman_CompressedFileHdr* pxCompFileHdr, void** ppvBfr );
    static u_int DecompressAll( const Asura_Huffman_Package* pxPackage, void* pvDest );

    Asura_Huffman_Decoder( void );
    ~Asura_Huffman_Decoder( void );

    void Begin( Asura_Handle iFileHandle );
    void Begin( const Asura_Huffman_Package* pxPackage );

    u_int GetDecompressedSize( void ) const { return m_xPackage.m_uUncompressedDataSize; }
    u_int Decompress( void* pvDest, u_int uNumBytes );

    void End( void );

private:

    void SetupCommonState( void );

#ifdef ASURA_PLATFORM_CAN_SUPPLY_FILE_DECOMPRESSOR_MEM
    void Platform_AllocTableBfr( void );
    void Platform_DeallocTableBfr( void );
#endif

    // These two are mutually exclusive.
    const u_int* m_puSrcData;
    Asura_Handle m_iFileHandle;

    Asura_Huffman_Package m_xPackage;
    int* m_piTable;

    // Decompression loop state.
    int m_iAvailable, m_iReserve, m_iWid;
    u_int m_uBits, m_uResBits;
    int m_iFill;
    u_int m_uBytesRead;
    bool m_bJumpIntoLoop;

	#ifdef ASURA_TOOLS
	bool m_bSwapEndian;
	#endif
        
};

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

#endif // #ifndef ___ASURA_HUFFMAN_H___
