//////////////////////////////////////////////////////////////////////////////
//
// Asura_Combined_List.h
//
// Copyright (c) 2004 Rebellion. All rights reserved.
//
// Description:
//
//     A "list" that contains both a hashed list - used for searching - and a
//     classified list - used for iterating through - giving the advantages
//     of each at the expense of the extra memory having both unavoidably
//     uses.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef ___ASURA_COMBINED_LIST_H___
#define ___ASURA_COMBINED_LIST_H___ 1

#include "Core/Asura_Classified_List.h"
#include "Core/Asura_HashedList.h"

template< typename T >
class Asura_Combined_List
{

public:

    Asura_Combined_List( u_int uLog2HashSize )
        : m_xHashedList( uLog2HashSize )
        , m_xClassifiedList()
    {
    }

    void AddToList( T* pxObject )
    {
        m_xHashedList.AddToList( pxObject );
        m_xClassifiedList.AddToList( pxObject );
    }
    void RemoveFromList( T* pxObject )
    {
        m_xHashedList.RemoveFromList( pxObject );
        m_xClassifiedList.RemoveFromList( pxObject );
    }

    // Search functions - use the hashed list
    inline T* FindObject( Asura_Guid uGuid, bool bIgnoreDestroyedEntities = true ) const
    {
        return m_xHashedList.FindObject( uGuid, bIgnoreDestroyedEntities );
    }
    inline T* FindEntity( Asura_Guid uGuid ) const
    {
        return m_xHashedList.FindEntity( uGuid );
    }

    // GetList, for creating an iterator, only allow access to classified list so only that can be iterated through
    inline Asura_Classified_List<T>* GetList( void )
    {
        return &m_xClassifiedList;
    }
    inline const Asura_Classified_List<T>* GetList( void ) const
    {
        return &m_xClassifiedList;
    }

    void DestroyAllObjects()
    {
        m_xClassifiedList.DestroyAllObjects();
    }

	int GetNumEntries(void) const	{ return(m_xHashedList.GetNumEntries()); }

private:

    Asura_Hashed_List<T>     m_xHashedList;
    Asura_Classified_List<T> m_xClassifiedList;

};

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

#endif // #ifndef ___ASURA_COMBINED_LIST_H___
