//
// Asura_Core.cpp
//
// Created by Kevin Lea - 12/06/2001 10:58:17
// 
// Copyright (C) 2001 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

#include "Core/Asura_Core.h"

#ifndef ASURA_DISABLE_AI_CODE
#include "AI/Asura_AI_Navigation.h"
#endif
#include "Animation/Asura_Animation.h"
#if defined(ASURA_SUPPORT_BODYPART_SYSTEM)
#include "Animation/Asura_Hierarchy_BodyPart_System.h"
#endif	// defined(ASURA_SUPPORT_BODYPART_SYSTEM)
#include "Animation/Asura_TextureAnimation.h"
#ifdef ASURA_USE_ANIMATION_IK_SOLVER
#include "Animation/IK/Asura_Animation_IK_Template_System.h"
#endif
#include "Blueprint/Asura_Blueprint_System.h"
#if defined(ASURA_SUPPORT_FLYCAM)
#include "Camera/Asura_FlyCam.h"
#endif	// defined(ASURA_SUPPORT_FLYCAM)
#include "ChunkSystem/Asura_BackgroundChunkProcessing_System.h"
#include "Core/Asura_Hash.h"
#include "Core/Asura_HashedLocalisedText.h"
#include "Core/Asura_LocalisedText.h"
#include "Core/Asura_MainLoop.h"
#include "Core/Asura_Material_System.h"
#include "Core/Asura_MaterialResponse_System.h"
#include "Core/Asura_Maths.h"
#include "Core/Asura_TextureManagement.h"
#include "Core/Asura_TiledTextureManagement.h"
#include "Core/Asura_Timers.h"
#include "Core/Asura_UserInput.h"

#ifdef ASURA_USE_AXON
#include "Core/Axon_System.h"
#endif
#ifdef ASURA_USE_TASK_SYSTEM
#include "Core/Threads/Asura_Task_System.h"
#endif
#if defined(ASURA_USE_MULTIPLE_HANDLERS)
#include "Core/Handlers/Asura_Handler_System.h"
#endif	//defined(ASURA_USE_MULTIPLE_HANDLERS)
#include "Core/Resources/Asura_Resource_System.h"
#include "Cutscene/Asura_Cutscene_System.h"
#include "Debugging/Asura_Assert.h"
#include "Debugging/Asura_DebugText.h"
#if defined( ASURA_USE_DEBUGOVERLAY )
#include "Debugging/Asura_DebugOverlay.h"
#endif // #if defined( ASURA_USE_DEBUGOVERLAY )
#if defined( ASURA_USE_PRIMITIVE_RENDER )
#include "Rendering/Asura_Primitive_Render.h"
#endif // #if defined( ASURA_USE_PRIMITIVE_RENDER )
#include "Debugging/Asura_Debug_Timers.h"
#ifdef ASURA_USE_PROFILING_SYSTEM
#include "Debugging/Asura_Profiling_System.h"
#endif
#include "Entity/Asura_Entity.h"
#include "Entity/Asura_StaticMessage_System.h"

#ifdef ASURA_USE_CONTAINER_SYSTEM
#include "Entity/Container/Asura_Container_System.h"
#endif //#ifdef ASURA_USE_CONTAINER_SYSTEM
#if defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)
#include "FacialAnimation/Asura_FacialAnimation_System.h"
#endif	//defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)
#include "FileAccess/Asura_FileAccess.h"

#ifdef ASURA_USE_VIDEO_TO_TEXTURE_SYSTEM
#include "FMV/Asura_VideoToTexture_System.h"
#endif

#if defined( ASURA_USE_GUIMENU_PROFILING )
#include "GUIMenu/Asura_Profiling_GUIMenu.h"
#endif

#include "GUIMenu/Asura_GUIMenu_Element_FMV.h"

#ifdef ASURA_SUPPORT_COMMAND_CONSOLE
#include "Interface/Asura_CommandConsole.h"
#endif
#include "Interface/Asura_Cursor.h"
#ifdef ASURA_TOOLS
#include "Interface/Asura_Interface.h"
#endif

#include "Light/Asura_GI_Solution.h"
#include "Light/Asura_Light.h"

#include "Mesh/Asura_CollisionMesh.h"
#include "Mesh/Asura_Env.h"
#include "Mesh/Asura_Env_System.h"
#ifdef ASURA_TOOLS
#include "Mesh/Asura_Mesh.h"
#include "Mesh/Asura_Mesh_System.h"
#endif
#ifdef ASURA_SUPPORT_DOWNLOADABLE_CONTENT
#include "Network/Asura_DLC_System.h"
#endif //#ifdef ASURA_SUPPORT_DOWNLOADABLE_CONTENT
#ifdef ASURA_SUPPORT_EXTERNAL_NETWORK
#include "Network/Asura_NetLinkManager.h"
#endif 
#ifdef ASURA_SUPPORT_NETMSG_SYSTEM
#include "Network/Asura_NetMsg.h"
#endif
#ifdef ASURA_ENABLE_QOS_SYSTEM
#include "Network/Asura_Qos_System.h"
#endif //ASURA_ENABLE_QOS_SYSTEM
#if defined(ASURA_SUPPORT_LOBBY)
#include "Network/Online/Asura_Lobby.h"
#endif	//defined(ASURA_SUPPORT_LOBBY)
#include "Rendering/Asura_Render.h"
#ifdef ASURA_TOOLS
#include "Rendering/Asura_Text_Output.h"
#endif
#ifdef ASURA_TOOLS
#include "Rendering/Asura_Surface.h"
#endif
#include "Rendering/Asura_Module.h"
#include "Rendering/Asura_Font_System.h"
#include "Rewards/Asura_Rewards_System.h"
#if defined(ASURA_USE_INDEPENDENT_SHADOW_SYSTEM)
#include "Shadows/Asura_Shadow_System.h"
#endif	// defined(ASURA_USE_INDEPENDENT_SHADOW_SYSTEM)
#include "Shapes/Asura_Shape_System.h"
#include "Sound/Asura_Audio_Systems_Manager.h"
#include "Sound/Asura_Sound.h"
#include "Sound/Asura_ModuleReverb_System.h"
#include "SpecialFX/Asura_SpecialFX.h"
#include "SpecialFX/Asura_StaticCorona_System.h"

#include "SpecialFX/Asura_Fragment_System.h"
#include "SpecialFX/Asura_Skybox.h"
#include "SpecialFX/PFX/Asura_PFX_System.h"

#ifdef ASURA_USE_BILLBOARD_PLANTS
#include "SpecialFX/Asura_BillboardPlant_TemplateStore.h"
#include "SpecialFX/Asura_BillboardPlant_System.h"
#endif //ASURA_USE_BILLBOARD_PLANTS

#ifdef ASURA_USE_COLLISIONFX
#include "SpecialFX/Asura_CollisionFX_System.h"
#endif //ASURA_USE_COLLISIONFX

#if defined(ASURA_USE_STREAMING)
#include "Streaming/Asura_Streaming_System.h"
#endif	// defined(ASURA_USE_STREAMING)

#if !defined(ASURA_DONT_USE_TELEPORTERS)
#include "Teleport/Asura_Teleport_System.h"
#endif	// !defined(ASURA_DONT_USE_TELEPORTERS)

#ifdef ASURA_TOOLS
#include "Entity/Tools_SecondaryEntityFile_System.h"
#include "FacialAnimation/Tools_FacialAnimation_System.h"
#include "Lighting/Tools_Environment_LOS.h"
#include "Lighting/Tools_PhotonMapping_System.h"
#include "Lighting/Tools_LightSets.h"
#include "Load&Save/Tools_AsrFileManager.h"
#include "Load&Save/Tools_LoadedFileList.h"
#include "Misc/Tools_Adjacency.h"
#include "Misc/Tools_MessageLog.h"
#include "Shapes/Tools_Shapes.h"
#ifdef ASURA_USE_STREAMING
#include "Streaming/Tools_StreamingModuleNames.h"
#endif // ASURA_USE_STREAMING
#endif //ASURA_TOOLS

#ifdef ASURA_USE_DEBUG_TEXT_MANAGER
#include "Debugging/Asura_DebugText_Manager.h"
#endif

#ifdef ASURA_USE_ARTICULATION_SYSTEM
#include "LipSync/Asura_Lipsync_Manager.h"
#endif

#include "Physics/Asura_Physics.h"


#ifdef ASURA_USE_BOTANICALS
#include "Botanicals/Asura_Botanicals_System.h"
#endif // #ifdef ASURA_USE_BOTANICALS

#ifdef ASURA_USE_CARPET
#ifdef ASURA_TOOLS
#include "Carpet/Asura_Carpet_StyleManager.h"
#endif // #ifdef ASURA_TOOLS
#include "Carpet/Asura_Carpet_MeshManager.h"
#endif // #ifdef ASURA_USE_CARPET

#ifdef ASURA_USE_FSFX
#include "FSFX/Asura_FSFX_System.h"
#ifdef ASURA_TOOLS
#include "FSFX/Tools_FSFX_System.h"
#endif
#endif // ASURA_USE_FSFX

#ifdef ASURA_USE_ATTRACTORS
#include "Attractors/Asura_Attractor_System.h"
#ifdef ASURA_TOOLS
#include "Attractors/Tools_Attractor_System.h"
#endif
#endif // ASURA_USE_ATTRACTORS

#if defined(ASURA_USE_VISIBILITY_SYSTEM) || defined( ASURA_USE_NEW_VISIBILITY_SYSTEM )
#include "Visibility/Asura_Visibility_System.h"
#endif	// defined(ASURA_USE_VISIBILITY_SYSTEM)

// stdlib
#ifdef ASURA_ASSERT
#include <wchar.h>
#endif // #ifdef ASURA_ASSERT

#ifdef ASURA_USE_GUI_MENU_SYSTEM
//#include "GUIMenu/Asura_GUIMenu_Message.h"
#include "GUIMenu/Asura_GUIMenu_System.h"
#endif


#ifdef ASURA_TOOLS
#include "HelperObject/Tools_HelperObject_System.h"
#endif

#ifdef ASURA_TARGET_SM3
#include "Projectors/Asura_Projector_System.h"
#include "Rendering/Asura_AmbientOcclusion.h"
#include "Rendering/Asura_DeferredShading.h"
#include "SpecialFX/Asura_LightShaft_System.h"
#endif

#ifdef ASURA_USE_CONTROLMAP_MANAGER
#include "Input/Asura_ControlManager.h"
#endif

#ifdef ASURA_TOOLS
#include "Tools_System.h"
#include "Misc/Tools_Render_Options.h"
#endif

#ifdef ASURA_USE_STATIC_DECAL_SYSTEM
#ifdef ASURA_TOOLS
#include "Decals/Tools_StaticDecal_System.h"
#endif
#endif

#if defined(ASURA_TARGET_SM3) && defined(ASURA_TOOLS)
#include "Rendering/Tools_RenderModules.h"
#endif

// S T A T I C   F U N C T I O N S ///////////////////////////////////////////
static void DoCompilerSettingsTests()
{
    // (MH - 19.01.2006) sanity check on type sizes assumed by Asura.
	// some platforms may need compiler settings adjusting.
    Asura_Assert( sizeof(Asura_UnicodeChar) == 2, "Incorrect size of wide characters - for GCC add -fshort-wchar to your compiler options." );
	Asura_Assert( wcslen(L"1234") == 4, "incorrect version of wcslen" );

	Asura_Assert( sizeof(char) == 1, "basic type size is incompatible with Asura");
	Asura_Assert( sizeof(u_char) == 1, "basic type size is incompatible with Asura");
	Asura_Assert( sizeof(short) == 2, "basic type size is incompatible with Asura");
	Asura_Assert( sizeof(u_short) == 2, "basic type size is incompatible with Asura");
	Asura_Assert( sizeof(int) == 4, "basic type size is incompatible with Asura");
	Asura_Assert( sizeof(u_int) == 4, "basic type size is incompatible with Asura");
	Asura_Assert( sizeof(float) == 4, "basic type size is incompatible with Asura");
}


// C L A S S   I M P L E M E N T A T I O N ///////////////////////////////////
void Asura_Core::EarlyInitialise(void)
{
	ASURA_CPU_PROFILE_SCOPED_TIMER( "Asura_Core::EarlyInitialise");

	// Initialise the resource-system - all other systems MAY use this...
	Asura_Resource_System::Initialise();

#ifdef ASURA_USE_TASK_SYSTEM
	Asura_Task_System::Initialise();
#endif

#ifdef ASURA_SUPPORT_COMMAND_CONSOLE
    Asura_CommandConsole::EarlyInitialise();
#ifdef ASURA_ASSERT
	Asura_CommandConsole::AddCmd( L"Asserts.Enable", AsuraAssert_ConsoleCmd_EnableAsserts,	Asura_CommandConsole::REMOVE_AT_SHUTDOWN);
#endif
#endif

	// After the command console so debug overlay can add commands to the console
#if defined( ASURA_USE_DEBUGOVERLAY )
	Asura_DebugOverlay::Initialise();
#endif // #if defined( ASURA_USE_DEBUGOVERLAY )

	// After the command console so debug overlay can add commands to the console
#if defined( ASURA_USE_PRIMITIVE_RENDER )
	Asura_Primitive_Render::Initialise();
#endif // #if defined( ASURA_USE_PRIMITIVE_RENDER )

	Asura_Timers::Initialise();

#ifdef ASURA_USE_PROFILING_SYSTEM
	Asura_Profiling_System::Initialise();
#endif
	
	Asura_FileAccess::Initialise();

    Asura_HashedFilename_System::Initialise();

#ifdef ASURA_TOOLS
	Tools_LoadedFileList::Initialise();
	Tools_Environment_LOS::Initialise();
#endif

	Asura_HashedLocalisedText::Initialise();

	// ARC - 12 September 2006 15:40:04 - Universal hash look-up
#if defined(ASURA_UNIVERSAL_HASH_LOOKUP)
	Asura_Universal_HashLookup::Initialise();
#endif // ASURA_UNIVERSAL_HASH_LOOKUP

	PlatformInitialise();
    PlatformProject_EarlyInitialise();

#if defined(ASURA_USE_BACKGROUND_CHUNK_PROCESSING)
	Asura_BackgroundChunkProcessing_System::Initialise();
#endif	// defined(ASURA_USE_BACKGROUND_CHUNK_PROCESSING)
#if defined(ASURA_USE_STREAMING)
	Asura_Streaming_System::Initialise();
#endif	// defined(ASURA_USE_STREAMING)
	Asura_Render::Initialise();
	Asura_Camera::SetToDefaults();
	
    Asura_Random::Initialise();
	
#ifdef ASURA_USE_CONTROLMAP_MANAGER
	Asura_ControlManager::Initialise();
#endif

	Asura_Blueprint_System::Initialise();

	Asura_UserInput::Initialise();

	Asura_Environment_System::Initialise();
	Asura_Module_System::Initialise();
#ifdef ASURA_TOOLS
	Asura_Mesh_System::Initialise();
	Asura_Surface_Mesh_System::Initialise();
#endif

	Asura_Material_System::Initialise();
	Asura_Texture_Management::Initialise();
	Asura_TiledTextureManagement::Initialise();
	Asura_Font_System::Initialise();

#if defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM
	Asura_MaterialResponse_System::Initialise();
#endif

#if defined( ASURA_USE_VISIBILITY_SYSTEM ) || defined( ASURA_USE_NEW_VISIBILITY_SYSTEM )
	Asura_Visibility_System::Initialise();
#endif	// defined( ASURA_USE_VISIBILITY_SYSTEM )

#if defined ASURA_TOOLS
	Tools_System::Allocate();
	Tools_MessageLog::Initialise();
#endif

#ifdef ASURA_TARGET_SM3
	Asura_DeferredShading::Initialise();
	Asura_Projector_System::Initialise();
	Asura_LightShaft_System::Initialise();
  	Asura_AmbientOcclusion::Initialise();
#endif

#if defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)
	Asura_FacialAnimation_System::Initialise();
#endif	//defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)

#ifdef ASURA_SUPPORT_DOWNLOADABLE_CONTENT
	Asura_DLC_System::Initialise();
#endif //#ifdef ASURA_SUPPORT_DOWNLOADABLE_CONTENT
}

void Asura_Core::LateInitialise()
{
	ASURA_CPU_PROFILE_SCOPED_TIMER( "Asura_Core::LateInitialise");
	
	PlatformProject_LateInitialise();

#ifdef ASURA_SUPPORT_COMMAND_CONSOLE
    Asura_CommandConsole::LateInitialise();
#endif

#if defined(ASURA_SUPPORT_FLYCAM)
	Asura_FlyCam::Initialise();
#endif	// defined(ASURA_SUPPORT_FLYCAM)

#ifdef ASURA_USE_DEBUG_TEXT
	// This causes Tools/font.dds to be loaded.
	Asura_DebugText::Initialise();
#endif //ASURA_USE_DEBUG_TEXT

#ifdef ASURA_USE_GUI_MENU_SYSTEM
	Asura_GUIMenu_System::Initialise();
#endif 

	Asura_TextureAnimation_System::Initialise();
	
	Asura_Animation_System::Initialise();
#if defined(ASURA_SUPPORT_BODYPART_SYSTEM)
	Asura_Hierarchy_BodyPart_System::Initialise();
#endif	// defined(ASURA_SUPPORT_BODYPART_SYSTEM)
 
	Asura_GI_Solution::Initialise();

	Asura_Lighting_System::Initialise(); // soon to be deprecated

#ifdef ASURA_USE_ARTICULATION_SYSTEM
	Asura_LipSync_Manager::Initialise();		// needs to go before sound system
#endif

#ifdef ASURA_USE_AUDIO_SYSTEM
	Asura_Audio_Systems_Manager::Initialise();
#else
	Asura_Sound_System::Initialise();
#endif

#ifndef ASURA_DISABLE_AI_CODE
    Asura_AI_Navigation_System::Initialise();
#endif

#ifdef ASURA_USE_AXON
    Axon_System::Initialise();
#endif

	Asura_SpecialFX_System::Initialise();
	Asura_StaticCorona_System::Initialise();

#ifdef ASURA_USE_DEBUG_TEXT_MANAGER
	Asura_DebugText_Manager::Initialise();
#endif

#ifdef ASURA_USE_CONTAINER_SYSTEM
	Asura_Container_System::Initialise();
#endif // #ifdef ASURA_USE_CONTAINER_SYSTEM

	// Must be before the entity system
	Asura_ClassifiedList_System::Initialise();

	Asura_Entity_System::Initialise();
	Asura_StaticMessage_System::Initialise();

	Asura_Cutscene_System::Initialise();

#ifdef ASURA_TOOLS
	Asura_Text_Output::Initialise();
#endif

	Asura_Shape_System::Initialise();

	Asura_Fragment_System::Initialise();
#ifdef ASURA_USE_BILLBOARD_PLANTS
	Asura_BillboardPlant_TemplateStore::Initialise();
	Asura_BillboardPlant_System::Initialise();
#endif //ASURA_USE_BILLBOARD_PLANTS

#ifdef ASURA_USE_BOTANICALS
	Asura_Botanicals_System::Initialise();	
#endif //ASURA_USE_BOTANICALS

#ifdef ASURA_USE_CARPET
#ifdef ASURA_TOOLS
	Asura_Carpet_StyleManager::Initialise();
#endif // #ifdef ASURA_TOOLS
	Asura_Carpet_MeshManager::Initialise();
#endif // ASURA_USE_CARPET


#ifdef ASURA_USE_STATIC_DECAL_SYSTEM
#ifdef ASURA_TOOLS
	Tools_StaticDecal_System::Initialise();
#endif // #ifdef ASURA_TOOLS
#endif

#ifdef ASURA_USE_FSFX
	Asura_FSFX_System::Initialise();
#endif // ASURA_USE_FSFX

#ifdef ASURA_USE_ATTRACTORS
	Asura_Attractor_System::Initialise();
#ifdef ASURA_TOOLS
	Tools_Attractor_System::Initialise();
#endif
#endif // ASURA_USE_ATTRACTORS

	Asura_ModuleReverb_System::Initialise();

	// KJFL - 2004-07-01 - The Photon Mapping (GI) system needs to exist outside the Tools
	// Initialise/Shutdown cycle, since 'quit to game' functionality needs the lighting
	// to be available *after* the Tools_System has been shutdown.
#ifdef ASURA_TOOLS
	Tools_Adjacency::Initialise();
	Tools_PhotonMapping_System::Initialise();
	Tools_LightSets::Initialise();
	Tools_SecondaryEntityFile_System::Initialise();
#endif

#if defined(ASURA_TARGET_SM3) && defined(ASURA_TOOLS)
	Tools_RenderModule_System::Initialise();
#endif

	Asura_Physics_System::Initialise();

#ifdef ASURA_USE_COLLISIONFX
	Asura_CollisionFX_System::Initialise();
#endif //ASURA_USE_COLLISIONFX

#if defined(ASURA_TOOLS) && defined(ASURA_USE_STREAMING)
	Tools_StreamingModuleNames::Initialise();
#endif

#if defined(ASURA_USE_INDEPENDENT_SHADOW_SYSTEM)
	Asura_Shadow_System::Initialise();
#endif	// defined(ASURA_USE_INDEPENDENT_SHADOW_SYSTEM)

#ifdef ASURA_USE_CURSOR
	Asura_Cursor::Initialise();
#endif
#ifdef ASURA_TOOLS
	Asura_Interface::Initialise();
	Tools_HelperObject_System::Initialise();
#endif

	DoCompilerSettingsTests();

#if !defined(ASURA_DONT_USE_TELEPORTERS)
	Asura_Teleport_System::Initialise();
#endif	// !defined(ASURA_DONT_USE_TELEPORTERS)

#if defined(ASURA_USE_MULTIPLE_HANDLERS)
	Asura_Handler_System::Initialise();
#endif	//defined(ASURA_USE_MULTIPLE_HANDLERS)

#ifdef ASURA_USE_ANIMATION_IK_SOLVER
	Asura_Animation_IK_Template_System::Initialise();
#endif

#ifdef ASURA_SUPPORT_EXTERNAL_NETWORK
	Asura_NetLinkManager::Initialise();
#endif 

#if defined(ASURA_SUPPORT_LOBBY)
	Asura_Lobby_System::AddConsoleCmdsAndVars();
#endif	//defined(ASURA_SUPPORT_LOBBY)

#ifdef ASURA_USE_REWARDS
	Asura_Rewards_System::Initialise();
#endif

#ifdef ASURA_ENABLE_QOS_SYSTEM
	Asura_Qos_System::Initialise();
#endif //ASURA_ENABLE_QOS_SYSTEM

#ifdef ASURA_USE_MAINLOOP
	Asura_MainLoop::Initialise();
#endif //ASURA_USE_MAINLOOP
}

void Asura_Core::Shutdown(void)
{
	ASURA_CPU_PROFILE_SCOPED_TIMER( "Asura_Core::Shutdown");
	
	// With deferred rendering on PS2, Asura_Render needs to wait for drawing to finish before
	// anything in the drawlist gets deleted.
	Asura_Render::EarlyShutdown();

#ifdef ASURA_USE_MAINLOOP
	Asura_MainLoop::Shutdown();
#endif //ASURA_USE_MAINLOOP

#ifdef ASURA_ENABLE_QOS_SYSTEM
	Asura_Qos_System::Shutdown();
#endif //ASURA_ENABLE_QOS_SYSTEM

#ifdef ASURA_SUPPORT_EXTERNAL_NETWORK
	Asura_NetLinkManager::Shutdown();
#endif 

#if defined(ASURA_USE_MULTIPLE_HANDLERS)
	Asura_Handler_System::Shutdown();
#endif	//defined(ASURA_USE_MULTIPLE_HANDLERS)

#if !defined(ASURA_DONT_USE_TELEPORTERS)
	Asura_Teleport_System::Shutdown();
#endif	// !defined(ASURA_DONT_USE_TELEPORTERS)

#ifdef ASURA_USE_VIDEO_TO_TEXTURE_SYSTEM
	Asura_VideoToTexture_System::Shutdown();
#endif

#ifdef ASURA_TOOLS
	Tools_System::Deallocate();
	Tools_MessageLog::Shutdown();
	Asura_Interface::Shutdown();
	Tools_HelperObject_System::Shutdown();
#if defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)
	Tools_FacialAnimation_System::Shutdown();
#endif	//defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)
#endif
#ifdef ASURA_USE_CURSOR
	Asura_Cursor::Shutdown();
#endif

#ifdef ASURA_TARGET_SM3
	Asura_LightShaft_System::Shutdown();
	Asura_Projector_System::Shutdown();
	Asura_DeferredShading::Shutdown();
  	Asura_AmbientOcclusion::Shutdown();
#endif

#if defined(ASURA_USE_INDEPENDENT_SHADOW_SYSTEM)
	Asura_Shadow_System::Shutdown();
#endif	// defined(ASURA_USE_INDEPENDENT_SHADOW_SYSTEM)

#if defined(ASURA_TOOLS) && defined(ASURA_USE_STREAMING)
	Tools_StreamingModuleNames::Shutdown();
#endif

#if defined(ASURA_USE_STREAMING)
	Asura_Streaming_System::Shutdown();
#endif	// defined(ASURA_USE_STREAMING)
#if defined(ASURA_USE_BACKGROUND_CHUNK_PROCESSING)
	Asura_BackgroundChunkProcessing_System::Shutdown();
#endif	// defined(ASURA_USE_BACKGROUND_CHUNK_PROCESSING)

#ifdef ASURA_USE_CONTROLMAP_MANAGER
	Asura_ControlManager::Shutdown();
#endif


#ifdef ASURA_USE_GUI_MENU_SYSTEM
	Asura_GUIMenu_System::Shutdown();
#endif 

	Asura_ModuleReverb_System::Shutdown();

	Asura_HashedLocalisedText::Shutdown();

	Asura_Fragment_System::Shutdown();

#ifdef ASURA_TOOLS
	Tools_ShapeTable::Shutdown();
#endif //ASURA_TOOLS

	Asura_Shape_System::Shutdown();

    Asura_Timers::Shutdown();

	Asura_Cutscene_System::Shutdown();
	
	Asura_StaticCorona_System::Shutdown();

	Asura_Entity_System::Shutdown();    // shutdown before Asura_SpecialFX_System, cos it removes its particles

	// Must be after the entity shutdown
	Asura_ClassifiedList_System::Shutdown();

#ifdef ASURA_USE_CONTAINER_SYSTEM
	// Must be after the entity shutdown
	Asura_Container_System::Shutdown();
#endif // #ifdef ASURA_USE_CONTAINER_SYSTEM

#ifdef ASURA_USE_COLLISIONFX
	Asura_CollisionFX_System::Shutdown();
#endif //ASURA_USE_COLLISIONFX

	Asura_SpecialFX_System::Shutdown();

#ifdef ASURA_SUPPORT_DOWNLOADABLE_CONTENT
	Asura_DLC_System::Shutdown();
#endif //#ifdef ASURA_SUPPORT_DOWNLOADABLE_CONTENT

#if defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)
	Asura_FacialAnimation_System::Shutdown();
#endif	//defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)

#ifdef ASURA_USE_AXON
    Axon_System::Shutdown();
#endif

    Asura_Blueprint_System::Shutdown();

#ifndef ASURA_DISABLE_AI_CODE
	Asura_AI_Navigation_System::Shutdown();
#endif
    Asura_StaticMessage_System::Shutdown();

	Asura_GI_Solution::Shutdown();
	Asura_Lighting_System::Shutdown(); // soon to be deprecated

#ifdef ASURA_USE_AUDIO_SYSTEM
	Asura_Audio_Systems_Manager::Shutdown();
#else
	Asura_Sound_System::Shutdown();
#endif

#ifdef ASURA_USE_ARTICULATION_SYSTEM
	Asura_LipSync_Manager::Shutdown();
#endif

#if defined(ASURA_SUPPORT_BODYPART_SYSTEM)
	Asura_Hierarchy_BodyPart_System::Shutdown();
#endif	// defined(ASURA_SUPPORT_BODYPART_SYSTEM)
	Asura_Animation_System::Shutdown();

	Asura_TextureAnimation_System::Shutdown();
	Asura_TiledTextureManagement::Shutdown();

#ifdef ASURA_SUPPORT_COMMAND_CONSOLE
    Asura_CommandConsole::Shutdown();
#endif

#if defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM
	Asura_MaterialResponse_System::Shutdown();
#endif

	Asura_Texture_Management::Shutdown();
	Asura_Material_System::Shutdown();

	Asura_UserInput::Shutdown();

#ifdef ASURA_TOOLS
	Asura_Surface_Mesh_System::Shutdown();
	Asura_Mesh_System::Shutdown();
#endif
    Asura_Environment_System::Shutdown();
	Asura_Module_System::Shutdown();
	
	Asura_Font_System::Shutdown();	
	Asura_LocalisedText::Shutdown();

#ifdef ASURA_USE_DEBUG_TEXT_MANAGER
	Asura_DebugText_Manager::Shutdown();
#endif

// ARC - 12 September 2006 15:40:53 - universal hash lookup
#if defined(ASURA_UNIVERSAL_HASH_LOOKUP)
	Asura_Universal_HashLookup::Shutdown();
#endif // ASURA_UNIVERSAL_HASH_LOOKUP

	Asura_HashedFilename_System::Shutdown();

	Asura_FileAccess::Shutdown();
#ifdef ASURA_USE_DEBUG_TEXT
	Asura_DebugText::Shutdown();
#endif //ASURA_USE_DEBUG_TEXT

#if defined(ASURA_SUPPORT_FLYCAM)
	Asura_FlyCam::Shutdown();
#endif	// defined(ASURA_SUPPORT_FLYCAM)

#ifdef ASURA_USE_BILLBOARD_PLANTS
	Asura_BillboardPlant_TemplateStore::Shutdown();
	Asura_BillboardPlant_System::Shutdown();
#endif

#ifdef ASURA_TOOLS
#ifdef ASURA_USE_STATIC_DECAL_SYSTEM
	Tools_StaticDecal_System::Shutdown();
#endif
#endif

#ifdef ASURA_USE_BOTANICALS
	Asura_Botanicals_System::Shutdown();	
#endif //ASURA_USE_BOTANICALS

#ifdef ASURA_USE_CARPET
#ifdef ASURA_TOOLS
	Asura_Carpet_StyleManager::Shutdown();
#endif // #ifdef ASURA_TOOLS
	Asura_Carpet_MeshManager::Shutdown();
#endif

#ifdef ASURA_USE_FSFX
	Asura_FSFX_System::Shutdown();
#endif // ASURA_USE_FSFX

#ifdef ASURA_USE_ATTRACTORS
	Asura_Attractor_System::Shutdown();
#ifdef ASURA_TOOLS
	Tools_Attractor_System::Shutdown();
#endif
#endif // ASURA_USE_ATTRACTORS

#ifdef ASURA_TOOLS
	Tools_LoadedFileList::Shutdown();
	Tools_AsrFileManager::Shutdown();
#endif

#if defined(ASURA_TARGET_SM3) && defined(ASURA_TOOLS)
	Tools_RenderModule_System::Shutdown();
#endif

	// this Shutdown call should at least be after over rendering related systems
	// (e.g. TextureManagement) are Shutdown
	Asura_Render::LateShutdown();

	PlatformShutdown();

	// KJFL - 2004-07-01 - The Photon Mapping (GI) system needs to exist outside the Tools
	// Initialise/Shutdown cycle, since 'quit to game' functionality needs the lighting
	// to be available *after* the Tools_System has been shutdown.
#ifdef ASURA_TOOLS
	Tools_LightSets::Shutdown();
	Tools_PhotonMapping_System::Shutdown();
	Tools_Adjacency::Shutdown();
	Tools_SecondaryEntityFile_System::Shutdown();
	Tools_Environment_LOS::Shutdown();
	Tools_Render_Options::FinalShutdown();
#endif

	Asura_Physics_System::Shutdown();

#if defined( ASURA_USE_VISIBILITY_SYSTEM ) || defined( ASURA_USE_NEW_VISIBILITY_SYSTEM )
	Asura_Visibility_System::Shutdown();
#endif	// defined( ASURA_USE_VISIBILITY_SYSTEM )

#if defined( ASURA_USE_DEBUGOVERLAY )
	Asura_DebugOverlay::Shutdown();
#endif // #if defined( ASURA_USE_DEBUGOVERLAY )

#if defined( ASURA_USE_PRIMITIVE_RENDER )
	Asura_Primitive_Render::Shutdown();
#endif // #if defined( ASURA_USE_PRIMITIVE_RENDER )

#ifdef ASURA_USE_TASK_SYSTEM
	Asura_Task_System::Shutdown();
#endif

#ifdef ASURA_USE_ANIMATION_IK_SOLVER
	Asura_Animation_IK_Template_System::Shutdown();
#endif

	// Last thing is to shutdown the resource-system - all other systems MAY use this...
	Asura_Resource_System::Shutdown();

#ifdef ASURA_USE_REWARDS
	Asura_Rewards_System::Shutdown();
#endif
}

void Asura_Core::Reset(void)
{
	ASURA_CPU_PROFILE_SCOPED_TIMER( "Asura_Core::Reset");
	
// (PSJE - 08.11.2002) Moved Asura_Render::Reset ahead of other resets.
// With deferred rendering, Asura_Render::Reset needs to wait for drawing to finish before
// anything in the drawlist get deleted.
	Asura_Render::Reset();

#ifdef ASURA_USE_TASK_SYSTEM
	Asura_Task_System::Reset();
#endif

#ifdef ASURA_TOOLS
	Tools_LoadedFileList::Reset();
	Tools_AsrFileManager::Reset();
#endif


#ifdef ASURA_SUPPORT_COMMAND_CONSOLE
    Asura_CommandConsole::Reset();
#endif

#ifdef ASURA_USE_DEBUG_TEXT
	Asura_DebugText::Reset();
#endif //ASURA_USE_DEBUG_TEXT

	PlatformReset();

    // (MP - 04.06.2003) Entity system should be reset before dynamics since
    // it relies upon it.
	Asura_Entity_System::Reset();
#if defined(ASURA_TOOLS)
	Asura_Entity_System::ResetNextToolsGuid();
#endif	// defined(ASURA_TOOLS)

	// Shutdown any FMV's in the GUIMenu system before shutting down the VideoToTexture system
	Asura_GUIMenu_FMV_System::Reset();

#ifdef ASURA_USE_VIDEO_TO_TEXTURE_SYSTEM
	Asura_VideoToTexture_System::Reset();
#endif

	// Removed as doesn't do anything useful if all entities deleted on reset, and kills things horribly if they aren't
//	Asura_ClassifiedList_System::Reset();

#if defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)
	Asura_FacialAnimation_System::Reset();
#endif	//defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)

#ifdef ASURA_USE_CONTAINER_SYSTEM
	// Must be after entity reset.
	Asura_Container_System::Reset();
#endif // #ifdef ASURA_USE_CONTAINER_SYSTEM

#ifdef ASURA_SUPPORT_NETMSG_SYSTEM
	Asura_NetMsg_System::Reset();
#endif // #ifdef ASURA_SUPPORT_NETMSG_SYSTEM

#ifndef ASURA_DISABLE_AI_CODE
	Asura_AI_Navigation_System::Reset();
#endif
	Asura_StaticMessage_System::Reset();

#ifdef ASURA_TOOLS
	Asura_Surface_Mesh_System::Shutdown();
	Asura_Surface_Mesh_System::Initialise();
	Asura_Mesh_System::Reset();
#endif
	Asura_Environment_System::Reset();
	//RAR 02/04/03 - Cutscenes need to be reset before animations/hierarchies so that there aren't
	//any problems with releasing tween handles.
	Asura_Cutscene_System::Reset();

	Asura_Animation_System::Reset();
#if defined(ASURA_SUPPORT_BODYPART_SYSTEM)
	Asura_Hierarchy_BodyPart_System::Reset();
#endif	// defined(ASURA_SUPPORT_BODYPART_SYSTEM)
	Asura_Module_System::Reset();
		
	Asura_LocalisedText::ResetPages();

	Asura_StaticCorona_System::Reset();

	Asura_TextureAnimation_System::Reset();
	Asura_TiledTextureManagement::Reset();

#if defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM
	Asura_MaterialResponse_System::Reset(false);
#endif

	Asura_Texture_Management::Reset();
	Asura_Material_System::Reset();
	Asura_Timers::Reset();

#ifdef ASURA_USE_COLLISIONFX
	Asura_CollisionFX_System::Reset();
#endif //ASURA_USE_COLLISIONFX

	Asura_SpecialFX_System::Reset();

	/// Kevin Lea - 15/08/2002 16:09:38 - lights and sounds want to be
	/// shutdown after special fx etc. so there are no ordering problems
	/// with deallocations

#ifdef ASURA_USE_AUDIO_SYSTEM
	Asura_Audio_Systems_Manager::Reset();
#else
	Asura_Sound_System::Reset();
#endif

#ifdef ASURA_USE_ARTICULATION_SYSTEM
	Asura_LipSync_Manager::Reset();
#endif

	Asura_GI_Solution::Reset();
	Asura_Lighting_System::Reset(); // soon to be deprecated
	
    Asura_Shape_System::Reset();

	Asura_Fragment_System::Reset();

#ifdef ASURA_TOOLS
	Tools_ShapeTable::Shutdown();
#endif //ASURA_TOOLS

#ifdef ASURA_USE_BILLBOARD_PLANTS
	Asura_BillboardPlant_TemplateStore::Reset();
	Asura_BillboardPlant_System::Reset();
#endif //ASURA_USE_BILLBOARD_PLANTS

#ifdef ASURA_USE_BOTANICALS
	Asura_Botanicals_System::Reset();	
#endif
#ifdef ASURA_USE_CARPET
	Asura_Carpet_MeshManager::Reset();
#endif

#ifdef ASURA_USE_FSFX
	Asura_FSFX_System::Reset();
#endif

#ifdef ASURA_USE_ATTRACTORS
	Asura_Attractor_System::Reset();
#ifdef ASURA_TOOLS
	Tools_Attractor_System::Reset();
#endif
#endif

#ifdef ASURA_USE_DEBUG_TEXT_MANAGER
	Asura_DebugText_Manager::Reset();
#endif

#ifdef ASURA_USE_AXON
	Axon_System::Reset();
#endif

	
	// NRM 17-12-02 - Don't really want to flush all the text between levels do we?
	// Asura_HashedLocalisedText::Reset();

	Asura_ModuleReverb_System::Reset();


	// KJFL - 2004-07-01 - The Photon Mapping (GI) system needs to exist outside the Tools
	// Initialise/Shutdown cycle, since 'quit to game' functionality needs the lighting
	// to be available *after* the Tools_System has been shutdown.
#ifdef ASURA_TOOLS	
	Tools_PhotonMapping_System::Reset();
	Tools_Adjacency::Shutdown();
	Tools_Adjacency::Initialise();

	Tools_SecondaryEntityFile_System::Reset();
#endif

#if defined(ASURA_TARGET_SM3) && defined(ASURA_TOOLS)
	Tools_RenderModule_System::Reset();
#endif	

	Asura_Physics_System::Reset();
	
#if defined( ASURA_USE_VISIBILITY_SYSTEM ) || defined( ASURA_USE_NEW_VISIBILITY_SYSTEM )
	Asura_Visibility_System::Reset();
#endif	// defined( ASURA_USE_VISIBILITY_SYSTEM )

#if defined(ASURA_USE_INDEPENDENT_SHADOW_SYSTEM)
	Asura_Shadow_System::Reset();
#endif	// defined(ASURA_USE_INDEPENDENT_SHADOW_SYSTEM)

#ifdef ASURA_USE_CONTROLMAP_MANAGER
	Asura_ControlManager::Reset();
#endif

#ifdef ASURA_USE_GUI_MENU_SYSTEM
	Asura_GUIMenu_System::Reset();
#endif 

#ifdef ASURA_TARGET_SM3
	Asura_Projector_System::Reset();
	Asura_LightShaft_System::Reset();
	Asura_DeferredShading::Reset();
  	Asura_AmbientOcclusion::Reset();
#endif

#if !defined(ASURA_DONT_USE_TELEPORTERS)
	Asura_Teleport_System::Reset();
#endif	// !defined(ASURA_DONT_USE_TELEPORTERS)

#if defined( ASURA_USE_DEBUGOVERLAY )
	Asura_DebugOverlay::Reset();
#endif // #if defined( ASURA_USE_DEBUGOVERLAY )

#if defined( ASURA_USE_PRIMITIVE_RENDER )
	Asura_Primitive_Render::Reset();
#endif // #if defined( ASURA_USE_PRIMITIVE_RENDER )

#if defined( ASURA_USE_GUIMENU_PROFILING )
	Asura_Profiling_GUIMenu::Reset();
#endif // #if defined( ASURA_USE_DEBUGOVERLAY )

}
