//
// Asura_Core.h
//
// Created by Kevin Lea - 12/06/2001 10:58:29
// 
// Copyright (C) 2001 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef __ASURA_CORE_H__
#define __ASURA_CORE_H__

// generic wrapper which handles many initialisation/shutdown calls

class Asura_Core
{
	public:

	// EarlyInitialise inits just enough Asura systems to display a localised text string. 
	static void EarlyInitialise(void);
	// LateInitialise initialises all other Asura systems
	static void LateInitialise(void);
	static void Shutdown(void);
	static void Reset(void);

	static bool IsMainThread(void) { return Platform_IsMainThread(); }

    private:

	// PlatformInitialise is assumed to be required to get anything at all working 
	// so it is called by EarlyInitialise. Currently no need for Platform_LateInitialise().
	static void PlatformInitialise(void);
	static void PlatformShutdown(void);
	static void PlatformReset(void);

	static bool Platform_IsMainThread(void);

    // PlatformProject_EarlyInitialise used to be used to allow setting of render 
	// buffer sizes per project on PS2, but that's now done using project defines.
	// It's currently used for initialising the language based on registry settings
	// on some projects. It shouldn't do anything that takes too long, e.g. loading.
    static void PlatformProject_EarlyInitialise(void);
	static void PlatformProject_LateInitialise(void);
};


#endif//__ASURA_CORE_H__
