//
// Asura_FunctionSequencer.cpp
//
// Created by RWM - 31/03/2009
//
// Copyright (C) 2009 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

// This file's header
#include "Core/Asura_FunctionSequencer.h"

#ifndef ASURA_NO_DEBUG_OUTPUT
#include "Debugging/Asura_CPUProfile.h"
#include "Debugging/Asura_Logfile.h"
#endif

Asura_Mempool Asura_FunctionSequencer::s_xFunctionNodePool( sizeof(Asura_ListElt<Asura_FunctionSequencer_Node>), 256 );

////////////////////////////////////////
// Asura_FunctionSequencer_Node
////////////////////////////////////////
#if defined(_DEBUG) || defined (ASURA_USE_PROFILING_SYSTEM)
Asura_FunctionSequencer_Node::Asura_FunctionSequencer_Node(void (*pfFunction)(void), const char* szFunctionString, const char* szReadableString, u_int uProfileID)
#else
Asura_FunctionSequencer_Node::Asura_FunctionSequencer_Node(void (*pfFunction)(void))
#endif
: m_pfFunction( pfFunction )
#if defined(_DEBUG) || defined (ASURA_USE_PROFILING_SYSTEM)
, m_szFunctionString( szFunctionString )
, m_szReadableString( szReadableString )
, m_uProfileID( uProfileID )
#endif
, m_bEnabled( true )
{
}

bool Asura_FunctionSequencer_Node::operator==(const Asura_FunctionSequencer_Node &other) const
{
	return (m_pfFunction == other.m_pfFunction);
}

////////////////////////////////////////
// Asura_FunctionSequencer_Node
////////////////////////////////////////
Asura_FunctionSequencer::Asura_FunctionSequencer()
{
	m_xFunctionSequence.SetMemorySupplyFunction( AllocateFromListNodeMempool );
	m_xFunctionSequence.SetMemoryReturnFunction( DeallocateToListNodeMempool );
}

Asura_FunctionSequencer::~Asura_FunctionSequencer()
{
	m_xFunctionSequence.Clear();
}

#if defined(_DEBUG) || defined (ASURA_USE_PROFILING_SYSTEM)
void Asura_FunctionSequencer::AddBefore( void(*pfnFunctionToAdd)(void), void(*pfnNextFunction)(void), const char* szFunctionName, const char* szReadableName, u_int uProfileSection )
#else
void Asura_FunctionSequencer::AddBefore( void(*pfnFunctionToAdd)(void), void(*pfnNextFunction)(void) )
#endif
{
#if defined(_DEBUG) || defined (ASURA_USE_PROFILING_SYSTEM)
	Asura_FunctionSequencer_Node xNodeToAdd( pfnFunctionToAdd, szFunctionName, szReadableName, uProfileSection);
#else
	Asura_FunctionSequencer_Node xNodeToAdd( pfnFunctionToAdd );
#endif

	if (m_xFunctionSequence.Size() == 0)
	{
		m_xFunctionSequence.PushFront( xNodeToAdd );
	}
	else
	{
#if defined(_DEBUG) || defined (ASURA_USE_PROFILING_SYSTEM)
		Asura_Assertf( !m_xFunctionSequence.Find( xNodeToAdd ), ("Function %s already in function sequencer.\n", szFunctionName) );
#endif
		if ( m_xFunctionSequence.Find( xNodeToAdd ) ) { return; }

		Asura_ListElt<Asura_FunctionSequencer_Node>* pxNextNode = FindListNodeByFunction( pfnNextFunction );
		Asura_Assert( pxNextNode, "Unable to find existing function to add before.\n" );
		if (!pxNextNode) { return; }

		m_xFunctionSequence.InsertBefore( pxNextNode, xNodeToAdd );
	}
}

#if defined(_DEBUG) || defined (ASURA_USE_PROFILING_SYSTEM)
void Asura_FunctionSequencer::AddAfter( void(*pfnFunctionToAdd)(void), void(*pfnPrevFunction)(void), const char* szFunctionName, const char* szReadableName, u_int uProfileSection )
#else
void Asura_FunctionSequencer::AddAfter( void(*pfnFunctionToAdd)(void), void(*pfnPrevFunction)(void) )
#endif
{
#if defined(_DEBUG) || defined (ASURA_USE_PROFILING_SYSTEM)
	Asura_FunctionSequencer_Node xNodeToAdd( pfnFunctionToAdd, szFunctionName, szReadableName, uProfileSection);
#else
	Asura_FunctionSequencer_Node xNodeToAdd( pfnFunctionToAdd );
#endif

	if (m_xFunctionSequence.Size() == 0)
	{
		m_xFunctionSequence.PushBack( xNodeToAdd );
	}
	else
	{
#if defined(_DEBUG) || defined (ASURA_USE_PROFILING_SYSTEM)
		Asura_Assertf( !m_xFunctionSequence.Find( xNodeToAdd ), ("Function %s already in function sequencer.\n", szFunctionName) );
#endif
		if ( m_xFunctionSequence.Find( xNodeToAdd ) ) { return; }

		Asura_ListElt<Asura_FunctionSequencer_Node>* pxPrevNode = FindListNodeByFunction( pfnPrevFunction );
		Asura_Assert( pxPrevNode, "Unable to find existing function to add after.\n" );
		if (!pxPrevNode) { return; }

		m_xFunctionSequence.InsertAfter( pxPrevNode, xNodeToAdd );
	}
}

#if defined(_DEBUG) || defined (ASURA_USE_PROFILING_SYSTEM)
void Asura_FunctionSequencer::AddHead( void(*pfnFunctionToAdd)(void), const char* szFunctionName, const char* szReadableName, u_int uProfileSection )
#else
void Asura_FunctionSequencer::AddHead( void(*pfnFunctionToAdd)(void) )
#endif
{
#if defined(_DEBUG) || defined (ASURA_USE_PROFILING_SYSTEM)
	Asura_FunctionSequencer_Node xNodeToAdd( pfnFunctionToAdd, szFunctionName, szReadableName, uProfileSection);
	Asura_Assertf( !m_xFunctionSequence.Find( xNodeToAdd ), ("Function %s already in function sequencer.\n", szFunctionName) );
#else
	Asura_FunctionSequencer_Node xNodeToAdd( pfnFunctionToAdd );
#endif

	if ( m_xFunctionSequence.Find( xNodeToAdd ) ) { return; }

	m_xFunctionSequence.PushFront( xNodeToAdd );
}

#if defined(_DEBUG) || defined (ASURA_USE_PROFILING_SYSTEM)
void Asura_FunctionSequencer::AddTail( void(*pfnFunctionToAdd)(void), const char* szFunctionName, const char* szReadableName, u_int uProfileSection )
#else
void Asura_FunctionSequencer::AddTail( void(*pfnFunctionToAdd)(void) )
#endif
{
#if defined(_DEBUG) || defined (ASURA_USE_PROFILING_SYSTEM)
	Asura_FunctionSequencer_Node xNodeToAdd( pfnFunctionToAdd, szFunctionName, szReadableName, uProfileSection);
	Asura_Assertf( !m_xFunctionSequence.Find( xNodeToAdd ), ("Function %s already in function sequencer.\n", szFunctionName) );
#else
	Asura_FunctionSequencer_Node xNodeToAdd( pfnFunctionToAdd );
#endif

	if ( m_xFunctionSequence.Find( xNodeToAdd ) ) { return; }

	m_xFunctionSequence.PushBack( xNodeToAdd );
}

void Asura_FunctionSequencer::Remove( void(*pfnFunctionToRemove)(void) )
{
	Asura_ListElt<Asura_FunctionSequencer_Node>* pxEltToRemove = FindListNodeByFunction( pfnFunctionToRemove );
	Asura_Assert( pxEltToRemove, "Function not in list. Unable to remove.\n" );
	if (!pxEltToRemove) { return; }

	m_xFunctionSequence.Remove( pxEltToRemove );
}

void Asura_FunctionSequencer::MoveBefore( void(*pfnFunctionToMove)(void), void(*pfnNextFunction)(void) )
{
	Asura_ListElt<Asura_FunctionSequencer_Node>* pxEltToMove = FindListNodeByFunction( pfnFunctionToMove );
	Asura_ListElt<Asura_FunctionSequencer_Node>* pxEltToMoveBefore = FindListNodeByFunction( pfnNextFunction );

	Asura_Assert( pxEltToMove && pxEltToMoveBefore, "Unable to move function, as one or more of the functions specified are not in the list.\n" );
	if (!pxEltToMove || !pxEltToMoveBefore) { return; }

	Asura_FunctionSequencer_Node xToReAdd( pxEltToMove->Data() );
	m_xFunctionSequence.Remove( pxEltToMove );
	m_xFunctionSequence.InsertBefore( pxEltToMoveBefore, xToReAdd );
}

void Asura_FunctionSequencer::MoveAfter( void(*pfnFunctionToMove)(void), void(*pfnPrevFunction)(void) )
{
	Asura_ListElt<Asura_FunctionSequencer_Node>* pxEltToMove = FindListNodeByFunction( pfnFunctionToMove );
	Asura_ListElt<Asura_FunctionSequencer_Node>* pxEltToMoveAfter = FindListNodeByFunction( pfnPrevFunction );

	Asura_Assert( pxEltToMove && pxEltToMoveAfter, "Unable to move function, as one or more of the functions specified are not in the list.\n" );
	if (!pxEltToMove || !pxEltToMoveAfter) { return; }

	Asura_FunctionSequencer_Node xToReAdd( pxEltToMove->Data() );
	m_xFunctionSequence.Remove( pxEltToMove );
	m_xFunctionSequence.InsertAfter( pxEltToMoveAfter, xToReAdd );
}

void Asura_FunctionSequencer::Execute()
{
	Asura_LinkList_ConstIt<Asura_FunctionSequencer_Node> xIt( m_xFunctionSequence );
	while (!xIt.Done())
	{
		if (xIt.GetCurrent()->m_bEnabled)
		{
			//ASURA_CPU_PROFILE__START_LABEL( xIt.GetCurrent()->m_uProfileID, xIt.GetCurrent()->m_szReadableString );

			xIt.GetCurrent()->m_pfFunction();

			//ASURA_CPU_PROFILE__STOP( xIt.GetCurrent()->m_uProfileID );
		}

		xIt.Next();
	}
}

void Asura_FunctionSequencer::Reset()
{
	m_xFunctionSequence.Clear();
}

#if (defined(_DEBUG) || defined (ASURA_USE_PROFILING_SYSTEM)) && !defined(ASURA_NO_DEBUG_OUTPUT)
void Asura_FunctionSequencer::OutputToDebugger()
{
	Asura_LinkList_ConstIt<Asura_FunctionSequencer_Node> xIt( m_xFunctionSequence );
	while (!xIt.Done())
	{
		Asura_OutputToDebugger_Retail(( "Function: %s, Name: %s, (Enabled=%s)\n", xIt.GetCurrent()->m_szFunctionString, xIt.GetCurrent()->m_szReadableString, xIt.GetCurrent()->m_bEnabled ? "true" : "false"));
		xIt.Next();
	}
}
#endif

Asura_ListElt<Asura_FunctionSequencer_Node>* Asura_FunctionSequencer::FindListNodeByFunction( void(*pfnFunctionToFind)(void) )
{
#if defined(_DEBUG) || defined (ASURA_USE_PROFILING_SYSTEM)
	Asura_FunctionSequencer_Node xNodeToFind( pfnFunctionToFind, NULL, NULL, 0 );
#else
	Asura_FunctionSequencer_Node xNodeToFind( pfnFunctionToFind );
#endif

	return m_xFunctionSequence.Find( xNodeToFind );
}

void Asura_FunctionSequencer::SetFunctionEnabled( void(*pfnFunctionToModify)(void), bool bEnable )
{
	Asura_ListElt<Asura_FunctionSequencer_Node>* pxNode = FindListNodeByFunction( pfnFunctionToModify );
	if (pxNode)
	{
		pxNode->Data().m_bEnabled = bEnable;
	}
}

bool Asura_FunctionSequencer::IsFunctionEnabled( void(*pfnFunctionToFind)(void) )
{
	Asura_ListElt<Asura_FunctionSequencer_Node>* pxNode = FindListNodeByFunction( pfnFunctionToFind );
	if (pxNode)
	{
		return pxNode->Data().m_bEnabled;
	}

	return false;
}
