//////////////////////////////////////////////////////////////////////////////
//
// Asura_Graph.cpp
//
// Copyright (c) 2004 Rebellion. All rights reserved.
//
// Created by Andrew Weinkove - 12/07/04
//
// Description: Non-inlined functions for Asura_Graph.
//
//////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"
#include "Core/Asura_Graph.h"

#include "Rendering/Asura_Colour.h"

#if !defined(__SNC__)
template<>
#endif	// !defined(__SNC__)
float Asura_Graph<float>::GetZeroElement() const
{
	return 0.0f;
}

#if !defined(__SNC__)
template<>
#endif	// !defined(__SNC__)
Asura_Vector_2 Asura_Graph<Asura_Vector_2>::GetZeroElement() const
{
	return Asura_Vector_2(0.f, 0.f);
}

#if !defined(__SNC__)
template<>
#endif	// !defined(__SNC__)
Asura_Vector_3 Asura_Graph<Asura_Vector_3>::GetZeroElement() const
{
	return Asura_Vector_3(0.f, 0.f, 0.f);
}

#if !defined(__SNC__)
template<>
#endif	// !defined(__SNC__)
Asura_RGBA Asura_Graph<Asura_RGBA>::GetZeroElement() const
{
	return Asura_RGBA(0.f, 0.f, 0.f, 0.f);
}

// Clamping functions.
#if !defined(__SNC__)
template<>
#endif	// !defined(__SNC__)
void Asura_Graph<float>::ClampToMinimum(float& fValue, float fClampMinimum) const
{
	fValue = Asura_Maths::Max(fValue, fClampMinimum);
}

#if !defined(__SNC__)
template<>
#endif	// !defined(__SNC__)
void Asura_Graph<float>::ClampToMaximum(float& fValue, float fClampMaximum) const
{
	fValue = Asura_Maths::Min(fValue, fClampMaximum);
}

#if !defined(__SNC__)
template<>
#endif	// !defined(__SNC__)
void Asura_Graph<Asura_Vector_2>::ClampToMinimum(Asura_Vector_2& xValue, float fClampMinimum) const
{
	xValue.x = Asura_Maths::Max(xValue.x, fClampMinimum);
	xValue.y = Asura_Maths::Max(xValue.y, fClampMinimum);
}

#if !defined(__SNC__)
template<>
#endif	// !defined(__SNC__)
void Asura_Graph<Asura_Vector_2>::ClampToMaximum(Asura_Vector_2& xValue, float fClampMaximum) const
{
	xValue.x = Asura_Maths::Min(xValue.x, fClampMaximum);
	xValue.y = Asura_Maths::Min(xValue.y, fClampMaximum);
}

#if !defined(__SNC__)
template<>
#endif	// !defined(__SNC__)
void Asura_Graph<Asura_Vector_3>::ClampToMinimum(Asura_Vector_3& xValue, float fClampMinimum) const
{
	xValue.x = Asura_Maths::Max(xValue.x, fClampMinimum);
	xValue.y = Asura_Maths::Max(xValue.y, fClampMinimum);
	xValue.z = Asura_Maths::Max(xValue.z, fClampMinimum);
}

#if !defined(__SNC__)
template<>
#endif	// !defined(__SNC__)
void Asura_Graph<Asura_Vector_3>::ClampToMaximum(Asura_Vector_3& xValue, float fClampMaximum) const
{
	xValue.x = Asura_Maths::Min(xValue.x, fClampMaximum);
	xValue.y = Asura_Maths::Min(xValue.y, fClampMaximum);
	xValue.z = Asura_Maths::Min(xValue.z, fClampMaximum);
}

#if !defined(__SNC__)
template<>
#endif	// !defined(__SNC__)
void Asura_Graph<Asura_RGBA>::ClampToMinimum(Asura_RGBA& xValue, float fClampMinimum) const
{
	xValue.m_fA = Asura_Maths::Max(xValue.m_fA, fClampMinimum);
	xValue.m_fR = Asura_Maths::Max(xValue.m_fR, fClampMinimum);
	xValue.m_fG = Asura_Maths::Max(xValue.m_fG, fClampMinimum);
	xValue.m_fB = Asura_Maths::Max(xValue.m_fB, fClampMinimum);
}

#if !defined(__SNC__)
template<>
#endif	// !defined(__SNC__)
void Asura_Graph<Asura_RGBA>::ClampToMaximum(Asura_RGBA& xValue, float fClampMaximum) const
{
	xValue.m_fA = Asura_Maths::Min(xValue.m_fA, fClampMaximum);
	xValue.m_fR = Asura_Maths::Min(xValue.m_fR, fClampMaximum);
	xValue.m_fG = Asura_Maths::Min(xValue.m_fG, fClampMaximum);
	xValue.m_fB = Asura_Maths::Min(xValue.m_fB, fClampMaximum);
}


//Functions for RecalculateSamples()
#ifdef ASURA_TOOLS
template<>
void Asura_Graph<float>::GetComponentWiseMin(const float& xVal1,const float& xVal2,float& xOutput) const
{
	xOutput = Asura_Maths::Min(xVal1,xVal2);
}

template<>
void Asura_Graph<float>::GetComponentWiseMax(const float& xVal1,const float& xVal2,float& xOutput) const
{
	xOutput = Asura_Maths::Max(xVal1,xVal2);
}

template<>
void Asura_Graph<float>::SetComponents(float& xVal,float fSet) const
{
	xVal = fSet;
}

template<>
bool Asura_Graph<float>::WithinTolerance(const float& xVal1,const float& xVal2,const float& xTol) const
{
	return Asura_Maths::Abs(xVal2-xVal1) <= Asura_Maths::Abs(xTol);
}

template<>
void Asura_Graph<Asura_Vector_2>::GetComponentWiseMin(const Asura_Vector_2& xVal1,const Asura_Vector_2& xVal2,Asura_Vector_2& xOutput) const
{
	xOutput.x = Asura_Maths::Min(xVal1.x,xVal2.x);
	xOutput.y = Asura_Maths::Min(xVal1.y,xVal2.y);
}

template<>
void Asura_Graph<Asura_Vector_2>::GetComponentWiseMax(const Asura_Vector_2& xVal1,const Asura_Vector_2& xVal2,Asura_Vector_2& xOutput) const
{
	xOutput.x = Asura_Maths::Max(xVal1.x,xVal2.x);
	xOutput.y = Asura_Maths::Max(xVal1.y,xVal2.y);
}

template<>
bool Asura_Graph<Asura_Vector_2>::WithinTolerance(const Asura_Vector_2& xVal1,const Asura_Vector_2& xVal2,const Asura_Vector_2& xTol) const
{
	return Asura_Maths::Abs(xVal2.x-xVal1.x) <= Asura_Maths::Abs(xTol.x) &&
			Asura_Maths::Abs(xVal2.y-xVal1.y) <= Asura_Maths::Abs(xTol.y);
}

template<>
void Asura_Graph<Asura_Vector_2>::SetComponents(Asura_Vector_2& xVal,float fSet) const
{
	xVal.x = fSet;
	xVal.y = fSet;
}

template<>
void Asura_Graph<Asura_Vector_3>::GetComponentWiseMin(const Asura_Vector_3& xVal1,const Asura_Vector_3& xVal2,Asura_Vector_3& xOutput) const
{
	xOutput.x = Asura_Maths::Min(xVal1.x,xVal2.x);
	xOutput.y = Asura_Maths::Min(xVal1.y,xVal2.y);
	xOutput.z = Asura_Maths::Min(xVal1.z,xVal2.z);
}

template<>
void Asura_Graph<Asura_Vector_3>::GetComponentWiseMax(const Asura_Vector_3& xVal1,const Asura_Vector_3& xVal2,Asura_Vector_3& xOutput) const
{
	xOutput.x = Asura_Maths::Max(xVal1.x,xVal2.x);
	xOutput.y = Asura_Maths::Max(xVal1.y,xVal2.y);
	xOutput.z = Asura_Maths::Max(xVal1.z,xVal2.z);
}

template<>
bool Asura_Graph<Asura_Vector_3>::WithinTolerance(const Asura_Vector_3& xVal1,const Asura_Vector_3& xVal2,const Asura_Vector_3& xTol) const
{
	return Asura_Maths::Abs(xVal2.x-xVal1.x) <= Asura_Maths::Abs(xTol.x) &&
			Asura_Maths::Abs(xVal2.y-xVal1.y) <= Asura_Maths::Abs(xTol.y) &&
			Asura_Maths::Abs(xVal2.z-xVal1.z) <= Asura_Maths::Abs(xTol.z);
}

template<>
void Asura_Graph<Asura_Vector_3>::SetComponents(Asura_Vector_3& xVal,float fSet) const
{
	xVal.x = fSet;
	xVal.y = fSet;
	xVal.z = fSet;
}

template<>
void Asura_Graph<Asura_RGBA>::GetComponentWiseMin(const Asura_RGBA& xVal1,const Asura_RGBA& xVal2,Asura_RGBA& xOutput) const
{
	xOutput.m_fR = Asura_Maths::Min(xVal1.m_fR,xVal2.m_fR);
	xOutput.m_fG = Asura_Maths::Min(xVal1.m_fG,xVal2.m_fG);
	xOutput.m_fB = Asura_Maths::Min(xVal1.m_fB,xVal2.m_fB);
	xOutput.m_fA = Asura_Maths::Min(xVal1.m_fA,xVal2.m_fA);
}

template<>
void Asura_Graph<Asura_RGBA>::GetComponentWiseMax(const Asura_RGBA& xVal1,const Asura_RGBA& xVal2,Asura_RGBA& xOutput) const
{
	xOutput.m_fR = Asura_Maths::Max(xVal1.m_fR,xVal2.m_fR);
	xOutput.m_fG = Asura_Maths::Max(xVal1.m_fG,xVal2.m_fG);
	xOutput.m_fB = Asura_Maths::Max(xVal1.m_fB,xVal2.m_fB);
	xOutput.m_fA = Asura_Maths::Max(xVal1.m_fA,xVal2.m_fA);
}

template<>
bool Asura_Graph<Asura_RGBA>::WithinTolerance(const Asura_RGBA& xVal1,const Asura_RGBA& xVal2,const Asura_RGBA& xTol) const
{
	return Asura_Maths::Abs(xVal1.m_fR-xVal2.m_fR) <= Asura_Maths::Abs(xTol.m_fR) &&
			Asura_Maths::Abs(xVal1.m_fG-xVal2.m_fG) <= Asura_Maths::Abs(xTol.m_fG) &&
			Asura_Maths::Abs(xVal1.m_fB-xVal2.m_fB) <= Asura_Maths::Abs(xTol.m_fB) &&
			Asura_Maths::Abs(xVal1.m_fA-xVal2.m_fA) <= Asura_Maths::Abs(xTol.m_fA);
}

template<>
void Asura_Graph<Asura_RGBA>::SetComponents(Asura_RGBA& xVal,float fSet) const
{
	xVal.m_fR = fSet;
	xVal.m_fG = fSet;
	xVal.m_fB = fSet;
	xVal.m_fA = fSet;
}

#endif // ASURA_TOOLS


/// End of file. ///
