//
// Asura_HashedLocalisedText.h
//
// Created by Richard Rice - 09/12/2002
// 
// Copyright (C) 2002 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef __ASURA_HASHEDLOCALISEDTEXT_H__
#define __ASURA_HASHEDLOCALISEDTEXT_H__

#include "Core/Asura_String.h"
#include "Core/Asura_HashTree.h"
#include "Core/Resources/Asura_ResourceSet.h"

//Defines for common asura page IDs for the localised text system
static const Asura_Hash_ID uASURA_HASHEDTEXTPAGE_MENU = 0x0033155F; //Asura_GetHashID("Menu")
static const Asura_Hash_ID uASURA_HASHEDTEXTPAGE_CUTSCENE = 0x25C2AF8A; //Asura_GetHashID("Cutscene") 


class Asura_HashedLocalisedText_Page;

class Asura_HashedLocalisedText
{
	public :
		// Public typedefs...
		typedef Asura_ResourceSet<Asura_HashedLocalisedText_Page> PageContainer;
		typedef Asura_ResourceSet_Iterator<Asura_HashedLocalisedText_Page> PageIterator;
		typedef Asura_ResourceSet_ConstIterator<Asura_HashedLocalisedText_Page> PageConstIterator;

		// Public functions...
		static void Initialise();
		static void Shutdown();
				
		static const Asura_UnicodeChar* GetString(Asura_Hash_ID uPageID,Asura_Hash_ID uStringID); //Gets a string from a specified page/file. asserts if it doesn't exist		
		static const Asura_UnicodeChar* GetStringByIndex(Asura_Hash_ID uPageID, u_int uIndex); //Gets a string from a specified page/file. asserts if it doesn't exist		
		static bool IsValidString(Asura_Hash_ID uPageID,Asura_Hash_ID uStringID); //Checks for existence of a string.

		static const Asura_HashedLocalisedText_Page* GetPage(Asura_Hash_ID uPageID);
		static u_int GetNumberOfStrings(Asura_Hash_ID uPageID); //returns the number of strings in this page.
		static u_int GetNumberOfPages(); 

#ifdef ASURA_TOOLS // As these calls GetDataByIndex or other fns that are tools only or require a potentially large array.
		static void UnloadPage(Asura_Hash_ID uPageID); //unloads the specified text page ,if it exists	
		static const Asura_UnicodeChar* GetStringFromAnyPage(Asura_Hash_ID uStringID); // Gets the first instance of the specified string in all pages.
		static const char* GetTextIDStringByIndex(Asura_Hash_ID uPageID, u_int uIndex);
		static const char* GetTextIDStringFromAnyPage(Asura_Hash_ID uStringID); 
		static Asura_Hash_ID GetTextIDByIndex(Asura_Hash_ID uPageID, u_int uIndex);
#endif

#ifdef ASURA_TOOLS
        static PageContainer& GetPageContainer( void ) { return s_xPageContainer ; }
#else
        // Currently no need for non-tools code to modify the container but
        // it may want to, eg., iterate through all pages.
        static const PageContainer& GetPageContainer( void ) { return s_xPageContainer ; }
#endif

	private :
		// Private functions...
		static Asura_HashedLocalisedText_Page* AllocateStorage(Asura_Hash_ID uPageHash,int iNumEntries, u_int uNumBytes);		

		// data
		static PageContainer	s_xPageContainer;
		
		friend class Asura_Chunk_HashedLocalisedText;
};


class Asura_HashedLocalisedText_Page
{
public :
	Asura_HashedLocalisedText_Page(int iNumEntries,u_int uNumBytes);	
	~Asura_HashedLocalisedText_Page();	

	const Asura_UnicodeChar* GetString(Asura_Hash_ID uStringID) const;
	const Asura_UnicodeChar* GetStringByIndex(u_int uIndex) const;
	bool IsValidString(Asura_Hash_ID uStringID) const;

	int GetNumberOfStrings() const {return m_iArraySize;}

#ifdef ASURA_TOOLS
	bool HasTextIDStrings() const {return m_pszTextIDs != 0;}
	const char* GetTextIDStringByIndex(int iIndex) const;
	const char* GetTextIDString(Asura_Hash_ID uHashID) const;
	Asura_Hash_ID GetTextIDByIndex(u_int uIndex) const;
	const char* GetName() const { return m_xName.GetString(); }
	const Asura_Hash_ID GetHashID() const { return m_xName.GetHash(); }
#endif

private :
	void AddString( Asura_Hash_ID uHash,Asura_UnicodeChar* wszString );

	int m_iArraySize;
	Asura_UnicodeChar** m_pwszStrings;
	char* m_pcStringBuffer;
	Asura_HashTree* m_pxHashTree;

#ifdef ASURA_TOOLS
	char** m_pszTextIDs;
	char* m_pcTextIDBuffer;
	Asura_String m_xName;
#endif

	friend class Asura_Chunk_HashedLocalisedText;
};


#endif //__ASURA_HASHEDLOCALISEDTEXT_H__
