//////////////////////////////////////////////////////////////////////////////
//
// Asura_LocalisedText.h
//
// Copyright (c) 2003 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header.
#include "Core/Asura_LocalisedText.h"

// Engine independent.
#include "ChunkSystem/Asura_Chunk_Loaders.h"
#include "Debugging/Asura_Assert.h"
#include "Interface/Asura_CommandConsole.h"

// C O N S T A N T S /////////////////////////////////////////////////////////

static const char* g_aszTwoCharCodes[ASURA_LT_L_NUM_LANGUAGES] =
{
    "en", // ASURA_LT_L_ENGLISH
    "fr", // ASURA_LT_L_FRENCH
    "ge", // ASURA_LT_L_GERMAN
    "it", // ASURA_LT_L_ITALIAN
    "sp", // ASURA_LT_L_SPANISH
    "ru", // ASURA_LT_L_RUSSIAN
	"am", // ASURA_LT_L_AMERICAN
	"jp", // ASURA_LT_L_JAPANESE
	"du", // ASURA_LT_L_DUTCH
	"po", // ASURA_LT_L_PORTUGUESE
    "da", // ASURA_LT_L_DANISH
    "fi", // ASURA_LT_L_FINNISH
    "no", // ASURA_LT_L_NORWEGIAN
};

static const char* g_aszLangNames[ASURA_LT_L_NUM_LANGUAGES] =
{
    "english",		// ASURA_LT_L_ENGLISH
    "french",		// ASURA_LT_L_FRENCH
    "german",		// ASURA_LT_L_GERMAN
    "italian",		// ASURA_LT_L_ITALIAN
    "spanish",		// ASURA_LT_L_SPANISH
    "russian",		// ASURA_LT_L_RUSSIAN
	"american",		// ASURA_LT_L_AMERICAN
	"japanese",		// ASURA_LT_L_JAPANESE
	"dutch",		// ASURA_LT_L_DUTCH
	"portuguese",	// ASURA_LT_L_PORTUGUESE
    "danish",       // ASURA_LT_L_DANISH
    "finnish",      // ASURA_LT_L_FINNISH
    "norwegian",    // ASURA_LT_L_NORWEGIAN
};

// G L O B A L S /////////////////////////////////////////////////////////////

static Asura_UnicodeChar**                 g_pwszLocalisedText  = NULL;
static char*							   g_pcStringBuffer		= NULL;
static u_int                               g_uMaxStrings        = 0;
static u_int                               g_uCurrentLanguage   = ASURA_LT_L_ENGLISH;
static Asura_LocalisedText_Page**          g_ppxPageStorage     = NULL;
static Asura_LocalisedText::PAGERESETGROUP g_eCurPageResetGroup = Asura_LocalisedText::PAGERESETGROUP_RESETTABLE;

// P R O T O T Y P E S   A N D   I N L I N E   F U N C T I O N S /////////////

static Asura_LocalisedText_Page** FindEntryInPageStorageByHashID( Asura_Hash_ID uHashID, bool bEmptySlotIsAcceptable );

// C L A S S   I M P L E M E N T A T I O N S /////////////////////////////////

//
// Asura_LocalisedText.
//

bool Asura_LocalisedText::s_bLocalisedTextOperational = false;

#ifdef _FLEXILEX // preprocessor directive for compiling the flexilex utility -
                 // this prevents us having to include all kinds of stuff(RM)

void Asura_LocalisedText::Initialise( u_int uLanguage ) {}
void Asura_LocalisedText::Initialise( const char* szTextFile, u_int uLanguage ) {}

#else // not def _FLEXILEX

void Asura_LocalisedText::Initialise( u_int uLanguage )
{
	if( s_bLocalisedTextOperational )
	{
		// (MP - 07.07.2003) Want to discourage this, though it may be
		// desirable to allow it in tools.
		Asura_Assert( false, "This is likely to cause serious memory fragmentation." );

		Shutdown();
	}

	s_bLocalisedTextOperational = true;

	Asura_Assert( uLanguage < ASURA_LT_L_NUM_LANGUAGES, "" );
	g_uCurrentLanguage = uLanguage;

#if ASURA_LOCALISEDTEXT_PAGE_STORAGE_SIZE > 0
	g_ppxPageStorage = new Asura_LocalisedText_Page*[ASURA_LOCALISEDTEXT_PAGE_STORAGE_SIZE];
	Asura_Assert( g_ppxPageStorage, "Out of memory." );
	for( u_int u = 0; u < ASURA_LOCALISEDTEXT_PAGE_STORAGE_SIZE; ++u )
	{
		g_ppxPageStorage[u] = NULL;
	}
#endif

	//Add console vars
#ifdef ASURA_SUPPORT_COMMAND_CONSOLE
	Asura_CommandConsole::AddVar( L"Asura.CurrentLanguage", &g_uCurrentLanguage, 0, ASURA_LT_L_NUM_LANGUAGES - 1, Asura_CommandConsole::REMOVE_AT_SHUTDOWN  );
#endif
}

void Asura_LocalisedText::Initialise( const char* szTextFile, u_int uLanguage )
{
	Initialise( uLanguage );

	if( !Asura_ChunkLoading_System::LoadAsuraFile(szTextFile) )
	{
		Asura_Assertf(0, ("Unable to load project text file. '%s'", szTextFile));
	} 
}

#endif // end not def _FLEXILEX

void Asura_LocalisedText::Shutdown()
{
    if( s_bLocalisedTextOperational ) 
	{
		delete[] g_pcStringBuffer;
		g_pcStringBuffer = NULL;

	    delete[] g_pwszLocalisedText;
	    g_pwszLocalisedText = NULL;
        
        s_bLocalisedTextOperational = false;

#if ASURA_LOCALISEDTEXT_PAGE_STORAGE_SIZE > 0
		for( u_int u = 0; u < ASURA_LOCALISEDTEXT_PAGE_STORAGE_SIZE; ++u )
		{
			delete g_ppxPageStorage[u];
		}
		
		delete[] g_ppxPageStorage;
		g_ppxPageStorage = NULL;
#endif
    }
}

Asura_LocalisedText_Language Asura_LocalisedText::GetCurrentLanguage( void )
{
    return static_cast<Asura_LocalisedText_Language>(g_uCurrentLanguage);
}

const char* Asura_LocalisedText::GetTwoCharCodeForCurLang( void )
{
    return g_aszTwoCharCodes[g_uCurrentLanguage];
}

const char* Asura_LocalisedText::GetTwoCharCodeForLang( Asura_LocalisedText_Language eLang )
{
    return g_aszTwoCharCodes[eLang];
}


const char* Asura_LocalisedText::GetNameForCurLang()
{
	return g_aszLangNames[g_uCurrentLanguage];
}

const char* Asura_LocalisedText::GetNameForLang( Asura_LocalisedText_Language eLang )
{
	return g_aszLangNames[eLang];
}

const char* const* Asura_LocalisedText::GetLanguageNames()
{
	return g_aszLangNames;
}


const Asura_UnicodeChar* Asura_LocalisedText::GetString( int iTextID )
{
    Asura_Assert( s_bLocalisedTextOperational, "Localised text not initialised!" );

    Asura_Assert( 0 <= iTextID && static_cast<u_int>(iTextID) < g_uMaxStrings, "Invalid text ID requested or executable out of date." );
		
	if (0 <= iTextID && static_cast<u_int>(iTextID) < g_uMaxStrings)
	{
		if( g_pwszLocalisedText && g_pwszLocalisedText[iTextID] )
		{
			return g_pwszLocalisedText[iTextID];
		}
	}

	Asura_Assert( false, "Localised text not loaded or no text for this ID." );
    return NULL;
}

char* Asura_LocalisedText::AllocateStringBuffer( u_int uNumBytes )
{
	Asura_Assert( !g_pcStringBuffer, "already allocated string buffer");
	g_pcStringBuffer = new char[uNumBytes];
	return g_pcStringBuffer;
}

bool Asura_LocalisedText::AllocateStringPtrArray( u_int uSizeReqd )
{
    Asura_Assert( s_bLocalisedTextOperational, "Localised text not initialised!" );

    Asura_Assert( !g_pwszLocalisedText, "Already allocated." );

	g_pwszLocalisedText = new Asura_UnicodeChar*[uSizeReqd];
	if( !g_pwszLocalisedText )
    {
        Asura_Assert( false, "Out of memory." );
        return false;
    }

    g_uMaxStrings = uSizeReqd;
	for( u_int u = 0; u < g_uMaxStrings; ++u )
	{
		g_pwszLocalisedText[u] = NULL;
	}

    return true;
}

bool Asura_LocalisedText::AddString( int iTextID, Asura_UnicodeChar* wszString )
{
    // Deal with executable being out of date. We see this on Dredd thanks to
    // broken dependencies on our text enum hdr.
    if( static_cast<u_int>(iTextID) > g_uMaxStrings )
    {
        Asura_Assert( false, "Text ID above max num tags. Maybe your executable is out of date?" );
        return false;
    }

	if( g_pwszLocalisedText[iTextID] )
    {
        Asura_Assert( false, "Text already loaded for this ID." );
        return false;
    }

	g_pwszLocalisedText[iTextID] = wszString;
    return true;
}

#ifdef ASURA_TOOLS
u_int Asura_LocalisedText::GetNumLoadedStrings( void )
{
    u_int uNum = 0;
    for( u_int u = 0; u < g_uMaxStrings; ++u )
    {
        if( g_pwszLocalisedText[u] ) { ++uNum; }
    }
    return uNum;
}
#endif // #ifdef ASURA_TOOLS

#if ASURA_LOCALISEDTEXT_PAGE_STORAGE_SIZE > 0

void Asura_LocalisedText::SetPageResetGroup( Asura_LocalisedText::PAGERESETGROUP eGroup )
{
    g_eCurPageResetGroup = eGroup;
}

Asura_LocalisedText::PAGERESETGROUP Asura_LocalisedText::GetCurPageResetGroup( void )
{
    return g_eCurPageResetGroup;
}

void Asura_LocalisedText::ResetPages( Asura_LocalisedText::PAGERESETGROUP eGroup )
{
    Asura_Assert( PAGERESETGROUP_PERMANENT != eGroup, "Surely you should just be calling Shutdown()...?" );

	if (!g_ppxPageStorage) return;

	for( u_int u = 0; u < ASURA_LOCALISEDTEXT_PAGE_STORAGE_SIZE; ++u )
	{
        if( g_ppxPageStorage[u] && g_ppxPageStorage[u]->GetResetGroup() == eGroup )
        {
            delete g_ppxPageStorage[u];
            g_ppxPageStorage[u] = NULL;
        }
	}
}

Asura_LocalisedText_Page* Asura_LocalisedText::AddPage( Asura_Hash_ID uNewID, u_int uNumStrings, u_int uNumBytesForStrings )
{
    Asura_LocalisedText_Page** ppxEntry = FindEntryInPageStorageByHashID( uNewID, true );
    
    if( !ppxEntry )
    {
        Asura_Assert( false, "Out of storage space for localised text pages. You need to increase ASURA_LOCALISEDTEXT_PAGE_STORAGE_SIZE." );
        return NULL;
    }

    if( !*ppxEntry )
    {
        *ppxEntry = new Asura_LocalisedText_Page( uNewID, uNumStrings, uNumBytesForStrings );
    }

    return *ppxEntry;
}

const Asura_LocalisedText_Page* Asura_LocalisedText::GetPage( Asura_Hash_ID uID )
{
    Asura_LocalisedText_Page** ppxEntry = FindEntryInPageStorageByHashID( uID, false );
    return( ppxEntry ? *ppxEntry : NULL );
}

#ifdef ASURA_TOOLS

u_int Asura_LocalisedText::GetNumLoadedPages( void )
{
    u_int uNum = 0;
    for( u_int u = 0; u < ASURA_LOCALISEDTEXT_PAGE_STORAGE_SIZE; ++u )
    {
        if( g_ppxPageStorage[u] ) { ++uNum; }
    }
    return uNum;
}

void Asura_LocalisedText::TraverseLoadedPages( void(*pfnCallback)(const Asura_LocalisedText_Page*) )
{
    for( u_int u = 0; u < ASURA_LOCALISEDTEXT_PAGE_STORAGE_SIZE; ++u )
    {
        if( g_ppxPageStorage[u] ) { pfnCallback( g_ppxPageStorage[u] ); }
    }
}


#endif // #ifdef ASURA_TOOLS

const Asura_UnicodeChar* Asura_LocalisedText::GetString( int iTextID, Asura_Hash_ID uPageID )
{
	const Asura_LocalisedText_Page* pxPage = Asura_LocalisedText::GetPage(uPageID);
	if(pxPage)
	{
		return pxPage->GetString(iTextID);
	}
	else
	{
        Asura_Assert( false, "Localised text requested from missing or invalid page." );
        return L"";
    }
}

//
// Asura_LocalisedText_Page.
//

Asura_LocalisedText_Page::Asura_LocalisedText_Page( Asura_Hash_ID uID, u_int uNumStrings, u_int uNumBytesForStrings )
{	
	m_uID         = uID;
    m_eResetGroup = g_eCurPageResetGroup;
    m_uNumStrings = uNumStrings;

	m_pwszLocalisedText = new Asura_UnicodeChar*[uNumStrings];
	m_pcStringBuffer = new char[uNumBytesForStrings];

    for( u_int u = 0; u < uNumStrings; ++u )
    {
        m_pwszLocalisedText[u] = NULL;
    }

#ifdef ASURA_TOOLS
	m_xToolsName.MakeBlank();
#endif
}

Asura_LocalisedText_Page::~Asura_LocalisedText_Page( void )
{
    if( m_pwszLocalisedText )
    {
	    delete[] m_pcStringBuffer;

        delete[] m_pwszLocalisedText;
    }
}

bool Asura_LocalisedText_Page::AddString( int iTextID, Asura_UnicodeChar* wszString )
{
    Asura_Assert( 0 <= iTextID && static_cast<u_int>(iTextID) < m_uNumStrings, "" );

    if( m_pwszLocalisedText[iTextID] )
    {
        Asura_Assert( false, "Text already loaded for this ID in this page." );
        return false;
    }
	
	m_pwszLocalisedText[iTextID] = wszString;
    return true;
}

const Asura_UnicodeChar* Asura_LocalisedText_Page::GetString( int iTextID ) const
{
    if ( 0 <= iTextID && static_cast<u_int>(iTextID) < m_uNumStrings )
    {
    	Asura_Assert( m_pwszLocalisedText[iTextID], "No text loaded for this ID in this page." );
	    return m_pwszLocalisedText[iTextID];
    }
    else
    {
        Asura_Assert( false, "bad localised text index" );
        return 0;
    }
}

// F U N C T I O N   I M P L E M E N T A T I O N S ///////////////////////////

static Asura_LocalisedText_Page** FindEntryInPageStorageByHashID( Asura_Hash_ID uHashID, bool bEmptySlotIsAcceptable )
{
    const u_int uStart = uHashID % ASURA_LOCALISEDTEXT_PAGE_STORAGE_SIZE;

	// If text system hasn't been initialised, just return null, instead of crashing
	// DMR - 19/10/06
	if (g_ppxPageStorage == NULL)
	{
		return NULL;
	}

    u_int uNext = uStart;
    do
    {
        if( g_ppxPageStorage[uNext] )
        {
            if( g_ppxPageStorage[uNext]->GetPageID() == uHashID ) { return &g_ppxPageStorage[uNext]; }
        }
        else
        {
            if( bEmptySlotIsAcceptable ) { return &g_ppxPageStorage[uNext]; }
        }

        if( ASURA_LOCALISEDTEXT_PAGE_STORAGE_SIZE == ++uNext ) { uNext = 0; }
    }
    while( uStart != uNext );

    return NULL;
}

#endif // #if ASURA_LOCALISEDTEXT_PAGE_STORAGE_SIZE > 0

