//
// Asura_MempoolFactory_Template.h
//
// Created by KJFL - 12 October 2004
//
// Copyright (C) 2004 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __ASURA_MEMPOOLFACTORY_TEMPLATE_H__
#define __ASURA_MEMPOOLFACTORY_TEMPLATE_H__

#include "Core/Asura_Mempool.h"


template <class T> class Asura_MempoolFactory_Template
{
	public:

		//
		// Factory methods, called by system in charge of factory
		// 

		// Factory constructor. Takes as a parameter the number of 'elements' per mempool.
		// The most optimum number is determined my class size and usage patterns; it defaults
		// to a safe value of 16, which should be reasonably cache friendly without wasting
		// too much memory.
		Asura_MempoolFactory_Template( const u_int uNumberOfInstancesPerPool = 16 )
		{
			// create the  handle pool for this class
			m_pxInstancePool = new Asura_TrackingHandlePool<T>( uNumberOfInstancesPerPool );
		}
		
		// Factory destructor - completely clears the mempool
		virtual ~Asura_MempoolFactory_Template()
		{
			// deallocate anything outstanding
			Reset();

			// delete the entire mempool 
			delete m_pxInstancePool;
		}

		// Factory reset - destroys the contents of the mempool but not the pool itself
		void Reset( void )
		{
			// scan through any existing instances and remove them
			Asura_TrackingHandlePool_Iterator<T> xIterator = m_pxInstancePool->GetIterator();

			for(xIterator.Begin(); !xIterator.End(); xIterator.Next())
			{
				Asura_Handle iHandle = xIterator.GetCurrentHandle();
				DestroyInstance( iHandle );
			}
			m_pxInstancePool->Reset();
		}


		//
		// Per class instance methods for creating/destroying/accessing the templated class
		//

		// Create an instance of the templated class. This returns a handle to the class,
		// rather than a direct pointer. GetInstancePointer() can be used to access a pointer.
		// ASURA_INVALID_HANDLE is returned if it was not possible to create an instance.
		Asura_Handle CreateInstance( void )
		{
			return m_pxInstancePool->Allocate();
		}

		// Destroy an instance, based on it's handle
		void DestroyInstance( const Asura_Handle iHandle )
		{
			m_pxInstancePool->Deallocate( iHandle );
		}

		// Get a pointer to an instance
		T* GetInstancePointer( const Asura_Handle iHandle )
		{
			return m_pxInstancePool->GetPointer( iHandle );
		}  

		// const version of method for when you're not intending to modify data
		const T* GetInstancePointer( const Asura_Handle iHandle ) const
		{
			return m_pxInstancePool->GetPointer( iHandle );
		}  

		// Get the  handle pool the instances are in (so you can iterate through
		// the pool, for example).
		Asura_TrackingHandlePool<T>* GetPool( void )
		{
			return m_pxInstancePool;
		}

		// const version of method for when you're not intending to modify data
		const Asura_TrackingHandlePool<T>* GetPool( void ) const
		{
			return m_pxInstancePool;
		}

	protected:

		Asura_TrackingHandlePool<T>* m_pxInstancePool;
};


#endif//__ASURA_MEMPOOLFACTORY_TEMPLATE_H__
