//
// Asura_OrientedVolume.h
//
// Created by Richard Rice - 25/01/05
// 
// Copyright (C) 2005 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef __ASURA_ORIENTEDVOLUME_H__
#define __ASURA_ORIENTEDVOLUME_H__

#include "Core/Asura_Maths.h"

class Asura_OrientedVolume
{
public :
	Asura_Bounding_Box m_xBox;
	Asura_Quat m_xOrient;

	bool Contains (const Asura_Vector_3& xPosition) const;
	float GetSqdMinDistFromPoint(const Asura_Vector_3& xPosition) const;
	Asura_Vector_3 CalculateCentre() const {return m_xBox.CalculateCentre();}
    // if we're outside the volume this will get us back in along the shortest route
    void CalcMinVectorIntoVolume( const Asura_Vector_3& xPosition, Asura_Vector_3& xVector ) const;

	//tests for intersection with an AABB
	bool Overlaps(const Asura_Bounding_Box& xBB) const {return Overlaps(m_xOrient, m_xBox, xBB);}

	//static version of overlap function where the oriented BB is passed in as well
	static bool Overlaps(const Asura_Quat& xOrient, const Asura_Bounding_Box xOBB, const Asura_Bounding_Box& xAABB);


};

// RAR 22/02/2008 - Moved from MultiDamageVolume , intermediate structure and function for overlap test. 

// (AW - 09.02.2004) - This structure contains the data needed for the OBB intersection test.
// The orientations and the centre are relative to the first bounding box.
class Asura_OBB_AdjustedForIntersectionTest
{
public:
	Asura_Vector_3 xCentre;
	Asura_Vector_3 axOrientationUnitVectors[3];
	Asura_Vector_3 xHalfLengths;
};

bool Asura_OBBIntersectTest(const Asura_Vector_3 &xHalfLengthsForA, const Asura_OBB_AdjustedForIntersectionTest &xB);


#endif //__ASURA_ORIENTEDVOLUME_H__
