//
// Asura_PrimeSearch.cpp
//
// Created by Andrew Weinkove - 10/10/05
// 
// Copyright (C) 2005 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"
#include "Core/Asura_PrimeSearch.h"

#include "Core/Asura_Maths.h"
#include "Debugging/Asura_Assert.h"

// The size of this array is fairly arbitrary, 
const int Asura_PrimeSearch::s_aiPrimeArray[] = 
{
	// All the primes up to 255.
	2, 3, 5, 7, 11, 13, 17, 19, 23, 29, 31, 37,
	41, 43, 47, 53, 59, 61, 67, 71, 73, 79, 83,
	89, 97, 101, 103, 107, 109, 113, 127, 131, 137, 
	149, 151, 157, 163, 167, 173, 179, 181, 191, 193, 
	197, 199, 211, 223, 227, 229, 233, 241, 251,
	// Skipping lots of primes now.... 
	509, 1009, 2099, 5003, 7411, 9901, 13001
};

Asura_PrimeSearch::Asura_PrimeSearch(int iNumElements)
{
	Asura_Assert(iNumElements > 0, "No elements in list");
	Asura_Assert(iNumElements < 12852, "Too many elements, risks overflow in quadratic");

	m_iMaxElements = iNumElements;
	Regenerate();
}

void Asura_PrimeSearch::Restart() 
{
	m_iCurrentPosition = Asura_Random::GetInt(0, m_iMaxElements - 1);
	m_iSearches = 0;
}

void Asura_PrimeSearch::Regenerate()
{
	int iRandomA = Asura_Random::GetInt(1, 13);
	int iRandomB = Asura_Random::GetInt(1, 7);
	int iRandomC = Asura_Random::GetInt(1, 5);

	m_iSkip = (iRandomA * m_iMaxElements * m_iMaxElements) + (iRandomB * m_iMaxElements) + iRandomC;
	m_iSkip &= ~0xc0000000; // This stops m_iSkip from becoming too big.

	Restart();

	m_piCurrentPrime = s_aiPrimeArray;

	#if defined ASURA_ASSERT
		int iNumEntries = sizeof(s_aiPrimeArray) / sizeof(s_aiPrimeArray[0]);
	#endif

	Asura_Assert(iNumEntries > 0, "No entries in prime array?!");
	Asura_Assert(s_aiPrimeArray[iNumEntries - 1] > m_iMaxElements, "Number of elements too large for prime search. Add more entries to the s_aiPrimeArray table");

	// Slow sequential search, should be OK as long as the table remains small.
	while (*m_piCurrentPrime < m_iMaxElements)
	{
		m_piCurrentPrime++;
	}

	int iTest = m_iSkip % *m_piCurrentPrime;
	if (!iTest)
	{
		m_iSkip++;
	}
}

int Asura_PrimeSearch::GetNext()
{
	if (Done())
	{
		return -1;
	}

	int iNextMember = m_iCurrentPosition;

	while (true)
	{
		iNextMember = iNextMember + m_iSkip;
		iNextMember %= *m_piCurrentPrime;
		m_iSearches++;

		if (iNextMember < m_iMaxElements)
		{
			m_iCurrentPosition = iNextMember;
			return m_iCurrentPosition;
		}
		else if (Done())
		{
			return -1;
		}
	}
}
