
///////////////////////////////////////////////////////////////////////////////
//
// File   : Asura_MemoryBank.h
// Author : Andy Tate
// Date   : 7-Feb-2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __ASURA_MEMORYBANK_H__
#define __ASURA_MEMORYBANK_H__

////////////////////////////////////////////////////////////////////////////////
// Defines...
////////////////////////////////////////////////////////////////////////////////

#if defined(ASURA_MAIN) && !defined(ASURA_MEMORYBANK_ALLOW_STATS) //RAR 28/07/2006 Disable outside of main game/tools since the rendering stuff won't be available
#define ASURA_MEMORYBANK_ALLOW_STATS	1
#endif

//#define ASURA_MEMORYBANK_INT_HANDLE		1

#if !defined(ASURA_NO_DEBUG_OUTPUT) && !defined(NDEBUG) && defined(ASURA_RESOURCE_ALLOCATOR_USES_MEMORY_BANK)
#define ASURA_MEMORYBANK_ALLOW_LOGGING	1
#endif	// !defined(ASURA_NO_DEBUG_OUTPUT) && !defined(NDEBUG) && defined(ASURA_RESOURCE_ALLOCATOR_USES_MEMORY_BANK)


////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura_Handle.h"
#include "Core/Asura_Maths.h"
#include "Core/Asura_String.h"


////////////////////////////////////////////////////////////////////////////////
// Forward class declarations...
////////////////////////////////////////////////////////////////////////////////

class Asura_Vector_2;


////////////////////////////////////////////////////////////////////////////////
// Class declaration...
////////////////////////////////////////////////////////////////////////////////

#if defined(ASURA_MEMORYBANK_ALLOW_STATS)

////////////////////////////////////////////////////////////////////////////////
// Class        : Asura_MemoryBank_StatsRender
// Author       : Andy Tate
// Date         : 15-Mar-2006
//
// Description  : 
////////////////////////////////////////////////////////////////////////////////

class Asura_MemoryBank_StatsRender
{
public:
	// Public enumerated types...
	enum Column
	{
		NAME_COLUMN,
		NUM_ALLOCATIONS_COLUMN,
		SIZE_COLUMN,

		MAX_COLUMNS
	};

	// Public functions...
	Asura_MemoryBank_StatsRender  (const Asura_Handle uFontHandle, const float fTitleHeight, const float fLineHeight, const float* pfColumnOffsets);
//	~Asura_MemoryBank_StatsRender ();

	// Render functions...
	void							RenderTitle		(const Column eColumn, const Asura_Char* szText) const;
	void							RenderLine		(const Column eColumn, const u_int uRow, const Asura_Char* szText) const;

	void							RenderRow		(const u_int uRow, const Asura_Char* szText) const;

	// Accessor functions...
	void							SetFontHandle	(const Asura_Handle uHandle)				{ m_uFontHandle				= uHandle; }
	void							SetColour		(const u_int uColour)						{ m_uColour					= uColour; }
	void							SetTopLeft		(const Asura_Vector_2& xTopLeft)			{ m_xTopLeft				= xTopLeft; }
	void							SetColumnOffset	(const Column eColumn, const float fOffset)	{ m_afColumnOffset[eColumn]	= fOffset; }
	void							SetTitleHeight	(const float fHeight)						{ m_fTitleHeight			= fHeight; }
	void							SetTitleScale	(const float fScale)						{ m_fTitleScale				= fScale; }
	void							SetLineHeight	(const float fHeight)						{ m_fLineHeight				= fHeight; }
	void							SetLineScale	(const float fScale)						{ m_fLineScale				= fScale; }


protected:
	// Protected variables...


	// Protected functions...


private:
	// Private variables...
	Asura_Handle	m_uFontHandle;
	u_int			m_uColour;
	Asura_Vector_2	m_xTopLeft;
	float			m_afColumnOffset[MAX_COLUMNS];

	float			m_fTitleHeight;
	float			m_fTitleScale;

	float			m_fLineHeight;
	float			m_fLineScale;


	// Private functions...

	Asura_MemoryBank_StatsRender  (void);
	Asura_MemoryBank_StatsRender  (const Asura_MemoryBank_StatsRender& xOther);

	Asura_MemoryBank_StatsRender&	operator=	(const Asura_MemoryBank_StatsRender& xOther);

};   // End of class 'Asura_MemoryBank_StatsRender'.



////////////////////////////////////////////////////////////////////////////////
// Class        : Asura_MemoryBank_Stats
// Author       : Andy Tate
// Date         : 7-Feb-2006
//
// Description  : Stats handling class for use with the memory-bank.
////////////////////////////////////////////////////////////////////////////////

class Asura_MemoryBank_Stats
{
public:
	// Public functions...
	Asura_MemoryBank_Stats  (void)	: m_xName("?"), m_uNumAllocations(0), m_uNumBytesAllocated(0)	{}
//	~Asura_MemoryBank_Stats ();

	// Data manipulation functions...
	void					Reset					(void)					{ m_uNumAllocations = 0; m_uNumBytesAllocated = 0; }

	void					AddAllocation			(const u_int uNumBytes)	{ m_uNumAllocations++; m_uNumBytesAllocated += uNumBytes; }

	// Debugging...
#if defined(ASURA_MEMORYBANK_ALLOW_STATS)
	void					DebugRender				(const Asura_MemoryBank_StatsRender& xRender, const u_int uRow) const;
#endif	// defined(ASURA_MEMORYBANK_ALLOW_STATS)
#if defined(ASURA_MEMORYBANK_ALLOW_LOGGING)
	void					OutputToDebugger		(void) const;
#endif	// defined(ASURA_MEMORYBANK_ALLOW_LOGGING)

	// Accessor functions...
	void					SetName					(const Asura_Char* szName)	{ m_xName = szName; }
	const Asura_Char*		GetName					(void) const				{ return(m_xName.GetString()); }

	void					SetNumAllocations		(const u_int uAllocs)		{ m_uNumAllocations = uAllocs; }
	u_int					GetNumAllocations		(void) const				{ return(m_uNumAllocations); }
		
	void					SetNumBytesAllocated	(const u_int uBytes)		{ m_uNumBytesAllocated = uBytes; }
	u_int					GetNumBytesAllocated	(void) const				{ return(m_uNumBytesAllocated); }


protected:
	// Protected variables...


	// Protected functions...


private:
	// Private variables...
	Asura_String	m_xName;					// The name of this type of stat.
	u_int			m_uNumAllocations;			// The number of allocations of this type.
	u_int			m_uNumBytesAllocated;		// The number of bytes we have allocated of this type.


	// Private functions...
	Asura_MemoryBank_Stats  (const Asura_MemoryBank_Stats& xOther);

	Asura_MemoryBank_Stats&	operator=				(const Asura_MemoryBank_Stats& xOther);

};   // End of class 'Asura_MemoryBank_Stats'.

#endif	// defined(ASURA_MEMORYBANK_ALLOW_STATS)


////////////////////////////////////////////////////////////////////////////////
// Class        : Asura_MemoryBank
// Author       : Andy Tate
// Date         : 7-Feb-2006
//
// Description  : Memory bank class - allows allocation of data from a bank of
//				  memory which has already been allocated. Does not allow deletion
//				  of data, we merely delete the bank as a whole when the time
//				  comes.
////////////////////////////////////////////////////////////////////////////////

class Asura_MemoryBank
{
public:
	// Public typedefs...
#if defined(ASURA_MEMORYBANK_INT_HANDLE)
	typedef Asura_Handle	Handle;
#else
	typedef u_char*			Handle;
#endif	// defined(ASURA_MEMORYBANK_INT_HANDLE)

	// Public functions...
	Asura_MemoryBank  (const Asura_Char* szName, const u_int uSizeOfData);
	Asura_MemoryBank  (const Asura_Char* szName, const u_int uSizeOfData, u_char* pucStartOfData);
	~Asura_MemoryBank ();

	// Data manipulation functions...
#if defined(ASURA_MEMORYBANK_ALLOW_STATS)
	void				CreateStats			(const u_int uNumStatsTypes, const Asura_Char** pszStatsNames);
#endif	// defined(ASURA_MEMORYBANK_ALLOW_STATS)

	void				Reset				(void);

	Handle				Allocate			(const u_int uNumEntries, const u_int uSizeOfEntry, const u_int uDataType, const ASURA_ALIGN_ID eAlignment);
	bool				Deallocate			(const u_int uDataType, const Handle xHandle);

#if defined(ASURA_MEMORYMANAGEMENT_ENABLED)
	Handle				Allocate			(const u_int uNumEntries, const u_int uSizeOfEntry, const u_int uDataType, const ASURA_ALIGN_ID eAlignment, const char* szFile, const int iLine, const u_int uReturnAddr);
	bool				Deallocate			(const u_int uDataType, const Handle xHandle, const char* szFile, const int iLine);
#endif	// defined(ASURA_MEMORYMANAGEMENT_ENABLED)

	// Debugging...
#if defined(ASURA_MEMORYBANK_ALLOW_STATS)
	void				DebugRenderStats	(const Asura_MemoryBank_StatsRender& xRender) const;
#endif	// defined(ASURA_MEMORYBANK_ALLOW_STATS)
#if defined(ASURA_MEMORYBANK_ALLOW_LOGGING)
	void				OutputToDebugger	(void) const;
#endif	// defined(ASURA_MEMORYBANK_ALLOW_LOGGING)

	// Accessor functions...
	void				SetAllowDeletion	(const bool bSet)				{ m_bAllowDeletion = bSet; }
	bool				IsDeletionAllowed	(void) const					{ return(m_bAllowDeletion); }

	const Asura_Char*	GetName				(void) const					{ return(m_xName.GetString()); }
#if defined(ASURA_MEMORYBANK_INT_HANDLE)
	Handle				GetHandle			(const void* pvData) const		{ return(((u_int)(pvData)) - ((u_int)(m_pucStartOfData))); }
	void*				GetAddress			(const Handle xHandle) const	{ return(&m_pucStartOfData[xHandle]); }
#else
	Handle				GetHandle			(const void* pvData) const		{ return(const_cast<u_char*>(static_cast<const u_char*>(pvData))); }
	void*				GetAddress			(const Handle xHandle) const	{ return(xHandle); }
#endif	// defined(ASURA_MEMORYBANK_INT_HANDLE)

	u_int				GetSizeOfEntry		(const Handle xHandle) const	{ const u_int* puData = static_cast<u_int*>(GetAddress(xHandle)); return(puData[-1]); }
	u_int				GetNumEntries		(const Handle xHandle) const	{ const u_int* puData = static_cast<u_int*>(GetAddress(xHandle)); return(puData[-2]); }


protected:
	// Protected variables...


	// Protected functions...


private:
	// Private variables...
			Asura_String			m_xName;					// The name of this memory-bank.
			bool					m_bAllowDeletion;			// Flags to say whether deletion should assert.
			bool					m_bDataAllocated;			// Flag to say whether we allocated the data.
			u_char*					m_pucStartOfData;			// The start of the data-block to use.
			u_int					m_uSizeOfData;				// The size (in bytes) of the data-block to use.
			u_int					m_uCurrentSize;				// The number of bytes currently allocated.

			Handle					m_xHandleOfLastAlloc;		// The handle returned for the last allocation.
			Handle					m_xHandleBeforeLastAlloc;	// The handle before the last allocation was performed.

#if defined(ASURA_MEMORYBANK_ALLOW_STATS)
			u_int					m_uNumStatsTypes;			// The number of types of data to collect stats for.
			Asura_MemoryBank_Stats*	m_pxStats;					// Stats gathering for the various types of data.
#endif	// defined(ASURA_MEMORYBANK_ALLOW_STATS)

	static	Handle					s_xInvalidHandle;			// Need for cases when failure is required.


	// Private functions...
	Asura_MemoryBank  (void);
	Asura_MemoryBank  (const Asura_MemoryBank& xOther);

	Asura_MemoryBank&	operator=			(const Asura_MemoryBank& xOther);

	// Useful functions...
	bool				Allocate			(const u_int uSize);
	bool				PadForAlignment		(const u_int uAlignment);

	// Accessor functions...
#if defined(ASURA_MEMORYBANK_INT_HANDLE)
	bool				IsHandleValid		(const Handle xHandle) const	{ return((xHandle != s_xInvalidHandle) && (xHandle < m_uSizeOfData)); }
	Handle				GetCurrentHandle	(void) const					{ return(m_uCurrentSize); }
	void				SetCurrentHandle	(const Handle xHandle)			{ if(IsHandleValid(xHandle)) m_uCurrentSize = xHandle; }
#else
	bool				IsHandleValid		(const Handle xHandle) const	{ return((xHandle != s_xInvalidHandle) && (xHandle >= &m_pucStartOfData[0]) && (xHandle < &m_pucStartOfData[m_uSizeOfData])); }
	Handle				GetCurrentHandle	(void) const					{ return(const_cast<u_char*>(&m_pucStartOfData[m_uCurrentSize])); }
	void				SetCurrentHandle	(const Handle xHandle)			{ if(IsHandleValid(xHandle)) m_uCurrentSize = xHandle - m_pucStartOfData; }
#endif	// defined(ASURA_MEMORYBANK_INT_HANDLE)

};   // End of class 'Asura_MemoryBank'.


#endif   // ifndef __ASURA_MEMORYBANK_H__


/// End of file. ///
