//
// Asura_Task_Queue.h
//
// Created by KJFL - 5 March 2008
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __ASURA_TASK_QUEUE_H__
#define __ASURA_TASK_QUEUE_H__

#ifdef ASURA_USE_TASK_SYSTEM

#include "Core/Threads/Asura_Task_System.h"

typedef void* (*Asura_Task_Queue_Callback) (void);

// KJFL - 5 March 2008 - Asura_Task_Queue is designed to act as a wrapper
// for any list or queue which you wish worker threads to iterate through.
// You provide a callback fn to get a void* to the next item (from whatever
// list/queue pattern you're using) and the usual worker function that will
// process that void*. The Asura_Task_System will then do the rest.
//
// The callback fn to get the next item THIS MUST BE THREADSAFE!
// (Using a lock-free scheme is recommended.)
class Asura_Task_Queue : public Asura_Task
{
	public:	
		Asura_Task_Queue( Asura_TaskFunction pfnTask, Asura_Task_Queue_Callback pfnQueueCallback );
		~Asura_Task_Queue();

		// Get something from the queue - called by worker threads.
		void* GetTaskData( void );

	private:
		
		bool				m_bTerminateTasks;

		// Callback function to get the next item on your own queue or list.
		// THIS MUST BE THREADSAFE! (A lock-free scheme is recommended)
		Asura_Task_Queue_Callback m_pfnQueueCallback;

};

#endif //#ifdef ASURA_USE_TASK_SYSTEM

#endif//__ASURA_TASK_QUEUE_H__

