//
// Asura_Debug_Texture_Viewer.cpp
//
// Created by KOR - 19 September 2008
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

#ifdef ASURA_USE_DEBUG_TEXTURE_VIEWER

// This file's header.
#include "Debugging/Asura_Debug_Texture_Viewer.h"

// Asura
#include "Core/Asura_TextureManagement.h"
#include "Core/Asura_UserInput.h"
#include "Input/Asura_ControlMap.h"
#ifdef ASURA_SUPPORT_COMMAND_CONSOLE
#include "Interface/Asura_CommandConsole.h"
#endif
#include "Rendering/Asura_Font_System.h"
#include "Rendering/Asura_Render.h"
#include "Rendering/Asura_Viewport.h"

bool Asura_Debug_Texture_Viewer::s_bEnabled = false;
bool Asura_Debug_Texture_Viewer::s_bIsSelected = false;
u_int Asura_Debug_Texture_Viewer::s_uSelectedX = 0;
u_int Asura_Debug_Texture_Viewer::s_uSelectedY = 0;
u_int Asura_Debug_Texture_Viewer::s_uOffsetY = 0;
u_int Asura_Debug_Texture_Viewer::s_uNumCols = 0;
u_int Asura_Debug_Texture_Viewer::s_uNumRows = 0;
u_int Asura_Debug_Texture_Viewer::s_uMaxOffsetY = 0;
Asura_Debug_Texture_Viewer::TRANSLUCENCY_MODE Asura_Debug_Texture_Viewer::s_eTranslucency = Asura_Debug_Texture_Viewer::TRANSLUCENCY_SOLID;
Asura_ControlMap Asura_Debug_Texture_Viewer::s_xControlMap;

DEBUGVAR u_int dbg_uThumbNailSize = 64;
DEBUGVAR u_int dbg_uThumbNailSpacing = 8;

void Asura_Debug_Texture_Viewer::Render()
{
	if(!s_bEnabled)
	{
		return;
	}
	const int iViewportX = Asura_Viewport::GetWidth();
	const int iViewportY = Asura_Viewport::GetHeight();
	const Asura_Bounding_Rectangle& xSafeArea = Asura_Render::GetSafeArea();
	Asura_Vector_2 xSafeTL = xSafeArea.GetTopLeft();
	xSafeTL.x *= iViewportX;
	xSafeTL.y *= iViewportY;
	Asura_Vector_2 xSafeDimensions = xSafeArea.GetDimension();
	xSafeDimensions.x *= iViewportX;
	xSafeDimensions.y *= iViewportY;

	// Shouldn't really change, but make sure anyway.
	s_uNumCols = (xSafeDimensions.x + dbg_uThumbNailSpacing) / (dbg_uThumbNailSize + dbg_uThumbNailSpacing);
	s_uNumRows = (xSafeDimensions.y + dbg_uThumbNailSpacing) / (dbg_uThumbNailSize + dbg_uThumbNailSpacing);

	const u_int uNumTextures = Asura_Texture_Management::GetNumberOfTextures();
	const u_int uLastRow = (uNumTextures / s_uNumCols) + 1;
	if(uLastRow <= s_uNumRows)
	{
		s_uNumRows = uLastRow;
		s_uMaxOffsetY = 0;
	}
	else
	{
		s_uMaxOffsetY = uLastRow - s_uNumRows;
	}
	HandleControls();
	const u_int uSelectedTexID = (s_uOffsetY + s_uSelectedY) * s_uNumCols + s_uSelectedX;
	if(s_bIsSelected)
	{
		u_int uPreviewSize = 1024;
		const u_int uMinDimension = Asura_Maths::FloatToInt_Chop(Asura_Maths::Min(xSafeDimensions.x, xSafeDimensions.y));
		while(uPreviewSize > uMinDimension)
		{
			uPreviewSize >>= 1;
		}
		const int iCentreX = iViewportX / 2;
		const int iCentreY = iViewportY / 2;
		const u_int uHalfPreviewSize = uPreviewSize / 2;
		Asura_Render::Overlay.SetRectUV(0.f, 0.f, 1.f, 1.f);
		Asura_Render::Overlay.SetRectXY(iCentreX - uHalfPreviewSize, iCentreY - uHalfPreviewSize, uPreviewSize, uPreviewSize);
		SetupAlpha();
		Asura_Render::Overlay.TextureIndex = uSelectedTexID;
		Asura_Render::Overlay.Render();
	}
	else
	{
		const u_int uNumTextures = Asura_Texture_Management::GetNumberOfTextures();
		// Render thumbnails
		for(u_int uY = 0; uY < s_uNumRows; ++uY )
		{
			for(u_int uX = 0; uX < s_uNumCols; ++uX )
			{
				const u_int uTexID = (s_uOffsetY + uY) * s_uNumCols + uX;
				if(uTexID >= uNumTextures)
				{
					break;
				}
				Asura_Render::Overlay.SetRectUV(0.f, 0.f, 1.f, 1.f);
				Asura_Vector_2 xPos = xSafeTL;
				xPos.x += (dbg_uThumbNailSize + dbg_uThumbNailSpacing) * uX;
				xPos.y += (dbg_uThumbNailSize + dbg_uThumbNailSpacing) * uY;
				if(uTexID == uSelectedTexID)
				{
					// Render highlight
					Asura_Render::Overlay.SetRectXY(xPos.x - 2.f, xPos.y - 2.f, dbg_uThumbNailSize + 4.f, dbg_uThumbNailSize + 4.f);
					Asura_Render::Overlay.TextureIndex = ASURA_TEXTURE_NONE;
					Asura_Render::Overlay.TranslucencyMode = ASURA_TRANSLUCENCY_OFF;
					Asura_Render::Overlay.Render();
				}
				Asura_Render::Overlay.SetRectXY(xPos.x, xPos.y, dbg_uThumbNailSize, dbg_uThumbNailSize);
				SetupAlpha();
				Asura_Render::Overlay.TextureIndex = uTexID;
				Asura_Render::Overlay.Render();
			}
		}
		// Current texture name
		const Asura_String* pxTextureName = Asura_Texture_Management::GetTextureName(uSelectedTexID);
		if(pxTextureName)
		{
			Asura_Vector_2 xTextPos = xSafeTL;
			xTextPos.y += (dbg_uThumbNailSize + dbg_uThumbNailSpacing) * s_uNumRows;
			Asura_Font_System::Print(xTextPos.x, xTextPos.y, 0xffffffff, ASURA_TEXTFORMAT_FORMATSTRING, "%u - %s", uSelectedTexID, pxTextureName->GetString());
		}
	}
}

void Asura_Debug_Texture_Viewer::HandleControls()
{
	if(!s_xControlMap.GetNumMappings())
	{
		AddDefaultControlMappings();
	}
	s_xControlMap.Update();
	// Direction controls
	if(s_xControlMap.GetValue(CONTROL_UP))
	{
		if(s_uSelectedY > 0)
		{
			--s_uSelectedY;
		}
		else
		{
			// Scroll?
			if(s_uOffsetY > 0)
			{
				--s_uOffsetY;
			}
		}
	}
	else if(s_xControlMap.GetValue(CONTROL_DOWN))
	{
		if(s_uSelectedY < s_uNumRows - 1)
		{
			++s_uSelectedY;
		}
		else
		{
			// Scroll?
			if(s_uOffsetY < s_uMaxOffsetY)
			{
				++s_uOffsetY;
			}
		}
	}
	else if(s_xControlMap.GetValue(CONTROL_LEFT))
	{
		if(s_uSelectedX > 0)
		{
			--s_uSelectedX;
		}
		else
		{
			s_uSelectedX = s_uNumCols - 1;
		}
	}
	else if(s_xControlMap.GetValue(CONTROL_RIGHT))
	{
		if(s_uSelectedX < s_uNumCols - 1)
		{
			++s_uSelectedX;
		}
		else
		{
			s_uSelectedX = 0;
		}
	}

	if(s_xControlMap.GetValue(CONTROL_SELECT))
	{
		s_bIsSelected = !s_bIsSelected;
	}
	if(s_xControlMap.GetValue(CONTROL_CHANGETRANSLUCENCY))
	{
		s_eTranslucency = static_cast<TRANSLUCENCY_MODE>(s_eTranslucency + 1);
		if(s_eTranslucency == NUM_TRANSLUCENCY_MODES)
		{
			s_eTranslucency = TRANSLUCENCY_SOLID;
		}
	}
}

void Asura_Debug_Texture_Viewer::SetupAlpha()
{
	if(s_eTranslucency == TRANSLUCENCY_SOLID)
	{
		// No background necessary.
		Asura_Render::Overlay.Colour = 0xffffffff;
		Asura_Render::Overlay.TranslucencyMode = ASURA_TRANSLUCENCY_OFF;
		return;
	}
	// Choose background colour
	if(s_eTranslucency == TRANSLUCENCY_ALPHABLACK)
	{
		Asura_Render::Overlay.Colour = 0xff000000;
	}
	else
	{
		Asura_Render::Overlay.Colour = 0xffff00ff;
	}
	// Render background
	Asura_Render::Overlay.TextureIndex = ASURA_TEXTURE_NONE;
	Asura_Render::Overlay.Render();
	Asura_Render::FlushDisplay();
	// Get ready for texture itself
	Asura_Render::Overlay.Colour = 0xffffffff;
	Asura_Render::Overlay.TranslucencyMode = ASURA_TRANSLUCENCY_NORMAL;
}

void Asura_Debug_Texture_Viewer::AddDefaultControlMappings()
{
	s_xControlMap.CreateMappings( NUM_CONTROL_MAPPINGS );
	s_xControlMap.AddTapAxisMapping(CONTROL_UP,		ASURA_AXIS_PAD1_LSTICK_V, 1, Asura_ControlMapping::ON_DEBOUNCED_PRESS, 1.f, 1.f,-.5f);
	s_xControlMap.AddTapAxisMapping(CONTROL_DOWN,	ASURA_AXIS_PAD1_LSTICK_V, 1, Asura_ControlMapping::ON_DEBOUNCED_PRESS, 1.f, 1.f, .5f);
	s_xControlMap.AddTapAxisMapping(CONTROL_LEFT,	ASURA_AXIS_PAD1_LSTICK_H, 1, Asura_ControlMapping::ON_DEBOUNCED_PRESS, 1.f, 1.f,-.5f);
	s_xControlMap.AddTapAxisMapping(CONTROL_RIGHT,	ASURA_AXIS_PAD1_LSTICK_H, 1, Asura_ControlMapping::ON_DEBOUNCED_PRESS, 1.f, 1.f, .5f);
	s_xControlMap.AddKeyMapping(CONTROL_SELECT, ASURA_KEYID_PAD1_BUTTON1, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
	s_xControlMap.AddKeyMapping(CONTROL_CHANGETRANSLUCENCY, ASURA_KEYID_PAD1_BUTTON2, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
}

#ifdef ASURA_SUPPORT_COMMAND_CONSOLE
void Asura_Debug_Texture_Viewer::AddConsoleCommands()
{
	Asura_CommandConsole::AddVar(L"Debug.EnableTextureViewer", &s_bEnabled, Asura_CommandConsole::REMOVE_AT_SHUTDOWN);
}
#endif

#endif//def ASURA_USE_DEBUG_TEXTURE_VIEWER