//*******************************************************************
// File: Asura_Debug_Timers.cpp
//
// Created: 31 May 2007 DLH
//
// Description: 
//
//*******************************************************************

// system libraries
#include "Core/Asura.h"

#if defined(ASURA_CPU_PROFILE_SCOPED_TIMER_ENABLE)

// main header
#include "Debugging/Asura_Debug_Timers.h"

// Asura
#include "Core/Asura_Timers.h"
#include "Debugging/Asura_Logfile.h"

//*******************************************************************
// Constants
//*******************************************************************
char ASURA_CPU_PROFILE_SCOPED_TIMER_LOGFILE[ 256] = "logfiles\\DebugTimer.txt";
char ASURA_CPU_PROFILE_SCOPED_TIMER_LOGFILE_XML[ 256] = "logfiles\\DebugTimer.xml";

//*******************************************************************
// Local Classes
//*******************************************************************
class Asura_CPUProfile_ScopedTimer_Global_Close
{
public:
	~Asura_CPUProfile_ScopedTimer_Global_Close();
};

//*******************************************************************
// Local Datas
//*******************************************************************

Asura_CPUProfile_ScopedTimer* Asura_CPUProfile_ScopedTimer::s_pxPresentTimer = NULL;
u_int Asura_CPUProfile_ScopedTimer::s_uNestedDepth = 0;
bool Asura_CPUProfile_ScopedTimer::s_bDebugOutputEnabled = false;
bool Asura_CPUProfile_ScopedTimer::s_bLogFileInitialised = false;
bool Asura_CPUProfile_ScopedTimer::s_bLogFileReset = true;
bool Asura_CPUProfile_ScopedTimer::s_bLogFileXML = true;
float Asura_CPUProfile_ScopedTimer::s_fReportTimeThreshold = 0.01f;

static Asura_CPUProfile_ScopedTimer_Global_Close s_xAsura_CPUProfile_ScopedTimer_Global_Close;

static Asura_TickCount s_axStoppedTime[ ]	= { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, };
static u_int s_uStoppedTimers				= sizeof( s_axStoppedTime ) / sizeof( s_axStoppedTime[ 0 ] );

//*******************************************************************
// Function Definitions
//*******************************************************************

Asura_CPUProfile_ScopedTimer_Global_Close::~Asura_CPUProfile_ScopedTimer_Global_Close()
{
	if( Asura_CPUProfile_ScopedTimer::s_bLogFileInitialised && 
		Asura_CPUProfile_ScopedTimer::s_bLogFileReset && 
		Asura_CPUProfile_ScopedTimer::s_bLogFileXML )
	{
		Asura_SimpleLogfileCloseXML(ASURA_CPU_PROFILE_SCOPED_TIMER_LOGFILE_XML);
	}
}

const u_int uNESTING_STRING_LENGTH = 32;
static void GetNestingString(u_int uNestedDepth, char* szNesting)
{
	const int iNestingDepth = Asura_Maths::Min( uNESTING_STRING_LENGTH - 1, uNestedDepth );
	for( int i = 0; i < iNestingDepth; i++ )
	{
		szNesting[i] = '.';
	}
	szNesting[iNestingDepth] = NULL;
}

Asura_CPUProfile_ScopedTimer::Asura_CPUProfile_ScopedTimer(const char* pszText) :
	m_pszText(pszText),
	m_pxParentTimer(NULL),
	m_xTimeChildren(0)
{
	// DLH - 31 May 2007 11:43:19 - Empty log file
	if(!s_bLogFileInitialised)
	{
		if(s_bLogFileReset)
		{
			Asura_SimpleLogfileClear(ASURA_CPU_PROFILE_SCOPED_TIMER_LOGFILE);
			if(s_bLogFileXML)
			{
				Asura_SimpleLogfileOpenXML(ASURA_CPU_PROFILE_SCOPED_TIMER_LOGFILE_XML);
			}
		}
		s_bLogFileInitialised = true;
	}

#if defined(ASURA_CPU_PROFILE_SCOPED_TIMER_ENABLE_MEMORY_LOG)
	m_iMemoryStart = 0;
	m_iMemoryChildrenDelta = 0;
	
	#if defined(ASURA_TOOLS)
	{
		MEMORYSTATUSEX xMemoryStatus = { 0};
		xMemoryStatus.dwLength = sizeof(xMemoryStatus);
		
		GlobalMemoryStatusEx(&xMemoryStatus);
		
		m_iMemoryStart = size_t(xMemoryStatus.ullTotalVirtual - xMemoryStatus.ullAvailVirtual);
	}
	#endif // defined(ASURA_TOOLS)
#endif // defined(ASURA_CPU_PROFILE_SCOPED_TIMER_ENABLE_MEMORY_LOG)

	// DLH - 31 May 2007 11:11:35 - Change the active timer to the present one.
	m_pxParentTimer = s_pxPresentTimer;
	s_pxPresentTimer = this;
	s_uNestedDepth += 1;

	m_xTimeStart = Asura_Timers::GetCurTicks();
	float fSinceLast = 0.0f;
	if( s_uNestedDepth <= s_uStoppedTimers )
	{
		const Asura_TickCount xPreviousStopTime = s_axStoppedTime[ s_uNestedDepth - 1 ];
		if( xPreviousStopTime != 0 )
		{
			fSinceLast	= Asura_Timers::TicksToSeconds( m_xTimeStart - xPreviousStopTime );
		}
	}

	char szNesting[uNESTING_STRING_LENGTH];
	GetNestingString(s_uNestedDepth, szNesting);

	if(s_bDebugOutputEnabled)
	{
		if( fSinceLast > 1.0f )
		{
			Asura_OutputToDebugger_Retail(("%s Unprofiled period, T:%.3fs\n", szNesting, fSinceLast ) );
		}
		Asura_OutputToDebugger_Retail(("%s Start '%s'.\n", szNesting, m_pszText));
	}

	if( fSinceLast > 1.0f )
	{
		Asura_SimpleLogfileOutput((ASURA_CPU_PROFILE_SCOPED_TIMER_LOGFILE, "%s Unprofiled period, T:%.3fs\n", szNesting, fSinceLast ) );
	}
	Asura_SimpleLogfileOutput((ASURA_CPU_PROFILE_SCOPED_TIMER_LOGFILE, "%s Start '%s'\n", szNesting, m_pszText ) );
	
	if(s_bLogFileXML)
	{
		if( fSinceLast > 1.0f )
		{
			Asura_SimpleLogfileOutput((ASURA_CPU_PROFILE_SCOPED_TIMER_LOGFILE_XML, "<timer><name>Unprofiled period</name><time>%.3f</time></timer>\n", fSinceLast ) );
		}
		const u_int uNameXMLSizeMax = 256;
		char szNameXML[ uNameXMLSizeMax];
		Asura_SimpleLogfileConvertToXML(m_pszText, szNameXML, uNameXMLSizeMax);
		Asura_SimpleLogfileOutput((ASURA_CPU_PROFILE_SCOPED_TIMER_LOGFILE_XML, "<timer><name>%s</name>\n", szNameXML) );
	}
}

Asura_CPUProfile_ScopedTimer::~Asura_CPUProfile_ScopedTimer()
{
	Asura_TickCount xTimeStop = Asura_Timers::GetCurTicks();
	Asura_TickCount xTimeDelta = xTimeStop - m_xTimeStart;

	if( s_uNestedDepth <= s_uStoppedTimers )
	{
		s_axStoppedTime[ s_uNestedDepth - 1 ] = xTimeStop;
		if( s_uNestedDepth < s_uStoppedTimers )
		{
			s_axStoppedTime[ s_uNestedDepth ] = 0;
		}
	}

	char szNesting[uNESTING_STRING_LENGTH];
	GetNestingString(s_uNestedDepth, szNesting);
	
#if defined(ASURA_CPU_PROFILE_SCOPED_TIMER_ENABLE_MEMORY_LOG)
	size_t iMemoryStop = 0;
	
	#if defined(ASURA_TOOLS)
	{
		MEMORYSTATUSEX xMemoryStatus = { 0};
		xMemoryStatus.dwLength = sizeof(xMemoryStatus);
		
		GlobalMemoryStatusEx(&xMemoryStatus);
		
		iMemoryStop = size_t(xMemoryStatus.ullTotalVirtual - xMemoryStatus.ullAvailVirtual);
	}
	#endif //defined(ASURA_TOOLS)
	size_t iMemoryDelta = iMemoryStop - m_iMemoryStart;
	
	const u_int uLogMemoryReportSizeMax = 80;
	char szLogMemory[uLogMemoryReportSizeMax];
	
	Asura_PrintfToBfr(szLogMemory, " MT:0x%08X MD:%i MU:%i", uLogMemoryReportSizeMax, true, iMemoryStop, iMemoryDelta, (iMemoryDelta - m_iMemoryChildrenDelta) );
#else // !defined(ASURA_CPU_PROFILE_SCOPED_TIMER_ENABLE_MEMORY_LOG)
	char szLogMemory[ 1] = { '\0'};
#endif

	{
		Asura_CPUProfile_ScopedTimer* pxParentTimer = m_pxParentTimer;
		if(pxParentTimer)
		{
			pxParentTimer->m_xTimeChildren += xTimeDelta;
#if defined(ASURA_CPU_PROFILE_SCOPED_TIMER_ENABLE_MEMORY_LOG)
			pxParentTimer->m_iMemoryChildrenDelta += iMemoryDelta;
#endif
		}
	}
	
	const float fTimeDelta = Asura_Timers::TicksToSeconds(xTimeDelta);
	Asura_SimpleLogfileOutput((ASURA_CPU_PROFILE_SCOPED_TIMER_LOGFILE, "%s Stop  '%s'", szNesting, m_pszText));
	if(s_fReportTimeThreshold < fTimeDelta)
	{
		Asura_SimpleLogfileOutput((ASURA_CPU_PROFILE_SCOPED_TIMER_LOGFILE, ", T:%.3fs, C:%.3fs, U:%.3fs%s", fTimeDelta, Asura_Timers::TicksToSeconds(m_xTimeChildren), Asura_Timers::TicksToSeconds(xTimeDelta - m_xTimeChildren), szLogMemory));
	}
	Asura_SimpleLogfileOutput((ASURA_CPU_PROFILE_SCOPED_TIMER_LOGFILE, "\n"));

	
	if(s_bLogFileXML)
	{
#if defined(ASURA_CPU_PROFILE_SCOPED_TIMER_ENABLE_MEMORY_LOG)
		Asura_SimpleLogfileOutput((ASURA_CPU_PROFILE_SCOPED_TIMER_LOGFILE_XML, "<memory value='%d'/>", iMemoryStop));
#endif
		Asura_SimpleLogfileOutput((ASURA_CPU_PROFILE_SCOPED_TIMER_LOGFILE_XML, "<time>%.3f</time>\n</timer>\n", fTimeDelta));
	}

	if(s_bDebugOutputEnabled)
	{
		Asura_OutputToDebugger_Retail(("%s Stop  '%s' Time Taken %f s. Child time %f s %s\n", szNesting, m_pszText, Asura_Timers::TicksToSeconds(xTimeDelta), Asura_Timers::TicksToSeconds(m_xTimeChildren), szLogMemory));
	}
	
	// DLH - 31 May 2007 11:11:20 - Reset Parent as the active timer
	s_uNestedDepth -= 1;
	s_pxPresentTimer = m_pxParentTimer;	
}

void Asura_CPUProfile_ScopedTimer::EnableLogFileReset(const bool bEnable)
{
	s_bLogFileReset = bEnable;
}

void Asura_CPUProfile_ScopedTimer::EnableXMLLogFile(const bool bEnable)
{
	s_bLogFileXML = bEnable;
}

void Asura_CPUProfile_ScopedTimer::SetFilename(const char* pszFilename)
{
	strncpy(ASURA_CPU_PROFILE_SCOPED_TIMER_LOGFILE, pszFilename, sizeof(ASURA_CPU_PROFILE_SCOPED_TIMER_LOGFILE) - 1);
}

void Asura_CPUProfile_ScopedTimer::SetXMLFilename(const char* pszFilename)
{
	strncpy(ASURA_CPU_PROFILE_SCOPED_TIMER_LOGFILE_XML, pszFilename, sizeof(ASURA_CPU_PROFILE_SCOPED_TIMER_LOGFILE_XML) - 1);
}

#endif //defined(ASURA_CPU_PROFILE_SCOPED_TIMER_ENABLE)


//*******************************************************************
//
// (c) 2006 Rebellion
//
// Warning: This computer program is protected by copyright law and 
// international treaties. Unauthorized reproduction or distribution
// of this program, or any portion of it, may result in severe civil and
// criminal penalties, and will be prosecuted to the maximum extent 
// possible under the law.
//
//*******************************************************************
