//
// Asura_ServerEntity_EnvTextureAnimControl.cpp
//
// Created by Richard Rice - 24/05/2005
// 
// Copyright (C) 2005 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"
#include "Entity/Animation/Asura_ServerEntity_EnvTextureAnimControl.h"

#include "Animation/Asura_TextureAnimation.h"
#include "ChunkSystem/Asura_Chunk_Macros.h"
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "Debugging/Asura_Assert.h"
#include "Entity/Asura_Entity_Classification.h"
#include "Network/Asura_Message.h"
#ifdef ASURA_USE_STREAMING
#include "Streaming/Asura_Streaming_System.h"
#endif

#define ASURA_SERVERENTITY_ENVTEXTUREANIMCONTROL_VERSION_NUMBER 1

Asura_ServerEntity_EnvTextureAnimControl::Asura_ServerEntity_EnvTextureAnimControl(Asura_Guid uGuid /*= ASURA_GUID_UNREGISTERED*/)
: Asura_Entity_Server(uGuid)
, m_uAnimHash(ASURA_HASH_ID_UNSET)
, m_uSequence(0)
, m_fSpeed(1)
{
	Classification = AsuraEntityClass_EnvTextureAnimControl;
	SetAlwaysUpdateClient(false);
}

Asura_ServerEntity_EnvTextureAnimControl::Asura_ServerEntity_EnvTextureAnimControl(Asura_Guid uGuid, const char*& pcChunkBuffer)
: Asura_Entity_Server(uGuid)
, m_uAnimHash(ASURA_HASH_ID_UNSET)
, m_uSequence(0)
, m_fSpeed(1)
{
	Classification = AsuraEntityClass_EnvTextureAnimControl;

	int iVersion;
	ASURA_CHUNK_EXTRACT( pcChunkBuffer, iVersion,int);
	Asura_Assert(iVersion>=0 && iVersion <= ASURA_SERVERENTITY_ENVTEXTUREANIMCONTROL_VERSION_NUMBER,"Unknown chunk version");

	if(iVersion >= 1)
	{
		Asura_Entity_Server::InitialiseFromChunkBuffer(pcChunkBuffer);
	}

	ASURA_CHUNK_EXTRACT(pcChunkBuffer,m_uAnimHash,Asura_Hash_ID);
	ASURA_CHUNK_EXTRACT(pcChunkBuffer,m_uSequence,u_int);
	ASURA_CHUNK_EXTRACT(pcChunkBuffer,m_fSpeed,float);

	SetAlwaysUpdateClient(false);
}

void Asura_ServerEntity_EnvTextureAnimControl::CreateFromChunkBuffer(Asura_Guid uGuid, const char*& pcChunkBuffer)
{
#ifdef TEST_ENTITY_CHUNK_STREAM
	TestStream<Asura_ServerEntity_EnvTextureAnimControl>(uGuid, pcChunkBuffer);
#endif
	Asura_ServerEntity_EnvTextureAnimControl* pxEnvTextAnimControl = new Asura_ServerEntity_EnvTextureAnimControl(uGuid);
	pxEnvTextAnimControl->InitialiseFromChunkBuffer(pcChunkBuffer);
}

void Asura_ServerEntity_EnvTextureAnimControl::InitialiseFromChunkBuffer(const char*& pcChunkBuffer)
{
	int iVersion;
	ASURA_CHUNK_EXTRACT( pcChunkBuffer, iVersion,int);
	Asura_Assert(iVersion>=0 && iVersion <= ASURA_SERVERENTITY_ENVTEXTUREANIMCONTROL_VERSION_NUMBER,"Unknown chunk version");

	if(iVersion >= 1)
	{
		Asura_Entity_Server::InitialiseFromChunkBuffer(pcChunkBuffer);
	}

	ASURA_CHUNK_EXTRACT(pcChunkBuffer,m_uAnimHash,Asura_Hash_ID);
	ASURA_CHUNK_EXTRACT(pcChunkBuffer,m_uSequence,u_int);
	ASURA_CHUNK_EXTRACT(pcChunkBuffer,m_fSpeed,float);

	SetNeverUpdates(true);

}

void Asura_ServerEntity_EnvTextureAnimControl::FillInChunkBuffer(char*& pcChunkBuffer)
{
	ASURA_CHUNK_FILLANDVERIFY( pcChunkBuffer, ASURA_SERVERENTITY_ENVTEXTUREANIMCONTROL_VERSION_NUMBER,int);
	Asura_Entity_Server::FillInChunkBuffer(pcChunkBuffer);

	ASURA_CHUNK_FILLANDVERIFY(pcChunkBuffer,m_uAnimHash,Asura_Hash_ID);
	ASURA_CHUNK_FILLANDVERIFY(pcChunkBuffer,m_uSequence,u_int);
	ASURA_CHUNK_FILLANDVERIFY(pcChunkBuffer,m_fSpeed,float);
}

void Asura_ServerEntity_EnvTextureAnimControl::WriteToChunkStream(Asura_Chunk_Stream& xStream) const
{
	// Version No...
	int iVersion = ASURA_SERVERENTITY_ENVTEXTUREANIMCONTROL_VERSION_NUMBER;
	xStream << iVersion;

	// Parent...
	Asura_Entity_Server::WriteToChunkStream(xStream);

	// Child...
	xStream << m_uAnimHash;
	xStream << m_uSequence;
	xStream << m_fSpeed;
}

void Asura_ServerEntity_EnvTextureAnimControl::ReadFromChunkStream(Asura_Chunk_Stream& xStream)
{
	// Version No...
	int iVersion = 0;
	xStream >> iVersion;
	Asura_Assert(iVersion>=0 && iVersion <= ASURA_SERVERENTITY_ENVTEXTUREANIMCONTROL_VERSION_NUMBER,"Unknown chunk version");

	// Parent...
	if(iVersion >= 1)
	{
		Asura_Entity_Server::ReadFromChunkStream(xStream);
	}

	// Child...
	xStream >> m_uAnimHash;
	xStream >> m_uSequence;
	xStream >> m_fSpeed;

	SetNeverUpdates(true);
}

int Asura_ServerEntity_EnvTextureAnimControl::GetChunkBufferSize()
{
	return 	Asura_Entity_Server::GetChunkBufferSize()
			+sizeof(int) //version
			+sizeof(Asura_Hash_ID)//anim hash
			+sizeof(u_int)//sequence
			+sizeof(float);//speed
}

bool Asura_ServerEntity_EnvTextureAnimControl::HandleMessage(const Asura_Message* pxMessage)
{
	if (!pxMessage) return false;

	switch(pxMessage->GetMsgID())
	{
		case ASURA_MSG_TEXTUREANIMATION :
		{
            const Asura_Message_TextureAnimation* pxData = reinterpret_cast
                <const Asura_Message_TextureAnimation*>(pxMessage->GetData());
            Asura_Assert(pxMessage->GetDataSize() == sizeof(Asura_Message_TextureAnimation), "serious error. data size has changed");

			Asura_Assertf(m_uAnimHash,("Texture Animation Controller %d is recieving messages, but doesn't have a texture anim assigned to it.",GetGuid()));
			
			//update our copy of the sequence and speed
			if(pxData->m_bChangeSequence)
			{
				m_uSequence =  pxData->m_uSequence;
			}
			if(pxData->m_bChangeSpeed)
			{
				m_fSpeed = pxData->m_fSpeed;
			}

			//and tell the clients to change as well
			// RWM - 2008-06-19 - If the anim control is told to change and is then immediately deactivated, a 
			// RequestClientUpdate will never send the update message
			MakeClientUpdateMessage();

			return true;
		}

		default:
		{
			break;
		}
	}

	return Asura_Entity_Server::HandleMessage(pxMessage);
}

void Asura_ServerEntity_EnvTextureAnimControl::MakeClientUpdateMessage()
{
	if(!m_uAnimHash) return;

   	Asura_Message* pxMessage = Asura_Network::CreateMessage(ASURA_MSG_ENVTEXTUREANIMATION,sizeof(Asura_Message_EnvironmentTextureAnimation));
   	Asura_Message_EnvironmentTextureAnimation* pxData = (Asura_Message_EnvironmentTextureAnimation*) pxMessage->GetData();

	pxData->m_uAnimHash = m_uAnimHash;
	pxData->m_sSpeed = (short)Asura_Maths::FloatToInt(m_fSpeed*100.0f);
	pxData->m_ucSequence = (unsigned char) m_uSequence;
	#ifdef ASURA_USE_STREAMING
	pxData->m_uZoneHash = m_uZoneHashID;
	#endif


   	pxMessage->To   = ASURA_GUID_ALL_CLIENTMACHINES;
   	pxMessage->From = Guid;

   	Asura_Network::Send(pxMessage);
	
}

void Asura_ServerEntity_EnvTextureAnimControl::PostLoadInitialise()
{
	Asura_Entity_Server::PostLoadInitialise();

	// Update client immediately after load rather than waiting for the burst to get around to updating it
	MakeClientUpdateMessage();
}

