///////////////////////////////////////////////////////////////////////////////
//
// File: Asura_Container_Classification.h
//
// Created by Martin G.Harris (27/5/2008)
// 
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//
// Classifications for different types of container.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef __ASURA_CONTAINER_CLASSIFICATION_H__
#define __ASURA_CONTAINER_CLASSIFICATION_H__

#ifdef ASURA_USE_CONTAINER_SYSTEM

// The maximum number of different container classifications that can be registered.
// Increase if necessary.
const int iCONTAINER_TYPES_MAX_NUMBER = 32;

// Classifications for the different types of container.

// *NOTE*: These values are saved out in chunks, so do not renumber the existing values. 
// Add new entries at the end of the list just before AsuraContainerClass_LastEngineClassification.
// These values should be contiguous as far as is possible.
// Projects can add their own classifications starting with the value AsuraContainerClass_ProjectSpecific.

enum ASURA_CONTAINER_CLASSIFICATION
{
	AsuraContainerClass_NotSet						= 0, // default value = 0

	AsuraContainerClass_Mesh						= 1,
	AsuraContainerClass_SoundEvent					= 2,
	AsuraContainerClass_Collection					= 3,
	AsuraContainerClass_Attachment					= 4,
	AsuraContainerClass_PFX							= 5,
	AsuraContainerClass_Corona						= 6,
	AsuraContainerClass_LightShaft					= 7,
	AsuraContainerClass_Projector					= 8,
	AsuraContainerClass_Photon						= 9,
	AsuraContainerClass_Strobe						= 10,
	AsuraContainerClass_GraphColour					= 11,
	AsuraContainerClass_Health						= 12,
	AsuraContainerClass_Physics						= 13,
	AsuraContainerClass_Blueprint					= 14,

	// Value is one greater than the largest valid engine classification.
	AsuraContainerClass_LastEngineClassification,

	// First id for project-specific containers.
	AsuraContainerClass_ProjectSpecific = 0x8000
};


#endif //#ifdef ASURA_USE_CONTAINER_SYSTEM

#endif //__ASURA_CONTAINER_CLASSIFICATION_H__

