//////////////////////////////////////////////////////////////////////////////
//
// Asura_ServerEntity_ServerCutsceneActor.h
//
// Created by Mike Healey.
//
// Copyright (c) 2002 Rebellion. All rights reserved.
//
// Entities to send updates from ServerCutscene to the client entities which 
// render the actors.
//
// (MP - 29.11.2004) Fairly major change:
//
// Previously this code really relied upon the engine configuration option
// ASURA_UPDATE_CLIENTENTITIES_EVERY_FRAME being turned on, as it was in GF2.
// To fit this into the current network code without turning that on and
// breaking our multiplayer stuff, we leave that off and do the following
// instead:
//
// - For relevant server entities, call SetUpdateClientEveryFrame() in the
//   constructor.
//
// - For the corresponding client entities, dispose of their
//   Asura_PosAndOriInterpolator instances, so that we use each frame's
//   server information directly.
//
// This fixes the jerky movement and animation that results in most cases if
// try to use the standard periodic updates and client interpolation.
//
// Obviously this assumes we'll never be running this stuff over a network,
// since any bandwidth limitations or latency would break this horribly.
// 
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef ___ASURA_SERVERENTITY_SERVERCUTSCENEACTOR_H___
#define ___ASURA_SERVERENTITY_SERVERCUTSCENEACTOR_H___ 1

#ifdef ASURA_SUPPORT_SERVERCUTSCENES	// the Gunfighter-style cutscenes with entities

#include "Entity/Asura_Entity.h"
#include "Core/Asura_String.h"
#include "Core/Asura_Handle.h"
#include "Core/Asura_Hash.h"

// flags in Asura_ServerEntity_ServerCutsceneActor
#define Asura_SESCU_Flag_UseAnimMovementData       (0x00000001)
#define Asura_SESCU_Flag_NotRendered		       (0x00000002)//copied from the hierarchy actor 'not rendered' flag
#define Asura_SESCU_Flag_ReservedForDerivedClasses (0xFFFF0000)

class Asura_Cutscene_Actor;

class Asura_ServerEntity_ServerCutsceneActor : public virtual Asura_Entity_Server
{

public:

    Asura_ServerEntity_ServerCutsceneActor( Asura_Cutscene_Actor* pxCutsceneActor, Asura_Guid uGUID );
    ~Asura_ServerEntity_ServerCutsceneActor( void );

	//tells entity that the cutscene actor is being deleted , so its time to lose any pointers
	//it has to it
	virtual void NotifyActorDestroyed(Asura_Cutscene_Actor* pxActor);

protected:
    
    Asura_Cutscene_Actor* m_pxActor;
    Asura_Hash_ID m_uCutsceneHash;
    u_int   m_uFlags;

private:

    typedef Asura_Entity_Server PARENT;

};

class Asura_ServerEntity_ServerCutsceneActorHierarchy : public Asura_ServerEntity_ServerCutsceneActor
{
    friend class Asura_ServerCutscene_Actor_Hierarchy;

public:
    
    Asura_ServerEntity_ServerCutsceneActorHierarchy( Asura_Cutscene_Actor* pxActor,
                                                     int iSkinID,
                                                     int iInitialAnimationID,
													 int iAnimationFlags,
													 float fAnimationDelay,
                                                     Asura_Guid uGUID );
    ~Asura_ServerEntity_ServerCutsceneActorHierarchy();
    
    Asura_ServerEntity_ServerCutsceneActorHierarchy(Asura_Guid guid, const char*& chunkBuffer);


	void ChangeAnimation(int animationID,float time,int flags,float tweenDuration) const;
    void ChangeAnimation(Asura_Hash_ID uAnimHashID,float time,int flags,float tweenDuration) const;
	void PlaySecondaryAnimation(int animationID,float time,int flags,float weight,float fTweenDuration) const;
	void PlaySecondaryAnimation(Asura_Hash_ID uAnimHashID,float fTime,int iFlags,float fWeight,float fTweenDuration) const;

	void PlayAnimationToPos(int iAnimationID,float fTime,float fStartTimer,float fEndTimer,int iFlags,float fTweenDuration) const;
	void PlaySecondaryAnimationToPos(int iAnimationID,float fTime,float fStartTimer,float fEndTimer,int iFlags,float fWeight,float fTweenDuration) const;

	void ChangeTextureAnimation(int iSequence,float fSpeed,u_int uFlags) const;

	void AddAttachment_Shape(u_int uID,Asura_Hash_ID uMarkerHash,Asura_Hash_ID uShapeHash);
	void AddAttachment_Hierarchy(u_int uID,Asura_Hash_ID uMarkerHash,int iSkinID,int iAnimID);
	void AddAttachment_Actor(u_int uID,Asura_Hash_ID uMarkerHash,Asura_Guid m_uActorID);
	void AddAttachment_Entity(u_int uID,Asura_Hash_ID uMarkerHash,Asura_Guid m_EntityID);
	void RemoveAttachment(u_int uID);

	virtual bool CanBeSavedToChunk() const {return false;}//Explicitly returning false may be redundant here , but can't do any harm. RAR 04/10/2006
	int GetChunkBufferSize(){return 0;}//This should stop these entites from being saved. RAR 11/09/02

protected:
    
    virtual void MakeClientUpdateMessage(void);

    int m_iSkinID;
	int m_iAnimationID;
	int m_iAnimationFlags;
	float m_fAnimationDelay;
	Asura_Hash_ID m_uTextureHash;

private:

    typedef Asura_ServerEntity_ServerCutsceneActor PARENT;

};

class Asura_ServerEntity_ServerCutsceneActorCamera : public Asura_ServerEntity_ServerCutsceneActor
{

public:

    Asura_ServerEntity_ServerCutsceneActorCamera(class Asura_Cutscene_Actor* pxActor);

protected:
    
    void MakeClientUpdateMessage(void);

private:

    typedef Asura_ServerEntity_ServerCutsceneActor PARENT;
       
};

class Asura_ServerEntity_ServerCutsceneActorShape : public Asura_ServerEntity_ServerCutsceneActor
{
    friend class Asura_ServerCutscene_Actor_Shape;

public:
    
    Asura_ServerEntity_ServerCutsceneActorShape( Asura_Cutscene_Actor* pxActor,
												 Asura_Hash_ID uShapeHashID,
                                                 Asura_Guid uGUID );

	int GetChunkBufferSize(){return 0;}//This should stop these entites from being saved. RAR 11/09/02

protected:
    
    void MakeClientUpdateMessage(void);

	Asura_Hash_ID m_uShapeID;

private:

    typedef Asura_ServerEntity_ServerCutsceneActor PARENT;
       
};

#endif // #ifdef ASURA_SUPPORT_SERVERCUTSCENES	// the Gunfighter-style cutscenes with entities

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

#endif // #ifndef ___ASURA_SERVERENTITY_SERVERCUTSCENEACTOR_H___
