
//////////////////////////////////////////////////////////////////////////////
//
// Asura_ClientEntity_DynamicLight.cpp
//
// Copyright (c) 2007 Rebellion (Derby). All rights reserved.
//
// Description:
//
// The all-new dynamic light entity
//
//////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

#ifdef ASURA_TARGET_SM3

// This file's header.
#include "Entity/DynamicLight/Asura_ClientEntity_DynamicLight.h"
#include "Entity/DynamicLight/Asura_Dynamiclight_Messages.h"

// Engine independent.
#include "Core/Asura_Maths.h"
#include "Core/Asura_Timers.h"
#include "Light/Asura_Photon.h"
#include "Camera/Asura_Camera.h"
#include "Network/Asura_Message.h"
#include "Rendering/Asura_Render.h"
#include "Core/Asura_TextureManagement.h"
#include "Projectors/Asura_Projector_System.h"
#include "Entity/Asura_Entity_Classification.h"
#include "Entity/EntityTemplate/Asura_ServerEntity_Template.h"
#include "Entity/PhysicalObject/Asura_ClientEntity_PhysicalObject.h"
#include "Entity/PhysicalObject/Asura_ServerEntity_PhysicalObject.h"

#ifdef ASURA_TOOLS
#include "Entity/Tools_Entity_Data.h"
#include "EntityTemplate/Tools_EntityTemplate_System.h"
#include "Entity/Tools_Entity_PhysicalObject.h"
#include "Misc/Tools_Render_Options.h"
#endif

#include "SpecialFX/Asura_Lightshaft_System.h"
#include "Entity/Lightshaft/Asura_ClientEntity_LightShaft.h"

// C L A S S   I M P L E M E N T A T I O N S /////////////////////////////////

Asura_Hashed_List<Asura_ClientEntity_DynamicLight> Asura_ClientEntity_DynamicLight::s_xList;

static bool g_bReadingFromChunkStream = false;

Asura_ClientEntity_DynamicLight* Asura_ClientEntity_DynamicLight::CreateFromChunkStream(Asura_Guid uGuid, Asura_Chunk_Stream &xStream)
{
	Asura_ClientEntity_DynamicLight* pxNew = new Asura_ClientEntity_DynamicLight(uGuid);

	g_bReadingFromChunkStream = true;

	pxNew->ReadFromServerLightChunkStream(xStream);

#ifdef ASURA_TOOLS
	Asura_Texture_Management::AddTexture(pxNew->m_xParams.m_xTextureName);	
	Asura_Texture_Management::AddTexture(pxNew->m_xParams.m_xDetailTextureName);
	Asura_Texture_Management::AddTexture(pxNew->m_xParams.m_xCoronaTextureName);
	Asura_Texture_Management::AddTexture(pxNew->m_xParams.m_xProjCubeTextureName);
	Asura_Texture_Management::AddTexture(pxNew->m_xParams.m_xProjCubeBlurredTextureName);
	Asura_Texture_Management::LoadTextures();
#endif

	pxNew->SetParams(pxNew->m_xParams);

	pxNew->Update();

	g_bReadingFromChunkStream = false;

	return pxNew;
}


#ifdef ASURA_TOOLS
Asura_ClientEntity_DynamicLight* Asura_ClientEntity_DynamicLight::CreateFromTools( Asura_Guid uGuid, const Asura_DynamicLight_Parameters &xParams)
{
	Asura_ClientEntity_DynamicLight* pxNew = new Asura_ClientEntity_DynamicLight(uGuid);

	pxNew->SetEntityActiveFromLoad(pxNew->IsEntityActive());
	pxNew->SetEntityActive(pxNew->IsEntityActive());

	pxNew->SetParams(xParams);

	pxNew->Update();

	return pxNew;
}
#endif

Asura_ClientEntity_DynamicLight::Asura_ClientEntity_DynamicLight( Asura_Guid uGuid)
: PARENT(uGuid)
, m_xParams()
, m_pxPhoton(0)
, m_pxProjector(0)
, m_iCoronaHandle(ASURA_INVALID_HANDLE)
, m_uLightShaftGUID(ASURA_GUID_UNREGISTERED)
, m_pxLightShaft(0)
{
	Classification = AsuraEntityClass_DynamicLight;

	m_xState.m_uDynamicLight_Flags	= 0;
	m_xState.m_fRed					= 1.0f;
	m_xState.m_fGreen				= 1.0f;
	m_xState.m_fBlue				= 1.0f;
	m_xState.m_fRangeScale			= 1.0f;
	m_xState.m_xPos					= Asura_Maths::ZeroVector3;
	m_xState.m_xOrient				= Asura_Maths::IdentityMatrix3x3;
	m_xState.m_fStrobeTimer			= 0.0f;
	m_xState.m_fGraphFadeDir		= 1.0f;
	m_xState.m_fGraphFadePos		= 0.0f;
	m_xState.m_fFadeStart			= 0.0f;
	m_xState.m_fFadeDuration		= 0.0f;
	m_xState.m_uFadeFlags			= 0;
	m_xState.m_fSrcR				= 1.0f;
	m_xState.m_fSrcG				= 1.0f;
	m_xState.m_fSrcB				= 1.0f;
	m_xState.m_fSrcIntensity		= 1.0f;
	m_xState.m_fSrcRange			= 0.0f;
	m_xState.m_fDestR				= 1.0f;
	m_xState.m_fDestG				= 1.0f;
	m_xState.m_fDestB				= 1.0f;
	m_xState.m_fDestIntensity		= 1.0f;
	m_xState.m_fDestRange			= 0.0f;
	m_xState.m_fCurrentBaseR		= 1.0f;
	m_xState.m_fCurrentBaseG		= 1.0f;
	m_xState.m_fCurrentBaseB		= 1.0f;
	m_xState.m_fCurrentBaseIntensity= 1.0f;
	m_xState.m_fCurrentBaseRange	= 0.0f;

	s_xList.AddToList(this);
}



Asura_ClientEntity_DynamicLight::~Asura_ClientEntity_DynamicLight()
{
	s_xList.RemoveFromList(this);

	if (m_pxProjector)
	{
		Asura_Projector_System::DestroyProjector(m_pxProjector);		
		m_pxProjector=NULL;
	}

	if(m_iCoronaHandle != ASURA_INVALID_HANDLE)
	{
		Asura_Corona_System::DestroyCorona(m_iCoronaHandle);
		m_iCoronaHandle = ASURA_INVALID_HANDLE;
	}

	if (m_pxPhoton)
	{
		Asura_Photon_System::DestroyPhoton(m_pxPhoton);
		m_pxPhoton = 0;
	}
}

void Asura_ClientEntity_DynamicLight::ReadFromServerLightChunkStream(Asura_Chunk_Stream& xStream)
{
	u_int uVersion;
	
	xStream >> uVersion;
	Asura_Assert( uVersion <= Asura_ServerEntity_DynamicLight::DYNLIGHT_CHUNK_VERSION, "" );

	bool bActive = false;

	if (uVersion >= 11)
	{
		Asura_ServerEntity_DynamicLight::SkippedServerData xSkippedData;
		Asura_ServerEntity_DynamicLight::SkipDataInChunkStream(xStream, &xSkippedData);
		bActive = xSkippedData.m_bActive && xSkippedData.m_bTemplateActive;
	}
	
	Asura_ServerEntity_DynamicLight::ExtractParamsFromChunkStream(m_xParams, xStream, uVersion );

	if (uVersion >= 12)
	{
		bool bHasClientState;
		xStream >> bHasClientState;

		if (bHasClientState)
		{
			Asura_ClientEntity_DynamicLight_State xState;
			xState.ExtractFromChunkStream(xStream, uVersion);			
			SetState(&xState);			
		}
	}

	if (uVersion < 11)
	{
		bActive = (m_xParams.m_uFlags & ASURA_DYNAMICLIGHTFLAG_ACTIVEATSTART) != 0;
	}

	SetEntityActiveFromLoad(bActive);
	SetEntityActive(bActive);
}

void Asura_ClientEntity_DynamicLight::OnEntityActivate()
{
	AddToUpdateableClientList(this);
	PARENT::OnEntityActivate();

	SetParams(m_xParams);
}

void Asura_ClientEntity_DynamicLight::OnEntityDeactivate()
{
	RemoveFromUpdateableClientList(this, false);
	PARENT::OnEntityDeactivate();

	SetParams(m_xParams);

	// If we've got any fades active, cancel them
	m_xState.m_bGraphFadeActive = false;				
	m_xState.m_fGraphFadePos    = 0.0f;
}

void Asura_ClientEntity_DynamicLight::Update()
{
	// If we have any active graphs, or if strobe is active, update the colour
	if (m_xState.m_bIsFadeActive || (m_xParams.m_uFlags & (ASURA_DYNAMICLIGHTFLAG_USEGRAPHRANGE | ASURA_DYNAMICLIGHTFLAG_USEGRAPHFLICKER
														  |ASURA_DYNAMICLIGHTFLAG_USEGRAPHCOLOUR | ASURA_DYNAMICLIGHTFLAG_USESTROBE)))
	{
		float fTimer = 0.0f;

		if (m_xParams.m_uFlags & ASURA_DYNAMICLIGHTFLAG_GRAPHISLOOPED)
		{
			// Looping graph - calculate graph time (0-1) to use
			fTimer = Asura_Maths::FloatMod ((Asura_Timers::GetLocalScaledTime()+m_xParams.m_fGraphTimerOffset)/m_xParams.m_fGraphDuration,1.0f);
		}
		else
		{
			// 'One-shot' graph - we're either clamped at one end or the other
			fTimer = m_xState.m_fGraphFadePos;

			if (m_xState.m_bGraphFadeActive)
			{
				float fStep = Asura_Timers::GetFrameTime()/m_xParams.m_fGraphDuration;

				m_xState.m_fGraphFadePos += (fStep * m_xState.m_fGraphFadeDir);

				if (m_xState.m_fGraphFadePos >=1.0f || m_xState.m_fGraphFadePos<=0.0f)
				{
					m_xState.m_fGraphFadePos = Asura_Maths::ClampToRange(m_xState.m_fGraphFadePos, 0.0f, 1.0f);
					m_xState.m_bGraphFadeActive = false;
				}
			}
		}

		if (m_xState.m_bIsFadeActive)
		{
			float fTime = Asura_Timers::GetLocalScaledTime();

			float fFade = 1.0f;
			
			if (m_xState.m_fFadeDuration>0.0f)
			{ 
				fFade = (fTime - m_xState.m_fFadeStart) / m_xState.m_fFadeDuration;
			}

			if (fFade>=1.0f)
			{
				m_xState.m_bIsFadeActive = false;

				if (m_xState.m_uFadeFlags & ASURA_DYNAMICLIGHT_FADE_FLAG_DEACTIVATE)
				{
					Asura_Message* pxMessage = Asura_Network::CreateMessage( ASURA_MSG_DEACTIVATE );
					if( pxMessage )
					{
						pxMessage->UseGuaranteedDelivery();
						Send( pxMessage, GetGuid() );
					}
				}
			}

			fFade = Asura_Maths::ClampToRange(fFade,0.0f,1.0f);
			float fOneMinusFade = 1.0f-fFade;

			m_xState.m_fCurrentBaseR		= m_xState.m_fDestR * fFade + m_xState.m_fSrcR * fOneMinusFade;
			m_xState.m_fCurrentBaseG		= m_xState.m_fDestG * fFade + m_xState.m_fSrcG * fOneMinusFade;
			m_xState.m_fCurrentBaseB		= m_xState.m_fDestB * fFade + m_xState.m_fSrcB * fOneMinusFade;
			m_xState.m_fCurrentBaseIntensity= m_xState.m_fDestIntensity * fFade + m_xState.m_fSrcIntensity * fOneMinusFade;
			m_xState.m_fCurrentBaseRange	= m_xState.m_fDestRange * fFade + m_xState.m_fSrcRange * fOneMinusFade;				
		}

		m_xState.m_fRed   = m_xState.m_fCurrentBaseR * m_xState.m_fCurrentBaseIntensity;
		m_xState.m_fGreen = m_xState.m_fCurrentBaseG * m_xState.m_fCurrentBaseIntensity;
		m_xState.m_fBlue  = m_xState.m_fCurrentBaseB * m_xState.m_fCurrentBaseIntensity;

		// Colour/Range fade
		Asura_RGBA xRGBA = m_xParams.m_xColourRangeGraph.GetValueAtTime(fTimer);

		if (m_xParams.m_uFlags & ASURA_DYNAMICLIGHTFLAG_USEGRAPHCOLOUR)
		{
			m_xState.m_fRed   *= xRGBA.m_fR;
			m_xState.m_fGreen *= xRGBA.m_fG;
			m_xState.m_fBlue  *= xRGBA.m_fB;
		}

		if (m_xParams.m_uFlags & ASURA_DYNAMICLIGHTFLAG_USEGRAPHRANGE)
		{
			m_xState.m_fRangeScale = xRGBA.m_fA;
		}

		// Flicker
		if (m_xParams.m_uFlags & ASURA_DYNAMICLIGHTFLAG_USEGRAPHFLICKER)
		{
			// (SB - 13/03/08) Scale flicker amount by time scale
			// This is so flickering stops when paused and perceptively the
			// flicker slows when timescale is decreased
			float fFlickerAmount = m_xParams.m_xFlickerGraph.GetValueAtTime(fTimer) * Asura_Timers::GetGameTimeScale();

			float fMin = Asura_Maths::ClampToRange(1.0f-fFlickerAmount,0.0f,1.0f);
			float fFlicker = Asura_Random::GetFloat(fMin, 1.0f);				

			m_xState.m_fRed   *= fFlicker;
			m_xState.m_fGreen *= fFlicker;
			m_xState.m_fBlue  *= fFlicker;
		}


		// Strobe
		if (m_xParams.m_uFlags & ASURA_DYNAMICLIGHTFLAG_USESTROBE)
		{
			if (Asura_Timers::GetLocalScaledTime()>m_xState.m_fStrobeTimer)
			{
				m_xState.m_bStrobeLightOn = !m_xState.m_bStrobeLightOn;

				if (m_xState.m_bStrobeLightOn)
				{
					m_xState.m_fStrobeTimer = Asura_Timers::GetLocalScaledTime() + Asura_Random::GetFloat(m_xParams.m_fStrobeMinTimeOn, m_xParams.m_fStrobeMaxTimeOn);
				}
				else
				{
					m_xState.m_fStrobeTimer = Asura_Timers::GetLocalScaledTime() + Asura_Random::GetFloat(m_xParams.m_fStrobeMinTimeOff, m_xParams.m_fStrobeMaxTimeOff);
				}
			}

			if (!m_xState.m_bStrobeLightOn)
			{
				m_xState.m_fRed = 
				m_xState.m_fGreen = 
				m_xState.m_fBlue = 0.0f;
			}

		}
		

		// Todo: Handle (none-graph) fades on/off
		UpdateLightColourAndRange();
	}
	if (m_xParams.m_uAttachedLightShaft != m_uLightShaftGUID)
	{
		UpdateLightShaft();
	}
}



void Asura_ClientEntity_DynamicLight::SetParams( const Asura_DynamicLight_Parameters& xNewParams )
{
	// This function will be called when the entity is created, and called constantly when it's selected in tools
	m_xParams = xNewParams;


	// Get the initial colour/range/intensity (fade messages can change these now)
	m_xState.m_fCurrentBaseR			= m_xParams.m_fBaseR;
	m_xState.m_fCurrentBaseG			= m_xParams.m_fBaseG;
	m_xState.m_fCurrentBaseB			= m_xParams.m_fBaseB;
	m_xState.m_fCurrentBaseIntensity	= m_xParams.m_fBaseIntensity;
	m_xState.m_fCurrentBaseRange		= m_xParams.m_fPhotonRange;	

	// Initial update of parameters
	m_xState.m_xPos			= m_xParams.m_xPosition;	
	m_xState.m_xOrient		= m_xParams.m_xOrientation;
	m_xState.m_fRangeScale  = 1.0f;
	m_xState.m_fRed			= m_xParams.m_fBaseR * m_xParams.m_fBaseIntensity;
	m_xState.m_fGreen		= m_xParams.m_fBaseG * m_xParams.m_fBaseIntensity;
	m_xState.m_fBlue		= m_xParams.m_fBaseB * m_xParams.m_fBaseIntensity;
		
	// Convert the custom light volume from object-relative into world space
	// This won't work for moving lights - if we want to move lights with custom volumes, this 
	// is going to need rethinking a bit..
	if ((m_xParams.m_uFlags & ASURA_DYNAMICLIGHTFLAG_USECUSTOMLIGHTVOLUME)
	 && !(m_xParams.m_uFlags & ASURA_DYNAMICLIGHTFLAG_WORLDSPACECLV))
	{
		Asura_Vector_3 xRelPos = m_xParams.m_xLightVolume.CalculateCentre();
		m_xParams.m_xLightVolume.Displace(-xRelPos);
		Asura_Vector_3 xAbsPos = m_xState.m_xPos + xRelPos * m_xState.m_xOrient;

		Asura_Matrix_3x3 xOrient;
		m_xParams.m_xLightVolumeOrient.ToMatrix(xOrient);		
		xOrient = xOrient * m_xState.m_xOrient;
		m_xParams.m_xLightVolumeOrient.FromMatrix(xOrient);		
		m_xParams.m_xLightVolume.Displace(xAbsPos);

		// Sometimes this function gets called multiple times on the same object. Set this flag to
		// ensure we don't perform the above transformation again
		m_xParams.m_uFlags |= ASURA_DYNAMICLIGHTFLAG_WORLDSPACECLV;
	}

	// Only do this if the light is active
	bool bDoActiveStuff = IsEntityActive();
#ifdef ASURA_TOOLS
	if (!bDoActiveStuff && Tools_System::IsActive() && Tools_Render_Options::ShouldForceDynamicLightsOn())
	{
		// in tools, pretend lights are active if they are standalone and start inactive, or if they are
		// in a template which starts inactive but they are themselves active.
		const Asura_ServerEntity_DynamicLight* pxServerEntity =  Asura_ServerEntity_DynamicLight::FindEntity(GetGuid());
		if(pxServerEntity && pxServerEntity->ToolsData)
		{
			if(pxServerEntity->ToolsData->IsMemberOfTemplate())
			{
				bDoActiveStuff = pxServerEntity->GetEntityActiveIgnoringTemplate();
			}
			else
			{
				bDoActiveStuff = true;
			}	
		}
	}
#endif
	if (bDoActiveStuff)
	{
		// Depending on the light type, we'll want to create and set up either a photon or projector
		switch (m_xParams.m_eType)
		{
			// ------------------------------------------------------------------------------------
			// Photon/Omni Light
			// ------------------------------------------------------------------------------------
			case ASURA_DYNAMICLIGHT_PHOTON:
			{			
				// If we've got a projector, get rid of it
				if (m_pxProjector)
				{
					Asura_Projector_System::DestroyProjector(m_pxProjector);		
					m_pxProjector=NULL;
				}

				// If we've not got a photon, create one
				if (!m_pxPhoton)
				{
					m_pxPhoton = Asura_Photon_System::CreatePhoton();
				}

				// Set photon parameters
				if (m_pxPhoton)
				{
					m_pxPhoton->SetAsActive(true);
					m_pxPhoton->SetRange(m_xParams.m_fPhotonRange);

					m_pxPhoton->SetFalloffPower(m_xParams.m_fFalloffPower);
					m_pxPhoton->SetFalloffScale(1.0f/(1.0f-m_xParams.m_fFalloffInnerRatio));

					m_pxPhoton->SetIsDirectional(false);

					if (m_xParams.m_uFlags & ASURA_DYNAMICLIGHTFLAG_USECUSTOMLIGHTVOLUME)
					{
						m_pxPhoton->SetCustomLightVolumeEnable(true);
						m_pxPhoton->SetCustomLightVolume(m_xParams.m_xLightVolume);
						m_pxPhoton->SetCustomLightVolumeOrient(m_xParams.m_xLightVolumeOrient);
					}
					else
					{
						m_pxPhoton->SetCustomLightVolumeEnable(false);
					}

					m_pxPhoton->SetDistanceFade(true, m_xParams.m_fFadeoutStart, m_xParams.m_fFadeoutDist);

					int iProjCubeTexture = Asura_Texture_Management::GetIndexFromHashID(m_xParams.m_xProjCubeTextureID);				
					m_pxPhoton->SetProjCubeTextureIndex(iProjCubeTexture);
					int iProjCubeBlurredTexture = Asura_Texture_Management::GetIndexFromHashID(m_xParams.m_xProjCubeBlurredTextureID);				
					m_pxPhoton->SetProjCubeBlurredTextureIndex(iProjCubeBlurredTexture);

					m_pxPhoton->SetProjCubeLerpDist(m_xParams.m_fProjCubeLerpDist);
				}

				break;
			}

			// ------------------------------------------------------------------------------------
			// Infinite Light (also a photon)
			// ------------------------------------------------------------------------------------
			case ASURA_DYNAMICLIGHT_INFINITE:
				{			
					// If we've got a projector, get rid of it
					if (m_pxProjector)
					{
						Asura_Projector_System::DestroyProjector(m_pxProjector);		
						m_pxProjector=NULL;
					}

					// If we've not got a photon, create one
					if (!m_pxPhoton)
					{
						m_pxPhoton = Asura_Photon_System::CreatePhoton();
					}

					// Set photon parameters
					if (m_pxPhoton)
					{
						m_pxPhoton->SetAsActive(true);
						m_pxPhoton->SetRange(m_xParams.m_fPhotonRange);

						m_pxPhoton->SetFalloffPower(m_xParams.m_fFalloffPower);
						m_pxPhoton->SetFalloffScale(1.0f/(1.0f-m_xParams.m_fFalloffInnerRatio));

						m_pxPhoton->SetIsDirectional(true);

						// Get the direction from the entity orientation
						m_pxPhoton->SetLightOrientation(m_xState.m_xOrient);

						if (m_xParams.m_uFlags & ASURA_DYNAMICLIGHTFLAG_USECUSTOMLIGHTVOLUME)
						{
							m_pxPhoton->SetCustomLightVolumeEnable(true);
							m_pxPhoton->SetCustomLightVolume(m_xParams.m_xLightVolume);
							m_pxPhoton->SetCustomLightVolumeOrient(m_xParams.m_xLightVolumeOrient);
						}
						else
						{
							m_pxPhoton->SetCustomLightVolumeEnable(false);
						}

						m_pxPhoton->SetDistanceFade(true, m_xParams.m_fFadeoutStart, m_xParams.m_fFadeoutDist);
					}

					break;
				}

			// ------------------------------------------------------------------------------------
			// Projector - Normal
			// ------------------------------------------------------------------------------------
			case ASURA_DYNAMICLIGHT_PROJECTOR:
			{			
				// If we've got a photon, remove it
				if (m_pxPhoton)
				{
					Asura_Photon_System::DestroyPhoton(m_pxPhoton);
					m_pxPhoton = NULL;
				}

				// If we've not got a projector, create one
				if (!m_pxProjector)
				{
					m_pxProjector=Asura_Projector_System::CreateProjector();
				}

				// Set projector parameters
				if (m_pxProjector)
				{
					int iTexture		= Asura_Texture_Management::GetIndexFromHashID(m_xParams.m_uTextureHash);
					int iDetailTexture  = Asura_Texture_Management::GetIndexFromHashID(m_xParams.m_uDetailTextureHash);

					m_pxProjector->m_iTextureIndex		 = iTexture;
					m_pxProjector->m_iDetailTextureIndex = iDetailTexture;
					m_pxProjector->m_fDetailTextureScale = m_xParams.m_fDetailTextureScale;

					// Update the frustum params
					m_pxProjector->m_fFoV	   = m_xParams.m_fFoV;
					m_pxProjector->m_fMinRange = m_xParams.m_fMinRange;
					m_pxProjector->m_fMaxRange = m_xParams.m_fMaxRange;
					m_pxProjector->m_eType     = (m_xParams.m_uFlags & ASURA_DYNAMICLIGHTFLAG_PARALLELPROJECTION)
												 ?ASURA_PROJECTOR_TYPE_PARALLEL:ASURA_PROJECTOR_TYPE_PERSPECTIVE;

					m_pxProjector->m_fUVAspect = m_xParams.m_fAspect;
					m_pxProjector->m_fScale    = m_xParams.m_fScale;

					// Update the Blending mode
					m_pxProjector->m_bDynamicSpotlight = false;
					if (m_xParams.m_iBlendMode>1)
					{
						m_pxProjector->m_bDynamicLight = false;
						m_pxProjector->m_bFakeLight = false;
						m_pxProjector->m_eBlendMode = static_cast<Asura_Projector_BlendMode>(m_xParams.m_iBlendMode-2);
					}
					else
					{
						if (m_xParams.m_iBlendMode==0)
						{
							m_pxProjector->m_bDynamicLight  = true;
							m_pxProjector->m_bFakeLight		= false;
						}
						else
						{
							m_pxProjector->m_bFakeLight		= true;
							m_pxProjector->m_bDynamicLight  = false;
						}
					}

					m_pxProjector->m_fDistFadeStart = m_xParams.m_fFadeoutStart;
					m_pxProjector->m_fDistFadeRange = m_xParams.m_fFadeoutDist;
					m_pxProjector->m_fIntensity     = 1.0f;

					m_pxProjector->m_fFalloffPower  = m_xParams.m_fFalloffPower;
					m_pxProjector->m_fFalloffScale  = 1.0f / (1.0f - m_xParams.m_fFalloffInnerRatio);

					m_pxProjector->m_fUScrollSpeed  = m_xParams.m_fUScrollSpeed;
					m_pxProjector->m_fVScrollSpeed  = m_xParams.m_fVScrollSpeed;
					m_pxProjector->m_fUScrollRange  = m_xParams.m_fUScrollRange;
					m_pxProjector->m_fVScrollRange  = m_xParams.m_fVScrollRange;
				}
				break;
			}


			// ------------------------------------------------------------------------------------
			// Projector - Spotlight
			// ------------------------------------------------------------------------------------
			case ASURA_DYNAMICLIGHT_SPOTLIGHT:
				{			
					// If we've got a photon, remove it
					if (m_pxPhoton)
					{
						Asura_Photon_System::DestroyPhoton(m_pxPhoton);
						m_pxPhoton = NULL;
					}

					// If we've not got a projector, create one
					if (!m_pxProjector)
					{
						m_pxProjector=Asura_Projector_System::CreateProjector();						
					}

					// Set projector parameters
					if (m_pxProjector)
					{
						// Update the frustum params
						m_pxProjector->SetSpotlightAngularFalloff(m_xParams.m_fInnerFoV,m_xParams.m_fFoV);
						m_pxProjector->m_fFoV	   = m_xParams.m_fFoV;
						m_pxProjector->m_fMinRange = m_xParams.m_fMinRange;
						m_pxProjector->m_fMaxRange = m_xParams.m_fMaxRange;
						m_pxProjector->m_eType     = ASURA_PROJECTOR_TYPE_PERSPECTIVE;
						
						// Update the Blending mode
						m_pxProjector->m_bDynamicLight	   = false;
						m_pxProjector->m_bDynamicSpotlight = true;
						
						m_pxProjector->m_fDistFadeStart = m_xParams.m_fFadeoutStart;
						m_pxProjector->m_fDistFadeRange = m_xParams.m_fFadeoutDist;
						m_pxProjector->m_fIntensity     = 1.0f;

						m_pxProjector->m_fFalloffPower  = m_xParams.m_fFalloffPower;
						m_pxProjector->m_fFalloffScale  = 1.0f/(1.0f-m_xParams.m_fFalloffInnerRatio);
					}
					break;
				}
		}


		// Set the shadow parameters
		if (m_pxPhoton)
		{
			m_pxPhoton->SetShadowParameters(m_xParams.m_xShadowParams);

			// If we have a custom light volume, set that too
		}

		if (m_pxProjector)
		{
			m_pxProjector->SetShadowParameters(m_xParams.m_xShadowParams);		
		}
		
		UpdatePosition();		
		UpdateCorona();
		UpdateLightColourAndRange();		
	}
	else
	{
		// If the light is inactive and this function is called, it's being called to turn
		// the light off (via a message), so remove any photon/projector
		if (m_pxProjector)
		{
			Asura_Projector_System::DestroyProjector(m_pxProjector);		
			m_pxProjector=NULL;
		}
		
		if (m_pxPhoton)
		{
			Asura_Photon_System::DestroyPhoton(m_pxPhoton);
			m_pxPhoton = NULL;
		}

		if(m_iCoronaHandle != ASURA_INVALID_HANDLE)
		{
			Asura_Corona_System::DestroyCorona(m_iCoronaHandle);
			m_iCoronaHandle = ASURA_INVALID_HANDLE;
		}

		if (m_pxLightShaft)
		{
			m_pxLightShaft->SetEnable(false);
		}
	}

}



void Asura_ClientEntity_DynamicLight::SetPositionAndOrientation(const Asura_Vector_3 & xPos, const Asura_Matrix_3x3& xMat)
{
	//I guess updating the dynamic light params will be best , so any swining etc can be applied on top
	//RAR 21/06/2007
	m_xParams.m_xPosition = xPos;
	m_xParams.m_xOrientation = xMat;
	UpdatePosition();
}

void Asura_ClientEntity_DynamicLight::UpdatePosition()
{	

	// Set the default position/orientation
	// ----------------------------------------------------------------------------------------
	m_xState.m_xPos	   = m_xParams.m_xPosition;
	m_xState.m_xOrient = m_xParams.m_xOrientation;



	// Apply code-driven movement (simple swinging/rotation for projectors)
	// ----------------------------------------------------------------------------------------
	if (m_xParams.m_uFlags & ASURA_DYNAMICLIGHTFLAG_MOVING)
	{
		Asura_Euler_3		xAngles;
		Asura_Matrix_3x3	xRot;

		float fTime  = Asura_Timers::GetLocalScaledTime();
		float fPhase = m_xParams.m_fRotationOffset;//there used to be a deg to rad conversion here, despite the fact this is a time offset in seconds...

		//loop through each axis
		for (u_int u = 0; u < 3; ++u)
		{
			xAngles[u] = (fPhase + fTime) * m_xParams.m_xRotSpeed[u]; 

			if (m_xParams.m_abSwing[u])
			{				
				xAngles[u] = m_xParams.m_xSwingDist[u] * Asura_Maths::Sine(xAngles[u]);
			}
		}

		xAngles.ToMatrix(xRot);

		m_xState.m_xOrient = m_xState.m_xOrient * xRot;
	}


	// Handle attachment to a physical object
	// ----------------------------------------------------------------------------------------

	if(m_xParams.m_uMasterEntity != ASURA_GUID_UNREGISTERED)
	{
		// This is a a pain, because most client entities don't exist in tools... and it's the clients that run the anims...		
		Asura_ServerEntity_PhysicalObject * pxMasterSvr	   = Asura_ServerEntity_PhysicalObject::FindEntity(m_xParams.m_uMasterEntity);
		Asura_ClientEntity_PhysicalObject * pxMasterClient = Asura_ClientEntity_PhysicalObject::FindEntity(m_xParams.m_uMasterEntity);

		if(pxMasterSvr || pxMasterClient)
		{	
			Asura_Matrix_3x3 xMasterOrient;
			Asura_Vector_3   xMasterPos;

			// Take whatever position we can find... client is preferred, but isn't available in tools
			bool bNoMaster = false;

			if (pxMasterClient)
			{				
				xMasterOrient = pxMasterClient->GetCurrentOrientation();
				xMasterPos	  = pxMasterClient->GetCurrentPosition();				
				bNoMaster	  = pxMasterClient->IsDestroyed();
			}
			else
			{				
				xMasterOrient = pxMasterSvr->GetOrientation();
				xMasterPos	  = pxMasterSvr->GetPosition();
				bNoMaster	  = pxMasterSvr->IsDestroyed();
			}
		
			
			if (bNoMaster && m_xState.m_bFoundMaster)
			{
				// If we're attached to an entity that's been destroyed, deactivate ourself
				m_xState.m_bSleeping = true;
				m_xState.m_bFoundMaster = false;
				Asura_Message* pxMessage = Asura_Network::CreateMessage( ASURA_MSG_DEACTIVATE );
				if( pxMessage )
				{
					pxMessage->UseGuaranteedDelivery();
					Send( pxMessage, GetGuid() );
				}
			}
			else
			{	
				m_xState.m_bFoundMaster = true;
				m_xState.m_bSleeping = false;
				if (!IsEntityActive())
				{
					Asura_Message* pxMessage = Asura_Network::CreateMessage( ASURA_MSG_ACTIVATE );
					if( pxMessage )
					{
						pxMessage->UseGuaranteedDelivery();
						Send( pxMessage, GetGuid() );
					}
				}

#ifdef ASURA_TOOLS
				// This is pretty horrible... we've got to search for all 3 entity objects (tools, client, server) - as
				// the client probably won't exist in editing mode, and the anim is played by the tools object...
				//
				// (All 3 will contain the entity position, but there's no guarantee that they're all the same/valid...)

				Tools_Entity_Object *pxToolsEnt = Tools_Entity_Object::FindEntity(m_xParams.m_uMasterEntity);						

				if (m_xParams.m_uMasterMarker && pxToolsEnt && pxToolsEnt->GetAnimation())
				{
					pxToolsEnt->GetAnimation()->Update(pxToolsEnt->Position, pxToolsEnt->Orientation, ANIMTIME_USEGAMETIME, ASURA_HIERARCHY_RENDER_DONTRENDER | ASURA_HIERARCHY_RENDER_UPDATEBONESALWAYS );
					pxToolsEnt->GetAnimation()->GetMarkerLocationFromHash(m_xParams.m_uMasterMarker,&m_xState.m_xPos, &m_xState.m_xOrient,true);
				}
				else
#endif
				{				
					if(m_xParams.m_uMasterMarker && pxMasterClient)
					{									
						pxMasterClient->GetMarkerLocationFromHash(m_xParams.m_uMasterMarker, &m_xState.m_xPos, &m_xState.m_xOrient);
					}
					else
					{				
						m_xState.m_xPos	   = m_xParams.m_xOffsetFromMaster;									
						m_xState.m_xPos	   = m_xState.m_xPos * xMasterOrient;
						m_xState.m_xPos	   = xMasterPos + m_xState.m_xPos;
						m_xState.m_xOrient = xMasterOrient; 
					}
				}
			}
		}
		else
		{
			if (!g_bReadingFromChunkStream)
			{
				// If we're attached to an entity that's been destroyed, deactivate ourself
				m_xState.m_bSleeping = true;
				Asura_Message* pxMessage = Asura_Network::CreateMessage( ASURA_MSG_DEACTIVATE );
				if( pxMessage )
				{
					pxMessage->UseGuaranteedDelivery();
					Send( pxMessage, GetGuid() );
				}
			}
		}
	}



	// Now set the pos/orient on the light+projector+corona
	// ----------------------------------------------------------------------------------------
	if(m_pxPhoton)
	{
		m_pxPhoton->SetPosition(m_xState.m_xPos);

		m_pxPhoton->SetLightOrientation(m_xState.m_xOrient);

		if (m_xParams.m_uFlags & ASURA_DYNAMICLIGHTFLAG_USEPHOTONLIMITINGANGLES)
		{
			m_pxPhoton->SetLimitAnglesEnabled(true);

			float fInner = m_xParams.m_fLimitingAngleInner;
			float fOuter = m_xParams.m_fLimitingAngleOuter;

			fInner = Asura_Maths::Cosine(0.5f*fInner);
			fOuter = Asura_Maths::Cosine(0.5f*fOuter);

			m_pxPhoton->SetLimitAngleInner(fInner);
			m_pxPhoton->SetLimitAngleOuter(fOuter);
		}
		else
		{		
			m_pxPhoton->SetLimitAnglesEnabled(false);
		}
	}

	if (m_pxProjector)
	{
		if (m_xParams.m_uFlags & ASURA_DYNAMICLIGHTFLAG_USELIMITINGPLANE)
		{
			m_pxProjector->EnableLimitingPlane(true);
			Asura_Vector_3 xPos = m_xParams.m_xLimitingPlanePosition+m_xState.m_xPos;
			m_pxProjector->SetLimitingPlane(xPos, m_xParams.m_xLimitingPlaneDirection);
		}
		else
		{
			m_pxProjector->EnableLimitingPlane(false);
		}

		m_pxProjector->SetPosition(m_xState.m_xPos);
		m_pxProjector->SetOrientation(m_xState.m_xOrient);
		m_pxProjector->UpdateVertices();
	}

	if(m_iCoronaHandle != ASURA_INVALID_HANDLE)
	{
		Asura_Corona_System::SetPosition(m_iCoronaHandle, m_xState.m_xPos);
	}

	UpdateLightShaft();
}


void Asura_ClientEntity_DynamicLight::UpdateLightShaft()
{
	// Handle attached lightshafts
	// ----------------------------------------------------------------------------------------
	if (m_xParams.m_uAttachedLightShaft != m_uLightShaftGUID)
	{		

		Asura_ClientEntity_LightShaft *pxShaft = Asura_ClientEntity_LightShaft::FindEntity(m_uLightShaftGUID);
		if (pxShaft)
		{
			pxShaft->DetachFromLight();

			if (m_pxLightShaft)
			{
				m_pxLightShaft = NULL;
				m_uLightShaftGUID = ASURA_GUID_UNREGISTERED;
			}
		}

		if (m_xParams.m_uAttachedLightShaft != ASURA_GUID_UNREGISTERED)
		{
			pxShaft = Asura_ClientEntity_LightShaft::FindEntity(m_xParams.m_uAttachedLightShaft);

			if (pxShaft)
			{			
				m_pxLightShaft = pxShaft->GetLightShaft();

				if (m_pxLightShaft)
				{			
					pxShaft->AttachToLight();

					m_uLightShaftGUID = m_xParams.m_uAttachedLightShaft;
					UpdatePosition();
				}
			}
		}		
	}

	if (m_pxLightShaft)
	{
		m_pxLightShaft->SetPosition(m_xState.m_xPos);
		m_pxLightShaft->SetOrientation(m_xState.m_xOrient);
		m_pxLightShaft->UpdateBoundingBox();
		m_pxLightShaft->SetEnable(IsEntityActive());
	}
}


void Asura_ClientEntity_DynamicLight::UpdateCorona()
{
	if(m_xParams.m_uFlags & ASURA_DYNAMICLIGHTFLAG_HASCORONA)
	{
		if(m_iCoronaHandle == ASURA_INVALID_HANDLE)
		{
			m_iCoronaHandle = Asura_Corona_System::CreateCorona();
		}
		
		int iTextureIndex = Asura_Texture_Management::GetIndexFromHashID(m_xParams.m_uCoronaTextureHash);
		
		// Update the corona values
		Asura_Corona_System::SetPosition(m_iCoronaHandle, m_xParams.m_xPosition);
		Asura_Corona_System::SetRadius(m_iCoronaHandle, m_xParams.m_fCoronaRadius);
		Asura_Corona_System::SetTextureIndex(m_iCoronaHandle,iTextureIndex);
		Asura_Corona_System::SetFlag_DontFadeWithDistance(m_iCoronaHandle, (m_xParams.m_uFlags & ASURA_DYNAMICLIGHTFLAG_DONTFADECORONAWITHDIST)!=0 );
		Asura_Corona_System::SetFlag_UseModuleLinking(m_iCoronaHandle, (m_xParams.m_uFlags & ASURA_DYNAMICLIGHTFLAG_CORONAUSESMODLINKING)!=0 );
		Asura_Corona_System::SetVolumetricDepthScale(m_iCoronaHandle,m_xParams.m_fCoronaVolumetricDepthScale);
		
		// Note - Corona colour not set yet - needs a call to UpdateLightColour
	}
	else if(m_iCoronaHandle != ASURA_INVALID_HANDLE)
	{
		Asura_Corona_System::DestroyCorona(m_iCoronaHandle);
		m_iCoronaHandle = ASURA_INVALID_HANDLE;
	}
}



void Asura_ClientEntity_DynamicLight::UpdateLightColourAndRange()
{

	// If we've got a photon, update it's colour
	if(m_pxPhoton)
	{
		m_pxPhoton->SetColours(m_xState.m_fRed, m_xState.m_fGreen, m_xState.m_fBlue);
		m_pxPhoton->SetRange(m_xState.m_fCurrentBaseRange * m_xState.m_fRangeScale);
	}

	// If we've got a projector, update it's colour
	if (m_pxProjector)
	{
		m_pxProjector->SetRGB(m_xState.m_fRed, m_xState.m_fGreen, m_xState.m_fBlue);		
		m_pxProjector->m_fFoV = m_xParams.m_fFoV * m_xState.m_fRangeScale;

		// If it's a spotlight, we need to scale it's inner FoV, too.
		if (m_pxProjector->m_bDynamicSpotlight)
		{		
			m_pxProjector->SetSpotlightAngularFalloff(m_xParams.m_fInnerFoV * m_xState.m_fRangeScale,m_xParams.m_fFoV * m_xState.m_fRangeScale);
		}

		m_pxProjector->UpdateVertices();
	}

	// If we've got a corona, update that, too
	if(m_iCoronaHandle != ASURA_INVALID_HANDLE)
	{
		// (This comes straight from the old DestructibleLight)
		float fCoronaRed   = Asura_Maths::ClampToRange(m_xState.m_fRed   * 2.0f, 0.0f, 1.0f);
		float fCoronaGreen = Asura_Maths::ClampToRange(m_xState.m_fGreen * 2.0f, 0.0f, 1.0f);
		float fCoronaBlue  = Asura_Maths::ClampToRange(m_xState.m_fBlue  * 2.0f, 0.0f, 1.0f);		
		Asura_Corona_System::SetColour(m_iCoronaHandle, fCoronaRed, fCoronaGreen, fCoronaBlue);
	}

	// And the attached lightshaft
	if (m_pxLightShaft)
	{
		Asura_RGBA xColour = Asura_RGBA(m_xState.m_fRed, m_xState.m_fGreen, m_xState.m_fBlue, 1);
		m_pxLightShaft->SetInheritedColour(xColour);
	}
}



void Asura_ClientEntity_DynamicLight::UpdatePositionAfterEntityUpdate()
{
	if (IsEntityActive() || m_xState.m_bSleeping)
	{	
		// If we're moving or attached to an entity, we need to update the pos/orient of
		// the light
		if (m_xParams.m_uFlags & ASURA_DYNAMICLIGHTFLAG_MOVING ||
			m_xParams.m_uMasterEntity != ASURA_GUID_UNREGISTERED ||
			m_xParams.m_uAttachedLightShaft != m_uLightShaftGUID)
		{	
			UpdatePosition();
		}
	}
}
Asura_Handle Asura_ClientEntity_DynamicLight::GetDynamicLightSourceHandle() const
{ 
	switch (GetDynamicLightType())
	{
	case ASURA_DYNAMICLIGHT_PHOTON:
		return m_pxPhoton ? m_pxPhoton->GetHandle() : ASURA_INVALID_HANDLE;
	case ASURA_DYNAMICLIGHT_PROJECTOR:
		return m_pxProjector ? (Asura_Handle)m_pxProjector->GetHandle() : ASURA_INVALID_HANDLE;
	case ASURA_DYNAMICLIGHT_SPOTLIGHT:
		return m_pxPhoton ? m_pxPhoton->GetHandle() : m_pxProjector ? m_pxProjector->GetHandle() : ASURA_INVALID_HANDLE;
	default:
		return ASURA_INVALID_HANDLE;
	}
}

const Asura_Vector_3 &Asura_ClientEntity_DynamicLight::GetDynamicLightPosition() const
{ 
	switch (GetDynamicLightType())
	{
	case ASURA_DYNAMICLIGHT_PHOTON:
		return m_pxPhoton ? m_pxPhoton->GetPosition() : Asura_Maths::ZeroVector3;
	case ASURA_DYNAMICLIGHT_PROJECTOR:
		return m_pxProjector ? m_pxProjector->GetPosition() : Asura_Maths::ZeroVector3;
	case ASURA_DYNAMICLIGHT_SPOTLIGHT:
		return m_pxPhoton ? m_pxPhoton->GetPosition() : m_pxProjector ? m_pxProjector->GetPosition() : Asura_Maths::ZeroVector3;
	default:
		return Asura_Maths::ZeroVector3;
	}
}

const Asura_Matrix_3x3 &Asura_ClientEntity_DynamicLight::GetDynamicLightOrientation() const
{ 
	switch (GetDynamicLightType())
	{
	case ASURA_DYNAMICLIGHT_PHOTON:
		return m_pxPhoton ? m_pxPhoton->GetLightOrientation() : Asura_Maths::IdentityMatrix3x3;
	case ASURA_DYNAMICLIGHT_PROJECTOR:
		return m_pxProjector ? m_pxProjector->GetOrient() : Asura_Maths::IdentityMatrix3x3;
	case ASURA_DYNAMICLIGHT_SPOTLIGHT:
		return m_pxPhoton ? m_pxPhoton->GetLightOrientation() : m_pxProjector ? m_pxProjector->GetOrient() : Asura_Maths::IdentityMatrix3x3;
	default:
		return Asura_Maths::IdentityMatrix3x3;
	}
}



void Asura_ClientEntity_DynamicLight::RenderLightVolume()
{
	// Projectors - can render their own frustum
	// ----------------------------------------------------------------------------------------
	if (m_pxProjector)
	{	
		m_pxProjector->RenderFrustum();
	}

	// Photons - render them as an AABB, do our own distance-fade test here
	// ----------------------------------------------------------------------------------------
	if (m_pxPhoton)
	{
		m_pxPhoton->RenderLightVolume();
	}
}




bool Asura_ClientEntity_DynamicLight::HandleMessage(const Asura_Message* pxMessage) 
{
	switch (pxMessage->GetMsgID())
	{			
		case ASURA_MSG_TRIGGER:
			// Play fade graph - one-shot, forwards
			m_xState.m_bGraphFadeActive = true;				
			m_xState.m_fGraphFadeDir	   = 1.0f;

			if (m_xState.m_fGraphFadePos<=0.0f || m_xState.m_fGraphFadePos>=1.0f)
			{
				m_xState.m_fGraphFadePos    = 0.0f;
			}
			
			return true;

		case ASURA_MSG_UNTRIGGER:
			// Play fade graph - one-shot, backwards
			m_xState.m_bGraphFadeActive = true;				
			m_xState.m_fGraphFadeDir	   =-1.0f;
			
			return true;
		
		case ASURA_MSG_DYNAMICLIGHT_FADE:
			{
				const Asura_Message_DynamicLight_Fade*	pxData	= static_cast<const Asura_Message_DynamicLight_Fade*>(pxMessage->GetData());
							
				m_xState.m_fSrcIntensity	 = m_xState.m_fDestIntensity	= m_xState.m_fCurrentBaseIntensity;
				m_xState.m_fSrcRange		 = m_xState.m_fDestRange		= m_xState.m_fCurrentBaseRange;
				m_xState.m_fSrcR			 = m_xState.m_fDestR			= m_xState.m_fCurrentBaseR;
				m_xState.m_fSrcG			 = m_xState.m_fDestG			= m_xState.m_fCurrentBaseG;
				m_xState.m_fSrcB			 = m_xState.m_fDestB			= m_xState.m_fCurrentBaseB;

				m_xState.m_uFadeFlags    = pxData->m_uFlags;				
				m_xState.m_fFadeStart    = Asura_Timers::GetLocalScaledTime();
				m_xState.m_fFadeDuration = pxData->m_fDuration;
				m_xState.m_bIsFadeActive = true;

				if (m_xState.m_uFadeFlags & ASURA_DYNAMICLIGHT_FADE_FLAG_USECOLOUR)
				{
					m_xState.m_fDestR = pxData->m_fTargetR;	
					m_xState.m_fDestG = pxData->m_fTargetG;
					m_xState.m_fDestB = pxData->m_fTargetB;
				}

				if (m_xState.m_uFadeFlags & ASURA_DYNAMICLIGHT_FADE_FLAG_USEINTENSITY)
				{
					m_xState.m_fDestIntensity = pxData->m_fTargetIntensity;						
				}

				if (m_xState.m_uFadeFlags & ASURA_DYNAMICLIGHT_FADE_FLAG_USERANGE)
				{
					m_xState.m_fDestRange = pxData->m_fTargetRange;
				}

				// Turn it on, if it's not already active
				if (m_xState.m_uFadeFlags & ASURA_DYNAMICLIGHT_FADE_FLAG_ACTIVATE)
				{
					if (!IsEntityActive())
					{
						Asura_Message* pxActivateMessage = Asura_Network::CreateMessage( ASURA_MSG_ACTIVATE );
						if( pxActivateMessage )
						{
							pxActivateMessage->UseGuaranteedDelivery();
							Send( pxActivateMessage, GetGuid() );
						}
					}
				}

				return true;
			}

			

		default:
			return PARENT::HandleMessage(pxMessage);
	}	
}





void Asura_ClientEntity_DynamicLight::SetState(Asura_ClientEntity_DynamicLight_State *pxState)
{
	m_xState = *pxState;	

	UpdatePosition();
	UpdateLightColourAndRange();
	UpdateCorona();
	UpdateLightShaft();
}



#ifdef ASURA_TOOLS
void Asura_ClientEntity_DynamicLight::ApplyChangesForTemplateInstance(class Tools_EntityTemplateInstance* pxInstance)
{
	m_xParams.m_xPosition			= pxInstance->TransformPosition(m_xParams.m_xPosition);
	m_xParams.m_xOrientation		= pxInstance->TransformMatrix(m_xParams.m_xOrientation);
	m_xParams.m_uMasterEntity		= pxInstance->ConvertGuid(m_xParams.m_uMasterEntity);	
	m_xParams.m_uAttachedLightShaft	= pxInstance->ConvertGuid(m_xParams.m_uAttachedLightShaft);	
	m_xParams.m_xOffsetFromMaster	= m_xParams.m_xOffsetFromMaster * pxInstance->GetRelativeMatrix();
}
#endif // ASURA_TOOLS




void Asura_ClientEntity_DynamicLight_State::WriteToChunkStream(Asura_Chunk_Stream& xStream) const
{
	xStream << m_uDynamicLight_Flags;

	xStream << m_fRed;
	xStream << m_fGreen;
	xStream << m_fBlue;
	xStream << m_fRangeScale;
	xStream << m_xPos;
	xStream << m_xOrient;

	xStream << m_fStrobeTimer;

	xStream << m_fGraphFadeDir;
	xStream << m_fGraphFadePos;

	float fFadeProgress = m_fFadeStart - Asura_Timers::GetLocalScaledTime();
	xStream << fFadeProgress;
	xStream << m_fFadeDuration;
	xStream << m_uFadeFlags;

	xStream << m_fSrcR;
	xStream << m_fSrcG;
	xStream << m_fSrcB;
	xStream << m_fSrcIntensity;
	xStream << m_fSrcRange;
	xStream << m_fDestR;
	xStream << m_fDestG;
	xStream << m_fDestB;
	xStream << m_fDestIntensity;
	xStream << m_fDestRange;

	xStream << m_fCurrentBaseR;
	xStream << m_fCurrentBaseG;
	xStream << m_fCurrentBaseB;
	xStream << m_fCurrentBaseIntensity;
	xStream << m_fCurrentBaseRange;	
}


void Asura_ClientEntity_DynamicLight_State::ExtractFromChunkStream(Asura_Chunk_Stream& xStream, uint32 uVersion)
{
	xStream >> m_uDynamicLight_Flags;

	xStream >> m_fRed;
	xStream >> m_fGreen;
	xStream >> m_fBlue;
	xStream >> m_fRangeScale;
	xStream >> m_xPos;
	xStream >> m_xOrient;

	xStream >> m_fStrobeTimer;
	//Reset the stored strobe timer , since I don't think the saved value will be good for anything
	//Since it will have been based on how long the game had been running when the save was done.
	//RAR 04/02/2008
	m_fStrobeTimer = 0.0f;

	xStream >> m_fGraphFadeDir;
	xStream >> m_fGraphFadePos;

	float fFadeProgress;
	xStream >> fFadeProgress;
	m_fFadeStart = fFadeProgress + Asura_Timers::GetLocalScaledTime();

	xStream >> m_fFadeDuration;
	xStream >> m_uFadeFlags;

	xStream >> m_fSrcR;
	xStream >> m_fSrcG;
	xStream >> m_fSrcB;
	xStream >> m_fSrcIntensity;
	xStream >> m_fSrcRange;
	xStream >> m_fDestR;
	xStream >> m_fDestG;
	xStream >> m_fDestB;
	xStream >> m_fDestIntensity;
	xStream >> m_fDestRange;

	xStream >> m_fCurrentBaseR;
	xStream >> m_fCurrentBaseG;
	xStream >> m_fCurrentBaseB;
	xStream >> m_fCurrentBaseIntensity;
	xStream >> m_fCurrentBaseRange;		
}

#ifdef ASURA_TOOLS
void Asura_ClientEntity_DynamicLight::HandleForceActiveChange()
{
	for(Asura_Hashed_List_It<Asura_ClientEntity_DynamicLight> xIt(&s_xList); !xIt.Done(); xIt.Next())
	{
		Asura_ClientEntity_DynamicLight* pxEntity = xIt.GetCurrent();

		//call SetParams() again to force the light to re-evaluate its active status
		pxEntity->SetParams(pxEntity->m_xParams);
	}
}
#endif //ASURA_TOOLS



#endif //#ifdef ASURA_TARGET_SM3
