
//////////////////////////////////////////////////////////////////////////////
//
// Asura_ClientEntity_DynamicLight.h
//
// Copyright (c) 2007 Rebellion (Derby). All rights reserved.
//
// Description:
//
// The all-new dynamic light entity
//
//////////////////////////////////////////////////////////////////////////////


#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef ___ASURA_CLIENTENTITY_DYNAMICLIGHT_H___
#define ___ASURA_CLIENTENTITY_DYNAMICLIGHT_H___ 1

#ifdef ASURA_TARGET_SM3


// Engine independent.
#include "Entity/Asura_Entity.h"

// Project independent.
#include "Entity/DynamicLight/Asura_ServerEntity_DynamicLight.h"
#include "SpecialFX/Asura_Corona_System.h"


class Asura_LightShaft;
class Asura_Photon;
class Asura_Projector;


// This is the current state of the light - not maintained by the
// server and needs saving at checkpoints
//
// It could also be put in an update/on create message if there's issues with dynamic lights in multiplayer games.
//
// I can't think of many cases  where this would actually be needed, though - as destructible lights should 'just work' - the
// light will turn off when the object it's attached to dies (which should be synced across the network already). It's only
// likely to be a problem if there's scripted events in MP levels turning lights on and off?

class Asura_ClientEntity_DynamicLight_State
{

public:

	void WriteToChunkStream(Asura_Chunk_Stream& xStream) const;
	void ExtractFromChunkStream(Asura_Chunk_Stream& xStream, uint32 uVersion);


	union
	{
		u_int m_uDynamicLight_Flags;
		struct
		{
			u_int		m_bStrobeLightOn			: 1,
						m_bIsFadeActive				: 1,
						m_bFoundMaster				: 1,
						m_bSleeping					: 1,
						m_bGraphFadeActive			: 1,
						m_bDynamicLightsUnusedBits	: 27;
		};
	};

	float				m_fRed;
	float				m_fGreen;
	float				m_fBlue;
	float				m_fRangeScale;
	Asura_Vector_3		m_xPos;
	Asura_Matrix_3x3	m_xOrient;

	float				m_fStrobeTimer;

	float				m_fGraphFadeDir;
	float				m_fGraphFadePos;

	// Fading support
	float				m_fFadeStart;
	float				m_fFadeDuration;
	u_int				m_uFadeFlags;

	float				m_fSrcR;
	float				m_fSrcG;
	float				m_fSrcB;
	float				m_fSrcIntensity;
	float				m_fSrcRange;
	float				m_fDestR;
	float				m_fDestG;
	float				m_fDestB;
	float				m_fDestIntensity;
	float				m_fDestRange;

	float				m_fCurrentBaseR;
	float				m_fCurrentBaseG;
	float				m_fCurrentBaseB;
	float				m_fCurrentBaseIntensity;
	float				m_fCurrentBaseRange;	
};





class Asura_ClientEntity_DynamicLight : public Asura_Entity_Client
{
friend class Asura_ServerEntity_DynamicLight;

public:
	// C O N S T R U C T O R S   /   D E S T R U C T O R S ///////////////        
	Asura_ClientEntity_DynamicLight( Asura_Guid uGuid);

#ifdef ASURA_TOOLS
	static Asura_ClientEntity_DynamicLight* CreateFromTools( Asura_Guid uGuid, const Asura_DynamicLight_Parameters &xParams);
#endif
	static Asura_ClientEntity_DynamicLight* CreateFromChunkStream(Asura_Guid uGuid, Asura_Chunk_Stream &xStream);

	virtual ~Asura_ClientEntity_DynamicLight();

	// C L A S S   M E T H O D S /////////////////////////////////////////

	// M E T H O D S /////////////////////////////////////////////////////

	virtual	void ReadFromServerLightChunkStream(Asura_Chunk_Stream& xStream);

	virtual void Update();
	virtual bool HandleMessage(const Asura_Message* pxMessage);			
	void		 SetParams( const Asura_DynamicLight_Parameters& xNewParams );
	void		 MakeClientUpdateMessage() {}

	void		 RenderLightVolume();

	//Functions for allowing cutscene attachments to control the client entities. RAR 21/06/2007
	void         SetPositionAndOrientation(const Asura_Vector_3 & xPos, const Asura_Matrix_3x3& xMat);

	void		 UpdatePositionAfterEntityUpdate();

	void								   SetState(Asura_ClientEntity_DynamicLight_State *pxState);
	Asura_ClientEntity_DynamicLight_State *GetState(void) {return &m_xState;}

	#ifdef ASURA_TOOLS
	//causes all the lights to be updated again to handle changes to the tools
	//'force dynamic light on' flag.
	static void HandleForceActiveChange();
	#endif
	bool CastsEnvironmentShadow() const { return (m_xParams.m_xShadowParams.m_uFlags & ASURA_SHADOWFLAG_CASTS_ENVIRONMENT_SHADOWS) == ASURA_SHADOWFLAG_CASTS_ENVIRONMENT_SHADOWS; }
	ASURA_DYNAMICLIGHT_TYPE GetDynamicLightType() const { return m_xParams.m_eType; }
	const Asura_DynamicLight_Parameters& GetDynamicLightParams() const { return m_xParams; }
	Asura_Handle			GetDynamicLightSourceHandle() const;
	const Asura_Vector_3&	GetDynamicLightPosition() const;
	const Asura_Matrix_3x3& GetDynamicLightOrientation() const;
	const Asura_Photon*		GetPhotonPtr() const { return m_pxPhoton; }
	const Asura_Projector*	GetProjectorPtr() const { return m_pxProjector; }

protected:

	virtual	void OnEntityActivate();
	virtual	void OnEntityDeactivate();

	virtual bool AllowOverwriteOnLoad() const		{ return true; }

	void	 	 UpdateLightShaft();
	void	 	 UpdateCorona();
	void		 UpdateLightColourAndRange();
	void		 UpdatePosition();		

private:

	// Current state - taking into account any effects
	Asura_ClientEntity_DynamicLight_State m_xState;
	

	// S T A T E /////////////////////////////////////////////////////////
	Asura_DynamicLight_Parameters				m_xParams;

	Asura_Photon*								m_pxPhoton;
	Asura_Projector*							m_pxProjector;
	Asura_Handle								m_iCoronaHandle;

	Asura_Guid									m_uLightShaftGUID;
	Asura_LightShaft*							m_pxLightShaft;		// Attached lightshaft, if we have one


	// T Y P E S /////////////////////////////////////////////////////////
	typedef Asura_Entity_Client PARENT;

#ifdef ASURA_TOOLS
	void ApplyChangesForTemplateInstance(class Tools_EntityTemplateInstance* pxInstance);
#endif

public:
	static Asura_Hashed_List<Asura_ClientEntity_DynamicLight>* GetList()
	{
		return &s_xList;
	}

	static Asura_ClientEntity_DynamicLight* FindEntity(Asura_Guid uGuid)
	{
		return s_xList.FindEntity(uGuid);
	}

private:
	static Asura_Hashed_List<Asura_ClientEntity_DynamicLight> s_xList;
};

// End class SS2_ClientEntity_DynamicLight


#endif // #ifdef ASURA_TARGET_SM3
#endif // ___ASURA_CLIENTENTITY_DYNAMICLIGHT_H___
