//////////////////////////////////////////////////////////////////////////////
//
// Asura_ServerEntity_DynamicLight.h
//
// Copyright (c) 2007 Rebellion (Derby). All rights reserved.
//
// Description:
//
// The all-new dynamic light entity
//
//////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef ___ASURA_SERVERENTITY_DYNAMICLIGHT_H___
#define ___ASURA_SERVERENTITY_DYNAMICLIGHT_H___ 1

#ifdef ASURA_TARGET_SM3

// I N C L U D E S ///////////////////////////////////////////////////////////

// Engine independent.
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "Core/Asura_String.h"
#include "Core/Asura_Graph.h"
#include "Rendering/Asura_Colour.h"
#include "Entity/Asura_Entity.h"
#include "Shadows/Asura_Shadow_System.h"
#include "SpecialFX/Asura_Corona_System.h"

// E N U M S /////////////////////////////////////////////////////////////////

enum ASURA_DYNAMICLIGHT_TYPE
{
	ASURA_DYNAMICLIGHT_PHOTON = 0,			// Omni light			(implemented as a photon)
	ASURA_DYNAMICLIGHT_PROJECTOR,			// Projector
	ASURA_DYNAMICLIGHT_SPOTLIGHT,			// Spotlight			(special case of a projector)
	ASURA_DYNAMICLIGHT_INFINITE,			// Infinite directional (not yet implemented)
};


// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////////

#ifdef ASURA_TOOLS
class Tools_Entity_DynamicLight;
#endif


class Asura_ClientEntity_DynamicLight;

// C L A S S E S /////////////////////////////////////////////////////////////

enum
{
	ASURA_DYNAMICLIGHTFLAG_SHOWVOLUME				=	(1<<0),
	ASURA_DYNAMICLIGHTFLAG_MOVING					=	(1<<1),
	ASURA_DYNAMICLIGHTFLAG_PARALLELPROJECTION		=	(1<<2),
	ASURA_DYNAMICLIGHTFLAG_GRAPHISLOOPED			=	(1<<3),
	ASURA_DYNAMICLIGHTFLAG_USEGRAPHCOLOUR			=	(1<<4),
	ASURA_DYNAMICLIGHTFLAG_USEGRAPHRANGE			=	(1<<5),
	ASURA_DYNAMICLIGHTFLAG_USEGRAPHFLICKER			=	(1<<6),
	ASURA_DYNAMICLIGHTFLAG_USESTROBE				=	(1<<7),
	ASURA_DYNAMICLIGHTFLAG_USEMASTERORIENT			=	(1<<8),
	ASURA_DYNAMICLIGHTFLAG_HASCORONA				=	(1<<9),
	ASURA_DYNAMICLIGHTFLAG_DONTFADECORONAWITHDIST	=	(1<<10),
	ASURA_DYNAMICLIGHTFLAG_USECUSTOMLIGHTVOLUME		=	(1<<11),
	ASURA_DYNAMICLIGHTFLAG_ACTIVEATSTART			=	(1<<12),//NY - semi-depricated, only used for legacy loading and for dynamic light templates
	ASURA_DYNAMICLIGHTFLAG_USELIMITINGPLANE			=	(1<<13),
	ASURA_DYNAMICLIGHTFLAG_USEPHOTONLIMITINGANGLES	=	(1<<14),
	ASURA_DYNAMICLIGHTFLAG_CORONAUSESMODLINKING		=	(1<<15),
	ASURA_DYNAMICLIGHTFLAG_WORLDSPACECLV			=	(1<<16),//Should only be used by clients
};



class Asura_DynamicLight_Parameters
{
public:


	// Light type
	// ----------------------------------------------------------------------------------------------------
	ASURA_DYNAMICLIGHT_TYPE m_eType;
	u_int					m_uFlags;					// Lots of boolean parameters in here

	// Pos/Orient
	// ----------------------------------------------------------------------------------------------------
	Asura_Vector_3			m_xPosition;
	Asura_Matrix_3x3		m_xOrientation;

	// Distance Fade
	// ----------------------------------------------------------------------------------------------------
	float					m_fFadeoutStart;			// Dist at which fadeout starts
	float					m_fFadeoutDist;				// Dist over which fade takes place

	// Movement Params
	// ----------------------------------------------------------------------------------------------------
	bool					m_abSwing[3];				// X/Y/Z Swinging enabled?
	Asura_Euler_3			m_xRotSpeed;
	Asura_Euler_3			m_xSwingDist;
	float					m_fRotationOffset;			// Time offset to de-sync movement from other lights

	// Photon Params
	// ----------------------------------------------------------------------------------------------------
	float					m_fPhotonRange;				// Outer range of the light
	Asura_Bounding_Box		m_xLightVolume;
	Asura_Quat				m_xLightVolumeOrient;

	float					m_fLimitingAngleInner;
	float					m_fLimitingAngleOuter;

	Asura_Hash_ID			m_xProjCubeTextureID;
#ifdef ASURA_TOOLS
	Asura_String			m_xProjCubeTextureName;
#endif // #ifdef ASURA_TOOLS

	Asura_Hash_ID			m_xProjCubeBlurredTextureID;
#ifdef ASURA_TOOLS
	Asura_String			m_xProjCubeBlurredTextureName;
#endif // #ifdef ASURA_TOOLS
	float					m_fProjCubeLerpDist;

	// Projector Params
	// ----------------------------------------------------------------------------------------------------
	Asura_Hash_ID			m_uTextureHash;				// Textures
#ifdef ASURA_TOOLS
	Asura_String			m_xTextureName;
#endif // #ifdef ASURA_TOOLS
	Asura_Hash_ID			m_uDetailTextureHash;
#ifdef ASURA_TOOLS
	Asura_String			m_xDetailTextureName;
#endif // #ifdef ASURA_TOOLS
	float					m_fDetailTextureScale;		// Detail texture repeats
	float					m_fMinRange;				// Falloff range
	float					m_fMaxRange;
	int						m_iBlendMode;				// Blend mode
	float					m_fFoV;						// FoV + projection type
	float					m_fInnerFoV;				// This is the inner angle for spotlights
	float					m_fScale;					// Parallel projection scale
	float					m_fAspect;

	float					m_fUScrollSpeed;			// Texture animation, for detail texture
	float					m_fVScrollSpeed;
	float					m_fUScrollRange;
	float					m_fVScrollRange;

	Asura_Vector_3			m_xLimitingPlanePosition;	// Light limiting plane (for spotlights as well as projectors)
	Asura_Vector_3			m_xLimitingPlaneDirection;

	// Falloff Params
	// ----------------------------------------------------------------------------------------------------
	float					m_fFalloffInnerRatio;
	float					m_fFalloffPower;

	// Light colour/flicker controls
	// ----------------------------------------------------------------------------------------------------
	float					m_fBaseIntensity;			// Overall intensity scale
	float					m_fBaseR;
	float					m_fBaseG;
	float					m_fBaseB;
	float					m_fGraphDuration;			// In seconds
	float					m_fGraphTimerOffset;		// Used to de-sync the effects
	Asura_Graph<Asura_RGBA> m_xColourRangeGraph;
	Asura_Graph<float>		m_xFlickerGraph;
	float					m_fStrobeMinTimeOff;
	float					m_fStrobeMaxTimeOff;
	float					m_fStrobeMinTimeOn;
	float					m_fStrobeMaxTimeOn;



	// Attachment to a physical object
	// ----------------------------------------------------------------------------------------------------
	Asura_Guid				m_uMasterEntity;			// GUID of a physical object
	Asura_Vector_3			m_xOffsetFromMaster;
	Asura_Hash_ID			m_uMasterMarker;

	// Attachment to a lightshaft object
	// ----------------------------------------------------------------------------------------------------
	Asura_Guid				m_uAttachedLightShaft;		// GUID of lightshaft

	// Shadow Parameters
	// ----------------------------------------------------------------------------------------------------
	int						m_iShadowMode;				// Tools UI mode doesn't direcly map to shadowparameters
	Asura_ShadowParameters	m_xShadowParams;			// Shadow parameters


	// Corona Parameters
	// ----------------------------------------------------------------------------------------------------
	float					m_fCoronaRadius;
	Asura_Hash_ID			m_uCoronaTextureHash;
#ifdef ASURA_TOOLS
	Asura_String			m_xCoronaTextureName;
#endif // #ifdef ASURA_TOOLS
	float					m_fCoronaVolumetricDepthScale;


	// Default Constructor - Set some sensible defaults
	// As there's a couple of objects in here, we can't just memset it to 0 - so we have to initialsie
	// everything here
	// ----------------------------------------------------------------------------------------------------
	Asura_DynamicLight_Parameters()
	{
		m_eType							= ASURA_DYNAMICLIGHT_PHOTON;
		m_uFlags						= ASURA_DYNAMICLIGHTFLAG_ACTIVEATSTART | ASURA_DYNAMICLIGHTFLAG_USEMASTERORIENT | ASURA_DYNAMICLIGHTFLAG_GRAPHISLOOPED;
		m_xPosition.Zero();
		m_xOrientation.Identity();
		m_fFadeoutStart					= 100.0f;
		m_fFadeoutDist					= 10.0f;

		for( int i=0; i<3; i++ )
		{
			m_abSwing[i]				= false;
		}
		float fInitValue = Asura_Maths::DegToRad(10.0f);
		m_xRotSpeed = Asura_Euler_3(fInitValue, fInitValue, fInitValue);
		m_xSwingDist = Asura_Euler_3(fInitValue, fInitValue, fInitValue);

		m_fRotationOffset				= 0.0f;
		m_fPhotonRange					= 10.0f;
		m_xLightVolumeOrient.Identity();
		m_uTextureHash					= 0;
		m_uDetailTextureHash				= 0;
		m_xProjCubeTextureID			= 0;
		m_xProjCubeBlurredTextureID		= 0;
		m_fProjCubeLerpDist				= 5.0f;
		m_fDetailTextureScale			= 4.0f;
		m_fMinRange						= 0.0f;
		m_fMaxRange						= 15.0f;
		m_iBlendMode					= 0;
		m_fFoV							= Asura_Maths::Deg90 / 2.0f;
		m_fInnerFoV						= Asura_Maths::Deg90 / 4.0f;
		m_fScale						= 1.0f;
		m_fAspect						= 1.0f;
		m_uMasterEntity					= ASURA_GUID_UNREGISTERED;
		m_uAttachedLightShaft			= ASURA_GUID_UNREGISTERED;
		m_xOffsetFromMaster.Zero();
		m_uMasterMarker					= ASURA_HASH_ID_UNSET;
		m_iShadowMode					= 1;
		m_xShadowParams.m_uFlags		= ASURA_SHADOWFLAG_CONTRIBUTES_TO_AVERAGE_SHADOW;
		m_fCoronaRadius					= 1.0f;
		m_fCoronaVolumetricDepthScale	= Asura_Corona_System::GetDefaultVolumetricDepthScale();
		m_uCoronaTextureHash				= 0;
		m_fFalloffInnerRatio			= 0.0f;
		m_fFalloffPower					= 1.0f;
		m_fBaseIntensity				= 1.0f;
		m_fBaseR						= 1.0f;
		m_fBaseG						= 1.0f;
		m_fBaseB						= 1.0f;
		m_fGraphDuration				= 10.0f;
		m_fGraphTimerOffset				= 0.0f;
		m_fStrobeMinTimeOff				= 1.0f;
		m_fStrobeMaxTimeOff				= 1.0f;
		m_fStrobeMinTimeOn				= 1.0f;
		m_fStrobeMaxTimeOn				= 1.0f;

		m_fUScrollSpeed					= 0.0f;
		m_fVScrollSpeed					= 0.0f;
		m_fUScrollRange					= 0.0f;
		m_fVScrollRange					= 0.0f;

		m_xLimitingPlanePosition		= Asura_Vector_3(0,0,0);
		m_xLimitingPlaneDirection		= Asura_Vector_3(0,-1,0);

		m_fLimitingAngleInner			= Asura_Maths::Deg180;
		m_fLimitingAngleOuter			= Asura_Maths::DegToRad(200.0f);

		// Init the graphs
		m_xColourRangeGraph.Initialise(16, Asura_RGBA(1.0f,1.0f,1.0f,1.0f),Asura_RGBA(1.0f,1.0f,1.0f,1.0f));
		m_xFlickerGraph.Initialise(16,0,0);

	}


	~Asura_DynamicLight_Parameters()
	{

	}

	// There's a couple of Asura_Strings and Asura_Graphs in here, so we need to handle copying
	// them correctly
	// ----------------------------------------------------------------------------------------------------
	Asura_DynamicLight_Parameters( const Asura_DynamicLight_Parameters& xRhs )
	{
	 	*this = xRhs;
	}

	Asura_DynamicLight_Parameters& operator = (const Asura_DynamicLight_Parameters& xParams)
	{
		CopyToThis(xParams);
		return *this;
	}


	void CopyToThis(const Asura_DynamicLight_Parameters& xParams,
					bool  bLightType		= true,
					bool  bPosOrient		= true,
					bool  bFadeAndFalloff	= true,
					bool  bMovement			= true,
					bool  bColourAndFlicker	= true,
					bool  bShadows			= true,
					bool  bSpecialFX		= true)
	{

		bool bFlagMoving		= (m_uFlags & ASURA_DYNAMICLIGHTFLAG_MOVING)?true:false;
		bool bFlagParallel		= (m_uFlags & ASURA_DYNAMICLIGHTFLAG_PARALLELPROJECTION)?true:false;
		bool bFlagLooped		= (m_uFlags & ASURA_DYNAMICLIGHTFLAG_GRAPHISLOOPED)?true:false;
		bool bFlagGraphColour	= (m_uFlags & ASURA_DYNAMICLIGHTFLAG_USEGRAPHCOLOUR)?true:false;
		bool bFlagGraphRange	= (m_uFlags & ASURA_DYNAMICLIGHTFLAG_USEGRAPHRANGE)?true:false;
		bool bFlagGraphFlicker	= (m_uFlags & ASURA_DYNAMICLIGHTFLAG_USEGRAPHFLICKER)?true:false;
		bool bFlagStrobe		= (m_uFlags & ASURA_DYNAMICLIGHTFLAG_USESTROBE)?true:false;
		bool bFlagMasterOrient	= (m_uFlags & ASURA_DYNAMICLIGHTFLAG_USEMASTERORIENT)?true:false;
		bool bFlagCorona		= (m_uFlags & ASURA_DYNAMICLIGHTFLAG_HASCORONA)?true:false;
		bool bFlagDontFadeCorona= (m_uFlags & ASURA_DYNAMICLIGHTFLAG_DONTFADECORONAWITHDIST)?true:false;
		bool bFlagCustomVolume	= (m_uFlags & ASURA_DYNAMICLIGHTFLAG_USECUSTOMLIGHTVOLUME)?true:false;
		bool bFlagActive		= (m_uFlags & ASURA_DYNAMICLIGHTFLAG_ACTIVEATSTART)?true:false;
		bool bFlagLimitingPlane	= (m_uFlags & ASURA_DYNAMICLIGHTFLAG_USELIMITINGPLANE)?true:false;
		bool bFlagLimitingAngles= (m_uFlags & ASURA_DYNAMICLIGHTFLAG_USEPHOTONLIMITINGANGLES)?true:false;
		bool bFlagCoronaUsesModLinking = (m_uFlags & ASURA_DYNAMICLIGHTFLAG_CORONAUSESMODLINKING)?true:false;
		bool bFlagWorldSpaceCLV	= (m_uFlags & ASURA_DYNAMICLIGHTFLAG_WORLDSPACECLV)?true:false;

		if (bLightType)
		{
			m_eType							= xParams.m_eType;
			bFlagActive						= (xParams.m_uFlags & ASURA_DYNAMICLIGHTFLAG_ACTIVEATSTART)?true:false;
		}

		if (bPosOrient)
		{
			m_xPosition						= xParams.m_xPosition;
			m_xOrientation					= xParams.m_xOrientation;
			m_xOffsetFromMaster				= xParams.m_xOffsetFromMaster;
			m_uMasterMarker					= xParams.m_uMasterMarker;
			bFlagMasterOrient				= (xParams.m_uFlags & ASURA_DYNAMICLIGHTFLAG_USEMASTERORIENT)?true:false;
		}

		if (bMovement)
		{
			for( int i=0; i<3; i++ )
			{
				m_abSwing[i]				= xParams.m_abSwing[i];
			}
			m_xRotSpeed = xParams.m_xRotSpeed;
			m_xSwingDist = xParams.m_xSwingDist;

			bFlagMoving						= (xParams.m_uFlags & ASURA_DYNAMICLIGHTFLAG_MOVING)?true:false;
		}

		m_fRotationOffset					= xParams.m_fRotationOffset;

		if (bLightType)
		{
			m_fPhotonRange					= xParams.m_fPhotonRange;
			m_xLightVolume					= xParams.m_xLightVolume;
			m_xLightVolumeOrient			= xParams.m_xLightVolumeOrient;

			m_fLimitingAngleInner			= xParams.m_fLimitingAngleInner;
			m_fLimitingAngleOuter			= xParams.m_fLimitingAngleOuter;
			bFlagLimitingAngles				= (xParams.m_uFlags & ASURA_DYNAMICLIGHTFLAG_USEPHOTONLIMITINGANGLES)?true:false;

			m_xProjCubeTextureID			= xParams.m_xProjCubeTextureID;
#ifdef ASURA_TOOLS
			m_xProjCubeTextureName			= xParams.m_xProjCubeTextureName;
#endif // #ifdef ASURA_TOOLS

			m_xProjCubeBlurredTextureID		= xParams.m_xProjCubeBlurredTextureID;
#ifdef ASURA_TOOLS
			m_xProjCubeBlurredTextureName	= xParams.m_xProjCubeBlurredTextureName;
#endif // #ifdef ASURA_TOOLS
			m_fProjCubeLerpDist				= xParams.m_fProjCubeLerpDist;

			m_uTextureHash					= xParams.m_uTextureHash;
#ifdef ASURA_TOOLS
			m_xTextureName					= xParams.m_xTextureName;
#endif // #ifdef ASURA_TOOLS
			m_uDetailTextureHash				= xParams.m_uDetailTextureHash;
#ifdef ASURA_TOOLS
			m_xDetailTextureName			= xParams.m_xDetailTextureName;
#endif // #ifdef ASURA_TOOLS
			m_fDetailTextureScale			= xParams.m_fDetailTextureScale;
			m_fMinRange						= xParams.m_fMinRange;
			m_fMaxRange						= xParams.m_fMaxRange;
			m_iBlendMode					= xParams.m_iBlendMode;
			m_fFoV							= xParams.m_fFoV;
			m_fInnerFoV						= xParams.m_fInnerFoV;
			m_fScale						= xParams.m_fScale;
			m_fAspect						= xParams.m_fAspect;
			m_uMasterEntity					= xParams.m_uMasterEntity;
			m_uAttachedLightShaft			= xParams.m_uAttachedLightShaft;

			bFlagParallel					= (xParams.m_uFlags & ASURA_DYNAMICLIGHTFLAG_PARALLELPROJECTION)?true:false;
			bFlagCustomVolume				= (xParams.m_uFlags & ASURA_DYNAMICLIGHTFLAG_USECUSTOMLIGHTVOLUME)?true:false;
			bFlagWorldSpaceCLV				= (xParams.m_uFlags & ASURA_DYNAMICLIGHTFLAG_WORLDSPACECLV)?true:false;

			m_fUScrollSpeed					= xParams.m_fUScrollSpeed;
			m_fVScrollSpeed					= xParams.m_fVScrollSpeed;
			m_fUScrollRange					= xParams.m_fUScrollRange;
			m_fVScrollRange					= xParams.m_fVScrollRange;

			m_xLimitingPlanePosition		= xParams.m_xLimitingPlanePosition;
			m_xLimitingPlaneDirection		= xParams.m_xLimitingPlaneDirection;
			bFlagLimitingPlane				= (xParams.m_uFlags & ASURA_DYNAMICLIGHTFLAG_USELIMITINGPLANE)?true:false;
		}


		if (bShadows)
		{
			m_iShadowMode					= xParams.m_iShadowMode;
			m_xShadowParams					= xParams.m_xShadowParams;
		}


		if (bSpecialFX)
		{
			m_fCoronaRadius					= xParams.m_fCoronaRadius;
			m_uCoronaTextureHash				= xParams.m_uCoronaTextureHash;
#ifdef ASURA_TOOLS
			m_xCoronaTextureName			= xParams.m_xCoronaTextureName;
#endif // #ifdef ASURA_TOOLS
			m_fCoronaVolumetricDepthScale	= xParams.m_fCoronaVolumetricDepthScale;

			bFlagCorona						= (xParams.m_uFlags & ASURA_DYNAMICLIGHTFLAG_HASCORONA)?true:false;
			bFlagDontFadeCorona				= (xParams.m_uFlags & ASURA_DYNAMICLIGHTFLAG_DONTFADECORONAWITHDIST)?true:false;
			bFlagCoronaUsesModLinking		= (xParams.m_uFlags & ASURA_DYNAMICLIGHTFLAG_CORONAUSESMODLINKING)?true:false;
		}

		if (bFadeAndFalloff)
		{
			m_fFalloffInnerRatio			= xParams.m_fFalloffInnerRatio;
			m_fFalloffPower					= xParams.m_fFalloffPower;
			m_fBaseIntensity				= xParams.m_fBaseIntensity;
			m_fFadeoutStart					= xParams.m_fFadeoutStart;
			m_fFadeoutDist					= xParams.m_fFadeoutDist;
		}


		if (bColourAndFlicker)
		{
			m_fBaseR						= xParams.m_fBaseR;
			m_fBaseG						= xParams.m_fBaseG;
			m_fBaseB						= xParams.m_fBaseB;
			m_fGraphDuration				= xParams.m_fGraphDuration;
			m_fGraphTimerOffset				= xParams.m_fGraphTimerOffset;
			m_xColourRangeGraph				= xParams.m_xColourRangeGraph;
			m_xFlickerGraph					= xParams.m_xFlickerGraph;
			m_fStrobeMinTimeOff				= xParams.m_fStrobeMinTimeOff;
			m_fStrobeMaxTimeOff				= xParams.m_fStrobeMaxTimeOff;
			m_fStrobeMinTimeOn				= xParams.m_fStrobeMinTimeOn;
			m_fStrobeMaxTimeOn				= xParams.m_fStrobeMaxTimeOn;

			bFlagLooped						= (xParams.m_uFlags & ASURA_DYNAMICLIGHTFLAG_GRAPHISLOOPED)?true:false;
			bFlagGraphColour				= (xParams.m_uFlags & ASURA_DYNAMICLIGHTFLAG_USEGRAPHCOLOUR)?true:false;
			bFlagGraphRange					= (xParams.m_uFlags & ASURA_DYNAMICLIGHTFLAG_USEGRAPHRANGE)?true:false;
			bFlagGraphFlicker				= (xParams.m_uFlags & ASURA_DYNAMICLIGHTFLAG_USEGRAPHFLICKER)?true:false;
			bFlagStrobe						= (xParams.m_uFlags & ASURA_DYNAMICLIGHTFLAG_USESTROBE)?true:false;
		}


		m_uFlags = 0;
		if (bFlagMoving)			m_uFlags |= ASURA_DYNAMICLIGHTFLAG_MOVING;
		if (bFlagParallel)			m_uFlags |= ASURA_DYNAMICLIGHTFLAG_PARALLELPROJECTION;
		if (bFlagLooped)			m_uFlags |= ASURA_DYNAMICLIGHTFLAG_GRAPHISLOOPED;
		if (bFlagGraphColour)		m_uFlags |= ASURA_DYNAMICLIGHTFLAG_USEGRAPHCOLOUR;
		if (bFlagGraphRange)		m_uFlags |= ASURA_DYNAMICLIGHTFLAG_USEGRAPHRANGE;
		if (bFlagGraphFlicker)		m_uFlags |= ASURA_DYNAMICLIGHTFLAG_USEGRAPHFLICKER;
		if (bFlagStrobe)			m_uFlags |= ASURA_DYNAMICLIGHTFLAG_USESTROBE;
		if (bFlagMasterOrient)		m_uFlags |= ASURA_DYNAMICLIGHTFLAG_USEMASTERORIENT;
		if (bFlagCorona)			m_uFlags |= ASURA_DYNAMICLIGHTFLAG_HASCORONA;
		if (bFlagDontFadeCorona)	m_uFlags |= ASURA_DYNAMICLIGHTFLAG_DONTFADECORONAWITHDIST;
		if (bFlagCustomVolume)		m_uFlags |= ASURA_DYNAMICLIGHTFLAG_USECUSTOMLIGHTVOLUME;
		if (bFlagActive)			m_uFlags |= ASURA_DYNAMICLIGHTFLAG_ACTIVEATSTART;
		if (bFlagLimitingPlane)		m_uFlags |= ASURA_DYNAMICLIGHTFLAG_USELIMITINGPLANE;
		if (bFlagLimitingAngles)	m_uFlags |= ASURA_DYNAMICLIGHTFLAG_USEPHOTONLIMITINGANGLES;
		if (bFlagCoronaUsesModLinking)	m_uFlags |= ASURA_DYNAMICLIGHTFLAG_CORONAUSESMODLINKING;
		if (bFlagWorldSpaceCLV)		m_uFlags |= ASURA_DYNAMICLIGHTFLAG_WORLDSPACECLV;
	}
};





class Asura_ServerEntity_DynamicLight : public Asura_Entity_Server
{
public:
	// C O N S T R U C T O R S   /   D E S T R U C T O R S ///////////////
#ifdef ASURA_TOOLS
	static Asura_ServerEntity_DynamicLight* CreateFromTools( Asura_Guid uGuid, const Asura_Vector_3& xCentre );
#endif
	Asura_ServerEntity_DynamicLight( Asura_Guid uGuid);

	// C L A S S   M E T H O D S /////////////////////////////////////////
	static void CreateFromChunkStream( Asura_Guid uGuid, Asura_Chunk_Stream &xStream );
	static void ExtractParamsFromChunkStream( Asura_DynamicLight_Parameters &xParams , Asura_Chunk_Stream &xStream, u_int uVersion);
	static void WriteParamsToChunkStream( const Asura_DynamicLight_Parameters &xParams , Asura_Chunk_Stream& xStream);

	// M E T H O D S /////////////////////////////////////////////////////
	// Query methods.
	const Asura_DynamicLight_Parameters&	GetParams( void ) const { return m_xParams; }

	// Function used to work out how far a point is away from this entity...
	float	CalculateSquaredDistanceTo(const Asura_Vector_3& xPosition) const	{ const Asura_Vector_3 xDistance(m_xParams.m_xPosition - xPosition); return(xDistance.MagnitudeSquared()); }

	// Set methods.
#ifdef ASURA_TOOLS
	void SetParams( const Asura_DynamicLight_Parameters& xNewParams );
#endif
	// Load / save.
	virtual	void ReadFromChunkStream(Asura_Chunk_Stream& xStream);
	virtual	void WriteToChunkStream(Asura_Chunk_Stream& xStream) const;
	virtual bool CanBeSavedToChunk() const				{ return true;  }
	virtual	bool IsFilledFromChunkBuffer( void ) const	{ return false; }

	static Asura_ServerEntity_DynamicLight* FindEntity( Asura_Guid uGUID ) { return s_xList.FindEntity( uGUID ); }
	static const Asura_Hashed_List<Asura_ServerEntity_DynamicLight>* GetList( void ) { return &s_xList; }

#ifdef ASURA_TOOLS
	virtual void ApplyChangesForTemplateInstance( class Tools_EntityTemplateInstance* pxInstance );
	virtual bool CanBeIncludedInTemplate() const		{ return true; }

	virtual void TraverseUsedTextures( void (*pfnCallback)( int iTextureID, void* pUserData ), void* pUserData ) const;
#endif

	virtual bool HandleMessage( const Asura_Message* pxMessage );

	static const u_int DYNLIGHT_CHUNK_VERSION = 12;

protected:

	// C O N S T R U C T O R S   /   D E S T R U C T O R S ///////////////
	virtual ~Asura_ServerEntity_DynamicLight( void );

	// S T A T E /////////////////////////////////////////////////////////
	Asura_DynamicLight_Parameters m_xParams;

private:

	static Asura_Hashed_List<Asura_ServerEntity_DynamicLight> s_xList;

	// F R I E N D S /////////////////////////////////////////////////////
#ifdef ASURA_TOOLS
	friend Tools_Entity_DynamicLight;
#endif

	// T Y P E S /////////////////////////////////////////////////////////
	typedef Asura_Entity_Server PARENT;
};
// End class Asura_ServerEntity_Projector



#endif //#ifdef ASURA_TARGET_SM3
#endif // ___ASURA_SERVERENTITY_DYNAMICLIGHT_H___
