
////////////////////////////////////////////////////////////////////////////////
//
// Asura_ServerEntity_FMVSpeaker.h
// Author    : Jason Turner
// Copyright (C) 2009 Rebellion, All Rights Reserved.
//
////////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef __ASURA_SERVERENTITY_FMVSPEAKER_H__
#define __ASURA_SERVERENTITY_FMVSPEAKER_H__

#include "Core/Asura_String.h"
#include "Core/Asura_TemplateTree.h"
#include "Entity/Triggers/Asura_ServerEntity_Activatable.h"

#ifdef ASURA_TOOLS
#include "Entity/Asura_Entity_Classification.h"
#endif

class Asura_VideoToTexture;

class Asura_ServerEntity_FMVSpeaker : public Asura_ServerEntity_ActivatableWithPosition
{
	typedef Asura_ServerEntity_ActivatableWithPosition PARENT;

	public:
		Asura_ServerEntity_FMVSpeaker(Asura_Guid uGuid);
		virtual ~Asura_ServerEntity_FMVSpeaker();

		static void CreateFromChunkStream( Asura_Guid uGuid, Asura_Chunk_Stream &xStream );

		static Asura_ServerEntity_FMVSpeaker* FindEntity( const Asura_Guid uGuid ) { return s_xList.FindEntity( uGuid ); }
		static Asura_Hashed_List< Asura_ServerEntity_FMVSpeaker >* GetList() { return &s_xList; }

	protected :

		// Overrides
		virtual bool HandleMessage(const Asura_Message* pxMessage);

		// Read/write to chunk streams
		virtual	bool IsFilledFromChunkBuffer(void) const { return false; }
		virtual void WriteToChunkStream( Asura_Chunk_Stream& xStream ) const;
		virtual void ReadFromChunkStream( Asura_Chunk_Stream& xStream );

		float m_fInnerRange;
		float m_fOuterRange;

		#ifdef ASURA_TOOLS
		virtual bool CanBeIncludedInTemplate() const {return Classification == AsuraEntityClass_FMVSpeaker;}
		friend class Tools_Entity_FMVSpeaker;
		#endif

    private:

		friend class Asura_ServerEntity_FMVController;

		#ifdef ASURA_TOOLS
		friend class Tools_Entity_FMVController;
		#endif

		Asura_ServerEntity_FMVSpeaker();

		// if you change this, make sure ReadFromChunkStream/WriteToChunkStream still support old versions
		enum { CURRENT_CHUNK_VERSION = 1 };

		// list of FMV Controllers using this speaker
		Asura_TemplateTree<Asura_Guid, Asura_Guid> m_xLinkedControllerGuids;

		static int GuidCompareFunction(const Asura_Guid& xKey1, const Asura_Guid& xKey2)
		{
			return static_cast<int>(xKey2 - xKey1);
		}
		void AddLinkedController(Asura_Guid uGuid);
		void RemoveLinkedController(Asura_Guid uGuid);
		void SetPosition(const Asura_Vector_3& xPos) {m_xPosition = xPos;}

		static Asura_Hashed_List< Asura_ServerEntity_FMVSpeaker > s_xList;
};

#endif //__ASURA_SERVERENTITY_FMVSPEAKER_H__
