//////////////////////////////////////////////////////////////////////////////
//
// Asura_ClientEntity_Glass.cpp
//
// Copyright (c) 2006 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header.
#include "Entity/Glass/Asura_ClientEntity_Glass.h"

// Engine independent.
#include "Animation/Asura_AnimationAttachment.h"
#include "Camera/Asura_Camera.h"
#include "ChunkSystem/Asura_Chunk_Macros.h"
#include "Core/Asura_LOS.h"
#include "Core/Asura_MaterialFlags.h"
#include "Core/Asura_String.h"
#include "Core/Asura_TextureManagement.h"
#include "Debugging/Asura_Assert.h"
#include "Entity/Asura_Entity_Classification.h"
#include "Entity/Glass/Asura_Entity_Glass_Messages.h"
#include "Entity/Glass/Asura_ServerEntity_Glass.h"
#include "Network/Asura_Message.h"
#include "Rendering/Asura_Module.h"
#include "Rendering/Asura_Render.h"
#include "Rendering/Asura_SortedRender.h"
#include "SpecialFX/Asura_SFX_GlassSmash.h"

// C L A S S   I M P L E M E N T A T I O N S /////////////////////////////////

//
// Asura_ClientEntity_Glass
//

#ifdef ASURA_TOOLS
Asura_Hashed_List<Asura_ClientEntity_Glass> Asura_ClientEntity_Glass::s_xList;
#endif

void Asura_ClientEntity_Glass::Create( const Asura_Message* pxMessage )
{
	Asura_ClientEntity_Glass* pxGlass = Project_Create(pxMessage);

	if(pxGlass)
	{
		pxGlass->HandleMessage(pxMessage);
	}
}


Asura_ClientEntity_Glass::Asura_ClientEntity_Glass()
: m_uColour(ASURA_ENTITY_GLASS_DEFAULT_COLOUR)
, m_iTextureIndex(ASURA_TEXTURE_NONE)
{
	Classification = AsuraEntityClass_Glass;

	m_afU[0] = m_afU[2] = m_afV[0] = m_afV[1] = 0.0f;
	m_afU[1] = m_afU[3] = m_afV[2] = m_afV[3] = 1.0f;
	
	// Initialise from chunkbuffer next....
#ifdef ASURA_TOOLS
	s_xList.AddToList(this);
#endif
}


void Asura_ClientEntity_Glass::InitialiseFromChunkBuffer(const char*& pcChunkBuffer)
{
	Asura_ServerEntity_Glass::ChunkData xData;
	ASURA_CHUNK_EXTRACT(pcChunkBuffer, xData, Asura_ServerEntity_Glass::ChunkData);
	//Asura_Assert(xData.m_iVersion == Asura_ServerEntity_Glass::ChunkData::CURRENT_VERSION, "Old version for glass");
	
	m_xPane   = xData.m_xPane;
 	m_uColour = xData.m_uColour;
	for (int i=0; i<4; i++)
	{
		m_afU[i] = xData.m_afU[i];
		m_afV[i] = xData.m_afV[i];
	}	

	if (xData.m_iVersion < 4)
	{
		m_uGlassFlags = 0;
	}
	else
	{
		ASURA_CHUNK_EXTRACT( pcChunkBuffer, m_uGlassFlags, u_int );
	}

	// Load texture if necessary
	Asura_String xTextureName;
	ASURA_CHUNK_EXTRACT_STRING(pcChunkBuffer, xTextureName);

	if (!xTextureName.IsBlank())
	{
		bool bIsNew;
		m_iTextureIndex = Asura_Texture_Management::AddTexture(xTextureName, &bIsNew);
		if (bIsNew)
		{
			Asura_Texture_Management::LoadTextures();
		}
	}
	SetupCullingData();
}

bool Asura_ClientEntity_Glass::HandleMessage(const Asura_Message* pxMessage)
{
	if (IsDestroyed()) return true; // no further action needed

	Asura_Assert(pxMessage,"");
	if (!pxMessage) return false;

	switch (pxMessage->GetMsgID())
	{
		case ASURA_MSG_GLASS_APPLYDECAL:
		{
			const Asura_Message_Glass_ApplyDecal* pxData = static_cast<const Asura_Message_Glass_ApplyDecal*>(pxMessage->GetData());
			ApplyDecal(pxData->m_xDecalCoordinates);
			return true;
		}
		case ASURA_MSG_GLASS_SMASHWITHPOSITION:
		{
			const Asura_Message_Glass_SmashWithPosition* pxData = static_cast<const Asura_Message_Glass_SmashWithPosition*>(pxMessage->GetData());
			SmashGlass(pxData->m_xHitCoordinates, pxData->m_fPower, pxData->m_xDirection);
			return true;
		}
		case ASURA_MSG_GLASS_SMASH:
		{
			const Asura_Message_Glass_Smash* pxData = static_cast<const Asura_Message_Glass_Smash*>(pxMessage->GetData());
			SmashGlass(pxData->m_fPower, pxData->m_bSmashInPositiveZDir);
			return true;
		}

		case ASURA_MSG_GLASS_CLIENTUPDATE:
		{
			/// DWE - 31/07/2003 16:48:45 - Gets info from the server entity as Glass server entities exist on the client
			Asura_ServerEntity_Glass* pxServerGlass = Asura_ServerEntity_Glass::FindEntity(Guid);

			if(pxServerGlass)
			{
				m_xPane = pxServerGlass->m_xPane;
				m_uGlassFlags = pxServerGlass->m_uGlassFlags;
				m_uColour = pxServerGlass->m_uColour;
			
				for(int i=0;i<4;++i)
				{
					m_afU[i] = pxServerGlass->m_afU[i];
					m_afV[i] = pxServerGlass->m_afV[i];
				}

				m_iTextureIndex = Asura_Texture_Management::AddTexture(pxServerGlass->m_xTextureName);
			}

			SetupCullingData();
			
			return true;
		}


	
		default:
			break;
	}

	return false; // Must not pass to parent
}


void Asura_ClientEntity_Glass::Render()
{
#ifdef ASURA_TOOLS
	// In tools, keep updating the culling data in case glass has moved\changed.
	SetupCullingData();
#endif
	unsigned uCamVisFlags = 0;
	for (int i=0;i<Asura_Camera::GetNumberOfCameras();i++)
	{
		Asura_Camera::SetCameraIndex( i );
		if (Asura_Camera::IsCameraActive())
		{
			if (!Asura_Camera::IsSphereInFrustum(m_xCentre, m_fRadius))
				continue;

#if defined(ASURA_USE_STREAMING)
			if (!Asura_Module_System::DoesBBIntersectVisibleModule(m_xBB, &m_iLastKnownModule, &m_uLastKnownZone))
				continue;
#else
			if (!Asura_Module_System::DoesBBIntersectVisibleModule(m_xBB, &m_iLastKnownModule))
				continue;
#endif //defined(ASURA_USE_STREAMING)

			uCamVisFlags |= (1<<i);
		}
	}
	if (uCamVisFlags)
	{
		if(Asura_SortedRender::IsActive())
		{
			Asura_SortedRender::AddGlass(this,uCamVisFlags);
		}
		else
		{
			Platform_Render(uCamVisFlags);
		}
	}
}

Asura_Vector_3 Asura_ClientEntity_Glass::GetBlastDirection(bool bSmashInPositiveZDir) const
{
	Asura_Vector_3 xDirection(0.0f, 0.0f, 1.0f);
	if (!bSmashInPositiveZDir)
	{
		xDirection.z = -1.0f;
	}
	return xDirection * m_xPane.Orientation;
}


void Asura_ClientEntity_Glass::ApplyDecal(const Asura_Vector_2& xDecalCoordinates) const
{
	// TODO
}

void Asura_ClientEntity_Glass::SmashGlass(const Asura_Vector_2& xHitCoordinates, float fPower, const Asura_Vector_3& xDirection)
{
	Asura_SFX_GlassSmash::Create(*this, fPower, xHitCoordinates, xDirection);
	SetAsDestroyed();
}

void Asura_ClientEntity_Glass::SmashGlass(float fPower, bool bSmashInPositiveZDir)
{
	Asura_SFX_GlassSmash::Create(*this, fPower, bSmashInPositiveZDir);
	SetAsDestroyed();
}

bool Asura_ClientEntity_Glass::LOSTest(const Asura_LOS_InputData* pxInputData, Asura_LOS_OutputData* pxOutputData)
{
	// Convert LOS test from world space to object space
	Asura_Vector_3 xAlpha = pxInputData->m_xOrigin;
	Asura_Vector_3 xBeta  = pxInputData->m_xDirection;

	xAlpha -= m_xPane.CornerPosition;

	Asura_Matrix_3x3 xMat = m_xPane.Orientation;
	xMat.Transpose();

	xAlpha = xAlpha * xMat;
	xBeta = xBeta * xMat;

	// Temporary code.
	// MGH 3/9/02: TODO replace this with something more efficient
	Asura_Bounding_Box xBox;
	xBox.Reset();
	xBox.Set(Asura_Maths::ZeroVector3);

	Asura_Vector_3 xTemp(m_xPane.Dimensions.x, m_xPane.Dimensions.y, 0.05f);
	xBox.Set(xTemp);

	//Tools_Render::RenderCube(xBox, m_xPane.CornerPosition, m_xPane.Orientation);

	float fDistance;
	Asura_Vector_3 xNormal;
	bool bStartedInside;
	if (Asura_Maths::RayIntersectsBoundingBox(xAlpha, xBeta, xBox, &fDistance, 0, &bStartedInside, &xNormal))
	{
		// Hit the BB
		if (fDistance < pxOutputData->m_fDistanceToIntersection)
		{	
				pxOutputData->m_uHitEntity = GetGuid();
				pxOutputData->m_bHitEnvironment = false;
				pxOutputData->m_fDistanceToIntersection = fDistance;
                pxOutputData->m_iHitAttachmentID = Asura_AnimationAttachment::INVALID_ID;
				pxOutputData->m_iHitBoneID = -1;
				pxOutputData->m_usPolyFlags = 0; // TODO set material flags

				#if defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM
					pxOutputData->m_uMaterialResponseHashID = ASURA_HASH_ID_UNSET;
				#endif
				pxOutputData->m_sMaterialID = ASURA_MATERIAL_NONE;

				pxOutputData->m_xNormal = xNormal * m_xPane.Orientation;
				pxOutputData->m_bStartedInside = bStartedInside;
				return true;
		}
	}

	return false;
}

void Asura_ClientEntity_Glass::SetupCullingData()
{
	m_xCentre = m_xPane.CalculateCentre();
	m_fRadius = (m_xPane.CornerPosition - m_xCentre).Magnitude();
	Asura_Vector_3 xCorners[4];
	m_xPane.CalculateCorners(xCorners);
	m_xBB.Reset();
	for (int i=0;i<4;i++)
	{
		m_xBB.Set(xCorners[i]);
	}
	m_iLastKnownModule = -1;

#if defined(ASURA_USE_STREAMING)
	m_uLastKnownZone = ASURA_HASH_ID_UNSET;
#endif //defined(ASURA_USE_STREAMING)
}
