//////////////////////////////////////////////////////////////////////////////
//
// Asura_ServerEntity_Glass.h
//
// Copyright (c) 2006 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef ASURA_SERVERENTITY_GLASS_H___
#define ASURA_SERVERENTITY_GLASS_H___ 1

// I N C L U D E S ///////////////////////////////////////////////////////////

// Engine independent.
#include "Core/Asura_String.h"
#include "Dynamics/Asura_Collision_Object.h"
#include "Entity/Glass/Asura_WorldSpace_Rectangle.h"
#include "Entity/PhysicalObject/Asura_ServerEntity_PhysicsObject.h"
#ifdef ASURA_TOOLS
#include "Entity/Asura_Entity_Classification.h"
#endif

// C L A S S E S /////////////////////////////////////////////////////////////

class Asura_ServerEntity_Glass : public virtual Asura_ServerEntity_PhysicsObject
{

public:

	static Asura_ServerEntity_Glass* FindEntity( Asura_Guid uGUID ) { return s_xList.FindEntity( uGUID ); }
	static const Asura_Hashed_List<Asura_ServerEntity_Glass>* GetList( void ) { return &s_xList; }

	virtual void FillInChunkBuffer(char*& pcChunkBuffer);
	virtual int GetChunkBufferSize();
	const Asura_Vector_3& GetCornerPosition() const {return m_xPane.CornerPosition;}
	virtual bool HandleMessage(const Asura_Message* pxMessage);
	virtual void InitialiseFromChunkBuffer(const char*& pcChunkBuffer);
	virtual void MakeDead();
	virtual void SetCornerPosition(const Asura_Vector_3& xCorner);
	virtual void Update();
	
	// Copy position and orientation from dynamics block to pane.
	// Call this whenever you change position/orientation in tools.
	// Changed to just doing orientation RAR 25/07/03
	void CopyOrientationToPane();

	// Power is in arbitrary units in range of 0-1.
	// Vectors are in world space.
	void ApplyDecal(const Asura_Vector_3& xHitPosition);
	void SmashGlass(const Asura_Vector_3& xHitPosition, float fPower, const Asura_Vector_3& xDirection);	
	void SmashGlass(float fPower, bool bSmashInPositiveZDir); // Un-specified hit position, (e.g. grenade hits)

#ifdef ASURA_TOOLS

	virtual void ApplyChangesForTemplateInstance(class Tools_EntityTemplateInstance* pxInstance);

#endif

protected:

	Asura_ServerEntity_Glass();
	virtual ~Asura_ServerEntity_Glass();

	// ChunkData
	struct ChunkData
	{
		enum {CURRENT_VERSION = 4};
		int m_iVersion;
		Asura_WorldSpace_Rectangle m_xPane;
		unsigned int m_uColour;
		float m_afU[4];
		float m_afV[4];
		// NOTE: Texture name is appended.
	};

	// Smash glass or apply decal in 2d glass coords.
	void ApplyDecal(const Asura_Vector_2& xDecalCoordinates);
	void SmashGlass(const Asura_Vector_2& xHitCoordinates, float fPower, const Asura_Vector_3& xDirection);	

	// Helper function
	void MapFromWorldToObject(const Asura_Vector_3& xWorldPosition, Asura_Vector_2& xGlassCoord) const;

	virtual void MakeClientUpdateMessage();

	Asura_WorldSpace_Rectangle m_xPane;
	Asura_Collision_Pane m_xCollisionPane;

	float m_afU[4];
	float m_afV[4];
	unsigned int m_uColour;
	Asura_String m_xTextureName;
    union
    {
        struct
        {
            u_int m_bUseEnvironmentMap :  1,
                  SpareGlassBits       : 31;
        };

        u_int m_uGlassFlags;
    };

	// Only update the client once...
	bool m_bHasClientBeenUpdated;

private:

    typedef Asura_ServerEntity_PhysicsObject PARENT;

	friend class Tools_Entity_Glass_Object;
	friend class Asura_ClientEntity_Glass;
	friend class Tools_Level_Export;
	friend class Tools_SaveAsura;

	static Asura_Hashed_List<Asura_ServerEntity_Glass> s_xList;

};

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

#endif // #ifndef ASURA_SERVERENTITY_GLASS_H___
