//////////////////////////////////////////////////////////////////////////////
//
// Asura_ServerEntity_LightShaft.h
//
// Copyright (c) 2007 Rebellion (Derby). All rights reserved.
//
// Description:
//
// Volumetric-ish light shafts
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef ___ASURA_SERVERENTITY_LIGHTSHAFT_H___
#define ___ASURA_SERVERENTITY_LIGHTSHAFT_H___ 1

#ifdef ASURA_TARGET_SM3

// I N C L U D E S ///////////////////////////////////////////////////////////

// Engine independent.
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "Core/Asura_String.h"
#include "Entity/Asura_Entity.h"
#include "Entity/LightShaft/Asura_ServerEntity_LightShaft.h"
#include "Rendering/Asura_Colour.h"

// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////////

#ifdef ASURA_TOOLS
class Tools_Entity_LightShaft;
#endif

class Asura_ClientEntity_LightShaft;


// C L A S S E S /////////////////////////////////////////////////////////////

class Asura_LightShaft_Parameters
{
public:
	Asura_Vector_3			m_xPosition;
	Asura_Matrix_3x3		m_xOrientation;

	u_int					m_uFlags;
	
	float					m_afRadii[2];						// top, bottom
	float					m_afDistances[2];					// top, bottom
	Asura_RGBA				m_axColours[2];						

	Asura_Hash_ID			m_axTextureIDs[2];					// 2 layers of texturing
	Asura_String			m_axTextureNames[2];				
	float					m_afTextureUScales[2];
	float					m_afTextureScrollRates[2];	

	// Distance Fade
	float					m_fFadeoutStart;					// Dist at which fadeout starts
	float					m_fFadeoutDist;						// Dist over which fade takes place
	float					m_fFadeCloseStart;					// Dist at which fadeout starts
	float					m_fFadeCloseDist;						// Dist over which fade takes place

	// Corona
	Asura_Hash_ID			m_xCoronaTextureID;
	Asura_String			m_xCoronaTextureName;
	float					m_fCoronaSize;
	float					m_fCoronaAngleRange;
	float					m_fCoronaDepth;						// Amount to move it forwards
	Asura_RGBA				m_xCoronaColour;


	// Default Constructor - Set some sensible defaults
	// As there's a couple of objects in here, we can't just memset it to 0 - so we have to initialise
	// everything here
	// ----------------------------------------------------------------------------------------------------
	Asura_LightShaft_Parameters()
	{
		SetDefaults();
	}


	void SetDefaults()
	{
		m_xPosition.Zero();
		m_xOrientation.Identity();
		
		m_uFlags = 0;

		m_afRadii[0] = 1.0f;
		m_afRadii[1] = 2.0f;

		m_afDistances[0] = 0.0f;
		m_afDistances[1] = 5.0f;

		m_axColours[0] = Asura_RGBA(1.0f,1.0f,1.0f,1.0f);
		m_axColours[1] = Asura_RGBA(0.0f,0.0f,0.0f,1.0f);

		m_axTextureIDs[0] = 0;
		m_axTextureIDs[1] = 0;
		m_axTextureNames[0] = "";
		m_axTextureNames[1] = "";
		m_afTextureUScales[0] = 1.0f;
		m_afTextureUScales[1] = 1.0f;
		m_afTextureScrollRates[0] = 0.0f;
		m_afTextureScrollRates[1] = 0.0f;

		m_fFadeoutStart = 100.0f;
		m_fFadeoutDist  = 50.0f;

		m_fFadeCloseStart = 0.0f;
		m_fFadeCloseDist = 10.0f;

		m_xCoronaTextureID		= 0;
		m_xCoronaTextureName	= "";
		m_fCoronaSize			= 3.0f;
		m_fCoronaAngleRange		= 0.6f;
		m_fCoronaDepth			= 2.0f;
		m_xCoronaColour			= Asura_RGBA(1.0f,1.0f,1.0f,1.0f);
	}


	Asura_LightShaft_Parameters( const Asura_LightShaft_Parameters& xRhs )
	{
		*this = xRhs;
	}

	Asura_LightShaft_Parameters& operator = (const Asura_LightShaft_Parameters& xParams)
	{
		CopyToThis(xParams);
		return *this;
	}

	void CopyToThis(const Asura_LightShaft_Parameters& xParams)
	{
		m_xPosition    = xParams.m_xPosition;
		m_xOrientation = xParams.m_xOrientation;
		m_uFlags	   = xParams.m_uFlags;

		for(u_int u = 0; u < 2; ++u)
		{
			m_afRadii[u]				= xParams.m_afRadii[u];
			m_afDistances[u]			= xParams.m_afDistances[u];
			m_axColours[u]				= xParams.m_axColours[u];
			m_axTextureIDs[u]			= xParams.m_axTextureIDs[u];
			m_axTextureNames[u]			= xParams.m_axTextureNames[u];
			m_afTextureUScales[u]		= xParams.m_afTextureUScales[u];
			m_afTextureScrollRates[u]	= xParams.m_afTextureScrollRates[u];
		}

		m_fFadeoutStart				= xParams.m_fFadeoutStart;
		m_fFadeoutDist				= xParams.m_fFadeoutDist;

		m_fFadeCloseStart				= xParams.m_fFadeCloseStart;
		m_fFadeCloseDist				= xParams.m_fFadeCloseDist;

		m_xCoronaTextureID		= xParams.m_xCoronaTextureID;		
		m_xCoronaTextureName	= xParams.m_xCoronaTextureName;
		m_fCoronaSize			= xParams.m_fCoronaSize;
		m_fCoronaAngleRange		= xParams.m_fCoronaAngleRange;
		m_fCoronaDepth			= xParams.m_fCoronaDepth;
		m_xCoronaColour			= xParams.m_xCoronaColour;
	}

};



class Asura_ServerEntity_LightShaft : public Asura_Entity_Server
{
public:
#ifdef ASURA_TOOLS
	static Asura_ServerEntity_LightShaft* CreateFromTools( Asura_Guid uGuid, const Asura_Vector_3& xCentre );
#endif
	Asura_ServerEntity_LightShaft( Asura_Guid uGuid);

	// C L A S S   M E T H O D S /////////////////////////////////////////
	static void CreateFromChunkStream( Asura_Guid uGuid, Asura_Chunk_Stream &xStream );
	static void ExtractParamsFromChunkStream( Asura_LightShaft_Parameters &xParams , Asura_Chunk_Stream &xStream, u_int uVersion);
	static void WriteParamsToChunkStream( const Asura_LightShaft_Parameters &xParams , Asura_Chunk_Stream& xStream);

	// M E T H O D S /////////////////////////////////////////////////////

	// Query methods.
	const		 Asura_LightShaft_Parameters& GetParams( void ) const { return m_xParams; }

	// Set methods.
#ifdef ASURA_TOOLS
	void		 SetParams( const Asura_LightShaft_Parameters& xNewParams );
#endif

	// Load / save.
	virtual	void ReadFromChunkStream(Asura_Chunk_Stream& xStream);
	virtual	void WriteToChunkStream(Asura_Chunk_Stream& xStream) const;		
	virtual bool CanBeSavedToChunk() const				{ return true;  }
	virtual	bool IsFilledFromChunkBuffer(void) const	{ return false; }

	static Asura_ServerEntity_LightShaft* FindEntity( Asura_Guid uGUID ) { return s_xList.FindEntity( uGUID ); }
	static const Asura_Hashed_List<Asura_ServerEntity_LightShaft>* GetList( void ) { return &s_xList; }

	// Function used to work out how far a point is away from this entity...
	float	CalculateSquaredDistanceTo(const Asura_Vector_3& xPosition) const	{ const Asura_Vector_3 xDistance(m_xParams.m_xPosition - xPosition); return(xDistance.MagnitudeSquared()); }

	#ifdef ASURA_TOOLS
	virtual void ApplyChangesForTemplateInstance(class Tools_EntityTemplateInstance* pxInstance);
	virtual bool CanBeIncludedInTemplate() const		{ return true;  }

	virtual void TraverseUsedTextures( void (*pfnCallback)( int iTextureID, void* pUserData ), void* pUserData ) const;
	#endif

	// S T A T E /////////////////////////////////////////////////////////
	Asura_LightShaft_Parameters m_xParams;

	// C O N S T A N T S /////////////////////////////////////////////////////////

	static const u_int LIGHTSHAFT_CHUNK_VERSION = 4;

protected:
	virtual ~Asura_ServerEntity_LightShaft( void );

private:

	static Asura_Hashed_List<Asura_ServerEntity_LightShaft> s_xList;

	// F R I E N D S /////////////////////////////////////////////////////
#ifdef ASURA_TOOLS
	friend						Tools_Entity_LightShaft;
#endif

	// T Y P E S /////////////////////////////////////////////////////////
	typedef						Asura_Entity_Server PARENT;
};




#endif //#ifdef ASURA_TARGET_SM3
#endif // ___ASURA_SERVERENTITY_LIGHTSHAFT_H___
