//////////////////////////////////////////////////////////////////////////////
//
// Asura_Gamescene_Node_Base.cpp
//
// Created by NJD - 12 October 2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"

#ifdef ASURA_USE_AXON

// header
#include "Entity/Locations/Asura_Gamescene_Node_Base.h"

// asura
#include "ChunkSystem/Asura_Chunk_Macros.h"
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "Core/Memory/Asura_ResourceAllocator.h"
#include "Entity/Asura_Entity_Classification.h"
#if defined(ASURA_USE_STREAMING)
#include "Streaming/Asura_Streaming_System.h"
#endif	// defined(ASURA_USE_STREAMING)

// axon
#include "AnimTemplates/Axon_Gamescene_AnimTemplate_Enums.h"
#include "AnimTemplates/Axon_Gamescene_AnimTemplate_System.h"

// Tools
#ifdef ASURA_TOOLS
#include "EntityTemplate/Tools_EntityTemplate_System.h"
#endif

//////////////////////////////////////////////////////////////////////////////
//
// Asura_Gamescene_Node_Base
//
//////////////////////////////////////////////////////////////////////////////

Asura_Gamescene_Node_Base::Asura_Gamescene_Node_Base()
: m_uGamesceneBaseFlags( 0 )
, m_iAStarIndex( -1 )
, m_fSpecialActionChance( 1.f )
, m_fMinExitWaitTime( 0.0f )
, m_fMaxExitWaitTime( 0.0f )
, m_uAnimationSequenceID( uAXON_ANIMATION_SEQUENCE_ID_NONE )
, m_uExitConditions( ASURA_GAMESCENE_EXIT_NONE )
, m_uSpecialAction( ASURA_GAMESCENE_SPECIAL_ACTION_NONE )
, m_fRangeFromActor( 0.0f )
, m_uRangeActorGuid( ASURA_GUID_UNREGISTERED )
{
	m_bCanCrouch    = true;
	m_bCanStand     = true;
	m_bFollowGround = true;
}

Asura_Gamescene_Node_Base::~Asura_Gamescene_Node_Base()
{
}

void Asura_Gamescene_Node_Base::WriteToChunkStream ( Asura_Chunk_Stream& xStream ) const
{
	// (TI - 5.9.2004) - DON'T FORGET TO UPDATE ApplyChangesForTemplateInstance IF YOU CHANGE STUFF.

	// version
    xStream << static_cast< int >( CURRENT_CHUNK_VERSION );

	// parent - no parent

	// our data
    Asura_WriteToStream_Bitfield_Wrapper< u_int > xFlagsBitfield( m_uGamesceneBaseFlags );
    xFlagsBitfield.Add( m_bCanCrouch, 1 );
    xFlagsBitfield.Add( m_bCanStand, 1 );
    xFlagsBitfield.Add( m_bCompleteBehaviourOnDeparture, 1 );
    xFlagsBitfield.Add( m_bDisableActorCollisions, 1 );
    xFlagsBitfield.Add( m_bDontTestDynamicsPosition, 1 );
    xFlagsBitfield.Add( m_bFollowGround, 1 );
    xFlagsBitfield.Add( m_bIgnoreCombat, 1 );
    xFlagsBitfield.Add( m_bToolsCancelledFollowGround, 1 );
    xStream << xFlagsBitfield;

	xStream << m_fMinExitWaitTime << m_fMaxExitWaitTime;
	xStream << m_fSpecialActionChance;
	xStream << m_fRangeFromActor;
	xStream << m_uRangeActorGuid;
	xStream << m_uAnimationSequenceID;
	xStream << m_uExitConditions;
	xStream << m_uSpecialAction;

	// (TI - 5.9.2004) - DON'T FORGET TO UPDATE ApplyChangesForTemplateInstance IF YOU CHANGE STUFF.
}

void Asura_Gamescene_Node_Base::ReadFromChunkStream( Asura_Chunk_Stream& xStream )
{
	Asura_ResourcePushType(ASURA_RESOURCE_GAMESCENE);

	int iVersion;
	xStream >> iVersion;

    // Parent, if we had one.

    if ( iVersion == CURRENT_CHUNK_VERSION )
    {
        // Us.
        Asura_ReadFromStream_Bitfield_Wrapper< u_int > xFlagsBitfield( m_uGamesceneBaseFlags );
        xStream >> xFlagsBitfield;

        xStream >> m_fMinExitWaitTime >> m_fMaxExitWaitTime;
        xStream >> m_fSpecialActionChance;
        xStream >> m_fRangeFromActor;
        xStream >> m_uRangeActorGuid;
#if defined( ASURA_USE_STREAMING ) && !defined( ASURA_TOOLS )
        m_uRangeActorGuid	= Asura_Streaming_System::MakeCurrentMasterZoneGuid( m_uRangeActorGuid );
#endif	// defined( ASURA_USE_STREAMING ) && !defined( ASURA_TOOLS )
        xStream >> m_uAnimationSequenceID;
        xStream >> m_uExitConditions;
        xStream >> m_uSpecialAction;
    }
	else if ( iVersion == 1 )
    {
        // Us.
        Asura_ReadFromStream_Bitfield_Wrapper< u_int > xFlagsBitfield( m_uGamesceneBaseFlags );
        xStream >> xFlagsBitfield;

        xStream >> m_fMinExitWaitTime;
		m_fMaxExitWaitTime = m_fMinExitWaitTime;

        xStream >> m_fSpecialActionChance;
        xStream >> m_fRangeFromActor;
        xStream >> m_uRangeActorGuid;
#if defined( ASURA_USE_STREAMING ) && !defined( ASURA_TOOLS )
        m_uRangeActorGuid	= Asura_Streaming_System::MakeCurrentMasterZoneGuid( m_uRangeActorGuid );
#endif	// defined( ASURA_USE_STREAMING ) && !defined( ASURA_TOOLS )
        xStream >> m_uAnimationSequenceID;
        xStream >> m_uExitConditions;
        xStream >> m_uSpecialAction;
    }
    else if ( iVersion == 0 )
	{
        u_int uGuard;
        xStream >> uGuard;
        Asura_Assert( uGuard == 0x33333333, "" );

		// Us.
        Asura_ReadFromStream_Bitfield_Wrapper< u_int > xFlagsBitfield( m_uGamesceneBaseFlags );
        xStream >> xFlagsBitfield;

		xStream >> m_fMinExitWaitTime;
		m_fMaxExitWaitTime = m_fMinExitWaitTime;

		xStream >> m_fSpecialActionChance;
		xStream >> m_fRangeFromActor;
		xStream >> m_uRangeActorGuid;
#if defined( ASURA_USE_STREAMING ) && !defined( ASURA_TOOLS )
		m_uRangeActorGuid	= Asura_Streaming_System::MakeCurrentMasterZoneGuid( m_uRangeActorGuid );
#endif	// defined( ASURA_USE_STREAMING ) && !defined( ASURA_TOOLS )
		xStream >> m_uAnimationSequenceID;

		xStream >> m_uExitConditions;
		xStream >> m_uSpecialAction;

        xStream >> uGuard;
        Asura_Assert( uGuard == 0x33333333, "" );
    }
	else
	{
		Asura_Assertf( false, ( "Asura_Gamescene_Node_Base::InitialiseFromChunkStream - unhandled chunk version (%d)", iVersion ) );
	}

	Asura_ResourcePopType();

	if ( m_uAnimationSequenceID != uAXON_ANIMATION_SEQUENCE_ID_NONE )
	{
		u_int uIndex = Axon_Gamescene_AnimTemplate_System::GetIndexFromUniqueID( m_uAnimationSequenceID );

		if ( uIndex == uAXON_ANIMATION_SEQUENCE_INDEX_NONE )
			m_uAnimationSequenceID = uAXON_ANIMATION_SEQUENCE_ID_NONE;
	}
}

bool Asura_Gamescene_Node_Base::IsBaseOrDerivativeOf(u_short usClassification) const
{
    // Buck stops here with this function, apparently.
	return usClassification == AsuraEntityClass_GamesceneBase;
}

#ifdef ASURA_TOOLS
void Asura_Gamescene_Node_Base::ApplyChangesForTemplateInstance( class Tools_EntityTemplateInstance* pxInstance )
{
	m_uRangeActorGuid = pxInstance->ConvertGuid( m_uRangeActorGuid );
}
#endif

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

#endif // ASURA_USE_AXON
