//////////////////////////////////////////////////////////////////////////////
//
// Asura_ServerEntity_Node.h
//
// Created by NJD - 11 October 2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __ASURA_SERVERENTITY_NODE_H__
#define __ASURA_SERVERENTITY_NODE_H__ 1

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// asura
#include "Core/Asura_HashedList.h"
#include "Entity/Asura_Entity.h"

// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////////

class Tools_EntityTemplateInstance;

//////////////////////////////////////////////////////////////////////////////
//
// Asura_ServerEntity_Node
//
//////////////////////////////////////////////////////////////////////////////

class Asura_ServerEntity_Node : public Asura_Entity_Server
{
    typedef Asura_Entity_Server PARENT;
    friend class Tools_Entity_Node;

public:
    Asura_ServerEntity_Node(Asura_Guid uGuid);
    virtual ~Asura_ServerEntity_Node();

    virtual bool HasClientEntities() const { return false; }

    // normal accessor functions
    virtual bool          IsAvailable()      const { return (IsEntityActive() && !m_bIsOccupied); }
    bool                  IsOccupied()       const { return m_bIsOccupied; } 
    bool                  IsValid()          const { return m_bIsValid; }
    bool                  IsSmoothOverNode() const { return m_bIsSmoothOverNode; }
    const Asura_Vector_3& GetPosition()      const { return m_xPosition; }

    void		 SetIsActive( const bool bIsActive )         { SetEntityActive( bIsActive ); }
    virtual void SetIsOccupied( const bool bIsOccupied )     { m_bIsOccupied = bIsOccupied; }
    void		 SetPosition( const Asura_Vector_3& xPos )   { m_xPosition   = xPos; }

    // Functions to enable nodes to be used with KD trees.
    u_int GetSplittingPlane() const { return static_cast< u_int >( m_usSplittingPlane ); }
    void  SetSplittingPlane( u_int uSplittingPlane ) { m_usSplittingPlane = static_cast< u_short >( uSplittingPlane ); }
    float GetElement( u_int uAxis ) const { return m_xPosition.GetElement( uAxis ); }

    u_int GetNumberLinks() const { return m_uNumberOfSplines; }
    Asura_Guid GetLink( u_int uIndex ) const 
    {
        Asura_Assert(uIndex < m_uNumberOfSplines, "Sanity check");
        return m_puSplines[uIndex];
    }
	u_int GetNumIncomingLinks() const;

	virtual bool IsBaseOrDerivativeOf( u_short usClassification ) const;

#ifndef ASURA_NO_DEBUG_OUTPUT
	virtual void RenderDebugInfo();
#endif

#ifdef ASURA_TOOLS
	virtual void ApplyChangesForTemplateInstance( Tools_EntityTemplateInstance* pxInstance );
    virtual bool CanBeIncludedInTemplate() const;

    // Function to be overwritten providing a validity check. Invalid nodes are not exported.
    virtual void CheckForValidity() { SetIsValid( true ); }
    virtual bool ShouldIncludeInLevelExport() const { return IsValid(); }

    void SetIsValid( const bool bIsValid )                   { m_bIsValid = bIsValid; }
    void SetIsSmoothOverNode( const bool bIsSmoothOverNode ) { m_bIsSmoothOverNode = bIsSmoothOverNode; }

    virtual void AddNewSpline( const Asura_Guid uGuid );
    virtual void RemoveSpline( const Asura_Guid uGuid );

	u_int GetToolsObjectColour() const { return m_uToolsObjectColour; }
	void SetToolsObjectColour( u_int uColour ) { m_uToolsObjectColour = uColour; }
#endif

	static void CreateFromChunkStream( Asura_Guid uGuid, Asura_Chunk_Stream& xStream );
	virtual bool CanBeSavedToChunk() const { return true; }
	virtual bool IsFilledFromChunkBuffer() const { return false; }
	virtual void WriteToChunkStream( Asura_Chunk_Stream& xStream ) const;
	virtual void ReadFromChunkStream( Asura_Chunk_Stream& xStream );

	static void TraverseStructure( const Asura_Guid uStartGuid, void(*pfnFunction)(Asura_ServerEntity_Node*,void*), void* pUserData );

	static Asura_Hashed_List< Asura_ServerEntity_Node >* GetList() { return &s_xNodeList; }
	static Asura_ServerEntity_Node* FindEntity( const Asura_Guid uGuid ) { return s_xNodeList.FindEntity( uGuid ); }

#ifndef ASURA_NO_DEBUG_OUTPUT
protected:
	const Asura_Vector_3& GetDebugTextPos() { return m_xDebugTextPos; }
	void IncrementDebugTextPos() { m_xDebugTextPos.y += 10.0f; }
	bool DebugTextPosOnScreen() const { return m_bDebugTextPosOnScreen; }
#endif

private:
	union
	{
		u_int m_uNodeFlags;
		struct
		{
			u_int   m_bObsoleteIsActive     : 1,
					m_bIsOccupied           : 1,
					m_usSplittingPlane      : 2,
					m_uNumberOfSplines      : 8,
					m_bIsSmoothOverNode     : 1,
					m_bIsValid              : 1,
				    m_uTraverseHelper       : 2,
					SpareBits_NodeFlags     : 16;
		};
	};
	Asura_Vector_3 m_xPosition;
	Asura_Guid*    m_puSplines;

#ifdef ASURA_TOOLS
	u_int m_uToolsObjectColour;
#endif

#ifndef ASURA_NO_DEBUG_OUTPUT
	Asura_Vector_3 m_xDebugTextPos;
	bool           m_bDebugTextPosOnScreen;
#endif

private:
	enum { CURRENT_CHUNK_VERSION = 3 };
	static Asura_Hashed_List< Asura_ServerEntity_Node > s_xNodeList;
};

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

#endif //__ASURA_SERVERENTITY_NODE_H__
