//////////////////////////////////////////////////////////////////////////////
//
// Asura_ServerNode_Coverpoint.cpp
//
// Created by NJD - 12 October 2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"

#ifdef ASURA_USE_AXON

// header
#include "Entity/Locations/OrientedNode/CoverpointNode/Asura_ServerNode_Coverpoint.h"

// asura
#include "AI/Asura_AI_Navigation.h"
#include "ChunkSystem/Asura_Chunk_Macros.h"
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "Dynamics/Asura_Collision_Object.h"
#include "Entity/Asura_Entity_Classification.h"
#include "Mesh/Asura_Mesh_PolygonFlags.h"
#include "Physics/Asura_Physics_Object.h"

#ifdef ASURA_TOOLS
#include "EntityTemplate/Tools_EntityTemplate_System.h"
#include "Waypointing/Tools_Mode_Waypointing.h"
#endif

//////////////////////////////////////////////////////////////////////////////
//
// Asura_ServerNode_Coverpoint
//
//////////////////////////////////////////////////////////////////////////////

u_int                                            Asura_ServerNode_Coverpoint::s_uNumValidCoverpoints = 0;
Asura_Hashed_List< Asura_ServerNode_Coverpoint > Asura_ServerNode_Coverpoint::s_xValidCoverpointList;

Asura_ServerNode_Coverpoint::Asura_ServerNode_Coverpoint( Asura_Guid uGuid )
: PARENT( uGuid )
, m_fCosHalfFOV( Asura_Maths::Cosine( Asura_Maths::DegToRad( 90.0f ) * 0.5f ) )
, m_fCoverAngle( 90.0f )
, m_fMinCoverRange( 0.0f )
, m_fMaxCoverRange( 40.0f )
, m_uCoverpointDataBitfield( 0 )
{
    Classification = AsuraEntityClass_CoverpointNode;

	m_bIsLowCover = false;
    m_usNearestWaypoint = USHRT_MAX;

    for ( u_int u = 0; u < MAX_NUM_EMERGE_POINTS; ++u )
    {
		m_axEmergePoint[ u ].m_xPosition	= Asura_Maths::ZeroVector3;
		m_axEmergePoint[ u ].m_xOrientation = Asura_Maths::ZeroEuler3;
        m_aucEmergePointFlags[ u ].m_ucEmergePointFlags = 0;
    }

#ifdef ASURA_TOOLS
    // We don't know if we're valid yet.
    SetIsValid( false );
#endif
}

Asura_ServerNode_Coverpoint::~Asura_ServerNode_Coverpoint()
{
#ifdef ASURA_TOOLS
    if ( IsValid() )
#endif
    {
        s_xValidCoverpointList.RemoveFromList( this );
        --s_uNumValidCoverpoints;
    }
}

void Asura_ServerNode_Coverpoint::CreateFromChunkStream( Asura_Guid uGuid, Asura_Chunk_Stream& xStream )
{
    Asura_ServerNode_Coverpoint* pxCoverpointNode = new Asura_ServerNode_Coverpoint( uGuid );
    if ( pxCoverpointNode )
    {
        pxCoverpointNode->ReadFromChunkStream( xStream );
    }
}

void Asura_ServerNode_Coverpoint::ReadFromChunkStream( Asura_Chunk_Stream& xStream )
{
    int iVersion;
    xStream >> iVersion;

    // Parent.
    PARENT::ReadFromChunkStream( xStream );

    // Us.
    if ( iVersion == CURRENT_CHUNK_VERSION )
	{
		xStream >> m_fCoverAngle;
        xStream >> m_fMinCoverRange >> m_fMaxCoverRange;

        Asura_ReadFromStream_Bitfield_Wrapper< u_int > xDataBitfield( m_uCoverpointDataBitfield );
        xStream >> xDataBitfield;

        for ( u_int u = 0; u < m_uEmergepointCount; ++u )
        {
            Asura_ReadFromStream_Bitfield_Wrapper< u_char > xEmergePointBitfield( m_aucEmergePointFlags[ u ].m_ucEmergePointFlags );
            xStream >> xEmergePointBitfield;
        }

        // Read in padding as well.
        const u_int uNumPadBytes = ( ( 4 - ( m_uEmergepointCount & 3 ) ) & 3 );
        for ( u_int u = 0; u < uNumPadBytes; ++u )
        {
            u_char ucPadding;
            xStream >> ucPadding;
        }

        for ( u_int u = 0; u < m_uEmergepointCount; ++u )
        {
            xStream >> m_axEmergePoint[ u ].m_xPosition;
			xStream >> m_axEmergePoint[ u ].m_xOrientation;
        }
	}
	else if ( iVersion == 6 )
	{
		xStream >> m_fCoverAngle;
        xStream >> m_fMinCoverRange >> m_fMaxCoverRange;

        Asura_ReadFromStream_Bitfield_Wrapper< u_int > xDataBitfield( m_uCoverpointDataBitfield );
        xStream >> xDataBitfield;

        for ( u_int u = 0; u < m_uEmergepointCount; ++u )
        {
            Asura_ReadFromStream_Bitfield_Wrapper< u_char > xEmergePointBitfield( m_aucEmergePointFlags[ u ].m_ucEmergePointFlags );
            xStream >> xEmergePointBitfield;
        }

        // Read in padding as well.
        const u_int uNumPadBytes = ( ( 4 - ( m_uEmergepointCount & 3 ) ) & 3 );
        for ( u_int u = 0; u < uNumPadBytes; ++u )
        {
            u_char ucPadding;
            xStream >> ucPadding;
        }

        for ( u_int u = 0; u < m_uEmergepointCount; ++u )
        {
            xStream >> m_axEmergePoint[ u ].m_xPosition;
        }
	}
    else if ( iVersion == 5 )
    {
        xStream >> m_fCoverAngle;
        xStream >> m_fMaxCoverRange;
		m_fMinCoverRange = 0.f;

        Asura_ReadFromStream_Bitfield_Wrapper< u_int > xDataBitfield( m_uCoverpointDataBitfield );
        xStream >> xDataBitfield;

        for ( u_int u = 0; u < m_uEmergepointCount; ++u )
        {
            Asura_ReadFromStream_Bitfield_Wrapper< u_char > xEmergePointBitfield( m_aucEmergePointFlags[ u ].m_ucEmergePointFlags );
            xStream >> xEmergePointBitfield;
        }

        // Read in padding as well.
        const u_int uNumPadBytes = ( ( 4 - ( m_uEmergepointCount & 3 ) ) & 3 );
        for ( u_int u = 0; u < uNumPadBytes; ++u )
        {
            u_char ucPadding;
            xStream >> ucPadding;
        }

        for ( u_int u = 0; u < m_uEmergepointCount; ++u )
        {
            xStream >> m_axEmergePoint[ u ].m_xPosition;
        }
    }
#ifdef ASURA_TOOLS
    else if ( iVersion == 4 )
    {
        xStream >> m_fCoverAngle;
        xStream >> m_fMaxCoverRange;
		m_fMinCoverRange = 0.f;

        Asura_ReadFromStream_Bitfield_Wrapper< u_int > xDataBitfield( m_uCoverpointDataBitfield );
        xStream >> xDataBitfield;

        for ( u_int u = 0; u < m_uEmergepointCount; ++u )
        {
            xStream >> m_axEmergePoint[ u ].m_xPosition;
        }
        for ( u_int u = 0; u < m_uEmergepointCount; ++u )
        {
            u_char ucDummy;
            xStream >> ucDummy;
            m_aucEmergePointFlags[ u ].m_bExists = true;
        }

        // Read in padding as well.
        const u_int uNumPadBytes = ( ( 4 - ( m_uEmergepointCount & 3 ) ) & 3 );
        for ( u_int u = 0; u < uNumPadBytes; ++u )
        {
            u_char ucPadding;
            xStream >> ucPadding;
        }
    }
    else if ( iVersion == 3 )
    {
        xStream >> m_fCoverAngle;
        xStream >> m_fMaxCoverRange;
		m_fMinCoverRange = 0.f;

        Asura_ReadFromStream_Bitfield_Wrapper< u_int > xDataBitfield( m_uCoverpointDataBitfield );
        xStream >> xDataBitfield;

        for ( u_int u = 0; u < m_uEmergepointCount; ++u )
        {
            xStream >> m_axEmergePoint[ u ].m_xPosition;
        }
        for ( u_int u = 0; u < m_uEmergepointCount; ++u )
        {
            u_char ucExists;
            xStream >> ucExists;
            m_aucEmergePointFlags[ u ].m_bExists = ( ucExists ? true : false );
        }
    }
    else if ( iVersion == 2 )
    {
        xStream >> m_fCoverAngle;
        xStream >> m_fMaxCoverRange;
		m_fMinCoverRange = 0.f;

        Asura_ReadFromStream_Bitfield_Wrapper< u_int > xDataBitfield( m_uCoverpointDataBitfield );
        xStream >> xDataBitfield;
    }
    else if ( iVersion == 1 )
    {
        xStream >> m_fCoverAngle;
        xStream >> m_fMaxCoverRange;
		m_fMinCoverRange = 0.f;

        Asura_ReadFromStream_Bitfield_Wrapper< u_int > xFlagsBitfield( m_uCoverpointDataBitfield );
        xStream >> xFlagsBitfield;

        int iEmergepointCount;
        xStream >> iEmergepointCount;
        m_uEmergepointCount = static_cast< u_int >( iEmergepointCount );
    }
    else if ( iVersion == 0 )
    {
		xStream >> m_fCoverAngle;
        xStream >> m_fMaxCoverRange;
		m_fMinCoverRange = 0.f;

		xStream >> m_uCoverpointDataBitfield;
        int iEmergepointCount;
        xStream >> iEmergepointCount;
        m_uEmergepointCount = static_cast< u_int >( iEmergepointCount );
    }
#endif
    else
    {
        Asura_Assertf( false, ( "Asura_ServerNode_Coverpoint::ReadFromChunkStream - unhandled chunk version (%d) - may still be handled in tools", iVersion ) );
    }

	m_fCosHalfFOV = Asura_Maths::Cosine( Asura_Maths::DegToRad( m_fCoverAngle ) * 0.5f );

    if ( IsValid() )
    {
        s_xValidCoverpointList.AddToList( this );
        ++s_uNumValidCoverpoints;
    }
}

void Asura_ServerNode_Coverpoint::WriteToChunkStream( Asura_Chunk_Stream& xStream ) const
{
    // (TI - 5.9.2004) - DON'T FORGET TO UPDATE ApplyChangesForTemplateInstance IF YOU CHANGE STUFF.

    // Version
    xStream << static_cast< int >( CURRENT_CHUNK_VERSION );

    // Parent
    PARENT::WriteToChunkStream(xStream);

    // Our data
	xStream << m_fCoverAngle;
    xStream << m_fMinCoverRange << m_fMaxCoverRange;

    Asura_WriteToStream_Bitfield_Wrapper< u_int > xFlagsBitfield( m_uCoverpointDataBitfield );
    xFlagsBitfield.Add( m_bIsLowCover,          1  );
    xFlagsBitfield.Add( m_uEmergepointCount,    2  );
    xFlagsBitfield.Add( 0,                      13 );
    xFlagsBitfield.Add( m_usNearestWaypoint,    16 );
    xStream << xFlagsBitfield;

    for ( u_int u = 0; u < MAX_NUM_EMERGE_POINTS; ++u )
    {
        if ( m_aucEmergePointFlags[ u ].m_bExists )
        {
            Asura_WriteToStream_Bitfield_Wrapper< u_char > xEmergePointBitfield( m_aucEmergePointFlags[ u ].m_ucEmergePointFlags );
            xEmergePointBitfield.Add( m_aucEmergePointFlags[ u ].m_bExists,  1 );
            xEmergePointBitfield.Add( m_aucEmergePointFlags[ u ].m_bIsValid, 1 );
            xEmergePointBitfield.Add( 0,                                     6 );
            xStream << xEmergePointBitfield;
        }
    }
    // Pad to 4-byte boundary.
    const u_int uNumPadBytes = ( ( 4 - ( m_uEmergepointCount & 3 ) ) & 3 );
    for ( u_int u = 0; u < uNumPadBytes; ++u )
    {
        xStream << static_cast< u_char >( 0 );
    }

    for ( u_int u = 0; u < MAX_NUM_EMERGE_POINTS; ++u )
    {
        if ( m_aucEmergePointFlags[ u ].m_bExists )
        {
			xStream << m_axEmergePoint[ u ].m_xPosition;
			xStream << m_axEmergePoint[ u ].m_xOrientation;
        }
    }

    // (TI - 5.9.2004) - DON'T FORGET TO UPDATE ApplyChangesForTemplateInstance IF YOU CHANGE STUFF.
}

bool Asura_ServerNode_Coverpoint::ProvidesCoverFrom( const Asura_Vector_3& xPosition ) const
{
    // Check whether the position is within our swept area
    Asura_Vector_3		xToPosition = xPosition - GetPosition();

	Asura_Matrix_3x3	xOri;
	GetOrientation( xOri );
	Asura_Vector_3		xDirection = Asura_Vector_3(0.f, 0.f, 1.f) * xOri;

	xToPosition.y = 0.0f;
	xDirection.y = 0.0f;

    const float			fRangeSq = xToPosition.MagnitudeSquared();

	if ( fRangeSq < m_fMinCoverRange * m_fMinCoverRange )
		return false;

    if ( xToPosition * xDirection < m_fCosHalfFOV * Asura_Maths::Sqrt( fRangeSq ) )
    {
        return false;
    }

    return true;
}

bool Asura_ServerNode_Coverpoint::ProvidesCoverFrom( int iThreatCount, const Asura_Vector_3* axThreatPositions ) const
{
    for ( int i = 0; i < iThreatCount; ++i )
    {
        if ( !ProvidesCoverFrom( axThreatPositions[ i ] ) )
        {
            return false;
        }
    }

    return true;
}

const Asura_Vector_3& Asura_ServerNode_Coverpoint::GetEmergePointPosition( unsigned int uEmergepointIndex )	const 
{ 
	if ( uEmergepointIndex < m_uEmergepointCount && uEmergepointIndex < MAX_NUM_EMERGE_POINTS )
	{
		return m_axEmergePoint[uEmergepointIndex].m_xPosition;
	}
	else
	{
		Asura_Assert(false, "Attempted to access a non-existant emerge point (from ServerNode_Coverpoint)");
		return Asura_Maths::ZeroVector3;
	}
}

// ARC - 10 February 2009 12:36:41 - Added missing implementation
void Asura_ServerNode_Coverpoint::SetEmergePointPosition( unsigned int uEmergepointIndex, const Asura_Vector_3& xPosition )
{
	if ( uEmergepointIndex < m_uEmergepointCount && uEmergepointIndex < MAX_NUM_EMERGE_POINTS )
	{
		m_axEmergePoint[uEmergepointIndex].m_xPosition = xPosition;
	}
	else
	{
		Asura_Assert(false, "Attempted to access a non-existant emerge point (from ServerNode_Coverpoint)");
	}
}

const Asura_Euler_3& Asura_ServerNode_Coverpoint::GetEmergePointOrientation( unsigned int uEmergepointIndex )	const 
{ 
	if ( uEmergepointIndex < m_uEmergepointCount && uEmergepointIndex < MAX_NUM_EMERGE_POINTS )
	{
		return m_axEmergePoint[uEmergepointIndex].m_xOrientation;
	}
	else
	{
		Asura_Assert(false, "Attempted to access a non-existant emerge point (from ServerNode_Coverpoint)");
		return Asura_Maths::ZeroEuler3;
	}
}

bool Asura_ServerNode_Coverpoint::IsBaseOrDerivativeOf(u_short usClassification) const
{
    if (usClassification == AsuraEntityClass_CoverpointNode)
    {
        return true;
    }

    return PARENT::IsBaseOrDerivativeOf(usClassification);
}

#ifdef ASURA_TOOLS
bool Asura_ServerNode_Coverpoint::CanBeIncludedInTemplate() const 
{
    return ( Classification == AsuraEntityClass_CoverpointNode );
}

void Asura_ServerNode_Coverpoint::ApplyChangesForTemplateInstance( Tools_EntityTemplateInstance* pxInstance )
{
    PARENT::ApplyChangesForTemplateInstance( pxInstance );

    for ( u_int u = 0; u < m_uEmergepointCount; ++u )
    {
        m_axEmergePoint[ u ].m_xPosition    = pxInstance->TransformPosition( m_axEmergePoint[ u ].m_xPosition );
		m_axEmergePoint[ u ].m_xOrientation = pxInstance->TransformEuler( m_axEmergePoint[ u ].m_xOrientation );
    }
}

void Asura_ServerNode_Coverpoint::CheckForValidity()
{
    SetIsValid( false );

    static const Asura_Vector_3 s_xMovementVector( 0.0f, -0.05f, 0.0f );
    static const float          s_fAllowedObjectMovement = 0.25f;
    static const Asura_Vector_3 s_xAllowedOffsetVector( 0.2f, 0.25f, 0.2f );

    // Check that we can put a box at the coverpoint position.
    Asura_Vector_3 xFinalPosition;
    if ( !Tools_Mode_Waypointing::PlaceObjectAt( GetPosition(), s_xMovementVector, s_fAllowedObjectMovement * s_fAllowedObjectMovement,
                                                 Tools_Mode_Waypointing::GetTestCollisionBox(), AsuraPolygonFlag_NoCollisions,
                                                 Asura_Maths::IdentityMatrix3x3, xFinalPosition ) )
    {
        return;
    }

    // Check that the coverpoint position is inside a waypoint.
    const u_short usNearest = Asura_AI_Navigation_System::FindWaypointNearestTo( GetPosition() );
    if ( usNearest == ASURA_INVALID_WAYPOINT_ID )
    {
        return;
    }
    const Asura_AI_NavVertex* pxNearest = Asura_AI_Navigation_System::GetWaypoint( usNearest );
    if ( ( pxNearest->m_xPosition - GetPosition() ).Magnitude() > pxNearest->GetRadius() )
    {
        return;
    }

    // Check that the coverpoint emerge points are in valid positions and that we can move to and from them.
    Asura_Physics_Object* pxPhysicsObject = Asura_Physics_Object::Create( ASURA_PHYSICS_OBJECT_ACTOR );
    if ( !pxPhysicsObject ) { return; }
    pxPhysicsObject->SetLinVelocity( Asura_Maths::ZeroVector3 );
    pxPhysicsObject->SetAxisConstraint( Asura_Physics_Object::AXIS_CONSTRAINT_ALL_AXES );
    pxPhysicsObject->SetFeelsGravity( true );
    pxPhysicsObject->SetOnlyCollideWithEnvironmentAndStaticObjects( true );
    pxPhysicsObject->SetCollisionObject( &Tools_Mode_Waypointing::GetTestCollisionBox() );
    pxPhysicsObject->SetIgnore( false );
    pxPhysicsObject->SetReportObjectCollisions( true );
    pxPhysicsObject->SetReportEnvironmentCollisions( true );

    bool bFoundValidEmergePoint = false;
    for ( u_int u = 0; u < MAX_NUM_EMERGE_POINTS; ++u )
    {
        m_aucEmergePointFlags[ u ].m_bIsValid = false;

        if ( m_aucEmergePointFlags[ u ].m_bExists )
        {
            // Check that the emerge point position is inside a waypoint.
            const int                 iNearest  = Asura_AI_Navigation_System::FindWaypointNearestTo( m_axEmergePoint[ u ].m_xPosition );
            const Asura_AI_NavVertex* pxNearest = Asura_AI_Navigation_System::GetWaypoint( iNearest );
            if ( ( pxNearest->m_xPosition - m_axEmergePoint[ u ].m_xPosition ).Magnitude() > pxNearest->GetRadius() )
            {
                continue;
            }

            // Check that we can put a box at the emerge point position.
            if ( !Tools_Mode_Waypointing::PlaceObjectAt( m_axEmergePoint[ u ].m_xPosition, s_xMovementVector, s_fAllowedObjectMovement * s_fAllowedObjectMovement,
                                                         Tools_Mode_Waypointing::GetTestCollisionBox(), AsuraPolygonFlag_NoCollisions,
                                                         Asura_Maths::IdentityMatrix3x3, xFinalPosition ) )
            {
                continue;
            }

            // Check we can get to the emerge point from the coverpoint (we don't want to fall much either).
            float fMaxFallDist;
            if ( !Tools_Mode_Waypointing::MoveToDestination( GetPosition(), m_axEmergePoint[ u ].m_xPosition, s_xAllowedOffsetVector, pxPhysicsObject, fMaxFallDist ) ||
                fMaxFallDist > 0.25f )
            {
                continue;
            }

            // Check we can get to the coverpoint from the emerge point (we don't want to fall much either).
            if ( !Tools_Mode_Waypointing::MoveToDestination( m_axEmergePoint[ u ].m_xPosition, GetPosition(), s_xAllowedOffsetVector, pxPhysicsObject, fMaxFallDist ) ||
                fMaxFallDist > 0.25f )
            {
                continue;
            }

            bFoundValidEmergePoint = true;
            m_aucEmergePointFlags[ u ].m_bIsValid = true;
        }
    }

    Asura_Physics_Object::Destroy( pxPhysicsObject );

    // Check we have some emerge points left if we are high cover.
    if ( !bFoundValidEmergePoint && !m_bIsLowCover )
    {
        return;
    }

    // We are valid.
    SetIsValid( true );
}
#endif

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

#endif //ASURA_USE_AXON
