#ifdef ASURA_USE_AXON

//////////////////////////////////////////////////////////////////////////////
//
// Asura_ServerNode_GamesceneAttractor.h
//
// Created by NJD - 13 October 2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __ASURA_SERVERNODE_GAMESCENEATTRACTOR_H__
#define __ASURA_SERVERNODE_GAMESCENEATTRACTOR_H__ 1

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// asura
#include "Entity/Locations/OrientedNode/GamesceneNode/Asura_ServerNode_GamesceneNode.h"

//////////////////////////////////////////////////////////////////////////////
//
// Support enumerations.
//
//////////////////////////////////////////////////////////////////////////////

// (AW - 18.11.2004) - Existing enum cannot be changed. 
// Please update s_aszAttractorTargetTypes if you add any new entries.
enum ASURA_ATTRACTOR_TARGET_TYPE
{
    ASURA_ATTRACTOR_TARGET_TYPE_GUID                   = 0,
    ASURA_ATTRACTOR_TARGET_TYPE_ANYONE,

    NUM_ASURA_ATTRACTOR_TARGET_TYPES,

    ASURA_ATTRACTOR_TARGET_TYPE_FIRST_PROJECT_SPECIFIC = 1000,
};

// (AW - 18.11.2004) - Existing enum cannot be changed. 
// Please update s_aszAttractorTargetConditions if you add any new entries.
enum ASURA_ATTRACTOR_CONDITION
{
    ASURA_ATTRACTOR_CONDITION_RANGE                  = 0,

    NUM_ASURA_ATTRACTOR_CONDITIONS,

    ASURA_ATTRACTOR_CONDITION_FIRST_PROJECT_SPECIFIC = 1000,
};

class Asura_Message;
class Asura_ServerEntity_PhysicalObject;
class Asura_ServerNode_GamesceneAttractor;
class Tools_EntityTemplateInstance;

// T Y P E D E F S ///////////////////////////////////////////////////////////

typedef bool ProjectAttractServerEntityFunction(const Asura_Guid uGuidToAttract,
												const Asura_ServerNode_GamesceneAttractor* pxAttractor);

//////////////////////////////////////////////////////////////////////////////
//
// Asura_ServerNode_GamesceneAttractor
//
//////////////////////////////////////////////////////////////////////////////

class Asura_ServerNode_GamesceneAttractor : public virtual Asura_ServerNode_GamesceneNode
{
    friend class Tools_EntityNode_AttractorNode;
    typedef Asura_ServerNode_GamesceneNode PARENT;

public:
    Asura_ServerNode_GamesceneAttractor( Asura_Guid uGuid );
	virtual ~Asura_ServerNode_GamesceneAttractor();

    virtual bool HandleMessage( const Asura_Message* pxMessage );
    virtual void OnEntityActivateMessage();
    virtual void OnEntityDeactivateMessage();
	virtual void Update();

	Asura_Guid GetLockedByGuid()    const { return m_uLockedByGuid; }
	u_int      GetTargetCondition() const { return m_uTargetCondition; }
	Asura_Guid GetTargetGuid()      const { return m_uTargetGuid; };
	float      GetTargetRange()     const { return m_fTargetRange; }
	u_int      GetTargetType()      const { return m_uTargetType; }
	bool       IsAttactorLocked()   const { return (m_uLockedByGuid != ASURA_GUID_UNREGISTERED); }

	void SetAttractorLockedByGuid(Asura_Guid uAttractorLock) { m_uLockedByGuid = uAttractorLock; }
	void SetTargetGuid(Asura_Guid uTargetGuid)               { m_uTargetGuid = uTargetGuid; };

	virtual bool IsBaseOrDerivativeOf(u_short usClassification) const;
#ifdef ASURA_TOOLS
	virtual void ApplyChangesForTemplateInstance( Tools_EntityTemplateInstance* pxInstance);
	virtual bool CanBeIncludedInTemplate() const;
#endif

	static void CreateFromChunkStream( Asura_Guid uGuid, Asura_Chunk_Stream& xStream );
    virtual void ReadFromChunkStream( Asura_Chunk_Stream& xStream );
    virtual void WriteToChunkStream (Asura_Chunk_Stream& xStream) const;

	static void SetProjectAttractServerEntityFunction(ProjectAttractServerEntityFunction* const pfnProjectAttractServerEntity)
										{ s_pfnProjectAttractServerEntity = pfnProjectAttractServerEntity; }

	static Asura_Hashed_List< Asura_ServerNode_GamesceneAttractor >* GetList() { return &s_xAttractorList; }
	static Asura_ServerNode_GamesceneAttractor* FindEntity( const Asura_Guid uGuid ) { return s_xAttractorList.FindEntity( uGuid ); }

protected:
	//Checks on all entities and if entity is not dead calls SendAttractionPulseToEntity.
	virtual void SendAttractionPulse();
	//Message the entity that locked us to say that we're no longer worth going to.
	virtual void CancelAttraction();
	//Checks Entity is within range and sends an attraction pulse message
	virtual void SendAttractionPulseToEntity( const Asura_ServerEntity_PhysicalObject* const pxPhysicalObject ) const;
	//Checks Entity is in the same waypoint zone as this attractor
	//virtual bool IsActorInSameWaypointZone(const Asura_Guid uGuid) const;

	u_int      m_uTargetType;
	Asura_Guid m_uTargetGuid;
	u_int      m_uTargetCondition;
	float      m_fTargetRange;
	float	   m_fAttractionSpeed;
	union
	{
		u_int m_uAttractorFlags;
		struct 
		{
            u_int   m_bDEPRECATED                   : 1,
                    m_bOverrideGamesceneBehaviour   : 1,
                    SpareBits_AttractorFlags		: 30;
		};
	};

	float        m_fCheckTimer;
	unsigned int m_uEntityMaskingCount;
	Asura_Guid   m_uLockedByGuid;        // This isn't saved because the free-AI won't remember who locked it.
	//int        m_iNearestWaypointZone; // This isn't saved as waypointing might have changed.

	static ProjectAttractServerEntityFunction* s_pfnProjectAttractServerEntity;

private:
    enum { CURRENT_CHUNK_VERSION = 2 };
	static Asura_Hashed_List< Asura_ServerNode_GamesceneAttractor > s_xAttractorList;
};

// M E S S A G E   S T R U C T U R E S ///////////////////////////////////////
struct Asura_Msg_AIAttractor_Set_Flags
{
	enum { CURRENT_VERSION = 0 };
	unsigned int	m_uVersion;
	
	union
	{
		u_int	m_uAttractorFlags;
		struct
		{
			u_int	m_bDEPRECATED                 : 1,
                    m_bOverrideGamesceneBehaviour : 1,
                    SpareBits                     : 30;
		};
	};

	// (TI - 08.11.2007) - Enumerations to match above bitfield to allow easy use of 
	// Asura_FlagTickbox_Dialog in Tools_Message_Editing, MAKE SURE THESE ARE KEPT IN SYNC!
#ifdef ASURA_TOOLS
	enum ATTRACTOR_MESSAGE_FLAGS
	{
		ATT_MSG_FLAG_REMAIN_ACTIVE      = 1 << 0,
		ATT_MSG_FLAG_OVERRIDE_GAMESCENE = 1 << 1,
	};
#endif
};
//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

#endif //__ASURA_SERVERNODE_GAMESCENEATTRACTOR_H__
#endif // ASURA_USE_AXON
