//////////////////////////////////////////////////////////////////////////////
//
// Asura_ServerNode_LiftNode.h
//
// Copyright (c) 2006 Rebellion. All rights reserved.
//
// Description:
//
//     Node class for lift splines
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef ___ASURA_SERVERNODE_LIFTNODE_H___
#define ___ASURA_SERVERNODE_LIFTNODE_H___ 1

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// asura
#include "Entity/Locations/OrientedNode/Asura_ServerNode_OrientedNode.h"

//////////////////////////////////////////////////////////////////////////////
//
// Asura_ServerNode_LiftNode
//
//////////////////////////////////////////////////////////////////////////////

class Asura_ServerNode_LiftNode : public Asura_ServerNode_OrientedNode
{
    typedef Asura_ServerNode_OrientedNode PARENT;
    friend class Tools_EntityNode_LiftNode;

public:
    Asura_ServerNode_LiftNode(Asura_Guid uGuid);
    virtual ~Asura_ServerNode_LiftNode();

	void SendOnArrivalMessages();
	void SendOnDepartureMessages();

	bool GetStopHere()			  const { return m_bStopHere; }
	bool GetBlockWhenOccupied()   const { return m_bBlockWhenOccupied; }
	//bool GetBlockWhenUnoccupied() const { return m_bBlockWhenUnoccupied; }

    void SetStopHere( bool bStopHere )         { m_bStopHere            = bStopHere; }
    void SetBlockWhenOccupied( bool bBlock )   { m_bBlockWhenOccupied   = bBlock; }
	//void SetBlockWhenUnoccupied( bool bBlock ) { m_bBlockWhenUnoccupied = bBlock; }

	virtual bool HandleMessage( const class Asura_Message* pxMessage );

	virtual bool IsBaseOrDerivativeOf(u_short usClassification) const;
#ifdef ASURA_TOOLS
    virtual bool CanBeIncludedInTemplate() const;
#endif

	static void CreateFromChunkStream( Asura_Guid uGuid, Asura_Chunk_Stream& xStream );
    virtual void WriteToChunkStream( Asura_Chunk_Stream& xStream ) const;
    virtual	void ReadFromChunkStream( Asura_Chunk_Stream& xStream );

    static Asura_Hashed_List< Asura_ServerNode_LiftNode >* GetList() { return &s_xNodeList; }
    static Asura_ServerNode_LiftNode* FindEntity( const Asura_Guid uGuid ) { return s_xNodeList.FindEntity( uGuid ); }

private:
	union
	{
		u_int m_uLiftNodeFlags;
		struct
		{
            u_int m_bStopHere            : 1,
                  m_bBlockWhenOccupied	 : 1,
			      m_bBlockWhenUnoccupied : 1,
                  SpareBits_LiftNode     : 29;
		};
	};

private:
    enum { CURRENT_CHUNK_VERSION = 0 };
    static Asura_Hashed_List< Asura_ServerNode_LiftNode > s_xNodeList;
};

class Asura_Message_Lift_Node_SetFlags
{
public:
	Asura_Message_Lift_Node_SetFlags()
	{
		Reset();
	}

	void Reset() { m_uFlags = 0;}

	void FillInChunkBuffer(char*& pcChunkBuffer) const;
	void WriteToChunkStream(Asura_Chunk_Stream& xStream) const;

	void InitialiseFromChunkBuffer(const char*& pcChunkBuffer);

	bool GetStopHereOn() const { return m_bStopHereOn;}
	bool GetStopHereOff() const { return m_bStopHereOff;}

	u_int* GetFlagsPtr() { return &m_uFlags;}

protected:
	enum { CURRENT_VERSION = 0};
	union
	{
		u_int m_uFlags;
		struct
		{
			u_int m_bStopHereOn			:  1,
			      m_bStopHereOff		:  1,
			      m_uSpareFlags			: 30;
		};
	};
};

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

#endif // #ifndef ___ASURA_SERVERNODE_LIFTNODE_H___
