#ifdef ASURA_USE_AXON
//////////////////////////////////////////////////////////////////////////////
//
// Asura_ServerNode_GS2Spline.h
//
// Copyright (C) 2008 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __ASURA_SERVERNODE_GS2SPLINE_H__
#define __ASURA_SERVERNODE_GS2SPLINE_H__ 1

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// asura
#include "Core/Asura_LinkList.h"
#include "Debugging/Asura_Assert.h"
#include "Entity/Locations/Spline/Asura_ServerNode_Spline.h"

// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////////

template < class T > class Asura_Graph;
class Axon_BehaviourParams;

//////////////////////////////////////////////////////////////////////////////
//
// Asura_ServerNode_GS2Spline
//
//////////////////////////////////////////////////////////////////////////////

class Asura_ServerNode_GS2Spline : public Asura_ServerNode_Spline
{
	typedef Asura_ServerNode_Spline PARENT;

public:
	Asura_ServerNode_GS2Spline( Asura_Guid uGuid );
	virtual ~Asura_ServerNode_GS2Spline();

	virtual void ReadFromChunkStream( Asura_Chunk_Stream& xStream );
	virtual void WriteToChunkStream( Asura_Chunk_Stream& xStream ) const;
	virtual bool HandleMessage( const Asura_Message* pxMessage );
	virtual bool IsBaseOrDerivativeOf( u_short usClassification ) const;

	void EnterSpline( const Asura_Guid uActorGuid );
	void ExitSpline( const Asura_Guid uActorGuid );

	bool ShouldUsePointOrientation() const { return m_bUsePointOrientation; }
	bool ShouldTestForCollisions()   const { return m_bTestForCollisions; }
	bool ShouldIgnoreCollisions()	 const { return m_bIgnoreCollisions; }
	bool ShouldFollowSurface( bool bIgnoreToolsOptimisation = false ) const
	{
		return ( ( bIgnoreToolsOptimisation && m_bFollowSurfaceTools ) || ( !bIgnoreToolsOptimisation && m_bFollowSurface ) );
	}
	float GetAverageLinkSpeed() const;
	float GetPositionFromTime(const float fTimeElapsed, Asura_Vector_3& xOutput, Asura_Quat* pxOutputOrientation = NULL) const;
	float GetTangentFromTime(const float fTimeElapsed, Asura_Vector_3& xOutput) const;
	float GetSpeedFromTime(const float fTimeElapsed) const;
	float GetFinalSpeed() const;
	float GetTotalLinkTime() const { return ( m_bOverrideDuration || m_bDistanceVsTimeGraph ) ? m_fDurationOverride : m_fTotalTimeRequired; }
	float GetProportionFromTime(float fTimeElapsed) const;

	int					  GetBehaviourSet()    const { return m_iBehaviourSet; }
	Axon_BehaviourParams* GetBehaviourParams() const { return m_pxBehaviourParams; }

#ifndef ASURA_NO_DEBUG_OUTPUT
	virtual void RenderDebugInfo();
	static void DoRenderDebugInfo();
#endif

	static void Create( const Asura_Guid uGuid, Asura_Chunk_Stream& xStream );
	static Asura_Hashed_List< Asura_ServerNode_GS2Spline >* GetList() { return &s_xList; }
	static Asura_ServerNode_GS2Spline* FindEntity( const Asura_Guid uGuid ) { return s_xList.FindEntity( uGuid ); }

#ifdef ASURA_TOOLS
	virtual bool CanBeIncludedInTemplate() const;
	virtual void ApplyChangesForTemplateInstance( Tools_EntityTemplateInstance* pxInstance );

	virtual void CreateBasePoints();
	virtual void OnChanged();
	void SetShouldFollowSurface( const bool bFollow ) { m_bFollowSurfaceTools = bFollow; }
	void UpdateFollowSurface();
	void ChangedDistanceTimeGraph();

	Asura_Graph<float>* GetSpeedOverTimeGraph();

	bool GetOverrideDurationFlag() const { return m_bOverrideDuration; }
	void SetOverrideDurationFlag( const bool bOverrideDuration ) 
	{ 
		m_bOverrideDuration = bOverrideDuration; 
		if (m_fDurationOverride == 0.0f)
		{
			m_fDurationOverride = m_fTotalTimeRequired;
		}
		if (m_bOverrideDuration) 
		{
			ChangedDistanceTimeGraph();
		}
	}
	float GetOverrideDurationValue() const { return m_fDurationOverride; }
	void SetOverrideDurationValue( const float fDurationOverride ) { m_fDurationOverride = fDurationOverride; }
	void SetDistanceVsTimeGraph( const bool bDistanceTimeGraph );
	bool GetDistanceVsTimeGraph() { return m_bDistanceVsTimeGraph; }

	void SetShouldTestForCollisions( const bool bTest ) { m_bTestForCollisions = bTest; }
	void SetShouldIgnoreCollisions( const bool bIgnore ) { m_bIgnoreCollisions = bIgnore; }

	float GetTimeFromProportion(float fProportion) const;
	float GetVelocityMultiplier() const {return m_bOverrideDuration ? m_fDurationOverride / m_fTotalTimeRequired : 1.0f; }

	void SetUsePointOrientation( bool bValue ) { m_bUsePointOrientation = bValue; }
	void SetBehaviourSet( int iBehaviourSet );
	virtual bool UpdateBehaviourTree();

#ifdef ASURA_USE_GAMESCENE
	float GetTotalTimeRequired() const { return m_fTotalTimeRequired; }
	void SetTotalTimeRequired( float fTime ) { m_fTotalTimeRequired = fTime; }
#endif
#endif

private:
	// The combined flags u_int and the m_bIntegratedGraph flag itself are mutable, as the flag can be altered during writing to chunk stream.
    union
    {
        mutable u_int m_uFlags;
        struct
        {
            u_int m_bUsePointOrientation			: 1;
			u_int m_bIgnoreCollisions				: 1;
			u_int m_bUNUSED_1						: 1;
			// Speed over time graph is actually proportional distance over time.
			u_int m_bDistanceVsTimeGraph			: 1; 
			// Override the duration of the spline.
            u_int m_bOverrideDuration				: 1;
			u_int m_bTestForCollisions				: 1;
			// Whether follow surface should be used.
			u_int m_bFollowSurface					: 1;
			// Whether follow surface has been set in tools.
            u_int m_bFollowSurfaceTools	            : 1;
			mutable u_int m_bIntegratedGraph		: 1;
            //SpareBits_GamesceneSpline				: 23;
        };
    };
    Asura_Graph< float >* m_pxSpeedOverTimeGraph;
    float m_fTotalTimeRequired;
	float m_fDurationOverride;

#if defined ASURA_TOOLS && defined ASURA_USE_GAMESCENE
public:
#endif
	int	                  m_iBehaviourSet;
	Axon_BehaviourParams* m_pxBehaviourParams;
#if defined ASURA_TOOLS && defined ASURA_USE_GAMESCENE
private:
#endif

	Asura_LinkList< Asura_Guid > m_xOccupierGuids;

private:
	friend class Tools_EntityNode_GS2Spline;

	enum { CURRENT_CHUNK_VERSION = 1 };
	static Asura_Hashed_List< Asura_ServerNode_GS2Spline > s_xList;
};

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

#endif //__ASURA_SERVERNODE_GS2SPLINE_H__
#endif // ASURA_USE_AXON
