#ifdef ASURA_USE_AXON
//////////////////////////////////////////////////////////////////////////////
//
// Asura_ServerNode_GamesceneSpline.h
//
// Created by NJD - 16 October 2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __ASURA_SERVERNODE_GAMESCENESPLINE_H__
#define __ASURA_SERVERNODE_GAMESCENESPLINE_H__ 1

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// asura
#include "Core/Asura_HashedList.h"
#include "Debugging/Asura_Assert.h"
#include "Entity/Locations/Asura_Gamescene_Node_Base.h"
#include "Entity/Locations/Spline/Asura_ServerNode_Spline.h"

template <class T> class Asura_Graph;

//////////////////////////////////////////////////////////////////////////////
//
// support types
//
//////////////////////////////////////////////////////////////////////////////

enum ASURA_GAMESCENE_PAUSE_CONDITION
{
    ASURA_GAMESCENE_PAUSE_NONE                             = 0,
    ASURA_GAMESCENE_PAUSE_WHEN_ALERTED,
    ASURA_GAMESCENE_PAUSE_WHEN_ENEMYSPOTTED,
    ASURA_GAMESCENE_PAUSE_WHEN_WITHIN_RANGE_OF_ACTOR,

    NUM_ASURA_GAMESCENE_PAUSE_CONDITIONS,

    ASURA_GAMESCENE_PAUSE_CONDITION_FIRST_PROJECT_SPECIFIC = 1000,
};

// NY - 13 November 2006 - Asura_DistanceTime_GraphPoint moved to Entity/Locations/Spline/Asura_ServerNode_Spline.h, as needed by other variable speed splines

//////////////////////////////////////////////////////////////////////////////
//
// Asura_ServerNode_GamesceneSpline
//
//////////////////////////////////////////////////////////////////////////////

class Asura_ServerNode_GamesceneSpline : public Asura_ServerNode_Spline, public virtual Asura_Gamescene_Node_Base
{
    typedef Asura_ServerNode_Spline   ENTITY_PARENT;
    typedef Asura_Gamescene_Node_Base GAMESCENE_PARENT;

public:
    Asura_ServerNode_GamesceneSpline(Asura_Guid uGuid);
    virtual ~Asura_ServerNode_GamesceneSpline();

    void  GetAnimTemplateStartPos(Asura_Vector_3& xPosition, Asura_Matrix_3x3& xOrientation) const;
    float GetAverageLinkSpeed() const;
    float GetPositionFromTime(const float fTimeElapsed, Asura_Vector_3& xOutput, Asura_Quat* pxOutputOrientation = NULL) const;
    void  GetPreviousTrackPointPos(Asura_Vector_3& xPosition, const float fTimeOnTrack) const;
    float GetSpeedFromTime(const float fTimeElapsed) const;
    float GetFinalSpeed() const;
    float GetTransitionEntryTime() const;
    void  GetTransitionPoints(Asura_Vector_3& xEntryPoint, Asura_Vector_3 &xControlPoint) const;
    float GetTotalLinkTime() const
    {
        Asura_Assert(m_pxDistanceTimeGraphPoints, "Unhandled null pointer");
		if (m_bDurationOverride||m_bDistanceVsTimeGraph)
		{
			return m_fDurationOverride;
		}

        //TI - rather than just crash here shall we actually do a check?
        return m_pxDistanceTimeGraphPoints[m_uNumDistanceTimeGraphEntries - 1].m_fCumulativeTime;
    }

    u_int      GetPauseCondition()               const { return m_uPauseCondition; }
    Asura_Guid GetPauseRangeActorGuid()          const { return m_uPauseRangeActorGuid; }
    float      GetPauseRangeFromActor()          const { return m_fPauseRangeFromActor; }
    bool       ShouldTravelBackwards()           const { return m_bTravelBackwards; }
    bool       IsConvoyLink()                    const { return m_bIsConvoyLink; }
    bool       IsFleeLink()                      const { return m_bIsFleeLink; }
    bool       ShouldTakeOrientationFromPoints() const { return m_bTakeOrientationFromPoints; }
    bool       ShouldTakeSpeedFromAnimation()    const { return m_bTakeSpeedFromAnimation; }
	bool	   ShouldUsePathFinding()			 const { return m_bUsePathFinding; }

	virtual bool IsBaseOrDerivativeOf(u_short usClassification) const;

#ifdef ASURA_TOOLS
	float GetTotalTimeRequired() const { return m_fTotalTimeRequired; }


    bool         ShouldUseSplineAdjustment() const { return m_bUseSplineAdjustment; }
    Asura_Quat&  GetTrackOrientation(const int iIndex);
    virtual void ChangedLinkPoints();
    virtual void UpdateNeedFollowGround();

    Asura_Graph<float>* GetSpeedOverTimeGraph();

	bool GetOverrideDurationFlag() const { return m_bDurationOverride; }
	void SetOverrideDurationFlag( const bool bOverrideDuration ) 
	{ 
		m_bDurationOverride = bOverrideDuration; 
		if (m_fDurationOverride == 0.0f)
		{
			m_fDurationOverride = m_fTotalTimeRequired;
		}
		if (m_bDurationOverride) 
		{
			ChangedDistanceTimeGraph();
		}
	}
	float GetOverrideDurationValue() const { return m_fDurationOverride; }
	void SetOverrideDurationValue( const float fDurationOverride ) 
	{ 
		m_fDurationOverride = fDurationOverride;  
		if (m_bDurationOverride) 
		{
			ChangedDistanceTimeGraph();
		}
	}
	void SetDistanceVsTimeGraph( const bool  bDistanceTimeGraph );
	bool GetDistanceVsTimeGraph( ) { return m_bDistanceVsTimeGraph; }

	float GetTimeFromProportion(float fProportion) const;

	float GetVelocityMultiplier() const {return m_bDurationOverride ? m_fDurationOverride / m_fTotalTimeRequired : 1.0f; }



    void SetIsFleeLink( const bool bIsFleeLink )                   { m_bIsFleeLink                = bIsFleeLink;          }
    void SetIsConvoyLink( const bool bIsConvoyLink )               { m_bIsConvoyLink              = bIsConvoyLink;        }
    void SetUseSplineAdjustment( const bool bUseSplineAdjustment ) { m_bUseSplineAdjustment       = bUseSplineAdjustment; }
    void SetTravelBackwards( const bool bTravelBackwards )         { m_bTravelBackwards           = bTravelBackwards;     }
    void SetTakeSpeedFromAnimation( const bool bTakeSpeed )        { m_bTakeSpeedFromAnimation    = bTakeSpeed;           }
	void SetShouldUsePathFinding( const bool bUsePathFinding )	   { m_bUsePathFinding			  = bUsePathFinding;	  }
	void SetTakeOrientationFromPoints( const bool bTakeOri )       { m_bTakeOrientationFromPoints = bTakeOri;             }

    void SetPauseCondition(u_int uPauseCondition)					{ m_uPauseCondition      = uPauseCondition;      }
    void SetPauseRangeActorGuid(Asura_Guid uPauseRangeActorGuid)	{ m_uPauseRangeActorGuid = uPauseRangeActorGuid; }
    Asura_Guid* GetPauseRangeActorGuidPointer()						{ return &m_uPauseRangeActorGuid; }
    void SetPauseRangeFromActor(float fPauseRangeFromActor)			{ m_fPauseRangeFromActor = fPauseRangeFromActor; }
    float* GetPauseRangeFromActorPointer()							{ return &m_fPauseRangeFromActor; }
    void ChangedDistanceTimeGraph();
    void EnableTakeOrientationFromPoints(bool bEnable)              { m_bTakeOrientationFromPoints = bEnable; }

    virtual bool CanBeIncludedInTemplate() const;
    virtual void ApplyChangesForTemplateInstance(class Tools_EntityTemplateInstance* pxInstance);
#endif //ASURA_TOOLS

	static void CreateFromChunkStream(Asura_Guid uGuid, Asura_Chunk_Stream& xStream);
    virtual void ReadFromChunkStream( Asura_Chunk_Stream& xStream );
    virtual void WriteToChunkStream(Asura_Chunk_Stream& xStream) const;

    static Asura_Hashed_List< Asura_ServerNode_GamesceneSpline >* GetList() { return &s_xGamesceneSplineList; }
    static Asura_ServerNode_GamesceneSpline* FindEntity( const Asura_Guid uGuid ) { return s_xGamesceneSplineList.FindEntity( uGuid ); }

	float GetProportionFromTime(float fTimeElapsed) const;

protected:
    void  GetTransitionTimes(float& fEntryTime, float& fControlPointTime) const;

protected:
    union
    {
        u_int m_uGamesceneSplineFlags;
        struct
        {
            u_int m_bIsConvoyLink              : 1,
                  m_bIsFleeLink                : 1,
                  m_bTravelBackwards           : 1,
                  m_bTakeOrientationFromPoints : 1,
                  m_bUseSplineAdjustment       : 1,
                  m_bTakeSpeedFromAnimation    : 1,
				  m_bUsePathFinding			   : 1,
										
				 // Speed over time graph is actually proportional distance over time
				  m_bDistanceVsTimeGraph	   : 1, 
				 // Override the duration of the spline
                  m_bDurationOverride          : 1,
                  SpareBits_GamesceneSpline    : 23;
        };
    };

    Asura_Graph<float>* m_pxSpeedOverTimeGraph; // Graph of speed over time.

    float m_fTotalTimeRequired;
	float m_fDurationOverride;
    u_int m_uNumDistanceTimeGraphEntries;
    Asura_DistanceTime_GraphPoint* m_pxDistanceTimeGraphPoints;

    //Variables for pause conditions
    u_int      m_uPauseCondition;
    Asura_Guid m_uPauseRangeActorGuid;
    float      m_fPauseRangeFromActor;

private:
    enum { CURRENT_CHUNK_VERSION = 2 };
    static Asura_Hashed_List< Asura_ServerNode_GamesceneSpline > s_xGamesceneSplineList;
};

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

#endif //__ASURA_SERVERNODE_GAMESCENESPLINE_H__
#endif // ASURA_USE_AXON
