//////////////////////////////////////////////////////////////////////////////
//
// Asura_ServerNode_LiftSpline.h
//
// Copyright (c) 2006 Rebellion. All rights reserved.
//
// Description:
//
//     spline class for lift entities to follow
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef ___ASURA_SERVERNODE_LIFTSPLINE_H___
#define ___ASURA_SERVERNODE_LIFTSPLINE_H___ 1

// I N C L U D E S ///////////////////////////////////////////////////////////

// asura
#include "Core/Asura_HashedList.h"
#include "Debugging/Asura_Assert.h"
#include "Entity/Locations/Spline/Asura_ServerNode_Spline.h"

template <class T> class Asura_Graph;

//////////////////////////////////////////////////////////////////////////////
//
// Asura_ServerNode_LiftSpline
//
//////////////////////////////////////////////////////////////////////////////

class Asura_ServerNode_LiftSpline : public Asura_ServerNode_Spline
{
    typedef Asura_ServerNode_Spline  PARENT;

public:
	Asura_ServerNode_LiftSpline(Asura_Guid uGuid);
	virtual ~Asura_ServerNode_LiftSpline();

	float GetAverageLinkSpeed() const;
	float GetPositionFromTime(const float fTimeElapsed, Asura_Vector_3& xOutput, Asura_Quat* pxOutputOrientation = NULL) const;
	float GetSpeedFromTime(const float fTimeElapsed) const;
	float GetTotalLinkTime() const { return ( m_bOverrideDuration || m_bDistanceVsTimeGraph ) ? m_fDurationOverride : m_fTotalTimeRequired; }

	const Asura_Quat& GetLinkOrientation(const int iIndex) const;

	bool TakeOrientationFromPoints() const { return m_bTakeOrientationFromPoints; }

	virtual bool IsBaseOrDerivativeOf(u_short usClassification) const;

#ifdef ASURA_TOOLS
	virtual void OnChanged();

	Asura_Graph<float>* GetSpeedOverTimeGraph();

	Asura_Quat& GetLinkOrientation(const int iIndex);

	bool GetOverrideDurationFlag() const { return m_bOverrideDuration; }
	void SetOverrideDurationFlag( const bool bOverrideDuration ) { m_bOverrideDuration = bOverrideDuration; }
	float GetOverrideDurationValue() const { return m_fDurationOverride; }
	void SetOverrideDurationValue( const float fDurationOverride ) { m_fDurationOverride = fDurationOverride; }
	void SetDistanceVsTimeGraph( const bool  bDistanceTimeGraph );
	bool GetDistanceVsTimeGraph( ) { return m_bDistanceVsTimeGraph; }

	float GetTimeFromProportion(float fProportion) const;

	float GetVelocityMultiplier() const {return m_bOverrideDuration ? m_fDurationOverride / m_fTotalTimeRequired : 1.0f; }

	void ChangedDistanceTimeGraph();
	void EnableTakeOrientationFromPoints(bool bEnable)   { m_bTakeOrientationFromPoints = bEnable; }

	virtual bool CanBeIncludedInTemplate() const;
#endif //ASURA_TOOLS

	static void CreateFromChunkStream(Asura_Guid uGuid, Asura_Chunk_Stream& xStream);
    virtual void ReadFromChunkStream( Asura_Chunk_Stream& xStream );
	virtual void WriteToChunkStream (Asura_Chunk_Stream& xStream) const;

    static Asura_Hashed_List< Asura_ServerNode_LiftSpline >* GetList() { return &s_xLiftSplineList; }
    static Asura_ServerNode_LiftSpline* FindEntity( const Asura_Guid uGuid ) { return s_xLiftSplineList.FindEntity( uGuid ); }

protected:
	float GetProportionFromTime(float fTimeElapsed) const;

protected:
	union
	{
        // (NY - 16.04.2007) the combined flags u_int and the m_bIntegratedGraph flag itself are mutable, as the flag can be altered during write
		mutable u_int m_uLiftSplineFlags;
		struct
		{
			u_int m_bTakeOrientationFromPoints	: 1;
			u_int m_bDeprecated1				: 1;//used to be tolerance toggle
			u_int m_bOverrideDuration			: 1;
			mutable u_int m_bIntegratedGraph	: 1;
			u_int m_bDistanceVsTimeGraph		: 1;
			u_int SpareBits_LiftSpline			: 27;
		};
	};

	Asura_Graph<float>* m_pxSpeedOverTimeGraph; // Graph of speed over time.

	float m_fTotalTimeRequired;
	float m_fDurationOverride;

private:
    enum { CURRENT_CHUNK_VERSION = 2 };
    static Asura_Hashed_List< Asura_ServerNode_LiftSpline > s_xLiftSplineList;
};

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

#endif // #ifndef ___ASURA_SERVERNODE_LIFTSPLINE_H___
