/**
*
* \file Asura_ClientEntity_PFX_Effect.h
*
* Created by Kim Randell - 22/03/2004 11:09:22
*
* Copyright (C) 2002 Rebellion, All Rights Reserved.
*
*
*
**/
#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif

#ifndef __ASURA_CLIENTENTITY_PFX_EFFECT_H__
#define __ASURA_CLIENTENTITY_PFX_EFFECT_H__

#include "Entity/Asura_Entity.h"
#include "Entity/ParticleGenerator/Asura_Entity_PFX_Effect_Data.h"

class Asura_Chunk_Stream;
class Asura_PFX_Entity_Effect;
template <typename T> class Asura_Spline;

class Asura_ClientEntity_PFX_Effect : public Asura_Entity_Client
{
	typedef Asura_Entity_Client PARENT;

public:
#ifdef ASURA_TOOLS
	Asura_ClientEntity_PFX_Effect(const Asura_Vector_3& xPos, const Asura_Matrix_3x3& xDir, Asura_Hash_ID uTemplateHash);
#endif
    Asura_ClientEntity_PFX_Effect();
    virtual ~Asura_ClientEntity_PFX_Effect( void );

	virtual void ReadFromChunkStream(Asura_Chunk_Stream& xStream);

    // virtual functions
	virtual bool HandleMessage(const Asura_Message* pxMessage);
    
	virtual void Update();
	
	virtual void PostCreateInitialise();

	//Updates just the local client version of the generator. So cutscenes can keep their client
	//generators in the right place
	void SetPositionAndOrientation(const Asura_Vector_3& xPosition,const Asura_Matrix_3x3& xOrient,bool bTeleport = false);

	//Activates/deactivates just the local client version of the generator
	void Activate(bool bActive);

	//Allows module linking tests to be disabled for generators controlled by cutscenes
	void SetIgnoreModuleLinking(bool bIgnore);

	// Sets the effect to be part of the skybox
	void SetSkyboxEffect(bool bSkybox);

	// Set the effect to be a water distorter (creates ripples in the SM3.0 water)
	void SetWaterDistorterEffect(bool bWaterDistorter);

	// Override current pos and ori functions, as the PosAndOriInterpolator is never used for these entities.
	virtual const Asura_Vector_3& GetCurrentPosition() const { return m_xCurrentPosition; }
	virtual const Asura_Matrix_3x3& GetCurrentOrientation() const {	return m_xCurrentOrientation; }

#ifdef ASURA_TOOLS
	virtual void ApplyChangesForTemplateInstance(class Tools_EntityTemplateInstance* pxInstance);

	//Flags effect as selected for determining whether it should be included in the particle count
	void SetSelectedInTools(bool bSet);

	//Enables/disables effect specific RNG for cutscene rendering ,and resets the seed. RAR 22/05/2006
	void SetUseEffectSpecificRNG(bool bUse);

	bool ShouldUpdate() const { return IsRandomRetrigger() && IsEntityActive(); }

	bool StartsActive() const {return m_bStartsActive;}
#endif

	Asura_Handle GetEffectID() const { return m_iEffectID; }

	void StopEffect(bool bImmediate = false);

	bool IsRandomRetrigger() const { return m_xPFXEffectData.m_fMaxRetriggerTime != 0.0f;}
	float	GetStartTime(void) const	{ return(m_fStartTime); }

	static void CreateFromChunkStream(Asura_Guid uGuid, Asura_Chunk_Stream& xStream);
	static Asura_ClientEntity_PFX_Effect* FindEntity( Asura_Guid uGUID ) { return s_xList.FindEntity( uGUID ); }
	static const Asura_Hashed_List<Asura_ClientEntity_PFX_Effect>* GetList( void ) { return &s_xList; }

protected:
	// virtual functions
	virtual u_int CreateEffect(const Asura_Vector_3& xPosition, const Asura_Matrix_3x3& xDirection, Asura_Hash_ID uTemplate, const Asura_PFX_Volume* pxVolumeOverride = 0);
    
private:

	void StartEffect();
	void RestartEffect() { StopEffect(true); m_fStartTime = 0.f; StartEffect(); }

	Asura_PFX_Entity_Effect* GetCurrentEffect() const;

#ifdef ASURA_TOOLS
	//update an active effect with any changed properties
	void UpdateEffectProperties();
	void UpdateNumberOfSources(int iNumberOfSources);
	void UpdateMaximumParticles(int iSourceNumber, int iMaximumParticles);
	void RenderVisibility();

	//Gets the hash id of the template being used by the effect (taking into account
	//platform overrides). Returns ASURA_HASH_ID_UNSET if the Asura_PFX_Entity_Effect doesn't exist
	Asura_Hash_ID GetCurrentActiveTemplateHash() const;

	bool IsEffectRunning() const;
#endif // #ifdef ASURA_TOOLS

	// D A T A 
	Asura_Handle			m_iEffectID;
	Asura_Entity_PFX_Effect_Data m_xPFXEffectData;

	// Effect start time, sent from server (0 if activate message used)
	float m_fStartTime;

	//For random retriggering.. time until effect should be started again.
	float m_fTriggerTimer;

#ifdef ASURA_TOOLS
	//Effect specific random generator for use while recording cutscenes. RAR 22/05/2006
	Asura_RandomGenerator	m_xEffectSpecificRNG;
	bool 					m_bUseEffectSpecificRNG;

	bool					m_bSelectedInTools;
	bool					m_bStartsActive;
#endif
	Asura_Vector_3			m_xCurrentPosition;
	Asura_Matrix_3x3		m_xCurrentOrientation;

	friend class Tools_Entity_PFX_Effect;

	// S T A T I C   D A T A
    static Asura_Hashed_List<Asura_ClientEntity_PFX_Effect> s_xList;
        
};

#endif//__ASURA_CLIENTENTITY_PFX_EFFECT_H__
