/**
*
* \file Asura_ClientEntity_AdvancedLight.h
*
* Created by Richard May - 07/01/2003 10:52:47
*
* Copyright (C) 2002 Rebellion, All Rights Reserved.
*
* \brief
*
**/


#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif

#ifndef __ASURA_CLIENTENTITY_ADVANCEDLIGHT_H__
#define __ASURA_CLIENTENTITY_ADVANCEDLIGHT_H__

#ifndef ASURA_TARGET_SM3

/************/
/* Includes.*/
/************/

#include "Entity/PhysicalObject/Asura_ClientEntity_DestructibleLight.h"

// M A C R O S ///////////////////////////////////////////////////////////////

// (MP - 12.06.2003) From my tests, advanced lights seem to generally span
// more than 4 modules, making this optimisation fairly redundant.
#define ASURA_CLIENTENTITY_ADVANCEDLIGHT_DO_VIS_CHECKS (0)

/************/
/* Classes. */
/************/

class Asura_Message;

/*************************************************************
 *
 * Class Name    : Asura_ClientEntity_AdvancedLight
 *
 * Author        : Richard May
 *
 * Description   : Client side version of the advanced light 
 *				   - a destructible light that allows flickering
 *					 strobing, fading and radii change, etc.
 *
 *************************************************************/
class Asura_ClientEntity_AdvancedLight : public virtual Asura_ClientEntity_DestructibleLight
{
	friend class Tools_Entity_AdvancedLight;

public:
	/* Constructors/destructor. */
	Asura_ClientEntity_AdvancedLight	();
	virtual ~Asura_ClientEntity_AdvancedLight	();

public:
	/* Public interface functions. */
    virtual bool HandleMessage(const Asura_Message* pxMessage);
	virtual void Update();

	//client side manipulation for cutscenes
	void SetCutsceneControlled(bool bSet){m_uCutsceneControlled = bSet;}
	void ClientSideActivate(bool bActivate);
	void SetPositionAndOrientation(const Asura_Vector_3& xPos, const Asura_Matrix_3x3& xMat);

	//projectors need to get the colour to inherit the strobe/flicker/colour changy stuff
	bool GetLightColour(float *afColour);

	static void Create(const Asura_Message* pxMessage);

protected:

	//sets the active/hidden state of the photon and corona
	void SetPhotonAndCoronaActive(bool bActive);

#if ASURA_CLIENTENTITY_ADVANCEDLIGHT_DO_VIS_CHECKS
    void RefreshIntersectingModuleList( void );
    static bool IntersectingModuleListCallback( u_short uModuleIdx );
#endif

protected:
	/* Data. */
	enum FADE_PHASE
	{
		FADE_PHASE_NONE = -1,

		FADE_PHASE_UP,
		FADE_PHASE_DOWN,
		FADE_PHASE_ATEND,
		FADE_PHASE_ATSTART,
	};
	
	// strobe and flicker
	float m_fStrobeTimer;
	float m_fCurrentStrobeDuration;

	float m_fMinTimeOff;
	float m_fMaxTimeOff;
	float m_fMinTimeOn;
	float m_fMaxTimeOn;

	// colour fade 
	float m_fFadeTimer;
	float m_fCurrentFadeDuration;
	FADE_PHASE m_eColourFadePhase;  

	float m_fEndColourR;
	float m_fEndColourG;
	float m_fEndColourB;
	
	float m_fStartColourR;
	float m_fStartColourG;
	float m_fStartColourB;
	
	float m_fMaxFadeDownDuration;
	float m_fMinFadeDownDuration;
	float m_fMaxFadeUpDuration;
	float m_fMinFadeUpDuration;
	
	float m_fMinStartColourTime;
	float m_fMaxStartColourTime;
	float m_fMinEndColourTime;
	float m_fMaxEndColourTime;
	

	// radius change
	float m_fRadiusTimer;
	float m_fCurrentRadiusDuration;
	FADE_PHASE m_eRadiusPhase;  

	float m_fStartRadius;
	float m_fEndRadius;
				
	float m_fMaxRadiusDownDuration;
	float m_fMinRadiusDownDuration;
	float m_fMaxRadiusUpDuration;
	float m_fMinRadiusUpDuration;
	
	float m_fMinStartRadiusTime;
	float m_fMaxStartRadiusTime;
	float m_fMinEndRadiusTime;
	float m_fMaxEndRadiusTime;
	

	// attached entity stuff
	Asura_Guid m_uMasterEntity;
	Asura_Vector_3 m_xOffsetFromMaster;
	Asura_Hash_ID m_uMasterMarker;
	
	u_int m_uLoopFade        : 1,
	      m_uLoopRadius      : 1,
	      m_uUseMasterOrient : 1,
#if ASURA_CLIENTENTITY_ADVANCEDLIGHT_DO_VIS_CHECKS
          m_uCanDoVisChecks  : 1,
          m_uVisible         : 1,
#endif
	      m_uActive          : 1,
	      m_uCutsceneControlled : 1;

#if ASURA_CLIENTENTITY_ADVANCEDLIGHT_DO_VIS_CHECKS
    enum { MAX_INTERSECTING_MODULES = 4 };                   // Should be a multiple of 2.
    u_int m_uFramesSinceVisCheck;
    short m_asIntersectingModules[MAX_INTERSECTING_MODULES]; // -1 signifies end of list. Note that we're cheekily assuming < 32768 modules here.
#endif

	////////////////////
// Static members //
////////////////////
public:
	static Asura_ClientEntity_AdvancedLight* FindEntity(Asura_Guid uGuid)
	{
		return s_xList.FindEntity(uGuid);
	}
		

private:
	static Asura_Hashed_List<Asura_ClientEntity_AdvancedLight> s_xList;

};
/* end of class Asura_ClientEntity_AdvancedLight */

#endif // #ifndef ASURA_TARGET_SM3

/****************/
/* End of file. */
/****************/
#endif//__ASURA_CLIENTENTITY_ADVANCEDLIGHT_H__
