//////////////////////////////////////////////////////////////////////////////
//
// Asura_ClientEntity_Lift.cpp
//
// Copyright (c) 2006 Rebellion. All rights reserved.
//
// Description:
//
//     Lift Entity - External control physics object moving along a spline
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header.
#include "Entity/PhysicalObject/Asura_ClientEntity_Lift.h"

// Engine independent.

#include "Entity/Asura_Entity_Classification.h"
#include "Network/Asura_Message.h"
#include "Network/Asura_Network.h"
#include "Sound/Asura_Sound_Event.h"
#include "Sound/Asura_Sound_Event_Storage.h"
#include "Sound/Asura_Sound_Event_System.h"

// C L A S S   I M P L E M E N T A T I O N S /////////////////////////////////

//
// Asura_ClientEntity_Lift
//

Asura_ClientEntity_Lift::Asura_ClientEntity_Lift( void ) 
: m_iTravellingSoundHandle(ASURA_INVALID_HANDLE)
{
  Classification = AsuraEntityClass_Lift;
}

Asura_ClientEntity_Lift::~Asura_ClientEntity_Lift()
{
	if( m_iTravellingSoundHandle != ASURA_INVALID_HANDLE )
	{
		Asura_Sound_Event_System::Stop(m_iTravellingSoundHandle);
	}
}

void Asura_ClientEntity_Lift::Update( void )
{
    // Suppress parent's handling due to inheritance hierarchies.
}

void Asura_ClientEntity_Lift::Render()
{
	// HE - 09/07/2007 - update position of sound. done in Render() because 1) Update
	// isn't currently being called, 2) it's fairly trivial so we'll save the overhead
	// of making it updateable
	if(m_iTravellingSoundHandle != ASURA_INVALID_HANDLE)
	{
		Asura_Sound_Event_System::UpdatePosition(m_iTravellingSoundHandle, GetCurrentPosition());
	}
	PARENT::Render();
}


bool Asura_ClientEntity_Lift::HandleMessage(const Asura_Message* pxMessage)
{
	Asura_Assert(pxMessage, "No message");
	switch (pxMessage->GetMsgID())
	{
		case ASURA_MSG_CLIENTUPDATE:
		{
			const Asura_Message_PhysicalObject_ClientUpdate* pxData = 
				static_cast<const Asura_Message_PhysicalObject_ClientUpdate*>(pxMessage->GetData());
			ProcessClientUpdate(pxData, pxMessage->GetSentTime());
			return true;
		}
		
		case ASURA_MSG_PLAYSOUNDEVENT:
		{
			const Asura_MsgData_PlaySoundEvent* pxData = 
				static_cast<const Asura_MsgData_PlaySoundEvent*>(pxMessage->GetData());

			//stop any previous sound - if there is one, it'll be the looping 'travelling' sound
			if( m_iTravellingSoundHandle != ASURA_INVALID_HANDLE )
			{
				Asura_Sound_Event_System::Stop( m_iTravellingSoundHandle );
				m_iTravellingSoundHandle = ASURA_INVALID_HANDLE;
			}

			const Asura_Sound_Event* pxEvent = Asura_Sound_Event_Storage::FindEvent( pxData->m_uSoundEventHash );
			if( pxEvent && pxEvent->IsEndlessPlayback() )
			{
				Asura_Sound_Event_System::PlayEvent( pxEvent, &m_iTravellingSoundHandle, &GetCurrentPosition(), 1.0f , GetGuid() );
			}
			else
			{
				if( pxEvent ) Asura_Sound_Event_System::PlayEvent( pxEvent, NULL, &GetCurrentPosition(), 1.0f, GetGuid() );
			}
			return true;
		}
		default:
		{
			// NOTE: Must not pass to parent.
			return false;
		}
	}
}
