/**
*
* \file Asura_ServerEntity_AdvancedLight.cpp
*
* Created by Richard May - 07/01/2003 10:08:20
*
* Copyright (C) 2002 Rebellion, All Rights Reserved.
*
* \brief
*
**/



/************/
/* Includes.*/
/************/

// main asura header
#include "Core/Asura.h"

#ifndef ASURA_TARGET_SM3

// local header
#include "Entity/PhysicalObject/Asura_ServerEntity_AdvancedLight.h"

// asura
#include "ChunkSystem/Asura_Chunk_Macros.h"
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "Entity/Asura_Entity_Classification.h"
#include "Network/Asura_Network.h"
#include "Network/Asura_Message.h"
#if defined(ASURA_USE_STREAMING) && !defined(ASURA_TOOLS)
#include "Streaming/Asura_Streaming_System.h"
#endif	// defined(ASURA_USE_STREAMING) && !defined(ASURA_TOOLS)

#ifdef ASURA_TOOLS
#include "EntityTemplate/Tools_EntityTemplate_System.h"
#endif

/*********/
/* Data. */
/*********/

Asura_Hashed_List<Asura_ServerEntity_AdvancedLight> Asura_ServerEntity_AdvancedLight::s_xList;

/**************/
/* Functions. */
/**************/

/* Constructor / Destructor */

Asura_ServerEntity_AdvancedLight::Asura_ServerEntity_AdvancedLight()
: Asura_Entity_Server(ASURA_GUID_UNREGISTERED) // ignored
, Asura_ServerEntity_DestructibleLight()
{
	// reset the flags
	m_uAdvLightFlags = 0;

	// flicker & strobe
	m_fMinTimeOff = -1.0f;
	m_fMaxTimeOff = -1.0f;
	m_fMinTimeOn = -1.0f;
	m_fMaxTimeOn = -1.0f;

	// colour fade
	m_fEndColourR = 0.0f;
	m_fEndColourG = 0.0f;
	m_fEndColourB = 0.0f;
	
	m_fStartColourR = 255.0f;
	m_fStartColourG = 255.0f;
	m_fStartColourB = 255.0f;
	
	m_fMaxFadeDownDuration = 1.0f;
	m_fMinFadeDownDuration = 1.0f;
	m_fMaxFadeUpDuration = 1.0f;
	m_fMinFadeUpDuration = 1.0f;
	
	m_fMinStartColourTime = -1.0f;
	m_fMaxStartColourTime = -1.0f;
	m_fMinEndColourTime = -1.0f;
	m_fMaxEndColourTime = -1.0f;

	m_uLoopFade = true;

	// radius change
	m_fStartRadius = 10.0f;
	m_fEndRadius = 10.0f;
			
	m_fMaxRadiusDownDuration = 1.0f;
	m_fMinRadiusDownDuration = 1.0f;
	m_fMaxRadiusUpDuration = 1.0f;
	m_fMinRadiusUpDuration = 1.0f;
	
	m_fMinStartRadiusTime = -1.0f;
	m_fMaxStartRadiusTime = -1.0f;
	m_fMinEndRadiusTime = -1.0f;
	m_fMaxEndRadiusTime = -1.0f;
	
	m_uLoopRadius = true;

	Classification = AsuraEntityClass_AdvancedLight;

	m_uMasterEntity = ASURA_GUID_UNREGISTERED;
	m_xOffsetFromMaster = Asura_Maths::ZeroVector3;
	m_uMasterMarker = ASURA_HASH_ID_UNSET;
	m_uUseMasterOrient  = true;
	m_uActive = true;

	s_xList.AddToList(this);
}

Asura_ServerEntity_AdvancedLight::~Asura_ServerEntity_AdvancedLight()
{
	s_xList.RemoveFromList(this);
}


/*** Chunk Buffer code ***/

void Asura_ServerEntity_AdvancedLight::WriteToChunkStream (Asura_Chunk_Stream& xStream) const
{
	Asura_WriteToStream_Bitfield_Wrapper<u_int>	xFlagsBitfield (m_uAdvLightFlags);

	/*
	This entity type can be used in template files , so any changes to WriteToChunkStream may
	require ApplyChangesForTemplateInstance() to be updated as well.
	Please see the comment at the top of Tools/EntityTemplate/Tools_EntityTemplate_System.h before 
	making any changes.
	RAR 29/04/2004
	*/
	const int	iVersion	= ChunkData::CURRENT_VERSION;


	// Create the bitfield...

	xFlagsBitfield.Add	(m_uLoopFade,			1);
	xFlagsBitfield.Add	(m_uLoopRadius,			1);
	xFlagsBitfield.Add	(m_uUseMasterOrient,	1);
	xFlagsBitfield.Add	(m_uActive,				1);


	// Write the data...

	xStream << iVersion;

	// flicker & strobe
	xStream << m_fMinTimeOff;
	xStream << m_fMaxTimeOff;
	xStream << m_fMinTimeOn;
	xStream << m_fMaxTimeOn;

	// colour fade 
	xStream << m_fEndColourR;
	xStream << m_fEndColourG;
	xStream << m_fEndColourB;
		
	xStream << m_fStartColourR;
	xStream << m_fStartColourG;
	xStream << m_fStartColourB;
		
	xStream << m_fMaxFadeDownDuration;
	xStream << m_fMinFadeDownDuration;
	xStream << m_fMaxFadeUpDuration;
	xStream << m_fMinFadeUpDuration;
		
	xStream << m_fMinStartColourTime;
	xStream << m_fMaxStartColourTime;
	xStream << m_fMinEndColourTime;
	xStream << m_fMaxEndColourTime;

	// radius change 
	xStream << m_fStartRadius;
	xStream << m_fEndRadius;
				
	xStream << m_fMaxRadiusDownDuration;
	xStream << m_fMinRadiusDownDuration;
	xStream << m_fMaxRadiusUpDuration;
	xStream << m_fMinRadiusUpDuration;
		
	xStream << m_fMinStartRadiusTime;
	xStream << m_fMaxStartRadiusTime;
	xStream << m_fMinEndRadiusTime;
	xStream << m_fMaxEndRadiusTime;

	xStream << xFlagsBitfield;

	// attached entity stuff
	xStream << m_uMasterEntity;
	xStream << m_xOffsetFromMaster;
	xStream << m_uMasterMarker;

	PARENT::WriteToChunkStream	(xStream);
	/*
	This entity type can be used in template files , so any changes to WriteToChunkStream may
	require ApplyChangesForTemplateInstance() to be updated as well.
	Please see the comment at the top of Tools/EntityTemplate/Tools_EntityTemplate_System.h before 
	making any changes.
	RAR 29/04/2004
	*/
}


void Asura_ServerEntity_AdvancedLight::ReadFromChunkStream(Asura_Chunk_Stream& xStream)
{
	int	iVersion;

	xStream >> iVersion;

	// flicker & strobe
	xStream >> m_fMinTimeOff;
	xStream >> m_fMaxTimeOff;
	xStream >> m_fMinTimeOn;
	xStream >> m_fMaxTimeOn;

	// colour fade 
	xStream >> m_fEndColourR;
	xStream >> m_fEndColourG;
	xStream >> m_fEndColourB;

	xStream >> m_fStartColourR;
	xStream >> m_fStartColourG;
	xStream >> m_fStartColourB;

	xStream >> m_fMaxFadeDownDuration;
	xStream >> m_fMinFadeDownDuration;
	xStream >> m_fMaxFadeUpDuration;
	xStream >> m_fMinFadeUpDuration;

	xStream >> m_fMinStartColourTime;
	xStream >> m_fMaxStartColourTime;
	xStream >> m_fMinEndColourTime;
	xStream >> m_fMaxEndColourTime;

	// radius change 
	xStream >> m_fStartRadius;
	xStream >> m_fEndRadius;

	xStream >> m_fMaxRadiusDownDuration;
	xStream >> m_fMinRadiusDownDuration;
	xStream >> m_fMaxRadiusUpDuration;
	xStream >> m_fMinRadiusUpDuration;

	xStream >> m_fMinStartRadiusTime;
	xStream >> m_fMaxStartRadiusTime;
	xStream >> m_fMinEndRadiusTime;
	xStream >> m_fMaxEndRadiusTime;

	if(iVersion >= 3)
	{
		ChunkData										xChunk;
		Asura_ReadFromStream_Bitfield_Wrapper<u_int>	xBitfield(xChunk.m_uFlags);

		/// Richard May - 05/03/2003 17:34:51 - these compensations are necessary cos there 
		/// was a mess up with the versioning between 2 & 3
		if( (m_fEndColourR <= 1.0f) && (m_fEndColourG <= 1.0f) && (m_fEndColourB <= 1.0f) )
		{
			m_fEndColourR *= 255.0f;
			m_fEndColourG *= 255.0f;
			m_fEndColourB *= 255.0f;
		}

		if( (m_fStartColourR <= 1.0f) && (m_fStartColourG <= 1.0f) && (m_fStartColourB <= 1.0f) )
		{
			m_fStartColourR *= 255.0f;
			m_fStartColourG *= 255.0f;
			m_fStartColourB *= 255.0f;
		}

		xStream >> xBitfield;

		m_uLoopFade = xChunk.m_uLoopFade;
		m_uLoopRadius = xChunk.m_uLoopRadius;
		m_uUseMasterOrient = xChunk.m_uUseMasterOrient;
		m_uActive = xChunk.m_uActive;

		// attached entity stuff
#if defined(ASURA_USE_STREAMING) && !defined(ASURA_TOOLS)
		{
			Asura_Guid	uMasterEntity;

			xStream >> uMasterEntity;
			m_uMasterEntity = Asura_Streaming_System::MakeCurrentMasterZoneGuid(uMasterEntity);
		}
#else
		xStream >> m_uMasterEntity;
#endif	// defined(ASURA_USE_STREAMING) && !defined(ASURA_TOOLS)
		xStream >> m_xOffsetFromMaster;

		if(iVersion >= 4)
		{
			xStream >> m_uMasterMarker;
		}
	}
#if defined(ASURA_TOOLS)
	else if( iVersion == 2 )
	{
		ChunkData_v2									xChunk;
		Asura_ReadFromStream_Bitfield_Wrapper<u_int>	xBitfield(xChunk.m_uFlags);

		xStream >> xBitfield;

		m_uLoopFade = xChunk.m_ucLoopFade;
		m_uLoopRadius = xChunk.m_ucLoopRadius;
		m_uUseMasterOrient = (xChunk.m_ucUseMasterOrient != 0) ? true : false;
		m_uActive = true;

		// attached entity stuff
#if defined(ASURA_USE_STREAMING) && !defined(ASURA_TOOLS)
		{
			Asura_Guid	uMasterEntity;

			xStream >> uMasterEntity;
			m_uMasterEntity = Asura_Streaming_System::MakeCurrentMasterZoneGuid(uMasterEntity);
		}
#else
		xStream >> m_uMasterEntity;
#endif	// defined(ASURA_USE_STREAMING) && !defined(ASURA_TOOLS)
		xStream >> m_xOffsetFromMaster;
	}
#endif	// defined(ASURA_TOOLS)
	else
	{
		Asura_Assert(false, "Unsupported chunk version.");
	}

	PARENT::ReadFromChunkStream(xStream);
}


/*************************************************************
 *
 * Name          : Asura_ServerEntity_AdvancedLight::FillInChunkBuffer
 *
 * Author        : Richard May
 *
 *************************************************************/
void Asura_ServerEntity_AdvancedLight::FillInChunkBuffer ( char*& pcChunkBuffer )
{
	Asura_Chunk_ChunkBuffer_Stream	xStream(pcChunkBuffer);

	Asura_ServerEntity_AdvancedLight::WriteToChunkStream(xStream);
}
/* end of function FillInChunkBuffer */


/*************************************************************
 *
 * Name          : Asura_ServerEntity_AdvancedLight::InitialiseFromChunkBuffer
 *
 * Author        : Richard May
 *
 *************************************************************/
void Asura_ServerEntity_AdvancedLight::InitialiseFromChunkBuffer ( const char*& pcChunkBuffer )
{
	Asura_Chunk_ChunkBuffer_Stream	xStream(pcChunkBuffer);

	Asura_ServerEntity_AdvancedLight::ReadFromChunkStream(xStream);
}
/* end of function InitialiseFromChunkBuffer */


/*** Message Handling ***/


/*************************************************************
 *
 * Name          : Asura_ServerEntity_AdvancedLight::HandleMessage
 *
 * Author        : Richard May
 *
 * Description   : message handler for advanced lights - currently
 *				   only handles creation request messages
 *
 * Parameters    : pxMessage - pointer to the message to handle
 *
 * Return value  : bool - whether or not the message was handled
 *						  successfully
 *
 *************************************************************/
bool Asura_ServerEntity_AdvancedLight::HandleMessage ( const Asura_Message* pxMessage )
{
	// paramter check
	if(!pxMessage) return false;

	switch(pxMessage->GetMsgID())
	{
	
	case ASURA_MSG_CLIENT_REQUESTCREATEMSG : MakeCreateMessage(pxMessage->GetFromGuid()); return true;

	case ASURA_MSG_ACTIVATE : m_uActive = true; return true;

	case ASURA_MSG_DEACTIVATE : m_uActive = false; return true;

	default : break;

	}

	// unhandled message, return false
	return false;
}
/* end of function HandleMessage */


/*************************************************************
 *
 * Name          : Asura_ServerEntity_AdvancedLight::Update
 *
 * Author        : Richard May
 *
 * Description   : Standard update function
 *
 *************************************************************/
void Asura_ServerEntity_AdvancedLight::Update ( void )
{	
	// update position of entity based on master
	if(m_uMasterEntity != ASURA_GUID_UNREGISTERED)
	{
		Asura_ServerEntity_PhysicalObject* pxMaster = Asura_ServerEntity_PhysicalObject::FindEntity(m_uMasterEntity);
		if(pxMaster)
		{
			if(m_uUseMasterOrient)
			{
				Asura_Vector_3 xOffset = m_xOffsetFromMaster;
				xOffset = xOffset * pxMaster->GetOrientation();
				SetPosition( pxMaster->GetPosition() + xOffset );
			}
			else
			{
				SetPosition( pxMaster->GetPosition() + m_xOffsetFromMaster );
			}	
		}
	}
	
}
/* end of function Update */


/*************************************************************
 *
 * Name          : Asura_ServerEntity_AdvancedLight::MakeClientUpdateMessage
 *
 * Author        : Richard May
 *
 *************************************************************/
void Asura_ServerEntity_AdvancedLight::MakeClientUpdateMessage ( void )
{
	Asura_Message* pxMessage = Asura_Network::CreateMessage(ASURA_MSG_CLIENTUPDATE,	sizeof(Asura_Message_AdvancedLight_ClientUpdate), Classification);
	Asura_Assert( pxMessage, "Couldn't create UpdateMessage" );
	
	if(pxMessage)
	{
		Asura_Message_AdvancedLight_ClientUpdate* pxData = static_cast<Asura_Message_AdvancedLight_ClientUpdate*>(pxMessage->GetData());
		Asura_Assert(pxData,"Couldn't get message data pointer");

		if(pxData)
		{
			pxData->FillAdvancedLight(this);
			
			// (MH - 03.09.2002) if health is 0, guarantee the message to ensure client is destroyed.
			// instead, might want to have guaranteed as a parameter to this function.
			if (pxData->m_fHealth == 0.0f)
			{
				pxMessage->UseGuaranteedDelivery();
			}
		}
		Send(pxMessage,ASURA_GUID_MY_CLIENT_ENTITIES);
	}
}
/* end of function MakeClientUpdateMessage */


/*************************************************************
 *
 * Name          : Asura_ServerEntity_AdvancedLight::MakeCreateMessage
 *
 * Author        : Richard May
 *
 * Description   : Sends a creation message to the machine 
 *				   identified by uGuidFrom. If this is 
 *				   UNREGISTERED then it's all machines.
 *
 *************************************************************/
void Asura_ServerEntity_AdvancedLight::MakeCreateMessage( Asura_Guid uRequestingClientGUID, bool bSendGuaranteed/* = false*/ )
{
	Asura_Message* pxMsg = Asura_Network::CreateMessage( ASURA_MSG_CLIENTCREATE,
                                                         sizeof(Asura_Message_AdvancedLight_Create),
                                                         GetClassification() );
    if( !pxMsg ) { return; }

	Asura_Message_AdvancedLight_Create* pxMsgData = static_cast<Asura_Message_AdvancedLight_Create*>( pxMsg->GetData() );
	pxMsgData->FillAdvancedLight( this );

	pxMsg->SetToGuid( ASURA_GUID_MY_CLIENT_ENTITIES );
	pxMsg->SetFromGuid( GetGuid() );
	if (bSendGuaranteed)
	{
		pxMsg->UseGuaranteedDelivery();
	}
	Asura_Network::SendToSpecificMachine( pxMsg, uRequestingClientGUID );
}
/* end of function MakeCreateMessage */


/*************************************************************
 *
 * Name          : Asura_ServerEntity_AdvancedLight::SetStrobe
 *
 * Author        : Richard May
 *
 * Description   : Set the durations for flicker and strobe
 *
 *************************************************************/
void Asura_ServerEntity_AdvancedLight::SetStrobe( float fMaxOn, float fMinOn, float fMaxOff, float fMinOff )
{
	Asura_Assert( (fMaxOn >= fMinOn), "Max time on must be greater than min time on!" );
	Asura_Assert( (fMaxOff >= fMinOff), "Max time off must be greater than min time off!" );

	m_fMinTimeOff = fMinOff;
	m_fMaxTimeOff = fMaxOff;
	m_fMinTimeOn  = fMinOn;
	m_fMaxTimeOn  = fMaxOn;	
}
/* end of function SetStrobe */


/*************************************************************
 *
 * Name          : Asura_ServerEntity_AdvancedLight::SetFadeColours
 *
 * Author        : Richard May
 *
 * Description   : Set the initial and final colours for a colour fade
 *
 *************************************************************/
void Asura_ServerEntity_AdvancedLight::SetFadeColours ( float fStartR, float fStartG, float fStartB, float fEndR, float fEndG, float fEndB )
{
	m_fEndColourR = fEndR;
	m_fEndColourG = fEndG;
	m_fEndColourB = fEndB;
	
	m_fStartColourR = fStartR;
	m_fStartColourG = fStartG;
	m_fStartColourB = fStartB;
}
/* end of function SetFadeColours */


/*************************************************************
 *
 * Name          : Asura_ServerEntity_AdvancedLight::SetFadeDurations
 *
 * Author        : Richard May
 *
 * Description   : Set the min and max possible times for fading up (end to start)
 *				   and fading down (start to end)
 *
 *************************************************************/
void Asura_ServerEntity_AdvancedLight::SetFadeDurations ( float fMaxDownTime, float fMinDownTime, float fMaxUpTime, float fMinUpTime )
{	
	Asura_Assert( (fMaxDownTime >= fMinDownTime), "Max fade down time must be greater than min fade down time!" );
	Asura_Assert( (fMaxUpTime >= fMinUpTime), "Max fade up time must be greater than min fade up time!" );

	m_fMaxFadeDownDuration = fMaxDownTime;
	m_fMinFadeDownDuration = fMinDownTime;
	m_fMaxFadeUpDuration = fMaxUpTime;
	m_fMinFadeUpDuration = fMinUpTime;
}
/* end of function SetFadeDurations */


/*************************************************************
 *
 * Name          : Asura_ServerEntity_AdvancedLight::SetFadeEndColourDurations
 *
 * Author        : Richard May
 *
 * Description   : Sets the min & max durations for the end colours
 *
 *************************************************************/
void Asura_ServerEntity_AdvancedLight::SetFadeEndColourDurations ( float fMaxStartTime, float fMinStartTime, float fMaxEndTime, float fMinEndTime )
{
	Asura_Assert( (fMaxStartTime >= fMinStartTime), "Max start colour time must be greater than min start colour time!" );
	Asura_Assert( (fMaxEndTime >= fMinEndTime), "Max end colour time must be greater than min end colour time!" );
	
	m_fMinStartColourTime = fMinStartTime;
	m_fMaxStartColourTime = fMaxStartTime;
	m_fMinEndColourTime = fMinEndTime;
	m_fMaxEndColourTime = fMaxEndTime;
}
/* end of function SetFadeEndColourDurations */


/*************************************************************
 *
 * Name          : Asura_ServerEntity_AdvancedLight::SetFadeLoop
 *
 * Author        : Richard May
 *
 * Description   : Sets the value of the fade loop control 
 *
 *************************************************************/
void Asura_ServerEntity_AdvancedLight::SetFadeLoop ( bool bLoop )
{
	m_uLoopFade = bLoop;
}
/* end of function SetFadeLoop */


/*************************************************************
 *
 * Name          : Asura_ServerEntity_AdvancedLight::SetFadeColours
 *
 * Author        : Richard May
 *
 * Description   : Set the initial and final colours for a colour fade
 *
 *************************************************************/
void Asura_ServerEntity_AdvancedLight::SetRadii ( float fStartRadius, float fEndRadius )
{
	m_fStartRadius = fStartRadius;
	m_fEndRadius = fEndRadius;
}
/* end of function SetFadeColours */


/*************************************************************
 *
 * Name          : Asura_ServerEntity_AdvancedLight::SetRadiusChangeDurations
 *
 * Author        : Richard May
 *
 * Description   : Set the min and max possible times for fading up (end to start)
 *				   and fading down (start to end)
 *
 *************************************************************/
void Asura_ServerEntity_AdvancedLight::SetRadiusChangeDurations ( float fMaxDownTime, float fMinDownTime, float fMaxUpTime, float fMinUpTime )
{	
	Asura_Assert( (fMaxDownTime >= fMinDownTime), "Max fade down time must be greater than min fade down time!" );
	Asura_Assert( (fMaxUpTime >= fMinUpTime), "Max fade up time must be greater than min fade up time!" );

	m_fMaxRadiusDownDuration = fMaxDownTime;
	m_fMinRadiusDownDuration = fMinDownTime;
	m_fMaxRadiusUpDuration = fMaxUpTime;
	m_fMinRadiusUpDuration = fMinUpTime;
}
/* end of function SetFadeDurations */


/*************************************************************
 *
 * Name          : Asura_ServerEntity_AdvancedLight::SetFadeEndColourDurations
 *
 * Author        : Richard May
 *
 * Description   : Sets the min & max durations for the end colours
 *
 *************************************************************/
void Asura_ServerEntity_AdvancedLight::SetRadiusEndDurations ( float fMaxStartTime, float fMinStartTime, float fMaxEndTime, float fMinEndTime )
{
	Asura_Assert( (fMaxStartTime >= fMinStartTime), "Max start colour time must be greater than min start colour time!" );
	Asura_Assert( (fMaxEndTime >= fMinEndTime), "Max end colour time must be greater than min end colour time!" );
	
	m_fMinStartRadiusTime = fMinStartTime;
	m_fMaxStartRadiusTime = fMaxStartTime;
	m_fMinEndRadiusTime = fMinEndTime;
	m_fMaxEndRadiusTime = fMaxEndTime;
}
/* end of function SetFadeEndColourDurations */


/*************************************************************
 *
 * Name          : Asura_ServerEntity_AdvancedLight::SetRadiusChangeLoop
 *
 * Author        : Richard May
 *
 * Description   : Sets the value of the RadiusChange loop control 
 *
 *************************************************************/
void Asura_ServerEntity_AdvancedLight::SetRadiusChangeLoop ( bool bLoop )
{
	m_uLoopRadius = bLoop;
}
/* end of function SetRadiusChangeLoop */


#ifdef ASURA_SUPPORT_NETMSG_SYSTEM
#include "Network/Asura_NetMsg.h"
void Asura_Message_AdvancedLight_ClientUpdate::RegisterFields(u_short usMsgID, u_short usEntityClassification)
{
	Asura_Assert(usMsgID == ASURA_MSG_CLIENTUPDATE, "wrong msg ID");
	Asura_Assert(usEntityClassification == AsuraEntityClass_AdvancedLight, "wrong entity classification");
	Asura_Assert(!Asura_NetMsg::FindNetMsgObject(usMsgID, usEntityClassification), "already exists");
	
	u_char ucNumFields = GetNumFields();

	Asura_NetMsg* pxNetMsg = new Asura_NetMsg(usMsgID, usEntityClassification, ucNumFields, sizeof(Asura_Message_AdvancedLight_ClientUpdate));
	RegisterFields(pxNetMsg);
	pxNetMsg->EndRegistration();
}

void Asura_Message_AdvancedLight_ClientUpdate::RegisterFields(class Asura_NetMsg* pxNetMsg)
{
	const Asura_Message_AdvancedLight_ClientUpdate xDummy;
	Asura_Message_DestructibleLight_ClientUpdate::RegisterFields(pxNetMsg);
	Asura_NetMsg_RegisterField(pxNetMsg, xDummy, Asura_FieldType_uchar,	m_ucAdvLightFlags,	1,  Asura_Threshold_AnyChange,	0.0f);
}
#endif//#ifdef ASURA_SUPPORT_NETMSG_SYSTEM



#ifdef ASURA_TOOLS
void Asura_ServerEntity_AdvancedLight::ApplyChangesForTemplateInstance(Tools_EntityTemplateInstance* pxInstance)
{
	m_uMasterEntity = pxInstance->ConvertGuid(m_uMasterEntity);

	m_xOffsetFromMaster = m_xOffsetFromMaster * pxInstance->GetRelativeMatrix();
}
#endif // ASURA_TOOLS

/****************/
/* End of file. */
/****************/

#endif // #ifndef ASURA_TARGET_SM3
