//////////////////////////////////////////////////////////////////////////////
//
// Asura_ServerEntity_DestructibleLight.h
//
// Copyright (c) 2006 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef ASURA_SERVERENTITY_DESTRUCTIBLELIGHT_H___
#define ASURA_SERVERENTITY_DESTRUCTIBLELIGHT_H___ 1

#ifndef ASURA_TARGET_SM3

// I N C L U D E S ///////////////////////////////////////////////////////////

// Engine independent.
#include "Entity/PhysicalObject/Asura_ServerEntity_PhysicsObject.h"

// C L A S S E S /////////////////////////////////////////////////////////////

// (MH - 04.09.2002) the constructor for this class is protected. That's because you will have
// to derive a project-specific class from this to cope with project-specific damage types etc.

//Set of defines for the bitfield entries for use by the tools tickboxes. RAR 21/05/03
#define AsuraDestructibleLightFlag_HasCorona			0x00000001
#define AsuraDestructibleLightFlag_UseBoundingBox		0x00000002
#define AsuraDestructibleLightFlag_CoronaDontFadeWithDistance 0x00000004
#define AsuraDestructibleLightFlag_DontLightEnv			0x00000008

class Asura_ServerEntity_DestructibleLight : public virtual Asura_ServerEntity_PhysicsObject
{

public:

	static Asura_ServerEntity_DestructibleLight* FindEntity( Asura_Guid uGUID ) { return s_xList.FindEntity( uGUID ); }
	static const Asura_Hashed_List<Asura_ServerEntity_DestructibleLight>* GetList( void ) { return &s_xList; }

	float GetRange() const {return m_fRange;}
	bool HasCorona() const {return m_bHasCorona;}
	float GetBrightness() const { return m_fBrightness; }
	bool GetUseBoundingBox() const {return m_bUseBoundingBox;}
	const Asura_Bounding_Box& GetBoundingBox() const {return m_xBoundingBox;}
	float GetCoronaRadius() const {return m_fCoronaRadius;}
	Asura_Hash_ID GetCoronaTextureHash() const {return m_uCoronaTexture;}
	bool GetCoronaFlag_DontFadeWithDistance() const {return m_bCorona_DontFadeWithDistance;}
	bool GetDontLightEnvFlag() const {return m_bDontLightEnv;}

	void GetRGB(float& fRed, float& fGreen, float& fBlue) const
	{
		fRed   = m_fR;
		fGreen = m_fG;
		fBlue  = m_fB;
	}

	void SetRGB(float fRed, float fGreen, float fBlue)
	{
		m_fR = fRed;
		m_fG = fGreen;
		m_fB = fBlue;
	}

	void SetRange(float fRange)
	{
		m_fRange = fRange;
	}

	void SetCorona(bool bCorona) { m_bHasCorona = bCorona; }
	void SetCoronaRadius(float fRadius) {m_fCoronaRadius = fRadius;}
	void SetCoronaTextureHash(Asura_Hash_ID uHash) { m_uCoronaTexture = uHash;}

	virtual bool HandleMessage(const Asura_Message* pxMessage) { return false; }
	virtual void Update() {}

//	virtual	bool	IsFilledFromChunkBuffer		(void) const;

	virtual	int		GetChunkBufferSize			(void)	{ return sizeof(ChunkData) + sizeof(int); }
	virtual	void	FillInChunkBuffer			(char*& pcChunkBuffer);
	virtual void	InitialiseFromChunkBuffer	(const char*& pcChunkBuffer);

	virtual	void	ReadFromChunkStream			(Asura_Chunk_Stream& xStream);
	virtual	void	WriteToChunkStream			(Asura_Chunk_Stream& xStream) const;

	#ifdef ASURA_TOOLS
	virtual void TraverseUsedTextures(void (*pfnCallback)(int iTextureID,void* pUserData),void* pUserData) const;
	#endif

	#ifdef ASURA_TOOLS
	virtual void ApplyChangesForTemplateInstance(class Tools_EntityTemplateInstance* pxInstance);
	#endif //ASURA_TOOLS

protected:

	Asura_ServerEntity_DestructibleLight();
    virtual ~Asura_ServerEntity_DestructibleLight();

	// ChunkData
	struct ChunkData
	{
		enum {CURRENT_VERSION = 5};
		float m_fR, m_fG, m_fB;
		float m_fRange;
		float m_fBrightness;
		Asura_Bounding_Box m_xBoundingBox;
		float m_fCoronaRadius;
		Asura_Hash_ID m_uCoronaTexture;
		union
		{
			unsigned int m_uFlags;
			struct
			{
				// NOTE: These bit's are written out, adding any new bits means
				// ~~~~~ that you will need to add those bits to the bitfield
				//       preparation code in "WriteToChunkStream".
				unsigned int m_bHasCorona : 1,
							 m_bUseBoundingBox : 1,
							 m_bCorona_DontFadeWithDistance : 1,
							 m_bDontLightEnv : 1,

							 Unused : 28;
			};
		};
	};
	struct ChunkData_v3
	{
		float m_fR, m_fG, m_fB;
		float m_fRange;
		float m_fBrightness;
		Asura_Bounding_Box m_xBoundingBox;
		union
		{
			unsigned int m_uFlags;
			struct
			{
				unsigned int m_bHasCorona : 1,
							 m_bUseBoundingBox : 1,
							 Unused : 30;
			};
		};
	};
	struct ChunkData_v2
	{
		float m_fR, m_fG, m_fB;
		float m_fRange;
		float m_fBrightness;
		union
		{
			unsigned int m_uFlags;
			struct
			{
				unsigned int m_bHasCorona : 1;
			};
		};
	};

	struct ChunkData_v1
	{
		float m_fR, m_fG, m_fB;
	    float m_fRange;
		union
		{
			unsigned int m_uFlags;
			struct
			{
				unsigned int m_bHasCorona : 1;
			};
		};
	};


	virtual void MakeClientUpdateMessage();

    float m_fRange;
	//RAR 20/05/03 - Colours are 0 to 255
    float m_fR, m_fG, m_fB;
	float m_fBrightness;
	Asura_Bounding_Box m_xBoundingBox;

	float m_fCoronaRadius;
	Asura_Hash_ID m_uCoronaTexture;

	union
	{
		u_int m_uDestructibleLightFlags;

		struct
		{
			// NOTE: These bit's are written out, adding any new bits means
			// ~~~~~ that you will need to add those bits to the bitfield
			//       preparation code in "WriteToChunkStream".

			//The bitfield ordering must match the flag defines used by the tools at the top of the file. RAR 21/05/03
			unsigned int m_bHasCorona : 1,
						 m_bUseBoundingBox : 1,
						 m_bCorona_DontFadeWithDistance : 1,
						 m_bDontLightEnv : 1;
		};
	};

	friend class Tools_Entity_DestructibleLight;
	friend class Tools_Entity_AdvancedLight;
	
private:

    typedef Asura_ServerEntity_PhysicsObject PARENT;

    static Asura_Hashed_List<Asura_ServerEntity_DestructibleLight> s_xList;

};


class Asura_Message_DestructibleLight_ClientUpdate : public Asura_MsgData_PhysicsObject_ClientUpdate
{

public:

#ifdef ASURA_SUPPORT_NETMSG_SYSTEM
	static void RegisterFields(u_short usMsgID, u_short usEntityClassification);
	static void RegisterFields(class Asura_NetMsg* pxNetMsg);
	static u_char GetNumFields() { return static_cast< u_char >( 14 + PARENT::GetNumFields() ); }
#endif

    void FillDestructibleLight( const Asura_ServerEntity_DestructibleLight* pxObject );

    float m_fRange;
    float m_fR, m_fG, m_fB;
	float m_fBrightness;
	Asura_Bounding_Box m_xBoundingBox;
	float m_fCoronaRadius;
	Asura_Hash_ID m_uCoronaTexture;
	union 
	{
		u_char m_ucDestLightFlags;
		struct 
		{
			u_char m_bHasCorona :1;
			u_char m_bUseBoundingBox :1;
			u_char m_bCorona_DontFadeWithDistance : 1;
			u_char m_bDontLightEnv : 1;
		};
	};

private:

    typedef Asura_MsgData_PhysicsObject_ClientUpdate PARENT;

};

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

#endif // #ifndef ASURA_TARGET_SM3

#endif // #ifndef ASURA_SERVERENTITY_DESTRUCTIBLELIGHT_H___
