//////////////////////////////////////////////////////////////////////////////
//
// Asura_ServerEntity_Lift.h
//
// Copyright (c) 2006 Rebellion. All rights reserved.
//
// Description:
//
//     Lift Entity - External control physics object moving along a spline
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef ___ASURA_SERVERENTITY_LIFT_H___
#define ___ASURA_SERVERENTITY_LIFT_H___ 1


// I N C L U D E S ///////////////////////////////////////////////////////////

// Engine independent.
#include "Entity/PhysicalObject/Asura_ServerEntity_PhysicalObject.h"

// C L A S S E S /////////////////////////////////////////////////////////////

class Asura_ServerNode_LiftNode;
class Asura_ServerNode_LiftSpline;
class Asura_Message_Lift_SetFlags;


class Asura_ServerEntity_Lift : public virtual Asura_ServerEntity_PhysicalObject
{
public:
    static Asura_ServerEntity_Lift* FindEntity( Asura_Guid uGUID ) { return s_xList.FindEntity( uGUID ); }
    static const Asura_Hashed_List<Asura_ServerEntity_Lift>* GetList( void ) { return &s_xList; }    

    virtual bool HandleMessage( const Asura_Message* pxMsg );
    virtual void InitialiseFromChunkBuffer( const char*& pcChunkBfr );
    virtual void Update( void );
	//inner working of the update function, farmed out into a static so that tools can borrow it for preview
	static bool UpdateLift(Asura_Guid& uSpline, float& fProgress,
		bool& bReverse, const bool bAutoReverse, Asura_Vector_3& xPos, Asura_Matrix_3x3& xOri,
		const Asura_Guid uGotoNodeGuid, Asura_ServerNode_LiftNode*& pxNodeArrivedAt, const bool bJustStarted );
	virtual void ReadFromChunkStream( Asura_Chunk_Stream& xStream );
    virtual void WriteToChunkStream( Asura_Chunk_Stream& xStream ) const;

	bool IsRunning() const { return m_bRunning; }
	bool GetReversing() const { return m_bReversing;}
	void SetRunning( const bool bRunning ); //may also send a message asking client to play sound
	void SetReversing(bool bReversing);
	void GotoNode(Asura_Guid uGuid);
	void TeleportToNode(Asura_Guid uGuid);

	bool GetAutoReverse() const { return m_bAutoReverse;}
	float GetProgress() const { return m_fProgress;}
	Asura_Guid GetSplineGuid() const { return m_uSplineGuid;}
	Asura_Guid* GetSplineGuidPtr() { return &m_uSplineGuid;}
	bool GetPauseOnCollide() const { return m_bPauseOnCollide;}
	bool GetCollideEnv() const { return m_bCollideEnv;}
	bool GetCollideStatic() const { return m_bCollideStatic;}
	bool GetCollideDynamic() const { return m_bCollideDynamic;}

	void SetAutoReverse(bool bAutoReverse) { m_bAutoReverse = bAutoReverse;}
	void SetProgress(float fProgress) { m_fProgress = fProgress;}
	void SetPauseOnCollide(bool bPauseOnCollide) { m_bPauseOnCollide = bPauseOnCollide;}
	void SetCollideEnv(bool bCollideEnv) { m_bCollideEnv = bCollideEnv;}
	void SetCollideStatic(bool bCollideStatic) { m_bCollideStatic = bCollideStatic;}
	void SetCollideDynamic(bool bCollideDynamic) { m_bCollideDynamic = bCollideDynamic;}

	Asura_Hash_ID GetStartSoundEventHash() { return m_uStartSoundEventHash; }
	Asura_Hash_ID GetStopSoundEventHash() { return m_uStopSoundEventHash; }
	Asura_Hash_ID* GetStartSoundEventHashPointer() { return &m_uStartSoundEventHash; }
	Asura_Hash_ID* GetStopSoundEventHashPointer() { return &m_uStopSoundEventHash; }
	void SendStartSound( bool bSend ) { m_bSendStartSound = bSend; }
	void SendStopSound( bool bSend ) { m_bSendStopSound = bSend; }
	bool ShouldSendStartSound() { return m_bSendStartSound; }
	bool ShouldSendStopSound() { return m_bSendStopSound; }

#ifdef ASURA_TOOLS
    virtual void ApplyChangesForTemplateInstance( Tools_EntityTemplateInstance* pxInstance );

	void SetInitialNode( const int iNode );
	bool IsAtNode() const { return m_bAtNode; }
	void SetInitialSplineGuid( const Asura_Guid uSplineGuid ) { m_uSplineGuid = uSplineGuid; }
#endif

	static const Asura_ServerNode_LiftSpline* GetSplineFromNode(const Asura_ServerNode_LiftNode* pxNodeArrivedAt, const bool bReverse);

protected:
	Asura_ServerEntity_Lift();
    virtual ~Asura_ServerEntity_Lift( void );

	virtual void MakeClientUpdateMessage( void );
	virtual void PostLoadInitialise();
	void HandleLiftFlagsMessage(const Asura_Message_Lift_SetFlags* pxLiftMessage);

	void TriggerMessagesFromMovement(const Asura_ServerNode_LiftSpline* pxStartSpline, float fStartProgress, bool bStartReversed,
		 								const Asura_ServerNode_LiftSpline* pxEndSpline, float fEndProgress, bool bEndReversed);


	// AJG - 09/08/07 - Virtual function for the tests if objects are in the way of the lift. The instances come from a broadphase test, so the number of calls should be relatively few.
	//					Passing in the lift position, as this test is done at the potential new position for the lift.
	virtual bool IsInstanceBlockingMovement(const Asura_Physics_RigidBodyInstance* pxInstance, const Asura_Vector_3& xLiftPos, const Asura_Matrix_3x3& xLiftOri) const;

	union
	{
		u_int m_uLiftFlags;
		struct
		{
			u_int m_bOldActive		  :  1,//old active flag, do not use
			      m_bReversing		  :  1,//going from end to start on current spline
			      m_bAutoReverse	  :  1,//automatically reverse at dead ends, rather than auto-deactivating
			      m_bPauseOnCollide	  :  1,//pause on collision (rather than treat as dead end)
			      m_bCollideEnv		  :  1,//collide with environment
			      m_bCollideStatic	  :  1,//collide with statics
			      m_bCollideDynamic	  :  1,//collide with dynamics
			      m_bRunning		  :  1,//lift on the move
			      m_bJustStarted	  :  1,//lift has just had a start message
				  m_bSendStartSound   :  1,
				  m_bSendStopSound    :  1,
                  m_bAtNode			  :  1,
                  m_uLiftSpareFlags	  : 20;
		};
	};
	float m_fProgress;//0-1 indicates how far along current spline
	Asura_Guid m_uSplineGuid;//current spline
	Asura_Guid m_uGotoNodeGuid;		// Heading for this node.

	Asura_Hash_ID m_uStartSoundEventHash;
	Asura_Hash_ID m_uStopSoundEventHash;
    
private:
	const Asura_ServerNode_LiftSpline* GetNextSpline(const Asura_ServerNode_LiftSpline* pxSpline, bool bForwards);
	void BlockLiftNodeLinks( const bool bBlock );

	typedef Asura_ServerEntity_PhysicalObject PARENT;

    static const u_int uCHUNK_VERSION = 4;

    static Asura_Hashed_List<Asura_ServerEntity_Lift> s_xList;
};

class Asura_Message_Lift_SetFlags
{
public:
	void Reset() { m_uFlags = 0;}

	void FillInChunkBuffer(char*& pcChunkBuffer) const;
	void WriteToChunkStream(Asura_Chunk_Stream& xStream) const;

	bool GetAutoReverseOn() const { return m_bAutoReverseOn;}
	bool GetPauseOnCollideOn() const { return m_bPauseOnCollideOn;}
	bool GetCollideEnvOn() const { return m_bCollideEnvOn;}
	bool GetCollideStaticOn() const { return m_bCollideStaticOn;}
	bool GetCollideDynamicOn() const { return m_bCollideDynamicOn;}

	bool GetAutoReverseOff() const { return m_bAutoReverseOff;}
	bool GetPauseOnCollideOff() const { return m_bPauseOnCollideOff;}
	bool GetCollideEnvOff() const { return m_bCollideEnvOff;}
	bool GetCollideStaticOff() const { return m_bCollideStaticOff;}
	bool GetCollideDynamicOff() const { return m_bCollideDynamicOff;}

#ifdef ASURA_TOOLS
	u_int* GetFlagsPtr() { return &m_uFlags;}
#endif // #ifdef ASURA_TOOLS

protected:
	union
	{
		u_int m_uFlags;
		struct
		{
			u_int m_bAutoReverseOn		:  1,//automatically reverse at dead ends, rather than auto-deactivating
			      m_bAutoReverseOff		:  1,//automatically reverse at dead ends, rather than auto-deactivating
			      m_bPauseOnCollideOn	:  1,//pause on collision (rather than treat as dead end)
			      m_bPauseOnCollideOff	:  1,//pause on collision (rather than treat as dead end)
			      m_bCollideEnvOn		:  1,//collide with environment
			      m_bCollideEnvOff		:  1,//collide with environment
			      m_bCollideStaticOn	:  1,//collide with statics
			      m_bCollideStaticOff	:  1,//collide with statics
			      m_bCollideDynamicOn	:  1,//collide with dynamics
			      m_bCollideDynamicOff	:  1,//collide with dynamics
			      m_uSpareFlags			: 22;
		};
	};
};



class Asura_MsgData_Lift_Goto_Node
{
public:
	void FillInChunkBuffer(char*& pcChunkBuffer) const;
	void WriteToChunkStream(Asura_Chunk_Stream& xStream) const;

	bool GetTeleportToNode() const { return m_bTeleportToNode;}
	Asura_Guid GetNodeGuid() const { return m_uNodeGuid;}

#ifdef ASURA_TOOLS
	u_int* GetFlagsPtr() { return &m_uFlags;}
	Asura_Guid* GetNodeGuidPtr() { return &m_uNodeGuid;}
#endif // #ifdef ASURA_TOOLS

protected:
	// NY - 16 July 2008 - there used to be a version u_int here, even though static messages don't do versioning :)
	// handily, it means there was a zero'd u_int sat there doing nothing, so we now have some flags :)
	union
	{
		u_int m_uFlags;
		struct
		{
			u_int	m_bTeleportToNode		: 1,//teleport to node, rather than trying to travel there
					m_uSpareFlags			: 31;
		};
	};
	Asura_Guid m_uNodeGuid;
};


// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

#endif // #ifndef ___ASURA_SERVERENTITY_LIFT_H___
