//////////////////////////////////////////////////////////////////////////////
//
// Asura_ServerEntity_Projector.h
//
// Copyright (c) 2007 Rebellion (Derby). All rights reserved.
//
// Description:
//
// An entity to project a texture onto the environment
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif

#ifndef ___ASURA_SERVERENTITY_PROJECTOR_H___
#define ___ASURA_SERVERENTITY_PROJECTOR_H___ 1

#ifdef ASURA_TARGET_SM3

// I N C L U D E S ///////////////////////////////////////////////////////////

// Engine independent.
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "Core/Asura_String.h"
#include "Entity/Asura_Entity.h"
#include "Entity/Projector/Asura_ServerEntity_Projector.h"


// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////////

#ifdef ASURA_TOOLS
class Tools_Entity_Projector;
#endif

class Asura_ClientEntity_Projector;

// C L A S S E S /////////////////////////////////////////////////////////////

class Asura_ServerEntity_Projector : public Asura_Entity_Server
{
public:

	// T Y P E S /////////////////////////////////////////////////////////

	// Projector Parameters
	class ProjectorParams
	{
	public:
		ProjectorParams();

		// Pos/Orient
		Asura_Vector_3   m_xPosition;
		Asura_Matrix_3x3 m_xOrientation;

		// Textures
		Asura_Hash_ID m_uTextureHash;
		Asura_Hash_ID m_uFalloffTextureHash;
		Asura_Hash_ID m_uDetailTextureHash;

		float m_fDetailTextureScale;

		// Falloff range
		float m_fMinRange;
		float m_fMaxRange;

		// Blend mode
		int   m_iBlendMode;

		// FoV + Type
		float m_fFoV;
		float m_fScale;
		float m_fAspect;

		// Fadeout Distance
		float m_fFadeoutStart;
		float m_fFadeoutDist;

		// Movement Params
		Asura_Euler_3 m_xRotSpeed;
		Asura_Euler_3 m_xSwingDist;

		// Extra bits
		float m_fRotationOffset;

		float m_fIntensity;

		// New since attachements were added
		float m_afRGB[3];
		Asura_Guid m_uMasterEntity;
		Asura_Vector_3 m_xOffsetFromMaster;
		Asura_Hash_ID m_uMasterMarker;

		union
		{
			u_int m_uProjectorParams_Flags;
			struct
			{
				u_int	m_bFakeLight					: 1,
						m_bDynamicLight					: 1,
						m_bParallel						: 1,
						m_bMoving						: 1,
						m_bSwingX						: 1,
						m_bSwingY						: 1,
						m_bSwingZ						: 1,
						m_bShowFrustum					: 1,
						m_bQuadIntensity				: 1,
						m_bUseMasterOrient				: 1,
						m_bProjectorParamsUnusedBits	: 22;
			};
		};

#ifdef ASURA_TOOLS
		Asura_String    m_xTextureName;
		Asura_String    m_xFalloffTextureName;
		Asura_String    m_xDetailTextureName;
#endif // #ifdef ASURA_TOOLS
	};



	// C O N S T R U C T O R S   /   D E S T R U C T O R S ///////////////
#ifdef ASURA_TOOLS
	static Asura_ServerEntity_Projector* CreateFromTools( Asura_Guid uGuid, const Asura_Vector_3& xCentre );
#endif
	Asura_ServerEntity_Projector( Asura_Guid uGuid);

	// C L A S S   M E T H O D S /////////////////////////////////////////
	static void CreateFromChunkStream( Asura_Guid uGuid, Asura_Chunk_Stream &xStream );
	static void ExtractParamsFromChunkStream( ProjectorParams &xParams , Asura_Chunk_Stream &xStream, u_int uVersion);
	static void WriteParamsToChunkStream( const ProjectorParams &xParams , Asura_Chunk_Stream& xStream);

	// M E T H O D S /////////////////////////////////////////////////////

	// Query methods.
	const ProjectorParams& GetParams( void ) const { return m_xParams; }

	// Set methods.
#ifdef ASURA_TOOLS
	void SetParams( const ProjectorParams& xNewParams );
#endif

	// Load / save.
	virtual	void ReadFromChunkStream(Asura_Chunk_Stream& xStream);
	virtual	void WriteToChunkStream(Asura_Chunk_Stream& xStream) const;

//        virtual int  GetChunkBufferSize( void );
	virtual bool CanBeSavedToChunk() const {return true;}
	virtual	bool  IsFilledFromChunkBuffer(void) const	{ return(false); }

	#ifdef ASURA_TOOLS
	virtual void ApplyChangesForTemplateInstance(class Tools_EntityTemplateInstance* pxInstance);
	virtual bool CanBeIncludedInTemplate() const { return true; }
	#endif

protected:

	// C O N S T R U C T O R S   /   D E S T R U C T O R S ///////////////

	virtual ~Asura_ServerEntity_Projector( void );

	// S T A T E /////////////////////////////////////////////////////////

	ProjectorParams m_xParams;

private:

	// F R I E N D S /////////////////////////////////////////////////////

#ifdef ASURA_TOOLS
	friend Tools_Entity_Projector;
#endif

	// T Y P E S /////////////////////////////////////////////////////////
	typedef Asura_Entity_Server PARENT;
};
// End class Asura_ServerEntity_Projector


// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

#endif //#ifdef ASURA_TARGET_SM3
#endif // ___ASURA_SERVERENTITY_PROJECTOR_H___
