
//----------------------------------------------------------------------------------------
//////////////////////////////////////////////////////////////////////////////////////////
//
// Asura_ClientEntity_Ragdoll.h
//
// Created by KJFL - Tuesday, November 09, 2004
//
// Copyright (C) 2004 Rebellion, All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////////////////
//----------------------------------------------------------------------------------------


#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __ASURA_CLIENTENTITY_RAGDOLL_H__
#define __ASURA_CLIENTENTITY_RAGDOLL_H__

#ifndef ASURA_PHYSICS_DISABLE_RAGDOLL

#include "Camera/Asura_Camera.h"
#include "Entity/Asura_Entity.h"




class Asura_Animation;
class Asura_Hierarchy_Skin;
class Asura_Physics_Ragdoll;
class Asura_Physics_Ragdoll_Description;

//----------------------------------------------------------------------------------------
//////////////////////////////////////////////////////////////////////////////////////////
//
//	CLASS    :  Asura_ClientEntity_Ragdoll
//
//  CREATED  :  Kevin Floyer-Lea (9.11.2004)
//
//  DESC     :  Acts as interface between Asura_Physics_Ragdoll and the Entity system
//
//////////////////////////////////////////////////////////////////////////////////////////
//----------------------------------------------------------------------------------------


class Asura_ClientEntity_Ragdoll : public Asura_Entity_Renderable
{
	protected:

		//////////////////////////////////////////////////////////////////////////
		// Constructor 
		//////////////////////////////////////////////////////////////////////////

		Asura_ClientEntity_Ragdoll( const Asura_Vector_3& xPosition, const Asura_Matrix_3x3& xOrientation, const Asura_Animation* pxAnimation, Asura_Guid uGuid, const Asura_Physics_Ragdoll_Description& xRagdollDesc, bool bAnimateInitialPositions = false );

	public:

		//////////////////////////////////////////////////////////////////////////
		//  Destructor and virtual member functions from base class
		//////////////////////////////////////////////////////////////////////////		
		
		virtual ~Asura_ClientEntity_Ragdoll();

		virtual void Render();
		virtual void Update();
		
		void ReadFromChunkStream( Asura_Chunk_Stream& xStream );
		void WriteToChunkStream( Asura_Chunk_Stream& xStream ) const;

		// Apply an impulse to a bone of the ragdoll
		virtual void ApplyImpulse( const int iHitBoneID, const Asura_Vector_3 &xImpulse );//, bool bReactivate = true );
		
		// Apply an impulse to the whole ragdoll
		virtual void ApplyImpulse( const Asura_Vector_3 &xImpulse );

		////////////////////////////////////////////////////////////
		//  Class methods
		////////////////////////////////////////////////////////////
	
		void SetAsFadingOut( void );

		bool IsFadingOut( void ) const;

		// Get the ID of the ragdoll. This is unique to each ragdoll, but it is *not* an entity guid
		// (since ragdolls are only client entities, and don't have a corresponding server entity).
		// This ID can be used in calls to FindRagdollEntity()
		//
		// KJFL - Friday, November 12, 2004 - I was going to make this a u_int, but existing Rogue code
		// uses an int to store ragdoll IDs.
		int GetRagdollID() const { return m_iRagdollID; }

		// Get the GUID of the original entity that spawned the ragdoll (e.g. an NPC).
		// Since ragdolls only exist on clients (i.e. their movement isn't duplicated 
		// exactly across networks) they don't have valid entity guids.
		u_int GetOriginalEntityGuid() const { return m_uOriginalEntityGuid; }

        void SetRenderFlags( u_int uFlags ) { m_uRenderFlags = uFlags; }

		u_int GetRenderFlags() const { return m_uRenderFlags; }

		// Accessor for the ragdoll's Asura_Animation 
		Asura_Animation* GetAnimation( void ) const { return m_pxAnimation; }

		// Accessor for the ragdoll's Asura_Hierarchy_Skin
		const Asura_Hierarchy_Skin* GetSkin( void ) const;

		// Query whether ragdoll is active (i.e. potentially moving around and taking up processor time) or just 'asleep'.
		bool IsActive( void ) const;

        // Make ragdoll inactive (i.e. it won't move or take processor time, unless something else makes it active such
		// as a player shooting at it). The physics system will generally make ragdolls inactive itself under the right
		// conditions, but this function is useful for situations where you know it should be inactive, e.g. you've just
		// made a ragdoll after completing a cutscene kill-move, and you just want a body in the right place.
		void Deactivate( void );

		void Activate( void );

		const Asura_Bounding_Box& GetBB( void ) const;

		Asura_Handle			GetRagdollWrapperHandle() const { return m_iRagdollWrapperHandle; }

		////////////////////////////////////////////////////////////
		//  Static member functions
		////////////////////////////////////////////////////////////

		// Factory function
		static Asura_ClientEntity_Ragdoll* Create( const Asura_Vector_3& xPosition, const Asura_Matrix_3x3& xOrientation, const Asura_Vector_3& xVelocity, const Asura_Vector_3& xImpactStrength, int iHitLocation, const Asura_Animation* pxAnimation, Asura_Guid uGuid, const Asura_Physics_Ragdoll_Description& xRagdollDesc );

		// Find a ragdoll from an ID. The ID of a ragdoll entity can be found by querying GetRagdollID().
		static Asura_ClientEntity_Ragdoll* FindRagdollEntity( int iRagdollID );
			
		// Grab the entity list for your own nefarious needs
		static const Asura_Hashed_List<Asura_ClientEntity_Ragdoll>* GetList()
		{
			return &s_xClientEntityRagdollList;
		}
		
	protected:

		Asura_Physics_Ragdoll*	GetPhysicsRagdollPointer( void ) const;

		////////////////////////////////////////////////////////////
		//  Static data members
		////////////////////////////////////////////////////////////
		static Asura_Hashed_List< Asura_ClientEntity_Ragdoll >	s_xClientEntityRagdollList;
		static int												s_iNextRagdollID;
		static u_int											s_uTotalNumberOfRagdolls;

	
		////////////////////////////////////////////////////////////
		//  Per instance data members
		////////////////////////////////////////////////////////////

		Asura_Animation*		m_pxAnimation;
		// Unique ID for the ragdolls so they can be searched for. Note this is NOT 
		// an entity guid. Since ragdolls only exist on clients (i.e. their movement
		// isn't duplicated exactly across networks) they don't have valid guids.
		// 
		// KJFL - Friday, November 12, 2004 - I was going to make this a u_int, but existing Rogue code
		// uses an int to store ragdoll IDs.
		int						m_iRagdollID;		
		
		Asura_Handle			m_iRagdollWrapperHandle;

		u_int					m_uRenderFlags;

		// The guid of the object that created the ragdoll. Since ragdolls only exist on 
		// clients (i.e. their movement isn't duplicated exactly across networks) they
		// don't have valid guids. Therefore they use the guid of the original entity
		// when answering LOS calls etc.
		Asura_Guid				m_uOriginalEntityGuid;

};


#endif // #ifndef ASURA_PHYSICS_DISABLE_RAGDOLL


#endif//__ASURA_CLIENTENTITY_RAGDOLL_H__
