//////////////////////////////////////////////////////////////////////////////
//
// Asura_Ragdoll_Wrapper_System.cpp
//
// Created by Will Plano 31-Jul-2007
//
// Copyright (c) 2007 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
// includes
//////////////////////////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"

#ifndef ASURA_PHYSICS_DISABLE_RAGDOLL

// header
#include "Entity/Ragdoll/Asura_Ragdoll_Wrapper.h"


#include "Animation/Asura_Animation.h"
//#include "Camera/Asura_Camera.h"

//#include "ChunkSystem/Asura_Chunk_Stream.h"

//#include "Core/Asura_LOS.h"
//#include "Core/Asura_MaterialFlags.h"
#include "Core/Asura_Timers.h"

#include "Entity/Asura_Entity_Renderable.h"
#include "Entity/Ragdoll/Asura_ClientEntity_Ragdoll.h"
#include "Network/Asura_Message.h"
#include "Network/Asura_Network.h"
#include "Physics/Asura_Physics_Ragdoll.h"
#include "ProjectSettings/Asura_PhysicsConfiguration.h"
#include "Rendering/Asura_Module.h"

Asura_Tracking_Handle_List						Asura_Ragdoll_Wrapper::s_xHandlerHandleList( uASURA_PHYSICS_UPPER_RAGDOLL_LIMIT );
Asura_Ragdoll_Wrapper*							Asura_Ragdoll_Wrapper::s_apxHandlers[ uASURA_PHYSICS_UPPER_RAGDOLL_LIMIT ];
int												Asura_Ragdoll_Wrapper::s_aiHandlerIDs[ uASURA_PHYSICS_UPPER_RAGDOLL_LIMIT ];
int												Asura_Ragdoll_Wrapper::s_iNextHandlerID = 0;

//////////////////////////////////////////////////////////////////////////
// Asura_Ragdoll_Wrapper
//////////////////////////////////////////////////////////////////////////

Asura_Ragdoll_Wrapper::Asura_Ragdoll_Wrapper( Asura_Guid uOwnerGuid, int iPriority, const Asura_Physics_Ragdoll_Description& xRagdollDesc, Asura_Animation* pxAnimation, const Asura_Vector_3& xPosition, const Asura_Matrix_3x3& xOrientation, bool bAnimateInitialPositions/* = false*/ )
{

	m_iHandlerID = ASURA_INVALID_HANDLE;

	if ( !s_xHandlerHandleList.GetNewHandle( m_iHandlerHandle ) )
	{
		Asura_Assert( false, "No more handles for ragdolls, RemoveMostObsoleteRagdoll() must've failed" );
		return;
	}

	m_iHandlerID = s_iNextHandlerID++;

	s_apxHandlers[ m_iHandlerHandle ] = this;
	s_aiHandlerIDs[ m_iHandlerHandle ] = m_iHandlerID;

#if !defined (ASURA_USE_NEW_VISIBILITY_SYSTEM )
	for ( int i = 0; i < ASURA_MAX_NO_OF_CAMERAS; ++i )
	{
		m_iLastKnownModule[ i ] = -1;

#if defined(ASURA_USE_STREAMING)
		m_auLastKnownZone[ i ] = ASURA_HASH_ID_UNSET;
#endif //defined(ASURA_USE_STREAMING)
	}
#endif

	m_iPhysicsRagdollHandle = Asura_Physics_Ragdoll_System::CreateRagdoll( xRagdollDesc, pxAnimation, xPosition, xOrientation, bAnimateInitialPositions );

	m_iPriority = iPriority;

	m_fFadeOutDuration = 1.0f / fASURA_PHYSICS_RAGDOLL_FADEOUT_SPEED;

	m_fFadeOutTimer = -1.0f;

	m_fFadeOutDelay = -1.0f; // < 0.0f = no auto fade out

	m_fFadeOutDelayTimer = -1.0f;

	m_uOwnerGuid = uOwnerGuid;

	m_uCreationFrameStamp = Asura_Timers::GetFrameNumber();

	m_uFlags	= 0;
	m_bFadeOut = true;

}

Asura_Ragdoll_Wrapper::~Asura_Ragdoll_Wrapper()
{
	Asura_Physics_Ragdoll_System::DestroyRagdoll( m_iPhysicsRagdollHandle );
	s_apxHandlers[ m_iHandlerHandle ] = NULL;
	s_aiHandlerIDs[ m_iHandlerHandle ] = -1;
	s_xHandlerHandleList.ReleaseHandle( m_iHandlerHandle );
	if ( m_bDestroyServerEntity )
	{
		SetServerEntityAsDestroyed();
	}
	if ( m_bDestroyClientEntity )
	{
		SetClientEntityAsDestroyed();
	}
}

void Asura_Ragdoll_Wrapper::Update( Asura_Animation* pxAnimation )
{
	// update our entity's position and orientatoin.
	Asura_Entity_Renderable* pxClient = GetClientEntity();
	
	Asura_Physics_Ragdoll* pxPhysicsRagdoll = GetPhysicsRagdollPointer();
	if ( pxClient && pxPhysicsRagdoll )
	{
		Asura_Vector_3 xPos( pxClient->GetCurrentPosition() );
		Asura_Matrix_3x3 xOri( pxClient->GetCurrentOrientation() );
		pxPhysicsRagdoll->GetRenderingPosition( xPos );
		pxPhysicsRagdoll->GetRenderingOrientation( xOri );
		pxClient->TeleportTo( xPos ,xOri );
	}

	// should we be fading out?
	if ( m_bFadeOut )
	{
		if ( m_fFadeOutDelay >= 0.0f )
		{
			if ( !IsActive() )
			{
				if ( m_fFadeOutDelayTimer >= 0.0f )
				{
					m_fFadeOutDelayTimer -= Asura_Timers::GetGameTimeSlice();
					if ( m_fFadeOutDelayTimer <= 0.0f )
					{
						m_fFadeOutDelayTimer = -1.0f;
						SetAsFadingOut();
					}
				}
			}
			else
			{
				m_fFadeOutDelayTimer = m_fFadeOutDelay;
			}
		}
		if ( IsFadingOut() )
		{
			m_fFadeOutTimer -= Asura_Timers::GetGameTimeSlice();

			if (m_fFadeOutTimer <= 0.0f)
			{
				m_fFadeOutTimer = -1.0f;
				Destroy( m_iHandlerID );
			}
			// set the ragdoll's alpha, and all its attachments too
			else
			{
				float fAlpha = m_fFadeOutTimer / m_fFadeOutDuration;
				if ( pxAnimation )
				{
					pxAnimation->SetAlpha( fAlpha, true );
				}
			}
		}
	}
#if 0 // for debugging, draw ragdolls transparent when asleep
	float fAlpha = 1.0f;

	if (!IsActive())
	{
		fAlpha = 0.8f;
	}

	if ( pxAnimation )
	{
		pxAnimation->SetAlpha( fAlpha, true );
	}
#endif
}

void Asura_Ragdoll_Wrapper::SetAsFadingOut( void )
{
	// don't restart the timer if it's already going
	if (!IsFadingOut())
	{
		m_fFadeOutTimer = m_fFadeOutDuration;
	}
}

bool Asura_Ragdoll_Wrapper::IsFadingOut( void ) const
{
	return ( m_fFadeOutTimer >= 0.0f );
}

void Asura_Ragdoll_Wrapper::SetServerEntityAsDestroyed( void ) const
{
	if ( Asura_Entity_Server::FindEntity( m_uOwnerGuid ) )
	{
		Asura_Message* pxMessage = Asura_Network::CreateMessage( ASURA_MSG_DESTRUCT, 0 );
		if ( pxMessage )
		{
			pxMessage->SetToGuid( m_uOwnerGuid );
			pxMessage->SetFromGuid( Asura_Network::GetClientID() );
			pxMessage->UseGuaranteedDelivery();
			Asura_Network::Send( pxMessage );
		}
	}
}

void Asura_Ragdoll_Wrapper::SetClientEntityAsDestroyed( void ) const
{
	Asura_Entity_Client* pxClient = GetClientEntity();
	if ( pxClient )
	{
		pxClient->SetAsDestroyed();
	}
}

bool Asura_Ragdoll_Wrapper::IsClientEntityDestroyed( void ) const
{
	const Asura_Entity_Client* pxClient = GetClientEntity();
	if (pxClient)
	{
		return pxClient->IsDestroyed();
	}
	return true;
}

Asura_Physics_Ragdoll* Asura_Ragdoll_Wrapper::GetPhysicsRagdollPointer( void ) const
{
	Asura_Physics_Ragdoll* pxRagdoll = Asura_Physics_Ragdoll_System::GetRagdollPointer( m_iPhysicsRagdollHandle );

	Asura_Assert( pxRagdoll, "No ragdoll pointer." );

	return pxRagdoll;
}

bool Asura_Ragdoll_Wrapper::IsBetterToRemoveThan( const Asura_Ragdoll_Wrapper* pxOther, bool bRemoveImmediately ) 
{
	// any opposition?
	if (!pxOther) return true;

	// A ragdoll is better to remove if it is...
	//
	// 1. Not in a linked in module
	//
	// 2. Off-screen (to be tested if 1. is not true)
	//
	// 3. Already fading if we have to remove it immediately
	//
	// 3. Older
	//

	// if candidate isn't linked in but the other one is, candidate wins
	if ( !m_bIsLinkedInToVisibleModule && pxOther->m_bIsLinkedInToVisibleModule )
	{
		return true;
	}
	// if the candidate is linked in but the other one isn't, other one wins
	else if ( m_bIsLinkedInToVisibleModule && !pxOther->m_bIsLinkedInToVisibleModule )
	{
		return false;
	}

	// otherwise they're either both linked in or both not linked in
	// in that case, the one which is off-screen wins

	if ( !m_bIsLinkedInAndInFrustum && pxOther->m_bIsLinkedInAndInFrustum )
	{
		return true;
	}
	else if ( m_bIsLinkedInAndInFrustum && !pxOther->m_bIsLinkedInAndInFrustum )
	{
		return false;
	}

	// okay both off-screen or on-screen
	// if we need to remove one immediately and one is already fading, let's take that one
	if ( bRemoveImmediately )
	{
		if ( IsFadingOut() && !pxOther->IsFadingOut() )
		{
			return true;
		}
		else if (!IsFadingOut() && pxOther->IsFadingOut() )
		{
			return false;
		}
	}
	// otherwise if one is already fading out then it should not be counted so take the other one
	else
	{
		if ( IsFadingOut() && !pxOther->IsFadingOut() )
		{
			return false;
		}
		else if (!IsFadingOut() && pxOther->IsFadingOut() )
		{
			return true;
		}
	}

	// if our priority is lower than the other one's, remove us first.
	if ( m_iPriority < pxOther->m_iPriority )
	{
		return true;
	}
	else if ( m_iPriority > pxOther->m_iPriority )
	{
		return false;
	}

	// comes down to a simple age test: candidate is better if it's older
	// (older means lower frame number stamp)
	return ( m_uCreationFrameStamp < pxOther->m_uCreationFrameStamp );

}

u_int Asura_Ragdoll_Wrapper::GetNumberOfFadingRagdolls() 
{
	u_int uNumFading = 0;
	
	Asura_Tracking_Handle_List_Iterator xIt = s_xHandlerHandleList.GetIterator();

	xIt.Begin();
	while( !xIt.End() )
	{
		int iHandle = xIt.GetCurrentHandle();
		xIt.Next();
		if ( s_apxHandlers[ iHandle ]->IsFadingOut() )
		{
			++uNumFading;
		}
	}

	return uNumFading;
}

void Asura_Ragdoll_Wrapper::Activate( void )
{
	Asura_Physics_Ragdoll* pxRagdoll = GetPhysicsRagdollPointer();

	if (pxRagdoll) 
		pxRagdoll->WakeUp();
}

void Asura_Ragdoll_Wrapper::Deactivate( void )
{
	Asura_Physics_Ragdoll* pxRagdoll = GetPhysicsRagdollPointer();

	if (pxRagdoll)
	{
		// a ragdoll automatically updates its anim as it falls asleep, since
		// the anim is never updated again until it's woken.
		pxRagdoll->Sleep();
	}
}


bool Asura_Ragdoll_Wrapper::IsActive( void ) const
{
	const Asura_Physics_Ragdoll* pxRagdoll = GetPhysicsRagdollPointer();

	if (pxRagdoll)
	{
		return !pxRagdoll->IsAsleep();
	}
	else
	{
		return false;
	}
}

const Asura_Bounding_Box& Asura_Ragdoll_Wrapper::GetBB( void ) const
{
	const Asura_Physics_Ragdoll* pxRagdoll = GetPhysicsRagdollPointer();

	// KJFL - Friday, November 12, 2004 - don't like the way this function can't cope if there isn't a ragdoll....
	// perhaps the client entity should have it's own bb...
	return pxRagdoll->GetBoundingBox();
}

Asura_Ragdoll_Wrapper* Asura_Ragdoll_Wrapper::GetRagdollWrapper( int iRagdollWrapperID )
{
	Asura_Tracking_Handle_List_Iterator xIt = s_xHandlerHandleList.GetIterator();

	xIt.Begin();
	while( !xIt.End() )
	{
		int iHandle = xIt.GetCurrentHandle();
		xIt.Next();
		if ( s_aiHandlerIDs[ iHandle ] == iRagdollWrapperID )
		{
			return s_apxHandlers[ iHandle ];
		}
	}

	return NULL;
}

bool Asura_Ragdoll_Wrapper::RemoveMostObsoleteRagdoll( const bool bRemoveImmediately, const bool bOnlyConsiderActiveRagdolls, Asura_Guid uParentGuid )
{
	Asura_Tracking_Handle_List_Iterator xIt = s_xHandlerHandleList.GetIterator();

	Asura_Ragdoll_Wrapper* pxMostFavourableCandidate = 0;

	xIt.Begin();
	while( !xIt.End() )
	{
		Asura_Ragdoll_Wrapper* pxRagdollEntity = s_apxHandlers[ xIt.GetCurrentHandle() ];
		xIt.Next();

		// if this entity has already been flagged for destruction, keep looking
		if ( pxRagdollEntity && pxRagdollEntity->IsClientEntityDestroyed() ) continue;

		// if we are only interested in active ragdolls, ignore any sleeping ones
		if ( bOnlyConsiderActiveRagdolls && (!pxRagdollEntity->IsActive()) ) continue;

#if defined (ASURA_USE_NEW_VISIBILITY_SYSTEM )
		{
			const u_int uNumberOfCameras = Asura_Camera::GetNumberOfCameras();

			pxRagdollEntity->m_bIsLinkedInToVisibleModule = false;
			pxRagdollEntity->m_bIsLinkedInAndInFrustum = false;

			for( u_int uCam = 0; uCam < uNumberOfCameras; ++uCam )
			{
				Asura_Camera::SetCameraIndex( uCam );

				if( !Asura_Camera::IsCameraActive() ) { continue; }

				if ( pxRagdollEntity->IsVisibleFromCurrentCamera() )
				{
					pxRagdollEntity->m_bIsLinkedInToVisibleModule = true;

					if( Asura_Camera::IsAABBInFrustum( pxRagdollEntity->GetBB() ) )
					{
						pxRagdollEntity->m_bIsLinkedInAndInFrustum = true;
						break;
					}

				}
			}
		}
#else //!defined (ASURA_USE_NEW_VISIBILITY_SYSTEM )
		{
			const u_int uNumberOfCameras = Asura_Camera::GetNumberOfCameras();

			const Asura_Bounding_Box& xBB = pxRagdollEntity->GetBB();
			Asura_Vector_3 xBSCentre = xBB.CalculateCentre();
			float          fBSRadius = 0.5f * xBB.CalculateExtents().Magnitude();

			pxRagdollEntity->m_bIsLinkedInToVisibleModule = false;
			pxRagdollEntity->m_bIsLinkedInAndInFrustum = false;

			for( u_int uCam = 0; uCam < uNumberOfCameras; ++uCam )
			{
				Asura_Camera::SetCameraIndex( uCam );

				if( !Asura_Camera::IsCameraActive() ) { continue; }

#if defined(ASURA_USE_STREAMING)
				if( Asura_Module_System::DoesBBIntersectVisibleModule( xBB, &pxRagdollEntity->m_iLastKnownModule[uCam], &pxRagdollEntity->m_auLastKnownZone[uCam]) )
#else
				if( Asura_Module_System::DoesBBIntersectVisibleModule( xBB, &pxRagdollEntity->m_iLastKnownModule[uCam]) )
#endif //defined(ASURA_USE_STREAMING)
				{
					pxRagdollEntity->m_bIsLinkedInToVisibleModule = true;

					if( Asura_Camera::IsSphereInFrustum(xBSCentre, fBSRadius) )
					{
						pxRagdollEntity->m_bIsLinkedInAndInFrustum = true;
						break;
					}

				}
			}
		}
#endif //!defined (ASURA_USE_NEW_VISIBILITY_SYSTEM )

		if( pxRagdollEntity->IsBetterToRemoveThan( pxMostFavourableCandidate, bRemoveImmediately ) && ( pxRagdollEntity->GetOwnerGuid() == ASURA_GUID_UNREGISTERED || pxRagdollEntity->GetOwnerGuid() != uParentGuid ) )
		{
			pxMostFavourableCandidate = pxRagdollEntity;
		}
	}

	if ( pxMostFavourableCandidate )
	{
		if ( bRemoveImmediately )
		{
			Destroy( pxMostFavourableCandidate->GetWrapperID() );
		}
		else
		{
			pxMostFavourableCandidate->SetAsFadingOut();
		}
		return true;
	}
	else
	{
		return false;
	}
}

int Asura_Ragdoll_Wrapper::Create( Asura_Guid uOwnerGuid, Asura_Guid uParentGuid, int iPriority, const Asura_Physics_Ragdoll_Description& xRagdollDesc, Asura_Animation* pxAnimation, const Asura_Vector_3& xPosition, const Asura_Matrix_3x3& xOrientation, const Asura_Vector_3& xVelocity /* = Asura_Maths::ZeroVector3 */, const Asura_Vector_3& xImpactStrength /* = Asura_Maths::ZeroVector3 */, const Asura_Vector_3& xImpactPosition /* = Asura_Maths::ZeroVector3 */ )
{
	bool bDoIt = true;
	u_int uNumberOfActiveRagdolls = Asura_Physics_Ragdoll_System::GetNumberOfActiveRagdolls();

	// if we've hit a number of active ragdoll limit, remove the active ragdoll which will 
	// cause the least visible impact
	while ( uNumberOfActiveRagdolls >= uASURA_PHYSICS_ACTIVE_RAGDOLL_LIMIT )
	{
		bDoIt = RemoveMostObsoleteRagdoll( true, true, uParentGuid );
		if ( bDoIt )
		{
			--uNumberOfActiveRagdolls;
		}
		else
		{
			break;	
		}
	}

	// if numbers are tight, remove an older ragdoll *before* we create the new one.
	u_int uNumRagdolls = s_xHandlerHandleList.GetNumberOfActiveHandles();
	
	while ( uNumRagdolls >= uASURA_PHYSICS_UPPER_RAGDOLL_LIMIT )
	{
		bDoIt = RemoveMostObsoleteRagdoll( true, false, uParentGuid );
		if ( bDoIt )
		{
			--uNumRagdolls;
		}
		else
		{
			break;	
		}
	}

	u_int uNumFading = GetNumberOfFadingRagdolls();
	Asura_Assert(uNumFading <= uNumRagdolls, "more ragdolls fading than are in existence");

	while ( uNumRagdolls - uNumFading >= uASURA_PHYSICS_LOWER_RAGDOLL_LIMIT )
	{
		bDoIt = RemoveMostObsoleteRagdoll( false, false, uParentGuid );
		if ( bDoIt )
		{
			--uNumRagdolls;
		}
		else
		{
			break;	
		}
	}
	

	if ( bDoIt )
	{	
		Asura_Ragdoll_Wrapper* pxRagdollWrapper = new Asura_Ragdoll_Wrapper( uOwnerGuid, iPriority, xRagdollDesc, pxAnimation, xPosition, xOrientation );

		// push the ragdoll around, based on the body's original velocity and the incoming impact if any
		Asura_Physics_Ragdoll* pxRagdoll = pxRagdollWrapper->GetPhysicsRagdollPointer();
		if( pxRagdoll )
		{
			// apply velocity
			if ( xVelocity != Asura_Maths::ZeroVector3 ) 
				pxRagdoll->ApplyImpulseToAll( xVelocity );

			// apply impact impulse
			if ( xImpactStrength != Asura_Maths::ZeroVector3 && xImpactPosition != Asura_Maths::ZeroVector3 )
				pxRagdoll->ApplyImpulseToNearbyNodes( xImpactPosition, xImpactStrength );

			return pxRagdollWrapper->GetWrapperID();
		}
		else
		{
			Asura_Assert( false, "Couldn't create ragdoll, bad things might happen!" );
			delete pxRagdollWrapper;
			return ASURA_INVALID_HANDLE;
		}
	}
	return ASURA_INVALID_HANDLE;
}

void Asura_Ragdoll_Wrapper::Destroy( int iID, bool bDestroyEntity /* = true */ )
{
	if ( Asura_Ragdoll_Wrapper* pxHandler = GetRagdollWrapper( iID ) )
	{
		pxHandler->m_bDestroyClientEntity &= bDestroyEntity;
		pxHandler->m_bDestroyServerEntity &= bDestroyEntity;
		delete pxHandler;
	}
}

Asura_Entity_Renderable* Asura_Ragdoll_Wrapper::GetClientEntity() const
{
	Asura_Entity_Renderable* pxEntity = NULL;

	Asura_Hashed_List_It<Asura_ClientEntity_Ragdoll> xIt(Asura_ClientEntity_Ragdoll::GetList());
	while(!xIt.Done())
	{
		Asura_ClientEntity_Ragdoll* pxRagdollEntity = xIt.GetCurrent();
		xIt.Next();
		if ( pxRagdollEntity )
		{
			if ( pxRagdollEntity->GetRagdollWrapperHandle() == m_iHandlerID )
			{
				pxEntity = pxRagdollEntity;
			}
		}
	}

	// if we can't find our entity in the Asura_ClientEntity_Ragdoll list then look in the renderable list for our owner guid
	if ( pxEntity == NULL )
	{
		pxEntity = Asura_Entity_Renderable::FindEntity( m_uOwnerGuid );
	}

	return pxEntity; 
}

#endif // #ifndef ASURA_PHYSICS_DISABLE_RAGDOLL
