// Asura_ServerEntity_DialogueController.h
// Thomas Young - 17.11.08
// Copyright (C) 2008 Rebellion, All Rights Reserved.

// F I L E   W R A P P E R ///////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef ___ASURA_SERVERENTITY_DIALOGUECONTROLLER_H___
#define ___ASURA_SERVERENTITY_DIALOGUECONTROLLER_H___

#ifdef ASURA_USE_DIALOGUE_SYSTEM

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura_Hash.h"
#include "Core/Asura_HeapArray.h"
#include "Entity/Asura_StaticMessage_System.h"
#include "Entity/Triggers/Asura_ServerEntity_Activatable.h"
#include "Sound/Asura_Dialogue.h"

// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////////

class Asura_Dialogue_Event;

// C L A S S   D E C L A R A T I O N S ///////////////////////////////////////

//_______________________ Asura_DialogueController_Line _________________________

struct Asura_DialogueController_Line
{
	enum { DIALOGUE_CTRL_LINE_CHUNK_VERSION = 1 };

	Asura_Hash_ID	m_uDialogueLine;
	unsigned short	m_usStartMsgBlock;
	unsigned short	m_usEndMsgBlock;
	unsigned char	m_ucWeight;
	bool			m_bActive;
	bool			m_bPlayNext;
	bool			m_bIsValid;

	void SetDefaults ()
	{
		m_uDialogueLine		= ASURA_HASH_ID_UNSET;
		m_usStartMsgBlock	= ASURA_STATICMESSAGE_EMPTYBLOCK;
		m_usEndMsgBlock		= ASURA_STATICMESSAGE_EMPTYBLOCK;
		m_ucWeight			= 1;
		m_bActive			= true;
		m_bPlayNext			= true;
		m_bIsValid			= true;
	}

	void WriteToChunkStream ( Asura_Chunk_Stream& xStream ) const
	{
		int iVersion = DIALOGUE_CTRL_LINE_CHUNK_VERSION;
		
		xStream << iVersion;

		xStream << m_uDialogueLine;
		xStream << m_usStartMsgBlock;
		xStream << m_usEndMsgBlock;
		xStream << m_ucWeight;
		xStream << m_bActive;
		xStream << m_bPlayNext;
	}

	void ReadFromChunkStream( Asura_Chunk_Stream& xStream )
	{
		int iVersion;

		xStream >> iVersion;
		
		Asura_Assert( iVersion <= DIALOGUE_CTRL_LINE_CHUNK_VERSION, "Unknown dialogue controller line chunk, you probably need new code" );
		
		xStream >> m_uDialogueLine;
		xStream >> m_usStartMsgBlock;
		xStream >> m_usEndMsgBlock;
		xStream >> m_ucWeight;
		xStream >> m_bActive;
		xStream >> m_bPlayNext;
	}
};


//_______________________ Asura_ServerEntity_DialogueController _________________________


class Asura_ServerEntity_DialogueController : public Asura_ServerEntity_Activatable
{
	#ifdef ASURA_TOOLS
	friend class Tools_Entity_DialogueController;
	friend class Tools_DialogueLineHash_DropDownList_Dialog;
	friend class Tools_Entity_DialogueTrigger;
	#endif

	public:

		// ----------------- Functions ------------------------------

		Asura_ServerEntity_DialogueController					( );
		Asura_ServerEntity_DialogueController					( Asura_Guid uGuid );
		~Asura_ServerEntity_DialogueController					( );

		static void				CreateFromChunkStream			(Asura_Guid uGuid, Asura_Chunk_Stream& xStream);

		#ifdef ASURA_TOOLS
		// Some helper functions for tools
		static void				HandleEventDeletion				( const Asura_Dialogue_Event* pxEvent );
		
		Asura_Hash_ID			GetEventHashID					( ) const;
		void					SetEventHashID					( const Asura_Hash_ID uEventHash, const bool bCopyAndClearLineMsgs = false );

		bool					IsEveryLinePresent				( ) const;
		bool					IsEveryLineValid				( ) const;
		bool					ContainsLine					( Asura_Hash_ID uLineHash ) const;
		
		virtual void			ApplyChangesForTemplateInstance	( class Tools_EntityTemplateInstance* pxInstance );
		virtual bool			CanBeIncludedInTemplate			( ) const;
		#endif

		void					ValidateLines					( );
		
		static Asura_ServerEntity_DialogueController*							FindEntity	( Asura_Guid uGuid )	{ return s_xList.FindEntity( uGuid ); }
		static const Asura_Hashed_List<Asura_ServerEntity_DialogueController>*	GetList		( )						{ return &s_xList; }

		#ifdef ASURA_TOOLS
		virtual bool SupportsCustomEvents() const { return true; }
		virtual int GetNumCustomEvents() const;
		virtual unsigned short* GetCustomEvent(int iIndex);
		virtual void GetCustomEventName(int iEventIndex, Asura_String& xEventName);
		#endif

	protected :

		// ----------------- Functions ------------------------------

		virtual void	OnTrigger							( );
		virtual bool	IsTriggered							( );

		virtual void	WriteToChunkStream					(Asura_Chunk_Stream& xStream) const;
		virtual void	ReadFromChunkStream					(Asura_Chunk_Stream& xStream);

		virtual	bool	IsFilledFromChunkBuffer				( ) const { return false; }

		virtual bool	HandleMessage						( const Asura_Message* pxMessage );

		virtual u_int	GetNumMappings						( ) const;

		virtual void	PlayNextLine						( );
		virtual void	PlayLine							( const int iLineIndex, bool bPlaySingleLine );
		virtual int		GetNextLineToPlay					( );

		virtual void	BuildCDRandomList					( );

		virtual void	Update								( );

		virtual void	CheckSendFinalMessages				( );

		// ----------------- Enumerations ---------------------------

		enum CONTROLLER_STATE
		{
			STATE_NOT_PLAYING		= 1,
			STATE_WAITING_TO_PLAY	= 2,
			STATE_PLAYING			= 3,
			
			NUM_STATES
		};

		// ----------------- Variables ------------------------------

		Asura_HeapArray<Asura_Dialogue_CharacterMapping>	m_xMappings;
		bool												m_bDeactivateLinesOnUse;
		bool												m_bUseRandomDelay;
		float												m_fMinRandomDelay;
		float												m_fMaxRandomDelay;
		Asura_HeapArray<Asura_DialogueController_Line>		m_xLines;
		
		Asura_Hash_ID										m_uEventHash;
		const Asura_Dialogue_Event*							m_pxEvent;

		int													m_iLastPlayed;
		float												m_fDelayTimer;
		CONTROLLER_STATE									m_eState;

		int*												m_piCDRandomList;
		int													m_iCDRandomListSize;
		int													m_iLastCDRandomPlayed;

		#ifdef ASURA_USE_STREAMING
		Asura_Hash_ID										m_uHandlerID;
		#endif

		bool												m_bShouldApplyPlayProbability;
		float												m_fLastPlayTime;

	private:
	
		void Constructor ();

		typedef Asura_ServerEntity_Activatable PARENT;
		static Asura_Hashed_List<Asura_ServerEntity_DialogueController> s_xList;
};


class Asura_Msg_Dialogue_ToggleLine
{
	public:

		Asura_Hash_ID	m_uLineID;
		bool			m_bEnable;

		void SetDefaults ()
		{
			m_uLineID = ASURA_HASH_ID_UNSET;
			m_bEnable = true;
		}
};

class Asura_Msg_Dialogue_TriggerLine
{
	public:

		Asura_Hash_ID	m_uLineID;

		void SetDefaults ()
		{
			m_uLineID = ASURA_HASH_ID_UNSET;
		}
};



class Asura_Msg_Dialogue_Interrupt
{
	public:
	
		bool m_bImmediate;

		void SetDefaults ()
		{
			m_bImmediate = false;
		}
};


class Asura_Msg_Dialogue_PlayLine_Controller
{
	public:

		Asura_Hash_ID	m_uLineID;
		Asura_Guid		m_uSpeaker;
		
		Asura_Hash_ID	m_uHandlerID;

		int				m_iNotifyFinishedMessageData;
};


class Asura_Msg_Dialogue_StopLine_Controller
{
	public:

		Asura_Hash_ID	m_uLineID;

		Asura_Hash_ID	m_uHandlerID;
};

#endif	// ASURA_USE_DIALOGUE_SYSTEM

#endif // ___ASURA_SERVERENTITY_DIALOGUECONTROLLER_H___

//eof
