/**
*
* \file Asura_Entity_Spawner.cpp
*
* Created by DWE - 27/01/2003 10:57:39
*
* Copyright (C) 2002 Rebellion, All Rights Reserved.
*
* \brief
*
**/

#include "Core/Asura.h"

#include "Entity/Spawners/Asura_Entity_Spawner.h"

#include "AI/Asura_AI_Navigation.h"
#include "Camera/Asura_Camera.h"
#include "ChunkSystem/Asura_Chunk_Macros.h"
#include "Core/Asura_Timers.h"
#include "Entity/PhysicalObject/Asura_ServerEntity_PhysicalObject.h"
#include "Network/Asura_Guid.h"
#include "Network/Asura_Message.h"
#include "Rendering/Asura_Module.h"

#ifdef ASURA_TOOLS
#include "EntityTemplate/Tools_EntityTemplate_System.h"
#endif


Asura_Hashed_List<Asura_Entity_Spawner> Asura_Entity_Spawner::s_xList;	


Asura_Entity_Spawner::Asura_Entity_Spawner(Asura_Guid uGuid)
: Asura_Entity_Server(uGuid)
{
	m_fTimer = 0.0f;
	m_fDelayBetweenSpawns = 1.0f;

	m_bActive = false;

	m_iCurrentAmount = 0;
	m_iMinimumAmount = 0;
	m_iTotalToSpawn = 1;
	m_iTotalSpawned = 0;

	m_xPosition.x = m_xPosition.y = m_xPosition.z = 0.0f;

	s_xList.AddToList(this);
	
}

Asura_Entity_Spawner::~Asura_Entity_Spawner()
{
	s_xList.RemoveFromList(this);
}

/*
void Asura_Entity_Spawner::SetupStorage(const int iTotalToSpawn)
{

}


void Asura_Entity_Spawner::Update()
{
	if(!m_bActive)
	{
		return;
	}

	if(m_iTotalSpawned >= m_iTotalToSpawn)
	{
		return;
	}

	for(int i=0;i<m_iCurrentAmount;++i)
	{
		Asura_ServerEntity_PhysicalObject* pxEntity = Asura_ServerEntity_PhysicalObject::FindEntity(m_puGuidList[i]);
		if(pxEntity)
		{
			if(pxEntity->IsDead())
			{
				// dead entity
				for(int j=i;j<m_iMinimumAmount-1;++j)
				{
					m_puGuidList[j] = m_puGuidList[j+1];
				}
				--m_iCurrentAmount;
			}				
		}
	}
				

	if(m_iCurrentAmount < m_iMinimumAmount)
	{
		m_fTimer -= Asura_Timers::GetGameTimeSlice();

		if(m_fTimer < 0.0f)
		{
			const Asura_Vector_3* pxPosition = GetSpawnPosition();
			if(pxPosition)
			{
				Spawn(*pxPosition);
			}
			m_fTimer = m_fDelayBetweenSpawns;
		}
	}
}


void Asura_Entity_Spawner::Spawn(const Asura_Vector_3& xPosition)
{	

}
*/


const Asura_Vector_3* Asura_Entity_Spawner::GetSpawnPosition()
{
	return &m_xPosition;
}

bool Asura_Entity_Spawner::HandleMessage(const Asura_Message* pxMessage)
{
	if(!pxMessage)
	{
		return false;
	}
	switch(pxMessage->GetMsgID())
	{
		case ASURA_MSG_ACTIVATE:
		{
			m_bActive = true;
			return true;
		}
		case ASURA_MSG_DEACTIVATE:
		{
			m_bActive = false;
			return true;
		}
		default:
			break;
	}

	// pass to parent.
	return Asura_Entity_Server::HandleMessage(pxMessage);
}


void Asura_Entity_Spawner::InitialiseFromChunkBuffer(const char*& pcChunkBuffer)
{
	int iVersion;
	ASURA_CHUNK_EXTRACT(pcChunkBuffer, iVersion, int);


	//read in parent class data if new enough
	if(iVersion >= 2)
	{
		Asura_Entity_Server::InitialiseFromChunkBuffer(pcChunkBuffer);
	}

	if(iVersion == ChunkData::CURRENT_VERSION || iVersion == 1)
	{
		// current version load the Chunk

		ChunkData xData;

		ASURA_CHUNK_EXTRACT(pcChunkBuffer, xData, ChunkData);

		m_fTimer = xData.m_fTimer;
		m_fDelayBetweenSpawns = xData.m_fDelayBetweenSpawns;

		m_bActive = xData.m_bActive;

		m_iCurrentAmount = xData.m_iCurrentAmount;
		m_iMinimumAmount = xData.m_iMinimumAmount;
		
		m_iTotalToSpawn = xData.m_iTotalToSpawn;
		m_iTotalSpawned = xData.m_iTotalSpawned;

		m_xPosition = xData.m_xPosition;
	
	}
	else if(iVersion == 0)
	{
		ChunkData_V0 xData;
		
		ASURA_CHUNK_EXTRACT(pcChunkBuffer, xData, ChunkData_V0);

		m_fTimer = xData.m_fTimer;
		m_fDelayBetweenSpawns = xData.m_fDelayBetweenSpawns;

		m_bActive = xData.m_bActive;

		m_iCurrentAmount = xData.m_iCurrentAmount;
		m_iMinimumAmount = xData.m_iMinimumAmount;
		m_xPosition = xData.m_xPosition;

		m_iTotalToSpawn = 1;
	}
	

}

void Asura_Entity_Spawner::FillInChunkBuffer(char*& pcChunkBuffer)
{
	//write version
	ASURA_CHUNK_FILLANDVERIFY(pcChunkBuffer, ChunkData::CURRENT_VERSION, int);

	//write parent clas
	Asura_Entity_Server::FillInChunkBuffer(pcChunkBuffer);

	//write this class
	ChunkData xData;

	xData.m_bActive = m_bActive;
	ASURA_CHUNK_BITFIELD_VERIFY(xData.m_ucFlags, unsigned char);
	xData.m_fTimer = m_fTimer;
	ASURA_CHUNK_VERIFY(xData.m_fTimer, float);
	xData.m_fDelayBetweenSpawns = m_fDelayBetweenSpawns;
	ASURA_CHUNK_VERIFY(xData.m_fDelayBetweenSpawns, float);
	xData.m_iCurrentAmount = m_iCurrentAmount;
	ASURA_CHUNK_VERIFY(xData.m_iCurrentAmount, int);
	xData.m_iMinimumAmount = m_iMinimumAmount;
	ASURA_CHUNK_VERIFY(xData.m_iMinimumAmount, int);
	xData.m_iTotalToSpawn = m_iTotalToSpawn;
	ASURA_CHUNK_VERIFY(xData.m_iTotalToSpawn, int);
	xData.m_iTotalSpawned = m_iTotalSpawned;
	ASURA_CHUNK_VERIFY(xData.m_iTotalSpawned, int);
	xData.m_xPosition  = m_xPosition;
	ASURA_CHUNK_ARRAY_VERIFY(xData.m_xPosition, float, 3);

    // (MP - 07.04.2006) This isn't used anywhere (anymore) but let's
    // initialise it just to stop compilers (specifically GCC 4) complaining.
    xData.m_iSizeOfEntityStorage = 0;
	
	ASURA_CHUNK_FILL(pcChunkBuffer, xData, ChunkData);
	
}

int Asura_Entity_Spawner::GetChunkBufferSize()
{
	return sizeof(int) + sizeof(ChunkData) + Asura_Entity_Server::GetChunkBufferSize();
}

//-----------------------------------------------------------------------------------
// Asura_Entity_BoundingBox_Spawner
//
// derived version of the spawner that will find a bounding box centred on a Guid
// and then find a position in that BB to spawn from.
//
//-----------------------------------------------------------------------------------

Asura_Entity_BoundingBox_Spawner::Asura_Entity_BoundingBox_Spawner(Asura_Guid uGuid)
: Asura_Entity_Spawner(uGuid)
{
	m_uGuidToCentreOn = ASURA_GUID_UNREGISTERED;

	m_fRange = 30.0f;
	m_fMinRange = 5.0f;
}

Asura_Entity_BoundingBox_Spawner::~Asura_Entity_BoundingBox_Spawner()
{
}

static float   s_fTestWaypointsRange = 5.0f;
static u_short s_usTestWaypointsWPID = ASURA_INVALID_WAYPOINT_ID;

bool Asura_Entity_BoundingBox_Spawner::FindWaypointOutOfSightCallback(u_short usWaypointIndex)
{
	Asura_AI_NavVertex* pxWaypoint = Asura_AI_Navigation_System::GetWaypoint( usWaypointIndex );

	Asura_Vector_3 xFacingDirection;

	if(pxWaypoint)
	{
		for(int i=0;i<Asura_Camera::GetNumberOfCameras();++i)
		{

			Asura_Camera::SetCameraIndex(i);

			if(!Asura_Camera::IsCameraActive()) {continue;}

			Asura_Camera::GetForwardVector(xFacingDirection);

			Asura_Vector_3 xOffset = pxWaypoint->m_xPosition - *Asura_Camera::GetPosition();
			xOffset.y = 0.0f;

			float fDistance = xOffset.Magnitude();

			if(fDistance > s_fTestWaypointsRange)
			{
				xOffset.Normalise();
			}
			else
			{
				// continue the search.
				return true;
			}

			if(xOffset*xFacingDirection < 0.0f)
			{
				// waypoint is behind us.
				// and far enough away.
				// stop the search.
				s_usTestWaypointsWPID = usWaypointIndex;
				return false;
			}
		}
	}

	return true;
}

const Asura_Vector_3* Asura_Entity_BoundingBox_Spawner::GetSpawnPosition()
{
	// First initialise a bounding box based on the position of our target guid.
	Asura_ServerEntity_PhysicalObject* pxObject = Asura_ServerEntity_PhysicalObject::FindEntity(m_uGuidToCentreOn);
	
	if(!pxObject)
	{
		return NULL;
	}

	Asura_Vector_3 xPosition = pxObject->GetPosition();

	Asura_Bounding_Box xBoundingBox;
	
	xBoundingBox.MinX = xBoundingBox.MaxX = xPosition.x;
	xBoundingBox.MinY = xBoundingBox.MaxY = xPosition.y;
	xBoundingBox.MinZ = xBoundingBox.MaxZ = xPosition.z;

	xBoundingBox.MinX -= m_fRange;
	xBoundingBox.MinY -= m_fRange;
	xBoundingBox.MinZ -= m_fRange;

	xBoundingBox.MaxX += m_fRange;
	xBoundingBox.MaxY += m_fRange;
	xBoundingBox.MaxZ += m_fRange;

	// Set the static data we need to search.
	s_fTestWaypointsRange = 5.0f;
	s_usTestWaypointsWPID = ASURA_INVALID_WAYPOINT_ID;
	// then search.
	Asura_AI_Navigation_System::SearchWaypoints(xBoundingBox, FindWaypointOutOfSightCallback);

	Asura_AI_NavVertex* pxWaypoint = 0;

	if(s_usTestWaypointsWPID != ASURA_INVALID_WAYPOINT_ID)
	{
		pxWaypoint = Asura_AI_Navigation_System::GetWaypoint( s_usTestWaypointsWPID );
	}
	if(pxWaypoint)
	{
		// nasty search of physical objects to check that there is nothing to spawn onto.
		Asura_Hashed_List_It<Asura_ServerEntity_PhysicalObject> xIterator( Asura_ServerEntity_PhysicalObject::GetList() );
		Asura_ServerEntity_PhysicalObject* pxEntity = 0;

		while(!xIterator.Done())
		{
			pxEntity = xIterator.GetCurrent();
			xIterator.Next();

			if(pxEntity && !pxEntity->IsDead())
			{
				if(pxWaypoint->m_xPosition.ApproximatelyEquals(pxEntity->GetPosition(), 4.0f))
				{
					return NULL;
				}
			}
		}

		return &pxWaypoint->m_xPosition;
	}
	else
	{
//		Asura_Assert(0,"No waypoint found by spawner."); 
		return NULL;
	}
}

void Asura_Entity_BoundingBox_Spawner::InitialiseFromChunkBuffer(const char*& pcChunkBuffer)
{
	int iVersion;
	ASURA_CHUNK_EXTRACT(pcChunkBuffer, iVersion, int);

	if(iVersion == ChunkData::CURRENT_VERSION)
	{
		ChunkData xData;

		ASURA_CHUNK_EXTRACT(pcChunkBuffer, xData, ChunkData);
		
	//	m_xBoundingBox = xData.m_xBoundingBox;
		m_fRange = xData.m_fRange;
		m_fMinRange = xData.m_fMinRange;
		m_uGuidToCentreOn = xData.m_uGuidToCentreOn;

	}
	else if(iVersion == 0)
	{
		ChunkData_V0 xData;

		ASURA_CHUNK_EXTRACT(pcChunkBuffer, xData, ChunkData_V0);
		
		m_fRange = xData.m_fRange;

		/// DWE - 24/07/2003 12:12:24 - default value
		m_fMinRange = 5.0f;
		m_uGuidToCentreOn = xData.m_uGuidToCentreOn;
	}

	Asura_Entity_Spawner::InitialiseFromChunkBuffer(pcChunkBuffer);
}

void Asura_Entity_BoundingBox_Spawner::FillInChunkBuffer(char*& pcChunkBuffer)
{

	ASURA_CHUNK_FILLANDVERIFY(pcChunkBuffer, ChunkData::CURRENT_VERSION, int);

	ChunkData xData;
//	xData.m_xBoundingBox = m_xBoundingBox;
	xData.m_fRange = m_fRange;
	ASURA_CHUNK_VERIFY(xData.m_fRange, float);
	xData.m_fMinRange = m_fMinRange;
	ASURA_CHUNK_VERIFY(xData.m_fMinRange, float);

	xData.m_uGuidToCentreOn = m_uGuidToCentreOn;
	ASURA_CHUNK_VERIFY(xData.m_uGuidToCentreOn, Asura_Guid);

	ASURA_CHUNK_FILL(pcChunkBuffer, xData, ChunkData);

	Asura_Entity_Spawner::FillInChunkBuffer(pcChunkBuffer);
}

int Asura_Entity_BoundingBox_Spawner::GetChunkBufferSize()
{
	return sizeof(int) + sizeof(ChunkData) + Asura_Entity_Spawner::GetChunkBufferSize();
}

//////////////////////////////////////////////////////////////////////////
// Asura_Entity_Quantum_Spawner
//
// Has multiple spawn points and spawns randomly at one
//////////////////////////////////////////////////////////////////////////

Asura_Entity_Quantum_Spawner::Asura_Entity_Quantum_Spawner(Asura_Guid uGuid)
: Asura_Entity_Spawner(uGuid)
{
	m_uNumberOfSpawnPoints = 0;
	m_pxPositionList = 0;
	m_piLastKnownModuleList = 0;
	m_bUseModuleLinking = true;
}

Asura_Entity_Quantum_Spawner::~Asura_Entity_Quantum_Spawner()
{
	if(m_pxPositionList)
	{
		delete [] m_pxPositionList;
	}
	if(m_piLastKnownModuleList)
	{
		delete [] m_piLastKnownModuleList;
	}
}

const Asura_Vector_3* Asura_Entity_Quantum_Spawner::GetSpawnPosition()
{
/// DWE - 20/02/2003 12:07:26 - Doesn't check for visibility yet


	if(m_uNumberOfSpawnPoints)
	{
		if(m_pxPositionList)
		{
			
			unsigned int uIndex = Asura_Random::GetIntU() %  m_uNumberOfSpawnPoints;

			if(uIndex < m_uNumberOfSpawnPoints)
			{
				if(!m_bUseModuleLinking || (m_bUseModuleLinking && !Asura_Module_System::IsPointInVisibleModule(m_pxPositionList[uIndex], &m_piLastKnownModuleList[uIndex])))
				{
					// nasty search of physical objects to check that there is nothing to spawn onto.
					Asura_Hashed_List_It<Asura_ServerEntity_PhysicalObject> xIterator( Asura_ServerEntity_PhysicalObject::GetList() );
					Asura_ServerEntity_PhysicalObject* pxEntity = 0;

					while(!xIterator.Done())
					{
						pxEntity = xIterator.GetCurrent();
						xIterator.Next();

						if(pxEntity && !pxEntity->IsDead())
						{
							if(m_pxPositionList[uIndex].ApproximatelyEquals(pxEntity->GetPosition(), 4.0f))
							{
								return NULL;
							}
						}
					}
					return &m_pxPositionList[uIndex];
				}
			}
		}
	}

	return NULL;
}

void Asura_Entity_Quantum_Spawner::InitialiseFromChunkBuffer(const char*& pcChunkBuffer)
{
	int iVersion;

	ASURA_CHUNK_EXTRACT(pcChunkBuffer, iVersion, int);

	
	if(iVersion == ChunkData::CURRENT_VERSION)
	{
		ChunkData xData;

		ASURA_CHUNK_EXTRACT(pcChunkBuffer, xData, ChunkData);
	
		m_uNumberOfSpawnPoints = xData.m_uNumberOfSpawnPoints;
		m_bUseModuleLinking = xData.m_bUseModuleLinking;
	}
	else if( iVersion == 0)
	{
		ChunkData_V0 xData;

		ASURA_CHUNK_EXTRACT(pcChunkBuffer, xData, ChunkData_V0);
	
		m_uNumberOfSpawnPoints = xData.m_uNumberOfSpawnPoints;
		m_bUseModuleLinking = true;

	}
	else
	{
		Asura_Assert(0,"Unhandled version");
	}

	if(m_pxPositionList)
	{
		delete m_pxPositionList;
	}
	m_pxPositionList = new Asura_Vector_3[m_uNumberOfSpawnPoints];
	
	for(unsigned int i=0; i < m_uNumberOfSpawnPoints; ++i)
	{
		ASURA_CHUNK_EXTRACT(pcChunkBuffer, m_pxPositionList[i], Asura_Vector_3);
	}

	if(m_piLastKnownModuleList)
	{
		delete [] m_piLastKnownModuleList;
	}
	m_piLastKnownModuleList = new int[m_uNumberOfSpawnPoints];
	for(unsigned int i=0; i < m_uNumberOfSpawnPoints; ++i)
	{
		ASURA_CHUNK_EXTRACT(pcChunkBuffer, m_piLastKnownModuleList[i], int);
	}


	Asura_Entity_Spawner::InitialiseFromChunkBuffer(pcChunkBuffer);
}

void Asura_Entity_Quantum_Spawner::FillInChunkBuffer(char*& pcChunkBuffer)
{
	ChunkData xData;

	ASURA_CHUNK_FILLANDVERIFY(pcChunkBuffer, ChunkData::CURRENT_VERSION,int);

	xData.m_uNumberOfSpawnPoints = m_uNumberOfSpawnPoints;
	ASURA_CHUNK_VERIFY(xData.m_uNumberOfSpawnPoints, unsigned int);
	xData.m_bUseModuleLinking = m_bUseModuleLinking;
	ASURA_CHUNK_BITFIELD_VERIFY(xData.m_uFlags, unsigned int);

	ASURA_CHUNK_FILL(pcChunkBuffer, xData, ChunkData);

	for(unsigned int i=0;i<m_uNumberOfSpawnPoints;++i)
	{
		ASURA_CHUNK_ARRAY_FILLANDVERIFY(pcChunkBuffer, m_pxPositionList[i], Asura_Vector_3, float, 3);
	}

	for(unsigned int i=0;i<m_uNumberOfSpawnPoints;++i)
	{
		ASURA_CHUNK_FILLANDVERIFY(pcChunkBuffer, m_piLastKnownModuleList[i], int);
	}

	Asura_Entity_Spawner::FillInChunkBuffer(pcChunkBuffer);
}

int Asura_Entity_Quantum_Spawner::GetChunkBufferSize()
{
	return sizeof(int) + sizeof(ChunkData) + sizeof(Asura_Vector_3)*m_uNumberOfSpawnPoints + sizeof(int)*m_uNumberOfSpawnPoints + Asura_Entity_Spawner::GetChunkBufferSize();
}


void Asura_Entity_Quantum_Spawner::SetNumberOfSpawnPoints(unsigned int uValue)
{
	if(uValue > m_uNumberOfSpawnPoints)
	{
		// we need more space.
		Asura_Vector_3* pxNewPositionList = new Asura_Vector_3[uValue];
		if(!pxNewPositionList)
		{
			return;
		}

		// create a list for the lastknownmodules 	
		if(m_piLastKnownModuleList)
		{
			delete [] m_piLastKnownModuleList;
		}
		m_piLastKnownModuleList = new int[uValue];
		
		for(unsigned int i=0;i<uValue;++i)
		{
			pxNewPositionList[i] = m_xPosition;
			m_piLastKnownModuleList[i] = Asura_Module_System::GetModuleIndexFromPosition(m_xPosition);
		}

		// copy the old data 
		for(unsigned int i=0;i<m_uNumberOfSpawnPoints;++i)
		{
			pxNewPositionList[i] = m_pxPositionList[i];
		}

		// then delete the old storage
		
		if(m_pxPositionList)
		{
			delete [] m_pxPositionList;
		}

	

		m_pxPositionList = pxNewPositionList;
		m_uNumberOfSpawnPoints = uValue;
	}
	else
	{
		m_uNumberOfSpawnPoints = uValue;
	}
}

#ifdef ASURA_TOOLS
void Asura_Entity_Spawner::ApplyChangesForTemplateInstance(Tools_EntityTemplateInstance* pxInstance)
{
	Asura_Entity_Server::ApplyChangesForTemplateInstance(pxInstance);

	m_xPosition = pxInstance->TransformPosition(m_xPosition);
}
#endif // ASURA_TOOLS

