
///////////////////////////////////////////////////////////////////////////////
//
// File   : Asura_ServerEntity_TeleportDestination.h
// Author : Andy Tate
// Date   : 2-Mar-2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __ASURA_SERVERENTITY_TELEPORTDESTINATION_H__
#define __ASURA_SERVERENTITY_TELEPORTDESTINATION_H__

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Entity/Spawners/Asura_ServerEntity_StartPoint.h"


////////////////////////////////////////////////////////////////////////////////
// Forward class declarations...
////////////////////////////////////////////////////////////////////////////////

class Tools_EntityTemplateInstance;


////////////////////////////////////////////////////////////////////////////////
// Class declaration...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Class        : Asura_ServerEntity_TeleportDestination
// Author       : Andy Tate
// Date         : 2-Mar-2007
//
// Description  : Class to handle the teleport destination - essentially its a
//				  start-point but with possibly streaming data.
////////////////////////////////////////////////////////////////////////////////

class Asura_ServerEntity_TeleportDestination : public Asura_Entity_Server
{
	friend class Tools_Entity_TeleportDestination;

public:
	// Public functions...
	static Asura_ServerEntity_TeleportDestination*	Create	(const Asura_Guid uGuid);

	// Overridden functions...

	// override of HasClientEntities() - we don't have any.
	virtual bool HasClientEntities() const { return false; }

	virtual	bool	IsFilledFromChunkBuffer	(void) const	{ return(false); }
	virtual bool 	CanBeSavedToChunk		(void) const	{ return(true); }

	virtual	void	ReadFromChunkStream		(Asura_Chunk_Stream& xStream);
	virtual	void	WriteToChunkStream		(Asura_Chunk_Stream& xStream) const;

	// Accessor functions...
	void	TeleportTo	(const Asura_Vector_3& xPosition, const Asura_Quat& xOrientation)	{ m_xPosition = xPosition; m_xOrientation = xOrientation; }

	const Asura_Vector_3&	GetPosition		(void) const	{ return(m_xPosition); }
	const Asura_Quat&		GetOrientation	(void) const	{ return(m_xOrientation); }

	// Static accessor functions...
	static Asura_ServerEntity_TeleportDestination*							FindEntity	(const Asura_Guid uGuid)	{ return(s_xList.FindEntity(uGuid)); }
	static const Asura_Hashed_List<Asura_ServerEntity_TeleportDestination>*	GetList		(void)						{ return(&s_xList); }

#if defined(ASURA_TOOLS)
	// Template functions...
	virtual	void	ApplyChangesForTemplateInstance	(Tools_EntityTemplateInstance* pxInstance);
	virtual	bool	CanBeIncludedInTemplate			(void) const	{ return(true); }
#endif	// defined(ASURA_TOOLS)


	////////////////////////////////////////////////////////////////////////////////
	// Constant declarations...
	////////////////////////////////////////////////////////////////////////////////

	static const u_int uTELEPORTDEST_CHUNK_VERSION = 1;


protected:
	// Protected variables...

	// Protected functions...
	Asura_ServerEntity_TeleportDestination          (const Asura_Guid uGuid);
	virtual ~Asura_ServerEntity_TeleportDestination ();


private:
	// Private typedefs...
	typedef Asura_Entity_Server PARENT;

	// Private variables...
	Asura_Vector_3	m_xPosition;		// The position to which we're teleported.
	Asura_Quat		m_xOrientation;		// The orientation we're to face once teleported.

	// Static list variables...
	static Asura_Hashed_List<Asura_ServerEntity_TeleportDestination>	s_xList;

	// Private functions...
	Asura_ServerEntity_TeleportDestination  (void);
	Asura_ServerEntity_TeleportDestination  (const Asura_ServerEntity_TeleportDestination& xOther);

	Asura_ServerEntity_TeleportDestination&	operator=	(const Asura_ServerEntity_TeleportDestination& xOther);

};   // End of class 'Asura_ServerEntity_TeleportDestination'.


#endif   // ifndef __ASURA_SERVERENTITY_TELEPORTDESTINATION_H__


/// End of file. ///
