//
// Asura_ClientEntity_DebugWaypointTrigger.cpp
//
// Created by JAT - 02 April 2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//

// This file's header.
#include "Core/Asura.h"

#ifdef ASURA_USE_DEBUG_MESSAGE_TRIGGER

//My Header
#include "entity/Triggers/Asura_ClientEntity_DebugWaypointTrigger.h"

//Asura headers
#include "Asura_ServerEntity_DebugWaypointTrigger.h"
#include "Animation/Asura_Animation.h"
#include "Camera/Asura_Camera.h"
#include "ChunkSystem/Asura_Chunk_Loaders.h"
#include "Core/Asura_Timers.h"
#include "Core/Memory/Asura_Mem.h"
#include "Entity/Asura_Entity_Classification.h"
#include "Entity/PhysicalObject/Asura_ServerEntity_PhysicalObject.h"
#include "Entity/Triggers/Asura_ServerEntity_DebugWaypointTrigger.h"
#include "Interface/Asura_CommandConsole.h"
#include "Rendering/Asura_Module.h"
#include "Network/Asura_Message.h"
#include "Shapes/Asura_Shape_System.h"

Asura_Hashed_List<Asura_ClientEntity_DebugWaypointTrigger> Asura_ClientEntity_DebugWaypointTrigger::s_xList(2);
bool Asura_ClientEntity_DebugWaypointTrigger::s_bRenderDebugWaypointTriggerShapes;

Asura_ClientEntity_DebugWaypointTrigger::Asura_ClientEntity_DebugWaypointTrigger()
{
	Classification = AsuraEntityClass_DebugWaypointTrigger;

	m_uFlags = 0;

	Asura_DebugWaypointTrigger_Messages::Reset();
	
	s_xList.AddToList(this);	
	AddToUpdateableClientList(this);	
}

Asura_ClientEntity_DebugWaypointTrigger::~Asura_ClientEntity_DebugWaypointTrigger()
{
	s_xList.RemoveFromList(this);
	// JAT 14/12/07 - this is done in the Asura_Entity_Client destructor now - stops an assert
	// RemoveFromUpdateableClientList(this);
}

void Asura_ClientEntity_DebugWaypointTrigger::Render(void)
{
	//If we are ready to render, not hidden and the console variable is set...
	if (m_bReady && !m_xServerData.m_bHidden && Asura_ServerEntity_DebugWaypointTrigger::GetRenderTriggerShapes())
	{			
		m_xShapeInstance.Render();
	}
}

// This is a replacement for a surreal bit of code which compared the distance to the sphere with the maximum length of the text string.
// To maintain the previous behaviour, I've put this in at the same value, but really it either shouldn't exist or should be an extra tools option.
DEBUGVAR float dbg_fLookAtMaxDistance = 40.0f;

void Asura_ClientEntity_DebugWaypointTrigger::Update()
{
	Asura_Vector_3 xScreenPos;
	u_int uColour = 0;
	bool bShowTitle = false;
	bool bShowText = false;
	// The radius and look-at tests combine, but if you're using the look-at test that will always pass if you're inside the trigger radius, so you never need to test both.
	bool bPassedRadiusOrLookAtTest = true;
	bool bPassedLOSTest = true;

	//_____ Check the look-at test for display ____
	if (m_xServerData.m_bUseLookAtTest)
	{
		bPassedRadiusOrLookAtTest = false;
		Asura_Vector_3 xRay1 = *Asura_Camera::GetPosition();
		Asura_Vector_3 xRay2;
		Asura_Camera::GetForwardVector(xRay2);
		xRay2.Normalise();

		//Check intersection
		float fDistance;
		if(Asura_Maths::RayIntersectsSphere(xRay1, xRay2, m_xServerData.m_xPosition, m_xServerData.m_fTriggerRadius, &fDistance))
		{
			if(fDistance < dbg_fLookAtMaxDistance)
			{
				bPassedRadiusOrLookAtTest = true;
			}
		}
	}
	//______ Check the radius for text display ____
	else if ( m_xServerData.m_bUseRadiusTriggering )
	{
		const Asura_Vector_3 xDistance = m_xServerData.m_xPosition - *Asura_Camera::GetPosition();
		bPassedRadiusOrLookAtTest = xDistance.MagnitudeSquared() < ( m_xServerData.m_fTriggerRadius * m_xServerData.m_fTriggerRadius );
	}

	// And finally the LOS test
	if (m_xServerData.m_bUseLOSOcclusion)
	{
#if defined(ASURA_USE_NEW_VISIBILITY_SYSTEM)
		bPassedLOSTest	= m_xVisibilityQuery.IsPositionVisibleFromCurrentCamera(m_xServerData.m_xPosition);
#else
		bPassedLOSTest	= Asura_Module_System::IsPointInVisibleModule(m_xServerData.m_xPosition);
#endif	//defined(ASURA_USE_NEW_VISIBILITY_SYSTEM)

		if(bPassedLOSTest)
		{
			const Asura_Vector_3&	xCameraPosition	= *Asura_Camera::GetPosition();
			Asura_Vector_3			xDirection(m_xServerData.m_xPosition - xCameraPosition);
			// Bump the y offset a little, so we're not firing straight into the ground (the tools shape's pivot is at its base, so most will have been placed flush with the ground).
			xDirection.y -= 0.5f;
			float fDistance = xDirection.Normalise();
			Asura_LOS_InputData xInput(*Asura_Camera::GetPosition(), xDirection, ASURA_GUID_UNREGISTERED, fDistance, true, AsuraPolygonFlag_DontRender | AsuraPolygonFlag_IgnoreLOS);
			const Asura_LOS_OutputData* pxOutput = Asura_LOS::TestAgainstEnvironment(&xInput);

			bPassedLOSTest = !pxOutput->m_bHitEnvironment;
		}
	}

	//____ Decide whether or not to display text ____
	const bool bAllTestsPassed = bPassedRadiusOrLookAtTest && bPassedLOSTest;
	if (bAllTestsPassed && !m_bTestsPassedLastFrame)
	{
		m_fTimer = 0.0f;
	}

	if(Asura_ServerEntity_DebugWaypointTrigger::GetRenderTriggerShapes())
	{
		bool bTimerPassed = (m_xServerData.m_fTimeToDisplayString <= 0.0f) || (m_fTimer < m_xServerData.m_fTimeToDisplayString);

		bShowTitle = m_xServerData.m_bAlwaysShowTitle || (bAllTestsPassed && bTimerPassed);
		bShowText = bAllTestsPassed && bTimerPassed;

		if (bAllTestsPassed)
		{
			m_fTimer += Asura_Timers::GetGameTimeSlice();
		}
	}
	

	//____ buffer up the text render ____

	if(bShowText || bShowTitle)
	{
		xScreenPos = m_xServerData.m_xPosition;
		xScreenPos.SetY(xScreenPos.GetY() - 0.5f);	// Move Text to middle of tool representation.
		Asura_Camera::ProjectWorldCoordsToScreenSpace(xScreenPos, true);

		uColour = 0;
		Asura_RGBA xColour( m_xServerData.m_fR/255.0f, m_xServerData.m_fG/255.0f, m_xServerData.m_fB/255.0f, 1.0f );
		uColour = xColour.ToUIntARGB();
	}
	if(bShowTitle && (xScreenPos.GetZ() >= 0.0f))
	{
		Asura_DebugWaypointTrigger_Messages::BufferDebugMessage( xScreenPos, -2.0f, uColour, m_xServerData.m_szTitle);
	}
	if(bShowText && (xScreenPos.GetZ() >= 0.0f))
	{
		for ( int i=0; i<8; i++ )
		{
			Asura_DebugWaypointTrigger_Messages::BufferDebugMessage(xScreenPos, static_cast<float>(i), uColour, m_xServerData.m_szText[i]);
		}		
	}

	m_bTestsPassedLastFrame = bAllTestsPassed;
}

void Asura_ClientEntity_DebugWaypointTrigger::SetPositionAndOrientation( const Asura_Vector_3& xPosition, const Asura_Matrix_3x3& xOrientation )
{
	m_xServerData.m_xPosition		= xPosition;
	m_xServerData.m_xOrientation	= xOrientation;
	m_xShapeInstance.SetNewPosAndOri(xPosition, xOrientation);
}

void Asura_ClientEntity_DebugWaypointTrigger::SetSkinAndAnimID(Asura_Hash_ID uShapeID, Asura_Hash_ID uAnimID, int iAnimFlags, float fAnimTimer)
{	
	m_xShapeInstance.SetSkinAndAnimID(uShapeID, uAnimID, iAnimFlags, fAnimTimer);
}

void Asura_ClientEntity_DebugWaypointTrigger::SetShapeID(Asura_Hash_ID uShapeID)
{
	m_xShapeInstance.SetShape(uShapeID);
}

bool Asura_ClientEntity_DebugWaypointTrigger::HandleMessage( const Asura_Message* pxMsg )
{
	switch( pxMsg->GetMsgID() )
	{
	case ASURA_MSG_CLIENTCREATE:
		ProcessCreateMessage( static_cast<const Asura_Message_DebugWaypoint_Create*>(pxMsg->GetData()), pxMsg->GetSentTime() );
		return true;
	case ASURA_MSG_CLIENTUPDATE:
		ProcessClientUpdate( static_cast<const Asura_Message_DebugWaypoint_ClientUpdate*>(pxMsg->GetData()), pxMsg->GetSentTime() );
		return true;
	default:
		break;
	}
	return PARENT::HandleMessage(pxMsg);
}

void Asura_ClientEntity_DebugWaypointTrigger::ProcessCreateMessage (const Asura_Message_DebugWaypoint_Create* pxData, float fTime)
{
	//make this our local copy
	Asura_Mem::Copy(&m_xServerData,pxData,sizeof(Asura_Message_DebugWaypoint_Create));

	//Shape and animation (if this is a shape, just set the shape, otherwise set the skin and anim)
	if(!m_xServerData.m_bSkin)
	{
		SetShapeID(m_xServerData.m_uShapeID);
	}
	else
	{
		SetSkinAndAnimID(m_xServerData.m_uShapeID, m_xServerData.m_uAnimID, ASURA_SHAPE_INSTANCE_NO_SHADOW | AsuraAnimationFlag_Loop, Asura_Timers::GetLocalUnscaledTime());
	}	
			
	//Position & Orientation
	SetPositionAndOrientation(m_xServerData.m_xPosition, m_xServerData.m_xOrientation);

	SetEntityActive(m_xServerData.m_bActive);

	//we can render now
	m_bReady = true;
}

void Asura_ClientEntity_DebugWaypointTrigger::ProcessClientUpdate (const Asura_Message_DebugWaypoint_ClientUpdate* pxData, float fTime)
{
	if (pxData)
	{
		//make this our local copy
		m_xServerData.m_uFlags = pxData->GetFlags(); 
	}
	else
	{
		Asura_Assert( false, "received message with no data" ); 
	}
}


Asura_ClientEntity_DebugWaypointTrigger* Asura_ClientEntity_DebugWaypointTrigger::Create( const Asura_Message* pxMsg )
{
	Asura_ClientEntity_DebugWaypointTrigger* pxEntity = new Asura_ClientEntity_DebugWaypointTrigger();
	pxEntity->HandleMessage(pxMsg); 
	return pxEntity; 
}

void Asura_ClientEntity_DebugWaypointTrigger::SendRequestCreateMessage(Asura_Guid uGuid)
{
	if (uGuid < ASURA_GUID_CLIENT_START)
	{
		Asura_Assertf(0, ("Invalid guid for create request (%u)", uGuid));
		return;
	}

	Asura_Message* pxRequestMsg = Asura_Network::CreateMessage(ASURA_MSG_CLIENT_REQUESTCREATEMSG, 0, AsuraEntityClass_DebugWaypointTrigger);
	if ( pxRequestMsg )
	{
		pxRequestMsg->SetToGuid( uGuid );
		pxRequestMsg->SetFromGuid( Asura_Network::GetClientID() );
		Asura_Network::Send( pxRequestMsg );
	}
}

#endif // #ifdef ASURA_USE_DEBUG_MESSAGE_TRIGGER

//eof
