//////////////////////////////////////////////////////////////////////
// Asura_ClientEntity_VolumeTrigger.h								//
// SJL 21/11/07														//
// Copyright (C) 2007 Rebellion, All Rights Reserved.			    //
//																	//
// A volume trigger with trigger and untrigger functionality that   //
// will check entities contained in project-specific lists.			//	
//////////////////////////////////////////////////////////////////////


#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __ASURA_CLIENTENTITY_VOLUMETRIGGER_H__
#define __ASURA_CLIENTENTITY_VOLUMETRIGGER_H__

#include "Entity/Triggers/Asura_ClientEntity_Trigger.h"
#include "Volume/Asura_VolumeGroup.h"

class Asura_ClientEntity_VolumeTrigger : public Asura_ClientEntity_Trigger
{
	public:

		// Constructor / Destructor
		Asura_ClientEntity_VolumeTrigger(Asura_Guid uGuid);
		~Asura_ClientEntity_VolumeTrigger();

		// Various functions that pass straight to the volume group, all pretty self explanatory
		virtual bool Contains(const Asura_Vector_3& xPosition) const { return m_pxVolumeGroup->Contains(xPosition); }
		u_int GetNumberOfVolumes() const { return m_pxVolumeGroup->GetNumberOfVolumes(); }
		Asura_Quat* GetVolumeOrient(u_int uIndex) const { return m_pxVolumeGroup->GetVolumeOrient(uIndex); }
		Asura_Volume* GetVolume(u_int uIndex) { return m_pxVolumeGroup->GetVolume(uIndex); }
		void AddVolume(Asura_Volume* pxVolume) { m_pxVolumeGroup->AddVolume(pxVolume); }
		void RemoveVolume(Asura_Volume* pxVolume) { m_pxVolumeGroup->RemoveAndDestroyVolume(pxVolume); }
		void RemoveVolume(u_int uIndex) { m_pxVolumeGroup->RemoveAndDestroyVolume(uIndex); }
		void DisplaceVolumes(Asura_Vector_3& xOffset) { m_pxVolumeGroup->DisplaceVolumes(xOffset); }

		// Accessors for 'specified guid', which does nothing at this level, but its assumed that it will
		// be generally useful for project implementations.
		void SetSpecificGuid(Asura_Guid uGuid) { m_uSpecificGuid = uGuid; }
		Asura_Guid GetSpecificGuid() const { return m_uSpecificGuid; }

		virtual void Update();

		virtual bool IsChunkBufferServerOnly() const {return (!m_bTriggersOnClient); }
		virtual	void	ReadFromChunkStream			(Asura_Chunk_Stream& xStream);

		static void* Create( Asura_Guid uGuid ) { return NULL; }
	protected:

		virtual bool IsTriggered(void);
		virtual bool IsUnTriggered(void);
		virtual void OnUnTrigger();
		virtual bool IsVolumeOccupied();

		virtual void Reset(void){} 

	

		Asura_VolumeGroup* m_pxVolumeGroup; // Container class for Asura_Volumes
		Asura_Guid m_uSpecificGuid; //Specific Guid to check for use by derived classes.

		union
		{
			struct
			{
				unsigned int m_bHasAlreadyTriggered		: 1,
							 m_bTriggerOnce				: 1,//Volume trigger is deactivated when un-triggered (so it only gets triggered once per activation)
							 m_bTriggerOnNotOccupied	: 1,//Becomes triggered upon becoming unoccupied instead
							 m_bTriggersOnClient		: 1,//Client entities check and trigger
							 SpareBits					: 28;
			};
			unsigned int m_uFlags;
		};

	private:
		typedef Asura_ClientEntity_Trigger PARENT;
};

#endif//__ASURA_CLIENTENTITY_VOLUMETRIGGER_H__

