///////////////////////////////////////////////////////////////////////////////
//
// File: Asura_ServerEntity_CameraVolume.h
//
// Created by Martin G. Harris (4/9/2006)
// 
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
// Activatable volume that has a set of project specific camera properties associated with it.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef __ASURA_SERVER_ENTITY_CAMERA_VOLUME_H__
#define __ASURA_SERVER_ENTITY_CAMERA_VOLUME_H__

#ifdef ASURA_USE_CAMERA_VOLUME

// Includes
#include "Entity/Triggers/Asura_ServerEntity_Activatable.h"

// Forward declarations
class Asura_Message;

/////////////////////////////////////
// Asura_ServerEntity_CameraVolume //
/////////////////////////////////////

// Project specific extensions of Asura_Server(Client)Entity_CameraVolume should
// still be classified as AsuraEntityClass_CameraVolume. A call to the derived class's
// static Create() method is then associated with the AsuraEntityClass_CameraVolume
// classification in project code (At least thats how we did it on  BF3). NDB - 181207
class Asura_ServerEntity_CameraVolume : public Asura_ServerEntity_Activatable
{
	typedef Asura_ServerEntity_Activatable PARENT;

public:
	virtual ~Asura_ServerEntity_CameraVolume();

	virtual bool HandleMessage(const Asura_Message* pxMessage);

	virtual void Update(); // Unused

	virtual void OnActivation()		{ SetActiveFlagAndUpdateClient(true); }
	virtual void OnDeactivation()	{ SetActiveFlagAndUpdateClient(false); }

	// Chunk stream functions
	virtual	void ReadFromChunkStream( Asura_Chunk_Stream& xStream );
	virtual	void WriteToChunkStream( Asura_Chunk_Stream& xStream ) const;

	// Chunk buffer functions
	virtual	int  GetChunkBufferSize();
	virtual	void FillInChunkBuffer(char*& pcChunkBuffer);
	virtual	void InitialiseFromChunkBuffer(const char*& pcChunkBuffer);

protected:
	Asura_ServerEntity_CameraVolume(bool bActive = true);
	Asura_ServerEntity_CameraVolume(Asura_Guid uGuid);

	void SetActiveFlagAndUpdateClient(bool bActive);

private:
	void SetActive (const bool bActive); // Don't use this function in the parent class.

	// private static data
	static int s_iCurrentChunkVersion; // chunk version used in *ChunkStream && *ChunkBuffer methods
};
#endif //#ifdef ASURA_USE_CAMERA_VOLUME

#endif //__ASURA_SERVER_ENTITY_CAMERA_VOLUME_H__
