//////////////////////////////////////////////////////////////////////////////
//
// Asura_ServerEntity_DialogueTrigger.cpp
//
// Copyright (c) 2006 Rebellion. All rights reserved.
//
// Description:
//
//     code for entity which triggers dialogue using the new dialogue events
//     system
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header.
#include "Entity/Triggers/Asura_ServerEntity_DialogueTrigger.h"

#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "Entity/Asura_Entity_Classification.h"
#include "Entity/Asura_StaticMessage_System.h"
#include "Entity/Asura_Entity_Event.h"
#include "Network/Asura_Message.h"
#include "Network/Asura_Network.h"
#include "Sound/Asura_Dialogue.h"
#include "Sound/Asura_Dialogue_System.h"
#if defined(ASURA_USE_STREAMING)
#include "Streaming/Asura_Streaming_System.h"
#endif

#ifdef ASURA_TOOLS
#include "EntityTemplate/Tools_EntityTemplate_System.h"
#endif



#ifdef ASURA_USE_DIALOGUE_SYSTEM

#ifdef ASURA_TOOLS
Asura_Hashed_List<Asura_ServerEntity_DialogueTrigger> Asura_ServerEntity_DialogueTrigger::s_xList(5);
#endif


static const int iASURA_DIALOGUE_TRIGGER_CHUNK_VERSION = 1;

Asura_ServerEntity_DialogueTrigger::Asura_ServerEntity_DialogueTrigger()
: Asura_ServerEntity_Activatable()	
{
	Classification = AsuraEntityClass_DialogueTrigger;

	#ifdef ASURA_TOOLS
	s_xList.AddToList(this);
	#endif


	#ifdef ASURA_USE_STREAMING
	//The streaming zone handlers should be set correctly at this point , so store the ID
	//for use when the entity gets triggered. RAR 13/11/2006
	m_uHandlerID = Asura_Dialogue_Storage::GetCurrentHandler().GetHandlerID();
	#endif

	m_uEventHash = ASURA_HASH_ID_UNSET;
	m_pxEvent = NULL;
	m_pxMappings = new Asura_Dialogue_CharacterMappingList;
	m_usMessageBlock = ASURA_STATICMESSAGE_EMPTYBLOCK;
}

Asura_ServerEntity_DialogueTrigger::Asura_ServerEntity_DialogueTrigger(Asura_Guid uGuid)
: Asura_ServerEntity_Activatable(uGuid)	
{
	Classification = AsuraEntityClass_DialogueTrigger;

#ifdef ASURA_TOOLS
	s_xList.AddToList(this);
#endif


#ifdef ASURA_USE_STREAMING
	//The streaming zone handlers should be set correctly at this point , so store the ID
	//for use when the entity gets triggered. RAR 13/11/2006
	m_uHandlerID = Asura_Dialogue_Storage::GetCurrentHandler().GetHandlerID();
#endif

	m_uEventHash = ASURA_HASH_ID_UNSET;
	m_pxEvent = NULL;
	m_pxMappings = new Asura_Dialogue_CharacterMappingList;
	m_usMessageBlock = ASURA_STATICMESSAGE_EMPTYBLOCK;

}

Asura_ServerEntity_DialogueTrigger::Asura_ServerEntity_DialogueTrigger( Asura_Guid uGuid, const char*& pcChunkBuffer )
: Asura_ServerEntity_Activatable(uGuid, pcChunkBuffer)	
{
	Classification = AsuraEntityClass_DialogueTrigger;

	#ifdef ASURA_TOOLS
	s_xList.AddToList(this);
	#endif


	#ifdef ASURA_USE_STREAMING
	//The streaming zone handlers should be set correctly at this point , so store the ID
	//for use when the entity gets triggered. RAR 13/11/2006
	m_uHandlerID = Asura_Dialogue_Storage::GetCurrentHandler().GetHandlerID();
	#endif

	//check the version
	int iVersion;
	ASURA_CHUNK_EXTRACT( pcChunkBuffer, iVersion, int);
	Asura_Assert( iVersion <= iASURA_DIALOGUE_TRIGGER_CHUNK_VERSION, "Unknown chunk version");

	//read in the rest
	ASURA_CHUNK_EXTRACT( pcChunkBuffer, m_uEventHash, Asura_Hash_ID );

	m_pxEvent = Asura_Dialogue_System::FindEvent( m_uEventHash );
	Asura_Assertf(m_pxEvent, ("Dialogue Trigger '%d' cannot find its dialogue event (Event ID '%x')",uGuid,m_uEventHash));

	u_int uNumMappings;
	ASURA_CHUNK_EXTRACT( pcChunkBuffer, uNumMappings, u_int );
	m_pxMappings = new Asura_Dialogue_CharacterMappingList;
	m_pxMappings->m_xMappings.SetSize(uNumMappings);

	for (u_int u = 0; u < uNumMappings; ++u)
	{
		ASURA_CHUNK_EXTRACT( pcChunkBuffer, m_pxMappings->m_xMappings[u].m_uCharacterHashID, Asura_Hash_ID );
		ASURA_CHUNK_EXTRACT( pcChunkBuffer, m_pxMappings->m_xMappings[u].m_uCharacterGuid, Asura_Guid );

		#if defined(ASURA_USE_STREAMING) && !defined(ASURA_TOOLS)
		m_pxMappings->m_xMappings[u].m_uCharacterGuid = Asura_Streaming_System::MakeCurrentMasterZoneGuid(m_pxMappings->m_xMappings[u].m_uCharacterGuid);
		#endif
	}
	if(iVersion >= 1)
	{
		ASURA_CHUNK_EXTRACT(pcChunkBuffer, m_usMessageBlock, unsigned short);
		pcChunkBuffer += 2;//skip 2 byte padding
	}
	else
	{
		m_usMessageBlock = ASURA_STATICMESSAGE_EMPTYBLOCK;
	}


	// This entity doesn't need to update.
	SetNeverUpdates(true);
}

Asura_ServerEntity_DialogueTrigger::~Asura_ServerEntity_DialogueTrigger()
{
	#ifdef ASURA_TOOLS
	s_xList.RemoveFromList(this);
	#endif
	
	delete m_pxMappings;
}

void Asura_ServerEntity_DialogueTrigger::CreateFromChunkBuffer(Asura_Guid uGuid, const char*& pcChunkBuffer)
{
#ifdef TEST_ENTITY_CHUNK_STREAM

	TestDialogueTriggerStream(uGuid, pcChunkBuffer);
#endif
	new Asura_ServerEntity_DialogueTrigger(uGuid, pcChunkBuffer);
}

void Asura_ServerEntity_DialogueTrigger::SetEventHashID(Asura_Hash_ID uEventHash)
{
	m_uEventHash = uEventHash;
	m_pxEvent = m_uEventHash ? Asura_Dialogue_System::FindEvent(m_uEventHash) : NULL;
}

void Asura_ServerEntity_DialogueTrigger::WriteToChunkStream (Asura_Chunk_Stream& xStream) const
{
	Asura_ServerEntity_Activatable::WriteToChunkStream( xStream );

	int iVersion = iASURA_DIALOGUE_TRIGGER_CHUNK_VERSION;
	xStream << iVersion;

	xStream << m_uEventHash;

	u_int uNumMappings = GetNumMappings();
	xStream << uNumMappings;

	for (u_int u = 0; u < uNumMappings; ++u)
	{
		xStream << m_pxMappings->m_xMappings[u].m_uCharacterHashID;
		xStream << m_pxMappings->m_xMappings[u].m_uCharacterGuid;
	}

	xStream << m_usMessageBlock;
	xStream.Write4BytePadding();
}

void Asura_ServerEntity_DialogueTrigger::ReadFromChunkStream (Asura_Chunk_Stream& xStream)
{
	Asura_ServerEntity_Activatable::ReadFromChunkStream( xStream );

	int iVersion;
	xStream >> iVersion;
	Asura_Assert( iVersion <= iASURA_DIALOGUE_TRIGGER_CHUNK_VERSION, "Unknown Asura_ServerEntity_DialogueTrigger chunk version");

	xStream >> m_uEventHash;
	m_pxEvent = Asura_Dialogue_System::FindEvent( m_uEventHash );
	Asura_Assertf(m_pxEvent, ("Dialogue Trigger cannot find its dialogue event (Event ID '%x')",m_uEventHash));

	u_int uNumMappings;
	xStream >> uNumMappings;
	m_pxMappings->m_xMappings.SetSize(uNumMappings);

	for (u_int u = 0; u < uNumMappings; ++u)
	{
		xStream >> m_pxMappings->m_xMappings[u].m_uCharacterHashID;
		xStream >> m_pxMappings->m_xMappings[u].m_uCharacterGuid;
	}

	if(iVersion >= 1)
	{
		xStream >> m_usMessageBlock;
		xStream.Skip4BytePadding();
	}
	else
	{
		m_usMessageBlock = ASURA_STATICMESSAGE_EMPTYBLOCK;
	}

	// This entity doesn't need to update.
	SetNeverUpdates(true);
}

Asura_Hash_ID Asura_ServerEntity_DialogueTrigger::GetEventHashID() const
{
//	return m_pxEvent ? m_pxEvent->GetHashID() : ASURA_HASH_ID_UNSET;
	return m_uEventHash;
}

u_int Asura_ServerEntity_DialogueTrigger::GetNumMappings() const
{
	return m_pxMappings ? m_pxMappings->m_xMappings.Size() : 0;
}

void Asura_ServerEntity_DialogueTrigger::OnTrigger()
{
	Asura_ServerEntity_Activatable::OnTrigger();

	// send a message to all client machines...
	u_int uNumMappings = GetNumMappings();
	Asura_Message* pxMsg = Asura_Network::CreateMessage( ASURA_MSG_DIALOGUE_TRIGGER, 
		sizeof( Asura_DialogueTrigger_MessageHeader ) + (uNumMappings * sizeof(Asura_Dialogue_CharacterMapping)));
	Asura_DialogueTrigger_MessageHeader* pxMessageData = reinterpret_cast
		<Asura_DialogueTrigger_MessageHeader*>( pxMsg->GetData() );

	pxMessageData->m_uEventID = GetEventHashID();
	pxMessageData->m_uNumMappings = uNumMappings;
	#ifdef ASURA_USE_STREAMING
	pxMessageData->m_uHandlerID = m_uHandlerID;
	#endif
	pxMessageData->m_usMessageBlock = m_usMessageBlock;

	Asura_Dialogue_CharacterMapping* pxMapping = reinterpret_cast<Asura_Dialogue_CharacterMapping*>(pxMessageData + 1);
	for (u_int u = 0; u < uNumMappings; ++u)
	{
		pxMapping[u] = m_pxMappings->m_xMappings[u];
	}
	pxMsg->To = ASURA_GUID_ALL_CLIENTMACHINES;
	pxMsg->From = GetGuid();

	Asura_Network::Send(pxMsg);
}

bool Asura_ServerEntity_DialogueTrigger::HandleMessage(const Asura_Message* pxMessage)
{
    if( IsDestroyed() ) { return true; } // ignore all further messages?

	if (pxMessage)
	{
		switch (pxMessage->GetMsgID())
		{
			case ASURA_MSG_NOTIFYFINISHED :
			{
				//Have finished playing the dialogue on the client ,so trigger any On Finished messages
				SendMessagesForEvent(ASURA_ENTITY_EVENT_ONFINISHED);
				return true;
			}
			case ASURA_MSG_ACTIVATE:
			{
				Asura_ServerEntity_Activatable::HandleMessage(pxMessage);
				SetNeverUpdates(true); // The evils of server entity activatable will turn this on in response
				// to an activate message which isnt desired.
				return true;
			}
		}
	}

	return Asura_ServerEntity_Activatable::HandleMessage(pxMessage);
}

#ifdef ASURA_TOOLS

bool Asura_ServerEntity_DialogueTrigger::CanBeIncludedInTemplate() const 
{
	return Classification == AsuraEntityClass_DialogueTrigger;
}

void Asura_ServerEntity_DialogueTrigger::ApplyChangesForTemplateInstance(class Tools_EntityTemplateInstance* pxInstance)
{
	Asura_ServerEntity_Activatable::ApplyChangesForTemplateInstance(pxInstance);

	//convert the message block
	m_usMessageBlock = pxInstance->ConvertMessageBlock(m_usMessageBlock);

	//convert the guids in the mappings list
	u_int uNumMappings = GetNumMappings();
	for (u_int u = 0; u < uNumMappings; ++u)
	{
		m_pxMappings->m_xMappings[u].m_uCharacterGuid = pxInstance->ConvertGuid(m_pxMappings->m_xMappings[u].m_uCharacterGuid);
	}
}

void Asura_ServerEntity_DialogueTrigger::HandleEventDeletion(const Asura_Dialogue_Event* pxEvent)
{

	for(Asura_Hashed_List_It<Asura_ServerEntity_DialogueTrigger> xIt(&s_xList);!xIt.Done();xIt.Next())
	{
		Asura_ServerEntity_DialogueTrigger* pxEntity = xIt.GetCurrent();

		if(pxEntity->m_pxEvent == pxEvent)
		{
			pxEntity->m_pxEvent = 0;
		}
	}
}

void Asura_ServerEntity_DialogueTrigger::TestDialogueTriggerStream(Asura_Guid uGuid, const char*& pcChunkBuffer)
{
	const char* const pcOriginalChunkBufferStart = pcChunkBuffer;
	const char* pcOriginalChunkBuffer = pcOriginalChunkBufferStart;
	
	Asura_Chunk_ChunkBuffer_Stream xOriginalReadStream(pcOriginalChunkBuffer);	
	Asura_ServerEntity_DialogueTrigger* pxDiaTrig = new Asura_ServerEntity_DialogueTrigger(uGuid);	
	pxDiaTrig->ReadFromChunkStream(xOriginalReadStream);	
	const u_int uOriginalChunkSize = pcOriginalChunkBuffer - pcOriginalChunkBufferStart;
	TestStreamWrite(pxDiaTrig, pcOriginalChunkBufferStart, uOriginalChunkSize);
	delete pxDiaTrig;
}

#endif // ASURA_TOOLS

#endif //ASURA_USE_DIALOGUE_SYSTEM
