//////////////////////////////////////////////////////////////////////////////
//
// Asura_ServerEntity_LookAtTrigger.h
//
// Created by Anthony Gregson - 22/04/04
//
// Copyright (c) 2004 Rebellion. All rights reserved.
//
// Description:
//
//     Trigger which is triggered by being looked at in a set way.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef ___ASURA_SERVERENTITY_LOOKATTRIGGER_H___
#define ___ASURA_SERVERENTITY_LOOKATTRIGGER_H___ 1

#include "Entity/Triggers/Asura_ServerEntity_Trigger.h"
#ifdef ASURA_TOOLS
#include "Entity/Asura_Entity_Classification.h"
#endif

// forward declarations
class Asura_NetMsg;

// AJG - 30/10/07 - Can't remember why I did this like this, looks pretty hacky to me.
const float fLOOKAT_TRIGGER_ONE_OVER_STANDARD_FOV = 1.0f / (0.5f * Asura_Maths::Pi);

const float fLOOKAT_TRIGGER_UPDATE_TIME = 0.2f;

enum ASURA_LOOKATTRIGGER_TRIGGERINGTYPES
{
	ASURA_LOOKATTRIGGER_TYPE_ALLPLAYERS = 1, // Default
	ASURA_LOOKATTRIGGER_TYPE_ALLOTHERACTORS = 2,
	ASURA_LOOKATTRIGGER_TYPE_ALLACTORS = ASURA_LOOKATTRIGGER_TYPE_ALLPLAYERS & ASURA_LOOKATTRIGGER_TYPE_ALLOTHERACTORS,
	ASURA_LOOKATTRIGGER_TYPE_SPECIFICPLAYER = 4,

	ASURA_LOOKATTRIGGER_NUM_BITS_FOR_TRIGGERING_TYPES = 3	// increase this is max value goes above 7
};

class Asura_ServerEntity_LookAtTrigger : public Asura_ServerEntity_Trigger
{
public:
	Asura_ServerEntity_LookAtTrigger(bool bActive = false);
	// Constructor when loading
	Asura_ServerEntity_LookAtTrigger(Asura_Guid uGuid);
	// Old load function, deprecated
	Asura_ServerEntity_LookAtTrigger(Asura_Guid uGuid, const char*& pcChunkBuffer);

	virtual ~Asura_ServerEntity_LookAtTrigger() {};

	// override of HasClientEntities() - most triggers don't, but we do.
	virtual bool HasClientEntities() const { return true; }

	virtual void Update();

	bool HandleMessage(const Asura_Message* pxMessage);

	const Asura_Vector_3& GetPosition() const { return m_xPosition; }

	#ifdef ASURA_TOOLS
	virtual void ApplyChangesForTemplateInstance(class Tools_EntityTemplateInstance* pxInstance);
	virtual bool CanBeIncludedInTemplate() const {return Classification == AsuraEntityClass_LookAtTrigger;}
	#endif

	#ifdef ASURA_SUPPORT_ENTITYWATCH
	virtual void DisplayEntityWatchText();
	#endif

	virtual bool GetHasCreateMessage() const { return true; }
	virtual void MakeCreateMessage(Asura_Guid uRequestingClient, bool bSendGuaranteed = false);

	static void CreateFromChunkStream(Asura_Guid uGuid, Asura_Chunk_Stream& xStream);

protected:

	virtual bool IsTriggered(void);
	virtual bool IsUnTriggered(void);

	virtual void OnActivation();
	virtual void OnDeactivation();

	// Virtual function overides for saving
	virtual void WriteToChunkStream(Asura_Chunk_Stream& xStream) const;
	virtual void ReadFromChunkStream(Asura_Chunk_Stream& xStream);
	virtual	bool IsFilledFromChunkBuffer(void) const { return false; }

	// These are now deprecated as we've switched over to streams
	virtual int GetChunkBufferSize();
	virtual void FillInChunkBuffer(char*& pcChunkBuffer);

private:
	// Control parameters

	float m_fMaxDistance;
	float m_fMinDistance;

	// Arguably, the next four floats could be condensed into two (and/or equivalently in the chunk data), but because they scale very differently we'll leave then separate for now
	float m_fCosMaxAngle; // Tools will use angles, but we'll store them ready-cosined for speed in-game
	float m_fCosMinAngle;
	float m_fScreenEdgeDeadzone; // Offset into screen that doesn't count for looking at the trigger
	float m_fScreenCentreDeadzone; // Offset from centre of screen that doesn't count for looking at the trigger

	float m_fTime; // amount of time delay before triggering (default 0)

	float m_fInwardConeCosMaxAngle; // if 1, assume we're not using the inward cone, as it's unlikely we'll ever want something that only triggers if you're on an exact line
	// TODO - Should this have a minimum too?

	u_int m_uTriggeringTypes; // Flags from the ASURA_LOOKATTRIGGER_TRIGGERINGTYPES enumeration, which a subclass could also add to if there are more types of entity that can look, or there's a different way of calculating the look.
	int m_iSpecifiedPlayer; // There may be a better way of doing this by putting all options in the enum, but this allows any number of players
	int m_iNumberRequired;

	Asura_Vector_3 m_xPosition;
	Asura_Vector_3 m_xDirection;

	// AJG - 30/10/07 - Replaced all the standalone bools with a bitfield.
	//					The ordering matches the order of the bitfield in the old chunkbuffer (which was being more sensible about its data storage than the class itself), to avoid needing to bump the version number.
	union
	{
		u_int m_uLookAtFlags;
		struct
		{
			u_int m_bUseScreenArea				: 1;
			u_int m_bUseLOS						: 1; // Would be better if can be used sparingly (so we'll default to false)
			u_int m_bResetTimeOnEarlyLookAway	: 1; // if true, the time remaining returns to full once the other settings are not met again, otherwise it stays where it is until someone looks again
			u_int m_bAdjustForFOV				: 1;
			u_int m_bTriggered					: 1;
			u_int m_bTriggerOnNotLooking		: 1;// If true then the trigger is based on the number of players not looking at it
		};
	};

	// Current data
	float m_fTimeRemaining; // Time left to go before being triggered
	int m_iNumberCurrentlyTriggering;
	float m_fUpdateTimer;

	// Temporary store of number looking that have been found on the clients, needed as long as the clients have to do the camera tests
	int m_iClientNumberCurrentlyTriggering;

	friend class Tools_Entity_LookAtTrigger_Object;
	typedef Asura_ServerEntity_Trigger PARENT;
};

class Asura_Msg_LookAtTrigger_ClientCreate
{
public:
	float m_fMaxDistance;
	float m_fMinDistance;
	float m_fMaxAngleOrEdgeDeadzone;	// Which of the two options this is is dictated by the m_bUseScreenArea bool
	float m_fMinAngleOrCentreDeadzone;	// Ditto
	float m_fInwardConeCosMaxAngle;
	u_int m_uTriggeringTypes;
	int m_iSpecifiedPlayer;
	Asura_Vector_3 m_xPosition;
	Asura_Vector_3 m_xDirection;
	union
	{
		u_int m_uClientLookAtFlags;
		struct
		{
			u_int m_bUseScreenArea			: 1;
			u_int m_bUseLOS					: 1;
			u_int m_bTriggerOnNotLooking	: 1;
			u_int m_bAdjustForFOV			: 1;
		};
	};

#ifdef ASURA_NETMSG_PACK_ALL_DATA
	void RegisterFields(Asura_NetMsg* pxNetMsg) const;
	static u_char GetNumFields() { return 10; }
#endif
};

class Asura_Msg_LookAtTrigger_ClientCountChange
{
public:
	int m_iPrevious;
	int m_iCurrent;
};

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

#endif // #ifndef ___ASURA_SERVERENTITY_LOOKATTRIGGER_H___
