/**
*
* \file Asura_ServerEntity_MultiDamageVolume.h
*
* Created by Andrew Weinkove - 06/02/2004
*
* Copyright (C) 2004 Rebellion, All Rights Reserved.
*
* A damage volume with multiple, orientable bounding boxes and optional dynamics.
* Based heavily on the implementation of advanced volume triggers.
* Because the volumes can have dynamics, this class checks against bounding boxes, 
* NOT points.
*
**/
#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif

#ifndef __ASURA_SERVERENTITY_MULTIDAMAGEVOLUME_H__
#define __ASURA_SERVERENTITY_MULTIDAMAGEVOLUME_H__

#include "Entity/Triggers/Asura_ServerEntity_AdvancedVolumeTrigger.h" // For Asura_OrientedVolume.
#include "Core/Asura_Maths.h"

#ifdef ASURA_TOOLS
#include "Entity/Asura_Entity_Classification.h"
#endif

class Asura_VolumeGroup;



//////////////////////////////////////////
// Asura_ServerEntity_MultiDamageVolume //
//////////////////////////////////////////
class Asura_ServerEntity_MultiDamageVolume : public Asura_ServerEntity_Activatable
{
	public:
		Asura_ServerEntity_MultiDamageVolume(Asura_Guid uGuid);
		Asura_ServerEntity_MultiDamageVolume(Asura_Bounding_Box& xBox, bool bInitiallyActive = false);
		//constructor from load
		Asura_ServerEntity_MultiDamageVolume(Asura_Guid uGuid,const char*& pcChunkBuffer);
		~Asura_ServerEntity_MultiDamageVolume();
        struct ChunkData
        {
            enum { CURRENT_VERSION = 1 };
        };

		int        m_iListFlags;	//Bit-field of lists to use - limits max lists to 32.
		
//		int GetNumberOfVolumes() const {return m_iNumberOfVolumes;}
//		const Asura_OrientedDamageVolume& GetVolume(int iIndex) const { return m_pxVolumes[iIndex]; }

		//checks to see if an AABB overlaps with any of the volumes in this trigger
		bool Overlaps(const Asura_Bounding_Box& xBox);

		//checks to see if a point lies in any of the volumes associated with this trigger
		bool Contains(const Asura_Vector_3& xPos) const;

		#ifdef ASURA_TOOLS
		virtual void ApplyChangesForTemplateInstance(class Tools_EntityTemplateInstance* pxInstance);
		virtual bool CanBeIncludedInTemplate() const {return Classification == AsuraEntityClass_MultiDamageVolume;}
		#endif

				
	protected:
		void DeleteDynamicObjects();
		void SynchroniseDynamicObjects();
		virtual void OnActivation();
		virtual void OnDeactivation();
		virtual void Update();
		virtual void ApplyDamage(); // (AW - 06.02.2004) - Override this function for functionality.

		virtual	void ReadFromChunkStream( Asura_Chunk_Stream& xStream );
		virtual	void WriteToChunkStream( Asura_Chunk_Stream& xStream ) const;

		//virtual function overides for saving
		int GetChunkBufferSize();
		void FillInChunkBuffer(char*& pcChunkBuffer);
		virtual void MakeClientUpdateMessage(void) {};

		Asura_VolumeGroup* m_pxVolumeGroup;

//		class Asura_Physics_Object **m_ppxDynamicObjects;

//		class Asura_Collision_Box **m_ppxCollisionBoxes;
		float m_fDamageRate; // It is up to the project ApplyDamage function to decide how often to apply this.

		union
		{
			struct
			{
				unsigned int m_bUsesDynamicsWhenActive : 1, //RAR 21/02/2008 Not implemented
							 m_bVolumesBlockWaypointCreation : 1, //RAR 21/02/2008 Not implemented
							 m_bInstantDeath : 1,
							// (AW - 09.02.2004) - This flag stops us doing multiple tests when dynamics
							// are on. We should only need one test each time this is enabled.
							 m_bDoneTestWhenUsingDynamics : 1, 
							 SpareBits				: 28;
			};
			unsigned int m_uFlags;
		};
		friend class Tools_Entity_MultiDamageVolume;
		friend class Tools_MultiDamageVolume_VolumeEdit_Controller;
		static const int s_iMaxNumberOfLists;

		typedef Asura_ServerEntity_Activatable PARENT;
};


#endif //__ASURA_SERVERENTITY_MULTIDAMAGEVOLUME
