//
// Asura_ServerEntity_SplitterBlock.h
//
// Created by Richard Rice - 18/09/2002
// 
// Copyright (C) 2002 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef __ASURA_SERVERENTITY_SPLITTERBLOCK_H__
#define __ASURA_SERVERENTITY_SPLITTERBLOCK_H__

#include "Asura_ServerEntity_Activatable.h"
#ifdef ASURA_TOOLS
#include "Entity/Asura_Entity_Classification.h"
#endif


struct Asura_SplitterBlockOutput
{
	void SetDefaultValues();

	unsigned short m_usMessageBlock;
	union
	{
		unsigned short m_usFlags;
		struct 
		{
			unsigned short m_bActive : 1,
						#ifdef ASURA_TOOLS
						   m_usSpareBits : 14,
						   m_bJunk : 1;//not used anymore , but may not be zero in chunks
						#else
						   m_usSpareBits : 15;
						#endif
		};
	};
	unsigned int m_uWeight;
	#ifdef ASURA_TOOLS
	Asura_String m_xName;//optional name for output
	#endif
};

#define Asura_SplitterBlockFlag_Random 0x00000001 //If set , pick one output at random from the active ones
#define Asura_SplitterBlockFlag_DeactivateOutputWhenUsed 0x00000002 //Whenever messages are sent from an output, deactivate it
#define Asura_SplitterBlockFlag_UseRandomDelay 0x00000004 // Delay output messages according to min and max random time.
#define Asura_SplitterBlockFlag_CycleOutputs 0x00000008 //One output used each time splitter is triggered. Cycles through the outputs in order
#define Asura_SplitterBlockFlag_SendRandomDelayImmediately 0x00000100 //Random delay messages get posted into static message system immediately , but wil only be recieved after the delay.

class Asura_ServerEntity_SplitterBlock : public Asura_ServerEntity_Activatable
{
	typedef Asura_ServerEntity_Activatable PARENT;

	public :
		Asura_ServerEntity_SplitterBlock();

		//constructor from load
		Asura_ServerEntity_SplitterBlock(Asura_Guid uGuid);
		virtual ~Asura_ServerEntity_SplitterBlock();
	
		virtual void Update();
		virtual bool HandleMessage(const Asura_Message* pxMessage);

		unsigned int GetNumberOfOutputs() const {return m_uNumberOfOutputs;}
		const Asura_SplitterBlockOutput* GetOutputs() { return m_pxOutputs; }

		#ifdef ASURA_TOOLS
		virtual void ApplyChangesForTemplateInstance(class Tools_EntityTemplateInstance* pxInstance);
		virtual bool CanBeIncludedInTemplate() const {return Classification == AsuraEntityClass_SplitterBlock;}
		#endif


		#ifdef ASURA_TOOLS
		static bool GetMessageDescription(const struct Asura_StaticMessage* pxMessage,Asura_String& xOutputDesc);
		#endif

		#ifdef ASURA_SUPPORT_ENTITYWATCH
		virtual void DisplayEntityWatchText();
		#endif

		bool HasTimerArray() const {return m_pfOutputTimers!=0;}
		bool NeedsTimerArray() const {return m_uNumberOfOutputs && (m_uSplitterFlags & Asura_SplitterBlockFlag_UseRandomDelay) && !(m_uSplitterFlags & Asura_SplitterBlockFlag_SendRandomDelayImmediately);}

		#ifdef ASURA_TOOLS
		void UpdateOutputTimerArray();
		#endif

		static void CreateFromChunkStream(Asura_Guid uGuid, Asura_Chunk_Stream& xStream);

		#ifdef ASURA_TOOLS
		virtual bool SupportsCustomEvents() const { return true; }
		virtual int GetNumCustomEvents() const { return m_uNumberOfOutputs; }
		virtual unsigned short* GetCustomEvent(int iIndex) { return &m_pxOutputs[iIndex].m_usMessageBlock; }
		virtual void GetCustomEventName(int iEventIndex, Asura_String& xEventName);
		#endif

	protected :

		void OnTrigger();
		void OnDeactivation();

		void TriggerOutput(int iIndex, bool bDelayOutput);

		//virtual function overides for saving/loading
		virtual	bool IsFilledFromChunkBuffer(void) const { return false; }
		virtual void ReadFromChunkStream(Asura_Chunk_Stream& xStream);
		virtual	void WriteToChunkStream(Asura_Chunk_Stream& xStream) const;
		// deprecated functions
		int GetChunkBufferSize();
		void FillInChunkBuffer(char*& pcChunkBuffer);
		#ifdef ASURA_TOOLS
		int GetToolsDataSize();
		#endif


		float m_fOutputMinDelayTime;
		float m_fOutputMaxDelayTime;
		unsigned int m_uSplitterFlags;

		unsigned int m_uNumberOfOutputs;
		Asura_SplitterBlockOutput* m_pxOutputs;
		float *m_pfOutputTimers;

		unsigned int m_uNextOutputToTrigger;//for cycling splitter blocks

		#ifdef ASURA_TOOLS
		public:
			static Asura_ServerEntity_SplitterBlock* FindEntity(Asura_Guid uGuid)
			{
				return s_xList.FindEntity(uGuid);
			}

			static const Asura_Hashed_List<Asura_ServerEntity_SplitterBlock>* GetList()
			{
				return &s_xList;
			}


		private:
			static Asura_Hashed_List<Asura_ServerEntity_SplitterBlock> s_xList;
		#endif // ASURA_TOOLS


		friend class Tools_Entity_SplitterBlock_Object;

};

//This message is saved out to static message system , so care needs to be taken about altering it
//Used by activate , deactivate and trigger output
struct Asura_Message_SplitterBlock_Output 
{
	unsigned m_uOutputIndex;	
};



#endif //__ASURA_SERVERENTITY_SPLITTERBLOCK_H__
