//////////////////////////////////////////////////////////////////////
// Asura_ServerEntity_VolumeTrigger.h								//
// SJL 6/11/07														//
// Copyright (C) 2007 Rebellion, All Rights Reserved.			    //
//																	//
// A volume trigger with trigger and untrigger functionality that   //
// will check entities contained in project-specific lists.			//	
//////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif

#ifndef __ASURA_SERVERENTITY_VOLUMETRIGGER_H__
#define __ASURA_SERVERENTITY_VOLUMETRIGGER_H__
#include "Entity/Asura_Entity_Classification.h"
#include "Entity/Triggers/Asura_ServerEntity_Trigger.h"
#include "Volume/Asura_VolumeGroup.h"

const int iASURA_VOLUME_TRIGGER_CHUNK_VERSION_SERVER = 1;

#ifdef ASURA_TOOLS
#ifdef ASURA_SUPPORT_COMMAND_CONSOLE
#ifndef ASURA_NO_DEBUG_OUTPUT
#define ASURA_VOLUME_TRIGGER_DEBUG_RENDERING
#endif
#endif
#endif

class Asura_ServerEntity_VolumeTrigger : public Asura_ServerEntity_Trigger
{
		friend class Tools_Entity_VolumeTrigger;
		friend class Tools_VolumeTrigger_VolumeEdit_Controller;

	public:

		// Constructor / Destructor
		Asura_ServerEntity_VolumeTrigger(Asura_Guid uGuid);
		~Asura_ServerEntity_VolumeTrigger();
     	
		// Various functions that pass straight to the volume group, all pretty self explanatory
		virtual bool Contains(const Asura_Vector_3& xPosition) const { return m_pxVolumeGroup->Contains(xPosition); }
		u_int GetNumberOfVolumes() const { return m_pxVolumeGroup->GetNumberOfVolumes(); }
		Asura_Quat* GetVolumeOrient(u_int uIndex) const { return m_pxVolumeGroup->GetVolumeOrient(uIndex); }
		Asura_Volume* GetVolume(u_int uIndex) { return m_pxVolumeGroup->GetVolume(uIndex); }
        const Asura_VolumeGroup* GetVolumeGroup( void ) const { return m_pxVolumeGroup; }
		void AddVolume(Asura_Volume* pxVolume) { m_pxVolumeGroup->AddVolume(pxVolume); }
		void RemoveVolume(Asura_Volume* pxVolume) { m_pxVolumeGroup->RemoveAndDestroyVolume(pxVolume); }
		void RemoveVolume(u_int uIndex) { m_pxVolumeGroup->RemoveAndDestroyVolume(uIndex); }
		void DisplaceVolumes(Asura_Vector_3& xOffset) { m_pxVolumeGroup->DisplaceVolumes(xOffset); }
		void RotateVolumesAbout(const Asura_Vector_3& xCentre, const Asura_Matrix_3x3& xOri) { m_pxVolumeGroup->RotateVolumesAbout(xCentre, xOri); }
		void SetPhysicalObjectParentGuid (const Asura_Guid uGuid) { m_uParentObjectGuid = uGuid; }
		Asura_Guid GetPhysicalObjectParentGuid () const { return m_uParentObjectGuid; }

		static const Asura_Hashed_List<Asura_ServerEntity_VolumeTrigger>* GetList() { return &s_xList; }
		static Asura_ServerEntity_VolumeTrigger* FindEntity(Asura_Guid uGuid) { return s_xList.FindEntity(uGuid); }

		// Accessors for 'specified guid', which does nothing at this level, but its assumed that it will
		// be generally useful for project implementations.
		void SetSpecificGuid(Asura_Guid uGuid) { m_uSpecificGuid = uGuid; }
		Asura_Guid GetSpecificGuid() const { return m_uSpecificGuid; }


		// Function used to work out how far a point is away from this entity...
		float CalculateSquaredDistanceTo(const Asura_Vector_3& xPosition) const
		{
			if(m_pxVolumeGroup == NULL)
			{
				return(Asura_Maths::LargeFloat);
			}
			else
			{
				Asura_Bounding_Box	xBB;

				m_pxVolumeGroup->GetEncompassingAABB(xBB);
				return(xBB.GetSqdMinDistFromPoint(xPosition));
			}
		}

		#ifdef ASURA_TOOLS
		virtual void ApplyChangesForTemplateInstance(class Tools_EntityTemplateInstance* pxInstance);
		virtual bool CanBeIncludedInTemplate() const {return (Classification == AsuraEntityClass_VolumeTrigger);}
		#endif

		#ifdef ASURA_SUPPORT_ENTITYWATCH
		virtual void DisplayEntityWatchText();
		#endif

		virtual void Update();

		#ifdef ASURA_VOLUME_TRIGGER_DEBUG_RENDERING
		static bool s_bRenderVolumeTriggers;
		static unsigned int s_uConsoleVarReferenceCount;
		#endif

	protected:

		virtual bool IsTriggered(void);
		virtual bool IsUnTriggered(void);
		virtual void OnUnTrigger();
		virtual bool IsVolumeOccupied() const;
	
		// hmm..
		virtual void OnActivation();
		virtual void OnDeactivation();
		virtual void OnDestruction();


		virtual void Reset(void){} 

		

		virtual bool    IsChunkBufferServerOnly() const {return (!m_bTriggersOnClient); } 
		virtual bool    IsFilledFromChunkBuffer( void ) const { return false; }
		virtual	void	ReadFromChunkStream			(Asura_Chunk_Stream& xStream);
		virtual	void	WriteToChunkStream			(Asura_Chunk_Stream& xStream) const;

		Asura_VolumeGroup* m_pxVolumeGroup; // Container class for Asura_Volumes
		Asura_Guid m_uSpecificGuid; //Specific Guid to check for use by derived classes.

		union
		{
			struct
			{
				unsigned int m_bHasAlreadyTriggered		: 1,
							 m_bTriggerOnce				: 1,//Volume trigger is deactivated when un-triggered (so it only gets triggered once per activation)
							 m_bTriggerOnNotOccupied	: 1,//Becomes triggered upon becoming unoccupied instead
							 m_bTriggersOnClient		: 1,//Client entities check and trigger
							 SpareBits					: 28;
			};
			unsigned int m_uFlags;
		};
		
	private:
		typedef Asura_ServerEntity_Trigger PARENT;

		Asura_Guid m_uParentObjectGuid;

		static Asura_Hashed_List<Asura_ServerEntity_VolumeTrigger> s_xList;
};

#endif//__ASURA_SERVERENTITY_VOLUMETRIGGER_H__

