//
// Asura_ServerEntity_EffectVolume.h
//
// Created by Emil Popovic - 15/12/2008
// 
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef __ASURA_SERVERENTITY_EFFECTVOLUME_H__
#define __ASURA_SERVERENTITY_EFFECTVOLUME_H__

#include "Entity/Asura_Entity.h"
#include "Entity/Triggers/EffectVolume/Asura_EffectVolume_EntityParameters.h"


// Entity that contains all data for EffectVolume. In fact EffectVolume is only ClientSide effect
// and it has no UpdateMessage from server side. ServerObject exists mainly for Creating, Activating,
// Deactivating and Destroying this object.

// Both Server and Client Entity contains only one value: Asura_EffectVolume_EntityParameters. So that means
// all relevant values are stored in the m_xData object. This object is quite complex, so when ServerSide is
// creating Client side object, it simply Call WriteToClientCreateStream. It will serialize it to "memfile"
// and this memfile is then send over network so ClientSide can ReadFromClientCreateStream. 

class Asura_ServerEntity_EffectVolume : public Asura_Entity_Server
{
#ifdef ASURA_TOOLS
	friend class Tools_Entity_EffectVolume;
#endif

public :
	virtual bool GetHasCreateMessage() const { return true; }
	virtual void MakeCreateMessage(Asura_Guid uRequestingClient, bool bSendGuaranteed = false);

	// Accessor to Effects.
	Asura_EffectVolume_Effect*	AddEffect( int x );
	void						RemoveEffect( u_int uNumOfEffectToRemove );
	Asura_EffectVolume_Effect*	GetEffect( u_int uNumOfEffectToGet );
	u_int						GetNumOfEffects();

	// Accessor function to Data object.
	Asura_EffectVolume_EntityParameters& GetData()	{return m_xData;}

	// System functions
	virtual void MakeClientUpdateMessage(void) {}
	static void CreateFromChunkStream( const Asura_Guid uGuid, Asura_Chunk_Stream& xStream );

#ifdef ASURA_TOOLS
	virtual void ApplyChangesForTemplateInstance(class Tools_EntityTemplateInstance* pxInstance);
	virtual bool CanBeIncludedInTemplate() const;
#endif


	// Accessor to s_xList
	static Asura_ServerEntity_EffectVolume* FindEntity( Asura_Guid uGuid ) { return s_xList.FindEntity( uGuid ); }
	static const Asura_Hashed_List<Asura_ServerEntity_EffectVolume>* GetList() { return &s_xList; }    

	// Constructor and Destructor.
	Asura_ServerEntity_EffectVolume( Asura_Guid uGuid );
	~Asura_ServerEntity_EffectVolume();

protected:
	
	// Chunking functions...
	virtual	bool IsFilledFromChunkBuffer() const { return false; }
	virtual bool CanBeSavedToChunk() const	{ return(true); }
	virtual	void ReadFromChunkStream( Asura_Chunk_Stream& xStream );
	virtual	void WriteToChunkStream( Asura_Chunk_Stream& xStream ) const;

	void WriteToClientCreateStream(Asura_Chunk_Memory_Stream& xStream) const;


private:
	static Asura_Hashed_List<Asura_ServerEntity_EffectVolume> s_xList;

	// Important value! Actually all values that are relevant to EffectVolume either Client or Server Side should be put here in this class! NOT OUTSIDE!
	// It is because that the same data are used in server and in client side object. And they should be synchronized all the time!
	Asura_EffectVolume_EntityParameters	m_xData;

	static const u_int s_uChunkVersion;	// The version used when reading/writing.
	typedef Asura_Entity_Server PARENT;

};


#endif //__ASURA_SERVERENTITY_EFFECTVOLUME_H__
