//
// Asura_FSFX_Template.h
//
// Created by JGP 02/02/06
// 
// Copyright (C) 2006 Rebellion (Derby), All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifdef ASURA_USE_FSFX

#ifndef __ASURA_FSFX_TEMPLATE_H__
#define __ASURA_FSFX_TEMPLATE_H__

#include "Core/Asura_LinkList.h"
#include "Core/Asura_String.h"
#include "Core/Asura_Hash.h"

class Asura_Chunk_Stream;
class Asura_FSFX_Component;
typedef Asura_ListElt<Asura_FSFX_Component*> Asura_FSFX_ComponentList;


#define DEFAULT_PRIORITY 5



class Asura_FSFX_Template
{

public:

	friend class Tools_Mode_FSFX_Template;

	// construct

	Asura_FSFX_Template();
	~Asura_FSFX_Template();

	// copy

#ifdef ASURA_TOOLS
	Asura_FSFX_Template* CreateCopy() const;
#endif // ASURA_TOOLS

	// render

	void Render( const float fIntensity ) const;
	static int Platform_GetPlatformFlag();

	// get / set

	Asura_Hash_ID GetNameHash() const { return m_xNameHash; }
	int GetPriority() const { return m_iPriority; }
	bool IsEnvironmentalEffect() const { return m_bIsEnvironmentalEffect; }
	void SetAsEnvironmentalEffect( bool bIsEnvironmentalEffect ) { m_bIsEnvironmentalEffect = bIsEnvironmentalEffect; }

#ifdef ASURA_TOOLS

	// tools only gets / sets

	Asura_FSFX_ComponentList* GetComponentList() const;
	void AddComponent( Asura_FSFX_Component* pxComponent );
	void DeleteComponent( Asura_FSFX_Component* pxComponent );
	void SwapComponents( Asura_FSFX_Component* pxComponent1, Asura_FSFX_Component* pxComponent2 );

	void SetName( const char* const szName );
	const char* const GetName() const { return m_xName.GetString(); }

	void SetLoadedFromFile( const char* szFilename ) { m_xLoadedFromFile = szFilename; }
	const char* GetLoadedFromFile() const { return m_xLoadedFromFile.GetString(); }
#endif // ASURA_TOOLS

	void ReadFromChunkStream( Asura_Chunk_Stream& xStream, const int iVersion );
#ifdef ASURA_TOOLS
	void WriteToChunkStream( Asura_Chunk_Stream& xStream ) const;
#endif // ASURA_TOOLS

private:

	void ClearComponentList();

	Asura_Hash_ID m_xNameHash;
	Asura_LinkList<Asura_FSFX_Component*> m_xComponentList;
	int m_iPriority;

#ifdef ASURA_TOOLS
	// Only in tools mode do we
	// actually store the name
	// otherwise it's just a hash...
	Asura_String m_xName;
	Asura_String m_xLoadedFromFile; // file this was loaded from (for debugging).
#endif // ASURA_TOOLS

	// (CJW - 08.10.2007)
	// Only one environmental effect can be active at any given time.
	// If a template that is marked as environmental is activated, all 
	// the other templates marked as environmental are deactivated in 
	// the FSFX handler.
	bool m_bIsEnvironmentalEffect;

};


#endif // __ASURA_FSFX_TEMPLATE_H__

#endif // ASURA_USE_FSFX
