
///////////////////////////////////////////////////////////////////////////////
//
// File   : Asura_FacialAnimation_BoneDataModifier.cpp
// Author : Will Plano
// Date   : 09/10/2008
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

#if defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)

// This file's header...
#include "FacialAnimation/Asura_FacialAnimation_BoneDataModifier.h"

#include "Animation/Asura_Animation.h"
#include "FacialAnimation/Asura_FacialAnimation_Face.h"
#include "FacialAnimation/Asura_FacialAnimation_System.h"


#ifndef ASURA_ANIMATION_BONEDATAMODIFIER_PRIORITY_FACIALANIMATION_UPDATETIMERS
#define ASURA_ANIMATION_BONEDATAMODIFIER_PRIORITY_FACIALANIMATION_UPDATETIMERS 0x00010000u
#endif

#ifndef ASURA_ANIMATION_BONEDATAMODIFIER_PRIORITY_FACIALANIMATION_UPDATEBONEDATA
#define ASURA_ANIMATION_BONEDATAMODIFIER_PRIORITY_FACIALANIMATION_UPDATEBONEDATA 0x00010000u
#endif

Asura_FacialAnimation_BoneDataModifier::Asura_FacialAnimation_BoneDataModifier( Asura_Animation* pxAnimation, const Asura_Hash_ID uFaceHash )
: PARENT( pxAnimation, ASURA_ANIMATION_BONEDATAMODIFIER_PRIORITY_FACIALANIMATION_UPDATETIMERS, ASURA_ANIMATION_BONEDATAMODIFIER_PRIORITY_FACIALANIMATION_UPDATEBONEDATA )
, m_uFaceHash( uFaceHash )
, m_xEyeParentRestPoseQuatOS( Asura_Maths::IdentityQuat )
, m_fEyeBlend( 1.0f )
, m_fLeftRight( 0.0f )
, m_fUpDown( 0.0f )
, m_fFocus( 0.0f )
, m_fClose( 0.0f )
, m_fSquint( 0.0f )
, m_fBalance( 0.0f )
{
	Asura_FacialAnimation_Face* pxFace = Asura_FacialAnimation_System::FindFace( uFaceHash );
	Asura_Assertf( pxFace, ( "Can't find face with hash 0x%08x", uFaceHash ) );
	if ( pxFace && pxFace->GetNumEyeBones())
	{
		const Asura_Hierarchy_Skin* pxSkin = pxAnimation ? pxAnimation->GetSkin() : NULL;
		if ( pxSkin )
		{
			int iBoneIndex = pxFace->GetEyeBone(0)->m_iBoneIndex;
			if ( iBoneIndex < pxSkin->NumberOfBones )
			{
				int iParentBone = pxSkin->ParentBone[ iBoneIndex  ];
				m_xEyeParentRestPoseQuatOS = pxSkin->BoneRestPose[ iParentBone ].Quat;
				while ( iParentBone > 0 && iParentBone < pxSkin->NumberOfBones )
				{
					iParentBone = pxSkin->ParentBone[ iParentBone ];
					m_xEyeParentRestPoseQuatOS = pxSkin->BoneRestPose[ iParentBone ].Quat * m_xEyeParentRestPoseQuatOS;
				}
			}
		}
	}
}

void Asura_FacialAnimation_BoneDataModifier::UpdateBoneData( const int iFirstBone, const int iLastBone )
{
	Asura_FacialAnimation_Face* pxFace = Asura_FacialAnimation_System::FindFace( m_uFaceHash );
	Asura_Assertf( pxFace, ( "Can't find face with hash 0x%08x", m_uFaceHash ) );
	const Asura_Hierarchy_Skin* pxSkin = m_pxAnimation ? m_pxAnimation->GetSkin() : NULL;
	if ( pxFace && pxSkin && m_fEyeBlend > 0.0f && pxFace->GetNumEyeBones())
	{
		Asura_Quat xInvEyeParentRestPoseQuatOS = m_xEyeParentRestPoseQuatOS;
		xInvEyeParentRestPoseQuatOS.Invert();

		float fLeftRightAngle_Left = 0.0f;
		float fUpDownAngle_Left = 0.0f;
		float fLeftRightAngle_Right = 0.0f;
		float fUpDownAngle_Right = 0.0f;
		bool bCalculatedAngles_Left = false;
		bool bCalculatedAngles_Right = false;
		
		// a focus of 1 is 100 degrees, so scale to get the radian value
		const float fFocusRad = Asura_Maths::DegToRad(m_fFocus*100.0f);

		for(u_int i=0; i<pxFace->GetNumEyeBones(); i++)
		{
			const Asura_FacialAnimation_Face_EyeBone* pxEyeBone = pxFace->GetEyeBone(i);
			int iBoneIndex = pxEyeBone->m_iBoneIndex;
			
			if(iBoneIndex < iFirstBone || iBoneIndex > iLastBone) continue;

			Asura_Euler_3 xEyeBoneEuler;
			if(pxEyeBone->m_bRight)
			{
				if(!bCalculatedAngles_Right)
				{
					//Doing our first ride side bone , so calculate the angles needed
					fLeftRightAngle_Right = Asura_Maths::ClampToRange( ( m_fLeftRight * pxFace->GetEyeLeftRightScale() ) + fFocusRad, -pxFace->GetEyeOutLimit(), pxFace->GetEyeInLimit() );

					float fLeftRightParametric = fLeftRightAngle_Right / pxFace->GetEyeLeftRightScale();
					fLeftRightParametric *= fLeftRightParametric;
					const float fUpDownMin = -Asura_Maths::Lerp( pxFace->GetEyeDownLimit(), pxFace->GetEyeDownSideLimit(), fLeftRightParametric );
					const float fUpDownMax = Asura_Maths::Lerp( pxFace->GetEyeUpLimit(), pxFace->GetEyeUpSideLimit(), fLeftRightParametric );
					fUpDownAngle_Right = Asura_Maths::ClampToRange( m_fUpDown * pxFace->GetEyeUpDownScale(), fUpDownMin, fUpDownMax );

					bCalculatedAngles_Right = true;
				}
				
				if(pxEyeBone->m_bEye)
				{
					//right eye
					xEyeBoneEuler.x = fUpDownAngle_Right;
					xEyeBoneEuler.y = -fLeftRightAngle_Right;
					xEyeBoneEuler.z = 0.0f;
				}
				else
				{
					//right eye lid
					float fEyelidAngle = m_fClose * Asura_Maths::Lerp( pxEyeBone->m_fRotClose, pxEyeBone->m_fRotSquint, m_fSquint ) ;
					if ( m_fBalance < 0.0f )
					{
						fEyelidAngle = ( 1.0f + m_fBalance ) * fEyelidAngle;
					}
					fEyelidAngle = Asura_Maths::ClampToRange( fUpDownAngle_Right - fEyelidAngle, pxEyeBone->m_fAngMin, pxEyeBone->m_fAngMax );

					xEyeBoneEuler.x = fEyelidAngle;
					xEyeBoneEuler.y = 0.0f;
					xEyeBoneEuler.z = 0.0f;
				}
			}
			else
			{
				if(!bCalculatedAngles_Left)
				{
					//Doing our first left side bone , so calculate the angles needed
					fLeftRightAngle_Left = Asura_Maths::ClampToRange( ( m_fLeftRight * pxFace->GetEyeLeftRightScale() ) - fFocusRad, -pxFace->GetEyeInLimit(), pxFace->GetEyeOutLimit() );

					float fLeftRightParametric = fLeftRightAngle_Left / pxFace->GetEyeLeftRightScale();
					fLeftRightParametric *= fLeftRightParametric;
					const float fUpDownMin = -Asura_Maths::Lerp( pxFace->GetEyeDownLimit(), pxFace->GetEyeDownSideLimit(), fLeftRightParametric );
					const float fUpDownMax = Asura_Maths::Lerp( pxFace->GetEyeUpLimit(), pxFace->GetEyeUpSideLimit(), fLeftRightParametric );
					fUpDownAngle_Left = Asura_Maths::ClampToRange( m_fUpDown * pxFace->GetEyeUpDownScale(), fUpDownMin, fUpDownMax );

					bCalculatedAngles_Left = true;
				}
				
				if(pxEyeBone->m_bEye)
				{
					//left eye
					xEyeBoneEuler.x = fUpDownAngle_Left;
					xEyeBoneEuler.y = -fLeftRightAngle_Left;
					xEyeBoneEuler.z = 0.0f;
				}
				else
				{
					//left eye lid
					float fEyelidAngle = m_fClose * Asura_Maths::Lerp( pxEyeBone->m_fRotClose, pxEyeBone->m_fRotSquint, m_fSquint ) ;
					if ( m_fBalance > 0.0f )
					{
						fEyelidAngle = ( 1.0f - m_fBalance ) * fEyelidAngle;
					}
					fEyelidAngle = Asura_Maths::ClampToRange( fUpDownAngle_Left - fEyelidAngle, pxEyeBone->m_fAngMin, pxEyeBone->m_fAngMax );

					xEyeBoneEuler.x = fEyelidAngle;
					xEyeBoneEuler.y = 0.0f;
					xEyeBoneEuler.z = 0.0f;
				}
			}

			Asura_Quat xEyeBoneQuat;
 			xEyeBoneQuat.FromEuler( xEyeBoneEuler );
			xEyeBoneQuat = xInvEyeParentRestPoseQuatOS * xEyeBoneQuat * m_xEyeParentRestPoseQuatOS;

 			const Asura_Quat& xEyeBoneRestPoseQuat = pxSkin->BoneRestPose[ iBoneIndex ].Quat;
			xEyeBoneQuat = xEyeBoneQuat * xEyeBoneRestPoseQuat;

			Asura_Quat xOldEyeBoneQuat;
			m_pxAnimation->GetTweenStore()->GetBoneData( iBoneIndex ).GetOrientation( xOldEyeBoneQuat );
 			xEyeBoneQuat.Slerp( xOldEyeBoneQuat, 1.0f - m_fEyeBlend );
			m_pxAnimation->GetTweenStore()->GetBoneData( iBoneIndex ).SetOrientation( xEyeBoneQuat );
		}		
	}
}

void Asura_FacialAnimation_BoneDataModifier::SetAnimatedParameters(	const float fEyeBlend,
																	const float fLeftRight,
																	const float fUpDown,
																	const float fFocus,
																	const float fClose,
																	const float fSquint,
																	const float fBalance )
{
	m_fEyeBlend		= fEyeBlend;
	m_fLeftRight	= fLeftRight;
	m_fUpDown		= fUpDown;
	m_fFocus		= fFocus;
	m_fClose		= fClose;
	m_fSquint		= fSquint;
	m_fBalance		= fBalance;
}

#endif // ASURA_USE_FACIAL_ANIMATION_SYSTEM
