
///////////////////////////////////////////////////////////////////////////////
//
// File   : Asura_FacialAnimation_MuscleGroup.cpp
// Author : Andy Tate
// Date   : 9-May-2008
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

#if defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)

// This file's header...
#include "FacialAnimation/Asura_FacialAnimation_MuscleGroup.h"

// Asura independent header-files...
#include "Animation/Asura_Animation.h"
#include "Animation/Asura_AnimationController_Animation.h"
#include "Animation/Asura_Hierarchy.h"
#include "ChunkSystem/Asura_Chunk_Stream.h"


////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Version	Author	Comment
// ~~~~~~~	~~~~~~	~~~~~~~
// 1		ART		Created.
////////////////////////////////////////////////////////////////////////////////
const u_int	Asura_FacialAnimation_MuscleGroup::s_uChunkVersion	= 1;


////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////

#if defined(ASURA_TOOLS)
const char*	Asura_FacialAnimation_MuscleGroup::s_aszControlTypeNames[MAX_CONTROL_TYPES] =
{
	"Single",
	"Single (Pair)",
	"Balance",
	"Balance (Pair)",
};
#endif	//defined(ASURA_TOOLS)


////////////////////////////////////////////////////////////////////////////////
// Private functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Protected functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Public functions...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_FacialAnimation_MuscleGroup::Asura_FacialAnimation_MuscleGroup
// Author       : Andy Tate
// Date         : 22-May-2008
////////////////////////////////////////////////////////////////////////////////

Asura_FacialAnimation_MuscleGroup::Asura_FacialAnimation_MuscleGroup(void)
: m_eControlType(CT_SINGLE)
, m_uNumAnimations(0)
{
}

/// End of function 'Asura_FacialAnimation_MuscleGroup::Asura_FacialAnimation_MuscleGroup'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_FacialAnimation_MuscleGroup::Asura_FacialAnimation_MuscleGroup
// Author       : Andy Tate
// Date         : 9-May-2008
////////////////////////////////////////////////////////////////////////////////

Asura_FacialAnimation_MuscleGroup::Asura_FacialAnimation_MuscleGroup(Asura_Chunk_Stream& xStream)
{
	ReadFromChunkStream(xStream);
}

/// End of function 'Asura_FacialAnimation_MuscleGroup::Asura_FacialAnimation_MuscleGroup'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_FacialAnimation_MuscleGroup::Asura_FacialAnimation_MuscleGroup
// Author       : Andy Tate
// Date         : 9-May-2008
////////////////////////////////////////////////////////////////////////////////

Asura_FacialAnimation_MuscleGroup::Asura_FacialAnimation_MuscleGroup(const Asura_Hash_ID uHash,
																	 Asura_Chunk_Stream& xStream)
{
	ReadFromChunkStream(xStream);
}

/// End of function 'Asura_FacialAnimation_MuscleGroup::Asura_FacialAnimation_MuscleGroup'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_FacialAnimation_MuscleGroup::UpdateAnimation
// Author       : Andy Tate
// Date         : 23-May-2008
////////////////////////////////////////////////////////////////////////////////
#ifdef ASURA_MAIN
u_int Asura_FacialAnimation_MuscleGroup::UpdateAnimation(const float fWeight,
														 const float fBalance,
														 Asura_Animation& xAnimation,
														 Asura_AnimationController_Group& xGroup) const
{
	Asura_AnimationController_Animation*	pxAnimation	= xAnimation.GetControllerAnimationFromHash(m_auAnimationHashes[0]);

	// Update the contribution of each animation AND the part of the animation we're displaying...
	if(pxAnimation == NULL)
	{
		Asura_Assert(false, "Mismatch between the muscle-groups and animations for this face.");
		return(0);
	}
	else
	{
		Asura_AnimationBlend*	pxPreviousBlend	= pxAnimation->GetBlendToPreviousController();

		if(pxPreviousBlend == NULL)
		{
			Asura_Assert(false, "All animations should have a previous blend - something's gone wrong.");
			return(0);
		}
		else
		{
			// Decide what weighting to use based on the control-type...
			switch(m_eControlType)
			{
				case CT_SINGLE:
				{
					pxPreviousBlend->SetWeight(fWeight);
					return(1);
				}

				case CT_SINGLE_PAIR:
				{
					Asura_AnimationBlend*	pxNextBlend	= pxAnimation->GetBlendToNextController();

					if(fWeight == 0.0f)
					{
						pxPreviousBlend->SetWeight(0.0f);
						pxNextBlend->SetWeight(0.0f);
					}
					else if(fWeight < 0.0f)
					{
						pxPreviousBlend->SetWeight(-fWeight);
						pxNextBlend->SetWeight(0.0f);
					}
					else
					{
						pxPreviousBlend->SetWeight(0.0f);
						pxNextBlend->SetWeight(fWeight);
					}
					return(2);
				}

				case CT_BALANCE:
				{
					pxPreviousBlend->SetWeight(fWeight);

					if(fWeight > 0.0f)
					{
						const float	fParametric	= (fBalance + 1.0f) * 0.5f;

						pxAnimation->SetParametricTimer(fParametric);
					}
					return(1);
				}

				case CT_BALANCE_PAIR:
				{
					Asura_AnimationBlend*	pxNextBlend	= pxAnimation->GetBlendToNextController();

					if(fWeight == 0.0f)
					{
						pxPreviousBlend->SetWeight(0.0f);
						pxNextBlend->SetWeight(0.0f);
					}
					else
					{
						Asura_AnimationController_Animation*	pxNextAnimation	= xAnimation.GetControllerAnimationFromHash(m_auAnimationHashes[1]);
						const float								fParametric		= (fBalance + 1.0f) * 0.5f;

						if(fWeight < 0.0f)
						{
							pxPreviousBlend->SetWeight(-fWeight);
							pxAnimation->SetParametricTimer(fParametric);
							pxNextBlend->SetWeight(0.0f);
						}
						else
						{
							pxPreviousBlend->SetWeight(0.0f);
							pxNextBlend->SetWeight(fWeight);
							pxNextAnimation->SetParametricTimer(fParametric);
						}
					}
					return(2);
				}

				default:
				{
					Asura_Assert(false, "Unexpected control-type found.");
					return(0);
				}
			}		// End of switch.
		}
	}
}
#endif //ASURA_MAIN
/// End of function 'Asura_FacialAnimation_MuscleGroup::UpdateAnimation'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_FacialAnimation_MuscleGroup::ReadFromChunkStream
// Author       : Andy Tate
// Date         : 9-May-2008
////////////////////////////////////////////////////////////////////////////////

void Asura_FacialAnimation_MuscleGroup::ReadFromChunkStream(Asura_Chunk_Stream& xStream)
{
	u_int	uVersion;

	// First, read (and verify) the chunk-version.
	xStream >> uVersion;

	switch(uVersion)
	{
		case 1:
		{
			u_int	uControlType;

			xStream >> uControlType;
			xStream >> m_uNumAnimations;

			for(u_int u = 0; u < m_uNumAnimations; u++)
			{
				xStream >> m_auAnimationHashes[u];
			}

			m_eControlType	= static_cast<ControlType>(uControlType);
			break;
		}

		default:
		{
			Asura_Assertf(false, ("Unexpected chunk-version %d", uVersion));
			break;
		}
	}		// End of switch.
}

/// End of function 'Asura_FacialAnimation_MuscleGroup::ReadFromChunkStream'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_FacialAnimation_MuscleGroup::WriteToChunkStream
// Author       : Andy Tate
// Date         : 9-May-2008
////////////////////////////////////////////////////////////////////////////////

void Asura_FacialAnimation_MuscleGroup::WriteToChunkStream(Asura_Chunk_Stream& xStream) const
{
	const u_int	uControlType	= m_eControlType;

	// First write the chunk-version...
	xStream << s_uChunkVersion;

	xStream << uControlType;
	xStream << m_uNumAnimations;

	for(u_int u = 0; u < m_uNumAnimations; u++)
	{
		xStream << m_auAnimationHashes[u];
	}
}

/// End of function 'Asura_FacialAnimation_MuscleGroup::WriteToChunkStream'. ///

#endif	//defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)


/// End of file. ///
