
///////////////////////////////////////////////////////////////////////////////
//
// File   : Asura_FacialAnimation_MuscleGroupAnimation.cpp
// Author : Andy Tate
// Date   : 9-May-2008
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

#if defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)

// This file's header...
#include "FacialAnimation/Asura_FacialAnimation_MuscleGroupAnimation.h"

// Asura independent header-files...
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "Core/Asura_LinkList.h"
#include "Core/Asura_Maths.h"
#include "Debugging/Asura_Logfile.h"

// Tools header files
#ifdef ASURA_TOOLS
#include "FacialAnimation/Tools_FacialAnimation_MuscleGroupAnimation.h"
#endif


////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////

#if 1
#define LOG_OUTPUT(s)
#else
#define LOG_OUTPUT(s)	Asura_OutputToDebugger_Retail(s)
#endif


////////////////////////////////////////////////////////////////////////////////
// Version	Author	Comment
// ~~~~~~~	~~~~~~	~~~~~~~
// 1		ART		Created.
////////////////////////////////////////////////////////////////////////////////
const u_int	Asura_FacialAnimation_MuscleGroupAnimation::s_uChunkVersion	= 1;


////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_TOOLS
DEBUGVAR float	dbg_fMuscleGroupAnimationIdenticalTimeThreshold	= 0.010f;
#endif

////////////////////////////////////////////////////////////////////////////////
// Private functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Protected functions...
////////////////////////////////////////////////////////////////////////////////

#if defined(ASURA_TOOLS)

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_FacialAnimation_MuscleGroupAnimation::Tools_MergeSameTimeKeyFrames
// Author       : Andy Tate
// Date         : 27-Aug-2008
////////////////////////////////////////////////////////////////////////////////

u_int Asura_FacialAnimation_MuscleGroupAnimation::Tools_MergeSameTimeKeyFrames(const u_int uNumInitialKeyFrames)
{
	Asura_FacialAnimation_MuscleGroupAnimation_KeyFrame*	pxCurrentKeyFrame	= m_pxKeyFrames;
	u_int													uNumKeyFrames		= 0;

	for(u_int u = 0; u < (uNumInitialKeyFrames - 1); u++, pxCurrentKeyFrame++)
	{
		const float												fTime	= pxCurrentKeyFrame->GetTime();

		Asura_FacialAnimation_MuscleGroupAnimation_KeyFrame*	pxNextKeyFrame;
		float													fTimeDifference;

		// Are the next 2 times identical (within tolerances)?
		pxNextKeyFrame	= pxCurrentKeyFrame + 1;
		fTimeDifference	= pxNextKeyFrame->GetTime() - fTime;

		if(fTimeDifference > dbg_fMuscleGroupAnimationIdenticalTimeThreshold)
		{
			// No - copy the key-frame data (if necessary)...
			if(uNumKeyFrames < u)
			{
				m_pxKeyFrames[uNumKeyFrames]	= *pxCurrentKeyFrame;
			}
			uNumKeyFrames++;
		}
		else
		{
			float	fMaxWeight		= pxCurrentKeyFrame->GetWeight();
			float	fMaxBalance		= pxCurrentKeyFrame->GetBalance();
			float	fMaxAbsWeight	= Asura_Maths::Abs(fMaxWeight);
			float	fMaxAbsBalance	= Asura_Maths::Abs(fMaxBalance);

			// We've got at least 2 identical times - check how many more we have...
			for(u_int v = (u + 1); v < uNumInitialKeyFrames; v++, pxNextKeyFrame++)
			{
				// Is this time no longer identical?
				fTimeDifference	= pxNextKeyFrame->GetTime() - fTime;

				if(fTimeDifference > dbg_fMuscleGroupAnimationIdenticalTimeThreshold)
				{
					// Times are sufficiently different - update the original key-frame...
					m_pxKeyFrames[uNumKeyFrames++].Set(fTime, fMaxWeight, fMaxBalance);

					// ...before skipping the other identical ones...
					pxCurrentKeyFrame	= pxNextKeyFrame - 1;
					u					= v - 1;
					v					= uNumInitialKeyFrames;
				}
				else
				{
					const float	fWeight		= pxNextKeyFrame->GetWeight();
					const float	fBalance	= pxNextKeyFrame->GetBalance();
					const float	fAbsWeight	= Asura_Maths::Abs(fWeight);
					const float	fAbsBalance	= Asura_Maths::Abs(fBalance);

					if(fAbsWeight > fMaxAbsWeight)
					{
						fMaxWeight		= fWeight;
						fMaxAbsWeight	= fAbsWeight;
					}

					if(fAbsBalance > fMaxAbsBalance)
					{
						fMaxBalance		= fBalance;
						fMaxAbsBalance	= fAbsBalance;
					}
				}
			}
		}
	}

	// Finally, copy the last frame (if necessary)...
	if(uNumKeyFrames < (uNumInitialKeyFrames - 1))
	{
		m_pxKeyFrames[uNumKeyFrames]	= *pxCurrentKeyFrame;
	}
	uNumKeyFrames++;

	// Return the number of key-frames we're left with...
	return(uNumKeyFrames);
}

/// End of function 'Asura_FacialAnimation_MuscleGroupAnimation::Tools_MergeSameTimeKeyFrames'. ///

#endif	//defined(ASURA_TOOLS)


////////////////////////////////////////////////////////////////////////////////
// Public functions...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_FacialAnimation_MuscleGroupAnimation::Asura_FacialAnimation_MuscleGroupAnimation
// Author       : Andy Tate
// Date         : 27-Aug-2008
////////////////////////////////////////////////////////////////////////////////

Asura_FacialAnimation_MuscleGroupAnimation::Asura_FacialAnimation_MuscleGroupAnimation(void)
: m_uNumKeyFrames(0)
, m_pxKeyFrames(NULL)
{
}

/// End of function 'Asura_FacialAnimation_MuscleGroupAnimation::Asura_FacialAnimation_MuscleGroupAnimation'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_FacialAnimation_MuscleGroupAnimation::Asura_FacialAnimation_MuscleGroupAnimation
// Author       : Andy Tate
// Date         : 9-May-2008
////////////////////////////////////////////////////////////////////////////////

Asura_FacialAnimation_MuscleGroupAnimation::Asura_FacialAnimation_MuscleGroupAnimation(const Asura_Hash_ID uHash,
																					   Asura_Chunk_Stream& xStream)
{
	ReadFromChunkStream(xStream);
}

/// End of function 'Asura_FacialAnimation_MuscleGroupAnimation::Asura_FacialAnimation_MuscleGroupAnimation'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_FacialAnimation_MuscleGroupAnimation::~Asura_FacialAnimation_MuscleGroupAnimation
// Author       : Andy Tate
// Date         : 27-Aug-2008
////////////////////////////////////////////////////////////////////////////////

Asura_FacialAnimation_MuscleGroupAnimation::~Asura_FacialAnimation_MuscleGroupAnimation()
{
	delete[](m_pxKeyFrames);
}

/// End of function 'Asura_FacialAnimation_MuscleGroupAnimation::~Asura_FacialAnimation_MuscleGroupAnimation'. ///


#if defined(ASURA_TOOLS)

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_FacialAnimation_MuscleGroupAnimation::Tools_AddKeyFrame
// Author       : Andy Tate
// Date         : 27-Aug-2008
////////////////////////////////////////////////////////////////////////////////

bool Asura_FacialAnimation_MuscleGroupAnimation::Tools_AddKeyFrame(const float fTime,
																   const float fWeight,
																   const float fBalance)
{
	const u_int												uNumNewKeyFrames	= m_uNumKeyFrames + 1;
	Asura_FacialAnimation_MuscleGroupAnimation_KeyFrame*	pxNewKeyFrames		= new Asura_FacialAnimation_MuscleGroupAnimation_KeyFrame [uNumNewKeyFrames];

	if(pxNewKeyFrames == NULL)
	{
		return(false);
	}
	else
	{
		const Asura_FacialAnimation_MuscleGroupAnimation_KeyFrame*	pxCurrentKeyFrame	= m_pxKeyFrames;
		Asura_FacialAnimation_MuscleGroupAnimation_KeyFrame*		pxNewKeyFrame		= pxNewKeyFrames;
		bool														bInserted			= false;

		// Add the new key-frame into the existing ones...
		for(u_int u = 0; u < m_uNumKeyFrames; u++, pxCurrentKeyFrame++, pxNewKeyFrame++)
		{
			const float	fKeyFrameTime	= pxCurrentKeyFrame->GetTime();

			if(!bInserted && (fTime < fKeyFrameTime))
			{
				bInserted	= true;
				pxNewKeyFrame->Set(fTime, fWeight, fBalance);
				pxNewKeyFrame++;
			}

			*pxNewKeyFrame	= *pxCurrentKeyFrame;
		}

		// If we didn't successfully insert, we need to do it now...
		if(!bInserted)
		{
			pxNewKeyFrame->Set(fTime, fWeight, fBalance);
		}

		// Remove the old key-frames and store the new ones...
		delete[](m_pxKeyFrames);
		m_pxKeyFrames	= pxNewKeyFrames;
		m_uNumKeyFrames	= uNumNewKeyFrames;

		return(true);
	}
}

/// End of function 'Asura_FacialAnimation_MuscleGroupAnimation::Tools_AddKeyFrame'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_FacialAnimation_MuscleGroupAnimation::Tools_RationaliseKeyFrames
// Author       : Andy Tate
// Date         : 27-Aug-2008
////////////////////////////////////////////////////////////////////////////////

void Asura_FacialAnimation_MuscleGroupAnimation::Tools_RationaliseKeyFrames(void)
{
	u_int	uNumKeyFrames;

	// Merge together data at the same point in time...
	uNumKeyFrames	= Tools_MergeSameTimeKeyFrames(m_uNumKeyFrames);

	// Finally, re-allocate the key-frame data (if necessary)...
	if(uNumKeyFrames < m_uNumKeyFrames)
	{
		const Asura_FacialAnimation_MuscleGroupAnimation_KeyFrame*	pxCurrentKeyFrame	= m_pxKeyFrames;
		Asura_FacialAnimation_MuscleGroupAnimation_KeyFrame*		pxNewKeyFrames		= new Asura_FacialAnimation_MuscleGroupAnimation_KeyFrame [uNumKeyFrames];

		for(u_int u = 0; u < uNumKeyFrames; u++, pxCurrentKeyFrame++)
		{
			pxNewKeyFrames[u]	= *pxCurrentKeyFrame;

			// For debugging purposes...
			LOG_OUTPUT(("\t(%.3f/%.3f @ %.3f)", pxCurrentKeyFrame->GetWeight(), pxCurrentKeyFrame->GetBalance(), pxCurrentKeyFrame->GetTime()));
		}

		delete[](m_pxKeyFrames);
		m_pxKeyFrames	= pxNewKeyFrames;
		m_uNumKeyFrames	= uNumKeyFrames;
	}
}

/// End of function 'Asura_FacialAnimation_MuscleGroupAnimation::Tools_RationaliseKeyFrames'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_FacialAnimation_MuscleGroupAnimation::Tools_SetKeyFrames
// Author       : Richard Rice
// Date         : 23-Sep-2008
////////////////////////////////////////////////////////////////////////////////

void Asura_FacialAnimation_MuscleGroupAnimation::Tools_SetKeyFrames(const Tools_FacialAnimation_MuscleGroupAnimation& xToolsAnimation)
{
	//delete any previous keyframes
	delete[](m_pxKeyFrames);

	const Asura_LinkList<Asura_FacialAnimation_MuscleGroupAnimation_KeyFrame>& xToolsKeyFrames = xToolsAnimation.GetKeyFrameContainer();
	
	//allocate the new keyframes
	m_uNumKeyFrames = xToolsKeyFrames.Size();
	m_pxKeyFrames = m_uNumKeyFrames ? new Asura_FacialAnimation_MuscleGroupAnimation_KeyFrame[m_uNumKeyFrames] : 0;

	//and copy from the tools list of keyframes
	Asura_FacialAnimation_MuscleGroupAnimation_KeyFrame* pxDest = m_pxKeyFrames;

	for(Asura_LinkList_ConstIt<Asura_FacialAnimation_MuscleGroupAnimation_KeyFrame> xIt(xToolsKeyFrames); !xIt.Done(); xIt.Next(),pxDest++)
	{
		*pxDest = *xIt.GetCurrent();
	}
}

/// End of function 'Asura_FacialAnimation_MuscleGroupAnimation::Tools_SetKeyFrames'. ///

float Asura_FacialAnimation_MuscleGroupAnimation::Tools_GetLastKeyFrameTime() const
{
	if(m_uNumKeyFrames)
	{
		return m_pxKeyFrames[m_uNumKeyFrames-1].GetTime();
	}
	else
	{
		return 0.0f;
	}
}


#endif	//defined(ASURA_TOOLS)



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_FacialAnimation_MuscleGroupAnimation::CalculateWeightAndBalance
// Author       : Andy Tate
// Date         : 27-Aug-2008
////////////////////////////////////////////////////////////////////////////////

bool Asura_FacialAnimation_MuscleGroupAnimation::CalculateWeightAndBalance(const float fTime,
																		   float& fWeight,
																		   float& fBalance) const
{
	const Asura_FacialAnimation_MuscleGroupAnimation_KeyFrame*	pxCurrentKeyFrame	= m_pxKeyFrames;


	if(fTime < pxCurrentKeyFrame->GetTime())
	{
		// Time is before the start of the animation...
		return(false);
	}
	else
	{
		// Initialise weight and balance for first frame. Later on, if there will be more that one frame, these values will be overriden by new calculated ones.
		fWeight		= pxCurrentKeyFrame->GetWeight();
		fBalance	= pxCurrentKeyFrame->GetBalance();

		const Asura_FacialAnimation_MuscleGroupAnimation_KeyFrame*	pxPreviousKeyFrame	= NULL;

		// Find the key-frames which 
		for(u_int u = 0; u < m_uNumKeyFrames; u++, pxCurrentKeyFrame++)
		{
			const float	fKeyFrameTime	= pxCurrentKeyFrame->GetTime();

			if(fTime <= fKeyFrameTime)
			{
				if( pxPreviousKeyFrame )
				{
					const float	fPreviousTime	= pxPreviousKeyFrame->GetTime();
					const float	fParamteric		= (fTime - fPreviousTime) / (fKeyFrameTime - fPreviousTime);

					fWeight		= Asura_Maths::Lerp(pxPreviousKeyFrame->GetWeight(), pxCurrentKeyFrame->GetWeight(), fParamteric);
					fBalance	= Asura_Maths::Lerp(pxPreviousKeyFrame->GetBalance(), pxCurrentKeyFrame->GetBalance(), fParamteric);
				}
				return(true);
			}
			else
			{
				pxPreviousKeyFrame	= pxCurrentKeyFrame;
			}
		}

		// Time MUST be beyond the end of the animation...
		return(true);
	}
}

/// End of function 'Asura_FacialAnimation_MuscleGroupAnimation::CalculateWeightAndBalance'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_FacialAnimation_MuscleGroupAnimation::ReadFromChunkStream
// Author       : Andy Tate
// Date         : 9-May-2008
////////////////////////////////////////////////////////////////////////////////

void Asura_FacialAnimation_MuscleGroupAnimation::ReadFromChunkStream(Asura_Chunk_Stream& xStream)
{
	u_int	uVersion;

	// First, read (and verify) the chunk-version.
	xStream >> uVersion;

	switch(uVersion)
	{
		case 1:
		{
			Asura_FacialAnimation_MuscleGroupAnimation_KeyFrame*	pxKeyFrame;

			xStream >> m_uNumKeyFrames;

			m_pxKeyFrames	= new Asura_FacialAnimation_MuscleGroupAnimation_KeyFrame [m_uNumKeyFrames];
			pxKeyFrame		= m_pxKeyFrames;

			for(u_int u = 0; u < m_uNumKeyFrames; u++, pxKeyFrame++)
			{
				float	fTime;
				float	fWeight;
				float	fBalance;

				xStream >> fTime;
				xStream >> fWeight;
				xStream >> fBalance;

				pxKeyFrame->Set(fTime, fWeight, fBalance);
			}
			break;
		}

		default:
		{
			Asura_Assertf(false, ("Unexpected chunk-version %d", uVersion));
			break;
		}
	}		// End of switch.
}

/// End of function 'Asura_FacialAnimation_MuscleGroupAnimation::ReadFromChunkStream'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_FacialAnimation_MuscleGroupAnimation::WriteToChunkStream
// Author       : Andy Tate
// Date         : 9-May-2008
////////////////////////////////////////////////////////////////////////////////

void Asura_FacialAnimation_MuscleGroupAnimation::WriteToChunkStream(Asura_Chunk_Stream& xStream) const
{
	const Asura_FacialAnimation_MuscleGroupAnimation_KeyFrame*	pxKeyFrame	= m_pxKeyFrames;

	// First write the chunk-version...
	xStream << s_uChunkVersion;
	xStream << m_uNumKeyFrames;

	for(u_int u = 0; u < m_uNumKeyFrames; u++, pxKeyFrame++)
	{
		const float	fTime		= pxKeyFrame->GetTime();
		const float	fWeight		= pxKeyFrame->GetWeight();
		const float	fBalance	= pxKeyFrame->GetBalance();

		xStream << fTime;
		xStream << fWeight;
		xStream << fBalance;
	}
}

/// End of function 'Asura_FacialAnimation_MuscleGroupAnimation::WriteToChunkStream'. ///

#endif	//defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)


/// End of file. ///
