//////////////////////////////////////////////////////////////////////////////
//
// Asura_GUIMenu_Command_Manager.cpp
//
// Copyright (c) 2007 Rebellion. All rights reserved.
//
// Description:
//
//     manager class to handle multiple commands based on defined button presses.
//    
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"
// This file's header.
#include "GUIMenu/Asura_GUIMenu_CommandManager.h"

#ifdef ASURA_USE_GUI_MENU_SYSTEM
#include "GUIMenu/Asura_GUIMenu_Audio.h"
#include "GUIMenu/Asura_GUIMenu_CommandConsole.h"
#include "GUIMenu/Asura_GUIMenu_Widget_Base.h"
#include "GUIMenu/Asura_GUIMenu_Widget_Button.h"
#ifdef ASURA_TOOLS
#include "GUIMenu/Asura_GUIMenu_System.h"
#include "GUIMenu/Tools_GUIMenu_XMLExporter.h"
#endif

#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "Interface/Asura_CommandConsole.h"
#ifdef ASURA_TOOLS
#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Overlay.h"
#include "Misc/Tools_Modal_Overlay.h"
#endif


// C O N S T A N T S /////////////////////////////////////////////////////////
/*
Versions: ASURA_CHUNK_ELEMENT_TEXT_VERSION

0:	Initial data 
1:	Added button action flags - SK 16/11/07
*/
static const u_int uASURA_GUIMENU_CONSOLEMANAGER_CHUNK_VERSION  = 1;

// C L A S S   I M P L E M E N T A T I O N ///////////////////////////////////

// T O O L    V A R S ////////////////////////////////////////////////////////

#ifdef ASURA_TOOLS
struct ActionAcceptButtonLookUp
{
	const Asura_Char* m_szName;
	u_int	m_uValue;
};

static ActionAcceptButtonLookUp g_axActionAcceptButtonLookUp[] =
{
	{"Accept",		 ASURA_GUIMENU_ACTION_BUTTON_ACCEPT},
	{"Alt Accept 1", ASURA_GUIMENU_ACTION_BUTTON_ALT_ACCEPT1	},
	{"Alt Accept 2", ASURA_GUIMENU_ACTION_BUTTON_ALT_ACCEPT2	},
	{"Alt Accept 3", ASURA_GUIMENU_ACTION_BUTTON_ALT_ACCEPT3	},
	{"Alt Accept 4", ASURA_GUIMENU_ACTION_BUTTON_ALT_ACCEPT4	},
	{"Alt Accept 5", ASURA_GUIMENU_ACTION_BUTTON_ALT_ACCEPT5	},
	{"Alt Accept 6", ASURA_GUIMENU_ACTION_BUTTON_ALT_ACCEPT6	},
	{"Alt Accept 7", ASURA_GUIMENU_ACTION_BUTTON_ALT_ACCEPT7	},
	{"Alt Accept 8", ASURA_GUIMENU_ACTION_BUTTON_ALT_ACCEPT8	},
	{"Alt Accept 9", ASURA_GUIMENU_ACTION_BUTTON_ALT_ACCEPT9	},
	{"Pad Up",		 ASURA_GUIMENU_ACTION_BUTTON_UP				},
	{"Pad Down",	 ASURA_GUIMENU_ACTION_BUTTON_DOWN			},
	{"Pad Left",	 ASURA_GUIMENU_ACTION_BUTTON_LEFT			},
	{"Pad Right",	 ASURA_GUIMENU_ACTION_BUTTON_RIGHT			},
};

static const Asura_Char* g_axActionAcceptButtonLookUpNames[] =
{
	g_axActionAcceptButtonLookUp[0].m_szName,
	g_axActionAcceptButtonLookUp[1].m_szName,
	g_axActionAcceptButtonLookUp[2].m_szName,
	g_axActionAcceptButtonLookUp[3].m_szName,
	g_axActionAcceptButtonLookUp[4].m_szName,
	g_axActionAcceptButtonLookUp[5].m_szName,
	g_axActionAcceptButtonLookUp[6].m_szName,
	g_axActionAcceptButtonLookUp[7].m_szName,
	g_axActionAcceptButtonLookUp[8].m_szName,
	g_axActionAcceptButtonLookUp[9].m_szName,
	g_axActionAcceptButtonLookUp[10].m_szName,
	g_axActionAcceptButtonLookUp[11].m_szName,
	g_axActionAcceptButtonLookUp[12].m_szName,
	g_axActionAcceptButtonLookUp[13].m_szName,
};

static const char* aszACTIONACCEPTNAMES[ASURA_GUIMENU_MAX_ALLOWED_MANAGER_COMMANDS] =
{
	"Action Command 1",
	"Action Command 2",
	"Action Command 3",
	"Action Command 4",
};

static const u_int g_uNumActionAcceptButtonLookUp = sizeof(g_axActionAcceptButtonLookUp) / sizeof(ActionAcceptButtonLookUp);
#endif // ASURA_TOOLS

//////////////////////////////////////////////////////////////////////////////
// Asura_GUIMenu_CommandManager
//////////////////////////////////////////////////////////////////////////////

Asura_GUIMenu_CommandManager::Asura_GUIMenu_CommandManager( void)
{
	ClearVariables();

#ifdef ASURA_TOOLS
	Initialise( ASURA_GUIMENU_MAX_ALLOWED_MANAGER_COMMANDS);
#endif
}

// ---------------------------------------------------------------------------

Asura_GUIMenu_CommandManager::~Asura_GUIMenu_CommandManager( void)
{
	Shutdown();
}

// ---------------------------------------------------------------------------

void Asura_GUIMenu_CommandManager::Initialise( u_int uMaxCommands)
{
	u_int u;

	if( m_pxActionButtonCommand != NULL)
	{
		delete []m_pxActionButtonCommand;
		m_pxActionButtonCommand = NULL;
	}

	if( m_pxActionButtonCommand == NULL)
	{
		m_pxActionButtonCommand = new Asura_GUIMenu_ActionButtonCommand[uMaxCommands];
		for( u=0; u<uMaxCommands; u++)
		{
#ifdef ASURA_TOOLS
			m_pxActionButtonCommand[u].m_iToolActionIndex = 0;
#endif
			m_pxActionButtonCommand[u].m_uButtonAction = ASURA_GUIMENU_ACTION_BUTTON_ACCEPT;
			m_pxActionButtonCommand[u].m_uButtonActionFlags = 0;
		}
	}
}

// ---------------------------------------------------------------------------

void Asura_GUIMenu_CommandManager::Shutdown( void)
{
#ifdef ASURA_TOOLS
	// can be NULL in non-tools builds
	Asura_Assert( m_pxActionButtonCommand != NULL, "Asura_GUIMenu_CommandManager::Shutdown - ActionCommandBuffer is not allocated.");
#endif
	if( m_pxActionButtonCommand != NULL)
	{
		delete []m_pxActionButtonCommand;
		m_pxActionButtonCommand = NULL;
	}

	m_uNumActionButtonCommands = 0;
}

// ---------------------------------------------------------------------------

bool Asura_GUIMenu_CommandManager::IsDefaultAcceptOveridden( void)
{
	u_int u;
	bool bOveride = false;

	for( u=0; u<m_uNumActionButtonCommands; u++)
	{
		if( m_pxActionButtonCommand[u].IsReplacementForAccept())
		{
			bOveride = true;
			break;
		}
	}

	return bOveride;

}

// ---------------------------------------------------------------------------

void Asura_GUIMenu_CommandManager::Execute( u_int uButtonAcceptType, bool bPlaySoundEffect, Asura_GUIMenu_Widget_Button* pxOwnerButton) const
{
	u_int u;
	u_int uSuccessfulCommandCount = 0;
	u_int uFailedCommandCount = 0;

	for( u=0; u<m_uNumActionButtonCommands; u++)
	{
		// Binary AND so that we can catch multiple button presses or overlapping mappings.
		if( m_pxActionButtonCommand[u].m_uButtonAction & uButtonAcceptType)
		{
			if( m_pxActionButtonCommand[u].IsReplacementForAccept() || 
				m_pxActionButtonCommand[u].IsUsedAsAccept())
			{
				pxOwnerButton->DoClickActions( ASURA_GUIMENU_ACTION_BUTTON_ACCEPT, true);
			}

			Asura_GUIMenu_ConsoleCmd& xCommand = m_pxActionButtonCommand[u].m_xCommand;
			// Make sure this is a valid command
			if( xCommand.m_xConsoleCmdName.GetLength() )
			{
				// Execute command
				if( xCommand.Execute(NULL) )
				{
					uSuccessfulCommandCount++;
				}
				else
				{
					uFailedCommandCount++;
				}
			}
		}
	}

	// 03.07.07 - CJW: Play only one sound effect.
	// Should there be both failing and successful commands then 
	// the sound being played defaults to success.
	if( bPlaySoundEffect )
	{
		if( uSuccessfulCommandCount > 0 )
		{
			// -- play the accept sound --
			Asura_GUIMenu_Audio::Play( ASURA_GUIMENU_AUDIO_TYPE_ON_ACCEPT);
		}
		else if( uFailedCommandCount > 0 )
		{
			// -- play the failure sound --
			Asura_GUIMenu_Audio::Play( ASURA_GUIMENU_AUDIO_TYPE_FAILURE);
		}
	}
}

// ---------------------------------------------------------------------------

// overloaded function that overrides the argument passed to the console cmd
void Asura_GUIMenu_CommandManager::Execute( u_int uButtonAcceptType, int iArgument, bool bPlaySoundEffect ) const
{
	u_int u;
	u_int uSuccessfulCommandCount = 0;
	u_int uFailedCommandCount = 0;

	char acBuffer[12];

	for( u=0; u<m_uNumActionButtonCommands; u++)
	{
		// Binary AND so that we can catch multiple button presses or overlapping mappings.
		if( m_pxActionButtonCommand[u].m_uButtonAction & uButtonAcceptType)
		{
			Asura_GUIMenu_ConsoleCmd& xCommand = m_pxActionButtonCommand[u].m_xCommand;

			// Make sure this is a valid command
			if( xCommand.m_xConsoleCmdName.GetLength() )
			{ 
				Asura_PrintfToBfr( acBuffer, "%d", 12, true, iArgument );

				// Execute command
				if( xCommand.Execute(acBuffer) )
				{
					uSuccessfulCommandCount++;
				}
				else
				{
					uFailedCommandCount++;
				}
			}
		}
	}

	// 09.07.07 - CJW: Play only one sound effect.
	// Should there be both failing and successful commands then 
	// the sound being played defaults to success.
	if( bPlaySoundEffect )
	{
		if( uSuccessfulCommandCount > 0 )
		{
			// -- play the accept sound --
			Asura_GUIMenu_Audio::Play( ASURA_GUIMENU_AUDIO_TYPE_ON_ACCEPT);
		}
		else if( uFailedCommandCount > 0 )
		{
			// -- play the failure sound --
			Asura_GUIMenu_Audio::Play( ASURA_GUIMENU_AUDIO_TYPE_FAILURE);
		}
	}
}

// ---------------------------------------------------------------------------

bool Asura_GUIMenu_CommandManager::ReadFromChunkStream( Asura_Chunk_Stream& xStream )
{
	u_int uVersion, u;
	xStream >> uVersion;
	if (uVersion > uASURA_GUIMENU_CONSOLEMANAGER_CHUNK_VERSION)
	{
		Asura_Assertf(false, ("GUIMenu Command Manager version unknown (%u > %u)", 
			uVersion, uASURA_GUIMENU_CONSOLEMANAGER_CHUNK_VERSION));
		return false;
	}

	xStream >> m_uNumActionButtonCommands;

#ifndef ASURA_TOOLS	// SK - This shouldn't be called in tool builds, hence the ifndef
	Initialise( m_uNumActionButtonCommands);
#endif

	for( u=0; u<m_uNumActionButtonCommands; u++)
	{
		xStream >> m_pxActionButtonCommand[u].m_uButtonAction;

		if( uVersion >= 1)
		{
			xStream >> m_pxActionButtonCommand[u].m_uButtonActionFlags;
		}
		m_pxActionButtonCommand[u].m_xCommand.ReadFromChunkStream( xStream);
	}

	return true;
}

// ---------------------------------------------------------------------------

#ifdef ASURA_TOOLS
void Asura_GUIMenu_CommandManager::WriteToChunkStream( Asura_Chunk_Stream& xStream ) const
{
	u_int u;

	xStream << uASURA_GUIMENU_CONSOLEMANAGER_CHUNK_VERSION;

	xStream << m_uNumActionButtonCommands;

	for( u=0; u<m_uNumActionButtonCommands; u++)
	{
		xStream << m_pxActionButtonCommand[u].m_uButtonAction;
		xStream << m_pxActionButtonCommand[u].m_uButtonActionFlags;
		m_pxActionButtonCommand[u].m_xCommand.WriteToChunkStream( xStream);
	}
}

void Asura_GUIMenu_CommandManager::ExportToXML( Tools_GUIMenu_XMLExporter* pxExporter ) const
{
	pxExporter->AddToBuffer( pxExporter->GetTabs() );
	pxExporter->AddToBuffer("<CommandManager>\n");
	pxExporter->IncTabs();

	pxExporter->AddToBuffer( pxExporter->GetTabs() );
	pxExporter->AddToBuffer( "<num_actionbutton_commands>" );
	pxExporter->AddToBuffer( m_uNumActionButtonCommands );
	pxExporter->AddToBuffer( "</num_actionbutton_commands>\n" );

	for( u_int u=0; u<m_uNumActionButtonCommands; u++)
	{
		pxExporter->AddToBuffer( pxExporter->GetTabs() );
		pxExporter->AddToBuffer( "<action_button num='" ); pxExporter->AddToBuffer( u ); pxExporter->AddToBuffer( "' >\n" );
		pxExporter->IncTabs();

		pxExporter->AddToBuffer( pxExporter->GetTabs() ); pxExporter->AddToBuffer( "<action>" );
		pxExporter->AddToBuffer( m_pxActionButtonCommand[u].m_uButtonAction ); pxExporter->AddToBuffer( "</action>\n" );

		pxExporter->AddToBuffer( pxExporter->GetTabs() ); pxExporter->AddToBuffer( "<action_flags>" );
		pxExporter->AddToBuffer( m_pxActionButtonCommand[u].m_uButtonActionFlags ); pxExporter->AddToBuffer( "</action_flags>\n" );

		m_pxActionButtonCommand[u].m_xCommand.ExportToXML( pxExporter );

		pxExporter->DecTabs();
		pxExporter->AddToBuffer( pxExporter->GetTabs() );
		pxExporter->AddToBuffer( "</action_button>\n" );		
	}

	pxExporter->DecTabs();
	pxExporter->AddToBuffer( pxExporter->GetTabs() );
	pxExporter->AddToBuffer("</CommandManager>\n");
}

// ---------------------------------------------------------------------------

void Asura_GUIMenu_CommandManager::AddOverlayControls(class Asura_Overlay* pxOverlay,class Asura_Vector_2& xOffset, bool bAllowArgs, bool bShowFlags, bool* pbChanged, bool* pbMaxActionsChanged)
{
	u_int u, v;
	Asura_Dialog* pxDialog;

#ifdef ASURA_TOOLS
	Asura_Char** ppcTextStrings = Asura_GUIMenu_System::GetMenuControlTextStrings();

	if( ppcTextStrings[Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_ACCEPT] != NULL)	g_axActionAcceptButtonLookUpNames[0] = ppcTextStrings[Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_ACCEPT];
	if( ppcTextStrings[Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_ALT_ACCEPT1] != NULL)	g_axActionAcceptButtonLookUpNames[1] = ppcTextStrings[Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_ALT_ACCEPT1];
	if( ppcTextStrings[Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_ALT_ACCEPT2] != NULL)	g_axActionAcceptButtonLookUpNames[2] = ppcTextStrings[Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_ALT_ACCEPT2];
	if( ppcTextStrings[Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_ALT_ACCEPT3] != NULL)	g_axActionAcceptButtonLookUpNames[3] = ppcTextStrings[Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_ALT_ACCEPT3];
	if( ppcTextStrings[Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_ALT_ACCEPT4] != NULL)	g_axActionAcceptButtonLookUpNames[4] = ppcTextStrings[Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_ALT_ACCEPT4];
	if( ppcTextStrings[Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_ALT_ACCEPT5] != NULL)	g_axActionAcceptButtonLookUpNames[5] = ppcTextStrings[Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_ALT_ACCEPT5];
	if( ppcTextStrings[Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_ALT_ACCEPT6] != NULL)	g_axActionAcceptButtonLookUpNames[6] = ppcTextStrings[Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_ALT_ACCEPT6];
	if( ppcTextStrings[Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_ALT_ACCEPT7] != NULL)	g_axActionAcceptButtonLookUpNames[7] = ppcTextStrings[Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_ALT_ACCEPT7];
	if( ppcTextStrings[Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_ALT_ACCEPT8] != NULL)	g_axActionAcceptButtonLookUpNames[8] = ppcTextStrings[Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_ALT_ACCEPT8];
	if( ppcTextStrings[Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_ALT_ACCEPT9] != NULL)	g_axActionAcceptButtonLookUpNames[9] = ppcTextStrings[Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_ALT_ACCEPT9];
#endif // ASURA_TOOLS


	xOffset.y += Asura_Dialog::SmallSpacing();
	// -- specify the amount of commands allowed, currently a max of 4 --
	pxDialog = new Asura_Numerical_Dialog("Number of Action Commands", xOffset, &m_uNumActionButtonCommands, 0, ASURA_GUIMENU_MAX_ALLOWED_MANAGER_COMMANDS, 1, "%d", pbMaxActionsChanged);
	pxDialog->SetHelpText("Action Commands Required for this widget.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	for( u=0; u<m_uNumActionButtonCommands; u++)
	{
		for( v=0; v<g_uNumActionAcceptButtonLookUp; v++)
		{
			if( m_pxActionButtonCommand[u].m_uButtonAction == g_axActionAcceptButtonLookUp[v].m_uValue)
			{
				m_pxActionButtonCommand[u].m_iToolActionIndex = v;
				break;
			}
		}

		pxDialog = new Asura_DropDown_List_Dialog( xOffset,
													&g_axActionAcceptButtonLookUpNames[0],
													&m_pxActionButtonCommand[u].m_iToolActionIndex,  
													g_uNumActionAcceptButtonLookUp/2,
													g_uNumActionAcceptButtonLookUp,
													NULL, pbChanged, aszACTIONACCEPTNAMES[u]);//name need to be in a static array, as unlike most dialog dropdowns don't make a copy

		Asura_DropDown_List_Dialog* pxDropList = static_cast<Asura_DropDown_List_Dialog*>(pxDialog);
		pxDropList->SetOnSelectCallback( AcceptTypeCallBack, (int)this);
		pxOverlay->AddDialog( pxDialog);
		xOffset.y += Asura_Dialog::DefaultSpacing();
	
		if( bShowFlags)
		{
			const int iRadioButtonGroupID = 101;

			pxDialog = new Asura_FlagTickbox_Dialog( "Use As 'Accept'", xOffset, &m_pxActionButtonCommand[u].m_uButtonActionFlags, Asura_GUIMenu_ActionButtonCommand::ASURA_GUIMENU_BUTTCOMM_USE_AS_ACCEPT, pbChanged);
			pxDialog->SetRadioButtonGroup(iRadioButtonGroupID,false);
			pxDialog->SetHelpText("If ticked, the specified 'Alt Accept' keypress press will be interpreted as an 'Accept' button press and will use the hotlink settings.  The normal 'Accept' button will still work");
			pxOverlay->AddDialog(pxDialog);
			xOffset.x += 15.0f;
			pxDialog = new Asura_FlagTickbox_Dialog( "Replace 'Accept'", xOffset, &m_pxActionButtonCommand[u].m_uButtonActionFlags, Asura_GUIMenu_ActionButtonCommand::ASURA_GUIMENU_BUTTCOMM_USE_REPLACE_DEFAULT_ACCEPT, pbChanged);
			pxDialog->SetRadioButtonGroup(iRadioButtonGroupID,false);
			pxDialog->SetHelpText("If ticked, the specified 'Alt Accept' keypress be use interpreted as the 'Accept' button and use the hotlink settings.  The default 'Accept' keypress will NOT work. ");
			pxOverlay->AddDialog(pxDialog);
			xOffset.x -= 15.0f;
			xOffset.y += Asura_Dialog::DefaultSpacing();
		}

		m_pxActionButtonCommand[u].m_xCommand.AddOverlayControls( pxOverlay, xOffset, "Console Cmd", 
			"The console command that will be executed when the button is pressed.", bAllowArgs, pbChanged);
	}
		xOffset.y += Asura_Dialog::SmallSpacing();
}

// ---------------------------------------------------------------------------

void Asura_GUIMenu_CommandManager::AcceptTypeCallBack( int iData)
{
	u_int u;
	Asura_GUIMenu_ActionButtonCommand* pxButtonCommand;
	Asura_GUIMenu_CommandManager* pxManager = reinterpret_cast<Asura_GUIMenu_CommandManager*>(iData);

	// SK - this is a bit of a cludge.  If any of the accept types are changed.
	// I have to make sure the others are correct by using the tool index and set the correct
	// data into uButtonAction.

	for( u=0; u<pxManager->GetActionButtonCount(); u++)
	{
		pxButtonCommand = pxManager->GetActionButtonCommandData( u);
		if( pxButtonCommand)
		{
			pxButtonCommand->m_uButtonAction = g_axActionAcceptButtonLookUp[pxButtonCommand->m_iToolActionIndex].m_uValue;
		}
	}
}

// ---------------------------------------------------------------------------

void Asura_GUIMenu_CommandManager::DefaultOutputCallback( const Asura_UnicodeChar* wszOutput )
{
#if _DEBUG
	// by default, echo the output to the console in debug.
	Asura_CommandConsole::Printf( L"GUIMenu command: %s", wszOutput );
#endif
}	

// ---------------------------------------------------------------------------

#endif // ASURA_TOOLS

u_int Asura_GUIMenu_CommandManager::GetActionButtonCount( void)
{
	return m_uNumActionButtonCommands;
}

// ---------------------------------------------------------------------------

void Asura_GUIMenu_CommandManager::Copy(const Asura_GUIMenu_CommandManager* pxCmdManager)
{
	u_int u;

	m_uNumActionButtonCommands = pxCmdManager->m_uNumActionButtonCommands;
	
	for( u=0; u<m_uNumActionButtonCommands; u++)
	{
#ifdef ASURA_TOOLS
		m_pxActionButtonCommand[u].m_iToolActionIndex = pxCmdManager->m_pxActionButtonCommand[u].m_iToolActionIndex;
#endif // ASURA_TOOLS
		m_pxActionButtonCommand[u].m_uButtonActionFlags	= pxCmdManager->m_pxActionButtonCommand[u].m_uButtonActionFlags;
		m_pxActionButtonCommand[u].m_uButtonAction = pxCmdManager->m_pxActionButtonCommand[u].m_uButtonAction;
		m_pxActionButtonCommand[u].m_xCommand.Copy( &pxCmdManager->m_pxActionButtonCommand[u].m_xCommand);
	}
}

// ---------------------------------------------------------------------------

void Asura_GUIMenu_CommandManager::SetActionButtonCount( u_int uAvailableActionButtons)
{
	Asura_Assert( uAvailableActionButtons < ASURA_GUIMENU_MAX_ALLOWED_MANAGER_COMMANDS, "Asura_GUIMenu_CommandManager::SetActionButtonCount - Max allowed action buttons is 4.");
	if( uAvailableActionButtons < ASURA_GUIMENU_MAX_ALLOWED_MANAGER_COMMANDS)
	{
		m_uNumActionButtonCommands = uAvailableActionButtons;
	}
}

// ---------------------------------------------------------------------------

Asura_GUIMenu_ActionButtonCommand* Asura_GUIMenu_CommandManager::GetActionButtonCommandData( u_int uIndex)
{
	Asura_Assert( uIndex < m_uNumActionButtonCommands, "Asura_GUIMenu_CommandManager::GetActionButtonCommandData - passed index is invalid.  Returning NULL pointer.");
	if( uIndex < m_uNumActionButtonCommands)
	{
		return &m_pxActionButtonCommand[uIndex];
	}

	return NULL;
}

// ---------------------------------------------------------------------------
// -- Private functions ------------------------------------------------------

void Asura_GUIMenu_CommandManager::ClearVariables( void)
{
	m_uNumActionButtonCommands	= 0;
	m_pxActionButtonCommand		= NULL;
}

// ---------------------------------------------------------------------------

#endif // ASURA_USE_GUI_MENU_SYSTEM
