//////////////////////////////////////////////////////////////////////////////
//
// Asura_GUIMenu_Command_Manager.h
//
// Copyright (c) 2007 Rebellion. All rights reserved.
//
// Description:
//
//     manager class to handle multiple commands based on defined button presses.
//    
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef ASURA_GUIMENU_COMMANDMANAGER_H___
#define ASURA_GUIMENU_COMMANDMANAGER_H___

#ifdef ASURA_USE_GUI_MENU_SYSTEM
#include "GUIMenu/Asura_GUIMenu_CommandConsole.h"

// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////////
class Asura_Chunk_Stream;

// D E F I N E S /////////////////////////////////////////////////////////////
#define ASURA_GUIMENU_MAX_ALLOWED_MANAGER_COMMANDS		(4)
// S T R U C T U R E S ///////////////////////////////////////////////////////
// Action buttons that are used in the structure below.
//ASURA_GUIMENU_ACTION_BUTTON_ACCEPT
//ASURA_GUIMENU_ACTION_BUTTON_ALT_ACCEPT1
//ASURA_GUIMENU_ACTION_BUTTON_ALT_ACCEPT2
//ASURA_GUIMENU_ACTION_BUTTON_ALT_ACCEPT3

class Asura_GUIMenu_ActionButtonCommand
{
public:
	// enums represent a bit index
	enum
	{
		ASURA_GUIMENU_BUTTCOMM_USE_AS_ACCEPT = (1<<0),
		ASURA_GUIMENU_BUTTCOMM_USE_REPLACE_DEFAULT_ACCEPT = (1<<1)
	};

	bool IsUsedAsAccept( void)	{	return ((m_uButtonActionFlags&ASURA_GUIMENU_BUTTCOMM_USE_AS_ACCEPT) != 0);	}
	bool IsReplacementForAccept( void)	{	return ((m_uButtonActionFlags&ASURA_GUIMENU_BUTTCOMM_USE_REPLACE_DEFAULT_ACCEPT) != 0);	}


public:

	u_int m_uButtonAction; // this is a value taken from the list of defines above.  These are defines in widget base
	u_int m_uButtonActionFlags;	// bit flag range for above enums.
	Asura_GUIMenu_ConsoleCmd	m_xCommand;
#ifdef ASURA_TOOLS
	int	 m_iToolActionIndex;
#endif
};


// C L A S S E S /////////////////////////////////////////////////////////////
class Asura_GUIMenu_Widget_Button;

class Asura_GUIMenu_CommandManager
{
public:
	Asura_GUIMenu_CommandManager();
	~Asura_GUIMenu_CommandManager();

	void	Initialise( u_int uMaxCommands );
	void	Shutdown( void);

	void	Execute( u_int uButtonAcceptType, bool bPlaySoundEffect, Asura_GUIMenu_Widget_Button* pxOwnerButton) const;
	void	Execute( u_int uButtonAcceptType, int iArgument, bool bPlaySoundEffect ) const;	// overloaded function that overrides the argument passed to the console cmd

    bool	ReadFromChunkStream( Asura_Chunk_Stream& xStream );
	void	SetActionButtonCount( u_int uAvailableActionButtons );
	Asura_GUIMenu_ActionButtonCommand* GetActionButtonCommandData( u_int uIndex );
	void	Copy(const Asura_GUIMenu_CommandManager* pxCmdManager);

	bool	IsDefaultAcceptOveridden( void);

#ifdef ASURA_TOOLS
	void	WriteToChunkStream( Asura_Chunk_Stream& xStream ) const;
	void	ExportToXML(Tools_GUIMenu_XMLExporter* pxExporter) const;
	void	AddOverlayControls(class Asura_Overlay* pxOverlay,class Asura_Vector_2& xOffset, bool bAllowArgs, bool bShowFlags, bool* pbChanged, bool* pbMaxActionsChanged);
#endif

	u_int	GetActionButtonCount( void);

private:
	void	ClearVariables( void);

	static void		AcceptTypeCallBack( int iData);
	static void		DefaultOutputCallback( const Asura_UnicodeChar* wszOutput );

private:
	Asura_GUIMenu_ActionButtonCommand*		m_pxActionButtonCommand;
	u_int									m_uNumActionButtonCommands;

#ifdef ASURA_TOOLS
	int	m_iSelectedActionType;
#endif
};


// E N D   F I L E   W R A P P E R ///////////////////////////////////////////
#endif // ASURA_USE_GUI_MENU_SYSTEM
#endif // #ifndef ASURA_GUIMENU_COMMANDMANAGER_H___
