// ------------------------------------------------------------------------------------ //
//																						//
//	File: Asura_GUIMenu_Element_Base.h													//
//																						//
//	Description: base class for all elements											//
//																						//
//	Created: Steve Keen - 9/11/06														//
//																						//
//	Copyright (C) 2006 Rebellion (Derby), All Rights Reserved.							//
// ------------------------------------------------------------------------------------ //

#ifndef __ASURA_GUIMENU_ELEMENT_BASE_H__
#define __ASURA_GUIMENU_ELEMENT_BASE_H__

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifdef ASURA_USE_GUI_MENU_SYSTEM

#include "Asura_GUIMenu_Root.h"

// -- Typedef -------------------------------------------------------------------------

// -- Enums ---------------------------------------------------------------------------

// -- Defines -------------------------------------------------------------------------
//#define ASURA_CHUNK_ELEMENT_BASE_VERSION_NEW 0

// these are the number allocated when in tools - non-tools builds allocate just the right number
#define ASURA_GUIMENU_MAX_ALLOWED_MODIFIERS	8
#define ASURA_GUIMENU_MAX_ALLOWED_ANIMATORS 8

#define ASURA_GUIMENU_ELEMENT_BASE_LOCAL_FLAGS_START	8

// -- Data Chunk Version --
// Versions: ASURA_CHUNK_ELEMENT_IMAGE_VERSION
// 0:	Initial data
// 1:   Added Percentage values used for scaling, also added new scaling flags u_int - SK 26.11.07
// 2:	? - NDB 130508
// 3:	Moved scaling flags to root class and removed Percentage values used for scaling - NDB 130508
// 4:	position/scaling fixup - NY 31/07/2008
#define ASURA_CHUNK_ELEMENT_BASE_VERSION 4

// -- Structures ----------------------------------------------------------------------

struct Asura_GUIMenu_UV
{
	void Zero()
	{
		u = 0.0f;
		v = 0.0f;
	}

	bool operator==( const Asura_GUIMenu_UV& xOther ) const { return (u == xOther.u) && (v == xOther.v); }
	bool operator!=( const Asura_GUIMenu_UV& xOther ) const { return !operator==(xOther); }

	float u;
	float v;
};

struct Asura_GUIMenu_UV_Set
{
	void Zero()
	{
		xUV0.Zero();
		xUV1.Zero();
	}

	Asura_GUIMenu_UV xUV0;
	Asura_GUIMenu_UV xUV1;
};


// -- Class ---------------------------------------------------------------------------

class Asura_GUIMenu_Element_Base : public Asura_GUIMenu_Root
{
	typedef Asura_GUIMenu_Root PARENT;

// -- Functions -----------------------------------------------------------------------
public:
	using PARENT::Asura_GUIMenu_Scaling_Flag;

	enum Asura_GUIMenu_Overide_Element_Base_Type
	{
		ASURA_GUIMENU_ELEMENT_BASE_LOCAL_FLAG_WIDTH	= (1<<(ASURA_GUIMENU_ELEMENT_BASE_LOCAL_FLAGS_START+0)),
		ASURA_GUIMENU_ELEMENT_BASE_LOCAL_FLAG_HEIGHT	= (1<<(ASURA_GUIMENU_ELEMENT_BASE_LOCAL_FLAGS_START+1)),

		//ASURA_GUIMENU_ELEMENT_BASE_INHERIT_FLAG_MAX is here to stop another programmer adding past bit flag 15
		ASURA_GUIMENU_ELEMENT_BASE_LOCAL_FLAG_MAX		= (1<<(ASURA_GUIMENU_ELEMENT_BASE_LOCAL_FLAGS_START+8))

	};

	Asura_GUIMenu_Element_Base();
	virtual ~Asura_GUIMenu_Element_Base();

	virtual void				Initialise();
	virtual void				Update( float fElapsedTime);
	virtual void 				Render( u_int uState);
	virtual void				Render( u_int uState, const Asura_Bounding_Rectangle& xBounds) = 0;


	virtual bool				Whisper( const Asura_Message_GUIMenu_Notify* pxNotify);

	virtual void				ReadFromChunkStream( Asura_Chunk_Stream& xStream, Asura_GUIMenu_Root* pxParentObject, u_int uChunkVersion);
#ifdef ASURA_TOOLS
	virtual void 				UpdateLocalDataFromStyle();

	virtual void				WriteToChunkStream( Asura_Chunk_Stream& xStream) const;
	virtual void				ExportToXML(Tools_GUIMenu_XMLExporter* pxExporter) const;

#endif
	void SetStateOverride(u_int uState) { m_uStateOverride = uState;}
	u_int GetStateOverride() const { return m_uStateOverride;}
protected:
	u_int m_uStateOverride;
};
#endif // ASURA_USE_GUI_MENU_SYSTEM
#endif // __ASURA_GUIMENU_ELEMENT_BASE_H__
