// ------------------------------------------------------------------------------------ //
//																						//
// Asura_GUIMenu_Element_FMV.cpp															//
//																						//
// Created by Martin Hutchings - 24/03/2009												//
//																						//
// Copyright (C) 2009 Rebellion, All Rights Reserved.									//
//																						//
// ------------------------------------------------------------------------------------ //

#include "Core/Asura.h"
#include "GUIMenu/Asura_GUIMenu_Element_FMV.h"

#ifdef ASURA_USE_GUI_MENU_SYSTEM

#include "Core/Asura_TextureManagement.h"
#include "Core/Asura_Timers.h"
#ifdef ASURA_USE_VIDEO_TO_TEXTURE_SYSTEM
	#include "FMV/Asura_VideoToTexture.h"
#endif
#include "GUIMenu/Asura_GUIMenu_Animator.h"
#include "GUIMenu/Asura_GUIMenu_Menu_Page.h"
#include "GUIMenu/Asura_GUIMenu_Modifier.h"
#include "GUIMenu/Asura_Profiling_GUIMenu.h"
#include "Rendering/Asura_Render.h"
#include "Rendering/Asura_Text_Output.h"

#ifdef ASURA_TOOLS
#include "GUIMenu/Tools_GUIMenu_XMLExporter.h"
#endif

#ifndef MAX_GUIMENU_PLAYING_FMV
#define MAX_GUIMENU_PLAYING_FMV	(1)
#endif

Asura_LinkList<Asura_GUIMenu_FMV_System::Asura_GUIMenu_FMV*> Asura_GUIMenu_FMV_System::s_xRegisteredFMV;	// Running 
Asura_Handle Asura_GUIMenu_FMV_System::s_iNextHandle = 1;
u_int Asura_GUIMenu_FMV_System::s_uNumMoviesPlaying = 0;

void Asura_GUIMenu_FMV_System::Initialise(void)
{
	s_uNumMoviesPlaying = 0;
}

void Asura_GUIMenu_FMV_System::Reset(void)
{
	Shutdown();
}

void Asura_GUIMenu_FMV_System::Shutdown(void)
{
	// Clear this list manually, since we need to delete the data.
	Asura_ListElt<Asura_GUIMenu_FMV*>* pxElement = s_xRegisteredFMV.Front();
	while (pxElement)
	{
		Asura_ListElt<Asura_GUIMenu_FMV*>* pxCurrentElement = pxElement;
		pxElement = pxElement->Next();
		Asura_GUIMenu_FMV* pxData = pxCurrentElement->Data();
		s_xRegisteredFMV.Remove(pxCurrentElement);

#ifdef ASURA_USE_VIDEO_TO_TEXTURE_SYSTEM
		delete pxData->m_pxVideoToTexture;
#endif

		delete pxData;
	}

	s_uNumMoviesPlaying = 0;
}

void Asura_GUIMenu_FMV_System::Update(void)
{
#ifdef ASURA_USE_VIDEO_TO_TEXTURE_SYSTEM
	Asura_ListElt<Asura_GUIMenu_FMV*>* pxElement;
	for (pxElement = s_xRegisteredFMV.Front(); pxElement; pxElement = pxElement->Next())
	{
		Asura_GUIMenu_FMV* pxData = pxElement->Data();

		if ( pxData->m_pxVideoToTexture != NULL )
		{
			if ( pxData->m_pxVideoToTexture->GetPlayerState() == Asura_VideoToTexture::PLAYER_STATE_FILE_OPENED )
			{
				 // Make sure play is called on the movie.
				 pxData->m_pxVideoToTexture->Play();
			}

			// Now check if anything has used this movie recently
			if ( pxData->m_uHeartBeatFrameNum + 4 < Asura_Timers::GetFrameNumber() )
			{
				// The FMV hasn't been poked in a while so it really needs to be shutdown.
				// Irrespective of the play count the FMV needs to be stopped as nothing
				// has tried to use it recently.
				pxData->m_iPlayCount = 0;
				s_uNumMoviesPlaying--;
				delete pxData->m_pxVideoToTexture;
				pxData->m_pxVideoToTexture = NULL;
			}
		}
	}
#endif
}

// Register an FMV with the system. A unique FMV is identified using
// its name, the texture it will render to while playing, if the sound is enabled,
// the sound track it plays and if the FMV loops.
// Each unique FMV is assigned a handle returned from this function.
// If a duplicate FMV is registered than the handle to the existing FMV is returned.
// Each time an FMV is registered the handle has its reference count increased.
// Every call to RegisterFMV must match with a call to DeregisterFMV.
// When the FMV is no longer needed its is destroyed.

// An FMV with the same name can be registered again if for example the texture index
// was different. This allows the same FMV to be played to two different textures. This also
// includes playing different sound tracks.
Asura_Handle Asura_GUIMenu_FMV_System::RegisterFMV(Asura_String xFilename,
													int iTextureRTIndex, 
													u_int uSoundTrackIndex,
													bool bPlaySound,
													bool bLoop)
{
#ifdef ASURA_USE_VIDEO_TO_TEXTURE_SYSTEM
	// Build the VideoToTexture flags
	u_int uFlags = Asura_VideoToTexture::ASURA_FMV_FLAG_DEFAULT;

	if (bLoop == true)
	{
		uFlags |= Asura_VideoToTexture::ASURA_FMV_FLAG_LOOP;
	}

	if (bPlaySound == false)
	{
		uFlags |= Asura_VideoToTexture::ASURA_FMV_FLAG_NO_SOUND;
	}

	Asura_VideoToTexture::ASURA_FMV_FLAGS eFlags = static_cast<Asura_VideoToTexture::ASURA_FMV_FLAGS>(uFlags);

	Asura_Hash_ID uFilenameHash = Asura_GetHashID(xFilename.GetString());

	// Check to see if FMV is already registered
	Asura_ListElt<Asura_GUIMenu_FMV*>* pxElement;
	for (pxElement = s_xRegisteredFMV.Front(); pxElement; pxElement = pxElement->Next())
	{
		Asura_GUIMenu_FMV* pxData = pxElement->Data();

		if (pxData->m_uFilenameHash == uFilenameHash &&
			pxData->m_iTextureRTIndex == iTextureRTIndex &&
			pxData->m_eFlags == eFlags &&
			pxData->m_uSoundTrackIndex == uSoundTrackIndex )
		{
			// Movie already registered. Increase the ref count
			pxData->m_iRegisterCount++;
			return pxData->m_iHandle;
		}

	}

	// Create a new FMV and add it to the registered list
	Asura_GUIMenu_FMV* pxFMV = new Asura_GUIMenu_FMV;

	if (pxFMV == NULL )
	{
		return GUIMENU_FMV_HANDLE_NONE;
	}

#ifdef ASURA_USE_VIDEO_TO_TEXTURE_SYSTEM
	pxFMV->m_pxVideoToTexture = NULL;
	pxFMV->m_eFlags = eFlags;
#endif

	pxFMV->m_uFilenameHash = uFilenameHash;
	pxFMV->m_xFilename = xFilename;
	pxFMV->m_iTextureRTIndex = iTextureRTIndex;

	pxFMV->m_iRegisterCount = 1;
	pxFMV->m_iPlayCount = 0;
	pxFMV->m_iHandle = s_iNextHandle;
	pxFMV->m_uSoundTrackIndex = uSoundTrackIndex;
	pxFMV->m_pxVideoToTexture = NULL;
	pxFMV->m_uHeartBeatFrameNum = 0;

	s_xRegisteredFMV.Push_Back(pxFMV);

	// Increment the next available handle
	s_iNextHandle++;

	return pxFMV->m_iHandle;
#else
	return GUIMENU_FMV_HANDLE_NONE;
#endif // #ifdef ASURA_USE_VIDEO_TO_TEXTURE_SYSTEM
}

// Call to show the system is no longer interested in the FMV represent by the handle.
// If the reference count on the FMV reaches 0 then the FMV object is destroyed.
void Asura_GUIMenu_FMV_System::DeregisterFMV(Asura_Handle& iHandle)
{
	Asura_GUIMenu_FMV* pxFMV = FindFMV(iHandle);

	if ( pxFMV == NULL )
	{
		return;
	}

	pxFMV->m_iRegisterCount--;

	if ( pxFMV->m_iRegisterCount <= 0 )
	{
		// Delete the object associated with this handle
		// After this call pxFMV is no longer valid
		DeleteFMV(iHandle);
		pxFMV = NULL; // Pointer no longer valid. Just to make it safe
	}

	iHandle = GUIMENU_FMV_HANDLE_NONE;
}

Asura_GUIMenu_FMV_System::Asura_GUIMenu_FMV* Asura_GUIMenu_FMV_System::FindFMV(Asura_Handle iHandle)
{
	// Search the registered list for an FMV with the matching Handle.
	Asura_ListElt<Asura_GUIMenu_FMV*>* pxElement;
	for (pxElement = s_xRegisteredFMV.Front(); pxElement; pxElement = pxElement->Next())
	{
		Asura_GUIMenu_FMV* pxData = pxElement->Data();

		if ( pxData->m_iHandle == iHandle )
		{
			return pxData;
		}
	}

	return NULL;
}

void Asura_GUIMenu_FMV_System::DeleteFMV(Asura_Handle iHandle)
{
	// Delete the FMV object with the matching Handle.
	Asura_ListElt<Asura_GUIMenu_FMV*>* pxElement;

	for (pxElement = s_xRegisteredFMV.Front(); pxElement; pxElement = pxElement->Next())
	{
		Asura_GUIMenu_FMV* pxData = pxElement->Data();
		if ( pxData->m_iHandle == iHandle )
		{
			s_xRegisteredFMV.Remove(pxElement);

#ifdef ASURA_USE_VIDEO_TO_TEXTURE_SYSTEM
			if ( pxData->m_pxVideoToTexture != NULL )
			{
				s_uNumMoviesPlaying--;
				delete pxData->m_pxVideoToTexture;
			}
#endif

			delete pxData;
			return;
		}
	}
}

// Is the FMV currently playing.
bool Asura_GUIMenu_FMV_System::IsPlaying(Asura_Handle iHandle)
{
	Asura_GUIMenu_FMV* pxFMV = FindFMV(iHandle);

	if ( pxFMV == NULL )
	{
		return false;
	}

#ifdef ASURA_USE_VIDEO_TO_TEXTURE_SYSTEM
	if ( pxFMV->m_pxVideoToTexture == NULL )
	{
		// No VideoToTextureso can't be playing
		return false;
	}

	// VideoToTexture already open
	if ( pxFMV->m_pxVideoToTexture->GetPlayerState() == Asura_VideoToTexture::PLAYER_STATE_PLAYING )
	{
		return true;
	}
#endif //#ifdef ASURA_USE_VIDEO_TO_TEXTURE_SYSTEM

	return false;
}

// Cause the FMV to start playing. If the FMV is already playing then it continues to play from where it is.
// This call increases the play reference count. A matching call to stop should be made for each call to Play, 
// but is not essential as the system will stop any FMV's playing that haven't received a HeartBeat (See below).
// If the FMV isn't playing the a new VideoToTexture object is created, which creates the streaming buffers and
// the texture for the FMV.
bool Asura_GUIMenu_FMV_System::Play(Asura_Handle iHandle)
{
	Asura_GUIMenu_FMV* pxFMV = FindFMV(iHandle);

	if ( pxFMV == NULL )
	{
		return false;
	}

#ifdef ASURA_USE_VIDEO_TO_TEXTURE_SYSTEM
	if ( pxFMV->m_pxVideoToTexture != NULL )
	{
		// VideoToTexture already open and playing
		pxFMV->m_uHeartBeatFrameNum = Asura_Timers::GetFrameNumber();
		pxFMV->m_iPlayCount++;
		return true;
	}

	// About to start up a new move and create a new VideoToTexture object.
	// Before do that check if there is already a running movie that is using the 
	// texture for this one.
	// If another movie is currently using this texture don't allow this
	// otherwise two movies will attempt to render to the same texture
	// and bad things will happen.
	Asura_ListElt<Asura_GUIMenu_FMV*>* pxElement;
	for (pxElement = s_xRegisteredFMV.Front(); pxElement; pxElement = pxElement->Next())
	{
		Asura_GUIMenu_FMV* pxData = pxElement->Data();

		if (  pxData != pxFMV &&
			  pxData->m_iTextureRTIndex == pxFMV->m_iTextureRTIndex )
		{
			// There is a different movie that is assigned the same texture
			// Make sure it hasn't got a VideoToTexture object
			if (  pxData->m_pxVideoToTexture != NULL )
			{
				return false;
			}
		}
	}

	if ( s_uNumMoviesPlaying >= MAX_GUIMENU_PLAYING_FMV )
	{
		// There are too many movies playing already.
		return false;
	}

	// Need to open the movie file and start playing it. 
	Asura_VideoToTexture* pxVideoToTexture = new Asura_VideoToTexture;

	if ( pxVideoToTexture != NULL )
	{
		pxVideoToTexture->SetPlayFlags(pxFMV->m_eFlags);

		if ( pxVideoToTexture->Open(pxFMV->m_xFilename.GetString(),
								pxFMV->m_iTextureRTIndex,
								pxFMV->m_uSoundTrackIndex) == true )
		{
			pxFMV->m_uHeartBeatFrameNum = Asura_Timers::GetFrameNumber();
			pxFMV->m_pxVideoToTexture = pxVideoToTexture;
			pxFMV->m_iPlayCount++;
			s_uNumMoviesPlaying++;
			return true;
		}

		// Movie failed to open so destroy the VideoToTexture;
		delete pxVideoToTexture;
	}
#endif //#ifdef ASURA_USE_VIDEO_TO_TEXTURE_SYSTEM

	return false;
}

// Tell the FMV to stop playing. This decrements the Play count. If the play count reaches 0 then the FMV is stopped,
// otherwise it will continue to play as something else has also called the Play function.
// Once the FMV is stopped the VideoToTexture object, along with the streaming buffers and texture are destroyed.
void Asura_GUIMenu_FMV_System::Stop(Asura_Handle iHandle)
{
	Asura_GUIMenu_FMV* pxFMV = FindFMV(iHandle);

	if ( pxFMV == NULL )
	{
		return;
	}

#ifdef ASURA_USE_VIDEO_TO_TEXTURE_SYSTEM
	if ( pxFMV->m_pxVideoToTexture == NULL )
	{
		return;
	}

	pxFMV->m_iPlayCount--;

	if ( pxFMV->m_iPlayCount <= 0 )
	{
		s_uNumMoviesPlaying--;
		delete pxFMV->m_pxVideoToTexture;
		pxFMV->m_pxVideoToTexture = NULL;
	}
#endif

}

// Stop all FMV's from playing irrespective of their play count and free up all the memory used by the
// VideoToTexture objects.
void Asura_GUIMenu_FMV_System::StopAllForced()
{
#ifdef ASURA_USE_VIDEO_TO_TEXTURE_SYSTEM
	Asura_ListElt<Asura_GUIMenu_FMV*>* pxElement;
	for (pxElement = s_xRegisteredFMV.Front(); pxElement; pxElement = pxElement->Next())
	{
		Asura_GUIMenu_FMV* pxData = pxElement->Data();

		if ( pxData->m_pxVideoToTexture != NULL )
		{
			pxData->m_iPlayCount = 0;
			s_uNumMoviesPlaying--;
			delete pxData->m_pxVideoToTexture;
			pxData->m_pxVideoToTexture = NULL;
		}
	}
#endif
}

#ifdef ASURA_USE_VIDEO_TO_TEXTURE_SYSTEM
// Get the VideoToTexture object. If the FMV hasn't been opened then this will be NULL.
// This also cause a heartbeat so the VideoToTexture object doesn't get destroyed.
// Don't keep the pointer returned from the function. Only use locally. The object
// maybe destroyed without notice.
Asura_VideoToTexture* Asura_GUIMenu_FMV_System::GetVideoToTexture(Asura_Handle iHandle)
{
	Asura_GUIMenu_FMV* pxFMV = FindFMV(iHandle);

	if ( pxFMV == NULL )
	{
		return NULL;
	}

	pxFMV->m_uHeartBeatFrameNum = Asura_Timers::GetFrameNumber();

	return pxFMV->m_pxVideoToTexture;
}
#endif

// Check that the handle is still valid. If the handle isn't valid then the FMV is no longer register with the system.
bool Asura_GUIMenu_FMV_System::ValidateHandle(Asura_Handle& iHandle)
{
	Asura_GUIMenu_FMV* pxFMV = FindFMV(iHandle);

	if ( pxFMV == NULL )
	{
		iHandle = Asura_GUIMenu_FMV_System::GUIMENU_FMV_HANDLE_NONE;
		return false;
	}

	return true;
}

// Keep the FMV playing. If the FMV is playing and hasn't been pulsed within the last few frames then
// it is automatically stopped by the system. This in turn will destroy the VideoToTexture object 
// if nothing else is playing the movie.
// This prevents the GUIMenu system from playing movies that aren't been used. i.e. Rendered. 
void Asura_GUIMenu_FMV_System::HeartBeat(Asura_Handle iHandle)
{
	Asura_GUIMenu_FMV* pxFMV = FindFMV(iHandle);

	if ( pxFMV == NULL )
	{
		return;
	}

	// Update the frame count for this FMV. It lets the FMV know that its
	// still needed, otherwise the system will shutdown the FMV after a while
	pxFMV->m_uHeartBeatFrameNum = Asura_Timers::GetFrameNumber();
}

void Asura_GUIMenu_FMV_System::DebugOutput()
{
#ifdef ASURA_USE_VIDEO_TO_TEXTURE_SYSTEM
	float fY = 1.0f;

	Asura_ListElt<Asura_GUIMenu_FMV*>* pxElement;
	for (pxElement = s_xRegisteredFMV.Front(); pxElement; pxElement = pxElement->Next())
	{
		Asura_GUIMenu_FMV* pxData = pxElement->Data();

		if ( pxData->m_pxVideoToTexture != NULL )
		{
			Asura_Text_Output::Print(3.0f,fY,0xff00ff00,0,"%s %d %d %d (Playing)", pxData->m_xFilename.GetString(),
					pxData->m_iTextureRTIndex,pxData->m_iRegisterCount, pxData->m_iPlayCount);
		}
		else
		{
			Asura_Text_Output::Print(3.0f,fY,0xff00ff00,0,"%s %d %d %d", pxData->m_xFilename.GetString(),
				pxData->m_iTextureRTIndex,pxData->m_iRegisterCount, pxData->m_iPlayCount);
		}

		fY++;
	}
#endif
}

// --------------------------------------------------------------------------------------- 
// -- Constructors -----------------------------------------------------------------------

Asura_GUIMenu_Element_FMV::Asura_GUIMenu_Element_FMV(void)
{
	m_uType = ASURA_GUIMENU_TYPEID_ELEMENT_FMV;

	ASURA_GUIMENU_PROFILE_COUNTINSTANCE(ASURA_GUIMENU_COUNTER__ELEMENT_FMV);

	m_iHandle = Asura_GUIMenu_FMV_System::GUIMENU_FMV_HANDLE_NONE;
	m_bPlaying = false;

	m_uSoundTrackIndex = 0;
	m_bPlaySound = true;
	m_bLoop = false;
}

// --------------------------------------------------------------------------------------- 
// -- Destructor -------------------------------------------------------------------------

Asura_GUIMenu_Element_FMV::~Asura_GUIMenu_Element_FMV(void)
{
	ASURA_GUIMENU_PROFILE_COUNTDESTROYINSTANCE(ASURA_GUIMENU_COUNTER__ELEMENT_FMV);

	// Stop the FMV if its playing
	Stop();
	// Deregister the interest in this FMV
	Asura_GUIMenu_FMV_System::DeregisterFMV(m_iHandle);
}

// --------------------------------------------------------------------------------------- 
// -- Public Functions ------------------------------------------------------------------- 

void Asura_GUIMenu_Element_FMV::Copy( const Asura_GUIMenu_Root* pxSource )
{
	PARENT::Copy( pxSource );

	const Asura_GUIMenu_Element_FMV* pxSourceImage = static_cast<const Asura_GUIMenu_Element_FMV*> (pxSource);

	// Copy the FMV data
	m_xFMVName = pxSourceImage->m_xFMVName;
	m_uSoundTrackIndex = pxSourceImage->m_uSoundTrackIndex;
	m_bPlaySound = pxSourceImage->m_bPlaySound;
	m_bLoop = pxSourceImage->m_bLoop;

	RegisterFMV();
}

// Register the FMV data with the GUIMenu's FMV system.
// This must be called after and changed to the FMV data are made.
void Asura_GUIMenu_Element_FMV::RegisterFMV()
{
	// Deregister any previous handle
	Asura_GUIMenu_FMV_System::DeregisterFMV(m_iHandle);

	// Register a new FMV as long as a filename and texture have been selected
	if ( GetTexture() != ASURA_TEXTURE_NONE &&
		 m_xFMVName.IsBlank() == false )
	{
		m_iHandle = Asura_GUIMenu_FMV_System::RegisterFMV(m_xFMVName.GetString(),
															GetTexture(),
															m_uSoundTrackIndex,
															m_bPlaySound,
															m_bLoop);
	}
}

void Asura_GUIMenu_Element_FMV::SetFMVFilename(Asura_String xFilename)
{
	m_xFMVName = xFilename;
}

// Start the FMV playing.
// Checks if the FMV is currently playing and if not will start it as
// long as the element is going to be rendered.
void Asura_GUIMenu_Element_FMV::Play()
{
	if ( m_bPlaying == true )
	{
		m_bPlaying = Asura_GUIMenu_FMV_System::IsPlaying(m_iHandle);
	}

	if ( m_bPlaying == false )
	{
		Asura_GUIMenu_FMV_System::ValidateHandle(m_iHandle);

		if ( IsHiddenByConditionVar() == false && 
			 IsRenderingDisabled() == false )
		{
			if ( m_iHandle != Asura_GUIMenu_FMV_System::GUIMENU_FMV_HANDLE_NONE )
			{
				m_bPlaying = Asura_GUIMenu_FMV_System::Play(m_iHandle);
			}
		}
	}
}

void Asura_GUIMenu_Element_FMV::Stop()
{
	if ( m_iHandle != Asura_GUIMenu_FMV_System::GUIMENU_FMV_HANDLE_NONE &&
		 m_bPlaying == true )
	{
		Asura_GUIMenu_FMV_System::Stop(m_iHandle);
		m_bPlaying = false;
	}
}

// --------------------------------------------------------------------------------------- 

void Asura_GUIMenu_Element_FMV::Update( float fElapsedTime)	//, unsigned int uButtons, POINT ptMousePos)
{
	ASURA_GUIMENU_PROFILE_START_TASK(ASURA_GUIMENU_PROFILE__ELEMENT_FMV);

	PARENT::Update( fElapsedTime);

	// If the FMV isn't playing then start it.
	Play();

	// If the FMV is playing and shouldn't be then stop it.
	if ( m_bPlaying == true )
	{
		if ( IsHiddenByConditionVar() == true || 
			IsRenderingDisabled() == true )
		{
			Stop();
		}
	}

	ASURA_GUIMENU_PROFILE_END_TASK();
}

// --------------------------------------------------------------------------------------- 

void Asura_GUIMenu_Element_FMV::Render( u_int uState)
{
	PARENT::Render( uState);
}


// --------------------------------------------------------------------------------------- 
void Asura_GUIMenu_Element_FMV::Render( u_int uState, const Asura_Bounding_Rectangle& xBounds)
{
	ASURA_GUIMENU_PROFILE_START_TASK(ASURA_GUIMENU_PROFILE__ELEMENT_FMV);

	// If the FMV is playing then keep it alive otherwise the system will stop it.
	if ( m_bPlaying == true )
	{
		Asura_GUIMenu_FMV_System::HeartBeat(m_iHandle);
	}

	PARENT::Render(uState,xBounds);

	ASURA_GUIMENU_PROFILE_END_TASK();
}

#ifdef ASURA_TOOLS

bool Asura_GUIMenu_Element_FMV::IsFMVValid()
{
	return Asura_GUIMenu_FMV_System::ValidateHandle(m_iHandle);
}

// --------------------------------------------------------------------------------------- 
void Asura_GUIMenu_Element_FMV::UpdateLocalDataFromStyle( void)
{
	Asura_GUIMenu_Element_FMV* pxStyleImage;

	PARENT::UpdateLocalDataFromStyle();

	if( m_pxStyleObject != NULL)
	{
		pxStyleImage = static_cast<Asura_GUIMenu_Element_FMV*>(m_pxStyleObject);

		// Copy the FMV data from the style
		m_xFMVName = pxStyleImage->m_xFMVName;
		m_uSoundTrackIndex = pxStyleImage->m_uSoundTrackIndex;
		m_bPlaySound = pxStyleImage->m_bPlaySound;
		m_bLoop = pxStyleImage->m_bLoop;

		// Update the FMV register
		RegisterFMV();
	}
}
#endif
// --------------------------------------------------------------------------------------- 
// -- Override tool helper function -------------------------------------------------------

void Asura_GUIMenu_Element_FMV::ReadFromChunkStream( Asura_Chunk_Stream& xStream, Asura_GUIMenu_Root* pxParentObject, u_int uChunkVersion)
{
	char acName[256];

	PARENT::PARENT::ReadFromChunkStream( xStream, pxParentObject, 0xffffffff);

	// read in image element specific data --
	xStream >> m_axUV[0].u;
	xStream >> m_axUV[0].v;
	xStream >> m_axUV[1].u;
	xStream >> m_axUV[1].v;

	xStream >> m_uHashTextureName;

	Asura_String xTextureName;
	xStream.ReadString( acName, sizeof(acName));
	xTextureName = acName;
	SetTexture( xTextureName, true); // this method sets tile data members to safe defaults	

	xStream >> m_fRotation;

	xStream.ReadString( acName, sizeof(acName));
	m_xFMVName = acName;

	xStream >> m_uSoundTrackIndex;

	xStream >> m_bPlaySound;
	xStream >> m_bLoop;

	// -- read in the container info
	ReadAllChildObjects( xStream, this);

	RegisterFMV();
}

// --------------------------------------------------------------------------------------- 

#ifdef ASURA_TOOLS
void Asura_GUIMenu_Element_FMV::WriteToChunkStream( Asura_Chunk_Stream& xStream) const
{
	WriteLocalChunkHeader( xStream, ASURA_CHUNK_ELEMENT_FMV_VERSION, GetType());

	// -- Write out grandparent data - Asura_GUIMenu_Element_Base --
	PARENT::PARENT::WriteToChunkStream( xStream);

	xStream << m_axUV[0].u;
	xStream << m_axUV[0].v;
	xStream << m_axUV[1].u;
	xStream << m_axUV[1].v;

	xStream << m_uHashTextureName;
	xStream << m_xTextureName;

	xStream << m_fRotation;

	xStream << m_xFMVName;
	xStream << m_uSoundTrackIndex;

	xStream << m_bPlaySound;
	xStream << m_bLoop;

	// -- write out any children --
	WriteChildObjectData( xStream);
}

void Asura_GUIMenu_Element_FMV::ExportToXML(Tools_GUIMenu_XMLExporter* pxExporter) const
{
	PARENT::PARENT::ExportToXML( pxExporter );

	pxExporter->AddToBuffer( pxExporter->GetTabs() );
	pxExporter->AddToBuffer( "<FMV_data>\n" );
	pxExporter->IncTabs();

	pxExporter->AddToBuffer( pxExporter->GetTabs() );
	pxExporter->AddToBuffer("<uv_set>\n");
	pxExporter->IncTabs();

	pxExporter->AddToBuffer( pxExporter->GetTabs() ); pxExporter->AddToBuffer( "<uv0_u>" ); 
	pxExporter->AddToBuffer( m_axUV[0].u ); pxExporter->AddToBuffer( "</uv0_u>\n" );

	pxExporter->AddToBuffer( pxExporter->GetTabs() ); pxExporter->AddToBuffer( "<uv0_v>" ); 
	pxExporter->AddToBuffer( m_axUV[0].v ); pxExporter->AddToBuffer( "</uv0_v>\n" );

	pxExporter->AddToBuffer( pxExporter->GetTabs() ); pxExporter->AddToBuffer( "<uv1_u>" ); 
	pxExporter->AddToBuffer( m_axUV[1].u ); pxExporter->AddToBuffer( "</uv1_u>\n" );

	pxExporter->AddToBuffer( pxExporter->GetTabs() ); pxExporter->AddToBuffer( "<uv1_v>" ); 
	pxExporter->AddToBuffer( m_axUV[1].v ); pxExporter->AddToBuffer( "</uv1_v>\n" );

	pxExporter->DecTabs();
	pxExporter->AddToBuffer( pxExporter->GetTabs() );
	pxExporter->AddToBuffer( "</uv_set>\n" );

	pxExporter->AddToBuffer( pxExporter->GetTabs() );
	pxExporter->AddToBuffer( "<hash_texture_name>" );
	pxExporter->AddToBufferHex( m_uHashTextureName );
	pxExporter->AddToBuffer( "</hash_texture_name>\n" );

	pxExporter->AddToBuffer( pxExporter->GetTabs() );
	pxExporter->AddToBuffer( "<texture_name>" );
	pxExporter->AddToBufferEscaped( m_xTextureName.GetString() );
	pxExporter->AddToBuffer( "</texture_name>\n" );

	pxExporter->AddToBuffer( pxExporter->GetTabs() );
	pxExporter->AddToBuffer( "<rotation>" );
	pxExporter->AddToBuffer( m_fRotation);
	pxExporter->AddToBuffer( "</rotation>\n" );

	pxExporter->DecTabs();
	pxExporter->AddToBuffer( pxExporter->GetTabs() );
	pxExporter->AddToBuffer( "</FMV_data>\n" );
}

#endif // ASURA_TOOLS

// --------------------------------------------------------------------------------------- 
// -- Protected Functions ----------------------------------------------------------------

// --------------------------------------------------------------------------------------- 
// -- Private Functions ------------------------------------------------------------------

// --------------------------------------------------------------------------------------- 

#endif // ASURA_USE_GUI_MENU_SYSTEM
