// ------------------------------------------------------------------------------------ //
//																						//
//	File: Asura_GUIMenu_Element_StatusText.h											//
//																						//
//	Description: status text element class 												//
//				 - the class stores a list of text id's matched to a specific int value	//
//				 - a command console var will be used to match against an int value		//
//																						//
//	Created: Steve Keen - 26/04/2007													//
//																						//
//	Copyright (C) 2006 Rebellion (Derby), All Rights Reserved.							//
// ------------------------------------------------------------------------------------ //

#ifndef __ASURA_GUIMENU_ELEMENT_STATUSTEXT_H__
#define __ASURA_GUIMENU_ELEMENT_STATUSTEXT_H__

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifdef ASURA_USE_GUI_MENU_SYSTEM
#include "Rendering/Asura_Font_System.h"

#include "GUIMenu/Asura_GUIMenu_Element_Base.h"

// -- Forward declarations ------------------------------------------------------------
class Asura_GUIMenu_Animator;

// -- Enums ---------------------------------------------------------------------------

enum Asura_GUIMenu_StatusText_Render_Flag
{
	ASURA_GUIMENU_STATUSTEXT_JUSTIFY_DEFAULT			= ASURA_TEXTFORMAT_DEFAULT,
	ASURA_GUIMENU_STATUSTEXT_JUSTIFY_LEFT				= ASURA_TEXTFORMAT_LEFTJUSTIFIED,
	ASURA_GUIMENU_STATUSTEXT_JUSTIFY_CENTRE				= ASURA_TEXTFORMAT_CENTREJUSTIFIED,
	ASURA_GUIMENU_STATUSTEXT_JUSTIFY_RIGHT				= ASURA_TEXTFORMAT_RIGHTJUSTIFIED,
	ASURA_GUIMENU_STATUSTEXT_JUSTIFY_VERTICAL_BOTTOM	= ASURA_TEXTFORMAT_VERTICALBOTTOMJUSTIFIED,
	ASURA_GUIMENU_STATUSTEXT_JUSTIFY_VERTICAL_CENTRE	= ASURA_TEXTFORMAT_VERTICALCENTREJUSTIFIED,
	ASURA_GUIMENU_STATUSTEXT_CAPITALISED				= ASURA_TEXTFORMAT_CAPITALISED,
	ASURA_GUIMENU_STATUSTEXT_DROPSHADOW					= ASURA_TEXTFORMAT_DROPSHADOWS,
	ASURA_GUIMENU_STATUSTEXT_DROPSHADOW_LIGHT			= ASURA_TEXTFORMAT_LIGHTDROPSHADOWS,
														  
	ASURA_GUIMENU_STATUSTEXT_SCALE_ON_PARENT_RESIZE     = 0x00008000,

};

// -- Defines -------------------------------------------------------------------------

#define ASURA_GUIMENU_STATUSTEXT_ADDITION_GRADIENT	5

#define ASURA_GUIMENU_ELEMENT_STATUSTEXT_LOCAL_FLAG_START	16

/*
Versions: ASURA_CHUNK_ELEMENT_STATUSTEXT_VERSION

0:	Initial data - SK 26.04.07
1:  ????
2:  Changed the export chunk process to only write out a hash of the font name, rather than the full name string.
    In a tools build, saving the chunk will write out the string as it has always done.
3:  Added Special text effects settings
*/
#define ASURA_CHUNK_ELEMENT_STATUSTEXT_VERSION 3


// -- Structures ----------------------------------------------------------------------

struct Asura_GUIMenu_StatusText_Info
{
	Asura_GUIMenu_Display_Condition_Params	m_xDisplayParams;
	Asura_Hash_ID							m_uTextPage;
	Asura_Hash_ID							m_uTextID;
	Asura_RGBA								m_xColour;
};

// -- class ---------------------------------------------------------------------------

class Asura_GUIMenu_Element_StatusText : public Asura_GUIMenu_Element_Base
{
	friend class Tools_GUIMenu_Element_StatusText;

public:
	enum Asura_GUIMenu_Inherit_Flag_Element_StatusText_Type
	{
		ASURA_GUIMENU_ELEMENT_STATUSTEXT_LOCAL_FLAG_FONT_SCALE	= (1<<(ASURA_GUIMENU_ELEMENT_STATUSTEXT_LOCAL_FLAG_START+0)),
		ASURA_GUIMENU_ELEMENT_STATUSTEXT_LOCAL_FLAG_FONT_NAME	= (1<<(ASURA_GUIMENU_ELEMENT_STATUSTEXT_LOCAL_FLAG_START+1)),
		ASURA_GUIMENU_ELEMENT_STATUSTEXT_LOCAL_FLAG_OFFSET_X		= (1<<(ASURA_GUIMENU_ELEMENT_STATUSTEXT_LOCAL_FLAG_START+2)),
		ASURA_GUIMENU_ELEMENT_STATUSTEXT_LOCAL_FLAG_OFFSET_Y		= (1<<(ASURA_GUIMENU_ELEMENT_STATUSTEXT_LOCAL_FLAG_START+3)),
		ASURA_GUIMENU_ELEMENT_STATUSTEXT_LOCAL_FLAG_RENDER_FLAGS	= (1<<(ASURA_GUIMENU_ELEMENT_STATUSTEXT_LOCAL_FLAG_START+4)),
		//ASURA_GUIMENU_ELEMENT_STATUSTEXT_OVERIDE_MAX is here to stop another programmer adding past bit flag 31
		ASURA_GUIMENU_ELEMENT_STATUSTEXT_LOCAL_FLAG_MAX			= (1<<(ASURA_GUIMENU_ELEMENT_STATUSTEXT_LOCAL_FLAG_START+15))
	};


	Asura_GUIMenu_Element_StatusText();
	virtual ~Asura_GUIMenu_Element_StatusText();

	virtual void 				Render( u_int uState);
	virtual void				Render( u_int uState, const Asura_Bounding_Rectangle& xBounds);
	virtual void				Copy( const Asura_GUIMenu_Root* pxSource );

	virtual void				Update( float fElapsedTime);

	bool						IsStatusConditionMet( Asura_GUIMenu_Display_Condition eCondition, int iHeldValue, int iValidate);

	// SK 28.02.07 - This is a compatibility function to allow old type calls.  This is bad because
	// Asura_String is a char* internally...hence no foreign text.
	//void						SetText( const Asura_String& xText);

	//void						SetText( const Asura_UnicodeChar* pwszTextString);
	//const Asura_UnicodeChar*	GetText() const;
	//bool						IsValidText() const;

	////Text IDs for localised text. If being used then m_xText will be blank
	//void 						SetTextID(Asura_Hash_ID uTextID);
	//Asura_Hash_ID				GetTextID() const;// {return m_uTextID;}
	//const Asura_UnicodeChar*	GetTextUsingTextID() const;	// returns string from Asura_Hashed_Localised_Text

	float						GetOffsetX();
	void						SetOffsetX( float fOffsetX)	{	m_fOffsetX = fOffsetX;	}
	float						GetOffsetY();
	void						SetOffsetY( float fOffsetY)	{	m_fOffsetY = fOffsetY;	}

	void						SetFontByName( const Asura_String& xFontName) { SetFontByName( xFontName.GetString());}
	void						SetFontByName( const char* szFontName);
#ifdef ASURA_TOOLS
	const char*					GetFontName() const;
#endif // ASURA_TOOLS
	int							GetFontID() const;
	
	Asura_Hash_ID				GetFontNameHash() const;

	void						SetFontScale( float fFontScale)	{	m_fFontScale = fFontScale; }
	float						GetFontScale()const;
	float*						GetFontScalePointer();
	
	bool						GetTextRenderFlag( u_int uFlag) const;
	void						SetTextRenderFlag( u_int uNewFlag, bool bFlagSet);
	u_int						GetTextRenderFlags() const;
	void						SetTextRenderFlags( u_int uFlags)	{	m_uTextRenderFlags = uFlags;	}

	void						AddTextInfo( Asura_Hash_ID uTextPage, Asura_Hash_ID uTextID, Asura_GUIMenu_Display_Condition eCondition, int iDisplayThreshold, Asura_RGBA& xColour);
	void						RemoveTextInfo( u_int uIndex);

	u_int						GetTextEntryCount() const	{	return m_uTextInfoEntryCount;	}
	u_int						GetMaxTextEntryCount() const	{	return m_uMaxTextInfoEntryCount;	}
	Asura_GUIMenu_StatusText_Info*	GetTextEntry( u_int uIndex) const;

	void						SetEnableSpecialEffects(bool bEnable);
	bool						AreSpecialEffectsEnabled() const { return m_pxSpecialEffectsSettings != NULL; }

	Asura_Font_Special_Effects*	GetSpecialEffectsSettings() const { return m_pxSpecialEffectsSettings; }

#ifdef ASURA_TOOLS
	virtual void				UpdateLocalDataFromStyle();

	void						ClearTextRenderFlags();

	virtual	void				ListUsedTextPages(Asura_HashTree& xTextPageTree) const;

	virtual void				WriteToChunkStream( Asura_Chunk_Stream& xStream) const;
	virtual void				ExportToXML(Tools_GUIMenu_XMLExporter* pxExporter) const;
#endif // ASURA_TOOLS

	virtual void				ReadFromChunkStream( Asura_Chunk_Stream& xStream, Asura_GUIMenu_Root* pxParentObject, u_int uChunkVersion);
protected:
	void						SwapTextEntries( u_int uIndex1, u_int uIndex2);

private:
	void						AllocateStatusTextArraySpace( u_int uMaxStatusTextCount);
	void						ExpandStatusTextArraySpace();

	void						GetConsoleVar();
	void						SetConsoleVar() const;

	void						ApplyModifierChanges( u_int uStateToMatch, Asura_Bounding_Rectangle& xRect, float& fOffsetX, float& fOffsetY, float& fFontScale, u_int& uColour, ASURA_TRANSLUCENCY_MODE& eTranslucencyMode,u_int& uRenderFlags, const Asura_UnicodeChar** ppwszText, int& iFontID);
	void						ApplyAnimatorChanges( Asura_GUIMenu_Animator* pxAnim, u_int uStateToMatch, Asura_Bounding_Rectangle& xRect, float& fOffsetX, float& fOffsetY, float& fFontScale, u_int& uColour, ASURA_TRANSLUCENCY_MODE& eTranslucencyMode,u_int& uRenderFlags, const Asura_UnicodeChar** ppwszText, int& iFontID);

	// -- Variables -------------------------------------------------------------------
public:

protected:
	Asura_GUIMenu_ConsoleVar		m_xConsoleVar;
	int								m_iCurrentConsoleVarValue;

private:
	int								m_iFontID;
#ifdef ASURA_TOOLS
	Asura_String					m_xFontName;
#endif 
	Asura_Hash_ID					m_uFontNameHash;

	float							m_fOffsetX;
	float							m_fOffsetY;

	float							m_fFontScale;
	u_int							m_uTextRenderFlags;

	u_int							m_uTextInfoEntryCount;
	u_int							m_uMaxTextInfoEntryCount;
	Asura_GUIMenu_StatusText_Info*	m_pxTextInfoEntries;

	Asura_Hash_ID					m_uActiveTextPage;
	Asura_Hash_ID					m_uActiveTextHash;
	Asura_RGBA						m_xActiveTextColour;

	// Special effects data. Can be NULL. If NULL then this text doesn't have any special effects
	Asura_Font_Special_Effects*		m_pxSpecialEffectsSettings;  
	// --------------------------------------------------------------------------------
	typedef Asura_GUIMenu_Element_Base	PARENT;
};
#endif // ASURA_USE_GUI_MENU_SYSTEM
#endif // __ASURA_UI_ELEMENT_STATUSTEXT_H__
