// ------------------------------------------------------------------------------------ //
//																						//
//	File: Asura_GUIMenu_Element_Text.cpp												//
//																						//
//	Description: text element class.													//
//																						//
//	Created: Steve Keen - 9/11/06														//
//																						//
//	Copyright (C) 2006 Rebellion (Derby), All Rights Reserved.							//
// ------------------------------------------------------------------------------------ //

#include "Core/Asura.h"

#ifdef ASURA_USE_GUI_MENU_SYSTEM
#include "Asura_GUIMenu_Element_Text.h"

#include <wchar.h>

#include "Core/Asura_Hash.h"
#include "Core/Asura_HashedLocalisedText.h"
#include "Core/Asura_String.h"
#include "Core/Asura_Timers.h"

#include "GUIMenu/Asura_GUIMenu_Animator.h"
#include "GUIMenu/Asura_GUIMenu_Menu_page.h"
#include "GUIMenu/Asura_GUIMenu_Modifier.h"
#include "GUIMenu/Asura_Profiling_GUIMenu.h"

#include "GUIMenu/Asura_GUIMenu_System.h"
//#include "Rendering/Asura_Font_System.h"

#ifdef ASURA_TOOLS
#include "GUIMenu/Tools_GUIMenu_XMLExporter.h"
#include "GUIMenu/Tools_Mode_GUIMenu.h"
#endif

// --------------------------------------------------------------------------------------- 
// -- Constructors -----------------------------------------------------------------------

//Padding used with scrolling text. This gets placed in between the end of one copy of the string , and the
//start of showing the next copy. 
//May want to put in some way of setting this on a project basis ,or on a per-element basis.
//RAR 04/05/2007
Asura_UnicodeChar* g_wszScrollPadText = L"  ---  ";

Asura_GUIMenu_Element_Text::Asura_GUIMenu_Element_Text(void)
: m_pwszText( NULL)
, m_uMaxTextCharsInclNull(0)
, m_iFontID( -1)
, m_uFontNameHash( ASURA_HASH_ID_UNSET)
, m_uTextID(ASURA_HASH_ID_UNSET)
, m_uTextPageHash(uASURA_HASHEDTEXTPAGE_MENU)
, m_fOffsetX( 0.0f)
, m_fOffsetY( 0.0f)
, m_fFontScale( 1.0f)
, m_uTextRenderFlags( ASURA_GUIMENU_TEXT_JUSTIFY_DEFAULT | ASURA_GUIMENU_TEXT_WORDWRAP)
, m_fScrollRate(0.0f)
, m_fScrollOffset(0.0f)
, m_fLineSpacing( 1.0f ) // MRD 15/06/07: Elements can now specify a line spacing value
{
	m_uType = ASURA_GUIMENU_TYPEID_ELEMENT_TEXT;

	// hack
#if 0 // defined(ASURA_TOOLS) && defined(_DEBUG)
	switch (Asura_Random::GetIntU(0,2))
	{
		case 0:
		{
			AddAnimator("test-animator")->SetSpeed(Asura_Random::GetFloat( 0.5f, 2.0f ));
			break;
		}
		case 1:
		{
			AddAnimator("test-animator2")->SetSpeed(Asura_Random::GetFloat( 0.9f, 1.1f ));
			break;
		}
		case 2:
		{
			AddAnimator("test-animator3")->SetSpeed(Asura_Random::GetFloat( 0.9f, 1.1f ));
			break;
		}
	}
#endif

	m_pxSpecialEffectsSettings = NULL;

	ASURA_GUIMENU_PROFILE_COUNTINSTANCE(ASURA_GUIMENU_COUNTER__ELEMENT_TEXT);
}

// --------------------------------------------------------------------------------------- 
// -- Destructor -------------------------------------------------------------------------

Asura_GUIMenu_Element_Text::~Asura_GUIMenu_Element_Text(void)
{
	ReleaseStringSpace();

	if ( m_pxSpecialEffectsSettings != NULL )
	{
		delete m_pxSpecialEffectsSettings;
	}

	ASURA_GUIMENU_PROFILE_COUNTDESTROYINSTANCE(ASURA_GUIMENU_COUNTER__ELEMENT_TEXT);
}

// --------------------------------------------------------------------------------------- 
// -- Public Functions ------------------------------------------------------------------- 

void Asura_GUIMenu_Element_Text::SetEnableSpecialEffects(bool bEnable)
{
	if ( bEnable == true && m_pxSpecialEffectsSettings == NULL )
	{
		m_pxSpecialEffectsSettings = new Asura_Font_Special_Effects;
	}
	else if ( bEnable == false && m_pxSpecialEffectsSettings != NULL )
	{
		delete m_pxSpecialEffectsSettings;
		m_pxSpecialEffectsSettings = NULL;
	}
}
// --------------------------------------------------------------------------------------- 

// --------------------------------------------------------------------------------------- 

void Asura_GUIMenu_Element_Text::Render( u_int uState)
{
	PARENT::Render( uState);
}

// --------------------------------------------------------------------------------------- 
void Asura_GUIMenu_Element_Text::Render( u_int uState, const Asura_Bounding_Rectangle& xBounds)
{
	ASURA_GUIMENU_PROFILE_START_TASK(ASURA_GUIMENU_PROFILE__ELEMENT_TEXT);

	const Asura_UnicodeChar* pwszText = GetResolvedText();
	float fFontScale = GetFontScale();
	if (GetTextRenderFlag(ASURA_GUIMENU_TEXT_SCALE_ON_PARENT_RESIZE))
	{
		if (m_xUnscaledLocalBounds.GetWidth() && m_xUnscaledLocalBounds.GetHeight())
		{
			//scale by smaller of parent dimension scales
			float fParentScale = m_xScaledWorldBounds.GetWidth() / m_xUnscaledLocalBounds.GetWidth();
			fParentScale = Asura_Maths::Min(fParentScale, m_xScaledWorldBounds.GetHeight() / m_xUnscaledLocalBounds.GetHeight());
			fFontScale *= fParentScale;
		}
	}
	float fOffsetX = GetOffsetX();
	float fOffsetY = GetOffsetY();
	u_int uRenderFlags = GetTextRenderFlags();
	float fScrollRate = GetScrollRate();
	float fLineSpacing = GetTextLineSpacing();

	int iFontID = GetFontID();

	Asura_Assert( iFontID != -1, "Asura_GUIMenu_Element_Text::Render - Font ID == -1");

	Asura_Bounding_Rectangle xRect(xBounds);
	xRect.MinX = (float)Asura_Maths::FloatToInt_Chop(xRect.MinX);
	xRect.MinY = (float)Asura_Maths::FloatToInt_Chop(xRect.MinY);

	u_int uCol = GetColour().ToUIntARGB();

	ASURA_TRANSLUCENCY_MODE eTransMode = GetTranslucencyMode();

	ApplyModifierChanges( uState, xRect, fScrollRate, fOffsetX, fOffsetY, fFontScale, uCol, eTransMode, uRenderFlags, &pwszText, iFontID);

	Asura_Bounding_Rectangle xCropRect;
	if (GetParentObject() && GetParentObject()->GetCropRectForChildren(xCropRect))
	{
		if (xRect.MaxX > xCropRect.MaxX)
		{
			xRect.MaxX = xCropRect.MaxX;
		}
		if (xRect.MaxY > xCropRect.MaxY)
		{
			xRect.MaxY = xCropRect.MaxY;
		}
		if (xRect.MinX < xCropRect.MinX)
		{
			fOffsetX -= xCropRect.MinX - xRect.MinX;
			xRect.MinX = xCropRect.MinX;
		}
		if (xRect.MinY < xCropRect.MinY)
		{
			fOffsetY -= xCropRect.MinY - xRect.MinY;
			xRect.MinY = xCropRect.MinY;
		}
	}

	ASURA_TRANSLUCENCY_MODE eOldMode = Asura_Font_System::SetTranslucencyMode( eTransMode);

	Asura_Font_System::SetFont( iFontID);
	Asura_Font_System::SetFontScale( iFontID,1.0f); // set font scale to default - NDB

	if(pwszText)
	{
		bool bScrolling = false;
		float fStringWidth = 0.0f;
		float fPadWidth = 0.0f;

		if(fScrollRate != 0.0f || m_fScrollOffset != 0.0f)
		{
			fStringWidth = Asura_Font_System::GetWidthOfString(pwszText,fFontScale);
			float fRectWidth = xRect.MaxX - xRect.MinX;

			//don't bother with scrolling if the string is small enough , or the rect is tiny
			if((fStringWidth <= fRectWidth) || (fRectWidth < 1.0f))
			{
				m_fScrollOffset = 0.0f;
			}
			else
			{

				//update the scroll offset
				m_fScrollOffset += Asura_Timers::GetFrameTime() * fScrollRate;

				//scrolling repeats after one copy of the string and one copy of the pad
				fPadWidth = Asura_Font_System::GetWidthOfString(g_wszScrollPadText,fFontScale);
				m_fScrollOffset = Asura_Maths::FloatMod(m_fScrollOffset,fStringWidth+fPadWidth);
	
				fOffsetX = -m_fScrollOffset;
				bScrolling = true;	


			}
		}

		#ifdef ASURA_TOOLS
		if(Tools_Mode_GUIMenu::ShouldUseOffsetAndZoom())
		{
			//Apply offset to coords from menu mode
			xRect.MinX = (xRect.MinX + Tools_Mode_GUIMenu::GetOffsetX())*Tools_Mode_GUIMenu::GetZoom();
			xRect.MaxX = (xRect.MaxX + Tools_Mode_GUIMenu::GetOffsetX())*Tools_Mode_GUIMenu::GetZoom();
			xRect.MinY = (xRect.MinY + Tools_Mode_GUIMenu::GetOffsetY())*Tools_Mode_GUIMenu::GetZoom();
			xRect.MaxY = (xRect.MaxY + Tools_Mode_GUIMenu::GetOffsetY())*Tools_Mode_GUIMenu::GetZoom();

			fOffsetX *= Tools_Mode_GUIMenu::GetZoom(); 
			fOffsetY *= Tools_Mode_GUIMenu::GetZoom(); 

			fFontScale *= Tools_Mode_GUIMenu::GetZoom();
			fStringWidth *= Tools_Mode_GUIMenu::GetZoom();
			fPadWidth *= Tools_Mode_GUIMenu::GetZoom();
		}
		#endif


		if(bScrolling)
		{
			uRenderFlags |= ASURA_TEXTFORMAT_RELATIVETORECT;
			//centre/right justification doesn't really make sense when scrolling
			uRenderFlags &=~ ASURA_TEXTFORMAT_CENTREJUSTIFIED|ASURA_TEXTFORMAT_RIGHTJUSTIFIED|ASURA_GUIMENU_TEXT_WORDWRAP;

			//draw the first copy of the text

			Asura_Font_System::PrintLongString(pwszText, &xRect, fOffsetX, fOffsetY,
				fLineSpacing, fFontScale, uCol, uRenderFlags);
			

			//draw the padding from the end of one copy to the start of the next
			fOffsetX += fStringWidth;

			Asura_Font_System::PrintLongString(g_wszScrollPadText, &xRect, fOffsetX, fOffsetY,
				fLineSpacing, fFontScale, uCol, uRenderFlags);

			//and draw a second copy of the text after the padding

			fOffsetX += fPadWidth;

			Asura_Font_System::PrintLongString(pwszText, &xRect, fOffsetX, fOffsetY,
				fLineSpacing, fFontScale, uCol, uRenderFlags);


		}
		else
		{
			if ( m_pxSpecialEffectsSettings != NULL )
			{
				if ( m_pxSpecialEffectsSettings->GetLastFrameNum()+1 < Asura_Timers::GetFrameNumber() )
				{
					// If text effects weren't rendered last frame then start the effect.
					m_pxSpecialEffectsSettings->Start();
				}

				// Update the special effects timers
				m_pxSpecialEffectsSettings->Update();
			}

			uRenderFlags |= ASURA_TEXTFORMAT_RELATIVETORECT;
			if (fOffsetX != 0.0f)
			{
				// no wordwrap or offset X doesn't do anything.
				uRenderFlags &=~ ASURA_GUIMENU_TEXT_WORDWRAP;
			}

			if ( m_pxSpecialEffectsSettings != NULL )
			{
				// If special effects are supported render them.
				Asura_Font_System::PrintSpecialEffectsString( 
															pwszText, 
															&xRect, 
															fOffsetX, 
															fOffsetY, 
															fLineSpacing, 
															fFontScale, 
															uCol, 
															uRenderFlags, 
															m_pxSpecialEffectsSettings );
			}
			else
			{
				Asura_Font_System::PrintLongString(pwszText, &xRect, fOffsetX, fOffsetY,
					fLineSpacing, fFontScale, uCol, uRenderFlags);
			}

		}
	}

	Asura_Font_System::SetTranslucencyMode(eOldMode);

	ASURA_GUIMENU_PROFILE_END_TASK();
}

// --------------------------------------------------------------------------------------- 

void Asura_GUIMenu_Element_Text::Copy( const Asura_GUIMenu_Root* pxSource)
{
	PARENT::Copy( pxSource);

	const Asura_GUIMenu_Element_Text* pxSourceText = static_cast<const Asura_GUIMenu_Element_Text*> (pxSource);
	
	//copy asura data
#ifdef ASURA_TOOLS
	SetFontByName(pxSourceText->GetFontName());
#else
	m_uFontNameHash = pxSourceText->GetFontNameHash();
#endif


	SetTextPageHash(pxSourceText->GetTextPageHash());
	if(pxSourceText->GetText())
	{
		SetText( pxSourceText->GetText());
	}
	else
	{
		SetTextID(pxSourceText->GetTextID());		
	}
	SetFontScale(pxSourceText->GetFontScale());
	SetTextRenderFlags( pxSourceText->GetTextRenderFlags());
	SetOffsetX( pxSourceText->GetOffsetX());
	SetOffsetY( pxSourceText->GetOffsetY());
	SetScrollRate(pxSourceText->GetScrollRate());
	SetTextLineSpacing( pxSourceText->GetTextLineSpacing());

	SetEnableSpecialEffects(pxSourceText->AreSpecialEffectsEnabled());

	if ( pxSourceText->GetSpecialEffectsSettings() != NULL )
	{
		m_pxSpecialEffectsSettings->Copy(pxSourceText->GetSpecialEffectsSettings());
	}
}

// --------------------------------------------------------------------------------------- 

bool Asura_GUIMenu_Element_Text::ContainsPos(const Asura_Vector_2& xPos) const
{
	//See if the position is within the extents of the element.
	//RAR 01/08/2007

	//Ignoring elements that are hidden for now
	if(IsRenderingDisabled()) return false;

	Asura_Bounding_Rectangle xBounds(GetScaledWorldBounds());

	Asura_Font_System::SetFont(GetFontID());
	float fFontScale = GetFontScale();
	if (GetTextRenderFlag(ASURA_GUIMENU_TEXT_SCALE_ON_PARENT_RESIZE))
	{
		if (m_xUnscaledLocalBounds.GetWidth() && m_xUnscaledLocalBounds.GetHeight())
		{
			//scale by smaller of parent dimension scales
			float fParentScale = m_xScaledWorldBounds.GetWidth() / m_xUnscaledLocalBounds.GetWidth();
			fParentScale = Asura_Maths::Min(fParentScale, m_xScaledWorldBounds.GetHeight() / m_xUnscaledLocalBounds.GetHeight());
			fFontScale *= fParentScale;
		}
	}
	float fTextWidth = Asura_Font_System::GetWidthOfString( GetResolvedText(), fFontScale);
	if (fTextWidth < xBounds.GetWidth())
	{
		if ((GetTextRenderFlags() & ASURA_TEXTFORMAT_RIGHTJUSTIFIED) != 0)
		{
			xBounds.MinX = xBounds.MaxX - fTextWidth;
		}
		else if ((GetTextRenderFlags() & ASURA_TEXTFORMAT_CENTREJUSTIFIED) != 0)
		{
			xBounds.MinX += (xBounds.GetWidth() - fTextWidth) / 2.0f;
			xBounds.MaxX = xBounds.MinX + fTextWidth;
		}
		else
		{
			xBounds.MaxX = xBounds.MinX + fTextWidth;
		}
	}

	return xBounds.Contains(xPos);
		
}

// --------------------------------------------------------------------------------------- 
// SK 28.02.07 - This is a compatibility function to allow old type calls.  This is bad because
// Asura_String is a char* internally...hence no foreign text.
void Asura_GUIMenu_Element_Text::SetText( const Asura_String& xText)
{
	Asura_UnicodeChar* pwszUnicodeString = new Asura_UnicodeChar[xText.GetLength()+1];
	if( pwszUnicodeString)
	{
		// -- convert the Asura_String to a unicode string.
		Asura_PrintfToBfr( pwszUnicodeString, L"%S", xText.GetLength()+1, true, xText.GetString());

		SetText( pwszUnicodeString);

		delete []pwszUnicodeString;
	}
}

// --------------------------------------------------------------------------------------- 

void Asura_GUIMenu_Element_Text::SetText( const Asura_UnicodeChar* pwszTextString)
{
	Asura_Assert( pwszTextString != NULL, "Asura_GUIMenu_Element_Text::SetText - pwszTextString == NULL.");

	if( pwszTextString)
	{
		AllocStringSpace( wcslen( pwszTextString)+1);
		wcscpy( m_pwszText, pwszTextString);

		if ( m_pxSpecialEffectsSettings != NULL )
		{
			m_pxSpecialEffectsSettings->Stop();
		}
	}
}

// --------------------------------------------------------------------------------------- 

void Asura_GUIMenu_Element_Text::ClearText( void)
{
	ReleaseStringSpace();

	if ( m_pxSpecialEffectsSettings != NULL )
	{
		m_pxSpecialEffectsSettings->Stop();
	}
}

// --------------------------------------------------------------------------------------- 

const Asura_UnicodeChar* Asura_GUIMenu_Element_Text::GetText() const
{
	return m_pwszText;
}

// --------------------------------------------------------------------------------------- 

const Asura_UnicodeChar* Asura_GUIMenu_Element_Text::GetResolvedText() const
{
	if( m_pwszText != NULL)
	{
		return m_pwszText;
	}
	else if( GetTextID())
	{
		return Asura_HashedLocalisedText::GetString(m_uTextPageHash, GetTextID());
	}
	return 0;
}

// --------------------------------------------------------------------------------------- 

bool Asura_GUIMenu_Element_Text::IsValidText( void) const
{
	return (m_pwszText != NULL);
}

bool Asura_GUIMenu_Element_Text::IsBlank() const
{
	// returns false if text string contains some text or text ID reffers to some text - NDB
	return !( (IsValidText() && *GetText()) || (GetTextID() != ASURA_HASH_ID_UNSET) );
}

// --------------------------------------------------------------------------------------- 

Asura_Hash_ID Asura_GUIMenu_Element_Text::GetTextID() const
{
	return m_uTextID;
}

// --------------------------------------------------------------------------------------- 

void Asura_GUIMenu_Element_Text::SetTextID( Asura_Hash_ID uTextID)
{
	m_uTextID = uTextID;

	ReleaseStringSpace();

	if ( m_pxSpecialEffectsSettings != NULL )
	{
		m_pxSpecialEffectsSettings->Stop();
	}
}


// --------------------------------------------------------------------------------------- 

const Asura_UnicodeChar* Asura_GUIMenu_Element_Text::GetTextUsingTextID( void) const
{
	return Asura_HashedLocalisedText::GetString(m_uTextPageHash, GetTextID());
}

// --------------------------------------------------------------------------------------- 

void Asura_GUIMenu_Element_Text::SetFontByName( const char* szFontName )
{
#ifdef ASURA_TOOLS
	m_xFontName = szFontName;
#endif
	m_iFontID = -1;

	m_uFontNameHash = Asura_GetHashID( szFontName);
}

// --------------------------------------------------------------------------------------- 
#ifdef ASURA_TOOLS
const char* Asura_GUIMenu_Element_Text::GetFontName( void) const
{
	return m_xFontName.GetString();
}
#endif // ASURA_TOOLS
// --------------------------------------------------------------------------------------- 

Asura_Hash_ID Asura_GUIMenu_Element_Text::GetFontNameHash( void ) const
{
	return m_uFontNameHash;
}

// --------------------------------------------------------------------------------------- 

float Asura_GUIMenu_Element_Text::GetFontScale( void)const
{

	return m_fFontScale;
}

// --------------------------------------------------------------------------------------- 

float* Asura_GUIMenu_Element_Text::GetFontScalePointer( void)
{
	return &m_fFontScale;
}

// --------------------------------------------------------------------------------------- 

int Asura_GUIMenu_Element_Text::GetFontID( void) const
{
	if( m_iFontID == -1)
	{
		if( m_uFontNameHash != ASURA_HASH_ID_UNSET)
		{
			m_iFontID = Asura_Font_System::GetFontHandle( m_uFontNameHash);
		}
	}
	return m_iFontID;
}

// --------------------------------------------------------------------------------------- 

#ifdef ASURA_TOOLS
void Asura_GUIMenu_Element_Text::UpdateLocalDataFromStyle( void)
{
	Asura_GUIMenu_Element_Text* pxStyleText;
	const Asura_UnicodeChar* pwszText = NULL;
	Asura_String xString;

	PARENT::UpdateLocalDataFromStyle();

	if( m_pxStyleObject != NULL)
	{
		if( (m_uStyleUseLocalDataFlags&ASURA_GUIMENU_ELEMENT_TEXT_LOCAL_FLAG_FONT_SCALE) == 0)
		{
			pxStyleText = static_cast<Asura_GUIMenu_Element_Text*>(m_pxStyleObject);
			SetFontScale( pxStyleText->GetFontScale());
		}

		if( (m_uStyleUseLocalDataFlags&ASURA_GUIMENU_ELEMENT_TEXT_LOCAL_FLAG_TEXT) == 0)
		{
			pxStyleText = static_cast<Asura_GUIMenu_Element_Text*>(m_pxStyleObject);
			pwszText = pxStyleText->GetText();
			if( pwszText != NULL)
			{
				SetText( pwszText);
			}
		}

		if( (m_uStyleUseLocalDataFlags&ASURA_GUIMENU_ELEMENT_TEXT_LOCAL_FLAG_FONT_NAME) == 0)
		{
			pxStyleText = static_cast<Asura_GUIMenu_Element_Text*>(m_pxStyleObject);
			xString = pxStyleText->GetFontName();
			SetFontByName( xString);
		}

		if( (m_uStyleUseLocalDataFlags&ASURA_GUIMENU_ELEMENT_TEXT_LOCAL_FLAG_TEXT_ID) == 0)
		{
			pxStyleText = static_cast<Asura_GUIMenu_Element_Text*>(m_pxStyleObject);
			SetTextID( pxStyleText->GetTextID());
		}

		if( (m_uStyleUseLocalDataFlags&ASURA_GUIMENU_ELEMENT_TEXT_LOCAL_FLAG_OFFSET_X) == 0)
		{
			pxStyleText = static_cast<Asura_GUIMenu_Element_Text*>(m_pxStyleObject);
			SetOffsetX( pxStyleText->GetOffsetX());
			SetScrollRate( pxStyleText->GetScrollRate());
		}

		if( (m_uStyleUseLocalDataFlags&ASURA_GUIMENU_ELEMENT_TEXT_LOCAL_FLAG_OFFSET_Y) == 0)
		{
			pxStyleText = static_cast<Asura_GUIMenu_Element_Text*>(m_pxStyleObject);
			SetOffsetY( pxStyleText->GetOffsetY());
		}

		if( (m_uStyleUseLocalDataFlags&ASURA_GUIMENU_ELEMENT_TEXT_LOCAL_FLAG_RENDER_FLAGS) == 0)
		{
			pxStyleText = static_cast<Asura_GUIMenu_Element_Text*>(m_pxStyleObject);
			SetTextRenderFlags( pxStyleText->GetTextRenderFlags());
		}
		if( (m_uStyleUseLocalDataFlags&ASURA_GUIMENU_ELEMENT_TEXT_LOCAL_FLAG_LINE_SPACING) == 0)
		{
			pxStyleText = static_cast<Asura_GUIMenu_Element_Text*>(m_pxStyleObject);
			SetTextLineSpacing( pxStyleText->GetTextLineSpacing());
		}

		pxStyleText = static_cast<Asura_GUIMenu_Element_Text*>(m_pxStyleObject);

		SetEnableSpecialEffects(pxStyleText->AreSpecialEffectsEnabled());

		if ( GetSpecialEffectsSettings() != NULL )
		{
			GetSpecialEffectsSettings()->Copy(pxStyleText->GetSpecialEffectsSettings());
		}
	}
}
#endif

// --------------------------------------------------------------------------------------- 

bool Asura_GUIMenu_Element_Text::GetTextRenderFlag( u_int uFlag) const
{
	return ((m_uTextRenderFlags&uFlag) != 0);
}

// --------------------------------------------------------------------------------------- 

u_int Asura_GUIMenu_Element_Text::GetTextRenderFlags( void) const
{
	return m_uTextRenderFlags;
}


// --------------------------------------------------------------------------------------- 

void Asura_GUIMenu_Element_Text::SetTextRenderFlag( u_int uFlagsToSet, bool bFlagSet)
{
	if( bFlagSet)
	{
		m_uTextRenderFlags |= uFlagsToSet;
	}
	else
	{
		m_uTextRenderFlags &=~ uFlagsToSet;
	}
}

// --------------------------------------------------------------------------------------- 
// -- Inheritance tool helper function -------------------------------------------------------

#ifdef ASURA_TOOLS
void Asura_GUIMenu_Element_Text::ClearTextRenderFlags( void)
{
	m_uTextRenderFlags = 0;
}
#endif


// --------------------------------------------------------------------------------------- 
// -- Chunk ------------------------------------------------------------------------------

void Asura_GUIMenu_Element_Text::ReadFromChunkStream( Asura_Chunk_Stream& xStream, Asura_GUIMenu_Root* pxParentObject, u_int uChunkVersion)
{
//	u_int uContainerCount, u;
	char acName[1024];
	Asura_UnicodeChar wszName[1024];
	Asura_String xFontName;

	PARENT::ReadFromChunkStream( xStream, pxParentObject, 0xffffffff);

	// KOR - removed this handling entirely in version 6, as we were pointlessly saving out a u_int storing 0.
	// Should be wrapped with #ifdef ASURA_TOOLS once all exports are up to date.
	if( uChunkVersion < 6)
	{
		// read in text element specific data --
		if( uChunkVersion < 4)	// this is pre unicode text output
		{
			xStream.ReadString( acName, sizeof(acName));
		}
		else
		{
			// -- read in the unicode string --
			u_int uCharCount, u;

			xStream >> uCharCount;

			wszName[0] = 0x0000;

			for( u=0; u<uCharCount; u++)
			{
				xStream >> wszName[u];
				wszName[u+1] = 0x0000;	// force a null terminator to the end.
			}
		}

		if( uChunkVersion < 4)
		{
			Asura_PrintfToBfr( &wszName[0], L"%s", 1024, true, acName);
		}

		if( wszName[0] != 0x0000)
		{
			SetText( &wszName[0]);
		}
	}

	if(uChunkVersion >= 6)
	{
		xStream >> m_uTextPageHash;
	}

	if(uChunkVersion >= 2)
	{
		Asura_Hash_ID uTextID;
		xStream >> uTextID;
		if(uTextID)
		{
			SetTextID(uTextID);
		}
	}

#ifdef ASURA_TOOLS
	xStream.ReadString( acName, sizeof(acName));
	xFontName = acName;
	SetFontByName( xFontName);
#else
	if( uChunkVersion >= 8)
	{
		xStream >> m_uFontNameHash;
	}
	else
	{
		xStream.ReadString( acName, sizeof(acName));
		m_uFontNameHash = Asura_GetHashID( acName);
//		xFontName = acName;
//		SetFontByName( xFontName);
	}
#endif

	if( uChunkVersion >= 1)
	{
		xStream >> m_fFontScale;
	}

	// -- read in the text render flags --
	if( uChunkVersion >= 3)
	{
		xStream >> m_uTextRenderFlags;
	}

	if(uChunkVersion < 5)
	{
		//word wrap flag is now editable , but is on by defalt for old text elements
		m_uTextRenderFlags |= ASURA_GUIMENU_TEXT_WORDWRAP;
	}
	
	if( uChunkVersion >= 5)
	{
		xStream >> m_fScrollRate;
	}

	// MRD 15/06/07 We can now set line spacing values to adjust
	// how much space is given to the next line
	if ( uChunkVersion >= 7 )
	{
		xStream >> m_fLineSpacing;
	}

	#ifdef ASURA_TOOLS
	if( uChunkVersion < 9)
	{
		//Line spacing can now be controlled by styles.
		//Update existing items so that they aren't handled by the style , so they behave the same as before
		//RAR 02/06/2008
		m_uStyleUseLocalDataFlags |= ASURA_GUIMENU_ELEMENT_TEXT_LOCAL_FLAG_LINE_SPACING;
		m_uStyleChildEditableFlags |= ASURA_GUIMENU_ELEMENT_TEXT_LOCAL_FLAG_LINE_SPACING;
	}
	#endif
	
	// AMM 30/10/08 read the x and y offset values if supported by chunk version
	if (uChunkVersion >= 10)
	{
		xStream >> m_fOffsetX;
		xStream >> m_fOffsetY;
	}

	// -- read in the container info
	ReadAllChildObjects( xStream, this);

	if(uChunkVersion >= 11)
	{
		bool bSpecialEffects;

		xStream >> bSpecialEffects;

		SetEnableSpecialEffects(bSpecialEffects);

		if ( bSpecialEffects && m_pxSpecialEffectsSettings != NULL )
		{
			m_pxSpecialEffectsSettings->ReadFromChunkStream(xStream);
		}
	}
}

// --------------------------------------------------------------------------------------- 
#ifdef ASURA_TOOLS
void Asura_GUIMenu_Element_Text::WriteToChunkStream( Asura_Chunk_Stream& xStream) const
{
	bool bIsExporting = Tools_Level_Export::IsGUIMenuExport() | Tools_Level_Export::IsLevelExport();

	WriteLocalChunkHeader( xStream, ASURA_CHUNK_ELEMENT_TEXT_VERSION, GetType());

	PARENT::WriteToChunkStream( xStream);

	// (MH - 01.03.2007) user-specified text no longer saved out
	xStream << m_uTextPageHash;
	xStream << m_uTextID;

	if( bIsExporting)
	{
		xStream << m_uFontNameHash;
	}
	else
	{
		xStream << m_xFontName;
	}

	xStream << m_fFontScale;
	xStream << m_uTextRenderFlags;
	xStream << m_fScrollRate;
	xStream << m_fLineSpacing; // MRD 15/06/07: Store the linespacing value
	
	// AMM 30/10/08 store the x and y offset values (chunk version number updated to take account of additions to stream)
	xStream << m_fOffsetX;
	xStream << m_fOffsetY;

	// -- write out any children --
	WriteChildObjectData( xStream);

	bool bSpecialEffects = AreSpecialEffectsEnabled();
	xStream << bSpecialEffects;
	if ( bSpecialEffects )
	{
		m_pxSpecialEffectsSettings->WriteToChunkStream(xStream);
	}
}

void Asura_GUIMenu_Element_Text::ExportToXML(Tools_GUIMenu_XMLExporter* pxExporter) const
{
	PARENT::ExportToXML( pxExporter );
	pxExporter->AddToBuffer( pxExporter->GetTabs() );
	pxExporter->AddToBuffer( "<Text_data>\n" );
	pxExporter->IncTabs();

	pxExporter->AddToBuffer( pxExporter->GetTabs() );
	pxExporter->AddToBuffer( "<text_page>" );
	pxExporter->AddToBufferHex( m_uTextPageHash );
	pxExporter->AddToBuffer( "</text_page>\n" );

	pxExporter->AddToBuffer( pxExporter->GetTabs() );
	pxExporter->AddToBuffer( "<text_id>" );
	pxExporter->AddToBufferHex( m_uTextID );
	pxExporter->AddToBuffer( "</text_id>\n" );

	pxExporter->AddToBuffer( pxExporter->GetTabs() );
	pxExporter->AddToBuffer( "<font_name>" );
	pxExporter->AddToBufferEscaped( m_xFontName.GetString() );
	pxExporter->AddToBuffer( "</font_name>\n" );

	pxExporter->AddToBuffer( pxExporter->GetTabs() );
	pxExporter->AddToBuffer( "<font_scale>" );
	pxExporter->AddToBuffer( m_fFontScale );
	pxExporter->AddToBuffer( "</font_scale>\n" );

	pxExporter->AddToBuffer( pxExporter->GetTabs() );
	pxExporter->AddToBuffer( "<text_render_flags>" );
	pxExporter->AddToBufferHex( m_uTextRenderFlags );
	pxExporter->AddToBuffer( "</text_render_flags>\n" );

	pxExporter->AddToBuffer( pxExporter->GetTabs() );
	pxExporter->AddToBuffer( "<scroll_rate>" );
	pxExporter->AddToBuffer( m_fScrollRate );
	pxExporter->AddToBuffer( "</scroll_rate>\n" );

	// MRD 15/06/07: allow adjustable linespacing 
	pxExporter->AddToBuffer( pxExporter->GetTabs() );
	pxExporter->AddToBuffer( "<font_linespacing>" );
	pxExporter->AddToBuffer( m_fLineSpacing );
	pxExporter->AddToBuffer( "</font_linespacing>\n" );

	// AMM 30/10/08 write out the x and y offset values if supported by chunk version
	pxExporter->AddToBuffer( pxExporter->GetTabs() );
	pxExporter->AddToBuffer( "<xOffset>" );
	pxExporter->AddToBuffer( m_fOffsetX );
	pxExporter->AddToBuffer( "</xOffset>\n" );

	pxExporter->AddToBuffer( pxExporter->GetTabs() );
	pxExporter->AddToBuffer( "<yOffset>" );
	pxExporter->AddToBuffer( m_fOffsetY );
	pxExporter->AddToBuffer( "</yOffset>\n" );

	pxExporter->DecTabs();
	pxExporter->AddToBuffer( pxExporter->GetTabs() );
	pxExporter->AddToBuffer( "</Text_data>\n" );
}

#endif // ASURA_TOOLS

// ----------------------------------------------------------------------------------------- 
// -- Protected Functions ------------------------------------------------------------------


// --------------------------------------------------------------------------------------- 
// -- Private Functions ------------------------------------------------------------------


// --------------------------------------------------------------------------------------- 

void Asura_GUIMenu_Element_Text::AllocStringSpace( u_int uSizeInUnicodeChars)
{
	// only alloc if we can't already fit in our buffer
	if (m_uMaxTextCharsInclNull < uSizeInUnicodeChars)
	{
		ReleaseStringSpace();

		m_pwszText = new Asura_UnicodeChar[uSizeInUnicodeChars];
		m_uMaxTextCharsInclNull = uSizeInUnicodeChars;
	}
	else
	{
		Asura_Assert(m_pwszText, "no buffer");
	}
}

// --------------------------------------------------------------------------------------- 

void Asura_GUIMenu_Element_Text::ReleaseStringSpace( void)
{
	if( m_pwszText)
	{
		delete []m_pwszText;
		m_pwszText = NULL;
		m_uMaxTextCharsInclNull = 0;
	}
}

// --------------------------------------------------------------------------------------- 

void Asura_GUIMenu_Element_Text::ApplyModifierChanges( u_int uStateToMatch, Asura_Bounding_Rectangle& xRect, float& fScrollRate, float& fOffsetX, float& fOffsetY, float& fFontScale, u_int& uColour, ASURA_TRANSLUCENCY_MODE& eTranslucencyMode, u_int& uRenderFlags, const Asura_UnicodeChar** ppwszText, int& iFontID)
{
	u_int u, uModifierCount;
	u_int uModifierState;
	
	Asura_GUIMenu_Modifier* pxMod;
	
	uModifierCount = GetModifierCount();

	for( u=0; u<uModifierCount; u++)
	{
		pxMod = GetModifier( u);
		if( pxMod)
		{
			uModifierState = pxMod->GetStateBeingModified();

			if( ((uModifierState&uStateToMatch) != 0) && pxMod->IsActivationConditionMet() )
			{
				switch (pxMod->GetModifyType())
				{
					case ASURA_GUIMENU_MODIFY_POSITION:
					{
						Asura_Vector_2 xDeltaPos = pxMod->GetVector2Data();

						xRect.MinX += xDeltaPos.x;
						xRect.MinY += xDeltaPos.y;
						xRect.MaxX += xDeltaPos.x;
						xRect.MaxY += xDeltaPos.y;

						break;
					}

					case ASURA_GUIMENU_MODIFY_POSITION_RELATIVE:
					{
						Asura_Vector_2 xDims(Asura_Maths::ZeroVector2);
						Asura_GUIMenu_Menu_Page* pxPage = FindOwnerMenu();

						Asura_Vector_2 xDeltaPos = pxMod->GetVector2Data();

						if( pxPage)
						{
							xDims = pxPage->GetScaledWorldBounds().GetDimension();
						}

						xRect.MinX += xDeltaPos.x * xDims.x;
						xRect.MinY += xDeltaPos.y * xDims.y;
						xRect.MaxX += xDeltaPos.x * xDims.x;
						xRect.MaxY += xDeltaPos.y * xDims.y;
						break;
					}

					case ASURA_GUIMENU_MODIFY_COLOUR:
					{
						uColour = pxMod->GetColourData().ToUIntARGB();
						break;
					}

					case ASURA_GUIMENU_MODIFY_TRANSLUCENCY_MODE:
					{
						eTranslucencyMode = (ASURA_TRANSLUCENCY_MODE)pxMod->GetIntData();
						break;
					}

					case ASURA_GUIMENU_MODIFY_WIDTH:
					{
						xRect.MaxX = xRect.MinX + pxMod->GetFloatData();
						break;
					}

					case ASURA_GUIMENU_MODIFY_HEIGHT:
					{
						xRect.MaxY = xRect.MinY + pxMod->GetFloatData();
						break;
					}

					case ASURA_GUIMENU_MODIFY_TEXT_STRING:
					{
						*ppwszText = pxMod->GetUnicodeCharStringData();
						break;
					}

					case ASURA_GUIMENU_MODIFY_TEXT_OFFSET_X:
					{
						fOffsetX = pxMod->GetFloatData();
						break;
					}

					case ASURA_GUIMENU_MODIFY_TEXT_OFFSET_Y:
					{
						fOffsetY = pxMod->GetFloatData();
						break;
					}

					case ASURA_GUIMENU_MODIFY_TEXT_RENDER_FLAGS:
					{
						uRenderFlags = pxMod->GetUIntData();
						break;
					}

					case ASURA_GUIMENU_MODIFY_TEXT_ID:
					{
						*ppwszText = Asura_HashedLocalisedText::GetString( m_uTextPageHash, pxMod->GetUIntData());
						break;
					}

					case ASURA_GUIMENU_MODIFY_FONT_SCALE:
					{
						fFontScale = pxMod->GetFloatData();
						break;
					}

					case ASURA_GUIMENU_MODIFY_FONT_NAME:
					{
						iFontID = Asura_Font_System::GetFontHandle( pxMod->GetCharStringData());
						break;
					}

					case ASURA_GUIMENU_MODIFY_ALPHA_SCALED:
					{
						int iAlpha = uColour >> 24;
						float fAlphaScale = pxMod->GetFloatData();
						Asura_Assert(fAlphaScale >= 0.0f && fAlphaScale <= 1.0f, "alpha scale out of range");
						float fNewAlpha = fAlphaScale * static_cast<float>(iAlpha);
						iAlpha = Asura_Maths::FloatToInt( fNewAlpha );
						Asura_Assert(iAlpha >= 0 && iAlpha <= 255, "alpha out of range");
						u_int uAlpha = iAlpha << 24;
						uColour = (uColour & 0x00ffffff) | uAlpha;
						break;
					}

					case ASURA_GUIMENU_MODIFY_TEXT_SCROLL_RATE :
					{
						fScrollRate = pxMod->GetFloatData();
						break;
					}


					default:
					{
						// possibly just ignore with no assert?
						Asura_Assert(false, "unhandled modify type");
						break;
					}
				}// end switch( pxMod->GetModifyType())
			}
		}
	}

	// apply animators from this object and all parent objects
	const Asura_GUIMenu_Root* pxElementItr = this;
	while ( pxElementItr )
	{
		for( u=0; u<pxElementItr->GetAnimatorCount(); u++)
		{
			Asura_GUIMenu_Animator* pxAnim = pxElementItr->GetAnimator( u);
			Asura_Assert( pxAnim, "shouldn't fail");
			ApplyAnimatorChanges( pxAnim, uStateToMatch, xRect, fScrollRate, fOffsetX, fOffsetY, fFontScale, uColour, eTranslucencyMode, uRenderFlags, ppwszText, iFontID );
		}// end loop on animators

		pxElementItr = pxElementItr->GetParentObject();
	}
}

void Asura_GUIMenu_Element_Text::ApplyAnimatorChanges( Asura_GUIMenu_Animator* pxAnim, u_int uStateToMatch, Asura_Bounding_Rectangle& xRect, float& fScrollRate, float& fOffsetX, float& fOffsetY, float& fFontScale, u_int& uColour, ASURA_TRANSLUCENCY_MODE& eTranslucencyMode,u_int& uRenderFlags, const Asura_UnicodeChar** ppwszText, int& iFontID)
{
	Asura_Assert(pxAnim, "null pointer");
	if(pxAnim->ShouldUseAnimator(uStateToMatch))
	{
		switch (pxAnim->GetModifyType())
		{
			case ASURA_GUIMENU_MODIFY_POSITION:
			{
				Asura_Vector_2 xPos = pxAnim->GetVector2Data();

				xRect.MinX += xPos.x;
				xRect.MinY += xPos.y;

				xRect.MaxX += xPos.x;
				xRect.MaxY += xPos.y;
				break;
			}

			case ASURA_GUIMENU_MODIFY_POSITION_RELATIVE:
			{
				Asura_Vector_2 xDims(Asura_Maths::ZeroVector2);
				Asura_GUIMenu_Menu_Page* pxPage = FindOwnerMenu();

				Asura_Vector_2 xPos = pxAnim->GetVector2Data();

				if( pxPage)
				{
					xDims = pxPage->GetScaledWorldBounds().GetDimension();
				}

				xRect.MinX += xPos.x * xDims.x;
				xRect.MinY += xPos.y * xDims.y;

				xRect.MaxX += xPos.x * xDims.x;
				xRect.MaxY += xPos.y * xDims.y;
				break;
			}


			case ASURA_GUIMENU_MODIFY_COLOUR:
			{
				u_int uAnimColour = pxAnim->GetRGBAData().ToUIntARGB();
				//Ignore the alpha component for now I think , since that wlll probably be more useful.
				//May need to add a flag later to determine which components to use.
				//RAR 25/07/2007
				uColour  = (uColour & 0xff000000) | (uAnimColour & 0x00ffffff);
				break;
			}

			case ASURA_GUIMENU_MODIFY_ALPHA_SCALED:
			{
				int iAlpha = uColour >> 24;
				float fAlphaScale = pxAnim->GetFloatData();
				Asura_Assert(fAlphaScale >= 0.0f && fAlphaScale <= 1.0f, "alpha scale out of range");
				float fNewAlpha = fAlphaScale * static_cast<float>(iAlpha);
				iAlpha = Asura_Maths::FloatToInt( fNewAlpha );
				Asura_Assert(iAlpha >= 0 && iAlpha <= 255, "alpha out of range");
				u_int uAlpha = iAlpha << 24;
				uColour = (uColour & 0x00ffffff) | uAlpha;
				break;
			}

			case ASURA_GUIMENU_MODIFY_WIDTH_SCALED:
			{
				float fWidth = xRect.MaxX - xRect.MinX;
				fWidth *= pxAnim->GetFloatData();
				xRect.MaxX = xRect.MinX + fWidth;
				break;
			}

			case ASURA_GUIMENU_MODIFY_HEIGHT_SCALED:
			{
				float fHeight = xRect.MaxY - xRect.MinY;
				fHeight *= pxAnim->GetFloatData();
				xRect.MaxY = xRect.MinY + fHeight;
				break;
			}

			case ASURA_GUIMENU_MODIFY_SIZE_SCALE:
			{
				Asura_Vector_2 xPos = pxAnim->GetVector2Data();
				float fWidth = xRect.MaxX - xRect.MinX;
				fWidth *= xPos.x;
				xRect.MaxX = xRect.MinX + fWidth;
				float fHeight = xRect.MaxY - xRect.MinY;
				fHeight *= xPos.y;
				xRect.MaxY = xRect.MinY + fHeight;
				break;
			}

			case ASURA_GUIMENU_MODIFY_TEXT_OFFSET_X:
			{
				fOffsetX = pxAnim->GetFloatData();
				break;
			}

			case ASURA_GUIMENU_MODIFY_TEXT_OFFSET_Y:
			{
				fOffsetY = pxAnim->GetFloatData();
				break;
			}

			case ASURA_GUIMENU_MODIFY_TEXT_SCROLL_RATE :
			{
				fScrollRate = pxAnim->GetFloatData();
				break;
			}

			case ASURA_GUIMENU_MODIFY_FONT_SCALE :
			{
				fFontScale *= pxAnim->GetFloatData();
				break;
			}

			default:
			{
				//Not sure its worth having an assert here , since if an animator is applied to a widget , then
				//the child elements will get some types fed to it that can't be handled.
				//RAR 17/04/2007
				//Asura_Assert(false, "unhandled data type");
				break;
			}
		} // end switch
	}// end if (state is affected)
	else
	{
		pxAnim->Start();
	}
}


// --------------------------------------------------------------------------------------- 

#ifdef ASURA_TOOLS
void Asura_GUIMenu_Element_Text::ListUsedTextPages(Asura_HashTree& xTextPageTree) const
{
	if(m_uTextPageHash)
	{
		int iJunkIndex;
		xTextPageTree.FindNodeIndexByHash(m_uTextPageHash, true, iJunkIndex);
	}
	
	PARENT::ListUsedTextPages(xTextPageTree);
}
#endif

#endif // ASURA_USE_GUI_MENU_SYSTEM
