// ------------------------------------------------------------------------------------ //
//																						//
//	File: Asura_GUIMenu_Element_Text.h													//
//																						//
//	Description: text element class														//
//																						//
//	Created: Steve Keen - 9/11/06														//
//																						//
//	Copyright (C) 2006 Rebellion (Derby), All Rights Reserved.							//
// ------------------------------------------------------------------------------------ //

#ifndef __ASURA_GUIMENU_ELEMENT_TEXT_H__
#define __ASURA_GUIMENU_ELEMENT_TEXT_H__

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifdef ASURA_USE_GUI_MENU_SYSTEM
#include "Rendering/Asura_Font_System.h"

#include "GUIMenu/Asura_GUIMenu_Element_Base.h"


// -- Forward declarations ------------------------------------------------------------
class Asura_GUIMenu_Animator;

// -- Enums ---------------------------------------------------------------------------

enum Asura_GUIMenu_Text_Render_Flag
{
	ASURA_GUIMENU_TEXT_JUSTIFY_DEFAULT			= ASURA_TEXTFORMAT_DEFAULT,
	ASURA_GUIMENU_TEXT_JUSTIFY_LEFT				= ASURA_TEXTFORMAT_LEFTJUSTIFIED,
	ASURA_GUIMENU_TEXT_JUSTIFY_CENTRE			= ASURA_TEXTFORMAT_CENTREJUSTIFIED,
	ASURA_GUIMENU_TEXT_JUSTIFY_RIGHT			= ASURA_TEXTFORMAT_RIGHTJUSTIFIED,
	ASURA_GUIMENU_TEXT_JUSTIFY_VERTICAL_BOTTOM	= ASURA_TEXTFORMAT_VERTICALBOTTOMJUSTIFIED,
	ASURA_GUIMENU_TEXT_JUSTIFY_VERTICAL_CENTRE	= ASURA_TEXTFORMAT_VERTICALCENTREJUSTIFIED,
	ASURA_GUIMENU_TEXT_CAPITALISED				= ASURA_TEXTFORMAT_CAPITALISED,
	ASURA_GUIMENU_TEXT_DROPSHADOW				= ASURA_TEXTFORMAT_DROPSHADOWS,
	ASURA_GUIMENU_TEXT_DROPSHADOW_LIGHT			= ASURA_TEXTFORMAT_LIGHTDROPSHADOWS,
	ASURA_GUIMENU_TEXT_WORDWRAP					= ASURA_TEXTFORMAT_WORDWRAP,
	ASURA_GUIMENU_TEXT_SCALE_ON_PARENT_RESIZE	= 0x00008000,
};

// -- Defines -------------------------------------------------------------------------

#define ASURA_GUIMENU_ELEMENT_TEXT_LOCAL_FLAG_START	16

/*
Versions: ASURA_CHUNK_ELEMENT_TEXT_VERSION

0:	Initial data - SK 5.12.06
1:	Added font scale - SK 29.1.07
2:	Added localised text ID - RAR 09.02.07
3:  Added text render flags - SK 20.02.07
4:  Changed the text storage from Asura_String (char*) to Asura_UnicodeChar* - SK 27.02.07
5:	Added scroll rate , and made wordwrap flag editable - RAR 04.05.07
6:  Added text page hash and stripped out user-specified text properly KOR 07/06/2007
7:	Added linespacing so you can specify the gap between text spanning multiple lines - MRD 15/06/07
8:  Changed the export so it will write out a hash of the font name, for the Asura_files version it will write out the full string
9:	Line spacing now controllable by style , so update old items to preserve old behaviour - RAR 02/06/2008
10: Write out x and y offset in chunk data - AMM 30/10/08
11: Added Special effects settings - 8/4/09
*/
#define ASURA_CHUNK_ELEMENT_TEXT_VERSION 11

// -- Structures ----------------------------------------------------------------------
class Asura_Font_Special_Effects;

class Asura_GUIMenu_Element_Text : public Asura_GUIMenu_Element_Base
{
public:
	enum Asura_GUIMenu_Inherit_Flag_Element_Text_Type
	{
		ASURA_GUIMENU_ELEMENT_TEXT_LOCAL_FLAG_FONT_SCALE	= (1<<(ASURA_GUIMENU_ELEMENT_TEXT_LOCAL_FLAG_START+0)),
		ASURA_GUIMENU_ELEMENT_TEXT_LOCAL_FLAG_TEXT			= (1<<(ASURA_GUIMENU_ELEMENT_TEXT_LOCAL_FLAG_START+1)),
		ASURA_GUIMENU_ELEMENT_TEXT_LOCAL_FLAG_FONT_NAME	= (1<<(ASURA_GUIMENU_ELEMENT_TEXT_LOCAL_FLAG_START+2)),
		ASURA_GUIMENU_ELEMENT_TEXT_LOCAL_FLAG_TEXT_ID		= (1<<(ASURA_GUIMENU_ELEMENT_TEXT_LOCAL_FLAG_START+3)),
		ASURA_GUIMENU_ELEMENT_TEXT_LOCAL_FLAG_OFFSET_X		= (1<<(ASURA_GUIMENU_ELEMENT_TEXT_LOCAL_FLAG_START+4)),//also used for scroll rate
		ASURA_GUIMENU_ELEMENT_TEXT_LOCAL_FLAG_OFFSET_Y		= (1<<(ASURA_GUIMENU_ELEMENT_TEXT_LOCAL_FLAG_START+5)),
		ASURA_GUIMENU_ELEMENT_TEXT_LOCAL_FLAG_RENDER_FLAGS	= (1<<(ASURA_GUIMENU_ELEMENT_TEXT_LOCAL_FLAG_START+6)),
		ASURA_GUIMENU_ELEMENT_TEXT_LOCAL_FLAG_LINE_SPACING	= (1<<(ASURA_GUIMENU_ELEMENT_TEXT_LOCAL_FLAG_START+7)),
		//ASURA_GUIMENU_ELEMENT_TEXT_OVERIDE_MAX is here to stop another programmer adding past bit flag 31
		ASURA_GUIMENU_ELEMENT_TEXT_LOCAL_FLAG_MAX			= (1<<(ASURA_GUIMENU_ELEMENT_TEXT_LOCAL_FLAG_START+15))
	};


	Asura_GUIMenu_Element_Text( void);
	virtual ~Asura_GUIMenu_Element_Text( void);

	virtual void 				Render( u_int uState);
	virtual void				Render( u_int uState, const Asura_Bounding_Rectangle& xBounds);
	virtual void				Copy( const Asura_GUIMenu_Root* pxSource );

	virtual bool				ContainsPos(const Asura_Vector_2& xPos) const;

	// SK 28.02.07 - This is a compatibility function to allow old type calls.  This is bad because
	// Asura_String is a char* internally...hence no foreign text.
	void						SetText( const Asura_String& xText);

	void						SetText( const Asura_UnicodeChar* pwszTextString);
	const Asura_UnicodeChar*	GetText( void) const;
	bool						IsValidText( void) const;
	const Asura_UnicodeChar*	GetResolvedText( void) const;

	// returns true if there is no text to render - NDB
	bool						IsBlank() const;

	// SK 5.6.07 - Added ClearText() to enable us to clear out a text entry.  Passing NULL to 
	// SetText() does not work.
	void						ClearText( void);

	//Text IDs for localised text. If being used then m_xText will be blank
	void 						SetTextID(Asura_Hash_ID uTextID);
	Asura_Hash_ID				GetTextID() const;// {return m_uTextID;}
	const Asura_UnicodeChar*	GetTextUsingTextID( void) const;	// returns string from Asura_Hashed_Localised_Text

	void 						SetTextPageHash(Asura_Hash_ID uTextPageHash) { m_uTextPageHash = uTextPageHash; }
	Asura_Hash_ID				GetTextPageHash() const { return m_uTextPageHash; }

	float						GetOffsetX( void) const {return m_fOffsetX;}
	void						SetOffsetX( float fOffsetX)	{	m_fOffsetX = fOffsetX;	}
	float						GetOffsetY( void) const {return m_fOffsetY;}
	void						SetOffsetY( float fOffsetY)	{	m_fOffsetY = fOffsetY;	}

	float						GetScrollRate( void) const {return m_fScrollRate;}
	void						SetScrollRate( float fScrollRate)	{	m_fScrollRate = fScrollRate; }


	void						SetFontByName( const Asura_String& xFontName) { SetFontByName( xFontName.GetString());}
	void						SetFontByName( const char* szFontName);
#ifdef ASURA_TOOLS
	const char*					GetFontName( void) const;
#endif // ASURA_TOOLS
	int							GetFontID( void) const;

	Asura_Hash_ID				GetFontNameHash( void) const;

	void						SetFontScale( float fFontScale)	{	m_fFontScale = fFontScale; }
	float						GetFontScale( void)const;
	float*						GetFontScalePointer( void);
	
	bool						GetTextRenderFlag( u_int uFlag) const;
	void						SetTextRenderFlag( u_int uFlagsToSet, bool bFlagSet);
	u_int						GetTextRenderFlags( void) const;
	void						SetTextRenderFlags( u_int uFlags)	{	m_uTextRenderFlags = uFlags;	}
	// MRD 15/06/07: Line spacing accessors
	void						SetTextLineSpacing( float fLineSpacing ) { m_fLineSpacing = fLineSpacing; }
	float						GetTextLineSpacing( void ) const { return m_fLineSpacing; }

	void						SetEnableSpecialEffects(bool bEnable);
	bool						AreSpecialEffectsEnabled() const { return m_pxSpecialEffectsSettings != NULL; }

	Asura_Font_Special_Effects*	GetSpecialEffectsSettings() const { return m_pxSpecialEffectsSettings; }

#ifdef ASURA_TOOLS
	virtual void				UpdateLocalDataFromStyle( void);

	void						ClearTextRenderFlags( void);

	virtual	void				ListUsedTextPages(Asura_HashTree& xTextPageTree) const;

	virtual void				WriteToChunkStream( Asura_Chunk_Stream& xStream) const;
	virtual void				ExportToXML(Tools_GUIMenu_XMLExporter* pxExporter) const;
#endif // ASURA_TOOLS

	virtual void				ReadFromChunkStream( Asura_Chunk_Stream& xStream, Asura_GUIMenu_Root* pxParentObject, u_int uChunkVersion);
protected:
	
private:
	void 						AllocStringSpace( u_int uSizeInUnicodeChars);
	void 						ReleaseStringSpace( void);

	void						ApplyModifierChanges( u_int uStateToMatch, Asura_Bounding_Rectangle& xRect, float& fScrollRate, float& fOffsetX, float& fOffsetY, float& fFontScale, u_int& uColour, ASURA_TRANSLUCENCY_MODE& eTranslucencyMode,u_int& uRenderFlags, const Asura_UnicodeChar** ppwszText, int& iFontID);
	void						ApplyAnimatorChanges( Asura_GUIMenu_Animator* pxAnim, u_int uStateToMatch, Asura_Bounding_Rectangle& xRect,float& fScrollRate, float& fOffsetX, float& fOffsetY, float& fFontScale, u_int& uColour, ASURA_TRANSLUCENCY_MODE& eTranslucencyMode,u_int& uRenderFlags, const Asura_UnicodeChar** ppwszText, int& iFontID);

	bool						UpdateScrolling(const Asura_UnicodeChar* wszText, float fRectWidth, float fFontScale);

	// -- Variables -------------------------------------------------------------------
public:

protected:

private:
	Asura_UnicodeChar*				m_pwszText;
	u_int							m_uMaxTextCharsInclNull;

	mutable int						m_iFontID;
#ifdef ASURA_TOOLS
	Asura_String					m_xFontName;
#endif 
	Asura_Hash_ID					m_uFontNameHash;

	Asura_Hash_ID					m_uTextID;//ID of localised text to use (if any)
	Asura_Hash_ID					m_uTextPageHash;

	float							m_fOffsetX;
	float							m_fOffsetY;
	float							m_fFontScale;
	u_int							m_uTextRenderFlags;

	float							m_fScrollRate;//scroll rate in pixels/second

	float							m_fScrollOffset;//current position of scrolling
														
	float							m_fLineSpacing; // MRD 15/06/07: Var for storing the linespacing value

	// Special effects data. Can be NULL. If NULL then this text doesn't have any special effects
	Asura_Font_Special_Effects*		m_pxSpecialEffectsSettings;
	// --------------------------------------------------------------------------------
	typedef Asura_GUIMenu_Element_Base	PARENT;
};
#endif // ASURA_USE_GUI_MENU_SYSTEM
#endif // __ASURA_UI_ELEMENT_IMAGE_H__
