// ------------------------------------------------------------------------------------ //
//																						//
//	File: Asura_GUIMenu_Menu_Page.h														//
//																						//
//	Description: Menu page, container for widgets and other menu pages					//
//																						//
//	Created: Steve Keen - 9/11/06														//
//																						//
//	Copyright (C) 2006 Rebellion (Derby), All Rights Reserved.							//
// ------------------------------------------------------------------------------------ //

#ifndef __ASURA_GUIMENU_MENU_PAGE_H__
#define __ASURA_GUIMENU_MENU_PAGE_H__

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifdef ASURA_USE_GUI_MENU_SYSTEM
#include "GUIMenu/Asura_GUIMenu_Root.h"

// -- Forward declarations ------------------------------------------------------------
class Asura_GUIMenu_Widget_Button;

// -- Defines -------------------------------------------------------------------------

#define ASURA_GUIMENU_MENUPAGE_FLAG_ALLOW_BACK_INTO_THIS	(1<<0)
#define ASURA_GUIMENU_MENUPAGE_FLAG_ALLOW_BACK_FROM_THIS	(1<<1)
#define ASURA_GUIMENU_MENUPAGE_FLAG_CLOSE_THIS_MENU_ON_BACK	(1<<2)
#define ASURA_GUIMENU_MENUPAGE_FLAG_CENTER_MENU_ON_SCREEN	(1<<3)

// SK 16.05.07
// this code update and render only flag is used to notify the system that the menu page cannot be
// activated or used in the normal way.  This is so these specific menu pages dont get updated and
// rendered unless directly  done so in code by a game. 
// Specifically, this is being done for HUD type systems where these HUDS
// are updated and rendered by the player client objects.
#define ASURA_GUIMENU_MENUPAGE_FLAG_CODE_UPDATE_AND_RENDER_ONLY	(1<<4)
#define ASURA_GUIMENU_MENUPAGE_FLAG_RENDER_TO_FULLSCREEN		(1<<5)
#define ASURA_GUIMENU_MENUPAGE_FLAG_PAUSE						(1<<6)
#define ASURA_GUIMENU_MENUPAGE_FLAG_ALLOW_AUTO_RESIZE			(1<<7)

// CLF 07.11.08
// Ability to disable key press ability to back from this menu, this 
// will be disabled automatically when back from this menu is disabled
#define ASURA_GUIMENU_MENUPAGE_FLAG_ALLOW_KEY_PRESS_BACK_FROM_THIS	(1<<8)

// MxH 17.04.09
#define ASURA_GUIMENU_MENUPAGE_FLAG_RENDER_WIDESCREEN_BARS			(1<<9)

#define ASURA_GUIMENU_MENU_PAGE_LOCAL_FLAG_START	16
/*
Versions: ASURA_CHUNK_GUIMENU_MENUPAGE_VERSION

0:	Initial data - SK 5.12.06
1:	Added filename info - RAR
2:	Added platform info - SK 12.02.07
3:	Added Init & DeInit Asura_GUIMenu_ConsoleCmd's - SK 13.03.07
4:  Added the menu page flags - SK 26.03.07
5:  Set defaults for OnInit/OnDeinit style flags. RAR 23/04/2007
6:	Added OnUpdate - CW - 12/02/2008
7:	Platform now transferred to base class. RAR 23/06/2008
8:	Size/position fixup NY 31/07/2008
9:  Addition of key press back flag
10:  Addition of virtual resolution NY 19/11/2008
11: Addition of Widescreen bars tickbox
*/
#define ASURA_CHUNK_GUIMENU_MENUPAGE_VERSION 11

// -- Structures ----------------------------------------------------------------------

// -- Class ---------------------------------------------------------------------------
class Asura_GUIMenu_Widget_Base;


class Asura_GUIMenu_Menu_Page : public Asura_GUIMenu_Root
{
	typedef Asura_GUIMenu_Root PARENT;

	friend class Tools_GUIMenu_Menu_Page;	// for access to m_xCommand when creating tools dialog.

	// -- Functions -----------------------------------------------------------------------
public:
	enum Asura_GUIMenu_Inherit_Flag_Widget_Menu_Page_Type
	{

// Platform no longer used  , there is a flag for the base class instead. RAR 23/06/2008
		OLD_ASURA_GUIMENU_MENU_PAGE_LOCAL_FLAG_PLATFORM		= (1<<(ASURA_GUIMENU_MENU_PAGE_LOCAL_FLAG_START+0)),
// SK 21.02.07 - Removed.  RAR said it's not necessary to have this.
//		ASURA_GUIMENU_MENU_PAGE_LOCAL_FLAG_FILENAME		= (1<<(ASURA_GUIMENU_MENU_PAGE_LOCAL_FLAG_START+1)),
		ASURA_GUIMENU_MENU_PAGE_LOCAL_FLAG_WIDTH		= (1<<(ASURA_GUIMENU_MENU_PAGE_LOCAL_FLAG_START+2)),
		ASURA_GUIMENU_MENU_PAGE_LOCAL_FLAG_HEIGHT		= (1<<(ASURA_GUIMENU_MENU_PAGE_LOCAL_FLAG_START+3)),
		ASURA_GUIMENU_MENU_PAGE_LOCAL_FLAG_MENUPAGE_FLAGS	= (1<<(ASURA_GUIMENU_MENU_PAGE_LOCAL_FLAG_START+4)),
		ASURA_GUIMENU_MENU_PAGE_LOCAL_FLAG_ONINIT		= (1<<(ASURA_GUIMENU_MENU_PAGE_LOCAL_FLAG_START+5)),
		ASURA_GUIMENU_MENU_PAGE_LOCAL_FLAG_ONDEINIT		= (1<<(ASURA_GUIMENU_MENU_PAGE_LOCAL_FLAG_START+6)),
		ASURA_GUIMENU_MENU_PAGE_LOCAL_FLAG_ONUPDATE		= (1<<(ASURA_GUIMENU_MENU_PAGE_LOCAL_FLAG_START+7)),
		//ASURA_GUIMENU_MENU_PAGE_OVERIDE_MAX is here to stop another programmer adding past bit flag 31
		ASURA_GUIMENU_MENU_PAGE_LOCAL_FLAG_MAX  			= (1<<(ASURA_GUIMENU_MENU_PAGE_LOCAL_FLAG_START+15))
	};

	Asura_GUIMenu_Menu_Page();
	virtual ~Asura_GUIMenu_Menu_Page();

	virtual bool				HandleMessage( const Asura_Message_GUIMenu_Notify* pxMessage);

	virtual void				Update( float fElapsedTime);
	virtual void 				Render( u_int uState);
	virtual void				Copy( const Asura_GUIMenu_Root* pxSource );

	virtual void				Resize( const Asura_Bounding_Rectangle& xBounds )			{ PARENT::Resize( xBounds ); }//work round hiding
	virtual void				Resize( const float fNewWidth, const float fNewHeight);

	virtual u_int				ProcessInputActivity( const Asura_GUIMenu_Input& xInput);

	// -- Widget and Element shared functions --
	virtual bool				IsWidgetOnMenu( Asura_Hash_ID uHashId);

#ifdef ASURA_GUIMENU_STORE_NAMES
	virtual void				ForceGenerateAllSubIDs();
	virtual void				OutputGUIMenuIDInfo( const Asura_String& xOutputFileName);
#endif	//ASURA_GUIMENU_STORE_NAMES

	virtual bool				Whisper( const Asura_Message_GUIMenu_Notify* pxNotify);

	virtual bool				AddToThisContainer( Asura_GUIMenu_Root* pxObject);

	Asura_Hash_ID				GetStartupWidgetID() const;
	void						SetStartupWidgetID( Asura_Hash_ID uHashStartupWidgetID);

	Asura_Hash_ID				GetStartupOverrideWidgetID() const;
	void						SetStartupOverrideWidgetID( Asura_Hash_ID uHashStartupOverrideWidgetID);

	Asura_Hash_ID				GetActiveWidgetHash() const {	return m_uHashActiveWidget;	}
	Asura_GUIMenu_Root*			GetActiveWidget()	{	return m_pxActiveWidget;	}
	const Asura_GUIMenu_Root*	GetActiveWidget() const {	return m_pxActiveWidget;	}

	virtual void				ReadFromChunkStream( Asura_Chunk_Stream& xStream, Asura_GUIMenu_Root* pxParentObject, u_int uChunkVersion);

#ifdef ASURA_TOOLS
	const char* 				GetFilename() const;// {return m_xFilename.GetString();}
	void						SetFilename(const char* szFile) {m_xFilename = szFile;}


	virtual void				WriteToChunkStream( Asura_Chunk_Stream& xStream) const;
	virtual void				ExportToXML(Tools_GUIMenu_XMLExporter* pxExporter) const;

	virtual void				UpdateLocalDataFromStyle();

	void						SetVirtualSize(Asura_Vector_2 xVirtualSize) {m_xVirtualSize = xVirtualSize;}
	Asura_Vector_2*				GetVirtualSizePtr() {return &m_xVirtualSize;}

#endif	// ASURA_TOOLS

	const Asura_Vector_2&		GetVirtualSize() const { return m_xVirtualSize;}
	void						GetUnscaledVirtualBounds(Asura_Bounding_Rectangle& xBounds) const;

	bool						HasVirtualSize() const;

	void						GetBackMenuAndWidgetHash( Asura_Hash_ID& uOutMenuHash, Asura_Hash_ID& uOutWidgetHash, bool bGetLocalOnly = false);//	{	return m_uHashBackMenuID;	}
	Asura_Hash_ID				GetBackMenuHash( bool bGetLocalOnly = false);//	{	return m_uHashBackMenuID;	}
	void						ClearBackMenuHash()	{	m_uHashBackMenuID = m_uHashBackMenuID = ASURA_HASH_ID_UNSET; 	}
	void						SetBackMenuHash( Asura_Hash_ID uBackMenuHash, Asura_Hash_ID uBackWidgetHash = ASURA_HASH_ID_UNSET);
	u_int*						GetMenuPageFlagsPtr()	{	return &m_uMenuPageFlags;	}

    void						SetMenuPageFlags( u_int uFlags)	{	m_uMenuPageFlags = uFlags;	}
    u_int						GetMenuPageFlags() const; //	{	return m_uMenuPageFlags;	}

	bool						DoesMenuRenderWidescreenBars() const;

	bool						CanBackIntoThisMenu() const;
	bool						CanBackOutOfThisMenu() const;
	bool						CanKeyPressBackOutOfThisMenu() const;	// CLF (07.11.08)
	bool						ShouldCloseMenuOnBackAction() const;
	void						SetBackButton( Asura_GUIMenu_Widget_Button* pxButton );
	void						ClearBackButton( Asura_GUIMenu_Widget_Button* pxButton );

	bool						CanAutoResize();

	bool						IsFadingOut() const { return (m_uState & ASURA_GUIMENU_ACTION_FLAG_IS_MENU_FADE_OUT) != 0; }
	bool						IsFadingIn() const { return (m_uState & ASURA_GUIMENU_ACTION_FLAG_IS_MENU_FADE_IN) != 0; }
	u_int						GetState() const { return m_uState; }

#ifdef ASURA_TOOLS
	virtual void				PostMenuPreviewCleanup();
#endif

	//Gets the init/deinit command , taking from the inherited object if needed. RAR 23/04/2007
	const Asura_GUIMenu_ConsoleCmd* GetOnInitCommand() const;
	const Asura_GUIMenu_ConsoleCmd* GetOnDeInitCommand() const;
	const Asura_GUIMenu_ConsoleCmd* GetOnUpdateCommand() const;

	void	IncRefCount()	{	m_uRefCount++;	}
	void	DecRefCount()	{	m_uRefCount--;	}
	u_int	GetRefCount()	{	return m_uRefCount;	}

	float	GetTimeActive() const;
	float	GetTimeFocused() const;

	bool	ShouldPause() const { return (m_uMenuPageFlags & ASURA_GUIMENU_MENUPAGE_FLAG_PAUSE) != 0; }

	void	AutoResize();

	void	SelectWidget( Asura_Hash_ID uWidgetHash);
	void	DeselectActiveWidget(void);

protected:

	// -- UI Message handler functions --
	virtual bool				OnInit( const Asura_Message_GUIMenu_Notify* pxMessage);//	{	return false;	}
	virtual bool				OnDeInit( const Asura_Message_GUIMenu_Notify* pxMessage);//	{	return false;	}
	virtual bool				OnDataChange( const Asura_Message_GUIMenu_Notify* pxMessage)	{ return false;	}
	virtual bool				OnDataRequest( const Asura_Message_GUIMenu_Notify* pxMessage);//	{ return false;	}
	virtual bool				OnClicked( const Asura_Message_GUIMenu_Notify* pxMessage)	{ return false;	}
	virtual bool				OnRollIn( const Asura_Message_GUIMenu_Notify* pxMessage)		{ return false;	}
	virtual bool				OnRollOut( const Asura_Message_GUIMenu_Notify* pxMessage)	{ return false;	}
	virtual bool				OnEnable( const Asura_Message_GUIMenu_Notify* pxMessage)	{ return false;	}

	// GetLongestAnimatorDuration() is to help decide transition lengths. Ignoring looping animators, 
	// find the longest duration of animators in this object matching the given state(s), and recurse 
	// into the contained objects.
	virtual float				GetLongestAnimatorDuration( u_int uStateFlags ) const;

	Asura_GUIMenu_Root*						m_pxActiveWidget;
	Asura_Hash_ID							m_uHashActiveWidget;

	Asura_Hash_ID							m_uHashStartupWidget;
	Asura_Hash_ID							m_uStartupWidgetOverride;
	
	Asura_GUIMenu_Widget_Button*			m_pxBackButton;
	Asura_Hash_ID							m_uHashBackMenuID;
	Asura_Hash_ID							m_uHashBackWidgetID;
	u_int									m_uMenuPageFlags;

	Asura_GUIMenu_ConsoleCmd				m_xOnInitCommand;
	Asura_GUIMenu_ConsoleCmd				m_xOnDeInitCommand;
	Asura_GUIMenu_ConsoleCmd				m_xOnUpdateCommand;

	Asura_Vector_2							m_xVirtualSize;

private:
//	void						ClearLocalVariables();
	Asura_GUIMenu_Widget_Base*	FindWidget( const Asura_Hash_ID uWidgetID) const;
	Asura_GUIMenu_Widget_Base*	FindBestWidget() const;

	void						RemoveActiveWidget();

#ifdef ASURA_TOOLS
	Asura_String							m_xFilename;//file that this menu belongs to (minus path/extension)
#endif

	//runtime data
	u_int									m_uState;		// ASURA_GUIMENU_ACTION_FLAG_IS_MENU_FADE_IN, FADE_OUT, ACTIVE or ASURA_GUIMENU_ACTION_NONE
	float									m_fStateTimer;	// for fadein/out states, how long until the menu leaves the transition.
	float									m_fFocusTimer;
	u_int									m_uRefCount;
};
#endif // ASURA_USE_GUI_MENU_SYSTEM
#endif // __ASURA_GUIMENU_MENU_PAGE_H__
