// ------------------------------------------------------------------------------------ //
//																						//
//	File: Asura_GUIMenu_Modifier.h														//
//																						//
//	Description: container for data modifier on objects									//
//																						//
//	Created: Steve Keen - 6/3/07														//
//																						//
//	Copyright (C) 2006 Rebellion (Derby), All Rights Reserved.							//
// ------------------------------------------------------------------------------------ //

#include "Core/Asura.h"

#ifdef ASURA_USE_GUI_MENU_SYSTEM
#include "GUIMenu/Asura_GUIMenu_Modifier.h"

#include <wchar.h>
#include "ChunkSystem/Asura_Chunk_Stream.h"

#include "Core/Asura_Maths.h"
#include "Core/Asura_TextureManagement.h"
#include "GUIMenu/Asura_GUIMenu_Root.h"
#include "GUIMenu/Asura_GUIMenu_Element_Base.h"
#include "GUIMenu/Asura_GUIMenu_Element_Image.h"
#include "GUIMenu/Asura_GUIMenu_Element_Text.h"
#include "GUIMenu/Asura_GUIMenu_Widget_Base.h"
#include "Rendering/Asura_Colour.h"

#ifdef ASURA_TOOLS
#include "GUIMenu/Tools_GUIMenu_XMLExporter.h"
#endif

// --------------------------------------------------------------------------------------- 
// -- Constructors -----------------------------------------------------------------------

Asura_GUIMenu_Modifier::Asura_GUIMenu_Modifier(void)
: m_xActivationConditions()
, m_uModifyState( ASURA_GUIMENU_WIDGET_STATE_DEFAULT)
, m_uModifyType( 0)
, m_eDataType( DATA_TYPE_NONE)
, m_pReserved( NULL)
{
}

Asura_GUIMenu_Modifier::Asura_GUIMenu_Modifier(const Asura_GUIMenu_Modifier& xOther)
: m_xActivationConditions()
, m_uModifyState( ASURA_GUIMENU_WIDGET_STATE_DEFAULT)
, m_uModifyType( 0)
, m_eDataType( DATA_TYPE_NONE)
, m_pReserved( NULL)
{
	*this = xOther;
}

Asura_GUIMenu_Modifier::~Asura_GUIMenu_Modifier(void)
{
	ReleaseExistingData();
}

Asura_GUIMenu_Modifier& Asura_GUIMenu_Modifier::operator=(const Asura_GUIMenu_Modifier& xOther)
{
	if (this == &xOther) 
	{
		Asura_Assert(false, "unexpected copy-to-self");
		return *this;
	}

	Copy( &xOther );
	return *this;
}

// --------------------------------------------------------------------------------------- 
// -- Public Functions ------------------------------------------------------------------- 

void Asura_GUIMenu_Modifier::SetData( u_int uData)
{
	AllocateDataSpace( DATA_TYPE_UINT);

	*m_puData = uData;
}

// --------------------------------------------------------------------------------------- 

void Asura_GUIMenu_Modifier::SetData( int iData)
{
	AllocateDataSpace( DATA_TYPE_INT);

	*m_piData = iData;
}

// --------------------------------------------------------------------------------------- 

void Asura_GUIMenu_Modifier::SetData( float fData)
{
	AllocateDataSpace( DATA_TYPE_FLOAT);

	*m_pfData = fData;
}

// --------------------------------------------------------------------------------------- 

void Asura_GUIMenu_Modifier::SetData( const Asura_RGBA& xColourData)
{
	AllocateDataSpace( DATA_TYPE_RGBA);

	*m_pxColourData = xColourData;
}

// --------------------------------------------------------------------------------------- 

void Asura_GUIMenu_Modifier::SetData( const Asura_Vector_2& xVector2Data)
{
	AllocateDataSpace( DATA_TYPE_VECTOR_2);

	*m_pxVector2Data = xVector2Data;
}

// --------------------------------------------------------------------------------------- 

void Asura_GUIMenu_Modifier::SetData( const Asura_UnicodeChar* pwszUnicodeString)
{
	u_int uStringLength = 0;

	if( pwszUnicodeString != NULL)
	{
		uStringLength = wcslen(pwszUnicodeString)+1;	
		AllocateDataSpace( DATA_TYPE_UNICODE_STRING, uStringLength);

		wcsncpy( m_pwszUnicodeCharStringData, pwszUnicodeString, uStringLength);
	}
}

// --------------------------------------------------------------------------------------- 

void Asura_GUIMenu_Modifier::SetData( const Asura_Char* pcszString)
{
	u_int uStringLength = 0;

	if( pcszString != NULL)
	{
		uStringLength = strlen(pcszString)+1;	
		AllocateDataSpace( DATA_TYPE_CHAR_STRING, uStringLength);

		strncpy( m_pcCharStringData, pcszString, uStringLength);
	}
}

// --------------------------------------------------------------------------------------- 

void Asura_GUIMenu_Modifier::SetData( const Asura_GUIMenu_UV_Set& xUVSet)
{
	AllocateDataSpace( DATA_TYPE_UV_SET);

	*m_pxUVSetData = xUVSet;

}

// --------------------------------------------------------------------------------------- 

void Asura_GUIMenu_Modifier::SetTextureIndexData( int iTextureIndex)
{
	AllocateDataSpace( DATA_TYPE_TEXTUREINDEX);

	*m_piData = iTextureIndex;
}


// --------------------------------------------------------------------------------------- 

u_int Asura_GUIMenu_Modifier::GetUIntData( void) const
{
	Asura_Assert( m_eDataType == DATA_TYPE_UINT, "Asura_GUIMenu_Modifier::GetUIntData - the stored data is not UINT type.");

	return *m_puData;
}

// --------------------------------------------------------------------------------------- 

int	Asura_GUIMenu_Modifier::GetIntData( void) const
{
	Asura_Assert( m_eDataType == DATA_TYPE_INT, "Asura_GUIMenu_Modifier::GetIntData - the stored data is not INT type.");

	return *m_piData;
}

// --------------------------------------------------------------------------------------- 

float Asura_GUIMenu_Modifier::GetFloatData( void) const
{
	Asura_Assert( m_eDataType == DATA_TYPE_FLOAT, "Asura_GUIMenu_Modifier::GetFloatData - the stored data is not FLOAT type.");

	return *m_pfData;
}

// --------------------------------------------------------------------------------------- 

Asura_RGBA Asura_GUIMenu_Modifier::GetColourData( void) const
{
	Asura_Assert( m_eDataType == DATA_TYPE_RGBA, "Asura_GUIMenu_Modifier::GetColourData - the stored data is not RGBA type.");

	return *m_pxColourData;

}

// --------------------------------------------------------------------------------------- 

Asura_Vector_2 Asura_GUIMenu_Modifier::GetVector2Data( void) const
{
	Asura_Assert( m_eDataType == DATA_TYPE_VECTOR_2, "Asura_GUIMenu_Modifier::GetVector2Data - the stored data is not Asura_Vector_2 type.");

	return *m_pxVector2Data;
}

// --------------------------------------------------------------------------------------- 

Asura_UnicodeChar* Asura_GUIMenu_Modifier::GetUnicodeCharStringData( void) const
{
	Asura_Assert( m_eDataType == DATA_TYPE_UNICODE_STRING, "Asura_GUIMenu_Modifier::GetUnicodeCharStringData - the stored data is not Asura_UnicodeChar* type.");

	return m_pwszUnicodeCharStringData;
}

// --------------------------------------------------------------------------------------- 

Asura_Char* Asura_GUIMenu_Modifier::GetCharStringData( void) const
{
	Asura_Assert( m_eDataType == DATA_TYPE_CHAR_STRING, "Asura_GUIMenu_Modifier::GetCharStringData - the stored data is not Asura_Char* type.");

	return m_pcCharStringData;
}

// --------------------------------------------------------------------------------------- 

Asura_GUIMenu_UV_Set* Asura_GUIMenu_Modifier::GetUVSetData( void) const
{
	Asura_Assert( m_eDataType == DATA_TYPE_UV_SET, "Asura_GUIMenu_Modifier::GetUVSetData - the stored data is not Asura_GUIMenu_UV_Set type.");

	return m_pxUVSetData;
}

// --------------------------------------------------------------------------------------- 

int	Asura_GUIMenu_Modifier::GetTextureIndexData( void) const
{
	Asura_Assert( m_eDataType == DATA_TYPE_TEXTUREINDEX, "Asura_GUIMenu_Modifier::GetTextureIndexData - the stored data is not TEXTUREINDEX type.");

	return *m_piData;
}

// --------------------------------------------------------------------------------------- 

bool Asura_GUIMenu_Modifier::ReadFromChunkStream( Asura_Chunk_Stream& xStream )
{
	u_int uChunkVersion;
	xStream >> uChunkVersion;

	if (uChunkVersion > ASURA_CHUNK_MODIFIER_VERSION)
	{
		Asura_Assertf(false, ("GUIMenu Modifier version unknown (%u > %u)", 
							uChunkVersion, ASURA_CHUNK_MODIFIER_VERSION));
		return false;
	}

	u_int uSpare, uCount;
	int iSpare;
	float fSpare;
	Asura_RGBA xColSpare;
	Asura_Vector_2 xVecSpare;
	Asura_GUIMenu_UV_Set xUVSet;

	if( uChunkVersion >= 1 )
	{
		m_xActivationConditions.ReadFromChunkStream( xStream ); // Activation conditions added in chunk version 1
	}

	xStream >> m_uModifyState;
	xStream >> m_uModifyType;
	xStream >> uSpare;

	m_eDataType = (MODIFIER_TYPE)uSpare;

	switch( m_eDataType)
	{
		case DATA_TYPE_UINT:
		{
			xStream >> uSpare;
			SetData( uSpare);
			break;
		}

		case DATA_TYPE_INT:
		{
			xStream >> iSpare;
			SetData( iSpare);
			break;
		}

		case DATA_TYPE_FLOAT:
		{
			xStream >> fSpare;
			SetData( fSpare);
			break;
		}

		case DATA_TYPE_RGBA:
		{
			xStream >> xColSpare.m_fR;
			xStream >> xColSpare.m_fG;
			xStream >> xColSpare.m_fB;
			xStream >> xColSpare.m_fA;
			SetData( xColSpare);
			break;
		}

		case DATA_TYPE_VECTOR_2:
		{
			xStream >> xVecSpare.x;
			xStream >> xVecSpare.y;
			if (uChunkVersion < 2)
			{
				xStream >> fSpare;
			}
			SetData( xVecSpare);
			break;
		}

		case DATA_TYPE_UNICODE_STRING:
		{
			u_int uStrLen;
			xStream >> uStrLen;
			Asura_UnicodeChar* pwszSpare = new Asura_UnicodeChar[uStrLen+1];

			for( uCount=0; uCount<uStrLen; uCount++)
			{
				xStream >> pwszSpare[uCount];
				pwszSpare[uCount+1] = 0x0000;
			}
			SetData( pwszSpare);
			delete []pwszSpare;
			break;
		}

		case DATA_TYPE_CHAR_STRING:
		{
			u_int uStrLen;
			xStream >> uStrLen;
			Asura_Char* pcSpare = new Asura_Char[uStrLen+1];

			for( uCount=0; uCount<uStrLen; uCount++)
			{
				xStream >> pcSpare[uCount];
				pcSpare[uCount+1] = 0x00;
			}
			
			SetData( pcSpare);
			delete []pcSpare;
			break;
		}


		case DATA_TYPE_UV_SET:
		{
			xStream >> xUVSet.xUV0.u;
			xStream >> xUVSet.xUV0.v;
			xStream >> xUVSet.xUV1.u;
			xStream >> xUVSet.xUV1.v;
			SetData( xUVSet);
			break;
		}


		case DATA_TYPE_TEXTUREINDEX :
		{
			//read in a texture name , and convert to an index
			char szTextureName[256];
			xStream.ReadString(szTextureName,sizeof(szTextureName));

			int iTextureIndex = ASURA_TEXTURE_NONE;
			if(szTextureName[0])
			{
				#ifdef ASURA_TOOLS
				bool bAlreadyAdded = Asura_Texture_Management::GetTextureIndex(szTextureName) != ASURA_TEXTURE_NONE;
				#endif
				
				iTextureIndex = Asura_Texture_Management::AddTexture(szTextureName);

				#ifdef ASURA_TOOLS
				if(iTextureIndex != ASURA_TEXTURE_NONE && !bAlreadyAdded)
				{
					Asura_Texture_Management::SetTextureFlags( iTextureIndex, Asura_GUIMenu_Element_Image::GetDefaultTextureFlags());
				}
				#endif
			}
			SetTextureIndexData(iTextureIndex);

			break;
		}

		case DATA_TYPE_NONE:
		{
			break;
		}

		default:
		{
			Asura_Assert(0, "Unhandled case");
			break;
		}
	}

	return true;
}

// --------------------------------------------------------------------------------------- 

void Asura_GUIMenu_Modifier::Copy( const Asura_GUIMenu_Modifier* pxOriginalModifier)
{
	m_xActivationConditions.Copy( &pxOriginalModifier->m_xActivationConditions ); // Activation conditions added in chunk version 1

	m_uModifyState = pxOriginalModifier->m_uModifyState;
	m_uModifyType  = pxOriginalModifier->m_uModifyType;

	switch(pxOriginalModifier->m_eDataType)
	{
		case DATA_TYPE_NONE :
			Asura_Assert(0,"Attempting to copy a modifier that hasn't been assigned a type yet");
			break;
		case DATA_TYPE_UINT :
			SetData(pxOriginalModifier->GetUIntData());
			break;
		case DATA_TYPE_INT :
			SetData(pxOriginalModifier->GetIntData());
			break;
		case DATA_TYPE_FLOAT :
			SetData(pxOriginalModifier->GetFloatData());
			break;
		case DATA_TYPE_RGBA :
			SetData(pxOriginalModifier->GetColourData());
			break;
		case DATA_TYPE_VECTOR_2 :
			SetData(pxOriginalModifier->GetVector2Data());
			break;
		case DATA_TYPE_UNICODE_STRING :
			SetData(pxOriginalModifier->GetUnicodeCharStringData());
			break;
		case DATA_TYPE_CHAR_STRING :
			SetData(pxOriginalModifier->GetCharStringData());
			break;
		case DATA_TYPE_UV_SET :
			SetData(*pxOriginalModifier->GetUVSetData());
			break;
		case DATA_TYPE_TEXTUREINDEX :
			SetTextureIndexData(pxOriginalModifier->GetTextureIndexData());
			break;

		default :
			Asura_Assert(0,"Attempting to copy a modifier that has an unknown type.");
			break;
				
	}

}

// --------------------------------------------------------------------------------------- 

#ifdef ASURA_TOOLS
void Asura_GUIMenu_Modifier::WriteToChunkStream( Asura_Chunk_Stream& xStream ) const
{
	u_int uSpare, uCount;

	xStream << ASURA_CHUNK_MODIFIER_VERSION;

	m_xActivationConditions.WriteToChunkStream( xStream ); // Activation conditions added in chunk version 1

	xStream << m_uModifyState;
	xStream << m_uModifyType;
	
	uSpare = (u_int)m_eDataType;
	xStream << uSpare;

	switch( m_eDataType)
	{
		case DATA_TYPE_UINT:
			if( m_puData)
			{
				xStream << *m_puData;
			}
			else
			{
				xStream << 0L;
			}
			break;

		case DATA_TYPE_INT:
			if( m_piData)
			{
				xStream << *m_piData;
			}
			else
			{
				xStream << 0L;
			}
			break;

		case DATA_TYPE_FLOAT:
			if( m_pfData)
			{
				xStream << *m_pfData;
			}
			else
			{
				xStream << 0.0f;
			}
			break;

		case DATA_TYPE_RGBA:
			if( m_pxColourData)
			{
				xStream << m_pxColourData->m_fR;
				xStream << m_pxColourData->m_fG;
				xStream << m_pxColourData->m_fB;
				xStream << m_pxColourData->m_fA;
			}
			else
			{
				xStream << 0.0f;
				xStream << 0.0f;
				xStream << 0.0f;
				xStream << 0.0f;
			}
			break;

		case DATA_TYPE_VECTOR_2:
			if( m_pxVector2Data)
			{
				xStream << m_pxVector2Data->x;
				xStream << m_pxVector2Data->y;
			}
			else
			{
				xStream << 0.0f;
				xStream << 0.0f;
			}
			break;

		case DATA_TYPE_UNICODE_STRING:
			if( m_pwszUnicodeCharStringData)
			{
				u_int uStrLen = wcslen( m_pwszUnicodeCharStringData);
				Asura_UnicodeChar wszChar;
				xStream << uStrLen;

				for( uCount=0; uCount<uStrLen; uCount++)
				{
					wszChar = m_pwszUnicodeCharStringData[uCount];
					xStream << wszChar;
				}
			}
			else
			{
				xStream << 0L;
			}
			break;

		case DATA_TYPE_CHAR_STRING:
			if( m_pcCharStringData)
			{
				u_int uStrLen = strlen( m_pcCharStringData);
				Asura_Char cByte;
				xStream << uStrLen;
				
				for( uCount=0; uCount<uStrLen; uCount++)
				{
					cByte = m_pcCharStringData[uCount];
					xStream << cByte;
				}
			}
			else
			{
				xStream << 0L;
			}
			break;

		case DATA_TYPE_UV_SET:
			if( m_pxUVSetData)
			{
				xStream << m_pxUVSetData->xUV0.u;
				xStream << m_pxUVSetData->xUV0.v;
				xStream << m_pxUVSetData->xUV1.u;
				xStream << m_pxUVSetData->xUV1.v;
			}
			else
			{
				xStream << 0.0f;
				xStream << 0.0f;
				xStream << 0.0f;
				xStream << 0.0f;
			}
			break;

		case DATA_TYPE_TEXTUREINDEX :
		{
			int iTextureIndex = m_piData ? *m_piData : ASURA_TEXTURE_NONE;

			const Asura_String* pxTextureName = Asura_Texture_Management::GetTextureName(iTextureIndex);
			if(pxTextureName)
			{
				xStream << *pxTextureName;
			}
			else
			{
				Asura_String xBlankString;
				xStream << xBlankString;
			}
			break;
	
		}

		case DATA_TYPE_NONE:
		default:
			break;
	}
}

void Asura_GUIMenu_Modifier::ExportToXML( Tools_GUIMenu_XMLExporter* pxExporter ) const
{
	pxExporter->AddToBuffer("<Modifier>\n");
	pxExporter->IncTabs();	

	m_xActivationConditions.ExportToXML( pxExporter ); // Activation conditions added in chunk version 1

	pxExporter->AddToBuffer( pxExporter->GetTabs() );
	pxExporter->AddToBuffer( "<modify_state>" );
	pxExporter->AddToBuffer( m_uModifyState );
	pxExporter->AddToBuffer( "</modify_state>\n" );

	pxExporter->AddToBuffer( pxExporter->GetTabs() );
	pxExporter->AddToBuffer( "<modify_type>" );
	pxExporter->AddToBuffer( m_uModifyType );
	pxExporter->AddToBuffer( "</modify_type>\n" );

	pxExporter->AddToBuffer( pxExporter->GetTabs() );
	pxExporter->AddToBuffer( "<data_type>" );
	pxExporter->AddToBuffer( static_cast<u_int>(m_eDataType) );
	pxExporter->AddToBuffer( "</data_type>\n" );

	pxExporter->AddToBuffer( pxExporter->GetTabs() );
	pxExporter->AddToBuffer( "<data>\n" );
	pxExporter->IncTabs();

	switch( m_eDataType)
	{
		case DATA_TYPE_UINT:
			if( m_puData )
			{
				pxExporter->AddToBuffer( pxExporter->GetTabs() );
				pxExporter->AddToBuffer("<uint>");
				pxExporter->AddToBuffer( *m_puData );
				pxExporter->AddToBuffer("</uint>");
			}
			break;

		case DATA_TYPE_INT:
			if( m_puData )
			{
				pxExporter->AddToBuffer( pxExporter->GetTabs() );
				pxExporter->AddToBuffer("<int>");
				pxExporter->AddToBuffer( *m_piData );
				pxExporter->AddToBuffer("</int>");
			}
			break;

		case DATA_TYPE_FLOAT:
			if( m_pfData )
			{
				pxExporter->AddToBuffer( pxExporter->GetTabs() );
				pxExporter->AddToBuffer("<float>");
				pxExporter->AddToBuffer( *m_pfData );
				pxExporter->AddToBuffer("</float>");
			}
			break;

		case DATA_TYPE_RGBA:
			if( m_pxColourData)
			{
				pxExporter->AddToBuffer( pxExporter->GetTabs() );
				pxExporter->AddToBuffer("<colour>\n");
				pxExporter->IncTabs();

				pxExporter->AddToBuffer( pxExporter->GetTabs() ); pxExporter->AddToBuffer( "<r>" );
				pxExporter->AddToBuffer( m_pxColourData->m_fR ); pxExporter->AddToBuffer( "</r>\n" );

				pxExporter->AddToBuffer( pxExporter->GetTabs() ); pxExporter->AddToBuffer( "<g>" );
				pxExporter->AddToBuffer( m_pxColourData->m_fG ); pxExporter->AddToBuffer( "</g>\n" );

				pxExporter->AddToBuffer( pxExporter->GetTabs() ); pxExporter->AddToBuffer( "<b>" ); 
				pxExporter->AddToBuffer( m_pxColourData->m_fB ); pxExporter->AddToBuffer( "</b>\n" );

				pxExporter->AddToBuffer( pxExporter->GetTabs() ); pxExporter->AddToBuffer( "<a>" ); 
				pxExporter->AddToBuffer( m_pxColourData->m_fA ); pxExporter->AddToBuffer( "</a>\n" );

				pxExporter->DecTabs();
				pxExporter->AddToBuffer( pxExporter->GetTabs() );
				pxExporter->AddToBuffer( "</colour>\n" );
			}
			break;

		case DATA_TYPE_VECTOR_2:
			if( m_pxVector2Data)
			{
				pxExporter->AddToBuffer( pxExporter->GetTabs() );
				pxExporter->AddToBuffer("<vector>\n");
				pxExporter->IncTabs();

				pxExporter->AddToBuffer( pxExporter->GetTabs() ); pxExporter->AddToBuffer( "<x>" ); 
				pxExporter->AddToBuffer( m_pxVector2Data->x ); pxExporter->AddToBuffer( "</x>\n" );

				pxExporter->AddToBuffer( pxExporter->GetTabs() ); pxExporter->AddToBuffer( "<y>" ); 
				pxExporter->AddToBuffer( m_pxVector2Data->y ); pxExporter->AddToBuffer( "</y>\n" );

				pxExporter->DecTabs();
				pxExporter->AddToBuffer( pxExporter->GetTabs() );
				pxExporter->AddToBuffer( "</vector>\n" );
			}
			break;

		case DATA_TYPE_UNICODE_STRING:
			if( m_pwszUnicodeCharStringData)
			{
				u_int uStrLen = wcslen( m_pwszUnicodeCharStringData );
				//the maximum size in chars the string could be is 3 times this, if all characters were triple byte
				char* szOut = new char[3*uStrLen];
				Asura_UnicodeToUTF8( m_pwszUnicodeCharStringData, szOut );

				pxExporter->AddToBuffer( pxExporter->GetTabs() );
				pxExporter->AddToBuffer("<unicode_str>");
				pxExporter->AddToBufferEscaped( szOut );
				pxExporter->AddToBuffer("</unicode_str>\n");
				delete[] szOut;
			}
			break;

		case DATA_TYPE_CHAR_STRING:
			if( m_pcCharStringData)
			{
				pxExporter->AddToBuffer( pxExporter->GetTabs() );
				pxExporter->AddToBuffer("<char_str>");
				pxExporter->AddToBufferEscaped( m_pcCharStringData );
				pxExporter->AddToBuffer("</char_str>\n");
			}
			break;

		case DATA_TYPE_UV_SET:
			if( m_pxUVSetData)
			{
				pxExporter->AddToBuffer( pxExporter->GetTabs() );
				pxExporter->AddToBuffer("<uv_set>\n");
				pxExporter->IncTabs();

				pxExporter->AddToBuffer( pxExporter->GetTabs() ); pxExporter->AddToBuffer( "<uv0_u>" ); 
				pxExporter->AddToBuffer( m_pxUVSetData->xUV0.u ); pxExporter->AddToBuffer( "</uv0_u>\n" );

				pxExporter->AddToBuffer( pxExporter->GetTabs() ); pxExporter->AddToBuffer( "<uv0_v>" ); 
				pxExporter->AddToBuffer( m_pxUVSetData->xUV0.v ); pxExporter->AddToBuffer( "</uv0_v>\n" );

				pxExporter->AddToBuffer( pxExporter->GetTabs() ); pxExporter->AddToBuffer( "<uv1_u>" ); 
				pxExporter->AddToBuffer( m_pxUVSetData->xUV1.u ); pxExporter->AddToBuffer( "</uv1_u>\n" );

				pxExporter->AddToBuffer( pxExporter->GetTabs() ); pxExporter->AddToBuffer( "<uv1_v>" ); 
				pxExporter->AddToBuffer( m_pxUVSetData->xUV1.v ); pxExporter->AddToBuffer( "</uv1_v>\n" );

				pxExporter->DecTabs();
				pxExporter->AddToBuffer( pxExporter->GetTabs() );
				pxExporter->AddToBuffer( "</uv_set>\n" );

			}
			break;

		case DATA_TYPE_TEXTUREINDEX :
		{
			int iTextureIndex = m_piData ? *m_piData : ASURA_TEXTURE_NONE;

			const Asura_String* pxTextureName = Asura_Texture_Management::GetTextureName(iTextureIndex);
			if(pxTextureName)
			{
				pxExporter->AddToBuffer( pxExporter->GetTabs() );
				pxExporter->AddToBuffer("<texture_name>");
				pxExporter->AddToBufferEscaped( pxTextureName->GetString() );
				pxExporter->AddToBuffer("</texture_name>\n");
			}
			break;
	
		}

		case DATA_TYPE_NONE:
		default:
			break;
	}

	pxExporter->DecTabs();
	pxExporter->AddToBuffer( pxExporter->GetTabs() );
	pxExporter->AddToBuffer( "</data>\n" );
	
	pxExporter->DecTabs();
	pxExporter->AddToBuffer("</Modifier>\n");
}

#endif

// --------------------------------------------------------------------------------------- 
// -- Private functions ------------------------------------------------------------------

void Asura_GUIMenu_Modifier::ReleaseExistingData( void)
{
	switch( m_eDataType)
	{
		case DATA_TYPE_UINT:
			if( m_puData)
			{
				delete m_puData;
				m_puData = NULL;
			}
			break;

		case DATA_TYPE_TEXTUREINDEX://texture index done as an int
		case DATA_TYPE_INT:
			if( m_piData)
			{
				delete m_piData;
				m_piData = NULL;
			}
			break;

		case DATA_TYPE_FLOAT:
			if( m_pfData)
			{
				delete m_pfData;
				m_pfData = NULL;
			}
			break;

		case DATA_TYPE_RGBA:
			if( m_pxColourData)
			{
				delete m_pxColourData;
				m_pxColourData = NULL;
			}
			break;

		case DATA_TYPE_VECTOR_2:
			if( m_pxVector2Data)
			{
				delete m_pxVector2Data;
				m_pxVector2Data = NULL;
			}
			break;

		case DATA_TYPE_UNICODE_STRING:
			if( m_pwszUnicodeCharStringData)
			{
				delete []m_pwszUnicodeCharStringData;
				m_pwszUnicodeCharStringData = NULL;
			}
			break;

		case DATA_TYPE_CHAR_STRING:
			if( m_pcCharStringData)
			{
				delete []m_pcCharStringData;
				m_pcCharStringData = NULL;
			}
			break;

		case DATA_TYPE_UV_SET:
			if( m_pxUVSetData)
			{
				delete m_pxUVSetData;
				m_pxUVSetData = NULL;
			}
			break;


		case DATA_TYPE_NONE:
		default:
			Asura_Assert( NULL == m_pReserved, "Asura_GUIMenu_Modifier::ReleaseExistingData - no data type set but data appears to be present.");
			m_pReserved = NULL;
			break;
	}
}

// --------------------------------------------------------------------------------------- 

void Asura_GUIMenu_Modifier::AllocateDataSpace( MODIFIER_TYPE eNewDataType, u_int uStringAllocationSize)
{
	ReleaseExistingData();

	switch( eNewDataType)
	{
		case DATA_TYPE_UINT:
			m_puData = new u_int;
			break;

		case DATA_TYPE_TEXTUREINDEX://texture index done as an int
		case DATA_TYPE_INT:
			m_piData = new int;
			break;

		case DATA_TYPE_FLOAT:
			m_pfData = new float;
			break;

		case DATA_TYPE_RGBA:
			m_pxColourData = new Asura_RGBA;
			break;

		case DATA_TYPE_VECTOR_2:
			m_pxVector2Data = new Asura_Vector_2;	
			break;

		case DATA_TYPE_UNICODE_STRING:
			m_pwszUnicodeCharStringData = new Asura_UnicodeChar[uStringAllocationSize];
			break;

		case DATA_TYPE_CHAR_STRING:
			m_pcCharStringData = new Asura_Char[uStringAllocationSize];
			break;

		case DATA_TYPE_UV_SET:
			m_pxUVSetData = new Asura_GUIMenu_UV_Set;
			break;

		case DATA_TYPE_NONE:
		default:
			m_pReserved = NULL;
			break;
	}

	m_eDataType = eNewDataType;
}

// --------------------------------------------------------------------------------------- 

// static function
Asura_GUIMenu_Modifier::MODIFIER_TYPE Asura_GUIMenu_Modifier::GetDataTypeForModifyType( ASURA_GUIMENU_MODIFY_TYPE eModifyType )
{
	switch (eModifyType)
	{
		case ASURA_GUIMENU_MODIFY_POSITION:                 return DATA_TYPE_VECTOR_2;
		case ASURA_GUIMENU_MODIFY_COLOUR:                   return DATA_TYPE_RGBA;
		case ASURA_GUIMENU_MODIFY_TRANSLUCENCY_MODE:        return DATA_TYPE_INT;
		case ASURA_GUIMENU_MODIFY_WIDTH:                    return DATA_TYPE_FLOAT;
		case ASURA_GUIMENU_MODIFY_HEIGHT:                   return DATA_TYPE_FLOAT;
		case ASURA_GUIMENU_MODIFY_TEXT_STRING:              return DATA_TYPE_UNICODE_STRING;
		case ASURA_GUIMENU_MODIFY_TEXT_OFFSET_X:            return DATA_TYPE_FLOAT;
		case ASURA_GUIMENU_MODIFY_TEXT_OFFSET_Y:            return DATA_TYPE_FLOAT;
		case ASURA_GUIMENU_MODIFY_TEXT_RENDER_FLAGS:        return DATA_TYPE_UINT;
		case ASURA_GUIMENU_MODIFY_TEXT_ID:                  return DATA_TYPE_INT;
		case ASURA_GUIMENU_MODIFY_FONT_SCALE:               return DATA_TYPE_FLOAT;
		case ASURA_GUIMENU_MODIFY_FONT_NAME:                return DATA_TYPE_CHAR_STRING;
		case ASURA_GUIMENU_MODIFY_TEXTURE:                  return DATA_TYPE_TEXTUREINDEX;
		case ASURA_GUIMENU_MODIFY_UVS:                      return DATA_TYPE_UV_SET;
		case ASURA_GUIMENU_MODIFY_ALPHA_SCALED:             return DATA_TYPE_FLOAT;
		case ASURA_GUIMENU_MODIFY_WIDTH_SCALED:             return DATA_TYPE_FLOAT;
		case ASURA_GUIMENU_MODIFY_HEIGHT_SCALED:            return DATA_TYPE_FLOAT;
		case ASURA_GUIMENU_MODIFY_TEXT_SCROLL_RATE:         return DATA_TYPE_FLOAT;
		case ASURA_GUIMENU_MODIFY_IMAGE_ROTATE:				return DATA_TYPE_FLOAT;
		case ASURA_GUIMENU_MODIFY_POSITION_RELATIVE:        return DATA_TYPE_VECTOR_2;
		case ASURA_GUIMENU_MODIFY_SIZE_SCALE:        return DATA_TYPE_VECTOR_2;
		default:
		{
			Asura_Assert(false, "invalid modify type");
			return DATA_TYPE_NONE;
		}
	}
}

#ifdef ASURA_TOOLS
// static function
u_int Asura_GUIMenu_Modifier::GetCorrespondingInheritanceFlag( ASURA_GUIMENU_MODIFY_TYPE eModifyType )
{
	switch (eModifyType)
	{
		case ASURA_GUIMENU_MODIFY_POSITION:                 return Asura_GUIMenu_Root::ASURA_GUIMENU_ROOT_LOCAL_FLAG_POSITION;
		case ASURA_GUIMENU_MODIFY_COLOUR:                   return Asura_GUIMenu_Root::ASURA_GUIMENU_ROOT_LOCAL_FLAG_COLOUR;
		case ASURA_GUIMENU_MODIFY_TRANSLUCENCY_MODE:        return Asura_GUIMenu_Root::ASURA_GUIMENU_ROOT_LOCAL_FLAG_TRANSLUCENCY;
		case ASURA_GUIMENU_MODIFY_WIDTH:                    return Asura_GUIMenu_Element_Base::ASURA_GUIMENU_ELEMENT_BASE_LOCAL_FLAG_WIDTH;
		case ASURA_GUIMENU_MODIFY_HEIGHT:                   return Asura_GUIMenu_Element_Base::ASURA_GUIMENU_ELEMENT_BASE_LOCAL_FLAG_HEIGHT;
		case ASURA_GUIMENU_MODIFY_TEXT_STRING:              return Asura_GUIMenu_Element_Text::ASURA_GUIMENU_ELEMENT_TEXT_LOCAL_FLAG_TEXT;
		case ASURA_GUIMENU_MODIFY_TEXT_OFFSET_X:            return Asura_GUIMenu_Element_Text::ASURA_GUIMENU_ELEMENT_TEXT_LOCAL_FLAG_OFFSET_X;
		case ASURA_GUIMENU_MODIFY_TEXT_OFFSET_Y:            return Asura_GUIMenu_Element_Text::ASURA_GUIMENU_ELEMENT_TEXT_LOCAL_FLAG_OFFSET_Y;
		case ASURA_GUIMENU_MODIFY_TEXT_RENDER_FLAGS:        return Asura_GUIMenu_Element_Text::ASURA_GUIMENU_ELEMENT_TEXT_LOCAL_FLAG_RENDER_FLAGS;
		case ASURA_GUIMENU_MODIFY_TEXT_ID:                  return Asura_GUIMenu_Element_Text::ASURA_GUIMENU_ELEMENT_TEXT_LOCAL_FLAG_TEXT_ID;
		case ASURA_GUIMENU_MODIFY_FONT_SCALE:               return Asura_GUIMenu_Element_Text::ASURA_GUIMENU_ELEMENT_TEXT_LOCAL_FLAG_FONT_SCALE;
		case ASURA_GUIMENU_MODIFY_FONT_NAME:                return Asura_GUIMenu_Element_Text::ASURA_GUIMENU_ELEMENT_TEXT_LOCAL_FLAG_FONT_NAME;
		case ASURA_GUIMENU_MODIFY_TEXTURE:                  return Asura_GUIMenu_Element_Image::ASURA_GUIMENU_ELEMENT_IMAGE_LOCAL_FLAG_TEXTURE;
		case ASURA_GUIMENU_MODIFY_UVS:                      return Asura_GUIMenu_Element_Image::ASURA_GUIMENU_ELEMENT_IMAGE_LOCAL_FLAG_UVS;
		case ASURA_GUIMENU_MODIFY_ALPHA_SCALED:				return Asura_GUIMenu_Root::ASURA_GUIMENU_ROOT_LOCAL_FLAG_COLOUR;
		case ASURA_GUIMENU_MODIFY_WIDTH_SCALED:             return Asura_GUIMenu_Element_Base::ASURA_GUIMENU_ELEMENT_BASE_LOCAL_FLAG_WIDTH;
		case ASURA_GUIMENU_MODIFY_HEIGHT_SCALED:            return Asura_GUIMenu_Element_Base::ASURA_GUIMENU_ELEMENT_BASE_LOCAL_FLAG_HEIGHT;
		case ASURA_GUIMENU_MODIFY_TEXT_SCROLL_RATE:			return Asura_GUIMenu_Element_Text::ASURA_GUIMENU_ELEMENT_TEXT_LOCAL_FLAG_OFFSET_X;
		case ASURA_GUIMENU_MODIFY_IMAGE_ROTATE:				return Asura_GUIMenu_Element_Image::ASURA_GUIMENU_ELEMENT_IMAGE_LOCAL_FLAG_ROTATION;
		case ASURA_GUIMENU_MODIFY_POSITION_RELATIVE:        return Asura_GUIMenu_Root::ASURA_GUIMENU_ROOT_LOCAL_FLAG_POSITION;
		case ASURA_GUIMENU_MODIFY_SIZE_SCALE:			    return Asura_GUIMenu_Element_Base::ASURA_GUIMENU_ELEMENT_BASE_LOCAL_FLAG_WIDTH | Asura_GUIMenu_Element_Base::ASURA_GUIMENU_ELEMENT_BASE_LOCAL_FLAG_HEIGHT;
		default:
		{
			Asura_Assert(false, "invalid modify type");
			return 0;
		}
	}
}

// static function
const char* Asura_GUIMenu_Modifier::GetModifyTypeName( ASURA_GUIMENU_MODIFY_TYPE eModifyType )
{
	switch (eModifyType)
	{
		case ASURA_GUIMENU_MODIFY_POSITION:				return "Position";
		case ASURA_GUIMENU_MODIFY_COLOUR:				return "Colour";
		case ASURA_GUIMENU_MODIFY_TRANSLUCENCY_MODE:	return "Translucency Mode";
		case ASURA_GUIMENU_MODIFY_WIDTH:				return "Width";
		case ASURA_GUIMENU_MODIFY_HEIGHT:				return "Height";
		case ASURA_GUIMENU_MODIFY_TEXT_STRING:			return "Text String";
		case ASURA_GUIMENU_MODIFY_TEXT_OFFSET_X:		return "Text X offset";
		case ASURA_GUIMENU_MODIFY_TEXT_OFFSET_Y:		return "Text Y offset";
		case ASURA_GUIMENU_MODIFY_TEXT_RENDER_FLAGS:	return "Text render flags";
		case ASURA_GUIMENU_MODIFY_TEXT_ID:				return "Text ID";
		case ASURA_GUIMENU_MODIFY_FONT_SCALE:			return "Font Scale";
		case ASURA_GUIMENU_MODIFY_FONT_NAME:			return "Font";
		case ASURA_GUIMENU_MODIFY_TEXTURE:				return "Texture";
		case ASURA_GUIMENU_MODIFY_UVS:					return "UVs";
		case ASURA_GUIMENU_MODIFY_ALPHA_SCALED:			return "Alpha Scale";
		case ASURA_GUIMENU_MODIFY_WIDTH_SCALED:			return "Width Scale";
		case ASURA_GUIMENU_MODIFY_HEIGHT_SCALED:		return "Height Scale";
		case ASURA_GUIMENU_MODIFY_TEXT_SCROLL_RATE:		return "Text Scroll Rate";
		case ASURA_GUIMENU_MODIFY_IMAGE_ROTATE:			return "Image Rotate";
		case ASURA_GUIMENU_MODIFY_POSITION_RELATIVE:	return "Relative Position";
		case ASURA_GUIMENU_MODIFY_SIZE_SCALE:			return "Size Scale";
		default:
		{
			Asura_Assert(false, "invalid type");
			return "unknown";
		}
	}
}
#endif

#endif // ASURA_USE_GUI_MENU_SYSTEM
