// ------------------------------------------------------------------------------------ //
//																						//
//	File: Asura_GUIMenu_Widget_Button.cpp												//
//																						//
//	Description: button widgets class													//
//																						//
//	Created: Steve Keen - 9/11/06														//
//																						//
//	Copyright (C) 2006 Rebellion (Derby), All Rights Reserved.							//
// ------------------------------------------------------------------------------------ //

#include "Core/Asura.h"
#include "GUIMenu/Asura_GUIMenu_Widget_Button.h"

#ifdef ASURA_USE_GUI_MENU_SYSTEM
#include "Core/Asura_Hash.h"
#include "Core/Asura_String.h"

#include "GUIMenu/Asura_GUIMenu_Audio.h"
#include "GUIMenu/Asura_GUIMenu_Animator.h"
#include "GUIMenu/Asura_GUIMenu_Message.h"
#include "GUIMenu/Asura_GUIMenu_MessagePump.h"
#include "GUIMenu/Asura_GUIMenu_Menu_Page.h"
#include "GUIMenu/Asura_GUIMenu_System.h"
#include "GUIMenu/Asura_Profiling_GUIMenu.h"

#ifdef ASURA_TOOLS
#include "GUIMenu/Tools_GUIMenu_PreviewOverlay.h"
#include "GUIMenu/Tools_GUIMenu_XMLExporter.h"
#include "GUIMenu/Tools_Mode_GUIMenu.h"
#endif //ASURA_TOOLS

// --------------------------------------------------------------------------------------- 
// -- Constructors -----------------------------------------------------------------------

// SK 23.03.07 - I've changed the default button flags to have the 'close menu on clicked flag' set
// as this would be the most common setting used.
Asura_GUIMenu_Widget_Button::Asura_GUIMenu_Widget_Button(void)
: m_uButtonFlags( ASURA_GUIMENU_BUTTON_FLAG_CLOSE_MENU_ONCLICKED )
, m_uHashLinkToObject( ASURA_HASH_ID_UNSET )
, m_fMinTimeBeforeActive( 0.0f )
, m_fTimeBeforeAutoclick( 0.0f )
{
	m_uType = ASURA_GUIMENU_TYPEID_WIDGET_BUTTON;

#if 0 && defined(ASURA_TOOLS) && defined(_DEBUG)
	if (Asura_Random::GetIntU(0,1))
	{
		AddAnimator("test-animator-flyon")->SetSpeed(Asura_Random::GetFloat( 0.3f, 1.0f ));
		AddAnimator("test-animator-flyoff")->SetSpeed(Asura_Random::GetFloat( 0.3f, 1.0f ));
	}
#endif

	ASURA_GUIMENU_PROFILE_COUNTINSTANCE(ASURA_GUIMENU_COUNTER__WIDGET_BUTTON);
}

// --------------------------------------------------------------------------------------- 
// -- Destructor -------------------------------------------------------------------------

Asura_GUIMenu_Widget_Button::~Asura_GUIMenu_Widget_Button(void)
{
	ASURA_GUIMENU_PROFILE_COUNTDESTROYINSTANCE(ASURA_GUIMENU_COUNTER__WIDGET_BUTTON);
}

// --------------------------------------------------------------------------------------- 
// -- Public Functions ------------------------------------------------------------------- 

// --------------------------------------------------------------------------------------- 

void Asura_GUIMenu_Widget_Button::Update( float fElapsedTime)
{
	ASURA_GUIMENU_PROFILE_START_TASK(ASURA_GUIMENU_PROFILE__WIDGET_BUTTON);

	const Asura_GUIMenu_Menu_Page* pxMenuPage = FindOwnerMenu();
	Asura_Assert(pxMenuPage && pxMenuPage->GetType() == Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_MENUPAGE, "");
	if (!pxMenuPage)
	{
		ASURA_GUIMENU_PROFILE_END_TASK();
		return;
	}

	float fMenuActiveTime = pxMenuPage->GetTimeFocused();
	bool bHiddenByTimer = fMenuActiveTime < m_fMinTimeBeforeActive;
	m_xDisplayConditions.SetForceHidden(bHiddenByTimer);

	// not calling parent update as that just changes roll on/off states to highlighted.
	// Root update manages the display conditions so do this after set forced hidden.
	Asura_GUIMenu_Root::Update( fElapsedTime);

	// check for autoclick
	if (IsSelectable() && IsButtonFlagSet(ASURA_GUIMENU_BUTTON_FLAG_AUTOCLICK) && 
		fMenuActiveTime >= m_fTimeBeforeAutoclick && 
		Asura_GUIMenu_System::GetFocusedMenuHashID() == pxMenuPage->GetID())
	{
#ifdef ASURA_TOOLS
		if (Tools_Mode_GUIMenu::IsPreview())
		{
			Tools_GUIMenu_PreviewOverlay::NotifyAutoclick( GetNameAsChar() );
			if (!Tools_GUIMenu_PreviewOverlay::IsAutoclickEnabled())
			{
				ASURA_GUIMENU_PROFILE_END_TASK();
				return;
			}
		}
#endif
		if (GetCurrentState() != ASURA_GUIMENU_WIDGET_STATE_CLICKED)
		{
			// PAZ - (possibly temporary) fix for seemingly non rendering any menu for a frame.
			Asura_GUIMenu_MessagePump::PostMessage( GetID(), GetID(), ASURA_GUIMENU_UI_MSG_ON_CLICKED, GetType(), ASURA_GUIMENU_ACTION_BUTTON_CLICK );
//			DoClickActions( ASURA_GUIMENU_ACTION_BUTTON_ACCEPT );
		}
		ASURA_GUIMENU_PROFILE_END_TASK();
		return;
	}

	if (IsSelectable() && IsButtonFlagSet(ASURA_GUIMENU_BUTTON_FLAG_POLL_BUTTON_PRESSES) && 
		Asura_GUIMenu_System::GetFocusedMenuHashID() == pxMenuPage->GetID())
	{
		Asura_GUIMenu_Menu_Page* pxPage = FindOwnerMenu();
		
		if( pxPage->GetActiveWidgetHash() != GetID())
		{
			u_int uAcceptType = 0;
			u_int uButtonPresses = Asura_GUIMenu_System::GetCurrentButtonPresses();
			u_int uAcceptButtons = 0;

			for( u_int u=ASURA_GUIMENU_ACTION_BITSHIFT_UP; u<=ASURA_GUIMENU_ACTION_BITSHIFT_ALT_ACCEPT9; u++)
			{
				uAcceptType = (1<<u);

				if( (uButtonPresses&uAcceptType) != 0)
				{
					uAcceptButtons |= uAcceptType;
				}
			}

			if( uAcceptButtons)
			{
				DoClickActions( uAcceptButtons);
				ASURA_GUIMENU_PROFILE_END_TASK();
				return;
			}
		}
	}

	switch( GetCurrentState())
	{
		case ASURA_GUIMENU_WIDGET_STATE_DEFAULT:
			break;

		case ASURA_GUIMENU_WIDGET_STATE_ROLL_ON:
	//		m_iCountDown--;
	//		if( m_iCountDown <= 0)
			{
				SetCurrentState( ASURA_GUIMENU_WIDGET_STATE_HILIGHTED);
			}
			break;

		case ASURA_GUIMENU_WIDGET_STATE_HILIGHTED:
		{
			if (!IsSelectable())
			{
				// find a linked item which is selectable
				for (u_int u = 0; u < ASURA_GUIMENU_UI_NAV_EXIT_MAX; ++u)
				{
					Asura_Hash_ID uExitHash = GetNavExitID( static_cast<Asura_GUIMenu_Navigation_Exit>(u) );
					if (uExitHash != ASURA_HASH_ID_UNSET)
					{
						// -- notify the next item to be selected
						Asura_GUIMenu_MessagePump::PostMessage( uExitHash, GetID(), ASURA_GUIMENU_UI_MSG_ON_ROLL_IN, 0);
						
						// -- notify the current item, to be deselected --
						Asura_GUIMenu_MessagePump::PostMessage( GetID(), GetID(), ASURA_GUIMENU_UI_MSG_ON_ROLL_OUT, 0);

						break;
					}
				}
			}
			break;
		}

		case ASURA_GUIMENU_WIDGET_STATE_CLICKED:
	//		m_iCountDown--;
	//		if( m_iCountDown <= 0)
			{
				SetCurrentState( ASURA_GUIMENU_WIDGET_STATE_HILIGHTED);
			}
			break;

		case ASURA_GUIMENU_WIDGET_STATE_ROLL_OFF:
	//		m_iCountDown--;
	//		if( m_iCountDown <= 0)
			{
				SetCurrentState( ASURA_GUIMENU_WIDGET_STATE_DEFAULT);
			}
			break;

		case ASURA_GUIMENU_WIDGET_STATE_EDITING:
			break;

		case ASURA_GUIMENU_WIDGET_STATE_DISABLED:
		default:
			break;
	}

	ASURA_GUIMENU_PROFILE_END_TASK();
}

// --------------------------------------------------------------------------------------- 

void Asura_GUIMenu_Widget_Button::Copy( const Asura_GUIMenu_Root* pxSource )
{
	PARENT::Copy( pxSource );

	const Asura_GUIMenu_Widget_Button* pxSourceButton = static_cast<const Asura_GUIMenu_Widget_Button*> (pxSource);

	SetButtonFlags(pxSourceButton->GetButtonFlags());
	SetLinkToHash(pxSourceButton->GetLinkToHash());
	//	m_xCommand.Copy(&pxSourceButton->m_xCommand);
	m_xCommandManager.Copy(&pxSourceButton->m_xCommandManager);

	m_fMinTimeBeforeActive = pxSourceButton->m_fMinTimeBeforeActive;
	m_fTimeBeforeAutoclick = pxSourceButton->m_fTimeBeforeAutoclick;
}

// --------------------------------------------------------------------------------------- 
// -- UI Message handler functions -------------------------------------------------------

bool Asura_GUIMenu_Widget_Button::OnInit( const Asura_Message_GUIMenu_Notify* pxMessage)
{
	// most widgets would post a data request here, but a button doesn't need to know anything so don't bother
	PARENT::OnInit( pxMessage);

	if (TrapBackKeyPress() && IsSelectable())
	{
		Asura_GUIMenu_Menu_Page* pxMenuPage = FindOwnerMenu();
		Asura_Assert(pxMenuPage, "buttons should always be on a page");
		if (pxMenuPage)
		{
			pxMenuPage->SetBackButton(this);
		}
	}

	m_xDisplayConditions.SetForceHidden(false);
	return true;
}

// --------------------------------------------------------------------------------------- 

bool Asura_GUIMenu_Widget_Button::OnDeInit( const Asura_Message_GUIMenu_Notify* pxMessage)
{
	if (TrapBackKeyPress() && IsSelectable())
	{
		Asura_GUIMenu_Menu_Page* pxMenuPage = FindOwnerMenu();
		Asura_Assert(pxMenuPage, "buttons should always be on a page");
		if (pxMenuPage)
		{
			pxMenuPage->ClearBackButton(this);
		}
	}

	m_xDisplayConditions.SetForceHidden(false);
	return true;
}

// --------------------------------------------------------------------------------------- 

bool Asura_GUIMenu_Widget_Button::OnDataChange( const Asura_Message_GUIMenu_Notify* pxMessage)
{
	return true;
}

// --------------------------------------------------------------------------------------- 

bool Asura_GUIMenu_Widget_Button::OnDataRequest( const Asura_Message_GUIMenu_Notify* pxMessage)
{
	return true;
}

// --------------------------------------------------------------------------------------- 

bool Asura_GUIMenu_Widget_Button::OnClicked( const Asura_Message_GUIMenu_Notify* pxMessage)
{
	if (!IsSelectable())
	{
		return false;
	}

	DoClickActions(pxMessage->uData);
	return true;
}

void Asura_GUIMenu_Widget_Button::DoClickActions( u_int uButton, bool bProcessAcceptOnly /*= false*/)
{
	// CW - 17/06/08 - Should not allow two pages to update their input in the same frame.
	// In some cases, update order dependent, on pressing a button to change page, the source and
	// target pages would both update their input. If both pages had identical buttons then they would be triggered.
	// The following check will return here if the page updating is not the one that first tried to handle the input.
	Asura_GUIMenu_Menu_Page* pxPage = FindOwnerMenu();
	if(Asura_GUIMenu_System::GetPageHandledInputHash() != ASURA_HASH_ID_UNSET)
	{
		if(pxPage && (pxPage->GetID() != Asura_GUIMenu_System::GetPageHandledInputHash())) return;	
	}
	Asura_GUIMenu_System::SetPageHandledInputHash(pxPage->GetID());

	//Only process clicks from buttons we're interested in (or accept if there aren't any defined)
	bool bShouldClick = (((m_xCommandManager.GetActionButtonCount() == 0) &&  (uButton & ASURA_GUIMENU_ACTION_BUTTON_ACCEPT)) || bProcessAcceptOnly);

	if (bShouldClick) SetCurrentState( ASURA_GUIMENU_WIDGET_STATE_CLICKED);
//	m_iCountDown = 20;

	//Converting 'click' to accept for the purposes of determining what to do.
	//RAR 01/08/2007
	if(uButton & ASURA_GUIMENU_ACTION_BUTTON_CLICK)
	{
		uButton &= ~ASURA_GUIMENU_ACTION_BUTTON_CLICK;
		uButton |= ASURA_GUIMENU_ACTION_BUTTON_ACCEPT;
	}

	// SK - 14.03.07 - I've changed it so the button widget will only use a hotlink when it's
	// activated by the main 'Accept' button (currently 'X')
	
	bool bCloseMenu = false;
	bool bShouldCommandManagerPlaySoundEffect = true;
	// Binary AND so that we can catch multiple button presses or overlapping mappings.

	if( (bProcessAcceptOnly == true) || (m_xCommandManager.IsDefaultAcceptOveridden() == false))
	{
		if( uButton & ASURA_GUIMENU_ACTION_BUTTON_ACCEPT )
		{
			bool bIsSoundEffectPlayed = false;
			bCloseMenu = UseHotLink( bIsSoundEffectPlayed );
			// (CJW - 09.07.2007) If UseHotLink() plays a sound effect, 
			// no need for another one made by the command manager.
			bShouldCommandManagerPlaySoundEffect = !bIsSoundEffectPlayed;
		}
	}

	if( bShouldClick &&
		(GetFlag( ASURA_GUIMENU_ACTION_FLAG_NOTIFY_DATA_CHANGE_IMMEDIATE) == true) && 
		(uButton & ASURA_GUIMENU_ACTION_BUTTON_ACCEPT))
	{
		Asura_GUIMenu_MessagePump::PostMessage( ASURA_GUIMENU_EXTERNAL_NOTIFY, GetID(), ASURA_GUIMENU_UI_MSG_ON_CLICKED, ASURA_GUIMENU_TYPEID_WIDGET_BUTTON);
	}

	// -- uData is the accept button type
	if( !bProcessAcceptOnly)
	{
		m_xCommandManager.Execute( uButton, bShouldCommandManagerPlaySoundEffect, this);
	}

	// close the menu if necessary
	if( bCloseMenu )
	{
		Asura_GUIMenu_Root* pxOwnerMenuOfThis = Asura_GUIMenu_System::FindParentMenuOfObjectID( GetID());
		if( pxOwnerMenuOfThis )
		{
			Asura_GUIMenu_System::CloseMenuPage( pxOwnerMenuOfThis->GetID() );
		}		
	}
}


// --------------------------------------------------------------------------------------- 

bool Asura_GUIMenu_Widget_Button::OnRollIn( const Asura_Message_GUIMenu_Notify* pxMessage)
{
	PARENT::OnRollIn(pxMessage); // CW - Added PARENT call
	
	SetCurrentState( ASURA_GUIMENU_WIDGET_STATE_ROLL_ON);
//	m_iCountDown = 20;
	
	return true;
}

// --------------------------------------------------------------------------------------- 

bool Asura_GUIMenu_Widget_Button::OnRollOut( const Asura_Message_GUIMenu_Notify* pxMessage)
{
	PARENT::OnRollOut(pxMessage); // CW - Added PARENT call

	SetCurrentState( ASURA_GUIMENU_WIDGET_STATE_ROLL_OFF);
//	m_iCountDown = 60;
	return true;
}

// --------------------------------------------------------------------------------------- 

bool Asura_GUIMenu_Widget_Button::OnEnable( const Asura_Message_GUIMenu_Notify* pxMessage)
{
	Asura_GUIMenu_Widget_State eNewState;

	if( pxMessage->uData != 0)
	{	
		if( IsFocusedObject())
		{	eNewState = ASURA_GUIMENU_WIDGET_STATE_HILIGHTED;	}
		else
		{	eNewState = ASURA_GUIMENU_WIDGET_STATE_DEFAULT;	}
	}
	else
	{
		eNewState = ASURA_GUIMENU_WIDGET_STATE_DISABLED;	
	}

	SetCurrentState( eNewState);

	return true;
}

// --------------------------------------------------------------------------------------- 

// UseHotLink returns true if the button should close the menu at the end of DoClickActions
bool Asura_GUIMenu_Widget_Button::UseHotLink( bool& bIsSoundEffectPlayed )
{
	Asura_Hash_ID uHashHotLink = GetLinkToHash();

	bIsSoundEffectPlayed = false;

	// -- Is there a hot link?
	if( uHashHotLink != ASURA_HASH_ID_UNSET)
	{
		// -- see if it's a valid object --
		Asura_GUIMenu_Root* pxHotLinkObject = Asura_GUIMenu_System::FindRootObjectByID( uHashHotLink);
		Asura_Assert(pxHotLinkObject, "failed to find hotlinked menu");
		if( pxHotLinkObject != NULL)
		{
			Asura_GUIMenu_Root* pxOwnerMenuOfThis = Asura_GUIMenu_System::FindParentMenuOfObjectID( GetID());
			Asura_GUIMenu_Root* pxOwnerMenuOfHotLink = Asura_GUIMenu_System::FindParentMenuOfObjectID( uHashHotLink);

			// -- Is it a menu id or not?
			if( pxHotLinkObject->GetType() != Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_MENUPAGE)
			{
				// -- it's a widget id --
				// -- figure out whether it's on same menu page as this button --
				if( pxOwnerMenuOfThis == pxOwnerMenuOfHotLink)
				{
					// -- pointers match, so it's on the same menu page --
					Asura_GUIMenu_MessagePump::PostMessage( uHashHotLink, GetID(), ASURA_GUIMENU_UI_MSG_ON_ROLL_IN, 0);
					Asura_GUIMenu_MessagePump::PostMessage( GetID(), GetID(), ASURA_GUIMENU_UI_MSG_ON_ROLL_OUT, 0);
				}
				else
				{
					Asura_GUIMenu_Menu_Page* pxHotLinkMenu = static_cast<Asura_GUIMenu_Menu_Page*>(pxOwnerMenuOfHotLink);

					PlayHotLinkSoundEffect();
					bIsSoundEffectPlayed = true;

					bool bControlFocus = true;
					bool bSetBackLink = !DontSetBackLink() && !DuplicateBackLink();
					Asura_GUIMenu_System::OpenMenuPageOnWidget( pxHotLinkMenu->GetID(), uHashHotLink, bControlFocus, bSetBackLink );
					if (DuplicateBackLink())
					{
						Asura_GUIMenu_Menu_Page* pxOwnerMenu = static_cast<Asura_GUIMenu_Menu_Page*>(pxOwnerMenuOfThis);
						Asura_Hash_ID uBackMenu = ASURA_HASH_ID_UNSET;
						Asura_Hash_ID uBackWidget = ASURA_HASH_ID_UNSET;
						pxOwnerMenu->GetBackMenuAndWidgetHash(uBackMenu, uBackWidget, true);
						pxHotLinkMenu->SetBackMenuHash(uBackMenu, uBackWidget);
					}

					// -- This is the original code, the line above is the new one.

					// -- need to startup another menu and activate the focused widget if any --
//					Asura_GUIMenu_System::OpenMenuPage( pxHotLinkMenu->GetID());
					//uHashNewMenu = pxHotLinkMenu->GetActiveWidgetHash();
					//Asura_GUIMenu_MessagePump::PostMessage( uHashHotLink, GetID(), ASURA_GUIMENU_UI_MSG_ON_ROLL_IN, 0);
					//// -- this makes sure that the roll-in & out widget aren't the same thing.
					//if( uHashHotLink != uHashNewMenu)
					//{
					//	Asura_GUIMenu_MessagePump::PostMessage( uHashNewMenu, GetID(), ASURA_GUIMENU_UI_MSG_ON_ROLL_OUT, 0);
					//}

					if( IsButtonFlagSet( ASURA_GUIMENU_BUTTON_FLAG_CLOSE_MENU_ONCLICKED) == true)
					{
						// -- Close the current menu page --
						return (pxOwnerMenuOfThis != NULL);
					}
				}
			}
			else
			{
				// -- it's a menu id --
				// need to startup another menu.  There is no widget to focus, so it would use the default widget --
				if( pxOwnerMenuOfThis != pxHotLinkObject)
				{
					Asura_GUIMenu_Menu_Page* pxHotLinkMenu = static_cast<Asura_GUIMenu_Menu_Page*>(pxHotLinkObject);

					PlayHotLinkSoundEffect();
					bIsSoundEffectPlayed = true;

					// -- make sure the hot link isn't the same menu page, it it is; don't do anything but assert.
					bool bControlFocus = true;
					bool bSetBackLink = !DontSetBackLink() && !DuplicateBackLink();
					Asura_GUIMenu_System::OpenMenuPage( uHashHotLink, bControlFocus, bSetBackLink );
					if (DuplicateBackLink())
					{
						Asura_GUIMenu_Menu_Page* pxOwnerMenu = static_cast<Asura_GUIMenu_Menu_Page*>(pxOwnerMenuOfThis);
						Asura_Hash_ID uBackMenu = ASURA_HASH_ID_UNSET;
						Asura_Hash_ID uBackWidget = ASURA_HASH_ID_UNSET;
						pxOwnerMenu->GetBackMenuAndWidgetHash(uBackMenu, uBackWidget, true);
						pxHotLinkMenu->SetBackMenuHash(uBackMenu, uBackWidget);
					}

					if( IsButtonFlagSet( ASURA_GUIMENU_BUTTON_FLAG_CLOSE_MENU_ONCLICKED) == true)
					{
						// -- Close the current menu page --
						return (pxOwnerMenuOfThis != NULL);
					}
				}
				else
				{
					Asura_Assertf(false, ("Asura_GUIMenu_Widget_Button::UseHotLink - The hotlink set in %s on Menu Page %s is set to activate the same menu page.  This will do nothing.", GetNameAsChar(), pxOwnerMenuOfThis->GetNameAsChar()));
				}
			}
		}
	}
	else
	{
		if (IsButtonFlagSet( ASURA_GUIMENU_BUTTON_FLAG_GO_BACK))
		{
			Asura_GUIMenu_Menu_Page* pxOwnerMenu = static_cast<Asura_GUIMenu_Menu_Page*>(Asura_GUIMenu_System::FindParentMenuOfObjectID(GetID()));
			if( pxOwnerMenu)
			{
				if (pxOwnerMenu->CanBackOutOfThisMenu())
				{
					bool bIsCodeUpdate = false;

					// -- play the cancel sound
					Asura_GUIMenu_Audio::Play( ASURA_GUIMENU_AUDIO_TYPE_ON_CANCEL);

					// -- returning back to the last page so don't set a backlink.
					Asura_Hash_ID uMenuHash, uWidgetHash;
					pxOwnerMenu->GetBackMenuAndWidgetHash( uMenuHash, uWidgetHash);
					
					if (uMenuHash)
					{
						Asura_GUIMenu_Menu_Page* pxPageToBackInto = Asura_GUIMenu_System::GetMenuPageByID( uMenuHash);
						if( pxPageToBackInto)
						{
							if( (pxPageToBackInto->GetMenuPageFlags() & ASURA_GUIMENU_MENUPAGE_FLAG_CODE_UPDATE_AND_RENDER_ONLY) != 0)
							{
								bIsCodeUpdate = true;
							}
						}

						if( bIsCodeUpdate)
						{
							Asura_GUIMenu_System::InitMenuPageData( uMenuHash, true);
						}
						else
						{
							Asura_GUIMenu_System::OpenMenuPageOnWidget( uMenuHash, uWidgetHash, true, false);
						}
					}

					pxOwnerMenu->ClearBackMenuHash();

					if( pxOwnerMenu->ShouldCloseMenuOnBackAction())
					{
						if( (pxOwnerMenu->GetMenuPageFlags() & ASURA_GUIMENU_MENUPAGE_FLAG_CODE_UPDATE_AND_RENDER_ONLY) != 0)
						{
							Asura_GUIMenu_System::DeInitMenuPageData( GetID());
						}
						else
						{
							Asura_GUIMenu_System::CloseMenuPage( GetID());
						}
					}
					return true;
				}
			}
		}
		else if( IsButtonFlagSet( ASURA_GUIMENU_BUTTON_FLAG_CLOSE_MENU_ONCLICKED) == true)
		{
			// -- Close the current menu page --
			Asura_GUIMenu_Root* pxOwnerMenuOfThis = Asura_GUIMenu_System::FindParentMenuOfObjectID( GetID());
			if( pxOwnerMenuOfThis != NULL)
			{
				PlayHotLinkSoundEffect();
				bIsSoundEffectPlayed = true;
				return true;
			}
		}
	}

	return false;
}

// (03.07.2007 - CJW)
void Asura_GUIMenu_Widget_Button::PlayHotLinkSoundEffect(void)
{
	// Don't play sound if this is an autoclick button.
	if( !IsButtonFlagSet( ASURA_GUIMENU_BUTTON_FLAG_AUTOCLICK ) )
	{
		if( TrapBackKeyPress() )
		{
			// play the cancel sound
			Asura_GUIMenu_Audio::Play( ASURA_GUIMENU_AUDIO_TYPE_ON_CANCEL );
		}
		else
		{
			// play the accept sound, we're leaving edit mode
			Asura_GUIMenu_Audio::Play( ASURA_GUIMENU_AUDIO_TYPE_ON_ACCEPT );
		}
	}
}

// --------------------------------------------------------------------------------------- 

Asura_Hash_ID Asura_GUIMenu_Widget_Button::GetLinkToHash( void) const
{
	return m_uHashLinkToObject;
}

// --------------------------------------------------------------------------------------- 

bool Asura_GUIMenu_Widget_Button::IsButtonFlagSet( u_int uButtonFlag) const
{
	return ((m_uButtonFlags&uButtonFlag) != 0);
}

bool Asura_GUIMenu_Widget_Button::DontSetBackLink(void) const
{
	return IsButtonFlagSet(ASURA_GUIMENU_BUTTON_FLAG_DONT_SET_BACKLINK);
}

bool Asura_GUIMenu_Widget_Button::DuplicateBackLink(void) const
{
	return IsButtonFlagSet(ASURA_GUIMENU_BUTTON_FLAG_DUPLICATE_BACKLINK);
}

bool Asura_GUIMenu_Widget_Button::TrapBackKeyPress(void) const
{
	return IsButtonFlagSet(ASURA_GUIMENU_BUTTON_FLAG_TRAP_BACK_KEYPRESS);
}


// --------------------------------------------------------------------------------------- 

void Asura_GUIMenu_Widget_Button::SetButtonFlag( u_int uButtonFlag, bool bTrueOrFalse)
{
	if( bTrueOrFalse)
	{
		m_uButtonFlags |= uButtonFlag;
	}
	else
	{
		m_uButtonFlags &= (0xffffffff - uButtonFlag);
	}
}

// --------------------------------------------------------------------------------------- 

u_int Asura_GUIMenu_Widget_Button::GetButtonFlags( void) const
{
	return m_uButtonFlags;
}

// --------------------------------------------------------------------------------------- 

u_int* Asura_GUIMenu_Widget_Button::GetButtonFlagsAsPtr( void)
{
	return &m_uButtonFlags;
}

// --------------------------------------------------------------------------------------- 
void Asura_GUIMenu_Widget_Button::SetButtonFlags( u_int uButtonFlags)
{
	m_uButtonFlags = uButtonFlags;
}

// --------------------------------------------------------------------------------------- 
#ifdef ASURA_TOOLS
void Asura_GUIMenu_Widget_Button::UpdateLocalDataFromStyle( void)
{
	Asura_GUIMenu_Widget_Button* pxStyleButton;

	PARENT::UpdateLocalDataFromStyle();

	if( m_pxStyleObject != NULL)
	{
		if( (m_uStyleUseLocalDataFlags&ASURA_GUIMENU_WIDGET_BUTTON_LOCAL_FLAG_LINKTO) == 0)
		{
			pxStyleButton = static_cast<Asura_GUIMenu_Widget_Button*>(m_pxStyleObject);
			SetLinkToHash( pxStyleButton->GetLinkToHash());
		}

		if( (m_uStyleUseLocalDataFlags&ASURA_GUIMENU_WIDGET_BUTTON_LOCAL_FLAG_FLAGS) == 0)
		{
			pxStyleButton = static_cast<Asura_GUIMenu_Widget_Button*>(m_pxStyleObject);
			SetButtonFlags( pxStyleButton->GetButtonFlags());

			m_fMinTimeBeforeActive = pxStyleButton->m_fMinTimeBeforeActive;
			m_fTimeBeforeAutoclick = pxStyleButton->m_fTimeBeforeAutoclick;
		}
	}
}
#endif
// --------------------------------------------------------------------------------------- 

void Asura_GUIMenu_Widget_Button::ReadFromChunkStream( Asura_Chunk_Stream& xStream, Asura_GUIMenu_Root* pxParentObject, u_int uChunkVersion)
{
//	u_int u, uContainerCount;

	PARENT::ReadFromChunkStream( xStream, pxParentObject, 0xffffffff);

	ReadAllChildObjects( xStream, this);

	xStream >> m_uHashLinkToObject;
	xStream >> m_uButtonFlags;

	// V1 - read console command.
	// TODO: deal with chunk read failure.
	if( uChunkVersion == 1)
	{
		m_xCommandManager.SetActionButtonCount( 1);
		Asura_GUIMenu_ActionButtonCommand* pxAction = m_xCommandManager.GetActionButtonCommandData( 0);
		if( pxAction)
		{
			pxAction->m_xCommand.ReadFromChunkStream( xStream);
		}
	}

	// V2 - read console commands through the manager system
	if( uChunkVersion >= 2)
	{
		m_xCommandManager.ReadFromChunkStream( xStream );
	}

	if (uChunkVersion >= 3)
	{
		xStream >> m_fMinTimeBeforeActive;
		xStream >> m_fTimeBeforeAutoclick;
	}
}

// --------------------------------------------------------------------------------------- 
#ifdef ASURA_TOOLS
void Asura_GUIMenu_Widget_Button::WriteToChunkStream( Asura_Chunk_Stream& xStream) const
{
	WriteLocalChunkHeader( xStream, ASURA_CHUNK_WIDGET_BUTTON_VERSION, GetType());

	PARENT::WriteToChunkStream( xStream);

	// -- write out any children --
	WriteChildObjectData( xStream);

	// -- write out button specific data --
	xStream << m_uHashLinkToObject;
	xStream << m_uButtonFlags;

	m_xCommandManager.WriteToChunkStream( xStream);

	xStream << m_fMinTimeBeforeActive;
	xStream << m_fTimeBeforeAutoclick;
}

void Asura_GUIMenu_Widget_Button::ExportToXML( Tools_GUIMenu_XMLExporter* pxExporter ) const
{
	PARENT::ExportToXML( pxExporter );
	pxExporter->AddToBuffer( pxExporter->GetTabs() );
	pxExporter->AddToBuffer( "<Button_data>\n" );
	pxExporter->IncTabs();
	
	// -- write out button specific data --
	pxExporter->AddToBuffer( pxExporter->GetTabs() );
	pxExporter->AddToBuffer( "<hash_link>" );
	pxExporter->AddToBufferHex( m_uHashLinkToObject );
	pxExporter->AddToBuffer( "</hash_link>\n" );

	pxExporter->AddToBuffer( pxExporter->GetTabs() );
	pxExporter->AddToBuffer( "<button_flags>" );
	pxExporter->AddToBufferHex( m_uButtonFlags );
	pxExporter->AddToBuffer( "</button_flags>\n" );

	m_xCommandManager.ExportToXML( pxExporter);

	pxExporter->AddToBuffer( pxExporter->GetTabs() );
	pxExporter->AddToBuffer( "<min_time_before_active>" );
	pxExporter->AddToBuffer( m_fMinTimeBeforeActive );
	pxExporter->AddToBuffer( "</min_time_before_active>" );

	pxExporter->AddToBuffer( pxExporter->GetTabs() );
	pxExporter->AddToBuffer( "<time_before_autoclick>" );
	pxExporter->AddToBuffer( m_fTimeBeforeAutoclick );
	pxExporter->AddToBuffer( "</time_before_autoclick>" );

	pxExporter->DecTabs();
	pxExporter->AddToBuffer( pxExporter->GetTabs() );
	pxExporter->AddToBuffer( "</Button_data>\n" );
}

#endif // ASURA_TOOLS

// --------------------------------------------------------------------------------------- 
// -- Protected Functions ------------------------------------------------------------------

void Asura_GUIMenu_Widget_Button::SetupDefaultElements( void)
{
}

// --------------------------------------------------------------------------------------- 
// -- Private Functions ------------------------------------------------------------------


// --------------------------------------------------------------------------------------- 

#endif // ASURA_USE_GUI_MENU_SYSTEM
