//
// Asura_GUIMenu_Widget_ListBox.h
//
// Created by RDM - 23 April 2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __ASURA_GUIMENU_WIDGET_LISTBOX_H__
#define __ASURA_GUIMENU_WIDGET_LISTBOX_H__

#ifdef ASURA_USE_GUI_MENU_SYSTEM

/************/
/* Includes */
/************/

// asura
#include "Core/Asura_HashTree.h"
#include "GUIMenu/Asura_GUIMenu_Element_Base.h"
#include "GUIMenu/Asura_GUIMenu_Widget_Base.h"
#include "GUIMenu/Asura_GUIMenu_CommandManager.h"

/*************/
/* Constants */
/*************/
enum Asura_GUIMenu_ListBox_Message
{
	ASURA_GUIMENU_LISTBOX_NONE = 0,
	ASURA_GUIMENU_LISTBOX_CLEAR,
	ASURA_GUIMENU_LISTBOX_GET_SELECTION_INDEX,
	ASURA_GUIMENU_LISTBOX_SET_SELECTION_INDEX,
	ASURA_GUIMENU_LISTBOX_END_OF_MESSAGES
};

#define fASURA_GUIMENU_WIDGET_LISTBOX_FLASH_TIME_DEFAULT	(0.1f)
#define fASURA_GUIMENU_WIDGET_LISTBOX_CONTROL_INITIAL_REPEAT_DELAY			( 0.5f)
#define fASURA_GUIMENU_WIDGET_LISTBOX_CONTROL_SECONDARY_REPEAT_DELAY		( 0.2f)

#define ASURA_GUIMENU_WIDGET_LISTBOX_LOCAL_FLAG_START	16

// 0 -> 5 ?
// 6 : size data moved to parent - NY - 01/08/2008
// 7 : added offset vector, to control orientation of list
// 8 : added flash code from numeric/text lists
// 9 : added init value console var
// 10 : added current value console var
// 11 : Don't export population/visibility functions for widgets that aren't exported for the current platform.
// 12 : NY obsoleted carousel, added partial scrolling
#define ASURA_CHUNK_WIDGET_LISTBOX_VERSION	12
// 0 : original version
// 1 : version bumped to deal with offset data coming into use
#define ASURA_CHUNK_WIDGET_LISTBOXENTRY_VERSION  1

/***********/
/* Classes */
/***********/

// forward declarations
class Asura_GUIMenu_Element_Base;
class Asura_GUIMenu_Element_Image;
class Asura_GUIMenu_Element_Path;
class Asura_GUIMenu_Widget_ListBox;

class Asura_GUIMenu_ListBoxEntryDesc_Image
{
public:

	Asura_GUIMenu_ListBoxEntryDesc_Image( int iTextureID, float fU1, float fV1, float fU2, float fV2, float fX, float fY, float fW, float fH )
	{
		//m_uMagicID = uASURA_GUIMENU_LISTBOXENTRY_IMAGE;
		m_bHasRenderCoords = true;
		m_iTextureID = iTextureID;
		m_xTopLeftUV.u = fU1;
		m_xTopLeftUV.v = fV1;
		m_xBottomRightUV.u = fU2;
		m_xBottomRightUV.v = fV2;
		m_fPosX = fX;
		m_fPosY = fY;
		m_xDims.x = fW;
		m_xDims.y = fH;
	}

	Asura_GUIMenu_ListBoxEntryDesc_Image( int iTextureID, float fU1, float fV1, float fU2, float fV2 )
	{
		//m_uMagicID = uASURA_GUIMENU_LISTBOXENTRY_IMAGE;
		m_bHasRenderCoords = false;
		m_iTextureID = iTextureID;
		m_xTopLeftUV.u = fU1;
		m_xTopLeftUV.v = fV1;
		m_xBottomRightUV.u = fU2;
		m_xBottomRightUV.v = fV2;
		m_fPosX = 0.0f;
		m_fPosY = 0.0f;
		m_xDims.x = 0.0f;
		m_xDims.y = 0.0f;
	}

	//bool IsValid() const { return m_uMagicID == uASURA_GUIMENU_LISTBOXENTRY_IMAGE; }
	bool HasRenderCoords() const { return m_bHasRenderCoords; }
	int GetTextureID() const { return m_iTextureID; }
	const Asura_GUIMenu_UV& GetTopLeftUV() const { return m_xTopLeftUV; }
	const Asura_GUIMenu_UV& GetBottomRightUV() const { return m_xBottomRightUV; }
	float GetPosX() const { return m_fPosX; }
	float GetPosY() const { return m_fPosY; }
	const Asura_Vector_2& GetDims() const { return m_xDims; }

private:

	//Asura_Hash_ID m_uMagicID;			// must be uASURA_GUIMENU_LISTBOXENTRY_IMAGE
	bool m_bHasRenderCoords;
	int m_iTextureID;
	Asura_GUIMenu_UV m_xTopLeftUV;
	Asura_GUIMenu_UV m_xBottomRightUV;
	float m_fPosX;
	float m_fPosY;
	Asura_Vector_2 m_xDims;
};

class Asura_GUIMenu_ListBoxEntryDesc_Anim
{
public:

	Asura_GUIMenu_ListBoxEntryDesc_Anim( Asura_Hash_ID uSkinID, Asura_Hash_ID uAnimID, u_int uAnimFlags, Asura_Vector_3 xCameraOffset, bool bColourise )
	{
		//m_uMagicID = uASURA_GUIMENU_LISTBOXENTRY_ANIM;
		m_uSkinID = uSkinID;
		m_uAnimID = uAnimID;
		m_uAnimFlags = uAnimFlags;
		m_xCameraOffset = xCameraOffset;
		m_bColourise = bColourise;
		m_bHasCameraOffset = true;
	}

	Asura_GUIMenu_ListBoxEntryDesc_Anim( Asura_Hash_ID uSkinID, Asura_Hash_ID uAnimID, u_int uAnimFlags )
	{
		//m_uMagicID = uASURA_GUIMENU_LISTBOXENTRY_ANIM;
		m_uSkinID = uSkinID;
		m_uAnimID = uAnimID;
		m_uAnimFlags = uAnimFlags;
		m_xCameraOffset.Zero();
		m_bColourise = false;
		m_bHasCameraOffset = false;
	}

	//bool IsValid() const { return m_uMagicID == uASURA_GUIMENU_LISTBOXENTRY_ANIM; }
	Asura_Hash_ID GetSkinID() const { return m_uSkinID; }
	Asura_Hash_ID GetAnimID() const { return m_uAnimID; }
	u_int GetAnimFlags() const { return m_uAnimFlags; }
	const Asura_Vector_3& GetCameraOffset() const { return m_xCameraOffset; }
	bool GetColourise() const { return m_bColourise; }
	bool HasCameraOffset() const { return m_bHasCameraOffset; }

private:

	//Asura_Hash_ID m_uMagicID;			// must be uASURA_GUIMENU_LISTBOXENTRY_ANIM
	Asura_Hash_ID m_uSkinID;
	Asura_Hash_ID m_uAnimID;
	u_int m_uAnimFlags;
	Asura_Vector_3 m_xCameraOffset;
	bool m_bColourise;
	bool m_bHasCameraOffset;

};


class Asura_GUIMenu_Widget_ListBoxEntry : public Asura_GUIMenu_Widget_Base
{
	friend class Tools_GUIMenu_Widget_ListBoxEntry;

private:
	typedef Asura_GUIMenu_Widget_Base PARENT;

public:
	// constructor/destructor
	Asura_GUIMenu_Widget_ListBoxEntry();
	virtual ~Asura_GUIMenu_Widget_ListBoxEntry() {};	

	// public functions
	virtual bool AddToThisContainer( Asura_GUIMenu_Root* pxObject );
	virtual bool RemoveFromThisContainer( Asura_GUIMenu_Root* pxObject );
	virtual void ChildIDChanged( Asura_Hash_ID uOldID, Asura_Hash_ID uNewID );
	virtual void ReadFromChunkStream( Asura_Chunk_Stream& xStream, Asura_GUIMenu_Root* pxParentObject, u_int uChunkVersion);
	virtual Asura_GUIMenu_Root*	GetParentObjectForIDGeneration() const;
#ifdef ASURA_TOOLS
	virtual void WriteToChunkStream( Asura_Chunk_Stream& xStream) const;
	virtual void ExportToXML(Tools_GUIMenu_XMLExporter* pxExporter) const;
#endif // ASURA_TOOLS
	virtual bool OnDeInit( const Asura_Message_GUIMenu_Notify* pxMessage){	return false;	}

	bool IsEntryTemplate() const;
	bool PopulateElements( u_int uEntry, const Asura_GUIMenu_Widget_ListBoxEntry& xTemplate );
	void SetParentListBox( Asura_GUIMenu_Widget_ListBox* pxParent );

	bool HasDataChanged() const { return m_bDataChanged; }

	// pseudo-static functions
	static void AddTextElementCallback( const Asura_UnicodeChar* wszPopulationData)  
	{ 
		s_pxPopulationEntry->AddTextElement(wszPopulationData); 
	}
	void AddTextElement( const Asura_UnicodeChar* wszPopulationData);

	static void AddImageElementOldCallback( const Asura_UnicodeChar* wszPopulationData);
	static void AddImageElementCallback( const Asura_GUIMenu_ListBoxEntryDesc_Image& xImageDesc )
	{ 
		s_pxPopulationEntry->AddImageElement(xImageDesc);
	}
	void AddImageElement( const Asura_GUIMenu_ListBoxEntryDesc_Image& xImageDesc );

	static void AddAnimElementCallback( const Asura_UnicodeChar* wszPopulationData)  
	{ 
		s_pxPopulationEntry->AddAnimElement(wszPopulationData); 
	}
	void AddAnimElement( const Asura_UnicodeChar* wszPopulationData);

protected:
	static void VisibilityCallback( const Asura_UnicodeChar* wszOutput );

	// helper functions
	void SetupDefaultElements() {};	
	bool ShouldUsePopulationFunction( const Asura_GUIMenu_ConsoleCmd* pxPopFunction ) const;
	Asura_GUIMenu_ConsoleCmd* GetPopulationFunctionForItem( Asura_Hash_ID uItem ) const;
	Asura_GUIMenu_Element_Base* GetElementToPopulate( Asura_GUIMenu_Element_Base* pxTemplateElement );
	Asura_GUIMenu_ConsoleCmd* GetVisibilityFunctionForItem( Asura_Hash_ID uItem ) const;

	bool IsDifferent( const Asura_UnicodeChar* pwszTextString ) const;
	bool IsDifferent( const u_int uSkinID, const u_int uAnimID, const u_int uAnimFlags ) const;
	bool IsDifferent( const Asura_GUIMenu_ListBoxEntryDesc_Image& xImageDesc ) const;

	// data
	Asura_GUIMenu_Widget_ListBox* m_pxParentListBox;
	bool m_bDataChanged;

	// static data
	static Asura_GUIMenu_Widget_ListBoxEntry* s_pxPopulationEntry;
	static const Asura_GUIMenu_Widget_ListBoxEntry* s_pxPopulationTemplate;
	static u_int s_uPopulationItem;
	static bool s_bVisibileFromCallback;
};



class Asura_GUIMenu_Widget_ListBox : public Asura_GUIMenu_Widget_Base
{	
	friend class Asura_Chunk_GUIMenu;			// for access to m_xCommand; until we have ReadFromChunkStream() member.
	friend class Tools_GUIMenu_Widget_ListBox;	// for access to m_xCommand when creating tools dialog.

private:
	typedef Asura_GUIMenu_Widget_Base PARENT;

public:
	enum Asura_GUIMenu_Inherit_Flag_Widget_ListBox_Type
	{
		ASURA_GUIMENU_WIDGET_LISTBOX_LOCAL_FLAG_WIDTH				= (1<<(ASURA_GUIMENU_WIDGET_LISTBOX_LOCAL_FLAG_START+0)),
		ASURA_GUIMENU_WIDGET_LISTBOX_LOCAL_FLAG_HEIGHT				= (1<<(ASURA_GUIMENU_WIDGET_LISTBOX_LOCAL_FLAG_START+1)),
		ASURA_GUIMENU_WIDGET_LISTBOX_LOCAL_FLAG_MAX_ALLOWED_ENTRIES = (1<<(ASURA_GUIMENU_WIDGET_LISTBOX_LOCAL_FLAG_START+2)),
		ASURA_GUIMENU_WIDGET_LISTBOX_LOCAL_FLAG_MAX_DISPLAY_ENTRIES = (1<<(ASURA_GUIMENU_WIDGET_LISTBOX_LOCAL_FLAG_START+3)),
		ASURA_GUIMENU_WIDGET_LISTBOX_LOCAL_FLAG_CONTROL_REPEAT		= (1<<(ASURA_GUIMENU_WIDGET_LISTBOX_LOCAL_FLAG_START+6)),
		ASURA_GUIMENU_WIDGET_LISTBOX_LOCAL_FLAG_FLASH_DETAILS		= (1<<(ASURA_GUIMENU_WIDGET_LISTBOX_LOCAL_FLAG_START+7)),
		ASURA_GUIMENU_WIDGET_LISTBOX_LOCAL_FLAG_OFFSET_VECTOR		= (1<<(ASURA_GUIMENU_WIDGET_LISTBOX_LOCAL_FLAG_START+8)),

		//ASURA_GUIMENU_WIDGET_BUTTON_OVERIDE_MAX is here to stop another programmer adding past bit flag 31
		ASURA_GUIMENU_WIDGET_LISTBOX_INHERIT_FLAG_MAX				= (1<<(ASURA_GUIMENU_WIDGET_LISTBOX_LOCAL_FLAG_START+15))
	};

	Asura_GUIMenu_Widget_ListBox();
	virtual ~Asura_GUIMenu_Widget_ListBox();

	// Virtual overrides	
	virtual void Update( float fElapsedTime );
	virtual void Render( u_int uState );	
	virtual void Copy( const Asura_GUIMenu_Root* pxSource );
	virtual u_int ProcessInputActivity( const Asura_GUIMenu_Input& xInput );
	virtual void SetUnscaledLocalBounds( const Asura_Bounding_Rectangle& xBounds );

	virtual bool GetCropRectForChildren(Asura_Bounding_Rectangle& xBounds) const;

	//Does a recursive search for a selectable widget at the given location
	virtual Asura_GUIMenu_Widget_Base*  FindSelectableWidgetAtPos(const Asura_Vector_2& xPos);

	virtual Asura_GUIMenu_Root* GetResizeChild( const u_int u );
	virtual u_int GetResizeChildCount() const;

	virtual void ReadFromChunkStream( Asura_Chunk_Stream& xStream, Asura_GUIMenu_Root* pxParentObject, u_int uChunkVersion);
#ifdef ASURA_TOOLS
	virtual void UpdateLocalDataFromStyle();
	virtual void WriteToChunkStream( Asura_Chunk_Stream& xStream) const;
	virtual void ExportToXML(Tools_GUIMenu_XMLExporter* pxExporter) const;
	virtual	void TraverseUsedTextures(void (*pfnCallback)(int iTextureID,void* pUserData),void* pUserData);
#endif // ASURA_TOOLS

	virtual bool OnInit( const Asura_Message_GUIMenu_Notify* pxMessage );
	virtual bool OnDeInit( const Asura_Message_GUIMenu_Notify* pxMessage );
	virtual bool OnDataChange( const Asura_Message_GUIMenu_Notify* pxMessage );
	virtual bool OnDataRequest( const Asura_Message_GUIMenu_Notify* pxMessage );
	virtual bool OnClicked( const Asura_Message_GUIMenu_Notify* pxMessage );
	virtual bool OnRollIn( const Asura_Message_GUIMenu_Notify* pxMessage );
	virtual bool OnRollOut( const Asura_Message_GUIMenu_Notify* pxMessage );
	virtual bool OnEnable( const Asura_Message_GUIMenu_Notify* pxMessage );

	virtual void SetName( const Asura_String& xName );
	virtual void SetName( const char* szName );

	void FocusOnSelected();
	Asura_GUIMenu_ConsoleCmd* AddPopulationFunction( Asura_Hash_ID uItem );
	void RemovePopulationFunction( Asura_Hash_ID uItem );
	void RemapPopulationFunction( Asura_Hash_ID uOldID, Asura_Hash_ID uNewID );
	Asura_GUIMenu_ConsoleCmd* AddVisibilityFunction( Asura_Hash_ID uItem );
	void RemoveVisibilityFunction( Asura_Hash_ID uItem );
	void RemapVisibilityFunction( Asura_Hash_ID uOldID, Asura_Hash_ID uNewID );
	const Asura_GUIMenu_Widget_ListBoxEntry& GetEntryTemplate() const { return m_xEntryTemplate; }
	Asura_GUIMenu_Widget_ListBoxEntry& GetEntryTemplate()  { return m_xEntryTemplate; }
	
	// accessors
	u_int* GetSelectedIndexPtr()					{ return &m_uSelectedIndex; }
	u_int GetSelectedIndex() const				{ return m_uSelectedIndex; }	
	u_int GetMaxAllowedEntries() const;
	u_int GetMaxDisplayedEntries() const;
	const Asura_Vector_2& GetOffsetVector() const {return m_xOffsetVector;}
	u_int GetEntryCount() const					{ return m_uEntryCount; }
	float GetEntryOffset() const { return m_fEntryOffset; }
	float GetDisplayOffset() const { return m_fDisplayListOffset * m_fEntryOffset; }
	float GetMaxListOffset() const;
	float GetInitialControlRepeatDelay();
	float GetSecondaryControlRepeatDelay();
	Asura_GUIMenu_ConsoleCmd* GetPopulationFunctionForItem( Asura_Hash_ID uItem );
	Asura_GUIMenu_ConsoleCmd* GetVisibilityFunctionForItem( Asura_Hash_ID uItem );

	bool HasDataChanged() const { return m_bEntryDataChanged || m_bDirtyRect || (m_uEntryCountLastFrame != m_uEntryCount); } 

	void SetSelectedIndex( u_int uSelected );
	void SetMaxAllowedEntries( u_int uMaxEntries );
	void SetMaxDisplayedEntries( u_int uMaxDisplayedEntries );
	void SetOffsetVector( Asura_Vector_2 xOffsetVector );
	void UpdateEntryOffset();
	void SetBounds( const Asura_Bounding_Rectangle& xBounds );
	void SetInitialControlRepeatDelay( float fDelay )	{ m_fInitialRepeatDelay = fDelay; }
	void SetSecondaryControlRepeatDelay( float fDelay )	{ m_fSecondaryRepeatDelay = fDelay;	}	

	const Asura_GUIMenu_Element_Image* GetDecImage() const;
	const Asura_GUIMenu_Element_Image* GetIncImage() const;
	const Asura_GUIMenu_Element_Path*  GetScrollBar() const;

	void						SetDecImage( Asura_GUIMenu_Element_Image* pxDecImage);
	void						SetIncImage( Asura_GUIMenu_Element_Image* pxDecImage);
	void						SetScrollBar( Asura_GUIMenu_Element_Path* pxScrollBar);

	const Asura_RGBA			GetFlashColour() const {return m_xFlashColour; }
	float						GetFlashTime() const { return m_fFlashTimerStartVal; }

	void						SetFlashColour( Asura_RGBA xColour) { m_xFlashColour = xColour; }
	void						SetFlashTime( float fFlashTime) { m_fFlashTimerStartVal = fFlashTime; }

#ifdef ASURA_TOOLS
	Asura_RGBA*					GetFlashColourPointer() {return &m_xFlashColour; }
	float*						GetFlashTimePointer() { return &m_fFlashTimerStartVal; }
#endif // #ifdef ASURA_TOOLS

	void						StartDecrementFlash();
	void						StartIncrementFlash();

	void						SetCarousel( bool bCarousel) { m_bCarousel = bCarousel; }
	bool						GetCarousel() const { return m_bCarousel; }

	virtual bool				RemoveFromThisContainer( Asura_GUIMenu_Root* pxObject);

protected:
	// helper functions
	virtual void SetupDefaultElements() {};

	void ClearAllEntries();
	void AllocateEntryList( u_int uNewListSize );
	void DeallocateEntryList();
	bool HandleInputAutoRepeat( bool bHeldValue );
	void UpdateIcons();
	void UpdateFlashingElements();

	// member data
	float m_fElapsedTime;

	u_int m_uButtonRepeatPhase;
	float m_fCurrentRepeatDelay;
	float m_fInitialRepeatDelay;
	float m_fSecondaryRepeatDelay;

	Asura_GUIMenu_Widget_ListBoxEntry   m_xEntryTemplate;
	Asura_GUIMenu_Widget_ListBoxEntry*	m_pxEntries;
	u_int m_uEntryCount;
	u_int m_uEntryCountLastFrame;

	Asura_GUIMenu_Element_Image*	m_pxDecImage;
	Asura_GUIMenu_Element_Image*	m_pxIncImage;
	Asura_GUIMenu_Element_Path*		m_pxScrollBar;
	bool							m_bDragging;

	Asura_RGBA						m_xFlashColour;
	float							m_fDecFlashTimerCountdown;
	float							m_fIncFlashTimerCountdown;
	float							m_fFlashTimerStartVal;

	u_int m_uSelectedIndex;
	float m_fDisplayListOffset;
	float m_fDisplayTargetOffset;
	float m_fMoveDir;

	u_int m_uMaxAllowedEntries;
	u_int m_uMaxDisplayedEntries;
	float m_fEntryOffset;
	float m_fBoxSizeAgainstOffset;

	bool m_bEntryDataChanged;
	bool m_bDirtyRect;

	union
	{
		u_int m_uListBoxFlags;
		struct
		{
			u_int	m_bLoopSelection	: 1,
					m_bCarousel			: 1,
					m_bAllowSubScroll	: 1,// NY - 03 April 2009 - allows listbox to scroll by partial steps - added to allow scrollable text boxes
					m_uUnsedBits		: 29;
		};
	};

	Asura_Vector_2 m_xOffsetVector;

	Asura_GUIMenu_ConsoleVar		m_xInitValueVar;
	Asura_GUIMenu_ConsoleVar		m_xCurrentValueVar;

	Asura_GUIMenu_CommandManager m_xCommandManager;	// commands executed on accept button presses
	Asura_TemplateHashTree<Asura_GUIMenu_ConsoleCmd> m_xPopulationFunctionTree;
	Asura_TemplateHashTree<Asura_GUIMenu_ConsoleCmd> m_xVisibilityFunctionTree;
};

#endif // ASURA_USE_GUI_MENU_SYSTEM

/***************/
/* End of File */
/***************/

#endif//__ASURA_GUIMENU_WIDGET_LISTBOX_H__
