// ------------------------------------------------------------------------------------ //
//																						//
//	File: Asura_GUIMenu_Widget_ProgressBar.h											//
//																						//
//	Description: progressbar widget class												//
//																						//
//	Created: Steve Keen - 25/01/07														//
//																						//
//	Copyright (C) 2006 Rebellion (Derby), All Rights Reserved.							//
// ------------------------------------------------------------------------------------ //

#ifndef __ASURA_GUIMENU_WIDGET_PROGRESSBAR_H__
#define __ASURA_GUIMENU_WIDGET_PROGRESSBAR_H__

#include "Asura_GUIMenu_Element_Base.h"

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifdef ASURA_USE_GUI_MENU_SYSTEM
#include "Asura_GUIMenu_Widget_Base.h"

// -- Typedef -------------------------------------------------------------------------

class Asura_GUIMenu_Element_Image;

// -- Enums ---------------------------------------------------------------------------

enum Asura_GUIMenu_ProgressBar_Axis_Movement
{
	ASURA_GUIMENU_PROGRESSBAR_AXIS_NONE,
	ASURA_GUIMENU_PROGRESSBAR_AXIS_NORTH,
	ASURA_GUIMENU_PROGRESSBAR_AXIS_EAST,
	ASURA_GUIMENU_PROGRESSBAR_AXIS_SOUTH,
	ASURA_GUIMENU_PROGRESSBAR_AXIS_WEST
};

enum Asura_GUIMenu_ProgressBar_Message
{
	ASURA_GUIMENU_PROGRESSBAR_NONE = 0,
	ASURA_GUIMENU_PROGRESSBAR_SETPOSITION,		// data passed is the position value
	ASURA_GUIMENU_PROGRESSBAR_GETPOSITION,		// data returned is the position value
	ASURA_GUIMENU_PROGRESSBAR_ADD_STEP,			// this notifies the progress bar to increment its position by 1 step unit
	ASURA_GUIMENU_PROGRESSBAR_SUBTRACT_STEP,	// this notifies the progress bar to decrement its position by 1 step unit
	ASURA_GUIMENU_PROGRESSBAR_END_OF_MESSAGES
};

// -- Defines -------------------------------------------------------------------------
#define ASURA_GUIMENU_MAX_PROGRESSBAR_IMAGE_OVERLAYS	(4)

#define ASURA_GUIMENU_PROGRESSBAR_FLAGS_CLIP_OVERLAY	(1<<0)

#define ASURA_GUIMENU_WIDGET_PROGRESSBAR_LOCAL_FLAG_START	16

// -- Data Chunk Version --
// Versions: ASURA_CHUNK_ELEMENT_IMAGE_VERSION
// 0:	Initial data
// 1:	Added console var position control
#define ASURA_CHUNK_WIDGET_PROGRESSBAR_VERSION 1


// ------------------------------------------------------------------------
class Asura_GUIMenu_ProgressBar_Overlay_Info
{
public:

	void Reset();
	void CopyFromElementImage( const Asura_GUIMenu_Element_Image* const pxImage );

	Asura_Hash_ID							m_uHashImageID;
	Asura_GUIMenu_ProgressBar_Axis_Movement	m_eAxisMovement;
	Asura_Bounding_Rectangle				m_xCurrentRect;

	Asura_GUIMenu_UV_Set					m_xOrgUV;
	Asura_GUIMenu_UV_Set					m_xCurrentUV;
	
	u_int									m_uOverlayFlags;
};

// -- Class ---------------------------------------------------------------------------
enum Asura_GUIMenu_Root::Asura_GUIMenu_Object_Type;
class Asura_GUIMenu_Widget_Base;

class Asura_GUIMenu_Widget_ProgressBar : public Asura_GUIMenu_Widget_Base
{
	// -- Functions -----------------------------------------------------------------------
public:
	enum Asura_GUIMenu_Inherit_Flag_Widget_ProgressBar_Type
	{
		ASURA_GUIMENU_WIDGET_PROGRESSBAR_LOCAL_FLAG_MINIMUM	  = (1<<(ASURA_GUIMENU_WIDGET_PROGRESSBAR_LOCAL_FLAG_START+0)),
		ASURA_GUIMENU_WIDGET_PROGRESSBAR_LOCAL_FLAG_MAXIMUM	  = (1<<(ASURA_GUIMENU_WIDGET_PROGRESSBAR_LOCAL_FLAG_START+1)),
		ASURA_GUIMENU_WIDGET_PROGRESSBAR_LOCAL_FLAG_STEP		  = (1<<(ASURA_GUIMENU_WIDGET_PROGRESSBAR_LOCAL_FLAG_START+2)),
		ASURA_GUIMENU_WIDGET_PROGRESSBAR_LOCAL_FLAG_OVERLAYS	  = (1<<(ASURA_GUIMENU_WIDGET_PROGRESSBAR_LOCAL_FLAG_START+3)),
		//ASURA_GUIMENU_WIDGET_PROGRESSBAR_OVERIDE_MAX is here to stop another programmer adding past bit flag 31
		ASURA_GUIMENU_WIDGET_PROGRESSBAR_LOCAL_FLAG_MAX  		  = (1<<(ASURA_GUIMENU_WIDGET_PROGRESSBAR_LOCAL_FLAG_START+15))
	};

	Asura_GUIMenu_Widget_ProgressBar();
	virtual ~Asura_GUIMenu_Widget_ProgressBar();

	virtual void 				Update( float fElapsedTime);
	virtual void 				Render( u_int uState);
	virtual void				Copy( const Asura_GUIMenu_Root* pxSource);
	
	void						CopyOverlayInfo( const Asura_GUIMenu_Widget_ProgressBar* pxSourceProgressBar);
	
	virtual void				Resize( const Asura_Bounding_Rectangle& xBounds );

	// -- UI Message handler functions --
	virtual bool				OnInit( const Asura_Message_GUIMenu_Notify* pxMessage);
	virtual bool				OnDeInit( const Asura_Message_GUIMenu_Notify* pxMessage);
	virtual bool				OnDataChange( const Asura_Message_GUIMenu_Notify* pxMessage);
	virtual bool				OnDataRequest( const Asura_Message_GUIMenu_Notify* pxMessage);
	virtual bool				OnClicked( const Asura_Message_GUIMenu_Notify* pxMessage);
	virtual bool				OnRollIn( const Asura_Message_GUIMenu_Notify* pxMessage);
	virtual bool				OnRollOut( const Asura_Message_GUIMenu_Notify* pxMessage);
	virtual bool				OnEnable( const Asura_Message_GUIMenu_Notify* pxMessage);

	void				SetActiveOverlayImageData( const u_int uIndex,
												   const Asura_Hash_ID uHashImageID,
												   const Asura_GUIMenu_ProgressBar_Axis_Movement eAxisMovement,
												   const u_int uOverlayFlags = 0);	

	// calls UpdateFromElementImage() for each object in m_axImageOverlayToChange - copys relevent parameters from GUI Image elements - NDB
	void				UpdateActiveOverlayImageData();


	Asura_GUIMenu_ProgressBar_Overlay_Info	GetActiveOverlayImageData( u_int uIndex) const;
	Asura_GUIMenu_ProgressBar_Overlay_Info* GetActiveOverlayImageDataPtr( u_int uIndex);

	void						SetMaximumValue( float fMax);
	void						SetMinimumValue( float fMin);
	void						SetMinAndMaxValue(float fMin,float fMax);
	void						SetStepValue( float fStep);

	float						GetMaximumValue() const; 	//{ return m_fMaxValue;	}
	float						GetMinimumValue() const;	//{ return m_fMinValue;	}
	float						GetStepValue() const;		//{	return m_fStepValue;	}

	void						AddStep();
	void						SubstractStep();

#ifdef ASURA_TOOLS
	virtual void				UpdateLocalDataFromStyle();

	virtual void				WriteToChunkStream( Asura_Chunk_Stream& xStream) const;
	virtual void				ExportToXML(Tools_GUIMenu_XMLExporter* pxExporter) const;

	Asura_GUIMenu_ConsoleVar&		GetConsoleVar() { return m_xConsoleVar; }

	void SetUpdatePositionFromConsoleVar( const bool bUpdateFromVar ) { m_bUpdatePositionFromConsoleVar = bUpdateFromVar; }	
#endif // ASURA_TOOLS

	const Asura_GUIMenu_ConsoleVar&	GetConsoleVar() const { return m_xConsoleVar; }

	virtual void				ReadFromChunkStream( Asura_Chunk_Stream& xStream, Asura_GUIMenu_Root* pxParentObject, u_int uChunkVersion);
	
#ifdef ASURA_TOOLS
	// applies search to local console vars after calling parent method
	virtual const bool			FindAndReplaceConsoleVarName( const Asura_UnicodeChar* wszName,const Asura_UnicodeChar* wszReplacementName );
#endif // ASURA_TOOLS

	virtual void ApplyScalingFlagsToChildren();
	virtual void ApplyAnchoringFlagsToChildren();

protected:
	void SetupDefaultElements() {};	
	float						GetBarPos() const { return m_fPosition; }
	void						UpdateBarPos( float fNewPos);
	float						ConvertToRange( float fNewPos);
	float						ConvertFromRange( float fNewPos);

	Asura_GUIMenu_Root*			SearchForOverlayContainerItem( Asura_Hash_ID uSearchID) const;

private:
	void						UpdateProgressBarElementInfo(Asura_GUIMenu_ProgressBar_Overlay_Info* pxOverlay, const Asura_GUIMenu_Element_Image* pxImage);
	void						UpdatePositionFromConsoleVar();
	void						RenderImageOverlay( u_int uOverlayIndex, u_int uContainerIndex);
	int							IsImageOverlay( Asura_Hash_ID uHashID);

	// -- Variables -----------------------------------------------------------------------

	float							m_fElapsedTime;
	float							m_fPosition;
	float							m_fOldPosition;

	float							m_fMinValue;
	float							m_fMaxValue;
	float							m_fStepValue;

	Asura_GUIMenu_ProgressBar_Overlay_Info	m_axImageOverlayToChange[ASURA_GUIMENU_MAX_PROGRESSBAR_IMAGE_OVERLAYS];

	Asura_GUIMenu_ConsoleVar m_xConsoleVar;
	bool m_bUpdatePositionFromConsoleVar;

	typedef Asura_GUIMenu_Widget_Base PARENT;
};
#endif // ASURA_USE_GUI_MENU_SYSTEM
#endif // __ASURA_GUIMENU_WIDGET_PROGRESSBAR_H__
