// ------------------------------------------------------------------------------------ //
//																						//
//	File: Asura_GUIMenu_Widget_Slider.cpp												//
//																						//
//	Description: Slider widgets class													//
//																						//
//	Created: Steve Keen - 9/11/06														//
//																						//
//	Copyright (C) 2006 Rebellion (Derby), All Rights Reserved.							//
// ------------------------------------------------------------------------------------ //

#include "Core/Asura.h"
#include "Asura_GUIMenu_Widget_Slider.h"

#ifdef ASURA_USE_GUI_MENU_SYSTEM
#include "Camera/Asura_Camera.h"
#include "Core/Asura_Hash.h"
#include "Core/Asura_String.h"
#include "Core/Asura_Graph.h"
#include "GUIMenu/Asura_Profiling_GUIMenu.h"
#include "Interface/Asura_CommandConsole.h"
#include "Asura_GUIMenu_Audio.h"
#include "Asura_GUIMenu_Element_Base.h"
#include "Asura_GUIMenu_Element_Path.h"

#include "Asura_GUIMenu_Message.h"
#include "Asura_GUIMenu_MessagePump.h"

#include "Sound/Asura_Sound_Event.h"
#include "Sound/Asura_Sound_Event_System.h"
#include "Sound/Asura_Sound_Event_Storage.h"

#include "Asura_GUIMenu_System.h"

#ifdef ASURA_TOOLS
#include "GUIMenu/Tools_GUIMenu_XMLExporter.h"
#endif

// --------------------------------------------------------------------------------------- 
// -- Constructors -----------------------------------------------------------------------

Asura_GUIMenu_Widget_Slider::Asura_GUIMenu_Widget_Slider(void)
: m_pxPath( NULL )
, m_fPosition( 0.0f )
, m_fOldPosition( 0.0f )
, m_fTimeElapsed( 0.0f )
, m_uNumSteps( 0 )
, m_uEditingAudioHash( ASURA_HASH_ID_UNSET )
, m_uSliderFlags( 0 )
, m_hAudio( ASURA_INVALID_HANDLE )
, m_fConsoleVarLowerLimit( 0.0f )
, m_fConsoleVarUpperLimit( 1.0f )
, m_bCursorDrag( false )
, m_uButtonRepeatPhase( 0 )
, m_fCurrentRepeatDelay( 0 )
, m_fInitialRepeatDelay( fASURA_GUIMENU_WIDGET_SLIDER_CONTROL_INITIAL_REPEAT_DELAY )
, m_fSecondaryRepeatDelay( fASURA_GUIMENU_WIDGET_SLIDER_CONTROL_SECONDARY_REPEAT_DELAY )
{
	m_uType = ASURA_GUIMENU_TYPEID_WIDGET_SLIDER;
	
	ASURA_GUIMENU_PROFILE_COUNTINSTANCE(ASURA_GUIMENU_COUNTER__WIDGET_SLIDER);
}

// --------------------------------------------------------------------------------------- 
// -- Destructor -------------------------------------------------------------------------

Asura_GUIMenu_Widget_Slider::~Asura_GUIMenu_Widget_Slider(void)
{
	ASURA_GUIMENU_PROFILE_COUNTDESTROYINSTANCE(ASURA_GUIMENU_COUNTER__WIDGET_SLIDER);
}

// --------------------------------------------------------------------------------------- 
// -- Public Functions ------------------------------------------------------------------- 

// --------------------------------------------------------------------------------------- 

//void Asura_GUIMenu_Widget_Slider::Render( u_int uState)
//{
//	PARENT::Render( uState);
//}

// --------------------------------------------------------------------------------------- 

void Asura_GUIMenu_Widget_Slider::Update( float fElapsedTime)	//, unsigned int uButtons, POINT ptMousePos)
{
#ifdef ASURA_TOOLS
	// In tools continue to always read the console var as this maybe changed in the
	// preview mode. Make sure the slider renders corectly when the value of the var is changed.
	InitFromConsoleVar();
#endif

	Asura_GUIMenu_Root::Update( fElapsedTime);

	m_fTimeElapsed = fElapsedTime;

	switch( GetCurrentState())
	{
		case ASURA_GUIMENU_WIDGET_STATE_DEFAULT:
			break;

		case ASURA_GUIMENU_WIDGET_STATE_ROLL_ON:
			if( (IsSliderFlagSet( ASURA_GUIMENU_WIDGET_SLIDER_FLAGS_ALWAYS_IN_EDIT_MODE_WHEN_HIGHLIGHTED)) &&
				(m_bCursorDrag == false))
			{
				SetCurrentState( ASURA_GUIMENU_WIDGET_STATE_EDITING);
			}
			else
			{
				SetCurrentState( ASURA_GUIMENU_WIDGET_STATE_HILIGHTED);
			}
			break;

		case ASURA_GUIMENU_WIDGET_STATE_HILIGHTED:
			break;

		case ASURA_GUIMENU_WIDGET_STATE_CLICKED:
			SetCurrentState( ASURA_GUIMENU_WIDGET_STATE_HILIGHTED);
			break;

		case ASURA_GUIMENU_WIDGET_STATE_ROLL_OFF:
			SetCurrentState( ASURA_GUIMENU_WIDGET_STATE_DEFAULT);
			break;

		case ASURA_GUIMENU_WIDGET_STATE_EDITING:
			if( (m_hAudio != ASURA_INVALID_HANDLE) &&
				(m_uSliderFlags&ASURA_GUIMENU_WIDGET_SLIDER_FLAGS_CHANGE_VOLUME_ON_LOCAL_AUDIO_EVENT))
			{
				if( (Asura_Sound_Event_System::IsLooping( m_hAudio) == false) &&
					(m_uSliderFlags&ASURA_GUIMENU_WIDGET_SLIDER_FLAGS_LOOP_LOCAL_AUDIO_EVENT))
				{
					// sample doesn't loop but the local flag requires it
					if( Asura_Sound_Event_System::IsPlaying( m_hAudio))
					{
						Asura_Sound_Event_System::SetVolumeScale( m_hAudio, m_fPosition);
					}
					else
					{
//						m_hAudio = ASURA_INVALID_HANDLE;
						StopEditingAudio();
						PlayEditingAudio();
					}
				}
				
			}
			break;

		case ASURA_GUIMENU_WIDGET_STATE_DISABLED:
		default:
			break;
	}

	const u_int uContainerCount = GetContainerCount();
	for( u_int u=0; u<uContainerCount; u++)
	{
		m_ppxContainerParts[u]->Update( fElapsedTime);
	}

	if( m_pxPath != NULL)
	{
		m_pxPath->SetCurrentPathValue( m_fPosition);

		if( m_fOldPosition != m_fPosition)
		{
			if( GetFlag( ASURA_GUIMENU_ACTION_FLAG_NOTIFY_DATA_CHANGE_IMMEDIATE) == true)
			{
				Asura_GUIMenu_MessagePump::PostMessage( ASURA_GUIMENU_EXTERNAL_NOTIFY, GetID(), ASURA_GUIMENU_UI_MSG_ON_DATA_CHANGE, ASURA_GUIMENU_TYPEID_WIDGET_SLIDER, (u_int)Asura_Maths::FloatToInt( m_fPosition * 100.0f));		
			}
		}

		m_fOldPosition = m_fPosition;
	}
}

// --------------------------------------------------------------------------------------- 

void Asura_GUIMenu_Widget_Slider::Copy( const Asura_GUIMenu_Root* pxSource)
{
	PARENT::Copy( pxSource);

	const Asura_GUIMenu_Widget_Slider* pxSourceSlider = static_cast<const Asura_GUIMenu_Widget_Slider*> (pxSource);

	SetEditingAudioHash( pxSourceSlider->GetEditingAudioHash());
	SetSliderFlags( pxSourceSlider->GetSliderFlags() );
	SetNumSteps( pxSourceSlider->GetNumSteps() );

	m_xConsoleVar.Copy(&pxSourceSlider->m_xConsoleVar);
}

// --------------------------------------------------------------------------------------- 

u_int Asura_GUIMenu_Widget_Slider::ProcessInputActivity( const Asura_GUIMenu_Input& xInput)
{
	u_int uButtonsTrapped = 0;
	const u_int uButtonPresses = xInput.m_uButtonPresses;
	float fStep = 0.0f;
	bool bStep = false;

	if ( m_uNumSteps < 2 )
	{
		fStep = m_fTimeElapsed * 1.0f;
		bStep = false;
	}
	else
	{
		fStep = 1.0f / m_uNumSteps;
		bStep = true;
	}

//	Asura_Hash_ID uHashExitID = ASURA_HASH_ID_UNSET;
	Asura_GUIMenu_Widget_State eCurrentWidgetState = GetCurrentState();

	switch( eCurrentWidgetState)
	{
		case ASURA_GUIMENU_WIDGET_STATE_EDITING:
		{
			bool bStopEdit = false;

			//If always in edit mode then we won't pick up the initial click from the non-edit state , so check here instead
			//RAR 17/08/2007
			if(xInput.m_bUseCursor && (uButtonPresses & ASURA_GUIMENU_ACTION_BUTTON_CLICK) && IsSliderFlagSet( ASURA_GUIMENU_WIDGET_SLIDER_FLAGS_ALWAYS_IN_EDIT_MODE_WHEN_HIGHLIGHTED))
			{
				//start editing if the cursor was clicked within the path element
				if(m_pxPath && m_pxPath->ContainsPos(xInput.m_xCursorPos))
				{
					uButtonsTrapped |= ASURA_GUIMENU_ACTION_BUTTON_CLICK;
					m_bCursorDrag = true;
				}

			}

			if(m_bCursorDrag)
			{
				if(xInput.m_bUseCursor && (uButtonPresses & ASURA_GUIMENU_ACTION_BUTTON_CLICK_HELD) && m_pxPath)
				{
					//move slider to point nearest the cursor
					SetSliderFromCursorPos(xInput.m_xCursorPos);
				}
				else
				{
					if(IsSliderFlagSet( ASURA_GUIMENU_WIDGET_SLIDER_FLAGS_ALWAYS_IN_EDIT_MODE_WHEN_HIGHLIGHTED))
					{
						m_bCursorDrag = false;
					}
					else
					{
						bStopEdit = true;	
					}
				}
			}
			else
			{
				bool bLeftPressed = ((uButtonPresses&ASURA_GUIMENU_ACTION_BUTTON_LEFT_HELD) != 0);
				bool bRightPressed = ((uButtonPresses&ASURA_GUIMENU_ACTION_BUTTON_RIGHT_HELD) != 0);

				if( bLeftPressed)
				{
					uButtonsTrapped |= ASURA_GUIMENU_ACTION_BUTTON_LEFT_HELD;
				}
				if( bRightPressed )
				{
					uButtonsTrapped |= ASURA_GUIMENU_ACTION_BUTTON_RIGHT_HELD;
				}

				bool bPressed = bLeftPressed | bRightPressed;

				if( HandleInputAutoRepeat( bPressed) == false )
				{
					bLeftPressed = false;
					bRightPressed = false;
				}

				if( bLeftPressed)
				{
					m_fPosition -= fStep;
					if( m_fPosition < 0.0f)
					{
						m_fPosition = 0.0f;	
						bStep = false;
					}

					SetConsoleVar();

					if( bStep)
					{
						//if( ((m_uSliderFlags&ASURA_GUIMENU_WIDGET_SLIDER_FLAGS_LOOP_LOCAL_AUDIO_EVENT) == 0) &&
						//	(uButtonPresses&ASURA_GUIMENU_ACTION_BUTTON_LEFT))
						if( (m_uSliderFlags&ASURA_GUIMENU_WIDGET_SLIDER_FLAGS_LOOP_LOCAL_AUDIO_EVENT) == 0)
						{
							PlayEditingAudio();
						}
					}
				}
				else if( bRightPressed)
				{
					m_fPosition += fStep;
					if( m_fPosition > 1.0f)
					{
						m_fPosition = 1.0f;	
						bStep = false;
					}
			
					SetConsoleVar();

					if( bStep)
					{
						if( bStep)
						{
							//if( ((m_uSliderFlags&ASURA_GUIMENU_WIDGET_SLIDER_FLAGS_LOOP_LOCAL_AUDIO_EVENT) == 0) &&
							//	(uButtonPresses&ASURA_GUIMENU_ACTION_BUTTON_RIGHT))
							if( (m_uSliderFlags&ASURA_GUIMENU_WIDGET_SLIDER_FLAGS_LOOP_LOCAL_AUDIO_EVENT) == 0)
							{
								PlayEditingAudio();
							}
						}
					}

				}
				else if( ((uButtonPresses&ASURA_GUIMENU_ACTION_BUTTON_ACCEPT) != 0) ||
					((uButtonPresses&ASURA_GUIMENU_ACTION_BUTTON_CANCEL) != 0))
				{
					if( IsSliderFlagSet( ASURA_GUIMENU_WIDGET_SLIDER_FLAGS_ALWAYS_IN_EDIT_MODE_WHEN_HIGHLIGHTED ) == false)
					{
						bStopEdit = true;

						if( (uButtonPresses&ASURA_GUIMENU_ACTION_BUTTON_ACCEPT) != 0)
						{
							uButtonsTrapped |= ASURA_GUIMENU_ACTION_BUTTON_ACCEPT;
							// -- play the accept sound, we're leaving edit mode --
							Asura_GUIMenu_Audio::Play( ASURA_GUIMENU_AUDIO_TYPE_ON_ACCEPT);
						}
						else
						{
							uButtonsTrapped |= ASURA_GUIMENU_ACTION_BUTTON_CANCEL;
							// -- play the cancel sound, we're leaving edit mode --
							Asura_GUIMenu_Audio::Play( ASURA_GUIMENU_AUDIO_TYPE_ON_CANCEL);
						}
					}
				}

				if( IsSliderFlagSet( ASURA_GUIMENU_WIDGET_SLIDER_FLAGS_ALWAYS_IN_EDIT_MODE_WHEN_HIGHLIGHTED ) == true)
				{
					bool bNavigateChange = false;
					Asura_Hash_ID uHashExitID = ASURA_HASH_ID_UNSET;

					if( (uButtonPresses&ASURA_GUIMENU_ACTION_BUTTON_UP) != 0)
					{
						uHashExitID = GetNavExitID( ASURA_GUIMENU_UI_NAV_EXIT_UP);
						uButtonsTrapped |= ASURA_GUIMENU_ACTION_BUTTON_UP;
						bNavigateChange = true;
					}
					else if( (uButtonPresses&ASURA_GUIMENU_ACTION_BUTTON_DOWN) != 0)
					{
						uHashExitID = GetNavExitID( ASURA_GUIMENU_UI_NAV_EXIT_DOWN);		
						uButtonsTrapped |= ASURA_GUIMENU_ACTION_BUTTON_DOWN;
						bNavigateChange = true;
					}

					// -- we have an exit hash
					if( uHashExitID != ASURA_HASH_ID_UNSET)
					{
						// -- notify the next item to be selected
						Asura_GUIMenu_MessagePump::PostMessage( uHashExitID, GetID(), ASURA_GUIMENU_UI_MSG_ON_ROLL_IN, 0);

						// -- notify the current item, to be deselected --
						Asura_GUIMenu_MessagePump::PostMessage( GetID(), GetID(), ASURA_GUIMENU_UI_MSG_ON_ROLL_OUT, 0);
					}

					if( bNavigateChange)
					{
						if( uHashExitID != ASURA_HASH_ID_UNSET)
						{
							// -- change to a different widget --
							Asura_GUIMenu_Audio::Play( ASURA_GUIMENU_AUDIO_TYPE_ON_NAVIGATE);
						}
						else
						{
							// -- notifies the player that they can't go in this direction.
							Asura_GUIMenu_Audio::Play( ASURA_GUIMENU_AUDIO_TYPE_FAILURE);
						}
					}
				}
			}

			if(bStopEdit)
			{
				SetCurrentState( ASURA_GUIMENU_WIDGET_STATE_HILIGHTED);
				SetConsoleVar();

				StopEditingAudio();
				
				m_bCursorDrag = false;
			}
		}
		break;

	default:

		uButtonsTrapped |= ProcessNavigationInput( uButtonPresses);

		bool bStartEdit = false;
		if( (uButtonPresses&ASURA_GUIMENU_ACTION_BUTTON_ACCEPT) != 0)
		{
			if( eCurrentWidgetState != ASURA_GUIMENU_WIDGET_STATE_DISABLED)
			{
				uButtonsTrapped |= ASURA_GUIMENU_ACTION_BUTTON_ACCEPT;
				bStartEdit = true;
				m_bCursorDrag = false;
			}
		}
		else if(xInput.m_bUseCursor && (uButtonPresses & ASURA_GUIMENU_ACTION_BUTTON_CLICK))
		{
			//start editing if the cursor was clicked within the path element
			if(m_pxPath && m_pxPath->ContainsPos(xInput.m_xCursorPos))
			{
				uButtonsTrapped |= ASURA_GUIMENU_ACTION_BUTTON_CLICK;
				bStartEdit = true;	
				m_bCursorDrag = true;
				SetSliderFromCursorPos(xInput.m_xCursorPos);
			}

		}
		if(bStartEdit)
		{
			SetCurrentState( ASURA_GUIMENU_WIDGET_STATE_EDITING);

			// -- play the accept sound, we're going into edit mode --
			Asura_GUIMenu_Audio::Play( ASURA_GUIMENU_AUDIO_TYPE_ON_ACCEPT);

			if( m_uSliderFlags&ASURA_GUIMENU_WIDGET_SLIDER_FLAGS_LOOP_LOCAL_AUDIO_EVENT)
			{
				PlayEditingAudio();
			}
		}
		break;
	}

	return uButtonsTrapped;
}

// --------------------------------------------------------------------------------------- 

bool Asura_GUIMenu_Widget_Slider::HandleInputAutoRepeat( bool bHeldValue )
{
	bool bRetVal = bHeldValue;

	if( bHeldValue == true )
	{
		switch( m_uButtonRepeatPhase )
		{
		case 1:
			m_fCurrentRepeatDelay -= m_fTimeElapsed;
			if( m_fCurrentRepeatDelay < 0.0f )
			{
				m_fCurrentRepeatDelay += m_fSecondaryRepeatDelay;
				m_uButtonRepeatPhase++;
			}
			else
			{
				bRetVal = false;
			}
			break;

		case 2:
			m_fCurrentRepeatDelay -= m_fTimeElapsed;
			if( m_fCurrentRepeatDelay < 0.0f )
			{
				m_fCurrentRepeatDelay += m_fSecondaryRepeatDelay;
			}
			else
			{
				bRetVal = false;		
			}
			break;

		default:
			// -- setup case 1: details
			m_uButtonRepeatPhase++;
			m_fCurrentRepeatDelay = m_fInitialRepeatDelay;	
			break;
		}
	}
	else
	{
		m_uButtonRepeatPhase = 0;
		m_fCurrentRepeatDelay = 0.0f;
	}

	return bRetVal;
}

// --------------------------------------------------------------------------------------- 

void Asura_GUIMenu_Widget_Slider::PlayEditingAudio( void)
{
	if( m_uEditingAudioHash != ASURA_HASH_ID_UNSET)
	{
		const Asura_Sound_Event* pxEvent = Asura_Sound_Event_Storage::FindEvent( m_uEditingAudioHash);
		if( pxEvent)
		{
			Asura_Vector_3 xPos = *Asura_Camera::GetPosition();

			if( m_uSliderFlags&ASURA_GUIMENU_WIDGET_SLIDER_FLAGS_LOOP_LOCAL_AUDIO_EVENT)
			{
				if( m_hAudio == ASURA_INVALID_HANDLE)
				{
					Asura_Sound_Event_System::PlayEvent( pxEvent, &m_hAudio, &xPos);
					if( m_uSliderFlags&ASURA_GUIMENU_WIDGET_SLIDER_FLAGS_CHANGE_VOLUME_ON_LOCAL_AUDIO_EVENT)
					{
						Asura_Sound_Event_System::SetVolumeScale( m_hAudio, m_fPosition);
					}
				}
			}
			else
			{
				Asura_Handle hTemp = ASURA_INVALID_HANDLE;

				Asura_Sound_Event_System::PlayEvent( pxEvent, &hTemp, &xPos);

				// ARC - 22 August 2007 11:45:06 - Prevent trying to set the volume on an invalid handle
				if( m_uSliderFlags&ASURA_GUIMENU_WIDGET_SLIDER_FLAGS_CHANGE_VOLUME_ON_LOCAL_AUDIO_EVENT && hTemp != ASURA_INVALID_HANDLE)
				{
					Asura_Sound_Event_System::SetVolumeScale( hTemp, m_fPosition);
				}

				if (hTemp != ASURA_INVALID_HANDLE)
				{
					Asura_Sound_Event_System::FinishedWithHandle( hTemp);
				}
			}
		}
	}
}

// --------------------------------------------------------------------------------------- 

void Asura_GUIMenu_Widget_Slider::StopEditingAudio( void)
{
	if( m_hAudio != ASURA_INVALID_HANDLE)
	{
		Asura_Sound_Event_System::Stop( m_hAudio);
		m_hAudio = ASURA_INVALID_HANDLE;
	}
}

// --------------------------------------------------------------------------------------- 
// -- UI Message handler functions -------------------------------------------------------

bool Asura_GUIMenu_Widget_Slider::OnInit( const Asura_Message_GUIMenu_Notify* pxMessage)
{
	PARENT::OnInit( pxMessage);

	// set internal data from value of console var
	InitFromConsoleVar();

	Asura_GUIMenu_MessagePump::PostMessage( ASURA_GUIMENU_EXTERNAL_REQUEST, GetID(), ASURA_GUIMENU_UI_MSG_ON_DATA_REQUEST, ASURA_GUIMENU_TYPEID_WIDGET_SLIDER);		
	return true;
}

// --------------------------------------------------------------------------------------- 

bool Asura_GUIMenu_Widget_Slider::OnDeInit( const Asura_Message_GUIMenu_Notify* pxMessage)
{
	if( GetFlag( ASURA_GUIMENU_ACTION_FLAG_NOTIFY_DATA_CHANGE_ON_EXIT) == true)
	{
		Asura_GUIMenu_MessagePump::PostMessage( ASURA_GUIMENU_EXTERNAL_NOTIFY, GetID(), ASURA_GUIMENU_UI_MSG_ON_DATA_CHANGE, ASURA_GUIMENU_TYPEID_WIDGET_SLIDER, (u_int)Asura_Maths::FloatToInt( m_fPosition * 100.0f));
		SetConsoleVar();
	}

	return true;
}

// --------------------------------------------------------------------------------------- 

bool Asura_GUIMenu_Widget_Slider::OnDataChange( const Asura_Message_GUIMenu_Notify* pxMessage)
{
	bool bHandled = false;

	if( pxMessage->uFromID != GetID())
	{
		// -- something external has changed the data --
		if( pxMessage->WidgetType == ASURA_GUIMENU_TYPEID_WIDGET_SLIDER)
		{
			m_fPosition = (float)pxMessage->uData/100.0f;
			SetConsoleVar();
		}

		bHandled = true;
	}
	return bHandled;
}

// --------------------------------------------------------------------------------------- 

bool Asura_GUIMenu_Widget_Slider::OnDataRequest( const Asura_Message_GUIMenu_Notify* pxMessage)
{
	bool bHandled = false;

	if( pxMessage->uFromID != GetID())
	{
		// -- something external has requested the data --
		Asura_GUIMenu_MessagePump::PostMessage( pxMessage->uFromID, GetID(), ASURA_GUIMENU_UI_MSG_ON_DATA_CHANGE, ASURA_GUIMENU_TYPEID_WIDGET_SLIDER, (u_int)Asura_Maths::FloatToInt( m_fPosition * 100.0f));		
		bHandled = true;
	}
	return bHandled;

}

// --------------------------------------------------------------------------------------- 

bool Asura_GUIMenu_Widget_Slider::OnClicked( const Asura_Message_GUIMenu_Notify* pxMessage)
{
	return true;
}

// --------------------------------------------------------------------------------------- 

bool Asura_GUIMenu_Widget_Slider::OnRollIn( const Asura_Message_GUIMenu_Notify* pxMessage)
{
	SetCurrentState( ASURA_GUIMENU_WIDGET_STATE_ROLL_ON);
	return true;
}

// --------------------------------------------------------------------------------------- 

bool Asura_GUIMenu_Widget_Slider::OnRollOut( const Asura_Message_GUIMenu_Notify* pxMessage)
{
	SetCurrentState( ASURA_GUIMENU_WIDGET_STATE_ROLL_OFF);
	return true;
}

// --------------------------------------------------------------------------------------- 

bool Asura_GUIMenu_Widget_Slider::OnEnable( const Asura_Message_GUIMenu_Notify* pxMessage)
{
	Asura_GUIMenu_Widget_State eNewState;

	if( pxMessage->uData != 0)
	{	
		if( IsFocusedObject())
		{	eNewState = ASURA_GUIMENU_WIDGET_STATE_HILIGHTED;	}
		else
		{	eNewState = ASURA_GUIMENU_WIDGET_STATE_DEFAULT;	}
	}
	else
	{
		eNewState = ASURA_GUIMENU_WIDGET_STATE_DISABLED;	
	}

	SetCurrentState( eNewState);

	return true;
}

// --------------------------------------------------------------------------------------- 

void Asura_GUIMenu_Widget_Slider::SetActivePathFromHash( Asura_Hash_ID uHashID)
{
	const u_int uContainerCount = GetContainerCount();
	for( u_int u=0; u<uContainerCount; u++)
	{
		if( (m_ppxContainerParts[u]->GetID() == uHashID) &&
			(m_ppxContainerParts[u]->GetType() == Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_ELEMENT_PATH))
		{
			m_pxPath = static_cast<Asura_GUIMenu_Element_Path*>(m_ppxContainerParts[u]);
			break;
		}
	}
}

// --------------------------------------------------------------------------------------- 

Asura_Hash_ID Asura_GUIMenu_Widget_Slider::GetActivePathHash( void) const
{
	Asura_Hash_ID uHashID = ASURA_HASH_ID_UNSET;

	if( m_pxPath != NULL)
	{	uHashID = m_pxPath->GetID();	}

	return uHashID;
}

// --------------------------------------------------------------------------------------- 

void Asura_GUIMenu_Widget_Slider::ReadFromChunkStream( Asura_Chunk_Stream& xStream, Asura_GUIMenu_Root* pxParentObject, u_int uChunkVersion)
{
	u_int uHashActivePath;

	PARENT::ReadFromChunkStream( xStream, pxParentObject, 0xffffffff);

	ReadAllChildObjects( xStream, this);

	xStream >> uHashActivePath;
	SetActivePathFromHash( uHashActivePath);

	switch ( uChunkVersion )
	{
		case 0: 
		{
			// nothing
		}
		break;

		case 1:
		{
			float fStep;
			xStream >> fStep;
			m_uNumSteps = fStep == 0.0f ? 0 : Asura_Maths::FloatToInt_Near( 1.0f / fStep );
			xStream >> m_uEditingAudioHash;
			xStream >> m_uSliderFlags;
		}
		break;
		
		case 2:
		{
			float fStep;
			xStream >> fStep;
			m_uNumSteps = fStep == 0.0f ? 0 : Asura_Maths::FloatToInt_Near( 1.0f / fStep );
			xStream >> m_uEditingAudioHash;
			xStream >> m_uSliderFlags;
			// console var
			m_xConsoleVar.ReadFromChunkStream( xStream);
		}
		break;

		case 3:
		{
			xStream >> m_uNumSteps;
			xStream >> m_uEditingAudioHash;
			xStream >> m_uSliderFlags;
			// console var
			m_xConsoleVar.ReadFromChunkStream( xStream);

		}
		break;

		default:
		{
			Asura_Assertf( false, ( "Unhandled chunk version ( %d ) for GUIMenu Slider", uChunkVersion ) );
		}
	}
}

// --------------------------------------------------------------------------------------- 

#ifdef ASURA_TOOLS
void Asura_GUIMenu_Widget_Slider::WriteToChunkStream( Asura_Chunk_Stream& xStream) const
{

	WriteLocalChunkHeader( xStream, ASURA_CHUNK_WIDGET_SLIDER_VERSION, GetType());

	PARENT::WriteToChunkStream( xStream);

	// -- write out any children --
	WriteChildObjectData( xStream);

	// -- write out slider data --
	u_int uValue = 0;

	if( m_pxPath)
	{
		uValue = m_pxPath->GetID();
	}
	xStream << uValue;

	xStream << m_uNumSteps;
	xStream << m_uEditingAudioHash;
	xStream << m_uSliderFlags;

	// write out checkbox data
	m_xConsoleVar.WriteToChunkStream( xStream );
}

void Asura_GUIMenu_Widget_Slider::ExportToXML( Tools_GUIMenu_XMLExporter* pxExporter ) const
{
	PARENT::ExportToXML( pxExporter );
	pxExporter->AddToBuffer( pxExporter->GetTabs() );
	pxExporter->AddToBuffer( "<Slider_data>\n" );
	pxExporter->IncTabs();

	// -- write out slider data --
	if( m_pxPath)
	{
		pxExporter->AddToBuffer( pxExporter->GetTabs() );
		pxExporter->AddToBuffer( "<value>" );
		pxExporter->AddToBufferHex( m_pxPath->GetID() );
		pxExporter->AddToBuffer( "</value>\n" );
	}

	pxExporter->AddToBuffer( pxExporter->GetTabs() );
	pxExporter->AddToBuffer( "<step>" );
	pxExporter->AddToBuffer( m_uNumSteps );
	pxExporter->AddToBuffer( "</step>\n" );

	pxExporter->AddToBuffer( pxExporter->GetTabs() );
	pxExporter->AddToBuffer( "<editing_audio_hash>" );
	pxExporter->AddToBufferHex( m_uEditingAudioHash );
	pxExporter->AddToBuffer( "</editing_audio_hash>\n" );

	pxExporter->AddToBuffer( pxExporter->GetTabs() );
	pxExporter->AddToBuffer( "<slider_flags>" );
	pxExporter->AddToBufferHex( m_uSliderFlags );
	pxExporter->AddToBuffer( "</slider_flags>\n" );

	// write out checkbox data
	m_xConsoleVar.ExportToXML( pxExporter ); 

	pxExporter->DecTabs();
	pxExporter->AddToBuffer( pxExporter->GetTabs() );
	pxExporter->AddToBuffer( "</Slider_data>\n" );
}

#endif

// --------------------------------------------------------------------------------------- 

#ifdef ASURA_TOOLS
void Asura_GUIMenu_Widget_Slider::UpdateLocalDataFromStyle( void)
{
	//	Asura_GUIMenu_Widget_Slider* pxInheritedSlider;

	PARENT::UpdateLocalDataFromStyle();
	// SK 16.02.07 - Temporarily disabled, i'm not sure this is necessary.
	//	if( m_pxInheritedObject != NULL)
	//	{
	//		if( (m_uOverideFlags&ASURA_GUIMENU_WIDGET_SLIDER_OVERIDE_PATH) != 0)
	//		{
	//			pxInheritedSlider = static_cast<Asura_GUIMenu_Widget_Slider*>(m_pxInheritedObject);
	// -- this wont work either, need to figure out new hash...PLACEHOLDER CODE
	//			SetActivePathFromHash( pxInheritedSlider->GetActivePathHash());
	//		}
	//	}
}
#endif

// --------------------------------------------------------------------------------------- 
// -- Protected Functions ------------------------------------------------------------------


// --------------------------------------------------------------------------------------- 

// --------------------------------------------------------------------------------------- 

void Asura_GUIMenu_Widget_Slider::SetupDefaultElements( void)
{
}

// --------------------------------------------------------------------------------------- 
// -- Private Functions ------------------------------------------------------------------

// --------------------------------------------------------------------------------------- 

// helpers to get/set the console var
void Asura_GUIMenu_Widget_Slider::InitFromConsoleVar()
{
	float fInitVal, fRange;

	if( m_xConsoleVar.IsBlank() == false)
	{
		m_xConsoleVar.GetFloatValue( fInitVal);

		Asura_CommandConsole_VarRepository::Var* pxVar = Asura_CommandConsole_VarRepository::Find( m_xConsoleVar.GetName().GetString());
		if( pxVar)
		{
			m_fConsoleVarLowerLimit = pxVar->m_xData.fLowerLimit;
			m_fConsoleVarUpperLimit = pxVar->m_xData.fUpperLimit;
		}

		fRange = m_fConsoleVarUpperLimit - m_fConsoleVarLowerLimit;
		m_fPosition = (fInitVal-m_fConsoleVarLowerLimit)/fRange;
	}
}

// --------------------------------------------------------------------------------------- 

void Asura_GUIMenu_Widget_Slider::SetConsoleVar() const
{
	float fFinalVal, fRange;

	if ( m_xConsoleVar.IsBlank() == false )
	{
		fRange = m_fConsoleVarUpperLimit - m_fConsoleVarLowerLimit;

		fFinalVal = (m_fPosition * fRange ) + m_fConsoleVarLowerLimit;

		m_xConsoleVar.SetFloatValue( fFinalVal );
	}
}

// --------------------------------------------------------------------------------------- 
void Asura_GUIMenu_Widget_Slider::SetSliderFromCursorPos(const Asura_Vector_2& xCursorPos)
{
	if(!m_pxPath) return;

	m_fPosition = m_pxPath->GetPathValueNearestPosition(xCursorPos);
	SetConsoleVar();
}

// --------------------------------------------------------------------------------------- 

void Asura_GUIMenu_Widget_Slider::SetSliderFlag( u_int uSliderFlag, bool bTrueOrFalse)
{
	if( bTrueOrFalse)
	{
		m_uSliderFlags |= uSliderFlag;
	}
	else
	{
		m_uSliderFlags &= (0xffffffff - uSliderFlag);
	}
}

// --------------------------------------------------------------------------------------- 

bool Asura_GUIMenu_Widget_Slider::IsSliderFlagSet( u_int uSliderFlag) const
{
	return ((m_uSliderFlags&uSliderFlag) != 0);
}

// --------------------------------------------------------------------------------------- 

bool Asura_GUIMenu_Widget_Slider::RemoveFromThisContainer( Asura_GUIMenu_Root* pxObject)
{
	if(m_pxPath == pxObject) m_pxPath = 0;

	return PARENT::RemoveFromThisContainer(pxObject);
}


// --------------------------------------------------------------------------------------- 

#ifdef ASURA_TOOLS
void Asura_GUIMenu_Widget_Slider::PostMenuPreviewCleanup()
{
	PARENT::PostMenuPreviewCleanup();

	//Need to reset the the slider back to zero while editing. RAR 08/03/2007
	m_fPosition = 0.0f;
	m_fOldPosition = 0.0f;
	if(m_pxPath)
	{
		m_pxPath->SetCurrentPathValue(0.0f);
	}
}
#endif //ASURA_TOOLS



#endif // ASURA_USE_GUI_MENU_SYSTEM
