///////////////////////////////////////////////////////////////////////////////
//
// File   : Asura_ControlMap.h
// Author : Andy Tate
// Date   : 1-Dec-2005
//
// Copyright (C) 2005 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __ASURA_CONTROLMAP_H__
#define __ASURA_CONTROLMAP_H__

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Input/Asura_ControlMapping.h"
#include "Input/Asura_ControlMapping_Axis.h"
#include "Input/Asura_ControlMapping_HoldKey.h"
#include "Input/Asura_ControlMapping_Combination.h"

////////////////////////////////////////////////////////////////////////////////
// Class declarations...
////////////////////////////////////////////////////////////////////////////////

class Asura_ControlMap
{
public:
	Asura_ControlMap(void);
	Asura_ControlMap(const u_int uNumberOfMappings);
	~Asura_ControlMap();

	// Functions to manipulate mappings...
	void CreateMappings(const u_int uNumberOfMappings);
	void DeleteMappings(void);

#ifdef ASURA_USE_NEW_INPUT
	void AddAxisMapping(const u_int uMapping, const u_int uPositiveID, const u_int uNegativeID, const float fScale = 1.0f, const float fResponseTime = 0.0f, const float fDecayTime = -1.0f, const ASURA_CONTROLMAPPING_AXIS_MASK eSection = ASURA_CONTROLMAPPING_AXIS_MASK_ALL, const float fExponent = 1.0f );
	void AddAxisMapping(const u_int uMapping, const u_int uPositiveID, const u_int uNegativeID, const float fScale, const float* const pfResponseTime, const float* const pfDecayTime, const ASURA_CONTROLMAPPING_AXIS_MASK eSection = ASURA_CONTROLMAPPING_AXIS_MASK_ALL, const float* const pfExponent = NULL );
#else
	void AddAxisMapping(const u_int uMapping, const int iAxisID, const float fScale = 1.0f, const float fResponseTime = 0.0f, const float fDecayTime = -1.0f, const ASURA_CONTROLMAPPING_AXIS_MASK eSection = ASURA_CONTROLMAPPING_AXIS_MASK_ALL, const float fExponent = 1.0f );
	void AddAxisMapping(const u_int uMapping, const int iAxisID, const float fScale, const float* const pfResponseTime, const float* const pfDecayTime, const ASURA_CONTROLMAPPING_AXIS_MASK eSection = ASURA_CONTROLMAPPING_AXIS_MASK_ALL, const float* const pfExponent = NULL );
#endif
	void AddKeyMapping(const u_int uMapping, const int iKeyID, const Asura_ControlMapping::KeyHoldType eHoldType, const float fScale = 1.0f);
	void AddHoldKeyMapping(const u_int uMapping, const int iKeyID, const float fMinDuration, const float fScale = 1.0f, const Asura_ControlMapping_HoldKey::RampType eRampType = Asura_ControlMapping_HoldKey::NO_RAMPING,  const Asura_ControlMapping::KeyHoldType eHoldType = Asura_ControlMapping::ON_PRESS);
	void AddTapKeyMapping(const u_int uMapping, const int iKeyID, const float fMaxPressedDuration, const float fScale = 1.0f);
	void AddTapKeyMapping(const u_int uMapping, const int iKeyID, const u_int uNumTaps, const Asura_ControlMapping::KeyHoldType eHoldType, const float fMaxPressDuration, const float fMaxReleaseDuration, const float fScale = 1.0f);
	void AddTapAxisMapping(const u_int uMapping, const int iAxisID, const float fMaxPressedDuration, const float fTapRegisterThreshold, const float fScale = 1.0f);
	void AddTapAxisMapping(const u_int uMapping, const int iAxisID, const u_int uNumTaps, const Asura_ControlMapping::KeyHoldType eHoldType, const float fMaxPressDuration, const float fMaxReleaseDuration, const float fTapRegisterThreshold, const float fScale = 1.0f);
	void AddCombinationMapping(const u_int uMapping, Asura_ControlMapping_Combination* pxCombinationMapping);

	void StoreMapping(const u_int uMapping, Asura_ControlMapping* pxMapping);

	void DeleteMapping(const u_int uMapping);

	void Activate(void);
	void Update(void);
	void Deactivate(void);

	// Accessor functions...
	float GetScale(const u_int uMapping) const					{ return(m_pfScales[uMapping]); }
	void  SetScale(const u_int uMapping, const float fScale)	{ m_pfScales[uMapping] = fScale; }

	float GetValue( const u_int uMapping ) const;
	float GetHUDValue( const u_int uMapping ) const;
	u_int GetNumMappings() const { return m_uNumberOfMappings; }
	u_int GetNumMappingsOfType( const Asura_Hash_ID uMapping ) const;
	const Asura_ControlMapping*	GetMapping( const u_int uMapping ) const { return m_ppxMappings[uMapping]; }

	// Clears the inputs, returns the mappings to their default states.
	void ResetValues();

	// Resets a specific mapping to it's default state
	void ResetValue(const u_int uMapping);

#ifdef ASURA_USE_NEW_INPUT
	void SetControllerIndex(u_int uIndex) { m_uControllerIndex = uIndex; }
	u_int GetControllerIndex() const { return m_uControllerIndex; }
#endif

protected:
#ifdef ASURA_USE_NEW_INPUT
	u_int m_uControllerIndex;
#endif

private:
	// Arrays of mappings...
	bool					m_bActive;
	u_int					m_uNumberOfMappings;
	Asura_ControlMapping**	m_ppxMappings;
	float*					m_pfScales;

	// Current mapping values...
	float*					m_pfValues;
	float*					m_pfHUDValues;

	Asura_ControlMap(const Asura_ControlMap& xOther);
	Asura_ControlMap& operator=(const Asura_ControlMap& xOther);
};

#endif   // ifndef __ASURA_CONTROLMAP_H__


/// End of file. ///
