
///////////////////////////////////////////////////////////////////////////////
//
// File   : Asura_ControlMapping.h
// Author : Andy Tate
// Date   : 1-Dec-2005
//
// Copyright (C) 2005 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __ASURA_CONTROLMAPPING_H__
#define __ASURA_CONTROLMAPPING_H__

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Forward class declarations...
////////////////////////////////////////////////////////////////////////////////
class Asura_ControlMap;

////////////////////////////////////////////////////////////////////////////////
// Class declaration...
////////////////////////////////////////////////////////////////////////////////

class Asura_ControlMapping
{
public:
	// Public enumerated types...

	enum KeyHoldType
	{
		ON_PRESS,
		ON_RELEASE,

		ON_DEBOUNCED_PRESS,
		ON_DEBOUNCED_RELEASE,

		MAX_KEY_HOLD_TYPES
	};

	// Public functions...

	Asura_ControlMapping(const float fScale)
	: m_fScale(fScale)
	, m_pxNextMapping(NULL)
#ifdef ASURA_USE_NEW_INPUT
	, m_pxOwner(NULL)
#endif
	, m_uFlags(0)
	{}
	virtual ~Asura_ControlMapping ()											{ delete (m_pxNextMapping); }

	// Abstract functions...
	virtual	void			ResetValue				(void)													= 0;
	virtual	void			UpdateValues			(const bool bActive, float& fValue, float& fHUDValue)	= 0;
	virtual Asura_Hash_ID	GetInstanceMappingType	(void) const											= 0;

	// Accessor functions...
	bool					IsActive		(void) const						{ return(m_bActive); }
	virtual	bool			IsComboMapping  (void) const						{ return false; }

	void					SetScale		(const float fScale)				{ m_fScale	= fScale; }
	float					GetScale		(void) const						{ return(m_fScale); }

	void					SetNextMapping	(Asura_ControlMapping* pxMapping)	{ m_pxNextMapping	= pxMapping; }
	Asura_ControlMapping*	GetNextMapping	(void) const						{ return(m_pxNextMapping); }

	virtual int				GetControlID		(void) const					{ return -1; } 

#ifdef ASURA_USE_NEW_INPUT
	virtual void			SetOwner(Asura_ControlMap* pxOwner) { m_pxOwner = pxOwner; }
#endif

	void SetPauseWhileInactive(bool bPauseWhileInactive)	{ m_bPauseWhileInactive = bPauseWhileInactive; }
	bool GetPauseWhileInactive() const						{ return m_bPauseWhileInactive; }

protected:
	// Protected variables...
	float					m_fScale;			// The scale to be applied to the result.

	// Linked list of control mappings...
	Asura_ControlMapping*	m_pxNextMapping;	// A pointer to the next control-mapping.

#ifdef ASURA_USE_NEW_INPUT
	Asura_ControlMap*		m_pxOwner;			// A pointer to the parent control-map.
#endif

	union
	{
		u_int m_uFlags;
		struct
		{
			u_int	m_bActive				: 1,	// Whether this control-mapping is currently active.
					m_bPauseWhileInactive	: 1,	// Bool to instruct not updating values when the control mapping is inactive.
											: 0;
		};
	};

	// Protected functions...


private:
	// Private variables...


	// Private functions...

	Asura_ControlMapping  (void);
	Asura_ControlMapping  (const Asura_ControlMapping& xOther);

			Asura_ControlMapping&	operator=		(const Asura_ControlMapping& xOther);

};   // End of class 'Asura_ControlMapping'.


#endif   // ifndef __ASURA_CONTROLMAPPING_H__


/// End of file. ///
