
///////////////////////////////////////////////////////////////////////////////
//
// File   : Asura_ControlMapping_Axis.h
// Author : Andy Tate
// Date   : 1-Dec-2005
//
// Copyright (C) 2005 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __ASURA_CONTROLMAPPING_AXIS_H__
#define __ASURA_CONTROLMAPPING_AXIS_H__

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Input/Asura_ControlMapping.h"

////////////////////////////////////////////////////////////////////////////////
// Forward class declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Class declaration...
////////////////////////////////////////////////////////////////////////////////

enum ASURA_CONTROLMAPPING_AXIS_MASK
{
	ASURA_CONTROLMAPPING_AXIS_MASK_ALL = 0,
	ASURA_CONTROLMAPPING_AXIS_MASK_POSITIVE,
	ASURA_CONTROLMAPPING_AXIS_MASK_NEGATIVE,
};

class Asura_ControlMapping_Axis : public Asura_ControlMapping
{
public:
	// Public functions...

#ifdef ASURA_USE_NEW_INPUT
	Asura_ControlMapping_Axis(const u_int uPositiveID, const u_int uNegativeID, const float fScale, const float fResponseTime, const float fDecayTime, const ASURA_CONTROLMAPPING_AXIS_MASK eSection = ASURA_CONTROLMAPPING_AXIS_MASK_ALL, const float fExponent = 1.0f);
	Asura_ControlMapping_Axis(const u_int uPositiveID, const u_int uNegativeID, const float fScale, const float* const pfResponseTime, const float* const pfDecayTime, const ASURA_CONTROLMAPPING_AXIS_MASK eSection = ASURA_CONTROLMAPPING_AXIS_MASK_ALL, const float* const pfExponent = NULL);
#else
	Asura_ControlMapping_Axis(const int iAxisID, const float fScale, const float fResponseTime, const float fDecayTime, const ASURA_CONTROLMAPPING_AXIS_MASK eSection = ASURA_CONTROLMAPPING_AXIS_MASK_ALL, const float fExponent = 1.0f);
	Asura_ControlMapping_Axis(const int iAxisID, const float fScale, const float* const pfResponseTime, const float* const pfDecayTime, const ASURA_CONTROLMAPPING_AXIS_MASK eSection = ASURA_CONTROLMAPPING_AXIS_MASK_ALL, const float* const pfExponent = NULL);
#endif
//	virtual ~Asura_ControlMapping_Axis ();

	// Overridden virtual functions...
	virtual void	ResetValue(void);
	virtual	void	UpdateValues(const bool bActive, float& fValue, float& fHUDValue);
#ifndef ASURA_USE_NEW_INPUT
	virtual int		GetControlID(void) const	{ return m_iAxisID; } 
#endif

	virtual Asura_Hash_ID	GetInstanceMappingType(void) const	{ return s_uMappingType; } 
	static	Asura_Hash_ID	GetClassMappingType( void )			{ return s_uMappingType; }

protected:
	// Protected variables...

	// The axis to check for...
#ifdef ASURA_USE_NEW_INPUT
	u_int m_uPositiveID;
	u_int m_uNegativeID;
#else
	int	  m_iAxisID;
#endif
	float m_fValueToReport;
	const float m_fResponseTime;
	const float m_fDecayTime;
	const float m_fExponent;
	const float* const m_pfResponseTime;
	const float* const m_pfDecayTime;
	const float* const m_pfExponent;
	ASURA_CONTROLMAPPING_AXIS_MASK m_eSection;


	// Protected functions...


private:
	// Private variables...
	static const Asura_Hash_ID s_uMappingType = 0x002DE7A1;	//Asura_GetHashId( "AXIS" )

	// Private functions...

	Asura_ControlMapping_Axis(void);
	Asura_ControlMapping_Axis(const Asura_ControlMapping_Axis& xOther);

	Asura_ControlMapping_Axis&	operator=	(const Asura_ControlMapping_Axis& xOther);

};   // End of class 'Asura_ControlMapping_Axis'.


#endif   // ifndef __ASURA_CONTROLMAPPING_AXIS_H__


/// End of file. ///
