
///////////////////////////////////////////////////////////////////////////////
//
// File   : Asura_ControlMapping_TapAxis.h
// Author : Anthony Gregson
// Date   : 07/06/06
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __ASURA_CONTROLMAPPING_TAPAXIS_H__
#define __ASURA_CONTROLMAPPING_TAPAXIS_H__

// This will no longer be necessary using the new input, due to it splitting axes.
#if !defined(ASURA_USE_NEW_INPUT) || defined(ASURA_USE_NEW_INPUT_LEGACY_SUPPORT)
////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Input/Asura_ControlMapping.h"


////////////////////////////////////////////////////////////////////////////////
// Forward class declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Class declaration...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Class  : Asura_ControlMapping_TapAxis
// Author : Anthony Gregson
// Date   : 07/06/06
//
// Description  : Handles simulating the tap-pressing of a key with an axis value moving above and below a threshold.
////////////////////////////////////////////////////////////////////////////////

class Asura_ControlMapping_TapAxis : public Asura_ControlMapping
{
public:
	// Public functions...

	Asura_ControlMapping_TapAxis		  (const int iAxisID, const float fMaxPressDuration, const float fTapRegisterThreshold, const float fScale);
	Asura_ControlMapping_TapAxis		  (const int iAxisID, const u_int uNumTaps, const KeyHoldType eHoldType, const float fMaxPressDuration, const float fMaxReleaseDuration, const float fTapRegisterThreshold, const float fScale);
//	virtual ~Asura_ControlMapping_TapAxis ();

	// Overridden virtual functions...
	virtual void	ResetValue(void);
	virtual	void	UpdateValues(const bool bActive, float& fValue, float& fHUDValue);
	virtual int		GetControlID(void) const	{ return m_iAxisID; } 

	virtual Asura_Hash_ID	GetInstanceMappingType(void) const	{ return s_uMappingType; } 
	static	Asura_Hash_ID	GetClassMappingType( void )			{ return s_uMappingType; }

protected:
	// Protected enumerated types...

	enum State
	{
		INACTIVE,
		PRESSING,
		RELEASING,

		MAX_STATES
	};


	// Protected variables...

	// The axis AND the type of hold to check-for...
	int				m_iAxisID;
	u_int			m_uNumTaps;
	KeyHoldType		m_eHoldType;
	float			m_fMaxPressDuration;
	float			m_fMaxReleaseDuration;
	float			m_fTapRegisterThreshold;

	// The values to be stored...
	State			m_eState;
	u_int			m_uCurrentNumTaps;
	float			m_fCurrentDuration;


	// Protected functions...


private:
	// Private variables...
	static const Asura_Hash_ID s_uMappingType = 0xA4311DC4;	//Asura_GetHashId( "TAPAXIS" )

	// Private functions...

	Asura_ControlMapping_TapAxis	(void);
	Asura_ControlMapping_TapAxis	(const Asura_ControlMapping_TapAxis& xOther);

			Asura_ControlMapping_TapAxis&	operator= (const Asura_ControlMapping_TapAxis& xOther);

};   // End of class 'Asura_ControlMapping_TapAxis'.

#endif//ndef ASURA_USE_NEW_INPUT

#endif   // ifndef __ASURA_CONTROLMAPPING_TAPAXIS_H__


/// End of file. ///
