
///////////////////////////////////////////////////////////////////////////////
//
// File   : Asura_ControlMapping_TapKey.h
// Author : Andy Tate
// Date   : 6-Jan-2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __ASURA_CONTROLMAPPING_TAPKEY_H__
#define __ASURA_CONTROLMAPPING_TAPKEY_H__

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Input/Asura_ControlMapping.h"


////////////////////////////////////////////////////////////////////////////////
// Forward class declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Class declaration...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Class        : Asura_ControlMapping_TapKey
// Author       : Andy Tate
// Date         : 6-Jan-2006
//
// Description  : Handles tap-pressing of a key.
////////////////////////////////////////////////////////////////////////////////

class Asura_ControlMapping_TapKey : public Asura_ControlMapping
{
public:
	// Public functions...

	Asura_ControlMapping_TapKey          (const int iKeyID, const float fMaxPressDuration, const float fScale);
	Asura_ControlMapping_TapKey          (const int iKeyID, const u_int uNumTaps, const KeyHoldType eHoldType, const float fMaxPressDuration, const float fMaxReleaseDuration, const float fScale);
//	virtual ~Asura_ControlMapping_TapKey ();

	// Overridden virtual functions...
	virtual void	ResetValue();
	virtual	void	UpdateValues(const bool bActive, float& fValue, float& fHUDValue);
	virtual int		GetControlID(void) const	{ return m_iKeyID; } 

	virtual Asura_Hash_ID	GetInstanceMappingType(void) const	{ return s_uMappingType; } 
	static	Asura_Hash_ID	GetClassMappingType( void )			{ return s_uMappingType; }

protected:
	// Protected enumerated types...

	enum State
	{
		INACTIVE,
		PRESSING,
		RELEASING,

		MAX_STATES
	};


	// Protected variables...

	// The key AND the type of hold to check-for...
	int				m_iKeyID;
	u_int			m_uNumTaps;
	KeyHoldType		m_eHoldType;
	float			m_fMaxPressDuration;
	float			m_fMaxReleaseDuration;

	// The values to be stored...
	State			m_eState;
	u_int			m_uCurrentNumTaps;
	float			m_fCurrentDuration;


	// Protected functions...


private:
	// Private variables...
	static const Asura_Hash_ID s_uMappingType = 0xCB7D97DC;	//Asura_GetHashId( "TAPKEY" )

	// Private functions...

	Asura_ControlMapping_TapKey  (void);
	Asura_ControlMapping_TapKey  (const Asura_ControlMapping_TapKey& xOther);

	Asura_ControlMapping_TapKey&	operator=	(const Asura_ControlMapping_TapKey& xOther);

};   // End of class 'Asura_ControlMapping_TapKey'.


#endif   // ifndef __ASURA_CONTROLMAPPING_TAPKEY_H__


/// End of file. ///
