//
// Asura_Input.cpp
//
// Created by KOR - 05 October 2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

#if ASURA_USE_NEW_INPUT

// This file's header.
#include "Input/Asura_Input.h"

// Asura headers
#include "Input/Asura_Controller.h"

// Static members
Asura_Controller*	Asura_Input::m_apxControllers[ASURA_INPUT_PLATFORM_MAX_CONTROLLERS];
bool				Asura_Input::s_bAnyKeyPressedEvaluated = false;
bool				Asura_Input::s_bAnyKeyPressed = false;
bool				Asura_Input::s_bDebouncedAnyKeyPressedHasBeenEvaluated = false;
bool				Asura_Input::s_bDebouncedAnyKeyHasBeenPressed = false;


void Asura_Input::Initialise()
{
	Asura_Input::Platform_Initialise();
}

void Asura_Input::Shutdown()
{
	Asura_Input::Platform_Shutdown();
}

void Asura_Input::Reset()
{
	Asura_Input::Platform_Reset();
}

void Asura_Input::Update()
{
	// Blank flags
	s_bAnyKeyPressedEvaluated = false;
	s_bDebouncedAnyKeyPressedHasBeenEvaluated = false;

	// Copy current to previous input
	for(u_int uController = 0; uController < ASURA_INPUT_PLATFORM_MAX_CONTROLLERS; ++uController)
	{
		Asura_Controller* pxController = m_apxControllers[uController];
		if(pxController)
		{
			pxController->Update();
		}
	}

	// Platform update
	Platform_Update();
}

void Asura_Input::RegisterController(u_int uControllerIndex, Asura_Controller* pxController)
{
	Asura_Assert(uControllerIndex < ASURA_INPUT_PLATFORM_MAX_CONTROLLERS, "Controller index out of range");
	Asura_Assert(!m_apxControllers[uControllerIndex], "There is already a controller registered in this port");
	m_apxControllers[uControllerIndex] = pxController;
}

void Asura_Input::UnregisterController(u_int uControllerIndex)
{
	Asura_Assert(uControllerIndex < ASURA_INPUT_PLATFORM_MAX_CONTROLLERS, "Controller index out of range");
	Asura_Assert(m_apxControllers[uControllerIndex], "There is no controller registered in this port");
	m_apxControllers[uControllerIndex] = NULL;
}

Asura_Controller* Asura_Input::GetController(u_int uControllerIndex)
{
	Asura_Assert(uControllerIndex < ASURA_INPUT_PLATFORM_MAX_CONTROLLERS, "Controller index out of range");
	return (uControllerIndex < ASURA_INPUT_PLATFORM_MAX_CONTROLLERS) ? m_apxControllers[uControllerIndex] : NULL;
}

bool Asura_Input::HasAnyKeyBeenPressed()
{
	if(s_bAnyKeyPressedEvaluated)
	{
		return s_bAnyKeyPressed;
	}
	s_bAnyKeyPressedEvaluated = true;
	for(u_int uController = 0; uController < ASURA_INPUT_PLATFORM_MAX_CONTROLLERS; ++uController)
	{
		if(m_apxControllers[uController]->HasAnyKeyBeenPressed())
		{
			s_bAnyKeyPressed = true;
			return true;
		}
	}
	s_bAnyKeyPressed = false;
	return false;
}

bool Asura_Input::DebouncedHasAnyKeyBeenPressed()
{
	for(u_int uController = 0; uController < ASURA_INPUT_PLATFORM_MAX_CONTROLLERS; ++uController)
	{
		if(m_apxControllers[uController]->DebouncedHasAnyKeyBeenPressed())
		{
			return true;
		}
	}
	return false;
}

bool Asura_Input::IsKeyPressed(u_int uControllerIndex, u_int uKeyID )
{
	Asura_Assert(uControllerIndex < ASURA_INPUT_PLATFORM_MAX_CONTROLLERS, "Controller index out of range");
	const Asura_Controller* pxController = m_apxControllers[uControllerIndex];
	return pxController ? pxController->IsKeyPressed(uKeyID) : false;
}

bool Asura_Input::IsDebouncedKeyPressed(u_int uControllerIndex, u_int uKeyID )
{
	Asura_Assert(uControllerIndex < ASURA_INPUT_PLATFORM_MAX_CONTROLLERS, "Controller index out of range");
	const Asura_Controller* pxController = m_apxControllers[uControllerIndex];
	return pxController ? pxController->IsDebouncedKeyPressed(uKeyID) : false;
}

bool Asura_Input::IsDebouncedKeyNotPressed(u_int uControllerIndex,  u_int uKeyID )
{
	Asura_Assert(uControllerIndex < ASURA_INPUT_PLATFORM_MAX_CONTROLLERS, "Controller index out of range");
	const Asura_Controller* pxController = m_apxControllers[uControllerIndex];
	return pxController ? pxController->IsDebouncedKeyNotPressed(uKeyID) : false;
}

float Asura_Input::GetKeyValue(u_int uControllerIndex, u_int uKeyID )
{
	Asura_Assert(uControllerIndex < ASURA_INPUT_PLATFORM_MAX_CONTROLLERS, "Controller index out of range");
	const Asura_Controller* pxController = m_apxControllers[uControllerIndex];
	return pxController ? pxController->GetKeyValue(uKeyID) : 0.f;
}

float Asura_Input::GetAxisValue(u_int uControllerIndex, u_int uPositiveKeyID, u_int uNegativeKeyID )
{
	Asura_Assert(uControllerIndex < ASURA_INPUT_PLATFORM_MAX_CONTROLLERS, "Controller index out of range");
	const Asura_Controller* pxController = m_apxControllers[uControllerIndex];
	return pxController ? pxController->GetAxisValue(uPositiveKeyID, uNegativeKeyID) : 0.f;
}

#endif // #if ASURA_USE_NEW_INPUT
