//////////////////////////////////////////////////////////////////////////////
//
// Asura_CfgFileReader.h
//
// Created by OR - 23/08/2004 14:24:05
//
// Copyright (C) 2004 Rebellion, All Rights Reserved.
//
// Description:
//
//     class to help with reading and executing a bunch of console commands 
//     from a .cfg file at startup. Used for dedicated server builds.
//
//////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif

#ifndef __ASURA_CFGFILEREADER_H__
#define __ASURA_CFGFILEREADER_H__

// I N C L U D E S ///////////////////////////////////////////////////////////

// Engine platform-specific.
#include "Core/Asura_PlatformBasics.h"
#include "Interface/Asura_CommandConsole.h"

// C L A S S E S /////////////////////////////////////////////////////////////
class Asura_CfgFileReader
{

    public: 

        // C L A S S   M E T H O D S /////////////////////////////////////////
        static bool ExecuteFile( const Asura_UnicodeChar* szFilename );

	protected:
		// was public.

        // T Y P E S /////////////////////////////////////////////////////////

        enum GETRES
        {
            GETRES_OK,
            GETRES_MISSING,
            GETRES_INVALID_FORMAT,
        };

        // C O N S T R U C T O R S   /   D E S T R U C T O R S ///////////////

        virtual ~Asura_CfgFileReader( void );

        // M E T H O D S /////////////////////////////////////////////////////

        void ItBegin( void ) const;
        bool ItDone( void ) const;
        const Asura_UnicodeChar* ItGetRawLine( void ) const;
        const Asura_UnicodeChar* ItGetName( void ) const;
        const Asura_UnicodeChar* ItGetValue( void ) const;
        u_int ItGetLineNum( void ) const;
        bool ItHasBeenUsed( void ) const;
        void ItNext( void ) const;

        GETRES GetBool( const Asura_UnicodeChar* wszName, bool* pbValue, u_int* puLineNum = NULL ) const;
        GETRES GetFloat( const Asura_UnicodeChar* wszName, float* pfValue, u_int* puLineNum = NULL ) const;
        GETRES GetInt( const Asura_UnicodeChar* wszName, int* piValue, u_int* puLineNum = NULL ) const;
        GETRES GetStr( const Asura_UnicodeChar* wszName, const Asura_UnicodeChar** pwszValue, u_int* puLineNum = NULL ) const;

    protected:
        
        // T Y P E S /////////////////////////////////////////////////////////

        enum READRES
        {
            READRES_OK,
            READRES_EOF,
            READRES_ERR,
        };

        // C O N S T R U C T O R S   /   D E S T R U C T O R S ///////////////

        Asura_CfgFileReader( void );

        // M E T H O D S /////////////////////////////////////////////////////

        virtual bool    Open( const Asura_UnicodeChar* wszFilename ) = 0;
        virtual READRES ReadLine( Asura_UnicodeChar* wszBfr, u_int uBfrSize ) = 0;
        virtual void    Close( void ) = 0;
        virtual bool	Parse( void );
        virtual bool	Execute( void );

    private:

        // F O R W A R D   D E C L A R A T I O N S ///////////////////////////

        class ListNode;

        // C L A S S   M E T H O D S /////////////////////////////////////////

        static Asura_CfgFileReader* Platform_Create( void );
        
        // M E T H O D S /////////////////////////////////////////////////////

        void DeallocateList( void );

        // S T A T E /////////////////////////////////////////////////////////

        ListNode* m_pxListHead;
        mutable const ListNode* m_pxItNode;

		friend class Asura_Platform_CfgFileReader;

};

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

#endif//__ASURA_CFGFILEREADER_H__
