////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

//
// Asura_Channel_Graph.cpp
//
// Created by Dan Clarke - 18/08/2008
//
// Copyright (C) 2008 Rebellion.  All Rights Reserved.
//

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

#ifdef ASURA_TOOLS

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include "Asura_Channel_Graph_Dialog.h"
#include "Rendering/Asura_Render.h"
#include "Core/Asura_TextureManagement.h"
#include "Core/Asura_UserInput.h"
#include "Interface/Asura_Cursor.h"
#include "Misc/Tools_Dialog.h"

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#define DP_EQUALITY_TEST(A,B,C)		(int)((A)*(100.0f*(C)))==(int)((B)*(100.0f*(C)))

#define CHANNEL_HEIGHT						(4.0f)
#define CHANNEL_COLOUR_1					(0x00666666 | Asura_OverlayBrightness)
#define CHANNEL_COLOUR_2					(0x00666666 | Asura_OverlayBrightness)
#define CHANNEL_LINE_GRAPH_COLOUR			(0x00ff0000 | Asura_OverlayBrightness)
#define CHANNEL_CONTROL_POINT_COLOUR		(0x00ff0000 | Asura_OverlayBrightness)
#define CHANNEL_CONTROL_POINT_COLOUR_FOCUS	(0x00FEAE50 | Asura_OverlayBrightness)
#define BLOCK_TEXT_COLOUR					(0x00999999 | Asura_OverlayBrightness)
#define PLAYBACK_LINE_COLOUR				(0x00629FFB | Asura_OverlayBrightness)
#define CHANNEL_MARGIN						(0.3f)
#define CHANNEL_CONTROL_POINT_SIZE			(0.75f)
#define BLOCK_PADDING						(1.0f)
#define DIALOGS_RIGHT_MARGIN				(1.5f)
#define MOVE_OR_RESIZE_WIDTH_TOLERANCE		(0.5f)
#define MIN_BLOCK_WIDTH						(1.0f)
#define CONTROL_POINT_SNAP_X_TOLERANCE		(0.01f)
#define CONTROL_POINT_SNAP_Y_TOLERANCE		(0.08f)
#define CHANNEL_DIALOG_TICK_HEIGHT			(5.0f)
#define CHANNEL_DIALOG_TICK_BACKGROUND		(0x00888888 | Asura_OverlayBrightness)

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

Asura_Channel_Graph_Dialog::Asura_Channel_Graph_Dialog ( const Asura_Vector_2& xOffset, const float fWidth, bool* pbModified ) : Asura_Dialog ("", xOffset)
{
	m_fWidth = fWidth;

	m_fDomainMin = 0.0f;
	m_fDomainMax = 1.0f;

	m_pbModified = pbModified;

	m_fPlaybackMarker = 0.0f;
	m_bPlaybackMarkerVisible = false;

	m_fSnapInterval = -1.0f;

	m_xMaxCoord.x = fWidth + DIALOGS_RIGHT_MARGIN;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

Asura_Channel_Graph_Dialog::~Asura_Channel_Graph_Dialog ()
{
	while( !m_xChannelList.IsEmpty() )
	{
		Asura_Channel_Graph_Channel* pxChannel = m_xChannelList.GetHeadObject ();
		delete pxChannel;
	}
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

Asura_Channel_Graph_Channel* Asura_Channel_Graph_Dialog::AddChannel ()
{
	Asura_Channel_Graph_Channel* pxNewChannel = new Asura_Channel_Graph_Channel (this);

	m_xChannelList.AddToList (pxNewChannel, false);

	m_xMaxCoord.y += CHANNEL_HEIGHT;

	UpdateSize ();

	return pxNewChannel;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void Asura_Channel_Graph_Dialog::Update (const Asura_Vector_2& xOrigin)
{
	if (!IsHidden ())
	{
		Asura_List_It <Asura_Channel_Graph_Channel> xIter (m_xChannelList);

		Asura_Vector_2 xOffset (CHANNEL_MARGIN, CHANNEL_MARGIN);

		while (!xIter.Done ())
		{
			Asura_Channel_Graph_Channel* pxChannel = xIter.GetCurrent ();

			pxChannel->Update (xOrigin + m_xOffset + xOffset);

			xOffset.y += (CHANNEL_HEIGHT) /** ASURA_TEXT_OUTPUT_FONT_HEIGHT*/;

			xIter.Next ();
		}

		return;
	}
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

float Asura_Channel_Graph_Channel::RoundMouseXToTicks ( float fMouseX, const Asura_Vector_2 xOrigin )
{
	const float fSnapInterval = m_pxParent->GetSnapInterval();
	if( fSnapInterval > 0.0f )
	{
		float fDomainMin = 0.0f;
		float fDomainMax = 0.0f;
		m_pxParent->GetGraphDomain ( &fDomainMin, &fDomainMax );

		const float fNormalisedX = ( fMouseX - xOrigin.x ) / m_fPhysicalWidth;
		const float fDomainX = fDomainMin + fNormalisedX * ( fDomainMax - fDomainMin );
		const int iNumIntervals = static_cast<int>( ( fDomainX + fSnapInterval * 0.5f ) / fSnapInterval );
		const float fSnappedNormalisedX = ( static_cast<float>( iNumIntervals ) * fSnapInterval ) / ( fDomainMax - fDomainMin );

		return ( xOrigin.x + fSnappedNormalisedX * m_fPhysicalWidth );
	}

	return fMouseX;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void Asura_Channel_Graph_Channel::SnapAllBlocksToTicks ()
{
	const float fSnapInterval = m_pxParent->GetSnapInterval();

	if( fSnapInterval <= 0.0f )
	{
		return;
	}

	float fDomainMin = 0.0f;
	float fDomainMax = 0.0f;
	m_pxParent->GetGraphDomain ( &fDomainMin, &fDomainMax );
		
	Asura_List_It<Asura_Channel_Graph_Block> xIter( m_xBlockList );
	
	while( !xIter.Done() )
	{
		Asura_Channel_Graph_Block* pxBlock = xIter.GetCurrent();

		const float fStart = pxBlock->GetStartPosition();
		const float fEnd = pxBlock->GetEndPosition();

		int iNumIntervals = static_cast<int>( ( fStart + fSnapInterval * 0.5f ) / fSnapInterval );
		const float fNewStartPos = fDomainMin + static_cast<float>( iNumIntervals ) * fSnapInterval;
		pxBlock->SetStartPosition( fNewStartPos, true );

		iNumIntervals = static_cast<int>( ( fEnd + fSnapInterval * 0.5f ) / fSnapInterval );
		const float fNewEndPos = fDomainMin + static_cast<float>( iNumIntervals ) * fSnapInterval;
		if( fNewEndPos > fDomainMax )
		{
			pxBlock->SetEndPosition( fDomainMax, true );
		}
		else
		{
			pxBlock->SetEndPosition( fNewEndPos, true );
		}
		
		xIter.Next ();
	}

	UpdateControlPointPositions();
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void Asura_Channel_Graph_Channel::Update (const Asura_Vector_2& xOrigin)
{
	if (Asura_UserInput::DebouncedKeyPressed (ASURA_KEYID_MOUSEBUTTON1))
	{
		// Hittest against nodes

		if ((m_iDraggedNode == -1) && (m_iDraggingBlock == -1))
		{
			for (u_int n = 0; n < ASURA_CHANNEL_GRAPH_MAX_CONTROL_POINTS; n ++)
			{
				if (m_axControlPoints [n].bValid)
				{
					const float fScreenX = ConvertChannelXToScreen (m_axControlPoints [n].fX);
					const float fScreenY = ConvertChannelYToScreen (m_axControlPoints [n].fY);

					Asura_Vector_2 xScreenPos (xOrigin.x + fScreenX, xOrigin.y + fScreenY);

					xScreenPos.x -= (CHANNEL_CONTROL_POINT_SIZE * 0.5f);
					xScreenPos.y -= (CHANNEL_CONTROL_POINT_SIZE * 0.5f);

					if (Asura_Cursor::Inside (xScreenPos, CHANNEL_CONTROL_POINT_SIZE, CHANNEL_CONTROL_POINT_SIZE))
					{
						m_iDraggedNode = n;
					}
				}
			}
		}

		// If no nodes were hit, then hittest against blocks

		if ((m_iDraggedNode == -1) && (m_iDraggingBlock == -1))
		{
			Asura_List_It <Asura_Channel_Graph_Block> xIter (m_xBlockList);

			Asura_Vector_2 xTopLeft (xOrigin.x, xOrigin.y);

			u_int uBlockIndex = 0;

			while (!xIter.Done ())
			{
				Asura_Channel_Graph_Block* pxBlock = xIter.GetCurrent ();

				if (pxBlock->GetCanSize () || pxBlock->GetCanMove ())
				{
					const Asura_Vector_2 xTopLeft (xOrigin.x + pxBlock->GetStartPositionScreenSpace () - 8.0f / ASURA_TEXT_OUTPUT_FONT_WIDTH, xOrigin.y - CHANNEL_MARGIN );

					if (Asura_Cursor::Inside (xTopLeft, pxBlock->GetWidthScreenSpace () + 16.0f / ASURA_TEXT_OUTPUT_FONT_WIDTH, 16.0f / ASURA_TEXT_OUTPUT_FONT_HEIGHT ) )
					{
						float fDummyY;

						// Detect if we're moving or resizing a block (based on how close the hittest was to the left or right of the block

						float fMouseX, fMouseY;

						Asura_Cursor::GetScreenCoords (fMouseX, fMouseY);

						if ( Asura_Maths::Abs( fMouseX - ( xTopLeft.x + 8.0f / ASURA_TEXT_OUTPUT_FONT_WIDTH ) ) < ( 8.0f / ASURA_TEXT_OUTPUT_FONT_WIDTH ) )
						{
							if (pxBlock->GetCanSize ())
							{
								// Check that this block is not anchored

								if ( !( m_bAnchorStartBlock && (uBlockIndex == 0) ) )
								{
									// We are resizing to the left
									m_iDraggingBlock = uBlockIndex;
									m_eDraggingMode = DRAG_MODE_RESIZE_LEFT;
									Asura_Cursor::GetScreenCoords (m_fInitialBlockDragMouseX, fDummyY);
									m_fInitialBlockDragMouseX = RoundMouseXToTicks( m_fInitialBlockDragMouseX, xOrigin );
									m_fInitialBlockWidth = pxBlock->GetWidthScreenSpace ();
									m_fInitialBlockXPos = pxBlock->GetStartPositionScreenSpace ();

									if (m_bRatioResizingEnabled && !Asura_UserInput::KeyPressed (ASURA_KEYID_LEFTCTRL))
									{
										CalculateBlockWidthRatiosForLeftDrag (uBlockIndex);
									}
									else
									{
										m_uNumRatioElements = 0;
									}

									m_iAttachedDragBlockIndex = -1;

									if (!Asura_UserInput::KeyPressed (ASURA_KEYID_LEFTSHIFT) || m_bIsDetachable)
									{
										if (m_iDraggingBlock > 0) // If not first block
										{
											Asura_Channel_Graph_Block* pxBlock2 = m_xBlockList.GetEntryByIndex (uBlockIndex - 1);

											if (DP_EQUALITY_TEST (pxBlock2->GetStartPositionScreenSpace () + pxBlock2->GetWidthScreenSpace (), m_fInitialBlockXPos, 2))
											{
												m_iAttachedDragBlockIndex = uBlockIndex - 1;
												m_fInitialAttachedBlockWidth = pxBlock2->GetWidthScreenSpace ();
											}
										}
									}
								}
							}
						}
						else if (((xTopLeft.x + pxBlock->GetWidthScreenSpace () + 8.0f / ASURA_TEXT_OUTPUT_FONT_WIDTH) - fMouseX) < ( 8.0f / ASURA_TEXT_OUTPUT_FONT_WIDTH ) )
						{
							if (pxBlock->GetCanSize ())
							{
								// Check that this block is not anchored

								if ( !( m_bAnchorEndBlock && (uBlockIndex == static_cast<u_int>(m_xBlockList.GetNumEntries() - 1 ) ) ) )
								{
									// We are resizing to the right
									m_iDraggingBlock = uBlockIndex;
									m_eDraggingMode = DRAG_MODE_RESIZE_RIGHT;
									Asura_Cursor::GetScreenCoords (m_fInitialBlockDragMouseX, fDummyY);
									m_fInitialBlockDragMouseX = RoundMouseXToTicks( m_fInitialBlockDragMouseX, xOrigin );
									m_fInitialBlockWidth = pxBlock->GetWidthScreenSpace ();
									m_fInitialBlockXPos = pxBlock->GetStartPositionScreenSpace ();

									if (m_bRatioResizingEnabled && !Asura_UserInput::KeyPressed (ASURA_KEYID_LEFTCTRL))
									{
										CalculateBlockWidthRatiosForRightDrag (uBlockIndex);
									}
									else
									{
										m_uNumRatioElements = 0;
									}

									m_iAttachedDragBlockIndex = -1;

									if (!Asura_UserInput::KeyPressed (ASURA_KEYID_LEFTSHIFT) || m_bIsDetachable)
									{
										if (m_iDraggingBlock < (m_xBlockList.GetNumEntries () - 1)) // If not last block
										{
											if (DP_EQUALITY_TEST (m_xBlockList.GetEntryByIndex (uBlockIndex + 1)->GetStartPositionScreenSpace (), m_fInitialBlockXPos + m_fInitialBlockWidth, 2))
											{
												m_iAttachedDragBlockIndex = uBlockIndex + 1;
												m_fInitialAttachedBlockWidth = m_xBlockList.GetEntryByIndex (uBlockIndex + 1)->GetWidthScreenSpace ();
											}
										}
									}
								}
							}
						}
						else
						{
							if (pxBlock->GetCanMove ())
							{
								// Check that this block is not anchored

								if (!((m_bAnchorStartBlock && (uBlockIndex == 0)) || (m_bAnchorEndBlock && (uBlockIndex == (u_int) (m_xBlockList.GetNumEntries () - 1)))))
								{
									// Detect if we're touching another block

									bool bTouchingAnotherBlock = false;

									Asura_Channel_Graph_Block* pxPrevBlock = (uBlockIndex == 0) ? NULL : m_xBlockList.GetEntryByIndex (uBlockIndex - 1);
									Asura_Channel_Graph_Block* pxNextBlock = ((int) uBlockIndex == (m_xBlockList.GetNumEntries () - 1)) ? NULL : m_xBlockList.GetEntryByIndex (uBlockIndex + 1);

									if (pxPrevBlock)
									{
										if (DP_EQUALITY_TEST (pxPrevBlock->GetStartPositionScreenSpace () + pxPrevBlock->GetWidthScreenSpace (), pxBlock->GetStartPositionScreenSpace (), 2))
										{
											bTouchingAnotherBlock = true;
										}
									}

									if (pxNextBlock)
									{
										if (DP_EQUALITY_TEST (pxNextBlock->GetStartPositionScreenSpace (), pxBlock->GetStartPositionScreenSpace () + pxBlock->GetWidthScreenSpace (), 2))
										{
											bTouchingAnotherBlock = true;
										}
									}

									if (m_bIsDetachable || !bTouchingAnotherBlock)
									{
										// We are moving the block
										m_iDraggingBlock = uBlockIndex;
										m_eDraggingMode = DRAG_MODE_MOVE;
										Asura_Cursor::GetScreenCoords (m_fInitialBlockDragMouseX, fDummyY);
										m_fInitialBlockDragMouseX = RoundMouseXToTicks( m_fInitialBlockDragMouseX, xOrigin );
										m_fInitialBlockXPos = pxBlock->GetStartPositionScreenSpace ();
										//CalculateBlockWidthRatios (uBlockIndex);
										m_iAttachedDragBlockIndex = -1;
									}
								}
							}
						}
					}
				}

				uBlockIndex ++;

				xIter.Next ();
			}
		}
	}
	else
	{
		if (m_iDraggedNode != -1)
		{
			if (Asura_UserInput::KeyPressed (ASURA_KEYID_MOUSEBUTTON1))
			{
				// Node is still being dragged, so update position based on cursor

				Asura_Assert (m_axControlPoints [m_iDraggedNode].bValid, "Control point not valid - shouldn't get here!");

				float fMouseX, fMouseY;

				Asura_Cursor::GetScreenCoords (fMouseX, fMouseY);

				float fNewX = ConvertScreenXToChannel (fMouseX - xOrigin.x);
				float fNewY = ConvertScreenYToChannel (fMouseY - xOrigin.y);

				if ((fNewX >= 0.0f) && (fNewX <= 1.0f) && (fNewY >= 0.0f) && (fNewY <= 1.0f))
				{
					if (!m_axControlPoints [m_iDraggedNode].bIsXAnchored)
					{
						// Also check that the point isn't being dragged past a neighbouring point

						if (m_iDraggedNode > 0)
						{
							if (fNewX < m_axControlPoints [m_iDraggedNode - 1].fX)
							{
								fNewX = m_axControlPoints [m_iDraggedNode - 1].fX;
							}
						}

						if (m_iDraggedNode < 2)
						{
							if (fNewX > m_axControlPoints [m_iDraggedNode + 1].fX)
							{
								fNewX = m_axControlPoints [m_iDraggedNode + 1].fX;
							}
						}

						// Check that the point isn't being dragged out of a block

						Asura_Channel_Graph_Block* pxBlockWithControlPointInside = CheckControlPointIsWithinBlock (fNewX);

						if (pxBlockWithControlPointInside == NULL)
						{
							// Control point has been dragged out of a block, so we need to snap to the side of the block that it's currently in

							const Asura_Channel_Graph_Block* pxContainerBlock = m_axControlPoints [m_iDraggedNode].pxContainerBlock;

							if (fNewX < ConvertScreenXToChannel (pxContainerBlock->GetStartPositionScreenSpace ()))
							{
								fNewX = pxContainerBlock->GetStartPositionScreenSpace ();
							}
							else if (fNewX > ConvertScreenXToChannel (pxContainerBlock->GetEndPositionScreenSpace ()))
							{
								fNewX = pxContainerBlock->GetEndPositionScreenSpace ();
							}
							else
							{
								Asura_Assert (FALSE, "Control point IS within the block - shouldn't be able to get here");
							}

							fNewX = ConvertScreenXToChannel (fNewX);
						}
						else
						{
							m_axControlPoints [m_iDraggedNode].pxContainerBlock = pxBlockWithControlPointInside;
						}

						m_axControlPoints [m_iDraggedNode].fX = fNewX;

						UpdateControlPointPositions ();
					}

					if (!m_axControlPoints [m_iDraggedNode].bIsYAnchored)
					{
						m_axControlPoints [m_iDraggedNode].fY = fNewY;

						UpdateControlPointPositions ();
					}
				}
			}
			else
			{
				m_iDraggedNode = -1;
				if( m_pxParent->m_pbModified )
				{
					*m_pxParent->m_pbModified = true;
				}
			}
		}

		if (m_iDraggingBlock != -1)
		{
			if (Asura_UserInput::KeyPressed (ASURA_KEYID_MOUSEBUTTON1))
			{
				// Block is still being dragged, so update position based on cursor

				float fMouseX, fMouseY;

				Asura_Cursor::GetScreenCoords (fMouseX, fMouseY);

				//Snap to our snap interval if appropriate
				fMouseX = RoundMouseXToTicks( fMouseX, xOrigin );

				float fDeltaMouseX = fMouseX - m_fInitialBlockDragMouseX;
				

				Asura_Channel_Graph_Block* pxBlock = m_xBlockList.GetEntryByIndex (m_iDraggingBlock);

				switch (m_eDraggingMode)
				{
					case DRAG_MODE_MOVE:
					{
						float fNewPosition = m_fInitialBlockXPos + fDeltaMouseX;

						float fMinX = 0.0f;
						float fMaxX = m_fPhysicalWidth - pxBlock->GetWidthScreenSpace ();

						if (m_iDraggingBlock != 0) // Not first block
						{
							Asura_Channel_Graph_Block* pxPrevBlock = m_xBlockList.GetEntryByIndex (m_iDraggingBlock - 1);

							fMinX = pxPrevBlock->GetStartPositionScreenSpace () + pxPrevBlock->GetWidthScreenSpace ();
						}

						if (m_iDraggingBlock != (m_xBlockList.GetNumEntries () - 1)) // Not last block
						{
							fMaxX = m_xBlockList.GetEntryByIndex (m_iDraggingBlock + 1)->GetStartPositionScreenSpace () - pxBlock->GetWidthScreenSpace ();
						}

						if (fNewPosition < fMinX)
						{
							fNewPosition = fMinX;
						}

						if (fNewPosition > fMaxX)
						{
							fNewPosition = fMaxX;
						}

						pxBlock->SetStartPositionScreenSpace (fNewPosition);

						UpdateControlPointPositions ();

						break;
					}

					case DRAG_MODE_RESIZE_LEFT:
					{
						if (m_uNumRatioElements > 0)
						{
							//
							// Do group block resize using ratios
							//

							const Asura_Channel_Graph_Block* pxFarRightGroupedBlock = m_xBlockList.GetEntryByIndex (m_axWidthRatios [m_uNumRatioElements - 1].iBlockIndex);

							float fNewPosition = m_fInitialBlockXPos + fDeltaMouseX;
							float fMaxWidth = pxFarRightGroupedBlock->GetStartPositionScreenSpace () + pxFarRightGroupedBlock->GetWidthScreenSpace ();
							float fNewWidth = m_fTotalRatioWidth - fDeltaMouseX;
							float fSnapLeft = 0.0f;

							// Check for collisions with any blocks that may be to the left (ignoring an attached node that is also being resized)

							if (m_axWidthRatios [0].iBlockIndex != 0)
							{
								int iOneLeftOfFarGroupedBlockIndex = m_axWidthRatios [0].iBlockIndex - 1;

								if (iOneLeftOfFarGroupedBlockIndex == m_iAttachedDragBlockIndex)
								{
									//iOneLeftOfFarGroupedBlockIndex --;

									if (iOneLeftOfFarGroupedBlockIndex >= 0)
									{
										const Asura_Channel_Graph_Block* pxOneLeftOfFarLeftGroupedBlock = m_xBlockList.GetEntryByIndex (iOneLeftOfFarGroupedBlockIndex);

										fSnapLeft = pxOneLeftOfFarLeftGroupedBlock->GetStartPositionScreenSpace () + MIN_BLOCK_WIDTH;
										fMaxWidth -= fSnapLeft;
									}
								}
								else
								{
									if (iOneLeftOfFarGroupedBlockIndex >= 0)
									{
										const Asura_Channel_Graph_Block* pxOneLeftOfFarLeftGroupedBlock = m_xBlockList.GetEntryByIndex (iOneLeftOfFarGroupedBlockIndex);

										fSnapLeft = pxOneLeftOfFarLeftGroupedBlock->GetStartPositionScreenSpace () + pxOneLeftOfFarLeftGroupedBlock->GetWidthScreenSpace ();
										fMaxWidth -= fSnapLeft;
									}
								}
							}

							if (fNewWidth > fMaxWidth)
							{
								fNewPosition = fSnapLeft;
								fNewWidth = fMaxWidth;
							}

							if (fNewWidth > (1.0f * (float) m_uNumRatioElements))
							{
								float fOffset = fNewPosition;

								// Iterate through all the blocks in the group resizing based on ratio of new group size

								for (u_int n = 0; n < m_uNumRatioElements; n ++)
								{
									Asura_Channel_Graph_Block* pxThisBlock = m_xBlockList.GetEntryByIndex (m_axWidthRatios [n].iBlockIndex);

									const float fThisWidth = fNewWidth * m_axWidthRatios [n].fRatio;

									pxThisBlock->SetStartPositionScreenSpace (fOffset);
									pxThisBlock->SetWidthScreenSpace (fThisWidth);

									if ((n == 0) && (m_iAttachedDragBlockIndex != -1)) // If far left element, and there's an attached node
									{
										Asura_Channel_Graph_Block* pxBlock2 = m_xBlockList.GetEntryByIndex (m_iAttachedDragBlockIndex);

										const float fNewWidth = fNewPosition - pxBlock2->GetStartPositionScreenSpace ();

										pxBlock2->SetWidthScreenSpace (fNewWidth);
									}

									fOffset += fThisWidth;
								}
							}
						}
						else
						{
							float fNewPosition = m_fInitialBlockXPos + fDeltaMouseX;
							float fNewSize = m_fInitialBlockWidth - fDeltaMouseX;

							float fMinX = 0.0f;

							if (m_iDraggingBlock != 0) // Not first block
							{
								int iPrevBlockIndex = m_iDraggingBlock - 1;

								if (iPrevBlockIndex == m_iAttachedDragBlockIndex)
								{
									//iPrevBlockIndex --;

									if (iPrevBlockIndex >= 0)
									{
										const Asura_Channel_Graph_Block* pxPrevBlock = m_xBlockList.GetEntryByIndex (iPrevBlockIndex);

										fMinX = pxPrevBlock->GetStartPositionScreenSpace () + MIN_BLOCK_WIDTH;
									}
								}
								else
								{
									if (iPrevBlockIndex >= 0)
									{
										const Asura_Channel_Graph_Block* pxPrevBlock = m_xBlockList.GetEntryByIndex (iPrevBlockIndex);

										fMinX = pxPrevBlock->GetStartPositionScreenSpace () + pxPrevBlock->GetWidthScreenSpace ();
									}
								}
							}

							if (fNewPosition < fMinX)
							{
								fNewPosition = fMinX;
								fNewSize = (m_fInitialBlockXPos + m_fInitialBlockWidth) - fMinX;
							}

							if (fNewSize >= MIN_BLOCK_WIDTH)
							{
								pxBlock->SetStartPositionScreenSpace (fNewPosition);
								pxBlock->SetWidthScreenSpace (fNewSize);

								if (m_iAttachedDragBlockIndex != -1)
								{
									Asura_Channel_Graph_Block* pxBlock2 = m_xBlockList.GetEntryByIndex (m_iAttachedDragBlockIndex);

									pxBlock2->SetWidthScreenSpace (fNewPosition - pxBlock2->GetStartPositionScreenSpace ());
								}
							}
						}

						UpdateControlPointPositions ();

						break;
					}

					case DRAG_MODE_RESIZE_RIGHT:
					{
						if (m_uNumRatioElements > 0)
						{
							//
							// Do group block resize using ratios
							//

							float fMaxWidth = m_fPhysicalWidth - m_xBlockList.GetEntryByIndex (m_axWidthRatios [m_uNumRatioElements - 1].iBlockIndex)->GetStartPositionScreenSpace ();
							float fNewWidth = m_fTotalRatioWidth + fDeltaMouseX;

							// Check for collisions with any blocks that may be to the right (ignoring an attached node that is also being resized)

							if (m_iDraggingBlock != (m_xBlockList.GetNumEntries () - 1)) // Not last block
							{
								int iNextBlockIndex = m_iDraggingBlock + 1;

								const Asura_Channel_Graph_Block* pxFarLeftGroupBlock = m_xBlockList.GetEntryByIndex (m_axWidthRatios [m_uNumRatioElements - 1].iBlockIndex);
								const Asura_Channel_Graph_Block* pxNextBlock = m_xBlockList.GetEntryByIndex (iNextBlockIndex);

								if (iNextBlockIndex == m_iAttachedDragBlockIndex)
								{
									if (iNextBlockIndex <= (m_xBlockList.GetNumEntries () - 1))
									{
										fMaxWidth = ((pxNextBlock->GetStartPositionScreenSpace () + pxNextBlock->GetWidthScreenSpace ()) - MIN_BLOCK_WIDTH) - pxFarLeftGroupBlock->GetStartPositionScreenSpace ();
									}
								}
								else
								{
									if (iNextBlockIndex <= (m_xBlockList.GetNumEntries () - 1))
									{
										fMaxWidth = pxNextBlock->GetStartPositionScreenSpace () - pxFarLeftGroupBlock->GetStartPositionScreenSpace ();
									}
								}
							}

							if (fNewWidth > fMaxWidth)
							{
								fNewWidth = fMaxWidth;
							}

							if (fNewWidth > (1.0f * (float) m_uNumRatioElements))
							{
								float fOffset = m_axWidthRatios [m_uNumRatioElements - 1].fPos; // Position of far left block in group

								// Iterate through all the blocks in the group resizing based on ratio of new group size

								for (int n = (m_uNumRatioElements - 1); n >= 0; n --)
								{
									Asura_Channel_Graph_Block* pxThisBlock = m_xBlockList.GetEntryByIndex (m_axWidthRatios [n].iBlockIndex);

									const float fThisWidth = fNewWidth * m_axWidthRatios [n].fRatio;

									pxThisBlock->SetStartPositionScreenSpace (fOffset);
									pxThisBlock->SetWidthScreenSpace (fThisWidth);

									if ((n == 0) && (m_iAttachedDragBlockIndex != -1)) // If far right element, and there's an attached node
									{
										Asura_Channel_Graph_Block* pxBlock2 = m_xBlockList.GetEntryByIndex (m_iAttachedDragBlockIndex);

										const float fNewPos = pxThisBlock->GetStartPositionScreenSpace () + fThisWidth;
										const float fOldAttachBlockEndPos = pxBlock2->GetEndPositionScreenSpace ();

										pxBlock2->SetStartPositionScreenSpace (fNewPos);
										pxBlock2->SetWidthScreenSpace (fOldAttachBlockEndPos - fNewPos);
									}

									fOffset += fThisWidth;
								}
							}
						}
						else
						{
							float fMaxWidth = m_fPhysicalWidth - pxBlock->GetStartPositionScreenSpace ();
							float fNewWidth = m_fInitialBlockWidth + fDeltaMouseX;

							if (m_iDraggingBlock != (m_xBlockList.GetNumEntries () - 1)) // Not last block
							{
								int iNextBlockIndex = m_iDraggingBlock + 1;

								const Asura_Channel_Graph_Block* pxNextBlock = m_xBlockList.GetEntryByIndex (iNextBlockIndex);

								if (iNextBlockIndex == m_iAttachedDragBlockIndex)
								{
									//iNextBlockIndex ++;

									if (iNextBlockIndex <= (m_xBlockList.GetNumEntries () - 1))
									{
										fMaxWidth = ((pxNextBlock->GetStartPositionScreenSpace () + pxNextBlock->GetWidthScreenSpace ()) - MIN_BLOCK_WIDTH) - pxBlock->GetStartPositionScreenSpace ();
									}
								}
								else
								{
									if (iNextBlockIndex <= (m_xBlockList.GetNumEntries () - 1))
									{
										fMaxWidth = pxNextBlock->GetStartPositionScreenSpace () - pxBlock->GetStartPositionScreenSpace ();
									}
								}
							}

							if (fNewWidth > fMaxWidth)
							{
								fNewWidth = fMaxWidth;
							}

							if (fNewWidth >= MIN_BLOCK_WIDTH)
							{
								pxBlock->SetWidthScreenSpace (fNewWidth);

								if (m_iAttachedDragBlockIndex != -1)
								{
									Asura_Channel_Graph_Block* pxBlock2 = m_xBlockList.GetEntryByIndex (m_iAttachedDragBlockIndex);

									pxBlock2->SetStartPositionScreenSpace (pxBlock->GetStartPositionScreenSpace () + fNewWidth);
									pxBlock2->SetWidthScreenSpace (m_fInitialAttachedBlockWidth + (m_fInitialBlockWidth - fNewWidth));
								}
							}
						}

						UpdateControlPointPositions ();

						break;
					}
				}
			}
			else
			{
				m_iDraggingBlock = -1;

				if( m_pxParent->m_pbModified )
				{
					*m_pxParent->m_pbModified = true;
				}
			}
		}
	}
}


////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// Checks all the control points to make sure that they're within a block.  If not, then move it so that it is!

void Asura_Channel_Graph_Channel::UpdateControlPointPositions ()
{
	for (u_int n = 0; n < ASURA_CHANNEL_GRAPH_MAX_CONTROL_POINTS; n ++)
	{
		if (m_axControlPoints [n].bValid)
		{
			float fX = m_axControlPoints [n].fX;
			float fY = m_axControlPoints [n].fY;

			Asura_Channel_Graph_Block* pxBlock = CheckControlPointIsWithinBlock (fX);

			float fBlockStartPosition = 0;
			float fBlockEndPosition	= 0;

			if (pxBlock)
			{
				m_axControlPoints [n].pxContainerBlock = pxBlock;

				fBlockStartPosition = ConvertScreenXToChannel (pxBlock->GetStartPositionScreenSpace ());
				fBlockEndPosition	= ConvertScreenXToChannel (pxBlock->GetEndPositionScreenSpace ());				
			}
			else
			{
				pxBlock = m_axControlPoints [n].pxContainerBlock;

				if( !pxBlock )
				{
					Asura_Assert( false, "Failed to find block for control point" );
					return;
				}
				fBlockStartPosition = ConvertScreenXToChannel (pxBlock->GetStartPositionScreenSpace ());
				fBlockEndPosition	= ConvertScreenXToChannel (pxBlock->GetEndPositionScreenSpace ());

				if (fX < (fBlockStartPosition + 0.0001f))
				{
					fX = fBlockStartPosition;
				}
				else if (fX > (fBlockEndPosition - 0.0001f))
				{
					fX = fBlockEndPosition;
				}
				else
				{
					Asura_Assert (FALSE, "Control point IS within the block - shouldn't be able to get here");
				}

				m_axControlPoints [n].fX = fX;
			}

			// Snap to block edge if nearby

			if (Asura_Maths::Abs (fBlockStartPosition - fX) < CONTROL_POINT_SNAP_X_TOLERANCE)
			{
				m_axControlPoints [n].fX = fBlockStartPosition;
			}
			else if (Asura_Maths::Abs (fBlockEndPosition - fX) < CONTROL_POINT_SNAP_X_TOLERANCE)
			{
				m_axControlPoints [n].fX = fBlockEndPosition;
			}

			if (fY < CONTROL_POINT_SNAP_Y_TOLERANCE)
			{
				m_axControlPoints [n].fY = 0.0f;
			}

			if (fY > (1.0f - CONTROL_POINT_SNAP_Y_TOLERANCE))
			{
				m_axControlPoints [n].fY = 1.0f;
			}
		}
	}
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void Asura_Channel_Graph_Channel::UpdateBlockAnchorage ()
{
	if (m_bAnchorStartBlock && (m_xBlockList.GetNumEntries () > 0))
	{
		m_axControlPoints [0].fX = 0.0f;
	}

	if (m_bAnchorEndBlock && (m_xBlockList.GetNumEntries () > 0))
	{
		Asura_Channel_Graph_Block* pxBlock = m_xBlockList.GetTailObject ();
		
		pxBlock->SetWidthScreenSpace (m_fPhysicalWidth - pxBlock->GetStartPositionScreenSpace ());
	}
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

Asura_Channel_Graph_Block* Asura_Channel_Graph_Channel::CheckControlPointIsWithinBlock (const float fX)
{
	Asura_List_It <Asura_Channel_Graph_Block> xIter (m_xBlockList);

	const float fPointX = ConvertChannelXToScreen (fX);

	while (!xIter.Done ())
	{
		Asura_Channel_Graph_Block* pxBlock = xIter.GetCurrent ();

		const float fBlockX = pxBlock->GetStartPositionScreenSpace ();
		const float fBlockWidth = pxBlock->GetWidthScreenSpace ();

		if ((fPointX >= fBlockX) && (fPointX <= (fBlockX + fBlockWidth)))
		{
			return pxBlock;
		}

		xIter.Next ();
	}

	return NULL;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void Asura_Channel_Graph_Dialog::Render (const Asura_Vector_2& xOrigin)
{
	if (!IsHidden ())
	{
		Asura_List_It <Asura_Channel_Graph_Channel> xIter (m_xChannelList);

		const float fX = (xOrigin.x + m_xOffset.x);
		float fY = (xOrigin.y + m_xOffset.y);

		const float fInitialY = fY;

		DWORD iCurrentBackgroundColour = CHANNEL_COLOUR_1;

		const float fWidth = m_fWidth;

		float fTotalY = fInitialY + static_cast<float>( m_xChannelList.GetNumEntries() ) * CHANNEL_HEIGHT;

		const float fLeft = fX * ASURA_TEXT_OUTPUT_FONT_WIDTH;
		const float fTop = fInitialY * ASURA_TEXT_OUTPUT_FONT_HEIGHT;
		const float fRight = (fX + fWidth) * ASURA_TEXT_OUTPUT_FONT_WIDTH;
		const float fBottom = fTotalY * ASURA_TEXT_OUTPUT_FONT_HEIGHT;

		// Render the border lines (not the bottom one, as this got drawn as part of the last channel)

		Asura_Render::Render2DLine (fLeft, fTop, fRight, fTop, 0xff000000);			// Top border
		Asura_Render::Render2DLine (fLeft, fTop, fLeft, fBottom, 0xff000000);		// Left border
		Asura_Render::Render2DLine (fRight, fTop, fRight, fBottom, 0xff000000);		// Right border

		while (!xIter.Done ())
		{
			Asura_Channel_Graph_Channel* pxChannel = xIter.GetCurrent ();

			pxChannel->Render (fX, fY, m_fWidth, iCurrentBackgroundColour);

			fY += CHANNEL_HEIGHT;

			iCurrentBackgroundColour = (iCurrentBackgroundColour == CHANNEL_COLOUR_1) ? CHANNEL_COLOUR_2 : CHANNEL_COLOUR_1;

			xIter.Next ();
		}	

		//Render the snap intervals if present
		if( m_fSnapInterval > 0.0f )
		{
			const float fChannelMargin = CHANNEL_MARGIN * ASURA_TEXT_OUTPUT_FONT_WIDTH;
			//Box
			Asura_Render::Overlay.Colour			= CHANNEL_DIALOG_TICK_BACKGROUND;
			Asura_Render::Overlay.TextureIndex		= ASURA_TEXTURE_NONE;
			Asura_Render::Overlay.TranslucencyMode	= ASURA_TRANSLUCENCY_NORMAL;
			Asura_Render::Overlay.SetRectXY( fLeft + fChannelMargin, fBottom, ( fRight - fLeft ) - fChannelMargin * 2.0f, CHANNEL_DIALOG_TICK_HEIGHT );
			Asura_Render::Overlay.Render();
			Asura_Render::FlushDisplay();

			Asura_Render::Render2DLine (fRight - fChannelMargin, fBottom, fRight - fChannelMargin, fBottom + CHANNEL_DIALOG_TICK_HEIGHT, Asura_OverlayBrightness );
			Asura_Render::Render2DLine (fLeft + fChannelMargin, fBottom, fLeft + fChannelMargin, fBottom + CHANNEL_DIALOG_TICK_HEIGHT, Asura_OverlayBrightness );
			Asura_Render::Render2DLine (fLeft, fBottom , fRight, fBottom, Asura_OverlayBrightness );

			int iTick = 0;
			float fLastTextPos = -100.0f;
			char szText[8];
			while( true )
			{
				const float fNormalisedX = ( static_cast<float>( iTick ) * m_fSnapInterval ) / ( m_fDomainMax - m_fDomainMin );
				if( fNormalisedX >= 1.0f )
				{
					break;
				}

				float fRenderPosX = fX * ASURA_TEXT_OUTPUT_FONT_WIDTH + fChannelMargin + ( m_fWidth * ASURA_TEXT_OUTPUT_FONT_WIDTH - fChannelMargin * 2.0f ) * fNormalisedX;
				if( fRenderPosX != 0.0f )
				{
					if( fRenderPosX - fLastTextPos > ( ASURA_TEXT_OUTPUT_FONT_WIDTH * 2.0f ) )
					{
						Asura_PrintfToBfr( szText, "%d", 8, true, iTick );
						Asura_Text_Output::Print ( fRenderPosX / ASURA_TEXT_OUTPUT_FONT_WIDTH - Asura_Text_Output::WidthOfString( szText, ASURA_TEXT_FORMAT_SMALLFONT ) * 0.5f, ( fBottom + CHANNEL_DIALOG_TICK_HEIGHT + 1.0f ) / ASURA_TEXT_OUTPUT_FONT_HEIGHT, CHANNEL_DIALOG_TICK_BACKGROUND | Asura_OverlayBrightness, ASURA_TEXT_FORMAT_SMALLFONT, szText );
						fLastTextPos = fRenderPosX;
					}

					Asura_Render::Render2DLine (fRenderPosX , fBottom, fRenderPosX, fBottom + CHANNEL_DIALOG_TICK_HEIGHT, Asura_OverlayBrightness );
				}
				++iTick;
			} 
		}
		else
		{
			// Render the time bar
			const float fChannelMargin = CHANNEL_MARGIN * ASURA_TEXT_OUTPUT_FONT_WIDTH;
			//Box
			Asura_Render::Overlay.Colour			= CHANNEL_DIALOG_TICK_BACKGROUND;
			Asura_Render::Overlay.TextureIndex		= ASURA_TEXTURE_NONE;
			Asura_Render::Overlay.TranslucencyMode	= ASURA_TRANSLUCENCY_NORMAL;
			Asura_Render::Overlay.SetRectXY( fLeft + fChannelMargin, fBottom, ( fRight - fLeft ) - fChannelMargin * 2.0f, CHANNEL_DIALOG_TICK_HEIGHT );
			Asura_Render::Overlay.Render();
			Asura_Render::FlushDisplay();

			Asura_Render::Render2DLine (fRight - fChannelMargin, fBottom, fRight - fChannelMargin, fBottom + CHANNEL_DIALOG_TICK_HEIGHT, Asura_OverlayBrightness );
			Asura_Render::Render2DLine (fLeft + fChannelMargin, fBottom, fLeft + fChannelMargin, fBottom + CHANNEL_DIALOG_TICK_HEIGHT, Asura_OverlayBrightness );
			Asura_Render::Render2DLine (fLeft, fBottom , fRight, fBottom, Asura_OverlayBrightness );

			// pick a decent tick seperation
			// This code presumes the domain is a time, which is true for the time being
			int iInterval = 1;
			int iMajorTick = 5;
			
			while( ( m_fDomainMax - m_fDomainMin ) / static_cast<float>( iInterval ) > m_fWidth * 0.33f )
			{
				switch( iInterval )
				{
					case 1:		iInterval = 5;		iMajorTick = 10;	break;
					case 5:		iInterval = 10;		iMajorTick = 30;	break;
					case 15:	iInterval = 20;		iMajorTick = 60;	break;
					case 20:	iInterval = 30;		iMajorTick = 120;	break;
					case 30:	iInterval = 60;		iMajorTick = 300;	break;
					case 60:	iInterval = 300;	iMajorTick = 600;	break;

					default:	Asura_Assert( false, "Failed to find appropriate tick scale" );
				}
			}

			const float fInterval = static_cast<float>( iInterval );
			float fRemainder = m_fDomainMin;
			while ( fRemainder > 0.0f )
			{
				fRemainder -= fInterval;
			}
			float fFirstTickTime = m_fDomainMin - fRemainder;

			int iTick = 0;
			float fLastTextPos = -100.0f;
			char szText[16];
			while( true )
			{
				const float fNormalisedX = ( static_cast<float>( iTick ) * fInterval ) / ( m_fDomainMax - m_fDomainMin );
				if( fNormalisedX >= 1.0f )
				{
					break;
				}

				float fRenderPosX = fX * ASURA_TEXT_OUTPUT_FONT_WIDTH + fChannelMargin + ( m_fWidth * ASURA_TEXT_OUTPUT_FONT_WIDTH - fChannelMargin * 2.0f ) * fNormalisedX;
				if( fRenderPosX != 0.0f )
				{
					if( ( iTick % iMajorTick ) != 0 )
					{
						const float fTickTime = fFirstTickTime + fInterval * static_cast<float>( iTick );

						int iMinutes = static_cast<int>(fTickTime / 60.0f);
						int iSeconds = static_cast<int>(fTickTime) % 60;
						Asura_PrintfToBfr(szText,"%02d:%02d", 16, true, iMinutes, iSeconds);

						if(	fRenderPosX - fLastTextPos > ( ASURA_TEXT_OUTPUT_FONT_WIDTH * 5.0f ) )
						{
							Asura_Text_Output::Print ( fRenderPosX / ASURA_TEXT_OUTPUT_FONT_WIDTH - Asura_Text_Output::WidthOfString( szText, ASURA_TEXT_FORMAT_SMALLFONT ) * 0.5f, ( fBottom + CHANNEL_DIALOG_TICK_HEIGHT + 1.0f ) / ASURA_TEXT_OUTPUT_FONT_HEIGHT, CHANNEL_DIALOG_TICK_BACKGROUND | Asura_OverlayBrightness, ASURA_TEXT_FORMAT_SMALLFONT, szText );
							fLastTextPos = fRenderPosX;
						}
					}

					Asura_Render::Render2DLine (fRenderPosX , fBottom, fRenderPosX, fBottom + CHANNEL_DIALOG_TICK_HEIGHT, Asura_OverlayBrightness );
				}
				++iTick;
			} 
		}


		// Render the playback marker

		if( m_bPlaybackMarkerVisible )
		{
			const float fFraction = ( m_fPlaybackMarker - m_fDomainMin ) / ( m_fDomainMax - m_fDomainMin );
			const float fRenderPosX = fLeft + (fRight - fLeft) * fFraction;

			Asura_Render::Render2DLine (fRenderPosX, fTop, fRenderPosX, fBottom, PLAYBACK_LINE_COLOUR);
		}
	}
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void Asura_Channel_Graph_Dialog::ShowPlaybackMarker ()
{
	m_bPlaybackMarkerVisible = true;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void Asura_Channel_Graph_Dialog::HidePlaybackMarker ()
{
	m_bPlaybackMarkerVisible = false;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void Asura_Channel_Graph_Dialog::SetPlaybackMarker	(const float fMarkerPos)
{
	Asura_Assert( ( fMarkerPos >= m_fDomainMin) && ( fMarkerPos <= m_fDomainMax ), "Playback marker is out of the graph domain" );
	m_fPlaybackMarker = fMarkerPos;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void	Asura_Channel_Graph_Dialog::SetSnapInterval		(const float fSnapInterval)	
{ 
	m_fSnapInterval = fSnapInterval;	
	
	Asura_List_It <Asura_Channel_Graph_Channel> xIter( m_xChannelList );

	while (!xIter.Done ())
	{
		Asura_Channel_Graph_Channel* pxChannel = xIter.GetCurrent ();

		pxChannel->SnapAllBlocksToTicks();

		xIter.Next ();
	}
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void Asura_Channel_Graph_Channel::Render (const float fX, const float fY, const float fWidth, const DWORD iBackgroundColour)
{
	const float fWidth2 = fWidth;
	const float fHeight = CHANNEL_HEIGHT;

	m_xTopLeft.x = fX;
	m_xTopLeft.y = fY;

	m_fPhysicalWidth = fWidth - (CHANNEL_MARGIN * 2.0f);
	m_fPhysicalHeight = CHANNEL_HEIGHT - (CHANNEL_MARGIN * 2.0f);

	// Render the channel's background

	Asura_Render::Overlay.Colour = iBackgroundColour;
	Asura_Render::Overlay.TextureIndex = ASURA_TEXTURE_NONE;
	Asura_Render::Overlay.TranslucencyMode = ASURA_TRANSLUCENCY_NORMAL;
	Asura_Render::Overlay.SetRectXY (fX * ASURA_TEXT_OUTPUT_FONT_WIDTH + 1.0f, fY * ASURA_TEXT_OUTPUT_FONT_HEIGHT, fWidth2 * ASURA_TEXT_OUTPUT_FONT_WIDTH, fHeight * ASURA_TEXT_OUTPUT_FONT_HEIGHT);
	Asura_Render::Overlay.Render ();

	// Render each of the internal white blocks

	Asura_List_It <Asura_Channel_Graph_Block> xIter (m_xBlockList);

	const float fBlockHeight = CHANNEL_HEIGHT - (CHANNEL_MARGIN * 2.0f);

	while (!xIter.Done ())
	{
		Asura_Channel_Graph_Block* pxBlock = xIter.GetCurrent ();

		float fBlockX = fX + CHANNEL_MARGIN + pxBlock->GetStartPositionScreenSpace ();

		const float fBlockY = fY + CHANNEL_MARGIN;

		pxBlock->Render (fBlockX, fBlockY, fBlockHeight);

		//fBlockX += pxBlock->GetWidthScreenSpace ();

		xIter.Next ();
	}

	Asura_Render::FlushDisplay ();
	// Render the line graph lines and blobs

	bool bPointHighlighted = false;
	if ( m_iDraggingBlock == -1 )
	{
		for (u_int n = 0; n < ASURA_CHANNEL_GRAPH_MAX_CONTROL_POINTS; n ++)
		{
			if (m_axControlPoints [n].bValid)
			{
				const float fScreenX = ConvertChannelXToScreen (m_axControlPoints [n].fX);
				const float fScreenY = ConvertChannelYToScreen (m_axControlPoints [n].fY);

				if (n != 0)
				{
					const float fPrevScreenX = ConvertChannelXToScreen (m_axControlPoints [n - 1].fX);
					const float fPrevScreenY = ConvertChannelYToScreen (m_axControlPoints [n - 1].fY);

					const float fX1 = (fX + CHANNEL_MARGIN + fPrevScreenX) * ASURA_TEXT_OUTPUT_FONT_WIDTH;
					const float fY1 = (fY + CHANNEL_MARGIN + fPrevScreenY) * ASURA_TEXT_OUTPUT_FONT_HEIGHT;
					const float fX2 = (fX + CHANNEL_MARGIN + fScreenX) * ASURA_TEXT_OUTPUT_FONT_WIDTH;
					const float fY2 = (fY + CHANNEL_MARGIN + fScreenY) * ASURA_TEXT_OUTPUT_FONT_HEIGHT;

					Asura_Render::Render2DLine (fX1, fY1, fX2, fY2, CHANNEL_LINE_GRAPH_COLOUR);
				}

				Asura_Vector_2 xScreenPos (fX + fScreenX, fY + fScreenY);

				xScreenPos.x -= (CHANNEL_CONTROL_POINT_SIZE * 0.5f);
				xScreenPos.y -= (CHANNEL_CONTROL_POINT_SIZE * 0.5f);

				if (Asura_Cursor::Inside (xScreenPos, CHANNEL_CONTROL_POINT_SIZE, CHANNEL_CONTROL_POINT_SIZE))
				{
					bPointHighlighted = true;
					RenderPoint (fX + CHANNEL_MARGIN + fScreenX, fY + CHANNEL_MARGIN + fScreenY, true);
				}
				else
				{
					RenderPoint (fX + CHANNEL_MARGIN + fScreenX, fY + CHANNEL_MARGIN + fScreenY);
				}
			}
		}
	}

	// Flush the rendering

	Asura_Render::FlushDisplay ();

	// Check if we should highlight an edge to drag
	if( !bPointHighlighted && ( m_iDraggedNode == -1 ) )
	{
		xIter.Restart();

		u_int uBlockIndex = 0;

		while (!xIter.Done ())
		{
			Asura_Channel_Graph_Block* pxBlock = xIter.GetCurrent ();

			if (pxBlock->GetCanSize() )
			{
				const Asura_Vector_2 xTopLeft (fX + pxBlock->GetStartPositionScreenSpace() + CHANNEL_MARGIN - 8.0f / ASURA_TEXT_OUTPUT_FONT_WIDTH, fY);

				if ( Asura_Cursor::Inside (xTopLeft, 16.0f / ASURA_TEXT_OUTPUT_FONT_WIDTH, CHANNEL_HEIGHT ) )
				{
					if ( !( m_bAnchorStartBlock && (uBlockIndex == 0) ) )
					{
						// We can potentially drag the left of this block
						const float fIconRow	= 3.0f;
						const float fIconColumn	= 0.0f;

						Asura_Render::Overlay.TextureIndex = Tools_Dialog::GetIconTexture();
						Asura_Render::Overlay.TranslucencyMode = ASURA_TRANSLUCENCY_NORMAL;
						Asura_Render::Overlay.SetRectUV (0.0625f * fIconColumn, 0.0625f * fIconRow, 0.0625f, 0.0625f);
						Asura_Render::Overlay.SetRectXY ( ( fX + pxBlock->GetStartPositionScreenSpace() + CHANNEL_MARGIN ) * ASURA_TEXT_OUTPUT_FONT_WIDTH - 8.0f, fY * ASURA_TEXT_OUTPUT_FONT_HEIGHT, 16.0f, 16.0f);
						Asura_Render::Overlay.Colour = 0xFFFFFFFF;
						Asura_Render::Overlay.Render ();

						Asura_Render::FlushDisplay();

						break;
					}
				}

				const Asura_Vector_2 xTopRight (fX + pxBlock->GetEndPositionScreenSpace() + CHANNEL_MARGIN - 8.0f / ASURA_TEXT_OUTPUT_FONT_WIDTH, fY);

				if ( Asura_Cursor::Inside (xTopRight, 16.0f / ASURA_TEXT_OUTPUT_FONT_WIDTH, CHANNEL_HEIGHT ) )
				{
					if ( !(m_bAnchorEndBlock && (uBlockIndex == static_cast<u_int>( m_xBlockList.GetNumEntries() - 1 ) ) ) )
					{
						// We can potentially drag the right of this block
						const float fIconRow	= 3.0f;
						const float fIconColumn	= 0.0f;

						Asura_Render::Overlay.TextureIndex = Tools_Dialog::GetIconTexture();
						Asura_Render::Overlay.TranslucencyMode = ASURA_TRANSLUCENCY_NORMAL;
						Asura_Render::Overlay.SetRectUV (0.0625f * fIconColumn, 0.0625f * fIconRow, 0.0625f, 0.0625f);
						Asura_Render::Overlay.SetRectXY ( ( fX + pxBlock->GetEndPositionScreenSpace() + CHANNEL_MARGIN ) * ASURA_TEXT_OUTPUT_FONT_WIDTH - 8.0f, fY * ASURA_TEXT_OUTPUT_FONT_HEIGHT, 16.0f, 16.0f);
						Asura_Render::Overlay.Colour = 0xFFFFFFFF;
						Asura_Render::Overlay.Render ();

						Asura_Render::FlushDisplay();

						break;
					}
				}
			}
			++uBlockIndex;
			xIter.Next();
		}
	}
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void Asura_Channel_Graph_Block::Render (const float fBlockX, const float fBlockY, const float fBlockHeight)
{
	const float fBlockWidth = m_fWidth;

	// Render the block

	Asura_Render::Overlay.Colour = m_uColour | Asura_OverlayBrightness;
	Asura_Render::Overlay.TextureIndex = ASURA_TEXTURE_NONE;
	Asura_Render::Overlay.TranslucencyMode = ASURA_TRANSLUCENCY_NORMAL;
	Asura_Render::Overlay.SetRectXY (fBlockX * ASURA_TEXT_OUTPUT_FONT_WIDTH, fBlockY * ASURA_TEXT_OUTPUT_FONT_HEIGHT, fBlockWidth * ASURA_TEXT_OUTPUT_FONT_WIDTH, fBlockHeight * ASURA_TEXT_OUTPUT_FONT_HEIGHT);
	Asura_Render::Overlay.Render ();

	Asura_Render::FlushDisplay ();

	// Render the text

	const float fTextX = fBlockX + BLOCK_PADDING;
	const float fTextY = fBlockY + (fBlockHeight * 0.4f);

	//Resize the text if it goes offscreen
	char szText[128];
	Asura_Mem::Copy( szText, m_szName.GetString(), m_szName.GetSizeInBytes() );

	int iLastChar = m_szName.GetLength();
	while( ( BLOCK_PADDING + Asura_Text_Output::WidthOfString( szText, ASURA_TEXT_FORMAT_SMALLFONT ) ) > fBlockWidth )
	{
		--iLastChar;
		szText[iLastChar] = 0;
		szText[iLastChar - 1] = '.';
		szText[iLastChar - 2] = '.';

		if( iLastChar == 3 )
		{
			break;
		}
	}

	if( BLOCK_PADDING + Asura_Text_Output::WidthOfString( szText, ASURA_TEXT_FORMAT_SMALLFONT ) < fBlockWidth )
	{
		Asura_Text_Output::Print (fTextX, fTextY, BLOCK_TEXT_COLOUR, ASURA_TEXT_FORMAT_SMALLFONT, szText );
	}
	
	// Render a border around the block

	const float fLeft = fBlockX * ASURA_TEXT_OUTPUT_FONT_WIDTH;
	const float fTop = fBlockY * ASURA_TEXT_OUTPUT_FONT_HEIGHT;
	const float fRight = (fBlockX + fBlockWidth) * ASURA_TEXT_OUTPUT_FONT_WIDTH;
	const float fBottom = (fBlockY + fBlockHeight) * ASURA_TEXT_OUTPUT_FONT_HEIGHT;

	Asura_Render::Render2DLine (fLeft, fTop, fRight, fTop, 0xff000000);
	Asura_Render::Render2DLine (fRight, fTop, fRight, fBottom, 0xff000000);
	Asura_Render::Render2DLine (fLeft, fBottom, fRight, fBottom, 0xff000000);
	Asura_Render::Render2DLine (fLeft, fBottom, fLeft, fTop, 0xff000000);
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

Asura_Channel_Graph_Block* Asura_Channel_Graph_Channel::AddBlock (const char* szName, const float fPos, const float fWidth, const bool bCanSize /* = true */, const bool bCanMove /* = true */)
{
	float fXPos = fPos;
	float fWidth2 = fWidth;

	if (m_bAnchorStartBlock && (m_xBlockList.GetNumEntries () == 0))
	{
		fXPos = 0.0f;
	}

	Asura_Channel_Graph_Block* pxNewBlock = new Asura_Channel_Graph_Block (m_pxParent, this, szName, fXPos, fWidth2, bCanSize, bCanMove);

	m_xBlockList.AddToList (pxNewBlock, false);

	SnapAllBlocksToTicks();

	return pxNewBlock;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void Asura_Channel_Graph_Channel::RenderPoint (float fX, float fY, bool bHighlight) const
{
	static int iTextureIndex = -1;

	if (iTextureIndex == -1)
	{
		iTextureIndex = Asura_Texture_Management::AddTextureWithResetGroup ("\\tools\\graphnode.tga", ASURA_TEXTURE_LOADED_PERMANENT);
		Asura_Texture_Management::LoadTextures ();
	}

	const float fRadius = CHANNEL_CONTROL_POINT_SIZE * 0.5f;

	const float fLeft = (fX - fRadius) * ASURA_TEXT_OUTPUT_FONT_WIDTH;
	const float fTop = (fY - fRadius) * ASURA_TEXT_OUTPUT_FONT_HEIGHT;
	const float fWidth = (fRadius * 2.0f) * ASURA_TEXT_OUTPUT_FONT_WIDTH;
	const float fHeight = (fRadius * 2.0f) * ASURA_TEXT_OUTPUT_FONT_HEIGHT;

	Asura_Render::Overlay.TextureIndex = iTextureIndex;
	Asura_Render::Overlay.TranslucencyMode = ASURA_TRANSLUCENCY_NORMAL;
	Asura_Render::Overlay.SetRectUV (0.0f, 0.0f, 1.0f, 1.0f);
	Asura_Render::Overlay.SetRectXY (fLeft, fTop, fWidth, fHeight);
	if( bHighlight )
	{
		Asura_Render::Overlay.Colour = CHANNEL_CONTROL_POINT_COLOUR_FOCUS;
	}
	else
	{
		Asura_Render::Overlay.Colour = CHANNEL_CONTROL_POINT_COLOUR;
	}
	
	Asura_Render::Overlay.Render ();	
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

float Asura_Channel_Graph_Channel::ConvertChannelXToScreen (const float fX) const
{
	return fX * m_fPhysicalWidth;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

float Asura_Channel_Graph_Channel::ConvertChannelYToScreen (const float fY) const
{
	return (1.0f - fY) * m_fPhysicalHeight;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

float Asura_Channel_Graph_Channel::ConvertScreenXToChannel (const float fX) const
{
	return fX / m_fPhysicalWidth;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

float Asura_Channel_Graph_Channel::ConvertScreenYToChannel (const float fY) const
{
	return 1.0f - (fY / m_fPhysicalHeight);
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool Asura_Channel_Graph_Dialog::IsPointInDialog (const Asura_Vector_2& xOrigin, const Asura_Vector_2& xPos) const
{
	Asura_Vector_2 xRelativePos = xPos - xOrigin;

	if ((xRelativePos.x > m_xOffset.x) && (xRelativePos.x < (m_xOffset.x + m_fWidth)))
	{
		if ((xRelativePos.y > m_xOffset.y) && (xRelativePos.y < (m_xMaxCoord.y)))
		{
			return true;
		}
	}

	return false;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void Asura_Channel_Graph_Channel::CalculateBlockWidthRatiosForRightDrag (const u_int uBlockIndex)
{
	if (uBlockIndex == 0)
	{
		m_uNumRatioElements = 0;
		return;
	}

	// Work backwards through the blocklist storing the width ratios of any blocks that are 'touching' uBlockIndex

	m_uNumRatioElements = 0;

	m_fTotalRatioWidth = 0.0f;

	for (int n = uBlockIndex; n >= 0; n --)
	{
		Asura_Channel_Graph_Block* pxThisBlock = m_xBlockList.GetEntryByIndex (n);

		bool bDoBreak = false;

		if (n != 0)
		{
			Asura_Channel_Graph_Block* pxPrevBlock = m_xBlockList.GetEntryByIndex (n - 1);

			const float fPrevBlockEnd = pxPrevBlock->GetStartPositionScreenSpace () + pxPrevBlock->GetWidthScreenSpace ();

			if (Asura_Maths::Abs (fPrevBlockEnd - pxThisBlock->GetStartPositionScreenSpace ()) > 0.0001f) // Inequality test with float rounding tolerance check
			{
				// If we get here, then we've found a non-attached block, so end here
				bDoBreak = true;
			}
		}

		m_axWidthRatios [m_uNumRatioElements].iBlockIndex = n;
		m_axWidthRatios [m_uNumRatioElements].fPos = pxThisBlock->GetStartPositionScreenSpace ();
		m_axWidthRatios [m_uNumRatioElements].fWidth = pxThisBlock->GetWidthScreenSpace ();
		m_uNumRatioElements ++;

		m_fTotalRatioWidth += pxThisBlock->GetWidthScreenSpace ();

		if (bDoBreak)
		{
			break;
		}
	}

	if (m_uNumRatioElements == 1)
	{
		// No point in doing the ratio thing if we've only one element
		m_uNumRatioElements = 0;
		return;
	}

	// Now reiterate through the ratio array calculating the ratios

	for (u_int n = 0; n < m_uNumRatioElements; n ++)
	{
		m_axWidthRatios [n].fRatio = m_axWidthRatios [n].fWidth / m_fTotalRatioWidth;
	}
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void Asura_Channel_Graph_Channel::CalculateBlockWidthRatiosForLeftDrag (const u_int uBlockIndex)
{
	if ((int) uBlockIndex == (m_xBlockList.GetNumEntries () - 1))
	{
		m_uNumRatioElements = 0;
		return;
	}

	// Work backwards through the blocklist storing the width ratios of any blocks that are 'touching' uBlockIndex

	m_uNumRatioElements = 0;

	m_fTotalRatioWidth = 0.0f;

	const u_int uNumBlocks = m_xBlockList.GetNumEntries ();

	for (u_int n = uBlockIndex; n < uNumBlocks; n ++)
	{
		Asura_Channel_Graph_Block* pxThisBlock = m_xBlockList.GetEntryByIndex (n);

		bool bDoBreak = false;

		if (n != (uNumBlocks - 1))
		{
			Asura_Channel_Graph_Block* pxNextBlock = m_xBlockList.GetEntryByIndex (n + 1);

			const float fNextBlockStart = pxNextBlock->GetStartPositionScreenSpace ();

			if (Asura_Maths::Abs (fNextBlockStart - (pxThisBlock->GetStartPositionScreenSpace () + pxThisBlock->GetWidthScreenSpace ())) > 0.0001f) // Inequality test with float rounding tolerance check
			{
				// If we get here, then we've found a non-attached block, so end here
				bDoBreak = true;
			}
		}

		m_axWidthRatios [m_uNumRatioElements].iBlockIndex = n;
		m_axWidthRatios [m_uNumRatioElements].fPos = pxThisBlock->GetStartPositionScreenSpace ();
		m_axWidthRatios [m_uNumRatioElements].fWidth = pxThisBlock->GetWidthScreenSpace ();
		m_uNumRatioElements ++;

		m_fTotalRatioWidth += pxThisBlock->GetWidthScreenSpace ();

		if (bDoBreak)
		{
			break;
		}
	}

	if (m_uNumRatioElements == 1)
	{
		// No point in doing the ratio thing if we've only one element
		m_uNumRatioElements = 0;
		return;
	}

	// Now reiterate through the ratio array calculating the ratios

	for (u_int n = 0; n < m_uNumRatioElements; n ++)
	{
		m_axWidthRatios [n].fRatio = m_axWidthRatios [n].fWidth / m_fTotalRatioWidth;
	}
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

Asura_Channel_Graph_Block* Asura_Channel_Graph_Channel::GetPrevBlock (const Asura_Channel_Graph_Block* pxBlock)
{
	// Find the object

	Asura_List_It <Asura_Channel_Graph_Block> xIter (m_xBlockList);

	int iIndex = 0;

	Asura_Channel_Graph_Block* pxLastBlock;

	while (!xIter.Done ())
	{
		if (iIndex != 0) // Not first entry
		{
			if (xIter.GetCurrent () == pxBlock)
			{
				return pxLastBlock;
			}
		}

		pxLastBlock = xIter.GetCurrent ();

		iIndex ++;
		xIter.Next ();
	}

	return NULL;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

Asura_Channel_Graph_Block* Asura_Channel_Graph_Channel::GetNextBlock (const Asura_Channel_Graph_Block* pxBlock)
{
	// Find the object

	Asura_List_It <Asura_Channel_Graph_Block> xIter (m_xBlockList);

	int iIndex = 0;

	while (!xIter.Done ())
	{
		if (iIndex != (m_xBlockList.GetNumEntries () - 1)) // Not last entry
		{
			if (xIter.GetCurrent () == pxBlock)
			{
				xIter.Next ();
				return xIter.GetCurrent ();
			}
		}

		iIndex ++;
		xIter.Next ();
	}

	return NULL;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

float Asura_Channel_Graph_Block::GetStartPosition ()
{
	// Convert unit from screen-space to domain-space

	float fDomainMin, fDomainMax;

	m_pxParentDialog->GetGraphDomain (&fDomainMin, &fDomainMax);

	const float fNormalisedX = GetStartPositionScreenSpace () / m_pxParentChannel->m_fPhysicalWidth;

	return ((fDomainMax - fDomainMin) * fNormalisedX) + fDomainMin;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

float Asura_Channel_Graph_Block::GetEndPosition ()
{
	// Convert unit from screen-space to domain-space

	float fDomainMin, fDomainMax;

	m_pxParentDialog->GetGraphDomain (&fDomainMin, &fDomainMax);

	const float fNormalisedX = GetStartPositionScreenSpace () / m_pxParentChannel->m_fPhysicalWidth;
	const float fNormalisedWidth = GetWidthScreenSpace () / m_pxParentChannel->m_fPhysicalWidth;
	const float fNormalisedEndX = fNormalisedX + fNormalisedWidth;

	return ((fDomainMax - fDomainMin) * fNormalisedEndX) + fDomainMin;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void Asura_Channel_Graph_Block::SetStartPosition (float fStartPos, const bool bSupressChecks)
{
	float fDomainMin, fDomainMax;

	m_pxParentDialog->GetGraphDomain (&fDomainMin, &fDomainMax);

	// Check for an out of range value

	Asura_Assert ((fStartPos >= fDomainMin) && (fStartPos <= fDomainMax), "Value out of range");

	fStartPos = Asura_Maths::Max (fStartPos, fDomainMin);
	fStartPos = Asura_Maths::Min (fStartPos, fDomainMax);

	// Convert unit from domain-space to normalised-space

	const float fNormalisedStartPos = (fStartPos - fDomainMin) / (fDomainMax - fDomainMin);

	// Convert unit from normalised-space to screen-space

	m_pxParentChannel->m_fPhysicalWidth = m_pxParentDialog->m_fWidth - (CHANNEL_MARGIN * 2.0f);

	float fScreenSpaceStartPos = fNormalisedStartPos * m_pxParentChannel->m_fPhysicalWidth;

	if( !bSupressChecks )
	{
		// Check for out of range value relative to previous and next block
		Asura_Channel_Graph_Block* pxPrevBlock = m_pxParentChannel->GetPrevBlock (this);
		Asura_Channel_Graph_Block* pxNextBlock = m_pxParentChannel->GetNextBlock (this);

		if (pxPrevBlock)
		{
			if (fScreenSpaceStartPos < (pxPrevBlock->GetStartPositionScreenSpace () + pxPrevBlock->GetWidthScreenSpace ()))
			{
				Asura_Assert (FALSE, "Value out of range");
				fScreenSpaceStartPos = pxPrevBlock->GetStartPositionScreenSpace () + pxPrevBlock->GetWidthScreenSpace ();
			}
		}
		else
		{
			if (fScreenSpaceStartPos < 0.0f)
			{
				Asura_Assert (FALSE, "Value out of range");
				fScreenSpaceStartPos = 0.0f;
			}
		}

		if (pxNextBlock)
		{
			if ((fScreenSpaceStartPos + m_fWidth) > pxNextBlock->GetStartPositionScreenSpace ())
			{
				Asura_Assert (FALSE, "Value out of range");
				fScreenSpaceStartPos = pxNextBlock->GetStartPositionScreenSpace () - m_fWidth;
			}
		}
		else
		{
			if ((fScreenSpaceStartPos + m_fWidth) > m_pxParentChannel->m_fPhysicalWidth)
			{
				Asura_Assert (FALSE, "Value out of range");
				fScreenSpaceStartPos = m_pxParentChannel->m_fPhysicalWidth;
			}
		}
	}

	// Set the start pos
	SetStartPositionScreenSpace (fScreenSpaceStartPos);
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void Asura_Channel_Graph_Block::SetEndPosition (float fEndPos, const bool bSupressChecks)
{
	float fDomainMin, fDomainMax;

	m_pxParentDialog->GetGraphDomain (&fDomainMin, &fDomainMax);

	// Check for an out of range value

	Asura_Assert ((fEndPos >= fDomainMin) && (fEndPos <= fDomainMax), "Value out of range");
	Asura_Assert (fEndPos > GetStartPosition (), "Value out of range");

	fEndPos = Asura_Maths::Max (fEndPos, fDomainMin);
	fEndPos = Asura_Maths::Min (fEndPos, fDomainMax);

	// Convert unit from domain-space to normalised-space

	const float fNormalisedEndPos = (fEndPos - fDomainMin) / (fDomainMax - fDomainMin);

	// Convert unit from normalised-space to screen-space

	m_pxParentChannel->m_fPhysicalWidth = m_pxParentDialog->m_fWidth - (CHANNEL_MARGIN * 2.0f);

	float fScreenSpaceEndPos = fNormalisedEndPos * m_pxParentChannel->m_fPhysicalWidth;

	if( !bSupressChecks )
	{
		// Check for out of range value relative to previous and next block

		Asura_Channel_Graph_Block* pxPrevBlock = m_pxParentChannel->GetPrevBlock (this);
		Asura_Channel_Graph_Block* pxNextBlock = m_pxParentChannel->GetNextBlock (this);

		if (pxPrevBlock)
		{
			if (fScreenSpaceEndPos < (pxPrevBlock->GetStartPositionScreenSpace () + pxPrevBlock->GetWidthScreenSpace ()))
			{
				Asura_Assert (FALSE, "Value out of range");
				fScreenSpaceEndPos = pxPrevBlock->GetStartPositionScreenSpace () + pxPrevBlock->GetWidthScreenSpace ();
			}
		}
		else
		{
			if (fScreenSpaceEndPos < 0.0f)
			{
				Asura_Assert (FALSE, "Value out of range");
				fScreenSpaceEndPos = 0.0f;
			}
		}

		if (pxNextBlock)
		{
			if (fScreenSpaceEndPos > pxNextBlock->GetStartPositionScreenSpace ())
			{
				Asura_Assert (FALSE, "Value out of range");
				fScreenSpaceEndPos = pxNextBlock->GetStartPositionScreenSpace ();
			}
		}
		else
		{
			if (fScreenSpaceEndPos > m_pxParentChannel->m_fPhysicalWidth)
			{
				Asura_Assert (FALSE, "Value out of range");
				fScreenSpaceEndPos = m_pxParentChannel->m_fPhysicalWidth;
			}
		}
	}

	// Set the end position

	SetEndPositionScreenSpace (fScreenSpaceEndPos);

	// Set the new width

	SetWidthScreenSpace (fScreenSpaceEndPos - m_fPosition);
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void Asura_Channel_Graph_Block::SetWidth (float fWidth, const bool bSupressChecks)
{
	float fDomainMin, fDomainMax;

	m_pxParentDialog->GetGraphDomain (&fDomainMin, &fDomainMax);

	// Check for an out of range value

	Asura_Assert (fWidth <= (fDomainMax - fDomainMin), "Value out of range");

	fWidth = Asura_Maths::Min (fWidth, (fDomainMax - fDomainMin));

	// Convert unit from domain-space to normalised-space

	const float fNormalisedWidth = fWidth / (fDomainMax - fDomainMin);

	// Convert unit from normalised-space to screen-space

	m_pxParentChannel->m_fPhysicalWidth = m_pxParentDialog->m_fWidth - (CHANNEL_MARGIN * 2.0f);

	float fScreenSpaceWidth = fNormalisedWidth * m_pxParentChannel->m_fPhysicalWidth;

	// Check for out of range value relative to previous and next block

	if( !bSupressChecks )
	{
		Asura_Channel_Graph_Block* pxNextBlock = m_pxParentChannel->GetNextBlock (this);

		if (pxNextBlock)
		{
			if ((int) ((m_fPosition + fScreenSpaceWidth) * 1000.0f) > (int) (pxNextBlock->GetStartPositionScreenSpace () * 1000.0f))
			{
				Asura_Assert (FALSE, "Value out of range");

				fScreenSpaceWidth = pxNextBlock->GetStartPosition () - m_fPosition;
			}
		}
		else
		{
			if ((int) ((m_fPosition + fScreenSpaceWidth) * 1000.0f) > (int) (m_pxParentChannel->m_fPhysicalWidth * 1000.0f))
			{
				Asura_Assert (FALSE, "Value out of range");

				fScreenSpaceWidth = m_pxParentChannel->m_fPhysicalWidth - m_fPosition;
			}
		}
	}

	// Set the width

	SetWidthScreenSpace (fScreenSpaceWidth);
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void Asura_Channel_Graph_Block::SetColour (u_int uColour)
{
	m_uColour = uColour & 0x00FFFFFF;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

float Asura_Channel_Graph_Block::GetWidth ()
{
	// Convert width from screen-space to normalised space

	const float fNormalisedWidth = m_fWidth / m_pxParentChannel->m_fPhysicalWidth;

	// Convert width from normalised space to domain-space

	float fDomainMin, fDomainMax;

	m_pxParentDialog->GetGraphDomain (&fDomainMin, &fDomainMax);

	const float fWidthDomainSpace = fNormalisedWidth * (fDomainMax - fDomainMin);

	return fWidthDomainSpace;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void Asura_Channel_Graph_Channel::Remove ()
{
	m_pxParent->m_xChannelList.RemoveFromList (this);
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void Asura_Channel_Graph_Channel::SetControlPoint (const u_int uIndex, const float fX, const float fY, const bool bIsXAnchored /*= false*/, const bool bIsYAnchored /*= false*/)
{
	Asura_Assert (uIndex < ASURA_CHANNEL_GRAPH_MAX_CONTROL_POINTS, "Index is out of range");

	// Convert the x position from domain-space to normalised-space

	float fDomainMin, fDomainMax;

	m_pxParent->GetGraphDomain (&fDomainMin, &fDomainMax);

	float fNormalisedX = (fX - fDomainMin) / (fDomainMax - fDomainMin);

	// Validate that the values are within range

	if (fNormalisedX < 0.0f)
	{
		Asura_Assert (FALSE, "Out of range");
		fNormalisedX = 0.0f;
	}

	if (fNormalisedX > 1.0f)
	{
		Asura_Assert (FALSE, "Out of range");
		fNormalisedX = 1.0f;
	}

	// Set the values

	m_axControlPoints [uIndex].bValid = true;
	m_axControlPoints [uIndex].fX = fNormalisedX;
	m_axControlPoints [uIndex].fY = fY;
	m_axControlPoints [uIndex].bIsXAnchored = bIsXAnchored;
	m_axControlPoints [uIndex].bIsYAnchored = bIsYAnchored;

	// Validate that this control point is within a block

	m_axControlPoints [uIndex].pxContainerBlock = CheckControlPointIsWithinBlock (m_axControlPoints [uIndex].fX);

	Asura_Assert (m_axControlPoints [uIndex].pxContainerBlock != NULL, "Trying to add a control point to a channel position that doesn't contain a block");
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

float Asura_Channel_Graph_Channel::GetControlPointX ( const u_int uIndex ) const
{
	Asura_Assert (uIndex < ASURA_CHANNEL_GRAPH_MAX_CONTROL_POINTS, "Index is out of range");
	Asura_Assert (m_axControlPoints [uIndex].bValid, "Index isn't valid");

	float fDomainMin, fDomainMax;

	m_pxParent->GetGraphDomain (&fDomainMin, &fDomainMax);

	return ( fDomainMin + ( fDomainMax - fDomainMin) * m_axControlPoints [uIndex].fX );
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

float Asura_Channel_Graph_Channel::GetControlPointY ( const u_int uIndex ) const
{
	Asura_Assert (uIndex < ASURA_CHANNEL_GRAPH_MAX_CONTROL_POINTS, "Index is out of range");
	Asura_Assert (m_axControlPoints [uIndex].bValid, "Index isn't valid");

	return m_axControlPoints [uIndex].fY;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void Asura_Channel_Graph_Block::Remove ()
{
	m_pxParentChannel->m_xBlockList.RemoveFromList (this);
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#endif

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
