////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

//
// Asura_Channel_Graph.h
//
// Created by Dan Clarke - 18/08/2008
//
// Copyright (C) 2008 Rebellion.  All Rights Reserved.
//

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif

#ifndef __ASURA_CHANNEL_GRAPH_DIALOG_H__
#define __ASURA_CHANNEL_GRAPH_DIALOG_H__

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include "Asura_Dialog.h"
#include "Core/Asura_List.h"
#include "Rendering/Asura_Text_Output.h"

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#define ASURA_CHANNEL_GRAPH_MAX_CONTROL_POINTS	8

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

extern unsigned int Asura_OverlayBrightness;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

class Asura_Channel_Graph_Block;

struct Asura_Channel_Graph_ControlPoint
{
	bool bValid;
	float fX, fY;
	bool bIsXAnchored, bIsYAnchored;
	Asura_Channel_Graph_Block* pxContainerBlock;
};

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

struct Asura_Channel_Graph_BlockRatioValue
{
	int iBlockIndex;
	float fRatio;
	float fPos;
	float fWidth;
};

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

class Asura_Channel_Graph_Dialog;

class Asura_Channel_Graph_Block
{
	friend class Asura_Channel_Graph_Channel;

	public:
		Asura_Channel_Graph_Block (Asura_Channel_Graph_Dialog* pxParentDialog, Asura_Channel_Graph_Channel* pxParentChannel, const char* szName, const float fPos, const float fWidth, const bool bCanSize, const bool bCanMove)
		{
			m_pxParentDialog = pxParentDialog;
			m_pxParentChannel = pxParentChannel;
			m_szName = szName;
			SetStartPosition (fPos);
			SetWidth (fWidth);
			m_bCanSize = bCanSize;
			m_bCanMove = bCanMove;
			m_uColour = 0x00ffffff;
		}
		~Asura_Channel_Graph_Block ()
		{
			Remove ();
		}

		inline void SetCanSize (const bool bFlag) { m_bCanSize = bFlag; }
		inline bool GetCanSize () const { return m_bCanSize; }

		inline void SetCanMove (const bool bFlag) { m_bCanMove = bFlag; }
		inline bool GetCanMove () const { return m_bCanMove; }

		void Remove ();

		float GetStartPosition ();
		float GetEndPosition ();

		void SetStartPosition (float fStartPos, const bool bSupressChecks = false);
		void SetEndPosition (float fEndPos, const bool bSupressChecks = false);

		float GetWidth ();
		void SetWidth (float fWidth, const bool bSupressChecks = false);

		void SetColour (u_int uColour);

		void Render (const float fBlockX, const float fBlockY, const float fBlockHeight);

	private:
		inline void SetWidthScreenSpace (const float fWidth) { m_fWidth = fWidth; }
		inline float GetWidthScreenSpace () const { return m_fWidth; }

		inline void SetStartPositionScreenSpace (const float fPos) { m_fPosition = fPos; }
		inline float GetStartPositionScreenSpace () const { return m_fPosition; }

		inline void SetEndPositionScreenSpace (const float fPos) { m_fWidth = fPos - m_fPosition; }
		inline float GetEndPositionScreenSpace () const { return m_fPosition + m_fWidth; }

		Asura_String m_szName;
		float m_fWidth;
		float m_fPosition;
		bool m_bCanSize;
		bool m_bCanMove;
		Asura_Channel_Graph_Dialog* m_pxParentDialog;
		Asura_Channel_Graph_Channel* m_pxParentChannel;
		u_int m_uColour;
};

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

class Asura_Channel_Graph_Channel
{
	friend class Asura_Channel_Graph_Block;
	
	public:
		Asura_Channel_Graph_Channel (Asura_Channel_Graph_Dialog* pxParent)
		{
			m_iDraggedNode = -1;
			m_iDraggingBlock = -1;
			m_uNumRatioElements = 0;
			m_pxParent = pxParent;
			m_iAttachedDragBlockIndex = -1;
			m_bIsDetachable = true;
			m_bRatioResizingEnabled = true;
			m_bAnchorStartBlock = false;
			m_bAnchorEndBlock = false;
			for( int iPoint = 0; iPoint < ASURA_CHANNEL_GRAPH_MAX_CONTROL_POINTS; ++iPoint )
			{
				m_axControlPoints [iPoint].bValid = false;
			}
		}
		~Asura_Channel_Graph_Channel ()
		{
			while( !m_xBlockList.IsEmpty() )
			{
				Asura_Channel_Graph_Block* pxBlock = m_xBlockList.GetHeadObject();
				delete pxBlock;
			}
			Remove();
		}

		Asura_Channel_Graph_Block* AddBlock (const char* szName, const float fPos, const float fWidth, const bool bCanSize = true, const bool bCanMove = true);

		void Render (const float fX, const float fY, const float fWidth, const DWORD iBackgroundColour);
		void Update (const Asura_Vector_2& xOrigin);

		void Remove ();

		void SetControlPoint (const u_int uIndex, const float fX, const float fY, const bool bIsXAnchored = false, const bool bIsYAnchored = false);

		float GetControlPointX ( const u_int uIndex ) const;
		float GetControlPointY ( const u_int uIndex ) const;

		void SetDetachableFlag (const bool bFlag) { m_bIsDetachable = bFlag; }
		void SetRatioResizingFlag (const bool bFlag) { m_bRatioResizingEnabled = bFlag; }
		void SetAnchorStartBlockFlag (const bool bFlag) { m_bAnchorStartBlock = bFlag; UpdateBlockAnchorage (); }
		void SetAnchorEndBlockFlag (const bool bFlag) { m_bAnchorEndBlock = bFlag; UpdateBlockAnchorage (); }

		Asura_Channel_Graph_Block* GetPrevBlock (const Asura_Channel_Graph_Block* pxBlock);
		Asura_Channel_Graph_Block* GetNextBlock (const Asura_Channel_Graph_Block* pxBlock);

		void SnapAllBlocksToTicks ();

	private:
		enum DragModeEnum
		{
			DRAG_MODE_RESIZE_LEFT,
			DRAG_MODE_RESIZE_RIGHT,
			DRAG_MODE_MOVE,
		};

		void RenderPoint (float fX, float fY, bool bHighlight = false) const;

		void CalculateBlockWidthRatiosForRightDrag (const u_int uBlockIndex);
		void CalculateBlockWidthRatiosForLeftDrag (const u_int uBlockIndex);

		float ConvertChannelXToScreen (const float fX) const;
		float ConvertChannelYToScreen (const float fY) const;
		float ConvertScreenXToChannel (const float fX) const;
		float ConvertScreenYToChannel (const float fY) const;

		void UpdateControlPointPositions ();
		void UpdateBlockAnchorage ();

		float RoundMouseXToTicks ( float fMouseX, const Asura_Vector_2 xOrigin );
		
		Asura_Channel_Graph_Block* CheckControlPointIsWithinBlock (const float fX);

		Asura_List <Asura_Channel_Graph_Block> m_xBlockList;

		Asura_Channel_Graph_ControlPoint m_axControlPoints [ASURA_CHANNEL_GRAPH_MAX_CONTROL_POINTS];

		Asura_Vector_2 m_xTopLeft;
		float m_fPhysicalWidth, m_fPhysicalHeight; // This is the physical width and height of the channel (after left and right margin has been removed)

		int m_iDraggedNode;
		int m_iDraggingBlock;
		DragModeEnum m_eDraggingMode;
		float m_fInitialBlockDragMouseX;
		float m_fInitialBlockXPos;
		float m_fInitialBlockWidth;
		float m_fInitialAttachedBlockWidth;
		int m_iAttachedDragBlockIndex;
		bool m_bIsDetachable;
		bool m_bRatioResizingEnabled;
		bool m_bAnchorStartBlock;
		bool m_bAnchorEndBlock;

		Asura_Channel_Graph_Dialog* m_pxParent;

		static const int MAX_RATIO_ELEMENTS = 8;

		Asura_Channel_Graph_BlockRatioValue m_axWidthRatios [MAX_RATIO_ELEMENTS];
		u_int m_uNumRatioElements;
		float m_fTotalRatioWidth;
};

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

class Asura_Channel_Graph_Dialog : public Asura_Dialog
{
	friend class Asura_Channel_Graph_Channel;
	friend class Asura_Channel_Graph_Block;

	public:
		Asura_Channel_Graph_Dialog ( const Asura_Vector_2& xOffset, const float fWidth, bool* pbModified = 0 );
		virtual ~Asura_Channel_Graph_Dialog ();

		inline void SetGraphDomain (const float fMin, const float fMax)
		{
			m_fDomainMin = fMin;
			m_fDomainMax = fMax;
		}

		inline void GetGraphDomain (float* pfMin, float* pfMax)
		{
			if (pfMin)	*pfMin = m_fDomainMin;
			if (pfMax)	*pfMax = m_fDomainMax;
		}

		virtual bool IsPointInDialog (const Asura_Vector_2& xOrigin, const Asura_Vector_2& xPos) const;

		Asura_Channel_Graph_Channel* AddChannel ();

		virtual void Render (const Asura_Vector_2& xOrigin);
		virtual void Update (const Asura_Vector_2& xOrigin);

		virtual void ShowPlaybackMarker		();
		virtual void HidePlaybackMarker		();
		virtual void SetPlaybackMarker		(const float fMarkerPos);

		virtual void	SetSnapInterval		(const float fSnapInterval);
		virtual float	GetSnapInterval		()							{ return m_fSnapInterval; }

		virtual Asura_Channel_Graph_Channel* GetChannelByIndex ( int iIndex ) { return m_xChannelList.GetEntryByIndex( iIndex ); }

	private:

		Asura_List <Asura_Channel_Graph_Channel> m_xChannelList;

		float	m_fWidth;
		float	m_fDomainMin;
		float	m_fDomainMax;
		
		float	m_fPlaybackMarker;
		bool	m_bPlaybackMarkerVisible;

		float	m_fSnapInterval;

		bool*	m_pbModified;
};

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#endif //__ASURA_CHANNEL_GRAPH_DIALOG_H__

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
