//
// Asura_Cursor.cpp
//
// Created by Kevin Lea - 12/06/2001 11:10:53
// 
// Copyright (C) 2001 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"
#include "Interface/Asura_Cursor.h"

#ifdef ASURA_USE_CURSOR

#include "Core/Asura_HashTree.h"
#include "Core/Asura_TextureManagement.h"
#include "Core/Asura_Timers.h"
#include "Core/Asura_UserInput.h"

#include "Rendering/Asura_DisplayLimits.h"
#include "Rendering/Asura_Render.h"
#include "Rendering/Asura_Text_Output.h"
#include "Rendering/Asura_Viewport.h"

Asura_Cursor_PosInfo								Asura_Cursor::s_axCursorPosInfo[ASURA_NUM_CURSORS];
u_int												Asura_Cursor::s_uCurrentCursorIndex = 0;
Asura_Hash_ID										Asura_Cursor::s_uCurrentCursorHash = ASURA_HASH_ID_UNSET;
Asura_Cursor_RenderInfo*							Asura_Cursor::s_pxCurrentCursor;
Asura_TemplateHashTree<Asura_Cursor_RenderInfo>		Asura_Cursor::s_xCursorTree;
bool Asura_Cursor::s_bHideCursor = false;

#ifdef ASURA_TOOLS
static const Asura_Hash_ID g_auDefaultCursorIDs[NO_OF_ASURA_CURSORS] =
{
	0x234B2326, //Asura_GetHashID("ASURA_CURSOR_CROSSHAIR");
	0x951A810D, //Asura_GetHashID("ASURA_CURSOR_ARROW");
	0x974D14A4, //Asura_GetHashID("ASURA_CURSOR_EYEDROP");
	0x253436BB, //Asura_GetHashID("ASURA_CURSOR_OBJECT");
};
#endif //ASURA_TOOLS

Asura_Cursor_PosInfo::Asura_Cursor_PosInfo()
: m_fX(0.5f)
, m_fY(0.5f)
, m_fDeltaX(0.f)
, m_fDeltaY(0.f)
, m_fOngoingDeltaX(0.f)
, m_fOngoingDeltaY(0.f)
, m_fCursorSpeed(1.f)
, m_fTimeSpentStatic(0.f)
, m_bCursorMovementEnabled(true)
, m_bTeleportedSinceLastUpdate(false)
, m_bOffscreen(false)
{

};

void Asura_Cursor::Initialise(void)
{
	s_uCurrentCursorHash = 0;
	s_pxCurrentCursor = 0;
	
	
	#ifdef ASURA_TOOLS
	//Setup the default cursors for tools
	Asura_Cursor_RenderInfo xInfo;
	xInfo.m_fCursorWidth = 32.0f;
	xInfo.m_fCursorHeight = 32.0f;
	// Default to full extent of texture.
	xInfo.m_fTileU1 = xInfo.m_fTileV1 = 0.0f;
	xInfo.m_fTileU2 = xInfo.m_fTileV2 = 1.0f;

	xInfo.m_iTextureID = Asura_Texture_Management::AddTextureWithResetGroup("\\tools\\cursor.dds", ASURA_TEXTURE_LOADED_PERMANENT );
	xInfo.m_fHotSpotX = 15.5f;
	xInfo.m_fHotSpotY = 15.5f;
	AddCursorType(g_auDefaultCursorIDs[ASURA_CURSOR_CROSSHAIR], xInfo);

	xInfo.m_iTextureID = Asura_Texture_Management::AddTextureWithResetGroup("\\tools\\arrow.dds", ASURA_TEXTURE_LOADED_PERMANENT );
	xInfo.m_fHotSpotX = 4.5f;
	xInfo.m_fHotSpotY = 3.5f;
	AddCursorType(g_auDefaultCursorIDs[ASURA_CURSOR_ARROW], xInfo);

	xInfo.m_iTextureID = Asura_Texture_Management::AddTextureWithResetGroup("\\tools\\eyedropper.dds", ASURA_TEXTURE_LOADED_PERMANENT );
	xInfo.m_fHotSpotX = 16.5f;
	xInfo.m_fHotSpotY = 15.5f;
	AddCursorType(g_auDefaultCursorIDs[ASURA_CURSOR_EYEDROP], xInfo);

	xInfo.m_iTextureID = Asura_Texture_Management::AddTextureWithResetGroup("\\tools\\cursor_object.dds", ASURA_TEXTURE_LOADED_PERMANENT );
	xInfo.m_fHotSpotX = 15.5f;
	xInfo.m_fHotSpotY = 15.5f;
	AddCursorType(g_auDefaultCursorIDs[ASURA_CURSOR_OBJECT], xInfo);

	Asura_Texture_Management::LoadTextures();

	SetType(ASURA_CURSOR_CROSSHAIR);
	#endif //ASURA_TOOLS
}

void Asura_Cursor::Shutdown()
{
	s_xCursorTree.Reset();
	s_uCurrentCursorHash = 0;
	s_pxCurrentCursor = 0;
}

void Asura_Cursor::OncePerFrameUpdate(void)
{
	for(u_int u = 0; u < ASURA_NUM_CURSORS; ++u)
	{
		Asura_Cursor_PosInfo& xPosInfo = s_axCursorPosInfo[u];
		if(!xPosInfo.m_bCursorMovementEnabled)
		{
			continue;
		}

		xPosInfo.m_fDeltaX = xPosInfo.m_fOngoingDeltaX;
		xPosInfo.m_fDeltaY = xPosInfo.m_fOngoingDeltaY;

		xPosInfo.m_fOngoingDeltaX = 0.f;
		xPosInfo.m_fOngoingDeltaY = 0.f;

	// if the cursor hasn't moved and no mouse buttons are pressed, increment the 'hasn't moved' timer
		if (Asura_Maths::Abs(xPosInfo.m_fDeltaX) < 0.001f && Asura_Maths::Abs(xPosInfo.m_fDeltaY) < 0.001f && !IsAnyMouseButtonPressed())
	{
			xPosInfo.m_fTimeSpentStatic += Asura_Timers::GetFrameTime();
	}
	else // otherwise reset it to zero
	{
			xPosInfo.m_fTimeSpentStatic = 0.0f;
	}
}
}
void Asura_Cursor::Update(float fDeltaX, float fDeltaY, u_int uCursorIndex)
{
	const int iViewportWidth = Asura_Viewport::GetWidth();
	const int iViewportHeight = Asura_Viewport::GetHeight();
	if( 0 == iViewportWidth || 0 == iViewportHeight)
	{
		return;
	}
	
	Asura_Cursor_PosInfo& xPosInfo = s_axCursorPosInfo[uCursorIndex];
	const float fOldX = xPosInfo.m_fX;
	const float fOldY = xPosInfo.m_fY;

	if (xPosInfo.m_bCursorMovementEnabled)
	{
		xPosInfo.m_fX += (fDeltaX / iViewportWidth)*xPosInfo.m_fCursorSpeed;
		xPosInfo.m_fY += (fDeltaY / iViewportHeight)*xPosInfo.m_fCursorSpeed;
	}

	Asura_Bounding_Rectangle xClipRect = Asura_DisplayLimits::Platform_GetBoundingRectangle();
	xPosInfo.m_fX = Asura_Maths::ClampToRange( xPosInfo.m_fX, xClipRect.MinX / iViewportWidth, xClipRect.MaxX / iViewportWidth );
	xPosInfo.m_fY = Asura_Maths::ClampToRange( xPosInfo.m_fY, xClipRect.MinY / iViewportHeight, xClipRect.MaxY / iViewportHeight );

	xPosInfo.m_fOngoingDeltaX += xPosInfo.m_fX - fOldX;
	xPosInfo.m_fOngoingDeltaY += xPosInfo.m_fY - fOldY;
	xPosInfo.m_bTeleportedSinceLastUpdate = false;
}

void Asura_Cursor::UpdateAbs(float fX, float fY, u_int uCursorIndex)
{
	const int iViewportWidth = Asura_Viewport::GetWidth();
	const int iViewportHeight = Asura_Viewport::GetHeight();
	if ( 0 == iViewportWidth || 0 == iViewportHeight )
	{
		return;
	}
	
	Asura_Cursor_PosInfo& xPosInfo = s_axCursorPosInfo[uCursorIndex];
	const float fOldX = xPosInfo.m_fX;
	const float fOldY = xPosInfo.m_fY;

	xPosInfo.m_fX = fX / iViewportWidth;
	xPosInfo.m_fY = fY / iViewportHeight;

	Asura_Bounding_Rectangle xClipRect = Asura_DisplayLimits::Platform_GetBoundingRectangle();
	xPosInfo.m_fX = Asura_Maths::ClampToRange( xPosInfo.m_fX, xClipRect.MinX / iViewportWidth, xClipRect.MaxX / iViewportWidth );
	xPosInfo.m_fY = Asura_Maths::ClampToRange( xPosInfo.m_fY, xClipRect.MinY / iViewportHeight, xClipRect.MaxY / iViewportHeight );

	xPosInfo.m_fOngoingDeltaX += xPosInfo.m_fX - fOldX;
	xPosInfo.m_fOngoingDeltaY += xPosInfo.m_fY - fOldY;
	xPosInfo.m_bTeleportedSinceLastUpdate = false;
}


void Asura_Cursor::Render(void)
{
	if(s_pxCurrentCursor && !s_bHideCursor)
	{
		Asura_Render::Overlay.Colour = 0xffffffff;
		Asura_Render::Overlay.TextureIndex = s_pxCurrentCursor->m_iTextureID;
		Asura_Render::Overlay.TranslucencyMode = ASURA_TRANSLUCENCY_NORMAL;

		// Just render cursor 0 for the moment.
		Asura_Cursor_PosInfo& xPosInfo = s_axCursorPosInfo[s_uCurrentCursorIndex];

		const float fX = Asura_Maths::FloatToInt(xPosInfo.m_fX * Asura_Viewport::GetWidth()) - s_pxCurrentCursor->m_fHotSpotX;
		const float fY = Asura_Maths::FloatToInt(xPosInfo.m_fY * Asura_Viewport::GetHeight()) - s_pxCurrentCursor->m_fHotSpotY;
		
		// RWM - Handling for tiled textures
		const float fTileWidth  = s_pxCurrentCursor->m_fTileU2 - s_pxCurrentCursor->m_fTileU1;
		const float fTileHeight = s_pxCurrentCursor->m_fTileV2 - s_pxCurrentCursor->m_fTileV1;
		Asura_Render::Overlay.SetRectUV(s_pxCurrentCursor->m_fTileU1, s_pxCurrentCursor->m_fTileV1, fTileWidth, fTileHeight);

		Asura_Render::Overlay.SetRectXY(fX, fY, s_pxCurrentCursor->m_fCursorWidth,s_pxCurrentCursor->m_fCursorHeight);

		Asura_Render::Overlay.Render();
	}
}

void Asura_Cursor::AddCursorType(Asura_Hash_ID uCursorID, const Asura_Cursor_RenderInfo& xCursorInfo)
{
	if(!s_xCursorTree.AddNode(uCursorID, xCursorInfo))
	{
		Asura_Assert(0,"A cursor with that ID already exists");
	}
}

#ifdef ASURA_TOOLS
void Asura_Cursor::SetType(enum ASURA_CURSOR_ID type)
{
	SetTypeByHash(g_auDefaultCursorIDs[type]);
}
#endif //ASURA_TOOLS

void Asura_Cursor::SetTypeByHash(Asura_Hash_ID uCursorID)
{
	if(uCursorID == s_uCurrentCursorHash) return;

	if(uCursorID)
	{
		Asura_Cursor_RenderInfo* pxCursor = s_xCursorTree.GetDataByHash(uCursorID);
		Asura_Assert(pxCursor, "Failed to find specified cursor");
		if(pxCursor)
		{
			s_pxCurrentCursor = pxCursor;
			s_uCurrentCursorHash = uCursorID;
		}
	}
	else
	{
		//revert back to having no cursor
		s_uCurrentCursorHash = 0;
		s_pxCurrentCursor = 0;
	}
}

#include "Camera/Asura_Camera.h"
void Asura_Cursor::GetLOSDirection(Asura_Vector_3& v)
{
	const Asura_ViewingArea* pxArea = Asura_Camera::GetViewingArea();
	if(pxArea->m_fWidth <= 0.f || pxArea->m_fHeight <= 0.f)
	{
		Asura_Assert(0, "Viewing area is zero");
		v = Asura_Vector_3(0.f, 0.f, 1.f);
		return;
	}
	const Asura_Cursor_PosInfo& xPosInfo = s_axCursorPosInfo[s_uCurrentCursorIndex];
	const float fViewX = (xPosInfo.m_fX - pxArea->m_fXOffset)/pxArea->m_fWidth;
	const float fViewY = (xPosInfo.m_fY - pxArea->m_fYOffset)/pxArea->m_fHeight;
	const float fRatio = pxArea->GetAspectRatio()*Asura_Camera::GetPhysicalScreenAspectRatio();
	
	v.x = fViewX - 0.5f;
	v.y = (fViewY - 0.5f)/fRatio;
	v.z = 0.5f*Asura_Camera::GetOneOverTanHalfFovX();

	v.Normalise();
}

void Asura_Cursor::GetScreenCoords(float& cx, float& cy)
{
	const Asura_Cursor_PosInfo& xPosInfo = s_axCursorPosInfo[s_uCurrentCursorIndex];
	cx = xPosInfo.m_fX * Asura_Viewport::GetWidth()/10.0f;
	cy = xPosInfo.m_fY * Asura_Viewport::GetHeight()/14.0f;
}
void Asura_Cursor::GetScreenDeltas(float& cx, float& cy)
{
	const Asura_Cursor_PosInfo& xPosInfo = s_axCursorPosInfo[s_uCurrentCursorIndex];
	cx = xPosInfo.m_fDeltaX * Asura_Viewport::GetWidth()/10.0f;
	cy = xPosInfo.m_fDeltaY * Asura_Viewport::GetHeight()/14.0f;
}
void Asura_Cursor::GetScreenSpaceCoords(int& sx, int& sy)
{
	const Asura_Cursor_PosInfo& xPosInfo = s_axCursorPosInfo[s_uCurrentCursorIndex];
	sx = Asura_Maths::FloatToInt(xPosInfo.m_fX * Asura_Viewport::GetWidth());
	sy = Asura_Maths::FloatToInt(xPosInfo.m_fY * Asura_Viewport::GetHeight());
}
void Asura_Cursor::GetScreenSpaceDeltas(int& sx, int& sy)
{
	const Asura_Cursor_PosInfo& xPosInfo = s_axCursorPosInfo[s_uCurrentCursorIndex];
	sx = Asura_Maths::FloatToInt(xPosInfo.m_fDeltaX * Asura_Viewport::GetWidth());
	sy = Asura_Maths::FloatToInt(xPosInfo.m_fDeltaY * Asura_Viewport::GetHeight());
}

void Asura_Cursor::SetScreenSpaceCoords(int sx,int sy)
{
	Asura_Cursor_PosInfo& xPosInfo = s_axCursorPosInfo[s_uCurrentCursorIndex];
	xPosInfo.m_fX = (float)sx / Asura_Viewport::GetWidth();
	xPosInfo.m_fY = (float)sy / Asura_Viewport::GetHeight();

	Asura_Bounding_Rectangle xClipRect = Asura_DisplayLimits::Platform_GetBoundingRectangle();
	xPosInfo.m_fX = Asura_Maths::ClampToRange(xPosInfo.m_fX, xClipRect.MinX / Asura_Viewport::GetWidth(), xClipRect.MaxX / Asura_Viewport::GetWidth());
	xPosInfo.m_fY = Asura_Maths::ClampToRange(xPosInfo.m_fY, xClipRect.MinY / Asura_Viewport::GetHeight(), xClipRect.MaxY / Asura_Viewport::GetHeight());

	xPosInfo.m_bTeleportedSinceLastUpdate = true;
}

bool Asura_Cursor::Inside( const Asura_Vector_2& xPoint, float fWidth, float fHeight )
{
	float fXCoord, fYCoord;

	GetScreenCoords(fXCoord, fYCoord);

	if (xPoint.x <= fXCoord && xPoint.x + fWidth  >= fXCoord)
	if (xPoint.y <= fYCoord && xPoint.y + fHeight >= fYCoord)
	{
		return true;
	}

	return false;
}

bool Asura_Cursor::InsideScreenSpace( const Asura_Vector_2& xPoint, float fWidth, float fHeight )
{
	int iXCoord, iYCoord;

	GetScreenSpaceCoords(iXCoord, iYCoord);

	if (xPoint.x <= iXCoord && xPoint.x + fWidth  >= iXCoord)
	if (xPoint.y <= iYCoord && xPoint.y + fHeight >= iYCoord)
	{
		return true;
	}

	return false;
}

bool Asura_Cursor::IsAnyMouseButtonPressed( void )
{
	if	( Asura_UserInput::KeyPressed(ASURA_KEYID_MOUSEWHEELUP)
		||Asura_UserInput::KeyPressed(ASURA_KEYID_MOUSEWHEELDOWN)
		||Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_MOUSEBUTTON1)
		||Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_MOUSEBUTTON2)
		||Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_MOUSEBUTTON3)
		||Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_MOUSEBUTTON4) )
	{
		return true;
	}
	else
	{
		return false;
	}
		       
}

void Asura_Cursor::SetCursorOffscreen(bool bOffscreen, u_int uCursorIndex)
{
	Asura_Cursor_PosInfo& xPosInfo = s_axCursorPosInfo[uCursorIndex];
	if(xPosInfo.m_bOffscreen != bOffscreen)
	{
		if(xPosInfo.m_bOffscreen)
		{
			xPosInfo.m_bTeleportedSinceLastUpdate = true;
		}
		xPosInfo.m_bOffscreen = bOffscreen;
	}
}

bool Asura_Cursor::IsCursorOffscreen()
{
	Asura_Cursor_PosInfo& xPosInfo = s_axCursorPosInfo[s_uCurrentCursorIndex];
	return xPosInfo.m_bOffscreen;

}

void Asura_Cursor::SetCursorIndex(u_int uIndex)
{
	Asura_Assert(uIndex < ASURA_NUM_CURSORS, "Cursor index out of range");
	s_uCurrentCursorIndex = Asura_Maths::Min(uIndex, static_cast<u_int>(ASURA_NUM_CURSORS - 1));
}

#endif //ASURA_USE_CURSOR
