//
// Asura_Cursor.h
//
// Created by Kevin Lea - 12/06/2001 11:11:37
// 
// Copyright (C) 2001 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef __ASURA_CURSOR_H__
#define __ASURA_CURSOR_H__

#ifdef ASURA_TOOLS
//force this to be available for tools builds. RAR 03/08/2007
#undef ASURA_USE_CURSOR
#define ASURA_USE_CURSOR 1
#endif // ASURA_TOOLS

#ifdef ASURA_USE_CURSOR

#ifndef ASURA_NUM_CURSORS
#define ASURA_NUM_CURSORS (1)
#endif

#include "Core/Asura_Maths.h"

template<typename T> class Asura_TemplateHashTree;

enum ASURA_CURSOR_ID
{
	ASURA_CURSOR_CROSSHAIR,
	ASURA_CURSOR_ARROW,
	ASURA_CURSOR_EYEDROP,
	ASURA_CURSOR_OBJECT,
	NO_OF_ASURA_CURSORS
};

class Asura_Cursor_RenderInfo
{
	public:

		float m_fHotSpotX;
		float m_fHotSpotY;

		float m_fCursorWidth;
		float m_fCursorHeight;

		// Cut from coords for tiled textures
		float m_fTileU1;
		float m_fTileV1;
		float m_fTileU2;
		float m_fTileV2;

		int m_iTextureID;
};

class Asura_Cursor_PosInfo
{
public:
	Asura_Cursor_PosInfo();

	float	m_fX;
	float	m_fY;
	float	m_fDeltaX;
	float	m_fDeltaY;
	float	m_fOngoingDeltaX;	
	float	m_fOngoingDeltaY;
	float	m_fCursorSpeed;
	float	m_fTimeSpentStatic;
	bool	m_bCursorMovementEnabled;
	bool	m_bTeleportedSinceLastUpdate;
	bool	m_bOffscreen;
};

class Asura_Cursor
{
	public:
		static void Initialise(void);
		static void Shutdown();

		static void Update(float deltaX, float deltaY, u_int uCursorIndex = s_uCurrentCursorIndex);
		static void UpdateAbs(float fX,float fY, u_int uCursorIndex = s_uCurrentCursorIndex);//update with absolute positions instead of deltas
		static void OncePerFrameUpdate(void);
		static void Render(void);

		static void GetLOSDirection(Asura_Vector_3& v);

		//Adds a new cursor type for use by the system
		static void AddCursorType(Asura_Hash_ID uCursorID, const Asura_Cursor_RenderInfo& xCursorInfo);

		#ifdef ASURA_TOOLS
		//switches to one of the predefined tools cursor types
		static void SetType(enum ASURA_CURSOR_ID type);
		#endif
		//set the current cursor type by hash ID
		static void SetTypeByHash(Asura_Hash_ID uCursorID);
		
		//Disables rendering of the cursor (currently used by tools if the windows cursor is being used instead)
		static void SetHideCursor(bool bHide){s_bHideCursor = bHide;}

		static void GetCoords(float& fX, float& fY) { fX = s_axCursorPosInfo[s_uCurrentCursorIndex].m_fX; fY = s_axCursorPosInfo[s_uCurrentCursorIndex].m_fY; }
		static void GetScreenCoords(float& cx, float& cy);
		static void GetScreenDeltas(float& cx, float& cy);
		static void GetScreenSpaceDeltas(int& sx, int& sy);
		static void GetScreenSpaceCoords(int& sx, int& sy);
		static void SetScreenSpaceCoords(int sx, int sy);

		static bool Inside( const Asura_Vector_2& p, float w, float h);
		static bool InsideScreenSpace( const Asura_Vector_2& p, float w, float h);

		//Determines whether cursor gets moved during Update().. replacement for automatic disabling of movement
		//when middle mouse button is held. RAR 1/07/02
		static void EnableCursorMovement(bool enable) { s_axCursorPosInfo[s_uCurrentCursorIndex].m_bCursorMovementEnabled=enable; }
		static bool IsCursorMovementEnabled(){return s_axCursorPosInfo[s_uCurrentCursorIndex].m_bCursorMovementEnabled;}

		//Sets a multiplier for the cursor movement speed
		static void SetCursorSpeed(float fSpeed){s_axCursorPosInfo[s_uCurrentCursorIndex].m_fCursorSpeed = fSpeed;}
		static float GetCursorSpeed() {return s_axCursorPosInfo[s_uCurrentCursorIndex].m_fCursorSpeed;}
	
		//! returns the number of seconds the cursor hasn't moved / user hasn't pressed a mouse button
		static float GetTimeSpentStatic( void ) { return s_axCursorPosInfo[s_uCurrentCursorIndex].m_fTimeSpentStatic; }

		//! function to see if the cursor has been classed as 'immobile'
		//! Currently hard coded to one-second: could be changed to use a variable setable by artists etc
		static bool IsStatic( void ) { return (s_axCursorPosInfo[s_uCurrentCursorIndex].m_fTimeSpentStatic >= 1.0f); }

		//! utility function to see if any of the mouse buttons/wheel is in use 
		//! called internally but made public in case it's of use to others
		static bool IsAnyMouseButtonPressed( void );

		//returns true if something has set the cursor position directly using SetScreenSpaceCoords since the last update
		static bool TeleportedSinceLastUpdate(){return s_axCursorPosInfo[s_uCurrentCursorIndex].m_bTeleportedSinceLastUpdate;}
		
		static void SetCursorOffscreen( bool bOffscreen, u_int uCursorIndex = s_uCurrentCursorIndex );
		static bool IsCursorOffscreen();
		
		static void SetCursorIndex(u_int uIndex);
		
	private:
		static Asura_Cursor_PosInfo s_axCursorPosInfo[ASURA_NUM_CURSORS];
		static u_int s_uCurrentCursorIndex;
		static bool s_bHideCursor;

		static Asura_Hash_ID s_uCurrentCursorHash;
		static Asura_Cursor_RenderInfo* s_pxCurrentCursor;
		static Asura_TemplateHashTree<Asura_Cursor_RenderInfo> s_xCursorTree;
};

#endif //ASURA_USE_CURSOR


#endif//__ASURA_CURSOR_H__
