//
// Asura_Graph_Gadget.cpp
//
// Created by KOR - 16/02/2004 17:15:49
//
// Copyright (C) 2004 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

#ifdef ASURA_TOOLS

// This file's header.
#include "Interface/Asura_Graph_Gadget.h"

// Asura includes
#include "Core/Asura_UserInput.h"
#include "Core/Asura_TextureManagement.h"
#include "Core/Asura_Timers.h"
#include "Interface/Asura_Cursor.h"
#include "Interface/Asura_Graph_Dialog.h"
#include "Rendering/Asura_Colour.h"
#include "Rendering/Asura_Render.h"

#define ASURA_GRAPH_COLOUR_F (0x00000000)

#define ASURA_GRAPH_COLOUR_X (0x00ff0000)
#define ASURA_GRAPH_COLOUR_Y (0x0000ff00)
#define ASURA_GRAPH_COLOUR_Z (0x000000ff)

#define ASURA_GRAPH_COLOUR_R (0x00ff0000)
#define ASURA_GRAPH_COLOUR_G (0x0000ff00)
#define ASURA_GRAPH_COLOUR_B (0x000000ff)
#define ASURA_GRAPH_COLOUR_A (0x00000000)



Asura_Graph_Source_Data<float> Asura_Graph_Gadget<float>::s_xPasteBuffer; 
Asura_Graph_Source_Data<Asura_Vector_2> Asura_Graph_Gadget<Asura_Vector_2>::s_xPasteBuffer; 
Asura_Graph_Source_Data<Asura_Vector_3> Asura_Graph_Gadget<Asura_Vector_3>::s_xPasteBuffer; 
Asura_Graph_Source_Data<Asura_RGBA> Asura_Graph_Gadget<Asura_RGBA>::s_xPasteBuffer; 

//Ideally this would be a static variable in Asur_Graph_Gadget , but I'm not sure there is any way of doing
//so that would be shared across all the template instances. RAR 16/10/2008
bool evilg_bAsura_Graph_Gadget_LimitShiftOrScaleToCurrentPoint = false;

void Asura_Graph_Gadget<float>::SetCurrentValue(float fValue)
{
	m_pxCurrentPoint->m_xValue = fValue;
}

void Asura_Graph_Gadget<Asura_Vector_2>::SetCurrentValue(float fValue)
{
	switch(m_ucCurrentDimension)
	{
		case 0:	m_pxCurrentPoint->m_xValue.x = fValue; break;
		case 1: m_pxCurrentPoint->m_xValue.y = fValue; break;
		default: Asura_Assert(0, "Dimension out of range. This should never happen.");
	}
}

void Asura_Graph_Gadget<Asura_Vector_3>::SetCurrentValue(float fValue)
{
	switch(m_ucCurrentDimension)
	{
		case 0:	m_pxCurrentPoint->m_xValue.x = fValue; break;
		case 1: m_pxCurrentPoint->m_xValue.y = fValue; break;
		case 2: m_pxCurrentPoint->m_xValue.z = fValue; break;
		default: Asura_Assert(0, "Dimension out of range. This should never happen.");
	}
}

void Asura_Graph_Gadget<Asura_RGBA>::SetCurrentValue(float fValue)
{
	switch(m_ucCurrentDimension)
	{
		case 0:	m_pxCurrentPoint->m_xValue.m_fR	= fValue; break;
		case 1: m_pxCurrentPoint->m_xValue.m_fG	= fValue; break;
		case 2: m_pxCurrentPoint->m_xValue.m_fB	= fValue; break;
		case 3: m_pxCurrentPoint->m_xValue.m_fA	= fValue; break;
		default: Asura_Assert(0, "Dimension out of range. This should never happen.");
	}
}

float Asura_Graph_Gadget<float>::GetValue(const float& fValue, u_char ucDimension) const
{
	return fValue;
}

float Asura_Graph_Gadget<Asura_Vector_2>::GetValue(const Asura_Vector_2& xValue, u_char ucDimension) const
{
	switch(ucDimension)
	{
		case 0:	return xValue.x;
		case 1:	return xValue.y;
		default: Asura_Assert(0, "Dimension out of range. This should never happen."); return 0.0f;
	}
}

float Asura_Graph_Gadget<Asura_Vector_3>::GetValue(const Asura_Vector_3& xValue, u_char ucDimension) const
{
	switch(ucDimension)
	{
		case 0:	return xValue.x;
		case 1:	return xValue.y;
		case 2:	return xValue.z;
		default: Asura_Assert(0, "Dimension out of range. This should never happen."); return 0.0f;
	}
}

float Asura_Graph_Gadget<Asura_RGBA>::GetValue(const Asura_RGBA& xValue, u_char ucDimension) const
{
	switch(ucDimension)
	{
		case 0:	return xValue.m_fR;
		case 1:	return xValue.m_fG;
		case 2:	return xValue.m_fB;
		case 3:	return xValue.m_fA;
		default: Asura_Assert(0, "Dimension out of range. This should never happen."); return 0.0f;
	}
}


u_int Asura_Graph_Gadget<float>::GetNumberOfDimensions()			const { return 1; }
u_int Asura_Graph_Gadget<Asura_Vector_2>::GetNumberOfDimensions()	const { return 2; }
u_int Asura_Graph_Gadget<Asura_Vector_3>::GetNumberOfDimensions()	const { return 3; }
u_int Asura_Graph_Gadget<Asura_RGBA>::GetNumberOfDimensions()		const { return 4; }

const char* Asura_Graph_Gadget<float>::GetLabelForDimension(u_char ucDimension) const
{
	return "V:";
}

const char* Asura_Graph_Gadget<Asura_Vector_2>::GetLabelForDimension(u_char ucDimension) const
{
	switch(ucDimension)
	{
		case 0:	return "X:";
		case 1:	return "Y:";
		default: Asura_Assert(0, "Dimension out of range. This should never happen."); return "";
	}
}

const char* Asura_Graph_Gadget<Asura_Vector_3>::GetLabelForDimension(u_char ucDimension) const
{
	switch(ucDimension)
	{
		case 0:	return "X:";
		case 1:	return "Y:";
		case 2:	return "Z:";
		default: Asura_Assert(0, "Dimension out of range. This should never happen."); return "";
	}
}

const char* Asura_Graph_Gadget<Asura_RGBA>::GetLabelForDimension(u_char ucDimension) const
{
	switch(ucDimension)
	{
		case 0:	return "R:";
		case 1:	return "G:";
		case 2:	return "B:";
		case 3:	return "A:";
		default: Asura_Assert(0, "Dimension out of range. This should never happen."); return "";
	}
}

void Asura_Graph_Gadget<float>::RenderBackground() const
{
	RenderWhiteBackground();
}

void Asura_Graph_Gadget<Asura_Vector_2>::RenderBackground() const
{
	RenderWhiteBackground();
}

void Asura_Graph_Gadget<Asura_Vector_3>::RenderBackground() const
{
	RenderWhiteBackground();
}

DEBUGVAR float dbg_fCheckSize = 10;

void Asura_Graph_Gadget<Asura_RGBA>::RenderBackground() const
{
	if(!IsEnabled())
	{
		RenderWhiteBackground();
		return;
	}

	Asura_Vector_2 xBase = GetGraphOrigin();
	// Render checked background
	const float fGraphWidth = m_pxGraphDialog->GetGraphPixelWidth();
	const float fGraphHeight = m_pxGraphDialog->GetGraphPixelHeight();

	for(u_int uX = 0; uX < ( fGraphWidth/ dbg_fCheckSize); ++uX)
	{
		const float fX = uX * dbg_fCheckSize;
		const float fCheckWidth = (fX+dbg_fCheckSize > fGraphWidth) ? (fGraphWidth-fX) : dbg_fCheckSize;
		
		for(u_int uY = 0; uY < (fGraphHeight / dbg_fCheckSize); ++uY)
		{
			const float fY = uY * dbg_fCheckSize;
			const float fCheckHeight = (fY+dbg_fCheckSize > fGraphHeight) ? (fGraphHeight-fY) : dbg_fCheckSize;
						
			Asura_Render::Overlay.TranslucencyMode = ASURA_TRANSLUCENCY_NORMAL;
			Asura_Render::Overlay.Colour = ((uX + uY) % 2) ? 0x00ffffff : 0x00CCCCCC;
			Asura_Render::Overlay.Colour |= Asura_OverlayBrightness;
			Asura_Render::Overlay.SetRectXY(xBase.x + fX, xBase.y + fY, fCheckWidth, fCheckHeight);
			Asura_Render::Overlay.Render();
		}
	}
	Asura_Render::FlushDisplay();
	// Render colour ramp
	for(u_int u = 0; u < m_pxGraphDialog->GetGraphPixelWidth(); ++u)
	{
		Asura_Render::Overlay.TranslucencyMode = ASURA_TRANSLUCENCY_NORMAL;

		// (AAM - 20.02.2008) Instead of clamping, scale down to find the LDR version of the colour
		{
			const float fTime = static_cast<float>(u) / m_pxGraphDialog->GetGraphPixelWidth();
			Asura_RGBA xColour = m_pxGraph->GetValueAtTime(fTime);

			const float fMaxComponent =
				Asura_Maths::Max(xColour.m_fR, Asura_Maths::Max(xColour.m_fG, xColour.m_fB));

			if(fMaxComponent > 1.0f)
			{
				const float fAlpha = xColour.m_fA;
				xColour *= 1.0f / fMaxComponent;
				xColour.m_fA = fAlpha;
			}

			Asura_Render::Overlay.Colour = xColour.ToUIntARGB(true);
		}

		Asura_Render::Overlay.SetRectXY(xBase.x + u, xBase.y, 1.f, m_pxGraphDialog->GetGraphPixelHeight());
		Asura_Render::Overlay.Render();
	}
	Asura_Render::FlushDisplay();
}


void Asura_Graph_Gadget<float>::PlaceCursorOnCurrentPoint()
{
	Asura_Cursor::SetScreenSpaceCoords(
		static_cast<int>(GraphToScreenPosX(m_pxCurrentPoint->m_fTime)),
		static_cast<int>(GraphToScreenPosY(m_pxCurrentPoint->m_xValue)));
}

void Asura_Graph_Gadget<Asura_Vector_2>::PlaceCursorOnCurrentPoint()
{
	Asura_Assert(m_pxCurrentPoint, "Shouldn't try to place cursor when there's no current point");
	if(!m_pxCurrentPoint) return;

	int iX = static_cast<int>(GraphToScreenPosX(m_pxCurrentPoint->m_fTime));
	int iY;
	switch(m_ucCurrentDimension)
	{
		case 0:
			iY = static_cast<int>(GraphToScreenPosY(m_pxCurrentPoint->m_xValue.x));
			break;
		case 1:
			iY = static_cast<int>(GraphToScreenPosY(m_pxCurrentPoint->m_xValue.y));
			break;
		default: Asura_Assert(0, "Dimension out of range. This should never happen."); return;
	}
	Asura_Cursor::SetScreenSpaceCoords(iX, iY);
}

void Asura_Graph_Gadget<Asura_Vector_3>::PlaceCursorOnCurrentPoint()
{
	Asura_Assert(m_pxCurrentPoint, "Shouldn't try to place cursor when there's no current point");
	if(!m_pxCurrentPoint) return;

	int iX = static_cast<int>(GraphToScreenPosX(m_pxCurrentPoint->m_fTime));
	int iY;
	switch(m_ucCurrentDimension)
	{
		case 0:
			iY = static_cast<int>(GraphToScreenPosY(m_pxCurrentPoint->m_xValue.x));
			break;
		case 1:
			iY = static_cast<int>(GraphToScreenPosY(m_pxCurrentPoint->m_xValue.y));
			break;
		case 2:
			iY = static_cast<int>(GraphToScreenPosY(m_pxCurrentPoint->m_xValue.z));
			break;
		default: Asura_Assert(0, "Dimension out of range. This should never happen."); return;
	}
	Asura_Cursor::SetScreenSpaceCoords(iX, iY);
}

void Asura_Graph_Gadget<Asura_RGBA>::PlaceCursorOnCurrentPoint()
{
	Asura_Assert(m_pxCurrentPoint, "Shouldn't try to place cursor when there's no current point");
	if(!m_pxCurrentPoint) return;

	int iX = static_cast<int>(GraphToScreenPosX(m_pxCurrentPoint->m_fTime));
	int iY;
	switch(m_ucCurrentDimension)
	{
		case 0:
			iY = static_cast<int>(GraphToScreenPosY(m_pxCurrentPoint->m_xValue.m_fR));
			break;
		case 1:
			iY = static_cast<int>(GraphToScreenPosY(m_pxCurrentPoint->m_xValue.m_fG));
			break;
		case 2:
			iY = static_cast<int>(GraphToScreenPosY(m_pxCurrentPoint->m_xValue.m_fB));
			break;
		case 3:
		{
			float fYValue = m_pxCurrentPoint->m_xValue.m_fA;

			if (m_bForceNormalisedAlpha)
			{
				// The alpha value goes from 0.0f to 1.0f, but we should draw this line using the full height of the graph if the m_bForceNormalisedAlpha flag is set

				fYValue = (fYValue * (m_fRangeMax - m_fRangeMin)) + m_fRangeMin;
			}

			iY = static_cast<int>(GraphToScreenPosY(fYValue));

			break;
		}
		default: Asura_Assert(0, "Dimension out of range. This should never happen."); return;
	}
	Asura_Cursor::SetScreenSpaceCoords(iX, iY);
}

u_int Asura_Graph_Gadget<float>::GetDimensionColour(u_int uDimension) const
{
	return ASURA_GRAPH_COLOUR_F;
}

u_int Asura_Graph_Gadget<Asura_Vector_2>::GetDimensionColour(u_int uDimension) const
{
	switch(uDimension)
	{
		case 0:	return ASURA_GRAPH_COLOUR_X;
		case 1:	return ASURA_GRAPH_COLOUR_Y;
		default: Asura_Assert(0, "Dimension out of range. This should never happen."); return 0;
	}
}

u_int Asura_Graph_Gadget<Asura_Vector_3>::GetDimensionColour(u_int uDimension) const
{
	switch(uDimension)
	{
		case 0:	return ASURA_GRAPH_COLOUR_X;
		case 1:	return ASURA_GRAPH_COLOUR_Y;
		case 2:	return ASURA_GRAPH_COLOUR_Z;
		default: Asura_Assert(0, "Dimension out of range. This should never happen."); return 0;
	}
}

u_int Asura_Graph_Gadget<Asura_RGBA>::GetDimensionColour(u_int uDimension) const
{
	switch(uDimension)
	{
		case 0:	return ASURA_GRAPH_COLOUR_R;
		case 1:	return ASURA_GRAPH_COLOUR_G;
		case 2:	return ASURA_GRAPH_COLOUR_B;
		case 3:	return ASURA_GRAPH_COLOUR_A;
		default: Asura_Assert(0, "Dimension out of range. This should never happen."); return 0;
	}
}
#endif // #ifdef ASURA_TOOLS
