///////////////////////////////////////////////////////////////////////////////
//
// File   : Asura_MessageTable_Dialog.h
// Author : Dan Clarke
// Date   : 05-Nov-2008
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __ASURA_MESSAGETABLE_DIALOG_H__
#define __ASURA_MESSAGETABLE_DIALOG_H__

#if defined(ASURA_TOOLS)

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include "Interface/Asura_Dialog.h"
#include "Core/Asura_List.h"
#include "Entity/Asura_StaticMessage_System.h"
#include "Core/Asura_Platform_Timers.h"
#include "Misc/Tools_DynamicArray.h"

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

class MessageTable_DragDrop_System
{
	public:
		MessageTable_DragDrop_System::MessageTable_DragDrop_System(class Asura_MessageTable_Dialog& xTable) : m_xTable(xTable), m_bIsDragging(false) {}

		void Begin(int iHitIndex, float fX, float fY);
		void End();
		void Process(float fX, float fY, float& fScrollValue, bool& bScrollChanged);
		void Render();
		void Drop(int iIndex);
		bool DropNormalMessage(int iIndex);
		bool DropSplitterBlockMessage(int iIndex);
		bool DropTimelineTriggerMessage(int iIndex);
		bool IsDragging() { return m_bIsDragging; }

	private:
		class Asura_MessageTable_Dialog& m_xTable;
		bool m_bIsDragging;
		class Asura_MessageTableRow* m_pxSourceRow;
		class Asura_MessageTableRow* m_pxTargetRow;
		float m_fMouseX, m_fMouseY;
		Asura_TickCount m_xDragStartTickCount;
};

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

class Asura_MessageTableRow
{
	friend class Asura_MessageTable_Dialog;

	public:
		enum TypeEnum
		{
			TYPE_OUTGOING_MESSAGE,
			TYPE_OUTGOING_EVENT_HEADER,
			TYPE_INCOMING_MESSAGE,
		};

		Asura_MessageTableRow(Asura_MessageTable_Dialog& xTable);
		virtual ~Asura_MessageTableRow() {}

		void SetSelected(bool bFlag) { m_bIsSelected = bFlag; SetTextColour(bFlag ? 0xffffff00 : 0x0); }
		bool IsSelected() const { return m_bIsSelected; }

		virtual void Render(const float fXOffset, const float fYOffset, const float fRowWidth) = 0;
		virtual void Update(Asura_Vector_2* pxOffset, bool bUpdatePositions, bool bHideControls) = 0;

		virtual bool IsExpanded() const { return true; }

		virtual bool InternalHitTest(float fx, float fy) { return false; }
		//virtual bool IsRenameable() const { return false; }

		virtual void SetTextColour(unsigned int uColour) = 0;

		void RenderRow(const float fXOffset, const float fYOffset, const float fRowWidth);

		virtual TypeEnum GetType() = 0;

		float GetRowYOffset() const { return m_fRowYOffset; }
		void SetRowYOffset(float fRowYOffset) { m_fRowYOffset = fRowYOffset; }

		void SetBlockId(u_short* pusBlockId) { m_pusBlockId = pusBlockId; }
		u_short* GetBlockId() const { return m_pusBlockId; }

		void SetEventId(int iEventId) { m_iEventId = iEventId; }
		int GetEventId() const { return m_iEventId; }

	protected:
		Asura_MessageTable_Dialog& m_xTable;
		bool m_bIsSelected;
		float m_fRowYOffset;
		u_short* m_pusBlockId;
		int m_iEventId;
};

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

class Asura_MessageTableRow_Outgoing : public Asura_MessageTableRow
{
	public:
		Asura_MessageTableRow_Outgoing(Asura_MessageTable_Dialog& xTable, Asura_StaticMessage* pxMessage);
		virtual ~Asura_MessageTableRow_Outgoing();

		virtual void Render(const float fXOffset, const float fYOffset, const float fRowWidth) { RenderRow(fXOffset, fYOffset, fRowWidth); }
		virtual void Update(Asura_Vector_2* pxOffset, bool bUpdatePositions, bool bHideControls);
		virtual TypeEnum GetType() { return TYPE_OUTGOING_MESSAGE; }

		void SetDelay(const float fDelay) { m_fDelay = fDelay; m_fLastDelay = fDelay; }
		void SetTargetText(const Asura_String& xText) { m_pxTargetString->m_xName = xText; }
		void SetProbability(const u_int uProb) { m_uProbability = uProb; m_uLastProbability = uProb; }
		void SetIsDisabled(const bool bIsDisabled) { m_bIsDisabled = bIsDisabled; }
		void SetAvailableMessages(const char** paszNames, const u_short* pausIds, const u_int uNumStrings, bool bStringNeedsFreeing = false);
		void SetMessageDropdownIndex(const int iIndex) { m_iMessageDropdownIndex = iIndex; }
		void SetOutgoingMessagePointer(Asura_StaticMessage* pxMessage) { m_pxOutgoingMessage = pxMessage; }
		void SetSplitterOutputBlockPointer(struct Asura_SplitterBlockOutput* pxBlock) { m_pxSplitterOutputBlock = pxBlock; }
		void SetTimelineTriggerMessageBlock(struct Asura_TimelineTrigger_MessageBlock* pxMessageBlock) { m_pxTimelineMessageBlock = pxMessageBlock; }
		void SetMessageIndex(int iMessageIndex) { m_iMessageIndex = iMessageIndex; }

		const char** CalculateIODropdownStrings();
		void HandleIODropdownOnChange();

		virtual bool InternalHitTest(float fx, float fy);

		Asura_StaticMessage* GetOutgoingMessagePointer() const { return m_pxOutgoingMessage; }
		struct Asura_SplitterBlockOutput* GetSplitterOutputBlockPointer() const { return m_pxSplitterOutputBlock; }
		struct Asura_TimelineTrigger_MessageBlock* GetTimelineTriggerMessageBlock() const { return m_pxTimelineMessageBlock; }
		int GetMessageIndex() const { return m_iMessageIndex; }

		virtual void SetTextColour(unsigned int uColour) { m_pxTargetString->SetColour(uColour); }

	private:
		Asura_DropDown_List_Dialog* m_pxMessageDropdown;
		Asura_StringDisplay_Dialog* m_pxTargetString;
		Asura_DropDown_List_Dialog* m_pxIODropdown;
		Asura_Numerical_Dialog* m_pxDelayNumericalControl;
		Asura_Numerical_Dialog* m_pxProbNumericalControl;
		Asura_Tickbox_Dialog* m_pxDisableTickbox;

		Asura_StaticMessage* m_pxOutgoingMessage;
		const unsigned short* m_pausIds;
		const char** m_paszNames;
		const char** m_pxDropdownStrings;
		u_int m_uNumNames;
		u_int m_uNumIODropdownStrings;

		float m_fDelay;
		u_int m_uProbability;
		bool m_bIsDisabled;
		int m_iMessageDropdownIndex;
		int m_iIODropdownIndex;
		struct Asura_SplitterBlockOutput* m_pxSplitterOutputBlock;
		struct Asura_TimelineTrigger_MessageBlock* m_pxTimelineMessageBlock;

		u_int m_uLastProbability;
		float m_fLastDelay;
		int m_iMessageIndex;

		bool m_bDisableTickboxChanged;
		bool m_bProbNumberControlChanged;
		bool m_bDelayNumberControlChanged;
		bool m_bMessageDropdownChanged;
		bool m_bIODropdownChanged;
		bool m_bStringsNeedsFreeing;
};

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

class Asura_MessageTableRow_Incoming : public Asura_MessageTableRow
{
	public:
		Asura_MessageTableRow_Incoming(Asura_MessageTable_Dialog& xTable);
		virtual ~Asura_MessageTableRow_Incoming();

		virtual void Render(const float fXOffset, const float fYOffset, const float fRowWidth) { RenderRow(fXOffset, fYOffset, fRowWidth); }
		virtual void Update(Asura_Vector_2* pxOffset, bool bUpdatePositions, bool bHideControls);
		virtual TypeEnum GetType() { return TYPE_INCOMING_MESSAGE; }

		void SetMessageText(const Asura_String& xText) { m_pxMessageString->m_xName = xText; }
		void SetSourceText(const Asura_String& xText) { m_pxSourceString->m_xName = xText; }
		void SetReceivedMessageData(class Tools_MessageEditor_ReceivedMessageData* pxMessageData) { m_pxMessageData = pxMessageData; }

		class Tools_MessageEditor_ReceivedMessageData* GetReceivedMessageData() { return m_pxMessageData; }

		virtual void SetTextColour(unsigned int uColour) { m_pxMessageString->SetColour(uColour); m_pxSourceString->SetColour(uColour); }

	private:
		Asura_StringDisplay_Dialog* m_pxMessageString;
		Asura_StringDisplay_Dialog* m_pxSourceString;
		class Tools_MessageEditor_ReceivedMessageData* m_pxMessageData;
};

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

class Asura_MessageTableRow_OutgoingEventHeader : public Asura_MessageTableRow
{
	public:
		Asura_MessageTableRow_OutgoingEventHeader(Asura_MessageTable_Dialog& xTable);
		virtual ~Asura_MessageTableRow_OutgoingEventHeader();

		virtual void Render(const float fXOffset, const float fYOffset, const float fRowWidth);
		virtual void Update(Asura_Vector_2* pOffset, bool bUpdatePositions, bool bHideControls);
		virtual TypeEnum GetType() { return TYPE_OUTGOING_EVENT_HEADER; }

		virtual void SetTextColour(unsigned int uColour) {}
		virtual bool InternalHitTest(float fx, float fy);
		//virtual bool IsRenameable() const;

		void SetText(const Asura_String& xText) { m_xText = xText; }
		void SetExpanded(const bool bFlag) { m_bIsExpanded = bFlag; }
		void SetSplitterOutputBlockPointer(struct Asura_SplitterBlockOutput* pxBlock) { m_pxSplitterOutputBlock = pxBlock; }
		void SetActiveArrayElementPtr(bool* pbActivePtr) { m_pbActiveArrayElementPtr = pbActivePtr; }
		void SetDelayArrayElementPtr(float* pfDelayPtr) { m_pfDelayActiveArrayElementPtr = pfDelayPtr; }
		void SetTimelineTriggerMessageBlock(struct Asura_TimelineTrigger_MessageBlock* pxMessageBlock) { m_pxTimelineMessageBlock = pxMessageBlock; }
		void SetWeight(u_int uWeight) { m_uWeight = uWeight; }
		void SetDelay(float fDelay) { m_fDelay = fDelay; }
		void SetIsDisabled(bool bFlag) { m_bIsDisabled = bFlag; }

		bool IsExpanded() const { return m_bIsExpanded; }

		struct Asura_SplitterBlockOutput* GetSplitterOutputBlockPointer() const { return m_pxSplitterOutputBlock; }
		struct Asura_TimelineTrigger_MessageBlock* GetTimelineTriggerMessageBlock() const { return m_pxTimelineMessageBlock; }
		const Asura_String GetText() { return m_xText; }

	private:
		Asura_String m_xText;
		bool m_bIsExpanded;
		struct Asura_SplitterBlockOutput* m_pxSplitterOutputBlock;
		struct Asura_TimelineTrigger_MessageBlock* m_pxTimelineMessageBlock;
		bool* m_pbActiveArrayElementPtr;
		float* m_pfDelayActiveArrayElementPtr;

		Asura_Numerical_Dialog* m_pxDelayNumericalControl;
		Asura_Numerical_Dialog* m_pxWeightNumericalControl;
		Asura_Tickbox_Dialog* m_pxDisableTickbox;
		Asura_StringDisplay_Dialog* m_pxWeightLabel;

		float m_fDelay;
		u_int m_uWeight;
		bool m_bIsDisabled;

		bool m_bDelayNumberControlChanged;
		bool m_bWeightNumberControlChanged;
		bool m_bDisableTickboxChanged;
};

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

class Asura_MessageTable_Dialog : public Asura_Panel_Dialog
{
	friend class MessageTable_DragDrop_System;

	public:
		enum TypeEnum
		{
			TYPE_NORMAL,
			TYPE_SPLITTER_BLOCK,
			TYPE_TIMELINE_TRIGGER,
		};

		enum TableTypeEnum
		{
			TABLE_TYPE_OUTGOING,
			TABLE_TYPE_INCOMING,
		};

		Asura_MessageTable_Dialog(const Asura_Vector_2& xOffset, float fRowWidth, Asura_Tabs_Dialog* pxParentTabDialog, bool& bIsMessageEditingMode);
		virtual ~Asura_MessageTable_Dialog();

		void SetType(TypeEnum eType) { m_eType = eType; }
		TypeEnum GetType() { return m_eType; }
		TableTypeEnum GetTableType() { return m_eTableType; }

		int RenderList(const Asura_Vector_2& xOrigin, int iPriority);

		virtual void Render(const Asura_Vector_2& xOrigin);
		virtual void Update(const Asura_Vector_2& xOrigin);
		virtual void PerFrameUpdate();

		virtual void RenderHeader(const Asura_Vector_2& xOrigin) = 0;

		virtual bool IsPointInDialog(const Asura_Vector_2& xOrigin,const Asura_Vector_2& xPos) const;

		virtual u_int GetPanelBorderColour() { return m_bIsMessageEditingMode ? 0xff0000 : 0xffffff; }

		void ClearAllRows();
		void AddRow(Asura_MessageTableRow* pxNewRow) { m_xRowList.AddToList(pxNewRow, false); }

		void SetDirty() { m_bOverlayDirty = true; m_bScrollChanged = true; }
		void SetTableVisible(bool bFlag) { m_bIsTableVisible = bFlag; }
		void RequestParentToRecreateTable(bool bBanDelayReordering = false) { m_bRequestParentToRecreateTable = true; m_bBanDelayReordering = bBanDelayReordering; }
		bool IsTableRecreateRequested(bool& bBanDelayReordering) { bBanDelayReordering = m_bBanDelayReordering; m_bBanDelayReordering = false; return m_bRequestParentToRecreateTable; }
		void ResetTableRecreateRequest() { m_bRequestParentToRecreateTable = false; }
		bool IsTableVisible() const { return m_bIsTableVisible; }

		void CollapseAllRows();
		void ExpandAllRows();

		Asura_List<Asura_MessageTableRow>& GetRows() { return m_xRowList; }

		float GetTableWidth() const { return GetWidth(); }

		void SetRowSelChangeCallback(void (*pfnCallback)(bool bSelected, Asura_MessageTableRow* pxRow)) { m_pfnRowSelChangeCallback = pfnCallback; }

		bool GetIsMessageEditingMode() { return m_bIsMessageEditingMode; }

		void DeselectAllRows();
		u_int GetNumSelectedRows();
		bool SwapMessages(int iIndex1, int iIndex2);

		static void LoadTexture();

		//Scrolls down if needed to make a row visible
		void ScrollToRow(int iRowIndex);

	protected:
		typedef Asura_Panel_Dialog PARENT;

		TypeEnum m_eType;
		TableTypeEnum m_eTableType;

		bool m_bOverlayDirty;
		bool m_bIsSourceATrigger;
		bool m_bRequestParentToRecreateTable;

		Asura_MessageTable_Dialog();
		Asura_MessageTable_Dialog(const Asura_MessageTable_Dialog& xOther);
		Asura_MessageTable_Dialog& operator= (const Asura_MessageTable_Dialog& xOther);

		Asura_Tabs_Dialog* m_pxParentTabDialog;

		Asura_List<Asura_MessageTableRow> m_xRowList;

		void (*m_pfnRowSelChangeCallback)(bool bSelected, Asura_MessageTableRow* pxRow);

	private:
		MessageTable_DragDrop_System m_xDragDrop;
		int m_iLastRowSelection;
		bool& m_bIsMessageEditingMode;
		Asura_Scrollbar_Gadget* m_pxVerticalBar;
		float m_fScrollValue;
		bool m_bScrollChanged;
		float m_fClipX;
		float m_fClipY;
		float m_fClipWidth;
		float m_fClipHeight;
		bool m_bBanDelayReordering;
		int m_iScrollNumHiddenRows;
		bool m_bIsTableVisible;
};

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

class Asura_MessageTable_Outgoing_Dialog : public Asura_MessageTable_Dialog
{
	public:
		Asura_MessageTable_Outgoing_Dialog(const Asura_Vector_2& xOffset, float fRowWidth, Asura_Tabs_Dialog* pxParentTabDialog, bool& bIsMessageEditingMode) : Asura_MessageTable_Dialog(xOffset, fRowWidth, pxParentTabDialog, bIsMessageEditingMode) { m_eTableType = TABLE_TYPE_OUTGOING; }
		virtual ~Asura_MessageTable_Outgoing_Dialog() {}

		virtual void Render(const Asura_Vector_2& xOrigin) { PARENT::Render(xOrigin); }
		virtual void Update(const Asura_Vector_2& xOrigin) { PARENT::Update(xOrigin); }
		virtual void PerFrameUpdate() { PARENT::PerFrameUpdate(); }
		virtual void RenderHeader(const Asura_Vector_2& xOrigin);

	private:
		typedef Asura_MessageTable_Dialog PARENT;

		Asura_MessageTable_Outgoing_Dialog();
		Asura_MessageTable_Outgoing_Dialog(const Asura_MessageTable_Outgoing_Dialog& xOther);
		Asura_MessageTable_Outgoing_Dialog& operator= (const Asura_MessageTable_Outgoing_Dialog& xOther);

};

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

class Asura_MessageTable_Incoming_Dialog : public Asura_MessageTable_Dialog
{
	public:
		Asura_MessageTable_Incoming_Dialog(const Asura_Vector_2& xOffset, float fRowWidth, Asura_Tabs_Dialog* pxParentTabDialog, bool& bIsMessageEditingMode) : Asura_MessageTable_Dialog(xOffset, fRowWidth, pxParentTabDialog, bIsMessageEditingMode) { m_eTableType = TABLE_TYPE_INCOMING; }
		virtual ~Asura_MessageTable_Incoming_Dialog() {}

		virtual void Render(const Asura_Vector_2& xOrigin) { PARENT::Render(xOrigin); }
		virtual void Update(const Asura_Vector_2& xOrigin) { PARENT::Update(xOrigin); }
		virtual void PerFrameUpdate() { PARENT::PerFrameUpdate(); }
		virtual void RenderHeader(const Asura_Vector_2& xOrigin);

	private:
		typedef Asura_MessageTable_Dialog PARENT;

		Asura_MessageTable_Incoming_Dialog();
		Asura_MessageTable_Incoming_Dialog(const Asura_MessageTable_Incoming_Dialog& xOther);
		Asura_MessageTable_Incoming_Dialog& operator= (const Asura_MessageTable_Incoming_Dialog& xOther);
};

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#endif	// defined(ASURA_TOOLS)

#endif   // ifndef __ASURA_MESSAGETABLE_DIALOG_H__

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

