//
// Asura_MultiLineString_Dialog.cpp
//
// Created by Richard Rice - 04/01/2008
// 
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

#ifdef ASURA_TOOLS

#include "Interface/Asura_MultiLineString_Dialog.h"

#include "Core/Asura_UserInput.h"
#include "Core/Asura_Timers.h"
#include "Core/Memory/Asura_Mem.h"
#include "Debugging/Asura_Assert.h"
#include "Interface/Asura_Cursor.h"
#include "Interface/Asura_Overlay_System.h"
#include "Rendering/Asura_Render.h"
#include "Rendering/Asura_Text_Output.h"
#include "Rendering/Asura_Font_System.h"

#include "Core/Asura_Windows.h"

extern unsigned int Asura_OverlayBrightness;

//////////////////////////////////////////////////////////////////////////////////////////////
/*------------------------------**
** Asura_MultiLineString_Dialog **
**------------------------------*/

Asura_MultiLineString_Dialog::Asura_MultiLineString_Dialog(const Asura_Vector_2& xOffset, class Asura_String* pxString, float fWidth, float fHeight, bool* pbChangedFlag)
:Asura_Dialog("",xOffset)
{
	//Determine space for the main string gadget view
	m_fViewWidth = fWidth - Asura_Scrollbar_Gadget::GetDefaultThickness(false);
	m_fViewHeight = fHeight;
	//Make sure there is going to be at least some area
	if(m_fViewWidth < 2.0f || m_fViewHeight < 2.0f)
	{
		Asura_Assert(0,"MultiLineString dialog too small");
		m_fViewWidth = Asura_Maths::Max(m_fViewWidth,2.0f);
		m_fViewHeight = Asura_Maths::Max(m_fViewHeight,2.0f);

		fWidth = m_fViewWidth + Asura_Scrollbar_Gadget::GetDefaultThickness(false);
		fHeight = m_fViewHeight;
	}
	
	m_pxStringGadget = new Asura_MultiLineString_Gadget(m_xOffset,this,pxString,m_fViewWidth,m_fViewHeight,pbChangedFlag);
	AddGadget(m_pxStringGadget);

	Asura_Vector_2 xPos;

	//Create the scrollbar
	m_fVScrollPos = 0.0f;
	xPos = m_xOffset;
	xPos.x += m_fViewWidth;
	m_pxVerticalBar = new Asura_Scrollbar_Gadget(xPos,&m_fVScrollPos,0.0f,0.0f,false,m_fViewHeight,0,0);
	AddGadget(m_pxVerticalBar);

	m_xMaxCoord.x = m_xOffset.x + fWidth;
	m_xMaxCoord.y = m_xOffset.y + fHeight;

	UpdateScrollbar();
}

void Asura_MultiLineString_Dialog::Render(const Asura_Vector_2& xOrigin)
{
	Asura_Dialog::Render(xOrigin);

	if(!IsHidden())
	{
		//draw a box around the control
		float fX = (float)Asura_Maths::FloatToInt_Near((xOrigin.x+m_xOffset.x)*ASURA_TEXT_OUTPUT_FONT_WIDTH);
		float fY = (float)Asura_Maths::FloatToInt_Near((xOrigin.y+m_xOffset.y)*ASURA_TEXT_OUTPUT_FONT_HEIGHT);
		float fWidth = (m_xMaxCoord.x - m_xOffset.x) * ASURA_TEXT_OUTPUT_FONT_WIDTH;
		float fHeight = (m_xMaxCoord.y - m_xOffset.y) * ASURA_TEXT_OUTPUT_FONT_HEIGHT;


		u_int uColour = Asura_OverlayBrightness|ASURA_GADGET_GREYED_COLOUR;
		Asura_Render::Render2DLine(fX,fY-1.0f,fX+fWidth+1.0f,fY-1.0f,uColour);
		Asura_Render::Render2DLine(fX+fWidth,fY,fX+fWidth,fY+fHeight,uColour);
		Asura_Render::Render2DLine(fX,fY+fHeight,fX+fWidth+1.0f,fY+fHeight,uColour);
		Asura_Render::Render2DLine(fX,fY,fX,fY+fHeight,uColour);
	}		
}

void Asura_MultiLineString_Dialog::UpdateScrollbar()
{
	//Update the vertical scrollbar
	float fHeightRequired = m_pxStringGadget->GetTextPixelHeight()/ASURA_TEXT_OUTPUT_FONT_HEIGHT;
	float fMaxScroll = Asura_Maths::Max(fHeightRequired - m_fViewHeight,0.0f);
	m_fVScrollPos = Asura_Maths::Min(m_fVScrollPos,fMaxScroll);

	float fSliderLength = m_fViewHeight;
	if(fMaxScroll > 0)
	{
		fSliderLength = (m_fViewHeight/fHeightRequired)*m_fViewHeight;
	}

	m_pxVerticalBar->SetSliderLength(fSliderLength);
	m_pxVerticalBar->SetMax(fMaxScroll);
}

void Asura_MultiLineString_Dialog::PerFrameUpdate()
{
	Asura_Dialog::PerFrameUpdate();

	UpdateScrollbar();
}

//////////////////////////////////////////////////////////////////////////////////////////////
/*------------------------------**
** Asura_MultiLineString_Gadget **
**------------------------------*/

Asura_MultiLineString_Gadget::Asura_MultiLineString_Gadget(const Asura_Vector_2& xOffset, const Asura_MultiLineString_Dialog* pxStringDialog,Asura_String* pxString, float fWidth, float fHeight, bool* pbChangedFlag)
: m_wszTextBuffer(0)
, m_iTextBufferSize(0)
, m_iCursorPos(0)
, m_iSelectionStart(0)
, m_bDefaultFocus(0)
, m_bDiscardChanges(false)
, m_bReadOnly(false)
, m_pxStringDialog(pxStringDialog)
, m_pxString(pxString)
, m_pbChangedFlag(pbChangedFlag)
, m_iNumLines(0)
, m_iLineArraySize(0)
, m_pxLines(0)
, m_fTextHeight(0.0f)
{
	m_xOffset = xOffset;
	m_fWidth = fWidth;
	m_fHeight = fHeight;

	FillTextBuffer();

}

Asura_MultiLineString_Gadget::~Asura_MultiLineString_Gadget()
{
	delete [] m_wszTextBuffer;
	delete [] m_pxLines;
}

void Asura_MultiLineString_Gadget::Update( const Asura_Vector_2& xOrigin )
{
	if(m_bDefaultFocus)
	{
		//grab keyboard focus
		Asura_Overlay_System::TakeFocus(this,OverlayFocusType_Keyboard);
		m_bDefaultFocus = false;
	}

	if( m_bIsMouseOver && m_bIsClickable && !m_bReadOnly && Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_MOUSEBUTTON1) )
	{
		Asura_Overlay_System::TakeFocus(this,OverlayFocusType_Keyboard);
        m_bClickedThisFrame = true;
	}

	if(m_iFocusFlags)
	{
		if(m_bReadOnly)
		{
			m_bDiscardChanges = true;
			Asura_Overlay_System::ReleaseFocus();
		}
		else
		{
			HandleKeyboardInput();
		}
	}
}

void Asura_MultiLineString_Gadget::Render( const Asura_Vector_2& xOrigin )
{
	if(!(m_iFocusFlags & OverlayFocusType_Keyboard))
	{
		FillTextBuffer();
	}

	const bool bEditing = (m_iFocusFlags & OverlayFocusType_Keyboard)!=0;

	//Get the rectangle for rendering
	Asura_Bounding_Rectangle xRect;
	xRect.MinX = (xOrigin.x + m_xOffset.x)*ASURA_TEXT_OUTPUT_FONT_WIDTH;
	xRect.MinY = (xOrigin.y + m_xOffset.y)*ASURA_TEXT_OUTPUT_FONT_HEIGHT;
	xRect.MaxX = xRect.MinX + m_fWidth*ASURA_TEXT_OUTPUT_FONT_WIDTH;
	xRect.MaxY = xRect.MinY + m_fHeight*ASURA_TEXT_OUTPUT_FONT_HEIGHT;

	xRect.MinX = (float)Asura_Maths::FloatToInt_Near(xRect.MinX);
	xRect.MinY = (float)Asura_Maths::FloatToInt_Near(xRect.MinY);
	xRect.MaxX = (float)Asura_Maths::FloatToInt_Near(xRect.MaxX);
	xRect.MaxY = (float)Asura_Maths::FloatToInt_Near(xRect.MaxY);

	float fPixelHeight = xRect.MaxY - xRect.MinY;
    Asura_Font_System::SetFont(Asura_Text_Output::GetSmallFontHandle());

	float fVScroll = m_pxStringDialog->GetVerticalScrollPos();

	float fYOffset = (-fVScroll) * ASURA_TEXT_OUTPUT_FONT_HEIGHT;
	fYOffset = (float) Asura_Maths::FloatToInt_Near(fYOffset);

	const float fLineHeight = Asura_Font_System::GetFontHeight();

	if(bEditing && m_iSelectionStart != m_iCursorPos)
	{
		//draw selection highlight

		//get range to highlight
		int iStartIndex = Asura_Maths::Min(m_iSelectionStart,m_iCursorPos);
		int iEndIndex = Asura_Maths::Max(m_iSelectionStart,m_iCursorPos);

		float fStartX,fEndX;
		int iStartLine,iEndLine;
		GetCharacterPosition(iStartIndex,false,fStartX,iStartLine);
		GetCharacterPosition(iEndIndex,true,fEndX,iEndLine);

		for(int iLine=iStartLine;iLine<=iEndLine;iLine++)
		{
			//skip if the line isn't in the visible area
			if(fYOffset+fLineHeight*(iLine+1)<=0.0f) continue;
			if(fYOffset+fLineHeight*iLine>=fPixelHeight) break;

			//get the extents of the line being highlighted in screen coordinates
			float fLeft = (xOrigin.x + m_xOffset.x)*ASURA_TEXT_OUTPUT_FONT_WIDTH;
			fLeft += (iLine==iStartLine) ? fStartX : 0.0f;

			float fRight = (xOrigin.x + m_xOffset.x)*ASURA_TEXT_OUTPUT_FONT_WIDTH;
			fRight += (iLine==iEndLine) ? fEndX : m_pxLines[iLine].m_fWidth;

			float fTop = (xOrigin.y + m_xOffset.y)*ASURA_TEXT_OUTPUT_FONT_HEIGHT;
			fTop += iLine*fLineHeight + fYOffset;

			float fBottom = fTop + fLineHeight;
			
			//clamp to the rectangle
			fLeft = Asura_Maths::ClampToRange(fLeft, xRect.MinX, xRect.MaxX);
			fRight = Asura_Maths::ClampToRange(fRight, xRect.MinX, xRect.MaxX);
			fTop = Asura_Maths::ClampToRange(fTop, xRect.MinY, xRect.MaxY);
			fBottom = Asura_Maths::ClampToRange(fBottom, xRect.MinY, xRect.MaxY);
			 
			
			//and draw a whit background over this area
			Asura_Render::Overlay.Colour = 0xafffffff;
			Asura_Render::Overlay.TextureIndex = ASURA_TEXTURE_NONE;
			Asura_Render::Overlay.TranslucencyMode = ASURA_TRANSLUCENCY_NORMAL;
			Asura_Render::Overlay.SetRectXY(fLeft,fTop,fRight-fLeft,fBottom-fTop);
			Asura_Render::Overlay.Render();
			Asura_Render::FlushDisplay();

		}
		
	}

	u_int uColour;
	if(IsEnabled())
	{
		//if(bEditing)
		//{
		//	uColour = ASURA_GADGET_HIGHLIGHT_COLOUR;
		//}
		//else
		//{
		//	uColour = ASURA_GADGET_DEFAULT_COLOUR;
		//}
		uColour = ASURA_GADGET_DEFAULT_COLOUR;
	}
	else
	{
		uColour = ASURA_GADGET_DISABLED_COLOUR;
	}
	uColour |= Asura_OverlayBrightness;

	//render each line
	for(int iLine=0; iLine<m_iNumLines; iLine++)
	{
		//skip if the line isn't in the visible area
		if(fYOffset+fLineHeight*(iLine+1)<=0.0f) continue;
		if(fYOffset+fLineHeight*iLine>=fPixelHeight) break;

		const Asura_UnicodeChar* wszText = m_wszTextBuffer + m_pxLines[iLine].m_iStart;
		Asura_UnicodeChar* pwcTerminator = m_wszTextBuffer + m_pxLines[iLine].m_iEnd+1;
		Asura_UnicodeChar wcCharCopy = *pwcTerminator;
		*pwcTerminator = 0;
        
		Asura_Font_System::PrintLongString(wszText, &xRect, 0.0f, fYOffset + iLine*fLineHeight, 1.0f, 1.0f, uColour, ASURA_TEXTFORMAT_RELATIVETORECT);

		*pwcTerminator = wcCharCopy;
	}

	//render the cursor if editing
	if(bEditing)
	{
		if(Asura_Maths::FloatToInt(Asura_Timers::GetLocalUnscaledTime()*4.0f)%2)
		{
			float fX,fY;
			int iLine;
			GetCharacterPosition(m_iCursorPos, false, fX, iLine);

			fX = Asura_Maths::Max(fX, fX-1.0f);
			fY = iLine*fLineHeight;

			Asura_Font_System::PrintLongString(L"|", &xRect, fX, fYOffset + fY, 1.0f, 1.0f, uColour, ASURA_TEXTFORMAT_RELATIVETORECT);
		}
	}


    Asura_Gadget::Render( xOrigin );
}


void Asura_MultiLineString_Gadget::TakeTabFocus()
{
	//grab keyboard focus
	Asura_Overlay_System::TakeFocus(this,OverlayFocusType_Keyboard);
}

void Asura_MultiLineString_Gadget::FocusGained(int focusType)
{
	Asura_Gadget::FocusGained(focusType);

	
	if(focusType & OverlayFocusType_Keyboard)
	{
		FillTextBuffer();
		
		m_bDiscardChanges = false;
		m_iCursorPos = wcslen(m_wszTextBuffer);
		m_iSelectionStart = 0;
	}
		
}

void Asura_MultiLineString_Gadget::FocusLost(int focusType)
{
	Asura_Gadget::FocusLost(focusType);

	
	if(focusType & OverlayFocusType_Keyboard)
	{
		if(!m_bDiscardChanges)
		{
			if(ReadFromTextBuffer())
			{
				if(m_pbChangedFlag) *m_pbChangedFlag = true;
			}
		}
	}
}


void Asura_MultiLineString_Gadget::HandleKeyboardInput()
{
	Asura_Assert(m_iFocusFlags,"");

	int iLength = wcslen(m_wszTextBuffer);

	//check for copy and paste
	if(Asura_UserInput::KeyPressed(ASURA_KEYID_CTRL))
	{
		if(Asura_UserInput::DebouncedKeyPressed('c'))
		{
			CopyText();
			return;
		}
		if(Asura_UserInput::DebouncedKeyPressed('x'))
		{
			CopyText();
			DeleteSelection();
			FindSplitPoints();
			return;
		}
		if(Asura_UserInput::DebouncedKeyPressed('v'))
		{
			PasteText();
			FindSplitPoints();
			return;
		}
	}

	if(IsRepeatableKeyPressed(ASURA_KEYID_LEFT))
	{
		//move cursor left
		m_iCursorPos--;
		if(m_iCursorPos<0)m_iCursorPos = 0;
		
		//clear selection if shift not held
		if(!Asura_UserInput::KeyPressed(ASURA_KEYID_SHIFT)) m_iSelectionStart = m_iCursorPos;
	}
	else if(IsRepeatableKeyPressed(ASURA_KEYID_RIGHT))
	{
		
		//move cursor right
		m_iCursorPos++;
		if(m_iCursorPos>iLength)m_iCursorPos = iLength;

		//clear selection if shift not held
		if(!Asura_UserInput::KeyPressed(ASURA_KEYID_SHIFT)) m_iSelectionStart = m_iCursorPos;
	}
	else if(IsRepeatableKeyPressed(ASURA_KEYID_UP))
	{
		
		//move cursor up
		int iLine;
		float fX;
		GetCharacterPosition(m_iCursorPos,false,fX,iLine);
		if(iLine > 0)
		{
			m_iCursorPos = GetIndexFromPosition(iLine-1,fX);
		}

		//clear selection if shift not held
		if(!Asura_UserInput::KeyPressed(ASURA_KEYID_SHIFT)) m_iSelectionStart = m_iCursorPos;
	}
	else if(IsRepeatableKeyPressed(ASURA_KEYID_DOWN))
	{
		//move cursor down
		int iLine;
		float fX;
		GetCharacterPosition(m_iCursorPos,false,fX,iLine);
		if(iLine < m_iNumLines-1)
		{
			m_iCursorPos = GetIndexFromPosition(iLine+1,fX);
		}

		//clear selection if shift not held
		if(!Asura_UserInput::KeyPressed(ASURA_KEYID_SHIFT)) m_iSelectionStart = m_iCursorPos;
	}
	else if(Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_HOME))
	{
		if(Asura_UserInput::KeyPressed(ASURA_KEYID_CTRL))
		{
			//move cursor to start of string
			m_iCursorPos = 0;
		}
		else
		{
			int iLine;
			float fX;
			GetCharacterPosition(m_iCursorPos,false,fX,iLine);
			m_iCursorPos = GetIndexFromPosition(iLine, 0.0f);
		}

		//clear selection if shift not held
		if(!Asura_UserInput::KeyPressed(ASURA_KEYID_SHIFT)) m_iSelectionStart = m_iCursorPos;
	}
	else if(Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_END))
	{
		if(Asura_UserInput::KeyPressed(ASURA_KEYID_CTRL))
		{
			//move cursor to end of string
			m_iCursorPos = iLength;
		}
		else
		{
			int iLine;
			float fX;
			GetCharacterPosition(m_iCursorPos,false,fX,iLine);

			m_iCursorPos = m_pxLines[iLine].m_iEnd;

			if((*(m_wszTextBuffer + m_iCursorPos + 1) == '\0'))
			{
				m_iCursorPos++;
			}
		}

		//clear selection if shift not held
		if(!Asura_UserInput::KeyPressed(ASURA_KEYID_SHIFT)) m_iSelectionStart = m_iCursorPos;
	}
	else if(IsRepeatableKeyPressed(ASURA_KEYID_DEL))
	{
		if(IsTextSelected())
		{
			DeleteSelection();
		}
		else
		{
			//delete the character at the cursor
			for(int i=m_iCursorPos;i<iLength;i++)
			{
				m_wszTextBuffer[i] = m_wszTextBuffer[i+1];
			}
		}
		FindSplitPoints();

	}
	else if(IsRepeatableKeyPressed(ASURA_KEYID_BACKSPACE))
	{
		if(IsTextSelected())
		{
			DeleteSelection();
		}
		else
		{
			//delete the character before the cursor
			if(m_iCursorPos>0)
			{
				for(int i=m_iCursorPos-1;i<iLength;i++)
				{
					m_wszTextBuffer[i] = m_wszTextBuffer[i+1];
				}
				m_iCursorPos--;
				m_iSelectionStart = m_iCursorPos;
			}
		}
		FindSplitPoints();

	}
	else if(Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_ENTER) || Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_NUMPADENTER))
	{
		if(Asura_UserInput::KeyPressed(ASURA_KEYID_CTRL) || Asura_UserInput::KeyPressed(ASURA_KEYID_SHIFT))
		{
			//enter with a modifier key adds a newline to the text
			InsertCharacter(L'\n');
			FindSplitPoints();
			Asura_Overlay_System::s_bUsedKeypressThisFrame = true;
		}
		else
		{
			//enter on its own accepts the current text and ends editing
			Asura_Overlay_System::ReleaseFocus();
			Asura_Overlay_System::s_bUsedKeypressThisFrame = true;
		}
	}
	else if(Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_ESCAPE))
	{
		m_bDiscardChanges = true;
		Asura_Overlay_System::ReleaseFocus();
		Asura_Overlay_System::s_bUsedKeypressThisFrame = true;
	}
	else
	{
		//Check the windows input buffer for the characters pressed
		u_int uInputBufferSize = Asura_Windows::GetSizeOfInputBuffer();
		const Asura_UnicodeChar* wszInputBuffer = Asura_Windows::GetInputBuffer();

		for(u_int i=0;i<uInputBufferSize;i++)
		{
			Asura_UnicodeChar wKeyPressed = (char)wszInputBuffer[i];

			//Ignore some characters that we don't want. Mainly to avoid backspaces from being printed here.
			if(wKeyPressed >=0 && wKeyPressed < L' ') continue;

			InsertCharacter(wKeyPressed);
			Asura_Overlay_System::s_bUsedKeypressThisFrame = true;
		}

		if(uInputBufferSize)
		{
			FindSplitPoints();
		}
	}
}

void Asura_MultiLineString_Gadget::InsertCharacter(Asura_UnicodeChar wcKeyPressed)
{
	if(IsTextSelected())
	{
		//typing a new character causes the selection to be deleted
		DeleteSelection();
	}
	int iLength = wcslen(m_wszTextBuffer);

	if(iLength+1 >= m_iTextBufferSize)
	{
		//enlarge the buffer
		int iNewTextBufferSize = iLength+16;

		Asura_UnicodeChar* wszNewTextBuffer = new Asura_UnicodeChar[iNewTextBufferSize];
		wcscpy(wszNewTextBuffer, m_wszTextBuffer);

		delete [] m_wszTextBuffer;
		m_wszTextBuffer = wszNewTextBuffer;
		m_iTextBufferSize = iNewTextBufferSize;
	}

	for(int i=iLength+1;i>m_iCursorPos;i--)
	{
		m_wszTextBuffer[i] = m_wszTextBuffer[i-1];
	}

	m_wszTextBuffer[m_iCursorPos++] = wcKeyPressed;
	m_iSelectionStart = m_iCursorPos;
}

bool Asura_MultiLineString_Gadget::IsRepeatableKeyPressed(int iKey)
{
	if(Asura_UserInput::DebouncedKeyPressed(iKey))
	{
		//debounced presses always accepted. Start timer before next press
		m_fRepeatTimer = 0.3f;
		m_iRepeatKey = iKey;
		return true;
	}
	else if(iKey == m_iRepeatKey)
	{
		if(Asura_UserInput::KeyPressed(iKey))
		{
			//key still being held down ,see if it is time for a repeat
			m_fRepeatTimer -= Asura_Timers::GetFrameTime();
			if(m_fRepeatTimer <= 0.0f)
			{
				m_fRepeatTimer = 0.08f;
				return true;	
			}
		}
		else
		{
			m_iRepeatKey = ASURA_KEYID_VOID;
		}
	}
	return false;
}

void Asura_MultiLineString_Gadget::DeleteSelection()
{
	//delete all characters in the selection , and move the cursor to the start
	int iCharsToDelete = Asura_Maths::Abs(m_iSelectionStart-m_iCursorPos);
	m_iCursorPos = Asura_Maths::Min(m_iCursorPos,m_iSelectionStart);
	m_iSelectionStart = m_iCursorPos;
	
	int iLength = wcslen(m_wszTextBuffer);
	for(int i=m_iCursorPos;(i+iCharsToDelete-1)<iLength;i++)
	{
		m_wszTextBuffer[i] = m_wszTextBuffer[i+iCharsToDelete];
	}	
}

bool Asura_MultiLineString_Gadget::IsTextSelected() const
{
	return m_iCursorPos != m_iSelectionStart;
}

void Asura_MultiLineString_Gadget::CopyText() const
{
	int iCopyStart = Asura_Maths::Min(m_iSelectionStart,m_iCursorPos);
	int iCharsSelected = Asura_Maths::Abs(m_iSelectionStart-m_iCursorPos);
	int iCharsToCopy = iCharsSelected;


	//copy to windows clipboard
	if( OpenClipboard(NULL) )
	{
		HGLOBAL hMem = GlobalAlloc( GMEM_MOVEABLE, iCharsToCopy + 1 );
		if( hMem )
		{
			char* szOut = reinterpret_cast<char*>( GlobalLock(hMem) );
			Asura_PrintfToBfr(szOut, "%S", iCharsToCopy, false, m_wszTextBuffer+iCopyStart);
			szOut[iCharsToCopy] = 0;
			GlobalUnlock( hMem );

			EmptyClipboard();
			SetClipboardData( CF_TEXT, hMem );
		}

		CloseClipboard();
	}

}

void Asura_MultiLineString_Gadget::PasteText()
{
	//pasting deletes the old selection
	DeleteSelection();


	//copy from the windows clipboard
	if( OpenClipboard(NULL) )
	{
		HANDLE hMem = GetClipboardData(CF_TEXT);
		if( hMem )
		{
			char* szPasteBuffer = reinterpret_cast<char*>( GlobalLock(hMem) );

			//determine the number of characters to move
			int iCharsToCopy = strlen(szPasteBuffer);

			//enlarge buffer if needed
			int iLength = wcslen(m_wszTextBuffer);
			int iSpaceRequired = iCharsToCopy+iLength+1;
			if( iSpaceRequired > m_iTextBufferSize)
			{
				Asura_UnicodeChar* wszNewTextBuffer = new Asura_UnicodeChar[iSpaceRequired];
				wcscpy(wszNewTextBuffer, m_wszTextBuffer);

				delete [] m_wszTextBuffer;
				m_wszTextBuffer = wszNewTextBuffer;
				m_iTextBufferSize = iSpaceRequired;
			}


			//move the characters after the cursor up to make room
			for(int i=iLength;i>=m_iCursorPos;i--)
			{
				m_wszTextBuffer[i+iCharsToCopy] = m_wszTextBuffer[i];
			}

			//can now paste the text in the space left
			Asura_PrintfToBfr(m_wszTextBuffer+m_iCursorPos, L"%S", iCharsToCopy, false, szPasteBuffer);

			//move the cursor to the end of the pasted text
			m_iCursorPos += iCharsToCopy;
			m_iSelectionStart = m_iCursorPos;

			GlobalUnlock( hMem );

		}

		CloseClipboard();
	}

}

void Asura_MultiLineString_Gadget::FillTextBuffer()
{
	int iSpaceRequired = m_pxString->GetLength()+1;

	if(iSpaceRequired > m_iTextBufferSize)
	{
		delete [] m_wszTextBuffer;
		m_wszTextBuffer = new Asura_UnicodeChar[iSpaceRequired];
		m_iTextBufferSize = iSpaceRequired;
	}

	Asura_PrintfToBfr(m_wszTextBuffer, L"%S", m_iTextBufferSize, true, m_pxString->IsNull() ? "" : m_pxString->GetString());
	FindSplitPoints();
}

bool Asura_MultiLineString_Gadget::ReadFromTextBuffer()
{
	int iLength = wcslen(m_wszTextBuffer);
	char* szTemp = new char[iLength+1];
	Asura_PrintfToBfr(szTemp, "%S", iLength+1, true, m_wszTextBuffer);

	bool bChanged = m_pxString->Compare(szTemp,false)!=0;

	*m_pxString = szTemp;
	delete [] szTemp;
	return bChanged;
}

void Asura_MultiLineString_Gadget::FindSplitPoints()
{
	Asura_Assert(m_wszTextBuffer,"");
	m_iNumLines = 0;

    Asura_Font_System::SetFont(Asura_Text_Output::GetSmallFontHandle());

	const float fMaxWidth = m_fWidth*ASURA_TEXT_OUTPUT_FONT_WIDTH;
	const Asura_UnicodeChar* pwcLineStart = m_wszTextBuffer;

	while(*pwcLineStart)
	{
		float fLineWidth = 0.0f;
		int iNumWords = 0;
		const Asura_UnicodeChar* pwcLineEnd = Asura_Font_System::FindSplitPoint(pwcLineStart, fMaxWidth, 1.0f, fLineWidth, iNumWords);

		//add an entry for this line
		if(m_iNumLines == m_iLineArraySize)
		{
			int iNewLineArraySize = m_iLineArraySize+16;
			LineData* pxNewLines = new LineData[iNewLineArraySize];
			if(m_iLineArraySize)
			{
				Asura_Mem::Copy(pxNewLines, m_pxLines, sizeof(LineData)*m_iLineArraySize);
			}

			delete [] m_pxLines;
			m_pxLines = pxNewLines;
			m_iLineArraySize = iNewLineArraySize;
		}

		m_pxLines[m_iNumLines].m_iStart = pwcLineStart - m_wszTextBuffer;
		m_pxLines[m_iNumLines].m_iEnd = (pwcLineEnd - m_wszTextBuffer)-1;
		m_pxLines[m_iNumLines].m_fWidth = fLineWidth;
		m_iNumLines++;

		if(!(*pwcLineEnd) && (*(pwcLineEnd - 1) == '\n'))
		{
			m_pxLines[m_iNumLines].m_iStart = pwcLineEnd - m_wszTextBuffer;
			m_pxLines[m_iNumLines].m_iEnd = (pwcLineEnd - m_wszTextBuffer)-1;
			m_pxLines[m_iNumLines].m_fWidth = 0.0f;
			m_iNumLines++;
		}

		pwcLineStart = pwcLineEnd;
	}

	m_fTextHeight = Asura_Font_System::GetFontHeight()*Asura_Maths::Max(1,m_iNumLines);
}

void Asura_MultiLineString_Gadget::GetCharacterPosition(int iIndex, bool bRight,float& fX,int& iLine) 
{
	fX = 0.0f;
	//find the line that the character belongs to
	for(iLine=0;iLine<m_iNumLines;iLine++)
	{
		if(m_pxLines[iLine].m_iStart<=iIndex && m_pxLines[iLine].m_iEnd>=iIndex)
		{
			
			//count up the width until the character is reached
			const Asura_UnicodeChar* pwcChar = m_wszTextBuffer + m_pxLines[iLine].m_iStart;

			for(int i=m_pxLines[iLine].m_iStart; i<iIndex; i++,pwcChar++)
			{
				fX += Asura_Font_System::GetWidthOfChar(*pwcChar);
			}

			//add the character being checked if the right side is wanted
			if(bRight)
			{
				fX += Asura_Font_System::GetWidthOfChar(*pwcChar);
			}
			return;
		}
	}
	if(m_iNumLines && iIndex>0)
	{
		//get the position at the end of the string
		iLine = m_iNumLines-1;
		fX = m_pxLines[iLine].m_fWidth;
 	}
}

int Asura_MultiLineString_Gadget::GetIndexFromPosition(int iLine , float fX)
{
	if(iLine<0) return 0;
	if(iLine>=m_iNumLines) return wcslen(m_wszTextBuffer)+1;

	float fPos = 0.0f;
	int iIndex;

	const Asura_UnicodeChar* pwcChar = m_wszTextBuffer + m_pxLines[iLine].m_iStart;
	for(iIndex=m_pxLines[iLine].m_iStart; iIndex<=m_pxLines[iLine].m_iEnd; iIndex++,pwcChar++)
	{
		if(*pwcChar == L'\n') break;
		float fCharWidth = Asura_Font_System::GetWidthOfChar(*pwcChar);
		if(fPos + fCharWidth >= fX)
		{
			//in the middle of this character , go for index to left/right depending on which is closer
			if(fPos + fCharWidth/2.0f <= fX) iIndex++;
			break;
		}
		fPos += fCharWidth;
	}
	if(iLine<m_iNumLines-1)
	{
		iIndex = Asura_Maths::Min(iIndex,m_pxLines[iLine].m_iEnd);
	}

	return iIndex;

}


#endif //#ifdef ASURA_TOOLS
