
///////////////////////////////////////////////////////////////////////////////
//
// File   : Asura_Table_Dialog.cpp
// Author : Andy Tate
// Date   : 26-Apr-2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

#if defined(ASURA_TOOLS)

// This file's header...
#include "Interface/Asura_Table_Dialog.h"

// Asura independent header-files...
#include "Interface/Asura_Cursor.h"
#include "Interface/Asura_Gadget.h"
#include "Interface/Asura_Overlay.h"
#include "Interface/Asura_Overlay_System.h"
#include "Rendering/Asura_Overlay_Quad.h"
#include "Rendering/Asura_Text_Output.h"
#include "Rendering/Asura_Render.h"

#include "Core/Asura_UserInput.h"

extern unsigned int Asura_OverlayBrightness;

////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////

DEBUGVAR Asura_Vector_2	dbg_xMinTableViewSize(5.0f, 5.0f);


////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////

bool				Asura_Table_Dialog::s_bSortBackwards		= false;
u_int				Asura_Table_Dialog::s_uSortIndex			= 0;
Asura_Table_Dialog*	Asura_Table_Dialog::s_pxSortTableDialogue	= NULL;


//////////////////////////////////////////////////////////////////////////////
//
// Asura_TableCell_TextDialog
//
//////////////////////////////////////////////////////////////////////////////

Asura_TableCell_TextDialog* Asura_TableCell_TextDialog::Create(Asura_Table_Dialog& xTableDialog,
															   const char* szName,
															   const Asura_Vector_2& xOffset,
															   const Asura_Vector_2& xMinCellSize,
															   const int iRow,
															   const int iColumn)
{
	Asura_TableCell_TextDialog*	pxNewDialogue	= new Asura_TableCell_TextDialog(xTableDialog, szName, xOffset, xMinCellSize, iRow, iColumn);

	if(pxNewDialogue == NULL)
	{
		return(NULL);
	}
	else
	{
		pxNewDialogue->UpdateLabel(szName);
		return(pxNewDialogue);
	}
}

int Asura_TableCell_TextDialog::Compare(const Asura_TableCell_Dialog& xOther) const
{
	const Asura_TableCell_TextDialog&	xOtherTextDialogue	= static_cast<const Asura_TableCell_TextDialog&>(xOther);

	return(m_xName.Compare(xOtherTextDialogue.m_xName));
}

//////////////////////////////////////////////////////////////////////////////
//
// Asura_TableColumnHeading_Dialog / Asura_TableRowHeading_Dialog
//
//////////////////////////////////////////////////////////////////////////////

unsigned Asura_TableColumnHeading_Dialog::GetColour( void ) const
{
	unsigned	uColour;

	if(m_xTableDialog.IsEnabled())
	{
		if(m_bClickToSort && (m_xTableDialog.GetColumnHeadingUnderCursor() == m_iColumn))
		{
			uColour	= (Asura_OverlayBrightness | ASURA_TABLE_DIALOG_COLUMN_SORT_HIGHLIGHT_COLOUR);
		}
		else if(m_xTableDialog.GetCurrentlySelectedColumn() == m_iColumn)
		{
			uColour	= (Asura_OverlayBrightness | ASURA_TABLE_DIALOG_COLUMN_SELECTED_COLOUR);
		}
		else if(m_xTableDialog.GetColumnUnderCursor() == m_iColumn)
		{
			uColour	= (Asura_OverlayBrightness | ASURA_TABLE_DIALOG_COLUMN_HIGHLIGHT_COLOUR);
		}
		else
		{
			uColour	= PARENT::GetColour();
		}
	}
	else
	{
		uColour	= Asura_OverlayBrightness | ASURA_GADGET_DISABLED_COLOUR;
	}

	return( m_xTableDialog.GetColour( this, uColour, m_iColumn, 0xffffffff ) );
}

unsigned Asura_TableRowHeading_Dialog::GetColour( void ) const
{
	unsigned	uColour;

	if(m_xTableDialog.IsEnabled())
	{
		if(m_bClickToSort && (m_xTableDialog.GetRowHeadingUnderCursor() == m_iRow))
		{
			uColour	= (Asura_OverlayBrightness | ASURA_TABLE_DIALOG_ROW_SORT_HIGHLIGHT_COLOUR);
		}
		else if(m_xTableDialog.GetCurrentlySelectedRow() == m_iRow)
		{
			uColour	= (Asura_OverlayBrightness | ASURA_TABLE_DIALOG_ROW_SELECTED_COLOUR);
		}
		else if(m_xTableDialog.GetRowUnderCursor() == m_iRow)
		{
			uColour	= (Asura_OverlayBrightness | ASURA_TABLE_DIALOG_ROW_HIGHLIGHT_COLOUR);
		}
		else
		{
			uColour	= PARENT::GetColour();
		}
	}
	else
	{
		uColour	= Asura_OverlayBrightness | ASURA_GADGET_DISABLED_COLOUR;
	}

	return( m_xTableDialog.GetColour( this, uColour, m_iRow, 0xffffffff ) );
}

////////////////////////////////////////////////////////////////////////////////
//
// Asura_TableColumnHeading_StandardDialog
//
////////////////////////////////////////////////////////////////////////////////

Asura_TableColumnHeading_StandardDialog::Asura_TableColumnHeading_StandardDialog( Asura_Table_Dialog& xTableDialog, const char* szName,
                                                                                  const Asura_Vector_2& xOffset, const Asura_Vector_2& xMinCellSize,
                                                                                  const u_int uColumn )
: PARENT( xTableDialog, szName, xOffset, xMinCellSize, uColumn )
{
}

Asura_TableColumnHeading_StandardDialog* Asura_TableColumnHeading_StandardDialog::Create( Asura_Table_Dialog& xTableDialog, const char* szName,
																						  const Asura_Vector_2& xOffset, const Asura_Vector_2& xMinCellSize,
																						  const u_int uColumn )
{
	Asura_TableColumnHeading_StandardDialog*	pxNewDialogue	= new Asura_TableColumnHeading_StandardDialog(xTableDialog, szName, xOffset, xMinCellSize, uColumn);

	if(pxNewDialogue == NULL)
	{
		return(NULL);
	}
	else
	{
		pxNewDialogue->UpdateLabel(szName);
		return(pxNewDialogue);
	}
}

////////////////////////////////////////////////////////////////////////////////
//
// Asura_TableRowHeading_StandardDialog
//
////////////////////////////////////////////////////////////////////////////////

Asura_TableRowHeading_StandardDialog::Asura_TableRowHeading_StandardDialog( Asura_Table_Dialog& xTableDialog, const char* szName,
                                                                            const Asura_Vector_2& xOffset, const Asura_Vector_2& xMinCellSize,
                                                                            const u_int uRow )
: PARENT( xTableDialog, szName, xOffset, xMinCellSize, uRow )
{
}

Asura_TableRowHeading_StandardDialog* Asura_TableRowHeading_StandardDialog::Create( Asura_Table_Dialog& xTableDialog, const char* szName,
																				    const Asura_Vector_2& xOffset, const Asura_Vector_2& xMinCellSize,
																					const u_int uRow )
{
	Asura_TableRowHeading_StandardDialog*	pxNewDialogue	= new Asura_TableRowHeading_StandardDialog(xTableDialog, szName, xOffset, xMinCellSize, uRow);

	if(pxNewDialogue == NULL)
	{
		return(NULL);
	}
	else
	{
		pxNewDialogue->UpdateLabel(szName);
		return(pxNewDialogue);
	}
}


////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Table_Dialog::UpdateList
// Author       : RAR
// Date         : 06-Aug-2007
////////////////////////////////////////////////////////////////////////////////

int Asura_Table_Dialog::UpdateList(const Asura_Vector_2& xOrigin, int iPriority)
{
	//update later siblings and this (which are typically higher on the overlay)
	int iNextPriority = PARENT::UpdateList(xOrigin, iPriority);

	u_int uRow;
	u_int uColumn;
	Asura_Vector_2 xUpdateOrigin;
	const Asura_Vector_2 xNewOrigin( xOrigin + m_xOffset );

	//update the cells with this priority
	for( uColumn = 0; uColumn < m_uNumColumns; uColumn++ )
	{
		if( IsColumnEnabled( uColumn ) )
		{
			for( uRow = 0; uRow < m_uNumRows; uRow++ )
			{
				if( ( IsRowEnabled( uRow ) ) && ( IsCellEnabled( uRow, uColumn ) ) )
				{
					Asura_Dialog* pxDialog = m_pppxCellDialogues[ uRow ][ uColumn ];
					xUpdateOrigin = xNewOrigin + m_xCellOffset;
					const int iChildNextPriority = pxDialog->UpdateList( xUpdateOrigin, iPriority );
					iNextPriority = Asura_Maths::Max(iNextPriority,iChildNextPriority);
				}
			}
		}
	}

    // Update the row headings
    if(m_ppxRowHeadingDialogues != NULL)
    {
        for( u_int uRow = 0; uRow < m_uNumRows; uRow++ )
        {
			Asura_TableRowHeading_Dialog* pxDialog = m_ppxRowHeadingDialogues[ uRow ];

            if( (pxDialog != NULL) && IsRowEnabled( uRow ) )
            {
				xUpdateOrigin = xNewOrigin + m_xCellOffset;
				const int iChildNextPriority = pxDialog->UpdateList( xUpdateOrigin, iPriority );
				iNextPriority = Asura_Maths::Max(iNextPriority,iChildNextPriority);
            }
        }
    }

    // Update the column headings
    if(m_ppxColumnHeadingDialogues != NULL)
    {
        for( u_int uColumn = 0; uColumn < m_uNumColumns; uColumn++ )
        {
			Asura_TableColumnHeading_Dialog* pxDialog = m_ppxColumnHeadingDialogues[ uColumn ];

            if( (pxDialog != NULL) && IsColumnEnabled( uColumn ) )
            {
				xUpdateOrigin = xNewOrigin + m_xCellOffset;
				const int iChildNextPriority = pxDialog->UpdateList( xUpdateOrigin, iPriority );
				iNextPriority = Asura_Maths::Max(iNextPriority,iChildNextPriority);
            }
        }
    }

	return iNextPriority;
}

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Table_Dialog::Update
// Author       : Andy Tate
// Date         : 1-May-2007
////////////////////////////////////////////////////////////////////////////////

void Asura_Table_Dialog::Update(const Asura_Vector_2& xOrigin)
{
	const int	iNumRows	= static_cast<int>(m_uNumRows);
	const int	iNumColumns	= static_cast<int>(m_uNumColumns);

	// Are our currently selected rows/columns within range?
	if(m_bRowSelectionEnabled && (m_iCurrentlySelectedRow >= iNumRows))
	{
		m_iCurrentlySelectedRow	= iNumRows - 1;
		if ( m_pbRowSelectionFlag )
		{
			*m_pbRowSelectionFlag = true;
		}
	}
	if(m_bColumnSelectionEnabled && (m_iCurrentlySelectedColumn >= iNumColumns))
	{
		m_iCurrentlySelectedColumn = iNumColumns - 1;
		if ( m_pbColumnSelectionFlag )
		{
			*m_pbColumnSelectionFlag = true;
		}
	}

	// Update the dimensions
	UpdateRowAndColumnHeadingDimensions(&xOrigin);
	UpdateRowAndColumnDimensions(&xOrigin);
	UpdateScrollbars(&xOrigin);
	UpdateKeys(&xOrigin);

	// Call the parent function...
	PARENT::Update(xOrigin);

	UpdateRowAndColumnHeadingPositions(&xOrigin);
	UpdateCellPositions(&xOrigin);
}

void Asura_Table_Dialog::PerFrameUpdate()
{
    PARENT::PerFrameUpdate();

    int iPriority = 0;
    // Update the cells...
    for(u_int uColumn = 0; uColumn < m_uNumColumns; uColumn++)
    {
        if( IsColumnEnabled( uColumn ) )
        {
            for( u_int uRow = 0; uRow < m_uNumRows; uRow++ )
            {
                if( ( IsRowEnabled( uRow ) ) && ( IsCellEnabled( uRow, uColumn ) ) )
                {
                    Asura_TableCell_Dialog* pxDialog = m_pppxCellDialogues[ uRow ][ uColumn ];
                    pxDialog->PerFrameUpdate();

					int iChildPriority = pxDialog->GetHighestPriority();
					iPriority = Asura_Maths::Max(iPriority,iChildPriority);
                }
            }
        }
    }

    // Update the row headings
    if(m_ppxRowHeadingDialogues != NULL)
    {
        for( u_int uRow = 0; uRow < m_uNumRows; uRow++ )
        {
			Asura_TableRowHeading_Dialog* pxDialog = m_ppxRowHeadingDialogues[ uRow ];

			if( (pxDialog != NULL) && IsRowEnabled( uRow ) )
            {
                pxDialog->PerFrameUpdate();

                int iChildPriority = pxDialog->GetHighestPriority();
                iPriority = Asura_Maths::Max(iPriority,iChildPriority);
            }
        }
    }

    // Update the column headings
    if(m_ppxColumnHeadingDialogues != NULL)
    {
        for( u_int uColumn = 0; uColumn < m_uNumColumns; uColumn++ )
        {
			Asura_TableColumnHeading_Dialog* pxDialog = m_ppxColumnHeadingDialogues[ uColumn ];

			if( (pxDialog != NULL) && IsColumnEnabled( uColumn ) )
            {
                pxDialog->PerFrameUpdate();

                int iChildPriority = pxDialog->GetHighestPriority();
                iPriority = Asura_Maths::Max(iPriority,iChildPriority);
            }
        }
    }

    //set the priority to match that of the highest child. RAR 16/10/2007
    m_iFocusPriority = iPriority;
}

void Asura_Table_Dialog::UpdateAll( void )
{
	UpdateRowAndColumnHeadingDimensions(NULL);
	UpdateRowAndColumnDimensions(NULL);
	UpdateScrollbars(NULL);
	UpdateRowAndColumnHeadingPositions(NULL);
	UpdateCellPositions(NULL);
	UpdateSize();
}

void Asura_Table_Dialog::UpdateKeys( const Asura_Vector_2* pxOrigin )
{
	const Asura_Vector_2 xViewSize( GetViewSize() );

	if( Asura_OverlayBrightness == 0x7f000000 )
	{
		// not in focus
		return;
	}
	if( Asura_Overlay_System::AGadgetHasMouseFocus() )
	{
		return;
	}
	if(pxOrigin != NULL)
	{
		Asura_Vector_2 xPos(*pxOrigin + m_xOffset);
		if( !Asura_Cursor::Inside( xPos, xViewSize.x, xViewSize.y ) )
		{
			return;
		}
	}

	float fCellHeight = 0.0f;
	for( u_int uRow = 0; uRow < m_uNumRows; uRow++ )
	{
		const float fRowHeight = m_pfRowHeights[ uRow ];
		if( fRowHeight > fCellHeight )
		{
			fCellHeight = fRowHeight;
		}
	}
	if( fCellHeight <= 0.0f )
	{
		return;
	}
	const int iItemsOnPage	= (int)( xViewSize.y / fCellHeight );
	const float fPageHeight	= (float)iItemsOnPage * fCellHeight;
	if( fPageHeight <= 0.0f )
	{
		return;
	}

	const bool bShiftPressed		= Asura_UserInput::KeyPressed( ASURA_KEYID_SHIFT );
	const float fWheelDelta			= ( bShiftPressed ) ? fPageHeight : fCellHeight;		// holding shift does a 'page' at a time
	const float fCursorDelta		= fCellHeight * ( ( bShiftPressed ) ? 5.0f : 1.0f );	//
	const float fPageDelta			= fPageHeight * ( ( bShiftPressed ) ? 5.0f : 1.0f );	//

	const float fHighestOffset		= Asura_Maths::Max( 0.0f, m_xTableSize.y - xViewSize.y );


	float fDelta = 0.0f;
	if( Asura_UserInput::KeyPressed( ASURA_KEYID_MOUSEWHEELUP ) )
	{
		fDelta -= fWheelDelta;
	}
	else if( Asura_UserInput::KeyPressed( ASURA_KEYID_MOUSEWHEELDOWN ) )
	{
		fDelta += fWheelDelta;
	}

	if(Asura_UserInput::DebouncedKeyPressed( ASURA_KEYID_MOUSEBUTTON1 ) )
	{
		// Do we want to sort our columns using the currently selected row-heading?
		if(m_iRowHeadingUnderCursor >= 0)
		{
			Asura_TableRowHeading_Dialog*	pxHeadingDialogue	= GetRowHeading(m_iRowHeadingUnderCursor, true);

			if((pxHeadingDialogue != NULL) && pxHeadingDialogue->IsSortedOnClick())
			{
				const bool	bBackwards	= (static_cast<int>(s_uSortIndex) == m_iRowHeadingUnderCursor)
										? !s_bSortBackwards
										: false;

				SortColumnsUsingRow(m_iRowHeadingUnderCursor, bBackwards);
			}
		}

		// Do we want to sort our rows using the currently selected column-heading?
		if(m_iColumnHeadingUnderCursor >= 0)
		{
			Asura_TableColumnHeading_Dialog*	pxHeadingDialogue	= GetColumnHeading(m_iColumnHeadingUnderCursor, true);

			if((pxHeadingDialogue != NULL) && pxHeadingDialogue->IsSortedOnClick())
			{
				const bool	bBackwards	= (static_cast<int>(s_uSortIndex) == m_iColumnHeadingUnderCursor)
										? !s_bSortBackwards
										: false;

				SortRowsUsingColumn(m_iColumnHeadingUnderCursor, bBackwards);
			}
		}

		// Do we need to select the current row?
		if(m_bRowSelectionEnabled)
		{
			if ( m_iCurrentlySelectedRow != m_iRowUnderCursor )
			{
				m_iCurrentlySelectedRow	= m_iRowUnderCursor;
				if ( m_pbRowSelectionFlag )
				{
					*m_pbRowSelectionFlag = true;
				}
			}
		}

		// Do we need to select the current column?
		if(m_bColumnSelectionEnabled)
		{
			if ( m_iCurrentlySelectedColumn != m_iColumnUnderCursor )
			{
				m_iCurrentlySelectedColumn = m_iColumnUnderCursor;
				if ( m_pbColumnSelectionFlag )
				{
					*m_pbColumnSelectionFlag = true;
				}
			}
		}
	}

	if( !Asura_UserInput::KeyPressed( ASURA_KEYID_CTRL ) )
	{
		if( Asura_UserInput::DebouncedKeyPressed( ASURA_KEYID_UP ) )
		{
			fDelta -= fCursorDelta;
		}
		if( Asura_UserInput::DebouncedKeyPressed( ASURA_KEYID_DOWN ) )
		{
			fDelta += fCursorDelta;
		}
		if( Asura_UserInput::DebouncedKeyPressed( ASURA_KEYID_PAGEUP ) )
		{
			fDelta -= fPageDelta;
		}
		if( Asura_UserInput::DebouncedKeyPressed( ASURA_KEYID_PAGEDOWN ) )
		{
			fDelta += fPageDelta;
		}
		if( Asura_UserInput::DebouncedKeyPressed( ASURA_KEYID_HOME ) )
		{
			m_xScrollBarValue.y = 0.0f;
		}
		if( Asura_UserInput::DebouncedKeyPressed( ASURA_KEYID_END ) )
		{
			m_xScrollBarValue.y = fHighestOffset;
		}
	}

	if( fDelta != 0.0f )
	{
		m_xScrollBarValue.y += fDelta;
		if( m_xScrollBarValue.y > fHighestOffset )
		{
			m_xScrollBarValue.y = fHighestOffset;
		}
		if( m_xScrollBarValue.y < 0.0f )
		{
			m_xScrollBarValue.y = 0.0f;
		}
	}
}

/// End of function 'Asura_Table_Dialog::Update'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Table_Dialog::Render
// Author       : Andy Tate
// Date         : 27-Apr-2007
////////////////////////////////////////////////////////////////////////////////

void Asura_Table_Dialog::Render(const Asura_Vector_2& xOrigin)
{
	if( IsHidden() )
	{
		// Call the parent function...
		PARENT::Render(xOrigin);
		return;
	}

	Asura_Vector_2				xRenderOrigin;
	const Asura_Vector_2		xViewSize( GetViewSize() );
	const Asura_Vector_2		xNewOrigin( xOrigin + m_xOffset );
	const Asura_Vector_2		xHeadingPosition( xNewOrigin );
	const Asura_Vector_2		xCellPosition( xHeadingPosition + m_xMaxHeadingSize );
	const Asura_Vector_2		xEndPosition( xHeadingPosition + xViewSize );

	Asura_Bounding_Rectangle	xDrawArea;

	//draw outline
	const float fW	= m_xViewSizeWithoutScrollBars.x * ASURA_TEXT_OUTPUT_FONT_WIDTH;
	const float fH	= m_xViewSizeWithoutScrollBars.y * ASURA_TEXT_OUTPUT_FONT_HEIGHT;
	const float fX1	= xNewOrigin.x * ASURA_TEXT_OUTPUT_FONT_WIDTH;
	const float fY1	= xNewOrigin.y * ASURA_TEXT_OUTPUT_FONT_HEIGHT;
	const float fX2	= fX1 + fW;
	const float fY2	= fY1 + fH;
	const float fY3	= fY1 + ( xViewSize.y * ASURA_TEXT_OUTPUT_FONT_HEIGHT );
	const u_int uColour = Asura_OverlayBrightness | ((m_bIsEnabled) ? ASURA_GADGET_DEFAULT_COLOUR : ASURA_GADGET_DISABLED_COLOUR);

	if( m_bEnableLines )
	{
		Asura_Render::Render2DLine( fX1, fY1, fX2, fY1, uColour);
		Asura_Render::Render2DLine( fX2, fY1, fX2, fY2, uColour);
		Asura_Render::Render2DLine( fX2, fY2, fX1, fY2, uColour);
		Asura_Render::Render2DLine( fX1, fY2, fX1, fY1, uColour);
	}

	// Set a custom draw-area...
	Asura_Overlay_Quad::SetToolsDrawArea(&xDrawArea);

	// Render the row headings...
	if( m_ppxRowHeadingDialogues != NULL )
	{
		// Update the draw-area...
		xDrawArea.MinX	= ASURA_TEXT_OUTPUT_FONT_WIDTH * xHeadingPosition.x;
		xDrawArea.MinY	= ASURA_TEXT_OUTPUT_FONT_HEIGHT * xCellPosition.y;
		xDrawArea.MaxX	= ASURA_TEXT_OUTPUT_FONT_WIDTH * xCellPosition.x;
		xDrawArea.MaxY	= ASURA_TEXT_OUTPUT_FONT_HEIGHT * xEndPosition.y;

		if( m_bEnableLines )
		{
			Asura_Render::Render2DLine( xDrawArea.MaxX, fY1, xDrawArea.MaxX, fY2, uColour );
		}

		// Render the row headings...
		for( u_int uRow = 0; uRow < m_uNumRows; uRow++ )
		{
			Asura_TableRowHeading_Dialog*	pxDialog	= m_ppxRowHeadingDialogues[ uRow ];

			if( (pxDialog != NULL) && IsRowEnabled( uRow ) )
			{
				xRenderOrigin = xNewOrigin + m_xRowHeadingOffset;
				pxDialog->Render( xRenderOrigin );
			}
		}
	}

	// Render the column headings...
	if(m_ppxColumnHeadingDialogues != NULL)
	{
		// Update the draw-area...
		xDrawArea.MinX	= ASURA_TEXT_OUTPUT_FONT_WIDTH * xCellPosition[0];
		xDrawArea.MinY	= ASURA_TEXT_OUTPUT_FONT_HEIGHT * xHeadingPosition[1];
		xDrawArea.MaxX	= ASURA_TEXT_OUTPUT_FONT_WIDTH * xEndPosition[0];
		xDrawArea.MaxY	= ASURA_TEXT_OUTPUT_FONT_HEIGHT * xCellPosition[1];

		if( m_bEnableLines )
		{
			Asura_Render::Render2DLine( fX1, xDrawArea.MaxY, fX2, xDrawArea.MaxY, uColour);
		}


		// Render the column headings...
		for(u_int uColumn = 0; uColumn < m_uNumColumns; uColumn++)
		{
			Asura_Dialog*	pxDialog	= m_ppxColumnHeadingDialogues[uColumn];

			if( (pxDialog != NULL) && IsColumnEnabled( uColumn ) )
			{
				if( m_bEnableLines )
				{
					const float fX = ( xNewOrigin.x + pxDialog->m_xOffset.x ) * ASURA_TEXT_OUTPUT_FONT_WIDTH;
					if( ( fX >= xDrawArea.MinX ) && ( fX <= xDrawArea.MaxX ) )
					{
						Asura_Render::Render2DLine( fX, fY1, fX, fY3, uColour);
					}
				}
				xRenderOrigin = xNewOrigin + m_xColumnHeadingOffset;
				pxDialog->Render( xRenderOrigin );
			}
		}
	}

	// Render the cells...
	if(m_pppxCellDialogues != NULL)
	{
		// Update the draw-area...
		xDrawArea.MinX	= ASURA_TEXT_OUTPUT_FONT_WIDTH * xCellPosition[0];
		xDrawArea.MinY	= ASURA_TEXT_OUTPUT_FONT_HEIGHT * xCellPosition[1];
		xDrawArea.MaxX	= ASURA_TEXT_OUTPUT_FONT_WIDTH * xEndPosition[0];
		xDrawArea.MaxY	= ASURA_TEXT_OUTPUT_FONT_HEIGHT * xEndPosition[1];

		// Render the cells (backwards, so that drop-down/etc dialogues do not get rendered over.
        u_int uColumn = m_uNumColumns;
        while ( uColumn-- )
        {
            const u_int uActualColumn = m_puColumnDisplayIndices[ uColumn ];
            if ( IsColumnEnabled( uActualColumn ) )
            {
                u_int uRow = m_uNumRows;
                while ( uRow-- )
                {
                    const u_int uActualRow = m_puRowDisplayIndices[ uRow ];
                    if ( ( IsRowEnabled( uActualRow ) ) && ( IsCellEnabled( uActualRow, uActualColumn ) ) )
                    {
                        Asura_TableCell_Dialog* pxDialog = m_pppxCellDialogues[ uActualRow ][ uActualColumn ];
                        xRenderOrigin = xNewOrigin + m_xCellOffset;
						if( pxDialog->ShouldClip( ) )
						{
							pxDialog->RenderThisAndChildDialogs( xRenderOrigin );
						}
						else
						{
							//temporarily turn clipping off
							Asura_Overlay_Quad::SetToolsDrawArea( NULL );
							pxDialog->RenderThisAndChildDialogs( xRenderOrigin );
							Asura_Overlay_Quad::SetToolsDrawArea( &xDrawArea );
						}
                    }
                }
            }
        }
	}

	// Remove the draw-area...
	Asura_Overlay_Quad::SetToolsDrawArea(NULL);

	// Call the parent function...
	PARENT::Render(xOrigin);
}

/// End of function 'Asura_Table_Dialog::Render'. ///


void Asura_Table_Dialog::OnHiddenStateChange()
{
	PARENT::OnHiddenStateChange();

	bool bHidden = IsHidden();

	//Mark all the child dialogs as hidden/unhidden
	//They wouldn't be rendered regardless, since the render function won't be called , 
	//but they need to know that they are hidden to prevent them from being clicked on etc 
	//during their updates.
	//RAR 27/07/2007
	
	for( u_int uColumn = 0; uColumn < m_uNumColumns; uColumn++ )
	{
		if( m_ppxColumnHeadingDialogues )
		{
			Asura_Dialog* pxDialog = m_ppxColumnHeadingDialogues[ uColumn ];
			if(pxDialog) pxDialog->SetHideFlag( ASURA_DIALOG_HIDEFLAG_TABLE, bHidden );
		}

		for( u_int uRow = 0; uRow < m_uNumRows; uRow++ )
		{
			Asura_Dialog* pxDialog = m_pppxCellDialogues[ uRow ][ uColumn ];
			if(pxDialog) pxDialog->SetHideFlag( ASURA_DIALOG_HIDEFLAG_TABLE, bHidden );
		}
	}
}


////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Table_Dialog::IsPointInDialog
// Author       : RAR
// Date         : 06-Aug-2007
////////////////////////////////////////////////////////////////////////////////

bool Asura_Table_Dialog::IsPointInDialog(const Asura_Vector_2& xOrigin,const Asura_Vector_2& xPos) const
{
	if(IsHidden()) return false;
	if(PARENT::IsPointInDialog(xOrigin, xPos)) return true;

	//check for the point being in any of the child dialogs
	u_int uRow;
	u_int uColumn;
	Asura_Vector_2 xUpdateOrigin;
	const Asura_Vector_2 xNewOrigin( xOrigin + m_xOffset );

	//Chuck away the constness while no one is looking , since unfortunately IsColumnEnabled() and IsRowEnabled() aren't const
	Asura_Table_Dialog* pxTable = const_cast<Asura_Table_Dialog*>(this);

	for( uColumn = 0; uColumn < m_uNumColumns; uColumn++ )
	{
		if( pxTable->IsColumnEnabled( uColumn ) )
		{
			for( uRow = 0; uRow < m_uNumRows; uRow++ )
			{
				if( ( pxTable->IsRowEnabled( uRow ) ) && ( pxTable->IsCellEnabled( uRow, uColumn ) ) )
				{
					const Asura_Dialog* pxDialog = m_pppxCellDialogues[ uRow ][ uColumn ];
					xUpdateOrigin = xNewOrigin + m_xCellOffset;

					for(; pxDialog; pxDialog = pxDialog->m_pxNextDialog)
					{
						if(pxDialog->IsPointInDialog(xUpdateOrigin, xPos)) return true;
					}
				}
			}
		}
	}

	return false;
}

void Asura_Table_Dialog::UpdateSize()
{
	const Asura_Vector_2	xOldMaxCoord(m_xMaxCoord);

    PARENT::UpdateSize();

    m_xMaxCoord = m_xOffset + m_xViewSizeWithoutScrollBars;

	if(m_xMaxCoord != xOldMaxCoord && m_pxOverlay != NULL)
	{
		m_pxOverlay->FlagForResize();
	}
}

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Table_Dialog::UpdateRowAndColumnHeadingDimensions
// Author       : Andy Tate
// Date         : 30-Apr-2007
////////////////////////////////////////////////////////////////////////////////

void Asura_Table_Dialog::UpdateRowAndColumnHeadingDimensions(const Asura_Vector_2* pxOrigin)
{
	float fWidth = 0.0f;
	float fHeight = 0.0f;

	// Update the row heading dimensions...
	if(m_ppxRowHeadingDialogues != NULL)
	{
		for( u_int uRow = 0; uRow < m_uNumRows; uRow++ )
		{
			const Asura_Dialog*	pxDialog	= m_ppxRowHeadingDialogues[ uRow ];

			if( (pxDialog != NULL) && IsRowEnabled( uRow ) )
			{
				const float	fHeadingWidth	= ( pxDialog->m_xMaxCoord.x - pxDialog->m_xOffset.x ) + m_xHeadingsSpacing.x;
				fWidth = Asura_Maths::Max( fHeadingWidth, fWidth );
			}
		}
	}

	// Update the column heading dimensions...
	if(m_ppxColumnHeadingDialogues != NULL)
	{
		for( u_int uColumn = 0; uColumn < m_uNumColumns; uColumn++ )
		{
			const Asura_Dialog*	pxDialog	= m_ppxColumnHeadingDialogues[uColumn];

			if( (pxDialog != NULL) && IsColumnEnabled( uColumn ) )
			{
				const float	fHeadingHeight	= ( pxDialog->m_xMaxCoord.y - pxDialog->m_xOffset.y ) + m_xHeadingsSpacing.y;
				fHeight	= Asura_Maths::Max( fHeadingHeight, fHeight);
			}
		}
	}
	m_xMaxHeadingSize.x = fWidth;
	m_xMaxHeadingSize.y	= fHeight;
}

/// End of function 'Asura_Table_Dialog::UpdateRowAndColumnHeadingDimensions'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Table_Dialog::UpdateRowAndColumnDimensions
// Author       : Andy Tate
// Date         : 30-Apr-2007
////////////////////////////////////////////////////////////////////////////////

void Asura_Table_Dialog::UpdateRowAndColumnDimensions(const Asura_Vector_2* pxOrigin)
{
	// Store the actual size of the table...
	m_xTableSize = m_xMaxHeadingSize;

	// Update the column widths...
	for( u_int uColumn = 0; uColumn < m_uNumColumns; uColumn++ )
	{
		float fWidth = 0.0f;
		if( IsColumnEnabled( uColumn ) )
		{
			for(u_int uRow = 0; uRow < m_uNumRows; uRow++)
			{
				if( ( IsRowEnabled( uRow ) ) && ( IsCellEnabled( uRow, uColumn ) ) )
				{
					const Asura_TableCell_Dialog*	pxCellDialog	= m_pppxCellDialogues[ uRow ][ uColumn ];
					const Asura_Vector_2&			xCellSize		= pxCellDialog->GetDialogSize();
					fWidth = Asura_Maths::Max( xCellSize.x, fWidth );
				}
			}
			if(m_ppxColumnHeadingDialogues != NULL)
			{
				const Asura_Dialog*	pxDialog	= m_ppxColumnHeadingDialogues[uColumn];

				if(pxDialog != NULL)
				{
					const float	fHeadingWidth	= pxDialog->m_xMaxCoord.x - pxDialog->m_xOffset.x;
					fWidth = Asura_Maths::Max( fHeadingWidth, fWidth );
				}
			}
			if( fWidth > 0.0f )
			{
				fWidth = Asura_Maths::Max( m_xMinCellSize.x, fWidth ) + m_xCellSpacing.x;
			}
		}
		m_pfColumnWidths[ uColumn ] = fWidth;
		m_xTableSize.x += fWidth;
	}

	// Update the row heights...
	for(u_int uRow = 0; uRow < m_uNumRows; uRow++)
	{
		float fHeight = 0.0f;
		if( IsRowEnabled( uRow ) )
		{
			for(u_int uColumn = 0; uColumn < m_uNumColumns; uColumn++)
			{
				if( ( IsColumnEnabled( uColumn ) ) && ( IsCellEnabled( uRow, uColumn ) ) )
				{
					const Asura_TableCell_Dialog*	pxCellDialog	= m_pppxCellDialogues[uRow][uColumn];
					const Asura_Vector_2&			xCellSize		= pxCellDialog->GetDialogSize();
					fHeight	= Asura_Maths::Max( xCellSize.y, fHeight );
				}
			}
			if( m_ppxRowHeadingDialogues != NULL )
			{
				const Asura_Dialog*	pxDialog	= m_ppxRowHeadingDialogues[ uRow ];

				if(pxDialog != NULL)
				{
					const float	fHeadingHeight	= pxDialog->m_xMaxCoord.y - pxDialog->m_xOffset.y;
					fHeight = Asura_Maths::Max( fHeadingHeight, fHeight );
				}
			}
			if( fHeight > 0.0f )
			{
				fHeight = Asura_Maths::Max( m_xMinCellSize.y, fHeight ) + m_xCellSpacing.y;
			}
		}
		m_pfRowHeights[ uRow ] = fHeight;
		m_xTableSize.y += fHeight;
	}

    if ( m_bResizeAsNecessary )
    {
        m_xViewSize.x = Asura_Maths::Min( m_xTableSize.x, m_xMaxViewSize.x );
        m_xViewSize.y = Asura_Maths::Min( m_xTableSize.y, m_xMaxViewSize.y );
        m_xViewSizeWithoutScrollBars = m_xViewSize;
        m_xViewSizeWithoutScrollBars.x += m_apxScrollBarGadgets[ SBO_VERTICAL ]->m_fWidth;
        m_xViewSizeWithoutScrollBars.y += m_apxScrollBarGadgets[ SBO_HORIZONTAL ]->m_fHeight;
    }
}

/// End of function 'Asura_Table_Dialog::UpdateRowAndColumnDimensions'. ///


////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Table_Dialog::UpdateRowAndColumnHeadingPositions
// Author       : Andy Tate
// Date         : 30-Apr-2007
////////////////////////////////////////////////////////////////////////////////

void Asura_Table_Dialog::UpdateRowAndColumnHeadingPositions(const Asura_Vector_2* pxOrigin)
{
	const Asura_Vector_2	xInitialOffset( 0.0f, 0.0f );
	const Asura_Vector_2	xInitialCellPosition( m_xMaxHeadingSize );
	const Asura_Vector_2	xFinalCellPosition( GetViewSize() );

	// Update the row-headings...
	{
		Asura_Vector_2	xOffset( xInitialOffset );
		xOffset.y += ( m_xMaxHeadingSize.y - m_xScrollBarValue.y );

		m_iRowUnderCursor			= -1;
		m_iRowHeadingUnderCursor	= -1;

		for( u_int uRow = 0; uRow < m_uNumRows; uRow++ )
		{
			const u_int		uActualRow	= m_puRowDisplayIndices[ uRow ];
			Asura_Dialog*	pxDialog	= GetRowHeading(uActualRow, true);

			if(pxDialog != NULL)
			{
				if( IsRowEnabled( uActualRow ) )
				{
					const bool				bVisible	= IsInView( xOffset[ 1 ], m_pfRowHeights[ uActualRow ], xInitialCellPosition[ 1 ], xFinalCellPosition[ 1 ] );
					const Asura_Vector_2	xSize( pxDialog->m_xMaxCoord - pxDialog->m_xOffset );

					pxDialog->m_xOffset	= xOffset;
					pxDialog->m_xMaxCoord	= xOffset + xSize;

					if(pxOrigin != NULL)
					{
						const Asura_Vector_2	xPosition(*pxOrigin + m_xOffset + xOffset);
						const float				fHeight	= m_pfRowHeights[uActualRow];

						if(Asura_Cursor::Inside(xPosition, xSize[0], fHeight))
						{
							m_iRowHeadingUnderCursor	= uActualRow;
						}

						if(m_bRowSelectionEnabled)
						{
							if(m_bSelectRowFromHeadingOnly)
							{
								m_iRowUnderCursor	= m_iRowHeadingUnderCursor;
							}
							else
							{
								if(Asura_Cursor::Inside(xPosition, m_xMaxViewSize[0], fHeight))
								{
									m_iRowUnderCursor	= uActualRow;
								}
							}
						}
					}

					pxDialog->SetHidden( !bVisible );
					pxDialog->SetEnabled( bVisible );
				}
				else
				{
					pxDialog->SetHidden( true);
					pxDialog->SetEnabled( false );
				}
				xOffset.y += m_pfRowHeights[ uActualRow ];
			}
		}
	}

	// Update the column-headings...
	{
		Asura_Vector_2	xOffset(xInitialOffset);
		xOffset.x += ( m_xMaxHeadingSize.x - m_xScrollBarValue.x );

		m_iColumnUnderCursor		= -1;
		m_iColumnHeadingUnderCursor	= -1;

		for( u_int uColumn = 0; uColumn < m_uNumColumns; uColumn++ )
		{
			const u_int		uActualColumn	= m_puColumnDisplayIndices[ uColumn ];
			Asura_Dialog*	pxDialog		= GetColumnHeading(uActualColumn, true);

			if(pxDialog != NULL)
			{
				if( IsColumnEnabled( uActualColumn ) )
				{
					const bool				bVisible		= IsInView( xOffset[ 0 ], m_pfColumnWidths[ uActualColumn ], xInitialCellPosition[ 0 ], xFinalCellPosition[ 0 ] );
					const Asura_Vector_2	xSize( pxDialog->m_xMaxCoord - pxDialog->m_xOffset );

					pxDialog->m_xOffset	= xOffset;
					pxDialog->m_xMaxCoord	= xOffset + xSize;

					if(pxOrigin != NULL)
					{
						const Asura_Vector_2	xPosition(*pxOrigin + m_xOffset + xOffset);
						const float				fWidth	= m_pfColumnWidths[uActualColumn];

						if(Asura_Cursor::Inside(xPosition, fWidth, xSize[1]))
						{
							m_iColumnHeadingUnderCursor	= uActualColumn;
						}

						if(m_bColumnSelectionEnabled)
						{
							if(m_bSelectColumnFromHeadingOnly)
							{
								m_iColumnUnderCursor	= m_iColumnHeadingUnderCursor;
							}
							else
							{
								if(Asura_Cursor::Inside(xPosition, fWidth, m_xMaxViewSize[1]))
								{
									m_iColumnUnderCursor	= uActualColumn;
								}
							}
						}
					}

					pxDialog->SetHidden( !bVisible );
					pxDialog->SetEnabled( bVisible );
				}
				else
				{
					pxDialog->SetHidden( true);
					pxDialog->SetEnabled( false );
				}
				xOffset.x += m_pfColumnWidths[ uActualColumn ];
			}
		}
	}
}

/// End of function 'Asura_Table_Dialog::UpdateRowAndColumnHeadingPositions'. ///


////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Table_Dialog::UpdateCellPositions
// Author       : Andy Tate
// Date         : 30-Apr-2007
////////////////////////////////////////////////////////////////////////////////

void Asura_Table_Dialog::UpdateCellPositions(const Asura_Vector_2* pxOrigin)
{
	const Asura_Vector_2	xInitialOffset( -m_xScrollBarValue.x, -m_xScrollBarValue.y );
	const Asura_Vector_2	xInitialCellPosition( m_xMaxHeadingSize );
	const Asura_Vector_2	xFinalCellPosition( GetViewSize() );
	Asura_Vector_2			xOffset( xInitialOffset + m_xMaxHeadingSize );

	// Update the individual cell positions...
	for( u_int uColumn = 0; uColumn < m_uNumColumns; uColumn++ )
	{
		xOffset.y = xInitialOffset.y + m_xMaxHeadingSize.y;
		const u_int	uColumnIndex	= m_puColumnDisplayIndices[uColumn];

		for( u_int uRow = 0; uRow < m_uNumRows; uRow++ )
		{
			const u_int	uRowIndex = m_puRowDisplayIndices[uRow];
			Asura_Dialog* pxCellDialog = m_pppxCellDialogues[uRowIndex][uColumnIndex];
			if( ( IsColumnEnabled( uColumnIndex ) ) && ( IsRowEnabled( uRowIndex ) ) && ( IsCellEnabled( uRowIndex, uColumnIndex ) ) )
			{
				const bool bVisibleHor		= IsInView( xOffset[ 0 ], m_pfColumnWidths[ uColumnIndex ], xInitialCellPosition[ 0 ], xFinalCellPosition[ 0 ] );
				const bool bVisibleVer		= IsInView( xOffset[ 1 ], m_pfRowHeights[ uRowIndex ], xInitialCellPosition[ 1 ], xFinalCellPosition[ 1 ] );
				const bool bVisible			= bVisibleHor && bVisibleVer;

				pxCellDialog->m_xOffset		= xOffset;
				pxCellDialog->m_xMaxCoord.x	= xOffset.x + m_pfColumnWidths[ uColumnIndex ];
				pxCellDialog->m_xMaxCoord.y	= xOffset.y + m_pfRowHeights[ uRowIndex ];

				pxCellDialog->SetHidden( !bVisible );
				pxCellDialog->SetEnabled( bVisible );
			}
			else
			{
				pxCellDialog->SetHidden( true );
				pxCellDialog->SetEnabled( false );
			}
			xOffset.y += m_pfRowHeights[ uRowIndex ];
		}
		xOffset.x += m_pfColumnWidths[ uColumnIndex ];
	}
}

/// End of function 'Asura_Table_Dialog::UpdateCellPositions'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Table_Dialog::UpdateScrollbars
// Author       : Andy Tate
// Date         : 27-Apr-2007
////////////////////////////////////////////////////////////////////////////////

void Asura_Table_Dialog::UpdateScrollbars(const Asura_Vector_2* pxOrigin)
{
	// vertical
	const bool bDefiniteHorizontal	= ( m_xTableSize.x > m_xViewSizeWithoutScrollBars.x );
	const bool bDefiniteVertical	= ( m_xTableSize.y > m_xViewSizeWithoutScrollBars.y );
	const bool bMaybeHorizontal		= ( !bDefiniteHorizontal ) && ( m_xTableSize.x > m_xViewSize.x );
	const bool bMaybeVertical		= ( !bDefiniteVertical ) &&	( m_xTableSize.y > m_xViewSize.y );
	const bool bVertical			= ( bDefiniteVertical ) || ( bMaybeVertical && bDefiniteHorizontal );
	const bool bHorizontal			= ( bDefiniteHorizontal ) || ( bMaybeHorizontal && bVertical );
	m_bScrollBarNeeded[ SBO_HORIZONTAL ] = bHorizontal;
	m_bScrollBarNeeded[ SBO_VERTICAL ]   = bVertical;
	
	const Asura_Vector_2 xViewSize( GetViewSize() );
	const Asura_Vector_2 xMaxScroll( m_xTableSize - xViewSize );
	Asura_Vector_2 xBarPosition[ NUM_SCROLL_BAR_ORIENTATIONS ];
    xBarPosition[ SBO_HORIZONTAL ].x = m_xOffset.x + m_xMaxHeadingSize.x;
    xBarPosition[ SBO_HORIZONTAL ].y = m_xOffset.y + m_xViewSize.y;
    xBarPosition[ SBO_VERTICAL ].x = m_xOffset.x + m_xViewSize.x;
    xBarPosition[ SBO_VERTICAL ].y = m_xOffset.y + m_xMaxHeadingSize.y;
	const Asura_Vector_2 xBarSize( xViewSize - m_xMaxHeadingSize );

	// Update each scroll-bar in-turn...
	for( u_int u = 0; u < 2; u++ )
	{
		// Update the position/size of the scroll-bar...
        m_apxScrollBarGadgets[ u ]->m_xOffset = xBarPosition[ u ];
		m_apxScrollBarGadgets[ u ]->SetBarLength( xBarSize[ u ] );

		// Calculate the new value for the scroll-bar...
		if( xMaxScroll[ u ] > 0.0f )
		{
			m_xScrollBarValue[ u ]	= Asura_Maths::Min( m_xScrollBarValue[ u ], xMaxScroll[ u ] );
			float fSliderLength		= ( ( xBarSize[ u ] / m_xTableSize[ u ] ) * xBarSize[ u ] );

			m_apxScrollBarGadgets[ u ]->SetSliderLength( fSliderLength );
			m_apxScrollBarGadgets[ u ]->SetMax( xMaxScroll[ u ] );
			m_apxScrollBarGadgets[ u ]->SetHidden( false );

		}
		else
		{
			m_xScrollBarValue[ u ]	= 0.0f;
			m_apxScrollBarGadgets[ u ]->SetHidden( true );
		}
	}
}

/// End of function 'Asura_Table_Dialog::UpdateScrollbars'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Table_Dialog::SortByRowHeadingDialogues
// Author       : Andy Tate
// Date         : 8-May-2007
////////////////////////////////////////////////////////////////////////////////

int Asura_Table_Dialog::SortByRowHeadingDialogues(const void* pElement1,
												  const void* pElement2)
{
	const u_int*						puElement1	= static_cast<const u_int*>(pElement1);
	const u_int*						puElement2	= static_cast<const u_int*>(pElement2);
	const Asura_TableRowHeading_Dialog*	pxRow1		= s_pxSortTableDialogue->m_ppxRowHeadingDialogues[*puElement1];
	const Asura_TableRowHeading_Dialog*	pxRow2		= s_pxSortTableDialogue->m_ppxRowHeadingDialogues[*puElement2];

    return ( Asura_Table_Dialog::ShouldSortBackwards() ? pxRow2->Compare( *pxRow1 ) : pxRow1->Compare( *pxRow2 ) );
}

/// End of function 'Asura_Table_Dialog::SortByRowHeadingDialogues'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Table_Dialog::SortByCellDialoguesInRow
// Author       : Andy Tate
// Date         : 8-May-2007
////////////////////////////////////////////////////////////////////////////////

int Asura_Table_Dialog::SortByCellDialoguesInRow(const void* pElement1,
												 const void* pElement2)
{
	const u_int*					puElement1	= static_cast<const u_int*>(pElement1);
	const u_int*					puElement2	= static_cast<const u_int*>(pElement2);
	const Asura_TableCell_Dialog*	pxCell1		= s_pxSortTableDialogue->m_pppxCellDialogues[s_uSortIndex][*puElement1];
	const Asura_TableCell_Dialog*	pxCell2		= s_pxSortTableDialogue->m_pppxCellDialogues[s_uSortIndex][*puElement2];

    return ( Asura_Table_Dialog::ShouldSortBackwards() ? pxCell2->Compare( *pxCell1 ) : pxCell1->Compare( *pxCell2 ) );
}

/// End of function 'Asura_Table_Dialog::SortByCellDialoguesInRow'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Table_Dialog::SortByColumnHeadingDialogues
// Author       : Andy Tate
// Date         : 8-May-2007
////////////////////////////////////////////////////////////////////////////////

int Asura_Table_Dialog::SortByColumnHeadingDialogues(const void* pElement1,
													 const void* pElement2)
{
	const u_int*							puElement1	= static_cast<const u_int*>(pElement1);
	const u_int*							puElement2	= static_cast<const u_int*>(pElement2);
	const Asura_TableColumnHeading_Dialog*	pxColumn1	= s_pxSortTableDialogue->m_ppxColumnHeadingDialogues[*puElement1];
	const Asura_TableColumnHeading_Dialog*	pxColumn2	= s_pxSortTableDialogue->m_ppxColumnHeadingDialogues[*puElement2];

    return ( Asura_Table_Dialog::ShouldSortBackwards() ? pxColumn2->Compare( *pxColumn1 ) : pxColumn1->Compare( *pxColumn2 ) );
}

/// End of function 'Asura_Table_Dialog::SortByColumnHeadingDialogues'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Table_Dialog::SortByCellDialoguesInColumn
// Author       : Andy Tate
// Date         : 8-May-2007
////////////////////////////////////////////////////////////////////////////////

int Asura_Table_Dialog::SortByCellDialoguesInColumn(const void* pElement1,
													const void* pElement2)
{
	const u_int*					puElement1	= static_cast<const u_int*>(pElement1);
	const u_int*					puElement2	= static_cast<const u_int*>(pElement2);
	const Asura_TableCell_Dialog*	pxCell1		= s_pxSortTableDialogue->m_pppxCellDialogues[*puElement1][s_uSortIndex];
	const Asura_TableCell_Dialog*	pxCell2		= s_pxSortTableDialogue->m_pppxCellDialogues[*puElement2][s_uSortIndex];

    return ( Asura_Table_Dialog::ShouldSortBackwards() ? pxCell2->Compare( *pxCell1 ) : pxCell1->Compare( *pxCell2 ) );
}

/// End of function 'Asura_Table_Dialog::SortByCellDialoguesInColumn'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Table_Dialog::SetClickToSortForAllColumnHeadings
// Author       : Andy Tate
// Date         : 16-May-2008
////////////////////////////////////////////////////////////////////////////////

void Asura_Table_Dialog::SetClickToSortForAllColumnHeadings(const bool bSet)
{
	if(m_ppxColumnHeadingDialogues != NULL)
	{
		for(u_int u = 0; u < m_uNumColumns; u++)
		{
			Asura_TableColumnHeading_Dialog*	pxHeadingDialogue	= m_ppxColumnHeadingDialogues[u];

			if(pxHeadingDialogue != NULL)
			{
				pxHeadingDialogue->SetClickToSort(bSet);
			}
		}
	}
}

/// End of function 'Asura_Table_Dialog::SetClickToSortForAllColumnHeadings'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Table_Dialog::SetClickToSortForAllRowHeadings
// Author       : Andy Tate
// Date         : 16-May-2008
////////////////////////////////////////////////////////////////////////////////

void Asura_Table_Dialog::SetClickToSortForAllRowHeadings(const bool bSet)
{
	if(m_ppxRowHeadingDialogues != NULL)
	{
		for(u_int u = 0; u < m_uNumRows; u++)
		{
			Asura_TableRowHeading_Dialog*	pxHeadingDialogue	= m_ppxRowHeadingDialogues[u];

			if(pxHeadingDialogue != NULL)
			{
				pxHeadingDialogue->SetClickToSort(bSet);
			}
		}
	}
}

/// End of function 'Asura_Table_Dialog::SetClickToSortForAllRowHeadings'. ///



////////////////////////////////////////////////////////////////////////////////
// Public functions...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Table_Dialog::Asura_Table_Dialog
// Author       : Andy Tate
// Date         : 27-Apr-2007
////////////////////////////////////////////////////////////////////////////////

Asura_Table_Dialog::Asura_Table_Dialog(const Asura_Vector_2& xOffset,
									   const float fWidth,
									   const float fHeight,
                                       const bool bResizeAsNecessary)
: PARENT("", xOffset)
, m_xTableSize(0.0f, 0.0f)
, m_xViewSize(0.0f, 0.0f)
, m_xMaxViewSize(0.0f, 0.0f)
, m_xViewSizeWithoutScrollBars(0.0f, 0.0f)
, m_xMaxHeadingSize(0.0f, 0.0f)
, m_xScrollBarValue(0.0f, 0.0f)
, m_bResizeAsNecessary(bResizeAsNecessary)
, m_pppxCellDialogues(NULL)
, m_ppxRowHeadingDialogues(NULL)
, m_ppxColumnHeadingDialogues(NULL)
, m_uNumRows(0)
, m_pfRowHeights(NULL)
, m_puRowDisplayIndices(NULL)
, m_bRowSelectionEnabled(false)
, m_bSelectRowFromHeadingOnly(false)
, m_iRowHeadingUnderCursor(-1)
, m_iRowUnderCursor(-1)
, m_iCurrentlySelectedRow(-1)
, m_pbRowSelectionFlag(NULL)
, m_uNumColumns(0)
, m_pfColumnWidths(NULL)
, m_puColumnDisplayIndices(NULL)
, m_bColumnSelectionEnabled(false)
, m_bSelectColumnFromHeadingOnly(false)
, m_iColumnUnderCursor(-1)
, m_iColumnHeadingUnderCursor(-1)
, m_iCurrentlySelectedColumn(-1)
, m_pbColumnSelectionFlag(NULL)
, m_bEnableLines( true )
, m_xCellSpacing( 1.0f, 1.0f )			// Spacing between the cells
, m_xHeadingsSpacing( 1.0f, 1.0f )		// Spacing for the headings ( .x = rows, .y = columns )
, m_xCellOffset( 0.5f, 0.5f )			// Cell offset
, m_xRowHeadingOffset( 0.5f, 0.5f )		// Row heading offset
, m_xColumnHeadingOffset( 0.5f, 0.5f )	// Column heading offset
, m_xMinCellSize(1.0f, 1.0f)
, m_pfnIsRowEnabled( DefaultIsRowEnabled )
, m_pxIsRowEnabledUserData( NULL )
, m_pfnIsColumnEnabled( DefaultIsColumnEnabled )
, m_pxIsColumnEnabledUserData( NULL )
, m_pfnIsCellEnabled( DefaultIsCellEnabled )
, m_pxIsCellEnabledUserData( NULL )
, m_pfnGetColour( DefaultGetColour )
, m_pxGetColourUserData( NULL )
, m_xPreResizeViewSize( 0.0f, 0.0f )
, m_bSortBackwards( false )
, m_uSortIndex( 0 )
{
	const Asura_Vector_2 xMaxViewSize(fWidth, fHeight);
	const Asura_Vector_2 xScrollBarSize( Asura_Scrollbar_Gadget::GetDefaultThickness( true ), Asura_Scrollbar_Gadget::GetDefaultThickness( false ) );

	// Determine space for the main table view...
	m_xMaxViewSize = xMaxViewSize - xScrollBarSize;

	// Create the scrollbars...
	for(u_int u = 0; u < 2; u++)
	{
		Asura_Vector_2	xPosition(xOffset);

		// Make sure there is going to be at least some area...
		if( m_xMaxViewSize[ u ] < dbg_xMinTableViewSize[ u ] )
		{
			Asura_Assert(false,"Table dialog too small");
			m_xMaxViewSize[ u ] = Asura_Maths::Max(m_xMaxViewSize[ u ], dbg_xMinTableViewSize[ u ] );
		}

		// Create the scroll-bar...
		xPosition[ u ^ 1 ]			+= m_xMaxViewSize[ u ^ 1 ];
		m_apxScrollBarGadgets[ u ]	= new Asura_Scrollbar_Gadget( xPosition, &m_xScrollBarValue[ u ], 0.0f, 0.0f, ( u == 0 ), m_xMaxViewSize[ u ], NULL, NULL );
		m_bScrollBarNeeded[ u ]		= true;

		// Add the gadget to this dialog...
		AddGadget( m_apxScrollBarGadgets[ u ] );
	}

	// Update the size of the scroll-bar...
	m_xMaxCoord	= xOffset + xMaxViewSize;

    m_xViewSize                  = m_xMaxViewSize;
	m_xViewSizeWithoutScrollBars = m_xMaxViewSize;
	m_xViewSizeWithoutScrollBars.x += m_apxScrollBarGadgets[ SBO_VERTICAL ]->m_fWidth;
	m_xViewSizeWithoutScrollBars.y += m_apxScrollBarGadgets[ SBO_HORIZONTAL ]->m_fHeight;

}

/// End of function 'Asura_Table_Dialog::Asura_Table_Dialog'. ///


const Asura_Vector_2 Asura_Table_Dialog::GetViewSize( void ) const
{
	Asura_Vector_2 xSize;
	xSize.x = ( m_bScrollBarNeeded[ SBO_VERTICAL ] )   ? m_xViewSize.x : m_xViewSizeWithoutScrollBars.x;
	xSize.y = ( m_bScrollBarNeeded[ SBO_HORIZONTAL ] ) ? m_xViewSize.y : m_xViewSizeWithoutScrollBars.y;
	return( xSize );
}

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Table_Dialog::~Asura_Table_Dialog
// Author       : Andy Tate
// Date         : 3-May-2007
////////////////////////////////////////////////////////////////////////////////

Asura_Table_Dialog::~Asura_Table_Dialog()
{
	DestroyTable();
}

/// End of function 'Asura_Table_Dialog::~Asura_Table_Dialog'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Table_Dialog::CreateTable
// Author       : Andy Tate
// Date         : 27-Apr-2007
////////////////////////////////////////////////////////////////////////////////

void Asura_Table_Dialog::CreateTable(const Asura_Vector_2& xMinCellSize,
									 const u_int uNumRows,
									 const u_int uNumColumns,
									 CreateCell* pfnCreateCell,
									 void* const pxCreateCellUserData,
									 const char* const* pszRowNames,
									 const char* const* pszColumnNames,
                                     CreateRowHeading* pfnCreateRowHeading, void* const pxCreateRowHeadingUserData,
                                     CreateColumnHeading* pfnCreateColumnHeading, void* const pxCreateColumnHeadingUserData )
{
	CreateTable( xMinCellSize, uNumRows, uNumColumns, pfnCreateCell, pxCreateCellUserData, pszRowNames, pszColumnNames, NULL, NULL,
                 pfnCreateRowHeading, pxCreateRowHeadingUserData, pfnCreateColumnHeading, pxCreateColumnHeadingUserData );
}

void Asura_Table_Dialog::CreateTable(const Asura_Vector_2& xMinCellSize,
									 const u_int uNumRows,
									 const u_int uNumColumns,
									 CreateCell* pfnCreateCell,
									 void* const pxCreateCellUserData,
									 const Asura_String* pxRowNames,
									 const Asura_String* pxColumnNames,
                                     CreateRowHeading* pfnCreateRowHeading, void* const pxCreateRowHeadingUserData,
                                     CreateColumnHeading* pfnCreateColumnHeading, void* const pxCreateColumnHeadingUserData )
{
	CreateTable( xMinCellSize, uNumRows, uNumColumns, pfnCreateCell, pxCreateCellUserData, NULL, NULL, pxRowNames, pxColumnNames,
                 pfnCreateRowHeading, pxCreateRowHeadingUserData, pfnCreateColumnHeading, pxCreateColumnHeadingUserData );
}

void Asura_Table_Dialog::CreateTable(const Asura_Vector_2& xMinCellSize,
									 const u_int uNumRows,
									 const u_int uNumColumns,
									 CreateCell* pfnCreateCell,
									 void* const pxCreateCellUserData,
									 const char* const* pszRowNames,
									 const char* const* pszColumnNames,
									 const Asura_String* pxRowNames,
									 const Asura_String* pxColumnNames,
                                     CreateRowHeading* pfnCreateRowHeading, void* const pxCreateRowHeadingUserData,
                                     CreateColumnHeading* pfnCreateColumnHeading, void* const pxCreateColumnHeadingUserData )
{
	if( m_pppxCellDialogues == NULL )
	{
		// Store the data...
		m_xMinCellSize		= xMinCellSize;
		m_uNumRows			= uNumRows;
		m_uNumColumns		= uNumColumns;

		// Create each row and column in-turn...
		m_pppxCellDialogues	= new Asura_TableCell_Dialog** [ m_uNumRows ];

		m_pfRowHeights			= new float [ m_uNumRows ];
		m_puRowDisplayIndices	= new u_int [ m_uNumRows ];

		m_pfColumnWidths			= new float [ m_uNumColumns ];
		m_puColumnDisplayIndices	= new u_int [ m_uNumColumns ];

		for(u_int uRow = 0; uRow < m_uNumRows; uRow++)
		{
			m_pfRowHeights[uRow]		= xMinCellSize[1];
			m_puRowDisplayIndices[uRow]	= uRow;
			m_pppxCellDialogues[uRow]	= new Asura_TableCell_Dialog* [ m_uNumColumns ];

			for(u_int uColumn = 0; uColumn < m_uNumColumns; uColumn++)
			{
				Asura_TableCell_Dialog* const pxCell = pfnCreateCell( *this, m_xOffset, m_xMinCellSize, uRow, uColumn, pxCreateCellUserData );
                Asura_Assert( pxCell, "Asura_Table_Dialog::CreateTable - you must create a table cell dialog for every cell" );
                // Few of the functions actually check m_pppxCellDialogues for validity, so we may as well crash now rather than later.
                // 27/07/2007 - NJD - See comment above SetOverlayPointer.
                pxCell->SetOverlayPointer( m_pxOverlay );
				pxCell->SetHideFlag(ASURA_DIALOG_HIDEFLAG_TABLE, IsHidden());
                m_pppxCellDialogues[ uRow ][ uColumn ] = pxCell;
			}
		}

		for(u_int uColumn = 0; uColumn < m_uNumColumns; uColumn++)
		{
			m_puColumnDisplayIndices[uColumn] = uColumn;
		}

		// Create the headings...
		if( ( pszRowNames != NULL ) || ( pxRowNames != NULL ) )
		{
            CreateRowHeading* pfnCreateRowHeadingLocal = pfnCreateRowHeading ? pfnCreateRowHeading : CreateStandardRowHeading;

			m_ppxRowHeadingDialogues	= new Asura_TableRowHeading_Dialog* [ m_uNumRows ];
			for( u_int uRow = 0; uRow < m_uNumRows; uRow++ )
			{
				const char* const szRowName = ( pszRowNames != NULL ) ? pszRowNames[ uRow ] : pxRowNames[ uRow ].GetString();
				m_ppxRowHeadingDialogues[uRow] =  pfnCreateRowHeadingLocal( *this, szRowName, m_xOffset, xMinCellSize, uRow, pxCreateRowHeadingUserData );
			}
		}

		if( ( pszColumnNames != NULL ) || ( pxColumnNames != NULL ) )
		{
            CreateColumnHeading* pfnCreateColumnHeadingLocal = pfnCreateColumnHeading ? pfnCreateColumnHeading : CreateStandardColumnHeading;

			m_ppxColumnHeadingDialogues	= new Asura_TableColumnHeading_Dialog* [ m_uNumColumns ];
			for( u_int uColumn = 0; uColumn < m_uNumColumns; uColumn++ )
			{
				const char* const szColumnName = ( pszColumnNames != NULL ) ? pszColumnNames[ uColumn ] : pxColumnNames[ uColumn ].GetString();
				m_ppxColumnHeadingDialogues[uColumn] = pfnCreateColumnHeadingLocal( *this, szColumnName, m_xOffset, xMinCellSize, uColumn, pxCreateColumnHeadingUserData );
			}
		}
	}
	else
	{
		Asura_Assert(false, "Already have cells for this table.");
	}
	UpdateAll();
}

/// End of function 'Asura_Table_Dialog::CreateTable'. ///

Asura_TableColumnHeading_Dialog* Asura_Table_Dialog::CreateStandardColumnHeading( Asura_Table_Dialog& xTableDialog, const char* szName,
                                                                                  const Asura_Vector_2& xOffset, const Asura_Vector_2& xMinCellSize,
																				  const u_int uColumn,
                                                                                  void* const pxUserData )
{
	return(Asura_TableColumnHeading_StandardDialog::Create( xTableDialog, szName, xOffset, xMinCellSize, uColumn ));
}

Asura_TableRowHeading_Dialog* Asura_Table_Dialog::CreateStandardRowHeading( Asura_Table_Dialog& xTableDialog, const char* szName,
                                                                            const Asura_Vector_2& xOffset, const Asura_Vector_2& xMinCellSize,
																			const u_int uRow,
                                                                            void* const pxUserData )
{
	return(Asura_TableRowHeading_StandardDialog::Create( xTableDialog, szName, xOffset, xMinCellSize, uRow ));
}

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Table_Dialog::DestroyTable
// Author       : Andy Tate
// Date         : 3-May-2007
////////////////////////////////////////////////////////////////////////////////

void Asura_Table_Dialog::DestroyTable(void)
{
	// Destroy the cells...
	if(m_pppxCellDialogues != NULL)
	{
		for(u_int uRow = 0; uRow < m_uNumRows; uRow++)
		{
			for(u_int uColumn = 0; uColumn < m_uNumColumns; uColumn++)
			{
				delete(m_pppxCellDialogues[uRow][uColumn]);
			}

			delete[](m_pppxCellDialogues[uRow]);
		}

		delete[](m_pppxCellDialogues);
		m_pppxCellDialogues	= NULL;
	}

	// Destroy the row-headings...
	if(m_ppxRowHeadingDialogues != NULL)
	{
		for(u_int uRow = 0; uRow < m_uNumRows; uRow++)
		{
			delete(m_ppxRowHeadingDialogues[uRow]);
		}

		delete[](m_ppxRowHeadingDialogues);
		m_ppxRowHeadingDialogues	= NULL;
	}

	// Destroy the column headings...
	if(m_ppxColumnHeadingDialogues != NULL)
	{
		for(u_int uColumn = 0; uColumn < m_uNumColumns; uColumn++)
		{
			delete(m_ppxColumnHeadingDialogues[uColumn]);
		}

		delete[](m_ppxColumnHeadingDialogues);
		m_ppxColumnHeadingDialogues	= NULL;
	}

	// Destroy the other arrays...
	delete[](m_pfRowHeights);
	delete[](m_puRowDisplayIndices);

	delete[](m_pfColumnWidths);
	delete[](m_puColumnDisplayIndices);

	// Reset the remaining data...
	m_uNumRows				= 0;
	m_pfRowHeights			= NULL;
	m_puRowDisplayIndices	= NULL;

	m_pfColumnWidths			= NULL;
	m_puColumnDisplayIndices	= NULL;
	m_uNumColumns				= 0;
}

/// End of function 'Asura_Table_Dialog::DestroyTable'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Table_Dialog::SortRowsUsingHeadings
// Author       : Andy Tate
// Date         : 8-May-2007
////////////////////////////////////////////////////////////////////////////////

void Asura_Table_Dialog::SortRowsUsingHeadings(const bool bBackwards)
{
	m_bSortBackwards = s_bSortBackwards	= bBackwards;
	s_pxSortTableDialogue = this;
	{
		qsort(m_puRowDisplayIndices, m_uNumRows, sizeof(m_puRowDisplayIndices[0]), SortByRowHeadingDialogues);
	}
	s_pxSortTableDialogue = NULL;

    UpdateAll();
}

/// End of function 'Asura_Table_Dialog::SortRowsUsingHeadings'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Table_Dialog::SortRowsUsingColumn
// Author       : Andy Tate
// Date         : 8-May-2007
////////////////////////////////////////////////////////////////////////////////

void Asura_Table_Dialog::SortRowsUsingColumn(const u_int uColumn, const bool bBackwards)
{
	m_bSortBackwards = s_bSortBackwards	= bBackwards;
	m_uSortIndex	 = s_uSortIndex		= uColumn;
	s_pxSortTableDialogue = this;
	{
		qsort(m_puRowDisplayIndices, m_uNumRows, sizeof(m_puRowDisplayIndices[0]), SortByCellDialoguesInColumn);
	}
	s_pxSortTableDialogue = NULL;

    UpdateAll();
}

/// End of function 'Asura_Table_Dialog::SortRowsUsingColumn'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Table_Dialog::SortColumnsUsingHeadings
// Author       : Andy Tate
// Date         : 8-May-2007
////////////////////////////////////////////////////////////////////////////////

void Asura_Table_Dialog::SortColumnsUsingHeadings(const bool bBackwards)
{
	m_bSortBackwards = s_bSortBackwards	= bBackwards;
	s_pxSortTableDialogue = this;
	{
		qsort(m_puColumnDisplayIndices, m_uNumColumns, sizeof(m_puColumnDisplayIndices[0]), SortByColumnHeadingDialogues);
	}
	s_pxSortTableDialogue = NULL;

    UpdateAll();
}

/// End of function 'Asura_Table_Dialog::SortColumnsUsingHeadings'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Table_Dialog::SortColumnsUsingRow
// Author       : Andy Tate
// Date         : 8-May-2007
////////////////////////////////////////////////////////////////////////////////

void Asura_Table_Dialog::SortColumnsUsingRow(const u_int uRow, const bool bBackwards)
{
	m_bSortBackwards = s_bSortBackwards	= bBackwards;
	m_uSortIndex	 = s_uSortIndex		= uRow;
	s_pxSortTableDialogue = this;
	{
		qsort(m_puColumnDisplayIndices, m_uNumColumns, sizeof(m_puColumnDisplayIndices[0]), SortByCellDialoguesInRow);
	}
	s_pxSortTableDialogue = NULL;

    UpdateAll();
}

/// End of function 'Asura_Table_Dialog::SortColumnsUsingRow'. ///


////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Table_Dialog::GetCell
// Author       : Mark Parry
// Date         : 5-July-2007
////////////////////////////////////////////////////////////////////////////////

Asura_TableCell_Dialog* Asura_Table_Dialog::GetCell( const u_int uRow, const u_int uColumn ) const
{
	if( ( uRow >= m_uNumRows ) || ( uColumn >= m_uNumColumns ) )
	{
		return( NULL );
	}
	return( m_pppxCellDialogues[ uRow ][ uColumn ] );
}

Asura_TableRowHeading_Dialog* Asura_Table_Dialog::GetRowHeading( const u_int uRow, const bool bCheckCells ) const
{
	if(uRow >= m_uNumRows)
	{
		return(NULL);
	}
	else if(m_ppxRowHeadingDialogues == NULL)
	{
		if(bCheckCells)
		{
			if(m_pppxCellDialogues == NULL)
			{
				return(NULL);
			}
			else
			{
				Asura_TableCell_Dialog*	pxCell	= m_pppxCellDialogues[uRow][0];

				return((pxCell->IsRowHeading()) ? static_cast<Asura_TableRowHeading_Dialog*>(pxCell) : NULL);
			}
		}
		else
		{
			return(NULL);
		}
	}
	else
	{
		return(m_ppxRowHeadingDialogues[uRow]);
	}
}

Asura_TableColumnHeading_Dialog* Asura_Table_Dialog::GetColumnHeading( const u_int uColumn, const bool bCheckCells ) const
{
	if(uColumn >= m_uNumColumns)
	{
		return(NULL);
	}
	else if(m_ppxColumnHeadingDialogues == NULL)
	{
		if(bCheckCells)
		{
			if(m_pppxCellDialogues == NULL)
			{
				return(NULL);
			}
			else
			{
				Asura_TableCell_Dialog*	pxCell	= m_pppxCellDialogues[0][uColumn];

				return((pxCell->IsColumnHeading()) ? static_cast<Asura_TableColumnHeading_Dialog*>(pxCell) : NULL);
			}
		}
		else
		{
			return(NULL);
		}
	}
	else
	{
		return(m_ppxColumnHeadingDialogues[uColumn]);
	}
}

void Asura_Table_Dialog::SetCurrentlySelectedRow( const int iRow )
{
	m_iCurrentlySelectedRow = iRow;

	// Position the row in the centre of the view, if possible.
	float fCellHeight = 0.0f;
	for ( u_int uRow = 0; uRow < m_uNumRows; ++uRow )
	{
		fCellHeight = Asura_Maths::Max( fCellHeight, m_pfRowHeights[ uRow ] );
	}
	if ( fCellHeight <= 0.0f )
	{
		return;
	}

	u_int uRenderRow = 0;
	for ( u_int uRow = 0; uRow < m_uNumRows; ++uRow )
	{
		if ( m_puRowDisplayIndices[ uRow ] == static_cast< u_int >( m_iCurrentlySelectedRow ) )
		{
			uRenderRow = uRow;
			break;
		}
	}
	const Asura_Vector_2 xViewSize( GetViewSize() );
	const float fHighestOffset = Asura_Maths::Max( 0.0f, m_xTableSize.y - xViewSize.y );

	m_xScrollBarValue.y = Asura_Maths::ClampToRange( uRenderRow * fCellHeight - xViewSize.y * 0.5f, 0.0f, fHighestOffset );
}

void Asura_Table_Dialog::StartResize(u_int uResizeFlags)
{
	m_xPreResizeViewSize = m_xViewSize;
}

void Asura_Table_Dialog::UpdateResize(u_int uResizeFlags, const Asura_Vector_2& xDelta)
{
	if(!m_bAllowResize) return;

	if(uResizeFlags & ASURA_OVERLAY_RESIZE_INITIAL)
	{
		//initialise the size based on the last size chosen by the user
		if(!m_pxUserSetSize) return;
		if(*m_pxUserSetSize == Asura_Maths::ZeroVector2) return;

		m_xMaxViewSize.x = Asura_Maths::Max(2.0f, m_pxUserSetSize->x);
		m_xMaxViewSize.y = Asura_Maths::Max(2.0f, m_pxUserSetSize->y);

	}
	else
	{
		m_xMaxViewSize.x = Asura_Maths::Max(2.0f, m_xPreResizeViewSize.x + xDelta.x);
		m_xMaxViewSize.y = Asura_Maths::Max(2.0f, m_xPreResizeViewSize.y + xDelta.y);

		if(m_pxUserSetSize)
		{
			m_pxUserSetSize->x = m_xMaxViewSize.x;
			m_pxUserSetSize->y = m_xMaxViewSize.y;
		}
	}

    {
        m_xViewSize.x = Asura_Maths::Min( m_xTableSize.x, m_xMaxViewSize.x );
        m_xViewSize.y = Asura_Maths::Min( m_xTableSize.y, m_xMaxViewSize.y );
        m_xViewSizeWithoutScrollBars = m_xViewSize;
        m_xViewSizeWithoutScrollBars.x += m_apxScrollBarGadgets[ SBO_VERTICAL ]->m_fWidth;
        m_xViewSizeWithoutScrollBars.y += m_apxScrollBarGadgets[ SBO_HORIZONTAL ]->m_fHeight;
    }
	
}

u_int Asura_Table_Dialog::GetAllowedResizeFlags() const
{
	return m_bAllowResize ? ASURA_OVERLAY_RESIZE_HORIZONTAL | ASURA_OVERLAY_RESIZE_VERTICAL : 0;
}

////////////////////////////////////////////////////////////////////////////////

#endif	// defined(ASURA_TOOLS)


/// End of file. ///
