//////////////////////////////////////////////////////////////////////////////
//
// Asura_TexturePicker_Dialog.h
//
// Copyright (c) 2007 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

#if defined ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#if !defined __ASURA_TEXTUREPICKER_DIALOG_H__
#define __ASURA_TEXTUREPICKER_DIALOG_H__

#if defined ASURA_TOOLS

/******************************************************************************/
/* Included Files                                                             */
/******************************************************************************/

// Asura Platform-independent
#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Gadget.h"
#include "Rendering/Asura_RenderStates.h"

/******************************************************************************/
/* Classes                                                                    */
/******************************************************************************/

////////////////////////////////////////////////////////////////////////////////
//
// Asura_TexturePicker_Gadget
//
// Gadget for displaying a texture
//
////////////////////////////////////////////////////////////////////////////////

class Asura_TexturePicker_Gadget : public Asura_Gadget
{
public:

	// Constructor
	Asura_TexturePicker_Gadget( const Asura_Vector_2& xOffset,
								int* const piTextureID,
								Asura_Hash_ID* const puTextureHash,
								Asura_String* const pxTextureName,
								const float fWidth,
								const float fHeight,
								bool* const pbFlag = NULL,
								const ASURA_TEXTUREOP_ID eTextureOp = ASURA_TEXTUREOP_SINGLE_PASS );

	Asura_TexturePicker_Gadget( const Asura_Vector_2& xOffset,
								Asura_Dialog_SharedValue<int>* const pxSharedTextureID,
								Asura_Dialog_SharedValue<Asura_Hash_ID>* const pxSharedTextureHash,
								Asura_Dialog_SharedValue<Asura_String>* const pxSharedTextureName,
								const float fWidth,
								const float fHeight,
								bool* const pbFlag = NULL,
								const ASURA_TEXTUREOP_ID eTextureOp = ASURA_TEXTUREOP_SINGLE_PASS );


	// Functions
	void Clear();
	void Reload();

	const int* GetID() const { return m_piTextureID; }
	const Asura_Hash_ID* GetHash() const { return m_puTextureHash; }
	const Asura_String* GetName() const { return m_pxTextureName; }

	void SetPickerFlags( const u_int uFlags ) { m_uPickerFlags = uFlags; }
	u_int GetPickerFlags() const { return m_uPickerFlags; }

	void SetValidateCubemap( const bool bCubemap ) { m_bCubemap = bCubemap; }
	bool GetValidateCubemap() const { return m_bCubemap; }

	// Virtual overrides
	virtual void Update( const Asura_Vector_2& xOrigin );

protected:
	void GetTextureIDAndHash(int* piTextureID, Asura_Hash_ID* puTextureHash);
	void PickTexture();


	// Virtual overrides
	virtual void Render( const Asura_Vector_2& xOrigin );

	// Data
	//for 'single target' version
	int*						m_piTextureID;
	Asura_Hash_ID*				m_puTextureHash;
	Asura_String*				m_pxTextureName;
	
	//for shared dialog version
	Asura_Dialog_SharedValue<int>* m_pxSharedTextureID;
	Asura_Dialog_SharedValue<Asura_Hash_ID>* m_pxSharedTextureHash;
	Asura_Dialog_SharedValue<Asura_String>* m_pxSharedTextureName;

	bool*						m_pbFlag;
	u_int						m_uPickerFlags;
	const ASURA_TEXTUREOP_ID	m_eTextureOp;
	bool						m_bCubemap;
};

////////////////////////////////////////////////////////////////////////////////
//
// Asura_TexturePicker_Dialog
//
// Displays a thumbnail image of a texture. When clicked, allows the user to
// choose a new one. Comes with a button to reset to ASURA_TEXTURE_NONE
//
////////////////////////////////////////////////////////////////////////////////

class Asura_TexturePicker_Dialog : public Asura_Dialog
{
public:

	// Constructors / destructor
	Asura_TexturePicker_Dialog( const char* const szName,
								const Asura_Vector_2& xOffset,
								int* const piTextureID,
								Asura_Hash_ID* const puTextureHash,
								Asura_String* const pxTextureName,
								const float fWidth,
								const float fHeight,
								bool* const pbFlag = NULL,
								const ASURA_TEXTUREOP_ID eTextureOp = ASURA_TEXTUREOP_SINGLE_PASS );

	Asura_TexturePicker_Dialog( const char* const szName,
								const Asura_Vector_2& xOffset,
								Asura_Dialog_SharedValue<int>* const pxSharedTextureID,
								Asura_Dialog_SharedValue<Asura_Hash_ID>* const pxSharedTextureHash,
								Asura_Dialog_SharedValue<Asura_String>* const pxSharedTextureName,
								const float fWidth,
								const float fHeight,
								bool* const pbFlag = NULL,
								const ASURA_TEXTUREOP_ID eTextureOp = ASURA_TEXTUREOP_SINGLE_PASS );


	void SetPickerFlags( const u_int uPickerFlags );
	u_int GetPickerFlags() const;

	void SetValidateCubemap( const bool bCubemap );
	bool GetValidateCubemap() const;

protected:

	// Virtual overrides
	virtual void ShowHelpText();

	// Static functions
	static void OnClear( int iData );
	static void OnReload( int iData );

	// Data
	Asura_TexturePicker_Gadget* m_pxTextureGadget;
	Asura_Icon_Gadget* m_pxClearButton;
	Asura_Icon_Gadget* m_pxReloadButton;
};

#endif // defined ASURA_TOOLS

#endif // !defined __ASURA_TEXTUREPICKER_DIALOG_H__
