//*******************************************************************
// File: Asura_Job_TaskQueue.cpp
//
// Created: 27 June 2007 DLH
//
// Description: 
//
//*******************************************************************

// system libraries
#include "Core/Asura.h"

#if defined( ASURA_SUPPORT_MULTITHREADED_JOB_QUEUE)

// main header
#include "Jobs/Asura_Job_TaskQueue.h"

// other libraries
#include "Core/Asura_Atomic.h"

// internal dependencies

//*******************************************************************
// Constants
//*******************************************************************

//ASURA_STATIC_ASSERT( 4 == sizeof( Asura_Job_TaskQueue::TaskQueuePointer::m_xData));

//*******************************************************************
// Local Classes
//*******************************************************************


//*******************************************************************
// Local Datas
//*******************************************************************


//*******************************************************************
// Function Definitions
//*******************************************************************

#if defined( ASURA_JOB_TASK_QUEUE_DEBUG)
Asura_Job_TaskQueue::TaskQueue::Debug::Debug()
{
	this->m_uTransactionID = 0;
	this->m_eTransactionType = UNSET;
	this->m_pxTask = NULL;
}

u_int Asura_Job_TaskQueue::TaskQueue::StartPop( )
{
	const u_int uActive = Asura_Atomic::Increment( this->m_uDebugActive);
	const u_int uDebugIndex = Asura_Atomic::Increment( this->m_uDebugIndex);
	const u_int uTransactionID = uDebugIndex;
	const u_int uDebugArrayIndex = uDebugIndex % this->s_uDebugSize;
	Debug& xDebug = this->m_axDebug[ uDebugArrayIndex];
	
	xDebug.m_eTransactionType = Debug::POP_START;
	xDebug.m_pxTask = NULL;
	xDebug.m_xTaskStart = this->m_xTasksQueueStart;
	xDebug.m_xTaskEnd = this->m_xTasksQueueLast;
	xDebug.m_uSize = this->m_uTasksQueueSize;
	xDebug.m_uTransactionID = uTransactionID;
	xDebug.m_uActive = uActive;
	
	return( uTransactionID);
}

void Asura_Job_TaskQueue::TaskQueue::EndPop( u_int uTransactionID, Task* pxTask)
{
	const u_int uDebugIndex = Asura_Atomic::Increment( this->m_uDebugIndex);
	const u_int uDebugArrayIndex = uDebugIndex % this->s_uDebugSize;
	const u_int uActive = Asura_Atomic::Decrement( this->m_uDebugActive);

	Debug& xDebug = this->m_axDebug[ uDebugArrayIndex];
	
	xDebug.m_eTransactionType = Debug::POP_END;
	xDebug.m_pxTask = pxTask;
	xDebug.m_xTaskStart = this->m_xTasksQueueStart;
	xDebug.m_xTaskEnd = this->m_xTasksQueueLast;
	xDebug.m_uSize = this->m_uTasksQueueSize;
	xDebug.m_uTransactionID = uTransactionID;
	xDebug.m_uActive = uActive + 1;
}

u_int Asura_Job_TaskQueue::TaskQueue::StartPush( Task* pxTask)
{
	const u_int uActive = Asura_Atomic::Increment( this->m_uDebugActive);
	const u_int uDebugIndex = Asura_Atomic::Increment( this->m_uDebugIndex);
	const u_int uTransactionID = uDebugIndex;
	const u_int uDebugArrayIndex = uDebugIndex % this->s_uDebugSize;
	Debug& xDebug = this->m_axDebug[ uDebugArrayIndex];
	
	xDebug.m_eTransactionType = Debug::PUSH_START;
	xDebug.m_pxTask = pxTask;
	xDebug.m_xTaskStart = this->m_xTasksQueueStart;
	xDebug.m_xTaskEnd = this->m_xTasksQueueLast;
	xDebug.m_uSize = this->m_uTasksQueueSize;
	xDebug.m_uTransactionID = uTransactionID;
	xDebug.m_uActive = uActive;
	
	return( uTransactionID);
}

void Asura_Job_TaskQueue::TaskQueue::EndPush( u_int uTransactionID)
{
	const u_int uDebugIndex = Asura_Atomic::Increment( this->m_uDebugIndex);
	const u_int uDebugArrayIndex = uDebugIndex % this->s_uDebugSize;
	const u_int uActive = Asura_Atomic::Decrement( this->m_uDebugActive);
	
	Debug& xDebug = this->m_axDebug[ uDebugArrayIndex];
	
	xDebug.m_eTransactionType = Debug::PUSH_END;
	xDebug.m_pxTask = NULL;
	xDebug.m_xTaskStart = this->m_xTasksQueueStart;
	xDebug.m_xTaskEnd = this->m_xTasksQueueLast;
	xDebug.m_uSize = this->m_uTasksQueueSize;
	xDebug.m_uTransactionID = uTransactionID;
	xDebug.m_uActive = uActive + 1;
}
#endif

Asura_Job_TaskQueue::Asura_Job_TaskQueue( u_int uTaskNumber, u_int uDataSize) :
	m_uFreeTaskSize( uTaskNumber),
	m_uTaskNumber( uTaskNumber),
	m_uDataSize( uDataSize)
{
	const u_int uTasksToAllocate = 2 + uTaskNumber;
	
	this->m_paxTasks = new Task[ uTasksToAllocate];
	this->m_pucData = new uint8[ uTaskNumber * uDataSize];
	
	Task* pxTaskNULL = NULL;
	
#if defined( ASURA_ASSERT)
	#if defined( ASURA_JOB_TASK_QUEUE_TAG)
	// DLH - Check that the TaskQueuePointer() stores the pointer data correctly
	Asura_Assert( NULL == TaskQueuePointer( NULL, this->m_paxTasks).GetPointer( this->m_paxTasks), "Pointers do not match");
	for( u_int uTaskIndex = 0; uTaskIndex < uTasksToAllocate; uTaskIndex += 1)
	{
		Task* pxTask = this->m_paxTasks + uTaskIndex;
		
		Asura_Assert( pxTask == TaskQueuePointer( pxTask, this->m_paxTasks).GetPointer( this->m_paxTasks), "Pointers do not match");
	}
		
	#endif
#endif
	
	// DLH - Setup Task Queue
	{
		Task &xTask = this->m_paxTasks[ 0];
		TaskQueue &xQueue = this->m_xTaskQueue;
		
		xTask.m_pData = NULL;
		xTask.m_xNext.SetPointer( pxTaskNULL, this->m_paxTasks);
		
		xQueue.m_uTasksQueueSize = 0;
		xQueue.m_xTasksQueueStart.SetPointer( &xTask, this->m_paxTasks);
		xQueue.m_xTasksQueueLast.SetPointer( &xTask, this->m_paxTasks);
		
#if defined( ASURA_JOB_TASK_QUEUE_USE_LOCK)	
		xQueue.m_uLocked =  0;
#endif
#if defined( ASURA_JOB_TASK_QUEUE_DEBUG)
		xQueue.m_uDebugIndex = 0;
		xQueue.m_uDebugActive = 0;
#endif
	}
	
	// DLH - Free Task Queue
	{
		Task &xTask = this->m_paxTasks[ 1];
		TaskQueue &xQueue = this->m_xFreeQueue;
		
		xTask.m_pData = NULL;
		xTask.m_xNext.SetPointer( pxTaskNULL, this->m_paxTasks);
		
		xQueue.m_uTasksQueueSize = 0;
		xQueue.m_xTasksQueueStart.SetPointer( &xTask, this->m_paxTasks);
		xQueue.m_xTasksQueueLast.SetPointer( &xTask, this->m_paxTasks);
		
#if defined( ASURA_JOB_TASK_QUEUE_USE_LOCK)	
		xQueue.m_uLocked =  0;
#endif
#if defined( ASURA_JOB_TASK_QUEUE_DEBUG)
		xQueue.m_uDebugIndex = 0;
		xQueue.m_uDebugActive = 0;
#endif
	}
	
	{
		// DLH - Initialise Tasks in array as all on the Free List 
		uint8* pucData = this->m_pucData;
		for( u_int uTaskIndex = 0; uTaskIndex < uTaskNumber; uTaskIndex += 1, pucData += uDataSize)
		{
			Task& xTask = this->m_paxTasks[ 2 + uTaskIndex];
			
			xTask.m_pData = pucData;
			xTask.m_xNext.SetPointer( pxTaskNULL, this->m_paxTasks);
			
			this->CS_Push( this->m_xFreeQueue, &xTask);
		}
	}
	
	Asura_PrintfToBfr( this->m_szName, "%s", sizeof( this->m_szName), true, "UNKNOWN");
}

Asura_Job_TaskQueue::~Asura_Job_TaskQueue()
{
	delete[] this->m_pucData;
	delete[] this->m_paxTasks;
	this->m_paxTasks = NULL;
	this->m_pucData = NULL;
}

#if defined( ASURA_JOB_TASK_QUEUE_DEBUG)
bool Asura_Job_TaskQueue::IsValid( TaskQueue &xQueue)
{
#if defined( ASURA_JOB_TASK_QUEUE_USE_LOCK)	
	while( 0 != Asura_Atomic::CompareAndSwap( xQueue.m_uLocked, 0, 1));
#endif
	
	u_int uTaskCount = 0;
	Task* pxTaskNULL = NULL; 
	Task* pxTask = xQueue.m_xTasksQueueStart.GetPointer( this->m_paxTasks);
	Task* pxTaskLast = NULL; 
	while( pxTaskNULL != pxTask)
	{
		pxTaskLast = pxTask;
		pxTask = pxTask->m_xNext.GetPointer( this->m_paxTasks);
		uTaskCount += 1;
	}
	
	if( xQueue.m_xTasksQueueLast.GetPointer( this->m_paxTasks) != pxTaskLast)
	{
		__debugbreak();
		
#if defined( ASURA_JOB_TASK_QUEUE_USE_LOCK)	
		while( 1 != Asura_Atomic::CompareAndSwap( xQueue.m_uLocked, 1, 0));
#endif
	
		return( false);
	}
	
	if( (1 + xQueue.m_uTasksQueueSize) != uTaskCount)
	{
		__debugbreak();
		
#if defined( ASURA_JOB_TASK_QUEUE_USE_LOCK)	
		while( 1 != Asura_Atomic::CompareAndSwap( xQueue.m_uLocked, 1, 0));
#endif
	
		return( false);
	}
	
#if defined( ASURA_JOB_TASK_QUEUE_USE_LOCK)	
	while( 1 != Asura_Atomic::CompareAndSwap( xQueue.m_uLocked, 1, 0));
#endif
	
	return( true);
}
#endif

bool Asura_Job_TaskQueue::IsValid()
{
#if defined( ASURA_JOB_TASK_QUEUE_DEBUG)
	bool bIsValidFree = this->IsValid( this->m_xFreeQueue);
	bool bIsValidTask = this->IsValid( this->m_xTaskQueue);
	
	bool bIsValid = bIsValidFree && bIsValidTask;
	
	if( 0 != this->m_uFreeTaskSize)
	{
		__debugbreak();
	}
	
	if( this->m_uTaskNumber != (this->m_xFreeQueue.m_uTasksQueueSize + this->m_xTaskQueue.m_uTasksQueueSize + this->m_uFreeTaskSize))
	{
		__debugbreak();
	}
	
	return( bIsValid);
#else
	return( true);
#endif
}

#if defined( ASURA_JOB_TASK_QUEUE_TAG)
//----------------------------------------------------------------------
// Queue 
// Uses an algorithum from [1]. It allows the queue to be access in a lock-free way.
// This means that no OS level syncronisation objects where used in the accessing of this queue!!
// Hence, improving performance.
//
// [1] Maged M Michael, "Simple, Fast, and Practical Non-Blocking and Blocking Concurrent Queue Algorithums", Univeristy of Rochester
// 
// Does not suffer form the ABA problem that the JDV algorithum does.
// But needs to Tag Pointers so compares know that NULL is not the same as it used to be.
// As we only have a uint32 CAS and not a uint64 CAS (Win32 does not support this) the pointer has had to be mangled with the Tag in the same 32 bit value.
// I have used the fact that we know all the Tasks are in a limited memory area (array) and not spreadout all over the memory space.
//
//----------------------------------------------------------------------

inline uint16 Asura_Job_TaskQueue::TaskQueuePointer::PointerToMem( Task* pxTask, Task* pxTaskBase)
{
	uint16 uMemCalc = (uint16)(pxTask - pxTaskBase) + 1;
	uint16 uMem = (NULL == pxTask) ? 0 : uMemCalc;
	return( uMem);
}

inline Asura_Job_TaskQueue::Task* Asura_Job_TaskQueue::TaskQueuePointer::MemToPointer( uint16 uTag, Task* pxTaskBase)
{
	const uint16 uMem = this->m_xData.m_xData32.m_uMem;
	Task* pxTaskCalc = pxTaskBase + (uMem - 1);
	
	Task* pxTask = (0 == uMem) ? NULL : pxTaskCalc;
	
	return( pxTask);
}

inline Asura_Job_TaskQueue::TaskQueuePointer::TaskQueuePointer()
{
	this->m_xData.m_xData32.m_uMem = NULL;
	this->m_xData.m_xData32.m_uTag = 0;
}

inline Asura_Job_TaskQueue::TaskQueuePointer::TaskQueuePointer( const TaskQueuePointer& xPointer)
{
	this->m_xData.m_uData32 = xPointer.m_xData.m_uData32;
}

inline Asura_Job_TaskQueue::TaskQueuePointer::TaskQueuePointer( Task* pxTask, Task* pxTaskBase, u_int uTag)
{
	this->m_xData.m_xData32.m_uMem = this->PointerToMem( pxTask, pxTaskBase);
	this->m_xData.m_xData32.m_uTag = (uint16)uTag;
}

inline Asura_Job_TaskQueue::TaskQueuePointer::TaskQueuePointer( Task* pxTask, Task* pxTaskBase)
{
	this->m_xData.m_xData32.m_uMem = this->PointerToMem( pxTask, pxTaskBase);
	this->m_xData.m_xData32.m_uTag = 0;
}

inline Asura_Job_TaskQueue::TaskQueuePointer& Asura_Job_TaskQueue::TaskQueuePointer::operator =( TaskQueuePointer& xPointer)
{
	this->m_xData.m_uData32 = xPointer.m_xData.m_uData32;
	return( *this);
}

//inline Asura_Job_TaskQueue::Task* Asura_Job_TaskQueue::TaskQueuePointer::operator =( Task* pxTask)
//{
//	this->m_xData.m_xData32.m_uMem = this->PointerToMem( pxTask);
//	
//	return( pxTask);
//}

inline bool Asura_Job_TaskQueue::TaskQueuePointer::operator ==( TaskQueuePointer& xPointer)
{
	bool bEqual = (this->m_xData.m_uData32 == xPointer.m_xData.m_uData32);
	
	return( bEqual);
}

inline uint32 volatile &Asura_Job_TaskQueue::TaskQueuePointer::GetAtomic()
{ 
	return( this->m_xData.m_uData32);
}

inline Asura_Job_TaskQueue::Task* Asura_Job_TaskQueue::TaskQueuePointer::GetPointer( Asura_Job_TaskQueue::Task* pxTaskBase)
{ 
	Task* pxTask = this->MemToPointer( this->m_xData.m_xData32.m_uMem, pxTaskBase);

	return( pxTask);
}

inline Asura_Job_TaskQueue::Task* Asura_Job_TaskQueue::TaskQueuePointer::SetPointer( Task* pxTask, Task* pxTaskBase)
{
	this->m_xData.m_xData32.m_uMem = this->PointerToMem( pxTask, pxTaskBase);
	
	return( pxTask);
}

inline u_int Asura_Job_TaskQueue::TaskQueuePointer::GetTag()
{ 
	return( this->m_xData.m_xData32.m_uTag);
}

inline void Asura_Job_TaskQueue::TaskQueuePointer::SetTag( u_int uTag)
{ 
	this->m_xData.m_xData32.m_uTag = (uint16)uTag;
}

Asura_Job_TaskQueue::Task* Asura_Job_TaskQueue::CS_Pop( TaskQueue &xQueue)
{
#if defined( ASURA_JOB_TASK_QUEUE_USE_LOCK)	
	while( 0 != Asura_Atomic::CompareAndSwap( xQueue.m_uLocked, 0, 1));
#endif
	
#if defined( ASURA_JOB_TASK_QUEUE_DEBUG)
	u_int xTransationID = xQueue.StartPop();
#endif

	Task* pxTaskNULL = NULL;
	Task* pxTask;
	void* pData = NULL;
	
	TaskQueuePointer xQueueStart;
	TaskQueuePointer xQueueLast;
	
	do
	{
		xQueueStart = xQueue.m_xTasksQueueStart;
		xQueueLast = xQueue.m_xTasksQueueLast;
	
		pxTask = xQueueStart.GetPointer( this->m_paxTasks);
	
		TaskQueuePointer xNext = pxTask->m_xNext;
		
		if( xQueueStart == xQueue.m_xTasksQueueStart)
		{
			Task* pxTaskNext = xNext.GetPointer( this->m_paxTasks);
			
			if( xQueueStart.GetPointer( this->m_paxTasks) == xQueueLast.GetPointer( this->m_paxTasks))
			{
				if( NULL == pxTaskNext)
				{
#if defined( ASURA_JOB_TASK_QUEUE_DEBUG)
					xQueue.EndPop( xTransationID, (Task*)0xFFFFFFFF);
#endif
#if defined( ASURA_JOB_TASK_QUEUE_USE_LOCK)	
					while( 1 != Asura_Atomic::CompareAndSwap( xQueue.m_uLocked, 1, 0));
#endif
					return( NULL);
				}
				
				{
					//TaskQueuePointer xLastNew( xNext.GetPointer( this->m_paxTasks), 1 + xNext.GetTag());
					TaskQueuePointer xLastNew( pxTaskNext, this->m_paxTasks, 1 + xQueueLast.GetTag());
					
					Asura_Atomic::CompareAndSwap( xQueue.m_xTasksQueueLast.GetAtomic(), xQueueLast.GetAtomic(), xLastNew.GetAtomic());
				}
			}
			else
			{
				if( NULL == pxTaskNext)
				{
					continue;
				}
				
				pData = pxTaskNext->m_pData;
				
				TaskQueuePointer xStartNew( pxTaskNext, this->m_paxTasks, 1 + xQueueStart.GetTag());
					
				TaskQueuePointer xTaskOld;
				xTaskOld.GetAtomic() = Asura_Atomic::CompareAndSwap( xQueue.m_xTasksQueueStart.GetAtomic(), xQueueStart.GetAtomic(), xStartNew.GetAtomic());
				if( xTaskOld == xQueueStart)
				{
					break;
				}
			}
		}
	}
	while( true);

	pxTask->m_xNext.SetPointer( pxTaskNULL, this->m_paxTasks);
	pxTask->m_pData = pData;
	
#if defined( ASURA_ASSERT) || defined( ASURA_JOB_TASK_QUEUE_DEBUG)
	u_int uTasksQueueSize =
#endif
	Asura_Atomic::Decrement( xQueue.m_uTasksQueueSize);
	
#if defined( ASURA_JOB_TASK_QUEUE_DEBUG)
	u_int uFreeTasksSize =
#endif
	Asura_Atomic::Increment( this->m_uFreeTaskSize);
	
#if defined( ASURA_JOB_TASK_QUEUE_DEBUG)
	if( this->m_uTaskNumber < uTasksQueueSize)
	{
		__debugbreak();
	}
	if( this->m_uTaskNumber < uFreeTasksSize)
	{
		__debugbreak();
	}
#endif
	
#if defined( ASURA_JOB_TASK_QUEUE_DEBUG)
	xQueue.EndPop( xTransationID, pxTask);
#endif

	Asura_Assert( pxTask != xQueue.m_xTasksQueueStart.GetPointer( this->m_paxTasks), "This is not good");
	Asura_Assert( NULL != pData, "This is not Good");
	
	Asura_Assertf( uTasksQueueSize <= this->m_uTaskNumber, ("Size %d Max %d", uTasksQueueSize, this->m_uTaskNumber));
	
#if defined( ASURA_JOB_TASK_QUEUE_USE_LOCK)	
	while( 1 != Asura_Atomic::CompareAndSwap( xQueue.m_uLocked, 1, 0));
#endif

	return( pxTask);
}

void Asura_Job_TaskQueue::CS_Push( TaskQueue &xQueue, Task* pxTask)
{
#if defined( ASURA_JOB_TASK_QUEUE_USE_LOCK)	
	while( 0 != Asura_Atomic::CompareAndSwap( xQueue.m_uLocked, 0, 1));
#endif
	
#if defined( ASURA_JOB_TASK_QUEUE_DEBUG)
	u_int xTransationID = xQueue.StartPush( pxTask);
#endif

	Task* const pxTaskNULL = NULL;
	
	pxTask->m_xNext.SetPointer( pxTaskNULL, this->m_paxTasks);
	
#if defined( ASURA_JOB_TASK_QUEUE_DEBUG)
	u_int uFreeTasksSize =
#endif
	Asura_Atomic::Decrement( this->m_uFreeTaskSize);

#if defined( ASURA_ASSERT) || defined( ASURA_JOB_TASK_QUEUE_DEBUG)
	u_int uTasksQueueSize =
#endif
	Asura_Atomic::Increment( xQueue.m_uTasksQueueSize);
	
#if defined( ASURA_JOB_TASK_QUEUE_DEBUG)
	if( this->m_uTaskNumber < uTasksQueueSize)
	{
		__debugbreak();
	}
	if( this->m_uTaskNumber < uFreeTasksSize)
	{
		__debugbreak();
	}
#endif
	
	Asura_Assertf( uTasksQueueSize <= this->m_uTaskNumber, ("Size %d Max %d", uTasksQueueSize, this->m_uTaskNumber));

	TaskQueuePointer xQueueLast;
	
	do
	{
		xQueueLast = xQueue.m_xTasksQueueLast;

		Task* pxTaskLast = xQueueLast.GetPointer( this->m_paxTasks);
	
		TaskQueuePointer xNext = pxTaskLast->m_xNext;
		
		if( xQueueLast == xQueue.m_xTasksQueueLast)
		{
			Task* pxTaskNext = xNext.GetPointer( this->m_paxTasks);
			if( NULL == pxTaskNext)
			{
				TaskQueuePointer xLastNextNew( pxTask, this->m_paxTasks, 1 + xNext.GetTag());
				TaskQueuePointer xLastNextOld;
				xLastNextOld.GetAtomic() = Asura_Atomic::CompareAndSwap( pxTaskLast->m_xNext.GetAtomic(), xNext.GetAtomic(), xLastNextNew.GetAtomic());
				if( xLastNextOld == xNext)
				{
					break;
				}
			}
			else
			{
				TaskQueuePointer xLastNew( pxTaskNext, this->m_paxTasks, 1 + xQueueLast.GetTag());
				Asura_Atomic::CompareAndSwap( xQueue.m_xTasksQueueLast.GetAtomic(), xQueueLast.GetAtomic(), xLastNew.GetAtomic());
			}
		}
		
	}
	while( true);
	
	{
		TaskQueuePointer xLastNew( pxTask, this->m_paxTasks, 1 + xQueueLast.GetTag());
		Asura_Atomic::CompareAndSwap( xQueue.m_xTasksQueueLast.GetAtomic(), xQueueLast.GetAtomic(), xLastNew.GetAtomic());
	}
	
#if defined( ASURA_JOB_TASK_QUEUE_DEBUG)
	xQueue.EndPush( xTransationID);
#endif

#if defined( ASURA_JOB_TASK_QUEUE_USE_LOCK)	
	while( 1 != Asura_Atomic::CompareAndSwap( xQueue.m_uLocked, 1, 0));
#endif
}
#endif

#if defined( ASURA_JOB_TASK_QUEUE_JDV)

//----------------------------------------------------------------------
// Queue 
// Uses an algorithum from [1]. It allows the queue to be access in a lock-free way.
// This means that no OS level syncronisation objects where used in the accessing of this queue!!
// Hence, improving performance.
//
// [1] John D. Valois, "Implementing Lock-Free Queues", Rensselaer Polytechnic Institute
// 
// Works until the ABA case kicks in. 
//		Node A is Pushed onto List X
//		Node B is Pushed onto List X. But is paused after it gets the pointer to A as the last item on the list
//		All the node on X are poped including X. Any amount can be pushed.
//		Node A is Pushed onto List Y. Node A's Next pointer is NULL the same a when Node was poped onto List X.
//		Node B being pushed onto List X is woken up. Node A still has a NULL next pointer. Node B is addedf to List Y and not List X.
//----------------------------------------------------------------------

inline Asura_Job_TaskQueue::TaskQueuePointer::TaskQueuePointer()
{
	this->m_pxTask = NULL;
}

inline Asura_Job_TaskQueue::TaskQueuePointer::TaskQueuePointer( const TaskQueuePointer& xPointer)
{
	this->m_pxTask = xPointer.m_pxTask;
}

inline Asura_Job_TaskQueue::TaskQueuePointer::TaskQueuePointer( Task* pxTask, Task* /*pxTaskBase*/)
{
	this->m_pxTask = pxTask;
}

inline Asura_Job_TaskQueue::TaskQueuePointer& Asura_Job_TaskQueue::TaskQueuePointer::operator=( TaskQueuePointer& xPointer)
{
	this->m_pxTask = xPointer.m_pxTask;
	return( *this);
}

//inline Asura_Job_TaskQueue::Task* Asura_Job_TaskQueue::TaskQueuePointer::operator=( Task* pxTask)
//{
//	this->m_pxTask = pxTask;
//	
//	return( pxTask);
//}

inline Asura_Job_TaskQueue::Task* volatile &Asura_Job_TaskQueue::TaskQueuePointer::GetAtomic()
{ 
	return( this->m_pxTask);
}

inline Asura_Job_TaskQueue::Task* Asura_Job_TaskQueue::TaskQueuePointer::SetPointer( Task* pxTask, Task* /*pxTaskBase*/) 
{ 
	this->m_pxTask = pxTask;
	
	return( pxTask);
}

inline Asura_Job_TaskQueue::Task* Asura_Job_TaskQueue::TaskQueuePointer::GetPointer( Task* pxTaskBase) 
{ 
	return( this->m_pxTask);
}

Asura_Job_TaskQueue::Task* Asura_Job_TaskQueue::CS_Pop( TaskQueue &xQueue)
{
#if defined( ASURA_JOB_TASK_QUEUE_USE_LOCK)	
	while( 0 != Asura_Atomic::CompareAndSwap( xQueue.m_uLocked, 0, 1));
#endif
	
#if defined( ASURA_JOB_TASK_QUEUE_DEBUG)
	u_int xTransationID = xQueue.StartPop();
#endif

#if defined( ASURA_ASSERT)
	TaskQueue xQueueOld = xQueue;
#endif

	Task* pxTaskNULL = NULL;
	Task* pxTask;
	void* pData = NULL;
	
	do
	{
		pxTask = xQueue.m_xTasksQueueStart.GetPointer( this->m_paxTasks);
		Task* pxTaskNext = pxTask->m_xNext.GetPointer( this->m_paxTasks);
		if( pxTaskNULL == pxTaskNext)
		{
#if defined( ASURA_JOB_TASK_QUEUE_USE_LOCK)	
			while( 1 != Asura_Atomic::CompareAndSwap( xQueue.m_uLocked, 1, 0));
#endif
#if defined( ASURA_JOB_TASK_QUEUE_DEBUG)
			xQueue.EndPop( xTransationID, (Task*)0xFFFFFFFF);
#endif
			return( NULL);
		}
		
		pData = pxTaskNext->m_pData;
		Task* pxQueueStartOld = Asura_Atomic::CompareAndSwapPointer( xQueue.m_xTasksQueueStart.GetAtomic(), pxTask, pxTaskNext);
		if( pxTask == pxQueueStartOld)
		{
			break;
		}
	}
	while( true);

	pxTask->m_xNext.SetPointer( pxTaskNULL, this->m_paxTasks);
	pxTask->m_pData = pData;
	
#if defined( ASURA_ASSERT) || defined( ASURA_JOB_TASK_QUEUE_DEBUG)
	u_int uTasksQueueSize =
#endif
	Asura_Atomic::Decrement( xQueue.m_uTasksQueueSize);
	
#if defined( ASURA_JOB_TASK_QUEUE_DEBUG)
	u_int uFreeTasksSize =
#endif
	Asura_Atomic::Increment( this->m_uFreeTaskSize);
	
#if defined( ASURA_JOB_TASK_QUEUE_DEBUG)
	if( this->m_uTaskNumber < uTasksQueueSize)
	{
		__debugbreak();
		xQueueOld = xQueueOld;
	}
	if( this->m_uTaskNumber < uFreeTasksSize)
	{
		__debugbreak();
		xQueueOld = xQueueOld;
	}
#endif
	
#if defined( ASURA_JOB_TASK_QUEUE_DEBUG)
	xQueue.EndPop( xTransationID, pxTask);
#endif

	Asura_Assert( pxTask != xQueue.m_xTasksQueueStart.GetPointer( this->m_paxTasks), "This is not good");
	Asura_Assert( NULL != pData, "This is not Good");
	
	Asura_Assertf( uTasksQueueSize <= this->m_uTaskNumber, ("Size %d Max %d", uTasksQueueSize, this->m_uTaskNumber));
	
#if defined( ASURA_JOB_TASK_QUEUE_USE_LOCK)	
	while( 1 != Asura_Atomic::CompareAndSwap( xQueue.m_uLocked, 1, 0));
#endif

	return( pxTask);
}

void Asura_Job_TaskQueue::CS_Push( TaskQueue &xQueue, Task* pxTask)
{
#if defined( ASURA_JOB_TASK_QUEUE_USE_LOCK)	
	while( 0 != Asura_Atomic::CompareAndSwap( xQueue.m_uLocked, 0, 1));
#endif
	
#if defined( ASURA_JOB_TASK_QUEUE_DEBUG)
	u_int xTransationID = xQueue.StartPush( pxTask);
#endif

	Task* const pxTaskNULL = NULL;
	Task* pxTaskLast;
	
	pxTask->m_xNext.SetPointer( pxTaskNULL, this->m_paxTasks);
	
#if defined( ASURA_JOB_TASK_QUEUE_DEBUG)
	u_int uFreeTasksSize =
#endif
	Asura_Atomic::Decrement( this->m_uFreeTaskSize);

#if defined( ASURA_ASSERT) || defined( ASURA_JOB_TASK_QUEUE_DEBUG)
	u_int uTasksQueueSize =
#endif
	Asura_Atomic::Increment( xQueue.m_uTasksQueueSize);
	
#if defined( ASURA_JOB_TASK_QUEUE_DEBUG)
	if( this->m_uTaskNumber < uTasksQueueSize)
	{
		__debugbreak();
	}
	if( this->m_uTaskNumber < uFreeTasksSize)
	{
		__debugbreak();
	}
#endif
	
	Asura_Assertf( uTasksQueueSize <= this->m_uTaskNumber, ("Size %d Max %d", uTasksQueueSize, this->m_uTaskNumber));

	do
	{
		pxTaskLast = xQueue.m_xTasksQueueLast.GetPointer( this->m_paxTasks);
		Task* pxTaskNextOld = Asura_Atomic::CompareAndSwapPointer( pxTaskLast->m_xNext.GetAtomic(), pxTaskNULL, pxTask);
		if( pxTaskNULL != pxTaskNextOld)
		{
			Asura_Atomic::CompareAndSwapPointer( xQueue.m_xTasksQueueLast.GetAtomic(), pxTaskLast, pxTaskNextOld);
		}
		else
		{
			break;
		}
	}
	while( true);
	
	Asura_Atomic::CompareAndSwapPointer( xQueue.m_xTasksQueueLast.GetAtomic(), pxTaskLast, pxTask);
	
#if defined( ASURA_JOB_TASK_QUEUE_DEBUG)
	xQueue.EndPush( xTransationID);
#endif

#if defined( ASURA_JOB_TASK_QUEUE_USE_LOCK)	
	while( 1 != Asura_Atomic::CompareAndSwap( xQueue.m_uLocked, 1, 0));
#endif
}

#endif

//----------------------------------------------------------------------
// Task Queue 
//----------------------------------------------------------------------

Asura_Job_TaskQueue::Task* Asura_Job_TaskQueue::CS_QueuePop()
{
	Task* pxTask = this->CS_Pop( this->m_xTaskQueue);
	
	return( pxTask);
}

void Asura_Job_TaskQueue::CS_QueuePush( Task* pxTask)
{
	this->CS_Push( this->m_xTaskQueue, pxTask);
}

//----------------------------------------------------------------------
// Free Queue
//----------------------------------------------------------------------

Asura_Job_TaskQueue::Task* Asura_Job_TaskQueue::CS_FreePop()
{
	Task* pxTask = this->CS_Pop( this->m_xFreeQueue);

	return( pxTask);
}

void Asura_Job_TaskQueue::CS_FreePush( Task* pxTask)
{
#if 1 || defined( _DEBUG)
	Asura_Assert( NULL != pxTask, "Not a valid pointer. Must be set.");
#endif

	this->CS_Push( this->m_xFreeQueue, pxTask);
}

void Asura_Job_TaskQueue::Clear( void)
{
	for( Task* pxTask = this->CS_QueuePop(); NULL != pxTask; pxTask = this->CS_QueuePop())
	{
		this->CS_FreePush( pxTask);
	}
}

u_int Asura_Job_TaskQueue::GetNumberOfQueuedTasks() const
{
	return( this->m_xTaskQueue.m_uTasksQueueSize);
}

u_int Asura_Job_TaskQueue::GetNumberOfFreeTasks() const
{
	return( this->m_xFreeQueue.m_uTasksQueueSize);
}

bool Asura_Job_TaskQueue::IsIdle() const
{
	return( this->m_uTaskNumber == this->m_xFreeQueue.m_uTasksQueueSize);
}

bool Asura_Job_TaskQueue::PopTask( Task* &pxTask)
{
	bool bResult = false;
	
	pxTask = this->CS_QueuePop();
	
	if( NULL != pxTask)
	{
		bResult = true;
	}
	else
	{
		// DLH - 28 June 2007 10:59:25 - This is here mainly so I can break point this condition
		bResult = false;
	}


	return bResult;
}

bool Asura_Job_TaskQueue::Push( const void* pData)
{
	bool bResult = false;

	Task* pxTask = this->CS_FreePop();
	
	if( NULL != pxTask)
	{
		
		// DLH - At this point the only knolledge of this Task is in this stack. It is not on any of the lists.
		memcpy( pxTask->GetData(), pData, this->m_uDataSize);
		
		this->CS_QueuePush( pxTask);

		bResult = true;
	}
	else
	{
		// DLH - 28 June 2007 10:59:25 - This is here mainly so I can break point this condition
		bResult = false;
	}

	return bResult;
}

bool Asura_Job_TaskQueue::Pop( void* pData)
{
	bool bResult = false;

	Task* pxTask = this->CS_QueuePop();
	
	if( NULL != pxTask)
	{
		// DLH - At this point the only knolledge of this Task is in this stack. It is not on any of the lists.
		memcpy( pData, pxTask->GetData(), this->m_uDataSize);
		
		this->CS_FreePush( pxTask);
		
		bResult = true;
	}

	return bResult;
}

bool Asura_Job_TaskQueue::ReturnTask( Task* &pxTask)
{
	this->CS_FreePush( pxTask);
	
	return( true);
}

void Asura_Job_TaskQueue::SetName( const char* pszName)
{
	Asura_PrintfToBfr( this->m_szName, "%s", sizeof( this->m_szName), true, pszName);
}

#endif // defined( ASURA_SUPPORT_MULTITHREADED_JOB_QUEUE)

//*******************************************************************
//
// (c) 2006 Rebellion
//
// Warning: This computer program is protected by copyright law and 
// international treaties. Unauthorized reproduction or distribution
// of this program, or any portion of it, may result in severe civil and
// criminal penalties, and will be prosecuted to the maximum extent 
// possible under the law.
//
//*******************************************************************
