//
// Asura_GI_Solution.cpp
//
// Created by KJFL - 2004-07-08 12:08:36
//
// Copyright (C) 2004 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

// This file's header.
#include "Light/Asura_GI_Solution.h"

#include "Core/Asura_DynamicBVH.h"
#include "Core/Asura_Mempool.h"
#include "Core/Memory/Asura_ResourceAllocator.h"
#include "Debugging/Asura_CPUProfile.h"
#include "Debugging/Asura_Debug_Timers.h"
#include "Light/Asura_Photon.h" // to get dynamic lights
#include "Rendering/Asura_Colour.h"

#if defined(ASURA_USE_DEBUGOVERLAY) && defined(ASURA_SUPPORT_COMMAND_CONSOLE)
#include "Debugging/Asura_DebugOverlay.h"
#include "Debugging/Asura_DebugLineRender.h"
#include "Interface/Asura_CommandConsole.h"
#include "Camera/Asura_Camera.h"
#endif

#ifdef ASURA_TOOLS
#include "Tools_System.h"
#include "Lighting/Tools_GlobalIllumination_EntitySamples.h"
#include "Lighting/Tools_PhotonMapping_System.h"
#include "Lighting/Tools_LightSets.h"
#include "Load&Save/Tools_SaveAsura.h"
#endif


//#include "Debugging/Asura_DebugText.h"
//#include "Rendering/Asura_Font_System.h"

static const u_int uMAX_NUMBER_OF_PROXIMAL_SAMPLES = 6;

#if defined(ASURA_USE_DEBUGOVERLAY) && defined(ASURA_SUPPORT_COMMAND_CONSOLE)

static int dbg_iLoops = 0;
static int dbg_iTotalCalls = 0;
static int dbg_iMaxLoopsPerFrame = 0;
static int dbg_iMaxDepthPerFrame = 0;

static int dbg_iMaxCalls = 0;
static int dbg_iMaxLoopsEver = 0;
static int dbg_iMaxDepthEver = 0;

static bool g_bShowGISamples = true; //only shown when in that debug mode anyway

#endif

///////////////////////
// Asura_GI_Solution //
///////////////////////

Asura_SphericalHarmonics_Order2_RGB					Asura_GI_Solution::s_xSHForCurrentRender;
Asura_SphericalHarmonics_Order2_RGB					Asura_GI_Solution::s_xStaticSHForCurrentRender;

#if defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM
Asura_SphericalHarmonics_Order2_RGB					Asura_GI_Solution::s_xDynamicSHForCurrentRender;
#endif

#ifdef ASURA_TOOLS
bool												Asura_GI_Solution::s_bDisableStaticLighting = false;
#endif

#ifdef ASURA_MULTIPLE_GI_HANDLER
Asura_GI_Solution_Handler*							Asura_GI_Solution::s_pxCurrentHandler;
Asura_TemplateHashTree<Asura_GI_Solution_Handler>	Asura_GI_Solution::s_xHandlerTree;
#else //#ifdef ASURA_MULTIPLE_GI_HANDLER
Asura_GI_Solution_Handler							Asura_GI_Solution::s_xHandler;
#endif //#else //#ifdef ASURA_MULTIPLE_GI_HANDLER

#if defined(ASURA_USE_DEBUGOVERLAY) && defined(ASURA_SUPPORT_COMMAND_CONSOLE)

void Asura_GI_Solution::DoDebugOverlay (void* const pxIgnore )
{
	int iAverageLoops = 0;
	if(dbg_iTotalCalls != 0)
	{
		iAverageLoops = dbg_iLoops / dbg_iTotalCalls;
	}	

	dbg_iMaxCalls = Asura_Maths::Max(dbg_iMaxCalls,dbg_iTotalCalls);
	dbg_iMaxLoopsEver = Asura_Maths::Max(dbg_iMaxLoopsEver,dbg_iMaxLoopsPerFrame);
	dbg_iMaxDepthEver = Asura_Maths::Max(dbg_iMaxDepthEver,dbg_iMaxDepthPerFrame);

	Asura_DebugOverlay::DrawConsoleText( ASURA_DO_STATIC_IRRADIANCE, "Calls To GetStaticIradiance:%4d (Peak: %4d)", dbg_iTotalCalls, dbg_iMaxCalls);
	Asura_DebugOverlay::DrawConsoleText( ASURA_DO_STATIC_IRRADIANCE, "Total samples: %4d", GetCurrentHandler().GetNumberOfSamples());
	Asura_DebugOverlay::DrawConsoleText( ASURA_DO_STATIC_IRRADIANCE, "Average Loop Iterations: %4d, Max iterations: %4d (Peak: %4d)", iAverageLoops, dbg_iMaxLoopsPerFrame, dbg_iMaxLoopsEver);
	Asura_DebugOverlay::DrawConsoleText( ASURA_DO_STATIC_IRRADIANCE, "Max depth: %4d (Peak: %4d)", dbg_iMaxDepthPerFrame, dbg_iMaxDepthEver);

	dbg_iLoops = 0;
	dbg_iTotalCalls = 0;
	dbg_iMaxLoopsPerFrame = 0;
	dbg_iMaxDepthPerFrame = 0;

	GetCurrentHandler().DebugDrawGISamples();
}

static bool ResetGIPeaks( bool bHelp )
{
	if ( bHelp )
	{
		Asura_CommandConsole::Printf("Reset statistics in static irradiance debug mode\n");
	}
	else
	{
		dbg_iMaxCalls = 0;
		dbg_iMaxLoopsEver = 0;
		dbg_iMaxDepthEver = 0;
	}

	return true;
}

static inline bool DebugGiTreeCullRight(Asura_Vector_3 xCamPos, Asura_Vector_3 xCamDir, float fDist, u_int uAxis, Asura_Vector_3 xPos )
{
	return true;
//BW for some reason this isn't working, culls stuff it shouldn't
//	//check if area to the +ve side of plane can be seen (45degree frustum cull)
//	if ( xCamPos[uAxis] > xPos[uAxis]
//		|| ( xCamDir[uAxis] > -0.72f && xCamPos[uAxis]+fDist > xPos[uAxis] )
//		return true;
//	return false;
}
static inline bool DebugGiTreeCullLeft(Asura_Vector_3 xCamPos, Asura_Vector_3 xCamDir, float fDist, u_int uAxis, Asura_Vector_3 xPos )
{
	return true;
//BW for some reason this isn't working, culls stuff it shouldn't
//	//check if area to the -ve side of plane can be seen(45degree frustum cull)
//	if ( xCamPos[uAxis] < xPos[uAxis]
//		|| ( xCamDir[uAxis] < 0.72f && xCamPos[uAxis]-fDist < xPos[uAxis] )
//		return true;
//	return false;
}

//show the GI samples - the vertices of any polys marked with "Use For Entity GI Samples" flag
//there's an overhead for looking up lighting for entities & decals roughly proportional to the number of samples
void Asura_GI_Solution_Handler::DebugDrawGISamples( void )
{
	static float fDist = 8.0f;
	float fDistSq = fDist*fDist;
		
	if ( !g_bShowGISamples || m_pxSamples == NULL || m_uNumberOfSamples == 0 )
		return;

	//show the GI samples
	//walk the tree to show all of them within a certain distance of the camera
	Asura_Vector_3 xCamPos = *Asura_Camera::GetPosition();
	Asura_Vector_3 xCamDir;
	Asura_Camera::GetForwardVector(xCamDir);
	u_int uNumDrawn = 0;
	u_int uNode = 1;
	while ( uNode != 0 && uNumDrawn < 300 )
	{
		Asura_Vector_3 xSampPos = Asura_Vector_3(m_pxSamples[uNode].m_afPosition);
		Asura_Vector_3 xSampPosRel = xSampPos - xCamPos;
		float fSampDistSq = xSampPosRel.MagnitudeSquared();
		if ( fSampDistSq < fDistSq
			&& ((xSampPosRel * xCamDir)/Asura_Maths::Sqrt(fSampDistSq) > 0.9f) //0.5f //simple frustum check (120 degree cone)
			)//&& Asura_Maths::GetFloorLog2(uNode) < 13 ) //skip the deepest branches, otherwise we just have too much stuff to draw
		{
			//pick a colour based on position in hierarchy
			
			//measure of left/rightishness
			u_int uLR = uNode << (Asura_Maths::GetFloorLog2(m_uNumberOfSamples)-Asura_Maths::GetFloorLog2(uNode));
			uLR = (uLR*0x100)/m_uNumberOfSamples;//16*Asura_Maths::GetFloorLog2(uNode);
			if ( uLR > 0xff ) uLR=0xff;

			//measure of up/downishness
			u_int uUD = 14*Asura_Maths::GetFloorLog2(uNode);
			if ( uUD > 0xff ) uUD=0xff;

			//red-green indicates left-right, blue indicates depth (blue=root, noblue = leaf)
			u_int uCol = 0xff000000 + 0x00000100*uLR + 0x00010000*(0xff-uLR) + 0x00000001*(0xff-uUD);

			//draw it
			//Asura_DebugLineRender::AddSphere(xSampPos, 0.3f, 0xFF00FF00, 8, 4);
			Asura_DebugLineRender::AddCross(xSampPos, uCol, 0.1f);
			uNumDrawn++;
		}

		//need to follow both branches
		if ( uNode <= (m_uNumberOfSamples>>1) && DebugGiTreeCullLeft(xCamPos,xCamDir,fDist,m_pxSamples[uNode].GetPlaneAxis(),m_pxSamples[uNode].m_afPosition) )
		{
			//go to left child
			uNode = (uNode<<1);
		}
		else
		{
			//go to parent
			u_int uParent = (uNode>>1);
			while ( uParent != 0 )
			{
				//if it has a right child, which is not where we just came from
				//BW could do this quicker: we basically want to look for 0 bits in uNode, set them to 1, and test them
				if ( (uParent<<1) + 1 != uNode && (uParent<<1) + 1 < m_uNumberOfSamples
					&& DebugGiTreeCullRight(xCamPos,xCamDir,fDist,m_pxSamples[uParent].GetPlaneAxis(),m_pxSamples[uParent].m_afPosition) )
					break;
				uNode = uParent;
				uParent = (uParent>>1);
			}
			if ( uParent == 0 )
				uNode = uParent;
			else
				uNode = (uParent<<1) + 1; //go to right child
		}
	}
	//tree layout:
	//position: 000 001 010 011 100 101 110 111
	//meaning:    .   p  pl  pr pll plr prl prr etc. (p is root parent, r is right branch, l is left, . is unused)
	
	//adjust draw dist based on num drawn, want to get close to but not over 300
	{
		static float fRate = 1.0f;
		float fDelta = 0.99f;


		if ( uNumDrawn == 300 )
		{
			if ( fRate > 1.0f ) fRate = 1.0f;
			fRate *= fDelta;
		}
		else if ( uNumDrawn < 250 && fDist < 50.0f )
		{
			if ( fRate < 1.0f ) fRate = 1.0f;
			fRate /= fDelta;
		}
		else
		{
			fRate = 1.0f;
		}
		fDist *= fRate;
	}
}

#endif //#if defined(ASURA_USE_DEBUGOVERLAY) && defined(ASURA_SUPPORT_COMMAND_CONSOLE)


void Asura_GI_Solution::Initialise()
{
#if defined(ASURA_USE_DEBUGOVERLAY) && defined(ASURA_SUPPORT_COMMAND_CONSOLE)
	Asura_DebugOverlay::RegisterMode( "Static Irradiance", ASURA_DO_STATIC_IRRADIANCE, DoDebugOverlay, NULL );

	Asura_CommandConsole::AddCmd( L"Render.GISamples.ResetGiPeaks", ResetGIPeaks, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::AddVar( L"Render.GISamples.Show", &g_bShowGISamples, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
#endif

#ifdef ASURA_MULTIPLE_GI_HANDLER

	#ifdef ASURA_TOOLS
	//For the multiple light sets I'm just going to use the index value for the 'hash'
	//The tree will be entirely unbalanced , but it probably won't matter
	//RAR 27/03/2006
	s_pxCurrentHandler = CreateHandler(ASURA_HASH_ID_UNSET);
	#endif

#else // #ifdef ASURA_MULTIPLE_GI_HANDLER

	s_xHandler.Initialise();
#endif //#else // #ifdef ASURA_MULTIPLE_GI_HANDLER
}

void Asura_GI_Solution::Shutdown()
{
#ifdef ASURA_MULTIPLE_GI_HANDLER
	// Just free up the whole tree
	s_xHandlerTree.Reset();

#else // #ifdef ASURA_MULTIPLE_GI_HANDLER

	s_xHandler.Shutdown();
#endif //#else // #ifdef ASURA_MULTIPLE_GI_HANDLER
}

void Asura_GI_Solution::Reset()
{
#ifdef ASURA_MULTIPLE_GI_HANDLER
	s_xHandlerTree.Traverse(Reset_Callback);

#else // #ifdef ASURA_MULTIPLE_GI_HANDLER

	s_xHandler.Reset();
#endif //#else // #ifdef ASURA_MULTIPLE_GI_HANDLER
}

#ifdef ASURA_MULTIPLE_GI_HANDLER
void Asura_GI_Solution::Reset_Callback(Asura_Hash_ID uHash, Asura_GI_Solution_Handler* pxData)
{
	// Don't need the hash, just reset the data in the pointer
	if (pxData)
	{
		pxData->Reset();
	}
}

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_GI_Solution::CreateHandler
// Author       : Anthony Gregson
// Date         : 24-Mar-2006
//
// Description  : Attempts to create a GI solution handler with the specified
//				  hash-ID.
//
// Parameters   : The hash-ID of the handler to create.
//
// Return value : A pointer to the handler created.
////////////////////////////////////////////////////////////////////////////////

Asura_GI_Solution_Handler* Asura_GI_Solution::CreateHandler(const Asura_Hash_ID uHash)
{
	Asura_GI_Solution_Handler* pxHandler = s_xHandlerTree.GetDataByHash(uHash);

	// Do we already have this handler?
	if(pxHandler == NULL)
	{
		// No - create one...
		pxHandler = s_xHandlerTree.AddNode(uHash);
	}
	else
	{
		// Yes - this shouldn't happen...
		Asura_Assert(false, "Already have a GI solution handler with this hash ID.");
	}

	// Return the handler...
	return(pxHandler);
}

/// End of function 'Asura_GI_Solution::CreateHandler'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_GI_Solution::DestroyHandler
// Author       : Anthony Gregson
// Date         : 24-Mar-2006
//
// Description  : Attempts to destroy the GI solution handler with the specified
//				  unique hash-ID.
//
// Parameters   : The hash-ID of the handler to destroy.
//
// Return value : "true" if destroyed, "false" if failed.
////////////////////////////////////////////////////////////////////////////////

bool Asura_GI_Solution::DestroyHandler(const Asura_Hash_ID uHash)
{
	// Call the delete function...
	return(s_xHandlerTree.DeleteNodeByHash(uHash));
}

#ifdef ASURA_TOOLS
void Asura_GI_Solution::Tools_InitialiseFromPhotonMappingSystem()
{
	ASURA_CPU_PROFILE_SCOPED_TIMER( "Asura_GI_Solution::Tools_InitialiseFromPhotonMappingSystem");
	
	//calculate for each light set
	u_int uInitialLightSets = Tools_LightSets::GetCurrentLightSets();
	for(Tools_UsedLightSetIterator xIt; !xIt.Done(); xIt.Next())
	{
		Tools_LightSets::SetCurrentLightSets(xIt.GetCurrent(),false);
		GetCurrentHandler().Tools_InitialiseFromPhotonMappingSystem();
	}
	Tools_LightSets::SetCurrentLightSets(uInitialLightSets,false);
}

void Asura_GI_Solution::Tools_GenerateFakeSample()
{
	//calculate for each light set
	u_int uInitialLightSets = Tools_LightSets::GetCurrentLightSets();
	for(Tools_UsedLightSetIterator xIt; !xIt.Done(); xIt.Next())
	{
		Tools_LightSets::SetCurrentLightSets(xIt.GetCurrent(),false);
		GetCurrentHandler().Tools_GenerateFakeSample();
	}
	Tools_LightSets::SetCurrentLightSets(uInitialLightSets,false);
}

#endif   //ASURA_TOOLS

#ifdef ASURA_TOOLS
bool Asura_GI_Solution::AreSamplesValid()
{
	//Check each light set in use
	bool bValid = true;

	u_int uInitialLightSets = Tools_LightSets::GetCurrentLightSets();
	for(Tools_UsedLightSetIterator xIt;!xIt.Done() && bValid;xIt.Next())
	{	
		Tools_LightSets::SetCurrentLightSets(xIt.GetCurrent(),false);
		bValid &= GetCurrentHandler().AreSamplesValid();
	}
	Tools_LightSets::SetCurrentLightSets(uInitialLightSets,false);

	return bValid;
}
#endif //ASURA_TOOLS

/// End of function 'Asura_GI_Solution::DestroyHandler'. ///
#endif //#ifdef ASURA_MULTIPLE_GI_HANDLER

///////////////////////////////
// Asura_GI_Solution_Handler //
///////////////////////////////

static u_int g_auStack[32];

Asura_GI_Solution_Handler::Asura_GI_Solution_Handler()
{
#ifdef ASURA_TOOLS
	m_bSamplesValid = false;
#endif //#ifdef ASURA_TOOLS

	// AJG - 27/03/06 - Non-streaming one doesn't need to initialise at this point
#ifdef ASURA_MULTIPLE_GI_HANDLER
	Initialise();
#endif //#ifdef ASURA_MULTIPLE_GI_HANDLER
}

Asura_GI_Solution_Handler::~Asura_GI_Solution_Handler()
{
	// AJG - 27/03/06 - Non-streaming one doesn't need to shutdown at this point
#ifdef ASURA_MULTIPLE_GI_HANDLER
	Shutdown();
#endif //#ifdef ASURA_MULTIPLE_GI_HANDLER
}

void Asura_GI_Solution_Handler::Initialise( void )
{
	Asura_ResourcePushType(ASURA_RESOURCE_GLOBAL_ILLUMATION);
	{
		m_uNumberOfSamples = 0;
		m_pxSamples = 0;

		Asura_ResourceAllocateArray(m_ppxProximalSamples, uMAX_NUMBER_OF_PROXIMAL_SAMPLES);
		Asura_ResourceAllocateArray(m_fProximalSampleDistancesSquared, uMAX_NUMBER_OF_PROXIMAL_SAMPLES);
	}
	Asura_ResourcePopType();
}

void Asura_GI_Solution_Handler::Shutdown( void )
{
	Asura_ResourcePushType(ASURA_RESOURCE_GLOBAL_ILLUMATION);
	{
		Asura_ResourceDeallocateArray(m_fProximalSampleDistancesSquared);
		Asura_ResourceDeallocateArray(m_ppxProximalSamples);

		Asura_ResourceDeallocateArray(m_pxSamples);
		m_pxSamples = 0;
		m_uNumberOfSamples = 0;

#ifdef ASURA_TOOLS
		m_bSamplesValid = false;
#endif
	}
	Asura_ResourcePopType();
}

void Asura_GI_Solution_Handler::Reset( void )
{
	Asura_ResourcePushType(ASURA_RESOURCE_GLOBAL_ILLUMATION);
	{
		// want to keep the proximal sample stuff - the rest can go

		Asura_ResourceDeallocateArray(m_pxSamples);
		m_pxSamples = 0;
		m_uNumberOfSamples = 0;
	}
	Asura_ResourcePopType();
}

void Asura_GI_Solution_Handler::GetStaticIrradiance( Asura_SphericalHarmonics_Order2_RGB& xResult, const Asura_Vector_3& xPosition )
{
#if defined( ASURA_USE_DEBUGOVERLAY )
	++dbg_iTotalCalls;
#endif

	xResult.Zero();
	const Asura_GI_IrradianceSample* *	ppxProximalSamples = m_ppxProximalSamples;
	float*								pfProximalSampleDistancesSquared = m_fProximalSampleDistancesSquared;

	// KJFL - 02 November 2004 - for continuity with older art work (e.g. currently porting WorldWarZero), option
	// to use original entity lights instead of using GI data
#ifdef ASURA_USE_OLD_ENTITY_LIGHTS
	
	GetStaticIrradianceFromOldLighting( xResult, xPosition );
	return;

#else //!ASURA_USE_OLD_ENTITY_LIGHTS

#ifdef ASURA_TOOLS
	if(Asura_GI_Solution::s_bDisableStaticLighting)
	{
		//Lighting disabled , so just stick with the zeroed result. RAR 05/08/2005
		return ;
	}

	// if we're in the tools, just lazy evaluate the values
	if ( Tools_System::IsActive() )
	{
		Tools_GlobalIllumination_EntitySamples::GetNearestSample( xPosition, xResult );
		return;
	}
#endif

	if (m_pxSamples==0 || m_uNumberOfSamples==0)
	{
		Asura_Assert( false, "There are no GI samples. Please flag level and/or select Recalc Entity GI Solution in the GI mode." );

		// KJFL - let's be nice and return an ambient white light from above so people can actually see things.
		// Note, I used to use the lazy eval scheme from above if this was a tools build, but people complained
		// that this made playing the game in tools builds too jerky, and debugging fa too annoying.
		xResult.m_afRedCoefficients[2] = xResult.m_afGreenCoefficients[2] = xResult.m_afBlueCoefficients[2] = -1.0f;
		
		return;
	}

	float fMaxDistanceSquared = 0;
	u_int uNumberOfProximalSamples = 0;
	u_int uFurthestSample = 0;
	u_int* puStack = g_auStack;

	*puStack++ = 0;

	u_int uIndex = 1;

	u_int uMaxDepth = 0;
	
	const float fPositionX = xPosition.x;
	const float fPositionY = xPosition.y;
	const float fPositionZ = xPosition.z;

	int iLoops = 0;
	while(true)
	{
		const Asura_GI_IrradianceSample& xSample = m_pxSamples[uIndex];
	
		const u_int uAxis = xSample.GetPlaneAxis();
	
		const float fPlaneDist = xPosition[uAxis] - xSample.m_afPosition[uAxis];

/**************************************************************************
 * Notes on optimization of this code, by RCD
 * I've made several changes to this code to make it compile better on the 
 * PS2 so here are some explanations of what I've done and why it helps.
 * 
 * Surprisingly the optimizer had already made a pretty good go at simplifying
 * the logic used, but I improved this further by noting that in the case
 * of positive or negative sides of the plane, all that really needed to
 * happen was to swap over the left and right indices.  So I made them share
 * the same value to start with and changed the value of whichever one was
 * the other side.
 *
 * Also, I was able to combine some of the previous cases because they 
 * were so similar, so that in the end very little had to be evaluated.
 * 
 * Altogether this simplified the output code to only two stores instead 
 * of the previous six.   
 *
 * In addition I placed several things into temporary local storage to cut 
 * down on the cost of constantly reloading or storing them.  For example
 * in the original code, the position vector was reloaded every time it was 
 * used (despite being const!) and so I've placed the there components in
 * local floats.  Also, when the distance was being calculated, the compiler
 * for whatever reason decided that it would be best to create a local vector
 * on the stack and then compute its length rather than do it all in registers
 * so another 3 floats were added to hold the interim results for this and so
 * removed the extra stores.
 *
 * Overall these changes only made a small difference to the code size (down 
 * by 38 instructions, out of about 260) but this was enough to make a 
 * difference to the performance - especially as most of the improvements 
 * are in the pieces which are executed the most - the loop which searches 
 * all the lighting sample data and finds the 6 nearest ones.
 **************************************************************************/
		// implicit children: 2*n is left, 2*n+1 is right
		u_int uLeftIndex = uIndex * 2;
		u_int uRightIndex = uLeftIndex;	// we'll add one to whichever one is needed in a minute

		if (uLeftIndex < m_uNumberOfSamples - 1)
		{
			// effectively this code swaps the two values in the negative case, so we no longer have to proces two cases !
			if (fPlaneDist>0)
			{
				uRightIndex++;
			}
			else
			{
				uLeftIndex++;	// note we can only get away with this because the value is not reused, but recreated on next loop
			}

			if ( (uNumberOfProximalSamples < uMAX_NUMBER_OF_PROXIMAL_SAMPLES )
			  || (fPlaneDist * fPlaneDist < fMaxDistanceSquared) )
			{
				*puStack++ = uLeftIndex;
			}
			*puStack++ = uRightIndex;

		}

		u_int uDepth = puStack - g_auStack;
		if (uDepth > uMaxDepth)
		{
			uMaxDepth = uDepth;
		}

		uIndex = *(--puStack); //stack only hangs onto neighbours to parse later
				

		if (uIndex == 0) break; //reached top of stack

		{
			const float fDistX = xSample.m_afPosition[0] - fPositionX;
			const float fDistY = xSample.m_afPosition[1] - fPositionY;
			const float fDistZ = xSample.m_afPosition[2] - fPositionZ;

			const float fDistanceSquared = (fDistX * fDistX) + (fDistY * fDistY) + (fDistZ * fDistZ);

			// if we haven't reached the maximum number of samples, just add it
			if (uNumberOfProximalSamples < uMAX_NUMBER_OF_PROXIMAL_SAMPLES)
			{
				// keep track of the max distance found, and which sample it was
				if (fDistanceSquared > fMaxDistanceSquared)
				{
					fMaxDistanceSquared = fDistanceSquared;
					uFurthestSample = uNumberOfProximalSamples;
				}

				// set the data to record the proximal sample
				ppxProximalSamples[uNumberOfProximalSamples] = &xSample;
				pfProximalSampleDistancesSquared[uNumberOfProximalSamples] = fDistanceSquared;

				++uNumberOfProximalSamples;
			}
			// otherwise only add it if it's closer than the samples we already have
			else if (fDistanceSquared < fMaxDistanceSquared)
			{
				// replace the furthest out sample
				ppxProximalSamples[uFurthestSample] = &xSample;
				pfProximalSampleDistancesSquared[uFurthestSample] = fDistanceSquared;

				// And find the new furthest out sample.
				// Since number of proximal samples is likely to always be small (<=5 say) we'll
				// just test the distances directly.
				uFurthestSample = 0;
				fMaxDistanceSquared = pfProximalSampleDistancesSquared[0];

				for (u_int uSample=1; uSample<uMAX_NUMBER_OF_PROXIMAL_SAMPLES; uSample++)
				{
					if (fMaxDistanceSquared < pfProximalSampleDistancesSquared[uSample])
					{
						fMaxDistanceSquared = pfProximalSampleDistancesSquared[uSample];
						uFurthestSample = uSample;
					}
				}
			}
		}

		iLoops++;
	}
#if defined( ASURA_USE_DEBUGOVERLAY )
	dbg_iLoops += iLoops;
	dbg_iMaxLoopsPerFrame = Asura_Maths::Max(dbg_iMaxLoopsPerFrame,iLoops);
	dbg_iMaxDepthPerFrame = Asura_Maths::Max(dbg_iMaxDepthPerFrame,(int)uMaxDepth);
#endif

	//Asura_DebugText::AddTextBlock(1,1,0xffffffff,0, "Searched %d, depth %d", uSearches, uMaxDepth);

	
	//Asura_Font_System::Print(0.102f, 0.252f, 0.7f, 0xff00ff00, ASURA_TEXTFORMAT_RELATIVEPOSITION | ASURA_TEXTFORMAT_FORMATSTRING, L"Searched %d, depth %d", uSearches, uMaxDepth);



	// we now have a list of most proximal samples

	if (uNumberOfProximalSamples)
	{
		float fTotal = 0;

		for (u_int u=0; u<uNumberOfProximalSamples; u++)
		{
			// nudge above zero to avoid singularities
			pfProximalSampleDistancesSquared[u] += 0.001f;
			const float fScale = 1.0f / pfProximalSampleDistancesSquared[u];

			fTotal += fScale;

			Asura_SphericalHarmonics_Order2_RGB xSample;

			// uncompress sample
			for (u_int v=0; v<Asura_SphericalHarmonics_Order2_RGB::NUMBER_OF_COEFFICIENTS; v++)
			{
				#ifdef ASURA_TARGET_SM3
					xSample[v] = ppxProximalSamples[u]->m_afCoefficients[v];
				#else
					xSample[v] = (static_cast<float>(ppxProximalSamples[u]->m_aucCompressedCoefficients[v])) * m_afDecompressCoefficientScale[v];
					xSample[v] += m_afDecompressCoefficientMinimum[v];
				#endif
			}
			
			xResult.ScaledAdd( xSample, fScale );
		
		}

		xResult.Scale(1.0f / (float)fTotal);
	}
#endif // ASURA_USE_OLD_ENTITY_LIGHTS ... else
}


class DynamicIrradianceBVHCallbackFunctor
{

public:

    DynamicIrradianceBVHCallbackFunctor( const Asura_Vector_3& xTestPos, Asura_SphericalHarmonics_Order2_RGB& xResult )
        : m_xTestPos( xTestPos )
        , m_xResult( xResult )
    {
    }

    void operator()( const Asura_Photon& xPhoton )
    {
		float fRange = xPhoton.GetRange();

        if( (fRange <= 0.0f) || !xPhoton.IsActive() ) { return; }

		Asura_Vector_3 xDir = xPhoton.GetPosition() - m_xTestPos;

		float fMag = xDir.Normalise();

		// TODO - check bounding box?
		// TODO - direction will be invalid when fMag = 0

		if (fMag < fRange)
		{
			float fBrightness = 1.0f - fMag / fRange;

			float fR = fBrightness * xPhoton.GetRedComponent();
			float fG = fBrightness * xPhoton.GetGreenComponent();
			float fB = fBrightness * xPhoton.GetBlueComponent();
				
			m_xResult.AddSampleForDirection( fR, fG, fB, xDir );		
		}
    }

    const Asura_Vector_3& m_xTestPos;
    Asura_SphericalHarmonics_Order2_RGB& m_xResult;

};


void Asura_GI_Solution_Handler::GetDynamicIrradiance( Asura_SphericalHarmonics_Order2_RGB& xResult, const Asura_Vector_3& xPosition )
{

#if 1

	xResult.Zero();

	DynamicIrradianceBVHCallbackFunctor xBVHCallbackFunctor( xPosition, xResult );
    Asura_Photon_System::GetDynamicPhotonBVH().TestPoint( xPosition, xBVHCallbackFunctor );

	xResult.ScaleCoefficientsForIrradiance();

#else

	xResult.Zero();

	Asura_TrackingHandlePool<Asura_Photon>* pxPhotonList = Asura_Photon_System::GetDynamicPhotonList();

	Asura_TrackingHandlePool<Asura_Photon>::Iterator xIterator = pxPhotonList->GetIterator();

	for (xIterator.Begin(); !xIterator.End(); xIterator.Next())
	{
        Asura_Photon* pxPhoton = xIterator.GetCurrentPointer();

		float fRange = pxPhoton->GetRange();

		if (fRange <= 0.0f || !pxPhoton->IsActive()) continue;

		Asura_Vector_3 xDir = pxPhoton->GetPosition() - xPosition;

		float fMag = xDir.Normalise();

		// TODO - check bounding box?
		// TODO - direction will be invalid when fMag = 0

		if (fMag < fRange)
		{
			float fBrightness = 1.0f - fMag / fRange;

			float fR = fBrightness * pxPhoton->GetRedComponent();
			float fG = fBrightness * pxPhoton->GetGreenComponent();
			float fB = fBrightness * pxPhoton->GetBlueComponent();
				
			xResult.AddSampleForDirection( fR, fG, fB, xDir );		
		}
    }

	xResult.ScaleCoefficientsForIrradiance();

#endif

}

void Asura_GI_Solution_Handler::CalculateDiffuseAtPoint( Asura_RGBA& xDiffuse,
														 const Asura_Vector_3& xPosition,
														 const Asura_Vector_3& xNormal,
														 bool bIncludeDynamic )
{
    Asura_SphericalHarmonics_Order2_RGB xTmpSH;
    if( bIncludeDynamic )
    {
        GetTotalIrradiance( xTmpSH, xPosition );
    }
    else
    {
        GetStaticIrradiance( xTmpSH, xPosition );
    }

    xDiffuse.m_fR = xTmpSH.m_afRedCoefficients[1] * xNormal.x
                  + xTmpSH.m_afRedCoefficients[2] * xNormal.y
                  + xTmpSH.m_afRedCoefficients[3] * xNormal.z
                  + xTmpSH.m_afRedCoefficients[0];

    xDiffuse.m_fG = xTmpSH.m_afGreenCoefficients[1] * xNormal.x
                  + xTmpSH.m_afGreenCoefficients[2] * xNormal.y
                  + xTmpSH.m_afGreenCoefficients[3] * xNormal.z
                  + xTmpSH.m_afGreenCoefficients[0];

    xDiffuse.m_fB = xTmpSH.m_afBlueCoefficients[1] * xNormal.x
                  + xTmpSH.m_afBlueCoefficients[2] * xNormal.y
                  + xTmpSH.m_afBlueCoefficients[3] * xNormal.z
                  + xTmpSH.m_afBlueCoefficients[0];
	// (AW - 27.08.2004) - Added clamp because I was sometimes getting negative values.
	xDiffuse.Clamp();
}

void Asura_GI_Solution_Handler::CalculateDiffuseAtPoint( Asura_RGBA& xDiffuse, const Asura_Vector_3& xPosition )
{
	Asura_SphericalHarmonics_Order2_RGB xTmpSH;
	GetTotalIrradiance( xTmpSH, xPosition );

	xDiffuse.m_fR = Asura_Maths::Abs( xTmpSH.m_afRedCoefficients[0]   )
		          + Asura_Maths::Abs( xTmpSH.m_afRedCoefficients[1]   )
				  + Asura_Maths::Abs( xTmpSH.m_afRedCoefficients[2]   )
				  + Asura_Maths::Abs( xTmpSH.m_afRedCoefficients[3]   );
	xDiffuse.m_fG = Asura_Maths::Abs( xTmpSH.m_afGreenCoefficients[0] )
		          + Asura_Maths::Abs( xTmpSH.m_afGreenCoefficients[1] )
				  + Asura_Maths::Abs( xTmpSH.m_afGreenCoefficients[2] )
				  + Asura_Maths::Abs( xTmpSH.m_afGreenCoefficients[3] );
	xDiffuse.m_fB = Asura_Maths::Abs( xTmpSH.m_afBlueCoefficients[0]  )
		          + Asura_Maths::Abs( xTmpSH.m_afBlueCoefficients[1]  )
				  + Asura_Maths::Abs( xTmpSH.m_afBlueCoefficients[2]  )
				  + Asura_Maths::Abs( xTmpSH.m_afBlueCoefficients[3]  );

	xDiffuse.Clamp();
}

void Asura_GI_Solution_Handler::GetTotalIrradiance( Asura_SphericalHarmonics_Order2_RGB& xResult, const Asura_Vector_3& xPosition )
{
	Asura_SphericalHarmonics_Order2_RGB xDynamic;
	
	GetDynamicIrradiance( xDynamic, xPosition );

	GetStaticIrradiance( xResult, xPosition );

	xResult += xDynamic;
}


DEBUGVAR float dbg_fEntityLightScale = 0.5f;

void Asura_GI_Solution_Handler::GetStaticIrradianceFromOldLighting( Asura_SphericalHarmonics_Order2_RGB& xResult, const Asura_Vector_3& xPosition )
{
	xResult.Zero();

	Asura_Bounding_Box xBoundingBox;

	xResult.m_afRedCoefficients[0] = Asura_Photon_System::GetEntityAmbientRed();
	xResult.m_afGreenCoefficients[0] = Asura_Photon_System::GetEntityAmbientGreen();
	xResult.m_afBlueCoefficients[0] = Asura_Photon_System::GetEntityAmbientBlue();

	xBoundingBox.Reset();
	xBoundingBox.Set(xPosition);

	Asura_Photon_System::MakePhotonListForBoundingBox(xBoundingBox, xPosition, true, false, 3);

	for (u_int u=0; u<Asura_Photon_System::NumberOfLocalPhotons; u++)
	{
		Asura_Local_Photon* pxPhoton = &Asura_Photon_System::LocalPhotons[u];

		float fRange = pxPhoton->GetRange();

		if (fRange <= 0.0f) continue;

		Asura_Vector_3 xDir = pxPhoton->GetPosition() - xPosition;

		float fMag = xDir.Normalise();

		// TODO - check bounding box?
		// TODO - direction will be invalid when fMag = 0

		if (fMag < fRange)
		{
			float fBrightness = (1.0f - fMag / fRange) * dbg_fEntityLightScale;

			float fR = fBrightness * pxPhoton->GetRedComponent();
			float fG = fBrightness * pxPhoton->GetGreenComponent();
			float fB = fBrightness * pxPhoton->GetBlueComponent();
				
			xResult.AddSampleForDirection( fR, fG, fB, xDir );
		}
	}
}

#ifdef ASURA_TOOLS
void Asura_GI_Solution_Handler::Tools_InitialiseFromPhotonMappingSystem( void )
{
	// throw away any previous lighting
	Reset();

	// reset tools stuff and let it calculate the positions/kd-tree
	Tools_GlobalIllumination_EntitySamples::Reset();
	Tools_GlobalIllumination_EntitySamples::ConstructSamples();
	//make sure env irradiance is up to date , since the lighting calculations will need it
	Tools_PhotonMapping_System::CalculateEnvironmentIrradiance();

	m_uNumberOfSamples = Tools_GlobalIllumination_EntitySamples::GetNumberOfSamples();
	Asura_ResourcePushType(ASURA_RESOURCE_GLOBAL_ILLUMATION);
	{
		Asura_ResourceAllocateArray(m_pxSamples, m_uNumberOfSamples);
	}
	Asura_ResourcePopType();

#ifndef ASURA_TARGET_SM3
	// We want to search through all the harmonics to see what the min & max values
	// of each coefficient is, so we can better compress them into 8bit values
	float afMinValues[Asura_SphericalHarmonics_Order2_RGB::NUMBER_OF_COEFFICIENTS];
	float afMaxValues[Asura_SphericalHarmonics_Order2_RGB::NUMBER_OF_COEFFICIENTS];

	// set the min & max values to extreme values 
	for (u_int u=0; u<Asura_SphericalHarmonics_Order2_RGB::NUMBER_OF_COEFFICIENTS; u++)
	{
		afMaxValues[u] = -Asura_Maths::LargeFloat;
		afMinValues[u] =  Asura_Maths::LargeFloat;

	}

	char szTitle[100];
	Asura_String xLightSetNames;
	Tools_LightSets::GetCurrentLightSetNames(xLightSetNames);
	Asura_PrintfToBfr(szTitle,"GI Samples : %s",100,true,xLightSetNames.GetString());

	Tools_PhotonMapping_System::UpdateProgress(0, m_uNumberOfSamples, szTitle);
	// scan through all samples...
	for (u_int u=1; u<m_uNumberOfSamples; u++)
	{
		const Tools_GlobalIllumination_EntitySample& xSample = Tools_GlobalIllumination_EntitySamples::GetSample(u);
		if(Tools_PhotonMapping_System::UpdateProgress(u, m_uNumberOfSamples, szTitle))
		{
			Reset();
			return;
		}

		for (u_int v=0; v<Asura_SphericalHarmonics_Order2_RGB::NUMBER_OF_COEFFICIENTS; v++)
		{
			// is this a new maximum?
			if (afMaxValues[v] < xSample.m_xSH[v])
			{
				afMaxValues[v] = xSample.m_xSH[v];
			}

			// is this a new minimum?
			if (afMinValues[v] > xSample.m_xSH[v])
			{
				afMinValues[v] = xSample.m_xSH[v];
			}
		}
	}

	// now find the ranges (i.e. spread of values from minima) of these coefficents
	float afRangeScale[Asura_SphericalHarmonics_Order2_RGB::NUMBER_OF_COEFFICIENTS];

	for (u_int v=0; v<Asura_SphericalHarmonics_Order2_RGB::NUMBER_OF_COEFFICIENTS; v++)
	{
		m_afDecompressCoefficientMinimum[v] = afMinValues[v];

		float fRange = afMaxValues[v] - afMinValues[v];

		if (fRange < 0.00001f)
		{
			afRangeScale[v] = 0.0f;

			//m_afDecompressCoefficientMinimum[v] = 0.0f;
		}
		else
		{
			afRangeScale[v] = 255.0f / fRange;

			m_afDecompressCoefficientScale[v] = fRange / 255.0f;
		}
	}
#endif

	// now make compressed samples for in-game use
	for (u_int u=1; u<m_uNumberOfSamples; u++)
	{
		const Tools_GlobalIllumination_EntitySample& xSample = Tools_GlobalIllumination_EntitySamples::GetSample(u);

		// position currently uncompressed for simplicity and speed
		m_pxSamples[u].m_afPosition[0] = xSample.m_xPosition.x;
		m_pxSamples[u].m_afPosition[1] = xSample.m_xPosition.y;
		m_pxSamples[u].m_afPosition[2] = xSample.m_xPosition.z;

		// however, I'm going to steal some accuracy from the y component, and store 2 bits indicating
		// the plane split in the least significant bits. (Using a union, btw).

		// mask out two bits (1 AND 2) and OR in the plane axis value
		m_pxSamples[u].m_uPlaneSplitFlag = (m_pxSamples[u].m_uPlaneSplitFlag & (~3)) | xSample.GetPlaneAxis();


		// Compress RGB irradiance SHs to 2nd order with byte-sized coefficients.
		// Effectively compress irradiance to a 12 dimensional sample,
		// with sub-percentage accuracy relative to full range in each dimension.
		// (AAM - 09.02.2009) No longer compressing to bytes in next-gen
		for (u_int v=0; v<Asura_SphericalHarmonics_Order2_RGB::NUMBER_OF_COEFFICIENTS; v++)
		{
			#ifdef ASURA_TARGET_SM3
				m_pxSamples[u].m_afCoefficients[v] = xSample.m_xSH[v];
			#else
				float fCoefficient = xSample.m_xSH[v];

				// transform range from 0 to (max - min)
				fCoefficient -= afMinValues[v];

				// transform range from 0 to 255
				fCoefficient *= afRangeScale[v];

				// check this maths has come out correctly - paranoia about floating point inaccuracies; a small error
				// will be rounded out when we move to integers.
				Asura_Assertf( fCoefficient >= -0.1f && fCoefficient <= 255.1f, ("Coefficient out of range: %f", fCoefficient) );

				u_int uCoefficient = Asura_Maths::FloatToInt_Near(fCoefficient);

				Asura_Assertf( uCoefficient <= 255, ("Coefficient out of range: %d", uCoefficient) );

				m_pxSamples[u].m_aucCompressedCoefficients[v] = static_cast<u_char>(uCoefficient);
			#endif
		}
	}

	//Mark as up to date/valid
	m_bSamplesValid = true;

	Tools_SaveAsura::SetGlobalIlluminationModified(true);
}

void Asura_GI_Solution_Handler::Tools_GenerateFakeSample()
{
	//Generates some default samples for projects that don't care about the lighting being valid
	//RAR 17/05/2007
	
	// throw away any previous lighting
	Reset();

	//Need more than one sample for the samples to be found correctly
	//4 seems to be sufficient
	m_uNumberOfSamples = 4;
	Asura_ResourcePushType(ASURA_RESOURCE_GLOBAL_ILLUMATION);
	{
		Asura_ResourceAllocateArray(m_pxSamples, m_uNumberOfSamples);
	}
	Asura_ResourcePopType();

	#ifndef ASURA_TARGET_SM3

		//set compression values for a range of 0 to 1 in each coeffcient
		for (u_int v=0; v<Asura_SphericalHarmonics_Order2_RGB::NUMBER_OF_COEFFICIENTS; v++)
		{
			m_afDecompressCoefficientMinimum[v] = 0.0f;
			m_afDecompressCoefficientScale[v] = 1.0f / 255.0f;
		}

	#endif
	
	// now make compressed samples for in-game use
	for (u_int u=1; u<m_uNumberOfSamples; u++)
	{
 		// position currently uncompressed for simplicity and speed
		m_pxSamples[u].m_afPosition[0] = 0.0f;
		m_pxSamples[u].m_afPosition[1] = 0.0f;
		m_pxSamples[u].m_afPosition[2] = 0.0f;

		#ifdef ASURA_TARGET_SM3

			for (u_int v=0; v<Asura_SphericalHarmonics_Order2_RGB::NUMBER_OF_COEFFICIENTS; v++)
			{
				m_pxSamples[u].m_afCoefficients[v] = 0.0f;
			}

			Asura_RGBA xAmbience = Tools_PhotonMapping_System::GetGlobalAmbience();
			m_pxSamples[u].m_afCoefficients[0] = xAmbience.m_fR;
			m_pxSamples[u].m_afCoefficients[4] = xAmbience.m_fG;
			m_pxSamples[u].m_afCoefficients[8] = xAmbience.m_fB;

		#else

			//Set compressed coefficients to match the global ambience
			for (u_int v=0; v<Asura_SphericalHarmonics_Order2_RGB::NUMBER_OF_COEFFICIENTS; v++)
			{
				m_pxSamples[u].m_aucCompressedCoefficients[v] = 0;
			}
			Asura_RGBA xAmbience = Tools_PhotonMapping_System::GetGlobalAmbience();
			m_pxSamples[u].m_aucCompressedCoefficients[0] = (u_char)Asura_Maths::ClampToRange(Asura_Maths::FloatToInt_Near(xAmbience.m_fR*255.0f),0,255);
			m_pxSamples[u].m_aucCompressedCoefficients[4] = (u_char)Asura_Maths::ClampToRange(Asura_Maths::FloatToInt_Near(xAmbience.m_fG*255.0f),0,255);
			m_pxSamples[u].m_aucCompressedCoefficients[8] = (u_char)Asura_Maths::ClampToRange(Asura_Maths::FloatToInt_Near(xAmbience.m_fB*255.0f),0,255);

		#endif
	}

	//Mark as up to date/valid
	m_bSamplesValid = true;
}

void Asura_GI_Solution_Handler::ApplyGlobalAmbienceDelta(const Asura_RGBA& xColourDiff)
{
	#ifdef ASURA_TARGET_SM3

		// Minimum not stored since we're not compressing any more. Affect each sample
		for( u_int u = 0; u < m_uNumberOfSamples; ++u )
		{
			m_pxSamples[u].m_afCoefficients[0] += xColourDiff.m_fR;
			m_pxSamples[u].m_afCoefficients[4] += xColourDiff.m_fG;
			m_pxSamples[u].m_afCoefficients[8] += xColourDiff.m_fB;
		}

	#else

		m_afDecompressCoefficientMinimum[0] += xColourDiff.m_fR;
		m_afDecompressCoefficientMinimum[4] += xColourDiff.m_fG;
		m_afDecompressCoefficientMinimum[8] += xColourDiff.m_fB;

	#endif
}

#endif //#ifdef ASURA_TOOLS
