//
// Asura_Lighting_Solution.h
//
// Created by Andrew Weinkove - 24/08/04
// 
// Copyright (C) 2004 Rebellion, All Rights Reserved.
//
// Moved class (by Rich Rice) from Asura_Photon.h and updated to use SH.
//

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef __ASURA_LIGHTING_SOLUTION_H__
#define __ASURA_LIGHTING_SOLUTION_H__

#include "Core/Asura_Maths.h"

class Asura_Lighting_Solution
{
public :
	Asura_Lighting_Solution(bool bKeepStatic,bool bKeepDynamic);
	~Asura_Lighting_Solution();
	void Reset();
	void ResetStatic(){m_bStaticValid = false;}
	void ResetDynamic(){m_bDynamicValid = false;}

	bool IsUpToDate(bool bWantStatic,bool bWantDynamic) const
	{
		if(bWantStatic && !m_bStaticValid) return false; 
		if(bWantDynamic && !m_bDynamicValid) return false; 
		#ifdef ASURA_TOOLS
		if(bWantStatic && HasToolsLightingChanged()) return false;
		#endif
		return true;
	}
		bool IsStaticValid() const 
	{
		return m_bStaticValid;
	}

	bool IsDynamicValid() const 
	{
		return m_bDynamicValid;
	}


	void UseStoredLighting() const;

	//Sets the position to do lighting from. If it is only a small change , then
	//the position is ignored in favour of keeping the old lighting valid.
	void SetPosition(const Asura_Vector_3& xPosition);
	
	//calculates/updates the local photons in Asura_Photon_System ,and keeps the results that
	//were specified in ths constructor for keeping
	void RecalculateLighting( 	const bool bIncludeStaticLight,
								const bool bIncludeDynamicLight);

	#ifdef ASURA_TOOLS
	bool HasToolsLightingChanged() const;
	void InvalidateIfToolsLightingChanged();
	#endif


private :	
	// Perhaps it would be worth using the compressed version here? Speed-Memory trade off.
	Asura_SphericalHarmonics_Order2_RGB* m_pxStoredStaticIrradiance;
	Asura_SphericalHarmonics_Order2_RGB* m_pxStoredDynamicIrradiance;
	//position at which static lighting was last calculated
	Asura_Vector_3 m_xLightingPosition;

	u_int m_bKeepStatic:1;
	u_int m_bKeepDynamic:1;
	u_int m_bStaticValid:1;
	u_int m_bDynamicValid:1;

	#ifdef ASURA_TOOLS
	u_int m_uGISamplesFrameStamp;//frame stamp of tools GI samples
	#endif
	
};

#endif//__ASURA_LIGHTING_SOLUTION_H__
