//
// Asura_Mesh_CollMeshGenData.cpp
//
// Created by Richard Rice - 18/03/2004
// 
// Copyright (C) 2004 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

#ifdef ASURA_TOOLS

#include "Mesh/Asura_Mesh_CollMeshGenData.h"

#include "Debugging/Asura_Assert.h"
#include "Mesh/Asura_Mesh.h"


Asura_Mesh_CollMeshGenData::Asura_Mesh_CollMeshGenData()
{
	m_iNumberOfPolygons = 0;
	m_pxPolygons = 0;

	m_uNumberOfBoxes = 0;
	m_pxBoxes = 0;
}

Asura_Mesh_CollMeshGenData::~Asura_Mesh_CollMeshGenData()
{
	delete [] m_pxPolygons;
	delete [] m_pxBoxes;
}

void Asura_Mesh_CollMeshGenData::InitialisePolygonArray(const Asura_Mesh* pxMesh)
{
	if(m_pxPolygons) return;

	m_iNumberOfPolygons = pxMesh->NumberOfPolygons;
	m_pxPolygons = new Asura_Mesh_CollPolyInfo[m_iNumberOfPolygons];

	for(int i=0;i<m_iNumberOfPolygons;i++)
	{
		m_pxPolygons[i].m_usFlags = 0;
		m_pxPolygons[i].m_usGroupID = 0;
	}
}

Asura_Mesh_CollBoxInfo* Asura_Mesh_CollMeshGenData::CreateBox()
{
	//find a free group ID
	u_int uGroupID = 1;
	for(u_int i=0;i<m_uNumberOfBoxes;i++)
	{
		if(m_pxBoxes[i].m_uGroupID >= uGroupID)
		{
			uGroupID = m_pxBoxes[i].m_uGroupID+1;
		}
	}

	//allocate an enlarged array of boxes , and copy the old values
	Asura_Mesh_CollBoxInfo* pxBoxes = new Asura_Mesh_CollBoxInfo[m_uNumberOfBoxes+1];
	for(u_int i=0;i<m_uNumberOfBoxes;i++)
	{
		pxBoxes[i] = m_pxBoxes[i];
	}

	//replace the old array
	delete [] m_pxBoxes;
	m_pxBoxes = pxBoxes;

	//initialise the new box
	pxBoxes[m_uNumberOfBoxes].m_uGroupID = uGroupID;
	pxBoxes[m_uNumberOfBoxes].m_xOrient.Identity();

	return &pxBoxes[m_uNumberOfBoxes++];
}

void Asura_Mesh_CollMeshGenData::RemoveUnusedGroups()
{
	if(!m_pxPolygons) return;

	for(u_int i=0;i<m_uNumberOfBoxes;i++)
	{
		u_short usGroupID = (u_short)m_pxBoxes[i].m_uGroupID;
		
		bool bUsed = false;
		//see if any polygon uses this
		for(int j=0;j<m_iNumberOfPolygons;j++)
		{
			if(m_pxPolygons[j].m_usGroupID == usGroupID)
			{
				bUsed = true;
				break;
			}
		}
		if(!bUsed)
		{
			//this box can be removed , copy the last one over it
			m_pxBoxes[i--] = m_pxBoxes[--m_uNumberOfBoxes];
		}
	}
}

void Asura_Mesh_CollMeshGenData::HandlePolygonConversion(const Asura_Mesh* pxMesh,const int* piPolyConv1,const int* piPolyConv2)
{
	if(!m_pxPolygons) return;

	//allocate and initialise a new polygon data array
	Asura_Mesh_CollPolyInfo* pxPolygons = new Asura_Mesh_CollPolyInfo[pxMesh->NumberOfPolygons];
	for(int i=0;i<pxMesh->NumberOfPolygons;i++)
	{
		pxPolygons[i].m_usFlags = 0;
		pxPolygons[i].m_usGroupID = 0;
	}

	//copy settings from the old array
	for(int i=0;i<m_iNumberOfPolygons;i++)
	{
		if(piPolyConv1 && piPolyConv1[i] >= 0)
		{
			pxPolygons[piPolyConv1[i]] = m_pxPolygons[i];
		}
		if(piPolyConv2 && piPolyConv2[i] >= 0)
		{
			pxPolygons[piPolyConv2[i]] = m_pxPolygons[i];
		}
	}

	//replace the polygon array ,and update the number of polygons
	delete [] m_pxPolygons;
	m_pxPolygons = pxPolygons;
	m_iNumberOfPolygons = pxMesh->NumberOfPolygons;
	
	RemoveUnusedGroups();	

}

#endif // #ifdef ASURA_TOOLS
