//////////////////////////////////////////////////////////////////////////////
//
// Asura_InternetSimulation.h
//
// Copyright (c) 2004 Rebellion. All rights reserved.
//
// Description:
//
//     functionality for introducing latency and packet loss to network
//     traffic
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef ___ASURA_INTERNETSIMULATION_H___
#define ___ASURA_INTERNETSIMULATION_H___ 1

#ifdef ASURA_ENABLE_INTERNET_SIMULATION

#include "Network/Asura_NetSocket.h"	// for Asura_Netaddress

typedef bool(*Asura_InternetSimulation_Callback)( void* pInstance, const char* pcMessage, int iLength, Asura_NetAddress xAddress, unsigned short usPort );

class Asura_NetSim_Settings
{
public:
	u_int m_uLatencyMean; 
	u_int m_uLatencyStddev; 
	float m_fDropPercent;
};

// NB currently latency and loss is only enabled on received packets. This means that all 
// machines should have the simulation on to get the correct results.
// An improvement would be to store latency mean and stddev for each machine (on the server)
// and tell each client what values to use. But this will work OK for now.
class Asura_InternetSimulation
{

public:

	static void Initialise();
	static void Shutdown();

	static void SetEnabled(bool bEnabled) { s_bEnabled = bEnabled; }
	static bool IsEnabled() { return s_bEnabled; }
	
	static void SetLatencyMean( u_int uLatencyMean );
	static void SetLatencyStandardDeviation( u_int uLatencyStddev );
	static void SetDropPercent( float fDropPercent );
	static u_int GetLatencyMean() { return s_xCurrentSettings.m_uLatencyMean; }
	static u_int GetLatencyStandardDeviation() { return s_xCurrentSettings.m_uLatencyStddev; }
	static float GetDropPercent() { return s_xCurrentSettings.m_fDropPercent; }

	static void SetSettings( const Asura_NetSim_Settings& xSettings );
	static const Asura_NetSim_Settings& GetSettings() { return s_xCurrentSettings; }

	// before sending received data through whatever path it needs to go, check if AddToBuffer()
	// returns true. If it does the data will be stored. The function to send it down the right 
	// path when it's time to deliver the data should be passed to the Update() function once
	// per frame. The callback will have the supplied pInstance parameter passed in.
	static void Update( void* pInstance, Asura_InternetSimulation_Callback pfCallback );
	static bool AddToBuffer( const char* pcMessage, int iLength, Asura_NetAddress xAddress, unsigned short usPort );

private:

	// internal use functions
	static u_int GetAddedLatency();
	static bool ShouldDrop();

	// static data
	static bool s_bEnabled;
	static Asura_NetSim_Settings s_xCurrentSettings;
};

#endif // #ifdef ASURA_ENABLE_INTERNET_SIMULATION

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

#endif // #ifndef ___ASURA_INTERNETSIMULATION_H___
