//////////////////////////////////////////////////////////////////////////////
//
// Asura_LobbyChat.h
//
// Copyright (c) 2004 Rebellion. All rights reserved.
//
// Description:
//
//     helper class for chat SDKs which buffers up messages and provides
//     functionality for the frontend to get the messages for rendering of
//     chat windows
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef ___ASURA_LOBBYCHAT_H___
#define ___ASURA_LOBBYCHAT_H___ 1


#include "Core/Asura_PlatformBasics.h"

static const u_int uASURA_CHAT_MAX_USERNAME_LEN = 51;
static const u_int uASURA_CHAT_MAX_MESSAGE_SIZE = 256;	// including NULL

#ifdef ASURA_SUPPORT_MULTIPLAYER_COMMS

class Asura_LobbyChat
{

	public:

		//Initialise takes an optional callback function, which get's called each time a new user is added
		//it allows a project to override mute status, set user IDs, and block the user from being added.
		static void Initialise(bool (*pfnAddUser)(const Asura_UnicodeChar*, bool*, int*) = 0);
		static void Shutdown( void );
		static void Reset( void );

		static bool AddUser(const Asura_UnicodeChar* wszUser);
		static bool RemoveUser(const Asura_UnicodeChar* wszUser);
		static void AddMessage(const Asura_UnicodeChar* wszUser, const Asura_UnicodeChar* wszMessage, bool bPrivate = false);

		static bool AddUser(const char* szUser);
		static bool RemoveUser(const char* szUser);
		static void AddMessage(const char* szUser, const char* szMessage, bool bPrivate = false);

		// this class stores the most recent 32 messages. use GetNumMessages to 
		// find how many are stored and use GetMessage() to retrieve message 0
		// up to GetNumMessages() - 1. Message 0 is the most recent.
		static u_int GetNumMessages( void );
		static const Asura_UnicodeChar* GetMessage(u_int uMessageNumber);

		static u_int GetNumUsers( void );
		static const Asura_UnicodeChar* GetUserName( u_int uUserNumber );//returns original username
		static void MuteUser( u_int uUserNumber );
		static void UnmuteUser( u_int uUserNumber );
		static const bool IsUserMuted( u_int uUserNumber );
		//each user has an int stored aginst it to allow projects to store user IDs, eg gamespy profiel IDs
		static const int GetUserID( u_int uUserNumber );
		static void SetUserID( u_int uUserNumber, int iChatUserID );


	private:
		
		static bool s_bInitialised;
		static u_int s_uNextHistoryEntry;
		static bool s_bHistoryHasWrapped;
		static Asura_UnicodeChar** s_awszChatHistory;
		static u_int s_uNumUsers;
		static Asura_UnicodeChar** s_awszChatUserNames;
		static bool* s_abChatUserMuted;
		static int* s_aiChatUserID;
		static bool (*s_pfnAddUser)(const Asura_UnicodeChar*, bool*, int*);

};

#endif //ASURA_SUPPORT_MULTIPLAYER_COMMS

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

#endif // #ifndef ___ASURA_LOBBYCHAT_H___
