//////////////////////////////////////////////////////////////////////////////
//
// Asura_Message.cpp
//
// Copyright (c) 2009 Rebellion. All rights reserved.
//
// Description:
//
//     code for generic Asura messages for those which aren't dealt with in
//     other files
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header.
#include "Network/Asura_Message.h"

// Asura independent
#include "Animation/Asura_TextureAnimation.h"
#include "Entity/Asura_Entity_Classification.h"
#include "Entity/Cutscene/Asura_ClientEntity_CutsceneController.h"
#include "Entity/PhysicalObject/Asura_ServerEntity_PhysicalObject.h"
#include "Entity/Triggers/Asura_ServerEntity_LookAtTrigger.h"
#ifdef ASURA_USE_FSFX
#include "FSFX/Asura_FSFX_System.h"
#endif
#include "Network/Asura_NetMsg.h"
#include "Teleport/Asura_Teleport_Messages.h"

#ifdef ASURA_NETMSG_PACK_ALL_DATA
const Asura_NetMsg* Asura_Message_Registration::RegisterNetMsg(u_short usMsgID, u_short usEntityClassification)
{
#define ASURA_REGISTER_NETMSG_CASE(x,y) case x: return Asura_NetMsg_RegistrationHelper<y>::RegisterFields(usMsgID, usEntityClassification)
#define ASURA_REGISTER_NETMSG_CASE_ENTITY_SPECIFIC(x,y,z) case x: { if (z == usEntityClassification) \
	return Asura_NetMsg_RegistrationHelper<y>::RegisterFields(usMsgID, usEntityClassification); } break

	switch (usMsgID)
	{
		ASURA_REGISTER_NETMSG_CASE(ASURA_MSG_VALIDCLIENTGUID, Asura_MsgData_ValidClientGuid);
		ASURA_REGISTER_NETMSG_CASE(ASURA_MSG_PLAYERCONNECTED, Asura_MsgData_PlayerConnected);
		ASURA_REGISTER_NETMSG_CASE(ASURA_MSG_PLAYERDISCONNECTED, Asura_MsgData_PlayerDisconnected);
#ifdef ASURA_USE_FSFX
		ASURA_REGISTER_NETMSG_CASE(ASURA_MSG_SET_FSFX_NEW, Asura_FSFX_Message);
#endif
		ASURA_REGISTER_NETMSG_CASE(ASURA_MSG_ENVTEXTUREANIMATION, Asura_Message_EnvironmentTextureAnimation);
		ASURA_REGISTER_NETMSG_CASE(ASURA_MSGID_TELEPORT, Asura_Message_Teleport);
		ASURA_REGISTER_NETMSG_CASE(ASURA_MSG_CUTSCENECONTROLLER_ACTIVATE, Asura_Msg_ActivateCutscene);
		ASURA_REGISTER_NETMSG_CASE(ASURA_MSG_LINK_TO_PARENT, Asura_Message_PhysicalObject_LinkToParent);

		case ASURA_MSG_CLIENTCREATE:
		{
			switch (usEntityClassification)
			{
			case AsuraEntityClass_LookAtTrigger:
				{
					return Asura_NetMsg_RegistrationHelper<Asura_Msg_LookAtTrigger_ClientCreate>::RegisterFields(usMsgID, usEntityClassification);
				}
				break;
			}
			break;
		}
	}

#undef ASURA_REGISTER_NETMSG_CASE
#undef ASURA_REGISTER_NETMSG_CASE_ENTITY_SPECIFIC

	return NULL;
}

void Asura_MsgData_ValidClientGuid::RegisterFields(Asura_NetMsg* pxNetMsg) const
{
	Asura_Assert(pxNetMsg->GetMsgID() == ASURA_MSG_VALIDCLIENTGUID, "unexpected message ID");
	Asura_Assert(pxNetMsg->GetEntityClassification() == AsuraEntityClass_NotSet, "unexpected entity classification");

	Asura_NetMsg_RegisterField(pxNetMsg, *this, Asura_FieldType_Guid, Guid, 1, Asura_Threshold_AnyChange, 0.0f);
}

void Asura_MsgData_PlayerConnected::RegisterFields(Asura_NetMsg* pxNetMsg) const
{
	// can't assert as this can be used as a base class
	//Asura_Assert(pxNetMsg->GetMsgID() == ASURA_MSG_PLAYERCONNECTED, "unexpected message ID");
	Asura_Assert(pxNetMsg->GetEntityClassification() == AsuraEntityClass_NotSet, "unexpected entity classification");

	Asura_NetMsg_RegisterField(pxNetMsg, *this, Asura_FieldType_Guid, PlayerGuid, 1, Asura_Threshold_AnyChange, 0.0f);
}

void Asura_MsgData_PlayerDisconnected::RegisterFields(Asura_NetMsg* pxNetMsg) const
{
	Asura_Assert(pxNetMsg->GetMsgID() == ASURA_MSG_PLAYERDISCONNECTED, "unexpected message ID");
	Asura_Assert(pxNetMsg->GetEntityClassification() == AsuraEntityClass_NotSet, "unexpected entity classification");

	Asura_NetMsg_RegisterField(pxNetMsg, *this, Asura_FieldType_Guid, PlayerGuid, 1, Asura_Threshold_AnyChange, 0.0f);
	Asura_NetMsg_RegisterIntFieldBits(pxNetMsg, *this, Asura_FieldType_uint_bits, 7, m_eReason, 1, Asura_Threshold_AnyChange);
}
#endif//#ifdef ASURA_NETMSG_PACK_ALL_DATA
