//
// Asura_NetTransport_System.h
//
// Created by KOR - 29/10/2003 17:35:42
//
// Copyright (C) 2003 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif

#ifndef __ASURA_NETTRANSPORT_SYSTEM_H__
#define __ASURA_NETTRANSPORT_SYSTEM_H__

#ifdef ASURA_SUPPORT_EXTERNAL_NETWORK

#include "Core/Asura_PlatformBasics.h"
#include "Network/Asura_Network_Session.h"
#include "Network/Asura_Message.h"
#include "Network/Asura_Platform_NetSocket.h"
#include "Network/Online/Asura_Lobby_Types.h"

// middleware includes
#ifdef ASURA_SUPPORT_NOVAWORLD
#include "NovaWorld/NovaworldAPI.h"
#endif

#if defined(ASURA_SUPPORT_GAMESPY)
#include "GameSpy/natneg/natneg.h"
#endif

// MACROS /////////////////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_NTS_NETWORK_BYTE_ORDER_IS_BIG_ENDIAN
// If the project has defined a network byte order, check whether it matches the platform byte order.
#ifdef ASURA_PLATFORM_IS_BIG_ENDIAN
	#if ASURA_NTS_NETWORK_BYTE_ORDER_IS_BIG_ENDIAN == (1)
		#define ASURA_NTS_NETWORK_BYTE_ORDER_MATCHES_PLATFORM_BYTE_ORDER (1)
	#endif
#else//!def ASURA_PLATFORM_IS_BIG_ENDIAN
	#if ASURA_NTS_NETWORK_BYTE_ORDER_IS_BIG_ENDIAN == (0)
		#define ASURA_NTS_NETWORK_BYTE_ORDER_MATCHES_PLATFORM_BYTE_ORDER (1)
	#endif
#endif

#else
	// Default to no swapping.
	#define ASURA_NTS_NETWORK_BYTE_ORDER_MATCHES_PLATFORM_BYTE_ORDER (1)
#endif

// ASURA_NTS_CHANGE_BYTE_ORDER swaps the bytes in __xData if necessary
// ASURA_NTS_CHANGE_BYTE_ORDER_RETVAL returns a copy of __xData, byte-swapped if necessary
#ifdef ASURA_NTS_NETWORK_BYTE_ORDER_MATCHES_PLATFORM_BYTE_ORDER
	// Nothing to do
	#define ASURA_NTS_CHANGE_BYTE_ORDER(__xData)
	#define ASURA_NTS_CHANGE_BYTE_ORDER_RETVAL(__xData) (__xData)
#else
	#include "Core/Asura_Endian.h"
	#define ASURA_NTS_CHANGE_BYTE_ORDER(__xData) Asura_Endian::SwapBytes(reinterpret_cast<char*>(&__xData), sizeof(__xData))
	#define ASURA_NTS_CHANGE_BYTE_ORDER_RETVAL(__xData) Asura_Endian::SwapBytes(__xData)
#endif

// CONSTANTS //////////////////////////////////////////////////////////////////////////////////////

// Time in milliseconds between guaranteed message re-sends
const u_int ASURA_NETTRANSPORT_RESEND_PERIOD = 500;
// Maximum messages to hold in buffer.
// When the maximum is reached, [the oldest messages will be removed.]
// (MH - 20.07.2004) I've changed this do it disconnects rather than not delivering. 
const u_int ASURA_NETWORK_BUFFER_MAX_MESSAGES = 128;

// constants to help compute average ping
const u_int ASURA_NETTRANSPORT_NUM_PING_SAMPLES_SHIFT = 2;
const u_int ASURA_NETTRANSPORT_NUM_PING_SAMPLES = (1 << ASURA_NETTRANSPORT_NUM_PING_SAMPLES_SHIFT);
const u_int ASURA_NETTRANSPORT_NUM_PING_SAMPLES_MASK = ASURA_NETTRANSPORT_NUM_PING_SAMPLES - 1;

// Buffer size limit
// (MH - 20.07.2004) we can store up to 128 message, averaga size is probably about 20 bytes
// as guaranteed messages are generally small. To be safe assume up to 128 bytes per message.
//const u_int ASURA_NETTRANSPORT_BUFFER_SIZE_LIMIT = 128 * 1024;
#ifndef ASURA_NETTRANSPORT_BUFFER_SIZE_LIMIT
const u_int ASURA_NETTRANSPORT_BUFFER_SIZE_LIMIT = 128 * 128; // 16kB
#endif

// Message IDs for system; sent as unsigned shorts
enum ASURA_NETTRANSPORT_MSG_ID
{
	ASURA_NETTRANSPORT_MSG_ENUMHOSTS = ASURA_MSG_TRANSPORT_LAYER,	// 0xf000
	ASURA_NETTRANSPORT_MSG_HOSTREPLY,								// 0xf001
	ASURA_NETTRANSPORT_MSG_CONNECTION_REQUEST,						// 0xf002
	ASURA_NETTRANSPORT_MSG_CONNECTION_ACCEPT,						// 0xf003
	ASURA_NETTRANSPORT_MSG_CONNECTION_REJECT,						// 0xf004
	ASURA_NETTRANSPORT_MSG_CONNECTION_CLOSE,						// 0xf005
	ASURA_NETTRANSPORT_MSG_KEEPALIVE,								// 0xf006
	ASURA_NETTRANSPORT_MSG_PING,									// 0xf007
	ASURA_NETTRANSPORT_MSG_PONG,									// 0xf008
	ASURA_NETTRANSPORT_MSG_ACK,										// 0xf009
	ASURA_NETTRANSPORT_MSG_CHALLENGE_REQUEST,						// 0xf00a
	ASURA_NETTRANSPORT_MSG_CHALLENGE_RESPONSE,						// 0xf00b
	ASURA_NETTRANSPORT_MSG_NETSIM_SETTINGS,							// 0xf00c
	ASURA_NETTRANSPORT_MSG_VOICE_DATA,								// 0xf00d - if used, it's handled by Platform_HandleMessage()
	ASURA_NETTRANSPORT_MSG_TESTNETMSG,								// 0xf00e
	ASURA_NETTRANSPORT_MSG_CLIENTLIST,								// 0xf00f
	ASURA_NETTRANSPORT_END_OF_MSG_IDS,
};

// CLASS DECLARATIONS /////////////////////////////////////////////////////////////////////////////

class Asura_Message;
class Asura_Network_ConnectData;
class Asura_Network_SessionData;
class Asura_Network_MachineInfo;
class Asura_Network_SessionInfo;
class Asura_NetSocket;
class Asura_NetConnection;
class Asura_NetSocket;
class Asura_HashTree;

// Asura_Close_Reason /////////////////////////////////////////////////////////////////////////////

typedef enum
{
	ASURA_CLOSE_LOCAL,
	ASURA_CLOSE_REMOTE,
	ASURA_CLOSE_ERROR_COMMUNICATION,
	ASURA_CLOSE_ERROR_SOCKETERROR,
	ASURA_CLOSE_ERROR_OUTOFMEMORY,
	ASURA_CLOSE_ERROR_BUFFERFULL,
	ASURA_CLOSE_ERROR_HOSTUNREACHABLE,
	ASURA_CLOSE_ERROR_MESSAGETOOLARGE,
	ASURA_CLOSE_ERROR_WRONGVERSION,
	ASURA_CLOSE_ERROR_CHECKSUM
} Asura_Close_Reason;

// Asura_Connection_State /////////////////////////////////////////////////////////////////////////

typedef enum
{
	ASURA_CONNECTION_CONNECTING,   // negotiating the connection
	ASURA_CONNECTION_CONNECTED,    // the connection is active
	ASURA_CONNECTION_CLOSING,      // the connection is being closed
	ASURA_CONNECTION_CLOSED        // the connection has been closed and can no longer be used
} Asura_Connection_State;

// Asura_Network_Result ///////////////////////////////////////////////////////////////////////////

typedef enum
{
	ASURA_NETWORK_SUCCESS,
	ASURA_NETWORK_OUTOFMEMORY,
	ASURA_NETWORK_REJECTED,
	ASURA_NETWORK_NETWORKERROR,
	ASURA_NETWORK_ADDRESSERROR,
	ASURA_NETWORK_DUPLICATEADDRESS,
	ASURA_NETWORK_TIMEDOUT,
	ASURA_NETWORK_NEGOTIATIONERROR,
} Asura_Network_Result;

// Asura_Network_Buffer_Entry /////////////////////////////////////////////////////////////////////

class Asura_Network_Buffer_Entry
{
	friend class Asura_Network_Buffer;
public:
	// Constructor
	Asura_Network_Buffer_Entry(): m_uSendTime(0), m_pcBuffer(NULL), m_uBufferSize(0)
	{
	}
	// Destructor
	~Asura_Network_Buffer_Entry()
	{
		DeleteBuffer();
	}
	void DeleteBuffer()
	{
		if(m_pcBuffer) delete [] m_pcBuffer;
	}
	void RemoveEntry()
	{
		m_uSendTime = m_uBufferSize = 0;
		m_pcBuffer = NULL;
	}
	// IsValid: Zero buffer or serial number is invalid
	bool IsValid() const
	{
#ifdef _DEBUG
		Asura_Assert(m_usSN == *(unsigned short*)m_pcBuffer, "Serial number does not match buffer; buffer possibly deleted?");
#endif
		return (m_uBufferSize && m_pcBuffer && *(unsigned short*)m_pcBuffer);
	}
	// Access methods
	unsigned int GetSendTime() const	{ return m_uSendTime; }
	unsigned int GetBufferSize() const	{ return m_uBufferSize; }
	const char *GetBuffer() const		{ return m_pcBuffer; }
private:
	// Members
	// Time message was sent
	unsigned int	m_uSendTime;
	// Buffer for message content. Serial number is first two bytes of this
	char			*m_pcBuffer;
	// Size of buffer
	unsigned int	m_uBufferSize;
	// Extra copy of serial number for debug integrity checks
#ifdef _DEBUG
	unsigned short	m_usSN;
#endif
};

// Asura_Network_Buffer ///////////////////////////////////////////////////////////////////////////

class Asura_Network_Buffer
{
public:
	// Constructor
	Asura_Network_Buffer();
	~Asura_Network_Buffer();

	void Reset();
	// Deletes message from buffer; used when a positive ACK is received
	void RemoveMessage(unsigned short usSN);
	// Adds a message; used on send
	bool AddMessage(unsigned int uSendTime, char *pcMessage, unsigned int uMessageLength);
	// Gets a message; used on periodic re-send of non-acknowledged messages
	//const Asura_Network_Buffer_Entry* GetMessage(unsigned short usSN) const;
	const Asura_Network_Buffer_Entry* GetMessage(unsigned int uIndex) const;
	// Returns the number of entries
	unsigned int GetNumberOfEntries() const { return m_uNumberOfMessages; }

	void CheckIntegrity() const;
private:
	// Our sequence numbers aren't really hashes,
	// but this tree will come in useful for random access using arbitrary numbers.
	// Holds indices into the following array of pointers.
	Asura_HashTree *m_pxIndexTree;
	// Pointer array stores the message data.
	Asura_Network_Buffer_Entry m_axMessageBuffer[ASURA_NETWORK_BUFFER_MAX_MESSAGES];
	// Current number of messages in array.
	unsigned int m_uNumberOfMessages;
	// Current total size of messages in array
	unsigned int m_uTotalMessageSize;
};

// Asura_NetTransport_System //////////////////////////////////////////////////////////////////////

class Asura_NetTransport_System
{
		friend class Asura_NetSocket;
		friend class Asura_NetConnection;
		friend class Asura_LanGameFinder;
		#ifdef ASURA_PC_SUPPORT_STEAMWORKS
		friend class Asura_PC_SteamworksClient;
		friend class Asura_PC_SteamworksServer;
		#endif

	public:

		// SNH: copied from Asura_GameSpy_GT2
		enum ClientConnectionState
		{
			CCS_NotConnected,
			CCS_WaitingForNatNeg,
			CCS_ReadyToConnect,
			CCS_WaitingForConnection,
			CCS_Connected,
			NumClientConnectionStates
		};

		static bool IsInitialised( void ) { return s_pxMachineInfo != 0; }
		static void Initialise( void );
		static void Update( void );
		static void EndOfFrame( void );
		static void Shutdown( void );
		
#ifdef ASURA_SUPPORT_COMMAND_CONSOLE
		static void AddConsoleCmdsAndVars( void );	// adds permanent commands (remove at shutdown).
#endif

		// socket needs updating from Peer when ANTS not initialised.
		static void UpdateSocket( void );

		/////////////////////////////////////////////////////////////////
		// session manager code
		/////////////////////////////////////////////////////////////////
		static void StartSessionManager( bool bUsingCustomSocket = false );
		static void StopSessionManager( void );
		static void UpdateLANSessions( void );

		/////////////////////////////////////////////////////////////////
		// server code
		/////////////////////////////////////////////////////////////////
		static bool StartServerAndHostSession( bool bInternetGame );
		static void StopServer( void );
		static void SetServerLANReporting( bool bEnable );
		static void SendMessageToClient( u_int uMachine, const Asura_Message* pxMessage );
		static void SendMessageToAllClients( const Asura_Message* pxMessage );
		static void SetServerName( const Asura_UnicodeChar* wszServerName );
		static void SetSessionData( const Asura_Network_SessionData* pxData, u_int uNumBytes );
		static u_int GetLatencyForMachine( u_int uMachine );
		static float GetPacketLossPercentForMachine( u_int uMachine );
#if defined(ASURA_SUPPORT_GAMESPY)
		static void ServerBeginNatNeg(int iCookie);
#endif
		static void CloseConnection( u_int uMachine, float fTime );

		/////////////////////////////////////////////////////////////////
		// OR - 26/11/2004 08:55:20 - Timeout code
		/////////////////////////////////////////////////////////////////
		static void  SetConnectionsTimeOutLimit(u_int uTimeInMS);
		static u_int GetConnectionsTimeOutLimit();
		static bool  CanConnectionsTimeOut();
		static bool  IsConnectionsTimeOutEnabled();
		static void  SetConnectionsTimeOutEnabled(bool bOnOff);
		static u_int GetIdleTimeForMachine( u_int uMachine );
		static u_int GetIdleTime( void );
		
		/////////////////////////////////////////////////////////////////
		// client code
		/////////////////////////////////////////////////////////////////
		static void StartClient(Asura_NetAddress xAddress);
		static void StopClient( void );
		static void ConnectToSession( u_int uSessionIndex );
		static void SendMessageToServer( const Asura_Message* pxMessage );
		static u_int GetLatency( void );
		static float GetPacketLossPercent( void );
		static u_int GetNumberOfActiveSessions() { return s_uNumberOfActiveSessions; }
#if defined(ASURA_SUPPORT_GAMESPY)
		// NAT Neg defaults to false - if server browsing detects that the server 
		// is behind a NAT and we need to use the NAT Negotiation SDK to connect, 
		// set this to true, and provide a random cookie.
		static void SetUseNatNeg( bool bNatNeg, int iCookie )
		{
			s_bUseNatNeg = bNatNeg;
			s_iNatNegCookie = iCookie;
		}
#endif

		/////////////////////////////////////////////////////////////////
		// client and server code
		/////////////////////////////////////////////////////////////////

		// GetSocket() returns the SOCKET object (this can be used in middleware code)
		static Asura_NetSocket* GetSocket();
		// CloseSocket() forces removal of the socket. Called when Asura_GameSpy_Peer is shutdown.
		//	Should not be called when ANTS is initialised.
		static void CloseSocket();
		static bool NeedToCloseSocketManually()	{ return !s_bSocketCreatedByANTS; }

		// SetPort()/GetPort() are used to specify/find the locally bound port
		// AJG - 24/01/08 - Note this sets the externally-set flag, so this should not be used by functions within Asura_NetTransport_System itself.
		static void SetPort(unsigned short usPort ) { s_usPort = usPort; s_bPortSetExternally = (usPort != 0); }
		static u_short GetPort( void ) { return s_usPort; }
		// SetServerPort()/GetServerPort() are for the server port to connect to.
		static void SetServerPort(unsigned short uPort ) { s_usServerPort = uPort; }
		static u_short GetServerPort( void ) { return s_usServerPort; }

		// this is an way of accessing which network adapter is used for platforms such as 
		// PC that support multiple ones. Other platforms just use 0.
		static void SetNetAdapter(int iAdapter) { s_iNetAdapter = iAdapter; }
		static int GetNetAdapter() { return s_iNetAdapter; }
		
		// functions to get IP address for each network adapter (as IP number, or string in the format
		// xxx.xxx.xxx.xxx:port (in chars or w_chars)
		static u_int GetNetAdapterIP( int iAdapter=0 ) { return Platform_GetNetAdaptorIP( iAdapter ); }
		static const char* GetNetAdapterIPStr( int iAdapter=0 );
		static const Asura_UnicodeChar* GetNetAdapterIPWStr( int iAdapter=0 );

		// GetStringFromAddress(): utility function to convert net address to string.
		// The string is returned in static buffer, so print or copy it immediately.
		static const char* GetStringFromAddress(Asura_NetAddress xAddress, unsigned short usPort) { return Platform_GetStringFromAddress( xAddress, usPort ); }
		// GetAddressFromString(): utility function to convert string to net address.
		static void GetAddressFromString(const char* szAddress, Asura_NetAddress* xAddress, unsigned short* pusPort) { return Platform_GetAddressFromString( szAddress, xAddress, pusPort ); }

		static void SetPassword( const char* szPassword );
		static u_int GetPasswordHash( void ) { return s_uPasswordHash; }
		static u_int GetMaxPlayersPerMachine( void ) { return s_uMaxPlayersPerMachine; }
		static void  SetMaxPlayersPerMachine( u_int uMaxPlayersPerMachine ) { s_uMaxPlayersPerMachine = uMaxPlayersPerMachine; }
		static const void* GetSessionData( void ) { return s_pucSessionData; }
		static u_int GetSessionDataSize( void ) { return s_uSessionDataSize; }
		static u_short Project_GetVersionID( void );
		static void GetNetworkBandwidthUsage(u_int& uSend, u_int& uReceive);
		static void GetPeakNetworkBandwidthUsage(u_int& uSend, u_int& uReceive);
		static void ResetNetworkBandwidthStats();
		static u_int GetMaxNumberOfMachines() { return s_uMaxNumberOfMachines; }
		static u_int GetServerSessionID() { return s_uServerSessionID; }
		static void  SetServerSessionID(u_int uServerSessionID) { s_uServerSessionID = uServerSessionID; }
		static void StartGame();

		// I'm not entirely comfortable with this function, as it exposes lower level functionality,
		// but it's necessary for the Xbox voice messages, which use peer to peer messaging shoehorned
		// over our client-server architecture
		static bool SendMessageToAddress(const void* pMessage, int iLength, Asura_NetAddress xAddress, bool bEncrypted);

		static Asura_Network_MachineInfo* GetMachineInfoFromIndex( u_int uMachine );
		static u_int GetMachineIndexFromInfo( const Asura_Network_MachineInfo* pxMachine );

		static bool ShouldAddMessageToQueue(u_int uMsgID);

#if _DEBUG
		static void BroadcastTestNetMsg(const void* pData, u_int uDataSize);
		static void ProcessTestNetMsg(Asura_NetAddress xAddress, u_short usPort, const char* pcData, u_int uDataLength);
#endif

		static u_short ComputeChecksum( const void* pData, int iLength );	// Needed by various networking classes

	protected: // Functions to be accessed by Asura_NetSocket and Asura_NetConnection

		/////////////////////////////////////////////////////////////////
		// server utility functions
		/////////////////////////////////////////////////////////////////
		static Asura_Network_MachineInfo* GetMachineInfo(Asura_NetConnection* pxConnection );

		static void SocketError( Asura_NetSocket* pxSocket );
		
		// functions to create the data required to initiate a connection. 
		// called by Asura_NetConnection when sending the request message.
		static u_int GetSizeOfConnectData();
		static void FillOutConnectData( char* pcData );
		
		/////////////////////////////////////////////////////////////////
		// server functions
		/////////////////////////////////////////////////////////////////
	public:

		// Asura_NetConnection needs access to this... plus any other class which wants to delay the accepting of net connections (e.g. Steamworks server)
		static void ProcessMachineConnection(Asura_NetConnection* pxConnection, const Asura_Network_ConnectData* pxConnectData, u_int uPing, const void* pExtraConnectData );
	
	private:
		
		static void ProcessEnumHostsRequest(Asura_NetAddress xAddress, u_short usPort, const char* pcData, u_int uDataLength );
		static void Project_RefreshSessionData();
		static u_int Project_GetExtraConnectDataSize( void );
		static void Project_FillOutExtraConnectData( void* pData );
		static bool Project_CheckExtraConnectData( const void* pData, u_int& uDisconnectReason );	// returns whether connection is allowed and the NDR reason if not.
		static void Project_ProcessConnection(Asura_NetAddress xAddress, u_short usPort, Asura_Guid uGuid, const void* pConnectData );
		static void Project_ProcessDisconnection( Asura_Guid uGuid );

		/////////////////////////////////////////////////////////////////
		// server callbacks
		/////////////////////////////////////////////////////////////////
		static void Server_ClientDisconnected(Asura_NetConnection* pxConnection);

#ifdef ASURA_SUPPORT_NOVAWORLD
		// Novaworld Project callbacks
		static void Project_ServerDataCallback(NovaworldAPI_ServerDataID eKey, char* acBuffer, u_int uBufferLength);
		static void Project_PlayerDataCallback(NovaworldAPI_PlayerDataID eKey, u_int uPlayerIndex, char* acBuffer, u_int uBufferLength);
#endif

#if defined(ASURA_SUPPORT_GAMESPY)
		//Callback prototype for your progress function
		static void NatNegProgress_ServerCallback(NegotiateState eState, void *pUserdata);
		//Callback prototype for your negotiation completed function
		static void NatNegComplete_ServerCallback(NegotiateResult eResult, SOCKET xGameSocket, struct sockaddr_in* pxRemoteAddr, void *pUserdata);
#endif

		/////////////////////////////////////////////////////////////////
		// client functions
		/////////////////////////////////////////////////////////////////
		static void ProcessEnumHostsResponse(Asura_NetAddress xAddress, u_short uPort, const char* pcData, int iDataSize );
		static void SendEnumHostsBroadcast( void );
		static void ConnectToHost(Asura_NetAddress xAddress, unsigned short uPort);
		
		/////////////////////////////////////////////////////////////////
		// client callbacks
		/////////////////////////////////////////////////////////////////
		static void Client_ClientConnected( Asura_NetConnection* pxConnection, Asura_Network_Result eResult, const char* pucMessage, int iMessageLen );
		static void Client_ClientDisconnected(Asura_NetConnection* pxConnection);

#if defined(ASURA_SUPPORT_GAMESPY)
		//Callback prototype for your progress function
		static void NatNegProgress_ClientCallback(NegotiateState eState, void *pUserdata);
		//Callback prototype for your negotiation completed function
		static void NatNegComplete_ClientCallback(NegotiateResult eResult, SOCKET xGameSocket, struct sockaddr_in* pxRemoteAddr, void *pUserdata);
#endif

		/////////////////////////////////////////////////////////////////
		// general callbacks
		/////////////////////////////////////////////////////////////////
		static void PingCallback( Asura_NetConnection* pxConnection, int iLatency );
		static bool HandleMessage( Asura_NetAddress xAddress, u_short usPort, const char* pcMessage, int iLength);

		/////////////////////////////////////////////////////////////////
		// general utility functions
		/////////////////////////////////////////////////////////////////
		static u_int Project_GetApplicationGuid( void );
		static u_int Project_GetNumPlayers( const void* pvProjectData );
		static u_int Project_GetMaxNumPlayers( const void* pvProjectData );
		static u_int SetTempGlobalBufferFromAsuraMessage( const Asura_Message* pxMessage );
		static void SendAsuraMessage(Asura_NetConnection* pxConnection, const Asura_Message* pxMessage );
		static bool Project_ShouldAddMessageToQueue(u_int uMsgID);
		static void UpdateBandwidthStats( void );
		static void AddDirectSendBytes(u_int uBytes) { s_uDirectSendBytes += ASURA_NETSOCKET_PROTOCOL_PACKETHEADERSIZE + uBytes; }
		static void AddDirectReceivedBytes(u_int uBytes) { s_uDirectReceiveBytes += ASURA_NETSOCKET_PROTOCOL_PACKETHEADERSIZE + uBytes; }
#if ASURA_NETWORK_LOGGING
		static bool GetMsgIDDesc(u_int uMsgID, const char** pszDesc);
#endif//ASURA_NETWORK_LOGGING
		/////////////////////////////////////////////////////////////////
		// Platform Functions
		/////////////////////////////////////////////////////////////////
		static void Platform_Initialise();
		static void Platform_Update();
		static void Platform_Shutdown();
		static void Platform_StartServer(bool bInternetGame);
		static void Platform_StopServer();
		static bool Platform_SendEnumHostsBroadcast();
		static void Platform_SendEnumHostsResponse( Asura_NetAddress xAddress, u_short usPort );
		static void Platform_ProcessEnumHostsRequest( Asura_NetAddress xAddress, u_short usPort, const char* pcData, u_int uDataLength );
		static void Platform_ProcessEnumHostsResponse( Asura_NetAddress xAddress, u_short usPort, const char* pcData, int iDataSize );
		static void Platform_Client_ClientConnected(Asura_NetConnection* pxConnection);
		static void Platform_Client_ClientDisconnected(Asura_NetConnection* pxConnection);
		static bool Platform_HandleMessage( Asura_NetAddress xAddress, u_short usPort, const char* pcMessage, int iLength);
		static void Platform_StartGame();
		static u_int Platform_GetNetAdaptorIP( int iAdapter );
		// Platform_AllocateNetSocket should return an instance of an appropriate class derived from 
		// Asura_NetSocket, allocated using new.
		static Asura_NetSocket* Platform_AllocateNetSocket( u_int uMaxMachines );
		static const char* Platform_GetStringFromAddress(Asura_NetAddress xAddress, unsigned short usPort);
		static void Platform_GetAddressFromString(const char* szAddress, Asura_NetAddress* xAddress, unsigned short* pusPort);

		/////////////////////////////////////////////////////////////////
		// For Xbox Matchmaking session registration
		/////////////////////////////////////////////////////////////////
		static void PlatformProject_StartInternetSession();
		static void PlatformProject_StopInternetSession();
		static void PlatformProject_UpdateInternetSession();

		/////////////////////////////////////////////////////////////////
		// static data
		/////////////////////////////////////////////////////////////////

		// static data - general

		// SNH - 18/10/2004 11:09:18 - GameSpy peer needs a socket before ANTS is initialised. It calls GetSocket to create
		//		one. To dump the players back to the lobby after game ends needs the socket to stick around. Therefore it's 
		//		only deleted if s_bSocketCreatedByANTS is true (eg if the socket was created in ANTS::Initialise)
		static bool							s_bSocketCreatedByANTS;
		static Asura_NetSocket*				s_pxSocket;

		static unsigned short				s_usPort;
		static int							s_iNetAdapter;
		static unsigned short				s_usServerPort;
		static u_int						s_uPasswordHash;
		static u_int						s_uServerSessionID;	// server-generated ID for this hosting session
		static u_int						s_auServerChallenge[2];	// current and previous server challenge value
		static u_int						s_uServerChallengeTime;
		static Asura_Network_MachineInfo*	s_pxMachineInfo;	// ptr to one machine info on client, and array on server.
		static u_int						s_uMaxNumberOfMachines;
		
		static u_char*						s_pucSessionData;
		static u_int						s_uSessionDataSize;

		// bandwidth usage
		static float						s_fTimeSinceLastBandwidthCalculation;
		static u_int						s_uSendBPS;
		static u_int						s_uReceiveBPS;
		static u_int						s_uPeakSendBPS;
		static u_int						s_uPeakReceiveBPS;
		static u_int						s_uDirectSendBytes;
		static u_int						s_uDirectReceiveBytes;

#ifdef ASURA_NETTRANSPORT_BANDWIDTH_LOG
		static u_int*						s_puAdverageBPSSent;
		static u_int*						s_puAdverageBPSReceived;
		static float						s_fTimeSinceLastBandwidthLog;
#endif

		// static data - timeout
		static u_int						s_uNetConnectionTimeOut;
		static bool							s_bDoConnectionTimeouts;

		// static data - server
		static bool							s_bServerReportingOn;
		static u_int						s_uMaxPlayersPerMachine;

		// static data - client
		static ClientConnectionState		s_eClientConnectionState;
		static u_int						s_uNumberOfActiveSessions;
		static Asura_Network_SessionInfo**	s_ppxSessionsStorage;
		static float						s_fLastSessionRequestBroadcastTime;
		static Asura_NetAddress				s_xAddressToConnectTo;
#if defined(ASURA_SUPPORT_GAMESPY)
		static bool s_bUseNatNeg;
		static int s_iNatNegCookie;
#endif
		// Flag for whether the port was set by the command line, to allow us to scan for ports each time we open the socket when it wasn't.
		static bool s_bPortSetExternally;
};

// Asura_Network_MachineInfo //////////////////////////////////////////////////////////////////////

class Asura_Network_MachineInfo
{
	public:

		Asura_Network_MachineInfo()
			: m_pxConnectionHandle(NULL)
			, m_uPing(999)
			, m_uPingTableEntry(0)
			, m_uTimeOfLastPing(0)
			, m_uSentBytes(0)
			, m_uRecvBytes(0)
			, m_uNumRetries(0)
			, m_uNumAcks(0)
		{}

		void SetConnectionHandle( Asura_NetConnection* pxConnection );

		Asura_NetConnection* GetConnectionHandle( void ) { return m_pxConnectionHandle; }

		void ClearConnection( void );
			
		bool IsValid( void ) const { return m_pxConnectionHandle != NULL; }
		
		// ping stuff
		u_int GetPing( void ) const
		{
			Asura_Assert(IsValid(), "ping of invalid connection requested");
			return m_uPing;
		}
		u_int GetTimeOfLastPing( void ) const { return m_uTimeOfLastPing; }
		void SetTimeOfLastPing( u_int uTime ) { m_uTimeOfLastPing = uTime; }
		void InitPing( u_int uPing )
		{
			m_uPing = uPing;
			for (u_int u = 0; u < ASURA_NETTRANSPORT_NUM_PING_SAMPLES; ++u)
			{
				m_auPingTable[u] = uPing;
			}
		}
		void SetPing( u_int uPing )
		{ 
			m_auPingTable[m_uPingTableEntry++] = uPing;
			m_uPingTableEntry &= ASURA_NETTRANSPORT_NUM_PING_SAMPLES_MASK;
			u_int uPingSum = 0;
			for (u_int u = 0; u < ASURA_NETTRANSPORT_NUM_PING_SAMPLES; ++u)
			{
				uPingSum += m_auPingTable[u];
			}
			m_uPing = uPingSum >> ASURA_NETTRANSPORT_NUM_PING_SAMPLES_SHIFT;
		}
		
		// how long is the connection been idle (not receiving any data).
		u_int GetIdleTime( void ) const;

		// bytes sent stuff
		// (MH - 20.03.2005) adding packetheader size in here rather than all over the code.
		// Add size of UDP header. This is about 48 on XBox and 28 on PC/PS2
		void AddSentBytes( u_int uNumBytes ) { m_uSentBytes += ASURA_NETSOCKET_PROTOCOL_PACKETHEADERSIZE + uNumBytes; }
		void AddReceivedBytes( u_int uNumBytes ) { m_uRecvBytes += ASURA_NETSOCKET_PROTOCOL_PACKETHEADERSIZE + uNumBytes; }
		u_int GetAndClearBytesSent( void ) 
		{
			u_int uReturn = m_uSentBytes;
			m_uSentBytes = 0;
			return uReturn; 
		}
		u_int GetAndClearBytesReceived( void ) 
		{
			u_int uReturn = m_uRecvBytes;
			m_uRecvBytes = 0;
			return uReturn;
		}
		
		// packet loss stats
		void IncrementNumRetries() { ++m_uNumRetries; }
		void AddAcks(u_int uNumAcks) { m_uNumAcks += uNumAcks; }
		float GetPacketLossPercent() const 
		{ 
			if (m_uNumAcks)
			{
				return 100.0f * static_cast<float>(m_uNumRetries) / static_cast<float>(m_uNumAcks + m_uNumRetries);
			}
			else
			{
				// 100% if we've had to resend anything, 0% if we haven't.
				return m_uNumRetries ? 100.0f : 0.0f;
			}
		}
		void ResetPacketLossStats();

	private:
		
		Asura_NetConnection* m_pxConnectionHandle;
		
		// ping stuff
		u_int m_uPing;
		u_int m_auPingTable[ASURA_NETTRANSPORT_NUM_PING_SAMPLES];
		u_int m_uPingTableEntry;
		u_int m_uTimeOfLastPing;

		// throughput stuff
		u_int m_uSentBytes;
		u_int m_uRecvBytes;

		// packet loss data
		u_int m_uNumRetries;
		u_int m_uNumAcks;
};


class Asura_NTS_PacketHeader
{
public:

	u_short m_usNBOSequenceNumber;
	u_short m_usNBOCheckSum;
	u_short m_usNBOMsgID;
	u_short m_usNBOGameVer;	// or if an asura message, this is entity classification.
};

// Asura_Network_ConnectData //////////////////////////////////////////////////////////////////////

class Asura_Network_ConnectData
{

	public:

		// should include password hash, game version, and, if supported, CD key
		u_short m_usNBOGameVersion;
		u_short m_usNBONumPlayers;
		u_int m_uNBOPasswordHash;
		u_int m_uNBOServerSessionID;
		u_int m_uNBOChallengeResponse;

		Asura_Lobby_GameConnectData m_xNBOLobbyData;
		u_int m_uNBOExtraDataSize;			// this much data 4 bytes on
		u_int m_uExtraPlatformDataSize;		// this much data m_uNBOExtraDataSize bytes on
};

// The following static assert checks that the size of the class is the same on all platforms.
// It's hard to justify why it works out the size it is, due to various bits of padding that
// get added by the compilers, but it simply has to match up on all platforms
ASURA_STATIC_ASSERT(sizeof(Asura_Network_ConnectData) == uASURA_PLATFORM_LOBBY_CONNECTDATA_SIZE + 24);

// Asura_Network_ConnectionRejectedData ///////////////////////////////////////////////////////////

class Asura_Network_ConnectionRejectedData
{

	public:
	
		u_int m_uNBOReason;
};

// Asura_Network_Challenge ////////////////////////////////////////////////

class Asura_Network_Challenge
{

	public:
		
		u_int m_uChallenge;	// actually used as 4 bytes for hashing so don't change to NBO

		u_int m_uAdditionalDataSize;	// Followed by this much data
};

// Asura_Network_ChallengeResponseMsg /////////////////////////////////////////////

class Asura_Network_ChallengeResponseMsg
{

	public:
		u_short m_usNBOSequenceNumber;	// always 0 - non-guaranteed
		u_short m_usNBOChecksum;
		u_short m_usNBOMsgID;			// always ASURA_NETTRANSPORT_MSG_CHALLENGE_RESPONSE
		u_short m_usNBOGameVer;
		Asura_Network_Challenge m_xData;
};


// Asura_Network_SessionInfo //////////////////////////////////////////////////////////////////////

class Asura_Network_SessionInfo
{
	public:

		Asura_Network_SessionInfo(Asura_NetAddress xHostAddr, u_short usHostPort, u_int uPing, u_int uServerSessionID, 
			const void* pPlatformData, u_int uPlatformDataSize, const void* pSessionData, u_int uSessionDataSize, 
			const Asura_UnicodeChar* pszSessionName, u_char ucNumPlayers, u_char ucMaxPlayers );
		~Asura_Network_SessionInfo();
		
		/*bool IsSameHost(u_int uHostAddr, u_short usHostPort) const 
		{ 
			return (uHostAddr == m_uHostAddr) && (usHostPort == m_usHostPort); 
		}*/

		bool Platform_IsSameHost(Asura_Network_SessionInfo* pxSession);

		void Refresh(u_int uPing, const void* pPlatformData, u_int uPlatformDataSize, const void* pSessionData, u_int uSessionDataSize, 
			const Asura_UnicodeChar* pszSessionName, u_char ucNumPlayers, u_char ucMaxPlayers);
		
		Asura_NetAddress GetHostAddr( void ) const { return m_xHostAddr; }
		u_short GetHostPort( void ) const { return m_usHostPort; }
		u_int GetNumPlayers( void ) const { return m_ucNumPlayers; }
		u_int GetMaxPlayers( void ) const { return m_ucMaxPlayers; }

		const Asura_UnicodeChar* GetSessionName( void ) const { return m_wszSessionName; }
		const void* GetSessionData( void ) const { return m_pSessionData; }
		u_int GetSessionDataSize( void ) const { return m_uSessionDataSize; }
		const void* GetPlatformData( void ) const { return m_pPlatformData; }
		u_int GetPlatformDataSize( void ) const { return m_uPlatformDataSize; }
		u_int GetPing( void ) const { return m_uPing; }
		u_int GetServerSessionID( void ) const { return m_uServerSessionID; }
		float GetLastUpdateTime( void ) const { return m_fLastUpdateTime; }
		void GetAddress( Asura_NetAddress* pxAddress, u_short* pusPort ) const { *pxAddress = m_xHostAddr; *pusPort = m_usHostPort; }

	private:
		
		// private and unimplemented copy construct and assignment operator
		Asura_Network_SessionInfo(const Asura_Network_SessionInfo& xFrom);
		Asura_Network_SessionInfo& operator=(Asura_Network_SessionInfo& xRHS);

		Asura_NetAddress m_xHostAddr;
		u_short m_usHostPort;

		float m_fLastUpdateTime;
		u_int m_uPing;
		u_int m_uServerSessionID;

		void* m_pPlatformData;
		u_int m_uPlatformDataSize;

		void* m_pSessionData;
		u_int m_uSessionDataSize;

		u_char m_ucNumPlayers;
		u_char m_ucMaxPlayers;

		Asura_UnicodeChar m_wszSessionName[uASURA_NETWORK_MAX_SESSION_NAME + 1];
};

#endif // #ifdef ASURA_SUPPORT_EXTERNAL_NETWORK

#endif//__ASURA_NETTRANSPORT_SYSTEM_H__
