//////////////////////////////////////////////////////////////////////////////
//
// Asura_ReliableSocket_TCP.h
//
// Copyright (c) 2009 Rebellion. All rights reserved.
//
// Description:
//
//     TCP socket implementations of Asura_ReliableSocket classes
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef ASURA_RELIABLESOCKET_TCP_H___
#define ASURA_RELIABLESOCKET_TCP_H___ 1

#ifdef ASURA_SUPPORT_EXTERNAL_NETWORK

#include "Network/Asura_ReliableSocket.h"
#include "Network/Asura_TCPSocket.h"

class Asura_ReliableSocket_Client_TCP : public Asura_ReliableSocket_Client
{
public:

	Asura_ReliableSocket_Client_TCP();

	virtual bool Connect( const Asura_NetAddress& xAddress, Asura_NetPort& xPort, const void* pConnectData, const u_short usConnectDataSize );
	virtual void Disconnect( const bool bImmediate, const ASURA_RELIABLE_SOCKET_DISCONNECT_REASON eReason );

	virtual void Update();

	virtual bool SendMessage( const u_short usMsgID, const u_short usMsgBodySize, const void* pMsgBody );

	virtual ASURA_RELIABLE_SOCKET_CLIENT_STATE GetState ();

private:

	static void MessageCallback(Asura_ReliableConnectionID xConnectionID, u_short usMsgID, u_short usMsgBodySize, const char* pcMsgBody, void* pUserData);
	static void DisconnectCallback(Asura_ReliableConnectionID xConnectionID, ASURA_TCP_DISCONNECT_REASON eReason, void* pUserData);

	Asura_TCPSocket_Connection m_xTCPSocket;
};

class Asura_ReliableSocket_Listener_TCP : public Asura_ReliableSocket_Listener
{
public:

	Asura_ReliableSocket_Listener_TCP();

	virtual bool Listen( const Asura_NetPort& xPort );
	virtual bool Listen( const Asura_NetAddress& xAddress, const Asura_NetPort& xPort );
	virtual void Close();
	virtual void Disconnect( const Asura_ReliableConnectionID& xConnectionID, const ASURA_RELIABLE_SOCKET_DISCONNECT_REASON eReason );

	virtual void Update();

	virtual bool SendMessage( const Asura_ReliableConnectionID& xConnectionID, const u_short usMsgID, const u_short usMsgBodySize, const void* pMsgBody );
	virtual bool SendMessageToAll( const u_short usMsgID, const u_short usMsgBodySize, const void* pMsgBody );
	virtual bool SendMessageToAllExcept( const u_short usMsgID, const u_short usMsgBodySize, const void* pMsgBody, const Asura_ReliableConnectionID& xException );

	virtual u_short GetListenPort() const;
	virtual u_int GetNumConnections() const;

	virtual ASURA_RELIABLE_SOCKET_LISTENER_STATE GetState ();

private:

	static bool ConnectCallback(Asura_ReliableConnectionID xConnectionID, u_short usMsgSize, const char* pcMsgBody, void* pUserData);
	static void DisconnectCallback(Asura_ReliableConnectionID xConnectionID, ASURA_TCP_DISCONNECT_REASON eReason, void* pUserData);
	static void MessageCallback(Asura_ReliableConnectionID xConnectionID, u_short usMsgID, u_short usMsgBodySize, const char* pcMsgBody, void* pUserData);

	Asura_TCPSocket_Listener m_xTCPSocket;
};

#endif // #ifdef ASURA_SUPPORT_EXTERNAL_NETWORK

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

#endif // #ifndef ASURA_RELIABLESOCKET_TCP_H___
