/**
*
* \file Asura_Server.cpp
*
* Created by Kevin Lea - 22/04/2002 10:36:58
*
* Copyright (C) 2002 Rebellion, All Rights Reserved.
*
* \brief
*
**/

#include "Core/Asura.h"

#include "Debugging/Asura_Assert.h"
#ifdef ASURA_SUPPORT_COMMAND_CONSOLE_REMOTE
#include "Interface/Asura_CommandConsole_Remote.h"
#endif
#ifdef ASURA_SUPPORT_MULTIPLAYER_COMMS
#include "Multiplayer/Asura_Multiplayer_Comms_System.h"
#endif
#include "Network/Asura_Message.h"
#include "Network/Asura_Network.h"
#ifdef ASURA_SUPPORT_NETMSG_SYSTEM
#include "Network/Asura_NetMsg.h"
#endif
#ifdef ASURA_SUPPORT_LOBBY
#include "Network/Online/Asura_Lobby.h"
#endif
#ifdef ASURA_USE_METAMUSICZONES
#include "Sound/Asura_MetaMusic_System.h"
#endif


#include "Physics/Asura_Physics_Broadphase.h"
#include "Physics/Asura_Physics_RigidBody.h"
#include "Entity/PhysicalObject/Asura_ApplyForce_Messages.h"
#include "Entity/PhysicalObject/Asura_ServerEntity_PhysicalObject.h"


void ProcessMessage_ApplyRadialForce( const Asura_Message* pxMessage )
{
	const Asura_Msg_ApplyForce_Radial* pxData = static_cast<const Asura_Msg_ApplyForce_Radial*>( pxMessage->GetData() );

	Asura_ServerEntity_PhysicalObject* pxSourceEntity  = Asura_ServerEntity_PhysicalObject::FindEntity( pxData->m_uSourcePosObjectGuid );
	if( !pxSourceEntity )
	{
		Asura_Assertf(false, ("Can't find Source pos object in Apply Radial Impulse message! (Guid %d). Set it in Message Editor.", pxData->m_uSourcePosObjectGuid ));

		pxSourceEntity = Asura_ServerEntity_PhysicalObject::FindEntity( pxMessage->GetFromGuid() );
		if( !pxSourceEntity )
		{
			Asura_Assertf(false, ("Can't even find the sender of Apply Linear Impulse message! (Guid %d). Apply Linear Impulse will have no effect!", pxMessage->GetFromGuid() ));
			return;
		}
	}

	ApplyForceMsg_ImpulseData xImpulseData;
	if( xImpulseData.FillFromBlueprint( pxData->m_uExtraInfoBPHash )==false )
	{
		// We couldn't find any info from this blueprint.
		return;
	}

	const Asura_Vector_3& xPos = pxSourceEntity->GetPosition();

	// Search for all entities that are around
	Asura_Physics_Broadphase_Query xQuery;
	Asura_Bounding_Box xBox;
	xBox.SetToPoint(xPos);
	xBox.Grow( xImpulseData.m_fFarRange );
	xQuery.MakeQuery(xBox, true, true, true, true);

	const Asura_Physics_Broadphase_Entry* pxCurrentEntry = xQuery.GetFirstEntry();
	while (pxCurrentEntry)
	{
		const Asura_Physics_RigidBodyInstance* pxBody = pxCurrentEntry->GetInstance();
		if( pxBody )
		{
			const Asura_Guid uEntityGuid = pxBody->GetAssociatedEntityGuid();
			Asura_ServerEntity_PhysicalObject* pxDestEntity = Asura_ServerEntity_PhysicalObject::FindEntity( uEntityGuid );
			if( pxDestEntity )
			{
				pxDestEntity->ApplyLinearImpuseFromMsg( xPos, Asura_Maths::ZeroVector3, xImpulseData );
			}
		}

		pxCurrentEntry = pxCurrentEntry->GetNextEntry();
	}
}


void Asura_Network::ServerMessageHandler(const Asura_Message* pxMessage)
{
	switch(pxMessage->GetMsgID())
	{
//#ifdef ASURA_SUPPORT_NETMSG_SYSTEM
//		case ASURA_MSG_NETMSGUPDATE:
//		{
//			// send messages to server from client (not from server to client)
//			Asura_NetMsg_System::ProcessSlaveUpdatePackage(false, reinterpret_cast<const u_char*>(pxMessage->GetData()), static_cast<u_short>(pxMessage->GetDataSize()), pxMessage->GetSentTime());
//			break;
//		}
//#endif//#ifdef ASURA_SUPPORT_NETMSG_SYSTEM
#ifdef ASURA_SUPPORT_COMMAND_CONSOLE
#ifdef ASURA_SUPPORT_COMMAND_CONSOLE_REMOTE
        case ASURA_MSG_COMMANDCONSOLE_REMOTE_NONCE_REQ:
        case ASURA_MSG_COMMANDCONSOLE_REMOTE_EXEC_CMD:
            Asura_CommandConsole_Remote::ProcessMessage( pxMessage );
            break;
#endif //ASURA_SUPPORT_COMMAND_CONSOLE_REMOTE
#endif //ASURA_SUPPORT_COMMAND_CONSOLE

#ifdef ASURA_SUPPORT_MULTIPLAYER_COMMS
		case ASURA_MSG_MULTIPLAYERCOMMS_TALK :
		{
			// broadcast to all players
			Asura_Network::Forward( pxMessage, ASURA_GUID_ALL_PLAYERS );
			break;
		}			
		case ASURA_MSG_MULTIPLAYERCOMMS_TELL :
		{
			// send to one player
			const Asura_Msg_MultiPlayerComms_Tell* pxData = static_cast<const Asura_Msg_MultiPlayerComms_Tell*> (pxMessage->GetData());
			if(pxData)
			{		
				Asura_Network::Forward( pxMessage, ASURA_GUID_PLAYER, pxData->m_uTo );
			}
			break;
		}
#endif			

// OR - 23/11/2004 16:17:59 - for BHD. Need to have the project handle these at the moment. 
// some files will be moved to asura once it's been finished, and all will be well.
#if 0 //#ifdef ASURA_SUPPORT_GAMESPY_VOICE
		case ASURA_MSG_VOICEDATA:
		{
			// OR - 23/11/2004 16:15:54 - project want s to handle that message. 
			if (s_pfnProject_ServerMessageHandler && s_pfnProject_ServerMessageHandler(pxMessage))
				return true;

			// broadcast to all other client machines
			Asura_Network::Forward( pxMessage, ASURA_GUID_OTHER_PLAYERS_CLIENTMACHINES );
			break;
		}
#endif // #ifdef ASURA_SUPPORT_GAMESPY_VOICE

#ifdef ASURA_USE_METAMUSICZONES
		case ASURA_MSG_METAMUSIC_ACTIVATEZONE :
		case ASURA_MSG_METAMUSIC_DEACTIVATEZONE :
			Asura_MetaMusic_System::HandleMessage(pxMessage);
	        //Forward this message to other clients as well , so it might have some vague
			//chance of working in multiplayer games.RAR 02/11/2005
	        Asura_Network::Forward(pxMessage,ASURA_GUID_ALL_OTHER_CLIENTMACHINES);
			break;
#endif // ASURA_USE_METAMUSICZONES

		case ASURA_MSG_SKIP_CUTSCENES:
			// Forward on to other clients, so everyone gets the skip message (originator has already skipped)
			Asura_Network::Forward(pxMessage, ASURA_GUID_OTHER_PLAYERS_CLIENTMACHINES);
			break;

		case ASURA_MSG_APPLY_RADIAL_FORCE:
			{
				ProcessMessage_ApplyRadialForce( pxMessage );
				break;
			}

		default:
		{
			//Check for platform specific messages. RAR 10/09/2003
			if(Platform_ServerMessageHandler(pxMessage))
			{
				break;
			}


			Asura_Assert(s_pfnProject_ServerMessageHandler,"This machine has been set to be server, but no server message handler has been provided for this project.");
			if (s_pfnProject_ServerMessageHandler)
			{
				s_pfnProject_ServerMessageHandler(pxMessage);
			}
			break;
		}
	}
}
