//
// Asura_DemonWare.h
//
// Created by Mark Dieterman - 19/10/07
//
// Copyright (C) 2007 Rebellion (Liverpool), All Rights Reserved.
//
// Wrapper class to allow integration of DemonWare's Matchmaking+ SDK
//
// Current DW version 1.80
//

#if defined(ASURA_USE_PRAGMA_ONCE)
#pragma once
#endif

#if !defined(__ASURA_DEMONWARE_H__)
#define __ASURA_DEMONWARE_H__

#if defined(ASURA_SUPPORT_EXTERNAL_NETWORK) && defined (ASURA_SUPPORT_DEMONWARE)
#include <bdLobby/bdLobbyErrorCode.h>

// Forward declarations
class Asura_DemonWare_PacketInterceptor;
class Asura_DemonWare_ConnectionListener;
class bdAuthService;
class bdAuthInfo;
class Asura_NetSocket;
class bdNetStartParams;
struct Asura_NetSocket_MessageInfo;
class bdMatchMakingInfo;
class bdRemoteTask;
class bdQueryResult;
class Asura_DemonWare_MatchMakingManager;
class bdConnection;
class bdCustomQueryParams;
class bdPlatformParams;
class bdString;
class bdInetAddr;
class bdLogSubscriber;
class bdGetHostByName;
class bdInetAddr;
class bdMatchMaking;
class Asura_Demonware_CustomMatchMakingInfo;
//
//	Asura_DemonWare
//
#define MAX_NUM_SESSIONS (10u)
#define BD_MAX_SESSIONNAME_LENGTH (32u + 1u)
// DemonWare server strings
#define ASURA_DEMONWARE_AUTHORISATION_SERVER "auth.eval.v1.80.mmp3.demonware.net"
#define ASURA_DEMONWARE_LOBBY_SERVER "lobby.eval.v1.80.mmp3.demonware.net"
#define LSG_PORT		(3074)
#define AUTH_PORT		(3074)


#define CHANGED_GAME_TYPE (2)
// MRD 12/10/07 - the list of tasks the Demonware system 
// can be asked to perform at any given time
enum Asura_Demonware_Task
{
	ASURA_DW_TASK_STOPPED,
	ASURA_DW_TASK_START_NETWORK,
	ASURA_DW_TASK_CHECK_DNS,
	ASURA_DW_TASK_CREATE_ACCOUNT,			
	ASURA_DW_TASK_AUTHENTICATE_ACCOUNT,
	ASURA_DW_TASK_LOBBY_CONNECT,
	ASURA_DW_TASK_ADVERTIZE_SESSION,
	ASURA_DW_TASK_FIND_SESSION_BY_ID,
	ASURA_DW_TASK_UPDATE_SESSION,
	ASURA_DW_TASK_FIND_SESSIONS_BY_GAME_TYPE,
	ASURA_DW_TASK_LOBBY_DISCONNECT,
	ASURA_DW_TASK_STOP_NETWORK,
};

// MRD 12/10/07 - how is the current task getting on
enum Asura_Demonware_TaskStatus
{
	ASURA_DW_READY,
	ASURA_DW_PENDING,
	ASURA_DW_DONE,
	ASURA_DW_FAILED,
	ASURA_DW_SKIPPED
};

class Asura_Demonware
{
public:

	Asura_Demonware();
	~Asura_Demonware();

	void Init();
	void ShutDown();

	static bool IsRunning()
	{
		return ( s_pxInstance != NULL );
	}
	static Asura_Demonware* GetInstance();
	// The master update function
	void Update( float fTimeSlice );
	// Task start request and pump routines
	void RequestStartTask( Asura_Demonware_Task eTask );
	void StartTask();
	void PumpTask();

	// Network startup functions
	void StartNetwork();
	void PumpStartNetwork();

	// DNS lookup
	void CheckDNSDecription( char* szDescription);
	void StartCheckDNS();
	void PumpCheckDNS();

	// Authentication Service
	bdAuthService* GetAuthService();
	const bdAuthInfo* GetAuthInfo();
	void CheckAuthServiceError();
	void PumpAuthServiceTask();

	// Create Account
	void CreateAccountDescription(char* szDescription);
	void StartCreateAccount();
	void PumpCreateAccount();

	// Authenticate account
	void AuthenticateAccountDescription(char* szDescription);
	void StartAuthenticateAccount();
	void PumpAuthenticateAccount();

	// Lobby service connect
	void LobbyServiceConnectDescription(char* szDescription);
	void StartLobbyServiceConnect();
	void PumpLobbyServiceConnect();

	// MatchMaking
	void CheckMatchMakingError();
	void PrintSessionList(char* szDescription) const;

	// Advertize session on master server
	void AdvertizeSessionDescription( char* szDescription );
	void StartAdvertizeSession();
	void PumpAdvertizeSession();

	// Find session by id
	void FindSessionByIDDescription( char* szDescription);
	void StartFindSessionByID();
	void PumpFindSessionByID();

	// Update session
	void UpdateSessionDescription( char* szDescription);
	void StartUpdateSession();
	void PumpUpdateSession();

	// Find sessions by game type
	void FindSessionsByGameTypeDescription( char* szDescription);
	void StartFindSessionsByGameType();
	void PumpFindSessionsByGameType();	

	// Lobby disconnect
	void LobbyServiceDisconnectDescription( char* szDescription);
	void StartLobbyServiceDisconnect();
	void PumpLobbyServiceDisconnect();

	// Network stop
	void StopNetworkDescription( char* szDescription);
	void StopNetwork();
	void HandleTaskEnd();

	int32 Platform_Initialise( bdNetStartParams& xParams);
	void Platform_ShutDown(void);

	bdMatchMaking* GetMatchMaking();
	Asura_Demonware_TaskStatus CheckRemoteTaskStatus(bdRemoteTask *const remoteTask);
	Asura_Demonware_TaskStatus StartGetHostByName(const char* domainNameStr,
		bdGetHostByName **getHostByName,
		const bdInetAddr resolvedAddr);
	Asura_Demonware_TaskStatus PumpGetHostByName(bdGetHostByName **getHostByName,
		bdInetAddr &resolvedAddr);

	SOCKET GetSocket(void);
	static unsigned short int GetPort(void)
	{
		return AUTH_PORT;
	}
	// Asura_DemonWare_PacketInterceptor deferred calls
	bool SendPacket(const Asura_NetSocket_MessageInfo* const pxMessageInfo);
	bool ArePacketsPending(void);
	bool HandleNextPacket(Asura_NetSocket* pxAsuraSocket);

private:

	static Asura_Demonware* s_pxInstance;
	bdLogSubscriber* m_pxLogSubscriber;
	Asura_Demonware_TaskStatus m_eTaskStatus;
	Asura_String m_lastTaskDescription;

	Asura_Demonware_Task m_eCurrentTask;
	bdInetAddr* m_pxAuthAddr;
	bdGetHostByName* m_pxAuthGetHostByName;

	bdAuthService* m_pxAuthService;

	bdInetAddr* m_pxlsgAddr;
	bdGetHostByName* m_pxLsgGetHostByName;

	Asura_Demonware_CustomMatchMakingInfo* m_pxSession;
	bdRemoteTask* m_pxRemoteTask;

	bool m_bPumpLobbyConnection;
	bdLobbyErrorCode m_eLobbyErrorCode;
	bdMatchMakingInfo* m_foundSessions;
	u_int				m_uNumSessions;

	Asura_DemonWare_PacketInterceptor* m_pxPacketInterceptor;
	Asura_DemonWare_ConnectionListener* m_pxConnectionListener;
};
#endif // ASURA_SUPPORT_EXTERNAL_NETWORK && ASURA_SUPPORT_DEMONWARE

#endif // __ASURA_DEMONWARE_H__