//
// Asura_DemonWare_PacketInterceptor.cpp
//
// Created by Andrew Catlender - 31 January 2006 16:35:20
//
// Copyright (C) 2006 Rebellion (Derby), All Rights Reserved.
//
//
// Wrapper class to allow integration of DemonWare's Matchmaking+ SDK
//

// Asura include
#include "Core/Asura.h"

#if defined(ASURA_SUPPORT_EXTERNAL_NETWORK) && defined (ASURA_SUPPORT_DEMONWARE)

// Middleware includes (must be guarded by Asura_MemoryManagement_DebugOff/On)
#if defined(SN_TARGET_PS2)
// ARC - 23 March 2006 16:13:24 - I know this is supposed to be platform independent, but it's necessary for the DemonWare stuff
#pragma mathwarn off
#endif // SN_TARGET_PS2

#include "Core/Asura_MemoryManagement_DebugOff.h"
#include "DemonWare/bdNet/bdNet.h"
#include "DemonWare/bdCore/bdCore.h"
#include "DemonWare/bdSocket/bdPacketInterceptor.h"

#if defined(SN_TARGET_PS2)
#pragma mathwarn on
#endif // SN_TARGET_PS2

// Engine includes
#include "Debugging/Asura_Assert.h"
#include "Network/Asura_DemonWare_LogSubscriber.h"
#include "Network/Asura_NetSocket.h"
#if defined(ASURA_ENABLE_INTERNET_SIMULATION)
#include "Network/Asura_InternetSimulation.h"
#endif // ASURA_ENABLE_INTERNET_SIMULATION

// This file's header
#include "Network/Asura_DemonWare_PacketInterceptor.h"

// Initialise static constants
static const char scGamePacketID = 0xffu;

//
//	~Asura_DemonWare_PacketInterceptor()
//
Asura_DemonWare_PacketInterceptor::~Asura_DemonWare_PacketInterceptor(void)
{
#if defined(_DEBUG)
	ASURA_DEMONWARE_LOG(("Asura_DemonWare_PacketBuffer high water mark [%i]", GetHighWaterMark()));
#endif
}

//
//	acceptPacket()
//
bdBool Asura_DemonWare_PacketInterceptor::acceptPacket(bdDefaultSocket *xSocket, bdAddr xAddr, void *pData, const bdUInt uiSize, const bdUByte8 type )
{
	// ASURA_NETWORK_MESSAGE_BUFFER_SIZE is defined in Asura_Network.cpp as
	// 1290 - sizeof(Asura_MessageHeader) - 4 so it should always be less than
	// BD_MAX_DATAGRAM_SIZE, which is 1328
	static char acDemonWareMessageBuffer[BD_MAX_DATAGRAM_SIZE];

	// Copy the incoming packet data so that the game ID byte is in the first
	// 32 bits, and the actual Asura game data is 32 bit aligned from byte 4 in
	// the buffer onwards
	memcpy(acDemonWareMessageBuffer + 3, pData, uiSize);
	char cFirstByte = *(reinterpret_cast<char*>(pData));

	if ((uiSize > 1) && (cFirstByte == scGamePacketID))
	{
		// Packet contains game traffic

		// Need to add the contents to the received data queue with the following
		// addToRecv(pucData + 1, uiSize - 1, xAddr);
		// or if the addresses aren't real addresses
		// addToRecv(pucData + 1, uiSize - 1, myAddToRealAddr.reverseGet(xAddr));

		AddPacket(acDemonWareMessageBuffer + 4, uiSize - 1, xAddr.getAddress().toUInt32(), xAddr.getPort());
		return true;
	}

	// Packet contains DemonWare traffic and should not be handled here
	return false;
}

//
//	sendPacket()
//
bool Asura_DemonWare_PacketInterceptor::sendPacket(const Asura_NetSocket_MessageInfo* const pxMessageInfo)
{
	// ASURA_NETWORK_MESSAGE_BUFFER_SIZE is defined in Asura_Network.cpp as
	// 1290 - sizeof(Asura_MessageHeader) - 4 so it should always be less than
	// BD_MAX_DATAGRAM_SIZE, which is 1328
	static char acDemonWareMessageBuffer[BD_MAX_DATAGRAM_SIZE];

	// Copy the message payload, prepended with the game ID byte so
	// acceptPacket() above can filter out game packets from DemonWare data
	acDemonWareMessageBuffer[0] = scGamePacketID;
	memcpy(acDemonWareMessageBuffer + 1, pxMessageInfo->m_pcMessage, pxMessageInfo->m_iLength);

	bdDefaultSocket* pxSocket = bdNet::getInstance()->getParams().m_socket;
	bdAddr xAddress(bdInetAddr(pxMessageInfo->m_xAddress), pxMessageInfo->m_usPort);
	bdInt iBytesSent = pxSocket->sendTo(xAddress, acDemonWareMessageBuffer, pxMessageInfo->m_iLength + 1);

	return (iBytesSent >= 0);
}

//
//	AddPacket()
//
void Asura_DemonWare_PacketInterceptor::AddPacket(void* pPacket, u_int uiLength, Asura_NetAddress xAddress, unsigned short int usPort)
{
	if (m_uiUsedSlots < ASURA_DEMONWARE_PACKET_BUFFER_SIZE)
	{
		m_xBuffer[m_uiHead].m_uiLength = uiLength;
		m_xBuffer[m_uiHead].m_xAddress = xAddress;
		m_xBuffer[m_uiHead].m_usPort = usPort;
		memcpy(m_xBuffer[m_uiHead].m_acPayload, pPacket, uiLength);

		m_uiHead++;
		if (m_uiHead >= ASURA_DEMONWARE_PACKET_BUFFER_SIZE)
		{
			m_uiHead = 0;
		}

		m_uiUsedSlots++;

		if (m_uiHighWaterMark < m_uiUsedSlots)
		{
			m_uiHighWaterMark = m_uiUsedSlots;
		}

		return;
	}

	Asura_Assert(false, "Asura_DemonWare_PacketBuffer::AddPacket() : unable to buffer any more packets - discarding");
}

//
//	ArePacketsPending()
//
bool Asura_DemonWare_PacketInterceptor::ArePacketsPending(void)
{
	if (m_uiUsedSlots > 0)
		return true;
	else
		return false;
}

//
//	HandleNextPacket()
//
bool Asura_DemonWare_PacketInterceptor::HandleNextPacket(Asura_NetSocket* pxAsuraSocket)
{
	if (m_uiUsedSlots > 0)
	{
		bool bResult = pxAsuraSocket->HandleMessage(m_xBuffer[m_uiTail].m_acPayload, m_xBuffer[m_uiTail].m_uiLength, m_xBuffer[m_uiTail].m_xAddress, m_xBuffer[m_uiTail].m_usPort);

		m_uiTail++;
		if (m_uiTail >= ASURA_DEMONWARE_PACKET_BUFFER_SIZE)
		{
			m_uiTail = 0;
		}

		m_uiUsedSlots--;

		return bResult;
	}

	Asura_Assert(false, "Asura_DemonWare_PacketBuffer::HandleNextPacket() : called when no packets were buffered");
	return false;
}

#if defined(_DEBUG)
//
//  GetHighWaterMark()
//
u_int Asura_DemonWare_PacketInterceptor::GetHighWaterMark(void)
{
	return m_uiHighWaterMark;
}
#endif

#endif // ASURA_SUPPORT_EXTERNAL_NETWORK && ASURA_SUPPORT_DEMONWARE
