/**
*
* \file Asura_GameSpy_Defines.h
*
* Created by Andrew Weinkove - 13/01/05
*
* Copyright (C) 2005 Rebellion, All Rights Reserved.
*
* \Gamespy defines seperated into their own file to reduce dependencies.
*
**/


#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif

#ifndef __ASURA_GAMESPY_DEFINES_H__
#define __ASURA_GAMESPY_DEFINES_H__

/************/
/* Includes.*/
/************/

// asura
#include "Core/Asura_PlatformBasics.h"

// gamespy
#include "GameSpy/GP/gp.h"

/*********/
/* Enums */
/*********/

// directly mapped to gamespy enum...
enum ASURA_GSGP_RESULT 
{
	ASURA_GSGP_OK				= GP_NO_ERROR,
	ASURA_GSGP_ERROR_MEMORY		= GP_MEMORY_ERROR,
	ASURA_GSGP_ERROR_PARAMETER	= GP_PARAMETER_ERROR,
	ASURA_GSGP_ERROR_NETWORK	= GP_NETWORK_ERROR,
	ASURA_GSGP_ERROR_SERVER		= GP_SERVER_ERROR,
	ASURA_GSGP_WORKING
};

// status flags
#define  ASURA_GSGP_STATUSFLAG_LOGGINGIN				0x01
#define  ASURA_GSGP_STATUSFLAG_CREATINGPROFILE			0x02
#define  ASURA_GSGP_STATUSFLAG_REGISTERINGNICK			0x04
#define  ASURA_GSGP_STATUSFLAG_GETTINGSUGGESTIONS		0x08
#define  ASURA_GSGP_STATUSFLAG_GETTINGINFO				0x10
#define  ASURA_GSGP_STATUSFLAG_GETTINGREVERSEBUDDIES	0x20

#define  ASURA_GSGP_STATUS_READY  0

enum ASURA_GSGP_ERRORCODE 
{	
	ASURA_GSGP_LOGINERROR_TIMEOUT		   = GP_LOGIN_TIMEOUT,
	ASURA_GSGP_LOGINERROR_BADNICK		   = GP_LOGIN_BAD_NICK,
	ASURA_GSGP_LOGINERROR_BADEMAIL		   = GP_LOGIN_BAD_EMAIL,
	ASURA_GSGP_LOGINERROR_BADPASSWORD	   = GP_LOGIN_BAD_PASSWORD,
	ASURA_GSGP_LOGINERROR_BADPROFILE	   = GP_LOGIN_BAD_PROFILE,
	ASURA_GSGP_LOGINERROR_PROFILEDELETED   = GP_LOGIN_PROFILE_DELETED,
	ASURA_GSGP_LOGINERROR_CONNECTIONFAILED = GP_LOGIN_CONNECTION_FAILED,
	ASURA_GSGP_LOGINERROR_AUTHFAILED       = GP_LOGIN_SERVER_AUTH_FAILED,
	ASURA_GSGP_LOGINERROR_BADUNIQUENICK    = GP_LOGIN_BAD_UNIQUENICK,
	ASURA_GSGP_LOGINERROR_BADPREAUTH	   = GP_LOGIN_BAD_PREAUTH,

	ASURA_GSGP_NEWUSERERROR_BADNICK           = GP_NEWUSER_BAD_NICK,
	ASURA_GSGP_NEWUSERERROR_BADPASSWORD       = GP_NEWUSER_BAD_PASSWORD,
	ASURA_GSGP_NEWUSERERROR_UNIQUENICKINVALID = GP_NEWUSER_UNIQUENICK_INVALID,
	ASURA_GSGP_NEWUSERERROR_UNIQUENICKINUSE   = GP_NEWUSER_UNIQUENICK_INUSE,
};

enum ASURA_GSGP_ACTIVESTATUS
{
	ASURA_GSGP_STATUS_UNKNOWN	= -1,
	ASURA_GSGP_STATUS_OFFLINE	= GP_OFFLINE,
	ASURA_GSGP_STATUS_ONLINE	= GP_ONLINE,
	ASURA_GSGP_STATUS_PLAYING	= GP_PLAYING,
	ASURA_GSGP_STATUS_STAGING	= GP_STAGING,
	ASURA_GSGP_STATUS_CHATTING	= GP_CHATTING,
	ASURA_GSGP_STATUS_AWAY		= GP_AWAY,
};

static const u_int uASURA_GAMESPY_EMAIL_MAX_LENGTH = GP_EMAIL_LEN;
static const u_int uASURA_GAMESPY_NICKNAME_MAX_LENGTH = GP_NICK_LEN;
static const u_int uASURA_GAMESPY_PASSWORD_MAX_LENGTH = GP_PASSWORD_LEN;
static const u_int uASURA_GAMESPY_MAX_NICKNAME_SUGGESTIONS = 10;
static const u_int uASURA_GAMESPY_MAX_BUDDIES = 50;
static const u_int uASURA_GAMESPY_MAX_BUDDY_REQUESTS = 8;
static const u_int uASURA_GAMESPY_MAX_BUDDY_MESSAGES = 4;// number held in queue for instant display
static const u_int uASURA_GAMESPY_MAX_BUDDY_MESSAGES_IN_ARCHIVE = 16;// number stored for later reading.
static const u_int uASURA_GAMESPY_MAX_BUDDY_MESSAGE_LENGTH = 1024;


#endif//__ASURA_GAMESPY_DEFINES_H__
