/**
*
* \file Asura_Network_StatLogging.h
*
* Created by Richard May - 20/11/2003 11:59:20
*
* Copyright (C) 2002 Rebellion, All Rights Reserved.
*
* \brief
*
**/


#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif

#ifndef __ASURA_GAMESPY_STATLOGGING_H__
#define __ASURA_GAMESPY_STATLOGGING_H__

#ifdef ASURA_SUPPORT_GAMESPY
/************/
/* Includes.*/
/************/

// asura
#include "Network/Asura_Guid.h"

/***********/
/* Defines */
/***********/

#define  ASURA_STATLOGGING_MAXSNAPSHOTSIZE  1024

/************/
/* Classes. */
/************/

/*************************************************************
 *
 * Class Name    : Asura_GameSpy_StatLogging
 *
 * Author        : Richard May
 *
 * Description   : Gamespy stat logging SDK interface. Used 
 *				   by the various platform-specific network stat
 *				   logging systems
 *
 *************************************************************/
class Asura_GameSpy_StatLogging
{	

public:
	/* functions */
	static void Initialise(void);
	static void Shutdown(void);

	static bool StartSession(void);
	static void EndSession(void);
	
	static bool SendSnapshot( bool bGameEnd );

protected:
	/* Helper functions */
	static void GenerateGameSpyKey( char* szKey, const class Asura_StatLogging_Stat* pxStat );
	static u_int GetPlayerIDFromGuid ( Asura_Guid uPlayer );
	static u_int GetPlayerIDFromID ( int iPlayerID );
	static u_int GetTeamIDFromTeam ( int iTeam );
	static void GetPlayerAuthentification( char* szAuth );	

	static void InitPlayerIDStorage(void);
	static void InitTeamIDStorage(void);

protected:
	/* data */
	static char s_szSnapshotBuffer[ASURA_STATLOGGING_MAXSNAPSHOTSIZE];
	
	static struct GuidAndPlayerID* s_pxPlayerIDStore;
	static u_int s_uNextPlayerID;	
	static u_int s_uPlayerIDStoreSize;
	
	static struct TeamAndTeamID*   s_pxTeamIDStore;
	static u_int s_uNextTeamID;
	static u_int s_uTeamIDStoreSize;
};
/* end of class Asura_GameSpy_StatLogging */

#endif //#ifdef ASURA_SUPPORT_GAMESPY
/****************/
/* End of file. */
/****************/
#endif//__ASURA_GAMESPY_STATLOGGING_H__
