//
// Asura_Physics_BodyState.h
//
// Created by KJFL - 30 March 2009
//
// Copyright (C) 2009 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __ASURA_PHYSICS_BODYSTATE_H__
#define __ASURA_PHYSICS_BODYSTATE_H__

#include "Core/Asura_Maths.h"

// KJFL - 30 March 2009 - Minimal data class designed to hold the
// physical properties of a body which are modified every frame
// (unlike say mass or elasticity).
// This data needs to be read by the solver routines, and then
// crucially written back to, potentially from different CPU cores
// (e.g. SPUs). By collecting these properties in one class we can
// reuse this across multiple body types (e.g. both ragdolls & 
// "normal" objects.)
class Asura_Physics_BodyState
{
	public:

		void Integrate( const Asura_AVector3& xLinearVelocity, const Asura_AVector3& xAngularVelocity )
		{
			m_xCentreOfMass += xLinearVelocity;

			Asura_AMatrix3x3 xDelta;
			xDelta.Tilde( xAngularVelocity );		
			xDelta = m_xOrientation*xDelta;
			m_xOrientation.Add( xDelta );
			m_xOrientation.Orthogonalise();
		}

		Asura_AMatrix3x3	m_xWorldInvInertiaTensor;
		Asura_AMatrix3x3	m_xOrientation;

		Asura_AVector3		m_xCentreOfMass;
		Asura_AVector3		m_xPreviousCentreOfMass;
	
		Asura_AVector3		m_xLinearVelocity;
		Asura_AVector3		m_xAngularVelocity;

};

#endif//__ASURA_PHYSICS_BODYSTATE_H__

