//
// Asura_Physics_Broadphase.h
//
// Created by KJFL - 20 March 2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __ASURA_PHYSICS_BROADPHASE_H__
#define __ASURA_PHYSICS_BROADPHASE_H__

#include "Core/Asura_Handle.h" 

class Asura_Physics_RigidBodyInstance;
class Asura_Physics_Broadphase_Query;
class Asura_Bounding_Box;
class Asura_OBB;

class Asura_Physics_Broadphase_System
{
	public:

		static void Reset( void );

		static void ResetStaticBroadphase( void );
		static void ResetDynamicBroadphase( void );

		static void AddObjectToStaticBroadphase( Asura_Physics_RigidBodyInstance* pxInstance, const bool bMakePairings, const bool bRememberObject );
		static void AddObjectToDynamicBroadphase( Asura_Physics_RigidBodyInstance* pxInstance, const bool bMakePairings, const bool bRememberObject );

		static bool DoesOBBIntersectStaticObjects( const Asura_OBB& xOBB, Asura_Handle iHandleOfObjectToIgnore );
		static bool DoesOBBIntersectDynamicObjects( const Asura_OBB& xOBB, Asura_Handle iHandleOfObjectToIgnore );
	
};


class Asura_Physics_Broadphase_Entry;

class Asura_Physics_Broadphase_Query
{
	public:

		Asura_Physics_Broadphase_Query();
		~Asura_Physics_Broadphase_Query();

		// Reset clears away any existing intersection list (i.e. frees up memory associated with this query)
		void Reset( void );

		// MakeQuery will add entries for any objects whose own broadphase BBs intersect the passed in xBroadphaseAABB parameter.
		// These entries can be queried via the GetFirstEntry() method below.
		void MakeQuery( const Asura_Bounding_Box& xBroadphaseAABB, bool bTestAgainstStatics, bool bTestAgainstExternalControl, bool bTestAgainstActors, bool bTestAgainstDynamics=false );

		// A MakeQuery version specifically tailored to Actor movement
		void MakeQuery_StandardActorMovement( const Asura_Bounding_Box& xBroadphaseAABB, bool bTestAgainstExternalControl, bool bTestAgainstActors,  bool bTestAgainstDynamics );

		// Get the first entry on the intersection list - entries are stored as a singly-linked list.
		const Asura_Physics_Broadphase_Entry* GetFirstEntry( void ) const { return m_pxFirstEntry; }

		// Add an entry; used as a callback by the spatial hash list etc
		// Users of this class shouldn't call this directly
		void AddEntry( Asura_Physics_RigidBodyInstance* pxInstance );

	private:

		Asura_Physics_Broadphase_Entry* m_pxFirstEntry;
};

class Asura_Physics_Broadphase_Entry
{
	public:

		static Asura_Physics_Broadphase_Entry* Create( Asura_Physics_RigidBodyInstance* pxInstance, Asura_Physics_Broadphase_Entry* pxNextEntry );

		static void Destroy( const Asura_Physics_Broadphase_Entry* pxEntry );

		Asura_Physics_RigidBodyInstance* GetInstance( void ) const
		{
			return m_pxInstance;
		}

		const Asura_Physics_Broadphase_Entry* GetNextEntry( void ) const
		{
			return m_pxNextEntry;
		}

	private: 

		Asura_Physics_Broadphase_Entry() {};
		~Asura_Physics_Broadphase_Entry() {};

		Asura_Physics_RigidBodyInstance*	m_pxInstance;
		Asura_Physics_Broadphase_Entry*		m_pxNextEntry;
};


#endif//__ASURA_PHYSICS_BROADPHASE_H__
