//
// Asura_Physics_CCD.h
//
// Created by KJFL - 17 November 2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __ASURA_PHYSICS_CCD_H__
#define __ASURA_PHYSICS_CCD_H__

#include "Core/Asura_Maths.h"

class Asura_Physics_RigidBodyInstance;
class Asura_Physics_Tri;
class Asura_Physics_MPR_Body;

class Asura_Physics_Ragdoll_CollisionObject;

class Asura_Physics_CCD
{
	public:
		static u_int FindContactPointsFromSupportFeatures( Asura_AVector3* pxContactPointsForA, Asura_AVector3* pxContactPointsForB, const Asura_Physics_RigidBodyInstance* pxInstanceA, const Asura_Physics_RigidBodyInstance* pxInstanceB, const Asura_AVector3& xAxis );

		// returns true if the two OBBs intersect, and if so also sets the fPenetration and xAxis parameters
		static bool FindPenetration_OBBVsOBB( const Asura_OBB& xObjectA, const Asura_OBB& xObjectB, float& fPenetration, Asura_AVector3& xAxis );

		static bool FindTimeOfIntersection_SweptOBBVsOBB( const Asura_OBB& xObjectA, const Asura_OBB& xObjectB, const Asura_AVector3& xRelativeVelocity, float& fTimeToImpact, Asura_AVector3& xAxisUsed, bool bIgnoreInitialOverlap = true );
		static float FindTimeOfIntersection_SweptOBBVsTri( const Asura_OBB& xOBB, const Asura_AVector3& xVelocity, const Asura_Physics_Tri& xTri, Asura_AVector3& xAxisUsed, u_int& uFirstPlane, bool& bHitEdge, bool bCheckVelocityVsNormal = true );

		static bool DoesOBBIntersectTri( const Asura_OBB& xOBB, const Asura_Physics_Tri& xTri, float& fPenetration, Asura_AVector3& xPenetrationAxis );
		static bool DoesOBBIntersectTri( const Asura_OBB& xOBB, const Asura_Physics_Tri& xTri );

		// penetration axis points towards OBB
		static bool DoesOBBIntersectSphere( const Asura_OBB& xOBB, const Asura_AVector3& xSphereCentre, const float fRadius, float& fPenetration, Asura_AVector3& xPenetrationAxis );

		static bool FindPenetration_SphereVsTri( const Asura_Physics_Tri& xTri, const Asura_AVector3& xSphereCentre, const float fRadius, float& fPenetration, Asura_AVector3& xPenetrationAxis, Asura_AVector3& xContactPoint );

		// get contact points for a primitive (box, sphere etc) intersecting a Tri
		static u_int FindContactPoints_PrimitiveVsTri( Asura_AVector3* pxContactPointsForPrimitive, const Asura_Physics_RigidBodyInstance* pxInstance, const Asura_Physics_Tri& xTri, const Asura_AVector3& xAxis );

		static u_int FindContactPoints_PrimitiveVsTri( Asura_AVector3* pxContactPointsForPrimitive, const Asura_Physics_MPR_Body* pxPrimitive, const Asura_Physics_Tri& xTri, const Asura_AVector3& xAxis );

		// get contact points for both a primitive (box, sphere etc) and a Tri that are intersecting
		static u_int FindContactPoints_PrimitiveVsTri( Asura_AVector3* pxContactPointsForPrimitive, Asura_AVector3* pxContactPointsForTri, const Asura_Physics_RigidBodyInstance* pxInstance, const Asura_Physics_Tri& xTri, const Asura_AVector3& xAxis );

		static void FindClosestPointOnTriangle( const Asura_AVector3& xP, const Asura_AVector3& xA, const Asura_AVector3& xB, const Asura_AVector3& xC, Asura_AVector3& xClosestPoint );

		static bool MPR_Test( const Asura_Physics_MPR_Body& xBody1, const Asura_Physics_MPR_Body& xBody2, Asura_AVector3& xAxis, float& fSeparation, bool bWarmStart = false );
		static bool Swept_MPR_Test( const Asura_Physics_MPR_Body& xBody1, const Asura_Physics_MPR_Body& xBody2, Asura_AVector3& xAxis, float& fRewindTime, float& fPenetration );

};



//////////////////////////////////////////////////////////////////////////
// NOTE: The winding on the vertices in "Asura_Physics_Tri" is 
// anti-clockwise in order to produce the correct normals...
//////////////////////////////////////////////////////////////////////////
class Asura_Physics_Tri
{
public:

	Asura_Physics_Tri(){};

	Asura_Physics_Tri( const u_int uFaceID ); // moved to the cpp to get rid of an include
	
	Asura_Physics_Tri( const Asura_AVector3& xVertex0, const Asura_AVector3& xVertex1, const Asura_AVector3& xVertex2 )
	{
		m_xV[0] = xVertex0;
		m_xV[1] = xVertex1;
		m_xV[2] = xVertex2;
	}

	inline void GetUnnormalisedNormal( Asura_AVector3& xNormal ) const
	{
		xNormal = ((m_xV[1] - m_xV[0])^(m_xV[2] - m_xV[0]));
	}

    inline void GetNormal( Asura_AVector3& xNormal ) const
    {
        xNormal = ((m_xV[1] - m_xV[0])^(m_xV[2] - m_xV[0]));
        xNormal.Normalise();
    }

	inline void GetEdge0( Asura_AVector3& xEdge ) const
	{
		xEdge = ( m_xV[1] - m_xV[0] );
	}

	inline void GetEdge1( Asura_AVector3& xEdge ) const
	{
		xEdge = ( m_xV[2] - m_xV[1] );
	}

	inline void GetEdge2( Asura_AVector3& xEdge ) const
	{
		xEdge = ( m_xV[0] - m_xV[2] );
	}

	inline Asura_AVector3 GetCentre( void ) const
	{
		return Asura_AVector3( (m_xV[0] + m_xV[1] + m_xV[2]) * 0.3333333333f );
	}

	inline Asura_AVector3* GetVertexArray( void ) 
	{
		return m_xV;
	}

	inline const Asura_AVector3* GetVertexArray( void ) const
	{
		return m_xV;
	}

	inline void Transform( const Asura_AMatrix3x3& xOri, const Asura_AVector3& xPos )
	{
		for (u_int u=0; u<3; ++u)
		{
			m_xV[u] = m_xV[u] * xOri + xPos;
		}
	}


	inline void GetCentreAndHalfExtentAlongAxis( const Asura_AVector3& xAxis, float& fCentre, float& fHalfExtent ) const
	{
		const float fDot0 = m_xV[0] * xAxis;
		const float fDot1 = m_xV[1] * xAxis;
		const float fDot2 = m_xV[2] * xAxis;

		float fMin, fMax;

		if ( fDot0 > fDot1 )
		{
			fMin = fDot1;
			fMax = fDot0;
		}
		else
		{
			fMin = fDot0;
			fMax = fDot1;
		}

		fMin = Asura_Maths::Min( fMin,  fDot2 );
		fMax = Asura_Maths::Max( fMax,  fDot2 );

		fCentre = (fMax + fMin )*0.5f;
		fHalfExtent = (fMax - fCentre);
	}

	u_int GetSupportFeature( Asura_AVector3* pxSupportPoints, const Asura_AVector3& xAxis ) const;

	Asura_AVector3 GetSupportPoint( const Asura_AVector3& xAxis ) const;


private:

	Asura_AVector3 m_xV[3];
	

};



static const u_int uASURA_MAX_NUMBER_OF_SPHERES_IN_SPHEREGROUP = 4;
class Asura_Physics_EnvironmentCache;

class Asura_Physics_CollisionData
{
public:
	Asura_AVector3	m_xNormal;
	u_int			m_uPolyFlags;
	u_int			m_uPolyID;
	bool			m_bIsEdge;

};
class Asura_Physics_SphereGroup
{
	public:

		Asura_Physics_SphereGroup( const Asura_Physics_RigidBodyInstance& xInstance );
	
		Asura_Physics_SphereGroup( const Asura_OBB& xOBB, bool bSphereStack );

		u_int GetNumberOfSpheres( void ) const;

		const Asura_AVector3* GetSphereCentres( void ) const;

		float GetSphereRadius( void ) const;

		void Displace( const Asura_AVector3& xDisplacement );

		float SweptSpheresVsEnvironment( const Asura_AVector3& xDirection, const float fDistance, const Asura_Physics_EnvironmentCache& xEnvCache,
											Asura_AVector3& xLastCollisionNormal, u_int& uLastCollisionFaceID, bool& bHitEdge  );

		float SweptSpheresVsEnvironment( const Asura_AVector3& xDirection, const float fDistance, const Asura_Physics_EnvironmentCache& xEnvCache,
			Asura_Physics_CollisionData* pxCollisionData, u_int& uNumberOfCollisions, u_int uMaxNumberOfCollisionsToRecord  );
	private:


		u_int			m_uNumberOfSpheres;
		float			m_fRadius;
		Asura_AVector3	m_axSphereCentres[uASURA_MAX_NUMBER_OF_SPHERES_IN_SPHEREGROUP];

};

class Asura_Physics_MPR_Body
{
	public:

		Asura_Physics_MPR_Body( const Asura_Physics_RigidBodyInstance* pxBody );
		Asura_Physics_MPR_Body( const Asura_Physics_RigidBodyInstance* pxBody, const Asura_AVector3& xStartPosition, const Asura_AVector3& xEndPosition );
		Asura_Physics_MPR_Body( const Asura_Physics_Tri* pxTri );
		Asura_Physics_MPR_Body( const Asura_Physics_Ragdoll_CollisionObject* pxObject );


		Asura_AVector3 GetSupportPoint( const Asura_AVector3& xAxis, bool bPreviousPosition = false ) const;
		
		Asura_AVector3 GetSweptSupportPoint( const Asura_AVector3& xAxis, const Asura_AVector3& xSweep ) const;

		Asura_AVector3 GetCentre( void ) const;
		Asura_AVector3 GetPreviousCentre( void ) const;

	private:
		
		Asura_AVector3 m_xStart;
		Asura_AVector3 m_xEnd;

		const Asura_Physics_RigidBodyInstance* m_pxBody;
		const Asura_Physics_Tri* m_pxTri;
		const Asura_Physics_Ragdoll_CollisionObject* m_pxRagdollObject;


};

#endif//__ASURA_PHYSICS_CCD_H__
