
// Asura_Physics_CollisionMesh.cpp
//
// Created by KJFL - 16 February 2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

// This file's header.
#include "Physics/Asura_Physics_CollisionMesh.h"

#include "Mesh/Asura_CollisionMesh.h"
#include "Physics/Asura_Physics_CCD.h"

Asura_Physics_CollisionMesh_Iterator::Asura_Physics_CollisionMesh_Iterator( const Asura_CollisionMesh* pxCollisionMesh )
{
	Asura_Assert( pxCollisionMesh, "Code bug: collision mesh ptr invalid" );

	m_pxCollisionMesh = pxCollisionMesh;
}

void Asura_Physics_CollisionMesh_Iterator::AccessFirstTri( void )
{
	m_uCurrentPoly = 0;
	m_bCurrentTriIsSecondInQuad = false;
}
void Asura_Physics_CollisionMesh_Iterator::AccessNextTri( void )
{
	if (m_uCurrentPoly >= m_pxCollisionMesh->GetNumberOfPolys())
	{
		// Either we've overrun because this has been incorrectly called,
		// or the collision mesh has zero polys
		Asura_Assert( false, "Code bug: past end of col mesh" );
		return;
	}

	const bool bQuad = m_pxCollisionMesh->IsQuad( m_uCurrentPoly );

	if (bQuad && !m_bCurrentTriIsSecondInQuad)
	{
		m_bCurrentTriIsSecondInQuad = true;
	}
	else
	{
		++m_uCurrentPoly;
		m_bCurrentTriIsSecondInQuad = false;
	}
}

bool Asura_Physics_CollisionMesh_Iterator::GetCurrentTri( Asura_Physics_Tri& xTri ) const
{
	if (m_uCurrentPoly >= m_pxCollisionMesh->GetNumberOfPolys()) return false;

	m_pxCollisionMesh->GetVertices( xTri.GetVertexArray(), m_uCurrentPoly, !m_bCurrentTriIsSecondInQuad );

	return true;
}


u_short Asura_Physics_CollisionMesh_Iterator::GetFlagsOfCurrentTri( void ) const
{
	if (m_uCurrentPoly >= m_pxCollisionMesh->GetNumberOfPolys()) return 0;

	// KJFL - if we get a type conversion here warning, we should change the old code in collision mesh 
	// to return u_shorts for 16bit flag fields rather than shorts. ;)
	return m_pxCollisionMesh->GetFlags( m_uCurrentPoly );
}


////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Physics_CollisionOBB_Iterator::Asura_Physics_CollisionOBB_Iterator
// Author       : Andy Tate
// Date         : 8-Aug-2007
////////////////////////////////////////////////////////////////////////////////

Asura_Physics_CollisionOBB_Iterator::Asura_Physics_CollisionOBB_Iterator(const Asura_OBB& xOBB)
: m_uCurrentPoly(static_cast<u_int>(-1))
, m_xOBB(xOBB)
{
	// Nothing more to initialise...
}

/// End of function 'Asura_Physics_CollisionOBB_Iterator::Asura_Physics_CollisionOBB_Iterator'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Physics_CollisionOBB_Iterator::AccessFirstTri
// Author       : Andy Tate
// Date         : 8-Aug-2007
////////////////////////////////////////////////////////////////////////////////

void Asura_Physics_CollisionOBB_Iterator::AccessFirstTri(void)
{
	const Asura_AVector3&	xCentre			= m_xOBB.GetCentre();
	const Asura_AMatrix3x3&	xOrientation	= m_xOBB.GetOrientation();
	const Asura_AVector3	xOffsetX		= xOrientation.GetColumn0() * m_xOBB.GetHalfExtentX();
	const Asura_AVector3	xOffsetY		= xOrientation.GetColumn1() * m_xOBB.GetHalfExtentY();
	const Asura_AVector3	xOffsetZ		= xOrientation.GetColumn2() * m_xOBB.GetHalfExtentZ();

	// Reset the poly-count...
	m_uCurrentPoly	= 0;

	// Initialise the vertices...
	m_xLTF	= xCentre - xOffsetX - xOffsetY - xOffsetZ;
	m_xLTB	= xCentre - xOffsetX - xOffsetY + xOffsetZ;
	m_xLBB	= xCentre - xOffsetX + xOffsetY + xOffsetZ;
	m_xLBF	= xCentre - xOffsetX + xOffsetY - xOffsetZ;

	m_xRTF	= xCentre + xOffsetX - xOffsetY - xOffsetZ;
	m_xRTB	= xCentre + xOffsetX - xOffsetY + xOffsetZ;
	m_xRBB	= xCentre + xOffsetX + xOffsetY + xOffsetZ;
	m_xRBF	= xCentre + xOffsetX + xOffsetY - xOffsetZ;
}

/// End of function 'Asura_Physics_CollisionOBB_Iterator::AccessFirstTri'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Physics_CollisionOBB_Iterator::GetCurrentTri
// Author       : Andy Tate
// Date         : 8-Aug-2007
////////////////////////////////////////////////////////////////////////////////

bool Asura_Physics_CollisionOBB_Iterator::GetCurrentTri(Asura_Physics_Tri& xTri) const
{
	switch(m_uCurrentPoly)
	{
		// Front-face...
		case FRONT_TRIANGLE_1:
		{
			xTri	= Asura_Physics_Tri(m_xLTF, m_xRBF, m_xRTF);
			return(true);
		}

		case FRONT_TRIANGLE_2:
		{
			xTri	= Asura_Physics_Tri(m_xLTF, m_xLBF, m_xRBF);
			return(true);
		}

		// Back-face...
		case BACK_TRIANGLE_1:
		{
			xTri	= Asura_Physics_Tri(m_xLTB, m_xRTB, m_xRBB);
			return(true);
		}

		case BACK_TRIANGLE_2:
		{
			xTri	= Asura_Physics_Tri(m_xLTB, m_xRBB, m_xLBB);
			return(true);
		}

		// Left-face...
		case LEFT_TRIANGLE_1:
		{
			xTri	= Asura_Physics_Tri(m_xLTB, m_xLBF, m_xLTF);
			return(true);
		}

		case LEFT_TRIANGLE_2:
		{
			xTri	= Asura_Physics_Tri(m_xLTB, m_xLBB, m_xLBF);
			return(true);
		}

		// Right-face...
		case RIGHT_TRIANGLE_1:
		{
			xTri	= Asura_Physics_Tri(m_xRTB, m_xRTF, m_xRBF);
			return(true);
		}

		case RIGHT_TRIANGLE_2:
		{
			xTri	= Asura_Physics_Tri(m_xRTB, m_xRBF, m_xRBB);
			return(true);
		}

		// Top-face...
		case TOP_TRIANGLE_1:
		{
			xTri	= Asura_Physics_Tri(m_xLTB, m_xRTF, m_xRTB);
			return(true);
		}

		case TOP_TRIANGLE_2:
		{
			xTri	= Asura_Physics_Tri(m_xLTB, m_xLTF, m_xRTF);
			return(true);
		}

		// Bottom-face...
		case BOTTOM_TRIANGLE_1:
		{
			xTri	= Asura_Physics_Tri(m_xLBB, m_xRBB, m_xRBF);
			return(true);
		}

		case BOTTOM_TRIANGLE_2:
		{
			xTri	= Asura_Physics_Tri(m_xLBB, m_xRBF, m_xLBF);
			return(true);
		}

		// We've reached the end of our list...
		case MAX_TRIANGLES:
		{
			return(false);
		}

		// Shouldn't get here...
		default:
		{
			Asura_Assert(false, "Code bug: Past end of collision bounding-box.");
			return(false);
		}
	}		// End of switch.
}

/// End of function 'Asura_Physics_CollisionOBB_Iterator::GetCurrentTri'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Physics_CollisionOBB_Iterator::AccessNextTri
// Author       : Andy Tate
// Date         : 8-Aug-2007
////////////////////////////////////////////////////////////////////////////////

void Asura_Physics_CollisionOBB_Iterator::AccessNextTri(void)
{
	if(m_uCurrentPoly >= MAX_TRIANGLES)
	{
		Asura_Assert(false, "Code bug: Past end of collision bounding-box.");
	}
	else
	{
		m_uCurrentPoly++;
	}
}

/// End of function 'Asura_Physics_CollisionOBB_Iterator::AccessNextTri'. ///


/// End of file. ///
