//
// Asura_Physics_CollisionMesh.h
//
// Created by KJFL - 16 February 2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __ASURA_PHYSICS_COLLISIONMESH_H__
#define __ASURA_PHYSICS_COLLISIONMESH_H__

#include "Core/Asura_Maths.h"

class Asura_Collision_Object;
class Asura_CollisionMesh;
class Asura_Physics_Tri;


// The main problem we have with Asura_CollisionMesh is that it uses quads, and the Physics system
// uses tris. So, we're going to wrap the Asura_CollisionMesh away and provide methods to transparently
// iterate over the tris - we can refactor under this level and not need to change the Physics code.	
// Made the iterator members non-static since we may at some point want to iterate over multiple
// meshes simultaneously.
class Asura_Physics_CollisionMesh_Iterator
{
	public:

		Asura_Physics_CollisionMesh_Iterator( const Asura_CollisionMesh* pxCollisionMesh );
		~Asura_Physics_CollisionMesh_Iterator()	{}
	
		void AccessFirstTri( void );

		bool GetCurrentTri( Asura_Physics_Tri& xTri ) const;

		u_short GetFlagsOfCurrentTri( void ) const;

		void AccessNextTri( void );

	private:

		u_int	m_uCurrentPoly;
		bool	m_bCurrentTriIsSecondInQuad;

		const Asura_CollisionMesh* m_pxCollisionMesh;
};


class Asura_Physics_CollisionOBB_Iterator
{
public:
	// Public functions...
	Asura_Physics_CollisionOBB_Iterator(const Asura_OBB& xOBB);
	~Asura_Physics_CollisionOBB_Iterator()	{}

	// Iteration functions...
	void AccessFirstTri(void);
	bool GetCurrentTri(Asura_Physics_Tri& xTri) const;
	void AccessNextTri(void);


private:
	// Private enumerated types...
	enum Triangles
	{
		FRONT_TRIANGLE_1,
		FRONT_TRIANGLE_2,
		BACK_TRIANGLE_1,
		BACK_TRIANGLE_2,
		LEFT_TRIANGLE_1,
		LEFT_TRIANGLE_2,
		RIGHT_TRIANGLE_1,
		RIGHT_TRIANGLE_2,
		TOP_TRIANGLE_1,
		TOP_TRIANGLE_2,
		BOTTOM_TRIANGLE_1,
		BOTTOM_TRIANGLE_2,

		MAX_TRIANGLES
	};
	// Private variables...
	u_int				m_uCurrentPoly;	// Index of the current poly being iterated through.
	const Asura_OBB&	m_xOBB;			// A reference to the OBB to use.

	Asura_AVector3		m_xLTF;			// Left-top-front vertex.
	Asura_AVector3		m_xRTF;			// Right-top-front vertex.
	Asura_AVector3		m_xRTB;			// Right-top-back vertex.
	Asura_AVector3		m_xLTB;			// Left-top-back vertex.

	Asura_AVector3		m_xLBF;			// Left-bottom-front vertex.
	Asura_AVector3		m_xRBF;			// Right-bottom-front vertex.
	Asura_AVector3		m_xRBB;			// Right-bottom-back vertex.
	Asura_AVector3		m_xLBB;			// Left-bottom-back vertex.
};

#endif//__ASURA_PHYSICS_COLLISIONMESH_H__


/// End of file. ///
