//
// Asura_Physics_Contact.cpp
//
// Created by KJFL - 2004-03-23 10:57:08
//
// Copyright (C) 2004 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

// This file's header.
#include "Physics/Asura_Physics_Contact.h"

#include "Core/Asura_Mempool.h"

// pointer initialised to zero to flag as invalid
Asura_ResizingMempool*	Asura_Physics_Contact::s_pxContactPool = 0;

void Asura_Physics_Contact::Initialise( void )
{
	// allocate a resizing mempool that can contain all Contacts. 

	Asura_Assert( s_pxContactPool==0, "Already been initialised?" );

	const u_int uNumberOfContactsPerPool = 8;
	s_pxContactPool = new Asura_ResizingMempool( uNumberOfContactsPerPool, sizeof( Asura_Physics_Contact ), __FILE__, __LINE__ );
}

void Asura_Physics_Contact::Reset( void )
{
	// just free up the elements in the pool, not free up all the memory
	Asura_Assert(s_pxContactPool, "Reset called when not Initialised - crash imminent.");
	s_pxContactPool->Reset();
}

void Asura_Physics_Contact::Shutdown( void )
{
	// delete the entire mempool and set the pointer to 0 so we know it's no longer valid
	delete s_pxContactPool;
	s_pxContactPool = 0;
}


Asura_Physics_Contact* Asura_Physics_Contact::Create( void )
{
	if (!s_pxContactPool)
	{
		Asura_Assert( s_pxContactPool, "Mempool not been initialised?" );
		return 0;
	}

	Asura_Physics_Contact* pxContact = static_cast<Asura_Physics_Contact*>(s_pxContactPool->Allocate());

	if (!pxContact)
	{
		Asura_Assert( pxContact, "Out of memory?" );
		return 0;
	}

	pxContact->m_bBroken = false;
	pxContact->m_pxNext = 0;
	pxContact->m_pxPrev = 0;
/*
	// Do some basic initialisation - set the type to be 'broken' (so if the caller doesn't do anything
	// with it, it won't seem like a valid Contact).
	pxContact->m_eContactType = CONTACT_BROKEN;

	// Did the caller supply a list to add the contact to?
	if (ppxContactList)
	{
		// set the new Contact's next pointer to the current head of the list
		pxContact->m_pxNextContact = *ppxContactList;

		// and make the new contact the head
		*ppxContactList = pxContact;
	}
*/
	return pxContact;
}

void Asura_Physics_Contact::Destroy( Asura_Physics_Contact* pxContact )
{
	if (!s_pxContactPool)
	{
		Asura_Assert( s_pxContactPool, "Mempool not been initialised?" );
		return;
	}
	if (!pxContact)
	{
		Asura_Assert( pxContact, "null Contact pointer" );
		return;
	}

	s_pxContactPool->Deallocate( pxContact );
}

/*
void Asura_Physics_Contact::RemoveBrokenContactsFromList( Asura_Physics_Contact** ppxContactList )
{
	Asura_Physics_Contact** ppxNextContactPointer = ppxContactList;

	while (*ppxNextContactPointer) // is there a 'next' Contact?
	{
		// let's get it
		Asura_Physics_Contact* pxContact = *ppxNextContactPointer;

		// is it broken?
		if (!pxContact->IsContactValid())
		{
			// make the previous 'next' point to this Contact's 'next'
			*ppxNextContactPointer = pxContact->m_pxNextContact;

			// destroy the broken Contact
			Asura_Physics_Contact::Destroy( pxContact );
		}
		else
		{
			// move onto next Contact in list
			ppxNextContactPointer = &(pxContact->m_pxNextContact);
		}
	}
}
*/
