//
// Asura_Physics_Environment.cpp
//
// Created by KJFL - 2004-03-24 16:44:32
//
// Copyright (C) 2004 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

// This file's header.
#include "Physics/Asura_Physics_Environment.h"
#include "Physics/Asura_Physics_EnvironmentCache.h"

#include "Core/Asura_AABB_Tree.h"
#include "Core/Asura_LOS.h"
#include "Core/Memory/Asura_ResourceAllocator.h"
#include "Debugging/Asura_Debug_Timers.h"
#include "Rendering/Asura_Render.h"
#include "Physics/Asura_Physics_RigidBody.h"
#if defined(ASURA_USE_STREAMING)
	#include "Physics/Asura_Physics_Ragdoll.h"
	#include "Streaming/Asura_Streaming_HandlerReplacement.h"
#endif //defined(ASURA_USE_STREAMING)

#ifdef ASURA_TOOLS
	#include "Load&Save/LevelExport/Tools_Level_Export.h"
	#include "Misc/Tools_ProgressIndicator.h"
#endif


////////////////////////////////////////////////////////////
//  
////////////////////////////////////////////////////////////

/********************************************************/
/* AABB-triangle overlap test code                      */
/* by Tomas Akenine-Mller                              */
/* Function: int triBoxOverlap(float boxcenter[3],      */
/*          float boxhalfsize[3],float triverts[3][3]); */
/* History:                                             */
/*   2001-03-05: released the code in its first version */
/*   2001-06-18: changed the order of the tests, faster */
/*                                                      */
/* Acknowledgement: Many thanks to Pierre Terdiman for  */
/* suggestions and discussions on how to optimize code. */
/* Thanks to David Hunt for finding a ">="-bug!         */
/********************************************************/
#include <stdio.h>

#define X 0
#define Y 1
#define Z 2

#define CROSS(dest,v1,v2) \
          dest[0]=v1[1]*v2[2]-v1[2]*v2[1]; \
          dest[1]=v1[2]*v2[0]-v1[0]*v2[2]; \
          dest[2]=v1[0]*v2[1]-v1[1]*v2[0]; 

#define DOT(v1,v2) (v1[0]*v2[0]+v1[1]*v2[1]+v1[2]*v2[2])

#define SUB(dest,v1,v2) \
          dest[0]=v1[0]-v2[0]; \
          dest[1]=v1[1]-v2[1]; \
          dest[2]=v1[2]-v2[2]; 

#define FINDMINMAX(x0,x1,x2,min,max) \
  min = max = x0;   \
  if(x1<min) min=x1;\
  if(x1>max) max=x1;\
  if(x2<min) min=x2;\
  if(x2>max) max=x2;

int planeBoxOverlap(float normal[3], float vert[3], float maxbox[3])	// -NJMP-
{
  int q;
  float vmin[3],vmax[3],v;
  for(q=X;q<=Z;q++)
  {
    v=vert[q];					// -NJMP-
    if(normal[q]>0.0f)
    {
      vmin[q]=-maxbox[q] - v;	// -NJMP-
      vmax[q]= maxbox[q] - v;	// -NJMP-
    }
    else
    {
      vmin[q]= maxbox[q] - v;	// -NJMP-
      vmax[q]=-maxbox[q] - v;	// -NJMP-
    }
  }
  if(DOT(normal,vmin)>0.0f) return 0;	// -NJMP-
  if(DOT(normal,vmax)>=0.0f) return 1;	// -NJMP-
  
  return 0;
}


/*======================== X-tests ========================*/
#define AXISTEST_X01(a, b, fa, fb)			   \
	p0 = a*v0[Y] - b*v0[Z];			       	   \
	p2 = a*v2[Y] - b*v2[Z];			       	   \
        if(p0<p2) {min=p0; max=p2;} else {min=p2; max=p0;} \
	rad = fa * boxhalfsize[Y] + fb * boxhalfsize[Z];   \
	if(min>rad || max<-rad) return 0;

#define AXISTEST_X2(a, b, fa, fb)			   \
	p0 = a*v0[Y] - b*v0[Z];			           \
	p1 = a*v1[Y] - b*v1[Z];			       	   \
        if(p0<p1) {min=p0; max=p1;} else {min=p1; max=p0;} \
	rad = fa * boxhalfsize[Y] + fb * boxhalfsize[Z];   \
	if(min>rad || max<-rad) return 0;

/*======================== Y-tests ========================*/
#define AXISTEST_Y02(a, b, fa, fb)			   \
	p0 = -a*v0[X] + b*v0[Z];		      	   \
	p2 = -a*v2[X] + b*v2[Z];	       	       	   \
        if(p0<p2) {min=p0; max=p2;} else {min=p2; max=p0;} \
	rad = fa * boxhalfsize[X] + fb * boxhalfsize[Z];   \
	if(min>rad || max<-rad) return 0;

#define AXISTEST_Y1(a, b, fa, fb)			   \
	p0 = -a*v0[X] + b*v0[Z];		      	   \
	p1 = -a*v1[X] + b*v1[Z];	     	       	   \
        if(p0<p1) {min=p0; max=p1;} else {min=p1; max=p0;} \
	rad = fa * boxhalfsize[X] + fb * boxhalfsize[Z];   \
	if(min>rad || max<-rad) return 0;

/*======================== Z-tests ========================*/

#define AXISTEST_Z12(a, b, fa, fb)			   \
	p1 = a*v1[X] - b*v1[Y];			           \
	p2 = a*v2[X] - b*v2[Y];			       	   \
        if(p2<p1) {min=p2; max=p1;} else {min=p1; max=p2;} \
	rad = fa * boxhalfsize[X] + fb * boxhalfsize[Y];   \
	if(min>rad || max<-rad) return 0;

#define AXISTEST_Z0(a, b, fa, fb)			   \
	p0 = a*v0[X] - b*v0[Y];				   \
	p1 = a*v1[X] - b*v1[Y];			           \
        if(p0<p1) {min=p0; max=p1;} else {min=p1; max=p0;} \
	rad = fa * boxhalfsize[X] + fb * boxhalfsize[Y];   \
	if(min>rad || max<-rad) return 0;


int triBoxOverlap(float boxcenter[3],float boxhalfsize[3],float triverts[3][3])

//int triBoxOverlap(float* boxcenter, float *boxhalfsize, float triverts[3][3])
{

  /*    use separating axis theorem to test overlap between triangle and box */
  /*    need to test for overlap in these directions: */
  /*    1) the {x,y,z}-directions (actually, since we use the AABB of the triangle */
  /*       we do not even need to test these) */
  /*    2) normal of the triangle */
  /*    3) crossproduct(edge from tri, {x,y,z}-directin) */
  /*       this gives 3x3=9 more tests */
   float v0[3],v1[3],v2[3];
//   float axis[3];
   float min,max,p0,p1,p2,rad,fex,fey,fez;		// -NJMP- "d" local variable removed
   float normal[3],e0[3],e1[3],e2[3];

   /* This is the fastest branch on Sun */
   /* move everything so that the boxcenter is in (0,0,0) */
   SUB(v0,triverts[0],boxcenter);
   SUB(v1,triverts[1],boxcenter);
   SUB(v2,triverts[2],boxcenter);

   /* compute triangle edges */
   SUB(e0,v1,v0);      /* tri edge 0 */
   SUB(e1,v2,v1);      /* tri edge 1 */
   SUB(e2,v0,v2);      /* tri edge 2 */

   /* Bullet 3:  */
   /*  test the 9 tests first (this was faster) */
   fex = Asura_Maths::Abs(e0[X]);
   fey = Asura_Maths::Abs(e0[Y]);
   fez = Asura_Maths::Abs(e0[Z]);
   AXISTEST_X01(e0[Z], e0[Y], fez, fey);
   AXISTEST_Y02(e0[Z], e0[X], fez, fex);
   AXISTEST_Z12(e0[Y], e0[X], fey, fex);

   fex = Asura_Maths::Abs(e1[X]);
   fey = Asura_Maths::Abs(e1[Y]);
   fez = Asura_Maths::Abs(e1[Z]);
   AXISTEST_X01(e1[Z], e1[Y], fez, fey);
   AXISTEST_Y02(e1[Z], e1[X], fez, fex);
   AXISTEST_Z0(e1[Y], e1[X], fey, fex);

   fex = Asura_Maths::Abs(e2[X]);
   fey = Asura_Maths::Abs(e2[Y]);
   fez = Asura_Maths::Abs(e2[Z]);
   AXISTEST_X2(e2[Z], e2[Y], fez, fey);
   AXISTEST_Y1(e2[Z], e2[X], fez, fex);
   AXISTEST_Z12(e2[Y], e2[X], fey, fex);

   /* Bullet 1: */
   /*  first test overlap in the {x,y,z}-directions */
   /*  find min, max of the triangle each direction, and test for overlap in */
   /*  that direction -- this is equivalent to testing a minimal AABB around */
   /*  the triangle against the AABB */

   /* test in X-direction */
   FINDMINMAX(v0[X],v1[X],v2[X],min,max);
   if(min>boxhalfsize[X] || max<-boxhalfsize[X]) return 0;

   /* test in Y-direction */
   FINDMINMAX(v0[Y],v1[Y],v2[Y],min,max);
   if(min>boxhalfsize[Y] || max<-boxhalfsize[Y]) return 0;

   /* test in Z-direction */
   FINDMINMAX(v0[Z],v1[Z],v2[Z],min,max);
   if(min>boxhalfsize[Z] || max<-boxhalfsize[Z]) return 0;

   /* Bullet 2: */
   /*  test if the box intersects the plane of the triangle */
   /*  compute plane equation of triangle: normal*x+d=0 */
   CROSS(normal,e0,e1);
   // -NJMP- (line removed here)
   if(!planeBoxOverlap(normal,v0,boxhalfsize)) return 0;	// -NJMP-

   return 1;   /* box and triangle overlaps */
}
#undef X
#undef Y
#undef Z

////////////////////////////////////////////////////////////
//
// Asura_Physics_Zone_Map
//
////////////////////////////////////////////////////////////
#if defined(ASURA_USE_STREAMING)
// Class to map between zone indices and pointers to Asura_Physics_Zone.
// Zone indices are assigned as zones are loaded and released as zones are unloaded.
class Asura_Physics_Zone_Map
{
public:
	Asura_Physics_Zone_Map() : m_uLastSlotUsed(0) { Reset(); }

	void Reset()
	{
		for (u_int u=0; u<uASURA_MAX_NUM_ZONES_LOADED; ++u)
		{
			m_apxZoneMap[u] = 0;
		}
		m_uLastSlotUsed = 0;
	}

	// Add the zone to the array. Return the index used.
	u_int AddZone(Asura_Physics_Zone* pxZone)
	{
		// Search for a free slot. Usually the next one will be free.
		for (u_int u=0; u<uASURA_MAX_NUM_ZONES_LOADED; ++u)
		{
			const u_int uCheckSlot = ((m_uLastSlotUsed + 1 + u) % uASURA_MAX_NUM_ZONES_LOADED);
			if (m_apxZoneMap[uCheckSlot] == 0)
			{
				// Found a free slot.
				m_apxZoneMap[uCheckSlot] = pxZone;
				m_uLastSlotUsed = uCheckSlot;
				return m_uLastSlotUsed;
			}
		}
		// Bad things are going to happen...
		Asura_Assert(0, "Error: loaded too many zones!");
		return 0;
	}

	// Remove the specified zone from the array.
	void RemoveZone(const Asura_Physics_Zone* pxZone, const u_int uZoneIndex)
	{
		if (uZoneIndex<uASURA_MAX_NUM_ZONES_LOADED)
		{
			if (m_apxZoneMap[uZoneIndex] == pxZone)
			{
				// Clear the slot.
				m_apxZoneMap[uZoneIndex] = 0;
				return;
			}
		}

		Asura_Assert(0, "Error: couldn't remove zone!");
	}

	Asura_Physics_Zone* GetZone(const u_int uZoneIndex)
	{
		if (uZoneIndex<uASURA_MAX_NUM_ZONES_LOADED)
		{
			return m_apxZoneMap[uZoneIndex];
		}
		Asura_Assert(0, "Invalid zone index!");
		return 0;
	}


private:
	Asura_Physics_Zone* m_apxZoneMap[uASURA_MAX_NUM_ZONES_LOADED];
	u_int m_uLastSlotUsed;
};

static Asura_Physics_Zone_Map g_xZoneMap;
#endif //defined(ASURA_USE_STREAMING)


////////////////////////////////////////////////////////////
//
//	Asura_Physics_Zone
//
////////////////////////////////////////////////////////////

u_int							Asura_Physics_Zone::s_uCurrentAABBSector	= 0;
Asura_Physics_EnvironmentCache*	Asura_Physics_Zone::s_pxCurrentCache		= NULL;
Asura_Physics_Zone*				Asura_Physics_Zone::s_pxCurrentZone			= NULL;

Asura_Physics_Zone::Asura_Physics_Zone( void )
: m_pxSectors(NULL)
, m_uNumberOfSectors(0)

, m_uTotalNumberOfFaces(0)
, m_pxFaces(NULL)
, m_pxFaceFlags(NULL)

, m_uTotalNumberOfVertices(0)
, m_pxVertices(NULL)

, m_uTotalNumberOfPolygonFlags(0)
, m_pusPolygonFlags(NULL)
#ifdef ASURA_TOOLS
, m_puToolsPolygonFlags( NULL )
#endif

, m_uTotalNumberOfMaterialIDs(0)

#ifdef ASURA_USE_MATERIAL_RESPONSE_SYSTEM
, m_puMaterialResponseHashIDs(NULL)
#endif
, m_psMaterialIDs(NULL)

, m_bTreesUseOneHugeNodeArray(false)
, m_pxOneHugeNodeArray(NULL)
, m_uZoneIndex(0)

#ifdef ASURA_TOOLS
, m_bInvalid( true )
#endif

#ifdef TOOLS_INCLUDE_PHYSICS_MODULE_AND_POLY_LOOKUP_DATA
, m_puSectorLookupArray( NULL )
, m_ppuPolyLookupArrays( NULL )
#endif
{
#if defined(ASURA_USE_STREAMING)
	m_uZoneIndex = g_xZoneMap.AddZone(this);
#endif //defined(ASURA_USE_STREAMING)
}

Asura_Physics_Zone::~Asura_Physics_Zone()
{
	Reset();
#if defined(ASURA_USE_STREAMING)
	g_xZoneMap.RemoveZone(this, m_uZoneIndex);

	// MGH 10/4/2006: Perform cleanup to avoid having dangling indices that refer to
	// the deleted zone. Any such index could cause a crash if we were to use it to
	// perform a lookup of a face/vertex/edge.
	Asura_Physics_RigidBodyInstance::RemoveZone(m_uZoneIndex);
	Asura_Physics_Ragdoll_System::RemoveZone(m_uZoneIndex);

	// Clean up all of the environment caches. All contacts should already have been cleaned up.
	Asura_Physics_EnvironmentCache::RemoveZone(m_uZoneIndex);
#endif //defined(ASURA_USE_STREAMING)
}

void Asura_Physics_Zone::Reset( void )
{
	Asura_ResourcePushType(ASURA_RESOURCE_COLLISION_MESH);
	{
		if (m_bTreesUseOneHugeNodeArray)
		{
			Asura_ResourceDeallocateArray(m_pxOneHugeNodeArray);
		}
		else if (m_pxSectors)
		{
			// damn, we have to deallocate everything one by one
			for (u_int u=0; u<m_uNumberOfSectors; u++)
			{
				m_pxSectors[u].m_xFaceTree.DeleteNodes();
			}

			// don't forget the tree of sectors
			m_xSectorsTree.DeleteNodes();
		}

#ifdef TOOLS_INCLUDE_PHYSICS_MODULE_AND_POLY_LOOKUP_DATA
		if ( m_ppuPolyLookupArrays )
		{
			for ( u_int u = 0; u < m_uNumberOfSectors; ++u )
			{
				delete [] m_ppuPolyLookupArrays[ u ];
			}
		}
		delete [] m_ppuPolyLookupArrays;
		delete [] m_puSectorLookupArray;
#endif

        m_uNumberOfSectors = 0;
		Asura_ResourceDeallocateArray(m_pxSectors);

        m_uTotalNumberOfVertices = 0;
		Asura_ResourceDeallocateArray(m_pxVertices);

        m_uTotalNumberOfFaces = 0;
		Asura_ResourceDeallocateArray(m_pxFaces);
		Asura_ResourceDeallocateArray(m_pxFaceFlags);

        m_uTotalNumberOfPolygonFlags = 0;
		Asura_ResourceDeallocateArray(m_pusPolygonFlags);
#ifdef ASURA_TOOLS
        Asura_ResourceDeallocateArray( m_puToolsPolygonFlags );
#endif

        m_uTotalNumberOfMaterialIDs = 0;
#ifdef ASURA_USE_MATERIAL_RESPONSE_SYSTEM
		Asura_ResourceDeallocateArray(m_puMaterialResponseHashIDs);
#endif
		Asura_ResourceDeallocateArray(m_psMaterialIDs);
	}
	Asura_ResourcePopType();
}

void Asura_Physics_Zone::FillCache( class Asura_Physics_EnvironmentCache* pxCache )
{
#ifdef ASURA_TOOLS
	if (m_uNumberOfSectors == 0 || m_bInvalid)
	{
		Tools_Create();
	}
#endif

	if(m_uNumberOfSectors > 0)
	{
		s_pxCurrentCache = pxCache;
		s_pxCurrentZone = this;

		m_xSectorsTree.TestTreeWithBB( pxCache->GetCacheBoundingBox(), FillCache_SectorCallback, 0 );
	}
}

void Asura_Physics_Zone::FillCache_SectorCallback( const u_int uIndex )
{
	s_uCurrentAABBSector = uIndex;

	const Asura_Physics_Environment_Sector& xSector = s_pxCurrentZone->m_pxSectors[s_uCurrentAABBSector];

	if (xSector.m_bAllFacesUseSamePolygonFlag)
	{
#ifdef ASURA_TOOLS
        if ( s_pxCurrentZone->Tools_GetFlagsForPolygonInSector( xSector, 0 ) & s_pxCurrentCache->Tools_GetFlagsOfPolysToIgnore() ) return;
#else
		if ( s_pxCurrentZone->GetFlagsForPolygonInSector( xSector, 0 ) & s_pxCurrentCache->GetFlagsOfPolysToIgnore() ) return;
#endif
    }

	xSector.m_xFaceTree.TestTreeWithBB( s_pxCurrentCache->GetCacheBoundingBox(), FaceSearchCallback, 1 );
}

void Asura_Physics_Zone::FaceSearchCallback( const u_int uIndex )
{
	// get the sector this face is part of
	const Asura_Physics_Environment_Sector& xSector = s_pxCurrentZone->m_pxSectors[s_uCurrentAABBSector];

	Asura_Assert( xSector.m_uNumberOfFaces > uIndex, "invalid index" );

#ifdef ASURA_TOOLS
    const u_int uPolyFlags = s_pxCurrentZone->Tools_GetFlagsForPolygonInSector( xSector, uIndex );
    if (uPolyFlags & s_pxCurrentCache->Tools_GetFlagsOfPolysToIgnore()) return;
#else
    const u_short usPolyFlags = s_pxCurrentZone->GetFlagsForPolygonInSector( xSector, uIndex );
    if (usPolyFlags & s_pxCurrentCache->GetFlagsOfPolysToIgnore()) return;
#endif

#if defined(ASURA_USE_STREAMING)
	// MGH 6/4/2006: Check that the sector number hasn't overflowed into the zone bits.
	Asura_Assert(((s_uCurrentAABBSector<<16) & uASURA_ZONE_INDEX_MASK) == 0, "Too many sectors. Overflowed available bits!");
	const u_int uZoneIndex = ((s_pxCurrentZone->GetZoneIndex())<<uASURA_ZONE_INDEX_SHIFT);
#else
	const u_int uZoneIndex = 0;
#endif

	// get the index we'll be giving to the physics cache to identify the poly	
	const u_int uSectorIndex = (s_uCurrentAABBSector<<16) + uZoneIndex;
	const u_int uFaceIndex = uSectorIndex + (uIndex<<1);
	
	//	if ( !DoesFaceIntersectAABB( uFaceIndex, s_pxCurrentCache->GetCacheBoundingBox() ) ) return;

	bool bFaceAdded = false;
	const Asura_Physics_Environment_Face& xFace = s_pxCurrentZone->m_pxFaces[xSector.m_uFaceIndexOffset + uIndex];

	if ( s_pxCurrentZone->DoesFaceIntersectAABB( uFaceIndex, s_pxCurrentCache->GetCacheBoundingBox() ) )
	{
		s_pxCurrentCache->AddFaceToCache( uFaceIndex );
		bFaceAdded = true;
	}

	if (xFace.m_usVertexID[3] != 0xffff)
	{
		if ( s_pxCurrentZone->DoesFaceIntersectAABB( uFaceIndex + 1, s_pxCurrentCache->GetCacheBoundingBox() ) )
		{
			s_pxCurrentCache->AddFaceToCache( uFaceIndex + 1 );
			bFaceAdded = true;
		}
	}

	if ( (!bFaceAdded) || s_pxCurrentCache->GetFacesOnly() ) return;

	// now look at the face's flags and see if there are any edges or verts we should also
	// be adding to the cache.

	const Asura_Physics_Environment_FaceFlags& uFlags = s_pxCurrentZone->m_pxFaceFlags[xSector.m_uFaceIndexOffset + uIndex];

	// no other features to add? 
	if (uFlags.m_ucFlags == 0) return;

	// all these conditionals are a pain - may need to optimise here
	const u_int uVertex0Index = xFace.m_usVertexID[0] + uSectorIndex;
	const u_int uVertex1Index = xFace.m_usVertexID[1] + uSectorIndex;
	const u_int uVertex2Index = xFace.m_usVertexID[2] + uSectorIndex;
	const u_int uVertex3Index = xFace.m_usVertexID[3] + uSectorIndex;

	if (uFlags.m_bEdge0IsConvex)
	{
		s_pxCurrentCache->AddEdgeToCache( uVertex0Index, uVertex1Index );
	}
	if (uFlags.m_bEdge1IsConvex)
	{
		s_pxCurrentCache->AddEdgeToCache( uVertex1Index, uVertex2Index );
	}
	if (uFlags.m_bEdge2IsConvex)
	{
		if (xFace.m_usVertexID[3]==0xffff)
		{
			s_pxCurrentCache->AddEdgeToCache( uVertex2Index, uVertex0Index );
		}
		else
		{
			s_pxCurrentCache->AddEdgeToCache( uVertex2Index, uVertex3Index );
		}
	}
	if (uFlags.m_bEdge3IsConvex)
	{
		s_pxCurrentCache->AddEdgeToCache( uVertex3Index, uVertex0Index );
	}

	if (uFlags.m_bVertex0IsConvex)
	{
		s_pxCurrentCache->AddVertexToCache( uVertex0Index );
	}
	if (uFlags.m_bVertex1IsConvex)
	{
		s_pxCurrentCache->AddVertexToCache( uVertex1Index );
	}
	if (uFlags.m_bVertex2IsConvex)
	{
		s_pxCurrentCache->AddVertexToCache( uVertex2Index );
	}
	if (uFlags.m_bVertex3IsConvex)
	{
		s_pxCurrentCache->AddVertexToCache( uVertex3Index );
	}
}


bool Asura_Physics_Zone::DoesFaceIntersectAABB( const u_int uFaceID, const Asura_Bounding_Box& xTestBB ) const
{
	const Asura_AVector3& xV0 = GetFaceVertex( uFaceID, 0 );
	const Asura_AVector3& xV1 = GetFaceVertex( uFaceID, 1 );
	const Asura_AVector3& xV2 = GetFaceVertex( uFaceID, 2 );


	float fTriVerts[3][3];

	fTriVerts[0][0] = xV0.GetX();
	fTriVerts[0][1] = xV0.GetY();
	fTriVerts[0][2] = xV0.GetZ();

	fTriVerts[1][0] = xV1.GetX();
	fTriVerts[1][1] = xV1.GetY();
	fTriVerts[1][2] = xV1.GetZ();

	fTriVerts[2][0] = xV2.GetX();
	fTriVerts[2][1] = xV2.GetY();
	fTriVerts[2][2] = xV2.GetZ();

	float boxcenter[3]    = { (xTestBB.MaxX + xTestBB.MinX)*0.5f, (xTestBB.MaxY + xTestBB.MinY)*0.5f, (xTestBB.MaxZ + xTestBB.MinZ)*0.5f };
	float boxhalfsize[3]  = { (xTestBB.MaxX - xTestBB.MinX)*0.5f, (xTestBB.MaxY - xTestBB.MinY)*0.5f, (xTestBB.MaxZ - xTestBB.MinZ)*0.5f };

	return (triBoxOverlap(boxcenter, boxhalfsize, fTriVerts) != 0);
}


bool Asura_Physics_Zone::DoesFaceIntersectOBB( const u_int uFaceID, const Asura_Collision_Object& xTestObject, const Asura_Vector_3 xObjectPosition, const Asura_Matrix_3x3 xObjectOrientation ) const
{
	// Move everything relative to the centre of the face
	Asura_AVector3 xV0(GetFaceVertex( uFaceID, 0 ));
	Asura_AVector3 xV1(GetFaceVertex( uFaceID, 1 ));
	Asura_AVector3 xV2(GetFaceVertex( uFaceID, 2 ));

	Asura_AVector3 xCentre = (xV0 + xV1 + xV2) / 3.0f;
	xV0 -= xCentre;
	xV1 -= xCentre;
	xV2 -= xCentre;

	Asura_AVector3 xPos(xObjectPosition);
	xPos -= xCentre;

	// Rotate the verts and the position by the transpose of the orientation. This should align things for an AABB test.
	Asura_AMatrix3x3 xTransposedOri;
	xTransposedOri.FromMat3(xObjectOrientation);
	xTransposedOri.Transpose();

	xV0 = xV0 * xTransposedOri;
	xV1 = xV1 * xTransposedOri;
	xV2 = xV2 * xTransposedOri;
	xPos = xPos * xTransposedOri;

	float fTriVerts[3][3];

	fTriVerts[0][0] = xV0.GetX();
	fTriVerts[0][1] = xV0.GetY();
	fTriVerts[0][2] = xV0.GetZ();

	fTriVerts[1][0] = xV1.GetX();
	fTriVerts[1][1] = xV1.GetY();
	fTriVerts[1][2] = xV1.GetZ();

	fTriVerts[2][0] = xV2.GetX();
	fTriVerts[2][1] = xV2.GetY();
	fTriVerts[2][2] = xV2.GetZ();

	Asura_Bounding_Box xTestBB = xTestObject.GetBoundingBox();

	float boxcenter[3]    = { ((xTestBB.MaxX + xTestBB.MinX)*0.5f) + xPos.GetX(), ((xTestBB.MaxY + xTestBB.MinY)*0.5f) + xPos.GetY(), ((xTestBB.MaxZ + xTestBB.MinZ)*0.5f) + xPos.GetZ() };
	float boxhalfsize[3]  = { (xTestBB.MaxX - xTestBB.MinX)*0.5f, (xTestBB.MaxY - xTestBB.MinY)*0.5f, (xTestBB.MaxZ - xTestBB.MinZ)*0.5f };

	return (triBoxOverlap(boxcenter, boxhalfsize, fTriVerts) != 0);
}


static const Asura_LOS_InputData* g_pxInputData;
static Asura_LOS_OutputData* g_pxOutputData;

static u_int g_uHitSector;
static u_int g_uHitPoly;

bool Asura_Physics_Zone::FindPolyInLOS(const class Asura_LOS_InputData* pxInputData, class Asura_LOS_OutputData* pxOutputData)
{
#ifdef ASURA_TOOLS
	if (m_uNumberOfSectors == 0 || m_bInvalid)
	{
		Tools_Create();
	}
#endif

	if(m_uNumberOfSectors == 0)
	{
		return(false);
	}

	g_pxInputData = pxInputData;
	g_pxOutputData = pxOutputData;

	g_uHitSector = 0xffffffff;
	s_pxCurrentZone = this;

	m_xSectorsTree.TestTreeWithRay(  pxInputData->m_xOrigin, pxInputData->m_xDirection, &(pxOutputData->m_fDistanceToIntersection), FindPoly_SectorCallback );

	if ( g_uHitSector != 0xffffffff )
	{
		const Asura_Physics_Environment_Sector& xSector = m_pxSectors[g_uHitSector];

		const Asura_Physics_Environment_Face& xFace = m_pxFaces[xSector.m_uFaceIndexOffset + g_uHitPoly ];

		const u_int uOffset = xSector.m_uVertexIndexOffset;
		const u_int uVertex0Index = xFace.m_usVertexID[0] + uOffset;
		const u_int uVertex1Index = xFace.m_usVertexID[1] + uOffset;
		const u_int uVertex2Index = xFace.m_usVertexID[2] + uOffset;

		const Asura_AVector3& xV0 = m_pxVertices[ uVertex0Index ].GetPosition();
		const Asura_AVector3& xV1 = m_pxVertices[ uVertex1Index ].GetPosition();
		const Asura_AVector3& xV2 = m_pxVertices[ uVertex2Index ].GetPosition();

		Asura_AVector3 xNormal = (xV1 - xV0)^(xV2 - xV0);
		xNormal.Normalise();

		pxOutputData->m_xNormal = xNormal;

		pxOutputData->m_usPolyFlags = GetFlagsForPolygonInSector( xSector, g_uHitPoly );

#if defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM
		pxOutputData->m_uMaterialResponseHashID = m_puMaterialResponseHashIDs[ xSector.m_uMaterialIDOffset + g_uHitPoly ];
#endif
		pxOutputData->m_sMaterialID = m_psMaterialIDs[ xSector.m_uMaterialIDOffset + g_uHitPoly ];

		// There seems to be no way to determine if we hit the second tri in a quad with the data we have here.
		// For now this isn't important for our needs, but it may become important later so should be beared in mind.
		// So, for now, the face index we return here will always be the first tri in a quad.
#if defined(ASURA_USE_STREAMING)
		Asura_Assert(((g_uHitSector<<16) & uASURA_ZONE_INDEX_MASK) == 0, "Too many sectors. Overflowed available bits!");
		const u_int uZoneIndex = (m_uZoneIndex<<uASURA_ZONE_INDEX_SHIFT);
#else
		const u_int uZoneIndex = 0;
#endif //defined(ASURA_USE_STREAMING)

		pxOutputData->m_uEnvironmentFaceIndex = (g_uHitSector << 16) + uZoneIndex;
		pxOutputData->m_uEnvironmentFaceIndex += g_uHitPoly << 1;
		// Add 1 if we find a way to show we hit is the second tri in a quad (and only when that turns out to be the case, obviously).

		return true;

	}
	else
	{
		return false;
	}
}

void Asura_Physics_Zone::FindPoly_SectorCallback( const u_int uIndex )
{
	s_uCurrentAABBSector = uIndex;

	const Asura_Physics_Environment_Sector& xSector = s_pxCurrentZone->m_pxSectors[s_uCurrentAABBSector];

	xSector.m_xFaceTree.TestTreeWithRay
		(
		g_pxInputData->m_xOrigin,
		g_pxInputData->m_xDirection,
		&(g_pxOutputData->m_fDistanceToIntersection),
		FindPoly_PolyCallback,
		1
		);		
}

void Asura_Physics_Zone::FindPoly_PolyCallback( const u_int uIndex )
{
	const Asura_Physics_Environment_Sector& xSector = s_pxCurrentZone->m_pxSectors[s_uCurrentAABBSector];
	Asura_Assert( xSector.m_uNumberOfFaces > uIndex, "invalid index" );

	const short sFlags = s_pxCurrentZone->GetFlagsForPolygonInSector( xSector, uIndex );

	// does this poly have any of the flags we've been told to look for (or do we not care)?
	if( (Asura_LOS::GetPolyFlagsToTestFor() != UINT_MAX) 
		&& !(Asura_LOS::GetPolyFlagsToTestFor() & sFlags) ) return;
	// does this poly have the flags which we've been told to ignore polys with?
	if( Asura_LOS::GetPolyFlagsToIgnore() & sFlags ) return;


	const Asura_Physics_Environment_Face& xFace = s_pxCurrentZone->m_pxFaces[xSector.m_uFaceIndexOffset + uIndex];

	const u_int uOffset = xSector.m_uVertexIndexOffset;
	const u_int uVertex0Index = xFace.m_usVertexID[0] + uOffset;
	const u_int uVertex1Index = xFace.m_usVertexID[1] + uOffset;
	const u_int uVertex2Index = xFace.m_usVertexID[2] + uOffset;



	const bool bDoubleSided = ( (sFlags & AsuraPolygonFlag_DoubleSided) || g_pxInputData->GetIncludeBackfacingTris() );
	bool bFacingToward;

	float fDistance =	Asura_Maths::RayIntersectsTri
		(
		g_pxInputData->m_xOrigin,
		g_pxInputData->m_xDirection,
		Asura_Vector_3(s_pxCurrentZone->m_pxVertices[uVertex0Index].m_xPosition),
		Asura_Vector_3(s_pxCurrentZone->m_pxVertices[uVertex1Index].m_xPosition),
		Asura_Vector_3(s_pxCurrentZone->m_pxVertices[uVertex2Index].m_xPosition),
		bFacingToward
		);

	if (fDistance >= 0.0f && fDistance < g_pxOutputData->m_fDistanceToIntersection)
	{
		if (bFacingToward || bDoubleSided)
		{
			g_pxOutputData->m_fDistanceToIntersection = fDistance;

			g_uHitSector = s_uCurrentAABBSector;
			g_uHitPoly = uIndex;
		}
	}
	else if (xFace.m_usVertexID[3] != 0xffff)
	{
		const u_int uVertex3Index = xFace.m_usVertexID[3] + uOffset;

		// A quad, check the other triangle.
		fDistance =	Asura_Maths::RayIntersectsTri
			(
			g_pxInputData->m_xOrigin,
			g_pxInputData->m_xDirection,
			Asura_Vector_3(s_pxCurrentZone->m_pxVertices[uVertex2Index].m_xPosition),
			Asura_Vector_3(s_pxCurrentZone->m_pxVertices[uVertex3Index].m_xPosition),
			Asura_Vector_3(s_pxCurrentZone->m_pxVertices[uVertex0Index].m_xPosition),
			bFacingToward
			);

		if (fDistance >= 0.0f && fDistance < g_pxOutputData->m_fDistanceToIntersection)
		{
			if (bFacingToward || bDoubleSided)
			{
				g_pxOutputData->m_fDistanceToIntersection = fDistance;

				g_uHitSector = s_uCurrentAABBSector;
				g_uHitPoly = uIndex;
			}
		}
	}
}


#ifdef ASURA_TOOLS

// stuff to be move to the Tools once completed

#include "Misc/Tools_Adjacency.h"	
#include "Rendering/Asura_Module.h"// only needed for Tools_Create()


void Asura_Physics_Zone::Tools_Create( bool (*pfnShouldIncludeModule)(int iModuleIndex) /*=0*/ )
{
	ASURA_CPU_PROFILE_SCOPED_TIMER("Asura_Physics_Zone::Tools_Create");
	if(!pfnShouldIncludeModule) pfnShouldIncludeModule = Tools_Level_Export::ShouldIncludeModuleInExport;

	Asura_ResourcePushType(ASURA_RESOURCE_COLLISION_MESH);
	{

		// KJFL - Monday, January 31, 2005 - 
		// PHYSICSTODO - cope with DONTEXPORT flag, and potentially modules with zero exportable polys

		// There may be an optimum size/density of 'sector', or we may just want to 
		// have as few sectors as possible (which will remove the number of verts
		// that appear in multiple sectors.
		//
		// For now we'll use the existing modules.

		Reset();
		m_bInvalid = false;

		//Any cached data in rigid body instances will be invalid now
		Asura_Physics_RigidBodyInstance::FlushCachedDataForAllInstances();

		u_int uEstimatedNumberOfSectors = Asura_Module_System::GetNumberOfModules();

		m_uNumberOfSectors = uEstimatedNumberOfSectors;
		Asura_ResourceAllocateArray(m_pxSectors, uEstimatedNumberOfSectors);

		// consider all modules
		u_int uNumberOfModules = Asura_Module_System::GetNumberOfModules();

		u_int uEstimatedNumberOfFaces = 0;
		u_int uEstimatedNumberOfVertices = 0;

		// estimate how many polys and verts we'll need in total,
		// and setup offset numbers for each sector
		for (u_int u=0; u<uNumberOfModules; u++)
		{
			const Asura_CollisionMesh* pxMesh = Asura_Module_System::GetCollisionMeshForModule(u);

			if (!pxMesh) continue;
			
			Asura_Physics_Environment_Sector& xSector = m_pxSectors[u];

			
			xSector.m_uFaceIndexOffset = uEstimatedNumberOfFaces;
			xSector.m_uVertexIndexOffset = uEstimatedNumberOfVertices;
			xSector.m_usFlags = 0;
			
			//		xSector.m_pxFacesTree = 0;

			//During exports , ignore modules that don't belong to the current light set
			//RAR 12/12/2007
			if(!pfnShouldIncludeModule(u))
			{
				xSector.m_uNumberOfFaces = 0;
			}
			else
			{
				xSector.m_uNumberOfFaces = pxMesh->GetNumberOfPolys();
				uEstimatedNumberOfFaces += pxMesh->GetNumberOfPolys();
				uEstimatedNumberOfVertices += pxMesh->GetNumberOfVertices();
			}
		}

		// allocate all the memory for the environment in big chunks
		// in game we'll be able to load in all this data in single, continuous large reads
		// which will improve loading times and memory allocation friendliness
		m_uTotalNumberOfFaces = uEstimatedNumberOfFaces;
		m_uTotalNumberOfVertices = uEstimatedNumberOfVertices;

		Asura_ResourceAllocateArray(m_pxFaces, uEstimatedNumberOfFaces);
		Asura_ResourceAllocateArray(m_pxFaceFlags, uEstimatedNumberOfFaces);
		Asura_ResourceAllocateArray(m_pxVertices, uEstimatedNumberOfVertices);

		for (u_int u=0; u<uNumberOfModules; u++)
		{
			const Asura_CollisionMesh* pxMesh = Asura_Module_System::GetCollisionMeshForModule(u);

			if (!pxMesh) continue;

			//During exports , ignore modules that don't belong to the current light set
			//RAR 12/12/2007
			if(!pfnShouldIncludeModule(u)) continue;


			Asura_Physics_Environment_Sector& xSector = m_pxSectors[u];

			// add this module's faces 
			u_int uNumberOfFaces = xSector.m_uNumberOfFaces;

			for (u_int v=0; v<uNumberOfFaces; v++)
			{
				Asura_Physics_Environment_Face& xFace = m_pxFaces[xSector.m_uFaceIndexOffset + v];

				xFace.m_usVertexID[0] = (u_short) pxMesh->GetVertexIndex(v,0);
				xFace.m_usVertexID[1] = (u_short) pxMesh->GetVertexIndex(v,1);
				xFace.m_usVertexID[2] = (u_short) pxMesh->GetVertexIndex(v,2);
				xFace.m_usVertexID[3] = (u_short) pxMesh->GetVertexIndex(v,3);

				m_pxFaceFlags[xSector.m_uFaceIndexOffset + v].m_ucFlags = 0;
			}

			// and then the verts
			u_int uNumberOfVertices = pxMesh->GetNumberOfVertices();

			for (u_int v=0; v<uNumberOfVertices; v++)
			{
				const Asura_Vector_3 xPos = pxMesh->GetVertex(v) + Asura_Module_System::GetPositionOfModule(u);

				m_pxVertices[xSector.m_uVertexIndexOffset + v].m_xPosition = xPos;

			}
		}


		if (!Tools_Adjacency::IsInitialised())
		{
			Tools_Adjacency::Initialise();
		}

		{
		ASURA_CPU_PROFILE_SCOPED_TIMER("Edge list calculation");
		// scan through collision polys and examine the edges between adjacent polys
		// if they are 'sticky-out' edges, add them to the edge list.
		Tools_ProgressIndicator* pxProgress = new Tools_ProgressIndicator("Creating Physics Zones", false);
		for (int u=0; u<Asura_Module_System::GetNumberOfModules(); ++u)
		{
			const Asura_CollisionMesh* pxMesh = Asura_Module_System::GetCollisionMeshForModule(u);

			if (!pxMesh) continue;

			//During exports , ignore modules that don't belong to the current light set
			//RAR 12/12/2007
			if(!pfnShouldIncludeModule(u)) continue;

#if defined( ASURA_ASSERT)
			const char* pszModuleMeshName = "UNKNOWN";
#if defined( ASURA_TOOLS)
			const Asura_Mesh* pxModuleMesh = Asura_Module_System::GetMeshForModule(u);
			if( NULL != pxModuleMesh)
			{
				pszModuleMeshName = pxModuleMesh->Name.GetString();
			}
#endif
#endif
			int uNumberOfPolys = (pxMesh->GetNumberOfPolys());

			for (int v=0; v<uNumberOfPolys; v++)
			{
				// skip any polys that aren't collision ones
				if ( (pxMesh->GetFlagsIgnoringAnim( v ) & AsuraPolygonFlag_NoCollisions) )
				{
					continue;
				}

				const Tools_Polygon_Adjacency* pxPolyAdj = Tools_Adjacency::GetCollPolygonAdjacencies( u, v );

				for(const Tools_Adjacent_Polygon* pxAdj = pxPolyAdj->GetFirstAdjacency(); pxAdj; pxAdj = pxAdj->GetNext())
				{

					// ignore polys that only meet at a vertex rather than an edge
					if (pxAdj->GetNumSharedVertices() != 2) continue;
					// RAR 02/05/2006 Ignore adjacencies between streaming zones for now
					if (pxAdj->IsBetweenZones()) continue;

					const int iAdjModuleIndex = pxAdj->GetModuleIndex();
					if ( iAdjModuleIndex < u)
					{
						// we've already done this module
						continue;
					}
					const int iAdjPolygonIndex = pxAdj->GetPolygonIndex();
					if (iAdjModuleIndex==u && iAdjPolygonIndex < v)
					{
						// we've already done this poly 
						continue;
					}

					// skip any polys that aren't collision ones
					{
						const Asura_CollisionMesh* pxAdjPolysMesh = Asura_Module_System::GetCollisionMeshForModule( iAdjModuleIndex );

						if ( (pxAdjPolysMesh->GetFlagsIgnoringAnim( iAdjPolygonIndex ) & AsuraPolygonFlag_NoCollisions) )
						{
							continue;
						}
					}

					// We have an edge - but is it a collision edge?
					// (i.e. to the two polygons face away from each other?)

					bool bStickyOutEdge = true;


					{
						Asura_Vector_3 xPointInsidePolyA;

						
						xPointInsidePolyA  = pxMesh->GetVertex(pxMesh->GetVertexIndex(v,0));
						xPointInsidePolyA += pxMesh->GetVertex(pxMesh->GetVertexIndex(v,1));
						xPointInsidePolyA += pxMesh->GetVertex(pxMesh->GetVertexIndex(v,2));
						xPointInsidePolyA /= 3.0f;
						xPointInsidePolyA += Asura_Module_System::GetPositionOfModule(u);

						const Asura_CollisionMesh* pxBMesh = Asura_Module_System::GetCollisionMeshForModule(iAdjModuleIndex);

						Asura_Vector_3 xBToA = xPointInsidePolyA - ( pxBMesh->GetVertex(pxBMesh->GetVertexIndex(iAdjPolygonIndex,0)) + Asura_Module_System::GetPositionOfModule(iAdjModuleIndex) );

						Asura_Vector_3 xBFaceNormal;
						pxBMesh->GetNormal(iAdjPolygonIndex,xBFaceNormal);

						float fDot = xBToA * xBFaceNormal;

						if (fDot>-0.01f) bStickyOutEdge	= false;

						if (!bStickyOutEdge) continue;
					}

					// What are the vertices?

					u_int uEdgeVertices[4];
					u_int uNumberOfVerticesFound = 0;

					u_int uEdgeIndicator = 0;


					
					for (int i=0; i<pxMesh->GetNumVerticesInPoly(v); i++)
					{
						u_int uVertexIndex = pxMesh->GetVertexIndex(v,i);


						const Tools_Polygon_Adjacency* xVertexAdj = Tools_Adjacency::GetCollVertexAdjacencies(u,uVertexIndex);
						if ( xVertexAdj->Contains( iAdjModuleIndex, iAdjPolygonIndex ) )
						{
							uEdgeVertices[uNumberOfVerticesFound++] = uVertexIndex;

							uEdgeIndicator += i*i;
						}
					}

					Asura_Assertf( uNumberOfVerticesFound == 2, ("that's no edge! On mesh '%s'", pszModuleMeshName) );

					if (uNumberOfVerticesFound != 2) continue;

					Asura_Physics_Environment_Sector& xSector = m_pxSectors[u];

					Asura_Physics_Environment_FaceFlags& uFlags = m_pxFaceFlags[xSector.m_uFaceIndexOffset + v];

					switch(uEdgeIndicator)
					{
					default:
						{
							// KJFL - 20 October 2005 - to clarify, since this just fired on a project:
							//
							// This isn't an outside edge made from the vertices of the poly but instead 
							// it's a diagonal or something across a quad. (Or the data is corrupt!)
							//
							// The former can happen with malformed mesh where two quads overlap, with 
							// differing internal diagonals that share vertices, or if a polygon edge is
							// adjacent to the internal diagonal of a quad, rather than an external edge.
							// Either way - the mesh needs fixing.

							// KJFL - 18 July 2006 - removing assert, since it seems to turn up occasionally
							// and the case is handled gracefully (i.e. do nothing) so it's pretty harmless.
							//Asura_Assert( false, "Invalid edge" );
							break;
						}

						// first two cases are for both quads and tris

					case 1:  // 0*0 + 1*1
						{
							uFlags.m_bEdge0IsConvex = true;

							uFlags.m_bVertex0IsConvex = true;
							uFlags.m_bVertex1IsConvex = true;
							break;
						}
					case 5:  // 1*1 + 2*2
						{
							uFlags.m_bEdge1IsConvex = true;

							uFlags.m_bVertex1IsConvex = true;
							uFlags.m_bVertex2IsConvex = true;
							break;
						}

						// triangle only case

					case 4:  // 2*2 + 0*0 
						{
							uFlags.m_bEdge2IsConvex = true;

							uFlags.m_bVertex2IsConvex = true;
							uFlags.m_bVertex0IsConvex = true;
							break;
						}

						// last two cases are quad only

					case 13: // 2*2 + 3*3
						{
							uFlags.m_bEdge2IsConvex = true;

							uFlags.m_bVertex2IsConvex = true;
							uFlags.m_bVertex3IsConvex = true;
							break;
						}
					case 9:  // 3*3 + 0*0
						{
							uFlags.m_bEdge3IsConvex = true;

							uFlags.m_bVertex3IsConvex = true;
							uFlags.m_bVertex0IsConvex = true;
							break;
						}

					}


				}

			}

			pxProgress->Update(u, Asura_Module_System::GetNumberOfModules() );
		}
		pxProgress->Finish();
		delete pxProgress;
		}

		m_uTotalNumberOfPolygonFlags = uEstimatedNumberOfFaces;
		m_uTotalNumberOfMaterialIDs = uEstimatedNumberOfFaces;

		Asura_ResourceAllocateArray(m_pusPolygonFlags, m_uTotalNumberOfPolygonFlags);
        Asura_ResourceAllocateArray( m_puToolsPolygonFlags, m_uTotalNumberOfPolygonFlags );

#ifdef ASURA_USE_MATERIAL_RESPONSE_SYSTEM
		Asura_ResourceAllocateArray(m_puMaterialResponseHashIDs,m_uTotalNumberOfMaterialIDs);
#endif
		Asura_ResourceAllocateArray(m_psMaterialIDs, m_uTotalNumberOfMaterialIDs);

		u_int uMaterialOffset = 0;
		u_int uPolyOffset = 0;
		for (int u=0; u<Asura_Module_System::GetNumberOfModules(); ++u)
		{
			const Asura_CollisionMesh* pxMesh = Asura_Module_System::GetCollisionMeshForModule(u);

			if (!pxMesh) continue;

			//During exports , ignore modules that don't belong to the current light set
			//RAR 12/12/2007
			if(!pfnShouldIncludeModule(u)) continue;


			int iNumberOfPolys = (pxMesh->GetNumberOfPolys());

			Asura_Physics_Environment_Sector& xSector = m_pxSectors[u];
			xSector.m_uMaterialIDOffset = uMaterialOffset;
			xSector.m_uPolygonFlagOffset = uPolyOffset;

			xSector.m_bAllFacesUseSamePolygonFlag = true;

			
			for ( int iPoly=0; iPoly<iNumberOfPolys; ++iPoly )
			{
				if ( pxMesh->GetFlagsIgnoringAnim(iPoly) != pxMesh->GetFlagsIgnoringAnim(0) )
				{
					xSector.m_bAllFacesUseSamePolygonFlag = false;
				}
                if ( pxMesh->Tools_GetFlagsIgnoringAnim( iPoly ) != pxMesh->Tools_GetFlagsIgnoringAnim( 0 ) )
                {
                    xSector.m_bAllFacesUseSamePolygonFlag = false;
                }
			}
			

			xSector.m_bAllFacesUseSameMaterialID = false;

			if (!xSector.m_bAllFacesUseSamePolygonFlag  || !iNumberOfPolys)
			{
				for ( int iPoly=0; iPoly<iNumberOfPolys; ++iPoly )
				{
					//(Setting of the flag AsuraPolygonFlag_Physics_WalkableSurface is no longer done here , 
					//and is dealt with when the collision mesh is created instead.)
					//RAR 08/06/2006

                    m_pusPolygonFlags[iPoly+uPolyOffset]     = pxMesh->GetFlagsIgnoringAnim( iPoly );
                    m_puToolsPolygonFlags[iPoly+uPolyOffset] = pxMesh->Tools_GetFlagsIgnoringAnim( iPoly );

				}
				uPolyOffset += iNumberOfPolys;
			}
			else
			{
                m_pusPolygonFlags[uPolyOffset]     = pxMesh->GetFlagsIgnoringAnim( 0 );
                m_puToolsPolygonFlags[uPolyOffset] = pxMesh->Tools_GetFlagsIgnoringAnim( 0 );

				++uPolyOffset;
			}

			for ( int iPoly=0; iPoly<iNumberOfPolys; ++iPoly )
			{
				#ifdef ASURA_USE_MATERIAL_RESPONSE_SYSTEM
					m_puMaterialResponseHashIDs[iPoly+uMaterialOffset] = pxMesh->GetMaterialResponseHashID(iPoly);
				#endif
				m_psMaterialIDs[iPoly+uMaterialOffset] = (short) pxMesh->GetMaterialID(iPoly);
			}

			uMaterialOffset += iNumberOfPolys;
		}

		m_uTotalNumberOfPolygonFlags = uPolyOffset;

		//Remove any sectors that have no polygons (it prevents the trees from getting upset).
		//This does mean that the sector indeces will no longer match the module indeces , but
		//since the sectors get reordered anyway in the tree creation below,this shouldn't be an issue.
		//RAR 12/06/2006
		for(u_int u=0;u<m_uNumberOfSectors;)
		{
			if(!m_pxSectors[u].m_uNumberOfFaces)
			{
				//remove this sector ,and replace with the last one
				m_pxSectors[u] = m_pxSectors[--m_uNumberOfSectors];
			}
			else
			{
				u++;
			}
		}


		// calculate AABB trees for sectors and their elements
		CreateFaceTrees();
	}

	Asura_ResourcePopType();
}

void Asura_Physics_Zone::CreateFaceTrees( void )
{
	ASURA_CPU_PROFILE_SCOPED_TIMER("Asura_Physics_Zone::CreateFaceTrees");
	Asura_ResourcePushType(ASURA_RESOURCE_COLLISION_MESH);
	{
		u_short* pusOrderedPolygonFlags;
        u_int* puOrderedToolsPolygonFlags;
		Asura_Physics_Environment_Face* pxOrderedFaces;
		Asura_Physics_Environment_FaceFlags* pxOrderedFaceFlags;
		Asura_Physics_Environment_Sector* pxOrderedSectors;

#ifdef ASURA_USE_MATERIAL_RESPONSE_SYSTEM
		Asura_Hash_ID* puOrderedMaterialResponseHashIDs;
		Asura_ResourceAllocateArray(puOrderedMaterialResponseHashIDs, m_uTotalNumberOfMaterialIDs);
#endif
		short* psOrderedMaterialIDs;
		Asura_ResourceAllocateArray(psOrderedMaterialIDs, m_uTotalNumberOfMaterialIDs);

		Asura_ResourceAllocateArray(pusOrderedPolygonFlags, m_uTotalNumberOfPolygonFlags);
        Asura_ResourceAllocateArray( puOrderedToolsPolygonFlags, m_uTotalNumberOfPolygonFlags );
		Asura_ResourceAllocateArray(pxOrderedFaces, m_uTotalNumberOfFaces);
		Asura_ResourceAllocateArray(pxOrderedFaceFlags, m_uTotalNumberOfFaces);
		Asura_ResourceAllocateArray(pxOrderedSectors, m_uNumberOfSectors);

#ifdef TOOLS_INCLUDE_PHYSICS_MODULE_AND_POLY_LOOKUP_DATA
		m_ppuPolyLookupArrays = new u_int*[ m_uNumberOfSectors ];
#endif

		Tools_ProgressIndicator* pxProgress = new Tools_ProgressIndicator("Creating Face Trees", false);
		for (u_int u=0; u<m_uNumberOfSectors; u++)
		{
			
			s_uCurrentAABBSector = u;

			Asura_Physics_Environment_Sector& xSector = m_pxSectors[u];

			if (xSector.m_uNumberOfFaces)
			{
				Asura_Bounding_Box* pxBBs = new Asura_Bounding_Box[xSector.m_uNumberOfFaces];
				Asura_Vector_3* pxCentres = new Asura_Vector_3[xSector.m_uNumberOfFaces];

				for (u_int uFace=0; uFace<xSector.m_uNumberOfFaces; uFace++)
				{
					pxBBs[uFace].Reset();

					for (u_int v=0; v<4; v++)
					{
						u_int uVertex = m_pxFaces[xSector.m_uFaceIndexOffset + uFace].GetVertexIndex(v);

						if (uVertex == 0xffff) continue;

						const Asura_AVector3& xVertex = m_pxVertices[xSector.m_uVertexIndexOffset + uVertex].GetPosition();

						pxBBs[uFace].Set( Asura_Vector_3(xVertex) );
					}

					pxCentres[uFace] = pxBBs[uFace].CalculateCentre();
				}
                				
				u_int* puLeafConversionArray = Tools_LBAABB_Tree::CreateRunTimeTree( xSector.m_xFaceTree, xSector.m_uNumberOfFaces, pxBBs, pxCentres );


				if ( xSector.m_bAllFacesUseSamePolygonFlag )
				{
					pusOrderedPolygonFlags[ xSector.m_uPolygonFlagOffset ] = m_pusPolygonFlags[ xSector.m_uPolygonFlagOffset ];
                    puOrderedToolsPolygonFlags[ xSector.m_uPolygonFlagOffset ] = m_puToolsPolygonFlags[ xSector.m_uPolygonFlagOffset ];
                }

				for (u_int uFace=0; uFace<xSector.m_uNumberOfFaces; uFace++)
				{
					u_int uNewFace = puLeafConversionArray[ uFace ];

					if ( !xSector.m_bAllFacesUseSamePolygonFlag )
					{
						pusOrderedPolygonFlags[ uFace + xSector.m_uPolygonFlagOffset ]	   = m_pusPolygonFlags[ uNewFace + xSector.m_uPolygonFlagOffset ];
                        puOrderedToolsPolygonFlags[ uFace + xSector.m_uPolygonFlagOffset ] = m_puToolsPolygonFlags[ uNewFace + xSector.m_uPolygonFlagOffset ];
                    }

#ifdef ASURA_USE_MATERIAL_RESPONSE_SYSTEM
					puOrderedMaterialResponseHashIDs[ uFace + xSector.m_uMaterialIDOffset ]	= m_puMaterialResponseHashIDs[ uNewFace + xSector.m_uMaterialIDOffset ];
#endif
					psOrderedMaterialIDs[ uFace + xSector.m_uMaterialIDOffset ]	= m_psMaterialIDs[ uNewFace + xSector.m_uMaterialIDOffset ];

                    pxOrderedFaces[ uFace + xSector.m_uFaceIndexOffset ]        = m_pxFaces[ uNewFace + xSector.m_uFaceIndexOffset ];
                    pxOrderedFaceFlags[ uFace + xSector.m_uFaceIndexOffset ]    = m_pxFaceFlags[ uNewFace + xSector.m_uFaceIndexOffset ];
				}

#ifdef TOOLS_INCLUDE_PHYSICS_MODULE_AND_POLY_LOOKUP_DATA
				m_ppuPolyLookupArrays[ u ] = puLeafConversionArray;
#else
				delete [] puLeafConversionArray;
#endif
                delete [] pxBBs;
				delete [] pxCentres;
			}
			pxProgress->Update(u, m_uNumberOfSectors );
		}
		pxProgress->Finish();
		delete pxProgress;

		// replace environment data with the correctly ordered versions
		Asura_ResourceDeallocateArray(m_pxFaces);
		Asura_ResourceDeallocateArray(m_pxFaceFlags);
		Asura_ResourceDeallocateArray(m_pusPolygonFlags);
        Asura_ResourceDeallocateArray( m_puToolsPolygonFlags );

#ifdef ASURA_USE_MATERIAL_RESPONSE_SYSTEM
		Asura_ResourceDeallocateArray(m_puMaterialResponseHashIDs);
		m_puMaterialResponseHashIDs = puOrderedMaterialResponseHashIDs;
#endif
		Asura_ResourceDeallocateArray(m_psMaterialIDs);
		m_psMaterialIDs = psOrderedMaterialIDs;

		m_pxFaces = pxOrderedFaces;
		m_pxFaceFlags = pxOrderedFaceFlags;
		m_pusPolygonFlags = pusOrderedPolygonFlags;
        m_puToolsPolygonFlags = puOrderedToolsPolygonFlags;

		if(m_uNumberOfSectors)
		{
			Asura_Bounding_Box* pxBBs = new Asura_Bounding_Box[m_uNumberOfSectors];
			Asura_Vector_3* pxCentres = new Asura_Vector_3[m_uNumberOfSectors];


			for (u_int u=0; u<m_uNumberOfSectors; u++)
			{
				Asura_Physics_Environment_Sector& xSector = m_pxSectors[u];

				pxBBs[u] = xSector.m_xFaceTree.GetBoundingBox();
				pxCentres[u] = pxBBs[u].CalculateCentre();
			}

			u_int* puLeafConversionArray = Tools_LBAABB_Tree::CreateRunTimeTree( m_xSectorsTree, m_uNumberOfSectors, pxBBs, pxCentres );

			for (u_int u=0; u<m_uNumberOfSectors; u++)
			{
				u_int uNewSector = puLeafConversionArray[ u ];

				pxOrderedSectors[ u ] = m_pxSectors[ uNewSector ];
			}

#ifdef TOOLS_INCLUDE_PHYSICS_MODULE_AND_POLY_LOOKUP_DATA
			m_puSectorLookupArray = puLeafConversionArray;
#else
			delete [] puLeafConversionArray;
#endif
			delete [] pxBBs;
			delete [] pxCentres;
		}

		Asura_ResourceDeallocateArray(m_pxSectors);
		m_pxSectors = pxOrderedSectors;

		m_bTreesUseOneHugeNodeArray = false;
	}
	Asura_ResourcePopType();
}

void Asura_Physics_Zone::ConvertMaterialIndecesToIDs(const int* piMaterialIndexToIDConversion)
{
	for(u_int u=0;u<m_uTotalNumberOfMaterialIDs;u++)
	{
		if(m_psMaterialIDs[u] != ASURA_MATERIAL_NONE)
		{
			Asura_Assert(m_psMaterialIDs[u]>= 0 && m_psMaterialIDs[u] < ASURA_MAX_NO_OF_MATERIALS,"Invalid material index");
			m_psMaterialIDs[u] = (short) piMaterialIndexToIDConversion[m_psMaterialIDs[u]];
		}
	}
}

#endif //ASURA_TOOLS


////////////////////////////////////////////////////////////
//
//	Asura_Physics_Environment
//
////////////////////////////////////////////////////////////

#if defined(ASURA_USE_STREAMING)
Asura_Physics_Zone*							Asura_Physics_Environment::s_pxCurrentZone	= NULL;		// Currently active handler.
Asura_TemplateHashTree<Asura_Physics_Zone>	Asura_Physics_Environment::s_xZoneTree;					// Tree of all of the handlers available.
#else
Asura_Physics_Zone							Asura_Physics_Environment::s_xZone;						// Single static-instance of a zone.
#endif	// defined(ASURA_USE_STREAMING)


void Asura_Physics_Environment::Initialise( void )
{
#if defined(ASURA_USE_STREAMING)
	g_xZoneMap.Reset();
#endif //defined(ASURA_USE_STREAMING)
}

void Asura_Physics_Environment::Reset( void )
{
	Shutdown();
	Initialise();
}

void Asura_Physics_Environment::Shutdown( void )
{
#if defined(ASURA_USE_STREAMING)
	// Reset the entire zone-tree...
	s_xZoneTree.Reset();

	// Remove the current zone...
	s_pxCurrentZone	= NULL;
#else
	Asura_Physics_Zone&	xZone	= GetCurrentZone();

	// Reset the current instance...
	xZone.Reset();
#endif	// defined(ASURA_USE_STREAMING)
}



#if defined(ASURA_USE_STREAMING)

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Physics_Environment::GetZoneFromIndex
// Author		: Martin G. Harris
//
// Parameters   : An ID containing the zone, sector and primitive index.
// Return value : A pointer to the relevant zone given by the zone index bits.
//				  NULL if the zone has been unloaded.
//
////////////////////////////////////////////////////////////////////////////////
Asura_Physics_Zone* Asura_Physics_Environment::GetZoneFromIndex(u_int uZoneSectorID)
{
	const u_int uZone = ((uZoneSectorID & uASURA_ZONE_INDEX_MASK)>>uASURA_ZONE_INDEX_SHIFT);
	return g_xZoneMap.GetZone(uZone);
}


////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Physics_Environment::CreateZone
// Author       : Andy Tate
// Date         : 14-Mar-2006
//
// Description  : Attempts to create a physics-zone with the specified hash-ID.
//
// Parameters   : The hash-ID of the zone to create.
//
// Return value : A pointer to the zone created.
////////////////////////////////////////////////////////////////////////////////

Asura_Physics_Zone* Asura_Physics_Environment::CreateZone (const Asura_Hash_ID uHash)
{
	Asura_Physics_Zone*	pxZone	= s_xZoneTree.GetDataByHash(uHash);

	// Do we already have this zone?
	if(pxZone == NULL)
	{
		// No - create one...
		pxZone	= s_xZoneTree.AddNode(uHash);
	}
	else
	{
		// Yes - this shouldn't happen...
		Asura_Assert(false, "Already have an physics-zone with this hash-ID.");
	}

	// Return the zone...
	return(pxZone);
}

/// End of function 'Asura_Physics_Environment::CreateZone'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Physics_Environment::DestroyZone
// Author       : Andy Tate
// Date         : 14-Mar-2006
//
// Description  : Attempts to destroy the physics-zone with the specified hash-ID.
//
// Parameters   : The hash-ID of the zone to destroy.
//
// Return value : "true" if destroyed, "false" if failed.
////////////////////////////////////////////////////////////////////////////////

bool Asura_Physics_Environment::DestroyZone (const Asura_Hash_ID uHash)
{
	// Is this the current zone?
	if(s_pxCurrentZone == s_xZoneTree.GetDataByHash(uHash))
	{
		s_pxCurrentZone	= NULL;
	}

	// Call the delete function...
	return(s_xZoneTree.DeleteNodeByHash(uHash));
}

/// End of function 'Asura_Physics_Environment::DestroyZone'. ///

#endif	// defined(ASURA_USE_STREAMING)


////////////////////////////////
// FindPolyInLOS_ZoneCallback //
////////////////////////////////
#if defined(ASURA_USE_STREAMING)
bool Asura_Physics_Environment::FindPolyInLOS_ZoneCallback(Asura_Hash_ID uHash, Asura_Physics_Zone* pxZone, void* pbHitPolygon)
{
	if (pxZone)
	{
		// MGH 12/6/2006: Replace the texture animation handler, which is needed for material flags.
		Asura_Streaming_TexAnimHandlerReplacement xReplace(uHash);

		const bool bResult = pxZone->FindPolyInLOS(g_pxInputData, g_pxOutputData);
		if (bResult && pbHitPolygon)
		{
			bool* pbReturnBool = static_cast<bool*>(pbHitPolygon);
			*pbReturnBool = true;
		}
	}
	return true;
}
#endif //defined(ASURA_USE_STREAMING)

///////////////////
// FindPolyInLOS //
///////////////////
bool Asura_Physics_Environment::FindPolyInLOS (const Asura_LOS_InputData* pxInputData, Asura_LOS_OutputData* pxOutputData)
{
#if defined(ASURA_USE_STREAMING)
	// Iterate over the zones looking for the nearest intersection.
	bool bHitPolygon = false;
	g_pxInputData  = pxInputData;
	g_pxOutputData = pxOutputData;
	s_xZoneTree.Traverse(FindPolyInLOS_ZoneCallback, &bHitPolygon);

	return bHitPolygon;
#else
	// Simply find the poly using this zone...
	return(s_xZone.FindPolyInLOS(pxInputData, pxOutputData));
#endif	// defined(ASURA_USE_STREAMING)
}

///////////////
// FillCache //
///////////////
void Asura_Physics_Environment::FillCache (Asura_Physics_EnvironmentCache* pxCache)
{
#if defined(ASURA_USE_STREAMING)
	// Iterate over all the zones filling the cache.
	s_xZoneTree.Traverse(FillCache_ZoneCallback, pxCache);
#else
	// Simply fill the cache using this zone...
	s_xZone.FillCache(pxCache);
#endif	// defined(ASURA_USE_STREAMING)
}

////////////////////////////
// FillCache_ZoneCallback //
////////////////////////////
#if defined(ASURA_USE_STREAMING)
bool Asura_Physics_Environment::FillCache_ZoneCallback(Asura_Hash_ID uHash, Asura_Physics_Zone* pxZone, void* pxCache)
{
	if (pxZone)
	{
		// MGH 12/6/2006: Replace the texture animation handler, which is needed for material flags.
		Asura_Streaming_TexAnimHandlerReplacement xReplace(uHash);

		pxZone->FillCache(static_cast<Asura_Physics_EnvironmentCache*>(pxCache));
	}

	return true;
}
#endif //defined(ASURA_USE_STREAMING)


bool Asura_Physics_Environment::DoesFaceIntersectAABB( const Asura_AVector3& xV0, const Asura_AVector3& xV1, const Asura_AVector3& xV2, const Asura_Bounding_Box& xTestBB )
{
	float fTriVerts[3][3];

	fTriVerts[0][0] = xV0.GetX();
	fTriVerts[0][1] = xV0.GetY();
	fTriVerts[0][2] = xV0.GetZ();

	fTriVerts[1][0] = xV1.GetX();
	fTriVerts[1][1] = xV1.GetY();
	fTriVerts[1][2] = xV1.GetZ();

	fTriVerts[2][0] = xV2.GetX();
	fTriVerts[2][1] = xV2.GetY();
	fTriVerts[2][2] = xV2.GetZ();

	float boxcenter[3]    = { (xTestBB.MaxX + xTestBB.MinX)*0.5f, (xTestBB.MaxY + xTestBB.MinY)*0.5f, (xTestBB.MaxZ + xTestBB.MinZ)*0.5f };
	float boxhalfsize[3]  = { (xTestBB.MaxX - xTestBB.MinX)*0.5f, (xTestBB.MaxY - xTestBB.MinY)*0.5f, (xTestBB.MaxZ - xTestBB.MinZ)*0.5f };

	return (triBoxOverlap(boxcenter, boxhalfsize, fTriVerts) != 0);
}

bool Asura_Physics_Environment::DoesEdgeIntersectAABB( const Asura_AVector3& xEdge0, const Asura_AVector3& xEdge1, const Asura_Bounding_Box& xTestBB )
{
	Asura_AVector3 xBoxExtents, xDiff, xDir, fAWdU;
	const Asura_AVector3 xBBCentre( xTestBB.CalculateCentre() );


	// (MH - 22.09.2005) TODO: this could be done in parallel
	xDir.SetX( 0.5f * (xEdge1.GetX() - xEdge0.GetX()) );
	xBoxExtents.SetX( xTestBB.MaxX - xBBCentre.GetX() );
	xDiff.SetX( (0.5f * (xEdge1.GetX() + xEdge0.GetX())) - xBBCentre.GetX() );
	fAWdU.SetX( Asura_Maths::Abs( xDir.GetX() ) );
	if ( Asura_Maths::Abs( xDiff.GetX() ) > xBoxExtents.GetX() + fAWdU.GetX() ) return false;


	xDir.SetY( 0.5f * (xEdge1.GetY() - xEdge0.GetY()) );
	xBoxExtents.SetY( xTestBB.MaxY - xBBCentre.GetY() );
	xDiff.SetY( (0.5f * (xEdge1.GetY() + xEdge0.GetY())) - xBBCentre.GetY() );
	fAWdU.SetY( Asura_Maths::Abs( xDir.GetY() ) );
	if ( Asura_Maths::Abs( xDiff.GetY() ) > xBoxExtents.GetY() + fAWdU.GetY() ) return false;


	xDir.SetZ( 0.5f * (xEdge1.GetZ() - xEdge0.GetZ()) );
	xBoxExtents.SetZ( xTestBB.MaxZ - xBBCentre.GetZ() );
	xDiff.SetZ( (0.5f * (xEdge1.GetZ() + xEdge0.GetZ())) - xBBCentre.GetZ() );
	fAWdU.SetZ( Asura_Maths::Abs( xDir.GetZ() ) );
	if ( Asura_Maths::Abs( xDiff.GetZ() ) > xBoxExtents.GetZ() + fAWdU.GetZ() ) return false;


	const float f0 = xDir.GetY() * xDiff.GetZ() - xDir.GetZ() * xDiff.GetY();
	if ( Asura_Maths::Abs( f0 ) > xBoxExtents.GetY() * fAWdU.GetZ() + xBoxExtents.GetZ() * fAWdU.GetY()) return false;

	const float f1 = xDir.GetZ() * xDiff.GetX() - xDir.GetX() * xDiff.GetZ();
	if ( Asura_Maths::Abs( f1 ) > xBoxExtents.GetX() * fAWdU.GetZ() + xBoxExtents.GetZ() * fAWdU.GetX()) return false;

	const float f2 = xDir.GetX() * xDiff.GetY() - xDir.GetY() * xDiff.GetX();
	if ( Asura_Maths::Abs( f2 ) > xBoxExtents.GetX() * fAWdU.GetY() + xBoxExtents.GetY() * fAWdU.GetX()) return false;

	return true;
}


/// End of file. ///
