//
// Asura_Physics_Ragdoll_Description.cpp
//
// Created by KJFL - 04 November 2004
//
// Copyright (C) 2004 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

// This file's header.
#include "Physics/Asura_Physics_Ragdoll_Description.h"

#include "Animation/Asura_Hierarchy.h"
#include "Animation/Asura_Animation.h"
#include "Core/Memory/Asura_Mem.h"
#include "Physics/Asura_Physics_Ragdoll.h"

#ifdef ASURA_TOOLS
#include "Camera/Asura_Camera.h"
#include "Hierarchy/Tools_Mode_Hierarchy.h"
#include "Interface/Asura_Cursor.h"
#include "Misc/Tools_Render.h"
#include "Rendering/Asura_Render.h"
#include "Rendering/Asura_Text_Output.h"
#endif

Asura_Physics_Ragdoll_Description::Asura_Physics_Ragdoll_Description()
{
	m_uNumberOfHierarchyNodes = 0;
	m_pxHierarchyNodes = 0;

	m_uNumberOfCollisionObjects = 0;
	m_pxCollisionObjects = 0;

	m_uNumberOfJoints = 0;
	m_pxJoints = 0;
}	

#if defined (ASURA_TOOLS)
void Asura_Physics_Ragdoll_Description::Tools_Initialise( u_int uNumberOfHierarchyNodes )
{
	m_uNumberOfHierarchyNodes = uNumberOfHierarchyNodes;
	m_pxHierarchyNodes = new Asura_Physics_Ragdoll_Desc_HierarchyNode[ uNumberOfHierarchyNodes ];
	
	// Allocate space for the collision objects, but set the number of them to zero. This allows
	// us to fill them in as required without reallocating. Obviously this limits the number of
	// collision objects and joints to be <= number of hierarchy nodes.
	m_uNumberOfCollisionObjects = 0;
	m_pxCollisionObjects = new Asura_Physics_Ragdoll_Desc_CollisionObject[ uNumberOfHierarchyNodes ];

	m_uNumberOfJoints = 0;
	m_pxJoints = new Asura_Physics_Ragdoll_Desc_Joint[ uNumberOfHierarchyNodes ];
}
#endif

Asura_Physics_Ragdoll_Description::~Asura_Physics_Ragdoll_Description()
{
	delete [] m_pxCollisionObjects;
	delete [] m_pxHierarchyNodes;
	delete [] m_pxJoints;
}

const Asura_Physics_Ragdoll_Desc_HierarchyNode& Asura_Physics_Ragdoll_Description::GetHierarchyNode( u_int uIndex ) const 
{
	return m_pxHierarchyNodes[uIndex];
}

const Asura_Physics_Ragdoll_Desc_CollisionObject& Asura_Physics_Ragdoll_Description::GetCollisionObject( u_int uIndex ) const
{
	return m_pxCollisionObjects[uIndex];
}

const Asura_Physics_Ragdoll_Desc_Joint& Asura_Physics_Ragdoll_Description::GetJoint( u_int uIndex ) const
{
	return m_pxJoints[uIndex];
}

void Asura_Physics_Ragdoll_Description::ReadFromChunkStream( Asura_Chunk_Stream& xStream, int iVersion ) 
{
	if ( iVersion < 10 )
	{
		Asura_Assert( false, "Old ragdoll description - will skip" );	
		return;
	}

	xStream >> m_uNumberOfHierarchyNodes;
	m_pxHierarchyNodes = new Asura_Physics_Ragdoll_Desc_HierarchyNode[ m_uNumberOfHierarchyNodes ];
	
	for ( u_int u=0; u<m_uNumberOfHierarchyNodes; ++u )
	{
		m_pxHierarchyNodes[u].ReadFromChunkStream( xStream, iVersion );
	}

	xStream >> m_uNumberOfCollisionObjects;
#ifdef ASURA_TOOLS
	m_pxCollisionObjects = new Asura_Physics_Ragdoll_Desc_CollisionObject[ m_uNumberOfHierarchyNodes ];
#else
	m_pxCollisionObjects = new Asura_Physics_Ragdoll_Desc_CollisionObject[ m_uNumberOfCollisionObjects ];
#endif

	for ( u_int u=0; u<m_uNumberOfCollisionObjects; ++u )
	{
		m_pxCollisionObjects[u].ReadFromChunkStream( xStream, iVersion );
	}

	xStream >> m_uNumberOfJoints;
#ifdef ASURA_TOOLS
	m_pxJoints = new Asura_Physics_Ragdoll_Desc_Joint[ m_uNumberOfHierarchyNodes ];
#else
	m_pxJoints = new Asura_Physics_Ragdoll_Desc_Joint[ m_uNumberOfCollisionObjects ];
#endif
	for ( u_int u=0; u<m_uNumberOfJoints; ++u )
	{
		m_pxJoints[u].ReadFromChunkStream( xStream, iVersion );
	}

}

void Asura_Physics_Ragdoll_Description::WriteToChunkStream( Asura_Chunk_Stream& xStream) const
{
	xStream << m_uNumberOfHierarchyNodes;
	for ( u_int u=0; u<m_uNumberOfHierarchyNodes; ++u )
	{
		m_pxHierarchyNodes[u].WriteToChunkStream( xStream );
	}

	xStream << m_uNumberOfCollisionObjects;
	for ( u_int u=0; u<m_uNumberOfCollisionObjects; ++u )
	{
		m_pxCollisionObjects[u].WriteToChunkStream( xStream );
	}

	xStream << m_uNumberOfJoints;
	for ( u_int u=0; u<m_uNumberOfJoints; ++u )
	{
		m_pxJoints[u].WriteToChunkStream( xStream );
	}

}

void Asura_Physics_Ragdoll_Desc_HierarchyNode::ReadFromChunkStream( Asura_Chunk_Stream& xStream, int iVersion )
{
	xStream >> m_xRelativeOrientation;
	xStream >> m_xRelativePosition;
	xStream >> m_uCollisionObject;

	if ( iVersion < 15 )
	{
		m_uOverrideCollisionObject = Asura_Physics_Ragdoll_Description::uINVALID_INDEX;
	}
	else
	{
		xStream >> m_uOverrideCollisionObject;
	}
}

void Asura_Physics_Ragdoll_Desc_HierarchyNode::WriteToChunkStream( Asura_Chunk_Stream& xStream) const
{
	xStream << m_xRelativeOrientation;
	xStream << m_xRelativePosition;
	xStream << m_uCollisionObject;
	xStream << m_uOverrideCollisionObject;
}

void Asura_Physics_Ragdoll_Desc_CollisionObject::ReadFromChunkStream( Asura_Chunk_Stream& xStream, int iVersion )
{
	xStream >> m_xRelativeOrientation;
	xStream >> m_xRelativePosition;

	xStream >> m_xCollisionExtents;

	xStream >> m_uHierarchyNode;

	int iDummy;
	xStream >> iDummy;
	
	// TODO - is this cast safe on all platforms/compilers?
	m_eCollisionProxy = (Asura_Physics_Ragdoll_Desc_CollisionObject::COLLISION_PROXY)iDummy;

	if ( iVersion<14 )
	{
		m_fInvMass = 1.0f;
	}
	else
	{
		xStream >> m_fInvMass;
	}

	CalculateInvInertiaTensorDiagonal();
}

void Asura_Physics_Ragdoll_Desc_CollisionObject::WriteToChunkStream( Asura_Chunk_Stream& xStream) const
{
	xStream << m_xRelativeOrientation;
	xStream << m_xRelativePosition;

	xStream << m_xCollisionExtents;

	xStream << m_uHierarchyNode;
	xStream << (int)m_eCollisionProxy;

	xStream << m_fInvMass;
}

void Asura_Physics_Ragdoll_Desc_CollisionObject::CalculateInvInertiaTensorDiagonal( void )
{
	// We're going to get infinities in our inverse interial values if we allow tiny->zero extents.
	// Cap things to 1cm radius.
	const float fMIN_EXTENT = 0.01f; 

	float fInvMass = GetInvMass();

	switch ( m_eCollisionProxy )
	{
		case COLLISION_SPHERE:
		{
			// Completely symmetric so comes down to calculating a scalar value:
			// I = (2/5)(Mr*r)
			// 1/I = (5/2)*(1/M)*(1/(r*r))

			const float fSphereRadius = Asura_Maths::Max( m_xCollisionExtents.GetX(), fMIN_EXTENT );
			const float fSphereRadiusSquared = fSphereRadius * fSphereRadius;

			const float fInvInertia = 2.5f * fInvMass / fSphereRadiusSquared;
			
			m_xInvInertiaTensorDiagonal = Asura_AVector3( fInvInertia, fInvInertia, fInvInertia );
	
			break;
		}
		default: // default to the OBB, e.g. for polygonal meshes or sphere stacks
		case COLLISION_CAPSULE:
		{
			const float fMin = fMIN_EXTENT;

			const float fX = Asura_Maths::Max( m_xCollisionExtents.GetX(), fMin );
			const float fY = Asura_Maths::Max( m_xCollisionExtents.GetY(), fMin );
			const float fZ = Asura_Maths::Max( m_xCollisionExtents.GetZ(), fMin );
		
			const float fXSquared = fX*fX;
			const float fYSquared = fY*fY;
			const float fZSquared = fZ*fZ;

			// usually quoted with a 12.0f constant, but using width, height, depth - we're using a half-extents 
			// so there is a factor of (2.0f*2.0f) to divide out, which gives us a 3.0f.
			float fMassConst = 3.0f * fInvMass;

			float fInvIX = fMassConst / ( fYSquared + fZSquared );
			float fInvIY = fMassConst / ( fXSquared + fZSquared );
			float fInvIZ = fMassConst / ( fXSquared + fYSquared );

			m_xInvInertiaTensorDiagonal = Asura_AVector3( fInvIX, fInvIY, fInvIZ );

			break;
		}

	}
}

void Asura_Physics_Ragdoll_Desc_Joint::ReadFromChunkStream( Asura_Chunk_Stream& xStream, int iVersion )
{
	if ( iVersion<12 )
	{
		m_xJointOrientation.Identity();
	}
	else
	{
		xStream >> m_xJointOrientation;
	}
	if ( iVersion<13 )
	{
		m_xOriginalJointOrientation = m_xJointOrientation;
	}
	else
	{
		xStream >> m_xOriginalJointOrientation;
	}

	xStream >> m_xJointPosRelativeToA;
	xStream >> m_xJointPosRelativeToB;

	xStream >> m_uObjectA;
	xStream >> m_uObjectB;

	xStream >> m_uHierarchyNode; 	

	if ( iVersion<11 ) // prior to angular constraints
	{
		m_xJointSpaceRelativeToA.Identity();
		m_xJointSpaceRelativeToB.Identity();

		m_bUseAngularConstraint = true;
		m_fTwistLimit = 0.0f;
		m_fSwingLimitX = 0.0f;
		m_fSwingLimitY = 0.0f;
	}
	else
	{
		xStream >> m_xJointSpaceRelativeToA;
		xStream >> m_xJointSpaceRelativeToB;

		xStream >> m_bUseAngularConstraint;
		xStream >> m_fTwistLimit;
		xStream >> m_fSwingLimitX;
		xStream >> m_fSwingLimitY;
	}
}

void Asura_Physics_Ragdoll_Desc_Joint::WriteToChunkStream( Asura_Chunk_Stream& xStream) const
{
	xStream << m_xJointOrientation;
	xStream << m_xOriginalJointOrientation;

	xStream << m_xJointPosRelativeToA;
	xStream << m_xJointPosRelativeToB;

	xStream << m_uObjectA;
	xStream << m_uObjectB;

	xStream << m_uHierarchyNode; 

	xStream << m_xJointSpaceRelativeToA;
	xStream << m_xJointSpaceRelativeToB;

	xStream << m_bUseAngularConstraint;
	xStream << m_fTwistLimit;
	xStream << m_fSwingLimitX;
	xStream << m_fSwingLimitY;
}
#if defined (ASURA_TOOLS)

// TODO - for clarity move this to the top when older system is removed
#include "Ragdoll/Tools_RagdollCollision_Object.h"
#include "Ragdoll/Tools_RagdollJoint_Object.h"

// non-const accessors for Tools write-access
Asura_Physics_Ragdoll_Desc_HierarchyNode* Asura_Physics_Ragdoll_Description::Tools_GetHierarchyNode( u_int uIndex )  
{
	return &m_pxHierarchyNodes[uIndex];
}

Asura_Physics_Ragdoll_Desc_CollisionObject* Asura_Physics_Ragdoll_Description::Tools_GetCollisionObject( u_int uIndex ) 
{
	return &m_pxCollisionObjects[uIndex];
}

Asura_Physics_Ragdoll_Desc_Joint* Asura_Physics_Ragdoll_Description::Tools_GetJoint( u_int uIndex )
{
	return &m_pxJoints[uIndex];
}


// Methods for adding new elements to the ragdoll description on the fly
Asura_Physics_Ragdoll_Desc_CollisionObject* Asura_Physics_Ragdoll_Description::Tools_AddCollisionObject( void )
{
	// KJFL - 23 October 2008 - Check to see if we run out of space. 
	// If this is happening and people are complaining, then my assumption that we won't want more
	// collision objects than there are hierarchy nodes in a model is flawed, and we should change
	// this. e.g. We could simply realloc and copy every time if over this limit.
	if ( m_uNumberOfCollisionObjects == m_uNumberOfHierarchyNodes )
	{
		return NULL;
	}

	return &(m_pxCollisionObjects[m_uNumberOfCollisionObjects++]);
}

Asura_Physics_Ragdoll_Desc_Joint* Asura_Physics_Ragdoll_Description::Tools_AddJoint( void )
{
	if ( m_uNumberOfJoints == m_uNumberOfHierarchyNodes )
	{
		return NULL;
	}

	return &(m_pxJoints[m_uNumberOfJoints++]);
}


// Methods for removing elements from the ragdoll description on the fly.
void Asura_Physics_Ragdoll_Description::Tools_RemoveCollisionObject( Asura_Physics_Ragdoll_Desc_CollisionObject* pxObjectToRemove )
{
	u_int uIndex = Asura_Physics_Ragdoll_Description::uINVALID_INDEX;

	for (u_int u=0; u<m_uNumberOfCollisionObjects; ++u)
	{
		if (&(m_pxCollisionObjects[u]) == pxObjectToRemove)
		{
			uIndex = u;
		}
	}

	if ( uIndex == Asura_Physics_Ragdoll_Description::uINVALID_INDEX )
	{
		Asura_Assert( false, "Code bug: invalid collision object pointer" );
		return;
	}

	// Destroy the collision object by copying the last one in the array over it, and then
	// reducing the number of collision objects by one.
	m_pxCollisionObjects[ uIndex ] = m_pxCollisionObjects[ --m_uNumberOfCollisionObjects ];

	// scan through all joints and fix up any collision index ids.
	for ( u_int uJoint=0; uJoint<m_uNumberOfJoints; ++uJoint )
	{
		Asura_Physics_Ragdoll_Desc_Joint& xJoint = m_pxJoints[uJoint];

		// Anything refering to the "uIndex" collision object now needs to refer to "invalid",
		// and anything refering to what was the last collision object now needs to refer to "uIndex".

		if ( xJoint.GetObjectA() == uIndex )
		{
			xJoint.SetObjectA( uINVALID_INDEX );
		}
		else if ( xJoint.GetObjectA() == m_uNumberOfCollisionObjects )
		{
			xJoint.SetObjectA( uIndex );
		}
		
		if ( xJoint.GetObjectB() == uIndex )
		{
			xJoint.SetObjectB( uINVALID_INDEX );
		}
		else if ( xJoint.GetObjectB() == m_uNumberOfCollisionObjects )
		{
			xJoint.SetObjectB( uIndex );
		}
	}

	// scan through the Tools collision object - we have to update any pointers to the one we've moved
	Asura_List_It<Tools_RagdollCollision_Object> xIt( Tools_RagdollCollision_Object::GetList() );

	for ( ; !xIt.Done() ; xIt.Next() )
	{
		Tools_RagdollCollision_Object* pxToolsObject = xIt.GetCurrent();

		Asura_Physics_Ragdoll_Desc_CollisionObject* pxCollisionObject = pxToolsObject->GetCollisionObject();

		if ( pxCollisionObject == &(m_pxCollisionObjects[m_uNumberOfCollisionObjects]))
		{
			pxToolsObject->SetCollisionObject( pxObjectToRemove, uIndex );
		}
	}

	// scan through all hierarchy nodes and fix up any collision index ids.
	for ( u_int uNode=0; uNode<m_uNumberOfHierarchyNodes; ++uNode )
	{
		Asura_Physics_Ragdoll_Desc_HierarchyNode& xNode = m_pxHierarchyNodes[uNode];

		// Anything refering to the "uIndex" collision object now needs to refer to "invalid",
		// and anything refering to what was the last collision object now needs to refer to "uIndex".

		if ( xNode.Tools_GetOverrideCollisionObject() == uIndex )
		{
			xNode.Tools_SetOverrideCollisionObject( uINVALID_INDEX );
		}
		else if ( xNode.Tools_GetOverrideCollisionObject() == m_uNumberOfCollisionObjects )
		{
			xNode.Tools_SetOverrideCollisionObject( uIndex );
		}
	}
}

void Asura_Physics_Ragdoll_Description::Tools_RemoveJoint( Asura_Physics_Ragdoll_Desc_Joint* pxJointToRemove )
{
	u_int uIndex = Asura_Physics_Ragdoll_Description::uINVALID_INDEX;

	for (u_int u=0; u<m_uNumberOfJoints; ++u)
	{
		if (&(m_pxJoints[u]) == pxJointToRemove)
		{
			uIndex = u;
		}
	}

	if ( uIndex == Asura_Physics_Ragdoll_Description::uINVALID_INDEX )
	{
		Asura_Assert( false, "Code bug: invalid joint pointer" );
		return;
	}

	// Destroy the collision object by copying the last one in the array over it, and then
	// reducing the number of collision objects by one.
	m_pxJoints[ uIndex ] = m_pxJoints[ --m_uNumberOfJoints ];

	// scan through the Tools joint objects - we have to update any pointers to the one we've moved
	Asura_List_It<Tools_RagdollJoint_Object> xIt( Tools_RagdollJoint_Object::GetList() );

	for ( ; !xIt.Done() ; xIt.Next() )
	{
		Tools_RagdollJoint_Object* pxToolsObject = xIt.GetCurrent();

		Asura_Physics_Ragdoll_Desc_Joint* pxJoint = pxToolsObject->GetJoint();

		if ( pxJoint == &(m_pxJoints[m_uNumberOfJoints]))
		{
			pxToolsObject->SetJoint(pxJointToRemove);
		}
	}
}

#endif // defined (ASURA_TOOLS)


