//
// Asura_Physics_RestFrame.cpp
//
// Created by KJFL - 13 October 2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"


#include "Physics/Asura_Physics_RestFrame.h"

#include "Debugging/Asura_Assert.h"
#include "Physics/Asura_Physics_RigidBody.h"

Asura_Physics_RestFrame* Asura_Physics_RestFrame::s_pxHeadOfList = 0;

void Asura_Physics_RestFrame::Initialise( void )
{
	Asura_Assert( s_pxHeadOfList == 0, "Code bug: Already been Initialised?" );
	s_pxHeadOfList = 0;
}

void Asura_Physics_RestFrame::Reset( void )
{
	//Asura_Assert( !s_pxHeadOfList, "Code bug: There are rest frames still allocated" );

	// if there are any rest frames still allocated, clear them up
	Asura_Physics_RestFrame* pxRestFrame = s_pxHeadOfList;

	while ( pxRestFrame )
	{
		Asura_Physics_RestFrame* pxNextRestFrame = pxRestFrame->m_pxNext;

		delete pxRestFrame;

		pxRestFrame = pxNextRestFrame;
	}

	s_pxHeadOfList = 0;
}

void Asura_Physics_RestFrame::Shutdown( void )
{
	Reset();
}

void Asura_Physics_RestFrame::UpdateAll( void )
{
	Asura_Physics_RestFrame** ppxNextRestFrame = &s_pxHeadOfList;

	while ( *ppxNextRestFrame )
	{
		Asura_Physics_RestFrame* pxRestFrame = *ppxNextRestFrame;

		if ( pxRestFrame->IsValid() )
		{
			pxRestFrame->Update();
			ppxNextRestFrame = &(pxRestFrame->m_pxNext);
		}
		else
		{
			if ( !pxRestFrame->IsUsed() )
			{
				// time to delete it
				*ppxNextRestFrame = pxRestFrame->m_pxNext;
				delete pxRestFrame;
			}
			else
			{
				ppxNextRestFrame = &(pxRestFrame->m_pxNext);
			}
		}
	}
}

// Create / Destroy a single RestFrame instance
Asura_Physics_RestFrame* Asura_Physics_RestFrame::Create( const Asura_Physics_RigidBodyInstance* pxParent )
{
	Asura_Physics_RestFrame* pxFrame = new Asura_Physics_RestFrame;

	if ( !pxFrame )
	{
		Asura_Assert( false, "Allocation failed" );
		return 0;
	}

	// add new RestFrame to head of list
	pxFrame->m_pxNext = s_pxHeadOfList;
	s_pxHeadOfList = pxFrame;

	pxFrame->m_uReferenceCount = 0;

	pxFrame->m_pxParentObject = pxParent;

	pxFrame->m_xLastPosition = pxParent->GetCentreOfMass();
	pxFrame->m_xLastOrientationTransposed = pxParent->GetOrientation();
	pxFrame->m_xLastOrientationTransposed.Transpose();

	return pxFrame;
}

void Asura_Physics_RestFrame::Destroy( Asura_Physics_RestFrame* pxRestFrame )
{
	// This doesn't actually destroy the object, just flag it as being without a parent.
	if (pxRestFrame)
	{
		pxRestFrame->m_pxParentObject = 0;
	}
}

void Asura_Physics_RestFrame::Link( void )
{
	++m_uReferenceCount;
}

void Asura_Physics_RestFrame::Unlink( void )
{
	if (m_uReferenceCount)
	{
		--m_uReferenceCount;
	}
	else
	{
		Asura_Assert( false, "Code bug: reference count already zero" );
	}
}

// Set the RestFrame's parent, from which it will take its movement
void Asura_Physics_RestFrame::SetParent( const Asura_Physics_RigidBodyInstance* pxParent )
{
	m_pxParentObject = pxParent;
}


void Asura_Physics_RestFrame::TransformPosition( Asura_AVector3& xPosition ) const
{
	xPosition -= m_xLastPosition;

	xPosition = xPosition * m_xLastOrientationTransposed;

	xPosition = xPosition * m_xOrientation;
	xPosition += m_xPosition;
}

void Asura_Physics_RestFrame::TransformOrientation( Asura_AMatrix3x3& xOrient ) const
{
	xOrient = (xOrient * m_xLastOrientationTransposed)*m_xOrientation;
}


void Asura_Physics_RestFrame::Update( void )
{
	m_xLastPosition = m_xPosition;
	m_xPosition = m_pxParentObject->GetCentreOfMass();


	m_xLastOrientationTransposed = m_xOrientation;
	m_xLastOrientationTransposed.Transpose();

	m_xOrientation = m_pxParentObject->GetOrientation();

}

