//
// Asura_Physics_RigidBodyDefinition.h
//
// Created by KJFL - 2004-03-22 10:50:34
//
// Copyright (C) 2004 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif

#ifndef __ASURA_PHYSICS_RIGIDBODYDEFINITION_H__
#define __ASURA_PHYSICS_RIGIDBODYDEFINITION_H__

#include "Core/Asura_Maths.h"

class Asura_Collision_Object;
class Asura_CollisionMesh;

class Asura_Physics_RigidBody_ObjectEdge
{
	public:

		inline const Asura_AVector3& GetNormal( void ) const
		{
			return m_xNormal;
		}
		
		u_int GetFirstVertexIndex( void ) const
		{
			return m_FirstVertexIndex;
		}
		
		u_int GetSecondVertexIndex( void ) const
		{
			return m_SecondVertexIndex;
		}

	private:

		friend class Asura_Physics_RigidBody_ObjectMesh;

		Asura_AVector3	m_xNormal;
		u_short			m_FirstVertexIndex;
		u_short			m_SecondVertexIndex;
};


class Asura_Physics_RigidBody_ObjectMesh
{
	public:

		// creation methods (new is private)
		
		static Asura_Physics_RigidBody_ObjectMesh* Create( void );

		static Asura_Physics_RigidBody_ObjectMesh* CreateCuboid( float fWidth, float fHeight, float fDepth );

		static Asura_Physics_RigidBody_ObjectMesh* CreateCuboid( const Asura_Bounding_Box& xBB );

		static Asura_Physics_RigidBody_ObjectMesh* CreateHexahedron( float fWidth, float fHeight );

		static void Destroy( Asura_Physics_RigidBody_ObjectMesh* pxMesh)
		{
			delete [] pxMesh->m_pxVertexPositions;
			delete [] pxMesh->m_pxVertexNormals;
	
			delete [] pxMesh->m_pxEdges;
			delete [] pxMesh->m_pxFaceNormals;
			delete [] pxMesh->m_pusFaceVertexIndices;


			delete pxMesh;
		}

		// Vertex access

		inline u_int GetNumberOfVertices( void ) const
		{
			return m_uNumberOfVertices;
		}

		inline const Asura_AVector3& GetVertexPosition( u_int uVertex ) const
		{
			return m_pxVertexPositions[uVertex];
		}

		inline const Asura_AVector3& GetVertexNormal( u_int uVertex ) const
		{
			return m_pxVertexNormals[uVertex];
		}

		// Edge access

		inline u_int GetNumberOfEdges( void ) const
		{
			return m_uNumberOfEdges;
		}

		inline const Asura_Physics_RigidBody_ObjectEdge& GetEdge( u_int uEdge ) const
		{
			return m_pxEdges[uEdge];
		}

		// Face access

		inline u_int GetNumberOfFaces( void ) const
		{
			return m_uNumberOfFaces;
		}
		
		inline u_int GetVertexIndexOnFace( u_int uFace, u_int uVertex ) const
		{
			return m_pusFaceVertexIndices[ 3*uFace + uVertex ];
		}

		inline const Asura_AVector3& GetFaceNormal( u_int uFace ) const
		{
			return m_pxFaceNormals[uFace];
		}

		u_int								m_uNumberOfVertices;
		Asura_AVector3*						m_pxVertexPositions;
		Asura_AVector3*						m_pxVertexNormals;

		u_int								m_uNumberOfEdges;
		Asura_Physics_RigidBody_ObjectEdge*	m_pxEdges;

		u_int								m_uNumberOfFaces;
		Asura_AVector3*						m_pxFaceNormals;
		u_short*							m_pusFaceVertexIndices;

	private:
		//! private constructor so people can't just call new without
		//! going through the system
		Asura_Physics_RigidBody_ObjectMesh(){};
};



class Asura_Physics_RigidBodyDefinition
{
	public:

		float GetOBBHalfExtentsX( void ) const { return m_fOBBHalfExtentsX; }
		float GetOBBHalfExtentsY( void ) const { return m_fOBBHalfExtentsY; }
		float GetOBBHalfExtentsZ( void ) const { return m_fOBBHalfExtentsZ; }

		// return the minimum radius of the cuboid, i.e. the smallest half-extent
		float GetMinimumRadius( void ) const
		{
			float fMinRadius = GetOBBHalfExtentsX();
			fMinRadius = Asura_Maths::Min( fMinRadius, GetOBBHalfExtentsY() );
			fMinRadius = Asura_Maths::Min( fMinRadius, GetOBBHalfExtentsZ() );

			return fMinRadius;
		}

		// return the maximum radius of the cuboid, i.e. the largest half-extent
		float GetMaximumRadius( void ) const
		{
			float fMaxRadius = GetOBBHalfExtentsX();
			fMaxRadius = Asura_Maths::Max( fMaxRadius, GetOBBHalfExtentsY() );
			fMaxRadius = Asura_Maths::Max( fMaxRadius, GetOBBHalfExtentsZ() );

			return fMaxRadius;
		}

		void GetSphereStackInfo( Asura_AVector3& xAxis, float& fRadius ) const
		{
			if ( GetOBBHalfExtentsX() > GetOBBHalfExtentsY() )
			{
				if ( GetOBBHalfExtentsX() > GetOBBHalfExtentsZ() )
				{
					// it's X
					xAxis = Asura_AVector3( GetOBBHalfExtentsX(), 0.0f, 0.0f );
					fRadius = Asura_Maths::Max( GetOBBHalfExtentsY(), GetOBBHalfExtentsZ() );
				}
				else // it's Z
				{
					xAxis = Asura_AVector3( 0.0f, 0.0f, GetOBBHalfExtentsZ()  );
					fRadius = Asura_Maths::Max( GetOBBHalfExtentsX(), GetOBBHalfExtentsY() );
				}
			}
			else if ( GetOBBHalfExtentsY() > GetOBBHalfExtentsZ() )
			{
				// it's Y
				xAxis = Asura_AVector3( 0.0f, GetOBBHalfExtentsY(), 0.0f  );
				fRadius = Asura_Maths::Max( GetOBBHalfExtentsX(), GetOBBHalfExtentsZ() );
			}
			else // it's Z
			{
				xAxis = Asura_AVector3( 0.0f, 0.0f, GetOBBHalfExtentsZ()  );
				fRadius = Asura_Maths::Max( GetOBBHalfExtentsX(), GetOBBHalfExtentsY() );
			}
		}

		const Asura_AVector3& GetOBBCentreOffset( void ) const
		{
			return m_xOBBCentreOffset;
		}

		bool HasCollisionMesh( void ) const { return m_pxCollisionMesh!=0; }

		const Asura_CollisionMesh* GetCollisionMesh( void ) const { return m_pxCollisionMesh; }


//	private:
		Asura_Physics_RigidBodyDefinition(){};

		Asura_Physics_RigidBodyDefinition( const Asura_Collision_Object* pxCollisionObject );
			
		~Asura_Physics_RigidBodyDefinition()
		{
			Asura_Physics_RigidBody_ObjectMesh::Destroy(m_pxObjectMesh);
		}

		Asura_Physics_RigidBody_ObjectMesh*		m_pxObjectMesh;

	private:



		// KJFL - 26 June 2006 -  Half-Extents of the unrotated bounding box of the object.
		// Combined with the orientation matrix of a particular instance this gives an
		// effective OBB definition in a form friendly to various algoritms (e.g. SAT tests).
		// Not entirely convinced this is the best place for them (inside _ObjectMesh?) as
		// part of the instance itself (in an OBB class?) but in theory multiple instances
		// can use the same rigid body definition.
		float m_fOBBHalfExtentsX;
		float m_fOBBHalfExtentsY;
		float m_fOBBHalfExtentsZ;

		Asura_AVector3 m_xOBBCentreOffset;

		const Asura_CollisionMesh*	m_pxCollisionMesh;

		void CalculateOBBHalfExtents( const Asura_Bounding_Box& xBB );
};

#endif//__ASURA_PHYSICS_RIGIDBODYDEFINITION_H__
