//
// Asura_Physics_Constraint.h
//
// Created by KJFL - 4 July 2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __ASURA_PHYSICS_CONSTRAINT_H__
#define __ASURA_PHYSICS_CONSTRAINT_H__

#ifdef ASURA_PHYSICS_SUPPORT_CONSTRAINTS

class Asura_Physics_RigidBodyInstance;

// KJFL - 5 July 2007 = I'd originally considered a pretty empty base class and then deriving two more
// virtual classes, one for single-body constraints and one for two-body constraints. However, this
// would have added a lot of complexity and virtual table overhead, and it seemed the lesser evil
// to store two body pointers for all constraints. Single-body constraints will all have the second
// body pointer set to null.
//
// Intentionally doesn't use a mempool since people will be creating derived versions of the class.
class Asura_Physics_Constraint
{
	public:

		Asura_Physics_Constraint( Asura_Physics_RigidBodyInstance* pxBody1, Asura_Physics_RigidBodyInstance* pxBody2 );

		virtual ~Asura_Physics_Constraint();
		
		// called during the Physics update for the bodies in question
		virtual void ResolveConstraint( void ) = 0;

		// Will be zero if rigid body has been destroyed.
		Asura_Physics_RigidBodyInstance* GetPrimaryBody( void ) const { return m_pxBody1; }

		// Will be zero if constraint is single body or if a body has been destroyed.
		Asura_Physics_RigidBodyInstance* GetSecondaryBody( void ) const { return m_pxBody2; }

		// Reset the body pointers and invalidate the constraint: called when one
		// of the bodies in question is destroyed and the pointer becomes invalid.
		void RemoveBody( Asura_Physics_RigidBodyInstance* pxBody );

	protected: 

		Asura_Physics_RigidBodyInstance*	m_pxBody1;
		Asura_Physics_RigidBodyInstance*	m_pxBody2;
};

// TODO - probably safe to have a mempool for the nodes since the size if fixed
class Asura_Physics_Constraint_Node
{
	public:

	//	static Asura_Physics_Constraint_Node* Create()

		Asura_Physics_Constraint_Node* GetNextNode( void ) { return m_pxNext; }
		Asura_Physics_Constraint* GetConstraint( void ) { return m_pxConstaint; }

	private: friend class Asura_Physics_Constraint_List;

		Asura_Physics_Constraint_Node*	m_pxNext;
		Asura_Physics_Constraint*		m_pxConstaint;
};

// KJFL - very likely to change internally; just want a light-weight wrapper for the constraints atm
// Overhead is just 4 bytes if empty. Will see if this will make a templatised class.
class Asura_Physics_Constraint_List
{
	public:
		Asura_Physics_Constraint_List() : m_pxHeadOfList(0) {}
		~Asura_Physics_Constraint_List()
		{
			ClearNodes();
		}

		void Reset( void )
		{
			ClearNodes();
		}

		Asura_Physics_Constraint_Node* GetList( void ) { return m_pxHeadOfList; }

		void Add( Asura_Physics_Constraint* pxConstraint );
		
		void Remove( Asura_Physics_Constraint* pxConstraint );
	

	private:

		void ClearNodes( void );
	
		Asura_Physics_Constraint_Node*	m_pxHeadOfList;
};
#endif

#endif//__ASURA_PHYSICS_CONSTRAINT_H__

