//
// Asura_Physics_Constraint_WorldPosition.cpp
//
// Created by KJFL - 4 July 2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//

#ifdef ASURA_PHYSICS_SUPPORT_CONSTRAINTS

#include "Core/Asura.h"

// This file's header.
#include "Physics/Constraints/Asura_Physics_Constraint_WorldPosition.h"
#include "Physics/Asura_Physics_RigidBody.h"

//#include "Debugging/Asura_DebugText.h"

Asura_Physics_Constraint_WorldPosition::Asura_Physics_Constraint_WorldPosition( const Asura_AVector3& xPositionOnBody, const Asura_AVector3& xWorldPosition, Asura_Physics_RigidBodyInstance* pxBody )
: Asura_Physics_Constraint( pxBody, 0 )
, m_xPositionOnBody( xPositionOnBody )
, m_xWorldPosition( xWorldPosition )
{
}

void Asura_Physics_Constraint_WorldPosition::ResolveConstraint( void )
{
	// must always check the body pointer is valid first
	if (!m_pxBody1) return;

	const Asura_AVector3 xRel = m_xPositionOnBody * m_pxBody1->GetOrientation();

	Asura_AVector3 xVelocity;

	m_pxBody1->CalculateVelocityOfRelativePointOnBody( xRel, xVelocity );

	// Modify velocity of relative point so it is zero when position constraint is satisfied, and otherwise is such that
	// it moves the relative point back to the world position over successive updates.

	Asura_AVector3 xDisplacement = m_xWorldPosition - (xRel + m_pxBody1->GetCentreOfMass());

	const float fDistance = xDisplacement.NormaliseReturningMagnitude();

	const float fALLOWED_DISTANCE_THRESHOLD = 0.01f; // a cm

	Asura_AVector3 xTargetVelocity;

//	Asura_DebugText::AddTextBlock( 10,8,0xffffffff, 0, "Disp: %f mm", fDistance*1000.0f);

	if ( fDistance > fALLOWED_DISTANCE_THRESHOLD )
	{
		xTargetVelocity = xDisplacement * ( fDistance - fALLOWED_DISTANCE_THRESHOLD ) / 4.0f;
	}
	else
	{
		xTargetVelocity.Zero();	
	}

	// calculate impulse required to change point's velocity to the target value
	Asura_AVector3 xNormal = xTargetVelocity - xVelocity;

	const float fImpulse = xNormal.NormaliseReturningMagnitude();

	if ( fImpulse > 0.0f )
	{
		Asura_AVector3 xDenom = xRel^xNormal;

		xDenom = xDenom * m_pxBody1->GetInvInertiaTensorDiagonal();

		xDenom = xDenom ^ xRel;

		const float fDenom = m_pxBody1->GetInvMass() + xDenom * xNormal;

		Asura_AVector3 xImpulse = xNormal * ( fImpulse / fDenom );

		m_pxBody1->ApplyImpulseAtRelativePosition( xRel, xImpulse );
	}
}

#endif


