//
// Asura_Physics_ForceField.cpp
//
// Created by KJFL - 17 November 2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

// This file's header.
#include "Physics/ForceFields/Asura_Physics_ForceField.h"

#include "Core/Asura_HashedList.h"

static u_int										g_uNextGuid = 0;
static Asura_Hashed_List<Asura_Physics_ForceField>	g_xForceFieldList;


// constructor merely gets a guid and adds itself to the list
Asura_Physics_ForceField::Asura_Physics_ForceField()
{
	m_uGuid = g_uNextGuid++;
	g_xForceFieldList.AddToList( this );
	m_bIsActive = true;
	m_bIsDestroyed = false;
}

// destructor must remove instance from list
Asura_Physics_ForceField::~Asura_Physics_ForceField()
{
	g_xForceFieldList.RemoveFromList( this );
}

void Asura_Physics_ForceField::UpdateAll( void )
{
	Asura_Hashed_List_Deleter_It<Asura_Physics_ForceField> xIterator( &g_xForceFieldList );

	while (!xIterator.Done())
	{
		Asura_Physics_ForceField* pxForceField = xIterator.GetCurrent();

		Asura_Assert( pxForceField, "Code bug: invalid force field pointer" );

		if ( pxForceField->IsActive() )
		{
			pxForceField->Update();
		}

		// KJFL - 18 April 2007 - I've been through the Asura_Hash_List and Asura_Hashed_List_Deleter_It
		// code extremely carefully, and I should be able to call Next() on the iterator and then delete
		// the object - which will then call RemoveFromList().
		xIterator.Next();
	
		if ( pxForceField->IsDestroyed() )
		{
			delete pxForceField;
		}
	}
}

void Asura_Physics_ForceField::ResetAll( void )
{
    Asura_Hashed_List_Deleter_It<Asura_Physics_ForceField> xIterator( &g_xForceFieldList );

	while (!xIterator.Done())
	{
		Asura_Physics_ForceField* pxForceField = xIterator.GetCurrent();

		// KJFL - 18 April 2007 - I've been through the Asura_Hash_List and Asura_Hashed_List_Deleter_It
		// code extremely carefully, and I should be able to call Next() on the iterator and then delete
		// the object - which will then call RemoveFromList().
		xIterator.Next();

		delete pxForceField;
	}

	g_uNextGuid = 0;
}


void Asura_Physics_ForceField::DebugRenderAll( void )
{
	Asura_Hashed_List_Deleter_It<Asura_Physics_ForceField> xIterator( &g_xForceFieldList );

	while (!xIterator.Done())
	{
		Asura_Physics_ForceField* pxForceField = xIterator.GetCurrent();

		pxForceField->DebugRender();

		xIterator.Next();
	}

}
