//
// Asura_Physics_Vehicle.h
//
// Created by KJFL - 14 September 2005
//
// Copyright (C) 2005 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __ASURA_PHYSICS_VEHICLE_H__
#define __ASURA_PHYSICS_VEHICLE_H__

#include "Core/Asura_HashedList.h"

class Asura_Physics_Vehicle
{
	public:

		// constructor merely gets a guid and adds itself to the list
		Asura_Physics_Vehicle()
		{
			m_uGuid = s_uNextGuid++;
			s_xVehicleList.AddToList( this );
		}

		// destructor must remove instance from list
		virtual ~Asura_Physics_Vehicle()
		{
			s_xVehicleList.RemoveFromList( this );
		}

		// Functions required in order to use Asura_Hashed_List;
		// may wish to make more use of IsDestroyed() later.
		u_int GetGuid() const { return m_uGuid; }
		bool IsDestroyed() const { return false; }


		// virtual functions derived classes will most likely want to implement
		virtual void Update( void ) {}
		virtual void DebuggingRender( void ) {}


	protected:

	

	private: 
			 
		// KJFL - 14 September 2005 - Guid of this instance, private so nothing can accidentally 
		// change it, including derived classes. 
		u_int m_uGuid;

		// Static data to hold and maintain list of all vehicles
		static u_int									s_uNextGuid;
     	static Asura_Hashed_List<Asura_Physics_Vehicle> s_xVehicleList;

	public:
		// Static method to access vehicle list - used by Asura_Physics_System
		// KJFL - 14 September 2005 - Seemed to make more sense than making Asura_Physics_System 
		// a friend of this class, but I guess other classes could be naughty and modify the list etc.
		// may revert to using friend if required.
		static Asura_Hashed_List<Asura_Physics_Vehicle>& GetVehicleList() { return s_xVehicleList; }

};

#endif//__ASURA_PHYSICS_VEHICLE_H__
