//*******************************************************************
//
// File: Asura_DIPCache.h           
//
// Created: 7 Apr 2006
//
// Description: 
//
//*******************************************************************

#ifndef _Asura_DIPCache_H_
#define _Asura_DIPCache_H_

// other libraries

// internal dependencies

// forward declarations
class Asura_DIPCache;

//*******************************************************************
// Constants
//*******************************************************************


//*******************************************************************
// Class Definitions
//*******************************************************************
class Asura_DIPCache_DIPData
{
public:
	void Set( const uint32 uIndexBufferStart, const uint32 uIndexBufferStop, const uint32 uVertexBufferStart, const uint32 uVertexBufferStop, const uint32 uVertexBufferOffset = 0);
	
	uint32 GetIndexBufferStart() const { return( this->m_uIndexBufferStart);}
	uint32 GetIndexBufferStop() const { return( this->m_uIndexBufferStop);}
	uint32 GetVertexBufferStart() const { return( this->m_uVertexBufferStart);}
	uint32 GetVertexBufferStop() const { return( this->m_uVertexBufferStop);}
	uint32 GetVertexBufferOffset() const { return( this->m_uVertexBufferOffset);}
	
private:
	uint32 m_uVertexBufferOffset;
	uint32 m_uVertexBufferStart;
	uint32 m_uVertexBufferStop;
	uint32 m_uIndexBufferStart;
	uint32 m_uIndexBufferStop;
	
	friend class Asura_DIPCache;
};

class Asura_DIPCache_Batch
{
public:
	Asura_DIPCache_Batch* m_pxLower;
	Asura_DIPCache_Batch* m_pxHigher;
	
	Asura_DIPCache_DIPData m_xData;
};

class Asura_DIPCache
{
protected:
	typedef bool (*Render_Callback)(const Asura_DIPCache_DIPData& xData, const uint32 uCallbackData);
	
	Asura_DIPCache( void);
	Asura_DIPCache( const Render_Callback pfnCallback, const uint32 uCallbackData);
	
	bool AddBufferPointers( Asura_DIPCache_Batch* pxBuffers, Asura_DIPCache_Batch** ppxBuffers, const uint32 uBufferSize);
	
public:
	void Clear();

	bool AddDIP( const Asura_DIPCache_DIPData& xData);
	bool Render();
	
	bool SetCallback( const Render_Callback pfnCallback, const uint32 uCallbackData);
	
	bool IsEmpty() const;
	
private:
	
private:
	Asura_DIPCache_Batch* GetNextBatch();
	
	uint32 m_uBufferMaxCount;
	uint32 m_uBufferUsedCount;
	
	Asura_DIPCache_Batch* m_pxBufferStart;
	Asura_DIPCache_Batch* m_pxBatchBuffers;
	Asura_DIPCache_Batch** m_ppxBatchBuffers;
	
	Render_Callback m_pfnCallback;
	uint32 m_uCallbackData;
};

template< uint32 uBufferSize> class Asura_DIPCache_Static : public Asura_DIPCache
{
public:
	Asura_DIPCache_Static() : Asura_DIPCache()
	{
		this->AddBufferPointers( this->m_axBuffer, this->m_pxBuffer, uBufferSize);
	}

	Asura_DIPCache_Static( const Render_Callback pfnCallback, const uint32 uCallbackData) : Asura_DIPCache( pfnCallback, uCallbackData)
	{
		this->AddBufferPointers( this->m_axBuffer, this->m_pxBuffer, uBufferSize);
	}
	
private:
	Asura_DIPCache_Batch m_axBuffer[ uBufferSize];
	Asura_DIPCache_Batch* m_pxBuffer[ uBufferSize];
};

//*******************************************************************
// Inline Function Definitions
//*******************************************************************

inline void Asura_DIPCache_DIPData::Set( const uint32 uIndexBufferStart, const uint32 uIndexBufferStop, const uint32 uVertexBufferStart, const uint32 uVertexBufferStop, const uint32 uVertexBufferOffset)
{
	this->m_uVertexBufferOffset = uVertexBufferOffset;
	this->m_uVertexBufferStart = uVertexBufferStart;
	this->m_uVertexBufferStop = uVertexBufferStop;
	this->m_uIndexBufferStart = uIndexBufferStart;
	this->m_uIndexBufferStop = uIndexBufferStop;
}

//*******************************************************************
//
// (c) 2006 Rebellion (Derby)
//
// Warning: This computer program is protected by copyright law and 
// international treaties. Unauthorized reproduction or distribution
// of this program, or any portion of it, may result in severe civil and
// criminal penalties, and will be prosecuted to the maximum extent 
// possible under the law.
//
//*******************************************************************

#endif // _Asura_DIPCache_H_
