/**
*
* \file Asura_Font_System.cpp
*
* Created by James Paulin - 18/07/2002 11:43:10
*
* Copyright (C) 2002 Rebellion, All Rights Reserved.
*
* \brief
*
**/

#include "core/Asura.h"

#include "rendering/Asura_Font_System.h"

	

#ifndef _FONTMAKER
	#include "Rendering/Asura_Viewport.h"
	#include "Core/Asura_TextureManagement.h"
    #include "Camera/Asura_Camera.h"
#endif

#include <stdarg.h>
#include <stdio.h>
#include "Core/Asura_LocalisedText.h"
#include "core/Asura_Maths.h"
#include "core/Asura_Handle.h"
#include "core/Asura_String.h"
#include "core/Asura_Timers.h"
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "Rendering/Asura_Colour.h"
#include "rendering/Asura_Render.h"
#include "rendering/Asura_DisplayLimits.h"

#include <wchar.h>

//*GLOBAL VARIABLES***************************

int	 Asura_Font_System::s_iCurrentFontID = 0;
bool Asura_Font_System::s_bAnyFontHasLoaded = false;

Asura_Font* 		Asura_Font_System::s_apxFontStorage[ASURA_MAX_FONTS];
float				Asura_Font_System::s_fLastPrintedYHeight = 0.0f;
float				Asura_Font_System::s_fLastPrintedXCoord = 0.0f;
float				Asura_Font_System::s_fMaxPrintedXCoord = 0.0f;
float				Asura_Font_System::s_fPixelSpacing = 0.f;
float				Asura_Font_System::s_fProportionalSpacing = 0.f;
bool				Asura_Font_System::s_bNextGlyphRequiresProjectRender = false;
u_int				Asura_Font_System::s_uCharacterColour = 0x00000000;
u_int				Asura_Font_System::s_uStringColour = 0x00000000;

u_int				Asura_Font_System::s_uCharacterFormat = 0;
float				Asura_Font_System::s_fShadowOffsetX = 1.5f;
float				Asura_Font_System::s_fShadowOffsetY = 1.5f;
bool				Asura_Font_System::s_bUsePlatformYScaleFactor = true;
ASURA_TRANSLUCENCY_MODE Asura_Font_System::s_eTranslucencyMode = ASURA_TRANSLUCENCY_NORMAL;

bool (*Asura_Font_System::s_pfnRenderSpecialEffectCharacterCallback)(Asura_Font_System::Render_Effects_Glyph_Data* pCharData) = Asura_Font_System::DefaultRenderSpecialEffects;

DEBUGVAR float dbg_fHeavyDropShadowAlpha = 160.0f;
DEBUGVAR float dbg_fLightDropShadowAlpha = 64.0f;

#ifdef ASURA_TOOLS
DEBUGVAR float dbg_fToolsColumnSpacing = 75.0f;
#endif

#ifdef ASURA_FONT_SYSTEM_SUPPORT_PIXEL_COORDINATES
//  AAB:  (08/09/2004 15:11:20):  Debugvar to enable / disable pixel boundary code
// for font rendering.
DEBUGVAR bool dbg_bUseExactPixelBoundaries = true;
#else
DEBUGVAR bool dbg_bUseExactPixelBoundaries = false;
#endif

DEBUGVAR bool dbg_bUseKerningData = true;


//*Asura_Font_Special_Effects***************************

#define ASURA_SPECIAL_EFFECTS_MAX_NUM_CHARS	(200000)
#define ASURA_SPECIAL_EFFECTS_MAX_CHAR_AGE_SECONDS	(24.0f*60.0f*60.0f)  // 24 Hours
#define ASURA_SPECIAL_EFFECTS_DEFAULT_CHARS_PER_SECOND (100)

Asura_Font_Special_Effects::Asura_Font_Special_Effects()
{
	// Initialise the effect settings.

	m_fTime = 0.0f;
	m_bRunning = false;
	m_uLastFrame = 0;
	m_fDelayTime = 0.0f;

	m_bTeletypeEnabled = false;
	m_fAlphaFadeTime = 0.0f;
	m_uCharsPerSecond = ASURA_SPECIAL_EFFECTS_DEFAULT_CHARS_PER_SECOND;
	m_fTimePerChar = 1.0f / m_uCharsPerSecond;

	m_uNumChars = ASURA_SPECIAL_EFFECTS_MAX_NUM_CHARS;
	m_eTeletypeDir = ASURA_GUIMENU_TELETYPE_TOP_LEFT;

	m_bDecryptEnabled = false;
	m_fDecryptChangeCharTime = 0.0f;
	m_fDecryptTime = 0.0f;

	m_bCursor = false;
	m_fCursorFlashTime = 0.0f;
	m_fCursorStopTime = 0.0f;
}

void Asura_Font_Special_Effects::Copy( const Asura_Font_Special_Effects* pxSource)
{
	m_fTime =  pxSource->m_fTime;
	m_bRunning =  pxSource->m_bRunning;
	m_uLastFrame =  pxSource->m_uLastFrame;
	m_fDelayTime =  pxSource->m_fDelayTime;

	m_bTeletypeEnabled = pxSource->m_bTeletypeEnabled;
	m_fAlphaFadeTime =  pxSource->m_fAlphaFadeTime;
	m_fTimePerChar =  pxSource->m_fTimePerChar;
	m_uCharsPerSecond = pxSource->m_uCharsPerSecond;

	m_uNumChars =  pxSource->m_uNumChars;
	m_eTeletypeDir =  pxSource->m_eTeletypeDir;

	m_bDecryptEnabled =  pxSource->m_bDecryptEnabled;
	m_fDecryptChangeCharTime =  pxSource->m_fDecryptChangeCharTime;
	m_fDecryptTime =  pxSource->m_fDecryptTime;

	m_bCursor =  pxSource->m_bCursor;
	m_fCursorFlashTime =  pxSource->m_fCursorFlashTime;
	m_fCursorStopTime = pxSource->m_fCursorStopTime;
}

// Start the effect running.
void Asura_Font_Special_Effects::Start()
{
	// Reset the time to 0.
	m_fTime = 0.0f;
	m_uNumChars = 0;
	m_bRunning = true;
	m_uLastFrame = Asura_Timers::GetFrameNumber();
}


void Asura_Font_Special_Effects::Stop() 
{ 
	m_bRunning = false; 
	m_uNumChars = ASURA_SPECIAL_EFFECTS_MAX_NUM_CHARS;
	m_uLastFrame = 0;
}

u_int Asura_Font_Special_Effects::Update()
{
	if ( m_bRunning == true )
	{
		// Update the last frame number. If the effects it rendered every frame then it will be reset at start again.
		m_uLastFrame = Asura_Timers::GetFrameNumber();

		// Increment the time that has past
		m_fTime += Asura_Timers::GetFrameTime();

		if ( m_bTeletypeEnabled)
		{
			// If tele-typing calculate the maximum number of characters that would be drawn.
			m_uNumChars = static_cast<u_int>((m_fTime-m_fDelayTime) / m_fTimePerChar);
		}
		else
		{
			m_uNumChars = ASURA_SPECIAL_EFFECTS_MAX_NUM_CHARS;
		}
	}
	return m_uNumChars;
}


// Calculate the age of a character. This is based on when the effect was started
// and, based on the position or the character, how long the character has been rendered for.
// This function can return a negative age meaning the character hasn't been drawn yet because
// the effect has run for long enough.
float Asura_Font_Special_Effects::CalcCharacterAge(u_int uCharPos, // Position within the string,
												   u_int uStringLength,  // Length of string
												   u_int uLineStartPos, // Char position for start of the current line
												   u_int uLineEndPos) const // Char position for end of the current line 
{
	if ( m_bRunning == false )
	{
		return ASURA_SPECIAL_EFFECTS_MAX_CHAR_AGE_SECONDS;
	}

	if ( m_bTeletypeEnabled == false)
	{
		return m_fTime-m_fDelayTime;
	}

	// The actual position of the char respective to the rendering order.
	// Therefore is the string is render backwards the last char will have a render position of 0
	// as its the first char to be rendered. This is used to work out the age of the character.
	// Note that the string rendering routine still renders from the start of the string to the end of string,
	// this is just working out theoretically when the char should be rendered.
	u_int uCharRenderPos = ASURA_SPECIAL_EFFECTS_MAX_NUM_CHARS; 

	float fCharAge;

	switch(m_eTeletypeDir)
	{
	case ASURA_GUIMENU_TELETYPE_TOP_LEFT:
		{
			// Standard left-to-right and top-to-bottom rendering of a word-wrapped string
			uCharRenderPos = uCharPos;
		}
		break;
	case ASURA_GUIMENU_TELETYPE_TOP_RIGHT:
		{
			// Get the number of chars from the start of the string to the start of the line.
			uCharRenderPos = uLineStartPos;

			// Now add the char from char pos to the end of the line
			uCharRenderPos += uLineEndPos-uCharPos;
		}
		break;
	case ASURA_GUIMENU_TELETYPE_BOTTOM_LEFT:
		{
			// Get the number of chars from the end of the line to the end of the string.
			uCharRenderPos = uStringLength-uLineEndPos;
			// Now add the chars from the start of the line to the char pos.
			uCharRenderPos += uCharPos-uLineStartPos;
		}
		break;
	case ASURA_GUIMENU_TELETYPE_BOTTOM_RIGHT:
		{
			// Count the number of chars from the end of the string.
			uCharRenderPos = uStringLength-uCharPos;
		}
		break;
	case ASURA_GUIMENU_TELETYPE_SWEEP_LEFT:
		{
			// Count the number of chars from the beginning of the line.
			uCharRenderPos = uCharPos-uLineStartPos;
		}
		break;
	case ASURA_GUIMENU_TELETYPE_SWEEP_RIGHT:
		{
			// Count the number of chars from the end of the line.
			uCharRenderPos = uLineEndPos-uCharPos;
		}
		break;
	default:
		Asura_Assert(false,"Unknown Teletype direction");
		break;
	}

	fCharAge = ((m_fTime-m_fDelayTime)-(uCharRenderPos * m_fTimePerChar));

	return fCharAge;
}

// Given the age of a character calculate the alpha value based on the fade time.
float Asura_Font_Special_Effects::GetAlpha(float fCharAge) const
{
	// Need to find out how long ago this char was drawn and at what stage the alpha is
	// based on the m_fAlphaFadeTime and the m_fTimePerChar;
	if ( m_fAlphaFadeTime > 0.0f )
	{
		// Set the alpha so its no greater than 1.0f.
		float fAlpha = Asura_Maths::Min(1.0f,fCharAge/m_fAlphaFadeTime);
		return fAlpha;
	}

	return 1.0f;
}

void Asura_Font_Special_Effects::ReadFromChunkStream( Asura_Chunk_Stream& xStream)
{
	u_int uVersion;

	xStream >> uVersion;

	xStream >> m_fDelayTime;

	// Teletype data
	xStream >> m_bTeletypeEnabled;

	u_int uCharsPerSecond;
	xStream >> uCharsPerSecond;
	SetCharsPerSecond(uCharsPerSecond);

	xStream >> m_fAlphaFadeTime;

	u_int uTeletypeDir;
	xStream >> uTeletypeDir;
	m_eTeletypeDir = static_cast<TeletypeDir>(uTeletypeDir);

	// Decryption Data
	xStream >> m_bDecryptEnabled;
	xStream >> m_fDecryptChangeCharTime;
	xStream >> m_fDecryptTime;

	xStream >> m_bCursor;
	xStream >> m_fCursorFlashTime;
	xStream >> m_fCursorStopTime;

}

// --------------------------------------------------------------------------------------- 
#ifdef ASURA_TOOLS

void Asura_Font_Special_Effects::WriteToChunkStream( Asura_Chunk_Stream& xStream) const
{
	u_int uVersion = ASURA_CHUNK_ELEMENT_TEXT_SPECIAL_EFFECTS_VERSION;

	xStream << uVersion;

	xStream << m_fDelayTime;

	// Teletype data
	xStream << m_bTeletypeEnabled;
	xStream << m_uCharsPerSecond;

	xStream << m_fAlphaFadeTime;

	u_int uTeletypeDir = m_eTeletypeDir;
	xStream << uTeletypeDir;

	// Decryption Data
	xStream << m_bDecryptEnabled;
	xStream << m_fDecryptChangeCharTime;
	xStream << m_fDecryptTime;

	xStream << m_bCursor;
	xStream << m_fCursorFlashTime;
	xStream << m_fCursorStopTime;
}

#endif // ASURA_TOOLS


//*Asura_Font_System***************************

static inline float AdjustCoordToWholePixel( float fStart )
{
    if( dbg_bUseExactPixelBoundaries )
    {
        // (MP - 14.03.2006) IEEE754 says 2.5 should round to 2 while 3.5
        // should round to 4, ie. in the case of a tie, go for the even
        // number. The PC maths code follows this. However, this is awkward
        // for our 2 pass code where the start coordinate can be different
        // (ie. odd vs even) and result in pixels being gained or lost as
        // we iterate.
        fStart = static_cast<float>( Asura_Maths::FloatToInt_Chop(fStart > 0.0f ? fStart + 0.5f : fStart - 0.5f) );
    }

    return fStart;
}

//*CONSTRUCTORS AND DESTRUCTORS****************
#ifndef  _FONTMAKER
	void Asura_Font_System::Initialise()
	{
        Project_Initialise();
	}
#else
	void Asura_Font_System::Initialise()
	{
		//FOR USE WITH FONTMAKER
	}
#endif //_FONTMAKER

void Asura_Font_System::Shutdown()
{
    Reset();
	Project_Shutdown();
}

void Asura_Font_System::Reset()
{
	for (int i=0; i<ASURA_MAX_FONTS; i++)
	{
		delete s_apxFontStorage[i];
		s_apxFontStorage[i] = NULL;
	}

	s_bAnyFontHasLoaded = false;
}

#ifndef  _FONTMAKER

float Asura_Font_System::GetWidthOfChar( Asura_Char cChar, float fScale /*= 1.0f*/ )
{
	Asura_UnicodeChar wChar = static_cast<Asura_UnicodeChar>( cChar & 0x00ff );
	return GetWidthOfChar( wChar, fScale );
}

float Asura_Font_System::GetWidthOfChar( Asura_UnicodeChar wChar, float fScale /*= 1.0f*/ )
{
    const Asura_Font* pxFont = s_apxFontStorage[s_iCurrentFontID];
    if( pxFont )
    {
		return GetWidthOfChar_Unscaled( wChar, fScale * pxFont->m_fFontScale );
    }

	return GetWidthOfChar_Unscaled(wChar, fScale);
}

//  AAB:  (03/09/2004 10:12:29):  With the addition of font specific scales, we need 
// an internal version of GetWidthOfChar.
float Asura_Font_System::GetWidthOfChar_Unscaled( Asura_UnicodeChar wChar, float fScale /*= 1.0f*/ )
{
	float fWidth = Project_GetWidthOfChar(wChar, fScale);
	if( fWidth <= 0.0f )
	{
        fWidth = 0.0f;

        const Asura_Font* pxFont = s_apxFontStorage[s_iCurrentFontID];
        if( pxFont )
        {
	        const Asura_Font_Glyph* pxThisGlyph = pxFont->GetGlyphData(wChar);
            if( pxThisGlyph )
            {
                fWidth = pxThisGlyph->width * fScale;
            }
        }
    }

    return fWidth;
}

float Asura_Font_System::GetWidthOfString( const Asura_Char* szStr, float fScale, u_int uNumCharsToCount )
{
    if ( szStr == NULL )
    {
        return 0.0f;
    }

    float fMaxWidth = 0.0f;

#ifdef ASURA_TOOLS
	float fNextToolsColumn = dbg_fToolsColumnSpacing;
#endif

	float fCurrentWidth = 0.0f;
    for( ; uNumCharsToCount; --uNumCharsToCount, ++szStr )
    {
        Asura_Char cCurChar = *szStr;
        if( '\0' == cCurChar )
        {
            break;
        }
        else if( '\n' == cCurChar )
		{
			fMaxWidth = Asura_Maths::Max( fCurrentWidth, fMaxWidth );
			fCurrentWidth = 0;
		}
#ifdef ASURA_TOOLS
		else if ( '\a' == cCurChar)
		{			
			if (fCurrentWidth<fNextToolsColumn)
			{
				fCurrentWidth=fNextToolsColumn;
			}
			fNextToolsColumn += dbg_fToolsColumnSpacing; 			
		}
#endif
		else
		{
            // If this is not the first character in this line then account
            // for spacing adjustments after the previous one.
            if( fCurrentWidth )
            {
                Asura_Char cPrevChar = *(szStr - 1);
                if( '\n' != cPrevChar )
                {
                    fCurrentWidth += GetCharSpacing( (Asura_UnicodeChar)cPrevChar, (Asura_UnicodeChar)cCurChar, fScale );
                    fCurrentWidth = AdjustCoordToWholePixel( fCurrentWidth );
                }
            }
            
            fCurrentWidth += GetWidthOfChar((Asura_UnicodeChar)cCurChar, fScale );
		}

    }

	fMaxWidth = Asura_Maths::Max( fCurrentWidth, fMaxWidth );
    
    return fMaxWidth;
}

float Asura_Font_System::GetWidthOfString( const Asura_UnicodeChar* wszStr, float fScale, u_int uNumCharsToCount )
{
    if ( wszStr == NULL )
    {
        return 0.0f;
    }

    float fMaxWidth = 0.0f;

	float fCurrentWidth = 0.0f;
    for( ; uNumCharsToCount; --uNumCharsToCount, ++wszStr )
    {
        Asura_UnicodeChar wCurChar = *wszStr;
        if( L'\0' == wCurChar )
        {
            break;
        }
		else if( L'\n' == wCurChar )
		{
			fMaxWidth = Asura_Maths::Max( fCurrentWidth, fMaxWidth );
			fCurrentWidth = 0.0f;
		}
		else
		{
            // If this is not the first character in this line then account
            // for spacing adjustments after the previous one.
            if( fCurrentWidth )
            {
                Asura_UnicodeChar wPrevChar = *(wszStr - 1);
                if( L'\n' != wPrevChar )
                {
                    fCurrentWidth += GetCharSpacing( wPrevChar, wCurChar, fScale );
                    fCurrentWidth = AdjustCoordToWholePixel( fCurrentWidth );
                }
            }

			fCurrentWidth += GetWidthOfChar( wCurChar, fScale );
		}
    }

	fMaxWidth = Asura_Maths::Max( fCurrentWidth, fMaxWidth );

    return fMaxWidth;
}

bool Asura_Font_System::PrintChar(const Asura_Bounding_Rectangle* pxPixelRect, // in screen coords (NULL if no clipping)
                                  float fStartX,                    // in screen coords
                                  float fStartY,                    // in screen coords
                                  const float fScale, 
                                  const u_int uColour, 
                                  const Asura_Font_Glyph* pxThisGlyph,
                                  const Asura_Font_GlyphPage* pxGlyphPage )
{
    // function to print a character
    // returns true if anything drawn

    if (!pxThisGlyph || !pxGlyphPage) return false;

    float fStartU = pxThisGlyph->u;
    float fStartV = pxThisGlyph->v;

#ifdef ASURA_REMOVE_FONT_GLYPH_UV_OFFSET
	fStartU -= 0.5f/pxGlyphPage->m_uTexWidth;
    fStartV -= 0.5f/pxGlyphPage->m_uTexHeight;
#endif

	float fEndU	  = fStartU + pxThisGlyph->width 
        / pxGlyphPage->m_uTexWidth;
    const float fTrueFontHeight = s_apxFontStorage[s_iCurrentFontID]->GetFontHeight();
	float fEndV   = fStartV + fTrueFontHeight /
        pxGlyphPage->m_uTexHeight;

	const float fPlatformYScale = s_bUsePlatformYScaleFactor ? GetPlatformYScaleFactor() : 1.0f;
    float fEndX = fStartX + pxThisGlyph->width * fScale; // Can't use GetWidthOfChar as we don't know what char it is, just get the default width (as that's what we always render in this function)
    float fEndY = fStartY + fTrueFontHeight * fScale * fPlatformYScale;

	const Asura_Bounding_Rectangle xScreenRect = Asura_DisplayLimits::Platform_GetBoundingRectangle();
    
    if (!pxPixelRect)
    {
        pxPixelRect = &xScreenRect;
    }

    if (fStartX < pxPixelRect->MaxX && fStartY < pxPixelRect->MaxY &&
        fEndX > pxPixelRect->MinX && fEndY > pxPixelRect->MinY)
    {
        // there's some overlap between character and rectangle

        // clip
        if ( fStartX < pxPixelRect->MinX )
	    {
		    // Left clip
		    float fClip = (pxPixelRect->MinX - fStartX) / (fEndX - fStartX);
		    fStartU += (fEndU-fStartU) * fClip;
		    fStartX = pxPixelRect->MinX;
	    }

	    if ( fEndX > pxPixelRect->MaxX )
	    {
		    // Right clip
		    float fClip = (pxPixelRect->MaxX - fStartX) / (fEndX - fStartX);
		    fEndU = fStartU + (fEndU-fStartU) * fClip;
		    fEndX = pxPixelRect->MaxX;
	    }
	    if ( fStartY < pxPixelRect->MinY )
	    {
		    // Top clip
		    float fClip = (pxPixelRect->MinY - fStartY) / (fEndY - fStartY);
		    fStartV += (fEndV-fStartV) * fClip;
		    fStartY = pxPixelRect->MinY;
	    }

	    if ( fEndY > pxPixelRect->MaxY ) 
	    {
		    // bottom clip
		    float fClip = (pxPixelRect->MaxY - fStartY) / (fEndY - fStartY);
		    fEndV = fStartV + (fEndV-fStartV) * fClip;
		    fEndY = pxPixelRect->MaxY;
        }

		Asura_Render::Overlay.TextureIndex		= pxGlyphPage->m_iFontTextureIndex;
		Asura_Render::Overlay.TranslucencyMode	= s_eTranslucencyMode;
		Asura_Render::Overlay.m_bNonRotated		= true;
		Asura_Render::Overlay.m_bGouraudShaded	= false;

		Asura_Render::Overlay.U[0] = Asura_Render::Overlay.U[3] = fStartU;
		Asura_Render::Overlay.V[0] = Asura_Render::Overlay.V[1] = fStartV;
		Asura_Render::Overlay.U[1] = Asura_Render::Overlay.U[2] = fEndU;
		Asura_Render::Overlay.V[2] = Asura_Render::Overlay.V[3] = fEndV;

        const u_int uCharARGB = s_uCharacterColour;

        const u_int uCharFormat = s_uCharacterFormat;
		if( uCharFormat & (ASURA_TEXTFORMAT_DROPSHADOWS | ASURA_TEXTFORMAT_LIGHTDROPSHADOWS) )
		{
            u_int uShadowARGB = 0;

            // Modulate the shadow colour alpha by the text alpha.
            {
                float fCharAlpha = static_cast<float>( static_cast<int>((uCharARGB >> 24u) & 0xFFu) );

                float fShadowAlpha = ( uCharFormat & ASURA_TEXTFORMAT_LIGHTDROPSHADOWS ? dbg_fLightDropShadowAlpha : dbg_fHeavyDropShadowAlpha );
                fShadowAlpha *= (fCharAlpha / 255.0f);

                uShadowARGB |= static_cast<u_int>(Asura_Maths::FloatToInt_Near(fShadowAlpha)) << 24u;
            }

			Asura_Render::Overlay.Colour = uShadowARGB;

			Asura_Render::Overlay.Y[0] = Asura_Render::Overlay.Y[1] = fStartY + s_fShadowOffsetX;
			Asura_Render::Overlay.Y[2] = Asura_Render::Overlay.Y[3] = fEndY	  + s_fShadowOffsetY;
			Asura_Render::Overlay.X[0] = Asura_Render::Overlay.X[3] = fStartX + s_fShadowOffsetX;
			Asura_Render::Overlay.X[1] = Asura_Render::Overlay.X[2] = fEndX	  + s_fShadowOffsetY;

			Asura_Render::Overlay.Render( false );
		}
	
  	    Asura_Render::Overlay.Colour = uCharARGB;

        Asura_Render::Overlay.Y[0] = Asura_Render::Overlay.Y[1] = fStartY;
		Asura_Render::Overlay.Y[2] = Asura_Render::Overlay.Y[3] = fEndY;
        Asura_Render::Overlay.X[0] = Asura_Render::Overlay.X[3] = fStartX;
		Asura_Render::Overlay.X[1] = Asura_Render::Overlay.X[2] = fEndX;
	
		Asura_Render::Overlay.Render();
        return true;
    }
    else
    {
        //outside drawable range
        return false;
    }
    
}

const int MAX_BUFFER_SIZE = 255;

void Asura_Font_System::Print(const float x, const float y, const u_int colour, 
							  const u_int format, const Asura_UnicodeChar* string, ...)
{
	//Only attempt to print text if a font has been loaded, and we have a valid string
	if(s_bAnyFontHasLoaded && string)
	{
		Asura_UnicodeChar buffer[MAX_BUFFER_SIZE];
		if (format & ASURA_TEXTFORMAT_FORMATSTRING)
		{
			va_list ap;
			va_start(ap, string);
			Asura_PrintfToBfr( buffer, string, ap, MAX_BUFFER_SIZE, true );
			va_end(ap);

            string = &buffer[0];
		}

        PrintLongString( string, NULL, x, y, 1.f, 1.f, colour, format );
	}
}

void Asura_Font_System::Print(const float x, const float y, const float fScale, const u_int colour, 
							  const u_int format, const Asura_UnicodeChar* string, ...)
{
	//We scale glyphs about their centres.

	//Only attempt to print text if a font has been loaded and we have a valid string
	if(s_bAnyFontHasLoaded && string)
	{
		Asura_UnicodeChar buffer[MAX_BUFFER_SIZE];
		if (format & ASURA_TEXTFORMAT_FORMATSTRING)
		{
			va_list ap;
			va_start(ap, string);
			Asura_PrintfToBfr( buffer, string, ap, MAX_BUFFER_SIZE, true );
			va_end(ap);

            string = &buffer[0];
		}

        PrintLongString( string, NULL, x, y, 1.f, fScale, colour, format );
		
	}
}

void Asura_Font_System::Print(const float x, const float y, const u_int colour, const u_int format, const Asura_Char* string, ...)
{
	//Convert Asura_Char string to unicode and call main Print function.

	//Do nothing if no fonts are loaded, or null string
	if(s_bAnyFontHasLoaded && string)
	{
		Asura_Char buffer[MAX_BUFFER_SIZE];
		if (format & ASURA_TEXTFORMAT_FORMATSTRING)
		{
			va_list ap;
			va_start( ap, string );
			Asura_PrintfToBfr( buffer, string, ap, MAX_BUFFER_SIZE );
			va_end(ap);

            string = &buffer[0];
		}

		PrintShortString( string, NULL, x, y, 1.f, 1.f, colour, format );
	}

}

bool Asura_Font_System::PrintShortString( const Asura_Char* szIn,
                                          const Asura_Bounding_Rectangle* pxRect,
                                          float fXOffset, float fYOffset, 
                                          float fLineSpacing,
                                          float fScale,
                                          u_int uColour,
                                          u_int uFormat )
{
    Asura_UnicodeChar wszBfr[MAX_BUFFER_SIZE];
    Asura_PrintfToBfr( wszBfr, L"%S", MAX_BUFFER_SIZE, true, szIn );
    return PrintLongString( wszBfr, pxRect, fXOffset, fYOffset, fLineSpacing, fScale, uColour, uFormat );
}

void Asura_Font_System::Print(const float x, const float y, const float fScale, const u_int colour, 
                              const u_int format, const Asura_Char* string, ...)
{
	//Convert Asura_Char string to unicode and call main Print function.

	//Do nothing if no fonts are loaded, or null string
	if(s_bAnyFontHasLoaded && string)
	{
		Asura_Char buffer[MAX_BUFFER_SIZE];
		if (format & ASURA_TEXTFORMAT_FORMATSTRING)
		{
			va_list ap;
			va_start(ap, string);
			Asura_PrintfToBfr( buffer, string, ap, MAX_BUFFER_SIZE );
			va_end(ap);

            string = &buffer[0];
		}

        PrintShortString( string, NULL, x, y, 1.f, fScale, colour, format );
	}
}


bool Asura_Font_System::Print( const Asura_Bounding_Rectangle &xRect,		// The rectangle the text should be printed inside
							   const float fXOffset,						// X Offset inside the rectangle so you can scroll the text left/right - Has no effect if WORDWRAPPING flag is set
							   const float fYOffset,						// Y Offset inside the rectangle so you can scroll the text up/down
							   const float fLineSpacing,					// Line spaceing mulitplier 0.75 seems good for the font I'm testing it with
							   const float fScale,							// Scale size for the font when it's drawn
							   const u_int uColour,							// The colour to print in
							   const u_int uFormat,							// Text formating flags see Asura_Font_System.h
							   const Asura_UnicodeChar* pcString, ...)		// The text
{
    // PSJE: The main rectangle print function

	//We scale glyphs about their centres.

	//Only attempt to print text if a font has been loaded and we have a valid string
	if( !s_bAnyFontHasLoaded || !pcString )
	{
		return false;
	}

	// Expand varg list into string
	Asura_UnicodeChar wszTempBuffer[MAX_BUFFER_SIZE];
	if (uFormat & ASURA_TEXTFORMAT_FORMATSTRING)
	{
		va_list ap;
		va_start(ap, pcString);
		Asura_PrintfToBfr( wszTempBuffer, pcString, ap, MAX_BUFFER_SIZE, true );
		va_end(ap);

//		// Make sure the buffer is null terminated
//		wszTempBuffer[MAX_BUFFER_SIZE-1] = '\0';

        pcString = &wszTempBuffer[0];
	}

    return PrintLongString( pcString, &xRect, fXOffset, fYOffset, fLineSpacing, fScale, uColour, uFormat | ASURA_TEXTFORMAT_RELATIVETORECT );
}

// function to get scale that may be set inside the text (e.g. to use smaller
// scale for text in some languages).
float Asura_Font_System::GetInlinedScaleFactor(const Asura_UnicodeChar* pcString)
{
    if (pcString && pcString[0] == ASURA_FONT_SIZE_SPECIAL_CHAR)
    {
        // all characters up to the next ASURA_FONT_SIZE_SPECIAL_CHAR specify scale as a fraction of fScale.

        // copy to a Asura_Char buffer and convert
        // LBF: I would rather use swscanf, but it isn't implemented on PS2...
        const int iMaxScaleChars = 10;
        Asura_Char szScale[iMaxScaleChars];
        pcString++;
        int iScaleChar = 0;
        while (true)
        {    
            if (*pcString == ASURA_FONT_SIZE_SPECIAL_CHAR)
            {
                // step over it
                pcString++;
                break;
            }
            else if ((iScaleChar >= iMaxScaleChars - 1) || *pcString == L'\0')
            {
                Asura_Assert(false, "Expected ASURA_FONT_SIZE_SPECIAL_CHAR");
                break;
            }
            szScale[iScaleChar] = (Asura_Char)(*pcString);
            iScaleChar++;
            pcString++;
        }
        szScale[iScaleChar] = '\0';
        float fScaleFactor;
        Asura_StringUtil::GetFloatValue(szScale, &fScaleFactor);
        return fScaleFactor;
    }
	return 1.0f;
}


void Asura_Font_System::GetPixelClipRectAndScaleOffsets(Asura_Bounding_Rectangle& xClipPixelRect,	// output
														float& fXOffset,							// output
														float& fYOffset,							// output
														const Asura_Bounding_Rectangle* pxClipRect, // input
														const u_int uFormat)						// input
{
    if (pxClipRect)
    {
        xClipPixelRect = *pxClipRect;
    }
    else
    {
		xClipPixelRect = Asura_DisplayLimits::Platform_GetBoundingRectangle();
    }

	if( uFormat & ASURA_TEXTFORMAT_RELATIVEPOSITION )
	{
        if (pxClipRect) 
        {
		    xClipPixelRect.MinX *= Asura_Viewport::GetWidth();
		    xClipPixelRect.MaxX *= Asura_Viewport::GetWidth();
		    xClipPixelRect.MinY *= Asura_Viewport::GetHeight();
		    xClipPixelRect.MaxY *= Asura_Viewport::GetHeight();
        }

		fXOffset *= Asura_Viewport::GetWidth();
		fYOffset *= Asura_Viewport::GetHeight();
	}
    const Asura_ViewingArea* pxViewArea = Asura_Camera::GetViewingArea();
	if(uFormat & ASURA_TEXTFORMAT_CAMERARELATIVEPOSITION && pxViewArea)
	{
        if (pxClipRect) 
        {
		    xClipPixelRect.MinX *= pxViewArea->m_fWidth *  Asura_Viewport::GetWidth();
			xClipPixelRect.MinX += pxViewArea->m_fXOffset * Asura_Viewport::GetWidth();
	
			xClipPixelRect.MaxX *= pxViewArea->m_fWidth *  Asura_Viewport::GetWidth();
			xClipPixelRect.MaxX += pxViewArea->m_fXOffset * Asura_Viewport::GetWidth();
		  
			xClipPixelRect.MinY *= pxViewArea->m_fHeight * Asura_Viewport::GetHeight();
			xClipPixelRect.MinY += pxViewArea->m_fYOffset * Asura_Viewport::GetHeight();
		   
			xClipPixelRect.MaxY *= pxViewArea->m_fHeight * Asura_Viewport::GetHeight();
			xClipPixelRect.MaxY += pxViewArea->m_fYOffset * Asura_Viewport::GetHeight();
		}
		else
		{
			fXOffset *= pxViewArea->m_fWidth   * Asura_Viewport::GetWidth();
			fXOffset += pxViewArea->m_fXOffset * Asura_Viewport::GetWidth();
			fYOffset *= pxViewArea->m_fHeight  * Asura_Viewport::GetHeight();
			fYOffset += pxViewArea->m_fYOffset * Asura_Viewport::GetHeight();
		}
	}
}


const Asura_UnicodeChar* Asura_Font_System::FindSplitPoint( const Asura_UnicodeChar* wszString,
                                                            float fWidthPixels, 
                                                            float fScale, 
                                                            float& fOutTextWidth,
                                                            int& iOutNumWords )
{
    const Asura_Font* pxFont = s_apxFontStorage[s_iCurrentFontID];
    if( pxFont )
    {
        fScale *= pxFont->m_fFontScale;
    }
	
	return FindSplitPoint_Unscaled( wszString, fWidthPixels, fScale, &fOutTextWidth, &iOutNumWords, NULL );
}

//  AAB:  (03/09/2004 10:15:02):  Added to allow font specific scales.
const Asura_UnicodeChar* Asura_Font_System::FindSplitPoint_Unscaled( const Asura_UnicodeChar* wszString, 
                                                                     float fMaxWidth, 
                                                                     float fScale,
                                                                     float* pfLineWidth,
                                                                     int* piNumWordsInLine,
                                                                     bool* pbNeedHyphen )
{
    // return 1st character of next line (or last if end of string) and line width as fOutTextWidth

    enum SPLIT
    {
        SPLIT_NEWLINE,
        SPLIT_SPACE,
        SPLIT_HYPHEN,
        SPLIT_MIDWORD,
        SPLIT_SINGLECHAR,
    };

	// (AW - 19/01/06) - In Japanese the rules for line-breaks are different. Because Japanese doesn't really
	// use spaces, line-breaks are allowed pretty much anywhere with some exceptions. We'll use the approach of
	// always using the hard-coded line-breaks, plus consider everywhere as a line-break candidate unless it 
	// breaks one of the rules specified in the code below. The initial set of rules are based on EJD's 
	// 'how to localise Japanese' document (http://asura/asuradocs/ow.asp?HowToLocaliseJapanese).
    const bool bJapanese = ( Asura_LocalisedText::GetCurrentLanguage() == ASURA_LT_L_JAPANESE );

    const float fHyphenWidth = GetWidthOfChar_Unscaled( L'-', fScale );

    const Asura_UnicodeChar* wszSplitPos = NULL;
    float fWidthForSplit = 0.0f;
    SPLIT eSplitType = SPLIT_MIDWORD;

    float fLineWidth = 0.0f;
    int iInWord = 0; // 1 if in word, 0 otherwise.
    int iNumWords = 0;
    float fSpacingForLastChar = 0.0f;

    const Asura_UnicodeChar* wszCurPos = wszString;
    Asura_UnicodeChar wCurChar = *wszCurPos;
    while( wCurChar )
    {
        // A newline in the string forces a split, obviously.
        if( L'\n' == wCurChar )
        {
            wszSplitPos = wszCurPos + 1;
            fWidthForSplit = fLineWidth;
            eSplitType = SPLIT_NEWLINE;

            break;
        }

        float fCharWidth = GetWidthOfChar_Unscaled( wCurChar, fScale );

        // Is this character going to take us outside the limit?
        float fStartOfCurChar = AdjustCoordToWholePixel( fLineWidth + fSpacingForLastChar );
        float fNewWidth = fStartOfCurChar + fCharWidth;
        if( fNewWidth > fMaxWidth )
        {
            // If we don't yet have a split position then we've only dealt
            // with at most one character so far. Print just the first
            // character in this case - regardless of whether or not it will
            // go over the bounds, since otherwise we'll never output
            // anything, and might well end up in an infinite loop in the
            // caller.
            if( !wszSplitPos )
            {
                Asura_Assert( wszCurPos <= wszString + 1, "" );

                wszSplitPos = wszString + 1;
                eSplitType = SPLIT_SINGLECHAR;

                Asura_UnicodeChar wFirstChar = *wszString;
                fWidthForSplit = GetWidthOfChar_Unscaled( wFirstChar, fScale );

                // Ensure in-word flag is correctly set.
                iInWord = ( L' ' != wFirstChar ? 1 : 0 );
            }

            break;
        }

        if( bJapanese )
        {
            const Asura_UnicodeChar wNextChar = *( wszCurPos + 1 );
            // Keep a record of the last point we encountered where we can
            // legitimately split the text.
			if (wNextChar != 0x3002 &&		// Don't break before the Japanese full stop.
				wNextChar != L'!' &&			// Don't break before an exclamation mark
				wNextChar != L'?' &&			// Don't break before a question mark.
				wNextChar != 0x30fc &&		// Don't break before the Japanese vowel extension character
				wNextChar != 0x300d &&		// Don't break before closing quotes
				wNextChar != 0x300f &&		// Don't break before closing quotes
				wNextChar != L')' &&			// Don't break before close bracket
				wCurChar != 0x300c &&				// Don't break after an opening quote
				wCurChar != 0x300e &&				// Don't break after an opening quote
				wCurChar != L'(' &&				// Don't break after an opening bracket.
				!(IsRomajiCharacter(wCurChar) && IsRomajiCharacter(wNextChar)) &&	// Don't break in the middle of a Romaji (i.e. English) word.
				fLineWidth + fCharWidth <= fMaxWidth)	// Don't overrun.
			{
                wszSplitPos = wszCurPos + 1;
                fWidthForSplit = fLineWidth;
                eSplitType = SPLIT_MIDWORD;

                iNumWords += iInWord;
                iInWord = 0;
            }
        }
        else
        {
            // Keep a record of the last point we encountered where we can
            // legitimately split the text.
            if( L' ' == wCurChar )
            {
                wszSplitPos = wszCurPos + 1;
                fWidthForSplit = fLineWidth;
                eSplitType = SPLIT_SPACE;

                iNumWords += iInWord;
                iInWord = 0;
            }
            else if( L'-' == wCurChar )
            {
                wszSplitPos = wszCurPos + 1;
                fWidthForSplit = fNewWidth;
                eSplitType = SPLIT_HYPHEN;
            }

            // We can fit this character in but maybe we won't manage the
            // next one. If we haven't found a better split point so far then
            // in that case we'll have to split the current word with a
            // hyphen. Keep the current state as a backup for that
            // eventuality. However, we need to avoid using the first
            // character as a split position since otherwise we could end up
            // not printing anything, and maybe end up in an infinite loop in
            // the caller.
            if( fLineWidth && SPLIT_MIDWORD == eSplitType )
            {
                // Also, stop updating the mid-word split position once we
                // can no longer safely fit a hyphen in.
                if( fStartOfCurChar + fHyphenWidth <= fMaxWidth )
                {
                    wszSplitPos = wszCurPos;
                    fWidthForSplit = fLineWidth;
                }
            }
        }

        if( L' ' != wCurChar )
        {
            iInWord = 1;
        }

        fLineWidth = fNewWidth;

        Asura_UnicodeChar wNextChar = *++wszCurPos;
        fSpacingForLastChar = GetCharSpacing( wCurChar, wNextChar, fScale );
        wCurChar = wNextChar;
    }

    bool bNeedHyphen = false;

    // Did we fail to reach the end of the string?
    if( wCurChar )
    {
        // Yes so fallback to whatever split position we managed to find.
        fLineWidth = fWidthForSplit;
        wszCurPos = wszSplitPos;

        Asura_Assert( wszCurPos > wszString, "" );

        // Ensure we don't end up with leading spaces on the next line.
        // (MP - 15.10.2005) Doing this is preferable, I think, but not that
        // it does introduce some slight inconsistencies. For example, if
        // a line 'aaaa   aaaaa' is split, you'll get just 'aaaa' and 'aaaaa'
        // being reported in terms of widths from this method. However,
        // PrintLongString() will still print the full 'aaaa   ' first line,
        // so GetLastPrintedXCoordinate(), etc., may be somewhat incorrect.
        // This shouldn't affect appearances, however.
        while( L' ' == *wszCurPos ) { ++wszCurPos; }

        // Hyphen required?
        if( SPLIT_MIDWORD == eSplitType && !bJapanese )
        {
            bNeedHyphen = true;

            // Update width then.
            fLineWidth = AdjustCoordToWholePixel( fLineWidth + GetCharSpacing(*(wszCurPos - 1), L'-', fScale) );
            fLineWidth += fHyphenWidth;
        }
    }

    // Count the last word we were processing, unless we're returning to the
    // space before it.
    if( !wCurChar || SPLIT_SPACE != eSplitType )
    {
        iNumWords += iInWord;
    }

    // Return values.
    if( pfLineWidth ) { *pfLineWidth = fLineWidth; }
    if( piNumWordsInLine ) { *piNumWordsInLine = iNumWords; }
    if( pbNeedHyphen ) { *pbNeedHyphen = bJapanese ? false : bNeedHyphen; }

    return wszCurPos;
}


float Asura_Font_System::GetTextHeight (const Asura_UnicodeChar* wszString,
										const u_int uFormat,
										const float fCharacterHeight,
										const float fRectWidthPixels,
										const float fScale,
										const float fLineHeight)
{
	const Asura_UnicodeChar*	wszCurrent	= wszString;

	// Are we using word-wrap?
	if (0 == (ASURA_TEXTFORMAT_WORDWRAP & uFormat))
	{
		float	fTextHeight	= fCharacterHeight;
		
		// Keep checking until we reach the end of the string...
		while (L'\0' != *wszCurrent)
		{   	    
			// Check for EOL...
			if (L'\n' == *wszCurrent)
			{
				// Move down to the next line...
				fTextHeight += fLineHeight;
			}

			// Next character...
			wszCurrent++;
		}

		// Return the height...
		return	(fTextHeight);
	}

	else
	{
		float	fTextHeight	= 0.0f;

		// Keep checking until we reach the end of the string...
		while (L'\0' != *wszCurrent)
		{
			float	fLineWidth;
			int		iNumWords;
            bool    bNeedHyphen;

			// Split the string at the next new-line...
			wszCurrent	= FindSplitPoint_Unscaled (wszCurrent, fRectWidthPixels, fScale, &fLineWidth, &iNumWords, &bNeedHyphen);

			// Update the height...
			fTextHeight	+= (0.0f == fTextHeight) ? fCharacterHeight : fLineHeight;
		}

		// Return the height...
		return	(fTextHeight);
	}
}


float Asura_Font_System::GetTextWidth (const Asura_UnicodeChar* wszString,
									   const u_int uFormat,
									   const float fRectWidthPixels,
									   const float fScale)
{
	const Asura_UnicodeChar*	wszCurrent	= wszString;
	const float					fSplitWidth	= (0 == (uFormat & ASURA_TEXTFORMAT_WORDWRAP))
											? Asura_Maths::LargeFloat
											: fRectWidthPixels;

	float						fTextWidth	= 0.0f;

	// Keep checking until we reach the end of the list...
	while (L'\0' != *wszCurrent)
	{
		float	fLineWidth;
		int		iNumWords;
        bool    bNeedHyphen;

		// Split the string at the next new-line...
		wszCurrent	= FindSplitPoint_Unscaled (wszCurrent, fSplitWidth, fScale, &fLineWidth, &iNumWords, &bNeedHyphen);

		// Update the width...
		fTextWidth	= Asura_Maths::Max (fTextWidth, fLineWidth);
	}

	// Return the width...
	return	(fTextWidth);
}


bool Asura_Font_System::PrintLongString( const Asura_UnicodeChar*  pcString,           // The text
    const Asura_Bounding_Rectangle* pxClipRect,		// The rectangle the text should be clipped to (NULL if full screen)
    float fXOffset,						            // X Offset inside the rectangle so you can scroll the text left/right - Has no effect if WORDWRAPPING flag is set
    float fYOffset,						            // Y Offset inside the rectangle so you can scroll the text up/down
    const float fLineSpacing,					    // Line spacing mulitplier
    float fScale,							        // Scale size for the font when it's drawn
    const u_int uColour,							// The colour to print in
    const u_int uFormat)							// Text formating flags see Asura_Font_System.h
{
	s_uCharacterColour = uColour;
	s_uStringColour	   = s_uCharacterColour;
	s_uCharacterFormat = uFormat;

    // LBF: the following doesn't support variable arguments, 
    // but doesn't rely on the string fitting into the buffer size (needed for GF2 credits scrolling)

	bool bDrawnAnything = false;

	//Only attempt to print text if a font has been loaded and we have a valid string
	if( !s_bAnyFontHasLoaded || !pcString )
	{
		return false;
	}

	// check for special character which scales the text further
	fScale *= GetInlinedScaleFactor(pcString);

	//We're going to need a 1:1 scale for button glyphs (because obviously the glyphs don't scale
	//when you change fonts).
	const float fGlyphScale = fScale;

	//For everything else, apply a scale factor per font.
    const Asura_Font* pxFont = s_apxFontStorage[s_iCurrentFontID];
	fScale *= pxFont->m_fFontScale;
	
	// How high will each line get us ?
	const float fPlatformYScale = s_bUsePlatformYScaleFactor ? GetPlatformYScaleFactor() : 1.0f;
	const float fCharacterHeight = pxFont->GetFontHeight() * fScale * fPlatformYScale;
	const float fLineHeight		 = fCharacterHeight * fLineSpacing;

	// Create a version of the clip rectangle in pixel coordinates, and scale the X and Y offsets to match.
	Asura_Bounding_Rectangle xClipPixelRect;
	GetPixelClipRectAndScaleOffsets(xClipPixelRect, fXOffset, fYOffset, pxClipRect, uFormat);

	const float fRectWidthPixels = xClipPixelRect.MaxX - xClipPixelRect.MinX;
	const float fRectHeightPixels = xClipPixelRect.MaxY - xClipPixelRect.MinY;
	if ( (fRectWidthPixels<=0.0f) || (fRectHeightPixels<=0.0f) )
	{
		Asura_Assert( fRectWidthPixels >= 0.f && fRectHeightPixels >= 0.f, "Invalid Rectangle" );
		return false;
	}

	// work out the text height so we can vertical center or bottom justify
	// (MH - 15.12.2003) note that all the steps done up to this point are 
	// done in the same way in the public function GetHeightOfText so changes
	// to the way text height is calculated should be copied into that function.
	float fTextHeight = GetTextHeight(pcString, uFormat, fCharacterHeight, fRectWidthPixels, fScale, fLineHeight);

	// Now where to start ...
    float x = s_fMaxPrintedXCoord = fXOffset;
	float y = s_fLastPrintedYHeight = fYOffset;

    if (uFormat & ASURA_TEXTFORMAT_RELATIVETORECT)
    {
        if (uFormat & ASURA_TEXTFORMAT_WORDWRAP) 
            x = 0.f;
        x += xClipPixelRect.MinX;
        y += xClipPixelRect.MinY;
    }
    

	// Update the vertical start point to take into account vertical centering
	if ( uFormat & ASURA_TEXTFORMAT_VERTICALCENTREJUSTIFIED )
	{
        if (uFormat & ASURA_TEXTFORMAT_RELATIVETORECT)
        {
		    // centred so subtract half of the height
		    y += 0.5f * (fRectHeightPixels - fTextHeight);
        }
        else
        {
            y -= 0.5f * fTextHeight;
        }
	}

	// Update the vertical start point to take into account vertical bottom justified
	if ( uFormat & ASURA_TEXTFORMAT_VERTICALBOTTOMJUSTIFIED )
	{
		// bottom justified so subtract entire string height from position
        if (uFormat & ASURA_TEXTFORMAT_RELATIVETORECT)
        {
		    y += fRectHeightPixels - fTextHeight;
        }
        else
        {
            y -= fTextHeight;
        }
        
		s_fLastPrintedYHeight = y;

	}

    // Deal with whole pixel stuff and ensure we don't fix up the start
    // position to be outside the clip rect when it wasn't before.
    {
        float fUnadjustedX = x;
        float fUnadjustedY = y;

        x = AdjustCoordToWholePixel( x );
	    y = AdjustCoordToWholePixel( y );

        // (MP - 10.12.2005) Ensure start point is inside the clip rect.
        if( dbg_bUseExactPixelBoundaries )
        {
            if( fUnadjustedX >= xClipPixelRect.MinX && x < xClipPixelRect.MinX ) { x += 1.0f; }

            if( fUnadjustedY >= xClipPixelRect.MinY && y < xClipPixelRect.MinY ) { y += 1.0f; }
        }
    }

	// Now print the string

	const Asura_UnicodeChar* wszLeftToPrint = pcString;
	const Asura_UnicodeChar* wszNextBit = NULL;
    bool bStringEnded = false;

    while( !bStringEnded )
    {   
        // Determine which bit of the text we're printing next. We can make
        // use of FindSplitPoint_Unscaled() here even if we're not actually
        // wordwrapping, since then it's equivalent to pretend we are but
        // with an effectively infinite width limit.

        float fLineWidth;
        int iNumWordsRemaining;
        bool bPrintHyphen;

#ifdef ASURA_TOOLS
		float fNextToolsColumn = fXOffset + dbg_fToolsColumnSpacing;
#endif

        wszNextBit = FindSplitPoint_Unscaled( wszLeftToPrint,
                                              ((uFormat & ASURA_TEXTFORMAT_WORDWRAP) ? fRectWidthPixels : Asura_Maths::LargeFloat),
                                              fScale,
                                              &fLineWidth,
                                              &iNumWordsRemaining,
                                              &bPrintHyphen );
        
        bStringEnded = !*wszNextBit;

        float fStartX = x;
		y = AdjustCoordToWholePixel( y );
		float fEndY	= y + fCharacterHeight;

		// If text alignment centered move start location of string accordingly
		if ( uFormat & ASURA_TEXTFORMAT_CENTREJUSTIFIED )
		{
			//centred so subtract half of this width
            if (uFormat & ASURA_TEXTFORMAT_RELATIVETORECT)
            {
				fStartX += 0.5f * (fRectWidthPixels - fLineWidth);
            }
            else
            {
                fStartX -= 0.5f * fLineWidth;
            }

		}

		// If text alignment right move start location of string accordingly
		if ( uFormat & ASURA_TEXTFORMAT_RIGHTJUSTIFIED )
		{
			//right justified so subtract entire string length from position
            if (uFormat & ASURA_TEXTFORMAT_RELATIVETORECT)
            {
				fStartX += fRectWidthPixels - fLineWidth;
            }
            else
            {
                fStartX -= fLineWidth;
            }
		}
	
		// if text fully justified, calculate how much extra space should be added between words to line both L and R edges.
		//	Ignore the last line.
		// And ignore lines ending in a newline character.
        float fWordSpacingRemaining;
		if( (uFormat & ASURA_TEXTFORMAT_FULLYJUSTIFIED) && iNumWordsRemaining > 1 && !bStringEnded && (*(wszNextBit-1)!= '\n'))
		{
			fWordSpacingRemaining = fRectWidthPixels - fLineWidth;
		}
        else
        {
            fWordSpacingRemaining = 0.0f;

            // Might as well ensure iNumWordsRemaining is 0 rather than
            // possibly 1, to be consistent with fWordSpacingRemaining.
            iNumWordsRemaining = 0;
        }

		// Check if this line of text is outside the top/bottom of the
        // rectangle. If so, skip it.
		if( y <= xClipPixelRect.MaxY && fEndY >= xClipPixelRect.MinY )
		{
            bool bPrevCharWasSpace = false;
            float fSpacingForLastChar = 0.0f;

			// It at least partially inside the rectangle
            const Asura_Font_GlyphPage* pxGlyphPage = NULL;
            const Asura_Font_Glyph* pxThisGlyph;
			for( ; wszLeftToPrint < wszNextBit && fStartX <= xClipPixelRect.MaxX; wszLeftToPrint++ )
			{
                Asura_UnicodeChar wCurChar = *wszLeftToPrint;

                if( L'\n' == wCurChar )
                {
                    // We're presumably heading onto the last line now.
                    // This should be the last character we're printing
                    // on this line, and we should probably only get here
                    // when wordwrapping.
                    continue;
                }

#ifdef ASURA_TOOLS
				// DMR 8/3/07 - 'bell' character used as the column separator
				// for faking with lists grids in tools
				if( L'\a' == wCurChar )
				{
					if (fStartX<fNextToolsColumn)
					{
						fStartX=fNextToolsColumn;
					}
					fNextToolsColumn += dbg_fToolsColumnSpacing;
					continue;
				}
#endif
				

                pxThisGlyph = pxFont->GetGlyphData( wCurChar, uFormat, &pxGlyphPage );
                if( !pxThisGlyph )
                {
                    continue;
                }

                fStartX += fSpacingForLastChar;

				// for each space, add the word spacing. This will be zero unless ASURA_TEXTFORMAT_FULLYJUSTIFY is set, 
				//	in which case it will have been calculated for this line to align both left and right edges of text.
                {
                    bool bSpace = ( L' ' == wCurChar );

                    // Note that for consecutive spaces, we only apply the
                    // justification spacing once, since we based our
                    // calculation on the number of words, which is
                    // independent of the size of the spacing between them.
                    // Besides, it would look bad anyway.
				    if( bSpace && !bPrevCharWasSpace && fWordSpacingRemaining )
				    {
                        // There'd better be some words left for us to use
                        // up that spacing then.
                        Asura_Assert( iNumWordsRemaining, "" );

                        --iNumWordsRemaining;

                        float fAdditionalSpacing;
                        if( iNumWordsRemaining > 1 )
                        {
                            fAdditionalSpacing = static_cast<float>( Asura_Maths::FloatToInt_Near(fWordSpacingRemaining / iNumWordsRemaining) );
                            Asura_Assert( fAdditionalSpacing <= fWordSpacingRemaining, "" );
                        }
                        else
                        {
                            // This is the last word coming up so use the
                            // remainder of the spacing.
							fAdditionalSpacing = static_cast<float>( Asura_Maths::FloatToInt_Chop(fWordSpacingRemaining) );
                        }

					    fStartX += fAdditionalSpacing;
					    fWordSpacingRemaining -= fAdditionalSpacing;
				    }

                    bPrevCharWasSpace = bSpace;
                }

                fStartX = AdjustCoordToWholePixel( fStartX );
				if( !(uFormat & ASURA_TEXTFORMAT_DONTRENDER) )
				{
					if (s_bNextGlyphRequiresProjectRender)
					{
						bDrawnAnything |= Project_PrintChar( &xClipPixelRect, fStartX, y, fGlyphScale, uColour, wCurChar );
					}
					else
					{
						bDrawnAnything |= PrintChar( &xClipPixelRect, fStartX, y, fScale, uColour, pxThisGlyph, pxGlyphPage );
					}
				}

				fStartX += GetWidthOfChar_Unscaled( wCurChar, fScale );

                // Note that the next character isn't technically correct
                // here if we're on the last one we're going to print but
                // that's ok since the spacing value won't then get used
                // anyway.
                fSpacingForLastChar = GetCharSpacing( wCurChar, *(wszLeftToPrint + 1), fScale );
			}

            if( bPrintHyphen )
            {
                Asura_UnicodeChar wHyphen = L'-';

                pxThisGlyph = pxFont->GetGlyphData( wHyphen, uFormat, &pxGlyphPage );

                fStartX = AdjustCoordToWholePixel( fStartX + GetCharSpacing(*(wszLeftToPrint - 1), wHyphen, fScale) );
				if( !(uFormat & ASURA_TEXTFORMAT_DONTRENDER) )
				{	
					if (s_bNextGlyphRequiresProjectRender)
					{
						bDrawnAnything |= Project_PrintChar( &xClipPixelRect, fStartX, y, fGlyphScale, uColour, wHyphen );
					}
					else
					{
						bDrawnAnything |= PrintChar( &xClipPixelRect, fStartX, y, fScale, uColour, pxThisGlyph, pxGlyphPage );
					}
				}

                fStartX += GetWidthOfChar_Unscaled( wHyphen, fScale );
            }
			
			s_fLastPrintedXCoord = fStartX;
			if(s_fMaxPrintedXCoord < fStartX) s_fMaxPrintedXCoord = fStartX;

			if( uFormat & ASURA_TEXTFORMAT_FULLYJUSTIFIED )
			{
                // (MP - 14.03.2006) Used to check against 0.1f but due to
                // the whole pixel stuff and not wanting to go outside the
                // clip bounds at all we may not use up all of it. However,
                // we shouldn't end up with a whole pixel outstanding.
				Asura_Assert( fWordSpacingRemaining < 1.0f, "" );

                // (MP - 14.03.2006) This used to assert that 0 words
                // remained but actually that may not be the case. Our
                // justification code attempts to space out the words
                // excluding the last space, meaning that if it gets it just
                // right then there'll be no room left for the final space
                // character(s) and the word count won't get its final
                // decrement above.
				Asura_Assert( iNumWordsRemaining <= 1, "" );
			}
		}

		// Move down to the next line
        y += fLineHeight;

        wszLeftToPrint = wszNextBit; 
    }

    // Store the final y coordinate (the start of the line after the text)
	if( !(uFormat & ASURA_TEXTFORMAT_VERTICALBOTTOMJUSTIFIED) )
	{
		s_fLastPrintedYHeight = y;
	}
	// And convert it back to the coordinate system being passed in
    const Asura_ViewingArea* pxViewArea = Asura_Camera::GetViewingArea();
	if( (uFormat & ASURA_TEXTFORMAT_CAMERARELATIVEPOSITION) && (pxViewArea) )
	{
		s_fLastPrintedYHeight -= pxViewArea->m_fYOffset * Asura_Viewport::GetHeight();
		s_fLastPrintedYHeight /= pxViewArea->m_fHeight  * Asura_Viewport::GetHeight();		

		s_fLastPrintedXCoord -= pxViewArea->m_fXOffset * Asura_Viewport::GetWidth();
		s_fLastPrintedXCoord /= pxViewArea->m_fWidth   * Asura_Viewport::GetWidth();

		s_fMaxPrintedXCoord -= pxViewArea->m_fXOffset * Asura_Viewport::GetWidth();
		s_fMaxPrintedXCoord /= pxViewArea->m_fWidth   * Asura_Viewport::GetWidth();
	}

	if( uFormat & ASURA_TEXTFORMAT_RELATIVEPOSITION )
	{
		s_fLastPrintedYHeight /= Asura_Viewport::GetHeight();
		s_fLastPrintedXCoord  /= Asura_Viewport::GetWidth();
		s_fMaxPrintedXCoord  /= Asura_Viewport::GetWidth();
	}

	return bDrawnAnything;
}

// Default function that supports Teletype, decryption and cursor rendering.
bool Asura_Font_System::DefaultRenderSpecialEffects(Asura_Font_System::Render_Effects_Glyph_Data* pxCharData)
{
	bool bDrawnAnything = false;

	Asura_Assert(pxCharData->m_pxSpecialEffectsSettings != NULL,"pxSpecialEffectsSettings == NULL" );

	// The the age of the character
	float fCharAge = pxCharData->m_pxSpecialEffectsSettings->CalcCharacterAge(pxCharData->m_uCharCount,
		pxCharData->m_uStringLength, 
		pxCharData->m_uStartLinePos,
		pxCharData->m_uEndLinePos);

	if ( pxCharData->m_bRenderCursor == true )
	{
		if ( pxCharData->m_pxSpecialEffectsSettings->GetCursor() == false)
		{
			// If its time to render a cursor and its not required by the effect early out.
			return bDrawnAnything;
		}

		// Is the cursor being rendered at the end of the string. If so start it flashing.
		if ( pxCharData->m_uCharCount >= pxCharData->m_uStringLength )
		{
			if (  pxCharData->m_pxSpecialEffectsSettings->GetCursorStopTime() > 0.0f &&
				  fCharAge > pxCharData->m_pxSpecialEffectsSettings->GetCursorStopTime() )
			{
				// The cursor has past it stop time so don't render it anymore
				return bDrawnAnything;
			}

			// Get the timer and test if the cursor should be on or off.
			float fBlinkRate = pxCharData->m_pxSpecialEffectsSettings->GetCursorFlashTime();

			if (fBlinkRate > 0.0f )
			{
				float fTime = pxCharData->m_pxSpecialEffectsSettings->GetTime();

				int iBlink = static_cast<int>(fTime * 1000.0f) % static_cast<int>((fBlinkRate * 2.0f) * 1000.0f);

				if ( iBlink < (fBlinkRate * 1000.0f) )
				{
					// The cursor has blinked off so don't render anything
					return bDrawnAnything;
				}
			}
		}
	}

	// If a cursor is being rendered or teletyping is off or the character is old enough to render,
	// then render it.
	if ( pxCharData->m_bRenderCursor == true ||
		 pxCharData->m_pxSpecialEffectsSettings->GetTeletype() == false || 
		 fCharAge > 0.0f )
	{
		u_int uNewColour = pxCharData->m_uColour;

		// Modify the colour for the alpha fade effect
		if ( pxCharData->m_bRenderCursor == false )
		{
			Asura_RGBA xColour(pxCharData->m_uColour);

			// Apply the alpha for the teletype effect
			xColour.m_fA *= pxCharData->m_pxSpecialEffectsSettings->GetAlpha(fCharAge);

			Asura_Font_System::s_uCharacterColour = xColour.ToUIntARGB(true);
		}

		// If decryption effect is one pick a random character
		if ( pxCharData->m_wCurChar != L' ' &&
			pxCharData->m_bRenderCursor == false &&
			 pxCharData->m_pxSpecialEffectsSettings->GetDecrypt() == true &&
			 pxCharData->m_pxSpecialEffectsSettings->GetDecryptTime() > fCharAge &&
			  pxCharData->m_pxSpecialEffectsSettings->GetDecryptChangeCharTime() > 0.0f)
		{
			// Replace char with a random one.
			const Asura_Font* pxFont = pxCharData->m_pxFont;

			const Asura_Font_GlyphPage* pxGlyphPage = pxCharData->m_pxGlyphPage;
			const Asura_Font_Glyph* pxThisGlyph = pxCharData->m_pxThisGlyph;
			Asura_UnicodeChar wCurChar = pxCharData->m_wCurChar;

			u_int encryptChar = wCurChar + pxCharData->m_uCharCount + static_cast<u_int>(fCharAge / pxCharData->m_pxSpecialEffectsSettings->GetDecryptChangeCharTime());

			if ( wCurChar >= L'A' && wCurChar <= L'Z')
			{
				// Capital letters
				encryptChar = encryptChar % 25;
				wCurChar = static_cast<Asura_UnicodeChar>(encryptChar+'A');
			}
			else if ( wCurChar >= L'a' && wCurChar <= L'z')
			{
				// Lower Case
				encryptChar = encryptChar % 25;
				wCurChar = static_cast<Asura_UnicodeChar>(encryptChar+'a');
			}
			else if ( wCurChar >= L'0' && wCurChar <= L'9')
			{
				encryptChar = encryptChar % 10;
				wCurChar = static_cast<Asura_UnicodeChar>(encryptChar+'0');
			}
			else
			{
				encryptChar = encryptChar % 25;
				wCurChar = static_cast<Asura_UnicodeChar>(encryptChar+'A');
			}

			pxThisGlyph = pxFont->GetGlyphData( wCurChar, pxCharData->m_uFormat, &pxGlyphPage );
			if( !pxThisGlyph )
			{
				return bDrawnAnything;
			}

			// Draw the random decryption character
			if (pxCharData->m_bGlyphRequiresProjectRender)
			{
				bDrawnAnything |= Asura_Font_System::Project_PrintChar( &pxCharData->m_xClipPixelRect, 
					pxCharData->m_fX, 
					pxCharData->m_fY, 
					pxCharData->m_fGlyphScale, 
					uNewColour, 
					wCurChar );
			}
			else
			{
				bDrawnAnything |= Asura_Font_System::PrintChar( &pxCharData->m_xClipPixelRect, 
					pxCharData->m_fX, 
					pxCharData->m_fY, 
					pxCharData->m_fScale, 
					uNewColour, 
					pxThisGlyph, 
					pxGlyphPage );
			}
		}
		else
		{
			// Render the normal character
			if (pxCharData->m_bGlyphRequiresProjectRender)
			{
				bDrawnAnything |= Asura_Font_System::Project_PrintChar( &pxCharData->m_xClipPixelRect, 
														pxCharData->m_fX, 
														pxCharData->m_fY, 
														pxCharData->m_fGlyphScale, 
														uNewColour, 
														pxCharData->m_wCurChar );
			}
			else
			{
				bDrawnAnything |= Asura_Font_System::PrintChar( &pxCharData->m_xClipPixelRect, 
												pxCharData->m_fX, 
												pxCharData->m_fY, 
												pxCharData->m_fScale, 
												uNewColour, 
												pxCharData->m_pxThisGlyph, 
												pxCharData->m_pxGlyphPage );
			}
		}

		// Restore the colour to what it was
		Asura_Font_System::s_uCharacterColour = pxCharData->m_uColour;
	}

	return bDrawnAnything;
}

bool Asura_Font_System::PrintSpecialEffectsString( const Asura_UnicodeChar*  wszString,           // The text
													const Asura_Bounding_Rectangle* pxClipRect,		// The rectangle the text should be clipped to (NULL if full screen)
													float fXOffset,						            // X Offset inside the rectangle so you can scroll the text left/right - Has no effect if WORDWRAPPING flag is set
													float fYOffset,						            // Y Offset inside the rectangle so you can scroll the text up/down
													const float fLineSpacing,					    // Line spacing multiplier
													float fScale,							        // Scale size for the font when it's drawn
													const u_int uColour,							// The colour to print in
													const u_int uFormat,							// Text formating flags see Asura_Font_System.h
													const Asura_Font_Special_Effects* pxSpecialEffectsSettings)
{
	Asura_Assert(pxSpecialEffectsSettings != NULL,"pxSpecialEffectsSettings is NULL");
	Asura_Assert(s_pfnRenderSpecialEffectCharacterCallback != NULL,"s_pfnRenderSpecialEffectCharacterCallback is NULL");

	Render_Effects_Glyph_Data xCharData;

	xCharData.m_wszString = wszString;
	xCharData.m_fScale = fScale;
	xCharData.m_uColour = uColour;
	xCharData.m_pxSpecialEffectsSettings = pxSpecialEffectsSettings;
	xCharData.m_uFormat = uFormat;
	xCharData.m_bRenderCursor = false;	

	s_uCharacterColour = uColour;
	s_uStringColour	   = s_uCharacterColour;
	s_uCharacterFormat = uFormat;


	// LBF: the following doesn't support variable arguments, 
	// but doesn't rely on the string fitting into the buffer size (needed for GF2 credits scrolling)

	bool bDrawnAnything = false;

	if ( pxSpecialEffectsSettings->GetTime() < pxSpecialEffectsSettings->GetDelayTime() )
	{
		return false;
	}

	//Only attempt to print text if a font has been loaded and we have a valid string
	if( !s_bAnyFontHasLoaded || !wszString )
	{
		return false;
	}

	// check for special character which scales the text further
	xCharData.m_fScale *= GetInlinedScaleFactor(wszString);

	//We're going to need a 1:1 scale for button glyphs (because obviously the glyphs don't scale
	//when you change fonts).
	xCharData.m_fGlyphScale = xCharData.m_fScale;

	//For everything else, apply a scale factor per font.
	const Asura_Font* pxFont = s_apxFontStorage[s_iCurrentFontID];

	xCharData.m_pxFont = pxFont;
	xCharData.m_fScale *= pxFont->m_fFontScale;

	// How high will each line get us ?
	const float fPlatformYScale = s_bUsePlatformYScaleFactor ? GetPlatformYScaleFactor() : 1.0f;
	const float fCharacterHeight = pxFont->GetFontHeight() * xCharData.m_fScale * fPlatformYScale;
	const float fLineHeight		 = fCharacterHeight * fLineSpacing;

	// Create a version of the clip rectangle in pixel coordinates, and scale the X and Y offsets to match.
	GetPixelClipRectAndScaleOffsets(xCharData.m_xClipPixelRect, fXOffset, fYOffset, pxClipRect, uFormat);

	const float fRectWidthPixels = xCharData.m_xClipPixelRect.MaxX - xCharData.m_xClipPixelRect.MinX;
	const float fRectHeightPixels = xCharData.m_xClipPixelRect.MaxY - xCharData.m_xClipPixelRect.MinY;
	if ( (fRectWidthPixels<=0.0f) || (fRectHeightPixels<=0.0f) )
	{
		Asura_Assert( fRectWidthPixels >= 0.f && fRectHeightPixels >= 0.f, "Invalid Rectangle" );
		return false;
	}

	// work out the text height so we can vertical center or bottom justify
	// (MH - 15.12.2003) note that all the steps done up to this point are 
	// done in the same way in the public function GetHeightOfText so changes
	// to the way text height is calculated should be copied into that function.
	float fTextHeight = GetTextHeight(wszString, uFormat, fCharacterHeight, fRectWidthPixels, xCharData.m_fScale, fLineHeight);

	// Now where to start ...
	float x = s_fMaxPrintedXCoord = fXOffset;
	float y = s_fLastPrintedYHeight = fYOffset;

	if (uFormat & ASURA_TEXTFORMAT_RELATIVETORECT)
	{
		if (uFormat & ASURA_TEXTFORMAT_WORDWRAP) 
		{
			x = 0.f;
		}
		x += xCharData.m_xClipPixelRect.MinX;
		y += xCharData.m_xClipPixelRect.MinY;
	}


	// Update the vertical start point to take into account vertical centering
	if ( uFormat & ASURA_TEXTFORMAT_VERTICALCENTREJUSTIFIED )
	{
		if (uFormat & ASURA_TEXTFORMAT_RELATIVETORECT)
		{
			// centered so subtract half of the height
			y += 0.5f * (fRectHeightPixels - fTextHeight);
		}
		else
		{
			y -= 0.5f * fTextHeight;
		}
	}

	// Update the vertical start point to take into account vertical bottom justified
	if ( uFormat & ASURA_TEXTFORMAT_VERTICALBOTTOMJUSTIFIED )
	{
		// bottom justified so subtract entire string height from position
		if (uFormat & ASURA_TEXTFORMAT_RELATIVETORECT)
		{
			y += fRectHeightPixels - fTextHeight;
		}
		else
		{
			y -= fTextHeight;
		}

		s_fLastPrintedYHeight = y;

	}

	// Deal with whole pixel stuff and ensure we don't fix up the start
	// position to be outside the clip rect when it wasn't before.
	{
		float fUnadjustedX = x;
		float fUnadjustedY = y;

		x = AdjustCoordToWholePixel( x );
		y = AdjustCoordToWholePixel( y );

		// (MP - 10.12.2005) Ensure start point is inside the clip rect.
		if( dbg_bUseExactPixelBoundaries )
		{
		if( fUnadjustedX >= xCharData.m_xClipPixelRect.MinX && x < xCharData.m_xClipPixelRect.MinX ) { x += 1.0f; }

		if( fUnadjustedY >= xCharData.m_xClipPixelRect.MinY && y < xCharData.m_xClipPixelRect.MinY ) { y += 1.0f; }
		}
	}

	// Now print the string

	const Asura_UnicodeChar* wszLeftToPrint = wszString;
	const Asura_UnicodeChar* wszNextBit = NULL;
	bool bStringEnded = false;


	xCharData.m_uCharCount = 0;
	xCharData.m_uStringLength = wcslen(wszString);

	u_int uCursorPos = xCharData.m_uStringLength;

	if ( pxSpecialEffectsSettings->GetTeletype() == true )
	{
		uCursorPos = pxSpecialEffectsSettings->GetNumChars()+1;

		if ( uCursorPos > xCharData.m_uStringLength)
		{
			uCursorPos = xCharData.m_uStringLength;
		}
	}

	while( !bStringEnded )
	{   
		// Determine which bit of the text we're printing next. We can make
		// use of FindSplitPoint_Unscaled() here even if we're not actually
		// wordwrapping, since then it's equivalent to pretend we are but
		// with an effectively infinite width limit.

		float fLineWidth;
		int iNumWordsRemaining;
		bool bPrintHyphen;

#ifdef ASURA_TOOLS
		//float fNextToolsColumn = fXOffset + dbg_fToolsColumnSpacing;
#endif

		wszNextBit = FindSplitPoint_Unscaled( wszLeftToPrint,
			((uFormat & ASURA_TEXTFORMAT_WORDWRAP) ? fRectWidthPixels : Asura_Maths::LargeFloat),
			xCharData.m_fScale,
			&fLineWidth,
			&iNumWordsRemaining,
			&bPrintHyphen );

		u_int uLineLength = wszNextBit - wszLeftToPrint;
		xCharData.m_uStartLinePos = wszLeftToPrint - wszString;
		xCharData.m_uEndLinePos = xCharData.m_uStartLinePos+uLineLength;

		bStringEnded = !*wszNextBit;

		float fStartX = x;
		//y = AdjustCoordToWholePixel( y );
		float fEndY	= y + fCharacterHeight;

		// If text alignment centered move start location of string accordingly
		if ( uFormat & ASURA_TEXTFORMAT_CENTREJUSTIFIED )
		{
			//centred so subtract half of this width
			if (uFormat & ASURA_TEXTFORMAT_RELATIVETORECT)
			{
				fStartX += 0.5f * (fRectWidthPixels - fLineWidth);
			}
			else
			{
				fStartX -= 0.5f * fLineWidth;
			}

		}

		// If text alignment right move start location of string accordingly
		if ( uFormat & ASURA_TEXTFORMAT_RIGHTJUSTIFIED )
		{
			//right justified so subtract entire string length from position
			if (uFormat & ASURA_TEXTFORMAT_RELATIVETORECT)
			{
				fStartX += fRectWidthPixels - fLineWidth;
			}
			else
			{
				fStartX -= fLineWidth;
			}
		}

		// if text fully justified, calculate how much extra space should be added between words to line both L and R edges.
		//	Ignore the last line.
		// And ignore lines ending in a newline character.
		float fWordSpacingRemaining;
		if( (uFormat & ASURA_TEXTFORMAT_FULLYJUSTIFIED) && iNumWordsRemaining > 1 && !bStringEnded && (*(wszNextBit-1)!= '\n'))
		{
			fWordSpacingRemaining = fRectWidthPixels - fLineWidth;
		}
		else
		{
			fWordSpacingRemaining = 0.0f;

			// Might as well ensure iNumWordsRemaining is 0 rather than
			// possibly 1, to be consistent with fWordSpacingRemaining.
			iNumWordsRemaining = 0;
		}

		// Check if this line of text is outside the top/bottom of the
		// rectangle. If so, skip it.
		if( y <= xCharData.m_xClipPixelRect.MaxY && fEndY >= xCharData.m_xClipPixelRect.MinY )
		{
			bool bPrevCharWasSpace = false;
			float fSpacingForLastChar = 0.0f;

			// It at least partially inside the rectangle
			xCharData.m_pxGlyphPage = NULL;

			for( ; wszLeftToPrint < wszNextBit && fStartX <= xCharData.m_xClipPixelRect.MaxX; wszLeftToPrint++ )
			{
				xCharData.m_wCurChar = *wszLeftToPrint;

				if( L'\n' == xCharData.m_wCurChar )
				{
					// We're presumably heading onto the last line now.
					// This should be the last character we're printing
					// on this line, and we should probably only get here
					// when wordwrapping.
					continue;
				}

				xCharData.m_pxThisGlyph = pxFont->GetGlyphData( xCharData.m_wCurChar, uFormat, &xCharData.m_pxGlyphPage );
				if( !xCharData.m_pxThisGlyph )
				{
					continue;
				}

				fStartX += fSpacingForLastChar;

				// for each space, add the word spacing. This will be zero unless ASURA_TEXTFORMAT_FULLYJUSTIFY is set, 
				//	in which case it will have been calculated for this line to align both left and right edges of text.
				{
					bool bSpace = ( L' ' == xCharData.m_wCurChar );

					// Note that for consecutive spaces, we only apply the
					// justification spacing once, since we based our
					// calculation on the number of words, which is
					// independent of the size of the spacing between them.
					// Besides, it would look bad anyway.
					if( bSpace && !bPrevCharWasSpace && fWordSpacingRemaining )
					{
						// There'd better be some words left for us to use
						// up that spacing then.
						Asura_Assert( iNumWordsRemaining, "" );

						--iNumWordsRemaining;

						float fAdditionalSpacing;
						if( iNumWordsRemaining > 1 )
						{
							fAdditionalSpacing = static_cast<float>( Asura_Maths::FloatToInt_Near(fWordSpacingRemaining / iNumWordsRemaining) );
							Asura_Assert( fAdditionalSpacing <= fWordSpacingRemaining, "" );
						}
						else
						{
							// This is the last word coming up so use the
							// remainder of the spacing.
							fAdditionalSpacing = static_cast<float>( Asura_Maths::FloatToInt_Chop(fWordSpacingRemaining) );
						}

						fStartX += fAdditionalSpacing;
						fWordSpacingRemaining -= fAdditionalSpacing;
					}

					bPrevCharWasSpace = bSpace;
				}

				fStartX = AdjustCoordToWholePixel( fStartX );

				// Call the Render function to render this Glyph. Projects can supply their own render
				// function for special effects.
				if( !(uFormat & ASURA_TEXTFORMAT_DONTRENDER) )
				{
					xCharData.m_fX = fStartX;
					xCharData.m_fY = y;
					xCharData.m_bGlyphRequiresProjectRender = s_bNextGlyphRequiresProjectRender;

					bDrawnAnything |= s_pfnRenderSpecialEffectCharacterCallback(&xCharData);
				}

				fStartX += GetWidthOfChar_Unscaled( xCharData.m_wCurChar, xCharData.m_fScale );

				// Note that the next character isn't technically correct
				// here if we're on the last one we're going to print but
				// that's ok since the spacing value won't then get used
				// anyway.
				fSpacingForLastChar = GetCharSpacing( xCharData.m_wCurChar, *(wszLeftToPrint + 1), xCharData.m_fScale );

				xCharData.m_uCharCount++;

				if ( uCursorPos == xCharData.m_uCharCount )
				{
					// Need to draw Caret
					xCharData.m_wCurChar = L'_';

					xCharData.m_pxThisGlyph = pxFont->GetGlyphData( xCharData.m_wCurChar, uFormat, &xCharData.m_pxGlyphPage );

					//fStartX = AdjustCoordToWholePixel( fStartX + GetCharSpacing(*(wszLeftToPrint - 1), wHyphen, fScale) );
					fStartX = fStartX + GetCharSpacing(*(wszLeftToPrint - 1), xCharData.m_wCurChar, xCharData.m_fScale);
					if( !(uFormat & ASURA_TEXTFORMAT_DONTRENDER) )
					{
						xCharData.m_fX = fStartX;
						xCharData.m_fY = y;
						xCharData.m_bGlyphRequiresProjectRender = s_bNextGlyphRequiresProjectRender;
						xCharData.m_bRenderCursor = true;	

						bDrawnAnything |= s_pfnRenderSpecialEffectCharacterCallback(&xCharData);

						xCharData.m_bRenderCursor = false;
					}
				}
			}

			if( bPrintHyphen )
			{
				Asura_UnicodeChar wHyphen = L'-';

				xCharData.m_pxThisGlyph = pxFont->GetGlyphData( wHyphen, uFormat, &xCharData.m_pxGlyphPage );

				//fStartX = AdjustCoordToWholePixel( fStartX + GetCharSpacing(*(wszLeftToPrint - 1), wHyphen, fScale) );
				fStartX = fStartX + GetCharSpacing(*(wszLeftToPrint - 1), wHyphen, xCharData.m_fScale);
				if( !(uFormat & ASURA_TEXTFORMAT_DONTRENDER) )
				{	
					if (s_bNextGlyphRequiresProjectRender)
					{
						bDrawnAnything |= Project_PrintChar( &xCharData.m_xClipPixelRect, fStartX, y, xCharData.m_fGlyphScale, xCharData.m_uColour, wHyphen );
					}
					else
					{
						bDrawnAnything |= PrintChar( &xCharData.m_xClipPixelRect, fStartX, y, xCharData.m_fScale, xCharData.m_uColour, xCharData.m_pxThisGlyph, xCharData.m_pxGlyphPage );
					}
				}

				fStartX += GetWidthOfChar_Unscaled( wHyphen, xCharData.m_fScale );
			}

			s_fLastPrintedXCoord = fStartX;
			if(s_fMaxPrintedXCoord < fStartX) s_fMaxPrintedXCoord = fStartX;

			if( uFormat & ASURA_TEXTFORMAT_FULLYJUSTIFIED )
			{
				// (MP - 14.03.2006) Used to check against 0.1f but due to
				// the whole pixel stuff and not wanting to go outside the
				// clip bounds at all we may not use up all of it. However,
				// we shouldn't end up with a whole pixel outstanding.
				Asura_Assert( fWordSpacingRemaining < 1.0f, "" );

				// (MP - 14.03.2006) This used to assert that 0 words
				// remained but actually that may not be the case. Our
				// justification code attempts to space out the words
				// excluding the last space, meaning that if it gets it just
				// right then there'll be no room left for the final space
				// character(s) and the word count won't get its final
				// decrement above.
				Asura_Assert( iNumWordsRemaining <= 1, "" );
			}
		}
		else
		{
			xCharData.m_uCharCount += uLineLength;
		}

		// Move down to the next line
		y += fLineHeight;

		wszLeftToPrint = wszNextBit; 
	}

	// Store the final y coordinate (the start of the line after the text)
	if( !(uFormat & ASURA_TEXTFORMAT_VERTICALBOTTOMJUSTIFIED) )
	{
		s_fLastPrintedYHeight = y;
	}
	// And convert it back to the coordinate system being passed in
	const Asura_ViewingArea* pxViewArea = Asura_Camera::GetViewingArea();
	if( (uFormat & ASURA_TEXTFORMAT_CAMERARELATIVEPOSITION) && (pxViewArea) )
	{
		s_fLastPrintedYHeight -= pxViewArea->m_fYOffset * Asura_Viewport::GetHeight();
		s_fLastPrintedYHeight /= pxViewArea->m_fHeight  * Asura_Viewport::GetHeight();		

		s_fLastPrintedXCoord -= pxViewArea->m_fXOffset * Asura_Viewport::GetWidth();
		s_fLastPrintedXCoord /= pxViewArea->m_fWidth   * Asura_Viewport::GetWidth();

		s_fMaxPrintedXCoord -= pxViewArea->m_fXOffset * Asura_Viewport::GetWidth();
		s_fMaxPrintedXCoord /= pxViewArea->m_fWidth   * Asura_Viewport::GetWidth();
	}

	if( uFormat & ASURA_TEXTFORMAT_RELATIVEPOSITION )
	{
		s_fLastPrintedYHeight /= Asura_Viewport::GetHeight();
		s_fLastPrintedXCoord  /= Asura_Viewport::GetWidth();
		s_fMaxPrintedXCoord  /= Asura_Viewport::GetWidth();
	}

	return bDrawnAnything;
}

// public function to get total height of text when rendered in the clip rectangle
// Useful for dealing with scrollbars.
float Asura_Font_System::GetHeightOfText(const Asura_UnicodeChar*  wszString,           // The text
										const Asura_Bounding_Rectangle* pxClipRect,		// The rectangle the text should be clipped to (NULL if full screen)
										const float fLineSpacing,					    // Line spacing mulitplier
										float fScale,							        // Scale size for the font when it's drawn
										const u_int uFormat)							// Text formating flags see Asura_Font_System.h
{
	//Only attempt to print text if a font has been loaded and we have a valid string
	if( !s_bAnyFontHasLoaded || !wszString )
	{
		return 0.0f;
	}

	// check for special character which scales the text further
	fScale *= GetInlinedScaleFactor(wszString);
	fScale *= s_apxFontStorage[s_iCurrentFontID]->m_fFontScale;
	
	// How high will each line get us ?
	const float fPlatformYScale = s_bUsePlatformYScaleFactor ? GetPlatformYScaleFactor() : 1.0f;
	const float fCharacterHeight = s_apxFontStorage[s_iCurrentFontID]->GetFontHeight() * fScale * fPlatformYScale;
	const float fLineHeight		 = fCharacterHeight * fLineSpacing;

	// Create a version of the clip rectangle in pixel coordinates, and scale the X and Y offsets to match.
	Asura_Bounding_Rectangle xClipPixelRect;
    float fXOffset = 0.0f;
    float fYOffset = 0.0f;
	GetPixelClipRectAndScaleOffsets(xClipPixelRect, fXOffset, fYOffset, pxClipRect, uFormat);

	const float fRectWidthPixels = xClipPixelRect.MaxX - xClipPixelRect.MinX;
	const float fRectHeightPixels = xClipPixelRect.MaxY - xClipPixelRect.MinY;
	if ( (fRectWidthPixels<=0.0f) || (fRectHeightPixels<=0.0f) )
	{
		Asura_Assert( fRectWidthPixels >= 0.f && fRectHeightPixels >= 0.f, "Invalid Rectangle" );
		return false;
	}

	// work out the text height
	float fTextHeight = GetTextHeight(wszString, uFormat, fCharacterHeight, fRectWidthPixels, fScale, fLineHeight);
    
	// convert it back to the original coordinate system
	const Asura_ViewingArea* pxViewArea = Asura_Camera::GetViewingArea();
	if( (uFormat & ASURA_TEXTFORMAT_CAMERARELATIVEPOSITION) && (pxViewArea) )
	{
		fTextHeight -= pxViewArea->m_fYOffset * Asura_Viewport::GetHeight();
		fTextHeight /= pxViewArea->m_fHeight  * Asura_Viewport::GetHeight();		
	}
	if( uFormat & ASURA_TEXTFORMAT_RELATIVEPOSITION )
	{
		fTextHeight /= Asura_Viewport::GetHeight();
	}
	return fTextHeight;
}


float Asura_Font_System::GetWidthOfText(const Asura_UnicodeChar* wszString,				// The text
										const Asura_Bounding_Rectangle* pxClipRect,		// The rectangle the text should be clipped to (NULL if full screen)
										float fScale,							        // Scale size for the font when it's drawn
										const u_int uFormat)							// Text formating flags see Asura_Font_System.h
{
	//Only attempt to print text if a font has been loaded and we have a valid string
	if ( s_bAnyFontHasLoaded && (NULL != wszString) )
	{
		const Asura_Font*			pxFont		= s_apxFontStorage [s_iCurrentFontID];

		float						fXOffset	= 0.0f;
		float						fYOffset	= 0.0f;

		Asura_Bounding_Rectangle	xClipPixelRect;

		// check for special character which scales the text further
		fScale *= GetInlinedScaleFactor (wszString);
		fScale *= pxFont->m_fFontScale;

		// Create a version of the clip rectangle in pixel coordinates,
		// and scale the X and Y offsets to match...
		GetPixelClipRectAndScaleOffsets	(xClipPixelRect, fXOffset, fYOffset, pxClipRect, uFormat);

		// Is the clip-rectangle valid?
		if ( (xClipPixelRect.MinX >= xClipPixelRect.MaxX) || (xClipPixelRect.MinY >= xClipPixelRect.MaxY) )
		{
			Asura_Assert	(false, "Invalid clip-rectangle");
			return			(false);
		}

		else
		{
			const Asura_ViewingArea*	pxViewArea		= Asura_Camera::GetViewingArea();
			const float					fScreenWidth	= static_cast<float>(Asura_Viewport::GetWidth());

			float						fTextWidth		= GetTextWidth (wszString, uFormat, xClipPixelRect.GetWidth(), fScale);

			// Convert it back to the original coordinate system...
			if ( (0 != (uFormat & ASURA_TEXTFORMAT_CAMERARELATIVEPOSITION)) && (NULL != pxViewArea) )
			{
				fTextWidth	-= pxViewArea->m_fXOffset * fScreenWidth;
				fTextWidth	/= pxViewArea->m_fWidth   * fScreenWidth;
			}

			if (0 != (uFormat & ASURA_TEXTFORMAT_RELATIVEPOSITION))
			{
				fTextWidth	/= fScreenWidth;
			}

			// Return the width calculated...
			return	(fTextWidth);
		}
	}

	else
	{
		return 0.0f;
	}

}


bool Asura_Font_System::Print( const Asura_Bounding_Rectangle &xRect,		// The rectangle the text should be printed inside
							   const float fXOffset,						// X Offset inside the rectangle so you can scroll the text left/right - Has no effect if WORDWRAPPING flag is set
							   const float fYOffset,						// Y Offset inside the rectangle so you can scroll the text up/down
							   const float fLineSpacing,					// Line spaceing mulitplier 0.75 seems good for the font I'm testing it with
							   const float fScale,							// Scale size for the font when it's drawn
							   const u_int uColour,							// The colour to print in
							   const u_int uFormat,							// Text formating flags see Asura_Font_System.h
							   const Asura_Char* string, ...)				// The text
{
	//Convert Asura_Char string to unicode and call main Print function.

	//Do nothing if no fonts are loaded, or null string
	if(s_bAnyFontHasLoaded && string)
	{
		Asura_Char buffer[MAX_BUFFER_SIZE];
		Asura_UnicodeChar uniString[MAX_BUFFER_SIZE];
		if (uFormat & ASURA_TEXTFORMAT_FORMATSTRING)
		{
			va_list ap;
			va_start(ap, string);
			Asura_PrintfToBfr( buffer, string, ap, MAX_BUFFER_SIZE);
			va_end(ap);
		}
		else
		{
			Asura_PrintfToBfr( buffer, "%s", MAX_BUFFER_SIZE, true, string); 
		}

        for(int i = 0; i < MAX_BUFFER_SIZE; i++)
		{
			uniString[i] = (Asura_UnicodeChar)buffer[i];
		}
        return Print( xRect, fXOffset, fYOffset, fLineSpacing, fScale, uColour, uFormat, uniString );
	}

	return false;
}

float Asura_Font_System::GetFontHeight(void)
{
    // note this returns the adjusted height  displayed onscreen and shouldn't be used for texture coordinates
    const Asura_Font* pxFont = s_apxFontStorage[s_iCurrentFontID];
	if( pxFont )
	{
		const float fPlatformYScale = s_bUsePlatformYScaleFactor ? GetPlatformYScaleFactor() : 1.0f;
		return pxFont->GetFontHeight() * pxFont->m_fFontScale * fPlatformYScale;
	}

    return 0.0f;
}

void Asura_Font_System::DestroyFont( Asura_Handle iFontHandle )
{
    const Asura_Font* pxFont = GetFont( iFontHandle );

    // Get rid of the texture. Technically this is somewhat outside our remit,
    // since we don't actually load it here, but let's try and be helpful.
    {
        // Not bothering to deal with a font having more than 1 page /
        // texture right now. Note that if this is implemented, you may need
        // to account for multiple glyph pages sharing a single texture.
        Asura_Assert( pxFont->m_uNumGlyphPages == 1, "" );
        const Asura_Font_GlyphPage& xFontGlyphPage = pxFont->m_pxGlyphPages[0];

        Asura_Texture_Management::DestroyTexture( xFontGlyphPage.m_iFontTextureIndex );
    }

    // And dispose of our internal font data.
    delete s_apxFontStorage[iFontHandle];
    s_apxFontStorage[iFontHandle] = NULL;
}

// KJFL - 16 February 2006 - I've #if'ed out this DoUnitTests fn since it's not actively called anywhere
// now the changes it was testing have been finalised, and because wcstok doesn't exist on the PS3 as a 
// 2 parameter function (instead it has 3 parameters, and because it's a C function it can't be overloaded). 
#if 0
#include <wchar.h>
#include "Debugging/Asura_Logfile.h"
void Asura_Font_System::DoUnitTests( void )
{
    // Keep a backup of the pixel spacing.
    const float fOldPixelSpacing = s_fPixelSpacing;

    #if defined ASURA_ASSERT
        const float fWIDTH_TOL = 0.001f;
    #endif

    // (MP - 15.10.2005) I've specifically excluded any cases with leading
    // or trailing whitespace (within a line, if there are newlines in the
    // string.) The code isn't quite consistent in these cases but it would
    // be rather a lot of messing around to fix it up and I think we should
    // regard such text as broken anyway.
    const Asura_UnicodeChar* awszTestStrings[] =
    {
        L"blah blah BLAH",                                                      // Simple text.
        L".   .   .   .",                                                       // Consecutive spaces.
        L"abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890",      // Long line.
        L"abcdefghij klmnopqrst uvwxyzABCDEFGHI JKLMNOPQRS TUVWXYZ123 4567890", // Long line with spaces.
        L"abcdefghij-klmnopqrst-uvwxyzABCDEFGHI-JKLMNOPQRS-TUVWXYZ123-4567890", // Long line with hyphens instead of spaces.
        L"hello\n\nhello\n\n\n",                                                // Lots of newlines, including consecutive ones.
    };
    
    const float afScales[] = { 1.0f, 0.5f, 0.75f, 1.8f };

    const float afPixelSpacings[] = { 0.0f, 0.5f, 2.0f };

    const Asura_Bounding_Rectangle axRects[] =
    {
        Asura_Bounding_Rectangle( 0.0f, 0.0f, Asura_Maths::LargeFloat, Asura_Maths::LargeFloat ), // Never split.
        Asura_Bounding_Rectangle( 0.0f, 0.0f, 200.0f, Asura_Maths::LargeFloat ),                  // Split long lines.
        Asura_Bounding_Rectangle( 0.0f, 0.0f, 1.0f, Asura_Maths::LargeFloat ),                    // Split after pretty much every character.
    };

    Asura_UnicodeChar wszLineBfr[1024];

    for( u_int u0 = 0; u0 < sizeof(awszTestStrings) / sizeof(awszTestStrings[0]); ++u0 )
    {
        for( u_int u1 = 0; u1 < sizeof(afScales) / sizeof(afScales[0]); ++u1 )
        {
            for( u_int u2 = 0; u2 < sizeof(afPixelSpacings) / sizeof(afPixelSpacings[0]); ++u2 )
            {
                for( u_int u3 = 0; u3 < sizeof(axRects) / sizeof(axRects[0]); ++u3 )
                {
                    const Asura_UnicodeChar* wszNext = awszTestStrings[u0];
                    const float fScale = afScales[u1];
                    s_fPixelSpacing = afPixelSpacings[u2];
                    const Asura_Bounding_Rectangle& xRect = axRects[u3];

                    while( *wszNext )
                    {
                        // Basically we're checking that certain functions
                        // give consistent results in various different
                        // situations:
                        //     FindSplitPoint_Unscaled()
                        //     GetWidthOfString()
                        //     PrintLongString()

                        float fSplitPoint;
                        int iNumWords;
                        bool bNeedHyphen;
                        const Asura_UnicodeChar* wszAfterSplit = FindSplitPoint_Unscaled( wszNext,
                                                                                          xRect.MaxX - xRect.MinX,
                                                                                          fScale,
                                                                                          &fSplitPoint,
                                                                                          &iNumWords,
                                                                                          &bNeedHyphen );

                        int iLen = wszAfterSplit - wszNext;
                        Asura_PrintfToBfr( wszLineBfr, L"%s", iLen + 1, true, wszNext );

                        if( bNeedHyphen )
                        {
                            wcscat( wszLineBfr, L"-" );
                        }
                        else
                        {
                            // Remove trailing spaces from that buffer, since
                            // the line splitting code skips over them after
                            // a split, and it effectively doesn't matter
                            // when printing. Don't bother unless we're
                            // actually splitting though.
                            if( *wszAfterSplit )
                            {
                                Asura_StringUtil::StripWhitespace( wszLineBfr, Asura_StringUtil::STRIP_TRAILING );
                                iLen = wcslen( wszLineBfr );
                            }
                        }

                        float fWidthOfThisLine = GetWidthOfString( wszLineBfr, fScale );
                        float fDiff = Asura_Maths::Abs( fSplitPoint - fWidthOfThisLine );
                        Asura_Assert( fDiff < fWIDTH_TOL, "" );

                        if( PrintLongString(wszLineBfr, &xRect, 0.0f, 0.0f, 1.0f, fScale, 0, 0) )
                        {
                            fDiff = Asura_Maths::Abs( GetLastPrintedXCoordinate() - fWidthOfThisLine );
                            Asura_Assert( fDiff < fWIDTH_TOL, "" );
                        }

                        if( PrintLongString( wszLineBfr, &xRect, 0.0f, 0.0f, 1.0f, fScale, 0, ASURA_TEXTFORMAT_FULLYJUSTIFIED | ASURA_TEXTFORMAT_WORDWRAP) )
                        {
                            fDiff = Asura_Maths::Abs( GetLastPrintedXCoordinate() - fWidthOfThisLine );
                            Asura_Assert( fDiff < fWIDTH_TOL, "" );
                        }

                        // Verify that FindSplitPoint_Unscaled() got its word
                        // count right as well.
                        {
                            int iCount = 0;

                            const Asura_UnicodeChar* szSeparators = L" \n";
                            Asura_UnicodeChar* wszToken = wcstok( wszLineBfr, szSeparators );
                            while( wszToken )
                            {
                                ++iCount;
                                wszToken = wcstok( NULL, szSeparators );
                            }

                            Asura_Assert( iCount == iNumWords, "" );
                        }

                        wszNext = wszAfterSplit;
                    }
                }
            }
        }
    }

    s_fPixelSpacing = fOldPixelSpacing;

    Asura_OutputToDebugger_Retail(( "done font system unit test\n" ));
}
#endif
float Asura_Font_System::GetCharSpacing( Asura_UnicodeChar wChar, Asura_UnicodeChar wNextChar, float fScale )
{
    float fSpacing = 0.0f;
    
    const Asura_Font* pxFont = s_apxFontStorage[s_iCurrentFontID];

    // Any kerning data for this font at all?
    const Asura_Font_KerningData* pxKerningData = ( pxFont ? pxFont->m_pxKerningData : NULL );
    if( pxKerningData && dbg_bUseKerningData )
    {
        const Asura_Font_Glyph* pxCurCharGlyph = pxFont->GetGlyphData( wChar );

        // Any kerning data for the character we're about to print?
        if ( pxCurCharGlyph != NULL )
        {
            u_int uIdx = pxCurCharGlyph->usKerningIndex;
            if( Asura_Font_Glyph::INVALID_KERNING_IDX != uIdx )
            {
                pxKerningData += uIdx;
                
                // Search for a match for the character that will be following.
                u_int uNumKerningValues = (pxKerningData++)->usCount;
                while( uNumKerningValues-- )
                {
                    if( pxKerningData->wFollowingChar == wNextChar )
                    {
                        // Note that this value is probably negative.
                        float fKernAmount = static_cast<float>( static_cast<int>((pxKerningData + 1)->sKernAmount) );

                        fSpacing += fKernAmount * fScale;

                        break;
                    }

                    pxKerningData += 2;
                }
            }
        }
    }

    fSpacing += Project_GetCharSpacing( wChar, wNextChar, fScale );

    return fSpacing;
}

#endif // #ifndef _FONTMAKER

bool Asura_Font_System::StoreFontDetails( Asura_Font* pxNewFont )
{
	///////////////////////////////////////////////////////////////
	/// James Paulin - 25/07/2002 16:01:00 - This function is   ///
	/// not intended to be called from outside Asura_Chunk_Font ///
	/// ::Process()!!!										    ///
	///														    ///
    ///////////////////////////////////////////////////////////////
        
#ifdef ASURA_ASSERT
    Asura_Assertf( FindFontInternal(pxNewFont->m_szFileName) == -1, ("Font '%s' already exists.", pxNewFont->m_szFileName) );
#endif

    // Look for the first free slot.
    int iFree = -1;
    for( int i = 0; i < ASURA_MAX_FONTS; ++i )
    {
        if( !s_apxFontStorage[i] )
        {
            iFree = i;
            break;
        }
    }

    if( iFree >= 0 )
    {
        s_apxFontStorage[iFree] = pxNewFont;
        s_bAnyFontHasLoaded = true;

        // (MP - 20.10.2005) Not much harm in switching to the new font
        // after load, I don't think, and it makes dealing with subsequent
        // (optional) font data easier.
        s_iCurrentFontID = iFree;

        return true;
    }
    else
    {
        Asura_Assert( false, "Too many fonts loaded. May need to increase ASURA_MAX_FONTS." );
        return false;
    }
}

bool Asura_Font_System::StoreKerningData( Asura_Font_KerningData* pxKerningData, u_int uNumEntries )
{
    // We assume that this chunk data will follow the main font data, and
    // that therefore (assuming that was loaded successfully) it's now the
    // current font.
    Asura_Font* pxFont = s_apxFontStorage[s_iCurrentFontID];

    Asura_Assert( !pxFont->m_pxKerningData, "" );
    pxFont->m_pxKerningData = pxKerningData;

#ifdef ASURA_ASSERT
    pxFont->m_uNumKerningDataEntries = uNumEntries;
#endif

    return true;
}

void Asura_Font_System::SetFont( Asura_Handle fontHandle )
{
	//Check for a valid font handle
	
	if(fontHandle != ASURA_INVALID_HANDLE && s_apxFontStorage[fontHandle])
	{
		s_iCurrentFontID = fontHandle;
	}
	else
	{
		s_iCurrentFontID = 0;
		Asura_Assert(0,"ERROR - No font available for selection with that handle!\nTrying to revert to default font.");
	}
}

//void Asura_Font_System::SetFont( const Asura_Char* szName )
//{
//	//This function searches for the specified font texture
//	//An alternative technique would be to use a hash ID. If this method
//	//of fint selection proves too cumbersome then we will change. However
//	//this will not save us any memory because we will still have to provide
//	//the string version of the hash ID in functions that require fonts.
//
//  int iIndex = FindFontInternal( szName );
//	if( iIndex >= 0 )
//	{
//		SetFont( iIndex );
//	}
//	else
//	{
//		Asura_Assert( false, "ERROR - No font available for selection with that name!\nTrying to revert to default font." );
//		SetFont( 0 );
//	}
//}

void Asura_Font_System::SetFontScale(Asura_Handle hFontHandle, float fScaleFactor)
{
	if(hFontHandle != ASURA_INVALID_HANDLE && s_apxFontStorage[hFontHandle])
	{
		s_apxFontStorage[hFontHandle]->m_fFontScale = fScaleFactor;
	}
}

float Asura_Font_System::GetFontScale( Asura_Handle iFontHandle )
{
	if( iFontHandle != ASURA_INVALID_HANDLE )
	{
		return s_apxFontStorage[iFontHandle]->m_fFontScale;
	}

	return 1.0f;
}


Asura_Handle Asura_Font_System::GetFontHandle( const Asura_Char* szName )
{
	int iIndex = FindFontInternal( szName );

    Asura_Assertf( -1 != iIndex, ("font '%s' not loaded", szName) );

    return iIndex;
}

Asura_Handle Asura_Font_System::GetFontHandle( const Asura_Hash_ID uFontNameHash )
{
	int iIndex = FindFontInternal( uFontNameHash );

	Asura_Assertf( -1 != iIndex, ("font name hash'%x' not loaded", uFontNameHash) );

	return iIndex;
}

bool Asura_Font_System::IsFontLoaded( const Asura_Char* szName )
{
    return( -1 != FindFontInternal(szName) );
}

bool Asura_Font_System::IsFontLoaded( const Asura_Hash_ID uFontNameHash )
{
	return( -1 != FindFontInternal( uFontNameHash ) );
}


int Asura_Font_System::FindFontInternal( const char* szName )
{
    for( int i = 0; i < ASURA_MAX_FONTS; ++i )
    {
        const Asura_Font* pxFont = s_apxFontStorage[i];
        if( pxFont )
        {
            // Compare names. Previously when we were using a name tree we
            // used to ignore extensions but as far as I can see extensions
            // are never included in the name here anyway so we can just use
            // a straightforward string comparison.
            if( !stricmp(szName, pxFont->m_szFileName) )
            {
                return i;
            }
        }
    }

    return -1;
}

int Asura_Font_System::FindFontInternal( const Asura_Hash_ID uFontNameHash)
{
	Asura_Hash_ID uFontHash;

	for( int i = 0; i < ASURA_MAX_FONTS; ++i )
	{
		const Asura_Font* pxFont = s_apxFontStorage[i];
		if( pxFont )
		{
			uFontHash = Asura_GetHashID( pxFont->m_szFileName);

			if( uFontHash == uFontNameHash)
			{
				return i;
			}
		}
	}

	return -1;
}

ASURA_TRANSLUCENCY_MODE Asura_Font_System::SetTranslucencyMode( ASURA_TRANSLUCENCY_MODE eNewMode )
{
	ASURA_TRANSLUCENCY_MODE eOldMode = s_eTranslucencyMode;
	s_eTranslucencyMode = eNewMode;
	return eOldMode;
}

/**************************************************************************************/
/// Implementation of Asura_Font class.
/// Asura_Font objects are intended only for use within the Asura_Font_System class.

//Overloaded constructor that does not load the font's texture.
Asura_Font::Asura_Font()
{
    m_pxGlyphPages = NULL;
    m_pxKerningData = NULL;
	m_fFontScale = 1.f;
}

Asura_Font::~Asura_Font()
{
    delete[] m_pxKerningData;
    delete[] m_pxGlyphPages;
}

const Asura_Font_Glyph* Asura_Font::GetGlyphData(Asura_UnicodeChar wcThisChar, u_int uFormat, const Asura_Font_GlyphPage** ppxGlyphPage) const
{
    // returns pointer to the glyph data for wcThisChar if it exists (else NULL) 
    // and pointer to the glyph page if an output pointer is specified

    if (!wcThisChar) return NULL;

    const Asura_Font_Glyph* pxThisGlyph = Asura_Font_System::Project_GetGlyphData(wcThisChar, uFormat, ppxGlyphPage);
    if (pxThisGlyph) return pxThisGlyph;

    if (uFormat & ASURA_TEXTFORMAT_CAPITALISED && wcThisChar >= L'a' && wcThisChar <= L'z')
    {
        wcThisChar += static_cast<Asura_UnicodeChar>(L'A' - L'a');
    }

    for (Asura_Font_GlyphPage* pxThisPage = m_pxGlyphPages; pxThisPage < m_pxGlyphPages + m_uNumGlyphPages; pxThisPage++)
    {
        if (wcThisChar >= pxThisPage->m_wcRangeMin && wcThisChar <= pxThisPage->m_wcRangeMax)
        {
         	// We're on the right page if the glyph exists, try to find it
			if ((static_cast<u_int>(pxThisPage->m_wcRangeMax - pxThisPage->m_wcRangeMin + 1) == pxThisPage->m_uNumChars) || 
				(wcThisChar == pxThisPage->m_wcRangeMin))
			{
				// Trivial cases - contiguous set of chars, or the char is the first on this page
				if (ppxGlyphPage) *ppxGlyphPage = pxThisPage;
				return pxThisPage->m_pxGlyphArray + wcThisChar - pxThisPage->m_wcRangeMin;
			}
			else if (wcThisChar == pxThisPage->m_wcRangeMax)
			{
				//More triviality- char is the last on this page
				if (ppxGlyphPage) *ppxGlyphPage = pxThisPage;
				return pxThisPage->m_pxGlyphArray + (Asura_UnicodeChar)(pxThisPage->m_uNumChars -1);
			}
			else
			{
				// Binary search the glyphs
				u_int uSearchStart = 0;
				u_int uSearchEnd = pxThisPage->m_uNumChars-1;
				// Note the extremes are already excluded as we've covered that case above - maintain that fact
				while (true)
				{
					u_int uSearchPos = (uSearchEnd + uSearchStart) >> 1;
					if (pxThisPage->m_pxGlyphArray[uSearchPos].wcThisChar == wcThisChar)
					{
						// Found
						if (ppxGlyphPage) *ppxGlyphPage = pxThisPage;
						return pxThisPage->m_pxGlyphArray + (Asura_UnicodeChar)uSearchPos;
					}
					else if (pxThisPage->m_pxGlyphArray[uSearchPos].wcThisChar > wcThisChar)
					{
						// Too high
						uSearchEnd = uSearchPos;
					}
					else
					{
						// Too low
						uSearchStart = uSearchPos;
					}

					// Test if we've finished the search in failure (start and end are not in the search, so we're done if they're adjacent)
					if (uSearchEnd - uSearchStart <= 1)
					{
						break;
					}
				}
			}
        }
    }
	switch(wcThisChar)
	{
		// recognised characters
		case L'\r':		
#ifdef ASURA_TOOLS
		// The column delimiter
		case L'\a':					
#endif
		{
			return NULL;
		}
		// unrecognised characters
		default:
		{
			Asura_Assertf(false, ("Undefined character (%i) (0x%x) encountered", wcThisChar, wcThisChar) );

            // (AW - 23/12/05) - Look up the project's unsupported character glyph. This could cause a stack overflow crash
			// if the glyph specified by the project doesn't exist.
			const Asura_UnicodeChar wcUnsupportedCharacterGlyph = Asura_Font_System::Project_GetUnsupportedGlyphReplacementCharacter();
			return wcUnsupportedCharacterGlyph ? GetGlyphData(wcUnsupportedCharacterGlyph, uFormat, ppxGlyphPage) : NULL;
		}
	}
}

float Asura_Font::GetFontHeight() const
{
	return m_fFontHeight;
}
