//
// Asura_Overlay_Quad.h
//
// Created by Andrew Weinkove - 03/06/04
// 
// Asura_Overlay_Quad class moved out of Asura_Render into its own file.
//
// Copyright (C) 2004 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef __ASURA_OVERLAY_QUAD_H__
#define __ASURA_OVERLAY_QUAD_H__

#include "Asura_RenderStates.h" // for ASURA_TRANSLUCENCY_MODE.

class Asura_Bounding_Rectangle;

struct Colour_Components
{
	unsigned char R;
	unsigned char G;
	unsigned char B;
	unsigned char A;
};

union Asura_Colour_Union
{
    unsigned int Colour;
    struct Colour_Components ColourComponents;
};

class Asura_Overlay_Quad
{
	public:

		Asura_Overlay_Quad() :
		  m_eTextureOp(ASURA_TEXTUREOP_SINGLE_PASS),
		  m_bGouraudShaded(false),
		  m_bNonRotated(false)
		{
		}

		float X[4];
		float Y[4];
		float U[4];
		float V[4];

		union
		{
			unsigned int Colour;
			struct Colour_Components ColourComponents;
			unsigned int GouraudColour[4];
			struct Colour_Components GouraudColourComponents[4];
		};

		int TextureIndex;

		enum ASURA_TRANSLUCENCY_MODE TranslucencyMode;
		enum ASURA_TEXTUREOP_ID m_eTextureOp;
        bool m_bGouraudShaded;
        bool m_bNonRotated;

		void SetRectUV(float tlu, float tlv, float du, float dv);
		void SetRectXY(float tlx, float tly, float dx, float dy);
		void SetClippedRectXY(float fX, float fY, float fWidth, float fHeight, float fMinX, float fMinY, float fMaxX, float fMaxY);
		void SetRotatedRectXY( float fX, float fY, float fWidth, float fHeight, float fTheta );
        void SetGouraudColour(u_int uGouraudColourTL, u_int uGouraudColourTR, u_int uGouraudColourBL, u_int uGouraudColourBR);
        void SetGouraudColour(Colour_Components xGouraudColourTL, 
                            Colour_Components xGouraudColourTR, 
                            Colour_Components xGouraudColourBL, 
                            Colour_Components xGouraudColourBR);
		
		void Render(bool bResetGouraudAndRotated = true);// (MH - 09.01.2003) having added these flags, existing code should behave as it used to.

#if defined(ASURA_TOOLS)
		static void SetToolsDrawArea(const Asura_Bounding_Rectangle*	pxToolsDrawArea)	{ s_pxToolsDrawArea = pxToolsDrawArea; }
#endif	// defined(ASURA_TOOLS)

	private:
#if defined(ASURA_TOOLS)
		static const Asura_Bounding_Rectangle*	s_pxToolsDrawArea;	// A pointer to the rectangle to clip the render using.
#endif	// defined(ASURA_TOOLS)

		void Platform_Render();// (MH - 09.01.2003) having added these flags, existing code should behave as it used to.
};

#endif//__ASURA_OVERLAY_QUAD_H__
