//
// Asura_Render.h
//
// Created by Kevin Lea - 12/06/2001 11:05:25
// 
// Copyright (C) 2001 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef __ASURA_RENDER_H__
#define __ASURA_RENDER_H__

#include "Core/Asura_Maths.h"
#include "Core/Asura_PlatformBasics.h"
#include "Rendering/Asura_Overlay_Quad.h"
#include "Rendering/Asura_RenderStates.h"

enum ASURA_RENDERSTYLE_ID
{
	ASURA_RENDERSTYLE_NORMAL,
	ASURA_RENDERSTYLE_GRAPHICNOVEL,
};

#define ASURA_RENDER_TTYPES 1

#if defined ASURA_RENDER_TTYPES
	enum ASURA_RENDER_TTYPE
	{
		ASURA_RENDER_TTYPE_ENV,			// Environment, non degenerate
		ASURA_RENDER_TTYPE_ENVGLUE,		// Environment, degenerate
		ASURA_RENDER_TTYPE_SELF,		// Self illuminated, any kind
		ASURA_RENDER_TTYPE_HIER,		// Hierarchy
		ASURA_RENDER_TTYPE_SKIN,		// Smooth skin
		ASURA_RENDER_TTYPE_OBJ,			// Object
		ASURA_RENDER_TTYPE_GLS,			// Glass
		ASURA_RENDER_TTYPE_WAT,			// Water
		ASURA_RENDER_TTYPE_SHA,			// Shadows, any kind
		ASURA_RENDER_TTYPE_PFX,			// Particle effects, any kind
		ASURA_RENDER_TTYPE_OTHER,		// Unknown/unclassified

		ASURA_RENDER_TTYPE_COUNT,
	};
#endif



// DMR 10-05-07 
// Set of flags to enable/disable each rendering system - controlled via the DVS
enum ASURA_RENDER_SYSTEMS
{
	ASURA_RENDER_SYSTEM_SKYBOX,
	ASURA_RENDER_SYSTEM_ENV,
	ASURA_RENDER_SYSTEM_OBJECTS,
	ASURA_RENDER_SYSTEM_BOTANICALS,
	ASURA_RENDER_SYSTEM_CARPET,
	ASURA_RENDER_SYSTEM_WATER,
	ASURA_RENDER_SYSTEM_SHADOWS,
	ASURA_RENDER_SYSTEM_BEAMS,
	ASURA_RENDER_SYSTEM_DECALS,
	ASURA_RENDER_SYSTEM_PHOTONS,
	ASURA_RENDER_SYSTEM_PROJECTORS,
	ASURA_RENDER_SYSTEM_PFX,	
	ASURA_RENDER_SYSTEM_CORONAS,	
	ASURA_RENDER_SYSTEM_LIGHTSHAFTS,	
	ASURA_RENDER_SYSTEM_FOG,
	ASURA_RENDER_SYSTEM_FSFX,	
	ASURA_RENDER_SYSTEM_HUD,	
	ASURA_RENDER_SYSTEM_CLEARSCREEN,
	ASURA_RENDER_SYSTEM_CSM,
	ASURA_RENDER_NUM_SYSTEMS
};



class Asura_Render
{
public:
	enum ScreenBorderType
	{
		SCREEN_BORDER_DISABLED,
		SCREEN_BORDER_ACTUAL_SIZE,
		SCREEN_BORDER_STRETCH_TO_FIT,

		MAX_SCREEN_BORDER_TYPES
	};

	static void Initialise( void );

    static void EarlyShutdown( void );
    static void LateShutdown( void );
	
	static void Reset(void);

	static void BeginFrame(void);
	static void EndFrame(void);
    static inline void ClearZBuffer( void ) { Platform_ClearZBuffer(); }
    static inline void FlushDisplay( void ) { Platform_FlushDisplay(); }

#ifdef ASURA_USE_HDR
	static void ApplyToneMapping( void ) { Platform_ApplyToneMapping(); m_bIsCurrentlyHDR = false; }
	static bool IsCurrentlyHDR( void ) { return m_bIsCurrentlyHDR; }
#endif

#ifdef ASURA_USE_ZPREPASS
	static void	SetZPrepass( bool bZPrepass ) { s_bIsZprepass = bZPrepass; }
	static bool IsZPrepass( void ) { return s_bIsZprepass; }
#endif

#ifdef ASURA_TARGET_SM3

	static void BeginMRT( void ) { Platform_BeginMRT(); }
	static void EndMRT( void ) { Platform_EndMRT(); }

	// Called by the independent skybox code to say we are currently rendering the
	// skybox pass, used to allow platform code to force objects behind the world if needed
	static void BeginSkyboxRendering(void)
	{
		Platform_BeginSkyboxRendering();
	}
	static void EndSkyboxRendering(void)
	{
		Platform_EndSkyboxRendering();
	};

#endif

	/// Kevin Lea - 12/03/2002 17:39:44 - ignore any split-screen effects - e.g. to display debug text, or
	/// full-screen messages/menus overlayed onto split-screen.
    static inline void AllowFullScreenRendering( void ) { Platform_AllowFullScreenRendering(); }
    static bool SetVideoMode( int iWidth, int iHeight, bool bFSAA = false );// { return Platform_SetVideoMode( iWidth, iHeight, bFSAA ); }
	/// Kevin Lea - 12/03/2002 17:39:23 - pass on camera position, orientation, fov & viewing area to hardware
    static inline void ApplyCameraSettings( void ) { Platform_ApplyCameraSettings(); }

    static inline void InitialiseSplashScreen( const char* szName ) { Platform_InitialiseSplashScreen( szName ); }
    static inline void ShutdownSplashScreen( void ) { Platform_ShutdownSplashScreen(); }
    static inline void SplashScreenRender( void ) { Platform_SplashScreenRender(); }

	// Bordered screen settings (used for PSP mode)...
	static void ActivateScreenBorder(const ScreenBorderType eType, const u_int uActualWidth, const u_int uActualHeight, const u_int uBorderColour = 0xFF000000);
	static void DeactivateScreenBorder(void);

    // These are currently for debugging purposes. Some platforms/projects
    // don't support 2D line drawing at all in release/retail builds.
	static void RenderLine(const Asura_Vector_3& xPoint1, const Asura_Vector_3& xPoint2, unsigned int uColour);
	static void RenderLine(const Asura_AVector3& xPoint1, const Asura_AVector3& xPoint2, unsigned int uColour);
    static void Render2DLine( float fX0, float fY0, float fX1, float fY1, u_int uColour );
	static void Render2DLine(const Asura_Vector_2& xPoint0, const Asura_Vector_2& xPoint1, u_int uColour );
    static void RenderBB( const Asura_Bounding_Box& xBB, const Asura_Vector_3& xPos, const Asura_Matrix_3x3& xOri, const u_int uARGB = 0xFFFFFFFFu );
	static void RenderAxes( const Asura_Vector_3& xCentre, const Asura_Matrix_3x3& xOrient, const float fSize, const float fAlpha = 1.0f );
	static void RenderAxes( const Asura_AVector3& xCentre, const Asura_AMatrix3x3& xOrient, const float fSize, const float fAlpha = 1.0f );


	#ifdef ASURA_TOOLS
	//RAR 24/07/2006 Used by tools overlay rendering code
	static bool Tools_GetClippingRectangle(float& fX,float& fY,float& fWidth,float& fHeight);
	static void Tools_SetClippingRectangle(float fX,float fY,float fWidth,float fHeight);
	static void Tools_SetClippingRectangle(const Asura_Bounding_Rectangle& xRect)
	{Tools_SetClippingRectangle(xRect.MinX, xRect.MinY, xRect.GetWidth(), xRect.GetHeight());}
	#endif

	static void SetEnableBatchedLineRendering(bool bEnable)
	{		
		s_bEnableBatchedLineRendering = bEnable;
	}
	static bool GetEnableBatchedLineRendering()
	{
		return s_bEnableBatchedLineRendering;
	}

	static ASURA_RENDERSTYLE_ID GetRenderStyle()
	{
		return s_eRenderStyle;
	}
	static void SetRenderStyle(ASURA_RENDERSTYLE_ID eRenderStyle)
	{
		s_eRenderStyle = eRenderStyle;
	}
    static bool GetEnableScreenClear()
	{
		return IsSystemEnabled(ASURA_RENDER_SYSTEM_CLEARSCREEN);
	}
    static void SetEnableScreenClear(bool bEnable)
	{
		SetSystemEnabled(ASURA_RENDER_SYSTEM_CLEARSCREEN, bEnable);
	}
	static bool GetPatchesOn(void)
	{
		return s_bPatchesOn;
	}
	static void SetPatchesOn(bool b)
	{
		s_bPatchesOn = b;
	}
	static int GetPatchesSubdivLevel()
	{
		return s_iPatchesSubdivLevel;
	}
	static void SetPatchesSubdivLevel(int i)
	{
		s_iPatchesSubdivLevel = i;
	}
	static bool GetStaticSubdivision(void)
	{
		return s_bStaticSubdivision;
	}
	static void SetStaticSubdivision(bool b)
	{
		s_bStaticSubdivision = b;
	}
	static bool GetCutsceneHasCameraControl(void)
	{
		return s_bCutsceneHasCameraControl;
	}
	static void SetCutsceneHasCameraControl(bool b)
	{
		s_bCutsceneHasCameraControl = b;
	}
	static bool ShouldRenderMirrored(void)
	{
		return s_bRenderMirrored;
	}
	static void SetRenderMirrored(bool b)
	{
		s_bRenderMirrored = b;
	}
	static void SetRenderForWidescreen(bool bEnable)
	{
		if (s_bRenderForWidescreen != bEnable)
		{
			FlushDisplay();
		}
		s_bRenderForWidescreen = bEnable;
	}
	static bool ShouldRenderForWidescreen()
	{
		return s_bRenderForWidescreen;
	}
	static void SetSystemEnabled( const ASURA_RENDER_SYSTEMS eSystem, const bool bEnable )
	{
		s_abSystemEnabled[eSystem] = bEnable;
	}
	static bool IsSystemEnabled(ASURA_RENDER_SYSTEMS eSystem)
	{
		return s_abSystemEnabled[eSystem];
	}

#if defined ASURA_RENDER_TTYPES
	static void AddToTrianglesRendered(const int iNumTriangles,
		const int iNumPasses = 1,
		const ASURA_RENDER_TTYPE eType = ASURA_RENDER_TTYPE_OTHER
	)
    {
	#if _DEBUG
        CheckInFrame();
	#endif
		s_aiNumberOfTrianglesRendered[eType] += iNumTriangles;
	    s_iNumTriPassesRendered += iNumTriangles * iNumPasses;
    }
#else
    static void AddToTrianglesRendered( int iNumTriangles, int iNumPasses = 1 )
    {
	#if _DEBUG
        CheckInFrame();
	#endif
	    s_iNumberOfTrianglesRendered += iNumTriangles;
	    s_iNumTriPassesRendered += iNumTriangles * iNumPasses;
    }
#endif

    static void AddToTriPassesRendered( int iNumTriPasses )
    {
#if _DEBUG
        CheckInFrame();
#endif
	    s_iNumTriPassesRendered += iNumTriPasses;
    }
    static void AddToPatchesRendered( int iNumPatches )
    {
#if _DEBUG
        CheckInFrame();
#endif
	    s_iNumberOfPatchesRendered += iNumPatches;
    }
	static bool IsRendering()
	{
		return s_bCurrentlyRendering;
	}

	static int GetNumberOfTrianglesRendered()
	{
	#if defined ASURA_RENDER_TTYPES
		u_int uTris = 0;
		for (int i = 0; i < ASURA_RENDER_TTYPE_COUNT; i++)
			uTris += s_aiNumberOfTrianglesRendered[i];
		return uTris;
	#else
		return s_iNumberOfTrianglesRendered;
	#endif
	}

	#if defined ASURA_RENDER_TTYPES
	static int GetNumberOfTrianglesRendered(ASURA_RENDER_TTYPE eType)
	{
		return s_aiNumberOfTrianglesRendered[eType];
	}
	#endif

	static int GetNumTriPassesRendered()
	{
		return s_iNumTriPassesRendered;
	}
	static int GetNumPatchesRendered()
	{
		return s_iNumberOfPatchesRendered;
	}

	// RJF - 30 August 2005 - Moving these here from Asura_PFX_System because Asura_PFX_System has no concept of a single frame and thus does not
	// know when to reset the counters correctly
	static inline void AddToParticlesRendered(unsigned int uCount)
	{ 
#if _DEBUG
		CheckInFrame();
#endif
		s_uNumParticlesRendered += uCount; 
	}

	static inline unsigned int GetParticlesRenderedThisFrame() 
	{ 
		return s_uNumParticlesRendered; 
	}
	// JAT 07/12/07 - count how many skins and hierarchies didn't get rendered e.g. not in view or module 
	static inline void AddToSkinHierarchiesOutOfView( unsigned int uCount = 1 )	{ s_uNumSkinHierarchiesOutOfView += uCount;	}
	static inline u_int GetSkinHierarchiesOutOfViewThisFrame( void ) { return s_uNumSkinHierarchiesOutOfView; }
	// KOR - 14/11/2005 - counters for things rendered
	static inline void AddToSmoothSkinsRendered(unsigned int uCount = 1)	{	s_uNumSmoothSkinsRendered += uCount;	}
	static inline void AddToHierarchiesRendered(unsigned int uCount = 1)	{	s_uNumHierarchiesRendered += uCount;	}
	static inline void AddToShapesRendered(unsigned int uCount = 1)			{	s_uNumShapesRendered += uCount;			}
	static inline unsigned int GetSmoothSkinsRenderedThisFrame()	{ return s_uNumSmoothSkinsRendered; }
	static inline unsigned int GetHierarchiesRenderedThisFrame()	{ return s_uNumHierarchiesRendered; }
	static inline unsigned int GetShapesRenderedThisFrame()			{ return s_uNumShapesRendered;		}

#ifndef ASURA_TARGET_SM3

	//Gets the texture index of the reflectance map
	static int GetReflectanceMap();
	//Gets the filename of the reflectance map
	static const char* GetReflectanceMapName();
	//Set the filename of the reflectance map
	static void SetReflectanceMapName(const char* pszName);

#elif defined(ASURA_TOOLS) //#ifndef ASURA_TARGET_SM3

	// Tools needs to have this function in order to create the render settings chunk.
	static const char* GetReflectanceMapName();

#endif //#elif defined(ASURA_TOOLS) //#ifndef ASURA_TARGET_SM3

	// Render shadow maps onto the environment, on platforms that deal with
    // shadows this way. This will typically be called by project-specific
    // code after all shadowcasting geometry has been rendered.
    static inline void RenderDynamicShadowsOntoEnvironment( void ) { Platform_RenderDynamicShadowsOntoEnvironment(); }
	
 	static class Asura_Overlay_Quad Overlay;
	static class Asura_Render_States CurrentStates;

	// ART (13/05/2005) - added sub-pixel offset for hi-res screenshot code...
	static void SetSubPixelOffset(const Asura_Vector_2& xOffset)	{ s_xSubPixelOffset	= xOffset; }
	static const Asura_Vector_2& GetSubPixelOffset(void)			{ return (s_xSubPixelOffset); }

#if defined(ASURA_USE_SCREENSHOT_SYSTEM)
	// ART (03/04/2008) - added flag to enable hi-res screenshots to have special behaviours...
	static void EnableHiResScreenshotMode(void)		{ s_bEnableHiResScreenshotMode = true; }
	static void DisableHiResScreenshotMode(void)	{ s_bEnableHiResScreenshotMode = false; }
	static bool IsHiResScreenshotModeEnabled(void)	{ return(s_bEnableHiResScreenshotMode); }
	static void EnableMipMapping(void)				{ s_bDisableMipMaps = false; }
	static void DisableMipMapping(void)				{ s_bDisableMipMaps = true; }
	static bool IsMipMappingDisabled(void)			{ return(s_bDisableMipMaps); }

#endif	//defined(ASURA_USE_SCREENSHOT_SYSTEM)

	//Special functions for forcing the player to render in front of everything
	static void PrepareToRenderFirstPersonMesh	()	{ Platform_PrepareToRenderFirstPersonMesh();	}
	static void FinishRenderFirstPersonMesh		()	{ Platform_FinishRenderFirstPersonMesh();		}

	#ifdef ASURA_ASSERT
	static bool IsInProtectedRenderCode () { return s_bInProtectedRenderCode; }
	#endif

	static void SetSafeArea(const Asura_Bounding_Rectangle& xArea ) { s_xSafeArea = xArea; }
	static const Asura_Bounding_Rectangle& GetSafeArea() { return s_xSafeArea; }

#ifdef ASURA_EDGE_ANTIALIASING
	static void SetCollectAAEdgesFlag( bool bCollect );
	static void RenderEdgeAntiAliasing( void );
#endif

	static bool GetWireframe() { return s_bWireframe; }
	static void SetWireframe( const bool bWireframe ) { s_bWireframe = bWireframe; }

private:

	static void Platform_Initialise(void);
    static void Platform_EarlyShutdown( void );
    static void Platform_LateShutdown( void );
	static void Platform_Reset(void);
	static void Platform_BeginFrame(void);
	static void Platform_EndFrame(void);
	static void Platform_ClearZBuffer(void);
	static void Platform_FlushDisplay(void);
	static void Platform_AllowFullScreenRendering(void);
	static bool Platform_SetVideoMode(int iWidth, int iHeight, bool bFsaa);
 	static void Platform_ApplyCameraSettings(void);
	static void Platform_InitialiseSplashScreen(const char* szName);
	static void Platform_ShutdownSplashScreen(void);
	static void Platform_SplashScreenRender(void);
 	static void Platform_RenderLine(const Asura_Vector_3& xPoint1, const Asura_Vector_3& xPoint2, unsigned int uColour);
	static void Platform_Render2DLine(const Asura_Vector_2& xPoint0, const Asura_Vector_2& xPoint1, u_int uColour );
	static void PlatformProject_Initialise( void );
	static void Platform_RenderDynamicShadowsOntoEnvironment();
	static void Platform_PrepareToRenderFirstPersonMesh ();	
	static void Platform_FinishRenderFirstPersonMesh	();

#ifdef ASURA_USE_HDR
	static void Platform_ApplyToneMapping( void );
#endif

#ifdef ASURA_TARGET_SM3
	static void Platform_BeginMRT(void);
	static void Platform_EndMRT(void);

	static void Platform_BeginSkyboxRendering(void);
	static void Platform_EndSkyboxRendering(void);
#endif

#ifdef ASURA_EDGE_ANTIALIASING
	static void Platform_SetCollectAAEdges( bool bCollect );
	static void Platform_RenderEdgeAntiAliasing( void );
#endif

#if _DEBUG
    static void CheckInFrame( void );
#else
	static void CheckInFrame( void ) {}
#endif

#if defined(ASURA_USE_SCREENSHOT_SYSTEM)
	static bool s_bEnableHiResScreenshotMode;
	static bool s_bDisableMipMaps;
#endif	//defined(ASURA_USE_SCREENSHOT_SYSTEM)
	static bool s_bEnableBatchedLineRendering;
	static bool s_bPatchesOn;
	static int  s_iPatchesSubdivLevel;
	static bool s_bStaticSubdivision;
	static enum ASURA_RENDERSTYLE_ID s_eRenderStyle; 
	static bool s_bCutsceneHasCameraControl;
	static bool s_bRenderMirrored;
	static bool s_bRenderForWidescreen;
	static bool s_bCurrentlyRendering;
	static bool s_bWireframe;
#if defined ASURA_RENDER_TTYPES
	static int s_aiNumberOfTrianglesRendered[ASURA_RENDER_TTYPE_COUNT];
#else
	static int s_iNumberOfTrianglesRendered;
#endif
    static int s_iNumTriPassesRendered;
	static int s_iNumberOfPatchesRendered;
	static u_int s_uNumParticlesRendered;
	static u_int s_uNumSkinHierarchiesOutOfView;
	static u_int s_uNumSmoothSkinsRendered;
	static u_int s_uNumHierarchiesRendered;
	static u_int s_uNumShapesRendered;
	static Asura_Bounding_Rectangle s_xSafeArea;

#ifndef ASURA_TARGET_SM3
	static int s_iReflectanceMapTextureIndex;
	static class Asura_String s_xReflectanceMapTextureName;
#endif //#ifndef ASURA_TARGET_SM3

	// ART (13/05/2005) - added sub-pixel offset for hi-res screenshot code...
	static	Asura_Vector_2	s_xSubPixelOffset;

	static bool						s_abSystemEnabled[ASURA_RENDER_NUM_SYSTEMS];
	static const Asura_UnicodeChar* s_aszSystemEnableDVSName[ASURA_RENDER_NUM_SYSTEMS];
#ifdef ASURA_USE_ZPREPASS
	static bool s_bIsZprepass;
#endif

#ifdef ASURA_ASSERT
	static void SetInProtectedRenderCode(bool bProtected)	{ s_bInProtectedRenderCode = bProtected;	}
	static bool s_bInProtectedRenderCode; //true if we are inside the beginframe/endframe functions
#endif

#ifdef ASURA_USE_HDR
	static bool m_bIsCurrentlyHDR;
#endif
};



#endif//__ASURA_RENDER_H__
