//
// Asura_Surface.h
//
// Created by Kevin Lea - 12/06/2001 11:05:54
// 
// Copyright (C) 2001 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef __ASURA_SURFACE_H__
#define __ASURA_SURFACE_H__

#include "Core/Asura_Maths.h"
#include "Mesh/Asura_Mesh.h"
#include "Mesh/Asura_Mesh_System.h"

class Asura_Bicubic_Patch;

class Asura_Surface
{
	public:
	int NumberOfVertices;
	int m_iMaterialID;

#if defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM
	Asura_Hash_ID m_auMaterialResponseHashID[TOOLS_NUM_MATERIALRESPONSE_PER_POLY];
#endif

	Asura_Vector_3 FaceNormal;

	int VertexID[4];
	Asura_Vector_3 LNormalID[4];
	Asura_Vector_3 CNormalID[4][2];
	unsigned int Colour[4];
	bool m_abStraightEdge[4];

	#if defined(ASURA_TARGET_SM3)
	
	Asura_SphericalHarmonics_Order3 m_axLightTransportHarmonic[4];
	Asura_SphericalHarmonics_Order2_RGB m_axDiffuseSH[4];

	Asura_Vector_3	m_axBrightestLightVector[4];	// VGP 13Feb2006

	Asura_Vector_3 m_axTangent[4];
	Asura_Vector_3 m_axBinormal[4];
	Asura_Vector_3 m_axTangent2[4];
	Asura_Vector_3 m_axBinormal2[4];

	#endif

	float U[4];
	float V[4];

	float U2[4];
	float V2[4];
	
	bool m_bHasNoCurveFlag;
	bool IsCurved;
	bool IsEndCap;

	void Initialise(const class Asura_Mesh* pxMesh,int iPolygonIndex);

	void CalculateIfCurved(void)
	{
		if (NumberOfVertices==0) return;

		for (int i=0; i<NumberOfVertices; i++)
		{
			Asura_Vector_3 epsilon = FaceNormal-CNormalID[i][0];
			if (epsilon.Magnitude()>0.001f*0.001f*3.0f)
			{
				IsCurved=true;
				return;
			}
		}
		IsCurved = false;
	}

	Asura_Vector_3 ControlPoint[4][4];
	bool ControlPointsValid;

	bool SubdivideDataValid;
	Asura_Vector_3 du0dv0[4];
	Asura_Vector_3 du2dv0[4];
	Asura_Vector_3 du0dv2[4];

	Asura_Vector_3 Normal[4];
};

class Asura_Surface_Mesh
{
	friend class Asura_Surface_Mesh_System;

	public:

		Asura_Surface_Mesh()
		{
			NumberOfSurfaces=0;
			SurfaceDataValid = false;
		}

		~Asura_Surface_Mesh()
		{
			if (SurfaceDataValid)
			{
				delete [] Surfaces;
				SurfaceDataValid = false;
			}
		}
		
		void Render(void);
		void Invalidate(void)
		{
			if (SurfaceDataValid)
			{
				delete [] Surfaces;
				SurfaceDataValid = false;
			}
		}
	
	///private:
		
		bool SurfaceDataValid;
		int NumberOfSurfaces;
		Asura_Surface* Surfaces;
		
		int MeshIndex;
};

class Asura_Surface_Mesh_System
{
	public:
#if defined(ASURA_TOOLS)
		// (SB - 11/12/06) Hook to generate vertex colour from  flags
		typedef unsigned int GeneratePatchVertexColourHook(Asura_Mesh&, Asura_Bicubic_Patch&, int, int);
#endif

		static void Asura_Surface_Mesh_System::Initialise(void);
		static void Asura_Surface_Mesh_System::Shutdown(void);

		static void Asura_Surface_Mesh_System::SortOnTexture(Asura_Surface_Mesh* surfaceMesh);
		static void InitMeshForSurfaceRender(Asura_Mesh& xMesh, const Asura_Vector_3& xPosition);
		static void Asura_Surface_Mesh_System::RenderAsSurfaces(Asura_Mesh& mesh, const Asura_Vector_3& position);
		static void Asura_Surface_Mesh_System::MakeVertexBufferAndIndexList(Asura_Mesh& mesh, const Asura_Vector_3& position);
		static void Asura_Surface_Mesh_System::RenderFromVertexBuffer(Asura_Mesh& xMesh);
		static Asura_Surface_Mesh* GetSurfaceMesh(int index)
		{
			return &s_axSurfaceMeshStorage[index];
		}
		static void Asura_Surface_Mesh_System::ProcessMeshIntoSurfaces(Asura_Mesh& mesh, const Asura_Vector_3& position);

		//version used in tools that will sort out curving across module boundaries
		static void Asura_Surface_Mesh_System::ProcessModuleMeshIntoSurfaces(int iModuleIndex);

		//invalidate all surface meshes
		static void InvalidateAll();

#if defined(ASURA_TOOLS)
		// (SB - 11/12/06) Set hook to generate vertex colour from flags
		static void SetGeneratePatchVertexColourHook(GeneratePatchVertexColourHook* const pfnGeneratePatchVertexColourHook = NULL) { s_pfnGeneratePatchVertexColourHook = pfnGeneratePatchVertexColourHook; };
#endif

	private:
		static Asura_Surface_Mesh s_axSurfaceMeshStorage[ASURA_MAX_NO_OF_MESHES];

#if defined(ASURA_TOOLS)
		// (SB - 11/12/06) Hook to generate vertex colour from  flags
		static GeneratePatchVertexColourHook* s_pfnGeneratePatchVertexColourHook;
#endif
};

#endif//__ASURA_SURFACE_H__
