
///////////////////////////////////////////////////////////////////////////////
//
// File   : Asura_Shadow.h
// Author : Andy Tate
// Date   : 28-Nov-2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __ASURA_SHADOW_H__
#define __ASURA_SHADOW_H__

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Shadows/Asura_Shadow_Instance.h"


////////////////////////////////////////////////////////////////////////////////
// Forward class declarations...
////////////////////////////////////////////////////////////////////////////////

class Asura_Animation;
class Asura_Shape_Instance;


////////////////////////////////////////////////////////////////////////////////
// Class declaration...
////////////////////////////////////////////////////////////////////////////////

class Asura_Shadow
{
public:
	// Public functions...
	virtual ~Asura_Shadow ()	{}

	virtual	void	GetBoundingBox		(Asura_Bounding_Box& xBoundingBox) const	{ m_pxInstance->GetBoundingBox(xBoundingBox); }

	// Abstract functions...
	virtual	void	PrepareForRendering	(void)	= 0;
	virtual	void	FinishedRendering	(void)	= 0;

	// Render function...
	void	Render	(void)	{ m_pxInstance->Render(*this); }

	// Accessor function...
	virtual void					SetInstance			(const Asura_Shadow_Instance* pxInstance)	{ m_pxInstance = pxInstance; }
	const Asura_Shadow_Instance*	GetInstance			(void) const									{ return(m_pxInstance); }

	const float*					GetLightDirection		(void) const							{ return(&m_pxInstance->GetLightDirection().x); }
	const Asura_Vector_3&			GetLightPosition		(void) const							{ return(m_pxInstance->GetLightPosition()); }
	void							GetLightOrientation		(Asura_Quat& xOrientation) const		{ m_pxInstance->GetLightOrientation(xOrientation); }
	void							GetLightOrientation		(Asura_Matrix_3x3& xOrientation) const	{ m_pxInstance->GetLightOrientation(xOrientation); }
	const Asura_Frustum&			GetFrustum				(void) const							{ return(m_pxInstance->GetFrustum()); }


	// Debug rendering functions...
	void	DEBUG_RenderFrustum	(void) const;




protected:
	// Protected variables...
	const Asura_Shadow_Instance*	m_pxInstance;

	// Protected functions...
	Asura_Shadow  (void)	: m_pxInstance(NULL)	{}
};


#endif   // ifndef __ASURA_SHADOW_H__


/// End of file. ///
