//////////////////////////////////////////////////////////////////////////////
//
// Asura_Dialogue_Playback.h
//
// Copyright (c) 2006 Rebellion. All rights reserved.
//
// Description:
//
//     classes to manage dialogue playback
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef ___ASURA_DIALOGUE_PLAYBACK_H___
#define ___ASURA_DIALOGUE_PLAYBACK_H___ 1

#ifdef ASURA_USE_DIALOGUE_SYSTEM

// I N C L U D E S ///////////////////////////////////////////////////////////
#include "Core/Asura_Hash.h"
#include "Core/Asura_Maths.h"
#include "Network/Asura_Guid.h"

// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////////
class Asura_Dialogue_CharacterMappingList;
class Asura_Dialogue_Controller;
class Asura_Dialogue_Playable;
class Asura_Dialogue_Event;
class Asura_Dialogue_Line;
class Asura_Vector_3;

// C L A S S E S /////////////////////////////////////////////////////////////

// Asura_Dialogue_System::PlayEvent() can return a handle from a list of Asura_Dialogue_PlaybackObject
class Asura_Dialogue_PlaybackObject
{
public:

	Asura_Dialogue_PlaybackObject();
	~Asura_Dialogue_PlaybackObject();

	void Update();
	void Play(const Asura_Dialogue_Event* pxEvent, const Asura_Dialogue_CharacterMappingList* pxMapping,Asura_Guid uSpeakerGuid, const Asura_Vector_3* pxPosition,unsigned short usMessageBlockFromInstance, Asura_Guid uInstanceGuid, Asura_Hash_ID uInstanceHandlerID);
	void Play(const Asura_Dialogue_Line* pxLine, Asura_Guid uSpeakerGuid, Asura_Guid uInstanceGuid = ASURA_GUID_UNREGISTERED, int iNotifyFinishedMessageData = 0);
	void Stop();
	void SkipLine();
	
#if defined(ASURA_TOOLS)
	void DialogueEventDeleted(const Asura_Dialogue_Event* pxEventDeleted);
#endif

	// FindGuidFromCharacter() looks up the character in the mapping and returns the guid.
	Asura_Guid FindGuidFromCharacter( Asura_Hash_ID uCharacterHashID ) const;

	// accessors
	bool NoExternalHandle() const { return m_bNoExternalHandle; }
	void SetHasExternalHandle(bool bHasExternalHandle) { m_bNoExternalHandle = !bHasExternalHandle; }
	bool IsPlaying() const { return m_bPlaying; }
	unsigned short GetMessageBlockFromInstance() const {return m_usMessageBlockFromInstance;}
	Asura_Hash_ID GetInstanceHandlerID() const {return m_uInstanceHandlerID;}

	// JAT 08/03/07 - Get & Set VolumeScale()
	void SetVolumeScale( const float fVolumeScale );
	float GetVolumeScale( void ) const;


	void SetPositionOverride(const Asura_Vector_3& xPosition);

	bool IsUsingPositionOverride() const {return m_bUsePositionOverride;}
	const Asura_Vector_3& GetPositionOverride() const {return m_xPositionOverride;}

	// helpers for getting info about playing dialogue. Returns info from first controller.
	// Any can return NULL.
	const Asura_Dialogue_Playable* GetEventOrLine() const;
	const Asura_Dialogue_Event* GetEvent() const;
	const Asura_Dialogue_Line* GetLine() const;

	Asura_Guid GetSpeakerGuid(void) const { return m_uSpeakerGuid; }
	
	void FadeOut(float fPeriod);
	bool GetFadingOut(void) const { return m_bFadingOut; }

private:
	void Reset();

	bool						m_bNoExternalHandle;	// managed by Asura_Dialogue_System
	bool						m_bPlaying;				// set during Update() based on sound status, play mode etc.
	bool						m_bUsePositionOverride;	// Causes the fixed position to be used instead of taking from entities
	Asura_Dialogue_Controller*	m_pxFirstController;
	Asura_Dialogue_CharacterMappingList* m_pxCharacterMapping;
	Asura_Guid					m_uSpeakerGuid; //used if there isn't a character mapping
	Asura_Vector_3				m_xPositionOverride; //used for the sound position if m_bUsePositionOverride is set

	unsigned short				m_usMessageBlockFromInstance;//message block from dialogue instance , that contains extra messages for the event
	Asura_Guid					m_uInstanceGuid;//guid of triggering dialogue instance (if any)
	int							m_iNotifyFinishedMessageData;//value to send in notify finished message back to dialogue controller
	Asura_Hash_ID				m_uInstanceHandlerID;//For streaming.. hash ID of the handler that the dialogue trigger belongs to
							  
	// non-copyable
	Asura_Dialogue_PlaybackObject(const Asura_Dialogue_PlaybackObject&);
	Asura_Dialogue_PlaybackObject* operator= (const Asura_Dialogue_PlaybackObject&);

	bool m_bFadingOut;
	float m_fFadeOutStartTime;
	float m_fFadeOutPeriod;
};

#endif // #ifdef ASURA_USE_DIALOGUE_SYSTEM

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

#endif // #ifndef ___ASURA_DIALOGUE_PLAYBACK_H___
