// Asura_Dynamic_Music_System_Triggers.cpp
// Thomas Young - 29.08.08
// Copyright (C) 2008 Rebellion, All Rights Reserved.

#include "Core/Asura.h"

#ifdef ASURA_SUPPORT_DYNAMIC_MUSIC_SYSTEM

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "core/Asura_Maths.h"
#include "Entity/Triggers/Asura_ServerEntity_DynamicMusicTrigger.h"
#include "Network/Asura_Guid.h"
#include "Sound/Asura_Dynamic_Music_System_Triggers.h"

#ifdef ASURA_TOOLS
#include "Entity/Tools_Entity_Data.h"
#endif //ASURA_TOOLS

// F U N C T I O N   D E F I N I T I O N S ///////////////////////////////////

static inline bool RoughlyEqual(const float fValue1, const float fValue2)
{
	return static_cast<bool>( ( Asura_Maths::Abs(fValue1) - Asura_Maths::Abs(fValue2) ) < 0.05f );
};

Asura_Dynamic_Music_Layer_Transition_Trigger_Data::Asura_Dynamic_Music_Layer_Transition_Trigger_Data ()
{
	m_uTrack					= ASURA_HASH_ID_UNSET;
	m_fPrevOutroStartOffset		= 10.0f;
	m_fPrevOutroEndOffset		= 20.0f;
	m_fNextIntroStartOffset		= 20.0f;
	m_fNextIntroEndOffset		= 30.0f;

	m_bUseIntro	= true;

	for( int i = 0; i < ASURA_DYNAMIC_MUSIC_SYSTEM_NUM_LAYERS; ++i )
	{
		m_bMuteLayers[i] = false;
	}

	//Set the default amplitude curves
	ASURA_STATIC_ASSERT ( ASURA_DYNAMIC_MUSIC_SYSTEM_FADE_RESOLUTION == 3 );

	m_xPrevTrackAmplitude[0].m_fTime		= 0.0f;
	m_xPrevTrackAmplitude[0].m_fAmplitude	= 1.0f;

	m_xPrevTrackAmplitude[1].m_fTime		= 20.0f;
	m_xPrevTrackAmplitude[1].m_fAmplitude	= 1.0f;

	m_xPrevTrackAmplitude[2].m_fTime		= 20.0f;
	m_xPrevTrackAmplitude[2].m_fAmplitude	= 0.0f;

	m_xNewTrackAmplitude[0].m_fTime			= 20.0f;
	m_xNewTrackAmplitude[0].m_fAmplitude	= 0.0f;

	m_xNewTrackAmplitude[1].m_fTime			= 20.0f;
	m_xNewTrackAmplitude[1].m_fAmplitude	= 1.0f;

	m_xNewTrackAmplitude[2].m_fTime			= 60.0f;
	m_xNewTrackAmplitude[2].m_fAmplitude	= 1.0f;
}

void Asura_Dynamic_Music_Layer_Transition_Trigger_Data::ReadFromChunkStream ( Asura_Chunk_Stream& xStream )
{
	int iVersion;
	xStream >> iVersion;
	Asura_Assert(iVersion <= CURRENT_DATA_VERSION, "Unknown dynamic music layer transition trigger chunk version");

	xStream >> m_uTrack;
	xStream >> m_fPrevOutroStartOffset;
	xStream >> m_fPrevOutroEndOffset;
	xStream >> m_fNextIntroStartOffset;
	xStream >> m_fNextIntroEndOffset;
	for(int i = 0; i < ASURA_DYNAMIC_MUSIC_SYSTEM_FADE_RESOLUTION; ++i )
	{
		xStream >> m_xPrevTrackAmplitude[i].m_fTime;
		xStream >> m_xPrevTrackAmplitude[i].m_fAmplitude;

		xStream >> m_xNewTrackAmplitude[i].m_fTime;
		xStream >> m_xNewTrackAmplitude[i].m_fAmplitude;
	}

	if( iVersion >= 2 )
	{
		xStream >> m_bUseIntro;

		for( int i = 0; i < ASURA_DYNAMIC_MUSIC_SYSTEM_NUM_LAYERS; ++i )
		{
			xStream >> m_bMuteLayers[i];
		}
	}
}


void Asura_Dynamic_Music_Layer_Transition_Trigger_Data::WriteToChunkStream( Asura_Chunk_Stream& xStream ) const
{
	int iVersion = CURRENT_DATA_VERSION;
	xStream << iVersion;

	xStream << m_uTrack;
	xStream << m_fPrevOutroStartOffset;
	xStream << m_fPrevOutroEndOffset;
	xStream << m_fNextIntroStartOffset;
	xStream << m_fNextIntroEndOffset;
	for(int i = 0; i < ASURA_DYNAMIC_MUSIC_SYSTEM_FADE_RESOLUTION; ++i )
	{
		xStream << m_xPrevTrackAmplitude[i].m_fTime;
		xStream << m_xPrevTrackAmplitude[i].m_fAmplitude;

		xStream << m_xNewTrackAmplitude[i].m_fTime;
		xStream << m_xNewTrackAmplitude[i].m_fAmplitude;
	}

	if( iVersion >= 2 )
	{
		xStream << m_bUseIntro;

		for( int i = 0; i < ASURA_DYNAMIC_MUSIC_SYSTEM_NUM_LAYERS; ++i )
		{
			xStream << m_bMuteLayers[i];
		}
	}
}


float Asura_Dynamic_Music_Layer_Transition_Trigger_Data::GetVolumeAtTime ( const float fTime, const Asura_Dynamic_Music_Layer_Transition_Trigger_Data::Track eTrack ) const
{
	//Helper function for interpolating volumes
	const Dynamic_Music_Volume_Point* pxTrackPoints = 0;
	if( eTrack == TRACK_PREVIOUS )
	{
		if( fTime <= m_xPrevTrackAmplitude[ 0 ].m_fTime )
		{
			return 1.0f;
		}
		else if(fTime >= m_xPrevTrackAmplitude[ ASURA_DYNAMIC_MUSIC_SYSTEM_FADE_RESOLUTION - 1 ].m_fTime )
		{
			return 0.0f;
		}
		pxTrackPoints = &m_xPrevTrackAmplitude[0];
	}
	else
	{
		if( fTime <= m_xNewTrackAmplitude[ 0 ].m_fTime )
		{
			return 0.0f;
		}
		else if(fTime >= m_xNewTrackAmplitude[ ASURA_DYNAMIC_MUSIC_SYSTEM_FADE_RESOLUTION - 1 ].m_fTime )
		{
			return 1.0f;
		}
		pxTrackPoints = &m_xNewTrackAmplitude[0];
	}
	int iNextPoint = 0;
	while ( ( iNextPoint < ASURA_DYNAMIC_MUSIC_SYSTEM_FADE_RESOLUTION ) && ( pxTrackPoints[ iNextPoint ].m_fTime < fTime ) )
	{
		++iNextPoint;
	}
	Asura_Assert( ( iNextPoint != 0 ) && ( pxTrackPoints[ iNextPoint ].m_fTime >= fTime ), "Failed to find time" );
	//Lerp between the previous and next point to get the amplitude at this time
	float fTotal = pxTrackPoints[ iNextPoint ].m_fTime - pxTrackPoints[ iNextPoint - 1 ].m_fTime;
	float fFraction = ( fTime - pxTrackPoints[ iNextPoint - 1 ].m_fTime ) / fTotal;
	return ( pxTrackPoints[ iNextPoint - 1 ].m_fAmplitude + ( pxTrackPoints[ iNextPoint ].m_fAmplitude - pxTrackPoints[ iNextPoint - 1 ].m_fAmplitude ) * fFraction );
}


float Asura_Dynamic_Music_Layer_Transition_Trigger_Data::GetFinishTime () const
{
	//Work out when we should finish, to do this we find the latest control point and shift it
	//forward in time as far as we can
	float fFinishTime = m_xPrevTrackAmplitude[ ASURA_DYNAMIC_MUSIC_SYSTEM_FADE_RESOLUTION - 1 ].m_fTime;
	
	if( m_uTrack == ASURA_HASH_ID_UNSET )
	{
		//User hasnt set a next track so ignore those control points
		if( RoughlyEqual( m_xPrevTrackAmplitude[ ASURA_DYNAMIC_MUSIC_SYSTEM_FADE_RESOLUTION - 1 ].m_fAmplitude, m_xPrevTrackAmplitude[ ASURA_DYNAMIC_MUSIC_SYSTEM_FADE_RESOLUTION - 2 ].m_fAmplitude ) )
		{
			fFinishTime = m_xPrevTrackAmplitude[ ASURA_DYNAMIC_MUSIC_SYSTEM_FADE_RESOLUTION - 2 ].m_fTime;
		}
		return fFinishTime;
	}

	float m_fPrevTrackEnd = m_xPrevTrackAmplitude[ ASURA_DYNAMIC_MUSIC_SYSTEM_FADE_RESOLUTION - 1 ].m_fTime;
	if( RoughlyEqual( m_xPrevTrackAmplitude[ ASURA_DYNAMIC_MUSIC_SYSTEM_FADE_RESOLUTION - 1 ].m_fAmplitude, m_xPrevTrackAmplitude[ ASURA_DYNAMIC_MUSIC_SYSTEM_FADE_RESOLUTION - 2 ].m_fAmplitude ) )
	{
		m_fPrevTrackEnd = m_xPrevTrackAmplitude[ ASURA_DYNAMIC_MUSIC_SYSTEM_FADE_RESOLUTION - 2 ].m_fTime;
	}

	float m_fNextTrackEnd = m_xNewTrackAmplitude[ ASURA_DYNAMIC_MUSIC_SYSTEM_FADE_RESOLUTION - 1 ].m_fTime;
	if( RoughlyEqual( m_xNewTrackAmplitude[ ASURA_DYNAMIC_MUSIC_SYSTEM_FADE_RESOLUTION - 1 ].m_fAmplitude, m_xNewTrackAmplitude[ ASURA_DYNAMIC_MUSIC_SYSTEM_FADE_RESOLUTION - 2 ].m_fAmplitude ) )
	{
		m_fNextTrackEnd = m_xNewTrackAmplitude[ ASURA_DYNAMIC_MUSIC_SYSTEM_FADE_RESOLUTION - 2 ].m_fTime;
	}

	fFinishTime = Asura_Maths::Max( m_fPrevTrackEnd, m_fNextTrackEnd );

	return fFinishTime;
}


bool Asura_Dynamic_Music_Layer_Transition_Trigger_Data::NeedToPlayOutro	() const
{
	// This function is so that we can avoid a seek to the track outro if possible
	float fFinishTime = m_xPrevTrackAmplitude[ ASURA_DYNAMIC_MUSIC_SYSTEM_FADE_RESOLUTION - 1 ].m_fTime;
	if( RoughlyEqual( m_xPrevTrackAmplitude[ ASURA_DYNAMIC_MUSIC_SYSTEM_FADE_RESOLUTION - 1 ].m_fAmplitude, m_xPrevTrackAmplitude[ ASURA_DYNAMIC_MUSIC_SYSTEM_FADE_RESOLUTION - 2 ].m_fAmplitude ) )
	{
		fFinishTime = m_xPrevTrackAmplitude[ ASURA_DYNAMIC_MUSIC_SYSTEM_FADE_RESOLUTION - 2 ].m_fTime;
	}
	return (fFinishTime > m_fPrevOutroStartOffset);
}

//--------------------------------------------------------------------------------------------------

Asura_Dynamic_Music_Stop_Trigger_Data::Asura_Dynamic_Music_Stop_Trigger_Data ()
{
	m_bUnqueueAllTriggers	= true;
	m_bJustOutro			= false;
	m_bActivateOnBarBoundry	= false;
	m_fBodyEndTime			= 5.0f;

	//Set the default amplitude curves
	ASURA_STATIC_ASSERT ( ASURA_DYNAMIC_MUSIC_SYSTEM_STOP_RESOLUTION == 4 );

	m_xTrackAmplitude[0].m_fTime		= 0.0f;
	m_xTrackAmplitude[0].m_fAmplitude	= 1.0f;

	m_xTrackAmplitude[1].m_fTime		= 0.0f;
	m_xTrackAmplitude[1].m_fAmplitude	= 0.0f;

	m_xTrackAmplitude[2].m_fTime		= 20.0f;
	m_xTrackAmplitude[2].m_fAmplitude	= 0.0f;

	m_xTrackAmplitude[3].m_fTime		= 30.0f;
	m_xTrackAmplitude[3].m_fAmplitude	= 0.0f;
}


Asura_Dynamic_Music_Stop_Trigger_Data::~Asura_Dynamic_Music_Stop_Trigger_Data ()
{

}


void Asura_Dynamic_Music_Stop_Trigger_Data::ReadFromChunkStream ( Asura_Chunk_Stream& xStream )
{
	int iVersion;
	xStream >> iVersion;

	Asura_Assert(iVersion <= CURRENT_DATA_VERSION, "Unknown dynamic music trigger chunk version");

	if( iVersion > 1 )
	{
		xStream >> m_bUnqueueAllTriggers;
		xStream >> m_bJustOutro;
		xStream >> m_bActivateOnBarBoundry;
		xStream >> m_fBodyEndTime;

		for(int i = 0; i < ASURA_DYNAMIC_MUSIC_SYSTEM_STOP_RESOLUTION; ++i )
		{
			xStream >> m_xTrackAmplitude[i].m_fTime;
			xStream >> m_xTrackAmplitude[i].m_fAmplitude;
		}
	}
}


void Asura_Dynamic_Music_Stop_Trigger_Data::WriteToChunkStream ( Asura_Chunk_Stream& xStream ) const
{
	int iVersion = CURRENT_DATA_VERSION;
	xStream << iVersion;

	xStream << m_bUnqueueAllTriggers;
	xStream << m_bJustOutro;
	xStream << m_bActivateOnBarBoundry;
	xStream << m_fBodyEndTime;

	for(int i = 0; i < ASURA_DYNAMIC_MUSIC_SYSTEM_STOP_RESOLUTION; ++i )
	{
		xStream << m_xTrackAmplitude[i].m_fTime;
		xStream << m_xTrackAmplitude[i].m_fAmplitude;
	}
}


float	Asura_Dynamic_Music_Stop_Trigger_Data::GetVolumeAtTime ( const float fTime ) const
{
	//Helper function for interpolating volumes
	if( fTime == 0.0f )
	{
		//If the time is zero find the last point @ 0.0 and return its amplitude
		int iPoint = 0;
		while( m_xTrackAmplitude[iPoint].m_fTime == 0.0f )
		{
			++iPoint;
		}
		return m_xTrackAmplitude[iPoint - 1].m_fAmplitude;
	}

	int iNextPoint = 0;
	while ( ( iNextPoint < ASURA_DYNAMIC_MUSIC_SYSTEM_STOP_RESOLUTION ) && ( m_xTrackAmplitude[ iNextPoint ].m_fTime < fTime ) )
	{
		++iNextPoint;
	}
	Asura_Assert( ( iNextPoint != 0 ) && ( m_xTrackAmplitude[ iNextPoint ].m_fTime >= fTime ), "Failed to find time" );
	//Lerp between the previous and next point to get the amplitude at this time
	float fTotal = m_xTrackAmplitude[ iNextPoint ].m_fTime - m_xTrackAmplitude[ iNextPoint - 1 ].m_fTime;
	float fFraction = ( fTime - m_xTrackAmplitude[ iNextPoint - 1 ].m_fTime ) / fTotal;
	return	(	m_xTrackAmplitude[ iNextPoint - 1].m_fAmplitude + 
			(	m_xTrackAmplitude[ iNextPoint	 ].m_fAmplitude - 
				m_xTrackAmplitude[ iNextPoint - 1].m_fAmplitude ) * fFraction );
}


float Asura_Dynamic_Music_Stop_Trigger_Data::GetFinishTime () const
{
	//Work out when we should finish, to do this we find the latest control point and shift it
	//forward in time as far as we can

	float fFinishTime = 0.0f;
	int iPoint = ASURA_DYNAMIC_MUSIC_SYSTEM_STOP_RESOLUTION - 1;
	while( m_xTrackAmplitude[iPoint].m_fAmplitude < 0.01f )
	{
		--iPoint;
	}
	fFinishTime = Asura_Maths::Max( fFinishTime, m_xTrackAmplitude[iPoint + 1].m_fTime );
	return fFinishTime;
}


bool Asura_Dynamic_Music_Stop_Trigger_Data::NeedToPlayOutro	() const
{
	// Helper function to let us know if we really need to play the outro
	if ( m_bJustOutro )
	{
		return true;
	}
			
	int iLastZero = ASURA_DYNAMIC_MUSIC_SYSTEM_STOP_RESOLUTION - 1;
	while( RoughlyEqual( m_xTrackAmplitude[ iLastZero ].m_fAmplitude, 0.0f ) )
	{
		--iLastZero;
	}
	
	return ( m_xTrackAmplitude[ iLastZero ].m_fTime > m_fBodyEndTime );
}


//--------------------------------------------------------------------------------------------------

Asura_Dynamic_Music_Play_Trigger_Data::Asura_Dynamic_Music_Play_Trigger_Data ()
{
	m_uTrack = ASURA_HASH_ID_UNSET;
	for( int iLayer = 0; iLayer < ASURA_DYNAMIC_MUSIC_SYSTEM_NUM_LAYERS; ++iLayer )
	{
		m_bMuteLayers[ iLayer ] = false;
	}
}


Asura_Dynamic_Music_Play_Trigger_Data::~Asura_Dynamic_Music_Play_Trigger_Data ()
{

}


void Asura_Dynamic_Music_Play_Trigger_Data::ReadFromChunkStream ( Asura_Chunk_Stream& xStream )
{
	int iVersion;
	xStream >> iVersion;
	Asura_Assert(iVersion <= CURRENT_DATA_VERSION, "Unknown dynamic music trigger chunk version, get new code");

	xStream >> m_uTrack;

	int iLayers = ASURA_DYNAMIC_MUSIC_SYSTEM_NUM_LAYERS;
	xStream >> iLayers;
	for( int i = 0; i < iLayers; ++i )
	{
		xStream >> m_bMuteLayers[i];
	}
}


void Asura_Dynamic_Music_Play_Trigger_Data::WriteToChunkStream ( Asura_Chunk_Stream& xStream ) const
{
	int iVersion = CURRENT_DATA_VERSION;
	xStream << iVersion;

	xStream << m_uTrack;

	int iLayers = ASURA_DYNAMIC_MUSIC_SYSTEM_NUM_LAYERS;
	xStream << iLayers;
	for( int i = 0; i < iLayers; ++i )
	{
		xStream << m_bMuteLayers[i];
	}
}

//--------------------------------------------------------------------------------------------------

Asura_Dynamic_Music_JoinRelease_Trigger_Data::Asura_Dynamic_Music_JoinRelease_Trigger_Data ()
{
	for( int iLayer = 0; iLayer < ASURA_DYNAMIC_MUSIC_SYSTEM_NUM_LAYERS; ++iLayer )
	{
		m_xLayerChanges[iLayer].m_eAction = ACTION_NONE;
		
		for( int iPoint = 0; iPoint < ASURA_DYNAMIC_MUSIC_SYSTEM_JOIN_REL_RESOLUTION; ++iPoint )
		{
			m_xLayerChanges[iLayer].m_axAmplitude[iPoint].m_fTime		= 0.0f;
			m_xLayerChanges[iLayer].m_axAmplitude[iPoint].m_fAmplitude	= 0.0f;
		}
	}
	m_bActivateOnBarBoundry = true;
}


Asura_Dynamic_Music_JoinRelease_Trigger_Data::~Asura_Dynamic_Music_JoinRelease_Trigger_Data ()
{

}

void Asura_Dynamic_Music_JoinRelease_Trigger_Data::ChangeLayerAction ( const int iLayer, const Actions eAction )
{
	Asura_Assert( ( iLayer >= 0 ) && ( iLayer < ASURA_DYNAMIC_MUSIC_SYSTEM_NUM_LAYERS ), "Invalid layer" );
	
	ASURA_STATIC_ASSERT( ASURA_DYNAMIC_MUSIC_SYSTEM_JOIN_REL_RESOLUTION == 4 );

	//This function sets some sensible defaults for each action type
	switch( eAction )
	{
		case ACTION_ENABLE:
		{
			m_xLayerChanges[iLayer].m_eAction						= ACTION_ENABLE;

			m_xLayerChanges[iLayer].m_axAmplitude[0].m_fTime		= 0.0f;
			m_xLayerChanges[iLayer].m_axAmplitude[0].m_fAmplitude	= 0.0f;

			m_xLayerChanges[iLayer].m_axAmplitude[1].m_fTime		= 10.0f;
			m_xLayerChanges[iLayer].m_axAmplitude[1].m_fAmplitude	= 0.0f;

			m_xLayerChanges[iLayer].m_axAmplitude[2].m_fTime		= 20.0f;
			m_xLayerChanges[iLayer].m_axAmplitude[2].m_fAmplitude	= 1.0f;

			m_xLayerChanges[iLayer].m_axAmplitude[3].m_fTime		= 30.0f;
			m_xLayerChanges[iLayer].m_axAmplitude[3].m_fAmplitude	= 1.0f;
		}
		break;

		case ACTION_DISABLE:
		{
			m_xLayerChanges[iLayer].m_eAction						= ACTION_DISABLE;

			m_xLayerChanges[iLayer].m_axAmplitude[0].m_fTime		= 0.0f;
			m_xLayerChanges[iLayer].m_axAmplitude[0].m_fAmplitude	= 1.0f;

			m_xLayerChanges[iLayer].m_axAmplitude[1].m_fTime		= 10.0f;
			m_xLayerChanges[iLayer].m_axAmplitude[1].m_fAmplitude	= 1.0f;

			m_xLayerChanges[iLayer].m_axAmplitude[2].m_fTime		= 20.0f;
			m_xLayerChanges[iLayer].m_axAmplitude[2].m_fAmplitude	= 0.0f;

			m_xLayerChanges[iLayer].m_axAmplitude[3].m_fTime		= 30.0f;
			m_xLayerChanges[iLayer].m_axAmplitude[3].m_fAmplitude	= 0.0f;
		}
		break;
		
		case ACTION_NONE:	break;
	}

	m_xLayerChanges[iLayer].m_eAction = eAction;
}

void Asura_Dynamic_Music_JoinRelease_Trigger_Data::ReadFromChunkStream ( Asura_Chunk_Stream& xStream )
{
	int iVersion;
	xStream >> iVersion;
	Asura_Assert(iVersion <= CURRENT_DATA_VERSION, "Unknown dynamic music trigger chunk version");

	xStream >> m_bActivateOnBarBoundry;

	int iNumLayers = 0;
	xStream >> iNumLayers;
	Asura_Assert( iNumLayers == ASURA_DYNAMIC_MUSIC_SYSTEM_NUM_LAYERS, "Unexpected number of layers" );

	for( int iLayer = 0; iLayer < iNumLayers; ++iLayer )
	{
		int iAction = 0;
		xStream >> iAction;
		m_xLayerChanges[iLayer].m_eAction = static_cast<Actions>( iAction );
		if(m_xLayerChanges[iLayer].m_eAction != ACTION_NONE)
		{
			int iRes = 0;
			xStream >> iRes;
			Asura_Assert( iRes == ASURA_DYNAMIC_MUSIC_SYSTEM_JOIN_REL_RESOLUTION, "Unexpected fade resolution");
			for( int iPoint = 0; iPoint < iRes; ++iPoint )
			{
				xStream >> m_xLayerChanges[iLayer].m_axAmplitude[iPoint].m_fTime;
				xStream >> m_xLayerChanges[iLayer].m_axAmplitude[iPoint].m_fAmplitude;
			}
		}
	}
}


void Asura_Dynamic_Music_JoinRelease_Trigger_Data::WriteToChunkStream ( Asura_Chunk_Stream& xStream ) const
{
	int iVersion = CURRENT_DATA_VERSION;
	xStream << iVersion;

	xStream << m_bActivateOnBarBoundry;

	int iLayers = ASURA_DYNAMIC_MUSIC_SYSTEM_NUM_LAYERS;
	xStream << iLayers;

	for( int iLayer = 0; iLayer < ASURA_DYNAMIC_MUSIC_SYSTEM_NUM_LAYERS; ++iLayer )
	{
		int iAction = static_cast<int>(m_xLayerChanges[iLayer].m_eAction);
		xStream << iAction;
		if(m_xLayerChanges[iLayer].m_eAction != ACTION_NONE)
		{
			//Save some space by not writing this stuff for no action		
			int iRes = 4;
			xStream << iRes;
			for( int iPoint = 0; iPoint < ASURA_DYNAMIC_MUSIC_SYSTEM_JOIN_REL_RESOLUTION; ++iPoint )
			{
				xStream << m_xLayerChanges[iLayer].m_axAmplitude[iPoint].m_fTime;
				xStream << m_xLayerChanges[iLayer].m_axAmplitude[iPoint].m_fAmplitude;
			}
		}
	}
}

float Asura_Dynamic_Music_JoinRelease_Trigger_Data::GetVolumeAtTime ( const int iLayer, const float fTime ) const
{
	//Helper function for interpolating volumes
	const Dynamic_Music_Volume_Point* pxTrackPoints = m_xLayerChanges[iLayer].m_axAmplitude;
	
	if( fTime == 0.0f )
	{
		//If the time is zero find the last point @ 0.0 and return its amplitude
		int iPoint = 0;
		while( pxTrackPoints[iPoint].m_fTime == 0.0f )
		{
			++iPoint;
		}
		return pxTrackPoints[iPoint - 1].m_fAmplitude;
	}

	int iNextPoint = 0;
	while ( ( iNextPoint < ASURA_DYNAMIC_MUSIC_SYSTEM_JOIN_REL_RESOLUTION ) && ( pxTrackPoints[ iNextPoint ].m_fTime < fTime ) )
	{
		++iNextPoint;
	}
	Asura_Assert( ( iNextPoint != 0 ) && ( pxTrackPoints[ iNextPoint ].m_fTime >= fTime ), "Failed to find time" );
	//Lerp between the previous and next point to get the amplitude at this time
	float fTotal = pxTrackPoints[ iNextPoint ].m_fTime - pxTrackPoints[ iNextPoint - 1 ].m_fTime;
	float fFraction = ( fTime - pxTrackPoints[ iNextPoint - 1 ].m_fTime ) / fTotal;
	return	(	pxTrackPoints[ iNextPoint - 1	].m_fAmplitude + 
			(	pxTrackPoints[ iNextPoint		].m_fAmplitude - 
				pxTrackPoints[ iNextPoint - 1	].m_fAmplitude ) * fFraction );
}

float Asura_Dynamic_Music_JoinRelease_Trigger_Data::GetFinishTime () const
{
	//Work out when we should finish, to do this we find the latest control point and shift it
	//forward in time as far as we can

	float fFinishTime = 0.0f;
	for( int iLayer = 0; iLayer < ASURA_DYNAMIC_MUSIC_SYSTEM_NUM_LAYERS; ++iLayer )
	{
		if( m_xLayerChanges[iLayer].m_eAction == ACTION_ENABLE )
		{
			int iPoint = ASURA_DYNAMIC_MUSIC_SYSTEM_JOIN_REL_RESOLUTION - 1;
			while( m_xLayerChanges[iLayer].m_axAmplitude[iPoint].m_fAmplitude > 0.99f )
			{
				--iPoint;
			}
			fFinishTime = Asura_Maths::Max( fFinishTime, m_xLayerChanges[iLayer].m_axAmplitude[iPoint + 1].m_fTime );
		}
		else if( m_xLayerChanges[iLayer].m_eAction == ACTION_DISABLE )
		{
			int iPoint = ASURA_DYNAMIC_MUSIC_SYSTEM_JOIN_REL_RESOLUTION - 1;
			while( m_xLayerChanges[iLayer].m_axAmplitude[iPoint].m_fAmplitude < 0.01f )
			{
				--iPoint;
			}
			fFinishTime = Asura_Maths::Max( fFinishTime, m_xLayerChanges[iLayer].m_axAmplitude[iPoint + 1].m_fTime );
		}
	}
	return fFinishTime;
}

//--------------------------------------------------------------------------------------------------

Asura_Dynamic_Music_Trigger_Data::Asura_Dynamic_Music_Trigger_Data ()
{
	m_uHash = ASURA_HASH_ID_UNSET;
	m_eType = DYNAMIC_MUSIC_TRIGGER_TRANSITION;
	m_bCodeTrigger = false;
	TriggerData.m_pxLayerTransition = 0;
	m_xCodeTriggerName.MakeBlank();
}

Asura_Dynamic_Music_Trigger_Data::~Asura_Dynamic_Music_Trigger_Data ()
{
	switch( m_eType )
	{
		case DYNAMIC_MUSIC_TRIGGER_TRANSITION:	delete TriggerData.m_pxLayerTransition;	TriggerData.m_pxLayerTransition = 0;	break;
		case DYNAMIC_MUSIC_TRIGGER_PLAY:		delete TriggerData.m_pxPlay;			TriggerData.m_pxPlay			= 0;	break;
		case DYNAMIC_MUSIC_TRIGGER_STOP:		delete TriggerData.m_pxStop;			TriggerData.m_pxStop			= 0;	break;
		case DYNAMIC_MUSIC_TRIGGER_JOINRELEASE:	delete TriggerData.m_pxJoinRelease;		TriggerData.m_pxJoinRelease		= 0;	break;
		
		case DYNAMIC_MUSIC_NUM_TRIGGER_TYPES:
		default:
		{
			Asura_Assert( false, "Unrecognised trigger type" );
		}
		break;
	}
}

void Asura_Dynamic_Music_Trigger_Data::SetTriggerType ( Asura_Dynamic_Music_TriggerTypes eNewType )
{
	switch( m_eType )
	{
		case DYNAMIC_MUSIC_TRIGGER_TRANSITION:	delete TriggerData.m_pxLayerTransition;		TriggerData.m_pxLayerTransition	= 0;	break;
		case DYNAMIC_MUSIC_TRIGGER_PLAY:		delete TriggerData.m_pxPlay;				TriggerData.m_pxPlay			= 0;	break;
		case DYNAMIC_MUSIC_TRIGGER_STOP:		delete TriggerData.m_pxStop;				TriggerData.m_pxStop			= 0;	break;
		case DYNAMIC_MUSIC_TRIGGER_JOINRELEASE:	delete TriggerData.m_pxJoinRelease;			TriggerData.m_pxJoinRelease		= 0;	break;

		case DYNAMIC_MUSIC_NUM_TRIGGER_TYPES:	//keep the PS3 happy
		default:
		{
			Asura_Assert( false, "Unrecognised trigger type" );
		}
		break;

	}

	switch ( eNewType )
	{
		case DYNAMIC_MUSIC_TRIGGER_TRANSITION:	TriggerData.m_pxLayerTransition = new Asura_Dynamic_Music_Layer_Transition_Trigger_Data ();	break;
		case DYNAMIC_MUSIC_TRIGGER_PLAY:		TriggerData.m_pxPlay			= new Asura_Dynamic_Music_Play_Trigger_Data				();	break;
		case DYNAMIC_MUSIC_TRIGGER_STOP:		TriggerData.m_pxStop			= new Asura_Dynamic_Music_Stop_Trigger_Data				();	break;
		case DYNAMIC_MUSIC_TRIGGER_JOINRELEASE:	TriggerData.m_pxJoinRelease		= new Asura_Dynamic_Music_JoinRelease_Trigger_Data		(); break;

		case DYNAMIC_MUSIC_NUM_TRIGGER_TYPES:	//keep the PS3 happy
		default:
		{
			Asura_Assert( false, "Unrecognised trigger type" );
		}
		break;
	}

	m_eType = eNewType;
}

void Asura_Dynamic_Music_Trigger_Data::ReplaceTrackReferences ( const Asura_Hash_ID uOldHash, const Asura_Hash_ID uNewHash )
{
	// this is for tools, for when we rename tracks
	switch( m_eType )
	{
		case DYNAMIC_MUSIC_TRIGGER_TRANSITION:
		{
			if( TriggerData.m_pxLayerTransition->m_uTrack == uOldHash )
			{
				TriggerData.m_pxLayerTransition->m_uTrack = uNewHash;
			}
		}
		case DYNAMIC_MUSIC_TRIGGER_PLAY:
		{
			if( TriggerData.m_pxPlay->m_uTrack == uOldHash )
			{
				TriggerData.m_pxPlay->m_uTrack = uNewHash;
			}
		}

		case DYNAMIC_MUSIC_TRIGGER_STOP:			//keep the PS3 happy
		case DYNAMIC_MUSIC_TRIGGER_JOINRELEASE:
		case DYNAMIC_MUSIC_NUM_TRIGGER_TYPES:
		break;
	}
}

void Asura_Dynamic_Music_Trigger_Data::ReadFromChunkStream ( Asura_Chunk_Stream& xStream )
{
	int iVersion;
	xStream >> iVersion;
	Asura_Assert(iVersion <= CURRENT_DATA_VERSION, "Unknown dynamic music layer transition trigger chunk version, get new code");

	if( iVersion == 1 )
	{
		Asura_String xIgnore;
		xStream >> xIgnore;
	}
	
	int iType = 0;
	xStream >> iType;

	// < version 3 is a GUID, which is still a unique unsigned int - so we're ok
	xStream >> m_uHash;

	if( iVersion >= 4 )
	{
		xStream >> m_bCodeTrigger;
		xStream >> m_xCodeTriggerName;
	}

	SetTriggerType( static_cast<Asura_Dynamic_Music_TriggerTypes>( iType ) );

	switch( m_eType )
	{
		case DYNAMIC_MUSIC_TRIGGER_TRANSITION:	TriggerData.m_pxLayerTransition	->ReadFromChunkStream( xStream ); break;
		case DYNAMIC_MUSIC_TRIGGER_PLAY:		TriggerData.m_pxPlay			->ReadFromChunkStream( xStream ); break;
		case DYNAMIC_MUSIC_TRIGGER_STOP:		TriggerData.m_pxStop			->ReadFromChunkStream( xStream ); break;
		case DYNAMIC_MUSIC_TRIGGER_JOINRELEASE:	TriggerData.m_pxJoinRelease		->ReadFromChunkStream( xStream ); break;

		case DYNAMIC_MUSIC_NUM_TRIGGER_TYPES:
		default:
		{
			Asura_Assert( false, "Unrecognised trigger type" );
		}
		break;
	}
}

void Asura_Dynamic_Music_Trigger_Data::WriteToChunkStream( Asura_Chunk_Stream& xStream ) const
{
	int iVersion = CURRENT_DATA_VERSION;
	xStream << iVersion;

	xStream << m_eType;
	xStream << m_uHash;
	xStream << m_bCodeTrigger;
	xStream << m_xCodeTriggerName;

	switch( m_eType )
	{
		case DYNAMIC_MUSIC_TRIGGER_TRANSITION:	TriggerData.m_pxLayerTransition	->WriteToChunkStream( xStream ); break;
		case DYNAMIC_MUSIC_TRIGGER_PLAY:		TriggerData.m_pxPlay			->WriteToChunkStream( xStream ); break;
		case DYNAMIC_MUSIC_TRIGGER_STOP:		TriggerData.m_pxStop			->WriteToChunkStream( xStream ); break;
		case DYNAMIC_MUSIC_TRIGGER_JOINRELEASE:	TriggerData.m_pxJoinRelease		->WriteToChunkStream( xStream ); break;
		
		case DYNAMIC_MUSIC_NUM_TRIGGER_TYPES:
		default:
		{
			Asura_Assert( false, "Unrecognised trigger type" );
		}
		break;
	}
}

#endif

//eof
