// Asura_Dynamic_Music_System_Triggers.h
// Thomas Young - 28.08.08
// Copyright (C) 2008 Rebellion, All Rights Reserved.

// F I L E   W R A P P E R ///////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __ASURA_DYNAMIC_MUSIC_SYSTEM_TRIGGERS__
#define __ASURA_DYNAMIC_MUSIC_SYSTEM_TRIGGERS__

#include "Core/Asura.h"

#ifdef ASURA_SUPPORT_DYNAMIC_MUSIC_SYSTEM

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "Core/Asura_String.h"
#include "Sound/Asura_Dynamic_Music_System.h"

// D E F I N E S /////////////////////////////////////////////////////////////

#define ASURA_DYNAMIC_MUSIC_SYSTEM_FADE_RESOLUTION			3	//How many points define a fade in/out
#define ASURA_DYNAMIC_MUSIC_SYSTEM_JOIN_REL_RESOLUTION		4	//How many points define a join/release amplitude curve
#define ASURA_DYNAMIC_MUSIC_SYSTEM_STOP_RESOLUTION			4	//How many points define a stop amplitude curve

// E N U M E R A T I O N S ///////////////////////////////////////////////////

enum Asura_Dynamic_Music_TriggerTypes
{
	DYNAMIC_MUSIC_TRIGGER_TRANSITION = 0,
	DYNAMIC_MUSIC_TRIGGER_PLAY,
	DYNAMIC_MUSIC_TRIGGER_STOP,
	DYNAMIC_MUSIC_TRIGGER_JOINRELEASE,

	DYNAMIC_MUSIC_NUM_TRIGGER_TYPES
};

// C L A S S   D E C L A R A T I O N S ///////////////////////////////////////

class Dynamic_Music_Volume_Point
{
	public:
		
		float	m_fTime;		
		float	m_fAmplitude;
};

class Asura_Dynamic_Music_Layer_Transition_Trigger_Data
{
	public:

		enum { CURRENT_DATA_VERSION = 2 };

		Asura_Dynamic_Music_Layer_Transition_Trigger_Data ();
		~Asura_Dynamic_Music_Layer_Transition_Trigger_Data() {};

		Asura_Hash_ID	m_uTrack;

		float			m_fPrevOutroStartOffset;
		float			m_fPrevOutroEndOffset;
		float			m_fNextIntroStartOffset;
		float			m_fNextIntroEndOffset;

		bool			m_bUseIntro;

		bool			m_bMuteLayers[ASURA_DYNAMIC_MUSIC_SYSTEM_NUM_LAYERS];

		// Quick sketch to show what these values mean
		// |  Body  | Outro |               | 1
		// |        |       |               | 0   volume
		// |            | Intro | Body      | 1 on this axis
		// |____________|_______|___________| 0
		//        Time on this axis

		//These points define the linear volume changes to fade in/out the tracks
		Dynamic_Music_Volume_Point	m_xPrevTrackAmplitude	[ASURA_DYNAMIC_MUSIC_SYSTEM_FADE_RESOLUTION];	
		Dynamic_Music_Volume_Point	m_xNewTrackAmplitude	[ASURA_DYNAMIC_MUSIC_SYSTEM_FADE_RESOLUTION];

		enum Track
		{
			TRACK_PREVIOUS	= 0,
			TRACK_NEXT		= 1,
		};

		void	ReadFromChunkStream	( Asura_Chunk_Stream& xStream );
		void	WriteToChunkStream	( Asura_Chunk_Stream& xStream ) const;

		float	GetVolumeAtTime		( const float fTime, const Asura_Dynamic_Music_Layer_Transition_Trigger_Data::Track eTrack ) const;
		float	GetFinishTime		() const;
		bool	NeedToPlayOutro		() const;
};

// -----------------------------------------------------------------------------

class Asura_Dynamic_Music_Play_Trigger_Data
{
	public:
		
		enum { CURRENT_DATA_VERSION = 1 };

		Asura_Dynamic_Music_Play_Trigger_Data	();
		~Asura_Dynamic_Music_Play_Trigger_Data	();

		Asura_Hash_ID	m_uTrack;
		bool			m_bMuteLayers[ASURA_DYNAMIC_MUSIC_SYSTEM_NUM_LAYERS];

		void ReadFromChunkStream	( Asura_Chunk_Stream& xStream );
		void WriteToChunkStream		( Asura_Chunk_Stream& xStream ) const;
};

// -----------------------------------------------------------------------------

class Asura_Dynamic_Music_Stop_Trigger_Data
{
	public:
	
		// ------------ Enumerations ----------------

		enum { CURRENT_DATA_VERSION = 2 };

		// ------------ Functions ----------------

		Asura_Dynamic_Music_Stop_Trigger_Data	();
		~Asura_Dynamic_Music_Stop_Trigger_Data	();

		float	GetVolumeAtTime					( const float fTime ) const;
		float	GetFinishTime					() const;
		bool	NeedToPlayOutro					() const;

		void ReadFromChunkStream				( Asura_Chunk_Stream& xStream )	;
		void WriteToChunkStream					( Asura_Chunk_Stream& xStream ) const;

		// ------------ Variables ----------------

		bool						m_bUnqueueAllTriggers;
		bool						m_bActivateOnBarBoundry;
		bool						m_bJustOutro;
		float						m_fBodyEndTime;
		Dynamic_Music_Volume_Point	m_xTrackAmplitude	[ASURA_DYNAMIC_MUSIC_SYSTEM_STOP_RESOLUTION];
};

// -----------------------------------------------------------------------------

class Asura_Dynamic_Music_JoinRelease_Trigger_Data
{
	public:
	
		// ------------ Enumerations ----------------
		
		enum { CURRENT_DATA_VERSION = 1 };
				
		enum Actions
		{
			ACTION_NONE		= 0,
			ACTION_ENABLE	,
			ACTION_DISABLE
		};

		// ------------ Structures ----------------

		struct LayerChange
		{
			Actions						m_eAction;
			Dynamic_Music_Volume_Point	m_axAmplitude[ASURA_DYNAMIC_MUSIC_SYSTEM_JOIN_REL_RESOLUTION];
		};

		// ------------ Functions ----------------

		Asura_Dynamic_Music_JoinRelease_Trigger_Data	();
		~Asura_Dynamic_Music_JoinRelease_Trigger_Data	();

		void	ReadFromChunkStream		( Asura_Chunk_Stream& xStream )	;
		void	WriteToChunkStream		( Asura_Chunk_Stream& xStream ) const;

		void	ChangeLayerAction		( const int iLayer, const Actions eAction );

		float	GetVolumeAtTime			( const int iLayer, const float fTime ) const;
		float	GetFinishTime			() const;

		// ------------ Variables ----------------
		
		LayerChange	m_xLayerChanges[ASURA_DYNAMIC_MUSIC_SYSTEM_NUM_LAYERS];
		bool		m_bActivateOnBarBoundry;

};

// -----------------------------------------------------------------------------

class Asura_Dynamic_Music_Trigger_Data
{
	public:
	
		// Versions:	1	- initial version
		//				2	- removed trigger name (waste of space)
		//				3	- changed m_xGuid to be an Asura_Hash_ID
		//				4	- added code trigger flag and name

		enum { CURRENT_DATA_VERSION = 4 };

		//------------ Variables --------------------
		
		Asura_Hash_ID						m_uHash;			// Identifier so we can look up the trigger
		Asura_Dynamic_Music_TriggerTypes	m_eType;
		bool								m_bCodeTrigger;		// true = this is a code trigger. false = standard trigger
		Asura_String						m_xCodeTriggerName;	// only valid if m_bCodeTrigger == true

		union TriggerDataPointers
		{
			Asura_Dynamic_Music_Layer_Transition_Trigger_Data*	m_pxLayerTransition;
			Asura_Dynamic_Music_Play_Trigger_Data*				m_pxPlay;
			Asura_Dynamic_Music_Stop_Trigger_Data*				m_pxStop;
			Asura_Dynamic_Music_JoinRelease_Trigger_Data*		m_pxJoinRelease;

		} TriggerData;

		//------------ Functions --------------------

		 Asura_Dynamic_Music_Trigger_Data ();
		~Asura_Dynamic_Music_Trigger_Data ();

		void SetTriggerType ( Asura_Dynamic_Music_TriggerTypes eNewType );

		void ReplaceTrackReferences ( const Asura_Hash_ID uOldHash, const Asura_Hash_ID uNewHash );

		void ReadFromChunkStream	( Asura_Chunk_Stream& xStream );
		void WriteToChunkStream		( Asura_Chunk_Stream& xStream ) const;

};

#endif //__ASURA_DYNAMIC_MUSIC_SYSTEM_TRIGGERS__

#endif //ASURA_SUPPORT_DYNAMIC_MUSIC_SYSTEM

//eof
