//////////////////////////////////////////////////////////////////////////////
//
// Asura_XAudio2_Buffer.h
//
// Copyright (c) 2008 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

#if defined ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#if !defined __ASURA_XAUDIO2_BUFFER_H__
#define __ASURA_XAUDIO2_BUFFER_H__

#if defined ASURA_USE_AUDIO_SYSTEM

/******************************************************************************/
/* Included Files                                                             */
/******************************************************************************/

// Asura Platform-independent
#include "Sound/Asura_Audio_System.h"

// Misc
#include <xaudio2.h>

/******************************************************************************/
/* Classes                                                                    */
/******************************************************************************/

class Asura_Chunk_Stream;

// Union of wave formats we support playing
union Asura_XAudio2_WaveFormat
{
	WAVEFORMATEXTENSIBLE m_xWFX;
	ADPCMWAVEFORMAT m_xADPCM;
	XMA2WAVEFORMATEX m_xXMA2;
};

////////////////////////////////////////////////////////////////////////////////
//
// Asura_XAudio2_Buffer
//
// Class wrapping an audio buffer (wave file)
//
////////////////////////////////////////////////////////////////////////////////

class Asura_XAudio2_Buffer
{
	friend class Asura_XAudio2;

public:

	Asura_XAudio2_Buffer();
	~Asura_XAudio2_Buffer();

	bool LoadWave( Asura_Chunk_Stream& xStream, const u_int uFileSize, const u_int uFlags );
	void SetLooping( const bool bLooping );

	#if defined ASURA_TOOLS
		bool LoadIfDelayed( Asura_Chunk_Stream* pxStream = NULL, const u_int uFileSize = 0 );
		void Unload();
		float GetDuration() const;
	#endif

	#if defined ASURA_AUDIO_STORE_FILENAMES
		const char* GetFilename() const { return m_szFilename; }
	#else
		const char* GetFilename() const { return "<Unknown>"; }
	#endif

	const XAUDIO2_BUFFER* GetParams() const { return &m_xParams; }
	const XAUDIO2_BUFFER_WMA* GetWMA() const;

protected:

	bool ParseWaveChunk( Asura_Chunk_Stream& xStream );

	XAUDIO2_BUFFER m_xParams;
	XAUDIO2_BUFFER_WMA m_xWMA;

	Asura_XAudio2_WaveFormat m_xFmt;

	void* m_pAudioData;

	#if defined ASURA_AUDIO_STORE_FILENAMES
		// Filename stored for easier debugging, and also delayed loading in Tools
		const char* m_szFilename;
	#endif

	#if defined ASURA_TOOLS
		void CreateBlankWave();

		// Used delayed loading for individual files in Tools builds
		bool m_bAttemptedLoad;
	#endif
};

////////////////////////////////////////////////////////////////////////////////
//
// Asura_XAudio2_WaveReader
//
// Helper class for loading wave files
//
////////////////////////////////////////////////////////////////////////////////

class Asura_XAudio2_WaveReader
{
public:

	Asura_XAudio2_WaveReader( Asura_Chunk_Stream& xStream, Asura_XAudio2_WaveFormat& xFmt,
		u_int& uTotalDataSize, void** ppAudioData = NULL );

	bool ReadWave();

protected:

	enum PCReturnCode
	{
		ASURA_PCRC_CONTINUE,
		ASURA_PCRC_FINISHED,
		ASURA_PCRC_ERROR,
	};

	PCReturnCode ParseChunk();

	Asura_Chunk_Stream& m_xStream;

	Asura_XAudio2_WaveFormat& m_xFmt;
	u_int& m_uTotalDataSize;

	void** m_ppAudioData;
};

#endif // defined ASURA_USE_AUDIO_SYSTEM

#endif // !defined __ASURA_XAUDIO2_BUFFER_H__
