//////////////////////////////////////////////////////////////////////////////
//
// Asura_XAudio2_Stream.h
//
// Copyright (c) 2008 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

#if defined ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#if !defined __ASURA_XAUDIO2_STREAM_H__
#define __ASURA_XAUDIO2_STREAM_H__

#if defined ASURA_USE_AUDIO_SYSTEM

/******************************************************************************/
/* Included Files                                                             */
/******************************************************************************/

// Asura Platform-independent
#include "Core/Asura_Multithread.h"
#include "Sound/Asura_XAudio2_Buffer.h"

// Misc
#include <xaudio2.h>

/******************************************************************************/
/* Macros                                                                     */
/******************************************************************************/

#if !defined ASURA_AUDIO_STREAMING_BUFFER_SIZE
	// Two of these allocated per stream
	#define ASURA_AUDIO_STREAMING_BUFFER_SIZE (64 * 1024)
#endif

/******************************************************************************/
/* Classes                                                                    */
/******************************************************************************/

////////////////////////////////////////////////////////////////////////////////
//
// Asura_XAudio2_Stream
//
// Represents a double-buffered audio stream, which can be played through an
// Asura_XAudio2_Voice
//
////////////////////////////////////////////////////////////////////////////////

class Asura_XAudio2_Stream
{
public:

	Asura_XAudio2_Stream();
	~Asura_XAudio2_Stream();

	void Reset();

	void Release() { m_bReleased = true; m_pxSourceVoice = NULL; }
	bool CanBeDestroyed() const { return m_bReleased && !m_bReadPending; }

	bool OpenFile( const char* const szFilename, const u_int uOffset = 0 );
	void CloseFile();

	const Asura_XAudio2_WaveFormat& GetFormat() const { return m_xFmt; }
	u_int GetSampleRate() const { return m_xFmt.m_xWFX.Format.nSamplesPerSec; }
	void Bind( IXAudio2SourceVoice* const pxVoice, const float fTimeOffset );

	void SetLooping( const bool bLoop ) { m_bLooping = bLoop; }
	bool IsLooping() const { return m_bLooping; }

	bool SeekAtPlaybackTime( const float fSeekTime, const float fPlaybackTime );
	bool SeekIsPending() const { return m_bSeekPending; }

	void EnqueueRead();

	Asura_Mutex& GetMutex() { return m_xMutex; }

	void BufferStarted();

protected:

	void Read();

	u_int SecondsToBytes( const float fSecs ) const;
	float BytesToSeconds( const u_int uBytes ) const;

	static void ReadFromStream( void* pContext, u_int uDataSize );

	int GetMinimumReadSize () const;

	Asura_Mutex m_xMutex;

	Asura_XAudio2_WaveFormat m_xFmt;

	IXAudio2SourceVoice* m_pxSourceVoice;

	void* m_apAudioData[2];
	u_int m_uCurrBuffer;

	u_int m_uStartLocation;
	u_int m_uBytesLeft;
	u_int m_uTotalDataSize;
	u_int m_uBytesTillSeek;
	u_int m_uSeekTarget;
	Asura_Handle m_iFileHandle;
	bool m_bLooping;
	bool m_bSeekPending;
	bool m_bReadPending;
	bool m_bReleased;

private:

	// Prevent copy construction and assignment
	Asura_XAudio2_Stream( const Asura_XAudio2_Stream& );
	Asura_XAudio2_Stream& operator=( const Asura_XAudio2_Stream& );
};

#endif // defined ASURA_USE_AUDIO_SYSTEM

#endif // !defined __ASURA_XAUDIO2_STREAM_H__
