// Asura_XAudio2_XAPO_FX.h
// Thomas Young - 20.03.09
// Copyright (C) 2009 Rebellion, All Rights Reserved.

// F I L E   W R A P P E R ///////////////////////////////////////////////////

#if defined ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#if !defined __ASURA_XAUDIO2_XAPO_FX_H__
#define __ASURA_XAUDIO2_XAPO_FX_H__

#ifdef ASURA_USE_AUDIO_SYSTEM

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Sound/Asura_Audio_DSP.h"

#include <xapobase.h>
#include <xapo.h>


// C L A S S   D E C L A R A T I O N S ///////////////////////////////////////

// _____________________________ Asura_XAPOBase ___________________________________

template<typename APOClass, typename ParameterClass> class Asura_XAPOBase : public CXAPOParametersBase
{
	public:
		
		STDMETHOD	( LockForProcess )	(	UINT32 InputLockedParameterCount,
											const XAPO_LOCKFORPROCESS_BUFFER_PARAMETERS *pInputLockedParameters,
											UINT32 OutputLockedParameterCount,
											const XAPO_LOCKFORPROCESS_BUFFER_PARAMETERS *pOutputLockedParameters );

		STDMETHOD_	( void, Process )	(	UINT32 InputProcessParameterCount,
											const XAPO_PROCESS_BUFFER_PARAMETERS *pInputProcessParameters,
											UINT32 OutputProcessParameterCount,
											XAPO_PROCESS_BUFFER_PARAMETERS *pOutputProcessParameters,
											BOOL IsEnabled );

	protected:
		
		Asura_XAPOBase		( );
		~Asura_XAPOBase		( );

		virtual void DoProcess ( float* pfInputBuffer, u_int uFrames, u_int uChannels, const ParameterClass& xParams ) = 0;

		WAVEFORMATEX& GetWaveFormat( )
		{ 
			return m_xWaveFormat; 
		}

		virtual bool AreParamsUpdated	( )	
		{ 
			return m_bParamsChanged; 
		}

		virtual void UpdateParameters( const ParameterClass& xParams ) { };

	private:
	
		void OnSetParameters (const void* pParams, UINT32 cbParams)
		{
			Asura_Assert( cbParams == sizeof( ParameterClass ), "" );
			cbParams;
			UpdateParameters( *(ParameterClass*)pParams );
		}

		ParameterClass	m_parameters[3];
		WAVEFORMATEX	m_xWaveFormat;
		bool			m_bParamsChanged;
		static XAPO_REGISTRATION_PROPERTIES m_xRegProps;
};


template<typename APOClass, typename ParameterClass> __declspec(selectany) XAPO_REGISTRATION_PROPERTIES Asura_XAPOBase<APOClass, ParameterClass>::m_xRegProps = 
{
	__uuidof(APOClass),
	L"Asura_XAPOBase",
	L"Copyright (C) 2009 Rebellion",
	1,
	0,
	XAPO_FLAG_INPLACE_REQUIRED			|
	XAPO_FLAG_CHANNELS_MUST_MATCH		|
	XAPO_FLAG_FRAMERATE_MUST_MATCH		|
	XAPO_FLAG_BITSPERSAMPLE_MUST_MATCH	|
	XAPO_FLAG_BUFFERCOUNT_MUST_MATCH	|
	XAPO_FLAG_INPLACE_SUPPORTED,
	1,
	1,
	1,
	1
};

template<typename APOClass, typename ParameterClass> Asura_XAPOBase<APOClass, ParameterClass>::Asura_XAPOBase( )
	: CXAPOParametersBase( &m_xRegProps, (BYTE*)m_parameters, sizeof( ParameterClass ), FALSE )
{
	#ifdef _XBOX_VER
	XMemSet( (BYTE*)m_parameters, 0, sizeof( m_parameters ) );
	#else
	ZeroMemory( m_parameters, sizeof( m_parameters ) );
	#endif
	m_bParamsChanged = false;
}


template<typename APOClass, typename ParameterClass> Asura_XAPOBase<APOClass, ParameterClass>::~Asura_XAPOBase()
{
}


template<typename APOClass, typename ParameterClass> HRESULT Asura_XAPOBase<APOClass, ParameterClass>::LockForProcess	(	UINT32 InputLockedParameterCount
																														,	const XAPO_LOCKFORPROCESS_BUFFER_PARAMETERS *pInputLockedParameters
																														,	UINT32 OutputLockedParameterCount
																														,	const XAPO_LOCKFORPROCESS_BUFFER_PARAMETERS *pOutputLockedParameters )
{
	HRESULT xResult = CXAPOParametersBase::LockForProcess	(	InputLockedParameterCount
															,	pInputLockedParameters
															,	OutputLockedParameterCount
															,	pOutputLockedParameters		);

	if( SUCCEEDED( xResult ) )
	{
		memcpy( &m_xWaveFormat, pInputLockedParameters[0].pFormat, sizeof( WAVEFORMATEX ) );
	}
	return xResult;
}

template<typename APOClass, typename ParameterClass> void Asura_XAPOBase<APOClass, ParameterClass>::Process (	UINT32 uInputProcessParameterCount
																											,	const XAPO_PROCESS_BUFFER_PARAMETERS *pxInputProcessParameters
																											,	UINT32 uOutputProcessParameterCount
																											,	XAPO_PROCESS_BUFFER_PARAMETERS *pxOutputProcessParameters
																											,	BOOL IsEnabled )
{
	Asura_Assert( IsLocked(), "XAPO DSP: Expected locked for processing" );
	Asura_Assert( uInputProcessParameterCount == 1, "XAPO DSP: Expected one input param" );
	Asura_Assert( uOutputProcessParameterCount == 1, "XAPO DSP: Expected one output param" );
	Asura_Assert( pxInputProcessParameters[0].pBuffer == pxOutputProcessParameters[0].pBuffer, "XAPO DSP: Input param should match output param" );

	UNREFERENCED_PARAMETER( uOutputProcessParameterCount );
	UNREFERENCED_PARAMETER( uInputProcessParameterCount );
	UNREFERENCED_PARAMETER( pxOutputProcessParameters );
	UNREFERENCED_PARAMETER( IsEnabled );

	// Save if the params have changed (cant call ParametersChanged() after BeginProcess() )
	m_bParamsChanged = ( ParametersChanged() != 0 );

	ParameterClass* pParams;
	pParams = (ParameterClass*)BeginProcess();
	if( pxInputProcessParameters[0].BufferFlags & XAPO_BUFFER_VALID )
	{
		DoProcess( ( FLOAT32* __restrict )pxInputProcessParameters[0].pBuffer, pxInputProcessParameters[0].ValidFrameCount, m_xWaveFormat.nChannels,*pParams );
	}
	EndProcess();
}

#endif	// defined ASURA_USE_AUDIO_SYSTEM

#endif	// !defined __ASURA_XAUDIO2_XAPO_FX_H__

//eof
