// Asura_XAPO_DSP_AddNoise.cpp
// Thomas Young - 26.03.09
// Copyright (C) 2009 Rebellion, All Rights Reserved.

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

#ifdef ASURA_USE_AUDIO_SYSTEM

#include "Sound/XAPO_DSP/Asura_XAPO_DSP_AddNoise.h"
#include "Sound/Asura_Audio_DSP.h"

// F U N C T I O N   D E F I N I T I O N S ///////////////////////////////////

Asura_XAPO_DSP_AddNoise::Asura_XAPO_DSP_AddNoise( ) : Asura_XAPOBase< Asura_XAPO_DSP_AddNoise, Asura_DSP_Params_AddNoise::Params >( )
{
	Asura_Mem::Set( &m_pfStatic[0], 0, sizeof( float ) * 480 );
}


Asura_XAPO_DSP_AddNoise::~Asura_XAPO_DSP_AddNoise()
{

}


void Asura_XAPO_DSP_AddNoise::DoProcess ( float* pfInputBuffer, u_int uFrames, u_int uChannels, const Asura_DSP_Params_AddNoise::Params& xParams )
{
	for( u_int uChannel = 0; uChannel < uChannels; ++uChannel )
	{
		for( u_int uSample = 0; uSample < uFrames; ++uSample )
		{
			const u_int uSamplePos = uSample * uChannels + uChannel;

			pfInputBuffer[ uSamplePos ] = pfInputBuffer[ uSamplePos ] * m_pfStatic[ uSample ];
		}
	}
	
}


void Asura_XAPO_DSP_AddNoise::UpdateParameters( const Asura_DSP_Params_AddNoise::Params& xParams )
{
	// Make some pink-ish noise
	const float fRecip = 1.0f / static_cast<float>( 480.0f );
	for( float fFreq = 2.0f; fFreq < 512.0f; fFreq *= 1.5f )
	{
		float fPhase = Asura_Random::GetFloat( 0.0f, 6.2831f );
		float fContribution = 1.0f - ( 1.0f / 512.0f * fFreq );
		for( u_int uSample = 0; uSample < 480; ++ uSample )
		{
			m_pfStatic[ uSample ] += Asura_Maths::Sine( fFreq * fRecip * static_cast<float>( uSample ) * 6.28318f + fPhase ) * fContribution;
			if( m_pfStatic[ uSample ] > 1.0f )
			{
				m_pfStatic[ uSample ] = 1.0f;
			}
			if( m_pfStatic[ uSample ] < -1.0f )
			{
				m_pfStatic[ uSample ] = -1.0f;
			}
		}
	}

	// Move the noise towards 1 to 'reduce' it when multiplying by the original signal
	for( u_int uSample = 0; uSample < 480; ++ uSample )
	{
		m_pfStatic[ uSample ] = 1.0f - ( 1.0f - m_pfStatic[ uSample ] ) * xParams.m_fNoiseAmount;
	}
}

#endif

//eof
