//////////////////////////////////////////////////////////////////////////////
//
// Asura_BillboardPlant_Batch.h
//
// Copyright (c) 2002 Rebellion. All rights reserved.
//
// Author: Andrew Weinkove - 09/12/02
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef ___ASURA_BILLBOARDPLANT_BATCH_H___
#define ___ASURA_BILLBOARDPLANT_BATCH_H___

#include "Core/Asura_Maths.h"
#include "Core/Asura_Handle.h"
#include "Rendering/Asura_Colour.h"
#ifdef ASURA_USE_WIND_SYSTEM
#include "SpecialFX/Asura_Wind_System.h"
#endif //#ifdef ASURA_USE_WIND_SYSTEM

class Asura_BillboardPlant
{
public:
#ifdef ASURA_USE_WIND_SYSTEM
	void Update(const float fTimeslice, const Asura_Wind_Data& xWindData, const float fStiffness, const float fMaxDistortion);
#else
	void Update(const float fTimeslice, const Asura_Vector_3& xPush, const float fStiffness, const float fMaxDistortion);
#endif //ASURA_USE_WIND_SYSTEM
	Asura_Vector_3 m_xPosition;
	Asura_Vector_2 m_xOffset;
	Asura_Vector_2 m_xVelocities;
	Asura_Vector_2 m_xScales;
};

// This class represents a bunch of bill-boarded plants.
// For speed, plants are collected into groups of the same type that
// exist in the same module. This allows fast culling for rendering or updates
// and efficient rendering because textures are the same and quads can be batched up.
// It also results in less memory fragmentation and less cache thrashing when iterating
// on entities.
class Asura_BillboardPlant_Batch
{
	friend class Tools_Mode_BillboardPlants;
	friend class Tools_Level_Export;
	friend class Tools_SaveAsura;
public:
	/* Constructors/destructor. */
	// Constructor from tools
	Asura_BillboardPlant_Batch(const unsigned uPlantType,
									const unsigned uNumberOfPlants,
									const Asura_Vector_3 *pxPositions,
									const int iModuleNumber);
	// Constructor from load.
	Asura_BillboardPlant_Batch(Asura_Handle fileHandle);
	virtual ~Asura_BillboardPlant_Batch();

	/* Public interface functions. */
	void Update(void);
	void Render(void);
	void FillVertices(Asura_Vector_3 *pxVertices, const int iPlantIndex);
	void Platform_Render(unsigned uCamVisFlags);
	int GetChunkBufferSize();
	void FillInChunkBuffer(char*& pcChunkBuffer);
	void SetupPlantData();

private:
#ifndef ASURA_USE_WIND_SYSTEM
	void Project_GetPlantPush(Asura_Vector_3& xPush) const;
#endif //ASURA_USE_WIND_SYSTEM
	void UpdateClipping();

	/* Data. */
	Asura_BillboardPlant *m_pxPlants;
	Asura_Bounding_Box m_xBB;
	Asura_Vector_3 m_xCentre;
	float m_fRadius;
	unsigned m_uNumberOfPlants;
	unsigned m_uPlantType;
	int m_iModuleNumber;
	int m_iPlantIndex;
	unsigned m_uCamVisFlags;
	Asura_RGBA m_xLighting;
	static const float s_fSizeVariation;
};

#endif // ___ASURA_BILLBOARDPLANT_BATCH_H___
