//
// Asura_Fragment_System.cpp
//
// Created by Richard Rice - 29/11/2002
// 
// Copyright (C) 2002 Rebellion, All Rights Reserved.
//


#include "Core/Asura.h"

#include "SpecialFX/Asura_Fragment_System.h"

#include "Core/Asura_HashTree.h"
#include "Core/Memory/Asura_ResourceAllocator.h"
#include "Debugging/Asura_Assert.h"



////////////////////////////////////////////////////////////////////////////////////////////
/*-----------------------**
** Asura_Fragment_System **
**-----------------------*/

Asura_HashTree* Asura_Fragment_System::s_pxFragmentTree = 0;
int Asura_Fragment_System::s_iNumberOfFragmentTypes = 0;
Asura_FragmentType* Asura_Fragment_System::s_apxFragmentTypes[ASURA_MAX_FRAGMENT_TYPES];

void Asura_Fragment_System::Initialise()
{
	s_iNumberOfFragmentTypes = 0;
	s_pxFragmentTree = new Asura_HashTree;
}

void Asura_Fragment_System::Shutdown()
{
	for(int i=0;i<s_iNumberOfFragmentTypes;i++)
	{
		delete s_apxFragmentTypes[i];
	}
	delete s_pxFragmentTree;

	s_iNumberOfFragmentTypes = 0;
	s_pxFragmentTree = 0;
}

void Asura_Fragment_System::Reset()
{
//	Shutdown();
//	Initialise();
	for(int i=0;i<s_iNumberOfFragmentTypes;i++)
	{
		delete s_apxFragmentTypes[i];
        s_apxFragmentTypes[i] = NULL;
	}
    s_iNumberOfFragmentTypes = 0;
    s_pxFragmentTree->Reset();
}

bool Asura_Fragment_System::NewFragmentType(Asura_Hash_ID uHash,Asura_FragmentType*& pxFragmentType)
{
	Asura_Assert(s_pxFragmentTree,"Not Initialised");

	int iIndex = 0;
	if(!s_pxFragmentTree->FindNodeIndexByHash(uHash,true,iIndex))
	{
        Asura_Assert(iIndex < ASURA_MAX_FRAGMENT_TYPES, "Maximum number of fragment types reached");

		s_iNumberOfFragmentTypes++;
		pxFragmentType = s_apxFragmentTypes[iIndex] = new Asura_FragmentType(uHash);
		return true;
		
	}
	else
	{
		pxFragmentType = s_apxFragmentTypes[iIndex];
		return false;
	}
}

Asura_FragmentType* Asura_Fragment_System::GetFragmentType(Asura_Hash_ID uHash)
{
	Asura_Assert(s_pxFragmentTree,"Not Initialised");

	int iIndex = -1;
	if(s_pxFragmentTree->FindNodeIndexByHash(uHash,false,iIndex))
	{
		return s_apxFragmentTypes[iIndex];
	}

	return 0;
}

Asura_FragmentType* Asura_Fragment_System::GetFragmentTypeByIndex(int iIndex)
{
	Asura_Assert(iIndex>=0 && iIndex<s_iNumberOfFragmentTypes,"");
	return s_apxFragmentTypes[iIndex];
}



////////////////////////////////////////////////////////////////////////////////////////////
/*--------------------**
** Asura_FragmentType **
**--------------------*/


Asura_FragmentType::Asura_FragmentType(Asura_Hash_ID uHash)
{
	m_uFragmentHash = uHash;
	m_iNumberOfFragments = 0;
	m_pxFragments = 0;
	m_bAnimatedFragment = false;
}

Asura_FragmentType::~Asura_FragmentType()
{
	Asura_ResourcePushType(ASURA_RESOURCE_FRAGMENT);
	{
		Asura_ResourceDeallocateArray(m_pxFragments);
	}
	Asura_ResourcePopType();
}



////////////////////////////////////////////////////////////////////////////////////////////
/*----------------**
** Asura_Fragment **
**----------------*/

Asura_Fragment::Asura_Fragment()
{
	m_uShapeHash = ASURA_HASH_ID_UNSET;
	m_uFlags = 0;
	m_iQuantity = 1;
}
