//
// Asura_Glow_System.cpp
//
// Created by RJF - 14 September 2004
//
// Copyright (C) 2004 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

// This file's header.
#include "SpecialFX/Asura_Glow_System.h"

#include "Camera/Asura_Camera.h"


/*******************************************************************************/
/* Static class members                                                        */
/*******************************************************************************/

float Asura_Glow_System::s_afColorParams[4] = { 0.5f, 0.5f, 0.5f, 0.605f };
float Asura_Glow_System::s_afCameraLevel[ASURA_MAX_NO_OF_CAMERAS] = { 1.0f, 1.0f, 1.0f, 1.0f };

/*******************************************************************************/
/* Asura_Glow_System class implementation                                      */
/*******************************************************************************/

void Asura_Glow_System::Initialise()
{
	Platform_Initialise();
}

void Asura_Glow_System::Reset()
{
	// set data that might be changed mid-level back to defaults
	for (u_int u = 0; u < ASURA_MAX_NO_OF_CAMERAS; ++u)
	{
		s_afCameraLevel[u] = 1.0f;
	}
}

void Asura_Glow_System::Shutdown()
{
	Platform_Shutdown();
}

void Asura_Glow_System::PreReset()
{
	Platform_PreReset();
}

void Asura_Glow_System::PostReset()
{
	Platform_PostReset();
}

void Asura_Glow_System::Render()
{
	// VGP 09Mar2005 Glow isn't free! Don't do it unless we have to.
	if (s_afColorParams[3] < 0.0001f)
		return;

	Platform_Render(s_afColorParams[0],
					s_afColorParams[1],
					s_afColorParams[2],
					s_afColorParams[3]);
}

void Asura_Glow_System::SetGlowTint(float fRed, float fGreen, float fBlue)
{
	s_afColorParams[0] = fRed;
	s_afColorParams[1] = fGreen;
	s_afColorParams[2] = fBlue;
}

void Asura_Glow_System::SetGlowLevel(float fLevel)
{
	s_afColorParams[3] = fLevel;
}

void Asura_Glow_System::SetGlowTint(Asura_RGBA values)
{
	u_int u = values.ToUIntARGB(false);
	s_afColorParams[0] = ((u >> 16) & 0x000000ff) / 255.0f;
	s_afColorParams[1] = ((u >> 8) & 0x000000ff) / 255.0f;
	s_afColorParams[2] = ((u     ) & 0x000000ff) / 255.0f;
}

Asura_RGBA Asura_Glow_System::GetGlowTint()
{
	return Asura_RGBA(s_afColorParams[0], s_afColorParams[1], s_afColorParams[2], 1.0f);
}

float Asura_Glow_System::GetGlowLevel()
{
	return s_afColorParams[3];
}

// (MH - 19.10.2004) added per-camera setting of glow level. 
// No glow will be done for level 0.0f and overall glow level done for 1.0f.
void Asura_Glow_System::SetCameraGlowLevel( float fLevel, u_int uCameraIndex )
{
	Asura_Assert(uCameraIndex < ASURA_MAX_NO_OF_CAMERAS, "invalid camera index");
	if (uCameraIndex < ASURA_MAX_NO_OF_CAMERAS)
	{
		s_afCameraLevel[uCameraIndex] = fLevel;
	}
}

float Asura_Glow_System::GetCameraGlowLevel( u_int uCameraIndex )
{
	Asura_Assert(uCameraIndex < ASURA_MAX_NO_OF_CAMERAS, "invalid camera index");
	if (uCameraIndex < ASURA_MAX_NO_OF_CAMERAS)
	{
		return s_afCameraLevel[uCameraIndex];
	}
	return 0.0f;
}

