//
// Asura_PerlinNoise_System.h
//
// Created by KOR - 16/12/2004
//
// A fairly cheap pseudorandom continuous noise function, returning a floating point value between -1 and 1,
// given a point in 1 to 3 dimensional space. See "Texturing and Modelling" by Ebert et al., Chapter 9.
//
// To get a 3-vector from a 3-vector, you can use:
// X' = GetNoise(Asura_Vector_2(Y, Z))
// Y' = GetNoise(Asura_Vector_2(Z, X))
// Z' = GetNoise(Asura_Vector_2(X, Y))
// Or for a moving version:
// X' = GetNoise(Asura_Vector_3(Y, Z, time))
// Y' = GetNoise(Asura_Vector_3(Z, X, time))
// Z' = GetNoise(Asura_Vector_3(X, Y, time))
// It would be possible to add a slightly optimised version of this to the system.
//
// You can also get the gradient of the noise function; this comes in useful for recalculating normals if used for spacewarping.
//
// It repeats every 16 units at the moment; if the pattern becomes too obvious in your application, it can be increased
//
// Copyright (C) 2004 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif

#ifndef __ASURA_PERLINNOISE_SYSTEM_H__
#define __ASURA_PERLINNOISE_SYSTEM_H__

#include "Core/Asura_Maths.h"

/// DEFINES ///////////////////////////////////////////////////////////////////////////////////////

#define ASURA_PERLINNOISE_REPEAT_DISTANCE 0x10
#define ASURA_PERLINNOISE_BITMASK (ASURA_PERLINNOISE_REPEAT_DISTANCE - 1)
#define ASURA_PERLINNOISE_OFFSET 0x1000

/// CLASSES ///////////////////////////////////////////////////////////////////////////////////////

class Asura_PerlinNoise_System
{
public:
	static void Initialise();

	static float GetNoise(float fValue, float* pfGradient = NULL);
	static inline float GetNoise2D(const float* afVec, Asura_Vector_2* pxGradient = NULL);
	static inline float GetNoise3D(const float* afVec, Asura_Vector_2* pxGradient = NULL);

	static void CalculateNoiseVector(const Asura_Vector_3& xPosition, Asura_Vector_3& xVector);
	static void CalculateNoiseVector(const Asura_AVector3& xPosition, Asura_AVector3& xVector);
	static void CalculateNoiseVector(const Asura_Vector_3& xPosition, const float fTime, Asura_Vector_3& xVector);
	static void CalculateNoiseVector(const Asura_AVector3& xPosition, const float fTime, Asura_AVector3& xVector);


private:
	static int				s_aiPermutation[ASURA_PERLINNOISE_REPEAT_DISTANCE * 2 + 2];
	static Asura_Vector_3	s_axGradients[ASURA_PERLINNOISE_REPEAT_DISTANCE * 2 + 2];
	static bool				s_bNoiseInitialised;

};

#endif//__ASURA_PERLINNOISE_SYSTEM_H__
