/**
*
* \file Asura_Shard_Effect.h
*
* Created by Andrew Weinkove - 03/12/02
*
* Copyright (C) 2002 Rebellion, All Rights Reserved.
*
**/

#ifndef __ASURA_SHARD_EFFECT_H__
#define __ASURA_SHARD_EFFECT_H__

#include "Core/Asura_Handle.h"
#include "Core/Asura_Maths.h"

enum SHARD_EFFECT_FLAGS
{
	SHARD_EFFECT_FLAGS_READY_TO_BE_DESTROYED = 1,
	SHARD_EFFECT_FLAGS_USE_ENVIRONMENT_MAP	 = 2,
};

static const int MAX_SHARDS_PER_EFFECT = 64;

class Asura_Shard;

class Asura_Shard_Effect
{
	friend class Asura_Shard_System;
	public:
		Asura_Shard_Effect();
		~Asura_Shard_Effect();
		void Initialise(Asura_Shard **ppxShards, const int iNumberOfShards, const float fAlphaOutTime, const unsigned uFlags, const unsigned uColour, const int iTextureIndex);
	private:
		
		inline void SetAsDestroyed(void)
		{
			m_uFlags |= SHARD_EFFECT_FLAGS_READY_TO_BE_DESTROYED;
		}

		inline bool IsReadyToBeDestroyed(void) const
		{
			return ((m_uFlags & SHARD_EFFECT_FLAGS_READY_TO_BE_DESTROYED) != 0);
		}
		inline bool IsUsingEnvironmentMap(void) const
		{
			return ((m_uFlags & SHARD_EFFECT_FLAGS_USE_ENVIRONMENT_MAP) != 0);
		}

		void Update(void);
		void Render(void);
		void Platform_Render(unsigned uCamVisFlags);

		unsigned m_uFlags;
		Asura_Bounding_Box m_xBB;
		float m_fAlphaOutTime;
		float m_fTimeSinceLastUpdate;
		Asura_Shard *m_apxShards[MAX_SHARDS_PER_EFFECT];
		int m_iNumberOfActiveShards;
		unsigned m_uColour;
		int m_iTextureIndex;
		float m_fAlpha;
		Asura_Handle m_iHandle;

};

#endif//__ASURA_SHARD_EFFECT_H__
