//
// Asura_SpecialFX.cpp
//
// Created by Kevin Lea - 03/10/2001 11:18:42
// 
// Copyright (C) 2001 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

#include "SpecialFX/Asura_SpecialFX.h"

#include "Affectors/Asura_Affector_System.h"
#include "Animation/Asura_TextureAnimation.h"
#include "Core/Asura_Timers.h"
#include "Cutscene/Asura_Cutscene_System.h"
#include "Debugging/Asura_CPUProfile.h"
#include "Debugging/Asura_Debug_Timers.h"
#include "FSFX/Asura_FSFX_System.h"
#include "Jobs/Asura_Job_PFX.h"
#include "Projectors/Asura_Projector_System.h"
#include "Rendering/Asura_DeferredShading.h"
#include "Rendering/Asura_Fog.h"
#include "Rendering/Asura_Render.h"
#ifdef ASURA_USE_WATER
	#include "Rendering/Asura_Water.h"
#endif
#include "Shadows/Asura_CSM.h"
#include "Sound/Asura_Dialogue_System.h"
#ifdef ASURA_USE_3DDECAL_SYSTEM
	#include "SpecialFX/Asura_3dDecal_System.h"
#endif
#include "SpecialFX/Asura_Beam_System.h"
#include "SpecialFX/Asura_BillboardPlant_System.h"
#include "SpecialFX/Asura_BillboardPlant_TemplateStore.h"
#include "SpecialFX/Asura_Corona_System.h"
#include "SpecialFX/Asura_Decal_Manager.h"
#include "SpecialFX/Asura_Decal_System.h"
#include "SpecialFX/Asura_FullScreenSFX_System.h"
#include "SpecialFX/Asura_LightShaft_System.h"
#ifdef ASURA_USE_SHADOWDECALS
	#include "SpecialFX/Asura_ShadowDecal_System.h"
#endif
#include "SpecialFX/Asura_Shard_System.h"
#include "SpecialFX/Asura_Skybox.h"
#ifdef ASURA_SUPPORT_WEATHER_SYSTEM
	#include "SpecialFX/Asura_Weather_System.h"
#endif
#include "SpecialFX/Asura_PerlinNoise_System.h"
#include "SpecialFX/Asura_SimplexNoise_System.h"
#include "SpecialFX/EffectVolume/Asura_EffectVolume_System.h"
#include "SpecialFX/PFX/Asura_PFX_System.h"

#ifdef ASURA_TOOLS
	#include "FSFX/Tools_FSFX_System.h"
#endif

// Kevin Lea - 03/10/2001 11:26:13 - the sphere code will need updating at some point
extern void Generate_Sphere(void); 


void Asura_SpecialFX_System::Initialise(void)
{
	ASURA_CPU_PROFILE_SCOPED_TIMER( "Asura_SpecialFX_System::Initialise");

    // (MP - 25.04.2002) I think this should be called first because it does
    // not require anything else, whereas some platform's particle and skybox
    // code may need it.
	Generate_Sphere();

	Asura_Shard_System::Initialise();
	Asura_Decal_System::Initialise();

#ifdef ASURA_USE_3DDECAL_SYSTEM
	Asura_3dDecal_System::Initialise();
#endif
	Asura_Corona_System::Initialise();
	Asura_Skybox::Initialise();
#ifdef ASURA_SUPPORT_WEATHER_SYSTEM
	Asura_Weather_System::Initialise();
#endif
	Asura_PFX_System::Initialise();
	Asura_PerlinNoise_System::Initialise();
	Asura_SimplexNoise_System::Initialise();

	Asura_Beam_System::Initialise();
    Asura_FullScreenSFX_System::Initialise();

#ifdef ASURA_USE_WATER
	Asura_Water_System::Initialise();
#endif

#ifdef ASURA_USE_SHADOWDECALS
	Asura_ShadowDecal_System::Initialise();
#endif

#ifdef ASURA_SUPPORT_MULTITHREADED_JOB_QUEUE
	Asura_Job_PFX_Update::InitialiseAllocations();
#endif

#if defined(ASURA_USE_AFFECTORS)
	Asura_Affector_System::Initialise();
#endif	// defined(ASURA_USE_AFFECTORS)

#ifdef ASURA_USE_DECAL_MANAGER
	Asura_Decal_Manager::Initialise();
#endif

#ifdef ASURA_USE_CSM
	Asura_CSM::Initialise();
#endif

	Asura_EffectVolume_System::Initialise();

	Platform_Initialise();

	Project_Initialise();
}

void Asura_SpecialFX_System::Shutdown(void)
{
	Project_Shutdown();

	Platform_Shutdown();

#ifdef ASURA_USE_CSM
	Asura_CSM::Shutdown();
#endif

#ifdef ASURA_USE_DECAL_MANAGER
	Asura_Decal_Manager::Shutdown();
#endif

#if defined(ASURA_USE_AFFECTORS)
	Asura_Affector_System::Shutdown();
#endif	// defined(ASURA_USE_AFFECTORS)

#ifdef ASURA_SUPPORT_MULTITHREADED_JOB_QUEUE
	Asura_Job_PFX_Update::FreeAllocations();
#endif

#ifdef ASURA_USE_WATER
	Asura_Water_System::Shutdown();
#endif

	Asura_FullScreenSFX_System::Shutdown();
    Asura_Beam_System::Shutdown();
#ifdef ASURA_SUPPORT_WEATHER_SYSTEM
	Asura_Weather_System::Shutdown();
#endif
	Asura_Skybox::Shutdown();
	Asura_Corona_System::Shutdown();
	Asura_Decal_System::Shutdown();
#ifdef ASURA_USE_3DDECAL_SYSTEM
	Asura_3dDecal_System::Shutdown();
#endif
	Asura_Shard_System::Shutdown();

	Asura_PFX_System::Shutdown();

#ifdef ASURA_USE_SHADOWDECALS
	Asura_ShadowDecal_System::Shutdown();
#endif

	Asura_EffectVolume_System::Shutdown();
}

void Asura_SpecialFX_System::Reset(void)
{
	Project_Reset();

	Platform_Reset();

#ifdef ASURA_USE_CSM
	Asura_CSM::Reset();
#endif

#ifdef ASURA_USE_DECAL_MANAGER
	Asura_Decal_Manager::Reset();
#endif

#if defined(ASURA_USE_AFFECTORS)
	Asura_Affector_System::Reset();
#endif	// defined(ASURA_USE_AFFECTORS)

#ifdef ASURA_SUPPORT_MULTITHREADED_JOB_QUEUE
	Asura_Job_PFX_Update::ResetAllocations();
#endif

	Asura_FullScreenSFX_System::Reset();
    Asura_Beam_System::Reset();
#ifdef ASURA_SUPPORT_WEATHER_SYSTEM
	Asura_Weather_System::Reset();
#endif
	Asura_Skybox::Reset();
	Asura_Corona_System::Reset();
	Asura_Decal_System::Reset();
#ifdef ASURA_USE_3DDECAL_SYSTEM
	Asura_3dDecal_System::Reset();
#endif
	Asura_Shard_System::Reset();

	Asura_PFX_System::Reset();

#ifdef ASURA_USE_WATER
	Asura_Water_System::Reset();
#endif

#ifdef ASURA_USE_SHADOWDECALS
	Asura_ShadowDecal_System::Reset();
#endif

	Asura_EffectVolume_System::Reset();
}

void Asura_SpecialFX_System::Update(void)
{
	ASURA_CPU_PROFILE__START_LABEL(ASURA_CPU_PROFILE__SPECIALFX_UPDATE, "SpecialFX Update");

	Asura_Shard_System::Update();

	#ifdef ASURA_SUPPORT_WEATHER_SYSTEM
	Asura_Weather_System::Update();
	#endif
	#ifdef ASURA_USE_BILLBOARD_PLANTS
	Asura_BillboardPlant_System::Update();
	#endif //ASURA_USE_BILLBOARD_PLANTS

	// BM- 17.03.2003 Added laser beam system
	Asura_Beam_System::Update();
#ifdef ASURA_USE_3DDECAL_SYSTEM
	Asura_3dDecal_System::Update();
#endif

	Asura_PFX_System::Update();

    Asura_FullScreenSFX_System::Update();

#ifdef ASURA_USE_WATER
	Asura_Water_System::Update(Asura_Timers::GetGameTimeSlice());
#endif

#ifdef ASURA_USE_FSFX
	Asura_FSFX_System::Update();
#endif

#if defined(ASURA_USE_AFFECTORS)
	Asura_Affector_System::Update();
#endif	// defined(ASURA_USE_AFFECTORS)

#ifdef ASURA_TARGET_SM3
	Asura_Projector_System::Update();
#endif

	//Update the global texture animation timers. RAR 31/08/2007
	Asura_TextureAnimation_System::Update();

#ifdef ASURA_USE_CSM
	Asura_CSM::Update();
#endif

	Asura_EffectVolume_System::Update();

	Platform_Update();

	Project_Update();

	ASURA_CPU_PROFILE__STOP(ASURA_CPU_PROFILE__SPECIALFX_UPDATE);

}

void Asura_SpecialFX_System::RenderDecals()
{
	ASURA_CPU_PROFILE__START_LABEL(ASURA_CPU_PROFILE__SPECIALFX_RENDER, "SpecialFX Render");
	Platform_BeginRender(RT_DECALS);
	{
		Asura_Decal_System::Render();
	}
	Platform_EndRender(RT_DECALS);
	ASURA_CPU_PROFILE__STOP(ASURA_CPU_PROFILE__SPECIALFX_RENDER);
}

void Asura_SpecialFX_System::RenderSkyboxEffects()
{
	ASURA_CPU_PROFILE__START_LABEL(ASURA_CPU_PROFILE__SPECIALFX_RENDER, "SpecialFX Render");
	Platform_BeginRender(RT_SKYBOX_EFFECTS);
	{
		if( Asura_Skybox::IsSkyboxVisible() )
		{
			Asura_PFX_System::Render(ASURA_PFX_RENDER_SKYBOX);
			Asura_PFX_System::RenderHeatHaze();
		}
	}
	Platform_EndRender(RT_SKYBOX_EFFECTS);
	ASURA_CPU_PROFILE__STOP(ASURA_CPU_PROFILE__SPECIALFX_RENDER);
}

void Asura_SpecialFX_System::RenderProjectEffects()
{
	ASURA_CPU_PROFILE__START_LABEL(ASURA_CPU_PROFILE__SPECIALFX_RENDER, "SpecialFX Render");
	Platform_BeginRender(RT_PROJECT_EFFECTS);
	{
		Asura_PFX_System::Render(ASURA_PFX_RENDER_PROJECT);
	}
	Platform_EndRender(RT_PROJECT_EFFECTS);
	ASURA_CPU_PROFILE__STOP(ASURA_CPU_PROFILE__SPECIALFX_RENDER);
}

#ifdef ASURA_USE_3DDECAL_SYSTEM
void Asura_SpecialFX_System::Render3dDecalsPreWorld()
{
	ASURA_CPU_PROFILE__START_LABEL(ASURA_CPU_PROFILE__SPECIALFX_RENDER, "SpecialFX Render");
	Platform_BeginRender(RT_3D_DECALS_PRE_WORLD);
	{
		Asura_3dDecal_System::EarlyRender();
	}
	Platform_EndRender(RT_3D_DECALS_PRE_WORLD);
	ASURA_CPU_PROFILE__STOP(ASURA_CPU_PROFILE__SPECIALFX_RENDER);
}

void Asura_SpecialFX_System::Render3dDecalsPostWorld()
{
	ASURA_CPU_PROFILE__START_LABEL(ASURA_CPU_PROFILE__SPECIALFX_RENDER, "SpecialFX Render");
	Platform_BeginRender(RT_3D_DECALS_POST_WORLD);
	{
		Asura_3dDecal_System::LateRender();
	}
	Platform_EndRender(RT_3D_DECALS_POST_WORLD);	
	ASURA_CPU_PROFILE__STOP(ASURA_CPU_PROFILE__SPECIALFX_RENDER);
}
#endif



void Asura_SpecialFX_System::FinishAllJobs(void)
{
#ifdef ASURA_SUPPORT_MULTITHREADED_JOB_QUEUE
	ASURA_CPU_PROFILE__START_LABEL(ASURA_CPU_PROFILE__SPECIALFX_FINISHALLJOBS, "SpecialFX FinishAllJobs");
	Asura_Job_PFX_Update::FinishAllJobs();
	ASURA_CPU_PROFILE__STOP(ASURA_CPU_PROFILE__SPECIALFX_FINISHALLJOBS);	
#endif
}


void Asura_SpecialFX_System::Render(void)
{

	Asura_SpecialFX_System::FinishAllJobs();

	ASURA_CPU_PROFILE__START_LABEL(ASURA_CPU_PROFILE__SPECIALFX_RENDER, "SpecialFX Render");
	Platform_BeginRender(RT_MAIN);
	{
		// KJFL - 15 March 2005 - ShadowDecals need to be drawn nice and early (e.g. before PFX) but after
		// environment and entities
#ifdef ASURA_USE_SHADOWDECALS
		Asura_ShadowDecal_System::Render();
#endif

#ifdef ASURA_USE_BILLBOARD_PLANTS
		Asura_BillboardPlant_System::Render();
#endif //ASURA_USE_BILLBOARD_PLANTS

		Project_Render();

#ifndef ASURA_TARGET_SM3
		Asura_Fog::Render();
#endif

		Asura_Shard_System::Render();

#ifdef ASURA_USE_WATER
		//(RAR 06/01/2005) No attention has been paid to ordering problems with where water should be rendered , 
		//so this may need to be moved.
		Asura_Water_System::Render();
#endif

		Asura_Corona_System::Render();

		// BM- 17.03.2003 Added laser beam system
		Asura_Beam_System::Render();

		Platform_Render();

#ifdef ASURA_TARGET_SM3
		Asura_LightShaft_System::Render();
#endif

		Asura_PFX_System::Render(ASURA_PFX_RENDER_NORMAL);
		Asura_PFX_System::RenderHeatHaze();	

		Project_RenderLate();

#ifdef ASURA_USE_HDR
		Asura_Render::ApplyToneMapping();

		Project_RenderPostToneMap();
#endif

		
#ifdef ASURA_EDGE_ANTIALIASING
		Asura_Render::RenderEdgeAntiAliasing();
#endif
		

#if defined( ASURA_USE_FSFX)
		Asura_FSFX_System::Render();
#ifdef ASURA_TOOLS
		Tools_FSFX_System::Render();
#endif
		// Need to make sure any FSFX delayed quads are rendered
		Asura_Render::FlushDisplay();
		Asura_Render::CurrentStates.RequestStencilEnabled(false);
#endif

		Asura_FullScreenSFX_System::Render_Early();

		// RJF - 15 August 2005 - I think this needs to be moved to before Project_RenderLate...
		// having it after the DoF/FullScreenSFX is almost certainly wrong
#ifdef ASURA_SUPPORT_WEATHER_SYSTEM
		Asura_Weather_System::Render();
#endif

		Asura_EffectVolume_System::Render();

		Asura_FullScreenSFX_System::Render_Late();
	}
	Platform_EndRender(RT_MAIN);
	ASURA_CPU_PROFILE__STOP(ASURA_CPU_PROFILE__SPECIALFX_RENDER);
}

void Asura_SpecialFX_System::RenderHUDEffects(void)
{
	ASURA_CPU_PROFILE__START_LABEL(ASURA_CPU_PROFILE__SPECIALFX_RENDER, "SpecialFX Render");
	Platform_BeginRender(RT_HUD_EFFECTS);
	{
		Asura_PFX_System::Render(ASURA_PFX_RENDER_HUD);
		Asura_PFX_System::RenderHeatHaze();
	}
	Platform_EndRender(RT_HUD_EFFECTS);
	ASURA_CPU_PROFILE__STOP(ASURA_CPU_PROFILE__SPECIALFX_RENDER);
}

void Asura_SpecialFX_System::Render2DEffects()
{
	ASURA_CPU_PROFILE__START_LABEL(ASURA_CPU_PROFILE__SPECIALFX_RENDER, "SpecialFX Render");
	Platform_BeginRender(RT_2D_EFFECTS);
	{
		// (AAM - 05.07.2004) These should be rendered last (and perhaps changed to reshape the viewport instead of
		// rendering the scene underneath them then blanking it?)
		// RAR 22/09/2006 Cutscene stuff moved out of the main Asura_SpecialFX_System::Render() so that it can
		// be done after rendering of all stuff in the world has been finished.

		Asura_FullScreenSFX_System::Render2DEffects();

		if (Asura_Cutscene_System::ShouldRenderWidescreenBars())
		{
			Asura_Cutscene_System::RenderWidescreenBars();
		}

		Asura_Cutscene_System::RenderSubtitles();

#ifdef ASURA_USE_DIALOGUE_SYSTEM
		Asura_Dialogue_System::RenderSubtitles();
#endif
	}
	Platform_EndRender(RT_2D_EFFECTS);
	ASURA_CPU_PROFILE__STOP(ASURA_CPU_PROFILE__SPECIALFX_RENDER);
}

#if defined(ASURA_USE_MULTIPLE_ZONE_HANDLERS)
// Create handlers for all special effect subsystems that need one
void Asura_SpecialFX_System::CreateSubsystemHandlers(const Asura_Hash_ID uHash)
{
	Asura_Corona_System::CreateHandler(uHash); // Static coronas aren't needed here, as they're initialised in Asura_Core anyway.
	Asura_PFX_System::CreateHandler(uHash);
	Asura_Decal_System::CreateHandler(uHash);
	// TODO - these will probably exist later
	//Asura_ShadowDecal_System::CreateHandler(uHash);
}

void Asura_SpecialFX_System::DestroySubsystemHandlers(const Asura_Hash_ID uHash)
{
	Asura_Corona_System::DestroyHandler(uHash);
	Asura_PFX_System::DestroyHandler(uHash);
	Asura_Decal_System::DestroyHandler(uHash);
	// TODO - these will probably exist later
	//Asura_ShadowDecal_System::DestroyHandler(uHash);
}

void Asura_SpecialFX_System::CreateAndSetSubsystemHandlers(const Asura_Hash_ID uHash)
{
	Asura_Corona_System::CreateAndSetHandler(uHash);
	Asura_PFX_System::CreateAndSetHandler(uHash);
	Asura_Decal_System::CreateAndSetHandler(uHash);
	// TODO - these will probably exist later
	//Asura_ShadowDecal_System::CreateAndSetHandler(uHash);
}

void Asura_SpecialFX_System::SetCurrentSubsystemHandlers(const Asura_Hash_ID uHash)
{
	Asura_Corona_System::SetCurrentHandler(uHash);
	Asura_PFX_System::SetCurrentHandler(uHash);
	Asura_Decal_System::SetCurrentHandler(uHash);
	// TODO - these will probably exist later
	//Asura_ShadowDecal_System::CreateAndSetHandler(uHash);
}
#endif //#f defined(ASURA_USE_MULTIPLE_ZONE_HANDLERS)
