//
// Asura_PFX_Entity_Effect.h
//
// Created by KOR - 15/07/2004
//
// Copyright (C) 2004 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif

#ifndef __ASURA_PFX_ENTITY_EFFECT_H__
#define __ASURA_PFX_ENTITY_EFFECT_H__

#include "Core/Asura_Handle.h"
#include "SpecialFX/PFX/Asura_PFX_Effect.h"
#include "Network/Asura_Guid.h"

class Asura_PFX_Effect_Template;
class Asura_PFX_Source;
class Asura_Photon;
class Asura_PFX_Volume;
template <typename T> class Asura_Spline;

/// Asura_PFX_Entity_Effect ///////////////////////////////////////////////////////////////////////
// An effect placed in tools
///////////////////////////////////////////////////////////////////////////////////////////////////

class Asura_PFX_Entity_Effect : public Asura_PFX_Effect
{
public:
	// Returns a handle to the created effect
	static unsigned int Create(const Asura_Vector_3& xPosition, const Asura_Matrix_3x3& xDirection, Asura_Hash_ID uTemplate, const Asura_PFX_Volume* pxVolumeOverride = 0, u_short usPFXFlags = ASURA_PFX_EFFECT_CAMERA_FLAGS, Asura_Guid uParentGuid = ASURA_GUID_UNREGISTERED);
	// Direction as vector instead of matrix
	static unsigned int Create(const Asura_Vector_3& xPosition, const Asura_Vector_3& xDirection, Asura_Hash_ID uTemplate, const Asura_PFX_Volume* pxVolumeOverride = 0, u_short usPFXFlags = ASURA_PFX_EFFECT_CAMERA_FLAGS, Asura_Guid uParentGuid = ASURA_GUID_UNREGISTERED);

	void Render();
	void Update();

	void Start();
	void Stop(bool bImmediate);
	//Setting bTeleport to true will  prevent the particles from being generated evenly across the path
	//that the generator has covered.
	virtual void SetPosition(const Asura_Vector_3& xPosition, const Asura_Matrix_3x3& xDirection,bool bTeleport = false);
	void SetPosition(const Asura_Vector_3& xPosition,bool bTeleport = false);
	const Asura_Vector_3& GetPosition() const { return m_xPosition; }
	void SetOrientation(const Asura_Vector_3& xDirection, int iOrientationID = 0);
	void SetOrientation(const Asura_Matrix_3x3& xOrientation, int iOrientationID = 0);
	// Warning - this function doesn't actually affect how the effect moves,
	// but changes the delta so that any particles created can use the velocity
	void SetVelocity(const Asura_Vector_3& xVelocity);
	// Set effect's age from 0 to 1.
	// Rewinding an effect may have strange results; this is mostly
	// intended for fast-forwarding looping effects so they don't sync up.
	void SetAge(float fAge);

	void SetInvisibleByAttachment( bool bInvisible );
#ifdef ASURA_TOOLS
	void UpdateNumberOfSources(u_int iNumberOfSources);
	void UpdateMaximumParticles(u_int uSourceNumber, u_int uMaximumParticles);

	//Gets the hash ID of the template being used (taking into account platform overrides).
	Asura_Hash_ID GetCurrentTemplateHash() const;
#endif
	void SetCollisionPlane(const Asura_Vector_3& xPosition, const Asura_Vector_3& xDirection)
	{
		m_xCollisionPosition = xPosition;
		m_xCollisionDirection = xDirection;
		m_bHasCollisionPlane = true;
	}
	bool GetCollisionPlane(Asura_Vector_3* pxPosition, Asura_Vector_3* pxDirection) const
	{
		Asura_Assert(pxPosition && pxDirection, "Null pointers given to GetCollisionPlane");
		if(pxPosition && pxDirection && m_bHasCollisionPlane)
		{
			*pxPosition = m_xCollisionPosition;
			*pxDirection = m_xCollisionDirection;
			return true;
		}
		return false;
	}
	bool GetGravityPosition(Asura_Vector_3* pxPosition) const
	{
		Asura_Assert(pxPosition, "Null pointer given to GetGravityPosition");
		if(pxPosition)
		{
			*pxPosition = m_xPosition;
			return true;
		}
		return false;
	}
	const Asura_PFX_Effect_Template* GetTemplate() const { return m_pxTemplate; }
	const Asura_Matrix_3x3* GetOrientation() const { return &m_xDirection; }
	void CalculateBoundingBox();
    void GenerateParticles(u_int uSourceNum, u_int uNumParticles);
    u_int GetCurNumParticles( void ) const;
	const class Asura_PFX_Source*	GetSource(u_int uSourceNum) const;
	void InitialisePhotons(u_int uMaxTotalPhotons = 0);
	void ShutdownPhotons();
	void UpdatePhotons();
	void UpdateSound(float fTimeSlice);
	void SetSpline(Asura_Spline<Asura_Vector_3>* pxSpline) { m_pxSpline = pxSpline; CalculateBoundingBox(); }

#if _DEBUG
    virtual void DumpStats( void );
#endif

#ifdef ASURA_TOOLS
	//Sets random number generator to use for cutscene recording purposes.	RAR 22/05/2006
	virtual void SetRNG(Asura_RandomGenerator* pxRNG);
#endif

protected:

	// Constructor and destructor
	Asura_PFX_Entity_Effect(const Asura_Vector_3& xPosition, const Asura_Matrix_3x3& xDirection, Asura_Hash_ID uTemplate, const Asura_PFX_Volume* pxVolumeOverride, u_short usPFXFlags, Asura_Guid uParentGuid);
	~Asura_PFX_Entity_Effect();

	Asura_PFX_Effect_Template*	m_pxTemplate; // Pointer to effect's template
    
private:
	// Internal helper for moving the effect. We always give a position, but the orientation is optional and hence a pointer.
	void SetPosAndOri(const Asura_Vector_3& xPosition, const Asura_Matrix_3x3* pxOri, bool bTeleport);
	void OnStart();
	void OnFinish();
	void Pregenerate();

	Asura_Vector_3				m_xPosition;
	Asura_Matrix_3x3			m_xDirection;
	// Collision plane attributes
	Asura_Vector_3				m_xCollisionPosition;
	Asura_Vector_3				m_xCollisionDirection;
	// Photon
	Asura_Photon**				m_ppxPhotons;
	u_int						m_uCurrentNumberOfPhotons;
	// Sound
	Asura_Handle				m_iSoundEventHandle;
	Asura_Spline<Asura_Vector_3>* m_pxSpline;

	float						m_fAge;
	Asura_Guid					m_uParentGuid;

	Asura_Bounding_Box			m_xRelativeStaticBB;//relative bounding box for when effect is stationary.
	Asura_Bounding_Box			m_xMovementBB;//bounding box tracking most recent movement
	Asura_Bounding_Box			m_xPrevMovementBB;//bounding box for tracking earlier movement

	float						m_fMaxParticleLifespan;//maximum lifespan of particles used by effect
	float						m_fMovementBBTimer;
	union
	{
		unsigned int			m_uFlags;
		struct
		{
			unsigned int		m_bSourceVolumeOverridden	:1;
			// (EJD - 26.10.2004) Added
			unsigned int		m_bHasCollisionPlane		:1;
			/// Richard May - 29/11/2004 11:04:27 - Added
			unsigned int		m_bPreserveWhileInactive	:1;
			//has moved recently , and currently tracking movement with the movement BBs
			unsigned int		m_bUsingMovementBBs			:1;
			unsigned int		m_bNeedToPregenerate		:1;
			unsigned int		m_bInvisibleByAttachment	:1;
			unsigned int		m_bWasLinkedIn				:1;
			unsigned int		m_bDonePhotonKillCheck		:1;
		};
	};
	// Source pointers
	Asura_PFX_Source**			m_ppxSources;

	#ifdef ASURA_TOOLS
	//Used for detecting changes to the template while editing
	unsigned int				m_bTemplateWasAlwaysOnTop:1;
	#endif

	#ifdef ASURA_USE_STREAMING
	//The current zone at the time that the effect was created. needed so that the sounds can be got from the correct place
	Asura_Hash_ID				m_uZoneHashID;
	#endif
};

#endif//__ASURA_PFX_ENTITY_EFFECT_H__
