
///////////////////////////////////////////////////////////////////////////////
//
// File   : Asura_Streaming_ZoneDataState.h
// Author : Andy Tate
// Date   : 6-Apr-2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

// RJT NOTE: Required for GCC builds as the file list generation doesn't account for VC project exclusion from build
#ifdef ASURA_USE_STREAMING

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __ASURA_STREAMING_ZONEDATASTATE_H__
#define __ASURA_STREAMING_ZONEDATASTATE_H__

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura_Hash.h"
#include "Core/Asura_String.h"


////////////////////////////////////////////////////////////////////////////////
// Forward class declarations...
////////////////////////////////////////////////////////////////////////////////

class Asura_Streaming_Zone;


////////////////////////////////////////////////////////////////////////////////
// Class declaration...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Class        : Asura_Streaming_ZoneDataState
// Author       : Andy Tate
// Date         : 6-Apr-2006
//
// Description  : Handles the decisions as to when this zone needs to have data
//				  loaded and when it's safe to be deleted.
////////////////////////////////////////////////////////////////////////////////

class Asura_Streaming_ZoneDataState
{
public:
	// Public functions...

	Asura_Streaming_ZoneDataState  (Asura_Streaming_Zone& xZone, const Asura_Hash_ID uHandlerHash, const char* szName, const char* szFilename, const bool bAllowEnvironment);
	Asura_Streaming_ZoneDataState  (Asura_Streaming_Zone& xZone, const Asura_Hash_ID uHandlerHash, const Asura_String& xName, const Asura_String& xFilename, const bool bAllowEnvironment);
	~Asura_Streaming_ZoneDataState ();

	// Sub-system handler functions...
	void							SetSubsystemHandlers		(void);

	// Manipulation functions...
	void							RequestLoad					(void);
	void							RequestDestruction			(void);

	bool							CommenceDataEmptyCreation	(const Asura_Hash_ID uParentHandlerHash, const bool bBlocking);
	bool							CommenceDataLoad			(const Asura_Hash_ID uParentHandlerHash, const bool bBlocking);
	bool							CommenceDataDestruction		(const bool bBlocking);

	void							MarkLoadCompletion			(void);
	void							MarkDestructionCompletion	(void);

	// Update functions...
	bool							Update						(void);

	// Accessor functions...
	Asura_Streaming_Zone&			GetZone						(void) const			{ return(m_xZone); }

	Asura_Hash_ID					GetHandlerHash				(void) const			{ return(m_uHandlerHash); }
	Asura_Hash_ID					GetParentHandlerHash		(void) const			{ return(m_uParentHandlerHash); }
	const Asura_String&				GetName						(void) const			{ return(m_xName); }
	const Asura_String&				GetFilename					(void) const			{ return(m_xFilename); }

	bool							IsDataIndexValid			(void) const			{ return(m_uDataIndex != static_cast<u_int>(-1)); }
	void							ResetDataIndex				(void)					{ m_uDataIndex = static_cast<u_int>(-1); }
	void							SetDataIndex				(const u_int uIndex)	{ m_uDataIndex = uIndex; }
	u_int							GetDataIndex				(void) const			{ return(m_uDataIndex); }

	bool							IsReadyForUse				(void) const			{ return(m_uParentHandlerHash != ASURA_HASH_ID_UNSET); }

	bool							IsTimeToLoad				(void) const			{ return(m_eState == TIME_TO_LOAD); }
	bool							IsFull						(void) const			{ return(m_eState == FULL); }
	bool							IsTimeToDestroy				(void) const			{ return(m_eState == TIME_TO_DESTROY); }
	bool							IsEmpty						(void) const			{ return(m_eState == EMPTY); }
	bool							IsUnused					(void) const			{ return(m_eState == UNUSED); }
	bool							IsNotPresent				(void) const			{ return(m_eState == NOT_PRESENT); }


protected:
	// Protected variables...


	// Protected functions...


private:
	// Private enumerated types...
	enum State
	{
		EMPTY,
		LOAD_PENDING,
		TIME_TO_LOAD,
		LOADING,
		FULL,
		DESTRUCTION_PENDING,
		TIME_TO_DESTROY,
		DESTROYING,

		UNUSED,
		NOT_PRESENT,

		MAX_STATES
	};

	// Private variables...
	Asura_Streaming_Zone&	m_xZone;				// The zone which contains this data-state.

	Asura_Hash_ID			m_uHandlerHash;			// The hash ID for this sub-system handler.
	Asura_Hash_ID			m_uParentHandlerHash;	// The hash ID used when hooking sub-system handlers together.
	Asura_String			m_xName;				// The name of the data for this zone.
	Asura_String			m_xFilename;			// The name of the file to load for this zone.
	u_int					m_uDataIndex;			// The data-index used for this zone.
	bool					m_bAllowEnvironment;	// Whether this zone has its own module system/environment/GI solution.

	bool					m_bChangeState;			// Flag to say whether a state change is pending.
	State					m_ePendingState;		// The state we're going to change to.
	State					m_eState;				// The current state for this part of the zone.
	float					m_fSecondsInState;		// The time in the current state.


	// Private functions...

	Asura_Streaming_ZoneDataState  (void);
	Asura_Streaming_ZoneDataState  (const Asura_Streaming_ZoneDataState& xOther);

	Asura_Streaming_ZoneDataState&	operator=				(const Asura_Streaming_ZoneDataState& xOther);

	// State functions...
	void							ChangeState				(const State eState);

	// Load functions...
	bool							LoadData				(const bool bBlocking);
	void							DestroyData				(const bool bBlocking);

};   // End of class 'Asura_Streaming_ZoneDataState'.


#endif   // ifndef __ASURA_STREAMING_ZONEDATASTATE_H__

#endif	// ASURA_USE_STREAMING

/// End of file. ///
