
///////////////////////////////////////////////////////////////////////////////
//
// File   : Asura_Streaming_ZoneEntities.h
// Author : Andy Tate
// Date   : 18-Apr-2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

// RJT NOTE: Required for GCC builds as the file list generation doesn't account for VC project exclusion from build
#ifdef ASURA_USE_STREAMING

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __ASURA_STREAMING_ZONEENTITIES_H__
#define __ASURA_STREAMING_ZONEENTITIES_H__

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura_Classified_List.h"
#include "Core/Asura_Combined_List.h"
#include "Core/Asura_HashedList.h"
#include "Core/Asura_LinkList.h"
#include "Entity/Asura_Entity.h"
#include "Entity/Asura_Entity_PeriodicClientUpdateList.h"


////////////////////////////////////////////////////////////////////////////////
// Forward class declarations...
////////////////////////////////////////////////////////////////////////////////

class Asura_Message;


////////////////////////////////////////////////////////////////////////////////
// Class declaration...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Class        : Asura_Streaming_ZoneEntities
// Author       : Andy Tate
// Date         : 18-Apr-2006
//
// Description  : Container class for entity-related operations for streaming-zones.
////////////////////////////////////////////////////////////////////////////////

class Asura_Streaming_ZoneEntities
{
public:
	// Public functions...
	Asura_Streaming_ZoneEntities  (void);
	~Asura_Streaming_ZoneEntities ();

	// Update/render functions...
	void	Update							(void);
	void	Render							(void);

	// Client update functions...
	void	ResetPeriodicClientUpdateList()	{ m_xPeriodicClientUpdateList.Reset(); }
	void	FinishPeriodicClientUpdates()	{ m_xPeriodicClientUpdateList.FinishClientUpdates(); }
	void	DoSomePeriodicClientUpdates()	{ m_xPeriodicClientUpdateList.DoSomeClientUpdates(); }

	void	ResetConstantClientUpdateList();
	void	DoConstantClientUpdates();

	// List functions...
	void	AddEntityToList					(Asura_Entity_Client& xEntity)											{ m_xClientEntityList.AddToList(&xEntity); }
	void	RemoveEntityFromList			(Asura_Entity_Client& xEntity)											{ m_xClientEntityList.RemoveFromList(&xEntity); }

	void	AddEntityToList					(Asura_Entity_Server& xEntity)											{ m_xServerEntityList.AddToList(&xEntity); }
	void	RemoveEntityFromList			(Asura_Entity_Server& xEntity)											{ m_xServerEntityList.RemoveFromList(&xEntity); }

	void	AddEntityToUpdateableList		(Asura_Entity_Client& xEntity)											{ m_xClientEntityUpdateableList.AddToList(&xEntity); }
	void	RemoveEntityFromUpdateableList	(Asura_Entity_Client& xEntity, const bool bShouldAlwaysBePresent)		{ m_xClientEntityUpdateableList.RemoveFromList(&xEntity, bShouldAlwaysBePresent); }

	void	AddEntityToUpdateableList		(Asura_Entity_Server& xEntity)											{ m_xServerEntityUpdateableList.AddToList(&xEntity); }
	void	RemoveEntityFromUpdateableList	(Asura_Entity_Server& xEntity, const bool bShouldAlwaysBePresent)		{ m_xServerEntityUpdateableList.RemoveFromList(&xEntity, bShouldAlwaysBePresent); }

	void	AddRenderableEntityToList		(Asura_Entity_Renderable& xEntity)										{ m_xRenderableEntityList.AddToList(&xEntity); }
	void	RemoveRenderableEntityFromList	(Asura_Entity_Renderable& xEntity, const bool bShouldAlwaysBePresent)	{ m_xRenderableEntityList.RemoveFromList(&xEntity, bShouldAlwaysBePresent); }

	void DestroySavedEntities();

	// Message-handling functions...
	void	AddMessageToQueue				(Asura_Message* pxMessage)		{ m_xMessageQueue.Push_Back(pxMessage); }
	void	FlushMessageQueue				(void);
	void	ClearMessageQueue				(void);

	// List accessors
	Asura_Entity_PeriodicClientUpdateList*	GetPeriodicClientUpdateList()	{ return &m_xPeriodicClientUpdateList; }
	Asura_Hashed_List<Asura_Entity_Server>* GetConstantClientUpdateList()	{ return &m_xConstantClientUpdateList; }
	Asura_Classified_List<Asura_Entity_Server>* GetServerEntityList()
	{
		return m_xServerEntityList.GetList();
	}
	Asura_Classified_List<Asura_Entity_Client>* GetClientEntityList()
	{
		return m_xClientEntityList.GetList();
	}
	Asura_Entity_Server* FindServerEntity(Asura_Guid uGuid) const
	{
		return m_xServerEntityList.FindEntity(uGuid);
	}
	Asura_Entity_Client* FindClientEntity(Asura_Guid uGuid) const
	{
		return m_xClientEntityList.FindEntity(uGuid);
	}

	static void MigrateEntity(Asura_Entity_Client* pxClientEntity, Asura_Streaming_ZoneEntities* pxOldEntities, Asura_Streaming_ZoneEntities* pxNewEntities);
	static void MigrateEntity(Asura_Entity_Renderable* pxRenderableEntity, Asura_Streaming_ZoneEntities* pxOldEntities, Asura_Streaming_ZoneEntities* pxNewEntities);
	static void MigrateEntity(Asura_Entity_Server* pxServerEntity, Asura_Streaming_ZoneEntities* pxOldEntities, Asura_Streaming_ZoneEntities* pxNewEntities);

protected:
	// Protected variables...


	// Protected functions...


private:
	// Private variables...
	Asura_Combined_List<Asura_Entity_Client>		m_xClientEntityList;			// List of client-entities contained by this zone.
	Asura_Combined_List<Asura_Entity_Server>		m_xServerEntityList;			// List of server-entities contained by this zone.

	Asura_Classified_List<Asura_Entity_Client>		m_xClientEntityUpdateableList;	// List of client-entities to update contained by this zone.
	Asura_Classified_List<Asura_Entity_Server>		m_xServerEntityUpdateableList;	// List of server-entities to update contained by this zone.

	Asura_Classified_List<Asura_Entity_Renderable>	m_xRenderableEntityList;		// List of entities-to-render contained by this zone.

	Asura_Entity_PeriodicClientUpdateList			m_xPeriodicClientUpdateList;	// List of server-entities that send periodic client update messages.
	Asura_Hashed_List<Asura_Entity_Server>			m_xConstantClientUpdateList;	// List of server-entities that send client update messages every frame.

	Asura_LinkList<Asura_Message*>					m_xMessageQueue;				// Queue of messages to be sent.

	// Private functions...

	Asura_Streaming_ZoneEntities  (const Asura_Streaming_ZoneEntities& xOther);

	Asura_Streaming_ZoneEntities&	operator=	(const Asura_Streaming_ZoneEntities& xOther);

};   // End of class 'Asura_Streaming_ZoneEntities'.


#endif   // ifndef __ASURA_STREAMING_ZONEENTITIES_H__

#endif	// ASURA_USE_STREAMING

/// End of file. ///
