
///////////////////////////////////////////////////////////////////////////////
//
// File   : Asura_Streaming_Job.h
// Author : Andy Tate
// Date   : 4-May-2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

// RJT NOTE: Required for GCC builds as the file list generation doesn't account for VC project exclusion from build
#ifdef ASURA_USE_STREAMING

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __ASURA_STREAMING_JOB_H__
#define __ASURA_STREAMING_JOB_H__

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Forward class declarations...
////////////////////////////////////////////////////////////////////////////////

class Asura_Streaming_JobQueue;


////////////////////////////////////////////////////////////////////////////////
// Class declaration...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Class        : Asura_Streaming_Job
// Author       : Andy Tate
// Date         : 4-May-2006
//
// Description  : Class which handles the behaviour of a particular job to be
//				  done during streaming. This class should be overridden for
//				  specific behaviours.
////////////////////////////////////////////////////////////////////////////////

class Asura_Streaming_Job
{
public:
	// Public functions...
	Asura_Streaming_Job          (const char* szName);
	virtual ~Asura_Streaming_Job ();

	// Manipulation functions...
	virtual	void		BeginUpdate		(void);
	virtual	bool		Update			(float& fTimeAllowed);
	virtual	void		EndUpdate		(void);

	// Accessor functions...
			bool		HasChildQueue	(void) const	{ return(m_pxChildQueue != NULL); }
			const char*	GetName			(void) const	{ return(m_szName); }


protected:
	// Protected variables...
	Asura_Streaming_JobQueue*	m_pxChildQueue;		// Queue of child jobs for this one.
	const char*					m_szName;			// The name of this job.
	u_int						m_uNumUpdates;		// The number of updates performed.


	// Protected functions...


private:
	// Private variables...


	// Private functions...
	Asura_Streaming_Job  (void);
	Asura_Streaming_Job  (const Asura_Streaming_Job& xOther);

	Asura_Streaming_Job&	operator=	(const Asura_Streaming_Job& xOther);

};   // End of class 'Asura_Streaming_Job'.


#endif   // ifndef __ASURA_STREAMING_JOB_H__

#endif	// ASURA_USE_STREAMING

/// End of file. ///
