//////////////////////////////////////////////////////////////////////////////
//
// Asura_SpeedTest_Memcpy.cpp
//
// Copyright (c) 2006 Rebellion. All rights reserved.
//
// Description:
//
//     speed test instances for memcpy
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

#ifdef ASURA_SUPPORT_UNITTEST

#include "UnitTest/Asura_SpeedTest.h"

#include "Core/Memory/Asura_Mem.h"

// C O N S T A N T S /////////////////////////////////////////////////////////

static const u_int uNUM_TESTS = 1000;
static const u_int uNUM_TESTS_1048576 = 100; // Do the 1 meg test less, as it takes a bloody long time and we'll still get usefully varying values with this amount

// G L O B A L S /////////////////////////////////////////////////////////////

// 1 meg buffer, with extra space to allow adjustment to a particular alignment
static char g_acMemcpyInput[(1024*1024)+64];
static char g_acMemcpyOutput[(1024*1024)+64];

ASURA_SPEEDTEST_INITIALISER(Memcpy)
{
}


ASURA_SPEEDTEST(Memcpy, Aligned1_Size0)
{
	for (u_int u=0; u<uNUM_TESTS; ++u)
	{
		Asura_Mem::Copy(g_acMemcpyOutput, g_acMemcpyInput, 0);
	}
}

ASURA_SPEEDTEST(Memcpy, Aligned1_Size1)
{
	for (u_int u=0; u<uNUM_TESTS; ++u)
	{
		Asura_Mem::Copy(g_acMemcpyOutput, g_acMemcpyInput, 1);
	}
}

ASURA_SPEEDTEST(Memcpy, Aligned1_Size2)
{
	for (u_int u=0; u<uNUM_TESTS; ++u)
	{
		Asura_Mem::Copy(g_acMemcpyOutput, g_acMemcpyInput, 2);
	}
}

ASURA_SPEEDTEST(Memcpy, Aligned1_Size4)
{
	for (u_int u=0; u<uNUM_TESTS; ++u)
	{
		Asura_Mem::Copy(g_acMemcpyOutput, g_acMemcpyInput, 4);
	}
}

ASURA_SPEEDTEST(Memcpy, Aligned1_Size1024)
{
	for (u_int u=0; u<uNUM_TESTS; ++u)
	{
		Asura_Mem::Copy(g_acMemcpyOutput, g_acMemcpyInput, 1024);
	}
}

ASURA_SPEEDTEST(Memcpy, Aligned1_Size2048)
{
	for (u_int u=0; u<uNUM_TESTS; ++u)
	{
		Asura_Mem::Copy(g_acMemcpyOutput, g_acMemcpyInput, 2048);
	}
}

ASURA_SPEEDTEST(Memcpy, Aligned1_Size4096)
{
	for (u_int u=0; u<uNUM_TESTS; ++u)
	{
		Asura_Mem::Copy(g_acMemcpyOutput, g_acMemcpyInput, 4096);
	}
}

ASURA_SPEEDTEST(Memcpy, Aligned1_Size8192)
{
	for (u_int u=0; u<uNUM_TESTS; ++u)
	{
		Asura_Mem::Copy(g_acMemcpyOutput, g_acMemcpyInput, 8192);
	}
}

ASURA_SPEEDTEST(Memcpy, Aligned1_Size16384)
{
	for (u_int u=0; u<uNUM_TESTS; ++u)
	{
		Asura_Mem::Copy(g_acMemcpyOutput, g_acMemcpyInput, 16384);
	}
}

ASURA_SPEEDTEST(Memcpy, Aligned1_Size32768)
{
	for (u_int u=0; u<uNUM_TESTS; ++u)
	{
		Asura_Mem::Copy(g_acMemcpyOutput, g_acMemcpyInput, 32768);
	}
}

ASURA_SPEEDTEST(Memcpy, Aligned1_Size1048576)
{
	for (u_int u=0; u<uNUM_TESTS_1048576; ++u)
	{
		Asura_Mem::Copy(g_acMemcpyOutput, g_acMemcpyInput, 1048576);
	}
}

ASURA_SPEEDTEST(Memcpy, Aligned2_Size0)
{
	char* pcAligned2Input = g_acMemcpyInput + ((int)g_acMemcpyInput & 1);
	char* pcAligned2Output = g_acMemcpyOutput + ((int)g_acMemcpyOutput & 1);
	for (u_int u=0; u<uNUM_TESTS; ++u)
	{
		Asura_Mem::Copy(pcAligned2Output, pcAligned2Input, 0);
	}
}

ASURA_SPEEDTEST(Memcpy, Aligned2_Size1)
{
	char* pcAligned2Input = g_acMemcpyInput + ((int)g_acMemcpyInput & 1);
	char* pcAligned2Output = g_acMemcpyOutput + ((int)g_acMemcpyOutput & 1);
	for (u_int u=0; u<uNUM_TESTS; ++u)
	{
		Asura_Mem::Copy(pcAligned2Output, pcAligned2Input, 1);
	}
}

ASURA_SPEEDTEST(Memcpy, Aligned2_Size2)
{
	char* pcAligned2Input = g_acMemcpyInput + ((int)g_acMemcpyInput & 1);
	char* pcAligned2Output = g_acMemcpyOutput + ((int)g_acMemcpyOutput & 1);
	for (u_int u=0; u<uNUM_TESTS; ++u)
	{
		Asura_Mem::Copy(pcAligned2Output, pcAligned2Input, 2);
	}
}

ASURA_SPEEDTEST(Memcpy, Aligned2_Size4)
{
	char* pcAligned2Input = g_acMemcpyInput + ((int)g_acMemcpyInput & 1);
	char* pcAligned2Output = g_acMemcpyOutput + ((int)g_acMemcpyOutput & 1);
	for (u_int u=0; u<uNUM_TESTS; ++u)
	{
		Asura_Mem::Copy(pcAligned2Output, pcAligned2Input, 4);
	}
}

ASURA_SPEEDTEST(Memcpy, Aligned2_Size1024)
{
	char* pcAligned2Input = g_acMemcpyInput + ((int)g_acMemcpyInput & 1);
	char* pcAligned2Output = g_acMemcpyOutput + ((int)g_acMemcpyOutput & 1);
	for (u_int u=0; u<uNUM_TESTS; ++u)
	{
		Asura_Mem::Copy(pcAligned2Output, pcAligned2Input, 1024);
	}
}

ASURA_SPEEDTEST(Memcpy, Aligned2_Size2048)
{
	char* pcAligned2Input = g_acMemcpyInput + ((int)g_acMemcpyInput & 1);
	char* pcAligned2Output = g_acMemcpyOutput + ((int)g_acMemcpyOutput & 1);
	for (u_int u=0; u<uNUM_TESTS; ++u)
	{
		Asura_Mem::Copy(pcAligned2Output, pcAligned2Input, 2048);
	}
}

ASURA_SPEEDTEST(Memcpy, Aligned2_Size4096)
{
	char* pcAligned2Input = g_acMemcpyInput + ((int)g_acMemcpyInput & 1);
	char* pcAligned2Output = g_acMemcpyOutput + ((int)g_acMemcpyOutput & 1);
	for (u_int u=0; u<uNUM_TESTS; ++u)
	{
		Asura_Mem::Copy(pcAligned2Output, pcAligned2Input, 4096);
	}
}

ASURA_SPEEDTEST(Memcpy, Aligned2_Size8192)
{
	char* pcAligned2Input = g_acMemcpyInput + ((int)g_acMemcpyInput & 1);
	char* pcAligned2Output = g_acMemcpyOutput + ((int)g_acMemcpyOutput & 1);
	for (u_int u=0; u<uNUM_TESTS; ++u)
	{
		Asura_Mem::Copy(pcAligned2Output, pcAligned2Input, 8192);
	}
}

ASURA_SPEEDTEST(Memcpy, Aligned2_Size16384)
{
	char* pcAligned2Input = g_acMemcpyInput + ((int)g_acMemcpyInput & 1);
	char* pcAligned2Output = g_acMemcpyOutput + ((int)g_acMemcpyOutput & 1);
	for (u_int u=0; u<uNUM_TESTS; ++u)
	{
		Asura_Mem::Copy(pcAligned2Output, pcAligned2Input, 16384);
	}
}

ASURA_SPEEDTEST(Memcpy, Aligned2_Size32768)
{
	char* pcAligned2Input = g_acMemcpyInput + ((int)g_acMemcpyInput & 1);
	char* pcAligned2Output = g_acMemcpyOutput + ((int)g_acMemcpyOutput & 1);
	for (u_int u=0; u<uNUM_TESTS; ++u)
	{
		Asura_Mem::Copy(pcAligned2Output, pcAligned2Input, 32768);
	}
}

ASURA_SPEEDTEST(Memcpy, Aligned2_Size1048576)
{
	char* pcAligned2Input = g_acMemcpyInput + ((int)g_acMemcpyInput & 1);
	char* pcAligned2Output = g_acMemcpyOutput + ((int)g_acMemcpyOutput & 1);
	for (u_int u=0; u<uNUM_TESTS_1048576; ++u)
	{
		Asura_Mem::Copy(pcAligned2Output, pcAligned2Input, 1048576);
	}
}

ASURA_SPEEDTEST(Memcpy, Aligned4_Size0)
{
	char* pcAligned4Input = g_acMemcpyInput + (4 - ((int)g_acMemcpyInput & 3));
	char* pcAligned4Output = g_acMemcpyOutput + (4 - ((int)g_acMemcpyOutput & 3));
	for (u_int u=0; u<uNUM_TESTS; ++u)
	{
		Asura_Mem::Copy(pcAligned4Output, pcAligned4Input, 0);
	}
}

ASURA_SPEEDTEST(Memcpy, Aligned4_Size1)
{
	char* pcAligned4Input = g_acMemcpyInput + (4 - ((int)g_acMemcpyInput & 3));
	char* pcAligned4Output = g_acMemcpyOutput + (4 - ((int)g_acMemcpyOutput & 3));
	for (u_int u=0; u<uNUM_TESTS; ++u)
	{
		Asura_Mem::Copy(pcAligned4Output, pcAligned4Input, 1);
	}
}

ASURA_SPEEDTEST(Memcpy, Aligned4_Size2)
{
	char* pcAligned4Input = g_acMemcpyInput + (4 - ((int)g_acMemcpyInput & 3));
	char* pcAligned4Output = g_acMemcpyOutput + (4 - ((int)g_acMemcpyOutput & 3));
	for (u_int u=0; u<uNUM_TESTS; ++u)
	{
		Asura_Mem::Copy(pcAligned4Output, pcAligned4Input, 2);
	}
}

ASURA_SPEEDTEST(Memcpy, Aligned4_Size4)
{
	char* pcAligned4Input = g_acMemcpyInput + (4 - ((int)g_acMemcpyInput & 3));
	char* pcAligned4Output = g_acMemcpyOutput + (4 - ((int)g_acMemcpyOutput & 3));
	for (u_int u=0; u<uNUM_TESTS; ++u)
	{
		Asura_Mem::Copy(pcAligned4Output, pcAligned4Input, 4);
	}
}

ASURA_SPEEDTEST(Memcpy, Aligned4_Size1024)
{
	char* pcAligned4Input = g_acMemcpyInput + (4 - ((int)g_acMemcpyInput & 3));
	char* pcAligned4Output = g_acMemcpyOutput + (4 - ((int)g_acMemcpyOutput & 3));
	for (u_int u=0; u<uNUM_TESTS; ++u)
	{
		Asura_Mem::Copy(pcAligned4Output, pcAligned4Input, 1024);
	}
}

ASURA_SPEEDTEST(Memcpy, Aligned4_Size2048)
{
	char* pcAligned4Input = g_acMemcpyInput + (4 - ((int)g_acMemcpyInput & 3));
	char* pcAligned4Output = g_acMemcpyOutput + (4 - ((int)g_acMemcpyOutput & 3));
	for (u_int u=0; u<uNUM_TESTS; ++u)
	{
		Asura_Mem::Copy(pcAligned4Output, pcAligned4Input, 2048);
	}
}

ASURA_SPEEDTEST(Memcpy, Aligned4_Size4096)
{
	char* pcAligned4Input = g_acMemcpyInput + (4 - ((int)g_acMemcpyInput & 3));
	char* pcAligned4Output = g_acMemcpyOutput + (4 - ((int)g_acMemcpyOutput & 3));
	for (u_int u=0; u<uNUM_TESTS; ++u)
	{
		Asura_Mem::Copy(pcAligned4Output, pcAligned4Input, 4096);
	}
}

ASURA_SPEEDTEST(Memcpy, Aligned4_Size8192)
{
	char* pcAligned4Input = g_acMemcpyInput + (4 - ((int)g_acMemcpyInput & 3));
	char* pcAligned4Output = g_acMemcpyOutput + (4 - ((int)g_acMemcpyOutput & 3));
	for (u_int u=0; u<uNUM_TESTS; ++u)
	{
		Asura_Mem::Copy(pcAligned4Output, pcAligned4Input, 8192);
	}
}

ASURA_SPEEDTEST(Memcpy, Aligned4_Size16384)
{
	char* pcAligned4Input = g_acMemcpyInput + (4 - ((int)g_acMemcpyInput & 3));
	char* pcAligned4Output = g_acMemcpyOutput + (4 - ((int)g_acMemcpyOutput & 3));
	for (u_int u=0; u<uNUM_TESTS; ++u)
	{
		Asura_Mem::Copy(pcAligned4Output, pcAligned4Input, 16384);
	}
}

ASURA_SPEEDTEST(Memcpy, Aligned4_Size32768)
{
	char* pcAligned4Input = g_acMemcpyInput + (4 - ((int)g_acMemcpyInput & 3));
	char* pcAligned4Output = g_acMemcpyOutput + (4 - ((int)g_acMemcpyOutput & 3));
	for (u_int u=0; u<uNUM_TESTS; ++u)
	{
		Asura_Mem::Copy(pcAligned4Output, pcAligned4Input, 32768);
	}
}

ASURA_SPEEDTEST(Memcpy, Aligned4_Size1048576)
{
	char* pcAligned4Input = g_acMemcpyInput + (4 - ((int)g_acMemcpyInput & 3));
	char* pcAligned4Output = g_acMemcpyOutput + (4 - ((int)g_acMemcpyOutput & 3));
	for (u_int u=0; u<uNUM_TESTS_1048576; ++u)
	{
		Asura_Mem::Copy(pcAligned4Output, pcAligned4Input, 1048576);
	}
}

ASURA_SPEEDTEST(Memcpy, Aligned8_Size0)
{
	char* pcAligned8Input = g_acMemcpyInput + (8 - ((int)g_acMemcpyInput & 7));
	char* pcAligned8Output = g_acMemcpyOutput + (8 - ((int)g_acMemcpyOutput & 7));
	for (u_int u=0; u<uNUM_TESTS; ++u)
	{
		Asura_Mem::Copy(pcAligned8Output, pcAligned8Input, 0);
	}
}

ASURA_SPEEDTEST(Memcpy, Aligned8_Size1)
{
	char* pcAligned8Input = g_acMemcpyInput + (8 - ((int)g_acMemcpyInput & 7));
	char* pcAligned8Output = g_acMemcpyOutput + (8 - ((int)g_acMemcpyOutput & 7));
	for (u_int u=0; u<uNUM_TESTS; ++u)
	{
		Asura_Mem::Copy(pcAligned8Output, pcAligned8Input, 1);
	}
}

ASURA_SPEEDTEST(Memcpy, Aligned8_Size2)
{
	char* pcAligned8Input = g_acMemcpyInput + (8 - ((int)g_acMemcpyInput & 7));
	char* pcAligned8Output = g_acMemcpyOutput + (8 - ((int)g_acMemcpyOutput & 7));
	for (u_int u=0; u<uNUM_TESTS; ++u)
	{
		Asura_Mem::Copy(pcAligned8Output, pcAligned8Input, 2);
	}
}

ASURA_SPEEDTEST(Memcpy, Aligned8_Size4)
{
	char* pcAligned8Input = g_acMemcpyInput + (8 - ((int)g_acMemcpyInput & 7));
	char* pcAligned8Output = g_acMemcpyOutput + (8 - ((int)g_acMemcpyOutput & 7));
	for (u_int u=0; u<uNUM_TESTS; ++u)
	{
		Asura_Mem::Copy(pcAligned8Output, pcAligned8Input, 4);
	}
}

ASURA_SPEEDTEST(Memcpy, Aligned8_Size1024)
{
	char* pcAligned8Input = g_acMemcpyInput + (8 - ((int)g_acMemcpyInput & 7));
	char* pcAligned8Output = g_acMemcpyOutput + (8 - ((int)g_acMemcpyOutput & 7));
	for (u_int u=0; u<uNUM_TESTS; ++u)
	{
		Asura_Mem::Copy(pcAligned8Output, pcAligned8Input, 1024);
	}
}

ASURA_SPEEDTEST(Memcpy, Aligned8_Size2048)
{
	char* pcAligned8Input = g_acMemcpyInput + (8 - ((int)g_acMemcpyInput & 7));
	char* pcAligned8Output = g_acMemcpyOutput + (8 - ((int)g_acMemcpyOutput & 7));
	for (u_int u=0; u<uNUM_TESTS; ++u)
	{
		Asura_Mem::Copy(pcAligned8Output, pcAligned8Input, 2048);
	}
}

ASURA_SPEEDTEST(Memcpy, Aligned8_Size4096)
{
	char* pcAligned8Input = g_acMemcpyInput + (8 - ((int)g_acMemcpyInput & 7));
	char* pcAligned8Output = g_acMemcpyOutput + (8 - ((int)g_acMemcpyOutput & 7));
	for (u_int u=0; u<uNUM_TESTS; ++u)
	{
		Asura_Mem::Copy(pcAligned8Output, pcAligned8Input, 4096);
	}
}

ASURA_SPEEDTEST(Memcpy, Aligned8_Size8192)
{
	char* pcAligned8Input = g_acMemcpyInput + (8 - ((int)g_acMemcpyInput & 7));
	char* pcAligned8Output = g_acMemcpyOutput + (8 - ((int)g_acMemcpyOutput & 7));
	for (u_int u=0; u<uNUM_TESTS; ++u)
	{
		Asura_Mem::Copy(pcAligned8Output, pcAligned8Input, 8192);
	}
}

ASURA_SPEEDTEST(Memcpy, Aligned8_Size16384)
{
	char* pcAligned8Input = g_acMemcpyInput + (8 - ((int)g_acMemcpyInput & 7));
	char* pcAligned8Output = g_acMemcpyOutput + (8 - ((int)g_acMemcpyOutput & 7));
	for (u_int u=0; u<uNUM_TESTS; ++u)
	{
		Asura_Mem::Copy(pcAligned8Output, pcAligned8Input, 16384);
	}
}

ASURA_SPEEDTEST(Memcpy, Aligned8_Size32768)
{
	char* pcAligned8Input = g_acMemcpyInput + (8 - ((int)g_acMemcpyInput & 7));
	char* pcAligned8Output = g_acMemcpyOutput + (8 - ((int)g_acMemcpyOutput & 7));
	for (u_int u=0; u<uNUM_TESTS; ++u)
	{
		Asura_Mem::Copy(pcAligned8Output, pcAligned8Input, 32768);
	}
}

ASURA_SPEEDTEST(Memcpy, Aligned8_Size1048576)
{
	char* pcAligned8Input = g_acMemcpyInput + (8 - ((int)g_acMemcpyInput & 7));
	char* pcAligned8Output = g_acMemcpyOutput + (8 - ((int)g_acMemcpyOutput & 7));
	for (u_int u=0; u<uNUM_TESTS_1048576; ++u)
	{
		Asura_Mem::Copy(pcAligned8Output, pcAligned8Input, 1048576);
	}
}

ASURA_SPEEDTEST(Memcpy, Aligned16_Size0)
{
	char* pcAligned16Input = g_acMemcpyInput + (16 - ((int)g_acMemcpyInput & 15));
	char* pcAligned16Output = g_acMemcpyOutput + (16 - ((int)g_acMemcpyOutput & 15));
	for (u_int u=0; u<uNUM_TESTS; ++u)
	{
		Asura_Mem::Copy(pcAligned16Output, pcAligned16Input, 0);
	}
}

ASURA_SPEEDTEST(Memcpy, Aligned16_Size1)
{
	char* pcAligned16Input = g_acMemcpyInput + (16 - ((int)g_acMemcpyInput & 15));
	char* pcAligned16Output = g_acMemcpyOutput + (16 - ((int)g_acMemcpyOutput & 15));
	for (u_int u=0; u<uNUM_TESTS; ++u)
	{
		Asura_Mem::Copy(pcAligned16Output, pcAligned16Input, 1);
	}
}

ASURA_SPEEDTEST(Memcpy, Aligned16_Size2)
{
	char* pcAligned16Input = g_acMemcpyInput + (16 - ((int)g_acMemcpyInput & 15));
	char* pcAligned16Output = g_acMemcpyOutput + (16 - ((int)g_acMemcpyOutput & 15));
	for (u_int u=0; u<uNUM_TESTS; ++u)
	{
		Asura_Mem::Copy(pcAligned16Output, pcAligned16Input, 2);
	}
}

ASURA_SPEEDTEST(Memcpy, Aligned16_Size4)
{
	char* pcAligned16Input = g_acMemcpyInput + (16 - ((int)g_acMemcpyInput & 15));
	char* pcAligned16Output = g_acMemcpyOutput + (16 - ((int)g_acMemcpyOutput & 15));
	for (u_int u=0; u<uNUM_TESTS; ++u)
	{
		Asura_Mem::Copy(pcAligned16Output, pcAligned16Input, 4);
	}
}

ASURA_SPEEDTEST(Memcpy, Aligned16_Size1024)
{
	char* pcAligned16Input = g_acMemcpyInput + (16 - ((int)g_acMemcpyInput & 15));
	char* pcAligned16Output = g_acMemcpyOutput + (16 - ((int)g_acMemcpyOutput & 15));
	for (u_int u=0; u<uNUM_TESTS; ++u)
	{
		Asura_Mem::Copy(pcAligned16Output, pcAligned16Input, 1024);
	}
}

ASURA_SPEEDTEST(Memcpy, Aligned16_Size2048)
{
	char* pcAligned16Input = g_acMemcpyInput + (16 - ((int)g_acMemcpyInput & 15));
	char* pcAligned16Output = g_acMemcpyOutput + (16 - ((int)g_acMemcpyOutput & 15));
	for (u_int u=0; u<uNUM_TESTS; ++u)
	{
		Asura_Mem::Copy(pcAligned16Output, pcAligned16Input, 2048);
	}
}

ASURA_SPEEDTEST(Memcpy, Aligned16_Size4096)
{
	char* pcAligned16Input = g_acMemcpyInput + (16 - ((int)g_acMemcpyInput & 15));
	char* pcAligned16Output = g_acMemcpyOutput + (16 - ((int)g_acMemcpyOutput & 15));
	for (u_int u=0; u<uNUM_TESTS; ++u)
	{
		Asura_Mem::Copy(pcAligned16Output, pcAligned16Input, 4096);
	}
}

ASURA_SPEEDTEST(Memcpy, Aligned16_Size8192)
{
	char* pcAligned16Input = g_acMemcpyInput + (16 - ((int)g_acMemcpyInput & 15));
	char* pcAligned16Output = g_acMemcpyOutput + (16 - ((int)g_acMemcpyOutput & 15));
	for (u_int u=0; u<uNUM_TESTS; ++u)
	{
		Asura_Mem::Copy(pcAligned16Output, pcAligned16Input, 8192);
	}
}

ASURA_SPEEDTEST(Memcpy, Aligned16_Size16384)
{
	char* pcAligned16Input = g_acMemcpyInput + (16 - ((int)g_acMemcpyInput & 15));
	char* pcAligned16Output = g_acMemcpyOutput + (16 - ((int)g_acMemcpyOutput & 15));
	for (u_int u=0; u<uNUM_TESTS; ++u)
	{
		Asura_Mem::Copy(pcAligned16Output, pcAligned16Input, 16384);
	}
}

ASURA_SPEEDTEST(Memcpy, Aligned16_Size32768)
{
	char* pcAligned16Input = g_acMemcpyInput + (16 - ((int)g_acMemcpyInput & 15));
	char* pcAligned16Output = g_acMemcpyOutput + (16 - ((int)g_acMemcpyOutput & 15));
	for (u_int u=0; u<uNUM_TESTS; ++u)
	{
		Asura_Mem::Copy(pcAligned16Output, pcAligned16Input, 32768);
	}
}

ASURA_SPEEDTEST(Memcpy, Aligned16_Size1048576)
{
	char* pcAligned16Input = g_acMemcpyInput + (16 - ((int)g_acMemcpyInput & 15));
	char* pcAligned16Output = g_acMemcpyOutput + (16 - ((int)g_acMemcpyOutput & 15));
	for (u_int u=0; u<uNUM_TESTS_1048576; ++u)
	{
		Asura_Mem::Copy(pcAligned16Output, pcAligned16Input, 1048576);
	}
}

ASURA_SPEEDTEST(Memcpy, Aligned32_Size0)
{
	char* pcAligned32Input = g_acMemcpyInput + (32 - ((int)g_acMemcpyInput & 31));
	char* pcAligned32Output = g_acMemcpyOutput + (32 - ((int)g_acMemcpyOutput & 31));
	for (u_int u=0; u<uNUM_TESTS; ++u)
	{
		Asura_Mem::Copy(pcAligned32Output, pcAligned32Input, 0);
	}
}

ASURA_SPEEDTEST(Memcpy, Aligned32_Size1)
{
	char* pcAligned32Input = g_acMemcpyInput + (32 - ((int)g_acMemcpyInput & 31));
	char* pcAligned32Output = g_acMemcpyOutput + (32 - ((int)g_acMemcpyOutput & 31));
	for (u_int u=0; u<uNUM_TESTS; ++u)
	{
		Asura_Mem::Copy(pcAligned32Output, pcAligned32Input, 1);
	}
}

ASURA_SPEEDTEST(Memcpy, Aligned32_Size2)
{
	char* pcAligned32Input = g_acMemcpyInput + (32 - ((int)g_acMemcpyInput & 31));
	char* pcAligned32Output = g_acMemcpyOutput + (32 - ((int)g_acMemcpyOutput & 31));
	for (u_int u=0; u<uNUM_TESTS; ++u)
	{
		Asura_Mem::Copy(pcAligned32Output, pcAligned32Input, 2);
	}
}

ASURA_SPEEDTEST(Memcpy, Aligned32_Size4)
{
	char* pcAligned32Input = g_acMemcpyInput + (32 - ((int)g_acMemcpyInput & 31));
	char* pcAligned32Output = g_acMemcpyOutput + (32 - ((int)g_acMemcpyOutput & 31));
	for (u_int u=0; u<uNUM_TESTS; ++u)
	{
		Asura_Mem::Copy(pcAligned32Output, pcAligned32Input, 4);
	}
}

ASURA_SPEEDTEST(Memcpy, Aligned32_Size1024)
{
	char* pcAligned32Input = g_acMemcpyInput + (32 - ((int)g_acMemcpyInput & 31));
	char* pcAligned32Output = g_acMemcpyOutput + (32 - ((int)g_acMemcpyOutput & 31));
	for (u_int u=0; u<uNUM_TESTS; ++u)
	{
		Asura_Mem::Copy(pcAligned32Output, pcAligned32Input, 1024);
	}
}

ASURA_SPEEDTEST(Memcpy, Aligned32_Size2048)
{
	char* pcAligned32Input = g_acMemcpyInput + (32 - ((int)g_acMemcpyInput & 31));
	char* pcAligned32Output = g_acMemcpyOutput + (32 - ((int)g_acMemcpyOutput & 31));
	for (u_int u=0; u<uNUM_TESTS; ++u)
	{
		Asura_Mem::Copy(pcAligned32Output, pcAligned32Input, 2048);
	}
}

ASURA_SPEEDTEST(Memcpy, Aligned32_Size4096)
{
	char* pcAligned32Input = g_acMemcpyInput + (32 - ((int)g_acMemcpyInput & 31));
	char* pcAligned32Output = g_acMemcpyOutput + (32 - ((int)g_acMemcpyOutput & 31));
	for (u_int u=0; u<uNUM_TESTS; ++u)
	{
		Asura_Mem::Copy(pcAligned32Output, pcAligned32Input, 4096);
	}
}

ASURA_SPEEDTEST(Memcpy, Aligned32_Size8192)
{
	char* pcAligned32Input = g_acMemcpyInput + (32 - ((int)g_acMemcpyInput & 31));
	char* pcAligned32Output = g_acMemcpyOutput + (32 - ((int)g_acMemcpyOutput & 31));
	for (u_int u=0; u<uNUM_TESTS; ++u)
	{
		Asura_Mem::Copy(pcAligned32Output, pcAligned32Input, 8192);
	}
}

ASURA_SPEEDTEST(Memcpy, Aligned32_Size16384)
{
	char* pcAligned32Input = g_acMemcpyInput + (32 - ((int)g_acMemcpyInput & 31));
	char* pcAligned32Output = g_acMemcpyOutput + (32 - ((int)g_acMemcpyOutput & 31));
	for (u_int u=0; u<uNUM_TESTS; ++u)
	{
		Asura_Mem::Copy(pcAligned32Output, pcAligned32Input, 16384);
	}
}

ASURA_SPEEDTEST(Memcpy, Aligned32_Size32768)
{
	char* pcAligned32Input = g_acMemcpyInput + (32 - ((int)g_acMemcpyInput & 31));
	char* pcAligned32Output = g_acMemcpyOutput + (32 - ((int)g_acMemcpyOutput & 31));
	for (u_int u=0; u<uNUM_TESTS; ++u)
	{
		Asura_Mem::Copy(pcAligned32Output, pcAligned32Input, 32768);
	}
}

ASURA_SPEEDTEST(Memcpy, Aligned32_Size1048576)
{
	char* pcAligned32Input = g_acMemcpyInput + (32 - ((int)g_acMemcpyInput & 31));
	char* pcAligned32Output = g_acMemcpyOutput + (32 - ((int)g_acMemcpyOutput & 31));
	for (u_int u=0; u<uNUM_TESTS_1048576; ++u)
	{
		Asura_Mem::Copy(pcAligned32Output, pcAligned32Input, 1048576);
	}
}

ASURA_SPEEDTEST(Memcpy, Aligned64_Size0)
{
	char* pcAligned64Input = g_acMemcpyInput + (64 - ((int)g_acMemcpyInput & 63));
	char* pcAligned64Output = g_acMemcpyOutput + (64 - ((int)g_acMemcpyOutput & 63));
	for (u_int u=0; u<uNUM_TESTS; ++u)
	{
		Asura_Mem::Copy(pcAligned64Output, pcAligned64Input, 0);
	}
}

ASURA_SPEEDTEST(Memcpy, Aligned64_Size1)
{
	char* pcAligned64Input = g_acMemcpyInput + (64 - ((int)g_acMemcpyInput & 63));
	char* pcAligned64Output = g_acMemcpyOutput + (64 - ((int)g_acMemcpyOutput & 63));
	for (u_int u=0; u<uNUM_TESTS; ++u)
	{
		Asura_Mem::Copy(pcAligned64Output, pcAligned64Input, 1);
	}
}

ASURA_SPEEDTEST(Memcpy, Aligned64_Size2)
{
	char* pcAligned64Input = g_acMemcpyInput + (64 - ((int)g_acMemcpyInput & 63));
	char* pcAligned64Output = g_acMemcpyOutput + (64 - ((int)g_acMemcpyOutput & 63));
	for (u_int u=0; u<uNUM_TESTS; ++u)
	{
		Asura_Mem::Copy(pcAligned64Output, pcAligned64Input, 2);
	}
}

ASURA_SPEEDTEST(Memcpy, Aligned64_Size4)
{
	char* pcAligned64Input = g_acMemcpyInput + (64 - ((int)g_acMemcpyInput & 63));
	char* pcAligned64Output = g_acMemcpyOutput + (64 - ((int)g_acMemcpyOutput & 63));
	for (u_int u=0; u<uNUM_TESTS; ++u)
	{
		Asura_Mem::Copy(pcAligned64Output, pcAligned64Input, 4);
	}
}

ASURA_SPEEDTEST(Memcpy, Aligned64_Size1024)
{
	char* pcAligned64Input = g_acMemcpyInput + (64 - ((int)g_acMemcpyInput & 63));
	char* pcAligned64Output = g_acMemcpyOutput + (64 - ((int)g_acMemcpyOutput & 63));
	for (u_int u=0; u<uNUM_TESTS; ++u)
	{
		Asura_Mem::Copy(pcAligned64Output, pcAligned64Input, 1024);
	}
}

ASURA_SPEEDTEST(Memcpy, Aligned64_Size2048)
{
	char* pcAligned64Input = g_acMemcpyInput + (64 - ((int)g_acMemcpyInput & 63));
	char* pcAligned64Output = g_acMemcpyOutput + (64 - ((int)g_acMemcpyOutput & 63));
	for (u_int u=0; u<uNUM_TESTS; ++u)
	{
		Asura_Mem::Copy(pcAligned64Output, pcAligned64Input, 2048);
	}
}

ASURA_SPEEDTEST(Memcpy, Aligned64_Size4096)
{
	char* pcAligned64Input = g_acMemcpyInput + (64 - ((int)g_acMemcpyInput & 63));
	char* pcAligned64Output = g_acMemcpyOutput + (64 - ((int)g_acMemcpyOutput & 63));
	for (u_int u=0; u<uNUM_TESTS; ++u)
	{
		Asura_Mem::Copy(pcAligned64Output, pcAligned64Input, 4096);
	}
}

ASURA_SPEEDTEST(Memcpy, Aligned64_Size8192)
{
	char* pcAligned64Input = g_acMemcpyInput + (64 - ((int)g_acMemcpyInput & 63));
	char* pcAligned64Output = g_acMemcpyOutput + (64 - ((int)g_acMemcpyOutput & 63));
	for (u_int u=0; u<uNUM_TESTS; ++u)
	{
		Asura_Mem::Copy(pcAligned64Output, pcAligned64Input, 8192);
	}
}

ASURA_SPEEDTEST(Memcpy, Aligned64_Size16384)
{
	char* pcAligned64Input = g_acMemcpyInput + (64 - ((int)g_acMemcpyInput & 63));
	char* pcAligned64Output = g_acMemcpyOutput + (64 - ((int)g_acMemcpyOutput & 63));
	for (u_int u=0; u<uNUM_TESTS; ++u)
	{
		Asura_Mem::Copy(pcAligned64Output, pcAligned64Input, 16384);
	}
}

ASURA_SPEEDTEST(Memcpy, Aligned64_Size32768)
{
	char* pcAligned64Input = g_acMemcpyInput + (64 - ((int)g_acMemcpyInput & 63));
	char* pcAligned64Output = g_acMemcpyOutput + (64 - ((int)g_acMemcpyOutput & 63));
	for (u_int u=0; u<uNUM_TESTS; ++u)
	{
		Asura_Mem::Copy(pcAligned64Output, pcAligned64Input, 32768);
	}
}

ASURA_SPEEDTEST(Memcpy, Aligned64_Size1048576)
{
	char* pcAligned64Input = g_acMemcpyInput + (64 - ((int)g_acMemcpyInput & 63));
	char* pcAligned64Output = g_acMemcpyOutput + (64 - ((int)g_acMemcpyOutput & 63));
	for (u_int u=0; u<uNUM_TESTS_1048576; ++u)
	{
		Asura_Mem::Copy(pcAligned64Output, pcAligned64Input, 1048576);
	}
}


ASURA_SPEEDTEST_ENDOFFILE(Memcpy)

#endif // #ifdef ASURA_SUPPORT_UNITTEST
