//////////////////////////////////////////////////////////////////////////////
//
// Asura_SpeedTest_Trig.cpp
//
// Copyright (c) 2005 Rebellion. All rights reserved.
//
// Description:
//
//     speed test instances for Sin, Cos etc.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

#ifdef ASURA_SUPPORT_UNITTEST

#include "UnitTest/Asura_SpeedTest.h"

#include "Core/Asura_Maths.h"

// C O N S T A N T S /////////////////////////////////////////////////////////

static const u_int uNUM_TESTS = 100000;
static const u_int uNUM_DATA  = 4;
static const u_int uDATA_MASK = uNUM_DATA - 1;

// G L O B A L S /////////////////////////////////////////////////////////////

static float g_afInput[uNUM_DATA];
static float g_afInputPositive[uNUM_DATA];
static float g_afOutput[uNUM_DATA];

ASURA_SPEEDTEST_INITIALISER(Trig)
{
	for( u_int u = 0; u < uNUM_DATA; ++u )
	{
		g_afInput[u] = Asura_Random::GetFloat(-Asura_Maths::Deg360, Asura_Maths::Deg360);
		g_afInputPositive[u] = Asura_Maths::Abs(g_afInput[u]);
	}
}


ASURA_SPEEDTEST(Trig, Sin)
{
	for( u_int u = 0; u < uNUM_TESTS - 1; ++u )
	{
		g_afOutput[u & uDATA_MASK] = Asura_Maths::Sine(g_afInput[u & uDATA_MASK]);
	}
}

ASURA_SPEEDTEST(Trig, Cos)
{
	for( u_int u = 0; u < uNUM_TESTS - 1; ++u )
	{
		g_afOutput[u & uDATA_MASK] = Asura_Maths::Cosine(g_afInput[u & uDATA_MASK]);
	}
}

ASURA_SPEEDTEST(Trig, Tan)
{
	for( u_int u = 0; u < uNUM_TESTS - 1; ++u )
	{
		g_afOutput[u & uDATA_MASK] = Asura_Maths::Tangent(g_afInput[u & uDATA_MASK]);
	}
}

ASURA_SPEEDTEST(Trig, ArcSin)
{
	for( u_int u = 0; u < uNUM_TESTS - 1; ++u )
	{
		g_afOutput[u & uDATA_MASK] = Asura_Maths::ArcSine(g_afInput[u & uDATA_MASK]);
	}
}

ASURA_SPEEDTEST(Trig, ArcCos)
{
	for( u_int u = 0; u < uNUM_TESTS - 1; ++u )
	{
		g_afOutput[u & uDATA_MASK] = Asura_Maths::ArcCosine(g_afInput[u & uDATA_MASK]);
	}
}

ASURA_SPEEDTEST(Trig, ArcTan)
{
	for( u_int u = 0; u < uNUM_TESTS - 1; ++u )
	{
		g_afOutput[u & uDATA_MASK] = Asura_Maths::ArcTangent(g_afInput[u & uDATA_MASK], g_afInput[(u+1) & uDATA_MASK]);
	}
}

ASURA_SPEEDTEST(Trig, SinCos)
{
	for( u_int u = 0; u < uNUM_TESTS - 1; ++u )
	{
		Asura_Maths::SinCos(g_afInput[u & uDATA_MASK], g_afOutput[u & uDATA_MASK], g_afOutput[(u+1) & uDATA_MASK]);
	}
}

ASURA_SPEEDTEST(Trig, SinCos2)
{
	for( u_int u = 0; u < uNUM_TESTS - 1; ++u )
	{
		Asura_Maths::SinCos2(g_afInput[(u+0) & uDATA_MASK], g_afInput[(u+1) & uDATA_MASK], 
			g_afOutput[u & uDATA_MASK], g_afOutput[(u+1) & uDATA_MASK], 
			g_afOutput[(u+2) & uDATA_MASK], g_afOutput[(u+3) & uDATA_MASK]);
	}
}

ASURA_SPEEDTEST(Trig, Sqrt)
{
  	for( u_int u = 0; u < uNUM_TESTS - 1; ++u )
  	{
  		g_afOutput[u & uDATA_MASK] = Asura_Maths::Sqrt(g_afInputPositive[u & uDATA_MASK]);
  	}
}

ASURA_SPEEDTEST(Trig, InvSqrt)
{
  	for( u_int u = 0; u < uNUM_TESTS - 1; ++u )
  	{
  		g_afOutput[u & uDATA_MASK] = Asura_Maths::InvSqrt(g_afInputPositive[u & uDATA_MASK]);
  	}
}

ASURA_SPEEDTEST(Trig, Abs)
{
  	for( u_int u = 0; u < uNUM_TESTS - 1; ++u )
  	{
  		g_afOutput[u & uDATA_MASK] = Asura_Maths::Abs(g_afInputPositive[u & uDATA_MASK]);
  	}
}

ASURA_SPEEDTEST_ENDOFFILE(Trig)

#endif // #ifdef ASURA_SUPPORT_UNITTEST

