//////////////////////////////////////////////////////////////////////////////
//
// Asura_UnitTest_Maths.cpp
//
// Copyright (c) 2005 Rebellion. All rights reserved.
//
// Description:
//
//     Unit tests for Asura_Maths
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

#ifdef ASURA_SUPPORT_UNITTEST

// KOR - dirty hack for Wii compiler.
#ifdef RVL_SDK
#pragma opt_usedef_mem_limit 556
#endif

#include "UnitTest/Asura_UnitTest.h"

#include "Core/Asura_LOS.h"
#include "Core/Asura_Maths.h"

#include <limits>

static const float fEPSILON = 2e-5f; //BW changed from 1e-5f because PS3 with fastmath is less accurate than other platforms
static const float fEPSILON_LARGE = 2e-3f;

// R A Y S   A N D   L I N E   S E G M E N T S ///////////////////////////////

ASURA_UNITTEST(NearestPointOnLineSegment, Segment)
{
	Asura_Vector_3 xLineStart(0.0f,2.0f,0.0f);
	Asura_Vector_3 xLineEnd(2.0f,0.0f,0.0f);
	Asura_Vector_3 xPoint(0.0f,0.0f,0.0f);
	Asura_Vector_3 xTarget(1.0f,1.0f,0.0f);
	Asura_Vector_3 xNearest;
	Asura_Maths::GetNearestPointOnLineSegment(xLineStart, xLineEnd, xPoint, xNearest);
	ASURA_UNITTEST_CHECK( xNearest.ApproximatelyEquals(xTarget) );
	Asura_AVector3 xALineStart(xLineStart);
	Asura_AVector3 xALineEnd(xLineEnd);
	Asura_AVector3 xAPoint(xPoint);
	Asura_AVector3 xATarget(xTarget);
	Asura_AVector3 xANearest;
	Asura_Maths::GetNearestPointOnLineSegment(xALineStart,xALineEnd,xAPoint,xANearest);
	ASURA_UNITTEST_CHECK( xANearest.ApproximatelyEquals(xATarget) );
}

ASURA_UNITTEST(GetMinDistSqdBetweenLineSegmentAndRay, Segment)
{
	Asura_Vector_3 xRayOrigin(0.0f,0.0f,0.0f);
	Asura_Vector_3 xRayDirection(0.0f,0.0f,1.0f);
	Asura_Vector_3 xLineSegmentOrigin(0.0f,1.0f,0.0f);
	Asura_Vector_3 xLineSegmentVector(0.0f,0.0f,1.0f);
	ASURA_UNITTEST_FLOATS_EQUAL( 1.0f, Asura_Maths::GetMinDistSqdBetweenLineSegmentAndRay(xRayOrigin,xRayDirection,xLineSegmentOrigin,xLineSegmentVector), 1.0e-6f );
}

ASURA_UNITTEST(RayIntersects, Segment)
{
	Asura_Vector_3 xAlpha( 0.0f,0.0f,0.0f );
	Asura_Vector_3 xBeta( 0.0f, 0.0f, 1.0f );
	Asura_Vector_3 xCentre( 1.0f,1.0f,1.0f );
	ASURA_UNITTEST_CHECK( Asura_Maths::RayIntersectsSphere( xAlpha, xBeta, xCentre, 1.5f ) );
	ASURA_UNITTEST_CHECK( ! Asura_Maths::RayIntersectsSphere( xAlpha, xBeta, xCentre, 0.8f ) );
	Asura_Vector_3 xSecondPoint( -1.0f,1.0f, 1.0f );
	Asura_Vector_3 xThirdPoint( 0.0f, -1.0f, 1.0f );
	Asura_Vector_3 xFourthPoint( 0.0f, 2.0f, 1.0f );
	bool b=false;
	ASURA_UNITTEST_FLOATS_EQUAL( 1.0f, Asura_Maths::RayIntersectsTri( xAlpha, xBeta, xCentre, xSecondPoint, xThirdPoint, b ), 1.0e-6f );
	ASURA_UNITTEST_CHECK( !b );
	ASURA_UNITTEST_FLOATS_EQUAL( 1.0f, Asura_Maths::RayIntersectsTri( xAlpha, xBeta, xCentre, xThirdPoint, xSecondPoint, b ), 1.0e-6f );
	ASURA_UNITTEST_CHECK( b );
	ASURA_UNITTEST_CHECK( ASURA_LOS_MAX_DISTANCE == Asura_Maths::RayIntersectsTri( xAlpha, xBeta, xCentre, xSecondPoint, xFourthPoint, b ) );
	Asura_Vector_3 xSecondBeta( 0.0f,1.0f,0.0f );
	ASURA_UNITTEST_CHECK( ASURA_LOS_MAX_DISTANCE == Asura_Maths::RayIntersectsTri( xAlpha, xSecondBeta, xCentre, xSecondPoint, xThirdPoint, b ) );
	Asura_AVector3 xAAlpha( xAlpha );
	Asura_AVector3 xABeta( xBeta );
	Asura_AVector3 xACentre( xCentre );
	Asura_AVector3 xASecondPoint( xSecondPoint );
	Asura_AVector3 xAThirdPoint( xThirdPoint );
	Asura_AVector3 xAFourthPoint( xFourthPoint );
	Asura_AVector3 xASecondBeta( xSecondBeta );
	ASURA_UNITTEST_FLOATS_EQUAL( 1.0f, Asura_Maths::RayIntersectsTri( xAAlpha, xABeta, xACentre, xASecondPoint, xAThirdPoint, b ), 1.0e-6f );
	ASURA_UNITTEST_CHECK( !b );
	ASURA_UNITTEST_FLOATS_EQUAL( 1.0f, Asura_Maths::RayIntersectsTri( xAAlpha, xABeta, xACentre, xAThirdPoint, xASecondPoint, b ), 1.0e-6f );
	ASURA_UNITTEST_CHECK( b );
	ASURA_UNITTEST_CHECK( ASURA_LOS_MAX_DISTANCE == Asura_Maths::RayIntersectsTri( xAAlpha, xABeta, xACentre, xASecondPoint, xAFourthPoint, b ) );
	ASURA_UNITTEST_CHECK( ASURA_LOS_MAX_DISTANCE == Asura_Maths::RayIntersectsTri( xAAlpha, xASecondBeta, xACentre, xASecondPoint, xAThirdPoint, b ) );
	Asura_Bounding_Box xBB;
	xBB.MaxX = 1;
	xBB.MinX = -1;
	xBB.MaxY = 1;
	xBB.MinY = -1;
	xBB.MinZ = 1;
	xBB.MaxZ = 2;
	float fDistance;
	Asura_Vector_3 xHitPosition;
	Asura_Vector_3 xNormal;
	ASURA_UNITTEST_CHECK( Asura_Maths::RayIntersectsBoundingBox( xAlpha, xBeta, xBB, &fDistance, &xHitPosition, &b, &xNormal ) );
	ASURA_UNITTEST_FLOATS_EQUAL( 1.0f, fDistance, 1.0e-6f );
	ASURA_UNITTEST_CHECK( !b );
	ASURA_UNITTEST_CHECK_EQUAL( xBeta, xHitPosition );
	ASURA_UNITTEST_CHECK_EQUAL( -xBeta, xNormal );
	Asura_Vector_3 xAlphaTwo( 0.0f,0.0f,1.5f );
	ASURA_UNITTEST_CHECK( Asura_Maths::RayIntersectsBoundingBox( xAlphaTwo, xBeta, xBB, &fDistance, &xHitPosition, &b, &xNormal ) );
	ASURA_UNITTEST_CHECK( b );
	ASURA_UNITTEST_CHECK( ! Asura_Maths::RayIntersectsBoundingBox( xAlpha, xSecondBeta, xBB, &fDistance, &xHitPosition, &b, &xNormal ) );

    xBB.MinZ = -1.0f;
    xBB.MaxZ = 1.0f;

    for( int i = 0; i < 100; ++i )
    {
        xAlpha.x = Asura_Random::GetFloat( -1.0f, 1.0f );
        xAlpha.y = Asura_Random::GetFloat( -1.0f, 1.0f );
        xAlpha.z = Asura_Random::GetFloat( -1.0f, 1.0f );

        // For the last 10 tests pick points on the faces.
        if( i > 90 )
        {
            int iFace = Asura_Random::GetInt( 0, 5 );
            xAlpha.SetElement( iFace >> 1, ((iFace & 1) ? 1.0f : -1.0f) );
        }

        xBeta.RandomDirection();

        ASURA_UNITTEST_CHECK( Asura_Maths::RayIntersectsBoundingBox_AllowInside(xAlpha, xBeta, xBB, &fDistance, &xHitPosition, &b, &xNormal) );
        ASURA_UNITTEST_CHECK( b );

        xAlphaTwo = xAlpha + xBeta * 4.0f; // anything > sqrt(12) would suffice
        float fDistance2;
        Asura_Vector_3 xHitPosition2, xNormal2;
        ASURA_UNITTEST_CHECK( Asura_Maths::RayIntersectsBoundingBox(xAlphaTwo, -xBeta, xBB, &fDistance2, &xHitPosition2, &b, &xNormal2) );
        ASURA_UNITTEST_CHECK( !b );
        ASURA_UNITTEST_FLOATS_EQUAL( fDistance, 4.0f - fDistance2, 1.0e-5f );
        ASURA_UNITTEST_VECTOR3_EQUAL( xHitPosition, xHitPosition2, 1.0e-6f );
        ASURA_UNITTEST_VECTOR3_EQUAL( xNormal, -xNormal2, 1.0e-6f );
    }
}

// I N T   V E C T O R S /////////////////////////////////////////////////////

ASURA_UNITTEST(ArrayAccessors, IntVector2)
{
	Asura_IntVector_2 xVec(0,1);
	ASURA_UNITTEST_CHECK_EQUAL( 1, xVec[1] );
	ASURA_UNITTEST_CHECK_EQUAL( 0, xVec[0] );
}

ASURA_UNITTEST(Scaling, IntVector2)
{
    ASURA_UNITTEST_CHECK_EQUAL( Asura_IntVector_2(1,2), Asura_IntVector_2(1,2) * 1);
    ASURA_UNITTEST_CHECK_EQUAL( Asura_IntVector_2(-2,-4), Asura_IntVector_2(1,2) * -2);
    ASURA_UNITTEST_CHECK_EQUAL( Asura_IntVector_2(6,-33), Asura_IntVector_2(2,-11) * 3);
}

ASURA_UNITTEST(Addition, IntVector2)
{
    ASURA_UNITTEST_CHECK_EQUAL( Asura_IntVector_2(2,4), Asura_IntVector_2(1,2) + Asura_IntVector_2(1,2) );
    ASURA_UNITTEST_CHECK_EQUAL( Asura_IntVector_2(5,16), Asura_IntVector_2(3,5) + Asura_IntVector_2(2,11) );
	ASURA_UNITTEST_CHECK_EQUAL( Asura_Maths::ZeroIntVector2, Asura_IntVector_2(-2,-4) + Asura_IntVector_2(2,4) );
}

ASURA_UNITTEST(PlusEquals, IntVector2)
{
	Asura_IntVector_2 xTemp(2,4);
	xTemp += Asura_IntVector_2(1,2);
    ASURA_UNITTEST_CHECK_EQUAL( xTemp, Asura_IntVector_2(3,6) );
}

ASURA_UNITTEST(Subtraction, IntVector2)
{
    ASURA_UNITTEST_CHECK_EQUAL( Asura_IntVector_2(1,2), Asura_IntVector_2(2,4) - Asura_IntVector_2(1,2) );
    ASURA_UNITTEST_CHECK_EQUAL( Asura_IntVector_2(1,-6), Asura_IntVector_2(3,5) - Asura_IntVector_2(2,11) );
	ASURA_UNITTEST_CHECK_EQUAL( Asura_Maths::ZeroIntVector2, Asura_IntVector_2(2,4) - Asura_IntVector_2(2,4) );
}

ASURA_UNITTEST(MinusEquals, IntVector2)
{
	Asura_IntVector_2 xTemp(-2,4);
	xTemp -= Asura_IntVector_2(1,2);
    ASURA_UNITTEST_CHECK_EQUAL( xTemp, Asura_IntVector_2(-3,2) );
}

ASURA_UNITTEST(Times, IntVector2)
{
	Asura_IntVector_2 xTemp(1,1);
	Asura_IntVector_2 xTemp2(2,2);
	ASURA_UNITTEST_CHECK_EQUAL( xTemp2, xTemp*2 );
}

ASURA_UNITTEST(TimesEquals, IntVector2)
{
	Asura_IntVector_2 xTemp(-2,4);
	xTemp *= 3;
    ASURA_UNITTEST_CHECK_EQUAL( xTemp, Asura_IntVector_2(-6,12) );
}

ASURA_UNITTEST(DivideEquals, IntVector2)
{
	Asura_IntVector_2 xTemp(-6,12);
	xTemp /= 3;
    ASURA_UNITTEST_CHECK_EQUAL( xTemp, Asura_IntVector_2(-2,4) );
}

ASURA_UNITTEST(Equality, IntVector2)
{
	Asura_IntVector_2 xTemp(-6,12);
	Asura_IntVector_2 xTemp2(-6,12);
	ASURA_UNITTEST_CHECK( xTemp == xTemp2 );
}

ASURA_UNITTEST(Inequality, IntVector2)
{
	Asura_IntVector_2 xTemp(-6,12);
	Asura_IntVector_2 xTemp2(3,12);
	ASURA_UNITTEST_CHECK( xTemp != xTemp2 );
}

// F L O A T I N G   P O I N T   E N V ///////////////////////////////////////

ASURA_UNITTEST(RoundingMode, FloatingPointEnvironment)
{
	// note that in testing these, Chop() must always round towards zero and
	// Near() may round 0.5 to 0 or 1 and 1.5 to 1 or 2. On PC, it rounds 0.5 to 0 
	// and 1.5 to 2. The function name is ambiguous enough that this shouldn't be 
	// a problem, even if other platforms always round 0.5 and 1.5 the same way.
	ASURA_UNITTEST_CHECK_EQUAL( 0,		Asura_Maths::FloatToInt_Chop( 0.5f ) );
	ASURA_UNITTEST_CHECK_EQUAL( 0,		Asura_Maths::FloatToInt_Chop( 0.999f ) );
	ASURA_UNITTEST_CHECK_EQUAL( 0,		Asura_Maths::FloatToInt_Near( 0.499f ) );
	ASURA_UNITTEST_CHECK_EQUAL( 1,		Asura_Maths::FloatToInt_Near( 0.501f ) );
	ASURA_UNITTEST_CHECK_EQUAL( 1,		Asura_Maths::FloatToInt_Chop( 1.5f ) );
	ASURA_UNITTEST_CHECK_EQUAL( 1,		Asura_Maths::FloatToInt_Chop( 1.999f ) );
	ASURA_UNITTEST_CHECK_EQUAL( 1,		Asura_Maths::FloatToInt_Near( 1.499f ) );
	ASURA_UNITTEST_CHECK_EQUAL( 2,		Asura_Maths::FloatToInt_Near( 1.501f ) );

	ASURA_UNITTEST_CHECK_EQUAL( 0,		Asura_Maths::FloatToInt_Chop( -0.5f ) );
	ASURA_UNITTEST_CHECK_EQUAL( 0,		Asura_Maths::FloatToInt_Chop( -0.999f ) );
	ASURA_UNITTEST_CHECK_EQUAL( 0,		Asura_Maths::FloatToInt_Near( -0.499f ) );
	ASURA_UNITTEST_CHECK_EQUAL( -1,		Asura_Maths::FloatToInt_Near( -0.501f ) );
	ASURA_UNITTEST_CHECK_EQUAL( -1,		Asura_Maths::FloatToInt_Chop( -1.5f ) );
	ASURA_UNITTEST_CHECK_EQUAL( -1,		Asura_Maths::FloatToInt_Chop( -1.999f ) );
	ASURA_UNITTEST_CHECK_EQUAL( -1,		Asura_Maths::FloatToInt_Near( -1.499f ) );
	ASURA_UNITTEST_CHECK_EQUAL( -2,		Asura_Maths::FloatToInt_Near( -1.501f ) );
}

ASURA_UNITTEST(Abs, Asura_Maths)
{
	ASURA_UNITTEST_CHECK_EQUAL( 0.0f, Asura_Maths::Abs( 0.0f ) );
	ASURA_UNITTEST_CHECK_EQUAL( 1.0f, Asura_Maths::Abs( 1.0f ) );
	ASURA_UNITTEST_CHECK_EQUAL( 1.0f, Asura_Maths::Abs( -1.0f ) );
	ASURA_UNITTEST_CHECK_EQUAL( 985.452f, Asura_Maths::Abs( 985.452f ) );
	ASURA_UNITTEST_CHECK_EQUAL( 985.452f, Asura_Maths::Abs( -985.452f ) );

	ASURA_UNITTEST_CHECK_EQUAL( 0, Asura_Maths::Abs( 0 ) );
	ASURA_UNITTEST_CHECK_EQUAL( 1, Asura_Maths::Abs( 1 ) );
	ASURA_UNITTEST_CHECK_EQUAL( 1, Asura_Maths::Abs( -1 ) );
	ASURA_UNITTEST_CHECK_EQUAL( 985, Asura_Maths::Abs( 985 ) );
	ASURA_UNITTEST_CHECK_EQUAL( 985, Asura_Maths::Abs( -985 ) );
}

ASURA_UNITTEST(Sign, Asura_Maths)
{
	ASURA_UNITTEST_CHECK_EQUAL( 0.0f, Asura_Maths::Sign( 0.0f ) );
	ASURA_UNITTEST_CHECK_EQUAL( 1.0f, Asura_Maths::Sign( 1.0f ) );
	ASURA_UNITTEST_CHECK_EQUAL( 1.0f, Asura_Maths::Sign( 2.0f ) );
	ASURA_UNITTEST_CHECK_EQUAL( 1.0f, Asura_Maths::Sign( 999.9f ) );
	ASURA_UNITTEST_CHECK_EQUAL( 1.0f, Asura_Maths::Sign( 1.2435345f ) );
	ASURA_UNITTEST_CHECK_EQUAL( 1.0f, Asura_Maths::Sign( Asura_Maths::LargeFloat ) );
	ASURA_UNITTEST_CHECK_EQUAL( -1.0f, Asura_Maths::Sign( -1.0f ) );
	ASURA_UNITTEST_CHECK_EQUAL( -1.0f, Asura_Maths::Sign( -2.0f ) );
	ASURA_UNITTEST_CHECK_EQUAL( -1.0f, Asura_Maths::Sign( -999.9f ) );
	ASURA_UNITTEST_CHECK_EQUAL( -1.0f, Asura_Maths::Sign( -1.2435345f ) );
	ASURA_UNITTEST_CHECK_EQUAL( -1.0f, Asura_Maths::Sign( -Asura_Maths::LargeFloat ) );

	ASURA_UNITTEST_CHECK_EQUAL( 0, Asura_Maths::Sign( 0 ) );
	ASURA_UNITTEST_CHECK_EQUAL( 1, Asura_Maths::Sign( 1 ) );
	ASURA_UNITTEST_CHECK_EQUAL( 1, Asura_Maths::Sign( 2 ) );
	ASURA_UNITTEST_CHECK_EQUAL( 1, Asura_Maths::Sign( 999 ) );
	ASURA_UNITTEST_CHECK_EQUAL( 1, Asura_Maths::Sign( 0x7fffffff ) );
	ASURA_UNITTEST_CHECK_EQUAL( -1, Asura_Maths::Sign( -1 ) );
	ASURA_UNITTEST_CHECK_EQUAL( -1, Asura_Maths::Sign( -2 ) );
	ASURA_UNITTEST_CHECK_EQUAL( -1, Asura_Maths::Sign( -999 ) );
	ASURA_UNITTEST_CHECK_EQUAL( -1, Asura_Maths::Sign( -0x7fffffff ) );
}

ASURA_UNITTEST(MinAndMax, Asura_Maths)
{
	ASURA_UNITTEST_CHECK_EQUAL( 0.0f, Asura_Maths::Min( 0.0f, 0.0f ) );
	ASURA_UNITTEST_CHECK_EQUAL( 1.0f, Asura_Maths::Min( 1.0f, 1.0f ) );
	ASURA_UNITTEST_CHECK_EQUAL( -1.0f, Asura_Maths::Min( -1.0f, -1.0f ) );
	ASURA_UNITTEST_CHECK_EQUAL( 0.0f, Asura_Maths::Min( 0.0f, 1.0f ) );
	ASURA_UNITTEST_CHECK_EQUAL( 0.0f, Asura_Maths::Min( 1.0f, 0.0f ) );
	ASURA_UNITTEST_CHECK_EQUAL( 0.0f, Asura_Maths::Min( 0.0f, 0.001f ) );
	ASURA_UNITTEST_CHECK_EQUAL( 0.0f, Asura_Maths::Min( 0.001f, 0.0f ) );
	ASURA_UNITTEST_CHECK_EQUAL( 0.001f, Asura_Maths::Min( 1.0f, 0.001f ) );
	ASURA_UNITTEST_CHECK_EQUAL( 0.001f, Asura_Maths::Min( 0.001f, 1.0f ) );
	ASURA_UNITTEST_CHECK_EQUAL( 999.8f, Asura_Maths::Min( 999.8f, 999.9f ) );
	ASURA_UNITTEST_CHECK_EQUAL( -1.0f, Asura_Maths::Min( 0.0f, -1.0f ) );
	ASURA_UNITTEST_CHECK_EQUAL( -1.0f, Asura_Maths::Min( -1.0f, 0.0f ) );
	ASURA_UNITTEST_CHECK_EQUAL( -0.001f, Asura_Maths::Min( 0.0f, -0.001f ) );
	ASURA_UNITTEST_CHECK_EQUAL( -0.001f, Asura_Maths::Min( -0.001f, 0.0f ) );
	ASURA_UNITTEST_CHECK_EQUAL( -1.0f, Asura_Maths::Min( -1.0f, -0.001f ) );
	ASURA_UNITTEST_CHECK_EQUAL( -1.0f, Asura_Maths::Min( -0.001f, -1.0f ) );
	ASURA_UNITTEST_CHECK_EQUAL( -999.9f, Asura_Maths::Min( -999.8f, -999.9f ) );

	ASURA_UNITTEST_CHECK_EQUAL( 0.0f, Asura_Maths::Max( 0.0f, 0.0f ) );
	ASURA_UNITTEST_CHECK_EQUAL( 1.0f, Asura_Maths::Max( 1.0f, 1.0f ) );
	ASURA_UNITTEST_CHECK_EQUAL( -1.0f, Asura_Maths::Max( -1.0f, -1.0f ) );
	ASURA_UNITTEST_CHECK_EQUAL( 1.0f, Asura_Maths::Max( 0.0f, 1.0f ) );
	ASURA_UNITTEST_CHECK_EQUAL( 1.0f, Asura_Maths::Max( 1.0f, 0.0f ) );
	ASURA_UNITTEST_CHECK_EQUAL( 0.001f, Asura_Maths::Max( 0.0f, 0.001f ) );
	ASURA_UNITTEST_CHECK_EQUAL( 0.001f, Asura_Maths::Max( 0.001f, 0.0f ) );
	ASURA_UNITTEST_CHECK_EQUAL( 1.0f, Asura_Maths::Max( 1.0f, 0.001f ) );
	ASURA_UNITTEST_CHECK_EQUAL( 1.0f, Asura_Maths::Max( 0.001f, 1.0f ) );
	ASURA_UNITTEST_CHECK_EQUAL( 999.9f, Asura_Maths::Max( 999.8f, 999.9f ) );
	ASURA_UNITTEST_CHECK_EQUAL( 0.0f, Asura_Maths::Max( 0.0f, -1.0f ) );
	ASURA_UNITTEST_CHECK_EQUAL( 0.0f, Asura_Maths::Max( -1.0f, 0.0f ) );
	ASURA_UNITTEST_CHECK_EQUAL( 0.0f, Asura_Maths::Max( 0.0f, -0.001f ) );
	ASURA_UNITTEST_CHECK_EQUAL( 0.0f, Asura_Maths::Max( -0.001f, 0.0f ) );
	ASURA_UNITTEST_CHECK_EQUAL( -0.001f, Asura_Maths::Max( -1.0f, -0.001f ) );
	ASURA_UNITTEST_CHECK_EQUAL( -0.001f, Asura_Maths::Max( -0.001f, -1.0f ) );
	ASURA_UNITTEST_CHECK_EQUAL( -999.8f, Asura_Maths::Max( -999.8f, -999.9f ) );

	ASURA_UNITTEST_CHECK_EQUAL( 0, Asura_Maths::Min( 0, 1 ) );
	ASURA_UNITTEST_CHECK_EQUAL( 0, Asura_Maths::Min( 1, 0 ) );
	ASURA_UNITTEST_CHECK_EQUAL( 0, Asura_Maths::Min( 0, 0 ) );
	ASURA_UNITTEST_CHECK_EQUAL( 1, Asura_Maths::Min( 1, 1 ) );
	ASURA_UNITTEST_CHECK_EQUAL( 1, Asura_Maths::Min( 1, 2 ) );
	ASURA_UNITTEST_CHECK_EQUAL( 1, Asura_Maths::Min( 2, 1 ) );
	ASURA_UNITTEST_CHECK_EQUAL( 998, Asura_Maths::Min( 998, 999 ) );
	ASURA_UNITTEST_CHECK_EQUAL( -1, Asura_Maths::Min( 0, -1 ) );
	ASURA_UNITTEST_CHECK_EQUAL( -1, Asura_Maths::Min( -1, 0 ) );
	ASURA_UNITTEST_CHECK_EQUAL( -2, Asura_Maths::Min( -2, -1 ) );
	ASURA_UNITTEST_CHECK_EQUAL( -2, Asura_Maths::Min( -1, -2 ) );
	ASURA_UNITTEST_CHECK_EQUAL( -999, Asura_Maths::Min( -998, -999 ) );

	ASURA_UNITTEST_CHECK_EQUAL( 1, Asura_Maths::Max( 0, 1 ) );
	ASURA_UNITTEST_CHECK_EQUAL( 1, Asura_Maths::Max( 1, 0 ) );
	ASURA_UNITTEST_CHECK_EQUAL( 0, Asura_Maths::Max( 0, 0 ) );
	ASURA_UNITTEST_CHECK_EQUAL( 1, Asura_Maths::Max( 1, 1 ) );
	ASURA_UNITTEST_CHECK_EQUAL( 2, Asura_Maths::Max( 1, 2 ) );
	ASURA_UNITTEST_CHECK_EQUAL( 2, Asura_Maths::Max( 2, 1 ) );
	ASURA_UNITTEST_CHECK_EQUAL( 999, Asura_Maths::Max( 998, 999 ) );
	ASURA_UNITTEST_CHECK_EQUAL( 0, Asura_Maths::Max( 0, -1 ) );
	ASURA_UNITTEST_CHECK_EQUAL( 0, Asura_Maths::Max( -1, 0 ) );
	ASURA_UNITTEST_CHECK_EQUAL( -1, Asura_Maths::Max( -2, -1 ) );
	ASURA_UNITTEST_CHECK_EQUAL( -1, Asura_Maths::Max( -1, -2 ) );
	ASURA_UNITTEST_CHECK_EQUAL( -998, Asura_Maths::Max( -998, -999 ) );
}

ASURA_UNITTEST(Power, Asura_Maths)
{
	ASURA_UNITTEST_CHECK_EQUAL( 1.0f, Asura_Maths::Power( 0.0f, 0.0f ) );

	ASURA_UNITTEST_CHECK_EQUAL( 1.0f, Asura_Maths::Power( 1.0f, 0.0f ) );
	ASURA_UNITTEST_CHECK_EQUAL( 1.0f, Asura_Maths::Power( 1.0f, 1.0f ) );
	ASURA_UNITTEST_CHECK_EQUAL( 1.0f, Asura_Maths::Power( 1.0f, 2.0f ) );

	ASURA_UNITTEST_CHECK_EQUAL( 1.0f, Asura_Maths::Power( 2.0f, 0.0f ) );
	ASURA_UNITTEST_CHECK_EQUAL( 2.0f, Asura_Maths::Power( 2.0f, 1.0f ) );
	ASURA_UNITTEST_CHECK_EQUAL( 4.0f, Asura_Maths::Power( 2.0f, 2.0f ) );
	ASURA_UNITTEST_CHECK_EQUAL( 8.0f, Asura_Maths::Power( 2.0f, 3.0f ) );

	ASURA_UNITTEST_CHECK_EQUAL( 1.0f, Asura_Maths::Power( 3.0f, 0.0f ) );
	ASURA_UNITTEST_CHECK_EQUAL( 3.0f, Asura_Maths::Power( 3.0f, 1.0f ) );
	ASURA_UNITTEST_FLOATS_EQUAL( 9.0f, Asura_Maths::Power( 3.0f, 2.0f ), fEPSILON );    //BW use fEPSILON because fastmath is turned on
	ASURA_UNITTEST_FLOATS_EQUAL( 27.0f, Asura_Maths::Power( 3.0f, 3.0f ), fEPSILON );   //BW

	ASURA_UNITTEST_FLOATS_EQUAL( 2.0f, Asura_Maths::Power( 4.0f, 1.0f / 2.0f ), 0.000001f );
	ASURA_UNITTEST_FLOATS_EQUAL( 2.0f, Asura_Maths::Power( 8.0f, 1.0f / 3.0f ), 0.000001f );
	ASURA_UNITTEST_FLOATS_EQUAL( 2.0f, Asura_Maths::Power( 16.0f, 1.0f / 4.0f) , 0.000001f );

	ASURA_UNITTEST_FLOATS_EQUAL( 3.0f, Asura_Maths::Power( 9.0f, 1.0f / 2.0f ), 0.000001f );
	ASURA_UNITTEST_FLOATS_EQUAL( 3.0f, Asura_Maths::Power( 27.0f, 1.0f / 3.0f ), 0.000001f );
	ASURA_UNITTEST_FLOATS_EQUAL( 3.0f, Asura_Maths::Power( 81.0f, 1.0f / 4.0f ), 0.000001f );

	ASURA_UNITTEST_FLOATS_EQUAL( 1.626707f, Asura_Maths::Power( 1.5f, 1.2f ), 0.000001f );
	ASURA_UNITTEST_FLOATS_EQUAL( 1.764118f, Asura_Maths::Power( 1.5f, 1.4f ), 0.000001f );
	ASURA_UNITTEST_FLOATS_EQUAL( 1.913136f, Asura_Maths::Power( 1.5f, 1.6f ), 0.000001f );
	ASURA_UNITTEST_FLOATS_EQUAL( 2.074742f, Asura_Maths::Power( 1.5f, 1.8f ), 0.000001f );

}


ASURA_UNITTEST( ClampToRange, Asura_Maths )
{
	ASURA_UNITTEST_CHECK_EQUAL( 3, Asura_Maths::ClampToRange<int>( 1,3,5) );
	ASURA_UNITTEST_CHECK_EQUAL( 20, Asura_Maths::ClampToRange<int>(192, 12, 20) );
	ASURA_UNITTEST_CHECK_EQUAL( 7, Asura_Maths::ClampToRange<int>(7, 1, 10) );
	ASURA_UNITTEST_CHECK_EQUAL( 12.0f, Asura_Maths::ClampToRange<float>(2.0f, 12.0f, 15.0f) );
	ASURA_UNITTEST_CHECK_EQUAL( 12.0f, Asura_Maths::ClampToRange<float>(22.0f, 2.0f, 12.0f) );
	ASURA_UNITTEST_CHECK_EQUAL( 12.0f, Asura_Maths::ClampToRange<float>(12.0f, 2.0f, 15.0f) );
}

ASURA_UNITTEST( SmoothStep, Asura_Maths )
{
	ASURA_UNITTEST_CHECK_EQUAL( 0.0f, Asura_Maths::SmoothStep<float>(1.0f, 2.0f, 3.0f) );
	ASURA_UNITTEST_CHECK_EQUAL( 0.0f, Asura_Maths::SmoothStep<float>(2.0f, 2.0f, 3.0f) );
	ASURA_UNITTEST_CHECK( Asura_Maths::SmoothStep<float>(2.1f, 2.0f, 3.0f) > Asura_Maths::SmoothStep<float>(2.0f, 2.0f, 3.0f));
	ASURA_UNITTEST_CHECK( Asura_Maths::SmoothStep<float>(2.2f, 2.0f, 3.0f) > Asura_Maths::SmoothStep<float>(2.1f, 2.0f, 3.0f));
	ASURA_UNITTEST_CHECK( Asura_Maths::SmoothStep<float>(2.3f, 2.0f, 3.0f) > Asura_Maths::SmoothStep<float>(2.2f, 2.0f, 3.0f));
	ASURA_UNITTEST_CHECK( Asura_Maths::SmoothStep<float>(2.4f, 2.0f, 3.0f) > Asura_Maths::SmoothStep<float>(2.3f, 2.0f, 3.0f));
	ASURA_UNITTEST_CHECK( Asura_Maths::SmoothStep<float>(2.5f, 2.0f, 3.0f) > Asura_Maths::SmoothStep<float>(2.4f, 2.0f, 3.0f));
	ASURA_UNITTEST_CHECK( Asura_Maths::SmoothStep<float>(2.6f, 2.0f, 3.0f) > Asura_Maths::SmoothStep<float>(2.5f, 2.0f, 3.0f));
	ASURA_UNITTEST_CHECK( Asura_Maths::SmoothStep<float>(2.7f, 2.0f, 3.0f) > Asura_Maths::SmoothStep<float>(2.6f, 2.0f, 3.0f));
	ASURA_UNITTEST_CHECK( Asura_Maths::SmoothStep<float>(2.8f, 2.0f, 3.0f) > Asura_Maths::SmoothStep<float>(2.7f, 2.0f, 3.0f));
	ASURA_UNITTEST_CHECK( Asura_Maths::SmoothStep<float>(2.9f, 2.0f, 3.0f) > Asura_Maths::SmoothStep<float>(2.8f, 2.0f, 3.0f));
	ASURA_UNITTEST_CHECK( Asura_Maths::SmoothStep<float>(3.0f, 2.0f, 3.0f) > Asura_Maths::SmoothStep<float>(2.9f, 2.0f, 3.0f));
	ASURA_UNITTEST_CHECK_EQUAL( 1.0f, Asura_Maths::SmoothStep<float>(3.0f, 2.0f, 3.0f) );
	ASURA_UNITTEST_CHECK_EQUAL( 1.0f, Asura_Maths::SmoothStep<float>(4.0f, 2.0f, 3.0f) );
}



ASURA_UNITTEST( Sort, Asura_Maths )
{
	int iA, iB;

	iA = 10;
	iB = -3;
	Asura_Maths::Sort( iA, iB );
	ASURA_UNITTEST_CHECK_EQUAL( iA, -3 );
	ASURA_UNITTEST_CHECK_EQUAL( iB, 10 );

	iA = 5;
	iB = 5;
	Asura_Maths::Sort( iA, iB );
	ASURA_UNITTEST_CHECK_EQUAL( iA, 5 );
	ASURA_UNITTEST_CHECK_EQUAL( iB, 5 );

	float fA, fB;

	fA = 0.0f;
	fB = 1.0f;
	Asura_Maths::Sort( fA, fB );
	ASURA_UNITTEST_FLOATS_EQUAL( fA, 0.0f, 0.000001f );
	ASURA_UNITTEST_FLOATS_EQUAL( fB, 1.0f, 0.000001f );

	fA = 100.0f;
	fB = 50.0f;
	Asura_Maths::Sort( fA, fB );
	ASURA_UNITTEST_FLOATS_EQUAL( fA, 50.0f, 0.000001f );
	ASURA_UNITTEST_FLOATS_EQUAL( fB, 100.0f, 0.000001f );
}

ASURA_UNITTEST( FtoD, Asura_Maths )
{
//	ASURA_UNITTEST_CHECK( 0.5 == Asura_Maths::FtoD( 0.5f ) );
//	ASURA_UNITTEST_CHECK( (-17.56 >= Asura_Maths::FtoD( -17.56f )-0.000001) && (-17.56 <= Asura_Maths::FtoD( -17.56f )+0.000001) );

    // (MP - 22.08.2008) FtoD() is only used for passing floats to var-arg
    // functions so let's just test that. The above tests cause compiler
    // warnings on PS2 because we're actually manipulating doubles (which is
    // a software operation there).

    char szTestBfr[32];
    float fTestValue;
    
    fTestValue = 1.1875f;
    Asura_PrintfToBfr( szTestBfr, "%6.4f", 32, true, Asura_Maths::FtoD(fTestValue) );
    ASURA_UNITTEST_CHECK( !strcmp(szTestBfr, "1.1875") );

    fTestValue = 0.0f;
    Asura_PrintfToBfr( szTestBfr, "%3.1f", 32, true, Asura_Maths::FtoD(fTestValue) );
    ASURA_UNITTEST_CHECK( !strcmp(szTestBfr, "0.0") );

    fTestValue = -8.046875f;
    Asura_PrintfToBfr( szTestBfr, "%9.6f", 32, true, Asura_Maths::FtoD(fTestValue) );
    ASURA_UNITTEST_CHECK( !strcmp(szTestBfr, "-8.046875") );
}

ASURA_UNITTEST( FloatMod, Asura_Maths )
{
	ASURA_UNITTEST_FLOATS_EQUAL( 0.3f, Asura_Maths::FloatMod( 4.3f, 1.0f ), 0.000001f );
	ASURA_UNITTEST_FLOATS_EQUAL( 0.1f, Asura_Maths::FloatMod( 5.5f, 0.2f ), 0.000001f );
}

ASURA_UNITTEST( Lerp, Asura_Maths )
{
	ASURA_UNITTEST_FLOATS_EQUAL( 1.0f, Asura_Maths::Lerp( 0.0f, 2.0f, 0.5f ), 0.000001f );
	ASURA_UNITTEST_FLOATS_EQUAL( 0.8f, Asura_Maths::Lerp( 0.0f, 1.0f, 0.8f ), 0.000001f );
}

#ifndef SN_TARGET_PS2
ASURA_UNITTEST( NaN, Asura_Maths )
{
	// These tests are slightly limited in that we need to be provided with the NaN/QNaN for it to be worth testing
	if( std::numeric_limits<float>::has_signaling_NaN )
	{
		ASURA_UNITTEST_CHECK( Asura_Maths::IsNaN( std::numeric_limits<float>::signaling_NaN() ) );
	}
	
	if( std::numeric_limits<float>::has_quiet_NaN )
	{
		ASURA_UNITTEST_CHECK( Asura_Maths::IsNaN( std::numeric_limits<float>::quiet_NaN() ) );
	}

	if( std::numeric_limits<float>::has_infinity )
	{
		ASURA_UNITTEST_CHECK( !Asura_Maths::IsNaN( std::numeric_limits<float>::infinity() ) );
	}

	// Could pick a random number here, but Asura_Random will always return the same pseudo-random number
	ASURA_UNITTEST_CHECK( !Asura_Maths::IsNaN( 0.0f ) );
	
}
#endif

// I N T E G E R   T E S T S /////////////////////////////////////////////////

ASURA_UNITTEST( FloorLog2, Asura_Maths )
{
	ASURA_UNITTEST_CHECK_EQUAL( 5, Asura_Maths::GetFloorLog2( 50 ) );
	ASURA_UNITTEST_CHECK_EQUAL( 10, Asura_Maths::GetFloorLog2( 1024 ) );
}

ASURA_UNITTEST( CeilLog2, Asura_Maths )
{
	ASURA_UNITTEST_CHECK_EQUAL( 6, Asura_Maths::GetCeilLog2( 50 ) );
}

ASURA_UNITTEST( FloorPow2, Asura_Maths )
{
	ASURA_UNITTEST_CHECK_EQUAL( 8, Asura_Maths::GetFloorPow2( 10 ) );
	ASURA_UNITTEST_CHECK_EQUAL( 1024, Asura_Maths::GetFloorPow2( 2000 ) );
}

ASURA_UNITTEST( CeilPow2, Asura_Maths )
{
	ASURA_UNITTEST_CHECK_EQUAL( 16, Asura_Maths::GetCeilPow2( 10 ) );
	ASURA_UNITTEST_CHECK_EQUAL( 256, Asura_Maths::GetCeilPow2( 129 ) );
}

// A S U R A   R A N D O M ///////////////////////////////////////////////////

ASURA_UNITTEST(Floats, Asura_Random)
{
	// JAT 08/03/07 - these tests aren't valid they can't be guaranteed
#if 0
	float fTotal = 0.0f;
	float fMin = Asura_Maths::LargeFloat;
	float fMax = -Asura_Maths::LargeFloat;

	const int iNumToGenerate = 10000;
	const float fNumGenerated = static_cast<float>(iNumToGenerate);
	const float fMinValueToGenerate = 0.1f;
	const float fMaxValueToGenerate = 999.9f;
	for (int i = 0; i < iNumToGenerate; ++i)
	{
		float fRandomFloat = Asura_Random::GetFloat(fMinValueToGenerate, fMaxValueToGenerate);
		fTotal += fRandomFloat;
		fMin = Asura_Maths::Min( fMin, fRandomFloat );
		fMax = Asura_Maths::Max( fMax, fRandomFloat );
	}

	float fMean = fTotal / fNumGenerated;
	ASURA_UNITTEST_FLOATS_EQUAL( (fMaxValueToGenerate - fMinValueToGenerate) / 2.0f, fMean, 2.0f);
	ASURA_UNITTEST_FLOATS_EQUAL( fMinValueToGenerate, fMin, 1.0f);
	ASURA_UNITTEST_FLOATS_EQUAL( fMaxValueToGenerate, fMax, 1.0f);
#endif // #if 0
}

// A N G L E T O R A N G E ///////////////////////////////////////////////////

ASURA_UNITTEST(RadiansToZeroCentredRange, AngleToRange)
{
	const float fEpsilon = 0.001f;
	const float fDelta = .1f;
	const float fPi = Asura_Maths::Pi;
	const float fLessThanPi = fPi - fDelta;
	const float fMoreThanPi = fPi + fDelta;
	// Around zero
	ASURA_UNITTEST_FLOATS_EQUAL(-fDelta,		Asura_Maths::RadiansToZeroCentredRange(-fDelta), fEpsilon);
	ASURA_UNITTEST_FLOATS_EQUAL(   0.f,			Asura_Maths::RadiansToZeroCentredRange( 0.f), fEpsilon);
	ASURA_UNITTEST_FLOATS_EQUAL( fDelta,		Asura_Maths::RadiansToZeroCentredRange( fDelta), fEpsilon);
	// Positive values around multiples of pi
	ASURA_UNITTEST_FLOATS_EQUAL( fLessThanPi,	Asura_Maths::RadiansToZeroCentredRange( fLessThanPi ), fEpsilon);
	ASURA_UNITTEST_FLOATS_EQUAL(-fPi,			Asura_Maths::RadiansToZeroCentredRange( fPi ), fEpsilon);
	ASURA_UNITTEST_FLOATS_EQUAL(-fLessThanPi,	Asura_Maths::RadiansToZeroCentredRange( fMoreThanPi ), fEpsilon);
	ASURA_UNITTEST_FLOATS_EQUAL(-fDelta,		Asura_Maths::RadiansToZeroCentredRange( fPi + fLessThanPi ), fEpsilon);
	ASURA_UNITTEST_FLOATS_EQUAL(   0.f,			Asura_Maths::RadiansToZeroCentredRange( fPi + fPi ), fEpsilon);
	ASURA_UNITTEST_FLOATS_EQUAL( fDelta,		Asura_Maths::RadiansToZeroCentredRange( fPi + fMoreThanPi ), fEpsilon);
	ASURA_UNITTEST_FLOATS_EQUAL( fLessThanPi,	Asura_Maths::RadiansToZeroCentredRange( fPi + fPi + fLessThanPi ), fEpsilon);
	
	float fPiTest = Asura_Maths::RadiansToZeroCentredRange( fPi + fPi + fPi);
	ASURA_UNITTEST_CHECK(fPiTest >= -fPi && fPiTest < fPi);
	if(fPiTest < 0)
	{
		ASURA_UNITTEST_FLOATS_EQUAL(-fPi,			fPiTest, fEpsilon);
	}else
	{
		ASURA_UNITTEST_FLOATS_EQUAL(fPi,			fPiTest, fEpsilon);
	}
	
	ASURA_UNITTEST_FLOATS_EQUAL(-fLessThanPi,	Asura_Maths::RadiansToZeroCentredRange( fPi + fPi + fMoreThanPi ), fEpsilon);
	ASURA_UNITTEST_FLOATS_EQUAL(-fDelta,		Asura_Maths::RadiansToZeroCentredRange( fPi + fPi + fPi + fLessThanPi ), fEpsilon);
	ASURA_UNITTEST_FLOATS_EQUAL(   0.f,			Asura_Maths::RadiansToZeroCentredRange( fPi + fPi + fPi + fPi), fEpsilon);
	ASURA_UNITTEST_FLOATS_EQUAL( fDelta,		Asura_Maths::RadiansToZeroCentredRange( fPi + fPi + fPi + fMoreThanPi ), fEpsilon);
	// Negative values around multiples of pi
	ASURA_UNITTEST_FLOATS_EQUAL(-fLessThanPi,	Asura_Maths::RadiansToZeroCentredRange(-fLessThanPi ), fEpsilon);
	ASURA_UNITTEST_FLOATS_EQUAL(-fPi,			Asura_Maths::RadiansToZeroCentredRange(-fPi ), fEpsilon);
	ASURA_UNITTEST_FLOATS_EQUAL( fLessThanPi,	Asura_Maths::RadiansToZeroCentredRange(-fMoreThanPi ), fEpsilon);
	ASURA_UNITTEST_FLOATS_EQUAL( fDelta,		Asura_Maths::RadiansToZeroCentredRange(-fPi - fLessThanPi ), fEpsilon);
	ASURA_UNITTEST_FLOATS_EQUAL(   0.f,			Asura_Maths::RadiansToZeroCentredRange(-fPi - fPi ), fEpsilon);
	ASURA_UNITTEST_FLOATS_EQUAL(-fDelta,		Asura_Maths::RadiansToZeroCentredRange(-fPi - fMoreThanPi ), fEpsilon);
	ASURA_UNITTEST_FLOATS_EQUAL(-fLessThanPi,	Asura_Maths::RadiansToZeroCentredRange(-fPi - fPi - fLessThanPi ), fEpsilon);
	ASURA_UNITTEST_FLOATS_EQUAL(-fPi,			Asura_Maths::RadiansToZeroCentredRange(-fPi - fPi - fPi ), fEpsilon);
	ASURA_UNITTEST_FLOATS_EQUAL( fLessThanPi,	Asura_Maths::RadiansToZeroCentredRange(-fPi - fPi - fMoreThanPi ), fEpsilon);
	ASURA_UNITTEST_FLOATS_EQUAL( fDelta,		Asura_Maths::RadiansToZeroCentredRange(-fPi - fPi - fPi - fLessThanPi ), fEpsilon);
	ASURA_UNITTEST_FLOATS_EQUAL(   0.f,			Asura_Maths::RadiansToZeroCentredRange(-fPi - fPi - fPi - fPi), fEpsilon);
	ASURA_UNITTEST_FLOATS_EQUAL(-fDelta,		Asura_Maths::RadiansToZeroCentredRange(-fPi - fPi - fPi - fMoreThanPi ), fEpsilon);
}

ASURA_UNITTEST(RadiansToZeroMinimumRange, AngleToRange)
{
	const float fEpsilon = 0.001f;
	const float fDelta = .1f;
	const float fPi = Asura_Maths::Pi;
	const float fLessThanPi = fPi - fDelta;
	const float fMoreThanPi = fPi + fDelta;
	// Around zero
	ASURA_UNITTEST_FLOATS_EQUAL( fPi + fLessThanPi,	Asura_Maths::RadiansToZeroMinimumRange(-fDelta), fEpsilon);
	ASURA_UNITTEST_FLOATS_EQUAL( 0.f,				Asura_Maths::RadiansToZeroMinimumRange( 0.f), fEpsilon);
	ASURA_UNITTEST_FLOATS_EQUAL( fDelta,			Asura_Maths::RadiansToZeroMinimumRange( fDelta), fEpsilon);
	// Positive values around multiples of pi
	ASURA_UNITTEST_FLOATS_EQUAL( fLessThanPi,		Asura_Maths::RadiansToZeroMinimumRange( fLessThanPi ), fEpsilon);
	ASURA_UNITTEST_FLOATS_EQUAL( fPi,				Asura_Maths::RadiansToZeroMinimumRange( fPi ), fEpsilon);
	ASURA_UNITTEST_FLOATS_EQUAL( fMoreThanPi,		Asura_Maths::RadiansToZeroMinimumRange( fMoreThanPi ), fEpsilon);
	ASURA_UNITTEST_FLOATS_EQUAL( fPi + fLessThanPi,	Asura_Maths::RadiansToZeroMinimumRange( fPi + fLessThanPi ), fEpsilon);
	ASURA_UNITTEST_FLOATS_EQUAL( 0.f,				Asura_Maths::RadiansToZeroMinimumRange( fPi + fPi ), fEpsilon);
	ASURA_UNITTEST_FLOATS_EQUAL( fDelta,			Asura_Maths::RadiansToZeroMinimumRange( fPi + fMoreThanPi ), fEpsilon);
	ASURA_UNITTEST_FLOATS_EQUAL( fLessThanPi,		Asura_Maths::RadiansToZeroMinimumRange( fPi + fPi + fLessThanPi ), fEpsilon);
	ASURA_UNITTEST_FLOATS_EQUAL( fPi,				Asura_Maths::RadiansToZeroMinimumRange( fPi + fPi + fPi ), fEpsilon);
	ASURA_UNITTEST_FLOATS_EQUAL( fMoreThanPi,		Asura_Maths::RadiansToZeroMinimumRange( fPi + fPi + fMoreThanPi ), fEpsilon);
	ASURA_UNITTEST_FLOATS_EQUAL( fPi + fLessThanPi,	Asura_Maths::RadiansToZeroMinimumRange( fPi + fPi + fPi + fLessThanPi ), fEpsilon);
	ASURA_UNITTEST_FLOATS_EQUAL( 0.f,				Asura_Maths::RadiansToZeroMinimumRange( fPi + fPi + fPi + fPi), fEpsilon);
	ASURA_UNITTEST_FLOATS_EQUAL( fDelta,			Asura_Maths::RadiansToZeroMinimumRange( fPi + fPi + fPi + fMoreThanPi ), fEpsilon);
	// Negative values around multiples of pi
	ASURA_UNITTEST_FLOATS_EQUAL( fMoreThanPi,		Asura_Maths::RadiansToZeroMinimumRange(-fLessThanPi ), fEpsilon);
	ASURA_UNITTEST_FLOATS_EQUAL( fPi,				Asura_Maths::RadiansToZeroMinimumRange(-fPi ), fEpsilon);
	ASURA_UNITTEST_FLOATS_EQUAL( fLessThanPi,		Asura_Maths::RadiansToZeroMinimumRange(-fMoreThanPi ), fEpsilon);
	ASURA_UNITTEST_FLOATS_EQUAL( fDelta,			Asura_Maths::RadiansToZeroMinimumRange(-fPi - fLessThanPi ), fEpsilon);
	ASURA_UNITTEST_FLOATS_EQUAL( 0.f,				Asura_Maths::RadiansToZeroMinimumRange(-fPi - fPi ), fEpsilon);
	ASURA_UNITTEST_FLOATS_EQUAL( fPi + fLessThanPi,	Asura_Maths::RadiansToZeroMinimumRange(-fPi - fMoreThanPi ), fEpsilon);
	ASURA_UNITTEST_FLOATS_EQUAL( fMoreThanPi,		Asura_Maths::RadiansToZeroMinimumRange(-fPi - fPi - fLessThanPi ), fEpsilon);
	ASURA_UNITTEST_FLOATS_EQUAL( fPi,				Asura_Maths::RadiansToZeroMinimumRange(-fPi - fPi - fPi ), fEpsilon);
	ASURA_UNITTEST_FLOATS_EQUAL( fLessThanPi,		Asura_Maths::RadiansToZeroMinimumRange(-fPi - fPi - fMoreThanPi ), fEpsilon);
	ASURA_UNITTEST_FLOATS_EQUAL( fDelta,			Asura_Maths::RadiansToZeroMinimumRange(-fPi - fPi - fPi - fLessThanPi ), fEpsilon);
	ASURA_UNITTEST_FLOATS_EQUAL( 0.f,				Asura_Maths::RadiansToZeroMinimumRange(-fPi - fPi - fPi - fPi), fEpsilon);
	ASURA_UNITTEST_FLOATS_EQUAL( fPi + fLessThanPi,	Asura_Maths::RadiansToZeroMinimumRange(-fPi - fPi - fPi - fMoreThanPi ), fEpsilon);
}

// F L O A T    A R R A Y S //////////////////////////////////////////////////

// KJS added 30/11/2005
ASURA_UNITTEST(RandomVec2, Asura_Vector_2)
{
	// Test that a set of randomly generated 2-d vectors results in a uniform spread
	// in 2*Pi & the mean magnitude is approx. correct.
	int iLoop = 100;
	float fMag = 0.0f;
	Asura_Vector_2 xVector;
	Asura_Vector_2 xVectorSum(0.0f, 0.0f);
	float afQuad[4] = {0.0f, 0.0f, 0.0f, 0.0f};

	for (int i = 0; i < iLoop; ++i)
	{
		xVector = xVector.RandomVector();
		fMag += xVector.Magnitude();
		xVectorSum = xVectorSum + xVector;
		// KJS 30/11/2005 : Quadrant Checks
		int iQuad = 0;
		if (xVector[0] > 0.0f && xVector[1] > 0.0f) iQuad=0;
		if (xVector[0] > 0.0f && xVector[1] < 0.0f) iQuad=1;
		if (xVector[0] < 0.0f && xVector[1] > 0.0f) iQuad=2;
		if (xVector[0] < 0.0f && xVector[1] < 0.0f) iQuad=3;
		++afQuad[iQuad];
	}
	// KJS 30/11/2005 : fDen = <mag> for perfectly uniform 2d-vector distbn
	float fDen = 1.0f/powf(2.0f, 1.0f/2.0f);
	fMag /= static_cast<float>(iLoop);
	ASURA_UNITTEST_FLOATS_EQUAL(fMag, fDen, 0.75f );
	// KJS 30/11/2005 : Quadrant Pop. Density Checks
	ASURA_UNITTEST_ARRAY_UNIFORM_POPULATION(afQuad, 4);
}
// V E C T O R   2 ///////////////////////////////////////////////////////////
static const Asura_Vector_2 xVec12(1.0f,2.0f);
static const Asura_Vector_2 xVec34(3.0f,4.0f);
static const Asura_Vector_2 xVec56(5.0f,6.0f);

ASURA_UNITTEST( Constructors, Asura_Vector_2 )
{
	Asura_Vector_2 xVector1( 2.0f, 1.0f );
	Asura_Vector_2 xVector2( xVector1 );
	ASURA_UNITTEST_CHECK( xVector1 == xVector2 );
}

ASURA_UNITTEST( BinaryOperators, Asura_Vector_2 )
{
	Asura_Vector_2 xVector1(2.0f,2.0f);
	ASURA_UNITTEST_CHECK( xVec56 == xVector1 + xVec34 );
	xVector1.x = 4.0f;
	xVector1.y = 6.0f;
	ASURA_UNITTEST_CHECK( xVec34 == xVector1 - xVec12 );
	xVector1.x = 1.5f;
	xVector1.y = 2.0f;
	ASURA_UNITTEST_CHECK( xVec34 == xVector1 * 2.0f );
	Asura_Vector_2 xVector2(4.0f,4.0f);
	xVector2+=xVec12;
	ASURA_UNITTEST_CHECK( xVec56 == xVector2 );
	xVector2.x = 4.0f;
	xVector2-=xVec12;
	ASURA_UNITTEST_CHECK( xVec34 == xVector2 );
	xVector2/=2.0f;
	ASURA_UNITTEST_CHECK( Asura_Vector_2(1.5f,2.0f)==xVector2 );
	ASURA_UNITTEST_CHECK( xVec34 != xVec12 );
}

ASURA_UNITTEST( DotProduct, Asura_Vector_2 )
{
	ASURA_UNITTEST_FLOATS_EQUAL( 11.0f, xVec12 * xVec34, 0.000001f );
}

ASURA_UNITTEST( Magnitude, Asura_Vector_2 )
{
	ASURA_UNITTEST_FLOATS_EQUAL( 5.0f, xVec34.Magnitude(), 0.000001f );
	ASURA_UNITTEST_FLOATS_EQUAL( 5.0f, xVec12.MagnitudeSquared(), 0.000001f );

	Asura_Vector_2 xVector1(2.0f,0.0f);
	xVector1.Normalise();

	ASURA_UNITTEST_FLOATS_EQUAL( 1.0f, xVector1.Magnitude(), 0.000001f );
}

ASURA_UNITTEST( Zero, Asura_Vector_2 )
{
	Asura_Vector_2 xVector1 = xVec56;
	xVector1.Zero();
	Asura_Vector_2 xZeroVector(0.0f,0.0f);
	ASURA_UNITTEST_CHECK( xVector1 == xZeroVector );
}

// V E C T O R   3 ///////////////////////////////////////////////////////////
static const Asura_Vector_3 xVec123(1.0f, 2.0f, 3.0f);
static const Asura_Vector_3 xVec456(4.0f, 5.0f, 6.0f);
static const Asura_Vector_3 xVec789(7.0f, 8.0f, 9.0f);
static const Asura_Vector_3 xVec246(2.0f, 4.0f, 6.0f);
static const Asura_Vector_3 xVec100(1.0f, 0.0f, 0.0f);
static const Asura_Vector_3 xVec010(0.0f, 1.0f, 0.0f);
static const Asura_Vector_3 xVec001(0.0f, 0.0f, 1.0f);
static const Asura_Vector_3 xVecMinus001(0.0f, 0.0f, -1.0f);
static const Asura_Vector_3 xVec111(1.0f, 1.0f, 1.0f);
static const Asura_Vector_3 xVecMinus111(-1.0f, -1.0f, -1.0f);
static const Asura_Vector_3 xVecMinus123(-1.0f, -2.0f, -3.0f);

ASURA_UNITTEST(Constructors, Asura_Vector_3)
{
	// from float array
	float afFloats[3] = { 1.0f, 2.0f, 3.0f };
	Asura_Vector_3 xVectorFromFloats(afFloats);
	ASURA_UNITTEST_CHECK( xVec123 == xVectorFromFloats );

	// from other vector
	Asura_Vector_3 xVectorFromOtherVec(xVec123);
	ASURA_UNITTEST_CHECK( xVec123 == xVectorFromOtherVec );

	// from 3 floats
	Asura_Vector_3 xVectorFrom3Floats(1.0f, 2.0f, 3.0f);
	ASURA_UNITTEST_CHECK( xVec123 == xVectorFrom3Floats );
}

ASURA_UNITTEST(Assignment, Asura_Vector_3)
{
	// from vector 3
	Asura_AVector3 xVector3(1.0f, 2.0f, 3.0f);
	Asura_Vector_3 xVectorFromOtherVec(0.0f, 0.0f, 0.0f);
	xVectorFromOtherVec = xVector3;
	ASURA_UNITTEST_CHECK( xVec123 == xVectorFromOtherVec );

	// from vector 4
	Asura_AVector3 xVector4(1.0f, 2.0f, 3.0f);
	Asura_Vector_3 xVectorFromVec4(0.0f, 0.0f, 0.0f);
	xVectorFromVec4 = xVector4;
	ASURA_UNITTEST_CHECK( xVec123 == xVectorFromVec4 );
}

ASURA_UNITTEST(PlusEquals, Asura_Vector_3)
{
	Asura_Vector_3 xVector;
	xVector.Zero();
	xVector += xVec100;
	ASURA_UNITTEST_CHECK( xVector == xVec100 );
	xVector += xVec010;
	xVector += xVec001;
	ASURA_UNITTEST_CHECK( xVector == xVec111 );
	xVector = xVec123;
	xVector += xVecMinus123;
	ASURA_UNITTEST_CHECK( xVector == Asura_Maths::ZeroVector3 );
}

ASURA_UNITTEST(MinusEquals, Asura_Vector_3)
{
	Asura_Vector_3 xVector;
	xVector.Zero();
	xVector -= xVec100;
	xVector -= xVec010;
	xVector -= xVec001;
	ASURA_UNITTEST_CHECK( xVector == xVecMinus111 );

	xVector = xVec123;
	xVector -= xVec123;
	ASURA_UNITTEST_CHECK( xVector == Asura_Maths::ZeroVector3 );
}

ASURA_UNITTEST(TimesEquals, Asura_Vector_3)
{
	Asura_Vector_3 xVector(xVec123);
	xVector *= 2.0f;
	ASURA_UNITTEST_CHECK( xVector == xVec246 );

	xVector = xVec123;
	xVector *= -1.0f;
	ASURA_UNITTEST_CHECK( xVector == xVecMinus123 );

	xVector = xVec123;
	xVector *= 0.0f;
	ASURA_UNITTEST_CHECK( xVector == Asura_Maths::ZeroVector3 );
}

ASURA_UNITTEST(DivideEquals, Asura_Vector_3)
{
	Asura_Vector_3 xVector(xVec246);
	xVector /= 2.0f;
	ASURA_UNITTEST_CHECK( xVector.ApproximatelyEquals(xVec123) );

	xVector = xVec123;
	xVector /= -1.0f;
	ASURA_UNITTEST_CHECK( xVector == xVecMinus123 );

	xVector = xVec123;
	xVector /= 1.0f;
	ASURA_UNITTEST_CHECK( xVector == xVec123 );
}

ASURA_UNITTEST(UnaryMinus, Asura_Vector_3)
{
	ASURA_UNITTEST_CHECK( -xVec123 == xVecMinus123 );
}

ASURA_UNITTEST(Addition, Asura_Vector_3)
{
	Asura_Vector_3 xVector;
	xVector = xVec123 + xVec123;
	ASURA_UNITTEST_CHECK( xVector == xVec246 );

	xVector = xVec123 + xVecMinus123;
	ASURA_UNITTEST_CHECK( xVector ==  Asura_Maths::ZeroVector3 );

	xVector = xVec123 + Asura_Maths::ZeroVector3;
	ASURA_UNITTEST_CHECK( xVector == xVec123 );

	xVector = xVec100 + xVec010 + xVec001;
	ASURA_UNITTEST_CHECK( xVector == xVec111 );
}

ASURA_UNITTEST(Subtraction, Asura_Vector_3)
{
	Asura_Vector_3 xVector;
	xVector = ((Asura_Maths::ZeroVector3 - xVec100) - xVec010) - xVec001;
	ASURA_UNITTEST_CHECK( xVector == xVecMinus111 );

	xVector = xVec123 - xVec123;
	ASURA_UNITTEST_CHECK( xVector == Asura_Maths::ZeroVector3 );

	xVector = xVec123 - xVecMinus123;
	ASURA_UNITTEST_CHECK( xVector == xVec246 );
}

ASURA_UNITTEST(ScalarMultiplication, Asura_Vector_3)
{
	Asura_Vector_3 xVector;
	xVector = xVec123 * 2.0f;
	ASURA_UNITTEST_CHECK( xVector.ApproximatelyEquals(xVec246) );

	xVector = xVec123 * -1.0f;
	ASURA_UNITTEST_CHECK( xVector.ApproximatelyEquals(xVecMinus123) );

	xVector = xVec123 * 0.0f;
	ASURA_UNITTEST_CHECK( xVector.ApproximatelyEquals(Asura_Maths::ZeroVector3) );
}

ASURA_UNITTEST(ScalarDivision, Asura_Vector_3)
{
	Asura_Vector_3 xVector;
	xVector = xVec246 / 2.0f;
	ASURA_UNITTEST_CHECK( xVector.ApproximatelyEquals(xVec123) );

	xVector = xVec123 / -1.0f;
	ASURA_UNITTEST_CHECK( xVector.ApproximatelyEquals(xVecMinus123) );

	xVector = xVec123 / 1.0f;
	ASURA_UNITTEST_CHECK( xVector.ApproximatelyEquals(xVec123) );
}

ASURA_UNITTEST(DotProduct, Asura_Vector_3)
{
	ASURA_UNITTEST_FLOATS_EQUAL( 0.0f, xVec001 * xVec100, 0.0f );
	ASURA_UNITTEST_FLOATS_EQUAL( 0.0f, xVec010 * xVec100, 0.0f );
	ASURA_UNITTEST_FLOATS_EQUAL( 0.0f, xVec100 * xVec010, 0.0f );
	ASURA_UNITTEST_FLOATS_EQUAL( 1.0f, xVec100 * xVec100, 0.0f );

	Asura_Vector_3 xVector(xVec456);
	xVector.Normalise();
	ASURA_UNITTEST_FLOATS_EQUAL( 1.0f, xVector * xVector, 1.0e-6f );
	ASURA_UNITTEST_FLOATS_EQUAL( 6.0f, xVec123 * xVec111, 1.0e-6f );
	ASURA_UNITTEST_FLOATS_EQUAL( 1.0f, xVec123 * xVec100, 1.0e-6f );
	ASURA_UNITTEST_FLOATS_EQUAL( 2.0f, xVec123 * xVec010, 1.0e-6f );
	ASURA_UNITTEST_FLOATS_EQUAL( 3.0f, xVec123 * xVec001, 1.0e-6f );
}

ASURA_UNITTEST(CrossProduct, Asura_Vector_3)
{
	ASURA_UNITTEST_CHECK( xVec100.ApproximatelyEquals(xVec010 ^ xVec001) );
	ASURA_UNITTEST_CHECK( xVec010.ApproximatelyEquals(xVec001 ^ xVec100) );
	ASURA_UNITTEST_CHECK( xVec001.ApproximatelyEquals(xVec100 ^ xVec010) );
	ASURA_UNITTEST_CHECK( (xVec123 ^ xVec456) == -(xVec456 ^ xVec123) );
}

ASURA_UNITTEST(Equality, Asura_Vector_3)
{
	ASURA_UNITTEST_CHECK( xVec100 == xVec100 );
	ASURA_UNITTEST_CHECK( xVec010 == xVec010 );
	ASURA_UNITTEST_CHECK( xVec001 == xVec001 );
	ASURA_UNITTEST_CHECK( !(xVec001 == xVec123) );
	ASURA_UNITTEST_CHECK( xVecMinus123 == xVecMinus123 );
	ASURA_UNITTEST_CHECK( !(xVec123 == xVecMinus123) );
}

ASURA_UNITTEST(ApproximateEquality, Asura_Vector_3)
{
	Asura_Vector_3 xVec1(0.1f, 0.2f, 0.3f);
	Asura_Vector_3 xVec2(0.1001f, 0.2001f, 0.30001f);
	Asura_Vector_3 xVec3(1.001f, 0.2001f, 0.30001f);

	ASURA_UNITTEST_CHECK( xVec1.ApproximatelyEquals(xVec2) );
	ASURA_UNITTEST_CHECK( !xVec1.ApproximatelyEquals(xVec3) );
}

ASURA_UNITTEST(Inequality, Asura_Vector_3)
{
	ASURA_UNITTEST_CHECK( !(xVec100 != xVec100) );
	ASURA_UNITTEST_CHECK( !(xVec010 != xVec010) );
	ASURA_UNITTEST_CHECK( !(xVec001 != xVec001) );
	ASURA_UNITTEST_CHECK( xVec001 != xVec123 );
	ASURA_UNITTEST_CHECK( !(xVecMinus123 != xVecMinus123) );
	ASURA_UNITTEST_CHECK( xVec123 != xVecMinus123 );
}

ASURA_UNITTEST(VectorMultiply, Asura_Vector_3)
{
	ASURA_UNITTEST_CHECK( (xVec100.Vector_Multiply( xVec100 )) == xVec100 );
	ASURA_UNITTEST_CHECK( (xVec010.Vector_Multiply( xVec010 )) == xVec010 );
	ASURA_UNITTEST_CHECK( (xVec001.Vector_Multiply( xVec001 )) == xVec001 );
	ASURA_UNITTEST_CHECK( (xVec123.Vector_Multiply( xVec001 )) == Asura_Vector_3(0.0f, 0.0f, 3.0f) );
	ASURA_UNITTEST_CHECK( (xVec123.Vector_Multiply( xVec010 )) == Asura_Vector_3(0.0f, 2.0f, 0.0f) );
	ASURA_UNITTEST_CHECK( (xVec123.Vector_Multiply( xVec100 )) == Asura_Vector_3(1.0f, 0.0f, 0.0f) );
	ASURA_UNITTEST_CHECK( (xVec123.Vector_Multiply( xVec123 )) == Asura_Vector_3(1.0f, 4.0f, 9.0f) );
}

ASURA_UNITTEST(Normalise, Asura_Vector_3)
{
	Asura_Vector_3 xVector = xVec001;
	ASURA_UNITTEST_FLOATS_EQUAL( 1.0f, xVector.Normalise(), 0.0001f) ;
	ASURA_UNITTEST_CHECK( xVector == xVec001 );
	xVector = xVec010;
	ASURA_UNITTEST_FLOATS_EQUAL( 1.0f, xVector.Normalise(), 0.0001f) ;
	ASURA_UNITTEST_CHECK( xVector == xVec010 );
	xVector = xVec100;
	ASURA_UNITTEST_FLOATS_EQUAL( 1.0f, xVector.Normalise(), 0.0001f) ;
	ASURA_UNITTEST_CHECK( xVector == xVec100 );

	xVector = Asura_Vector_3(2.0f, 0.0f, 0.0f);
	ASURA_UNITTEST_FLOATS_EQUAL( 2.0f, xVector.Normalise(), 0.0001f) ;
	ASURA_UNITTEST_CHECK( xVector == xVec100 );
	xVector = Asura_Vector_3(0.0f, 2.0f, 0.0f);
	ASURA_UNITTEST_FLOATS_EQUAL( 2.0f, xVector.Normalise(), 0.0001f) ;
	ASURA_UNITTEST_CHECK( xVector == xVec010 );
	xVector = Asura_Vector_3(0.0f, 0.0f, 2.0f);
	ASURA_UNITTEST_FLOATS_EQUAL( 2.0f, xVector.Normalise(), 0.0001f) ;
	ASURA_UNITTEST_CHECK( xVector == xVec001 );

	xVector = xVec456;
	ASURA_UNITTEST_FLOATS_EQUAL( Asura_Maths::Sqrt(16.0f + 25.0f + 36.0f), xVector.Normalise(), 0.0001f) ;
	ASURA_UNITTEST_FLOATS_EQUAL( 1.0f, xVector.Magnitude(), 0.001f );
}

ASURA_UNITTEST(Magnitude, Asura_Vector_3)
{
	ASURA_UNITTEST_FLOATS_EQUAL( 1.0f, xVec001.Magnitude(), 0.001f );
	ASURA_UNITTEST_FLOATS_EQUAL( 1.0f, xVec010.Magnitude(), 0.001f );
	ASURA_UNITTEST_FLOATS_EQUAL( 1.0f, xVec100.Magnitude(), 0.001f );
	ASURA_UNITTEST_FLOATS_EQUAL( Asura_Maths::Sqrt(3.0f), xVec111.Magnitude(), 0.001f );
	ASURA_UNITTEST_FLOATS_EQUAL( Asura_Maths::Sqrt(4.0f + 16.0f + 36.0f), xVec246.Magnitude(), 0.001f );
	ASURA_UNITTEST_FLOATS_EQUAL( Asura_Maths::Sqrt(1.0f + 4.0f + 9.0f), xVec123.Magnitude(), 0.001f );
}

ASURA_UNITTEST(MagnitudeSquared, Asura_Vector_3)
{
	ASURA_UNITTEST_FLOATS_EQUAL( 1.0f, xVec001.MagnitudeSquared(), 0.001f );
	ASURA_UNITTEST_FLOATS_EQUAL( 1.0f, xVec010.MagnitudeSquared(), 0.001f );
	ASURA_UNITTEST_FLOATS_EQUAL( 1.0f, xVec100.MagnitudeSquared(), 0.001f );
	ASURA_UNITTEST_FLOATS_EQUAL( 3.0f, xVec111.MagnitudeSquared(), 0.001f );
	ASURA_UNITTEST_FLOATS_EQUAL( (4.0f + 16.0f + 36.0f), xVec246.MagnitudeSquared(), 0.001f );
	ASURA_UNITTEST_FLOATS_EQUAL( (1.0f + 4.0f + 9.0f), xVec123.MagnitudeSquared(), 0.001f );
}

ASURA_UNITTEST( Lerp, Asura_Vector_3 )
{
	Asura_Vector_3 xStartPoint(0.0f,0.0f,0.0f);
	Asura_Vector_3 xEndPoint( 2.0f,2.0f,2.0f);
	Asura_Vector_3 xTarget( 1.0f, 1.0f, 1.0f );
	xStartPoint.Lerp( xEndPoint, 0.5f );
	ASURA_UNITTEST_CHECK( xTarget == xStartPoint );
	xStartPoint.Lerp( xTarget - xTarget, xEndPoint, 0.5f );
	ASURA_UNITTEST_CHECK( xTarget == xStartPoint );
}

ASURA_UNITTEST(Zero, Asura_Vector_3)
{
	Asura_Vector_3 xVector = xVec001;
	xVector.Zero();
	ASURA_UNITTEST_CHECK( xVector == Asura_Maths::ZeroVector3);
	xVector = xVec010;
	xVector.Zero();
	ASURA_UNITTEST_CHECK( xVector == Asura_Maths::ZeroVector3);
	xVector = xVec100;
	xVector.Zero();
	ASURA_UNITTEST_CHECK( xVector == Asura_Maths::ZeroVector3);
	xVector = xVec456;
	xVector.Zero();
	ASURA_UNITTEST_CHECK( xVector == Asura_Maths::ZeroVector3);
}

ASURA_UNITTEST(GetXYZ, Asura_Vector_3)
{
	ASURA_UNITTEST_FLOATS_EQUAL( 1.0f, xVec123.GetX(), 0.0f );
	ASURA_UNITTEST_FLOATS_EQUAL( 2.0f, xVec123.GetY(), 0.0f );
	ASURA_UNITTEST_FLOATS_EQUAL( 3.0f, xVec123.GetZ(), 0.0f );
	ASURA_UNITTEST_FLOATS_EQUAL( 4.0f, xVec456.GetX(), 0.0f );
	ASURA_UNITTEST_FLOATS_EQUAL( 5.0f, xVec456.GetY(), 0.0f );
	ASURA_UNITTEST_FLOATS_EQUAL( 6.0f, xVec456.GetZ(), 0.0f );
}

ASURA_UNITTEST(SetXYZ, Asura_Vector_3)
{
	Asura_Vector_3 xVec;

	xVec.Zero();
	xVec.SetX(1.0f);
	ASURA_UNITTEST_CHECK( xVec == xVec100 );

	xVec.Zero();
	xVec.SetY(1.0f);
	ASURA_UNITTEST_CHECK( xVec == xVec010 );

	xVec.Zero();
	xVec.SetZ(1.0f);
	ASURA_UNITTEST_CHECK( xVec == xVec001 );

	xVec.Zero();
	xVec.SetX(4.0f);
	xVec.SetY(5.0f);
	xVec.SetZ(6.0f);
	ASURA_UNITTEST_CHECK( xVec == xVec456 );
}

ASURA_UNITTEST(GetElement, Asura_Vector_3)
{
	ASURA_UNITTEST_FLOATS_EQUAL( 1.0f, xVec123.GetElement(0), 0.0f );
	ASURA_UNITTEST_FLOATS_EQUAL( 2.0f, xVec123.GetElement(1), 0.0f );
	ASURA_UNITTEST_FLOATS_EQUAL( 3.0f, xVec123.GetElement(2), 0.0f );
	ASURA_UNITTEST_FLOATS_EQUAL( 4.0f, xVec456.GetElement(0), 0.0f );
	ASURA_UNITTEST_FLOATS_EQUAL( 5.0f, xVec456.GetElement(1), 0.0f );
	ASURA_UNITTEST_FLOATS_EQUAL( 6.0f, xVec456.GetElement(2), 0.0f );
}

ASURA_UNITTEST(SetElement, Asura_Vector_3)
{
	Asura_Vector_3 xVec;

	xVec.Zero();
	xVec.SetElement(0, 1.0f);
	ASURA_UNITTEST_CHECK( xVec == xVec100 );

	xVec.Zero();
	xVec.SetElement(1, 1.0f);
	ASURA_UNITTEST_CHECK( xVec == xVec010 );

	xVec.Zero();
	xVec.SetElement(2, 1.0f);
	ASURA_UNITTEST_CHECK( xVec == xVec001 );

	xVec.Zero();
	xVec.SetElement(0, 4.0f);
	xVec.SetElement(1, 5.0f);
	xVec.SetElement(2, 6.0f);
	ASURA_UNITTEST_CHECK( xVec == xVec456 );
}

// KJS added 30/11/2005
ASURA_UNITTEST(RandomVec3, Asura_Vector_3)
{
	// Test that a set of randomly generated 2-d vectors results in a uniform spread
	// over 4*Pi & the mean magnitude is approx. correct.
	int iLoop = 100;
	float fMag = 0.0f;
	Asura_Vector_3 xVector;
	Asura_Vector_3 xVectorSum(0.0f, 0.0f, 0.0f);
	float afOct[8] = {0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f};

	for (int i = 0; i < iLoop; ++i)
	{
		xVector = xVector.RandomVector();
		fMag += xVector.Magnitude();
		xVectorSum = xVectorSum + xVector;
		// KJS 30/11/2005 : Octant Checks
		int iOct = 0;
		if (xVector[0]> 0.0f && xVector[1] > 0.0f && xVector[2] > 0.0f) iOct = 0;
		if (xVector[0]> 0.0f && xVector[1] > 0.0f && xVector[2] < 0.0f) iOct = 1;
		if (xVector[0]> 0.0f && xVector[1] < 0.0f && xVector[2] > 0.0f) iOct = 2;
		if (xVector[0]> 0.0f && xVector[1] < 0.0f && xVector[2] < 0.0f) iOct = 3;
		if (xVector[0]< 0.0f && xVector[1] > 0.0f && xVector[2] > 0.0f) iOct = 4;
		if (xVector[0]< 0.0f && xVector[1] < 0.0f && xVector[2] > 0.0f) iOct = 5;
		if (xVector[0]< 0.0f && xVector[1] > 0.0f && xVector[2] < 0.0f) iOct = 6;
		if (xVector[0]< 0.0f && xVector[1] < 0.0f && xVector[2] < 0.0f) iOct = 7;
		++afOct[iOct];
	}
	// KJS 30/11/2005 : fDen = <mag> for perfectly uniform 3d-vector distbn
	float fDen = 1.0f/powf(2.0f, 1.0f/3.0f );
	fMag /= static_cast<float>(iLoop);
	ASURA_UNITTEST_FLOATS_EQUAL(fMag, fDen, 0.75f );
	// KJS 30/11/2005 : Octant Pop. density Checks
	ASURA_UNITTEST_ARRAY_UNIFORM_POPULATION(afOct, 8 );
}

// M A T R I X   3 X 3 ///////////////////////////////////////////////////////

ASURA_UNITTEST(Constructors, Asura_Matrix_3x3)
{
	// from 9 floats
	Asura_Matrix_3x3 xMatrix(1.0f, 2.0f, 3.0f, 4.0f, 5.0f, 6.0f, 7.0f, 8.0f, 9.0f);
	ASURA_UNITTEST_CHECK( 1.0f == xMatrix.e11 );
	ASURA_UNITTEST_CHECK( 2.0f == xMatrix.e12 );
	ASURA_UNITTEST_CHECK( 3.0f == xMatrix.e13 );
	ASURA_UNITTEST_CHECK( 4.0f == xMatrix.e21 );
	ASURA_UNITTEST_CHECK( 5.0f == xMatrix.e22 );
	ASURA_UNITTEST_CHECK( 6.0f == xMatrix.e23 );
	ASURA_UNITTEST_CHECK( 7.0f == xMatrix.e31 );
	ASURA_UNITTEST_CHECK( 8.0f == xMatrix.e32 );
	ASURA_UNITTEST_CHECK( 9.0f == xMatrix.e33 );

	// from other matrix
	Asura_Matrix_3x3 xMatFromOtherMat(xMatrix);
	ASURA_UNITTEST_CHECK( xMatrix == xMatFromOtherMat );
}

ASURA_UNITTEST(Assignment, Asura_Matrix_3x3)
{
	const Asura_Matrix_3x3 xMatrix1to9(1.0f, 2.0f, 3.0f, 4.0f, 5.0f, 6.0f, 7.0f, 8.0f, 9.0f);
	Asura_Matrix_3x3 xMatrix = xMatrix1to9;
	ASURA_UNITTEST_CHECK( xMatrix1to9 == xMatrix );
}

ASURA_UNITTEST(ArrayOperators, Asura_Matrix_3x3)
{
	Asura_Matrix_3x3 xMatrix(1.0f, 2.0f, 3.0f, 4.0f, 5.0f, 6.0f, 7.0f, 8.0f, 9.0f);
	ASURA_UNITTEST_CHECK( xVec123 == xMatrix[0] );
	ASURA_UNITTEST_CHECK( xVec456 == xMatrix[1] );
	ASURA_UNITTEST_CHECK( xVec789 == xMatrix[2] );
}

ASURA_UNITTEST(Transpose, Asura_Matrix_3x3)
{
	Asura_Matrix_3x3 xMatrix(1.0f, 2.0f, 3.0f, 4.0f, 5.0f, 6.0f, 7.0f, 8.0f, 9.0f);
	xMatrix.Transpose();
	ASURA_UNITTEST_CHECK( Asura_Matrix_3x3(1.0f, 4.0f, 7.0f, 2.0f, 5.0f, 8.0f, 3.0f, 6.0f, 9.0f) == xMatrix );
	// KJS added 29/11/2005
	xMatrix.Transpose();
	ASURA_UNITTEST_CHECK( Asura_Matrix_3x3(1.0f, 2.0f, 3.0f, 4.0f, 5.0f, 6.0f, 7.0f, 8.0f, 9.0f) == xMatrix );
	
	float afRan[18] = {0.0f};
	for (int i=0; i<18; ++i)
	{
		afRan[i] = Asura_Random::GetFloat(-10.0f, 10.0f);
	}

	// Check that AB = (B^(t)A^(t))^t
	Asura_Matrix_3x3 xA(afRan[0], afRan[1], afRan[2],
						afRan[3], afRan[4], afRan[5],
						afRan[6], afRan[7], afRan[8]);
	Asura_Matrix_3x3 xB(afRan[9], afRan[10], afRan[11],
						afRan[12], afRan[13], afRan[14],
						afRan[15], afRan[16], afRan[17]);
	Asura_Matrix_3x3 xC1(0.0f, 0.0f, 0.0f,
						 0.0f, 0.0f, 0.0f,
						 0.0f, 0.0f, 0.0f);
	Asura_Matrix_3x3 xC2(0.0f, 0.0f, 0.0f,
						 0.0f, 0.0f, 0.0f,
						 0.0f, 0.0f, 0.0f);
	xC1 = xA*xB;
	xA.Transpose();
	xB.Transpose();
	xC2 = xB*xA;
	xC2.Transpose();
	ASURA_UNITTEST_CHECK( xC1 == xC2 );
}

ASURA_UNITTEST(ApplyMatrixToVector, Asura_Matrix_3x3)
{
	Asura_Matrix_3x3 xMatrix(1.0f, 2.0f, 3.0f, 4.0f, 5.0f, 6.0f, 7.0f, 8.0f, 9.0f);
	Asura_Vector_3 xVec = xVec111 * xMatrix;
	ASURA_UNITTEST_CHECK( xVec.ApproximatelyEquals(Asura_Vector_3(6.0f, 15.0f, 24.0f)) );
}

ASURA_UNITTEST(ApplyTransposeToVector, Asura_Matrix_3x3)
{
	Asura_Matrix_3x3 xMatrix(1.0f, 2.0f, 3.0f, 4.0f, 5.0f, 6.0f, 7.0f, 8.0f, 9.0f);
	Asura_Vector_3 xVec = xMatrix.ApplyTranspose(xVec111);
	ASURA_UNITTEST_CHECK( xVec.ApproximatelyEquals(Asura_Vector_3(12.0f, 15.0f, 18.0f)) );
}

ASURA_UNITTEST(Identity, Asura_Matrix_3x3)
{
	Asura_Matrix_3x3 xMatrix(1.0f, 0.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f, 0.0f, 1.0f);
	Asura_Matrix_3x3 xMatrix2;
	xMatrix2.Identity();
	ASURA_UNITTEST_CHECK( Asura_Maths::IdentityMatrix3x3 == xMatrix );
	ASURA_UNITTEST_CHECK( Asura_Maths::IdentityMatrix3x3 == xMatrix2 );
}

ASURA_UNITTEST(OriFromDirection, Asura_Matrix_3x3)
{
	Asura_Matrix_3x3 xMatrix1(-1.0f,0.0f,0.0f,0.0f,0.0f,1.0f,0.0f,1.0f,0.0f);
	Asura_Matrix_3x3 xResultMatrix;
	Asura_Vector_3 xDirection(0.0f,1.0f,0.0f);
	Asura_Maths::OrientationFromDirection( xResultMatrix, xDirection );
	ASURA_UNITTEST_CHECK( xMatrix1 == xResultMatrix );
	ASURA_UNITTEST_VECTOR3_EQUAL( ( Asura_Vector_3( xResultMatrix.e11, xResultMatrix.e21, xResultMatrix.e31 ) ^ Asura_Vector_3( xResultMatrix.e12, xResultMatrix.e22, xResultMatrix.e32 ) ), Asura_Vector_3( xResultMatrix.e13, xResultMatrix.e23, xResultMatrix.e33 ), fEPSILON );
	ASURA_UNITTEST_VECTOR3_EQUAL( ( Asura_Vector_3( xResultMatrix.e12, xResultMatrix.e22, xResultMatrix.e32 ) ^ Asura_Vector_3( xResultMatrix.e13, xResultMatrix.e23, xResultMatrix.e33 ) ), Asura_Vector_3( xResultMatrix.e11, xResultMatrix.e21, xResultMatrix.e31 ), fEPSILON );
	ASURA_UNITTEST_VECTOR3_EQUAL( ( Asura_Vector_3( xResultMatrix.e13, xResultMatrix.e23, xResultMatrix.e33 ) ^ Asura_Vector_3( xResultMatrix.e11, xResultMatrix.e21, xResultMatrix.e31 ) ), Asura_Vector_3( xResultMatrix.e12, xResultMatrix.e22, xResultMatrix.e32 ), fEPSILON );

	xDirection = Asura_Vector_3(1.0f,2.0f,3.0f);
	xDirection.Normalise();
	Asura_Vector_3 xFwd(0.0f,0.0f,1.0f);
	Asura_Maths::OrientationFromDirection( xResultMatrix, xDirection );
	Asura_Vector_3 xRotatedVector = xFwd * xResultMatrix;
	ASURA_UNITTEST_VECTOR3_EQUAL( xRotatedVector, xDirection, fEPSILON );
	ASURA_UNITTEST_VECTOR3_EQUAL( ( Asura_Vector_3( xResultMatrix.e11, xResultMatrix.e21, xResultMatrix.e31 ) ^ Asura_Vector_3( xResultMatrix.e12, xResultMatrix.e22, xResultMatrix.e32 ) ), Asura_Vector_3( xResultMatrix.e13, xResultMatrix.e23, xResultMatrix.e33 ), fEPSILON );
	ASURA_UNITTEST_VECTOR3_EQUAL( ( Asura_Vector_3( xResultMatrix.e12, xResultMatrix.e22, xResultMatrix.e32 ) ^ Asura_Vector_3( xResultMatrix.e13, xResultMatrix.e23, xResultMatrix.e33 ) ), Asura_Vector_3( xResultMatrix.e11, xResultMatrix.e21, xResultMatrix.e31 ), fEPSILON );
	ASURA_UNITTEST_VECTOR3_EQUAL( ( Asura_Vector_3( xResultMatrix.e13, xResultMatrix.e23, xResultMatrix.e33 ) ^ Asura_Vector_3( xResultMatrix.e11, xResultMatrix.e21, xResultMatrix.e31 ) ), Asura_Vector_3( xResultMatrix.e12, xResultMatrix.e22, xResultMatrix.e32 ), fEPSILON );
}

ASURA_UNITTEST(OriFromNormal, Asura_Matrix_3x3)
{
	Asura_Matrix_3x3 xResultMatrix;
	Asura_Vector_3 xNormal(1.0f,2.0f,3.0f);
	xNormal.Normalise();
	Asura_Vector_3 xUp(0.0f,-1.0f,0.0f);
	Asura_Maths::OrientationFromNormal( xResultMatrix, xNormal );
	Asura_Vector_3 xRotatedVector = xUp * xResultMatrix;
	ASURA_UNITTEST_VECTOR3_EQUAL( xRotatedVector, xNormal, fEPSILON );
	ASURA_UNITTEST_VECTOR3_EQUAL( ( Asura_Vector_3( xResultMatrix.e11, xResultMatrix.e21, xResultMatrix.e31 ) ^ Asura_Vector_3( xResultMatrix.e12, xResultMatrix.e22, xResultMatrix.e32 ) ), Asura_Vector_3( xResultMatrix.e13, xResultMatrix.e23, xResultMatrix.e33 ), fEPSILON );
	ASURA_UNITTEST_VECTOR3_EQUAL( ( Asura_Vector_3( xResultMatrix.e12, xResultMatrix.e22, xResultMatrix.e32 ) ^ Asura_Vector_3( xResultMatrix.e13, xResultMatrix.e23, xResultMatrix.e33 ) ), Asura_Vector_3( xResultMatrix.e11, xResultMatrix.e21, xResultMatrix.e31 ), fEPSILON );
	ASURA_UNITTEST_VECTOR3_EQUAL( ( Asura_Vector_3( xResultMatrix.e13, xResultMatrix.e23, xResultMatrix.e33 ) ^ Asura_Vector_3( xResultMatrix.e11, xResultMatrix.e21, xResultMatrix.e31 ) ), Asura_Vector_3( xResultMatrix.e12, xResultMatrix.e22, xResultMatrix.e32 ), fEPSILON );

	xNormal = Asura_Vector_3(3.0f,2.0f,1.0f);
	xNormal.Normalise();
	Asura_Maths::OrientationFromNormal( xResultMatrix, xNormal );
	xRotatedVector = xUp * xResultMatrix;
	ASURA_UNITTEST_VECTOR3_EQUAL( xRotatedVector, xNormal, fEPSILON );
	ASURA_UNITTEST_VECTOR3_EQUAL( ( Asura_Vector_3( xResultMatrix.e11, xResultMatrix.e21, xResultMatrix.e31 ) ^ Asura_Vector_3( xResultMatrix.e12, xResultMatrix.e22, xResultMatrix.e32 ) ), Asura_Vector_3( xResultMatrix.e13, xResultMatrix.e23, xResultMatrix.e33 ), fEPSILON );
	ASURA_UNITTEST_VECTOR3_EQUAL( ( Asura_Vector_3( xResultMatrix.e12, xResultMatrix.e22, xResultMatrix.e32 ) ^ Asura_Vector_3( xResultMatrix.e13, xResultMatrix.e23, xResultMatrix.e33 ) ), Asura_Vector_3( xResultMatrix.e11, xResultMatrix.e21, xResultMatrix.e31 ), fEPSILON );
	ASURA_UNITTEST_VECTOR3_EQUAL( ( Asura_Vector_3( xResultMatrix.e13, xResultMatrix.e23, xResultMatrix.e33 ) ^ Asura_Vector_3( xResultMatrix.e11, xResultMatrix.e21, xResultMatrix.e31 ) ), Asura_Vector_3( xResultMatrix.e12, xResultMatrix.e22, xResultMatrix.e32 ), fEPSILON );

	xNormal = Asura_Vector_3(1.0f,2.0f,-3.0f);
	xNormal.Normalise();
	Asura_Maths::OrientationFromNormal( xResultMatrix, xNormal );
	xRotatedVector = xUp * xResultMatrix;
	ASURA_UNITTEST_VECTOR3_EQUAL( xRotatedVector, xNormal, fEPSILON );
	ASURA_UNITTEST_VECTOR3_EQUAL( ( Asura_Vector_3( xResultMatrix.e11, xResultMatrix.e21, xResultMatrix.e31 ) ^ Asura_Vector_3( xResultMatrix.e12, xResultMatrix.e22, xResultMatrix.e32 ) ), Asura_Vector_3( xResultMatrix.e13, xResultMatrix.e23, xResultMatrix.e33 ), fEPSILON );
	ASURA_UNITTEST_VECTOR3_EQUAL( ( Asura_Vector_3( xResultMatrix.e12, xResultMatrix.e22, xResultMatrix.e32 ) ^ Asura_Vector_3( xResultMatrix.e13, xResultMatrix.e23, xResultMatrix.e33 ) ), Asura_Vector_3( xResultMatrix.e11, xResultMatrix.e21, xResultMatrix.e31 ), fEPSILON );
	ASURA_UNITTEST_VECTOR3_EQUAL( ( Asura_Vector_3( xResultMatrix.e13, xResultMatrix.e23, xResultMatrix.e33 ) ^ Asura_Vector_3( xResultMatrix.e11, xResultMatrix.e21, xResultMatrix.e31 ) ), Asura_Vector_3( xResultMatrix.e12, xResultMatrix.e22, xResultMatrix.e32 ), fEPSILON );
}

ASURA_UNITTEST(MatrixMultiply, Asura_Matrix_3x3)
{
	Asura_Matrix_3x3 xMatrix1(1.0f, 2.0f, 3.0f, 4.0f, 5.0f, 6.0f, 7.0f, 8.0f, 9.0f);
	Asura_Matrix_3x3 xMatrix2(1.0f, 2.0f, 3.0f, 4.0f, 5.0f, 6.0f, 7.0f, 8.0f, 9.0f);
    Asura_Matrix_3x3 xMatrix3 = xMatrix1 * xMatrix2;
	ASURA_UNITTEST_CHECK( Asura_Vector_3(30.0f, 36.0f, 42.0f).ApproximatelyEquals(xMatrix3[0]) );
	ASURA_UNITTEST_CHECK( Asura_Vector_3(66.0f, 81.0f, 96.0f).ApproximatelyEquals(xMatrix3[1]) );
	ASURA_UNITTEST_CHECK( Asura_Vector_3(102.0f, 126.0f, 150.0f).ApproximatelyEquals(xMatrix3[2]) );
}

ASURA_UNITTEST(Scaling, Asura_Matrix_3x3)
{
	Asura_Matrix_3x3 xMatrix = Asura_Matrix_3x3(1.0f, 2.0f, 3.0f, 4.0f, 5.0f, 6.0f, 7.0f, 8.0f, 9.0f) * 2.0f;
	ASURA_UNITTEST_CHECK( Asura_Vector_3(2.0f, 4.0f, 6.0f).ApproximatelyEquals(xMatrix[0]) );
	ASURA_UNITTEST_CHECK( Asura_Vector_3(8.0f, 10.0f, 12.0f).ApproximatelyEquals(xMatrix[1]) );
	ASURA_UNITTEST_CHECK( Asura_Vector_3(14.0f, 16.0f, 18.0f).ApproximatelyEquals(xMatrix[2]) );
}

ASURA_UNITTEST(Zero, Asura_Matrix_3x3)
{
	Asura_Matrix_3x3 xMatrix = Asura_Matrix_3x3(1.0f, 2.0f, 3.0f, 4.0f, 5.0f, 6.0f, 7.0f, 8.0f, 9.0f);
	xMatrix.Zero();
	for (u_int u = 0; u < 9; ++u)
	{
		ASURA_UNITTEST_CHECK_EQUAL( 0.0f, xMatrix[u/3][u%3] );
	}
}

ASURA_UNITTEST(Equality, Asura_Matrix_3x3)
{
	Asura_Matrix_3x3 xMatrix0 = Asura_Matrix_3x3(1.0f, 2.0f, 3.0f, 4.0f, 5.0f, 6.0f, 7.0f, 8.0f, 9.0f);
	Asura_Matrix_3x3 xMatrix1 = Asura_Matrix_3x3(0.0f, 2.0f, 3.0f, 4.0f, 5.0f, 6.0f, 7.0f, 8.0f, 9.0f);
	Asura_Matrix_3x3 xMatrix2 = Asura_Matrix_3x3(1.0f, 0.0f, 3.0f, 4.0f, 5.0f, 6.0f, 7.0f, 8.0f, 9.0f);
	Asura_Matrix_3x3 xMatrix3 = Asura_Matrix_3x3(1.0f, 2.0f, 0.0f, 4.0f, 5.0f, 6.0f, 7.0f, 8.0f, 9.0f);
	Asura_Matrix_3x3 xMatrix4 = Asura_Matrix_3x3(1.0f, 2.0f, 3.0f, 0.0f, 5.0f, 6.0f, 7.0f, 8.0f, 9.0f);
	Asura_Matrix_3x3 xMatrix5 = Asura_Matrix_3x3(1.0f, 2.0f, 3.0f, 4.0f, 0.0f, 6.0f, 7.0f, 8.0f, 9.0f);
	Asura_Matrix_3x3 xMatrix6 = Asura_Matrix_3x3(1.0f, 2.0f, 3.0f, 4.0f, 5.0f, 0.0f, 7.0f, 8.0f, 9.0f);
	Asura_Matrix_3x3 xMatrix7 = Asura_Matrix_3x3(1.0f, 2.0f, 3.0f, 4.0f, 5.0f, 6.0f, 0.0f, 8.0f, 9.0f);
	Asura_Matrix_3x3 xMatrix8 = Asura_Matrix_3x3(1.0f, 2.0f, 3.0f, 4.0f, 5.0f, 6.0f, 7.0f, 0.0f, 9.0f);
	Asura_Matrix_3x3 xMatrix9 = Asura_Matrix_3x3(1.0f, 2.0f, 3.0f, 4.0f, 5.0f, 6.0f, 7.0f, 8.0f, 0.0f);
	Asura_Matrix_3x3 xMatrix10 = Asura_Matrix_3x3(1.0f, 2.0f, 3.0f, 4.0f, 5.0f, 6.0f, 7.0f, 8.0f, 9.0f);
	ASURA_UNITTEST_CHECK( xMatrix0 == xMatrix0 );
	ASURA_UNITTEST_CHECK( xMatrix0 == xMatrix10 );
	ASURA_UNITTEST_CHECK( !(xMatrix0 == xMatrix1) );
	ASURA_UNITTEST_CHECK( !(xMatrix0 == xMatrix2) );
	ASURA_UNITTEST_CHECK( !(xMatrix0 == xMatrix3) );
	ASURA_UNITTEST_CHECK( !(xMatrix0 == xMatrix4) );
	ASURA_UNITTEST_CHECK( !(xMatrix0 == xMatrix5) );
	ASURA_UNITTEST_CHECK( !(xMatrix0 == xMatrix6) );
	ASURA_UNITTEST_CHECK( !(xMatrix0 == xMatrix7) );
	ASURA_UNITTEST_CHECK( !(xMatrix0 == xMatrix8) );
	ASURA_UNITTEST_CHECK( !(xMatrix0 == xMatrix9) );
}

ASURA_UNITTEST(Inquality, Asura_Matrix_3x3)
{
	Asura_Matrix_3x3 xMatrix0 = Asura_Matrix_3x3(1.0f, 2.0f, 3.0f, 4.0f, 5.0f, 6.0f, 7.0f, 8.0f, 9.0f);
	Asura_Matrix_3x3 xMatrix1 = Asura_Matrix_3x3(0.0f, 2.0f, 3.0f, 4.0f, 5.0f, 6.0f, 7.0f, 8.0f, 9.0f);
	Asura_Matrix_3x3 xMatrix2 = Asura_Matrix_3x3(1.0f, 0.0f, 3.0f, 4.0f, 5.0f, 6.0f, 7.0f, 8.0f, 9.0f);
	Asura_Matrix_3x3 xMatrix3 = Asura_Matrix_3x3(1.0f, 2.0f, 0.0f, 4.0f, 5.0f, 6.0f, 7.0f, 8.0f, 9.0f);
	Asura_Matrix_3x3 xMatrix4 = Asura_Matrix_3x3(1.0f, 2.0f, 3.0f, 0.0f, 5.0f, 6.0f, 7.0f, 8.0f, 9.0f);
	Asura_Matrix_3x3 xMatrix5 = Asura_Matrix_3x3(1.0f, 2.0f, 3.0f, 4.0f, 0.0f, 6.0f, 7.0f, 8.0f, 9.0f);
	Asura_Matrix_3x3 xMatrix6 = Asura_Matrix_3x3(1.0f, 2.0f, 3.0f, 4.0f, 5.0f, 0.0f, 7.0f, 8.0f, 9.0f);
	Asura_Matrix_3x3 xMatrix7 = Asura_Matrix_3x3(1.0f, 2.0f, 3.0f, 4.0f, 5.0f, 6.0f, 0.0f, 8.0f, 9.0f);
	Asura_Matrix_3x3 xMatrix8 = Asura_Matrix_3x3(1.0f, 2.0f, 3.0f, 4.0f, 5.0f, 6.0f, 7.0f, 0.0f, 9.0f);
	Asura_Matrix_3x3 xMatrix9 = Asura_Matrix_3x3(1.0f, 2.0f, 3.0f, 4.0f, 5.0f, 6.0f, 7.0f, 8.0f, 0.0f);
	Asura_Matrix_3x3 xMatrix10 = Asura_Matrix_3x3(1.0f, 2.0f, 3.0f, 4.0f, 5.0f, 6.0f, 7.0f, 8.0f, 9.0f);
	ASURA_UNITTEST_CHECK( !(xMatrix0 != xMatrix0) );
	ASURA_UNITTEST_CHECK( !(xMatrix0 != xMatrix10) );
	ASURA_UNITTEST_CHECK( (xMatrix0 != xMatrix1) );
	ASURA_UNITTEST_CHECK( (xMatrix0 != xMatrix2) );
	ASURA_UNITTEST_CHECK( (xMatrix0 != xMatrix3) );
	ASURA_UNITTEST_CHECK( (xMatrix0 != xMatrix4) );
	ASURA_UNITTEST_CHECK( (xMatrix0 != xMatrix5) );
	ASURA_UNITTEST_CHECK( (xMatrix0 != xMatrix6) );
	ASURA_UNITTEST_CHECK( (xMatrix0 != xMatrix7) );
	ASURA_UNITTEST_CHECK( (xMatrix0 != xMatrix8) );
	ASURA_UNITTEST_CHECK( (xMatrix0 != xMatrix9) );
}

ASURA_UNITTEST( ApproxEquals, Asura_Matrix_3x3 )
{
	Asura_Matrix_3x3 xMatrix0;
	Asura_Matrix_3x3 xIdentity;
	xIdentity.Identity();
	xMatrix0.Identity();
	xMatrix0.e11 += 0.000001f;
	ASURA_UNITTEST_CHECK( xMatrix0.ApproximatelyEquals( xIdentity ) );
}

// E U L E R   3 /////////////////////////////////////////////////////////////

// used to check 2 eulers represent the same rotation within an epsilon,
// but not necessarily the same twist.
static bool EulersRepresentsSameRotation(const Asura_Euler_3& xEuler1, const Asura_Euler_3& xEuler2)
{
	float fEpsilon = 0.001f;
	for (u_int u = 0; u < 3; ++u)
	{
		float fDiff = Asura_Maths::Abs(xEuler1.Element[u]
                                        - xEuler2.Element[u]);
		if (fDiff > (Asura_Maths::Deg360 - fEpsilon))
		{
			fDiff = Asura_Maths::Abs(fDiff - Asura_Maths::Deg360);
		}
		if (fDiff > fEpsilon)
		{
			return false;
		}
	}
	return true;
}

ASURA_UNITTEST(ToAndFromMatrix, Asura_Euler_3)
{
	Asura_Euler_3 xEulerSource1(0.0f, 0.0f, 1.0f);
	Asura_Euler_3 xEulerSource2(1.0f, 0.0f, 0.0f);
	Asura_Euler_3 xEulerSource3(0.0f, 1.0f, 0.0f);
	Asura_Euler_3 xEulerSource4(1.2f, 3.0f, -2.0f);
	Asura_Matrix_3x3 xTempMat1;
	Asura_Matrix_3x3 xTempMat2;
	Asura_Matrix_3x3 xTempMat3;
	Asura_Matrix_3x3 xTempMat4;
	Asura_Euler_3 xEulerDest1;
	Asura_Euler_3 xEulerDest2;
	Asura_Euler_3 xEulerDest3;
	Asura_Euler_3 xEulerDest4;

	xEulerSource1.ToMatrix(xTempMat1);
	xEulerSource2.ToMatrix(xTempMat2);
	xEulerSource3.ToMatrix(xTempMat3);
	xEulerSource4.ToMatrix(xTempMat4);

  	xEulerDest1.FromMatrix(xTempMat1);
  	xEulerDest2.FromMatrix(xTempMat2);
  	xEulerDest3.FromMatrix(xTempMat3);
  	xEulerDest4.FromMatrix(xTempMat4);

	ASURA_UNITTEST_CHECK( EulersRepresentsSameRotation(xEulerSource1, xEulerDest1) );
	ASURA_UNITTEST_CHECK( EulersRepresentsSameRotation(xEulerSource2, xEulerDest2) );
	ASURA_UNITTEST_CHECK( EulersRepresentsSameRotation(xEulerSource3, xEulerDest3) );
	ASURA_UNITTEST_CHECK( EulersRepresentsSameRotation(xEulerSource4, xEulerDest4) );
}

// KJS Added 29/11/2005
ASURA_UNITTEST(Orthogonal, Asura_Euler_3)
{
	// Creates a Rotation Matrix from a Euler set & checks it's orthogonal (A^(t)A=I)
	// Which it really ought to be. KJS 29/11/2005
	const float fPhi   = Asura_Maths::Pi/2.44f;
	const float fTheta = Asura_Maths::Pi/6.2f;
	const float fPsi   = Asura_Maths::Pi/3.9f;

	Asura_Euler_3 xEul1(fPhi,fTheta,fPsi);
	Asura_Matrix_3x3 xMat1;
	xEul1.ToMatrix(xMat1);
	Asura_Matrix_3x3 xMat1t(xMat1);
	xMat1t.Asura_Matrix_3x3::Transpose();

	Asura_Matrix_3x3 result = xMat1t*xMat1;
	xMat1.Identity();
	ASURA_UNITTEST_CHECK(result.ApproximatelyEquals(xMat1) );
}	

ASURA_UNITTEST(Rotate, Asura_Euler_3)
{
	const float fPhi   = Asura_Maths::Pi/2.44f;
	const float fTheta = Asura_Maths::Pi/6.2f;
	const float fPsi   = Asura_Maths::Pi/3.9f;
	// Check rotation is equal to that given by multiplying the components
	Asura_Euler_3 xEul1(fPhi,fTheta,fPsi);
	Asura_Matrix_3x3 xMat1; 
	
	xEul1.ToMatrix(xMat1);

	float fC1;
	float fC2;
	float fC3;
	float fS1;
	float fS2;
	float fS3;

	Asura_Maths::SinCos(fPhi, fS1, fC1);
	Asura_Maths::SinCos(fTheta, fS2, fC2);
	Asura_Maths::SinCos(fPsi, fS3, fC3);

	// nb. y is defined as down not up !!
	Asura_Matrix_3x3 xRot3( fC3,-fS3, 0.0,   // rot.z  (roll)
						    fS3, fC3, 0.0,
					        0.0, 0.0, 1.0);
	Asura_Matrix_3x3 xRot2( fC2, 0.0, fS2,    // rot.y  (yaw)
						    0.0, 1.0, 0.0,
						   -fS2, 0.0, fC2);
	Asura_Matrix_3x3 xRot1( 1.0, 0.0, 0.0,    // rot.x  (pitch)
						    0.0, fC1,-fS1,
						    0.0, fS1, fC1);
	
	Asura_Matrix_3x3 xProd = xRot2 * xRot1 * xRot3;

	ASURA_UNITTEST_CHECK(xProd.ApproximatelyEquals(xMat1) );
}

ASURA_UNITTEST(MultiRotate, Asura_Euler_3)
{
	// KJS 29/11/2005 Just checking lots of rotations do the right thing
	// Same as rotate but loads of times. Go from -360 to 360, y-p-r.
	float fPhi;
	float fTheta;
	float fPsi;

	float fC1;
	float fC2;
	float fC3; 
	float fS1;
	float fS2;
	float fS3;
	
	Asura_Euler_3 xEul1;
	Asura_Matrix_3x3 xRot1;
	Asura_Matrix_3x3 xRot2;
	Asura_Matrix_3x3 xRot3;
	Asura_Matrix_3x3 xProd;

	for (int i = 0; i < 100; ++i)
	{
		fPhi   = Asura_Random::GetFloat(Asura_Maths::Pi*-2.0f,Asura_Maths::Pi*2.0f);
		fTheta = Asura_Random::GetFloat(Asura_Maths::Pi*-2.0f,Asura_Maths::Pi*2.0f);
		fPsi   = Asura_Random::GetFloat(Asura_Maths::Pi*-2.0f,Asura_Maths::Pi*2.0f);

		Asura_Maths::SinCos(fPhi, fS1, fC1);
		Asura_Maths::SinCos(fTheta, fS2, fC2);
		Asura_Maths::SinCos(fPsi, fS3, fC3);

		xEul1  = Asura_Euler_3(fPhi,fTheta,fPsi);
		Asura_Matrix_3x3 xMat1; 
		xEul1.ToMatrix(xMat1);
		
		xRot3 = Asura_Matrix_3x3(fC3,-fS3, 0.0,   // rot.z  (rotate - roll)
								 fS3, fC3, 0.0,
								 0.0, 0.0, 1.0);
		xRot2 = Asura_Matrix_3x3(fC2, 0.0, fS2,    // rot.y  (rotate - yaw)
								 0.0, 1.0, 0.0,
								-fS2, 0.0, fC2);
		xRot1 = Asura_Matrix_3x3(1.0, 0.0, 0.0,    // rot.x  (rotate - pitch)
								 0.0, fC1,-fS1,
								 0.0, fS1, fC1);

		xProd = xRot2*xRot1*xRot3; 
	
		ASURA_UNITTEST_CHECK(xProd.ApproximatelyEquals(xMat1) );
	}
}

ASURA_UNITTEST( BinaryOperators, Asura_Euler_3 )
{
	Asura_Euler_3 xE1(0.0f,0.0f,0.0f);
	Asura_Euler_3 xE2(1.0f,1.0f,1.0f);
	xE1+=xE2;
	ASURA_UNITTEST_CHECK( EulersRepresentsSameRotation(xE2, xE1) );
	Asura_Euler_3 xE3(0.0f,0.0f,0.0f);
	xE1-=xE2;
	ASURA_UNITTEST_CHECK( EulersRepresentsSameRotation(xE3, xE1) );
	Asura_Euler_3 xE4(2.0f,2.0f,2.0f);
	xE2*=2.0f;
	ASURA_UNITTEST_CHECK( EulersRepresentsSameRotation(xE4, xE2) );
	xE2*=2.0f;
	xE2/=2.0f;
	ASURA_UNITTEST_CHECK( EulersRepresentsSameRotation(xE4, xE2) );
	xE2/=2.0f;
	ASURA_UNITTEST_CHECK( EulersRepresentsSameRotation(xE4, xE2 + xE2) );
	ASURA_UNITTEST_CHECK( EulersRepresentsSameRotation(xE2, xE4 - xE2) );
	ASURA_UNITTEST_CHECK( EulersRepresentsSameRotation(xE4, xE2 * 2 ) );
	ASURA_UNITTEST_CHECK( EulersRepresentsSameRotation(xE2, xE4 / 2 ) );
}

ASURA_UNITTEST( ArrayOperators, Asura_Euler_3 )
{
	Asura_Euler_3 xE1(1.0f,0.0f,2.0f);
	ASURA_UNITTEST_CHECK_EQUAL( 1.0f, xE1[0] );
	ASURA_UNITTEST_CHECK_EQUAL( 2.0f, xE1[2] );
	ASURA_UNITTEST_CHECK_EQUAL( 0.0f, xE1[1] );
	xE1[1] = 3.0f;
	ASURA_UNITTEST_CHECK_EQUAL( 3.0f, xE1[1] );
}
	
// Q U A T ///////////////////////////////////////////////////////////////////

ASURA_UNITTEST(Constructors, Asura_Quat)
{
	Asura_Quat xTempIdentity(0.0f, 0.0f, 0.0f, 1.0f);
	ASURA_UNITTEST_CHECK( Asura_Maths::IdentityQuat == xTempIdentity );

	Asura_Quat xTempFromEuler(Asura_Euler_3(0.0f,0.0f,0.0f));
	ASURA_UNITTEST_CHECK( Asura_Maths::IdentityQuat.ApproximatelyEquals(xTempFromEuler) );

	Asura_Quat xTempFromMatrix(Asura_Maths::IdentityMatrix3x3);
	ASURA_UNITTEST_CHECK( Asura_Maths::IdentityQuat == xTempFromMatrix );
}

        
ASURA_UNITTEST(Identity, Asura_Quat)
{
	Asura_Quat xTemp;
	xTemp.Identity();
	ASURA_UNITTEST_CHECK(Asura_Maths::IdentityQuat == xTemp );
}

ASURA_UNITTEST(Invert, Asura_Quat)
{
	Asura_Quat xTemp(1.0f, -2.0f, 3.0f, -0.5f);
	xTemp.Invert();
	ASURA_UNITTEST_CHECK(Asura_Quat(-1.0f, 2.0f, -3.0f, -0.5f) == xTemp );
}

ASURA_UNITTEST(Scaling, Asura_Quat)
{
  	Asura_Quat xTemp(1.0f, -2.0f, 3.0f, -0.5f);
  	xTemp = xTemp * 2.0f;
  	ASURA_UNITTEST_CHECK(Asura_Quat(2.0f, -4.0f, 6.0f, -1.0f) == xTemp );
}

ASURA_UNITTEST(RotateVector, Asura_Quat)
{
	Asura_Vector_3 xTemp = Asura_Maths::IdentityQuat.RotateVector(xVec123);
	ASURA_UNITTEST_CHECK( xVec123 == xTemp );

	xTemp = Asura_Maths::IdentityQuat.RotateVector(xVecMinus123);
	ASURA_UNITTEST_CHECK( xVecMinus123 == xTemp );

	Asura_Matrix_3x3 xMatrix;
	Asura_Quat xQuat; // kjs note : it's just rotating an axis about itself. (around z-roll)
	xQuat.FromEuler( Asura_Euler_3(0.0f, Asura_Maths::Deg90, 0.0f) );
	xTemp = xQuat.RotateVector( xVec010);
	ASURA_UNITTEST_CHECK( xVec010.ApproximatelyEquals(xTemp) );
	xTemp = xQuat.RotateVector( xVec001);
	ASURA_UNITTEST_CHECK( xVec100.ApproximatelyEquals(xTemp) );

	xQuat.FromEuler( Asura_Euler_3(-Asura_Maths::Deg90, 0.0f, 0.0f) ); // (around x-pitch)
	xTemp = xQuat.RotateVector( xVec100);
	ASURA_UNITTEST_CHECK( xVec100.ApproximatelyEquals(xTemp) );
	xTemp = xQuat.RotateVector( xVec001);
	ASURA_UNITTEST_CHECK( xVec010.ApproximatelyEquals(xTemp) );
	Asura_Euler_3(-Asura_Maths::Deg90, 0.0f, 0.0f).ToMatrix(xMatrix);
	xTemp = xVec001*xMatrix;
	ASURA_UNITTEST_CHECK( xVec010.ApproximatelyEquals(xTemp) );

	xQuat.FromEuler( Asura_Euler_3( 0.0f, 0.0f, Asura_Maths::Deg90) ); // (around y-yaw)
	xTemp = xQuat.RotateVector( xVec001);
	ASURA_UNITTEST_CHECK( xVec001.ApproximatelyEquals(xTemp) );
	xTemp = xQuat.RotateVector( xVec100);
	ASURA_UNITTEST_CHECK( xVec010.ApproximatelyEquals(xTemp) );
}

ASURA_UNITTEST(Multiply, Asura_Quat)
{
	ASURA_UNITTEST_CHECK( Asura_Maths::IdentityQuat * Asura_Maths::IdentityQuat == Asura_Maths::IdentityQuat );
	Asura_Quat xTemp1;
	Asura_Quat xTemp2;
	Asura_Quat xTemp3;
	xTemp1.FromRotation( Asura_Maths::Deg90, xVec001 );
	xTemp2.FromRotation( Asura_Maths::Deg180, xVec001 );
	xTemp3.FromRotation( -Asura_Maths::Deg90, xVec001 );
	ASURA_UNITTEST_CHECK( xTemp2.ApproximatelyEquals(xTemp1 * xTemp1) );
	ASURA_UNITTEST_CHECK( Asura_Maths::IdentityQuat.ApproximatelyEquals(xTemp1 * xTemp3) );
	ASURA_UNITTEST_CHECK( xTemp1.ApproximatelyEquals(Asura_Maths::IdentityQuat * xTemp1) );
	ASURA_UNITTEST_CHECK( xTemp1.ApproximatelyEquals(xTemp1 * Asura_Maths::IdentityQuat) );
}

ASURA_UNITTEST(Negate, Asura_Quat)
{
	Asura_Quat xTemp(1.0f, -2.0f, 3.0f, -4.0f);
	xTemp = -xTemp;
	ASURA_UNITTEST_CHECK(Asura_Quat(-1.0f, 2.0f, -3.0f, 4.0f) == xTemp );
}

ASURA_UNITTEST(Add, Asura_Quat)
{
	Asura_Quat xTemp1(1.0f, 2.0f, 3.0f, 0.5f);
	Asura_Quat xTemp2(-1.0f, -2.0f, -3.0f, -0.5f);
	Asura_Quat xTemp3(2.0f, 4.0f, 6.0f, 1.0f);
	Asura_Quat xTemp4(0.0f, 0.0f, 0.0f, 0.0f);
	ASURA_UNITTEST_CHECK(xTemp3 == xTemp1 + xTemp1);
	ASURA_UNITTEST_CHECK(xTemp4 == xTemp1 + xTemp2);
}

ASURA_UNITTEST(ApproxEquals, Asura_Quat)
{
	Asura_Quat xTemp1(1.0f, 2.0f, 3.0f, 0.5f);
	Asura_Quat xTemp2(-1.0f, -2.0f, -3.0f, -0.5f);
	Asura_Quat xTemp3(2.0f, 3.0f, 5.0f, 0.8f);
	xTemp1.Normalise();
	xTemp2.Normalise();
	xTemp3.Normalise();
	ASURA_UNITTEST_CHECK( xTemp1.ApproximatelyEquals(xTemp2) );
	ASURA_UNITTEST_CHECK( !xTemp1.ApproximatelyEquals(xTemp3) );
	ASURA_UNITTEST_CHECK( !xTemp2.ApproximatelyEquals(xTemp3) );
	ASURA_UNITTEST_CHECK( xTemp1.ApproximatelyEquals(xTemp1) );
	ASURA_UNITTEST_CHECK( xTemp2.ApproximatelyEquals(xTemp2) );
	ASURA_UNITTEST_CHECK( !Asura_Maths::IdentityQuat.ApproximatelyEquals(xTemp1) );
	ASURA_UNITTEST_CHECK( !Asura_Maths::IdentityQuat.ApproximatelyEquals(xTemp2) );
	ASURA_UNITTEST_CHECK( !Asura_Maths::IdentityQuat.ApproximatelyEquals(xTemp3) );
}

ASURA_UNITTEST(Equality, Asura_Quat)
{
	Asura_Quat xTemp1(1.0f, 2.0f, 3.0f, 0.5f);
	Asura_Quat xTemp2(-1.0f, -2.0f, -3.0f, -0.5f);
	Asura_Quat xTemp3(2.0f, 4.0f, 6.0f, 1.0f);
	ASURA_UNITTEST_CHECK( xTemp1 == xTemp1 );
	ASURA_UNITTEST_CHECK( xTemp2 == xTemp2 );
	ASURA_UNITTEST_CHECK( xTemp3 == xTemp3 );
	ASURA_UNITTEST_CHECK( !(xTemp1 == xTemp2) );
	ASURA_UNITTEST_CHECK( !(xTemp1 == xTemp3) );
	ASURA_UNITTEST_CHECK( !(xTemp2 == xTemp3) );
	ASURA_UNITTEST_CHECK( xTemp1 == xTemp1 );
	ASURA_UNITTEST_CHECK( xTemp2 == xTemp2 );
	ASURA_UNITTEST_CHECK( !(Asura_Maths::IdentityQuat == xTemp1) );
	ASURA_UNITTEST_CHECK( !(Asura_Maths::IdentityQuat == xTemp2) );
	ASURA_UNITTEST_CHECK( !(Asura_Maths::IdentityQuat == xTemp3) );
}

ASURA_UNITTEST(Inequality, Asura_Quat )
{
	Asura_Quat xTemp1(1.0f, 2.0f, 3.0f, 0.5f);
	Asura_Quat xTemp2(-1.0f, 2.0f, 3.0f, 0.5f);
	ASURA_UNITTEST_CHECK( xTemp1 != xTemp2 );
}

ASURA_UNITTEST(Magnitude, Asura_Quat)
{
	Asura_Quat xTemp1(4.0f, 2.0f, 3.0f, 1.0f);
	ASURA_UNITTEST_CHECK( xTemp1.MagnitudeSquared() == 30.0f );
	ASURA_UNITTEST_CHECK( Asura_Maths::IdentityQuat.MagnitudeSquared() == 1.0f );
}


ASURA_UNITTEST(ToAndFromMatrix, Asura_Quat)
{
	Asura_Quat xQuat1(1.0f, 2.0f, 3.0f, 0.0f);
	Asura_Quat xQuat2(2.0f, 3.0f, 5.0f, -1.0f);
	Asura_Quat xQuat3(3.0f, 2.0f, 3.0f, 1.0f);
	Asura_Quat xQuat4(1.0f, -2.0f, 4.0f, -0.2f);

	xQuat1.Normalise();
	xQuat2.Normalise();
	xQuat3.Normalise();
	xQuat4.Normalise();

	Asura_Quat xTempQuat1;
	Asura_Quat xTempQuat2;
	Asura_Quat xTempQuat3;
	Asura_Quat xTempQuat4;
	
	{
		Asura_Matrix_3x3 xTempMat1;
		Asura_Matrix_3x3 xTempMat2;
		Asura_Matrix_3x3 xTempMat3;
		Asura_Matrix_3x3 xTempMat4;

		xQuat1.ToMatrix(xTempMat1);
		xQuat2.ToMatrix(xTempMat2);
		xQuat3.ToMatrix(xTempMat3);
		xQuat4.ToMatrix(xTempMat4);
		
		xTempQuat1.FromMatrix(xTempMat1);
		xTempQuat2.FromMatrix(xTempMat2);
		xTempQuat3.FromMatrix(xTempMat3);
		xTempQuat4.FromMatrix(xTempMat4);

		ASURA_UNITTEST_CHECK( xTempQuat1.ApproximatelyEquals(xQuat1) );
		ASURA_UNITTEST_CHECK( xTempQuat2.ApproximatelyEquals(xQuat2) );
		ASURA_UNITTEST_CHECK( xTempQuat3.ApproximatelyEquals(xQuat3) );
		ASURA_UNITTEST_CHECK( xTempQuat4.ApproximatelyEquals(xQuat4) );
	}

	{
		Asura_AMatrix3x3 xTempMat1;
		Asura_AMatrix3x3 xTempMat2;
		Asura_AMatrix3x3 xTempMat3;
		Asura_AMatrix3x3 xTempMat4;

		xQuat1.ToMatrix(xTempMat1);
		xQuat2.ToMatrix(xTempMat2);
		xQuat3.ToMatrix(xTempMat3);
		xQuat4.ToMatrix(xTempMat4);
		
		xTempQuat1.FromMatrix(xTempMat1);
		xTempQuat2.FromMatrix(xTempMat2);
		xTempQuat3.FromMatrix(xTempMat3);
		xTempQuat4.FromMatrix(xTempMat4);

		ASURA_UNITTEST_CHECK( xTempQuat1.ApproximatelyEquals(xQuat1) );
		ASURA_UNITTEST_CHECK( xTempQuat2.ApproximatelyEquals(xQuat2) );
		ASURA_UNITTEST_CHECK( xTempQuat3.ApproximatelyEquals(xQuat3) );
		ASURA_UNITTEST_CHECK( xTempQuat4.ApproximatelyEquals(xQuat4) );
	}
}

ASURA_UNITTEST(ToAndFromEuler, Asura_Quat)
{
	Asura_Quat xQuat1(1.0f, 2.0f, 3.0f, 0.0f);
	Asura_Quat xQuat2(2.0f, 3.0f, 5.0f, -1.0f);
	Asura_Quat xQuat3(3.0f, 2.0f, 3.0f, 1.0f);
	Asura_Quat xQuat4(1.0f, -2.0f, 4.0f, -0.2f);

	xQuat1.Normalise();
	xQuat2.Normalise();
	xQuat3.Normalise();
	xQuat4.Normalise();

	Asura_Euler_3 xTempEuler1;
	Asura_Euler_3 xTempEuler2;
	Asura_Euler_3 xTempEuler3;
	Asura_Euler_3 xTempEuler4;
	Asura_Quat xTempQuat1;
	Asura_Quat xTempQuat2;
	Asura_Quat xTempQuat3;
	Asura_Quat xTempQuat4;
	
	xQuat1.ToEuler(xTempEuler1);
	xQuat2.ToEuler(xTempEuler2);
	xQuat3.ToEuler(xTempEuler3);
	xQuat4.ToEuler(xTempEuler4);
	
	xTempQuat1.FromEuler(xTempEuler1);
	xTempQuat2.FromEuler(xTempEuler2);
	xTempQuat3.FromEuler(xTempEuler3);
	xTempQuat4.FromEuler(xTempEuler4);

	ASURA_UNITTEST_CHECK( xTempQuat1.ApproximatelyEquals(xQuat1) );
	ASURA_UNITTEST_CHECK( xTempQuat2.ApproximatelyEquals(xQuat2) );
	ASURA_UNITTEST_CHECK( xTempQuat3.ApproximatelyEquals(xQuat3) );
	ASURA_UNITTEST_CHECK( xTempQuat4.ApproximatelyEquals(xQuat4) );
}


ASURA_UNITTEST(FromRotation, Asura_Quat)
{
	// x,y,z are from axis * sin(angle/2), w = cos(angle/2)
	Asura_Quat xQuat;
	Asura_Vector_3 xTempVec1(xVec111);
	Asura_Vector_3 xTempVec2(xVec123);
	xTempVec1.Normalise();
	xTempVec2.Normalise();
	xQuat.FromRotation(0.0f, xTempVec1);
	ASURA_UNITTEST_CHECK( xQuat.ApproximatelyEquals(Asura_Maths::IdentityQuat) );
	xQuat.FromRotation(Asura_Maths::Deg90, xTempVec2);
	float fSin45 = Asura_Maths::Sqrt(0.5f);
	float fCos45 = fSin45;
	float fInvMag123 = 1.0f / Asura_Maths::Sqrt(14.0f);
	Asura_Quat xQuat2(1.0f * fSin45 * fInvMag123, 2.0f * fSin45 * fInvMag123, 3.0f * fSin45 * fInvMag123, fCos45);
	ASURA_UNITTEST_CHECK( xQuat.ApproximatelyEquals(xQuat2) );
}

ASURA_UNITTEST(ToRotation, Asura_Quat)
{
    Asura_Vector_3 xAxisIn;
    Asura_Vector_3 xAxisOut;
    Asura_Angle    fAngleOut;
    Asura_Quat     xQuat;

    xAxisIn = xVec123;
    xAxisIn.Normalise();
    for ( Asura_Angle fAngleIn = 0.0f; fAngleIn < Asura_Maths::Deg180; fAngleIn += Asura_Maths::Deg180 / 360.0f )
    {
        xQuat.FromRotation( fAngleIn, xAxisIn );
        fAngleOut = xQuat.ToRotation( xAxisOut );
        if ( fAngleIn != 0.0f )
        {
            // Zero angle has an arbitrary axis.
            ASURA_UNITTEST_VECTOR3_EQUAL( xAxisIn, xAxisOut, fEPSILON );
        }
        ASURA_UNITTEST_FLOATS_EQUAL( fAngleIn, fAngleOut, fEPSILON_LARGE );
    }

    xQuat.FromRotation( Asura_Maths::Deg180, xVec001 );
    fAngleOut = xQuat.ToRotation( xAxisOut );
    ASURA_UNITTEST_VECTOR3_EQUAL( xVec001, xAxisOut, fEPSILON );
    ASURA_UNITTEST_FLOATS_EQUAL( Asura_Maths::Deg180, Asura_Maths::Abs(fAngleOut), fEPSILON );
}

ASURA_UNITTEST(RotationFromAToB, Asura_Quat)
{
    Asura_Vector_3 xAxisOut;
    Asura_Angle    fAngleOut;
    Asura_Quat     xQuat;
    Asura_Quat     xQuat2;
    Asura_Vector_3 xCross;

    xQuat.RotationFromAToB( xVec001, xVec100 );
    xCross = xVec001 ^ xVec100;
    xQuat2.FromRotation( Asura_Maths::Deg90, xCross );
    ASURA_UNITTEST_CHECK( xQuat.ApproximatelyEquals(xQuat2) );	 //BW fastmath means not ==

    xQuat.RotationFromAToB( xVec001, -xVec001 );
    fAngleOut = xQuat.ToRotation( xAxisOut );
    // Axis is arbitrary.
    ASURA_UNITTEST_FLOATS_EQUAL( Asura_Maths::Deg180, fAngleOut, fEPSILON );

	Asura_Vector_3	xVec1( 1.0f, 2.0f, 3.0f );
	xVec1.Normalise();
	Asura_Vector_3	xVec2( 4.0f, 5.0f, 6.0f );
	xVec2.Normalise();
	xQuat.RotationFromAToB( xVec1, xVec2 );

	Asura_Vector_3 xVecOut = xQuat.RotateVector( xVec1 );
	ASURA_UNITTEST_CHECK( xVecOut.ApproximatelyEquals(xVec2) );	 //BW fastmath means not ==
}

ASURA_UNITTEST(Normalise, Asura_Quat)
{
	Asura_Quat xQuat(4.0f, 2.0f, 3.0f, 1.0f);
	xQuat.Normalise();
	ASURA_UNITTEST_FLOATS_EQUAL( xQuat.MagnitudeSquared(), 1.0f, fEPSILON );

	xQuat = Asura_Maths::IdentityQuat;
	xQuat.Normalise();
	ASURA_UNITTEST_CHECK( Asura_Maths::IdentityQuat.ApproximatelyEquals(xQuat) );
}

ASURA_UNITTEST(Dot, Asura_Quat)
{
	// just check the maths comes out right. 
	// in practice, quats will be normalised.
	Asura_Quat xQuat1(1.0f, 2.0f, 3.0f, 0.0f);
	Asura_Quat xQuat2(2.0f, 3.0f, 5.0f, -1.0f);
	Asura_Quat xQuat3(1.0f, 2.0f, 3.0f, 1.0f);
	ASURA_UNITTEST_CHECK( xQuat1.Dot(xQuat2) == (2.0f + 6.0f + 15.0f) );
	ASURA_UNITTEST_CHECK( xQuat2.Dot(xQuat3) == (2.0f + 6.0f + 15.0f - 1.0f) );
}

ASURA_UNITTEST(Slerp, Asura_Quat)
{
	Asura_Quat xQuat;
	Asura_Quat xQuat1(2.0f, 3.0f, 5.0f, -1.0f);
	Asura_Quat xQuat2(1.0f, 2.0f, 3.0f, 1.0f);
	xQuat1.Normalise();
	xQuat2.Normalise();

	xQuat.Slerp( xQuat1, xQuat2, 0.0f );
	ASURA_UNITTEST_CHECK( xQuat.ApproximatelyEquals(xQuat1) );
	xQuat.Slerp( xQuat1, xQuat2, 1.0f );
	ASURA_UNITTEST_CHECK( xQuat.ApproximatelyEquals(xQuat2) );

	Asura_Quat xQuat3;
	Asura_Quat xQuat4;
	Asura_Quat xQuat5;
	xQuat3.FromRotation( Asura_Maths::Deg90, xVec001 );
	xQuat4.FromRotation( Asura_Maths::Deg90 * 0.9f, xVec001 );
	xQuat5.FromRotation( Asura_Maths::Deg90 * 1.1f, xVec001 );
	xQuat3.Normalise();
	xQuat4.Normalise();
	xQuat5.Normalise();
	xQuat.Slerp( xQuat4, xQuat5, 0.5f );
	ASURA_UNITTEST_CHECK( xQuat.ApproximatelyEquals(xQuat3) );

	// do it the other way around to check the shortest path code is working
	xQuat.Slerp( -xQuat4, xQuat5, 0.5f );
	ASURA_UNITTEST_CHECK( xQuat.ApproximatelyEquals(xQuat3) );

}

ASURA_UNITTEST(NLerp, Asura_Quat)
{
	Asura_Quat xQuat;
	Asura_Quat xQuat1(2.0f, 3.0f, 5.0f, -1.0f);
	Asura_Quat xQuat2(1.0f, 2.0f, 3.0f, 1.0f);
	xQuat1.Normalise();
	xQuat2.Normalise();

	xQuat.NLerp( xQuat1, xQuat2, 0.0f );
	ASURA_UNITTEST_CHECK( xQuat.ApproximatelyEquals(xQuat1) );
	xQuat.NLerp( xQuat1, xQuat2, 1.0f );
	ASURA_UNITTEST_CHECK( xQuat.ApproximatelyEquals(xQuat2) );

	Asura_Quat xQuat3;
	Asura_Quat xQuat4;
	Asura_Quat xQuat5;
	Asura_Quat xQuat6;
	xQuat3.FromRotation( Asura_Maths::Deg90, xVec001 );
	xQuat4.FromRotation( Asura_Maths::Deg90 * 0.9f, xVec001 );
	xQuat5.FromRotation( Asura_Maths::Deg90 * 1.1f, xVec001 );
	xQuat6.FromRotation( Asura_Maths::Deg90, xVecMinus001 );
	xQuat3.Normalise();
	xQuat4.Normalise();
	xQuat5.Normalise();
	xQuat6.Normalise();
	xQuat.NLerp( xQuat4, xQuat5, 0.5f );
	ASURA_UNITTEST_CHECK( xQuat.ApproximatelyEquals(xQuat3) );

	// do it the other way around to check the shortest path code is working
	xQuat.NLerp( -xQuat4, xQuat5, 0.5f );
	ASURA_UNITTEST_CHECK( xQuat.ApproximatelyEquals(xQuat3) );

	// check that if we have quats pointing 180 degrees away from each other (cos(o) == 0)
	// that the 0 and 1 cases still work.
	xQuat.NLerp( xQuat3, xQuat6, 0.0f );
	ASURA_UNITTEST_CHECK( xQuat.ApproximatelyEquals(xQuat3) );

	xQuat.NLerp( xQuat3, xQuat6, 1.0f );
	ASURA_UNITTEST_CHECK( xQuat.ApproximatelyEquals(xQuat6) );
}

// KJS 5/12/2005
ASURA_UNITTEST(Pak, Asura_Quat)
{
	Asura_Quat xQ1(1.5f,0.2f,0.4f,0.2f);
	Asura_Quat xQC = xQ1;
	xQC.Normalise();
	xQ1.Normalise();
	Asura_Comp_Quat xCQ1(xQ1);
	ASURA_UNITTEST_CHECK( xQ1.ApproximatelyEquals(xQC)); // Chk compression leaves Q1 intact
	xCQ1.ToQuat(xQC); // uncompress into xQC then check it is still the same as xQ1
	ASURA_UNITTEST_CHECK( xQ1.ApproximatelyEquals(xQC,0.1e-2f));

	Asura_Vector_3 xInit(2.4f,2.2f,7.7f);
	Asura_Vector_3 xRot1 = xQ1.RotateVector(xInit);
	Asura_Vector_3 xRot2 = xQC.RotateVector(xInit);
	ASURA_UNITTEST_CHECK( xRot1.ApproximatelyEquals(xRot2,0.01f));
}

ASURA_UNITTEST(FastPak, Asura_Quat)
{
	Asura_Quat xQ1(1.5f,0.2f,0.4f,0.2f);
	Asura_Quat xQC = xQ1;
	xQC.Normalise();
	xQ1.Normalise();
	Asura_Comp_Quat_Fast xCQ1(xQ1);
	ASURA_UNITTEST_CHECK( xQ1.ApproximatelyEquals(xQC)); // Chk compression leaves Q1 intact
	xCQ1.ToQuat(xQC); // uncompress into xQC then check it is still the same as xQ1
	ASURA_UNITTEST_CHECK( xQ1.ApproximatelyEquals(xQC));

	Asura_Vector_3 xInit(2.4f,2.2f,7.7f);
	Asura_Vector_3 xRot1 = xQ1.RotateVector(xInit);
	Asura_Vector_3 xRot2 = xQC.RotateVector(xInit);
	ASURA_UNITTEST_CHECK( xRot1.ApproximatelyEquals(xRot2,0.01f));
}

// COMPLEX NUMBERS 
// KJS 5/12/2005
ASURA_UNITTEST(Constructors, Asura_Complex)
{
	Asura_Complex xC1;
	xC1.SetR(3.2f);
	xC1.SetI(4.1f);
	ASURA_UNITTEST_CHECK(xC1.GetR() == 3.2f && xC1.GetI() == 4.1f );
	Asura_Complex xC2(2.0f,5.0f);
	ASURA_UNITTEST_CHECK(xC2.GetR() == 2.0f && xC2.GetI() == 5.0f );
	Asura_Complex xC3(xC2);
	ASURA_UNITTEST_CHECK(xC3.GetR() == 2.0f && xC3.GetI() == 5.0f );
	Asura_Complex xC4 = xC2;
	ASURA_UNITTEST_CHECK(xC4.GetR() == 2.0f && xC4.GetI() == 5.0f );
}

	
ASURA_UNITTEST(Add, Asura_Complex)
{
	Asura_Complex xC1(1.0f, 3.0f);
	Asura_Complex xC2(2.0f, 4.0f);
	Asura_Complex xC3;

	xC3 = xC1 + xC2;

	ASURA_UNITTEST_CHECK(xC3.GetR() == 3.0f && xC3.GetI() == 7.0f );
}

ASURA_UNITTEST(AddEquals, Asura_Complex)
{
	Asura_Complex xC1(1.0f,3.0f);
	Asura_Complex xC2(2.0f,4.0f);

	xC2 += xC1;

	ASURA_UNITTEST_CHECK(xC2.GetR() == 3.0f && xC2.GetI() == 7.0f );
}

ASURA_UNITTEST(Subtract, Asura_Complex)
{
	Asura_Complex xC1(1.0f, 3.0f);
	Asura_Complex xC2(2.0f, 4.0f);
	Asura_Complex xC3;

	xC3 = xC1 - xC2;

	ASURA_UNITTEST_CHECK(xC3.GetR() == -1.0f && xC3.GetI() == -1.0f );
}

ASURA_UNITTEST(SubtractEquals, Asura_Complex)
{
	Asura_Complex xC1(1.0f, 3.0f);
	Asura_Complex xC2(2.0f, 4.0f);

	xC2 -= xC1;

	ASURA_UNITTEST_CHECK(xC2.GetR() == 1.0f && xC2.GetI() == 1.0f );
}

ASURA_UNITTEST(Multiply, Asura_Complex)
{
	Asura_Complex xC1(1.0f, 3.0f);
	Asura_Complex xC2(2.0f, 4.0f);
	Asura_Complex xC3;

	xC3 = xC1 * xC2;

	ASURA_UNITTEST_CHECK(xC3.GetR() == -10.0f && xC3.GetI() == 10.0f );
}

ASURA_UNITTEST(MultiplyEquals, Asura_Complex)
{
	Asura_Complex xC1(1.0f, 3.0f);
	Asura_Complex xC2(2.0f, 4.0f);

	xC2 *= xC1;

	ASURA_UNITTEST_CHECK(xC2.GetR() == -10.0f && xC2.GetI() == 10.0f );
}

ASURA_UNITTEST(Conjugate, Asura_Complex)
{
	Asura_Complex xC1(1.0f, 3.0f); 
	xC1.Conjugate();
	ASURA_UNITTEST_CHECK(xC1.GetR() == 1.0f && xC1.GetI() == -3.0f );

	Asura_Complex xC2(-5.3f, -4.2f);
	Asura_Complex xC3 = xC2.GetConjugate();
	ASURA_UNITTEST_CHECK(xC2.GetR() == -5.3f && xC2.GetI() == -4.2f );
	ASURA_UNITTEST_CHECK(xC3.GetR() == -5.3f && xC3.GetI() == 4.2f );
}

// Bounding Box Checks : KJS 7/12/2005

ASURA_UNITTEST(Assignment, Bounding_Box)
{
	Asura_Bounding_Box xBox1;
	Asura_Vector_3 xVec1(2.0f, 5.0f, 3.0f); // xyz
	Asura_Vector_3 xVec2(1.0f, 10.0f, 1.0f);
	Asura_Vector_3 xVec3(2.5f, 7.5f, 2.0f);

	xBox1.Reset();
	xBox1.Set(xVec1);
	xBox1.Set(xVec2);


	ASURA_UNITTEST_CHECK(xBox1.MinX == 1.0f && xBox1.MaxX == 2.0f
					  && xBox1.MinY == 5.0f && xBox1.MaxY == 10.0f
					  && xBox1.MinZ == 1.0f && xBox1.MaxZ ==  3.0f );

	Asura_Bounding_Box xBox2;
	xBox2.Reset();
	xBox2.SetToPoint(xVec3);

	ASURA_UNITTEST_CHECK(xBox2.MinX == 2.5f && xBox2.MaxX == 2.5f
					  && xBox2.MinY == 7.5f && xBox2.MaxY == 7.5f
					  && xBox2.MinZ == 2.0f && xBox2.MaxZ == 2.0f );

	Asura_Bounding_Box xBox3;
	xBox3.Reset();
	xBox3.Set(xBox1);

	ASURA_UNITTEST_CHECK(xBox3.MinX == 1.0f && xBox3.MaxX == 2.0f
					  && xBox3.MinY == 5.0f && xBox3.MaxY == 10.0f
					  && xBox3.MinZ == 1.0f && xBox3.MaxZ == 3.0f );

	xBox2.Set(xBox1);

	ASURA_UNITTEST_CHECK(xBox2.MinX == 1.0f && xBox2.MaxX == 2.5f
					  && xBox2.MinY == 5.0f && xBox2.MaxY == 10.0f
					  && xBox2.MinZ == 1.0f && xBox2.MaxZ == 3.0f );

}

ASURA_UNITTEST(Operators, Bounding_Box)
{
	Asura_Vector_3 xVec1(8.0f, 11.0f, 9.0f );
	Asura_Bounding_Box xBox1;
	xBox1.SetToPoint(xVec1);
	xBox1.Grow(4.0f); 
	
	Asura_Vector_3 xVec2(4.0f, 7.0f, 5.0f );
	Asura_Vector_3 xVec3(12.0f, 15.0f, 13.0f );
	Asura_Bounding_Box xBox2;
	xBox2.Reset();
	xBox2.Set(xVec2);
	xBox2.Set(xVec3);

	Asura_Vector_3 xVec4(7.0f, 9.5f, 9.5f );
	Asura_Bounding_Box xBox3;
	xBox3.SetToPoint(xVec4);
 	xBox3.Grow(2.0f);

	ASURA_UNITTEST_CHECK(xBox1 == xBox2);
	ASURA_UNITTEST_CHECK(xBox2 == xBox1);

	ASURA_UNITTEST_CHECK(xBox1 != xBox3);
	ASURA_UNITTEST_CHECK(xBox3 != xBox1);

	Asura_Vector_3 xOrigin(0.0f,0.0f,0.0f);
	Asura_Bounding_Box xBox4;
	xBox4.SetToPoint( xOrigin );
	xBox4.Grow(2.0f);
	xBox4.Scale(2.0f);
	Asura_Bounding_Box xBox5;
	xBox5.SetToPoint( xOrigin );
	xBox5.Grow(4.0f);
	ASURA_UNITTEST_CHECK(xBox4==xBox5);
}

ASURA_UNITTEST(Zero, Bounding_Box)
{
	Asura_Vector_3 xVec1(2.0f, 5.0f, 3.0f);
	Asura_Bounding_Box xBox1;

	xBox1.SetToPoint(xVec1);
	xBox1.Zero();

	ASURA_UNITTEST_CHECK(xBox1.MinX == 0.0f && xBox1.MaxX == 0.0f
					  && xBox1.MinY == 0.0f && xBox1.MaxY == 0.0f
				      && xBox1.MinZ == 0.0f && xBox1.MaxZ == 0.0f );

	Asura_Vector_3 xVec2(4.0f, 7.0f, 5.0f );
	Asura_Vector_3 xVec3(10.0f, 15.0f, 15.0f );
	Asura_Bounding_Box xBox2;
	xBox2.Reset();
	xBox2.Set(xVec2);
	xBox2.Set(xVec3);
	xBox2.Zero();

	ASURA_UNITTEST_CHECK(xBox2.MinX == 0.0f && xBox2.MaxX == 0.0f
					  && xBox2.MinY == 0.0f && xBox2.MaxY == 0.0f
					  && xBox2.MinZ == 0.0f && xBox2.MaxZ == 0.0f );

	Asura_Bounding_Box xBox3;
	xBox3.Zero();
	
	ASURA_UNITTEST_CHECK(xBox3.MinX == 0.0f && xBox3.MaxX == 0.0f
					  && xBox3.MinY == 0.0f && xBox3.MaxY == 0.0f
					  && xBox3.MinZ == 0.0f && xBox3.MaxZ == 0.0f );
}

ASURA_UNITTEST(Grow, Bounding_Box)
{
	Asura_Vector_3 xVec1(2.0f, 5.0f, 0.0f);
	Asura_Bounding_Box xBox1;

	xBox1.SetToPoint(xVec1);
	xBox1.Grow(2.0f);

	ASURA_UNITTEST_CHECK(xBox1.MinX == 0.0f && xBox1.MaxX == 4.0f
					&& xBox1.MinY ==  3.0f && xBox1.MaxY == 7.0f
					&& xBox1.MinZ == -2.0f && xBox1.MaxZ == 2.0f );

	xBox1.Grow(3.0f);
	ASURA_UNITTEST_CHECK(xBox1.MinX == -3.0f && xBox1.MaxX == 7.0f
					&& xBox1.MinY ==  0.0f && xBox1.MaxY == 10.0f
					&& xBox1.MinZ == -5.0f && xBox1.MaxZ == 5.0f );
}

ASURA_UNITTEST(Displace, Bounding_Box)
{
	Asura_Vector_3 xVec1(2.0f, 5.0f, 0.0f);
	Asura_Vector_3 xVec2(1.0f, 2.0f, 1.0f);
	Asura_Bounding_Box xBox1;

	xBox1.SetToPoint(xVec1);
	xBox1.Grow(2.0f);
	xBox1.Displace(xVec2);

	ASURA_UNITTEST_CHECK(xBox1.MinX == 1.0f && xBox1.MaxX == 5.0f
					  && xBox1.MinY == 5.0f && xBox1.MaxY == 9.0f
					  && xBox1.MinZ == -1.0f && xBox1.MaxZ == 3.0f );

	xBox1.Displace(xVec2);
	ASURA_UNITTEST_CHECK(xBox1.MinX == 2.0f && xBox1.MaxX == 6.0f
					  && xBox1.MinY == 7.0f && xBox1.MaxY == 11.0f
					  && xBox1.MinZ == 0.0f && xBox1.MaxZ == 4.0f );
}

ASURA_UNITTEST(Volume, Bounding_Box)
{
	Asura_Vector_3 xVec1(2.0f, 5.0f, 0.0f);
	Asura_Bounding_Box xBox1;

	xBox1.SetToPoint(xVec1);
	xBox1.Grow(2.0f);
	float fVol = xBox1.GetVolume();

	ASURA_UNITTEST_FLOATS_EQUAL(fVol, 64.0f, fEPSILON);	

	xBox1.Grow(2.0f);
	ASURA_UNITTEST_FLOATS_EQUAL(xBox1.GetVolume(), 512.0f, fEPSILON);	
	
	Asura_Vector_3 xVec2(4.0f, 7.0f, 5.0f );
	Asura_Vector_3 xVec3(10.0f, 15.0f, 15.0f );
	Asura_Bounding_Box xBox2;
	xBox2.Reset();
	xBox2.Set(xVec2);
	xBox2.Set(xVec3);

	ASURA_UNITTEST_FLOATS_EQUAL(xBox2.GetVolume(), 480.0f, fEPSILON);	
}

ASURA_UNITTEST(Surface, Bounding_Box)
{
	Asura_Vector_3 xVec1(2.0f, 5.0f, 0.0f);
	Asura_Bounding_Box xBox1;

	xBox1.SetToPoint(xVec1);
	xBox1.Grow(2.0f);
	float fSurf = xBox1.GetSurfaceArea();

	ASURA_UNITTEST_FLOATS_EQUAL(fSurf, 96.0f, fEPSILON);	// 6 eq. sides (16 each)
	
	xBox1.Grow(2.0f);
	ASURA_UNITTEST_FLOATS_EQUAL(xBox1.GetSurfaceArea(), 384.0f, fEPSILON);

	Asura_Vector_3 xVec2(4.0f, 7.0f, 5.0f );
	Asura_Vector_3 xVec3(10.0f, 15.0f, 15.0f );
	Asura_Bounding_Box xBox2;
	xBox2.Reset();
	xBox2.Set(xVec2);
	xBox2.Set(xVec3);

	ASURA_UNITTEST_FLOATS_EQUAL(xBox2.GetSurfaceArea(), 376.0f , fEPSILON);
}
ASURA_UNITTEST(Centre, Bounding_Box)
{
	Asura_Vector_3 xVec1(2.0f, 5.0f, 0.0f);
	Asura_Bounding_Box xBox1;

	xBox1.SetToPoint(xVec1);
	xBox1.Grow(2.0f);
	Asura_Vector_3 xVecR = xBox1.CalculateCentre();

	ASURA_UNITTEST_CHECK_EQUAL(xVecR, xVec1);

	xBox1.Grow(3.0f);
	ASURA_UNITTEST_CHECK_EQUAL( xBox1.CalculateCentre(), xVec1);

	Asura_Bounding_Box xBox2;
	xBox2.Reset();
	xBox2.Set(Asura_Vector_3(4.0f, 7.0f, 5.0f ));
	xBox2.Set(Asura_Vector_3(10.0f, 15.0f, 15.0f ));

	ASURA_UNITTEST_CHECK_EQUAL( xBox2.CalculateCentre(), Asura_Vector_3(7.0f, 11.0f, 10.0f ));
}

ASURA_UNITTEST(Extent, Bounding_Box)
{
	Asura_Vector_3 xVec1(2.0f, 5.0f, 0.0f);
	Asura_Bounding_Box xBox1;

	xBox1.SetToPoint(xVec1);
	xBox1.Grow(2.0f);
	Asura_Vector_3 xVecR = xBox1.CalculateExtents();

	ASURA_UNITTEST_CHECK_EQUAL(xVecR, Asura_Vector_3(4.0, 4.0, 4.0) )

	Asura_Vector_3 xVec2(4.0f, 7.0f, 5.0f );
	Asura_Vector_3 xVec3(10.0f, 15.0f, 15.0f );
	Asura_Bounding_Box xBox2;
	xBox2.Reset();
	xBox2.Set(xVec2);
	xBox2.Set(xVec3);

	ASURA_UNITTEST_CHECK_EQUAL(xBox2.CalculateExtents(), Asura_Vector_3(6.0, 8.0, 10.0) )
}

ASURA_UNITTEST(Radius, Bounding_Box)
{
	// ie. the min. radius of a sphere that encompasses the rectangle 
	Asura_Vector_3 xVec1(2.0f, 5.0f, 0.0f );
	Asura_Bounding_Box xBox1;

	xBox1.SetToPoint(xVec1);
	xBox1.Grow(2.0f);
	float fRad = xBox1.CalculateRadius();

	ASURA_UNITTEST_FLOATS_EQUAL(fRad, sqrtf(48.0f)*0.5f, fEPSILON );

	xBox1.Grow(2.0f);
	ASURA_UNITTEST_FLOATS_EQUAL(xBox1.CalculateRadius(), sqrtf(192.0f)*0.5f, fEPSILON );
}

ASURA_UNITTEST(IntersectSphere, Bounding_Box)
{
	// ie. centre of a sphere
	// checked code ... this really doesn't do a sphere it
	// does the square surrounding the sphere !
	Asura_Vector_3 xCenS(8.0f, 11.0f, 9.0f ); // 6.928

	Asura_Vector_3 xVec1(2.0f, 5.0f, 3.0f );
	Asura_Bounding_Box xBox1;

	xBox1.SetToPoint(xVec1);
	xBox1.Grow(2.0f);
	bool bTest = xBox1.IntersectsSphere(xCenS, 150.1f);

	ASURA_UNITTEST_CHECK( bTest );
	ASURA_UNITTEST_CHECK( xBox1.IntersectsSphere(xCenS,4.1f) ); // nb. this would fail if a sphere
	ASURA_UNITTEST_CHECK(!xBox1.IntersectsSphere(xCenS,3.9f) );
	ASURA_UNITTEST_CHECK( xBox1.IntersectsSphere(xCenS,4.0f) ); // touching the sphere-box
	ASURA_UNITTEST_CHECK( xBox1.IntersectsSphere(xCenS,5.5f) );
}

ASURA_UNITTEST(IntersectBox, Bounding_Box)
{
	Asura_Vector_3 xVec1(8.0f, 11.0f, 9.0f );
	Asura_Bounding_Box xBox1;
	xBox1.SetToPoint(xVec1);
	xBox1.Grow(4.0f);

	Asura_Vector_3 xVec2(2.0f, 5.0f, 3.0f );
	Asura_Bounding_Box xBox2;
	xBox2.SetToPoint(xVec2);
	xBox2.Grow(2.5f);

	Asura_Vector_3 xVec3(12.0f, -15.0f, 13.0f );
	Asura_Bounding_Box xBox3;
	xBox3.SetToPoint(xVec3);
	xBox3.Grow(1.5f);

	Asura_Vector_3 xVec4(12.0f, 15.0f, 13.0f );
	Asura_Vector_3 xVec5(16.0f, 18.0f, 35.0f );
	Asura_Bounding_Box xBox4;
	xBox4.Reset();
	xBox4.Set(xVec4);
	xBox4.Set(xVec5);

	bool bTest = xBox1.Intersects(xBox2);
	ASURA_UNITTEST_CHECK(bTest);

	ASURA_UNITTEST_CHECK(!xBox1.Intersects(xBox3));
	ASURA_UNITTEST_CHECK(!xBox2.Intersects(xBox3));

	ASURA_UNITTEST_CHECK(xBox1.Intersects(xBox4)); // these touch on corner

	xBox4.Displace(Asura_Vector_3(0.1f, 0.0f, 0.0f) ); // shift by 0.1 in x
	ASURA_UNITTEST_CHECK(!xBox1.Intersects(xBox4)); // don't touch anymore
}

ASURA_UNITTEST(IntersectDisplacedBox, Bounding_Box)
{
	Asura_Vector_3 xVec1(8.0f, 11.0f, 9.0f );
	Asura_Bounding_Box xBox1;
	xBox1.SetToPoint(xVec1);
	xBox1.Grow(4.0f);

	Asura_Vector_3 xVec2(-2.0f, -5.0f, -3.0f );
	Asura_Bounding_Box xBox2;
	xBox2.SetToPoint(xVec2);
	xBox2.Grow(2.5f);

	Asura_Vector_3 xVecD(4.0f, 10.0f, 6.0f);
	bool bTest = xBox1.Intersects(xBox2, xVecD);
	
	ASURA_UNITTEST_CHECK(bTest);
	ASURA_UNITTEST_CHECK(!xBox1.Intersects(xBox2, Asura_Vector_3(-20.0f, -20.0f, -20.0f ) ) );

	// now only touching in x & y
	ASURA_UNITTEST_CHECK( xBox1.Intersects(xBox2, Asura_Vector_3( 3.5f, 9.5f, 6.0f ) ) );

	Asura_Bounding_Box xBox3;
	xBox3.Reset();
	xBox3.Set(Asura_Vector_3(2.0f, 3.0f, 5.0f ) );
	xBox3.Set(Asura_Vector_3(6.0f, 8.0f, 7.0f ) );

	ASURA_UNITTEST_CHECK( xBox1.Intersects(xBox3, Asura_Vector_3( 0.0f, 0.0f, 0.0f ) ) );
	ASURA_UNITTEST_CHECK(!xBox2.Intersects(xBox3, Asura_Vector_3( 0.0f, 0.0f, 0.0f ) ) );
}

ASURA_UNITTEST(Intersection, Bounding_Box)
{
	Asura_Vector_3 xVec1(8.0f, 11.0f, 3.0f );
	Asura_Bounding_Box xBox1;   //  12    15     7
	xBox1.SetToPoint(xVec1);    //   4     7    -1
	xBox1.Grow(4.0f);

	Asura_Vector_3 xVec2(2.0f, 5.0f, 9.0f );
	Asura_Bounding_Box xBox2;   //  4.5   7.5  11.5
	xBox2.SetToPoint(xVec2);    // -0.5   2.5   6.5
 	xBox2.Grow(2.5f);

	Asura_Bounding_Box xBox3;
	xBox3.Zero();
	xBox3 = xBox1.Intersection(xBox2);

	ASURA_UNITTEST_CHECK(xBox3.MinX == 4.0f && xBox3.MaxX == 4.5f
				   	  && xBox3.MinY == 7.0f && xBox3.MaxY == 7.5f
					  && xBox3.MinZ == 6.5f && xBox3.MaxZ == 7.0f );

	// Ensure the original copies have not been changed
	ASURA_UNITTEST_CHECK(xBox1.MinX ==  4.0f && xBox1.MaxX == 12.0f
					  && xBox1.MinY ==  7.0f && xBox1.MaxY == 15.0f
					  && xBox1.MinZ == -1.0f && xBox1.MaxZ ==  7.0f )
	ASURA_UNITTEST_CHECK(xBox2.MinX == -0.5f && xBox2.MaxX ==  4.5f
				   	  && xBox2.MinY ==  2.5f && xBox2.MaxY ==  7.5f
					  && xBox2.MinZ ==  6.5f && xBox2.MaxZ == 11.5f )
}

ASURA_UNITTEST(Contains, Bounding_Box)
{
	Asura_Vector_3 xVec1(8.0f, 11.0f, 9.0f );
	Asura_Bounding_Box xBox1;
	xBox1.SetToPoint(xVec1);
	xBox1.Grow(4.0f); 
	
	Asura_Vector_3 xVec2(7.0f, 9.5f, 9.5f );
	Asura_Bounding_Box xBox2;
	xBox2.SetToPoint(xVec2);
 	xBox2.Grow(2.0f);

	ASURA_UNITTEST_CHECK(xBox1.Contains(xBox2) )
	ASURA_UNITTEST_CHECK(!xBox2.Contains(xBox1) )
	
	Asura_Vector_3 xVec3(2.0f, 1.5f, 3.2f);
	Asura_Vector_3 xVec4(4.0f, 6.0f, 9.0f);
	Asura_Vector_3 xVec5(10.0f, 12.0f, 11.0f);
	Asura_Vector_3 xVec6(-3.0f, 0.0f, 8.0f);
	
	ASURA_UNITTEST_CHECK(!xBox1.Contains(xVec3) )
	ASURA_UNITTEST_CHECK(!xBox1.Contains(xVec4) )
	ASURA_UNITTEST_CHECK( xBox1.Contains(xVec5) )
	ASURA_UNITTEST_CHECK(!xBox1.Contains(xVec6) )
}

ASURA_UNITTEST(MinDistance, Bounding_Box)
{
	Asura_Vector_3 xVec1(3.0f, 1.0f, 4.0f );
	Asura_Bounding_Box xBox1;
	xBox1.SetToPoint(xVec1);
	xBox1.Grow(4.0f); 

	Asura_Vector_3 xVec2( 2.0f,  1.5f,  3.2f);
	Asura_Vector_3 xVec3( 4.0f,  6.0f,  9.0f);
	Asura_Vector_3 xVec4(10.0f, 12.0f, 11.0f);
	Asura_Vector_3 xVec5(-3.0f,  0.0f,  8.0f);

	ASURA_UNITTEST_FLOATS_EQUAL(xBox1.GetSqdMinDistFromPoint(xVec2),  0.0f, fEPSILON )
	ASURA_UNITTEST_FLOATS_EQUAL(xBox1.GetSqdMinDistFromPoint(xVec3),  2.0f, fEPSILON )
	ASURA_UNITTEST_FLOATS_EQUAL(xBox1.GetSqdMinDistFromPoint(xVec4), 67.0f, fEPSILON )
	ASURA_UNITTEST_FLOATS_EQUAL(xBox1.GetSqdMinDistFromPoint(xVec5),  4.0f, fEPSILON )

	Asura_Bounding_Box xBox2;
	xBox2.SetToPoint(xVec2);
 	xBox2.Grow(1.0f);

	Asura_Bounding_Box xBox3;
	xBox3.SetToPoint(xVec4);
 	xBox3.Grow(1.0f);
	
	ASURA_UNITTEST_FLOATS_EQUAL(xBox1.GetSqdMinDistFromBB(xBox2),  0.0f, fEPSILON )
	ASURA_UNITTEST_FLOATS_EQUAL(xBox1.GetSqdMinDistFromBB(xBox3), 44.0f, fEPSILON )
}

ASURA_UNITTEST(FromOBB, Bounding_Box)
{
	Asura_Bounding_Box xObb;
	Asura_Vector_3 xOrigin(0.0f,0.0f,0.0f);
	xObb.SetToPoint( xOrigin );
	xObb.Grow( 2.0f );
	xObb.MaxX = 4.0f;
	xObb.MinX = -4.0f;
	Asura_Matrix_3x3 xOri(0.0f,1.0f,0.0f,-1.0f,0.0f,0.0f,0.0f,0.0f,1.0f);
	Asura_Bounding_Box xAABB;
	xAABB.FromOBB( xObb, xOri );
	Asura_Bounding_Box xTarget;
	xTarget.SetToPoint( xOrigin );
	xTarget.Grow( 2.0f );
	xTarget.MaxY = 4.0f;
	xTarget.MinY = -4.0f;
	ASURA_UNITTEST_CHECK( xTarget == xAABB );
}

// H E R M I T E   S P L I N E ///////////////////////////////////////////////

ASURA_UNITTEST(Get, Hermite_Spline)
{
	Asura_Vector_3 xPos0(0.0f,0.0f,0.0f);
	Asura_Vector_3 xPos1(1.0f,1.0f,1.0f);
	Asura_Vector_3 xVel0(0.0f,1.0f,0.0f);
	Asura_Vector_3 xVel1(0.0f,1.0f,0.0f);

	Asura_HermiteSpline xSpline;
	xSpline.Recalculate(xPos0, xPos1, xVel0, xVel1);

	Asura_Vector_3 xOut0, xOut1, xOut2, xOut3, xOut4, xOut5;
	xSpline.GetPosition(0.0f, xOut0);
	xSpline.GetPosition(1.0f, xOut2);
	xSpline.GetPosition(0.5f, xOut4);
	xSpline.GetVelocity(0.0f, xOut1);
	xSpline.GetVelocity(1.0f, xOut3);
	xSpline.GetVelocity(0.5f, xOut5);

	Asura_Vector_3 xTargetPos(0.5f,0.5f,0.5f);
	Asura_Vector_3 xTargetVel(1.5f,1.0f,1.5f);

	ASURA_UNITTEST_CHECK( xOut0 == xPos0 );
	ASURA_UNITTEST_CHECK( xOut2 == xPos1 );
	ASURA_UNITTEST_CHECK( xOut4 == xTargetPos );
	ASURA_UNITTEST_CHECK( xOut1 == xVel0 );
	ASURA_UNITTEST_CHECK( xOut3 == xVel1 );
	ASURA_UNITTEST_CHECK( xOut5 == xTargetVel );
}

// S P H E R I C A L   H A R M O N I C S /////////////////////////////////////

ASURA_UNITTEST(Zero, SphericalHarmonics_3)
{
	Asura_SphericalHarmonics_Order3 xSH;
	xSH[0] = 1;
	ASURA_UNITTEST_CHECK( !xSH.IsZero() );
	xSH.Zero();
	ASURA_UNITTEST_CHECK( xSH.IsZero() );
}

ASURA_UNITTEST(Equality, SphericalHarmonics_3)
{
	Asura_SphericalHarmonics_Order3 xSH;
	xSH[0] = 1.0f;
	Asura_SphericalHarmonics_Order3 xSH2(xSH);
	ASURA_UNITTEST_CHECK( xSH == xSH2 );
}

ASURA_UNITTEST(AddSampleForDirection, SphericalHarmonics_3)
{
	Asura_SphericalHarmonics_Order3 xSH;
	xSH.Zero();
	Asura_Vector_3 xNormal(0.0f,0.0f,1.0f);
	xSH.AddSampleForDirection(1.0f,xNormal);
	ASURA_UNITTEST_CHECK_EQUAL( 1.0f, xSH[0] );
	ASURA_UNITTEST_CHECK_EQUAL( 0.0f, xSH[1] );
	ASURA_UNITTEST_CHECK_EQUAL( 0.0f, xSH[2] );
	ASURA_UNITTEST_CHECK_EQUAL( 1.0f, xSH[3] );
	ASURA_UNITTEST_CHECK_EQUAL( 0.0f, xSH[4] );
	ASURA_UNITTEST_CHECK_EQUAL( 0.0f, xSH[5] );
	ASURA_UNITTEST_CHECK_EQUAL( 0.0f, xSH[6] );
	ASURA_UNITTEST_CHECK_EQUAL( 2.0f, xSH[7] );
	ASURA_UNITTEST_CHECK_EQUAL( 0.0f, xSH[8] );
}

ASURA_UNITTEST(ZerothOrder, SphericalHarmonics_3)
{
	Asura_SphericalHarmonics_Order3 xSH;
	xSH[0] = 15.0f;
	ASURA_UNITTEST_CHECK_EQUAL( 15.0f, xSH.GetZerothOrderOcclusionTerm() );
}

ASURA_UNITTEST(EvaluateInDirection, SphericalHarmonics_3)
{
	Asura_SphericalHarmonics_Order3 xSH;
	Asura_Vector_3 xNormal(0.0f,0.0f,1.0f);
	Asura_Vector_3 xSecond(1.0f,1.0f,1.0f);
	xSecond.Normalise();
	xSH.Zero();
	xSH.AddSampleForDirection( 1.0f, xSecond );
	float f = xSH.EvaluateInDirection( xNormal );
	ASURA_UNITTEST_FLOATS_EQUAL( 1.577350f, f, 0.000001f );
}

ASURA_UNITTEST(Integrate, SphericalHarmonics_3)
{
	Asura_SphericalHarmonics_Order3 xSH1;
	Asura_SphericalHarmonics_Order3 xSH2;
	Asura_Vector_3 xNormal(2.0f,0.0f,1.0f);
	xNormal.Normalise();
	Asura_Vector_3 xSecond(0.0f,1.0f,1.0f);
	xSecond.Normalise();
	xSH1.Zero();
	xSH2.Zero();
	xSH1.AddSampleForDirection( 1.0f, xNormal );
	xSH2.AddSampleForDirection( 1.0f, xSecond );
	float f = xSH1.Integrate( xSH2 );
	ASURA_UNITTEST_FLOATS_EQUAL( 0.716228f, f, 0.000001f );
}

ASURA_UNITTEST( Scale, SphericalHarmonics_3 )
{
	Asura_SphericalHarmonics_Order3 xSH;
	xSH[0] = 1.0f; xSH[1] = 2.0f;
	xSH.ScaleCoefficientsForNSamples( 2.0f );
	ASURA_UNITTEST_FLOATS_EQUAL( 0.5f, xSH[0], 0.000001f );
	ASURA_UNITTEST_FLOATS_EQUAL( 1.0f, xSH[1], 0.000001f );
	xSH.Scale( 2.0f );
	ASURA_UNITTEST_FLOATS_EQUAL( 1.0f, xSH[0], 0.000001f );
	ASURA_UNITTEST_FLOATS_EQUAL( 2.0f, xSH[1], 0.000001f );
}

ASURA_UNITTEST( PlusEquals, SphericalHarmonics_3 )
{
	Asura_SphericalHarmonics_Order3 xSH, xSH2;
	xSH[0] = 2.0f;
	xSH2[0] = 3.0f;
	xSH += xSH2;
	ASURA_UNITTEST_CHECK_EQUAL( 5.0f, xSH[0] );
}

ASURA_UNITTEST( TimesEquals, SphericalHarmonics_3 )
{
	Asura_SphericalHarmonics_Order3 xSH, xSH2;
	xSH[0] = 2.0f;
	xSH2[0] = 3.0f;
	xSH *= xSH2;
	ASURA_UNITTEST_CHECK_EQUAL( 6.0f, xSH[0] );
}

ASURA_UNITTEST( Set, SphericalHarmonics_2 )
{
	Asura_SphericalHarmonics_Order3 xSHR, xSHG, xSHB;
	xSHR[0] = 4.0f;
	xSHG[0] = 2.0f;
	xSHB[0] = 3.0f;
	Asura_SphericalHarmonics_Order2_RGB xSH;
	xSH.Set( xSHR, xSHG, xSHB );
	ASURA_UNITTEST_CHECK_EQUAL( 4.0f, xSH[0] );
	ASURA_UNITTEST_CHECK_EQUAL( 2.0f, xSH[4] );
	ASURA_UNITTEST_CHECK_EQUAL( 3.0f, xSH[8] );
}

ASURA_UNITTEST( ScaledAdd, SphericalHarmonics_2 )
{
	Asura_SphericalHarmonics_Order3 xSHR, xSHG, xSHB;
	xSHR[0] = 4.0f;
	xSHG[0] = 2.0f;
	xSHB[0] = 3.0f;
	Asura_SphericalHarmonics_Order2_RGB xSH, xSH2;
	xSH.Set( xSHR, xSHG, xSHB );
	xSH2.Set( xSHR, xSHG, xSHB );
	xSH.ScaledAdd( xSH2, 2.0f );
	ASURA_UNITTEST_CHECK_EQUAL( 12.0f, xSH[0] );
}

ASURA_UNITTEST( Scale, SphericalHarmonics_2 )
{
	Asura_SphericalHarmonics_Order3 xSHR, xSHG, xSHB;
	xSHR[0] = 4.0f;
	xSHG[0] = 2.0f;
	xSHB[0] = 3.0f;
	Asura_SphericalHarmonics_Order2_RGB xSH;
	xSH.Set( xSHR, xSHG, xSHB );
	xSH.Scale( 3.0f );
	ASURA_UNITTEST_CHECK_EQUAL( 12.0f, xSH[0] );
}

ASURA_UNITTEST( Zero, SphericalHarmonics_2 )
{
	Asura_SphericalHarmonics_Order3 xSHR, xSHG, xSHB;
	xSHR[0] = 4.0f;
	xSHG[0] = 2.0f;
	xSHB[0] = 3.0f;
	Asura_SphericalHarmonics_Order2_RGB xSH;
	xSH.Set( xSHR, xSHG, xSHB );
	xSH.Zero();
	ASURA_UNITTEST_CHECK_EQUAL( 0.0f, xSH[0] );
}

ASURA_UNITTEST( PlusEquals, SphericalHarmonics_2 )
{
	Asura_SphericalHarmonics_Order3 xSHR, xSHG, xSHB;
	xSHR[0] = 4.0f;
	xSHG[0] = 2.0f;
	xSHB[0] = 3.0f;
	Asura_SphericalHarmonics_Order2_RGB xSH, xSH2;
	xSH.Set( xSHR, xSHG, xSHB );
	xSH2.Set( xSHG, xSHB, xSHR );
	xSH += xSH2;
	ASURA_UNITTEST_CHECK_EQUAL( 6.0f, xSH[0] );
}

ASURA_UNITTEST( AddSampleForDirection, SphericalHarmonics_2 )
{
	Asura_SphericalHarmonics_Order2_RGB xSH;
	Asura_Vector_3 xNormal( 0.0f, 0.0f, 1.0f );
	xSH.Zero();
	xSH.AddSampleForDirection( 0.25f, 0.5f, 1.0f, xNormal );
	ASURA_UNITTEST_CHECK_EQUAL( 0.25f, xSH[0] );
	ASURA_UNITTEST_CHECK_EQUAL( 0.0f, xSH[1] );
	ASURA_UNITTEST_CHECK_EQUAL( 0.0f, xSH[2] );
	ASURA_UNITTEST_CHECK_EQUAL( 0.25f, xSH[3] );
	ASURA_UNITTEST_CHECK_EQUAL( 0.5f, xSH[4] );
	ASURA_UNITTEST_CHECK_EQUAL( 1.0f, xSH[8] );
}

ASURA_UNITTEST( RotateToModelSpace, SphericalHarmonics_2 )
{
	Asura_Matrix_3x3 xOri( 0.0f, 1.0f, 0.0f, -1.0f, 0.0f, 0.0f, 0.0f, 0.0f, 1.0f );
	Asura_SphericalHarmonics_Order2_RGB xSH;
	xSH.Zero();
	xSH[0] = 1.0f;
	xSH[1] = 1.0f;
	xSH[2] = 0.3f;
	xSH.RotateToModelspace( xOri );
	ASURA_UNITTEST_FLOATS_EQUAL( -0.3f, xSH[1], 1.0e-6f );
}

ASURA_UNITTEST( GetBrighestDirection, SphericalHarmonics_2 )
{
	Asura_Vector_3 xDirection;
	Asura_Vector_3 xNormal(1.0f,1.0f,0.0f);
	xNormal.Normalise();
	Asura_SphericalHarmonics_Order2_RGB xSH;
	xSH.Zero();
	xSH.AddSampleForDirection( 0.2f,0.2f,0.2f,xNormal );
	xSH.GetBrightestDirection( xDirection );
	Asura_Vector_3 xTarget( 0.707107f, 0.707107f, 0.0f );
	ASURA_UNITTEST_CHECK ( xTarget.ApproximatelyEquals(xDirection) );
}

ASURA_UNITTEST( Colourise, SphericalHarmonics_2 )
{
	Asura_Vector_3 xDirection;
	Asura_Vector_3 xNormal(1.0f,1.0f,0.0f);
	xNormal.Normalise();
	Asura_SphericalHarmonics_Order2_RGB xSH;
	xSH.Zero();
	xSH.AddSampleForDirection( 0.2f,0.2f,0.2f,xNormal );
	xSH.GetBrightestDirection( xDirection );
	xSH.Colourise( 1.0f, 0.5f, 0.5f, 0.8f );
	ASURA_UNITTEST_FLOATS_EQUAL( 0.2f, xSH[0], 0.000001f );
	ASURA_UNITTEST_FLOATS_EQUAL( 0.12f, xSH[4], 0.000001f );
	ASURA_UNITTEST_FLOATS_EQUAL( 0.12f, xSH[8], 0.000001f );
}

// O R I E N T E D   B O U N D I N G   B O X /////////////////////////////////

ASURA_UNITTEST(Constructors, OBB)
{
	Asura_AVector3 xHalfExtents( 2.0f, 1.0f, 1.0f );
	Asura_AVector3 xCentre( 0.0f, 0.0f, 0.0f );
	Asura_Vector_3 xOrigin(0.0f,0.0f,0.0f);
	Asura_AMatrix3x3 xMatrix(0.0f,1.0f,0.0f,-1.0f,0.0f,0.0f,0.0f,0.0f,1.0f);
	Asura_Bounding_Box xBox;
	xBox.SetToPoint(xOrigin);
	xBox.Grow(1.0f);
	xBox.MaxY = 2.0f;
	xBox.MinY = -2.0f;
	Asura_OBB xOBB1(xHalfExtents,xCentre,xMatrix);
	Asura_OBB xOBB2(xBox);
	Asura_OBB xOBB3(1.0f);
	Asura_OBB xOBB4(xBox,xHalfExtents);
	Asura_OBB xOBB5(xBox,xMatrix);
	Asura_OBB xOBB6(xBox,xHalfExtents,xMatrix);

	Asura_Bounding_Box xRes1, xRes2;
	xOBB1.CalculateEnclosingAABB(xRes1);
	xOBB2.CalculateEnclosingAABB(xRes2);
	ASURA_UNITTEST_CHECK(xRes1==xRes2);
	xRes1.MinX = xRes1.MinY = xRes1.MinZ = -1.0f;
	xRes1.MaxX = xRes1.MaxY = xRes1.MaxZ = 1.0f;
	xOBB3.CalculateEnclosingAABB(xRes2);
	ASURA_UNITTEST_CHECK(xRes1==xRes2);
	xOBB4.CalculateEnclosingAABB(xRes2);
	xRes1.MinX = 1.0f; xRes1.MinY = -1.0f; xRes1.MinZ = 0.0f;
	xRes1.MaxX = 3.0f; xRes1.MaxY = 3.0f; xRes1.MaxZ = 2.0f;
	ASURA_UNITTEST_CHECK(xRes1==xRes2);
	xOBB5.CalculateEnclosingAABB(xRes2);
	xRes1.MinX = -2.0f; xRes1.MinY = -1.0f; xRes1.MinZ = -1.0f;
	xRes1.MaxX = 2.0f; xRes1.MaxY = 1.0f; xRes1.MaxZ = 1.0f;
	ASURA_UNITTEST_CHECK(xRes1==xRes2);
	xOBB6.CalculateEnclosingAABB(xRes2);
	xRes1.MinX = 0.0f; xRes1.MinY = 0.0f; xRes1.MinZ = 0.0f;
	xRes1.MaxX = 4.0f; xRes1.MaxY = 2.0f; xRes1.MaxZ = 2.0f;
	ASURA_UNITTEST_CHECK(xRes1==xRes2);
}

ASURA_UNITTEST( BasicAccessors, OBB)
{
	Asura_Vector_3 xOrigin(0.0f,0.0f,0.0f);
	Asura_OBB xObb(1.0f);
	// get/set half extents
	Asura_AVector3 xHalfExtents( 1.0f, 1.0f, 1.0f );
	xObb.SetHalfExtents( xHalfExtents );
	xHalfExtents = xObb.GetHalfExtents();
	Asura_AVector3 xTarget(1.0f,1.0f,1.0f);
	ASURA_UNITTEST_CHECK( xTarget == xHalfExtents );
	// set using BB
	Asura_Bounding_Box xBox;
	xBox.SetToPoint(xOrigin);
	xBox.Grow(1.0f);
	xObb.SetHalfExtents( xBox );
	xHalfExtents = xObb.GetHalfExtents();
	ASURA_UNITTEST_CHECK( xTarget == xHalfExtents );

	Asura_AMatrix3x3 xOri(0.0f,1.0f,0.0f,1.0f,0.0,0.0f,0.0f,0.0,1.0f);
	xObb.SetOrientation( xOri );
	ASURA_UNITTEST_CHECK( xOri == xObb.GetOrientation() );

	xHalfExtents = Asura_AVector3( 4.0f, 3.0f, 2.0f );
	xObb.SetHalfExtents( xHalfExtents );
	ASURA_UNITTEST_CHECK_EQUAL( 2.0f, xObb.GetMinimumHalfExtent() );
	ASURA_UNITTEST_CHECK_EQUAL( 4.0f, xObb.GetMaximumHalfExtent() );
	xObb.SetCentre( xHalfExtents );
	Asura_AVector3 xRes = xObb.GetMinPosition();
	Asura_AVector3 xAOrigin(0.0f,0.0f,0.0f);
	ASURA_UNITTEST_CHECK( xAOrigin == xRes );
	xTarget = Asura_AVector3( 8.0f, 6.0f, 4.0f );
	xRes = xObb.GetMaxPosition();
	ASURA_UNITTEST_CHECK( xTarget == xRes );
}

ASURA_UNITTEST( GetCentreAndHalfExtentAlongAxis, OBB )
{
	Asura_OBB xObb( 1.0f );
	Asura_AVector3 xHalfExtents( 1.0f, 2.0f, 3.0f );
	xObb.SetHalfExtents( xHalfExtents );
	xObb.SetCentre( xHalfExtents );
	Asura_AVector3 xAxis( 1.0f, 1.0f, 0.0f );
	xAxis.Normalise();
	float fCentre, fHalfExtent;
	xObb.GetCentreAndHalfExtentAlongAxis( xAxis, fCentre, fHalfExtent );
	ASURA_UNITTEST_FLOATS_EQUAL( 2.121320f, fCentre, 0.000001f );
	ASURA_UNITTEST_FLOATS_EQUAL( 2.121320f, fHalfExtent, 0.000001f );
}

ASURA_UNITTEST( GetExtentsAlongAxis, OBB )
{
	Asura_OBB xObb( 1.0f );
	Asura_AVector3 xHalfExtents( 1.0f, 2.0f, 3.0f );
	xObb.SetHalfExtents( xHalfExtents );
	xObb.SetCentre( xHalfExtents );
	Asura_AVector3 xAxis( 1.0f, 1.0f, 0.0f );
	xAxis.Normalise();
	float fMinExtent, fMaxExtent;
	xObb.GetExtentsAlongAxis( xAxis, fMinExtent, fMaxExtent );
	ASURA_UNITTEST_FLOATS_EQUAL( 0.0f, fMinExtent, 0.000001f );
	ASURA_UNITTEST_FLOATS_EQUAL( 4.242640f, fMaxExtent, 0.000001f );
}

ASURA_UNITTEST( GetPrimaryAxis, OBB )
{
	Asura_OBB xObb( 1.0f );
	Asura_AVector3 xHalfExtents( 1.0f, 2.0f, 3.0f );
	xObb.SetHalfExtents( xHalfExtents );
	xObb.SetCentre( xHalfExtents );
	Asura_AMatrix3x3 xOri(0.0f,1.0f,0.0f,1.0f,0.0,0.0f,0.0f,0.0,1.0f);
	xObb.SetOrientation( xOri );

	Asura_AVector3 xAxis;
	float fRadius;
	xObb.GetPrimaryAxisAndItsRadius( xAxis, fRadius );
	Asura_AVector3 xTarget(0.0f,0.0f,3.0f);
	ASURA_UNITTEST_CHECK( xTarget == xAxis );
	ASURA_UNITTEST_CHECK_EQUAL( 2.0f, fRadius );
}

ASURA_UNITTEST( Intersects, OBB )
{
	Asura_OBB xObb( 1.0f );
	Asura_AVector3 xHalfExtents( 1.0f, 2.0f, 3.0f );
	xObb.SetHalfExtents( xHalfExtents );
	xObb.SetCentre( xHalfExtents );
	Asura_AMatrix3x3 xOri(0.0f,1.0f,0.0f,1.0f,0.0,0.0f,0.0f,0.0,1.0f);
	xObb.SetOrientation( xOri );

	Asura_OBB xSecondObb( 1.0f );
	ASURA_UNITTEST_CHECK( xObb.Intersects( xSecondObb ) );

	Asura_OBB xThirdObb( 0.9f );
	ASURA_UNITTEST_CHECK( ! xObb.Intersects( xThirdObb ) );
}

#endif // #ifdef ASURA_SUPPORT_UNITTEST

// E N D   O F   F I L E /////////////////////////////////////////////////////
