//////////////////////////////////////////////////////////////////////////////
//
// Asura_UnitTest_Maths_SoA.cpp
//
// Copyright (c) 2005 Rebellion. All rights reserved.
//
// Description:
//
//     Unit Tests for SoA (Structure of Arrays) types
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

#include "UnitTest/Asura_UnitTest.h"

#include "Core/Asura_Maths.h"

#ifdef ASURA_SUPPORT_UNITTEST

// A S U R A _ S O A _ F L O A T /////////////////////////////////////////////
const Asura_SoA_Float xSoaFloat1234(1.0f, 2.0f, 3.0f, 4.0f);
const Asura_SoA_Float xSoaFloat1234B(1.0f, 2.0f, 3.0f, 4.0f);
const Asura_SoA_Float xSoaFloat0234(0.0f, 2.0f, 3.0f, 4.0f);
const Asura_SoA_Float xSoaFloat1034(1.0f, 0.0f, 3.0f, 4.0f);
const Asura_SoA_Float xSoaFloat1204(1.0f, 2.0f, 0.0f, 4.0f);
const Asura_SoA_Float xSoaFloat1230(1.0f, 2.0f, 3.0f, 0.0f);
const Asura_SoA_Float xSoaFloat0000(0.0f, 0.0f, 0.0f, 0.0f);
const Asura_SoA_Float xSoaFloat1000(1.0f, 0.0f, 0.0f, 0.0f);
const Asura_SoA_Float xSoaFloat0100(0.0f, 1.0f, 0.0f, 0.0f);
const Asura_SoA_Float xSoaFloat0010(0.0f, 0.0f, 1.0f, 0.0f);
const Asura_SoA_Float xSoaFloat0001(0.0f, 0.0f, 0.0f, 1.0f);
const Asura_SoA_Float xSoaFloat1111(1.0f, 1.0f, 1.0f, 1.0f);
const Asura_SoA_Float xSoaFloat0111(0.0f, 1.0f, 1.0f, 1.0f);
const Asura_SoA_Float xSoaFloat1011(1.0f, 0.0f, 1.0f, 1.0f);
const Asura_SoA_Float xSoaFloat1101(1.0f, 1.0f, 0.0f, 1.0f);
const Asura_SoA_Float xSoaFloat1110(1.0f, 1.0f, 1.0f, 0.0f);
const Asura_SoA_Float xSoaFloat2468(2.0f, 4.0f, 6.0f, 8.0f);
const Asura_SoA_Float xSoaFloat2222(2.0f, 2.0f, 2.0f, 2.0f);
const Asura_SoA_Float xSoaFloat3333(3.0f, 3.0f, 3.0f, 3.0f);

ASURA_UNITTEST(Equality, Asura_SoA_Float)
{
	ASURA_UNITTEST_CHECK( xSoaFloat1234 == xSoaFloat1234 );
	ASURA_UNITTEST_CHECK( xSoaFloat1234 == xSoaFloat1234B );
	ASURA_UNITTEST_CHECK( xSoaFloat0000 == xSoaFloat0000 );
	ASURA_UNITTEST_CHECK( xSoaFloat1111 == xSoaFloat1111 );
	ASURA_UNITTEST_CHECK( !(xSoaFloat0234 == xSoaFloat1234) );
	ASURA_UNITTEST_CHECK( !(xSoaFloat1034 == xSoaFloat1234) );
	ASURA_UNITTEST_CHECK( !(xSoaFloat1204 == xSoaFloat1234) );
	ASURA_UNITTEST_CHECK( !(xSoaFloat1230 == xSoaFloat1234) );
	ASURA_UNITTEST_CHECK( !(xSoaFloat0000 == xSoaFloat1234) );
	ASURA_UNITTEST_CHECK( !(xSoaFloat0000 == xSoaFloat0234) );
	ASURA_UNITTEST_CHECK( !(xSoaFloat0000 == xSoaFloat1034) );
	ASURA_UNITTEST_CHECK( !(xSoaFloat0000 == xSoaFloat1204) );
	ASURA_UNITTEST_CHECK( !(xSoaFloat0000 == xSoaFloat1230) );
	ASURA_UNITTEST_CHECK( !(xSoaFloat0000 == xSoaFloat1111) );
}

ASURA_UNITTEST(Inequality, Asura_SoA_Float)
{
	ASURA_UNITTEST_CHECK( !(xSoaFloat1234 != xSoaFloat1234) );
	ASURA_UNITTEST_CHECK( !(xSoaFloat1234 != xSoaFloat1234B) );
	ASURA_UNITTEST_CHECK( !(xSoaFloat0000 != xSoaFloat0000) );
	ASURA_UNITTEST_CHECK( !(xSoaFloat1111 != xSoaFloat1111) );
	ASURA_UNITTEST_CHECK( (xSoaFloat0234 != xSoaFloat1234) );
	ASURA_UNITTEST_CHECK( (xSoaFloat1034 != xSoaFloat1234) );
	ASURA_UNITTEST_CHECK( (xSoaFloat1204 != xSoaFloat1234) );
	ASURA_UNITTEST_CHECK( (xSoaFloat1230 != xSoaFloat1234) );
	ASURA_UNITTEST_CHECK( (xSoaFloat0000 != xSoaFloat1234) );
	ASURA_UNITTEST_CHECK( (xSoaFloat0000 != xSoaFloat0234) );
	ASURA_UNITTEST_CHECK( (xSoaFloat0000 != xSoaFloat1034) );
	ASURA_UNITTEST_CHECK( (xSoaFloat0000 != xSoaFloat1204) );
	ASURA_UNITTEST_CHECK( (xSoaFloat0000 != xSoaFloat1230) );
	ASURA_UNITTEST_CHECK( (xSoaFloat0000 != xSoaFloat1111) );
}

ASURA_UNITTEST(Constructors, Asura_SoA_Float)
{
	ASURA_UNITTEST_CHECK( Asura_SoA_Float(0.0f) == xSoaFloat0000 );
	ASURA_UNITTEST_CHECK( Asura_SoA_Float(1.0f) == xSoaFloat1111 );
}

ASURA_UNITTEST(Multiply, Asura_SoA_Float)
{
	ASURA_UNITTEST_CHECK( xSoaFloat0000 == (xSoaFloat0000 * xSoaFloat1234) );
	ASURA_UNITTEST_CHECK( xSoaFloat1111 == (xSoaFloat1111 * xSoaFloat1111) );
	ASURA_UNITTEST_CHECK( xSoaFloat1234 == (xSoaFloat1111 * xSoaFloat1234) );
	ASURA_UNITTEST_CHECK( xSoaFloat1234 == (xSoaFloat1234 * xSoaFloat1111) );
	ASURA_UNITTEST_CHECK( xSoaFloat0234 == (xSoaFloat1234 * xSoaFloat0111) );
	ASURA_UNITTEST_CHECK( xSoaFloat1034 == (xSoaFloat1234 * xSoaFloat1011) );
	ASURA_UNITTEST_CHECK( xSoaFloat1204 == (xSoaFloat1234 * xSoaFloat1101) );
	ASURA_UNITTEST_CHECK( xSoaFloat1230 == (xSoaFloat1234 * xSoaFloat1110) );
}

ASURA_UNITTEST(Addition, Asura_SoA_Float)
{
	ASURA_UNITTEST_CHECK( xSoaFloat0000 == (xSoaFloat0000 + xSoaFloat0000) );
	ASURA_UNITTEST_CHECK( xSoaFloat1111 == (xSoaFloat1111 + xSoaFloat0000) );
	ASURA_UNITTEST_CHECK( xSoaFloat1111 == (xSoaFloat0000 + xSoaFloat1111) );
	ASURA_UNITTEST_CHECK( xSoaFloat1234 == (xSoaFloat0000 + xSoaFloat1234) );
	ASURA_UNITTEST_CHECK( xSoaFloat1234 == (xSoaFloat1234 + xSoaFloat0000) );
	ASURA_UNITTEST_CHECK( xSoaFloat2468 == (xSoaFloat1234 + xSoaFloat1234) );
}

// A S U R A _ S O A _ V E C T O R _ 3 ///////////////////////////////////////
const Asura_SoA_Vector_3 xVec_100_100_100_100( xSoaFloat1111, 
											   xSoaFloat0000, 
											   xSoaFloat0000 );
const Asura_SoA_Vector_3 xVec_010_010_010_010( xSoaFloat0000, 
											   xSoaFloat1111, 
											   xSoaFloat0000 );
const Asura_SoA_Vector_3 xVec_001_001_001_001( xSoaFloat0000, 
											   xSoaFloat0000, 
											   xSoaFloat1111 );
const Asura_SoA_Vector_3 xVec_100_010_001_000( xSoaFloat1000, 
											   xSoaFloat0100, 
											   xSoaFloat0010 );
const Asura_SoA_Vector_3 xVec_123_123_123_123( xSoaFloat1111, 
											   xSoaFloat2222, 
											   xSoaFloat3333 );
const Asura_SoA_Vector_3 xVec_111_111_111_111( xSoaFloat1111, 
											   xSoaFloat1111, 
											   xSoaFloat1111 );
const Asura_SoA_Vector_3 xVec_111_000_000_000( xSoaFloat1000, 
											   xSoaFloat1000, 
											   xSoaFloat1000);
const Asura_SoA_Vector_3 xVec_000_111_000_000( xSoaFloat0100, 
											   xSoaFloat0100, 
											   xSoaFloat0100);
const Asura_SoA_Vector_3 xVec_000_000_111_000( xSoaFloat0010, 
											   xSoaFloat0010, 
											   xSoaFloat0010);
const Asura_SoA_Vector_3 xVec_000_000_000_111( xSoaFloat0001, 
											   xSoaFloat0001, 
											   xSoaFloat0001);
														 
ASURA_UNITTEST(Get, Asura_SoA_Vector_3)
{
	Asura_SoA_Float xSoaFloat1471(1.0f, 4.0f, 7.0f, 1.0f);
	Asura_SoA_Float xSoaFloat2582(2.0f, 5.0f, 8.0f, 2.0f);
	Asura_SoA_Float xSoaFloat3693(3.0f, 6.0f, 9.0f, 3.0f);
	Asura_SoA_Vector_3 xVec_123_456_789_123( xSoaFloat1471, xSoaFloat2582, xSoaFloat3693 );

	ASURA_UNITTEST_CHECK( xSoaFloat1471 == xVec_123_456_789_123.GetXXXX() );
	ASURA_UNITTEST_CHECK( xSoaFloat2582 == xVec_123_456_789_123.GetYYYY() );
	ASURA_UNITTEST_CHECK( xSoaFloat3693 == xVec_123_456_789_123.GetZZZZ() );
}


ASURA_UNITTEST(DotProduct, Asura_SoA_Vector_3)
{
	ASURA_UNITTEST_CHECK( xSoaFloat0000 == xVec_001_001_001_001 * xVec_100_100_100_100 );
	ASURA_UNITTEST_CHECK( xSoaFloat0000 == xVec_010_010_010_010 * xVec_100_100_100_100 );
	ASURA_UNITTEST_CHECK( xSoaFloat0000 == xVec_100_100_100_100 * xVec_010_010_010_010 );
	ASURA_UNITTEST_CHECK( xSoaFloat1111 == xVec_100_100_100_100 * xVec_100_100_100_100 );
	ASURA_UNITTEST_CHECK( xSoaFloat1111 == xVec_010_010_010_010 * xVec_010_010_010_010 );
	ASURA_UNITTEST_CHECK( xSoaFloat1111 == xVec_001_001_001_001 * xVec_001_001_001_001 );

	ASURA_UNITTEST_CHECK( Asura_SoA_Float(14.0f) == xVec_123_123_123_123 * xVec_123_123_123_123 );
	ASURA_UNITTEST_CHECK( Asura_SoA_Float(1.0f, 2.0f, 3.0f, 0.0f) == xVec_123_123_123_123 * xVec_100_010_001_000 );
}

ASURA_UNITTEST(ApplyMatrix, Asura_SoA_Vector_3)
{
	const Asura_AMatrix3x3 xMatrix(1.0f, 2.0f, 3.0f, 4.0f, 5.0f, 6.0f, 7.0f, 8.0f, 9.0f);
	Asura_SoA_Vector_3 xVec = xVec_111_111_111_111 * xMatrix;
	ASURA_UNITTEST_CHECK( Asura_SoA_Float(6.0f, 6.0f, 6.0f, 6.0f) == xVec.GetXXXX() );
	ASURA_UNITTEST_CHECK( Asura_SoA_Float(15.0f, 15.0f, 15.0f, 15.0f) == xVec.GetYYYY() );
	ASURA_UNITTEST_CHECK( Asura_SoA_Float(24.0f, 24.0f, 24.0f, 24.0f) == xVec.GetZZZZ() );

	xVec = xVec_111_000_000_000 * xMatrix;
	ASURA_UNITTEST_CHECK( Asura_SoA_Float( 6.0f, 0.0f, 0.0f, 0.0f) == xVec.GetXXXX() );
	ASURA_UNITTEST_CHECK( Asura_SoA_Float(15.0f, 0.0f, 0.0f, 0.0f) == xVec.GetYYYY() );
	ASURA_UNITTEST_CHECK( Asura_SoA_Float(24.0f, 0.0f, 0.0f, 0.0f) == xVec.GetZZZZ() );

	xVec = xVec_000_111_000_000 * xMatrix;
	ASURA_UNITTEST_CHECK( Asura_SoA_Float(0.0f,  6.0f, 0.0f, 0.0f) == xVec.GetXXXX() );
	ASURA_UNITTEST_CHECK( Asura_SoA_Float(0.0f, 15.0f, 0.0f, 0.0f) == xVec.GetYYYY() );
	ASURA_UNITTEST_CHECK( Asura_SoA_Float(0.0f, 24.0f, 0.0f, 0.0f) == xVec.GetZZZZ() );

	xVec = xVec_000_000_111_000 * xMatrix;
	ASURA_UNITTEST_CHECK( Asura_SoA_Float(0.0f, 0.0f,  6.0f, 0.0f) == xVec.GetXXXX() );
	ASURA_UNITTEST_CHECK( Asura_SoA_Float(0.0f, 0.0f, 15.0f, 0.0f) == xVec.GetYYYY() );
	ASURA_UNITTEST_CHECK( Asura_SoA_Float(0.0f, 0.0f, 24.0f, 0.0f) == xVec.GetZZZZ() );

	xVec = xVec_000_000_000_111 * xMatrix;
	ASURA_UNITTEST_CHECK( Asura_SoA_Float(0.0f, 0.0f, 0.0f,  6.0f) == xVec.GetXXXX() );
	ASURA_UNITTEST_CHECK( Asura_SoA_Float(0.0f, 0.0f, 0.0f, 15.0f) == xVec.GetYYYY() );
	ASURA_UNITTEST_CHECK( Asura_SoA_Float(0.0f, 0.0f, 0.0f, 24.0f) == xVec.GetZZZZ() );
}

#endif

// E N D   O F   F I L E /////////////////////////////////////////////////////
