//////////////////////////////////////////////////////////////////////////////
//
// Asura_UnitTest_Maths_Vector3.cpp
//
// Copyright (c) 2005 Rebellion. All rights reserved.
//
// Description:
//
//     Unit tests for Asura_AVector3
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

#ifdef ASURA_SUPPORT_UNITTEST

#include "UnitTest/Asura_UnitTest.h"

// Asura Platform-independent
#include "Core/Asura_HalfFloat.h"
#include "Core/Asura_Maths.h"

// V E C T O R   3 ///////////////////////////////////////////////////////////
static const Asura_AVector3 xVec123(1.0f, 2.0f, 3.0f);
static const Asura_AVector3 xVec456(4.0f, 5.0f, 6.0f);
static const Asura_AVector3 xVec789(7.0f, 8.0f, 9.0f);
static const Asura_AVector3 xVec246(2.0f, 4.0f, 6.0f);
static const Asura_AVector3 xVec100(1.0f, 0.0f, 0.0f);
static const Asura_AVector3 xVec010(0.0f, 1.0f, 0.0f);
static const Asura_AVector3 xVec001(0.0f, 0.0f, 1.0f);
static const Asura_AVector3 xVec200(2.0f, 0.0f, 0.0f);
static const Asura_AVector3 xVec020(0.0f, 2.0f, 0.0f);
static const Asura_AVector3 xVec002(0.0f, 0.0f, 2.0f);
static const Asura_AVector3 xVec111(1.0f, 1.0f, 1.0f);
static const Asura_AVector3 xVec222(2.0f, 2.0f, 2.0f);
static const Asura_AVector3 xVec333(3.0f, 3.0f, 3.0f);
static const Asura_AVector3 xVecMinus111(-1.0f, -1.0f, -1.0f);
static const Asura_AVector3 xVecMinus123(-1.0f, -2.0f, -3.0f);

static const Asura_AMatrix3x3 xMatrix1to9(1.0f, 2.0f, 3.0f, 4.0f, 5.0f, 6.0f, 7.0f, 8.0f, 9.0f);
static const Asura_AMatrix3x3 xMatrix9to1(9.0f, 8.0f, 7.0f, 6.0f, 5.0f, 4.0f, 3.0f, 2.0f, 1.0f);


ASURA_UNITTEST(Constructors, Asura_AVector3)
{
	// from other vector
	Asura_AVector3 xVectorFromOtherVec(xVec123);
	ASURA_UNITTEST_CHECK( xVec123 == xVectorFromOtherVec );

	// from 3 floats
	Asura_AVector3 xVectorFrom3Floats(1.0f, 2.0f, 3.0f);
	ASURA_UNITTEST_CHECK( xVec123 == xVectorFrom3Floats );

	// from vector 3
	Asura_Vector_3 xVec3(1.0f, 2.0f, 3.0f);
	Asura_AVector3 xVectorFromVec3(xVec3);
	ASURA_UNITTEST_CHECK( xVec123 == xVectorFromVec3 );
}

ASURA_UNITTEST(Assignment, Asura_AVector3)
{
	// from vector 3
	Asura_Vector_3 xVector3(1.0f, 2.0f, 3.0f);
	Asura_AVector3 xVectorFromOtherVec(0.0f, 0.0f, 0.0f);
	xVectorFromOtherVec = xVector3;
	ASURA_UNITTEST_CHECK( xVec123 == xVectorFromOtherVec );

	// from vector 4
	Asura_AVector3 xVector4(1.0f, 2.0f, 3.0f);
	Asura_AVector3 xVectorFromVec4(0.0f, 0.0f, 0.0f);
	xVectorFromVec4 = xVector4;
	ASURA_UNITTEST_CHECK( xVec123 == xVectorFromVec4 );
}

ASURA_UNITTEST(LoadAndStore, Asura_AVector3)
{
	// from float array
	float* pfFloats = reinterpret_cast<float*>(Asura_AllocateAlign(6*sizeof(float), ASURA_ALIGN_16_BYTE));
	// fill out array
	{
		float afFloats[6] = {1.0f, 2.0f, 3.0f, 4.0f, 5.0f, 6.0f};
		memcpy( pfFloats, afFloats, 6*sizeof(float) );
	}
	Asura_AVector3 xVectorFromFloats;

	xVectorFromFloats.Zero();
	xVectorFromFloats.LoadAligned(pfFloats);
	ASURA_UNITTEST_CHECK( xVec123 == xVectorFromFloats );

	xVectorFromFloats.Zero();
	xVectorFromFloats.LoadUnaligned(&pfFloats[3]);
	ASURA_UNITTEST_CHECK( xVec456 == xVectorFromFloats );

	// to float array
	memset(pfFloats, 0, 6*sizeof(float));
	xVec456.StoreAligned(pfFloats);
	xVec123.StoreUnaligned(&pfFloats[3]);
	ASURA_UNITTEST_CHECK( pfFloats[0] == 4.0f );
	ASURA_UNITTEST_CHECK( pfFloats[1] == 5.0f );
	ASURA_UNITTEST_CHECK( pfFloats[2] == 6.0f );
	ASURA_UNITTEST_CHECK( pfFloats[3] == 1.0f );
	ASURA_UNITTEST_CHECK( pfFloats[4] == 2.0f );
	ASURA_UNITTEST_CHECK( pfFloats[5] == 3.0f );

	Asura_DeallocateAlign(pfFloats);

	// to half array
	Asura_HalfFloat axHalves[6];
	xVec456.StoreAlignedHalf(&axHalves[0]);
	xVec123.StoreAlignedHalf(&axHalves[3]);
	ASURA_UNITTEST_CHECK( axHalves[0] == 4.0f );
	ASURA_UNITTEST_CHECK( axHalves[1] == 5.0f );
	ASURA_UNITTEST_CHECK( axHalves[2] == 6.0f );
	ASURA_UNITTEST_CHECK( axHalves[3] == 1.0f );
	ASURA_UNITTEST_CHECK( axHalves[4] == 2.0f );
	ASURA_UNITTEST_CHECK( axHalves[5] == 3.0f );
}

ASURA_UNITTEST(PlusEquals, Asura_AVector3)
{
	Asura_AVector3 xVector;
	xVector.Zero();
	xVector += xVec100;
	ASURA_UNITTEST_CHECK( xVector == xVec100 );
	xVector += xVec010;
	xVector += xVec001;
	ASURA_UNITTEST_CHECK( xVector == xVec111 );
	xVector = xVec123;
	xVector += xVecMinus123;
	ASURA_UNITTEST_CHECK( xVector == Asura_Maths::ZeroAVector3 );
}

ASURA_UNITTEST(MinusEquals, Asura_AVector3)
{
	Asura_AVector3 xVector;
	xVector.Zero();
	xVector -= xVec100;
	xVector -= xVec010;
	xVector -= xVec001;
	ASURA_UNITTEST_CHECK( xVector == xVecMinus111 );

	xVector = xVec123;
	xVector -= xVec123;
	ASURA_UNITTEST_CHECK( xVector == Asura_Maths::ZeroAVector3 );
}

ASURA_UNITTEST(TimesEquals, Asura_AVector3)
{
	Asura_AVector3 xVector(xVec123);
	xVector *= 2.0f;
	ASURA_UNITTEST_CHECK( xVector == xVec246 );

	xVector = xVec123;
	xVector *= -1.0f;
	ASURA_UNITTEST_CHECK( xVector == xVecMinus123 );

	xVector = xVec123;
	xVector *= 0.0f;
	ASURA_UNITTEST_CHECK( xVector == Asura_Maths::ZeroAVector3 );
}

ASURA_UNITTEST(DivideEquals, Asura_AVector3)
{
	Asura_AVector3 xVector(xVec246);
	xVector /= 2.0f;
	ASURA_UNITTEST_CHECK( xVector.ApproximatelyEquals(xVec123) );

	xVector = xVec123;
	xVector /= -1.0f;
	ASURA_UNITTEST_CHECK( xVector.ApproximatelyEquals( xVecMinus123 ) );

	xVector = xVec123;
	xVector /= 1.0f;
	ASURA_UNITTEST_CHECK( xVector.ApproximatelyEquals( xVec123 ) );
}

ASURA_UNITTEST(UnaryMinus, Asura_AVector3)
{
	ASURA_UNITTEST_CHECK( -xVec123 == xVecMinus123 );
}

ASURA_UNITTEST(Addition, Asura_AVector3)
{
	Asura_AVector3 xVector;
	xVector = xVec123 + xVec123;
	ASURA_UNITTEST_CHECK( xVector == xVec246 );

	xVector = xVec123 + xVecMinus123;
	ASURA_UNITTEST_CHECK( xVector ==  Asura_Maths::ZeroAVector3);

	xVector = xVec123 + Asura_Maths::ZeroAVector3;
	ASURA_UNITTEST_CHECK( xVector == xVec123 );

	xVector = xVec100 + xVec010 + xVec001;
	ASURA_UNITTEST_CHECK( xVector == xVec111 );
}

ASURA_UNITTEST(Subtraction, Asura_AVector3)
{
	Asura_AVector3 xVector;
	xVector = ((Asura_Maths::ZeroAVector3 - xVec100) - xVec010) - xVec001;
	ASURA_UNITTEST_CHECK( xVector == xVecMinus111 );

	xVector = xVec123 - xVec123;
	ASURA_UNITTEST_CHECK( xVector == Asura_Maths::ZeroAVector3 );

	xVector = xVec123 - xVecMinus123;
	ASURA_UNITTEST_CHECK( xVector == xVec246 );
}

ASURA_UNITTEST(ScalarMultiplication, Asura_AVector3)
{
	Asura_AVector3 xVector;
	xVector = xVec123 * 2.0f;
	ASURA_UNITTEST_CHECK( xVector.ApproximatelyEquals(xVec246) );

	xVector = xVec123 * -1.0f;
	ASURA_UNITTEST_CHECK( xVector.ApproximatelyEquals(xVecMinus123) );

	xVector = xVec123 * 0.0f;
	ASURA_UNITTEST_CHECK( xVector.ApproximatelyEquals(Asura_Maths::ZeroAVector3) );
}

ASURA_UNITTEST(ScalarDivision, Asura_AVector3)
{
	Asura_AVector3 xVector;
	xVector = xVec246 / 2.0f;
	ASURA_UNITTEST_CHECK( xVector.ApproximatelyEquals(xVec123) );

	xVector = xVec123 / -1.0f;
	ASURA_UNITTEST_CHECK( xVector.ApproximatelyEquals(xVecMinus123) );

	xVector = xVec123 / 1.0f;
	ASURA_UNITTEST_CHECK( xVector.ApproximatelyEquals(xVec123) );
}

ASURA_UNITTEST(DotProduct, Asura_AVector3)
{
	ASURA_UNITTEST_FLOATS_EQUAL( 0.0f, xVec001 * xVec100, 0.0f );
	ASURA_UNITTEST_FLOATS_EQUAL( 0.0f, xVec010 * xVec100, 0.0f );
	ASURA_UNITTEST_FLOATS_EQUAL( 0.0f, xVec100 * xVec010, 0.0f );
	ASURA_UNITTEST_FLOATS_EQUAL( 1.0f, xVec100 * xVec100, 0.0f );

	Asura_AVector3 xVector(xVec456);
	xVector.Normalise();
	ASURA_UNITTEST_FLOATS_EQUAL( 1.0f, xVector * xVector, 1.0e-6f );
	ASURA_UNITTEST_FLOATS_EQUAL( 6.0f, xVec123 * xVec111, 1.0e-6f );
	ASURA_UNITTEST_FLOATS_EQUAL( 1.0f, xVec123 * xVec100, 1.0e-6f );
	ASURA_UNITTEST_FLOATS_EQUAL( 2.0f, xVec123 * xVec010, 1.0e-6f );
	ASURA_UNITTEST_FLOATS_EQUAL( 3.0f, xVec123 * xVec001, 1.0e-6f );
}

ASURA_UNITTEST(CrossProduct, Asura_AVector3)
{
	ASURA_UNITTEST_CHECK( xVec100.ApproximatelyEquals(xVec010 ^ xVec001) );
	ASURA_UNITTEST_CHECK( xVec010.ApproximatelyEquals(xVec001 ^ xVec100) );
	ASURA_UNITTEST_CHECK( xVec001.ApproximatelyEquals(xVec100 ^ xVec010) );
	ASURA_UNITTEST_CHECK( (xVec123 ^ xVec456) == -(xVec456 ^ xVec123) );
}

ASURA_UNITTEST(Equality, Asura_AVector3)
{
	ASURA_UNITTEST_CHECK( xVec100 == xVec100 );
	ASURA_UNITTEST_CHECK( xVec010 == xVec010 );
	ASURA_UNITTEST_CHECK( xVec001 == xVec001 );
	ASURA_UNITTEST_CHECK( !(xVec001 == xVec123) );
	ASURA_UNITTEST_CHECK( xVecMinus123 == xVecMinus123 );
	ASURA_UNITTEST_CHECK( !(xVec123 == xVecMinus123) );
}

ASURA_UNITTEST(ApproximateEquality, Asura_AVector3)
{
	Asura_AVector3 xVec1(0.1f, 0.2f, 0.3f);
	Asura_AVector3 xVec2(0.1001f, 0.2001f, 0.30001f);
	Asura_AVector3 xVec3(1.001f, 0.2001f, 0.30001f);

	ASURA_UNITTEST_CHECK( xVec1.ApproximatelyEquals(xVec2) );
	ASURA_UNITTEST_CHECK( !xVec1.ApproximatelyEquals(xVec3) );
}

ASURA_UNITTEST(Inequality, Asura_AVector3)
{
	ASURA_UNITTEST_CHECK( !(xVec100 != xVec100) );
	ASURA_UNITTEST_CHECK( !(xVec010 != xVec010) );
	ASURA_UNITTEST_CHECK( !(xVec001 != xVec001) );
	ASURA_UNITTEST_CHECK( xVec001 != xVec123 );
	ASURA_UNITTEST_CHECK( !(xVecMinus123 != xVecMinus123) );
	ASURA_UNITTEST_CHECK( xVec123 != xVecMinus123 );
}

ASURA_UNITTEST(Comparisons, Asura_AVector3)
{
	// equality part of <= and >=
	ASURA_UNITTEST_CHECK( (xVec100 <= xVec100) );
	ASURA_UNITTEST_CHECK( (xVec010 <= xVec010) );
	ASURA_UNITTEST_CHECK( (xVec001 <= xVec001) );
	ASURA_UNITTEST_CHECK( (xVec123 <= xVec123) );
	ASURA_UNITTEST_CHECK( (xVec100 >= xVec100) );
	ASURA_UNITTEST_CHECK( (xVec010 >= xVec010) );
	ASURA_UNITTEST_CHECK( (xVec001 >= xVec001) );
	ASURA_UNITTEST_CHECK( (xVec123 >= xVec123) );

	// lt, le
	ASURA_UNITTEST_CHECK( (xVec001 < xVec123) );
	ASURA_UNITTEST_CHECK( (xVec123 < xVec456) );
	ASURA_UNITTEST_CHECK( (xVec111 <= xVec123) );
	ASURA_UNITTEST_CHECK( !(xVec111 < xVec123) );
	ASURA_UNITTEST_CHECK( (xVecMinus123 < xVec123) );
	ASURA_UNITTEST_CHECK( (xVecMinus123 <= xVec123) );

	ASURA_UNITTEST_CHECK( !(xVec123 < xVec001) );
	ASURA_UNITTEST_CHECK( !(xVec456 < xVec123) );
	ASURA_UNITTEST_CHECK( !(xVec123 <= xVec111) );
	ASURA_UNITTEST_CHECK( !(xVec123 < xVec111) );
	ASURA_UNITTEST_CHECK( !(xVec123 < xVecMinus123) );
	ASURA_UNITTEST_CHECK( !(xVec123 <= xVecMinus123) );

	// gt, ge
	ASURA_UNITTEST_CHECK( !(xVec001 > xVec123) );
	ASURA_UNITTEST_CHECK( !(xVec123 > xVec456) );
	ASURA_UNITTEST_CHECK( !(xVec111 >= xVec123) );
	ASURA_UNITTEST_CHECK( !(xVec111 > xVec123) );
	ASURA_UNITTEST_CHECK( !(xVecMinus123 > xVec123) );
	ASURA_UNITTEST_CHECK( !(xVecMinus123 >= xVec123) );

	ASURA_UNITTEST_CHECK( (xVec123 >= xVec001) );
	ASURA_UNITTEST_CHECK( (xVec456 > xVec123) );
	ASURA_UNITTEST_CHECK( (xVec123 >= xVec111) );
	ASURA_UNITTEST_CHECK( (xVec123 >= xVec111) );
	ASURA_UNITTEST_CHECK( (xVec123 > xVecMinus123) );
	ASURA_UNITTEST_CHECK( (xVec123 >= xVecMinus123) );
}

//ASURA_UNITTEST(VectorMultiply, Asura_AVector3)
//{
//	ASURA_UNITTEST_CHECK( (xVec100.Vector_Multiply( xVec100 )) == xVec100 );
//	ASURA_UNITTEST_CHECK( (xVec010.Vector_Multiply( xVec010 )) == xVec010 );
//	ASURA_UNITTEST_CHECK( (xVec001.Vector_Multiply( xVec001 )) == xVec001 );
//	ASURA_UNITTEST_CHECK( (xVec123.Vector_Multiply( xVec001 )) == Asura_AVector3(0.0f, 0.0f, 3.0f) );
//	ASURA_UNITTEST_CHECK( (xVec123.Vector_Multiply( xVec010 )) == Asura_AVector3(0.0f, 2.0f, 0.0f) );
//	ASURA_UNITTEST_CHECK( (xVec123.Vector_Multiply( xVec100 )) == Asura_AVector3(1.0f, 0.0f, 0.0f) );
//	ASURA_UNITTEST_CHECK( (xVec123.Vector_Multiply( xVec123 )) == Asura_AVector3(1.0f, 4.0f, 9.0f) );
//}

ASURA_UNITTEST(VectorTimesEquals, Asura_AVector3)
{
	Asura_AVector3 xTemp;
	xTemp = xVec100;
	xTemp *= xVec100;
  	ASURA_UNITTEST_CHECK( xTemp == xVec100 );

	xTemp = xVec010;
	xTemp *= xVec010;
	ASURA_UNITTEST_CHECK( xTemp == xVec010 );

	xTemp = xVec001;
	xTemp *= xVec001;
	ASURA_UNITTEST_CHECK( xTemp == xVec001 );

	xTemp = xVec123;
	xTemp *= xVec001;
  	ASURA_UNITTEST_CHECK( Asura_AVector3(0.0f, 0.0f, 3.0f) == xTemp );

	xTemp = xVec123;
	xTemp *= xVec010;
  	ASURA_UNITTEST_CHECK( Asura_AVector3(0.0f, 2.0f, 0.0f) == xTemp );
  	
	xTemp = xVec123;
	xTemp *= xVec100;
	ASURA_UNITTEST_CHECK( Asura_AVector3(1.0f, 0.0f, 0.0f) == xTemp );
}

ASURA_UNITTEST(Normalise, Asura_AVector3)
{
	Asura_AVector3 xVector = xVec001;
	xVector.Normalise();
	ASURA_UNITTEST_CHECK( xVector.ApproximatelyEquals(xVec001) );
	xVector = xVec010;
	xVector.Normalise();
	ASURA_UNITTEST_CHECK( xVector.ApproximatelyEquals(xVec010) );
	xVector = xVec100;
	xVector.Normalise();
	ASURA_UNITTEST_CHECK( xVector.ApproximatelyEquals(xVec100) );

	xVector = Asura_AVector3(2.0f, 0.0f, 0.0f);
	xVector.Normalise();
	ASURA_UNITTEST_CHECK( xVector.ApproximatelyEquals(xVec100) );
	xVector = Asura_AVector3(0.0f, 2.0f, 0.0f);
	xVector.Normalise();
	ASURA_UNITTEST_CHECK( xVector.ApproximatelyEquals(xVec010) );
	xVector = Asura_AVector3(0.0f, 0.0f, 2.0f);
	xVector.Normalise();
	ASURA_UNITTEST_CHECK( xVector.ApproximatelyEquals(xVec001) );

	xVector = xVec456;
	xVector.Normalise();
	ASURA_UNITTEST_FLOATS_EQUAL( 1.0f, xVector.Magnitude(), 0.001f );
	
	// DLH - 31 January 2007 15:17:40 - This checks for the required accuracy from the Normalise() routine. It has been assumed that MagnitudeSquared does is just a dp3 and no sqrt to confuse the issue.
	// DLH - 31 January 2007 15:18:36 - The present tolerance is setup to wotk on the Xbox360
	xVector = Asura_AVector3( 1.0f, 2.0f, 3.0f);
	xVector.Normalise();
	ASURA_UNITTEST_FLOATS_EQUAL( 1.0f, xVector.MagnitudeSquared(), 0.000001f );
}

ASURA_UNITTEST(NormaliseReturningMagnitude, Asura_AVector3)
{
	Asura_AVector3 xVector = xVec001;
	ASURA_UNITTEST_FLOATS_EQUAL( 1.0f, xVector.NormaliseReturningMagnitude(), 0.001f) ;
	ASURA_UNITTEST_CHECK( xVector.ApproximatelyEquals(xVec001) );
	xVector = xVec010;
	ASURA_UNITTEST_FLOATS_EQUAL( 1.0f, xVector.NormaliseReturningMagnitude(), 0.001f) ;
	ASURA_UNITTEST_CHECK( xVector.ApproximatelyEquals(xVec010) );
	xVector = xVec100;
	ASURA_UNITTEST_FLOATS_EQUAL( 1.0f, xVector.NormaliseReturningMagnitude(), 0.001f) ;
	ASURA_UNITTEST_CHECK( xVector.ApproximatelyEquals(xVec100) );

	xVector = Asura_AVector3(2.0f, 0.0f, 0.0f);
	ASURA_UNITTEST_FLOATS_EQUAL( 2.0f, xVector.NormaliseReturningMagnitude(), 0.001f) ;
	ASURA_UNITTEST_CHECK( xVector.ApproximatelyEquals(xVec100) );
	xVector = Asura_AVector3(0.0f, 2.0f, 0.0f);
	ASURA_UNITTEST_FLOATS_EQUAL( 2.0f, xVector.NormaliseReturningMagnitude(), 0.001f) ;
	ASURA_UNITTEST_CHECK( xVector.ApproximatelyEquals(xVec010) );
	xVector = Asura_AVector3(0.0f, 0.0f, 2.0f);
	ASURA_UNITTEST_FLOATS_EQUAL( 2.0f, xVector.NormaliseReturningMagnitude(), 0.001f) ;
	ASURA_UNITTEST_CHECK( xVector.ApproximatelyEquals(xVec001) );

	xVector = xVec456;
	ASURA_UNITTEST_FLOATS_EQUAL( Asura_Maths::Sqrt(16.0f + 25.0f + 36.0f), xVector.NormaliseReturningMagnitude(), 0.001f) ;
	ASURA_UNITTEST_FLOATS_EQUAL( 1.0f, xVector.Magnitude(), 0.001f );

	xVector.Zero();
	Asura_AVector3 xVecZero = xVector;
	ASURA_UNITTEST_CHECK( xVector.NormaliseReturningMagnitude() == 0.0f );
	ASURA_UNITTEST_CHECK( xVector == xVecZero );
	ASURA_UNITTEST_CHECK( (xVector + xVec123) == xVec123 );
}

ASURA_UNITTEST(Magnitude, Asura_AVector3)
{
	ASURA_UNITTEST_FLOATS_EQUAL( 1.0f, xVec001.Magnitude(), 0.001f );
	ASURA_UNITTEST_FLOATS_EQUAL( 1.0f, xVec010.Magnitude(), 0.001f );
	ASURA_UNITTEST_FLOATS_EQUAL( 1.0f, xVec100.Magnitude(), 0.001f );
	ASURA_UNITTEST_FLOATS_EQUAL( Asura_Maths::Sqrt(3.0f), xVec111.Magnitude(), 0.001f );
	ASURA_UNITTEST_FLOATS_EQUAL( Asura_Maths::Sqrt(4.0f + 16.0f + 36.0f), xVec246.Magnitude(), 0.001f );
	ASURA_UNITTEST_FLOATS_EQUAL( Asura_Maths::Sqrt(1.0f + 4.0f + 9.0f), xVec123.Magnitude(), 0.001f );
}

ASURA_UNITTEST(MagnitudeSquared, Asura_AVector3)
{
	ASURA_UNITTEST_FLOATS_EQUAL( 1.0f, xVec001.MagnitudeSquared(), 0.001f );
	ASURA_UNITTEST_FLOATS_EQUAL( 1.0f, xVec010.MagnitudeSquared(), 0.001f );
	ASURA_UNITTEST_FLOATS_EQUAL( 1.0f, xVec100.MagnitudeSquared(), 0.001f );
	ASURA_UNITTEST_FLOATS_EQUAL( 3.0f, xVec111.MagnitudeSquared(), 0.001f );
	ASURA_UNITTEST_FLOATS_EQUAL( (4.0f + 16.0f + 36.0f), xVec246.MagnitudeSquared(), 0.001f );
	ASURA_UNITTEST_FLOATS_EQUAL( (1.0f + 4.0f + 9.0f), xVec123.MagnitudeSquared(), 0.001f );
}

ASURA_UNITTEST(Zero, Asura_AVector3)
{
	Asura_AVector3 xVector = xVec001;
	xVector.Zero();
	ASURA_UNITTEST_CHECK( xVector == Asura_Maths::ZeroAVector3);
	xVector = xVec010;
	xVector.Zero();
	ASURA_UNITTEST_CHECK( xVector == Asura_Maths::ZeroAVector3);
	xVector = xVec100;
	xVector.Zero();
	ASURA_UNITTEST_CHECK( xVector == Asura_Maths::ZeroAVector3);
	xVector = xVec456;
	xVector.Zero();
	ASURA_UNITTEST_CHECK( xVector == Asura_Maths::ZeroAVector3);
}

ASURA_UNITTEST(GetXYZ, Asura_AVector3)
{
	ASURA_UNITTEST_FLOATS_EQUAL( 1.0f, xVec123.GetX(), 0.0f );
	ASURA_UNITTEST_FLOATS_EQUAL( 2.0f, xVec123.GetY(), 0.0f );
	ASURA_UNITTEST_FLOATS_EQUAL( 3.0f, xVec123.GetZ(), 0.0f );
	ASURA_UNITTEST_FLOATS_EQUAL( 4.0f, xVec456.GetX(), 0.0f );
	ASURA_UNITTEST_FLOATS_EQUAL( 5.0f, xVec456.GetY(), 0.0f );
	ASURA_UNITTEST_FLOATS_EQUAL( 6.0f, xVec456.GetZ(), 0.0f );
}

ASURA_UNITTEST(SetXYZ, Asura_AVector3)
{
	Asura_AVector3 xVec4;

	xVec4.Zero();
	xVec4.SetX(1.0f);
	ASURA_UNITTEST_CHECK( xVec4 == xVec100 );

	xVec4.Zero();
	xVec4.SetY(1.0f);
	ASURA_UNITTEST_CHECK( xVec4 == xVec010 );

	xVec4.Zero();
	xVec4.SetZ(1.0f);
	ASURA_UNITTEST_CHECK( xVec4 == xVec001 );

	xVec4.Zero();
	xVec4.SetX(4.0f);
	xVec4.SetY(5.0f);
	xVec4.SetZ(6.0f);
	ASURA_UNITTEST_CHECK( xVec4 == xVec456 );
}

ASURA_UNITTEST(GetElement, Asura_AVector3)
{
	ASURA_UNITTEST_FLOATS_EQUAL( 1.0f, xVec123.GetElement(0), 0.0f );
	ASURA_UNITTEST_FLOATS_EQUAL( 2.0f, xVec123.GetElement(1), 0.0f );
	ASURA_UNITTEST_FLOATS_EQUAL( 3.0f, xVec123.GetElement(2), 0.0f );
	ASURA_UNITTEST_FLOATS_EQUAL( 4.0f, xVec456.GetElement(0), 0.0f );
	ASURA_UNITTEST_FLOATS_EQUAL( 5.0f, xVec456.GetElement(1), 0.0f );
	ASURA_UNITTEST_FLOATS_EQUAL( 6.0f, xVec456.GetElement(2), 0.0f );
}

ASURA_UNITTEST(SetElement, Asura_AVector3)
{
	Asura_AVector3 xVec4;

	xVec4.Zero();
	xVec4.SetElement(0, 1.0f);
	ASURA_UNITTEST_CHECK( xVec4 == xVec100 );

	xVec4.Zero();
	xVec4.SetElement(1, 1.0f);
	ASURA_UNITTEST_CHECK( xVec4 == xVec010 );

	xVec4.Zero();
	xVec4.SetElement(2, 1.0f);
	ASURA_UNITTEST_CHECK( xVec4 == xVec001 );

	xVec4.Zero();
	xVec4.SetElement(0, 4.0f);
	xVec4.SetElement(1, 5.0f);
	xVec4.SetElement(2, 6.0f);
	ASURA_UNITTEST_CHECK( xVec4 == xVec456 );
}

ASURA_UNITTEST(Select, Asura_AVector3)
{
	Asura_AVector3 xTemp;

	// EQ
	xTemp.SelectEQ(xVec123, xVec123, xVec456, xVec789);
	ASURA_UNITTEST_CHECK( xVec456 == xTemp );
	xTemp.SelectEQ(xVec123, xVec111, xVec456, xVec789);
	ASURA_UNITTEST_CHECK( Asura_AVector3(4.0f, 8.0f, 9.0f) == xTemp );
	xTemp.SelectEQ(xVec123, xVec222, xVec456, xVec789);
	ASURA_UNITTEST_CHECK( Asura_AVector3(7.0f, 5.0f, 9.0f) == xTemp );
	xTemp.SelectEQ(xVec123, xVec333, xVec456, xVec789);
	ASURA_UNITTEST_CHECK( Asura_AVector3(7.0f, 8.0f, 6.0f) == xTemp );

	// NE
	xTemp.SelectNE(xVec123, xVec123, xVec456, xVec789);
	ASURA_UNITTEST_CHECK( xVec789 == xTemp );
	xTemp.SelectNE(xVec123, xVec111, xVec456, xVec789);
	ASURA_UNITTEST_CHECK( Asura_AVector3(7.0f, 5.0f, 6.0f) == xTemp );
	xTemp.SelectNE(xVec123, xVec222, xVec456, xVec789);
	ASURA_UNITTEST_CHECK( Asura_AVector3(4.0f, 8.0f, 6.0f) == xTemp );
	xTemp.SelectNE(xVec123, xVec333, xVec456, xVec789);
	ASURA_UNITTEST_CHECK( Asura_AVector3(4.0f, 5.0f, 9.0f) == xTemp );

	// LT
	xTemp.SelectLT(xVec123, xVec123, xVec456, xVec789);
	ASURA_UNITTEST_CHECK( xVec789 == xTemp );
	xTemp.SelectLT(xVec456, xVec123, xVec456, xVec789);
	ASURA_UNITTEST_CHECK( xVec789 == xTemp );
	xTemp.SelectLT(xVec123, xVec456, xVec456, xVec789);
	ASURA_UNITTEST_CHECK( xVec456 == xTemp );
	xTemp.SelectLT(xVec001, xVec111, xVec456, xVec789);
	ASURA_UNITTEST_CHECK( Asura_AVector3(4.0f, 5.0f, 9.0f) == xTemp );
	xTemp.SelectLT(xVec010, xVec111, xVec456, xVec789);
	ASURA_UNITTEST_CHECK( Asura_AVector3(4.0f, 8.0f, 6.0f) == xTemp );
	xTemp.SelectLT(xVec100, xVec111, xVec456, xVec789);
	ASURA_UNITTEST_CHECK( Asura_AVector3(7.0f, 5.0f, 6.0f) == xTemp );

	// LE
	xTemp.SelectLE(xVec123, xVec123, xVec456, xVec789);
	ASURA_UNITTEST_CHECK( xVec456 == xTemp );
	xTemp.SelectLE(xVec456, xVec123, xVec456, xVec789);
	ASURA_UNITTEST_CHECK( xVec789 == xTemp );
	xTemp.SelectLE(xVec123, xVec456, xVec456, xVec789);
	ASURA_UNITTEST_CHECK( xVec456 == xTemp );
	xTemp.SelectLE(xVec002, xVec111, xVec456, xVec789);
	ASURA_UNITTEST_CHECK( Asura_AVector3(4.0f, 5.0f, 9.0f) == xTemp );
	xTemp.SelectLE(xVec020, xVec111, xVec456, xVec789);
	ASURA_UNITTEST_CHECK( Asura_AVector3(4.0f, 8.0f, 6.0f) == xTemp );
	xTemp.SelectLE(xVec200, xVec111, xVec456, xVec789);
	ASURA_UNITTEST_CHECK( Asura_AVector3(7.0f, 5.0f, 6.0f) == xTemp );

	// GT
	xTemp.SelectGT(xVec123,  xVec123, xVec456, xVec789);
	ASURA_UNITTEST_CHECK( xVec789 == xTemp );
	xTemp.SelectGT(xVec123,  xVec456, xVec456, xVec789);
	ASURA_UNITTEST_CHECK( xVec789 == xTemp );
	xTemp.SelectGT(xVec456,  xVec123, xVec456, xVec789);
	ASURA_UNITTEST_CHECK( xVec456 == xTemp );
	xTemp.SelectGT(xVec111,  xVec001, xVec456, xVec789);
	ASURA_UNITTEST_CHECK( Asura_AVector3(4.0f, 5.0f, 9.0f) == xTemp );
	xTemp.SelectGT(xVec111,  xVec010, xVec456, xVec789);
	ASURA_UNITTEST_CHECK( Asura_AVector3(4.0f, 8.0f, 6.0f) == xTemp );
	xTemp.SelectGT(xVec111,  xVec100, xVec456, xVec789);
	ASURA_UNITTEST_CHECK( Asura_AVector3(7.0f, 5.0f, 6.0f) == xTemp );

	// GE
	xTemp.SelectGE(xVec123,  xVec123, xVec456, xVec789);
	ASURA_UNITTEST_CHECK( xVec456 == xTemp );
	xTemp.SelectGE(xVec123,  xVec456, xVec456, xVec789);
	ASURA_UNITTEST_CHECK( xVec789 == xTemp );
	xTemp.SelectGE(xVec456,  xVec123, xVec456, xVec789);
	ASURA_UNITTEST_CHECK( xVec456 == xTemp );
	xTemp.SelectGE(xVec111,  xVec002, xVec456, xVec789);
	ASURA_UNITTEST_CHECK( Asura_AVector3(4.0f, 5.0f, 9.0f) == xTemp );
	xTemp.SelectGE(xVec111,  xVec020, xVec456, xVec789);
	ASURA_UNITTEST_CHECK( Asura_AVector3(4.0f, 8.0f, 6.0f) == xTemp );
	xTemp.SelectGE(xVec111,  xVec200, xVec456, xVec789);
	ASURA_UNITTEST_CHECK( Asura_AVector3(7.0f, 5.0f, 6.0f) == xTemp );
}

// M A T R I X   3 X 3 ///////////////////////////////////////////////////////
ASURA_UNITTEST(Constructors, Asura_AMatrix3x3)
{
	// from 9 floats
	const Asura_AMatrix3x3 xMatrix(1.0f, 2.0f, 3.0f, 4.0f, 5.0f, 6.0f, 7.0f, 8.0f, 9.0f);
	ASURA_UNITTEST_CHECK( 1.0f == xMatrix.GetElement(0,0) );
	ASURA_UNITTEST_CHECK( 2.0f == xMatrix.GetElement(0,1) );
	ASURA_UNITTEST_CHECK( 3.0f == xMatrix.GetElement(0,2) );
	ASURA_UNITTEST_CHECK( 4.0f == xMatrix.GetElement(1,0) );
	ASURA_UNITTEST_CHECK( 5.0f == xMatrix.GetElement(1,1) );
	ASURA_UNITTEST_CHECK( 6.0f == xMatrix.GetElement(1,2) );
	ASURA_UNITTEST_CHECK( 7.0f == xMatrix.GetElement(2,0) );
	ASURA_UNITTEST_CHECK( 8.0f == xMatrix.GetElement(2,1) );
	ASURA_UNITTEST_CHECK( 9.0f == xMatrix.GetElement(2,2) );

	// from 3 columns
	const Asura_AMatrix3x3 xMatrixFromColumns( xVec123, xVec456, xVec789 );
	ASURA_UNITTEST_CHECK( xVec123 == xMatrixFromColumns.GetColumn0() );
	ASURA_UNITTEST_CHECK( xVec456 == xMatrixFromColumns.GetColumn1() );
	ASURA_UNITTEST_CHECK( xVec789 == xMatrixFromColumns.GetColumn2() );

	// from other matrix
	Asura_AMatrix3x3 xMatrixFromOtherMatrix( xMatrix );
	ASURA_UNITTEST_CHECK( xMatrix == xMatrixFromOtherMatrix );
}

ASURA_UNITTEST(FromToMat3, Asura_AMatrix3x3)
{
	const Asura_Matrix_3x3 xMat3(1.0f, 2.0f, 3.0f, 4.0f, 5.0f, 6.0f, 7.0f, 8.0f, 9.0f);
	Asura_Matrix_3x3 xMat3Temp;
	Asura_AMatrix3x3 xMat4Temp;
	
	xMatrix1to9.ToMat3(xMat3Temp);
	ASURA_UNITTEST_CHECK( xMat3Temp == xMat3 );

	xMat4Temp.FromMat3(xMat3);
	ASURA_UNITTEST_CHECK( xMat4Temp == xMatrix1to9 );
}

ASURA_UNITTEST(Assignment, Asura_AMatrix3x3)
{
	Asura_AMatrix3x3 xMatrix;
	
	xMatrix	= xMatrix1to9;
	ASURA_UNITTEST_CHECK( xMatrix1to9 == xMatrix );
}

ASURA_UNITTEST(GetColumns, Asura_AMatrix3x3)
{
	ASURA_UNITTEST_CHECK( Asura_AVector3(1.0f, 4.0f, 7.0f) == xMatrix1to9.GetColumn0() );
	ASURA_UNITTEST_CHECK( Asura_AVector3(2.0f, 5.0f, 8.0f) == xMatrix1to9.GetColumn1() );
	ASURA_UNITTEST_CHECK( Asura_AVector3(3.0f, 6.0f, 9.0f) == xMatrix1to9.GetColumn2() );
}

ASURA_UNITTEST(Transpose, Asura_AMatrix3x3)
{
	Asura_AMatrix3x3 xMatrix(xMatrix1to9);
	xMatrix.Transpose();
	ASURA_UNITTEST_CHECK( Asura_AMatrix3x3(1.0f, 4.0f, 7.0f, 2.0f, 5.0f, 8.0f, 3.0f, 6.0f, 9.0f) == xMatrix );
}

ASURA_UNITTEST(ApplyMatrixToVector, Asura_AMatrix3x3)
{
	Asura_AVector3 xVec = xVec111 * xMatrix1to9;
	ASURA_UNITTEST_CHECK( xVec.ApproximatelyEquals(Asura_AVector3(6.0f, 15.0f, 24.0f)) );
}

ASURA_UNITTEST(ApplyTransposeToVector, Asura_AMatrix3x3)
{
	Asura_AVector3 xVec = xMatrix1to9.ApplyTranspose(xVec111);
	ASURA_UNITTEST_CHECK( xVec.ApproximatelyEquals(Asura_AVector3(12.0f, 15.0f, 18.0f)) );
}

ASURA_UNITTEST(Identity, Asura_AMatrix3x3)
{
	Asura_AMatrix3x3 xMatrix(1.0f, 0.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f, 0.0f, 1.0f);
	Asura_AMatrix3x3 xMatrix2;
	xMatrix2.Identity();
	ASURA_UNITTEST_CHECK( Asura_Maths::IdentityAMatrix3x3 == xMatrix );
	ASURA_UNITTEST_CHECK( Asura_Maths::IdentityAMatrix3x3 == xMatrix2 );
}

ASURA_UNITTEST(MatrixMultiply, Asura_AMatrix3x3)
{
    Asura_AMatrix3x3 xMatrix = xMatrix1to9 * xMatrix9to1;
	ASURA_UNITTEST_CHECK( Asura_AVector3(90.0f, 54.0f, 18.0f).ApproximatelyEquals(xMatrix.GetColumn0()) );
	ASURA_UNITTEST_CHECK( Asura_AVector3(114.0f, 69.0f, 24.0f).ApproximatelyEquals(xMatrix.GetColumn1()) );
	ASURA_UNITTEST_CHECK( Asura_AVector3(138.0f, 84.0f, 30.0f).ApproximatelyEquals(xMatrix.GetColumn2()) );
}

ASURA_UNITTEST(Scaling, Asura_AMatrix3x3)
{
#if 0
    /* Original */
    Asura_AMatrix3x3 xMatrix = Asura_AMatrix3x3(1.0f, 2.0f, 3.0f, 4.0f, 5.0f, 6.0f, 7.0f, 8.0f, 9.0f) * 2.0f;
    ASURA_UNITTEST_CHECK( Asura_AVector3(2.0f, 8.0f, 14.0f).ApproximatelyEquals(xMatrix.GetColumn0()) );
    ASURA_UNITTEST_CHECK( Asura_AVector3(4.0f, 10.0f, 16.0f).ApproximatelyEquals(xMatrix.GetColumn1()) );
    ASURA_UNITTEST_CHECK( Asura_AVector3(6.0f, 12.0f, 18.0f).ApproximatelyEquals(xMatrix.GetColumn2()) );
#elif 0
    /* Vector temporaries */
    Asura_AMatrix3x3 xMatrix(1.0f, 2.0f, 3.0f, 4.0f, 5.0f, 6.0f, 7.0f, 8.0f, 9.0f);
    xMatrix = xMatrix * 2.0f;
    ASURA_UNITTEST_CHECK( Asura_AVector3(2.0f, 8.0f, 14.0f).ApproximatelyEquals(xMatrix.GetColumn0()) );
    ASURA_UNITTEST_CHECK( Asura_AVector3(4.0f, 10.0f, 16.0f).ApproximatelyEquals(xMatrix.GetColumn1()) );
    ASURA_UNITTEST_CHECK( Asura_AVector3(6.0f, 12.0f, 18.0f).ApproximatelyEquals(xMatrix.GetColumn2()) );
#elif 0
    /* Matrix temporary */
    Asura_AMatrix3x3 xMatrix = Asura_AMatrix3x3(1.0f, 2.0f, 3.0f, 4.0f, 5.0f, 6.0f, 7.0f, 8.0f, 9.0f) * 2.0f;
    const Asura_AVector3 xVec1(2.0f, 8.0f, 14.0f), xVec2(4.0f, 10.0f, 16.0f), xVec3(6.0f, 12.0f, 18.0f);
    ASURA_UNITTEST_CHECK( xVec1.ApproximatelyEquals(xMatrix.GetColumn0()) );
    ASURA_UNITTEST_CHECK( xVec2.ApproximatelyEquals(xMatrix.GetColumn1()) );
    ASURA_UNITTEST_CHECK( xVec3.ApproximatelyEquals(xMatrix.GetColumn2()) );
#else
    /* No temporaries */
    Asura_AMatrix3x3 xMatrix(1.0f, 2.0f, 3.0f, 4.0f, 5.0f, 6.0f, 7.0f, 8.0f, 9.0f);
    const Asura_AVector3 xVec1(2.0f, 8.0f, 14.0f), xVec2(4.0f, 10.0f, 16.0f), xVec3(6.0f, 12.0f, 18.0f);
    xMatrix = xMatrix * 2.0f;
    ASURA_UNITTEST_CHECK( xVec1.ApproximatelyEquals(xMatrix.GetColumn0()) );
    ASURA_UNITTEST_CHECK( xVec2.ApproximatelyEquals(xMatrix.GetColumn1()) );
    ASURA_UNITTEST_CHECK( xVec3.ApproximatelyEquals(xMatrix.GetColumn2()) );
#endif
}

ASURA_UNITTEST(Zero, Asura_AMatrix3x3)
{
	Asura_AMatrix3x3 xMatrix(xMatrix1to9);
	xMatrix.Zero();
	for (u_int u = 0; u < 9; ++u)
	{
		ASURA_UNITTEST_CHECK_EQUAL( 0.0f, xMatrix.GetElement(u/3,u%3) );
	}
}

ASURA_UNITTEST(Equality, Asura_AMatrix3x3)
{
	Asura_AMatrix3x3 xMatrix0 = Asura_AMatrix3x3(1.0f, 2.0f, 3.0f, 4.0f, 5.0f, 6.0f, 7.0f, 8.0f, 9.0f);
	Asura_AMatrix3x3 xMatrix1 = Asura_AMatrix3x3(0.0f, 2.0f, 3.0f, 4.0f, 5.0f, 6.0f, 7.0f, 8.0f, 9.0f);
	Asura_AMatrix3x3 xMatrix2 = Asura_AMatrix3x3(1.0f, 0.0f, 3.0f, 4.0f, 5.0f, 6.0f, 7.0f, 8.0f, 9.0f);
	Asura_AMatrix3x3 xMatrix3 = Asura_AMatrix3x3(1.0f, 2.0f, 0.0f, 4.0f, 5.0f, 6.0f, 7.0f, 8.0f, 9.0f);
	Asura_AMatrix3x3 xMatrix4 = Asura_AMatrix3x3(1.0f, 2.0f, 3.0f, 0.0f, 5.0f, 6.0f, 7.0f, 8.0f, 9.0f);
	Asura_AMatrix3x3 xMatrix5 = Asura_AMatrix3x3(1.0f, 2.0f, 3.0f, 4.0f, 0.0f, 6.0f, 7.0f, 8.0f, 9.0f);
	Asura_AMatrix3x3 xMatrix6 = Asura_AMatrix3x3(1.0f, 2.0f, 3.0f, 4.0f, 5.0f, 0.0f, 7.0f, 8.0f, 9.0f);
	Asura_AMatrix3x3 xMatrix7 = Asura_AMatrix3x3(1.0f, 2.0f, 3.0f, 4.0f, 5.0f, 6.0f, 0.0f, 8.0f, 9.0f);
	Asura_AMatrix3x3 xMatrix8 = Asura_AMatrix3x3(1.0f, 2.0f, 3.0f, 4.0f, 5.0f, 6.0f, 7.0f, 0.0f, 9.0f);
	Asura_AMatrix3x3 xMatrix9 = Asura_AMatrix3x3(1.0f, 2.0f, 3.0f, 4.0f, 5.0f, 6.0f, 7.0f, 8.0f, 0.0f);
	Asura_AMatrix3x3 xMatrix10 = Asura_AMatrix3x3(1.0f, 2.0f, 3.0f, 4.0f, 5.0f, 6.0f, 7.0f, 8.0f, 9.0f);
	ASURA_UNITTEST_CHECK( xMatrix0 == xMatrix0 );
	ASURA_UNITTEST_CHECK( xMatrix0 == xMatrix10 );
	ASURA_UNITTEST_CHECK( !(xMatrix0 == xMatrix1) );
	ASURA_UNITTEST_CHECK( !(xMatrix0 == xMatrix2) );
	ASURA_UNITTEST_CHECK( !(xMatrix0 == xMatrix3) );
	ASURA_UNITTEST_CHECK( !(xMatrix0 == xMatrix4) );
	ASURA_UNITTEST_CHECK( !(xMatrix0 == xMatrix5) );
	ASURA_UNITTEST_CHECK( !(xMatrix0 == xMatrix6) );
	ASURA_UNITTEST_CHECK( !(xMatrix0 == xMatrix7) );
	ASURA_UNITTEST_CHECK( !(xMatrix0 == xMatrix8) );
	ASURA_UNITTEST_CHECK( !(xMatrix0 == xMatrix9) );
}

ASURA_UNITTEST(Inquality, Asura_AMatrix3x3)
{
	Asura_AMatrix3x3 xMatrix0 = Asura_AMatrix3x3(1.0f, 2.0f, 3.0f, 4.0f, 5.0f, 6.0f, 7.0f, 8.0f, 9.0f);
	Asura_AMatrix3x3 xMatrix1 = Asura_AMatrix3x3(0.0f, 2.0f, 3.0f, 4.0f, 5.0f, 6.0f, 7.0f, 8.0f, 9.0f);
	Asura_AMatrix3x3 xMatrix2 = Asura_AMatrix3x3(1.0f, 0.0f, 3.0f, 4.0f, 5.0f, 6.0f, 7.0f, 8.0f, 9.0f);
	Asura_AMatrix3x3 xMatrix3 = Asura_AMatrix3x3(1.0f, 2.0f, 0.0f, 4.0f, 5.0f, 6.0f, 7.0f, 8.0f, 9.0f);
	Asura_AMatrix3x3 xMatrix4 = Asura_AMatrix3x3(1.0f, 2.0f, 3.0f, 0.0f, 5.0f, 6.0f, 7.0f, 8.0f, 9.0f);
	Asura_AMatrix3x3 xMatrix5 = Asura_AMatrix3x3(1.0f, 2.0f, 3.0f, 4.0f, 0.0f, 6.0f, 7.0f, 8.0f, 9.0f);
	Asura_AMatrix3x3 xMatrix6 = Asura_AMatrix3x3(1.0f, 2.0f, 3.0f, 4.0f, 5.0f, 0.0f, 7.0f, 8.0f, 9.0f);
	Asura_AMatrix3x3 xMatrix7 = Asura_AMatrix3x3(1.0f, 2.0f, 3.0f, 4.0f, 5.0f, 6.0f, 0.0f, 8.0f, 9.0f);
	Asura_AMatrix3x3 xMatrix8 = Asura_AMatrix3x3(1.0f, 2.0f, 3.0f, 4.0f, 5.0f, 6.0f, 7.0f, 0.0f, 9.0f);
	Asura_AMatrix3x3 xMatrix9 = Asura_AMatrix3x3(1.0f, 2.0f, 3.0f, 4.0f, 5.0f, 6.0f, 7.0f, 8.0f, 0.0f);
	Asura_AMatrix3x3 xMatrix10 = Asura_AMatrix3x3(1.0f, 2.0f, 3.0f, 4.0f, 5.0f, 6.0f, 7.0f, 8.0f, 9.0f);
	ASURA_UNITTEST_CHECK( !(xMatrix0 != xMatrix0) );
	ASURA_UNITTEST_CHECK( !(xMatrix0 != xMatrix10) );
	ASURA_UNITTEST_CHECK( (xMatrix0 != xMatrix1) );
	ASURA_UNITTEST_CHECK( (xMatrix0 != xMatrix2) );
	ASURA_UNITTEST_CHECK( (xMatrix0 != xMatrix3) );
	ASURA_UNITTEST_CHECK( (xMatrix0 != xMatrix4) );
	ASURA_UNITTEST_CHECK( (xMatrix0 != xMatrix5) );
	ASURA_UNITTEST_CHECK( (xMatrix0 != xMatrix6) );
	ASURA_UNITTEST_CHECK( (xMatrix0 != xMatrix7) );
	ASURA_UNITTEST_CHECK( (xMatrix0 != xMatrix8) );
	ASURA_UNITTEST_CHECK( (xMatrix0 != xMatrix9) );
}


// Q U A T   W I T H   A L I G N E D   V E C T O R ///////////////////////////
ASURA_UNITTEST(RotateVectorAligned, Asura_Quat)
{
	Asura_AVector3 xTemp = Asura_Maths::IdentityQuat.RotateVector(xVec123);
	ASURA_UNITTEST_CHECK( xVec123 == xTemp );

	xTemp = Asura_Maths::IdentityQuat.RotateVector(xVecMinus123);
	ASURA_UNITTEST_CHECK( xVecMinus123 == xTemp );

	Asura_Matrix_3x3 xMatrix;
	Asura_Quat xQuat; // kjs note : it's just rotating an axis about itself. (around z-roll)
	xQuat.FromEuler( Asura_Euler_3(0.0f, Asura_Maths::Deg90, 0.0f) );
	xTemp = xQuat.RotateVector( xVec010 );
	ASURA_UNITTEST_CHECK( xVec010.ApproximatelyEquals(xTemp) );
	xTemp = xQuat.RotateVector( xVec001 );
	ASURA_UNITTEST_CHECK( xVec100.ApproximatelyEquals(xTemp) );

	xQuat.FromEuler( Asura_Euler_3(-Asura_Maths::Deg90, 0.0f, 0.0f) ); // (around x-pitch)
	xTemp = xQuat.RotateVector( xVec100 );
	ASURA_UNITTEST_CHECK( xVec100.ApproximatelyEquals(xTemp) );
	xTemp = xQuat.RotateVector( xVec001 );
	ASURA_UNITTEST_CHECK( xVec010.ApproximatelyEquals(xTemp) );
	Asura_Euler_3(-Asura_Maths::Deg90, 0.0f, 0.0f).ToMatrix(xMatrix);
	xTemp = xVec001*xMatrix;
	ASURA_UNITTEST_CHECK( xVec010.ApproximatelyEquals(xTemp) );

	xQuat.FromEuler( Asura_Euler_3( 0.0f, 0.0f, Asura_Maths::Deg90) ); // (around y-yaw)
	xTemp = xQuat.RotateVector( xVec001 );
	ASURA_UNITTEST_CHECK( xVec001.ApproximatelyEquals(xTemp) );
	xTemp = xQuat.RotateVector( xVec100 );
	ASURA_UNITTEST_CHECK( xVec010.ApproximatelyEquals(xTemp) );
}

ASURA_UNITTEST(FromRotationAligned, Asura_Quat)
{
	// x,y,z are from axis * sin(angle/2), w = cos(angle/2)
	Asura_Quat xQuat;
	Asura_AVector3 xTempVec1(xVec111);
	Asura_AVector3 xTempVec2(xVec123);
	xTempVec1.Normalise();
	xTempVec2.Normalise();
	xQuat.FromRotation(0.0f, xTempVec1);
	ASURA_UNITTEST_CHECK( xQuat.ApproximatelyEquals(Asura_Maths::IdentityQuat) ); //BW) == Asura_Maths::IdentityQuat );
	xQuat.FromRotation(Asura_Maths::Deg90, xTempVec2);
	float fSin45 = Asura_Maths::Sqrt(0.5f);
	float fCos45 = fSin45;
	float fInvMag123 = 1.0f / Asura_Maths::Sqrt(14.0f);
	Asura_Quat xQuat2(1.0f * fSin45 * fInvMag123, 2.0f * fSin45 * fInvMag123, 3.0f * fSin45 * fInvMag123, fCos45);
	ASURA_UNITTEST_CHECK( xQuat.ApproximatelyEquals(xQuat2) );
}

#endif

// E N D   O F   F I L E /////////////////////////////////////////////////////
