//////////////////////////////////////////////////////////////////////////////
//
// Asura_UnitTest_Multithread.cpp
//
// Copyright (c) 2008 Rebellion. All rights reserved.
//
// Description:
//
//     tests for multithreading code
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

#ifdef ASURA_SUPPORT_UNITTEST

#include "Core/Asura_Multithread.h"
#include "Debugging/Asura_Logfile.h"
#include "UnitTest/Asura_UnitTest.h"

static const u_int uUNITTEST_ARRAY_SIZE = 10;
static int g_aiQueue[uUNITTEST_ARRAY_SIZE];
static u_int g_uQueuePos;
static u_int g_uQueueSize;
static int g_iSum;
static Asura_Mutex* g_pxUnitTestMutex;
static Asura_Semaphore* g_pxUnitTestSemaphore;
static Asura_Semaphore* g_pxUnitTestSemaphore2;
static Asura_Semaphore* g_pxUnitTestSemaphoreWaitForThreadsToStart;
static volatile bool g_bRunning = true;

static void AddNumbers( const void* )
{
	g_pxUnitTestSemaphoreWaitForThreadsToStart->Signal();

	while (g_bRunning)
	{
		// wait for data
		g_pxUnitTestSemaphore->Wait();
		if (!g_bRunning)
		{
			return;
		}

		// remove item from queue
		g_pxUnitTestMutex->Lock();

		// should have work to do - check the queue size is bigger than last entry read
		if (g_uQueuePos >= g_uQueueSize)
		{
			// abort
//			Asura_OutputToDebugger_Retail(("read beyond queue size\n"));
			// finished work - make all threads stop
			g_bRunning = false;
			// signal main thread to stop waiting
			g_pxUnitTestSemaphore2->Signal();
			g_pxUnitTestMutex->Unlock();
			return;
		}
		
		int iNumberFromQueue = g_aiQueue[g_uQueuePos];
//		Asura_OutputToDebugger_Retail(("thread %u read %d from queue\n", Asura_Thread_System::GetCurrentThreadID(), iNumberFromQueue));
		g_iSum += iNumberFromQueue;
		++g_uQueuePos;

		if (g_uQueuePos == uUNITTEST_ARRAY_SIZE)
		{
			// finished work - make all threads stop
			g_bRunning = false;
			// signal main thread to stop waiting
			g_pxUnitTestSemaphore2->Signal();
		}

		// finished with queue
		if(g_pxUnitTestMutex != NULL)
		{	g_pxUnitTestMutex->Unlock();}
	}
}

ASURA_UNITTEST(Multithread, MutexAndSemaphore)
{
	// initialise
	g_pxUnitTestMutex = new Asura_Mutex;
	g_pxUnitTestSemaphore = new Asura_Semaphore("unittest_semaphore", 0, 256);
	g_pxUnitTestSemaphore2 = new Asura_Semaphore("unittest_semaphore2", 0, 256);
	g_pxUnitTestSemaphoreWaitForThreadsToStart = new Asura_Semaphore("unittest_semaphore3", 0, 256);

	int aiNumbers[uUNITTEST_ARRAY_SIZE] = { 2, 3, 5, 7, 11, 13, 17, 19, 23, 29 };
	int iExpectedSum = 0; 
	for (u_int u = 0; u < uUNITTEST_ARRAY_SIZE; u++)
	{
		iExpectedSum += aiNumbers[u];
	}

	const u_int uThreadsToCreate = 4;
	for (u_int u = 0; u < uThreadsToCreate; ++u)
	{
		Asura_Thread_System::CreateThread("unittest_worker", AddNumbers, 0);
		// without this semaphore to wait for threads to start, only one thread gets used on PS3.
		// It's only here to try and ensure that multiple threads read data off the queue.
		g_pxUnitTestSemaphoreWaitForThreadsToStart->Wait();
	}

	// check semaphore for work finished is not signalled
	ASURA_UNITTEST_CHECK(!g_pxUnitTestSemaphore2->TryWait());

	// start adding to queue
	for (u_int u = 0; u < uUNITTEST_ARRAY_SIZE; u++)
	{
		// lock queue
		g_pxUnitTestMutex->Lock();

		// add data to queue
		g_aiQueue[u] = aiNumbers[u];
		g_uQueueSize++;

//		Asura_OutputToDebugger_Retail(("added %d to queue\n", aiNumbers[u]));

		// work is available
		g_pxUnitTestSemaphore->Signal();

		// finished with queue
		g_pxUnitTestMutex->Unlock();
	}

	// wait for a thread to finish processing the queue
	g_pxUnitTestSemaphore2->Wait();

	// wake up remaining threads to make them exit
	for (u_int u = 0; u < uThreadsToCreate - 1; ++u)
	{
		g_pxUnitTestSemaphore->Signal();
	}

	// clean up
	delete g_pxUnitTestMutex;
	g_pxUnitTestMutex = NULL;
	delete g_pxUnitTestSemaphore;
	g_pxUnitTestSemaphore = NULL;
	delete g_pxUnitTestSemaphore2;
	g_pxUnitTestSemaphore2 = NULL;
	delete g_pxUnitTestSemaphoreWaitForThreadsToStart;
	g_pxUnitTestSemaphoreWaitForThreadsToStart = NULL;

	// check the sum was computed correctly
	ASURA_UNITTEST_CHECK(g_iSum == iExpectedSum);
}

#endif // ASURA_SUPPORT_UNITTEST
