//////////////////////////////////////////////////////////////////////////////
//
// Asura_UnitTest_Physics_Ragdoll.cpp
//
// Copyright (c) 2005 Rebellion. All rights reserved.
//
// Description:
//
//     tests that ragdoll code is behaving as expected
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

#ifdef ASURA_SUPPORT_UNITTEST

#include "UnitTest/Asura_UnitTest.h"

#include "Core/Asura_Maths.h"

#if 0
static void RotationFromAToB( const Asura_AVector3& xA, const Asura_AVector3& xB, Asura_Quat &xResult )
{
	// Find the angle between the two directions, and an axis perpendicular to both directions
	// that's all we need to make our quaternion.

	const float fDot = xA * xB;
	const Asura_AVector3 xAxis = (xA^xB);

	// KJFL - Friday, December 10, 2004 
	//
	// The actual maths leads to the following expression: 
	//
	//  xResult = Asura_Quat(xAxis.m_fX*0.5f, xAxis.m_fY*0.5f, xAxis.m_fZ*0.5f, (fDot + 1.0f) * 0.5f);
	//
	// but the 0.5f constants all disappear when we normalise, so they aren't actually required.

	xResult.x = xAxis.GetX();
	xResult.y = xAxis.GetY();
	xResult.z = xAxis.GetZ();
	xResult.w = (fDot + 1.0f);

	// KJFL - Friday, December 10, 2004 - in theory the vector component of the quat should be
	// a unit vector at this point, so one might be able to speed up the normalise operation
	// if required - but you'd have to take into account cases where the directions are parallel
	// and so xAxis is actually a zero vector.
	
	xResult.Normalise();
}
#endif // #if 0

#if 0
static bool PhysicsRayIntersectsTri(const Asura_AVector3& alpha, const Asura_AVector3& beta, const Asura_AVector3& v0, const Asura_AVector3& v1, const Asura_AVector3& v2, bool& facingToward, float& distance)
{
	const float epsilon = 1.0e-5f;
	Asura_AVector3 e1 = v1-v0;
	Asura_AVector3 e2 = v2-v0;

	Asura_AVector3 p = beta^e2;
	//Asura_Vector_3 n = e1^e2;
	float a = e1 * p;

	if (a>-epsilon && a<epsilon) return false;
	// if a is negative, it's facing away from the ray
	
	a = 1.0f/a; //because a can be negative, must take care of expression such as u > a && u/a > 1

	Asura_AVector3 s = alpha-v0;
	float u = (s*p)*a;

	if (u<0.0f || u>1.0f) return false;

	Asura_AVector3 q = s^e1;
	float v = (beta*q)*a;

	if (v<0.0f || u+v>1.0f) return false;

	float lambda = (e2*q)*a;

	if (a<0.0f) facingToward = false;
	else facingToward = true;

	distance = lambda;
	return true;
}
#else
#define PrintVector(xVec) { printf("%s = %.06f, %.06f, %.06f\n", #xVec, xVec.GetX(), xVec.GetY(), xVec.GetZ()); }
static bool PhysicsRayIntersectsTri(const Asura_AVector3& alpha, const Asura_AVector3& beta, const Asura_AVector3& v0, const Asura_AVector3& v1, const Asura_AVector3& v2, bool& facingToward, float& distance)
{
	const float epsilon = 1.0e-5f;
	Asura_AVector3 e1 = v1-v0;
	Asura_AVector3 e2 = v2-v0;

	Asura_AVector3 p = beta^e2;
	//Asura_Vector_3 n = e1^e2;
	float a = e1 * p;

	if (a>-epsilon && a<epsilon) 
	{
		//printf("a = %f, return false\n", a);
		return false;
	}
	// if a is negative, it's facing away from the ray
	
	a = 1.0f/a; //because a can be negative, must take care of expression such as u > a && u/a > 1

	Asura_AVector3 s = alpha-v0;
	float u = (s*p)*a;

	if (u<0.0f || u>1.0f) 
	{
		//printf("u = %f, return false\n", u);
		return false;
	}

	Asura_AVector3 q = s^e1;
	float v = (beta*q)*a;

	if (v<0.0f || u+v>1.0f) 
	{
		//printf("v = %f, return false\n", v);
		return false;
	}

	float lambda = (e2*q)*a;

	if (a<0.0f) facingToward = false;
	else facingToward = true;

	distance = lambda;

//	PrintVector(e1);
//	PrintVector(e2);
//	PrintVector(p);
//	PrintVector(s);
//	PrintVector(q);
//	printf("a = %f\n", a);
//	printf("u = %f\n", u);
//	printf("v = %f, return true\n", v);
	return true;
}
#endif

ASURA_UNITTEST(RayIntersectsTri, Asura_Physics)
{
	// real world example
	bool bFacingTowards;
	float fDistance;
	Asura_AVector3 xStart(-6.99686f, -1.16979E-01f, -2.97924f);
	Asura_AVector3 xDisp(-2.65562E-02f, 9.62396E-01f, 2.70348E-01f );
	Asura_AVector3 xV0(-2.76f, 0.0f, -2.76f);
	Asura_AVector3 xV1(-8.28f, 0.0f, -2.76f);
	Asura_AVector3 xV2(-8.28f, 0.0f, -8.28f);
	bool bIntersect = PhysicsRayIntersectsTri(xStart, xDisp, xV0, xV1, xV2, bFacingTowards, fDistance);
	ASURA_UNITTEST_CHECK( bIntersect );
	if (bIntersect)
	{
		ASURA_UNITTEST_CHECK( bFacingTowards );
		ASURA_UNITTEST_FLOATS_EQUAL( 1.21549E-01f, fDistance, 0.00001f );
	}
}

#endif // #ifdef ASURA_SUPPORT_UNITTEST
