//////////////////////////////////////////////////////////////////////
// Asura_Volume														//
// SJL 6/11/07														//
// Copyright (C) 2007 Rebellion, All Rights Reserved.			    //
//																	//
// A pure virtual volume class								    	//
//////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef ASURA_VOLUME_H
#define ASURA_VOLUME_H

#include "Core/Asura_Maths.h"

// Forward class declaration
class Asura_Chunk_Stream;
class Asura_DebugLineRender_LineSet;

// Enumeration for volume classification
enum Asura_Volume_Classification
{
	ASURA_VOLUME_CLASSIFICATION_NULL,
	ASURA_VOLUME_CLASSIFICATION_CUBOID,
	ASURA_VOLUME_CLASSIFICATION_SPHERE,
	ASURA_VOLUME_CLASSIFICATION_CYLINDER,
	ASURA_VOLUME_CLASSIFICATION_NUMBEROFCLASSES,
};


class Asura_Volume
{
	public:

		// Constructor/Destructor
		Asura_Volume();
		virtual ~Asura_Volume() = 0;

		// Simple accessors
		int GetClassification() const { return m_iClassification; }

		// Virtual functions
		virtual bool Contains (const Asura_Vector_3& xPosition) const = 0;
		virtual bool Overlaps (const Asura_Bounding_Box& xBB) const {return false;}
		virtual Asura_Quat* GetVolumeOrient() = 0;
		virtual void SetPosition(const Asura_Vector_3 &xPos) = 0;
		virtual void Displace(const Asura_Vector_3& xPos) = 0;
		virtual void RotateAbout(const Asura_Vector_3& xCentre, const Asura_Matrix_3x3& xOri) = 0;
		virtual Asura_Volume* Duplicate() const = 0;
#if !defined(ASURA_NO_DEBUG_OUTPUT)
		virtual void AddToLineSet(const u_int uARGB, Asura_DebugLineRender_LineSet& xLineSet) const = 0;
#endif	//!defined(ASURA_NO_DEBUG_OUTPUT)

		// GetAABB should set xBB to an axis aligned bounding box which entirely encapsulates the volume.
		virtual void GetAABB(Asura_Bounding_Box& xBB) const = 0;

		#ifdef ASURA_TOOLS
		//Moves/rotates volumes that are part of an entity template
		virtual void ApplyChangesForTemplateInstance(class Tools_EntityTemplateInstance* pxInstance) = 0;
		#endif

		// load/save stream functions.
		virtual	void ReadFromChunkStream(Asura_Chunk_Stream& xStream) = 0;
		virtual	void WriteToChunkStream(Asura_Chunk_Stream& xStream) const = 0;

	protected:

		// Data.
		int m_iClassification;

	private:
};

#endif // ASURA_VOLUME_H

