//////////////////////////////////////////////////////////////////////
// Asura_Volume_Cuboid												//
// SJL 6/11/07														//
// Copyright (C) 2007 Rebellion, All Rights Reserved.			    //
//																	//
// An oriented cuboid volume class								    //
//////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef ASURA_VOLUME_CUBOID_H
#define ASURA_VOLUME_CUBOID_H

// Base header.
#include "Volume/Asura_Volume.h"

// Forward class declaration
class Asura_Chunk_Stream;

class Asura_Volume_Cuboid : public Asura_Volume
{
	public:

		// Constructor / Destructor
		Asura_Volume_Cuboid();
		Asura_Volume_Cuboid(const Asura_Bounding_Box& xBox, const Asura_Quat& xOrient);
		virtual ~Asura_Volume_Cuboid();

		// Public interface
		virtual bool Contains (const Asura_Vector_3& xPosition) const;
		virtual bool Overlaps (const Asura_Bounding_Box& xBB) const;

		// Pointer member access to facilitate generic tools use mainly
		virtual Asura_Quat* GetVolumeOrient() { return &m_xOrient; }
		virtual Asura_Bounding_Box* GetVolumeBox() { return &m_xBox; }

		// Various copy and move functions.
		virtual void SetBoundingBox(const Asura_Bounding_Box& xBox){m_xBox = xBox;}
		virtual void SetPosition(const Asura_Vector_3& xPos) { m_xBox.Displace(xPos - m_xBox.CalculateCentre()); }
		virtual void Displace(const Asura_Vector_3& xOffset) { m_xBox.Displace(xOffset); }
		virtual void RotateAbout(const Asura_Vector_3& xCentre, const Asura_Matrix_3x3& xOri);

		virtual Asura_Volume* Duplicate() const;
		void Copy(const Asura_Volume_Cuboid* pxFrom);

		virtual void GetAABB(Asura_Bounding_Box& xBB) const;

#if !defined(ASURA_NO_DEBUG_OUTPUT)
		virtual void AddToLineSet(const u_int uARGB, Asura_DebugLineRender_LineSet& xLineSet) const;
#endif	//!defined(ASURA_NO_DEBUG_OUTPUT)

		#ifdef ASURA_TOOLS
		//Moves/rotates volumes that are part of an entity template
		virtual void ApplyChangesForTemplateInstance(class Tools_EntityTemplateInstance* pxInstance);
		#endif


		// load/save stream functions.
		virtual	void ReadFromChunkStream(Asura_Chunk_Stream& xStream);
		virtual	void WriteToChunkStream(Asura_Chunk_Stream& xStream) const;

		// Cuboid only functions.
		Asura_Vector_3 CalculateCentre() const {return m_xBox.CalculateCentre();}
		// These two functions don't seem essential so I wont enforce their existence as a pure virtual functions in the base class		
		// until such time as they are needed. 
		float GetSqdMinDistFromPoint(const Asura_Vector_3& xPosition) const;
		void CalcMinVectorIntoVolume(const Asura_Vector_3& xPosition, Asura_Vector_3& xVector ) const;
        
	protected:
	
		// A cuboid is defined by an Asura_Bounding_Box with an orientation
		// It has no true 'position' only an implied one from the center of the BB

		// Member variables
		Asura_Quat m_xOrient;
		Asura_Bounding_Box m_xBox;
	
	private:

		// load/save version number
		static const u_int s_uCHUNKVERSION;
};

#endif // ASURA_VOLUME_CUBOID_H

