//
// Asura_Zone.h
//
// Created by Richard Rice - 25/01/05
// 
// Copyright (C) 2005 Rebellion, All Rights Reserved.
//

#ifndef __ASURA_ZONE_H__
#define __ASURA_ZONE_H__

//////////////////////////////////////////////////////////////////////////////
//
// includes
//
//////////////////////////////////////////////////////////////////////////////

#include "Core/Asura_String.h"

//////////////////////////////////////////////////////////////////////////////
//
// forward declarations
//
//////////////////////////////////////////////////////////////////////////////

class Asura_OrientedVolume;

//////////////////////////////////////////////////////////////////////////////
//
// Asura_Zone
//
//////////////////////////////////////////////////////////////////////////////

class Asura_Zone
{
	friend class Tools_Zone_Editor;
	friend class Tools_Zone_VolumeEdit_Controller;

public :
	Asura_Zone( const bool bCreateFromLoad );
	virtual ~Asura_Zone();

	//does this zone contain this point
	bool Contains(const class Asura_Vector_3& xPos) const;
	//does the iIndex'th volume contain this point
	bool Contains(const Asura_Vector_3& xPos, int iVolumeIndex) const;

	//If a zone is disabled then Contains() will return false for it
	virtual bool IsEnabled() const {return true;}

	//GetPriority() is used for determining which zone to use when a position
	//is in more than one zone. The zone with the higher value will be selected.
	//This is only used if UsePriorityForOverlaps() returns true in Asura_Zone_Controller
	virtual float GetPriority(int iVolumeIndex) const {return 0.0f;}

	//zone is flagged as the default zone to use when in no other zone
	bool  IsDefaultZone() const { return m_bDefaultZone; }
    u_int GetZoneID()     const { return m_uZoneID; }

	//loading/saving (use of chunkbuffer or chunkstream versions will be dependent on derived class)
	//chunkbuffer functions
	virtual void InitialiseFromChunkBuffer(const char*& pcChunkBuffer);

#ifdef ASURA_TOOLS
	virtual void FillInChunkBuffer(char*& pcChunkBuffer);
	virtual int GetChunkBufferSize();
#endif 

	//chunkstream functions
	virtual void ReadFromChunkStream(class Asura_Chunk_Stream& xStream);
#ifdef ASURA_TOOLS
	virtual void WriteToChunkStream(class Asura_Chunk_Stream& xStream);
#endif

	int GetNumberOfVolumes() const {return m_iNumberOfVolumes;}

#ifdef ASURA_TOOLS
	//gets an axially aligned bounding box that contains the iIndex'th volume
	void GetAABBForVolume(int iIndex, Asura_Bounding_Box& xOutputBox) const;
#endif
	


#ifdef ASURA_TOOLS
	const char* GetName() const { return m_xName.GetString();}
#endif

protected :
	//Gets the size of the iVolumeIndex'th volume for use in priority stuff
	float GetVolumeSize(int iVolumeIndex) const;

public:
    enum { ASURA_INVALID_ZONE_ID = 0 };

private :
    u_int                 m_uZoneID;
	int                   m_iNumberOfVolumes;
	Asura_OrientedVolume* m_pxVolumes;
	union
	{
		u_int m_uFlags;
		struct
		{
			u_int m_bDefaultZone : 1;
		};
	};
	mutable int m_iPreviousVolumeIndex;

#ifdef ASURA_TOOLS
	Asura_String m_xName;
#endif

private:
    struct ChunkData
    {
        enum { CURRENT_VERSION = 1 };

        u_int m_uZoneID;
        int   m_iNumberOfVolumes;
        u_int m_uFlags;

        // then follows a list of volumes m_iNumberOfVolumes long and then the name of the zone
    };

private:
    static u_int s_uNextZoneID;
};

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

#endif //__ASURA_ZONE_H__
