//////////////////////////////////////////////////////////////////////////////
//
// Asura_PC-CLI_Main.cpp
//
// Description: provides PC-CLI WinMain
//     
// Copyright (c) 2009 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

// Asura
#include "Debugging/Asura_Logfile.h"

// Asura PC
#include "Core/Asura_Windows.h"
#include "Main/Asura_PC-CLI_Manager.h"

// Any game
#include "Main/Asura_Project_Main.h"

#if defined( ASURA_ENABLE_SEH_HANDLING)
#include "Core/Asura_PC_SEH_Handler.h"
#endif

#ifdef ASURA_ENABLE_FLOATING_POINT_EXCEPTIONS
#include <float.h>
#include <fpieee.h>
#include <excpt.h>
#endif//def ASURA_ENABLE_FLOATING_POINT_EXCEPTIONS

// C O N S T A N T S /////////////////////////////////////////////////////////

static const char* szCRASH_TXT_LOGFILE = "logfiles\\crash.txt";
static char szCRASH_XML_LOGFILE[ MAX_PATH] = "logfiles\\crash.xml";

int APIENTRY Asura_WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
#ifdef ASURA_ASSERT
	if (!IsDebuggerPresent())
	{
		AsuraAssert_SetIgnoreAll(true);
	}
#endif

	Asura_PCCLI_Manager::InitLogFiles(lpCmdLine);

	Asura_Windows::Initialise(hInstance,hPrevInstance,lpCmdLine,nCmdShow,NULL,false);

	Asura_PCCLI_Manager::InitCommandLineArgs(lpCmdLine);

	if(!Asura_PCCLI_Manager::Initialise(lpCmdLine))
	{
		return -1;
	}

	while(Asura_Project::MainLoop())
	{
		Asura_Windows::CheckForWindowsMessages();

		Asura_PCCLI_Manager::UpdateFrontEnd();
		Asura_PCCLI_Manager::Update();
	}

	Asura_PCCLI_Manager::Shutdown();

	return 0;
}

int APIENTRY WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
	int iError = 0;

#if( ASURA_PC_USE_SINGLEINSTANCEMUTEX == 1 )
	// Prevent more than one copy of the game running at once
	CreateMutex( NULL, FALSE, ASURA_PC_APPLICATION_NAME "SingleInstanceMutex" );
	if( GetLastError() != ERROR_SUCCESS )
	{
		// If the mutex has previously been created, GetLastError() returns ERROR_ALREADY_EXISTS
		return 0;
	}
#endif // ( ASURA_PC_USE_SINGLEINSTANCEMUTEX == 1 )

#if defined( ASURA_ENABLE_SEH_HANDLING)
	__try 
	{
		//Asura_PC_EnableFPExceptions();
#endif

		iError = Asura_WinMain( hInstance, hPrevInstance, lpCmdLine, nCmdShow);

#if defined( ASURA_ENABLE_SEH_HANDLING)
	} 
	__except ( Asura_PC_SEH_Handler::ProcessException( GetExceptionInformation()))
	{
		// code that gets control if fpieee_handler returns
		// EXCEPTION_EXECUTE_HANDLER goes here
#if defined ASURA_LOGFILES_ENABLED || defined ASURA_ASSERT
#if defined( ASURA_TOOLS)
		if( true == g_bXMLLogFiles)
		{
			const char* szCrashReportXML = Asura_PC_SEH_Handler::GetReportXML();
			Asura_SimpleLogfileOutputXML(( szCRASH_XML_LOGFILE, szCrashReportXML));
		}
#endif
		const char* szCrashReportTXT = Asura_PC_SEH_Handler::GetReportTXT();
#endif
		Asura_SimpleLogfileOutput(( szCRASH_TXT_LOGFILE, szCrashReportTXT));
#if defined( ASURA_TOOLS)
		if( false == Tools_BatchProcessing::IsBatch())
#endif
		{
			Asura_Assertf(false, ("unhandled exception\n\n%s\n\n", szCrashReportTXT));
		}

		iError = 1;
	}
#endif

	Asura_Assertf( 0 == iError, ("Process finishing with Error Value %d", iError));
	return iError;
}
