//////////////////////////////////////////////////////////////////////////////
//
// Asura_Platform_Render.cpp
//
// Copyright (c) 2003 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

// Engine independent.
#include "Core/Asura.h"
#include "Core/Asura_String.h"
#include "Debugging/Asura_DebugText.h"
#include "GUIMenu/Asura_GUIMenu_System.h"
#include "Rendering/Asura_Font_System.h"
#include "Rendering/Asura_Render.h"
#include "Entity/Glass/Asura_ClientEntity_Glass.h"
#include "SpecialFX/Asura_Shard_Effect.h"
#include "SpecialFX/Asura_Shard_System.h"
// C L A S S   I M P L E M E N T A T I O N S /////////////////////////////////

//
// Asura_Font_System.
//

float Asura_Font_System::GetPlatformYScaleFactor(void)
{
	return 1.0f;
}

// enable beams - otherwise they won't update
bool Asura_Render::s_abSystemEnabled[ASURA_RENDER_NUM_SYSTEMS] = 
{
	false, // ASURA_RENDER_SYSTEM_SKYBOX,
	false, // ASURA_RENDER_SYSTEM_ENV,
	false, // ASURA_RENDER_SYSTEM_OBJECTS,
	false, // ASURA_RENDER_SYSTEM_BOTANICALS,
	false, // ASURA_RENDER_SYSTEM_CARPET,
	false, // ASURA_RENDER_SYSTEM_WATER,
	false, // ASURA_RENDER_SYSTEM_SHADOWS,
	true, // ASURA_RENDER_SYSTEM_BEAMS,
	false, // ASURA_RENDER_SYSTEM_DECALS,
	false, // ASURA_RENDER_SYSTEM_PHOTONS,
	false, // ASURA_RENDER_SYSTEM_PROJECTORS,
	false, // ASURA_RENDER_SYSTEM_PFX,	
	false, // ASURA_RENDER_SYSTEM_CORONAS,	
	false, // ASURA_RENDER_SYSTEM_FOG,
	false, // ASURA_RENDER_SYSTEM_FSFX,	
	false, // ASURA_RENDER_SYSTEM_HUD,	
};

u_int Asura_Render::s_uNumSkinHierarchiesOutOfView = 0;
unsigned int Asura_Render::s_uNumSmoothSkinsRendered = 0;
unsigned int Asura_Render::s_uNumHierarchiesRendered = 0;
void Asura_Render::Platform_RenderDynamicShadowsOntoEnvironment()
{
}
Asura_Vector_2 Asura_Render::s_xSubPixelOffset = Asura_Vector_2(0.0f,0.0f);
int Asura_Render::s_aiNumberOfTrianglesRendered[] = {0,0,0,0,0,0,0,0,0,0,0};

//void Asura_Render::DEBUG_RenderAxis(class Asura_Vector_3 const &,class Asura_Matrix_3x3 const &,float) {}
//void Asura_Render::DEBUG_RenderSphere( const Asura_Vector_3& xPosition, const float fRadius, const u_int uARGB ) {}
void Asura_Render::EarlyShutdown (void) {};
void Asura_Render::LateShutdown (void) {};

bool Asura_Render::s_bCutsceneHasCameraControl = false;
bool Asura_Render::s_bPatchesOn = false;
int Asura_Render::s_iPatchesSubdivLevel = 3;
bool Asura_Render::s_bStaticSubdivision = true;
enum ASURA_RENDERSTYLE_ID Asura_Render::s_eRenderStyle = ASURA_RENDERSTYLE_NORMAL;
bool Asura_Render::s_bRenderMirrored = false;
bool Asura_Render::s_bRenderForWidescreen = false;
int Asura_Render::s_iNumTriPassesRendered = 0;
int Asura_Render::s_iNumberOfPatchesRendered = 0;
bool Asura_Render::s_bCurrentlyRendering = false;
Asura_Overlay_Quad Asura_Render::Overlay;
Asura_Render_States Asura_Render::CurrentStates;
bool  Asura_Render::s_bEnableBatchedLineRendering = true;

bool  Asura_Render::s_bWireframe = false;


void Asura_Render::RenderLine(class Asura_Vector_3 const &,class Asura_Vector_3 const &,unsigned int) { }
void Asura_Render::Render2DLine(float,float,float,float,unsigned int) {}
void Asura_Render::Render2DLine(class Asura_Vector_2 const &,class Asura_Vector_2 const &,unsigned int) {}
void Asura_Render::RenderBB(class Asura_Bounding_Box const &,class Asura_Vector_3 const &,class Asura_Matrix_3x3 const &,unsigned int) {}
void Render_Sphere(class Asura_Vector_3 const &,float,int) {}

void Asura_Overlay_Quad::Platform_Render(void) {}


//
// Asura_Render.
//

void Asura_Render::Platform_EarlyShutdown( void ) {}
void Asura_Render::Platform_LateShutdown( void ) {}
void Asura_Render::Platform_AllowFullScreenRendering( void ) { }
void Asura_Render::Platform_ApplyCameraSettings( void ) { }
void Asura_Render::BeginFrame( void ) { }
void Asura_Render::EndFrame( void ) 
{
#ifdef ASURA_USE_DEBUG_TEXT
	Asura_DebugText::RenderAll();
#endif
}
void Asura_Render::Platform_FlushDisplay( void ) { }
void Asura_Render::Initialise( void ) 
{
	Asura_CommandConsole::AddVar(L"Render.Enable.HUD", &s_abSystemEnabled[16], Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
}
void Asura_Render::Platform_ClearZBuffer( void ) { }
void Asura_Render::Reset( void ) { }
void Asura_Render::RenderLine(const Asura_AVector3 &xPoint1,const Asura_AVector3 &xPoint2, unsigned int uColour ){}
//void Asura_Render::Platform_ClearZBuffer(void) {}
bool Asura_Render::Platform_SetVideoMode( int, int, bool ) { return true; }
//void Asura_Render::RenderDynamicShadowsOntoEnvironment( void ) { }

#ifdef ASURA_TARGET_SM3
void Asura_Render::Platform_BeginSkyboxRendering(void) {}
void Asura_Render::Platform_EndSkyboxRendering(void) {}
void Asura_Render::Platform_EndMRT() {}
#endif

#ifdef ASURA_USE_HDR
bool Asura_Render::m_bIsCurrentlyHDR = false;
void Asura_Render::Platform_ApplyToneMapping() {}
#endif

#if defined(ASURA_USE_SCREENSHOT_SYSTEM)
bool  Asura_Render::s_bEnableHiResScreenshotMode = false;
bool  Asura_Render::s_bDisableMipMaps = false;
#endif	//defined(ASURA_USE_SCREENSHOT_SYSTEM)

#ifndef ASURA_TARGET_SM3
void Asura_Render::SetReflectanceMapName(const char* pszName) {}
#endif //ndef ASURA_TARGET_SM3
void Asura_Render::RenderAxes( const Asura_Vector_3& xCentre, const Asura_Matrix_3x3& xOrient, const float fSize, const float fAlpha ) {}

// Asura_2d_Graphics
//void Asura_2D_Graphics::Platform_StartBatch(int,enum ASURA_TRANSLUCENCY_MODE,float) {}
//void Asura_2D_Graphics::Platform_AddToBatch(int,int,int,int,int,int,unsigned int) {}
//void Asura_2D_Graphics::Platform_EndBatch(void) {}

void Asura_ClientEntity_Glass::Platform_Render(unsigned int) {}

#ifdef ASURA_TARGET_SM3
void Asura_Render::Platform_BeginMRT(void) {}
#endif //def ASURA_TARGET_SM3

void Asura_Render::DeactivateScreenBorder(void) {}

bool Asura_Render::SetVideoMode( int iWidth, int iHeight, bool bFSAA )
{
	return true;
}
