//*******************************************************************
//
// File: Asura_Direct3D_Botanicals_Game_Render.cpp
//
// Created: 13 Mar 2006
//
// Description: 
//
//*******************************************************************

// system libraries
#include "Core/Asura.h"

#if defined( ASURA_USE_BOTANICALS)

// main header
#include "Botanical/Asura_Direct3D_Botanicals_Game_Render.h"
#include "Botanical/Asura_Direct3D_Botanicals_Game_Render_Manager.h"

// other libraries

// internal dependencies
#include "Rendering/Asura_Module.h"

// Engine tools.
#ifdef ASURA_TOOLS
#include "Botanicals/Tools_Mode_Botanicals.h"
#include "Misc/Tools_Render.h"
#include "Tools_System.h"
#endif

//*******************************************************************
// Constants
//*******************************************************************


//*******************************************************************
// Local Classes
//*******************************************************************

//*******************************************************************
// Local Data
//*******************************************************************

static Asura_Direct3D_BGR_Manager g_xBGRManager;

//*******************************************************************
// Function Definitions
//*******************************************************************

Asura_BGR_Manager& Asura_BGR_Manager::GetBGRManager( void)
{
	return( g_xBGRManager);
}

Asura_Direct3D_BGR_Manager& Asura_Direct3D_Botanicals_Game_Render::GetDirect3DBGRManager()
{
	return( g_xBGRManager);
}

Asura_Mutex& Asura_Direct3D_Botanicals_Game_Render::GetLoadingMutex()
{
	return g_xBGRManager.GetLoadingMutex();
}

Asura_Direct3D_BGR_CacheType::Asura_Direct3D_BGR_CacheType()
{
	this->m_pxTree = NULL;
}


bool Asura_Direct3D_Botanicals_Game_Render::Render()
{
	Asura_Direct3D_BGR_Manager& xBGRManager = GetDirect3DBGRManager();
	
#if defined( ASURA_SUPPORT_COMMAND_CONSOLE)
	if( false == xBGRManager.s_bDebugVarShow)
	{
		return( true);
	}
#endif
	
	SCOPEDPIXEVENT((0xff00ff00, L"Botanical Game Render"));
	
	xBGRManager.ClearInstanceFromTypeCache();
	
#if defined(ASURA_USE_NEW_VISIBILITY_SYSTEM)
	const u_int	uNumInstanceHeaders	= xBGRManager.m_xArray_pxInstanceHeaders.Size();

	// Check whether we can see each instance in-turn...
	for(u_int u = 0; u < uNumInstanceHeaders; u++)
	{
		const Asura_Direct3D_BGR_Instances_Header*	pxInstanceHeader	= xBGRManager.m_xArray_pxInstanceHeaders[u];

        if( !pxInstanceHeader ) { continue; }

#ifdef ASURA_TOOLS
        if( Tools_System::GetCurrentMode() != TOOLS_MODE_BOTANICALS || Tools_Mode_Botanicals::ShouldCheckInstanceVisibility() )
#endif
        {
            if( !pxInstanceHeader->IsVisibleFromCurrentCamera() ) { continue; }
        }

		xBGRManager.AddToCache(pxInstanceHeader);

#ifdef ASURA_TOOLS
        if( Tools_System::GetCurrentMode() == TOOLS_MODE_BOTANICALS && Tools_Mode_Botanicals::ShouldShowBoundingVolumes() )
        {
            const Asura_Bounding_Box& xBB = pxInstanceHeader->GetBoundingBox();

            // Render in yellow if it's contained within the environment
            // and red otherwise.
            u_int uLastKnownVisNode = uASURA_VISIBILITY_INVALID_NODE_INDEX;
            if( Asura_Visibility_System::FindNodeContainingAABB(xBB, uLastKnownVisNode) )
            {
                Tools_Render::RenderCube( xBB, 0xFFFFFF22, true );
            }
            else
            {
                Tools_Render::RenderCube( xBB, 0xFFFF0000, true );
            }
        }
#endif
	}
#else
	const uint32 uModuleSystemCount = Asura_Module_System::GetNumberOfModules();
	const uint32 uModuleBGRCount = xBGRManager.m_xArray_xModule.Size();
	const uint32 uModuleUseCount = Asura_Maths::Min( uModuleBGRCount, uModuleSystemCount);
	
	if( 0 == uModuleUseCount)
	{
		// DLH - 20 April 2006 17:11:06 - No module information so Add every thing!
		xBGRManager.AddToCache( xBGRManager.m_xArray_pxInstanceHeaders);
	}
	else
	{
		for( uint32 uModuleIndex = 0; uModuleIndex < uModuleUseCount; uModuleIndex += 1)
		{
			if( false == Asura_Module_System::IsModuleVisible( uModuleIndex))
			{
				continue;
			}
			
			const Asura_Direct3D_BGR_Module& xModule = xBGRManager.m_xArray_xModule[ uModuleIndex];
			const uint32 uInstanceHeaderCount = xModule.m_uInstanceHeadeCount;
			if( 0 == uInstanceHeaderCount)
			{
				continue;
			}
			
			Asura_Direct3D_BGR_Instances_Header** ppxInstanceHeader = &(xBGRManager.m_xArray_pxInstanceHeaders[ xModule.m_uInstanceHeaderStartIndex]);
			Asura_Direct3D_BGR_Instances_Header** ppxInstanceHeaderEnd = ppxInstanceHeader + uInstanceHeaderCount;
			for( ; ppxInstanceHeader < ppxInstanceHeaderEnd; ppxInstanceHeader += 1)
			{
				xBGRManager.AddToCache( *ppxInstanceHeader);
			}
		}

		// DLH - 11 May 2006 11:38:57 - Draw the trees that do not have module values
		for( uint32 uModuleIndex = uModuleUseCount; uModuleIndex < uModuleBGRCount; uModuleIndex += 1)
		{
			const Asura_Direct3D_BGR_Module& xModule = xBGRManager.m_xArray_xModule[ uModuleIndex];
			const uint32 uInstanceHeaderCount = xModule.m_uInstanceHeadeCount;
			if( 0 == uInstanceHeaderCount)
			{
				continue;
			}
			
			Asura_Direct3D_BGR_Instances_Header** ppxInstanceHeader = &(xBGRManager.m_xArray_pxInstanceHeaders[ xModule.m_uInstanceHeaderStartIndex]);
			Asura_Direct3D_BGR_Instances_Header** ppxInstanceHeaderEnd = ppxInstanceHeader + uInstanceHeaderCount;
			for( ; ppxInstanceHeader < ppxInstanceHeaderEnd; ppxInstanceHeader += 1)
			{
				xBGRManager.AddToCache( *ppxInstanceHeader);
			}
		}
	}
#endif	//defined(ASURA_USE_NEW_VISIBILITY_SYSTEM)
	
	xBGRManager.DrawCache();
	
	return( true);
}


bool Asura_Direct3D_Botanicals_Game_Render::HasDataSet()
{
	Asura_Direct3D_BGR_Manager& xBGRManager = GetDirect3DBGRManager();
	
	const uint32 uInstanceHeaderCount = xBGRManager.m_xArray_pxInstanceHeaders.Size();
	const bool bHasData = (0 < uInstanceHeaderCount);
	
	return( bHasData);
}

//*******************************************************************
//
// (c) 2006 Rebellion (Derby)
//
// Warning: This computer program is protected by copyright law and 
// international treaties. Unauthorized reproduction or distribution
// of this program, or any portion of it, may result in severe civil and
// criminal penalties, and will be prosecuted to the maximum extent 
// possible under the law.
//
//*******************************************************************

#endif //defined( ASURA_USE_BOTANICALS)
