//*******************************************************************
//
// File: Asura_Direct3D_Botanicals_Game_Render_Cache.h
//
// Created: 16 Mar 2006
//
// Description: 
//
//*******************************************************************

#ifndef _ASURA_DIRECT3D_BOTANICALS_GAME_RENDER_CACHE_H_
#define _ASURA_DIRECT3D_BOTANICALS_GAME_RENDER_CACHE_H_

#if defined( ASURA_USE_BOTANICALS)

// other libraries
#include "Rendering/Asura_DIPCache.h"

// internal dependencies

// forward declarations

class Asura_Direct3D_BGR_Type;
class Asura_Direct3D_BGR_Instances_Header;
struct Asura_Direct3D_BGR_Type_VertexInstanceSM30;

//*******************************************************************
// Constants
//*******************************************************************

#define ASURA_DIRECT3D_BGR_VS_CONSTANT_GLOBAL_SIZE 12

#define ASURA_DIRECT3D_BGR_VS_CONSTANT_GLOBAL (0)
#define ASURA_DIRECT3D_BGR_VS_CONSTANT_DYNAMICLIGHT (12)
#define ASURA_DIRECT3D_BGR_VS_CONSTANT_VERTEXSCALE (16)
#define ASURA_DIRECT3D_BGR_VS_CONSTANT_INSTANCE0 (20)
//#define ASURA_DIRECT3D_BGR_VS_CONSTANT_INSTANCE0 30

#define ASURA_DIRECT3D_BGR_VS_CONSTANT_INSTANCE_SIZE_ROTATION 4

#define ASURA_DIRECT3D_BGR_VS_CONSTANT_INSTANCE_SIZE_LIGHTING 3

#define ASURA_DIRECT3D_BGR_VS_CONSTANT_INSTANCE_SIZE (ASURA_DIRECT3D_BGR_VS_CONSTANT_INSTANCE_SIZE_ROTATION + ASURA_DIRECT3D_BGR_VS_CONSTANT_INSTANCE_SIZE_LIGHTING)


//*******************************************************************
// Class Definitions
//*******************************************************************

template< class Tdata>
class Asura_Direct3D_BGR_TemplateCache
{
public:
	Asura_Direct3D_BGR_TemplateCache();
	Asura_Direct3D_BGR_TemplateCache( const uint32 uInitSize);
	~Asura_Direct3D_BGR_TemplateCache();

	void Reset();
	void Clear();
	bool ClearDirect3DData();
	bool CreateDirect3DData();
	
	uint32 Capacity() const;
	uint32 Size() const;
	Tdata& GetNext();
	const Tdata& operator[]( const uint32 uIndex) const;
	Tdata& operator[]( const uint32 uIndex);
	void SetSize( const uint32 uSize);
	void SetCapacity( const uint32 uSize);

private:		
	
	uint32 m_uSize;
	Asura_Botanical_Array< Tdata> m_xArray;
};

template< class Tdata>
class Asura_Direct3D_BGR_VertexBufferCache
{
public:
	Asura_Direct3D_BGR_VertexBufferCache();
	Asura_Direct3D_BGR_VertexBufferCache( const uint32 uInitSize);
	~Asura_Direct3D_BGR_VertexBufferCache();

	void Clear();
	bool ClearDirect3DData();
	bool CreateDirect3DData();
	
	uint32 Capacity() const;
	uint32 Size() const;
	Tdata& GetNext();
	const Tdata& operator[]( const uint32 uIndex) const;
	Tdata& operator[]( const uint32 uIndex);
	void SetSize( const uint32 uSize);
	void SetCapacity( const uint32 uSize);

	bool Lock();
	void Unlock();
	
	bool IsLocked();
	
	void GetVertexBuffer( Direct3D_VertexBuffer*& pxD3DVertexBuffer, u_int& uVertexBufferOffset) const;
	
private:		
	uint32 m_uSize;
	uint32 m_uCapacity;
	uint32 m_uVBStartByteOffset;
	Tdata* m_ptVBStartVertex;
	Asura_Direct3D_VertexBuffer_Dynamic m_xVertexBuffer;
};

template< class Tdata>
class Asura_Direct3D_BGR_IndexBufferCache
{
public:
	Asura_Direct3D_BGR_IndexBufferCache();
	Asura_Direct3D_BGR_IndexBufferCache( const uint32 uInitSize);
	~Asura_Direct3D_BGR_IndexBufferCache();

	void Clear();
	bool ClearDirect3DData();
	bool CreateDirect3DData();
	
	uint32 Capacity() const;
	uint32 Size() const;
	Tdata& GetNext();
	const Tdata& operator[]( const uint32 uIndex) const;
	Tdata& operator[]( const uint32 uIndex);
	void SetSize( const uint32 uSize);
	void SetCapacity( const uint32 uSize);

	bool Lock();
	void Unlock();
	
	bool IsLocked();
	
	void GetIndexBuffer( Direct3D_IndexBuffer*& pxD3DIndexBuffer, u_int& uIndexBufferOffset) const;
	u_int GetIndexBufferOffset( void) const;
	
private:		
	uint32 m_uSize;
	uint32 m_uCapacity;
	uint32 m_uIBStartByteOffset;
	Tdata* m_ptIBStartIndex;
	Asura_Direct3D_IndexBuffer_Dynamic m_xIndexBuffer;
};

struct Asura_Direct3D_BGR_GlobalConstants
{
	Asura_Direct3D_BGR_GlobalConstants() { ASURA_STATIC_ASSERT( sizeof( m_afConstantData) == sizeof( m_xData));}
	union
	{
		// DLH - 30 March 2006 11:09:58 - Do not use this variable to set Data, EVER
		float m_afConstantData[ ASURA_DIRECT3D_BGR_VS_CONSTANT_GLOBAL_SIZE][4];
		
		// DLH - 30 March 2006 11:09:58 - USE THESE to set the Data
		struct
		{
			float m_afViewProj[ 16];
			float m_afCameraRight[4];
			float m_afCameraUp[4];
			float m_afCameraForwards[4];
			union
			{
				struct  
				{
					float m_afFogLinearColour[4];
					float m_afFogLinearPlane[4];
				};
				struct 
				{
					float m_afFogGraphPlane[4];
					float m_afFogGraphScale[2][4];
					float m_afFogGraphCoeffs[2][4];
				};
			};
		}m_xData;
	};
};

#define ASURA_DIRECT3D_BGR_VS_CONSTANT_DYNAMIC_LIGHT_SIZE 3

struct Asura_Direct3D_BGR_DynamicLightConstants
{
	Asura_Direct3D_BGR_DynamicLightConstants() { ASURA_STATIC_ASSERT( sizeof( m_afConstantData) == sizeof( m_xData));}
	union
	{
		// DLH - 30 March 2006 11:09:58 - Do not use this variable to set Data, EVER
		float m_afConstantData[ ASURA_DIRECT3D_BGR_VS_CONSTANT_DYNAMIC_LIGHT_SIZE][4];
		
		// DLH - 30 March 2006 11:09:58 - USE THESE to set the Data
		struct
		{
			float m_afTransformX[4];
			float m_afTransformY[4];
			float m_afTransformZ[4];
		}m_xData;
	};
};

#if defined( ASURA_DIRECT3D_BGR_SUPPORT_SM30) && 1 == ASURA_DIRECT3D_BGR_SUPPORT_SM30
struct Asura_Direct3D_BGR_Type_VertexInstanceSM30
{
	float m_afTransform0[ 4];		// DLH - 
	float m_afTransform1[ 4];		// DLH - 
	float m_afTransform2[ 4];		// DLH - 
	
	float m_afSphericalR[4];				// DLH -
	float m_afSphericalG[4];			// DLH -
	float m_afSphericalB[4];			// DLH -
	
	float m_afBone[ 3];				// DLH - 
	float m_fAlphaCut;				// DLH - Is the .w of the Bone information
};
#endif

#if defined( ASURA_DIRECT3D_BGR_SUPPORT_SM11) && 1 == ASURA_DIRECT3D_BGR_SUPPORT_SM11
struct Asura_Direct3D_BGR_GeometryInstanceConstants
{
	Asura_Direct3D_BGR_GeometryInstanceConstants() { ASURA_STATIC_ASSERT( sizeof( m_afConstantData) == sizeof( m_xData));}
	union
	{
		// DLH - 30 March 2006 11:09:58 - Do not use this variable to set Data, EVER
		float m_afConstantData[ ASURA_DIRECT3D_BGR_VS_CONSTANT_INSTANCE_SIZE][4];
		
		// DLH - 30 March 2006 11:09:58 - USE THESE to set the Data
		struct
		{
			float m_afTransform0[4];
			float m_afTransform1[4];
			float m_afTransform2[4];
			
			float m_fDummy3[3];
			float m_fAlphaCut;
			
			float m_afSphericalR[4];
			float m_afSphericalG[4];
			float m_afSphericalB[4];
		}m_xData;
	};
};
#endif

#if defined( ASURA_DIRECT3D_BGR_SUPPORT_SM30) && 1 == ASURA_DIRECT3D_BGR_SUPPORT_SM30
class Asura_Direct3D_BGR_GeometryVertexInstanceCache : public Asura_Direct3D_BGR_VertexBufferCache< Asura_Direct3D_BGR_Type_VertexInstanceSM30>
{
public:
};

class Asura_Direct3D_BGR_GeometryIndexInstanceCache : public Asura_Direct3D_BGR_IndexBufferCache< uint16>
{
public:
	void Clear();
	
	Asura_DIPCache_Static< 10 > m_xDIPCache;
};
#endif

#if defined( ASURA_DIRECT3D_BGR_SUPPORT_SM11) && 1 == ASURA_DIRECT3D_BGR_SUPPORT_SM11
class Asura_Direct3D_BGR_GeometryVertexInstanceCache : public Asura_Direct3D_BGR_TemplateCache< Asura_Direct3D_BGR_GeometryInstanceConstants>
{
};

class Asura_Direct3D_BGR_GeometryIndexInstanceCache : public Asura_Direct3D_BGR_TemplateCache< uint16>
{
};
#endif

class Asura_Direct3D_BGR_GeometryVertexCache : public Asura_Direct3D_BGR_TemplateCache< Asura_Direct3D_BGR_GeometryVertexInstanceCache>
{
};

class Asura_Direct3D_BGR_GeometryIndexCache : public Asura_Direct3D_BGR_TemplateCache< Asura_Direct3D_BGR_GeometryIndexInstanceCache>
{
};

class Asura_Direct3D_BGR_InstanceCache
{
public:
	float m_fDistance;
	
#if defined( ASURA_DIRECT3D_BGR_SUPPORT_SM11) && 1 == ASURA_DIRECT3D_BGR_SUPPORT_SM11
	Asura_Direct3D_BGR_GeometryInstanceConstants m_xConstants;
#endif
#if defined( ASURA_DIRECT3D_BGR_SUPPORT_SM30) && 1 == ASURA_DIRECT3D_BGR_SUPPORT_SM30
	#if defined( ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS) && 1 == ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS
		const Asura_BGR_Version_ID* m_pxSectionVersions;
	#endif
	
	const Asura_Direct3D_BGR_Instances_Header* m_pxInstanceHeader;
	u_int m_uInstanceHeaderInstanceIndex;
	
	Asura_Direct3D_BGR_Type_VertexInstanceSM30 m_xInstanceVertex;
#endif
};

class Asura_Direct3D_BGR_Cache : public Asura_Direct3D_BGR_TemplateCache< Asura_Direct3D_BGR_InstanceCache>
{
public:
	Asura_Direct3D_BGR_Cache( const uint32 uInitSize);
};

class Asura_Direct3D_BGR_CacheType
{
public:
	Asura_Direct3D_BGR_CacheType();
	
	Asura_Direct3D_BGR_Type* m_pxTree;
	
	Asura_LinkList< const Asura_Direct3D_BGR_Instances_Header* > m_xListRegionDescriptions;
};


//*******************************************************************
// Inline Function Definitions
//*******************************************************************

template< class Tdata> 
Asura_Direct3D_BGR_TemplateCache< Tdata>::Asura_Direct3D_BGR_TemplateCache() :
	m_uSize( 0)
{
}
template< class Tdata> 
Asura_Direct3D_BGR_TemplateCache< Tdata>::Asura_Direct3D_BGR_TemplateCache( const uint32 uInitSize)
{
	this->m_xArray.SetSize( uInitSize);
}
template< class Tdata> 
Asura_Direct3D_BGR_TemplateCache< Tdata>::~Asura_Direct3D_BGR_TemplateCache()
{
}

template< class Tdata> 
inline void Asura_Direct3D_BGR_TemplateCache< Tdata>::Reset()
{ 
	this->m_uSize = 0;
	this->m_xArray.Clear();
}

template< class Tdata> 
inline void Asura_Direct3D_BGR_TemplateCache< Tdata>::Clear()
{ 
	this->m_uSize = 0;
}

template< class Tdata> 
inline bool Asura_Direct3D_BGR_TemplateCache< Tdata>::ClearDirect3DData() 
{ 
	if( false == this->m_xArray.ClearDirect3DData())
	{
		return( false);
	}
	return( true);
}

template< class Tdata> 
inline bool Asura_Direct3D_BGR_TemplateCache< Tdata>::CreateDirect3DData()
{
	if( false == this->m_xArray.CreateDirect3DData())
	{
		return( false);
	}
	return( true);
}

template< class Tdata> 
inline uint32 Asura_Direct3D_BGR_TemplateCache< Tdata>::Capacity() const 
{ 
	return( this->m_xArray.Size());
}

template< class Tdata> 
inline uint32 Asura_Direct3D_BGR_TemplateCache< Tdata>::Size() const 
{ 
	return( this->m_uSize);
}

template< class Tdata> 
inline Tdata& Asura_Direct3D_BGR_TemplateCache< Tdata>::GetNext() 
{ 
	uint32 uIndex = this->m_uSize; 
	this->m_uSize += 1;
	Asura_Assert( uIndex < this->m_xArray.Size(), "Index out of Range");
	
	return( this->m_xArray[ uIndex]);
}

template< class Tdata> 
inline const Tdata& Asura_Direct3D_BGR_TemplateCache< Tdata>::operator[]( const uint32 uIndex) const 
{
	Asura_Assert( uIndex < this->m_xArray.Size(), "Index out of Range");
	return( this->m_xArray[ uIndex]);
}

template< class Tdata> 
inline Tdata& Asura_Direct3D_BGR_TemplateCache< Tdata>::operator[]( const uint32 uIndex) 
{
	Asura_Assert( uIndex < this->m_xArray.Size(), "Index out of Range");
	return( this->m_xArray[ uIndex]);
}

template< class Tdata> 
inline void Asura_Direct3D_BGR_TemplateCache< Tdata>::SetSize( const uint32 uSize)
{
	this->m_xArray.Clear();
	this->m_xArray.SetSize( uSize);
	this->m_uSize = uSize;
}

template< class Tdata> 
inline void Asura_Direct3D_BGR_TemplateCache< Tdata>::SetCapacity( const uint32 uSize)
{
	Asura_Assert( this->m_uSize <= uSize, "Size is smaller");
	this->m_xArray.Clear();
	this->m_xArray.SetSize( uSize);
}


//*******************************************************************
// DLH - Asura_Direct3D_BGR_VertexBufferCache
//*******************************************************************

template< class Tdata> 
inline Asura_Direct3D_BGR_VertexBufferCache< Tdata>::Asura_Direct3D_BGR_VertexBufferCache() :
	m_uSize( 0),
	m_uCapacity( 0),
	m_uVBStartByteOffset( 0),
	m_ptVBStartVertex( NULL),
	m_xVertexBuffer()
{
}

template< class Tdata> 
inline Asura_Direct3D_BGR_VertexBufferCache< Tdata>::Asura_Direct3D_BGR_VertexBufferCache( const uint32 uInitSize) :
	m_uSize( 0),
	m_uCapacity( uInitSize),
	m_uVBStartByteOffset( 0),
	m_ptVBStartVertex( NULL),
	m_xVertexBuffer()
{
}

template< class Tdata> 
inline Asura_Direct3D_BGR_VertexBufferCache< Tdata>::~Asura_Direct3D_BGR_VertexBufferCache()
{
	Asura_Assert( NULL == this->m_ptVBStartVertex, "The VB is locked. You must unlock it before calling this method.");
}

template< class Tdata> 
inline void Asura_Direct3D_BGR_VertexBufferCache< Tdata>::Clear()
{ 
	Asura_Assert( NULL == this->m_ptVBStartVertex, "The VB is locked. You must unlock it before calling this method.");
	
	this->m_uSize = 0;
	this->m_uVBStartByteOffset = 0;
	this->m_ptVBStartVertex = NULL;
}

template< class Tdata> 
inline bool Asura_Direct3D_BGR_VertexBufferCache< Tdata>::ClearDirect3DData()
{ 
	Asura_Assert( NULL == this->m_ptVBStartVertex, "The VB is locked. You must unlock it before calling this method.");
	
	this->m_uSize = 0;
	this->m_uVBStartByteOffset = 0;
	this->m_ptVBStartVertex = NULL;
	
	this->m_xVertexBuffer.ClearDirect3DData();
	
	return( true);
}

template< class Tdata> 
inline bool Asura_Direct3D_BGR_VertexBufferCache< Tdata>::CreateDirect3DData()
{ 
	Asura_Assert( NULL == this->m_ptVBStartVertex, "The VB is locked. You must unlock it before calling this method.");
	
	this->m_xVertexBuffer.CreateDirect3DData();
	
	return( true);
}

template< class Tdata> 
inline uint32 Asura_Direct3D_BGR_VertexBufferCache< Tdata>::Capacity() const 
{ 
	return( this->m_uCapacity);
}

template< class Tdata> 
inline uint32 Asura_Direct3D_BGR_VertexBufferCache< Tdata>::Size() const 
{ 
	return( this->m_uSize);
}

template< class Tdata> 
inline Tdata& Asura_Direct3D_BGR_VertexBufferCache< Tdata>::GetNext() 
{ 
	Asura_Assert( NULL != this->m_ptVBStartVertex, "The VB is NOT locked. You must Lock it before calling this method.");
	uint32 uIndex = this->m_uSize; 
	this->m_uSize += 1;
	Asura_Assert( uIndex < this->m_uCapacity, "Index out of Range");
	
	Tdata& tVertex = this->m_ptVBStartVertex[ uIndex];
	
	Asura_Assert( true == this->m_xVertexBuffer.IsLocked(), "Vertex Buffer Not Locked");
	Asura_Assert( this->m_xVertexBuffer.Assert_LockedMemoryStart() <= &tVertex, "Cache Data is out of Vertex Buffer Locked Range.");
	Asura_Assert( ((&tVertex)+1) <= this->m_xVertexBuffer.Assert_LockedMemoryEnd(), "Cache Data is out of Vertex Buffer Locked Range.");
	
	return( tVertex);
}

template< class Tdata> 
inline const Tdata& Asura_Direct3D_BGR_VertexBufferCache< Tdata>::operator[]( const uint32 uIndex) const 
{
	Asura_Assert( NULL != this->m_ptVBStartVertex, "The VB is NOT locked. You must Lock it before calling this method.");
	Asura_Assert( uIndex < this->m_uCapacity, "Index out of Range");
	return( this->m_ptVBStartVertex[ uIndex]);
}

template< class Tdata> 
inline Tdata& Asura_Direct3D_BGR_VertexBufferCache< Tdata>::operator[]( const uint32 uIndex) 
{
	Asura_Assert( NULL != this->m_ptVBStartVertex, "The VB is NOT locked. You must Lock it before calling this method.");
	Asura_Assert( uIndex < this->m_uCapacity, "Index out of Range");
	return( this->m_ptVBStartVertex[ uIndex]);
}

template< class Tdata> 
inline void Asura_Direct3D_BGR_VertexBufferCache< Tdata>::SetSize( const uint32 uSize)
{
	Asura_Assert( NULL == this->m_ptVBStartVertex, "The VB is locked. You must unlock it before calling this method.");
	
	this->m_uCapacity = uSize;
	this->m_uSize = uSize;
}

template< class Tdata> 
inline void Asura_Direct3D_BGR_VertexBufferCache< Tdata>::SetCapacity( const uint32 uSize)
{
	Asura_Assert( NULL == this->m_ptVBStartVertex, "The VB is locked. You must unlock it before calling this method.");
	Asura_Assert( this->m_uSize <= uSize, "Size is smaller");
	this->m_uCapacity = uSize;
	
	Asura_Direct3D_VertexBuffer_Dynamic_Type< Tdata> xVertexBuffer( this->m_xVertexBuffer);
	
	xVertexBuffer.SetSize( 3 * uSize);
}

template< class Tdata> 
inline bool Asura_Direct3D_BGR_VertexBufferCache< Tdata>::IsLocked()
{
	return( NULL != this->m_ptVBStartVertex);
}

template< class Tdata> 
inline bool Asura_Direct3D_BGR_VertexBufferCache< Tdata>::Lock()
{
	Asura_Assert( NULL == this->m_ptVBStartVertex, "The VB is locked. You must unlock it before calling this method.");
	
	Asura_Direct3D_VertexBuffer_Dynamic_Type< Tdata> xVertexBuffer( this->m_xVertexBuffer);
	
	uint32 uVBStartByteOffset = 0;
	Tdata* ptVBStartVertex = NULL;
	const bool bLockStatus = xVertexBuffer.Lock( this->m_uCapacity, uVBStartByteOffset, ptVBStartVertex);
	if( false == bLockStatus)
	{
		return( false);
	}
	
	this->m_uVBStartByteOffset = uVBStartByteOffset;
	this->m_ptVBStartVertex = ptVBStartVertex;
	
	return( true);
}

template< class Tdata> 
inline void Asura_Direct3D_BGR_VertexBufferCache< Tdata>::Unlock()
{
	Asura_Assert( NULL != this->m_ptVBStartVertex, "The VB is NOT locked. You must Lock it before calling this method.");
	
	Asura_Direct3D_VertexBuffer_Dynamic_Type< Tdata> xVertexBuffer( this->m_xVertexBuffer);
	
	xVertexBuffer.Unlock( this->m_uSize);
	
	this->m_ptVBStartVertex = NULL;
}

template< class Tdata> 
inline void Asura_Direct3D_BGR_VertexBufferCache< Tdata>::GetVertexBuffer( Direct3D_VertexBuffer*& pxD3DVertexBuffer, u_int& uVertexBufferOffset) const
{
	Asura_Assert( NULL == this->m_ptVBStartVertex, "The VB is locked. You must unlock it before calling this method.");
	pxD3DVertexBuffer = this->m_xVertexBuffer.GetVertexBuffer();
	uVertexBufferOffset	= this->m_uVBStartByteOffset;
}

//*******************************************************************
// DLH - Asura_Direct3D_BGR_IndexBufferCache
//*******************************************************************

template< class Tdata> 
inline Asura_Direct3D_BGR_IndexBufferCache< Tdata>::Asura_Direct3D_BGR_IndexBufferCache() :
	m_uSize( 0),
	m_uCapacity( 0),
	m_uIBStartByteOffset( 0),
	m_ptIBStartIndex( NULL),
	m_xIndexBuffer()
{
}

template< class Tdata> 
inline Asura_Direct3D_BGR_IndexBufferCache< Tdata>::Asura_Direct3D_BGR_IndexBufferCache( const uint32 uInitSize) :
	m_uSize( 0),
	m_uCapacity( uInitSize),
	m_uIBStartByteOffset( 0),
	m_ptIBStartIndex( NULL),
	m_xIndexBuffer()
{
}

template< class Tdata> 
inline Asura_Direct3D_BGR_IndexBufferCache< Tdata>::~Asura_Direct3D_BGR_IndexBufferCache()
{
	Asura_Assert( NULL == this->m_ptIBStartIndex, "The IB is locked. You must unlock it before calling this method.");
}

template< class Tdata> 
inline void Asura_Direct3D_BGR_IndexBufferCache< Tdata>::Clear()
{ 
	Asura_Assert( NULL == this->m_ptIBStartIndex, "The IB is locked. You must unlock it before calling this method.");
	
	this->m_uSize = 0;
	this->m_uIBStartByteOffset = 0;
	this->m_ptIBStartIndex = NULL;
}

template< class Tdata> 
inline bool Asura_Direct3D_BGR_IndexBufferCache< Tdata>::ClearDirect3DData()
{ 
	Asura_Assert( NULL == this->m_ptIBStartIndex, "The IB is locked. You must unlock it before calling this method.");
	
	this->m_uSize = 0;
	this->m_uIBStartByteOffset = 0;
	this->m_ptIBStartIndex = NULL;
	
	this->m_xIndexBuffer.ClearDirect3DData();
	
	return( true);
}

template< class Tdata> 
inline bool Asura_Direct3D_BGR_IndexBufferCache< Tdata>::CreateDirect3DData()
{ 
	Asura_Assert( NULL == this->m_ptIBStartIndex, "The IB is locked. You must unlock it before calling this method.");
	
	this->m_xIndexBuffer.CreateDirect3DData();
	
	return( true);
}

template< class Tdata> 
inline uint32 Asura_Direct3D_BGR_IndexBufferCache< Tdata>::Capacity() const 
{ 
	return( this->m_uCapacity);
}

template< class Tdata> 
inline uint32 Asura_Direct3D_BGR_IndexBufferCache< Tdata>::Size() const 
{ 
	return( this->m_uSize);
}

template< class Tdata> 
inline Tdata& Asura_Direct3D_BGR_IndexBufferCache< Tdata>::GetNext() 
{ 
	Asura_Assert( NULL != this->m_ptIBStartIndex, "The IB is NOT locked. You must Lock it before calling this method.");
	uint32 uIndex = this->m_uSize; 
	this->m_uSize += 1;
	Asura_Assert( uIndex < this->m_uCapacity, "Index out of Range");
	
	Tdata& tIndex = this->m_ptIBStartIndex[ uIndex];
	
	Asura_Assert( true == this->m_xIndexBuffer.IsLocked(), "Index Buffer Not Locked");
	Asura_Assert( this->m_xIndexBuffer.Assert_LockedMemoryStart() <= &tIndex, "Cache Data is out of Index Buffer Locked Range.");
	Asura_Assert( ((&tIndex)+1) <= this->m_xIndexBuffer.Assert_LockedMemoryEnd(), "Cache Data is out of Index Buffer Locked Range.");
	
	return( tIndex);
}

template< class Tdata> 
inline const Tdata& Asura_Direct3D_BGR_IndexBufferCache< Tdata>::operator[]( const uint32 uIndex) const 
{
	Asura_Assert( NULL != this->m_ptIBStartIndex, "The IB is NOT locked. You must Lock it before calling this method.");
	Asura_Assert( uIndex < this->m_uCapacity, "Index out of Range");
	return( this->m_ptIBStartIndex[ uIndex]);
}

template< class Tdata> 
inline Tdata& Asura_Direct3D_BGR_IndexBufferCache< Tdata>::operator[]( const uint32 uIndex) 
{
	Asura_Assert( NULL != this->m_ptIBStartIndex, "The IB is NOT locked. You must Lock it before calling this method.");
	Asura_Assert( uIndex < this->m_uCapacity, "Index out of Range");
	return( this->m_ptIBStartIndex[ uIndex]);
}

template< class Tdata> 
inline void Asura_Direct3D_BGR_IndexBufferCache< Tdata>::SetSize( const uint32 uSize)
{
	Asura_Assert( NULL == this->m_ptIBStartIndex, "The IB is locked. You must unlock it before calling this method.");
	
	this->m_uCapacity = uSize;
	this->m_uSize = uSize;
}

template< class Tdata> 
inline void Asura_Direct3D_BGR_IndexBufferCache< Tdata>::SetCapacity( const uint32 uSize)
{
	Asura_Assert( NULL == this->m_ptIBStartIndex, "The IB is locked. You must unlock it before calling this method.");
	Asura_Assert( this->m_uSize <= uSize, "Size is smaller");
	this->m_uCapacity = uSize;
	
	Asura_Direct3D_IndexBuffer_Dynamic_Type< Tdata> xIndexBuffer( this->m_xIndexBuffer);
	
	xIndexBuffer.SetSize( 3 * uSize);
}

template< class Tdata> 
inline bool Asura_Direct3D_BGR_IndexBufferCache< Tdata>::IsLocked()
{
	return( NULL != this->m_ptIBStartIndex);
}

template< class Tdata> 
inline bool Asura_Direct3D_BGR_IndexBufferCache< Tdata>::Lock()
{
	Asura_Assert( NULL == this->m_ptIBStartIndex, "The IB is locked. You must unlock it before calling this method.");
	
	Asura_Direct3D_IndexBuffer_Dynamic_Type< Tdata> xIndexBuffer( this->m_xIndexBuffer);
	
	uint32 uIBStartByteOffset = 0;
	Tdata* ptIBStartIndex = NULL;
	const bool bLockStatus = xIndexBuffer.Lock( this->m_uCapacity, uIBStartByteOffset, ptIBStartIndex);
	if( false == bLockStatus)
	{
		return( false);
	}
	
	this->m_uIBStartByteOffset = uIBStartByteOffset;
	this->m_ptIBStartIndex = ptIBStartIndex;
	
	return( true);
}

template< class Tdata> 
inline void Asura_Direct3D_BGR_IndexBufferCache< Tdata>::Unlock()
{
	Asura_Assert( NULL != this->m_ptIBStartIndex, "The IB is NOT locked. You must Lock it before calling this method.");
	
	Asura_Direct3D_IndexBuffer_Dynamic_Type< Tdata> xIndexBuffer( this->m_xIndexBuffer);
	
	xIndexBuffer.Unlock( this->m_uSize);
	
	this->m_ptIBStartIndex = NULL;
}

template< class Tdata> 
inline void Asura_Direct3D_BGR_IndexBufferCache< Tdata>::GetIndexBuffer( Direct3D_IndexBuffer*& pxD3DIndexBuffer, u_int& uIndexBufferOffset) const
{
	Asura_Assert( NULL == this->m_ptIBStartIndex, "The IB is locked. You must unlock it before calling this method.");
	pxD3DIndexBuffer = this->m_xIndexBuffer.GetIndexBuffer();
	uIndexBufferOffset = this->m_uIBStartByteOffset;
}

template< class Tdata> 
inline u_int Asura_Direct3D_BGR_IndexBufferCache< Tdata>::GetIndexBufferOffset() const
{
	Asura_Assert( 0 == (this->m_uIBStartByteOffset & (sizeof( Tdata)-1)), "Pointer not aligned!" );
	return( this->m_uIBStartByteOffset / sizeof( Tdata));
}

#if defined( ASURA_DIRECT3D_BGR_SUPPORT_SM30) && 1 == ASURA_DIRECT3D_BGR_SUPPORT_SM30
inline void Asura_Direct3D_BGR_GeometryIndexInstanceCache::Clear()
{
	this->Asura_Direct3D_BGR_IndexBufferCache<uint16>::Clear();
	this->m_xDIPCache.Clear();
}
#endif

//*******************************************************************
//
// (c) 2006 Rebellion (Derby)
//
// Warning: This computer program is protected by copyright law and 
// international treaties. Unauthorized reproduction or distribution
// of this program, or any portion of it, may result in severe civil and
// criminal penalties, and will be prosecuted to the maximum extent 
// possible under the law.
//
//*******************************************************************

#endif //defined( ASURA_USE_BOTANICALS)

#endif // _ASURA_DIRECT3D_BOTANICALS_GAME_RENDER_CACHE_H_
