//*******************************************************************
//
// File: Asura_Direct3D_Botanicals_Game_Render_Manager.cpp
//
// Created: 15 Mar 2006
//
// Description: 
//
//*******************************************************************

// system libraries
#include "Core/Asura.h"

#if defined( ASURA_USE_BOTANICALS)

// main header
#include "Asura_Direct3D_Botanicals_Game_Render_Manager.h"

// other libraries

// internal dependencies
#include "Camera/Asura_Camera.h"

#include "Core/Asura_Mempool.h"
#include "Core/Asura_TextureManagement.h"
#include "Core/Memory/Asura_ResourceAllocator.h"

#include "Light/Asura_GI_Solution.h"
#include "Light/Asura_Lighting_Solution.h"

#include "Rendering/Asura_Direct3D.h"
#include "Rendering/Asura_DIPCache.h"
#include "Rendering/Asura_Viewport.h"

#if defined( ASURA_TOOLS)
#include "Botanicals/Tools_Botanicals_Statistics.h"
#include "Botanicals/Tools_Mode_Botanicals.h"
#include "Misc/Tools_Render.h"
#include "Tools_System.h"
#endif

#if defined( ASURA_DIRECT3D_BGR_SUPPORT_SM30) && 1 == ASURA_DIRECT3D_BGR_SUPPORT_SM30
	#include "Botanical/Asura_Direct3D_Botanicals_Game_Render_Manager_SM30.h"
#endif
#if defined( ASURA_DIRECT3D_BGR_SUPPORT_SM11) && 1 == ASURA_DIRECT3D_BGR_SUPPORT_SM11
	#include "Botanical/Asura_Direct3D_Botanicals_Game_Render_Manager_SM11.h"
#endif

//*******************************************************************
// Constants
//*******************************************************************

#if defined( ASURA_DIRECT3D_BGR_SUPPORT_SM11)
	#define ASURA_DIRECT3D_BGR_SUPPORT_PHOTONS	(1)
#endif

#if defined( ASURA_BOTANICAL_SUPPORT_INTERACTIVE) && 1 == ASURA_BOTANICAL_SUPPORT_INTERACTIVE
	#if !defined( ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA) || 0 == ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA
		#error I want ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA turned on as is ASURA_BOTANICAL_SUPPORT_INTERACTIVE is turned on
	#endif
#endif

#if defined( ASURA_DIRECT3D_BGR_SUPPORT_DYNAMIC_DATA)
	#pragma message( __FILE__ ": #define ASURA_DIRECT3D_BGR_SUPPORT_DYNAMIC_DATA is not used anymore please use #define ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA")
#endif

//*******************************************************************
// Local Classes
//*******************************************************************

class Asura_Direct3D_BGR_ProcessInstanceHeader
{
public:
	float m_fScreenDistanceScale;
	float m_fLodLimitNear;
	float m_fInverseLodLimitDelta;
	Asura_AVector3 m_xCameraPos;
};

//*******************************************************************
// Function Definitions
//*******************************************************************

Asura_Direct3D_BGR_Material::Asura_Direct3D_BGR_Material()
{
#if defined( ASURA_USE_MATERIAL_RESPONSE_SYSTEM )
	this->m_xMaterialResponseHashID = ASURA_HASH_ID_UNSET;
#else
	this->m_iDiffuseTexture0 = ASURA_TEXTURE_NONE;
	this->m_iDiffuseTexture1 = ASURA_TEXTURE_NONE;
	this->m_bDiffuseDualPassTexture = false;
	
	this->m_iSpecularTexture0 = ASURA_TEXTURE_NONE;
	this->m_iSpecularTexture1 = ASURA_TEXTURE_NONE;
	this->m_bSpecularDualPassTexture = false;

	this->m_bHasNormalMaps = false;
	this->m_iNormalTexture0 = ASURA_TEXTURE_NONE;
	this->m_iNormalTexture1 = ASURA_TEXTURE_NONE;
	this->m_bNormalDualPassTexture = false;

	this->m_bParallaxMap = false;
	this->m_bSelfIlluminating = false;
	this->m_bSpecular = false;
	this->m_bOutputAlphaOfTexture = false;
#endif
}

Asura_Direct3D_BGR_Cache::Asura_Direct3D_BGR_Cache( const uint32 uInitSize) :
	Asura_Direct3D_BGR_TemplateCache< Asura_Direct3D_BGR_InstanceCache>( uInitSize)
{
}

inline bool Asura_Direct3D_BGR_Type_GeometryLODRange::IsInRange( const float fDelta) const
{
	if( this->m_afDelta[ LOW_0] <= fDelta && fDelta < this->m_afDelta[ HIGH_0])
	{
		return( true);
	}
	
	return( false);
}

float Asura_Direct3D_BGR_Type_GeometryLODRange::CalcStrength(const float fDelta) const
{
//	CDC_ASSERT( true == this->IsInRange( delta));
	
	float fRamp0 = (fDelta - this->m_afDelta[ LOW_0]) * this->m_afDelta[ LOW_SCALE];
	float fRamp1 = (this->m_afDelta[ HIGH_0] - fDelta) * this->m_afDelta[ HIGH_SCALE];

	float fAlpha = Asura_Maths::Min( fRamp0, fRamp1);
	
	fAlpha = Asura_Maths::Max( 0.0f, fAlpha);	
	fAlpha = Asura_Maths::Min( fAlpha, 1.0f);	
	
	return( fAlpha);
}

Asura_Direct3D_BGR_Type_GeometryLODRange::Asura_Direct3D_BGR_Type_GeometryLODRange()
{
	this->m_afDelta[ LOW_0] = 1.0f;
	this->m_afDelta[ LOW_1] = 0.0f;
	this->m_afDelta[ LOW_SCALE] = 0.0f;
	this->m_afDelta[ HIGH_1] = 0.0f;
	this->m_afDelta[ HIGH_0] = 1.0f;
	this->m_afDelta[ HIGH_SCALE] = 0.0f;
}

void Asura_Direct3D_BGR_Type_GeometryLODRange::CalcScales()
{
//DeadCode CSB 23/05/2006 	this->m_afDelta[ LOW_SCALE] = 1e3;
//DeadCode CSB 23/05/2006 	this->m_afDelta[ HIGH_SCALE] = 1e3;
	m_afDelta[LOW_SCALE]  = Asura_Maths::LargeFloat;	//CSB 23/05/2006
	m_afDelta[HIGH_SCALE] = Asura_Maths::LargeFloat;	//CSB 23/05/2006
	
	if( this->m_afDelta[ LOW_0] < this->m_afDelta[ LOW_1])
	{
		this->m_afDelta[ LOW_SCALE] = 1.0f / (this->m_afDelta[ LOW_1] - this->m_afDelta[ LOW_0]);
	}
	
	if( this->m_afDelta[ HIGH_1] < this->m_afDelta[ HIGH_0])
	{
		this->m_afDelta[ HIGH_SCALE] = 1.0f / (this->m_afDelta[ HIGH_0] - this->m_afDelta[ HIGH_1]);
	}
}

Asura_Direct3D_BGR_Type_Geometry::Asura_Direct3D_BGR_Type_Geometry()
{
	this->m_pxVertexBuffer = NULL;
	this->m_pxIndexBuffer = NULL;
	
	//this->m_xLodRange;
	this->m_fAlphaCutValue = 0.5f;
	
//	this->m_iTextureDiffuse = ASURA_TEXTURE_INVALID;
//	this->m_iTextureBump = ASURA_TEXTURE_INVALID;
	
	this->m_uBufferInstances = 0;
	
	this->m_uVBStart = 0;
	this->m_uVBSize = 0;
	
	this->m_uIBStart = 0;
	this->m_uIBSize = 0;
	
	this->m_uPrimitiveCount = 0;
}

Asura_Direct3D_BGR_Type_Geometry::~Asura_Direct3D_BGR_Type_Geometry()
{
	Asura_Assert( NULL == this->m_pxIndexBuffer, "Index Buffer not released");
	Asura_Assert( NULL == this->m_pxVertexBuffer, "Vertex Buffer not released");
}

Asura_Direct3D_BGR_Type_Geometry& Asura_Direct3D_BGR_Type_Geometry::operator =( const Asura_Direct3D_BGR_Type_Geometry& xRHS)
{
	Asura_Assert( NULL == this->m_pxVertexBuffer, "Pointer Not Clear");
	this->m_pxVertexBuffer = xRHS.m_pxVertexBuffer;
	if( NULL != this->m_pxVertexBuffer)
	{
		this->m_pxVertexBuffer->AddRef();
	}
	Asura_Assert( NULL == this->m_pxIndexBuffer, "Pointer Not Clear");
	this->m_pxIndexBuffer = xRHS.m_pxIndexBuffer;
	if( NULL != this->m_pxIndexBuffer)
	{
		this->m_pxIndexBuffer->AddRef();
	}
	
	this->m_xLodRange = xRHS.m_xLodRange;
	
	this->m_fAlphaCutValue = xRHS.m_fAlphaCutValue;
	
//	this->m_iTextureDiffuse = xRHS.m_iTextureDiffuse;
//	this->m_iTextureBump = xRHS.m_iTextureBump;
	this->xMaterial = xRHS.xMaterial;
	
	this->m_uBufferInstances = xRHS.m_uBufferInstances;
	
	this->m_uVBStart = xRHS.m_uVBStart;
	this->m_uVBSize = xRHS.m_uVBSize;
	
	this->m_uIBStart = xRHS.m_uIBStart;
	this->m_uIBSize = xRHS.m_uIBSize;
	
	this->m_uPrimitiveCount = xRHS.m_uPrimitiveCount;
	
	return( *this);
}

u_int Asura_Direct3D_BGR_Manager::GetInstanceHeaderCount( void) const
{
	return( this->m_xArray_pxInstanceHeaders.Size());
}

const Asura_BGR_Instances_Header* Asura_Direct3D_BGR_Manager::GetInstanceHeader( const u_int uInstanceHeaderIndex) const
{
	Asura_Direct3D_BGR_Instances_Header* pxInstanceHeader = this->m_xArray_pxInstanceHeaders[ uInstanceHeaderIndex];
	return( pxInstanceHeader);
}

Asura_BGR_Instances_Header* Asura_Direct3D_BGR_Manager::GetInstanceHeader( const u_int uInstanceHeaderIndex)
{
	Asura_Direct3D_BGR_Instances_Header* pxInstanceHeader = this->m_xArray_pxInstanceHeaders[ uInstanceHeaderIndex];
	return( pxInstanceHeader);
}

Asura_BGR_Type* Asura_Direct3D_BGR_Manager::GetType( const Asura_BGR_TypeID xTypeID)
{
	Asura_Direct3D_BGR_Type* pxType = this->GetDirect3DType( xTypeID);
	return( pxType);	
}

uint32 Asura_Direct3D_BGR_Manager::GetTypeIDCeiling( void) const 
{
	uint32 uTypeID = 0;
	const Asura_Botanical_Array< Asura_Direct3D_BGR_Type> &xTypes = this->m_xArray_xTypes;
	
	const uint32 uTypeCount = xTypes.Size();
	if( 0 == uTypeCount)
	{
		return( Asura_BGR_TypeID::INVALID_ID);
	}
	
	for( uint32 uTypeIndex = 0; uTypeIndex < uTypeCount; uTypeIndex += 1)
	{
		const Asura_Direct3D_BGR_Type& xBotanicalTree = xTypes[ uTypeIndex];
		const Asura_BGR_TypeID xTreeTypeID = xBotanicalTree.GetType();
		if( xTreeTypeID == Asura_BGR_TypeID::INVALID_ID)
		{
			continue;
		}
		const u_int uTreeTypeID = xTreeTypeID.GetID();
		uTypeID = Asura_Maths::Max( uTypeID, uTreeTypeID);
	}

	return( uTypeID);
}

Asura_Direct3D_BGR_Type* Asura_Direct3D_BGR_Manager::GetDirect3DType( const Asura_BGR_TypeID& xTypeID)
{
	Asura_Assert( xTypeID != Asura_BGR_TypeID::INVALID_ID, "A Valid ID would be nice");
	const uint32 uTypeID = xTypeID.GetID();
	Asura_Botanical_Array< Asura_Direct3D_BGR_Type> &xTypes = this->m_xArray_xTypes;
	
	const uint32 uTypeCount = xTypes.Size();
	for( uint32 uTypeIndex = 0; uTypeIndex < uTypeCount; uTypeIndex += 1)
	{
		Asura_Direct3D_BGR_Type& xBotanicalTree = xTypes[ uTypeIndex];
		if( xBotanicalTree.GetTreeId() == uTypeID)
		{
			return( &xBotanicalTree);
		}
	}

	return( NULL);
}

bool Asura_Direct3D_BGR_RenderCache::AddPhotonToDynamicLightCache( const Asura_Photon* pxPhoton)
{
	Asura_Botanical_Array< Asura_Direct3D_BGR_DynamicLightCache>& xArray_xDynamicLightCache = this->m_xArray_xDynamicLightCache;
	const u_int uDynamicLightCacheCount = xArray_xDynamicLightCache.Size();
	for( u_int uDynamicLightCacheIndex = 0; uDynamicLightCacheIndex < uDynamicLightCacheCount; uDynamicLightCacheIndex += 1)
	{
		Asura_Direct3D_BGR_DynamicLightCache& xDynamicLightCache = xArray_xDynamicLightCache[ uDynamicLightCacheIndex];
		
		if( pxPhoton == xDynamicLightCache.m_pxPhoton)
		{
			return( true);
		}
		
		if( NULL == xDynamicLightCache.m_pxPhoton)
		{
			xDynamicLightCache.m_pxPhoton = pxPhoton;
			return( true);
		}
	}
	
	return( false);
}

// MGH 21/1/2008:  Free up all of the allocated memory and return the render cache to the state
// it was in before doing the first level load. This is to prevent memory fragmentation.
void Asura_Direct3D_BGR_RenderCache::Reset()
{
	Clear();

	// Free up allocated memory.
	m_xCacheVertexGeometrys.Reset();
	m_xCacheIndexGeometrys.Reset();

#if defined( ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS) && 1 == ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS
	m_xArraySection_xArrayVersion_bUsed.Clear();
#endif
}

void Asura_Direct3D_BGR_RenderCache::Clear()
{
#if defined( ASURA_ASSERT)
	bool bFoundNULLPhoton = false;
#endif
#if defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS)
	this->m_xDIPCache.Clear();
#endif
	this->m_xCacheInstances.Clear();
	
	const u_int uDynamicLightCacheCount = this->m_xArray_xDynamicLightCache.Size();
	for( u_int uDynamicLightCacheIndex = 0; uDynamicLightCacheIndex < uDynamicLightCacheCount; uDynamicLightCacheIndex += 1)
	{
		Asura_Direct3D_BGR_DynamicLightCache& xDynamicLightCache = this->m_xArray_xDynamicLightCache[ uDynamicLightCacheIndex];

		Asura_Assertf( false == bFoundNULLPhoton || NULL == xDynamicLightCache.m_pxPhoton, ("Photon Cache %d is in use but an earlier Photon Cache is empty.\n This should never happen.", uDynamicLightCacheIndex));
		#if defined( ASURA_ASSERT)
		if( NULL == xDynamicLightCache.m_pxPhoton)
		{
			bFoundNULLPhoton = true;
		}
		#endif
				
		xDynamicLightCache.m_pxPhoton = NULL;
		xDynamicLightCache.m_xCacheInstances.Clear();
#if defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS)
		xDynamicLightCache.m_xDIPCache.Clear();
#endif
	}
}

void Asura_Direct3D_BGR_RenderCache::ClearInstances( const Asura_Direct3D_BGR_Type* pxTreeDescription)
{
	this->m_bDynamicLightInstancesExist = false;
	this->m_xCacheInstances.Clear();
	
	const u_int uDynamicLightCacheCount = this->m_xArray_xDynamicLightCache.Size();
	for( u_int uDynamicLightCacheIndex = 0; uDynamicLightCacheIndex < uDynamicLightCacheCount; uDynamicLightCacheIndex += 1)
	{
		Asura_Direct3D_BGR_DynamicLightCache& xDynamicLightCache = this->m_xArray_xDynamicLightCache[ uDynamicLightCacheIndex];
		
		xDynamicLightCache.m_xCacheInstances.Clear();
	}
	
#if defined( ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS) && 1 == ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS
	if( NULL != pxTreeDescription)
	{
		const u_int uSectionCount = pxTreeDescription->m_xArray_xSections.Size();
		for( u_int uSectionIndex = 0; uSectionIndex < uSectionCount; uSectionIndex += 1)
		{
			Asura_Botanical_Array< bool>& xArrayVersion_bUsed = this->m_xArraySection_xArrayVersion_bUsed[ uSectionIndex];
			
			const Asura_Direct3D_BGR_Section& xSection = pxTreeDescription->m_xArray_xSections[ uSectionIndex];
			
			const u_int uSectionVersionCount = xSection.m_xArray_xVersions.Size();
			for( u_int uSectionVersionIndex = 0; uSectionVersionIndex < uSectionVersionCount; uSectionVersionIndex += 1)
			{
				xArrayVersion_bUsed[ uSectionVersionIndex] = false;
			}
		}
	}
#endif
}


template< bool bUsePhotonSources, bool bUseInteractive> void Asura_Direct3D_BGR_Manager::DrawCache_ProcessInstanceHeader( const Asura_Direct3D_BGR_ProcessInstanceHeader& xProcessInstanceHeader, const Asura_Direct3D_BGR_Type* pxTreeDescription, Asura_Direct3D_BGR_RenderCache& xRenderCache, const Asura_Direct3D_BGR_Instances_Header* pxInstanceHeader, const u_int uIndexTreeBatchStart, const u_int uIndexTreeBatchStop)
{
	Asura_Direct3D_BGR_Cache &xCacheInstances = xRenderCache.m_xCacheInstances;
	
#if defined( ASURA_DIRECT3D_BGR_SUPPORT_SM30) && 1 == ASURA_DIRECT3D_BGR_SUPPORT_SM30
#if defined( ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS) && 1 == ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS
	const u_int uSectionCountMax = pxTreeDescription->m_xArray_xSections.Size();
#endif
#endif	// defined( ASURA_DIRECT3D_BGR_SUPPORT_SM30) && 1 == ASURA_DIRECT3D_BGR_SUPPORT_SM30

	for( uint32 uIndexTreeBatch = uIndexTreeBatchStart; uIndexTreeBatch < uIndexTreeBatchStop; uIndexTreeBatch += 1)
	{
		const Asura_Direct3D_BGR_Instance* pxInstance = pxInstanceHeader->GetDirect3DInstance( uIndexTreeBatch);

#if defined( ASURA_TOOLS)
		if( false == pxInstance->IsActive())
		{
			continue;
		}
#endif

		Asura_AVector3 xInstancePosition;
		pxInstance->GetInstancePosition( xInstancePosition);
		Asura_Vector_3 xInstancePositionV3( xInstancePosition);

        const float fInstanceBoundingRadius = pxTreeDescription->GetBoundingRadius() * pxInstance->GetScale();

#ifdef ASURA_USE_NEW_VISIBILITY_SYSTEM

#ifdef ASURA_TOOLS
        if( Tools_System::GetCurrentMode() != TOOLS_MODE_BOTANICALS || Tools_Mode_Botanicals::ShouldCheckInstanceVisibility() )
#endif
        {
		    if( !pxInstanceHeader->m_xInstanceVisQueries[uIndexTreeBatch].IsSphereVisibleFromCurrentCamera(xInstancePositionV3, fInstanceBoundingRadius) )
		    {
			    continue;
		    }
        }

#ifdef ASURA_TOOLS
        if( Tools_System::GetCurrentMode() == TOOLS_MODE_BOTANICALS && Tools_Mode_Botanicals::ShouldShowBoundingVolumes() )
        {
            InstanceBoundingSphere xBS;
            xBS.xCentre = xInstancePositionV3;
            xBS.fRadius = fInstanceBoundingRadius;

            m_xRenderedInstanceBoundingSpheres.Push_Back( xBS );
        }
#endif

#endif // #ifdef ASURA_USE_NEW_VISIBILITY_SYSTEM

		Asura_Direct3D_BGR_InstanceCache &xCacheInstance = xCacheInstances.GetNext();
		
		Asura_AVector3 xDirectionToCamera( xProcessInstanceHeader.m_xCameraPos - xInstancePosition);
		float fDistance = ((xProcessInstanceHeader.m_fScreenDistanceScale * xDirectionToCamera.Magnitude()) - xProcessInstanceHeader.m_fLodLimitNear) * xProcessInstanceHeader.m_fInverseLodLimitDelta;
		fDistance = Asura_Maths::Max( 0.0f, fDistance);
		fDistance = Asura_Maths::Min( 1.0f, fDistance);
		xCacheInstance.m_fDistance = fDistance;
		
#if defined( ASURA_DIRECT3D_BGR_SUPPORT_SM11) && 1 == ASURA_DIRECT3D_BGR_SUPPORT_SM11
		pxInstance->GetInstanceData( xCacheInstance.m_xConstants.m_xData);
#endif

#if defined( ASURA_DIRECT3D_BGR_SUPPORT_SM30) && 1 == ASURA_DIRECT3D_BGR_SUPPORT_SM30
		xCacheInstance.m_pxInstanceHeader = pxInstanceHeader;
		xCacheInstance.m_uInstanceHeaderInstanceIndex = uIndexTreeBatch;

		pxInstance->GetInstanceData( xCacheInstance.m_xInstanceVertex);

		pxInstance->m_bVisible = true;
		
		#if defined( ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA) && 1 == ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA
		{
			// DLH - If using sections and versions we must have the Dynamic Instance pointer
			#if defined( ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS) && 1 == ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS
			const Asura_BGR_DynamicInstance* pxDynamicInstance = pxInstanceHeader->GetDynamicInstanceData( uIndexTreeBatch);
		
			{
				// DLH - 27 November 2006 16:11:58 - Get the Section Version for this instance
				const Asura_BGR_Version_ID* pxSectionVersions0 = pxInstanceHeader->GetDynamicSectionVersions( pxDynamicInstance);

				// DLH - 27 November 2006 16:12:12 - Save the Section Version array for the render
				xCacheInstance.m_pxSectionVersions = pxSectionVersions0;

				// DLH - 27 November 2006 16:12:30 - Scan the Section Version array for which ones are in use and cache the data
				for( u_int uSectionIndex = 0; uSectionIndex < uSectionCountMax; uSectionIndex += 1)
				{
					Asura_BGR_Version_ID xVersionID = pxSectionVersions0[ uSectionIndex];
					if( ASURA_BGR_VERSION_ID_OFF == xVersionID)
					{
						continue;
					}
					
					const Asura_Direct3D_BGR_Section& xSection = pxTreeDescription->m_xArray_xSections[ uSectionIndex];
					
					u_int uVersionIndex = xSection.m_xArray_uVersionIDToVersionIndex[ xVersionID];
					
					if( ASURA_BGR_VERSION_ID_OFF != uVersionIndex)
					{
						xRenderCache.m_xArraySection_xArrayVersion_bUsed[ uSectionIndex][ uVersionIndex] = true;
					}
				}
			}
			#else
			// DLH - Only get the pointer if it is needed
			const Asura_BGR_DynamicInstance* pxDynamicInstance;
			if( true == bUseInteractive)
			{
				pxDynamicInstance = pxInstanceHeader->GetDynamicInstanceData( uIndexTreeBatch);
			}
			#endif

			if( true == bUseInteractive)
			{
				const Asura_BGR_InteractiveBasic* pxInteractiveBasic = pxInstanceHeader->GetDynamicInteractiveData( pxDynamicInstance);

				xCacheInstance.m_xInstanceVertex.m_afBone[0] = pxInteractiveBasic->m_afPosition[0];
				xCacheInstance.m_xInstanceVertex.m_afBone[1] = pxInteractiveBasic->m_afPosition[1];
				xCacheInstance.m_xInstanceVertex.m_afBone[2] = pxInteractiveBasic->m_afPosition[2];
			}
			else
			{
				xCacheInstance.m_xInstanceVertex.m_afBone[0] = 0;
				xCacheInstance.m_xInstanceVertex.m_afBone[1] = 0;
				xCacheInstance.m_xInstanceVertex.m_afBone[2] = 0;
			}
		}
		
		#else
		
		xCacheInstance.m_xInstanceVertex.m_afBone[0] = 0;
		xCacheInstance.m_xInstanceVertex.m_afBone[1] = 0;
		xCacheInstance.m_xInstanceVertex.m_afBone[2] = 0;
		
		#endif
#endif

		if( true == bUsePhotonSources)
		{
			this->AdjustInstanceDataWithDynamicPhotoEmitter( xInstancePositionV3, *pxTreeDescription, xRenderCache, xCacheInstance);
		}
	}
}

template< bool bUsePhotonSources> void Asura_Direct3D_BGR_Manager::DrawCache( Asura_TrackingHandlePool<Asura_Photon>* pxPhotonList)
{
	const uint32 uCountTypes = this->m_xArrayTypeCache.Size();
	if( 0 == uCountTypes)
	{
		return;
	}
	
	if( true == bUsePhotonSources && ASURA_TEXTURE_INVALID == this->m_iAttenuationMap)
	{
		// DLH - Have to do this here and not in the DirectX Resource allocation as a texture tree crashes
		this->m_iAttenuationMap = Asura_Texture_Management::AddTextureWithResetGroup("\\graphicnovel\\expattenuation.dds", ASURA_TEXTURE_LOADED_PERMANENT );
		//this->m_iAttenuationMap = Asura_Texture_Management::AddTextureWithResetGroup("\\Users\\Duncanh\\DynamicLightGrid.tga", ASURA_TEXTURE_LOADED_PERMANENT );
	}
	
#if defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS)
	if( true == this->m_bDirect3DImpostersOutofDate)
	{
		this->CreateDirect3DData_Imposters();
	}
#endif
	
	this->m_bDrawGlobalRenderStatesSet = false;
	
	Asura_Direct3D_BGR_ProcessInstanceHeader xProcessInstanceHeader;
	
	xProcessInstanceHeader.m_xCameraPos = *Asura_Camera::GetPosition();
	
	Asura_Direct3D_BGR_RenderCache& xRenderCache = this->m_xRenderCache;
	Asura_Direct3D_BGR_Cache &xCacheInstances = xRenderCache.m_xCacheInstances;
#if defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS)
	Asura_DIPCache_Static< 50>& xDIPCache = xRenderCache.m_xDIPCache;
#endif
//	Asura_DIPCache_Static< 50> xDIPCache( DrawTreeBillboardInstancesDIPCache, 0);
	
	for( uint32 uIndexType = 0; uIndexType < uCountTypes; uIndexType += 1)
	{
		Asura_Direct3D_BGR_CacheType &xCacheType = this->m_xArrayTypeCache[ uIndexType];
		Asura_Direct3D_BGR_Type* pxTreeDescription = xCacheType.m_pxTree;
		
		if( NULL == pxTreeDescription)
		{
			continue;
		}
		
#if defined( ASURA_TOOLS)
		if( true == pxTreeDescription->m_bHidden)
		{
			continue;
		}
#endif
#if defined( ASURA_TOOLS)
//		Asura_Assertf( 0 < pxTreeDescription->m_xArray_xSections.Size(), ("Botanical '%d' does not seem to have any sections to draw!", pxTreeDescription->m_xName.GetString()));
#endif
		if( 0 == pxTreeDescription->m_xArray_xSections.Size())
		{
			continue;
		}
		
#if defined( ASURA_TOOLS)
		SCOPEDPIXEVENT((0xff00ff00, L"Botanical Draw Cache Type '%S'", pxTreeDescription->m_xName.GetString()));
#else
		SCOPEDPIXEVENT((0xff00ff00, L"Botanical Draw Cache Type"));
#endif
		
		
		const float fTreeTypeRadius = pxTreeDescription->GetBoundingRadius();

		xProcessInstanceHeader.m_fLodLimitNear = pxTreeDescription->m_fLodLimitNear;
		xProcessInstanceHeader.m_fInverseLodLimitDelta = 1.0f / pxTreeDescription->m_fLodLimitDelta;
		
#if defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS)
		xDIPCache.Clear();
#endif
		xRenderCache.Clear();
		xRenderCache.ClearInstances( pxTreeDescription);

		// DLH - Calc the scale to change the distances to account for the difference in screen res
		xProcessInstanceHeader.m_fScreenDistanceScale = 1.0f;
		
		{
			const float fPresentRes = static_cast< float>( Asura_Viewport::GetWidth());
			
			const float fScaleRes = pxTreeDescription->m_fLodAuthoredRes / fPresentRes;
			
			xProcessInstanceHeader.m_fScreenDistanceScale *= fScaleRes;
		}

		// DLH - Calc the scale to change the distance to account for the difference in FOV 
		{
			const float fAuthorDistance = pxTreeDescription->m_fLodLimitFar;
			const float fAuthorWidth = pxTreeDescription->m_fLodAuthoredFOVWidth;
			
			const float fPresentFOV = 0.5f * Asura_Camera::GetHorizontalFieldOfView();
			const float fPresentTanFOV = Asura_Maths::Tangent( fPresentFOV);
			const float fPresentWidth = fAuthorDistance * fPresentTanFOV;
			
			const float fScaleFOV = fPresentWidth / fAuthorWidth;
			
			xProcessInstanceHeader.m_fScreenDistanceScale *= fScaleFOV;
		}
					
		uint32 uCountTotalInstances = 0;
		
		Asura_LinkList< const Asura_Direct3D_BGR_Instances_Header*> &xListArrayInstanceHeader = xCacheType.m_xListRegionDescriptions;
		
		for( Asura_ListElt< const Asura_Direct3D_BGR_Instances_Header*>* xIttInstanceHeader = xListArrayInstanceHeader.Front(); NULL != xIttInstanceHeader; xIttInstanceHeader = xIttInstanceHeader->Next())
		{
			const Asura_Direct3D_BGR_Instances_Header* pxInstanceHeader = xIttInstanceHeader->Data();
			
			Asura_Assert( NULL != pxInstanceHeader, "Bad Instance Pointer");
			
			if( 0 == pxInstanceHeader->GetInstanceCount())
			{
				continue;
			}
			
			if( false == Asura_Camera::IsAABBInFrustum( pxInstanceHeader->GetBoundingBox()))
			{
				continue;
			}
			
			// DLH - 12 June 2006 12:20:42 - Find out if any of the Photon Emitters effect this InstanceHeader
			bool bInstanceHeaderContainsPhotoEmitters = false;
			if( true == bUsePhotonSources && pxPhotonList)
			{
				Asura_TrackingHandlePool<Asura_Photon>::Iterator xIterator = pxPhotonList->GetIterator();

				const Asura_Bounding_Box& xInstanceBoundingBox = pxInstanceHeader->GetBoundingBox();
					
				for (xIterator.Begin(); !xIterator.End(); xIterator.Next())
				{
					Asura_Photon* pxPhoton = xIterator.GetCurrentPointer();

					float fRange = pxPhoton->GetRange();

					if (fRange <= 0.0f || !pxPhoton->IsActive()) continue;
					
					const Asura_Vector_3& xPosition = pxPhoton->GetPosition();

					const float fRadiusMaximised = fRange + fTreeTypeRadius;
					const bool bIntersects = xInstanceBoundingBox.IntersectsSphere( xPosition, fRadiusMaximised);
					if( false == bIntersects)
					{
						continue;
					}

					// DLH - I would rather have done this in the parent routine but I do not want to generate another cached list of data!					
					const bool bPhotonInFrustrum = Asura_Camera::IsSphereInFrustum( xPosition, fRadiusMaximised);
					if( false == bPhotonInFrustrum)
					{
						continue;
					}
					
					bInstanceHeaderContainsPhotoEmitters = true;
					
					#if defined( ASURA_ASSERT)
					const bool bAddedToCache = 
					#endif
					xRenderCache.AddPhotonToDynamicLightCache( pxPhoton);
					
					Asura_Assertf( true == bAddedToCache, ("Run out of Photo caches.\nThere are %d caches for drawing dynamic lights on a type of Botanical. These have all been used up. Cut down on the number of Active Dynamic lights in this view.", xRenderCache.m_xArray_xDynamicLightCache.Size()));
				}
			}
			
#if defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS)
			// DLH - 09 June 2006 16:06:12 - Add the Imposters
			if( true == pxTreeDescription->m_bHasImposters && 0 != pxInstanceHeader->m_uImposterIBSize)
			{
#if defined( ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS) && 1 == ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS
				if( false == pxInstanceHeader->HasDynamicSectionVersionData())
#endif
				{
					const Asura_Bounding_Box& xBBInstance = pxInstanceHeader->GetBoundingBox();
					Asura_AVector3 xBBCenter( xBBInstance.CalculateCentre());
					const float fBBRadius = xBBInstance.CalculateRadius();
					Asura_AVector3 xDirectionToCamera( xProcessInstanceHeader.m_xCameraPos - xBBCenter);
					
					float fDistance = (((xProcessInstanceHeader.m_fScreenDistanceScale * xDirectionToCamera.Magnitude()) + fBBRadius) - xProcessInstanceHeader.m_fLodLimitNear) * xProcessInstanceHeader.m_fInverseLodLimitDelta;

					if( pxTreeDescription->m_fLodLimitDeltaImposterNear <= fDistance)
					{
						Asura_DIPCache_DIPData xDIPData;
						
						xDIPData.Set(
							pxInstanceHeader->m_uImposterIBStart, pxInstanceHeader->m_uImposterIBStart + pxInstanceHeader->m_uImposterIBSize,
							pxInstanceHeader->m_uImposterVBStart, pxInstanceHeader->m_uImposterVBStart + pxInstanceHeader->m_uImposterVBSize,
							pxInstanceHeader->m_uImposterVBOffset
						);
						
						xDIPCache.AddDIP( xDIPData);
						
						// DLH - 30 June 2006 11:33:01 - Add DIP ranges to each photons data
						if( true == bUsePhotonSources)
						{
							Asura_Botanical_Array< Asura_Direct3D_BGR_DynamicLightCache>& xArray_xDynamicLightCache = xRenderCache.m_xArray_xDynamicLightCache;
							const u_int uDynamicLightCacheCount = xArray_xDynamicLightCache.Size();
							for( u_int uDynamicLightCacheIndex = 0; uDynamicLightCacheIndex < uDynamicLightCacheCount; uDynamicLightCacheIndex += 1)
							{
								Asura_Direct3D_BGR_DynamicLightCache& xDynamicLightCache = xArray_xDynamicLightCache[ uDynamicLightCacheIndex];
								const Asura_Photon* pxPhoton = xDynamicLightCache.m_pxPhoton;

								if( NULL == pxPhoton)
								{
									// DLH - 30 June 2006 14:39:56 - Photons should be stored in a linear maner, so there should not be any more
									break;
								}
								
	// DLH - 27 June 2006 17:01:02 - Does not return a valid AABB
	//				const Asura_Bounding_Box& xPhotonBoundingBox = pxPhoton->GetBoundingBox();
								const Asura_Vector_3& xPhotonPosition = pxPhoton->GetPosition();
								const float fPhotonRange = pxPhoton->GetRange();
								Asura_Bounding_Box xPhotonBoundingBox;
								xPhotonBoundingBox.SetToPoint( xPhotonPosition);
								xPhotonBoundingBox.Grow( fPhotonRange + fBBRadius);
								
								if( true == xBBInstance.Intersects( xPhotonBoundingBox))
								{
									xDynamicLightCache.m_xDIPCache.AddDIP( xDIPData);
								}
							}
						}
					}
				}
			}
#endif
			
			{
				// DLH - 29 August 2006 14:42:38 - Break the list of instances into batches no bigger than the cache able size.
				const uint32 uCountTrees = pxInstanceHeader->GetInstanceCount();
				const uint32 uCapacityTrees = xCacheInstances.Capacity();
				
				u_int uRemainingTrees = uCountTrees;

				for( u_int uIndexTreeBatchStart = 0; 0 < uRemainingTrees; )
				{
					u_int uIndexTreeBatchSize = Asura_Maths::Min( uCapacityTrees, uRemainingTrees);
					Asura_Assert( uIndexTreeBatchSize <= uRemainingTrees, "Mis match in the instance batch counting");
					uRemainingTrees -= uIndexTreeBatchSize;
					
					// DLH - Do the space test here to cut down on tests. Makes the instance look tighter
					if( uCapacityTrees < (xCacheInstances.Size() + uIndexTreeBatchSize))
					{
						// DLH - Draw Cached instances
						bool bDrawPhotonInstances = bUsePhotonSources && xRenderCache.m_bDynamicLightInstancesExist;
						if( true == bDrawPhotonInstances)
						{
							this->DrawCache< true>( pxTreeDescription, xRenderCache);
						}
						else
						{
							this->DrawCache< false>( pxTreeDescription, xRenderCache);
						}
						xRenderCache.ClearInstances( pxTreeDescription);
					}
					
					const u_int uIndexTreeBatchStop = uIndexTreeBatchStart + uIndexTreeBatchSize;
					
					{
						bool bProcessPhotons = (true == bUsePhotonSources && true == bInstanceHeaderContainsPhotoEmitters);
#if defined( ASURA_BOTANICAL_SUPPORT_INTERACTIVE)
						const bool bInstanceHeaderHasInteractiveData = pxInstanceHeader->HasDynamicInteractiveData();
						if( true == bInstanceHeaderHasInteractiveData)
						{
							if( bProcessPhotons)
							{
								this->DrawCache_ProcessInstanceHeader< true, true>( xProcessInstanceHeader, pxTreeDescription, xRenderCache, pxInstanceHeader, uIndexTreeBatchStart, uIndexTreeBatchStop);
							}
							else
							{
								this->DrawCache_ProcessInstanceHeader< false, true>( xProcessInstanceHeader, pxTreeDescription, xRenderCache, pxInstanceHeader, uIndexTreeBatchStart, uIndexTreeBatchStop);
							}
						}
						else
#endif
						{
							if( bProcessPhotons)
							{
								this->DrawCache_ProcessInstanceHeader< true, false>( xProcessInstanceHeader, pxTreeDescription, xRenderCache, pxInstanceHeader, uIndexTreeBatchStart, uIndexTreeBatchStop);
							}
							else
							{
								this->DrawCache_ProcessInstanceHeader< false, false>( xProcessInstanceHeader, pxTreeDescription, xRenderCache, pxInstanceHeader, uIndexTreeBatchStart, uIndexTreeBatchStop);
							}
						}
					}
					
					uIndexTreeBatchStart = uIndexTreeBatchStop;
				}
				
				Asura_Assert( 0 == uRemainingTrees, "Tree Instances left over");
				uCountTotalInstances += uCountTrees;
                #if defined( ASURA_TOOLS)
            	TOOLS_BOTANICAL_STATISTICS_ADD( s_uInstanceCount, 1);
                #endif
			}
		}	
		
		if( 0 < xCacheInstances.Size())
		{
			// DLH - Draw Cached instances
			bool bDrawPhotonInstances = bUsePhotonSources && xRenderCache.m_bDynamicLightInstancesExist;
			if( true == bDrawPhotonInstances)
			{
				this->DrawCache< true>( pxTreeDescription, xRenderCache);
			}
			else
			{
				this->DrawCache< false>( pxTreeDescription, xRenderCache);
			}
			xRenderCache.ClearInstances( pxTreeDescription);
		}
		
		if( xCacheInstances.Capacity() < uCountTotalInstances)
		{
			xCacheInstances.SetCapacity( uCountTotalInstances);
		}
		
#if defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS)
		if( false == xDIPCache.IsEmpty())
		{
			SCOPEDPIXEVENT((0xff00ff00, L"Botanical Imposters"));
			
			const u_int uSectionCount = pxTreeDescription->m_xArray_xSections.Size();
			for( u_int uSectionIndex = 0; uSectionIndex < uSectionCount; uSectionIndex += 1)
			{
		#if defined( ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS) && 1 == ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS
				const Asura_Botanical_Array< bool>& xArrayVersion_bUsed = xRenderCache.m_xArraySection_xArrayVersion_bUsed[ uSectionIndex];
				//Asura_BGR_Section_ID xSectionID = static_cast< Asura_BGR_Section_ID>( uSectionIndex);
		#endif
				Asura_Direct3D_BGR_Section& xSection = pxTreeDescription->m_xArray_xSections[ uSectionIndex];
				
				const u_int uSectionVersionCount = xSection.m_xArray_xVersions.Size();
				for( u_int uSectionVersionIndex = 0; uSectionVersionIndex < uSectionVersionCount; uSectionVersionIndex += 1)
				{
		#if defined( ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS) && 1 == ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS
					bool bVersionUsed = xArrayVersion_bUsed[ uSectionVersionIndex];
					if( false == bVersionUsed)
					{
						//continue;
					}
					
					//Asura_BGR_Version_ID xSectionVersionID = xSection.m_xArray_uVersionIndexToVersionID[ uSectionVersionIndex];
		#endif
					Asura_Direct3D_BGR_Section_Version& xSectionVersion = xSection.m_xArray_xVersions[ uSectionVersionIndex];
					Asura_Direct3D_BGR_Geometry& xGeometry = xSectionVersion.m_xGeometry;
					
					uint32 uImposterTypeCount = xGeometry.m_xArrayBillboards.Size();
					for( uint32 uImposterTypeIndex = 0; uImposterTypeIndex < uImposterTypeCount; uImposterTypeIndex += 1)
					{
						const Asura_Direct3D_BGR_Type_GeometryBillboard& xImposterType = xGeometry.m_xArrayBillboards[ uImposterTypeIndex];
						Asura_Direct3D_Effect* pxD3DEffect = NULL;
						
						if( true == this->DrawTreeBillboardInstancesDIPCacheRenderStates_Start( *pxTreeDescription, xProcessInstanceHeader.m_fScreenDistanceScale, xImposterType, pxD3DEffect))
						{
							xDIPCache.Render();
							
							this->DrawTreeBillboardInstancesDIPCacheRenderStates_End( pxD3DEffect);
							
							if( true == bUsePhotonSources)
							{
								Asura_Botanical_Array< Asura_Direct3D_BGR_DynamicLightCache>& xArray_xDynamicLightCache = xRenderCache.m_xArray_xDynamicLightCache;
								const u_int uDynamicLightCacheCount = xArray_xDynamicLightCache.Size();
								for( u_int uDynamicLightCacheIndex = 0; uDynamicLightCacheIndex < uDynamicLightCacheCount; uDynamicLightCacheIndex += 1)
								{
									Asura_Direct3D_BGR_DynamicLightCache& xDynamicLightCache = xArray_xDynamicLightCache[ uDynamicLightCacheIndex];
									const Asura_Photon* pxPhoton = xDynamicLightCache.m_pxPhoton;

									if( NULL == pxPhoton)
									{
										// DLH - 30 June 2006 14:39:56 - Photons should be stored in a linear maner, so there should not be any more
										break;
									}
									
									if( true == this->DrawTreeBillboardDynamicRenderStates( *pxPhoton, pxD3DEffect))
									{
										xDynamicLightCache.m_xDIPCache.Render();
										
										this->DrawTreeBillboardInstancesDIPCacheRenderStates_End( pxD3DEffect);
									}
								}
							}
							
						}
					}
				}
			}
		}
#endif
	}
	
	this->ResetRenderStates();

#ifdef ASURA_TOOLS
    RenderDebugInfo();
#endif
}


#ifdef ASURA_TOOLS
void Asura_Direct3D_BGR_Manager::RenderDebugInfo( void )
{
    for( Asura_LinkList_ConstIt<InstanceBoundingSphere> xIt(&m_xRenderedInstanceBoundingSpheres); !xIt.Done(); xIt.Next() )
    {
        const InstanceBoundingSphere* pxBS = xIt.GetCurrent();

        const Asura_Vector_3 xSpheroidExtents( pxBS->fRadius, pxBS->fRadius, pxBS->fRadius );

        // Render in green if it's contained within the environment and red
        // otherwise.
        u_int uLastKnownVisNode = uASURA_VISIBILITY_INVALID_NODE_INDEX;
        if( Asura_Visibility_System::FindNodeContainingSphere(pxBS->xCentre, pxBS->fRadius, uLastKnownVisNode) )
        {
            Tools_Render::RenderSpheroid( pxBS->xCentre, Asura_Maths::IdentityMatrix3x3, xSpheroidExtents, 0xFF00FF00, true );
        }
        else
        {
            Tools_Render::RenderSpheroid( pxBS->xCentre, Asura_Maths::IdentityMatrix3x3, xSpheroidExtents, 0xFFFF0000, true );
        }
    }

    m_xRenderedInstanceBoundingSpheres.Clear();
}
#endif


void Asura_Direct3D_BGR_Manager::AdjustInstanceDataWithDynamicPhotoEmitter( const Asura_Vector_3& xInstancePosition, const Asura_Direct3D_BGR_Type& xTreeDescription, Asura_Direct3D_BGR_RenderCache& xRenderCache, Asura_Direct3D_BGR_InstanceCache& xInstanceCache)
{
	Asura_SphericalHarmonics_Order2_RGB xSHOriginal;
	Asura_SphericalHarmonics_Order2_RGB xSHDynamic;
	
	xSHDynamic.Zero();
	
#if defined( ASURA_DIRECT3D_BGR_SUPPORT_SM11) && 1 == ASURA_DIRECT3D_BGR_SUPPORT_SM11
	xSHOriginal.m_afRedCoefficients[1]	= xInstanceCache.m_xConstants.m_xData.m_afSphericalR[0];
	xSHOriginal.m_afRedCoefficients[2]	= xInstanceCache.m_xConstants.m_xData.m_afSphericalR[1];
	xSHOriginal.m_afRedCoefficients[3]	= xInstanceCache.m_xConstants.m_xData.m_afSphericalR[2];
	xSHOriginal.m_afRedCoefficients[0]	= xInstanceCache.m_xConstants.m_xData.m_afSphericalR[3];
									
	xSHOriginal.m_afGreenCoefficients[1]	= xInstanceCache.m_xConstants.m_xData.m_afSphericalG[0];
	xSHOriginal.m_afGreenCoefficients[2]	= xInstanceCache.m_xConstants.m_xData.m_afSphericalG[1];
	xSHOriginal.m_afGreenCoefficients[3]	= xInstanceCache.m_xConstants.m_xData.m_afSphericalG[2];
	xSHOriginal.m_afGreenCoefficients[0]	= xInstanceCache.m_xConstants.m_xData.m_afSphericalG[3];
									
	xSHOriginal.m_afBlueCoefficients[1]	= xInstanceCache.m_xConstants.m_xData.m_afSphericalB[0];
	xSHOriginal.m_afBlueCoefficients[2]	= xInstanceCache.m_xConstants.m_xData.m_afSphericalB[1];
	xSHOriginal.m_afBlueCoefficients[3]	= xInstanceCache.m_xConstants.m_xData.m_afSphericalB[2];
	xSHOriginal.m_afBlueCoefficients[0]	= xInstanceCache.m_xConstants.m_xData.m_afSphericalB[3];
#endif
#if defined( ASURA_DIRECT3D_BGR_SUPPORT_SM30) && 1 == ASURA_DIRECT3D_BGR_SUPPORT_SM30
	xSHOriginal.m_afRedCoefficients[1]	= xInstanceCache.m_xInstanceVertex.m_afSphericalR[0];
	xSHOriginal.m_afRedCoefficients[2]	= xInstanceCache.m_xInstanceVertex.m_afSphericalR[1];
	xSHOriginal.m_afRedCoefficients[3]	= xInstanceCache.m_xInstanceVertex.m_afSphericalR[2];
	xSHOriginal.m_afRedCoefficients[0]	= xInstanceCache.m_xInstanceVertex.m_afSphericalR[3];
									
	xSHOriginal.m_afGreenCoefficients[1]	= xInstanceCache.m_xInstanceVertex.m_afSphericalG[0];
	xSHOriginal.m_afGreenCoefficients[2]	= xInstanceCache.m_xInstanceVertex.m_afSphericalG[1];
	xSHOriginal.m_afGreenCoefficients[3]	= xInstanceCache.m_xInstanceVertex.m_afSphericalG[2];
	xSHOriginal.m_afGreenCoefficients[0]	= xInstanceCache.m_xInstanceVertex.m_afSphericalG[3];
									
	xSHOriginal.m_afBlueCoefficients[1]	= xInstanceCache.m_xInstanceVertex.m_afSphericalB[0];
	xSHOriginal.m_afBlueCoefficients[2]	= xInstanceCache.m_xInstanceVertex.m_afSphericalB[1];
	xSHOriginal.m_afBlueCoefficients[3]	= xInstanceCache.m_xInstanceVertex.m_afSphericalB[2];
	xSHOriginal.m_afBlueCoefficients[0]	= xInstanceCache.m_xInstanceVertex.m_afSphericalB[3];
#endif

	bool bAdjusted = false;
	const float fTypeBoundingRadius = xTreeDescription.GetBoundingRadius();

	Asura_Botanical_Array< Asura_Direct3D_BGR_DynamicLightCache>& xArray_xDynamicLightCache = xRenderCache.m_xArray_xDynamicLightCache;
	const u_int uDynamicLightCacheCount = xArray_xDynamicLightCache.Size();
	for( u_int uDynamicLightCacheIndex = 0; uDynamicLightCacheIndex < uDynamicLightCacheCount; uDynamicLightCacheIndex += 1)
	{
		Asura_Direct3D_BGR_DynamicLightCache& xDynamicLightCache = xArray_xDynamicLightCache[ uDynamicLightCacheIndex];
	    const Asura_Photon* pxPhoton = xDynamicLightCache.m_pxPhoton;

		if( NULL == pxPhoton)
		{
			// DLH - 30 June 2006 14:39:56 - Photons should be stored in a linear maner, so there should not be any more
			break;
		}

		const float fRange = pxPhoton->GetRange() + fTypeBoundingRadius;

		Asura_Vector_3 xDir = pxPhoton->GetPosition() - xInstancePosition;

		float fMag = xDir.Normalise();

		const float fScaledTypeDistanceMethod = 4.0f;

		if (fMag < fRange)
		{
			if( fMag < (fScaledTypeDistanceMethod * fTypeBoundingRadius))
			{
				const u_int uDynamicCacheInstanceSize = xDynamicLightCache.m_xCacheInstances.Size();
				const u_int uDynamicCacheInstanceCapacity = xDynamicLightCache.m_xCacheInstances.Capacity();
				const bool bSpaceAvalible = uDynamicCacheInstanceSize < uDynamicCacheInstanceCapacity;
				Asura_Assert( true == bSpaceAvalible, "Run out of cache storage for Dynmic Lights. Applying as Object level lighting.");
				if( true == bSpaceAvalible)
				{
					Asura_Direct3D_BGR_InstanceCache &xDynamicCacheInstance = xDynamicLightCache.m_xCacheInstances.GetNext();
					
					xDynamicCacheInstance = xInstanceCache;
					xRenderCache.m_bDynamicLightInstancesExist = true;
					
					#if defined( ASURA_TOOLS)
						TOOLS_BOTANICAL_STATISTICS_ADD( s_uDynamicLightVertexCount, 1);
					#endif
				
					continue;
				}
			}
			
			{
				const float fBrightness = 1.0f - fMag / fRange;

				const float fR = fBrightness * pxPhoton->GetRedComponent();
				const float fG = fBrightness * pxPhoton->GetGreenComponent();
				const float fB = fBrightness * pxPhoton->GetBlueComponent();
					
				xSHDynamic.AddSampleForDirection( fR, fG, fB, xDir );
				bAdjusted = true;
				#if defined( ASURA_TOOLS)
					TOOLS_BOTANICAL_STATISTICS_ADD( s_uDynamicLightObjectCount, 1);
				#endif
			}
		}
    }

	if( true == bAdjusted)
	{
		xSHDynamic.ScaleCoefficientsForIrradiance();
		
		xSHOriginal += xSHDynamic;
		
#if defined( ASURA_DIRECT3D_BGR_SUPPORT_SM11) && 1 == ASURA_DIRECT3D_BGR_SUPPORT_SM11
		xInstanceCache.m_xConstants.m_xData.m_afSphericalR[0] = xSHOriginal.m_afRedCoefficients[1];
		xInstanceCache.m_xConstants.m_xData.m_afSphericalR[1] = xSHOriginal.m_afRedCoefficients[2];
		xInstanceCache.m_xConstants.m_xData.m_afSphericalR[2] = xSHOriginal.m_afRedCoefficients[3];
		xInstanceCache.m_xConstants.m_xData.m_afSphericalR[3] = xSHOriginal.m_afRedCoefficients[0];

		xInstanceCache.m_xConstants.m_xData.m_afSphericalG[0] = xSHOriginal.m_afGreenCoefficients[1];
		xInstanceCache.m_xConstants.m_xData.m_afSphericalG[1] = xSHOriginal.m_afGreenCoefficients[2];
		xInstanceCache.m_xConstants.m_xData.m_afSphericalG[2] = xSHOriginal.m_afGreenCoefficients[3];
		xInstanceCache.m_xConstants.m_xData.m_afSphericalG[3] = xSHOriginal.m_afGreenCoefficients[0];

		xInstanceCache.m_xConstants.m_xData.m_afSphericalB[0] = xSHOriginal.m_afBlueCoefficients[1];
		xInstanceCache.m_xConstants.m_xData.m_afSphericalB[1] = xSHOriginal.m_afBlueCoefficients[2];
		xInstanceCache.m_xConstants.m_xData.m_afSphericalB[2] = xSHOriginal.m_afBlueCoefficients[3];
		xInstanceCache.m_xConstants.m_xData.m_afSphericalB[3] = xSHOriginal.m_afBlueCoefficients[0];
#endif
#if defined( ASURA_DIRECT3D_BGR_SUPPORT_SM30) && 1 == ASURA_DIRECT3D_BGR_SUPPORT_SM30
		xInstanceCache.m_xInstanceVertex.m_afSphericalR[0] = xSHOriginal.m_afRedCoefficients[1];
		xInstanceCache.m_xInstanceVertex.m_afSphericalR[1] = xSHOriginal.m_afRedCoefficients[2];
		xInstanceCache.m_xInstanceVertex.m_afSphericalR[2] = xSHOriginal.m_afRedCoefficients[3];
		xInstanceCache.m_xInstanceVertex.m_afSphericalR[3] = xSHOriginal.m_afRedCoefficients[0];

		xInstanceCache.m_xInstanceVertex.m_afSphericalG[0] = xSHOriginal.m_afGreenCoefficients[1];
		xInstanceCache.m_xInstanceVertex.m_afSphericalG[1] = xSHOriginal.m_afGreenCoefficients[2];
		xInstanceCache.m_xInstanceVertex.m_afSphericalG[2] = xSHOriginal.m_afGreenCoefficients[3];
		xInstanceCache.m_xInstanceVertex.m_afSphericalG[3] = xSHOriginal.m_afGreenCoefficients[0];

		xInstanceCache.m_xInstanceVertex.m_afSphericalB[0] = xSHOriginal.m_afBlueCoefficients[1];
		xInstanceCache.m_xInstanceVertex.m_afSphericalB[1] = xSHOriginal.m_afBlueCoefficients[2];
		xInstanceCache.m_xInstanceVertex.m_afSphericalB[2] = xSHOriginal.m_afBlueCoefficients[3];
		xInstanceCache.m_xInstanceVertex.m_afSphericalB[3] = xSHOriginal.m_afBlueCoefficients[0];
#endif	
	}
}

void Asura_Direct3D_BGR_Manager::DrawCache()
{
	SCOPEDPIXEVENT((0xff00ff00, L"Botanical DrawCache"));

#if defined( ASURA_DIRECT3D_BGR_SUPPORT_PHOTONS) && 1 == ASURA_DIRECT3D_BGR_SUPPORT_PHOTONS
	Asura_TrackingHandlePool<Asura_Photon>* pxPhotonList = Asura_Photon_System::GetDynamicPhotonList();

#if defined( ASURA_TOOLS)
	if( NULL != pxPhotonList)
	{
		Asura_TrackingHandlePool<Asura_Photon>::Iterator xIterator = pxPhotonList->GetIterator();
			
		for (xIterator.Begin(); !xIterator.End(); xIterator.Next())
		{
			Asura_Photon* pxPhoton = xIterator.GetCurrentPointer();

			if( true == pxPhoton->IsActive()) 
			{
				TOOLS_BOTANICAL_STATISTICS_ADD( s_uDynamicLightTotalCount, 1);
			}
		}
	}
#endif

	if( NULL != pxPhotonList)
	{
		// DLH - 09 June 2006 16:03:54 - Find out if there are any Dynamic Photon Emitters
		Asura_TrackingHandlePool<Asura_Photon>::Iterator xIterator = pxPhotonList->GetIterator();
		bool bFoundActiveLight = false;
			
		for (xIterator.Begin(); !xIterator.End(); xIterator.Next())
		{
			Asura_Photon* pxPhoton = xIterator.GetCurrentPointer();

			if( true == pxPhoton->IsActive()) 
			{
				bFoundActiveLight = true;
				break;
			}
		}
		
		if( false == bFoundActiveLight)
		{
			pxPhotonList = NULL;
		}
	}

	// DLH - 09 June 2006 16:08:50 - Call the optimal version of DrawCache
	if( NULL != pxPhotonList)
	{
		this->DrawCache< true>( pxPhotonList);
	}
	else
#endif // ASURA_DIRECT3D_BGR_SUPPORT_PHOTONS
	{
		this->DrawCache< false>( NULL);
	}
}

//-------------------------------------------------------------------
// Function:    Asura_Direct3D_BGR_Manager::SetupSectionCache
//
// Description: This Routine reserves enough space for every type of botanical to cache which section versions are used. 
//-------------------------------------------------------------------
#if defined( ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS) && 1 == ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS
void Asura_Direct3D_BGR_Manager::SetupSectionCache( void)
{
	Asura_Botanical_Array< Asura_Direct3D_BGR_Type> &xTypes = this->m_xArray_xTypes;
	
	const uint32 uTypeCount = xTypes.Size();
	
	// DLH - 27 November 2006 16:43:52 - Find the Range of Sections
	u_int uSectionCountMax = 0;
	
	for( uint32 uTypeIndex = 0; uTypeIndex < uTypeCount; uTypeIndex += 1)
	{
		Asura_Direct3D_BGR_Type& xBotanicalTree = xTypes[ uTypeIndex];

		// DLH - Tree must have a valid idValue
		const uint32 uIdTree = xBotanicalTree.GetTreeId();
		if( uIdTree == Asura_BGR_TypeID::INVALID_ID)
		{
			continue;
		}
		Asura_Assert( Asura_BGR_TypeID::INVALID_ID != uIdTree, "Bad ID");			
		
		const u_int uSectionCount = xBotanicalTree.m_xArray_xSections.Size();
		uSectionCountMax = Asura_Maths::Max( uSectionCountMax, uSectionCount);
	}
	
	// DLH - 27 November 2006 17:27:22 - Create and Initialise the Section's Version Count
	this->m_xArraySection_xVersionCountMax.Clear();
	this->m_xArraySection_xVersionCountMax.SetSize( uSectionCountMax);

	for( u_int uSectionIndex = 0; uSectionIndex < uSectionCountMax; uSectionIndex += 1)
	{	
		this->m_xArraySection_xVersionCountMax[ uSectionIndex] = 0;
	}

	for( uint32 uTypeIndex = 0; uTypeIndex < uTypeCount; uTypeIndex += 1)
	{
		Asura_Direct3D_BGR_Type& xBotanicalTree = xTypes[ uTypeIndex];

		// DLH - Tree must have a valid idValue
		const uint32 uIdTree = xBotanicalTree.GetTreeId();
		if( uIdTree == Asura_BGR_TypeID::INVALID_ID)
		{
			continue;
		}
		Asura_Assert( Asura_BGR_TypeID::INVALID_ID != uIdTree, "Bad ID");
		
		const u_int uSectionCount = xBotanicalTree.m_xArray_xSections.Size();
		for( u_int uSectionIndex = 0; uSectionIndex < uSectionCount; uSectionIndex += 1)
		{
			const Asura_Direct3D_BGR_Section &xSection = xBotanicalTree.m_xArray_xSections[ uSectionIndex];
			u_int& uVersionCountMax = this->m_xArraySection_xVersionCountMax[ uSectionIndex];
			
			const u_int uVersionCount = xSection.m_xArray_xVersions.Size();
			
			uVersionCountMax = Asura_Maths::Max( uVersionCount, uVersionCountMax);
		}
	}
	
}
#endif
	
//-------------------------------------------------------------------
// Function:    Asura_Direct3D_BGR_Manager::SetupTypeCache
//
// Description: This Routine creates a cached list of types. 
//				It should only need doing after the type list has changed
//-------------------------------------------------------------------
void Asura_Direct3D_BGR_Manager::SetupTypeCache( void)
{
	Asura_Botanical_Array< Asura_Direct3D_BGR_Type> &xTypes = this->m_xArray_xTypes;
	
	const uint32 uTypeCount = xTypes.Size();
	uint32 uCountTrees = 0;
	
	// DLH - Find the range of Trees that exist
	{
		for( uint32 uTypeIndex = 0; uTypeIndex < uTypeCount; uTypeIndex += 1)
		{
			Asura_Direct3D_BGR_Type& xBotanicalTree = xTypes[ uTypeIndex];

			// DLH - Tree must have a valid idValue
			const uint32 uIdTree = xBotanicalTree.GetTreeId();
			if( uIdTree == Asura_BGR_TypeID::INVALID_ID)
			{
				continue;
			}
			Asura_Assert( Asura_BGR_TypeID::INVALID_ID != uIdTree, "Bad ID");			
			uCountTrees = Asura_Maths::Max( uCountTrees, uIdTree);
		}
	}
	
	if( 0 == uCountTrees)
	{
//		return;
	}
	
	const uint32 uNewCacheSize = 1 + uCountTrees;
	if( this->m_xArrayTypeCache.Size() < uNewCacheSize)
	{
		this->m_xArrayTypeCache.Clear();
		this->m_xArrayTypeCache.SetSize( uNewCacheSize);
	}

	// DLH - Cache the list of trees 	
	{
		for( uint32 uTypeIndex = 0; uTypeIndex < uTypeCount; uTypeIndex += 1)
		{
			Asura_Direct3D_BGR_Type& xBotanicalTree = xTypes[ uTypeIndex];
			const uint32 uIdTree = xBotanicalTree.GetTreeId();
			if( uIdTree == Asura_BGR_TypeID::INVALID_ID)
			{
				continue;
			}

			// DLH - Tree must have a valid idValue
			//Asura_Assert( Asura_BGR_TypeID::INVALID_ID != uIdTree, "Bad ID");	
			Asura_Assert( uIdTree < this->m_xArrayTypeCache.Size(), "ID out of Range");	
			
			Asura_Direct3D_BGR_CacheType& xCacheType = this->m_xArrayTypeCache[ uIdTree];
			
			xCacheType.m_pxTree = &xBotanicalTree;
			xCacheType.m_xListRegionDescriptions.Clear();
		}
	}
}

void Asura_Direct3D_BGR_Manager::ClearInstanceFromTypeCache( void)
{
	const uint32 uTypeCount = this->m_xArrayTypeCache.Size();
	for( uint32 uTypeIndex = 0; uTypeIndex < uTypeCount; uTypeIndex += 1)
	{
		Asura_Direct3D_BGR_CacheType& xCacheType = this->m_xArrayTypeCache[ uTypeIndex];
		
		xCacheType.m_xListRegionDescriptions.Clear();
	}
}

void Asura_Direct3D_BGR_Manager::AddToCache( const Asura_Botanical_Array< Asura_Direct3D_BGR_Instances_Header*>& xArray_pxInstanceHeader)
{
	const uint32 uInstanceHeaderCount = xArray_pxInstanceHeader.Size();
	for( uint32 uInstanceHeaderIndex = 0; uInstanceHeaderIndex < uInstanceHeaderCount; uInstanceHeaderIndex += 1)
	{
		const Asura_Direct3D_BGR_Instances_Header* pxInstanceHeader = xArray_pxInstanceHeader[ uInstanceHeaderIndex];
		if(NULL == pxInstanceHeader)
		{
			continue;
		}
		const uint32 uInstanceHeaderInstanceCount = pxInstanceHeader->GetInstanceCount();
		if( 0 == uInstanceHeaderInstanceCount)
		{
			continue;
		}
		
		const uint32 uIdTree = pxInstanceHeader->GetType().GetID();
		if( Asura_BGR_TypeID::INVALID_ID == uIdTree)
		{
			continue;
		}
		
		Asura_Assert( uIdTree < this->m_xArrayTypeCache.Size(), "ID out of Range");
		Asura_Direct3D_BGR_CacheType& xCacheType = this->m_xArrayTypeCache[ uIdTree];
		if( NULL == xCacheType.m_pxTree)
		{
			continue;
		}
		
		Asura_Assert( NULL != xCacheType.m_pxTree && uIdTree == xCacheType.m_pxTree->GetTreeId(), "Pointer or ID not Setup!");
		
		xCacheType.m_xListRegionDescriptions.Push_Back( pxInstanceHeader);
	}
}

//*******************************************************************
//
// (c) 2006 Rebellion (Derby)
//
// Warning: This computer program is protected by copyright law and 
// international treaties. Unauthorized reproduction or distribution
// of this program, or any portion of it, may result in severe civil and
// criminal penalties, and will be prosecuted to the maximum extent 
// possible under the law.
//
//*******************************************************************

#endif //defined( ASURA_USE_BOTANICALS)
