//*******************************************************************
//
// File: Asura_Direct3D_Botanicals_Game_Render_Manager_Interactive.cpp
//
// Created: 13 Nov 2006
//
// Description: 
//
//*******************************************************************

#if 0

// system libraries
#include "Core/Asura.h"

#if defined( ASURA_BOTANICAL_SUPPORT_INTERACTIVE)

#if !defined( ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA) || 0 == ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA
	#error If ASURA_BOTANICAL_SUPPORT_INTERACTIVE is defined then you will also need ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA
#endif

// main header
#include "Asura_Direct3D_Botanicals_Game_Render_Manager.h"

// other libraries
#include "Affectors/Asura_Affector.h"
#include "Core/Asura_Mempool.h"

// internal dependencies

//*******************************************************************
// Constants
//*******************************************************************


//*******************************************************************
// Local Classes
//*******************************************************************

class Asura_Direct3D_BGR_Manager_Update_InstanceHeader
{
public:
	bool ProcessInstanceHeader();
	
private:
	template< bool bUseAffectors, bool bTypeHasImposters> 
	bool ProcessInstanceHeaderInstances();
	
public:
	float m_fTime;
	Asura_Direct3D_BGR_Instances_Header* m_pxInstanceHeader;
	const Asura_BGR_Type_BasicInteractive* m_pxBasicInteractive;
	const Asura_Direct3D_BGR_Type* m_pxType;
#if defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS_INTERACTIVE)
	Asura_Direct3D_BGR_Type_VertexGeometryImposterInteractive* m_pxD3DBoneVertex0;
#endif
#if defined( ASURA_USE_AFFECTORS) && 1 == ASURA_USE_AFFECTORS
	bool m_bDoTheUpdateAffector;
#endif

private:
	static const u_int m_uAffectorArraySizeMax = 16;
	Asura_Affector* m_apxAffectors[ m_uAffectorArraySizeMax];
	
	u_int m_uAffectorArraySize;
};


//*******************************************************************
// Function Definitions
//*******************************************************************

template< bool bUseAffectors, bool bTypeHasImposters>
bool Asura_Direct3D_BGR_Manager_Update_InstanceHeader::ProcessInstanceHeaderInstances()
{
#if defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS_INTERACTIVE)
	Asura_Direct3D_BGR_Type_VertexGeometryImposterInteractive* pxD3DBoneVertex = NULL;
	if( true == bTypeHasImposters)
	{
		pxD3DBoneVertex = this->m_pxD3DBoneVertex0 + (this->m_pxInstanceHeader->m_uImposterVBStart);
	}
#endif
	
	const float fTime = this->m_fTime;
	const float fOneOverTime = 1.0f/ fTime;	
	const Asura_AVector3 xOneOverTime( fOneOverTime, fOneOverTime, fOneOverTime);
	const Asura_AVector3 xTime( fTime, fTime, fTime);
	Asura_AVector3 xTimeTimeOver2 = xTime;
	xTimeTimeOver2 *= xTime;
	xTimeTimeOver2 *= 0.5f;

	const float fOneOverMass = 1.0f / this->m_pxBasicInteractive->m_fMass;
	const Asura_AVector3 xOneOverMass( fOneOverMass, fOneOverMass, fOneOverMass);
	
	// DLH - 14 November 2006 13:30:00 - The minus is to reverse the vector direction from the bone position offset
	const float fSpringStrength = -0.5f * this->m_pxBasicInteractive->m_fSpringStrength;
	const Asura_AVector3 xSpringStrength( fSpringStrength, fSpringStrength, fSpringStrength);
	
	const float fDrag = -this->m_pxBasicInteractive->m_fDrag;
	const Asura_AVector3 xDrag( fDrag, fDrag, fDrag);
	
	const float fBoneLength = this->m_pxBasicInteractive->m_fBoneLength;
	const Asura_AVector3 xBoneLength( fBoneLength, fBoneLength, fBoneLength);
	
	const float fSpringLength = this->m_pxBasicInteractive->m_fSpringLength;
	
	const float fTypeBoundingRadius = this->m_pxType->m_fBoundingRadius;
	
	const Asura_AVector3 xBonePosition1 = this->m_pxBasicInteractive->m_xBonePosition1;

	const u_int uInstanceCount = this->m_pxInstanceHeader->GetInstanceCount();
	for( u_int uInstanceIndex = 0; uInstanceIndex < uInstanceCount; uInstanceIndex += 1)
	{
		const Asura_Direct3D_BGR_Instance* pxInstance = this->m_pxInstanceHeader->GetInstance( uInstanceIndex);
		
#if defined( ASURA_TOOLS)
		if( false == pxInstance->IsActive())
		{
			continue;
		}
#endif

		Asura_AVector3 xInstancePosition;
		
		pxInstance->GetInstancePosition( xInstancePosition);
		
		Asura_BGR_DynamicInstance* const pxDynamicInstance = this->m_pxInstanceHeader->GetDynamicInstanceData( uInstanceIndex);
		Asura_BGR_InteractiveBasic* const pxInteractiveBasic = this->m_pxInstanceHeader->GetDynamicInteractiveData( pxDynamicInstance);
		
		Asura_AVector3 xWorldForce;
		
		xWorldForce.Zero();
		
#if defined( ASURA_USE_AFFECTORS) && 1 == ASURA_USE_AFFECTORS
		if( bUseAffectors)
		{
			for( u_int uAffectorIndex = 0; uAffectorIndex < this->m_uAffectorArraySize; uAffectorIndex += 1)
			{
				Asura_Affector* pxAffector = this->m_apxAffectors[ uAffectorIndex];
				
				const Asura_AVector3 xAffectorPosition = pxAffector->GetPositionA();
				const Asura_AVector3 xAffectorToInstance = xInstancePosition - xAffectorPosition;
				Asura_AVector3 xAffectorForce = xAffectorToInstance;
				const float fAffectorToInstanceDistance = xAffectorForce.NormaliseReturningMagnitude();
				const float fAffectorRange = pxAffector->GetRange();
				
				const float fAffectorToInstanceMaximumRange = fAffectorRange + fTypeBoundingRadius;
				if( fAffectorToInstanceMaximumRange < fAffectorToInstanceDistance)
				{
					continue;
				}
				const float fAffectorDistanceStrength = 1 - (fAffectorToInstanceDistance / fAffectorRange);
				const float fAffectorDistanceStrengthScale = Asura_Maths::Max( 0.0f, fAffectorDistanceStrength);
				
				xAffectorForce *= fAffectorDistanceStrengthScale * pxAffector->GetStrength();
				
				xWorldForce += xAffectorForce;
			}
		}
#endif
		Asura_AMatrix3x3 xInstanceOrientation;
		
		pxInstance->GetOrientationTransposed( xInstanceOrientation);

		Asura_AVector3 xInstanceForces = xWorldForce * xInstanceOrientation;
		
		Asura_AVector3 xOldInstanceBoneOffset;
		xOldInstanceBoneOffset.LoadUnaligned( pxInteractiveBasic->m_afPosition);
		
		{
			// DLH - 14 November 2006 13:30:34 - Force = 0.5 * (k*x*x)
			Asura_AVector3 xSpringForce = xOldInstanceBoneOffset;
			
			float fSpringLength = xSpringForce.NormaliseReturningMagnitude();
			
			xSpringForce *= fSpringLength * fSpringLength;
			
			xSpringForce *= xSpringStrength;
			
			xInstanceForces += xSpringForce;
		}
		
		Asura_AVector3 xOldVelocity;
		xOldVelocity.LoadUnaligned( pxInteractiveBasic->m_afVelocity);
		
		// DLH - 14 November 2006 14:16:18 - Drag Force
		Asura_AVector3 xInstanceDragForce = xDrag;
		xInstanceDragForce *= xOldVelocity;
		
		xInstanceForces += xInstanceDragForce;

		// DLH - 14 November 2006 12:56:35 - Force = mass * acceleration
		Asura_AVector3 xInstanceAccelaration = xInstanceForces;
		xInstanceAccelaration *= xOneOverMass;
		
		// DLH - 14 November 2006 12:59:05 - x = (u*t) + (0.5*a*t*t)
		Asura_AVector3 xNewInstanceBoneOffset = xOldVelocity;
		
		xNewInstanceBoneOffset *= xTime;
		
		Asura_AVector3 xInstancePositionAcceleration = xInstanceAccelaration;
		xInstancePositionAcceleration *= xTimeTimeOver2;
		
		xNewInstanceBoneOffset += xInstancePositionAcceleration;
		xNewInstanceBoneOffset += xOldInstanceBoneOffset;
		
		{
			// DLH - 14 November 2006 16:50:02 - Move the new bone position to the end of the pole it is ment to be on
			Asura_AVector3 xLooseInstanceBonePosition = xNewInstanceBoneOffset + xBonePosition1;
			
			xLooseInstanceBonePosition.Normalise();
			xLooseInstanceBonePosition *= xBoneLength;
			
			xNewInstanceBoneOffset = xLooseInstanceBonePosition - xBonePosition1;
		}
		
		{
			// DLH - 14 November 2006 16:50:02 - Move the new bone position to the end of the pole it is ment to be on
			Asura_AVector3 xLooseInstanceBoneOffset = xNewInstanceBoneOffset;
			
			const float fOffsetLength = xLooseInstanceBoneOffset.NormaliseReturningMagnitude();
			if( fSpringLength < fOffsetLength)
			{
				xLooseInstanceBoneOffset *= fSpringLength;
				xNewInstanceBoneOffset = xLooseInstanceBoneOffset;
			}
		}
		
//			Asura_Assert( xNewInstanceBoneOffset.Magnitude() < 10000.0f, "The new bone offset position seems to be really large");
		xNewInstanceBoneOffset.StoreUnaligned( pxInteractiveBasic->m_afPosition);
		
		{
			// DLH - 16 November 2006 16:24:08 - Calculate the Velocity
			const Asura_AVector3 xInstanceMoved = xNewInstanceBoneOffset - xOldInstanceBoneOffset;
			Asura_AVector3 xNewInstanceVelocity = xInstanceMoved;
			xNewInstanceVelocity *= xOneOverTime;
			
			xNewInstanceVelocity.StoreUnaligned( pxInteractiveBasic->m_afVelocity);
		}
		
#if defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS_INTERACTIVE)
		if( true == bTypeHasImposters)
		{
			Asura_AMatrix3x3 xInstanceOrientationLocalToWorld;
			
			pxInstance->GetOrientation( xInstanceOrientationLocalToWorld);

			Asura_AVector3 xNewWorldInstanceBoneOffset = xNewInstanceBoneOffset * xInstanceOrientationLocalToWorld;
			
			__m128 xPositionSSE = xNewWorldInstanceBoneOffset.GetM128();
			
			_mm_storeu_ps( (pxD3DBoneVertex + 0)->m_afBonePosition, xPositionSSE);
			_mm_storeu_ps( (pxD3DBoneVertex + 1)->m_afBonePosition, xPositionSSE);
			_mm_storeu_ps( (pxD3DBoneVertex + 2)->m_afBonePosition, xPositionSSE);
			_mm_storeu_ps( (pxD3DBoneVertex + 3)->m_afBonePosition, xPositionSSE);
			pxD3DBoneVertex += 4;
		}
#endif
	}
	
	return( false);
}

bool Asura_Direct3D_BGR_Manager_Update_InstanceHeader::ProcessInstanceHeader()
{
#if defined( ASURA_USE_AFFECTORS) && 1 == ASURA_USE_AFFECTORS
	this->m_uAffectorArraySize = NULL;
	if( true == this->m_bDoTheUpdateAffector)
	{
		const Asura_Bounding_Box& xInstanceHeaderBoundingBox = this->m_pxInstanceHeader->m_xBoundingBox;
		
		Asura_Affector_System::MakeAffectorListForBoundingBox( xInstanceHeaderBoundingBox, this->m_uAffectorArraySizeMax, this->m_apxAffectors, this->m_uAffectorArraySize);
	}
#endif

#if defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS_INTERACTIVE)
	const bool bInstanceHasImposters = (true == this->m_pxType->m_bHasImposters && NULL != this->m_pxD3DBoneVertex0);
#endif
	
	const u_int uCodePathMaskAffectors = 0x01;
	const u_int uCodePathMaskImposters = 0x02;
	u_int uCodePath = 0;
#if defined( ASURA_USE_AFFECTORS) && 1 == ASURA_USE_AFFECTORS
	uCodePath |= (0 != m_bDoTheUpdateAffector) ? uCodePathMaskAffectors : 0;
#endif
#if defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS_INTERACTIVE)
	uCodePath |= (true == bInstanceHasImposters) ? uCodePathMaskImposters : 0;
#endif
	switch( uCodePath)
	{		
		case 0                                                     : this->ProcessInstanceHeaderInstances< false, false>(); break;
		case uCodePathMaskAffectors /*|   uCodePathMaskImposters*/ : this->ProcessInstanceHeaderInstances< true, false>(); break;
		case /*uCodePathMaskAffectors |*/ uCodePathMaskImposters   : this->ProcessInstanceHeaderInstances< false, true>(); break;
		case uCodePathMaskAffectors   |   uCodePathMaskImposters   : this->ProcessInstanceHeaderInstances< true, true>(); break;
//		default: Asura_Assert( false, "Unknown combination"); break;
	}
		
	return( true);
}

bool Asura_Direct3D_BGR_Manager::Update( const float fTime)
{
	// DLH - 14 November 2006 12:49:12 - Have to do a lot of initalise the variable and then modify it afterwards so the operator*= is used to multiply as the operator* is a dot product
#if defined( ASURA_USE_AFFECTORS) && 1 == ASURA_USE_AFFECTORS
	bool bDoTheUpdateAffector = false;
	
	{
		// DLH - 14 November 2006 10:48:09 - Check for any active affectors
		Asura_TrackingHandlePool<Asura_Affector>* pxList_xAffectors = Asura_Affector_System::GetDynamicAffectorList();
		if( NULL != pxList_xAffectors)
		{
			Asura_TrackingHandlePool< Asura_Affector>::Iterator xIteratorAffector = pxList_xAffectors->GetIterator();
			for ( xIteratorAffector.Begin(); !xIteratorAffector.End(); xIteratorAffector.Next())
			{
				Asura_Affector* pxAffector = xIteratorAffector.GetCurrentPointer();

				if( false == pxAffector->IsActive())
				{
					continue;
				}
				
				bDoTheUpdateAffector = true;
				break;
			}
		}
	}
#endif

	const u_int uInstanceHeaderCount = this->m_xArray_pxInstanceHeaders.Size();
	for( u_int uInstanceHeaderIndex = 0; uInstanceHeaderIndex < uInstanceHeaderCount; uInstanceHeaderIndex += 1)
	{
		Asura_Direct3D_BGR_Instances_Header* pxInstanceHeader = this->m_xArray_pxInstanceHeaders[ uInstanceHeaderIndex];
		
		const bool bHasInteractiveData = pxInstanceHeader->HasDynamicInteractiveData();
		if( false == bHasInteractiveData)
		{
			continue;
		}
		
		Asura_Direct3D_BGR_Type* pxType = this->GetType( pxInstanceHeader->m_xTypeID);
		if( NULL == pxType)
		{
			continue;
		}
		
		const Asura_BGR_Type_BasicInteractive* pxBasicInteractive = pxType->m_pxBasicInteractive;
		if( NULL == pxBasicInteractive)
		{
			continue;
		}

#if defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS_INTERACTIVE)
		Asura_Direct3D_BGR_Type_VertexGeometryImposterInteractive* pxD3DBoneVertex0 = this->m_xD3DBufferImposterInteractive.m_pxArrayVertex;
		
		if( NULL == pxD3DBoneVertex0)
		{		
			const bool bInstanceHasImposters = pxType->m_bHasImposters;

			if( true == bInstanceHasImposters) 
			{
				this->m_xD3DBufferImposterInteractive.Lock( true);
				pxD3DBoneVertex0 = this->m_xD3DBufferImposterInteractive.m_pxArrayVertex;
				//Asura_Assert( NULL != pxD3DBoneVertex0, "Vertex Buffer has not locked.");
			}
		}
#endif
		
		Asura_Direct3D_BGR_Manager_Update_InstanceHeader xInstanceHeaderUpdate;
		
		xInstanceHeaderUpdate.m_pxInstanceHeader = pxInstanceHeader;
		xInstanceHeaderUpdate.m_pxBasicInteractive = pxBasicInteractive;
		xInstanceHeaderUpdate.m_pxType = pxType;
		xInstanceHeaderUpdate.m_fTime = fTime;
#if defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS_INTERACTIVE)
		xInstanceHeaderUpdate.m_pxD3DBoneVertex0 = pxD3DBoneVertex0;
#endif
#if defined( ASURA_USE_AFFECTORS) && 1 == ASURA_USE_AFFECTORS
		xInstanceHeaderUpdate.m_bDoTheUpdateAffector = bDoTheUpdateAffector;
#endif
		
		xInstanceHeaderUpdate.ProcessInstanceHeader( );
	}
	
#if defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS_INTERACTIVE)
	if( NULL != this->m_xD3DBufferImposterInteractive.m_pxArrayVertex) 
	{
		this->m_xD3DBufferImposterInteractive.Unlock();
		Asura_Assert( NULL == this->m_xD3DBufferImposterInteractive.m_pxArrayVertex, "Vertex Buffer has bot unlocked.");
	}
#endif

	return( true);
}
#endif

//*******************************************************************
//
// (c) 2006 Rebellion
//
// Warning: This computer program is protected by copyright law and 
// international treaties. Unauthorized reproduction or distribution
// of this program, or any portion of it, may result in severe civil and
// criminal penalties, and will be prosecuted to the maximum extent 
// possible under the law.
//
//*******************************************************************

#endif	
