//*******************************************************************
//
// File: Asura_Direct3D_Botanicals_Game_Render_Shaders.cpp
//
// Created: 23 Oct 2006
//
// Description: 
//
//*******************************************************************

// system libraries
#include "Core/Asura.h"

#if defined( ASURA_USE_BOTANICALS)

// main header
#include "Botanical/Asura_Direct3D_Botanicals_Game_Render.h"

// other libraries
#include "Botanical/Asura_Direct3D_Botanicals_Game_Render_Manager.h"

#if defined( ASURA_DIRECT3D_BGR_SUPPORT_SM30) && 1 == ASURA_DIRECT3D_BGR_SUPPORT_SM30

#if !defined( ASURA_USE_MATERIAL_RESPONSE_SYSTEM)
	#error ASURA_DIRECT3D_BGR_SUPPORT_SM30 and ASURA_USE_MATERIAL_RESPONSE_SYSTEM must be defined
#endif

#if defined( ASURA_DIRECT3D_BGR_SUPPORT_SM11)
	#error ASURA_DIRECT3D_BGR_SUPPORT_SM30 and ASURA_DIRECT3D_BGR_SUPPORT_SM11 cannot be used together
#endif

// internal dependencies
#include "Camera/Asura_Camera.h"

#include "Core/Asura_Maths.h"
#include "Core/Asura_MaterialResponse_System.h"

#include "Rendering/Asura_Direct3d.h"
#include "Rendering/Asura_DIPCache.h"
#include "Rendering/Asura_Direct3D_Shaders.h"
#include "Rendering/Asura_Direct3D_Effect.h"
#include "Rendering/Asura_Direct3D_Effects_System.h"
#include "Rendering/Asura_Fog.h"
#include "Rendering/Asura_Render.h"

#if defined( ASURA_TOOLS)
#include "Botanicals/Tools_Botanicals_Statistics.h"
#include "Misc/Tools_Render.h"
#include "Misc/Tools_Render_Options.h"
#endif

//*******************************************************************
// Constants
//*******************************************************************

#if !defined( ASURA_DIRECT3D_BGR_SUPPORT_NORMALS)
	#error I want ASURA_DIRECT3D_BGR_SUPPORT_NORMALS turned on
#endif


// DLH - Use this to turn off all the D3D calls to measure just calculation times
#define ASURA_DIRECT3D_BGR_ENABLE_D3D_CALLS

//ASURA_STATIC_ASSERT( ASURA_DIRECT3D_BGR_VS_CONSTANT_GLOBAL_SIZE <= ASURA_DIRECT3D_BGR_VS_CONSTANT_INSTANCE0);

static Direct3D_VertexElement g_xaVSD_Botanical_Geometry3D[] =
{
	{ 0,  0, D3DDECLTYPE_FLOAT3,   D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_POSITION, 0 },
	{ 0, 12, D3DDECLTYPE_FLOAT3,   D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_NORMAL, 0 },
	{ 0, 24, D3DDECLTYPE_FLOAT4,   D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_TANGENT, 0 },
	{ 0, 40, D3DDECLTYPE_FLOAT4,   D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_BINORMAL, 0 },
	{ 0, 56, D3DDECLTYPE_FLOAT4,   D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_TEXCOORD, 0 },
	{ 0, 72, D3DDECLTYPE_D3DCOLOR, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_COLOR,    0 },
	
	{ 1,  0, D3DDECLTYPE_FLOAT4,   D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_POSITION, 1 },
	{ 1, 16, D3DDECLTYPE_FLOAT4,   D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_POSITION, 2 },
	{ 1, 32, D3DDECLTYPE_FLOAT4,   D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_POSITION, 3 },
	{ 1, 48, D3DDECLTYPE_FLOAT4,   D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_COLOR,    1 },
	{ 1, 64, D3DDECLTYPE_FLOAT4,   D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_COLOR,    2 },
	{ 1, 80, D3DDECLTYPE_FLOAT4,   D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_COLOR,    3 },
	{ 1, 96, D3DDECLTYPE_FLOAT4,   D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_POSITION, 4 },
	
	D3DDECL_END()
};

static Direct3D_VertexElement g_xaVSD_Botanical_GeometryLeaf[] =
{
	{ 0,  0, D3DDECLTYPE_FLOAT3,   D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_POSITION, 0 },
	{ 0, 12, D3DDECLTYPE_FLOAT3,   D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_NORMAL, 0 },
	{ 0, 24, D3DDECLTYPE_FLOAT4,   D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_TANGENT, 0 },
	{ 0, 40, D3DDECLTYPE_FLOAT4,   D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_BINORMAL, 0 },
	{ 0, 56, D3DDECLTYPE_FLOAT4,   D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_TEXCOORD, 0 },
	{ 0, 72, D3DDECLTYPE_D3DCOLOR, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_COLOR,    0 },
	{ 0, 76, D3DDECLTYPE_FLOAT2,   D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_TEXCOORD, 1 },
	
	{ 1,  0, D3DDECLTYPE_FLOAT4,   D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_POSITION, 1 },
	{ 1, 16, D3DDECLTYPE_FLOAT4,   D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_POSITION, 2 },
	{ 1, 32, D3DDECLTYPE_FLOAT4,   D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_POSITION, 3 },
	{ 1, 48, D3DDECLTYPE_FLOAT4,   D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_COLOR,    1 },
	{ 1, 64, D3DDECLTYPE_FLOAT4,   D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_COLOR,    2 },
	{ 1, 80, D3DDECLTYPE_FLOAT4,   D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_COLOR,    3 },
	{ 1, 96, D3DDECLTYPE_FLOAT4,   D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_POSITION, 4 },
	
	D3DDECL_END()
};

static Direct3D_VertexElement g_xaVSD_Botanical_GeometryImposter[] =
{
	{ 0,  0, D3DDECLTYPE_FLOAT3, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_POSITION, 0 },
	{ 0, 12, D3DDECLTYPE_FLOAT3, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_NORMAL,   0 },
	{ 0, 24, D3DDECLTYPE_FLOAT4, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_COLOR,    0 },
	{ 0, 40, D3DDECLTYPE_FLOAT4, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_COLOR,    1 },
	{ 0, 56, D3DDECLTYPE_FLOAT4, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_COLOR,    2 },
	{ 0, 72, D3DDECLTYPE_UBYTE4, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_COLOR,    3 },

	D3DDECL_END()
};

static Direct3D_VertexElement g_xaVSD_Botanical_Interactive_GeometryImposter[] =
{
	{ 0,  0, D3DDECLTYPE_FLOAT3, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_POSITION, 0 },
	{ 0, 12, D3DDECLTYPE_FLOAT3, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_NORMAL,   0 },
	{ 0, 24, D3DDECLTYPE_FLOAT4, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_COLOR,    0 },
	{ 0, 40, D3DDECLTYPE_FLOAT4, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_COLOR,    1 },
	{ 0, 56, D3DDECLTYPE_FLOAT4, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_COLOR,    2 },
	{ 0, 72, D3DDECLTYPE_UBYTE4, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_COLOR,    3 },

	// DLH - 21 November 2006 11:33:35 - This only needs to be 3 floats but to allow the processor to do less work by using the SSE memory instructions it has been set to 4 floats
	{ 1,  0, D3DDECLTYPE_FLOAT4, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_POSITION, 1 },
	D3DDECL_END()
};

//*******************************************************************
// Local Classes
//*******************************************************************


//*******************************************************************
// Function Definitions
//*******************************************************************

bool Asura_Direct3D_BGR_Manager::ClearDirect3DData_SM30()
{
	SAFE_RELEASE( this->m_pxVertexDeclaration_3DGeom);
	SAFE_RELEASE( this->m_pxVertexDeclaration_Interactive_Imposter);
	SAFE_RELEASE( this->m_pxVertexDeclaration_Imposter);
	SAFE_RELEASE( this->m_pxVertexDeclaration_Leaf);
	
	return( true);
}

bool Asura_Direct3D_BGR_Manager::CreateDirect3DData_SM30()
{
	HRESULT xResult;
	
	Asura_Assert( NULL == this->m_pxVertexDeclaration_3DGeom, "Vertex Decl already setup");
	xResult = Asura_Direct3D::D3DDevice->CreateVertexDeclaration( g_xaVSD_Botanical_Geometry3D, &(this->m_pxVertexDeclaration_3DGeom) );
	ASURA_D3D_OK( xResult);
	
	Asura_Assert( NULL == this->m_pxVertexDeclaration_Leaf, "Vertex Decl already setup");
	xResult = Asura_Direct3D::D3DDevice->CreateVertexDeclaration( g_xaVSD_Botanical_GeometryLeaf, &(this->m_pxVertexDeclaration_Leaf) );
	ASURA_D3D_OK( xResult);
	
	Asura_Assert( NULL == this->m_pxVertexDeclaration_Imposter, "Vertex Decl already setup");
	xResult = Asura_Direct3D::D3DDevice->CreateVertexDeclaration( g_xaVSD_Botanical_GeometryImposter, &(this->m_pxVertexDeclaration_Imposter) );
	ASURA_D3D_OK( xResult);
	
	Asura_Assert( NULL == this->m_pxVertexDeclaration_Interactive_Imposter, "Vertex Decl already setup");
	xResult = Asura_Direct3D::D3DDevice->CreateVertexDeclaration( g_xaVSD_Botanical_Interactive_GeometryImposter, &(this->m_pxVertexDeclaration_Interactive_Imposter) );
	ASURA_D3D_OK( xResult);
	
	
	return( true);
}

static void Asura_Direct3D_BGR_Manager_DrawTree_SM30_EffectSetupCamera( Asura_Direct3D_Effect* pxD3DEffect)
{
	if( NULL == pxD3DEffect)
	{
		return;
	}
	
	Asura_Vector_3 xCameraUp;
	Asura_Vector_3 xCameraRight;
	Asura_Vector_3 xCameraForwards;
	
	Asura_Camera::GetUpVector( xCameraUp);
	Asura_Camera::GetRightVector( xCameraRight);
	Asura_Camera::GetForwardVector( xCameraForwards);
	
	xCameraUp.Normalise();
	xCameraRight.Normalise();
	xCameraForwards.Normalise();
	
	D3DXVECTOR4 xD3DCameraRight( xCameraRight.GetX(), xCameraRight.GetY(), xCameraRight.GetZ(), 1.0f);
	D3DXVECTOR4 xD3DCameraUp( xCameraUp.GetX(), xCameraUp.GetY(), xCameraUp.GetZ(), 1.0f);
	D3DXVECTOR4 xD3DCameraForwards( xCameraForwards.GetX(), xCameraForwards.GetY(), xCameraForwards.GetZ(), 1.0f);
	
	pxD3DEffect->SetParameterByName( "g_xCameraRightVector", xD3DCameraRight);
	pxD3DEffect->SetParameterByName( "g_xCameraUpVector", xD3DCameraUp);
	pxD3DEffect->SetParameterByName( "g_xCameraForwardsVector", xD3DCameraForwards);
}



bool Asura_Direct3D_BGR_Manager::ResetRenderStates( void)
{
	SCOPEDPIXEVENT((0xff00ff00, L"SM30_ResetRenderStates"));

	if( false == this->m_bDrawGlobalRenderStatesSet)
	{
		return( true);
	}
	
#if 1 && defined( ASURA_DIRECT3D_BGR_ENABLE_D3D_CALLS)
	HRESULT xResult;
	
	Direct3D_Device* pxD3DDevice = Asura_Direct3D::D3DDevice;
	
	// DLH - Have to add these two lines otherwise NOTHING else renders correctly!
	Asura_Render::CurrentStates.RequestTextureOpMode(ASURA_TEXTUREOP_WHITE);
	Asura_Direct3D_Shaders::RequestFixedFunctionVertexShader( D3DFVF_GENERICVERTEX);
	
	xResult = pxD3DDevice->SetRenderState(D3DRS_FOGENABLE, FALSE);
	ASURA_D3D_OK( xResult);	
	
	Asura_Render::CurrentStates.RequestAlphaTest( false);
	Asura_Render::CurrentStates.RequestZBufferEnabled(true);
	Asura_Render::CurrentStates.RequestZBufferWriteEnabled(true);
	Asura_Render::CurrentStates.RequestZBufferCompareMode(ASURA_ZBUFFERCMP_LESSEQUAL);
	Asura_Render::CurrentStates.RequestCullMode( ASURA_CULLMODE_NONE);

	xResult = pxD3DDevice->SetStreamSourceFreq( 0, 1);
	ASURA_D3D_OK( xResult);	
	xResult = pxD3DDevice->SetStreamSourceFreq( 1, 1);
	ASURA_D3D_OK( xResult);	

	for( u_int u = 0; u < 16; ++u )
	{
		Asura_Direct3D::D3DDevice->SetSamplerState( u, D3DSAMP_SRGBTEXTURE, FALSE );
	}

	//Asura_Render::CurrentStates.Request(ASURA_TEXTUREOP_WHITE);
	//Asura_Render::CurrentStates.RequestTextureOpMode(ASURA_TEXTUREOP_WHITE);
#endif
	
	return( true);
}

bool Asura_Direct3D_BGR_Manager::Draw_SM30_GlobalRenderStates( void)
{
	SCOPEDPIXEVENT((0xff00ff00, L"SM30_DrawGlobalRenderStates"));

	if( true == this->m_bDrawGlobalRenderStatesSet)
	{
		return( true);
	}
	
	this->m_bDrawGlobalRenderStatesSet = true;

#if 1 && defined( ASURA_DIRECT3D_BGR_ENABLE_D3D_CALLS)
	#if defined( ASURA_ASSERT )
		Direct3D_Device* pxD3DDevice = Asura_Direct3D::D3DDevice;
	#endif

	Asura_Assert( NULL != pxD3DDevice, "It would be nice for Direct3D to be setup!");

	Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_OFF);
	Asura_Render::CurrentStates.RequestWireFrameMode( false);
	Asura_Render::CurrentStates.RequestAlphaTest( true, 0x00);
	Asura_Render::CurrentStates.RequestZBufferEnabled(true);
	Asura_Render::CurrentStates.RequestZBufferWriteEnabled(true);
	Asura_Render::CurrentStates.RequestZBufferCompareMode(ASURA_ZBUFFERCMP_LESSEQUAL);
	
#if 1 && defined( ASURA_TOOLS)
	if (Tools_Render_Options::GetWireframeMode() != TOOLS_WIREFRAME_OFF)
	{
		Asura_Render::CurrentStates.RequestTextureOpMode(ASURA_TEXTUREOP_WHITE);
		Asura_Render::CurrentStates.RequestWireFrameMode( true);
	}
#endif

	return( true);
#else
	return( true);
#endif
}

bool Asura_Direct3D_BGR_Manager::Draw_SM30_StaticLightRenderStates()
{
	SCOPEDPIXEVENT((0xff00ff00, L"SM30_DrawStaticLightRenderStates"));

	Asura_Render::CurrentStates.RequestZBufferWriteEnabled(true);
	Asura_Render::CurrentStates.RequestZBufferCompareMode(ASURA_ZBUFFERCMP_LESSEQUAL);

	return( true);
}
	
template< class Tgeometry > void Asura_Direct3D_BGR_Manager::DrawTree_SM30_MaterialRenderStates( const Tgeometry &tGeometry, Asura_Direct3D_Effect* pxEffect)
{
	SCOPEDPIXEVENT(( 0xFF800000, L"Set data for material "));

	const Asura_Direct3D_BGR_Material& xMaterial = tGeometry.xMaterial;

	Asura_Direct3D_MaterialResponse_ShaderData xMaterialResponse_ShaderData_Base;
	
	const Asura_MaterialResponse* pxMaterialResponse = Asura_MaterialResponse_System::GetMaterialByHashID( xMaterial.m_xMaterialResponseHashID);
	if( NULL == pxMaterialResponse)
	{
		xMaterialResponse_ShaderData_Base.Clear();
	}
	else
	{
		xMaterialResponse_ShaderData_Base.SetupCommon();
		xMaterialResponse_ShaderData_Base.Setup( pxMaterialResponse, NULL, NULL);
	}
	
	xMaterialResponse_ShaderData_Base.SetParameterByName( pxEffect);
}

#if defined( ASURA_DIRECT3D_BGR_SUPPORT_LEAVES)
uint32 Asura_Direct3D_BGR_Manager::DrawTree_SM30_LeafInstances( const Asura_Direct3D_BGR_Type& xTreeDescription, const Asura_Direct3D_BGR_Type_Geometry &xGeometry, Asura_Direct3D_BGR_GeometryVertexInstanceCache& xCacheGeometryInstances, const void* pxDataPointer, Asura_Direct3D_Effect* pxD3DEffect)
{
	Asura_Assert( NULL != pxD3DEffect, "Effect not Setup");
	 
	Direct3D_VertexBuffer* pxVertexBufferInstance;
	u_int uVertexBufferInstanceOffset;
	xCacheGeometryInstances.GetVertexBuffer( pxVertexBufferInstance, uVertexBufferInstanceOffset);
	if( NULL == pxVertexBufferInstance)
	{
		return( 0);
	}

	Direct3D_VertexBuffer* pxVertexBufferGeometry = xGeometry.m_pxVertexBuffer;
	if( NULL == pxVertexBufferGeometry)
	{
		return( 0);
	}

	Direct3D_IndexBuffer* pxIndexBuffer = xGeometry.m_pxIndexBuffer;
	if( NULL == pxIndexBuffer)
	{
		return( 0);
	}
	
	const uint32 uPrimitiveCount = xGeometry.m_uPrimitiveCount;
	
	if( 0 == uPrimitiveCount)
	{
		return( 0);
	}
	
	const uint32 uCountCacheInstances = xCacheGeometryInstances.Size();
	const uint32 uCountInstance = uCountCacheInstances;
	
	SCOPEDPIXEVENT((0xff00ff00, L"SM30_DrawTreeLeafInstances( %d)", uCountInstance));
	
#if 1 && defined( ASURA_DIRECT3D_BGR_ENABLE_D3D_CALLS)
	HRESULT xResult;
	
	Direct3D_Device* pxD3DDevice = Asura_Direct3D::D3DDevice;

	// Set index & vertex buffer.
	xResult = pxD3DDevice->SetVertexDeclaration(this->m_pxVertexDeclaration_Leaf);
	ASURA_D3D_OK( xResult);	
	xResult = pxD3DDevice->SetStreamSource( 0, pxVertexBufferGeometry, xGeometry.m_uVBStart * sizeof( Asura_Direct3D_BGR_Type_VertexGeometryLeaf), sizeof( Asura_Direct3D_BGR_Type_VertexGeometryLeaf));
	ASURA_D3D_OK( xResult);	
	xResult = pxD3DDevice->SetStreamSourceFreq( 0, D3DSTREAMSOURCE_INDEXEDDATA | uCountInstance);
	ASURA_D3D_OK( xResult);	
	xResult = pxD3DDevice->SetIndices( pxIndexBuffer);
	ASURA_D3D_OK( xResult);	

	xResult = pxD3DDevice->SetStreamSource( 1, pxVertexBufferInstance, uVertexBufferInstanceOffset, sizeof( Asura_Direct3D_BGR_Type_VertexInstanceSM30));
	ASURA_D3D_OK( xResult);	
	xResult = pxD3DDevice->SetStreamSourceFreq( 1, uint32( D3DSTREAMSOURCE_INSTANCEDATA) | 1);
	ASURA_D3D_OK( xResult);	
	
	//-- Setup render states.

	//-- Finally, the actual rendering!

	uint32 uIndexIBStart = xGeometry.m_uIBStart;
	{
		const uint32 uBaseVertexIndex = 0;
		const uint32 uVBStart = 0;
		const uint32 uVBSize = xGeometry.m_uVBSize;
		const uint32 uIBSize = xGeometry.m_uIBSize;
		const uint32 uPrimitiveCount = uIBSize / 3;
		
		pxD3DEffect->RenderAllIndexedPrimitivePasses( D3DPT_TRIANGLELIST, 0, uVBSize, uIndexIBStart, uPrimitiveCount, NULL, uBaseVertexIndex + uVBStart);

		#if defined( ASURA_TOOLS)
			TOOLS_BOTANICAL_STATISTICS_ADD( s_uDIPCount, 1);
			TOOLS_BOTANICAL_STATISTICS_ADD( s_uDIPLeafCount, 1);
			TOOLS_BOTANICAL_STATISTICS_ADD( s_uDIPIndexCount, uIBSize * uCountInstance);
			TOOLS_BOTANICAL_STATISTICS_ADD( s_uDIPVertexCount, uVBSize * uCountInstance);
			TOOLS_BOTANICAL_STATISTICS_ADD( s_uDIPPrimitiveCount, uPrimitiveCount * uCountInstance);
			TOOLS_BOTANICAL_STATISTICS_ADD( s_uDIPLeafPrimitiveCount, uPrimitiveCount * uCountInstance);
		#endif
	}
#endif

	return( uCountInstance);
}
#endif

#if defined( ASURA_DIRECT3D_BGR_SUPPORT_LEAVES)
bool Asura_Direct3D_BGR_Manager::DrawTree_SM30_LeafRenderStates( const Asura_Direct3D_BGR_Type& xTreeDescription, const Asura_Direct3D_BGR_Type_Geometry &xGeometry, const void* pxDataPointer, Asura_Direct3D_Effect*& pxD3DEffect)
{
	pxD3DEffect = NULL;

	const uint32 primitiveCount = xGeometry.m_uPrimitiveCount;
	
	if( 0 == primitiveCount)
	{
		return( true);
	}
	
	SCOPEDPIXEVENT((0xff00ff00, L"SM30_DrawTreeLeafRenderStates"));

#if 1 && defined( ASURA_DIRECT3D_BGR_ENABLE_D3D_CALLS)
	this->Draw_SM30_GlobalRenderStates();
	
	#if defined( ASURA_ASSERT )
		Direct3D_Device* pxD3DDevice = Asura_Direct3D::D3DDevice;
	#endif
	
	Asura_Assert( NULL != pxD3DDevice, "It would be nice for Direct3D to be setup!");

	const Asura_Photon* pxPhoton = reinterpret_cast< const Asura_Photon*>( pxDataPointer);
	
	{
		
		// Set vertex shader.	
		if( NULL == pxPhoton)
		{
			pxD3DEffect = Asura_Direct3D_Effects_System::BeginEffect( "Bot_Billboard_Prelighting_Pass" );

			if ( NULL == pxD3DEffect || false == pxD3DEffect->IsD3DEffectValid())
			{
				return( false);
			}

			Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_OFF);
			Asura_Render::CurrentStates.ResetTextureIndexToLast( 1);
			
			this->Draw_SM30_StaticLightRenderStates();
		}
#if 0
		else
		{
			if( true == Asura_Fog::IsGraphFogEnabled())
			{
				Asura_Direct3D_Shaders::RequestVertexShader( ASURA_D3D_VS_BOTANICAL_LEAF_FOGG_LIGHTD);
			}
			else
			{
				Asura_Direct3D_Shaders::RequestVertexShader( ASURA_D3D_VS_BOTANICAL_LEAF_FOGL_LIGHTD);
			}
			Asura_Direct3D_Shaders::RequestPixelShader( ASURA_D3D_PS_BOTANICAL_GEOMETRY3D_LIGHTD);
			Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_ADD);
			
			Asura_Render::CurrentStates.ResetTextureIndexToLast( 3);
			
			this->Draw_SM30_DynamicLightRenderStates( *pxPhoton);
		}
#endif
	}

	Asura_Render::CurrentStates.RequestCullMode( ASURA_CULLMODE_NONE);
	
#if 1 && defined( ASURA_TOOLS)
	if (Tools_Render_Options::GetWireframeMode() != TOOLS_WIREFRAME_OFF)
	{
		Asura_Render::CurrentStates.RequestTextureOpMode(ASURA_TEXTUREOP_WHITE);
		Asura_Render::CurrentStates.RequestWireFrameMode( true);
	}
#endif

	Asura_Direct3D_BGR_Manager_DrawTree_SM30_EffectSetupCamera( pxD3DEffect);
	
	this->DrawTree_SM30_MaterialRenderStates( xGeometry, pxD3DEffect);
	
	return( true);
#else
	return( true);
#endif
}
#endif

#if defined( ASURA_DIRECT3D_BGR_SUPPORT_GEOMETRY3D)
uint32 Asura_Direct3D_BGR_Manager::DrawTree_SM30_3D1SidedInstances( const Asura_Direct3D_BGR_Type& xTreeDescription, const Asura_Direct3D_BGR_Type_Geometry &xGeometry, Asura_Direct3D_BGR_GeometryVertexInstanceCache& xCacheGeometryInstances, const void* pxDataPointer, Asura_Direct3D_Effect* pxD3DEffect)
{
	Asura_Assert( NULL != pxD3DEffect, "Effect not Setup");
	 
	Direct3D_VertexBuffer* pxVertexBufferInstance;
	u_int uVertexBufferInstanceOffset;
	xCacheGeometryInstances.GetVertexBuffer( pxVertexBufferInstance, uVertexBufferInstanceOffset);
	if( NULL == pxVertexBufferInstance)
	{
		return( 0);
	}

	Direct3D_VertexBuffer* pxVertexBufferGeometry = xGeometry.m_pxVertexBuffer;
	if( NULL == pxVertexBufferGeometry)
	{
		return( 0);
	}

	Direct3D_IndexBuffer* pxIndexBuffer = xGeometry.m_pxIndexBuffer;
	if( NULL == pxIndexBuffer)
	{
		return( 0);
	}
	
	const uint32 uPrimitiveCount = xGeometry.m_uPrimitiveCount;
	
	if( 0 == uPrimitiveCount)
	{
		return( 0);
	}
	
	const uint32 uCountCacheInstances = xCacheGeometryInstances.Size();
	const uint32 uCountInstance = uCountCacheInstances;
	
	SCOPEDPIXEVENT((0xff00ff00, L"SM30_DrawTree3D1SidedInstances( %d)", uCountInstance));
	
#if 1 && defined( ASURA_DIRECT3D_BGR_ENABLE_D3D_CALLS)
	HRESULT xResult;
	
	Direct3D_Device* pxD3DDevice = Asura_Direct3D::D3DDevice;

	// Set index & vertex buffer.
	xResult = pxD3DDevice->SetVertexDeclaration(this->m_pxVertexDeclaration_3DGeom);
	ASURA_D3D_OK( xResult);	
	xResult = pxD3DDevice->SetStreamSource( 0, pxVertexBufferGeometry, xGeometry.m_uVBStart * sizeof( Asura_Direct3D_BGR_Type_VertexGeometry3DBasic), sizeof( Asura_Direct3D_BGR_Type_VertexGeometry3DBasic));
	ASURA_D3D_OK( xResult);	
	xResult = pxD3DDevice->SetStreamSourceFreq( 0, D3DSTREAMSOURCE_INDEXEDDATA | uCountInstance);
	ASURA_D3D_OK( xResult);	
	xResult = pxD3DDevice->SetIndices( pxIndexBuffer);
	ASURA_D3D_OK( xResult);	

	xResult = pxD3DDevice->SetStreamSource( 1, pxVertexBufferInstance, uVertexBufferInstanceOffset, sizeof( Asura_Direct3D_BGR_Type_VertexInstanceSM30));
	ASURA_D3D_OK( xResult);	
	xResult = pxD3DDevice->SetStreamSourceFreq( 1, uint32( D3DSTREAMSOURCE_INSTANCEDATA) | 1);
	ASURA_D3D_OK( xResult);	
	
	//-- Setup render states.

	//-- Finally, the actual rendering!

	uint32 uIndexIBStart = xGeometry.m_uIBStart;
	{
		const uint32 uBaseVertexIndex = 0;
		const uint32 uVBStart = 0;
		const uint32 uVBSize = xGeometry.m_uVBSize;
		const uint32 uIBSize = xGeometry.m_uIBSize;
		const uint32 uPrimitiveCount = uIBSize / 3;
		
		pxD3DEffect->RenderAllIndexedPrimitivePasses( D3DPT_TRIANGLELIST, 0, uVBSize, uIndexIBStart, uPrimitiveCount, NULL, uBaseVertexIndex + uVBStart);

		#if defined( ASURA_TOOLS)
			TOOLS_BOTANICAL_STATISTICS_ADD( s_uDIPCount, 1);
			TOOLS_BOTANICAL_STATISTICS_ADD( s_uDIP3DCount, 1);
			TOOLS_BOTANICAL_STATISTICS_ADD( s_uDIPIndexCount, uIBSize * uCountInstance);
			TOOLS_BOTANICAL_STATISTICS_ADD( s_uDIPVertexCount, uVBSize * uCountInstance);
			TOOLS_BOTANICAL_STATISTICS_ADD( s_uDIPPrimitiveCount, uPrimitiveCount * uCountInstance);
			TOOLS_BOTANICAL_STATISTICS_ADD( s_uDIP3DPrimitiveCount, uPrimitiveCount * uCountInstance);
		#endif
	}
#endif

	return( uCountInstance);
}
#endif

#if defined( ASURA_DIRECT3D_BGR_SUPPORT_GEOMETRY3D)
bool Asura_Direct3D_BGR_Manager::DrawTree_SM30_3D1SidedRenderStates( const Asura_Direct3D_BGR_Type& xTreeDescription, const Asura_Direct3D_BGR_Type_Geometry &xGeometry, const void* pxDataPointer, Asura_Direct3D_Effect*& pxD3DEffect)
{
	pxD3DEffect = NULL;

	const uint32 primitiveCount = xGeometry.m_uPrimitiveCount;
	
	if( 0 == primitiveCount)
	{
		return( true);
	}
	
	SCOPEDPIXEVENT((0xff00ff00, L"SM30_DrawTree3D1SidedRenderStates"));
	
#if 1 && defined( ASURA_DIRECT3D_BGR_ENABLE_D3D_CALLS)
	this->Draw_SM30_GlobalRenderStates();
	
	#if defined( ASURA_ASSERT )
		Direct3D_Device* pxD3DDevice = Asura_Direct3D::D3DDevice;
	#endif

	Asura_Assert( NULL != pxD3DDevice, "It would be nice for Direct3D to be setup!");

	const Asura_Photon* pxPhoton = reinterpret_cast< const Asura_Photon*>( pxDataPointer);
	
	{
		// Set vertex shader.	
		if( NULL == pxPhoton)
		{
			pxD3DEffect = Asura_Direct3D_Effects_System::BeginEffect( "Bot_3D_Prelighting_Pass" );

			if ( NULL == pxD3DEffect || false == pxD3DEffect->IsD3DEffectValid())
			{
				return( false);
			}

			Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_OFF);
			Asura_Render::CurrentStates.ResetTextureIndexToLast( 1);
			
			// (AAM - 24.01.2008) Disable dual-sided lighting for single-sided botanicals
			pxD3DEffect->SetParameterByHash( "g_fDualSidedLightingLerp", 0xA8CBD626, 0.0f );

			this->Draw_SM30_StaticLightRenderStates();
		}
#if 0
		else
		{
			if( true == Asura_Fog::IsGraphFogEnabled())
			{
				Asura_Direct3D_Shaders::RequestVertexShader( ASURA_D3D_VS_BOTANICAL_GEOMETRY3D_FOGG_LIGHTD);
			}
			else
			{
				Asura_Direct3D_Shaders::RequestVertexShader( ASURA_D3D_VS_BOTANICAL_GEOMETRY3D_FOGL_LIGHTD);
			}
			
			Asura_Direct3D_Shaders::RequestPixelShader( ASURA_D3D_PS_BOTANICAL_GEOMETRY3D_LIGHTD);
			Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_ADD);
			
			Asura_Render::CurrentStates.ResetTextureIndexToLast( 3);
			
			this->Draw_SM30_DynamicLightRenderStates( *pxPhoton);
		}
#endif
	}
	
	Asura_Render::CurrentStates.RequestCullMode( ASURA_CULLMODE_CW);
	
#if 1 && defined( ASURA_TOOLS)
	if (Tools_Render_Options::GetWireframeMode() != TOOLS_WIREFRAME_OFF)
	{
		Asura_Render::CurrentStates.RequestTextureOpMode(ASURA_TEXTUREOP_WHITE);
		Asura_Render::CurrentStates.RequestWireFrameMode( true);
	}
#endif

	this->DrawTree_SM30_MaterialRenderStates( xGeometry, pxD3DEffect);

	return( true);
#else
	return( true);
#endif
}
#endif
	
#if defined( ASURA_DIRECT3D_BGR_SUPPORT_GEOMETRY3D)
uint32 Asura_Direct3D_BGR_Manager::DrawTree_SM30_3D2SidedInstances( const Asura_Direct3D_BGR_Type& xTreeDescription, const Asura_Direct3D_BGR_Type_Geometry &xGeometry, Asura_Direct3D_BGR_GeometryVertexInstanceCache& xCacheGeometryInstances, const void* pxDataPointer, Asura_Direct3D_Effect* pxD3DEffect)
{
	Asura_Assert( NULL != pxD3DEffect, "Effect not Setup");
	 
	Direct3D_VertexBuffer* pxVertexBufferInstance;
	u_int uVertexBufferInstanceOffset;
	xCacheGeometryInstances.GetVertexBuffer( pxVertexBufferInstance, uVertexBufferInstanceOffset);
	if( NULL == pxVertexBufferInstance)
	{
		return( 0);
	}

	Direct3D_VertexBuffer* pxVertexBufferGeometry = xGeometry.m_pxVertexBuffer;
	if( NULL == pxVertexBufferGeometry)
	{
		return( 0);
	}

	Direct3D_IndexBuffer* pxIndexBuffer = xGeometry.m_pxIndexBuffer;
	if( NULL == pxIndexBuffer)
	{
		return( 0);
	}
	
	const uint32 uPrimitiveCount = xGeometry.m_uPrimitiveCount;
	
	if( 0 == uPrimitiveCount)
	{
		return( 0);
	}
	
	const uint32 uCountCacheInstances = xCacheGeometryInstances.Size();
	const uint32 uCountInstance = uCountCacheInstances;
	
	SCOPEDPIXEVENT((0xff00ff00, L"SM30_DrawTree3D2SidedInstances( %d)", uCountInstance));
	
#if 1 && defined( ASURA_DIRECT3D_BGR_ENABLE_D3D_CALLS)
	HRESULT xResult;
	
	Direct3D_Device* pxD3DDevice = Asura_Direct3D::D3DDevice;

	// Set index & vertex buffer.
	xResult = pxD3DDevice->SetVertexDeclaration(this->m_pxVertexDeclaration_3DGeom);
	ASURA_D3D_OK( xResult);	
	xResult = pxD3DDevice->SetStreamSource( 0, pxVertexBufferGeometry, xGeometry.m_uVBStart * sizeof( Asura_Direct3D_BGR_Type_VertexGeometry3DBasic), sizeof( Asura_Direct3D_BGR_Type_VertexGeometry3DBasic));
	ASURA_D3D_OK( xResult);	
	xResult = pxD3DDevice->SetStreamSourceFreq( 0, D3DSTREAMSOURCE_INDEXEDDATA | uCountInstance);
	ASURA_D3D_OK( xResult);	
	xResult = pxD3DDevice->SetIndices( pxIndexBuffer);
	ASURA_D3D_OK( xResult);	

	xResult = pxD3DDevice->SetStreamSource( 1, pxVertexBufferInstance, uVertexBufferInstanceOffset, sizeof( Asura_Direct3D_BGR_Type_VertexInstanceSM30));
	ASURA_D3D_OK( xResult);	
	xResult = pxD3DDevice->SetStreamSourceFreq( 1, uint32( D3DSTREAMSOURCE_INSTANCEDATA) | 1);
	ASURA_D3D_OK( xResult);	
	
	//-- Setup render states.

	//-- Finally, the actual rendering!

	uint32 uIndexIBStart = xGeometry.m_uIBStart;
	{
		const uint32 uBaseVertexIndex = 0;
		const uint32 uVBStart = 0;
		const uint32 uVBSize = xGeometry.m_uVBSize;
		const uint32 uIBSize = xGeometry.m_uIBSize;
		const uint32 uPrimitiveCount = uIBSize / 3;
		
		pxD3DEffect->RenderAllIndexedPrimitivePasses( D3DPT_TRIANGLELIST, 0, uVBSize, uIndexIBStart, uPrimitiveCount, NULL, uBaseVertexIndex + uVBStart);

		#if defined( ASURA_TOOLS)
			TOOLS_BOTANICAL_STATISTICS_ADD( s_uDIPCount, 1);
			TOOLS_BOTANICAL_STATISTICS_ADD( s_uDIP3DCount, 1);
			TOOLS_BOTANICAL_STATISTICS_ADD( s_uDIPIndexCount, uIBSize * uCountInstance);
			TOOLS_BOTANICAL_STATISTICS_ADD( s_uDIPVertexCount, uVBSize * uCountInstance);
			TOOLS_BOTANICAL_STATISTICS_ADD( s_uDIPPrimitiveCount, uPrimitiveCount * uCountInstance);
			TOOLS_BOTANICAL_STATISTICS_ADD( s_uDIP3DPrimitiveCount, uPrimitiveCount * uCountInstance);
		#endif
	}
#endif

	return( uCountInstance);
}
#endif

#if defined( ASURA_DIRECT3D_BGR_SUPPORT_GEOMETRY3D)
bool Asura_Direct3D_BGR_Manager::DrawTree_SM30_3D2SidedRenderStates( const Asura_Direct3D_BGR_Type& xTreeDescription, const Asura_Direct3D_BGR_Type_Geometry &xGeometry, const void* pxDataPointer, Asura_Direct3D_Effect*& pxD3DEffect)
{
	SCOPEDPIXEVENT((0xff00ff00, L"SM30_DrawTree3D2SidedRenderStates"));

	pxD3DEffect = NULL;

	const uint32 primitiveCount = xGeometry.m_uPrimitiveCount;
	
	if( 0 == primitiveCount)
	{
		return( true);
	}
	
#if 1 && defined( ASURA_DIRECT3D_BGR_ENABLE_D3D_CALLS)
	this->Draw_SM30_GlobalRenderStates();
	
	#if defined( ASURA_ASSERT )
		Direct3D_Device* pxD3DDevice = Asura_Direct3D::D3DDevice;
	#endif

	Asura_Assert( NULL != pxD3DDevice, "It would be nice for Direct3D to be setup!");

	const Asura_Photon* pxPhoton = reinterpret_cast< const Asura_Photon*>( pxDataPointer);
	
	Asura_Render::CurrentStates.RequestTextureOpMode( ASURA_TEXTUREOP_CUSTOM);
	
	{
		// Set vertex shader.	
		if( NULL == pxPhoton)
		{
			pxD3DEffect = Asura_Direct3D_Effects_System::BeginEffect( "Bot_3D_Prelighting_Pass" );

			if ( NULL == pxD3DEffect || false == pxD3DEffect->IsD3DEffectValid())
			{
				return( false);
			}
		
			Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_OFF);
			Asura_Render::CurrentStates.ResetTextureIndexToLast( 1);
			
			pxD3DEffect->SetParameterByHash( "g_fDualSidedLightingLerp", 0xA8CBD626, s_fDualSidedLightingLerp );

			this->Draw_SM30_StaticLightRenderStates();
		}
#if 0
		else
		{
			if( true == Asura_Fog::IsGraphFogEnabled())
			{
				Asura_Direct3D_Shaders::RequestVertexShader( ASURA_D3D_VS_BOTANICAL_GEOMETRY3D_FOGG_LIGHTD);
			}
			else
			{
				Asura_Direct3D_Shaders::RequestVertexShader( ASURA_D3D_VS_BOTANICAL_GEOMETRY3D_FOGL_LIGHTD);
			}
			
			Asura_Direct3D_Shaders::RequestPixelShader( ASURA_D3D_PS_BOTANICAL_GEOMETRY3D_LIGHTD);
			Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_ADD);
			
			Asura_Render::CurrentStates.ResetTextureIndexToLast( 3);
			
			this->Draw_SM30_DynamicLightRenderStates( *pxPhoton);
		}
#endif
	}
	
	Asura_Render::CurrentStates.RequestCullMode( ASURA_CULLMODE_NONE);
	
#if 1 && defined( ASURA_TOOLS)
	if (Tools_Render_Options::GetWireframeMode() != TOOLS_WIREFRAME_OFF)
	{
		Asura_Render::CurrentStates.RequestTextureOpMode(ASURA_TEXTUREOP_WHITE);
		Asura_Render::CurrentStates.RequestWireFrameMode( true);
	}
#endif

	this->DrawTree_SM30_MaterialRenderStates( xGeometry, pxD3DEffect);

	return( true);
#else
	return( true);
#endif
}
#endif
	
#if defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS)
uint32 Asura_Direct3D_BGR_Manager::DrawTreeBillboardInstances( const Asura_Direct3D_BGR_Type_Geometry &xGeometry, const struct Asura_Direct3D_BGR_GeometryInstanceConstants *xCacheConstants, const uint32 uCountCacheInstances, const void* pxDataPointer)
{
	SCOPEDPIXEVENT((0xff00ff00, L"SM30_DrawTreeBillboardInstances"));
	return( 0);
}
#endif

#if defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS)
bool Asura_Direct3D_BGR_Manager::DrawTreeBillboardInstancesDIPCacheRenderStates_End( Asura_Direct3D_Effect* pxD3DEffect)
{
	if( NULL != pxD3DEffect)
	{
		pxD3DEffect->EndPass();
	}
	return( true);
}
#endif

#if defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS)
bool Asura_Direct3D_BGR_Manager::DrawTreeBillboardInstancesDIPCacheRenderStates_Start_IB( const Asura_Direct3D_BGR_Type& xTreeDescription, const float fScreenDistanceScale, const Asura_Direct3D_BGR_Type_Geometry &xGeometry, Asura_Direct3D_Effect*& pxD3DEffect)
{
	//Direct3D_IndexBuffer* pxIndexBuffer = xGeometry.m_pxIndexBuffer;
	Direct3D_IndexBuffer* pxIndexBuffer = this->m_xD3DBufferImposter.m_pxIndexBuffer;
	if( NULL == pxIndexBuffer)
	{
		return( false);
	}
	
#if 1 && defined( ASURA_DIRECT3D_BGR_ENABLE_D3D_CALLS)
	HRESULT xResult;
	
	xResult = Asura_Direct3D::D3DDevice->SetIndices( pxIndexBuffer);
	ASURA_D3D_OK( xResult);	
#endif

	return( true);
}
#endif

#if defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS)
bool Asura_Direct3D_BGR_Manager::DrawTreeBillboardInstancesDIPCacheRenderStates_Start_All( const Asura_Direct3D_BGR_Type& xTreeDescription, const float fScreenDistanceScale, const Asura_Direct3D_BGR_Type_Geometry &xGeometry, Asura_Direct3D_Effect*& pxD3DEffect)
{
	//Direct3D_VertexBuffer* pxVertexBuffer = xGeometry.m_pxVertexBuffer;
	Direct3D_VertexBuffer* pxVertexBuffer = this->m_xD3DBufferImposter.m_pxVertexBuffer;
	if( NULL == pxVertexBuffer)
	{
		return( false);
	}

#if defined( ASURA_BOTANICAL_SUPPORT_INTERACTIVE)
	const Asura_BGR_Type_BasicInteractive *pxBasicInteractive = xTreeDescription.GetBasicInteractive();
#endif
#if defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS_INTERACTIVE)
	Direct3D_VertexBuffer* pxVertexBufferInteractive = this->m_xD3DBufferImposterInteractive.m_pxVertexBuffer;
	if( NULL != pxBasicInteractive)
	{
		if( NULL == pxVertexBufferInteractive)
		{
			return( false);
		}

	}
#endif
	
#if defined( ASURA_BOTANICAL_SUPPORT_INTERACTIVE)
	SCOPEDPIXEVENT((0xff00ff00, L"SM30_DrawTreeBillboardRenderStates%s", ((NULL != pxBasicInteractive) ? L" Interactive" : L"")));
#else
	SCOPEDPIXEVENT((0xff00ff00, L"SM30_DrawTreeBillboardRenderStates"));
#endif

	Asura_Vector_3 xCameraDirection;
	Asura_Camera::GetForwardVector( xCameraDirection);
	xCameraDirection *= -1.0f;
	xCameraDirection.SetY( 0);
	xCameraDirection.Normalise();
	
	const Asura_Direct3D_BGR_Type_GeometryBillboard &xGeometryImposter = static_cast< const Asura_Direct3D_BGR_Type_GeometryBillboard& >( xGeometry);
	
	const uint32 uSignificantImposterMaxCount = 2;
	struct SignificantImposter
	{
		const Asura_Direct3D_BGR_Type_VertexGeometryImposterStorage* m_pxImposterType;
		float m_fAlphaFade;
	};
	SignificantImposter axMSImposters[ uSignificantImposterMaxCount] = 
	{ 
		{NULL, 1.0f},
		{NULL, 1.0f}
	};
	
	const uint32 uImposterCount = xGeometryImposter.m_xArrayBillboards.Size();
	for( uint32 uImposterIndex = 0; uImposterIndex < uImposterCount; uImposterIndex += 1)
	{
		const Asura_Direct3D_BGR_Type_VertexGeometryImposterStorage* pxImposter = &(xGeometryImposter.m_xArrayBillboards[ uImposterIndex]);
		float fAlphaRotateFade = 1.0f;
		const float fImposterDotCamera = xCameraDirection * Asura_Vector_3( pxImposter->m_xDirection);
		if( fImposterDotCamera < pxImposter->m_fFieldOfViewDotOutter)
		{
			continue;
		}
		const float fImposterAngleCamera = Asura_Maths::ArcCosine( fImposterDotCamera);
		fAlphaRotateFade = 0.5f * (fImposterAngleCamera - pxImposter->m_fFieldOfViewAngleInner) / (pxImposter->m_fFieldOfViewAngleOutter - pxImposter->m_fFieldOfViewAngleInner);
		fAlphaRotateFade = Asura_Maths::Min( fAlphaRotateFade, 1.0f);
		fAlphaRotateFade = Asura_Maths::Max( fAlphaRotateFade, 0.0f);
		
		for( uint32 uSignificantImposterCheckIndex = 0; uSignificantImposterCheckIndex < uSignificantImposterMaxCount; uSignificantImposterCheckIndex += 1)
		{
			SignificantImposter& xMSImposter = axMSImposters[ uSignificantImposterCheckIndex];
			if( NULL != xMSImposter.m_pxImposterType && xMSImposter.m_fAlphaFade <= fAlphaRotateFade)
			{
				continue;
			}

			const Asura_Direct3D_BGR_Type_VertexGeometryImposterStorage* pxImposterOld = xMSImposter.m_pxImposterType;
			const float fAlphaRotateFadeOld = xMSImposter.m_fAlphaFade;
			
			xMSImposter.m_pxImposterType = pxImposter;
			xMSImposter.m_fAlphaFade = fAlphaRotateFade;
			
			pxImposter = pxImposterOld;
			fAlphaRotateFade = fAlphaRotateFadeOld;
		}
	}
	
	const Asura_Direct3D_BGR_Type_VertexGeometryImposterStorage* pxImposterType0 = axMSImposters[0].m_pxImposterType;
	const Asura_Direct3D_BGR_Type_VertexGeometryImposterStorage* pxImposterType1 = axMSImposters[1].m_pxImposterType;

	Asura_Assert( (pxImposterType0 == NULL && pxImposterType1 == NULL) || (pxImposterType0 != NULL && pxImposterType1 == NULL) || (pxImposterType0 != NULL && pxImposterType1 != NULL), "Most Significant Imposters does not seem right");
	uint32 uSignificantImposterCount = ((NULL != pxImposterType0) ? 1 : 0) + ((NULL != pxImposterType1) ? 1 : 0);
	
	if( 0 == uSignificantImposterCount)
	{
		return( false);
	}

#if 1 && defined( ASURA_DIRECT3D_BGR_ENABLE_D3D_CALLS)
	HRESULT xResult;
	
	switch( uSignificantImposterCount)
	{
	
		case 1:
		{
		#if defined( ASURA_BOTANICAL_SUPPORT_INTERACTIVE)
			if( NULL != pxBasicInteractive)
			{
				pxD3DEffect = Asura_Direct3D_Effects_System::BeginEffect( "Bot_Imposter1_Interactive_Prelighting_Pass" );
			}
			else
		#endif
			{
				pxD3DEffect = Asura_Direct3D_Effects_System::BeginEffect( "Bot_Imposter1_Prelighting_Pass" );
			}

			if ( NULL == pxD3DEffect || false == pxD3DEffect->IsD3DEffectValid())
			{
				return( false);
			}
			break;
		}
		case 2:
		{
		#if defined( ASURA_BOTANICAL_SUPPORT_INTERACTIVE)
			if( NULL != pxBasicInteractive)
			{
				pxD3DEffect = Asura_Direct3D_Effects_System::BeginEffect( "Bot_Imposter2_Interactive_Prelighting_Pass" );
			}
			else
		#endif
			{
				pxD3DEffect = Asura_Direct3D_Effects_System::BeginEffect( "Bot_Imposter2_Prelighting_Pass" );
			}

			if ( NULL == pxD3DEffect || false == pxD3DEffect->IsD3DEffectValid())
			{
				return( false);
			}
			break;
		}
		default:
		{
			Asura_Assert( false, "Unhandled number of Signifiant Imposters");
			return( false);
		}	
	}
	
	this->Draw_SM30_GlobalRenderStates();
	
	Direct3D_Device* pxD3DDevice = Asura_Direct3D::D3DDevice;

	Asura_Assert( NULL != pxD3DDevice, "It would be nice for Direct3D to be setup!");

#if defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS_INTERACTIVE)
	if( NULL != pxBasicInteractive)
	{
		xResult = pxD3DDevice->SetVertexDeclaration(this->m_pxVertexDeclaration_Interactive_Imposter);
		ASURA_D3D_OK( xResult);	
		
		xResult = Asura_Direct3D::D3DDevice->SetStreamSource( 1, pxVertexBufferInteractive, 0, sizeof( Asura_Direct3D_BGR_Type_VertexGeometryImposterInteractive));
		ASURA_D3D_OK( xResult);	
		// DLH - 21 November 2006 11:34:31 - I wanted this to be (1,4) but it turns out D3D only supports this in software VP mode when using DrawPrimitive (DrawIndexPrimitive does not work).
		// DLH - 21 November 2006 11:35:28 - So I have had to place 4 bone enties into the stream per instance to match the corner vertices.
		xResult = Asura_Direct3D::D3DDevice->SetStreamSourceFreq( 1, 1);
		ASURA_D3D_OK( xResult);	
	}
	else
#endif
	{
		xResult = pxD3DDevice->SetVertexDeclaration(this->m_pxVertexDeclaration_Imposter);
		ASURA_D3D_OK( xResult);	
		
		xResult = Asura_Direct3D::D3DDevice->SetStreamSource( 1, NULL, 0, 0);
		ASURA_D3D_OK( xResult);	
		xResult = Asura_Direct3D::D3DDevice->SetStreamSourceFreq( 1, 1);
		ASURA_D3D_OK( xResult);	
	}
	
	xResult = Asura_Direct3D::D3DDevice->SetStreamSource( 0, pxVertexBuffer, 0, sizeof( Asura_Direct3D_BGR_Type_VertexGeometryImposter));
	ASURA_D3D_OK( xResult);	
	xResult = Asura_Direct3D::D3DDevice->SetStreamSourceFreq( 0, 1);
	ASURA_D3D_OK( xResult);	
	
	//-- Setup render states.
	
	// DLH - 07 April 2006 15:52:09 - Setup the Render States

	{
		Asura_Direct3D_BGR_Manager_DrawTree_SM30_EffectSetupCamera( pxD3DEffect);
		
		#define ASURA_DIRECT3D_BGR_VS_CONSTANT_IMPOSTER_SIZE 18
		struct Asura_Direct3D_BGR_ImposterConstants
		{
			Asura_Direct3D_BGR_ImposterConstants() { ASURA_STATIC_ASSERT( sizeof( m_afConstantData) == sizeof( m_xData));}
			union
			{
				// DLH - 30 March 2006 11:09:58 - Do not use this variable to set Data, EVER
				float m_afConstantData[ ASURA_DIRECT3D_BGR_VS_CONSTANT_IMPOSTER_SIZE][4];
				
				// DLH - 30 March 2006 11:09:58 - USE THESE to set the Data
				struct
				{
					float m_afAlphaFade[4];
					float m_afAlphaRotate[4];
					float m_afXY[4][4];	// DLH - X, Y, Bone Weight, ???
					float m_afUV[ uSignificantImposterMaxCount][4][4];
					float m_xTangent[2][4];
					float m_xBinormal[2][4];
				}m_xData;
			};
		};

		Asura_Direct3D_BGR_ImposterConstants xImposterConstants;
		
		memset( &xImposterConstants, 0, sizeof( xImposterConstants));
		
		{
			const float fDeltaL0 = xGeometryImposter.m_xLodRange.m_afDelta[ Asura_Direct3D_BGR_Type_GeometryLODRange::LOW_0];
			const float fDeltaL1 = xGeometryImposter.m_xLodRange.m_afDelta[ Asura_Direct3D_BGR_Type_GeometryLODRange::LOW_1];
			const float fDistanceL0 = (xTreeDescription.m_fLodLimitNear + (xTreeDescription.m_fLodLimitDelta * fDeltaL0)) / fScreenDistanceScale;
			const float fDistanceL1 = (xTreeDescription.m_fLodLimitNear + (xTreeDescription.m_fLodLimitDelta * fDeltaL1)) / fScreenDistanceScale;

			const float fDistanceDelta = (fDistanceL1 != fDistanceL0) ? fDistanceL1 - fDistanceL0 : 1e-3f;
			const float fGradient = -0.5f / (fDistanceDelta);
			
			xImposterConstants.m_xData.m_afAlphaFade[0] = fGradient;
			// DLH - 18 April 2006 17:11:31 - Alpha Fade Imposter 0
			xImposterConstants.m_xData.m_afAlphaFade[1] = (-fDistanceL1 * fGradient) + 0.5f;
			// DLH - 18 April 2006 17:11:31 - Alpha Fade Imposter 1
			xImposterConstants.m_xData.m_afAlphaFade[2] = (-fDistanceL1 * fGradient) + 0.5f;
			xImposterConstants.m_xData.m_afAlphaFade[3] = 0;
		}
		
		if( NULL != pxImposterType0)
		{
			const float fRadius = 0.5f * pxImposterType0->m_fWidth;
			const float fHeight = pxImposterType0->m_fHeight;
			
			xImposterConstants.m_xData.m_afXY[ 0][ 0] = -fRadius;
			xImposterConstants.m_xData.m_afXY[ 0][ 1] = -fHeight;
			xImposterConstants.m_xData.m_afXY[ 0][ 2] = 1.0f;
			xImposterConstants.m_xData.m_afXY[ 1][ 0] = -fRadius;
			xImposterConstants.m_xData.m_afXY[ 1][ 1] = 0.0f;
			xImposterConstants.m_xData.m_afXY[ 1][ 2] = 0.0f;
			xImposterConstants.m_xData.m_afXY[ 2][ 0] = +fRadius;
			xImposterConstants.m_xData.m_afXY[ 2][ 1] = -fHeight;
			xImposterConstants.m_xData.m_afXY[ 2][ 2] = 1.0f;
			xImposterConstants.m_xData.m_afXY[ 3][ 0] = +fRadius;
			xImposterConstants.m_xData.m_afXY[ 3][ 1] = 0.0f;
			xImposterConstants.m_xData.m_afXY[ 3][ 2] = 0.0f;
		}
		
		u_int uSignificantImposterMaterialsCount = 0;
		Asura_Direct3D_MaterialResponse_ShaderData xMaterialResponseShaderData;
		
		xMaterialResponseShaderData.SetupCommon();
		
		for( uint32 uSignificantImposterIndex = 0; uSignificantImposterIndex < uSignificantImposterMaxCount; uSignificantImposterIndex += 1)
		{
			const Asura_Direct3D_BGR_Type_VertexGeometryImposterStorage* pxImposterType = axMSImposters[ uSignificantImposterIndex].m_pxImposterType;
			
			if( NULL == pxImposterType)
			{
				xImposterConstants.m_xData.m_afUV[ uSignificantImposterIndex][ 0][ 0] = 0.5f;
				xImposterConstants.m_xData.m_afUV[ uSignificantImposterIndex][ 0][ 1] = 0.5f;
				xImposterConstants.m_xData.m_afUV[ uSignificantImposterIndex][ 0][ 2] = 0.5f;
				xImposterConstants.m_xData.m_afUV[ uSignificantImposterIndex][ 0][ 3] = 0.5f;
				
				xImposterConstants.m_xData.m_afUV[ uSignificantImposterIndex][ 1][ 0] = 0.5f;
				xImposterConstants.m_xData.m_afUV[ uSignificantImposterIndex][ 1][ 1] = 0.5f;
				xImposterConstants.m_xData.m_afUV[ uSignificantImposterIndex][ 1][ 2] = 0.5f;
				xImposterConstants.m_xData.m_afUV[ uSignificantImposterIndex][ 1][ 3] = 0.5f;
				
				xImposterConstants.m_xData.m_afUV[ uSignificantImposterIndex][ 2][ 0] = 0.5f;
				xImposterConstants.m_xData.m_afUV[ uSignificantImposterIndex][ 2][ 1] = 0.5f;
				xImposterConstants.m_xData.m_afUV[ uSignificantImposterIndex][ 2][ 2] = 0.5f;
				xImposterConstants.m_xData.m_afUV[ uSignificantImposterIndex][ 2][ 3] = 0.5f;
				
				xImposterConstants.m_xData.m_afUV[ uSignificantImposterIndex][ 3][ 0] = 0.5f;
				xImposterConstants.m_xData.m_afUV[ uSignificantImposterIndex][ 3][ 1] = 0.5f;
				xImposterConstants.m_xData.m_afUV[ uSignificantImposterIndex][ 3][ 2] = 0.5f;
				xImposterConstants.m_xData.m_afUV[ uSignificantImposterIndex][ 3][ 3] = 0.5f;

				xImposterConstants.m_xData.m_afAlphaFade[2 + uSignificantImposterIndex] = 1.0f;
				
				Asura_Render::CurrentStates.RequestTextureIndex( uSignificantImposterIndex, ASURA_TEXTURE_INVALID);
				continue;
			}
			
			float fUMin = pxImposterType->m_afTextcoord0[ 0];
			float fVMin = pxImposterType->m_afTextcoord0[ 1];
			float fUMax = pxImposterType->m_afTextcoord1[ 0];
			float fVMax = pxImposterType->m_afTextcoord1[ 1];
			
			xImposterConstants.m_xData.m_afUV[ uSignificantImposterIndex][ 0][ 0] = fUMin;
			xImposterConstants.m_xData.m_afUV[ uSignificantImposterIndex][ 0][ 1] = fVMin;
			xImposterConstants.m_xData.m_afUV[ uSignificantImposterIndex][ 0][ 2] = 1.0f;
			xImposterConstants.m_xData.m_afUV[ uSignificantImposterIndex][ 0][ 3] = 1.0f;
			
			xImposterConstants.m_xData.m_afUV[ uSignificantImposterIndex][ 1][ 0] = fUMin;
			xImposterConstants.m_xData.m_afUV[ uSignificantImposterIndex][ 1][ 1] = fVMax;
			xImposterConstants.m_xData.m_afUV[ uSignificantImposterIndex][ 1][ 2] = 1.0f;
			xImposterConstants.m_xData.m_afUV[ uSignificantImposterIndex][ 1][ 3] = 1.0f;
			
			xImposterConstants.m_xData.m_afUV[ uSignificantImposterIndex][ 2][ 0] = fUMax;
			xImposterConstants.m_xData.m_afUV[ uSignificantImposterIndex][ 2][ 1] = fVMin;
			xImposterConstants.m_xData.m_afUV[ uSignificantImposterIndex][ 2][ 2] = 1.0f;
			xImposterConstants.m_xData.m_afUV[ uSignificantImposterIndex][ 2][ 3] = 1.0f;
			
			xImposterConstants.m_xData.m_afUV[ uSignificantImposterIndex][ 3][ 0] = fUMax;
			xImposterConstants.m_xData.m_afUV[ uSignificantImposterIndex][ 3][ 1] = fVMax;
			xImposterConstants.m_xData.m_afUV[ uSignificantImposterIndex][ 3][ 2] = 1.0f;
			xImposterConstants.m_xData.m_afUV[ uSignificantImposterIndex][ 3][ 3] = 1.0f;
			
			{
				Asura_Vector_3 xTangent( ((fUMin < fUMax) ? +1.0f : -1.0f) , 0.0f, 0.0f);
				
				xImposterConstants.m_xData.m_xTangent[ uSignificantImposterIndex][ 0] = xTangent.GetX();
				xImposterConstants.m_xData.m_xTangent[ uSignificantImposterIndex][ 1] = xTangent.GetY();
				xImposterConstants.m_xData.m_xTangent[ uSignificantImposterIndex][ 2] = xTangent.GetZ();
				xImposterConstants.m_xData.m_xTangent[ uSignificantImposterIndex][ 3] = 1.0f;
			}
			
			{	
				Asura_Vector_3 xBinormal( 0.0f, ((fVMin < fVMax) ? +1.0f : -1.0f) , 0.0f);
				
				xImposterConstants.m_xData.m_xBinormal[ uSignificantImposterIndex][ 0] = xBinormal.GetX();
				xImposterConstants.m_xData.m_xBinormal[ uSignificantImposterIndex][ 1] = xBinormal.GetY();
				xImposterConstants.m_xData.m_xBinormal[ uSignificantImposterIndex][ 2] = xBinormal.GetZ();
				xImposterConstants.m_xData.m_xBinormal[ uSignificantImposterIndex][ 3] = 1.0f;
			}
			
			xImposterConstants.m_xData.m_afAlphaFade[2 + uSignificantImposterIndex] = axMSImposters[ uSignificantImposterIndex].m_fAlphaFade;
			
			Asura_Hash_ID xMaterialResponseHashID = pxImposterType->m_uMaterialResponseHashID;
			const Asura_MaterialResponse* pxMaterialResponse = Asura_MaterialResponse_System::GetMaterialByHashID( xMaterialResponseHashID);
			if( NULL != pxMaterialResponse)
			{
				if( 0 == uSignificantImposterMaterialsCount)
				{
					xMaterialResponseShaderData.Setup_SingleMapsOnly( pxMaterialResponse, NULL);
				}
				else
				{
#define SETUP_TEXTURE_SECOND( SRC_TEXTURE1, DST_TEXTURE1, DST_DUAL) \
{	\
	const int iTexture1 = SRC_TEXTURE1;	\
	\
	DST_DUAL = (ASURA_TEXTURE_NONE != iTexture1);	\
	\
	Direct3D_BaseTexture* pxTexture1 = Asura_Direct3D::GetDirect3DTextureByIndex( iTexture1); \
	DST_TEXTURE1 = pxTexture1;	\
}

					SETUP_TEXTURE_SECOND( pxMaterialResponse->GetTexture( Asura_MaterialResponse::ALBEDO_TEXTURE ), xMaterialResponseShaderData.m_pxAlbedoMap2, xMaterialResponseShaderData.m_bDualAlbedo);
					SETUP_TEXTURE_SECOND( pxMaterialResponse->GetTexture( Asura_MaterialResponse::NORMAL_MAP ), xMaterialResponseShaderData.m_pxNormalMap2, xMaterialResponseShaderData.m_bDualNormalMap);
					SETUP_TEXTURE_SECOND( pxMaterialResponse->GetTexture( Asura_MaterialResponse::SPECULAR_MAP ), xMaterialResponseShaderData.m_pxSpecularMap2, xMaterialResponseShaderData.m_bDualSpecularMap);
					SETUP_TEXTURE_SECOND( pxMaterialResponse->GetTexture( Asura_MaterialResponse::TRANSPARENCY_MAP ), xMaterialResponseShaderData.m_pxTransparencyMap2, xMaterialResponseShaderData.m_bDualTransparencyMap);

					#if defined ASURA_TOOLS
						const bool bEnableTexturing = Tools_Render::TexturingIsOn();
						xMaterialResponseShaderData.m_bDualAlbedo &= bEnableTexturing;
					#endif

				}			
				uSignificantImposterMaterialsCount += 1;
			}
		}
		
		{
			const float fRotateFade0 =  0.5f - xImposterConstants.m_xData.m_afAlphaFade[2];
			const float fRotateFade1 =  0.5f - xImposterConstants.m_xData.m_afAlphaFade[3];
			const float fRotateFadeTotal = fRotateFade0 + fRotateFade1;
			const float fRotateFade = (0 != fRotateFadeTotal) ? (1.0f - (fRotateFade0 / fRotateFadeTotal)) : 0.0f;
			xImposterConstants.m_xData.m_afAlphaRotate[0] = fRotateFade;
		}

		pxD3DEffect->SetParameterByName( "g_xImposterData", &xImposterConstants, sizeof( xImposterConstants));
		
		xMaterialResponseShaderData.SetParameterByName( pxD3DEffect);
	}
	
	Asura_Render::CurrentStates.RequestCullMode( ASURA_CULLMODE_NONE);
	Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_OFF);
	Asura_Render::CurrentStates.RequestZBufferWriteEnabled( true);
	
	#if 1 && defined( ASURA_TOOLS)
		if (Tools_Render_Options::GetWireframeMode() != TOOLS_WIREFRAME_OFF)
		{
			Asura_Render::CurrentStates.RequestTextureOpMode(ASURA_TEXTUREOP_WHITE);
			Asura_Render::CurrentStates.RequestWireFrameMode( true);
		}
	#endif
	
	pxD3DEffect->BeginPass( 0);

	return( true);
#else
	return( true);
#endif
}
#endif

#if defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS)
bool Asura_Direct3D_BGR_Manager::DrawTreeBillboardDynamicRenderStates( const Asura_Photon& xPhoton, Asura_Direct3D_Effect*& pxD3DEffect)
{
	SCOPEDPIXEVENT((0xff00ff00, L"SM30_DrawTreeBillboardDynamicRenderStates"));

	return( false);
}
#endif

#if defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS)
bool Asura_Direct3D_BGR_Manager::DrawTreeBillboardInstancesDIPCacheRenderStates_Start( const Asura_Direct3D_BGR_Type& xTreeDescription, const float fScreenDistanceScale, const Asura_Direct3D_BGR_Type_Geometry &xGeometry, Asura_Direct3D_Effect*& pxD3DEffect)
{
	if( false == this->DrawTreeBillboardInstancesDIPCacheRenderStates_Start_All( xTreeDescription, fScreenDistanceScale, xGeometry, pxD3DEffect))
	{
		return( false);
	}
	
	if( false == this->DrawTreeBillboardInstancesDIPCacheRenderStates_Start_IB( xTreeDescription, fScreenDistanceScale, xGeometry, pxD3DEffect))
	{
		return( false);
	}
	
	return( true);
}
#endif

#if defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS)
bool Asura_Direct3D_BGR_Manager::DrawTreeBillboardInstancesDIPCache( const Asura_DIPCache_DIPData& xDIPData, const uint32 uCallbackData)
{
	SCOPEDPIXEVENT((0xff00ff00, L"SM30_DrawTreeBillboardInstancesDIPCache"));

#if 1 && defined( ASURA_DIRECT3D_BGR_ENABLE_D3D_CALLS)
	HRESULT xResult;

	{
		Direct3D_VertexBuffer* pxVertexBuffer = NULL;
		UINT uOffset = 0;
		UINT uStride = 0;
		xResult = Asura_Direct3D::D3DDevice->GetStreamSource( 0, &pxVertexBuffer, &uOffset, &uStride);
		ASURA_D3D_OK( xResult);	
		
		const UINT uOffsetNew = uStride * xDIPData.GetVertexBufferOffset();
		if( uOffsetNew != uOffset)
		{
			xResult = Asura_Direct3D::D3DDevice->SetStreamSource( 0, pxVertexBuffer, uOffsetNew, uStride);
			ASURA_D3D_OK( xResult);	
		}
		
		SAFE_RELEASE( pxVertexBuffer);
	}

	{
		Direct3D_VertexBuffer* pxVertexBuffer = NULL;
		UINT uOffset = 0;
		UINT uStride = 0;
		xResult = Asura_Direct3D::D3DDevice->GetStreamSource( 1, &pxVertexBuffer, &uOffset, &uStride);
		ASURA_D3D_OK( xResult);	
		
		const UINT uOffsetNew = uStride * xDIPData.GetVertexBufferOffset();
		if( uOffsetNew != uOffset)
		{
			xResult = Asura_Direct3D::D3DDevice->SetStreamSource( 1, pxVertexBuffer, uOffsetNew, uStride);
			ASURA_D3D_OK( xResult);	
		}
		
		SAFE_RELEASE( pxVertexBuffer);
	}
//	const static uint32 uIndexMaxCount = Asura_Direct3D_Carpet_Game_Render::GetBatchIndexBufferSize();
	const uint32 uIndexMaxCount = 3 * uint32(0x10000 / 3);
	
	uint32 uIndexBufferOffset = xDIPData.GetIndexBufferStart();
	uint32 uIndexTotalCount = xDIPData.GetIndexBufferStop() - xDIPData.GetIndexBufferStart();
	const uint32 uVertexBufferMin = xDIPData.GetVertexBufferStart();
	const uint32 uVertexBufferCount = xDIPData.GetVertexBufferStop() - uVertexBufferMin;
	while( 0 < uIndexTotalCount)
	{
		const uint32 uIndexCount = Asura_Maths::Min( uIndexMaxCount, uIndexTotalCount);
		const uint32 uCountPrimitives = uIndexCount / 3;
		const uint32 uVertexCount = 2 * uCountPrimitives;
		(void)uVertexCount;
		xResult = Asura_Direct3D::D3DDevice->DrawIndexedPrimitive( 
			D3DPT_TRIANGLELIST, 
			0, uVertexBufferMin, uVertexBufferCount, 
			uIndexBufferOffset, uCountPrimitives);
//			xDIPData.GetVertexBufferStart() + uVertexBufferOffset, 0, uVertexCount, 
//			0, uCountPrimitives);
		if( ASURA_D3D_FAILED( xResult))
		{
			return( false);
		}
		
		#if defined( ASURA_TOOLS)
			TOOLS_BOTANICAL_STATISTICS_ADD( s_uDIPCount, 1);
			TOOLS_BOTANICAL_STATISTICS_ADD( s_uDIPImposterCount, 1);
			TOOLS_BOTANICAL_STATISTICS_ADD( s_uDIPIndexCount, uIndexCount);
			TOOLS_BOTANICAL_STATISTICS_ADD( s_uDIPVertexCount, uVertexCount);
			TOOLS_BOTANICAL_STATISTICS_ADD( s_uDIPPrimitiveCount, uCountPrimitives);
			TOOLS_BOTANICAL_STATISTICS_ADD( s_uDIPImposterPrimitiveCount, uCountPrimitives);
		#endif
		
		//Asura_Render::AddToTrianglesRendered()
		
		uIndexTotalCount -= uIndexCount;
		uIndexBufferOffset += uIndexCount;
		//uVertexBufferOffset += uVertexCount;
	}
#endif

	return( true);
}
#endif

#if defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS)
bool Asura_Direct3D_BGR_Manager::DrawTree_SM30_ImposterRenderStates( const Asura_Direct3D_BGR_Type& xTreeDescription, const Asura_Direct3D_BGR_Type_Geometry &xGeometry, const void* pxDataPointer, Asura_Direct3D_Effect*& pxD3DEffect)
{
	float fScreenDistanceScale = 1.0f;
	if( false == this->DrawTreeBillboardInstancesDIPCacheRenderStates_Start_All( xTreeDescription, fScreenDistanceScale, xGeometry, pxD3DEffect))
	{
		return( false);
	}
	
	return( true);
}
#endif

#if defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS)
uint32 Asura_Direct3D_BGR_Manager::DrawTree_SM30_ImposterInstances( const Asura_Direct3D_BGR_Type& xTreeDescription, const Asura_Direct3D_BGR_Type_Geometry &geometry, Asura_Direct3D_BGR_GeometryIndexInstanceCache& xCacheGeometryInstances, const void* pxDataPointer, Asura_Direct3D_Effect* pxD3DEffect)
{
	Direct3D_IndexBuffer* pxIndexBuffer = NULL;
	u_int uIndexBufferOffset = 0;
	xCacheGeometryInstances.GetIndexBuffer( pxIndexBuffer, uIndexBufferOffset);
	if( NULL == pxIndexBuffer)
	{
		return( false);
	}
	
#if 1 && defined( ASURA_DIRECT3D_BGR_ENABLE_D3D_CALLS)
	HRESULT xResult;
	
	xResult = Asura_Direct3D::D3DDevice->SetIndices( pxIndexBuffer);
	ASURA_D3D_OK( xResult);	

	xCacheGeometryInstances.m_xDIPCache.SetCallback( DrawTreeBillboardInstancesDIPCache, 0);
	xCacheGeometryInstances.m_xDIPCache.Render();
	
	if( false == this->DrawTreeBillboardInstancesDIPCacheRenderStates_End( pxD3DEffect))
	{
		return( false);
	}
	
#endif

	return( 0);
}
#endif


//*******************************************************************
//
// (c) 2006 Rebellion (Derby)
//
// Warning: This computer program is protected by copyright law and 
// international treaties. Unauthorized reproduction or distribution
// of this program, or any portion of it, may result in severe civil and
// criminal penalties, and will be prosecuted to the maximum extent 
// possible under the law.
//
//*******************************************************************

#endif 
#endif //defined( ASURA_USE_BOTANICALS)
