//
// Asura_Chunk_ResourceFile_PC_Character.cpp
//
// Created by KJL - 13/04/2003 15:49:04
//
// Copyright (C) 2003 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

#if !defined(ASURA_PC_DISABLE_PRESM3_SMOOTH_SKIN_RENDERER) || !ASURA_PC_DISABLE_PRESM3_SMOOTH_SKIN_RENDERER

// This file's header.
#include "Asura_Chunk_ResourceFile_PC_Character.h"

#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "Core/Asura_Material_System.h"
#include "FileAccess/Asura_FileAccess.h"

#if !defined(ASURA_PC_DISABLE_PRESM3_SMOOTH_SKIN_RENDERER) || !ASURA_PC_DISABLE_PRESM3_SMOOTH_SKIN_RENDERER
#include "Rendering/Asura_PlatformRenderCharacter.h"
#endif
#include "Rendering/Asura_Direct3D.h"
#include "Rendering/Asura_Direct3D_Caps.h"

#include "Core/Memory/Asura_ResourceAllocator.h"

#ifdef ASURA_TOOLS
#include "Debugging/Asura_Logfile.h"
#include "Mesh/Asura_Mesh_System.h"
#endif

bool Asura_Chunk_ResourceFile_PC_Character::Process( Asura_Handle hFile )
{
	Asura_Chunk_File_Stream xStream(hFile);
	bool bWarnIfMeshExists = true;
	bool bConvertMaterialIDs = true;
	return ProcessFromChunkStream( xStream, bWarnIfMeshExists, bConvertMaterialIDs ) != NULL;
}

Asura_Direct3D_Character_Strip* 
Asura_Chunk_ResourceFile_PC_Character::ProcessFromChunkStream( Asura_Chunk_Stream& xStream, 
															    bool bWarnIfMeshExists, 
																bool bConvertMaterialIDs ) const
{
	Asura_Direct3D_CharacterRender_Handler&	xHandler	= Asura_Direct3D_CharacterRenderer::GetCurrentHandler();
	bool bSupportsMultipleMaterials = (this->Version > 0);

	Asura_Direct3D_Character_Strip* pxStrip = NULL;
	Asura_Direct3D_Character_Strip* pxFirstStrip = NULL;
	Asura_Direct3D_Character_Strip* pxParentStrip = NULL;

	// (AAM - 11.09.2007) Can't use whether pxFirstStrip is non-NULL to determine whether we've read
	// the vertex buffer, because if the first strip allocation fails we'll skip over the vertices
	// but we won't record a first strip pointer. That makes subsequent strips skip over what they
	// think is the vertex data again, and the whole file gets screwed
	bool bReadVertices = false;

	// In a multi-material skin, only the first strip actually has vertex data
	// To easily cover both single- and multi-material skins, we'll load the vertex data
	// into a local buffer and then assign that to the relevant strip at the end.
	Direct3D_VertexBuffer* pxVertices = NULL;

	Asura_String xTempString;
	xStream >> xTempString;

	const Asura_Char* szName = xTempString.GetString();

#ifdef ASURA_TOOLS
	//Warn if a mesh already exists with the same name RAR 23/06/2005
	if (bWarnIfMeshExists)
	{
		Asura_Mesh* pxMesh = Asura_Mesh_System::GetMesh(szName);
		if(pxMesh)
		{
			Asura_SimpleLogfileOutput((ASURA_FILE_CONTENTS_LOGFILE,"Stripified Skin : %s - Mesh with same name has been loaded alrady!\n",szName));
			Asura_Assertf(0,("Found a mesh named '%s' while loading a stripified skin with the same name.\nIf the mesh is used by the environment or a shape file , then it will need to be renamed.\nCheck Logfiles/file_contents.txt to find which file the mesh came from.",szName));
		}
	}
#endif

	const bool bHasNormalMapData = (Flags & uASURA_CHUNK_PC_CHARACTER_FLAG_HASNORMALMAPDATA) != 0;
	const bool bTriList = (Flags & uASURA_CHUNK_PC_CHARACTER_FLAG_TRILIST) != 0;

	Asura_Assert(!bHasNormalMapData, "Old normal-mapped skins are no longer supported");

	while(1)
	{
		if(pxParentStrip)
		{
			pxStrip = new Asura_Direct3D_Character_Strip();
			pxParentStrip->m_pxNextStrip = pxStrip;
		}
		else
		{
			pxStrip = xHandler.AddCharacterStrip(szName);
			pxFirstStrip = pxStrip;
		}
		szName = "";

		if (pxStrip)
		{
			u_int uNumberOfTriangles;
			u_int uNumberOfVertices;
			u_int uNumberOfIndices;
			u_int uMaterialIndex;
			int iElementID = 0;
		
			xStream.Read(sizeof(u_int), &uNumberOfTriangles);
			xStream.Read(sizeof(u_int), &uNumberOfVertices);
			xStream.Read(sizeof(u_int), &uNumberOfIndices);
			xStream.Read(sizeof(u_int), &uMaterialIndex);	
			if (this->Version >= 4)
			{
				xStream.Read(sizeof(int), &iElementID);
			}
		
			pxStrip->m_uNumberOfTriangles = uNumberOfTriangles;
			pxStrip->m_uNumberOfVertices = uNumberOfVertices;
			pxStrip->m_uNumberOfIndices = uNumberOfIndices;
			pxStrip->m_iMaterialIndex = bConvertMaterialIDs ? Asura_Material_System::ConvertIndex(uMaterialIndex) : static_cast<int>(uMaterialIndex);
			pxStrip->m_iElementID = iElementID; // (SB - 04/12/06) Read element ID
			pxStrip->m_ucRenderBoneCount = 0;
			pxStrip->m_aucRenderBones = 0;

			if(bTriList)
			{
				pxStrip->m_ePrimitiveType = D3DPT_TRIANGLELIST;
			}

			int flags = D3DUSAGE_WRITEONLY;
			
  			// (MH - 26.05.2006) this is to preserve old behaviour, I'm not sure why, but
			// the software processing flag was always on for non-normal-mapped characters.
			if ( (!bHasNormalMapData) || (!Asura_Direct3D_Caps::HasHardwareVertexShaders()) )
  			{
				flags |= D3DUSAGE_SOFTWAREPROCESSING;
  			}

			// Only the first strip has vertex data included
			if(!bReadVertices)
			{
				u_int uSizeOfVertexData = bHasNormalMapData ? 
					(uNumberOfVertices * sizeof(Asura_D3D_CharacterVertex_NormalMapped)) : 
					(uNumberOfVertices * sizeof(Asura_Direct3D_CharacterVertex));

				// Create vertex and index buffers for the strip
				Asura_Direct3D::D3DDevice->CreateVertexBuffer
				(
					uSizeOfVertexData,
					flags,
					0, // D3DFVF_CHARACTERVERTEX,
					D3DPOOL_MANAGED,
					&pxVertices,
					0
				);

				// lock, fill in data and unlock
				void* pVertexData;
				if (SUCCEEDED(pxVertices->Lock(0,uSizeOfVertexData,&pVertexData,0)))
				{
					xStream.Read(uSizeOfVertexData, pVertexData);
					bReadVertices = true;

					pxVertices->Unlock();
				} else {
					Asura_Assert(false,	"Failed to lock vertex buffer");
					xStream.Skip(uSizeOfVertexData);
				}



				// VGP 20Mar2006 Read in render bone indices
				if (this->Version >= 2)
				{				
					u_char ucRenderBoneCount = 0;
					xStream.Read(sizeof(u_char), &ucRenderBoneCount);
					
					// (MH - 01.06.2006) normal-mapped character renderer doesn't have render bone indices.
					if (bHasNormalMapData)
					{
						xStream.Skip(sizeof(u_char) * ucRenderBoneCount);
					}
					else
					{
						pxStrip->m_ucRenderBoneCount = ucRenderBoneCount;

						if (ucRenderBoneCount)
						{
							Asura_ResourcePushType(ASURA_RESOURCE_SKINNED_MESH);

							Asura_ResourceAllocateArray(pxStrip->m_aucRenderBones, ucRenderBoneCount);
							xStream.Read(sizeof(u_char) * ucRenderBoneCount, pxStrip->m_aucRenderBones);

							Asura_ResourcePopType();
						}
					}
				}
			}

			// All strips have index buffers
			Asura_Direct3D::D3DDevice->CreateIndexBuffer
			(
				uNumberOfIndices*sizeof(unsigned short),
				flags,
				D3DFMT_INDEX16,
				D3DPOOL_MANAGED,
				&pxStrip->m_pxIndexBuffer,
				0
			);

			unsigned short* pusIndexData;
			if (SUCCEEDED(pxStrip->m_pxIndexBuffer->Lock(0,sizeof(u_short)*uNumberOfIndices, (void**)&pusIndexData,0)))
			{
				xStream.Read(sizeof(u_short)*uNumberOfIndices, pusIndexData);
					
				pxStrip->m_pxIndexBuffer->Unlock();
			} else {
				Asura_Assert(false,	"Failed to lock Index buffer");
				xStream.Skip(sizeof(u_short)*uNumberOfIndices);
			}
		} // pxStrip
		else
		{
			// KJL - 16/04/2003 15:17:06 - either we've ran out of strip space or the character already existed
			// either way, let's skip the data

			u_int uNumberOfTriangles;
			u_int uNumberOfVertices;
			u_int uNumberOfIndices;
			u_int uMaterialIndex;
			int iElementID;
			
			xStream.Read(sizeof(u_int), &uNumberOfTriangles);
			xStream.Read(sizeof(u_int), &uNumberOfVertices);
			xStream.Read(sizeof(u_int), &uNumberOfIndices);
			xStream.Read(sizeof(u_int), &uMaterialIndex);
			if (this->Version >= 4)
			{
				xStream.Read(sizeof(int), &iElementID); // (SB - 12/11/06 ) Dummy read element ID
			}
			
			if(!bReadVertices)
			{
				u_int uSizeOfVertexData = bHasNormalMapData ? 
					(uNumberOfVertices * sizeof(Asura_D3D_CharacterVertex_NormalMapped)) : 
					(uNumberOfVertices * sizeof(Asura_Direct3D_CharacterVertex));

				// If we failed to create the strip, and it would have been the first strip, we need to skip vertex data
				xStream.Skip(uSizeOfVertexData);
				bReadVertices = true;

				// VGP 20Mar2006 Skip render bone indices
				if (this->Version >= 2)
				{				
					u_char ucRenderBoneCount = 0;
					xStream.Read(sizeof(u_char), &ucRenderBoneCount);

					if (ucRenderBoneCount)
					{
						xStream.Skip(sizeof(u_char) * ucRenderBoneCount);
					}
				}
			}
			xStream.Skip(sizeof(u_short)*uNumberOfIndices);
		}

		if(!bSupportsMultipleMaterials) break;

		// Is there another strip to come?
		u_char ucMoreStrips;
		xStream.Read(sizeof(u_char), &ucMoreStrips);
		if(!ucMoreStrips) break;

		pxParentStrip = pxStrip;
	} // !pxStrip

	if(pxFirstStrip)
	{
		pxFirstStrip->m_pxVertexBuffer = pxVertices;
	}

	return pxFirstStrip;
}

#endif //!defined(ASURA_PC_DISABLE_PRESM3_SMOOTH_SKIN_RENDERER) || !ASURA_PC_DISABLE_PRESM3_SMOOTH_SKIN_RENDERER
