//
// Asura_Chunk_ResourceFile_PC_Environment_SM3.cpp
//
// Created by RJF - 09 August 2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

#if defined( ASURA_TARGET_SM3)

// This file's header.
#include "Asura_Chunk_ResourceFile_PC_Environment_SM3.h"

#include "Rendering/Asura_Direct3D_EnvironmentRenderer.h"
#include "Rendering/Asura_Direct3D_IndexBuffer_Dynamic.h"

#include "FileAccess/Asura_FileAccess.h"
#include "Core/Memory/Asura_ResourceAllocator.h"
#include "Core/Asura_Material_System.h"

bool Asura_Chunk_ResourceFile_PC_Environment_SM3::Process(Asura_Handle hFile)
{
	return Process(hFile, true);
}

bool Asura_Chunk_ResourceFile_PC_Environment_SM3::Process( Asura_Handle hFile, bool bConvertMaterialIDs )
{
	HRESULT xResult;
	
	bool bAnyErrors = false;

	if(Version != ASURA_CHUNK_RESOURCEFILE_PC_ENVIRONMENT_SM3_VERSION)
	{
		Asura_Assertf(0,("Incorrect version of environment resource chunk.n\nVersion is %d , current version is %d",Version,ASURA_CHUNK_RESOURCEFILE_PC_ENVIRONMENT_SM3_VERSION));
		return false;
	}

	const bool bMaterialResponse = (Flags & ChunkFlag_MaterialResponse) != 0;
	const bool bUsingTriLists = (Flags & ChunkFlag_UseTriLists) != 0;

	#if defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM
	if(!bMaterialResponse)
	{
		Asura_Assert(0,"Attempting to load a non material response environment into a material response build.\nThis will be skipped");
		return false;
	}
	#else
	if(bMaterialResponse)
	{
		Asura_Assert(0,"Attempting to load a material response environment into a non material response build.\nThis will be skipped");
		return false;
	}
	#endif


	Asura_ResourcePushType(ASURA_RESOURCE_ENVIRONMENT_MESH);
	{
		// Vertex buffers
		// Just do one per strip for now. Eventually we should look at coalescing buffers together, but it will need to be done at load-time
		// because the ideal buffer size is system-dependent (available video memory, available AGP memory, bus speed, etc).
		
		u_int uNumberOfVBs;
		Asura_FileAccess::ReadData(hFile, &uNumberOfVBs, sizeof(u_int));
		
		Asura_Direct3D_EnvironmentRenderer::s_uTotalNumberOfVertexBuffers = uNumberOfVBs;
		Asura_ResourceAllocateArray( Asura_Direct3D_EnvironmentRenderer::s_ppxVertexBuffer, Asura_Direct3D_EnvironmentRenderer::s_uTotalNumberOfVertexBuffers);
		memset( Asura_Direct3D_EnvironmentRenderer::s_ppxVertexBuffer, 0, sizeof(Direct3D_VertexBuffer*) * Asura_Direct3D_EnvironmentRenderer::s_uTotalNumberOfVertexBuffers);

		for( u_int uVBIndex = 0; uVBIndex < uNumberOfVBs; uVBIndex += 1)
		{
			uint32 uVBSize;
			
			Asura_FileAccess::ReadData( hFile, &uVBSize, sizeof( uVBSize));
			
			Direct3D_VertexBuffer*& pxBuffer = Asura_Direct3D_EnvironmentRenderer::s_ppxVertexBuffer[ uVBIndex];

			xResult = Asura_Direct3D::D3DDevice->CreateVertexBuffer( sizeof(Asura_Direct3D_EnvironmentRenderer_Vertex) * uVBSize, D3DUSAGE_WRITEONLY, 0, D3DPOOL_MANAGED, &pxBuffer, NULL);
			ASURA_D3D_OK( xResult);
		}

		// Index buffers
		// Again, we eventually want to coalesce these, but we'll do one per strip for now
		
		u_int uNumberOfIBs;
		Asura_FileAccess::ReadData(hFile, &uNumberOfIBs, sizeof(u_int));
		
		Asura_Direct3D_EnvironmentRenderer::s_uTotalNumberOfIndexBuffers = uNumberOfIBs;
		Asura_ResourceAllocateArray( Asura_Direct3D_EnvironmentRenderer::s_ppxIndexBuffer, Asura_Direct3D_EnvironmentRenderer::s_uTotalNumberOfIndexBuffers);
		memset( Asura_Direct3D_EnvironmentRenderer::s_ppxIndexBuffer, 0, sizeof(Direct3D_IndexBuffer*) * Asura_Direct3D_EnvironmentRenderer::s_uTotalNumberOfIndexBuffers);
		
		// DLH - Alloc storage for Memory Copy of index data
		Asura_Assert( NULL == Asura_Direct3D_EnvironmentRenderer::s_ppusIndexData, "Index Data is already setp!");
		Asura_ResourceAllocateArray( Asura_Direct3D_EnvironmentRenderer::s_ppusIndexData, Asura_Direct3D_EnvironmentRenderer::s_uTotalNumberOfIndexBuffers);
		memset( Asura_Direct3D_EnvironmentRenderer::s_ppusIndexData, 0, sizeof(uint16*) * Asura_Direct3D_EnvironmentRenderer::s_uTotalNumberOfIndexBuffers);
		
		u_int uIBSizeMax = 0;
		uint16* const pusIndexNULL = NULL;
		uint16* pusIndexOffset = pusIndexNULL;
		{
			uint16** ppusIndexOffset = Asura_Direct3D_EnvironmentRenderer::s_ppusIndexData;
			
			for( u_int uIBIndex = 0; uIBIndex < uNumberOfIBs; uIBIndex += 1)
			{
				uint32 uIBSize;
				
				Asura_FileAccess::ReadData( hFile, &uIBSize, sizeof( uIBSize));
				
				uIBSizeMax = Asura_Maths::Max( uIBSizeMax, uIBSize);
				
				// DLH - Store off offset in index array of IB
				*ppusIndexOffset = pusIndexOffset;
				ppusIndexOffset += 1;
				pusIndexOffset += uIBSize;

				// DLH - Create IB
				Direct3D_IndexBuffer*& pxBuffer = Asura_Direct3D_EnvironmentRenderer::s_ppxIndexBuffer[ uIBIndex];
				
				xResult = Asura_Direct3D::D3DDevice->CreateIndexBuffer( sizeof(unsigned short) * (uIBSize), D3DUSAGE_WRITEONLY, D3DFMT_INDEX16, D3DPOOL_MANAGED, &pxBuffer, NULL);
				ASURA_D3D_OK( xResult);
			}
		}
		
		// DLH - Create Index Data Storage
		Asura_Direct3D_EnvironmentRenderer::s_uTotalNumberOfIndices = pusIndexOffset - pusIndexNULL;
		
		Asura_Assert( NULL == Asura_Direct3D_EnvironmentRenderer::s_pusIndexData, "Index Data is already setp!");
		Asura_ResourceAllocateArray( Asura_Direct3D_EnvironmentRenderer::s_pusIndexData, Asura_Direct3D_EnvironmentRenderer::s_uTotalNumberOfIndices);
		memset( Asura_Direct3D_EnvironmentRenderer::s_pusIndexData, ~0, sizeof(uint16) * Asura_Direct3D_EnvironmentRenderer::s_uTotalNumberOfIndices);

		// DLH - Update the IB based Index Data pointer array
		// DLH - Is is done like this as the file format did not know how many indices where saved. This calculates it.
		{
			uint16* pusIndexBase = Asura_Direct3D_EnvironmentRenderer::s_pusIndexData;
			uint16** ppusIndexOffset = Asura_Direct3D_EnvironmentRenderer::s_ppusIndexData;
			
			for( u_int uIBIndex = 0; uIBIndex < uNumberOfIBs; uIBIndex += 1)
			{
				*ppusIndexOffset = pusIndexBase + (*ppusIndexOffset - pusIndexNULL);
				ppusIndexOffset += 1;
			}
		}

		// Strips
		u_int uNumberOfStrips;
		Asura_FileAccess::ReadData(hFile, &uNumberOfStrips, sizeof(u_int));
		Asura_Direct3D_EnvironmentRenderer::s_uTotalNumberOfStrips = uNumberOfStrips;
		
		Asura_ResourceAllocateArray(Asura_Direct3D_EnvironmentRenderer::s_pxStrips, uNumberOfStrips);
		if(Asura_Direct3D_EnvironmentRenderer::s_pxStrips)
		{
			Asura_FileAccess::ReadData(hFile, Asura_Direct3D_EnvironmentRenderer::s_pxStrips, sizeof(Asura_Direct3D_EnvironmentRenderer_Strip) * uNumberOfStrips);
		}
		else
		{
			bAnyErrors |= true;
			Asura_FileAccess::SkipData(hFile, sizeof(Asura_Direct3D_EnvironmentRenderer_Strip) * uNumberOfStrips);
		}

		#if !defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM || defined ASURA_MATERIAL_RESPONSE_CHARACTERS_ONLY

			// Convert material indices
			if(bConvertMaterialIDs)
			{
				for(u_int uStrip = 0; uStrip < uNumberOfStrips; ++uStrip)
				{
					Asura_Direct3D_EnvironmentRenderer_Strip& xStrip = Asura_Direct3D_EnvironmentRenderer::s_pxStrips[uStrip];
					xStrip.m_iMaterial = Asura_Material_System::ConvertIndex(xStrip.m_iMaterial);
				}
			}

		#endif // !defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM || defined ASURA_MATERIAL_RESPONSE_CHARACTERS_ONLY
		
		for(u_int uStrip = 0; uStrip < uNumberOfStrips; ++uStrip)
		{
			Asura_Direct3D_EnvironmentRenderer_Strip& xStrip = Asura_Direct3D_EnvironmentRenderer::s_pxStrips[uStrip];
			Direct3D_VertexBuffer*& pxBuffer = Asura_Direct3D_EnvironmentRenderer::s_ppxVertexBuffer[ xStrip.m_uVertexBuffer];
			void* pvDest = NULL;
			if(pxBuffer && SUCCEEDED(pxBuffer->Lock(0, 0, &pvDest, 0)))
			{
				Asura_Direct3D_EnvironmentRenderer_Vertex* pxVertexBase = (Asura_Direct3D_EnvironmentRenderer_Vertex*)(pvDest);
				Asura_Direct3D_EnvironmentRenderer_Vertex* pxVertexStart = pxVertexBase + xStrip.m_uStartVertex;
				
				Asura_FileAccess::ReadData(hFile, pxVertexStart, sizeof(Asura_Direct3D_EnvironmentRenderer_Vertex) * xStrip.m_uNumberOfVertices);

				pxBuffer->Unlock();
			}
			else
			{
				bAnyErrors |= true;
				Asura_FileAccess::SkipData(hFile, sizeof(Asura_Direct3D_EnvironmentRenderer_Vertex) * xStrip.m_uNumberOfVertices);
			}
		}

		for(u_int uStrip = 0; uStrip < uNumberOfStrips; ++uStrip)
		{
			Asura_Direct3D_EnvironmentRenderer_Strip& xStrip = Asura_Direct3D_EnvironmentRenderer::s_pxStrips[uStrip];
			const u_int uIndexBufferIndex = xStrip.m_uIndexBuffer;

			// if we're using tri-lists we've got 3 indices per tri, otherwise for strips we have (number of tris + 2) indices
			const u_int uIndexDataSized = bUsingTriLists ? sizeof( uint16) * (xStrip.m_uNumberOfPrimitives*3) : sizeof( uint16) * (xStrip.m_uNumberOfPrimitives+2);
			
			uint16* pusIndexDataBase = Asura_Direct3D_EnvironmentRenderer::s_ppusIndexData[ uIndexBufferIndex];
			uint16* pucIndexDataStart = pusIndexDataBase + xStrip.m_uStartIndex;
			Asura_FileAccess::ReadData(hFile, pucIndexDataStart, uIndexDataSized);
			
			Direct3D_IndexBuffer*& pxBuffer = Asura_Direct3D_EnvironmentRenderer::s_ppxIndexBuffer[ uIndexBufferIndex];
			void* pvDest = NULL;
			if(pxBuffer && SUCCEEDED(pxBuffer->Lock(0, 0, &pvDest, 0)))
			{
				uint16* pusIndexBufferBase = (uint16*)pvDest;
				uint16* pucIndexBufferStart = pusIndexBufferBase + xStrip.m_uStartIndex;

				memcpy( pucIndexBufferStart, pucIndexDataStart, uIndexDataSized);

				pxBuffer->Unlock();
			}
		}
		

		Asura_Direct3D_EnvironmentRenderer::s_bVertexAndIndexBuffersCreatedInternally = true; // Not technically true, but this really means "owns my own buffers"
#ifdef ASURA_TOOLS
		Asura_Direct3D_EnvironmentRenderer::s_bStripsValid = true;
#endif
		Asura_Direct3D_EnvironmentRenderer::s_bUsingPreloadedData = true;

		Asura_Direct3D_EnvironmentRenderer::s_bUseTriLists = bUsingTriLists;
	}

	Asura_ResourcePopType();

	// Should this return the errors bool? We skip data whenever things fail so we should have read the chunk completely anyway.
	return true;
}

#endif //defined( ASURA_TARGET_SM3)
