//
// Asura_Chunk_ResourceFile_PC_ObjectHierarchy.cpp
//
// Created by KJL - 08/05/2003 14:45:54
//
// Copyright (C) 2003 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

// This file's header.
#include "ChunkSystem/Asura_Chunk_ResourceFile_PC_ObjectHierarchy.h"

#include "Core/Asura_Material_System.h"
#include "FileAccess/Asura_FileAccess.h"
#include "Rendering/Asura_Platform_RenderObjectHierarchy.h"
#include "Rendering/Asura_Direct3D.h"
#include "Rendering/Asura_Direct3D_Caps.h"

bool Asura_Chunk_ResourceFile_PC_ObjectHierarchy::Process( Asura_Handle hFile )
{
	Asura_Direct3D_ObjectHierarchyRender_Handler&	xHandler	= Asura_Direct3D_ObjectHierarchyRenderer::GetCurrentHandler();
	const bool bSupportsMultiMaterials = (this->Version > 0);
	Asura_Assert(Version >= 3 && Version<=ASURA_CHUNK_RESOURCEFILE_PC_OBJECTHIERARCHY_VERSION, "This object hierarchy chunk is either too old or too new. Please re-export the level or PCChars.");

	if(Version >= 4)
	{
		const bool bMaterialResponse = (Flags & ChunkFlag_MaterialResponse) != 0;
		#if defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM
		if(!bMaterialResponse)
		{
			Asura_Assert(0,"Attempting to load a non material response object hierarchy into a material response build.\nThis will be skipped");
			return false;
		}
		#else
		if(bMaterialResponse)
		{
			Asura_Assert(0,"Attempting to load a material response object hierarchy into a non material response build.\nThis will be skipped");
			return false;
		}
		#endif
	}


	Asura_String name = Asura_FileAccess::ReadString(hFile);

	// Check if there is enough room for a new character
	Asura_Direct3D_ObjectHierarchy* pxStripedHierarchy = xHandler.AddHierarchy(name);

	if (!pxStripedHierarchy) { return false; }
	
#if defined(ASURA_TARGET_SM3)
	pxStripedHierarchy->m_bHasNormalMappedStrip = true;
#endif
	u_int uNumberOfStrips;
	u_int uTotalNumberOfVertices;
	u_int uTotalNumberOfIndices;

	Asura_FileAccess::ReadData(hFile, &uNumberOfStrips, sizeof(u_int));
	Asura_FileAccess::ReadData(hFile, &uTotalNumberOfVertices, sizeof(u_int));
	Asura_FileAccess::ReadData(hFile, &uTotalNumberOfIndices, sizeof(u_int));

	pxStripedHierarchy->m_uNumberOfStrips = uNumberOfStrips;
	pxStripedHierarchy->m_uTotalNumberOfVertices = uTotalNumberOfVertices;
	pxStripedHierarchy->m_uTotalNumberOfIndices = uTotalNumberOfIndices;

	Asura_Direct3D_ObjectHierarchy_Strip* pxStrip = new Asura_Direct3D_ObjectHierarchy_Strip[uNumberOfStrips]; 
	
	// assign to hierarchy
	pxStripedHierarchy->m_pxHierarchyStrip = pxStrip;
	
	u_int uNumberOfTriangles;
	u_int uStartIndex;
	#if defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM
	Asura_Hash_ID auMaterialResponseHashID[2] = {0,0};
	#else
	int iMaterialIndex;
	#endif
	u_int uLowestVertexUsed;
	u_int uNumberOfVertices;
	u_short usBoneIndex;
	
	for(u_int u=0; u< uNumberOfStrips; u++)
	{
		Asura_FileAccess::ReadData(hFile, &uNumberOfTriangles, sizeof(u_int));
		Asura_FileAccess::ReadData(hFile, &uStartIndex, sizeof(u_int));
		#if defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM
		Asura_FileAccess::ReadData(hFile, &auMaterialResponseHashID[0], sizeof(Asura_Hash_ID));
		if(Version >= 4) Asura_FileAccess::ReadData(hFile, &auMaterialResponseHashID[1], sizeof(Asura_Hash_ID));
		#else
		Asura_FileAccess::ReadData(hFile, &iMaterialIndex, sizeof(int));
		#endif
		Asura_FileAccess::ReadData(hFile, &uLowestVertexUsed, sizeof(u_int));	
		Asura_FileAccess::ReadData(hFile, &uNumberOfVertices, sizeof(u_int));
		if(bSupportsMultiMaterials)
		{
			Asura_FileAccess::ReadData(hFile, &usBoneIndex, sizeof(u_short));
		}
		else
		{
			usBoneIndex = (u_short)(u&0xffff);
		}
	
		pxStrip[u].m_uNumberOfTriangles = uNumberOfTriangles;
		pxStrip[u].m_uStartVertexIndex = uStartIndex;

		#if defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM
			pxStrip[u].m_auMaterialResponseHashID[0] = auMaterialResponseHashID[0];
			pxStrip[u].m_auMaterialResponseHashID[1] = auMaterialResponseHashID[1];
		#else
			pxStrip[u].m_iMaterialIndex = Asura_Material_System::ConvertIndex(iMaterialIndex);
		#endif

		pxStrip[u].m_uLowestVertexUsed = uLowestVertexUsed;
		pxStrip[u].m_uNumberOfVertices = uNumberOfVertices;
		pxStrip[u].m_usBoneIndex = usBoneIndex;
	}

	int flags = D3DUSAGE_WRITEONLY;
			
	if (!Asura_Direct3D_Caps::HasHardwareVertexShaders())
	{
		flags |= D3DUSAGE_SOFTWAREPROCESSING;
	}

	if (uTotalNumberOfVertices!=0)
	{
#if defined(ASURA_TARGET_SM3)
#define ASURA_DIRECT3D_OBJECTHIERARCHY_VERTEX_USED Asura_Direct3D_ObjectHierarchy_Vertex_NormalMapped
#else
#define ASURA_DIRECT3D_OBJECTHIERARCHY_VERTEX_USED Asura_Direct3D_ObjectHierarchy_Vertex
#endif
	
		
		// Create vertex and buffers for the strips
		Asura_Direct3D::D3DDevice->CreateVertexBuffer
		(
			pxStripedHierarchy->m_uTotalNumberOfVertices * sizeof( ASURA_DIRECT3D_OBJECTHIERARCHY_VERTEX_USED),
			flags,
			0, 
			D3DPOOL_MANAGED,
			&(pxStripedHierarchy->m_pxVertexBuffer),
			0
		);

		// lock, fill in data and unlock
		Asura_Direct3D_ObjectHierarchy_Vertex* pxVertexData;
		if (SUCCEEDED(pxStripedHierarchy->m_pxVertexBuffer->Lock(0,sizeof( ASURA_DIRECT3D_OBJECTHIERARCHY_VERTEX_USED) * pxStripedHierarchy->m_uTotalNumberOfVertices,(void**)&pxVertexData,0)))
		{
			if(bSupportsMultiMaterials)
			{
				Asura_FileAccess::ReadData(hFile, pxVertexData, sizeof( ASURA_DIRECT3D_OBJECTHIERARCHY_VERTEX_USED) * pxStripedHierarchy->m_uTotalNumberOfVertices);
			}
			else
			{
				for(u_int u = 0; u < pxStripedHierarchy->m_uTotalNumberOfVertices; ++u)
				{
					Asura_FileAccess::ReadData(hFile, &pxVertexData[u], sizeof(float) * 8);
					pxVertexData[u].m_fAlpha = 1.f;
				}
			}
			pxStripedHierarchy->m_pxVertexBuffer->Unlock();
		} else {
			Asura_Assert(false, "Failed to lock vertex buffer");
			if(bSupportsMultiMaterials)
			{
				Asura_FileAccess::SkipData(hFile, sizeof( ASURA_DIRECT3D_OBJECTHIERARCHY_VERTEX_USED) * pxStripedHierarchy->m_uTotalNumberOfVertices);
			}
			else
			{
				Asura_FileAccess::SkipData(hFile, sizeof(float) * 8 * pxStripedHierarchy->m_uTotalNumberOfVertices);
			}
		}

		
#undef ASURA_DIRECT3D_OBJECTHIERARCHY_VERTEX_USED

	}

	if (uTotalNumberOfIndices!=0)
	{
		Asura_Direct3D::D3DDevice->CreateIndexBuffer
		(
			pxStripedHierarchy->m_uTotalNumberOfIndices * sizeof(unsigned short),
			flags,
			D3DFMT_INDEX16,
			D3DPOOL_MANAGED,
			&(pxStripedHierarchy->m_pxIndexBuffer),
			0
		);
				
		unsigned short* pusIndexData;
		pxStripedHierarchy->m_pxIndexBuffer->Lock
		(
			0,
			sizeof(u_short) * pxStripedHierarchy->m_uTotalNumberOfIndices, 
			(void**)&pusIndexData,
			0
		);
			
		Asura_FileAccess::ReadData(hFile, pusIndexData, sizeof(u_short) * pxStripedHierarchy->m_uTotalNumberOfIndices);
					
		pxStripedHierarchy->m_pxIndexBuffer->Unlock();
	}

	return true;
}
