//
// Asura_Chunk_ResourceFile_PC_StaticDecals.h
//
// Created by DMR - 20 Aug 2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//

/******************************************************************************/
/* Included Files                                                             */
/******************************************************************************/

#include "Core/Asura.h"

#ifdef ASURA_USE_STATIC_DECAL_SYSTEM

#include "ChunkSystem/Asura_Chunk_ResourceFile_PC_StaticDecals.h"

// Asura Platform-independent
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "Core/Asura_Material_System.h"
#include "SpecialFX/Asura_Platform_Decal_SM3.h"
#include "ChunkSystem/Asura_Chunk_Resourcefile_PC.h"

/******************************************************************************/
/* Asura_Chunk_ResourceFile_PC_StaticDecals Class Functions              */
/******************************************************************************/

bool Asura_Chunk_ResourceFile_PC_StaticDecals::Process( const char* szFilename, Asura_Chunk_Stream& xStream )
{
	if( Version != ASURA_PC_STATICDECALS_CHUNK_VERSION )
	{
		Asura_Assertf( false, ("Invalid static decals chunk version (%d) encountered. Should be %d. You may need new code, or to re-export the level.", Version, ASURA_PC_STATICDECALS_CHUNK_VERSION) );
		return false;
	}


	int iNumDecalBuffers;	

	// Read the data...
	xStream >> iNumDecalBuffers;

	Asura_Platform_Decal_System_SM3::AllocateStaticDecalBuffers( iNumDecalBuffers );

	for(int i=0; i<iNumDecalBuffers; i++)
	{
		Asura_Direct3D_StaticDecalBuffer *pxBuffer = Asura_Platform_Decal_System_SM3::GetStaticDecalBuffer( i );

		xStream >> pxBuffer->m_iModuleIndex;
		xStream >> pxBuffer->m_uDecalMaterialHash;
		xStream >> pxBuffer->m_iNumVertices;
		xStream >> pxBuffer->m_iNumIndices;
		xStream >> pxBuffer->m_xAABB;

		pxBuffer->m_iDecalMaterialIndex = -1;
			
		if (pxBuffer->m_iNumVertices>0)
		{		
			// Read the vertices
			int iVertSize = 0;

			// Normal vertices - With tangent/normal/binormal
			iVertSize = sizeof(D3D_DecalVertex);
			
			Asura_Direct3D::D3DDevice->CreateVertexBuffer(iVertSize * pxBuffer->m_iNumVertices,  D3DUSAGE_WRITEONLY, 0, D3DPOOL_MANAGED, &pxBuffer->m_pxVertexBuffer, NULL); 
		
			Asura_Assert(pxBuffer->m_pxVertexBuffer, "Fatal: Failed to create vertex buffer");
						
			D3D_DecalVertex *pxVerts = NULL;
			if (SUCCEEDED(pxBuffer->m_pxVertexBuffer->Lock(0,0,(void**)&pxVerts,0)))
			{
				xStream.Read( iVertSize * pxBuffer->m_iNumVertices, pxVerts);
				pxBuffer->m_pxVertexBuffer->Unlock();
			} else {
				Asura_Assert(0, "failed to lock vertex buffer");
				xStream.Skip( iVertSize * pxBuffer->m_iNumVertices);
			}

			// Read the indices
			Asura_Direct3D::D3DDevice->CreateIndexBuffer(pxBuffer->m_iNumIndices * 2, D3DUSAGE_WRITEONLY, D3DFMT_INDEX16, D3DPOOL_MANAGED, &pxBuffer->m_pxIndexBuffer, NULL);

			Asura_Assert(pxBuffer->m_pxIndexBuffer, "Fatal: Failed to create index buffer");

			uint16 *pxIndices = NULL;
			if (SUCCEEDED(pxBuffer->m_pxIndexBuffer->Lock(0,0, (void**)&pxIndices, 0)))
			{
				xStream.Read( 2 * pxBuffer->m_iNumIndices, pxIndices);
				pxBuffer->m_pxIndexBuffer->Unlock();
			} else {
				Asura_Assert(0, "failed to lock index buffer");
				xStream.Skip(  2 * pxBuffer->m_iNumIndices);
			}

		}
		
	}

	return true;
}


#endif // ASURA_USE_STATIC_DECAL_SYSTEM