//*******************************************************************
//
// File: Asura_SEH_Handler.cpp            
//
// Created: 10 Jan 2006
//
// Description: 
//
//*******************************************************************

// system libraries
#include "Core/Asura.h"

// Handles basic Windows functionality: initialisation, message handling & shutdown.
#include "Core/Asura_Atomic.h"
#include "Core/Asura_Windows.h"

#include "Debugging/Asura_Assert.h"
#include "Debugging/Asura_LogFile.h"
#include "Rendering/Asura_Viewport.h"
#include "Input/Asura_PlatformInput.h"

#if defined( ASURA_ENABLE_SEH_HANDLING)

// main header
#include "Asura_PC_SEH_Handler.h"
//#include <dbghelp.h>
#include <shellapi.h>
#include <shlobj.h>

#include <float.h>
#include <fpieee.h>
#include <excpt.h>

// other libraries
#pragma comment( lib, "DbgHelp")

// internal dependencies

//*******************************************************************
// Constants
//*******************************************************************

#if defined( ASURA_ENABLE_SEH_MINIDUMP)
	#define ASURA_ENABLE_DEBUGHELP_LIBRARY
#endif

//*******************************************************************
// Local Classes
//*******************************************************************

#if defined( ASURA_ENABLE_DEBUGHELP_LIBRARY)
class Asura_DebugHelp_Library
{
public:
	Asura_DebugHelp_Library();
	~Asura_DebugHelp_Library();
	
	BOOL MiniDumpWriteDump( HANDLE hProcess, DWORD ProcessId, HANDLE hFile, MINIDUMP_TYPE DumpType, PMINIDUMP_EXCEPTION_INFORMATION ExceptionParam, PMINIDUMP_USER_STREAM_INFORMATION UserStreamParam, PMINIDUMP_CALLBACK_INFORMATION CallbackParam);
	bool IsFuncValid_MiniDumpWriteDump();

private:
	typedef BOOL (WINAPI *Func_MiniDumpWriteDump)( HANDLE hProcess, DWORD ProcessId, HANDLE hFile, MINIDUMP_TYPE DumpType, PMINIDUMP_EXCEPTION_INFORMATION ExceptionParam, PMINIDUMP_USER_STREAM_INFORMATION UserStreamParam, PMINIDUMP_CALLBACK_INFORMATION CallbackParam);

	Func_MiniDumpWriteDump m_pfnMiniDumpWriteDump;
	
	HMODULE m_hLibrary;
};
#endif

//*******************************************************************
// Local Data
//*******************************************************************

char Asura_PC_SEH_Handler::s_szReportTXT[32*1024] = {0};
#if defined( ASURA_TOOLS)
char Asura_PC_SEH_Handler::s_szReportXML[32*1024] = {0};
#endif
HANDLE Asura_PC_SEH_Handler::s_xProcessHandle = NULL;

#if defined( ASURA_ENABLE_DEBUGHELP_LIBRARY)
	//Asura_DebugHelp_Library s_xLibraryDbgHelp;
#endif
	

//*******************************************************************
// Function Definitions
//*******************************************************************

#if defined( ASURA_ENABLE_DEBUGHELP_LIBRARY)
Asura_DebugHelp_Library::Asura_DebugHelp_Library() :
	m_hLibrary( NULL),
	m_pfnMiniDumpWriteDump( NULL)
{
	this->m_hLibrary = LoadLibrary( "DbgHelp");
	if( NULL == this->m_hLibrary)
	{
		return;
	}
	
	this->m_pfnMiniDumpWriteDump = (Func_MiniDumpWriteDump)GetProcAddress( this->m_hLibrary, "MiniDumpWriteDump");
}

Asura_DebugHelp_Library::~Asura_DebugHelp_Library()
{
	this->m_pfnMiniDumpWriteDump = NULL;
	if( NULL != this->m_hLibrary)
	{
		HMODULE hLibrary = this->m_hLibrary;
		this->m_hLibrary = NULL;
		
		FreeLibrary( hLibrary);
	}
}

BOOL Asura_DebugHelp_Library::MiniDumpWriteDump( HANDLE hProcess, DWORD ProcessId, HANDLE hFile, MINIDUMP_TYPE DumpType, PMINIDUMP_EXCEPTION_INFORMATION ExceptionParam, PMINIDUMP_USER_STREAM_INFORMATION UserStreamParam, PMINIDUMP_CALLBACK_INFORMATION CallbackParam)
{
	if( NULL == this->m_pfnMiniDumpWriteDump)
	{
		return( FALSE);
	}
	return( (*this->m_pfnMiniDumpWriteDump)( hProcess, ProcessId, hFile, DumpType, ExceptionParam, UserStreamParam, CallbackParam));
}

bool Asura_DebugHelp_Library::IsFuncValid_MiniDumpWriteDump()
{
	return( NULL != this->m_pfnMiniDumpWriteDump);
}

#endif

const char* const Asura_PC_SEH_Handler::GetReportTXT()
{
	return( s_szReportTXT);
}

#if defined( ASURA_TOOLS)
const char* const Asura_PC_SEH_Handler::GetReportXML()
{
	return( s_szReportXML);
}
#endif

HANDLE Asura_PC_SEH_Handler::GetProcess( void )
{
	if( NULL == s_xProcessHandle)
	{
		s_xProcessHandle = GetCurrentProcess();
	}
	
	return s_xProcessHandle;
}

void Asura_PC_SEH_Handler::SetProcess( const HANDLE xProcess )
{
	s_xProcessHandle = xProcess;
}

//-------------------------------------------------------------------
// Function:    Asura_SEH_Handler::AddToReport
//
// Description: Appends a string to the exception report
//
// Parameters:  i_pcszText      - String to add to the exception report
//-------------------------------------------------------------------
void Asura_PC_SEH_Handler::AddToReportTXT( const char* const szText )
{
	const int iSizeOfReportArray = sizeof( s_szReportTXT ) - 1;
	const size_t iSizeOfReportString = strlen( s_szReportTXT );
	const size_t iSizeOfReportRemaining = iSizeOfReportArray - iSizeOfReportString;
	
	char* szReportAddTo = s_szReportTXT + iSizeOfReportString;
	
	strncpy( szReportAddTo, szText, iSizeOfReportRemaining );
	const size_t iSizeOfTextString = strlen( szText);
	szReportAddTo[iSizeOfTextString] = 0;
}

#if defined( ASURA_TOOLS)
void Asura_PC_SEH_Handler::AddToReportXML( const char* const szText )
{
	const int iSizeOfReportArray = sizeof( s_szReportXML ) - 1;
	const size_t iSizeOfReportString = strlen( s_szReportXML );
	const size_t iSizeOfReportRemaining = iSizeOfReportArray - iSizeOfReportString;
	
	char* szReportAddTo = s_szReportXML + iSizeOfReportString;
	
	strncpy( szReportAddTo, szText, iSizeOfReportRemaining );
	const size_t iSizeOfTextString = strlen( szText);
	szReportAddTo[iSizeOfTextString] = 0;
}
#endif

bool Asura_PC_SEH_Handler::GetFunctionInfo( const DWORD_PTR& iAddress )
{
#define FUNCTIONINFO_BUFFERSIZE (4*1024)

	const int iTempStringSize = 1024;
	char szTempString[ iTempStringSize + 1] = {0};
	
	DWORD uDisplacement = 0;
	
	PIMAGEHLP_SYMBOL pxSymbol = (PIMAGEHLP_SYMBOL) GlobalAlloc(GMEM_FIXED, FUNCTIONINFO_BUFFERSIZE);
	memset( pxSymbol, 0, FUNCTIONINFO_BUFFERSIZE);
	pxSymbol->SizeOfStruct = FUNCTIONINFO_BUFFERSIZE;
	pxSymbol->MaxNameLength = FUNCTIONINFO_BUFFERSIZE - sizeof(PIMAGEHLP_SYMBOL);
	
//	AddToReport( "");
	DWORD uResult = 0;
	
	{
		IMAGEHLP_LINE xLineNumber;
		xLineNumber.SizeOfStruct = sizeof(xLineNumber);
		
		if( SymGetLineFromAddr( GetProcess(), iAddress, &uDisplacement, &xLineNumber ))
		{
			_snprintf( szTempString, iTempStringSize, " %s(%d)\t", xLineNumber.FileName, xLineNumber.LineNumber);
			AddToReportTXT( szTempString );
#if defined( ASURA_TOOLS)
			{
				char szFilenameXMLString[ iTempStringSize + 1] = {0};
				Asura_SimpleLogfileConvertToXML( xLineNumber.FileName, szFilenameXMLString, iTempStringSize);
				_snprintf( szTempString, iTempStringSize, "<source filename=\"%s\" fileline=\"%d\"/>", szFilenameXMLString, xLineNumber.LineNumber);
			}
			AddToReportXML( szTempString );
#endif
		}
	}

#if defined( _WIN64)
	DWORD64 uDisplacement64 = uDisplacement;
	if(SymGetSymFromAddr( GetProcess(), iAddress, &uDisplacement64, pxSymbol ))
#else
	if(SymGetSymFromAddr( GetProcess(), iAddress, &uDisplacement, pxSymbol ))
#endif
	{
		uResult = UnDecorateSymbolName( pxSymbol->Name, szTempString, iTempStringSize, 
			UNDNAME_COMPLETE | UNDNAME_NO_THISTYPE | 
			UNDNAME_NO_SPECIAL_SYMS | UNDNAME_NO_MEMBER_TYPE | 
			UNDNAME_NO_MS_KEYWORDS | UNDNAME_NO_ACCESS_SPECIFIERS
		);
		AddToReportTXT(szTempString);
#if defined( ASURA_TOOLS)
		AddToReportXML("<symbols>");
		{
			char szSymbolXMLString[ iTempStringSize + 1] = {0};
			Asura_SimpleLogfileConvertToXML( szTempString, szSymbolXMLString, iTempStringSize);
			AddToReportXML( szSymbolXMLString);
		}
		AddToReportXML("</symbols>");
#endif
	}
	
	GlobalFree(pxSymbol);
	
	return uResult != 0;
#undef FUNCTIONINFO_BUFFERSIZE
}

bool Asura_PC_SEH_Handler::GetStack( const EXCEPTION_POINTERS* const pxExceptionPointers )
{
	// Fill the stack with information from the context
	const int iTempStringSize = 1024;
	char szTempString[ iTempStringSize + 1] = {0};
	
#if defined( _WIN64)
	STACKFRAME64 xCallStack;
#else
	STACKFRAME xCallStack;
#endif
	memset( &xCallStack, 0, sizeof( xCallStack));
#if defined( _WIN64)
	xCallStack.AddrPC.Offset = pxExceptionPointers->ContextRecord->Rip;
	xCallStack.AddrStack.Offset = pxExceptionPointers->ContextRecord->Rsp;
	xCallStack.AddrFrame.Offset = pxExceptionPointers->ContextRecord->Rbp;
#else
	xCallStack.AddrPC.Offset = pxExceptionPointers->ContextRecord->Eip;
	xCallStack.AddrStack.Offset = pxExceptionPointers->ContextRecord->Esp;
	xCallStack.AddrFrame.Offset = pxExceptionPointers->ContextRecord->Ebp;
#endif
	xCallStack.AddrPC.Mode = AddrModeFlat;
	xCallStack.AddrStack.Mode = AddrModeFlat;
	xCallStack.AddrFrame.Mode = AddrModeFlat;
	
	HANDLE xCurrentThread = GetCurrentThread();
	HANDLE xCurrentProcess = GetProcess();
	
#if defined( ASURA_TOOLS)
	AddToReportXML( "<stack>");
#endif
	while( StackWalk( IMAGE_FILE_MACHINE_I386, xCurrentProcess, xCurrentThread, &xCallStack, NULL, NULL, SymFunctionTableAccess, SymGetModuleBase, NULL))
	{
		if( xCallStack.AddrFrame.Offset == 0) break;	
		
#if defined( ASURA_TOOLS)
		AddToReportXML( "<level>");
#endif
		
		_snprintf( szTempString, iTempStringSize, "0x%08x: ", xCallStack.AddrPC.Offset );
		AddToReportTXT( szTempString);
#if defined( ASURA_TOOLS)
		_snprintf( szTempString, iTempStringSize, "<address value=\"0x%08x\"/>", xCallStack.AddrPC.Offset );
		AddToReportXML( szTempString);
#endif
		
		GetFunctionInfo( xCallStack.AddrPC.Offset );
		
		AddToReportTXT( "\r\n" );
#if defined( ASURA_TOOLS)
		AddToReportXML( "</level>");
#endif
	}	
#if defined( ASURA_TOOLS)
	AddToReportXML( "</stack>");
#endif
	return true;
}

#if defined( ASURA_ENABLE_SEH_MINIDUMP)
// DLH - 17 April 2007 16:04:54 - This code was borrowed from the windows help
// DLH - 17 April 2007 16:04:54 - It creates a MiniDump that can be loaded back into VisualStudio for Debugging
static bool GenerateDump( const EXCEPTION_POINTERS* const pExceptionPointers, char* pszFilename, const u_int pszFilenameSizeMax)
{
	Asura_DebugHelp_Library s_xLibraryDbgHelp;
	if( false == s_xLibraryDbgHelp.IsFuncValid_MiniDumpWriteDump())
	{
		return( false);
	}

	// (MH - 04.09.2008) only do this for the first exception, otherwise we end up with 2 dumps for each crash.
	static bool ls_bFirstTime = true;
	if (ls_bFirstTime)
	{
		ls_bFirstTime = false;
	}
	else
	{
		return false;
	}
	
    BOOL bMiniDumpSuccessful;
    CHAR szModuleName[ MAX_PATH];
    HANDLE hDumpFile;
    SYSTEMTIME stLocalTime;
    MINIDUMP_EXCEPTION_INFORMATION ExpParam;

    GetLocalTime( &stLocalTime );
    
    GetModuleFileName( NULL, pszFilename, pszFilenameSizeMax);
    
    _splitpath( pszFilename, NULL, NULL, szModuleName, NULL);

    _snprintf( pszFilename, pszFilenameSizeMax, "%s-%04d%02d%02d-%02d%02d%02d-%02d-%ld-%ld.dmp", 
               szModuleName,
               stLocalTime.wYear, stLocalTime.wMonth, stLocalTime.wDay, 
               stLocalTime.wHour, stLocalTime.wMinute, stLocalTime.wSecond, stLocalTime.wMilliseconds,
               GetCurrentProcessId(), GetCurrentThreadId());
    hDumpFile = CreateFile( pszFilename, GENERIC_WRITE, 0, 0, CREATE_ALWAYS, 0, 0);
	if( INVALID_HANDLE_VALUE == hDumpFile)
	{
		return( false);
	}
	
	ExpParam.ThreadId = GetCurrentThreadId();
	ExpParam.ExceptionPointers = const_cast< EXCEPTION_POINTERS*>( pExceptionPointers);
	ExpParam.ClientPointers = TRUE;

	bMiniDumpSuccessful = s_xLibraryDbgHelp.MiniDumpWriteDump( GetCurrentProcess(), GetCurrentProcessId(), hDumpFile, MiniDumpWithDataSegs, &ExpParam, NULL, NULL);
                    
	CloseHandle( hDumpFile);
	
	return( TRUE == bMiniDumpSuccessful);
}
#endif

static int fpieee_handler( _FPIEEE_RECORD *pieee )
{
	// user-defined ieee trap handler routine:
	// there is one handler for all 
	// IEEE exceptions

	// Assume the user wants all invalid 
	// operations to return 0.

	//if (pieee->Result.Format == _FpFormatFp32)
	{
		/*
		Kim's instructions for finding the point at which the exception happened :

		1)Go up the stack once into _fpieee_flt(). You won't have the source for this, but don't worry...
		2)...all we want is the local variables, so open a watch and get the value of p->ContextRecord->Eip
		3) Go to the disassembly, and enter the address you got from the previous step.
		4) Right-click the instruction and select Go To Source Code
		5) Breakpoint the next line.
		6) Resume execution. You now have the actual stack at (well, just after) the point of the exception.

		RAR 17/01/2006 - At step 4 it may be easier to put a breakpoint on the next assembly instruction instead , 
		and then go up the stack when the breakpoint is reached.

		*/
		
		if (pieee->Cause.InvalidOperation)
		{
			//Asura_Assert(false, "floating point invalid operation");
			pieee->Result.Value.Fp32Value = 0.0F;
			return EXCEPTION_CONTINUE_EXECUTION;
		}
		if (pieee->Cause.Overflow)
		{
			//Asura_Assert(false, "floating point overflow");
			pieee->Result.Value.Fp32Value = 0.0F;
			return EXCEPTION_CONTINUE_EXECUTION;
		}
		if (pieee->Cause.ZeroDivide)
		{
			//Asura_Assert(false, "floating point divide by zero");
			pieee->Result.Value.Fp32Value = 0.0F;
			return EXCEPTION_CONTINUE_EXECUTION;
		}
		// KOR - 26/10/2005 - underflow can be quite common, and the exception handler sometimes munges the stack,
		// so I'm turned underflow catching off. If we start finding denormalised numbers are a problem, we should
		// turn this back on (with the assert) and actually fix underflows.
/*		if (pieee->Cause.Underflow)
		{
			// AJG - 20/10/05 - Just returning 0 rather than asserting, since this happens often enough that it's not really a useful assert.
			pieee->Result.Value.Fp32Value = 0.0F;
			return EXCEPTION_CONTINUE_EXECUTION;
		}*/
	}

	return EXCEPTION_EXECUTE_HANDLER;
}

// DLH - Guard against multiple Entry
static u_int Asura_PC_SEH_Handler_ProcessException_CodeLock = 0;
DWORD Asura_PC_SEH_Handler::ProcessException( EXCEPTION_POINTERS * const pxExceptionPointers )
{
	if (IsDebuggerPresent())
	{
		return( EXCEPTION_CONTINUE_SEARCH);
	}
	
	if( 1 == Asura_Atomic::CompareAndSwap( Asura_PC_SEH_Handler_ProcessException_CodeLock, 0, 1))
	{
		return( EXCEPTION_CONTINUE_SEARCH);
	}
	
	u_int iCode = pxExceptionPointers->ExceptionRecord->ExceptionCode;

	// DLH - Deal with the IEEE Floating point cases first
	// DLH - This is done like this so the problem can be fixed.
	DWORD uReturnCode = _fpieee_flt( iCode, pxExceptionPointers, fpieee_handler);

#if defined( ASURA_ENABLE_SEH_MINIDUMP)
    CHAR szMinidumpFileName[MAX_PATH]; 

	bool bMiniDumped = false;
	if( EXCEPTION_CONTINUE_EXECUTION != uReturnCode)
	{
		bMiniDumped = GenerateDump( pxExceptionPointers, szMinidumpFileName, MAX_PATH);
	}
#endif

	SymSetOptions( (SymGetOptions() | SYMOPT_LOAD_LINES | SYMOPT_DEFERRED_LOADS) & ~SYMOPT_UNDNAME );
	SymInitialize( GetProcess(), NULL, TRUE );

	// ARC - 22 March 2006 12:58:25 - fix for signed/unsigned mismatch later on

	static struct ExceptDesc 
	{
		DWORD m_uCode;
		char* m_szDesc;
	} const xExceptDescTable[] =
	{
		{ EXCEPTION_ACCESS_VIOLATION, "Access violation" },
		{ EXCEPTION_DATATYPE_MISALIGNMENT, "Data misalignment" },
		{ EXCEPTION_BREAKPOINT, "A breakpoint was encountered" },
		{ EXCEPTION_SINGLE_STEP, "Single step" },
		{ EXCEPTION_ARRAY_BOUNDS_EXCEEDED, "Array bounds exceeded" },
		{ EXCEPTION_FLT_DENORMAL_OPERAND, "Floating point denormal" },
		{ EXCEPTION_FLT_DIVIDE_BY_ZERO, "Floating point division by zero" },
		{ EXCEPTION_FLT_INEXACT_RESULT, "Floating point inexact result" },
		{ EXCEPTION_FLT_INVALID_OPERATION, "Floating point invalid operation" },
		{ EXCEPTION_FLT_OVERFLOW, "Floating point overflow" },
		{ EXCEPTION_FLT_STACK_CHECK, "Floating point stack overflow or underflow" },
		{ EXCEPTION_FLT_UNDERFLOW, "Floating point underflow" },
		{ STATUS_FLOAT_MULTIPLE_TRAPS, "Floating point multiple Traps"},
		{ STATUS_FLOAT_MULTIPLE_FAULTS, "Floating point multiple Faults"},
		{ EXCEPTION_INT_DIVIDE_BY_ZERO, "Integer divide by zero" },
		{ EXCEPTION_INT_OVERFLOW, "Integer overflow" },
		{ EXCEPTION_PRIV_INSTRUCTION, "Priviledged instruction" },
		{ EXCEPTION_IN_PAGE_ERROR, "Page error" },
		{ EXCEPTION_ILLEGAL_INSTRUCTION, "Illegal instruction" },
		{ EXCEPTION_NONCONTINUABLE_EXCEPTION, "Non continuable" },
		{ EXCEPTION_STACK_OVERFLOW, "Stack overflow" },
		{ EXCEPTION_INVALID_DISPOSITION, "Invalid disposition" },
		{ EXCEPTION_GUARD_PAGE, "Guard page" },
		{ EXCEPTION_INVALID_HANDLE, "Invalid handle" },
		{ 0, NULL }
	};

	AddToReportTXT( "Please 'Cut and Paste' this error report and send it to the relavent programmer. Thanks.\r\n\r\n" );
#if defined( ASURA_TOOLS)
	AddToReportXML( "<crash>" );
#endif

	const int iTempStringSize = 1024;
	char szTempString[ iTempStringSize + 1] = {0};
	
	SYSTEMTIME xTime;
	GetSystemTime( &xTime );
	_snprintf( szTempString, iTempStringSize, "Exception at: %02d/%02d/%d at %02d:%02d:%02d\r\n", xTime.wDay, xTime.wMonth, xTime.wYear, xTime.wHour, xTime.wMinute, xTime.wSecond);
	AddToReportTXT( szTempString );
#if defined( ASURA_TOOLS)
	_snprintf( szTempString, iTempStringSize, "<time>%04d-%02d-%02dT%02d:%02d:%02d</time>", xTime.wYear, xTime.wMonth, xTime.wDay, xTime.wHour, xTime.wMinute, xTime.wSecond);
	AddToReportXML( szTempString );
#endif

	// translate exception code
	const ExceptDesc* pxCurDesc = xExceptDescTable;
	while( pxCurDesc->m_uCode && pxCurDesc->m_uCode != iCode ) pxCurDesc++;

#if defined( ASURA_TOOLS)
	_snprintf( szTempString, iTempStringSize, "<exception code=\"0x%08x\" address=\"0x%08x\">", pxExceptionPointers->ExceptionRecord->ExceptionCode, pxExceptionPointers->ExceptionRecord->ExceptionAddress);
	AddToReportXML( szTempString );
#endif
	if ( !pxCurDesc->m_szDesc )
	{
		_snprintf( szTempString, iTempStringSize, "Exception code: 0x%08x\r\n", pxExceptionPointers->ExceptionRecord->ExceptionCode );
		AddToReportTXT( szTempString );
	}
	else
	{
		_snprintf( szTempString, iTempStringSize, "Exception code: %s (%08x): ", pxCurDesc->m_szDesc, pxExceptionPointers->ExceptionRecord->ExceptionCode );
		AddToReportTXT( szTempString );

		// some extended information on access violations
		if ( pxCurDesc->m_uCode == EXCEPTION_ACCESS_VIOLATION )
		{
			if ( !pxExceptionPointers->ExceptionRecord->ExceptionInformation[0] )
			{
				AddToReportTXT( " (Read of address: " );
#if defined( ASURA_TOOLS)
				_snprintf( szTempString, iTempStringSize, "<read address=\"0x%08x\"/>", pxExceptionPointers->ExceptionRecord->ExceptionInformation[1]);
				AddToReportXML( szTempString);
#endif
			}
			else
			{
				AddToReportTXT( " (Write to address: " );
#if defined( ASURA_TOOLS)
				_snprintf( szTempString, iTempStringSize, "<write address=\"0x%08x\"/>", pxExceptionPointers->ExceptionRecord->ExceptionInformation[1]);
				AddToReportXML( szTempString);
#endif
			}
			
			_snprintf( szTempString, iTempStringSize, "0x%08x)\r\n", pxExceptionPointers->ExceptionRecord->ExceptionInformation[1]);
			AddToReportTXT( szTempString);
		}		
	}
	
	_snprintf( szTempString, iTempStringSize, "Exception address: 0x%08x\r\n\r\n", pxExceptionPointers->ExceptionRecord->ExceptionAddress );
	AddToReportTXT( szTempString);

#if defined( ASURA_ENABLE_SEH_MINIDUMP)
	if( true == bMiniDumped)
	{
		_snprintf( szTempString, iTempStringSize, "\r\nminidump: \"%s\"\r\n\r\n", szMinidumpFileName);
		AddToReportTXT( szTempString );
#if defined( ASURA_TOOLS)
		_snprintf( szTempString, iTempStringSize, "<minidump>%s</minidump>", szMinidumpFileName);
		AddToReportXML( szTempString );
#endif
	}
#endif

	GetStack( pxExceptionPointers);
	
	AddToReportTXT( szTempString);

	_snprintf( szTempString, iTempStringSize, 
		"CPU info:\r\n"
		"---------\r\n"
#if defined( _WIN64)
		"EAX = 0x%016I64x\r\n"
		"EBX = 0x%016I64x\r\n"
		"ECX = 0x%016I64x\r\n"
		"EDX = 0x%016I64x\r\n"
		"ESI = 0x%016I64x\r\n"
		"EDI = 0x%016I64x\r\n"
		"EIP = 0x%016I64x\r\n"
		"ESP = 0x%016I64x\r\n"
		"EBP = 0x%016I64x\r\n"
		"\r\n",
		pxExceptionPointers->ContextRecord->Rax,
		pxExceptionPointers->ContextRecord->Rbx,
		pxExceptionPointers->ContextRecord->Rcx,
		pxExceptionPointers->ContextRecord->Rdx,
		pxExceptionPointers->ContextRecord->Rsi,
		pxExceptionPointers->ContextRecord->Rdi,
		pxExceptionPointers->ContextRecord->Rip,
		pxExceptionPointers->ContextRecord->Rsp,
		pxExceptionPointers->ContextRecord->Rbp
#else
		"EAX = 0x%08x\r\n"
		"EBX = 0x%08x\r\n"
		"ECX = 0x%08x\r\n"
		"EDX = 0x%08x\r\n"
		"ESI = 0x%08x\r\n"
		"EDI = 0x%08x\r\n"
		"EIP = 0x%08x\r\n"
		"ESP = 0x%08x\r\n"
		"EBP = 0x%08x\r\n"
		"\r\n",
		pxExceptionPointers->ContextRecord->Eax,
		pxExceptionPointers->ContextRecord->Ebx,
		pxExceptionPointers->ContextRecord->Ecx,
		pxExceptionPointers->ContextRecord->Edx,
		pxExceptionPointers->ContextRecord->Esi,
		pxExceptionPointers->ContextRecord->Edi,
		pxExceptionPointers->ContextRecord->Eip,
		pxExceptionPointers->ContextRecord->Esp,
		pxExceptionPointers->ContextRecord->Ebp
#endif
		);
	AddToReportTXT( szTempString );
	
#if defined( ASURA_TOOLS)
	_snprintf( szTempString, iTempStringSize, 
		"<cpu>"
#if defined( _WIN64)
		"<register name=\"RAX\" value=\"0x%016I64x\"/>"
		"<register name=\"RBX\" value=\"0x%016I64x\"/>"
		"<register name=\"RCX\" value=\"0x%016I64x\"/>"
		"<register name=\"RDX\" value=\"0x%016I64x\"/>"
		"<register name=\"RSI\" value=\"0x%016I64x\"/>"
		"<register name=\"RDI\" value=\"0x%016I64x\"/>"
		"<register name=\"RIP\" value=\"0x%016I64x\"/>"
		"<register name=\"RSP\" value=\"0x%016I64x\"/>"
		"<register name=\"RBP\" value=\"0x%016I64x\"/>"
#else
		"<register name=\"EAX\" value=\"0x%08x\"/>"
		"<register name=\"EBX\" value=\"0x%08x\"/>"
		"<register name=\"ECX\" value=\"0x%08x\"/>"
		"<register name=\"EDX\" value=\"0x%08x\"/>"
		"<register name=\"ESI\" value=\"0x%08x\"/>"
		"<register name=\"EDI\" value=\"0x%08x\"/>"
		"<register name=\"EIP\" value=\"0x%08x\"/>"
		"<register name=\"ESP\" value=\"0x%08x\"/>"
		"<register name=\"EBP\" value=\"0x%08x\"/>"
#endif
		"</cpu>",
#if defined( _WIN64)
		pxExceptionPointers->ContextRecord->Rax,
		pxExceptionPointers->ContextRecord->Rbx,
		pxExceptionPointers->ContextRecord->Rcx,
		pxExceptionPointers->ContextRecord->Rdx,
		pxExceptionPointers->ContextRecord->Rsi,
		pxExceptionPointers->ContextRecord->Rdi,
		pxExceptionPointers->ContextRecord->Rip,
		pxExceptionPointers->ContextRecord->Rsp,
		pxExceptionPointers->ContextRecord->Rbp
#else
		pxExceptionPointers->ContextRecord->Eax,
		pxExceptionPointers->ContextRecord->Ebx,
		pxExceptionPointers->ContextRecord->Ecx,
		pxExceptionPointers->ContextRecord->Edx,
		pxExceptionPointers->ContextRecord->Esi,
		pxExceptionPointers->ContextRecord->Edi,
		pxExceptionPointers->ContextRecord->Eip,
		pxExceptionPointers->ContextRecord->Esp,
		pxExceptionPointers->ContextRecord->Ebp
#endif
		);
	AddToReportXML( szTempString );
		
	_snprintf( szTempString, iTempStringSize, "</exception>");
	AddToReportXML( szTempString);
	
	AddToReportXML( "</crash>" );
#endif
	
	//SymCleanup( GetProcess() );
	
	if( EXCEPTION_CONTINUE_EXECUTION == uReturnCode)
	{
		// DLH - I do not like doing this inside the crash handler but the problem needs to be reported
		Asura_Assertf(false, ("unhandled exception\n\n%s\n\n", GetReportTXT()));
	}
	Asura_Atomic::CompareAndSwap( Asura_PC_SEH_Handler_ProcessException_CodeLock, 1, 0);
	return( uReturnCode);
}

#endif //defined( ASURA_ENABLE_SEH_HANDLING)

//*******************************************************************
//
// (c) 2006 Rebellion (Derby)
//
// Warning: This computer program is protected by copyright law and 
// international treaties. Unauthorized reproduction or distribution
// of this program, or any portion of it, may result in severe civil and
// criminal penalties, and will be prosecuted to the maximum extent 
// possible under the law.
//
//*******************************************************************

