/**
*
* \file Asura_PlatformCore.cpp
*
* Created by Kevin Lea - 27/11/2001 12:11:07
*
* Copyright (C) 2001 Rebellion, All Rights Reserved.
*
* \brief
*
**/

#include "Core/Asura.h"
#include "Core/Asura_Core.h"
#ifdef ASURA_USE_PROFILING_SYSTEM
#include "Debugging/Asura_CPUProfile_Renderer.h"
#endif
#if defined( ASURA_SUPPORT_MULTITHREADED_JOB_QUEUE)
#include "Jobs/Asura_Job_Manager.h"
#endif

// PC includes
#include "Core/Asura_Windows.h"
#ifdef ASURA_PC_SUPPORT_STEAMWORKS
#include "Network/Steamworks/Asura_PC_Steamworks.h"
#endif

// library includes
#include <time.h>



void Asura_Core::PlatformInitialise(void)
{
	// XOR the time with the process ID. This should give us different seeds 
	// even if the time is exactly the same for 2 processes starting on the 
	// same machine.
	DWORD uProcessID = GetCurrentProcessId();
	u_int uSeed = static_cast<u_int>(time(NULL)) ^ uProcessID;
	Asura_Random::SetSeed(uSeed);

#ifdef ASURA_USE_PROFILING_SYSTEM
    Asura_CPUProfile_Renderer::Initialise();
#endif
#if defined( ASURA_SUPPORT_MULTITHREADED_JOB_QUEUE)
    Asura_Job_Manager::Initialise();
#endif

#ifdef ASURA_PC_SUPPORT_STEAMWORKS
	Asura_PC_Steamworks::Initialise();
#endif
}

void Asura_Core::PlatformShutdown(void)
{
#if defined( ASURA_SUPPORT_MULTITHREADED_JOB_QUEUE)
    Asura_Job_Manager::Shutdown();
#endif
#ifdef ASURA_USE_PROFILING_SYSTEM
    Asura_CPUProfile_Renderer::Shutdown();
#endif

#ifdef ASURA_PC_SUPPORT_STEAMWORKS
	Asura_PC_Steamworks::Shutdown();
#endif
}

void Asura_Core::PlatformReset(void)
{
}

bool Asura_Core::Platform_IsMainThread(void)
{
	return Asura_Windows::IsMainThread();
}
