
///////////////////////////////////////////////////////////////////////////////
//
// File   : Asura_Platform_ScreenShots.cpp (PC version)
// Author : Andy Tate
// Date   : 10-Jun-2005
//
// Copyright (C) 2005 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

#ifdef ASURA_USE_SCREENSHOT_SYSTEM

// This file's header...
#include "Debugging/Asura_ScreenShot_System.h"

// Asura header-files...
#include "Core/Asura_Stream.h"
#include "Core/Asura_UserInput.h"
#include "Rendering/Asura_Render.h"

// PC-specific
#include "Core/Asura_Windows.h"
#include "Rendering/Asura_Direct3D.h"
#include "Rendering/Asura_Direct3D_UserOptions.h"

// Game header-files...


////////////////////////////////////////////////////////////////////////////////
// Type declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Private functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Protected functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Public functions...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_ScreenShot_System::Platform_RenderProgressBar
// Author       : Andy Tate
// Date         : 16-Jun-2005
//
// Description  : Handles the platform-specific rendering of the progress-bar.
//
// Parameters   : References to the position and size of the progress bar along
//				  with the colour it's to be rendered.
//
// Return value : None.
////////////////////////////////////////////////////////////////////////////////

void Asura_ScreenShot_System::Platform_RenderProgressBar (const Asura_Vector_2& xPosition,
														  const Asura_Vector_2& xSize,
														  const u_int uARGB)
{
	// Prepare the debug quad...

// (MH - 29.06.2005) this is appearing in the captured image so I'm turning it off
//	Asura_Render::Overlay.TextureIndex		= ASURA_TEXTURE_NONE;
//	Asura_Render::Overlay.TranslucencyMode	= ASURA_TRANSLUCENCY_OFF;
//	Asura_Render::Overlay.Colour			= uARGB;
//
//	Asura_Render::Overlay.SetRectXY	(xPosition.x, xPosition.y, xSize.x, xSize.y);
//
//
//	// Render the quad...
//
//	Asura_Render::Overlay.Render	();
//
//	Asura_Direct3D::FlushFrame		();
}

/// End of function 'Asura_ScreenShot_System::Platform_RenderProgressBar'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_ScreenShot_System::Platform_GetScreenDimensions
// Author       : Andy Tate
// Date         : 10-Jun-2005
//
// Description  : Gets the specified screen dimensions.
//
// Parameters   : Pointers to the position, size and bit-depth to get, if these
//				  are NULL then don't bother fetching the data.
//
// Return value : None.
////////////////////////////////////////////////////////////////////////////////

void Asura_ScreenShot_System::Platform_GetScreenDimensions (Asura_IntVector_2* pxPosition,
															Asura_IntVector_2* pxSize,
															u_int* puFormat)
{
	// Do we need to get the position?

	if (NULL != pxPosition)
	{
		// Yes - do so...

		pxPosition->iX	= 0;
		pxPosition->iY	= 0;
	}


	// Do we need to get the size?

	if (NULL != pxSize)
	{
		// Yes - do so...

		pxSize->iX	= Asura_Direct3D::GetPresentParameters().BackBufferWidth;
		pxSize->iY	= Asura_Direct3D::GetPresentParameters().BackBufferHeight;
	}


	// Do we need to get the bit-depth?

	if (NULL != puFormat)
	{
		// Decide upon the bit-depth...

		*puFormat	= Asura_Direct3D::GetPresentParameters().BackBufferFormat;
	}
}

/// End of function 'Asura_ScreenShot_System::Platform_GetScreenDimensions'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_ScreenShot_System::Platform_WriteScreenAs24Bit
// Author       : Andy Tate
// Date         : 10-Jun-2005
//
// Description  : Writes the specified portion of the screen to the specified
//				  stream as a 24-bit BMP.
//
// Parameters   : A reference to the stream to write to, along with the position
//				  size and format of the screen.
//
// Return value : None.
////////////////////////////////////////////////////////////////////////////////

void Asura_ScreenShot_System::Platform_WriteScreenAs24Bit (Asura_Stream& xStream,
														   const Asura_IntVector_2& xPosition,
														   const Asura_IntVector_2& xSize,
														   const u_int uFormat)
{
	Direct3D_Surface*	pxBackBufferSurface;
	HRESULT				hResult;


	// Ensure that we've finished rendering...

	Asura_Render::FlushDisplay	();


	// Get the back-buffer as a surface...

	hResult	= Asura_Direct3D::D3DDevice->GetRenderTarget (0, &pxBackBufferSurface);

	if (FAILED(hResult))
	{
		// Failed to get the render-target...

		Asura_Assert	(false, "Failed to get render-target 0 for screenshot.");
	}

	else
	{
		D3DLOCKED_RECT	xD3DRect;
		HRESULT			hResult;


		// Lock the back-buffer rectange...

		hResult	= pxBackBufferSurface->LockRect	(&xD3DRect, NULL, D3DLOCK_READONLY);

		if (FAILED(hResult))
		{
			// Failed to lock the back-buffer...

			Asura_Assert	(false, "Failed to lock back-buffer for screenshot.");
		}

		else
		{
			D3DSURFACE_DESC	xD3DSurfaceDesc;


			// Get the surface description...

			hResult	= pxBackBufferSurface->GetDesc	(&xD3DSurfaceDesc);

			if (FAILED(hResult))
			{
				// Falied to get the back-buffer description...

				Asura_Assert	(false, "Failed to get backbuffer description for screenshot.");
			}

			else
			{
				const int				iWidth			= xSize.iX;
				const int				iHeight			= xSize.iY;
				const u_int				uBytesPerLine	= ((iWidth * 3)+3)&~3;//Each line needs to be a multiple of 4 bytes

				const unsigned char*	pucBackBuffer	= static_cast<unsigned char *>(xD3DRect.pBits) + (xD3DRect.Pitch * (iHeight - 1));
				unsigned char*			pucOutputBuffer	= new unsigned char [uBytesPerLine];
				
				///make sure any padding at the end of the line will be zeroed.
				u_int uPadPos = iWidth*3;
				while(uPadPos < uBytesPerLine)
				{
					pucOutputBuffer[uPadPos++] = 0;
				}

				// Assume that the back buffer format is either "D3DFMT_X8R8G8B8" or "D3DFMT_A8R8G8B8"...
			
				for (int y = (iHeight - 1); y >= 0; y--)
				{
					const unsigned int*	puSource	= reinterpret_cast<const unsigned int*>(pucBackBuffer);
					unsigned char*		pucDest		= pucOutputBuffer;


					// Go through each pixel on this line...

					for (int x = 0; x < iWidth; x++)
					{
						const unsigned int	uColour	= *(puSource++);


						// Write the colour to our output buffer...

						*(pucDest++)	= static_cast<unsigned char>(uColour & 0x000000FF);				// Blue
						*(pucDest++)	= static_cast<unsigned char>((uColour & 0x0000FF00) >> 8);		// Green
						*(pucDest++)	= static_cast<unsigned char>((uColour & 0x00FF0000) >> 16);		// Red
					}


					// Write this line...

					xStream.WriteData	(uBytesPerLine, pucOutputBuffer);


					// Go back a line...

					pucBackBuffer	-= xD3DRect.Pitch;
				}


				// Free-up the memory...

				delete []	(pucOutputBuffer);
			}


			// Unlock the back-buffer...

			pxBackBufferSurface->UnlockRect ();
		}


		// Release the texture...

		pxBackBufferSurface->Release	();
	}
}

/// End of function 'Asura_ScreenShot_System::Platform_WriteScreenAs24Bit'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_ScreenShot_System::Platform_AddScreenToFPBuffer
// Author       : Andy Tate
// Date         : 10-Jun-2005
//
// Description  : Adds the specified portion of the screen to the floating-point
//				  RGB buffer.
//
// Parameters   : 
//
// Return value : None.
////////////////////////////////////////////////////////////////////////////////

void Asura_ScreenShot_System::Platform_AddScreenToFPBuffer (const Asura_IntVector_2& xPosition,
															const Asura_IntVector_2& xSize,
															const u_int uFormat,
															float* pfFPBuffer)
{
	Direct3D_Surface*	pxBackBufferSurface;
	HRESULT				hResult;


	// Ensure that we've finished rendering...

	Asura_Render::FlushDisplay	();


	// Get the back-buffer as a surface...

	hResult	= Asura_Direct3D::D3DDevice->GetRenderTarget (0, &pxBackBufferSurface);

	if (FAILED(hResult))
	{
		// Failed to get the render-target...

		Asura_Assert	(false, "Failed to get render-target 0 for screenshot.");
	}

	else
	{
		D3DLOCKED_RECT	xD3DRect;


		// Lock the back-buffer rectange...

		hResult	= pxBackBufferSurface->LockRect	(&xD3DRect, NULL, D3DLOCK_READONLY);

		if (FAILED(hResult))
		{
			// Failed to lock the back-buffer...

			Asura_Assert	(false, "Failed to lock backbuffer for screenshot.");
		}

		else
		{
			D3DSURFACE_DESC	xD3DSurfaceDesc;


			// Get the surface description...

			hResult	= pxBackBufferSurface->GetDesc (&xD3DSurfaceDesc);

			if (FAILED(hResult))
			{
				// Falied to get the back-buffer description...

				Asura_Assert	(false, "Failed to get backbuffer description for screenshot.");
			}

			else
			{
				const int				iWidth			= xSize.iX;
				const int				iHeight			= xSize.iY;

				const unsigned char*	pucBackBuffer	= static_cast<unsigned char *>(xD3DRect.pBits) + (xD3DRect.Pitch * (iHeight - 1));


				// Assume that the back buffer format is either "D3DFMT_X8R8G8B8" or "D3DFMT_A8R8G8B8"...
			
				for (int y = (iHeight - 1); y >= 0; y--)
				{
					const unsigned int*	puSource	= reinterpret_cast<const unsigned int*>(pucBackBuffer);


					// Go through each pixel on this line...

					for (int x = 0; x < iWidth; x++)
					{
						const unsigned int	uColour	= *(puSource++);


						// Write the colour to our output buffer...

						*(pfFPBuffer++)	+= static_cast<float>(uColour & 0x000000FF);			// Blue
						*(pfFPBuffer++)	+= static_cast<float>((uColour & 0x0000FF00) >> 8);		// Green
						*(pfFPBuffer++)	+= static_cast<float>((uColour & 0x00FF0000) >> 16);	// Red
					}


					// Go back a line...

					pucBackBuffer	-= xD3DRect.Pitch;
				}
			}


			// Unlock the back-buffer...

			pxBackBufferSurface->UnlockRect	();
		}


		// Release the back-buffer...

		pxBackBufferSurface->Release	();
	}
}

/// End of function 'Asura_ScreenShot_System::Platform_AddScreenToFPBuffer'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_ScreenShot_System::Platform_BeginHiResRender
// Author       : Andy Tate
// Date         : 13-Jun-2005
//
// Description  : Handles the platform specific start of the high-resolution render.
//
// Return value : None.
////////////////////////////////////////////////////////////////////////////////

void Asura_ScreenShot_System::Platform_BeginHiResRender (void)
{
	// Turn off the mip-mapping...

	Asura_Direct3D::D3DDevice->SetSamplerState	(0, D3DSAMP_MIPFILTER, D3DTEXF_NONE);
	Asura_Direct3D::D3DDevice->SetSamplerState	(1, D3DSAMP_MIPFILTER, D3DTEXF_NONE);
}

/// End of function 'Asura_ScreenShot_System::Platform_BeginHiResRender'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_ScreenShot_System::Platform_EndHiResRender
// Author       : Andy Tate
// Date         : 13-Jun-2005
//
// Description  : Handles the platform specific end of the high-resolution render.
//
// Return value : None.
////////////////////////////////////////////////////////////////////////////////

void Asura_ScreenShot_System::Platform_EndHiResRender (void)
{
	// Restore the render states...

	Asura_Direct3D::ResetStates	();
}

/// End of function 'Asura_ScreenShot_System::Platform_EndHiResRender'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_ScreenShot_System::Platform_TakeScreenShot
// Author       : Andy Tate
// Date         : 10-Jun-2005
//
// Description  : Handles the taking of screen-shots.
//
// Parameters   : The name of the file to be saved.
//
// Return value : The result of the screen shot function.
////////////////////////////////////////////////////////////////////////////////

int Asura_ScreenShot_System::Platform_TakeScreenShot (const char* szFilename)
{
	Asura_IntVector_2	xScreenPosition;
	Asura_IntVector_2	xScreenSize;
	u_int			uScreenFormat;


	// Take the normal screen-shot as a BMP...

	Platform_GetScreenDimensions	(&xScreenPosition, &xScreenSize, &uScreenFormat);
	return							(BMPTakeScreenShot(szFilename,  xScreenPosition, xScreenSize, uScreenFormat));
}

/// End of function 'Asura_ScreenShot_System::Platform_TakeScreenShot'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_ScreenShot_System::Platform_NotTakingScreenShot
// Author       : Andy Tate
// Date         : 10-Jun-2005
//
// Description  : Handles the case of being in screen-shot mode but not actually
//				  taking a screen-shot.
//
// Return value : None.
////////////////////////////////////////////////////////////////////////////////

void Asura_ScreenShot_System::Platform_NotTakingScreenShot (void)
{
	// Nothing to do...
}

/// End of function 'Asura_ScreenShot_System::Platform_NotTakingScreenShot'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_ScreenShot_System::Platform_AllocateFloatBuffer
// Author       : Mike Healey
// Date         : 28-Jun-2005
//
// Description  : returns float buffer big enough for screenshots.
//
// Return value : ptr to float buffer.
////////////////////////////////////////////////////////////////////////////////
float* Asura_ScreenShot_System::Platform_AllocateFloatBuffer(u_int uNumFloats)
{
	return new float[uNumFloats];
}
/// End of function 'Asura_ScreenShot_System::Platform_AllocateFloatBuffer'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_ScreenShot_System::Platform_FreeFloatBuffer
// Author       : Mike Healey
// Date         : 28-Jun-2005
//
// Description  : frees float buffer allocated earlier (if necessary)
////////////////////////////////////////////////////////////////////////////////
void Asura_ScreenShot_System::Platform_FreeFloatBuffer(float* pfBuffer)
{
	delete[] pfBuffer;
}
/// End of function 'Asura_ScreenShot_System::Platform_FreeFloatBuffer'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_ScreenShot_System::Platform_RenderPrompt
// Author       : Mike Healey
// Date         : 28-Jun-2005
//
// Description  : render user input prompt - not supported on PC
////////////////////////////////////////////////////////////////////////////////
void Asura_ScreenShot_System::Platform_RenderPrompt(void)
{
}
/// End of function 'Asura_ScreenShot_System::Platform_RenderPrompt'. ///

#endif // #ifdef ASURA_USE_SCREENSHOT_SYSTEM

/// End of file. ///
