//////////////////////////////////////////////////////////////////////////////
//
// Asura_Platform_FSFX_ColourMap.cpp
//
// Copyright (c) 2007 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

/******************************************************************************/
/* Included Files                                                             */
/******************************************************************************/

#include "Core/Asura.h"

#if defined ASURA_USE_FSFX && defined ASURA_TARGET_SM3

#include "FSFX/Asura_FSFX_Component.h"

// Asura Platform-independent
#include "Rendering/Asura_Render.h"
#include "Rendering/Asura_Viewport.h"

// Asura PC
#include "FSFX/Asura_Platform_FSFX_SM30.h"
#include "Rendering/Asura_Direct3D.h"
#include "Rendering/Asura_Direct3D_BackBufferCapture.h"
#include "Rendering/Asura_Direct3D_Effect.h"
#include "Rendering/Asura_Direct3D_Effects_System.h"

// Misc
#include <d3d9.h>

/******************************************************************************/
/* Asura_FSFX_Component_ColourMap Class Functions                             */
/******************************************************************************/

void Asura_FSFX_Component_ColourMap::Platform_Render( const float fIntensity )
{
	Asura_Direct3D_Effect* const pxEffect = Asura_Platform_FSFX_SM30::g_pxColourMapEffect;
	if( !pxEffect || !pxEffect->IsD3DEffectValid() ) return;

	SCOPEDPIXEVENT(( 0xFFE0D0C0, L"FSFX: Colour Map" ));

	// Setup
	Asura_Direct3D_BackBufferCapture::CaptureCurrentBackBuffer();

	pxEffect->SetParameterTexture( Asura_Platform_FSFX_SM30::g_xColourMapInitialImageHandle,
								   Asura_Direct3D_BackBufferCapture::GetCaptureTexture() );

	Direct3D_BaseTexture* const pxColourMap
		= Asura_Direct3D::GetDirect3DTextureByIndex( m_iColourMapIndex );

	pxEffect->SetParameterTexture( Asura_Platform_FSFX_SM30::g_xColourMapHandle, pxColourMap );
	pxEffect->SetParameter( Asura_Platform_FSFX_SM30::g_xColourMapIntensityHandle, fIntensity );

	Asura_Render::AllowFullScreenRendering();

	Asura_Render::CurrentStates.RequestAlphaTest( false );
	Asura_Render::CurrentStates.RequestCullMode( ASURA_CULLMODE_NONE );
	Asura_Render::CurrentStates.RequestTranslucencyMode( ASURA_TRANSLUCENCY_OFF );
	Asura_Render::CurrentStates.RequestZBufferEnabled( false );

	Direct3D_VertexDeclaration* const pxVD = Asura_Platform_FSFX_SM30::g_pxColourMapVertexDecl;
	Asura_Direct3D::D3DDevice->SetVertexDeclaration( pxVD );

	const float fWidth = static_cast< float >( Asura_Viewport::GetWidth() );
	const float fHeight = static_cast< float >( Asura_Viewport::GetHeight() );

	const ColourMapVertex axVerts[] =
	{
		{ -1.0f,  1.0f, 0.5f / fWidth,            0.5f / fHeight },
		{  1.0f,  1.0f, (fWidth + 0.5f) / fWidth, 0.5f / fHeight },
		{  1.0f, -1.0f, (fWidth + 0.5f) / fWidth, (fHeight + 0.5f) / fHeight },
		{ -1.0f, -1.0f, 0.5f / fWidth,            (fHeight + 0.5f) / fHeight },
	};

	Asura_Direct3D_Effects_System::BeginEffect( *pxEffect );
	pxEffect->BeginPass(0);

	Asura_Direct3D::D3DDevice->DrawPrimitiveUP( D3DPT_TRIANGLEFAN, 2, axVerts,
												sizeof( ColourMapVertex ) );

	pxEffect->EndPass();
	Asura_Direct3D_Effects_System::FlushEffects();

	// Reset this parameter to NULL since the texture is in the default pool (for device resets)
	pxEffect->SetParameterTexture( Asura_Platform_FSFX_SM30::g_xColourMapInitialImageHandle, NULL );
	Asura_Direct3D_BackBufferCapture::AdviseNextFrame();

	for( u_int u = 0; u < 16; ++u )
	{
		Asura_Direct3D::D3DDevice->SetSamplerState( 0, D3DSAMP_SRGBTEXTURE, FALSE );
	}
}

#endif // defined ASURA_USE_FSFX && defined ASURA_TARGET_SM3
