//*******************************************************************
//
// File: Asura_FSFX_FishEye.cpp        
//
// Created: 9/10/2007 
//
// Description: Handles the Fisheye fullscreen effect
//
//*******************************************************************

// system libraries
#include "Core/Asura.h"

#if defined( ASURA_USE_FSFX) && defined (ASURA_TARGET_SM3)

// main header
#include "FSFX/Asura_FSFX_System.h"
#include "FSFX/Asura_FSFX_Template.h"
#include "FSFX/Asura_FSFX_Component.h"

#include "Core/Asura_Timers.h"

#include "Camera/Asura_Camera.h"
#include "Rendering/Asura_ViewPort.h"
#include "Rendering/Asura_Render.h"

#include "Rendering/Asura_Direct3D_RenderTargetManager.h"
#include "Rendering/Asura_Direct3D_BackBufferCapture.h"
#include "Rendering/Asura_Direct3D_Shaders.h"
#include "Rendering/Asura_Direct3D_Effects_System.h"

// other libraries

// internal dependencies

//*******************************************************************
// Constants
//*******************************************************************


//*******************************************************************
// Local Classes
//*******************************************************************


struct FishEyeVertex
{
	float m_fX, m_fY, m_fZ, m_fW;
	float m_fU, m_fV;
};


const Direct3D_VertexElement axVERTEX_DECLARATION[] =
{
	{ 0,  0, D3DDECLTYPE_FLOAT4, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_POSITION, 0 },
	{ 0, 16, D3DDECLTYPE_FLOAT2, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_TEXCOORD, 0 },
	D3DDECL_END()
};

static Direct3D_VertexDeclaration* g_pxVertexDecl = NULL;



//*******************************************************************
// Function Definitions
//*******************************************************************

static Asura_Direct3D_Effect* g_pxEffect	= NULL;
static D3DXHANDLE g_xInitialImageHandle		= NULL;
static D3DXHANDLE g_xFOVHandle				= NULL;
static D3DXHANDLE g_xMultiplierHandle		= NULL;
//static D3DXHANDLE g_xIntensityHandle;
static D3DXHANDLE g_xModeHandle				= NULL;
static D3DXHANDLE g_xWidthHandle			= NULL;
static D3DXHANDLE g_xHeightHandle			= NULL;
static D3DXHANDLE g_xScaleHandle			= NULL;
static D3DXHANDLE g_xOneOverTanFOV			= NULL;
static D3DXHANDLE g_xSinHalfFOV				= NULL;
static D3DXHANDLE g_xTanHalfFOV				= NULL;


static void OnEffectUnloaded(Asura_Direct3D_Effect* pxEffect)
{
	g_pxEffect = NULL;
}

static void OnEffectLoaded(Asura_Direct3D_Effect* pxEffect)
{
	g_pxEffect = Asura_Direct3D_Effects_System::GetEffect("FSFX_SM30_FishEye");

	g_xInitialImageHandle = g_pxEffect->GetD3DEffect()->GetParameterByName( NULL, "g_xTexture" );
	g_xFOVHandle = g_pxEffect->GetD3DEffect()->GetParameterByName( NULL, "g_fFOV" );
	g_xMultiplierHandle = g_pxEffect->GetD3DEffect()->GetParameterByName( NULL, "g_fMult" );
	//g_xIntensityHandle = g_pxEffect->GetD3DEffect()->GetParameterByName( NULL, "g_fIntensity" );
	g_xModeHandle = g_pxEffect->GetD3DEffect()->GetParameterByName( NULL, "g_iFishEyeMode" );
	g_xWidthHandle = g_pxEffect->GetD3DEffect()->GetParameterByName( NULL, "g_fWidth" );
	g_xHeightHandle = g_pxEffect->GetD3DEffect()->GetParameterByName( NULL, "g_fHeight" );
	g_xScaleHandle = g_pxEffect->GetD3DEffect()->GetParameterByName( NULL, "g_xScale" );
	g_xOneOverTanFOV = g_pxEffect->GetD3DEffect()->GetParameterByName( NULL, "g_fOneOverTanFOV" );
	g_xSinHalfFOV = g_pxEffect->GetD3DEffect()->GetParameterByName( NULL, "g_fSinHalfFOV" );
	g_xTanHalfFOV = g_pxEffect->GetD3DEffect()->GetParameterByName( NULL, "g_fTanHalfFOV" );

#if ASURA_LOAD_EFFECT_FILES == 1
	g_pxEffect->SetOnLoadCallback(OnEffectLoaded);
	g_pxEffect->SetOnUnloadCallback(OnEffectUnloaded);
#endif

}


void Asura_FSFX_Component_SM30_FishEye::Platform_Initialise()
{
	// when doing batch exporting of levels sometimes we don't have a D3D device to use
	if ( NULL == Asura_Direct3D::D3DDevice )
	{
		return;
	}

	OnEffectLoaded(NULL);


	Asura_Direct3D::D3DDevice->CreateVertexDeclaration( axVERTEX_DECLARATION, &g_pxVertexDecl );
}



void Asura_FSFX_Component_SM30_FishEye::Platform_Reset()
{
	Platform_Shutdown();
	Platform_Initialise();
}


void Asura_FSFX_Component_SM30_FishEye::Platform_Shutdown()
{
	// when doing batch exporting of levels sometimes we don't have a D3D device to use
	if ( NULL == Asura_Direct3D::D3DDevice )
	{
		return;
	}
	g_pxVertexDecl->Release();
	g_pxVertexDecl = NULL;

}


void Asura_FSFX_Component_SM30_FishEye::Platform_Render( const float fIntensity )
{
	Asura_Direct3D_Effect* const pxEffect = g_pxEffect;
	if( !pxEffect || !pxEffect->IsD3DEffectValid() ) return;
	SCOPEDPIXEVENT(( D3DCOLOR_XRGB( 0xFF, 0xFF, 0xFF ), L"FSFX: FishEye"));


	// Setup
	Asura_Direct3D_BackBufferCapture::CaptureCurrentBackBuffer();

	pxEffect->SetParameterTexture( g_xInitialImageHandle, Asura_Direct3D_BackBufferCapture::GetCaptureTexture() );

	float fDynamicIntensity = fIntensity;
	if ( m_uFlags & DynamicSine )
	{
		// Calculate sine value and clip
		const float fTime = Asura_Timers::GetLocalScaledTime();
		const float fSineClipped = Asura_Maths::Max( m_fSineClip, Asura_Maths::Sine( fTime * m_fSpeed ) / 2.0f + 0.5f );

		// Work out max value after clipping
		float fScaleValue = 1.0f;
		if ( m_fSineClip != 1.0f )
		{
			fScaleValue = 1.0f / ( 1.0f - m_fSineClip );
		}

		// Normalise sine back up to fill 0 to 1 range
		const float fParameter = ( fSineClipped - m_fSineClip ) * fScaleValue;

		// Lerp and multiply by intensity
		fDynamicIntensity *= Asura_Maths::Lerp( m_fMinMultiplier, m_fMaxMultiplier, fParameter );
	}

//	pxEffect->SetParameter( g_xIntensityHandle, fIntensity );
	//pxEffect->SetParameter( g_xIntensityHandle, 1.0f ); // makes little sense to alpha out effect ?
	pxEffect->SetParameter( g_xMultiplierHandle, m_fMultiplier * fDynamicIntensity);
	pxEffect->SetParameter( g_xFOVHandle, m_fFov*0.5f);
	pxEffect->SetParameter( g_xModeHandle, m_eType);

	pxEffect->SetParameter( g_xOneOverTanFOV, 1.0f / tanf(m_fFov * 0.5f) );
	pxEffect->SetParameter( g_xSinHalfFOV, sinf( m_fFov * 0.5f * 0.5f ) );
	pxEffect->SetParameter( g_xTanHalfFOV, tanf( m_fFov * 0.5f * 0.5f ) );

	Asura_Render::CurrentStates.RequestAlphaTest( false );
	Asura_Render::CurrentStates.RequestCullMode( ASURA_CULLMODE_NONE );
	Asura_Render::CurrentStates.RequestTranslucencyMode( ASURA_TRANSLUCENCY_NORMAL );
	Asura_Render::CurrentStates.RequestZBufferEnabled( false );

	Asura_Direct3D::D3DDevice->SetVertexDeclaration( g_pxVertexDecl );

	const float fWidth = static_cast< float >( Asura_Viewport::GetWidth() );
	const float fHeight = static_cast< float >( Asura_Viewport::GetHeight() );

	D3DXVECTOR4 xScale;
	if (fWidth > fHeight)
	{
		xScale.x = m_fXScale;
		xScale.y = m_fYScale * fHeight / fWidth;
	} 
	else 
	{
		xScale.x = m_fXScale;
		xScale.y = m_fYScale * fHeight / fWidth;
	}
	xScale.z = 1.0f / xScale.x;
	xScale.w = 1.0f / xScale.y;
	pxEffect->SetParameter( g_xScaleHandle, xScale );

	const Asura_ViewingArea *pxViewingArea = Asura_Camera::GetViewingArea();
	const float fSqrt2 = 1.0f / 1.4142135623730950488016887242097f; // 1.0f / sqrt(2)
	
	float fPosLeft = -1.0f;
	float fPosRight = 1.0f;
	float fPosBottom = -1.0f;
	float fPosTop = 1.0f;
	float fUVLeft = -fSqrt2 + pxViewingArea->m_fXOffset * 2.0f * fSqrt2;
	float fUVRight = fUVLeft + pxViewingArea->m_fWidth * 2.0f * fSqrt2;
	float fUVBottom = -fSqrt2 + pxViewingArea->m_fYOffset * 2.0f * fSqrt2;
	float fUVTop = fUVBottom + pxViewingArea->m_fHeight * 2.0f * fSqrt2;
	const FishEyeVertex axVerts[] =
	{
		{ fPosLeft,  fPosTop, 0.0f, 1.0f,		fUVLeft, fUVBottom },
		{ fPosRight, fPosTop, 0.0f, 1.0f,		fUVRight, fUVBottom },
		{ fPosRight, fPosBottom, 0.0f, 1.0f,	fUVRight,  fUVTop },
		{ fPosLeft, fPosBottom, 0.0f, 1.0f,		fUVLeft,  fUVTop },
	};

	//Asura_Direct3D::SetUsingDeferredRenderingTarget( false );
	Asura_Direct3D::BindDefaultTargets();
	Asura_Direct3D::D3DDevice->SetStreamSource(0, NULL, 0, sizeof( FishEyeVertex ));

	Asura_Direct3D_Effects_System::BeginEffect( *pxEffect );
	{
		pxEffect->BeginPass(m_eType);
		{
			Asura_Direct3D::D3DDevice->DrawPrimitiveUP( D3DPT_TRIANGLEFAN, 2, axVerts, sizeof( FishEyeVertex ) );
		}
		pxEffect->EndPass();
	}
 	Asura_Direct3D_Effects_System::EndEffect(*pxEffect);

	// Reset this parameter to NULL since the texture is in the default pool (for device resets)
	pxEffect->SetParameterTexture( g_xInitialImageHandle, NULL );

	////// Finished
	Asura_Render::CurrentStates.RequestZBufferEnabled(true);
	Asura_Render::CurrentStates.RequestZBufferWriteEnabled(true);

	Asura_Direct3D_Shaders::RequestPixelShader(ASURA_D3D_PS_NONE);
	Asura_Render::CurrentStates.RequestZBufferCompareMode(ASURA_ZBUFFERCMP_LESSEQUAL);

	Asura_Direct3D_BackBufferCapture::AdviseNextFrame();

	Asura_Render::ApplyCameraSettings();

	for( u_int u = 0; u < 16; ++u )
	{
		Asura_Direct3D::D3DDevice->SetSamplerState( 0, D3DSAMP_SRGBTEXTURE, FALSE );
	}
}


//*******************************************************************
//
// (c) 2007 Rebellion
//
// Warning: This computer program is protected by copyright law and 
// international treaties. Unauthorized reproduction or distribution
// of this program, or any portion of it, may result in severe civil and
// criminal penalties, and will be prosecuted to the maximum extent 
// possible under the law.
//
//*******************************************************************

#endif //defined( ASURA_USE_FSFX)
