//////////////////////////////////////////////////////////////////////////////
//
// Asura_PC_Input_Joystick.h
//
// Created by AAM - 14/01/2004 14:36:17
//
// Copyright (C) 2004 Rebellion, All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

#if defined ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif

#if !defined __ASURA_PC_INPUT_JOYSTICK_H__
#define __ASURA_PC_INPUT_JOYSTICK_H__

/******************************************************************************/
/* Included Files                                                             */
/******************************************************************************/

#include "Core/Asura_List.h"
#include "Core/Asura_String.h"
#include "Core/Asura_UserInput.h"
#include "Input/Asura_DirectInput.h"

/******************************************************************************/
/* Asura_PC_Input_Joystick                                                    */
/******************************************************************************/

#ifdef ASURA_PC_SUPPORT_XINPUT
class Asura_PC_Input_XInputPad;
#endif

class Asura_PC_Input_Joystick
{
	// (AAM - 19.01.2004)
	// for m_pxDevice
	friend class Asura_Rumble_System;
	friend class Asura_Rumble_DirectInput;

#ifdef ASURA_PC_SUPPORT_XINPUT
	friend class Asura_Rumble_XInput;
#endif


public:
	enum ASURA_PC_JS_TYPE
	{
		ASURA_PC_JS_TYPE_UNRECOGNISED,

		ASURA_PC_JS_TYPE_PS2_GAMEPAD,
		ASURA_PC_JS_TYPE_PS2_JOYSTICK,		// Used by Derby
		ASURA_PC_JS_TYPE_PS3_GAMEPAD,
		ASURA_PC_JS_TYPE_XBOX360_GAMEPAD,

		#ifdef ASURA_PC_SUPPORT_XINPUT
		ASURA_PC_JS_TYPE_XBOX360_XINPUT_GAMEPAD,
		#endif

		ASURA_PC_JS_NUM_TYPES
	};

	//enum for all the controller objects we support, so projects can refer to them by name
	enum ASURA_PC_JS_OBJECT
	{
		ASURA_PC_JS_OBJECT_FIRST_AXIS,
			ASURA_PC_JS_OBJECT_X_AXIS			= ASURA_PC_JS_OBJECT_FIRST_AXIS,
			ASURA_PC_JS_OBJECT_Y_AXIS,
			ASURA_PC_JS_OBJECT_Z_AXIS,
		ASURA_PC_JS_OBJECT_LAST_AXIS			= ASURA_PC_JS_OBJECT_Z_AXIS,

		//"rotation" axes (rudders often map to one of these, right sticks sometimes use the 1st 2)
		ASURA_PC_JS_OBJECT_FIRST_ROTATION_AXIS,
			ASURA_PC_JS_OBJECT_XROT_AXIS		= ASURA_PC_JS_OBJECT_FIRST_ROTATION_AXIS,
			ASURA_PC_JS_OBJECT_YROT_AXIS,
			ASURA_PC_JS_OBJECT_ZROT_AXIS,
		ASURA_PC_JS_OBJECT_LAST_ROTATION_AXIS	= ASURA_PC_JS_OBJECT_ZROT_AXIS,

		//"slider" axes (anything from right stick, through trim controls, to actual sliders!)
		ASURA_PC_JS_OBJECT_FIRST_SLIDER_AXIS,
			ASURA_PC_JS_OBJECT_S1_AXIS			= ASURA_PC_JS_OBJECT_FIRST_SLIDER_AXIS,
			ASURA_PC_JS_OBJECT_S2_AXIS,
		ASURA_PC_JS_OBJECT_LAST_SLIDER_AXIS		= ASURA_PC_JS_OBJECT_S2_AXIS,

		ASURA_PC_JS_OBJECT_FIRST_BUTTON,
			ASURA_PC_JS_OBJECT_BUTTON_1			= ASURA_PC_JS_OBJECT_FIRST_BUTTON,
			ASURA_PC_JS_OBJECT_BUTTON_2,
			ASURA_PC_JS_OBJECT_BUTTON_3,
			ASURA_PC_JS_OBJECT_BUTTON_4,
			ASURA_PC_JS_OBJECT_BUTTON_5,
			ASURA_PC_JS_OBJECT_BUTTON_6,
			ASURA_PC_JS_OBJECT_BUTTON_7,
			ASURA_PC_JS_OBJECT_BUTTON_8,
			ASURA_PC_JS_OBJECT_BUTTON_9,
			ASURA_PC_JS_OBJECT_BUTTON_10,
			ASURA_PC_JS_OBJECT_BUTTON_11,
			ASURA_PC_JS_OBJECT_BUTTON_12,
			ASURA_PC_JS_OBJECT_BUTTON_13,
			ASURA_PC_JS_OBJECT_BUTTON_14,
			ASURA_PC_JS_OBJECT_BUTTON_15,
			ASURA_PC_JS_OBJECT_BUTTON_16,
			ASURA_PC_JS_OBJECT_BUTTON_17,
			ASURA_PC_JS_OBJECT_BUTTON_18,
			ASURA_PC_JS_OBJECT_BUTTON_19,
			ASURA_PC_JS_OBJECT_BUTTON_20,
			ASURA_PC_JS_OBJECT_BUTTON_21,
			ASURA_PC_JS_OBJECT_BUTTON_22,
			ASURA_PC_JS_OBJECT_BUTTON_23,
			ASURA_PC_JS_OBJECT_BUTTON_24,
			ASURA_PC_JS_OBJECT_BUTTON_25,
			ASURA_PC_JS_OBJECT_BUTTON_26,
			ASURA_PC_JS_OBJECT_BUTTON_27,
			ASURA_PC_JS_OBJECT_BUTTON_28,
			ASURA_PC_JS_OBJECT_BUTTON_29,
			ASURA_PC_JS_OBJECT_BUTTON_30,
			ASURA_PC_JS_OBJECT_BUTTON_31,
			ASURA_PC_JS_OBJECT_BUTTON_32,
		ASURA_PC_JS_OBJECT_LAST_BUTTON			= ASURA_PC_JS_OBJECT_BUTTON_32,

		ASURA_PC_JS_OBJECT_POV_HAT,		//directX reports hats as the angle they are pointing in - some gamepads report their dpad as a hat (?)

		ASURA_PC_JS_NUM_OBJECTS,

		ASURA_PC_JS_OBJECT_POV_HAT_UP,	//special cases for hat directions - these return the name	//of ASURA_PC_JS_OBJECT_POV_HAT if their name is
		ASURA_PC_JS_OBJECT_POV_HAT_DOWN,	//of ASURA_PC_JS_OBJECT_POV_HAT if their name is queried;
		ASURA_PC_JS_OBJECT_POV_HAT_LEFT,	//Asura_PlatformInput returns them to indicate which hat direction a button maps to
		ASURA_PC_JS_OBJECT_POV_HAT_RIGHT,
	};

	// constructor
	Asura_PC_Input_Joystick();

	// destructor
	~Asura_PC_Input_Joystick()
	{
		Shutdown();
	}

	// accessor functions for the current button and axis state

	unsigned char GetButtonState(u_int uButton) const	{ return m_aucButtonPressed[uButton]; }
	float GetHAxis(int iStickNum) const	{ return m_fHAxis[iStickNum]; }

	float GetVAxis(int iStickNum) const
	{
		// DirectInput drivers default to -ve values up, and +ve down; Asura is opposite
		return -m_fVAxis[iStickNum];
	}

	float GetZAxis()	{ return m_fZAxis; }
	float GetXRot()		{ return m_fXRot; }
	float GetYRot()		{ return m_fYRot; }
	float GetZRot()		{ return m_fZRot; }
	u_int GetPOVHat()	{ return m_uiPOV; }

	bool Initialise(REFGUID xDeviceGuid);
	void Shutdown();
	void Update();

	bool HasDevice() const { return m_pxDevice != 0; }

	// Return the product name of this joystick
	const char* GetProductName() const { return m_xProductName.GetString(); }

	const char* GetObjectName(int iObject) const;

	//init and shutdown functions for list of temporary joysticks
	static void InitJoystickList();
	static void ShutdownJoystickList();

	//list of temporary joysticks can have an initialised device for each, to scan for
	//user input on any. temporary list needs to be initialised before calling these
	//these functions init, update and shutdown these devices, and return a list of which have a button pressed
	static void InitTempDevices();
	static void UpdateTempDevices();
	static void ShutdownTempDevices();
	static const bool* GetTempDevicesStatus();

	static int GetNumJoysticks() { return s_iNumJoysticks; }

	//name of joystick on temp list
	static char* GetJoystickName(u_int uIndex);

	//find type of joystick on temp list...
	static ASURA_PC_JS_TYPE GetJoystickType(u_int uIndex);

	//find guid of joystick on temp list
	static bool GetJoystickGUID(u_int uIndex, GUID& xGUID);

	//find guid of joystick on temp list from friendly name
	static bool GetJoystickGUIDFromName(char* szName, GUID& xGUID);

	#ifdef ASURA_PC_SUPPORT_XINPUT
	static bool InitialiseJoystickInstanceFromGUID( Asura_PC_Input_XInputPad* const pxJoystick, const GUID& xGUID );
	#endif

protected:
	// data for each entry on temporary joystick list
	struct JoystickDefinition
	{
		Asura_String m_xJoystickName;
		GUID m_xJoystickGUID;
		ASURA_PC_JS_TYPE m_eJoystickType;
		IDirectInputDevice8* m_pxJoystickTempDevice;

		#ifdef ASURA_PC_SUPPORT_XINPUT
		bool m_bXInputDevice;
		int m_iXInputID;
		#endif
	};

	static float ConvertJoystickAxisToAsuraAxis(LONG lAxisValue, ASURA_PC_JS_OBJECT eAxis);
	static ASURA_PC_JS_TYPE ConvertJoystickToAsuraType(const JoystickDefinition& xDefinition);
	
	// used by the device enumeration function
	static int s_iNumJoysticks;
	static int s_iNumXInputDevices;

	//list itself
	static Asura_List<JoystickDefinition>* s_pxJoystickList;

	//whether any temp devices have a button pressed
	static bool* s_pbTempDevicesStatus;

	//helper function to get a joysticks data from it's index
	static JoystickDefinition* GetJoystickFromIndex(u_int uIndex);

	//names of each object for this joystick
	Asura_String m_axObjectNames[ASURA_PC_JS_NUM_OBJECTS];

	// one byte for each button: Asura supports up to 16
	unsigned char m_aucButtonPressed[16];

	// current axis values
	float m_fHAxis[2], m_fVAxis[2], m_fZAxis, m_fXRot, m_fYRot, m_fZRot;

	// POV hat is represented by an integer (see DX docs)
	u_int m_uiPOV;

	IDirectInputDevice8* m_pxDevice;

	// (AAM - 06.04.2004) The product name of the joystick
	Asura_String	m_xProductName;

private:

	static BOOL CALLBACK EnumDevicesCallback(const DIDEVICEINSTANCE* pxDeviceInstance, void* pData);
	static BOOL CALLBACK EnumObjectsCallback(const DIDEVICEOBJECTINSTANCE* pxObject, void* pData);
};

#endif // !defined __ASURA_PC_INPUT_JOYSTICK_H__
