///////////////////////////////////////////////////////////////////////////////
//
// File   : Asura_PC_Input_XInput.h
// Author : Steve Barnett
// Date   : 19-Mar-2008
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif

#ifndef __ASURA_PC_INPUT_XINPUT_H__
#define __ASURA_PC_INPUT_XINPUT_H__

#ifdef ASURA_PC_SUPPORT_XINPUT

/******************************************************************************/
/* Included Files                                                             */
/******************************************************************************/

#include <XInput.h>

#include "Input/Asura_PC_Input_Joystick.h"

/******************************************************************************/
/* Types                                                                      */
/******************************************************************************/

enum ASURA_XBOX360_AXISID
{
	ASURA_XBOX360_AXISID_PAD1_LSTICK_H	= ASURA_AXIS_PAD1_LSTICK_H,
	ASURA_XBOX360_AXISID_PAD1_LSTICK_V	= ASURA_AXIS_PAD1_LSTICK_V,
	ASURA_XBOX360_AXISID_PAD1_RSTICK_H	= ASURA_AXIS_PAD1_RSTICK_H,
	ASURA_XBOX360_AXISID_PAD1_RSTICK_V	= ASURA_AXIS_PAD1_RSTICK_V,

	ASURA_XBOX360_AXISID_PAD2_LSTICK_H	= ASURA_AXIS_PAD2_LSTICK_H,
	ASURA_XBOX360_AXISID_PAD2_LSTICK_V	= ASURA_AXIS_PAD2_LSTICK_V,
	ASURA_XBOX360_AXISID_PAD2_RSTICK_H	= ASURA_AXIS_PAD2_RSTICK_H,
	ASURA_XBOX360_AXISID_PAD2_RSTICK_V	= ASURA_AXIS_PAD2_RSTICK_V,

	ASURA_XBOX360_AXISID_PAD3_LSTICK_H	= ASURA_AXIS_PAD3_LSTICK_H,
	ASURA_XBOX360_AXISID_PAD3_LSTICK_V	= ASURA_AXIS_PAD3_LSTICK_V,
	ASURA_XBOX360_AXISID_PAD3_RSTICK_H	= ASURA_AXIS_PAD3_RSTICK_H,
	ASURA_XBOX360_AXISID_PAD3_RSTICK_V	= ASURA_AXIS_PAD3_RSTICK_V,

	ASURA_XBOX360_AXISID_PAD4_LSTICK_H	= ASURA_AXIS_PAD4_LSTICK_H,
	ASURA_XBOX360_AXISID_PAD4_LSTICK_V	= ASURA_AXIS_PAD4_LSTICK_V,
	ASURA_XBOX360_AXISID_PAD4_RSTICK_H	= ASURA_AXIS_PAD4_RSTICK_H,
	ASURA_XBOX360_AXISID_PAD4_RSTICK_V	= ASURA_AXIS_PAD4_RSTICK_V,
};

/******************************************************************************/
/* Constants                                                                  */
/******************************************************************************/

const u_int uASURA_XBOX360_NUM_BUTTONS_IN_PAD = 16;
const u_int uASURA_XBOX360_NUM_AXES_IN_PAD = 4;

/******************************************************************************/
/* XInput wrapper classes - used by Asura_PC_Input_Joystick                   */
/******************************************************************************/

class Asura_PC_Input_XInputPad : public Asura_PC_Input_Joystick
{
	friend class Asura_PC_Input_Joystick;

public:
	Asura_PC_Input_XInputPad( void );
	~Asura_PC_Input_XInputPad( void );

	void Shutdown();
	void Update();

	// Static functions
	static BOOL IsXInputDevice( const GUID* pGuidProductFromDirectInput );

	static const Asura_String* GetXInputProductName( void );

protected:
	void InitialiseXInput( const u_int uID );
	void UpdateXInputData( void );
	void ResetXInputData( void );

    // Device properties
    XINPUT_CAPABILITIES m_xCaps;

	// DirectInput or XInput device
	bool m_bIsXInputDevice;

    // Thumb stick values converted to range [-1,+1]
    float		m_fLeftX;
    float		m_fLeftY;

    float		m_fRightX;
    float		m_fRightY;

	// Records which buttons were pressed this frame - only set on
    // the frame that the button is first pressed.
    u_short		m_usPressedButtons;
	
	// The user index associated with this gamepad
    u_int		m_usUserIndex;

	// trigger info, similar to the button state data above
    u_char		m_ucLeftTrigger;
    u_char		m_ucRightTrigger;

	// Flag indicating if pad is currently connected
    bool		m_bConnected;

    // Flags for whether game pad was just inserted or removed
    bool		m_bInserted;
    bool		m_bRemoved;

private:
	// Product name for XInput pads
	static const Asura_String s_xXInputProductName;
};

#endif	// ASURA_PC_SUPPORT_XINPUT
#endif   // ifndef __ASURA_PC_INPUT_XINPUT_H__
