//
// Asura_PlatformInput.cpp
//
// Created by Kevin Lea - 05/10/2001 16:40:45
// 
// Copyright (C) 2001 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

#include "Input/Asura_PlatformInput.h"
#include "Interface/Asura_Cursor.h"
#include "core/Asura_windows.h"
#include <windows.h>
#include <winuser.h>

// (AAM - 14.01.2004)
#include "Input/Asura_DirectInput.h"	// Asura_DirectInput

int Asura_UserInput::Platform_GetNumberOfActiveControllers(bool bConsecutive)
{
    // Always one controller for the PC
    // (NRM - 20.05.2003) Acutally - make it four so we can still run split screen on the PC for test purposes
    return 4;
}

void Asura_UserInput::Platform_Initialise(void)
{
	// (AAM - 14.01.2004) moved Asura_DirectInput::Initialise() call here from Asura_Mouse::Initialise()

	if (Asura_DirectInput::Initialise())
	{
		Asura_PlatformInput::Mouse.Initialise();

		// (AAM - 14.01.2004)

		/*GUID xFirstJoystick;

		if (Asura_PC_Input_Joystick::GetFirstJoystick(&xFirstJoystick))
		{
			Asura_PlatformInput::s_xJoystick.Initialise(xFirstJoystick);
		}*/

		Asura_PC_Input_Joystick::InitJoystickList();
	}
}

void Asura_UserInput::Platform_Shutdown(void)
{
	Asura_PlatformInput::Mouse.Shutdown();

	// (AAM - 14.01.2004)

	Asura_PlatformInput::ShutdownJoystick();

	Asura_PC_Input_Joystick::ShutdownJoystickList();

	// moved here from Asura_Mouse::Shutdown()
	Asura_DirectInput::Shutdown();
}
void Asura_UserInput::Platform_Reset(void)
{
	Asura_PlatformInput::KeyboardInput_ClearAsciiKeyBuffer();
}

void Asura_UserInput::Platform_Update(void)
{
	/// Kevin Lea - 10/01/2002 16:20:38 - read mouse input
	{
		Asura_PlatformInput::Mouse.OncePerFrameUpdate();
		
		/* mouse wheel - read using windows messages */
		{
			if (Asura_PlatformInput::Mouse.WheelStatus>0)
			{
				Asura_PlatformInput::KeyPressedThisFrameBuffer[ASURA_KEYID_MOUSEWHEELUP] = true;
			}
			else if (Asura_PlatformInput::Mouse.WheelStatus<0)
			{
				Asura_PlatformInput::KeyPressedThisFrameBuffer[ASURA_KEYID_MOUSEWHEELDOWN] = true;
			}
			Asura_PlatformInput::Mouse.WheelStatus = 0;
		}

		AxisValue[ASURA_AXIS_MOUSE_V] = Asura_PlatformInput::Mouse.VerticalVelocity/512.0f;
		AxisValue[ASURA_AXIS_MOUSE_H] = Asura_PlatformInput::Mouse.HorizontalVelocity/512.0f;
	}

	/// Kevin Lea - 10/01/2002 16:20:51 - set CurrentInput fields
	for(int i=0; i<MAX_NUMBER_OF_ASURA_KEYIDS; i++)
	{
		if((i<ASURA_KEYID_END_OF_KEYBOARD && Asura_PlatformInput::AsciiKeyBuffer[i]) || Asura_PlatformInput::KeyPressedThisFrameBuffer[i])
		{
			CurrentInput[i] = 255;
		}
		Asura_PlatformInput::KeyPressedThisFrameBuffer[i]=false;
	}

	// used to update Asura_MouseThread here.
	if(Asura_Windows::IsApplicationActive())
	{
		if((Asura_Mouse::IsExclusive() || Asura_Mouse::GetRestrictWindowsCursorToWindow())&& !Asura_Mouse::GetUseWindowsCursor())
		{
			//don't care about the windows cursor... just update our one
			float deltaX,deltaY;
			Asura_PlatformInput::Mouse.Update(deltaX,deltaY);
			#ifdef ASURA_USE_CURSOR
			Asura_Cursor::Update(deltaX,deltaY);
			#endif

			if(Asura_Mouse::GetRestrictWindowsCursorToWindow())
			{
				//Not using the windows cursor ,so just put it back it to the centre each frame
				RECT xWindowRect;
				Asura_Windows::GetWindowViewportCoords(&xWindowRect);
				SetCursorPos((xWindowRect.left+xWindowRect.right)/2, (xWindowRect.top+xWindowRect.bottom)/2);
			}
		}
		else
		{
			//In non-exclusive mode we want to keep windows and our mouse cursor in sync
			//RAR 23/08/2006
			float deltaX,deltaY;
			Asura_PlatformInput::Mouse.Update(deltaX,deltaY);
		
			RECT xWindowRect;
			Asura_Windows::GetWindowViewportCoords(&xWindowRect);

			
			#ifdef ASURA_USE_CURSOR
			if(Asura_Cursor::IsCursorMovementEnabled() && !Asura_Cursor::TeleportedSinceLastUpdate())
			{
				//set the cursor to match the windows cursor position
				POINT xCursorPos;
				GetCursorPos(&xCursorPos);
				

				float fX = (float)(xCursorPos.x - xWindowRect.left);
				float fY = (float)(xCursorPos.y - xWindowRect.top);

				Asura_Cursor::UpdateAbs(fX,fY);

				if(Asura_Mouse::GetRestrictWindowsCursorToWindow())
				{
					//If the position was clamped in Asura_Cursor , then update the windows cursor to match
					int iX,iY;
					Asura_Cursor::GetScreenSpaceCoords(iX,iY);
					if(iX != Asura_Maths::FloatToInt(fX) || iY != Asura_Maths::FloatToInt(fY))
					{
						iX += xWindowRect.left;
						iY += xWindowRect.top;
						SetCursorPos(iX,iY);
					}
				}
			}
			else
			{
				Asura_Cursor::Update(deltaX,deltaY);

				//Cursor movement is disabled while using mouse for rotation etc
				//Force the windows cursor to stay put.
				int iX,iY;
				Asura_Cursor::GetScreenSpaceCoords(iX,iY);

				iX += xWindowRect.left;
				iY += xWindowRect.top;
				SetCursorPos(iX,iY);
			}
			#endif //ASURA_USE_CURSOR
		}
	}

	//Determine whether the windows cursor should be drawn.
	//Probably doesn't really belong here , but should do as well as any. RAR 23/08/2006
	//Also check when window not focused, allows mouse to reappear more consistently. PJC 30/04/09
	Asura_Windows::UpdateWindowsCursorVisibility();

    s_wszInputBuffer = Asura_Windows::GetInputBuffer();

    s_uInputBufferSize = Asura_Windows::GetSizeOfInputBuffer();

	// (AAM - 14.01.2004) Update Asura joystick 1 with the stick data

	Asura_PlatformInput::s_xJoystick.Update();

	AxisValue[ASURA_AXIS_PAD1_LSTICK_H] = Asura_PlatformInput::s_xJoystick.GetHAxis(0);
	AxisValue[ASURA_AXIS_PAD1_LSTICK_V] = Asura_PlatformInput::s_xJoystick.GetVAxis(0);

	AxisValue[ASURA_AXIS_PAD1_RSTICK_H] = Asura_PlatformInput::s_xJoystick.GetHAxis(1);
	AxisValue[ASURA_AXIS_PAD1_RSTICK_V] = Asura_PlatformInput::s_xJoystick.GetVAxis(1);

	AxisValue[ASURA_AXIS_PAD1_Z] = Asura_PlatformInput::s_xJoystick.GetZAxis();

	AxisValue[ASURA_AXIS_PAD1_ROTATE_X] = Asura_PlatformInput::s_xJoystick.GetXRot();
	AxisValue[ASURA_AXIS_PAD1_ROTATE_Y] = Asura_PlatformInput::s_xJoystick.GetYRot();
	AxisValue[ASURA_AXIS_PAD1_ROTATE_Z] = Asura_PlatformInput::s_xJoystick.GetZRot();

	for (int i = 0; i < 16; i++)
	{
		CurrentInput[ASURA_KEYID_PAD1_BUTTON1 + i] = Asura_PlatformInput::s_xJoystick.GetButtonState(i);
	}

	// NY - 16 August 2005 - map hat to four hat keys, giving us an 8 directional hat (most are 4 or 8 that i've ever seen)
	// these are currently mapped to spare buttons for the unused "asura joystick" - the device buttons themselves use pad 1's buttons
	u_int uiPOV = Asura_PlatformInput::s_xJoystick.GetPOVHat();

	// NY - 16 August 2005 - directInput passes the hat value as an angle in hundredths of a degree - presumably there is such a thing as an
	// analogue hat out there to justify this?

	if (uiPOV == 1 || uiPOV > 36000) //some sticks report hat centred as 1; some report it as 65535
	{
		CurrentInput[ASURA_PC_KEYID_HAT_UP]    = 0;
		CurrentInput[ASURA_PC_KEYID_HAT_RIGHT] = 0;
		CurrentInput[ASURA_PC_KEYID_HAT_DOWN]  = 0;
		CurrentInput[ASURA_PC_KEYID_HAT_LEFT]  = 0;
	}
	else
	{
		CurrentInput[ASURA_PC_KEYID_HAT_UP] = (uiPOV <= 4500 || (uiPOV >= 31500 && uiPOV <= 36000))?255:0;

		CurrentInput[ASURA_PC_KEYID_HAT_RIGHT] = (uiPOV >= 4500 && uiPOV <= 13500)?255:0;

		CurrentInput[ASURA_PC_KEYID_HAT_DOWN] = (uiPOV >= 13500 && uiPOV <= 22500)?255:0;

		CurrentInput[ASURA_PC_KEYID_HAT_LEFT] = (uiPOV >= 22500 && uiPOV <= 31500)?255:0;
	}
}





Asura_Mouse Asura_PlatformInput::Mouse;

#ifdef ASURA_PC_SUPPORT_XINPUT
Asura_PC_Input_XInputPad Asura_PlatformInput::s_xJoystick;
#else
Asura_PC_Input_Joystick Asura_PlatformInput::s_xJoystick;
#endif

bool Asura_PlatformInput::AsciiKeyBuffer[ASURA_KEYID_END_OF_KEYBOARD];
unsigned char Asura_PlatformInput::KeyPressedThisFrameBuffer[MAX_NUMBER_OF_ASURA_KEYIDS];

//////////////////////////////////////////////////////////////////////////////
//
// FUNCTION     Asura_PlatformInput::InitJoystickFromGUID().
// PARAMETERS   xGuid - GUID of joystick to initialise
// RETURNS      true if successful
// DESCRIPTION  initialises the master static joystick object
// AUTHOR       Neil Young - 16.08.2005.
//
//////////////////////////////////////////////////////////////////////////////
bool Asura_PlatformInput::InitJoystickFromGUID(GUID& xGuid)
{
	#ifdef ASURA_PC_SUPPORT_XINPUT
	return Asura_PC_Input_Joystick::InitialiseJoystickInstanceFromGUID( &s_xJoystick, xGuid );
	#else
	return s_xJoystick.Initialise(xGuid);
	#endif
}
// End function Asura_PlatformInput::InitJoystickFromGUID().

//////////////////////////////////////////////////////////////////////////////
//
// FUNCTION     Asura_PlatformInput::ShutdownJoystick().
// PARAMETERS   None.
// RETURNS      Nothing.
// DESCRIPTION  Shuts down the master static joystick object
// AUTHOR       Neil Young - 16.08.2005.
//
//////////////////////////////////////////////////////////////////////////////
void Asura_PlatformInput::ShutdownJoystick()
{
	s_xJoystick.Shutdown();
}
// End function Asura_PlatformInput::ShutdownJoystick().

//////////////////////////////////////////////////////////////////////////////
//
// FUNCTION     Asura_PlatformInput::GetObjectName().
// PARAMETERS   iObject - enum of object to fetch
// RETURNS      friendly name of object, or 0 if none
// DESCRIPTION  returns directX friendly name of joystick object
// AUTHOR       Neil Young - 16.08.2005.
//
//////////////////////////////////////////////////////////////////////////////
const char* Asura_PlatformInput::GetObjectName(int iObject)
{
	return Asura_PlatformInput::s_xJoystick.GetObjectName(iObject);
}
// End function Asura_PlatformInput::GetObjectName().

//////////////////////////////////////////////////////////////////////////////
//
// FUNCTION     Asura_PlatformInput::GetAxisObject().
// PARAMETERS   eAxis - asura axis
// RETURNS      enum of object, or -1 if not found
// DESCRIPTION  maps asura axes to joystick objects
// AUTHOR       Neil Young - 16.08.2005.
//
//////////////////////////////////////////////////////////////////////////////
int Asura_PlatformInput::GetAxisObject(ASURA_AXIS_ID eAxis)
{
	switch (eAxis)
	{
	case ASURA_AXIS_PAD1_LSTICK_H:
		return Asura_PC_Input_Joystick::ASURA_PC_JS_OBJECT_X_AXIS;
	case ASURA_AXIS_PAD1_LSTICK_V:
		return Asura_PC_Input_Joystick::ASURA_PC_JS_OBJECT_Y_AXIS;
	case ASURA_AXIS_PAD1_Z:
		return Asura_PC_Input_Joystick::ASURA_PC_JS_OBJECT_Z_AXIS;

	case ASURA_AXIS_PAD1_ROTATE_X:
		return Asura_PC_Input_Joystick::ASURA_PC_JS_OBJECT_XROT_AXIS;
	case ASURA_AXIS_PAD1_ROTATE_Y:
		return Asura_PC_Input_Joystick::ASURA_PC_JS_OBJECT_YROT_AXIS;
	case ASURA_AXIS_PAD1_ROTATE_Z:
		return Asura_PC_Input_Joystick::ASURA_PC_JS_OBJECT_ZROT_AXIS;

	case ASURA_AXIS_PAD1_RSTICK_H:
		return Asura_PC_Input_Joystick::ASURA_PC_JS_OBJECT_S1_AXIS;
	case ASURA_AXIS_PAD1_RSTICK_V:
		return Asura_PC_Input_Joystick::ASURA_PC_JS_OBJECT_S2_AXIS;

	default:
		break;
	}
	return -1;
}
// End function Asura_PlatformInput::GetAxisObject().

//////////////////////////////////////////////////////////////////////////////
//
// FUNCTION     Asura_PlatformInput::GetKeyObject().
// PARAMETERS   eKey - asura key id
// RETURNS      enum of object, or -1 if not found
// DESCRIPTION  maps asura keys to joystick objects
// AUTHOR       Neil Young - 16.08.2005.
//
//////////////////////////////////////////////////////////////////////////////
int Asura_PlatformInput::GetKeyObject(ASURA_KEYID eKey)
{
	if (eKey >= ASURA_KEYID_PAD1_BUTTON1 && eKey <= ASURA_KEYID_PAD1_BUTTON16)
	{
		return eKey - ASURA_KEYID_PAD1_BUTTON1 + Asura_PC_Input_Joystick::ASURA_PC_JS_OBJECT_BUTTON_1;
	}

	if (eKey >= ASURA_KEYID_JOYSTICK_BUTTON_1 && eKey <= ASURA_KEYID_JOYSTICK_BUTTON_4)
	{
		return eKey - ASURA_KEYID_JOYSTICK_BUTTON_1 + Asura_PC_Input_Joystick::ASURA_PC_JS_OBJECT_POV_HAT_UP;
	}
	return -1;
}
// End function Asura_PlatformInput::GetKeyObject().

void Asura_PlatformInput::KeyboardInput_AsciiKeyDown(int key)
{
	// JAT 07/03/06 pressing 'w' on my keyboard generates a weird key up message with key = 511
    // (MP - 10.03.2006) Actually, pressing certain keys can legitimately
    // generate values beyond ASURA_KEYID_END_OF_KEYBOARD, eg.
    // VK_MEDIA_PLAY_PAUSE. While we certainly want to prevent writing beyond
    // the end of the buffer (as we must have been in the past in this case),
    // the assert is incorrect. Incidentally, a value here of 511 corresponds
    // to a VK code of 255 which winuser.h mentions is reserved. Possibly
    // this was a keyboard driver issue.
//	Asura_Assertf( key < ASURA_KEYID_END_OF_KEYBOARD, ( "AsciiKeyDown key too large 0x%X", key ) );
	if ( key >= ASURA_KEYID_END_OF_KEYBOARD )
	{
		return;
	}
	AsciiKeyBuffer[key]=true;
	KeyPressedThisFrameBuffer[key]=true;
}

void Asura_PlatformInput::KeyboardInput_AsciiKeyUp(int key)
{
	// JAT 07/03/06 pressing 'w' on my keyboard generates a weird key up message with key = 511
    // (MP - 10.03.2006) See comment in KeyboardInput_AsciiKeyDown() above.
//	Asura_Assertf( key < ASURA_KEYID_END_OF_KEYBOARD, ( "AsciiKeyUp key too large 0x%X", key ) );
	if ( key >= ASURA_KEYID_END_OF_KEYBOARD )
	{
		return;
	}
	AsciiKeyBuffer[key]=false;
}

void Asura_PlatformInput::KeyboardInput_ClearAsciiKeyBuffer(void)
{
	int i = MAX_NUMBER_OF_ASURA_KEYIDS;
	while(i--)
	{
		KeyPressedThisFrameBuffer[i]=false;
	}

	i = ASURA_KEYID_END_OF_KEYBOARD;
	while(i--)
	{
		AsciiKeyBuffer[i] = false;
	}

}

void Asura_PlatformInput::SetKeyPressedThisFrame(ASURA_KEYID eKey)
{
	if(eKey>ASURA_KEYID_VOID && eKey<MAX_NUMBER_OF_ASURA_KEYIDS)
	{
		KeyPressedThisFrameBuffer[eKey] = true;
	}
}

bool Asura_UserInput::Platform_IsControllerActive(int iPlayerNum)
{
    return iPlayerNum == 0;
}

bool Asura_UserInput::Platform_IsKeyToggled(int iKey)
{
    return (GetKeyState(iKey - ASURA_NUMBER_OF_ASCII_KEYS) << 1) != 0;
}


void Asura_UserInput::Platform_UpdateInputBuffer( Asura_UnicodeChar* wszBuffer, u_int uMaxChars, u_int& uInputCursorPos,
												  bool (*pfnHandleInputOverride)(ASURA_KEYID eKey), const Asura_UnicodeChar* wszInputBuffer,
												  u_int uInputBufferSize )
{
	// only want to update the main buffer
	if (wszInputBuffer != s_wszInputBuffer) return;
	
	// some keys won't appear in the standard input buffer, check for them separately
	const ASURA_KEYID aeSpecialKeys[] = 
	{
		ASURA_KEYID_HOME, ASURA_KEYID_END, ASURA_KEYID_LEFT, ASURA_KEYID_RIGHT, ASURA_KEYID_UP, ASURA_KEYID_DOWN, 
		ASURA_KEYID_DEL, ASURA_KEYID_PAGEUP, ASURA_KEYID_PAGEDOWN
	};

	u_int u = sizeof(aeSpecialKeys) / sizeof(aeSpecialKeys[0]);
	for( const ASURA_KEYID* pxKey = aeSpecialKeys; u--; ++pxKey )
	{
		if( Asura_UserInput::DebouncedKeyPressed(*pxKey) && (!pfnHandleInputOverride || !pfnHandleInputOverride(*pxKey)) )
		{
			HandleKeyInput( *pxKey, wszBuffer, uMaxChars, uInputCursorPos );
		}
	}	
}

