//
// Asura_PC_DirectPlay.h
//
// Created by Kevin Lea - 13/06/2001 11:34:31
// 
// Copyright (C) 2001 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef __ASURA_PC_DIRECTPLAY_H__
#define __ASURA_PC_DIRECTPLAY_H__

// I N C L U D E S ///////////////////////////////////////////////////////////

// Engine independent.
#include "Core/Asura_String.h"

// Engine platform-specific.
#include "Core/Asura_DirectX.h"

// M A C R O S ///////////////////////////////////////////////////////////////

// (MP - 01.09.2003) TODO: Probably not a good idea to have this constant
// across projects. And it shouldn't really be fixed either; this should just
// be a default, since otherwise you stop people from hosting multiple
// servers on a single machine.
// (MH - 11.09.2003) can now set the port e.g. on the command line.
#define ASURA_PC_DIRECTPLAY_DEFAULTPORT (25857)//(0x6501) // arbitrary port number for this app

// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////////

class Asura_PC_DirectPlay_Client;
class Asura_PC_DirectPlay_Server;
class Asura_Message;
class Asura_Network;

// C L A S S E S /////////////////////////////////////////////////////////////

class Asura_PC_DirectPlay_SessionInfo
{
	public:
		DPN_APPLICATION_DESC*	AppDesc;
		IDirectPlay8Address*	HostAddr;
		IDirectPlay8Address*	DeviceAddr;

		int						LastPollTime;
		bool		            IsValid;
		int						RoundTripLatency;
    
		void CopyInfo(PDPNMSG_ENUM_HOSTS_RESPONSE msg);
		void UpdateInfo(PDPNMSG_ENUM_HOSTS_RESPONSE msg);
		void FreeMemory(void);
};


class Asura_PC_DirectPlay_PlayerInfo
{
	public:

		bool			IsValid;

		DPNID			NetworkID;
		Asura_String	Name;
		int				Index;

		int				ClientGuid;

		Asura_PC_DirectPlay_PlayerInfo()
		{
			IsValid = false;
		}
};


class Asura_PC_DirectPlay
{
	friend Asura_Network;
    friend Asura_PC_DirectPlay_Client;
    friend Asura_PC_DirectPlay_Server;

public:

	static void SetUseNetworkSimulator(bool bEnable) { s_bUseNetworkSimulator = bEnable; }
	static bool IsUsingNetworkSimulator() { return s_bUseNetworkSimulator; }

    static void SetServerName(const Asura_UnicodeChar*);
	static void SetPort(u_int uPort) { s_uPort = uPort; }
	static u_int GetPort() { return s_uPort; }
	static void SetPassword(const char* szPassword);
	static u_int GetPasswordHash() { return s_uPasswordHash; }
    static const Asura_PC_DirectPlay_SessionInfo* GetSessionInfo(u_int uSessionIndex);

#ifdef ASURA_DIRECTPLAY_MESSAGELOGGING
    enum MSG_LOG_LEVEL
    {
        MSG_LOG_LEVEL_NONE,
        MSG_LOG_LEVEL_HEADER_ONLY,
        MSG_LOG_LEVEL_NUM_LEVELS,
    };
    static MSG_LOG_LEVEL GetMsgLogLevel( void ) { return s_eMsgLogLevel; }
    static void SetMsgLogLevel( MSG_LOG_LEVEL eLevel );
#endif

	static u_int Project_GetVersionID( void );
	
	static const Asura_PC_DirectPlay_PlayerInfo* GetPlayerInfo(int iIndex);
	static int GetPlayerIPAddress(int iIndex);

	static void DisconnectPlayer(int iIndex);

private:

	static void Initialise(void);
	static void Shutdown(void);

	static void UpdateBandwidthStats(void);

	static const Asura_UnicodeChar* s_wszSessionName;    // if NULL use computer name

	static void ShowSessions(void);
    static u_int Project_ModifyCurNumPlayers( u_int uCurPlayers, const void* pvProjectData );
    static u_int Project_ModifyMaxNumPlayers( u_int uMaxPlayers, const void* pvProjectData );



	// Kevin Lea - 14/06/2001 10:17:47 - Client
	static Asura_PC_DirectPlay_Client* s_pxClient;
	static DPNHANDLE AsyncOp_EnumSessions;
	static DPNHANDLE AsyncOp_Connecting;
	static CRITICAL_SECTION CriticalSection_EnumSessions;
	static Asura_PC_DirectPlay_SessionInfo** SessionsStorage;
	static int NumberOfActiveSessions;
	static bool EnumSessionsActive;

	// if IP address specified, in StartClientAndFindSessions, attempt to 
	// connect to the specified session instead of enumerating LAN games.
	static bool StartClientAndFindSessions(const char* szIPAddress = 0);
	static void ConnectToSession(u_int uSessionIndex);

	static HRESULT WINAPI ClientMessageHandler( PVOID pvUserContext, DWORD dwMessageId, PVOID pMsgBuffer );
	static void AddSessionInfo(PDPNMSG_ENUM_HOSTS_RESPONSE msg);
	static void StopSessionsEnum(void);
	static void DeleteSessionsInfo(void);

	static void ClientSendMessage(void* data, unsigned int size);

	// Kevin Lea - 14/06/2001 10:17:57 - Server 
	static Asura_PC_DirectPlay_Server* s_pxServer;
	static Asura_PC_DirectPlay_PlayerInfo* PlayerInfo;

	static bool StartServerAndHostSession(void);
	static HRESULT WINAPI ServerMessageHandler( PVOID pvUserContext, DWORD dwMessageId, PVOID pMsgBuffer );
	static void SetServerReady(bool bReady);
	static void UpdateServerPassword(u_int uPasswordHash);
    static bool IsPlayerBanned( const DPNMSG_INDICATE_CONNECT* pxIndicateConnectMsg );
    static bool Project_ShouldAcceptNewPlayer( const DPNMSG_INDICATE_CONNECT* pxIndicateConnectMsg );

	static void PackMessageIntoBuffer(const Asura_Message* message, DPN_BUFFER_DESC& buffer);
	static void UnpackMessageFromBuffer(const BYTE* buffer, u_int uNumBytes, bool bWasGuaranteed);

	static bool ShouldAddMessageToQueue(u_int uMsgID);

	// static data, neither client or server specific
    static bool s_bUseNetworkSimulator;
	static u_int s_uPort;
	static u_int s_uPasswordHash;

#ifdef ASURA_DIRECTPLAY_MESSAGELOGGING
    static void Project_GetMsgIDDesc( u_int uMsgID, const char** pszDesc );
    static void LogMsgInfo( const DPN_BUFFER_DESC* pxBfrDesc, bool bIndent = false );
    static MSG_LOG_LEVEL s_eMsgLogLevel;
#endif

};


class Asura_PC_DirectPlayer_UserConnectData
{
	public:
		
		u_int m_uVersionID;
		u_int m_uPasswordHash;
};

class Asura_PC_DirectPlay_UserConnectReplyData
{
	public:

		u_int m_uResultCode;
};

#endif//__ASURA_PC_DIRECTPLAY_H__
