//////////////////////////////////////////////////////////////////////////////
//
// Asura_PC_DirectPlay_Server.h
//
// Copyright (c) 2003 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef ___ASURA_PC_DIRECTPLAY_SERVER_H___
#define ___ASURA_PC_DIRECTPLAY_SERVER_H___ 1

// I N C L U D E S ///////////////////////////////////////////////////////////

// Engine platform-specific.
#include "Core/Asura_DirectX.h"

// C L A S S E S /////////////////////////////////////////////////////////////

class Asura_PC_DirectPlay_Server
{

    public:

        // C L A S S   M E T H O D S /////////////////////////////////////////

        static void CreateInstance( Asura_PC_DirectPlay_Server** ppxInstance );
        static void DestroyInstance( Asura_PC_DirectPlay_Server** ppxInstance );

        // M E T H O D S /////////////////////////////////////////////////////

        // Methods from IDirectPlay8Server that we want to override in some
        // manner.
        HRESULT SendTo( const DPNID dpnid, const DPN_BUFFER_DESC *const prgBufferDesc, const DWORD cBufferDesc, const DWORD dwTimeOut, void *const pvAsyncContext, DPNHANDLE *const phAsyncHandle, const DWORD dwFlags );

        // Methods from IDirectPlay8Server that we just want to call straight
        // through to.
        HRESULT QueryInterface(DP8REFIID riid, LPVOID *ppvObj) { return m_pxDPServer->QueryInterface( riid, ppvObj ); }
        ULONG   AddRef( void ) { return m_pxDPServer->AddRef(); }
        ULONG   Release( void ) { return m_pxDPServer->Release(); }
        HRESULT Initialize(PVOID const pvUserContext, const PFNDPNMESSAGEHANDLER pfn, const DWORD dwFlags) { return m_pxDPServer->Initialize( pvUserContext, pfn, dwFlags ); }
        HRESULT EnumServiceProviders( const GUID *const pguidServiceProvider,const GUID *const pguidApplication,DPN_SERVICE_PROVIDER_INFO *const pSPInfoBuffer,PDWORD const pcbEnumData,PDWORD const pcReturned,const DWORD dwFlags ) { return m_pxDPServer->EnumServiceProviders( pguidServiceProvider, pguidApplication, pSPInfoBuffer, pcbEnumData, pcReturned, dwFlags ); }
        HRESULT CancelAsyncOperation( const DPNHANDLE hAsyncHandle,const DWORD dwFlags ) { return m_pxDPServer->CancelAsyncOperation( hAsyncHandle, dwFlags ); }
        HRESULT GetSendQueueInfo( const DPNID dpnid,DWORD *const pdwNumMsgs, DWORD *const pdwNumBytes, const DWORD dwFlags ) { return m_pxDPServer->GetSendQueueInfo( dpnid, pdwNumMsgs, pdwNumBytes, dwFlags ); }
        HRESULT GetApplicationDesc( DPN_APPLICATION_DESC *const pAppDescBuffer, DWORD *const pcbDataSize, const DWORD dwFlags ) { return m_pxDPServer->GetApplicationDesc( pAppDescBuffer, pcbDataSize, dwFlags ); }
        HRESULT SetServerInfo( const DPN_PLAYER_INFO *const pdpnPlayerInfo,PVOID const pvAsyncContext, DPNHANDLE *const phAsyncHandle, const DWORD dwFlags ) { return m_pxDPServer->SetServerInfo( pdpnPlayerInfo, pvAsyncContext, phAsyncHandle, dwFlags ); }
        HRESULT GetClientInfo( const DPNID dpnid,DPN_PLAYER_INFO *const pdpnPlayerInfo,DWORD *const pdwSize,const DWORD dwFlags ) { return m_pxDPServer->GetClientInfo( dpnid, pdpnPlayerInfo, pdwSize, dwFlags ); }
        HRESULT GetClientAddress( const DPNID dpnid,IDirectPlay8Address **const pAddress,const DWORD dwFlags ) { return m_pxDPServer->GetClientAddress( dpnid, pAddress, dwFlags ); }
        HRESULT GetLocalHostAddresses( IDirectPlay8Address **const prgpAddress,DWORD *const pcAddress,const DWORD dwFlags ) { return m_pxDPServer->GetLocalHostAddresses( prgpAddress, pcAddress, dwFlags ); }
        HRESULT SetApplicationDesc( const DPN_APPLICATION_DESC *const pad, const DWORD dwFlags ) { return m_pxDPServer->SetApplicationDesc( pad, dwFlags ); }
        HRESULT Host( const DPN_APPLICATION_DESC *const pdnAppDesc,IDirectPlay8Address **const prgpDeviceInfo,const DWORD cDeviceInfo,const DPN_SECURITY_DESC *const pdnSecurity, const DPN_SECURITY_CREDENTIALS *const pdnCredentials,void *const pvPlayerContext,const DWORD dwFlags ) { return m_pxDPServer->Host( pdnAppDesc, prgpDeviceInfo, cDeviceInfo, pdnSecurity, pdnCredentials, pvPlayerContext, dwFlags ); }
        HRESULT CreateGroup( const DPN_GROUP_INFO *const pdpnGroupInfo,void *const pvGroupContext,void *const pvAsyncContext,DPNHANDLE *const phAsyncHandle,const DWORD dwFlags ) { return m_pxDPServer->CreateGroup( pdpnGroupInfo, pvGroupContext, pvAsyncContext, phAsyncHandle, dwFlags ); }
        HRESULT DestroyGroup( const DPNID idGroup, PVOID const pvAsyncContext, DPNHANDLE *const phAsyncHandle, const DWORD dwFlags ) { return m_pxDPServer->DestroyGroup( idGroup, pvAsyncContext, phAsyncHandle, dwFlags ); }
        HRESULT AddPlayerToGroup( const DPNID idGroup, const DPNID idClient, PVOID const pvAsyncContext, DPNHANDLE *const phAsyncHandle, const DWORD dwFlags ) { return m_pxDPServer->AddPlayerToGroup( idGroup, idClient, pvAsyncContext, phAsyncHandle, dwFlags ); }
        HRESULT RemovePlayerFromGroup( const DPNID idGroup, const DPNID idClient, PVOID const pvAsyncContext, DPNHANDLE *const phAsyncHandle, const DWORD dwFlags ) { return m_pxDPServer->RemovePlayerFromGroup( idGroup, idClient, pvAsyncContext, phAsyncHandle, dwFlags ); }
        HRESULT SetGroupInfo( const DPNID dpnid,DPN_GROUP_INFO *const pdpnGroupInfo,PVOID const pvAsyncContext,DPNHANDLE *const phAsyncHandle, const DWORD dwFlags ) { return m_pxDPServer->SetGroupInfo( dpnid, pdpnGroupInfo, pvAsyncContext, phAsyncHandle, dwFlags ); }
        HRESULT GetGroupInfo( const DPNID dpnid,DPN_GROUP_INFO *const pdpnGroupInfo,DWORD *const pdwSize,const DWORD dwFlags ) { return m_pxDPServer->GetGroupInfo( dpnid, pdpnGroupInfo, pdwSize, dwFlags ); }
        HRESULT EnumPlayersAndGroups( DPNID *const prgdpnid, DWORD *const pcdpnid, const DWORD dwFlags ) { return m_pxDPServer->EnumPlayersAndGroups( prgdpnid, pcdpnid, dwFlags ); }
        HRESULT EnumGroupMembers( const DPNID dpnid, DPNID *const prgdpnid, DWORD *const pcdpnid, const DWORD dwFlags ) { return m_pxDPServer->EnumGroupMembers( dpnid, prgdpnid, pcdpnid, dwFlags ); }
        HRESULT Close( const DWORD dwFlags ) { return m_pxDPServer->Close( dwFlags ); }
        HRESULT DestroyClient( const DPNID dpnidClient, const void *const pvDestroyData, const DWORD dwDestroyDataSize, const DWORD dwFlags ) { return m_pxDPServer->DestroyClient( dpnidClient, pvDestroyData, dwDestroyDataSize, dwFlags ); }
        HRESULT ReturnBuffer( const DPNHANDLE hBufferHandle,const DWORD dwFlags ) { return m_pxDPServer->ReturnBuffer( hBufferHandle, dwFlags ); }
        HRESULT GetPlayerContext( const DPNID dpnid,PVOID *const ppvPlayerContext,const DWORD dwFlags ) { return m_pxDPServer->GetPlayerContext( dpnid, ppvPlayerContext, dwFlags ); }
        HRESULT GetGroupContext( const DPNID dpnid,PVOID *const ppvGroupContext,const DWORD dwFlags ) { return m_pxDPServer->GetGroupContext( dpnid, ppvGroupContext, dwFlags ); }
        HRESULT GetCaps( DPN_CAPS *const pdpCaps,const DWORD dwFlags ) { return m_pxDPServer->GetCaps( pdpCaps, dwFlags ); }
        HRESULT SetCaps( const DPN_CAPS *const pdpCaps, const DWORD dwFlags ) { return m_pxDPServer->SetCaps( pdpCaps, dwFlags ); }
        HRESULT SetSPCaps( const GUID * const pguidSP, const DPN_SP_CAPS *const pdpspCaps, const DWORD dwFlags  ) { return m_pxDPServer->SetSPCaps( pguidSP, pdpspCaps, dwFlags  ); }
        HRESULT GetSPCaps( const GUID * const pguidSP, DPN_SP_CAPS *const pdpspCaps,const DWORD dwFlags ) { return m_pxDPServer->GetSPCaps( pguidSP, pdpspCaps, dwFlags ); }
        HRESULT GetConnectionInfo( const DPNID dpnid, DPN_CONNECTION_INFO *const pdpConnectionInfo,const DWORD dwFlags ) { return m_pxDPServer->GetConnectionInfo( dpnid, pdpConnectionInfo, dwFlags ); }
        HRESULT RegisterLobby( const DPNHANDLE dpnHandle, struct IDirectPlay8LobbiedApplication *const pIDP8LobbiedApplication,const DWORD dwFlags ) { return m_pxDPServer->RegisterLobby( dpnHandle, pIDP8LobbiedApplication, dwFlags ); }

    private:
        
        // S T A T E /////////////////////////////////////////////////////////

        IDirectPlay8Server* m_pxDPServer;

};

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

#endif // #ifndef ___ASURA_PC_DIRECTPLAY_SERVER_H___

