//
// Asura_PC_GCDKey.h
//
// Created by Richard Rice - 09/09/2003
// 
// Copyright (C) 2003 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef __ASURA_PC_GCDKEY__
#define __ASURA_PC_GCDKEY__


#include "Core/Asura_String.h"
#include "GameSpy/gcdkey/gcdkeyc.h"
#include "Network/Asura_Guid.h"


class Asura_PC_GCDKey
{
public :
	static void Initialise();
	static void Update();
	static void Shutdown();

	static void AddPlayer(int iIndex);
	static void RemovePlayer(int iIndex);

	static void HandleGCDChallenge(Asura_Guid uClientGuid,const char* pszChallenge);
	static void HandleGCDResponse(Asura_Guid uClientGuid,const char* pszResponse);

	//functions for use on the client to see whether authorisation has been successful
	static void SetWaitingForAuthorisation(bool bWaiting)
	{
		s_bWaitingForAuthorisation = bWaiting;
		if(bWaiting) s_xErrorMessage.MakeNull();
	}
	static void SetAuthorised(bool bAuthorised){s_bAuthorised = bAuthorised;}
	static void SetErrorMessage(const char* pszErrorMessage)
	{
		if(pszErrorMessage && strlen(pszErrorMessage)) s_xErrorMessage = pszErrorMessage;
		else s_xErrorMessage.MakeNull();
	}

	#if 1
	static bool IsWaitingForAuthorisation(){return s_bWaitingForAuthorisation;}
	static bool	IsAuthorised(){return s_bAuthorised;}
	#else
	//disable for now
	static bool IsWaitingForAuthorisation(){return false;}
	static bool	IsAuthorised(){return true;}
	#endif
	static const char* GetErrorMessage(){return s_xErrorMessage.GetString();}



private :
	static void SendAuthorisationMessage(int iIndex,bool bAuthorised,const char* pszErrorMessage);
	//gets the gamespy product ID for CD key checking
	static int Project_GetProductID();
	static bool Project_GetCDKey(char* pszBuffer,int iBufferSize);

	static void GCDAuthenticationCallback(int iProductID, int iLocalID, int iAuthenticated, char *pszErrmsg, void *pUserData);

	static bool s_bInitialised;
	static int s_iMaxPlayers;
	static struct Asura_GCD_PlayerInfo* s_pxPlayerInfo;

	static bool s_bWaitingForAuthorisation;
	static bool s_bAuthorised;
	static Asura_String s_xErrorMessage;
};


class Asura_Msg_GCD_Challenge
{
public :
	char m_szChallenge[8];
	Asura_Guid m_uClientGuid;
};

class Asura_Msg_GCD_Response
{
public :
	char m_szResponse[RESPONSE_SIZE];
};

class Asura_Msg_GCD_Authorisation
{
public :
	unsigned char m_ucAuthorised;
	//followed by a string
};


#endif //__ASURA_PC_GCDKEY__
