//
// Asura_Platform_NetSocket.cpp
//
// PC version
//
// Created by MH - 19.07.2004
//
// Copyright (C) 2004 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

#ifdef ASURA_SUPPORT_EXTERNAL_NETWORK

// This file's header.
#include "Network/Asura_NetSocket.h"

#ifdef ASURA_PC_SUPPORT_STEAMWORKS
#include "Network/Steamworks/Asura_PC_NetSocketSteamworks.h"
#include "Network/Steamworks/Asura_PC_Steamworks.h"
#endif

// (MH - 19.07.2004) these are platform functions in order to work around funny packet headers on the
// XBox (used for Voice Data) and the XBox limit of 1304 bytes per sendto call. On PC/PS2 we don't need
// to fiddle about with headers or impose a limit (the independent code should keep packet sizes down 
// below about 1kb anyway).
bool Asura_NetSocket::Platform_SendTo(const Asura_NetSocket_MessageInfo* pxMessageInfo)
{

	#ifdef ASURA_PC_SUPPORT_STEAMWORKS
	if( Asura_PC_Steamworks::IsSteamworksEnabled() )
	{
		Asura_Assert(false, "Unexpected call");
		return false;
	}
	else
	#endif
	{
		SOCKADDR_IN xAddress;
		memset(&xAddress, 0, sizeof(SOCKADDR_IN));
		xAddress.sin_family = AF_INET;
		xAddress.sin_addr.s_addr = pxMessageInfo->m_xAddress.m_uAddress;
		xAddress.sin_port = htons(pxMessageInfo->m_usPort);

		return sendto(m_xSocket, pxMessageInfo->m_pcMessage, pxMessageInfo->m_iLength, 0, (SOCKADDR *)&xAddress, sizeof(SOCKADDR_IN)) != SOCKET_ERROR;
	}
}

bool Asura_NetSocket::Platform_GetData(const char*& pcMessage, int& iLength)
{
	return true;
}

bool Asura_NetSocket::Platform_SetToNonBlocking()
{
    DWORD dwNonBlocking = 1;
    int iResult = ioctlsocket( m_xSocket, FIONBIO, &dwNonBlocking );
    Asura_Assert( iResult != SOCKET_ERROR, "Error setting socket to non blocking" );
	return iResult != SOCKET_ERROR;
}

// needed for ps2 only.
void Asura_NetSocket::Platform_Update()
{
	#ifdef ASURA_PC_SUPPORT_STEAMWORKS
	if( Asura_PC_Steamworks::IsSteamworksEnabled() )
	{
		Asura_PC_NetSocketSteamworks::Update();
	}
	#endif
}

// if Asura_NetAddress is u_int this function is easy, just return it. On PSP things are harder...
u_int Asura_NetSocket::Platform_GetINetAddress( const Asura_NetAddress& xAddress )
{
	return xAddress.m_uAddress;
}

Asura_NetAddress::Asura_NetAddress(unsigned int uAddress)
{
	#ifdef ASURA_PC_SUPPORT_STEAMWORKS
	m_xSteamID	= CSteamID();
	#endif

	m_uAddress = uAddress;
}


bool Asura_NetAddress::operator ==( const Asura_NetAddress& m_xOther ) const
{
	#ifdef ASURA_PC_SUPPORT_STEAMWORKS
	if( Asura_PC_Steamworks::IsSteamworksEnabled() )
	{
		// Always prefer to compare steamID, if not possible compare addresses

		if ( m_xSteamID.IsValid() && m_xOther.m_xSteamID.IsValid() )
		{
			return m_xSteamID == m_xOther.m_xSteamID;
		}
		else
		{
			return  m_uAddress == m_xOther.m_uAddress;
		}
	}
	else	
	#endif
	{
		return ( m_uAddress == m_xOther.m_uAddress );
	}
}

#endif // #ifdef ASURA_SUPPORT_EXTERNAL_NETWORK
