//
// Asura_Platform_Network_GT2.cpp
//
// Created by Kevin Lea - 13/06/2001 11:34:22
// 
// Copyright (C) 2001 Rebellion, All Rights Reserved.
//

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

// Engine independent.
#include "Debugging/Asura_Assert.h"
#include "Debugging/Asura_Logfile.h"
#ifdef ASURA_SUPPORT_EXTERNAL_NETWORK
#include "Network/Asura_GameSpy_GT2.h"
#ifdef ASURA_SUPPORT_GAMESPY
#include "Network/Asura_GameSpy_QR2.h"
#endif // #ifdef ASURA_SUPPORT_GAMESPY
#endif
#include "Network/Asura_Network.h"

// Engine platform-specific.
#ifdef ASURA_PC_SUPPORT_GCDKEY
#include "Network/Asura_PC_GCDKey.h"
#endif


// C L A S S   I M P L E M E N T A T I O N S /////////////////////////////////
void Asura_Network::PlatformInitialise(void)
{
#ifdef ASURA_SUPPORT_EXTERNAL_NETWORK
	#ifdef ASURA_PC_SUPPORT_GCDKEY
	//RAR - To be safe assume player is allowed to join , until we get to a point where we
	//can contact the server to check.
	Asura_PC_GCDKey::SetAuthorised(true);
	Asura_PC_GCDKey::SetErrorMessage(0);
	#endif

	Asura_GameSpy_GT2::Initialise();
#endif // #ifdef ASURA_SUPPORT_EXTERNAL_NETWORK
}


void Asura_Network::PlatformShutdown(void)
{
#ifdef ASURA_SUPPORT_EXTERNAL_NETWORK
	Asura_GameSpy_GT2::Shutdown();
#endif // #ifdef ASURA_SUPPORT_EXTERNAL_NETWORK
}


void Asura_Network::PlatformUpdate(void)
{
#ifdef ASURA_SUPPORT_EXTERNAL_NETWORK
	Asura_GameSpy_GT2::Update();

#ifdef ASURA_SUPPORT_GAMESPY
	if (IsServer())
	{
		Asura_GameSpy_QR2::Update();
	}
#endif // #ifdef ASURA_SUPPORT_GAMESPY
#endif // #ifdef ASURA_SUPPORT_EXTERNAL_NETWORK
}

void Asura_Network::Platform_StartGame()
{
}

void Asura_Network::Platform_SetServerLANReporting(bool bEnable)
{
#ifdef ASURA_SUPPORT_EXTERNAL_NETWORK
	Asura_Assert(IsServer() && IsExternalNetworkActive(), "");
	Asura_GameSpy_GT2::SetServerLANReporting(bEnable);
#endif // #ifdef ASURA_SUPPORT_EXTERNAL_NETWORK
}

void Asura_Network::PlatformStartClient(const char* szIPAddressToConnectTo)
{
#ifdef ASURA_SUPPORT_EXTERNAL_NETWORK
    #ifdef ASURA_PC_SUPPORT_GCDKEY
	Asura_PC_GCDKey::SetWaitingForAuthorisation(true);
	Asura_PC_GCDKey::SetAuthorised(false);
	#endif
	
	Asura_GameSpy_GT2::StartClient(szIPAddressToConnectTo);
#endif // #ifdef ASURA_SUPPORT_EXTERNAL_NETWORK
}


void Asura_Network::PlatformStopClient(void)
{
#ifdef ASURA_SUPPORT_EXTERNAL_NETWORK
	Asura_GameSpy_GT2::StopClient();
#endif // #ifdef ASURA_SUPPORT_EXTERNAL_NETWORK
}

// (MH - 13.07.2004) when a player is kicked, the player's machine must be removed from 
// the game. Platform_RemoveMachineFromGame should result in the machine being removed
// within the delay time. Allowing some delay for buffered and guaranteed messages to 
// arrive is advisable, e.g. to ensure the player is informed of why he was kicked.
void Asura_Network::Platform_RemoveMachineFromGame( int iMachine, float fDelay )
{
#ifdef ASURA_SUPPORT_EXTERNAL_NETWORK
	Asura_Assert(false, "not implemented yet");
#endif
}

bool Asura_Network::PlatformStartServer(void)
{
#ifdef ASURA_SUPPORT_EXTERNAL_NETWORK
    #ifdef ASURA_PC_SUPPORT_GCDKEY
	if(!IsDedicatedServer())
	{
		Asura_PC_GCDKey::SetWaitingForAuthorisation(true);
		Asura_PC_GCDKey::SetAuthorised(false);
	}
	#endif
	bool bLanOnly = !IsPlayingOnInternet();
	return Asura_GameSpy_GT2::StartServerAndHostSession(bLanOnly);
#else
	return true;
#endif // #ifdef ASURA_SUPPORT_EXTERNAL_NETWORK
}

void Asura_Network::PlatformStopServer( void )
{
#ifdef ASURA_SUPPORT_EXTERNAL_NETWORK
	Asura_GameSpy_GT2::StopServer();
#ifdef ASURA_SUPPORT_GAMESPY
	Asura_GameSpy_QR2::Shutdown();
#endif // #ifdef ASURA_SUPPORT_GAMESPY
#endif // #ifdef ASURA_SUPPORT_EXTERNAL_NETWORK
}

void Asura_Network::Platform_SendMessageToServer(const Asura_Message* pxMessage)
{
#ifdef ASURA_SUPPORT_EXTERNAL_NETWORK
	Asura_GameSpy_GT2::SendMessageToServer(pxMessage);
#endif // #ifdef ASURA_SUPPORT_EXTERNAL_NETWORK
}


void Asura_Network::Platform_SendMessageToAllClients(const Asura_Message* pxMessage)
{
#ifdef ASURA_SUPPORT_EXTERNAL_NETWORK
	Asura_GameSpy_GT2::SendMessageToAllClients(pxMessage);
#endif // #ifdef ASURA_SUPPORT_EXTERNAL_NETWORK
}


void Asura_Network::Platform_SendMessageToClientMachine(const Asura_Message* pxMessage, int iMachine)
{
#ifdef ASURA_SUPPORT_EXTERNAL_NETWORK
	Asura_Assert(iMachine>=0 && iMachine<(int)Asura_Network::GetMaxNumberOfMachines(), "invalid machine");

	Asura_GameSpy_GT2::SendMessageToClient(iMachine, pxMessage);
#endif // #ifdef ASURA_SUPPORT_EXTERNAL_NETWORK
}


void Asura_Network::Platform_LockMessageQueue(void)
{
}


void Asura_Network::Platform_UnlockMessageQueue(void)
{
}


float Asura_Network::Platform_GetLatency(void)
{
#ifdef ASURA_SUPPORT_EXTERNAL_NETWORK
	if (s_bServer) return 0.0f;

	u_int uLatency = Asura_GameSpy_GT2::GetLatency();
	return static_cast<float>(uLatency)/1000.0f;
#else
	return 0.0f;
#endif // #ifdef ASURA_SUPPORT_EXTERNAL_NETWORK
}

u_int Asura_Network::Platform_GetLatencyForPlayer(Asura_Guid uPlayerGuid)
{
#ifdef ASURA_SUPPORT_EXTERNAL_NETWORK
	Asura_Assert(IsServer(), "not a server, don't know other machine's pings");
	u_int uMachineIndex = GetMachineForPlayer(uPlayerGuid, true);	// include bots
	if ( (uMachineIndex > 0) && (uMachineIndex < GetNumberOfMachines()) )
	{
		return Asura_GameSpy_GT2::GetLatencyForMachine(uMachineIndex);
	}
#endif // #ifdef ASURA_SUPPORT_EXTERNAL_NETWORK
	return 0;
}

void Asura_Network::Platform_GetNetworkBandwidthUsage(u_int& uSend, u_int& uReceive)
{
#ifdef ASURA_SUPPORT_EXTERNAL_NETWORK
	Asura_GameSpy_GT2::GetNetworkBandwidthUsage(uSend, uReceive);
#endif
}

void Asura_Network::Platform_GetPeakNetworkBandwidthUsage(u_int& uSend, u_int& uReceive)
{
#ifdef ASURA_SUPPORT_EXTERNAL_NETWORK
	Asura_GameSpy_GT2::GetPeakNetworkBandwidthUsage(uSend, uReceive);
#endif
}

float Asura_Network::Platform_GetPacketLossPercent()
{
	// TODO
	return 0.0f;
}

void Asura_Network::Platform_EndOfFrame(void)
{
#ifdef ASURA_SUPPORT_EXTERNAL_NETWORK
	Asura_GameSpy_GT2::EndOfFrame();
#endif // #ifdef ASURA_SUPPORT_EXTERNAL_NETWORK
}


void Asura_Network::Platform_SetServerName(const Asura_UnicodeChar* wszServerName)
{
#ifdef ASURA_SUPPORT_EXTERNAL_NETWORK
	Asura_GameSpy_GT2::SetServerName(wszServerName);
#endif // #ifdef ASURA_SUPPORT_EXTERNAL_NETWORK
}
