//////////////////////////////////////////////////////////////////////////////
//
// Asura_Platform_Leaderboards.cpp
//
// Copyright (c) 2009 Rebellion. All rights reserved.
//
// Description:
//
//     PC support for leaderboards class
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

#ifdef ASURA_SUPPORT_LEADERBOARDS

#include "Network/Online/Asura_Leaderboards.h"

#include "Network/Steamworks/Asura_PC_Steamworks.h"
#include "Network/Steamworks/Asura_PC_SteamworksLeaderboards.h"


void Asura_Leaderboard_System::Platform_Initialise()
{
	// NY - 22 April 2009 - nothing to do here - we only rely on the stats system, and that's init'd elsewhere
}

void Asura_Leaderboard_System::Platform_Shutdown()
{
	// NY - 22 April 2009 - nothing to do here - we only rely on the stats system, and that's shutdown elsewhere
}

void Asura_Leaderboard::Platform_Initialise()
{
	if( Asura_PC_Steamworks::IsInitialised() )
	{
		m_pPlatformData = new Asura_PC_SteamworksLeaderboard_Reader(m_uLeaderboardIndex, m_bFriendsOnly);
	}
}

void Asura_Leaderboard::Platform_Shutdown()
{
	Asura_PC_SteamworksLeaderboard_Reader* pxReader = reinterpret_cast
		<Asura_PC_SteamworksLeaderboard_Reader*>(m_pPlatformData);
	delete pxReader;
}

bool Asura_Leaderboard::Platform_FetchData()
{
	Asura_PC_SteamworksLeaderboard_Reader* pxReader = reinterpret_cast
		<Asura_PC_SteamworksLeaderboard_Reader*>(m_pPlatformData);

	Asura_Assert(pxReader, "no leaderboard reader!");

	if (pxReader)
	{
		return pxReader->FetchRows(m_uFirstRow);
	}
	return false;
}

bool Asura_Leaderboard::Platform_Update()
{
	Asura_PC_SteamworksLeaderboard_Reader* pxReader = reinterpret_cast
		<Asura_PC_SteamworksLeaderboard_Reader*>(m_pPlatformData);

	Asura_Assert(pxReader, "no leaderboard reader!");
	if (pxReader)
	{
		if (!pxReader->IsWorking())
		{
			if (pxReader->IsErrored())
			{
				m_bError = true;
				return true;
			}

			m_uTotalEntries = pxReader->GetTotalRows();

			m_uNumRows = pxReader->GetNumRows();
			const u_int uNumCols = Asura_Leaderboard_System::Project_GetNumLeaderboardColumns(m_uLeaderboardIndex);
			for (u_int u = 0; u < m_uNumRows; ++u)
			{
				SetUTF8NameAndRank(u, pxReader->GetRowName(u), pxReader->GetRowRank(u));
				SetData(u, 0, static_cast<float>(pxReader->GetRowScore(u)));
				// columns 1 to n are the unrated columns
				for (u_int v = 1; v < uNumCols; ++v)
				{
					if (Asura_Leaderboard_System::Project_GetDataFormat(m_uLeaderboardIndex, v) 
						== ASURA_LB_DATAFORMAT_FLOAT)
					{
						SetData(u, v, static_cast<float>(pxReader->GetRowData(u, v - 1) / Asura_PC_SteamworksLeaderboards::GetFixedpointFactor()));
					}
					else
					{
						SetData(u, v, static_cast<float>(pxReader->GetRowData(u, v - 1)));
					}
				}
			}

			pxReader->ClearLoadedData();
			return true;
		}
	}
	else
	{
		m_bError = true;
		return true;
	}
	return false;
}

#endif // #ifdef ASURA_SUPPORT_LEADERBOARDS
