//////////////////////////////////////////////////////////////////////////////
//
// Asura_NetSocketSteamworks.h
//
// Copyright (c) 2008 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef ___ASURA_NETSOCKET_STEAMWORKS_H___
#define ___ASURA_NETSOCKET_STEAMWORKS_H___ 1

#ifdef ASURA_PC_SUPPORT_STEAMWORKS

#include "Network/Asura_NetSocket.h"

enum ASURA_PC_STEAMSOCKET_TYPE
{
	ASURA_PC_STEAMSOCKET_TYPE_AUTO,
	ASURA_PC_STEAMSOCKET_TYPE_SERVER,
	ASURA_PC_STEAMSOCKET_TYPE_CLIENT
};

class Asura_PC_NetSocketSteamworks;

// Pimpl interface class for Asura_PC_NetSocketSteamworks
// have 2 actual implementations - one for client and one for server.
class Asura_PC_NetSocketSteamworksImp
{
public:
	
	Asura_PC_NetSocketSteamworksImp( Asura_PC_NetSocketSteamworks* pxParent ) 
	{ 
		m_pxParent = pxParent; 
		m_bUseReliable = false;
	}
	virtual ~Asura_PC_NetSocketSteamworksImp() {}
	virtual bool Bind(const Asura_NetAddress& xInAddress, u_short usPort) =0;
	virtual bool SendTo(const Asura_NetSocket_MessageInfo* pxMessageInfo) =0;
	virtual bool ReceiveMessage(char* pcBuffer, int* piSize, Asura_NetAddress* pxSourceAddress, u_short* pusSourcePort ) =0;
	virtual void Close() =0;
	virtual int GetLastError() const =0;
	
	void UseReliableSend ( const bool bReliable = true )	{ m_bUseReliable = bReliable; }

protected:

	// TY - Sorry to break the lovely P.IMPL interface but the client implementation needs a pointer to the
	// base socket so that it can update the address once it has connected
	Asura_PC_NetSocketSteamworks* m_pxParent;

	bool m_bUseReliable;
};

// Steamworks Socket
class Asura_PC_NetSocketSteamworks : public Asura_NetSocket
{

	friend class Asura_PC_SteamworksClient;

public:
	Asura_PC_NetSocketSteamworks( u_int uMaxMachines, ASURA_PC_STEAMSOCKET_TYPE eType = ASURA_PC_STEAMSOCKET_TYPE_AUTO );
	virtual ~Asura_PC_NetSocketSteamworks();

	virtual int API_GetLastError();

	static const char* GetStringFromAddress(const Asura_NetAddress& xAddress, unsigned short usPort);
	static bool GetAddressFromString(const char* pcAddress, Asura_NetAddress* pxAddress, unsigned short* pusPort);
	static bool Initialise();
	static void Update();

	void UpdateAddress ( const Asura_NetAddress xAddress, const unsigned short usPort );
	void UseReliableSend ( const bool bReliable = true )	{ m_pxImp->UseReliableSend( bReliable ); }

protected:

	virtual bool	API_Bind(Asura_NetAddress xAddress, u_short& usPort );
	virtual void	API_Close();
	virtual bool	API_ReceiveMessages();
	virtual bool	API_SendTo(const Asura_NetSocket_MessageInfo* pxMessageInfo);
	virtual void	API_SetBroadcastEnabled(bool bEnabled);

private:

	Asura_PC_NetSocketSteamworksImp *m_pxImp;
	
};

#endif // ASURA_PC_SUPPORT_STEAMWORKS

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

#endif // #ifndef ___ASURA_NETSOCKET_STEAMWORKS_H___
