// Asura_PC_SteamworksReliableSocket.h
// Thomas Young - 16.04.09
// Copyright (C) 2009 Rebellion, All Rights Reserved.

// F I L E   W R A P P E R ///////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef ___ASURA_PC_STEAMWORKS_RELIABLESOCKET_H___
#define ___ASURA_PC_STEAMWORKS_RELIABLESOCKET_H___ 1

#ifdef ASURA_PC_SUPPORT_STEAMWORKS

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura_List.h"
#include "Network/Asura_ReliableSocket.h"
#include "Network/Steamworks/Asura_PC_Steamworks.h"
#include "Network/Steamworks/Asura_PC_SteamworksClient.h"

#include "Steam/Steam_api.h"

// E N U M E R A T I O N S ///////////////////////////////////////////////////

enum ASURA_STEAM_RELIABLESOCKET_CLIENT_STATE
{
	ASRSC_STATE_INACTIVE			= 0,	// Not connected
	ASRSC_STATE_AWAITING_CALLBACK	,		// Waiting for callback to say the socket is connected
	ASRSC_STATE_AWAITING_CHALLENGE	,		// Waiting for host data
	ASRSC_STATE_AWAITING_AUTH		,		// [optional] Sent auth blob, waiting for accept/reject from server
	ASRSC_STATE_AWAITING_CONNECT	,		// [optional] Sent connect data, waiting for accept/reject
	ASRSC_STATE_CONNECTED			,		// connection (possibly authorised and) accepted by the server
	ASRSC_STATE_DISCONNECTING		,		// disconnecting
};

enum ASURA_STEAM_RELIABLESOCKET_LISTENER_STATE
{
	ASRSL_STATE_INACTIVE			= 0,	// Not listening
	ASRSL_STATE_LISTENING			,		// Listening for connections
};

// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////////

enum ASURA_PC_STEAMWORKS_AUTH_LEVEL;

class	Asura_ReliableSocket_Client;
class	Asura_ReliableSocket_Listener;
class	Asura_NetPort;

struct	Asura_NetAddress;


// C L A S S   D E C L A R A T I O N S ///////////////////////////////////////

// ___________________ Asura_SteamReliableSocket_Client ______________________

class Asura_SteamReliableSocket_Client : public Asura_ReliableSocket_Client
{
	public:

		Asura_SteamReliableSocket_Client	( );
		~Asura_SteamReliableSocket_Client	( );

		bool Connect		( const Asura_NetAddress& xAddress, Asura_NetPort& xPort, const void* pConnectData, const u_short usConnectDataSize );
		void Disconnect		( const bool bImmediate, const ASURA_RELIABLE_SOCKET_DISCONNECT_REASON eReason );

		void Update			( );

		bool SendMessage	( const u_short usMsgID, const u_short usMsgBodySize, const void* pMsgBody );

		ASURA_RELIABLE_SOCKET_CLIENT_STATE GetState ();

	private:

		void HandleMessage			( char* pcData, const u_int uDataSize );

		void SendChallenge			( );
		void SendAuthData			( const CSteamID xSteamID );
		void SendConnectData		( );
		void SendDisconnecting		( const ASURA_RELIABLE_SOCKET_DISCONNECT_REASON eReason );
		void SendDisconnectAck		( );

		void ClearConnectData		( );

		static void _ClientAccepted ( const CSteamID xSteamID, void* pUserData );
		static void _ClientRejected ( const CSteamID xSteamID, const u_int uReason, void* pUserData );

		void ClientAccepted ( const CSteamID xSteamID );
		void ClientRejected ( const CSteamID xSteamID, const u_int uReason );

		STEAM_CALLBACK( Asura_SteamReliableSocket_Client, OnSocketStatusCallback, SocketStatusCallback_t, m_pfnCallbackSocketStatus );

		Asura_SteamReliableSocket_Client& operator= ( const Asura_SteamReliableSocket_Client& xRhs )	{}
		Asura_SteamReliableSocket_Client( const Asura_SteamReliableSocket_Client& xToCopy )	: Asura_ReliableSocket_Client()
		, m_pfnCallbackSocketStatus( this, &Asura_SteamReliableSocket_Client::OnSocketStatusCallback ) {}


		SNetSocket_t		m_xSocket;
		float				m_fTimeout;

		char*				m_pcConnectData;
		u_short				m_usConnectDataSize;

		Asura_NetAddress	m_xAddress;
		Asura_NetPort		m_xPort;

		bool				m_bAuthenticate;

		ASURA_STEAM_RELIABLESOCKET_CLIENT_STATE m_eState;

		ASURA_RELIABLE_SOCKET_DISCONNECT_REASON m_eDisconnectReason;
};

// __________________ Asura_SteamReliableSocket_Connection _____________________

class Asura_SteamReliableSocket_Connection
{
	public:

		Asura_SteamReliableSocket_Connection()
		{
			m_xID				= GetNextConnectionID();
			m_xSocket			= 0;
			m_fTimeout			= 0.0f;
			m_eState			= ASRSC_STATE_INACTIVE;
			m_eDisconnectReason	= ASURA_RS_DISCONNECT_NOT_DISCONNECTED;
		}

		Asura_ReliableConnectionID	m_xID;
		SNetSocket_t				m_xSocket;
		Asura_NetAddress			m_xAddress;
		Asura_NetPort				m_xPort;
		float						m_fTimeout;

		ASURA_STEAM_RELIABLESOCKET_CLIENT_STATE	m_eState;
		ASURA_RELIABLE_SOCKET_DISCONNECT_REASON m_eDisconnectReason;

		static Asura_ReliableConnectionID s_uNextConnectionID;

		static Asura_ReliableConnectionID GetNextConnectionID ()
		{
			++s_uNextConnectionID;
			return s_uNextConnectionID;
		}

		bool IsLocalUser ()
		{
			if( m_xAddress.m_uAddress == Asura_PC_Steamworks::GetLoopbackAddress() )
			{
				return true;
			}
			if( m_xAddress.m_xSteamID == Asura_PC_SteamworksClient::GetSteamID() )
			{
				return true;
			}
			return false;
		}
};

// __________________ Asura_SteamReliableSocket_Listener _____________________

class Asura_SteamReliableSocket_Listener : public Asura_ReliableSocket_Listener
{
	public:

		Asura_SteamReliableSocket_Listener	( );
		~Asura_SteamReliableSocket_Listener ( );

		bool Listen		( const Asura_NetPort& xPort );
		bool Listen		( const Asura_NetAddress& xAddress, const Asura_NetPort& xPort );
		void Close		( );
		void Disconnect	( const Asura_ReliableConnectionID& xConnectionID, const ASURA_RELIABLE_SOCKET_DISCONNECT_REASON eReason );

		void Update	( );

		bool SendMessage			( const Asura_ReliableConnectionID& xConnectionID, const u_short usMsgID, const u_short usMsgBodySize, const void* pMsgBody );
		bool SendMessageToAll		( const u_short usMsgID, const u_short usMsgBodySize, const void* pMsgBody );
		bool SendMessageToAllExcept	( const u_short usMsgID, const u_short usMsgBodySize, const void* pMsgBody, const Asura_ReliableConnectionID& xException );

		void EnableAuthentication	( const bool bEnable = true );

		ASURA_RELIABLE_SOCKET_LISTENER_STATE GetState ();

		u_short GetListenPort		( ) const;
		u_int	GetNumConnections	( ) const;

	private:

		void Disconnect_Internal ( const Asura_ReliableConnectionID& xConnectionID, const ASURA_RELIABLE_SOCKET_DISCONNECT_REASON eReason );

		bool SendMessage ( Asura_SteamReliableSocket_Connection* pxConnection, const u_short usMsgID, const u_short usMsgBodySize, const void* pMsgBody );

		void HandleMessage ( char* pcData, const u_int uDataSize, Asura_SteamReliableSocket_Connection* pxConnection );
		
		Asura_SteamReliableSocket_Connection* AddConnection		( const SNetSocket_t xSocket );
		Asura_SteamReliableSocket_Connection* FindConnection	( const Asura_ReliableConnectionID xConnectionID );
		Asura_SteamReliableSocket_Connection* FindConnection	( const SNetSocket_t xSocket );
		Asura_SteamReliableSocket_Connection* FindConnection	( const CSteamID xSteamID );
		
		void SendChallengeResponse		( Asura_SteamReliableSocket_Connection* pxConnection );
		void SendAuthDataResponse		( Asura_SteamReliableSocket_Connection* pxConnection, const bool bAccept );
		void SendConnectDataResponse	( Asura_SteamReliableSocket_Connection* pxConnection, const bool bAccept );
		void SendDisconnecting			( Asura_SteamReliableSocket_Connection* pxConnection, const ASURA_RELIABLE_SOCKET_DISCONNECT_REASON eReason );
		void SendDisconnectAck			( Asura_SteamReliableSocket_Connection* pxConnection );
		
		static void _ClientAccepted ( const CSteamID xSteamID, void* pUserData );
		static void _ClientRejected ( const CSteamID xSteamID, const u_int uReason, void* pUserData );

		void ClientAccepted ( const CSteamID xSteamID );
		void ClientRejected ( const CSteamID xSteamID, const u_int uReason );

		STEAM_CALLBACK( Asura_SteamReliableSocket_Listener, OnSocketStatusCallback, SocketStatusCallback_t, m_pfnCallbackSocketStatus );
		
		Asura_SteamReliableSocket_Listener& operator= ( const Asura_SteamReliableSocket_Listener& xRhs )	{}
		Asura_SteamReliableSocket_Listener( const Asura_SteamReliableSocket_Listener &xToCopy )	: Asura_ReliableSocket_Listener()
			, m_pfnCallbackSocketStatus( this, &Asura_SteamReliableSocket_Listener::OnSocketStatusCallback ) {}
		
		SNetListenSocket_t	m_xListenSocket;

		Asura_NetAddress	m_xAddress;
		Asura_NetPort		m_xPort;

		bool				m_bAuthenticate;

		ASURA_STEAM_RELIABLESOCKET_LISTENER_STATE m_eState;

		Asura_List<Asura_SteamReliableSocket_Connection> m_xConnections;
};

#endif	// ASURA_PC_SUPPORT_STEAMWORKS

#endif	// ___ASURA_PC_STEAMWORKS_RELIABLESOCKET_H___

//eof
