//
// Asura_Direct3D.h
//
// Created by Kevin Lea - 02/07/2001 11:11:56
// 
// Copyright (C) 2001 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef __ASURA_DIRECT3D_H__
#define __ASURA_DIRECT3D_H__

#include "Core/Asura_MemoryManagement_DebugOff.h"

#include <d3dx9.h>

// Check which version of the SDK we're building against
// If older than June 2005, throw an error (Mantis #27373)

#include <dxsdkver.h>			// People on Windows 2000 are using really old SDKs because they can't upgrade - and they don't have this file.
#if defined(_DXSDK_BUILD_MAJOR) // To get around it they're dropping a blank dxsdkver.h in their SDK directory, thus we can't perform this test.
#if _DXSDK_BUILD_MAJOR < 239	// Checking whether the symbol is defined is a horrible hack and we need to remove it once everyone is on XP.
#error Your DirectX SDK seems to be older than June 2005, which is incompatible with Asura. Please upgrade your SDK.
#endif
#endif

#ifdef _MSC_VER 
// nonstandard extension used : nameless struct/union
#pragma warning(disable:4201)
#endif

#include "Core/Asura_Maths.h"
#include "Core/Asura_MemoryManagement_DebugOn.h"
#include "Rendering/Asura_Overlay_Quad.h"

// PIX handling: set this define to 1 to have D3DPERFCALL() macros compile to actual PIX calls, and 0 to have them compile out

#if defined( _WIN64)
	#undef ASURA_D3D_ENABLE_PIX
#endif

#if ASURA_D3D_ENABLE_PIX
#include <stdarg.h>
#include "Core/Asura_String.h"

struct Asura_D3D_PIXWrapper
{
	Asura_D3D_PIXWrapper(u_int uColour, const Asura_UnicodeChar* szEventName, ...)
	{
		va_list xArgs; va_start( xArgs, szEventName );
		Asura_UnicodeChar wszBuf[200];
		Asura_PrintfToBfr(wszBuf, szEventName, xArgs, 200, true );
		D3DPERF_BeginEvent(uColour, wszBuf);
	}
	~Asura_D3D_PIXWrapper() { D3DPERF_EndEvent(); }
	
	static void Monitor( const char* pszArgs);
};

// DLH - 24 August 2006 15:47:05 - I get a crash in PIX when trying to load a run file. This is here to help me track this problem down
//#define D3DPERFCALL(arg) { Asura_D3D_PIXWrapper::Monitor( #arg); D3DPERF_##arg;}
#define D3DPERFCALL(arg) D3DPERF_##arg

#define SCOPEDPIXEVENT(args) Asura_D3D_PIXWrapper xScopedPIXEvent##args##;

#else
#define D3DPERFCALL(arg)
#define SCOPEDPIXEVENT(args)
#endif

#if !defined ASURA_DIRECT3D_GRAPHICS_NEAR_CLIP_PLANE
#define ASURA_DIRECT3D_GRAPHICS_NEAR_CLIP_PLANE 0.1f
#endif

#if !defined ASURA_DIRECT3D_GRAPHICS_FAR_CLIP_PLANE
#define ASURA_DIRECT3D_GRAPHICS_FAR_CLIP_PLANE 2000.0f
#endif

#ifdef ASURA_TARGET_SM3
#define ASURA_DIRECT3D_DEFFEDRENDERTARGET_FRONT_INDEX 0
#define ASURA_DIRECT3D_DEFFEDRENDERTARGET_NORMAL_INDEX 1
#define ASURA_DIRECT3D_DEFFEDRENDERTARGET_ALBEDO_INDEX 2
#endif

typedef struct IDirect3D9				Direct3D_Object;
typedef struct IDirect3DDevice9			Direct3D_Device;
typedef struct IDirect3DTexture9		Direct3D_Texture;
typedef struct IDirect3DCubeTexture9	Direct3D_CubeTexture;
typedef struct IDirect3DVertexBuffer9	Direct3D_VertexBuffer;
typedef struct IDirect3DIndexBuffer9	Direct3D_IndexBuffer;
typedef struct IDirect3DSurface9		Direct3D_Surface;
typedef struct IDirect3DBaseTexture9	Direct3D_BaseTexture;
typedef struct IDirect3DVertexDeclaration9 Direct3D_VertexDeclaration;
typedef struct IDirect3DPixelShader9	Direct3D_PixelShader;
typedef struct IDirect3DVertexShader9	Direct3D_VertexShader;
typedef struct IDirect3DSwapChain9		Direct3D_SwapChain;

typedef D3DVIEWPORT9					Direct3D_Viewport;
typedef D3DMATERIAL9					Direct3D_Material;
typedef D3DVERTEXELEMENT9				Direct3D_VertexElement;
typedef D3DCAPS9						Direct3D_Caps;
typedef D3DLIGHT9						Direct3D_Light;

typedef struct ID3DXEffect				Direct3D_Effect;
typedef struct ID3DXEffectPool			Direct3D_EffectPool;

typedef struct IDirect3DQuery9			Direct3D_Query;

// Handy Xbox/360 token, mysteriously absent from PC D3D headers
#define D3DCOLORWRITEENABLE_ALL (D3DCOLORWRITEENABLE_RED | D3DCOLORWRITEENABLE_GREEN \
	| D3DCOLORWRITEENABLE_BLUE | D3DCOLORWRITEENABLE_ALPHA)

#define NUMBER_OF_QUAD_BATCHES 8
#define NUMBER_OF_VERTICES_PER_QUAD_BATCH (4*256)
#define D3DFVF_QUADVERTEX (D3DFVF_XYZRHW|D3DFVF_DIFFUSE|D3DFVF_TEX1)

struct D3DQuadVertex
{
	float X;
	float Y;
	float Z;
	float RHW;
	unsigned int Diffuse;
	float U;
	float V;
};

class Asura_Quad_Batch
{
	public:
		Asura_Quad_Batch()
		{
			Clear();
		}

		void Clear()
		{
			NumberOfVertices = 0;
			VertexBuffer = 0;
			uSwapChainIndex = 0;
			TextureIndex = ASURA_TEXTURE_INVALID;
			m_pxOverrideEffect = 0;
			TranslucencyMode = ASURA_TRANSLUCENCY_NOT_SET;
			m_eTextureOp = ASURA_TEXTUREOP_INVALID;
			VertexBufferLocked = false;
			Vertex = NULL;
		}

		// DJG 20/03/2007 - Added parameter to allow overriding of the effect used to render the quad. The additional
		// parameter is ignored on non-SM3 builds
		bool CanAddPolygonToBatch(class Asura_Overlay_Quad* quad, class Asura_Direct3D_Effect* pxOverrideEffect=NULL);
		void AddPolygonToBatch(class Asura_Overlay_Quad* quad);
		void FlushPolygons(void);

		unsigned int NumberOfVertices;
		
		Direct3D_VertexBuffer* VertexBuffer;

		u_int uSwapChainIndex;

	//private:
		
		void LockVertexBuffer(void);
		void UnlockVertexBuffer(void);
		
		int TextureIndex;
		class Asura_Direct3D_Effect*	m_pxOverrideEffect;

		enum ASURA_TRANSLUCENCY_MODE TranslucencyMode;
		enum ASURA_TEXTUREOP_ID m_eTextureOp;
		bool VertexBufferLocked;
		struct D3DQuadVertex* Vertex;


		static Direct3D_IndexBuffer* s_pxIndexBuffer;

};


#define D3DFVF_GENERICVERTEX (D3DFVF_XYZ|D3DFVF_NORMAL|D3DFVF_DIFFUSE|D3DFVF_TEX2)
struct D3DGenericVertex
{
	float X,Y,Z;
	class Asura_Vector_3 Normal;
	unsigned int Diffuse;
	float U;
	float V;
	float U2;
	float V2;
};

struct VertexFormat
{

	float X;
	float Y;
	float Z;

	float m_NX;
	float m_NY;
	float m_NZ;

	union
	{
		unsigned int Colour;
		struct Colour_Components ColourComponents;
	};

	float U0;
	float V0;

	float U1;
	float V1;

	inline bool operator == ( const VertexFormat& v) const
	{
	    return (X == v.X && Y == v.Y && Z == v.Z && m_NX == v.m_NX && m_NY == v.m_NY && m_NZ == v.m_NZ && Colour == v.Colour && U0 == v.U0 && V0 == v.V0);
	}

	inline bool operator > ( const VertexFormat& v) const
	{
	    if (X < v.X) return false;
	    else if (X > v.X) return true;
	
		if (Y < v.Y) return false;
		else if (Y > v.Y) return true;

		if (Z < v.Z) return false;
		else if (Z > v.Z) return true;

		if (m_NX < v.m_NX) return false;
		else if (m_NX > v.m_NX) return true;

		if (m_NY < v.m_NY) return false;
		else if (m_NY > v.m_NY) return true;

		if (m_NZ < v.m_NZ) return false;
		else if (m_NZ > v.m_NZ) return true;

		if (Colour < v.Colour) return false;
		else if (Colour > v.Colour) return true;

		if (U0 < v.U0) return false;
		else if (U0 > v.U0) return true;

		if (V0 < v.V0) return false;
		else if (V0 > v.V0) return true;

		return false;
	}

	inline bool operator < ( const VertexFormat& v) const
	{
	    if (X > v.X) return false;
	    else if (X < v.X) return true;
	
		if (Y > v.Y) return false;
		else if (Y < v.Y) return true;

		if (Z > v.Z) return false;
		else if (Z < v.Z) return true;

		if (m_NX > v.m_NX) return false;
		else if (m_NX < v.m_NX) return true;

		if (m_NY > v.m_NY) return false;
		else if (m_NY < v.m_NY) return true;

		if (m_NZ > v.m_NZ) return false;
		else if (m_NZ < v.m_NZ) return true;

		if (Colour > v.Colour) return false;
		else if (Colour < v.Colour) return true;

		if (U0 > v.U0) return false;
		else if (U0 < v.U0) return true;

		if (V0 > v.V0) return false;
		else if (V0 < v.V0) return true;

		return false;
	}
};


struct Asura_Polygon_Packet
{
	int StartVertex;
	int EndVertex;
	int IndexStart;
	int NumberOfIndices;
	int m_iMaterialID;
};

class Asura_Direct3D_RenderTarget;

class Asura_Direct3D
{
	public:
		typedef bool (*CallbackPostReset)( void*);
		typedef bool (*CallbackPreReset)( void*);
		
		static Direct3D_Object* D3DObject;
		static Direct3D_Device* D3DDevice;

		static bool Initialised;

#ifdef ASURA_TARGET_SM3
		static Asura_Direct3D_RenderTarget* GetDeferredRenderingTarget() { return s_pxDeferredRenderingTarget; }
#endif
	
		static bool Initialise(void);
		static void CreateD3DObject();
		static void Shutdown(void);
		static void RecalcPresentParameters();
		static bool RestoreAll( const bool bForceReset = false, const bool bRetryOnFail = true);
		static bool HasBeenInitialised(void);
		static void ResetStates(void);
		
		static void AddPostResetCallback( CallbackPostReset pfnPostCallback, void* pData);
		static void RemovePostResetCallback( CallbackPostReset pfnPostCallback, void* pData);
		
		static void AddPreResetCallback( CallbackPreReset pfnPreCallback, void* pData);
		static void RemovePreResetCallback( CallbackPreReset pfnPreCallback, void* pData);

		static void SetViewingArea(void); 
        
        static u_int GetNumValidVideoModes();
        static D3DDISPLAYMODE GetValidVideoMode(u_int uModeNo);
		static bool IsDisplayModeValid(const u_int uWidth, const u_int uHeight, const D3DFORMAT xFormat = D3DFMT_UNKNOWN, const u_int uRefreshRate = 0);

		// KJFL - 2004-02-24 11:32:08 - note: the fsaa & depth parameters are not used!
		static bool SetVideoMode( const int width, const int height, const int depth = 0, const bool fsaa = false, const bool bRetryOnFail = true);

		static void ClearZBuffer(void);

		static void BeginFrame(void);
		static void EndFrame(void);
		static void FlushFrame(void);

		static bool HasFrameBegun(){return FrameHasBegun;}


		// DJG 20/03/2007 - Added parameter to allow overriding of the effect used to render the quad. The additional
		// parameter is ignored on non-SM3 builds
		static void AddQuadToBatch(class Asura_Overlay_Quad* quad, class Asura_Direct3D_Effect* pxOverrideEffect=NULL);
		
		static Asura_Quad_Batch* QuadBatch;
		static u_int m_uQuadBatchNumSwapChains;
		static bool  m_bQuadBatchIsValid;

#if defined(ASURA_TARGET_SM3)
		static Direct3D_VertexDeclaration* s_pxQuadBatchVertexDeclaration;
#endif // defined(ASURA_TARGET_SM3)

		static void UpdateCursor(void);

#ifdef ASURA_TARGET_SM3
		static void BeginDeferredRendering();
		static bool IsUsingDeferredRenderingTarget() { return( s_bIsUsingDeferredRenderingTarget);}
		static void SetUsingDeferredRenderingTarget( const bool bDeferred ) { s_bIsUsingDeferredRenderingTarget = bDeferred; }
#endif

		static void Project_HandleInitialisationFailure(void);

		static Direct3D_Texture* GetUntexturedTexture() {return s_pxUntexturedTexture;}
		static Direct3D_BaseTexture* GetDirect3DTextureByIndex(int iIndex);

#if defined( ASURA_D3D_ENABLE_D3D_OK)
		static void PrintHRESULT( const HRESULT xResult, const char* const pszFilename, const uint32 uiLine);
#endif

		static const D3DXMATRIX& GetViewMatrix() { return s_xViewMatrix; }
		static const D3DXMATRIX& GetProjectionMatrix() { return s_xProjectionMatrix; }
		static const D3DXMATRIX& GetViewProjectionMatrix() { return s_xViewProjMatrix; }

		// Ideally we wouldn't publically expose this but a couple of things like shadow buffers need to mess with the matrices directly
		static void SetCameraMatrices(const D3DXMATRIX& xView, const D3DXMATRIX& xProj);

		// RJF 15-08-06: Bind the rendertarget/depthstencil surfaces that were bound at the beginning of the frame, i.e. the ones our
		//				 final image goes into, so that we can call SetRenderTarget elsewhere without having to store the existing one
		static void BindDefaultTargets();

		static u_int GetNumPresentParameters() { return s_uNumPresentParameters; }
		static u_int GetNumSwapChains() { return s_uNumSwapChains; }

		static D3DPRESENT_PARAMETERS& GetPresentParameters(u_int uParameterIndex = 0) 
		{ 
			Asura_Assert(uParameterIndex < s_uNumPresentParameters, "Invalid index passed to GetPresentParameters" );
			return s_pxPresentParameters[uParameterIndex]; 
		}

		static Direct3D_SwapChain* GetSwapChain(u_int uSwapChainIndex)
		{
			Asura_Assert(uSwapChainIndex < s_uNumSwapChains, "Invalid index passed to GetSwapChain");
			if(uSwapChainIndex >= s_uNumSwapChains) return 0;
			return s_ppxSwapChains[uSwapChainIndex];
		}

		static void RequestSwapChain(u_int uSwapChainIndex);
		static void InvalidateSwapChain();
		static void BindSwapChain(u_int uSwapChainIndex);
		static Asura_Bounding_Rectangle GetSwapChainDisplayLimits(u_int uSwapChainIndex);

		//Sets  a clipping rectangle for rendering. Just used for overlay rendering at the moment , 
		//so may or may not interact with other stuff nicely. RAR 06/09/2006
		static bool GetClippingRectangle(float& fX,float& fY,float& fWidth,float& fHeight);
		static void SetClippingRectangle(float fX,float fY,float fWidth,float fHeight);
		static void ResetClippingRectangle();
		
		static void ClearRenderTargets( const u_int uStartRenderTargetIndex = 0);

		static void SetupD3DMatrix( D3DXMATRIX& xD3DMatrix, const Asura_Matrix_3x3& xAsuraMatrix, const Asura_Vector_3& xAsuraPos, bool bTranspose);

		static Direct3D_VertexDeclaration* GetFullScreenVD() { return s_pxFullScreenVD; }
		static Direct3D_VertexBuffer*      GetFullScreenVB() { return s_pxFullScreenVB; }

		static void SetViewportMaxZ(float fMaxZ)	{ s_fViewportMaxZ = fMaxZ; }
		static void SetViewportMinZ(float fMinZ)	{ s_fViewportMinZ = fMinZ; }

		static bool IsReferenceRasterizer() {return s_eD3DDeviceType == D3DDEVTYPE_REF;}

		static bool UseGammaCorrection() { return s_bUseGammaCorrection; }

	private:
		static bool FrameHasBegun;
		static bool UsingStencil;

		static D3DPRESENT_PARAMETERS* s_pxPresentParameters; 
		static u_int s_uNumPresentParameters;
		static Direct3D_SwapChain** s_ppxSwapChains;
		static u_int s_uNumSwapChains; // Usually this will be the same as NumAdapterHeads, but it doesn't hurt to be flexible
		static u_int s_uCurrentlyBoundSwapChain;

		static bool s_bUseClipRect;
		static float s_fClipRectX;
		static float s_fClipRectY;
		static float s_fClipRectWidth;
		static float s_fClipRectHeight;

		// Multimon support has required that we stop using the EnableAutoDepthStencil feature of CreateDevice, because we need separate
		// depth buffers for each swap chain (due to different sizes). I think we only need the one depth buffer - for the primary
		// display - so we create it by hand instead.
		static Direct3D_Surface* s_pxAutoDepthBuffer;
		static void CreateAutoDepthBuffers();
		static void DestroyAutoDepthBuffers();

		static void InitialiseQuadBatches(void);
		static void ShutdownQuadBatches(void);
	public:
		static void FlushQuadBatches(void);
	private:

		//Creates the default texture to use for un-textured polygons.RAR 10/03/2005
		static void CreateUntexturedTexture();
		
#ifdef ASURA_TARGET_SM3
		static bool CreateDeferredRenderTargetF16( bool);
		static bool CreateDeferredRenderTargetF32( bool);
		static bool CreateDeferredRenderTargetI8( bool);
#endif

#ifdef ASURA_SUPPORT_COMMAND_CONSOLE
		static void AddConsoleCommands();
#endif


		static Direct3D_Texture* s_pxUntexturedTexture;

		static D3DXMATRIX s_xViewMatrix;
		static D3DXMATRIX s_xProjectionMatrix;
		static D3DXMATRIX s_xViewProjMatrix;

#ifdef ASURA_TARGET_SM3
		static Asura_Direct3D_RenderTarget* s_pxDeferredRenderingTarget;
		static bool s_bIsUsingDeferredRenderingTarget;
#endif

		static Direct3D_VertexDeclaration* s_pxFullScreenVD;
		static Direct3D_VertexBuffer* s_pxFullScreenVB;

		static float s_fViewportMaxZ;
		static float s_fViewportMinZ;

		static D3DDEVTYPE s_eD3DDeviceType;

		static bool s_bUseGammaCorrection;
};

// DLH - 17 January 2006 15:27:36 - This is an Direct3D Error processor handler.
// DLH - 17 January 2006 15:30:53 - #ifndef ASURA_D3D_ENABLE_D3D_OK assume always successful!
// DLH - 17 January 2006 15:30:53 - ASURA_D3D_ENABLE_D3D_OK == 1 simple
// DLH - 17 January 2006 15:30:53 - ASURA_D3D_ENABLE_D3D_OK == 2 verbose
#if defined( ASURA_D3D_ENABLE_D3D_OK)
	#if ASURA_D3D_ENABLE_D3D_OK == 2
		inline bool _Asura_D3D_Ok( const HRESULT xResult, const char* const pszFilename, const uint32 uiLine)
		{
			if( SUCCEEDED( xResult)) { return( true);}

			Asura_Direct3D::PrintHRESULT( xResult, pszFilename, uiLine);
			return( false);
		}

		inline bool _Asura_D3D_Failed( const HRESULT xResult, const char* const pszFilename, const uint32 uiLine)
		{
			if( SUCCEEDED( xResult)) { return( false);}

			Asura_Direct3D::PrintHRESULT( xResult, pszFilename, uiLine);
			return( true);
		}
		
		#define ASURA_D3D_OK( _HRESULT) _Asura_D3D_Ok( (_HRESULT), __FILE__, __LINE__)
		#define ASURA_D3D_FAILED( _HRESULT) _Asura_D3D_Failed( (_HRESULT), __FILE__, __LINE__)
	#elif ASURA_D3D_ENABLE_D3D_OK == 1
		#define ASURA_D3D_OK( _HRESULT) SUCCEEDED( (_HRESULT))
		#define ASURA_D3D_FAILED( _HRESULT) FAILED( (_HRESULT))
	#else
		#error ASURA_D3D_ENABLE_D3D_OK is set to an invalid value
	#endif
#else
	#define ASURA_D3D_OK( _HRESULT) true
	#define ASURA_D3D_FAILED( _HRESULT) false
#endif

////////////////////////////////////////////////////////////////////////////////
//
// Stencil Masks
//
// Keep track of who is using which bits of the stencil buffer to ensure the play
// nicely with each other
//
////////////////////////////////////////////////////////////////////////////////

enum {
	//Nothing in asura using it on PC
	ASURA_STENCIL_BIT_FIRST_PROJECT_SPECIFIC = 0,
};
#define ASURA_STENCIL_MASK_PROJECT_SPECIFIC					(0xFF -  ((1<<ASURA_STENCIL_BIT_FIRST_PROJECT_SPECIFIC) - 1))


#endif//__ASURA_DIRECT3D_H__
