
///////////////////////////////////////////////////////////////////////////////
//
// File   : Asura_Direct3D_Effects_System.cpp
// Author : Andy Tate
// Date   : 11-May-2005
//
// Copyright (C) 2005 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include <stdio.h>

#include "Core/Asura.h"

// This file's header...
#include "Rendering/Asura_Direct3D_Effects_System.h"

// Asura header-files...
#include "Camera/Asura_Camera.h"
#include "Core/Asura_Timers.h"
#include "Debugging/Asura_Debug_Timers.h"
#include "FileAccess/Asura_FileAccess.h"
#include "Rendering/Asura_Colour.h"
#include "Rendering/Asura_Direct3D_Effect.h"
#include "Rendering/Asura_Fog.h"
#include "Rendering/Asura_Render.h"
#include "Rendering/Asura_Direct3D_EffectParameterBlock.h"
#if defined(ASURA_USE_MATERIAL_RESPONSE_SYSTEM) && (((ASURA_LOAD_EFFECT_FILES == 1) && !defined( ASURA_NO_LOAD_EFFECTS_FILES_FROM_FILES)) || defined(ASURA_TOOLS))
#include "Rendering/Asura_Direct3D_MRPShaderCompile.h"
#endif

#if ASURA_LOAD_EFFECT_FILES == 1
#include "Shared.h"										

#include "Env_ZPass.h"										
#include "Env_ZPassAABB.h"									

// Custom-view effects...
#include "Env_XRay.h"										

// Object shaders
#include "Object_TextureAndLight.h"							
#if defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM
#include "Env_Water.h"
#include "Env_Shadow.h"
#include "Env_MaterialResponse.h"
#include "Object_MaterialResponse.h"
#include "SmoothSkin_MaterialResponse.h"
#endif

#include "SmoothSkin_Wireframe.h"

#if defined(ASURA_PC_USE_SHADER_MODEL_3_SMOOTH_SKIN_RENDERER) && ASURA_PC_USE_SHADER_MODEL_3_SMOOTH_SKIN_RENDERER
// Smooth skin renderer
#include "SmoothSkin_SM3.h"
#include "SmoothSkin_SM3_Shadow.h"
#if defined (ASURA_USE_DAMAGE_MAPS)
#include "SmoothSkin_SM3_DamageMapped.h"
#endif
#endif


#if defined(ASURA_TARGET_SM3)
#include "ApplyShadows.h"
#include "Object_Shadow.h"
#include "Fog.h"
#include "Skybox.h"
#if defined( ASURA_USE_MATERIAL_RESPONSE_SYSTEM )
#include "ApplyProjectedTexture_MaterialResponse.h"									
#endif
#include "FSFX_SM30_Bloom.h"
#include "FSFX_ColourMap.h"
#include "FSFX_SM30_DepthOfField.h"
#include "FSFX_SM30_FishEye.h"
#include "FSFX_SM30_DoubleVision.h"
#include "FSFX_SM30_MotionBlur.h"
#endif
#if defined (ASURA_USE_FASTPATH_PFX)
#include "PFX_FastPath.h"									
#endif
#if defined(ASURA_TARGET_SM3)
#include "PFX_SlowPath.h"									
#include "Decals.h"
#include "Coronas.h"
#include "LightShaft.h"
#endif	// defined(ASURA_TARGET_SM3)

#if defined( ASURA_DIRECT3D_BGR_SUPPORT_SM30) && ASURA_DIRECT3D_BGR_SUPPORT_SM30
#include "Bot_3D_Prelighting_Pass.h"	
#include "Bot_Billboard_Prelighting_Pass.h"	
#include "Bot_Imposter1_Prelighting_Pass.h"	
#include "Bot_Imposter2_Prelighting_Pass.h"	
#include "Bot_Imposter1_Interactive_Prelighting_Pass.h"	
#include "Bot_Imposter2_Interactive_Prelighting_Pass.h"	
#endif
#if defined( ASURA_USE_CARPET)
#include "Carpet.h"	
#endif
#if defined( ASURA_TARGET_SM3)
#include "ApplyDeferredPhoton_MaterialResponse.h"
#endif

#if defined( ASURA_TOOLS )
#include "OverlayQuad_Cubemap.h"
#include "OverlayQuad_HDRHistogram.h"
#endif

#if defined( ASURA_USE_HDR )
#include "HDR.h"
#endif

#if defined( ASURA_USE_CSM )
#include "CSM.h"
#endif

#if defined( ASURA_USE_AMBIENT_OCCLUSION )
#include "ScreenSpaceAmbientOcclusion.h"
#endif

#endif //#if ASURA_LOAD_EFFECT_FILES == 1

#if defined(ASURA_TOOLS)
// Tools_Render referred to by SetSharedParameters - can be removed if that fn goes elsewhere.
#include "Misc/Tools_Render.h"
// EffectParameterBlock referred to by Tools_ReloadAllEffects
#endif

// Game header-files...


////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////

// Shared effects...

Asura_Direct3D_Effect*	Asura_Direct3D_Effects_System::s_pxSharedEffect	= NULL;


// Currently begun effect...

Asura_Direct3D_Effect*	Asura_Direct3D_Effects_System::s_pxCurrentEffect	= NULL;
u_int					Asura_Direct3D_Effects_System::s_uCurrentTechnique	= NULL;

// Tree to store the effects in...

Asura_Direct3D_Effects_System::EffectTree	Asura_Direct3D_Effects_System::s_xEffectTree;

// Shared parameter handles

Asura_Direct3D_Effects_System::SharedParameters Asura_Direct3D_Effects_System::s_xSharedParameterHandles;



#ifdef ASURA_TARGET_SM3
bool Asura_Direct3D_Effects_System::s_bLoadShadersFromDisc = false;
#else // #ifdef ASURA_TARGET_SM3
bool Asura_Direct3D_Effects_System::s_bLoadShadersFromDisc = true;
#endif // #ifdef ASURA_TARGET_SM3


#if defined(ASURA_LOAD_EFFECT_FILES) && ASURA_LOAD_EFFECT_FILES == 1
	Asura_Direct3D_Effects_System_Effect *Asura_Direct3D_Effects_System_Effect::s_pxFirst = NULL;

	Asura_Direct3D_Effects_System_Effect::Asura_Direct3D_Effects_System_Effect(const char* szName, const void* pCompiledShader, u_int  uCompiledShaderSize) 
	: m_pxNext(s_pxFirst), m_szName(szName), m_pCompiledShader(pCompiledShader), m_uCompiledShaderSize(uCompiledShaderSize)
	{
		s_pxFirst = this;
	}
#endif // #if defined(ASURA_LOAD_EFFECT_FILES) && ASURA_LOAD_EFFECT_FILES == 1



// Visibility-system effects...
ASURA_DECLARE_DIRECT3D_EFFECT(Env_ZPass);									
ASURA_DECLARE_DIRECT3D_EFFECT(Env_ZPassAABB);

// Custom-view effects...
ASURA_DECLARE_DIRECT3D_EFFECT(Env_XRay);

// Object shaders
ASURA_DECLARE_DIRECT3D_EFFECT(Object_TextureAndLight);
#if defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM
#if !defined ASURA_MATERIAL_RESPONSE_CHARACTERS_ONLY
ASURA_DECLARE_DIRECT3D_EFFECT(Env_Water);
ASURA_DECLARE_DIRECT3D_EFFECT(Env_Shadow);
ASURA_DECLARE_DIRECT3D_EFFECT(Env_MaterialResponse);
#endif
ASURA_DECLARE_DIRECT3D_EFFECT(Object_MaterialResponse);
ASURA_DECLARE_DIRECT3D_EFFECT(SmoothSkin_MaterialResponse);

#endif

ASURA_DECLARE_DIRECT3D_EFFECT(SmoothSkin_Wireframe);

#if defined(ASURA_PC_USE_SHADER_MODEL_3_SMOOTH_SKIN_RENDERER) && ASURA_PC_USE_SHADER_MODEL_3_SMOOTH_SKIN_RENDERER
// Smooth skin renderer

ASURA_DECLARE_DIRECT3D_EFFECT(SmoothSkin_SM3);
ASURA_DECLARE_DIRECT3D_EFFECT(SmoothSkin_SM3_Shadow);
#if defined (ASURA_USE_DAMAGE_MAPS)
ASURA_DECLARE_DIRECT3D_EFFECT(SmoothSkin_SM3_DamageMapped);
#endif
#endif


#if defined(ASURA_TARGET_SM3)
ASURA_DECLARE_DIRECT3D_EFFECT(ApplyShadows);
ASURA_DECLARE_DIRECT3D_EFFECT(Object_Shadow);
ASURA_DECLARE_DIRECT3D_EFFECT(Fog);
ASURA_DECLARE_DIRECT3D_EFFECT(Skybox);
#if defined( ASURA_USE_MATERIAL_RESPONSE_SYSTEM )
ASURA_DECLARE_DIRECT3D_EFFECT(ApplyProjectedTexture_MaterialResponse);
#endif
ASURA_DECLARE_DIRECT3D_EFFECT(FSFX_SM30_Bloom);
ASURA_DECLARE_DIRECT3D_EFFECT(FSFX_ColourMap);
ASURA_DECLARE_DIRECT3D_EFFECT(FSFX_SM30_DepthOfField);
ASURA_DECLARE_DIRECT3D_EFFECT(FSFX_SM30_FishEye);
ASURA_DECLARE_DIRECT3D_EFFECT(FSFX_SM30_DoubleVision);
ASURA_DECLARE_DIRECT3D_EFFECT(FSFX_SM30_MotionBlur);
#endif
#if defined (ASURA_USE_FASTPATH_PFX)
ASURA_DECLARE_DIRECT3D_EFFECT(PFX_FastPath);
#endif
#if defined(ASURA_TARGET_SM3)
ASURA_DECLARE_DIRECT3D_EFFECT(PFX_SlowPath);
ASURA_DECLARE_DIRECT3D_EFFECT(Decals);
ASURA_DECLARE_DIRECT3D_EFFECT(Coronas);
ASURA_DECLARE_DIRECT3D_EFFECT(Lightshaft);
#endif	// defined(ASURA_TARGET_SM3)

#if defined( ASURA_DIRECT3D_BGR_SUPPORT_SM30) && ASURA_DIRECT3D_BGR_SUPPORT_SM30
ASURA_DECLARE_DIRECT3D_EFFECT(Bot_3D_Prelighting_Pass);
ASURA_DECLARE_DIRECT3D_EFFECT(Bot_Billboard_Prelighting_Pass);
ASURA_DECLARE_DIRECT3D_EFFECT(Bot_Imposter1_Prelighting_Pass);
ASURA_DECLARE_DIRECT3D_EFFECT(Bot_Imposter2_Prelighting_Pass);
ASURA_DECLARE_DIRECT3D_EFFECT(Bot_Imposter1_Interactive_Prelighting_Pass);
ASURA_DECLARE_DIRECT3D_EFFECT(Bot_Imposter2_Interactive_Prelighting_Pass);
#endif
#if defined( ASURA_USE_CARPET)
ASURA_DECLARE_DIRECT3D_EFFECT(Carpet);
#endif
#if defined( ASURA_TARGET_SM3)
ASURA_DECLARE_DIRECT3D_EFFECT(ApplyDeferredPhoton_MaterialResponse);
#endif

#if defined( ASURA_TOOLS )
ASURA_DECLARE_DIRECT3D_EFFECT(OverlayQuad_Cubemap);
ASURA_DECLARE_DIRECT3D_EFFECT(OverlayQuad_HDRHistogram);
#endif

#if defined( ASURA_USE_HDR )
ASURA_DECLARE_DIRECT3D_EFFECT(HDR);
#endif

#if defined( ASURA_USE_CSM )
ASURA_DECLARE_DIRECT3D_EFFECT(CSM);
#endif

#if defined( ASURA_USE_AMBIENT_OCCLUSION )
ASURA_DECLARE_DIRECT3D_EFFECT(ScreenSpaceAmbientOcclusion);
#endif

////////////////////////////////////////////////////////////////////////////////
// Private functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Protected functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Public functions...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Direct3D_Effects_System::Initialise
// Author       : Andy Tate
// Date         : 11-May-2005
//
// Description  : Handles the initialisation of the Direct3D effects system.
//
// Return value : None.
////////////////////////////////////////////////////////////////////////////////

void Asura_Direct3D_Effects_System::Initialise (void)
{
	if( NULL == Asura_Direct3D::D3DDevice)
	{
		return;
	}
	
	ASURA_CPU_PROFILE_SCOPED_TIMER( "Asura_Direct3D_Effects_System::Initialise");
	// Create the Direct3D effect-pool...

	Asura_Direct3D_Effect::CreateEffectPool	();


// JAT 22/02/07 only want to load effects in tools builds on current-gen (but on all builds for next-gen)
#if ASURA_LOAD_EFFECT_FILES == 1

	// Load the shared effects...
#if !defined( ASURA_NO_LOAD_EFFECTS_FILES_FROM_FILES)
	if (s_bLoadShadersFromDisc)
	{
		SetSharedEffect( LoadEffectFromFile("Shared", "Shaders\\Shared.fx"));
	} 
	else
#endif
	{
		SetSharedEffect( LoadEffectFromMemory(Asura_GetHashID ("Shared"), g_auShared, sizeof(g_auShared)));
	}


	// Load the remaining effects...
	for (Asura_Direct3D_Effects_System_Effect *pxEffect = Asura_Direct3D_Effects_System_Effect::s_pxFirst; 
		NULL != pxEffect; pxEffect = pxEffect->m_pxNext )
	{
#if ASURA_LOAD_EFFECT_FILES == 1 && !defined( ASURA_NO_LOAD_EFFECTS_FILES_FROM_FILES)
		if (s_bLoadShadersFromDisc)
		{
			char szFilename [256];

			// Create the filename...
			Asura_PrintfToBfr( szFilename, "Shaders\\%s.fx", 255, true, pxEffect->m_szName );

			// Load this effect...
			Asura_Direct3D_Effects_System::LoadEffectFromFile(pxEffect->m_szName, szFilename);
		} 
		else
#endif
		{
			Asura_Direct3D_Effects_System::LoadEffectFromMemory(Asura_GetHashID(pxEffect->m_szName), pxEffect->m_pCompiledShader, pxEffect->m_uCompiledShaderSize);
		}

	}
#endif	// #if ASURA_LOAD_EFFECT_FILES == 1

	Asura_Direct3D::AddPreResetCallback( Asura_Direct3D_Effects_System::PreResetCallback, NULL);
}

/// End of function 'Asura_Direct3D_Effects_System::Initialise'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Direct3D_Effects_System::Reset
// Author       : Andy Tate
// Date         : 11-May-2005
//
// Description  : Simply shuts down the effects system then re-initialises it.
//
// Return value : None.
////////////////////////////////////////////////////////////////////////////////

void Asura_Direct3D_Effects_System::Reset (void)
{
	//Shutdown();
	//Initialise();
	
	// MGH 15/1/2008: Reset the parameter tree to avoid memory fragmentation,
	// which would mess up the memory checkpointing otherwise.
	for ( Asura_TemplateHashTree_Iterator<Asura_Direct3D_Effect*> xIt(s_xEffectTree);
		 !xIt.Done();
		  xIt.Next() )
	{
		Asura_Direct3D_Effect** ppxEffect = xIt.GetCurrent();
		if (ppxEffect && *ppxEffect)
		{
			(*ppxEffect)->ResetParameterTree();
		}
	}
}

/// End of function 'Asura_Direct3D_Effects_System::Reset'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Direct3D_Effects_System::Shutdown
// Author       : Andy Tate
// Date         : 11-May-2005
//
// Description  : Handles the shutdown of the Direct3D effects system.
//
// Return value : None.
////////////////////////////////////////////////////////////////////////////////

void Asura_Direct3D_Effects_System::Shutdown (void)
{
	if( NULL == Asura_Direct3D::D3DDevice)
	{
		return;
	}
#if !defined( ASURA_NO_LOAD_EFFECTS_FILES_FROM_FILES)
#if ASURA_LOAD_EFFECT_FILES == 1 && defined(ASURA_USE_MATERIAL_RESPONSE_SYSTEM)
	Asura_Direct3D_MRPShaderCompile::SetLoadShadersFromFile(false);
#endif	
#endif
	Asura_Direct3D::RemovePreResetCallback( Asura_Direct3D_Effects_System::PreResetCallback, NULL);
	
	Asura_Hash_ID*	auEffectIDs	= s_xEffectTree.MakeHashList ();


	// Do we have any effects to delete?

	if (NULL != auEffectIDs)
	{
		// Yes - go through each effect and delete it's data...

		for (int i = 0; i < s_xEffectTree.GetNumberOfNodes(); i++)
		{
			Asura_Hash_ID			uEffectID	= auEffectIDs [i];
			Asura_Direct3D_Effect**	ppxEffect	= s_xEffectTree.GetDataByHash (uEffectID);


			// Delete this effect...

			delete	(*ppxEffect);

			*ppxEffect	= NULL;
		}


		// Empty the tree AND delete the hash-list we've made...

		s_xEffectTree.Reset	();
		delete []			(auEffectIDs);
	}


	// Destroy the Direct3D effect-pool...

	Asura_Direct3D_Effect::DestroyEffectPool	();
}

/// End of function 'Asura_Direct3D_Effects_System::Shutdown'. ///

bool Asura_Direct3D_Effects_System::PreResetCallback( void* pData)
{
	Asura_Hash_ID*	auEffectIDs	= s_xEffectTree.MakeHashList ();

	// Do we have any effects to delete?
	if (NULL != auEffectIDs)
	{
		// Yes - go through each effect and prereset it's data...
		for (int i = 0; i < s_xEffectTree.GetNumberOfNodes(); i++)
		{
			Asura_Hash_ID			uEffectID	= auEffectIDs [i];
			Asura_Direct3D_Effect**	ppxEffect	= s_xEffectTree.GetDataByHash (uEffectID);
			if( NULL == ppxEffect)
			{
				continue;
			}
			
			Asura_Direct3D_Effect* pxEffect = *ppxEffect;
			if( NULL == pxEffect)
			{
				continue;
			}
			
			pxEffect->PreReset();
		}
		// Finally, delete the hash-list we've made...
		delete []			(auEffectIDs);
	}

	return( true);
}


// JAT 22/02/07 only want to load effects in tools builds on current-gen (but on all builds for next-gen)
#if ASURA_LOAD_EFFECT_FILES == 1 

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Direct3D_Effects_System::LoadEffectFromFile
// Author       : Andy Tate
// Date         : 11-May-2005
//
// Description  : Loads the specified Direct3D effect from a file, and maps it
//				  to the effect name provided.
//
// Parameters   : The name of the effect to load AND the corresponding filename.
//
// Return value : A pointer to the effect loaded, NULL if we couldn't load the
//				  requested effect.
////////////////////////////////////////////////////////////////////////////////

Asura_Direct3D_Effect* Asura_Direct3D_Effects_System::LoadEffectFromFile( char const* szEffectName, char const* szFilename )															
{
	Asura_Direct3D_Effect*	pxEffect	= new Asura_Direct3D_Effect( szEffectName, szFilename );
	Asura_Hash_ID			uEffectID	= Asura_GetHashID (szEffectName);


	// Did we add the node to the tree successfully?

	if (s_xEffectTree.AddNode(uEffectID, pxEffect))
	{
		// Yes - return the effect...

		return	(pxEffect);
	}

	else
	{
		// No - delete the effect AND return failure...

		delete	(pxEffect);
		return	(NULL);
	}
}

/// End of function 'Asura_Direct3D_Effects_System::LoadEffectFromFile'. ///

#if !defined( ASURA_NO_LOAD_EFFECTS_FILES_FROM_FILES)

void Asura_Direct3D_Effects_System::SetShadersPath( const char* const szShadersPath )
{
	Asura_Direct3D_Effect::SetShadersPath(szShadersPath);
	s_bLoadShadersFromDisc = true;
#if ASURA_LOAD_EFFECT_FILES == 1 && defined(ASURA_USE_MATERIAL_RESPONSE_SYSTEM)
	Asura_Direct3D_MRPShaderCompile::SetLoadShadersFromFile(s_bLoadShadersFromDisc);
#endif
}


////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Direct3D_Effects_System::Tools_ReloadAllEffects
// Author       : Andy Tate
// Date         : 11-May-2005
//
// Description  : Closes down ALL of the previously loaded effects and reloads 
//				  them all for further use.
//
//				  NOTE: This function allows effects to be modified and reloaded
//				  ~~~~~ within the current game without exitting and restarting
//				        the application.
//
// Return value : None.
////////////////////////////////////////////////////////////////////////////////

void Asura_Direct3D_Effects_System::Tools_ReloadAllEffects (void)
{
	ASURA_CPU_PROFILE_SCOPED_TIMER( "Reload Effects");
	
	Asura_Hash_ID*	auEffectIDs	= s_xEffectTree.MakeHashList ();

	// Unset the shared effect to kill off its parameter handles
	Asura_Direct3D_Effect* pxSharedEffect = s_pxSharedEffect;
	SetSharedEffect(NULL);

	// Do we have any effects to delete?

	if (NULL != auEffectIDs)
	{
		// Yes - go through each effect and reload it...

		// KOR - 17/06/2005 - was crashing due to the current directory being changed during texture loads
#if defined( ASURA_PC_ALLOW_LOAD_FROM_NETWORK )
		SetCurrentDirectory(Asura_FileAccess::PlatformGetAsuraFilesDir());
#endif
		for (int i = 0; i < s_xEffectTree.GetNumberOfNodes(); i++)
		{
			Asura_Hash_ID			uEffectID	= auEffectIDs [i];
			Asura_Direct3D_Effect**	ppxEffect	= s_xEffectTree.GetDataByHash (uEffectID);


			// Reload this effect...

			(*ppxEffect)->Tools_Reload	();
		}


		// Finally, delete the hash-list we've made...

		delete []	(auEffectIDs);
	}

	// Reset the shared effect
	SetSharedEffect(pxSharedEffect);

	// Reset all parameter blocks
	Asura_Direct3D_EffectParameterBlock::Tools_RegenerateAll();
}

/// End of function 'Asura_Direct3D_Effects_System::Tools_ReloadAllEffects'. ///

#endif

#endif // #if ASURA_LOAD_EFFECT_FILES == 1



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Direct3D_Effects_System::LoadEffectFromMemory
// Author       : Andy Tate
// Date         : 11-May-2005
//
// Description  : Loads the specified Direct3D effect from memory (where it's
//                been previously loaded in a .asr file), and maps it to the
//				  effect name provided.
//
// Parameters   : The ID of the effect to load, along with the location in 
//				  memory where we can find the effect data.
//
// Return value : A pointer to the effect loaded, NULL if we couldn't load the
//				  requested effect.
////////////////////////////////////////////////////////////////////////////////

Asura_Direct3D_Effect* Asura_Direct3D_Effects_System::LoadEffectFromMemory (const Asura_Hash_ID uEffectID,
																			void const* pvData,
																			const u_int uDataSize)
{
	Asura_Direct3D_Effect*	pxEffect	= new Asura_Direct3D_Effect (pvData, uDataSize);


	// Did we add the node to the tree successfully?

	if (s_xEffectTree.AddNode(uEffectID, pxEffect))
	{
		// Yes - return the effect...

		return	(pxEffect);
	}

	else
	{
		// No - delete the effect AND return failure...

		delete	(pxEffect);
		return	(NULL);
	}
}

/// End of function 'Asura_Direct3D_Effects_System::LoadEffectFromMemory'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Direct3D_Effects_System::GetEffect
// Author       : Andy Tate
// Date         : 11-May-2005
//
// Description  : Attempts to find the Direct3D effect which matches the specified
//				  effect-name.
//
// Parameters   : The ID of the effect to find.
//
// Return value : A pointer to the effect found, NULL if we can't find a mapping.
////////////////////////////////////////////////////////////////////////////////

Asura_Direct3D_Effect* Asura_Direct3D_Effects_System::GetEffect (const Asura_Hash_ID uEffectID)
{
	Asura_Direct3D_Effect**	ppxEffect	= s_xEffectTree.GetDataByHash (uEffectID);


	// Did we find an effect?

	if (NULL == ppxEffect)
	{
		// No - return failure...

		return	(NULL);
	}

	else
	{
		// Yes - return the effect itself...

		return	(*ppxEffect);
	}
}

/// End of function 'Asura_Direct3D_Effects_System::GetEffect'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Direct3D_Effects_System::SetSharedParameters
// Author       : Andy Tate
// Date         : 18-May-2005
//
// Description  : Sets the effect parameters required by the view.
//
// Return value : None.
////////////////////////////////////////////////////////////////////////////////

void Asura_Direct3D_Effects_System::SetSharedParameters (void)
{
    if( !s_pxSharedEffect ) { return; }

	SCOPEDPIXEVENT((0xffeebbcc, L"Set effects system shared parameters"));

	Direct3D_Effect* pxEffect = s_pxSharedEffect->GetD3DEffect();
	Asura_Assert(pxEffect, "Shared effect is not null, but the underlying D3D effect is null");
	if(!pxEffect) return;

	float				fCurrentTime				= Asura_Timers::GetLocalScaledTime ();

	D3DXVECTOR4			xCameraPosition (Asura_Camera::GetPosition()->x, Asura_Camera::GetPosition()->y, Asura_Camera::GetPosition()->z, 0.0f);

	D3DXMATRIX			xViewMatrix;
	D3DXMATRIX			xViewProjectionMatrix;

	// (AAM - 01.06.2006) Ensure the fixed-function matrices are up to date with the latest camera data
	Asura_Direct3D::SetViewingArea();

	// Calculate the view-projection matrix...

	Asura_Direct3D::D3DDevice->GetTransform	(D3DTS_VIEW,		&xViewMatrix);
	Asura_Direct3D::D3DDevice->GetTransform	(D3DTS_PROJECTION,	&xViewProjectionMatrix);

	xViewProjectionMatrix	= xViewMatrix * xViewProjectionMatrix;


	// Set the parameters...

	pxEffect->SetMatrix		(s_xSharedParameterHandles.m_xViewProjectionMatrix,	&xViewProjectionMatrix);
	pxEffect->SetFloat		(s_xSharedParameterHandles.m_xCurrentTime,			fCurrentTime);
	pxEffect->SetFloatArray	(s_xSharedParameterHandles.m_xCameraPosition,		xCameraPosition, 4);

#if defined(ASURA_TOOLS)
	Asura_Assert(s_xSharedParameterHandles.m_xDisableLighting, "Null DisableLighting Handle, Possibly due to inconsistent definition of ASURA_TOOLS");
	if (s_xSharedParameterHandles.m_xDisableLighting)
	{
		// Disable lighting if the Tools user has requested it... some times the Tools defaults to lighting
		// being disabled, like when you load an object / character rather than an environment.
		pxEffect->SetBool( s_xSharedParameterHandles.m_xDisableLighting, !(Tools_Render::LightingIsOn()) );
	}
#if defined( ASURA_TARGET_SM3 )
	Asura_Assert(s_xSharedParameterHandles.m_xShowPerPixelNormals, "Null ShowPerPixelNormals Handle, Possibly due to inconsistent definition of ASURA_TOOLS");
	if (s_xSharedParameterHandles.m_xShowPerPixelNormals)
	{
		pxEffect->SetBool( s_xSharedParameterHandles.m_xShowPerPixelNormals, Tools_Render::ShowPerPixelNormalsIsOn() );
	}
#endif
#endif // #if defined(ASURA_TOOLS)

	pxEffect->SetBool( s_xSharedParameterHandles.m_xUseGammaCorrection, Asura_Direct3D::UseGammaCorrection() );
}

/// End of function 'Asura_Direct3D_Effects_System::SetSharedParameters'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Direct3D_Effects_System::BeginEffect
// Author       : Andy Tate
// Date         : 25-May-2005
//
// Description  : Begins the specified Direct3D effect. If this effect has
//				  already begun then we simply returns the pointer to the
//				  existing effect.
//
// Parameters   : A reference to the effect to begin, and optional technique index.
//
// Return value : A pointer to the effect we've begun.
////////////////////////////////////////////////////////////////////////////////

Asura_Direct3D_Effect* Asura_Direct3D_Effects_System::BeginEffect (Asura_Direct3D_Effect& xEffect,
																   const u_int uTechniqueIndex /*= 0*/)
{
	// Do we have a currently active effect?

	if ( (NULL != s_pxCurrentEffect)
	  && (s_pxCurrentEffect != &xEffect || s_uCurrentTechnique != uTechniqueIndex) )
	{
		// Yes - end this effect...

		EndEffect	(*s_pxCurrentEffect);
	}


	// Do we have a change of effect?

	if (s_pxCurrentEffect != &xEffect || s_uCurrentTechnique != uTechniqueIndex)
	{
		// Yes - is the new effect valid?
		
		if (xEffect.IsD3DEffectValid())
		{
			// Yes - begin the new effect...

			xEffect.Begin	(false, uTechniqueIndex);


			// ...and set this as the current effect...

			s_pxCurrentEffect	= &xEffect;
			s_uCurrentTechnique	= uTechniqueIndex;
		}
	}


	// Return the current effect...

	return	(&xEffect);
}

/// End of function 'Asura_Direct3D_Effects_System::BeginEffect'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Direct3D_Effects_System::BeginEffect
// Author       : Andy Tate
// Date         : 25-May-2005
//
// Description  : Begins the Direct3D effect with the specified name. If this
//				  effect is already begun then we simply returns the pointer
//				  to the existing effect.
//
// Parameters   : The name of the effect to begin, and optional technique index.
//
// Return value : A pointer to the effect we've begun.
////////////////////////////////////////////////////////////////////////////////

Asura_Direct3D_Effect* Asura_Direct3D_Effects_System::BeginEffect (const char* szEffectName,
																   const u_int uTechniqueIndex /*= 0*/)
{
	Asura_Direct3D_Effect*	pxEffect		= GetEffect (szEffectName);
	bool					bEffectPresent	= (NULL != pxEffect);


	// Do we have an effect?

	if (bEffectPresent)
	{
		// Yes - begin this effect...

		return	(BeginEffect(*pxEffect, uTechniqueIndex));
	}

	else
	{
		// No - report this as an error...

		Asura_Assert	(bEffectPresent, "Cannot find specified effect.");


		// Do we have a current effect?

		if (NULL != s_pxCurrentEffect)
		{
			// Yes - better end this...

			s_pxCurrentEffect->End	();

			s_pxCurrentEffect	= NULL;
		}


		// Return NULL...

		return	(s_pxCurrentEffect);
	}
}

/// End of function 'Asura_Direct3D_Effects_System::BeginEffect'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Direct3D_Effects_System::EndEffect
// Author       : Andy Tate
// Date         : 25-May-2005
//
// Description  : Ends the specified effect.
//
// Parameters   : A reference to the effect to end.
//
// Return value : None.
////////////////////////////////////////////////////////////////////////////////

void Asura_Direct3D_Effects_System::EndEffect (const Asura_Direct3D_Effect& xEffect)
{
	// Is this the current effect?

	if (s_pxCurrentEffect == &xEffect)
	{
		// Yes - end the current effect...

		s_pxCurrentEffect->End	();


		// ...then reset it...

		s_pxCurrentEffect	= NULL;
	}

	else
	{
		// No - that's bad...

		Asura_Assert	(false, "Cannot end this effect, it's not the currently active one.");
	}
}

/// End of function 'Asura_Direct3D_Effects_System::EndEffect'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Direct3D_Effects_System::FlushEffects
// Author       : Andy Tate
// Date         : 25-May-2005
//
// Description  : Ends the specified effect.
//
// Return value : None.
////////////////////////////////////////////////////////////////////////////////

void Asura_Direct3D_Effects_System::FlushEffects (void)
{
	// Do we have a current effect?

	if (NULL != s_pxCurrentEffect)
	{
		SCOPEDPIXEVENT(( D3DCOLOR_XRGB( 0xFF, 0xFF, 0xFF ), L"Flush effect system" ));

		// Yes - end the current effect...

		s_pxCurrentEffect->End	();


		// ...then reset it...

		s_pxCurrentEffect	= NULL;

		// (AAM - 31.08.2006) Textures set using effects will invalidate the Asura render state cache
		Asura_Render::CurrentStates.ResetTextureIndexToLast(0);
	}
}

/// End of function 'Asura_Direct3D_Effects_System::FlushEffects'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Direct3D_Effects_System::SetSharedEffect
// Author       : Anthony Gregson
// Date         : 10-June-2005
//
// Description  : Sets the pointer for the shared effect.
//
// Return value : None.
////////////////////////////////////////////////////////////////////////////////

void Asura_Direct3D_Effects_System::SetSharedEffect(Asura_Direct3D_Effect* pxEffect)
{
	Asura_Assert(!s_pxSharedEffect || !pxEffect, "Shared effect already set");
	s_pxSharedEffect = pxEffect;
	if(s_pxSharedEffect)
	{
		s_xSharedParameterHandles.Get(pxEffect->GetD3DEffect());
	}
	else
	{
		s_xSharedParameterHandles.Reset();
	}
}

/// End of function 'Asura_Direct3D_Effects_System::SetSharedEffect'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Direct3D_Effects_System::SharedParameters::Reset
// Author       : Richard Fine
// Date         : 10-August-2006
//
// Description  : Initializes all the shared parameter handles to NULL.
//
// Return value : None.
////////////////////////////////////////////////////////////////////////////////

void Asura_Direct3D_Effects_System::SharedParameters::Reset()
{
	// This is safe because SharedParameter is a POD type. If anyone adds any virtual functions or anything then it will need to be replaced.
	memset(this, 0, sizeof(SharedParameters));
}

/// End of function 'Asura_Direct3D_Effects_System::SharedParameters::Reset'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Direct3D_Effects_System::SharedParameters::Get
// Author       : Richard Fine
// Date         : 10-August-2006
//
// Description  : Retrieves all the shared parameter handles from the effect.
//
// Return value : None.
////////////////////////////////////////////////////////////////////////////////

void Asura_Direct3D_Effects_System::SharedParameters::Get(Direct3D_Effect* pxEffect)
{
	Asura_Assert(pxEffect, "Null shared effect!");
	if(!pxEffect) return;

	// Misc parameters
	s_xSharedParameterHandles.m_xViewProjectionMatrix = pxEffect->GetParameterByName( NULL, "g_xViewProjectionMatrix" );
	s_xSharedParameterHandles.m_xCurrentTime		  = pxEffect->GetParameterByName( NULL, "g_fCurrentTime" );
	s_xSharedParameterHandles.m_xCameraPosition		  = pxEffect->GetParameterByName( NULL, "g_xCameraPosition" );
#ifdef ASURA_TOOLS
	s_xSharedParameterHandles.m_xDisableLighting	  = pxEffect->GetParameterByName( NULL, "g_bDisableLighting" );
	s_xSharedParameterHandles.m_xShowPerPixelNormals  = pxEffect->GetParameterByName( NULL, "g_bShowPerPixelNormals" );
#endif // #ifdef ASURA_TOOLS

	s_xSharedParameterHandles.m_xUseGammaCorrection   = pxEffect->GetParameterByName( NULL, "g_bUseGammaCorrection" );
}

/// End of function 'Asura_Direct3D_Effects_System::SharedParameters::Reset'. ///

/// End of file. ///
