//*******************************************************************
//
// File: Asura_Direct3D_MaterialResponse.h
//
// Created: 02 March 2007 DLH
//
// Description: 
//
//*******************************************************************

#ifndef __ASURA_DIRECT3D_MATERIALRESPONSE_H__
#define __ASURA_DIRECT3D_MATERIALRESPONSE_H__

#if defined( ASURA_USE_MATERIAL_RESPONSE_SYSTEM)

// other libraries
#include "Rendering/Asura_Direct3D.h"

// internal dependencies
#include "Core/Asura_MaterialResponse_System.h"
#include "Core/Asura_FixedArray.h"
#include "Core/Asura_HashTree.h"

// forward declarations
class Asura_Animation;
class Asura_Direct3D_EffectParameterBlock;
class Asura_MaterialResponse;

#ifndef ASURA_PC_MATERIALS_TEXTURE_HEIGHT 

#ifdef ASURA_TOOLS
#define ASURA_PC_MATERIALS_TEXTURE_HEIGHT (2048)
#else // #ifdef ASURA_TOOLS
#define ASURA_PC_MATERIALS_TEXTURE_HEIGHT (1024)
#endif // #ifdef ASURA_TOOLS

#endif // #ifndef ASURA_PC_MATERIALS_TEXTURE_HEIGHT 

//*******************************************************************
// Class Definitions
//*******************************************************************

class Asura_Direct3D_MRPShader
{
public:

	Asura_Direct3D_MRPShader( const void* const pShader );
	~Asura_Direct3D_MRPShader();

	Direct3D_PixelShader* GetShader() { return m_pxShader; }
	int GetSamplerRegister( const u_int uTexture ) const { return m_aiSamplers[ uTexture ]; }

protected:

	Direct3D_PixelShader* m_pxShader;

	Asura_FixedArray< int, Asura_MaterialResponse::NUMBER_OF_TEXTURE_SLOTS * 2 > m_aiSamplers;

private:

	// Prevent copy construction and assignment
	Asura_Direct3D_MRPShader( const Asura_Direct3D_MRPShader& xOther );
	Asura_Direct3D_MRPShader& operator=( const Asura_Direct3D_MRPShader& xOther );
};

class Asura_Direct3D_MaterialResponse
{
public:
	static void Initialise();
	static void Reset();
	static void Shutdown();
	
	static Direct3D_Texture* GetMaterialsTexture( const bool bUpdate = true);
	static void SetMaterialsTextureValid( const bool bValid);
	
	static float GetMaterialsTextureScale() { return( s_fMaterialsTextureScale);}
	static D3DFORMAT GetMaterialsTextureFormat() { return( s_eMaterialsTextureFormat);}

	static void SetBlendStates( const Asura_MaterialResponse* const pxMaterial );

	static void SetEffectParameters( const Asura_MaterialResponse* const pxMaterial1,
		const Asura_MaterialResponse* const pxMaterial2, const Asura_Animation* const pxAnim,
		Asura_Direct3D_Effect& xEffect );

	static void SetSHParameters( const Asura_MaterialResponse* const pxMaterial,
								 Asura_Direct3D_Effect& xEffect );

	static void SetFogParameters(bool bIsForwardPass, const Asura_MaterialResponse* const pxMaterial, Asura_Direct3D_Effect& xEffect );

	static Asura_Direct3D_MRPShader* AddPixelShader( const Asura_Hash_ID uHash, const void* const pData );
	static Asura_Direct3D_MRPShader* ReplacePixelShader( const Asura_Hash_ID uHash, const void* const pData );
	static Asura_Direct3D_MRPShader* GetPixelShader( const Asura_Hash_ID uHash );

	static void SetMaterialConstants( const Asura_Hash_ID uPSHash,
		const Asura_MaterialResponse* const pxMaterial1,
		const Asura_MaterialResponse* const pxMaterial2, const Asura_Animation* const pxAnim );

private:
	static void MakeTexture();

private:
	static bool s_bMaterialsTextureValid;	
	static Direct3D_Texture* s_pxMaterialsTexture;
	static D3DFORMAT s_eMaterialsTextureFormat;
	static float s_fMaterialsTextureScale;

	static Asura_TemplateHashTree< Asura_Direct3D_MRPShader* > s_xPSTree;
};

class Asura_Direct3D_MaterialResponse_ShaderData
{
public:
	Asura_Direct3D_MaterialResponse_ShaderData();
	
	void Clear();
	void Clear_AllDual();
	void Clear_AllUseMaps();
	
	bool AddDataToParameterBlock( Asura_Direct3D_EffectParameterBlock* pxParameterBlock);
	bool SetParameterByName( Asura_Direct3D_Effect* pxD3DEffect) const;
	
	bool SetupCommon();
	bool Setup( const Asura_MaterialResponse* pxMaterialResponse1, const Asura_MaterialResponse* pxMaterialResponse2, const Asura_TextureAnimationController* pxTextureAnimationController);
	bool Setup_SingleMapsOnly( const Asura_MaterialResponse* pxMaterialResponse, const Asura_TextureAnimationController* pxTextureAnimationController);
	
private:
	bool Setup_TextureAnimation( const Asura_MaterialResponse* pxMaterialResponse1, const Asura_MaterialResponse* pxMaterialResponse2);
	bool Setup_SelfIlluminating( const Asura_MaterialResponse* pxMaterialResponse, const Asura_TextureAnimationController* pxTextureAnimationController);
	
public:

	Direct3D_BaseTexture* m_pxAlbedoMap;
	Direct3D_BaseTexture* m_pxNormalMap;
	Direct3D_BaseTexture* m_pxSpecularMap;
	Direct3D_BaseTexture* m_pxTransparencyMap;
	Direct3D_BaseTexture* m_pxSelfIlluminationMap;
	Direct3D_BaseTexture* m_pxEnvironmentMap;
	Direct3D_BaseTexture* m_pxReflectivityMask;

	Direct3D_BaseTexture* m_pxAlbedoMap2;
	Direct3D_BaseTexture* m_pxNormalMap2;
	Direct3D_BaseTexture* m_pxSpecularMap2;
	Direct3D_BaseTexture* m_pxTransparencyMap2;
	Direct3D_BaseTexture* m_pxSelfIlluminationMap2;

	Direct3D_BaseTexture* m_pxMaterialsTexture;

	float m_afMaterialProperties[4];
	float m_afMaterialsTextureProperties[2];

	bool m_bUseAlbedo;
	bool m_bUseNormalMap;
	bool m_bUseSpecularMap;
	bool m_bUseTransparencyMap;
	bool m_bUseSelfIlluminationMap;
	bool m_bUseEnvironmentMap;
	bool m_bUseReflectivityMask;

	bool m_bDualAlbedo;
	bool m_bDualNormalMap;
	bool m_bDualSpecularMap;
	bool m_bDualTransparencyMap;
	bool m_bDualSelfIlluminationMap;

	float m_afUV_Offsets[4];
	
	D3DXVECTOR4 m_xSelfIlluminationParameters;

	float m_afAlbedoModifier[3];

	bool m_bBlendIntoAlbedo;
};

class Asura_Direct3D_MaterialResponse_ShaderData_Env
{
public:
	Asura_Direct3D_MaterialResponse_ShaderData_Env();
	
	void Clear();
	
	bool AddDataToParameterBlock( Asura_Direct3D_EffectParameterBlock* pxParameterBlock);
	bool SetParameterByName( Asura_Direct3D_Effect* pxD3DEffect) const;
	
	bool Setup( const Asura_MaterialResponse* pxMaterialResponse1, const Asura_MaterialResponse* pxMaterialResponse2);
	
public:
	D3DXVECTOR4 m_xPrelightingColourVector;
};

class Asura_Direct3D_MaterialResponse_ShaderData_Parallax
{
public:
	Asura_Direct3D_MaterialResponse_ShaderData_Parallax();
	
	void Clear();
	
	bool AddDataToParameterBlock( Asura_Direct3D_EffectParameterBlock* pxParameterBlock);
	bool SetParameterByName( Asura_Direct3D_Effect* pxD3DEffect) const;
	
	bool Setup( const Asura_MaterialResponse* pxMaterialResponse1, const Asura_MaterialResponse* pxMaterialResponse2);
	
public:
	Direct3D_BaseTexture* m_pxParallaxMap;
	Direct3D_BaseTexture* m_pxParallaxMap2;

	bool m_bUseParallaxMap;
	bool m_bDualParallaxMap;
};

//*******************************************************************
//
// (c) 2006 Rebellion
//
// Warning: This computer program is protected by copyright law and 
// international treaties. Unauthorized reproduction or distribution
// of this program, or any portion of it, may result in severe civil and
// criminal penalties, and will be prosecuted to the maximum extent 
// possible under the law.
//
//*******************************************************************

#endif //defined( ASURA_USE_MATERIAL_RESPONSE_SYSTEM)

#endif // __ASURA_DIRECT3D_MATERIALRESPONSE_H__
