//
// Asura_Direct3D.cpp
//
// Created by Kevin Lea - 09/08/2004
// 
// Copyright (C) 2004 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

#include "Camera/Asura_Camera.h"
#include "Core/Asura_TextureManagement.h"
#include "Rendering/Asura_Direct3D.h"
#include "Rendering/Asura_Direct3D_Caps.h"
#include "Rendering/Asura_Direct3D_LineRendering.h"
#include "Rendering/Asura_Direct3D_Shaders.h"
#include "Rendering/Asura_Direct3D_StoreRenderTargets.h"
#include "Rendering/Asura_Render.h"

#include "Entity/Glass//Asura_WorldSpace_Rectangle.h"
#include "Light/Asura_Photon.h"


// KJL - Monday January 22 2001 15:55:24
// 
// Quick line drawing fn; added for visual representation of waypoint connections
// Will probably evolve into something with a nicer interface and more options


#define D3DFVF_LINEVERTEX (D3DFVF_XYZ|D3DFVF_DIFFUSE)
struct D3DLineVertex
{
	float X,Y,Z;
	unsigned int Diffuse;
};

void Asura_Render::Platform_RenderLine(const Asura_Vector_3& xP0, const Asura_Vector_3& xP1, unsigned int uColour)
{
	if (Asura_Render::GetEnableBatchedLineRendering())
	{	
		Asura_Direct3D_LineRenderer::Line3D xLine( xP0, xP1, uColour );
		Asura_Direct3D_LineRenderer::AddLine( xLine ); 
	}
	else	
	{	
		D3DLineVertex vertices[2];

		vertices[0].X = xP0.x;
		vertices[0].Y = xP0.y;
		vertices[0].Z = xP0.z;
		vertices[0].Diffuse = uColour;

		vertices[1].X = xP1.x;
		vertices[1].Y = xP1.y;
		vertices[1].Z = xP1.z;
		vertices[1].Diffuse = uColour;

		Asura_Direct3D_Shaders::RequestFixedFunctionVertexShader( D3DFVF_LINEVERTEX );
		Asura_Render::CurrentStates.RequestTextureIndex( 0,ASURA_TEXTURE_NONE );
		Asura_Render::CurrentStates.RequestTextureOpMode( ASURA_TEXTUREOP_SINGLE_PASS );
		Asura_Render::CurrentStates.RequestTranslucencyMode( ASURA_TRANSLUCENCY_NORMAL );
		

		Asura_Direct3D::D3DDevice->DrawPrimitiveUP
		(
			D3DPT_LINELIST,
			1,
			(void*)vertices,
			sizeof(D3DLineVertex)
		);
	}
}

static D3DLineVertex g_axOutlineVertices[5];

static void RenderOutlineInternal( int iNumPoints, bool bWireframe, bool bBackFaceCull )
{
	SCOPEDPIXEVENT(( 0xFF008000, L"%S outline (%d points)", bWireframe ? "Wireframe" : "Solid", iNumPoints ));

#if defined( ASURA_TARGET_SM3)
	// DLH - 14 February 2007 12:33:53 - Store and Clear all the RenderTargets after the first one
	Asura_Direct3D_StoreRenderTargets xRenderTargetStore;
	
	xRenderTargetStore.StoreRenderTargets();
	Asura_Direct3D::ClearRenderTargets( 1);
#endif

	Asura_Assert( iNumPoints == 3 || iNumPoints == 4, "invalid shape" );

	Asura_Direct3D_Shaders::RequestFixedFunctionVertexShader(D3DFVF_LINEVERTEX);

	Asura_Render::CurrentStates.RequestTextureIndex(0,ASURA_TEXTURE_NONE);
	Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_NORMAL);
	Asura_Render::CurrentStates.RequestTextureOpMode(ASURA_TEXTUREOP_COLOUR);
	Asura_Render::CurrentStates.RequestZBufferEnabled( true );

	const void* pvVertexData = reinterpret_cast<const void*>(g_axOutlineVertices);
	if( bWireframe )
	{
		// Need to complete loop.
		g_axOutlineVertices[iNumPoints].X       = g_axOutlineVertices[0].X;
		g_axOutlineVertices[iNumPoints].Y       = g_axOutlineVertices[0].Y;
		g_axOutlineVertices[iNumPoints].Z       = g_axOutlineVertices[0].Z;
		g_axOutlineVertices[iNumPoints].Diffuse = g_axOutlineVertices[0].Diffuse;

		Asura_Direct3D::D3DDevice->DrawPrimitiveUP( D3DPT_LINESTRIP, iNumPoints, pvVertexData, sizeof(D3DLineVertex) );
	}
	else
	{
		u_int uNumPrims = ( 4 == iNumPoints ? 2 : 1 );
        Asura_Render::CurrentStates.RequestCullMode( bBackFaceCull ? ASURA_CULLMODE_CW : ASURA_CULLMODE_NONE );
		Asura_Direct3D::D3DDevice->DrawPrimitiveUP( D3DPT_TRIANGLEFAN, uNumPrims, pvVertexData, sizeof(D3DLineVertex) );
		Asura_Render::CurrentStates.RequestCullMode(ASURA_CULLMODE_CCW);
	}
	
#if defined( ASURA_TARGET_SM3)
	xRenderTargetStore.RestoreRenderTargets();
#endif
}

void RenderOutline( const Asura_Vector_3* pxPoints, int iNumPoints, const u_int* puColours, bool bWireframe )
{
	Asura_Assert( iNumPoints == 3 || iNumPoints == 4, "invalid shape" );

	for( int i = 0; i < iNumPoints; ++i )
	{
		g_axOutlineVertices[i].X       = pxPoints[i].x;
		g_axOutlineVertices[i].Y       = pxPoints[i].y;
		g_axOutlineVertices[i].Z       = pxPoints[i].z;
		g_axOutlineVertices[i].Diffuse = puColours[i];
	}

	RenderOutlineInternal( iNumPoints, bWireframe, true );
}


void RenderOutline( const Asura_Vector_3* pxPoints, int iNumPoints, u_int uColour, bool bWireframe, bool bBackFaceCull )
{
	Asura_Assert( iNumPoints == 3 || iNumPoints == 4, "invalid shape" );

	for( int i = 0; i < iNumPoints; ++i )
	{
		g_axOutlineVertices[i].X       = pxPoints[i].x;
		g_axOutlineVertices[i].Y       = pxPoints[i].y;
		g_axOutlineVertices[i].Z       = pxPoints[i].z;
		g_axOutlineVertices[i].Diffuse = uColour;
	}

	RenderOutlineInternal( iNumPoints, bWireframe, bBackFaceCull );
}


void RenderOutline( const Asura_Vector_3* pxPoints, int iNumPoints, u_int uColour, bool bWireframe )
{
    RenderOutline( pxPoints, iNumPoints, uColour, bWireframe, true );
}

#define D3DFVF_2DLINEVERTEX (D3DFVF_XYZRHW|D3DFVF_DIFFUSE)
struct D3D2DLineVertex
{
	float X,Y,Z,RHW;
	unsigned int Diffuse;
};

void Asura_Render::Platform_Render2DLine( const Asura_Vector_2& xP0, const Asura_Vector_2& xP1, u_int uColour)
{
	Asura_Direct3D_LineRenderer::Line2D xLine( xP0, xP1, uColour );
	Asura_Direct3D_LineRenderer::AddLine( xLine ); 
}

// KJL - Monday January 22 2001 15:54:16
// 
// Quick and hacky icon drawer - point sprites are a new feature in DX8 but current hardware
// doesn't support them in TnL. Need some time to think over a nice way of drawing lots of particles...
void RenderIcon( const Asura_Vector_3& position, int icon, float size, u_int uColour )
{
	SCOPEDPIXEVENT(( 0xFFFFFFFF, L"Render Icon @ (%.2f, %.2f, %.2f)",
		position.x, position.y, position.z ));

	Asura_Vector_3 vertex[4];

	Asura_Direct3D_Shaders::RequestFixedFunctionVertexShader(D3DFVF_GENERICVERTEX);

	Asura_Render::CurrentStates.RequestTextureOpMode(ASURA_TEXTUREOP_SINGLE_PASS);
	Asura_Render::CurrentStates.RequestCullMode(ASURA_CULLMODE_CCW);



	const Asura_Vector_3* cameraPosition = Asura_Camera::GetPosition();
	const Asura_Matrix_3x3* cameraOrientation = Asura_Camera::GetOrientation();

	{
		Asura_Vector_3 disp = position - *cameraPosition;
		float dist = disp.Magnitude();

		if (dist>20.0f) size *= dist/20.0f;

	}

	//if ((Asura_Maths::FloatToInt(Asura_Timers::GetLocalUnscaledTime()*2.0f))&1) colour = 0xffff0000;

	//colour &= ((Asura_Maths::FloatToInt(Asura_Timers::GetLocalUnscaledTime()*255.0f))<<24)+0xffffff;

	Asura_Vector_3 deltaX = Asura_Vector_3(cameraOrientation->e11,cameraOrientation->e21,cameraOrientation->e31)*size;
	Asura_Vector_3 deltaY = Asura_Vector_3(cameraOrientation->e12,cameraOrientation->e22,cameraOrientation->e32)*size;

	vertex[0] = position - deltaX - deltaY; 
	vertex[1] = position + deltaX - deltaY;
	vertex[2] = position + deltaX + deltaY;
	vertex[3] = position - deltaX + deltaY;

	{
		D3DGenericVertex v[4];

		v[0].U = 0.0f;
		v[0].V = 0.0f;

		v[1].U = 1.0f;
		v[1].V = 0.0f;

		v[2].U = 1.0f;
		v[2].V = 1.0f;

		v[3].U = 0.0f;
		v[3].V = 1.0f;

		for (int i=0; i<4; i++)
		{
			v[i].X = vertex[i].x;
			v[i].Y = vertex[i].y;
			v[i].Z = vertex[i].z;

			v[i].Diffuse = uColour;
		}

		Asura_Render::CurrentStates.RequestTextureIndex(0,icon);

		if (01)//icon!=398&&icon!=409)
		{
			Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_NORMAL);
		}
		else
		{

			Asura_Render::CurrentStates.RequestZBufferWriteEnabled(false);
			Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_GLOWING);
		}


		Asura_Direct3D::D3DDevice->DrawPrimitiveUP(D3DPT_TRIANGLEFAN, 2, (LPVOID)v, sizeof(D3DGenericVertex) );

	}	


}
void RenderIcon( const Asura_Vector_3& position, int icon, float size )
{
	RenderIcon(position,icon,size,0xffffffff);
}


#include "Light/Asura_Light.h"



#define SPHERE_VERTEX_DESC (D3DFVF_XYZ|D3DFVF_DIFFUSE|D3DFVF_TEX1) 
#include "SpecialFX/Asura_Sphere.h"
int FogTexture = 0;

void Render_Sphere_Main( const Asura_Vector_3& position, float radius, const Asura_Vector_3* direction, float angle, int colour )
{
	SCOPEDPIXEVENT((0xffffffff, L"Render Sphere"));

	Asura_Direct3D_Shaders::RequestFixedFunctionVertexShader(SPHERE_VERTEX_DESC);

	if (!FogTexture)
	{
		FogTexture = Asura_Texture_Management::AddTextureWithResetGroup("\\tools\\fog.bmp", ASURA_TEXTURE_LOADED_PERMANENT );
		Asura_Texture_Management::LoadTextures();
	}


	Asura_Render::CurrentStates.RequestCullMode(ASURA_CULLMODE_NONE);

	Asura_Render::CurrentStates.RequestTextureOpMode(ASURA_TEXTUREOP_TOON);
	Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_NORMAL);


	Asura_Render::CurrentStates.RequestTextureOpMode(ASURA_TEXTUREOP_SINGLE_PASS);
	Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_GLOWING);

	Asura_Render::CurrentStates.RequestTextureIndex(0,FogTexture);
	Asura_Render::CurrentStates.RequestZBufferEnabled(true);
	Asura_Render::CurrentStates.RequestZBufferWriteEnabled(false);

	D3DXMATRIX matWorld;

	matWorld._12 = matWorld._13 = matWorld._14 = matWorld._21 = matWorld._23 = matWorld._24 = 0.0f;
	matWorld._31 = matWorld._32 = matWorld._34 = matWorld._41 = matWorld._42 = matWorld._43 = 0.0f;
	matWorld._11 = matWorld._22 = matWorld._33 = radius;
	matWorld._44 = 1.0f;
	matWorld._41 = position.x;
	matWorld._42 = position.y;
	matWorld._43 = position.z;

	Asura_Direct3D::D3DDevice->SetTransform( D3DTS_WORLD,      &matWorld );

	if (angle == 360.0f || direction == 0)
	{
		for (int i=0; i<SPHERE_VERTICES; i++)
		{
			SphereVertex[i].Colour = colour;
		}
	}
	else
	{
		for (int i=0; i<SPHERE_VERTICES; i++)
		{
			float dotangle = (direction->x*SphereVertex[i].x + direction->y*SphereVertex[i].y + direction->z*SphereVertex[i].z
				- Asura_Maths::Cosine(Asura_Maths::Pi/360.0f*angle));

			if (dotangle>0)
			{
				int a = (int)(dotangle*256.0f);
				if (a>63) a=63;
				SphereVertex[i].Colour = 0xffffff+(a<<24);
			}
			else
			{
				SphereVertex[i].Colour = 0x1fff0000;
			}
		}
	}

	//Asura_Direct3D::D3DDevice->DrawIndexedPrimitive(D3DPT_TRIANGLELIST, SPHERE_VERTEX_DESC, (LPVOID)SphereVertex, SPHERE_VERTICES, SphereIndices, SPHERE_INDICES, 0);

	//HRESULT h = 
	Asura_Direct3D::D3DDevice->DrawIndexedPrimitiveUP
		(
		D3DPT_TRIANGLELIST,
		0,
		SPHERE_VERTICES,
		SPHERE_INDICES/3,
		(void*)SphereIndices,
		D3DFMT_INDEX16,
		(void*)SphereVertex,
		sizeof(Asura_Sphere_Vertex)
		);


	D3DXMatrixTranslation( &matWorld, 0,0,0 );
	Asura_Direct3D::D3DDevice->SetTransform( D3DTS_WORLD,      &matWorld );
	Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_OFF);
	Asura_Render::CurrentStates.RequestZBufferWriteEnabled(true);


}

void Render_Sphere( const Asura_Vector_3& xCentre, float fRadius, const Asura_Vector_3* pxDir, float fAngle )
{
	Render_Sphere_Main( xCentre, fRadius, pxDir, fAngle, 0x1fffffff );
}

void Render_Sphere( const Asura_Vector_3& xCentre, float fRadius, int iColour )
{
	Render_Sphere_Main( xCentre, fRadius, NULL, 0.0f, iColour );
}

void Asura_Render_Capsule( const Asura_AVector3& xA, const Asura_AVector3& xB, float fRadius, u_int uColour )
{
	SCOPEDPIXEVENT((0xffffffff, L"Render Capsule"));

	Asura_Sphere_Vertex axCapsuleVerts[ SPHERE_VERTICES ];

	memcpy( axCapsuleVerts, SphereVertex, sizeof(Asura_Sphere_Vertex) * SPHERE_VERTICES );

	Asura_Direct3D_Shaders::RequestFixedFunctionVertexShader(SPHERE_VERTEX_DESC);

	Asura_Render::CurrentStates.RequestCullMode(ASURA_CULLMODE_NONE);

	Asura_Render::CurrentStates.RequestTextureOpMode(ASURA_TEXTUREOP_SINGLE_PASS);
	Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_GLOWING);

	Asura_Render::CurrentStates.RequestTextureIndex(0,ASURA_TEXTURE_NONE);
	Asura_Render::CurrentStates.RequestZBufferWriteEnabled(false);

	D3DXMATRIX matWorld;

	matWorld._12 = matWorld._13 = matWorld._14 = matWorld._21 = matWorld._23 = matWorld._24 = 0.0f;
	matWorld._31 = matWorld._32 = matWorld._34 = matWorld._41 = matWorld._42 = matWorld._43 = 0.0f;
	matWorld._11 = matWorld._22 = matWorld._33 = fRadius;
	matWorld._44 = 1.0f;
	matWorld._41 = xA.GetX();
	matWorld._42 = xA.GetY();
	matWorld._43 = xA.GetZ();

	Asura_Direct3D::D3DDevice->SetTransform( D3DTS_WORLD,      &matWorld );

	Asura_AVector3 xDisp = (xB - xA) * (1.0f / fRadius);

	for (int i=0; i<SPHERE_VERTICES; i++)
	{
		axCapsuleVerts[i].Colour = uColour;
		
		float fDotProduct = axCapsuleVerts[i].x * xDisp.GetX() + 
							axCapsuleVerts[i].y * xDisp.GetY() + 
							axCapsuleVerts[i].z * xDisp.GetZ();

		if (fDotProduct > 0)
		{
			axCapsuleVerts[i].x += xDisp.GetX();
			axCapsuleVerts[i].y += xDisp.GetY();
			axCapsuleVerts[i].z += xDisp.GetZ();
		}
	}


	Asura_Direct3D::D3DDevice->DrawIndexedPrimitiveUP
	(
		D3DPT_TRIANGLELIST,
		0,
		SPHERE_VERTICES,
		SPHERE_INDICES/3,
		(void*)SphereIndices,
		D3DFMT_INDEX16,
		(void*)axCapsuleVerts,
		sizeof(Asura_Sphere_Vertex)
	);


	D3DXMatrixTranslation( &matWorld, 0,0,0 );
	Asura_Direct3D::D3DDevice->SetTransform( D3DTS_WORLD,      &matWorld );
	Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_OFF);
	Asura_Render::CurrentStates.RequestZBufferWriteEnabled(true);

	Asura_Render::CurrentStates.RequestCullMode(ASURA_CULLMODE_CW);
}

#define D3DFVF_CUBEVERTEX (D3DFVF_XYZ|D3DFVF_DIFFUSE)
struct D3DCubeVertex
{
	float X,Y,Z;
	unsigned int Diffuse;
};

void Render_Cuboid(const Asura_Bounding_Box& box,const Asura_Vector_3& xPosition,const Asura_Matrix_3x3& xOrient, int colour,bool forceWireframe = false,int iHighlightSide = -1,int iHighlightColour = 0xffff0000)
{
	SCOPEDPIXEVENT((0xffffffff, L"Render Cuboid"));
	
#if defined( ASURA_TARGET_SM3)
	// DLH - 14 February 2007 12:33:53 - Store and Clear all the RenderTargets after the first one
	Asura_Direct3D_StoreRenderTargets xRenderTargetStore;
	
	xRenderTargetStore.StoreRenderTargets();
	Asura_Direct3D::ClearRenderTargets( 1);
#endif

	if(box.MaxX<=box.MinX || box.MaxY<=box.MinY || box.MaxZ<=box.MinZ) return;
	Asura_Direct3D_Shaders::RequestFixedFunctionVertexShader(D3DFVF_CUBEVERTEX);

	D3DCubeVertex vertices[16];
	for(int i=0;i<8;i++)
	{
		vertices[i].X = (i&1) ? box.MaxX : box.MinX;
		vertices[i].Y = (i&2) ? box.MaxY : box.MinY;
		vertices[i].Z = (i&4) ? box.MaxZ : box.MinZ;
		vertices[i].Diffuse = colour;
	}

	int iVertsUsed = 8;
	if(iHighlightSide>=0 && iHighlightSide<6)
	{
		for(int i=8;i<16;i++)
		{
			vertices[i] = vertices[i-8];
			vertices[i].Diffuse = iHighlightColour;
		}
		iVertsUsed = 16;
	}

	Asura_Render::CurrentStates.RequestCullMode(ASURA_CULLMODE_NONE);

	Asura_Render::CurrentStates.RequestTextureOpMode(ASURA_TEXTUREOP_SINGLE_PASS);
	Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_NORMAL);
	Asura_Render::CurrentStates.RequestTextureIndex(0,ASURA_TEXTURE_NONE);




	Asura_Render::CurrentStates.RequestZBufferWriteEnabled(false);

	D3DXMATRIX matWorld;
	Asura_Direct3D::SetupD3DMatrix(matWorld, xOrient, xPosition, true);

	Asura_Direct3D::D3DDevice->SetTransform( D3DTS_WORLD,      &matWorld );

	if(forceWireframe)
	{
		short indices[24] = 
		{
			0,2,2,3,3,1,1,0,
				0,4,2,6,3,7,1,5,
				4,6,6,7,7,5,5,4,
		};

		if(iHighlightSide>=0 && iHighlightSide<6)
		{
			//switch the indices for the highlighted side 

			int iBit = 1<<(iHighlightSide/2);
			bool bMax = (iHighlightSide&1) ? true : false;

			for(int i=0;i<24;i+=2)
			{
				if((bMax && (indices[i] & iBit) && (indices[i+1] & iBit)) ||
					(!bMax && !(indices[i] & iBit) && !(indices[i+1] & iBit)))
				{
					indices[i]+=8;
					indices[i+1]+=8;
				}
			}
		}


		Asura_Direct3D::D3DDevice->DrawIndexedPrimitiveUP
			(
			D3DPT_LINELIST,
			0,
			iVertsUsed,
			12,
			(void*)indices,
			D3DFMT_INDEX16,
			(void*)vertices,
			sizeof(D3DCubeVertex)
			);

	}
	else
	{
		short indices[36] = 
		{
			0,4,6,//min x
				0,6,2,
				3,7,5,//max x
				3,5,1,
				1,5,4,//min y
				1,4,0,
				2,6,7,//max y
				2,7,3,
				0,2,3,//min z
				0,3,1,
				4,6,7,//max z
				4,7,5
		};
		if(iHighlightSide>=0 && iHighlightSide<6)
		{
			//switch the indices for the highlighted side 
			for(int i=iHighlightSide*6;i<iHighlightSide*6+6;i++)
			{
				indices[i]+=8;
			}
		}

		Asura_Direct3D::D3DDevice->DrawIndexedPrimitiveUP
			(
			D3DPT_TRIANGLELIST,
			0,
			iVertsUsed,
			12,
			(void*)indices,
			D3DFMT_INDEX16,
			(void*)vertices,
			sizeof(D3DCubeVertex)
			);
	}

	D3DXMatrixTranslation( &matWorld, 0,0,0 );
	Asura_Direct3D::D3DDevice->SetTransform( D3DTS_WORLD,      &matWorld );
	Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_OFF);
	Asura_Render::CurrentStates.RequestZBufferWriteEnabled(true);

#if defined( ASURA_TARGET_SM3)
	xRenderTargetStore.RestoreRenderTargets();
#endif
}

void Render_Rectangle( const Asura_WorldSpace_Rectangle& xRect, u_int uColour, int iTexIdx, const float* pfUs, const float* pfVs, bool bWireframe )
{
	SCOPEDPIXEVENT((0xffffffff, L"Render Rectangle"));

	if (xRect.Dimensions.x <= 0.0f || xRect.Dimensions.y <= 0.0f) return;

	Asura_Vector_3 Vertices[4];
	Vertices[0] = xRect.CornerPosition;
	Vertices[1] = xRect.CornerPosition + (Asura_Vector_3(xRect.Dimensions.x, 0.0f, 0.0f) * xRect.Orientation);
	Vertices[2] = xRect.CornerPosition + (Asura_Vector_3(0.0f, xRect.Dimensions.y, 0.0f) * xRect.Orientation);
	Vertices[3] = xRect.CornerPosition + (Asura_Vector_3(xRect.Dimensions.x, xRect.Dimensions.y, 0.0f) * xRect.Orientation);


	D3DGenericVertex v[4];

	for (int i=0;i<4;i++)
	{
		v[i].X = Vertices[i].x;
		v[i].Y = Vertices[i].y;
		v[i].Z = Vertices[i].z;
		v[i].Diffuse = uColour;
	}

	if( ASURA_TEXTURE_NONE != iTexIdx )
	{
		for (int i=0;i<4;i++)
		{
			Asura_Assert( pfUs && pfVs, "" );
			v[i].U = pfUs[i];
			v[i].V = pfVs[i];
		}
	}

	Asura_Direct3D_Shaders::RequestFixedFunctionVertexShader(D3DFVF_GENERICVERTEX);
	Asura_Render::CurrentStates.RequestCullMode(ASURA_CULLMODE_NONE);
	Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_NORMAL);
	Asura_Render::CurrentStates.RequestTextureOpMode(ASURA_TEXTUREOP_SINGLE_PASS);
	Asura_Render::CurrentStates.RequestTextureIndex( 0, iTexIdx );

	Asura_Render::CurrentStates.RequestZBufferWriteEnabled(false);

	D3DXMATRIX matWorld;

	matWorld._12 = matWorld._13 = matWorld._14 = matWorld._21 = matWorld._23 = matWorld._24 = 0.0f;
	matWorld._31 = matWorld._32 = matWorld._34 = matWorld._41 = matWorld._42 = matWorld._43 = 0.0f;
	matWorld._11 = matWorld._22 = matWorld._33 = 1.0f;
	matWorld._44 = 1.0f;
	matWorld._41 = 0.0f;
	matWorld._42 = 0.0f;
	matWorld._43 = 0.0f;

	Asura_Direct3D::D3DDevice->SetTransform( D3DTS_WORLD,      &matWorld );


	if( bWireframe )
	{
		short indices[8] = 
		{
			0,2,2,3,3,1,1,0,
		};
		Asura_Direct3D::D3DDevice->DrawIndexedPrimitiveUP
			(
			D3DPT_LINELIST,
			0,
			4,
			4,
			(void*)indices,
			D3DFMT_INDEX16,
			(void*)v,
			sizeof(D3DGenericVertex)
			);
	}
	else
	{
		short indices[12] = 
		{
			0,1,2,
				0,2,1,
				1,2,3,
				1,3,2,
		};
		Asura_Direct3D::D3DDevice->DrawIndexedPrimitiveUP
			(
			D3DPT_TRIANGLELIST,
			0,
			4,
			4,
			(void*)indices,
			D3DFMT_INDEX16,
			(void*)v,
			sizeof(D3DGenericVertex)
			);
	}

	D3DXMatrixTranslation( &matWorld, 0,0,0 );
	Asura_Direct3D::D3DDevice->SetTransform( D3DTS_WORLD,      &matWorld );
	Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_OFF);
	Asura_Render::CurrentStates.RequestZBufferWriteEnabled(true);
	Asura_Render::AddToTrianglesRendered(4);	
}

// JAT 07/04/07 - debug render world space untextured triangle - copied from Render_Rectangle
void Render_DebugTriangle( const Asura_AVector3& xV0, const Asura_AVector3& xV1, const Asura_AVector3& xV2,
						   const u_int uColour0, const u_int uColour1, u_int uColour2, 
						   const bool bWireframe )
{
	SCOPEDPIXEVENT((0xffffffff, L"Render DebugTriangle"));

	D3DGenericVertex xVerts[3];

	xVerts[0].X = xV0.GetX();
	xVerts[0].Y = xV0.GetY();
	xVerts[0].Z = xV0.GetZ();
	xVerts[0].Diffuse = uColour0;

	xVerts[1].X = xV1.GetX();
	xVerts[1].Y = xV1.GetY();
	xVerts[1].Z = xV1.GetZ();
	xVerts[1].Diffuse = uColour1;

	xVerts[2].X = xV2.GetX();
	xVerts[2].Y = xV2.GetY();
	xVerts[2].Z = xV2.GetZ();
	xVerts[2].Diffuse = uColour2;

	Asura_Direct3D_Shaders::RequestFixedFunctionVertexShader(D3DFVF_GENERICVERTEX);
	Asura_Render::CurrentStates.RequestCullMode(ASURA_CULLMODE_NONE);
	Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_NORMAL);
	Asura_Render::CurrentStates.RequestTextureOpMode(ASURA_TEXTUREOP_SINGLE_PASS);
	Asura_Render::CurrentStates.RequestTextureIndex( 0, ASURA_TEXTURE_NONE );

	Asura_Render::CurrentStates.RequestZBufferWriteEnabled(false);

	D3DXMATRIX xMatWorld;

	xMatWorld._12 = xMatWorld._13 = xMatWorld._14 = xMatWorld._21 = xMatWorld._23 = xMatWorld._24 = 0.0f;
	xMatWorld._31 = xMatWorld._32 = xMatWorld._34 = xMatWorld._41 = xMatWorld._42 = xMatWorld._43 = 0.0f;
	xMatWorld._11 = xMatWorld._22 = xMatWorld._33 = 1.0f;
	xMatWorld._44 = 1.0f;
	xMatWorld._41 = 0.0f;
	xMatWorld._42 = 0.0f;
	xMatWorld._43 = 0.0f;

	Asura_Direct3D::D3DDevice->SetTransform( D3DTS_WORLD, &xMatWorld );

	if ( bWireframe )
	{
		const short sIndices[4] = { 
									0, 1, 2, 0
								  };
		Asura_Direct3D::D3DDevice->DrawIndexedPrimitiveUP(
								D3DPT_LINESTRIP,
								0,
								3,
								3,
								(void*)sIndices,
								D3DFMT_INDEX16,
								(void*)xVerts,
								sizeof(D3DGenericVertex)
														 );
	}
	else
	{
		const short sIndices[3] = { 
									0,1,2
								  };
		Asura_Direct3D::D3DDevice->DrawIndexedPrimitiveUP(
								D3DPT_TRIANGLELIST,
								0,
								3,
								1,
								(void*)sIndices,
								D3DFMT_INDEX16,
								(void*)xVerts,
								sizeof(D3DGenericVertex)
														 );
	}

	D3DXMatrixTranslation( &xMatWorld, 0,0,0 );
	Asura_Direct3D::D3DDevice->SetTransform( D3DTS_WORLD, &xMatWorld );
	Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_OFF);
	Asura_Render::CurrentStates.RequestZBufferWriteEnabled(true);
	Asura_Render::AddToTrianglesRendered(1);
}

#ifdef ASURA_TOOLS
#include "Mesh/Asura_Mesh.h"
extern void InitialiseRMVB();
extern Direct3D_IndexBuffer* RenderMeshIndexBuffer;
extern Direct3D_VertexBuffer* RenderMeshVertexBuffer;
void RenderAnimationMesh_GraphicNovel(class Asura_Mesh* mesh,  const class Asura_Vector_3& position)
{
//	return;
	
	Asura_Direct3D_Shaders::RequestFixedFunctionVertexShader(D3DFVF_GENERICVERTEX);

	int totalIndices = mesh->NumberOfTriangles*3;
	Asura_Render::CurrentStates.RequestAlphaTest(false);
	
    //Asura_Vector_3 light(451.3f,108.0f,42.4f);
//	Asura_Vector_3 light(457.1f,106.9f,42.4f);
	InitialiseRMVB();

	Asura_Bounding_Box bb;
	bb.Reset();
	bb.Set(position);

	Asura_Vector_3 xBBCentre = bb.CalculateCentre();
	Asura_Photon_System::MakePhotonListForBoundingBox(bb, xBBCentre, true, true);

	Asura_Vector_3 light;
	Asura_Photon_System::CalculateAverageLocalLightPos(light);


	int numberOfVertices = mesh->NumberOfTriangles*3;
	int indexNumber=0;

//	const Asura_Photon_Sampling_Result* pxResult = 
	Asura_Photon_System::SamplePhotonList(position);
	
	Asura_Vector_3 direction = light - position;
	direction.Normalise();


	Asura_Vector_3 xSH[9];

	for (u_int u=0; u<9; u++)
	{
		xSH[u].Zero();
	}

	float fConst1 = 0.235294465f;
	float fConst2 = 0.470589175497f;
	float fConst3 = 0.882352245676f;
	float fConst4 = 0.073529614476f;
	float fConst5 = 0.220588061419f;

	for (u_int p=0; p<Asura_Photon_System::NumberOfLocalPhotons; p++)
	{
		Asura_Vector_3 xColour;
		xColour.x = Asura_Photon_System::LocalPhotons[p].GetRedComponent();
		xColour.y = Asura_Photon_System::LocalPhotons[p].GetGreenComponent();
		xColour.z = Asura_Photon_System::LocalPhotons[p].GetBlueComponent();

		Asura_Vector_3 xDirection = Asura_Photon_System::LocalPhotons[p].GetPosition() - position;
		xDirection.Normalise();


		xSH[0] += xColour * fConst1;
		xSH[1] += xColour * fConst2 * xDirection.x;
		xSH[2] += xColour * fConst2 * xDirection.y;
		xSH[3] += xColour * fConst2 * xDirection.z;
		xSH[4] += xColour * fConst3 * (xDirection.x*xDirection.z);
		xSH[5] += xColour * fConst3 * (xDirection.z*xDirection.y);
		xSH[6] += xColour * fConst3 * (xDirection.y*xDirection.x);
		xSH[7] += xColour * fConst4 * (3.0f*xDirection.z*xDirection.z-1.0f);
		xSH[8] += xColour * fConst5 * (xDirection.x*xDirection.x - xDirection.y*xDirection.y);
	}

	{
		D3DGenericVertex* vertexData;
		unsigned short* indexData;
	
		RenderMeshIndexBuffer->Lock(0,sizeof(unsigned short)*totalIndices,(void**)&indexData,D3DLOCK_DISCARD);

		RenderMeshVertexBuffer->Lock(0,sizeof(D3DGenericVertex)*numberOfVertices,(void**)&vertexData,D3DLOCK_DISCARD);

		int i=0;
	
		for (int p=0; p<mesh->NumberOfPolygons; p++)
		{	
			int v=0;
	
		
			do
			{
			
				int vID =  mesh->PolygonDataPtr[p].VertexID[v];
//mesh->VertexDataPtr[vID].Position.Normalise();
				Asura_Vector_3 pos = mesh->VertexDataPtr[vID].Position+position;
			//	pos.y-=1.0f;
				Asura_Vector_3 n = mesh->NormalsDataPtr[p].VertexNormal[v];

				vertexData->X = pos.x;
				vertexData->Y = pos.y;
				vertexData->Z = pos.z;
				vertexData->Normal = n;
				vertexData->U = mesh->TextureDataPtr[p].U[0][v];
				vertexData->V = mesh->TextureDataPtr[p].V[0][v];
			
#if 0
				float scale = 255.0f * (n*direction);	 
				if (scale<0.0f) scale = 0.0f;

				unsigned int r = unsigned int (pxResult->m_fDiffuseR*scale);
				if (r>255) r = 255;

				unsigned int g = unsigned int (pxResult->m_fDiffuseG*scale);
				if (g>255) g = 255;

				unsigned int b = unsigned int (pxResult->m_fDiffuseB*scale);
				if (b>255) b = 255;

				vertexData->Diffuse = (r<<16) + (g<<8) + (b) + 0xff000000;
#else
				Asura_Vector_3 xColour = xSH[0];
				xColour += xSH[1] * n.x;
				xColour += xSH[2] * n.y;
				xColour += xSH[3] * n.z;
				xColour += xSH[4] * (n.x*n.z);
				xColour += xSH[5] * (n.z*n.y);
				xColour += xSH[6] * (n.y*n.x);
				xColour += xSH[7] * (3.0f* n.z*n.z - 1.0f);
				xColour += xSH[8] * (n.x*n.x - n.y*n.y);
			
				int r = int (xColour.x * 128.0f);
				if (r>255) r = 255;
				if (r<0) r = 0;

				int g = int (xColour.y * 128.0f);
				if (g>255) g = 255;
				if (g<0) g = 0;

				int b = int (xColour.z * 128.0f);
				if (b>255) b = 255;
				if (b<0) b = 0;

				vertexData->Diffuse = (r<<16) + (g<<8) + (b) + 0xff000000;
				
#endif
								
				vertexData++;
				v++;
			}
			while(v<4 && mesh->PolygonDataPtr[p].VertexID[v]!=0xffff);

			indexData[i++] = (unsigned short)(indexNumber+0);
			indexData[i++] = (unsigned short)(indexNumber+1);
			indexData[i++] = (unsigned short)(indexNumber+2);
			if (mesh->PolygonDataPtr[p].IsQuad())
			{
				indexData[i++] = (unsigned short)(indexNumber+2);
				indexData[i++] = (unsigned short)(indexNumber+3);
				indexData[i++] = (unsigned short)(indexNumber+0);
				indexNumber+=4;
			}
			else
			{
				indexNumber+=3;
			}
		}
		
		RenderMeshIndexBuffer->Unlock();
		RenderMeshVertexBuffer->Unlock();
	}

	Asura_Render::CurrentStates.RequestCullMode(ASURA_CULLMODE_CW);
	Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_OFF);
	Asura_Direct3D::D3DDevice->SetStreamSource(0,RenderMeshVertexBuffer,0,sizeof(D3DGenericVertex));
	Asura_Direct3D::D3DDevice->SetIndices(RenderMeshIndexBuffer);

	Asura_Render::CurrentStates.RequestTextureOpMode(ASURA_TEXTUREOP_SINGLE_PASS_2X);

	{
		int currentTexture = Asura_Material_System::GetTextureID(mesh->TextureDataPtr[0].m_iMaterialID);
		int start = 0;
		int startIndex = 0;
		int trisToDraw = 0;
		for (int p=0; p<mesh->NumberOfPolygons; p++)
		{
			int iTextureID = Asura_Material_System::GetTextureID(mesh->TextureDataPtr[p].m_iMaterialID);
			if (iTextureID != currentTexture)
			{
				Asura_Render::CurrentStates.RequestTextureIndex(0,currentTexture);
				Asura_Direct3D::D3DDevice->DrawIndexedPrimitive(D3DPT_TRIANGLELIST,0,0,totalIndices,startIndex,trisToDraw);
				startIndex = start;
				trisToDraw = 0;
				currentTexture = iTextureID;
			}

			start+=3;
			trisToDraw++;
			
			if (mesh->PolygonDataPtr[p].IsQuad())
			{
				start+=3;
				trisToDraw++;
			}

			if(p==mesh->NumberOfPolygons-1)
			{
				Asura_Render::CurrentStates.RequestTextureIndex(0,currentTexture);
				Asura_Direct3D::D3DDevice->DrawIndexedPrimitive(D3DPT_TRIANGLELIST,0,0,totalIndices,startIndex,trisToDraw);
			}
		}
	}

}
#endif //ASURA_TOOLS
