/**
*
* \file Asura_Direct3D_Shaders.h
*
* Created by Kevin Lea - 12/07/2002 14:38:25
*
* Copyright (C) 2002 Rebellion, All Rights Reserved.
*
* \brief
*
**/
#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __ASURA_DIRECT3D_SHADERS_H__
#define __ASURA_DIRECT3D_SHADERS_H__
#include "Asura_Direct3D.h"

enum ASURA_D3D_PIXELSHADER_ID
{
	ASURA_D3D_PS_NONE = -1,

	ASURA_D3D_PS_NORMALMAPPEDSKIN,

#if defined ASURA_TOOLS
	ASURA_D3D_PS_TOOLS_SPECULARSOFT,
	ASURA_D3D_PS_TOOLS_SPECULARSHARP,
#endif

	ASURA_D3D_PS_OBJECT_REFLECTIVE,
	ASURA_D3D_PS_OBJECT_REFLECTIVE_TEXALPHA,

	ASURA_D3D_PS_SPECULAR_SOFT,
	ASURA_D3D_PS_SPECULAR_SOFT_TEXALPHA, // like softspecular but factors basetexalpha into the output alpha
	ASURA_D3D_PS_SPECULAR_SHARP,
	ASURA_D3D_PS_SPECULAR_SHARP_TEXALPHA, // like sharpspecular but factors basetexalpha into the output alpha

	ASURA_D3D_PS_ENVIRONMENT_REFLECTIVE,
	ASURA_D3D_PS_ENVIRONMENT_REFLECTIVE_NORMAL,

	ASURA_D3D_PS_EXTRACTGLOW,
	ASURA_D3D_PS_APPLYBLUR,

	// RJF - 23 June 2005 - the vision mode effect being used in Rogue
	// Like the hologram effect below, this is going into Asura itself because to do project-specific shaders we'd need to use the effects system
	// which for reasons beyond my ken we don't want to use. However, all future projects would seem to use that system, so /this/ system almost
	// becomes Rogue-specific anyway.
	ASURA_D3D_PS_VISIONMODE,
	ASURA_D3D_PS_VISIONMODE_SILHOUETTE,

	ASURA_D3D_PS_SHADOWMAPPING_DEPTHTEXTURE,

	ASURA_D3D_PS_PFX_HEATHAZE,

#ifdef ASURA_USE_WATER
	ASURA_D3D_PS_WATER,
	ASURA_D3D_PS_WATER_BLENDED,
	ASURA_D3D_PS_WATER_FALLBACK,
#endif

	ASURA_D3D_PS_SUNGLARE,
	ASURA_D3D_PS_SUNGLARE_3X,

#ifdef ASURA_USE_CARPET	
	ASURA_D3D_PS_CARPET_LIGHTS,
	ASURA_D3D_PS_CARPET_LIGHTD,
#endif

#if defined( ASURA_USE_BOTANICALS)
  #if !defined( ASURA_DIRECT3D_BGR_SUPPORT_SM30) || 0 == ASURA_DIRECT3D_BGR_SUPPORT_SM30	
	ASURA_D3D_PS_BOTANICAL_GEOMETRY3D,
	ASURA_D3D_PS_BOTANICAL_GEOMETRY3D_LIGHTD,
	ASURA_D3D_PS_BOTANICAL_IMPOSTER_T1_LIGHTS,
	ASURA_D3D_PS_BOTANICAL_IMPOSTER_T2_LIGHTS,
	ASURA_D3D_PS_BOTANICAL_IMPOSTER_T1_LIGHTD,
	ASURA_D3D_PS_BOTANICAL_IMPOSTER_T2_LIGHTD,
  #endif
#endif

#if defined( ASURA_USE_FSFX)

	ASURA_D3D_PS_QUADRATIC,
	ASURA_D3D_PS_CUBIC,	
	ASURA_D3D_PS_LINEAR,
	ASURA_D3D_PS_DESATURATE_AND_TINT,
	ASURA_D3D_PS_FSFX_BLUR,
#endif

#if defined(ASURA_USE_LINE_RENDERING)
	// (SB - 07/11/06)
	// This is going into Asura itself because there is no simple way to do project-specific shaders 
	// Simpsons outline shader
	ASURA_D3D_PS_OUTLINE,
	// Simpsons ID render shader
	ASURA_D3D_PS_IDRENDER,
#endif

#if defined( ASURA_TARGET_SM3)
	ASURA_D3D_PS_DEFERRED_DEBUGSHOWBUFFERS,
	ASURA_D3D_PS_DEFERRED_DEBUGSHOWBUFFERS_ABS,
	ASURA_D3D_PS_DEFERRED_DEBUGSHOWBUFFERS_NORMALS,
	ASURA_D3D_PS_DEFERRED_DEBUGSHOWBUFFERS_SHADOWBUFFER,
#endif
	
	ASURA_D3D_NUMBER_OF_PIXELSHADERS
};

enum ASURA_D3D_VERTEXSHADER_ID
{
	//	ASURA_D3D_VS_NONE = -1,

	ASURA_D3D_VS_OBJECT_PLANARSHADOW,
	ASURA_D3D_VS_OBJECT_ORDER2SH,
	ASURA_D3D_VS_OBJECT_ORDER2SH_WITHALPHA,
	ASURA_D3D_VS_OBJECT_REFLECTIVE,
	ASURA_D3D_VS_OBJECT_REFLECTIVE_WITHALPHA,

	ASURA_D3D_VS_OBJECT_ORDER2SH_UNCOMPRESSED,
	ASURA_D3D_VS_OBJECT_PLANARSHADOW_UNCOMPRESSED,

	ASURA_D3D_VS_SKIN_ORDER2SH_4BONES,
	ASURA_D3D_VS_SKIN_ORDER2SH_4BONES_VS1,	// VGP 27Mar2006

#ifdef ASURA_TOOLS
	//RAR 08/07/04 Tools renderers for testing characters using fewer than 4 bones per vertex
	ASURA_D3D_VS_SKIN_ORDER2SH_3BONES,
	ASURA_D3D_VS_SKIN_ORDER2SH_2BONES,
	ASURA_D3D_VS_SKIN_ORDER2SH_1BONE,
#endif
	ASURA_D3D_VS_SKIN_ORDER2SH_4BONES_NORMALMAPPED,

	ASURA_D3D_VS_SKIN_ORDER2SH_4BONES_REFLECTIVE,
	ASURA_D3D_VS_SKIN_ORDER2SH_4BONES_REFLECTIVE_VS1,	// VGP 27Mar2006

	ASURA_D3D_VS_SKIN_ORDER2SH_4BONES_SPECULAR,
	ASURA_D3D_VS_SKIN_ORDER2SH_4BONES_SPECULAR_VS1,	// VGP 27Mar2006

	//  RJF:  (12/05/2005 15:28:00):  The holographic effect being used in Rogue.
	ASURA_D3D_VS_SKIN_ORDER2SH_4BONES_HOLOGRAM,
	ASURA_D3D_VS_OBJECT_HOLOGRAM,

	ASURA_D3D_VS_SKIN_PLANARSHADOW,

	ASURA_D3D_VS_MESHHIGHLIGHT,
	ASURA_D3D_VS_MESHHIGHLIGHT_WITHOFFSET,

	ASURA_D3D_VS_FLOWINGWATER,

#if defined ASURA_TOOLS
	ASURA_D3D_VS_TOOLS_SPECULAR,
#endif

	ASURA_D3D_VS_EXTRACTGLOW,
	ASURA_D3D_VS_APPLYBLUR,

#if defined ASURA_TOOLS
	ASURA_D3D_VS_TOOLS_MODULE,
	ASURA_D3D_VS_TOOLS_MODULE_LINEARFOG,
	ASURA_D3D_VS_TOOLS_MODULE_GRAPHFOG,
	ASURA_D3D_VS_TOOLS_MODULE_REFLECTIVE,
	ASURA_D3D_VS_TOOLS_CMESH_ZOFFSET,
#endif
	ASURA_D3D_VS_ENVIRONMENT_LIGHTING,
	ASURA_D3D_VS_ENVIRONMENT_AMBIENT,
	ASURA_D3D_VS_ENVIRONMENT_TEXTURES,
	ASURA_D3D_VS_ENVIRONMENT_TEXTURE_DUPLICATECOORDS,
	ASURA_D3D_VS_ENVIRONMENT_REFLECTIVE,
	ASURA_D3D_VS_ENVIRONMENT_SHADOWS,

	ASURA_D3D_VS_ENVIRONMENT_LINEARFOG,
	ASURA_D3D_VS_ENVIRONMENT_GRAPHFOG,

	ASURA_D3D_VS_PFX_HEATHAZE,

#ifdef ASURA_USE_WATER
	ASURA_D3D_VS_WATER,
	ASURA_D3D_VS_WATER_VS11,
#endif

	ASURA_D3D_VS_SUNGLARE,
	ASURA_D3D_VS_SUNRECT,
	
#if defined( ASURA_USE_CARPET) && !defined( ASURA_DIRECT3D_BGR_SUPPORT_SM30)
	ASURA_D3D_VS_CARPET_FOGL_LIGHTS,
	ASURA_D3D_VS_CARPET_FOGG_LIGHTS,
	ASURA_D3D_VS_CARPET_FOGL_LIGHTD,
	ASURA_D3D_VS_CARPET_FOGG_LIGHTD,
#endif

#if defined( ASURA_USE_BOTANICALS)
 #if defined ASURA_TOOLS
	ASURA_D3D_VS_TOOLS_BOTANICALS_LINEARFOG,
	ASURA_D3D_VS_TOOLS_BOTANICALS_GRAPHFOG,
 #endif
 #if !defined( ASURA_DIRECT3D_BGR_SUPPORT_SM30) || 0 == ASURA_DIRECT3D_BGR_SUPPORT_SM30	
	ASURA_D3D_VS_BOTANICAL_GEOMETRY3D_FOGL_LIGHTS,
	ASURA_D3D_VS_BOTANICAL_LEAF_FOGL_LIGHTS,
	ASURA_D3D_VS_BOTANICAL_IMPOSTER_T1_FOGL_LIGHTS,
	ASURA_D3D_VS_BOTANICAL_IMPOSTER_T2_FOGL_LIGHTS,
	
	ASURA_D3D_VS_BOTANICAL_GEOMETRY3D_FOGG_LIGHTS,
	ASURA_D3D_VS_BOTANICAL_LEAF_FOGG_LIGHTS,
	ASURA_D3D_VS_BOTANICAL_IMPOSTER_T1_FOGG_LIGHTS,
	ASURA_D3D_VS_BOTANICAL_IMPOSTER_T2_FOGG_LIGHTS,
	
	ASURA_D3D_VS_BOTANICAL_GEOMETRY3D_FOGL_LIGHTD,
	ASURA_D3D_VS_BOTANICAL_LEAF_FOGL_LIGHTD,
	ASURA_D3D_VS_BOTANICAL_IMPOSTER_T1_FOGL_LIGHTD,
	ASURA_D3D_VS_BOTANICAL_IMPOSTER_T2_FOGL_LIGHTD,
	
	ASURA_D3D_VS_BOTANICAL_GEOMETRY3D_FOGG_LIGHTD,
	ASURA_D3D_VS_BOTANICAL_LEAF_FOGG_LIGHTD,
	ASURA_D3D_VS_BOTANICAL_IMPOSTER_T1_FOGG_LIGHTD,
	ASURA_D3D_VS_BOTANICAL_IMPOSTER_T2_FOGG_LIGHTD,
  #endif
#endif

#if defined( ASURA_USE_FSFX)
	ASURA_D3D_VS_QUADRATIC,
	ASURA_D3D_VS_CUBIC,	
	ASURA_D3D_VS_LINEAR,	
	ASURA_D3D_VS_FSFX_RADIALBLUR,
	//ASURA_D3D_VS_DESATURATE_AND_TINT,
	ASURA_D3D_VS_FSFX_BLUR,
#endif
	
#if defined (ASURA_USE_LINE_RENDERING)
	// (SB - 07/11/06)
	// This is going into Asura itself because there is no simple way to do project-specific shaders 
	// Simpsons outline shader
	ASURA_D3D_VS_OUTLINE,
	// Shader for skinning with line masking data in normals
	ASURA_D3D_VS_IDRENDER_SKIN_4BONES,
	ASURA_D3D_VS_IDRENDER_OBJECT,
#endif

#if defined( ASURA_TARGET_SM3)
	ASURA_D3D_VS_DEFERRED_DEBUGSHOWBUFFERS,
#endif
	
	ASURA_D3D_NUMBER_OF_VERTEXSHADERS
};

// VGP 15Feb2006
#define ASURA_D3D_NUMBER_OF_VERTEX_FORMATS	(2*ASURA_D3D_NUMBER_OF_VERTEXSHADERS)

enum ASURA_D3D_VS_CODE_TYPE
{
	ASURA_D3D_VS_CODE_TYPE_ASM,
	ASURA_D3D_VS_CODE_TYPE_HLSL,
};

// Pre-assembly code, etc.
struct Asura_Direct3D_VS_Definition
{
	const char*					m_szCode;
//DeadCode VGP 15/02/2006 	Direct3D_VertexElement*		m_pxVertexDeclElements;
	unsigned int				m_uVertexDeclIndex;
	float						m_fVersion;
	ASURA_D3D_VS_CODE_TYPE		m_eCodeType;
	const char*					m_pszEntryFunction;
};

// Assembled version
struct Asura_Direct3D_VS_Storage
{
	Direct3D_VertexShader*		m_pxVertexShader;
//DeadCode VGP 15/02/2006 	Direct3D_VertexDeclaration*	m_pxVertexShaderDecl;
	unsigned int				m_uVertexDeclIndex;
};

enum ASURA_D3D_PS_CODE_TYPE
{
	ASURA_D3D_PS_CODE_TYPE_ASM,
	ASURA_D3D_PS_CODE_TYPE_HLSL,
};

// Pre-assembly code, etc.
struct Asura_Direct3D_PS_Definition
{
	const char*					m_szCode;
	float						m_fVersion;
	ASURA_D3D_PS_CODE_TYPE		m_eCodeType;
	const char*					m_pszEntryFunction;
};

class Asura_Direct3D_Shaders
{
	friend class Asura_Direct3D_PixelShader;
	friend class Asura_Direct3D_VertexShader;
public:
	/// Kevin Lea - 12/07/2002 15:05:14 - initialise and shutdown to be called just once
	static void Initialise(void);
	static void Shutdown(void);

	// VGP 15Feb2006 Registers a vertex format and returns an index into the array of formats
	static int RegisterVertexFormat(Direct3D_VertexElement* pxElements);

	static void SetVertexDeclaration(unsigned int uDeclIndex);

	/// Kevin Lea - 12/07/2002 15:06:08 - Pixel and vertex shaders are usually trashed 
	/// when a Direct3D app loses focus (e.g. user presses alt-tab), so we need an
	/// interface to release and then later re-create the shaders - rather than a usual
	/// one shot Reset() function.
	static void CreateShaders(void);
	static void ReleaseShaders(void);

	/// Kevin Lea - 12/07/2002 15:45:03 - request vertex/pixel shader setting
	/// redundant changes are ignored
	static void RequestPixelShader(enum ASURA_D3D_PIXELSHADER_ID shader);
	static void RequestVertexShader(enum ASURA_D3D_VERTEXSHADER_ID shader, bool bSetDeclaration = true);
	static void RequestFixedFunctionVertexShader(DWORD FVFCode);
	
	static inline ASURA_D3D_VERTEXSHADER_ID GetCurrentVertexShader() { return( s_eCurrentVertexShader);}
	static inline ASURA_D3D_PIXELSHADER_ID GetCurrentPixelShader() { return( s_eCurrentPixelShader);}

	// VGP 15Feb2006 Resets 'last used' values
	static void ClearLastShader()
	{
		s_eCurrentVertexShader = ASURA_D3D_NUMBER_OF_VERTEXSHADERS;
		s_eCurrentPixelShader = ASURA_D3D_NUMBER_OF_PIXELSHADERS;
		s_uCurrentVertexDecl = ASURA_D3D_NUMBER_OF_VERTEX_FORMATS;
	}

    static HRESULT SetVertexShaderConstantF( const u_int uStartRegister, const float* pfConstantData, const u_int uVector4fCount);

private:

	static bool									s_bBeenInitialised;
	static bool									s_bShadersReady;

	// Vertex shaders (definition contains code, vertex elements, version, and storage contains vertex declaration, shader itself)
	static Asura_Direct3D_VS_Definition			s_axVertexShaderDefs[ASURA_D3D_NUMBER_OF_VERTEXSHADERS];
	static Asura_Direct3D_VS_Storage			s_axVertexShaders[ASURA_D3D_NUMBER_OF_VERTEXSHADERS];
	static ASURA_D3D_VERTEXSHADER_ID			s_eCurrentVertexShader;	// VGP 15Feb2006

	// VGP 15Feb2006 Vertex formats
	// Switching vertex declarations is expensive. Keep a list of unique declarations and keep track of
	// the last one used.
	static Direct3D_VertexElement*				s_apxVertexDeclElements[ASURA_D3D_NUMBER_OF_VERTEX_FORMATS];
	static Direct3D_VertexDeclaration*			s_apxVertexDeclarations[ASURA_D3D_NUMBER_OF_VERTEX_FORMATS];
	static unsigned int							s_uVertexDeclCount;
	static unsigned int							s_uCurrentVertexDecl;

	// Pixel shaders 
	static Asura_Direct3D_PS_Definition			s_axPixelShaderDef[ASURA_D3D_NUMBER_OF_PIXELSHADERS];
	static Direct3D_PixelShader*				s_apxPixelShaders[ASURA_D3D_NUMBER_OF_PIXELSHADERS];
	static ASURA_D3D_PIXELSHADER_ID				s_eCurrentPixelShader;	// VGP 15Feb2006
};

class Asura_Direct3D_PixelShader
{
public:
	Asura_Direct3D_PixelShader(enum ASURA_D3D_PIXELSHADER_ID eID, const char* szCode, float fVersion = 1.1f, const ASURA_D3D_PS_CODE_TYPE eCodeType = ASURA_D3D_PS_CODE_TYPE_ASM, const char* const pszEntryFunction = NULL)
	{
		Asura_Direct3D_PS_Definition& xPixelShaderDef = Asura_Direct3D_Shaders::s_axPixelShaderDef[ eID];
		
		xPixelShaderDef.m_szCode = szCode;
		xPixelShaderDef.m_fVersion = fVersion;
		xPixelShaderDef.m_eCodeType = eCodeType;
		xPixelShaderDef.m_pszEntryFunction = pszEntryFunction;
	}
};

class Asura_Direct3D_VertexShader
{
public:
	Asura_Direct3D_VertexShader(enum ASURA_D3D_VERTEXSHADER_ID eID, const char* szCode, Direct3D_VertexElement* pxVertexDeclaration, float fVersion = 1.1f, const ASURA_D3D_VS_CODE_TYPE eCodeType = ASURA_D3D_VS_CODE_TYPE_ASM, const char* const pszEntryFunction = NULL)
	{
		Asura_Direct3D_Shaders::s_axVertexShaderDefs[eID].m_szCode = szCode;
//DeadCode VGP 15/02/2006 		Asura_Direct3D_Shaders::s_axVertexShaderDefs[eID].m_pxVertexDeclElements = pxVertexDeclaration;
		Asura_Direct3D_Shaders::s_axVertexShaderDefs[eID].m_uVertexDeclIndex = Asura_Direct3D_Shaders::RegisterVertexFormat(pxVertexDeclaration);
		Asura_Direct3D_Shaders::s_axVertexShaderDefs[eID].m_fVersion = fVersion;
		Asura_Direct3D_Shaders::s_axVertexShaderDefs[eID].m_eCodeType = eCodeType;
		Asura_Direct3D_Shaders::s_axVertexShaderDefs[eID].m_pszEntryFunction = pszEntryFunction;
	}
};

// VGP 15Feb2006
class Asura_Direct3D_VertexFormatDescription
{
public:
	Asura_Direct3D_VertexFormatDescription(Direct3D_VertexElement* pxVertexDeclaration)
	{
		m_uVertexDeclIndex = Asura_Direct3D_Shaders::RegisterVertexFormat(pxVertexDeclaration);
	}

	unsigned int GetVertexDeclIndex() const
	{
		return m_uVertexDeclIndex;
	}

	void Set() const
	{
		Asura_Direct3D_Shaders::SetVertexDeclaration(m_uVertexDeclIndex);
	}

private:
	unsigned int m_uVertexDeclIndex;
};

#endif//__ASURA_DIRECT3D_SHADERS_H__
