//*******************************************************************
//
// File: Asura_Direct3D_VertexBuffer_Dynamic.cpp
//
// Created: 24 Oct 2006
//
// Description: 
//
//*******************************************************************

// system libraries
#include "Core/Asura.h"

// main header
#include "Asura_Direct3D_VertexBuffer_Dynamic.h"

// other libraries

// internal dependencies

//*******************************************************************
// Constants
//*******************************************************************


//*******************************************************************
// Local Classes
//*******************************************************************


//*******************************************************************
// Function Definitions
//*******************************************************************
Asura_Direct3D_VertexBuffer_Dynamic::Asura_Direct3D_VertexBuffer_Dynamic()
{
	this->Init();
}

Asura_Direct3D_VertexBuffer_Dynamic::~Asura_Direct3D_VertexBuffer_Dynamic()
{
	this->Clear();
}

void Asura_Direct3D_VertexBuffer_Dynamic::Init()
{
	this->m_uVertexBufferSizeBytes = 0;
	this->m_pxD3DVertexBuffer = NULL;

	this->m_uVertexBufferUsedSizeBytes = 0;
	this->m_uVertexBufferLockedSizeBytes = 0;
	this->m_pVertexBufferLockedStart = NULL;
#if defined( ASURA_ASSERT)
	this->m_pVertexBufferLockedEnd = NULL;
#endif
}
	
void Asura_Direct3D_VertexBuffer_Dynamic::Clear()
{
	Asura_Assert( NULL == this->m_pVertexBufferLockedStart, "The Vertex Buffer is locked. Unlock() before calling this method.");
	
	SAFE_RELEASE( this->m_pxD3DVertexBuffer);
	
	this->Init();
}

bool Asura_Direct3D_VertexBuffer_Dynamic::ClearDirect3DData()
{
	Asura_Assert( NULL == this->m_pVertexBufferLockedStart, "The Vertex Buffer is locked. Unlock() before calling this method.");
	
	SAFE_RELEASE( this->m_pxD3DVertexBuffer);
	
	this->m_pxD3DVertexBuffer = NULL;

	this->m_uVertexBufferUsedSizeBytes = 0;
	this->m_uVertexBufferLockedSizeBytes = 0;
	this->m_pVertexBufferLockedStart = NULL;
#if defined( ASURA_ASSERT)
	this->m_pVertexBufferLockedEnd = NULL;
#endif
	
	return( true);
}

bool Asura_Direct3D_VertexBuffer_Dynamic::CreateDirect3DData()
{
	Asura_Assert( NULL == this->m_pVertexBufferLockedStart, "The Vertex Buffer is locked. Unlock() before calling this method.");

	return( this->SetSize( this->m_uVertexBufferSizeBytes));	
}

bool Asura_Direct3D_VertexBuffer_Dynamic::SetSize( const u_int uSizeBytes)
{
	HRESULT xResult;
	Asura_Assert( NULL == this->m_pVertexBufferLockedStart, "The Vertex Buffer is locked. Unlock() before calling this method.");
	
	this->Clear();
	
	Direct3D_Device* pxD3DDevice = Asura_Direct3D::D3DDevice;
	
	Asura_Assert( NULL != pxD3DDevice, "D3D needs to be setup");
	
	const u_int uUsageFlags = D3DUSAGE_DYNAMIC | D3DUSAGE_WRITEONLY;
	const D3DPOOL ePool = D3DPOOL_DEFAULT;
	Direct3D_VertexBuffer* pxD3DVertexBuffer = NULL;
	xResult = pxD3DDevice->CreateVertexBuffer( uSizeBytes, uUsageFlags, 0, ePool, &pxD3DVertexBuffer, NULL);
	if( ASURA_D3D_FAILED( xResult))
	{
		return( false);
	}
	
	this->m_pxD3DVertexBuffer = pxD3DVertexBuffer;
	this->m_uVertexBufferSizeBytes = uSizeBytes;
	
	return( true);
}

bool Asura_Direct3D_VertexBuffer_Dynamic::Lock( const u_int uSizeBytesRequired, u_int& uLockedVertexBufferOffsetBytes, void*& pLockedMemory)
{
	HRESULT xResult;
	
	const u_int uAlignment = 4;
	u_int uAlignedSizeBytesRequired = uSizeBytesRequired & (~(uAlignment-1));
	if( uAlignedSizeBytesRequired != uSizeBytesRequired)
	{
		uAlignedSizeBytesRequired += uAlignment;
	}

	Asura_Assert( 0 == (uAlignedSizeBytesRequired & (uAlignment-1)), "Not aligned!");
	
	Asura_Assert( NULL != this->m_pxD3DVertexBuffer, "No vertex buffer has been allocated yet.");
	Asura_Assert( NULL == this->m_pVertexBufferLockedStart, "The Vertex Buffer is locked. Unlock() before calling this method.");
	Asura_Assert( uAlignedSizeBytesRequired <= this->m_uVertexBufferSizeBytes, "The Vertdex Buffer does not have this many bytes in it" );
	u_int uLockFlags = D3DLOCK_NOOVERWRITE;
	
	// DLH - 24 October 2006 15:35:20 - There is not enough room left in the VB so ask for a clean one 
	if( this->m_uVertexBufferSizeBytes < (this->m_uVertexBufferUsedSizeBytes + uAlignedSizeBytesRequired))
	{
		uLockFlags = D3DLOCK_DISCARD;
		this->m_uVertexBufferUsedSizeBytes = 0;
	}
	
	void* pLockedMemoryParam = NULL;
	xResult = this->m_pxD3DVertexBuffer->Lock( this->m_uVertexBufferUsedSizeBytes, uAlignedSizeBytesRequired, &pLockedMemoryParam, uLockFlags);
	if( ASURA_D3D_FAILED( xResult))
	{
		return( false);
	}
	
	this->m_pVertexBufferLockedStart = pLockedMemoryParam;
	this->m_uVertexBufferLockedSizeBytes = uAlignedSizeBytesRequired;	
#if defined( ASURA_ASSERT)
	this->m_pVertexBufferLockedEnd = (static_cast< uint8*>( pLockedMemoryParam)) + uAlignedSizeBytesRequired;
#endif
	
	pLockedMemory = pLockedMemoryParam;
	uLockedVertexBufferOffsetBytes = this->m_uVertexBufferUsedSizeBytes;
	
	return( true);
}

void Asura_Direct3D_VertexBuffer_Dynamic::Unlock( const u_int uSizeBytesUsed)
{
	HRESULT xResult;

	Asura_Assert( NULL != this->m_pxD3DVertexBuffer, "No vertex buffer has been allocated yet.");
	Asura_Assert( NULL != this->m_pVertexBufferLockedStart, "The Vertex Buffer is NOT locked. Lock() before calling this method.");
	
	Asura_Assert( (this->m_uVertexBufferUsedSizeBytes + uSizeBytesUsed) <= this->m_uVertexBufferSizeBytes, "More data has been used than is in the Vertex Buffer");
	Asura_Assert( uSizeBytesUsed <= m_uVertexBufferLockedSizeBytes, "More data has been used than was locked");
	
	xResult = this->m_pxD3DVertexBuffer->Unlock();
	ASURA_D3D_OK( xResult);
	
	this->m_uVertexBufferUsedSizeBytes += uSizeBytesUsed;
	
	this->m_pVertexBufferLockedStart = NULL;
	this->m_uVertexBufferLockedSizeBytes = 0;
#if defined( ASURA_ASSERT)
	this->m_pVertexBufferLockedEnd = NULL;
#endif
}


//*******************************************************************
//
// (c) 2006 Rebellion Derby
//
// Warning: This computer program is protected by copyright law and 
// international treaties. Unauthorized reproduction or distribution
// of this program, or any portion of it, may result in severe civil and
// criminal penalties, and will be prosecuted to the maximum extent 
// possible under the law.
//
//*******************************************************************

