//
// Asura_PlatformRenderEnvironment.cpp
//
// Created by KJL - 28/02/2003 10:48:15
//
// Copyright (C) 2003 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

// This file's header.
#include "Rendering/Asura_PlatformRenderEnvironment.h"

// Asura headers
#include "Animation/Asura_TextureAnimation.h"
#include "Camera/Asura_Camera.h"
#include "Core/Asura_Material_System.h"
#include "Core/Asura_TextureManagement.h"
#include "Core/Asura_Timers.h"
#include "Core/Memory/Asura_ResourceAllocator.h"
#include "Debugging/Asura_Logfile.h"
#include "Light/Asura_Photon.h"
#include "Mesh/Asura_CollisionMesh.h" // for bounding box
#include "Rendering/Asura_Fog.h"
#include "Rendering/Asura_Render.h"
#include "Rendering/Asura_Module.h"
#include "Rendering/Asura_Direct3D_Shaders.h"
#include "Rendering/Asura_Direct3D_UserOptions.h"
#include "SpecialFX/Asura_Weather_System.h"

#ifdef ASURA_TOOLS
#include "Light/Asura_Light.h"
#include "Mesh/Asura_Mesh.h"
#include "Misc/Tools_Timed_Messages.h"
#include "Rendering/Asura_Surface.h"
#include "Rendering/Asura_PlatformRenderMesh.h"
#endif

#ifndef fASURA_DETAIL_MAP_SCALE
#define fASURA_DETAIL_MAP_SCALE 32.0f
#endif

// Water sim parameters
const float fWATER_TEXTURE_SPEED_U = 0.1525f;
const float fWATER_TEXTURE_SPEED_V = 0.0f;

const float fWATER_AMPLITUDE_U = 0.1f;
const float fWATER_AMPLITUDE_V = 0.1f;

const float fWATER_WAVE_SPEED_U = 1.2f;
const float fWATER_WAVE_SPEED_V = 3.0f;

// Scrolling-texture-only parameters
const float fSCROLL_TEXTURE_SPEED_U = 0.3f;
const float fSCROLL_TEXTURE_SPEED_V = 0.0f;

static const float fNORMAL_DECOMPRESS[4] = { 2.f, -1.f, 0, 1.f };

// (SB - 11/12/06) Render state override hook
Asura_Direct3D_Env_Zone::RenderStateOverride* Asura_Direct3D_Env_Zone::s_pfnRenderStateOverride = NULL;

////////////////////////////////////////////////////////////////////////////////
//
// Asura_Direct3D_Env_Zone
//
////////////////////////////////////////////////////////////////////////////////

int					Asura_Direct3D_Env_Zone::s_iAttenuationMap		= -1;
int					Asura_Direct3D_Env_Zone::s_iDetailMap			= -1;
int					Asura_Direct3D_Env_Zone::s_iRainSplashTexture	= -1;

D3DPRIMITIVETYPE	Asura_Direct3D_Env_Zone::s_eCurrentPrimitiveType;


Asura_Direct3D_Env_Zone::Asura_Direct3D_Env_Zone (void) :
	m_uTotalNumberOfVertexBuffers(0),
	m_uNumberOfMeshes(0),
	m_uNumberOfStrips(0),
	m_pxMeshes(NULL),
	m_pxStrips(NULL),

	m_ppxVertexBuffer(NULL),
	m_ppxIndexBuffer(NULL),

	m_bUsingPrecomputedStrips(false),
	m_bUsingPrecomputedData(false),
	m_bUsingOwnVertexBuffer( false ),

	m_fWaterTextureOffsetU(0.0f),
	m_fWaterTextureOffsetV(0.0f),
	m_fWaterPhaseU(0.0f),
	m_fWaterPhaseV(0.0f),
	m_fScrollTextureOffsetU(0.0f),
	m_fScrollTextureOffsetV(0.0f)
{
	// Nothing more to initialise...
}


Asura_Direct3D_Env_Zone::~Asura_Direct3D_Env_Zone ()
{
	Asura_ResourcePushType(ASURA_RESOURCE_ENVIRONMENT_MESH);
	{
		Asura_ResourceDeallocateArray(m_pxMeshes);
		Asura_ResourceDeallocateArray(m_pxStrips);

		// JAT 26/01/07 - if we are here because of the static s_xZone variable and 
		// there is data to release it won't work because Direct3D has been closed

		// If we were using the tri-list from the Asura_Surface_Mesh_System, that will clean the buffers up for us
		for (u_int u=0; u<m_uTotalNumberOfVertexBuffers; u++)
		{
			// JAT 26/01/07 - new flag to say the vertex buffers need to be deleted e.g. GenerateTriLists
			if ( m_bUsingOwnVertexBuffer || m_bUsingPrecomputedStrips )
			{
				if (m_ppxVertexBuffer[u])
				{
					m_ppxVertexBuffer[u]->Release();
				}
			}
			if( m_bUsingPrecomputedStrips )
			{
				if (m_ppxIndexBuffer[u])
				{
					m_ppxIndexBuffer[u]->Release();
				}
			}
		}
		Asura_ResourceDeallocateArray(m_ppxVertexBuffer);
		Asura_ResourceDeallocateArray(m_ppxIndexBuffer);
	}
	Asura_ResourcePopType();
}

/*
Asura_Direct3D_Env_Zone& Asura_Direct3D_Env_Zone::operator = ( const Asura_Direct3D_Env_Zone& xOther )
{
	m_uTotalNumberOfVertexBuffers = xOther.m_uTotalNumberOfVertexBuffers;
	m_uNumberOfMeshes = xOther.m_uNumberOfMeshes;
	m_uNumberOfStrips = xOther.m_uNumberOfStrips;

	//Asura_Direct3D_Env_Mesh*	m_pxMeshes;
	//Asura_Direct3D_Env_Strip*	m_pxStrips;

	//Direct3D_VertexBuffer**		m_ppxVertexBuffer;
	//Direct3D_IndexBuffer**		m_ppxIndexBuffer;

	m_bUsingPrecomputedStrips = xOther.m_bUsingPrecomputedStrips;
	m_bUsingPrecomputedData = xOther.m_bUsingPrecomputedData;

	m_fWaterTextureOffsetU = xOther.m_fWaterTextureOffsetU;
	m_fWaterTextureOffsetV = xOther.m_fWaterTextureOffsetV;
	m_fWaterPhaseU = xOther.m_fWaterPhaseU;
	m_fWaterPhaseV = xOther.m_fWaterPhaseV;
	m_fScrollTextureOffsetU = xOther.m_fScrollTextureOffsetU;
	m_fScrollTextureOffsetV = xOther.m_fScrollTextureOffsetV;

	return *this;
}
*/

void Asura_Direct3D_Env_Zone::Reset (void)
{
	Asura_ResourcePushType(ASURA_RESOURCE_ENVIRONMENT_MESH);
	{
		Asura_ResourceDeallocateArray(m_pxMeshes);
		Asura_ResourceDeallocateArray(m_pxStrips);

		// If we were using the tri-list from the Asura_Surface_Mesh_System, that will clean the buffers up for us

		for (u_int u=0; u<m_uTotalNumberOfVertexBuffers; u++)
		{
			// JAT 26/01/07 - new flag to say the vertex buffers need to be deleted e.g. GenerateTriLists
			if ( m_bUsingOwnVertexBuffer || m_bUsingPrecomputedStrips )
			{
				if (m_ppxVertexBuffer[u])
				{
					m_ppxVertexBuffer[u]->Release();
				}
			}
			if(m_bUsingPrecomputedStrips)
			{
				if (m_ppxIndexBuffer[u])
				{
					m_ppxIndexBuffer[u]->Release();
				}
			}
		}
		Asura_ResourceDeallocateArray(m_ppxVertexBuffer);
		Asura_ResourceDeallocateArray(m_ppxIndexBuffer);

		// Reset the data...
		m_uTotalNumberOfVertexBuffers	= 0;
		m_uNumberOfMeshes				= 0;
		m_uNumberOfStrips				= 0;

		m_bUsingPrecomputedStrips		= false;
		m_bUsingPrecomputedData			= false;

		m_fWaterTextureOffsetU			= 0.0f;
		m_fWaterTextureOffsetV			= 0.0f;
		m_fWaterPhaseU					= 0.0f;
		m_fWaterPhaseV					= 0.0f;
		m_fScrollTextureOffsetU			= 0.0f;
		m_fScrollTextureOffsetV			= 0.0f;
	}
	Asura_ResourcePopType();
}


void Asura_Direct3D_Env_Zone::Render(void)
{
	if (m_uTotalNumberOfVertexBuffers==0) return;

	D3DPERFCALL(BeginEvent(D3DCOLOR_ARGB(0xff, 0xff, 0x00, 0x00), L"Environment"));

	if (s_iAttenuationMap==-1)
	{
		s_iAttenuationMap = Asura_Texture_Management::AddTextureWithResetGroup("\\graphicnovel\\expattenuation.dds", ASURA_TEXTURE_LOADED_PERMANENT );
		s_iDetailMap = Asura_Texture_Management::AddTextureWithResetGroup("\\graphicnovel\\detail.dds", ASURA_TEXTURE_LOADED_PERMANENT );
		s_iRainSplashTexture = Asura_Texture_Management::AddTextureWithResetGroup("\\specialfx\\splash.bmp", ASURA_TEXTURE_LOADED_PERMANENT );
		Asura_Texture_Management::LoadTextures();
	}

	Asura_Render::CurrentStates.RequestCullMode(ASURA_CULLMODE_CW);

	Asura_Direct3D::D3DDevice->SetVertexShaderConstantF(94, &fNORMAL_DECOMPRESS[0], 1);

	Asura_Direct3D::D3DDevice->SetSamplerState(0, D3DSAMP_ADDRESSU, D3DTADDRESS_WRAP );
	Asura_Direct3D::D3DDevice->SetSamplerState(0, D3DSAMP_ADDRESSV, D3DTADDRESS_WRAP );
	Asura_Direct3D::D3DDevice->SetSamplerState(1, D3DSAMP_ADDRESSU, D3DTADDRESS_WRAP );
	Asura_Direct3D::D3DDevice->SetSamplerState(1, D3DSAMP_ADDRESSV, D3DTADDRESS_WRAP );

	if (Asura_Direct3D_UserOptions::IsTrilinearFilteringOn())
	{
		Asura_Direct3D::D3DDevice->SetSamplerState(0,D3DSAMP_MIPFILTER,D3DTEXF_LINEAR);
		Asura_Direct3D::D3DDevice->SetSamplerState(1,D3DSAMP_MIPFILTER,D3DTEXF_LINEAR);
	}
	else
	{
		Asura_Direct3D::D3DDevice->SetSamplerState(0,D3DSAMP_MIPFILTER,D3DTEXF_POINT);
		Asura_Direct3D::D3DDevice->SetSamplerState(1,D3DSAMP_MIPFILTER,D3DTEXF_POINT);
	}

	if (Asura_Fog::IsFogRenderable())
	{
		// Switch on pixel fog...
		Asura_Direct3D::D3DDevice->SetRenderState(D3DRS_FOGTABLEMODE, D3DFOG_LINEAR);	
	}

	s_eCurrentPrimitiveType = m_bUsingPrecomputedStrips ? D3DPT_TRIANGLESTRIP : D3DPT_TRIANGLELIST;

	for(int i=0; i<Asura_Camera::GetNumberOfCameras(); i++)
	{
		Asura_Camera::SetCameraIndex(i);

		if (Asura_Camera::IsCameraActive())
		{
			Asura_Render::ApplyCameraSettings();

			// (AAM - 26.04.2004) Upload view-proj matrix to vertex shader
			D3DXMATRIX xView, xViewProj;

			Asura_Direct3D::D3DDevice->GetTransform(D3DTS_VIEW, &xView);
			Asura_Direct3D::D3DDevice->GetTransform(D3DTS_PROJECTION, &xViewProj);
			D3DXMatrixMultiply(&xViewProj, &xView, &xViewProj);
			D3DXMatrixTranspose(&xViewProj, &xViewProj);

			D3DXMATRIX xTransposedView;
			D3DXMatrixTranspose(&xTransposedView, &xView);

			Asura_Direct3D::D3DDevice->SetVertexShaderConstantF(0, xViewProj, 4);
			Asura_Direct3D::D3DDevice->SetVertexShaderConstantF(4, xTransposedView, 4);

			RenderAmbientPass();
			RenderDynamicLightsPass();
			RenderBasicTextures();
			RenderRainAndDetailTextures();
			RenderSelfIlluminatingPass();
#ifndef ASURA_TARGET_SM3
			RenderReflectivePass();
#endif //#ifndef ASURA_TARGET_SM3

#ifdef ASURA_ALPHA_DECAL_HACK_FOR_PSP
			RenderAlphaDecals();
#endif
			RenderFlowingWater();
			RenderAdditiveAlpha();
			if (Asura_Fog::IsFogRenderable())
			{
				RenderFog();
			}

		}
	}

	if (Asura_Fog::IsFogRenderable())
	{
		// And switch back to the object fog
		Asura_Direct3D::D3DDevice->SetRenderState(D3DRS_FOGTABLEMODE, D3DFOG_NONE);	
	}

	Asura_Render::CurrentStates.RequestZBufferEnabled(true);
	Asura_Render::CurrentStates.RequestZBufferWriteEnabled(true);
	Asura_Render::CurrentStates.RequestZBufferCompareMode(ASURA_ZBUFFERCMP_LESSEQUAL);

	// Update the water simulation
	UpdateWater();

	// Update scrolling textures
	UpdateScrollingTextures();

	D3DPERFCALL(EndEvent());
}

void Asura_Direct3D_Env_Zone::RenderAmbientPass()
{
	D3DPERFCALL(BeginEvent(0xffdf0000, L"Ambient pass"));

	Asura_Direct3D_Shaders::RequestVertexShader(ASURA_D3D_VS_ENVIRONMENT_AMBIENT);

	// As well as building a picture of ambient light from the SH, this pass also generates
	// a depth image of the environment into the Z buffer, so that future passes can be
	// culled quickly by the Z test.
	Asura_Render::CurrentStates.RequestZBufferEnabled(true);
	Asura_Render::CurrentStates.RequestZBufferWriteEnabled(true);
	Asura_Render::CurrentStates.RequestZBufferCompareMode(ASURA_ZBUFFERCMP_LESSEQUAL);

	// If we use a texture in stage 0, it will be a 1-bit alpha texture, which we do not want to filter.
	Asura_Direct3D::D3DDevice->SetSamplerState(0, D3DSAMP_MIPFILTER, D3DTEXF_NONE );

	u_int uCurrentBuffer = (u_int)-1;

	for(u_int uStripIndex = 0; uStripIndex < m_uNumberOfStrips; ++uStripIndex)
	{
		Asura_Direct3D_Env_Strip& xStrip = m_pxStrips[uStripIndex];

		if(!Asura_Module_System::IsModuleVisible(xStrip.m_iModuleIndex)) continue;

		if(Asura_Material_System::IsMaterialDontRender(xStrip.m_iMaterialIndex)) continue;
		if(Asura_Material_System::IsMaterialFlowingWater(xStrip.m_iMaterialIndex)) continue;
		if(Asura_Material_System::IsMaterialPS2AdditiveAlpha(xStrip.m_iMaterialIndex)) continue;
#ifdef ASURA_ALPHA_DECAL_HACK_FOR_PSP
		if (Asura_Material_System::IsMaterialAlphaDecal(xStrip.m_iMaterialIndex)) continue;
#endif

		if(Asura_Material_System::IsMaterialTransparent(xStrip.m_iMaterialIndex))
		{
			int iTexture = Asura_Material_System::GetTextureID(xStrip.m_iMaterialIndex);

			// DMR (21/11/05) - Handle alpha-test without blending, essential for vegetation
			if (Asura_Texture_Management::GetTextureFlags(iTexture) & AsuraTextureFlag_8BitAlpha)
			{
				Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_NORMAL);
				Asura_Render::CurrentStates.RequestAlphaTest(true);
			}
			else
			{
				Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_OFF);
				Asura_Render::CurrentStates.RequestAlphaTest(true, 0x80);
			}
			Asura_Render::CurrentStates.RequestTextureOpMode(ASURA_TEXTUREOP_JUSTALPHA);
			Asura_Render::CurrentStates.RequestTextureIndex(0, iTexture);

			Asura_TextureAnimationOutput xTextureAnim;
			Asura_Material_System::GetTextureAnimationState(xStrip.m_iMaterialIndex, 0, &xTextureAnim);
			float fTexAnimConstants[4] = { xTextureAnim.m_fUOffset, xTextureAnim.m_fVOffset, 0.f, 0.f };
			Asura_Direct3D::D3DDevice->SetVertexShaderConstantF(4, fTexAnimConstants, 1);
		}
		else
		{
			Asura_Render::CurrentStates.RequestAlphaTest(false);
			Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_OFF);
			Asura_Render::CurrentStates.RequestTextureOpMode(ASURA_TEXTUREOP_SINGLE_PASS);
			Asura_Render::CurrentStates.RequestTextureIndex(0,ASURA_TEXTURE_NONE);
		}

		if(xStrip.m_uVertexBufferID != uCurrentBuffer)
		{
			if(!m_ppxVertexBuffer[xStrip.m_uVertexBufferID] || !m_ppxIndexBuffer[xStrip.m_uVertexBufferID]) continue;
			Asura_Direct3D::D3DDevice->SetStreamSource(0, m_ppxVertexBuffer[xStrip.m_uVertexBufferID], 0, sizeof(Asura_Direct3D_Env_Vertex));
			Asura_Direct3D::D3DDevice->SetIndices(m_ppxIndexBuffer[xStrip.m_uVertexBufferID]);
			uCurrentBuffer = xStrip.m_uVertexBufferID;
		}

		Asura_Direct3D::D3DDevice->DrawIndexedPrimitive
			(
			s_eCurrentPrimitiveType,
			0,
			xStrip.m_uLowestVertex,
			xStrip.m_uNumberOfVertices,
			xStrip.m_uStartVertexIndex,
			xStrip.m_uNumberOfTriangles
			);

		Asura_Render::AddToTriPassesRendered(xStrip.m_uNumberOfTriangles);
	}

	D3DPERFCALL(EndEvent());
}

void Asura_Direct3D_Env_Zone::RenderDynamicLightsPass()
{
	D3DPERFCALL(BeginEvent(0xff808050, L"Dynamic Lights"));

	Asura_Render::CurrentStates.RequestAlphaTest(false);
	Asura_Render::CurrentStates.RequestZBufferEnabled(true);
	Asura_Render::CurrentStates.RequestZBufferWriteEnabled(false);
	Asura_Render::CurrentStates.RequestZBufferCompareMode(ASURA_ZBUFFERCMP_EQUALS);

	Asura_Render::CurrentStates.RequestTextureIndex(0,s_iAttenuationMap);
	Asura_Render::CurrentStates.RequestTextureIndex(1,s_iAttenuationMap);

	Asura_Direct3D::D3DDevice->SetSamplerState(0, D3DSAMP_ADDRESSU, D3DTADDRESS_CLAMP );
	Asura_Direct3D::D3DDevice->SetSamplerState(0, D3DSAMP_ADDRESSV, D3DTADDRESS_CLAMP );
	Asura_Direct3D::D3DDevice->SetSamplerState(0, D3DSAMP_MIPFILTER, D3DTEXF_NONE );

	Asura_Direct3D::D3DDevice->SetSamplerState(1, D3DSAMP_ADDRESSU, D3DTADDRESS_CLAMP );
	Asura_Direct3D::D3DDevice->SetSamplerState(1, D3DSAMP_ADDRESSV, D3DTADDRESS_CLAMP );
	Asura_Direct3D::D3DDevice->SetSamplerState(1, D3DSAMP_MIPFILTER, D3DTEXF_NONE );

	Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_GLOWING);
	Asura_Direct3D_Shaders::RequestVertexShader(ASURA_D3D_VS_ENVIRONMENT_LIGHTING);
	Asura_Render::CurrentStates.RequestTextureOpMode(ASURA_TEXTUREOP_LIGHTPASS);

	float afConstants[4] = { 0.f, 0.f, 0.f, 0.f };
	Asura_Direct3D::D3DDevice->SetVertexShaderConstantF(18, &afConstants[0], 1);

	D3DXMATRIX matInvView, matTransposedView;
	Asura_Direct3D::D3DDevice->GetTransform( D3DTS_VIEW, &matInvView );
	D3DXMatrixTranspose(&matTransposedView, &matInvView);
	Asura_Direct3D::D3DDevice->SetVertexShaderConstantF(4, matTransposedView, 4);
	D3DXMatrixInverse( &matInvView,  NULL, &matInvView );

	D3DXMATRIX matProjectXZ;
	D3DXMATRIX matProjectY;

	u_int uCurrentBuffer = (u_int)-1;

	for(u_int uModuleIndex = 0; uModuleIndex < m_uNumberOfMeshes; ++uModuleIndex)
	{
		if(!Asura_Module_System::IsModuleVisible(uModuleIndex)) continue;

		// Collect lights for this module
		const Asura_CollisionMesh* pxCollisionMesh = Asura_Module_System::GetCollisionMeshForModule(uModuleIndex);
		Asura_Bounding_Box xBB = pxCollisionMesh->GetBoundingBox();
		xBB.Displace(Asura_Module_System::GetPositionOfModule(uModuleIndex));
		Asura_Photon_System::MakePhotonListForBoundingBox( xBB, Asura_Module_System::GetCentreOfModule(uModuleIndex), false, true );

		if(!Asura_Photon_System::NumberOfLocalPhotons) continue;

		if(uCurrentBuffer != m_pxMeshes[uModuleIndex].m_uVertexBufferID)
		{
			if(!m_ppxVertexBuffer[m_pxMeshes[uModuleIndex].m_uVertexBufferID] || !m_ppxIndexBuffer[m_pxMeshes[uModuleIndex].m_uVertexBufferID]) continue;
			Asura_Direct3D::D3DDevice->SetStreamSource(0, m_ppxVertexBuffer[m_pxMeshes[uModuleIndex].m_uVertexBufferID], 0, sizeof(Asura_Direct3D_Env_Vertex));
			Asura_Direct3D::D3DDevice->SetIndices(m_ppxIndexBuffer[m_pxMeshes[uModuleIndex].m_uVertexBufferID]);
			uCurrentBuffer = m_pxMeshes[uModuleIndex].m_uVertexBufferID;
		}

		for(u_int uPhoton = 0; uPhoton < Asura_Photon_System::NumberOfLocalPhotons; ++uPhoton)
		{
			Asura_Local_Photon* photon = &Asura_Photon_System::LocalPhotons[uPhoton];
			const Asura_Vector_3& xPosition = photon->GetPosition();

			float afConstants[2][4] = { xPosition.x, xPosition.y, xPosition.z, 1.0f,
				0.5f*photon->GetRedComponent(), 0.5f*photon->GetGreenComponent(), 0.5f*photon->GetBlueComponent(), 1.0f };
			Asura_Direct3D::D3DDevice->SetVertexShaderConstantF(16, &afConstants[0][0], 2);

			// Build projection matrices for this light
			ZeroMemory(&matProjectXZ, sizeof(D3DXMATRIX)); 
			matProjectXZ._32 = matProjectXZ._11 = 1.0f/(2.0f*photon->GetRange());
			matProjectXZ._41 = 0.5f - matProjectXZ._11*xPosition.x;
			matProjectXZ._42 = 0.5f - matProjectXZ._11*xPosition.z;
			D3DXMatrixMultiplyTranspose(&matProjectXZ, &matInvView, &matProjectXZ);

			ZeroMemory(&matProjectY, sizeof(D3DXMATRIX)); 
			matProjectY._21 = 1.0f/(2.0f*photon->GetRange());
			matProjectY._41 = 0.5f - matProjectY._21*xPosition.y;
			matProjectY._42 = 0.5f;
			D3DXMatrixMultiplyTranspose(&matProjectY, &matInvView, &matProjectY);

			// Upload them to the shader
			Asura_Direct3D::D3DDevice->SetVertexShaderConstantF(8, matProjectXZ, 4);
			Asura_Direct3D::D3DDevice->SetVertexShaderConstantF(12, matProjectY, 4);

			// Render this module's strips
			for(u_int uStrip = 0; uStrip < m_pxMeshes[uModuleIndex].m_uNumberOfStrips; ++uStrip)
			{
				Asura_Direct3D_Env_Strip& xStrip = m_pxStrips[uStrip + m_pxMeshes[uModuleIndex].m_uStartStripIndex];

				if(Asura_Material_System::IsMaterialDontRender(xStrip.m_iMaterialIndex)) continue;

#ifdef ASURA_ALPHA_DECAL_HACK_FOR_PSP
				if (Asura_Material_System::IsMaterialAlphaDecal(xStrip.m_iMaterialIndex)) continue;
#endif
				if (Asura_Material_System::IsMaterialFlowingWater(xStrip.m_iMaterialIndex)) continue;
				if (Asura_Material_System::IsMaterialPS2AdditiveAlpha(xStrip.m_iMaterialIndex)) continue;

				Asura_Direct3D::D3DDevice->DrawIndexedPrimitive
					(
					s_eCurrentPrimitiveType,
					0,
					xStrip.m_uLowestVertex,
					xStrip.m_uNumberOfVertices,
					xStrip.m_uStartVertexIndex,
					xStrip.m_uNumberOfTriangles
					);

				Asura_Render::AddToTriPassesRendered(xStrip.m_uNumberOfTriangles);
			}
		}
	}

	// Reset some state
	Asura_Direct3D::D3DDevice->SetSamplerState(0, D3DSAMP_ADDRESSU, D3DTADDRESS_WRAP );
	Asura_Direct3D::D3DDevice->SetSamplerState(0, D3DSAMP_ADDRESSV, D3DTADDRESS_WRAP );
	Asura_Direct3D::D3DDevice->SetSamplerState(1, D3DSAMP_ADDRESSU, D3DTADDRESS_WRAP );
	Asura_Direct3D::D3DDevice->SetSamplerState(1, D3DSAMP_ADDRESSV, D3DTADDRESS_WRAP );

	if (Asura_Direct3D_UserOptions::IsTrilinearFilteringOn())
	{
		Asura_Direct3D::D3DDevice->SetSamplerState(0,D3DSAMP_MIPFILTER,D3DTEXF_LINEAR);
		Asura_Direct3D::D3DDevice->SetSamplerState(1,D3DSAMP_MIPFILTER,D3DTEXF_LINEAR);
	}
	else
	{
		Asura_Direct3D::D3DDevice->SetSamplerState(0,D3DSAMP_MIPFILTER,D3DTEXF_POINT);
		Asura_Direct3D::D3DDevice->SetSamplerState(1,D3DSAMP_MIPFILTER,D3DTEXF_POINT);
	}

	D3DPERFCALL(EndEvent());
}

void Asura_Direct3D_Env_Zone::RenderBasicTextures()
{
	D3DPERFCALL(BeginEvent(0xffff00ff, L"Basic Textures"));

	Asura_Direct3D_Shaders::RequestVertexShader(ASURA_D3D_VS_ENVIRONMENT_TEXTURES);
	Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_DOUBLECOLOUR);

	D3DXMATRIX xScrollingTextureMatrix;
	D3DXMatrixIdentity(&xScrollingTextureMatrix);
	Asura_Direct3D::D3DDevice->SetVertexShaderConstantF(8, xScrollingTextureMatrix, 4);

	D3DXMATRIX matIdentity;
	D3DXMatrixIdentity(&matIdentity);

	Asura_Render::CurrentStates.RequestZBufferEnabled(true);
	Asura_Render::CurrentStates.RequestZBufferWriteEnabled(false);
	Asura_Render::CurrentStates.RequestZBufferCompareMode(ASURA_ZBUFFERCMP_EQUALS);

	Asura_Render::CurrentStates.RequestAlphaTest(false);

	u_int uCurrentBuffer = (u_int)-1;

	for(u_int uStripIndex = 0; uStripIndex < m_uNumberOfStrips; ++uStripIndex)
	{
		Asura_Direct3D_Env_Strip& xStrip = m_pxStrips[uStripIndex];

		if(!Asura_Module_System::IsModuleVisible(xStrip.m_iModuleIndex)) continue;

		if(Asura_Material_System::IsMaterialDontRender(xStrip.m_iMaterialIndex)) continue;

		if(Asura_Material_System::IsMaterialFlowingWater(xStrip.m_iMaterialIndex)) continue;
		if(Asura_Material_System::IsMaterialPS2AdditiveAlpha(xStrip.m_iMaterialIndex)) continue;
#ifdef ASURA_ALPHA_DECAL_HACK_FOR_PSP
		if (Asura_Material_System::IsMaterialAlphaDecal(xStrip.m_iMaterialIndex)) continue;
#endif
		//#ifdef ASURA_SUPPORT_WEATHER_SYSTEM
		//		if (Asura_Weather_System::IsRaining() && Asura_Material_System::IsMaterialRainSplashes(xStrip.m_iMaterialIndex)) continue;
		//#endif
		if (Asura_Material_System::IsMaterialUsingDetailMap(xStrip.m_iMaterialIndex)) continue;

		int iTexture = Asura_Material_System::GetTextureID(xStrip.m_iMaterialIndex);
		if (iTexture == ASURA_TEXTURE_NONE) continue;

		if(xStrip.m_uVertexBufferID != uCurrentBuffer)
		{
			if(!m_ppxVertexBuffer[xStrip.m_uVertexBufferID] || !m_ppxIndexBuffer[xStrip.m_uVertexBufferID]) continue;
			Asura_Direct3D::D3DDevice->SetStreamSource(0, m_ppxVertexBuffer[xStrip.m_uVertexBufferID], 0, sizeof(Asura_Direct3D_Env_Vertex));
			Asura_Direct3D::D3DDevice->SetIndices(m_ppxIndexBuffer[xStrip.m_uVertexBufferID]);
			uCurrentBuffer = xStrip.m_uVertexBufferID;
		}

		Asura_Render::CurrentStates.RequestTextureIndex(0, iTexture);

		if( Asura_Material_System::IsMaterialDualTexture( xStrip.m_iMaterialIndex ) )
		{
			int iDualTextureID = Asura_Material_System::GetTextureID( xStrip.m_iMaterialIndex, 1);
			if( iDualTextureID == ASURA_TEXTURE_NONE )
			{
				iDualTextureID = iTexture;
			}
			Asura_Render::CurrentStates.RequestTextureIndex( 1, iDualTextureID );
			Asura_Render::CurrentStates.RequestTextureOpMode( ASURA_TEXTUREOP_DUALTEXTURE );
			Asura_Direct3D::D3DDevice->SetVertexShaderConstantF(12, matIdentity, 4);
		}
		else
		{
			Asura_Render::CurrentStates.RequestTextureOpMode(ASURA_TEXTUREOP_OPAQUE);
			Asura_Direct3D::D3DDevice->SetVertexShaderConstantF(12, matIdentity, 4);
		}

		bool bAnimatedTexture = false;
		Asura_TextureAnimationOutput xTextureAnim;
		if(Asura_Material_System::GetTextureAnimationState(xStrip.m_iMaterialIndex,0,&xTextureAnim))
		{
			bAnimatedTexture = true;

			xScrollingTextureMatrix._14 = xTextureAnim.m_fUOffset;
			xScrollingTextureMatrix._24 = xTextureAnim.m_fVOffset;
		}
		//check for waterfall effect
		else if (Asura_Material_System::IsMaterialWaterfall(xStrip.m_iMaterialIndex))
		{
			bAnimatedTexture = true;
			//get the waterfall offsets
			xScrollingTextureMatrix._14 = m_fScrollTextureOffsetU;
			xScrollingTextureMatrix._24 = m_fScrollTextureOffsetV;
		}

		if (bAnimatedTexture)
		{
			Asura_Direct3D::D3DDevice->SetVertexShaderConstantF(8, xScrollingTextureMatrix, 4);
		}

		// (SB - 11/12/06) Allow project to override render state
		if (s_pfnRenderStateOverride)
		{
			s_pfnRenderStateOverride();
		}

		Asura_Direct3D::D3DDevice->DrawIndexedPrimitive
			(
			s_eCurrentPrimitiveType,
			0,
			xStrip.m_uLowestVertex,
			xStrip.m_uNumberOfVertices,
			xStrip.m_uStartVertexIndex,
			xStrip.m_uNumberOfTriangles
			);		

		if (bAnimatedTexture)
		{
			Asura_Direct3D::D3DDevice->SetVertexShaderConstantF(8, matIdentity, 4);
		}

	}

	D3DPERFCALL(EndEvent());
}

void Asura_Direct3D_Env_Zone::RenderRainAndDetailTextures()
{
	D3DPERFCALL(BeginEvent(0xffff00ff, L"Rain splashes and detail maps"));

	Asura_Direct3D_Shaders::RequestVertexShader(ASURA_D3D_VS_ENVIRONMENT_TEXTURE_DUPLICATECOORDS);
	Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_DOUBLECOLOUR);

	D3DXMATRIX xRainMat;
	D3DXMatrixIdentity(&xRainMat);
	//xRainMat._11 = xRainMat._22 = 2.0f;
	xRainMat._14 = (Asura_Timers::GetGameTimeScale() == 0.0f) ? 0.0f : Asura_Random::GetFloat()*5.0f;	
	xRainMat._24 = (Asura_Timers::GetGameTimeScale() == 0.0f) ? 0.0f : Asura_Random::GetFloat()*5.0f;

	D3DXMATRIX xDetailMat;
	D3DXMatrixIdentity(&xDetailMat);
	xDetailMat._11 = xDetailMat._22 = fASURA_DETAIL_MAP_SCALE;

	D3DXMATRIX xScrollingTextureMatrix;
	D3DXMatrixIdentity(&xScrollingTextureMatrix);
	Asura_Direct3D::D3DDevice->SetVertexShaderConstantF(8, xScrollingTextureMatrix, 4);

	D3DXMATRIX matIdentity;
	D3DXMatrixIdentity(&matIdentity);

	Asura_Render::CurrentStates.RequestZBufferEnabled(true);
	Asura_Render::CurrentStates.RequestZBufferWriteEnabled(false);
	Asura_Render::CurrentStates.RequestZBufferCompareMode(ASURA_ZBUFFERCMP_EQUALS);

	u_int uCurrentBuffer = (u_int)-1;

	for(u_int uStripIndex = 0; uStripIndex < m_uNumberOfStrips; ++uStripIndex)
	{
		Asura_Direct3D_Env_Strip& xStrip = m_pxStrips[uStripIndex];

		if(!Asura_Module_System::IsModuleVisible(xStrip.m_iModuleIndex)) continue;
		if(Asura_Material_System::IsMaterialDontRender(xStrip.m_iMaterialIndex)) continue;

		if(!Asura_Material_System::IsMaterialRainSplashes( xStrip.m_iMaterialIndex ) 
			&& !Asura_Material_System::IsMaterialUsingDetailMap( xStrip.m_iMaterialIndex) ) continue;

		if(Asura_Material_System::IsMaterialFlowingWater(xStrip.m_iMaterialIndex)) continue;
		if(Asura_Material_System::IsMaterialPS2AdditiveAlpha(xStrip.m_iMaterialIndex)) continue;
#ifdef ASURA_ALPHA_DECAL_HACK_FOR_PSP
		if (Asura_Material_System::IsMaterialAlphaDecal(xStrip.m_iMaterialIndex)) continue;
#endif

		int iTexture = Asura_Material_System::GetTextureID(xStrip.m_iMaterialIndex);
		if (iTexture == ASURA_TEXTURE_NONE) continue;

		if(xStrip.m_uVertexBufferID != uCurrentBuffer)
		{
			if(!m_ppxVertexBuffer[xStrip.m_uVertexBufferID] || !m_ppxIndexBuffer[xStrip.m_uVertexBufferID]) continue;
			Asura_Direct3D::D3DDevice->SetStreamSource(0, m_ppxVertexBuffer[xStrip.m_uVertexBufferID], 0, sizeof(Asura_Direct3D_Env_Vertex));
			Asura_Direct3D::D3DDevice->SetIndices(m_ppxIndexBuffer[xStrip.m_uVertexBufferID]);
			uCurrentBuffer = xStrip.m_uVertexBufferID;
		}

		Asura_Render::CurrentStates.RequestTextureIndex(0, iTexture);

		//		#ifdef ASURA_SUPPORT_WEATHER_SYSTEM
		//		if (Asura_Weather_System::IsRaining() && Asura_Material_System::IsMaterialRainSplashes(xStrip.m_iMaterialIndex))
		//		{				
		//			Asura_Render::CurrentStates.RequestTextureIndex(1,s_iRainSplashTexture);
		//			Asura_Direct3D::D3DDevice->SetVertexShaderConstantF(12, xRainMat, 4);
		//			Asura_Render::CurrentStates.RequestTextureOpMode(ASURA_TEXTUREOP_PLAINTEXTURE);
		//		}
		//		else
		//		#endif
		if (Asura_Material_System::IsMaterialUsingDetailMap(xStrip.m_iMaterialIndex))
		{
			Asura_Render::CurrentStates.RequestTextureIndex(1,s_iDetailMap);
			Asura_Direct3D::D3DDevice->SetVertexShaderConstantF(12, xDetailMat, 4);
			Asura_Render::CurrentStates.RequestTextureOpMode(ASURA_TEXTUREOP_PLAINTEXTURE);
		}

		bool bAnimatedTexture = false;
		Asura_TextureAnimationOutput xTextureAnim;
		if(Asura_Material_System::GetTextureAnimationState(xStrip.m_iMaterialIndex,0,&xTextureAnim))
		{
			bAnimatedTexture = true;

			xScrollingTextureMatrix._14 = xTextureAnim.m_fUOffset;
			xScrollingTextureMatrix._24 = xTextureAnim.m_fVOffset;
		}
		//check for waterfall effect
		else if (Asura_Material_System::IsMaterialWaterfall(xStrip.m_iMaterialIndex))
		{
			bAnimatedTexture = true;
			//get the waterfall offsets
			xScrollingTextureMatrix._14 = m_fScrollTextureOffsetU;
			xScrollingTextureMatrix._24 = m_fScrollTextureOffsetV;
		}

		if (bAnimatedTexture)
		{
			Asura_Direct3D::D3DDevice->SetVertexShaderConstantF(8, xScrollingTextureMatrix, 4);
		}

		Asura_Direct3D::D3DDevice->DrawIndexedPrimitive
			(
			s_eCurrentPrimitiveType,
			0,
			xStrip.m_uLowestVertex,
			xStrip.m_uNumberOfVertices,
			xStrip.m_uStartVertexIndex,
			xStrip.m_uNumberOfTriangles
			);		

		if (bAnimatedTexture)
		{
			Asura_Direct3D::D3DDevice->SetVertexShaderConstantF(8, matIdentity, 4);
		}

	}

	D3DPERFCALL(EndEvent());
}

void Asura_Direct3D_Env_Zone::RenderSelfIlluminatingPass()
{
	D3DPERFCALL(BeginEvent(0xff507090, L"Self Illuminating pass"));

	Asura_Direct3D_Shaders::RequestVertexShader(ASURA_D3D_VS_ENVIRONMENT_TEXTURES);
	Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_GLOWING);
	Asura_Render::CurrentStates.RequestTextureOpMode(ASURA_TEXTUREOP_OPAQUE);

	D3DXMATRIX xScrollingTextureMatrix;
	D3DXMatrixIdentity(&xScrollingTextureMatrix);
	Asura_Direct3D::D3DDevice->SetVertexShaderConstantF(8, xScrollingTextureMatrix, 4);

	D3DXMATRIX matIdentity;
	D3DXMatrixIdentity(&matIdentity);

	u_int uCurrentBuffer = (u_int)-1;

	for(u_int uStripIndex = 0; uStripIndex < m_uNumberOfStrips; ++uStripIndex)
	{
		Asura_Direct3D_Env_Strip& xStrip = m_pxStrips[uStripIndex];

		if(!Asura_Module_System::IsModuleVisible(xStrip.m_iModuleIndex)) continue;
		if(Asura_Material_System::IsMaterialDontRender(xStrip.m_iMaterialIndex)) continue;

		if(!Asura_Material_System::IsMaterialSelfIlluminating(xStrip.m_iMaterialIndex)) continue;

		int iTexture = Asura_Material_System::GetTextureID(xStrip.m_iMaterialIndex);
		if (iTexture == ASURA_TEXTURE_NONE) continue;
		Asura_Render::CurrentStates.RequestTextureIndex(0, iTexture);

		if(xStrip.m_uVertexBufferID != uCurrentBuffer)
		{
			if(!m_ppxVertexBuffer[xStrip.m_uVertexBufferID] || !m_ppxIndexBuffer[xStrip.m_uVertexBufferID]) continue;
			Asura_Direct3D::D3DDevice->SetStreamSource(0, m_ppxVertexBuffer[xStrip.m_uVertexBufferID], 0, sizeof(Asura_Direct3D_Env_Vertex));
			Asura_Direct3D::D3DDevice->SetIndices(m_ppxIndexBuffer[xStrip.m_uVertexBufferID]);
			uCurrentBuffer = xStrip.m_uVertexBufferID;
		}

		bool bAnimatedTexture = false;
		Asura_TextureAnimationOutput xTextureAnim;
		if(Asura_Material_System::GetTextureAnimationState(xStrip.m_iMaterialIndex,0,&xTextureAnim))
		{
			bAnimatedTexture = true;

			xScrollingTextureMatrix._14 = xTextureAnim.m_fUOffset;
			xScrollingTextureMatrix._24 = xTextureAnim.m_fVOffset;
		}
		//check for waterfall effect
		else if (Asura_Material_System::IsMaterialWaterfall(xStrip.m_iMaterialIndex))
		{
			bAnimatedTexture = true;
			//get the waterfall offsets
			xScrollingTextureMatrix._14 = m_fScrollTextureOffsetU;
			xScrollingTextureMatrix._24 = m_fScrollTextureOffsetV;
		}

		if (bAnimatedTexture)
		{
			Asura_Direct3D::D3DDevice->SetVertexShaderConstantF(8, xScrollingTextureMatrix, 4);
		}

		Asura_Direct3D::D3DDevice->DrawIndexedPrimitive
			(
			s_eCurrentPrimitiveType,
			0,
			xStrip.m_uLowestVertex,
			xStrip.m_uNumberOfVertices,
			xStrip.m_uStartVertexIndex,
			xStrip.m_uNumberOfTriangles
			);		

		if (bAnimatedTexture)
		{
			Asura_Direct3D::D3DDevice->SetVertexShaderConstantF(8, matIdentity, 4);
		}

	}

	D3DPERFCALL(EndEvent());
}

#ifndef ASURA_TARGET_SM3

void Asura_Direct3D_Env_Zone::RenderReflectivePass()
{
	D3DPERFCALL(BeginEvent(0xff507090, L"Reflective pass"));

	Asura_Direct3D_Shaders::RequestVertexShader(ASURA_D3D_VS_ENVIRONMENT_REFLECTIVE);
	Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_GLOWING);
	Asura_Direct3D_Shaders::RequestPixelShader(ASURA_D3D_PS_ENVIRONMENT_REFLECTIVE);
	Asura_Render::CurrentStates.RequestTextureIndex( 1, Asura_Render::GetReflectanceMap());

	D3DXMATRIX xReflectiveMat;
	D3DXMATRIX matView, xTransposedView;
	Asura_Direct3D::D3DDevice->GetTransform( D3DTS_VIEW, &matView );
	D3DXMatrixTranspose( &xTransposedView, &matView );
	matView._14 = matView._24 = matView._34 = 0.0f;
	matView._41 = matView._42 = matView._43 = 0.0f;
	D3DXMatrixInverse( &matView,  NULL, &matView );

	xReflectiveMat._11 = 0.5f; xReflectiveMat._12 = 0.0f; xReflectiveMat._13 = 0.0f; xReflectiveMat._14 = 0.0f; 
	xReflectiveMat._21 = 0.0f; xReflectiveMat._22 = -0.5f; xReflectiveMat._23 = 0.0f; xReflectiveMat._24 = 0.0f; 
	xReflectiveMat._31 = 0.0f; xReflectiveMat._32 = 0.0f; xReflectiveMat._33 = 1.0f; xReflectiveMat._34 = 0.0f; 
	xReflectiveMat._41 = 0.5f; xReflectiveMat._42 = 0.5f; xReflectiveMat._43 = 0.0f; xReflectiveMat._44 = 1.0f; 

	D3DXMatrixMultiplyTranspose(&xReflectiveMat,&matView,&xReflectiveMat);

	Asura_Direct3D::D3DDevice->SetVertexShaderConstantF(4, xTransposedView, 4);
	Asura_Direct3D::D3DDevice->SetVertexShaderConstantF(9, xReflectiveMat, 4);
	float fConstants[4] = { 2.0f, 1.0f, 0.0f, 0.0f };
	Asura_Direct3D::D3DDevice->SetVertexShaderConstantF(8, fConstants, 1);

	D3DXMATRIX xScrollingTextureMatrix;
	D3DXMatrixIdentity(&xScrollingTextureMatrix);
	Asura_Direct3D::D3DDevice->SetVertexShaderConstantF(14, xScrollingTextureMatrix, 4);

	D3DXMATRIX matIdentity;
	D3DXMatrixIdentity(&matIdentity);

	u_int uCurrentBuffer = (u_int)-1;

	for(u_int uStripIndex = 0; uStripIndex < m_uNumberOfStrips; ++uStripIndex)
	{
		Asura_Direct3D_Env_Strip& xStrip = m_pxStrips[uStripIndex];

		if(!Asura_Module_System::IsModuleVisible(xStrip.m_iModuleIndex)) continue;
		if(Asura_Material_System::IsMaterialDontRender(xStrip.m_iMaterialIndex)) continue;

		if(!Asura_Material_System::IsMaterialReflective(xStrip.m_iMaterialIndex)) continue;

		int iTexture = Asura_Material_System::GetTextureID(xStrip.m_iMaterialIndex);
		if (iTexture == ASURA_TEXTURE_NONE) continue;
		Asura_Render::CurrentStates.RequestTextureIndex(0, iTexture);

		if(xStrip.m_uVertexBufferID != uCurrentBuffer)
		{
			if(!m_ppxVertexBuffer[xStrip.m_uVertexBufferID] || !m_ppxIndexBuffer[xStrip.m_uVertexBufferID]) continue;
			Asura_Direct3D::D3DDevice->SetStreamSource(0, m_ppxVertexBuffer[xStrip.m_uVertexBufferID], 0, sizeof(Asura_Direct3D_Env_Vertex));
			Asura_Direct3D::D3DDevice->SetIndices(m_ppxIndexBuffer[xStrip.m_uVertexBufferID]);
			uCurrentBuffer = xStrip.m_uVertexBufferID;
		}

		bool bAnimatedTexture = false;
		Asura_TextureAnimationOutput xTextureAnim;
		if(Asura_Material_System::GetTextureAnimationState(xStrip.m_iMaterialIndex,0,&xTextureAnim))
		{
			bAnimatedTexture = true;

			xScrollingTextureMatrix._14 = xTextureAnim.m_fUOffset;
			xScrollingTextureMatrix._24 = xTextureAnim.m_fVOffset;
		}
		//check for waterfall effect
		else if (Asura_Material_System::IsMaterialWaterfall(xStrip.m_iMaterialIndex))
		{
			bAnimatedTexture = true;
			//get the waterfall offsets
			xScrollingTextureMatrix._14 = m_fScrollTextureOffsetU;
			xScrollingTextureMatrix._24 = m_fScrollTextureOffsetV;
		}

		if (bAnimatedTexture)
		{
			Asura_Direct3D::D3DDevice->SetVertexShaderConstantF(14, xScrollingTextureMatrix, 4);
		}

		Asura_Direct3D::D3DDevice->DrawIndexedPrimitive
			(
			s_eCurrentPrimitiveType,
			0,
			xStrip.m_uLowestVertex,
			xStrip.m_uNumberOfVertices,
			xStrip.m_uStartVertexIndex,
			xStrip.m_uNumberOfTriangles
			);		

		if (bAnimatedTexture)
		{
			Asura_Direct3D::D3DDevice->SetVertexShaderConstantF(14, matIdentity, 4);
		}

	}

	Asura_Direct3D_Shaders::RequestPixelShader(ASURA_D3D_PS_NONE);

	D3DPERFCALL(EndEvent());
}
#endif //#ifndef ASURA_TARGET_SM3


#ifdef ASURA_ALPHA_DECAL_HACK_FOR_PSP
void Asura_Direct3D_Env_Zone::RenderAlphaDecals()
{
	D3DPERFCALL(BeginEvent(0xffdf0000, L"Alpha decals"));

	Asura_Direct3D_Shaders::RequestVertexShader(ASURA_D3D_VS_ENVIRONMENT_TEXTURES);
	Asura_Render::CurrentStates.RequestAlphaTest(true);
	Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_NORMAL);
	Asura_Render::CurrentStates.RequestTextureOpMode(ASURA_TEXTUREOP_SINGLE_PASS_2X);

	Asura_Render::CurrentStates.RequestZBufferEnabled(true);
	Asura_Render::CurrentStates.RequestZBufferCompareMode(ASURA_ZBUFFERCMP_LESSEQUAL);
	Asura_Render::CurrentStates.RequestZBufferWriteEnabled(false);

	u_int uCurrentBuffer = (u_int)-1;

	for(u_int uStripIndex = 0; uStripIndex < m_uNumberOfStrips; ++uStripIndex)
	{
		Asura_Direct3D_Env_Strip& xStrip = m_pxStrips[uStripIndex];

		if(!Asura_Module_System::IsModuleVisible(xStrip.m_iModuleIndex)) continue;
		if(Asura_Material_System::IsMaterialDontRender(xStrip.m_iMaterialIndex)) continue;

		if(!Asura_Material_System::IsMaterialAlphaDecal(xStrip.m_iMaterialIndex)) continue;

		if(xStrip.m_uVertexBufferID != uCurrentBuffer)
		{
			if(!m_ppxVertexBuffer[xStrip.m_uVertexBufferID] || !m_ppxIndexBuffer[xStrip.m_uVertexBufferID]) continue;
			Asura_Direct3D::D3DDevice->SetStreamSource(0, m_ppxVertexBuffer[xStrip.m_uVertexBufferID], 0, sizeof(Asura_Direct3D_Env_Vertex));
			Asura_Direct3D::D3DDevice->SetIndices(m_ppxIndexBuffer[xStrip.m_uVertexBufferID]);
			uCurrentBuffer = xStrip.m_uVertexBufferID;
		}

		Asura_Render::CurrentStates.RequestTextureIndex(0,Asura_Material_System::GetTextureID(xStrip.m_iMaterialIndex));

		Asura_Direct3D::D3DDevice->DrawIndexedPrimitive
			(
			s_eCurrentPrimitiveType,
			0,
			xStrip.m_uLowestVertex,
			xStrip.m_uNumberOfVertices,
			xStrip.m_uStartVertexIndex,
			xStrip.m_uNumberOfTriangles
			);
	}
	D3DPERFCALL(EndEvent());
}
#endif

void Asura_Direct3D_Env_Zone::RenderFlowingWater()
{
	D3DPERFCALL(BeginEvent(0xffdf0000, L"Flowing water"));

	// (AAM - 22.04.2004) Now render the water and scrolling strips in all the modules
	// (AAM - 26.04.2004) Vertex shader constants for the flowing water shader
	const float afC4[4] = { m_fWaterTextureOffsetU, m_fWaterTextureOffsetV, 0.0f, 0.0f };
	const float afC7[4] = { fWATER_AMPLITUDE_U * Asura_Maths::Sine(m_fWaterPhaseU),
		fWATER_AMPLITUDE_V * Asura_Maths::Cosine(m_fWaterPhaseV),
		8.0f * Asura_Maths::Pi,
		0.0f };
	const float afC10[4] = { Asura_Maths::Pi, 0.5f, 2.0f * Asura_Maths::Pi, 1.0f / (2.0f * Asura_Maths::Pi) };
	const float afC11[4] = { 1.0f, -0.5f,             1.0f / (4.f*3.f*2.f),     -1.0f / (6.f*5.f*4.f*3.f*2.f)     };
	const float afC12[4] = { 1.0f, -1.0f / (3.f*2.f), 1.0f / (5.f*4.f*3.f*2.f), -1.0f / (7.f*6.f*5.f*4.f*3.f*2.f) };

	// These must be uploaded now. Asura's shader loader doesn't process the constants buffer so we can't def constants
	// inside shaders
	Asura_Direct3D::D3DDevice->SetVertexShaderConstantF(4, afC4, 1);
	Asura_Direct3D::D3DDevice->SetVertexShaderConstantF(7, afC7, 1);
	Asura_Direct3D::D3DDevice->SetVertexShaderConstantF(10, afC10, 1);
	Asura_Direct3D::D3DDevice->SetVertexShaderConstantF(11, afC11, 1);
	Asura_Direct3D::D3DDevice->SetVertexShaderConstantF(12, afC12, 1);

	if (Asura_Direct3D_UserOptions::IsTrilinearFilteringOn())
	{
		Asura_Direct3D::D3DDevice->SetSamplerState(0, D3DSAMP_MIPFILTER, D3DTEXF_LINEAR);
	}

	else
	{
		Asura_Direct3D::D3DDevice->SetSamplerState(0, D3DSAMP_MIPFILTER, D3DTEXF_POINT);
	}

	Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_NORMAL);

	Asura_Render::CurrentStates.RequestZBufferEnabled(true);
	Asura_Render::CurrentStates.RequestZBufferCompareMode(ASURA_ZBUFFERCMP_LESSEQUAL);
	Asura_Render::CurrentStates.RequestZBufferWriteEnabled(false);

	Asura_Render::CurrentStates.RequestTextureOpMode(ASURA_TEXTUREOP_OPAQUE);
	Asura_Direct3D_Shaders::RequestVertexShader(ASURA_D3D_VS_FLOWINGWATER);

	u_int uCurrentBuffer = (u_int)-1;

	for(u_int uStripIndex = 0; uStripIndex < m_uNumberOfStrips; ++uStripIndex)
	{
		Asura_Direct3D_Env_Strip& xStrip = m_pxStrips[uStripIndex];

		if(!Asura_Module_System::IsModuleVisible(xStrip.m_iModuleIndex)) continue;
		if(Asura_Material_System::IsMaterialDontRender(xStrip.m_iMaterialIndex)) continue;

		if(!Asura_Material_System::IsMaterialFlowingWater(xStrip.m_iMaterialIndex)) continue;

		if(xStrip.m_uVertexBufferID != uCurrentBuffer)
		{
			if(!m_ppxVertexBuffer[xStrip.m_uVertexBufferID] || !m_ppxIndexBuffer[xStrip.m_uVertexBufferID]) continue;
			Asura_Direct3D::D3DDevice->SetStreamSource(0, m_ppxVertexBuffer[xStrip.m_uVertexBufferID], 0, sizeof(Asura_Direct3D_Env_Vertex));
			Asura_Direct3D::D3DDevice->SetIndices(m_ppxIndexBuffer[xStrip.m_uVertexBufferID]);
			uCurrentBuffer = xStrip.m_uVertexBufferID;
		}

		Asura_Render::CurrentStates.RequestTextureIndex(0,Asura_Material_System::GetTextureID(xStrip.m_iMaterialIndex));

		Asura_Direct3D::D3DDevice->DrawIndexedPrimitive
			(
			s_eCurrentPrimitiveType,
			0,
			xStrip.m_uLowestVertex,
			xStrip.m_uNumberOfVertices,
			xStrip.m_uStartVertexIndex,
			xStrip.m_uNumberOfTriangles
			);
	}
	D3DPERFCALL(EndEvent());
}

void Asura_Direct3D_Env_Zone::RenderFog()
{
	D3DPERFCALL(BeginEvent(0xffdf0000, L"Fog pass"));

	Asura_Render::CurrentStates.RequestZBufferWriteEnabled(false);
	Asura_Render::CurrentStates.RequestZBufferCompareMode(ASURA_ZBUFFERCMP_EQUALS);

	Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_NORMAL);
	Asura_Render::CurrentStates.RequestTextureOpMode(ASURA_TEXTUREOP_WHITE);

	Asura_Render::CurrentStates.RequestTextureIndex(0, ASURA_TEXTURE_NONE);

	Asura_Direct3D::D3DDevice->SetRenderState(D3DRS_TEXTUREFACTOR, Asura_Fog::GetFogColourAsUint());

	if(Asura_Fog::IsGraphFogEnabled())
	{
		Asura_Direct3D_Shaders::RequestVertexShader(ASURA_D3D_VS_ENVIRONMENT_GRAPHFOG);

		Asura_Vector_2 xCenterpoint = Asura_Fog::GetGraphFogPoint();

		float fShaderConstants[7][4];
		fShaderConstants[0][0] = fShaderConstants[0][1] = fShaderConstants[0][2] = fShaderConstants[0][3] = 1.0f;

		fShaderConstants[1][0] = Asura_Fog::GetOneOverFogPlaneSeparation();
		fShaderConstants[1][1] = -Asura_Fog::GetFogNearPlane() * Asura_Fog::GetOneOverFogPlaneSeparation();
		fShaderConstants[1][2] = xCenterpoint.x;
		fShaderConstants[1][3] = 1.0f;

		fShaderConstants[2][0] = 1.0f / xCenterpoint.x;
		fShaderConstants[2][1] = 0.0f;
		fShaderConstants[2][2] = 0.0f;
		fShaderConstants[2][3] = 0.0f;

		fShaderConstants[3][0] = 1.0f / (1.0f - xCenterpoint.x);
		fShaderConstants[3][1] = -xCenterpoint.x / (1.0f - xCenterpoint.x);
		fShaderConstants[3][2] = 0.0f;
		fShaderConstants[3][3] = 0.0f;

		fShaderConstants[4][0] = -xCenterpoint.y + Asura_Fog::GetFogValueAtFarPlane() *  0.5f;
		fShaderConstants[4][1] =  xCenterpoint.y + Asura_Fog::GetFogValueAtFarPlane() * -0.5f;
		fShaderConstants[4][2] =  xCenterpoint.y;
		fShaderConstants[4][3] =  0.f;

		fShaderConstants[5][0] =  xCenterpoint.y		 + Asura_Fog::GetFogValueAtFarPlane() * -0.5f;
		fShaderConstants[5][1] =  xCenterpoint.y * -2.0f + Asura_Fog::GetFogValueAtFarPlane();
		fShaderConstants[5][2] =  0.5f * Asura_Fog::GetFogValueAtFarPlane();
		fShaderConstants[5][3] =  xCenterpoint.y;

		fShaderConstants[6][0] = 0.0f;
		fShaderConstants[6][1] = 1.0f;
		fShaderConstants[6][2] = 0.0f;
		fShaderConstants[6][3] = 0.0f;

		Asura_Direct3D::D3DDevice->SetVertexShaderConstantF(4, &fShaderConstants[0][0], 7);

	}
	else
	{
		Asura_Direct3D_Shaders::RequestVertexShader(ASURA_D3D_VS_ENVIRONMENT_LINEARFOG);

		float fShaderConstants[2][4];
		fShaderConstants[0][0] = fShaderConstants[0][1] = fShaderConstants[0][2] = fShaderConstants[0][3] = 1.0f;
		fShaderConstants[1][0] = Asura_Fog::GetOneOverFogPlaneSeparation() * Asura_Fog::GetFogValueAtFarPlane();
		fShaderConstants[1][1] = -Asura_Fog::GetFogNearPlane() * Asura_Fog::GetOneOverFogPlaneSeparation() * Asura_Fog::GetFogValueAtFarPlane();
		fShaderConstants[1][2] = 0.0f;
		fShaderConstants[1][3] = Asura_Fog::GetFogValueAtFarPlane();

		// We shouldn't need to read these constants back so we can use SetVSCFast
		// Constants 0-3 have already been set to viewproj matrix
		Asura_Direct3D::D3DDevice->SetVertexShaderConstantF(4, &fShaderConstants[0][0], 2);
	} // if(Asura_Fog::IsGraphFogEnabled())

	u_int uCurrentBuffer = (u_int)-1;

	for(u_int uStripIndex = 0; uStripIndex < m_uNumberOfStrips; ++uStripIndex)
	{
		Asura_Direct3D_Env_Strip& xStrip = m_pxStrips[uStripIndex];

		if(!Asura_Module_System::IsModuleVisible(xStrip.m_iModuleIndex)) continue;
		if(Asura_Material_System::IsMaterialDontRender(xStrip.m_iMaterialIndex)) continue;

		if(xStrip.m_uVertexBufferID != uCurrentBuffer)
		{
			if(!m_ppxVertexBuffer[xStrip.m_uVertexBufferID] || !m_ppxIndexBuffer[xStrip.m_uVertexBufferID]) continue;
			Asura_Direct3D::D3DDevice->SetStreamSource(0, m_ppxVertexBuffer[xStrip.m_uVertexBufferID], 0, sizeof(Asura_Direct3D_Env_Vertex));
			Asura_Direct3D::D3DDevice->SetIndices(m_ppxIndexBuffer[xStrip.m_uVertexBufferID]);
			uCurrentBuffer = xStrip.m_uVertexBufferID;
		}

		Asura_Direct3D::D3DDevice->DrawIndexedPrimitive
			(
			s_eCurrentPrimitiveType,
			0,
			xStrip.m_uLowestVertex,
			xStrip.m_uNumberOfVertices,
			xStrip.m_uStartVertexIndex,
			xStrip.m_uNumberOfTriangles
			);
	}
	D3DPERFCALL(EndEvent());
}

void Asura_Direct3D_Env_Zone::RenderAdditiveAlpha()
{
	D3DPERFCALL(BeginEvent(0xffdf0000, L"Additive Alpha"));

	Asura_Direct3D_Shaders::RequestVertexShader(ASURA_D3D_VS_ENVIRONMENT_TEXTURES);
	Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_GLOWING);
	Asura_Render::CurrentStates.RequestTextureOpMode(ASURA_TEXTUREOP_SINGLE_PASS_2X);
	Asura_Render::CurrentStates.RequestZBufferWriteEnabled(false);
	Asura_Render::CurrentStates.RequestZBufferEnabled(true);
	Asura_Render::CurrentStates.RequestZBufferCompareMode(ASURA_ZBUFFERCMP_LESSEQUAL);
	D3DXMATRIX matIdentity;
	D3DXMatrixIdentity(&matIdentity);
	Asura_Direct3D::D3DDevice->SetVertexShaderConstantF(8, matIdentity, 4);

	D3DXMATRIX xScrollingTextureMatrix;
	D3DXMatrixIdentity(&xScrollingTextureMatrix);

	u_int uCurrentBuffer = (u_int)-1;

	for(u_int uStripIndex = 0; uStripIndex < m_uNumberOfStrips; ++uStripIndex)
	{
		Asura_Direct3D_Env_Strip& xStrip = m_pxStrips[uStripIndex];

		if(!Asura_Module_System::IsModuleVisible(xStrip.m_iModuleIndex)) continue;
		if(Asura_Material_System::IsMaterialDontRender(xStrip.m_iMaterialIndex)) continue;

		if(!Asura_Material_System::IsMaterialPS2AdditiveAlpha(xStrip.m_iMaterialIndex)) continue;

		Asura_Render::CurrentStates.RequestTextureIndex(0, Asura_Material_System::GetTextureID(xStrip.m_iMaterialIndex));

		if(xStrip.m_uVertexBufferID != uCurrentBuffer)
		{
			if(!m_ppxVertexBuffer[xStrip.m_uVertexBufferID] || !m_ppxIndexBuffer[xStrip.m_uVertexBufferID]) continue;
			Asura_Direct3D::D3DDevice->SetStreamSource(0, m_ppxVertexBuffer[xStrip.m_uVertexBufferID], 0, sizeof(Asura_Direct3D_Env_Vertex));
			Asura_Direct3D::D3DDevice->SetIndices(m_ppxIndexBuffer[xStrip.m_uVertexBufferID]);
			uCurrentBuffer = xStrip.m_uVertexBufferID;
		}

		bool bAnimatedTexture = false;
		Asura_TextureAnimationOutput xTextureAnim;
		if(Asura_Material_System::GetTextureAnimationState(xStrip.m_iMaterialIndex,0,&xTextureAnim))
		{
			bAnimatedTexture = true;

			xScrollingTextureMatrix._14 = xTextureAnim.m_fUOffset;
			xScrollingTextureMatrix._24 = xTextureAnim.m_fVOffset;
		}
		//check for waterfall effect
		else if (Asura_Material_System::IsMaterialWaterfall(xStrip.m_iMaterialIndex))
		{
			bAnimatedTexture = true;
			//get the waterfall offsets
			xScrollingTextureMatrix._14 = m_fScrollTextureOffsetU;
			xScrollingTextureMatrix._24 = m_fScrollTextureOffsetV;
		}

		if (bAnimatedTexture)
		{
			Asura_Direct3D::D3DDevice->SetVertexShaderConstantF(8, xScrollingTextureMatrix, 4);
		}


		Asura_Direct3D::D3DDevice->DrawIndexedPrimitive
			(
			s_eCurrentPrimitiveType,
			0,
			xStrip.m_uLowestVertex,
			xStrip.m_uNumberOfVertices,
			xStrip.m_uStartVertexIndex,
			xStrip.m_uNumberOfTriangles
			);

		if (bAnimatedTexture)
		{
			Asura_Direct3D::D3DDevice->SetVertexShaderConstantF(8, matIdentity, 4);
		}

	}
	D3DPERFCALL(EndEvent());
}

//
// UpdateWater()
//
// Parameters:		None
// Return value:	None
//
// Updates the water simulation parameters
//

void Asura_Direct3D_Env_Zone::UpdateWater()
{
	const float fTimeSlice = Asura_Timers::GetGameTimeSlice();

	m_fWaterTextureOffsetU += fWATER_TEXTURE_SPEED_U * fTimeSlice;
	m_fWaterTextureOffsetV += fWATER_TEXTURE_SPEED_V * fTimeSlice;

	// Wrap texture offsets to [0, 1] to avoid eventual overflow

	if (m_fWaterTextureOffsetU < 0.0f) m_fWaterTextureOffsetU += 1.0f;
	if (m_fWaterTextureOffsetU > 1.0f) m_fWaterTextureOffsetU -= 1.0f;

	if (m_fWaterTextureOffsetV < 0.0f) m_fWaterTextureOffsetV += 1.0f;
	if (m_fWaterTextureOffsetV > 1.0f) m_fWaterTextureOffsetV -= 1.0f;

	m_fWaterPhaseU += fWATER_WAVE_SPEED_U * fTimeSlice;
	m_fWaterPhaseV += fWATER_WAVE_SPEED_V * fTimeSlice;

	// Wrap water phase to [0, 2 pi)

	if (m_fWaterPhaseU <= 0.0f) m_fWaterPhaseU += 2.0f * Asura_Maths::Pi;
	if (m_fWaterPhaseU > 2.0f * Asura_Maths::Pi) m_fWaterPhaseU -= 2.0f * Asura_Maths::Pi;

	if (m_fWaterPhaseV <= 0.0f) m_fWaterPhaseV += 2.0f * Asura_Maths::Pi;
	if (m_fWaterPhaseV > 2.0f * Asura_Maths::Pi) m_fWaterPhaseV -= 2.0f * Asura_Maths::Pi;
}

//
// UpdateScrollingTextures()
//
// Parameters:		None
// Return value:	None
//
// Updates the scrolling textures
//

void Asura_Direct3D_Env_Zone::UpdateScrollingTextures()
{
	const float fTimeSlice = Asura_Timers::GetGameTimeSlice();

	m_fScrollTextureOffsetU += fSCROLL_TEXTURE_SPEED_U * fTimeSlice;
	m_fScrollTextureOffsetV += fSCROLL_TEXTURE_SPEED_V * fTimeSlice;

	// Wrap texture offsets to [0, 1] to avoid eventual overflow

	if (m_fScrollTextureOffsetU < 0.0f) m_fScrollTextureOffsetU += 1.0f;
	if (m_fScrollTextureOffsetU > 1.0f) m_fScrollTextureOffsetU -= 1.0f;

	if (m_fScrollTextureOffsetV < 0.0f) m_fScrollTextureOffsetV += 1.0f;
	if (m_fScrollTextureOffsetV > 1.0f) m_fScrollTextureOffsetV -= 1.0f;
}

#ifdef ASURA_TOOLS
extern Asura_Direct3D_Mesh* RenderMeshStorage[];

#include "Misc/Tools_Render.h"
#include "PC/Tools_PC_Export.h"

void Asura_Direct3D_Env_Zone::GenerateTriLists()
{
	Asura_ResourcePushType(ASURA_RESOURCE_ENVIRONMENT_MESH);
	{
		// Use the surface mesh system to generate the (lit) mesh information, and count the number of strips
		Asura_Render::SetPatchesOn(true);
		int iNumStrips = 0;
		for(int iModule = 0; iModule < Asura_Module_System::GetNumberOfModules(); iModule++)
		{
			Asura_Mesh* pxMesh = Asura_Module_System::GetMeshForModule(iModule);
			const Asura_Vector_3& xPosition = Asura_Module_System::GetPositionOfModule(iModule);
			if(!pxMesh->PrelightingDataValid && Tools_Render::LightingIsOn())
			{
				Asura_Lighting_System::CalculatePrelighting(*pxMesh, xPosition, iModule);
			}
			Asura_Surface_Mesh_System::InitMeshForSurfaceRender(*pxMesh, xPosition);
			Asura_Surface_Mesh_System::MakeVertexBufferAndIndexList(*pxMesh, xPosition);
			iNumStrips += RenderMeshStorage[pxMesh->Index]->GetNumStrips();
		}
		// Now that we've got our vertex and index buffers, copy them into our structures
		m_uNumberOfMeshes = Asura_Module_System::GetNumberOfModules();
		m_uTotalNumberOfVertexBuffers = m_uNumberOfMeshes;

		Asura_ResourceAllocateArray(m_pxMeshes, m_uNumberOfMeshes);
		Asura_ResourceAllocateArray(m_pxStrips, iNumStrips);
		m_uNumberOfStrips = iNumStrips;

		Asura_ResourceAllocateArray(m_ppxVertexBuffer, m_uNumberOfMeshes);
		Asura_ResourceAllocateArray(m_ppxIndexBuffer, m_uNumberOfMeshes);

		u_int uStripStart = 0;
		for(u_int uModule = 0; uModule < m_uNumberOfMeshes; ++uModule)
		{
			Asura_Mesh* pxMesh = Asura_Module_System::GetMeshForModule(uModule);
			int iMeshIndex = pxMesh->Index;
			
			// Fill out mesh info
			m_pxMeshes[uModule].m_uNumberOfStrips = RenderMeshStorage[iMeshIndex]->GetNumStrips();
			m_pxMeshes[uModule].m_uStartStripIndex = uStripStart;
			m_pxMeshes[uModule].m_uVertexBufferID = uModule;
			int iNumVertices = RenderMeshStorage[iMeshIndex]->GetNumUniqueVertices();
			// Use the same index buffer
			m_ppxIndexBuffer[uModule] = RenderMeshStorage[iMeshIndex]->GetIndexBuffer();
			// Check for empty modules
			if(!m_ppxIndexBuffer[uModule] || !RenderMeshStorage[iMeshIndex]->GetVertexBuffer())
			{
				m_ppxVertexBuffer[uModule] = 0;
				continue;
			}
			// Create new vertex buffer
			Asura_Direct3D::D3DDevice->CreateVertexBuffer
				(
				iNumVertices * sizeof(Asura_Direct3D_Env_Vertex),
				D3DUSAGE_WRITEONLY,
				0,
				D3DPOOL_MANAGED,
				&m_ppxVertexBuffer[uModule],
				0
				);
			// Lock vertex buffers
			D3DGenericVertex* pxReadVertices;
			Asura_Direct3D_Env_Vertex* pxWriteVertices;
			if (SUCCEEDED(RenderMeshStorage[iMeshIndex]->GetVertexBuffer()->Lock(0, iNumVertices * sizeof(D3DGenericVertex), (void**)&pxReadVertices, 0)))
			{
				if (SUCCEEDED(m_ppxVertexBuffer[uModule]->Lock(0, iNumVertices * sizeof(Asura_Direct3D_Env_Vertex), (void**)&pxWriteVertices, 0)))
				{
					// Copy vertex info into our new buffer
					for(int iVertex = 0; iVertex < iNumVertices; ++iVertex)
					{
						pxWriteVertices[iVertex].m_xPosition.x = pxReadVertices[iVertex].X;
						pxWriteVertices[iVertex].m_xPosition.y = pxReadVertices[iVertex].Y;
						pxWriteVertices[iVertex].m_xPosition.z = pxReadVertices[iVertex].Z;
						pxWriteVertices[iVertex].m_uNormal = Tools_PC_Export::CompressNormalToUByte4(pxReadVertices[iVertex].Normal);
						pxWriteVertices[iVertex].m_fU = pxReadVertices[iVertex].U;
						pxWriteVertices[iVertex].m_fV = pxReadVertices[iVertex].V;
						pxWriteVertices[iVertex].m_fU2 = pxReadVertices[iVertex].U2;
						pxWriteVertices[iVertex].m_fV2 = pxReadVertices[iVertex].V2;
						pxWriteVertices[iVertex].m_uDiffuseRGBA = pxReadVertices[iVertex].Diffuse;
					}

					// Unlock buffers
					m_ppxVertexBuffer[uModule]->Unlock();
				} else {
					Asura_Assert(0, "Failed to lock VB");
				}
				RenderMeshStorage[iMeshIndex]->GetVertexBuffer()->Unlock();
			} else {
				Asura_Assert(0, "Failed to lock VB");
			}
			for(u_int uStrip = 0; uStrip < m_pxMeshes[uModule].m_uNumberOfStrips; ++uStrip)
			{
				int iIndex = m_pxMeshes[uModule].m_uStartStripIndex + uStrip;
				const Asura_Polygon_Packet* pxStrip = RenderMeshStorage[iMeshIndex]->GetStrip(uStrip);
				// Fill out strip info
				m_pxStrips[iIndex].m_iMaterialIndex = pxStrip->m_iMaterialID;
				m_pxStrips[iIndex].m_uLowestVertex = 0;
				m_pxStrips[iIndex].m_uNumberOfTriangles = pxStrip->NumberOfIndices / 3;
				m_pxStrips[iIndex].m_uNumberOfVertices = iNumVertices;
				m_pxStrips[iIndex].m_uStartVertexIndex = pxStrip->IndexStart;
				m_pxStrips[iIndex].m_iModuleIndex = uModule;
				m_pxStrips[iIndex].m_uVertexBufferID = uModule;
			}
			uStripStart += m_pxMeshes[uModule].m_uNumberOfStrips;
		}
		// JAT 26/01/07 - mark the vertex buffer as being under our control
		m_bUsingOwnVertexBuffer = true;
		m_bUsingPrecomputedStrips = false;
		m_bUsingPrecomputedData = true;
	}
	Asura_ResourcePopType();
}
#endif


void Asura_Direct3D_Env_Zone::RenderModuleShadows(u_short usModuleIndex)
{
	const Asura_Direct3D_Env_Mesh& xMesh = m_pxMeshes[usModuleIndex];
	D3DPRIMITIVETYPE ePrimitiveType = m_bUsingPrecomputedStrips ? D3DPT_TRIANGLESTRIP : D3DPT_TRIANGLELIST;

	Asura_Direct3D::D3DDevice->SetStreamSource(0,m_ppxVertexBuffer[xMesh.m_uVertexBufferID],0,sizeof(Asura_Direct3D_Env_Vertex));
	Asura_Direct3D::D3DDevice->SetIndices(m_ppxIndexBuffer[xMesh.m_uVertexBufferID]);				

	for(u_int s=0; s<xMesh.m_uNumberOfStrips; s++)
	{
		Asura_Direct3D_Env_Strip& xStrip = m_pxStrips[s+xMesh.m_uStartStripIndex];

		if(Asura_Material_System::IsMaterialDontRender(xStrip.m_iMaterialIndex)) continue;

		Asura_Direct3D::D3DDevice->DrawIndexedPrimitive
			(
			ePrimitiveType,
			0,
			xStrip.m_uLowestVertex,
			xStrip.m_uNumberOfVertices,
			xStrip.m_uStartVertexIndex,
			xStrip.m_uNumberOfTriangles
			);
	}
}




////////////////////////////////////////////////////////////////////////////////
//
// Asura_Direct3D_EnvRenderer
//
////////////////////////////////////////////////////////////////////////////////

#if defined(ASURA_USE_STREAMING)
Asura_Direct3D_Env_Zone*						Asura_Direct3D_EnvRenderer::s_pxCurrentZone	= NULL;		// Currently active handler.
Asura_TemplateHashTree<Asura_Direct3D_Env_Zone>	Asura_Direct3D_EnvRenderer::s_xZoneTree;				// Tree of all of the handlers available.
#else
Asura_Direct3D_Env_Zone							Asura_Direct3D_EnvRenderer::s_xZone;					// Single static-instance of a zone.
#endif	// defined(ASURA_USE_STREAMING)


void Asura_Direct3D_EnvRenderer::Initialise(void)
{
	// Nothing to initialise...
}

void Asura_Direct3D_EnvRenderer::Reset(void)
{
	Shutdown();
	Initialise();
}

void Asura_Direct3D_EnvRenderer::Shutdown(void)
{
#if defined(ASURA_USE_STREAMING)
	// Reset the entire zone-tree...
	s_xZoneTree.Reset();

	// Remove the current zone...
	s_pxCurrentZone	= NULL;
#else
	// Reset the instance...
	s_xZone.Reset();
#endif	// defined(ASURA_USE_STREAMING)
}

void Asura_Direct3D_EnvRenderer::ResetZone (Asura_Hash_ID uHash, Asura_Direct3D_Env_Zone* pxZone)
{
	pxZone->Reset();
}

void Asura_Direct3D_EnvRenderer::ClearRenderMesh (void)
{
#if defined(ASURA_USE_STREAMING)
	// Reset ALL of the zones...
	s_xZoneTree.Traverse(ResetZone);
#else
	// Simply reset the zone...
	s_xZone.Reset();
#endif	// defined(ASURA_USE_STREAMING)
}

#if defined(ASURA_USE_STREAMING)

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Direct3D_EnvRenderer::CreateZone
// Author       : Andy Tate
// Date         : 14-Mar-2006
//
// Description  : Attempts to create an environment-zone with the specified 
//				  hash-ID.
//
// Parameters   : The hash-ID of the zone to create.
//
// Return value : A pointer to the zone created.
////////////////////////////////////////////////////////////////////////////////

Asura_Direct3D_Env_Zone* Asura_Direct3D_EnvRenderer::CreateZone (const Asura_Hash_ID uHash)
{
	Asura_Direct3D_Env_Zone*	pxZone	= s_xZoneTree.GetDataByHash(uHash);

	// Do we already have this zone?
	if(pxZone == NULL)
	{
		// No - create one...
		pxZone	= s_xZoneTree.AddNode(uHash);
	}
	else
	{
		// Yes - this shouldn't happen...
		Asura_Assert(false, "Already have a render-environment-zone with this hash-ID.");
	}

	// Return the zone...
	return(pxZone);
}

/// End of function 'Asura_Direct3D_EnvRenderer::CreateZone'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Direct3D_EnvRenderer::DestroyZone
// Author       : Andy Tate
// Date         : 14-Mar-2006
//
// Description  : Attempts to destroy the environment-zone with the specified
//				  hash-ID.
//
// Parameters   : The hash-ID of the zone to destroy.
//
// Return value : "true" if destroyed, "false" if failed.
////////////////////////////////////////////////////////////////////////////////

bool Asura_Direct3D_EnvRenderer::DestroyZone (const Asura_Hash_ID uHash)
{
	// Is this the current zone?
	if(s_pxCurrentZone == s_xZoneTree.GetDataByHash(uHash))
	{
		s_pxCurrentZone	= NULL;
	}

	// Call the delete function...
	return(s_xZoneTree.DeleteNodeByHash(uHash));
}

/// End of function 'Asura_Direct3D_EnvRenderer::DestroyZone'. ///

#endif	// defined(ASURA_USE_STREAMING)


void Asura_Direct3D_EnvRenderer::Render(void)
{
	// ARTTODO - Render from the current mode outwards...
	GetCurrentZone().Render();
}


void Asura_Direct3D_EnvRenderer::BeginEnvironmentShadows(const D3DMATRIX& xShadowTransform, float fBaseHeight, float fFalloffPerMetre, float initialAlpha)
{
	Asura_Direct3D_Shaders::RequestVertexShader(ASURA_D3D_VS_ENVIRONMENT_SHADOWS);

	D3DXMATRIX xMatView, xMatProj, xMat;
	Asura_Direct3D::D3DDevice->GetTransform(D3DTS_VIEW, &xMatView);
	Asura_Direct3D::D3DDevice->GetTransform(D3DTS_PROJECTION, &xMatProj);
	D3DXMatrixMultiply(&xMat, &xMatView, &xMatProj);
	D3DXMatrixTranspose(&xMat, &xMat);
	Asura_Direct3D::D3DDevice->SetVertexShaderConstantF(0, xMat, 4);

	D3DXMatrixTranspose(&xMat, reinterpret_cast<const D3DXMATRIX*>(&xShadowTransform));
	Asura_Direct3D::D3DDevice->SetVertexShaderConstantF(4, xMat, 4);

	float afConstants[8] = { -fFalloffPerMetre, 1.f + fFalloffPerMetre*fBaseHeight, -0.1f, initialAlpha,
							 1.0f, 0.0f, 0.0f, 0.0f };
	Asura_Direct3D::D3DDevice->SetVertexShaderConstantF(8, afConstants, 2);
}


void Asura_Direct3D_EnvRenderer::RenderModuleShadows (u_short usModuleIndex)
{
	// ARTTODO - Render from the current mode outwards...
	GetCurrentZone().RenderModuleShadows(usModuleIndex);
}


/// End of file. ///
