/**
*
* \file Asura_PlatformRenderObject.cpp
*
* Created by Kevin Lea - 04/07/2002 17:57:56
*
* Copyright (C) 2002 Rebellion, All Rights Reserved.
*
* \brief
*
**/

#include "Core/Asura.h"

#include "Camera/Asura_Camera.h"
#include "Core/Asura_HashTree.h"
#include "Core/Asura_MaterialResponse_System.h"
#include "Core/Asura_Stream.h"
#include "Core/Asura_TextureManagement.h"
#include "Core/Asura_Material_System.h"

#include "Animation/Asura_Animation.h"
#include "Animation/Asura_TextureAnimation.h"

#include "Mesh/Asura_Mesh_System.h"
#include "Rendering/Asura_PlatformRenderObject.h"
#include "Rendering/Asura_Render.h"
#include "Shapes/Asura_Shapes.h"
#include "Rendering/Asura_Fog.h"
#include "Rendering/Asura_Colour.h"
#include "Rendering/Asura_SortedRender.h"

#ifdef ASURA_TOOLS
#include "PC/Tools_PC_Export.h"
#endif

#include "Rendering/Asura_Direct3D_Shaders.h"
#include "Rendering/Asura_Direct3D_Caps.h"
#include "Rendering/Asura_Direct3D_UserOptions.h"
#if defined( ASURA_USE_MATERIAL_RESPONSE_SYSTEM)
#include "Rendering/Asura_Direct3D_MaterialResponse.h"
#endif
#ifdef ASURA_TARGET_SM3
#include "Rendering/Asura_Direct3D_Effect.h"
#include "Rendering/Asura_Direct3D_Effects_System.h"

#ifdef ASURA_TOOLS
#include "Misc/Tools_Render.h"
#include "Tools_System.h"
#endif
#endif //#ifdef ASURA_TARGET_SM3

#if defined(ASURA_TARGET_SM3)
#include "Shadows/Asura_Shadow_System.h"
#endif

#include "LIght/Asura_GI_Solution.h"

#include "Mesh/Asura_Mesh.h"

// KJL - 07/05/2003 18:04:12 - the hash tree is has no limits, so we should probably get rid of the 
// constraint below on the number of objects (though it's only 4 voids of memory per unit increase
// if we need to up this, since it's just the size of a pointer array)
#ifndef ASURA_MAX_NO_OF_OBJECT_STRIPS
#define ASURA_MAX_NO_OF_OBJECT_STRIPS (256)
#endif

static const float fASURA_PC_OBJECT_RENDERER_SHADOW_LOS_DISTANCE = 1000.0f; // Used to use Asura_Maths::LargeFloat. This should suffice (fixes fp-exceptions).

static const float fNORMAL_DECOMPRESS[4] = { 2.f, -1.f, 0, 1.f };

static const float fTEXCOORD_DECOMPRESS_MAX		=	10.f;
static const float fTEXCOORD_DECOMPRESS_MIN		=	-10.f;
static const float fTEXCOORD_DECOMPRESS_SCALE	=	(fTEXCOORD_DECOMPRESS_MAX - fTEXCOORD_DECOMPRESS_MIN) / 65535.f;
static const float fTEXCOORD_DECOMPRESS_OFFSET	=	((-32768.f / 65535.f) * (fTEXCOORD_DECOMPRESS_MAX - fTEXCOORD_DECOMPRESS_MIN)) - fTEXCOORD_DECOMPRESS_MIN;
static const float fTEXCOORD_DECOMPRESS[4]		=	{ fTEXCOORD_DECOMPRESS_SCALE, fTEXCOORD_DECOMPRESS_OFFSET, 0.f, 1.f };

#ifdef ASURA_TOOLS
// KJL - 07/05/2003 18:03:19 - preprocessor fn required for Tools builds - I'll probably tidy this into a tools file if it's appropriate
#include "MeshProcessing/Tools_ObjectProcessor.h"

#if defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM
const Asura_Direct3D_Object_Strip* Asura_Direct3D_ObjectRender_Handler::InitialiseStripFromRawData(Asura_Mesh* pxMesh, Asura_Hash_ID uMatResponse1, Asura_Hash_ID uMatResponse2, const Asura_Hash_ID uHashID)
#else
const Asura_Direct3D_Object_Strip* Asura_Direct3D_ObjectRender_Handler::InitialiseStripFromRawData(Asura_Mesh* pxMesh, const int iMaterialIndex, const Asura_Hash_ID uHashID)
#endif
{
	#if defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM
	Tools_ObjectProcessor::Initialise(pxMesh, uMatResponse1, uMatResponse2,false);
	#else
	Tools_ObjectProcessor::Initialise(pxMesh,iMaterialIndex,false);
	#endif

	const Tools_ObjectProcessor_Strip* pxToolsStrip = Tools_ObjectProcessor::GetProcessedStrip();

	Asura_Direct3D_Object_Strip* pxStrip = AddObjectStrip(uHashID);

	if(pxStrip)
	{

		pxStrip->m_uNumberOfTriangles = pxToolsStrip->m_uNumberOfTriangles;
		pxStrip->m_uNumberOfVertices = pxToolsStrip->m_uNumberOfVertices;
		pxStrip->m_uNumberOfIndices = pxToolsStrip->m_uNumberOfIndices;
		pxStrip->m_ePrimitiveType = D3DPT_TRIANGLELIST;

		#if defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM
			pxStrip->m_auMaterialResponseHashID[0] = uMatResponse1;
			pxStrip->m_auMaterialResponseHashID[1] = uMatResponse2;
		#else
			pxStrip->m_iMaterialIndex = iMaterialIndex;
		#endif

		int flags = D3DUSAGE_WRITEONLY;

		if (!Asura_Direct3D_Caps::HasHardwareVertexShaders())
		{
			flags |= D3DUSAGE_SOFTWAREPROCESSING;
		}

		// load in all vertex buffer data
		if(pxStrip->m_uNumberOfVertices)
		{
			// Create vertex and buffers for the strip
			Asura_Direct3D::D3DDevice->CreateVertexBuffer
				(
				pxStrip->m_uNumberOfVertices * sizeof(Asura_Direct3D_ObjectVertex),
				flags,
				0, 
				D3DPOOL_MANAGED,
				&(pxStrip->m_pxVertexBuffer),
				0
				);

			// lock, fill in data and unlock
			Asura_Direct3D_ObjectVertex* pxVertexData;
			pxStrip->m_pxVertexBuffer->Lock
				(
				0,
				sizeof(Asura_Direct3D_ObjectVertex) * pxStrip->m_uNumberOfVertices,
				(void**)&pxVertexData,
				0
				);

#if defined(ASURA_TARGET_SM3)
			for (u_int u=0; u<pxStrip->m_uNumberOfVertices; u++)
			{
				pxVertexData[u].m_xPosition.x = pxToolsStrip->m_pxVertices[u].m_fX;
				pxVertexData[u].m_xPosition.y = pxToolsStrip->m_pxVertices[u].m_fY;
				pxVertexData[u].m_xPosition.z = pxToolsStrip->m_pxVertices[u].m_fZ;

				pxVertexData[u].m_xNormal.x = pxToolsStrip->m_pxVertices[u].m_fNX;
				pxVertexData[u].m_xNormal.y = pxToolsStrip->m_pxVertices[u].m_fNY;
				pxVertexData[u].m_xNormal.z = pxToolsStrip->m_pxVertices[u].m_fNZ;

				pxVertexData[u].m_fU = pxToolsStrip->m_pxVertices[u].m_fU;
				pxVertexData[u].m_fV = pxToolsStrip->m_pxVertices[u].m_fV;

				pxVertexData[u].m_fAlpha = 1.0f;
			}
#else
			bool bOutOfRange, bAnyOutOfRange = false;

			for (u_int u=0; u<pxStrip->m_uNumberOfVertices; u++)
			{
				pxVertexData[u].m_xPosition.x = pxToolsStrip->m_pxVertices[u].m_fX;
				pxVertexData[u].m_xPosition.y = pxToolsStrip->m_pxVertices[u].m_fY;
				pxVertexData[u].m_xPosition.z = pxToolsStrip->m_pxVertices[u].m_fZ;

				pxVertexData[u].m_uNormal = Tools_PC_Export::CompressNormalToUByte4(Asura_Vector_3(pxToolsStrip->m_pxVertices[u].m_fNX,
					pxToolsStrip->m_pxVertices[u].m_fNY,
					pxToolsStrip->m_pxVertices[u].m_fNZ));

				pxVertexData[u].m_sU = Tools_PC_Export::CompressTexCoordToShort(pxToolsStrip->m_pxVertices[u].m_fU, bOutOfRange);
				bAnyOutOfRange |= bOutOfRange;
				pxVertexData[u].m_sV = Tools_PC_Export::CompressTexCoordToShort(pxToolsStrip->m_pxVertices[u].m_fV, bOutOfRange);
				bAnyOutOfRange |= bOutOfRange;
			}

			Asura_Assertf(!bAnyOutOfRange, ("Some texcoords on mesh %s were out of range and have been clamped.", pxMesh->Name));
#endif	// defined(ASURA_TARGET_SM3)

			pxStrip->m_pxVertexBuffer->Unlock();
		}

		// load in all index buffer data
		if(pxStrip->m_uNumberOfIndices)
		{
			Asura_Direct3D::D3DDevice->CreateIndexBuffer
				(
				pxStrip->m_uNumberOfIndices * sizeof(unsigned short),
				flags,
				D3DFMT_INDEX16,
				D3DPOOL_MANAGED,
				&(pxStrip->m_pxIndexBuffer),
				0
				);

			unsigned short* pusIndexData;
			pxStrip->m_pxIndexBuffer->Lock
				(
				0,
				sizeof(u_short) * pxStrip->m_uNumberOfIndices, 
				(void**)&pusIndexData,
				0
				);

			//	Asura_FileAccess::ReadData(hFile, pusIndexData, sizeof(u_short) * pxStrip->m_uNumberOfIndices);


			for (u_int u=0; u<pxStrip->m_uNumberOfIndices; u++)
			{
				pusIndexData[u] = pxToolsStrip->m_pusIndices[u];
			}

			pxStrip->m_pxIndexBuffer->Unlock();
		}
	}

	Tools_ObjectProcessor::Shutdown();

	if(pxStrip)
	{
		//Link up LOD models. RAR 05/09/03
		pxStrip->m_fMinDistance = pxMesh->m_fMinDistance;
		if(pxMesh->m_pxNextLowerMesh)
		{
			pxStrip->m_pxNextLowerStrip = GetObjectStrip(pxMesh->m_pxNextLowerMesh);
		}
	}

	return pxStrip;
}

#ifdef ASURA_TARGET_SM3
#include "MeshProcessing/Tools_ObjectProcessor_NormalMapped.h"

const Asura_Direct3D_Object_Strip* Asura_Direct3D_ObjectRender_Handler::InitialiseNormalMappedStripFromRawData(Asura_Mesh* pxMesh,
																											   const int iMaterialIndex,
																											   const Asura_Hash_ID uHashID,
																											   const Asura_Hash_ID uMaterialResponseHashID1 /*= ASURA_HASH_ID_UNSET*/,
																											   const Asura_Hash_ID uMaterialResponseHashID2 /*= ASURA_HASH_ID_UNSET*/)
																											   
{

	Tools_ObjectProcessor_NormalMapped::Initialise(pxMesh,iMaterialIndex,false,Tools_TriStripper::OUTPUT_TRISTRIP_WITH_RESET_INDICES,uMaterialResponseHashID1,uMaterialResponseHashID2);

	const Tools_ObjectProcessor_NormalMapped_Strip* pxToolsStrip = Tools_ObjectProcessor_NormalMapped::GetProcessedStrip();

	Asura_Direct3D_Object_Strip* pxStrip = AddObjectStrip(uHashID);

	if(pxStrip)
	{
		pxStrip->m_bNormalMapped = true;
		pxStrip->m_ePrimitiveType = D3DPT_TRIANGLELIST;

		pxStrip->m_uNumberOfTriangles = pxToolsStrip->m_uNumberOfTriangles;
		pxStrip->m_uNumberOfVertices = pxToolsStrip->m_uNumberOfVertices;
		pxStrip->m_uNumberOfIndices = pxToolsStrip->m_uNumberOfIndices;

		#if defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM
			pxStrip->m_auMaterialResponseHashID[0] = pxToolsStrip->m_auMaterialResponseHashID[0];
			pxStrip->m_auMaterialResponseHashID[1] = pxToolsStrip->m_auMaterialResponseHashID[1];
		#else
			pxStrip->m_iMaterialIndex = iMaterialIndex;
		#endif

		int flags = D3DUSAGE_WRITEONLY;

		if (!Asura_Direct3D_Caps::HasHardwareVertexShaders())
		{
			flags |= D3DUSAGE_SOFTWAREPROCESSING;
		}

		// load in all vertex buffer data
		if(pxStrip->m_uNumberOfVertices)
		{
			// Create vertex and buffers for the strip
			Asura_Direct3D::D3DDevice->CreateVertexBuffer
				(
				pxStrip->m_uNumberOfVertices * sizeof(Asura_Direct3D_ObjectVertex_NormalMapped),
				flags,
				0, 
				D3DPOOL_MANAGED,
				&(pxStrip->m_pxVertexBuffer),
				0
				);

			// lock, fill in data and unlock
			Asura_Direct3D_ObjectVertex_NormalMapped* pxVertexData;
			pxStrip->m_pxVertexBuffer->Lock
				(
				0,
				sizeof(Asura_Direct3D_ObjectVertex_NormalMapped) * pxStrip->m_uNumberOfVertices,
				(void**)&pxVertexData,
				0
				);

			for (u_int u=0; u<pxStrip->m_uNumberOfVertices; u++)
			{
				pxVertexData[u].m_xPosition = pxToolsStrip->m_pxVertices[u].m_xPosition;

				pxVertexData[u].m_xNormal = pxToolsStrip->m_pxVertices[u].m_xNormal;
				pxVertexData[u].m_xTangent = pxToolsStrip->m_pxVertices[u].m_xTangent;
				pxVertexData[u].m_xBinormal = pxToolsStrip->m_pxVertices[u].m_xBinormal;

				pxVertexData[u].U = pxToolsStrip->m_pxVertices[u].m_fU;
				pxVertexData[u].V = pxToolsStrip->m_pxVertices[u].m_fV;
				pxVertexData[u].U2 = pxToolsStrip->m_pxVertices[u].m_fU2;
				pxVertexData[u].V2 = pxToolsStrip->m_pxVertices[u].m_fV2;
				pxVertexData[u].fAlpha = pxToolsStrip->m_pxVertices[u].m_fAlpha;
			}

			pxStrip->m_pxVertexBuffer->Unlock();
		}

		// load in all index buffer data
		if(pxStrip->m_uNumberOfIndices)
		{
			Asura_Direct3D::D3DDevice->CreateIndexBuffer
				(
				pxStrip->m_uNumberOfIndices * sizeof(unsigned short),
				flags,
				D3DFMT_INDEX16,
				D3DPOOL_MANAGED,
				&(pxStrip->m_pxIndexBuffer),
				0
				);

			unsigned short* pusIndexData;
			pxStrip->m_pxIndexBuffer->Lock
				(
				0,
				sizeof(u_short) * pxStrip->m_uNumberOfIndices, 
				(void**)&pusIndexData,
				0
				);

			//	Asura_FileAccess::ReadData(hFile, pusIndexData, sizeof(u_short) * pxStrip->m_uNumberOfIndices);


			for (u_int u=0; u<pxStrip->m_uNumberOfIndices; u++)
			{
				pusIndexData[u] = pxToolsStrip->m_pusIndices[u];
			}

			pxStrip->m_pxIndexBuffer->Unlock();
		}
	}

	Tools_ObjectProcessor_NormalMapped::Shutdown();

	if(pxStrip)
	{
		//Link up LOD models. RAR 05/09/03
		pxStrip->m_fMinDistance = pxMesh->m_fMinDistance;
		if(pxMesh->m_pxNextLowerMesh)
		{
			pxStrip->m_pxNextLowerStrip = GetObjectStrip(pxMesh->m_pxNextLowerMesh);
		}
	}

	return pxStrip;
}
#endif//ASURA_TARGET_SM3

const Asura_Direct3D_Object_Strip* Asura_Direct3D_ObjectRender_Handler::InitialiseStripFromShape(const Asura_Shape& xShape)
{
	Asura_Shape_RenderData* pxShapeRenderData = xShape.GetRenderData();

	if (pxShapeRenderData==0) return 0;

	Asura_Mesh* pxMesh = pxShapeRenderData->GetMesh();

	if (pxMesh==0) return 0;

#if defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM

	return InitialiseNormalMappedStripFromRawData( pxMesh, ASURA_MATERIAL_NONE, xShape.GetShapeHashID(), xShape.GetMaterialIdx() );

#else // !defined ASURA_MATERIAL_RESPONSE_SYSTEM

#ifdef ASURA_TARGET_SM3
	// See if there is a normal map for this skin
	int iNormalMapID = ASURA_TEXTURE_NONE;
	if(pxMesh && Asura_Direct3D_Caps::GetHardwareVertexShaderVersion() >= 2.f)
	{
		if(pxMesh->TextureDataValid)
		{
			iNormalMapID = Asura_Material_System::GetNormalMapID(xShape.GetMaterialIdx());
		}
	}
	if(iNormalMapID != ASURA_TEXTURE_NONE)
	{
		return InitialiseNormalMappedStripFromRawData(pxMesh, xShape.GetMaterialIdx(), xShape.GetShapeHashID());
	}
	else
#endif//ASURA_TARGET_SM3
	{
		return InitialiseStripFromRawData( pxMesh, xShape.GetMaterialIdx(), xShape.GetShapeHashID() );
	}

#endif // !defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM
}

#endif	// #ifdef ASURA_TOOLS



////////////////////////////////////////////////////////////////////////////////
//
// Asura_Direct3D_ObjectRender_Handler
//
////////////////////////////////////////////////////////////////////////////////

void Asura_Direct3D_ObjectRender_Handler::Reset (void)
{
	m_xHashTree.Reset();
}

const Asura_Direct3D_Object_Strip* Asura_Direct3D_ObjectRender_Handler::GetObjectStrip(const Asura_Shape& xShape)
{
	const Asura_Hash_ID					uHash	= xShape.GetShapeHashID();
	const Asura_Direct3D_Object_Strip*	pxStrip	= m_xHashTree.GetDataByHash(uHash);

	// KJL - 14/04/2003 14:38:52 - get strip for given shape
	if (pxStrip == NULL)
	{
		// KJL - 14/04/2003 12:58:46 - Object wasn't found i.e. no Object strip of that name has
		// been loaded in. Now, if this is a tools build there's a chance that there'll be an Asura_Mesh 
		// and the associated data around to make a strip ourselves. If this isn't a tools build just
		// return NULL and the Object won't be drawn (assuming all the functions that call this handle
		// the case gracefully) - we'll assert so people know there's a problem.

#ifdef ASURA_TOOLS
		{
			pxStrip = InitialiseStripFromShape(xShape);
			Asura_Assertf(pxStrip, ("A Object strip that wasn't loaded as part of an export\ncouldn't be made from existing files either."));

			return pxStrip;
		}
#else
		{
			Asura_Assertf(false, ("No strip data for Object") );
			return 0;
		}
#endif
	}

	return pxStrip;

}

int Asura_Direct3D_ObjectRender_Handler::GetMaterialIdx( const Asura_Shape& xShape ) const
{
#if defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM

	const Asura_Hash_ID					uHash	= xShape.GetShapeHashID();
	const Asura_Direct3D_Object_Strip*	pxStrip	= m_xHashTree.GetDataByHash(uHash);

	if (pxStrip == NULL)
	{
#ifdef ASURA_TOOLS
		const Asura_Mesh* pxMesh = xShape.GetRenderData()->GetMesh();
		if( pxMesh && pxMesh->TextureDataValid )
		{
			return pxMesh->TextureDataPtr[0].m_auMaterialResponseHashID[0];
		}
#endif

		return ASURA_HASH_ID_UNSET;
	}
	else
	{
		return pxStrip->m_auMaterialResponseHashID[0];
	}

#else // !defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM

	const Asura_Hash_ID					uHash	= xShape.GetShapeHashID();
	const Asura_Direct3D_Object_Strip*	pxStrip	= m_xHashTree.GetDataByHash(uHash);

	if (pxStrip == NULL)
	{
#ifdef ASURA_TOOLS
		const Asura_Mesh* pxMesh = xShape.GetRenderData()->GetMesh();
		if( pxMesh && pxMesh->TextureDataValid )
		{
			return pxMesh->TextureDataPtr[0].m_iMaterialID;
		}
#endif

		return ASURA_MATERIAL_NONE;
	}
	else
	{
		return pxStrip->m_iMaterialIndex;
	}

#endif // !defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM
}

Asura_Direct3D_Object_Strip* Asura_Direct3D_ObjectRender_Handler::AddObjectStrip( const Asura_Hash_ID uHash )
{ 
	bool bDidExist = true;
	Asura_Direct3D_Object_Strip* pxObj = m_xHashTree.AddOrFindNode(uHash,&bDidExist,true); 

	Asura_Assert(pxObj,"Out of memory creating new hash tree node");
	Asura_Assertf(!(pxObj && bDidExist),("Duplicate item 0x08%x",uHash));

	return (bDidExist) ? NULL : pxObj;
}

#ifdef ASURA_TOOLS

const Asura_Direct3D_Object_Strip* Asura_Direct3D_ObjectRender_Handler::GetObjectStrip(Asura_Mesh* pxMesh)													
{
	if (!pxMesh) return 0;

	// KJL - 14/04/2003 14:38:52 - get strip for given mesh
	Asura_Hash_ID						uHash	= Asura_GetHashID(pxMesh->Name.GetString());
	const Asura_Direct3D_Object_Strip*	pxStrip	= m_xHashTree.GetDataByHash(uHash);

	int iMaterial;
	Asura_Hash_ID auMaterialResponseHashID[2];

	if( pxMesh->TextureDataPtr )
	{
		iMaterial = pxMesh->TextureDataPtr[0].m_iMaterialID;
		auMaterialResponseHashID[0] = pxMesh->TextureDataPtr[0].m_auMaterialResponseHashID[0];
		auMaterialResponseHashID[1] = pxMesh->TextureDataPtr[0].m_auMaterialResponseHashID[1];
	}
	else
	{
		iMaterial = ASURA_MATERIAL_NONE;
		auMaterialResponseHashID[0] = ASURA_HASH_ID_UNSET;
		auMaterialResponseHashID[1] = ASURA_HASH_ID_UNSET;
	}

	if (pxStrip == NULL)
	{
		// KJL - 14/04/2003 12:58:46 - Object wasn't found i.e. no Object strip of that name has
		// been loaded in. Now, if this is a tools build there's a chance that there'll be an Asura_Mesh 
		// and the associated data around to make a strip ourselves. If this isn't a tools build just
		// return NULL and the Object won't be drawn (assuming all the functions that call this handle
		// the case gracefully) - we'll assert so people know there's a problem.

#ifdef ASURA_TOOLS
		{
#if defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM

			// Always output normals etc.
			if( pxMesh && pxMesh->TextureDataValid )
			{
				pxStrip = InitialiseNormalMappedStripFromRawData( pxMesh, iMaterial, uHash, auMaterialResponseHashID[0], auMaterialResponseHashID[1]);
			}

#else // !defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM

#ifdef ASURA_TARGET_SM3
			// See if there is a normal map for this skin
			int iNormalMapID = ASURA_TEXTURE_NONE;
			if(pxMesh && Asura_Direct3D_Caps::GetHardwareVertexShaderVersion() >= 2.f)
			{
				if(pxMesh->TextureDataValid)
				{
					iNormalMapID = Asura_Material_System::GetNormalMapID(pxMesh->TextureDataPtr->m_iMaterialID);
				}
			}
			if(iNormalMapID != ASURA_TEXTURE_NONE)
			{
				pxStrip = InitialiseNormalMappedStripFromRawData(pxMesh, iMaterial, uHash);
			}
			else
#endif//ASURA_TARGET_SM3
			{
				pxStrip = InitialiseStripFromRawData(pxMesh,iMaterial,uHash);
			}

#endif // !defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM

			Asura_Assertf(pxStrip, ("A Object strip that wasn't loaded as part of an export\ncouldn't be made from existing files either."));

			return pxStrip;
		}
#else
		{
			Asura_Assertf(false, ("No strip data for Object") );
			return 0;
		}
#endif
	}

	return pxStrip;

}

#endif

////////////////////////////////////////////////////////////////////////////////
//
// Asura_Direct3D_ObjectRenderer
//
////////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_TARGET_SM3
bool Asura_Direct3D_ObjectRenderer::s_bUseNewRenderers = false;
bool Asura_Direct3D_ObjectRenderer::s_bNewRenderersSupported = false;

static Direct3D_VertexDeclaration* g_pxStandardVertexDecl = 0;
static Direct3D_VertexDeclaration* g_pxNormalMapVertexDecl = 0;

DEBUGVAR float dbg_fObjectPerPixelFogMinDimension = 0.5f;
#endif //#ifdef ASURA_TARGET_SM3

#if defined(ASURA_USE_MULTIPLE_HANDLERS)
Asura_Direct3D_ObjectRender_Handler*						Asura_Direct3D_ObjectRenderer::s_pxCurrentHandler	= NULL;		// Currently active handler.
Asura_TemplateHashTree<Asura_Direct3D_ObjectRender_Handler>	Asura_Direct3D_ObjectRenderer::s_xHandlerTree;					// Tree of all of the handlers available.
#else
Asura_Direct3D_ObjectRender_Handler							Asura_Direct3D_ObjectRenderer::s_xHandler;						// Single static-instance of a zone.
#endif


void Asura_Direct3D_ObjectRenderer::Initialise(void)
{
#ifdef ASURA_TARGET_SM3
	s_bNewRenderersSupported = (Asura_Direct3D_Caps::GetHardwarePixelShaderVersion() >= 2.0f) && (Asura_Direct3D_Caps::GetHardwareVertexShaderVersion() >= 2.0f);
	s_bUseNewRenderers = s_bNewRenderersSupported;

	if( NULL != Asura_Direct3D::D3DDevice)
	{
		{
			Direct3D_VertexElement auVSD[] =
			{
				{	0,	0,	D3DDECLTYPE_FLOAT3,		D3DDECLMETHOD_DEFAULT,	D3DDECLUSAGE_POSITION,	0 },
				{	0,	12,	D3DDECLTYPE_FLOAT3,		D3DDECLMETHOD_DEFAULT,	D3DDECLUSAGE_NORMAL,	0 },
				{	0,	24,	D3DDECLTYPE_FLOAT2,		D3DDECLMETHOD_DEFAULT,	D3DDECLUSAGE_TEXCOORD,	0 },
				{	0,	32,	D3DDECLTYPE_FLOAT1,		D3DDECLMETHOD_DEFAULT,	D3DDECLUSAGE_TEXCOORD,	1 },
				D3DDECL_END()
			};
			Asura_Direct3D::D3DDevice->CreateVertexDeclaration( auVSD, &(g_pxStandardVertexDecl) );
		}
		{
			Direct3D_VertexElement auVSD[] =
			{
				{	0,	0,	D3DDECLTYPE_FLOAT3,		D3DDECLMETHOD_DEFAULT,	D3DDECLUSAGE_POSITION,	0 },
				{	0,	12,	D3DDECLTYPE_FLOAT3,		D3DDECLMETHOD_DEFAULT,	D3DDECLUSAGE_NORMAL,	0 },
				{	0,	24,	D3DDECLTYPE_FLOAT2,		D3DDECLMETHOD_DEFAULT,	D3DDECLUSAGE_TEXCOORD,	0 },
				{	0,	32,	D3DDECLTYPE_FLOAT2,		D3DDECLMETHOD_DEFAULT,	D3DDECLUSAGE_TEXCOORD,	1 },
				{	0,	40,	D3DDECLTYPE_FLOAT1,		D3DDECLMETHOD_DEFAULT,	D3DDECLUSAGE_TEXCOORD,	2 },
				{	0,	44,	D3DDECLTYPE_FLOAT3,		D3DDECLMETHOD_DEFAULT,	D3DDECLUSAGE_BINORMAL,	0 },
				{	0,	56,	D3DDECLTYPE_FLOAT3,		D3DDECLMETHOD_DEFAULT,	D3DDECLUSAGE_TANGENT,	0 },
				D3DDECL_END()
			};
			Asura_Direct3D::D3DDevice->CreateVertexDeclaration( auVSD, &(g_pxNormalMapVertexDecl) );
		}
	}
#endif //#ifdef ASURA_TARGET_SM3
}

void Asura_Direct3D_ObjectRenderer::Reset(void)
{
#if defined(ASURA_USE_MULTIPLE_HANDLERS)
	GetCurrentHandler().Reset();
#else
	Shutdown();
	Initialise();
#endif
}

void Asura_Direct3D_ObjectRenderer::Shutdown(void)
{
#ifdef ASURA_TARGET_SM3
	if(g_pxStandardVertexDecl)
	{
		g_pxStandardVertexDecl->Release();
		g_pxStandardVertexDecl = NULL;
	}
	if(g_pxNormalMapVertexDecl)
	{
		g_pxNormalMapVertexDecl->Release();
		g_pxNormalMapVertexDecl = NULL;
	}
#endif
#if defined(ASURA_USE_MULTIPLE_HANDLERS)
	s_xHandlerTree.Reset();
	s_pxCurrentHandler	= NULL;
#else
	s_xHandler.Reset();
#endif
}

#if defined(ASURA_USE_MULTIPLE_HANDLERS)

void Asura_Direct3D_ObjectRenderer::ReparentHandler(Asura_Direct3D_ObjectRender_Handler& xSourceHandler, const Asura_Hash_ID uNewParentHash)
{
	Asura_Direct3D_ObjectRender_Handler*	pxParentHandler	= s_xHandlerTree.GetDataByHash(uNewParentHash);

	if(pxParentHandler != NULL)
	{
		xSourceHandler.m_xHashTree.SetParent(&(pxParentHandler->m_xHashTree));
	}
}

void Asura_Direct3D_ObjectRenderer::ReparentHandler(const Asura_Hash_ID uSourceHash, const Asura_Hash_ID uNewParentHash)
{
	Asura_Direct3D_ObjectRender_Handler*	pxHandler	= s_xHandlerTree.GetDataByHash(uSourceHash);

	// Do we already have this handler?
	if(pxHandler != NULL)
	{
		ReparentHandler(*pxHandler, uNewParentHash);
	}
}

Asura_Direct3D_ObjectRender_Handler* Asura_Direct3D_ObjectRenderer::CreateHandler (const Asura_Hash_ID uHash, const Asura_Hash_ID uParentHandlerHash)
{
	Asura_Direct3D_ObjectRender_Handler*	pxHandler	= s_xHandlerTree.GetDataByHash(uHash);

	// Do we already have this zone?
	if(pxHandler == NULL)
	{
		// No - create one...
		pxHandler	= s_xHandlerTree.AddNode(uHash);

		if(pxHandler != NULL)
		{
			ReparentHandler(*pxHandler, uParentHandlerHash);
		}
	}
	else
	{
		// Yes - this shouldn't happen...
		Asura_Assert(false, "Already have an object render-handler with this hash-ID.");
	}

	// Return the zone...
	return(pxHandler);
}

bool Asura_Direct3D_ObjectRenderer::DestroyHandler (const Asura_Hash_ID uHash)
{
	// Is this the current zone?
	if(s_pxCurrentHandler == s_xHandlerTree.GetDataByHash(uHash))
	{
		s_pxCurrentHandler	= NULL;
	}

	// Call the delete function...
	return(s_xHandlerTree.DeleteNodeByHash(uHash));
}

void Asura_Direct3D_ObjectRenderer::WriteHandlerReport(u_int uHandler, Asura_Stream& xLog)
{
	const u_int uLOGBUFFERSIZE = 64;
	char szLogBuffer[uLOGBUFFERSIZE];

	Asura_Direct3D_ObjectRender_Handler* pxHandler = s_xHandlerTree.GetDataByHash(uHandler);
	if (pxHandler)
	{
		if (!pxHandler->m_xHashTree.GetNumberOfNodes())
		{
			xLog << "PC Obj:\t\t\t\tNone\n";
		}
		else
		{
			Asura_PrintfToBfr( szLogBuffer, "PC Obj:\t\t\t\t%d Item(s)\n", uLOGBUFFERSIZE, true, pxHandler->m_xHashTree.GetNumberOfNodes() );
			xLog << szLogBuffer;
		}
	}
	else
	{
		xLog << "PC Obj:\t\t\t\tNo Handler\n";
	}
}

#endif	// defined(ASURA_USE_MULTIPLE_HANDLERS)

void Asura_Direct3D_ObjectRenderer::Render(const class Asura_Shape* pxShape, const class Asura_Shape_Instance* pxShapeInst, const u_int uARGB)
{
	D3DPERFCALL(BeginEvent(0xff845fb4, L"Object Render"));

	const Asura_Direct3D_Object_Strip* pxStrip = GetObjectStrip(*pxShape);

	const Asura_Vector_3& xPosition = pxShapeInst->GetPosition();
	const Asura_Matrix_3x3& xOrientation = pxShapeInst->GetOrientation();

	pxShapeInst->UpdateLighting();

	if(pxStrip && pxStrip->m_pxNextLowerStrip)
	{
		//Get the correct LOD model. RAR 05/09/03
		float fDistance = (*Asura_Camera::GetPosition() - xPosition).Magnitude();	
		pxStrip = pxStrip->GetDetailMeshFromDistance(fDistance);
		Asura_Assert(pxStrip,"");
	}

#if defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM

	Render_MaterialResponse( *pxStrip, xPosition, xOrientation, Asura_Vector_3( 1.0f, 1.0f, 1.0f ) );
	D3DPERFCALL( EndEvent() );

#else // !defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM

	Asura_Vector_2 xUVOffset(0.0f, 0.0f);
	Asura_TextureAnimationOutput xTextureAnim;
	const Asura_Animation* pxAnim = pxShapeInst->GetAnimation();
	if(pxAnim && Asura_Material_System::GetTextureAnimationState(pxStrip->m_iMaterialIndex, pxAnim->GetTextureAnimController(),&xTextureAnim))
	{
		xUVOffset.x = xTextureAnim.m_fUOffset;
		xUVOffset.y = xTextureAnim.m_fVOffset;
	}

	float fAlpha = (float)(uARGB>>24)/255.0f;

#ifdef ASURA_TARGET_SM3
	if (s_bUseNewRenderers)
	{
		bool bUsePerPixelFog = false;
		if (pxShape->GetDynamicsCollisionData())
		{
			Asura_Vector_3 xExtents(pxShape->GetDynamicsCollisionData()->GetBoundingBox().CalculateExtents());
			bUsePerPixelFog |= (xExtents.x >= dbg_fObjectPerPixelFogMinDimension) || (xExtents.y >= dbg_fObjectPerPixelFogMinDimension) || (xExtents.z >= dbg_fObjectPerPixelFogMinDimension);
		}
		RenderWithEffectSystem(pxStrip, xPosition, xOrientation, xUVOffset, fAlpha, true, 1.0f, bUsePerPixelFog);
	}
	else
#endif //#ifdef ASURA_TARGET_SM3
	{
		RenderBasic(pxStrip, xPosition, xOrientation, xUVOffset, fAlpha, true);
	}

	D3DPERFCALL(EndEvent());
#if !defined(ASURA_TARGET_SM3)
	// KJFL - 2004-02-23 17:05:55 - currently only support planar shadows
	if (Asura_Direct3D_UserOptions::GetObjectShadows() != Asura_Direct3D_UserOptions::SHADOW_PLANAR) return;

	// does this shape instance have a shadow?
	if (!(pxShapeInst->GetFlags() & ASURA_SHAPE_INSTANCE_NO_SHADOW ))
	{
		D3DPERFCALL(BeginEvent(0xff845fb4, L"Object Shadow"));

		Asura_Vector_3 xShadowPosition = xPosition;

		// the bounding box of the object may not be in contact with the ground; we'll have to do a line of sight test

		xShadowPosition.y-=0.1f;
		Asura_Vector_3 xDownwards(0.0f,1.0f,0.0f);

		Asura_LOS_InputData xInputData( xShadowPosition, xDownwards, ASURA_GUID_UNREGISTERED, fASURA_PC_OBJECT_RENDERER_SHADOW_LOS_DISTANCE );

		const Asura_LOS_OutputData* pxOutputData = Asura_LOS::TestAgainstEnvironment(&xInputData);
	
		xShadowPosition.y += pxOutputData->m_fDistanceToIntersection;
	
		// lift the shadow up off the floor ever so slightly (1cm) to relieve z-fighting problems
		xShadowPosition.y -= 0.01f;

		// go for it.
		RenderShadow(pxStrip, xShadowPosition, xOrientation);

		D3DPERFCALL(EndEvent());
	}
#endif

#endif // !defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM
}

#if !defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM

void Asura_Direct3D_ObjectRenderer::RenderBasic(const Asura_Direct3D_Object_Strip* pxStrip, const Asura_Vector_3& xPosition, const Asura_Matrix_3x3& xOrientation, const Asura_Vector_2& xUVOffset, float fAlpha, bool bLight, float fScale/* = 1.0f*/)
{
	RenderBasic(pxStrip, xPosition, xOrientation, xUVOffset, fAlpha, bLight, Asura_Vector_3(fScale,fScale,fScale), ASURA_TEXTURE_NONE);
}

void Asura_Direct3D_ObjectRenderer::RenderBasic(const Asura_Direct3D_Object_Strip* pxStrip, const Asura_Vector_3& xPosition, const Asura_Matrix_3x3& xOrientation, const Asura_Vector_2& xUVOffset, float fAlpha, bool bLight, const Asura_Vector_3& xScale,int iTextureID)
{
	SCOPEDPIXEVENT((0xff58ba34, L"Object RenderBasic"));

	if (!pxStrip) return;
	
	if (!(pxStrip->m_pxVertexBuffer && pxStrip->m_pxIndexBuffer && pxStrip->m_uNumberOfTriangles))
	{
		return;
	}

	u_int uFirstUnusedTexture = 0;

	// turn on fog if we have to...
	if (Asura_Fog::IsFogRenderable())
	{
		Asura_Direct3D::D3DDevice->SetRenderState(D3DRS_FOGENABLE, TRUE);
	}
	
	Asura_Render::AddToTrianglesRendered(pxStrip->m_uNumberOfTriangles);

	Asura_Direct3D_Shaders::RequestVertexShader(ASURA_D3D_VS_OBJECT_ORDER2SH);

	Asura_Direct3D::D3DDevice->SetVertexShaderConstantF(94, &fNORMAL_DECOMPRESS[0], 1);
	Asura_Direct3D::D3DDevice->SetVertexShaderConstantF(95, &fTEXCOORD_DECOMPRESS[0], 1);

#ifdef ASURA_TARGET_SM3
	if (pxStrip->m_bNormalMapped)
	{
		Asura_Direct3D::D3DDevice->SetStreamSource(0,pxStrip->m_pxVertexBuffer,0,sizeof(Asura_Direct3D_ObjectVertex_NormalMapped));
	}
	else
#endif //#ifdef ASURA_TARGET_SM3
	{
		Asura_Direct3D::D3DDevice->SetStreamSource(0,pxStrip->m_pxVertexBuffer,0,sizeof(Asura_Direct3D_ObjectVertex));
	}
	Asura_Direct3D::D3DDevice->SetIndices(pxStrip->m_pxIndexBuffer);

	Asura_Render::CurrentStates.RequestCullMode(ASURA_CULLMODE_CW); //CW

	if(iTextureID == ASURA_TEXTURE_NONE)
		Asura_Render::CurrentStates.RequestTextureIndex(0, Asura_Material_System::GetTextureID(pxStrip->m_iMaterialIndex));
	else
		Asura_Render::CurrentStates.RequestTextureIndex(0, iTextureID);
		
	uFirstUnusedTexture = 1;

	{
		D3DXMATRIX mat;
		D3DXMATRIX matView, matProj;

		Asura_Direct3D::D3DDevice->GetTransform( D3DTS_VIEW,       &matView );
		Asura_Direct3D::D3DDevice->GetTransform( D3DTS_PROJECTION,       &matProj );

        D3DXMatrixMultiply( &mat, &matView, &matProj );
        D3DXMatrixTranspose( &mat, &mat );

        Asura_Direct3D::D3DDevice->SetVertexShaderConstantF(  0, (float*)&mat,  4 );
	
		mat._11 = xOrientation.e11 * xScale.x;
		mat._12 = xOrientation.e12 * xScale.y;
		mat._13 = xOrientation.e13 * xScale.z;
		mat._14 = xPosition.x;
		mat._21 = xOrientation.e21 * xScale.x;
		mat._22 = xOrientation.e22 * xScale.y;
		mat._23 = xOrientation.e23 * xScale.z;
		mat._24 = xPosition.y;
		mat._31 = xOrientation.e31 * xScale.x;
		mat._32 = xOrientation.e32 * xScale.y;
		mat._33 = xOrientation.e33 * xScale.z;
		mat._34 = xPosition.z;
		mat._41 = 0;
		mat._42 = 0;
		mat._43 = 0;
		mat._44 = 1.0f;
		Asura_Direct3D::D3DDevice->SetVertexShaderConstantF(  7, (float*)&mat,  4 );
	
	}

	{
		//if (!mesh->BoundingBoxDataValid) mesh->CalculateBoundingBox();
		//Asura_Bounding_Box bb;// = mesh->BoundingBox;
		//bb.Reset();
		//bb.Set(xPosition);

		float afLights[4*3];
		const Asura_SphericalHarmonics_Order2_RGB &xSH = Asura_GI_Solution::s_xSHForCurrentRender;

		afLights[0] = xSH.m_afRedCoefficients[1]; 
		afLights[1] = xSH.m_afRedCoefficients[2]; 
		afLights[2] = xSH.m_afRedCoefficients[3]; 
		afLights[3] = xSH.m_afRedCoefficients[0]; 

		afLights[4] = xSH.m_afGreenCoefficients[1]; 
		afLights[5] = xSH.m_afGreenCoefficients[2]; 
		afLights[6] = xSH.m_afGreenCoefficients[3]; 
		afLights[7] = xSH.m_afGreenCoefficients[0]; 

		afLights[8] = xSH.m_afBlueCoefficients[1]; 
		afLights[9] = xSH.m_afBlueCoefficients[2];
		afLights[10] = xSH.m_afBlueCoefficients[3];
		afLights[11] = xSH.m_afBlueCoefficients[0];

		Asura_Direct3D::D3DDevice->SetVertexShaderConstantF( 11, afLights, 3 );
	}

	// uv animation 
	float afUVOffset[4] = { xUVOffset.x,xUVOffset.y,0.0f,0.0f };
	Asura_Direct3D::D3DDevice->SetVertexShaderConstantF( 19, afUVOffset, 1);

	// fog parameters 
	const float afFogValues[4] = 
	{ 
		1.f - Asura_Fog::GetFogDensityAtDistance((xPosition - (*Asura_Camera::GetPosition())).Magnitude()), 
		0.f, 
		0.f, 
		fAlpha 
	};
	Asura_Direct3D::D3DDevice->SetVertexShaderConstantF( 20, afFogValues, 1);

	const bool bMaterialTransparent = Asura_Material_System::IsMaterialTransparent( pxStrip->m_iMaterialIndex );

	if (fAlpha < 1.0f)
	{
		Asura_Render::CurrentStates.RequestTextureOpMode(ASURA_TEXTUREOP_SINGLE_PASS_2X);
		Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_NORMAL);
		Asura_Render::CurrentStates.RequestAlphaTest(true);
	}
	/// Kevin Lea - 07/02/2003 15:33:24 - first off, is texture transparent
	else if (Asura_Material_System::IsMaterialSelfIlluminating(pxStrip->m_iMaterialIndex))
	{
	
		Asura_Render::CurrentStates.RequestTextureIndex(1,Asura_Material_System::GetTextureID(pxStrip->m_iMaterialIndex));
		Asura_Render::CurrentStates.RequestTextureOpMode(ASURA_TEXTUREOP_SELFILLUMINATE);
		Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_OFF);
		Asura_Render::CurrentStates.RequestAlphaTest(false);
		
		uFirstUnusedTexture = 2;
	}
	else if( bMaterialTransparent )
	{
		Asura_Render::CurrentStates.RequestTextureOpMode(ASURA_TEXTUREOP_SINGLE_PASS_2X);
		
		// DMR (21/11/05) - Handle alpha-test without blending, essential for vegetation
		int iTextureID=Asura_Material_System::GetTextureID(pxStrip->m_iMaterialIndex);

		if (Asura_Texture_Management::GetTextureFlags(iTextureID) & AsuraTextureFlag_8BitAlpha)
		{
			Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_NORMAL);
			Asura_Render::CurrentStates.RequestAlphaTest(true);
		}
		else
		{
			Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_OFF);
			Asura_Render::CurrentStates.RequestAlphaTest(true, 0x80);
		}
	}
	else if (Asura_Material_System::IsMaterialPS2AdditiveAlpha(pxStrip->m_iMaterialIndex))
	{
		Asura_Render::CurrentStates.RequestTextureOpMode(ASURA_TEXTUREOP_SINGLE_PASS_2X);

		Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_GLOWING);
		
		Asura_Render::CurrentStates.RequestZBufferWriteEnabled(false);		
		Asura_Render::CurrentStates.RequestAlphaTest(true);
	}
	else
	{
		Asura_Render::CurrentStates.RequestTextureOpMode(ASURA_TEXTUREOP_SINGLE_PASS_2X);
		Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_OFF);
		Asura_Render::CurrentStates.RequestAlphaTest(false);
	}

	if( fAlpha < 1.0f && !bMaterialTransparent )
	{
		// If the material's not transparent, we shouldn't get the alpha from the texture.
		// This will be unmunged when the texture op is requested again after rendering
		Asura_Direct3D::D3DDevice->SetTextureStageState( 0, D3DTSS_ALPHAOP, D3DTOP_SELECTARG2 );
	}
	
	// DLH - 28 June 2006 12:54:11 - Clear the texture stages from 1-N
	Asura_Render::CurrentStates.ResetTextureIndexToLast( uFirstUnusedTexture);

	/// Kevin Lea - 07/02/2003 15:36:42 - draw base mesh
	Asura_Direct3D::D3DDevice->DrawIndexedPrimitive(pxStrip->m_ePrimitiveType,0,0,pxStrip->m_uNumberOfVertices,0,pxStrip->m_uNumberOfTriangles);

	if(Asura_Material_System::IsMaterialReflective(pxStrip->m_iMaterialIndex))
	{
		// reflective pass
		
		D3DXMATRIX xReflectiveMat;
		D3DXMATRIX matView, xTransposedView;
		Asura_Direct3D::D3DDevice->GetTransform( D3DTS_VIEW, &matView );
		D3DXMatrixTranspose( &xTransposedView, &matView );
		matView._14 = matView._24 = matView._34 = 0.0f;
		matView._41 = matView._42 = matView._43 = 0.0f;
		D3DXMatrixInverse( &matView,  NULL, &matView );

		xReflectiveMat._11 = 0.5f; xReflectiveMat._12 = 0.0f; xReflectiveMat._13 = 0.0f; xReflectiveMat._14 = 0.0f; 
		xReflectiveMat._21 = 0.0f; xReflectiveMat._22 = -0.5f; xReflectiveMat._23 = 0.0f; xReflectiveMat._24 = 0.0f; 
		xReflectiveMat._31 = 0.0f; xReflectiveMat._32 = 0.0f; xReflectiveMat._33 = 1.0f; xReflectiveMat._34 = 0.0f; 
		xReflectiveMat._41 = 0.5f; xReflectiveMat._42 = 0.5f; xReflectiveMat._43 = 0.0f; xReflectiveMat._44 = 1.0f; 

		D3DXMatrixMultiplyTranspose(&xReflectiveMat,&matView,&xReflectiveMat);

		Asura_Direct3D::D3DDevice->SetVertexShaderConstantF( 21, (float*)&xTransposedView, 3 );
		Asura_Direct3D::D3DDevice->SetVertexShaderConstantF( 24, (float*)&xReflectiveMat, 3 );
		

		Asura_Direct3D_Shaders::RequestVertexShader(ASURA_D3D_VS_OBJECT_REFLECTIVE);
		Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_GLOWING);
		if(!Asura_Material_System::IsMaterialTransparent(pxStrip->m_iMaterialIndex))
		{
			Asura_Direct3D_Shaders::RequestPixelShader(ASURA_D3D_PS_OBJECT_REFLECTIVE);
		}
		else
		{
			Asura_Direct3D_Shaders::RequestPixelShader(ASURA_D3D_PS_OBJECT_REFLECTIVE_TEXALPHA);
		}
		Asura_Render::CurrentStates.RequestTextureIndex(1, Asura_Render::GetReflectanceMap());
		
		uFirstUnusedTexture = 2;

		Asura_Render::CurrentStates.ResetTextureIndexToLast( uFirstUnusedTexture);
		Asura_Direct3D::D3DDevice->DrawIndexedPrimitive(pxStrip->m_ePrimitiveType,0,0,pxStrip->m_uNumberOfVertices,0,pxStrip->m_uNumberOfTriangles);

		Asura_Direct3D_Shaders::RequestPixelShader(ASURA_D3D_PS_NONE);
	}

	if (Asura_Material_System::IsMaterialPS2AdditiveAlpha(pxStrip->m_iMaterialIndex))
	{
		Asura_Render::CurrentStates.RequestZBufferWriteEnabled(true);
	}

	Asura_Render::CurrentStates.RequestAlphaTest(false);

	Asura_Render::CurrentStates.RequestTextureOpMode(ASURA_TEXTUREOP_SINGLE_PASS);
	Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_OFF);

	// turn off fog if we have to...
	if (Asura_Fog::IsFogRenderable())
	{
		Asura_Direct3D::D3DDevice->SetRenderState(D3DRS_FOGENABLE, FALSE);
	}
}

#ifdef ASURA_TARGET_SM3
static bool g_bObjectPerPixelFogEnabled = true;

void Asura_Direct3D_ObjectRenderer::RenderWithEffectSystem(const Asura_Direct3D_Object_Strip* pxStrip, const Asura_Vector_3& xPosition, const Asura_Matrix_3x3& xOrientation, const Asura_Vector_2& xUVOffset, float fAlpha, bool bLight, float fScale/* = 1.0f*/, bool bUsePerPixelFog/* = false*/)
{
	RenderWithEffectSystem(pxStrip, xPosition, xOrientation, xUVOffset, fAlpha, bLight, Asura_Vector_3(fScale,fScale,fScale), bUsePerPixelFog);
}

void Asura_Direct3D_ObjectRenderer::RenderWithEffectSystem(const Asura_Direct3D_Object_Strip* pxStrip, const Asura_Vector_3& xPosition, const Asura_Matrix_3x3& xOrientation, const Asura_Vector_2& xUVOffset, float fAlpha, bool bLight, const Asura_Vector_3& xScale, bool bUsePerPixelFog/* = false*/)
{
	SCOPEDPIXEVENT((0xff58ba34, L"Object RenderWithEffectSystem"));

	if (!pxStrip) { return; }

	if (!(pxStrip->m_pxVertexBuffer && pxStrip->m_pxIndexBuffer && pxStrip->m_uNumberOfTriangles))
	{
		return;
	}
	
	Asura_Direct3D_Effect* pxEffect;

	if (pxStrip->m_bNormalMapped)
	{
		// Dual currently unsupported (as dual texturing isn't supported either)
//		if (Asura_Material_System::IsMaterialDualNormalMap(pxStrip->m_iMaterialIndex))
//		{
//			pxEffect = Asura_Direct3D_Effects_System::BeginEffect("Object_TextureAndLight_DualNormalMap");
//		}
//		else
		{
			pxEffect = Asura_Direct3D_Effects_System::BeginEffect("Object_TextureAndLight_NormalMap");
		}
	}
	else
	{
		pxEffect = Asura_Direct3D_Effects_System::BeginEffect("Object_TextureAndLight");
	}

	if (!pxEffect || !pxEffect->IsD3DEffectValid()) { return; }

	Asura_Render::AddToTrianglesRendered(pxStrip->m_uNumberOfTriangles);

	// Set up our render states
	Asura_Render::CurrentStates.RequestCullMode	(ASURA_CULLMODE_CW);

	// Are we using alpha yet? This could possibly be off.
	const bool bIsAdditiveAlpha	= Asura_Material_System::IsMaterialPS2AdditiveAlpha(pxStrip->m_iMaterialIndex);
	const bool bIsTransparent	= Asura_Material_System::IsMaterialTransparent(pxStrip->m_iMaterialIndex);

	if (bIsTransparent)
	{
		Asura_Render::CurrentStates.RequestTranslucencyMode		(ASURA_TRANSLUCENCY_NORMAL);
		Asura_Render::CurrentStates.RequestZBufferWriteEnabled	(false);

		// (AAM - 23.11.2006) Can't disable blending on the normal-depth RT, so disable writes
		Asura_Direct3D::D3DDevice->SetRenderState( D3DRS_COLORWRITEENABLE1, 0 );
	}

	else if (bIsAdditiveAlpha)
	{
		Asura_Render::CurrentStates.RequestTranslucencyMode		(ASURA_TRANSLUCENCY_GLOWING);
		Asura_Render::CurrentStates.RequestZBufferWriteEnabled	(false);

		Asura_Direct3D::D3DDevice->SetRenderState( D3DRS_COLORWRITEENABLE1, 0 );
	}
	
	else
	{
		Asura_Render::CurrentStates.RequestTranslucencyMode		(ASURA_TRANSLUCENCY_OFF);
		Asura_Render::CurrentStates.RequestZBufferWriteEnabled	(true);

		Asura_Direct3D::D3DDevice->SetRenderState( D3DRS_COLORWRITEENABLE1, D3DCOLORWRITEENABLE_RED | D3DCOLORWRITEENABLE_GREEN | D3DCOLORWRITEENABLE_BLUE | D3DCOLORWRITEENABLE_ALPHA );
	}

	pxEffect->SetTextureRenderStates(pxStrip->m_iMaterialIndex);

	// Set up streams
	if (pxStrip->m_bNormalMapped)
	{
		Asura_Direct3D::D3DDevice->SetStreamSource(0,pxStrip->m_pxVertexBuffer,0,sizeof(Asura_Direct3D_ObjectVertex_NormalMapped));
		Asura_Direct3D::D3DDevice->SetVertexDeclaration(g_pxNormalMapVertexDecl);
	}
	else
	{
		Asura_Direct3D::D3DDevice->SetStreamSource(0,pxStrip->m_pxVertexBuffer,0,sizeof(Asura_Direct3D_ObjectVertex));
		Asura_Direct3D::D3DDevice->SetVertexDeclaration(g_pxStandardVertexDecl);
	}
	Asura_Direct3D::D3DDevice->SetIndices(pxStrip->m_pxIndexBuffer);

	// Set up parameters. Shared should have been set up already by BeginFrame()
	D3DXMATRIX xD3DMat;
	xD3DMat._11 = xOrientation.e11 * xScale.x;
	xD3DMat._12 = xOrientation.e21 * xScale.x;
	xD3DMat._13 = xOrientation.e31 * xScale.x;
	xD3DMat._14 = 0;			 
	xD3DMat._21 = xOrientation.e12 * xScale.y;
	xD3DMat._22 = xOrientation.e22 * xScale.y;
	xD3DMat._23 = xOrientation.e32 * xScale.y;
	xD3DMat._24 = 0;			 
	xD3DMat._31 = xOrientation.e13 * xScale.z;
	xD3DMat._32 = xOrientation.e23 * xScale.z;
	xD3DMat._33 = xOrientation.e33 * xScale.z;
	xD3DMat._34 = 0;
	xD3DMat._41 = xPosition.x;
	xD3DMat._42 = xPosition.y;
	xD3DMat._43 = xPosition.z;
	xD3DMat._44 = 1.0f;
	pxEffect->SetParameterByName("g_xWorldMatrix", xD3DMat);

	const Asura_SphericalHarmonics_Order2_RGB &xSH = Asura_GI_Solution::s_xSHForCurrentRender;
	Asura_Vector_3 xBrightestLightDirection;
	xSH.GetBrightestDirection(xBrightestLightDirection);
	D3DXVECTOR4 xD3DVector;

	xD3DVector.x = xSH.m_afRedCoefficients[1];
	xD3DVector.y = xSH.m_afRedCoefficients[2];
	xD3DVector.z = xSH.m_afRedCoefficients[3];
	xD3DVector.w = xSH.m_afRedCoefficients[0];
	pxEffect->SetParameterByName("g_xObjectRedSH", xD3DVector);
	xD3DVector.x = xSH.m_afGreenCoefficients[1];
	xD3DVector.y = xSH.m_afGreenCoefficients[2];
	xD3DVector.z = xSH.m_afGreenCoefficients[3];
	xD3DVector.w = xSH.m_afGreenCoefficients[0];
	pxEffect->SetParameterByName("g_xObjectGreenSH", xD3DVector);
	xD3DVector.x = xSH.m_afBlueCoefficients[1];
	xD3DVector.y = xSH.m_afBlueCoefficients[2];
	xD3DVector.z = xSH.m_afBlueCoefficients[3];
	xD3DVector.w = xSH.m_afBlueCoefficients[0];
	pxEffect->SetParameterByName("g_xObjectBlueSH", xD3DVector);
	xD3DVector.x = xBrightestLightDirection.x;
	xD3DVector.y = xBrightestLightDirection.y;
	xD3DVector.z = xBrightestLightDirection.z;
	xD3DVector.w = 0;
	pxEffect->SetParameterByName("g_xObjectBrightestLightDirection", xD3DVector);

	xD3DVector.x = xUVOffset.x;
	xD3DVector.y = xUVOffset.y;
	xD3DVector.z = 1.0f;
	xD3DVector.w = 1.0f;
	pxEffect->SetParameterByName("g_xTextureCoordOffsetAndScale", xD3DVector);

	pxEffect->SetParameterByName("g_bObjectSpecular", (Asura_Material_System::IsMaterialSpecularSharp(pxStrip->m_iMaterialIndex) || Asura_Material_System::IsMaterialSpecularSoft(pxStrip->m_iMaterialIndex)) && (xBrightestLightDirection != Asura_Maths::ZeroVector3));
	pxEffect->SetParameterByName("g_bObjectTextureTransparancy", bIsTransparent || bIsAdditiveAlpha);
	pxEffect->SetParameterByName("g_bSelfIlluminating", Asura_Material_System::IsMaterialSelfIlluminating(pxStrip->m_iMaterialIndex));
	pxEffect->SetParameterByName("g_fObjectAlpha", fAlpha);

#ifdef ASURA_TOOLS
	pxEffect->SetParameterByName("g_bObjectUseLighting", !Tools_System::IsActive() || Tools_Render::LightingIsOn());
#else
	pxEffect->SetParameterByName("g_bObjectUseLighting", true);
#endif //#ifdef ASURA_TOOLS

	pxEffect->RenderAllIndexedPrimitivePasses(pxStrip->m_ePrimitiveType, 
												0,
												pxStrip->m_uNumberOfVertices,
												0,
												pxStrip->m_uNumberOfTriangles,
												NULL);

	// Flush any effects we may still be using...
	Asura_Direct3D_Effects_System::FlushEffects	();

	// Turn off any pixel shaders we may have used...
	Asura_Direct3D_Shaders::RequestPixelShader	(ASURA_D3D_PS_NONE);
	Asura_Direct3D::D3DDevice->SetPixelShader	(NULL);

	// Restore normal-depth RT writes in case the last strip was transparent
	Asura_Direct3D::D3DDevice->SetRenderState( D3DRS_COLORWRITEENABLE1, D3DCOLORWRITEENABLE_RED | D3DCOLORWRITEENABLE_GREEN | D3DCOLORWRITEENABLE_BLUE | D3DCOLORWRITEENABLE_ALPHA );
}
#endif //#ifdef ASURA_TARGET_SM3

#endif // !defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM

void Asura_Direct3D_ObjectRenderer::RenderShadow(const Asura_Direct3D_Object_Strip* pxStrip, const Asura_Vector_3& xPosition, const Asura_Matrix_3x3& xOrientation)
{
	SCOPEDPIXEVENT((0xff58ba34, L"Object RenderShadow"));

	if (!pxStrip) return;

	if (!(pxStrip->m_pxVertexBuffer && pxStrip->m_pxIndexBuffer && pxStrip->m_uNumberOfTriangles))
	{
		return;
	}

	Asura_Render::AddToTrianglesRendered(pxStrip->m_uNumberOfTriangles);
	Asura_Direct3D_Shaders::RequestVertexShader(ASURA_D3D_VS_OBJECT_PLANARSHADOW);

#ifdef ASURA_TARGET_SM3
	if (pxStrip->m_bNormalMapped)
	{
		Asura_Direct3D::D3DDevice->SetStreamSource(0,pxStrip->m_pxVertexBuffer,0,sizeof(Asura_Direct3D_ObjectVertex_NormalMapped));
	}
	else
#endif //#ifdef ASURA_TARGET_SM3
	{
		Asura_Direct3D::D3DDevice->SetStreamSource(0,pxStrip->m_pxVertexBuffer,0,sizeof(Asura_Direct3D_ObjectVertex));
	}
	Asura_Direct3D::D3DDevice->SetIndices(pxStrip->m_pxIndexBuffer);

	Asura_Render::CurrentStates.RequestCullMode(ASURA_CULLMODE_CW); //CW

	// orientate and position object in vertex shader
	D3DXMATRIX xMat;
	D3DXMATRIX xMatView, xMatProj;

	Asura_Direct3D::D3DDevice->GetTransform( D3DTS_VIEW, &xMatView );
	Asura_Direct3D::D3DDevice->GetTransform( D3DTS_PROJECTION, &xMatProj );

    D3DXMatrixMultiply( &xMat, &xMatView, &xMatProj );
    D3DXMatrixTranspose( &xMat, &xMat );

    Asura_Direct3D::D3DDevice->SetVertexShaderConstantF(  0, (float*)&xMat,  4 );
	
	float afTransformationData[] = 
	{
		// x rotation 
		xOrientation.e11, xOrientation.e12, xOrientation.e13, xPosition.x,

		// z rotation
		xOrientation.e31, xOrientation.e32, xOrientation.e33, xPosition.z,
	};

	Asura_Direct3D::D3DDevice->SetVertexShaderConstantF(  4, afTransformationData,  2 );

	float fFogDensity = Asura_Fog::GetFogDensityAtDistance((xPosition - (*Asura_Camera::GetPosition())).Magnitude());
	Asura_RGBA xFogColour = Asura_Fog::GetFogColour();
	xFogColour *= fFogDensity;

	float afShadowData[] = 
	{
		// y position and w value
		0.0f, xPosition.y, 0.0f, 1.0f,

		// colour & alpha of shadow
		xFogColour.m_fR, xFogColour.m_fG, xFogColour.m_fB, 0.25f*(1-fFogDensity) + xFogColour.m_fA,
	};

	Asura_Direct3D::D3DDevice->SetVertexShaderConstantF(  6, afShadowData,  2 );

	// set up stencil modes
	Asura_Render::CurrentStates.RequestStencilEnabled(true);
	Asura_Direct3D::D3DDevice->SetRenderState(D3DRS_STENCILPASS, D3DSTENCILOP_INVERT);
	Asura_Direct3D::D3DDevice->SetRenderState(D3DRS_STENCILFAIL, D3DSTENCILOP_KEEP);
	Asura_Direct3D::D3DDevice->SetRenderState(D3DRS_STENCILZFAIL, D3DSTENCILOP_KEEP);
	Asura_Direct3D::D3DDevice->SetRenderState(D3DRS_STENCILFUNC, D3DCMP_EQUAL);

	// set to no texture, single pass, normal translucency
	Asura_Render::CurrentStates.RequestTextureIndex(0, ASURA_TEXTURE_NONE);
	Asura_Render::CurrentStates.RequestTextureOpMode(ASURA_TEXTUREOP_SINGLE_PASS);
	Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_NORMAL);
	Asura_Render::CurrentStates.RequestAlphaTest(false);

	// draw the mesh...
	Asura_Direct3D::D3DDevice->DrawIndexedPrimitive(pxStrip->m_ePrimitiveType,0,0,pxStrip->m_uNumberOfVertices,0,pxStrip->m_uNumberOfTriangles);

	// turn off stencil
	Asura_Render::CurrentStates.RequestStencilEnabled(false);
	Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_OFF);
}

#if defined(ASURA_TARGET_SM3)
void Asura_Direct3D_ObjectRenderer::RenderShadow(const class Asura_Shape& xShape, const class Asura_Shape_Instance& xShapeInst, const class Asura_Shadow& xShadow)
{
	SCOPEDPIXEVENT((0xff58ba34, L"SM3 Object shadow (hash 0x%08X)", xShape.GetShapeHashID()));

	const Asura_Direct3D_Object_Strip* pxStrip = GetObjectStrip(xShape);

	const Asura_Vector_3& xPosition = xShapeInst.GetPosition();
	const Asura_Matrix_3x3& xOrientation = xShapeInst.GetOrientation();

	if (!pxStrip) { return; }

	if (!(pxStrip->m_pxVertexBuffer && pxStrip->m_pxIndexBuffer && pxStrip->m_uNumberOfTriangles))
	{
		return;
	}

	const Asura_Hash_ID uHash = pxStrip->m_auMaterialResponseHashID[0];
	const Asura_MaterialResponse* const pxMaterial =
		Asura_MaterialResponse_System::GetMaterialByHashID( uHash );

	if( pxMaterial && (pxMaterial->GetRenderType() == Asura_MaterialResponse::RENDER_TYPE_RENDER_BUT_DONT_CAST_SHADOW) ) return;

	Asura_Direct3D_Effect* pxEffect = Asura_Direct3D_Effects_System::BeginEffect("Object_Shadow");
	if (!pxEffect || !pxEffect->IsD3DEffectValid()) { return; }

	Asura_Render::AddToTrianglesRendered(pxStrip->m_uNumberOfTriangles);

	// Set up our render states
	Asura_Render::CurrentStates.RequestCullMode(ASURA_CULLMODE_CW);
	Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_OFF);
	Asura_Render::CurrentStates.RequestZBufferEnabled(true);
	Asura_Render::CurrentStates.RequestZBufferCompareMode(ASURA_ZBUFFERCMP_LESSEQUAL);
	Asura_Render::CurrentStates.RequestZBufferWriteEnabled(true);

	// Set up streams
	if (pxStrip->m_bNormalMapped)
	{
		Asura_Direct3D::D3DDevice->SetStreamSource(0,pxStrip->m_pxVertexBuffer,0,sizeof(Asura_Direct3D_ObjectVertex_NormalMapped));
		Asura_Direct3D::D3DDevice->SetVertexDeclaration(g_pxNormalMapVertexDecl);
	}
	else
	{
		Asura_Direct3D::D3DDevice->SetStreamSource(0,pxStrip->m_pxVertexBuffer,0,sizeof(Asura_Direct3D_ObjectVertex));
		Asura_Direct3D::D3DDevice->SetVertexDeclaration(g_pxStandardVertexDecl);
	}
	Asura_Direct3D::D3DDevice->SetIndices(pxStrip->m_pxIndexBuffer);

	D3DXMATRIX xViewProj, xTemp;

	const Asura_Vector_3& xLightPos = xShadow.GetLightPosition();
	D3DXMatrixTranslation( &xViewProj, -xLightPos.x, -xLightPos.y, -xLightPos.z );

	// Light orientation
	Asura_Matrix_3x3 xLightOri;
	xShadow.GetLightOrientation(xLightOri);
	memset( &xTemp, 0, sizeof( D3DXMATRIX ) );
	for( u_int u = 0; u < 3; ++u )
	{
		for( u_int v = 0; v < 3; ++v )
		{
			xTemp.m[u][v] = xLightOri.Element[u][v];
			if( v == 1 )
			{
				xTemp.m[u][v] = -xTemp.m[u][v];
			}
		}
	}
	xTemp._44 = 1.0f;
	xViewProj *= xTemp;

	// Projection
	const float fVerticalFOV = Asura_Maths::DegToRad( xShadow.GetFrustum().GetVerticalFOVInDegrees() );
	D3DXMatrixPerspectiveFovLH( &xTemp, fVerticalFOV, 1.0f, xShadow.GetFrustum().GetNearPlane(), xShadow.GetFrustum().GetFarPlane() );
	xViewProj *= xTemp;

	// Set up parameters. Shared should have been set up already by BeginFrame()
	D3DXMATRIX xWorld;
	xWorld._11 = xOrientation.e11;	xWorld._12 = xOrientation.e21;	xWorld._13 = xOrientation.e31;	xWorld._14 = 0;			 
	xWorld._21 = xOrientation.e12;	xWorld._22 = xOrientation.e22;	xWorld._23 = xOrientation.e32;	xWorld._24 = 0;			 
	xWorld._31 = xOrientation.e13;	xWorld._32 = xOrientation.e23;	xWorld._33 = xOrientation.e33;	xWorld._34 = 0;
	xWorld._41 = xPosition.x;		xWorld._42 = xPosition.y;		xWorld._43 = xPosition.z;		xWorld._44 = 1.0f;

	pxEffect->SetParameterByName("g_xWorldViewProjectionMatrix", xWorld * xViewProj);

	pxEffect->RenderAllIndexedPrimitivePasses(pxStrip->m_ePrimitiveType, 
		0,
		pxStrip->m_uNumberOfVertices,
		0,
		pxStrip->m_uNumberOfTriangles,
		NULL);

	// Flush any effects we may still be using...
	Asura_Direct3D_Effects_System::FlushEffects	();

	// Turn off any pixel shaders we may have used...
	Asura_Direct3D_Shaders::RequestPixelShader	(ASURA_D3D_PS_NONE);
	Asura_Direct3D::D3DDevice->SetPixelShader	(NULL);
}
#endif


#ifdef ASURA_TOOLS

void Asura_Direct3D_ObjectRenderer::ToolsRender(Asura_Mesh* pxMesh, const Asura_Vector_3& xPosition, const Asura_Matrix_3x3& xOrientation, bool bWithLighting, const float fAlpha/*=1.0f*/, const float fScale /*=1.0f*/, int iTextureID /*= ASURA_TEXTURE_NONE*/,u_int uColour /*0xffffff*/ )
{
	ToolsRender(pxMesh, xPosition, xOrientation, bWithLighting, fAlpha, Asura_Vector_3(fScale,fScale,fScale), iTextureID,uColour);
}


void Asura_Direct3D_ObjectRenderer::ToolsRender(Asura_Mesh* pxMesh, const Asura_Vector_3& xPosition, const Asura_Matrix_3x3& xOrientation, bool bWithLighting, const float fAlpha, const Asura_Vector_3& xScale, int iTextureID /*= ASURA_TEXTURE_NONE*/,u_int uColour /*0xffffff*/)
{
	SCOPEDPIXEVENT((0xffffaaaa, L"Object ToolsRender"));

	if(Asura_SortedRender::IsActive())
	{
		//send any meshes with alpha polygons through the sorted renderer.RAR 24/07/2006
		#if defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM

			const Asura_Hash_ID uMaterialHash = pxMesh->TextureDataPtr[0].m_auMaterialResponseHashID[0];
			const Asura_MaterialResponse* const pxMaterial = Asura_MaterialResponse_System::GetMaterialByHashID( uMaterialHash );
			const bool bTransparent = (pxMaterial && pxMaterial->GetBlendMode() != Asura_MaterialResponse::BLEND_MODE_OPAQUE);

		#else // !defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM

			int iMaterialIndex = pxMesh->TextureDataPtr[0].m_iMaterialID;
			const bool bTransparent = Asura_Material_System::IsMaterialTransparent( iMaterialIndex )
								   || Asura_Material_System::IsMaterialPS2AdditiveAlpha( iMaterialIndex );

		#endif // !defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM

		if(fAlpha != 1.0f || bTransparent)
		{
			Asura_SortedRender::AddToolsMesh(pxMesh,xPosition,xOrientation,bWithLighting,fAlpha,xScale,iTextureID);
			return;
		}
	}
	
	Asura_Direct3D_ObjectRender_Handler&	xHandler	= GetCurrentHandler();
	const Asura_Direct3D_Object_Strip*		pxStrip		= xHandler.GetObjectStrip(pxMesh);

	Asura_SphericalHarmonics_Order2_RGB xSH;

	if (bWithLighting)
	{
#if !defined( ASURA_TARGET_SM3)
		Asura_GI_Solution::GetTotalIrradiance( xSH, xPosition );
#else
		if( Asura_SortedRender::IsTransparentPass() )
		{
			// Transparent pass objects need the total irradiance since they
			// aren't lit by deferred lights
			Asura_GI_Solution::GetTotalIrradiance( xSH, xPosition );
		}
		else
		{
			Asura_GI_Solution::GetStaticIrradiance( xSH, xPosition );
		}
#endif
	}
	else
	{
		xSH.Zero();

		Asura_Vector_3 xV = *Asura_Camera::GetPosition() - xPosition;
		xV.Normalise();

		float fR = ((uColour>>16)&0xff)/255.0f;
		float fG = ((uColour>>8)&0xff)/255.0f;
		float fB = ((uColour)&0xff)/255.0f;

		xSH.AddSampleForDirection(fR, fG, fB, xV);
		xSH.ScaleCoefficientsForIrradiance();
	}

	Asura_GI_Solution::s_xSHForCurrentRender = xSH;

#if defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM
	//Material response system seems to use these for lighting the object , so
	//set with our fake lighting values. RAR 16/10/2007
	Asura_GI_Solution::s_xStaticSHForCurrentRender = xSH;
	Asura_GI_Solution::s_xDynamicSHForCurrentRender.Zero();
#endif

	Asura_Vector_2 xUVOffset(0.0f, 0.0f);

#if defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM
	Render_MaterialResponse( *pxStrip, xPosition, xOrientation, xScale );
#else

#ifdef ASURA_TARGET_SM3
	if (s_bUseNewRenderers)
	{
		//TBD - Use texture override. RAR 08/02/2006
		Asura_Vector_3 xExtents(pxMesh->BoundingBox.CalculateExtents());
		bool bUsePerPixelFog = (xExtents.x >= dbg_fObjectPerPixelFogMinDimension) || (xExtents.y >= dbg_fObjectPerPixelFogMinDimension) || (xExtents.z >= dbg_fObjectPerPixelFogMinDimension);
		RenderWithEffectSystem(pxStrip, xPosition, xOrientation, xUVOffset, fAlpha, true, xScale, bUsePerPixelFog);
	}
	else
#endif //#ifdef ASURA_TARGET_SM3
	{
		RenderBasic(pxStrip, xPosition, xOrientation, xUVOffset, fAlpha, true, xScale,iTextureID);
	}

#endif // !defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM
}

#if !defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM

void Asura_Direct3D_ObjectRenderer::ToolsRender2ndOrderSH(Asura_Mesh* pxMesh, const Asura_Vector_3& xPosition, const Asura_Matrix_3x3& xOrientation, const float fAlpha/*=1.0f*/ )
{
	Asura_Direct3D_ObjectRender_Handler&	xHandler	= GetCurrentHandler();
	const Asura_Direct3D_Object_Strip*		pxStrip		= xHandler.GetObjectStrip(pxMesh);

	if (!pxStrip) return;

		
	// turn on fog if we have to...
	if (Asura_Fog::IsFogRenderable())
	{
		Asura_Direct3D::D3DDevice->SetRenderState(D3DRS_FOGENABLE, TRUE);
	}

	Asura_Render::AddToTrianglesRendered(pxStrip->m_uNumberOfTriangles);
	Asura_Direct3D_Shaders::RequestVertexShader(ASURA_D3D_VS_OBJECT_ORDER2SH_UNCOMPRESSED);

#ifdef ASURA_TARGET_SM3
	if (pxStrip->m_bNormalMapped)
	{
		Asura_Direct3D::D3DDevice->SetStreamSource(0,pxStrip->m_pxVertexBuffer,0,sizeof(Asura_Direct3D_ObjectVertex_NormalMapped));
	}
	else
#endif //#ifdef ASURA_TARGET_SM3
	{
		Asura_Direct3D::D3DDevice->SetStreamSource(0,pxStrip->m_pxVertexBuffer,0,sizeof(Asura_Direct3D_ObjectVertex));
	}
	Asura_Direct3D::D3DDevice->SetIndices(pxStrip->m_pxIndexBuffer);

	Asura_Render::CurrentStates.RequestCullMode(ASURA_CULLMODE_CW); //CW



	Asura_Render::CurrentStates.RequestTextureIndex(0, Asura_Material_System::GetTextureID(pxStrip->m_iMaterialIndex));

	{
		D3DXMATRIX mat;
		D3DXMATRIX matView, matProj;

		Asura_Direct3D::D3DDevice->GetTransform( D3DTS_VIEW,       &matView );
		Asura_Direct3D::D3DDevice->GetTransform( D3DTS_PROJECTION,       &matProj );

        D3DXMatrixMultiply( &mat, &matView, &matProj );
        D3DXMatrixTranspose( &mat, &mat );

        Asura_Direct3D::D3DDevice->SetVertexShaderConstantF(  0, (float*)&mat,  4 );
	
		mat._11 = xOrientation.e11;
		mat._12 = xOrientation.e12;
		mat._13 = xOrientation.e13;
		mat._14 = xPosition.x;
		mat._21 = xOrientation.e21;
		mat._22 = xOrientation.e22;
		mat._23 = xOrientation.e23;
		mat._24 = xPosition.y;
		mat._31 = xOrientation.e31;
		mat._32 = xOrientation.e32;
		mat._33 = xOrientation.e33;
		mat._34 = xPosition.z;
		mat._41 = 0;
		mat._42 = 0;
		mat._43 = 0;
		mat._44 = 1.0f;
		Asura_Direct3D::D3DDevice->SetVertexShaderConstantF(  7, (float*)&mat,  4 );
	
	}
	
	{
		float afLights[4*3];

		Asura_SphericalHarmonics_Order2_RGB xSH;

//		Tools_PhotonMapping_System::GetRGBSHAtPoint( xPosition, xSH );
		Asura_GI_Solution::GetTotalIrradiance( xSH, xPosition );

		afLights[0] = xSH.m_afRedCoefficients[1]; 
		afLights[1] = xSH.m_afRedCoefficients[2]; 
		afLights[2] = xSH.m_afRedCoefficients[3]; 
		afLights[3] = xSH.m_afRedCoefficients[0]; 

		afLights[4] = xSH.m_afGreenCoefficients[1]; 
		afLights[5] = xSH.m_afGreenCoefficients[2]; 
		afLights[6] = xSH.m_afGreenCoefficients[3]; 
		afLights[7] = xSH.m_afGreenCoefficients[0]; 

		afLights[8] = xSH.m_afBlueCoefficients[1]; 
		afLights[9] = xSH.m_afBlueCoefficients[2];
		afLights[10] = xSH.m_afBlueCoefficients[3];
		afLights[11] = xSH.m_afBlueCoefficients[0];

		Asura_Direct3D::D3DDevice->SetVertexShaderConstantF( 11, afLights, 3 );
	}


	// uv animation 
	const float afUVOffset[4] = { 0.0f,0.0f,0.0f,0.0f };
	Asura_Direct3D::D3DDevice->SetVertexShaderConstantF( 19, afUVOffset, 1);

	// fog parameters 
	const float afFogValues[4] = { Asura_Fog::GetFogFarPlane(), Asura_Fog::GetOneOverFogPlaneSeparation(), 0.0f, 0.0f };
	Asura_Direct3D::D3DDevice->SetVertexShaderConstantF( 20, afFogValues, 1);

	/// Kevin Lea - 07/02/2003 15:33:24 - first off, is texture transparent
	if (Asura_Material_System::IsMaterialSelfIlluminating(pxStrip->m_iMaterialIndex))
	{
	
		Asura_Render::CurrentStates.RequestTextureIndex(1,Asura_Material_System::GetTextureID(pxStrip->m_iMaterialIndex));
		Asura_Render::CurrentStates.RequestTextureOpMode(ASURA_TEXTUREOP_SELFILLUMINATE);
		Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_OFF);
		Asura_Render::CurrentStates.RequestAlphaTest(false);
	}
	else if( Asura_Material_System::IsMaterialTransparent(pxStrip->m_iMaterialIndex) || (fAlpha < 1.0f) )
	{
		Asura_Render::CurrentStates.RequestTextureOpMode(ASURA_TEXTUREOP_SINGLE_PASS_2X);
		Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_NORMAL);
		Asura_Render::CurrentStates.RequestAlphaTest(true);
	}
	else
	{
		Asura_Render::CurrentStates.RequestTextureOpMode(ASURA_TEXTUREOP_SINGLE_PASS_2X);
		Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_OFF);
		Asura_Render::CurrentStates.RequestAlphaTest(false);
	}



	/// Kevin Lea - 07/02/2003 15:36:42 - draw base mesh
	Asura_Direct3D::D3DDevice->DrawIndexedPrimitive(pxStrip->m_ePrimitiveType,0,0,pxStrip->m_uNumberOfVertices,0,pxStrip->m_uNumberOfTriangles);


	Asura_Render::CurrentStates.RequestAlphaTest(false);

	Asura_Render::CurrentStates.RequestTextureOpMode(ASURA_TEXTUREOP_SINGLE_PASS);
	Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_OFF);

	
	// turn off fog if we have to...
	if (Asura_Fog::IsFogRenderable())
	{
		Asura_Direct3D::D3DDevice->SetRenderState(D3DRS_FOGENABLE, FALSE);
	}

}

#endif // !defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM

#endif

#if defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM

void Asura_Direct3D_ObjectRenderer::Render_MaterialResponse( const Asura_Direct3D_Object_Strip& xStrip,
															 const Asura_Vector_3& xPosition,
															 const Asura_Matrix_3x3& xOrientation,
															 const Asura_Vector_3& xScale )
{
	if( !xStrip.m_pxVertexBuffer || !xStrip.m_pxIndexBuffer || xStrip.m_uNumberOfTriangles == 0 )
	{
		return;
	}

	const bool bTransparentPass = Asura_SortedRender::IsTransparentPass();

	const Asura_Hash_ID uMaterialHash1 = xStrip.m_auMaterialResponseHashID[0];
	const Asura_MaterialResponse* pxMaterial1
		= Asura_MaterialResponse_System::GetMaterialByHashID( uMaterialHash1 );

	if( pxMaterial1 )
	{
		if( pxMaterial1->GetDontRender() ) return;

		const bool bTransparentStrip = pxMaterial1->IsTransparent();
		if( bTransparentPass != bTransparentStrip ) return;
	}

	SCOPEDPIXEVENT(( 0xFF58BA34, L"Object (Material Response)" ));

	const u_int uTechniqueIndex = bTransparentPass ? 1 : 0;

	Asura_Direct3D_Effect* const pxEffect = Asura_Direct3D_Effects_System::BeginEffect(
		"Object_MaterialResponse", uTechniqueIndex );

	if( !pxEffect || !pxEffect->IsD3DEffectValid() ) return;

	// Set main materials data texture
	{
		pxEffect->SetParameterByName( "g_xMaterialsTexture", Asura_Direct3D_MaterialResponse::GetMaterialsTexture());

		const D3DXVECTOR4 xMaterialProperties( 1.0f / ASURA_MATERIAL_RESPONSE_TEXTURE_WIDTH,
			Asura_Direct3D_MaterialResponse::GetMaterialsTextureScale(), 0.0f, 0.0f );

		pxEffect->SetParameterByName( "g_xMaterialsTextureProperties", xMaterialProperties );
	}

	// Set material data
	{
		const Asura_Hash_ID uMaterialHash2 = xStrip.m_auMaterialResponseHashID[1];
		const Asura_MaterialResponse* pxMaterial2 = Asura_MaterialResponse_System::GetMaterialByHashID( uMaterialHash2 );

		Asura_Direct3D_MaterialResponse::SetEffectParameters( pxMaterial1, pxMaterial2, NULL, *pxEffect );
		Asura_Direct3D_MaterialResponse::SetBlendStates( pxMaterial1 );
		Asura_Direct3D_MaterialResponse::SetSHParameters( pxMaterial1, *pxEffect );
	}

	// Set world and world-view-proj matrices
	{
		D3DXMATRIX xWorldViewProj;

		// Rotation
		Asura_Mem::Set( &xWorldViewProj, 0, sizeof( xWorldViewProj ) );
		for( u_int u = 0; u < 3; ++u )
		{
			for( u_int v = 0; v < 3; ++v )
			{
				xWorldViewProj.m[u][v] = xOrientation.Element[v][u] * xScale[u];
			}
		}

		xWorldViewProj._44 = 1.0f;

		// Translation
		D3DXMATRIX xTemp;
		D3DXMatrixTranslation( &xTemp, xPosition.x, xPosition.y, xPosition.z );
		xWorldViewProj *= xTemp;

		pxEffect->SetParameterByName( "g_xWorldMatrix", xWorldViewProj );

		// View-projection
		xWorldViewProj *= Asura_Direct3D::GetViewProjectionMatrix();

		pxEffect->SetParameterByName( "g_xWorldViewProjectionMatrix", xWorldViewProj );
	}

	// Set render states
	{
		Asura_Render::CurrentStates.RequestAlphaTest( false );
		Asura_Render::CurrentStates.RequestCullMode( ASURA_CULLMODE_CW );
		Asura_Render::CurrentStates.RequestDitheringMode( true );
		Asura_Render::CurrentStates.RequestDynamicLights( false );
		Asura_Render::CurrentStates.RequestZBufferCompareMode( ASURA_ZBUFFERCMP_LESSEQUAL );
		Asura_Render::CurrentStates.RequestZBufferEnabled( true );

		// Leave wireframe as it was set elsewhere
	}

	// Set up streams and draw

	Asura_Direct3D::D3DDevice->SetVertexDeclaration( g_pxNormalMapVertexDecl );
	Asura_Direct3D::D3DDevice->SetStreamSource( 0, xStrip.m_pxVertexBuffer, 0, sizeof( Asura_Direct3D_ObjectVertex_NormalMapped ) );
	Asura_Direct3D::D3DDevice->SetIndices( xStrip.m_pxIndexBuffer );

	pxEffect->RenderAllIndexedPrimitivePasses( xStrip.m_ePrimitiveType, 0, xStrip.m_uNumberOfVertices,
											   0, xStrip.m_uNumberOfTriangles, NULL );

	Asura_Render::AddToTrianglesRendered( xStrip.m_uNumberOfTriangles );

	for( u_int u = 0; u < 16; ++u )
	{
		Asura_Direct3D::D3DDevice->SetSamplerState( u, D3DSAMP_SRGBTEXTURE, FALSE );
	}

	const float fDepthBias = 0.0f;
	Asura_Direct3D::D3DDevice->SetRenderState( D3DRS_DEPTHBIAS, *(DWORD*)&fDepthBias );

	Asura_Direct3D::D3DDevice->SetRenderState( D3DRS_COLORWRITEENABLE1, D3DCOLORWRITEENABLE_RED | D3DCOLORWRITEENABLE_GREEN | D3DCOLORWRITEENABLE_BLUE | D3DCOLORWRITEENABLE_ALPHA );
	Asura_Direct3D_Effects_System::FlushEffects();
}

#endif // defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM
