//*******************************************************************
// File: Asura_Platform_DeferredShading.cpp
//
// Created: 21 February 2007 DLH
//
// Description: 
//
//*******************************************************************

// system libraries
#include "Core/Asura.h"

#if defined( ASURA_TARGET_SM3)

// main header
#include "Rendering/Asura_DeferredShading.h"

// Asura platform-independent
#include "Camera/Asura_Camera.h"
#include "Light/Asura_Photon.h"
#include "Rendering/Asura_Font_System.h"
#include "Rendering/Asura_Module.h"
#include "Rendering/Asura_Render.h"
#include "Rendering/Asura_Viewport.h"

// Asura PC
#include "Rendering/Asura_Direct3D.h"
#include "Rendering/Asura_Direct3D_Effects_System.h"
#include "Rendering/Asura_Direct3D_EffectParameterBlock.h"
#include "Rendering/Asura_Direct3D_MaterialResponse.h"
#include "Rendering/Asura_Direct3D_RenderTargetManager.h"
#include "Rendering/Asura_Direct3D_Shaders.h"

// Tools
#if defined( ASURA_TOOLS)
	#include "Tools_System.h"
	#include "Misc/Tools_Render.h"
	#include "Misc/Tools_Render_Options.h"
#endif


//*******************************************************************
// Constants
//*******************************************************************

enum
{
	// Effect Pass Indices
	DeferredLight_Pass_ClearDestAlpha	= 0,
	DeferredLight_Pass_Photon			= 1,			// point light
	DeferredLight_Pass_LimitedPhoton    = 2,			// photon with angular limits
	DeferredLight_Pass_Infinite			= 3,			// parallel light
};


//*******************************************************************
// Local Classes
//*******************************************************************
class Asura_Direct3D_DeferredPhotonShading_ParamBlock
{
public:
	D3DXMATRIX				m_xInvViewMatrix;
	D3DXMATRIX				m_xLightOrientation;	

	D3DXVECTOR4				m_xTwoOverScreen;
	D3DXVECTOR4				m_xCameraTanHalfFOV;
	D3DXVECTOR4				m_xLightDirection;	
	D3DXVECTOR4				m_xLightPosScale;
	D3DXVECTOR4				m_xLightColour;
	D3DXVECTOR4				m_xFalloff;
	D3DXVECTOR4				m_xMeshPosAndScale;

	Direct3D_Texture*		m_pxNormalDepthBuffer;
	Direct3D_Texture*		m_pxAlbedoBuffer;
	Direct3D_BaseTexture*   m_pxProjCubeMap;
	Direct3D_BaseTexture*   m_pxProjCubeMapBlurred;
	float					m_fProjCubeLerpDist;

	bool					m_bUseProjCube;
	bool					m_bUseDualProjCube;
	bool					m_bUseAngularRange;

};

class Asura_Direct3D_DeferredShading
{
//	friend class Asura_DeferredShading;

public:
	static void Initialise(void);
	static void Shutdown(void);
	static void Reset(void);	
	static void RenderPhotons(bool bWithDarkeningShadows, bool bWithRealShadows, bool bWithNoShadows);

	static void RenderPhotonOutlines_Callback( Asura_Photon* pxPhoton, void* pData );

	static	Direct3D_VertexBuffer*			   s_apxSphereVB[4];
	static int								   s_aiSphereNumVerts[4];

private:
	static void RenderPhotons_Callback( Asura_Photon* pxPhoton, void* pData);

	static Asura_Direct3D_Effect* 			   s_pxDeferredPhotonEffect;
	static Asura_Direct3D_Effect* 			   s_pxRenderNormalBufferEffect;
	static Direct3D_VertexDeclaration*		   s_pxDeferredPhotonVertexDeclaration;

	static Asura_Direct3D_EffectParameterBlock* s_pxDeferredPhotonParamBlock;

	
#if defined( ASURA_USE_MATERIAL_RESPONSE_SYSTEM ) || defined( ASURA_MATERIAL_RESPONSE_CHARACTERS_ONLY )
	static Asura_Direct3D_MaterialResponse_ShaderData s_xMaterialResponse_ShaderData;
#endif

};

void Asura_Direct3D_GetPhotonSphereVB(u_int uLOD, Direct3D_VertexBuffer *&pxVertexBuffer, u_int &uNumVerts)
{
	pxVertexBuffer = Asura_Direct3D_DeferredShading::s_apxSphereVB[uLOD];
	uNumVerts = Asura_Direct3D_DeferredShading::s_aiSphereNumVerts[uLOD];
	
}


//*******************************************************************
// Local Datas
//*******************************************************************

static const D3DVERTEXELEMENT9 axVERTEX_DECLARATION[] =
{
	{ 0,  0, D3DDECLTYPE_FLOAT3,    D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_POSITION,     0 },		// Vertex pos
	D3DDECL_END()
};

static Direct3D_VertexElement g_xaVSD_Deferred_DebugShowBuffers[] =
{
	{ 0,  0, D3DDECLTYPE_FLOAT3,    D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_POSITION,     0 },		// Vertex pos
	{ 0, 12, D3DDECLTYPE_FLOAT2,    D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_TEXCOORD,     0 },		// Light pos (xyz) and distance scale (w)
	D3DDECL_END()
};

static const char* const g_szPS_Deferred_DebugShowBuffers_Code =
	"ps_3_0								\n"
	"dcl_texcoord0 v0.xy				\n"
	"dcl_2d s0							\n"

	"texld r1, v0, s0					\n"
	"dp4 r0.r, r1, c0					\n"
	"dp4 r0.g, r1, c1					\n"
	"dp4 r0.b, r1, c2					\n"
	"dp4 r0.a, r1, c3					\n"
	"add r0, r0, c4					    \n"
	"mov oC0, r0						\n"

;
static const char* const g_szPS_Deferred_DebugShowBuffers_ABS_Code =
	"ps_3_0								\n"
	"dcl_texcoord0 v0.xy				\n"
	"dcl_2d s0							\n"

	"texld r1, v0, s0					\n"
	"abs r1, r1							\n"
	"dp4 r0.r, r1, c0					\n"
	"dp4 r0.g, r1, c1					\n"
	"dp4 r0.b, r1, c2					\n"
	"dp4 r0.a, r1, c3					\n"
	"add r0, r0, c4					    \n"
	"mov oC0, r0						\n"

;
static const char* const g_szPS_Deferred_DebugShowBuffers_Normals_Code =
	"ps_3_0								\n"
	"dcl_texcoord0 v0.xy				\n"
	"dcl_2d s0							\n"
	
	"def c6, 1, 1, 0, 0					\n"

	"texld r1, v0, s0					\n"
	// DLH - Mask XY
	"mul r0, r1, c6						\n"
	// DLH - (X*X) + (Y*Y)
	"dp3 r0.x, r0, r0					\n"
	// DLH - 1 - ((X*X) + (Y*Y))
	"add r0.x, c6.x, -r0.x				\n"
	// DLH - 1.0 / ((X*X) + (Y*Y))
	"rcp r0.x, r0.x						\n"
	// DLH - 1.0 / sqrt(1.0 / ((X*X) + (Y*Y))
	"rsq r0.x, r0.x						\n"
	
	// DLH - Sign
	"cmp r1.z, r1.z, r0.x, -r0.x		\n"
	
	"dp4 r0.r, r1, c0					\n"
	"dp4 r0.g, r1, c1					\n"
	"dp4 r0.b, r1, c2					\n"
	"dp4 r0.a, r1, c3					\n"
	"add r0, r0, c4					    \n"
	"mov oC0, r0						\n"

;
static const char* const g_szVS_Deferred_DebugShowBuffers_Code =
	"vs_3_0								\n"
	"dcl_position v0					\n"
	"dcl_texcoord0 v1					\n"
	
	"dcl_position o0					\n"
	"dcl_texcoord0 o1					\n"

	"mov o0, v0						\n"
	"mov o1, v1						\n"

;

// Register the shader with Asura
const Asura_Direct3D_PixelShader xPS_Deferred_DebugShowBuffers_Shader( ASURA_D3D_PS_DEFERRED_DEBUGSHOWBUFFERS, g_szPS_Deferred_DebugShowBuffers_Code, 3.0);
const Asura_Direct3D_PixelShader xPS_Deferred_DebugShowBuffers_Shader_ABS( ASURA_D3D_PS_DEFERRED_DEBUGSHOWBUFFERS_ABS, g_szPS_Deferred_DebugShowBuffers_ABS_Code, 3.0);
const Asura_Direct3D_PixelShader xPS_Deferred_DebugShowBuffers_Shader_Normals( ASURA_D3D_PS_DEFERRED_DEBUGSHOWBUFFERS_NORMALS, g_szPS_Deferred_DebugShowBuffers_Normals_Code, 3.0);

const Asura_Direct3D_VertexShader xVS_Deferred_DebugShowBuffers_Shader( ASURA_D3D_VS_DEFERRED_DEBUGSHOWBUFFERS, g_szVS_Deferred_DebugShowBuffers_Code, g_xaVSD_Deferred_DebugShowBuffers, 3.0);

Asura_Direct3D_Effect*  				Asura_Direct3D_DeferredShading::s_pxDeferredPhotonEffect;
Direct3D_VertexDeclaration*				Asura_Direct3D_DeferredShading::s_pxDeferredPhotonVertexDeclaration = NULL;
Asura_Direct3D_EffectParameterBlock*	Asura_Direct3D_DeferredShading::s_pxDeferredPhotonParamBlock = NULL;
static Asura_Direct3D_DeferredPhotonShading_ParamBlock s_xDeferredPhotonParamBlock;

Direct3D_VertexBuffer*					Asura_Direct3D_DeferredShading::s_apxSphereVB[4] = {0,0,0,0};
int										Asura_Direct3D_DeferredShading::s_aiSphereNumVerts[4] = {0,0,0,0};


#if defined( ASURA_USE_MATERIAL_RESPONSE_SYSTEM ) || defined( ASURA_MATERIAL_RESPONSE_CHARACTERS_ONLY )
Asura_Direct3D_MaterialResponse_ShaderData Asura_Direct3D_DeferredShading::s_xMaterialResponse_ShaderData;
#endif


//*******************************************************************
// Function Definitions
//*******************************************************************


// Asura_Direct3D_DeferredShading implementation

void Asura_Direct3D_DeferredShading::Initialise(void)
{
	HRESULT xResult;
	Direct3D_Device* pxD3DDevice = Asura_Direct3D::D3DDevice;
	if( NULL == pxD3DDevice)
	{
		return;
	}
	
	xResult = pxD3DDevice->CreateVertexDeclaration( axVERTEX_DECLARATION, &s_pxDeferredPhotonVertexDeclaration );
	ASURA_D3D_OK( xResult);
	Asura_Assert( NULL != s_pxDeferredPhotonVertexDeclaration, "Failed to create deferred photon vertex declaration." );

	Asura_Assert( NULL == s_pxDeferredPhotonEffect, "deferred photon shader already setup!" );
#if defined( ASURA_USE_MATERIAL_RESPONSE_SYSTEM ) || defined( ASURA_MATERIAL_RESPONSE_CHARACTERS_ONLY )
	s_pxDeferredPhotonEffect = Asura_Direct3D_Effects_System::GetEffect( "ApplyDeferredPhoton_MaterialResponse" );
#else
	s_pxDeferredPhotonEffect = Asura_Direct3D_Effects_System::GetEffect( "ApplyDeferredPhoton" );
#endif
	Asura_Assert( NULL != s_pxDeferredPhotonEffect, "Failed to create deferred photon shader" );
	if( NULL == s_pxDeferredPhotonEffect)
	{
		return;
	}

#if defined( ASURA_ASSERT )
	Direct3D_Effect* pxDeferredPhotonD3DEffect = 
#endif // #if defined( ASURA_ASSERT )
		s_pxDeferredPhotonEffect->GetD3DEffect();
	Asura_Assert( NULL != pxDeferredPhotonD3DEffect, "Failed to create deferred photon shader" );
	
	s_pxDeferredPhotonParamBlock = new Asura_Direct3D_EffectParameterBlock();
	s_pxDeferredPhotonParamBlock->AddDynamicParameter<D3DXMATRIX>( "g_xInvViewMatrix",					&s_xDeferredPhotonParamBlock.m_xInvViewMatrix);
	s_pxDeferredPhotonParamBlock->AddDynamicParameter<D3DXMATRIX>( "g_xLightOrientation",				&s_xDeferredPhotonParamBlock.m_xLightOrientation);
	s_pxDeferredPhotonParamBlock->AddDynamicParameter<D3DXVECTOR4>( "g_xTwoOverScreen",					&s_xDeferredPhotonParamBlock.m_xTwoOverScreen );
	s_pxDeferredPhotonParamBlock->AddDynamicParameter<D3DXVECTOR4>( "g_xCameraTanHalfFOV",				&s_xDeferredPhotonParamBlock.m_xCameraTanHalfFOV);
	s_pxDeferredPhotonParamBlock->AddDynamicParameter<Direct3D_Texture*>( "g_xNormalDepthBuffer",		&s_xDeferredPhotonParamBlock.m_pxNormalDepthBuffer);
	s_pxDeferredPhotonParamBlock->AddDynamicParameter<Direct3D_Texture*>( "g_xAlbedoBuffer",			&s_xDeferredPhotonParamBlock.m_pxAlbedoBuffer);	
	s_pxDeferredPhotonParamBlock->AddDynamicParameter<D3DXVECTOR4>( "g_xLightDirection",				&s_xDeferredPhotonParamBlock.m_xLightDirection);
	s_pxDeferredPhotonParamBlock->AddDynamicParameter<D3DXVECTOR4>( "g_xLightPosScale",					&s_xDeferredPhotonParamBlock.m_xLightPosScale);
	s_pxDeferredPhotonParamBlock->AddDynamicParameter<D3DXVECTOR4>( "g_xLightColour",					&s_xDeferredPhotonParamBlock.m_xLightColour);
	s_pxDeferredPhotonParamBlock->AddDynamicParameter<D3DXVECTOR4>( "g_xFalloff",						&s_xDeferredPhotonParamBlock.m_xFalloff);
	s_pxDeferredPhotonParamBlock->AddDynamicParameter<D3DXVECTOR4>( "g_xMeshScaleAndPos",				&s_xDeferredPhotonParamBlock.m_xMeshPosAndScale);
	s_pxDeferredPhotonParamBlock->AddDynamicParameter<Direct3D_BaseTexture*>( "g_xProjCubeMap",  		&s_xDeferredPhotonParamBlock.m_pxProjCubeMap);	
	s_pxDeferredPhotonParamBlock->AddDynamicParameter<Direct3D_BaseTexture*>( "g_xProjCubeMapBlurred",	&s_xDeferredPhotonParamBlock.m_pxProjCubeMapBlurred);	
	s_pxDeferredPhotonParamBlock->AddDynamicParameter<float>( "g_fProjCubeLerpDist",					&s_xDeferredPhotonParamBlock.m_fProjCubeLerpDist);
	
	// Return of the static branches... too many combinations to just use passes now, so there's static branches in the 'slow pass'
	s_pxDeferredPhotonParamBlock->AddDynamicParameter<bool>( "g_bUseProjCube",		 &s_xDeferredPhotonParamBlock.m_bUseProjCube);
	s_pxDeferredPhotonParamBlock->AddDynamicParameter<bool>( "g_bUseDualProjCube",	 &s_xDeferredPhotonParamBlock.m_bUseDualProjCube);
	s_pxDeferredPhotonParamBlock->AddDynamicParameter<bool>( "g_bUseAngularRange",	 &s_xDeferredPhotonParamBlock.m_bUseAngularRange);

	
#if defined( ASURA_USE_MATERIAL_RESPONSE_SYSTEM ) || defined( ASURA_MATERIAL_RESPONSE_CHARACTERS_ONLY )
	if( false == s_xMaterialResponse_ShaderData.AddDataToParameterBlock( s_pxDeferredPhotonParamBlock))
	{
		return;
	}
#endif

	s_pxDeferredPhotonParamBlock->Bind( s_pxDeferredPhotonEffect);


#if 0
	s_pxRenderNormalBufferEffect    = Asura_Direct3D_Effects_System::CreateEffectFromCompiledFile( "Shaders\\NormalBufferOverlayRenderer.fxo" );
	Asura_Assert( s_pxRenderNormalBufferEffect, "Failed to create normal map debug shader" );
#endif
#if defined( ASURA_USE_MATERIAL_RESPONSE_SYSTEM ) || defined( ASURA_MATERIAL_RESPONSE_CHARACTERS_ONLY )
	// Not implemented yet for material response builds
//	Asura_DeferredShading::SetUseDeferredPhotonsForObjects( false );
#endif // #if defined( ASURA_USE_MATERIAL_RESPONSE_SYSTEM ) || defined( ASURA_MATERIAL_RESPONSE_CHARACTERS_ONLY )


	// Create VBs for a sphere, used to render the deferred passes, at 4 different LOD levels
	for (int iLod=0; iLod<4; iLod++)
	{
		int   iSphereVerticalSegments = 10;
		int   iSphereRadialSegments   = 20;
		float fSphereRadiusScale      = 1.0f;

		switch(iLod)
		{
		case 3:
			fSphereRadiusScale		= 1.1f;
			iSphereVerticalSegments = 6;
			iSphereRadialSegments	= 12;				
			break;
		case 2:
			fSphereRadiusScale		= 1.075f;
			iSphereVerticalSegments = 8;
			iSphereRadialSegments	= 16;
			break;
		case 1:
			fSphereRadiusScale		= 1.05f;
			iSphereVerticalSegments = 10;
			iSphereRadialSegments   = 20;
			break;
		case 0:
			fSphereRadiusScale		= 1.025f;
			iSphereVerticalSegments = 16;
			iSphereRadialSegments   = 32;
			break;
		}

		int iNumSphereVerts = ((iSphereRadialSegments*4)+2) * iSphereVerticalSegments;		
		HRESULT hr = Asura_Direct3D::D3DDevice->CreateVertexBuffer( iNumSphereVerts * sizeof(Asura_Vector_3), D3DUSAGE_WRITEONLY, 0, D3DPOOL_MANAGED, &s_apxSphereVB[iLod], NULL );		
		Asura_Assert( !FAILED(hr) , "Fatal - Failed to create VB" );

		Asura_Vector_3 *pxVerts = 0;		
		hr = s_apxSphereVB[iLod]->Lock(0,0,(void **)&pxVerts,0);		
		Asura_Assert( !FAILED(hr) , "Fatal - Failed to lock VB" );

		Asura_DeferredShading::CreateSphere(pxVerts, iNumSphereVerts, iSphereRadialSegments, iSphereVerticalSegments, fSphereRadiusScale);				
		s_aiSphereNumVerts[iLod] = iNumSphereVerts;
		
		s_apxSphereVB[iLod]->Unlock();
	}

}

void Asura_Direct3D_DeferredShading::Shutdown(void)
{

//	Asura_Direct3D_Effects_System::Destroy( s_pxDeferredPhotonEffect );
	s_pxDeferredPhotonEffect = NULL;
	SAFE_RELEASE( s_pxDeferredPhotonVertexDeclaration);	
	delete s_pxDeferredPhotonParamBlock;
	s_pxDeferredPhotonParamBlock = NULL;

	for(int i=0;i<4;i++)
	{
		SAFE_RELEASE( s_apxSphereVB[i] );
	}
}


void Asura_Direct3D_DeferredShading::Reset(void)
{

}

void SetZBufferStates(const Asura_Vector_3 &xPos, bool bInside)
{	
	// Workaround for the cached states in Asura_Render::CurrentStates getting out-of-sync with D3D
	Asura_Render::CurrentStates.RequestCullMode( ASURA_CULLMODE_NONE );
	Asura_Render::CurrentStates.RequestZBufferCompareMode(ASURA_ZBUFFERCMP_ALWAYS);		

	if (bInside)		
	{
		Asura_Render::CurrentStates.RequestCullMode( ASURA_CULLMODE_CW );
		Asura_Render::CurrentStates.RequestZBufferCompareMode(ASURA_ZBUFFERCMP_GREATER);		
	}
	else
	{	
		Asura_Render::CurrentStates.RequestCullMode( ASURA_CULLMODE_CCW );		
		Asura_Render::CurrentStates.RequestZBufferCompareMode(ASURA_ZBUFFERCMP_LESSEQUAL);		
	}
}




class Asura_Direct3D_DeferredShading_RenderPhotons_Callback_Data
{
public:
	Asura_Vector_3		m_xCameraPos;
	Direct3D_Effect*	m_pxD3DEffect;
	#if !defined (ASURA_USE_NEW_VISIBILITY_SYSTEM)
	bool				m_bBoundingBoxValid;
	Asura_Bounding_Box	m_xBoundingBox;
	#endif
	bool				m_bWithDarkeningShadows;
	bool				m_bWithRealShadows;
	bool				m_bWithNoShadows;
	u_int				m_uPhotonsUsed;
};

void Asura_Direct3D_DeferredShading::RenderPhotons_Callback( Asura_Photon* pxPhoton, void* pData)
{
	HRESULT xResult;
	bool bInside = false;
	bool bRenderAsCuboid = false;
	int  iSphereLOD = 0;

	Asura_Direct3D_DeferredShading_RenderPhotons_Callback_Data* pxData = static_cast< Asura_Direct3D_DeferredShading_RenderPhotons_Callback_Data* >( pData);
	Asura_Photon& xPhoton = *pxPhoton;
	if( false == xPhoton.IsActive())
	{
		return;
	}
	
	float fRadius = xPhoton.GetRange();
	if( fRadius <= 0.0f)
	{
		return;
	}

	// Check this light has one of the types of shadows that we're rendering this pass
	bool bAccept = false;
	uint32 uFlags =  pxPhoton->GetShadowParameters().m_uFlags;

	if (pxData->m_bWithRealShadows)
	{
		if ((uFlags & (ASURA_SHADOWFLAG_CASTS_OWN_SHADOWS|ASURA_SHADOWFLAG_CASTS_ENVIRONMENT_SHADOWS)) && ((uFlags & ASURA_SHADOWFLAG_IS_DARKENING_PASS) == 0))
		{
			if (uFlags & ASURA_SHADOWFLAG_CASTS_ENVIRONMENT_SHADOWS)
			{
				if ((uFlags & ASURA_SHADOWFLAG_CONTRIBUTES_TO_AVERAGE_SHADOW) == 0)
				{
					bAccept = true;
				}
			} else {
				bAccept = true;
			}
		}
	}

	if (pxData->m_bWithDarkeningShadows)
	{
		if (((uFlags & (ASURA_SHADOWFLAG_CASTS_OWN_SHADOWS|ASURA_SHADOWFLAG_CASTS_ENVIRONMENT_SHADOWS)) && (uFlags & ASURA_SHADOWFLAG_IS_DARKENING_PASS)) || (uFlags & ASURA_SHADOWFLAG_CONTRIBUTES_TO_AVERAGE_SHADOW))
		{
			bAccept = true;
		}			
	}

	if (pxData->m_bWithNoShadows)
	{
		if (0 == (uFlags & ((ASURA_SHADOWFLAG_CASTS_OWN_SHADOWS|ASURA_SHADOWFLAG_CASTS_ENVIRONMENT_SHADOWS) | ASURA_SHADOWFLAG_CONTRIBUTES_TO_AVERAGE_SHADOW)))
		{
			bAccept = true;
		}			
	}

	if (!bAccept)
	{		
		return;
	}

	// Photons now support a distance fade-out, like the projectors
	float fDistanceFade = 1.0f;
	if (xPhoton.GetDistanceFadeEnable())
	{	
		Asura_Vector_3 xOfs = xPhoton.GetPosition()-pxData->m_xCameraPos;
		float fDist = xOfs.Magnitude();
		if (fDist> xPhoton.GetDistanceFadeStart())
		{
			fDistanceFade = 1.0f - ((fDist - xPhoton.GetDistanceFadeStart())/xPhoton.GetDistanceFadeDist());
		}

		if (fDistanceFade<=0.0f)
		{
			return;
		}
	}

#if defined (ASURA_USE_NEW_VISIBILITY_SYSTEM)
	if ( !pxPhoton->IsVisible() )
	{
		return;
	}
	
	const Asura_Vector_3& xPos = xPhoton.GetPosition();		

#else// !defined(ASURA_USE_NEW_VISIBILITY_SYSTEM )
	
	Asura_Vector_3 xPos = xPhoton.GetPosition();		
	
	if(true == pxData->m_bBoundingBoxValid)
	{
		const bool bIsPhotonInBoundingBox = pxData->m_xBoundingBox.IntersectsSphere( xPos, fRadius);
		if( false == bIsPhotonInBoundingBox)
		{
			return;
		}
	}
	
	const bool bIsPhotonInFrustrum = Asura_Camera::IsSphereInFrustum( xPos, fRadius);
	if( false == bIsPhotonInFrustrum)
	{
		return;
	}

#if defined ASURA_TOOLS
	// (AAM - 14.11.2007) In Tools, this test only makes sense if the environment is visible
	// (this was breaking in hierarchy mode, for instance)
	if( !Tools_System::IsActive() || !Tools_Render_Options::HideEnvironment )
#endif
	{
		Asura_Bounding_Box xBB;
		xBB.Reset();
		xBB.Set(xPos + Asura_Vector_3(fRadius,fRadius,fRadius));
		xBB.Set(xPos - Asura_Vector_3(fRadius,fRadius,fRadius));
		if (!Asura_Module_System::DoesBBIntersectVisibleModule(xBB, &pxPhoton->m_iLastKnownModule))
		{
			return;
		}
	}
#endif // !defined(ASURA_USE_NEW_VISIBILITY_SYSTEM )

	const Asura_Vector_3& xCameraPos = pxData->m_xCameraPos;
	Direct3D_Effect* pxD3DEffect = pxData->m_pxD3DEffect;
	Direct3D_Device* pxD3DDevice = Asura_Direct3D::D3DDevice;

	SCOPEDPIXEVENT((0xff00ff00, L"Deferred Photon: %.1f %.1f %.1f", xPos.GetX(), xPos.GetY(), xPos.GetZ()));

	Asura_Vector_3 xUp, xRight;
	Asura_Camera::GetUpVector(xUp);
	Asura_Camera::GetRightVector(xRight);

	Asura_Vector_3 xVerts[8];

	if (pxPhoton->GetCustomLightVolumeEnable())
	{
		// Rendering a cuboid in world-space
		bRenderAsCuboid = true;
		s_xDeferredPhotonParamBlock.m_xMeshPosAndScale.x = 0;
		s_xDeferredPhotonParamBlock.m_xMeshPosAndScale.y = 0;
		s_xDeferredPhotonParamBlock.m_xMeshPosAndScale.z = 0;
		s_xDeferredPhotonParamBlock.m_xMeshPosAndScale.w = 1;

		// Custom bounding box
		const Asura_Bounding_Box& xBB = pxPhoton->GetCustomLightVolume();
		const Asura_Quat& xOrient = pxPhoton->GetCustomLightVolumeOrient();

		xVerts[0].x = xBB.MinX;		xVerts[0].y = xBB.MinY;		xVerts[0].z = xBB.MinZ;
		xVerts[1].x = xBB.MaxX;		xVerts[1].y = xBB.MinY;		xVerts[1].z = xBB.MinZ;
		xVerts[2].x = xBB.MaxX;		xVerts[2].y = xBB.MaxY;		xVerts[2].z = xBB.MinZ;
		xVerts[3].x = xBB.MinX;		xVerts[3].y = xBB.MaxY;		xVerts[3].z = xBB.MinZ;
		xVerts[4].x = xBB.MinX;		xVerts[4].y = xBB.MinY;		xVerts[4].z = xBB.MaxZ;
		xVerts[5].x = xBB.MaxX;		xVerts[5].y = xBB.MinY;		xVerts[5].z = xBB.MaxZ;
		xVerts[6].x = xBB.MaxX;		xVerts[6].y = xBB.MaxY;		xVerts[6].z = xBB.MaxZ;
		xVerts[7].x = xBB.MinX;		xVerts[7].y = xBB.MaxY;		xVerts[7].z = xBB.MaxZ;

		// It's an oriented box, so rotate the verts	
		Asura_Vector_3 xCentre = xBB.CalculateCentre();		
		for(int i=0;i<8;i++)
		{
			xVerts[i] -= xCentre;
			xVerts[i] = xOrient.RotateVector(xVerts[i]);
			xVerts[i] += xCentre;
		}

		// Are we inside the volume?
		
		//need to get position relative to volume		
		Asura_Vector_3 xRelative = xCameraPos - xCentre;

		//and rotate into the space of the volume
		Asura_Quat xOrientInverse = xOrient;
        xOrientInverse.Invert();
		xRelative = xOrientInverse.RotateVector(xRelative);		
		Asura_Vector_3 xTestPos = xRelative+xCentre;
		
		float fAdjust = 0.1f;
		bInside = ((xTestPos.x>=xBB.MinX-fAdjust && xTestPos.x<=xBB.MaxX+fAdjust) &&
				   (xTestPos.y>=xBB.MinY-fAdjust && xTestPos.y<=xBB.MaxY+fAdjust) &&
				   (xTestPos.z>=xBB.MinZ-fAdjust && xTestPos.z<=xBB.MaxZ+fAdjust));

	}
	else
	{	
		if (Asura_DeferredShading::GetUseCubesForPhotons())
		{
			// Standard cube
			for(int i=0;i<8;i++)
			{
				// Rendering a cube in world-space
				bRenderAsCuboid = true;
				s_xDeferredPhotonParamBlock.m_xMeshPosAndScale.x = 0;
				s_xDeferredPhotonParamBlock.m_xMeshPosAndScale.y = 0;
				s_xDeferredPhotonParamBlock.m_xMeshPosAndScale.z = 0;
				s_xDeferredPhotonParamBlock.m_xMeshPosAndScale.w = 1;

				static const float afUp[8]    = {-1.0f, -1.0f,  1.0f,  1.0f, -1.0f, -1.0f, 1.0f,  1.0f};
				static const float afRight[8] = {-1.0f,  1.0f,  1.0f, -1.0f, -1.0f,  1.0f, 1.0f, -1.0f};
				static const float afFwd[8]   = {-1.0f, -1.0f, -1.0f, -1.0f,  1.0f,  1.0f, 1.0f,  1.0f};				

				xVerts[i].x = xPos.x + (afRight[i]*fRadius);
				xVerts[i].y = xPos.y + (afUp[i]*fRadius);
				xVerts[i].z = xPos.z + (afFwd[i]*fRadius);	
			}

			// Are we inside the volume?
			float fRadiusAdjusted = fRadius+0.1f;
			bInside = ((xCameraPos.x>=xPos.x-fRadiusAdjusted && xCameraPos.x<=xPos.x+fRadiusAdjusted) &&
					   (xCameraPos.y>=xPos.y-fRadiusAdjusted && xCameraPos.y<=xPos.y+fRadiusAdjusted) &&
					   (xCameraPos.z>=xPos.z-fRadiusAdjusted && xCameraPos.z<=xPos.z+fRadiusAdjusted));
		}
		else
		{
			// Rendering a sphere - use the offset+scale
			bRenderAsCuboid = false;
			s_xDeferredPhotonParamBlock.m_xMeshPosAndScale.x = xPos.x;
			s_xDeferredPhotonParamBlock.m_xMeshPosAndScale.y = xPos.y;
			s_xDeferredPhotonParamBlock.m_xMeshPosAndScale.z = xPos.z;
			s_xDeferredPhotonParamBlock.m_xMeshPosAndScale.w = fRadius;

			// Are we inside the volume?
			const Asura_Vector_3 &xCameraPos = pxData->m_xCameraPos;
			float fRadiusAdjusted = (fRadius*1.025f)+0.1f;
			Asura_Vector_3 xOfs = xCameraPos - xPos;
			float fDist = xOfs.Magnitude();
			bInside = ( fDist<fRadiusAdjusted );			

			// Pick a sphere LOD. Switch down quickly to the low LOD, as we only need a more round
			// sphere when the camera is about to move inside it, for a more accurate inside-outside test
			iSphereLOD = (int)(fDist*0.5f/fRadius);
			if (iSphereLOD>3)
			{
				iSphereLOD = 3;
			}		
			else if (iSphereLOD<0)
			{
				iSphereLOD = 0;
			}
		}
	}

	const uint16 au16Indices[] =
	{			
		0,1,2 ,2,3,0,
		1,5,6 ,6,2,1,
		7,6,5 ,5,4,7,
		0,3,7 ,7,4,0,
		4,5,1 ,1,0,4,
		7,3,2 ,2,6,7
	};




	// Is this a directional light?
	int iPass = DeferredLight_Pass_Photon;
	if (pxPhoton->GetIsDirectional() || pxPhoton->GetLimitAnglesEnabled() || pxPhoton->GetProjCubeTextureIndex()!=ASURA_TEXTURE_NONE)
	{		
		const Asura_Vector_3& xDir = pxPhoton->GetLightDirection();
		s_xDeferredPhotonParamBlock.m_xLightDirection.x = xDir.x;
		s_xDeferredPhotonParamBlock.m_xLightDirection.y = xDir.y;
		s_xDeferredPhotonParamBlock.m_xLightDirection.z = xDir.z;

		if (pxPhoton->GetIsDirectional())
		{
			iPass = DeferredLight_Pass_Infinite;
		}
		else
		{
			iPass = DeferredLight_Pass_LimitedPhoton;
		}
	}	


	// -----------------------------------------------------------------------------------------------------------------------
	// DMR - 03/04/07 - Shadow support.
	// If this light is a shadow caster, we need to build the shadow mask now (in the destination alpha channel), and set the
	// appropriate blending mode.
	//
	// Before projecting the shadows, we need to clear the area of the shadow mask that's used by this light - we do that by
	// rendering the light volume, with colour writes only enabled to the alpha channel
	// -----------------------------------------------------------------------------------------------------------------------
	bool bShadows = false;
	if (pxPhoton->GetShadowParameters().m_uFlags & (ASURA_SHADOWFLAG_CASTS_OWN_SHADOWS|ASURA_SHADOWFLAG_CASTS_ENVIRONMENT_SHADOWS))
	{	
		// Are there any active shadows for this light?
		if (Asura_Shadow_System::LightHasActiveShadows(ASURA_SHADOW_LIGHTSOURCE_PHOTON,pxPhoton->GetHandle()))
		{	
			SCOPEDPIXEVENT((0xff00ff00, L"Shadow: %.1f %.1f %.1f", xPos.GetX(), xPos.GetY(), xPos.GetZ()));
			// Set this flag, so we know to restore the renderstates
			bShadows = true;

			// First, clear the destination alpha to 0 for the area affected by our shadows			
			Asura_Direct3D::D3DDevice->SetRenderState( D3DRS_COLORWRITEENABLE, D3DCOLORWRITEENABLE_ALPHA);
			Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_OFF); 			
			SetZBufferStates(xPos, bInside);
						
			// Render the light volume, filling the dest. alpha with 0			
			s_pxDeferredPhotonParamBlock->Apply();
			xResult = pxD3DEffect->BeginPass( DeferredLight_Pass_ClearDestAlpha );
			ASURA_D3D_OK( xResult);
			xResult = pxD3DEffect->CommitChanges();								
			ASURA_D3D_OK( xResult);


			if (bRenderAsCuboid)
			{
				xResult = pxD3DDevice->DrawIndexedPrimitiveUP(D3DPT_TRIANGLELIST,0,8,12, (void*)au16Indices, D3DFMT_INDEX16, (void *)xVerts , sizeof(Asura_Vector_3));
			}
			else
			{
				pxD3DDevice->SetStreamSource(0, s_apxSphereVB[iSphereLOD],0, sizeof (Asura_Vector_3));
				xResult = pxD3DDevice->DrawPrimitive(D3DPT_TRIANGLESTRIP,0,s_aiSphereNumVerts[iSphereLOD]-2);
				pxD3DDevice->SetStreamSource(0, NULL, 0, 0);
			}

			ASURA_D3D_OK( xResult);
			xResult = pxD3DEffect->EndPass();
			ASURA_D3D_OK( xResult);
			s_pxDeferredPhotonEffect->End();
			
			// Then render the shadow mask into the destination alpha
			Asura_Shadow_System::BuildShadowMask(ASURA_SHADOW_LIGHTSOURCE_PHOTON, pxPhoton->GetHandle());			
			Asura_Direct3D::ClearRenderTargets( 1);

			// Re-enable RGB writes
			Asura_Direct3D::D3DDevice->SetRenderState( D3DRS_COLORWRITEENABLE, D3DCOLORWRITEENABLE_ALPHA|D3DCOLORWRITEENABLE_RED|D3DCOLORWRITEENABLE_GREEN|D3DCOLORWRITEENABLE_BLUE);

			// Restore the renderstates we need for the deferred photon pass
			xResult = pxD3DDevice->SetVertexDeclaration( s_pxDeferredPhotonVertexDeclaration );
			ASURA_D3D_OK( xResult);

			Asura_Render::CurrentStates.RequestAlphaTest( false );	
			Asura_Render::CurrentStates.RequestZBufferEnabled( true );		
			Asura_Render::CurrentStates.RequestZBufferWriteEnabled( false );
			Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_ADD); 

			// Asura doesn't have a mode for this, so I'm changing it manually here, then restoring it when we're done
			Asura_Direct3D::D3DDevice->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_INVDESTALPHA );

			// Re-start rendering with the deferred photon shader
			s_pxDeferredPhotonEffect->Begin();			
		}
	}

	
	SetZBufferStates(xPos, bInside);
	
	
	s_xDeferredPhotonParamBlock.m_bUseProjCube = (ASURA_TEXTURE_NONE != pxPhoton->GetProjCubeTextureIndex());

	if (s_xDeferredPhotonParamBlock.m_bUseProjCube)
	{	
		s_xDeferredPhotonParamBlock.m_pxProjCubeMap = Asura_Direct3D::GetDirect3DTextureByIndex(pxPhoton->GetProjCubeTextureIndex());		

		s_xDeferredPhotonParamBlock.m_bUseDualProjCube = (ASURA_TEXTURE_NONE != pxPhoton->GetProjCubeBlurredTextureIndex());

		if (s_xDeferredPhotonParamBlock.m_bUseDualProjCube)
		{
			s_xDeferredPhotonParamBlock.m_pxProjCubeMapBlurred = Asura_Direct3D::GetDirect3DTextureByIndex(pxPhoton->GetProjCubeBlurredTextureIndex());		
			s_xDeferredPhotonParamBlock.m_fProjCubeLerpDist = pxPhoton->GetProjCubeLerpDist();
		}
	 	
		const Asura_Matrix_3x3& xPhotonOrient = pxPhoton->GetLightOrientation();
		for(int i=0;i<3;i++)
		{
			for(int j=0;j<3;j++)
			{
				s_xDeferredPhotonParamBlock.m_xLightOrientation.m[i][j] = xPhotonOrient.Element[i][j];				
			}			
		}
	}
	else
	{
		s_xDeferredPhotonParamBlock.m_pxProjCubeMap = NULL;
		s_xDeferredPhotonParamBlock.m_pxProjCubeMapBlurred = NULL;
	}

	s_xDeferredPhotonParamBlock.m_xFalloff.x = pxPhoton->GetFalloffScale();
	s_xDeferredPhotonParamBlock.m_xFalloff.y = pxPhoton->GetFalloffPower();

	s_xDeferredPhotonParamBlock.m_bUseAngularRange = pxPhoton->GetLimitAnglesEnabled();
	if (s_xDeferredPhotonParamBlock.m_bUseAngularRange)
	{	
		// Handle angular fade - same way as spotlights
		s_xDeferredPhotonParamBlock.m_xFalloff.z = -pxPhoton->GetLimitAngleOuter();
		s_xDeferredPhotonParamBlock.m_xFalloff.w = -(pxPhoton->GetLimitAngleOuter() - pxPhoton->GetLimitAngleInner());
		if (s_xDeferredPhotonParamBlock.m_xFalloff.w != 0.0f)
		{
			s_xDeferredPhotonParamBlock.m_xFalloff.w = 1.0f / s_xDeferredPhotonParamBlock.m_xFalloff.w;
		}
		else
		{
			// Nasty case where Inner angle == Outer angle
			s_xDeferredPhotonParamBlock.m_xFalloff.w = 1e15f;
		}
	}

	{
		const Asura_Vector_3& xPosition = pxPhoton->GetPosition();
		s_xDeferredPhotonParamBlock.m_xLightPosScale.x = xPosition.GetX();
		s_xDeferredPhotonParamBlock.m_xLightPosScale.y = xPosition.GetY();
		s_xDeferredPhotonParamBlock.m_xLightPosScale.z = xPosition.GetZ();
		s_xDeferredPhotonParamBlock.m_xLightPosScale.w = 1.0f / pxPhoton->GetRange();
	}

	{
		const float fR = fDistanceFade * pxPhoton->GetRedComponent();
		const float fG = fDistanceFade * pxPhoton->GetGreenComponent();
		const float fB = fDistanceFade * pxPhoton->GetBlueComponent();
		s_xDeferredPhotonParamBlock.m_xLightColour.x = fR;
		s_xDeferredPhotonParamBlock.m_xLightColour.y = fG;
		s_xDeferredPhotonParamBlock.m_xLightColour.z = fB;
		s_xDeferredPhotonParamBlock.m_xLightColour.w = (fR + fG + fB) / 3.0f;

		// (AAM - 07.01.2008) Roll a x2 from the pixel shader in here
		s_xDeferredPhotonParamBlock.m_xLightColour.x *= 2.0f;
		s_xDeferredPhotonParamBlock.m_xLightColour.y *= 2.0f;
		s_xDeferredPhotonParamBlock.m_xLightColour.z *= 2.0f;
	}

	s_pxDeferredPhotonParamBlock->Apply();

	xResult = pxD3DEffect->BeginPass( iPass );
	ASURA_D3D_OK( xResult);
	
	xResult = pxD3DEffect->CommitChanges();
	ASURA_D3D_OK( xResult);

	if (bRenderAsCuboid)
	{
		xResult = pxD3DDevice->DrawIndexedPrimitiveUP(D3DPT_TRIANGLELIST,0,8,12, (void*)au16Indices, D3DFMT_INDEX16, (void *)xVerts , sizeof(Asura_Vector_3));
	}
	else
	{
		pxD3DDevice->SetStreamSource(0, s_apxSphereVB[iSphereLOD],0, sizeof (Asura_Vector_3));
		xResult = pxD3DDevice->DrawPrimitive(D3DPT_TRIANGLESTRIP,0,s_aiSphereNumVerts[iSphereLOD]-2);
		pxD3DDevice->SetStreamSource(0, NULL, 0, 0);
	}

	ASURA_D3D_OK( xResult);
	
	xResult = pxD3DEffect->EndPass();
	ASURA_D3D_OK( xResult);

	if (bShadows)
	{
		// If we've been rendering with shadows, restore states, so we aren't doing the multiply by the dest alpha	
		Asura_Direct3D::D3DDevice->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_ONE );
	}

	for( u_int u = 0; u < 16; ++u )
	{
		Asura_Direct3D::D3DDevice->SetSamplerState( u, D3DSAMP_SRGBTEXTURE, FALSE );
	}
	
	pxData->m_uPhotonsUsed += 1;
}

void Asura_Direct3D_DeferredShading::RenderPhotons(bool bWithDarkeningShadows, bool bWithRealShadows, bool bWithNoShadows)
{
	Asura_Direct3D_DeferredShading_RenderPhotons_Callback_Data xCallbackData;
	
	xCallbackData.m_uPhotonsUsed = 0;
	#if !defined (ASURA_USE_NEW_VISIBILITY_SYSTEM)
	xCallbackData.m_bBoundingBoxValid = false;
	#endif
	xCallbackData.m_bWithDarkeningShadows = bWithDarkeningShadows;
	xCallbackData.m_bWithRealShadows	  = bWithRealShadows;
	xCallbackData.m_bWithNoShadows		  = bWithNoShadows;

#if !defined (ASURA_USE_NEW_VISIBILITY_SYSTEM)
	Asura_Bounding_Box& xBB = xCallbackData.m_xBoundingBox;

#if 1
	// Get a bounding box around all visible modules
	xBB.MinX=xBB.MinY=xBB.MinZ = 1e12f;
	xBB.MaxX=xBB.MaxY=xBB.MaxZ = -1e12f;
	
	{
		const u_int uModuleCount = Asura_Module_System::GetNumberOfModules();
		for( u_int uModuleIndex = 0; uModuleIndex < uModuleCount; uModuleIndex += 1)
		{
			if( false == Asura_Module_System::IsModuleVisible( uModuleIndex))
			{
				continue;
			}
			
		 	Asura_Bounding_Box xModuleBoundingBox = Asura_Module_System::GetBoundingBoxOfModule( uModuleIndex);
			xModuleBoundingBox.Displace( Asura_Module_System::GetPositionOfModule(uModuleIndex) );

		 	xBB.Set( xModuleBoundingBox);
			xCallbackData.m_bBoundingBoxValid = true;
		}
	}
#else
	xBB.MinX=xBB.MinY=xBB.MinZ = -1e12f;
	xBB.MaxX=xBB.MaxY=xBB.MaxZ = 1e12f;
#endif
	
	const Asura_Vector_3 xBBCentre = xBB.CalculateCentre();

#if 0
	Asura_Photon_System::MakePhotonListForBoundingBox( xBB, xBBCentre, false, true );

	if( 0 == Asura_Photon_System::NumberOfLocalPhotons)
	{
		return;
	}
#endif
	
#endif // #if !defined (ASURA_USE_NEW_VISIBILITY_SYSTEM)

	HRESULT xResult;
	Direct3D_Device* pxD3DDevice = Asura_Direct3D::D3DDevice;
	
	Asura_Direct3D_RenderTarget* pxDeferredRenderTarget = Asura_Direct3D::GetDeferredRenderingTarget();
	if( NULL == pxDeferredRenderTarget)
	{
		return;
	}
	
	Direct3D_Surface* pxDeferredRenderTargetFrontSurface = pxDeferredRenderTarget->GetSurface( ASURA_DIRECT3D_DEFFEDRENDERTARGET_FRONT_INDEX);
	if( NULL == pxDeferredRenderTargetFrontSurface)
	{
		return;
	}
		
	Direct3D_Texture* pxDeferredRenderTargetNormalTexture = pxDeferredRenderTarget->GetTexture( ASURA_DIRECT3D_DEFFEDRENDERTARGET_NORMAL_INDEX);
	if( NULL == pxDeferredRenderTargetNormalTexture)
	{
		return;
	}
		
	Direct3D_Texture* pxDeferredRenderTargetAlbedoTexture = pxDeferredRenderTarget->GetTexture( ASURA_DIRECT3D_DEFFEDRENDERTARGET_ALBEDO_INDEX);
	if( NULL == pxDeferredRenderTargetAlbedoTexture)
	{
		return;
	}
		
	Asura_Direct3D_Effect* pxEffect = s_pxDeferredPhotonEffect;
	if( NULL == pxEffect)
	{
		return;
	}
	Direct3D_Effect* pxD3DEffect = pxEffect->GetD3DEffect();
	if( NULL == pxD3DEffect)
	{
		return;
	}
	xCallbackData.m_pxD3DEffect = pxD3DEffect;
	
	SCOPEDPIXEVENT((0xff00ff00, L"Deferred Photons"));

	Asura_Render::CurrentStates.RequestAlphaTest( false );	
	Asura_Render::CurrentStates.RequestZBufferEnabled( true );		
	Asura_Render::CurrentStates.RequestZBufferWriteEnabled( false );
	Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_ADD); 

	Asura_Direct3D::ClearRenderTargets( 1);

	xResult = pxD3DDevice->SetRenderTarget( 0, pxDeferredRenderTargetFrontSurface);
	ASURA_D3D_OK( xResult);
	
	xResult = pxD3DDevice->SetVertexDeclaration( s_pxDeferredPhotonVertexDeclaration );
	ASURA_D3D_OK( xResult);

	s_pxDeferredPhotonEffect->Begin();

	// Set the shader params (constant for all lights)
	D3DXMATRIX xViewMatrix;
	D3DXMATRIX xViewMatrixInv;
	xViewMatrix = Asura_Direct3D::GetViewMatrix();	
	D3DXMatrixInverse(&xViewMatrixInv,NULL,&xViewMatrix);
	
	s_xDeferredPhotonParamBlock.m_xInvViewMatrix = xViewMatrixInv;
	
	float fCameraTanHalfVertFOV	= Asura_Maths::Tangent( Asura_Camera::GetVerticalFieldOfView() * 0.5f );
	float fCameraTanHalfHorizFOV= fCameraTanHalfVertFOV * Asura_Camera::GetXOverYAspectRatio();
	float fTwoOverScreenWidth	= 2.0f / Asura_Viewport::GetWidth();
	float fTwoOverScreenHeight	= 2.0f / Asura_Viewport::GetHeight();	
	
	s_xDeferredPhotonParamBlock.m_xCameraTanHalfFOV = D3DXVECTOR4( fCameraTanHalfHorizFOV, fCameraTanHalfVertFOV, 0, 0);
	s_xDeferredPhotonParamBlock.m_xTwoOverScreen = D3DXVECTOR4( fTwoOverScreenWidth, fTwoOverScreenHeight, 0, 0);

	s_xDeferredPhotonParamBlock.m_pxAlbedoBuffer = pxDeferredRenderTargetAlbedoTexture;
	s_xDeferredPhotonParamBlock.m_pxNormalDepthBuffer = pxDeferredRenderTargetNormalTexture;
			
	s_xMaterialResponse_ShaderData.m_bUseSpecularMap = true;
	s_xMaterialResponse_ShaderData.SetupCommon();

	xCallbackData.m_xCameraPos = *Asura_Camera::GetPosition();
	
	Asura_Photon_System::TraverseAll( Asura_Direct3D_DeferredShading::RenderPhotons_Callback, static_cast< void*>( &xCallbackData), false, true);

	// JAT 25/02/07 - this is nasty but until the asura_render class understands renderstate changes by effects then we are stuck
	// Don't leave render targets bound to the effect - can cause bad things to happen when resetting the device?		
	s_xDeferredPhotonParamBlock.m_pxAlbedoBuffer = NULL;
	s_xDeferredPhotonParamBlock.m_pxNormalDepthBuffer = NULL;
	s_pxDeferredPhotonParamBlock->Apply();
	xResult = pxD3DEffect->CommitChanges();
	ASURA_D3D_OK( xResult);

	s_pxDeferredPhotonEffect->End();
	
	Asura_Render::CurrentStates.RequestCullMode( ASURA_CULLMODE_CCW );
	Asura_Render::CurrentStates.RequestZBufferCompareMode(ASURA_ZBUFFERCMP_LESSEQUAL);
	Asura_Render::CurrentStates.RequestZBufferEnabled( true );		
	Asura_Render::CurrentStates.RequestZBufferWriteEnabled( true );
	Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_OFF); 

	// Flush any effects we may still be using...
	Asura_Direct3D_Effects_System::FlushEffects();
	Asura_Direct3D::GetDeferredRenderingTarget()->Bind();
}

void Asura_Direct3D_DeferredShading::RenderPhotonOutlines_Callback( Asura_Photon* pxPhoton, void* pData )
{
	pxPhoton->RenderLightVolume();
}


void Asura_DeferredShading::Platform_Initialise()
{
	Asura_Direct3D_DeferredShading::Initialise();
}

void Asura_DeferredShading::Platform_Shutdown()
{
	Asura_Direct3D_DeferredShading::Shutdown();
}

void Asura_DeferredShading::Platform_Reset()
{
	Asura_Direct3D_DeferredShading::Reset();
}

void Asura_DeferredShading::Platform_BeginRender(void)
{
	D3DPERFCALL(BeginEvent(D3DCOLOR_XRGB(0xFF, 0xFF, 0xFF), L"Deferred Shading"));
}

void Asura_DeferredShading::Platform_RenderPhotons(bool bWithDarkeningShadows, bool bWithRealShadows, bool bWithNoShadows)
{
	if (bWithDarkeningShadows)
	{	
		if (Asura_DeferredShading::IsRenderingPhotonOutlines())
		{	
			Asura_Photon_System::TraverseAll( Asura_Direct3D_DeferredShading::RenderPhotonOutlines_Callback, NULL, false, true);
		}
	}

	Asura_Direct3D_DeferredShading::RenderPhotons(bWithDarkeningShadows, bWithRealShadows, bWithNoShadows);
}

void Asura_DeferredShading::Platform_RenderDebugOutput()
{

	if( false == Asura_DeferredShading::IsShowingDeferredBuffers())
	{
		return;
	}
	
	{
		SCOPEDPIXEVENT((0xff00ff00, L"Show Deferred Buffers"));
		HRESULT xResult;
		Direct3D_Device* pxD3DDevice = Asura_Direct3D::D3DDevice;
		
		Direct3D_Surface* pxDSTSurface = NULL;
		Direct3D_SwapChain* pxChain = Asura_Direct3D::GetSwapChain( 0);
		if( NULL != pxChain)
		{
			xResult = pxChain->GetBackBuffer(0, D3DBACKBUFFER_TYPE_MONO, &pxDSTSurface);
			ASURA_D3D_OK( xResult);
		}
		
		if( NULL != pxDSTSurface) 
		{
			Asura_Direct3D_RenderTarget* pxDeferredRenderingTarget = Asura_Direct3D::GetDeferredRenderingTarget();
			//u_int uMRTCount = pxDeferredRenderingTarget->GetMRTCount();
			
			Asura_Render::CurrentStates.RequestTextureIndex( 0, ASURA_TEXTURE_INVALID);
			Asura_Render::CurrentStates.RequestCullMode(ASURA_CULLMODE_NONE);
			Asura_Direct3D_Shaders::RequestFixedFunctionVertexShader(D3DFVF_QUADVERTEX);
			Asura_Render::CurrentStates.RequestZBufferEnabled( false);
			Asura_Render::CurrentStates.RequestZBufferWriteEnabled(false);
			Asura_Render::CurrentStates.RequestTranslucencyMode( ASURA_TRANSLUCENCY_OFF);
			Asura_Render::CurrentStates.RequestAlphaTest(false);
			Asura_Render::CurrentStates.RequestWireFrameMode(false);

			Asura_Direct3D::ClearRenderTargets( 1);

			Asura_Direct3D_Shaders::RequestVertexShader( ASURA_D3D_VS_DEFERRED_DEBUGSHOWBUFFERS);
			
			enum DrawTextureIndex
			{
				DT_FRONT,
				DT_ALBEDO,
				DT_NORMAL,
				DT_MATERIAL,
				DT_SIZE
			};
			
			Direct3D_Texture* apxDrawTextures[ DT_SIZE] = 
			{
				pxDeferredRenderingTarget->GetTexture( ASURA_DIRECT3D_DEFFEDRENDERTARGET_FRONT_INDEX),
				pxDeferredRenderingTarget->GetTexture( ASURA_DIRECT3D_DEFFEDRENDERTARGET_ALBEDO_INDEX),
				pxDeferredRenderingTarget->GetTexture( ASURA_DIRECT3D_DEFFEDRENDERTARGET_NORMAL_INDEX),
				Asura_Direct3D_MaterialResponse::GetMaterialsTexture()
			};
			
			struct DrawTarget
			{
				float m_xStart[2];
				float m_xEnd[2];
				ASURA_D3D_PIXELSHADER_ID m_ePixelShader;
				DrawTextureIndex m_eDrawTargetTexture;
				D3DTEXTUREFILTERTYPE m_eFilter;
				float m_afPSConsts[5][4];
				const char* m_szLabel;
				u_int m_uLabelColour;
			};
			
			const static float fOutputSizeTotalX = 4.0f;
			const static float fOutputSizeTotalY = 1.0f;
			const static float fScaleDistance = 1.0f / 50.0f;
			static DrawTarget axDrawTargets[] = 
			{
				/* RGB */   { { 0.0f, 0.0f}, { 1.0f, 1.0f}, ASURA_D3D_PS_DEFERRED_DEBUGSHOWBUFFERS,         DT_FRONT,  D3DTEXF_LINEAR, { { 1.0f, 0.0f, 0.0f, 0.0f}, { 0.0f, 1.0f, 0.0f, 0.0f}, { 0.0f, 0.0f, 1.0f, 0.0f}, { 0.0f, 0.0f, 0.0f, 1.0f}, { 0.0f, 0.0f, 0.0f, 0.0f}}, "Base", 0xFFFFFFFF},
				/* .1 RGB */{ { 0.8f, 0.0f}, { 1.0f, 0.2f}, ASURA_D3D_PS_DEFERRED_DEBUGSHOWBUFFERS,         DT_FRONT,  D3DTEXF_LINEAR, { { 0.1f, 0.0f, 0.0f, 0.0f}, { 0.0f, 0.1f, 0.0f, 0.0f}, { 0.0f, 0.0f, 0.1f, 0.0f}, { 0.0f, 0.0f, 0.0f, 0.1f}, { 0.0f, 0.0f, 0.0f, 0.0f}}, ""},
				/*-.1 RGB */{ { 0.8f, 0.8f}, { 1.0f, 1.0f}, ASURA_D3D_PS_DEFERRED_DEBUGSHOWBUFFERS,         DT_FRONT,  D3DTEXF_LINEAR, { {-1.0f, 0.0f, 0.0f, 0.0f}, { 0.0f,-1.0f, 0.0f, 0.0f}, { 0.0f, 0.0f,-1.0f, 0.0f}, { 0.0f, 0.0f, 0.0f,-1.0f}, { 0.0f, 0.0f, 0.0f, 0.0f}}, ""},
				/*INF RGB */{ { 0.8f, 0.2f}, { 1.0f, 0.4f}, ASURA_D3D_PS_DEFERRED_DEBUGSHOWBUFFERS,         DT_FRONT,  D3DTEXF_LINEAR, { { .01f, 0.0f, 0.0f, 0.0f}, { 0.0f, .01f, 0.0f, 0.0f}, { 0.0f, 0.0f, .01f, 0.0f}, { 0.0f, 0.0f, 0.0f, .01f}, { 0.5f, 0.5f, 0.5f, 0.0f}}, ""},
				/* Albedo*/ { { 1.0f, 0.0f}, { 2.0f, 1.0f}, ASURA_D3D_PS_DEFERRED_DEBUGSHOWBUFFERS,         DT_ALBEDO, D3DTEXF_LINEAR, { { 1.0f, 0.0f, 0.0f, 0.0f}, { 0.0f, 1.0f, 0.0f, 0.0f}, { 0.0f, 0.0f, 1.0f, 0.0f}, { 0.0f, 0.0f, 0.0f, 1.0f}, { 0.0f, 0.0f, 0.0f, 0.0f}}, "Albedo", 0xFFFFFFFF},
				/* Spec  */ { { 3.0f, 0.0f}, { 4.0f, 1.0f}, ASURA_D3D_PS_DEFERRED_DEBUGSHOWBUFFERS,         DT_NORMAL, D3DTEXF_LINEAR, { { 0.0f, 0.0f, 0.0f, 1.0f}, { 0.0f, 0.0f, 0.0f, 1.0f}, { 0.0f, 0.0f, 0.0f, 1.0f}, { 0.0f, 0.0f, 0.0f, 1.0f}, { 0.0f, 0.0f, 0.0f, 0.0f}}, "Specular Mask", 0xFFFF0000},
				/* Normal*/ { { 2.0f, 0.0f}, { 3.0f, 1.0f}, ASURA_D3D_PS_DEFERRED_DEBUGSHOWBUFFERS_NORMALS, DT_NORMAL, D3DTEXF_LINEAR, { { 0.5f, 0.0f, 0.0f, 0.0f}, { 0.0f, 0.5f, 0.0f, 0.0f}, { 0.0f, 0.0f, 0.5f, 0.0f}, { 0.0f, 0.0f, 0.0f, 0.0f}, { 0.5f, 0.5f, 0.5f, 0.0f}}, "Normals", 0xFF000000},
				/*Material*/{ { 3.0f,-4.0f}, { 4.0f,-1.0f}, ASURA_D3D_PS_DEFERRED_DEBUGSHOWBUFFERS_ABS,     DT_MATERIAL, D3DTEXF_LINEAR, { { 1.0f, 0.0f, 0.0f, 0.0f}, { 0.0f, 1.0f, 0.0f, 0.0f}, { 0.0f, 0.0f, 1.0f, 0.0f}, { 0.0f, 0.0f, 0.0f, 1.0f}, { 0.0f, 0.0f, 0.0f, 0.0f}}, ""},
				/* Dist  */ { { 3.0f,-1.0f}, { 4.0f, 0.0f}, ASURA_D3D_PS_DEFERRED_DEBUGSHOWBUFFERS_ABS,     DT_NORMAL, D3DTEXF_POINT,  { { 0.0f, 0.0f, fScaleDistance, 0.0f}, { 0.0f, 0.0f, fScaleDistance, 0.0f}, { 0.0f, 0.0f, fScaleDistance, 0.0f}, { 0.0f, 0.0f, fScaleDistance, 0.0f}, { 0.0f, 0.0f, 0.0f, 0.0f}}, "Depth", 0xFFFF0000},
			};

			Asura_Font_System::SetFont(0);

			// Store the current minification filter. We must use point sampling on the depth view,
			// for instance, because the values in the texture are -ve if the world-space normal's z
			// value is -ve. Linear filtering happens before we get a chance to correct this, so
			// things go wrong at edges between -ve & +ve texel values

			DWORD uOldMinFilter;
			xResult = Asura_Direct3D::D3DDevice->GetSamplerState( 0, D3DSAMP_MINFILTER, &uOldMinFilter);
			ASURA_D3D_OK( xResult);

			const u_int uDrawTargetCount = sizeof( axDrawTargets) / sizeof( axDrawTargets[0]);
			for( u_int uDrawTargetIndex = 0; uDrawTargetIndex < uDrawTargetCount; uDrawTargetIndex += 1)
			{
				const DrawTarget& xDrawTarget = axDrawTargets[ uDrawTargetIndex];
				const DrawTextureIndex eRenderTargetTexture = xDrawTarget.m_eDrawTargetTexture;
				//if( uMRTCount <= uRenderTargetTexture)
				{
					//continue;
				}
				Direct3D_Texture* pxMRTTexture = apxDrawTextures[ eRenderTargetTexture];
				if( NULL == pxMRTTexture)
				{
					continue;
				}
				
				Asura_Direct3D_Shaders::RequestPixelShader( xDrawTarget.m_ePixelShader);
				
				xResult = pxD3DDevice->SetPixelShaderConstantF( 0, xDrawTarget.m_afPSConsts[0], 5);
				ASURA_D3D_OK( xResult);
				xResult = pxD3DDevice->SetTexture( 0, pxMRTTexture);
				ASURA_D3D_OK( xResult);

				xResult = Asura_Direct3D::D3DDevice->SetSamplerState( 0, D3DSAMP_MINFILTER, xDrawTarget.m_eFilter);
				ASURA_D3D_OK( xResult);
				
				const float fStartX = xDrawTarget.m_xStart[0] / fOutputSizeTotalX;
				const float fEndX = xDrawTarget.m_xEnd[0]/ fOutputSizeTotalX;
				const float fStartY = xDrawTarget.m_xStart[1] / fOutputSizeTotalY;
				const float fEndY = xDrawTarget.m_xEnd[1] / fOutputSizeTotalY;
				
				struct _D3DQuadVertex
				{
				public:
					float X, Y, Z;
					float U, V;
				};
				{
					_D3DQuadVertex axVert[4] =
					{
						{ -1.0f + (2.0f * fStartX), -0.6f - (0.4f * fStartY), 1.0f, 0.0f, 0.0f},
						{ -1.0f + (2.0f * fStartX), -0.6f - (0.4f * fEndY), 1.0f, 0.0f, 1.0f},
						{ -1.0f + (2.0f * fEndX), -0.6f - (0.4f * fStartY), 1.0f, 1.0f, 0.0f},
						{ -1.0f + (2.0f * fEndX), -0.6f - (0.4f * fEndY), 1.0f, 1.0f, 1.0f}
					};

					xResult = pxD3DDevice->DrawPrimitiveUP( D3DPT_TRIANGLESTRIP, 2, axVert, sizeof( axVert[0]));
					ASURA_D3D_OK( xResult);
				}

				// Print label
				{
					const float fLabelX = fStartX;
					const float fLabelY = fStartY * 0.2f + 0.8f;
					Asura_Font_System::Print( fLabelX, fLabelY, 0.5f, xDrawTarget.m_uLabelColour, ASURA_TEXTFORMAT_RELATIVEPOSITION, xDrawTarget.m_szLabel);
				}
			}
			
			// Restore previous minification filter
			xResult = Asura_Direct3D::D3DDevice->SetSamplerState( 0, D3DSAMP_MINFILTER, uOldMinFilter);
			ASURA_D3D_OK( xResult);

			Asura_Direct3D::RequestSwapChain(0);
			
			pxDSTSurface->Release();
		}
	}
}

void Asura_DeferredShading::Platform_EndRender(void)
{
	D3DPERFCALL(EndEvent());
}

//*******************************************************************
//
// (c) 2006 Rebellion
//
// Warning: This computer program is protected by copyright law and 
// international treaties. Unauthorized reproduction or distribution
// of this program, or any portion of it, may result in severe civil and
// criminal penalties, and will be prosecuted to the maximum extent 
// possible under the law.
//
//*******************************************************************
#endif
