//
// Asura_Platform_RenderObjectHierarchy.h
//
// Created by KJL - 08/05/2003 15:11:13
//
// Copyright (C) 2003 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif

#ifndef __ASURA_PLATFORM_RENDEROBJECTHIERARCHY_H__
#define __ASURA_PLATFORM_RENDEROBJECTHIERARCHY_H__

#include "Animation/Asura_Hierarchy.h"
#include "Core/Asura_String.h"
#include "Core/Asura_HashTree.h"
#include "Rendering/Asura_Direct3D.h"


class Asura_Animation;
class Asura_Shadow;
class Asura_Stream;

#ifndef ASURA_TARGET_SM3

// the old renderer uses compressed vertices now

#pragma pack(push, 1)

class Asura_Direct3D_ObjectHierarchy_Vertex
{
	public:
		Asura_Vector_3	m_xPosition;
		u_int			m_uNormal;			// compressed into 8,8,8,X format
		short			m_sU, m_sV;			// compressed
		float			m_fAlpha;
};

#pragma pack(pop)

#else

// the new renderer still uses uncompressed vertices

class Asura_Direct3D_ObjectHierarchy_Vertex
{
	public:
		float	m_fX;
		float	m_fY;
		float	m_fZ;
		float	m_fNX;
		float	m_fNY;
		float	m_fNZ;
		float	m_fU;
		float	m_fV;
		float	m_fAlpha;
};

#endif

#ifdef ASURA_TARGET_SM3
class Asura_Direct3D_ObjectHierarchy_Vertex_NormalMapped
{
public:
	Asura_Vector_3 m_xPosition;
	Asura_Vector_3 m_xNormal;
	float	m_fU;
	float	m_fV;
	float	m_fU2;
	float	m_fV2;

	float m_fAlpha;
	Asura_Vector_3 m_xBinormal;
	Asura_Vector_3 m_xTangent;
};
#endif //#ifdef ASURA_TARGET_SM3

class Asura_Direct3D_ObjectHierarchy_Strip
{
	public:			
		u_int			m_uNumberOfTriangles;
		u_int			m_uStartVertexIndex;

#if defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM
		Asura_Hash_ID	m_auMaterialResponseHashID[2];
#else
		int				m_iMaterialIndex;
#endif

		u_int			m_uLowestVertexUsed;
		u_int			m_uNumberOfVertices;
		u_short			m_usBoneIndex;
};


class Asura_Direct3D_ObjectHierarchy
{
	public:
		Asura_Direct3D_ObjectHierarchy()
		{
			m_pxHierarchyStrip = 0;
			m_pxVertexBuffer = 0;
			m_pxIndexBuffer = 0;

			m_uNumberOfStrips = 0;
			m_uTotalNumberOfVertices = 0;
			m_uTotalNumberOfIndices = 0;

			m_ePrimitiveType = D3DPT_TRIANGLESTRIP;

#ifdef ASURA_TARGET_SM3
			m_bHasNormalMappedStrip = false;
#endif //#ifdef ASURA_TARGET_SM3
		}

		~Asura_Direct3D_ObjectHierarchy()
		{	
			if (m_pxVertexBuffer)
			{
				m_pxVertexBuffer->Release();
			}
			if (m_pxIndexBuffer)
			{
				m_pxIndexBuffer->Release();
			}

			delete [] m_pxHierarchyStrip;
		}

		Asura_Direct3D_ObjectHierarchy_Strip*	m_pxHierarchyStrip;
		Direct3D_VertexBuffer*					m_pxVertexBuffer;
		Direct3D_IndexBuffer*					m_pxIndexBuffer;
												
		u_int									m_uNumberOfStrips;
		u_int									m_uTotalNumberOfVertices;
		u_int									m_uTotalNumberOfIndices;

		D3DPRIMITIVETYPE						m_ePrimitiveType;


#ifdef ASURA_TARGET_SM3
		// Since all strips use the same vertex buffer, the hierarchy will use the normal-mapped vertex for all vertices if there are any normal-mapped strips.
		// The only time the standard vertex is used is if all strips in the hierarchy are normal-mapless.
		bool m_bHasNormalMappedStrip; // Must be set on initialisation, and never changed after then.
#endif //#ifdef ASURA_TARGET_SM3
};


class Asura_Direct3D_ObjectHierarchyRender_Handler
{
	friend class Asura_Direct3D_ObjectHierarchyRenderer;
	friend class Asura_Chunk_ResourceFile_PC_ObjectHierarchy;
	friend class Tools_Animation_Files;

public:
	// Public functions...
	Asura_Direct3D_ObjectHierarchyRender_Handler  (void)	{}
//	~Asura_Direct3D_ObjectHierarchyRender_Handler ()		{}

	void									Reset									(void);

	// However, that function was only being used to check whether the skin was transparent or not (as far as I can tell).
	// We can still do that...
	bool									IsAnyMaterialTransparent				(const Asura_Hierarchy_Skin* pxSkin) const;

#ifdef ASURA_TOOLS
	//! for detecting the annoying skin(s) that contain no meshes. RAR 01/09/03
	bool									DoesSkinHaveAnyTriangles				(const Asura_Hierarchy_Skin* pxSkin) const;
#endif

	//Deletes the render data for all LODs of a model , forcing them to be recalculated the next time it is rendered
	void ResetAllLODs(const Asura_Hierarchy_Skin* pxSkin);


private:
	// Private variables...
	Asura_TemplateHashTree<Asura_Direct3D_ObjectHierarchy>	m_xHashTree;

	// Private functions...
	Asura_Direct3D_ObjectHierarchy*			AddHierarchy							(const Asura_String& xName);

	//! Deletes the render data for this skin. RAR 14/11/2003
	void									DeleteHierarchy							(const Asura_Hierarchy_Skin* pxSkin)		{ m_xHashTree.DeleteNodeByHash(pxSkin->m_uNameHashID); }

	const Asura_Direct3D_ObjectHierarchy*	GetHierarchy							(const Asura_Hierarchy_Skin* pxSkin);

#if defined(ASURA_TOOLS)
	const Asura_Direct3D_ObjectHierarchy*	InitialiseStripsFromData				(const Asura_Hierarchy_Skin* pxSkin);
#if defined(ASURA_TARGET_SM3)
	const Asura_Direct3D_ObjectHierarchy*	InitialiseNormalMappedStripsFromData	(const Asura_Hierarchy_Skin* pxSkin);
#endif	// defined(ASURA_TARGET_SM3)
#endif	// defined(ASURA_TOOLS)
};


class Asura_Direct3D_ObjectHierarchyRenderer
{
	friend class Asura_Direct3D_ObjectHierarchyRender_Handler;

public:
	// Public typedefs
	typedef void SetupPixelShader(Asura_Direct3D_ObjectHierarchy_Strip const * const);
	typedef void RevertPixelShader(void);

	//! One-time initialisation function, called (indirectly) by Asura_Core::Initialise()
	static void Initialise(void);
	//! One-time shutdown function, called (indirectly) by Asura_Core::Shutdown()
	static void Shutdown(void);
	//! Reset function called by Asura_Render::Platform_Reset()
	static void Reset(void);

	//! BeginRender() should be called once per hierarchy before the Render() calls
	static void BeginRender( const Asura_Animation* pxAnim, const Asura_Hierarchy_Skin* pxSkin );
	//! Draws one object from an object hierarchy.
	//! All Render() calls for an hierarchy should be surrounded by just one BeginRender/EndRender pairing.
	static void Render( const Asura_Animation* pxAnim,const Asura_Hierarchy_Skin* pxSkin, const int iSectionDataIndex, const Asura_Vector_3& xPosition, const Asura_Matrix_3x3& xOrientation, bool bDepthOnly = false);
	//! EndRender() should be called once per hierarchy after Render() calls are completed
	static void EndRender( void );

#ifdef ASURA_TOOLS
	//! Dodgy tools renderer for drawing the selection polyline outline
	static void Render_Outline(const Asura_Animation* pxAnim,const Asura_Hierarchy_Skin* pxSkin, const int iSectionDataIndex, const Asura_Vector_3& xPosition, const Asura_Matrix_3x3& xOrientation);
#endif

#ifdef ASURA_TARGET_SM3
	//! BeginRender() should be called once per hierarchy before the Render() calls
	static void BeginRender_EffectsSystem( const Asura_Animation* pxAnim, const Asura_Hierarchy_Skin* pxSkin );
	//! Draws one object from an object hierarchy.
	//! All Render() calls for an hierarchy should be surrounded by just one BeginRender/EndRender pairing.
	static void Render_EffectsSystem( const Asura_Animation* pxAnim,const Asura_Hierarchy_Skin* pxSkin, const int iSectionDataIndex, const Asura_Vector_3& xPosition, const Asura_Matrix_3x3& xOrientation, bool bDepthOnly = false);
	//! EndRender() should be called once per hierarchy after Render() calls are completed
	static void EndRender_EffectsSystem( void );

#if defined(ASURA_TARGET_SM3)
	// (SB - 01/05/08) Added options distance parameter to support shadow LODs
	static void RenderShadow_EffectsSystem( const Asura_Animation& xAnim, const Asura_Hierarchy_Skin& xSkin, const Asura_Shadow& xShadow, const float fDistance = 0.0f );
#endif

#ifdef ASURA_TOOLS
	//! Dodgy tools renderer for drawing the selection polyline outline
	static void Render_Outline_EffectsSystem(const Asura_Animation* pxAnim,const Asura_Hierarchy_Skin* pxSkin, const int iSectionDataIndex, const Asura_Vector_3& xPosition, const Asura_Matrix_3x3& xOrientation);
#endif

#ifdef ASURA_USE_MATERIAL_RESPONSE_SYSTEM

	static void BeginRender_MaterialResponse( const Asura_Animation* pxAnim, const Asura_Hierarchy_Skin* pxSkin );
	static void Render_MaterialResponse( const Asura_Animation* pxAnim, const Asura_Hierarchy_Skin* pxSkin, const int iSectionDataIndex, const Asura_Vector_3& xPosition, const Asura_Matrix_3x3& xOrientation, bool bDepthOnly = false );
	static void EndRender_MaterialResponse();

#endif // defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM

	static bool ToggleNewRenderers()
	{
		if (s_bNewRenderersSupported)
		{
			s_bUseNewRenderers = !s_bUseNewRenderers;
		}
		return s_bUseNewRenderers;
	}
	static bool UseNewRenderers()
	{
		return s_bUseNewRenderers;
	}
#endif //#ifdef ASURA_TARGET_SM3

	//! BeginShadowRender() should be called once per hierarchy before the RenderShadow() calls
	static void BeginShadowRender( const Asura_Hierarchy_Skin* pxSkin, const Asura_Vector_3& xShadowPosition );
	//! Draws one object from an object hierarchy.
	//! All RenderShadow calls for an hierarchy should be surrounded by just one BeginShadowRender/EndShadowRender pairing.
	static void RenderShadow( const int iSectionDataIndex, const Asura_Vector_3& xPosition, const Asura_Matrix_3x3& xOrientation );
	//! EndShadowRender() should be called once per hierarchy after RenderShadow() calls are completed
	static void EndShadowRender( void );

	//! Gets the material used by the skin.
	// RJF - 16 June 2005 - This function no longer makes any sense as skins support multiple materials
	//static int GetMaterialIndex(const Asura_Hierarchy_Skin* pxSkin);

	// However, that function was only being used to check whether the skin was transparent or not (as far as I can tell).
	// We can still do that...
	static bool IsAnyMaterialTransparent(const Asura_Hierarchy_Skin* pxSkin)						{ return(GetCurrentHandler().IsAnyMaterialTransparent(pxSkin)); }

#ifdef ASURA_TOOLS
	//! for detecting the annoying skin(s) that contain no meshes. RAR 01/09/03
	static bool DoesSkinHaveAnyTriangles(const Asura_Hierarchy_Skin* pxSkin)						{ return(GetCurrentHandler().DoesSkinHaveAnyTriangles(pxSkin)); }
#endif

	static const Asura_Direct3D_ObjectHierarchy* GetHierarchy(const Asura_Hierarchy_Skin* pxSkin)	{ return(GetCurrentHandler().GetHierarchy(pxSkin)); }

#if defined(ASURA_USE_MULTIPLE_HANDLERS)					
	// Handler functions...
	static	void											ReparentHandler(const Asura_Hash_ID uSourceHash, const Asura_Hash_ID uNewParentHash);
	static	void											ReparentHandler(Asura_Direct3D_ObjectHierarchyRender_Handler& xSourceHandler, const Asura_Hash_ID uNewParentHash);

	static	Asura_Direct3D_ObjectHierarchyRender_Handler*	CreateHandler		(const Asura_Hash_ID uHash, const Asura_Hash_ID uParentHandlerHash);
	static	bool											DestroyHandler		(const Asura_Hash_ID uHash);
	static	void											WriteHandlerReport	(u_int uHandler, Asura_Stream& xLog);

	static	void											CreateAndSetHandler	(const Asura_Hash_ID uHash, const Asura_Hash_ID uParentHandlerHash)	{ s_pxCurrentHandler = CreateHandler(uHash, uParentHandlerHash); }
	static	void											SetCurrentHandler	(Asura_Direct3D_ObjectHierarchyRender_Handler* pxHandler)			{ s_pxCurrentHandler = pxHandler; }
	static	void											SetCurrentHandler	(const Asura_Hash_ID uHash)											{ s_pxCurrentHandler = s_xHandlerTree.GetDataByHash(uHash); }
	static	Asura_Direct3D_ObjectHierarchyRender_Handler&	GetCurrentHandler	(void)																{ return(*s_pxCurrentHandler); }
#else
	// Handler functions...
	static	Asura_Direct3D_ObjectHierarchyRender_Handler&	GetCurrentHandler	(void)																{ return(s_xHandler); }
#endif

	static void SetPixelShaderOverrides(SetupPixelShader* const pfnSetupShader = NULL, RevertPixelShader* const pfnRevertShader = NULL)
	{
		s_pfnSetupPixelShader = pfnSetupShader;
		s_pfnRevertPixelShader = pfnRevertShader;
	}

	//Deletes the render data for all LODs of a model , forcing them to be recalculated the next time it is rendered
	static void ResetAllLODs(const Asura_Hierarchy_Skin* pxSkin) { GetCurrentHandler().ResetAllLODs(pxSkin);}


private:
#if defined(ASURA_USE_MULTIPLE_HANDLERS)
	// Private variables...
	static	Asura_Direct3D_ObjectHierarchyRender_Handler*							s_pxCurrentHandler;		// Currently active handler.
	static	Asura_TemplateHashTree<Asura_Direct3D_ObjectHierarchyRender_Handler>	s_xHandlerTree;			// Tree of all of the handlers available.
#else
	// Private variables...
	static	Asura_Direct3D_ObjectHierarchyRender_Handler							s_xHandler;				// Single static-instance of a zone.
#endif

	// data on current status of drawing
	static bool										s_bNormalRendering;
	static bool										s_bShadowRendering;
	static const Asura_Direct3D_ObjectHierarchy*	s_pxCurrentHierarchyObject;

#ifdef ASURA_TARGET_SM3
	static bool s_bUseNewRenderers;
	static bool s_bNewRenderersSupported;
#endif //#ifdef ASURA_TARGET_SM3

	static SetupPixelShader* s_pfnSetupPixelShader;
	static RevertPixelShader* s_pfnRevertPixelShader;
};


#endif//__ASURA_PLATFORM_RENDEROBJECTHIERARCHY_H__
