//
// Asura_Screenshot.cpp
//
// Created by Kevin Lea - 05/10/2001 16:39:44
// 
// Copyright (C) 2001 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"


#if 00
bool ScreenShot(unsigned int number)
{
	return false;
}
#else

#include "Core/Asura_PlatformBasics.h"
#include "Rendering/Asura_Render.h"

#include "Rendering/Asura_Viewport.h"
#include "Rendering/Asura_Direct3D.h"
#include "Rendering/Asura_Bmp.h"

#include "Debugging/Asura_Assert.h"

typedef VOID (*PutWord_F)(WORD, FILE *);
typedef VOID (*PutDword_F)(DWORD, FILE *);


VOID PutByte(BYTE v, FILE *fp)
{
  fputc(v, fp);
}

VOID PutLittleWord(WORD v, FILE *fp)
{
  unsigned char c1 = (unsigned char)(v & 0xff);
  unsigned char c2 = (unsigned char)((v >> 8) & 0xff);
  fputc(c1, fp);
  fputc(c2, fp);
}

VOID PutLittleDword(DWORD v, FILE *fp)
{
  unsigned char c1 = (unsigned char)(v & 0xff);
  unsigned char c2 = (unsigned char)((v >> 8) & 0xff);
  unsigned char c3 = (unsigned char)((v >> 16) & 0xff);
  unsigned char c4 = (unsigned char)((v >> 24) & 0xff);

  fputc(c1, fp);
  fputc(c2, fp);
  fputc(c3, fp);
  fputc(c4, fp);
}


bool ScreenShot(const char* filename)
{

	FILE * fp = fopen(filename,"wb");
	if (!fp)
	{
		return false;
	}
	char* hugeBuffer = 0;
	bool success = false;
	
	BMPHEADER2 h;

	int iWidth = Asura_Viewport::GetWidth();
	int iHeight = Asura_Viewport::GetHeight();
	int iPitch = ((iWidth*3)+3)&~3;

	// fill out header
	
	h.Header.Type      = 'B'+'M'*256;
	h.Header.Reserved1 = 0;
	h.Header.Reserved2 = 0;
	h.Header.Offset    = 14+40+0;

	/*
	** The type of information found in a BMP structure is indicated by
	** the Size (Information Headere Size) field with a non-zero value.
	*/
	h.PmInfo.Size   = 0;
	h.Pm2Info.Size  = 0;

	h.WinInfo.Size          = 40;
	h.WinInfo.Width         = iWidth;
	h.WinInfo.Height        = iHeight;
	h.WinInfo.Planes        = 1;
	h.WinInfo.BitCount      = 24;
	h.WinInfo.Compression   = 0;
	h.WinInfo.SizeImage     = iPitch*iHeight;
	h.WinInfo.XPelsPerMeter = 0;
	h.WinInfo.YPelsPerMeter = 0;
	h.WinInfo.ClrUsed       = 0;
	h.WinInfo.ClrImportant  = 0;

	h.Header.FileSize  = h.WinInfo.SizeImage + h.Header.Offset + 8;

	// write header

	PutWord_F PutWord = PutLittleWord;
	PutDword_F PutDword = PutLittleDword;

    PutWord(h.Header.Type, fp);
    PutDword(h.Header.FileSize, fp);
    PutWord(h.Header.Reserved1, fp);
    PutWord(h.Header.Reserved2, fp);
    PutDword(h.Header.Offset, fp);

	PutDword(h.WinInfo.Size, fp);

    PutDword(h.WinInfo.Width, fp);
    PutDword(h.WinInfo.Height, fp);
    PutWord(h.WinInfo.Planes, fp);
    PutWord(h.WinInfo.BitCount, fp);
    PutDword(h.WinInfo.Compression, fp);
    PutDword(h.WinInfo.SizeImage, fp);
    PutDword(h.WinInfo.XPelsPerMeter, fp);
    PutDword(h.WinInfo.YPelsPerMeter, fp);
    PutDword(h.WinInfo.ClrUsed, fp);
    PutDword(h.WinInfo.ClrImportant, fp);

	Direct3D_Surface* backbuffer = 0;
	HRESULT hres;
	hres = Asura_Direct3D::D3DDevice->GetBackBuffer(0,0,D3DBACKBUFFER_TYPE_MONO,&backbuffer);
	if(FAILED(hres))
	{
		Asura_Assert(0,"Failed to get backbuffer for screenshot");
		goto Cleanup;
	}

	// Get the surface desc
	D3DSURFACE_DESC ddsd;
	backbuffer->GetDesc(&ddsd);

	// Create a lockable RT surface to copy to
	Direct3D_Surface* pxTempSurface = 0;
	hres = Asura_Direct3D::D3DDevice->CreateRenderTarget(ddsd.Width, ddsd.Height, ddsd.Format, D3DMULTISAMPLE_NONE, 0, TRUE, &pxTempSurface, NULL);
	if(FAILED(hres))
	{
		Asura_Assert(0, "Failed to create temporary screenshot surface");
		goto Cleanup;
	}

	hres = Asura_Direct3D::D3DDevice->StretchRect(backbuffer, NULL, pxTempSurface, NULL, D3DTEXF_POINT);
	ASURA_D3D_OK( hres);
	
	D3DLOCKED_RECT rect;
	hres = pxTempSurface->LockRect(&rect,NULL,D3DLOCK_READONLY);
	if(FAILED(hres))
	{
		Asura_Assert(0,"Failed to lock backbuffer for screenshot");
		goto Cleanup;
	}

	//allocate buffer for screenshot
	hugeBuffer = new char[iPitch*iHeight];

	//determine the pixel format
	int red_shift,red_scale,red_mask;
	int green_shift,green_scale,green_mask;
	int blue_shift,blue_scale,blue_mask;
	int bytesPerPixel;

	switch(ddsd.Format)
	{
		case D3DFMT_X1R5G5B5 :
			red_mask = 31<<10;
			red_shift = 10;
			red_scale = 255/31;
			green_mask = 31<<5;
			green_shift = 5;
			green_scale = 255/31;
			blue_mask = 31;
			blue_shift = 0;
			blue_scale = 255/31;
			bytesPerPixel = 2;
			break;
		case D3DFMT_R5G6B5 :
			red_mask = 31<<11;
			red_shift = 11;
			red_scale = 255/31;
			green_mask = 63<<5;
			green_shift = 5;
			green_scale = 255/63;
			blue_mask = 31;
			blue_shift = 0;
			blue_scale = 255/31;
			bytesPerPixel = 2;
			break;
		case D3DFMT_X8R8G8B8 :
		case D3DFMT_A8R8G8B8 :
			red_mask = 255<<16;
			red_shift = 16;
			red_scale = 1;
			green_mask = 255<<8;
			green_shift = 8;
			green_scale = 1;
			blue_mask = 255;
			blue_shift = 0;
			blue_scale = 1;
			bytesPerPixel = 4;
			break;

		default :
			Asura_Assert(0,"Unexpected pixel format for screenshot");
			goto Cleanup;

	}
		
	// write 24 bit image
	
	
	unsigned char* BufferPtr;
	BufferPtr=(unsigned char *)rect.pBits+rect.Pitch*(h.WinInfo.Height-1);
	char* pcDestLineStart = hugeBuffer;

	for (int i=h.WinInfo.Height-1; i>=0; --i)
	{
		unsigned int j;
	
		{
			unsigned int colour;
			char* pcDest = pcDestLineStart;
			for (j=0; j<h.WinInfo.Width; ++j)
			{
				if(bytesPerPixel == 4)
				{
					colour=*(unsigned int*)&BufferPtr[j*4];
				}
				else
				{
					colour=*(unsigned short*)&BufferPtr[j*2];
				}
				*pcDest++ = (BYTE)(((colour&blue_mask)>>blue_shift)*blue_scale);
				*pcDest++ = (BYTE)(((colour&green_mask)>>green_shift)*green_scale);
				*pcDest++ = (BYTE)(((colour&red_mask)>>red_shift)*red_scale);
			}
		}
	
		pcDestLineStart += iPitch;
		BufferPtr-=rect.Pitch;
	}
	fwrite( hugeBuffer, 1, iHeight*iPitch,fp );

	success = true;

	Cleanup :
	if(pxTempSurface)
	{
		pxTempSurface->UnlockRect();
		pxTempSurface->Release();
	}
	if(backbuffer)
	{
		backbuffer->Release();
	}
	delete [] hugeBuffer;

	fclose(fp);

	if(!success)
	{
		//delete the partially written file
		DeleteFile(filename);
	}

	return success;
	
}

bool ScreenShot(unsigned int number)
{
	if (number>9999) return false;

	char Name[40];
	CreateDirectory("Screenshots",0);
	strcpy(Name,"Screenshots//Asura0000.bmp");
	int length=5+13;
	
	{
		Name[length]  =(char)(number/1000+'0');
		Name[length+1]=(char)((number/100)%10+'0');
		Name[length+2]=(char)((number/10)%10+'0');
		Name[length+3]=(char)((number)%10+'0');
	}

	return ScreenShot(Name);
}

#endif
