//
// Asura_D3D_PS_Env_HighEnd.cpp
//
// Created by VGP - 10Mar2006
//
// Copyright (C) 2006 Rebellion (Strangelite), All Rights Reserved.
//
// Pixel shaders for HighEnd environment renderer
//


#include "Core/Asura.h"

#ifdef ASURA_USE_SPECULAR_MAPS

#include "Rendering/Asura_Direct3D_Shaders.h"


#define PS_HLSL_FRAGMENT_LOAD_DIFFUSE1( SAMPLER1)	\
"sampler sDiffuse1 : register( "SAMPLER1");	\n"	\
"half4 GetDiffuseTexture( const PS_INPUT_20 xInput)	\n"	\
"{	\n"	\
"	half4 cDiffuseTexture = tex2D( sDiffuse1, xInput.uvTexCoord1); \n"	\
"	return( cDiffuseTexture);"	\
"}	\n"

#define PS_HLSL_FRAGMENT_LOAD_SPECULAR1( SAMPLER1)	\
"sampler sSpecular1 : register( "SAMPLER1");	\n"	\
"half4 GetSpecularTexture( const PS_INPUT_20 xInput)	\n"	\
"{	\n"	\
"	half4 cSpecularTexture = tex2D( sSpecular1, xInput.uvTexCoord1); \n"	\
"	return( cSpecularTexture);"	\
"}	\n"

#define PS_HLSL_FRAGMENT_LOAD_NORMAL1( SAMPLER1)	\
"sampler sNormal1 : register( "SAMPLER1");	\n"	\
"half4 GetNormalTexture( const PS_INPUT_20 xInput)	\n"	\
"{	\n"	\
"	half4 cNormalTexture = tex2D( sNormal1, xInput.uvTexCoord1); \n"	\
"	return( cNormalTexture);"	\
"}	\n"

#define PS_HLSL_FRAGMENT_LOAD_DIFFUSE2( SAMPLER1, SAMPLER2)	\
"sampler sDiffuse1 : register( "SAMPLER1");	\n"	\
"sampler sDiffuse2 : register( "SAMPLER2");	\n"	\
"half4 GetDiffuseTexture( const PS_INPUT_20 xInput)	\n"	\
"{	\n"	\
"	half4 hTextureLerp = xInput.cColour.a;	\n"	\
"	half4 cDiffuse1Texture = tex2D( sDiffuse1, xInput.uvTexCoord1); \n"	\
"	half4 cDiffuse2Texture = tex2D( sDiffuse2, xInput.uvTexCoord2); \n"	\
"	half4 cDiffuseTexture = lerp( cDiffuse2Texture, cDiffuse1Texture, hTextureLerp); \n"	\
"	return( cDiffuseTexture);"	\
"}	\n"

#define PS_HLSL_FRAGMENT_LOAD_SPECULAR2( SAMPLER1, SAMPLER2)	\
"sampler sSpecular1 : register( "SAMPLER1");	\n"	\
"sampler sSpecular2 : register( "SAMPLER2");	\n"	\
"half4 GetSpecularTexture( const PS_INPUT_20 xInput)	\n"	\
"{	\n"	\
"	half4 hTextureLerp = xInput.cColour.a;	\n"	\
"	half4 cSpecular1Texture = tex2D( sSpecular1, xInput.uvTexCoord1); \n"	\
"	half4 cSpecular2Texture = tex2D( sSpecular2, xInput.uvTexCoord1); \n"	\
"	half4 cSpecularTexture = lerp( cSpecular2Texture, cSpecular1Texture, hTextureLerp); \n"	\
"	return( cSpecularTexture);"	\
"}	\n"

#define PS_HLSL_FRAGMENT_LOAD_NORMAL2( SAMPLER1, SAMPLER2)	\
"sampler sNormal1 : register( "SAMPLER1");	\n"	\
"sampler sNormal2 : register( "SAMPLER2");	\n"	\
"half4 GetNormalTexture( const PS_INPUT_20 xInput)	\n"	\
"{	\n"	\
"	half4 hTextureLerp = xInput.cColour.a;	\n"	\
"	half4 cNormal1Texture = tex2D( sNormal1, xInput.uvTexCoord1); \n"	\
"	half4 cNormal2Texture = tex2D( sNormal2, xInput.uvTexCoord1); \n"	\
"	half4 cNormalTexture = lerp( cNormal2Texture, cNormal1Texture, hTextureLerp); \n"	\
"	return( cNormalTexture);"	\
"}	\n"

#define PS_HLSL_FRAGMENT_LOAD_LIGHT( SAMPLER1)	\
"sampler sLight1 : register( "SAMPLER1");	\n"	\
"half3 GetDynamicLightTexture( const PS_INPUT_20 xInput)	\n"	\
"{	\n"	\
"	half4 cLight1Texture = tex2D( sLight1, xInput.uvLightCoord1); \n"	\
"	half4 cLight2Texture = tex2D( sLight1, xInput.uvLightCoord2); \n"	\
"	half3 cLightTexture = cLight1Texture.xyz * cLight2Texture.xyz; \n"	\
"	return( cLightTexture);"	\
"}	\n"

// Ambient_Diffuse1_SelfIllum
//
//	 Expected constants:
//		c0.a     :	Self Illumination factor
//
const char* const szCODE_Ambient_Diffuse1_SelfIllum =
{
	"ps.1.1						\n"
	"tex t0						\n"	

	"mul_x2 r0.rgb, t0, v0		\n"		// Tex * lighting

	"mad r0.rgb, t0, c0.w, r0	\n"		// Add self-illum
	"+ mov r0.a, t0.a			\n"
};

static Asura_Direct3D_PixelShader xPS_ENV_HIGHEND_AMBIENT_DIFFUSE1(
	ASURA_D3D_PS_ENV_HIGHEND_AMBIENT_DIFFUSE1, 
	szCODE_Ambient_Diffuse1_SelfIllum);


// Ambient_Normal_Diffuse1_SelfIllum
//
//	 Expected constants:
//		c0.a     :	Self Illumination factor
//
const char* const szCODE_Ambient_Normal_Diffuse1_SelfIllum =
{
	"ps.1.1						\n"
	
	"tex t0						\n"		// Diffuse 1
	"tex t1						\n"		// Normal 1
	"texcoord t3				\n"		// Light Direction

	"dp3_d2 r0, t1_bx2, t3_bx2	\n"
	"mad r0, v0, r0, v0		\n"
	
	"mul_x2 r0.rgb, t0, r0		\n"		// Tex * lighting
	"mad_sat r0.rgb, t0, c0.w, r0	\n"		// Add self-illum
	"+ mov r0.a, t0.a			\n"
};

static Asura_Direct3D_PixelShader xPS_ENV_HIGHEND_AMBIENT_NORMAL_DIFFUSE1(
	ASURA_D3D_PS_ENV_HIGHEND_AMBIENT_NORMAL_DIFFUSE1, 
	szCODE_Ambient_Normal_Diffuse1_SelfIllum);


// Ambient_Diffuse2_SelfIllum
//
//	 Expected constants:
//		c0.a     :	Self Illumination factor
//
const char* const szCODE_Ambient_Diffuse2_SelfIllum =
{
	"ps.1.1						\n"
	"tex t0						\n"	
	"tex t1						\n"	

	"lrp r1, v0.a, t0, t1		\n"		// Blend textures

	"mul_x2 r0.rgb, r1, v0		\n"		// Tex * lighting

	"mad r0.rgb, r1, c0.w, r0	\n"		// Add self-illum
	"+ mov r0.a, t0.a			\n"
};

static Asura_Direct3D_PixelShader xPS_ENV_HIGHEND_AMBIENT_DIFFUSE2(
	ASURA_D3D_PS_ENV_HIGHEND_AMBIENT_DIFFUSE2, 
	szCODE_Ambient_Diffuse2_SelfIllum);


// Ambient_Normal_Diffuse2_SelfIllum
//
//	 Expected constants:
//		c0.a     :	Self Illumination factor
//
const char* const szCODE_Ambient_Normal_Diffuse2_SelfIllum =
{
	"ps.1.1						\n"

	"tex t0						\n"		// Diffuse 1
	"tex t1						\n"		// Normal 1
	"tex t2						\n"		// Diffuse 2
	"texcoord t3				\n"		// Light Direction

	"dp3_d2 r0, t1_bx2, t3_bx2	\n"
	"mad r0, v0, r0, v0		\n"
	
	"lrp r1, v0.a, t0, t2		\n"		// Blend textures

	"mul_x2 r0.rgb, r1, r0		\n"		// Tex * lighting

	"mad r0.rgb, r1, c0.w, r0	\n"		// Add self-illum
	"+ mov r0.a, t0.a			\n"
};

static Asura_Direct3D_PixelShader xPS_ENV_HIGHEND_AMBIENT_NORMAL_DIFFUSE2(
	ASURA_D3D_PS_ENV_HIGHEND_AMBIENT_NORMAL_DIFFUSE2, 
	szCODE_Ambient_Normal_Diffuse2_SelfIllum);

// Ambient_Diffuse1_Specular1_SelfIllum
//
//	 Expected constants:
//		c0.x     :	Specular power
//		c0.a     :	Self Illumination factor
//
const char* const szCODE_Ambient_Diffuse1_Specular1_SelfIllum =
{
	"ps_2_0							\n"				

	"dcl_pp t0.xy					\n" 		// Tex coords
	"dcl_pp t2.xyz					\n" 		// Get view vector
	"dcl_pp t3.xyz					\n" 		// Get normal
	"dcl_pp t4						\n" 		// Get light direction
	"dcl v0							\n" 		// Get vertex color
	"dcl_2d s0						\n" 		// Sampler for diffuse tex
	"dcl_2d s1						\n" 		// Sampler for specular map

	"def c1, 2,0,2,2				\n"

	// Sample textures
	"texld_pp r0, t0, s0			\n" 		// Sample diffuse tex
	"texld_pp r1, t0, s1			\n" 		// Sample specular map

	// Calculate half angle vector
	"nrm_pp r2.xyz, t2				\n"			// Normalize view vector
	"nrm_pp r4.xyz, t4				\n"			// Normalize light vector
	"add_pp r4.xyz, r2, r4			\n"			// Half angle = view + light
	"nrm_pp r2.xyz, r4				\n"			// Normalize half angle vector

	// Dot with surface normal and raise to power
	"nrm_pp r3, t3					\n" 		// Normalize surface normal
	"dp3_pp r3.w, r3, r2			\n"			// N.H
	"max_pp r3.w, r3.w, c1.y		\n"			// Clamp( N.H)
	"pow_pp r2.w, r3.w, c0.x		\n"			// N.H^power

	// Calculate final color
	"mad_pp r1.rgb, r1, r2.w, r0	\n"			// Diffuse tex + specular map * N.H^power
	"mul_pp r1.rgb, r1, v0			\n"			// Multiply by lighting
	"mul_pp r1.rgb, r1, c1.x			\n"			// x2
	"mad_pp r0.rgb, r0, c0.w, r1	\n"			// Add self-illum
	"mov_pp oC0, r0					\n"
};

static Asura_Direct3D_PixelShader xPS_ENV_HIGHEND_AMBIENT_DIFFUSE1_SPECULAR1(
	ASURA_D3D_PS_ENV_HIGHEND_AMBIENT_DIFFUSE1_SPECULAR1, 
	szCODE_Ambient_Diffuse1_Specular1_SelfIllum);

// Ambient_Normal_Diffuse1_Specular1_SelfIllum
//
//	 Expected constants:
//		c0.x     :	Specular power
//		c0.a     :	Self Illumination factor
//

const char* const szCODE_Ambient_Normal_Diffuse1_Specular1_SelfIllum =
{

"half4 g_ah4Const0 : register( c0);	\n"

"sampler sDiffuse1 : register( s0);	\n"
"sampler sSpecular1 : register( s1);	\n"
"sampler sNormal1 : register( s2);	\n"

"struct PS_INPUT_20	\n"
"{	\n"
"	float2 uvTexCoord1 : TEXCOORD0;	\n"
"	float2 uvTexCoord2 : TEXCOORD1;	\n"
"	float3 uvwViewDirection	: TEXCOORD2;	\n"
"	float3 uvwLightDirection	: TEXCOORD3;	\n"
"	half4  cColour	: COLOR0;	\n"
"		\n"
"};	\n"

"struct PS_OUTPUT_20	\n"
"{	\n"
"	half4 cColour0	: COLOR0;	\n"
"};	\n"

"PS_OUTPUT_20 main_Ambient_Normal_Diffuse1_Specular1_SelfIllum( const PS_INPUT_20 xInput)	\n"
"{	\n"
"	PS_OUTPUT_20 xOutput;	\n"

"	half4 cDiffuseTexture = tex2D( sDiffuse1, xInput.uvTexCoord1); \n"
"	half4 cSpecularTexture = tex2D( sSpecular1, xInput.uvTexCoord1); \n"
"	half4 cNormalTexture = tex2D( sNormal1, xInput.uvTexCoord1); \n"
"	half4 cVertexColour = xInput.cColour;	\n"

"	half3 vView = normalize( xInput.uvwViewDirection.xyz);	\n"
"	half3 vLight = normalize( xInput.uvwLightDirection.xyz);	\n"
"	half3 vHalf = normalize( vView + vLight);	\n"
"	half3 vNormal = normalize( (2.0 * cNormalTexture.xyz) - 1.0);	\n"

"	half hNdotL = dot( vNormal, vLight);	\n"
"	half hNdotH = max( 0, dot( vNormal, vHalf));	\n"
"	half hSpecular = pow( hNdotH, g_ah4Const0.x);	\n"

//"	half3 cColourDiffuse = cDiffuseTexture.xyz * (1.0 + hNdotL);	\n"
"	half3 cColourDiffuse = cDiffuseTexture.xyz;	\n"
"	half3 cColourSpecular = cSpecularTexture.xyz * hSpecular;	\n"
"	half3 cColourSelfIlluminating = cDiffuseTexture.xyz * g_ah4Const0.w;	\n"
"	half3 cColour = (2.0 * cVertexColour * (cColourDiffuse + cColourSpecular)) + cColourSelfIlluminating;	\n"
"	\n"

"	xOutput.cColour0.rgb = cColour;	\n"
"	xOutput.cColour0.a = cDiffuseTexture.a;		\n"

"	return( xOutput);	\n"
"	\n"
"}	\n"

};

static Asura_Direct3D_PixelShader xPS_ENV_HIGHEND_AMBIENT_NORMAL_DIFFUSE1_SPECULAR1(
	ASURA_D3D_PS_ENV_HIGHEND_AMBIENT_NORMAL_DIFFUSE1_SPECULAR1, 
	szCODE_Ambient_Normal_Diffuse1_Specular1_SelfIllum, 2.0, ASURA_D3D_PS_CODE_TYPE_HLSL, "main_Ambient_Normal_Diffuse1_Specular1_SelfIllum");

// Ambient_Diffuse2_Specular1_SelfIllum
//
//	 Expected constants:
//		c0.x     :	Specular power
//		c0.a     :	Self Illumination factor
//
const char* const szCODE_Ambient_Diffuse2_Specular1_SelfIllum =
{
	"ps_2_0							\n"				

	"dcl_pp t0.xy					\n" 		// Tex coords
	"dcl_pp t1.xy					\n" 		// Tex coords
	"dcl_pp t2.xyz					\n" 		// Get view vector
	"dcl_pp t3.xyz					\n" 		// Get normal
	"dcl_pp t4						\n" 		// Get light direction
	"dcl v0							\n" 		// Get vertex color
	"dcl_2d s0						\n" 		// Sampler for diffuse tex 1
	"dcl_2d s1						\n" 		// Sampler for specular map
	"dcl_2d s2						\n" 		// Sampler for diffuse tex 2

	"def c1, 2,0,2,2				\n"

	// Sample textures
	"texld_pp r0, t0, s0			\n" 		// Sample diffuse tex 1
	"texld_pp r1, t0, s1			\n" 		// Sample specular map
	"texld_pp r2, t1, s2			\n" 		// Sample diffuse tex 2

	// Blend diffuse textures
	"lrp_pp r0, v0.a, r0, r2		\n"

	// Calculate half angle vector
	"nrm_pp r2.xyz, t2				\n"			// Normalize view vector
	"nrm_pp r4.xyz, t4				\n"			// Normalize light vector
	"add_pp r4.xyz, r2, r4			\n"			// Half angle = view + light
	"nrm_pp r2.xyz, r4				\n"			// Normalize half angle vector

	// Dot with surface normal and raise to power
	"nrm_pp r3, t3					\n" 		// Normalize surface normal
	"dp3_pp r3.w, r3, r2			\n"			// N.H
	"max_pp r3.w, r3.w, c1.y		\n"			// Clamp( N.H)
	"pow_pp r2.w, r3.w, c0.x		\n"			// N.H^power

	// Calculate final color
	"mad_pp r1.rgb, r1, r2.w, r0	\n"			// Diffuse tex + specular map * N.H^power
	"mul_pp r1.rgb, r1, v0			\n"			// Multiply by lighting
	"mul_pp r1.rgb, r1, c1.x			\n"			// x2
	"mad_pp r0.rgb, r0, c0.w, r1	\n"			// Add self-illum
	"mov_pp oC0, r0					\n"
};

static Asura_Direct3D_PixelShader xPS_ENV_HIGHEND_AMBIENT_DIFFUSE2_SPECULAR1(
	ASURA_D3D_PS_ENV_HIGHEND_AMBIENT_DIFFUSE2_SPECULAR1, 
	szCODE_Ambient_Diffuse2_Specular1_SelfIllum);


// Ambient_Normal_Diffuse2_Specular1_SelfIllum
//
//	 Expected constants:
//		c0.x     :	Specular power
//		c0.a     :	Self Illumination factor
//
const char* const szCODE_Ambient_Normal_Diffuse2_Specular1_SelfIllum =
{
"half4 g_ah4Const0 : register( c0);	\n"

"sampler sDiffuse1 : register( s0);	\n"
"sampler sSpecular1 : register( s1);	\n"
"sampler sDiffuse2 : register( s2);	\n"
"sampler sNormal1 : register( s3);	\n"

"struct PS_INPUT_20	\n"
"{	\n"
"	float2 uvTexCoord1 : TEXCOORD0;	\n"
"	float2 uvTexCoord2 : TEXCOORD1;	\n"
"	float3 uvwViewDirection	: TEXCOORD2;	\n"
"	float3 uvwLightDirection	: TEXCOORD3;	\n"
"	half4  cColour	: COLOR0;	\n"
"		\n"
"};	\n"

"struct PS_OUTPUT_20	\n"
"{	\n"
"	half4 cColour0	: COLOR0;	\n"
"};	\n"

"PS_OUTPUT_20 main_Ambient_Normal_Diffuse2_Specular1_SelfIllum( const PS_INPUT_20 xInput)	\n"
"{	\n"
"	PS_OUTPUT_20 xOutput;	\n"

"	half4 cDiffuse1Texture = tex2D( sDiffuse1, xInput.uvTexCoord1); \n"
"	half4 cDiffuse2Texture = tex2D( sDiffuse2, xInput.uvTexCoord2); \n"
"	half4 hTextureLerp = xInput.cColour.a;	\n"
"	half4 cDiffuseTexture = lerp( cDiffuse2Texture, cDiffuse1Texture, hTextureLerp); \n"

"	half4 cSpecularTexture = tex2D( sSpecular1, xInput.uvTexCoord1); \n"
"	half4 cNormalTexture = tex2D( sNormal1, xInput.uvTexCoord1); \n"
"	half4 cVertexColour = xInput.cColour;	\n"

"	half3 vView = normalize( xInput.uvwViewDirection.xyz);	\n"
"	half3 vLight = normalize( xInput.uvwLightDirection.xyz);	\n"
"	half3 vHalf = normalize( vView + vLight);	\n"
"	half3 vNormal = normalize( (2.0 * cNormalTexture.xyz) - 1.0);	\n"

"	half hNdotL = dot( vNormal, vLight);	\n"
"	half hNdotH = max( 0, dot( vNormal, vHalf));	\n"
"	half hSpecular = pow( hNdotH, g_ah4Const0.x);	\n"

//"	half3 cColourDiffuse = cDiffuseTexture.xyz * (1.0 + hNdotL);	\n"
"	half3 cColourDiffuse = cDiffuseTexture.xyz;	\n"
"	half3 cColourSpecular = cSpecularTexture.xyz * hSpecular;	\n"
"	half3 cColourSelfIlluminating = cDiffuseTexture.xyz * g_ah4Const0.w;	\n"
"	half3 cColour = (2.0 * cVertexColour * (cColourDiffuse + cColourSpecular)) + cColourSelfIlluminating;	\n"
"	\n"

"	xOutput.cColour0.rgb = cColour;	\n"
"	xOutput.cColour0.a = cDiffuseTexture.a;		\n"

"	return( xOutput);	\n"
"	\n"
"}	\n"
};

static Asura_Direct3D_PixelShader xPS_ENV_HIGHEND_AMBIENT_NORMAL_DIFFUSE2_SPECULAR1(
	ASURA_D3D_PS_ENV_HIGHEND_AMBIENT_NORMAL_DIFFUSE2_SPECULAR1, 
	szCODE_Ambient_Normal_Diffuse2_Specular1_SelfIllum, 2.0, ASURA_D3D_PS_CODE_TYPE_HLSL, "main_Ambient_Normal_Diffuse2_Specular1_SelfIllum");


// Ambient_Diffuse2_Specular2_SelfIllum
//
//	 Expected constants:
//		c0.x     :	Specular power
//		c0.a     :	Self Illumination factor
//
const char* const szCODE_Ambient_Diffuse2_Specular2_SelfIllum =
{
	"ps_2_0							\n"				

	"dcl_pp t0.xy					\n" 		// Tex 1 coords
	"dcl_pp t1.xy					\n" 		// Tex 2 coords
	"dcl_pp t2.xyz					\n" 		// Get view vector
	"dcl_pp t3.xyz					\n" 		// Get normal
	"dcl_pp t4						\n" 		// Get light direction
	"dcl v0							\n" 		// Get vertex color
	"dcl_2d s0						\n" 		// Sampler for diffuse tex 1
	"dcl_2d s1						\n" 		// Sampler for specular map 1
	"dcl_2d s2						\n" 		// Sampler for diffuse tex 2
	"dcl_2d s3						\n" 		// Sampler for specular map 2

	"def c1, 2,0,2,2				\n"

	// Sample textures
	"texld_pp r0, t0, s0			\n" 		// Sample diffuse tex 1
	"texld_pp r1, t0, s1			\n" 		// Sample specular map 1
	"texld_pp r2, t1, s2			\n" 		// Sample diffuse tex 2
	"texld_pp r3, t1, s2			\n" 		// Sample specular map 2

	// Blend diffuse & specular textures
	"lrp_pp r0, v0.a, r0, r2		\n"
	"lrp_pp r1, v0.a, r1, r3		\n"

	// Calculate half angle vector
	"nrm_pp r2.xyz, t2				\n"			// Normalize view vector
	"nrm_pp r4.xyz, t4				\n"			// Normalize light vector
	"add_pp r4.xyz, r2, r4			\n"			// Half angle = view + light
	"nrm_pp r2.xyz, r4				\n"			// Normalize half angle vector

	// Dot with surface normal and raise to power
	"nrm_pp r3, t3					\n" 		// Normalize surface normal
	"dp3_pp r3.w, r3, r2			\n"			// N.H
	"max_pp r3.w, r3.w, c2.z		\n"			// Clamp( N.H)
	"pow_pp r2.w, r3.w, c0.x		\n"			// N.H^power

	// Calculate final color
	"mad_pp r1.rgb, r1, r2.w, r0	\n"			// Diffuse tex + specular map * N.H^power
	"mul_pp r1.rgb, r1, v0			\n"			// Multiply by lighting
	"mul_pp r1.rgb, r1, c1.x			\n"			// x2
	"mad_pp r0.rgb, r0, c0.w, r1	\n"			// Add self-illum
	"mov_pp oC0, r0					\n"
};

static Asura_Direct3D_PixelShader xPS_ENV_HIGHEND_AMBIENT_DIFFUSE2_SPECULAR2(
	ASURA_D3D_PS_ENV_HIGHEND_AMBIENT_DIFFUSE2_SPECULAR2, 
	szCODE_Ambient_Diffuse2_Specular2_SelfIllum);


// Ambient_Normal_Diffuse2_Specular2_SelfIllum
//
//	 Expected constants:
//		c0.x     :	Specular power
//		c0.a     :	Self Illumination factor
//
const char* const szCODE_Ambient_Normal_Diffuse2_Specular2_SelfIllum =
{
"half4 g_ah4Const0 : register( c0);	\n"

"struct PS_INPUT_20	\n"
"{	\n"
"	float2 uvTexCoord1 : TEXCOORD0;	\n"
"	float2 uvTexCoord2 : TEXCOORD1;	\n"
"	float3 uvwViewDirection	: TEXCOORD2;	\n"
"	float3 uvwLightDirection	: TEXCOORD3;	\n"
"	half4  cColour	: COLOR0;	\n"
"		\n"
"};	\n"

"struct PS_OUTPUT_20	\n"
"{	\n"
"	half4 cColour0	: COLOR0;	\n"
"};	\n"
PS_HLSL_FRAGMENT_LOAD_DIFFUSE2( "s0", "s2")
PS_HLSL_FRAGMENT_LOAD_SPECULAR2( "s1", "s3")
PS_HLSL_FRAGMENT_LOAD_NORMAL1( "s4")
"PS_OUTPUT_20 main_Ambient_Normal_Diffuse2_Specular2_SelfIllum( const PS_INPUT_20 xInput)	\n"
"{	\n"
"	half4 cDiffuseTexture = GetDiffuseTexture( xInput); \n"
"	half4 cSpecularTexture = GetSpecularTexture( xInput); \n"
"	half4 cNormalTexture = GetNormalTexture( xInput); \n"
"	half4 cVertexColour = xInput.cColour;	\n"

"	half3 vView = normalize( xInput.uvwViewDirection.xyz);	\n"
"	half3 vLight = normalize( xInput.uvwLightDirection.xyz);	\n"
"	half3 vHalf = normalize( vView.xyz + vLight.xyz);	\n"
"	half3 vNormal = normalize( (2.0 * cNormalTexture.xyz) - 1.0);	\n"

"	half hNdotL = dot( vNormal.xyz, vLight.xyz);	\n"
"	half hNdotH = max( 0, dot( vNormal.xyz, vHalf.xyz));	\n"
"	half hSpecular = pow( hNdotH, g_ah4Const0.x);	\n"

//"	half3 cColourDiffuse = cDiffuseTexture.xyz * (1.0 + hNdotL);	\n"
"	half3 cColourDiffuse = cDiffuseTexture.xyz;	\n"
"	half3 cColourSpecular = cSpecularTexture.xyz * hSpecular;	\n"
"	half3 cColourSelfIlluminating = cDiffuseTexture.xyz * g_ah4Const0.w;	\n"
"	half3 cColour = (2.0 * cVertexColour * (cColourDiffuse + cColourSpecular.xyz)) + cColourSelfIlluminating;	\n"
"	\n"
"	PS_OUTPUT_20 xOutput;	\n"
"	\n"
"	xOutput.cColour0.rgb = cColour;	\n"
"	xOutput.cColour0.a = cDiffuseTexture.a;		\n"

"	return( xOutput);	\n"
"	\n"
"}	\n"
};

static Asura_Direct3D_PixelShader xPS_ENV_HIGHEND_AMBIENT_NORMAL_DIFFUSE2_SPECULAR2(
	ASURA_D3D_PS_ENV_HIGHEND_AMBIENT_NORMAL_DIFFUSE2_SPECULAR2, 
	szCODE_Ambient_Normal_Diffuse2_Specular2_SelfIllum, 2.0, ASURA_D3D_PS_CODE_TYPE_HLSL, "main_Ambient_Normal_Diffuse2_Specular2_SelfIllum");


//
// Dynamic_Diffuse1
//
const char* const szCODE_Dynamic_Diffuse1 =
{
	"ps.1.1						\n"

	"tex t0						\n"	
	"tex t2						\n"	
	"tex t3						\n"	

	"mul r0, t0, t2				\n"		// Tex * Light Mask XY
	"mul r0, r0, t3				\n"		// * Light Mask Z
	"mul_x2 r0.rgb, r0, v0		\n"		// * 2 * light color
	"mul r0, r0, v1.a			\n"		// Apply fog
};

static Asura_Direct3D_PixelShader xPS_ENV_HIGHEND_DYNAMIC_DIFFUSE1(
	ASURA_D3D_PS_ENV_HIGHEND_DYNAMIC_DIFFUSE1, 
	szCODE_Dynamic_Diffuse1);

//
// Dynamic_Diffuse2
//
const char* const szCODE_Dynamic_Diffuse2 =
{
	"ps.1.1						\n"

	"tex t0						\n"	
	"tex t1						\n"	
	"tex t2						\n"	
	"tex t3						\n"	

	"lrp r0, v0.a, t0, t1		\n"		// Blend textures
	"mul r0, r0, t2				\n"		// Tex * Light Mask XY
	"mul r0, r0, t3				\n"		// * Light Mask Z
	"mul_x2 r0.rgb, r0, v0		\n"		// * 2 * light color
	"mul r0, r0, v1.a			\n"		// Apply fog
};

static Asura_Direct3D_PixelShader xPS_ENV_HIGHEND_DYNAMIC_DIFFUSE2(
	ASURA_D3D_PS_ENV_HIGHEND_DYNAMIC_DIFFUSE2, 
	szCODE_Dynamic_Diffuse2);

//
// Dynamic_Normal_Diffuse1
//
const char* const szCODE_Dynamic_Normal_Diffuse1 =
{
	"ps.2.0						\n"

	"def c1, 0,1,2,0.5			\n"
	"def c2, 2,-1,-1,-1			\n"
	
	"dcl_pp t0.xy					\n" 		// Tex coords
	"dcl_pp t1.xy					\n" 		// Tex coords
	"dcl_pp t2.xy					\n" 		// Camera Space Light Texture
	"dcl_pp t3.xy					\n" 		// Camera Space ZW Light Texture
	"dcl_pp t4						\n" 		// Get light direction
	
	"dcl v0							\n" 		// Get vertex color
	
	"dcl_2d s0						\n" 		// Sampler for diffuse tex 1
	"dcl_2d s1						\n" 		// Sampler for normal texture
	"dcl_2d s2						\n" 		// Sampler for light texture

	// Sample textures
	"texld_pp r0, t0, s0			\n" 		// Sample diffuse tex 1
	"texld_pp r2, t2, s2			\n" 		// Sample light xy
	"texld_pp r3, t3, s2			\n" 		// Sample light zw
	"texld_pp r5, t0, s1			\n" 		// Sample compressed normal map
	
	"mad_pp r5, r5, c2.x, c2.y	\n"			// Decompress normal
	"mul_pp r2.rgb, r2, r3	\n"				// Light XY * Light ZW

	// Blend diffuse & specular textures
//	"lrp_pp r0, v0.a, r0, r2		\n"

	// Dot with surface normal and raise to power
	"nrm_pp r3, r5					\n" 		// Normalize surface normal
	"dp3_pp r3.w, r3, t4			\n"			// N.L
	"max_pp r3.w, r3.w, c1.x		\n"			// Clamp N.L

	// Calculate final color
	"mul_pp r1.rgb, r0, r3.w		\n"				// Diffuse tex * Normal Lighting
	"mul_pp r1.rgb, r1, r2		\n"				// Diffuse tex * Light Textures
	"add_pp r1.rgb, r1, r1		\n"			// x2
	"mul_pp r0.rgb, r1, v0		\n"			// Multiply by light colour
	
	"mov_pp oC0, r0				\n"
};

static Asura_Direct3D_PixelShader xPS_ENV_HIGHEND_DYNAMIC_NORMAL_DIFFUSE1(
	ASURA_D3D_PS_ENV_HIGHEND_DYNAMIC_NORMAL_DIFFUSE1, 
	szCODE_Dynamic_Normal_Diffuse1);

//
// Dynamic_Diffuse2
//
const char* const szCODE_Dynamic_Normal_Diffuse2 =
{
	"ps.2.0						\n"

	"def c1, 0,1,2,0.5			\n"
	"def c2, 2,-1,-1,-1			\n"
	
	"dcl_pp t0.xy					\n" 		// Tex coords
	"dcl_pp t1.xy					\n" 		// Tex coords
	"dcl_pp t2.xy					\n" 		// Camera Space XY Light Texture
	"dcl_pp t3.xy					\n" 		// Camera Space ZW Light Texture
	"dcl_pp t4						\n" 		// Get light direction
	
	"dcl v0							\n" 		// Get vertex color
	
	"dcl_2d s0						\n" 		// Sampler for diffuse tex 1
	"dcl_2d s1						\n" 		// Sampler for diffuse tex 2
	"dcl_2d s4						\n" 		// Sampler for normal texture
	"dcl_2d s2						\n" 		// Sampler for light texture

	// Sample textures
	"texld_pp r0, t0, s0			\n" 		// Sample diffuse tex 1
	"texld_pp r1, t1, s1			\n" 		// Sample diffuse tex 2
	"texld_pp r5, t0, s4			\n" 		// Sample compressed normal map
	"texld_pp r2, t2, s2			\n" 		// Sample light xy
	"texld_pp r3, t3, s2			\n" 		// Sample light zw
	
	"mad_pp r5, r5, c2.x, c2.y	\n"			// Decompress normal
	"mul_pp r2.rgb, r2, r3	\n"				// Light XY * Light ZW

	// Blend diffuse & specular textures
	"lrp_pp r0, v0.a, r0, r1		\n"

	// Dot with surface normal and raise to power
	"nrm_pp r3, r5					\n" 		// Normalize surface normal
	"dp3_pp r3.w, r3, t4			\n"			// N.L
	"max_pp r3.w, r3.w, c1.x		\n"			// Clamp N.L

	// Calculate final color
	"mul_pp r1.rgb, r0, r3.w		\n"				// Diffuse tex * Normal Lighting
	"mul_pp r1.rgb, r1, r2		\n"				// Diffuse tex * Light Textures
	"add_pp r1.rgb, r1, r1		\n"			// x2
	"mul_pp r0.rgb, r1, v0		\n"			// Multiply by light colour
	
	"mov_pp oC0, r0				\n"
};

static Asura_Direct3D_PixelShader xPS_ENV_HIGHEND_DYNAMIC_NORMAL_DIFFUSE2(
	ASURA_D3D_PS_ENV_HIGHEND_DYNAMIC_NORMAL_DIFFUSE2, 
	szCODE_Dynamic_Normal_Diffuse2);

#define PS_HLSL_FRAGMENT_DYNAMIC_DIFFUSE_SPECULAR_HEADER	\
"half4 g_ah4Const0 : register( c0);	\n"	\
"\n"	\
"struct PS_INPUT_20	\n"	\
"{	\n"	\
"	float2 uvTexCoord1 : TEXCOORD0;	\n"	\
"	float2 uvTexCoord2 : TEXCOORD1;	\n"	\
"	float2 uvLightCoord1 : TEXCOORD2;	\n"	\
"	float2 uvLightCoord2 : TEXCOORD3;	\n"	\
"	float3 uvwLightDirection	: TEXCOORD4;	\n"	\
"	float3 uvwViewDirection	: TEXCOORD5;	\n"	\
"	half4  cColour	: COLOR0;	\n"	\
"		\n"	\
"};	\n"

#define PS_HLSL_FRAGMENT_DYNAMIC_DIFFUSE_SPECULAR_BODY( ENTRY_POINT)	\
"struct PS_OUTPUT_20	\n"	\
"{	\n"	\
"	half4 cColour0	: COLOR0;	\n"	\
"};	\n"	\
"	\n"	\
"PS_OUTPUT_20 "ENTRY_POINT"( const PS_INPUT_20 xInput)	\n"	\
"{	\n"	\
"	half4 cDiffuseTexture = GetDiffuseTexture( xInput); \n"	\
"	half4 cSpecularTexture = GetSpecularTexture( xInput); \n"	\
"	half4 cNormalTexture = GetNormalTexture( xInput); \n"	\
"	half3 cLightTexture = GetDynamicLightTexture( xInput); \n"	\
"	half4 cVertexColour = xInput.cColour;	\n"	\
"	\n"	\
"	half3 vView = normalize( xInput.uvwViewDirection.xyz);	\n"	\
"	half3 vLight = normalize( xInput.uvwLightDirection.xyz);	\n"	\
"	half3 vHalf = normalize( vView.xyz + vLight.xyz);	\n"	\
"	half3 vNormal = normalize( (2.0 * cNormalTexture.xyz) - 1.0);	\n"	\
"	\n"	\
"	half hNdotL = max( 0, dot( vNormal.xyz, vLight.xyz));	\n"	\
"	half hNdotH = max( 0, dot( vNormal.xyz, vHalf.xyz));	\n"	\
"	half hSpecular = pow( hNdotH, g_ah4Const0.x);	\n"	\
"	\n"	\
"	half3 cColourDiffuse = cDiffuseTexture.xyz * hNdotL;	\n"	\
"	half3 cColourSpecular = cSpecularTexture.xyz * hSpecular;	\n"	\
"	half3 cColour = 2.0 * cVertexColour * cLightTexture * (cColourDiffuse + cColourSpecular.xyz);	\n"	\
"	\n"	\
"	PS_OUTPUT_20 xOutput;	\n"	\
"	\n"	\
"	xOutput.cColour0.rgb = cColour;	\n"	\
"	xOutput.cColour0.a = cDiffuseTexture.a;		\n"	\
"	\n"	\
"	return( xOutput);	\n"	\
"	\n"	\
"}	\n"

// Dynamic_Normal_Diffuse1_Specular1
//
//	 Expected constants:
//		c0.x     :	Specular power
//		c0.a     :	Self Illumination factor
//

const char* const szCODE_Dynamic_Normal_Diffuse1_Specular1 =
{
PS_HLSL_FRAGMENT_DYNAMIC_DIFFUSE_SPECULAR_HEADER
"	\n"
PS_HLSL_FRAGMENT_LOAD_DIFFUSE1( "s0")
PS_HLSL_FRAGMENT_LOAD_SPECULAR1( "s1")
PS_HLSL_FRAGMENT_LOAD_NORMAL1( "s4")
PS_HLSL_FRAGMENT_LOAD_LIGHT( "s2")
"	\n"
PS_HLSL_FRAGMENT_DYNAMIC_DIFFUSE_SPECULAR_BODY("main_Dynamic_Normal_Diffuse1_Specular1")
};

static Asura_Direct3D_PixelShader xPS_ENV_HIGHEND_DYNAMIC_NORMAL_DIFFUSE1_SPECULAR1(
	ASURA_D3D_PS_ENV_HIGHEND_DYNAMIC_NORMAL_DIFFUSE1_SPECULAR1, 
	szCODE_Dynamic_Normal_Diffuse1_Specular1, 2.0, ASURA_D3D_PS_CODE_TYPE_HLSL, "main_Dynamic_Normal_Diffuse1_Specular1");

// Dynamic_Normal_Diffuse2_Specular1
//
//	 Expected constants:
//		c0.x     :	Specular power
//		c0.a     :	Self Illumination factor
//

const char* const szCODE_Dynamic_Normal_Diffuse2_Specular1 =
{
PS_HLSL_FRAGMENT_DYNAMIC_DIFFUSE_SPECULAR_HEADER
"	\n"
PS_HLSL_FRAGMENT_LOAD_DIFFUSE2( "s0", "s5")
PS_HLSL_FRAGMENT_LOAD_SPECULAR1( "s1")
PS_HLSL_FRAGMENT_LOAD_NORMAL1( "s4")
PS_HLSL_FRAGMENT_LOAD_LIGHT( "s2")
"	\n"
PS_HLSL_FRAGMENT_DYNAMIC_DIFFUSE_SPECULAR_BODY("main_Dynamic_Normal_Diffuse2_Specular1")
};

static Asura_Direct3D_PixelShader xPS_ENV_HIGHEND_DYNAMIC_NORMAL_DIFFUSE2_SPECULAR1(
	ASURA_D3D_PS_ENV_HIGHEND_DYNAMIC_NORMAL_DIFFUSE2_SPECULAR1, 
	szCODE_Dynamic_Normal_Diffuse2_Specular1, 2.0, ASURA_D3D_PS_CODE_TYPE_HLSL, "main_Dynamic_Normal_Diffuse2_Specular1");

// Dynamic_Normal_Diffuse1_Specular2
//
//	 Expected constants:
//		c0.x     :	Specular power
//		c0.a     :	Self Illumination factor
//

const char* const szCODE_Dynamic_Normal_Diffuse1_Specular2 =
{
PS_HLSL_FRAGMENT_DYNAMIC_DIFFUSE_SPECULAR_HEADER
"	\n"
PS_HLSL_FRAGMENT_LOAD_DIFFUSE1( "s0")
PS_HLSL_FRAGMENT_LOAD_SPECULAR2( "s1", "s5")
PS_HLSL_FRAGMENT_LOAD_NORMAL1( "s4")
PS_HLSL_FRAGMENT_LOAD_LIGHT( "s2")
"	\n"
PS_HLSL_FRAGMENT_DYNAMIC_DIFFUSE_SPECULAR_BODY("main_Dynamic_Normal_Diffuse1_Specular2")
};

static Asura_Direct3D_PixelShader xPS_ENV_HIGHEND_DYNAMIC_NORMAL_DIFFUSE1_SPECULAR2(
	ASURA_D3D_PS_ENV_HIGHEND_DYNAMIC_NORMAL_DIFFUSE1_SPECULAR2, 
	szCODE_Dynamic_Normal_Diffuse1_Specular2, 2.0, ASURA_D3D_PS_CODE_TYPE_HLSL, "main_Dynamic_Normal_Diffuse1_Specular2");

// Dynamic_Normal_Diffuse2_Specular1
//
//	 Expected constants:
//		c0.x     :	Specular power
//		c0.a     :	Self Illumination factor
//

const char* const szCODE_Dynamic_Normal_Diffuse2_Specular2 =
{
PS_HLSL_FRAGMENT_DYNAMIC_DIFFUSE_SPECULAR_HEADER
"	\n"
PS_HLSL_FRAGMENT_LOAD_DIFFUSE2( "s0", "s5")
PS_HLSL_FRAGMENT_LOAD_SPECULAR2( "s1", "s6")
PS_HLSL_FRAGMENT_LOAD_NORMAL1( "s4")
PS_HLSL_FRAGMENT_LOAD_LIGHT( "s2")
"	\n"
PS_HLSL_FRAGMENT_DYNAMIC_DIFFUSE_SPECULAR_BODY("main_Dynamic_Normal_Diffuse2_Specular2")
};

static Asura_Direct3D_PixelShader xPS_ENV_HIGHEND_DYNAMIC_NORMAL_DIFFUSE2_SPECULAR2(
	ASURA_D3D_PS_ENV_HIGHEND_DYNAMIC_NORMAL_DIFFUSE2_SPECULAR2, 
	szCODE_Dynamic_Normal_Diffuse2_Specular2, 2.0, ASURA_D3D_PS_CODE_TYPE_HLSL, "main_Dynamic_Normal_Diffuse2_Specular2");

//
// ZOnly_AlphaTest
//
const char* const szCODE_ZOnly_AlphaTest =
{
	"ps.1.1						\n"

	"tex t0						\n"	
	"mov r0, t0					\n"
};

static Asura_Direct3D_PixelShader xPS_ENV_HIGHEND_ZONLY_ALPHATEST(
	ASURA_D3D_PS_ENV_HIGHEND_ZONLY_ALPHATEST, 
	szCODE_ZOnly_AlphaTest);


//
// AdditiveAlpha
//
const char* const szCODE_AdditiveAlpha =
{
	"ps.1.1						\n"

	"tex t0						\n"	
	"mul_x2 r0, t0, v0			\n"		// Tex * lighting

};

static Asura_Direct3D_PixelShader xPS_ENV_HIGHEND_ADDITIVEALPHA(
	ASURA_D3D_PS_ENV_HIGHEND_ADDITIVEALPHA, 
	szCODE_AdditiveAlpha);





#ifdef ASURA_TOOLS

static const char* szCODE_Tools_Specular1 =
{
	// Expected constants :
	// c0.w		Specular power

	"ps_2_0							\n"				

	"dcl_pp t0.xy					\n" 		// Tex coords for specular map 1
	"dcl_pp t2.xyz					\n" 		// Get view vector
	"dcl_pp t3.xyz					\n" 		// Get normal
	"dcl_pp t4						\n" 		// Get light direction
	"dcl v0							\n" 		// Get vertex color
	"dcl_2d s0						\n" 		// Sampler for specular map 1

	"def c1, 2,2,2,2				\n"

	// Sample textures
	"texld_pp r0, t0, s0			\n" 		// Sample specular map 1

	// Calculate half angle vector
	"nrm_pp r2.xyz, t2				\n"			// Normalize view vector
	"nrm_pp r4.xyz, t4				\n"			// Normalize light vector
	"add_pp r4.xyz, r2, r4			\n"			// Half angle = view + light
	"nrm_pp r2.xyz, r4				\n"			// Normalize half angle vector

	// Dot with surface normal and raise to power
	"nrm_pp r3, t3					\n" 		// Normalize surface normal
	"dp3_pp r3.w, r3, r2			\n"			// N.H
	"pow_pp r0.w, r3.w, c0.w		\n"			// N.H^power

	// Multiply by lighting
	"mul_pp r0.rgb, r0, v0			\n"
	"mul_pp r0.rgb, r0, c1			\n"

	// Output specular color
	"mov_pp oC0, r0					\n"
};

static Asura_Direct3D_PixelShader xToolsSpecular1(
	ASURA_D3D_PS_TOOLS_ENVIRONMENT_SPECULAR_MAPS, 
	szCODE_Tools_Specular1);

static const char* szCODE_Tools_Specular2 =
{
	// Expected constants :
	// c0.w		Specular power

	"ps_2_0							\n"				

	"dcl_pp t0.xy					\n" 		// Tex coords for specular map 1
	"dcl_pp t1.xy					\n" 		// Tex coords for specular map 1
	"dcl_pp t2.xyz					\n" 		// Get view vector
	"dcl_pp t3.xyz					\n" 		// Get normal
	"dcl_pp t4						\n" 		// Get light direction
	"dcl v0							\n" 		// Get vertex color
	"dcl_2d s0						\n" 		// Sampler for specular map 1
	"dcl_2d s1						\n" 		// Sampler for specular map 1

	"def c1, 2,2,2,2				\n"

	// Sample textures
	"texld_pp r0, t0, s0			\n" 		// Sample specular map 1
	"texld_pp r1, t1, s1			\n" 		// Sample specular map 2

	// Calculate half angle vector
	"nrm_pp r2.xyz, t2				\n"			// Normalize view vector
	"nrm_pp r4.xyz, t4				\n"			// Normalize light vector
	"add_pp r4.xyz, r2, r4			\n"			// Half angle = view + light
	"nrm_pp r2.xyz, r4				\n"			// Normalize half angle vector

	// Dot with surface normal and raise to power
	"nrm_pp r3, t3					\n" 		// Normalize surface normal
	"dp3_pp r3.w, r3, r2			\n"			// N.H
	"pow_pp r0.w, r3.w, c0.w		\n"			// N.H^power

	// Blend specular maps using vertex alpha
	"lrp_pp r0.rgb, v0.a, r0, r1	\n"

	// Multiply by lighting
	"mul_pp r0.rgb, r0, v0			\n"
	"mul_pp r0.rgb, r0, c1			\n"

	// Output specular color
	"mov_pp oC0, r0					\n"
};

static Asura_Direct3D_PixelShader xToolsSpecular2(
	ASURA_D3D_PS_TOOLS_ENVIRONMENT_DUAL_SPECULAR_MAPS, 
	szCODE_Tools_Specular2);

#endif // ASURA_TOOLS

#endif // ASURA_USE_SPECULAR_MAPS
