//
// Asura_D3D_PS_Outliner.cpp
//
// Created by SB - 07 November 2006
//
// Copyright (C) 2005 Rebellion, All Rights Reserved.
//

#if defined(ASURA_USE_LINE_RENDERING)

#include "Core/Asura.h"

// This file's header.
#include "Rendering/Asura_Direct3D_Shaders.h"

static Asura_Direct3D_PixelShader xIDRenderPixelShader(
	ASURA_D3D_PS_IDRENDER,

	// ID comes in in c0.rg
	"float4 id : register (c0);\n"				// ID data from vertex parameters
	// Options come in in c[123].r
	"float alphacutoff : register (c1);\n"		// Alpha cutoff (and enable/disable on/off alpha)
	"float2 idfromvertex : register (c2);\n"	// Controls whether runtime and part IDs come from vertex or parameters
	"float maskfromnormal : register (c3);\n"	// Controls whether line masking data comes from normal or vertex
	"sampler2D tex0 : register (s0);\n"

	"float4 main(float4 colour : COLOR, float4 texcoord : TEXCOORD0, float4 normal : TEXCOORD2) : COLOR\n"
	"{\n"
	// Pretty straightforward - render to gb, masking comes from red channel of colour
	// Alpha comes from texture tested against a threshold set by on/off transparency flag
	// On PC depth value should go into the red channel to allow distance fading of outlines
	// Added options to allow ID to come from vertex colour and for mask to come from normal
	// if required
	"	float alpha = tex2D(tex0, texcoord).a;\n"
	"	float4 idtouse = id;\n"
	"	float masktouse = colour.r;\n"
	"	if (alpha >= alphacutoff)\n"
	"	{\n"
	"		alpha = 1.0f;\n"
	"	}\n"
	"	else\n"
	"	{\n"
	"		alpha = 0.0f;\n"
	"	}\n"
	"	if (idfromvertex.x > 0.5f)\n" // RuntimeID from vertex colour?
	"	{\n"
	"		idtouse.r = colour.g;\n"
	"	}\n"
	"	if (idfromvertex.y > 0.5f)\n" // PartID from vertex colour?
	"	{\n"
	"		idtouse.g = colour.b;\n"
	"	}\n"
	"	if (maskfromnormal.x > 0.5f)\n" // Mask from normal?
	"	{\n"
	"		masktouse = abs(normal.x + normal.y + normal.z);\n"
	"	}\n"
	"	return float4(masktouse, idtouse.r, idtouse.g, alpha);\n"
	"}\n"
	,

	2.0f,
	ASURA_D3D_PS_CODE_TYPE_HLSL,
	"main"
);

#endif