//
// Asura_D3D_PS_Outliner.cpp
//
// Created by SB - 07 November 2006
//
// Copyright (C) 2005 Rebellion, All Rights Reserved.
//

#if defined(ASURA_USE_LINE_RENDERING)

#include "Core/Asura.h"

// This file's header.
#include "Rendering/Asura_Direct3D_Shaders.h"

////////////////////////////////////////////////////////////////////////////////
// For some inexplicable reason any constants                                 //
// I define in the shader turn to zeros                                       //
////////////////////////////////////////////////////////////////////////////////

// Official Simpsons outline colour
#define OUTLINE_COLOUR "float4(0.0f, 0.0f, 0.0f, 1.0f)"

static Asura_Direct3D_PixelShader xOutlinerPixelShader(
	ASURA_D3D_PS_OUTLINE,

	// x, y : Line offset - set by game to give 1 pixel lines
	// z : Show ID buffer flag
	// w : Masking threshold
	"float4 cOffset : register (c0);\n"
	
	"sampler2D tex0 : register (s0);\n"
	"sampler2D idbuf : register (s1);\n"

	"float diff(float4 colour0, float4 colour1)\n"
	"{\n"
	"	float3 diff = colour1.xyz - colour0.xyz;\n"
	// Currently Mask Line vertex flag goes to red channel
	"	if (colour0.r >= cOffset.w) diff = 0.0f;\n"
	"	return abs(diff.g) + abs(diff.b);\n"
	"}\n"

	"float4 main(float2 texCoord0 : TEXCOORD0) : COLOR\n"
	"{\n"
	"	float4 Out;\n"

	"	float2 baseSamplePoint = texCoord0 + float2(cOffset.x * 0.5, cOffset.y * 0.5);\n"

	"	if (cOffset.z < 0.5f)\n"
	"	{\n"
	"		float4 colour = tex2D(tex0, baseSamplePoint);\n"

	"		float4 sample0 = tex2D(idbuf, baseSamplePoint);\n"
	"		float4 sample1 = tex2D(idbuf, baseSamplePoint + float2(cOffset.x, 0.0f));\n"
	"		float4 sample2 = tex2D(idbuf, baseSamplePoint + float2(0.0f, cOffset.y));\n"
	
	"		float diff0 = diff(sample0, sample1);\n"
	"		float diff1 = diff(sample0, sample2);\n"
	
	"		if (diff0 + diff1 != 0.0f)\n"
	"		{\n"
	"			Out = " OUTLINE_COLOUR ";\n;"
	"		}\n"
	"		else\n"
	"		{\n"
	"			Out = colour;\n"
	"		}\n"
	"	}\n"
	"	else\n"
	"	{\n"
	"		Out = tex2D(idbuf, baseSamplePoint);\n"
	"		if (Out.r < cOffset.w)\n"
	"		{\n"
	"			Out.r = 0.0f;\n"
	"		}\n"
	"	}\n"

	"	return Out;\n"
	"}\n"
	,

	2.0f,
	ASURA_D3D_PS_CODE_TYPE_HLSL,
	"main"
);

#endif
