/**
*
* \file Asura_D3D_VS_Object_3Lights.cpp
*
* Created by Kevin Lea - 09/10/2002 10:11:58
*
* Copyright (C) 2002 Rebellion, All Rights Reserved.
*
* \brief
*
**/

#include "Core/Asura.h"

#include "Rendering/Asura_Direct3D_Shaders.h"

static DWORD Object3LightsVertex_Decl[] =
{
    D3DVSD_STREAM( 0 ),
    D3DVSD_REG( 0,	D3DVSDT_FLOAT3 ),
    D3DVSD_REG( 1,	D3DVSDT_FLOAT3 ),
    D3DVSD_REG( 2,	D3DVSDT_FLOAT2 ),
   
	D3DVSD_END()
};

static char* Object3LightsVertexShaderCode=
{
	"vs.1.1 \n"

	// rotate the vertex and normal into world space ready for lighting
	"dp4 r1.x, v0, c[7]	\n"
	"dp4 r1.y, v0, c[8]	\n"
	"dp4 r1.z, v0, c[9]	\n"
	"dp4 r1.w, v0, c[10] \n"
	"dp3 r3.x, v1, c[7]	\n"
	"dp3 r3.y, v1, c[8]	\n"
	"dp3 r3.z, v1, c[9]	\n"
	"mov r3.w, v1.w	\n"
				
	// copy over the uv co-ords into the first and second stages, adding
	// in the uv animation offset for this object
	"add oT0.xy, v2, c[19] \n"
	"add oT1.xy, v2, c[19] \n"

	// move in the ambient light and alpha value
	"mov r5, c[18] \n"

	// take normal dotted with light1 direction
	"dp3 r4, r3, c[11] \n"
	// make sure it's clamped >= 0
	"max r4, r4, c[17] \n"
	// multiple by light1 colour and add to light total
	"mad r5, c[12], r4.x, r5 \n"
	
	// now do the same for light2
	"dp3 r4, r3, c[13] \n"
	"max r4, r4, c[17] \n"
	"mad r5, c[14], r4.x, r5 \n"
	
	// and then light3, finally moving it to the output diffuse colour
	"dp3 r4, r3, c[15] \n"
	"max r4, r4, c[17] \n"
	"mad oD0, c[16], r4.x, r5 \n"
	
	// move the vertex into viewspace (unfortunately you can't read from
	// oPos, so we have to have to use another register, and copy it into
	// oPos later).
	"dp4 r2.x, r1, c[0] \n"
	"dp4 r2.y, r1, c[1] \n"
	"dp4 r2.z, r1, c[2] \n"
	"dp4 r2.w, r1, c[3] \n"

	// handy hint the viewspace w is actually the camera space z value
	// so we can use this for fogging (DirectX demos use another matrix
	// for the camera and do the work twice).
	
	// calculate FogFar - CameraZ
	"add r0.x, -r2.w, c[20].x \n"
	// and multiply by 1/(FogFar-FogNear)
	"mul oFog, r0.x, c[20].y \n"

	// 1) oFog is zero for completely fogged, and one for no fog, the 
	// opposite of what you might think
	// 2) I'm not clamping oFog - the hardware seems to clamp 0<=oFog<=1
	// for me, so we can save the two instructions it would require

	// Output the viewspace position
	"mov oPos, r2 \n"
};





static Asura_Direct3D_VertexShader Shader(ASURA_D3D_VS_OBJECT3LIGHTS,Object3LightsVertexShaderCode, Object3LightsVertex_Decl);
