//
// Asura_D3D_VSF_Environment.cpp
//
// Created by Kim Randell - 19/11/2004
// 
// Copyright (C) 2004 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

#include "Rendering/Asura_Direct3D_Shaders.h"

// Projection shader fragments
// Simple projection
const char* const szASURA_D3D_VSF_PROJECTION_SIMPLE =

"float4x4 mProj : register(c0); \n" // Projection matrix
"float4x4 mView : register(c4); \n" // View matrix
"void Projection(	in  float4 vPosObject	: POSITION, \n"
"					in	float4 vNormObject	: NORMAL, \n"
"					out float4 vPosProj		: POSITION, \n"
"					out float4 vPosProjCopy : r_vPosProj, \n"
"					out float4 vPosView		: r_vPosView, \n"
"					out float4 vNormView	: r_vNormView, \n"
"					out float3 vColourShift	: r_vColourShift \n"
"					) \n"
"{ \n"
"	vPosProjCopy = vPosProj = mul(vPosObject, mProj); \n" // Projection
"	vPosView = mul(vPosObject, mView); \n" // Worldspace
"	vNormView.xyz = mul(vNormObject, (float3x3)mView); \n"
"	vNormView.w = 0.f; \n"
"	vColourShift = 0.f; \n"
"} \n"
"vertexfragment Asura_D3D_VSF_Projection_Simple = compile_fragment vs_1_1 Projection(); \n";

// Register with Asura_Direct3D_Shaders
Asura_Direct3D_VertexShaderFragment xASURA_D3D_VSF_PROJECTION_SIMPLE(   ASURA_D3D_VSF_PROJECTION_SIMPLE,
																		szASURA_D3D_VSF_PROJECTION_SIMPLE,
																		"Asura_D3D_VSF_Projection_Simple",
																		true);
// Test warped projection
const char* const szASURA_D3D_VSF_PROJECTION_WARPED =

"float4x4 mProj : register(c0); \n" // Projection matrix
"float4x4 mView : register(c4); \n" // View matrix
"float4 c50: register(c50); \n" // Time
"float4 vGravitrons[16] : register(c51); \n"
"void Projection(	in  float4 vPosObject	: POSITION, \n"
"					in	float4 vNormObject	: NORMAL, \n"
"					out float4 vPosProj		: POSITION, \n"
"					out float4 vPosProjCopy : r_vPosProj, \n"
"					out float4 vPosView		: r_vPosView, \n"
"					out float4 vNormView	: r_vNormView, \n"
"					out float3 vColourShift	: r_vColourShift \n"
"					) \n"
"{ \n"
//"   vPosObject.y += sin(vPosObject.x + c50.x); \n" // Deform using sine wave
"		float3 vOffset = float3(0.f, 0.f, 0.f); \n"

"		float3 vDiff = vGravitrons[0].xyz - vPosObject.xyz; \n"
"		float fDist = length(vDiff); \n"
//"		vOffset += vDiff * smoothstep(vGravitrons[1].x, 0.f, saturate(fDist * vGravitrons[0].w)) / fDist; \n"
"		vOffset.y += lerp(vGravitrons[1].x, 0.f, saturate(fDist * vGravitrons[0].w)); \n"

"		vDiff = vGravitrons[2].xyz - vPosObject.xyz; \n"
"		fDist = length(vDiff); \n"
//"		vOffset += vDiff * smoothstep(vGravitrons[3].x, 0.f, saturate(fDist * vGravitrons[2].w)) / fDist; \n"
"		vOffset.y += lerp(vGravitrons[3].x, 0.f, saturate(fDist * vGravitrons[2].w)); \n"

"		vPosObject.xyz += vOffset; \n"
"		vColourShift.xyz = -vOffset.yyy; \n"
/*
"	for(int i = 0; i < 16; i++) \n"
"	{ \n"
"		float3 fDiff = vPosObject.xyz - vGravitrons[i].xyz; \n"
"		float fDist = length(fDiff); \n"
"		float fOffset = 1.f - smoothstep(0.f, 1.f, fDist / vGravitrons[i].w); \n"
"		vPosObject.y += fOffset; \n"
"	} \n"
*/
"	vPosProjCopy = vPosProj = mul(vPosObject, mProj); \n" // Projection
"	vPosView = mul(vPosObject, mView); \n" // Worldspace
"	vNormView.xyz = mul(vNormObject, (float3x3)mView); \n"
"	vNormView.w = 0.f; \n"
"} \n"
"vertexfragment Asura_D3D_VSF_Projection_Warped = compile_fragment vs_1_1 Projection(); \n";

Asura_Direct3D_VertexShaderFragment xASURA_D3D_VSF_PROJECTION_WARPED(   ASURA_D3D_VSF_PROJECTION_WARPED,
																		szASURA_D3D_VSF_PROJECTION_WARPED,
																		"Asura_D3D_VSF_Projection_Warped",
																		true);
// T&L shader fragments

// Environment ambient 
const char* const szASURA_D3D_VSF_TANDL_ENV_AMBIENT =
"void Ambient(  in  float4 vColourIn	: COLOR0, \n"
"				in  float4 vTexCoordIn	: TEXCOORD0, \n"
"				in  float3 vColourShift	: r_vColourShift, \n"
"				out float4 vColourOut	: COLOR0,  \n"
"				out float4 vTexCoordOut	: TEXCOORD0 \n"
"				) \n"
"{ \n"
"	vColourOut = vColourIn; \n"
"	vColourOut.xyz *= 1.f + vColourShift; \n"
"	vTexCoordOut = vTexCoordIn; \n"
"} \n"
"vertexfragment Asura_D3D_VSF_TandL_Env_Ambient = compile_fragment vs_1_1 Ambient(); \n";

// Register with Asura_Direct3D_Shaders
Asura_Direct3D_VertexShaderFragment xASURA_D3D_VSF_TANDL_ENV_AMBIENT(   ASURA_D3D_VSF_TANDL_ENV_AMBIENT,
																		szASURA_D3D_VSF_TANDL_ENV_AMBIENT,
																		"Asura_D3D_VSF_TandL_Env_Ambient",
																		false);

// Environment lighting
const char* const szASURA_D3D_VSF_TANDL_ENV_LIGHTING =
"float4x4 mTex0 : register(c8); \n"			// Texture matrix 0
"float4x4 mTex1 : register(c12); \n"		// Texture matrix 1
"float4 vLightPos : register(c16); \n"		// Light position
"float4 vLightColour : register(c17); \n"	// Light colour
"void Lighting( in  float4 vPosObject	: POSITION, \n"
"				in  float4 vNormal		: NORMAL, \n"
"				in  float4 vPosView		: r_vPosView, \n"
"				out float4 vOutColour	: COLOR0,  \n"
"				out float4 vTexCoord0	: TEXCOORD0, \n"
"				out float4 vTexCoord1	: TEXCOORD1 \n"
"				) \n"
"{ \n"
"	float3 vToLight = normalize(vLightPos.xyz - vPosObject.xyz); \n"
"	float fDotProd = dot(vToLight, vNormal); \n"
"	vOutColour = saturate(fDotProd * vLightColour + vLightColour); \n"
"	vTexCoord0 = mul(vPosView, mTex0); \n"
"	vTexCoord1 = mul(vPosView, mTex1); \n"
"} \n"
"vertexfragment Asura_D3D_VSF_TandL_Env_Lighting = compile_fragment vs_1_1 Lighting(); \n";

Asura_Direct3D_VertexShaderFragment xASURA_D3D_VSF_TANDL_ENV_LIGHTING(	ASURA_D3D_VSF_TANDL_ENV_LIGHTING,
																		szASURA_D3D_VSF_TANDL_ENV_LIGHTING,
																		"Asura_D3D_VSF_TandL_Env_Lighting",
																		false);

// Environment textures
const char* const szASURA_D3D_VSF_TANDL_ENV_TEXTURES =
"float4x4 mTex0 : register(c8); \n"			// Texture matrix 0
"float4x4 mTex1 : register(c12); \n"		// Texture matrix 1
"void Textures( in  float4 vColourIn	: COLOR0,		\n"
"				in  float4 vTexCoordIn0	: TEXCOORD0,	\n"
"				in  float4 vTexCoordIn1	: TEXCOORD1,	\n"
"				out float4 vColourOut	: COLOR0,		\n"
"				out float4 vTexCoordOut0: TEXCOORD0,	\n"
"				out float4 vTexCoordOut1: TEXCOORD1		\n"
"				) \n"
"{ \n"
"	vColourOut = vColourIn; \n"
"	vTexCoordOut0 = mul(vTexCoordIn0, mTex0); \n"
"	vTexCoordOut1 = mul(vTexCoordIn1, mTex1); \n"
"} \n"
"vertexfragment Asura_D3D_VSF_TandL_Env_Textures = compile_fragment vs_1_1 Textures(); \n";

Asura_Direct3D_VertexShaderFragment xASURA_D3D_VSF_TANDL_ENV_TEXTURES(	ASURA_D3D_VSF_TANDL_ENV_TEXTURES,
																		szASURA_D3D_VSF_TANDL_ENV_TEXTURES,
																		"Asura_D3D_VSF_TandL_Env_Textures",
																		false);

// Environment reflective
const char* const szASURA_D3D_VSF_TANDL_ENV_REFLECTIVE =
"float4x4 mProj		: register(c0); \n"
"float4x4 mView		: register(c4); \n"
"float4 c8			: register(c8); \n"
"float3x3 mReflect	: register(c9); \n"			// Reflection matrix
"vertexfragment Asura_D3D_VSF_TandL_Env_Reflective = \n"
"	asm_fragment \n"
"	{ \n"
"		vs_1_1 \n"
//"		m4x4 oPos, v_position0, c_mProj \n"					// Do the position
"		mov oD0, v_color0 \n"								// Diffuse
"		mov oT0, v_texcoord0 \n"							// Texcoord0
"		mov r_vEye, r_vPosView \n"
//"		m4x4 r_vEye, v_position0, c_mView \n"				// Texcoord1 is camera-space reflection vector - Calc eye vector into r0
"		dp3 r_vTemp, r_vEye, r_vEye \n"
"		rsq r_vTemp, r_vTemp \n"
"		mul r_vEye, r_vEye, -r_vTemp \n"
"		mov r_vTemp, r_vNormView \n"
//"		m3x3 r_vTemp, v_normal, c_mView \n"					// Calc normal vector into vTemp
"		dp3 r_vReflect.x, r_vEye.xyz, r_vTemp.xyz \n"		// Reflection vector is 2(E dot N)N - E
"		mul r_vReflect.x, r_vReflect.x, c_c8.x \n"
"		mul r_vReflect.xyz, r_vReflect.xxx, r_vTemp.xyz \n"
"		add r_vReflect.xyz, r_vReflect.xyz, -r_vEye.xyz \n"
"		m3x3 oT1, r_vReflect, c_mReflect \n"
"	}; \n";

Asura_Direct3D_VertexShaderFragment xASURA_D3D_VSF_TANDL_ENV_REFLECTIVE(	ASURA_D3D_VSF_TANDL_ENV_REFLECTIVE,
																			szASURA_D3D_VSF_TANDL_ENV_REFLECTIVE,
																			"Asura_D3D_VSF_TandL_Env_Reflective",
																			false);

// Environment shadows
const char* const szASURA_D3D_VSF_TANDL_ENV_SHADOWS =
"float4x4 mShadow	: register(c4); \n"	// Shadow matrix
"float4 c8			: register(c8); \n"	// -fFalloffPerMetre, 1.f + fFalloffPerMetre*fBaseHeight, -0.1f, initialAlpha
"void Shadows(  in  float4 vPosObj		: POSITION,		\n"
"				in  float4 vNormal		: NORMAL,		\n"
"				out float4 vOutColour	: COLOR0,		\n"
"				out float3 vTexCoord0	: TEXCOORD0,	\n"
"				out float3 vTexCoord1	: TEXCOORD1,	\n"
"				out float4 vTexCoord2	: TEXCOORD2,	\n"
"				out float4 vTexCoord3	: TEXCOORD3		\n"
"				) \n"
"{ \n"
"	if(vNormal.y < -0.1f) \n"
"	{ \n"
"		vOutColour.xyz = c8.www; \n"
"	} \n"
"	else \n"
"	{ \n"
"		vOutColour.xyz = 0.f; \n"
"	} \n"
"	vOutColour.w = 1.f; \n"
"	vTexCoord3 = vPosObj.yyyy * c8.xxxx + c8.yyyy; \n"
"	vTexCoord2 = mul(vPosObj, mShadow); \n"
//"	vTexCoord2.w = max(vTexCoord2.w, 0.f); \n"
"	vTexCoord0.xy = vTexCoord2.xy / vTexCoord2.w; \n"
"	vTexCoord1.xy = float2(1.f, 1.f) - vTexCoord0.xy; \n"
"	vTexCoord0.z = vTexCoord0.y; \n"
"	vTexCoord1.z = vTexCoord1.y; \n"
"} \n"
"vertexfragment Asura_D3D_VSF_TandL_Env_Shadows = compile_fragment vs_1_1 Shadows(); \n";

Asura_Direct3D_VertexShaderFragment xASURA_D3D_VSF_TANDL_ENV_SHADOWS(	ASURA_D3D_VSF_TANDL_ENV_SHADOWS,
																		szASURA_D3D_VSF_TANDL_ENV_SHADOWS,
																		"Asura_D3D_VSF_TandL_Env_Shadows",
																		false);
// Environment linear fog
const char* const szASURA_D3D_VSF_TANDL_ENV_LINEARFOG =
"float4 vWhite	: register(c4); \n"
"float4 vFog	: register(c5); \n"
"void LinearFog(in  float4 vPosProj		: r_vPosProj, \n"
"				out float4 vColourOut	: COLOR0  \n"
"				) \n"
"{ \n"
"	vColourOut = vWhite; \n"
"	vColourOut.w = clamp(vPosProj.w * vFog.x + vFog.y, vFog.z, vFog.w); \n"
"} \n"
"vertexfragment Asura_D3D_VSF_TandL_Env_LinearFog = compile_fragment vs_1_1 LinearFog(); \n";

Asura_Direct3D_VertexShaderFragment xASURA_D3D_VSF_TANDL_ENV_LINEARFOG(	ASURA_D3D_VSF_TANDL_ENV_LINEARFOG,
																		szASURA_D3D_VSF_TANDL_ENV_LINEARFOG,
																		"Asura_D3D_VSF_TandL_Env_LinearFog",
																		false);
// Environment graph fog
const char* const szASURA_D3D_VSF_TANDL_ENV_GRAPHFOG =
"float4 vWhite		: register(c4); \n"
"float4 vFog		: register(c5); \n"
"float4 vScale[2]	: register(c6); \n"
"float4 vSpline[2]	: register(c8); \n"
"void GraphFog(	in  float4 vPosProj		: r_vPosProj, \n"
"				out float4 vColourOut	: COLOR0  \n"
"				) \n"
"{ \n"
"	vColourOut = vWhite; \n"
"	float4 fDistance; \n"
"	fDistance.z = saturate(vPosProj.w * vFog.x + vFog.y); \n"
"	float fIndex; \n"
"	if(fDistance.z > vFog.z) \n"
"	{ \n"
"		fIndex = 1.f; \n"
"	} \n"
"	else \n"
"	{ \n"
"		fIndex = 0.f; \n"
"	} \n"
"	fDistance.z = fDistance.z * vScale[fIndex].x + vScale[fIndex].y; \n"
"	fDistance.w = vFog.w; \n"
"	fDistance.y = fDistance.z * fDistance.z; \n"
"	fDistance.x = fDistance.z * fDistance.y; \n"
"	vColourOut.w = dot(fDistance, vSpline[fIndex]); \n"
"} \n"
"vertexfragment Asura_D3D_VSF_TandL_Env_GraphFog = compile_fragment vs_1_1 GraphFog(); \n";

Asura_Direct3D_VertexShaderFragment xASURA_D3D_VSF_TANDL_ENV_GRAPHFOG(	ASURA_D3D_VSF_TANDL_ENV_GRAPHFOG,
																		szASURA_D3D_VSF_TANDL_ENV_GRAPHFOG,
																		"Asura_D3D_VSF_TandL_Env_GraphFog",
																		false);
