#include "Bot_Shared.fxh"

//------------------------------------
struct VS_Input
{
	// DLH - 23 October 2006 15:11:09 - Mesh Data
	float4	xPosition					: POSITION0;
	float3	xNormal						: NORMAL0;
	float3	xTangent					: TANGENT0;
	float4	xBinormal					: BINORMAL0;
	float4	axUV0UV1					: TEXCOORD0;
    float4 	xDiffuse					: COLOR0;
    float2  xCorner						: TEXCOORD1;

	// DLH - 23 October 2006 15:11:20 - Instance Data
	float4	xTransform0					: POSITION1;
	float4	xTransform1					: POSITION2;
	float4	xTransform2					: POSITION3;
    float4 	xRedSH						: COLOR1;
    float4 	xGreenSH					: COLOR2;
    float4 	xBlueSH						: COLOR3;
	float4	xBone						: POSITION4;
};

//------------------------------------
float GetInstanceScale( float4 xTransform0,
							   float4 xTransform1,
							   float4 xTransform2)
{
	return( length( xTransform0.xyz));
}

shared 	float3	g_xCameraRightVector = float3( 1, 0, 0);
shared 	float3	g_xCameraUpVector = float3( 0, 1, 0);
shared 	float3	g_xCameraForwardsVector = float3( 0, 0, 1);

VS_Output VS_Bot_Billboard_Prelighting_Pass (VS_Input IN)
{
	VS_Output xOut;

    // Calculate the vertex position...
    float4 xPositionObject = IN.xPosition;
    {
		float fBoneWeight = IN.xBinormal.w;
		//float fBoneWeight = 1.0f;
		xPositionObject.xyz = Bot_CalcNewBonedPosition( xPositionObject.xyz, fBoneWeight, IN.xBone.xyz);
    }
     
    float4 xPositionWorld = xPositionObject;
    
    xPositionWorld.x = dot( xPositionObject.xyzw, IN.xTransform0.xyzw);
    xPositionWorld.y = dot( xPositionObject.xyzw, IN.xTransform1.xyzw);
    xPositionWorld.z = dot( xPositionObject.xyzw, IN.xTransform2.xyzw);
    
	float fInstanceScale = GetInstanceScale( IN.xTransform0, IN.xTransform1, IN.xTransform2);
	float2 xCorner = fInstanceScale * IN.xCorner.xy;
	xPositionWorld.xyz += g_xCameraRightVector * xCorner.x;
	xPositionWorld.xyz += g_xCameraUpVector * xCorner.y;

	xOut.m_xPosition = mul( xPositionWorld, g_xViewProjectionMatrix);
    
	xOut.m_xColour = IN.xDiffuse;

    // Calculate the vertex texture-coordinates...
   	xOut.m_xUVPair = IN.axUV0UV1;
   	
   	float3 xTangentNormal = g_xCameraForwardsVector;
   	float3 xWorldNormal = g_xCameraForwardsVector;
   	float3 xWorldTangent = RotateVectorIntoWorldSpace( IN.xTangent.xyz, g_xCameraRightVector, g_xCameraUpVector, g_xCameraForwardsVector);
   	float3 xWorldBinormal = RotateVectorIntoWorldSpace( IN.xBinormal.xyz, g_xCameraRightVector, g_xCameraUpVector, g_xCameraForwardsVector);
   	
	
	// Light it - from front and back, then lerp
#if defined( ASURA_TOOLS)
	if (g_bDisableLighting )
	{
		xOut.m_xColour.rgb = 0.5f;
		xOut.m_xBackColour.rgb = 0.5f;
	}
	else
#endif	// defined( ASURA_TOOLS)
	{	
		half3 xColour, xBackColour;		
		half4 xNormal = half4(xWorldNormal,1);
		
		xColour.r	= dot (xNormal, IN.xRedSH);
		xColour.g	= dot (xNormal, IN.xGreenSH);
		xColour.b	= dot (xNormal, IN.xBlueSH);	
		xOut.m_xColour.rgb = max (xColour.rgb, 0);	
			
		xNormal.rgb=-xNormal.rgb;
		xBackColour.r	= dot (xNormal, IN.xRedSH);
		xBackColour.g	= dot (xNormal, IN.xGreenSH);
		xBackColour.b	= dot (xNormal, IN.xBlueSH);		
		xOut.m_xBackColour.rgb = max (xBackColour.rgb, 0);		
	}

	xOut.m_afAlphaCut = IN.xBone.w;

	xOut.m_xTangent = xWorldTangent;
	xOut.m_xBinormal = xWorldBinormal;
	xOut.m_xNormal = xWorldNormal;

	xOut.m_xCameraDirDistance.xyz = g_xCameraPosition.xyz - xPositionWorld;
	xOut.m_xCameraDirDistance.w = xOut.m_xPosition.w;

	return xOut;
}



//-----------------------------------
technique Bot_Billboard_Prelighting_Pass
{
    pass p0
    {
		VertexShader	= compile vs_3_0 VS_Bot_Billboard_Prelighting_Pass();
		PixelShader		= compile ps_3_0 PS_Bot_3D_Prelighting_Pass();
    }
}
 