#include "Shared.fxh"
#include "MaterialResponse_Shared.fxh"

float   g_fDualSidedLightingLerp = 0.35f;

float3 Bot_CalcNewBonedPosition( float3 xOldPosition, float fBoneWeight, float3 xBone)
{
#if 0
	float3 xNewPosition = xOldPosition + (fBoneWeight * xBone);
#endif
	
#if 1
	float fOldPositionLength = sqrt( dot( xOldPosition.xyz, xOldPosition.xyz));
	float3 xBonedPosition = xOldPosition + (fBoneWeight * xBone);
	float3 xBoneDirection = normalize( xBonedPosition);
	float3 xNewPosition = xBoneDirection * fOldPositionLength;
#endif
	
	return( xNewPosition);
}

float3 RotateVectorIntoWorldSpace (float3 xVector,
							   float3 xTransform0,
							   float3 xTransform1,
							   float3 xTransform2)
{
	float3	xRotatedVector;

	// Rotate the x, y and z components into tangent-space...
	xRotatedVector.x	= dot(xVector, xTransform0);
	xRotatedVector.y	= dot(xVector, xTransform1);
	xRotatedVector.z	= dot(xVector, xTransform2);
	
	xRotatedVector = normalize( xRotatedVector);

	// Return the rotated vector...
	return	(xRotatedVector);
}

struct VS_Output
{
	float4 m_xPosition          : POSITION;
	float4 m_xColour            : COLOR;
	float3 m_xBackColour        : TEXCOORD0;

	float4 m_xUVPair            : TEXCOORD1;
	
	float4 m_xCameraDirDistance : TEXCOORD2; // Camera direction, and distance from view plane in w
	
	float4 m_afAlphaCut			: TEXCOORD3;

	float3 m_xTangent           : TANGENT;
	float3 m_xBinormal          : BINORMAL;
	float3 m_xNormal            : NORMAL;
};

struct PS_Input
{
	float4 m_xColour            : COLOR;
	float3 m_xBackColour        : TEXCOORD0;
	
	float4 m_xUVPair            : TEXCOORD1;
	float4 m_xCameraDirDistance : TEXCOORD2; // Camera direction, and distance from view plane in w
	float4 m_afAlphaCut			: TEXCOORD3;
	
	float3 m_xTangent           : TANGENT;
	float3 m_xBinormal          : BINORMAL;
	float3 m_xNormal            : NORMAL;
	
	float	fFace						: VFACE;
};


//------------------------------------
DeferredOutputPixel PS_Bot_3D_Prelighting_Pass (PS_Input xIn)
{
	const float3x3 xTangentToWorldMatrix = float3x3( normalize( xIn.m_xTangent ),
													 normalize( xIn.m_xBinormal ),
													 normalize( xIn.m_xNormal ) );

	const float4 xUVPair = MaterialResponse_AnimateUVs( xIn.m_xUVPair );

	//const float4 xTangentSpaceNormal = NormalLookup( xUVPair, xIn.m_xColour.a );
	float4 xWorldSpaceNormal = NormalLookup( xUVPair, 0, xTangentToWorldMatrix, xTangentToWorldMatrix);


	half4 xColour;		
	if( 0.0f < xIn.fFace)
	{
		xWorldSpaceNormal.xyz = -xWorldSpaceNormal.xyz;
		xColour.rgb = lerp(xIn.m_xBackColour.rgb, xIn.m_xColour.rgb, g_fDualSidedLightingLerp);
	}
	else
	{
		xColour.rgb = lerp(xIn.m_xColour.rgb, xIn.m_xBackColour.rgb, g_fDualSidedLightingLerp);
	}
	
	const float fDualTextureLerp = xIn.m_xColour.a;

	
										
	// -----------------------------------------------------------------
	// Diffuse-only SH prelighting 
	// -----------------------------------------------------------------	
	const float fMaterialsTextureY = g_xMaterialProperties.x;
	half3 xAlbedo;
	const float3 xAlbedoTimes2 = AlbedoLookup( xUVPair,
											   0.0,
											   xWorldSpaceNormal.xyz,
											   normalize( xIn.m_xCameraDirDistance.xyz ),
											   xAlbedo );

//	xColour.rgb =  xIn.m_xColour.rgb;
				
	xColour.rgb = xColour.rgb* xAlbedoTimes2 + SelfIlluminationLookup( xUVPair, 0.0, xAlbedo );
	
	xColour.a = TransparencyLookup( xUVPair, 0.0 );
		
	xColour.a = xColour.a - xIn.m_afAlphaCut.x;

	half hSpecular = SpecularLookup( xUVPair, 0.0 );		
	
	DeferredOutputPixel xOut = Deferred_FillOutputPixel(
										xColour,
										xWorldSpaceNormal.xyz, 
										xIn.m_xCameraDirDistance.w,
										xAlbedo,
										hSpecular,
										fMaterialsTextureY
										);										
	return( xOut);
}

