
// Coronas.fx - Corona shader

#include "Shared.fxh"


const float4  g_xOneOverScreenDimensions;

const texture g_xTexture;
const sampler g_xTextureSampler : register(s0) = sampler_state
{
	Texture = <g_xTexture>;
	AddressU = CLAMP;
	AddressV = CLAMP;
	sRGBTexture = <g_bUseGammaCorrection>;
};

const texture g_xDepthTexture;
const sampler g_xDepthTextureSampler : register(s1) = sampler_state
{
	Texture = <g_xDepthTexture>;
	AddressU = CLAMP;
	AddressV = CLAMP;
	MinFilter = POINT;		// bilinear doesn't work on FP16 textures? getting weird glitches
	MagFilter = POINT;
};



//------------------------------------
struct VS_Input
{
	float4	xPosition					: POSITION0;
	float3	xNormal						: NORMAL;
	float4	xDiffuse					: COLOR0;
	float4	xUV							: TEXCOORD0;    
};

struct VS_Output
{
    float4	xPosition					: POSITION0;    
	float4	xUV							: TEXCOORD0;
	float4 	xDiffuse					: TEXCOORD1;    
};

//------------------------------------
VS_Output VS_Coronas (VS_Input IN)
{
	VS_Output	OUT;

    // Calculate the vertex position...
    OUT.xPosition	= mul(IN.xPosition, g_xViewProjectionMatrix);    
    OUT.xUV         = IN.xUV;
    OUT.xDiffuse    = IN.xDiffuse;

	// Packing our softness values into the UV interpolator
	// xUV.z already contains fSize * fVolumetricDepthScale	
	
	OUT.xUV.w = OUT.xPosition.w;		// depth

	return OUT;
}

//------------------------------------
float4 PS_Coronas (VS_Output IN) : COLOR
{
	// Bog standard corona
	float4 xColour = tex2D(g_xTextureSampler, IN.xUV.xy); 	
	return IN.xDiffuse * xColour;
}


float4 PS_Coronas_Volumetric (const VS_Output IN, const float2 xScreenPos : VPOS ) : COLOR
{
	// Soft/volumetric corona
	
	float4 xColour = tex2D(g_xTextureSampler, IN.xUV.xy); 	
	
	const float2 xNormalizedCoords = (xScreenPos.xy + float2(0.5, 0.5)) * g_xOneOverScreenDimensions.xy;
	const float4 xDepth = tex2D(g_xDepthTextureSampler, xNormalizedCoords);
		
	float fDepth = abs(xDepth.z);
					
	float fSoftValue = min(abs(fDepth-IN.xUV.w)/(IN.xUV.z),1.0f);
	xColour.a *= fSoftValue;
		
	return IN.xDiffuse * xColour;	
}


//-----------------------------------
technique Coronas
{
    pass p0
    {		
		// Pass 0 - Standard
		VertexShader	= compile vs_3_0 VS_Coronas();
		PixelShader		= compile ps_3_0 PS_Coronas();
    }
    
    pass p1
    {		
		// Pass 1 - Soft
		VertexShader	= compile vs_3_0 VS_Coronas();
		PixelShader		= compile ps_3_0 PS_Coronas_Volumetric();
    }
}
