/******************************************************************************/
/* Parameters                                                                 */
/******************************************************************************/

// Hack to replace albedo buffer alpha channel output (which usually contains
// material index) with transparency value, causing colour to blend into albedo.
// Needed for decal-type polygons on walls, which come before deferred lights

#if defined( NOT_EFFECT )

	#if !defined( BLEND_INTO_ALBEDO )
		#define BLEND_INTO_ALBEDO false
	#endif

	static bool g_bBlendIntoAlbedo = BLEND_INTO_ALBEDO;

#else // !defined( NOT_EFFECT )

	bool g_bBlendIntoAlbedo = false;

#endif // !defined( NOT_EFFECT )

/******************************************************************************/
/* Types                                                                      */
/******************************************************************************/

struct DeferredOutputPixel
{
	half4 m_xBufferColour         : COLOR0;
	half4 m_xBufferNormalDistance : COLOR1;
	half4 m_xBufferAlbedo         : COLOR2;
};

/******************************************************************************/
/* Functions                                                                  */
/******************************************************************************/

DeferredOutputPixel Deferred_FillOutputPixel( half4 xRGBA, float3 xNormal, float fDistance, half3 xAlbedo, half hSpecular, float fMaterial)
{
	DeferredOutputPixel xOut;

#if defined( ASURA_TOOLS )
	if ( g_bShowPerPixelNormals == true )
	{
		xOut.m_xBufferColour.rgb = ( xNormal * 0.5f ) + float3( 0.5f, 0.5f, 0.5f );
		xOut.m_xBufferColour.a = xRGBA.a;
		xOut.m_xBufferAlbedo.rgb = xOut.m_xBufferColour.rgb;
	}
	else
#endif // #if defined( ASURA_TOOLS )
	{
		xOut.m_xBufferColour = xRGBA;
		xOut.m_xBufferAlbedo.rgb = xAlbedo;
	}

	xOut.m_xBufferNormalDistance.xy = xNormal.xy;
	xOut.m_xBufferNormalDistance.z = (xNormal.z < 0) ? -fDistance : fDistance;
	xOut.m_xBufferNormalDistance.w = hSpecular;

	// For transparent materials, we need to blend into the albedo, and this is the only way to do it.
	// (writes to this channel must be masked out!)

	xOut.m_xBufferAlbedo.a = g_bBlendIntoAlbedo ? xRGBA.a : fMaterial;

	return xOut;
}

void Deferred_GetNormalDepthAlbedoSpecularMaterial( half4 xBufferNormal, half4 xBufferAlbedo, out half3 xOutputNormal, out float fOutputDistance, out half3 xOutputAlbedo, out half hOutputSpecular, out float fMaterial)
{
	xOutputNormal.xy = xBufferNormal.xy;
	// DLH - Reconstruct the Z of the normal from a sphere equation and the sign of the distance
	xOutputNormal.z = sqrt( 1.0 - dot(xOutputNormal.xy, xOutputNormal.xy));
	xOutputNormal.z = (0 < xBufferNormal.z) ? xOutputNormal.z : -xOutputNormal.z;

	fOutputDistance = abs( xBufferNormal.z);
	xOutputAlbedo = xBufferAlbedo.xyz;
	hOutputSpecular = xBufferNormal.w;
	
	fMaterial = xBufferAlbedo.w;
}

void Deferred_GetNormalDepthAlbedoSpecular( half4 xBufferNormal, half4 xBufferAlbedo, out half3 xOutputNormal, out float fOutputDistance, out half3 xOutputAlbedo, out half hOutputSpecular)
{
	xOutputNormal.xy = xBufferNormal.xy;
	// DLH - Reconstruct the Z of the normal from a sphere equation and the sign of the distance
	xOutputNormal.z = sqrt( 1.0 - dot(xOutputNormal.xy, xOutputNormal.xy));
	xOutputNormal.z = (0 < xBufferNormal.z) ? xOutputNormal.z : -xOutputNormal.z;

	fOutputDistance = abs( xBufferNormal.z);
	xOutputAlbedo = xBufferAlbedo.xyz;
	hOutputSpecular = xBufferNormal.w;
}

void Deferred_GetNormalDepth( half4 xBufferNormal, out half3 xOutputNormal, out float fOutputDistance)
{
	xOutputNormal.xy = xBufferNormal.xy;
	// DLH - Reconstruct the Z of the normal from a sphere equation and the sign of the distance
	xOutputNormal.z = sqrt( 1.0 - dot(xOutputNormal.xy, xOutputNormal.xy));
	xOutputNormal.z = (0 < xBufferNormal.z) ? xOutputNormal.z : -xOutputNormal.z;

	fOutputDistance = abs( xBufferNormal.z);
}




