/******************************************************************************/
/* Included Files                                                             */
/******************************************************************************/

#include "Shared.fxh"

/******************************************************************************/
/* Parameters                                                                 */
/******************************************************************************/

const texture g_xInitialImageTexture;
const sampler2D g_xInitialImage = sampler_state
{
	Texture = <g_xInitialImageTexture>;
	MinFilter = POINT;
	MagFilter = POINT;
	MipFilter = NONE;
	AddressU = CLAMP;
	AddressV = CLAMP;
	sRGBTexture = <g_bUseGammaCorrection>;
};

const texture g_xColourMapTexture;
const sampler1D g_xColourMap = sampler_state
{
	Texture = <g_xColourMapTexture>;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
	MipFilter = NONE;
	AddressU = CLAMP;
	sRGBTexture = <g_bUseGammaCorrection>;
};

const float g_fIntensity;

/******************************************************************************/
/* Constants                                                                  */
/******************************************************************************/

// Luminance = 0.27r + 0.67g + 0.06b
static float3 xLUMINANCE_CONSTANTS = { 0.27f, 0.67f, 0.06f };

/******************************************************************************/
/* Types                                                                      */
/******************************************************************************/

struct Vertex
{
	float4 m_xPosition : POSITION;
	float2 m_xUV       : TEXCOORD;
};

/******************************************************************************/
/* Functions                                                                  */
/******************************************************************************/

Vertex VShader( const Vertex xIn )
{
	return xIn;
}

float4 PShader( const Vertex xIn ) : COLOR
{
	const half3 xInitialColour = tex2D( g_xInitialImage, xIn.m_xUV ).rgb;

	// No need to saturate here, use clamp addressing
	const half hLuminance = dot( xInitialColour.rgb, xLUMINANCE_CONSTANTS );

	half3 xFinalColour = tex1D( g_xColourMap, hLuminance ).rgb;
	xFinalColour = lerp( xInitialColour, xFinalColour, g_fIntensity );

	return half4( xFinalColour, 1.0h );
}

/******************************************************************************/
/* Techniques                                                                 */
/******************************************************************************/

technique
{
	pass
	{
		VertexShader = compile vs_3_0 VShader();
		PixelShader = compile ps_3_0 PShader();
	}
}