/******************************************************************************/
/* Parameters                                                                 */
/******************************************************************************/

const texture g_xBlurredTexture;
const sampler g_xBlurredTextureSampler : register(s0) = sampler_state
{
	Texture = <g_xBlurredTexture>;
	AddressU	= CLAMP;
	AddressV	= CLAMP;
   	MINFILTER	= LINEAR;
   	MAGFILTER	= LINEAR;

};

const texture g_xDepthTexture;
const sampler g_xDepthTextureSampler : register(s1) = sampler_state
{
	Texture = <g_xDepthTexture>;
	AddressU = CLAMP;
	AddressV = CLAMP;
	MinFilter = POINT;		
	MagFilter = POINT;
};
			  
const float4  g_xOneOverScreenDimensions;
const float4  g_xFocalDistances;

/******************************************************************************/
/* Types                                                                      */
/******************************************************************************/

struct InputVertex
{
	float2 m_xPosition   : POSITION;
	float2 m_xUV	     : TEXCOORD0;
};

struct OutputVertex
{
	float4 m_xPosition   : POSITION;
	float2 m_xUV	     : TEXCOORD0;
};

/******************************************************************************/
/* Functions                                                                  */
/******************************************************************************/

OutputVertex VShader( const InputVertex xIn)
{
	OutputVertex xOut;

	xOut.m_xPosition = float4(xIn.m_xPosition.x,xIn.m_xPosition.y,0,1);
	xOut.m_xUV = xIn.m_xUV;
	
	return xOut;
}


			
					
float4 PShader( const OutputVertex xIn,  const float2 xScreenPos : VPOS ) : COLOR
{
	float4 xCol;
	xCol = tex2D(g_xBlurredTextureSampler, xIn.m_xUV);	
	
	const float2 xNormalizedCoords = (xScreenPos.xy + float2(0.5, 0.5)) * g_xOneOverScreenDimensions.xy;	
	float fDepth = abs(tex2D(g_xDepthTextureSampler, xNormalizedCoords).z);
	
	float fBlurNear = saturate(1.0 - (fDepth-g_xFocalDistances.x) / (g_xFocalDistances.y-g_xFocalDistances.x));
	float fBlurFar = saturate((fDepth - g_xFocalDistances.z) / (g_xFocalDistances.w-g_xFocalDistances.z));
	
	xCol.a = saturate(max(max(fBlurNear,xCol.a), fBlurFar)); 	
	xCol.a = xCol.a * g_xOneOverScreenDimensions.z;

	return saturate(xCol);
}


/******************************************************************************/
/* Techniques                                                                 */
/******************************************************************************/

technique
{
	pass
	{
		AlphaTestEnable = FALSE;
		CullMode = NONE;
		ZEnable = FALSE;
		ZWriteEnable = FALSE;

		VertexShader = compile vs_3_0 VShader();
		PixelShader = compile ps_3_0 PShader();
	}
}