/******************************************************************************/
/* Included Files                                                             */
/******************************************************************************/

#include "Shared.fxh"

/******************************************************************************/
/* Parameters                                                                 */
/******************************************************************************/

texture g_xTexture;
sampler g_xTextureSampler = sampler_state
{
	Texture = <g_xTexture>;
	AddressU = WRAP;
	AddressV = WRAP;
	sRGBTexture = <g_bUseGammaCorrection>;
};

float4 g_xScale;						// .xy - scaleX, scaleY; .zw - 1/scaleX, 1/scaleY
float g_fFOV;
float g_fMult;

float g_fOneOverTanFOV;
float g_fSinHalfFOV;
float g_fTanHalfFOV;

static float g_fIntensity = 1.0;		// since it's currently always 1, make it static


int g_iFishEyeMode = 1;


/******************************************************************************/
/* Types                                                                      */
/******************************************************************************/
struct Vertex
{
	float4 m_xPosition : POSITION;
	float2 m_xUV       : TEXCOORD;
};

/******************************************************************************/
/* Functions                                                                  */
/******************************************************************************/

Vertex VShader( const Vertex xIn )
{
	return xIn;
}

float FishEyeTransform(float fInput, uniform int iFishEyeMode)
{
	if (iFishEyeMode == 0)
	{
	    // Gnomonical, // perspective correct, r = tan(Theta) / tan(m_fFov)
	    return fInput;
	} 
	else if (iFishEyeMode == 1)
	{
		// Linear, r = Theta / fov
		float fTheta = fInput * g_fFOV;
		return tan(fTheta) * g_fOneOverTanFOV;
	} 
	else if (iFishEyeMode == 2)
	{
		// EqualArea, // r = 2 . sin(Theta / 2) / tan(m_fFov)
		float fTheta = 2 * asin(fInput * g_fSinHalfFOV);
		return tan(fTheta) * g_fOneOverTanFOV;
	} 
	else 
	{
		// Stereographic, // r = 2 . tan(Theta / 2) / tan(m_fFov)
		float fTheta = 2 * atan(fInput * g_fTanHalfFOV);
		return tan(fTheta) * g_fOneOverTanFOV;
	}
}

float4 PShader( const Vertex xIn, uniform int iFishEyeMode ) : COLOR
{
	const float2 InputVec = xIn.m_xUV * g_xScale.xy;
	
	float fInputVecLength = length(InputVec);
	
	float fOutputLength = FishEyeTransform(fInputVecLength, iFishEyeMode);
	
	float2 xUV = InputVec * (fOutputLength / fInputVecLength);
	
	xUV = lerp(InputVec, xUV, g_fMult) * g_xScale.zw;
	xUV = xUV * 0.5f * sqrt(2)  + 0.5f;					// this sqrt gets compiled out
	
	return float4(tex2D( g_xTextureSampler, xUV ).rgb, g_fIntensity);
}

/******************************************************************************/
/* Techniques                                                                 */
/******************************************************************************/

technique
{
	pass p0
	{
		VertexShader = compile vs_3_0 VShader();
		PixelShader = compile ps_3_0 PShader(0);
	}
	pass p1
	{
		VertexShader = compile vs_3_0 VShader();
		PixelShader = compile ps_3_0 PShader(1);
	}
	pass p2
	{
		VertexShader = compile vs_3_0 VShader();
		PixelShader = compile ps_3_0 PShader(2);
	}
	pass p3
	{
		VertexShader = compile vs_3_0 VShader();
		PixelShader = compile ps_3_0 PShader(3);
	}
}