/******************************************************************************/
/* Parameters                                                                 */
/******************************************************************************/

//#include "Shared.fxh"

texture g_xTexture;
sampler g_xTextureSampler : register(s0) = sampler_state
{
	Texture = <g_xTexture>;
	AddressU = CLAMP;
	AddressV = CLAMP;
   	MINFILTER	= LINEAR;
   	MAGFILTER	= LINEAR;
//	sRGBTexture = <g_bUseGammaCorrection>;
};

const texture g_xDepthTexture;
const sampler g_xDepthTextureSampler : register(s1) = sampler_state
{
	Texture = <g_xDepthTexture>;
	AddressU = CLAMP;
	AddressV = CLAMP;
	MinFilter = POINT;		
	MagFilter = POINT;
};
			  
const float4  g_xOneOverScreenDimensions;

const float4x4 g_xInvViewMatrix;
const float4x4 g_xViewProjMatrix;
const float4x4 g_xPrevViewProjMatrix;

const float    g_fTwoOverScreenWidth;						// Dimensions of screen, in pixels. Used to convert VPOS pixel shader input into normalised coordinates.
const float    g_fTwoOverScreenHeight;
const float    g_fCameraTanHalfHorizFOV;					// Slopes of camera frustum. Used to reconstruct viewspace positions from screen coordinates and depth.
const float    g_fCameraTanHalfVertFOV;

const float    g_fIntensity;

/******************************************************************************/
/* Types                                                                      */
/******************************************************************************/

struct InputVertex
{
	float2 m_xPosition   : POSITION;
	float2 m_xUV	     : TEXCOORD0;
};

struct OutputVertex
{
	float4 m_xPosition   : POSITION;
	float2 m_xUV	     : TEXCOORD0;
};

/******************************************************************************/
/* Functions                                                                  */
/******************************************************************************/

OutputVertex VShader( const InputVertex xIn)
{
	OutputVertex xOut;

	xOut.m_xPosition = float4(xIn.m_xPosition.x,xIn.m_xPosition.y,0,1);
	xOut.m_xUV = xIn.m_xUV;
	
	return xOut;
}

float4 ScreenToWorldPosition( float2 xDepthTexturePixelPos, float fDepth )
{
	const float4 xViewSpacePosition = float4(	g_fCameraTanHalfHorizFOV * fDepth * (xDepthTexturePixelPos.x * g_fTwoOverScreenWidth  - 1.0),
												g_fCameraTanHalfVertFOV  * fDepth * -(xDepthTexturePixelPos.y * g_fTwoOverScreenHeight - 1.0),
												fDepth, 1.0);
	
	float4 xWorldPosition = mul( xViewSpacePosition, g_xInvViewMatrix );
	return xWorldPosition;
}

float4 SampleMotionBlur( const float2 xScreenPosUV, const float2 xVelocity )
{
	const int iSteps = 16;
	const float fOneStep = 1.0 / (float)iSteps;

	float4 xColor = float4( 0.0, 0.0, 0.0, 0.0 );
	float fSum = 0.0f;

	[unroll]
	for( int i = 0; i < iSteps; i++ )
	{
		float4 xPixel = tex2D( g_xTextureSampler, xScreenPosUV + xVelocity * (fOneStep * float(i)) );

		const float fWeight = 0.1f + (float)(iSteps - i) / (float)iSteps;
		
		fSum += fWeight;
		xColor += xPixel * fWeight;
	}

	return xColor / fSum;
}
					
float4 PShader( const OutputVertex xIn,  const float2 xScreenPos : VPOS ) : COLOR
{
	float2 xScreenPosUV = (xScreenPos + float2( 0.5, 0.5 )) * g_xOneOverScreenDimensions.xy;

	float4 xCol;
	
	//xCol = tex2D( g_xTextureSampler, xScreenPosUV );
	//xCol.r = 0.5;
	//return xCol;

	const float2 xNormalizedCoords = (xScreenPos.xy + float2(0.5, 0.5)) * g_xOneOverScreenDimensions.xy;	
	float fDepth = abs(tex2D(g_xDepthTextureSampler, xNormalizedCoords).z);
	
 	float4 xPosition = ScreenToWorldPosition( xScreenPos + float2( 0.5, 0.5 ), fDepth );
	xPosition = mul( xPosition, g_xPrevViewProjMatrix );
	xPosition /= xPosition.w;

	float2 xPrevScreen = float2(xPosition.x * 0.5 + 0.5, -xPosition.y * 0.5 + 0.5);
	
	float2 xDiff = xScreenPosUV - xPrevScreen;
	
	xCol.xyz = SampleMotionBlur( xScreenPosUV, xDiff );

	xCol.a = g_fIntensity;

	return xCol;
}


/******************************************************************************/
/* Techniques                                                                 */
/******************************************************************************/

technique
{
	pass
	{
		AlphaTestEnable = FALSE;
		CullMode = NONE;
		ZEnable = FALSE;
		ZWriteEnable = FALSE;

		VertexShader = compile vs_3_0 VShader();
		PixelShader = compile ps_3_0 PShader();
	}
}