/******************************************************************************/
/* Parameters                                                                 */
/******************************************************************************/

#include "Shared.fxh"
#include "Deferred_Shared.fxh"

const float3 g_xFogColour;

const float g_fFogNearPlane;
const float g_fOneOverFogVolumeDepth; // = 1 / (fog far plane - fog near plane)

const float g_fFarClipPlane;

const float2 g_fDepthTextureDimensions;

const texture g_xDepthTexture;

const sampler g_xDepthTextureSampler : register(s0) = sampler_state
{
	Texture = <g_xDepthTexture>;
	MagFilter = POINT;
	MinFilter = POINT;
	MipFilter = POINT;
	AddressU = CLAMP;
	AddressV = CLAMP;
};


const texture g_xFogTexture;

const sampler g_xFogTextureSampler : register(s1) = sampler_state
{
	Texture = <g_xFogTexture>;
	MagFilter = LINEAR;
	MinFilter = LINEAR;
	MipFilter = POINT;
	AddressU = CLAMP;
	AddressV = CLAMP;
};

/******************************************************************************/
/* Functions                                                                  */
/******************************************************************************/


float4 VShader( const float4 xIn : POSITION ) : POSITION
{
	return xIn;
}

float4 PShader( const float2 xScreenPos : VPOS ) : COLOR
{
	float4 xBufferNormal = tex2D(g_xDepthTextureSampler, (xScreenPos + float2(0.5, 0.5)) / g_fDepthTextureDimensions);
	half3 xNormal;
	float fScreenDepth;
	
	Deferred_GetNormalDepth( xBufferNormal, xNormal, fScreenDepth);
	float fDepth = fScreenDepth;

	// Kill pixel if the distance is beyond the far clip plane (assume it's skybox)
	clip( g_fFarClipPlane - fDepth );

	// Look up fog value from texture
	float2 fLookup;
	fLookup.x = (fDepth - g_fFogNearPlane) * g_fOneOverFogVolumeDepth;
	fLookup.y = 0;
	
	float4 fFogTextureColour = tex2D(g_xFogTextureSampler, fLookup);
	
	return float4( g_xFogColour, fFogTextureColour.a );	
}

/******************************************************************************/
/* Techniques                                                                 */
/******************************************************************************/

technique
{
	pass
	{
		VertexShader = compile vs_3_0 VShader();
		PixelShader = compile ps_3_0 PShader();
	}
}