// This is intended for Tools only, to render the front face of a cubemap
// through the Asura overlay quad system

/******************************************************************************/
/* Parameters                                                                 */
/******************************************************************************/

const texture g_xTexture;
const samplerCUBE g_xCubemapSampler = sampler_state
{
	Texture = <g_xTexture>;
};

/******************************************************************************/
/* Types                                                                      */
/******************************************************************************/

struct Vertex
{
	float4 m_xPosition  : POSITION;
	float4 m_xColour    : COLOR;
	float2 m_xTexcoords : TEXCOORD;
};

/******************************************************************************/
/* Functions                                                                  */
/******************************************************************************/

Vertex VShader( const Vertex xIn )
{
	// Would prefer to not have a vertex shader and use transformed input
	// vertices, but that's not the way the Asura overlay quad system will pass
	// the input coordinates

	Vertex xOut = xIn;

	// Touch up the z and w values
	xOut.m_xPosition.z = 0.5f;
	xOut.m_xPosition.w = 1.0f;

	return xOut;
}

float4 PShader( const Vertex xIn ) : COLOR
{
	// Only 2D texcoords come through from the overlay quad system. Set the z-
	// coord to -1 to address the front face of the cubemap. We assume the input
	// coords run from [-1,-1] to [1,1]

	const float3 xDirection = normalize( float3( xIn.m_xTexcoords.xy, -1.0f ) );
	return texCUBE( g_xCubemapSampler, xDirection ) * xIn.m_xColour;
}

/******************************************************************************/
/* Techniques                                                                 */
/******************************************************************************/

technique
{
	pass
	{
		VertexShader = compile vs_3_0 VShader();
		PixelShader = compile ps_3_0 PShader();
	}
}