// Reflective functions
shared	bool	g_bReflectiveTexture = false;
shared float	g_fMaterialReflectivity = 1.0f; // If 1, the only adjustments are by alpha in the textures

// Assumes our sampler is a sphere map
float3 CalculateReflectiveColour_SphereMap(float3 xFromEyeDir, float3 xNormal, float4 xTextureColour, sampler xSampler)
{
	// Get our reflection vector
	float3 xReflect = reflect(xFromEyeDir, xNormal);
	// Map to the sphere
	float2 xCoords;
	xCoords.x = (xReflect.x + 1) * 0.5f;
	xCoords.y = (1 - xReflect.y) * 0.5f;
	// Get the reflection colour
	float4 xReflectContrib = tex2D(xSampler, xCoords);
	// Multiply our colour by the main texture alpha and the reflectivity of the material
	xReflectContrib.rgb *= xTextureColour.a * g_fMaterialReflectivity;
	// Return our reflective colour
	return xReflectContrib.rgb/* * xTextureColour.rgb*/; // This should really use the material's specular colour too.
}

// Assumes our sampler is a cube map
float3 CalculateReflectiveColour_CubeMap(float3 xFromEyeDir, float3 xNormal, float4 xTextureColour, sampler xSampler)
{
	// Get our reflection vector
	float3 xReflect = reflect(xFromEyeDir, xNormal);
	// Get the reflection colour
	float4 xReflectContrib = texCUBE(xSampler, xReflect);
	// Multiply our colour by the main texture alpha and the reflectivity of the material
	xReflectContrib.rgb *= xTextureColour.a * g_fMaterialReflectivity;
	// Return our reflective colour
	return xReflectContrib.rgb/* * xTextureColour.rgb*/; // This should really use the material's specular colour too.
}

