/******************************************************************************/
/* Parameters                                                                 */
/******************************************************************************/

float2   g_xShadowFrustum : register(c2); // (x = near plane, y = 1 / (far plane - near plane)
float4x4 g_xWorldViewProj : register(c6);
row_major float3x4 g_axBoneMatrices[80] : register(c11);

int g_iNumBoneWeightings  : register(i0);

/******************************************************************************/
/* Types                                                                      */
/******************************************************************************/

struct InputVertex
{
	float4 m_xPosition     : POSITION;
	float4 m_xBlendIndices : BLENDINDICES;
	float4 m_xBlendWeights : BLENDWEIGHT;
};

/******************************************************************************/
/* Functions                                                                  */
/******************************************************************************/

float3x4 FetchBone( const int fIndex )
{
	float3x4 xMatrix = g_axBoneMatrices[ fIndex ];

	// Per-bone texturing blend factors ("damage values") are packed into one of the matrix
	// elements. We can reconstruct the row (assuming they're orthogonal) with a cross product
	xMatrix[2].xyz = cross( xMatrix[0].xyz, xMatrix[1].xyz );

	return xMatrix;
}

float4 main( const InputVertex xIn, out float fDepth : TEXCOORD ) : POSITION
{
	// Accumulate the blended skinning matrix

	float3x4 xBoneMat =
	{
		{ 0.0f, 0.0f, 0.0f, 0.0f },
		{ 0.0f, 0.0f, 0.0f, 0.0f },
		{ 0.0f, 0.0f, 0.0f, 0.0f },
	};

	for( int i = 0; i < g_iNumBoneWeightings; ++i )
	{
		const float fWeight = xIn.m_xBlendWeights[i];

		[branch]
		if( fWeight > 0.0f )
		{
			const float3x4 xMat = FetchBone( xIn.m_xBlendIndices[i] );
			xBoneMat += xMat * fWeight;
		}
	}

	const float4 xPos = float4( mul( xBoneMat, xIn.m_xPosition ), 1.0f );
	const float4 xOutPos = mul( g_xWorldViewProj, xPos );

	fDepth = (xOutPos.w - g_xShadowFrustum.x) * g_xShadowFrustum.y;

	return xOutPos;
}