#include "shared.fxh"

/******************************************************************************/
/* Parameters                                                                 */
/******************************************************************************/

const float4x4 g_xWorldViewProj;
const float4x3 g_axBoneMatrices[80];

/******************************************************************************/
/* Types                                                                      */
/******************************************************************************/

struct InputVertex
{
	float4 Position    : POSITION;
	int4   BoneIndices : BLENDINDICES;
	float4 BoneWeights : BLENDWEIGHT;
};

struct OutputVertex
{
	float4 Position : POSITION;
	float  LightspaceDistance : TEXCOORD0;
};

/******************************************************************************/
/* Functions                                                                  */
/******************************************************************************/

OutputVertex VShader( const InputVertex xIn )
{
	OutputVertex xOut;

	xOut.Position.xyz = mul( xIn.Position, g_axBoneMatrices[ xIn.BoneIndices[0] ] ) * xIn.BoneWeights[0]
					  + mul( xIn.Position, g_axBoneMatrices[ xIn.BoneIndices[1] ] ) * xIn.BoneWeights[1]
					  + mul( xIn.Position, g_axBoneMatrices[ xIn.BoneIndices[2] ] ) * xIn.BoneWeights[2]
					  + mul( xIn.Position, g_axBoneMatrices[ xIn.BoneIndices[3] ] ) * xIn.BoneWeights[3];

	xOut.Position.w = 1.0;

	xOut.Position = mul( xOut.Position, g_xWorldViewProj );
	xOut.LightspaceDistance = (xOut.Position.w - g_fShadowFrustumNearPlaneDistance) * g_fOneOverShadowFrustumPlaneSeparation;

	return xOut;
}

float4 PShader( const OutputVertex xIn ) : COLOR
{
	return xIn.LightspaceDistance.xxxx;
}

/******************************************************************************/
/* Techniques                                                                 */
/******************************************************************************/

technique
{
	pass
	{
//		Cullmode	= CW;
		VertexShader= compile vs_3_0 VShader();
		PixelShader = compile ps_3_0 PShader();
	}
}