/******************************************************************************/
/* Parameters                                                                 */
/******************************************************************************/

const float4x4 g_xWorldViewProjectionMatrix;
const float4x3 g_axBoneMatrices[80];

const float4 g_xWireframeColour;

/******************************************************************************/
/* Types                                                                      */
/******************************************************************************/

struct InputVertex
{
	float4 m_xPosition     : POSITION;
	int4   m_aiBoneIndices : BLENDINDICES;
	float4 m_afBoneWeights : BLENDWEIGHT;
};

/******************************************************************************/
/* Functions                                                                  */
/******************************************************************************/

float4 VShader( const InputVertex xIn ) : POSITION
{
	float4 xObjectSpacePos;

	xObjectSpacePos.xyz = mul( xIn.m_xPosition, g_axBoneMatrices[ xIn.m_aiBoneIndices[0] ] ) * xIn.m_afBoneWeights[0]
						+ mul( xIn.m_xPosition, g_axBoneMatrices[ xIn.m_aiBoneIndices[1] ] ) * xIn.m_afBoneWeights[1]
						+ mul( xIn.m_xPosition, g_axBoneMatrices[ xIn.m_aiBoneIndices[2] ] ) * xIn.m_afBoneWeights[2]
						+ mul( xIn.m_xPosition, g_axBoneMatrices[ xIn.m_aiBoneIndices[3] ] ) * xIn.m_afBoneWeights[3];

	xObjectSpacePos.w = 1.0;

	return mul( xObjectSpacePos, g_xWorldViewProjectionMatrix );
}

float4 PShader() : COLOR
{
	return g_xWireframeColour;
}

/******************************************************************************/
/* Techniques                                                                 */
/******************************************************************************/

technique
{
	pass
	{
		VertexShader = compile vs_3_0 VShader();
		PixelShader = compile ps_3_0 PShader();
	}
}