/**
*
* \file Asura_PlatformStreamSound.cpp
*
* Created by Kevin Lea - 24/04/2002 14:19:24
*
* Copyright (C) 2002 Rebellion, All Rights Reserved.
*
* \brief
*
**/

#include "Core/Asura.h"

#ifndef ASURA_USE_AUDIO_SYSTEM

#include "Sound/Asura_PlatformStreamSound.h"

#include "Core/Asura_String.h"

#include "BinkPC/bink.h"
#include "Sound/Asura_PC_Sound.h"

#if defined( ASURA_PC_ALLOW_LOAD_FROM_SEARCHPATHS )
	#include "FileAccess/Asura_FileAccess.h"
#endif

#if defined( _WIN64)
	#undef BinkSoundUseDirectSound
	#define BinkSoundUseDirectSound( _A)

	#undef BinkSoundUseWaveOut
	#define BinkSoundUseWaveOut()
	
	#undef BinkGoto
	#define BinkGoto( _A, _B, _C)
	
	#undef BinkWait
	#define BinkWait( _A) (false)
	
	#undef BinkDoFrame
	#define BinkDoFrame( _A)
	
	#undef BinkNextFrame
	#define BinkNextFrame( _A)
	
	#undef BinkOpen
	#define BinkOpen( _A, _B) (NULL)
	
	#undef BinkClose
	#define BinkClose( _A)
	
	#undef BinkPause
	#define BinkPause( _A, _B)
	
	#undef BinkSetVolume
	#define BinkSetVolume( _A, _B, _C)
#endif

class Asura_PC_StreamingSound
{
public:

	Asura_PC_StreamingSound() :
        m_pxBinkHandle( 0 ),
        m_bLooping( false ),
		m_bPausedWhenAppLostFocus( false )
    {
    }

    void Update();
    bool Create( const char* szBinkFilename );
    void Play( bool bLooping );
    void Stop();
    void Release();
    void Pause();
    void Resume();
    void SetVolume( float fVol );
	void UpdateVolume();

    HBINK m_pxBinkHandle;
    bool m_bLooping;
	bool m_bPausedWhenAppLostFocus;	// true if needs to be unpaused when app is activated again
	float m_fVolume;
};

Asura_PC_StreamingSound* Asura_PlatformStreamSound::s_axStreamingSounds = NULL;

void Asura_PlatformStreamSound::Initialise()
{
	// AP 8-10-07 - do NOT use BinkSoundUseWaveOut, as changing the volume of Bink then also changes 
	// the volumes of wave files in Windows itself (eg. through Media Player!).
	BinkSoundUseDirectSound( 0 );

	s_axStreamingSounds = new Asura_PC_StreamingSound[ASURA_PC_MAX_CONCURRENT_SOUNDSTREAMS];
}

void Asura_PlatformStreamSound::Shutdown()
{
    delete [] s_axStreamingSounds;
    s_axStreamingSounds = NULL;
}

bool Asura_PlatformStreamSound::Create( int iStreamID, const char* szFilename )
{
    Asura_String xPath;
	Asura_String xName;
    Asura_String xFilename = szFilename;

    xFilename.SplitFilename( &xPath, &xName, 0 );
    xFilename = xPath;
    xFilename += "\\";
    xFilename += xName;
    xFilename += ".bik";

#if defined( ASURA_PC_ALLOW_LOAD_FROM_SEARCHPATHS)
	const u_int uFullFilenameSize = MAX_PATH;
	char szFullFilename[ uFullFilenameSize + 1] = { 0};
	Asura_FileAccess::PlatformSearchPathsGetExistingFilename( xFilename.GetString(), szFullFilename, uFullFilenameSize );
	
    xFilename = szFullFilename;
#endif

    return s_axStreamingSounds[iStreamID].Create( xFilename.GetString() );

}

void Asura_PlatformStreamSound::Play( int iStreamID, bool bLooping )
{
    s_axStreamingSounds[iStreamID].Play( bLooping );
}

void Asura_PlatformStreamSound::Stop( int iStreamID )
{
    s_axStreamingSounds[iStreamID].Stop();
}

void Asura_PlatformStreamSound::Release(int iStreamID)
{
    s_axStreamingSounds[iStreamID].Release();
}

void Asura_PlatformStreamSound::Update( void )
{
	// MGH 25/8/2005: Originally update was only called once. However the
	// bink audio was recorded at typically 15fps and when the overall
	// frame rate fell below 15fps the audio became choppy.
	// Calling update 3 times should now allow the frame rate to fall to 5fps
	// before the audio becomes choppy.
    for ( u_int u = 0; u < ASURA_PC_MAX_CONCURRENT_SOUNDSTREAMS; ++u )
    {
        s_axStreamingSounds[u].Update();
		s_axStreamingSounds[u].Update();
		s_axStreamingSounds[u].Update();
    }
}

void Asura_PlatformStreamSound::Pause( int iStreamID )
{
    s_axStreamingSounds[iStreamID].Pause();
}

void Asura_PlatformStreamSound::Resume( int iStreamID )
{
    s_axStreamingSounds[iStreamID].Resume();
}

void Asura_PlatformStreamSound::SetVolume( int iStreamID, float fVol )
{
    s_axStreamingSounds[iStreamID].SetVolume( fVol );
}

void Asura_PlatformStreamSound::UpdateVolume()
{
	for ( int i = 0; i < ASURA_PC_MAX_CONCURRENT_SOUNDSTREAMS; ++i )
	{
		if ( s_axStreamingSounds[i].m_pxBinkHandle )
		{
			s_axStreamingSounds[i].UpdateVolume();	
		}
	}
}

bool Asura_PlatformStreamSound::IsFinished( int iStreamID )
{
	HBINK pxBink = s_axStreamingSounds[iStreamID].m_pxBinkHandle;

	if ( !pxBink )
	{
		return true;
	}

	return pxBink->Frames == pxBink->FrameNum && !s_axStreamingSounds[iStreamID].m_bLooping;
}

float Asura_PlatformStreamSound::GetPosition( int iStreamID )
{
	HBINK pxBink = s_axStreamingSounds[iStreamID].m_pxBinkHandle;

	if ( !pxBink )
	{
		return 0;
	}

	if ( !pxBink->FrameRate || !pxBink->Frames )
	{
		return 0;
	}

	float fSecsPerFrame = static_cast<float>( pxBink->FrameRateDiv ) / static_cast<float>( pxBink->FrameRate );

	return fSecsPerFrame * pxBink->FrameNum;
}

void Asura_PlatformStreamSound::SetPosition( int iStreamID, float fTime )
{
	HBINK pxBink = s_axStreamingSounds[iStreamID].m_pxBinkHandle;
	
	if ( !pxBink )
	{
		return;
	}
	
	if ( !pxBink->FrameRate || !pxBink->Frames )
	{
		return;
	}

	// Determine the desired frame
	int iFrame = Asura_Maths::FloatToInt( ( fTime * pxBink->FrameRate ) / pxBink->FrameRateDiv );
	Asura_Assert( pxBink->Frames < INT_MAX, "Asura code doesn't handle this many frame due to signed-unsigned mismatch" );
	iFrame = Asura_Maths::Min( iFrame, static_cast<int>( pxBink->Frames - 1 ) );

	BinkGoto( pxBink, iFrame, BINKGOTOQUICK | BINKGOTOQUICKSOUND );
}

float Asura_PlatformStreamSound::GetLength( int iStreamID )
{
	HBINK pxBink = s_axStreamingSounds[iStreamID].m_pxBinkHandle;

	if ( !pxBink )
	{
		return 0.0f;
	}
	
	if ( !pxBink->FrameRate )
	{
		return 0.0f;
	}

	float fSecsPerFrame = static_cast<float>( pxBink->FrameRateDiv ) / static_cast<float>( pxBink->FrameRate );
	return fSecsPerFrame * pxBink->Frames;
}

void Asura_PlatformStreamSound::LoseFocus_PauseAllSounds()
{
	if ( s_axStreamingSounds )
	{
		for ( int i = 0; i < ASURA_PC_MAX_CONCURRENT_SOUNDSTREAMS; ++i )
		{
			if ( s_axStreamingSounds[i].m_pxBinkHandle )
			{
				if ( !s_axStreamingSounds[i].m_pxBinkHandle->Paused )	
				{
					s_axStreamingSounds[i].Pause();
					s_axStreamingSounds[i].m_bPausedWhenAppLostFocus = true;
				}
			}
		}
	}
}

void Asura_PlatformStreamSound::GainFocus_UnpauseSounds()
{
	if ( s_axStreamingSounds )
	{
		for ( int i = 0; i < ASURA_PC_MAX_CONCURRENT_SOUNDSTREAMS; ++i )
		{
			if ( s_axStreamingSounds[i].m_pxBinkHandle )
			{
				if ( s_axStreamingSounds[i].m_pxBinkHandle->Paused && s_axStreamingSounds[i].m_bPausedWhenAppLostFocus )
				{
					s_axStreamingSounds[i].Resume();
					s_axStreamingSounds[i].m_bPausedWhenAppLostFocus = false;
				}
			}
		}
	}
}

void Asura_PC_StreamingSound::Update()
{
    if ( !m_pxBinkHandle )
	{
		return;
	}

	if ( !BinkWait( m_pxBinkHandle ) )
	{
		BinkDoFrame( m_pxBinkHandle );
		
        if ( !m_bLooping && ( m_pxBinkHandle->Frames == m_pxBinkHandle->FrameNum ) )
        {
            // Changed from Stop to Pause , so it doesn't go back to the beginning
			// (To make the information about where the track has got to more helpful).
			// RAR 20/01/04
            Pause();
        }
        else
        {
		    BinkNextFrame( m_pxBinkHandle );
        }
	}
}

bool Asura_PC_StreamingSound::Create( const char* szBinkFilename )
{
    m_fVolume = 1.0f;
    m_pxBinkHandle = BinkOpen( szBinkFilename, 0 );

    if ( m_pxBinkHandle )
    {
        BinkPause( m_pxBinkHandle, 1 );
        UpdateVolume();
    }

    Asura_Assertf( NULL != this->m_pxBinkHandle, ( "Cannot open file '%s'", szBinkFilename ) );
    return m_pxBinkHandle != 0;
}

void Asura_PC_StreamingSound::Play( bool bLooping )
{
    Asura_Assert( m_pxBinkHandle, "" );

    if ( m_pxBinkHandle ) 
    {
		BinkPause( m_pxBinkHandle, 0 );
		m_bLooping = bLooping;
	}
}

void Asura_PC_StreamingSound::Stop()
{
    Asura_Assert( m_pxBinkHandle, "" );

    if ( m_pxBinkHandle ) 
    {
		// pause and goto start
		BinkPause( m_pxBinkHandle, 1 );
		BinkGoto( m_pxBinkHandle, 0, 0 );
	}
}

void Asura_PC_StreamingSound::Release()
{
    Asura_Assert( m_pxBinkHandle, "" );

    if ( m_pxBinkHandle ) 
    {
		BinkClose( m_pxBinkHandle );
		m_pxBinkHandle = 0;
	}
}

void Asura_PC_StreamingSound::Pause()
{
    Asura_Assert( m_pxBinkHandle, "" );

    if ( m_pxBinkHandle ) 
    {
	    BinkPause( m_pxBinkHandle, 1 );
	}
}

void Asura_PC_StreamingSound::Resume()
{
    Asura_Assert( m_pxBinkHandle, "" );

    if ( m_pxBinkHandle ) 
    {
	    BinkPause( m_pxBinkHandle, 0 );
	}
}

void Asura_PC_StreamingSound::SetVolume( float fVol )
{
	m_fVolume = fVol;
	UpdateVolume();
}

void Asura_PC_StreamingSound::UpdateVolume()
{
    int iVol = Asura_Maths::FloatToInt( m_fVolume * 32768.0f );

	if ( iVol < 100 )
	{
		iVol = 100; // Setting volume to 0 appears to result in max volume - TBD is this still true? And by design?
	}

    BinkSetVolume( m_pxBinkHandle, 0, iVol );
}

#endif // ndef ASURA_USE_AUDIO_SYSTEM