//////////////////////////////////////////////////////////////////////////////
//
// Asura_Platform_BillboardPlant.h
//
// Copyright (c) 2002 Rebellion. All rights reserved.
//
// Author: Andrew Weinkove - 11/12/02
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////
#include "Core/Asura.h"

#ifdef ASURA_USE_BILLBOARD_PLANTS

#include "SpecialFX/Asura_BillboardPlant_Batch.h"
#include "SpecialFX/Asura_BillboardPlant_TemplateStore.h"
#include "Camera/Asura_Camera.h"
#include "Rendering/Asura_Direct3d.h"
#include "Rendering/Asura_Direct3D_Shaders.h"
#include "Rendering/Asura_Render.h"

// Placeholder PC renderer, only works with one camera doesn't use proper lighting
// and is probably very inefficient.
void Asura_BillboardPlant_Batch::Platform_Render(unsigned uCamVisFlags)
{
	Asura_BillboardPlant_Template *pxTemplate = Asura_BillboardPlant_TemplateStore::GetTemplate(m_iPlantIndex);
	Asura_Assert(pxTemplate, "");
	if (!pxTemplate)
		return;
	Asura_Camera::SetCameraIndex(0);
	Asura_Render::ApplyCameraSettings();
	Asura_Direct3D_Shaders::RequestFixedFunctionVertexShader(D3DFVF_GENERICVERTEX);
	Asura_Direct3D::D3DDevice->SetRenderState(D3DRS_CULLMODE, D3DCULL_NONE);
	Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_NORMAL);
	Asura_Render::CurrentStates.RequestTextureOpMode(ASURA_TEXTUREOP_SINGLE_PASS);
	Asura_Render::CurrentStates.RequestTextureIndex(0,pxTemplate->m_iTextureIndex);
	Asura_Render::CurrentStates.RequestAlphaTest( true );
	Asura_Render::CurrentStates.RequestZBufferWriteEnabled(true);
	D3DXMATRIX matWorld;
    matWorld._12 = matWorld._13 = matWorld._14 = matWorld._21 = matWorld._23 = matWorld._24 = 0.0f;
    matWorld._31 = matWorld._32 = matWorld._34 = matWorld._41 = matWorld._42 = matWorld._43 = 0.0f;
    matWorld._11 = matWorld._22 = matWorld._33 = 1.0f;
	matWorld._44 = 1.0f;
	matWorld._41 = 0.0f;
	matWorld._42 = 0.0f;
	matWorld._43 = 0.0f;
	Asura_Direct3D::D3DDevice->SetTransform( D3DTS_WORLD,      &matWorld );

	D3DGenericVertex v[4];
	for (int i=0;i<4;i++)
	{
		v[i].U = (i % 2) ? pxTemplate->m_fBottomRightU : pxTemplate->m_fTopLeftU;
		v[i].V = (i < 2) ? pxTemplate->m_fTopLeftV : pxTemplate->m_fBottomRightV;
		v[i].Diffuse = 0xffffffff;
	}

	for (unsigned i=0;i<m_uNumberOfPlants;i++)
	{
		Asura_Vector_3 axVertices[4];
		FillVertices(axVertices, i);
		for (int i=0;i<4;i++)
		{
			v[i].X = axVertices[i].x;
			v[i].Y = axVertices[i].y;
			v[i].Z = axVertices[i].z;
		}
		// Only render the front facing triangles because we can guarantee that the
		// backs will never be visible (the billboards turn to face the camera).
		short indices[6] = 
		{
			0,1,2,
			1,2,3,
		};
		Asura_Direct3D::D3DDevice->DrawIndexedPrimitiveUP
		(
		D3DPT_TRIANGLELIST,
			0,
			4,
			2,
			(void*)indices,
			D3DFMT_INDEX16,
			(void*)v,
			sizeof(D3DGenericVertex)
		);
	}
}

#endif // #ifdef ASURA_USE_BILLBOARD_PLANTS
