
//
// Asura_Platform_PFX_FastPath.h
//
// Created by DMR 15/11/06
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
// Faster renderer for most PFX types, using hardware instancing
//

#ifndef __ASURA_PFX_SYSTEM_FASTPATH_H__
#define __ASURA_PFX_SYSTEM_FASTPATH_H__

 #ifdef ASURA_USE_FASTPATH_PFX

// Asura PC
#include "Rendering/Asura_Direct3D.h"

// Forward declares
class Asura_Direct3D_Effect;
class Asura_PFX_Effect;
class Asura_PFX_Particle;
class Asura_PFX_Particle_Template;
class Asura_RGBA;


struct Asura_Platform_PFXInstanceData
{
	float X, Y, Z, Orientation;					
	float UVSet0[4];							
	float UVSet1[4];							
	float Width, Height;						
	float TextureAlignment, CrossFadeBlendWeight;
	float Diffuse[4];					
};

struct Asura_Platform_PFX_Vertex
{
	float fVertexIndex;
};




class Asura_Platform_PFX_System_FastPath
{
public:

	static void Initialise();
	static void Shutdown();	
	
	static void PreD3DReset();
	static void PostD3DReset();

	static bool IsEnabled(void) {return s_bIsEnabled;}
	static void SetEnabled(bool bEnabled) {s_bIsEnabled = bEnabled;}

	static void UpdateInstanceBuffersForNewFrame();
	static void *GetInstanceBufferArea(uint32 uNumParticles);

	static uint32 GetNumFastPathParticlesRenderedThisFrame() {return s_uFastPathParticlesRenderedThisFrame;}
	static float  GetNumFastPathParticleScreenfullsRenderedThisFrame() {return s_fApproxScreenfullsRenderedThisFrame;}
	
	static bool   WasVertexBufferFull()	{return s_bVertexBufferOverflow;}
	
	static void RenderParticles(				
		const  Asura_PFX_Effect *pxEffect,
		const  Asura_PFX_Particle* pxParticles,
		u_int  uNumParticles,
		const  Asura_PFX_Particle_Template* pxTemplate,
		const  Asura_RGBA& xLightColour,
		const  uint16 usCameraFlags, 
		const  uint16 usPFXFlags,														 
		void* pPlatformRenderData);


	static void UpdateInstanceBuffer(const  Asura_PFX_Particle_Template *pxTemplate, 
									 const  Asura_PFX_Particle *pxParticles,
						  			 Asura_Platform_PFXInstanceData *pxParticleInstanceData, 
									 uint32 uNumParticles);

	static void RenderStats(void);

private:
	static void CaptureEffect(void);
	static void OnEffectUnloaded(Asura_Direct3D_Effect* pxEffect);
	static void OnEffectLoaded(Asura_Direct3D_Effect* pxEffect);

	static bool									s_bIsEnabled;
	static Direct3D_VertexDeclaration*		    s_pxPFXVertexDecl;	
	static Asura_Direct3D_Effect*			    s_pxPFXEffect;
	
	static D3DXHANDLE							s_xTextureHandle;
	static D3DXHANDLE							s_xDepthTextureHandle;
	static D3DXHANDLE							s_xFirstInstanceHandle;
	static D3DXHANDLE							s_xUseLightingHandle;
	static D3DXHANDLE							s_xCrossfadeTextureFramesHandle;
	static D3DXHANDLE							s_xZReadEnableHandle;
	static D3DXHANDLE							s_xXDeltaHandle;
	static D3DXHANDLE							s_xYDeltaHandle;		
	static D3DXHANDLE							s_xLightColourHandle;
	static D3DXHANDLE							s_xVolumetricDepthScaleHandle;
	static D3DXHANDLE							s_xPosOffsetAndSizeScaleHandle;
	static D3DXHANDLE							s_xNearFarFadeHandle;
	static D3DXHANDLE							s_xTwoOverScreenWidthHandle;
	static D3DXHANDLE							s_xTwoOverScreenHeightHandle;
	static D3DXHANDLE							s_xIsVolumetricHandle;
	
	static bool									s_bVertexBufferLocked;
	static bool									s_bVertexBufferOverflow;
	static int						 			s_iCurrentVertexBufferPosition;
	static Asura_Platform_PFXInstanceData	   *s_pxCurrentVertexBuffer;

	static uint32								s_uFastPathParticlesRenderedThisFrame;
	static float								s_fApproxScreenfullsRenderedThisFrame;
	

	enum 
	{
#ifdef ASURA_FASTPATH_PFX_VERTEXBUFFER_SIZE		    
		VERTEXBUFFER_SIZE   = ASURA_FASTPATH_PFX_VERTEXBUFFER_SIZE
#else
		VERTEXBUFFER_SIZE   = 7500
#endif
	};



	static Direct3D_VertexBuffer*				s_pxVertexBuffer;
	static Direct3D_VertexBuffer*			    s_pxVertexDataBuffer;
	static Direct3D_IndexBuffer*			    s_pxIndexBuffer;


	
};

#endif 

#endif // __ASURA_PFX_SYSTEM_FASTPATH_H__