//
// Asura_Platform_Visibility_System.cpp
//
// Created by KJFL - 06 May 2005
//
// Copyright (C) 2005 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

#if defined(ASURA_USE_VISIBILITY_SYSTEM)

// This file's header.
#include "Visibility/Asura_Visibility_System.h"

//#include "Core/Asura_Maths.h"
#include "Camera/Asura_Camera.h"
#include "Rendering/Asura_Direct3D.h"
#include "Rendering/Asura_Direct3D_EnvironmentRenderer.h"


class Asura_Direct3D_OcclusionQuery
{
	public:

		static Asura_Direct3D_OcclusionQuery* Create( u_int uNodeIndex )
		{
			Direct3D_Query* pxQuery = 0;
			Asura_Direct3D::D3DDevice->CreateQuery( D3DQUERYTYPE_OCCLUSION, &pxQuery );

			if (pxQuery)
			{
				Asura_Direct3D_OcclusionQuery* pxOccQuery = new Asura_Direct3D_OcclusionQuery;

				pxOccQuery->m_pxQuery = pxQuery;
				pxOccQuery->m_uNodeIndex = uNodeIndex;
				pxOccQuery->m_pxNext = 0;
				
				return pxOccQuery;
			}
			else
			{
				return 0;
			}
		}

		~Asura_Direct3D_OcclusionQuery()
		{
			m_pxQuery->Release();
		}

		void Start( void )
		{
			m_pxQuery->Issue(D3DISSUE_BEGIN);
		}

		void End( void )
		{
			m_pxQuery->Issue(D3DISSUE_END);
		}

		int GetResult( bool bForceFlush )
		{
			int uPixelsDrawn = -1;

			HRESULT iRes = m_pxQuery->GetData( &(uPixelsDrawn), sizeof(DWORD), bForceFlush ? D3DGETDATA_FLUSH : 0 );

			if ( iRes == S_FALSE )
			{
				return -1;
			}
			else if ( iRes == S_OK )
			{
				return uPixelsDrawn;
			}
			else
			{
				return 0;
			}
		}

	
	
		Direct3D_Query*	m_pxQuery;
		u_int			m_uNodeIndex;

		Asura_Direct3D_OcclusionQuery* m_pxNext;
};


	

Asura_Direct3D_OcclusionQuery* g_pxActiveQueries = 0;
Asura_Direct3D_OcclusionQuery* g_pxEndOfQueue = 0;
u_int g_uNumberOfActiveQueries = 0;

void Asura_Visibility_System::Platform_RenderStripZOnly( u_int uStripID )
{
	Asura_Direct3D_EnvironmentRenderer::RenderStripZOnly( uStripID );
}

void Asura_Visibility_System::Platform_IssueOcclusionQuery( u_int uNodeIndex, const Asura_Bounding_Box& xBB )
{
	Asura_Direct3D_OcclusionQuery* pxQuery = Asura_Direct3D_OcclusionQuery::Create( uNodeIndex );

    if (!pxQuery) return;

	pxQuery->Start();
	Asura_Direct3D_EnvironmentRenderer::RenderAABB( xBB );
	pxQuery->End();
#if 0
	pxQuery->m_pxNext = g_pxActiveQueries;
	g_pxActiveQueries = pxQuery;

	g_uNumberOfActiveQueries++;
#else

	g_pxEndOfQueue = g_pxActiveQueries;
	if ( g_pxEndOfQueue )
	{
		while (g_pxEndOfQueue->m_pxNext)
		{
			g_pxEndOfQueue = g_pxEndOfQueue->m_pxNext;
		}

		g_pxEndOfQueue->m_pxNext = pxQuery;
		g_pxEndOfQueue = pxQuery;
	}
	else
	{
		g_pxActiveQueries = pxQuery;
	}

	g_uNumberOfActiveQueries++;
#endif
}

void Asura_Visibility_System::Platform_IssueOcclusionQuery( u_int uNodeIndex, u_int uStripID )
{
	Asura_Direct3D_OcclusionQuery* pxQuery = Asura_Direct3D_OcclusionQuery::Create( uNodeIndex );

    if (!pxQuery) return;

	pxQuery->Start();
	Asura_Direct3D_EnvironmentRenderer::RenderStripZOnly( uStripID );
	pxQuery->End();
#if 0
	pxQuery->m_pxNext = g_pxActiveQueries;
	g_pxActiveQueries = pxQuery;

	g_uNumberOfActiveQueries++;
#else
	g_pxEndOfQueue = g_pxActiveQueries;
	if ( g_pxEndOfQueue )
	{
		while (g_pxEndOfQueue->m_pxNext)
		{
			g_pxEndOfQueue = g_pxEndOfQueue->m_pxNext;
		}

		g_pxEndOfQueue->m_pxNext = pxQuery;
		g_pxEndOfQueue = pxQuery;
	}
	else
	{
		g_pxActiveQueries = pxQuery;
	}

	g_uNumberOfActiveQueries++;
#endif
}

u_int Asura_Visibility_System::Platform_GetNumberOfOutstandingOcclusionQueries( void )
{
	return g_uNumberOfActiveQueries;
}

bool Asura_Visibility_System::Platform_GetOcclusionResults( u_int& uNodeIndex, bool bForceFlush )
{
	Asura_Direct3D_OcclusionQuery** ppxActiveQueries = &g_pxActiveQueries;

	while (*ppxActiveQueries)
	{
		Asura_Direct3D_OcclusionQuery* pxThisQuery = (*ppxActiveQueries);

		int iReturn = pxThisQuery->GetResult( bForceFlush );

		if (iReturn >= 0)
		{
			bForceFlush = false;

			uNodeIndex = pxThisQuery->m_uNodeIndex;

			*ppxActiveQueries = (pxThisQuery->m_pxNext);

			delete pxThisQuery;

			g_uNumberOfActiveQueries--;

			if (iReturn > 0)
			{
				return true;
			}
		}
		
		if (*ppxActiveQueries)
		{
			ppxActiveQueries = &((*ppxActiveQueries)->m_pxNext);
		}
		
	}


	return false;
}

#endif	// defined(ASURA_USE_VISIBILITY_SYSTEM)


/// End of file. ///
