//////////////////////////////////////////////////////////////////////////////
//
// Asura_PS2_Hierarchy.h
//
// Copyright (c) 2004 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef ___ASURA_PS2_HIERARCHY_H___
#define ___ASURA_PS2_HIERARCHY_H___ 1

// I N C L U D E S ///////////////////////////////////////////////////////////

// Engine independent.
#include "Core/Asura_Hash.h"
#include "Core/Asura_HashTree.h"
#include "Core/Asura_TextureFlags.h"

// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////////

struct Asura_PS2_HierSkin_SkinHdr;
struct Asura_PS2_SmoothSkin_SkinHdr;
class  Asura_Vector_3;
class  Asura_Stream;

// D E F I N E S /////////////////////////////////////////////////////////////

#define ASURA_PS2_SKINSYSTEM_MAX_LODS_PER_SKIN 4

// C L A S S E S /////////////////////////////////////////////////////////////

class Asura_PS2_SkinEntry
{
public:

	// C L A S S   M E T H O D S /////////////////////////////////////////
	Asura_PS2_SkinEntry()
		: m_uNameHashID(ASURA_HASH_ID_UNSET)
		, m_iTexID(ASURA_TEXTURE_NONE)
		, m_bIsSmooth(false)
		, m_bAllMaterialAreOpaque(false)
		, m_usNumLODs(0)
#ifdef _DEBUG
		, m_szName(NULL)
		, m_bHasBeenScaled(false)
#endif
	{
		for (int i =0; i < ASURA_PS2_SKINSYSTEM_MAX_LODS_PER_SKIN; i++)
		{
			m_apvLODs[i] = NULL;
			m_afThresholdsSqd[i] = 0.0f;
		}
	}

	~Asura_PS2_SkinEntry()
	{
		while( m_usNumLODs-- )
		{
			Asura_DeallocateAlign( m_apvLODs[m_usNumLODs] );
		}

#ifdef _DEBUG
		if (m_szName) delete[] m_szName;
#endif
	}

	// C L A S S   D A T A ///////////////////////////////////////////////
	u_int   m_uNameHashID;
	int     m_iTexID;
	bool    m_bIsSmooth;
	bool    m_bAllMaterialAreOpaque;
	u_short m_usNumLODs;
	void*   m_apvLODs[ASURA_PS2_SKINSYSTEM_MAX_LODS_PER_SKIN];
	float   m_afThresholdsSqd[ASURA_PS2_SKINSYSTEM_MAX_LODS_PER_SKIN];
#if _DEBUG
	char*   m_szName;
	bool	m_bHasBeenScaled;
#endif
};


class Asura_PS2_Skin_Handler
{
	friend class Asura_PS2_SkinSystem;

public:
	// C L A S S   M E T H O D S /////////////////////////////////////////
	Asura_PS2_Skin_Handler() : m_fAppliedThresholdMultiplier(0.0f) {}

	void Reset( void );

	Asura_PS2_SkinEntry* AddSkin( const char* szName );
	//float* GetThresholdArrayPtr( int iSkinIdx );

	// These two methods return true iff the LOD was added successfully.
	// The caller should deallocate the data on failure. The PS2-specific
	// data needs to be loaded and added immediately after the
	// independent code's version is loaded since otherwise the texture /
	// material index conversion will break. LODs are assumed to appear
	// in decreasing order of detail.
	bool AddHierSkinLOD( const char* szSkinName, void* pvHierSkinData, unsigned uDataSizeInBytes );
	bool AddSmoothSkinLOD( const char* szSkinName, void* pvSmoothSkinData, unsigned uDataSizeInBytes );

	// Assuming the skin index is valid and that there's at least one LOD
	// loaded for this skin, these methods will not fail. These methods
	// will determine which LOD to use based on the nearest camera and
	// return a pointer to the corresponding header.
	const Asura_PS2_HierSkin_SkinHdr* GetHierSkinHdr( const Asura_PS2_SkinEntry* pxSkinEntry, const Asura_Vector_3& xPos, u_int uCamFlags, float* pfDistFromNearestCamSqd );
	const Asura_PS2_SmoothSkin_SkinHdr* GetSmoothSkinHdr( const Asura_PS2_SkinEntry* pxSkinEntry, const Asura_Vector_3& xPos, u_int uCamFlags, float* pfDistFromNearestCamSqd );

	// Returns -1 iff no PS2 skin has been loaded with the specified name.
	//int GetSkinIdx( Asura_Hash_ID uNameHashID ) { return FindEntryForHashID( uNameHashID ); }
	// Returns NULL if no PS2 skin has been loaded with the specified name.
	//const SkinEntry* GetSkinEntry( Asura_Hash_ID uNameHashID ) { return FindEntryForHashID( uNameHashID ); }

	void SetNumViewports( u_int uNumViewports );

	int  GetTextureID( Asura_Hash_ID uNameHashID );
	bool IsOpaque( Asura_Hash_ID uNameHashID );

private:
	// C L A S S   M E T H O D S /////////////////////////////////////////

	Asura_PS2_SkinEntry* FindEntryForHashID( Asura_Hash_ID uNameHashID, bool bAddIfNotFound = false );

	bool AddSkinLODInternal( const char* szSkinName, void* pvSkinData, u_int uDataSizeInBytes, bool bIsSmooth );
	u_int CalculateLOD( const Asura_PS2_SkinEntry* pxSkinEntry, const Asura_Vector_3& xPos, u_int uCamVisFlags, float* pfDistFromNearestCamSqd );
	bool PreprocessHierSkin( const char* szSkinName, void* pvHierSkinData, u_int uNumBytes, Asura_PS2_SkinEntry* pxSkinEntry, bool bIsFirstLOD );
	bool PreprocessSmoothSkin( const char* szSkinName, void* pvSmoothSkinData, u_int uNumBytes, Asura_PS2_SkinEntry* pxSkinEntry, bool bIsFirstLOD );
	int CountSkins();

	// C L A S S   D A T A ///////////////////////////////////////////////
	Asura_TemplateHashTree<Asura_PS2_SkinEntry> m_xSkinStorage;
	float m_fAppliedThresholdMultiplier;
};

class Asura_PS2_SkinSystem
{
	public:

        // T Y P E S /////////////////////////////////////////////////////////
        union HierOrSmoothSkinPtr
        {
            Asura_PS2_HierSkin_SkinHdr*   pxHier;
            Asura_PS2_SmoothSkin_SkinHdr* pxSmooth;
        };

        // C L A S S   M E T H O D S /////////////////////////////////////////
		static void Initialise( void );
		static void Reset( void );
		static void Shutdown( void );

		static Asura_PS2_SkinEntry* AddSkin( const char* szName )			{ return GetCurrentHandler().AddSkin(szName); }
		//static float* GetThresholdArrayPtr( int iSkinIdx )				{ return GetCurrentHandler().GetThresholdArrayPtr(iSkinIdx); }

		static bool AddHierSkinLOD( const char* szSkinName, void* pvHierSkinData, unsigned uDataSizeInBytes )				{ return GetCurrentHandler().AddHierSkinLOD(szSkinName, pvHierSkinData, uDataSizeInBytes); }
		static bool AddSmoothSkinLOD( const char* szSkinName, void* pvSmoothSkinData, unsigned uDataSizeInBytes )			{ return GetCurrentHandler().AddSmoothSkinLOD(szSkinName, pvSmoothSkinData, uDataSizeInBytes); }

		static const Asura_PS2_HierSkin_SkinHdr* GetHierSkinHdr( const Asura_PS2_SkinEntry* pxSkinEntry, const Asura_Vector_3& xPos, u_int uCamFlags, float* pfDistFromNearestCamSqd )		{ return GetCurrentHandler().GetHierSkinHdr(pxSkinEntry, xPos, uCamFlags, pfDistFromNearestCamSqd); }
        static const Asura_PS2_SmoothSkin_SkinHdr* GetSmoothSkinHdr( const Asura_PS2_SkinEntry* pxSkinEntry, const Asura_Vector_3& xPos, u_int uCamFlags, float* pfDistFromNearestCamSqd )	{ return GetCurrentHandler().GetSmoothSkinHdr(pxSkinEntry, xPos, uCamFlags, pfDistFromNearestCamSqd); }

		//static int GetSkinIdx( Asura_Hash_ID uNameHashID )		{ return GetCurrentHandler().GetSkinIdx( uNameHashID ); }
		static const Asura_PS2_SkinEntry* GetSkinEntry( Asura_Hash_ID uNameHashID )	{ return GetCurrentHandler().FindEntryForHashID(uNameHashID); }

		static void SetNumViewports( u_int uNumViewports );

		static int  GetTextureID( Asura_Hash_ID uNameHashID )	{ return GetCurrentHandler().GetTextureID( uNameHashID ); }
		static bool IsOpaque( Asura_Hash_ID uNameHashID )		{ return GetCurrentHandler().IsOpaque( uNameHashID ); }

		static void SetOnOffTransparencyConsideredOpaque(bool bSet);

#if defined(ASURA_USE_MULTIPLE_HANDLERS) || defined(ASURA_USE_STREAMING)
		// Handler functions...
		static	void					ReparentHandler		(Asura_PS2_Skin_Handler& xSourceHandler, const Asura_Hash_ID uNewParentHash);
		static	void					ReparentHandler		(const Asura_Hash_ID uSourceHash, const Asura_Hash_ID uNewParentHash);

		static	Asura_PS2_Skin_Handler*	CreateHandler		(const Asura_Hash_ID uHash, const Asura_Hash_ID uParentHandlerHash);
		static	bool					DestroyHandler		(const Asura_Hash_ID uHash);
		static	void					WriteHandlerReport	(u_int uHandler, Asura_Stream& xLog);

		static	void					CreateAndSetHandler	(const Asura_Hash_ID uHash, const Asura_Hash_ID uParentHandlerHash)	{ s_pxCurrentHandler = CreateHandler(uHash, uParentHandlerHash); }
		static	void					SetCurrentHandler	(Asura_PS2_Skin_Handler* pxHandler)									{ s_pxCurrentHandler = pxHandler; }
		static	void					SetCurrentHandler	(const Asura_Hash_ID uHash)											{ s_pxCurrentHandler = s_xHandlerTree.GetDataByHash(uHash); }
		static	Asura_PS2_Skin_Handler&	GetCurrentHandler	(void)																{ return(*s_pxCurrentHandler); }
		static	Asura_PS2_Skin_Handler*	GetHandler			(const Asura_Hash_ID uHash)											{ return(s_xHandlerTree.GetDataByHash(uHash)); }
#else
		// Handler functions...
		static	Asura_PS2_Skin_Handler&	GetCurrentHandler	(void)																{ return(s_xHandler); }
#endif

private:
		static int CountSkins();

	// C L A S S   D A T A //////////////////////////////////////////////
#if defined(ASURA_USE_MULTIPLE_HANDLERS) || defined(ASURA_USE_STREAMING)
	// Private variables...
	static	Asura_PS2_Skin_Handler*							s_pxCurrentHandler;		// Currently active handler.
	static	Asura_TemplateHashTree<Asura_PS2_Skin_Handler>	s_xHandlerTree;			// Tree of all of the handlers available.
#else
	// Private variables...
	static	Asura_PS2_Skin_Handler							s_xHandler;				// Single static-instance of a zone.
#endif
};

// End class Asura_PS2_SkinSystem.

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

#endif // #ifndef ___ASURA_PS2_HIERARCHY_H___
