;//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;//;	
;//;	FILE			VU0_Skinning.vcl
;//;	
;//;	DESCRIPTION		VU0 microcode for skinning characters
;//;	
;//;	AUTHOR			Mike Healey 28.11.01
;//;	
;//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;//;
;//;    Changes:        MH 12.3.02 -- only using first bone for normals, as
;//;                    the previous method of a weighted sum needed a square 
;//;                    root to renormalise at the end.
;//;    
;//;                    MP 22.04.2002 - Skinning in modelspace. Code now 
;//;                    expects VI01 to be preloaded with the value 0xff.
;//;
;//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

#include "Asura_PS2_CharDefines.h"

#define INPUT_DATA_START (ASURA_PS2_MAX_BONES_PER_MODEL * 4) 

.syntax new

.init_vf_all	 	; for all vf registers. 
.init_vi_all		; for all vi registers.

--enter
    ;// This must contain 0xff.
    in_vi LOOP_CTR_MASK(VI01)
--endenter

    IADDIU  INPUT_CTR,   VI00,     INPUT_DATA_START

    ;// load in our data
    LQI     INPUT_NORMAL,   (INPUT_CTR++)
    LQI     ARRAY_BONE_IDS, (INPUT_CTR++)
    LQI     ARRAY_WEIGHTS,  (INPUT_CTR++)
    LQ.xyz  INPUT_POINT,    0(INPUT_CTR)        ;// Would just be another LQI but VCL complains.

    ;// set output point to (0, 0, 0, 1)
    MOVE    OUTPUT_POINT,   VF00

    ;// load num weights from normal's w and mask off the ADC bit, if it's
    ;// there
    MTIR    LOOP_CTR,   INPUT_NORMAL[w]
    IAND    LOOP_CTR,   LOOP_CTR, LOOP_CTR_MASK

    ;// Extract next matrix address and rotate bone matrix address array
    ;// ready for next bone.
    MTIR    MATRIX_ADDRESS, ARRAY_BONE_IDS[x]
    MR32    ARRAY_BONE_IDS, ARRAY_BONE_IDS

    ;// Load first bone matrix.
    LQI.xyz BONE_MATRIX1, (MATRIX_ADDRESS++)
    LQI.xyz BONE_MATRIX2, (MATRIX_ADDRESS++)
    LQI.xyz BONE_MATRIX3, (MATRIX_ADDRESS++)
    LQ.xyz  BONE_MATRIX4, 0(MATRIX_ADDRESS)     ;// Would just be another LQI but VCL complains.

    ;// Prepare matrix address for next bone.
    MTIR    MATRIX_ADDRESS,   ARRAY_BONE_IDS[x]

    ;// transform normal to output reg
    MULAx.xyz  ACC,           BONE_MATRIX1, INPUT_NORMAL
    MADDAy.xyz ACC,           BONE_MATRIX2, INPUT_NORMAL
    MADDz.xyz  OUTPUT_NORMAL, BONE_MATRIX3, INPUT_NORMAL

    ;// Copy input normal w to output normal.
    MAXx.w OUTPUT_NORMAL, INPUT_NORMAL, VF00

LOOP:

    --LoopCS	1,4 ;// minimum 1 time, can handle up to 4 times

    ;// transform point to temp reg 
    MULAx.xyz  ACC,   BONE_MATRIX1, INPUT_POINT
    MADDAy.xyz ACC,   BONE_MATRIX2, INPUT_POINT
    MADDAz.xyz ACC,   BONE_MATRIX3, INPUT_POINT
    MADDw.xyz  TEMP1, BONE_MATRIX4, VF00

    ;// load next bone matrix
    LQI.xyz BONE_MATRIX1, (MATRIX_ADDRESS++)
    LQI.xyz BONE_MATRIX2, (MATRIX_ADDRESS++)
    LQI.xyz BONE_MATRIX3, (MATRIX_ADDRESS++)
    LQ.xyz  BONE_MATRIX4, 0(MATRIX_ADDRESS)     ;// Would just be another LQI but VCL complains.

    ;// Rotate bone matrix address array ready for next bone.
    MR32    ARRAY_BONE_IDS,   ARRAY_BONE_IDS

    ;// decrement loop counter
    IADDI   LOOP_CTR,   LOOP_CTR,   -1

    ;// scale transformed point by weight and add to output point
    MULx.xyz TEMP1, TEMP1, ARRAY_WEIGHTS
    ADD.xyz  OUTPUT_POINT, OUTPUT_POINT, TEMP1

    ;// Prepare matrix address for next bone.
    MTIR    MATRIX_ADDRESS,   ARRAY_BONE_IDS[x]
    
    ;// Rotate bone weight array ready for next bone.
    MR32    ARRAY_WEIGHTS,   ARRAY_WEIGHTS

    ;// Are we done yet?
    IBNE    LOOP_CTR,   VI00,   LOOP

END_LOOP:
    
;// end program
--exit
    out_vi LOOP_CTR_MASK(VI01)
    out_vf OUTPUT_POINT(VF09)
    out_vf OUTPUT_NORMAL(VF10)
--endexit

