//////////////////////////////////////////////////////////////////////////////
//
// Asura_Chunk_ResourceFile_PS2_IRX.cpp
//
// By: Mike Healey.
//
// Copyright (c) 2002 Rebellion. All rights reserved.
//
// resource file IRX module loader
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header.
#include "ChunkSystem/Asura_Chunk_ResourceFile_PS2_IRX.h"

// Stdlib platform-specific.
#include <sif.h>
#include <sifdev.h>

// Engine independent.
#include "ChunkSystem/Asura_Chunk_ResourceFile.h"
#include "Core/Asura_Handle.h"
#include "Debugging/Asura_Assert.h"
#include "Debugging/Asura_Logfile.h"
#include "FileAccess/Asura_FileAccess.h"

// Engine platform-specific.
#include "Core/Asura_PlatformBasics.h"

// C L A S S   I M P L E M E N T A T I O N S /////////////////////////////////

bool Asura_Chunk_ResourceFile_PS2_IRX::Process(Asura_Handle hFile)
{
    // (MP - 19.09.2002) Ensure buffer is cache-aligned to speed loading and
    // to avoid problems when we DMA it below.
    const u_int uCacheAlignedFileSize = (((this->FileSize + 63) >> 6) << 6);
    
	u_char* pucTempBuffer = reinterpret_cast<u_char*>( Asura_AllocateAlign(uCacheAlignedFileSize, ASURA_ALIGN_128_BYTE) );
    Asura_Assert( pucTempBuffer, "failed to allocate EE temp buffer for IRX load" );
    // (MP - 19.09.2002) Not much in actually dealing with this; we're pretty
    // much screwed if we can't load the IRXs (and if we're out of memory
    // this early.)

	Asura_FileAccess::ReadData( hFile, Asura_PS2_UncachedAcceleratedMem(pucTempBuffer), this->FileSize );

    void* pvIopAddr = sceSifAllocIopHeap( this->FileSize );
    Asura_Assert( pvIopAddr, "failed to create IOP temp buffer for IRX load");
    // (MP - 19.09.2002) See comment above.

    // Send module data to IOP.
    sceSifDmaData xSifDmaData;
    xSifDmaData.data = reinterpret_cast<u_int>( pucTempBuffer );
    xSifDmaData.addr = reinterpret_cast<u_int>( pvIopAddr );
    xSifDmaData.size = this->FileSize;
    xSifDmaData.mode = 0;
    sceSifSetDma( &xSifDmaData, 1 );

    // Load module buffer.
    if( sceSifLoadModuleBuffer(pvIopAddr, 0, NULL) < 0 ) 
    {
        // Loading failed: Error processing
        Asura_OutputToDebugger_Retail(("failed to load IRX from resource file.\n"));
    }

    sceSifFreeIopHeap( pvIopAddr );
    Asura_DeallocateAlign( pucTempBuffer );

    return true;
}
