//////////////////////////////////////////////////////////////////////////////
//
// Asura_Chunk_ResourceFile_PS2_PartiallyStreamedSounds.cpp
//
// Copyright (c) 2003 Rebellion. All rights reserved.
//
// Description:
//
//     none
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

#ifdef ASURA_PS2_SUPPORT_PARTIALLY_STREAMED_SOUNDS

#include "Asura_Chunk_ResourceFile_PS2_PartiallyStreamedSounds.h"

// engine independent headers
#include "ChunkSystem/Asura_Chunk_Macros.h"
#include "Sound/Asura_Sound.h"

// engine PS2 headers
#include "Sound/Asura_PS2_Multistream.h"

// C L A S S   I M P L E M E N T A T I O N S /////////////////////////////////

bool Asura_Chunk_ResourceFile_PS2_PartiallyStreamedSounds::Process( Asura_Handle iFileHandle )
{
    int iVersion;
    ASURA_CHUNK_FILEEXTRACT( iFileHandle, iVersion, int );

    // Check version.
    if( iVersion > ASURA_PS2_PSS_CHUNK_VERSION )
    {
        Asura_Assert( false, "Chunk data is newer than this build. You need new code." );
    }
    else if( !iVersion )
    {
        Asura_Assert( false, "This version of the PSS chunk data is no longer supported. You will probably get more asserts about unrecognised chunk types now." );
    }

    int iNumEntries;
    ASURA_CHUNK_FILEEXTRACT( iFileHandle, iNumEntries, int );
    Asura_Assert( iNumEntries < ASURA_MAX_NUMBER_OF_LOADED_SOUNDS, "Too many PSSs." );

    Asura_PS2_Multistream::InitPartiallyStreamingSounds( iNumEntries, Asura_FileAccess::GetFilename(iFileHandle) );

	bool bSoundIDsAreDynamic = (Flags & uASURA_PS2_PSS_FLAG_SOUNDIDS_ARE_DYNAMIC) != 0;

    HdrEntry xHdrEntry;
    for( int i = 0; i < iNumEntries; ++i )
    {
        ASURA_CHUNK_FILEEXTRACT( iFileHandle, xHdrEntry, HdrEntry );

		Asura_PS2_Multistream::AddPartiallyStreamingSoundData( bSoundIDsAreDynamic ? ASURA_SID_DYNAMIC : xHdrEntry.iSoundID,
                                                               xHdrEntry.iFrequency,
                                                               xHdrEntry.iOffset1,
                                                               xHdrEntry.iSize1,
                                                               xHdrEntry.iOffset2,
                                                               xHdrEntry.iSize2,
															   bSoundIDsAreDynamic ? xHdrEntry.iSoundID : ASURA_HASH_ID_UNSET );
    }

    // Return false so that the main chunk loading code will skip the rest
    // of the data for us. It has an optimisation for where the data's at the
    // end of the file, which should be the case for PSSs.
    return false;
}

#endif // #ifdef ASURA_PS2_SUPPORT_PARTIALLY_STREAMED_SOUNDS
