//////////////////////////////////////////////////////////////////////////////
//
// Asura_Chunk_ResourceFile_PS2_SkinLODThresholds.cpp
//
// Created by Morgan Parry.
//
// Copyright (c) 2002 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

// Asura independent headers.
#include "ChunkSystem/Asura_Chunk_ResourceFile.h"
#include "FileAccess/Asura_FileAccess.h"
#include "Core/Asura_Handle.h"
#include "Core/Asura_Maths.h"

// Asura PS2 headers.
#include "ChunkSystem/Asura_Chunk_ResourceFile_PS2_SkinLODThresholds.h"
#include "Animation/Asura_PS2_Hierarchy.h"

// C L A S S   I M P L E M E N T A T I O N S /////////////////////////////////

bool Asura_Chunk_ResourceFile_PS2_SkinLODThresholds::Process( Asura_Handle hFile )
{
    // Read in skin name.
    u_int uSkinNameSizeInInts;
    const char* szSkinName = Asura_FileAccess::ReadString( hFile, NULL, &uSkinNameSizeInInts );

    Asura_PS2_SkinEntry* pxSkinEntry = Asura_PS2_SkinSystem::AddSkin( szSkinName );
    if( !pxSkinEntry ) { return false; }

    float* const pfThresholdArray = pxSkinEntry->m_afThresholdsSqd;

    const u_int uNumThresholds = (this->FileSize - (uSkinNameSizeInInts << 2)) >> 2;
    Asura_Assert( uNumThresholds <= ASURA_PS2_SKINSYSTEM_MAX_LODS_PER_SKIN, "" );

	// Read in data.
	if(uNumThresholds > ASURA_PS2_SKINSYSTEM_MAX_LODS_PER_SKIN)
	{
		Asura_FileAccess::ReadData( hFile, pfThresholdArray, ASURA_PS2_SKINSYSTEM_MAX_LODS_PER_SKIN * sizeof(float) );
		Asura_FileAccess::SkipData( hFile, (uNumThresholds - ASURA_PS2_SKINSYSTEM_MAX_LODS_PER_SKIN) * sizeof(float) );
	}
	else
	{
		Asura_FileAccess::ReadData( hFile, pfThresholdArray, (this->FileSize - (uSkinNameSizeInInts << 2)) );
	}

    // Never use other LODs.
    for( u_int u = uNumThresholds; u < ASURA_PS2_SKINSYSTEM_MAX_LODS_PER_SKIN; ++u )
    {
        pfThresholdArray[u] = 0.0f;
    }

    // Currently expect highest LOD to have a threshold of 0, ie. it should
    // be used unless any other thresholds are exceeded.
    Asura_Assert( pfThresholdArray[0] == 0.0f, "Highest LOD's threshold isn't 0." );

    // Also, we expect thresholds to be a strictly increasing sequence.
    for( u_int u = 1; u < uNumThresholds; ++u )
    {
        Asura_Assert( pfThresholdArray[u] > pfThresholdArray[u - 1], "Threshold sequence isn't strictly increasing." );
    }

    // Verify the values are vaguely sensible as well. This upper limit is
    // arbitrary but should be sufficient to detect junk data in most cases.
    for( u_int u = 0; u < uNumThresholds; ++u )
    {
        Asura_Assert( 0.0f <= pfThresholdArray[u] && pfThresholdArray[u] < 500.0f * 500.0f, "LOD threshold seems to be garbage." );
    }

    return true;
}
