//////////////////////////////////////////////////////////////////////////////
//
// Asura_Chunk_ResourceFile_PS2_StrippedSmoothSkin.cpp
//
// Created by Mike Healey.
//
// Copyright (c) 2002 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header.
#include "ChunkSystem/Asura_Chunk_ResourceFile_PS2_StrippedSmoothSkin.h"

// Engine independent.
#include "ChunkSystem/Asura_Chunk_ResourceFile.h"
#include "Core/Asura_Handle.h"
#include "FileAccess/Asura_FileAccess.h"
#include "Debugging/Asura_Logfile.h"

// Engine platform-specific.
#include "Animation/Asura_PS2_Hierarchy.h"
#include "Animation/Asura_PS2_SkinTypes.h"
#include "Core/Asura_PlatformBasics.h"

// C L A S S   I M P L E M E N T A T I O N S /////////////////////////////////

bool Asura_Chunk_ResourceFile_PS2_StrippedSmoothSkin::Process( Asura_Handle hFile )
{
    Asura_Assert( Version >= ASURA_PS2_SMOOTH_SKIN_FILE_FORMAT_VERSION, "This version of the stripped smooth skin chunk is no longer supported. Please re-export." );

    // Read in name.
    u_int uSkinNameSizeInInts;
    const char* szSkinName = Asura_FileAccess::ReadString( hFile, NULL, &uSkinNameSizeInInts );

    // Allocate buffer for data.
    // (MP - 19.09.2002) Ensure buffer is cache-aligned to speed loading.
    const u_int uRemainingBytes = FileSize - (uSkinNameSizeInInts << 2);
    const u_int uCacheAlignedBytes = (uRemainingBytes + 63) & ~63;
    void* pvBuffer = Asura_AllocateAlign( uCacheAlignedBytes, ASURA_ALIGN_64_BYTE );
    if( !pvBuffer )
    {
        Asura_OutputToDebugger_Retail(( "ERROR: Out of memory; couldn't allocate buffer for stripped smooth skin data." ));
        return false;
    }

    // Read data in.
    Asura_FileAccess::ReadData( hFile, Asura_PS2_UncachedAcceleratedMem(pvBuffer), uRemainingBytes );

    // Store it.
    bool bOK = Asura_PS2_SkinSystem::AddSmoothSkinLOD( szSkinName, pvBuffer, uRemainingBytes );
    if( !bOK )
    {
        Asura_DeallocateAlign( pvBuffer );
    }

    return true;
}
