//////////////////////////////////////////////////////////////////////////////
//
// Asura_Platform_Chunk_Mesh.cpp
//
// Copyright (c) 2002 Rebellion. All rights reserved.
//
// Description:
//
//     Platform implementations of mesh chunk processing functionality.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"
#include "ChunkSystem/Asura_Chunk_Mesh.h"


#include "Core/Asura_Handle.h"
#include "FileAccess/Asura_FileAccess.h"
#include "Core/Asura_Material_System.h"
#include "Mesh/Asura_Mesh.h"
#include "Mesh/Asura_Mesh_System.h"


// C L A S S   I M P L E M E N T A T I O N S /////////////////////////////////

bool Asura_Chunk_Mesh_Normals::Platform_Process( Asura_Handle hFile ) const
{
	const Asura_Char* szMeshname = Asura_FileAccess::ReadString( hFile );
	
	Asura_Mesh* pxMeshPtr = Asura_Mesh_System::GetMesh( szMeshname, true );

	if( pxMeshPtr->NormalsDataValid )
	{
		// Kevin Lea - 13/09/2001 15:52:55 - mesh already exists
		// this could be a naming problem (duplicates)
		// otherwise if this happens regularly we'll have to read off
		// the remainder of the chunk to keep things in sync
		Asura_Assertf( false, ("Mesh: '%s'  - Already exists", szMeshname) );
        Asura_FileAccess::SkipData( hFile, (5 * sizeof(Asura_Vector_3)) * NumberOfPolygons );
		return true;
	}
	Asura_Assert( pxMeshPtr->NumberOfPolygons == NumberOfPolygons, "Wrong number of polygons in normal data" );

	pxMeshPtr->NormalsDataPtr = new Asura_Mesh_Normals[NumberOfPolygons];
    for( int i = 0; i < NumberOfPolygons; ++i )
    {
	    Asura_FileAccess::ReadData( hFile, &pxMeshPtr->NormalsDataPtr[i].FaceNormal, sizeof(Asura_Vector_3) );

        // We don't care about vertex normals on PS2 because they're embedded
        // in our stripped environment.
        Asura_FileAccess::SkipData( hFile, 4 * sizeof(Asura_Vector_3) );
    }

	pxMeshPtr->NormalsDataValid = true;

	return true;
}

bool Asura_Chunk_Mesh_TextureInfo::Platform_Process( Asura_Handle hFile ) const
{
	const Asura_Char* szMeshname = Asura_FileAccess::ReadString( hFile );
	
	Asura_Mesh* pxMeshPtr = Asura_Mesh_System::GetMesh( szMeshname, true );

	if( pxMeshPtr->TextureDataValid )
	{
		// Kevin Lea - 13/09/2001 15:52:55 - mesh already exists
		// this could be a naming problem (duplicates)
		// otherwise if this happens regularly we'll have to read off
		// the remainder of the chunk to keep things in sync
		Asura_Assertf( false, ("Mesh: '%s'  - Already exists", szMeshname) );
        Asura_FileAccess::SkipData( hFile, (sizeof(int) + 8 * sizeof(float)) * NumberOfPolygons );
		return true;
	}

	pxMeshPtr->TextureDataPtr = new Asura_Mesh_TextureInfo[NumberOfPolygons];
	
	Asura_Mesh_TextureInfo* pxDestPtr = pxMeshPtr->TextureDataPtr;
	for( int i = 0; i < NumberOfPolygons; ++pxDestPtr, ++i )
	{
        // Read Material ID.
        Asura_FileAccess::ReadData( hFile, &pxDestPtr->m_iMaterialID, sizeof(int) );

        // We don't care about UVs on PS2 because they're embedded in our
        // stripped environment.
        Asura_FileAccess::SkipData( hFile, 8 * sizeof(float) );

        // Convert texture ID if necessary.
		if( -1 != pxDestPtr->m_iMaterialID )
		{
			pxDestPtr->m_iMaterialID = Asura_Material_System::ConvertIndex( pxDestPtr->m_iMaterialID );
		}
	}

	pxMeshPtr->TextureDataValid = true;
	return true;
}
