/////////////////////////////////////////////////////////////////////////////////////////
//
//	FILE		ASURA_PS2_OS.H
//
//	AUTHOR		Richard M. Smith (21 August 2001)
//
/////////////////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef	__ASURA_PS2_OS_H__
#define	__ASURA_PS2_OS_H__

#include "Core/Asura_Hash.h"
#include <eetypes.h>	// u_long128

// M A C R O S ///////////////////////////////////////////////////////////////

#ifdef ASURA_PS2_CDEMUL

    #define ASURA_PS2_FILEROOT   ""
	#define ASURA_PS2_CD_DEVICENAME	 "cdrom0:"

#else // not defined ASURA_PS2_CDEMUL

    // (MH - 10.05.2002) to make non cd builds work with atwinmon, 
    // use an array which gets filled out at startup time.
    //#define ASURA_PS2_FILEROOT ASURA_PS2_PCFILEROOT
    extern char ASURA_PS2_FILEROOT[256];

#endif // else not defined ASURA_PS2_CDEMUL

// projects can override this define if they need to
#ifndef ASURA_PS2_MAX_LOADED_DLLS
#define ASURA_PS2_MAX_LOADED_DLLS 2
#endif

// C O N S T A N T S /////////////////////////////////////////////////////////

const u_int ASURA_PS2_FULL_PATH_BFR_SIZE = 256;
    
// NRM - 12/03/03 - to get various middleware libraries to perform correctly (multistream & liblgkbm 
// in particular), we need to define the main app thread priority and then the middleware threads
// to have a higher priority (lower integer value)
const int ASURA_PS2_MAIN_THREAD_PRIORITY        = 60;
const int ASURA_PS2_MULTISTREAM_DTS_THREAD_PRIORITY = ASURA_PS2_MAIN_THREAD_PRIORITY - 3;
const int ASURA_PS2_MULTISTREAM_THREAD_PRIORITY = ASURA_PS2_MAIN_THREAD_PRIORITY - 2;
const int ASURA_PS2_LGKBM_THREAD_PRIORITY       = ASURA_PS2_MAIN_THREAD_PRIORITY - 1;
const int ASURA_PS2_LIBHTTP_THREAD_PRIORITY		= ASURA_PS2_MAIN_THREAD_PRIORITY - 1;

// C L A S S E S /////////////////////////////////////////////////////////////

class Asura_PS2_OS
{

    public:

        // C L A S S   M E T H O D S /////////////////////////////////////////

        static void Initialise( void );
        static void Reset( void );
        static void Shutdown( void );
        static void ParseCommandLine( int argc, char **argv );
        
#ifdef ASURA_PS2_SONY_SUBMISSION
        static inline bool IsTOOL( void )        { return false; }
        static inline bool IsTestKit( void )     { return false; }
        static inline bool IsNTSCTestKit( void ) { return false; }
		static void DecodeSceeErrNo( const int /* iErrNo */ ) { return; }
#else
		static void DecodeSceeErrNo( const int iErrNo );
        static inline bool IsTOOL( void )        { return s_bIsTOOL;        }
        static inline bool IsTestKit( void )     { return s_bIsTestKit;     }
        static inline bool IsNTSCTestKit( void ) { return s_bIsNTSCTestKit; }
#endif

#ifdef ASURA_PS2_CDEMUL
        static inline bool IsRunningUnderAtwinmon( void ) { return false; }
#else
        static inline bool IsRunningUnderAtwinmon( void ) { return s_bIsRunningUnderAtwinmon; }
#endif

        // Returns ptr to internal buffer. Each call will overwrite the
        // results of the previous one. The filename passed in should be
        // relative to the root; in particular, it should not include stuff
        // like "host0:", "cdrom0:" or ";1".
        static const char* GetFullPath( const char* szFilename, bool bPrependDeviceName = false, const char* szExtensionReplacement = NULL, bool bIsDirectory = false );
#ifdef ASURA_PS2_CDEMUL
        static const char* GetFullPathOnPC( const char* szFilename );
#endif

        // Since the above two calls trash the internal buffer, here's some
        // functionality for backing up and restoring that data to some
        // externally provided buffer. It's assumed the buffer is at least
        // ASURA_PS2_FULL_PATH_BFR_SIZE bytes in size.
        static void BackupFullPath( char* szBackupBfr );
        static void RestoreFullPath( char* szBackupBfr );

        static void UpdateMemoryUsageStats();   // to snapshot at a particular point
        // GetMemStats calls UpdateMemoryUsageStats()
        static void GetMemStats(int& iUsedMem, int& iMaxUsedMem, int& iFreeMem, int& iMinFreeMem);

		// (PSJE-14.01.2003)	Function to sleep the current thread for n milliseconds.
		static void DelayThread( int milliseconds );

		static int LoadIRXModule(const char *szFilename, int argc, const char *argv, bool bForceHost0 = false, const u_int uMaxRetries = 0, int* piResult = NULL);
		static int UnloadIRXModule(const int iId);
		static int UnloadIRXModuleByName(const char *szName);
		
#ifdef ASURA_PS2_SUPPORT_DLLS
		// WARNING: don't set a breakpoint on the call of any function in a DLL 
		// as it interferes with the debugger's ability to patch the jal address.

		// DLLs are identified by the hash ID of their original filename
		// stripped of path and extension.
		// call AllocateDllMem to get memory to load a Dll to.
		static bool IsDllLoaded( const char* szName );
		static u_long128* AllocateDllMem( u_int uSize );
		// call InitDll when you have loaded a DLL into that memory to initialise it.
		static void InitDll(const char* szName, u_long128* puDllAddr);
		// call ShutdownAndUnloadDll to free the memory the DLL was allocated.
		static void ShutdownAndUnloadDll(const char* szName);
#endif

#ifdef ASURA_SUPPORT_EXTERNAL_NETWORK
		static void PreparePowerOff(void);
		static void UndoPowerOff(void);
		static bool IsNetworkAdaptorConnected(void) { return s_bIsNetworkAdaptorConnected; }
		static void SetNetworkAdaptorConnected(bool bConnected) { s_bIsNetworkAdaptorConnected = bConnected; }
#endif
		
        static void DumpMemoryInfo( void );
        static int  GetMemAlloced( void );
        static int  GetMemFree( void );

        static u_char* LoadFileAligned( const char* szFilename, ASURA_ALIGN_ID eAlignment, u_int* puFileSizeInBytes = NULL );

		static bool IsMainThread( void );
		
    private:

#ifndef ASURA_PS2_SONY_SUBMISSION
        static void CheckForTOOLOrTestKit( void );
#endif

#ifdef ASURA_SUPPORT_EXTERNAL_NETWORK
		static void PowerOffThread(void *arg);
		static void PowerOffHandler(void *arg);
#endif
		
#ifdef ASURA_PS2_SUPPORT_DLLS
		// for internal use - returns index of DLL in loaded array, -1 if not.
		static int GetDllIndex( Asura_Hash_ID uID );
#endif
        // C L A S S   S T A T E /////////////////////////////////////////////

#ifndef ASURA_PS2_SONY_SUBMISSION
        static bool s_bIsTOOL;
        static bool s_bIsTestKit;
        static bool s_bIsNTSCTestKit;
#endif
#ifndef ASURA_PS2_CDEMUL
        static bool s_bIsRunningUnderAtwinmon;
#endif
#ifdef ASURA_SUPPORT_EXTERNAL_NETWORK
		static bool s_bIsNetworkAdaptorConnected;
#endif
		static int s_iMainThreadID;
			
        static int s_iUsedMem;
        static int s_iMaxUsedMem;
        static int s_iFreeMem;
        static int s_iMinFreeMem;

#ifdef ASURA_PS2_SUPPORT_DLLS
		// structure to hold info on loaded DLLs
		struct LoadedDllInfo
		{
			Asura_Hash_ID m_uID;
			u_long128* m_puAddress;
		};

		static LoadedDllInfo s_axLoadedDllInfo[ASURA_PS2_MAX_LOADED_DLLS];
		static u_int s_uNumLoadedDlls;
#endif

};


#endif // __ASURA_PS2_OS_H__



////////////////////////////////////// END OF FILE //////////////////////////////////////
