//////////////////////////////////////////////////////////////////////////////
//
// Asura_PlatformBasics.h
//
// By: Kevin Lea, Morgan Parry.
//
// Copyright (c) 2001 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef ___ASURA_PLATFORMBASICS_H___
#define ___ASURA_PLATFORMBASICS_H___ 1

// I N C L U D E S ///////////////////////////////////////////////////////////

// Standard independent.
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

// Standard platform-specific.
#include <eetypes.h>
#include <libvu0.h>

// Engine independent.
#include "Debugging/Asura_Assert.h"

// M A C R O S ///////////////////////////////////////////////////////////////

#define ASURA_PS2_UNCACHED_MEM_FLAG             (0x20000000)
#define ASURA_PS2_UNCACHED_ACCELERATED_MEM_FLAG (0x30000000)
#define ASURA_PS2_SPR_MEM_BASE                  (0x70000000)
#define ASURA_PS2_CACHED_MEM_MASK               (0x0fffffff)

#if _DEBUG
#define Asura_PS2_SPR_Alloc( __uQWOffset, __uNumQWs ) ( Asura_PS2_DebugSPR::Alloc( (__uQWOffset), (__uNumQWs), __SN_FILE__, __LINE__ ) )
#define Asura_PS2_SPR_Dealloc( __pxMem ) { Asura_PS2_DebugSPR::Dealloc( (__pxMem) ); }
#else
#define Asura_PS2_SPR_Alloc( __uQWOffset, __uNumQWs ) ( reinterpret_cast<Asura_PS2_QW*>(ASURA_PS2_SPR_MEM_BASE) + (__uQWOffset) )
#define Asura_PS2_SPR_Dealloc( __pxMem )
#endif

#define ASURA_PS2_SPR_SIZE_QWS 1024
#define ASURA_PS2_SPR_SIZE_BYTES (16*ASURA_PS2_SPR_SIZE_QWS)

// T Y P E S /////////////////////////////////////////////////////////////////

typedef char* LPTSTR ;

//typedef unsigned char  uint8;
//typedef unsigned short uint16;
//typedef unsigned int   uint32;
typedef unsigned long  uint64;

typedef char    Asura_Char;
typedef wchar_t Asura_UnicodeChar;

#ifndef _MSC_VER
#define ASURA_UNICODE_CHAR_DISTINCT_TYPE 1
#endif

union Asura_PS2_QW
{
    short         i16s[8];
    u_short       u16s[8];
    float         f32s[4];
    int           i32s[4];
    u_int         u32s[4];
    u_long        u64s[2];
    u_long128     u128;
    sceVu0FVECTOR Vu0FVec;
    sceVu0IVECTOR Vu0IVec;
}
__attribute__(( aligned(16) ));

// Macros for initialising the components of the union. Only work under GNU,
// though MSVC macros are provided so it still compiles. 
#ifdef _MSC_VER
    #define ASURA_PS2_QW_INIT_F32S( f1, f2, f3, f4 ) { 0 }
    #define ASURA_PS2_QW_INIT_I32S( i1, i2, i3, i4 ) { 0 }
    #define ASURA_PS2_QW_INIT_U32S( u1, u2, u3, u4 ) { 0 }
    #define ASURA_PS2_QW_INIT_U64S( u1, u2         ) { 0 }
    #define ASURA_PS2_QW_INIT_U128( u1             ) { 0 }
#else
    #ifndef __GNUC__
        #error This only supports GNU and MSVC!
    #endif
    
    #define ASURA_PS2_QW_INIT_F32S( f1, f2, f3, f4 ) { f32s: { f1, f2, f3, f4 } }
    #define ASURA_PS2_QW_INIT_I32S( i1, i2, i3, i4 ) { i32s: { i1, i2, i3, i4 } }
    #define ASURA_PS2_QW_INIT_U32S( u1, u2, u3, u4 ) { u32s: { u1, u2, u3, u4 } }
    #define ASURA_PS2_QW_INIT_U64S( u1, u2         ) { u64s: { u1, u2         } }
    #define ASURA_PS2_QW_INIT_U128( u1             ) { u128: { u1             } }
#endif

// So that you can specify the LL suffix required by GCC for 128-bit
// constants without upsetting MSVC...
#ifdef _MSC_VER
    #define ASURA_PS2_INIT_ULONG128( u ) (0)
#else
    #ifndef __GNUC__
        #error This only supports GNU and MSVC!
    #endif  
    #define ASURA_PS2_INIT_ULONG128( u ) (u)
#endif

// C L A S S E S /////////////////////////////////////////////////////////////

#if _DEBUG
class Asura_PS2_DebugSPR
{

    public:

        static void Initialise( void );
        static void Shutdown( void );

        static Asura_PS2_QW* Alloc( u_int uQWOffset, u_int uNumQWs, const char* szFile, int iLine );
        static void Dealloc( Asura_PS2_QW* pxMem );

        static bool IsAllocated( u_int uQWOffset, u_int uNumQWs );
        static void VerifyAllDeallocated( bool bForceAnyOutstandingToBeDeallocated );

};
#endif

// P R O T O T Y P E S   A N D   I N L I N E   F U N C T I O N S /////////////

template< typename T >
static inline T* Asura_PS2_CachedMem( T* p )
{
    return reinterpret_cast<T*>( reinterpret_cast<u_int>(p) & ASURA_PS2_CACHED_MEM_MASK );
}

// Identical to Asura_PS2_CachedMem() but makes more sense in certain
// contexts...
template< typename T >
static inline T* Asura_PS2_DMAMem( T* p )
{
    return Asura_PS2_CachedMem( p );
}

template< typename T >
static inline T* Asura_PS2_UncachedMem( T* __p )
{
    return reinterpret_cast<T*>( reinterpret_cast<u_int>(__p) | ASURA_PS2_UNCACHED_MEM_FLAG );
}

template< typename T >
static inline T* Asura_PS2_UncachedAcceleratedMem( T* __p )
{
    return reinterpret_cast<T*>( reinterpret_cast<u_int>(__p) | ASURA_PS2_UNCACHED_ACCELERATED_MEM_FLAG );
}

// Most of the wide string functions are available on our libraries but
// inexplicably some of them aren't in the header.
extern "C"
{
    int iswalpha( Asura_UnicodeChar wChar );
    int iswalnum( Asura_UnicodeChar wChar );
    int iswdigit( Asura_UnicodeChar wChar );
    int iswlower( Asura_UnicodeChar wChar );
    int iswspace( Asura_UnicodeChar wChar );
    int iswupper( Asura_UnicodeChar wChar );

    int wcsicmp( const Asura_UnicodeChar* wszA, const Asura_UnicodeChar* wszB );
    int wcsnicmp( const Asura_UnicodeChar* wszA, const Asura_UnicodeChar* wszB, size_t uMax );
};

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

#endif // #ifndef ___ASURA_PLATFORMBASICS_H___
