/////////////////////////////////////////////////////////////////////////////
//
// Asura_PS2_Breakpoint.cpp
//
// Copyright (c) 2003 Rebellion. All rights reserved.
//
// Description:
//
//	Code to allow the setting of data breakpoints at runtime
//	Code taken from EE Core User's Manual V6 9.3.3
//
//////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header.
#include "Debugging/Asura_PS2_Breakpoint.h"

#ifndef ASURA_PS2_SONY_SUBMISSION

void Asura_PS2_Breakpoint::SetDataBreakpoint( u_int uAddress, u_int uAddressMask, const bool bOnRead, const bool bOnWrite )
{
	const u_int uDBpClear = ~( (1<<30) |	// DRE
						 	   (1<<29) |	// DWE
//							   (1<<28) |	// DVE
							   (1<<21) |	// DUE
							   (1<<20) |	// DSE
							   (1<<19) |	// DKE
							   (1<<18) |	// DXE
							   (1<<16) |	// DTE
							   (1<<2)  |	// DWB
							   (1<<1) );	// DRB

	const u_int uDBpSet = ((bOnRead) ? (1<<30): 0)	|	// DRE = 1 : Enables breakpoint when reading
					 	  ((bOnWrite)? (1<<29): 0)	|	// DWE = 1 : Enables breakpoint when writting
						  (1<<21) |	// DUE = 1 : Enables breakpoint in user mode
						  (1<<20) |	// DSE = 1 : Enables breakpoint in supervisor mode
						  (1<<19);// |	// DKE = 1 : Enables breakpoint in kernel mode
//						  (1<<18);	// DXE = 1 : Enables breakpoint in level 1 exception handler
	asm __volatile__(	".set noreorder					\n"	\
						"sync.l							\n"	\
						"mfbpc	$8						\n"	\
						"and	$8, $8, %0				\n"	\
						"mtbpc	$8						\n"	\
						"sync.p							\n"	\
						"mtdab	%1						\n"	\
						"mtdabm %2						\n"	\
						"or		$9, $8, %3				\n"	\
						"mtbpc	$9						\n"	\
						"sync.p							\n"	\
						".set reorder					\n"	\
						:									\
						: "r"(uDBpClear),					\
						  "r"(uAddress),					\
						  "r"(uAddressMask),				\
						  "r"(uDBpSet)						\
						: "$8", "$9" );
}

void Asura_PS2_Breakpoint::ClearDataBreakpoint()
{
    asm __volatile__(	".set noreorder			\n"	\
						"sync.l					\n"	\
						"mfbpc	$8				\n"	\
						"li		$9, 0x9fffffff	\n"	\
			  			"and	$8, $8, $9		\n"	\
						"mtbpc	$8				\n"	\
						"sync.p					\n"	\
	                    ".set reorder			\n"	\
                      :							\
                      :							\
					  : "$8", "$9"				);
}

void Asura_PS2_Breakpoint::SetInstructionBreakpoint( u_int uAddress, u_int uAddressMask/*=0xffffffff*/ )
{
	// This code set's a hardware breakpoint that will trigger the PA when it's set to EE trigger
	unsigned int rg;
    asm volatile("sync.l");
    rg = 0x87828000;
    asm __volatile__("mtbpc %0" : : "r" (rg));// set breakpoint command register
    rg = uAddress;
    asm __volatile__("mtiab %0" : : "r" (rg));// set breakpoint address
    rg = uAddressMask;
    asm __volatile__("mtiabm %0" : : "r" (rg));// set breakpoint mask
    asm __volatile__("sync.p");

}

#endif
