//////////////////////////////////////////////////////////////////////////////
//
// Asura_PS2_fmv.h
//
// Copyright (c) 2004-2005 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef ___ASURA_PS2_FMV_H___
#define ___ASURA_PS2_FMV_H___ 1

// I N C L U D E S ///////////////////////////////////////////////////////////

// Engine platform-specific.
#include "Core/Asura_PlatformBasics.h"
#include "Rendering/Asura_PS2_GS.h"

// C L A S S E S /////////////////////////////////////////////////////////////

class Asura_PS2_FMV
{

public:

    // T Y P E S /////////////////////////////////////////////////////////////

    enum PLAYRESULT
    {
        PLAYRESULT_FINISHED,
        PLAYRESULT_DIDNOTSTART,
        PLAYRESULT_ABORTED,
        PLAYRESULT_SKIPALL,
    };

    struct MovieSizeParams
    {
        u_int uDstX, uDstY, uDstWidth, uDstHeight;
        float fSrcS0, fSrcT0, fSrcS1, fSrcT1;
        u_int uDispWidth, uDispHeight; // Just for the callback's info; modifying them won't achieve anything.
    };

    typedef void(*RENDER_SUBTITLE_CB_FN)(float);
    typedef void(*MOVIE_SIZE_CB_FN)(MovieSizeParams*);

    // C L A S S   M E T H O D ///////////////////////////////////////////////

    static PLAYRESULT Play( const char* szFilename,
                            float fTimeBeforeSkipAllowed = 0.0f,
                            RENDER_SUBTITLE_CB_FN pfnRenderSubtitleCB = NULL,
                            Asura_PS2_VideoMode eForceVideoMode = ASURA_PS2_VIDEOMODE_USEDEFAULT,
                            u_int uAudioStreamIdx = 0,
                            MOVIE_SIZE_CB_FN pfnMovieSizeCB = NULL );

#ifdef ASURA_PS2_SUPPORT_DLLS
    static bool IsDLLLoaded( void );
    static bool LoadDLL( void );
    static void UnloadDLL( void );
#endif

    static u_int GetNumFramesBeforeSkipAllowed( void ) { return s_uFramesBeforeSkipAllowed; }
    static RENDER_SUBTITLE_CB_FN GetRenderSubtitleCB( void ) { return s_pfnRenderSubtitleCB; }
    static Asura_PS2_VideoMode GetVideoModeToUse( void ) { return s_eVideoModeToUse; }
    static u_int GetAudioStreamIdx( void ) { return s_uAudioStreamIdx; }
    static MOVIE_SIZE_CB_FN GetMovieSizeCB( void ) { return s_pfnMovieSizeCB; }

private:

    // C L A S S   M E T H O D S /////////////////////////////////////////////

    static PLAYRESULT PlayInternal( const char* szFilename );

    // C L A S S   S T A T E /////////////////////////////////////////////////
    
    static u_int                 s_uFramesBeforeSkipAllowed;
    static RENDER_SUBTITLE_CB_FN s_pfnRenderSubtitleCB;
    static Asura_PS2_VideoMode   s_eVideoModeToUse;
    static u_int                 s_uAudioStreamIdx;
    static MOVIE_SIZE_CB_FN      s_pfnMovieSizeCB;

};
// End class Asura_PS2_FMV.

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

#endif // #ifndef ___ASURA_PS2_FMV_H___
