/* SCEI CONFIDENTIAL
 "PlayStation 2" Programmer Tool Runtime Library Release 2.4.2
 */
/*
 *              Emotion Engine Library Sample Program
 *
 *                    - easy mpeg streaming -
 *
 *                         Version 0.10
 *                             ASCII
 *
 *      Copyright (C) 2001 Sony Computer Entertainment Inc.
 *                        All Rights Reserved.
 *
 *                           playpss.c
 *             main function for mpeg streaming program
 *
 *       Version        Date            Design   Log
 *  --------------------------------------------------------------------
 *       0.10           08.21.2001      mikedean the first version
 */

#include "Core/Asura.h"

#include "Core/Asura_PS2_OS.h"
#include "Debugging/Asura_Logfile.h"

#include <stdio.h>
#include <malloc.h>
#include <libmpeg.h>
#include <sifdev.h>

#include "Asura_PS2_ezmpegstr.h"
#include "Asura_PS2_playpss.h"
#include "Asura_PS2_strfile.h"

#define MPEG_BUFF_SIZE  (SCE_MPEG_BUFFER_SIZE(DISP_WIDTH, MAX_HEIGHT))
#define DEMUX_BUFF_SIZE (192 * 1024)
#define IOP_BUFF_SIZE   (12288 * 2) // 512 * 48
#define AUDIO_BUFF_SIZE (IOP_BUFF_SIZE * 2)

static playPssRsrcs rsrcs;

// ///////////////////////////////////////////////////////////////
//
//  Allocate EE memory
//
static void *memalign64(size_t size, const char *desc)
{
  void *ret = Asura_AllocateAlign(size, ASURA_ALIGN_64_BYTE);
  
  if (!ret) 
  {
      Asura_OutputToDebugger_Retail(("Cannot allocate %d bytes of EE memory for %s\n", size, desc));
  }
  else 
  {
      Asura_OutputToDebugger_Retail(("Allocated %7d bytes of  EE memory for %s\n", size, desc));
  }
  return ret;

}

// ///////////////////////////////////////////////////////////////
//
//  Allocate IOP memory
//
static int iopalloc(unsigned int size, char *desc)
{
  void *ret;
  
  if (!(ret = sceSifAllocIopHeap(size))) {
    Asura_OutputToDebugger_Retail(("Cannot allocate %d bytes of IOP memory for %s\n", size, desc));
  }
  else Asura_OutputToDebugger_Retail(("Allocated %7d bytes of IOP memory for %s\n", size, desc));
  return (int) ret;
}

// ///////////////////////////////////////////////////////////////////
//
//  MH additions: init/shutdown functions.
//
static bool PlayMovieInit()
{
    Asura_OutputToDebugger(("before FMV: max free mem on IOP = %d.\n", sceSifQueryMaxFreeMemSize()));

  rsrcs.mpegBuff      = (u_char*) memalign64(MPEG_BUFF_SIZE, "mpeg work area");
  rsrcs.mpegBuffSize  =            MPEG_BUFF_SIZE;
  
  rsrcs.rgb32 = (sceIpuRGB32 *)
    memalign64(MAX_MBX * MAX_MBY * sizeof(sceIpuRGB32), "rgb32 buffer");
  
  rsrcs.path3tag      = (u_int *) memalign64(PATH3_TAG_SIZE(MAX_MBX, MAX_MBY),
				   "path 3 tags");
  
  rsrcs.demuxBuff     = (char *) memalign64(DEMUX_BUFF_SIZE, "demux buffer");
  rsrcs.demuxBuffSize =            DEMUX_BUFF_SIZE;
  
  rsrcs.audioBuff     = (u_char *) memalign64(AUDIO_BUFF_SIZE, "audio buffer");
  rsrcs.audioBuffSize =            AUDIO_BUFF_SIZE;
  
  rsrcs.iopBuff       = iopalloc(IOP_BUFF_SIZE, "audio buffer");
  rsrcs.iopBuffSize   =          IOP_BUFF_SIZE;
  
  rsrcs.zeroBuff      = (u_char *)memalign64(ZERO_BUFF_SIZE, "zero buffer");  
  rsrcs.iopZeroBuff   = iopalloc  (ZERO_BUFF_SIZE, "zero buffer");

  if (rsrcs.mpegBuff && rsrcs.rgb32 && rsrcs.path3tag && rsrcs.demuxBuff && rsrcs.audioBuff 
      && rsrcs.iopBuff && rsrcs.zeroBuff && rsrcs.iopZeroBuff)
  {
      return true;
  }
  else
  {
#if 1//_DEBUG
      Asura_OutputToDebugger_Retail(("movie resource alloc failed"));
#endif
      return false;
  }
}

static void PlayMovieShutdown()
{
    sceSifFreeIopHeap( reinterpret_cast<void*>(rsrcs.iopZeroBuff) );
    Asura_DeallocateAlign( rsrcs.zeroBuff );
    sceSifFreeIopHeap( reinterpret_cast<void*>(rsrcs.iopBuff) );
    Asura_DeallocateAlign( rsrcs.audioBuff );
    Asura_DeallocateAlign( rsrcs.demuxBuff );
    Asura_DeallocateAlign( rsrcs.path3tag );
    Asura_DeallocateAlign( rsrcs.rgb32 );
    Asura_DeallocateAlign( rsrcs.mpegBuff );

    Asura_OutputToDebugger(("after FMV: max free mem on IOP = %d.\n", sceSifQueryMaxFreeMemSize()));
}

Asura_PS2_FMV::PLAYRESULT Asura_PS2_FMV::PlayInternal( const char* szFilename )
{
	// //////////////////////////////////////
	// 
	//  Allocate resources required for playback
	// 
	bool ok = PlayMovieInit();

	PLAYRESULT eRetVal = PLAYRESULT_DIDNOTSTART;
	if (ok)
	{
	  char* filename = const_cast<char*>(Asura_PS2_OS::GetFullPath(szFilename, true));
	  if (!strFileOpen(filename)) 
	  {
		ok = false;
	  }

	  if (ok)
	  {
		  eRetVal = playPss(strFileRead, &rsrcs);

		  strFileClose();
	  }
	}
	// //////////////////////////////////////
	// 
	//  Deallocate resources
	//     
	PlayMovieShutdown();

	return eRetVal;
}
