/* SCEI CONFIDENTIAL
 "PlayStation 2" Programmer Tool Runtime Library Release 2.4.2
 */
/*
 *              Emotion Engine Library Sample Program
 *
 *                    - easy mpeg streaming -
 *
 *                         Version 0.10
 *                             ASCII
 *
 *      Copyright (C) 2001 Sony Computer Entertainment Inc.
 *                        All Rights Reserved.
 *
 *                           playpss.c
 *             main function for mpeg streaming program
 *
 *       Version        Date            Design   Log
 *  --------------------------------------------------------------------
 *       0.10           08.21.2001      mikedean the first version
 *       0.20           09.04.2001      mikedean timing change, buffer bugfix
 */

#include "Core/Asura.h"

// This file's header.
#include "FMV/Asura_PS2_playpss.h"

// Stdlib independent.
#include <stdio.h>
#include <string.h>

// Stdlib platform-specific.
#include <eekernel.h>
#include <libgraph.h>
#include <libipu.h>
#include <libmpeg.h>
#include <libsdr.h>
#include <sdrcmd.h>

// Engine independent.
#include "Core/Asura_UserInput.h"
#include "Debugging/Asura_Logfile.h"
#include "Sound/Asura_Sound.h"

// Engine platform-specific.
#include "Core/Asura_PS2_DMA.h"
#include "FMV/Asura_PS2_audiodec.h"
#include "FMV/Asura_PS2_ezmpegstr.h"
#include "Input/Asura_PlatformInput.h"
#include "Rendering/Asura_Font_System.h"
#include "Rendering/Asura_PlatformRenderStates.h"
#include "Rendering/Asura_PS2_2D_Graphics.h"
#include "Rendering/Asura_PS2_DeferredRendering.h"
#include "Rendering/Asura_PS2_GS.h"
#include "Rendering/Asura_PlatformTextureManagement.h"
#include "Rendering/Asura_Viewport.h"

#include "ProjectSettings/Asura_PS2_ProjectControllerDefines.h"
#if ASURA_PS2_SUPPORT_GUNCON2
#include "Input/Asura_PS2_USBGuncon2.h"
#endif

// ////////////////////////////////////////////////////////////////
//
//  Definitions
//
#define READCHUNKSIZE   16384
#define DMACHUNKSIZE    4096          // bytes
#define PK_SIZE         (MAX_MBX * MAX_MBY * 6 + 10)
#define P3TAGBUFF1      0
#define P3TAGBUFF2      (bound(PK_SIZE * 4, 64))
#define CSCPERPIX       (0.000151f)   // number of scanlines to CSC one pixel
#define FILLBUFFMAXTIME (25)          // max scanlines that fillbuff can need
                                      // (empirical)

// ////////////////////////////////////////////////////////////////
//
//  Variables
//
static int isStarted;
static AudioDec audioDec;
static int (*fileRead) (void *, int);
static int bgDuration;

// ////////////////////////////////////////////////////////////////
//
// Buffer for multiplexed pss data
// 
static char* muxBuff = NULL;
static int   muxBuffFullness;

// ////////////////////////////////////////////////////////////////
// 
//  Data for uncached access
// 
static u_int *path3tag;

// ////////////////////////////////////////////////////////////////
//
//  Buffer for demultiplexed m2v data (uncached access)
//
//  Regions: times are relative to DMA and circular wrt BuffStart, BuffSize
//
//  demuxBuffPresent -> demuxBuffFuture   = Area currently being processed
//  demuxBuffFuture  -> demuxBuffPast     = Demuxed data ready for processing
//  demuxBuffPast    -> demuxBuffPresent  = Processed data
//
static char *demuxBuff;
static int   demuxBuffSize;
static char *demuxBuffPast;
static char *demuxBuffPresent;
static char *demuxBuffFuture;


// ////////////////////////////////////////////////////////////////
//
//  Function declarations
//
static int videoCallback(sceMpeg *mp, sceMpegCbDataStr *cbstr, void *data);
static int audioCallback(sceMpeg *mp, sceMpegCbDataStr *cbstr, void *data);
static int nodataCallback(sceMpeg *mp, sceMpegCbData *cbstr, void *data);
static int backgroundCallback(sceMpeg *mp, sceMpegCbData *cbstr, void *data);
static int errorCallback(sceMpeg *mp, sceMpegCbData *cbstr, void *data);

static int fillBuff(sceMpeg *mp,int blocking);

// (MP - 13.10.2005) Unnecessary since we always change video mode now.
//static void clearBackground( u_int uWidth, u_int uHeight, u_int uColour );

static void UpdateSubtitles( u_int uFrameNumber );

// ///////////////////////////////////////////////////////////////////
// 
//  Play back pss file
// 
Asura_PS2_FMV::PLAYRESULT playPss(int (*cbFileRead) (void *, int), playPssRsrcs *rsrcs)
{
    const Asura_PS2_VideoMode eVideoModeToUse = Asura_PS2_FMV::GetVideoModeToUse();

    const u_int uDispHeight = ( ASURA_PS2_VIDEOMODE_PAL == eVideoModeToUse ? DISP_HEIGHT_PAL : DISP_HEIGHT_NTSC ); 

    const Asura_PS2_VideoMode ePrevVideoMode = Asura_PS2_GS::GetVideoMode();
	const u_int uPrevScreenWidth      = Asura_PS2_GS::GetScreenWidth();
    const u_int uPrevScreenHeight     = Asura_PS2_GS::GetScreenHeight();
    const u_int uPrevFrontBfrBitDepth = Asura_PS2_GS::GetDispBfrBitDepth();

    // (MP - 13.10.2005) Always change video mode, since this will also deal
    // with clearing VRAM for us. (We didn't used to unless settings had
    // changed.)
    // (MP - 28.11.2005) It will also ensure deferred rendering is complete.
    Asura_PS2_GS::ChangeVideoMode( eVideoModeToUse, DISP_WIDTH, uDispHeight, 32 );

    // Any proper texture uploads we do, eg. when rendering subtitles, need
    // to go along PATH2 to avoid synchronisation issues with our upload of
    // the movie image data.
    Asura_PS2_TextureManagement::ForcePATH2Uploads( true );

  sceMpeg mp;
  int drawid = 1;
  sceGsDispEnv disp;

// (MP - 12.10.2005) I don't think there's any need to do this again?
//// //////////////////////////////////////
//// 
////  Initialize GS
//// 
//sceGsSyncV(0);
//sceGsResetGraph(0, SCE_GS_INTERLACE, Asura_PS2_GS::VideoModeIsPal() ? SCE_GS_PAL : SCE_GS_NTSC, SCE_GS_FIELD);
  
//  sceGsResetPath();
  path3tag = rsrcs->path3tag;
  
  // //////////////////////////////////////
  // 
  //  Initialize IPU and MPEG library
  // 
  sceMpegInit();
  
  // /////////////////////////////
  // 
  //  Initialize audio decoder
  // 
//  sceSdRemoteInit();
//  sceSdRemote(1, rSdInit, SD_INIT_COLD);
  
  audioDecCreate(&audioDec,
		 rsrcs->audioBuff,   rsrcs->audioBuffSize,
		 rsrcs->iopBuff,     rsrcs->iopBuffSize,
		 rsrcs->zeroBuff,    rsrcs->iopZeroBuff,    ZERO_BUFF_SIZE);
  
  isStarted = 0;

// (MP - 13.10.2005) Unnecessary since we always change video mode now. (See
// comment above.)
//// ////////////////////////////
//// 
////  Clear background
//// 
//clearBackground( DISP_WIDTH, uDispHeight, 0x80000000 );
  
  // ////////////////////////////
  // 
  //  Create MPEG decoder
  // 
  sceMpegCreate(&mp, rsrcs->mpegBuff, rsrcs->mpegBuffSize);
  
  sceMpegAddStrCallback(&mp, sceMpegStrM2V, 0,
			(sceMpegCallback) videoCallback, NULL);
  
  sceMpegAddStrCallback(&mp, sceMpegStrPCM, Asura_PS2_FMV::GetAudioStreamIdx(),
			(sceMpegCallback) audioCallback, NULL);
  
  sceMpegAddCallback(&mp, sceMpegCbNodata, nodataCallback, NULL);
  sceMpegAddCallback(&mp, sceMpegCbBackground, backgroundCallback, NULL);
  sceMpegAddCallback(&mp, sceMpegCbError, errorCallback, NULL);
    
  // ////////////////////////////
  // 
  //  Initialise read buffer for multiplexed pss data
  // 
  Asura_Assert( !muxBuff, "" );
  muxBuff = static_cast<char*>( Asura_AllocateAlign(READCHUNKSIZE, ASURA_ALIGN_64_BYTE) );
  muxBuffFullness = 0;
  
  // ////////////////////////////
  // 
  //  Initialise buffer for demultiplexed m2v data
  // 
  demuxBuff        = rsrcs->demuxBuff;
  demuxBuffSize    = rsrcs->demuxBuffSize;
  demuxBuffPast    = demuxBuff + 16;
  demuxBuffPresent = demuxBuff;
  demuxBuffFuture  = demuxBuff + 16;
  
  // ////////////////////////////
  // 
  //  Prebuffer (bounded by which of the m2v and pcm buffers fills first)
  // 
  fileRead = cbFileRead;
  while (fillBuff(&mp, 1));
  
  // ////////////////////////////
  // 
  //  Initialise display
  // 

    const u_int uFBP0 = 0;
    const u_int uFBP1 = (bound(DISP_WIDTH, 64) * bound(uDispHeight, 32)) >> 11;

    sceGsSyncV( 0 );
    sceGsSetDefDispEnv(&disp, SCE_GS_PSMCT32, DISP_WIDTH, uDispHeight, 0, 0);
    disp.dispfb.FBW = bound(DISP_WIDTH, 64) / 64;
    sceGsPutDispEnv(&disp);

    // Since we don't use sceGsPutDrawEnv() here, we'd better setup a few
    // registers here that generally we assume are done for us.
    {
        const u_int uNumRegisters = 3;
        const u_int uQWCForDMA    = 1 + uNumRegisters;
        const u_int uTotalQWs     = 1 + uQWCForDMA;

        Asura_PS2_QW* const pxStart = Asura_PS2_DeferredRendering::GetVifBuffer( uTotalQWs );
        Asura_PS2_QW* pxOut = pxStart;

        (reinterpret_cast<Asura_PS2_DmaTag*>(pxOut++))->Set( uQWCForDMA,
                                                             Asura_PS2_DmaTag::PCE_NOCHANGE,
                                                             Asura_PS2_DmaTag::ID_CNT,
                                                             false,
                                                             0,
                                                             false,
                                                             SCE_VIF1_SET_NOP(false),
                                                             SCE_VIF1_SET_DIRECT(uQWCForDMA, false) );

        Asura_PS2_GifTag* const pxGifTag = reinterpret_cast<Asura_PS2_GifTag*>( pxOut++ );
        pxGifTag->SetExceptRegs( uNumRegisters, false, false, 0, SCE_GIF_PACKED, 1 );
        pxGifTag->uAllRegs = SCE_GIF_PACKED_AD;

        pxOut->u64s[0] = SCE_GS_SET_PRMODECONT( 1 );
        pxOut->u64s[1] = SCE_GS_PRMODECONT;
        ++pxOut;
        pxOut->u64s[0] = SCE_GS_SET_DTHE( 0 );
        pxOut->u64s[1] = SCE_GS_DTHE;
        ++pxOut;
        pxOut->u64s[0] = SCE_GS_SET_COLCLAMP( 1 );
        pxOut->u64s[1] = SCE_GS_COLCLAMP;
        ++pxOut;

        Asura_Assert( static_cast<u_int>(pxOut - pxStart) == uTotalQWs, "") ;
    }

    Asura_PS2_GS::SetupViewport( -1 );
	Asura_PS2_DeferredRendering::SendDeferredRenderingBlock();
	Asura_PS2_DeferredRendering::WaitForDeferredRendering();

  // ////////////////////////////
  // 
  //  Wait for even field
  // 
  while (sceGsSyncV(0) == 1);
  
   // Keep track of the frame number for subtitles
  u_int uFrameNumber = 0;

  // (MH - 07.02.2003) return value
  Asura_PS2_FMV::PLAYRESULT eRetVal = Asura_PS2_FMV::PLAYRESULT_FINISHED;
  
  // ////////////////////////////
  // 
  //  Main loop to decode MPEG bitstream
  //
  const u_int uNumFramesBeforeSkipAllowed = Asura_PS2_FMV::GetNumFramesBeforeSkipAllowed();
  while (!sceMpegIsEnd(&mp)) 
  {
        if( uFrameNumber >= uNumFramesBeforeSkipAllowed )
        {         
            Asura_UserInput::Update();

#if ASURA_PS2_SUPPORT_GUNCON2
            // (NRM - 25.11.2003) update guncons, updates called around the vsync
            Asura_PS2_Input::TranslateGuncon2(0);
            Asura_PS2_Input::TranslateGuncon2(1);
#endif

            if( Asura_UserInput::DebouncedKeyPressed(ASURA_PS2_KEYID_PAD1_START)
                || Asura_UserInput::DebouncedKeyPressed(ASURA_PS2_KEYID_PAD2_START) )
            {
                eRetVal = Asura_PS2_FMV::PLAYRESULT_SKIPALL;
                break;
            }

            if( Asura_UserInput::DebouncedKeyPressed(ASURA_PS2_KEYID_PAD1_CROSS)
                || Asura_UserInput::DebouncedKeyPressed(ASURA_PS2_KEYID_PAD2_CROSS) )
            {
                 eRetVal = Asura_PS2_FMV::PLAYRESULT_ABORTED;
                 break;
            }

#if ASURA_PS2_SUPPORT_GUNCON2
            // Additional Guncon checks.

            if (Asura_UserInput::KeyPressed(ASURA_PS2_KEYID_GUNCON1_A) && 
                Asura_UserInput::KeyPressed(ASURA_PS2_KEYID_GUNCON1_B) && 
                Asura_UserInput::DebouncedKeyPressed(ASURA_PS2_KEYID_GUNCON1_TRIGGER))
            {
                eRetVal = Asura_PS2_FMV::PLAYRESULT_SKIPALL;
                break;
            }

            if (Asura_UserInput::KeyPressed(ASURA_PS2_KEYID_GUNCON2_A) && 
                Asura_UserInput::KeyPressed(ASURA_PS2_KEYID_GUNCON2_B) && 
                Asura_UserInput::DebouncedKeyPressed(ASURA_PS2_KEYID_GUNCON2_TRIGGER))
            {
                eRetVal = Asura_PS2_FMV::PLAYRESULT_SKIPALL;
                break;
            }

            if( Asura_UserInput::DebouncedKeyPressed(ASURA_PS2_KEYID_GUNCON1_TRIGGER)
                || Asura_UserInput::DebouncedKeyPressed(ASURA_PS2_KEYID_GUNCON2_TRIGGER) )
            {
                 eRetVal = Asura_PS2_FMV::PLAYRESULT_ABORTED;
                 break;
            }
#endif
        }
			         
    //*T0_MODE  = 1 << 7 | 3;  // Count hsyncs
	//*T0_COUNT = 0;
	// (MH - 04.06.2004) we already set the mode. 
	// I've switched from t0 to t1 to avoid a conflict with the logitech lgkbm library.
	Asura_Assert(*T1_MODE == ((1<<7) | 3), "Timer 1 not set correctly");
	*T1_COUNT = 0;
		
    // ////////////////////////////
    // 
    //  Get decoded picture in sceIpuRGB32 format
    // 
    while (sceMpegGetPicture(&mp, rsrcs->rgb32, MAX_MBX * MAX_MBY) < 0) {
      Asura_OutputToDebugger_Retail( ("sceMpegGetPicture failed\n"));
      while (1);
    }
  
    // ////////////////////////////
    // 
    //  Initialize DMA tags for path3 (uncached access)
    // 
    if( mp.frameCount == 0 )
    {
        // (MP - 11.10.2005) I've changed this to upload the image data to
        // spare memory above the front and back buffers. We'll use it for a
        // textured sprite shortly.
        int iY = bound(uDispHeight, 32) << 1;
        setLoadImageTags( &path3tag[P3TAGBUFF1], rsrcs->rgb32, 0, iY, mp.width, mp.height );
        setLoadImageTags( &path3tag[P3TAGBUFF2], rsrcs->rgb32, 0, iY, mp.width, mp.height );

        bgDuration = (int)((float)(mp.width * mp.height) * CSCPERPIX);

        scePrintf( "movie is %dx%d\n", mp.width, mp.height );
    }
    
    // ////////////////////////////
    // 
    //  Load decoded image to GS via path3
    // 
    sceGsSyncPath(0, 0);
    loadImage(drawid ? &path3tag[P3TAGBUFF2] : &path3tag[P3TAGBUFF1]);
    
    // ////////////////////////////
    //
    //  Send audio data to IOP
    //
    audioDecSendToIOP(&audioDec);
    
    if (!isStarted && audioDecIsPreset(&audioDec)) {
      audioDecStart(&audioDec);
      isStarted = 1;
    }

	const u_int uNumRegisters = Asura_PlatformRenderStates::NUM_RENDERSTATE_PACKET_QWS_EXCL_GIF_TAG + 13;
    const u_int uQWCForDMA    = 1 + uNumRegisters;
    const u_int uTotalQWs     = 1 + uQWCForDMA;

    Asura_PS2_QW* const pxStart = Asura_PS2_DeferredRendering::GetVifBuffer( uTotalQWs );
    Asura_PS2_QW* pxOut = pxStart;

    // DMA tag. We need to wait for the PATH3 image upload started above to
    // complete before we start rendering this so include a FLUSHA. Since
    // we mess around for a while before sending this and the PATH3 transfer
    // was kicked off directly and immediately, we shouldn't need to worry
    // about the more exotic synchronisation issues involved in the main
    // texture management routines.
    (reinterpret_cast<Asura_PS2_DmaTag*>(pxOut++))->Set( uQWCForDMA,
                                                         Asura_PS2_DmaTag::PCE_NOCHANGE,
                                                         Asura_PS2_DmaTag::ID_CNT,
                                                         false,
                                                         0,
                                                         false,
                                                         SCE_VIF1_SET_FLUSHA(false),
                                                         SCE_VIF1_SET_DIRECT(uQWCForDMA, false) );

    // PRIM setting for upcoming sprite. We'll jam it in the GIF tag.
    u_long uPRIM = SCE_GS_SET_PRIM( SCE_GS_PRIM_SPRITE, 0, true, false, false, false, 1, 0, 0 );

    // GIF tag.
    Asura_PS2_GifTag* pxGIFTag = reinterpret_cast<Asura_PS2_GifTag*>( pxOut++ );
    pxGIFTag->SetExceptRegs( uNumRegisters, true, true, uPRIM, SCE_GIF_PACKED, 1 );
    pxGIFTag->uAllRegs = SCE_GIF_PACKED_AD;

    // Reset renderstate, and specifically z-buffer stuff, since any 2D
    // rendering we do (for subtitles) will interfere with it.
    Asura_PlatformRenderStates::CreateDefaultRenderStatePacket( Asura_PlatformRenderStates::CONTEXT_1, pxOut, false, true, false );
    Asura_PlatformRenderStates::SetPixelTest( false, false );
    Asura_PlatformRenderStates::SetTextureClamp( true );
    Asura_PlatformRenderStates::ForgetMyBuffer( pxOut );
    pxOut += Asura_PlatformRenderStates::NUM_RENDERSTATE_PACKET_QWS_EXCL_GIF_TAG;

    // FRAME.
    pxOut->u64s[0] = SCE_GS_SET_FRAME_1( (drawid ? uFBP1 : uFBP0), // FBP   - Frame buffer base pointer as word address / 2048.
                                         disp.dispfb.FBW,          // FBW   - Frame buffer width in units of pixels / 64.
                                         disp.dispfb.PSM,          // PSM   - Frame buffer storage format.
                                         0 );                      // FBMSK - Frame buffer write mask - allow writing to all
    pxOut->u64s[1] = SCE_GS_FRAME_1;
    ++pxOut;

    int iSpriteX, iSpriteY, iSpriteWidth, iSpriteHeight;       

    const Asura_PS2_GS::ViewportParams& xViewportParams = Asura_PS2_GS::GetViewportParamsForFullScreenRendering();
    const int iTransX = xViewportParams.sPrimTransX;
    const int iTransY = xViewportParams.sPrimTransY;

    // Render a textured sprite, using the image data we just uploaded above
    // as our sprite.
    {
        // TEX1.
        pxOut->u64s[0] = SCE_GS_SET_TEX1( 1, 0, SCE_GS_LINEAR, SCE_GS_LINEAR, 0, 0, 0 );
        pxOut->u64s[1] = SCE_GS_TEX1_1;
        ++pxOut;

        // TEXFLUSH.
        pxOut->u64s[1] = SCE_GS_TEXFLUSH;
        ++pxOut;

        // TEX0.
        pxOut->u64s[0] = SCE_GS_SET_TEX0( (uFBP1 + (uFBP1 - uFBP0)) << 5,
                                          disp.dispfb.FBW,
                                          SCE_GS_PSMCT32,
                                          Asura_Maths::GetCeilLog2(DISP_WIDTH),
                                          Asura_Maths::GetCeilLog2(uDispHeight),
                                          0,
                                          SCE_GS_DECAL,
                                          0, 0, 0, 0, 0 );
        pxOut->u64s[1] = SCE_GS_TEX0_1;
        ++pxOut;
                
// (MP - 25.08.2007) What we did on Rogue.
//      // Calculate sprite details.
//      {
//          iSpriteX     = (DISP_WIDTH - mp.width) >> 1;
//          iSpriteWidth = mp.width;
//
//          if( Asura_Camera::IsWidescreen() )
//          {
//              // (MP - 11.10.2005) Rogue-specific change. We're rendering
//              // out our FMVs with a 16:9 aspect ratio so on a 16:9 display
//              // we want to fill the whole buffer. (So like anamorphic
//              // format on DVDs.)
//              iSpriteY      = 0;
//              iSpriteHeight = uDispHeight;
//          }
//          else
//          {
//              iSpriteY      = (uDispHeight - mp.height) >> 1;
//              iSpriteHeight = mp.height;
//          }
//
//          iSpriteX      <<= 4;
//          iSpriteY      <<= 4;
//          iSpriteWidth  <<= 4;
//          iSpriteHeight <<= 4;
//
//          iSpriteX += iTransX;
//          iSpriteY += iTransY;
//      }

        Asura_PS2_FMV::MovieSizeParams xMovieSizeParams;

        // Fill out defaults. This just renders the movie 1:1 centred in
        // the buffer.
        xMovieSizeParams.uDstX       = (DISP_WIDTH  - mp.width ) >> 1;
        xMovieSizeParams.uDstY       = (uDispHeight - mp.height) >> 1;
        xMovieSizeParams.uDstWidth   = mp.width;
        xMovieSizeParams.uDstHeight  = mp.height;
        xMovieSizeParams.fSrcS0      = 0.0f;
        xMovieSizeParams.fSrcT0      = 0.0f;
        xMovieSizeParams.fSrcS1      = 1.0f;
        xMovieSizeParams.fSrcT1      = 1.0f;
        xMovieSizeParams.uDispWidth  = DISP_WIDTH;
        xMovieSizeParams.uDispHeight = uDispHeight;

        // See if any callback wants to have a say.
        Asura_PS2_FMV::MOVIE_SIZE_CB_FN pfnMovieSizeCB = Asura_PS2_FMV::GetMovieSizeCB();
        if( pfnMovieSizeCB )
        {
            pfnMovieSizeCB( &xMovieSizeParams );
        }

        // Translate STs to UVs.
        u_int uSrcU0 = Asura_Maths::FloatToInt_Near( xMovieSizeParams.fSrcS0 * mp.width  * 16.0f );
        u_int uSrcV0 = Asura_Maths::FloatToInt_Near( xMovieSizeParams.fSrcT0 * mp.height * 16.0f );
        u_int uSrcU1 = Asura_Maths::FloatToInt_Near( xMovieSizeParams.fSrcS1 * mp.width  * 16.0f );
        u_int uSrcV1 = Asura_Maths::FloatToInt_Near( xMovieSizeParams.fSrcT1 * mp.height * 16.0f );

        // And translate coordinates.
        iSpriteX      = (xMovieSizeParams.uDstX << 4) + iTransX;
        iSpriteY      = (xMovieSizeParams.uDstY << 4) + iTransY;
        iSpriteWidth  = xMovieSizeParams.uDstWidth  << 4;
        iSpriteHeight = xMovieSizeParams.uDstHeight << 4;

        // Top-left vertex.
        pxOut->u64s[0] = SCE_GS_SET_UV( uSrcU0, uSrcV0 );
        pxOut->u64s[1] = SCE_GS_UV;
        ++pxOut;
        pxOut->u64s[0] = SCE_GS_SET_XYZ( iSpriteX, iSpriteY, 0 );
        pxOut->u64s[1] = SCE_GS_XYZ2;
        ++pxOut;
        
        // Bottom-right vertex.
        pxOut->u64s[0] = SCE_GS_SET_UV( uSrcU1, uSrcV1 );
        pxOut->u64s[1] = SCE_GS_UV;
        ++pxOut;
        pxOut->u64s[0] = SCE_GS_SET_XYZ( iSpriteX + iSpriteWidth, iSpriteY + iSpriteHeight, 0 );
        pxOut->u64s[1] = SCE_GS_XYZ2;
        ++pxOut;

        // We need to ensure that's finished before we go uploading font
        // textures, etc., since there isn't room for both our temporary
        // movie image data and the usual texture buffers.
        pxOut->u64s[1] = SCE_GS_TEXFLUSH;
        ++pxOut;
    }

    // For good measure, clear any space below the movie, in case we're
    // rendering any subtitles down there.
    if( iSpriteY + iSpriteHeight < iTransY + static_cast<int>(uDispHeight << 4) )
    {
        pxOut->u64s[0] = SCE_GS_SET_PRIM( SCE_GS_PRIM_SPRITE, 0, false, false, false, false, 1, 0, 0 );
        pxOut->u64s[1] = SCE_GS_PRIM;
        ++pxOut;

        const float fOne = 1.0f;
        pxOut->u64s[0] = SCE_GS_SET_RGBAQ( 0, 0, 0, 0, reinterpret_cast<const u_int&>(fOne) );
        pxOut->u64s[1] = SCE_GS_RGBAQ;
        ++pxOut;

        pxOut->u64s[0] = SCE_GS_SET_XYZ( iTransX + (0 << 4), iSpriteY + iSpriteHeight, 0 );
        pxOut->u64s[1] = SCE_GS_XYZ2;
        ++pxOut;
        
        pxOut->u64s[0] = SCE_GS_SET_XYZ( iTransX + (DISP_WIDTH << 4), iTransY + (uDispHeight << 4), 0 );
        pxOut->u64s[1] = SCE_GS_XYZ2;
        ++pxOut;
    }
    else
    {
        for( u_int u = 0; u < 4; ++u )
        {
            (pxOut++)->u64s[1] = SCE_GS_NOP;
        }
    }

    Asura_Assert( static_cast<u_int>(pxOut - pxStart) == uTotalQWs, "" );

    UpdateSubtitles( uFrameNumber );

	Asura_PS2_DeferredRendering::SendDeferredRenderingBlock( true );

	u_int uLines = Asura_PS2_GS::VideoModeIsPal() ? 625 : 525;
    while (*T1_COUNT < (uLines - FILLBUFFMAXTIME))
    {
		fillBuff(&mp, 0);
    }

	Asura_PS2_DeferredRendering::WaitForDeferredRendering();

	sceGsSyncPath( 0, 0 );

    // ////////////////////////////
    // 
    //  Wait until two vblank happen
    // 
#if ASURA_PS2_SUPPORT_GUNCON2
    Asura_PS2_Input::UpdateGuncon2(0);
#endif

	while (sceGsSyncV(0) == 1);
    
#if ASURA_PS2_SUPPORT_GUNCON2
    Asura_PS2_Input::UpdateGuncon2(1);
#endif
    // ////////////////////////////
    // 
    //  Switch display
    // 
    if (drawid) 
	{
		disp.dispfb.FBP = uFBP1;
		drawid = 0;
    } 
	else 
	{
		disp.dispfb.FBP = uFBP0;
		drawid = 1;
    }
    sceGsPutDispEnv(&disp);

	// update frame number
	uFrameNumber++;
  }
  audioDecReset(&audioDec);
  
    // ////////////////////////////
    // 
    //  Reset MPEG decoder to the initial state
    // 
    sceMpegReset(&mp);

    Asura_DeallocateAlign( muxBuff );
    muxBuff = NULL;

    // ////////////////////////////
    // 
    //  Delete MPEG decoder
    // 
    sceMpegDelete(&mp);

    Asura_PS2_GS::ChangeVideoMode( ePrevVideoMode, uPrevScreenWidth, uPrevScreenHeight, uPrevFrontBfrBitDepth );

    Asura_PS2_TextureManagement::ForcePATH2Uploads( false );
  
// (MP - 13.10.2005) Don't really see the need for this either. We're
// invalidating the texture buffers below and we'll clear the z buffer when
// we begin the next frame.
//  Asura_PS2_GS::ClearVRAM();

    // We've stomped all over VRAM so let texture management know about that.
    Asura_PS2_TextureManagement::InvalidateAllTextureBuffers();

    return eRetVal;
}

static int videoCallback(sceMpeg *mp, sceMpegCbDataStr *cbstr, void *data)
{
  int availSpace;
  int spill;
  
  availSpace = demuxBuffPresent - demuxBuffPast;    // processed data
  if (availSpace < 0) availSpace += demuxBuffSize;  // ring buffer

  if ((availSpace == 0) && (demuxBuffPresent == demuxBuffFuture))
    availSpace = demuxBuffSize;    // special case - buffer is empty, not full
  
  if( availSpace < static_cast<int>(cbstr->len) ) { return 0; }

  spill = (demuxBuffPast - demuxBuff) + cbstr->len - demuxBuffSize;
  
  if (spill > 0) {
    memcpy(UncachedAddr(demuxBuffPast), cbstr->data, cbstr->len - spill);
    memcpy(UncachedAddr(demuxBuff), cbstr->data + (cbstr->len - spill),
	   spill);
    demuxBuffPast = demuxBuff + spill;
  }
  
  else {
    memcpy(UncachedAddr(demuxBuffPast), cbstr->data, cbstr->len);
    demuxBuffPast += cbstr->len;
  }
  
  return 1;
}

static int audioCallback(sceMpeg *mp, sceMpegCbDataStr *cbstr, void *data)
{
  u_char *pd0, *pd1;
  int d0, d1;
  int spill;
  
  audioDecBeginPut(&audioDec, &pd0, &d0, &pd1, &d1);
  
  cbstr->len  -= 4;  // first four bytes are sub-stream ID
  cbstr->data += 4;
  
  if( static_cast<int>(cbstr->len) > (d0 + d1) ) { return 0; }
  
  spill = cbstr->len - d0;
  
  if (spill > 0) {
    memcpy(pd0, cbstr->data, cbstr->len - spill);
    memcpy(pd1, cbstr->data + (cbstr->len - spill), spill);
  }
  
  else memcpy(pd0, cbstr->data, cbstr->len);
  
  audioDecEndPut(&audioDec, cbstr->len);
  return 1;
}

static int nodataCallback(sceMpeg *mp, sceMpegCbData *cbstr, void *data)
{
  int dmaSize;
  int availData;
  
  demuxBuffPresent = demuxBuffFuture;
  
  availData = demuxBuffPast - demuxBuffFuture;    // data ready for processing
  if (availData < 0) availData += demuxBuffSize;  // ring buffer
  
  while (availData < DMACHUNKSIZE) {
    
    if (fillBuff(mp, 1) == 0) {
      
      // /////////////////////////////////////////////////////////////////
      //
      //  More data cannot be obtained from the demultiplexer so this must
      //  be the end of the stream. Round up the data to ensure there's no
      //  remainder from the quadword-multiple transfer, otherwise the
      //  decoder won't receive the MPEG end sequence code.
      // 
      Asura_OutputToDebugger_Retail(("End of stream presumed\n"));
      availData += 15;
      break;
    }  
    
    availData = demuxBuffPast - demuxBuffFuture;   // data ready for processing
    if (availData < 0) availData += demuxBuffSize; // ring buffer
  }
  
  dmaSize = availData & ~0xf;
  if (dmaSize > DMACHUNKSIZE) dmaSize = DMACHUNKSIZE;
  
  if ((demuxBuffPresent + dmaSize) > (demuxBuff + demuxBuffSize))
    dmaSize = demuxBuff + demuxBuffSize - demuxBuffPresent;  
  
  // ////////////////////////////
  // 
  //  DMA must have stopped so these fields are safe to write to
  // 
  *D4_QWC  = dmaSize / 16;
  *D4_MADR = (int) demuxBuffPresent;
  *D4_CHCR = 1 | 1 << 8;               // start, normal, from memory
  
  demuxBuffFuture = demuxBuffPresent + dmaSize;
  
  if (demuxBuffFuture >= (demuxBuff + demuxBuffSize))  // ring buffer
    demuxBuffFuture -= demuxBuffSize;  
  
  return 1;
}

static int backgroundCallback(sceMpeg *mp, sceMpegCbData *cbstr, void *data)
{
  //int dueTime = *T0_COUNT + bgDuration;
  u_int dueTime = *T1_COUNT + bgDuration;

  while (*T1_COUNT < dueTime) fillBuff(mp, 0);
  return 1;
}

static int errorCallback(sceMpeg *mp, sceMpegCbData *cbstr, void *data)
{
#ifndef ASURA_NO_DEBUG_OUTPUT
  sceMpegCbDataError *cberr= (sceMpegCbDataError *) cbstr;
#endif
  Asura_OutputToDebugger_Retail(("MPEG decoding error: '%s'\n", cberr->errMessage));  
  // (MH - 04.06.2003) while (1);
  return 1;
}

// /////////////////////////////////////////////////
//
//  Fill the demultiplex buffers from at most READCHUNKSIZE worth of
//  multiplexed data. Returns number of bytes put into buffers.
//
static int fillBuff(sceMpeg *mp, int blocking)
{
  char *startPos;
  int procBytes;
  
  if (muxBuffFullness) startPos = muxBuff + READCHUNKSIZE - muxBuffFullness;
  
  else {
    do {
      muxBuffFullness = fileRead(muxBuff, READCHUNKSIZE);
      
      if ((muxBuffFullness == 0) && !blocking) return 0;
      
    } while (muxBuffFullness == 0);
    
    startPos = muxBuff;
  }

  procBytes = sceMpegDemuxPss(mp, (u_char *) startPos, muxBuffFullness);
  muxBuffFullness -= procBytes;

  return procBytes;
}

// ///////////////////////////////////////////////////////////////////
// 
//  Clear background of display
// 
// (MP - 13.10.2005) Unnecessary since we always change video mode now.
#if 0
static void clearBackground( u_int uWidth, u_int uHeight, u_int uColour )
{
  int i;
  
  static u_int backTile[16*16] bss_align(64);
  u_int *p = (u_int*) UncachedAddr(backTile);
  
  for (i = 0; i < 16*16; i++) p[i] = uColour;
  
  setLoadImageTagsTile( path3tag, backTile, 0, 0, uWidth, uHeight * 2 );
  loadImage(path3tag);
  sceGsSyncPath(0, 0);  
}
#endif

DEBUGVAR float dbg_fFMVSubtitleBoxHeight = 0.2f;
DEBUGVAR Asura_Bounding_Rectangle dbg_xFMVSubtitleRect( 0.06f, 1.0f - dbg_fFMVSubtitleBoxHeight, 0.94f, 1.0f );
//DEBUGVAR Asura_Bounding_Rectangle dbg_xFMVSubtitleRect( 0.2f, 1.0f - dbg_fFMVSubtitleBoxHeight, 0.94f, 1.0f );
DEBUGVAR float dbg_fFMVSubtitleXOffset = 0.0f;
DEBUGVAR float dbg_fFMVSubtitleYOffset = 0.0f;
DEBUGVAR float dbg_fFMVSubtitleLinespacing = 0.75f;
DEBUGVAR float dbg_fFMVSubtitleScale = 1.0f;

static void UpdateSubtitles( u_int uFrameNumber )
{
    const Asura_PS2_FMV::RENDER_SUBTITLE_CB_FN pfnRenderSubtitleCB = Asura_PS2_FMV::GetRenderSubtitleCB();
    if( !pfnRenderSubtitleCB ) { return; }

    // Before doing anything, tell texture management that all bets are off.
    // We'll have trashed anything it thought it already had uploaded.
    Asura_PS2_TextureManagement::InvalidateAllTextureBuffers();

	float fInverseFramesPerSecond = ( Asura_PS2_GS::VideoModeIsPal() ? 0.04f : 0.03336670003f );

    pfnRenderSubtitleCB( static_cast<float>(uFrameNumber) * fInverseFramesPerSecond );
    Asura_PS2_2D_Graphics::Flush();
}
