// ------------------------------------------------------------------------------------ //
//																						//
//	File: Asura_Platform_GUIMenuInput.cpp												//
//																						//
//	Description: this handles the platform specific menu control system					//
//				 ps2 version.															//
//																						//
//	Created: Steve Keen - 02/01/07														//
//																						//
//	Copyright (C) 2006 Rebellion (Derby), All Rights Reserved.							//
// ------------------------------------------------------------------------------------ //

#include "Core/Asura.h"

#ifdef ASURA_USE_GUI_MENU_SYSTEM
#include "Core/Asura_Hash.h"
#include "Core/Asura_Maths.h"
#include "Core/Asura_String.h"
#include "Core/Asura_Timers.h"

#include "Input/Asura_PlatformInput.h"
#include "Input/Asura_ControlMap.h"

#include "GUIMenu/Asura_GUIMenu_Message.h"
#include "GUIMenu/Asura_GUIMenu_OnScreenKeyboard.h"
#include "GUIMenu/Asura_GUIMenu_System.h"
#include "GUIMenu/Asura_GUIMenu_Widget_Base.h"

static void PS2_SetupMenuControlMap( Asura_ControlMap* pxControlMap, u_int uButtonOffsetID);

// --------------------------------------------------------------------------------------- 
#define ASURA_GUIMENU_MAX_CONTROLLERS	4

void Asura_GUIMenu_System::Platform_InitInputMappings( void)
{
	if( s_pxMenuControlMaps == NULL)
	{
		s_pxMenuControlMaps = new Asura_ControlMap[ASURA_GUIMENU_MAX_CONTROLLERS];

		PS2_SetupMenuControlMap( &s_pxMenuControlMaps[0], ASURA_PS2_KEYID_PAD1_CROSS);
		PS2_SetupMenuControlMap( &s_pxMenuControlMaps[1], ASURA_PS2_KEYID_PAD2_CROSS);
		PS2_SetupMenuControlMap( &s_pxMenuControlMaps[2], ASURA_PS2_KEYID_PAD3_CROSS);
		PS2_SetupMenuControlMap( &s_pxMenuControlMaps[3], ASURA_PS2_KEYID_PAD4_CROSS);
	}
}

// --------------------------------------------------------------------------------------- 

void Asura_GUIMenu_System::Platform_ShutdownInputMappings( void)
{
	u_int u;

	if( s_pxMenuControlMaps)
	{
		for( u=0; u<ASURA_GUIMENU_MAX_CONTROLLERS; u++)
		{
			s_pxMenuControlMaps[u].DeleteMappings();
		}
	
		delete []s_pxMenuControlMaps;
		s_pxMenuControlMaps = NULL;
	}
}

// --------------------------------------------------------------------------------------- 

u_int Asura_GUIMenu_System::Platform_GetNumControlMaps( void)
{
	return ASURA_GUIMENU_MAX_CONTROLLERS;
}

// --------------------------------------------------------------------------------------- 

bool Asura_GUIMenu_System::Platform_ShouldUpdateControls()
{
	return true;
}

// --------------------------------------------------------------------------------------- 

bool Asura_GUIMenu_System::Platform_ShouldRenderMenuPages()
{
	return true;
}

// --------------------------------------------------------------------------------------- 

void PS2_SetupMenuControlMap( Asura_ControlMap* pxControlMap, u_int uButtonOffsetID)
{
	if( pxControlMap)
	{
		u_int uPadOffset = uButtonOffsetID - (u_int)ASURA_PS2_KEYID_PAD1_CROSS;

		pxControlMap->CreateMappings( Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_MAX);

		// -- This is a fix for the dodgy mapping on my pad --
		pxControlMap->AddKeyMapping( Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_UP, uPadOffset + ASURA_PS2_KEYID_PAD1_UP, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
		pxControlMap->AddKeyMapping( Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_DOWN, uPadOffset + ASURA_PS2_KEYID_PAD1_DOWN, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
		pxControlMap->AddKeyMapping( Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_LEFT, uPadOffset + ASURA_PS2_KEYID_PAD1_LEFT, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
		pxControlMap->AddKeyMapping( Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_RIGHT, uPadOffset + ASURA_PS2_KEYID_PAD1_RIGHT, Asura_ControlMapping::ON_DEBOUNCED_PRESS);

		pxControlMap->AddKeyMapping( Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_ACCEPT, uPadOffset + ASURA_PS2_KEYID_PAD1_CROSS, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
		pxControlMap->AddKeyMapping( Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_ALT_ACCEPT1, uPadOffset + ASURA_PS2_KEYID_PAD1_SQUARE, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
		pxControlMap->AddKeyMapping( Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_ALT_ACCEPT2, uPadOffset + ASURA_PS2_KEYID_PAD1_CIRCLE, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
		pxControlMap->AddKeyMapping( Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_ALT_ACCEPT3, uPadOffset + ASURA_PS2_KEYID_PAD1_R3, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
		pxControlMap->AddKeyMapping( Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_ALT_ACCEPT4, uPadOffset + ASURA_PS2_KEYID_PAD1_SELECT, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
		pxControlMap->AddKeyMapping( Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_ALT_ACCEPT5, uPadOffset + ASURA_PS2_KEYID_PAD1_START, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
		pxControlMap->AddKeyMapping( Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_ALT_ACCEPT6, uPadOffset + ASURA_PS2_KEYID_PAD1_L1, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
		pxControlMap->AddKeyMapping( Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_ALT_ACCEPT7, uPadOffset + ASURA_PS2_KEYID_PAD1_R1, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
		pxControlMap->AddKeyMapping( Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_ALT_ACCEPT8, uPadOffset + ASURA_PS2_KEYID_PAD1_L2, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
		pxControlMap->AddKeyMapping( Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_ALT_ACCEPT9, uPadOffset + ASURA_PS2_KEYID_PAD1_R2, Asura_ControlMapping::ON_DEBOUNCED_PRESS);

		pxControlMap->AddKeyMapping( Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_CANCEL, uPadOffset + ASURA_PS2_KEYID_PAD1_TRIANGLE, Asura_ControlMapping::ON_DEBOUNCED_PRESS);

		pxControlMap->AddKeyMapping( Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_UP_HELD, uPadOffset + ASURA_PS2_KEYID_PAD1_UP, Asura_ControlMapping::ON_PRESS);
		pxControlMap->AddKeyMapping( Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_DOWN_HELD, uPadOffset + ASURA_PS2_KEYID_PAD1_DOWN, Asura_ControlMapping::ON_PRESS);
		pxControlMap->AddKeyMapping( Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_LEFT_HELD, uPadOffset + ASURA_PS2_KEYID_PAD1_LEFT, Asura_ControlMapping::ON_PRESS);
		pxControlMap->AddKeyMapping( Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_RIGHT_HELD, uPadOffset + ASURA_PS2_KEYID_PAD1_RIGHT, Asura_ControlMapping::ON_PRESS);

		pxControlMap->AddKeyMapping( Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_ACCEPT_HELD, uPadOffset + ASURA_PS2_KEYID_PAD1_CROSS, Asura_ControlMapping::ON_PRESS);
		pxControlMap->AddKeyMapping( Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_ALT_ACCEPT1_HELD, uPadOffset + ASURA_PS2_KEYID_PAD1_SQUARE, Asura_ControlMapping::ON_PRESS);
		pxControlMap->AddKeyMapping( Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_ALT_ACCEPT2_HELD, uPadOffset + ASURA_PS2_KEYID_PAD1_CIRCLE, Asura_ControlMapping::ON_PRESS);
		pxControlMap->AddKeyMapping( Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_ALT_ACCEPT3_HELD, uPadOffset + ASURA_PS2_KEYID_PAD1_R3, Asura_ControlMapping::ON_PRESS);
		pxControlMap->AddKeyMapping( Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_ALT_ACCEPT4_HELD, uPadOffset + ASURA_PS2_KEYID_PAD1_SELECT, Asura_ControlMapping::ON_PRESS);
		pxControlMap->AddKeyMapping( Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_ALT_ACCEPT5_HELD, uPadOffset + ASURA_PS2_KEYID_PAD1_START, Asura_ControlMapping::ON_PRESS);
		pxControlMap->AddKeyMapping( Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_ALT_ACCEPT6_HELD, uPadOffset + ASURA_PS2_KEYID_PAD1_L1, Asura_ControlMapping::ON_PRESS);
		pxControlMap->AddKeyMapping( Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_ALT_ACCEPT7_HELD, uPadOffset + ASURA_PS2_KEYID_PAD1_R1, Asura_ControlMapping::ON_PRESS);
		pxControlMap->AddKeyMapping( Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_ALT_ACCEPT8_HELD, uPadOffset + ASURA_PS2_KEYID_PAD1_L2, Asura_ControlMapping::ON_PRESS);
		pxControlMap->AddKeyMapping( Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_ALT_ACCEPT9_HELD, uPadOffset + ASURA_PS2_KEYID_PAD1_R2, Asura_ControlMapping::ON_PRESS);

		pxControlMap->AddKeyMapping( Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_CANCEL_HELD, uPadOffset + ASURA_PS2_KEYID_PAD1_TRIANGLE, Asura_ControlMapping::ON_PRESS);

		//CLF (02.09.2008) - Addition of support for PS2 analogue sticks to navigate the menus.
		pxControlMap->AddAxisMapping( Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_AXIS_H, uPadOffset + ASURA_AXIS_PAD1_LSTICK_H );
		pxControlMap->AddAxisMapping( Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_AXIS_V, uPadOffset + ASURA_AXIS_PAD1_LSTICK_V, -1.0f );

		pxControlMap->AddAxisMapping( Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_AXIS_H, uPadOffset + ASURA_AXIS_PAD1_RSTICK_H );
		pxControlMap->AddAxisMapping( Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_AXIS_V, uPadOffset + ASURA_AXIS_PAD1_RSTICK_V, -1.0f );


#ifdef ASURA_USE_CONTROLMAP_MANAGER
//		pxControlMap->AddKeyMapping( Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_DVS_ACTIVATE, uPadOffset + ASURA_PS2_KEYID_PAD1_SELECT, Asura_ControlMapping::ON_DEBOUNCED_PRESS );
		pxControlMap->AddHoldKeyMapping(Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_DVS_ACTIVATE, uPadOffset + ASURA_PS2_KEYID_PAD1_SELECT, 1.0f );
#endif 
	}
}

// --------------------------------------------------------------------------------------- 


// ARC - 01 March 2007 12:56:27 - Added stub functions for PS2
static bool g_bDoingTextInput = false;
static Asura_UnicodeChar* g_wszTextInputBuffer;
static u_int g_uTextInputBufferMaxChars;
static u_int g_uTextInputCursorPos;
static u_int g_uTextInputFrameNumber;

void Asura_GUIMenu_System::Platform_BeginTextInput( const Asura_UnicodeChar* wszMessage, 
												   Asura_UnicodeChar* wszBuffer, 
												   u_int uBufferMaxChars )
{
#ifdef ASURA_USE_GUI_OSK
	Asura_GUIMenu_OnScreenKeyboard::Initialise( wszBuffer, uBufferMaxChars);
#else
	Asura_Assert(!g_bDoingTextInput, "already doing text input");
	g_bDoingTextInput = true;
	g_wszTextInputBuffer = wszBuffer;
	g_uTextInputBufferMaxChars = uBufferMaxChars;
	g_uTextInputCursorPos = uBufferMaxChars;// this indicates "clear on first input"
	g_uTextInputFrameNumber = Asura_Timers::GetFrameNumber();
#endif // ASURA_USE_GUI_OSK
}

void Asura_GUIMenu_System::Platform_UpdateTextInput()
{
#ifdef ASURA_USE_GUI_OSK
	Asura_GUIMenu_OnScreenKeyboard::Update();
#else	
	//Asura_UserInput::UpdateInputBuffer( g_wszTextInputBuffer, g_uTextInputBufferMaxChars, g_uTextInputCursorPos, NULL );
#endif // ASURA_USE_GUI_OSK
}

#ifdef ASURA_DEBUG_VARIABLE_SYSTEM
void Asura_GUIMenu_System::Platform_RenderSafetyBorder( void)
{
}
#endif // ASURA_DEBUG_VARIABLE_SYSTEM

void Asura_GUIMenu_System::Platform_BeginRender()
{
}

void Asura_GUIMenu_System::Platform_RenderTextInput()
{
	// just leave the menu to render it? what about cursors?
}

void Asura_GUIMenu_System::Platform_EndRender()
{
}

bool Asura_GUIMenu_System::Platform_IsTextInputFinished()
{
#ifdef ASURA_USE_GUI_OSK
	bool bRet = false;

	switch( Asura_GUIMenu_OnScreenKeyboard::GetExitType())
	{
	case ASURA_GUIMENU_OSK_DONE:
	case ASURA_GUIMENU_OSK_CANCEL:
		bRet = true;
		break;

	case ASURA_GUIMENU_OSK_NOEXIT:
	default:
		break;
	}

	return bRet;
#else

	//if (g_uTextInputFrameNumber == Asura_Timers::GetFrameNumber())
	//{
	//	return false;
	//}
	//return Asura_UserInput::DebouncedKeyPressed( ASURA_KEYID_ENTER );
	return true;
#endif // ASURA_USE_GUI_OSK
}

bool Asura_GUIMenu_System::Platform_IsTextInputCancelled()
{
#ifdef ASURA_USE_GUI_OSK
	return (Asura_GUIMenu_OnScreenKeyboard::GetExitType() == ASURA_GUIMENU_OSK_CANCEL);
#else
	return false;
#endif
}

void Asura_GUIMenu_System::Platform_EndTextInput()
{
#ifdef ASURA_USE_GUI_OSK
	Asura_GUIMenu_OnScreenKeyboard::Shutdown();
#else
	Asura_Assert(g_bDoingTextInput, "not doing text input");
	g_bDoingTextInput = false;
	g_wszTextInputBuffer = NULL;
	g_uTextInputBufferMaxChars = 0;
#endif // ASURA_USE_GUI_OSK
}

// --------------------------------------------------------------------------------------- 

bool Asura_GUIMenu_System::Platform_ProcessMenuBackControls( u_int uButtonsPressed, u_int uButtonsTrapped)
{
	bool bRet = false;

	// filters out any buttons presses not required, then we do an equality check
	uButtonsPressed &= (ASURA_GUIMENU_ACTION_BUTTON_CANCEL|ASURA_GUIMENU_ACTION_BUTTON_CANCEL_HELD);

	if( uButtonsPressed == (ASURA_GUIMENU_ACTION_BUTTON_CANCEL|ASURA_GUIMENU_ACTION_BUTTON_CANCEL_HELD) )
	{
		if( (uButtonsTrapped&ASURA_GUIMENU_ACTION_BUTTON_CANCEL) == 0)
		{
			bRet = true;
		}
	}

	return bRet;
}

// --------------------------------------------------------------------------------------- 

#endif // ASURA_USE_GUI_MENU_SYSTEM
