//////////////////////////////////////////////////////////////////////////////
//
// Asura_PS2_USBDevice.h
//
// Created by Neil Massey.
//
// Copyright (c) 2002 Rebellion. All rights reserved.
//
// Base class for USB devices
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef ___ASURA_PS2_USBDEVICE_H___
#define ___ASURA_PS2_USBDEVICE_H___ 1

#include "Input/Asura_PS2_Input.h"

// Enum for supported devices plugged into USB ports
enum ASURA_PS2_USB_DeviceType {
	USB_DT_NO_DEVICE = 0xFF,
	USB_DT_KEYBOARD = 0,
	USB_DT_MOUSE	= 1,
	USB_DT_GUNCON2	= 2,
	USB_DT_UNKNOWN	= 3
};

enum ASURA_PS2_USB_PHASE
{
	USB_PHASE_NONE			= -1,
	USB_PHASE_INITIALISING  = 0,
	USB_PHASE_ENUMERATING,
	USB_PHASE_ENUMERATED,
	USB_PHASE_GET_LOCATIONS,
	USB_PHASE_BUSY,
	USB_PHASE_ERROR,
	USB_PHASE_NO_DEVICES,
	USB_PHASE_CALIBRATE,
	USB_PHASE_READY,
	USB_PHASE_DEVICE_SPECIFIC = 100,
};

class Asura_PS2_USBDevice
{
public:
	Asura_PS2_USBDevice(int iDeviceNumber);
	virtual ~Asura_PS2_USBDevice(void);

	virtual void Update(void)= 0;
	virtual void Translate(u_char *piAsuraInputBufferOffset, float *piAsuraAxis_BufferOffset)= 0;
	virtual bool Available(void)= 0;
	virtual void SetCalibration(float iData1, float iData2){};
	ASURA_PS2_USB_DeviceType GetId(void) const {return m_xDeviceType;}

    // Static functions for initialise, update, etc. - to perform the function on all USB devices
    static void Initialise(void);

protected:
	ASURA_PS2_USB_DeviceType m_xDeviceType;
    
	// (MH - 15.01.2004) as far as I can tell, this m_iDeviceNumber is a way of
	// identifying which device from 0 to (MAX_DEVICES_OF_TYPE - 1) is controlled
	// by this class instance. It is set when the class instance is constructed, 
	// so keyboard 0 might not actually be connected.
	// It is not related to "USB location" in any way.
	int m_iDeviceNumber;
};

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

#endif // #ifndef ___ASURA_PS2_USBDEVICE_H___
