//////////////////////////////////////////////////////////////////////////////
//
// Asura_PS2_USBkeyboard.h
//
// Created by Neil Massey.
//
// Copyright (c) 2002 Rebellion. All rights reserved.
//
// Derived class implementation of USB keyboard
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef ___ASURA_PS2_USBKEYBOARD_H___
#define ___ASURA_PS2_USBKEYBOARD_H___ 1

#include "Input/Asura_PS2_USBDevice.h"
#include "Input/Logitech/liblgkbm.h"
#include "Core/Asura_UserInput.h"

const u_int uASURA_PS2_KEYBUFFERSIZE = 20;

class Asura_PS2_USBKeyboard : public Asura_PS2_USBDevice
{
public:
    Asura_PS2_USBKeyboard(int iPortNumber);
    virtual ~Asura_PS2_USBKeyboard(void);

    virtual void Update(void);
    virtual void Translate(u_char *piAsuraInputBufferOffset, float *piAsuraAxis_BufferOffset);
    virtual bool Available(void);

    void Initialise(void);
    void SetRepeatEnabled( bool bEnable );

    bool IsKeyToggled(int iKey) const;

    int* GetKeyStrokeBuffer()
    {
        return m_aiAsuraKeyStrokeBuffer;
    }

    u_int GetKeyStrokeBufferSize()
    {
        return m_uNextBufferEntry;
    }

    void ResetKeyStrokeBuffer()
    {
        m_uNextBufferEntry = 0;
    }

    bool* GetKeyStrokeShiftBuffer()
    {
        return m_abAsuraKeyStrokeShiftBuffer;
    }

    bool* GetKeyStrokeCapsBuffer()
    {
        return m_abAsuraKeyStrokeCapsBuffer;
    }

private:
    lgKbdDeviceDesc     m_xKbdDeviceDesc;
    lgKbdEventList      m_xKbdEventList;
    lgKbdStateInfo      m_xKbdState;
    int                 m_iEnumNeeded;
    int                 m_iKbdDevice;
    int                 m_iLocation;
    ASURA_PS2_USB_PHASE m_eCurrentPhase;
    bool                m_abAsuraKeyBuffer[ASURA_KEYID_END_OF_KEYBOARD];
    bool                m_bDoTranslate;
    int                 m_iRepeatedKey;
    bool                m_bLEDStateValid; // LED state is only valid after a keystroke.
    int                 m_aiAsuraKeyStrokeBuffer[uASURA_PS2_KEYBUFFERSIZE];
    bool                m_abAsuraKeyStrokeShiftBuffer[uASURA_PS2_KEYBUFFERSIZE];
    bool                m_abAsuraKeyStrokeCapsBuffer[uASURA_PS2_KEYBUFFERSIZE];
    u_int               m_uNextBufferEntry;
};

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

#endif // #ifndef ___ASURA_PS2_USBKEYBOARD_H___
