//////////////////////////////////////////////////////////////////////////////
//
// Asura_Platform_Rumble.cpp
//
// Copyright (c) 2003 Rebellion. All rights reserved.
//
// Created by (NRM - 28.03.2003) 
//
//
// Description:
//
//     PS2 dualshock 2 rumble
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"
#include "Input/Asura_Rumble_System.h"

#if (defined ASURA_SUPPORT_CONTROLLER_RUMBLE) && (ASURA_SUPPORT_CONTROLLER_RUMBLE != 0)

#include "Input/Asura_PS2_Input.h"
#include "Input/Asura_PS2_ControlPortDevice.h"
#include "ProjectSettings/Asura_PS2_ProjectControllerDefines.h"

Asura_Platform_Rumble* Asura_Rumble_System::ASURA_STANDARD_RUMBLE_EXPLOSION;
Asura_Platform_Rumble* Asura_Rumble_System::ASURA_STANDARD_RUMBLE_SINGLESHOT;
Asura_Platform_Rumble* Asura_Rumble_System::ASURA_STANDARD_RUMBLE_HEARTBEAT;
Asura_Platform_Rumble* Asura_Rumble_System::ASURA_STANDARD_RUMBLE_HIT;

void Asura_Rumble_System::Platform_PlayCurrentRumbles()
{
#if ASURA_PS2_SUPPORT_MULTITAP
    for (int iCurrentPort=0; iCurrentPort<2; ++iCurrentPort)
    {
        int iNumberOfSlots = Asura_PS2_Input::MultitapAvailable(iCurrentPort) ? 4 : 1;
	    for (int iCurrentSlot = 0; iCurrentSlot < iNumberOfSlots; iCurrentSlot++)
	    {
			int iControllerNumber = iCurrentPort * 4 + iCurrentSlot;
            Asura_PS2_ControlPortDevice* pxControlPortDevice = Asura_PS2_Input::GetControlPortDevice(iControllerNumber);
            if(pxControlPortDevice)
            {
                // Check to see if we are still transferring current motor value
                // We can alter the motor values
                int iRumbleSlot = Asura_PS2_Input::GetPlayerNumber(iControllerNumber);
                u_char ucLargeAmount = static_cast<u_char>(255.f * s_fCurrentMotorMagnitudes[iRumbleSlot][ASURA_PS2_VM_LARGE_MOTOR]);
                u_char ucSmallAmount = static_cast<u_char>(255.f * s_fCurrentMotorMagnitudes[iRumbleSlot][ASURA_PS2_VM_SMALL_MOTOR]);
                // Only set small motor if magnitude is greater than a half
                pxControlPortDevice->SetRumble(ucSmallAmount, ucLargeAmount);
            }
        }
    }
#else
    for (int iCurrentPad=0; iCurrentPad<2; iCurrentPad++)
    {
        Asura_PS2_ControlPortDevice* pxControlPortDevice = Asura_PS2_Input::GetControlPortDevice(iCurrentPad * 4);
        if(pxControlPortDevice)
        {
            // Check to see if we are still transferring current motor value
            // We can alter the motor values
            u_char ucLargeAmount = static_cast<u_char>(255.f * s_fCurrentMotorMagnitudes[iCurrentPad][ASURA_PS2_VM_LARGE_MOTOR]);
            u_char ucSmallAmount = static_cast<u_char>(255.f * s_fCurrentMotorMagnitudes[iCurrentPad][ASURA_PS2_VM_SMALL_MOTOR]);
            // Only set small motor if magnitude is greater than a half
            pxControlPortDevice->SetRumble(ucSmallAmount, ucLargeAmount);
        }
    }
#endif
}

void Asura_Rumble_System::Platform_Initialise()
{
    // (NRM - 28.03.2003) This has just been copied and pasted from the XBox version :)
    // Shouldn't these be created on a project specific basis?
	Asura_Handle iHeartbeatSlowMotor = Asura_Rumble::CreateAsuraRumble(RUMBLE_BASE_CONSTANT, RUMBLE_ADJUST_NONE, 0.0f, 0.0f, 5.0f, 1);
	Asura_Rumble::AddRumbleStep(iHeartbeatSlowMotor, RUMBLE_BASE_CONSTANT, RUMBLE_ADJUST_NONE, 1.0f, 0.0f, 3.0f, 1);
	Asura_Rumble::AddRumbleStep(iHeartbeatSlowMotor, RUMBLE_BASE_CONSTANT, RUMBLE_ADJUST_NONE, 0.0f, 0.0f, 1.0f, 1);
	Asura_Handle iHeartbeatRumble = s_pxDynamicPlatformRumbleList->Allocate();
	ASURA_STANDARD_RUMBLE_HEARTBEAT = s_pxDynamicPlatformRumbleList->GetPointer(iHeartbeatRumble);
    ASURA_STANDARD_RUMBLE_HEARTBEAT->m_apxMotorRumbles[ASURA_PS2_VM_SMALL_MOTOR] = NULL;
	ASURA_STANDARD_RUMBLE_HEARTBEAT->m_apxMotorRumbles[ASURA_PS2_VM_LARGE_MOTOR] = Asura_Rumble::GetRumblePointer(iHeartbeatSlowMotor);

	Asura_Handle iHitSlowMotor = Asura_Rumble::CreateAsuraRumble(RUMBLE_BASE_LINEAR, RUMBLE_ADJUST_REVERSE, 1.0f, 0.0f, 1.0f, 1);
	Asura_Handle iHitRumble = s_pxDynamicPlatformRumbleList->Allocate();
	ASURA_STANDARD_RUMBLE_HIT = s_pxDynamicPlatformRumbleList->GetPointer(iHitRumble);
    ASURA_STANDARD_RUMBLE_HIT->m_apxMotorRumbles[ASURA_PS2_VM_SMALL_MOTOR] = NULL;
	ASURA_STANDARD_RUMBLE_HIT->m_apxMotorRumbles[ASURA_PS2_VM_LARGE_MOTOR] = Asura_Rumble::GetRumblePointer(iHitSlowMotor);

	Asura_Handle iExplosionFastMotor = Asura_Rumble::CreateAsuraRumble(RUMBLE_BASE_CONSTANT, RUMBLE_ADJUST_NONE, 1.0f, 0.0f, 2.5f, 1);
	Asura_Rumble::AddRumbleStep(iExplosionFastMotor, RUMBLE_BASE_LINEAR, RUMBLE_ADJUST_REVERSE, 1.0f, 0.0f, 1.0f, 1);
	Asura_Handle iExplosionSlowMotor = Asura_Rumble::CreateAsuraRumble(RUMBLE_BASE_CONSTANT, RUMBLE_ADJUST_NONE, 1.0f, 0.0f, 2.5f, 1);
	Asura_Rumble::AddRumbleStep(iExplosionSlowMotor, RUMBLE_BASE_LINEAR, RUMBLE_ADJUST_REVERSE, 1.0f, 0.0f, 0.5f, 1);
	Asura_Handle iExplosionRumble = s_pxDynamicPlatformRumbleList->Allocate();
	ASURA_STANDARD_RUMBLE_EXPLOSION = s_pxDynamicPlatformRumbleList->GetPointer(iExplosionRumble);
	ASURA_STANDARD_RUMBLE_EXPLOSION->m_apxMotorRumbles[ASURA_PS2_VM_SMALL_MOTOR] = Asura_Rumble::GetRumblePointer(iExplosionFastMotor);
	ASURA_STANDARD_RUMBLE_EXPLOSION->m_apxMotorRumbles[ASURA_PS2_VM_LARGE_MOTOR] = Asura_Rumble::GetRumblePointer(iExplosionSlowMotor);

	Asura_Handle iSingleShotSlowMotor = Asura_Rumble::CreateAsuraRumble(RUMBLE_BASE_CONSTANT, RUMBLE_ADJUST_NONE, 1.0f, 0.0f, 2.5f, 1);
	Asura_Handle iSingleShotRumble = s_pxDynamicPlatformRumbleList->Allocate();
	ASURA_STANDARD_RUMBLE_SINGLESHOT = s_pxDynamicPlatformRumbleList->GetPointer(iSingleShotRumble);
    ASURA_STANDARD_RUMBLE_SINGLESHOT->m_apxMotorRumbles[ASURA_PS2_VM_SMALL_MOTOR] = NULL;
	ASURA_STANDARD_RUMBLE_SINGLESHOT->m_apxMotorRumbles[ASURA_PS2_VM_LARGE_MOTOR] = Asura_Rumble::GetRumblePointer(iSingleShotSlowMotor);
}

void Asura_Rumble_System::Platform_Shutdown()
{
	Platform_StopAllRumbles();
}

void Asura_Rumble_System::Platform_Reset()
{
	Platform_Shutdown();
	Platform_Initialise();
}

void Asura_Rumble_System::Platform_StopAllRumbles()
{
    for (int iCurrentPort=0; iCurrentPort<2; ++iCurrentPort)
    {
        int iNumberOfSlots = Asura_PS2_Input::MultitapAvailable(iCurrentPort) ? 4 : 1;
	    for (int iCurrentSlot = 0; iCurrentSlot < iNumberOfSlots; iCurrentSlot++)
	    {
            Asura_PS2_ControlPortDevice* pxControlPortDevice = Asura_PS2_Input::GetControlPortDevice(iCurrentPort * 4 + iCurrentSlot);
            if(pxControlPortDevice)
            {
                // Only set small motor if magnitude is greater than a half
                pxControlPortDevice->SetRumble(0, 0);
                pxControlPortDevice->Update();
            }
        }
    }
}

#endif	// (defined ASURA_SUPPORT_CONTROLLER_RUMBLE) && (ASURA_SUPPORT_CONTROLLER_RUMBLE != 0)
