//////////////////////////////////////////////////////////////////////////////
//
// Asura_PS2_MemoryCard_Shell.h
//
// Copyright (c) 2003 Rebellion. All rights reserved.
//
// Created by : (NRM - 19.03.2003) 
//
// Description:
//
//     Shell for memory card access to help with TRC
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef ___ASURA_PS2_MEMORYCARD_SHELL_H___
#define ___ASURA_PS2_MEMORYCARD_SHELL_H___ 1

#include "MemoryCard/Asura_PS2_MemoryCard.h"
#include "MemoryCard/Asura_PS2_MemoryCard_SubSystem.h"

const int iICON_SYS_SIZE = 964;

class Asura_PS2_MemoryCard_Shell
{
    public:        
        // Get display messages
        enum DisplayMessage
        {
        /*****************************/
        // Normal messages
            MSG_CHECKING_CARD,
            MSG_LOADING,
            MSG_SAVING,
            MSG_AUTOSAVING,
            MSG_FORMATTING,
            MSG_DELETING,
            MSG_OVERWRITING,

            MSG_LOAD_SUCCESSFUL,
            MSG_SAVE_SUCCESSFUL,
            MSG_FORMAT_SUCCESSFUL,
            MSG_DELETE_SUCCESSFUL,
            MSG_OVERWRITE_SUCCESSFUL,

        /*****************************/
        // Error messages
            MSG_ERR_NO_CARD_LOAD,
            MSG_ERR_NO_CARD_SAVE,
            MSG_ERR_NO_CARD_ON_BOOT,
            MSG_ERR_NO_FORMAT,
            MSG_ERR_NO_SAVE_CREATE,
            MSG_ERR_NO_SAVES,
            MSG_ERR_NO_ROOM,
            MSG_ERR_FILE_DOES_NOT_EXIST,

            MSG_ERR_FORMAT_FAILED,
            MSG_ERR_LOAD_FAILED,
            MSG_ERR_SAVE_FAILED,
            MSG_ERR_DELETE_FAILED,
            MSG_ERR_OVERWRITE_FAILED,
            MSG_ERR_SAVE_CORRUPT,

            MSG_ERR_SAVE_EXISTS,
            MSG_ERR_CARD_CHANGED,
            MSG_ERR_CONFIRM_DELETE,
            MSG_ERR_CONFIRM_DELETE_SAVE_DIRECTORY,
            MSG_ERR_CONFIRM_OVERWRITE,

            MSG_ERR_RETRY_SAVE,
            MSG_ERR_RETRY_LOAD,
            MSG_ERR_RETRY_DELETE,
            MSG_ERR_RETRY_OVERWRITE,
			MSG_ERR_RETRY_FORMAT,

        /*****************************/
            MSG_NONE,
            MSG_INVALID,
        };

        enum GetInputMessage
        {
            MSG_GI_RETRY,

            MSG_GI_YES_NO,
            MSG_GI_YES_NO_REVERSE,

            MSG_GI_FORMAT,
            MSG_GI_DELETE,

        /*****************************/
            MSG_GI_NONE,
            MSG_GI_INVALID,
        };

        enum State
        {
            STATE_RUNNING,
            STATE_ERROR,
            STATE_FINISHED,

            STATE_INVALID,
        };

        enum PollResult
        {
            POLL_BUSY,
            POLL_OK,
            POLL_NO_CARD,
            POLL_CARD_CHANGED,
            POLL_CARD_UNFORMATTED,

            POLL_INVALID,
        };

        // Initialise, Update, Shutdown
        static void Initialise(void);
        static void Update(void);
        static void Shutdown(void);

        // Function for polling the card each frame
        static PollResult PollCard(void);

        // Check card when booting
        static void CheckCardOnBoot(void);

        // Save the main save containing progress, high scores etc.
        static void SaveMain(const void* pData, const int iDataSize);
        static void AutoSaveMain(const void* pData, const int iDataSize);

        // Save secondary saves, 
        static void Save(const char* szSaveName, const void* pData, const int iDataSize);

        // Get list of secondary saves
        static void GetSaveNameList(char* szSaveNameList);
      
        // Load main save 
        static void LoadMain(void* pData, const int iDataSize);
        static void AutoLoadMain(void* pData, const int iDataSize);

        // Load a secondary save
        static void Load(const char* szLoadName, void* pData, const int iDataSize);

        // Delete a file
        static void Delete(const char* szFileName);

        // Delete save directory contents - use with caution ;)
        static void DeleteSaveDirectory(void);        

        // Functions to query state machine
        static State            GetState(void) {return s_eCurrentState;}
        static DisplayMessage   GetCurrentMessage(void) {return s_eCurrentMessage;}
        static GetInputMessage  GetCurrentInputMessage(void) {return s_eCurrentGetInputMessage;}

        // for progress bars etc.
        static float GetProgressOfCurrentOperation(void) {return s_fProgressOfCurrentOperation;}

        // Functions to help in setting up of data
        // A quick way to create a reasonable looking icon
        static void CreateDefaultIcon(const char* szIconName);

        // Set port and slot to use - (NRM - 19.03.2003) recommend 0 and 0 and support just one memory card
        static void SetPortAndSlot(const int iPort, const int iSlot) 
        {
            s_iCurrentPort = iPort; 
            s_iCurrentPort = iSlot;
        }

        // Whether processing has finished
        static bool Finished(void) {return s_eCurrentState == STATE_FINISHED;}
        static bool GetFinishedSuccessfully(void) {return s_bFinishedSuccessfully;}
        static bool WasCardChanged(void) {return s_bCardChanged;}

        // (NRM - 19.05.2003) Start a batch operation - this stops success and checking messages appearing
        static void StartBatchOperation(void)  
        {
            s_xBatchInformation.bFirstOperationInBatch = true;
            s_xBatchInformation.bSaveWasCancelledInBatch = false;
            s_xBatchInformation.bLastOperationInBatch = false;
            s_xBatchInformation.bInBatchOperation = true;
        }
        static void FinishBatchOperation(void) 
        {
            s_xBatchInformation.bLastOperationInBatch = false;
            s_xBatchInformation.bSaveWasCancelledInBatch = false;
            s_xBatchInformation.bInBatchOperation = false;
        }

        static void SetLastOperationInBatch(void)
        {
            s_xBatchInformation.bLastOperationInBatch = true;            
        }

        // These functions are basically just wrappers for the Asura_MemoryCard_System
        // here so the user only has to worry about one interface :)
    	static void SetProductCode(const char* szProductCode);
        static void SetProductName(const char* szProductName);
        // (NRM - 05.08.2003) had to make this an array of data sizes to cope with cluster size adjustment
        static int  SetDataSize(const int* aiDataSize, const int iNumberOfFiles);

        static void SetAutomaticallyOverwrite(bool bAutomaticallyOverwrite) 
        {
            s_xBatchInformation.bAutomaticallyOverwrite = bAutomaticallyOverwrite;
        }

        static bool FileExists(const char* szFileName); // check file exists in s_szDirList

        static void SetSilentlyDelete(bool bSilentlyDelete)
        {
            s_xBatchInformation.bSilentlyDelete = bSilentlyDelete;
        }

        static void ResetUserConfirmations(void)
        {
            s_xBatchInformation.bUserConfirmedCardUnavailable = false;
            s_xBatchInformation.bUserConfirmedCardUnformatted = false;
            s_xBatchInformation.bUserConfirmedNoRoom          = false;
        }

        static bool UserCancelledOverwrite(void)
        {
            return s_xBatchInformation.bUserCancelledOverwrite;
        }

        static void ResetUserCancelledOverwrite(void)
        {
            s_xBatchInformation.bUserCancelledOverwrite = false;
        }

        // This function is used just for checking the save size - 
        // Sony allow you to round up but you have to check against the rounded up size when checking for free space
        static void OverrideSaveSize(int iSaveSize)
        {
            s_iOverrideDataSize = iSaveSize;
        }

        // (NRM - 07.07.2003) set cancel / retry sounds
        static void SetCancelSound(int iSound)
        {
            s_iCancelSound = iSound;
        }

        static void SetRetrySound(int iSound)
        {
            s_iRetrySound = iSound;
        }

        static void SetSoundVolume(float fVolume)
        {
            s_fMenuSoundVolume = fVolume;
        }

        // Set saving as actually overwriting
        static void SetAsOverwrite(bool bOverwrite)
        {
            s_xBatchInformation.bSetAsOverwrite = bOverwrite;
        }

        static bool GetOverwrite(void)
        {
            return s_xBatchInformation.bSetAsOverwrite;
        }

        // (NRM - 14.08.2003) this allows background loading / silently failing auto load
        static void SetSilentlyFailAutoLoad(bool bSilentlyFailAutoLoad)
        {
            s_xBatchInformation.bUserConfirmedCardUnavailable = bSilentlyFailAutoLoad;
        }
		
		static bool GetCreateIconAndSaveDir()
		{
			return s_bCreateIconAndDir;
		}
		
    private:
        // Issue user commands to the state machine
        enum Command
        {
            CMD_CHECK_ON_BOOT,

            CMD_START_SAVE,
            CMD_START_LOAD,

            CMD_START_AUTOLOAD_MAIN,
            CMD_START_LOAD_MAIN,
            CMD_START_AUTOSAVE_MAIN,
            CMD_START_SAVE_MAIN,

            CMD_START_GET_SAVELIST,

            CMD_START_DELETE,
            CMD_START_DELETE_SAVE_DIRECTORY,

            CMD_INVALID,
        };

        enum Phase
        {
            // check phases
            PHASE_CHECK_FOR_CARD,
            PHASE_CHECK_FOR_ROOM,
            PHASE_CHECK_FORMATTED,
            PHASE_CHECK_FOR_SAVE,
            PHASE_CHECK_FOR_SUB_SAVE,

            // "do" phases
            PHASE_FORMATTING,
            PHASE_SAVING,
            PHASE_LOADING,
            PHASE_GET_SAVELIST,
            PHASE_DELETING,
            PHASE_DELETING_SAVE_DIRECTORY,

            // prompt phases
            PHASE_PROMPT_CREATE_SAVE,
            PHASE_PROMPT_CARD_CHANGED,
            PHASE_PROMPT_NO_CARD,
            PHASE_PROMPT_OVERWRITE_SUB_SAVE,
            PHASE_PROMPT_SAVE_DOES_NOT_EXIST,
            PHASE_PROMPT_DELETE_CONFIRM,

            // failure phases
            PHASE_FORMAT_FAILED,
            PHASE_SAVE_FAILED,
            PHASE_LOAD_FAILED,
            PHASE_DELETE_FAILED,

            PHASE_FINISHED,

            PHASE_INVALID,
        };

        enum CommandStatus
        {
            CMD_STATUS_ISSUE,
            CMD_STATUS_WAIT,
            CMD_STATUS_CARD_CHECK,
            CMD_STATUS_WAIT_ON_CHANGE_DIRECTORY,
            CMD_STATUS_INVALID,
        };

        enum UserInput
        {
            UI_RETRY  = 0,
            UI_CANCEL = 1,
            UI_YES    = UI_RETRY,
            UI_NO     = UI_CANCEL,
            UI_ACCEPT = UI_RETRY,
            UI_REJECT = UI_CANCEL,
            UI_FORMAT = 2,
            UI_DELETE = UI_FORMAT,
            UI_DO_NOTHING = 16,
        };

        // Phase / command / state / message to display
        static Phase            s_eCurrentPhase;
        static Command          s_eEntryCommand;
        static CommandStatus    s_eCurrentCommandStatus;
        static State            s_eCurrentState;
        static DisplayMessage   s_eCurrentMessage;
        static GetInputMessage  s_eCurrentGetInputMessage;

        static float            s_fProgressOfCurrentOperation;
        static float            s_fWaitTimer;
        static int              s_iCurrentPort;
        static int              s_iCurrentSlot;
        static int              s_iDataSize;
        static int              s_iCheckDataSize;
        static int              s_iIconFileSize;
        static const void*      s_pSaveDataPointer;
        static void*            s_pLoadDataPointer;
        static char             s_szDirList[512];
        static bool             s_bCreateIconAndDir;
        static bool             s_bFinishedSuccessfully;
        static bool             s_bCardChanged;
        static char             s_szFileName[32];
        static char             s_szProductName[32];
        static char             s_szProductCode[32];
        static char*            s_szSaveListOutput;
        static char             s_szLocalSaveList[512];
        static bool             s_bDisableAutosave;
        static int              s_iOverrideDataSize;
        static bool             s_bSaveStarted;

        // (NRM - 19.05.2003) Some variables for batching loads and saves and not showing the "checking messages" all the time
        struct BatchInfo
        {
            bool             bAutomaticallyOverwrite;
            bool             bInBatchOperation;
            bool             bLastOperationInBatch;
            bool             bFirstOperationInBatch;
            bool             bSaveWasCancelledInBatch;
            bool             bUserConfirmedCardUnavailable;
            bool             bUserConfirmedCardUnformatted;
            bool             bUserConfirmedNoRoom;
            bool             bUserCancelledOverwrite;
            bool             bSilentlyDelete;
            bool             bSetAsOverwrite;
        };
        static BatchInfo     s_xBatchInformation;

        static int           s_iCancelSound;
        static int           s_iRetrySound;
        static float         s_fMenuSoundVolume;

        // IssueCommand 
        static void IssueCommand(const Command eCommand);

        // User input query
        static UserInput GetUserInput(void);

        // Finished function to reset state machine
        static void NowFinished(void);

        // Phase process functions
        // check phases
        static void ProcessCheckForCard(void);          // PHASE_CHECK_FOR_CARD                 
        static void ProcessCheckForRoom(void);          // PHASE_CHECK_FOR_ROOM,
        static void ProcessCheckFormatted(void);        // PHASE_CHECK_FORMATTED
        static void ProcessCheckForSave(void);          // PHASE_CHECK_FOR_SAVE,
        static void ProcessCheckForSubSave(void);       // PHASE_CHECK_FOR_SUB_SAVE,

        // "do" phases
        static void ProcessFormatting(void);            // PHASE_FORMATTING,
        static void ProcessSaving(void);                // PHASE_SAVING,
        static void ProcessLoading(void);               // PHASE_LOADING,
        static void ProcessGetSaveList(void);           // PHASE_GET_SAVELIST
        static void ProcessDeleting(void);              // PHASE_DELETING,
        static void ProcessDeletingSaveDirectory(void); // PHASE_DELETING_SAVE_DIRECTORY,

        // prompt phases
        static void ProcessPromptCreateSave(void);      // PHASE_PROMPT_CREATE_SAVE
        static void ProcessPromptCardChanged(void);     // PHASE_PROMPT_CARD_CHANGED
        static void ProcessPromptOverwriteSubSave(void);// PHASE_PROMPT_OVERWRITE_SUB_SAVE
        static void ProcessPromptNoCard(void);
        static void ProcessPromptSaveDoesNotExist(void);// PHASE_PROMPT_SAVE_DOES_NOT_EXIST
        static void ProcessPromptDelete(void);          // PHASE_PROMPT_DELETE_CONFIRM

        // failure phases
        static void ProcessFormatFailed(void);          // PHASE_FORMAT_FAILED,
        static void ProcessSaveFailed(void);            // PHASE_SAVE_FAILED,
        static void ProcessLoadFailed(void);            // PHASE_LOAD_FAILED,
        static void ProcessDeleteFailed(void);          // PHASE_DELETE_FAILED,

        // Helper functions
        static void WaitForInput(const DisplayMessage eMessage, const Phase eNextPhase);
        static void SetMessageToCheckingForCard(bool bCheckIfInBatch = true);     
        static void CheckForCard(void);

        static void PlayRetrySound(void);           // also play delete / format sound
        static void PlayCancelSound(void);

		static void SetState(State eState);
		static void SetPhase(Phase ePhase);

#ifdef ASURA_LOGFILES_ENABLED
		static Asura_Char* GetStateDescription(State eState);
		static Asura_Char* GetPhaseDescription(Phase ePhase);
#endif
};

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

#endif // #ifndef ___ASURA_PS2_MEMORYCARD_SHELL_H___
