//////////////////////////////////////////////////////////////////////////////
//
// Asura_Platform_ProgressBar.cpp
//
// Copyright (c) 2004 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

// Engine independent.
#include "Menus/Asura_ProgressBar.h"

// Engine platform-specific.
#include "Core/Asura_PS2_DMA.h"
#include "Rendering/Asura_PlatformRenderStates.h"
#include "Rendering/Asura_PS2_DeferredRendering.h"
#include "Rendering/Asura_PS2_GS.h"

// C L A S S   I M P L E M E N T A T I O N S /////////////////////////////////

//
// Asura_ProgressDisplay_BackgroundFiller.
//

void Asura_ProgressDisplay_BackgroundFiller::Platform_Render( void )
{
    // (MP - 19.05.2004) Ideally this would go through the
    // Asura_PS2_2D_Graphics interface but that currently entirely revolves
    // around the crappy independent ASURA_TRANSLUCENCY_MODE stuff. We can't
    // do the blend we require here using that.

	Asura_PlatformRenderStates::CreateDefaultRenderStatePacket( Asura_PlatformRenderStates::CONTEXT_1 );
    Asura_PlatformRenderStates::SetAlphaBlend( SCE_GS_ALPHA_CS, SCE_GS_ALPHA_CD, SCE_GS_ALPHA_AD, SCE_GS_ALPHA_CD, 0 );
    Asura_PlatformRenderStates::SetPixelTest( false, false );
    Asura_PlatformRenderStates::ApplyRenderStates();

    const u_int uNumRegs  = 3;
    const u_int uQWC      = 1 + uNumRegs;
    const u_int uTotalQWs = 1 + uQWC;

    Asura_PS2_QW* const pxBase = Asura_PS2_DeferredRendering::GetVifBuffer( uTotalQWs );
    Asura_PS2_QW* pxOut = pxBase;

    (reinterpret_cast<Asura_PS2_DmaTag*>(pxOut++))->Set( uQWC,
                                                         Asura_PS2_DmaTag::PCE_NOCHANGE,
                                                         Asura_PS2_DmaTag::ID_CNT,
                                                         false,
                                                         0,
                                                         false,
                                                         SCE_VIF1_SET_NOP(false),
                                                         SCE_VIF1_SET_DIRECT(uQWC, false) );

    Asura_PS2_GifTag* pxGifTag = reinterpret_cast<Asura_PS2_GifTag*>( pxOut++ );
    pxGifTag->SetExceptRegs( 1,
                             true,
                             true,
                             SCE_GS_SET_PRIM(SCE_GS_PRIM_SPRITE, 0, false, false, true, false, 0, 0, 0),
                             SCE_GIF_PACKED,
                             uNumRegs );
    pxGifTag->uAllRegs = (SCE_GIF_PACKED_AD << 0) | (SCE_GS_XYZ2 << 4) | (SCE_GS_XYZ2 << 8);

    pxOut->u64s[0] = SCE_GS_SET_RGBAQ( (m_xFillColour.ColourComponents.B + 1) >> 1, // Colour components are wrong so we need to manually swizzle.
                                       (m_xFillColour.ColourComponents.G + 1) >> 1,
                                       (m_xFillColour.ColourComponents.R + 1) >> 1,
                                       0,                                           // Alpha is irrelevant with our blend.
                                       0x3F800000u );
	pxOut->u64s[1] = SCE_GS_RGBAQ;
    ++pxOut;

    const Asura_PS2_GS::ViewportParams& xViewportParams = ( Asura_PS2_GS::IsFullScreenRendering()
                                                                ? Asura_PS2_GS::GetViewportParamsForFullScreenRendering()
                                                                : Asura_PS2_GS::GetViewportParams(Asura_Camera::GetCameraIndex()) );

    pxOut->u32s[0] = xViewportParams.sPrimTransX + Asura_Maths::FloatToInt_Near( m_xCurTL.x * 16.0f );
    pxOut->u32s[1] = xViewportParams.sPrimTransY + Asura_Maths::FloatToInt_Near( m_xCurTL.y * 16.0f );
    pxOut->u64s[1] = 0;
    ++pxOut;

    pxOut->u32s[0] = xViewportParams.sPrimTransX + Asura_Maths::FloatToInt_Near( (m_xCurTL.x + m_xCurDimensions.x) * 16.0f );
    pxOut->u32s[1] = xViewportParams.sPrimTransY + Asura_Maths::FloatToInt_Near( (m_xCurTL.y + m_xCurDimensions.y) * 16.0f );
    pxOut->u64s[1] = 0;
    ++pxOut;

    Asura_Assert( static_cast<u_int>(pxOut - pxBase) == uTotalQWs, "" );
}

void Asura_ProgressDisplay_BackgroundFiller::Platform_Reset( void )
{
	// (AAM - 14.06.2004) Nothing to do on PS2
}
