//////////////////////////////////////////////////////////////////////////////
//
// Asura_PS2_eznetcnf.cpp
//
// Copyright (c) 2003 Rebellion. All rights reserved.
//
// Description:
//
//     Read ntcnf format file from memory card and set up the network based
//     on that. based on eznetcnf-0.9a\eznetcnf\sample\basic\testCnf.c
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

#include "Asura_PS2_eznetcnf.h"

#ifdef ASURA_SUPPORT_EXTERNAL_NETWORK

// PS2 libraries
#include <libeenet.h>
#include <eenetctl.h>
#include <eenetctlerrno.h>
#include <sys/socket.h>
#include <net/if.h>
#include <eekernel.h>
#include <sifdev.h>
#include <string.h>
#include <stdio.h>
#include <libmc.h>
#include <time.h>

// PS2 middleware
#include "Sony/eznetcnf/include/eznetcnf.h"
#include "Sony/eznetcnf/include/eznetctl_i18n.h"

// Asura independent
#include "core/Asura_LocalisedText.h"
#include "Core/Asura_Time_System.h"
#include "Core/Asura_Timers.h"
#include "Debugging/Asura_Assert.h"
#include "Debugging/Asura_Logfile.h"

// Asura PS2
#include "Core/Asura_PS2_OS.h"

// D E F I N E S /////////////////////////////////////////////////////////////
#define kNetcnfArgs "icon=foo\0iconsys=bar"

#define MAIN_MOD_ROOT     "IRX\\NET\\"

#define IOP_MOD_NETCNF		MAIN_MOD_ROOT "netcnf.irx"
#define IOP_MOD_EZNETCNF	MAIN_MOD_ROOT "eznetcnf.irx"


#define IOP_MOD_EZENTCTL	MAIN_MOD_ROOT "ezentctl.irx"
#define IOP_MOD_ENT_DEVM	MAIN_MOD_ROOT "ent_devm.irx"
#define IOP_MOD_EENETCTL	MAIN_MOD_ROOT "eenetctl.irx"
#define IOP_MOD_ENT_SMAP	MAIN_MOD_ROOT "ent_smap.irx"

#define IS_TRUE(ch)   ( (ch) && (ch) != 0xff)
#define IS_FALSE(ch)  (!(ch) || (ch) == 0xff)

#define IS_SCE(vendor)  (strncmp((vendor), "SCE", 3) == 0)


// stuff for eenet
#if defined(ASURA_SUPPORT_DEMONWARE) && defined(ASURA_DEMONWARE_USE_EENET_MEMORYMANAGEMENT)
// ARC - 28 April 2006 09:09:30 - If we're using DemonWare and sceEENetXXX memory functions, we need a bigger heap
#define EENET_MEM_SIZE		ASURA_DEMONWARE_HEAP_SIZE
#else
#define EENET_MEM_SIZE      (300 * 1024)    // adjust as necessary
#endif // ASURA_SUPPORT_DEMONWARE && ASURA_DEMONWARE_USE_EENET_MEMORYMANAGEMENT
#define EENET_STACK_SIZE    8192            // should not adjust
#define EENET_TOP_PRIO      32              // should not adjust
#define EENET_DEV_PRIO      48              // should not adjust

static void* g_pEENetMemory = NULL;
// end stuff for eenet

/*  --------------------------------------------------------
 *  global variables (ensures 64-byte address alignment)
 *  --------------------------------------------------------
 */

// stores list of combinations for one netdb
static ezNetCnfCombinationList_t g_xCombinationList;

// stores properties for one combination
static struct sceNetcnfifData g_xSelectedCombinationProperties;

// stores network connection status updates
static ezNetCtlStatus_t g_xCurrentConnectionStatus;

DEBUGVAR u_int dbg_uNetModuleRetries = 100;
DEBUGVAR bool dbg_bEENetNoDebug = false;	// debugging on if _DEBUG
DEBUGVAR u_int dbg_uEZNetCnfConnectionTimeout = 30000;	// msec
DEBUGVAR u_int dbg_uEZNetCnfTimeBetweenUpdatesWhenConnected = 1000;	// msec

// (MH - 31.03.2003) hard-code the netcnf data to be loaded from 1st memory card
static const char* const g_szNetdbPath = "mc0:";
//static const char* const g_szNetdbPath = "host0:/usr/local/sce/conf/net/net.db";	// test settings...

Asura_PS2_EZNetCnf::NetState Asura_PS2_EZNetCnf::s_eState = Asura_PS2_EZNetCnf::NetState_None;
Asura_PS2_EZNetCnf::NetCnfError Asura_PS2_EZNetCnf::s_eError = Asura_PS2_EZNetCnf::Error_None;
u_int Asura_PS2_EZNetCnf::s_uNumConfigs = 0;

Asura_PS2_EZNetCnf::MemCardState Asura_PS2_EZNetCnf::s_eMemoryCardState = Asura_PS2_EZNetCnf::MemCard_Recheck;
int Asura_PS2_EZNetCnf::s_iMemoryCardIsFormatted = 1;
bool Asura_PS2_EZNetCnf::s_bMemoryCardCheckInProgress = false;

bool Asura_PS2_EZNetCnf::s_bCanUseLANBroadcast = false;
bool Asura_PS2_EZNetCnf::s_bNetCnfAndNetCtlIRXsLoaded = false;

char Asura_PS2_EZNetCnf::s_szMsg[kStrBufferSize];
u_int Asura_PS2_EZNetCnf::s_uTimerValue;

bool Asura_PS2_EZNetCnf::s_bIsPPPoE = false;
bool Asura_PS2_EZNetCnf::s_bAssumePPPoEConnectionDead = false;

static u_int g_uTimeOfLastGetStatusCall = 0;
static u_int g_uTimeCableWasDisconnected = 0;

/*  --------------------------------------------------------
 *  function prototypes
 *  --------------------------------------------------------
 */

#if _DEBUG
static const char *makeIfname(ezNetCtlStatus_t *);
#endif

/*  --------------------------------------------------------
 *  main function
 *  --------------------------------------------------------
 */

void Asura_PS2_EZNetCnf::BeginNetCnfSelection( void )
{
	Asura_Assert(!s_bHasConnection, "connection already exists");
	ShutdownNetworkConnection();	// shut it down in case that fired.

	// set up language based on console settings
	//ezNetCtlSetScfLanguage();
    
	switch(Asura_LocalisedText::GetCurrentLanguage())
	{
	default:
		Asura_Assert(0,"invalid language");
		//fallthrough
	case ASURA_LT_L_ENGLISH:
		//fallthrough
	case ASURA_LT_L_AMERICAN:
		ezNetCtlSetLanguage(1);
		break;
	case ASURA_LT_L_FRENCH:
		ezNetCtlSetLanguage(2);
		break;
	case ASURA_LT_L_SPANISH:
		ezNetCtlSetLanguage(3);
		break;
	case ASURA_LT_L_GERMAN:
		ezNetCtlSetLanguage(4);
		break;
	case ASURA_LT_L_ITALIAN:
		ezNetCtlSetLanguage(5);
		break;
	}

	// assume the memory card lib is already set up correctly
	s_eState = NetState_LoadingIRXs;
	s_eError = Error_None;
	s_eMemoryCardState = MemCard_Recheck;
	UpdateMemoryCard();

	s_bCanUseLANBroadcast = false;

	s_szMsg[0] = 0;

	g_uTimeCableWasDisconnected = 0;
}


void Asura_PS2_EZNetCnf::RestartNetCnfSelection( void )
{
	Asura_Assert(!s_bHasConnection, "connection already exists");
	
	if ( !s_bNetCnfAndNetCtlIRXsLoaded )
	{
		s_eState = NetState_LoadingIRXs;
	}
	else
	{
		s_eState = NetState_WaitingForMemCardState;
	}
	s_eError = Error_None;
	s_eMemoryCardState = MemCard_Recheck;
	UpdateMemoryCard();

	s_szMsg[0] = 0;

	g_uTimeCableWasDisconnected = 0;
}


DEBUGVAR int dbg_iForceNetCnfErrorInSelection = -1;

bool Asura_PS2_EZNetCnf::UpdateNetCnfSelection( void )
{
	if( dbg_iForceNetCnfErrorInSelection > Error_None && dbg_iForceNetCnfErrorInSelection <= Error_LoadingNetworkIRXsFailed && NetState_Error != s_eState )
	{
		SetError( static_cast<NetCnfError>(dbg_iForceNetCnfErrorInSelection) );
		return false;
	}

	switch (s_eState)
	{
		case NetState_LoadingIRXs:
		{
			LoadNetCnfAndNetCtlIRXs();
			break;
		}

		case NetState_WaitingForMemCardState:
		{
			UpdateMemoryCard();
			if (s_eMemoryCardState == MemCard_OK)
			{
				s_eState = NetState_GettingConfigs;
			}
			else if (s_eMemoryCardState == MemCard_NotPresent)
			{
				SetError(Error_NoMemoryCardInSlot1);
			}
			else if (s_eMemoryCardState == MemCard_NotFormatted)
			{
				SetError(Error_MemoryCardNotFormatted);
			}
			break;
		}

		case NetState_GettingConfigs:
		{
			GetConfigs();
			break;
		}

		case NetState_DisplayingConfigs:
		{
			UpdateMemoryCard();
			
			if (s_eMemoryCardState == MemCard_NotPresent)
			{
				SetError(Error_NoMemoryCardInSlot1);
			}
			else if (s_eMemoryCardState == MemCard_NotFormatted)
			{
				SetError(Error_MemoryCardNotFormatted);
			}
			break;
		}

		case NetState_Initialising:
		{
			UpdateInitialisingState();
			break;
		}
		
		case NetState_Connected:
		{
			// do nothing
			break;
		}

		case NetState_Error:
		{
			break;
		}

		default:
		{
			Asura_Assert(false, "unhandled state");
			break;
		}
	}

	return (s_eState != NetState_Error);
}


void Asura_PS2_EZNetCnf::FinishNetCnfSelection( void )
{
	UnloadNetCnfAndNetCtlIRXs();
}


bool Asura_PS2_EZNetCnf::IsCablePluggedIn( void )
{
	Asura_Assert(s_bHasConnection, "no connection - can't determine cable link status");
	if (!s_bHasConnection)
	{
		if (IsPPPoE())
		{
			s_bAssumePPPoEConnectionDead = true;
		}
		return false;
	}
	
	// call private update function
	UpdateConnectedStatus();

	switch (g_xCurrentConnectionStatus.link)
	{
		case 0:
		{
			// it is not safe to report cable unplugged until link == 0 for 10 seconds.
			// see tech note on ps2-pro.com
			u_int uMilliSecsSinceStart = Asura_Timers::GetMSecsSinceStart();
			if (g_uTimeCableWasDisconnected == 0)
			{
				g_uTimeCableWasDisconnected = uMilliSecsSinceStart;
			}
			u_int uTimeDisconnected = uMilliSecsSinceStart - g_uTimeCableWasDisconnected;
			// return false (cable unplugged) if time disconnected >= 10000 msecs
			if (uTimeDisconnected >= 10000 && IsPPPoE())
			{
				s_bAssumePPPoEConnectionDead = true;
			}
			return (uTimeDisconnected < 10000);
		}
		case 1:
		{
			g_uTimeCableWasDisconnected = 0;
			return true;
		}
		default:
		{
			Asura_Assert(false, "couldn't determine cable link status");
			if (IsPPPoE())
			{
				s_bAssumePPPoEConnectionDead = true;
			}
			return false;
		}
	}
}


bool Asura_PS2_EZNetCnf::IsConnectionOK( void )
{
	Asura_Assert(s_bHasConnection, "no connection - can't determine connection status");
	if (!s_bHasConnection)
	{
		return false;
	}
	
	// call private update function
	UpdateConnectedStatus();

	if (g_xCurrentConnectionStatus.state != eStateStarted && IsPPPoE())
	{
		s_bAssumePPPoEConnectionDead = true;
	}

	if (IsPPPoE() && s_bAssumePPPoEConnectionDead)
	{
		return false;
	}

	return (g_xCurrentConnectionStatus.state == eStateStarted);
}


void Asura_PS2_EZNetCnf::UpdateConnectedStatus( void )
{
	u_int uMsecSinceStart = Asura_Timers::GetMSecsSinceStart();
	u_int uTimeSinceLastUpdate = uMsecSinceStart - g_uTimeOfLastGetStatusCall;
	if (uTimeSinceLastUpdate > dbg_uEZNetCnfTimeBetweenUpdatesWhenConnected)
	{
		g_uTimeOfLastGetStatusCall = uMsecSinceStart;
		int iErr = ezNetCtlGetStatus(&g_xCurrentConnectionStatus, NULL, 0);
		Asura_Assertf(iErr == 0, ("ezNetCtlGetStatus() returned %d", ezNetCtlGetStatus));
	}
}


void Asura_PS2_EZNetCnf::ShutdownNetworkConnection_Internal( void )
{
    Asura_OutputToDebugger(("\nDownInterface(%s)\n", makeIfname(&g_xCurrentConnectionStatus)));
    while (0 == ezNetCtlDownInterface()) 
	{
        int iErr = ezNetCtlWaitEventTO(10 * 1000);
        if ( (iErr < 0) || (iErr == eIEV_DownCompleted) )
		{
            break;
        }
    }
	
	// we left the RPC functions registered - this will remove them
	ezNetCtlExit(0);

	ShutdownEENet();

	s_bHasConnection = false;
}


u_int Asura_PS2_EZNetCnf::GetNumConfigs( void )
{
	Asura_Assert(s_eState == NetState_DisplayingConfigs, "configs not retrieved yet");
	return s_uNumConfigs;
}


const char* Asura_PS2_EZNetCnf::GetConfigName( u_int uConfigNumber )
{
	Asura_Assert(uConfigNumber > 0 && uConfigNumber <= 4, "invalid config number");
	if (!(uConfigNumber > 0 && uConfigNumber <= 4))
	{
		return "";
	}
	return ezNetCnfGetCombination(&g_xCombinationList, uConfigNumber)->ifcName;
}


// config type (e.g. SCE/Ethernet (Network Adaptor))
const char* Asura_PS2_EZNetCnf::GetConfigType( u_int uConfigNumber )
{
	Asura_Assert(uConfigNumber > 0 && uConfigNumber <= 4, "invalid config number");
	if (!(uConfigNumber > 0 && uConfigNumber <= 4))
	{
		return "";
	}
	return ezNetCnfGetCombination(&g_xCombinationList, uConfigNumber)->devName;
}


// used to filter out empty slots
bool Asura_PS2_EZNetCnf::ConfigIsPresent( u_int uConfigNumber )
{
	Asura_Assert(uConfigNumber > 0 && uConfigNumber <= 4, "invalid config number");
	const char* szCombinationName = ezNetCnfGetCombination(&g_xCombinationList, uConfigNumber)->combinationName;
	return (szCombinationName[0] != 0);
}


// used to filter out invalid entries
bool Asura_PS2_EZNetCnf::ConfigIsSelectable( u_int uConfigNumber )
{
	Asura_Assert(uConfigNumber > 0 && uConfigNumber <= 4, "invalid config number");
	
	const ezNetCnfCombination_t* pxCombination = ezNetCnfGetCombination(&g_xCombinationList, uConfigNumber);
	if (pxCombination->status >= 0)
	{
		// broadband-only, ethernet or PPPoE only.
		// note this will not filter out USB ethernet configs - we will return an error if one is selected
		return ((pxCombination->iftype == eIftypeEther) || (pxCombination->iftype == eIftypePPPoE));
	}
	else 
	{
		// invalid entry
		return false;
	}
}


u_int Asura_PS2_EZNetCnf::GetDefaultConfig( void )
{
	return g_xCombinationList.defaultNumber;
}

	
void Asura_PS2_EZNetCnf::SetError( NetCnfError eError )
{
	if (s_eState == NetState_Initialising)
	{
		UnloadNetCnfAndNetCtlIRXs();

		ShutdownNetworkConnection();
	}
	s_eState = NetState_Error;
	s_eError = eError;
}


void Asura_PS2_EZNetCnf::LoadNetCnfAndNetCtlIRXs( void )
{
	Asura_Assert(!s_bNetCnfAndNetCtlIRXsLoaded, "IRXs already loaded");

    // netcnf must be loaded before eznetcnf.
    int iErr = Asura_PS2_OS::LoadIRXModule(IOP_MOD_NETCNF, sizeof(kNetcnfArgs), kNetcnfArgs, false, dbg_uNetModuleRetries);
	if (iErr < 0)
	{
		SetError(Error_LoadingIRXsFailed);
		return;
	}

	// load eznetcnf.irx, then bind it with eznetcnf.a.
	iErr = Asura_PS2_OS::LoadIRXModule(IOP_MOD_EZNETCNF, 0, NULL, false, dbg_uNetModuleRetries);
	if (iErr < 0)
	{
		SetError(Error_LoadingIRXsFailed);
		return;
	}

    iErr = ezNetCnfInit();
	if (iErr < 0)
	{
		SetError(Error_LoadingIRXsFailed);
		return;
	}
	
    s_eState = NetState_WaitingForMemCardState;
	s_bNetCnfAndNetCtlIRXsLoaded = true;
}


void Asura_PS2_EZNetCnf::GetConfigs( void )
{
    int iErr;
    
    // build up a combination list.
	u_int uFlags = ezNetCnfRejectAOLDialup | ezNetCnfRejectModem | ezNetCnfRejectUSB;	// we don't support AOL
    iErr = ezNetCnfGetCombinationList(g_szNetdbPath, uFlags, &g_xCombinationList);

    if (iErr <= 0) 
	{
        switch (iErr) 
		{
			case 0:
			{
				Asura_OutputToDebugger(("No network configurations at %s\n", g_szNetdbPath));
				SetError(Error_NoConfigsOnMemoryCard);
				break;
			}
			case sceNETCNFIF_MAGIC_ERROR:
			{
				Asura_OutputToDebugger(("sceNETCNFIF_MAGIC_ERROR at %s\n", g_szNetdbPath));
				SetError(Error_MagicError);
				break;
			}
//			case -21:
//			{
//				Asura_OutputToDebugger(("error -21 (no card?) at %s\n", g_szNetdbPath));
//				SetError(Error_NoMemoryCardInSlot1);
//				break;
//			}
			default:
			{
				Asura_OutputToDebugger(("Other error %d at %s\n", iErr, g_szNetdbPath));
				SetError(Error_CorruptedData);
				break;
			}
		}
    }
	else
	{
		s_uNumConfigs = static_cast<u_int>(iErr);
		s_eState = NetState_DisplayingConfigs;
	}
}

bool Asura_PS2_EZNetCnf::IsPPPoE()
{
	Asura_Assert(s_bHasConnection, "no connection");
	return s_bIsPPPoE;
}

static bool g_bReachedStarting;

void Asura_PS2_EZNetCnf::InitConfig( u_int uConfigNumber )
{	
	Asura_Assert(!s_bHasConnection, "connection already exists");
	
	const char* comboName = ezNetCnfGetCombination(&g_xCombinationList, uConfigNumber)->combinationName;

	s_bIsPPPoE = (ezNetCnfGetCombination(&g_xCombinationList, uConfigNumber)->iftype == eIftypePPPoE);
	s_bAssumePPPoEConnectionDead = false;
    // show the specified combination in detail.
    int iErr = ezNetCnfGetProperties(g_szNetdbPath, comboName, &g_xSelectedCombinationProperties);
    if (iErr < 0) 
	{
        Asura_Assertf(false, ("Failed to get properties of \"%s\"\n", comboName));
        if (iErr == sceNETCNFIF_ENTRY_NOT_FOUND)
		{
			SetError(Error_NoMemoryCardInSlot1);
		}
		else
		{
			// not sure what the error is
			SetError(Error_CorruptedData);
		}
		return;
    }

    // load network modules required by this configuration.
	if ( (g_xSelectedCombinationProperties.dev_type == sceNetcnfifData_type_eth) || 
		 (g_xSelectedCombinationProperties.dev_type == sceNetcnfifData_type_ppp) )
	{
		SetError(Error_HardwareNotSupported);
		return;
	}

	bool bOK = EENetStartSpecific(&g_xSelectedCombinationProperties);

    if (bOK)
	{
		// load eznetctl.irx, then bind it with eznetcnf.a.
		bOK = (0 <= Asura_PS2_OS::LoadIRXModule(IOP_MOD_EZENTCTL, 0, NULL, false, dbg_uNetModuleRetries));
	}

	if (!bOK)
	{
		SetError(Error_LoadingNetworkIRXsFailed);
		return;
	}
	
    ezNetCtlInit();

    // bind specified network configuration to a network interface.
    int id = ezNetCtlSetCombination(g_szNetdbPath, comboName);

    if (id <= 0) 
	{
        Asura_Assertf(false, ("The hardware \"%s/%s\" is not supported or not connected.\n",
            g_xSelectedCombinationProperties.vendor, g_xSelectedCombinationProperties.product));
        SetError(Error_HardwareNotConnected);

		// SNH: copied these two lines from BHD's branch.
		// Comment was 'fix for crash bug (mem card removal on conf selection)'
		ezNetCtlExit(0);
		ShutdownEENet();

		return;
    }
    
    /*
     *  use ezNetCtlUpInterface() to start configured network I/F.
     *  the stack then performs data link negotiation.
     *
     *  use ezNetCtlGetStatus() or ezNetCtlWaitEvent() to check progress
     */
    iErr = ezNetCtlUpInterface();
    iErr = ezNetCtlGetStatus(&g_xCurrentConnectionStatus, NULL, 0);
    Asura_OutputToDebugger(("UpInterface(%s)\n", makeIfname(&g_xCurrentConnectionStatus)));

	s_eState = NetState_Initialising;

	s_bHasConnection = true;

	s_uTimerValue = Asura_Timers::GetMSecsSinceStart();

	g_bReachedStarting = false;
}

void Asura_PS2_EZNetCnf::UpdateInitialisingState ( void )
{
	// non-blocking poll for connection start
	int iErr = ezNetCtlGetStatus(&g_xCurrentConnectionStatus, s_szMsg, kStrBufferSize);
    
	if (iErr) 
	{
        Asura_OutputToDebugger(("before UpCompleted: ezNetCtlGetStatus returned %d\n", iErr));
		SetError(Error_ConnectionFailed);
    }
    else
	{
        
        // immediately after UpInterface() and before libeenet threads
        // begin processing, the initial I/F state reports Stopped.
        // track a reachedStarting variable to determine if the I/F
        // is really Stopped for good.
        if (g_xCurrentConnectionStatus.state == eStateStarting) 
		{
            g_bReachedStarting = true;
        } 
		else if (g_xCurrentConnectionStatus.state == eStateStarted) 
		{
			Asura_OutputToDebugger(("%s started\n", makeIfname(&g_xCurrentConnectionStatus)));
			s_eState = NetState_Connected;
			s_bCanUseLANBroadcast = (g_xCurrentConnectionStatus.iftype == sceEENET_IFTYPE_ETHER);
			
			time_t xTimeTTime = sceEENetTime(0);
			Asura_Time xAsuraTime;
			xAsuraTime.FillFromLibCTime( (u_int)xTimeTTime );
			xAsuraTime.ConvertUTCToLocalTime();
			
			tm* pxTm;
			pxTm = gmtime( &xTimeTTime );
			
			Asura_OutputToDebugger(( "gmtime = %02d:%02d:%02d\n", pxTm->tm_hour, pxTm->tm_min, pxTm->tm_sec ));

			pxTm = localtime( &xTimeTTime );
			
			Asura_OutputToDebugger(( "localtime = %02d:%02d:%02d\n", pxTm->tm_hour, pxTm->tm_min, pxTm->tm_sec ));

			Asura_OutputToDebugger(( "Asura_Time_System convertion of sceEENetTime = %S\n", Asura_Time_System::GetDateAndTimeString(xAsuraTime, ASURA_DATE_SHORT) ));

			Asura_OutputToDebugger(( "Asura_Time_System::GetCurrentTimeString = %S\n", Asura_Time_System::GetCurrentTimeString() ));
		} 
		else if (g_xCurrentConnectionStatus.state == eStateStopped && g_bReachedStarting) 
		{
			Asura_OutputToDebugger(("%s stopped, message:\n%s\n", 
				   makeIfname(&g_xCurrentConnectionStatus),
				   s_szMsg));
			if (sceEENETCTL_E_PppAuth == g_xCurrentConnectionStatus.error)
			{
				SetError(Error_AuthorisationError);
			}
			else
			{
				SetError(Error_ConnectionFailed);
			}
        }
	}

	// SNH: copied from BHD's branch. Checkin comment was:
	//	'fix so displays correct error on PPPoE incorrect username or password (was displaying cable error)'
	if (g_xCurrentConnectionStatus.link == 0)
	{
		u_int uMilliSecsSinceStart = Asura_Timers::GetMSecsSinceStart();
		if (g_uTimeCableWasDisconnected == 0)
		{
			g_uTimeCableWasDisconnected = uMilliSecsSinceStart;
		}
		u_int uTimeDisconnected = uMilliSecsSinceStart - g_uTimeCableWasDisconnected;
		// return false (cable unplugged) if time disconnected >= 10000 msecs
		if (uTimeDisconnected >= 10000)
		{
			SetError(Error_CableUnplugged);
		}
	}

	u_int uElapsedTime = Asura_Timers::GetMSecsSinceStart() - s_uTimerValue;
	if (uElapsedTime > dbg_uEZNetCnfConnectionTimeout)	// should be debugvar
	{
		SetError(Error_ConnectionTimedOut);
	}
}


void Asura_PS2_EZNetCnf::UpdateMemoryCard( void )
{
    if (!s_bMemoryCardCheckInProgress) 
	{
		int iPort = 0;
        int iResult = sceMcGetInfo(iPort, 0, NULL, NULL, &s_iMemoryCardIsFormatted);

		if (s_eMemoryCardState == MemCard_Recheck)
		{
			if (iResult == 0)
			{
				s_eMemoryCardState = MemCard_Unknown;
			}
		}
    }

    // check the status of the ongoing sceMcGetInfo() call.
	int iFunctionNumber;
	int iResult;
    int iSyncResult = sceMcSync(1, &iFunctionNumber, &iResult);
	s_bMemoryCardCheckInProgress = (iSyncResult == sceMcExecRun);
	
	if (s_eMemoryCardState != MemCard_Recheck)
	{
		if ( (iSyncResult == sceMcExecFinish) && (iFunctionNumber == sceMcFuncNoCardInfo) )
		{
			if ((iResult == sceMcResChangedCard) || (iResult == sceMcResSucceed))
			{
				if (s_iMemoryCardIsFormatted == 1)
				{
					s_eMemoryCardState = MemCard_OK;
				}
				else
				{
					s_eMemoryCardState = MemCard_NotFormatted;
				}
			}
			else
			{
				s_eMemoryCardState = MemCard_NotPresent;
			}
		}
	}
}


void Asura_PS2_EZNetCnf::UnloadNetCnfAndNetCtlIRXs( void )
{
	Asura_Assert(s_bNetCnfAndNetCtlIRXsLoaded, "IRXs not loaded");

    // unbind and unload eznetcnf and eznetctl.
    // these do not affect the active network connection.
    
	int iNetCnfExitRet;

	do 
	{
		iNetCnfExitRet = ezNetCnfExit();
		if (iNetCnfExitRet < 0)
		{
			Asura_OutputToDebugger_Retail(("ezNetCnfExit failed, retrying.\n"));
			Asura_PS2_OS::DelayThread(10);
		}
	}
	while (iNetCnfExitRet < 0);

    ezNetCtlExit(ezNetCtlKeepEventHandler);
	
	s_bNetCnfAndNetCtlIRXsLoaded = false;
}


#if _DEBUG
static const char * makeIfname(ezNetCtlStatus_t *status)
{
    static char tmp[64];

    if (strlen(status->ifname2)) {
        sprintf(tmp, "I/F=%s+%s", status->ifname, status->ifname2);
    } else {
        sprintf(tmp, "I/F=%s", status->ifname);
    }

    return tmp;
}
#endif


bool Asura_PS2_EZNetCnf::LoadEENetCore()
{
    bool bOK = true;

    // netcnf.irx is needed for both Inet and EEnet
    if (sceSifSearchModuleByName("NET_configuration") < 0) 
	{
        bOK = (0 <= Asura_PS2_OS::LoadIRXModule(IOP_MOD_NETCNF, sizeof(kNetcnfArgs), kNetcnfArgs, false, dbg_uNetModuleRetries));
    }

    if (bOK)
	{
		bOK = (0 <= Asura_PS2_OS::LoadIRXModule(IOP_MOD_ENT_DEVM, 0, NULL, false, dbg_uNetModuleRetries));
	}
    
	if (bOK)
	{
		bOK = (0 <= Asura_PS2_OS::LoadIRXModule(IOP_MOD_EENETCTL, 0, NULL, false, dbg_uNetModuleRetries));
	}

    if (!bOK) 
	{
        return false;
    }

    // replace this with your memory allocator, if necessary
    g_pEENetMemory = Asura_AllocateAlign(EENET_MEM_SIZE, ASURA_ALIGN_64_BYTE);
    if (g_pEENetMemory == NULL) 
	{
        Asura_Assert(false, "Failed to allocate memory for libeenet");
        return false;
    }

    int iErr = sceEENetInit(g_pEENetMemory, EENET_MEM_SIZE, EENET_TOP_PRIO, EENET_STACK_SIZE, EENET_DEV_PRIO);
    if (iErr) 
	{
        Asura_Assertf(false, ("sceEENetInit failed with %d", iErr));
        return false;
    }

#if _DEBUG
    if (!dbg_bEENetNoDebug)
	{
		sceEENetSetLogLevel(EENET_LOG_INFO);
	}
#endif

#if _DEBUG
	int iVerbose = dbg_bEENetNoDebug ? 0 : 1;
#else
	int iVerbose = 0;
#endif
	
    iErr = sceEENetCtlInit(EENET_STACK_SIZE, EENET_DEV_PRIO, // RPC thread
                          EENET_STACK_SIZE, EENET_DEV_PRIO, // eenetctl thread
                          EENET_STACK_SIZE, EENET_DEV_PRIO, // I/F thread
                          iVerbose, 1);            // verbose, no_auto

    if (iErr) 
	{
        Asura_Assertf(false, ("sceEENetCtlInit failed with %d", iErr));
        return false;
    }

	// (MH - 29.09.2004) not sure where to put these, but the defaults result in DNS lookup failures taking 1m15s
	// The timeout gets doubled for each retry, so the defaults (5,4) go 5, 10, 20, 40
	// If we use (1,4) you get 1+2+4+8 = 15 seconds which is long enough I think.
	int iRetransTime = 1;
	int iRetries = 4;
	sceEENetSetResolvTO(iRetransTime, iRetries);// NB this only affects the calling thread.
	Asura_OutputToDebugger(("set EEnet DNS timeouts for main thread only, to %d retries, initial timeout %d second\n", iRetries, iRetransTime));

    return true;
}

/*
 *  load SCE Ethernet drivers, then register
 */
bool Asura_PS2_EZNetCnf::LoadEENetSMAP( void )
{
    Asura_Assert(sceSifSearchModuleByName("dev9") >= 0, "dev9.irx not loaded!");
    
	bool bOK = (0 <= Asura_PS2_OS::LoadIRXModule(IOP_MOD_ENT_SMAP, 0, NULL, false, dbg_uNetModuleRetries));
    if (bOK)
	{
		bOK = (0 == sceEENetDeviceSMAPReg(EENET_STACK_SIZE, EENET_STACK_SIZE));
	}
    return bOK;
}


/*
 *  startup libeenet with only the necessary device drivers
 *  for the given network configuration.
 */
bool Asura_PS2_EZNetCnf::EENetStartSpecific(sceNetcnfifData_t* pxData)
{
#if _DEBUG
	// printf the settings for debugging
	Asura_OutputToDebugger(("\n-------------------\n"));
	if (pxData->pppoe == sceNetcnfifData_pppoe_use)
	{
		Asura_OutputToDebugger(("Setting uses PPPoE.\n"));
		Asura_OutputToDebugger((" user = %s, password = %s.\n", pxData->auth_name, pxData->auth_key));
	}
	else if (pxData->dhcp == sceNetcnfifData_dhcp_use)
	{
		Asura_OutputToDebugger(("Setting uses DHCP.\n"));
	}
	else
	{
		Asura_OutputToDebugger(("Setting does not use DHCP.\n"
			" IP address   = %s.\n"
			" Subnet mask  = %s.\n"
			" Gateway      = %s.\n"
			" DNS server 1 = %s.\n"
			" DNS server 2 = %s.\n",
			pxData->address, pxData->netmask, pxData->gateway, pxData->dns1_address, pxData->dns2_address ));
	}
	Asura_OutputToDebugger(("\n-------------------\n"));
#endif
	
	bool bOK = LoadEENetCore();
	if (bOK)
	{
		// check sceNetcnfifData_t.dev_type for device driver selection
		if (pxData->dev_type == sceNetcnfifData_type_eth) 
		{
			//iErr |= eenetLoadETH();
			Asura_Assert(false, "USB ethernet not supported");
		} 
		else if (pxData->dev_type == sceNetcnfifData_type_nic) 
		{
			bOK = LoadEENetSMAP();
		} 
		else if (pxData->dev_type == sceNetcnfifData_type_ppp) 
		{
			//iErr |= eenetLoadPPP();
			Asura_Assert(false, "PPP not supported");
		}
	}
    return bOK;
}

/*
 *  shutdown and unload libeenet modules and resources
 *
 *  dev9.irx cannot be unloaded.
 */
bool Asura_PS2_EZNetCnf::ShutdownEENet( void )
{
    int iErr = 0;
    
    // if any interfaces are still up (or in the process of shutting down),
    // sceEENETCTL_E_Busy and sceEENETCTL_E_DevBusy errors will occur.
//    if ( 0 <= sceSifSearchModuleByName("ent_eth") ) {
//        iErr |= sceEENetDeviceETHUnreg();
//    }
    if ( 0 <= sceSifSearchModuleByName("ent_smap") ) {
        iErr |= sceEENetDeviceSMAPUnreg();
    }
//    if ( 0 <= sceSifSearchModuleByName("ent_ppp") ) {
//        iErr |= sceEENetDevicePPPUnreg();
//    }
    if (iErr) {
        Asura_Assert(false, "Some interfaces did not detach\n");
    }

    if ( (iErr = sceEENetCtlTerm()) < 0 ) {
        Asura_Assertf(false, ("sceEENetCtlTerm() failed %d\n", iErr));
    }
    if ( (iErr = sceEENetTerm()) < 0 ) {
        Asura_Assertf(false, ("sceEENetTerm() failed %d\n", iErr));
    }

    if (g_pEENetMemory) 
	{
        Asura_DeallocateAlign(g_pEENetMemory);
		g_pEENetMemory = NULL;
    }

//#if (SUPPORT_SCE_MODEM)
//    iErr |= myUnloadModuleByName("INET_SPEED_UART_driver");
//    iErr |= myUnloadModuleByName("ent_ppp");
//#endif
    iErr |= Asura_PS2_OS::UnloadIRXModuleByName("ent_smap");
//    iErr |= myUnloadModuleByName("ent_eth");
    // NOTE: ezentctl must be unloaded before eenetctl can be unloaded
    iErr |= Asura_PS2_OS::UnloadIRXModuleByName("eenetctl");
    iErr |= Asura_PS2_OS::UnloadIRXModuleByName("ent_devm");
    iErr |= Asura_PS2_OS::UnloadIRXModuleByName("NET_configuration");

    if (iErr) 
	{
        Asura_Assert(false, "Some modules did not unload. (Forgot to call FinishNetCnfSelection?)\n");
    }
    return (0 != iErr);
}


const char* Asura_PS2_EZNetCnf::GetStatusMessage( void )
{
	return s_szMsg;
}


// function which gives an example of usage for reference, and can be used
// to quickly implement config selection for prototypes that don't have menus.
// if bAutoChooseDefault is set to true, the default config is selected.
// Set to false if you want to test using the DEBUGVAR dbg_uHACKConfigChoice

#include "Core/Asura_Timers.h"
#include <libgraph.h>
DEBUGVAR u_int dbg_uHACKConfigChoice = 0;

void Asura_PS2_EZNetCnf::FakeConfigSelectionMenu(bool bAutoChooseDefault /* = true */)
{
    Asura_PS2_EZNetCnf::BeginNetCnfSelection();

	static char szLastMessage[kStrBufferSize] = "";
	bool bDoneDisplay = false;
	bool bProcessing = true;
	u_int uLastConfigChoice;

	while (bProcessing)
	{
		bool bOK = Asura_PS2_EZNetCnf::UpdateNetCnfSelection();
		Asura_Timers::HandlePrimaryTimer();
		if (!bOK)
		{
			Asura_PS2_EZNetCnf::NetCnfError eError = Asura_PS2_EZNetCnf::GetError();
			switch (eError)
			{
				case Asura_PS2_EZNetCnf::Error_None: Asura_OutputToDebugger(("<UI ERROR DISPLAY> Error_None\n")); break;
				case Asura_PS2_EZNetCnf::Error_LoadingIRXsFailed: Asura_OutputToDebugger(("<UI ERROR DISPLAY> Error_LoadingIRXsFailed\n")); break;
				case Asura_PS2_EZNetCnf::Error_NoConfigsOnMemoryCard: Asura_OutputToDebugger(("<UI ERROR DISPLAY> Error_NoConfigsOnMemoryCard\n")); break;
				case Asura_PS2_EZNetCnf::Error_MagicError: Asura_OutputToDebugger(("<UI ERROR DISPLAY> Error_MagicError\n")); break;
				case Asura_PS2_EZNetCnf::Error_NoMemoryCardInSlot1: Asura_OutputToDebugger(("<UI ERROR DISPLAY> Error_NoMemoryCardInSlot1\n")); break;
				case Asura_PS2_EZNetCnf::Error_MemoryCardNotFormatted: Asura_OutputToDebugger(("<UI ERROR DISPLAY> Error_MemoryCardNotFormatted\n")); break;
				case Asura_PS2_EZNetCnf::Error_CorruptedData: Asura_OutputToDebugger(("<UI ERROR DISPLAY> Error_CorruptedData\n")); break;
				case Asura_PS2_EZNetCnf::Error_HardwareNotConnected: Asura_OutputToDebugger(("<UI ERROR DISPLAY> The hardware %s is not connected\n", Asura_PS2_EZNetCnf::GetConfigType(uLastConfigChoice))); break;
				case Asura_PS2_EZNetCnf::Error_HardwareNotSupported: Asura_OutputToDebugger(("<UI ERROR DISPLAY> The hardware %s is not supported\n", Asura_PS2_EZNetCnf::GetConfigType(uLastConfigChoice))); break;
				case Asura_PS2_EZNetCnf::Error_CableUnplugged: Asura_OutputToDebugger(("<UI ERROR DISPLAY> Error_CableUnplugged\n")); break;
				case Asura_PS2_EZNetCnf::Error_ConnectionFailed: Asura_OutputToDebugger(("<UI ERROR DISPLAY> Error_ConnectionFailed.\n%s\n", Asura_PS2_EZNetCnf::GetStatusMessage())); break;
				case Asura_PS2_EZNetCnf::Error_AuthorisationError: Asura_OutputToDebugger(("<UI ERROR DISPLAY> Error_AuthorisationError\n")); break;
				case Asura_PS2_EZNetCnf::Error_ConnectionTimedOut: Asura_OutputToDebugger(("<UI ERROR DISPLAY> Error_ConnectionTimedOut\n")); break;
				case Asura_PS2_EZNetCnf::Error_LoadingNetworkIRXsFailed: Asura_OutputToDebugger(("<UI ERROR DISPLAY> Error_LoadingNetworkIRXsFailed\n")); break;
			}
			Asura_Assert(false, "");
			Asura_PS2_EZNetCnf::RestartNetCnfSelection();
			bDoneDisplay = false;
			szLastMessage[0] = 0;
		}
		else
		{
			const char* szCurMsg = Asura_PS2_EZNetCnf::GetStatusMessage();
			if (strcmp(szCurMsg, szLastMessage))
			{
				Asura_OutputToDebugger(("<UI STATUS DISPLAY> %s\n", szCurMsg));
				strcpy(szLastMessage, szCurMsg);
			}
			
			switch (Asura_PS2_EZNetCnf::GetState())
			{
				case Asura_PS2_EZNetCnf::NetState_None: 
				{
					Asura_Assert(false, "");
					break;
				}

				case Asura_PS2_EZNetCnf::NetState_LoadingIRXs: 
				case Asura_PS2_EZNetCnf::NetState_WaitingForMemCardState:
				case Asura_PS2_EZNetCnf::NetState_GettingConfigs:
				{
					break;
				}

				case Asura_PS2_EZNetCnf::NetState_DisplayingConfigs: 
				{
					if (!bDoneDisplay)
					{
						bDoneDisplay = true;
						u_int uDefaultConfig = Asura_PS2_EZNetCnf::GetDefaultConfig();
						for (u_int u = 1; u < Asura_PS2_EZNetCnf::GetNumConfigs() + 1; ++u)
						{
							if (Asura_PS2_EZNetCnf::ConfigIsPresent(u))
							{
								bool bConfigIsSelectable = Asura_PS2_EZNetCnf::ConfigIsSelectable(u);
								const char* szConfigName = Asura_PS2_EZNetCnf::GetConfigName(u);
								const char* szConfigType = Asura_PS2_EZNetCnf::GetConfigType(u);
								const char* szDefault = (uDefaultConfig == u) ? "(default) " : "";
								if (!bConfigIsSelectable)
								{
									Asura_OutputToDebugger((" (greyed-out) "));
								}
								Asura_OutputToDebugger(("%d: %s%s.\n", u, szDefault, szConfigName));
								if (!bConfigIsSelectable)
								{
									Asura_OutputToDebugger((" (greyed-out) "));
								}
								Asura_OutputToDebugger((" : %s.\n", szConfigType));
							}
							else
							{
								Asura_OutputToDebugger(("%d: %s.\n", u, "<EMPTY>"));
							}
						}
					}

                    u_int uChosenConfig  = 0;
#if _DEBUG
                    if (bAutoChooseDefault)
                    {
                        uChosenConfig = static_cast<int>(Asura_PS2_EZNetCnf::GetDefaultConfig());
                    }
                    else
                    {
					    if (dbg_uHACKConfigChoice != 0)
					    {
						    if (Asura_PS2_EZNetCnf::ConfigIsSelectable(dbg_uHACKConfigChoice))
						    {
							    uChosenConfig = dbg_uHACKConfigChoice;
							    uLastConfigChoice = dbg_uHACKConfigChoice;
							    dbg_uHACKConfigChoice = 0;
						    }
					    }
                    }
#else
					// choose 1st config in release TODO implement proper menu for this
					uChosenConfig = 1;
					uLastConfigChoice = 1;
#endif
                    if (uChosenConfig != 0)
                    {
                        Asura_PS2_EZNetCnf::ChooseConfig(uChosenConfig);
                    }
					break;
				}
				
				case Asura_PS2_EZNetCnf::NetState_Initialising: 
				{
					break;
				}
				
				case Asura_PS2_EZNetCnf::NetState_Connected: 
				{
					bProcessing = false;
					break;
				}

				case Asura_PS2_EZNetCnf::NetState_Error:
				{
					break;
				}

			}
		}

		sceGsSyncV(0);
	}// end of while (bProcessing)

	Asura_PS2_EZNetCnf::FinishNetCnfSelection();
}
#else

Asura_PS2_EZNetCnf::NetState Asura_PS2_EZNetCnf::s_eState = Asura_PS2_EZNetCnf::NetState_None;

#endif //#ifdef ASURA_SUPPORT_EXTERNAL_NETWORK
