/********************************************************************
    filename:   Asura_PS2_DeferredRendering.h
    created:    10.10.2002
    author:     Pete Ellacott
    
    purpose:    header for deferred rendering system
*********************************************************************/

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef ___ASURA_PS2_DEFERREDRENDERING_H___
#define ___ASURA_PS2_DEFERREDRENDERING_H___ 1

#ifdef ASURA_PS2_DEFERRED_RENDERING

// M A C R O S ///////////////////////////////////////////////////////////////

#define ASURA_PS2_DEFERRED_RENDERING__MIN_PATH3_UPLOAD (8*1024) // Minimum 8Kb uploads via path 3.

// F O R W A R D   D E C L A R A T I O N S ///////////////////////////

union Asura_PS2_QW;

// C L A S S E S /////////////////////////////////////////////////////

class Asura_PS2_DeferredRendering
{

public:

    // C L A S S   M E T H O D S /////////////////////////////////////////////

    static void Initialise( void );                                          // Allocates drawlist etc
    static void Reset( void );                                               // Wait for current rendering to finish and cancel next frames drawlist
    static void Shutdown( void );                                            // Deallocates drawlist etc

    static void SendDeferredRenderingBlock( bool bPATH1Only = false );       // Finishes building a block of deferred rendering and sends it
    static void WaitForDeferredRendering();                                  // Waits for the deferred rendering to complete

    static Asura_PS2_QW* GetVifBuffer( u_int uQWs );                         // Request uQWs of space in the current VIF buffer.
    static void          ReturnVifBuffer( Asura_PS2_QW* pxBfr, u_int uQWs ); // Return (some of) the buffer returned from last GetVifBuffer() call.
    static Asura_PS2_QW* GetGifBuffer( u_int uQWs );                         // Request uQWs of space in the current GIF buffer.

    static void AddPath3Gate();                                              // Adds an open/close path3 to the draw list
    static void CancelLastPath3Gate();                                       // Cancels the last open/close path3 we added by overwritting the call tag with and empty cnt tag

//  static void             AddPAMark();                                     // Adds a DDA-Check mark for the performance analyser

    static const Asura_PS2_QW* GetCurVifBfrPtr( void ) { return s_pxVifPtr; }

private:

    // C L A S S   M E T H O D S /////////////////////////////////////////////

    static void MaskPath3( const bool bMask );
    static int  BufferSwapInteruptHandler( int iChannel );
    
    // C L A S S   S T A T E /////////////////////////////////////////////////

    static Asura_PS2_QW*  s_pxVifPtr;            // Pointer to the current position in the VIF buffer
    static Asura_PS2_QW*  s_pxGifPtr;            // Pointer to the current position in the GIF buffer    

    static Asura_PS2_QW*  s_pxVifEnd;            // Pointer to the end of the VIF buffer
    static Asura_PS2_QW*  s_pxGifEnd;            // Pointer to the end of the GIF buffer
    
    static Asura_PS2_QW*  s_pxPath3Gate;         // A DMA subroutine (CALL/RET) which does a (MSK)PATH3:Open - 24Qws NOPS - (MSK)PATH3:Close
    static Asura_PS2_QW*  s_pxLastPath3Gate;     // Keeps track of the last path3 open/close, so we can remove it if the current texture goes through path2 instead of path3
    static Asura_PS2_QW*  s_pxMaskPath3;         // A tiny bit of memory for opening and closing path 3 - Not allocated in SPAD becuase of assert issues
    
    static u_int          s_uCurrentGifBuffer;   // The current GIF buffer being filled

    static volatile u_int sv_uBuildingBuffer;    // The current buffer being built
    static volatile u_int sv_uBufferDrawing;     // Which buffer is currently being drawn
    static volatile u_int sv_uLastBufferToDraw;  // The last buffer in the current chain of drawing buffers

    static int            s_iVIF1DmacHandlerID;  // The ID of our Dmac VIF1 interrupt handler
};

#endif // #ifdef ASURA_PS2_DEFERRED_RENDERING

#endif // #ifndef ___ASURA_PS2_DEFERREDRENDERING_H___
