//////////////////////////////////////////////////////////////////////////////
//
// Asura_PlatformRenderStates.h
//
// By: Morgan Parry
//
// Copyright (c) 2002 Rebellion. All rights reserved.
//
// Platform-specific render state functionality.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef ___ASURA_PLATFORMRENDERSTATES_H___
#define ___ASURA_PLATFORMRENDERSTATES_H___ 1

// I N C L U D E S ///////////////////////////////////////////////////////////

#include <libgraph.h>               // For SCE_GS_ALPHA_GEQUAL, etc.

#include "Core/Asura_PlatformBasics.h"
#include "Rendering/Asura_RenderStates.h"

// C L A S S E S /////////////////////////////////////////////////////////////

class Asura_PlatformRenderStates
{

    public:

        // T Y P E S /////////////////////////////////////////////////////////

        enum CONTEXT
        {
            CONTEXT_1,
            CONTEXT_2,
        };

        enum RENDERSTATE_PACKET_QW_ID
        {
            QWID_TAG,
            QWID_ZBUF,
            QWID_TEXA,
            QWID_ALPHA,
            QWID_TEST,
            QWID_CLAMP,
            // NB: All indices 1 lower if GIF tag not being written.

            NUM_RENDERSTATE_PACKET_QWS_EXCL_GIF_TAG = QWID_CLAMP,
            NUM_RENDERSTATE_PACKET_QWS,
        };

        // C L A S S   M E T H O D S /////////////////////////////////////////

        static void SetZBufferWriteEnabled( bool bEnable );

        static void SetAlphaBlend( int iA, int iB, int iC, int iD, int iFIX );

        static void SetTranslucencyMode( ASURA_TRANSLUCENCY_MODE eMode );

		static void SetDefaultAlphaValue( bool bTreatBlackAsTransparent=false, uint32 uTA0=0, uint32 uTA1=128 );

        static void SetPixelTest( bool bEnableAlphaTest, bool bEnableDepthTest,
								  int iAlphaLimit = 128, int iAlphaMethod = SCE_GS_ALPHA_GEQUAL, int iAlphaFail = SCE_GS_AFAIL_KEEP );

        static void SetTextureClamp( bool bClamp );

        // (MH - 09.05.2002) to set renderstates you need to call CreateDefaultRenderStatePacket()
        // to begin the process, then any changes to the renderstate from the default can be 
        // set using the SetXXX functions above. Call ApplyRenderStates() when you're done.
        // (MP - 30.05.2002) Optional buffer argument allows you to ask for
        // this GIF packet to be output elsewhere; you're then responsible 
        // for sending this. If you specify NULL for the buffer, a temporary
        // one is setup at the end of SPR and then you should use
        // ApplyRenderStates() to send it to the GS.
        // (MH - 04.07.2002) Optional bRememberMyBuffer argument allows the buffer you specified above
        // to be remembered by this system until ForgetMyBuffer() is called. This is to allow user-
        // buffers to use the setup functions defined here.
        static void CreateDefaultRenderStatePacket( CONTEXT eContextID, 
                                                    Asura_PS2_QW* pxBuffer = NULL, 
                                                    bool bSetEOP = true,  bool bRememberMyBuffer = true, bool bWriteGifTag = true );
        static void ApplyRenderStates( void );
        static void ForgetMyBuffer(Asura_PS2_QW* pxBuffer);   // if you asked to keep it open on creation.

#if _DEBUG
		static bool IsTranslucencyModeSupported( ASURA_TRANSLUCENCY_MODE eMode );
#endif

};

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

#endif // #ifndef ___ASURA_PLATFORMRENDERSTATES_H___

