//////////////////////////////////////////////////////////////////////////////
//
// Asura_Platform_RenderHierarchy.h
//
// Copyright (c) 2005 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef ___ASURA_PLATFORM_RENDERHIERARCHY_H___
#define ___ASURA_PLATFORM_RENDERHIERARCHY_H___ 1

// I N C L U D E S ///////////////////////////////////////////////////////////

// Engine platform-specific.
#include "Rendering/Asura_PS2_GSMRenderer.h"

// M A C R O S ///////////////////////////////////////////////////////////////

// This many QWs at the beginning of SPR are required for internal use by
// the hierarchical skin rendering code. Note that this includes the memory
// used by the GSM renderer that's underlying this.
#define ASURA_PS2_HIERSKIN_SPR_QWS_RESERVED (ASURA_PS2_GSM_SPR_QWS_RESERVED + 0x10)

// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////////

class Asura_Animation;
class Asura_SphericalHarmonics_Order2_RGB;
class Asura_Vector_3;

class Asura_PS2_HierSkin_SkinHdr;

// C L A S S E S /////////////////////////////////////////////////////////////

// This is built on top of the GSM renderer. It should not be necessary to
// use the GSM renderer directly; instead, all calls should go through this
// class. The exception is where common RS modification callbacks are
// required, in which case it is valid to pass 
class Asura_PS2_HierarchySkin_Renderer
{

public:

    // C L A S S   M E T H O D S /////////////////////////////////////////////

    //
    // Startup.
    //

    // False return value indicates the skin cannot or will not be rendered.
    // Rendering should be aborted at this point.
    static bool Begin( const Asura_Animation* pxAnim,
                       u_int uCamVisFlags,
                       const Asura_Vector_3& xBSCentre,
                       float fBSRadius );
    
    static void GetHdrData( const Asura_PS2_HierSkin_SkinHdr** ppxSkinHdr, const u_short** ppusSecMatCounts );

    //
    // Renderstate.
    //
    
    static void SetTexture( int iTexID );

    static void SetRenderstate( u_int uMaterialFlags, Asura_PS2_GSMRenderer::RSCallback pfnManualRSModifyCB = NULL )
    {
        Asura_PS2_GSMRenderer::SetRenderstate( uMaterialFlags );

        if( pfnManualRSModifyCB ) { Asura_PS2_GSMRenderer::ManuallyModifyRenderstate( pfnManualRSModifyCB ); }
    }

    //
    // Shadows.
    //
    
    // Only call this if a shadow is required.
    static void SetupShadow( void );

    //
    // Lighting.
    //

    static void SetLighting( const Asura_SphericalHarmonics_Order2_RGB& xSH, float fAlpha );

    //
    // Transforms.
    //

    static void SetupMatrices( void );

    static void InsertWorldspaceTransform( const Asura_PS2_QW* pxTransform );

    static void SetupSection( u_int uSectionIdx, bool bResetLighting );

    //
    // Data transfer and actual rendering.
    //

    static void RenderStripChain( u_int uCALLAddr ) { Asura_PS2_GSMRenderer::RenderStripChain( uCALLAddr ); }

    //
    // Tidy up.
    //

    static void End( void );

    //
    // Misc.
    //

    // Only valid between Begin() and End().
    static u_int GetNumCamsVisFor( void ) { return Asura_PS2_GSMRenderer::GetNumCamsVisFor(); }
    static u_int GetNumPassesPerTri( void ) { return Asura_PS2_GSMRenderer::GetNumPassesPerTri(); }

};
// End class Asura_PS2_HierarchySkin_Renderer.

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

#endif // #ifndef ___ASURA_PLATFORM_RENDERHIERARCHY_H___
