//////////////////////////////////////////////////////////////////////////////
//
// VU1_Beam.vcl
//
// Created by Pete Ellacott
//
// Changes:
//
//     (MP - 11.07.2003) Added clipping code.
//
// Description:
//
//     Renders laser beams and the like
//
//     The renderer assumes the camera count is in the range [1, 4].
//
//     See Asura_PS2_BeamDefines.h for memory layout details.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "vcl_sml.i"
#include "Asura_PS2_BeamDefines.h"

// M A C R O S ///////////////////////////////////////////////////////////////

// Which planes need clipping against. These match the VU1 clipping bits.
#define CLIP_PLANE_RIGHT  (0x001)
#define CLIP_PLANE_LEFT   (0x002)
#define CLIP_PLANE_BOTTOM (0x004)
#define CLIP_PLANE_TOP    (0x008)
#define CLIP_PLANE_FAR    (0x010)
#define CLIP_PLANE_NEAR   (0x020)

// V C L   S E T U P /////////////////////////////////////////////////////////

.syntax new
.init_vf_all
.init_vi_all

// C O D E ///////////////////////////////////////////////////////////////////

--enter
--endenter
	// Setup output buffer pointers
	IADDIU	pThisOutputBuffer,	VI00,	(ASURA_PS2_BEAM__ADDR__OUTPUT_BUFFER_0)
	IADDIU	pNextOutputBuffer,	VI00,	(ASURA_PS2_BEAM__ADDR__OUTPUT_BUFFER_1)
	// Write the setup gif tag into both output buffers. This tag never
    // changes.
	LQ.xyzw	fSetupGifTag,	0(VI00)
	SQ.xyzw	fSetupGifTag,	0(pThisOutputBuffer)
	SQ.xyzw	fSetupGifTag,	0(pNextOutputBuffer)
	// Write the drawing gif tag into both output buffers. The only part of
    // this tag that changes is the NLOOP value.
	LQ.yzw	fDrawingGifTag,	1(VI00)
	SQ.yzw	fDrawingGifTag,	3(pThisOutputBuffer)
	SQ.yzw	fDrawingGifTag,	3(pNextOutputBuffer)

START_STRIP:
	--cont
	// Setup camera data pointers
	//	Get the address of the current INPUT double buffer. This is where the new strip data lies.
	XTOP	pIn
	// Load the vertex count and camera buffer flag
	XITOP	iItops
	// Load the camera data and create setup gif packet
	IADDIU	pCameraData,	VI00,	ASURA_PS2_BEAM__ADDR__CAMERA_BUFFER_0
	IADDIU	iTemp,	VI00,	ASURA_PS2_BEAM__FLAG__CAMERA_BUFFER
	IAND	iTemp,	iTemp,	iITops
	IBEQ	iTemp,	VI00,	CameraPointerSetup
	IADDIU	pCameraData,	pCameraData,	ASURA_PS2_BEAM__SIZE__CAMERA_BUFFER
CameraPointerSetup:
    LQ.xyzw	fScissor,  12(pCameraData)
    LQ.xyzw	fOffset,   13(pCameraData)
    SQ.xyzw	fScissor,   1(pThisOutputBuffer)
    SQ.xyzw	fOffset,    2(pThisOutputBuffer)
	// Extract vertex count
	IADDIU	iTemp,		VI00,		ASURA_PS2_BEAM__MASK__CAMERA_BUFFER
	IAND	iVertexQty,	iTemp,		iITops
	IADDIU	iTemp,		iVertexQty,	0x7fff
	IADDIU	iTemp,		iTemp,		1
	ISW.x	iTemp,		3(pThisOutputBuffer)
	// Setup end of input pointer
	IADD	pInEnd,	iVertexQty, iVertexQty
	IADD	pInEnd,	pInEnd, iVertexQty
	IADD	pInEnd,	pInEnd, pIn
	// setup output pointer
	IADDIU	pOut,	pThisOutputBuffer,	4
	// Load in the camera matricies
	LQ.xyzw	W2S_Matrix[0], 0(pCameraData)
	LQ.xyzw	W2S_Matrix[1], 1(pCameraData)
	LQ.xyzw	W2S_Matrix[2], 2(pCameraData)
	LQ.xyzw	W2S_Matrix[3], 3(pCameraData)
	LQ.xyzw	W2C_Matrix[0], 4(pCameraData)
	LQ.xyzw	W2C_Matrix[1], 5(pCameraData)
	LQ.xyzw	W2C_Matrix[2], 6(pCameraData)
	LQ.xyzw	W2C_Matrix[3], 7(pCameraData)
	// Setup ST's Q component
	MAXw.z		vSTQ,	VF00,	VF00
	// Reset clipping flags
	FCSET	0

    LOI    128.0
    IADDIU iAnyClipped, VI00, 0
Vertex_Loop:
    --LoopCS 3, 0
		// Load the vertex
		LQ.xyz	vVertex,	0(pIn)
		LQ.xy	vSTQ,		1(pIn)
		LQ.xyzw	vColour,	2(pIn)
		ILW.w	iADC,		0(pIn)
		// Project vertex to clip space
		MULAw.xyzw	ACC,		W2C_Matrix[3],	VF00[w]
		MADDAx.xyzw	ACC,		W2C_Matrix[0],	vVertex[x]
		MADDAy.xyzw	ACC,		W2C_Matrix[1],	vVertex[y]
		MADDz.xyzw	vClipPos,	W2C_Matrix[2],	vVertex[z]
		// Perform clipping judgement.
		CLIPw.xyz	vClipPos,	vClipPos
		// Project vertex to screen space
		MULAw.xyzw	ACC,		W2S_Matrix[3],	VF00[w]
		MADDAx.xyzw	ACC,		W2S_Matrix[0],	vVertex[x]
		MADDAy.xyzw	ACC,		W2S_Matrix[1],	vVertex[y]
		MADDz.xyzw	vScrnPos,	W2S_Matrix[2],	vVertex[z]
		// Perspective divide XYZ and STs.
		DIV			Q,			VF00[w],	vScrnPos[w]
		MULq.xyz    vScrnPos,	vScrnPos,	Q
		MULq.xyz	vSTQOut,	vSTQ,		Q
		// Scale colour
		MULi.xyzw	vColour,	vColour,	I
		// Convert XYZ and colour for the GIF packet
		FTOI4.xyz	vScrnPos,	vScrnPos
		FTOI0.xyzw	vColour,	vColour
		// Store STQ, RGBA and XYZ for this vertex
		SQ.xyz		vSTQOut,	0(pOut)
		SQ.xyzw		vColour,	1(pOut)
		SQ.xyz		vScrnPos,	2(pOut)
		// Process clipping result.
		FCAND	VI01,	0x3ffff
		IADDIU	iClipFlag,	VI01,	0x7fff
        IOR iAnyClipped, iAnyClipped, VI01
        // Store some clipping information, in case this vertex is part of a
        // tri that ends up needing scissoring. We can just overwrite some of
        // the input data.
        SQ.xyzw vClipPos,   0(pIn)
        FCGET   iClipFlags
        ISW.z   iClipFlags, 1(pIn)
        ISW.w   iADC,       1(pIn)
		// Combine the clipping flag with the adc flag
		IOR   iADCOut, iADC, iClipFlag
		ISW.w iADCOut, 2(pOut)
		// Move on to next vertex, if there are any left.
		IADDIU	pIn,	pIn,	3
		IADDIU	pOut,	pOut,	3
		IBNE	pIn,	pInEnd,	Vertex_Loop
    // Render that lot, clip tris as necessary and stop.
    BAL pReturnAddress, KICK_AND_SWAP_BUFFERS
    IBNE iAnyClipped, VI00, CLIP_TRIANGLES
    B START_STRIP

KICK_AND_SWAP_BUFFERS:
--barrier
	XGKICK	pThisOutputBuffer
	IADD	iTemp,				VI00,	pThisOutputBuffer
	IADD	pThisOutputBuffer,	VI00,	pNextOutputBuffer
	IADD	pNextOutputBuffer,	VI00,	iTemp
    JR      pReturnAddress

CLIP_FINISHED:
--barrier
	// Anything in buffer that needs rendering? (NB: Could kick anyway but
    // want to avoid swapping output buffers if possible.)
	ILW.x	iVertexCount,	3(pThisOutputBuffer)
	IADDIU	iTemp,	VI00,	0x7FFF
	IAND	iTemp,	iTemp,	iVertexCount
	IBEQ	iTemp,	VI00,	START_STRIP
	// Ok, we have something to render.
	BAL pReturnAddress, KICK_AND_SWAP_BUFFERS
    B START_STRIP

CLIP_TRIANGLES:
--barrier
    // Copy SCISSOR and XYOFFSET register settings to other output buffer.
    // Once we've done this, both output buffers will contain the current
    // camera's settings and we won't need to worry about it again for this
    // strip.
    LQ.xyzw	fScissor, 12(pCameraData)
    LQ.xyzw	fOffset,  13(pCameraData)
    SQ.xyzw	fScissor,  1(pThisOutputBuffer)
    SQ.xyzw	fOffset,   2(pThisOutputBuffer)
    // Setup the output pointers, where the final strips go
    IADDIU pOut, pThisOutputBuffer, 4
	// Write a gif tag to new output buffer (with EOP set and an NLOOP of 0.)
	IADDIU	iTemp,	VI00,	0x7fff
	IADDIU	iTemp,	iTemp,	1
	ISW.x	iTemp,	3(pThisOutputBuffer)
	// Setup input pointer and clip mask.
    XTOP   pIn
    IADDIU iClipMask, VI00, 0x3F
	// Get clipping results for vertex 0.
	LQ.xyzw		vClipPos,		0(pIn)
	CLIPw.xyz	vClipPos,		vClipPos
	ILW.z		iClipFlag2,		1(pIn)
	IAND		iClipFlag2,		iClipFlag2,		iClipMask
	// And for vertex 1.
	LQ.xyzw		vClipPos,		3(pIn)
	CLIPw.xyz	vClipPos,		vClipPos
	ILW.z		iClipFlag1,		4(pIn)
	IAND		iClipFlag1,		iClipFlag1,		iClipMask
	// Calculate half of first clipping test
	IAND		iClipAll21,		iClipFlag2,		iClipFlag1
    // Jump to vertex 2.
    IADDIU pIn, pIn, 6
FIND_CLIP_TRIANGLE_LOOP:
    --LoopCS 1, 0
	// Have we finished yet ?
	IBEQ		pIn,		pInEnd,		CLIP_FINISHED
	// Get clipping results for vertex N
	LQ.xyzw		vClipPos,	0(pIn)
	ILW.z		iClipFlag0,	1(pIn)
	IAND		iClipFlag0,	iClipFlag0,	iClipMask
	// Calculate second half of clipping test
	IAND		iClipAll10,	iClipFlag1,	iClipFlag0
	IAND		iClipAll,	iClipAll21,	iClipAll10
	// Check for none clipped
	CLIPw.xyz	vClipPos,	vClipPos
	// Move clipping flags/result along 1
	IADD		iClipFlag1,	VI00,		iClipFlag0
	IADD		iClipAll21,	VI00,		iClipAll10
    // Move input ptr along.
    IADDIU pIn, pIn, 3
    // Check if we found a triangle that need clipping
	FCAND		VI01,		0x3FFFF
	IBEQ		VI01,		VI00,	FIND_CLIP_TRIANGLE_LOOP	// The triangle is complete unclipped
	IBNE		iClipAll,	VI00,	FIND_CLIP_TRIANGLE_LOOP	// The triangle is complete clipped to at least one plane
	// Check for degenracy
	ILW.w		iDegenerate, -2(pIn)
	IBNE		iDegenerate, VI00,	FIND_CLIP_TRIANGLE_LOOP	// Degenerate vert.. if this triangle needs clipping it'll be picked up later
	// This triangle needs clipping - Temporarily store some int registers in a float register
	MFIR.x	fStack0,	pIn
	MFIR.y	fStack0,	pInEnd
	MFIR.z	fStack0,	iClipFlag1
	MFIR.w	fStack0,	iClipAll21
--barrier
//	XITOP	iClipMask
	// Setup the clipping variables
	ISUBIU	pClipSrc,	pIn,		9                                         // First vertex in clipped triangle is 3 vertices back from pIn
	IADDIU	pClipDst,	VI00,		ASURA_PS2_BEAM__ADDR__CLIPPER_WORKSPACE_0 // Work buffer swap is done using:  pSrc = pDst
	IADDIU	pClipMid,	pClipDst,	ASURA_PS2_BEAM__SIZE__CLIPPER_WORKSPACE   // pDst = Buffer1-pSrc+Buffer0 : so if pSrc = Buffer1 : pDst = Buffer1-Buffer1+Buffer0 = Buffer0
	IADD	pClipMid,	pClipMid,	pClipDst							      // if pSrc = Buffer0 : pDst = Buffer1-Buffer0+Buffer0 = Buffer1.  so we keep Buffer0+Buffer1 then just subtract pSrc
	IADDIU	iSrcPointQty, VI00, 3
	// Find out which planes this triangle clips against - didn't seem to work...
	//	ILW.w	iClip0,	 3(pClipSrc)
	//	ILW.w	iClip1,	 7(pClipSrc)
	//	ILW.w	iClip2,	11(pClipSrc)
	//	IOR		iClipPlanes, iClip0, iClip1
	//	IOR		iClipPlanes, iClipPlanes, iClip2

	//----------------------------------
	// Clip to the near plane (Z -ve)
	//----------------------------------
	IADDIU	iCurrentClipMask, VI00, CLIP_PLANE_NEAR		// Setup the current clip mask
//	IAND	iTemp,	iClipPlanes, iCurrentClipMask
//	IBEQ	iTemp,	VI00,	CLIP_FAR_PLANE
	SUBw.xw	fPlaneSign,	VF00, VF00							// Clear destination vertex count and set plane sign to -ve
//	IADDIU	pInterpolate, VI00, INTERPOLATE_Z				// No VCL JUMP TABLE SUPPORT :o(
	IADDIU	pInterpolate, VI00, 0							// No VCL JUMP TABLE SUPPORT :o(
	BAL		pRtA, CLIP_POINTS								// Clip the points
	IADDIU	pClipSrc, pClipDst, 0							// Setup the source pointer,
	ISUB	pClipDst, pClipMid, pClipSrc					// destination pointer
	FTOI0.w	fPlaneSign, fPlaneSign							//	and counters for next clip
	MTIR	iSrcPointQty, fPlaneSign[w]
	IBEQ	iSrcPointQty, VI00,	CLIP_FINISHED_THIS_TRIANGLE	// All the points have been clipped away...

	//----------------------------------
	// Clip to the far plane (Z +ve)
	//----------------------------------
CLIP_FAR_PLANE:
	IADDIU	iCurrentClipMask, VI00, CLIP_PLANE_FAR		// Setup the current clip mask
//	IAND	iTemp,	iClipPlanes, iCurrentClipMask
//	IBEQ	iTemp,	VI00,	CLIP_TOP_PLANE
	ADDw.x	fPlaneSign,	VF00, VF00							// Set plane sig to +ve
	SUB.w	fPlaneSign, VF00, VF00							// Clear vertex count
//	IADDIU	pInterpolate, VI00, INTERPOLATE_Z				// No VCL JUMP TABLE SUPPORT :o(
	IADDIU	pInterpolate, VI00, 0							// No VCL JUMP TABLE SUPPORT :o(
	BAL		pRtA, CLIP_POINTS								// Clip the points
	IADDIU	pClipSrc, pClipDst, 0							// Setup the source pointer,
	ISUB	pClipDst, pClipMid, pClipSrc					// destination pointer
	FTOI0.w	fPlaneSign, fPlaneSign							//	and counters for next clip
	MTIR	iSrcPointQty, fPlaneSign[w]
	IBEQ	iSrcPointQty, VI00,	CLIP_FINISHED_THIS_TRIANGLE	// All the points have been clipped away...

	//----------------------------------
	// Clip to the top plane (Y -ve)
	//----------------------------------
CLIP_TOP_PLANE:
	IADDIU	iCurrentClipMask, VI00, CLIP_PLANE_TOP		// Setup the current clip mask
//	IAND	iTemp,	iClipPlanes, iCurrentClipMask
//	IBEQ	iTemp,	VI00,	CLIP_BOTTOM_PLANE
	SUBw.xw	fPlaneSign,	VF00, VF00							// Clear destination vertex count and set plane sign to -ve
//	IADDIU	pInterpolate, VI00, INTERPOLATE_Y				// No VCL JUMP TABLE SUPPORT :o(
	IADDIU	pInterpolate, VI00, 1							// No VCL JUMP TABLE SUPPORT :o(
	BAL		pRtA, CLIP_POINTS								// Clip the points
	IADDIU	pClipSrc, pClipDst, 0							// Setup the source pointer,
	ISUB	pClipDst, pClipMid, pClipSrc					// destination pointer
	FTOI0.w	fPlaneSign, fPlaneSign							//	and counters for next clip
	MTIR	iSrcPointQty, fPlaneSign[w]
	IBEQ	iSrcPointQty, VI00,	CLIP_FINISHED_THIS_TRIANGLE	// All the points have been clipped away...

	//----------------------------------
	// Clip to the bottom plane (Y +ve)
	//----------------------------------
CLIP_BOTTOM_PLANE:
	IADDIU	iCurrentClipMask, VI00, CLIP_PLANE_BOTTOM	// Setup the current clip mask
//	IAND	iTemp,	iClipPlanes, iCurrentClipMask
//	IBEQ	iTemp,	VI00,	CLIP_LEFT_PLANE
	ADDw.x	fPlaneSign,	VF00, VF00							// Set plane sign to +ve
	SUB.w	fPlaneSign, VF00, VF00							// Clear destination vertex counter
//	IADDIU	pInterpolate, VI00, INTERPOLATE_Y				// No VCL JUMP TABLE SUPPORT :o(
	IADDIU	pInterpolate, VI00, 1							// No VCL JUMP TABLE SUPPORT :o(
	BAL		pRtA, CLIP_POINTS								// Clip the points
	IADDIU	pClipSrc, pClipDst, 0							// Setup the source pointer,
	ISUB	pClipDst, pClipMid, pClipSrc					// destination pointer
	FTOI0.w	fPlaneSign, fPlaneSign							//	and counters for next clip
	MTIR	iSrcPointQty, fPlaneSign[w]
	IBEQ	iSrcPointQty, VI00,	CLIP_FINISHED_THIS_TRIANGLE	// All the points have been clipped away...

	//----------------------------------
	// Clip to the left plane (X -ve)
	//----------------------------------
CLIP_LEFT_PLANE:
	IADDIU	iCurrentClipMask, VI00, CLIP_PLANE_LEFT		// Setup the current clip mask
//	IAND	iTemp,	iClipPlanes, iCurrentClipMask
//	IBEQ	iTemp,	VI00,	CLIP_RIGHT_PLANE
	SUBw.xw	fPlaneSign,	VF00, VF00							// Set plane sign to -ve and clear destination vertex count
//	IADDIU	pInterpolate, VI00, INTERPOLATE_X				// No VCL JUMP TABLE SUPPORT :o(
	IADDIU	pInterpolate, VI00, 2							// No VCL JUMP TABLE SUPPORT :o(
	BAL		pRtA, CLIP_POINTS								// Clip the points
	IADDIU	pClipSrc, pClipDst, 0							// Setup the source pointer,
	ISUB	pClipDst, pClipMid, pClipSrc					// destination pointer
	FTOI0.w	fPlaneSign, fPlaneSign							//	and counters for next clip
	MTIR	iSrcPointQty, fPlaneSign[w]
	IBEQ	iSrcPointQty, VI00,	CLIP_FINISHED_THIS_TRIANGLE	// All the points have been clipped away...

	//----------------------------------
	// Clip to the right plane (X +ve)
	//----------------------------------
CLIP_RIGHT_PLANE:
	IADDIU	iCurrentClipMask, VI00, CLIP_PLANE_RIGHT	// Setup the current clip mask
//	IAND	iTemp,	iClipPlanes, iCurrentClipMask
//	IBEQ	iTemp,	VI00,	CLIP_FINISHED_PLANES
	ADDw.x	fPlaneSign,	VF00, VF00							// Set plane sign to +ve
	SUB.w	fPlaneSign, VF00, VF00							// Clear destination vertex count
//	IADDIU	pInterpolate, VI00, INTERPOLATE_X				// No VCL JUMP TABLE SUPPORT :o(
	IADDIU	pInterpolate, VI00, 2							// No VCL JUMP TABLE SUPPORT :o(
	BAL		pRtA, CLIP_POINTS								// Clip the points
	IADDIU	pClipSrc, pClipDst, 0							// Setup the source pointer,
	FTOI0.w	fPlaneSign, fPlaneSign							//	and counters
	MTIR	iSrcPointQty, fPlaneSign[w]				
	IBEQ	iSrcPointQty, VI00,	CLIP_FINISHED_THIS_TRIANGLE	// All the points have been clipped away...

	//-----------------------------------------------
	// Finished clipping - Project in to screen space
	//-----------------------------------------------
CLIP_FINISHED_PLANES:
    // Check there is enough space in the output buffer for this many more
    // vertices.
    ISUB   iTemp, pOut,  pThisOutputBuffer
    ISUBIU iTemp, iTemp, 4                                             // now contains (3 * num verts output so far)
    IADD   iTemp, iTemp, iSrcPointQty                                  // \ 
    IADD   iTemp, iTemp, iSrcPointQty                                  //  |- + (3 * num verts in this strip)
    IADD   iTemp, iTemp, iSrcPointQty                                  // / 
    ISUBIU iTemp, iTemp, (ASURA_PS2_BEAM__MAX_VERTICES_PER_BUFFER * 3) // now < 0 if there is space for this strip
    IBLTZ  iTemp, CLIP_ENOUGH_SPACE_IN_OUTPUT
    // Not enough space for this strip so kick what we have and swap buffers.
    BAL pReturnAddress, KICK_AND_SWAP_BUFFERS
    // Write gif tag to new output buffer (with EOP & NLOOP=0)
    IADDIU	iTemp,	VI00,	0x7fff
    IADDIU	iTemp,	iTemp,	1
    ISW.x	iTemp,	3(pThisOutputBuffer)
    // Reset output ptr.
    IADDIU	pOut,	pThisOutputBuffer,	4
	
CLIP_ENOUGH_SPACE_IN_OUTPUT:
	// Increment NLOOP count in gif tag
	ILW.x	iTemp,	3(pThisOutputBuffer)
	IADD	iTemp,	iTemp,	iSrcPointQty
	ISW.x	iTemp,	3(pThisOutputBuffer)
	// Setup the input pointers
	IADD	pClipSrcE, iSrcPointQty, iSrcPointQty
	IADD	pClipSrcE, pClipSrcE, iSrcPointQty
	IADD	pClipSrcE, pClipSrcE, pClipSrc
	ISUBIU	pClipSrcE, pClipSrcE, 3					// Last vertex in fan
	// Load the clip to screen matrix
    MatrixLoad mC2S, 8, pCameraData
	// Set Q component of vSTQ to 1.0
	ADDw.z	vSTQ,	VF00,	VF00
	// Set degeneracy flag for first two output verts
	IADDIU	iADC,	VI00,	0x6000

CLIP_OUTPUT_STRIP_LOOP

	    --LoopCS 2,0
        
	    // Load vertex from the start
	    LQ.xyzw	vClipPos,	0(pClipSrc)
	    LQ.xy	vSTQ,		1(pClipSrc)
		LQ.xyzw	vRGBA,		2(pClipSrc)
		IADDIU	pClipSrc, pClipSrc,	3
	    // Transform clip space position to screenspace.
	    MatrixMultiplyVertex vScrnPos, mC2S, vClipPos
	    // Calculate 1/w.
		DIV Q, VF00[w], vScrnPos[w]
		// Perspective divide.
		MULq.xyz vScrnPos, vScrnPos, Q
		MULq.xyz vTexOut,  vSTQ,     Q
		// Convert XYZ to GS format.
        FTOI4.xyz vScrnPos, vScrnPos
        // And RGBA. Note that the interpolated alpha can be slightly out,
        // even when the alpha values its interpolating between are 128 and
        // 128, so we need to add on a little bit to stop alpha 127 being
        // written when it should be alpha 128 (due to alpha test limits,
        // etc.)
        LOI 128.0
        MULi.xyzw vRGBA, vRGBA, I
        LOI 0.5
        ADDi.w vRGBA, vRGBA, I
        FTOI0 vRGBA, vRGBA
		// Output.
		SQ.xyz	vTexOut,  0(pOut)
		SQ.xyzw	vRGBA,	  1(pOut)
		SQ.xyz	vScrnPos, 2(pOut)
		IADDIU	pOut,	pOut,	3
		// Set the ADC bit (for degenerates)
		IADD	iADC,	iADC,	iADC
		ISW.w	iADC,	-1(pOut)
		// Check if we've finished yet
		ISUB	iTemp,	pClipSrcE,	pClipSrc
		IBLTZ 	iTemp,	CLIP_FINISHED_OUTPUT_THIS_TRIANGLE
		// Load vertex from the end
	    LQ.xyzw	vClipPos,	0(pClipSrcE)
	    LQ.xy	vSTQ,		1(pClipSrcE)
		LQ.xyzw	vRGBA,		2(pClipSrcE)
		ISUBIU	pClipSrcE,	pClipSrcE,	3
	    // Transform clip space position to screenspace.
	    MatrixMultiplyVertex vScrnPos, mC2S, vClipPos
	    // Calculate 1/w.
		DIV Q, VF00[w], vScrnPos[w]
		// Perspective divide.
		MULq.xyz	vScrnPos, vScrnPos, Q
		MULq.xyz	vTexOut,  vSTQ,     Q
		// Convert XYZ to GS format.
        FTOI4.xyz vScrnPos, vScrnPos
        // And RGBA. See comment above.
        LOI 128.0
        MULi.xyzw vRGBA, vRGBA, I
        LOI 0.5
        ADDi.w vRGBA, vRGBA, I
        FTOI0 vRGBA, vRGBA
		// Output.
		SQ.xyz	vTexOut,  0(pOut)
		SQ.xyzw	vRGBA,	  1(pOut)
		SQ.xyz	vScrnPos, 2(pOut)
		IADDIU	pOut,	pOut,	3
		// Set the ADC bit (for degenerates)
		IADD	iADC,	iADC,	iADC
		ISW.w	iADC,	-1(pOut)
		// Check if we've finished yet
		ISUB	iTemp,	pClipSrcE,	pClipSrc
		IBGEZ 	iTemp,	CLIP_OUTPUT_STRIP_LOOP

CLIP_FINISHED_OUTPUT_THIS_TRIANGLE:
CLIP_FINISHED_THIS_TRIANGLE:
	// Retrieve Temporarily stored int registers from float register
	MTIR	pIn,		fStack0[x]
	MTIR	pInEnd,		fStack0[y]
	MTIR	iClipFlag1,	fStack0[z]
	MTIR	iClipAll21,	fStack0[w]
	IADDIU	iClipMask,	VI00, 0x3f
    // Carry out previous 2 clips again so the clip find triangle loop has the correct clip history
    LQ.xyzw   vClipPos, -6(pIn)
    CLIPw.xyz vClipPos, vClipPos
    LQ.xyzw   vClipPos, -3(pIn)
    CLIPw.xyz vClipPos, vClipPos

	B FIND_CLIP_TRIANGLE_LOOP

--barrier

CLIP_POINTS:
	// Keep track of the start of the input buffer so we can add the first vertex to the end of the list at the end
	IADDIU	pClipSrcStart, pClipSrc, 0
	// Create an output pointer
	IADDIU	pClipOut, pClipDst, 0
    // Load the first vertex from the buffer
    LQ.xyzw	vNVert,   0(pClipSrc)
    LQ.xy	vNSTQ,    1(pClipSrc)
    LQ.xyzw	vNColour, 2(pClipSrc)
    IADDIU	pClipSrc, pClipSrc, 3
	// Check if it's in or out
	CLIPw.xyz	vNVert, vNVert[w]
	FCGET		VI01
	IAND		iTemp, VI01, iCurrentClipMask
	IBEQ		iTemp, VI00, CLIP_CURRENT_IN

CLIP_CURRENT_OUT:
	// Current point is out
	// Anymore points ?
	ISUBIU	iSrcPointQty, iSrcPointQty, 1
	IBEQ	iSrcPointQty, VI00, CLIP_POINTS_END
	// Move next point to current point
	MOVE.xyzw	vCVert,		vNVert
	MOVE.xy		vCSTQ,		vNSTQ
	MOVE.xyzw	vCColour,	vNColour
    // Load the next point
    LQ.xyzw vNVert,		0(pClipSrc)
    LQ.xy   vNSTQ,		1(pClipSrc)
    LQ.xyzw vNColour,	2(pClipSrc)
    IADDIU  pClipSrc,	pClipSrc,	3
	// Check if it's in or out
	CLIPw.xyz	vNVert, vNVert[w]
	FCGET		VI01
	IAND		iTemp, VI01, iCurrentClipMask
	IBEQ		iTemp, VI00, CLIP_OUT_IN
CLIP_OUT_OUT:
	// Next point
	B	CLIP_CURRENT_OUT
CLIP_OUT_IN:
	// Adding 1 to destination point count
	ADD.w	fPlaneSign, fPlaneSign, VF00
	// Write INTERPOLATED - Setup retrun address to jump to the next point
//	IADDIU	pRtA2, VI00, CLIP_CURRENT_IN
//	JR		pInterpolate
	// Hack until VCL support JumpTables or I re write it in VSM
	BAL	pRtA2,	INTERPOLATE
	B	CLIP_CURRENT_IN

CLIP_CURRENT_IN:
	// Current point is in
	// Anymore points ?
	ISUBIU	iSrcPointQty, iSrcPointQty, 1
	IBEQ	iSrcPointQty, VI00, CLIP_POINTS_END
	// Move next point to current point
	MOVE.xyzw	vCVert,		vNVert
	MOVE.xy		vCSTQ,		vNSTQ
	MOVE.xyzw	vCColour,	vNColour
	// Load the next point
	LQ.xyzw	vNVert,		0(pClipSrc)
	LQ.xy	vNSTQ,		1(pClipSrc)
	LQ.xyzw	vNColour,	2(pClipSrc)
	IADDIU	pClipSrc,	pClipSrc,	3
	// Check if it's in or out
	CLIPw.xyz	vNVert, vNVert[w]
	FCGET		VI01
	IAND		iTemp, VI01, iCurrentClipMask
	IBNE		iTemp, VI00, CLIP_IN_OUT
CLIP_IN_IN:
	// Adding 1 to destination point count
	ADD.w	fPlaneSign, fPlaneSign, VF00
	// Write CURRENT
	SQ.xyzw	vCVert,		0(pClipOut)
	SQ.xy	vCSTQ,		1(pClipOut)
	SQ.xyzw	vCColour,	2(pClipOut)
	IADDIU	pClipOut,	pClipOut,	3
	// Next point
	B	CLIP_CURRENT_IN
CLIP_IN_OUT:
	// Adding 2 to destination point count
	ADD.w	fPlaneSign, fPlaneSign, VF00
	ADD.w	fPlaneSign, fPlaneSign, VF00
	// Write CURRENT
	SQ.xyzw	vCVert,		0(pClipOut)
	SQ.xy	vCSTQ,		1(pClipOut)
	SQ.xyzw	vCColour,	2(pClipOut)
	IADDIU	pClipOut,	pClipOut,	3
	// Write INTERPOLATED - Setup retrun address to jump to the next point
//	IADDIU	pRtA2, VI00, CLIP_CURRENT_OUT
//	JR		pInterpolate
	// Hack until VCL support JumpTables or I re write it in VSM
	BAL	pRtA2,	INTERPOLATE
	B	CLIP_CURRENT_OUT

CLIP_POINTS_END:
	// Check if we've done the final line yet (last->first)
	IBNE	pClipSrcStart, VI00, CLIP_POINT_LAST_SEGMENT
	JR	pRtA

CLIP_POINT_LAST_SEGMENT
	// Get the start of the input buffer and clear the input buffer pointer so we know we've finished
	IADDIU	pClipSrc, pClipSrcStart, 0
	IADDIU	pClipSrcStart, VI00, 0
	// One final segment to do
	IADDIU	iSrcPointQty, iSrcPointQty, 2
	// Check if the last clip was in or out
	FCGET	VI01
	IAND	iTemp, VI01, iCurrentClipMask
	IBEQ	iTemp, VI00, CLIP_CURRENT_IN
	B		CLIP_CURRENT_OUT

--barrier

	//		 |-------L-------|
	//		C ----------|---- N
	//					P
	//		 |----A-----l--B-|
	//					a
	//					n
	//					e
	//
	//	we need to find the ratio R = |A/L|

	// Hack until VCL support JumpTables or I re write it in VSM
INTERPOLATE:
	IBEQ	pInterpolate, VI00, INTERPOLATE_Z
	ISUBIU	iInt, pInterpolate, 1
	IBEQ	iInt, VI00, INTERPOLATE_Y
	// Interpolate the current line segment against the clip plane
INTERPOLATE_X:
	// Get distance from clip plane to current (taking into account -/+ clip plane)		(-A in above diagram)
	ADDAx.x	ACC, VF00, vCVert						; ACC.x = CVert.z
	MSUBw.x fC2Plane, fPlaneSign, vCVert			; fC2Plane.x = (CVert.z) - (fPlaneSign, CVert.w)
	// Get distance from next to clip plane (taking into account -/+ clip plane)			(B in above diagram)
	ADDAx.x	ACC, VF00, vNVert						; ACC.x = Vert.z
	MSUBw.x fN2Plane, fPlaneSign, vNVert			; fN2Plane.x = (CVert.z) - (fPlaneSign, CVert.w)
	B	INTERPOLATE_COMMON
INTERPOLATE_Y:
	// Get distance from clip plane to current (taking into account -/+ clip plane)		(-A in above diagram)
	ADDAy.x	ACC, VF00, vCVert						; ACC.x = CVert.z
	MSUBw.x fC2Plane, fPlaneSign, vCVert			; fC2Plane.x = (CVert.z) - (fPlaneSign, CVert.w)
	// Get distance from next to clip plane (taking into account -/+ clip plane)			(B in above diagram)
	ADDAy.x	ACC, VF00, vNVert						; ACC.x = Vert.z
	MSUBw.x fN2Plane, fPlaneSign, vNVert			; fN2Plane.x = (CVert.z) - (fPlaneSign, CVert.w)
	B	INTERPOLATE_COMMON
INTERPOLATE_Z:
	// Get distance from clip plane to current (taking into account -/+ clip plane)		(-A in above diagram)
	ADDAz.x	ACC, VF00, vCVert						; ACC.x = CVert.z
	MSUBw.x fC2Plane, fPlaneSign, vCVert			; fC2Plane.x = (CVert.z) - (fPlaneSign, CVert.w)
	// Get distance from next to clip plane (taking into account -/+ clip plane)			(B in above diagram)
	ADDAz.x	ACC, VF00, vNVert						; ACC.x = Vert.z
	MSUBw.x fN2Plane, fPlaneSign, vNVert			; fN2Plane.x = (CVert.z) - (fPlaneSign, CVert.w)
	//B	INTERPOLATE_COMMON
INTERPOLATE_COMMON:
	// Calculate L = B + A = B - -A
	SUB.x	fDiff, fN2Plane, fC2Plane
	// Calculate R = |A/L|
	DIV		Q, fC2Plane[x], fDiff[x]
	ADDq.x	fR, vf00, Q
	ABS.x	fR, fR
	// Create a new vertex from the old verts and the ratio (New = Current + (Next-Current)*R)
	// XYZ
	ADDAx.xyzw	ACC,		vCVert,		VF00		// = Current
	MADDAx.xyzw	ACC,		vNVert,		fR			//	= Current + (Next*R)
	MSUBx.xyzw	vNewVert,	vCVert,		fR			//	= Current + (Next*R) - (Current*R)
	SQ.xyzw		vNewVert,	0(pClipOut)
	// ST
	ADDAx.xy	ACC,		vCSTQ,		VF00		// = Current
	MADDAx.xy	ACC,		vNSTQ,		fR			//	= Current + (Next*R)
	MSUBx.xy	vNewSTQ,	vCSTQ,		fR			//	= Current + (Next*R) - (Current*R)
	SQ.xy		vNewSTQ,	1(pClipOut)
	// RGBA
	ADDAx.xyzw	ACC,		vCColour,	VF00		// = Current
	MADDAx.xyzw	ACC,		vNColour,	fR			//	= Current + (Next*R)
	MSUBx.xyzw	vNewColour,	vCColour,	fR			//	= Current + (Next*R) - (Current*R)
	SQ.xyzw		vNewColour,	2(pClipOut)
	// Done
	IADDIU		pClipOut,	pClipOut,	3
	JR	pRtA2

.end
