//////////////////////////////////////////////////////////////////////////////
//
// VU1_Botanicals.vcl
//
// Copyright (c) 2006 Rebellion. All rights reserved.
//
// See Asura_PS2_BotaniclsDefines.h for memory map.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Asura_PS2_BotanicalsDefines.h"
#include "vcl_sml.i"
#include "VU1_CommonMacros.i"
#include "VU1_DynamicLighting.i"

// V C L   S E T U P /////////////////////////////////////////////////////////

.syntax new

.init_vf_all
.init_vi_all

.global VU1_BOTANICALS_INIT
.global VU1_BOTANICALS_SWITCH_CAM_DATA_BFR
.global VU1_BOTANICALS_SWITCH_LIGHT_DATA_BFR
.global VU1_BOTANICALS_SWITCH_RENDERSTATE_BFR
.global VU1_BOTANICALS_LOAD_MESH_FLAGS
.global VU1_BOTANICALS_LOAD_CAM_FLAGS
.global VU1_BOTANICALS_BEGIN_STRIP
.global VU1_BOTANICALS_BEGIN_STRIP_BREAK
.global VU1_BOTANICALS_NOP

// M A C R O S ///////////////////////////////////////////////////////////////

#define NORMALISE_CLIPPED_STS (1)

// These values are assumed in VU1_BOTANICALS_INTERPOLATE.
#define INTERPOLATE_X (2)
#define INTERPOLATE_Y (1)
#define INTERPOLATE_Z (0)

// Sets up mask for last three vertices clip flags for a particular plane.
#define MAKE_PLANE_MASK_FOR_LAST_3_VERTS(plane_flag) (((plane_flag) << 12) | ((plane_flag) << 6) | (plane_flag))

.macro floatExchange  reg1, reg2, componentmask=xyzw
.raw
    ADDx.\componentmask     \reg1, \reg2, VF00          MOVE.\componentmask  \reg2, \reg1
.endraw
.endm

// C O D E ///////////////////////////////////////////////////////////////////

--enter
    in_hw_clip clip
--endenter

    // (*) Note that there are some slightly subtle ordering issues in the
    // first few branches below. We're pretending to VCL that we're MSCNTing
    // our way straight through this stuff while in reality we'll mostly be
    // MSCALing all over the place. We can let VCL know about all the
    // dependencies we require with careful organisation and fake
    // conditionals.

VU1_BOTANICALS_INIT:

    // Setup initial values of the input ptrs that are controlled from EE.
    IADDIU pLightDataBfr,   VI00, ASURA_PS2_BOTANICALS__ADDR__LIGHT_DATA_BFR_0
    IADDIU pCamDataBfr,     VI00, ASURA_PS2_BOTANICALS__ADDR__PER_OBJ_CAM_DATA_BFR_0
    IADDIU pRenderstateBfr, VI00, ASURA_PS2_BOTANICALS__ADDR__RENDERSTATE_BFR_0
    // Push onto integer register stack, to help relieve register pressure.
    ISW.x pLightDataBfr,   ASURA_PS2_BOTANICALS__ADDR__VI_STACK_2(VI00)
    ISW.y pCamDataBfr,     ASURA_PS2_BOTANICALS__ADDR__VI_STACK_2(VI00)
    ISW.w pRenderstateBfr, ASURA_PS2_BOTANICALS__ADDR__VI_STACK_3(VI00)
    // Setup output buffer ptr.
    IADDIU pThisOutputBuffer, VI00, ASURA_PS2_BOTANICALS__ADDR__GEOM_OUTPUT_BFR_0
    // Setup the normal unpacking register.
    MakeNormalUnpackScale fUnpackScale
    // Reset stored cam flags so we recognise they've changed the first time
    // they're uploaded.
    ISW.w VI00, ASURA_PS2_BOTANICALS__ADDR__VI_STACK_1(VI00)
    --cont

VU1_BOTANICALS_LOAD_MESH_FLAGS:

    // Put this first to force VCL to avoid overwriting the register
    // containing iMeshFlags in the ...SWITCH... stuff below. See the (*)
    // comment above.
    XITOP iMeshFlags
    // Determine whether or not mesh is prelit and store result to stack.
    IADDIU iMask, VI00, ASURA_PS2_BOTANICALS__FLAG__PRELIT
    IAND iIsPrelit, iMeshFlags, iMask
    ISW.y iIsPrelit, ASURA_PS2_BOTANICALS__ADDR__VI_STACK_4(VI00)
    // Determine whether or not mesh is point lit and store result to stack.
    IADDIU iMask, VI00, ASURA_PS2_BOTANICALS__FLAG__POINT_LIT
    IAND iIsPointLit, iMeshFlags, iMask
    ISW.y iIsPointLit, ASURA_PS2_BOTANICALS__ADDR__VI_STACK_5(VI00)
    // Determine whether or not mesh is a billboard mesh or a 3D geometry mesh
    IADDIU iMask, VI00, ASURA_PS2_BOTANICALS__FLAG__BILLBOARD
    IAND iIsBillboardMesh, iMeshFlags, iMask
    ISW.x iIsBillboardMesh, ASURA_PS2_BOTANICALS__ADDR__VI_STACK_6(VI00)
    --cont

VU1_BOTANICALS_LOAD_CAM_FLAGS:

    XITOP iCamFlags
    ILW.w iOldVisAndClipFlags, ASURA_PS2_BOTANICALS__ADDR__VI_STACK_1(VI00)
    // Extract components.
    IADDIU iMask, VI00, ASURA_PS2_BOTANICALS__FLAG__VIS_FOR_MORE_THAN_ONE_CAM
    IAND iMoreThanOneCamFlag, iCamFlags, iMask
    IADDIU iMask, VI00, ASURA_PS2_BOTANICALS__MASK__CAM_VIS_AND_CLIP_FLAGS
    IAND iVisAndClipFlags, iCamFlags, iMask
    // Store to stack.
    ISW.x iMoreThanOneCamFlag, ASURA_PS2_BOTANICALS__ADDR__VI_STACK_5(VI00)
    ISW.w iVisAndClipFlags,    ASURA_PS2_BOTANICALS__ADDR__VI_STACK_1(VI00)
    // Have the vis flags changed?
    IADDIU iMask, VI00, ASURA_PS2_BOTANICALS__MASK__CAM_VIS_FLAGS
    IAND iOldVisFlags, iOldVisAndClipFlags, iMask
    IAND iNewVisFlags, iVisAndClipFlags, iMask
    IBEQ iOldVisFlags, iNewVisFlags, VU1_BOTANICALS_FINISHED_SETTING_UP_CAM_FLAGS
    // Yes, they've changed so force RS to be sent next time. Note that
    // writing any non-zero value will do here.
    ISW.w iMask, ASURA_PS2_BOTANICALS__ADDR__VI_STACK_4(VI00)

VU1_BOTANICALS_FINISHED_SETTING_UP_CAM_FLAGS:

    --cont

VU1_BOTANICALS_SWITCH_CAM_DATA_BFR:

    ILW.y pCamDataBfr, ASURA_PS2_BOTANICALS__ADDR__VI_STACK_2(VI00)
    SwitchBfr_Constants pCamDataBfr, ASURA_PS2_BOTANICALS__ADDR__PER_OBJ_CAM_DATA_BFR_0, ASURA_PS2_BOTANICALS__ADDR__PER_OBJ_CAM_DATA_BFR_1
    ISW.y pCamDataBfr, ASURA_PS2_BOTANICALS__ADDR__VI_STACK_2(VI00)
    --cont

VU1_BOTANICALS_SWITCH_LIGHT_DATA_BFR:

    ILW.x pLightDataBfr, ASURA_PS2_BOTANICALS__ADDR__VI_STACK_2(VI00)
    SwitchBfr_Constants pLightDataBfr, ASURA_PS2_BOTANICALS__ADDR__LIGHT_DATA_BFR_0, ASURA_PS2_BOTANICALS__ADDR__LIGHT_DATA_BFR_1
    ISW.x pLightDataBfr, ASURA_PS2_BOTANICALS__ADDR__VI_STACK_2(VI00)
    --cont

VU1_BOTANICALS_SWITCH_RENDERSTATE_BFR:

    // Swap ptrs.
    ILW.w pRenderstateBfr, ASURA_PS2_BOTANICALS__ADDR__VI_STACK_3(VI00)
    SwitchBfr_Constants pRenderstateBfr, ASURA_PS2_BOTANICALS__ADDR__RENDERSTATE_BFR_0, ASURA_PS2_BOTANICALS__ADDR__RENDERSTATE_BFR_1
    ISW.w pRenderstateBfr, ASURA_PS2_BOTANICALS__ADDR__VI_STACK_3(VI00)
    // Set need-to-send-RS flag for next strip. Writing any non-zero value
    // will do.
    ISW.w pRenderstateBfr, ASURA_PS2_BOTANICALS__ADDR__VI_STACK_4(VI00)
    --cont

VU1_BOTANICALS_DUMMY: // Never executed.

    // Make VCL think we may branch to either VU1_BOTANICALS_END_STRIP
    XITOP dummy
    IBNE dummy, VI00, VU1_BOTANICALS_END_STRIP
    --cont

VU1_BOTANICALS_BEGIN_STRIP_BREAK:
    B[D] VU1_BOTANICALS_BEGIN_STRIP

VU1_BOTANICALS_NOP:
VU1_BOTANICALS_END_STRIP:

    --cont
    // NB: This is the one and only point we expect to do an MSCNT from.

VU1_BOTANICALS_BEGIN_STRIP:
    
    // Retrieve main cam data bfr ptr from stack.
    ILW.y pCamDataBfr,   ASURA_PS2_BOTANICALS__ADDR__VI_STACK_2(VI00)
    // Reset cam data ptrs.
    IADDIU pCurConstCamData, VI00, ASURA_PS2_BOTANICALS__ADDR__CONST_CAM_DATA
    IADDIU pCurPerObjCamData, pCamDataBfr, 0
    // Reset remaining cam flags.
    ILW.w iRemainingCamFlags, ASURA_PS2_BOTANICALS__ADDR__VI_STACK_1(VI00)
    ISW.x iRemainingCamFlags, ASURA_PS2_BOTANICALS__ADDR__VI_STACK_4(VI00)
    // Push some registers onto stack.
    ISW.y   pCamDataBfr,       ASURA_PS2_BOTANICALS__ADDR__VI_STACK_2(VI00)
    ISW.z   pCurPerObjCamData, ASURA_PS2_BOTANICALS__ADDR__VI_STACK_2(VI00)
    ISW.w   pCurConstCamData,  ASURA_PS2_BOTANICALS__ADDR__VI_STACK_2(VI00)
    
    // Save the vertex qty for later use 
    // ITOP can also be the number of billboards so need to save off the real number of vertices = billboards*4
    XITOP	iVertexQty
    ISW.y   iVertexQty, ASURA_PS2_BOTANICALS__ADDR__VI_STACK_6(VI00)
    
    // Does this mesh need manipulating e.g. a billboard mesh
    IADDIU  iMask, VI00, ASURA_PS2_BOTANICALS__MASK__ANY_VTX_MANIPULATIONS
    IAND    iAnyVtxManipulations, iMeshFlags, iMask
    IBEQ    iAnyVtxManipulations, VI00, VU1_BOTANICALS_POST_PREPROCESS_MANIPULATIONS
    
    // Transform the input billboard input data 
    // into processed billboard data
    // Do the object2world transform on the billboard centre
    // Hide the billboard XYZ position in W component of 4 vertices
    // Hide the billboard XY size in ZW of ST data
    
    // Have to do the object2world transform as well (1st)
    XITOP	iNumBillboards
    IADD    iVertexQty, iNumBillboards, iNumBillboards // 4 vertices per billboard
    IADD    iVertexQty, iVertexQty, iVertexQty		   // 4 vertices per billboard
    ISW.y   iVertexQty, ASURA_PS2_BOTANICALS__ADDR__VI_STACK_6(VI00)
    
    XTOP    pIn
    IADD    iTemp, iVertexQty, iVertexQty // Assuming here that we have 4
    IADD    iTemp, iTemp, iTemp           // QWs per input vertex.
    IADD    pInEnd, pIn, iTemp
    
    // Do the object2world transform on the billboard centre
    // Hide the billboard XYZ position in XYZ of input RGBA 1
    // Hide the billboard XY size in XY of input RGBA 2
		
	// Load the object2world matrix
    MatrixLoad mO2W, ASURA_PS2_BOTANICALS__OFFSET__CAM_PER_OBJ_O2W_MAT, pCurPerObjCamData
		
    VU1_BOTANICALS_BILLBOARD_PREPROCESS_LOOP:

        --LoopCS  1, 0
        
        // Load the billboard centre
        LQ.xyz fOPos, (0*4 + 0)(pIn)
        
        // Load the billboard xy size
        LQ.xy fBBsize, (1*4 + 0)(pIn)
        
        // 1. Transform the centre from object space to world space
        MatrixMultiplyVertexW1 fBBcentreW, mO2W, fOPos
        
		// Load the object scale which is w-component of the 1st camera horizontal vector for billboarding
		LQ.w fInstanceScale, ASURA_PS2_BOTANICALS__OFFSET__CAM_PER_OBJ_HORIZ_VEC( pCurPerObjCamData )
		
        // Include the instance object scale in the billboard width & height
        MULw.xy fBBsize, fBBsize, fInstanceScale[w]
        
		// Hide the billboard XYZ position in XYZ of input RGBA 1
        SQ.xyz fBBcentreW, (0*4 + 2)(pIn)
        
		// Hide the billboard XY size in XY of input RGBA 2
        SQ.xy fBBsize, (1*4 + 2)(pIn)
	    
        // Update loop state.
        IADDIU pIn,  pIn,  (4*4)
        
        // Done yet?
        ISUB iRemainder, pInEnd, pIn
        IBGTZ iRemainder,  VU1_BOTANICALS_BILLBOARD_PREPROCESS_LOOP

VU1_BOTANICALS_POST_PREPROCESS_MANIPULATIONS:

VU1_BOTANICALS_NEXT_CAMERA:

    // Prepare mask value for loop and setup first iteration test.
    IADDIU iVisBitMask, VI00, 0x80
    IAND iRenderForThisCam, iRemainingCamFlags, iVisBitMask
    // Any cameras left to render this strip for?
    IBEQ iRemainingCamFlags, VI00, VU1_BOTANICALS_FINISHED_FOR_ALL_CAMERAS
    // Reload the vertex count and the light and cam ptrs. Could do this once
    // per strip, rather than per camera, but this helps with register
    // pressure.
    ILW.y iVertexQty,		 ASURA_PS2_BOTANICALS__ADDR__VI_STACK_6(VI00)
    ILW.x pLightDataBfr,     ASURA_PS2_BOTANICALS__ADDR__VI_STACK_2(VI00)
    ILW.z pCurPerObjCamData, ASURA_PS2_BOTANICALS__ADDR__VI_STACK_2(VI00)
    ILW.w pCurConstCamData,  ASURA_PS2_BOTANICALS__ADDR__VI_STACK_2(VI00)

    VU1_BOTANICALS_FIND_NEXT_CAM_LOOP:

        // Render for this cam?
        IBNE iRenderForThisCam, VI00, VU1_BOTANICALS_FOUND_NEXT_CAM
        // No. Shift flags up 2.
        IADD iRemainingCamFlags, iRemainingCamFlags, iRemainingCamFlags
        IADD iRemainingCamFlags, iRemainingCamFlags, iRemainingCamFlags
        // Setup next iteration test.
        IAND iRenderForThisCam, iRemainingCamFlags, iVisBitMask
        // And move ptrs on.
        IADDIU pCurPerObjCamData, pCurPerObjCamData, ASURA_PS2_BOTANICALS__OFFSET__CAM_PER_OBJ_NEXT_CAM
        IADDIU pCurConstCamData,  pCurConstCamData,  ASURA_PS2_BOTANICALS__OFFSET__CONST_CAM_NEXT_CAM
        // Go on to check for next cam.
        B VU1_BOTANICALS_FIND_NEXT_CAM_LOOP

VU1_BOTANICALS_FOUND_NEXT_CAM:

    // Store registers back to stack.
    ISW.x iRemainingCamFlags, ASURA_PS2_BOTANICALS__ADDR__VI_STACK_4(VI00)
    ISW.z pCurPerObjCamData,  ASURA_PS2_BOTANICALS__ADDR__VI_STACK_2(VI00)
    ISW.w pCurConstCamData,   ASURA_PS2_BOTANICALS__ADDR__VI_STACK_2(VI00)
    // Fix up NLOOP in the GIF tag in the output buffer.
    IADDIU iNLOOPandEOP, iVertexQty,   0x7FFF
    IADDIU iNLOOPandEOP, iNLOOPandEOP, 1
    ISW.x  iNLOOPandEOP, ASURA_PS2_BOTANICALS__OFFSET__GEOM_OUT_DRAW_GIF_TAG(pThisOutputBuffer)
    
	// Set the fade alpha scale for this mesh (per camera setting)
	// Stored in w-component of camera vertical view vector
	LQ.w fAlphaScale, ASURA_PS2_BOTANICALS__OFFSET__CAM_PER_OBJ_HEIGHT_VEC( pCurPerObjCamData )
	
	// Store the alpha scale in the light data
    SQ.w fAlphaScale, 0(pLightDataBfr) // afSHMatrix[0].w
	
	// Ignore this object if alpha is 0
	MTIR iAlphaScale, fAlphaScale[w]
	IBEQ iAlphaScale, vi00, VU1_BOTANICALS_FINISHED_FOR_CUR_CAMERA
	
	// Store it 
		
    // Determine whether or not clipping will be required.
    IADDIU iMask, VI00, 0x40
    IAND iClipReqd, iRemainingCamFlags, iMask
    // Does this mesh need manipulating e.g. a billboard mesh
    IADDIU  iMask, VI00, ASURA_PS2_BOTANICALS__MASK__ANY_VTX_MANIPULATIONS
    IAND    iAnyVtxManipulations, iMeshFlags, iMask
    IBEQ    iAnyVtxManipulations, VI00, VU1_BOTANICALS_POST_VTX_MANIPULATIONS
    
    // Transform the input billboard input data 
    // Have to do the object2world transform as well (1st)
    XITOP	iNumBillboards
    IADD    iVertexQty, iNumBillboards, iNumBillboards // 4 vertices per billboard
    IADD    iVertexQty, iVertexQty, iVertexQty		   // 4 vertices per billboard
    ISW.y   iVertexQty, ASURA_PS2_BOTANICALS__ADDR__VI_STACK_6(VI00)
    
    XTOP    pIn
    IADD    iTemp, iVertexQty, iVertexQty // Assuming here that we have 4
    IADD    iTemp, iTemp, iTemp           // QWs per input vertex.
    IADD    pInEnd, pIn, iTemp
    
    // Loop over making quad positions from the billboard data
    // Loop over per billboard outputting the 4 vertices
		
	// Load the object2world matrix
    MatrixLoad mO2W, ASURA_PS2_BOTANICALS__OFFSET__CAM_PER_OBJ_O2W_MAT, pCurPerObjCamData
		
    VU1_BOTANICALS_BILLBOARD_LOOP:

        --LoopCS  1, 0
        
        // the billboard centre is stored in xyz of RGBA 0
        LQ.xyz fBBcentreW, (0*4 + 2)(pIn)
        
		// Load the camera horizontal vector for billboarding
		// w-component is the object scale
		LQ.xyzw fWidthView, ASURA_PS2_BOTANICALS__OFFSET__CAM_PER_OBJ_HORIZ_VEC( pCurPerObjCamData )
		// Load the camera height direction
		LQ.xyz fHeightView, ASURA_PS2_BOTANICALS__OFFSET__CAM_PER_OBJ_HEIGHT_VEC( pCurPerObjCamData )
		// Load the camera fwd direction (used for the billboard normal)
		LQ.xyz fFwdView, ASURA_PS2_BOTANICALS__OFFSET__CAM_PER_OBJ_FWD_VEC( pCurPerObjCamData )
	
        // billboard size is stored in xy of RGBA 1 (this already has the instance object scale in it)
        LQ.xy fBBsize, (1*4 + 2)(pIn)
        
        // Make the quad from this position
        MULx.xyz fWidth, fWidthView, fBBsize[x]
        MULy.xyz fHeight, fHeightView, fBBsize[y]
        ADD.xyz fHeight2, fHeight, fHeight
        
        // * = fBBcentreW
        // 0 = top-left
        // 1 = top-right
        // 2 = bottom-left
        // 3 = bottom-right
        // 
        // 0---1
        // |  /|
        // | * |
        // |/  |
        // 2---3
        //
        // xHeight & xWidth are the half width & height
        // vertex[0] = xPos - xWidth - xHeight
        SUB.xyz fPos0, fBBcentreW, fWidth
        SUB.xyz fPos0, fPos0, fHeight

        // vertex[3] = xPos + xWidth + xHeight
        ADD.xyz fPos3, fBBcentreW, fWidth
        ADD.xyz fPos3, fPos3, fHeight

        // vertex[1] = vertex[3] - xHeight * 2
        SUB.xyz fPos1, fPos3, fHeight2
        
        // vertex[2] = vertex[0] + xHeight * 2
        ADD.xyz fPos2, fPos0, fHeight2
        
	    // Set the ADC bit in w part of normal
	    IADDIU iADCon, vi00, 0x4000
	    IADDIU iADCon, iADCon, 0x4000
	    
        SQ.xyz fPos0, (0*4 + 0)(pIn)
        SQ.xyz fPos1, (1*4 + 0)(pIn)
        SQ.xyz fPos2, (2*4 + 0)(pIn)
        SQ.xyz fPos3, (3*4 + 0)(pIn)
        
        // The ADC bits
        // ADC on for vertex 0, 1
	    ISW.w iADCon, (0*4 + 3)(pIn)
	    ISW.w iADCon, (1*4 + 3)(pIn)
        // ADC off for vertex 2, 3
	    ISW.w vi00,   (2*4 + 3)(pIn)
	    ISW.w vi00,   (3*4 + 3)(pIn)
	    
	    // The normal
	    // Convert floating point normal to integer representation ( + 1.0f then * 8 then FTOI4 )
	    ADDw.xyz fFwdView, fFwdView, vf00[w]
	    LOI 8.0
	    MULI.xyz fFwdView, fFwdView, I
	    FTOI4.xyz fIntFwdView, fFwdView
	    
		SQ.xyz fIntFwdView, (0*4 + 3)(pIn)
		SQ.xyz fIntFwdView, (1*4 + 3)(pIn)
		SQ.xyz fIntFwdView, (2*4 + 3)(pIn)
		SQ.xyz fIntFwdView, (3*4 + 3)(pIn)
	    
        // Update loop state.
        IADDIU pIn,  pIn,  (4*4)
        
        // Done yet?
        ISUB iRemainder, pInEnd, pIn
        IBGTZ iRemainder, VU1_BOTANICALS_BILLBOARD_LOOP

VU1_BOTANICALS_POST_VTX_MANIPULATIONS:

    // Determine whether or not reflective STs are required.
    IADDIU iMask, VI00, ASURA_PS2_BOTANICALS__FLAG__REFLECTIVE
    IAND iReflective, iMeshFlags, iMask
    
    // Setup input and end-of-input ptrs.
    XTOP pIn
    IADD iTemp, iVertexQty, iVertexQty // Assuming here that we have 4
    IADD iTemp, iTemp, iTemp           // QWs per input vertex.
    IADD pInEnd, pIn, iTemp
    
    // Reflective pass enabled?
    IBEQ iReflective, VI00, VU1_BOTANICALS_POST_REFLECTIVE_ST_CREATE

VU1_BOTANICALS_REFLECTIVE_ST_CREATE:

    // Setup output ptr.
    IADDIU pOut, VI00, ASURA_PS2_BOTANICALS__ADDR__ST_OVERWRITE_BFR
    // Load in matrices.
    MatrixLoad mO2V, ASURA_PS2_BOTANICALS__OFFSET__CAM_PER_OBJ_O2V_MAT, pCurPerObjCamData
    MatrixLoad mTex, ASURA_PS2_BOTANICALS__OFFSET__CAM_PER_OBJ_TEX_MAT, pCurPerObjCamData

    VU1_BOTANICALS_REFLECTIVE_ST_CREATE_LOOP:

        --LoopCS  3, 0
        --LoopAbs 26

        // Load objectspace vertex and normal.
        LQ.xyz fPos, 0(pIn)
        LQ.xyz fNrm, 3(pIn)
        // Convert input data.
        UnpackNormal fNrm, fUnpackScale
        // Transform position and normal to viewspace.
        MatrixMultiplyVertexW1 fPos, mO2V, fPos
        MatrixMultiplyVector   fNrm, mO2V, fNrm
        // Normalise the view space position. This is then just a direction
        // from the camera.
        VectorNormalize fDir, fPos
        // Calculate 2(D.N).
        VectorDotProduct fDot, fDir, fNrm
        ADD.x fDot, fDot, fDot
        // Calculate R = D - 2(D.N)N
        MULx.xyz fR, fNrm, fDot
        SUB.xyz  fR, fDir, fR
        // Apply texture coordinate matrix.
        MatrixMultiplyVertexW1 fST, mTex, fR
        // Output.
        SQI.xy fST, (pOut++)
        // Update loop state.
        IADDIU pIn, pIn, 4
        // Done yet?
        IBNE pIn, pInEnd, VU1_BOTANICALS_REFLECTIVE_ST_CREATE_LOOP

    // Reset input ptr.
    XTOP pIn

VU1_BOTANICALS_POST_REFLECTIVE_ST_CREATE:

    // Retrieve the is-prelit and is-point-lit flags from stack.
    ILW.y iIsPrelit,   ASURA_PS2_BOTANICALS__ADDR__VI_STACK_4(VI00)
    ILW.y iIsPointLit, ASURA_PS2_BOTANICALS__ADDR__VI_STACK_5(VI00)
    // Setup input and output pointers for main render loop.
    IADDIU pPrelightIn, pIn, 2
    IADDIU pOut, pThisOutputBuffer, ASURA_PS2_BOTANICALS__OFFSET__GEOM_OUT_FIRST_VERTEX
    // Load in O->GBC matrix and GBC->S scale and translation vectors.
    MatrixLoad mO2GBC, ASURA_PS2_BOTANICALS__OFFSET__CAM_PER_OBJ_O2GBC_MAT, pCurPerObjCamData
    LQ fGBC2SScale, ASURA_PS2_BOTANICALS__OFFSET__CONST_CAM_GBC2S_SCALE(pCurConstCamData)
    LQ fGBC2STrans, ASURA_PS2_BOTANICALS__OFFSET__CONST_CAM_GBC2S_TRANS(pCurConstCamData)
    // Set input ST's Q component to 1. This will be maintained throughout
    // the loops below.
    MAXw.z fInSTQ, VF00, VF00
    // Clipping required for current camera?
    IBNE iClipReqd, VI00, VU1_BOTANICALS_CLIPPING_RENDERER
    // Projected texture for shadows ?
    IADDIU  iMask, VI00, ASURA_PS2_BOTANICALS__FLAG__PROJECTED_TEXTURE
    IAND    iShadowProjection, iMeshFlags, iMask
    IBNE iShadowProjection, VI00, VU1_BOTANICALS_NO_CLIP_PROJECT
    // Point lit? (This implies prelit.)
    IBNE iIsPointLit, VI00, VU1_BOTANICALS_NO_CLIP_LIT_POINT
    // Just prelit?
    IBNE iIsPrelit, VI00, VU1_BOTANICALS_NO_CLIP_PRELIT
    // Must be SH lit then.

VU1_BOTANICALS_NO_CLIP_LIT_SH:

    // Load light data.
    LQ     afSHMatrix[0], 0(pLightDataBfr)
    LQ     afSHMatrix[1], 1(pLightDataBfr)
    LQ.xyz afSHMatrix[2], 2(pLightDataBfr)
    LQ.xyz afSHMatrix[3], 3(pLightDataBfr)
    // Get value ready in I register for clamping RGBAs.
    LOI 255.0

    VU1_BOTANICALS_NO_CLIP_LIT_SH_LOOP:
        
        --LoopCS  3, 0
        --LoopAbs 21

        // Load the vertex.
        LQ.xyz  fOPos,     0(pIn)
        LQ.xy   fInSTQ,    1(pIn)
        LQ.w    fInRGBA,   2(pIn)
        LQ      fNrm,      3(pIn)
        // Convert input data.
        ITOF12.xy       fInSTQ, fInSTQ
        UnpackNormalADC fNrm,   fUnpackScale
        // Transform to GB clip space.
        MatrixMultiplyVertexW1 fGBCPos, mO2GBC, fOPos
        // Scale and translate to screenspace.
        MULA  ACC,   fGBCPos,     fGBC2SScale
        MADDw fSPos, fGBC2STrans, fGBCPos[w]
        // Perspective divide XYZ and STs.
        DIV      Q,     VF00[w], fSPos[w]
        MULq.xyz fSPos, fSPos,   Q
        MULq.xyz fSTQ,  fInSTQ,  Q
        // Light vertex.
        LightVtxWithSH fRGBA, fNrm, afSHMatrix
        // Clamp result to [0, 255] and scale alpha.
        MINIi.xyz fRGBA, fRGBA, I
        MUL.w fRGBA, fRGBA, fInRGBA
        // Convert output data.
        FTOI4.xyz fSPos, fSPos
        FTOI0     fRGBA, fRGBA
        // Store.
        SQ.xyz  fSTQ,  0(pOut)
        SQ      fRGBA, 1(pOut)
        SQ.xyz  fSPos, 2(pOut)
        SQ.w    fNrm,  2(pOut)
        // Update loop state.
        IADDIU pIn,  pIn,  4
        IADDIU pOut, pOut, 3
        // Done yet?
        IBNE pIn, pInEnd, VU1_BOTANICALS_NO_CLIP_LIT_SH_LOOP

    B VU1_BOTANICALS_NO_CLIP_POST_T_AND_L

VU1_BOTANICALS_NO_CLIP_PRELIT:

    VU1_BOTANICALS_NO_CLIP_PRELIT_LOOP:

        --LoopCS  3, 0
        --LoopAbs 12

        // Load the vertex.
        LQ.xyz fOPos,  0(pIn)
        LQ.xy  fInSTQ, 1(pIn)
        LQ     fRGBA,  2(pIn)
        LQ.w   fNrm,   3(pIn)
        // Convert input data.
        ITOF12.xy fInSTQ, fInSTQ
        // Transform to GB clip space.
        MatrixMultiplyVertexW1 fGBCPos, mO2GBC, fOPos
        // Scale and translate to screenspace.
        MULA  ACC,   fGBCPos,     fGBC2SScale
        MADDw fSPos, fGBC2STrans, fGBCPos[w]
        // Perspective divide XYZ and STs.
        DIV      Q,     VF00[w], fSPos[w]
        MULq.xyz fSPos, fSPos,   Q
        MULq.xyz fSTQ,  fInSTQ,  Q
        // Convert output data.
        FTOI4.xyz fSPos, fSPos
        // Grab degeneracy ADC bit from input normal.
        UnpackADC fSPos, fNrm, fUnpackScale
        // Store.
        SQ.xyz fSTQ,  0(pOut)
        SQ     fRGBA, 1(pOut)
        SQ     fSPos, 2(pOut)
        // Update loop state.
        IADDIU pIn,  pIn,  4
        IADDIU pOut, pOut, 3
        // Done yet?
        IBNE pIn, pInEnd, VU1_BOTANICALS_NO_CLIP_PRELIT_LOOP

    B VU1_BOTANICALS_NO_CLIP_POST_T_AND_L

VU1_BOTANICALS_NO_CLIP_PROJECT:
 
    IADDIU  iZero21, VI00, 0
    IADDIU  iZero1, VI00, 0
    IADDIU  iZeroMask, VI00, 1

    LQ fShadowFallOff, ASURA_PS2_BOTANICALS__ADDR__CONST_SHADOW_DATA(VI00)

    VU1_BOTANICALS_NO_CLIP_PROJECT_LOOP:

        --LoopCS  3, 0
        --LoopAbs 19

        // Load the vertex.
        LQ.xyz fOPos,  0(pIn)
#if NO_PERSPECTIVE_ADJUST
        LQ.xy   fInSTQ, 1(pIn)
#else
        LQ.xyz  fInSTQ, 1(pIn)
#endif
        LQ     fRGBA,  2(pIn)

        // Transform to GB clip space.
        MatrixMultiplyVertexW1 fGBCPos, mO2GBC, fOPos
        // Scale and translate to screenspace.
        MULA  ACC,   fGBCPos,     fGBC2SScale
        MADDw fSPos, fGBC2STrans, fGBCPos[w]
        // Perspective divide XYZ and STs.
        DIV      Q,     VF00[w], fSPos[w]
        MULq.xyz fSPos, fSPos,   Q
        MULq.xyz fSTQ,  fInSTQ,  Q
        // Convert output data.
        FTOI4.xyz fSPos, fSPos

        // now compute reduction in alpha component for distance from camera
        //cheat for now - hardwire 20 metre fade off over 5 metres
        SUBz.w  fAlpha, fSPos, fShadowFallOff[z]
        MULw.w  fAlpha, fAlpha, fShadowFallOff[w]
        MINI.w  fAlpha, fAlpha, VF00    // upper limit 1.0f
        MAXx.w  fAlpha, fAlpha, VF00[x] // lower limit 0.0f
        MUL.w   fAlpha, fRGBA, fAlpha   // combine with distance from light value
        FMAND   iZero0, iZeroMask       // need to know if this was zero w
        FTOI0.w fRGBA, fAlpha           // and convert to 8 bit integer format

        // Grab degeneracy ADC bit from input normal :: note due to preprocessing this is one of 0x0, 0x40, 0x80 and 0xC0
        ILW.w   iADC,   3(pIn)
        IADDIU  iADC, iADC, 0x7FC0  // sets top bit for all rejected cases

        // now check whether the last 3 verts have had zero alpha, and if so set ADC bit for this tri
        IAND    iZero21, iZero21, iZero0    // combine new result with previous two
        IADDIU  iZero21, iZero21, 0x7fff    // convert to 0x8000 if set
        IOR     iADC, iADC, iZero21         // if either is set then don't draw
        MFIR.w  fSPos, iADC

        IAND    iZero21, iZero1, iZero0     // combine last two flags
        IADDIU  iZero1, iZero0, 0           // and push most recent one down

        // Store.
        SQ.xyz fSTQ,  0(pOut)
        SQ     fRGBA, 1(pOut)
        SQ     fSPos, 2(pOut)
        // Update loop state.
        IADDIU pIn,  pIn,  4
        IADDIU pOut, pOut, 3
        // Done yet?
        IBNE pIn, pInEnd, VU1_BOTANICALS_NO_CLIP_PROJECT_LOOP

    B VU1_BOTANICALS_NO_CLIP_POST_T_AND_L

VU1_BOTANICALS_NO_CLIP_LIT_POINT:
    
    // Load light qty.
    ILW.w iLightQty, ASURA_PS2_BOTANICALS__OFFSET__QW_CONTAINING_QTY(pLightDataBfr)

    VU1_BOTANICALS_NO_CLIP_LIT_POINT_LOOP:

        --LoopCS  3, 0
//TODO  --LoopAbs

        // Load the vertex.
        LQ.xyz  fOPos,  0(pIn)
        LQ.xy   fInSTQ, 1(pIn)
        LQ      fRGBA,  2(pIn)
        LQ      fNrm,   3(pIn)
        // Convert input data.
        ITOF12.xy       fInSTQ, fInSTQ
        ITOF0           fRGBA,  fRGBA
        UnpackNormalADC fNrm,   fUnpackScale
        // Transform to GB clip space.
        MatrixMultiplyVertexW1 fGBCPos, mO2GBC, fOPos
        // Scale and translate to screenspace.
        MULA  ACC,   fGBCPos,     fGBC2SScale
        MADDw fSPos, fGBC2STrans, fGBCPos[w]
        // Perspective divide XYZ and STs.
        DIV      Q,     VF00[w], fSPos[w]
        MULq.xyz fSPos, fSPos,   Q
        MULq.xyz fSTQ,  fInSTQ,  Q
        // Light the vertex.
        LightEnvironment fOPos, fNrm, fRGBA, iLightQty, pLightDataBfr
        // Convert output data.
        FTOI4.xyz fSPos, fSPos
        FTOI0     fRGBA, fRGBA
        // Store.
        SQ.xyz  fSTQ,  0(pOut)
        SQ      fRGBA, 1(pOut)
        SQ.xyz  fSPos, 2(pOut)
        SQ.w    fNrm,  2(pOut)
        // Update loop state.
        IADDIU pIn,  pIn,  4
        IADDIU pOut, pOut, 3
        // Done yet?
        IBNE pIn, pInEnd, VU1_BOTANICALS_NO_CLIP_LIT_POINT_LOOP

VU1_BOTANICALS_NO_CLIP_POST_T_AND_L:

    // Render all the passes.
    IADDIU pOverwriteSTs, VI00, ASURA_PS2_BOTANICALS__ADDR__ST_OVERWRITE_BFR
    BAL pRenderAllPassesRA, VU1_BOTANICALS_RENDER_ALL_PASSES
    // And off to the next camera.
    B VU1_BOTANICALS_FINISHED_FOR_CUR_CAMERA

VU1_BOTANICALS_CLIPPING_RENDERER:

    // Flags to record whether we need to clip anything or if we clipped
    // everything.
    IADDIU iAnyClipped, VI00, 0
    IADDIU iAllClipped, VI00, 0x3F
    // Setup output ptr to clipper input buffer.
    IADDIU pIntermediateOut, VI00, ASURA_PS2_BOTANICALS__ADDR__CLIP_INPUT_BFR
    // Projected texture for shadows ?
    IADDIU  iMask, VI00, ASURA_PS2_BOTANICALS__FLAG__PROJECTED_TEXTURE
    IAND    iShadowProjection, iMeshFlags, iMask
    IBNE iShadowProjection, VI00, VU1_BOTANICALS_CLIP_PROJECT
    // Point lit? (This implies prelit.)
    IBNE iIsPointLit, VI00, VU1_BOTANICALS_CLIP_LIT_POINT
    // Just prelit?
    IBNE iIsPrelit, VI00, VU1_BOTANICALS_CLIP_PRELIT
    // Must be SH lit then.

VU1_BOTANICALS_CLIP_LIT_SH:

    // Load light data.
    LQ     afSHMatrix[0], 0(pLightDataBfr)
    LQ     afSHMatrix[1], 1(pLightDataBfr)
    LQ.xyz afSHMatrix[2], 2(pLightDataBfr)
    LQ.xyz afSHMatrix[3], 3(pLightDataBfr)
    // Get value ready in I register for clamping RGBAs.
    LOI 255.0

    VU1_BOTANICALS_CLIP_LIT_SH_LOOP:
        
        --LoopCS  3, 0
        --LoopAbs 24

        // Load the vertex.
        LQ.xyz fOPos,   0(pIn)
        LQ.xy  fInSTQ,  1(pIn)
        LQ.w   fInRGBA, 2(pIn)
        LQ     fNrm,    3(pIn)
        // Convert input data.
        ITOF12.xy       fInSTQ, fInSTQ
        UnpackNormalADC fNrm,   fUnpackScale
        // Transform to GB clip space.
        MatrixMultiplyVertexW1 fGBCPos, mO2GBC, fOPos
        // Perform clipping judgement.
        CLIPw.xyz fGBCPos, fGBCPos
        // Scale and translate to screenspace.
        MULA  ACC,   fGBCPos,     fGBC2SScale
        MADDw fSPos, fGBC2STrans, fGBCPos[w]
        // Perspective divide XYZ and STs.
        DIV      Q,     VF00[w], fSPos[w]
        MULq.xyz fSPos, fSPos,   Q
        MULq.xyz fSTQ,  fInSTQ,  Q
        // Light vertex.
        LightVtxWithSH fRGBA, fNrm, afSHMatrix
        // Clamp result to [0, 255] and scale alpha.
        MINIi.xyz fRGBA, fRGBA, I
        MUL.w fRGBA, fRGBA, fInRGBA
        // Get clip flags and AND them with iAllClipped. If iAllClipped is
        // anything other than 0 at the end, then all verts were outside one
        // of the planes.
        FCGET iClipFlags
        IAND  iAllClipped, iAllClipped, iClipFlags
        IOR   iAnyClipped, iAnyClipped, iClipFlags
        // Store (GB) clip space vertex, unpacked ST and colour.
        SQI    fGBCPos, (pIntermediateOut++)
        SQI.xy fInSTQ,  (pIntermediateOut++)
        SQI    fRGBA,   (pIntermediateOut++)
        // Get clipping results.
        FCAND  VI01, 0x3FFFF
        IADDIU iClipADC, VI01, 0x7FFF
        // Combine the clip ADC with the degeneracy ADC.
        MTIR iInADC,  fNrm[w]
        IOR  iOutADC, iInADC, iClipADC
        // In spare w component of intermediate STQ, store sum of iInADC and
        // iClipADC. This will be 0x8000 iff the vertex was rejected here
        // but is not degenerate.  In the other case (degenerate but not rejected)
        // the result will be 0xFFFF, hence the equality test used later in 
        IADD  iSum, iInADC, iClipADC
        ISW.w iSum, -2(pIntermediateOut)
        // Convert output data.
        FTOI4.xyz fSPos, fSPos
        FTOI0     fRGBA, fRGBA
        // Store.
        SQ.xyz fSTQ,    0(pOut)
        SQ     fRGBA,   1(pOut)
        SQ.xyz fSPos,   2(pOut)
        ISW.w  iOutADC, 2(pOut)
        // Update ptrs.
        IADDIU pIn,  pIn,  4
        IADDIU pOut, pOut, 3
        // Done yet?
        IBNE pIn, pInEnd, VU1_BOTANICALS_CLIP_LIT_SH_LOOP

    B VU1_BOTANICALS_CLIP_POST_T_AND_L

VU1_BOTANICALS_CLIP_PRELIT:

    VU1_BOTANICALS_CLIP_PRELIT_LOOP:

        --LoopCS  3, 0
        --LoopAbs 23

        // Load the vertex.
        LQI.xyz fOPos,  (pIn++)
        LQI.xy  fInSTQ, (pIn++)
        LQI     fRGBA,  (pIn++)
        LQI.w   fNrm,   (pIn++)
        // Convert input data.
        ITOF12.xy fInSTQ, fInSTQ
        UnpackADC fNrm, fNrm, fUnpackScale
        // Transform to GB clip space.
        MatrixMultiplyVertexW1 fGBCPos, mO2GBC, fOPos
        // Perform clipping judgement.
        CLIPw.xyz fGBCPos, fGBCPos
        // Scale and translate to screenspace.
        MULA  ACC,   fGBCPos,     fGBC2SScale
        MADDw fSPos, fGBC2STrans, fGBCPos[w]
        // Perspective divide XYZ and STs.
        DIV      Q,     VF00[w], fSPos[w]
        MULq.xyz fSPos, fSPos,   Q
        MULq.xyz fSTQ,  fInSTQ,  Q
        // Get clip flags and AND them with iAllClipped. If iAllClipped is
        // anything other than 0 at the end, then all verts were outside one
        // of the planes.
        FCGET iClipFlags
        IAND  iAllClipped, iAllClipped, iClipFlags
        IOR   iAnyClipped, iAnyClipped, iClipFlags
        // Intermediate colour needs to be in float format.
        ITOF0 fIntermediateRGBA, fRGBA
        // Store (GB) clip space vertex, unpacked ST and colour.
        SQI    fGBCPos,           (pIntermediateOut++)
        SQI.xy fInSTQ,            (pIntermediateOut++)
        SQI    fIntermediateRGBA, (pIntermediateOut++)
        // Get clipping results.
        FCAND  VI01, 0x3FFFF
        IADDIU iClipADC, VI01, 0x7FFF
        // Combine the clip ADC with the degeneracy ADC.
        MTIR iInADC,  fNrm[w]
        IOR  iOutADC, iInADC, iClipADC
        // In spare w component of intermediate STQ, store sum of iInADC and
        // iClipADC. This will be 0x8000 iff the vertex was rejected here
        // but is not degenerate.
        IADD  iSum, iInADC, iClipADC
        ISW.w iSum, -2(pIntermediateOut)
        // Convert output data.
        FTOI4.xyz fSPos, fSPos
        // Store.
        SQ.xyz fSTQ,    0(pOut)
        SQ     fRGBA,   1(pOut)
        SQ.xyz fSPos,   2(pOut)
        ISW.w  iOutADC, 2(pOut)
        // Update ptrs.
        IADDIU pOut, pOut, 3
        // Done yet?
        IBNE pIn, pInEnd, VU1_BOTANICALS_CLIP_PRELIT_LOOP

    // Ok, we've finished transforming this strip.
    B VU1_BOTANICALS_CLIP_POST_T_AND_L

VU1_BOTANICALS_CLIP_PROJECT:

    // Particularly tight for integer registers at this point...
    ISW.x pThisOutputBuffer, ASURA_PS2_BOTANICALS__ADDR__VI_STACK_3(VI00)
    ISW.y iVertexQty,        ASURA_PS2_BOTANICALS__ADDR__VI_STACK_3(VI00)
    ISW.z iMeshFlags,        ASURA_PS2_BOTANICALS__ADDR__VI_STACK_3(VI00)

    LQ fShadowFallOff, ASURA_PS2_BOTANICALS__ADDR__CONST_SHADOW_DATA(VI00)

    IADDIU iZero21, VI00, 0
    IADDIU iZero1, VI00, 0

VU1_BOTANICALS_CLIP_PROJECT_LOOP:

        --LoopCS  3, 0
        --LoopAbs 35

        // Load the vertex.
        LQI.xyz fOPos,  (pIn++)
#if NO_PERSPECTIVE_ADJUST
        LQI.xy  fInSTQ, (pIn++) // z defaults to 1.0f
#else
        LQI.xyz fInSTQ, (pIn++)
#endif
        LQI     fRGBA,  (pIn++)
        ILW.w   iInADC, 0(pIn)  // ADC bit will be used in code, and is in mixed form (0x40 = rejected, 0x80 = ADC bit)
        IADDIU  pIn, pIn, 1     // which unfortunately means we can't us ++ :-(
        IADDIU  iInADC, iInADC, 0x7FC0  // push value to ensure top bit set

        IADDIU iZeroMask, VI00, 0x1
        IADDIU i0x8000, iZeroMask, (0x8000 - 0x1)   

        IAND    iInADC, iInADC, i0x8000

        // Transform to GB clip space.
        MatrixMultiplyVertexW1 fGBCPos, mO2GBC, fOPos
        // Perform clipping judgement.
        CLIPw.xyz fGBCPos, fGBCPos
        // Scale and translate to screenspace.
        MULA  ACC,   fGBCPos,     fGBC2SScale
        MADDw fSPos, fGBC2STrans, fGBCPos[w]
        // Perspective divide XYZ and STs.
        DIV      Q,     VF00[w], fSPos[w]
        MULq.xyz fSPos, fSPos,   Q
        MULq.xyz fSTQ,  fInSTQ,  Q  // note that Z incoming already contains Q biasing from shadow projection
        // Get clip flags and AND them with iAllClipped. If iAllClipped is
        // anything other than 0 at the end, then all verts were outside one
        // of the planes.
        FCGET iClipFlags
        IAND  iAllClipped, iAllClipped, iClipFlags
        IOR   iAnyClipped, iAnyClipped, iClipFlags
        // Intermediate colour needs to be in float format.
        ITOF0.xyz fRGBA, fRGBA      // NOTE : alpha is already in float format
        // Store (GB) clip space vertex, unpacked ST and colour.
        SQI    fGBCPos,           (pIntermediateOut++)
        SQI.xyz fInSTQ,            (pIntermediateOut++)
        // Get clipping results.
        FCAND  VI01, 0x3FFFF
        IADDIU iClipADC, VI01, 0x7FFF
        // Combine the clip ADC with the degeneracy ADC.
// loaded direct        MTIR iInADC,  fNrm[w]
        IOR  iOutADC, iInADC, iClipADC
        // at this stage iOutADC can only be 0 or 0x8000
        // but iClipADC can be 7FFF or 8000

        // now compute reduction in alpha component for distance from camera
        //cheat for now - hardwire 20 metre fade off over 5 metres
        SUBz.w  fAlpha, fSPos, fShadowFallOff[z]
        MULw.w  fAlpha, fAlpha, fShadowFallOff[w]
        MINI.w  fAlpha, fAlpha, VF00    // upper limit 1.0f
        MAXx.w  fAlpha, fAlpha, VF00[x] // lower limit 0.0f

        MUL.w   fRGBA, fRGBA, fAlpha    // combine with distance from light value
        FMAND   iZero0, iZeroMask       // need to know if this was zero w
        SQI    fRGBA, (pIntermediateOut++)  // had to be moved down to ensure alpha updated before clipping
        FTOI0  fRGBA, fRGBA         // and convert to 8 bit integer format

        // now check whether the last 3 verts have had zero alpha, and if so set ADC bit for this tri
        IAND    iZero21, iZero21, iZero0    // combine new result with previous two
        IADDIU  iZero21, iZero21, 0x7fff    // convert to 0x8000 if set
        IAND    iZero21, iZero21, i0x8000   // mask out to just top bit
        IOR     iOutADC, iOutADC, iZero21           // if either is set then don't draw
        // this will now be either 0 or 0x8000
        
        // need this result to also be passed through in the intermediate results
        // as if this vertex was rejected as degenerate, because then it won't bother clipping it
        // only way to do this is to OR it into iInADC, but only the top bit
        IOR     iInADC, iInADC, iZero21     // and combine with existing

        // prepare the partial results for the next loop round
        IAND    iZero21, iZero1, iZero0     // combine last two flags
        IADDIU  iZero1, iZero0, 0           // and push most recent one down

        // In spare w component of intermediate STQ, store sum of iInADC and
        // iClipADC. This will be 0x8000 iff the vertex was rejected here
        // but is not degenerate.
        IADD  iSum, iInADC, iClipADC
        ISW.w iSum, -2(pIntermediateOut)
        // Convert output data.
        FTOI4.xyz fSPos, fSPos
        // Store.
        SQ.xyz fSTQ,    0(pOut)
        SQ     fRGBA,   1(pOut)
        SQ.xyz fSPos,   2(pOut)
        ISW.w  iOutADC, 2(pOut)
        // Update ptrs.
        IADDIU pOut, pOut, 3
        // Done yet?
        IBNE pIn, pInEnd, VU1_BOTANICALS_CLIP_PROJECT_LOOP

    // Retrieve registers from stack.
    ILW.x pThisOutputBuffer, ASURA_PS2_BOTANICALS__ADDR__VI_STACK_3(VI00)
    ILW.y iVertexQty,        ASURA_PS2_BOTANICALS__ADDR__VI_STACK_3(VI00)
    ILW.z iMeshFlags,        ASURA_PS2_BOTANICALS__ADDR__VI_STACK_3(VI00)

    // Ok, we've finished transforming this strip.
    B VU1_BOTANICALS_CLIP_POST_T_AND_L

VU1_BOTANICALS_CLIP_LIT_POINT:
    
    // Particularly tight for integer registers at this point...
    ISW.x pThisOutputBuffer, ASURA_PS2_BOTANICALS__ADDR__VI_STACK_3(VI00)
    ISW.y iVertexQty,        ASURA_PS2_BOTANICALS__ADDR__VI_STACK_3(VI00)
    ISW.z iMeshFlags,        ASURA_PS2_BOTANICALS__ADDR__VI_STACK_3(VI00)
    // Load light qty.
    ILW.w iLightQty, ASURA_PS2_BOTANICALS__OFFSET__QW_CONTAINING_QTY(pLightDataBfr)
    
    VU1_BOTANICALS_CLIP_LIT_POINT_LOOP:

        --LoopCS 3, 0
//TODO  --LoopAbs 

        // Load the vertex.
        LQ.xyz fOPos,  0(pIn)
        LQ.xy  fInSTQ, 1(pIn)
        LQ     fRGBA,  2(pIn)
        LQ     fNrm,   3(pIn)
        // Convert input data.
        ITOF12.xy       fInSTQ, fInSTQ
        ITOF0           fRGBA,  fRGBA
        UnpackNormalADC fNrm,   fUnpackScale
        // Transform to GB clip space.
        MatrixMultiplyVertexW1 fGBCPos, mO2GBC, fOPos
        // Perform clipping judgement.
        CLIPw.xyz fGBCPos, fGBCPos
        // Scale and translate to screenspace.
        MULA  ACC,   fGBCPos,     fGBC2SScale
        MADDw fSPos, fGBC2STrans, fGBCPos[w]
        // Perspective divide XYZ and STs.
        DIV      Q,     VF00[w], fSPos[w]
        MULq.xyz fSPos, fSPos,   Q
        MULq.xyz fSTQ,  fInSTQ,  Q
        // Light the vertex.
        LightEnvironment fOPos, fNrm, fRGBA, iLightQty, pLightDataBfr
        // Get clip flags and AND them with iAllClipped. If iAllClipped is
        // anything other than 0 at the end, then all verts were outside one
        // of the planes.
        FCGET iClipFlags
        IAND  iAllClipped, iAllClipped, iClipFlags
        IOR   iAnyClipped, iAnyClipped, iClipFlags
        // Store (GB) clip space vertex, unpacked ST and colour.
        SQ    fGBCPos, 0(pIntermediateOut)
        SQ.xy fInSTQ,  1(pIntermediateOut)
        SQ    fRGBA,   2(pIntermediateOut)
        // Get clipping results.
        FCAND  VI01, 0x3FFFF
        IADDIU iClipADC, VI01, 0x7FFF
        // Combine the clip ADC with the degeneracy ADC.
        MTIR iInADC,  fNrm[w]
        IOR  iOutADC, iInADC, iClipADC
        // In spare w component of intermediate STQ, store sum of iInADC and
        // iClipADC. This will be 0x8000 iff the vertex was rejected here
        // but is not degenerate.
        IADD  iSum, iInADC, iClipADC
        ISW.w iSum, 1(pIntermediateOut)
        // Convert output data.
        FTOI4.xyz fSPos, fSPos
        FTOI0     fRGBA, fRGBA
        // Store.
        SQ.xyz fSTQ,    0(pOut)
        SQ     fRGBA,   1(pOut)
        SQ.xyz fSPos,   2(pOut)
        ISW.w  iOutADC, 2(pOut)
        // Update ptrs.
        IADDIU pIn,              pIn,              4
        IADDIU pOut,             pOut,             3
        IADDIU pIntermediateOut, pIntermediateOut, 3
        // Done yet?
        IBNE pIn, pInEnd, VU1_BOTANICALS_CLIP_LIT_POINT_LOOP

    // Retrieve registers from stack.
    ILW.x pThisOutputBuffer, ASURA_PS2_BOTANICALS__ADDR__VI_STACK_3(VI00)
    ILW.y iVertexQty,        ASURA_PS2_BOTANICALS__ADDR__VI_STACK_3(VI00)
    ILW.z iMeshFlags,        ASURA_PS2_BOTANICALS__ADDR__VI_STACK_3(VI00)

VU1_BOTANICALS_CLIP_POST_T_AND_L:
    
    // Check if entire packet was outside one of the clip planes. If so, just
    // move on to the next camera.
    IBNE iAllClipped, VI00, VU1_BOTANICALS_FINISHED_FOR_CUR_CAMERA
    // Ok, render all the passes for the tris that don't require scissoring.
    IADDIU pOverwriteSTs, VI00, ASURA_PS2_BOTANICALS__ADDR__ST_OVERWRITE_BFR
    BAL pRenderAllPassesRA, VU1_BOTANICALS_RENDER_ALL_PASSES
    
    // Any tris that do require scissoring?
    IBNE iAnyClipped, VI00, VU1_BOTANICALS_CLIP_TRIANGLES

VU1_BOTANICALS_FINISHED_FOR_CUR_CAMERA:

    // Mask top two bits off cam flags so that next time we run the
    // VU1_BOTANICALS_FIND_NEXT_CAM_LOOP we'll be forced to move ptrs on, etc.
    ILW.x iRemainingCamFlags, ASURA_PS2_BOTANICALS__ADDR__VI_STACK_4(VI00)
    IADDIU iMask, VI00, 0x3F
    IAND iRemainingCamFlags, iRemainingCamFlags, iMask
    // And on to the next camera.
    B VU1_BOTANICALS_NEXT_CAMERA

VU1_BOTANICALS_FINISHED_FOR_ALL_CAMERAS:

    ILW.w pRSInputBfr,   ASURA_PS2_BOTANICALS__ADDR__VI_STACK_3(VI00)

    B VU1_BOTANICALS_END_STRIP

VU1_BOTANICALS_RENDER_ALL_PASSES:

VU1_BOTANICALS_RENDER_PASS_0:

    // Retrieve flag and ptrs from stack.
    ILW.w iNeedToSendRS, ASURA_PS2_BOTANICALS__ADDR__VI_STACK_4(VI00)
    ILW.w pRSInputBfr,   ASURA_PS2_BOTANICALS__ADDR__VI_STACK_3(VI00)
    // Reset need-to-send-RS flag for next time. For the default value, use
    // the can-more-than-camera-see-this-mesh flag value; we'll need to send
    // RS every time if there is more than one camera involved.
    ILW.x iMoreThanOneCamFlag, ASURA_PS2_BOTANICALS__ADDR__VI_STACK_5(VI00)
    ISW.w iMoreThanOneCamFlag, ASURA_PS2_BOTANICALS__ADDR__VI_STACK_4(VI00)
    // Setup RS output ptr. May not need it, of course.
    IADDIU pRSOutputBfr, VI00, ASURA_PS2_BOTANICALS__ADDR__RS_OUTPUT_BFR
    // Is previous renderstate still ok?
    IBEQ iNeedToSendRS, VI00, VU1_BOTANICALS_RENDER_PASS_0_KICK

VU1_BOTANICALS_RENDER_PASS_0_RS:

    ILW.w pCurConstCamData, ASURA_PS2_BOTANICALS__ADDR__VI_STACK_2(VI00)
    // Build base pass camera and renderstate setup list.
    LQ fSetupGIFTag, (ASURA_PS2_BOTANICALS__ADDR__GIF_TAGS + ASURA_PS2_BOTANICALS__OFFSET__GIF_TAGS_GEOM_SETUP)(VI00)
    LQ fSCISSOR,     ASURA_PS2_BOTANICALS__OFFSET__CONST_CAM_SCISSOR(pCurConstCamData)
    LQ fXYOFFSET,    ASURA_PS2_BOTANICALS__OFFSET__CONST_CAM_XYOFFSET(pCurConstCamData)
    LQ fALPHA,       ASURA_PS2_BOTANICALS__OFFSET__RS_IN_BASE_ALPHA(pRSInputBfr)
    LQ fTEX1,        ASURA_PS2_BOTANICALS__OFFSET__RS_IN_BASE_TEX1(pRSInputBfr)
    LQ fTEX0,        ASURA_PS2_BOTANICALS__OFFSET__RS_IN_BASE_TEX0(pRSInputBfr)
    LQ fMIPTBP1,     ASURA_PS2_BOTANICALS__OFFSET__RS_IN_BASE_MIPTBP1(pRSInputBfr)
    LQ fTEST,        ASURA_PS2_BOTANICALS__OFFSET__RS_IN_BASE_TEST(pRSInputBfr)
    SQ fSetupGIFTag, ASURA_PS2_BOTANICALS__OFFSET__RS_OUT_0_SETUP_GIF_TAG(pRSOutputBfr)
    SQ fSCISSOR,     ASURA_PS2_BOTANICALS__OFFSET__RS_OUT_0_SCISSOR(pRSOutputBfr)
    SQ fXYOFFSET,    ASURA_PS2_BOTANICALS__OFFSET__RS_OUT_0_XYOFFSET(pRSOutputBfr)
    SQ fALPHA,       ASURA_PS2_BOTANICALS__OFFSET__RS_OUT_0_ALPHA(pRSOutputBfr)
    SQ fTEX1,        ASURA_PS2_BOTANICALS__OFFSET__RS_OUT_0_TEX1(pRSOutputBfr)
    SQ fMIPTBP1,     ASURA_PS2_BOTANICALS__OFFSET__RS_OUT_0_MIPTBP1(pRSOutputBfr)
    SQ fTEX0,        ASURA_PS2_BOTANICALS__OFFSET__RS_OUT_0_TEX0(pRSOutputBfr)
    SQ fTEST,        ASURA_PS2_BOTANICALS__OFFSET__RS_OUT_0_TEST(pRSOutputBfr)
    // And send it.
    XGKICK pRSOutputBfr

VU1_BOTANICALS_RENDER_PASS_0_KICK:

    // Copy draw GIF tag to output buffer. Note that we assume NLOOP is
    // correct for the base pass setup requirements and that EOP is set.
    LQ.yzw fDrawGIFTag, (ASURA_PS2_BOTANICALS__ADDR__GIF_TAGS + ASURA_PS2_BOTANICALS__OFFSET__GIF_TAGS_GEOM_DRAW)(VI00)
    SQ.yzw fDrawGIFTag, ASURA_PS2_BOTANICALS__OFFSET__GEOM_OUT_DRAW_GIF_TAG(pThisOutputBuffer)
    // Swap output buffers.
    IADDIU pOtherOutputBuffer, pThisOutputBuffer, 0
    SwitchBfr_Constants pThisOutputBuffer, ASURA_PS2_BOTANICALS__ADDR__GEOM_OUTPUT_BFR_0, ASURA_PS2_BOTANICALS__ADDR__GEOM_OUTPUT_BFR_1
    // Determine whether or not we have further passes to do.
    IADDIU iMask, VI00, ASURA_PS2_BOTANICALS__MASK__MULTIPASS
    IAND iMultipass, iMeshFlags, iMask
    // And send base pass.
    XGKICK pOtherOutputBuffer
    // Go off and do other passes if necessary.
    IBNE iMultipass, VI00, VU1_BOTANICALS_MULTIPASS_REQUIRED
    // Ok, no other passes so we're done.
    JR pRenderAllPassesRA

VU1_BOTANICALS_MULTIPASS_REQUIRED:

    // Copy draw GIF tag to new output buffer. We'll then have it ready it
    // in both buffers, regardless of how many more passes we do.
    LQ fDrawGIFTag, ASURA_PS2_BOTANICALS__OFFSET__GEOM_OUT_DRAW_GIF_TAG(pOtherOutputBuffer)
    SQ fDrawGIFTag, ASURA_PS2_BOTANICALS__OFFSET__GEOM_OUT_DRAW_GIF_TAG(pThisOutputBuffer)
    // Put 0x8000 in a register. We'll need it for setting up further passes'
    // GIF tags.
    IADDIU i0x7FFF, VI00,    0x7FFF
    IADDIU i0x8000, i0x7FFF, 0x0001
    // Copy NLOOP+EOP from draw GIF tag to integer register and mask off EOP,
    // to get vertex count.
    MTIR iVertexCount, fDrawGIFTag[x]
    IAND iVertexCount, iVertexCount, i0x7FFF
    // Set the need-to-send-RS flag ready for base pass next time, since
    // we're modifying state for these extra passes. Writing any non-zero
    // value will do.
    ISW.w i0x7FFF, ASURA_PS2_BOTANICALS__ADDR__VI_STACK_4(VI00)

VU1_BOTANICALS_RENDER_PASS_1:

    // Check if we need the self illuminated pass. If not then we must need
    // pass 3 since otherwise none of the multipass flags would have been on.
    IADDIU iMask, VI00, ASURA_PS2_BOTANICALS__FLAG__SELFILLUMINATED
    IAND iSelfIllum, iMeshFlags, iMask
    IBEQ iSelfIllum, VI00, VU1_BOTANICALS_RENDER_PASS_2
    // Build the self-illuminated pass setup list.
    LQ fALPHA, ASURA_PS2_BOTANICALS__OFFSET__RS_IN_SELF_ILLUM_ALPHA(pRSInputBfr)
    SQ fALPHA, ASURA_PS2_BOTANICALS__OFFSET__RS_OUT_1_ALPHA(pRSOutputBfr)
    // Fix up NLOOP and EOP.
    IADDIU iNLOOPandEOP, i0x8000, ASURA_PS2_BOTANICALS__RS_OUT_1_NLOOP
    ISW.x iNLOOPandEOP, ASURA_PS2_BOTANICALS__OFFSET__RS_OUT_1_SETUP_GIF_TAG(pRSOutputBfr)
    // Setup input and output ptrs.
    IADDIU pCopyIn,  pOtherOutputBuffer, ASURA_PS2_BOTANICALS__OFFSET__GEOM_OUT_FIRST_VERTEX
    IADDIU pCopyOut, pThisOutputBuffer,  ASURA_PS2_BOTANICALS__OFFSET__GEOM_OUT_FIRST_VERTEX
    // Setup end-of-input ptr.
    IADD pCopyInEnd, pCopyIn,    iVertexCount
    IADD pCopyInEnd, pCopyInEnd, iVertexCount
    IADD pCopyInEnd, pCopyInEnd, iVertexCount
    // Setup fullbright colour. We'll grab the alpha from the first pass
    // vertices.
    LOI 128.0
    MAXi.xyz fRGBA, VF00, I
    FTOI0.xyz fRGBA, fRGBA

    VU1_BOTANICALS_PASS_1_LOOP:

        --LoopCS  3, 0
        --LoopAbs 7

        // Get data from other buffer.
        LQI.xyz fSTQ,     (pCopyIn++)
        LQI.w   fRGBA,    (pCopyIn++)
        LQI     fScrnPos, (pCopyIn++)
        // Copy to new buffer and insert RGBA.
        SQI.xyz fSTQ,     (pCopyOut++)
        SQI     fRGBA,    (pCopyOut++)
        SQI     fScrnPos, (pCopyOut++)
        // Done yet?
        IBNE pCopyIn, pCopyInEnd, VU1_BOTANICALS_PASS_1_LOOP

    // Kick RS.
    XGKICK pRSOutputBfr
    // Swap output buffers.
    IADDIU pOtherOutputBuffer, pThisOutputBuffer, 0
    SwitchBfr_Constants pThisOutputBuffer, ASURA_PS2_BOTANICALS__ADDR__GEOM_OUTPUT_BFR_0, ASURA_PS2_BOTANICALS__ADDR__GEOM_OUTPUT_BFR_1
    // Determine whether or not we have further passes to do.
    IADDIU iMask, VI00, ASURA_PS2_BOTANICALS__MASK__PASS_2
    IAND iNeedPass2, iMeshFlags, iMask
    // Kick geometry.
    XGKICK pOtherOutputBuffer
    // Go off and do other passes if necessary.
    IBNE iNeedPass2, VI00, VU1_BOTANICALS_RENDER_PASS_2
    // Ok, no other passes so we're done.
    JR pRenderAllPassesRA

VU1_BOTANICALS_RENDER_PASS_2:

    // Build the ST overwrite pass setup list.
    LQ fALPHA,    ASURA_PS2_BOTANICALS__OFFSET__RS_IN_ST_OV_ALPHA(pRSInputBfr)
    LQ fTEX1,     ASURA_PS2_BOTANICALS__OFFSET__RS_IN_ST_OV_TEX1(pRSInputBfr)
    LQ fTEX0,     ASURA_PS2_BOTANICALS__OFFSET__RS_IN_ST_OV_TEX0(pRSInputBfr)
    LQ fMIPTBP1,  ASURA_PS2_BOTANICALS__OFFSET__RS_IN_ST_OV_MIPTBP1(pRSInputBfr)
    SQ fALPHA,    ASURA_PS2_BOTANICALS__OFFSET__RS_OUT_2_ALPHA(pRSOutputBfr)
    SQ fTEX1,     ASURA_PS2_BOTANICALS__OFFSET__RS_OUT_2_TEX1(pRSOutputBfr)
    SQ fMIPTBP1,  ASURA_PS2_BOTANICALS__OFFSET__RS_OUT_2_MIPTBP1(pRSOutputBfr)
    SQ fTEX0,     ASURA_PS2_BOTANICALS__OFFSET__RS_OUT_2_TEX0(pRSOutputBfr)
    // Fix up NLOOP and EOP.
    IADDIU iNLOOPandEOP, i0x8000, ASURA_PS2_BOTANICALS__RS_OUT_2_NLOOP
    ISW.x iNLOOPandEOP, ASURA_PS2_BOTANICALS__OFFSET__RS_OUT_2_SETUP_GIF_TAG(pRSOutputBfr)
    // Setup input / output ptrs.
    IADDIU pCopyIn,  pOtherOutputBuffer, ASURA_PS2_BOTANICALS__OFFSET__GEOM_OUT_FIRST_VERTEX
    IADDIU pCopyOut, pThisOutputBuffer,  ASURA_PS2_BOTANICALS__OFFSET__GEOM_OUT_FIRST_VERTEX
    // Setup end-of-input ptr.
    IADD pOverwriteSTEnd, pOverwriteSTs, iVertexCount

    VU1_BOTANICALS_PASS_2_LOOP:

        // Note: This loop works for the second pass of dual texture
        // materials because we fiddle with the first pass's ALPHA setting
        // to ignore the vertex alpha. However, it'll still be there in the
        // buffer, ready for use here with an ALPHA that pays attention to
        // it.

        --LoopCS  3, 0
        --LoopAbs 9

        // Load data from other buffer and ST buffer.
        LQI.z   fSTQ,     (pCopyIn++)
        LQI.xy  fSTQ,     (pOverwriteSTs++)
        LQI     fRGBA,    (pCopyIn++)
        LQI     fScrnPos, (pCopyIn++)
        // Perspective divide texture coordinates.
        MULz.xy fSTQ, fSTQ, fSTQ[z]
        // Copy to new output buffer.
        SQI.xyz fSTQ,     (pCopyOut++)
        SQI     fRGBA,    (pCopyOut++)
        SQI     fScrnPos, (pCopyOut++)
        // Done yet?
        IBNE pOverwriteSTs, pOverwriteSTEnd, VU1_BOTANICALS_PASS_2_LOOP

    // Kick RS.
    XGKICK pRSOutputBfr
    // Swap output buffers.
    IADDIU pOtherOutputBuffer, pThisOutputBuffer, 0
    SwitchBfr_Constants pThisOutputBuffer, ASURA_PS2_BOTANICALS__ADDR__GEOM_OUTPUT_BFR_0, ASURA_PS2_BOTANICALS__ADDR__GEOM_OUTPUT_BFR_1
    // Kick geometry.
    XGKICK pOtherOutputBuffer
    // And we're done.
    JR pRenderAllPassesRA

VU1_BOTANICALS_RENDER_TO_SHADOW_BFR:

    // Load shadow projection matrix.
    ILW.z pShadowDataBfr, ASURA_PS2_BOTANICALS__ADDR__VI_STACK_4(VI00)   
    MatrixLoad mShadowProjMat, ASURA_PS2_BOTANICALS__OFFSET__SHADOW_PROJ_MAT, pShadowDataBfr
    // Retrieve vertex count. (Doing this again here helps with register
    // pressure.)
    ILW.y iVertexQty,		 ASURA_PS2_BOTANICALS__ADDR__VI_STACK_6(VI00)
    // Prepare input/output ptrs.
    XTOP pIn
    IADDIU pOut, pThisOutputBuffer, ASURA_PS2_BOTANICALS__OFFSET__SHADOW_OUT_FIRST_VTX
    // And an end-of-output ptr. (Output is easier to calculate since it's
    // only 1 QW per vertex.)
    IADD pOutEnd, pOut, iVertexQty

    // Deal with the vertices 3 at a time. This allows us to amortise some of
    // the latencies involved in the 1/w operations. The output buffer has
    // plenty of space for this since we're only writing 1 QW per vertex, as
    // opposed to 3 in the other render loops. This brought the cost per
    // vertex down to 8, from 10.
    VU1_BOTANICALS_RENDER_TO_SHADOW_BFR_VTX_LOOP:

        --LoopCS  1, 0
        --LoopAbs 24

        // Load input data.
        LQ.xyz fOPos0, (0*4 + 0)(pIn)
        LQ.w   fNrm0,  (0*4 + 3)(pIn)
        LQ.xyz fOPos1, (1*4 + 0)(pIn)
        LQ.w   fNrm1,  (1*4 + 3)(pIn)
        LQ.xyz fOPos2, (2*4 + 0)(pIn)
        LQ.w   fNrm2,  (2*4 + 3)(pIn)
        // Convert ADC bits.
        UnpackADC fSPos0, fNrm0, fUnpackScale
        UnpackADC fSPos1, fNrm1, fUnpackScale
        UnpackADC fSPos2, fNrm2, fUnpackScale
        // Transform each.
        MatrixMultiplyVertexW1 fSPosPreDiv0, mShadowProjMat, fOPos0
        MatrixMultiplyVertexW1 fSPosPreDiv1, mShadowProjMat, fOPos1
        MatrixMultiplyVertexW1 fSPosPreDiv2, mShadowProjMat, fOPos2
        // Calculate 1/w and perform perspective divide. Use EFU for 1 and
        // FDIV for the other.
        DIV Q, VF00[w], fSPosPreDiv0[w]
        MULq.xyz fSPosPreDiv0, fSPosPreDiv0, Q
        ERCPR P, fSPosPreDiv1[w]
        MFP.w fOneOverSPos1W, P
        MULw.xyz fSPosPreDiv1, fSPosPreDiv1, fOneOverSPos1W[w]
        DIV Q, VF00[w], fSPosPreDiv2[w]
        MULq.xyz fSPosPreDiv2, fSPosPreDiv2, Q
        // Convert positions for GS.
        FTOI4.xyz fSPos0, fSPosPreDiv0
        FTOI4.xyz fSPos1, fSPosPreDiv1
        FTOI4.xyz fSPos2, fSPosPreDiv2
        // Output.
        SQ fSPos0, 0(pOut)
        SQ fSPos1, 1(pOut)
        SQ fSPos2, 2(pOut)
        // Update loop state.
        IADDIU pIn,  pIn,  (3*4)
        IADDIU pOut, pOut, (3*1)
        // Done yet?
        ISUB iRemainder, pOutEnd, pOut
        IBGTZ iRemainder, VU1_BOTANICALS_RENDER_TO_SHADOW_BFR_VTX_LOOP

    // Load GIF tags and setup QWs.
    LQ fSetupGIFTag, (ASURA_PS2_BOTANICALS__ADDR__GIF_TAGS + ASURA_PS2_BOTANICALS__OFFSET__GIF_TAGS_SHADOW_SETUP)(VI00)
    LQ fDrawGIFTag,  (ASURA_PS2_BOTANICALS__ADDR__GIF_TAGS + ASURA_PS2_BOTANICALS__OFFSET__GIF_TAGS_SHADOW_DRAW)(VI00)
    LQ fRGBAQ,       (ASURA_PS2_BOTANICALS__ADDR__CONST_SHADOW_DATA + ASURA_PS2_BOTANICALS__OFFSET__CONST_SHADOW_RGBAQ)(VI00)
    // Copy to output.
    SQ fSetupGIFTag, ASURA_PS2_BOTANICALS__OFFSET__SHADOW_OUT_SETUP_GIF_TAG(pThisOutputBuffer)
    SQ fRGBAQ, ASURA_PS2_BOTANICALS__OFFSET__SHADOW_OUT_RGBAQ(pThisOutputBuffer)
    SQ fDrawGIFTag, ASURA_PS2_BOTANICALS__OFFSET__SHADOW_OUT_DRAW_GIF_TAG(pThisOutputBuffer)
    // Setup NLOOP (and EOP) in draw GIF tag.
    IADDIU iNLOOPandEOP, VI00, 0x7FFF
    IADDIU iNLOOPandEOP, iNLOOPandEOP, 0x1
    IOR iNLOOPandEOP, iNLOOPandEOP, iVertexQty
    ISW.x iNLOOPandEOP, ASURA_PS2_BOTANICALS__OFFSET__SHADOW_OUT_DRAW_GIF_TAG(pThisOutputBuffer)
    // Ok, send that to GS.
    XGKICK pThisOutputBuffer
    // Switch output buffers.
    SwitchBfr_Constants pThisOutputBuffer, ASURA_PS2_BOTANICALS__ADDR__GEOM_OUTPUT_BFR_0, ASURA_PS2_BOTANICALS__ADDR__GEOM_OUTPUT_BFR_1
    // And we're done.
    B VU1_BOTANICALS_END_STRIP

VU1_BOTANICALS_CLIP_FINISHED:

    // Check if there's anything in the output buffer that needs rendering.
    // We could just kick the packet anyway but we don't want to do all the
    // multipass stuff if there's nothing in the buffer.
    ILW.x iNLOOPandEOP, ASURA_PS2_BOTANICALS__OFFSET__GEOM_OUT_DRAW_GIF_TAG(pThisOutputBuffer)
    IADDIU iMask, VI00, 0x7FFF
    IAND iNumVerticesInBuffer, iNLOOPandEOP, iMask
    IBEQ iNumVerticesInBuffer, VI00, VU1_BOTANICALS_CLIP_FINISHED_POST_RENDER
    // We have something to render.
    IADDIU pOverwriteSTs, pThisOutputBuffer, ASURA_PS2_BOTANICALS__OFFSET__CLIPPED_STS
    BAL pRenderAllPassesRA, VU1_BOTANICALS_RENDER_ALL_PASSES

VU1_BOTANICALS_CLIP_FINISHED_POST_RENDER:

    // And we're done.
    B VU1_BOTANICALS_FINISHED_FOR_CUR_CAMERA

VU1_BOTANICALS_CLIP_TRIANGLES:

    // Clipper input data format:
    //
    //        x    y    z    w                  -- in GB clip space
    //        s    t    X    should clip flag
    //        r    g    b    a                  -- floating point in range [0.0, 255.0]
    //
    // where X is "don't care".

    // Load GBC->SC scale vector.
    ILW.w pCurConstCamData, ASURA_PS2_BOTANICALS__ADDR__VI_STACK_2(VI00)
    LQ fGBC2SCScale, ASURA_PS2_BOTANICALS__OFFSET__CONST_CAM_GBC2SC_SCALE(pCurConstCamData)
    // Setup output pointers.
    IADDIU pOut,   pThisOutputBuffer, ASURA_PS2_BOTANICALS__OFFSET__GEOM_OUT_FIRST_VERTEX
    IADDIU pSTOut, pThisOutputBuffer, ASURA_PS2_BOTANICALS__OFFSET__CLIPPED_STS
    // Push onto stack.
    ISW.x pOut,   ASURA_PS2_BOTANICALS__ADDR__VI_STACK_1(VI00)
    ISW.y pSTOut, ASURA_PS2_BOTANICALS__ADDR__VI_STACK_1(VI00)
    // Setup some stuff for loop.
    IADDIU pIn,        VI00,      ASURA_PS2_BOTANICALS__ADDR__CLIP_INPUT_BFR
    IADD   pInEnd,     pIn,       iVertexQty
    IADD   pInEnd,     pInEnd,    iVertexQty
    IADD   pInEnd,     pInEnd,    iVertexQty
    IADDIU iClipMask,  VI00,      0x3F
    IADDIU i0x8000,    iClipMask, (0x8000 - 0x3F)
    // Write a GIF tag to new output buffer (with EOP=1 and NLOOP=0.)
    ISW.x i0x8000, ASURA_PS2_BOTANICALS__OFFSET__GEOM_OUT_DRAW_GIF_TAG(pThisOutputBuffer)

#define ASSUME_NOT_MANY_TRIS_WILL_NEED_CLIPPING (0)

#if ASSUME_NOT_MANY_TRIS_WILL_NEED_CLIPPING

    // Look at first two vertices.
    ILW.w iShouldClipFlag2, 1(pIn)
    ILW.w iShouldClipFlag1, 4(pIn)
    IOR iShouldClipFlag21, iShouldClipFlag2, iShouldClipFlag1
    // Skip over them.
    IADDIU pIn, pIn, 6
    IADDIU pInExtraST, VI00, (ASURA_PS2_BOTANICALS__ADDR__ST_OVERWRITE_BFR + 2)

    VU1_BOTANICALS_FIND_CLIP_TRIANGLE_LOOP:

        --LoopCS 1, 0

        // Are we done yet?
        IBEQ pIn, pInEnd, VU1_BOTANICALS_CLIP_FINISHED
        // Load should-clip flag for current vtx.
        ILW.w iShouldClipFlag0, 1(pIn)
        // Update loop state.
        IADDIU pIn, pIn, 3
        IADDIU pInExtraST, pInExtraST, 1
        // Calculate should-clip flag for current tri.
        IOR iShouldClipTri, iShouldClipFlag21, iShouldClipFlag0
        // Move old flags along.
        IOR iShouldClipFlag21, iShouldClipFlag1, iShouldClipFlag0
        IADDIU iShouldClipFlag1, iShouldClipFlag0, 0
        // Continue to next vertex if none of the last three may need
        // clipping.
        IBEQ iShouldClipTri, VI00, VU1_BOTANICALS_FIND_CLIP_TRIANGLE_LOOP

    // Current tri may need clipping. Generate last three vertices' clip
    // results.
    LQ fGBCPos0, -9(pIn)
    MUL fSCPos0, fGBCPos0, fGBC2SCScale
    CLIPw.xyz fSCPos0, fSCPos0
    FCGET iClipFlags0
    LQ fGBCPos1, -6(pIn)
    MUL fSCPos1, fGBCPos1, fGBC2SCScale
    CLIPw.xyz fSCPos1, fSCPos1
    FCGET iClipFlags1
    LQ fGBCPos2, -3(pIn)
    MUL fSCPos2, fGBCPos2, fGBC2SCScale
    CLIPw.xyz fSCPos2, fSCPos2
    FCGET iClipFlags2
    // AND them together.
    IAND iClipFlags01, iClipFlags0, iClipFlags1
    IAND iClipFlagsAll, iClipFlags01, iClipFlags2
    IAND iClipFlagsAll, iClipFlagsAll, iClipMask
    // If any bit is high, all three vertices are outside one plane so we
    // can just reject this tri.
    IBNE iClipFlagsAll, VI00, VU1_BOTANICALS_FIND_CLIP_TRIANGLE_LOOP

#else

    // Get clipping results for vertex (n - 2) - against guardband. (used to be against screen)
    LQ        fClipPos, 0(pIn)
//    MUL       fClipPos, fClipPos, fGBC2SCScale // RCD - changed to test against guard band instead of screen
    CLIPw.xyz fClipPos, fClipPos
    FCGET     iClipFlag2
    IAND      iClipFlag2, iClipFlag2, iClipMask
    // Same for vertex (n - 1)...
    LQ        fClipPos, 3(pIn)
//    MUL       fClipPos, fClipPos, fGBC2SCScale  // RCD - changed to test against guard band instead of screen
    CLIPw.xyz fClipPos, fClipPos
    FCGET     iClipFlag1
    IAND      iClipFlag1, iClipFlag1, iClipMask
    // Skip ahead to last vertex of first tri.
    IADDIU pIn,  pIn, 6
    IADDIU pInExtraST, VI00, (ASURA_PS2_BOTANICALS__ADDR__ST_OVERWRITE_BFR + 2)
    // Calculate half of first clipping test.
    IAND iClipAll21, iClipFlag2, iClipFlag1

    VU1_BOTANICALS_FIND_CLIP_TRIANGLE_LOOP:

        --LoopCS 1, 0

        // Have we finished yet?
        IBEQ pIn, pInEnd, VU1_BOTANICALS_CLIP_FINISHED
        // Get clipping results for current vertex - but against our smaller
        // screen frustum, not the guardband one.
        LQ        fClipPos, 0(pIn)
//        MUL       fClipPos, fClipPos, fGBC2SCScale    // RCD - changed to test against guard band instead of screen
        CLIPw.xyz fClipPos, fClipPos

        // Load should-clip flag early, to try and encourage VCL to schedule
        // it in some of the latencies above.
        ILW.w iShouldClipFlag, 1(pIn)

        // Grab clip result for current vertex.
        FCGET iClipFlag0

#if 0 // poorly optimised old code
        IAND  iClipFlag0, iClipFlag0, iClipMask
        // Calculate second half of clipping test.
        IAND iClipAll10, iClipFlag1, iClipFlag0
        IAND iClipAll,   iClipAll21, iClipAll10
        // Move clipping flags/result along 1.
        IADD iClipFlag1, VI00, iClipFlag0
        IADD iClipAll21, VI00, iClipAll10
#else // new improved fast code
        // Calculate second half of clipping test.
        IAND iClipAll,   iClipAll21, iClipFlag0
        // Move clipping flags/result along 1.
        IAND iClipAll21, iClipFlag1, iClipFlag0
        IAND  iClipFlag1, iClipFlag0, iClipMask
#endif

        // Update pointers.
        IADDIU pIn,        pIn,        3
        IADDIU pInExtraST, pInExtraST, 1
        // Don't bother clipping if vertex was degenerate or if we accepted it
        // in the main render loop. We have to check the latter since we're
        // using a smaller frustum here and may end up scissoring tris that
        // actually fit inside the larger one.
        IBNE iShouldClipFlag, i0x8000, VU1_BOTANICALS_FIND_CLIP_TRIANGLE_LOOP  
        // If the triangle is completely outside at least one plane then
        // just reject.
        IBNE iClipAll, VI00, VU1_BOTANICALS_FIND_CLIP_TRIANGLE_LOOP
        // At this point, we know the triangle is not completely inside all
        // planes since otherwise it would have been accepted in the main
        // render loop. We now also know it's not entirely outside one plane;
        // hence, it must cross one and require scissoring.

#endif

    // Ok, this triangle needs clipping. Push some int registers onto stack.
    ISW.x pIn,           ASURA_PS2_BOTANICALS__ADDR__VI_STACK_0(VI00)
    ISW.y pInEnd,        ASURA_PS2_BOTANICALS__ADDR__VI_STACK_0(VI00)
#if ASSUME_NOT_MANY_TRIS_WILL_NEED_CLIPPING
    ISW.z iShouldClipFlag21, ASURA_PS2_BOTANICALS__ADDR__VI_STACK_0(VI00)
    ISW.w iShouldClipFlag1,  ASURA_PS2_BOTANICALS__ADDR__VI_STACK_0(VI00)
#else
    ISW.z iClipFlag1,    ASURA_PS2_BOTANICALS__ADDR__VI_STACK_0(VI00)
    ISW.w iClipAll21,    ASURA_PS2_BOTANICALS__ADDR__VI_STACK_0(VI00)
#endif
    ISW.z pInExtraST,    ASURA_PS2_BOTANICALS__ADDR__VI_STACK_1(VI00)
    // Setup buffers and initial vtx count.
    IADDIU pClipSrc,     pThisOutputBuffer, ASURA_PS2_BOTANICALS__OFFSET__CLIP_WORKSPACE
    IADDIU pClipDst,     pClipSrc,          ASURA_PS2_BOTANICALS__SIZE__CLIP_WORKSPACE
    IADDIU iSrcPointQty, VI00,              3
    // Copy the source vertices into the first clip workspace. We have to do
    // this because we have split input data
    LQ    fClipPos, -9(pIn)
    LQ.xy fSTQ,     -8(pIn)
    LQ    fRGBA,    -7(pIn)
    LQ.xy fExSTQ,   -3(pInExtraST)
    SQ    fRGBA,     0(pClipSrc)
    SQ    fClipPos,  1(pClipSrc)
    SQ.xy fExSTQ,    2(pClipSrc)
    SQ.xy fSTQ,      3(pClipSrc)
    LQ    fClipPos, -6(pIn)
    LQ.xy fSTQ,     -5(pIn)
    LQ    fRGBA,    -4(pIn)
    LQ.xy fExSTQ,   -2(pInExtraST)
    SQ    fRGBA,     4(pClipSrc)
    SQ    fClipPos,  5(pClipSrc)
    SQ.xy fExSTQ,    6(pClipSrc)
    SQ.xy fSTQ,      7(pClipSrc)
    LQ    fClipPos, -3(pIn)
    LQ.xy fSTQ,     -2(pIn)
    LQ    fRGBA,    -1(pIn)
    LQ.xy fExSTQ,   -1(pInExtraST)
    SQ    fRGBA,     8(pClipSrc)
    SQ    fClipPos,  9(pClipSrc)
    SQ.xy fExSTQ,   10(pClipSrc)
    SQ.xy fSTQ,     11(pClipSrc)
#if NORMALISE_CLIPPED_STS
    // Initialise min and max STs.
    LOI 0x49742400 // Arbitrary large float.
    ADDi.xy fMinSTs, VF00, I
    SUBi.xy fMaxSTs, VF00, I
#endif

    // Special case for far plane: if any vertex is on or outside it, just
    // cull instead of clipping. With sensible poly sizes and a reasonably
    // distant far clip, this shouldn't be noticeable.
    FCAND VI01, MAKE_PLANE_MASK_FOR_LAST_3_VERTS(ASURA_PS2_BOTANICALS__CLIP_FAR)
    IBNE VI01, VI00, VU1_BOTANICALS_CLIP_FINISHED_THIS_TRIANGLE
    
    // See which planes we need to clip against. We don't want to bother
    // clipping against a plane unless at least one vertex from the source
    // tri lies on or outside it.
    // -z
    FCAND VI01, MAKE_PLANE_MASK_FOR_LAST_3_VERTS(ASURA_PS2_BOTANICALS__CLIP_NEAR)
    IADD iPlaneClipFlags, VI00, VI01
    // +z
    // Just shift; we already dealt with this case above.
    IADD iPlaneClipFlags, iPlaneClipFlags, iPlaneClipFlags
    // -y
    IADD iPlaneClipFlags, iPlaneClipFlags, iPlaneClipFlags
    FCAND VI01, MAKE_PLANE_MASK_FOR_LAST_3_VERTS(ASURA_PS2_BOTANICALS__CLIP_TOP)    //       "
    IADD iPlaneClipFlags, iPlaneClipFlags, VI01
    // +y
    IADD iPlaneClipFlags, iPlaneClipFlags, iPlaneClipFlags
    FCAND VI01, MAKE_PLANE_MASK_FOR_LAST_3_VERTS(ASURA_PS2_BOTANICALS__CLIP_BOTTOM) //       "
    IADD iPlaneClipFlags, iPlaneClipFlags, VI01
    // -x
    IADD iPlaneClipFlags, iPlaneClipFlags, iPlaneClipFlags
    FCAND VI01, MAKE_PLANE_MASK_FOR_LAST_3_VERTS(ASURA_PS2_BOTANICALS__CLIP_LEFT)   //       "
    IADD iPlaneClipFlags, iPlaneClipFlags, VI01
    // +x
    IADD iPlaneClipFlags, iPlaneClipFlags, iPlaneClipFlags
    FCAND VI01, MAKE_PLANE_MASK_FOR_LAST_3_VERTS(ASURA_PS2_BOTANICALS__CLIP_RIGHT)  //       "
    IADD iPlaneClipFlags, iPlaneClipFlags, VI01

VU1_BOTANICALS_CLIP_NEAR_PLANE: // -z

    // Setup clip mask.
    IADDIU iCurrentClipMask, VI00, ASURA_PS2_BOTANICALS__CLIP_NEAR
    // Skip if no vertices are on or outside this plane.
    IAND iAnyOnOrOutside, iPlaneClipFlags, iCurrentClipMask
    IBEQ iAnyOnOrOutside, VI00, VU1_BOTANICALS_CLIP_FAR_PLANE
    // Clear output vertex count and set plane sign to negative.
    SUBw.xw fPlaneSign, VF00, VF00
    // Clip against this plane.
    IADDIU iInterpolate, VI00, INTERPOLATE_Z
    BAL pClipPointsRA, VU1_BOTANICALS_CLIP_POINTS
    // Switch buffers.
    Swap pClipSrc, pClipDst
    // Extract output vertex count.
    FTOI0.w fPlaneSign, fPlaneSign
    MTIR iSrcPointQty, fPlaneSign[w]
    // Check tri hasn't been entirely clipped out. While we try to reject
    // such cases in the find-clip-tri loop above, I think numerical
    // inaccuracies can lead to it still happening.
    IBEQ iSrcPointQty, VI00, VU1_BOTANICALS_CLIP_FINISHED_THIS_TRIANGLE
    // If we've clipped to the near plane, force x and y clips. I think the
    // vertex being behind the near plane somehow invalidates our results
    // above. Ought to double-check the maths behind this to make sure that's
    // actually the case and that this isn't just hiding a bug somewhere.
    IADDIU iXandYClips, VI00, (ASURA_PS2_BOTANICALS__CLIP_LEFT | ASURA_PS2_BOTANICALS__CLIP_RIGHT | ASURA_PS2_BOTANICALS__CLIP_TOP | ASURA_PS2_BOTANICALS__CLIP_BOTTOM)
    IOR iPlaneClipFlags, iPlaneClipFlags, iXandYClips

VU1_BOTANICALS_CLIP_FAR_PLANE: // +z

    // Setup clip mask.
    IADDIU iCurrentClipMask, VI00, ASURA_PS2_BOTANICALS__CLIP_FAR
    // Skip if no vertices are on or outside this plane.
    IAND iAnyOnOrOutside, iPlaneClipFlags, iCurrentClipMask
    IBEQ iAnyOnOrOutside, VI00, VU1_BOTANICALS_CLIP_TOP_PLANE
    // Clear output vertex count and set plane sign to positive.
    SUBw.w fPlaneSign, VF00, VF00
    ADDw.x fPlaneSign, VF00, VF00
    // Clip against this plane.
    IADDIU iInterpolate, VI00, INTERPOLATE_Z
    BAL pClipPointsRA, VU1_BOTANICALS_CLIP_POINTS
    // Switch buffers.
    Swap pClipSrc, pClipDst
    // Extract output vertex count.
    FTOI0.w fPlaneSign, fPlaneSign
    MTIR iSrcPointQty, fPlaneSign[w]
    // Check tri hasn't been entirely clipped out. (See also comment in
    // VU1_BOTANICALS_CLIP_NEAR_PLANE.)
    IBEQ iSrcPointQty, VI00, VU1_BOTANICALS_CLIP_FINISHED_THIS_TRIANGLE

VU1_BOTANICALS_CLIP_TOP_PLANE: // -y

    // Setup clip mask.
    IADDIU iCurrentClipMask, VI00, ASURA_PS2_BOTANICALS__CLIP_TOP
    // Skip if no vertices are on or outside this plane.
    IAND iAnyOnOrOutside, iPlaneClipFlags, iCurrentClipMask
    IBEQ iAnyOnOrOutside, VI00, VU1_BOTANICALS_CLIP_BOTTOM_PLANE
    // Clear output vertex count and set plane sign to negative.
    SUBw.xw fPlaneSign, VF00, VF00
    // Clip against this plane.
    IADDIU iInterpolate, VI00, INTERPOLATE_Y
    BAL pClipPointsRA, VU1_BOTANICALS_CLIP_POINTS
    // Switch buffers.
    Swap pClipSrc, pClipDst
    // Extract output vertex count.
    FTOI0.w fPlaneSign, fPlaneSign
    MTIR iSrcPointQty, fPlaneSign[w]
    // Check tri hasn't been entirely clipped out. (See also comment in
    // VU1_BOTANICALS_CLIP_NEAR_PLANE.)
    IBEQ iSrcPointQty, VI00, VU1_BOTANICALS_CLIP_FINISHED_THIS_TRIANGLE

VU1_BOTANICALS_CLIP_BOTTOM_PLANE: // +y

    // Setup clip mask.
    IADDIU iCurrentClipMask, VI00, ASURA_PS2_BOTANICALS__CLIP_BOTTOM
    // Skip if no vertices are on or outside this plane.
    IAND iAnyOnOrOutside, iPlaneClipFlags, iCurrentClipMask
    IBEQ iAnyOnOrOutside, VI00, VU1_BOTANICALS_CLIP_LEFT_PLANE
    // Clear output vertex count and set plane sign to positive.
    SUBw.w fPlaneSign, VF00, VF00
    ADDw.x fPlaneSign, VF00, VF00
    // Clip against this plane.
    IADDIU iInterpolate, VI00, INTERPOLATE_Y
    BAL pClipPointsRA, VU1_BOTANICALS_CLIP_POINTS
    // Switch buffers.
    Swap pClipSrc, pClipDst
    // Extract output vertex count.
    FTOI0.w fPlaneSign, fPlaneSign
    MTIR iSrcPointQty, fPlaneSign[w]
    // Check tri hasn't been entirely clipped out. (See also comment in
    // VU1_BOTANICALS_CLIP_NEAR_PLANE.)
    IBEQ iSrcPointQty, VI00, VU1_BOTANICALS_CLIP_FINISHED_THIS_TRIANGLE

VU1_BOTANICALS_CLIP_LEFT_PLANE: // -x

    // Setup clip mask.
    IADDIU iCurrentClipMask, VI00, ASURA_PS2_BOTANICALS__CLIP_LEFT
    // Skip if no vertices are on or outside this plane.
    IAND iAnyOnOrOutside, iPlaneClipFlags, iCurrentClipMask
    IBEQ iAnyOnOrOutside, VI00, VU1_BOTANICALS_CLIP_RIGHT_PLANE
    // Clear output vertex count and set plane sign to negative.
    SUBw.xw fPlaneSign, VF00, VF00
    // Clip against this plane.
    IADDIU iInterpolate, VI00, INTERPOLATE_X
    BAL pClipPointsRA, VU1_BOTANICALS_CLIP_POINTS
    // Switch buffers.
    Swap pClipSrc, pClipDst
    // Extract output vertex count.
    FTOI0.w fPlaneSign, fPlaneSign
    MTIR iSrcPointQty, fPlaneSign[w]
    // Check tri hasn't been entirely clipped out. (See also comment in
    // VU1_BOTANICALS_CLIP_NEAR_PLANE.)
    IBEQ iSrcPointQty, VI00, VU1_BOTANICALS_CLIP_FINISHED_THIS_TRIANGLE

VU1_BOTANICALS_CLIP_RIGHT_PLANE: // +x

    // Setup clip mask.
    IADDIU iCurrentClipMask, VI00, ASURA_PS2_BOTANICALS__CLIP_RIGHT
    // Skip if no vertices are on or outside this plane.
    IAND iAnyOnOrOutside, iPlaneClipFlags, iCurrentClipMask
    IBEQ iAnyOnOrOutside, VI00, VU1_BOTANICALS_CLIP_FINISHED_PLANES
    // Clear output vertex count and set plane sign to positive.
    SUBw.w fPlaneSign, VF00, VF00
    ADDw.x fPlaneSign, VF00, VF00
    // Clip against this plane.
    IADDIU iInterpolate, VI00, INTERPOLATE_X
    BAL pClipPointsRA, VU1_BOTANICALS_CLIP_POINTS
    // Switch buffers.
    Swap pClipSrc, pClipDst
    // Extract output vertex count.
    FTOI0.w fPlaneSign, fPlaneSign
    MTIR iSrcPointQty, fPlaneSign[w]
    // Check tri hasn't been entirely clipped out. (See also comment in
    // VU1_BOTANICALS_CLIP_NEAR_PLANE.)
    IBEQ iSrcPointQty, VI00, VU1_BOTANICALS_CLIP_FINISHED_THIS_TRIANGLE

VU1_BOTANICALS_CLIP_FINISHED_PLANES:

    // Retrieve int registers from stack.
    ILW.x pOut,   ASURA_PS2_BOTANICALS__ADDR__VI_STACK_1(VI00)
    ILW.y pSTOut, ASURA_PS2_BOTANICALS__ADDR__VI_STACK_1(VI00)
    // Check there is enough space in the output buffer for this many
    // vertices. (Easier to check the ST overwrite pointer since that's just
    // 1QW per vertex.)
    ISUB   iTemp, pSTOut, pThisOutputBuffer
    ISUBIU iTemp, iTemp,  ASURA_PS2_BOTANICALS__OFFSET__CLIPPED_STS // space used in buffer
    IADD   iTemp, iTemp,  iSrcPointQty                       // + space required for this strip
    IADDIU iTemp, iTemp,  1                                  // + space for overrun (see comment by VU1_BOTANICALS_CLIP_OUTPUT_STRIP_LOOP below)
    ISUBIU iTemp, iTemp,  ASURA_PS2_BOTANICALS__SIZE__CLIPPED_STS   // Now < 0 if there is space for these vertices.
    IBLTZ  iTemp, VU1_BOTANICALS_CLIP_ENOUGH_SPACE_IN_OUTPUT

VU1_BOTANICALS_CLIP_NOT_ENOUGH_SPACE_IN_OUTPUT:

    // Not enough space so kick what we have and swap buffers.
    IADDIU pOverwriteSTs, pThisOutputBuffer, ASURA_PS2_BOTANICALS__OFFSET__CLIPPED_STS
    BAL pRenderAllPassesRA, VU1_BOTANICALS_RENDER_ALL_PASSES
    // Reset output pointers.
    IADDIU pOut,   pThisOutputBuffer, ASURA_PS2_BOTANICALS__OFFSET__GEOM_OUT_FIRST_VERTEX
    IADDIU pSTOut, pThisOutputBuffer, ASURA_PS2_BOTANICALS__OFFSET__CLIPPED_STS
    // Write new GIF tag to output buffer with EOP=1 and NLOOP=0.
    IADDIU iNLOOPandEOP, VI00,         0x7FFF
    IADDIU iNLOOPandEOP, iNLOOPandEOP, 1
    ISW.x  iNLOOPandEOP, ASURA_PS2_BOTANICALS__OFFSET__GEOM_OUT_DRAW_GIF_TAG(pThisOutputBuffer)
    
VU1_BOTANICALS_CLIP_ENOUGH_SPACE_IN_OUTPUT:

    // Increment NLOOP count in GIF tag.
    ILW.x    iTemp,    ASURA_PS2_BOTANICALS__OFFSET__GEOM_OUT_DRAW_GIF_TAG(pThisOutputBuffer)
    IADD    iTemp,    iTemp,    iSrcPointQty
    ISW.x    iTemp,    ASURA_PS2_BOTANICALS__OFFSET__GEOM_OUT_DRAW_GIF_TAG(pThisOutputBuffer)
    // Setup the input pointers
    IADD   pClipSrcE, iSrcPointQty, iSrcPointQty
    IADD   pClipSrcE, pClipSrcE,    pClipSrcE
    IADD   pClipSrcE, pClipSrcE,    pClipSrc
    ISUBIU pClipSrcE, pClipSrcE,    4            // Point to last vertex in fan.
    // Fix Q component of fInSTQ at 1. This will be maintained throughout
    // the render loop below.
    ADDw.z fInSTQ, VF00, VF00
    // Set degeneracy flag for first two output verts. We'll add this value
    // to itself each time; that'll give us 0xC000, 0x8000, 0x0000 (and
    // obviously 0s thereafter), which means the ADC bit is on for precisely
    // the first two vertices.
    IADDIU iADC, VI00, 0x6000
    // The interpolate alpha can be slightly out, even when the alpha values
    // it's interpolating between are both 128, so we need to add on a little
    // bit to stop alpha 127 beeing written when it should be alpha 128. This
    // is necessary due to alpha test limits, etc.
#if NORMALISE_CLIPPED_STS
    LOI 0.5
    IADDIU  iMask, VI00, ASURA_PS2_BOTANICALS__FLAG__PROJECTED_TEXTURE
    IAND    iShadowProjection, iMeshFlags, iMask

    // Calc ST normalising offset.
    ADD.xy   fNormST, fMinSTs, fMaxSTs
    MULi.xy  fNormST, fNormST, I
    FTOI0.xy fNormST, fNormST
    ITOF0.xy fNormST, fNormST
    // if its a shadow then zero the normalization factor
    IBEQ    iShadowProjection, VI00, VU1_BOTANICALS_CLIP_OUTPUT_STRIP

    SUB.xy  fNormST, VF00, VF00 // zero it
#endif

VU1_BOTANICALS_CLIP_OUTPUT_STRIP:

    VU1_BOTANICALS_CLIP_OUTPUT_STRIP_LOOP:

        --LoopCS  2, 0
        --LoopAbs 28

        // (MP - 23.10.2003) We'll always write a multiple of two vertices
        // here to save branching in the loop. This then allows VCL to
        // optimise it for us. Of course, this means we'll overrun by up to
        // one vertex so we have to account for this when checking for free
        // space above. The GIF tag's NLOOP won't count this extra vertex but
        // we'll have to fix up the output ptr after the loop.

        // Load vertex from the start.
        LQ     fRGBA,       0(pClipSrc)
        LQ     fGBCClipPos, 1(pClipSrc)
        LQ.xy  fExSTQ,      2(pClipSrc)
        LQ.xy  fInSTQ,      3(pClipSrc)
        // Scale and translate to screenspace.
        MULA  ACC,      fGBCClipPos, fGBC2SScale
        MADDw fScrnPos, fGBC2STrans, fGBCClipPos[w]
        // Calculate 1/w.
        DIV Q, VF00[w], fScrnPos[w]
#if NORMALISE_CLIPPED_STS
        // Normalise ST.
        SUB.xy fInSTQ, fInSTQ, fNormST
#endif
        // Perspective divide.
        MULq.xyz fScrnPos, fScrnPos, Q
        MULq.xyz fSTQ,     fInSTQ,   Q
        // Apply alpha bias.
        ADDi.w fRGBA, fRGBA, I
        // Convert output data.
        FTOI4.xyz fScrnPos, fScrnPos
        FTOI0     fRGBA,    fRGBA
        // Store.
        SQ.xyz fSTQ,     0(pOut)
        SQ     fRGBA,    1(pOut)
        SQ.xyz fScrnPos, 2(pOut)
        SQI.xy fExSTQ,   (pSTOut++)
        // Set the ADC bit.
        IADD  iADC, iADC, iADC
        ISW.w iADC, 2(pOut)
        // Update loop state.
        IADDIU pClipSrc, pClipSrc, 4
        IADDIU pOut,     pOut,     3
        // Load vertex from the end.
        LQ     fRGBA,       0(pClipSrcE)
        LQ     fGBCClipPos, 1(pClipSrcE)
        LQ.xy  fExSTQ,      2(pClipSrcE)
        LQ.xy  fInSTQ,      3(pClipSrcE)
        // Scale and translate to screenspace.
        MULA  ACC,      fGBCClipPos, fGBC2SScale
        MADDw fScrnPos, fGBC2STrans, fGBCClipPos[w]
        // Calculate 1/w.
        DIV Q, VF00[w], fScrnPos[w]
#if NORMALISE_CLIPPED_STS
        // Normalise ST.
        SUB.xy fInSTQ, fInSTQ, fNormST
#endif
        // Perspective divide.
        MULq.xyz fScrnPos, fScrnPos, Q
        MULq.xyz fSTQ,     fInSTQ,   Q
        // Apply alpha bias.
        ADDi.w fRGBA, fRGBA, I
        // Convert output data.
        FTOI4.xyz fScrnPos, fScrnPos
        FTOI0     fRGBA,    fRGBA
        // Store.
        SQ.xyz fSTQ,     0(pOut)
        SQ     fRGBA,    1(pOut)
        SQ.xyz fScrnPos, 2(pOut)
        SQI.xy fExSTQ,   (pSTOut++)
        // Set the ADC bit.
        IADD  iADC, iADC, iADC
        ISW.w iADC, 2(pOut)
        // Update loop state.
        ISUBIU pClipSrcE, pClipSrcE, 4
        IADDIU pOut,      pOut,      3
        // Done yet?
        ISUB  iRemaining, pClipSrcE, pClipSrc
        IBGEZ iRemaining, VU1_BOTANICALS_CLIP_OUTPUT_STRIP_LOOP

VU1_BOTANICALS_CLIP_FINISHED_OUTPUT_THIS_TRIANGLE:

    // Did we have an odd number of vertices?
    IADDIU iMask, VI00,         0x1
    IAND   iOdd,  iSrcPointQty, iMask
    IBEQ   iOdd,  VI00,         VU1_BOTANICALS_CLIP_SKIP_OUTPUT_PTR_FIX_UP
    // Yes. We need to correct the output ptrs.
    ISUBIU pOut,   pOut,   3
    ISUBIU pSTOut, pSTOut, 1

VU1_BOTANICALS_CLIP_SKIP_OUTPUT_PTR_FIX_UP:

    // Push int registers back onto stack.
    ISW.x pOut,   ASURA_PS2_BOTANICALS__ADDR__VI_STACK_1(VI00)
    ISW.y pSTOut, ASURA_PS2_BOTANICALS__ADDR__VI_STACK_1(VI00)

VU1_BOTANICALS_CLIP_FINISHED_THIS_TRIANGLE:

    // Retrieve int registers from stack.
    ILW.x pIn,           ASURA_PS2_BOTANICALS__ADDR__VI_STACK_0(VI00)
    ILW.y pInEnd,        ASURA_PS2_BOTANICALS__ADDR__VI_STACK_0(VI00)
#if ASSUME_NOT_MANY_TRIS_WILL_NEED_CLIPPING
    ILW.z iShouldClipFlag21, ASURA_PS2_BOTANICALS__ADDR__VI_STACK_0(VI00)
    ILW.w iShouldClipFlag1,  ASURA_PS2_BOTANICALS__ADDR__VI_STACK_0(VI00)
#else
    ILW.z iClipFlag1,    ASURA_PS2_BOTANICALS__ADDR__VI_STACK_0(VI00)
    ILW.w iClipAll21,    ASURA_PS2_BOTANICALS__ADDR__VI_STACK_0(VI00)
#endif
    ILW.z pInExtraST,    ASURA_PS2_BOTANICALS__ADDR__VI_STACK_1(VI00)
    // Recreate some constant values so VCL doesn't have to preserve them for
    // us.
    IADDIU iClipMask, VI00,      0x3F
    IADDIU i0x8000,   iClipMask, (0x8000 - 0x3F)
#if !ASSUME_NOT_MANY_TRIS_WILL_NEED_CLIPPING
    // Carry out previous 2 clips again so the find triangle loop has the
    // correct clip history.
    LQ        fClipPos, -6(pIn)
//    MUL       fClipPos, fClipPos, fGBC2SCScale //RCD - removed to make consistent with other tests
    CLIPw.xyz fClipPos, fClipPos
    LQ        fClipPos, -3(pIn)
//    MUL       fClipPos, fClipPos, fGBC2SCScale
    CLIPw.xyz fClipPos, fClipPos
#endif
    // And look for further tris that require scissoring.
    B VU1_BOTANICALS_FIND_CLIP_TRIANGLE_LOOP

VU1_BOTANICALS_CLIP_POINTS:

    // (MP - 23.10.2003) We work in guardband clip space in general. However,
    // our clipping logic works in the screen clip space, so we don't end up
    // sending large bits of polys that are outside the screen bounds. Hence
    // the following code keeps track of the current and next positions in
    // both spaces.

    // Keep track of the start of the input buffer so we can add the first
    // vertex to the end of the list when we're done.
    IADDIU pClipSrcStart, pClipSrc, 0
    // Setup an output ptr.
    IADDIU pClipOut, pClipDst, 0
    // Load the first vertex from the buffer.
    LQ    fNColour,  0(pClipSrc)
    LQ    fNGBCVert, 1(pClipSrc)
    LQ.xy fNExSTQ,   2(pClipSrc)
    LQ.xy fNSTQ,     3(pClipSrc)
    IADDIU pClipSrc, pClipSrc, 4
#if NORMALISE_CLIPPED_STS
    // Track min and max STs so we can normalise later.
    MINI.xy fMinSTs, fMinSTs, fNSTQ
    MAX.xy  fMaxSTs, fMaxSTs, fNSTQ
#endif
    // Check if this first one is in or out.
    MUL       fNSCVert, fNGBCVert, fGBC2SCScale
//    CLIPw.xyz fNSCVert, fNSCVert[w]       // RCD - changed to test against guard band instead of screen
    CLIPw.xyz fNGBCVert, fNGBCVert[w]
    FCGET     iTemp
    IAND      iTemp, iTemp, iCurrentClipMask
    IBEQ      iTemp, VI00, VU1_BOTANICALS_CLIP_CURRENT_IN

VU1_BOTANICALS_CLIP_CURRENT_OUT:

    // Done yet?
    ISUBIU iSrcPointQty, iSrcPointQty, 1
    IBEQ   iSrcPointQty, VI00, VU1_BOTANICALS_CLIP_POINTS_END
    // Move next point to current point.
    MOVE    fCColour,  fNColour
    MOVE    fCGBCVert, fNGBCVert
    MOVE    fCSCVert,  fNSCVert
    MOVE.xy fCExSTQ,   fNExSTQ
    MOVE.xy fCSTQ,     fNSTQ
    // And load a new point.
    LQ    fNColour,  0(pClipSrc)
    LQ    fNGBCVert, 1(pClipSrc)
    LQ.xy fNExSTQ,   2(pClipSrc)
    LQ.xy fNSTQ,     3(pClipSrc)
    IADDIU pClipSrc, pClipSrc, 4
#if NORMALISE_CLIPPED_STS
    // Track min and max STs so we can normalise later.
    MINI.xy fMinSTs, fMinSTs, fNSTQ
    MAX.xy  fMaxSTs, fMaxSTs, fNSTQ
#endif
    // Check if this new vertex is in or out.
    MUL       fNSCVert, fNGBCVert, fGBC2SCScale
//    CLIPw.xyz fNSCVert, fNSCVert[w]       // RCD - changed to test against guard band instead of screen
    CLIPw.xyz fNGBCVert, fNGBCVert[w]
    FCGET     iTemp
    IAND      iTemp, iTemp, iCurrentClipMask
    IBEQ      iTemp, VI00, VU1_BOTANICALS_CLIP_OUT_IN

VU1_BOTANICALS_CLIP_OUT_OUT:

    // Just carry on to next point.
    B VU1_BOTANICALS_CLIP_CURRENT_OUT

VU1_BOTANICALS_CLIP_OUT_IN:

    // swap current and next over to ensure same direction clipping
    floatExchange   fCColour,  fNColour
    floatExchange   fCGBCVert, fNGBCVert
    floatExchange   fCSCVert,  fNSCVert
    floatExchange   fCExSTQ,   fNExSTQ,     xy
    floatExchange   fCSTQ,     fNSTQ,       xy
    
    // Add 1 to output point count.
    ADD.w fPlaneSign, fPlaneSign, VF00
    // Interpolate new point on clip plane.
    BAL pInterpolateRA, VU1_BOTANICALS_INTERPOLATE
    // Ok, on to next point.
//  B VU1_BOTANICALS_CLIP_CURRENT_IN

    // send current back to next where it belongs
    MOVE    fNColour,  fCColour
    MOVE    fNGBCVert, fCGBCVert
    MOVE    fNSCVert,  fCSCVert
    MOVE.xy fNExSTQ,   fCExSTQ
    MOVE.xy fNSTQ,     fCSTQ


VU1_BOTANICALS_CLIP_CURRENT_IN:
    
    // Done yet?
    ISUBIU iSrcPointQty, iSrcPointQty, 1
    IBEQ   iSrcPointQty, VI00, VU1_BOTANICALS_CLIP_POINTS_END
    // Move next point to current point.
    MOVE    fCColour,  fNColour
    MOVE    fCGBCVert, fNGBCVert
    MOVE    fCSCVert,  fNSCVert
    MOVE.xy fCExSTQ,   fNExSTQ
    MOVE.xy fCSTQ,     fNSTQ
    // And load a new point.
    LQ    fNColour,  0(pClipSrc)
    LQ    fNGBCVert, 1(pClipSrc)
    LQ.xy fNExSTQ,   2(pClipSrc)
    LQ.xy fNSTQ,     3(pClipSrc)
    IADDIU pClipSrc, pClipSrc, 4
#if NORMALISE_CLIPPED_STS
    // Track min and max STs so we can normalise later.
    MINI.xy fMinSTs, fMinSTs, fNSTQ
    MAX.xy  fMaxSTs, fMaxSTs, fNSTQ
#endif
    // Check if this new vertex is in or out.
    MUL       fNSCVert, fNGBCVert, fGBC2SCScale
//    CLIPw.xyz fNSCVert, fNSCVert[w]       // RCD - changed to test against guard band instead of screen
    CLIPw.xyz fNGBCVert, fNGBCVert[w]
    FCGET     iTemp
    IAND      iTemp, iTemp, iCurrentClipMask
    IBNE      iTemp, VI00, VU1_BOTANICALS_CLIP_IN_OUT

VU1_BOTANICALS_CLIP_IN_IN:

    // Add 1 to output point count.
    ADD.w fPlaneSign, fPlaneSign, VF00
    // Copy current to output.
    SQ    fCColour,  0(pClipOut)
    SQ    fCGBCVert, 1(pClipOut)
    SQ.xy fCExSTQ,   2(pClipOut)
    SQ.xy fCSTQ,     3(pClipOut)
    IADDIU pClipOut, pClipOut, 4
    // And on to next vertex.
    B VU1_BOTANICALS_CLIP_CURRENT_IN

VU1_BOTANICALS_CLIP_IN_OUT:

    // Add 2 to output point count.
    ADD.w fPlaneSign, fPlaneSign, VF00
    ADD.w fPlaneSign, fPlaneSign, VF00
    // Copy current to output.
    SQ    fCColour,  0(pClipOut)
    SQ    fCGBCVert, 1(pClipOut)
    SQ.xy fCExSTQ,   2(pClipOut)
    SQ.xy fCSTQ,     3(pClipOut)
    IADDIU pClipOut, pClipOut, 4
    // Interpolate a new point on the clip plane.
    BAL pInterpolateRA, VU1_BOTANICALS_INTERPOLATE
    // And on to next vertex.
    B VU1_BOTANICALS_CLIP_CURRENT_OUT

VU1_BOTANICALS_CLIP_POINTS_END:

    // Check if we've still got to do the final line, ie. last -> first.
    IBNE pClipSrcStart, VI00, VU1_BOTANICALS_CLIP_POINT_LAST_SEGMENT
    // Ok, we're finished.
    JR pClipPointsRA

VU1_BOTANICALS_CLIP_POINT_LAST_SEGMENT:

    // Get the start of the input buffer and clear the input buffer pointer
    // so we know we've finished when we get to VU1_BOTANICALS_CLIP_POINTS_END again.
    IADDIU pClipSrc,      pClipSrcStart, 0
    IADDIU pClipSrcStart, VI00,          0
    // One final line segment to do.
    IADDIU iSrcPointQty, iSrcPointQty, 2
    // Check if the last clip was in or out.
    FCGET iTemp
    IAND  iTemp, iTemp, iCurrentClipMask
    IBEQ  iTemp, VI00, VU1_BOTANICALS_CLIP_CURRENT_IN
    B VU1_BOTANICALS_CLIP_CURRENT_OUT

VU1_BOTANICALS_INTERPOLATE:

    //       |-------L-------|
    //      C ----------|---- N
    //                  P
    //       |----A-----l--B-|
    //                  a
    //                  n
    //                  e
    //
    // We need to find the ratio R =  A / L 

/*
  RCD : this has now changed slightly, because we are using two different planes the clipping 
  decision is based on the guard band plane, but we are going to clip to the screen

  obviously this can cause problems so here is how we deal with each case :

  in-in - no problem, just pass through
  out-out - simply ignore vertex
  out-in - swap vertices and proceed as for in-out, then swap them back afterwards
  in-out - main case to handle
  given vertex C is inside guard band, vertex N is outside guardband, it is possible that C is 
  either inside or outside the screen.

  for the case where C is inside the screen, when we compute the intersection with the plane,
  it is, as in the diagram above, within the line C-N and so we get a result in the range 0-1
  
  this result is then used unchanged to interpolate the new vertex, giving a vertex on the screen edge

  for the case where C is outside the screen, the diagram is different :

       |
       P
       l--A-|----------L------|
     ..a....C --------------- N
       n------------B---------|
       e

  in fact it is even posible for C to be further outside the screen than N is, because of perspective changes
  so, in the case above, when we calculate  A / L  we get a negative number, and in the case where C is 
  further out than N we get a large ( > 1.0f ) positive number.

  (Note that it is possible for C-N to be parallel to the screen plane, in which case we will get 
  either positive or negative infinity - but these are OK too, as they will still be treated as 
  very large positive or negative numbers.

  In both cases, we want to just use the vertex C as our next vertex (note : this MAY result in some
  vertex duplication, but this is ok on PS2 as vertex processing is cheap, and degenerate triangles
  are cheap too) so we proceed as follows :

  we clamp to 0, so that negative cases produce C
  we also test against 1.0f, and if the result is bigger than 1.0f, we write out the current 
  vertex and return - noting that this is significantly cheaper than lerping a new vertex!

*/

    IBEQ   iInterpolate, VI00, VU1_BOTANICALS_INTERPOLATE_Z
    ISUBIU iTemp, iInterpolate, 1
    IBEQ   iTemp, VI00, VU1_BOTANICALS_INTERPOLATE_Y

VU1_BOTANICALS_INTERPOLATE_X:

    // Get distance from clip plane to current, taking into account +/- clip
    // plane. This is -A in the above diagram.
    ADDAx.x ACC, VF00, fCSCVert
    MSUBw.x fC2Plane, fPlaneSign, fCSCVert  // -A = current.x - plane sign * current.w
    // Get distance from next to clip plane, again taking into account plane
    // sign. This is B above.
    ADDAx.x ACC, VF00, fNSCVert
    MSUBw.x fN2Plane, fPlaneSign, fNSCVert  // B = next.x - plane sign * next.w
    // And use these values to interpolate a new point.
    B VU1_BOTANICALS_INTERPOLATE_COMMON

VU1_BOTANICALS_INTERPOLATE_Y:

    // Get distance from clip plane to current, taking into account +/- clip
    // plane. This is -A in the above diagram.
    ADDAy.x ACC, VF00, fCSCVert
    MSUBw.x fC2Plane, fPlaneSign, fCSCVert  // -A = current.y - plane sign * current.w
    // Get distance from next to clip plane, again taking into account plane
    // sign. This is B above.
    ADDAy.x ACC, VF00, fNSCVert
    MSUBw.x fN2Plane, fPlaneSign, fNSCVert  // B = next.y - plane sign * next.w
    // And use these values to interpolate a new point.
    B VU1_BOTANICALS_INTERPOLATE_COMMON

VU1_BOTANICALS_INTERPOLATE_Z:

    // Get distance from clip plane to current, taking into account +/- clip
    // plane. This is -A in the above diagram.
    ADDAz.x ACC, VF00, fCSCVert
    MSUBw.x fC2Plane, fPlaneSign, fCSCVert  // -A = current.z - plane sign * current.w
    // Get distance from next to clip plane, again taking into account plane
    // sign. This is B above.
    ADDAz.x ACC, VF00, fNSCVert
    MSUBw.x fN2Plane, fPlaneSign, fNSCVert  // B = next.z - plane sign * next.w
    // And use these values to interpolate a new point.
//  B VU1_BOTANICALS_INTERPOLATE_COMMON

VU1_BOTANICALS_INTERPOLATE_COMMON:

    // note : it may be possible to make this even faster, but fundamentally doing
    // a DIV is always going to be slow.  

    // Calculate L = B + A = B - -A
    SUB.x fL, fN2Plane, fC2Plane
    // Calculate -R =  -A / L  (note that the value in fC2Plane[x] is MINUS A)
    DIV    Q, fC2Plane[x], fL[x]

    // compute A / L, which is minus the above result
    SUBq.x fR, VF00, Q
    MAX.x   fR,fR, VF00[x]  // min value 0.0f

    // test for A / L > 1.0f
    ADDq.w VF00, VF00, Q    // get 1+(-R) so if R is > 1 then this will be negative
    FSAND  iCatchMe, 0x2    // test for sign being negative 
    IBEQ   iCatchMe, VI00, NORMAL_INTERPOLATOR // branch if positive to normal code

    // store current first vert as next vert
    SQ fCGBCVert,  1(pClipOut)
    SQ.xy fCExSTQ, 2(pClipOut)
    SQ.xy fCSTQ,   3(pClipOut)
    SQ fCColour,   0(pClipOut)
    // Update output ptr.
    IADDIU pClipOut, pClipOut, 4
    // Done.
    JR pInterpolateRA

NORMAL_INTERPOLATOR:
    // R is then our lerp coefficient along the current->next vector for the
    // position and texture coordinates.
    Lerp fNewGBCPos, fCGBCVert, fNGBCVert, fR
    Lerp fNewExST,   fCExSTQ,   fNExSTQ,   fR, xy
    Lerp fNewST,     fCSTQ,     fNSTQ,     fR, xy
    // Output.
    SQ    fNewGBCPos, 1(pClipOut)
    SQ.xy fNewExST,   2(pClipOut)
    SQ.xy fNewST,     3(pClipOut)
    // RGBA interpolation is a bit more complex...
    IBEQ iInterpolate, VI00, VU1_BOTANICALS_INTERPOLATE_RGBA_Z

VU1_BOTANICALS_INTERPOLATE_RGBA_XY:

    // Clipping with smaller bounds seems to greatly exaggerate the effects
    // of the GS's colour interpolation, in particular its lack of
    // perspective correction. We can work around this by applying an
    // "inverse perspective correction", which mostly compensates. There are
    // further details in the "Clipping colors in screen space" thread
    // (26/10/2001) in sce.dev.prog.gs. As noted there, we don't want to
    // bother when clipping against near z, hence the separate branch below.
    // (I don't think far z matters either way. It's so far out that we
    // almost never clip against it; even if we do, the geometry will be so
    // distant that the colour errors won't be noticeable.)
    MULw fRGBA0, fCColour, fCGBCVert[w]
    MULw fRGBA1, fNColour, fNGBCVert[w]
    Lerp fNewColour, fRGBA0, fRGBA1, fR
    DIV Q, VF00[w], fNewGBCPos[w]
    MULq fNewColour, fNewColour, Q
    // Finish up.
    B VU1_BOTANICALS_INTERPOLATE_END

VU1_BOTANICALS_INTERPOLATE_RGBA_Z:

    // Standard lerp.
    Lerp fNewColour, fCColour,  fNColour,  fR
    // Finish up.
//  B VU1_BOTANICALS_INTERPOLATE_END

VU1_BOTANICALS_INTERPOLATE_END:

    // Output RGBA.
    SQ fNewColour, 0(pClipOut)
    // Update output ptr.
    IADDIU pClipOut, pClipOut, 4
    // Done.
    JR pInterpolateRA

.end
