//////////////////////////////////////////////////////////////////////////////
//
// VU1_CarpetMesh.vcl
//
// Copyright (c) 2006 Rebellion. All rights reserved.
//
// See Asura_PS2_CarpetMeshDefines.h for memory map.
//
// This VCL render code is based on the GSM VCL renderer
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Asura_PS2_CarpetMeshDefines.h"
#include "vcl_sml.i"
#include "VU1_CommonMacros.i"
#include "VU1_DynamicLighting.i"

// V C L   S E T U P /////////////////////////////////////////////////////////

.syntax new

.init_vf_all
.init_vi_all

.global VU1_CARPET_MESH_INIT
.global VU1_CARPET_MESH_SWITCH_CAM_DATA_BFR
.global VU1_CARPET_MESH_SWITCH_LIGHT_DATA_BFR
.global VU1_CARPET_MESH_SWITCH_RENDERSTATE_BFR
.global VU1_CARPET_MESH_LOAD_SEED_FOR_NEW_MODULE
.global VU1_CARPET_MESH_SWITCH_STYLE_DATA_BFR
.global VU1_CARPET_MESH_LOAD_MESH_FLAGS
.global VU1_CARPET_MESH_LOAD_CAM_FLAGS
.global VU1_CARPET_MESH_BEGIN_STRIP
.global VU1_CARPET_MESH_BEGIN_STRIP_BREAK
.global VU1_CARPET_MESH_NOP

// M A C R O S ///////////////////////////////////////////////////////////////

#define NORMALISE_CLIPPED_STS (1)

// These values are assumed in VU1_CARPET_MESH_INTERPOLATE.
#define INTERPOLATE_X (2)
#define INTERPOLATE_Y (1)
#define INTERPOLATE_Z (0)

// Sets up mask for last three vertices clip flags for a particular plane.
#define MAKE_PLANE_MASK_FOR_LAST_3_VERTS(plane_flag) (((plane_flag) << 12) | ((plane_flag) << 6) | (plane_flag))

.macro floatExchange  reg1, reg2, componentmask=xyzw
.raw
    ADDx.\componentmask     \reg1, \reg2, VF00          MOVE.\componentmask  \reg2, \reg1
.endraw
.endm

// C O D E ///////////////////////////////////////////////////////////////////

--enter
    in_hw_clip clip
--endenter

    // (*) Note that there are some slightly subtle ordering issues in the
    // first few branches below. We're pretending to VCL that we're MSCNTing
    // our way straight through this stuff while in reality we'll mostly be
    // MSCALing all over the place. We can let VCL know about all the
    // dependencies we require with careful organisation and fake
    // conditionals.

VU1_CARPET_MESH_INIT:

    // Setup initial values of the input ptrs that are controlled from EE.
    IADDIU pLightDataBfr,   VI00, ASURA_PS2_CARPET_MESH__ADDR__LIGHT_DATA_BFR_0
    IADDIU pCamDataBfr,     VI00, ASURA_PS2_CARPET_MESH__ADDR__PER_OBJ_CAM_DATA_BFR_0
    IADDIU pRenderstateBfr, VI00, ASURA_PS2_CARPET_MESH__ADDR__RENDERSTATE_BFR_0
    IADDIU pStyleDataBfr,   VI00, ASURA_PS2_CARPET_MESH__ADDR__STYLE_DATA_BFR_0
    // Push onto integer register stack, to help relieve register pressure.
    ISW.x pLightDataBfr,   ASURA_PS2_CARPET_MESH__ADDR__VI_STACK_2(VI00)
    ISW.y pCamDataBfr,     ASURA_PS2_CARPET_MESH__ADDR__VI_STACK_2(VI00)
    ISW.w pRenderstateBfr, ASURA_PS2_CARPET_MESH__ADDR__VI_STACK_3(VI00)
    ISW.x pStyleDataBfr,   ASURA_PS2_CARPET_MESH__ADDR__VI_STACK_6(VI00)
    // Setup output buffer ptr.
    IADDIU pThisOutputBuffer, VI00, ASURA_PS2_CARPET_MESH__ADDR__GEOM_OUTPUT_BFR_0
    // Setup the normal unpacking register.
    MakeNormalUnpackScale fUnpackScale
    // Reset stored cam flags so we recognise they've changed the first time
    // they're uploaded.
    ISW.w VI00, ASURA_PS2_CARPET_MESH__ADDR__VI_STACK_1(VI00)
    --cont

VU1_CARPET_MESH_LOAD_MESH_FLAGS:

    // Put this first to force VCL to avoid overwriting the register
    // containing iMeshFlags in the ...SWITCH... stuff below. See the (*)
    // comment above.
    XITOP iMeshFlags
    ISW.y vi00, ASURA_PS2_CARPET_MESH__ADDR__VI_STACK_4(VI00)
    // Determine whether or not mesh is point lit and store result to stack.
    IADDIU iMask, VI00, ASURA_PS2_CARPET_MESH__FLAG__POINT_LIT
    IAND iIsPointLit, iMeshFlags, iMask
    ISW.y iIsPointLit, ASURA_PS2_CARPET_MESH__ADDR__VI_STACK_5(VI00)
    --cont

VU1_CARPET_MESH_LOAD_CAM_FLAGS:

    XITOP iCamFlags
    ILW.w iOldVisAndClipFlags, ASURA_PS2_CARPET_MESH__ADDR__VI_STACK_1(VI00)
    // Extract components.
    IADDIU iMask, VI00, ASURA_PS2_CARPET_MESH__FLAG__VIS_FOR_MORE_THAN_ONE_CAM
    IAND iMoreThanOneCamFlag, iCamFlags, iMask
    IADDIU iMask, VI00, ASURA_PS2_CARPET_MESH__MASK__CAM_VIS_AND_CLIP_FLAGS
    IAND iVisAndClipFlags, iCamFlags, iMask
    // Store to stack.
    ISW.x iMoreThanOneCamFlag, ASURA_PS2_CARPET_MESH__ADDR__VI_STACK_5(VI00)
    ISW.w iVisAndClipFlags,    ASURA_PS2_CARPET_MESH__ADDR__VI_STACK_1(VI00)
    // Have the vis flags changed?
    IADDIU iMask, VI00, ASURA_PS2_CARPET_MESH__MASK__CAM_VIS_FLAGS
    IAND iOldVisFlags, iOldVisAndClipFlags, iMask
    IAND iNewVisFlags, iVisAndClipFlags, iMask
    IBEQ iOldVisFlags, iNewVisFlags, VU1_CARPET_MESH_FINISHED_SETTING_UP_CAM_FLAGS
    // Yes, they've changed so force RS to be sent next time. Note that
    // writing any non-zero value will do here.
    ISW.w iMask, ASURA_PS2_CARPET_MESH__ADDR__VI_STACK_4(VI00)

VU1_CARPET_MESH_FINISHED_SETTING_UP_CAM_FLAGS:

    --cont

VU1_CARPET_MESH_SWITCH_CAM_DATA_BFR:

    ILW.y pCamDataBfr, ASURA_PS2_CARPET_MESH__ADDR__VI_STACK_2(VI00)
    SwitchBfr_Constants pCamDataBfr, ASURA_PS2_CARPET_MESH__ADDR__PER_OBJ_CAM_DATA_BFR_0, ASURA_PS2_CARPET_MESH__ADDR__PER_OBJ_CAM_DATA_BFR_1
    ISW.y pCamDataBfr, ASURA_PS2_CARPET_MESH__ADDR__VI_STACK_2(VI00)
    --cont

VU1_CARPET_MESH_SWITCH_LIGHT_DATA_BFR:

    ILW.x pLightDataBfr, ASURA_PS2_CARPET_MESH__ADDR__VI_STACK_2(VI00)
    SwitchBfr_Constants pLightDataBfr, ASURA_PS2_CARPET_MESH__ADDR__LIGHT_DATA_BFR_0, ASURA_PS2_CARPET_MESH__ADDR__LIGHT_DATA_BFR_1
    ISW.x pLightDataBfr, ASURA_PS2_CARPET_MESH__ADDR__VI_STACK_2(VI00)
    --cont

VU1_CARPET_MESH_SWITCH_STYLE_DATA_BFR:

    ILW.x pStyleDataBfr, ASURA_PS2_CARPET_MESH__ADDR__VI_STACK_6(VI00)
    SwitchBfr_Constants pStyleDataBfr, ASURA_PS2_CARPET_MESH__ADDR__STYLE_DATA_BFR_0, ASURA_PS2_CARPET_MESH__ADDR__STYLE_DATA_BFR_1
    ISW.x pStyleDataBfr, ASURA_PS2_CARPET_MESH__ADDR__VI_STACK_6(VI00)
    --cont

VU1_CARPET_MESH_SWITCH_RENDERSTATE_BFR:

    // Swap ptrs.
    ILW.w pRenderstateBfr, ASURA_PS2_CARPET_MESH__ADDR__VI_STACK_3(VI00)
    SwitchBfr_Constants pRenderstateBfr, ASURA_PS2_CARPET_MESH__ADDR__RENDERSTATE_BFR_0, ASURA_PS2_CARPET_MESH__ADDR__RENDERSTATE_BFR_1
    ISW.w pRenderstateBfr, ASURA_PS2_CARPET_MESH__ADDR__VI_STACK_3(VI00)
    // Set need-to-send-RS flag for next strip. Writing any non-zero value
    // will do.
    ISW.w pRenderstateBfr, ASURA_PS2_CARPET_MESH__ADDR__VI_STACK_4(VI00)
    --cont
    
VU1_CARPET_MESH_LOAD_SEED_FOR_NEW_MODULE:

    // Initialise the RNG.
    LQ.w fSeed, ASURA_PS2_CARPET_MESH__ADDR__MODULE_POS_BFR(VI00)
	RINIT R, fSeed[w]
	--cont

VU1_CARPET_MESH_DUMMY: // Never executed.

    // Make VCL think we will branch to VU1_CARPET_MESH_BEGIN_STRIP
    XITOP dummy
    IBNE dummy, VI00, VU1_CARPET_MESH_END_STRIP
    --cont

VU1_CARPET_MESH_BEGIN_STRIP_BREAK:
    B[D] VU1_CARPET_MESH_BEGIN_STRIP

VU1_CARPET_MESH_NOP:
VU1_CARPET_MESH_END_STRIP:

    --cont
    // NB: This is the one and only point we expect to do an MSCNT from.

VU1_CARPET_MESH_BEGIN_STRIP:
    
    // Retrieve main cam data bfr ptr from stack.
    ILW.y pCamDataBfr,   ASURA_PS2_CARPET_MESH__ADDR__VI_STACK_2(VI00)
    // Reset cam data ptrs.
    IADDIU pCurConstCamData, VI00, ASURA_PS2_CARPET_MESH__ADDR__CONST_CAM_DATA
    IADDIU pCurPerObjCamData, pCamDataBfr, 0
    // Reset remaining cam flags.
    ILW.w iRemainingCamFlags, ASURA_PS2_CARPET_MESH__ADDR__VI_STACK_1(VI00)
    ISW.x iRemainingCamFlags, ASURA_PS2_CARPET_MESH__ADDR__VI_STACK_4(VI00)
    // Push some registers onto stack.
    ISW.y   pCamDataBfr,       ASURA_PS2_CARPET_MESH__ADDR__VI_STACK_2(VI00)
    ISW.z   pCurPerObjCamData, ASURA_PS2_CARPET_MESH__ADDR__VI_STACK_2(VI00)
    ISW.w   pCurConstCamData,  ASURA_PS2_CARPET_MESH__ADDR__VI_STACK_2(VI00)
    // Store the initial value of the R register for this tri so that we can
    // restore it for each camera. This will keep the results consistent for
    // them all (and should also mean that R ends up the same ready for the
    // next tri regardless of how many cameras we render for).
    RGET.x fRBackup, R
    
VU1_CARPET_MESH_NEXT_CAMERA:

    // Prepare mask value for loop and setup first iteration test.
    IADDIU iVisBitMask, VI00, 0x80
    IAND iRenderForThisCam, iRemainingCamFlags, iVisBitMask
    // Any cameras left to render this strip for?
    IBEQ iRemainingCamFlags, VI00, VU1_CARPET_MESH_FINISHED_FOR_ALL_CAMERAS
    // Reload the vertex count and the light and cam ptrs. Could do this once
    // per strip, rather than per camera, but this helps with register
    // pressure.
    XITOP iNumTris
    ILW.x pLightDataBfr,     ASURA_PS2_CARPET_MESH__ADDR__VI_STACK_2(VI00)
    ILW.z pCurPerObjCamData, ASURA_PS2_CARPET_MESH__ADDR__VI_STACK_2(VI00)
    ILW.w pCurConstCamData,  ASURA_PS2_CARPET_MESH__ADDR__VI_STACK_2(VI00)

    VU1_CARPET_MESH_FIND_NEXT_CAM_LOOP:

        // Render for this cam?
        IBNE iRenderForThisCam, VI00, VU1_CARPET_MESH_FOUND_NEXT_CAM
        // No. Shift flags up 2.
        IADD iRemainingCamFlags, iRemainingCamFlags, iRemainingCamFlags
        IADD iRemainingCamFlags, iRemainingCamFlags, iRemainingCamFlags
        // Setup next iteration test.
        IAND iRenderForThisCam, iRemainingCamFlags, iVisBitMask
        // And move ptrs on.
        IADDIU pCurPerObjCamData, pCurPerObjCamData, ASURA_PS2_CARPET_MESH__OFFSET__CAM_PER_OBJ_NEXT_CAM
        IADDIU pCurConstCamData,  pCurConstCamData,  ASURA_PS2_CARPET_MESH__OFFSET__CONST_CAM_NEXT_CAM
        // Go on to check for next cam.
        B VU1_CARPET_MESH_FIND_NEXT_CAM_LOOP

VU1_CARPET_MESH_FOUND_NEXT_CAM:

    // Determine whether or not clipping will be required.
    IADDIU iMask, VI00, 0x40
    IAND iClipReqd, iRemainingCamFlags, iMask
    // Store registers back to stack.
    ISW.x iRemainingCamFlags, ASURA_PS2_CARPET_MESH__ADDR__VI_STACK_4(VI00)
    ISW.z pCurPerObjCamData,  ASURA_PS2_CARPET_MESH__ADDR__VI_STACK_2(VI00)
    ISW.w pCurConstCamData,   ASURA_PS2_CARPET_MESH__ADDR__VI_STACK_2(VI00)
    // Re-initialise R for this camera.
    RINIT R, fRBackup[x]
    // Setup input and end-of-input ptrs.
    XTOP pInTriangle

    // Load the triangle data
    LQ fVert0, 0(pInTriangle)
    LQ fVert1, 1(pInTriangle)
    LQ fVert2, 2(pInTriangle)
    LQ fRGBA0, 3(pInTriangle)
    LQ fRGBA1, 4(pInTriangle)
    LQ fRGBA2, 5(pInTriangle)
    // Convert xyz of each.
    ITOF4.xyz fVert0, fVert0
    ITOF4.xyz fVert1, fVert1
    ITOF4.xyz fVert2, fVert2
    // And the density value, in the w component.
    ITOF12.w fVert0, fVert0
    ITOF12.w fVert1, fVert1
    ITOF12.w fVert2, fVert2
    // And the RGBAs.
    ITOF0 fRGBA0, fRGBA0
    ITOF0 fRGBA1, fRGBA1
    ITOF0 fRGBA2, fRGBA2
    
    // Ptr to the style data
    ILW.x pStyleDataBfr,   ASURA_PS2_CARPET_MESH__ADDR__VI_STACK_6(VI00)
    
    // Calculate the number of quads we should generate for this tri. This is
    //
    //     tri_area * style_density * avg_vert_density
    //
    // plus some clamping.
    SUB.xyz fEdge10, fVert1, fVert0
    SUB.xyz fEdge20, fVert2, fVert0
    VectorCrossProduct fCross, fEdge10, fEdge20
    ELENG P, fCross
    ADD.w fDensity, fVert0, fVert1
    ADD.w fDensity, fDensity, fVert2
    LOI 0x3e2aaaab // 1/6
    MULi.w fDensity, fDensity, I // Actually (avg density / 2), so that when we multiplied by (2*area) we get the right result
    LQ.w fStyleDensity, 8(pStyleDataBfr)
    MUL.w fDensity, fDensity, fStyleDensity // style_density * tri_density * 0.5
    MFP.w fDoubleArea, P
    MUL.w fNumQuads, fDoubleArea, fDensity // style_density * tri_density * 0.5 * area * 2, ie. what we were after
    // Clamp to [1, max_quads].
    MAX.w fNumQuads, fNumQuads, VF00
    LOI ASURA_PS2_CARPET_MESH__MAX_QUADS_PER_STRIP
    MINIi.w fNumQuads, fNumQuads, I
    // Move to integer register.
    FTOI0.w fNumQuads, fNumQuads
    MTIR iNumQuads, fNumQuads[w]
    ISW.y iNumQuads, ASURA_PS2_CARPET_MESH__ADDR__VI_STACK_6(VI00)
	
    // Offset triangle positions by module position.
    LQ.xyz fModPos, ASURA_PS2_CARPET_MESH__ADDR__MODULE_POS_BFR(VI00)
    ADD.xyz fVert0, fVert0, fModPos
    ADD.xyz fVert1, fVert1, fModPos
    ADD.xyz fVert2, fVert2, fModPos
    
	IADDIU pQuadWork, VI00, ASURA_PS2_CARPET_MESH__ADDR__WORK_QUAD_BFR
	// Make the end pointer: 12 QW's per quad (4 verts * 3 QW's)
    IADD iTemp, iNumQuads, iNumQuads	// 2
    IADD iTemp2, iTemp, iTemp			// 4
    IADD iTemp, iTemp, iTemp2			// 6
    IADD iTemp, iTemp, iTemp			// 12
    IADD pQuadWorkEnd, pQuadWork, iTemp
	
    // Start the movement index
    RNEXT.x fMovementIndex, R
    MTIR iMovementIndex, fMovementIndex[x]
     
	// Compute view distance for each vertex
	// Make a matrix which is
	// Vert0_x Vert1_x Vert2_x
	// Vert0_y Vert1_y Vert2_y
	// Vert0_z Vert1_z Vert2_z
	// Then can compute the 3 dot products at once
	ADD.x fVertMatrix0, vf00, fVert0[x]
	ADD.y fVertMatrix0, vf00, fVert1[x]
	ADD.z fVertMatrix0, vf00, fVert2[x]
     
	ADD.x fVertMatrix1, vf00, fVert0[y]
	ADD.y fVertMatrix1, vf00, fVert1[y]
	ADD.z fVertMatrix1, vf00, fVert2[y]
     
	ADD.x fVertMatrix2, vf00, fVert0[z]
	ADD.y fVertMatrix2, vf00, fVert1[z]
	ADD.z fVertMatrix2, vf00, fVert2[z]
     
	// Load the camera view forward direction (w = is camera pos . fwd direction)
	LQ.xyzw fFwd, ASURA_PS2_CARPET_MESH__OFFSET__CAM_PER_OBJ_VIEW_FWD_VEC(pCurPerObjCamData)
	
	// Do the 3 dot-products at once!
	MULA.xyz ACC, fVertMatrix0, fFwd[x]
	MADDA.xyz ACC, fVertMatrix1, fFwd[y]
	MADD.xyz fUnitlessDistance, fVertMatrix2, fFwd[z]
	SUB.xyz fUnitlessDistance, fUnitlessDistance, fFwd[w]
	
	// Do the 1st slope y = ( x - x0 ) * scale0 = x * scale0 - K0
	// Load K0 which is styleData[11].w
    LQ.w fK0, 11(pStyleDataBfr)
	// Load scale0 which is styleData[12].w
    LQ.w fScale0, 12(pStyleDataBfr)
	// Do the 1st slope y = x * scale0 - K0
	MUL.xyz fFade0, fUnitlessDistance, fScale0[w]
	SUB.xyz fFade0, fFade0, fK0[w]
	
	// Do the 2nd slope y = ( x - x1 ) * scale1 = x * scale1 - K1
	// Load K1 which is styleData[13].w
    LQ.w fK1, 13(pStyleDataBfr)
	// Load scale1 which is styleData[14].w
    LQ.w fScale1, 14(pStyleDataBfr)
	// Do the 1st slope y = x * scale1 - K1
	MUL.xyz fFade1, fUnitlessDistance, fScale1[w]
	SUB.xyz fFade1, fFade1, fK1[w]
	
	// The fade value is the min of the 2 curves
	MINI.xyz fFade, fFade0, fFade1
	// Clamp fade to 0->1
	MINI.xyz fFade, fFade, vf00[w]
	MAX.xyz fFade, fFade, vf00[x]
	
	// Need to convert from 0->1 to 0.5->1 curves
	// Put the fade values into fRGBA0.w, fRGBA1.w, fRGBA2.w 
	// by multiplying into the existing alpha value
	MUL.w fRGBA0, fRGBA0, fFade[x]
	MUL.w fRGBA1, fRGBA1, fFade[y]
	MUL.w fRGBA2, fRGBA2, fFade[z]
		
VU1_CARPET_MESH_MAKE_QUADS_LOOP:

        --LoopCS  1, 0
//        --LoopAbs 12

		// Make setup data for this quad
	    // fA = 1.0 -> 2.0
	    // fB = 1.0 -> 2.0
	    RNEXT.x fABC, R
	    RNEXT.y fABC, R
	    // make them 0.0 -> 1.0
	    SUB.xy fABC, fABC, VF00[w]
	    // fA + fB
	    ADD.x fABSum, fABC, fABC[y]
	    // Set fABC.z = 1.0
	    ADD.z fABC, vf00, vf00[w]
	    // Need to do if ( fA + fB > 1 ) then fA = 1 - fA, fB = 1 - fB
	    // So clamp the sum to a MAX 1 then do (fA+fB)clamp - {fA,fB}
	    // Which gives: 1 - fA, 1 - fB if sum > 1 OR fB, fA if sum < 1
	    MINI.x fABSum, fABSum, vf00[w]
	    // move fABSum.x into  fABSum.y
	    ADD.y fABSum, vf00, fABSum[x]
	    // Now do (fA+fB)clamp - { fA, fB }
	    SUB.xy fABC, fABSum, fABC
	    // Now do fC = 1 - ( fA + fB )
	    SUB.z fABC, fABC, fABC[x]
	    SUB.z fABC, fABC, fABC[y]
	    
	    // So make the quad origin and scale value
	    // pos = vert0 * fA + vert1 * fB + vert2 * fC
	    // pos.w = scale = density0 * fA + density1 * fB + denstiy2 * fC
	    MULA ACC, fVert0, fABC[x]
	    MADDA ACC, fVert1, fABC[y]
	    MADD fPos, fVert2, fABC[z]
	    
	    // colour = colour0 * fA + colour1 * fB + colour2 * fC
	    MULA ACC, fRGBA0, fABC[x]
	    MADDA ACC, fRGBA1, fABC[y]
	    MADD fRGBA, fRGBA2, fABC[z]
        
        // Load the global fade value
        // which is styleData[15].w
        LQ.w fGlobalFade, 15(pStyleDataBfr)
        MUL.w fRGBA, fRGBA, fGlobalFade
        
		// Load the camera view vector for billboarding
		LQ.xyz fWidth, ASURA_PS2_CARPET_MESH__OFFSET__CAM_PER_OBJ_VIEW_WIDTH_VEC(pCurPerObjCamData)
		// Load the camera height direction (world space up for billboard around Y axis)
		LQ.xyz fHeight, ASURA_PS2_CARPET_MESH__OFFSET__CAM_PER_OBJ_VIEW_HEIGHT_VEC(pCurPerObjCamData)        
        // Get the width & height from the style data
        // Make the style index from the random number
        FTOI12.x fStyleIndex, fABC
        MTIR iStyleIndex, fStyleIndex[x]
        
        // Limit to 0 -> 15
        IADDIU iMask, vi00, 0xF
        IAND iStyleIndex, iStyleIndex, iMask
        // Get the billboard index
        // Make ptr to the billboardi ndex
        IADD iBillBoardPtr, iStyleIndex, pStyleDataBfr
        ILW.x iBillboardIndex, 0(iBillBoardPtr)
        // Make ptr to the data for this billboard
        IADD iDataPtr, iBillboardIndex, pStyleDataBfr
        // Get the width & height value which is z,w
        // Data is: x = ignored, y = movement, z = width, w = height
        LQ.zw fWidthHeight, 0(iDataPtr)
        
        // Get the min scale from the style
        // Data is: x = ignored, y = ignore, z = minScale, w = ignored
        // MinScale is 8QW on from width & height
        LQ.z fMinScale, 8(iDataPtr)
        
	    //  fScale = ( ( 1.0f - fScale ) * pxStyle->fMinScale[iStyle] ) + fScale
	    //  fOneMinScale = ( ( 1.0f - fScale )
	    SUB.w fOneMinScale, vf00, fPos[w]
	    //  ACC = ( ( 1.0f - fScale ) * pxStyle->fMinScale[iStyle] )
	    MULA.w ACC, fOneMinScale, fMinScale[z]
	    //  fScale = ACC + fScale
	    MADD.w fPos, fPos, vf00[w]
        
        // Scale the width & height by the scale value
        MUL.zw fWidthHeight, fWidthHeight, fPos[w]
        
        // Make the quad from this position
        MUL.xyz fWidth, fWidth, fWidthHeight[z]
        MUL.xyz fHeight, fHeight, fWidthHeight[w]
        
        // vertex[2] = xPos - xWidth 
        SUB.xyz fPos2, fPos, fWidth

        // vertex[3] = xPos + xWidth 
        ADD.xyz fPos3, fPos, fWidth

        // vertex[0] = xPos - xWidth - xHeight = vertex[2] - xHeight
        SUB.xyz fPos0, fPos2, fHeight
        
        // vertex[1] = xPos + xWidth - xHeight = vertex[3] - xHeight
        SUB.xyz fPos1, fPos3, fHeight
        
        // Get the movement vector
        IADDIU iMask, vi00, 0xF
        IAND iWaveIndex, iMovementIndex, iMask
        IADD iWavePtr, iWaveIndex, pStyleDataBfr
        LQ.y fWave, 0(iWavePtr)
        // Be nicer if wave data was in column x not column y
        ADD.x fPos0, fPos0, fWave[y]
        ADD.x fPos1, fPos1, fWave[y]
        
        // UV data is 16 QW's beyond the style data
        // x = U0, y = V0, z = U1, w = V1
        LQ.xyzw fUVInfo, 16(iDataPtr)
        
        // ST0 = U0(x), V1(w)
        ADD.x fST0, vf00, fUVInfo[x]
        ADD.y fST0, vf00, fUVInfo[w]
        
        // ST1 = U1(z), V1(w)
        ADD.x fST1, vf00, fUVInfo[z]
        ADD.y fST1, vf00, fUVInfo[w]
        
        // ST2 = U0(x), V0(y)
        ADD.xy fST2, vf00, fUVInfo
        
        // ST3 = U1(z), V0(y)
        ADD.x fST3, vf00, fUVInfo[z]
        ADD.y fST3, vf00, fUVInfo[y]
        
        // Convert ST to integer fixed point 12-bit representation
        FTOI12.xy fST0, fST0
        FTOI12.xy fST1, fST1
        FTOI12.xy fST2, fST2
        FTOI12.xy fST3, fST3
        
        // Convert RGBA to integer fixed point 0-bit representation
        FTOI0.xyzw fRGBA, fRGBA
        
	    // Set the ADC bit in w part of position
	    IADDIU iADCon, vi00, 0x4000
	    IADDIU iADCon, iADCon, 0x4000
	    
        // Save vertex0: adc on
        SQ.xyz fPos0,  0(pQuadWork)
        SQ.xy fST0,   1(pQuadWork)
        SQ.xyzw fRGBA,  2(pQuadWork)
	    ISW.w iADCon, 0(pQuadWork)
	    
        // Save vertex1: adc on
        SQ.xyz fPos1,  3(pQuadWork)
        SQ.xy fST1,   4(pQuadWork)
        SQ.xyzw fRGBA,  5(pQuadWork)
	    ISW.w iADCon, 3(pQuadWork)
	    
        // Save vertex2: adc off
        SQ.xyz fPos2,  6(pQuadWork)
        SQ.xy fST2,   7(pQuadWork)
        SQ.xyzw fRGBA,  8(pQuadWork)
	    ISW.w vi00,    6(pQuadWork)
        
        // Save vertex3: adc off
        SQ.xyz fPos3,  9(pQuadWork)
        SQ.xy fST3,   10(pQuadWork)
        SQ.xyzw fRGBA,  11(pQuadWork)
	    ISW.w vi00,    9(pQuadWork)
        
        // Next quad
		IADDI iMovementIndex, iMovementIndex, 1
		
        // Advance quad vertex buffer along by 12 QW's (4 vertices by 3 QW's)
    	IADDI pQuadWork, pQuadWork, 12
    	
        IBNE pQuadWork, pQuadWorkEnd, VU1_CARPET_MESH_MAKE_QUADS_LOOP

	// Set iVertexQty = iNumQuads * 4
    ILW.y iVertexQty,   ASURA_PS2_CARPET_MESH__ADDR__VI_STACK_6(VI00)
	IADD iVertexQty, iVertexQty, iVertexQty // 2
	IADD iVertexQty, iVertexQty, iVertexQty // 4
    
    // Fix up NLOOP in the GIF tag in the output buffer.
    IADDIU iNLOOPandEOP, iVertexQty,   0x7FFF
    IADDIU iNLOOPandEOP, iNLOOPandEOP, 1
    ISW.x  iNLOOPandEOP, ASURA_PS2_CARPET_MESH__OFFSET__GEOM_OUT_DRAW_GIF_TAG(pThisOutputBuffer)
    
    // Setup the pointers for the tri-strip loop to loop over the working quad bufer
	IADDIU pIn, VI00, ASURA_PS2_CARPET_MESH__ADDR__WORK_QUAD_BFR
    IADD iTemp, iVertexQty, iVertexQty // Assuming here that we have 3
    IADD iTemp, iTemp, iVertexQty      // QWs per input vertex.
    IADD pInEnd, pIn, iTemp
    
    // Retrieve the is-point-lit flags from stack.
    ILW.y iIsPointLit, ASURA_PS2_CARPET_MESH__ADDR__VI_STACK_5(VI00)
    // Setup input and output pointers for main render loop.
    IADDIU pOut, pThisOutputBuffer, ASURA_PS2_CARPET_MESH__OFFSET__GEOM_OUT_FIRST_VERTEX
    // Load in O->GBC matrix and GBC->S scale and translation vectors.
    MatrixLoad mO2GBC, ASURA_PS2_CARPET_MESH__OFFSET__CAM_PER_OBJ_O2GBC_MAT, pCurPerObjCamData
    LQ fGBC2SScale, ASURA_PS2_CARPET_MESH__OFFSET__CONST_CAM_GBC2S_SCALE(pCurConstCamData)
    LQ fGBC2STrans, ASURA_PS2_CARPET_MESH__OFFSET__CONST_CAM_GBC2S_TRANS(pCurConstCamData)
    // Set input ST's Q component to 1. This will be maintained throughout
    // the loops below.
    MAXw.z fInSTQ, VF00, VF00
    // Clipping required for current camera?
    IBNE iClipReqd, VI00, VU1_CARPET_MESH_CLIPPING_RENDERER
    // Point lit? (This implies prelit.)
    IBNE iIsPointLit, VI00, VU1_CARPET_MESH_NO_CLIP_LIT_POINT
    // Just prelit?
    IBNE iIsPrelit, VI00, VU1_CARPET_MESH_NO_CLIP_PRELIT

VU1_CARPET_MESH_NO_CLIP_PRELIT:

    VU1_CARPET_MESH_NO_CLIP_PRELIT_LOOP:

        --LoopCS  3, 0
        --LoopAbs 12

        // Load the vertex.
        LQ.xyz fOPos,  0(pIn)
        LQ.xy  fInSTQ, 1(pIn)
        LQ     fRGBA,  2(pIn)
        LQ.w   fNrm,   0(pIn)
        // Convert input data.
        ITOF12.xy fInSTQ, fInSTQ
        // Transform to GB clip space.
        MatrixMultiplyVertexW1 fGBCPos, mO2GBC, fOPos
        // Scale and translate to screenspace.
        MULA  ACC,   fGBCPos,     fGBC2SScale
        MADD fSPos, fGBC2STrans, fGBCPos[w]
        // Perspective divide XYZ and STs.
        DIV      Q,     VF00[w], fSPos[w]
        MULq.xyz fSPos, fSPos,   Q
        MULq.xyz fSTQ,  fInSTQ,  Q
        // Convert output data.
        FTOI4.xyz fSPos, fSPos
        // Grab degeneracy ADC bit from input normal.
        UnpackADC fSPos, fNrm, fUnpackScale
        // Store.
        SQ.xyz fSTQ,  0(pOut)
        SQ     fRGBA, 1(pOut)
        SQ     fSPos, 2(pOut)
        // Update loop state.
        IADDIU pIn,  pIn,  3 //IADDIU pIn,  pIn,  4
        IADDIU pOut, pOut, 3
        // Done yet?
        IBNE pIn, pInEnd, VU1_CARPET_MESH_NO_CLIP_PRELIT_LOOP

    B VU1_CARPET_MESH_NO_CLIP_POST_T_AND_L

VU1_CARPET_MESH_NO_CLIP_LIT_POINT:
    
    // Load light qty.
    ILW.w iLightQty, ASURA_PS2_CARPET_MESH__OFFSET__QW_CONTAINING_QTY(pLightDataBfr)

    VU1_CARPET_MESH_NO_CLIP_LIT_POINT_LOOP:

        --LoopCS  3, 0
//TODO  --LoopAbs

        // Load the vertex.
        LQ.xyz  fOPos,  0(pIn)
        LQ.xy   fInSTQ, 1(pIn)
        LQ      fRGBA,  2(pIn)
        LQ.w    fNrm,   0(pIn)
        // Convert input data.
        ITOF12.xy       fInSTQ, fInSTQ
        ITOF0           fRGBA,  fRGBA
        UnpackADC fNrm, fNrm, fUnpackScale
        // Transform to GB clip space.
        MatrixMultiplyVertexW1 fGBCPos, mO2GBC, fOPos
        // Scale and translate to screenspace.
        MULA  ACC,   fGBCPos,     fGBC2SScale
        MADD fSPos, fGBC2STrans, fGBCPos[w]
        // Perspective divide XYZ and STs.
        DIV      Q,     VF00[w], fSPos[w]
        MULq.xyz fSPos, fSPos,   Q
        MULq.xyz fSTQ,  fInSTQ,  Q
        // Light the vertex.
        LightEnvironment fOPos, vf00, fRGBA, iLightQty, pLightDataBfr
        // Convert output data.
        FTOI4.xyz fSPos, fSPos
        FTOI0     fRGBA, fRGBA
        // Store.
        SQ.xyz  fSTQ,  0(pOut)
        SQ      fRGBA, 1(pOut)
        SQ.xyz  fSPos, 2(pOut)
        SQ.w    fNrm,  2(pOut)
        // Update loop state.
        IADDIU pIn,  pIn,  3 // IADDIU pIn,  pIn,  4
        IADDIU pOut, pOut, 3
        // Done yet?
        IBNE pIn, pInEnd, VU1_CARPET_MESH_NO_CLIP_LIT_POINT_LOOP

VU1_CARPET_MESH_NO_CLIP_POST_T_AND_L:

    // Render all the passes.
    //IADDIU pOverwriteSTs, VI00, ASURA_PS2_CARPET_MESH__ADDR__ST_OVERWRITE_BFR
    BAL pRenderAllPassesRA, VU1_CARPET_MESH_RENDER_ALL_PASSES
    // And off to the next camera.
    B VU1_CARPET_MESH_FINISHED_FOR_CUR_CAMERA

VU1_CARPET_MESH_CLIPPING_RENDERER:

    // Flags to record whether we need to clip anything or if we clipped
    // everything.
    IADDIU iAnyClipped, VI00, 0
    IADDIU iAllClipped, VI00, 0x3F
    // Setup output ptr to clipper input buffer.
    IADDIU pIntermediateOut, VI00, ASURA_PS2_CARPET_MESH__ADDR__CLIP_INPUT_BFR
    // Point lit? (This implies prelit.)
    IBNE iIsPointLit, VI00, VU1_CARPET_MESH_CLIP_LIT_POINT

VU1_CARPET_MESH_CLIP_PRELIT:

    VU1_CARPET_MESH_CLIP_PRELIT_LOOP:

        --LoopCS  3, 0
        --LoopAbs 23

        // Load the vertex.
        LQI.xyz fOPos,  (pIn++)
        LQI.xy  fInSTQ, (pIn++)
        LQI     fRGBA,  (pIn++)
        LQ.w    fNrm,   -3(pIn) 
        // Convert input data.
        ITOF12.xy fInSTQ, fInSTQ
        UnpackADC fNrm, fNrm, fUnpackScale
        // Transform to GB clip space.
        MatrixMultiplyVertexW1 fGBCPos, mO2GBC, fOPos
        // Perform clipping judgement.
        CLIPw.xyz fGBCPos, fGBCPos
        // Scale and translate to screenspace.
        MULA  ACC,   fGBCPos,     fGBC2SScale
        MADD fSPos, fGBC2STrans, fGBCPos[w]
        // Perspective divide XYZ and STs.
        DIV      Q,     VF00[w], fSPos[w]
        MULq.xyz fSPos, fSPos,   Q
        MULq.xyz fSTQ,  fInSTQ,  Q
        // Get clip flags and AND them with iAllClipped. If iAllClipped is
        // anything other than 0 at the end, then all verts were outside one
        // of the planes.
        FCGET iClipFlags
        IAND  iAllClipped, iAllClipped, iClipFlags
        IOR   iAnyClipped, iAnyClipped, iClipFlags
        // Intermediate colour needs to be in float format.
        ITOF0 fIntermediateRGBA, fRGBA
        // Store (GB) clip space vertex, unpacked ST and colour.
        SQI    fGBCPos,           (pIntermediateOut++)
        SQI.xy fInSTQ,            (pIntermediateOut++)
        SQI    fIntermediateRGBA, (pIntermediateOut++)
        // Get clipping results.
        FCAND  VI01, 0x3FFFF
        IADDIU iClipADC, VI01, 0x7FFF
        // Combine the clip ADC with the degeneracy ADC.
        MTIR iInADC,  fNrm[w]
        IOR  iOutADC, iInADC, iClipADC
        // In spare w component of intermediate STQ, store sum of iInADC and
        // iClipADC. This will be 0x8000 iff the vertex was rejected here
        // but is not degenerate.
        IADD  iSum, iInADC, iClipADC
        ISW.w iSum, -2(pIntermediateOut)
        // Convert output data.
        FTOI4.xyz fSPos, fSPos
        // Store.
        SQ.xyz fSTQ,    0(pOut)
        SQ     fRGBA,   1(pOut)
        SQ.xyz fSPos,   2(pOut)
        ISW.w  iOutADC, 2(pOut)
        // Update ptrs.
        IADDIU pOut, pOut, 3
        // Done yet?
        IBNE pIn, pInEnd, VU1_CARPET_MESH_CLIP_PRELIT_LOOP

    // Ok, we've finished transforming this strip.
    B VU1_CARPET_MESH_CLIP_POST_T_AND_L

VU1_CARPET_MESH_CLIP_LIT_POINT:
    
    // Particularly tight for integer registers at this point...
    ISW.x pThisOutputBuffer, ASURA_PS2_CARPET_MESH__ADDR__VI_STACK_3(VI00)
    ISW.y iVertexQty,        ASURA_PS2_CARPET_MESH__ADDR__VI_STACK_3(VI00)
    ISW.z iMeshFlags,        ASURA_PS2_CARPET_MESH__ADDR__VI_STACK_3(VI00)
    // Load light qty.
    ILW.w iLightQty, ASURA_PS2_CARPET_MESH__OFFSET__QW_CONTAINING_QTY(pLightDataBfr)
    
    VU1_CARPET_MESH_CLIP_LIT_POINT_LOOP:

        --LoopCS 3, 0
//TODO  --LoopAbs 

        // Load the vertex.
        LQ.xyz fOPos,  0(pIn)
        LQ.xy  fInSTQ, 1(pIn)
        LQ     fRGBA,  2(pIn)
        LQ.w   fNrm,   0(pIn)
        // Convert input data.
        ITOF12.xy       fInSTQ, fInSTQ
        ITOF0           fRGBA,  fRGBA
        UnpackADC fNrm, fNrm, fUnpackScale
        // Transform to GB clip space.
        MatrixMultiplyVertexW1 fGBCPos, mO2GBC, fOPos
        // Perform clipping judgement.
        CLIPw.xyz fGBCPos, fGBCPos
        // Scale and translate to screenspace.
        MULA  ACC,   fGBCPos,     fGBC2SScale
        MADD fSPos, fGBC2STrans, fGBCPos[w]
        // Perspective divide XYZ and STs.
        DIV      Q,     VF00[w], fSPos[w]
        MULq.xyz fSPos, fSPos,   Q
        MULq.xyz fSTQ,  fInSTQ,  Q
        // Light the vertex.
        LightEnvironment fOPos, vf00, fRGBA, iLightQty, pLightDataBfr
        // Get clip flags and AND them with iAllClipped. If iAllClipped is
        // anything other than 0 at the end, then all verts were outside one
        // of the planes.
        FCGET iClipFlags
        IAND  iAllClipped, iAllClipped, iClipFlags
        IOR   iAnyClipped, iAnyClipped, iClipFlags
        // Store (GB) clip space vertex, unpacked ST and colour.
        SQ    fGBCPos, 0(pIntermediateOut)
        SQ.xy fInSTQ,  1(pIntermediateOut)
        SQ    fRGBA,   2(pIntermediateOut)
        // Get clipping results.
        FCAND  VI01, 0x3FFFF
        IADDIU iClipADC, VI01, 0x7FFF
        // Combine the clip ADC with the degeneracy ADC.
        MTIR iInADC,  fNrm[w]
        IOR  iOutADC, iInADC, iClipADC
        // In spare w component of intermediate STQ, store sum of iInADC and
        // iClipADC. This will be 0x8000 iff the vertex was rejected here
        // but is not degenerate.
        IADD  iSum, iInADC, iClipADC
        ISW.w iSum, 1(pIntermediateOut)
        // Convert output data.
        FTOI4.xyz fSPos, fSPos
        FTOI0     fRGBA, fRGBA
        // Store.
        SQ.xyz fSTQ,    0(pOut)
        SQ     fRGBA,   1(pOut)
        SQ.xyz fSPos,   2(pOut)
        ISW.w  iOutADC, 2(pOut)
        // Update ptrs.
        IADDIU pIn,              pIn,              3 // IADDIU pIn,              pIn,              4
        IADDIU pOut,             pOut,             3
        IADDIU pIntermediateOut, pIntermediateOut, 3
        // Done yet?
        IBNE pIn, pInEnd, VU1_CARPET_MESH_CLIP_LIT_POINT_LOOP

    // Retrieve registers from stack.
    ILW.x pThisOutputBuffer, ASURA_PS2_CARPET_MESH__ADDR__VI_STACK_3(VI00)
    ILW.y iVertexQty,        ASURA_PS2_CARPET_MESH__ADDR__VI_STACK_3(VI00)
    ILW.z iMeshFlags,        ASURA_PS2_CARPET_MESH__ADDR__VI_STACK_3(VI00)

VU1_CARPET_MESH_CLIP_POST_T_AND_L:
    
    // Check if entire packet was outside one of the clip planes. If so, just
    // move on to the next camera.
    IBNE iAllClipped, VI00, VU1_CARPET_MESH_FINISHED_FOR_CUR_CAMERA
    // Ok, render all the passes for the tris that don't require scissoring.
    //IADDIU pOverwriteSTs, VI00, ASURA_PS2_CARPET_MESH__ADDR__ST_OVERWRITE_BFR
    BAL pRenderAllPassesRA, VU1_CARPET_MESH_RENDER_ALL_PASSES
    
    // Any tris that do require scissoring?
	// Turn off any full clipping
	// IADD iAnyClipped, VI00, VI00
	
    IBNE iAnyClipped, VI00, VU1_CARPET_MESH_CLIP_TRIANGLES
    
VU1_CARPET_MESH_FINISHED_FOR_CUR_CAMERA:

    // Mask top two bits off cam flags so that next time we run the
    // VU1_CARPET_MESH_FIND_NEXT_CAM_LOOP we'll be forced to move ptrs on, etc.
    ILW.x iRemainingCamFlags, ASURA_PS2_CARPET_MESH__ADDR__VI_STACK_4(VI00)
    IADDIU iMask, VI00, 0x3F
    IAND iRemainingCamFlags, iRemainingCamFlags, iMask
    // And on to the next camera.
    B VU1_CARPET_MESH_NEXT_CAMERA

VU1_CARPET_MESH_FINISHED_FOR_ALL_CAMERAS:

    //ILW.w pRSInputBfr,   ASURA_PS2_CARPET_MESH__ADDR__VI_STACK_3(VI00)
    B  VU1_CARPET_MESH_END_STRIP

VU1_CARPET_MESH_RENDER_ALL_PASSES:

VU1_CARPET_MESH_RENDER_PASS_0:

    // Retrieve flag and ptrs from stack.
    ILW.w iNeedToSendRS, ASURA_PS2_CARPET_MESH__ADDR__VI_STACK_4(VI00)
    ILW.w pRSInputBfr,   ASURA_PS2_CARPET_MESH__ADDR__VI_STACK_3(VI00)
    // Reset need-to-send-RS flag for next time. For the default value, use
    // the can-more-than-camera-see-this-mesh flag value; we'll need to send
    // RS every time if there is more than one camera involved.
    ILW.x iMoreThanOneCamFlag, ASURA_PS2_CARPET_MESH__ADDR__VI_STACK_5(VI00)
    ISW.w iMoreThanOneCamFlag, ASURA_PS2_CARPET_MESH__ADDR__VI_STACK_4(VI00)
    // Setup RS output ptr. May not need it, of course.
    IADDIU pRSOutputBfr, VI00, ASURA_PS2_CARPET_MESH__ADDR__RS_OUTPUT_BFR
    // Is previous renderstate still ok?
    IBEQ iNeedToSendRS, VI00, VU1_CARPET_MESH_RENDER_PASS_0_KICK

VU1_CARPET_MESH_RENDER_PASS_0_RS:

    ILW.w pCurConstCamData, ASURA_PS2_CARPET_MESH__ADDR__VI_STACK_2(VI00)
    // Build base pass camera and renderstate setup list.
    LQ fSetupGIFTag, (ASURA_PS2_CARPET_MESH__ADDR__GIF_TAGS + ASURA_PS2_CARPET_MESH__OFFSET__GIF_TAGS_GEOM_SETUP)(VI00)
    LQ fSCISSOR,     ASURA_PS2_CARPET_MESH__OFFSET__CONST_CAM_SCISSOR(pCurConstCamData)
    LQ fXYOFFSET,    ASURA_PS2_CARPET_MESH__OFFSET__CONST_CAM_XYOFFSET(pCurConstCamData)
    LQ fALPHA,       ASURA_PS2_CARPET_MESH__OFFSET__RS_IN_BASE_ALPHA(pRSInputBfr)
    LQ fTEX1,        ASURA_PS2_CARPET_MESH__OFFSET__RS_IN_BASE_TEX1(pRSInputBfr)
    LQ fTEX0,        ASURA_PS2_CARPET_MESH__OFFSET__RS_IN_BASE_TEX0(pRSInputBfr)
    LQ fMIPTBP1,     ASURA_PS2_CARPET_MESH__OFFSET__RS_IN_BASE_MIPTBP1(pRSInputBfr)
    LQ fTEST,        ASURA_PS2_CARPET_MESH__OFFSET__RS_IN_BASE_TEST(pRSInputBfr)
    SQ fSetupGIFTag, ASURA_PS2_CARPET_MESH__OFFSET__RS_OUT_0_SETUP_GIF_TAG(pRSOutputBfr)
    SQ fSCISSOR,     ASURA_PS2_CARPET_MESH__OFFSET__RS_OUT_0_SCISSOR(pRSOutputBfr)
    SQ fXYOFFSET,    ASURA_PS2_CARPET_MESH__OFFSET__RS_OUT_0_XYOFFSET(pRSOutputBfr)
    SQ fALPHA,       ASURA_PS2_CARPET_MESH__OFFSET__RS_OUT_0_ALPHA(pRSOutputBfr)
    SQ fTEX1,        ASURA_PS2_CARPET_MESH__OFFSET__RS_OUT_0_TEX1(pRSOutputBfr)
    SQ fMIPTBP1,     ASURA_PS2_CARPET_MESH__OFFSET__RS_OUT_0_MIPTBP1(pRSOutputBfr)
    SQ fTEX0,        ASURA_PS2_CARPET_MESH__OFFSET__RS_OUT_0_TEX0(pRSOutputBfr)
    SQ fTEST,        ASURA_PS2_CARPET_MESH__OFFSET__RS_OUT_0_TEST(pRSOutputBfr)
    // And send it.
    XGKICK pRSOutputBfr

VU1_CARPET_MESH_RENDER_PASS_0_KICK:

    // Copy draw GIF tag to output buffer. Note that we assume NLOOP is
    // correct for the base pass setup requirements and that EOP is set.
    LQ.yzw fDrawGIFTag, (ASURA_PS2_CARPET_MESH__ADDR__GIF_TAGS + ASURA_PS2_CARPET_MESH__OFFSET__GIF_TAGS_GEOM_DRAW)(VI00)
    SQ.yzw fDrawGIFTag, ASURA_PS2_CARPET_MESH__OFFSET__GEOM_OUT_DRAW_GIF_TAG(pThisOutputBuffer)
    // Swap output buffers.
    IADDIU pOtherOutputBuffer, pThisOutputBuffer, 0
    SwitchBfr_Constants pThisOutputBuffer, ASURA_PS2_CARPET_MESH__ADDR__GEOM_OUTPUT_BFR_0, ASURA_PS2_CARPET_MESH__ADDR__GEOM_OUTPUT_BFR_1
    // And send base pass.
    XGKICK pOtherOutputBuffer
    // Ok, no other passes so we're done.
    JR pRenderAllPassesRA

VU1_CARPET_MESH_CLIP_FINISHED:

    // Check if there's anything in the output buffer that needs rendering.
    ILW.x iNLOOPandEOP, ASURA_PS2_CARPET_MESH__OFFSET__GEOM_OUT_DRAW_GIF_TAG(pThisOutputBuffer)
    IADDIU iMask, VI00, 0x7FFF
    IAND iNumVerticesInBuffer, iNLOOPandEOP, iMask
    IBEQ iNumVerticesInBuffer, VI00, VU1_CARPET_MESH_CLIP_FINISHED_POST_RENDER
    // We have something to render.
    IADDIU pOverwriteSTs, pThisOutputBuffer, ASURA_PS2_CARPET_MESH__OFFSET__CLIPPED_STS
    BAL pRenderAllPassesRA, VU1_CARPET_MESH_RENDER_ALL_PASSES

VU1_CARPET_MESH_CLIP_FINISHED_POST_RENDER:

    // And we're done.
    B VU1_CARPET_MESH_FINISHED_FOR_CUR_CAMERA

VU1_CARPET_MESH_CLIP_TRIANGLES:

    // Clipper input data format:
    //
    //        x    y    z    w                  -- in GB clip space
    //        s    t    X    should clip flag
    //        r    g    b    a                  -- floating point in range [0.0, 255.0]
    //
    // where X is "don't care".

    // Load GBC->SC scale vector.
    ILW.w pCurConstCamData, ASURA_PS2_CARPET_MESH__ADDR__VI_STACK_2(VI00)
    LQ fGBC2SCScale, ASURA_PS2_CARPET_MESH__OFFSET__CONST_CAM_GBC2SC_SCALE(pCurConstCamData)
    // Setup output pointers.
    IADDIU pOut,   pThisOutputBuffer, ASURA_PS2_CARPET_MESH__OFFSET__GEOM_OUT_FIRST_VERTEX
    IADDIU pSTOut, pThisOutputBuffer, ASURA_PS2_CARPET_MESH__OFFSET__CLIPPED_STS
    // Push onto stack.
    ISW.x pOut,   ASURA_PS2_CARPET_MESH__ADDR__VI_STACK_1(VI00)
    ISW.y pSTOut, ASURA_PS2_CARPET_MESH__ADDR__VI_STACK_1(VI00)
    // Setup some stuff for loop.
    IADDIU pIn,        VI00,      ASURA_PS2_CARPET_MESH__ADDR__CLIP_INPUT_BFR
    IADD   pInEnd,     pIn,       iVertexQty
    IADD   pInEnd,     pInEnd,    iVertexQty
    IADD   pInEnd,     pInEnd,    iVertexQty
    IADDIU iClipMask,  VI00,      0x3F
    IADDIU i0x8000,    iClipMask, (0x8000 - 0x3F)
    // Write a GIF tag to new output buffer (with EOP=1 and NLOOP=0.)
    ISW.x i0x8000, ASURA_PS2_CARPET_MESH__OFFSET__GEOM_OUT_DRAW_GIF_TAG(pThisOutputBuffer)

#define ASSUME_NOT_MANY_TRIS_WILL_NEED_CLIPPING (0)

#if ASSUME_NOT_MANY_TRIS_WILL_NEED_CLIPPING

    // Look at first two vertices.
    ILW.w iShouldClipFlag2, 1(pIn)
    ILW.w iShouldClipFlag1, 4(pIn)
    IOR iShouldClipFlag21, iShouldClipFlag2, iShouldClipFlag1
    // Skip over them.
    IADDIU pIn, pIn, 6
    IADDIU pInExtraST, VI00, (ASURA_PS2_CARPET_MESH__ADDR__ST_OVERWRITE_BFR + 2)

    VU1_CARPET_MESH_FIND_CLIP_TRIANGLE_LOOP:

        --LoopCS 1, 0

        // Are we done yet?
        IBEQ pIn, pInEnd, VU1_CARPET_MESH_CLIP_FINISHED
        // Load should-clip flag for current vtx.
        ILW.w iShouldClipFlag0, 1(pIn)
        // Update loop state.
        IADDIU pIn, pIn, 3
        IADDIU pInExtraST, pInExtraST, 1
        // Calculate should-clip flag for current tri.
        IOR iShouldClipTri, iShouldClipFlag21, iShouldClipFlag0
        // Move old flags along.
        IOR iShouldClipFlag21, iShouldClipFlag1, iShouldClipFlag0
        IADDIU iShouldClipFlag1, iShouldClipFlag0, 0
        // Continue to next vertex if none of the last three may need
        // clipping.
        IBEQ iShouldClipTri, VI00, VU1_CARPET_MESH_FIND_CLIP_TRIANGLE_LOOP

    // Current tri may need clipping. Generate last three vertices' clip
    // results.
    LQ fGBCPos0, -9(pIn)
    MUL fSCPos0, fGBCPos0, fGBC2SCScale
    CLIPw.xyz fSCPos0, fSCPos0
    FCGET iClipFlags0
    LQ fGBCPos1, -6(pIn)
    MUL fSCPos1, fGBCPos1, fGBC2SCScale
    CLIPw.xyz fSCPos1, fSCPos1
    FCGET iClipFlags1
    LQ fGBCPos2, -3(pIn)
    MUL fSCPos2, fGBCPos2, fGBC2SCScale
    CLIPw.xyz fSCPos2, fSCPos2
    FCGET iClipFlags2
    // AND them together.
    IAND iClipFlags01, iClipFlags0, iClipFlags1
    IAND iClipFlagsAll, iClipFlags01, iClipFlags2
    IAND iClipFlagsAll, iClipFlagsAll, iClipMask
    // If any bit is high, all three vertices are outside one plane so we
    // can just reject this tri.
    IBNE iClipFlagsAll, VI00, VU1_CARPET_MESH_FIND_CLIP_TRIANGLE_LOOP

#else

    // Get clipping results for vertex (n - 2) - against guardband. (used to be against screen)
    LQ        fClipPos, 0(pIn)
//    MUL       fClipPos, fClipPos, fGBC2SCScale // RCD - changed to test against guard band instead of screen
    CLIPw.xyz fClipPos, fClipPos
    FCGET     iClipFlag2
    IAND      iClipFlag2, iClipFlag2, iClipMask
    // Same for vertex (n - 1)...
    LQ        fClipPos, 3(pIn)
//    MUL       fClipPos, fClipPos, fGBC2SCScale  // RCD - changed to test against guard band instead of screen
    CLIPw.xyz fClipPos, fClipPos
    FCGET     iClipFlag1
    IAND      iClipFlag1, iClipFlag1, iClipMask
    // Skip ahead to last vertex of first tri.
    IADDIU pIn,  pIn, 6
    IADDIU pInExtraST, VI00, (ASURA_PS2_CARPET_MESH__ADDR__ST_OVERWRITE_BFR + 2)
    // Calculate half of first clipping test.
    IAND iClipAll21, iClipFlag2, iClipFlag1

    VU1_CARPET_MESH_FIND_CLIP_TRIANGLE_LOOP:

        --LoopCS 1, 0

        // Have we finished yet?
        IBEQ pIn, pInEnd, VU1_CARPET_MESH_CLIP_FINISHED
        // Get clipping results for current vertex - but against our smaller
        // screen frustum, not the guardband one.
        LQ        fClipPos, 0(pIn)
//        MUL       fClipPos, fClipPos, fGBC2SCScale    // RCD - changed to test against guard band instead of screen
        CLIPw.xyz fClipPos, fClipPos

        // Load should-clip flag early, to try and encourage VCL to schedule
        // it in some of the latencies above.
        ILW.w iShouldClipFlag, 1(pIn)

        // Grab clip result for current vertex.
        FCGET iClipFlag0

#if 0 // poorly optimised old code
        IAND  iClipFlag0, iClipFlag0, iClipMask
        // Calculate second half of clipping test.
        IAND iClipAll10, iClipFlag1, iClipFlag0
        IAND iClipAll,   iClipAll21, iClipAll10
        // Move clipping flags/result along 1.
        IADD iClipFlag1, VI00, iClipFlag0
        IADD iClipAll21, VI00, iClipAll10
#else // new improved fast code
        // Calculate second half of clipping test.
        IAND iClipAll,   iClipAll21, iClipFlag0
        // Move clipping flags/result along 1.
        IAND iClipAll21, iClipFlag1, iClipFlag0
        IAND  iClipFlag1, iClipFlag0, iClipMask
#endif

        // Update pointers.
        IADDIU pIn,        pIn,        3
        IADDIU pInExtraST, pInExtraST, 1
        // Don't bother clipping if vertex was degenerate or if we accepted it
        // in the main render loop. We have to check the latter since we're
        // using a smaller frustum here and may end up scissoring tris that
        // actually fit inside the larger one.
        IBNE iShouldClipFlag, i0x8000, VU1_CARPET_MESH_FIND_CLIP_TRIANGLE_LOOP  
        // If the triangle is completely outside at least one plane then
        // just reject.
        IBNE iClipAll, VI00, VU1_CARPET_MESH_FIND_CLIP_TRIANGLE_LOOP
        // At this point, we know the triangle is not completely inside all
        // planes since otherwise it would have been accepted in the main
        // render loop. We now also know it's not entirely outside one plane;
        // hence, it must cross one and require scissoring.

#endif

    // Ok, this triangle needs clipping. Push some int registers onto stack.
    ISW.x pIn,           ASURA_PS2_CARPET_MESH__ADDR__VI_STACK_0(VI00)
    ISW.y pInEnd,        ASURA_PS2_CARPET_MESH__ADDR__VI_STACK_0(VI00)
#if ASSUME_NOT_MANY_TRIS_WILL_NEED_CLIPPING
    ISW.z iShouldClipFlag21, ASURA_PS2_CARPET_MESH__ADDR__VI_STACK_0(VI00)
    ISW.w iShouldClipFlag1,  ASURA_PS2_CARPET_MESH__ADDR__VI_STACK_0(VI00)
#else
    ISW.z iClipFlag1,    ASURA_PS2_CARPET_MESH__ADDR__VI_STACK_0(VI00)
    ISW.w iClipAll21,    ASURA_PS2_CARPET_MESH__ADDR__VI_STACK_0(VI00)
#endif
    ISW.z pInExtraST,    ASURA_PS2_CARPET_MESH__ADDR__VI_STACK_1(VI00)
    // Setup buffers and initial vtx count.
    IADDIU pClipSrc,     pThisOutputBuffer, ASURA_PS2_CARPET_MESH__OFFSET__CLIP_WORKSPACE
    IADDIU pClipDst,     pClipSrc,          ASURA_PS2_CARPET_MESH__SIZE__CLIP_WORKSPACE
    IADDIU iSrcPointQty, VI00,              3
    // Copy the source vertices into the first clip workspace. We have to do
    // this because we have split input data
    LQ    fClipPos, -9(pIn)
    LQ.xy fSTQ,     -8(pIn)
    LQ    fRGBA,    -7(pIn)
    //LQ.xy fExSTQ,   -3(pInExtraST)
    SQ    fRGBA,     0(pClipSrc)
    SQ    fClipPos,  1(pClipSrc)
    //SQ.xy fExSTQ,    2(pClipSrc)
    SQ.xy fSTQ,      3(pClipSrc)
    LQ    fClipPos, -6(pIn)
    LQ.xy fSTQ,     -5(pIn)
    LQ    fRGBA,    -4(pIn)
    //LQ.xy fExSTQ,   -2(pInExtraST)
    SQ    fRGBA,     4(pClipSrc)
    SQ    fClipPos,  5(pClipSrc)
    //SQ.xy fExSTQ,    6(pClipSrc)
    SQ.xy fSTQ,      7(pClipSrc)
    LQ    fClipPos, -3(pIn)
    LQ.xy fSTQ,     -2(pIn)
    LQ    fRGBA,    -1(pIn)
    //LQ.xy fExSTQ,   -1(pInExtraST)
    SQ    fRGBA,     8(pClipSrc)
    SQ    fClipPos,  9(pClipSrc)
    //SQ.xy fExSTQ,   10(pClipSrc)
    SQ.xy fSTQ,     11(pClipSrc)
#if NORMALISE_CLIPPED_STS
    // Initialise min and max STs.
    LOI 0x49742400 // Arbitrary large float.
    ADDi.xy fMinSTs, VF00, I
    SUBi.xy fMaxSTs, VF00, I
#endif

    // Special case for far plane: if any vertex is on or outside it, just
    // cull instead of clipping. With sensible poly sizes and a reasonably
    // distant far clip, this shouldn't be noticeable.
    FCAND VI01, MAKE_PLANE_MASK_FOR_LAST_3_VERTS(ASURA_PS2_CARPET_MESH__CLIP_FAR)
    IBNE VI01, VI00, VU1_CARPET_MESH_CLIP_FINISHED_THIS_TRIANGLE
    
    // See which planes we need to clip against. We don't want to bother
    // clipping against a plane unless at least one vertex from the source
    // tri lies on or outside it.
    // -z
    FCAND VI01, MAKE_PLANE_MASK_FOR_LAST_3_VERTS(ASURA_PS2_CARPET_MESH__CLIP_NEAR)
    IADD iPlaneClipFlags, VI00, VI01
    // +z
    // Just shift; we already dealt with this case above.
    IADD iPlaneClipFlags, iPlaneClipFlags, iPlaneClipFlags
    // -y
    IADD iPlaneClipFlags, iPlaneClipFlags, iPlaneClipFlags
    FCAND VI01, MAKE_PLANE_MASK_FOR_LAST_3_VERTS(ASURA_PS2_CARPET_MESH__CLIP_TOP)    //       "
    IADD iPlaneClipFlags, iPlaneClipFlags, VI01
    // +y
    IADD iPlaneClipFlags, iPlaneClipFlags, iPlaneClipFlags
    FCAND VI01, MAKE_PLANE_MASK_FOR_LAST_3_VERTS(ASURA_PS2_CARPET_MESH__CLIP_BOTTOM) //       "
    IADD iPlaneClipFlags, iPlaneClipFlags, VI01
    // -x
    IADD iPlaneClipFlags, iPlaneClipFlags, iPlaneClipFlags
    FCAND VI01, MAKE_PLANE_MASK_FOR_LAST_3_VERTS(ASURA_PS2_CARPET_MESH__CLIP_LEFT)   //       "
    IADD iPlaneClipFlags, iPlaneClipFlags, VI01
    // +x
    IADD iPlaneClipFlags, iPlaneClipFlags, iPlaneClipFlags
    FCAND VI01, MAKE_PLANE_MASK_FOR_LAST_3_VERTS(ASURA_PS2_CARPET_MESH__CLIP_RIGHT)  //       "
    IADD iPlaneClipFlags, iPlaneClipFlags, VI01

VU1_CARPET_MESH_CLIP_NEAR_PLANE: // -z

    // Setup clip mask.
    IADDIU iCurrentClipMask, VI00, ASURA_PS2_CARPET_MESH__CLIP_NEAR
    // Skip if no vertices are on or outside this plane.
    IAND iAnyOnOrOutside, iPlaneClipFlags, iCurrentClipMask
    IBEQ iAnyOnOrOutside, VI00, VU1_CARPET_MESH_CLIP_FAR_PLANE
    // Clear output vertex count and set plane sign to negative.
    SUBw.xw fPlaneSign, VF00, VF00
    // Clip against this plane.
    IADDIU iInterpolate, VI00, INTERPOLATE_Z
    BAL pClipPointsRA, VU1_CARPET_MESH_CLIP_POINTS
    // Switch buffers.
    Swap pClipSrc, pClipDst
    // Extract output vertex count.
    FTOI0.w fPlaneSign, fPlaneSign
    MTIR iSrcPointQty, fPlaneSign[w]
    // Check tri hasn't been entirely clipped out. While we try to reject
    // such cases in the find-clip-tri loop above, I think numerical
    // inaccuracies can lead to it still happening.
    IBEQ iSrcPointQty, VI00, VU1_CARPET_MESH_CLIP_FINISHED_THIS_TRIANGLE
    // If we've clipped to the near plane, force x and y clips. I think the
    // vertex being behind the near plane somehow invalidates our results
    // above. Ought to double-check the maths behind this to make sure that's
    // actually the case and that this isn't just hiding a bug somewhere.
    IADDIU iXandYClips, VI00, (ASURA_PS2_CARPET_MESH__CLIP_LEFT | ASURA_PS2_CARPET_MESH__CLIP_RIGHT | ASURA_PS2_CARPET_MESH__CLIP_TOP | ASURA_PS2_CARPET_MESH__CLIP_BOTTOM)
    IOR iPlaneClipFlags, iPlaneClipFlags, iXandYClips

VU1_CARPET_MESH_CLIP_FAR_PLANE: // +z

    // Setup clip mask.
    IADDIU iCurrentClipMask, VI00, ASURA_PS2_CARPET_MESH__CLIP_FAR
    // Skip if no vertices are on or outside this plane.
    IAND iAnyOnOrOutside, iPlaneClipFlags, iCurrentClipMask
    IBEQ iAnyOnOrOutside, VI00, VU1_CARPET_MESH_CLIP_TOP_PLANE
    // Clear output vertex count and set plane sign to positive.
    SUBw.w fPlaneSign, VF00, VF00
    ADDw.x fPlaneSign, VF00, VF00
    // Clip against this plane.
    IADDIU iInterpolate, VI00, INTERPOLATE_Z
    BAL pClipPointsRA, VU1_CARPET_MESH_CLIP_POINTS
    // Switch buffers.
    Swap pClipSrc, pClipDst
    // Extract output vertex count.
    FTOI0.w fPlaneSign, fPlaneSign
    MTIR iSrcPointQty, fPlaneSign[w]
    // Check tri hasn't been entirely clipped out. (See also comment in
    // VU1_CARPET_MESH_CLIP_NEAR_PLANE.)
    IBEQ iSrcPointQty, VI00, VU1_CARPET_MESH_CLIP_FINISHED_THIS_TRIANGLE

VU1_CARPET_MESH_CLIP_TOP_PLANE: // -y

    // Setup clip mask.
    IADDIU iCurrentClipMask, VI00, ASURA_PS2_CARPET_MESH__CLIP_TOP
    // Skip if no vertices are on or outside this plane.
    IAND iAnyOnOrOutside, iPlaneClipFlags, iCurrentClipMask
    IBEQ iAnyOnOrOutside, VI00, VU1_CARPET_MESH_CLIP_BOTTOM_PLANE
    // Clear output vertex count and set plane sign to negative.
    SUBw.xw fPlaneSign, VF00, VF00
    // Clip against this plane.
    IADDIU iInterpolate, VI00, INTERPOLATE_Y
    BAL pClipPointsRA, VU1_CARPET_MESH_CLIP_POINTS
    // Switch buffers.
    Swap pClipSrc, pClipDst
    // Extract output vertex count.
    FTOI0.w fPlaneSign, fPlaneSign
    MTIR iSrcPointQty, fPlaneSign[w]
    // Check tri hasn't been entirely clipped out. (See also comment in
    // VU1_CARPET_MESH_CLIP_NEAR_PLANE.)
    IBEQ iSrcPointQty, VI00, VU1_CARPET_MESH_CLIP_FINISHED_THIS_TRIANGLE

VU1_CARPET_MESH_CLIP_BOTTOM_PLANE: // +y

    // Setup clip mask.
    IADDIU iCurrentClipMask, VI00, ASURA_PS2_CARPET_MESH__CLIP_BOTTOM
    // Skip if no vertices are on or outside this plane.
    IAND iAnyOnOrOutside, iPlaneClipFlags, iCurrentClipMask
    IBEQ iAnyOnOrOutside, VI00, VU1_CARPET_MESH_CLIP_LEFT_PLANE
    // Clear output vertex count and set plane sign to positive.
    SUBw.w fPlaneSign, VF00, VF00
    ADDw.x fPlaneSign, VF00, VF00
    // Clip against this plane.
    IADDIU iInterpolate, VI00, INTERPOLATE_Y
    BAL pClipPointsRA, VU1_CARPET_MESH_CLIP_POINTS
    // Switch buffers.
    Swap pClipSrc, pClipDst
    // Extract output vertex count.
    FTOI0.w fPlaneSign, fPlaneSign
    MTIR iSrcPointQty, fPlaneSign[w]
    // Check tri hasn't been entirely clipped out. (See also comment in
    // VU1_CARPET_MESH_CLIP_NEAR_PLANE.)
    IBEQ iSrcPointQty, VI00, VU1_CARPET_MESH_CLIP_FINISHED_THIS_TRIANGLE

VU1_CARPET_MESH_CLIP_LEFT_PLANE: // -x

    // Setup clip mask.
    IADDIU iCurrentClipMask, VI00, ASURA_PS2_CARPET_MESH__CLIP_LEFT
    // Skip if no vertices are on or outside this plane.
    IAND iAnyOnOrOutside, iPlaneClipFlags, iCurrentClipMask
    IBEQ iAnyOnOrOutside, VI00, VU1_CARPET_MESH_CLIP_RIGHT_PLANE
    // Clear output vertex count and set plane sign to negative.
    SUBw.xw fPlaneSign, VF00, VF00
    // Clip against this plane.
    IADDIU iInterpolate, VI00, INTERPOLATE_X
    BAL pClipPointsRA, VU1_CARPET_MESH_CLIP_POINTS
    // Switch buffers.
    Swap pClipSrc, pClipDst
    // Extract output vertex count.
    FTOI0.w fPlaneSign, fPlaneSign
    MTIR iSrcPointQty, fPlaneSign[w]
    // Check tri hasn't been entirely clipped out. (See also comment in
    // VU1_CARPET_MESH_CLIP_NEAR_PLANE.)
    IBEQ iSrcPointQty, VI00, VU1_CARPET_MESH_CLIP_FINISHED_THIS_TRIANGLE

VU1_CARPET_MESH_CLIP_RIGHT_PLANE: // +x

    // Setup clip mask.
    IADDIU iCurrentClipMask, VI00, ASURA_PS2_CARPET_MESH__CLIP_RIGHT
    // Skip if no vertices are on or outside this plane.
    IAND iAnyOnOrOutside, iPlaneClipFlags, iCurrentClipMask
    IBEQ iAnyOnOrOutside, VI00, VU1_CARPET_MESH_CLIP_FINISHED_PLANES
    // Clear output vertex count and set plane sign to positive.
    SUBw.w fPlaneSign, VF00, VF00
    ADDw.x fPlaneSign, VF00, VF00
    // Clip against this plane.
    IADDIU iInterpolate, VI00, INTERPOLATE_X
    BAL pClipPointsRA, VU1_CARPET_MESH_CLIP_POINTS
    // Switch buffers.
    // JAT - VCL REMOVES THE CODE GENERATED BY THIS SWAP MACRO!
    Swap pClipSrc, pClipDst
    // Extract output vertex count.
    FTOI0.w fPlaneSign, fPlaneSign
    MTIR iSrcPointQty, fPlaneSign[w]
    // Check tri hasn't been entirely clipped out. (See also comment in
    // VU1_CARPET_MESH_CLIP_NEAR_PLANE.)
    IBEQ iSrcPointQty, VI00, VU1_CARPET_MESH_CLIP_FINISHED_THIS_TRIANGLE

VU1_CARPET_MESH_CLIP_FINISHED_PLANES:

    // Retrieve int registers from stack.
    ILW.x pOut,   ASURA_PS2_CARPET_MESH__ADDR__VI_STACK_1(VI00)
    ILW.y pSTOut, ASURA_PS2_CARPET_MESH__ADDR__VI_STACK_1(VI00)
    // Check there is enough space in the output buffer for this many
    // vertices. (Easier to check the ST overwrite pointer since that's just
    // 1QW per vertex.)
    ISUB   iTemp, pSTOut, pThisOutputBuffer
    ISUBIU iTemp, iTemp,  ASURA_PS2_CARPET_MESH__OFFSET__CLIPPED_STS // space used in buffer
    IADD   iTemp, iTemp,  iSrcPointQty                       // + space required for this strip
    IADDIU iTemp, iTemp,  1                                  // + space for overrun (see comment by VU1_CARPET_MESH_CLIP_OUTPUT_STRIP_LOOP below)
    ISUBIU iTemp, iTemp,  ASURA_PS2_CARPET_MESH__SIZE__CLIPPED_STS   // Now < 0 if there is space for these vertices.
    IBLTZ  iTemp, VU1_CARPET_MESH_CLIP_ENOUGH_SPACE_IN_OUTPUT

VU1_CARPET_MESH_CLIP_NOT_ENOUGH_SPACE_IN_OUTPUT:

    // Not enough space so kick what we have and swap buffers.
    //IADDIU pOverwriteSTs, pThisOutputBuffer, ASURA_PS2_CARPET_MESH__OFFSET__CLIPPED_STS
    BAL pRenderAllPassesRA, VU1_CARPET_MESH_RENDER_ALL_PASSES
    // Reset output pointers.
    IADDIU pOut,   pThisOutputBuffer, ASURA_PS2_CARPET_MESH__OFFSET__GEOM_OUT_FIRST_VERTEX
    IADDIU pSTOut, pThisOutputBuffer, ASURA_PS2_CARPET_MESH__OFFSET__CLIPPED_STS
    // Write new GIF tag to output buffer with EOP=1 and NLOOP=0.
    IADDIU iNLOOPandEOP, VI00,         0x7FFF
    IADDIU iNLOOPandEOP, iNLOOPandEOP, 1
    ISW.x  iNLOOPandEOP, ASURA_PS2_CARPET_MESH__OFFSET__GEOM_OUT_DRAW_GIF_TAG(pThisOutputBuffer)
    
VU1_CARPET_MESH_CLIP_ENOUGH_SPACE_IN_OUTPUT:

    // Increment NLOOP count in GIF tag.
    ILW.x    iTemp,    ASURA_PS2_CARPET_MESH__OFFSET__GEOM_OUT_DRAW_GIF_TAG(pThisOutputBuffer)
    IADD    iTemp,    iTemp,    iSrcPointQty
    ISW.x    iTemp,    ASURA_PS2_CARPET_MESH__OFFSET__GEOM_OUT_DRAW_GIF_TAG(pThisOutputBuffer)
    // Setup the input pointers
    IADD   pClipSrcE, iSrcPointQty, iSrcPointQty
    IADD   pClipSrcE, pClipSrcE,    pClipSrcE
    IADD   pClipSrcE, pClipSrcE,    pClipSrc
    ISUBIU pClipSrcE, pClipSrcE,    4            // Point to last vertex in fan.
    // Fix Q component of fInSTQ at 1. This will be maintained throughout
    // the render loop below.
    ADDw.z fInSTQ, VF00, VF00
    // Set degeneracy flag for first two output verts. We'll add this value
    // to itself each time; that'll give us 0xC000, 0x8000, 0x0000 (and
    // obviously 0s thereafter), which means the ADC bit is on for precisely
    // the first two vertices.
    IADDIU iADC, VI00, 0x6000
    // The interpolate alpha can be slightly out, even when the alpha values
    // it's interpolating between are both 128, so we need to add on a little
    // bit to stop alpha 127 beeing written when it should be alpha 128. This
    // is necessary due to alpha test limits, etc.
#if NORMALISE_CLIPPED_STS
    LOI 0.5
    
    // Calc ST normalising offset.
    ADD.xy   fNormST, fMinSTs, fMaxSTs
    MULi.xy  fNormST, fNormST, I
    FTOI0.xy fNormST, fNormST
    ITOF0.xy fNormST, fNormST
#endif

VU1_CARPET_MESH_CLIP_OUTPUT_STRIP:

    VU1_CARPET_MESH_CLIP_OUTPUT_STRIP_LOOP:

        --LoopCS  2, 0
        --LoopAbs 28

        // (MP - 23.10.2003) We'll always write a multiple of two vertices
        // here to save branching in the loop. This then allows VCL to
        // optimise it for us. Of course, this means we'll overrun by up to
        // one vertex so we have to account for this when checking for free
        // space above. The GIF tag's NLOOP won't count this extra vertex but
        // we'll have to fix up the output ptr after the loop.

        // Load vertex from the start.
        LQ     fRGBA,       0(pClipSrc)
        LQ     fGBCClipPos, 1(pClipSrc)
        //LQ.xy  fExSTQ,      2(pClipSrc)
        LQ.xy  fInSTQ,      3(pClipSrc)
        // Scale and translate to screenspace.
        MULA  ACC,      fGBCClipPos, fGBC2SScale
        MADD fScrnPos, fGBC2STrans, fGBCClipPos[w]
        // Calculate 1/w.
        DIV Q, VF00[w], fScrnPos[w]
#if NORMALISE_CLIPPED_STS
        // Normalise ST.
        SUB.xy fInSTQ, fInSTQ, fNormST
#endif
        // Perspective divide.
        MULq.xyz fScrnPos, fScrnPos, Q
        MULq.xyz fSTQ,     fInSTQ,   Q
        // Apply alpha bias.
        ADDi.w fRGBA, fRGBA, I
        // Convert output data.
        FTOI4.xyz fScrnPos, fScrnPos
        FTOI0     fRGBA,    fRGBA
        // Store.
        SQ.xyz fSTQ,     0(pOut)
        SQ     fRGBA,    1(pOut)
        SQ.xyz fScrnPos, 2(pOut)
        IADDIU pSTOut, pSTOut, 1 //SQI.xy fExSTQ,   (pSTOut++)
        // Set the ADC bit.
        IADD  iADC, iADC, iADC
        ISW.w iADC, 2(pOut)
        // Update loop state.
        IADDIU pClipSrc, pClipSrc, 4
        IADDIU pOut,     pOut,     3
        // Load vertex from the end.
        LQ     fRGBA,       0(pClipSrcE)
        LQ     fGBCClipPos, 1(pClipSrcE)
        //LQ.xy  fExSTQ,      2(pClipSrcE)
        LQ.xy  fInSTQ,      3(pClipSrcE)
        // Scale and translate to screenspace.
        MULA  ACC,      fGBCClipPos, fGBC2SScale
        MADD fScrnPos, fGBC2STrans, fGBCClipPos[w]
        // Calculate 1/w.
        DIV Q, VF00[w], fScrnPos[w]
#if NORMALISE_CLIPPED_STS
        // Normalise ST.
        SUB.xy fInSTQ, fInSTQ, fNormST
#endif
        // Perspective divide.
        MULq.xyz fScrnPos, fScrnPos, Q
        MULq.xyz fSTQ,     fInSTQ,   Q
        // Apply alpha bias.
        ADDi.w fRGBA, fRGBA, I
        // Convert output data.
        FTOI4.xyz fScrnPos, fScrnPos
        FTOI0     fRGBA,    fRGBA
        // Store.
        SQ.xyz fSTQ,     0(pOut)
        SQ     fRGBA,    1(pOut)
        SQ.xyz fScrnPos, 2(pOut)
        IADDIU pSTOut, pSTOut, 1 //SQI.xy fExSTQ,   (pSTOut++)
        // Set the ADC bit.
        IADD  iADC, iADC, iADC
        ISW.w iADC, 2(pOut)
        // Update loop state.
        ISUBIU pClipSrcE, pClipSrcE, 4
        IADDIU pOut,      pOut,      3
        // Done yet?
        ISUB  iRemaining, pClipSrcE, pClipSrc
        IBGEZ iRemaining, VU1_CARPET_MESH_CLIP_OUTPUT_STRIP_LOOP

VU1_CARPET_MESH_CLIP_FINISHED_OUTPUT_THIS_TRIANGLE:

    // Did we have an odd number of vertices?
    IADDIU iMask, VI00,         0x1
    IAND   iOdd,  iSrcPointQty, iMask
    IBEQ   iOdd,  VI00,         VU1_CARPET_MESH_CLIP_SKIP_OUTPUT_PTR_FIX_UP
    // Yes. We need to correct the output ptrs.
    ISUBIU pOut,   pOut,   3
    ISUBIU pSTOut, pSTOut, 1

VU1_CARPET_MESH_CLIP_SKIP_OUTPUT_PTR_FIX_UP:

    // Push int registers back onto stack.
    ISW.x pOut,   ASURA_PS2_CARPET_MESH__ADDR__VI_STACK_1(VI00)
    ISW.y pSTOut, ASURA_PS2_CARPET_MESH__ADDR__VI_STACK_1(VI00)

VU1_CARPET_MESH_CLIP_FINISHED_THIS_TRIANGLE:

    // Retrieve int registers from stack.
    ILW.x pIn,           ASURA_PS2_CARPET_MESH__ADDR__VI_STACK_0(VI00)
    ILW.y pInEnd,        ASURA_PS2_CARPET_MESH__ADDR__VI_STACK_0(VI00)
#if ASSUME_NOT_MANY_TRIS_WILL_NEED_CLIPPING
    ILW.z iShouldClipFlag21, ASURA_PS2_CARPET_MESH__ADDR__VI_STACK_0(VI00)
    ILW.w iShouldClipFlag1,  ASURA_PS2_CARPET_MESH__ADDR__VI_STACK_0(VI00)
#else
    ILW.z iClipFlag1,    ASURA_PS2_CARPET_MESH__ADDR__VI_STACK_0(VI00)
    ILW.w iClipAll21,    ASURA_PS2_CARPET_MESH__ADDR__VI_STACK_0(VI00)
#endif
    ILW.z pInExtraST,    ASURA_PS2_CARPET_MESH__ADDR__VI_STACK_1(VI00)
    // Recreate some constant values so VCL doesn't have to preserve them for
    // us.
    IADDIU iClipMask, VI00,      0x3F
    IADDIU i0x8000,   iClipMask, (0x8000 - 0x3F)
#if !ASSUME_NOT_MANY_TRIS_WILL_NEED_CLIPPING
    // Carry out previous 2 clips again so the find triangle loop has the
    // correct clip history.
    LQ        fClipPos, -6(pIn)
//    MUL       fClipPos, fClipPos, fGBC2SCScale //RCD - removed to make consistent with other tests
    CLIPw.xyz fClipPos, fClipPos
    LQ        fClipPos, -3(pIn)
//    MUL       fClipPos, fClipPos, fGBC2SCScale
    CLIPw.xyz fClipPos, fClipPos
#endif
    // And look for further tris that require scissoring.
    B VU1_CARPET_MESH_FIND_CLIP_TRIANGLE_LOOP

VU1_CARPET_MESH_CLIP_POINTS:

    // (MP - 23.10.2003) We work in guardband clip space in general. However,
    // our clipping logic works in the screen clip space, so we don't end up
    // sending large bits of polys that are outside the screen bounds. Hence
    // the following code keeps track of the current and next positions in
    // both spaces.

    // Keep track of the start of the input buffer so we can add the first
    // vertex to the end of the list when we're done.
    IADDIU pClipSrcStart, pClipSrc, 0
    // Setup an output ptr.
    IADDIU pClipOut, pClipDst, 0
    // Load the first vertex from the buffer.
    LQ    fNColour,  0(pClipSrc)
    LQ    fNGBCVert, 1(pClipSrc)
    LQ.xy fNExSTQ,   2(pClipSrc)
    LQ.xy fNSTQ,     3(pClipSrc)
    IADDIU pClipSrc, pClipSrc, 4
#if NORMALISE_CLIPPED_STS
    // Track min and max STs so we can normalise later.
    MINI.xy fMinSTs, fMinSTs, fNSTQ
    MAX.xy  fMaxSTs, fMaxSTs, fNSTQ
#endif
    // Check if this first one is in or out.
    MUL       fNSCVert, fNGBCVert, fGBC2SCScale
//    CLIPw.xyz fNSCVert, fNSCVert[w]       // RCD - changed to test against guard band instead of screen
    CLIPw.xyz fNGBCVert, fNGBCVert[w]
    FCGET     iTemp
    IAND      iTemp, iTemp, iCurrentClipMask
    IBEQ      iTemp, VI00, VU1_CARPET_MESH_CLIP_CURRENT_IN

VU1_CARPET_MESH_CLIP_CURRENT_OUT:

    // Done yet?
    ISUBIU iSrcPointQty, iSrcPointQty, 1
    IBEQ   iSrcPointQty, VI00, VU1_CARPET_MESH_CLIP_POINTS_END
    // Move next point to current point.
    MOVE    fCColour,  fNColour
    MOVE    fCGBCVert, fNGBCVert
    MOVE    fCSCVert,  fNSCVert
    MOVE.xy fCExSTQ,   fNExSTQ
    MOVE.xy fCSTQ,     fNSTQ
    // And load a new point.
    LQ    fNColour,  0(pClipSrc)
    LQ    fNGBCVert, 1(pClipSrc)
    LQ.xy fNExSTQ,   2(pClipSrc)
    LQ.xy fNSTQ,     3(pClipSrc)
    IADDIU pClipSrc, pClipSrc, 4
#if NORMALISE_CLIPPED_STS
    // Track min and max STs so we can normalise later.
    MINI.xy fMinSTs, fMinSTs, fNSTQ
    MAX.xy  fMaxSTs, fMaxSTs, fNSTQ
#endif
    // Check if this new vertex is in or out.
    MUL       fNSCVert, fNGBCVert, fGBC2SCScale
//    CLIPw.xyz fNSCVert, fNSCVert[w]       // RCD - changed to test against guard band instead of screen
    CLIPw.xyz fNGBCVert, fNGBCVert[w]
    FCGET     iTemp
    IAND      iTemp, iTemp, iCurrentClipMask
    IBEQ      iTemp, VI00, VU1_CARPET_MESH_CLIP_OUT_IN

VU1_CARPET_MESH_CLIP_OUT_OUT:

    // Just carry on to next point.
    B VU1_CARPET_MESH_CLIP_CURRENT_OUT

VU1_CARPET_MESH_CLIP_OUT_IN:

    // swap current and next over to ensure same direction clipping
    floatExchange   fCColour,  fNColour
    floatExchange   fCGBCVert, fNGBCVert
    floatExchange   fCSCVert,  fNSCVert
    floatExchange   fCExSTQ,   fNExSTQ,     xy
    floatExchange   fCSTQ,     fNSTQ,       xy
    
    // Add 1 to output point count.
    ADD.w fPlaneSign, fPlaneSign, VF00
    // Interpolate new point on clip plane.
    BAL pInterpolateRA, VU1_CARPET_MESH_INTERPOLATE
    // Ok, on to next point.
//  B VU1_CARPET_MESH_CLIP_CURRENT_IN

    // send current back to next where it belongs
    MOVE    fNColour,  fCColour
    MOVE    fNGBCVert, fCGBCVert
    MOVE    fNSCVert,  fCSCVert
    MOVE.xy fNExSTQ,   fCExSTQ
    MOVE.xy fNSTQ,     fCSTQ


VU1_CARPET_MESH_CLIP_CURRENT_IN:
    
    // Done yet?
    ISUBIU iSrcPointQty, iSrcPointQty, 1
    IBEQ   iSrcPointQty, VI00, VU1_CARPET_MESH_CLIP_POINTS_END
    // Move next point to current point.
    MOVE    fCColour,  fNColour
    MOVE    fCGBCVert, fNGBCVert
    MOVE    fCSCVert,  fNSCVert
    MOVE.xy fCExSTQ,   fNExSTQ
    MOVE.xy fCSTQ,     fNSTQ
    // And load a new point.
    LQ    fNColour,  0(pClipSrc)
    LQ    fNGBCVert, 1(pClipSrc)
    LQ.xy fNExSTQ,   2(pClipSrc)
    LQ.xy fNSTQ,     3(pClipSrc)
    IADDIU pClipSrc, pClipSrc, 4
#if NORMALISE_CLIPPED_STS
    // Track min and max STs so we can normalise later.
    MINI.xy fMinSTs, fMinSTs, fNSTQ
    MAX.xy  fMaxSTs, fMaxSTs, fNSTQ
#endif
    // Check if this new vertex is in or out.
    MUL       fNSCVert, fNGBCVert, fGBC2SCScale
//    CLIPw.xyz fNSCVert, fNSCVert[w]       // RCD - changed to test against guard band instead of screen
    CLIPw.xyz fNGBCVert, fNGBCVert[w]
    FCGET     iTemp
    IAND      iTemp, iTemp, iCurrentClipMask
    IBNE      iTemp, VI00, VU1_CARPET_MESH_CLIP_IN_OUT

VU1_CARPET_MESH_CLIP_IN_IN:

    // Add 1 to output point count.
    ADD.w fPlaneSign, fPlaneSign, VF00
    // Copy current to output.
    SQ    fCColour,  0(pClipOut)
    SQ    fCGBCVert, 1(pClipOut)
    SQ.xy fCExSTQ,   2(pClipOut)
    SQ.xy fCSTQ,     3(pClipOut)
    IADDIU pClipOut, pClipOut, 4
    // And on to next vertex.
    B VU1_CARPET_MESH_CLIP_CURRENT_IN

VU1_CARPET_MESH_CLIP_IN_OUT:

    // Add 2 to output point count.
    ADD.w fPlaneSign, fPlaneSign, VF00
    ADD.w fPlaneSign, fPlaneSign, VF00
    // Copy current to output.
    SQ    fCColour,  0(pClipOut)
    SQ    fCGBCVert, 1(pClipOut)
    SQ.xy fCExSTQ,   2(pClipOut)
    SQ.xy fCSTQ,     3(pClipOut)
    IADDIU pClipOut, pClipOut, 4
    // Interpolate a new point on the clip plane.
    BAL pInterpolateRA, VU1_CARPET_MESH_INTERPOLATE
    // And on to next vertex.
    B VU1_CARPET_MESH_CLIP_CURRENT_OUT

VU1_CARPET_MESH_CLIP_POINTS_END:

    // Check if we've still got to do the final line, ie. last -> first.
    IBNE pClipSrcStart, VI00, VU1_CARPET_MESH_CLIP_POINT_LAST_SEGMENT
    // Ok, we're finished.
    JR pClipPointsRA

VU1_CARPET_MESH_CLIP_POINT_LAST_SEGMENT:

    // Get the start of the input buffer and clear the input buffer pointer
    // so we know we've finished when we get to VU1_CARPET_MESH_CLIP_POINTS_END again.
    IADDIU pClipSrc,      pClipSrcStart, 0
    IADDIU pClipSrcStart, VI00,          0
    // One final line segment to do.
    IADDIU iSrcPointQty, iSrcPointQty, 2
    // Check if the last clip was in or out.
    FCGET iTemp
    IAND  iTemp, iTemp, iCurrentClipMask
    IBEQ  iTemp, VI00, VU1_CARPET_MESH_CLIP_CURRENT_IN
    B VU1_CARPET_MESH_CLIP_CURRENT_OUT

VU1_CARPET_MESH_INTERPOLATE:

    //       |-------L-------|
    //      C ----------|---- N
    //                  P
    //       |----A-----l--B-|
    //                  a
    //                  n
    //                  e
    //
    // We need to find the ratio R =  A / L 

/*
  RCD : this has now changed slightly, because we are using two different planes the clipping 
  decision is based on the guard band plane, but we are going to clip to the screen

  obviously this can cause problems so here is how we deal with each case :

  in-in - no problem, just pass through
  out-out - simply ignore vertex
  out-in - swap vertices and proceed as for in-out, then swap them back afterwards
  in-out - main case to handle
  given vertex C is inside guard band, vertex N is outside guardband, it is possible that C is 
  either inside or outside the screen.

  for the case where C is inside the screen, when we compute the intersection with the plane,
  it is, as in the diagram above, within the line C-N and so we get a result in the range 0-1
  
  this result is then used unchanged to interpolate the new vertex, giving a vertex on the screen edge

  for the case where C is outside the screen, the diagram is different :

       |
       P
       l--A-|----------L------|
     ..a....C --------------- N
       n------------B---------|
       e

  in fact it is even posible for C to be further outside the screen than N is, because of perspective changes
  so, in the case above, when we calculate  A / L  we get a negative number, and in the case where C is 
  further out than N we get a large ( > 1.0f ) positive number.

  (Note that it is possible for C-N to be parallel to the screen plane, in which case we will get 
  either positive or negative infinity - but these are OK too, as they will still be treated as 
  very large positive or negative numbers.

  In both cases, we want to just use the vertex C as our next vertex (note : this MAY result in some
  vertex duplication, but this is ok on PS2 as vertex processing is cheap, and degenerate triangles
  are cheap too) so we proceed as follows :

  we clamp to 0, so that negative cases produce C
  we also test against 1.0f, and if the result is bigger than 1.0f, we write out the current 
  vertex and return - noting that this is significantly cheaper than lerping a new vertex!

*/

    IBEQ   iInterpolate, VI00, VU1_CARPET_MESH_INTERPOLATE_Z
    ISUBIU iTemp, iInterpolate, 1
    IBEQ   iTemp, VI00, VU1_CARPET_MESH_INTERPOLATE_Y

VU1_CARPET_MESH_INTERPOLATE_X:

    // Get distance from clip plane to current, taking into account +/- clip
    // plane. This is -A in the above diagram.
    ADDAx.x ACC, VF00, fCSCVert
    MSUBw.x fC2Plane, fPlaneSign, fCSCVert  // -A = current.x - plane sign * current.w
    // Get distance from next to clip plane, again taking into account plane
    // sign. This is B above.
    ADDAx.x ACC, VF00, fNSCVert
    MSUBw.x fN2Plane, fPlaneSign, fNSCVert  // B = next.x - plane sign * next.w
    // And use these values to interpolate a new point.
    B VU1_CARPET_MESH_INTERPOLATE_COMMON

VU1_CARPET_MESH_INTERPOLATE_Y:

    // Get distance from clip plane to current, taking into account +/- clip
    // plane. This is -A in the above diagram.
    ADDAy.x ACC, VF00, fCSCVert
    MSUBw.x fC2Plane, fPlaneSign, fCSCVert  // -A = current.y - plane sign * current.w
    // Get distance from next to clip plane, again taking into account plane
    // sign. This is B above.
    ADDAy.x ACC, VF00, fNSCVert
    MSUBw.x fN2Plane, fPlaneSign, fNSCVert  // B = next.y - plane sign * next.w
    // And use these values to interpolate a new point.
    B VU1_CARPET_MESH_INTERPOLATE_COMMON

VU1_CARPET_MESH_INTERPOLATE_Z:

    // Get distance from clip plane to current, taking into account +/- clip
    // plane. This is -A in the above diagram.
    ADDAz.x ACC, VF00, fCSCVert
    MSUBw.x fC2Plane, fPlaneSign, fCSCVert  // -A = current.z - plane sign * current.w
    // Get distance from next to clip plane, again taking into account plane
    // sign. This is B above.
    ADDAz.x ACC, VF00, fNSCVert
    MSUBw.x fN2Plane, fPlaneSign, fNSCVert  // B = next.z - plane sign * next.w
    // And use these values to interpolate a new point.
//  B VU1_CARPET_MESH_INTERPOLATE_COMMON

VU1_CARPET_MESH_INTERPOLATE_COMMON:

    // note : it may be possible to make this even faster, but fundamentally doing
    // a DIV is always going to be slow.  

    // Calculate L = B + A = B - -A
    SUB.x fL, fN2Plane, fC2Plane
    // Calculate -R =  -A / L  (note that the value in fC2Plane[x] is MINUS A)
    DIV    Q, fC2Plane[x], fL[x]

    // compute A / L, which is minus the above result
    SUBq.x fR, VF00, Q
    MAX.x   fR,fR, VF00[x]  // min value 0.0f

    // test for A / L > 1.0f
    ADDq.w VF00, VF00, Q    // get 1+(-R) so if R is > 1 then this will be negative
    FSAND  iCatchMe, 0x2    // test for sign being negative 
    IBEQ   iCatchMe, VI00, NORMAL_INTERPOLATOR // branch if positive to normal code

    // store current first vert as next vert
    SQ fCGBCVert,  1(pClipOut)
    SQ.xy fCExSTQ, 2(pClipOut)
    SQ.xy fCSTQ,   3(pClipOut)
    SQ fCColour,   0(pClipOut)
    // Update output ptr.
    IADDIU pClipOut, pClipOut, 4
    // Done.
    JR pInterpolateRA

NORMAL_INTERPOLATOR:
    // R is then our lerp coefficient along the current->next vector for the
    // position and texture coordinates.
    Lerp fNewGBCPos, fCGBCVert, fNGBCVert, fR
    Lerp fNewExST,   fCExSTQ,   fNExSTQ,   fR, xy
    Lerp fNewST,     fCSTQ,     fNSTQ,     fR, xy
    // Output.
    SQ    fNewGBCPos, 1(pClipOut)
    SQ.xy fNewExST,   2(pClipOut)
    SQ.xy fNewST,     3(pClipOut)
    // RGBA interpolation is a bit more complex...
    IBEQ iInterpolate, VI00, VU1_CARPET_MESH_INTERPOLATE_RGBA_Z

VU1_CARPET_MESH_INTERPOLATE_RGBA_XY:

    // Clipping with smaller bounds seems to greatly exaggerate the effects
    // of the GS's colour interpolation, in particular its lack of
    // perspective correction. We can work around this by applying an
    // "inverse perspective correction", which mostly compensates. There are
    // further details in the "Clipping colors in screen space" thread
    // (26/10/2001) in sce.dev.prog.gs. As noted there, we don't want to
    // bother when clipping against near z, hence the separate branch below.
    // (I don't think far z matters either way. It's so far out that we
    // almost never clip against it; even if we do, the geometry will be so
    // distant that the colour errors won't be noticeable.)
    MULw fRGBA0, fCColour, fCGBCVert[w]
    MULw fRGBA1, fNColour, fNGBCVert[w]
    Lerp fNewColour, fRGBA0, fRGBA1, fR
    DIV Q, VF00[w], fNewGBCPos[w]
    MULq fNewColour, fNewColour, Q
    // Finish up.
    B VU1_CARPET_MESH_INTERPOLATE_END

VU1_CARPET_MESH_INTERPOLATE_RGBA_Z:

    // Standard lerp.
    Lerp fNewColour, fCColour,  fNColour,  fR
    // Finish up.
//  B VU1_CARPET_MESH_INTERPOLATE_END

VU1_CARPET_MESH_INTERPOLATE_END:

    // Output RGBA.
    SQ fNewColour, 0(pClipOut)
    // Update output ptr.
    IADDIU pClipOut, pClipOut, 4
    // Done.
    JR pInterpolateRA

.end
