;// PSJE 20.3.03 -- Updated macro, using local variables, also takes a pointer to the current light data buffer
.macro  LightEnvironment	vVertex, vNormal, vColour, iLightQty, pLightData
    ;// copy normal into 3 registers' w for quicker dot product
    MULx.w	vNormalX,	VF00,	\vNormal
    MULy.w	vNormalY,	VF00,	\vNormal
    MULz.w	vNormalZ,	vf00,	\vNormal
	;// Setup input pointer and end pointer
    IADD	pLightIn,  \pLightData, VI00
	IADD	pLightEnd, \iLightQty,  \iLightQty
	IADD	pLightEnd, pLightEnd,   \pLightData
ENV_LIGHTING_LOOP\@:
		;// Check if we've finished
		IBEQ	pLightIn, pLightEnd, ENV_NO_MORE_LIGHTS\@
		;// Load the data for this light
		LQ.xyzw	vL1Position,	0(pLightIn)
		LQ.xyz	vL1Colour,		1(pLightIn)
		LQ.xyzw	vL2Position,	2(pLightIn)
		LQ.xyz	vL2Colour,		3(pLightIn)
		;// Update input pointer
		IADDIU	pLightIn, pLightIn, 2
	    ;// calculate light1 to vertex vector
		SUB.xyz	vL1V,		\vVertex,	vL1Position
		MUL.xyz	fL1VSqr,	vL1V,		vL1V
		ADDy.x	fL1VSqr,	fL1VSqr,	fL1VSqr
		ADDz.x	fL1VSqr,	fL1VSqr,	fL1VSqr
		SQRT	Q,			fL1VSqr[x]
		;// calculate dot product later after range test?
	    MULAx.w		ACC,		vNormalX, vL1V
	    MADDAy.w	ACC,		vNormalY, vL1V
		MADDz.w		fL1VdotN,	vNormalZ, vL1V
	    ;// clamp to 0, or skip if greater than 0?
		MINIx.w		fL1VdotN,	fL1VdotN, VF00
		;// calculate light2 to vertex vector
		SUB.xyz		vL2V,	\vVertex,	vL2Position
		ELENG		P,		vL2V
		;// calculate dot product later after range test?
		MULAx.w		ACC,		vNormalX,	vL2V
		MADDAy.w	ACC,		vNormalY,	vL2V
		MADDz.w		fL2VdotN,	vNormalZ,	vL2V
ENV_LIGHT1\@:
       ;// if (dist - range > 0), out of range
		;// i.e. (range - dist < 0), out of range
		SUBq.w	fRangeMinusDist1,	vL1Position,	Q
		IADDIU	iTemp,	VI00,	0x10
		FMAND	iTemp,	iTemp
		IBNE	iTemp,	VI00,	ENV_LIGHT2\@
	    ;// divide dot product by dist
		MULq.w	fDist1,	VF00,	Q
		DIV		Q,		fL1VdotN[w],	fDist1[w]
	    ;// calculate lightrgb * (dist-range)
		MULw.xyz	fScale1,	vL1Colour,	fRangeMinusDist1
	    ;// set accumulator to current RGB
		ADDA.xyz	ACC,	VF00,	\vColour
        ;// set output to (lightrgb * (dist-range) * dotproduct / dist) + rgbout
		;// i.e. rgbout - (lightrgb * (range-dist) * dotproduct / dist)
		MSUBq.xyz   \vColour,	fScale1,	Q
ENV_LIGHT2\@:
		;// Check if we've finished
		IBEQ	pLightIn, pLightEnd, ENV_NO_MORE_LIGHTS\@
		;// Update input pointer
		IADDIU	pLightIn, pLightIn, 2
	    ;// clamp to 0, or skip if greater than 0?
		MINIx.w	fL2VdotN,	fL2VdotN, VF00
		MFP.w	fDist2,		P
    	;// if (dist - range > 0), out of range
		SUB.w	fDistMinusRange2, fDist2, vL2Position
		IADDIU	iTemp, VI00, 0x10
		FMAND	iTemp, iTemp
		IBEQ	iTemp,	VI00,	ENV_LIGHTING_LOOP\@
		;// divide dot product by dist
		DIV		Q,	fL2VdotN[w],	fDist2[w]
		;// calculate lightrgb * (dist-range)
		MULw.xyz	fScale2, vL2Colour,	fDistMinusRange2
		;// set accumulator to current RGB
		ADDA.xyz	ACC,	VF00,	\vColour
		;// set output to (lightrgb * (dist-range) * dotproduct / dist) + rgbout
		MADDq.xyz   \vColour,	fScale2,	Q
    	;// back to start of loop
		B	ENV_LIGHTING_LOOP\@
ENV_NO_MORE_LIGHTS\@:
	;// Clamp lights to 0-255
	LOI		255.0
    MINIi.xyz \vColour,	\vColour,	I
.endm

#if 0
;// PSJE 20.3.03 -- Based on above macro, but using local variables + tweaked
;//					data layout for the environment renderer
;// Re jigged the lighting calculation to reduce it from a Sqrt + Div per light, to a RQRT per light
;//		 C = LCol*LRan*A.N/LRan  -  LCol*Dist*A.N/Lran
;//		=
;//		 C = LCol*(A.N*Dist)*(1/D) - LCol*(A.N*Dist)*(1/Lran)
;//		:
;//		 J = A.N*Dist
;//		 R = 1/Dist
;//		 C = Lcol*J*R - Lcol*J*1/Lran
;//		:
;//		 S = Lcol*J
;//		 C = J*R - J*1/Lran
;//		:
;//		 C = D*(R-1/Lran)
;//
;//	THIS IS NOT FINISHED AND WONT COMPILE OR WORK AT PRESENT
;//
.macro  LightEnvironment	vVertex, vNormal, vColour, iLightQty, pLightData
    ;// copy normal into 3 registers' w for quicker dot product
    MULx.w	vNormalX,	VF00,	\vNormal
    MULy.w	vNormalY,	VF00,	\vNormal
    MULz.w	vNormalZ,	vf00,	\vNormal
	;// Setup input pointer and end pointer
    IADD	pLightIn,  \pLightData, VI00
	IADD	pLightEnd, \iLightQty,  \iLightQty
	IADD	pLightEnd, pLightEnd,   \pLightData
ENV_LIGHTING_LOOP\@:
		;// Check if we've finished
		IBEQ	pLightIn, pLightEnd, ENV_NO_MORE_LIGHTS\@
		;// Load the data for this light
		LQ.xyzw	vL1P,	0(pLightIn)
		LQ.xyzw	vL1C,	1(pLightIn)
		LQ.xyzw	vL2P,	2(pLightIn)
		LQ.xyzw	vL2C,	3(pLightIn)
		;// Update input pointer
		IADDIU	pLightIn, pLightIn, 2
	    ;// Calculate A = Vertex - Light Pos 
		SUB.xyz	A,		\vVertex,	vL1P
		;// Normalise A
		MUL.xyz	ASqrd,	A,		A
		ADDy.x	ASqrd,	ASqrd,	ASqrd
		ADDz.x	ASqrd,	ASqrd,	ASqrd
		RSQRT	Q,		ASqrd[x]
		;// Calculate J  = Distnace * A.N  = un normalised A dot N - normalisig done later
	    MULAx.w		ACC,	vNormalX, A
	    MADDAy.w	ACC,	vNormalY, A
		MADDz.w		J,		vNormalZ, A
	    ;// clamp to 0
		MINIx.w		J,	J,	VF00
		;// Light Vertex
		MUL.xyz		S,	 vL1C,	J
		MSUBAw.xyz	ACC, S,		vL1C
		MADDAq.xyz	ACC, S,		Q
		MADDw.xyz	\vColour,	\vColour,	VF00
RANGE CHECK NEEDS TO GO IN HERE
       ;// if (dist - range > 0), out of range
		;// i.e. (range - dist < 0), out of range
		SUBq.w	fRangeMinusDist1,	vL1Position,	Q
		IADDIU	iTemp,	VI00,	0x10
		FMAND	iTemp,	iTemp
		IBNE	iTemp,	VI00,	ENV_LIGHT2\@
ENV_LIGHT2\@:
		;// Check if we've finished
		IBEQ	pLightIn, pLightEnd, ENV_NO_MORE_LIGHTS\@
		;// Update input pointer
		IADDIU	pLightIn, pLightIn, 2
	    ;// Calculate B = Vertex - Light Pos 
		SUB.xyz	B,		\vVertex,	vL1P
		;// Normalise B
		MUL.xyz	BSqrd,	B,		B
		ADDy.x	BSqrd,	BSqrd,	BSqrd
		ADDz.x	BSqrd,	BSqrd,	BSqrd
		ERSQRT	P,		BSqrd[x]
		;// Calculate K  = Distnace * B.N  = un normalised B dot N - normalisig done later
	    MULAx.w		ACC,	vNormalX, B
	    MADDAy.w	ACC,	vNormalY, B
		MADDz.w		K,		vNormalZ, B
	    ;// clamp to 0
		MINIx.w		K,	K,	VF00
		;// Light Vertex
		MUL.xyz		T,	 vL1C,	K
		MSUBAw.xyz	ACC, T,		vL2C
		MFP.w		fP,	 P
		MADDAw.xyz	ACC, T,		fP
		MADDw.xyz	\vColour,	\vColour,	VF00
    	;// back to start of loop
		B	ENV_LIGHTING_LOOP\@
ENV_NO_MORE_LIGHTS\@:
	;// Clamp lights to 0-255
	LOI		255.0
    MINIi.xyz \vColour,	\vColour,	I
.endm
#endif
