;/////////////////////////////////////////////////////////////////////////////
;//
;// Extneded Particled Renderer.
;//
;//	Currently only a simple quad renderer with all the particle
;//	Effects being done on the EE.  If EE time is too much, the
;//	calculation can be moved to vu1
;//
;//	A post process stage of fogging is optionally carried out as required.
;//
;//			Memory layout
;//		  -----------------
;//
;//    0    /---------------------------\ 
;//         |		  FOG TABLE			|     256 QWs
;//   256   |---------------------------|
;//         |		ALPHA NORMAL		|     1 QWs
;//   256   |---------------------------|
;//         |		ALPHA NORMAL		|     1 QWs
;//   257   |---------------------------|
;//         |	    ALPHA GLOWING		|	  1 QWs
;//   258   |---------------------------|
;//         |		Drawing Giftag		|	  1 QWs 
;//   259   |---------------------------|
;//         |  World to Screen Matrix   |	  4 QWs
;//   263   |---------------------------|
;//         |   World to Clip Matrix    |	  4 QWs
;//   267   |---------------------------|
;//         |		    Unused			|	  5 QWs
;//   272   |---------------------------|
;//         |	    Input Buffer 1		|	125	QWs
;//   397   |---------------------------|
;//			|	    Input Buffer 2		|	125 QWs
;//   522   |---------------------------|
;//         |	   Output Buffer 1		|	251 QWs
;//   773   |---------------------------|
;//         |	   Output Buffer 2		|	251 QWs
;//  1024   \---------------------------/ 
;//
;//
;//	Output data (per particle) = ALPHA,ST,RGBA,XYZ,ST,XYZ,ST,XYZ,ST,XYZ = 10
;//	Output buffer size = 1 + NP * 10
;//
;//	Input data per particle = RGBA+XYZ+XDELTA+YDELTA+ST = 5
;// Input buffer size = NP * 5
;//
;//	Free Space = 1024 - 267 = 757
;//
;//	757 = 2*(1 + 13NP + 5NP) = 2 + 30NP
;// 755 = 30NP
;// NP = 25 particles per buffer
;//
;// Input buffer size  = 5NP = 125 qws
;//	Output buffer size = 1 + 10NP = 251 qws
;//
;/////////////////////////////////////////////////////////////////////////////

.init_vf_all
.init_vi_all

.syntax new

--enter
--endenter

;// ==== INCLUDES ==== //;
#include "Asura_PS2_ExtendedParticleDefines.h"

;// ==== DEFINES ====//;
#define ADDITIVE_FOG_REDUCTION	(1.0/3.0)

;// ==== CODE ==== //;
StartUp:
	;// Clear clip flags for VCL
    FCSET 0
    ;// initialise output buffers
    IADDIU  pThisOutputBuffer, VI00, ASURA_PS2_EXPART__ADDR__OUTPUT_BUFFER1
    IADDIU  pNextOutputBuffer, VI00, ASURA_PS2_EXPART__ADDR__OUTPUT_BUFFER2
	;// Load GIFTAG and write it to both output buffers
	LQ.yzw	fGifTag,	ASURA_PS2_EXPART__ADDR__GIFTAG(VI00)
	SQ.yzw	fGifTag,	0(pThisOutputBuffer)
	SQ.yzw	fGifTag,	0(pNextOutputBuffer)
	;// Create kick flag
	IADDIU	i0x8000,	VI00,		0x7fff
	IADDIU	i0x8000,	i0x8000,	1
	;// Load fogging flag
	XITOP	iFoggingRequired
	;// Load camera matricies
	LQ.xyzw	mW2Screen[0],	(ASURA_PS2_EXPART__ADDR__WORLD2SCREEN+0)(VI00)
	LQ.xyzw	mW2Screen[1],	(ASURA_PS2_EXPART__ADDR__WORLD2SCREEN+1)(VI00)
	LQ.xyzw	mW2Screen[2],	(ASURA_PS2_EXPART__ADDR__WORLD2SCREEN+2)(VI00)
	LQ.xyzw	mW2Screen[3],	(ASURA_PS2_EXPART__ADDR__WORLD2SCREEN+3)(VI00)
	LQ.xyzw	mW2Clip[0],		(ASURA_PS2_EXPART__ADDR__WORLD2CLIP+0)(VI00)
	LQ.xyzw	mW2Clip[1],		(ASURA_PS2_EXPART__ADDR__WORLD2CLIP+1)(VI00)
	LQ.xyzw	mW2Clip[2],		(ASURA_PS2_EXPART__ADDR__WORLD2CLIP+2)(VI00)
	LQ.xyzw	mW2Clip[3],		(ASURA_PS2_EXPART__ADDR__WORLD2CLIP+3)(VI00)

StartPacket:
    --cont
	;// Get buffer start address
	XTOP	pIn
	;// Get particle count
	XITOP	iParticleQty
	;// Setup output pointer
	IADDIU	pOut,	pThisOutputBuffer, 1
	;// Create end of output pointer
	IADD	pTemp,		iParticleQty,	iParticleQty	;// iParticleQty * 2
	IADD	pOutEnd,	pTemp,			pTemp			;// iParticleQty * 4
	IADD	pOutEnd,	pOutEnd,		pTemp			;// iParticleQty * 6
	IADD	pOutEnd,	pOutEnd,		pOutEnd			;// iParticleQty * 12
	IADD	pOutEnd,	iParticleQty,	pOutEnd			;// iParticleQty * 13
	IADD	pOutEnd,	pOut,			pOutEnd			
	;// Write particle count to giftag (with EOP)
	IOR		iNLoopEOP,	iParticleQty,	i0x8000
	ISW.x	iNLoopEOP,	0(pThisOutputBuffer)
	;// Setup Q component of STQ vector
	ADDw.z	vSTQ,	VF00,	VF00
	
	;// Render all the particles
ParticleLoop:
    --LoopCS 1, 0
		;// Load the particle atributes
		LQ.xyzw		vRGBA,		0(pIn)
		LQ.xyz		vPos,		1(pIn)
		LQ.xyz		vDX,		2(pIn)
		LQ.xyz		vDY,		3(pIn)
		LQ.xyzw		vSTQs,		4(pIn)
		;// Setup the blend type
		ILW.w	pAlpha,	1(pIn)
		LQ.xyzw	vAlpha,	ASURA_PS2_EXPART__ADDR__ALPHANORMAL(pAlpha)
		SQ.xyzw	vAlpha,	0(pOut)
	;// Create the 1st Vertex
		SUB.xyz		vVertex0,	vPos,		vDX
		SUB.xyz		vVertex0,	vVertex0,	vDY
		ADD.xy		vSTQ,	VF00,	vSTQs
		;// Project to screen space
		MULAw.xyzw	ACC,		mW2Screen[3],	VF00[w]
		MADDAx.xyzw	ACC,		mW2Screen[0],	vVertex0[x]
		MADDAy.xyzw	ACC,		mW2Screen[1],	vVertex0[y]
		MADDz.xyzw	vSSVertex0,	mW2Screen[2],	vVertex0[z]
		;// Perspective divide XYZ and STs.
		DIV			Q,			VF00[w],	vSSVertex0[w]
		MULq.xyz    vSSVertex0,	vSSVertex0,	Q
		MULq.xyz	vSTQOut,	vSTQ,		Q
		;// Project vertex to clip space and clip
		MULAw.xyzw	ACC,		mW2Clip[3],	VF00[w]
		MADDAx.xyzw	ACC,		mW2Clip[0],	vVertex0[x]
		MADDAy.xyzw	ACC,		mW2Clip[1],	vVertex0[y]
		MADDz.xyzw	vCSVertex0,	mW2Clip[2],	vVertex0[z]
		CLIPw.xyz	vCSVertex0,	vCSVertex0
		;// Convert XYZ for the GIF packet
		FTOI4.xyz	vSSVertex0,	vSSVertex0
		;// Store STQ, RGBA and XYZ for this vertex
		SQ.xyz		vSTQOut,	1(pOut)
		SQ.xyzw		vRGBA,		2(pOut)
		SQ.xyz		vSSVertex0,	3(pOut)
		ISW.w		i0x8000,	3(pOut)
	;// Create the 2nd vertex
		ADD.xyz		vVertex1,	vPos,		vDX
		SUB.xyz		vVertex1,	vVertex1,	vDY
		ADDz.x		vSTQ,	VF00,	vSTQs
		;// Project to screen space
		MULAw.xyzw	ACC,		mW2Screen[3],	VF00[w]
		MADDAx.xyzw	ACC,		mW2Screen[0],	vVertex1[x]
		MADDAy.xyzw	ACC,		mW2Screen[1],	vVertex1[y]
		MADDz.xyzw	vSSVertex1,	mW2Screen[2],	vVertex1[z]
		;// Perspective divide XYZ and STs.
		DIV			Q,			VF00[w],	vSSVertex1[w]
		MULq.xyz    vSSVertex1,	vSSVertex1,	Q
		MULq.xyz	vSTQOut,	vSTQ,		Q
		;// Project vertex to clip space and clip
		MULAw.xyzw	ACC,		mW2Clip[3],	VF00[w]
		MADDAx.xyzw	ACC,		mW2Clip[0],	vVertex1[x]
		MADDAy.xyzw	ACC,		mW2Clip[1],	vVertex1[y]
		MADDz.xyzw	vCSVertex1,	mW2Clip[2],	vVertex1[z]
		CLIPw.xyz	vCSVertex1,	vCSVertex1
		;// Convert XYZ for the GIF packet
		FTOI4.xyz	vSSVertex1,	vSSVertex1
		;// Store STQ and XYZ for this vertex
		SQ.xyz		vSTQOut,	4(pOut)
		SQ.xyzw		vRGBA,		5(pOut)
		SQ.xyz		vSSVertex1,	6(pOut)
		ISW.w		i0x8000,	6(pOut)
	;// Create the 3rd vertex
		SUB.xyz		vVertex2,	vPos,		vDX
		ADD.xyz		vVertex2,	vVertex2,	vDY
		ADD.x		vSTQ,	VF00,	vSTQs
		ADDw.y		vSTQ,	VF00,	vSTQs
		;// Project to screen space
		MULAw.xyzw	ACC,		mW2Screen[3],	VF00[w]
		MADDAx.xyzw	ACC,		mW2Screen[0],	vVertex2[x]
		MADDAy.xyzw	ACC,		mW2Screen[1],	vVertex2[y]
		MADDz.xyzw	vSSVertex2,	mW2Screen[2],	vVertex2[z]
		;// Perspective divide XYZ and STs.
		DIV			Q,			VF00[w],	vSSVertex2[w]
		MULq.xyz    vSSVertex2,	vSSVertex2,	Q
		MULq.xyz	vSTQOut,	vSTQ,		Q
		;// Project vertex to clip space and clip
		MULAw.xyzw	ACC,		mW2Clip[3],	VF00[w]
		MADDAx.xyzw	ACC,		mW2Clip[0],	vVertex2[x]
		MADDAy.xyzw	ACC,		mW2Clip[1],	vVertex2[y]
		MADDz.xyzw	vCSVertex2,	mW2Clip[2],	vVertex2[z]
		CLIPw.xyz	vCSVertex2,	vCSVertex2
		;// Convert XYZ for the GIF packet
		FTOI4.xyz	vSSVertex2,	vSSVertex2
		;// Store STQ and XYZ for this vertex
		SQ.xyz		vSTQOut,	7(pOut)
		SQ.xyzw		vRGBA,		8(pOut)
		SQ.xyz		vSSVertex2,	9(pOut)
		;// Store clip result for previous 3
		FCAND		VI01,	0x3ffff
		IADDIU		iClipFlag,	VI01,	0x7fff
		ISW.w		iClipFlag,	9(pOut)
	;// Create the 4th vertex
		ADD.xyz		vVertex3,	vPos,		vDX
		ADD.xyz		vVertex3,	vVertex3,	vDY
		ADDz.x		vSTQ,	VF00,	vSTQs
		;// Project to screen space
		MULAw.xyzw	ACC,		mW2Screen[3],	VF00[w]
		MADDAx.xyzw	ACC,		mW2Screen[0],	vVertex3[x]
		MADDAy.xyzw	ACC,		mW2Screen[1],	vVertex3[y]
		MADDz.xyzw	vSSVertex3,	mW2Screen[2],	vVertex3[z]
		;// Perspective divide XYZ and STs.
		DIV			Q,			VF00[w],	vSSVertex3[w]
		MULq.xyz    vSSVertex3,	vSSVertex3,	Q
		MULq.xyz	vSTQOut,	vSTQ,		Q
		;// Project vertex to clip space and clip
		MULAw.xyzw	ACC,		mW2Clip[3],	VF00[w]
		MADDAx.xyzw	ACC,		mW2Clip[0],	vVertex3[x]
		MADDAy.xyzw	ACC,		mW2Clip[1],	vVertex3[y]
		MADDz.xyzw	vCSVertex3,	mW2Clip[2],	vVertex3[z]
		CLIPw.xyz	vCSVertex3,	vCSVertex3
		;// Convert XYZ for the GIF packet
		FTOI4.xyz	vSSVertex3,	vSSVertex3
		;// Store STQ and XYZ for this vertex
		SQ.xyz		vSTQOut,	10(pOut)
		SQ.xyzw		vRGBA,		11(pOut)
		SQ.xyz		vSSVertex3,	12(pOut)
		;// Store clip result for previous 3
		FCAND		VI01,	0x3ffff
		IADDIU		iClipFlag,	VI01,	0x7fff
		ISW.w		iClipFlag,	12(pOut)
	;// Update pointers and check for completion
		IADDIU	pIn,	pIn,		5
		IADDIU	pOut,	pOut,		13
		IBNE	pOut,	pOutEnd,	ParticleLoop
	;// Finished packet, Apply fog if required else, send it and on to the next
	IBEQ	iFoggingRequired,	vi00,	FinishedPacket
	;// Fog the particles
	IADDIU	pInOut,		pThisOutputBuffer, 1
	IADDIU	iAdditiveMask,	VI00,	ASURA_PS2_EXPART__MASK__ADDITIVE
	;// Traverse output packet applying fogging
ParticleFoggingLoop:
    --LoopCS 1, 0
		;//	Output data (per particle) = ALPHA,ST,RGBA,XYZ,ST,RGBA,XYZ,ST,RGBA,XYZ,ST,RGBA,XYZ = 13QWs
		;// Load the first vertex projected z value
		LQ.z		vVertex,	3(pInOut)
		ITOF0.z		vVertex,	vVertex
		;// Convert Z to palette index (Z/256)
		LOI		(1.0/256.0)
		MULi.z		vVertex,	vVertex,	I
		;// Clamp Z to <256
		LOI		(255.0)
		MINIi.z		vVertex,	vVertex,	I
		;// Convert Z to palette index
		FTOI0.z		vVertex,	vVertex
		MTIR		iIndex,		vVertex[z]
		;// Look up the fog colour for this z depth
		LQ.xyzw		vFogC,		ASURA_PS2_EXPART__ADDR__FOGTABLE(iIndex)
		ITOF0.xyzw	vFogC,		vFogC
		;// Load the vertex colour
		LQ.xyz		vVertC,		2(pInOut)
		ITOF0.xyz	vVertC,		vVertC
		;// Is this particle normal or additive
		ILW.x		iABlend,	0(pInOut)
		IAND		iABlend,	iABlend,	iAdditiveMask
		IBEQ		iABlend,	iAdditiveMask,	FogAdditiveParticle
		;// Blend in fog colour - Vc = (Fc*Fa) + Vc*(1-Fa) = (Fc-Vc * Fa) + Vc
		SUB.xyz		vOutC,		vFogC,		vVertC
		MULw.xyz	vOutC,		vOutC,		vFogC
		LOI		(1.0/128.0)
		MULi.xyz	vOutC,		vOutC,		I
		ADD.xyz		vOutC,		vOutC,		vVertC
		B	FinishedParticleBlend
FogAdditiveParticle:
		;// Fog additive particle - Vc = Vc * (1-Fa) = Vc - (Vc*Fa)
		LOI		(ADDITIVE_FOG_REDUCTION)
		MULi.w		vFogC,		vFogC,		I
		SUBw.xyz	vOutC,		vVertC,		vFogC
		MAXx.xyz	vOutC,		vOutC,		VF00
FinishedParticleBlend:
		;// Write back the colour
		FTOI0.xyz	vOutC,		vOutC
		SQ.xyz		vOutC,		 2(pInOut)
		SQ.xyz		vOutC,		 5(pInOut)
		SQ.xyz		vOutC,		 8(pInOut)
		SQ.xyz		vOutC,		11(pInOut)
		;// Move pointer and check for finish
		IADDIU		pInOut,		pInOut,		13
		IBNE		pInOut,		pOutEnd,	ParticleFoggingLoop
	;// Finished fogging the particles
FinishedPacket:
	--barrier
	XGKICK	pThisOutputBuffer
	IADD	pTemp,				pThisOutputBuffer,	VI00
	IADD	pThisOutputBuffer,	pNextOutputBuffer,	VI00
	IADD	pNextOutputBuffer,	pTemp,				VI00

	B	StartPacket
.end
