;//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;//;	
;//;	FILE			VU1_RainQuadRenderer.vcl
;//;	
;//;	DESCRIPTION		VU1 microcode for rendering Rain Quads
;//;
;//;	AUTHOR			Pete Ellacott 08.12.02
;//;	
;//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;//; 
;//;
;//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;//;
;//;	Constants uploaded at startup:
;//;	------------------------------
;//;	0x000	Drawing Gif Tag
;//;	0x001	Camera 1 Screeen Matrix
;//;	0x005	Camera 1 Clipping Matrix
;//;	0x009	Camera SCISSOR
;//;	0x00a	Camera OFFSET
;//;	0x00b	Camera 2 Screeen Matrix
;//;	0x00f	Camera 2 Clipping Matrix
;//;	0x013	Camera SCISSOR
;//;	0x014	Camera OFFSET
;//;	0x015	Camera 3 Screeen Matrix
;//;	0x019	Camera 3 Clipping Matrix
;//;	0x01d	Camera SCISSOR
;//;	0x01e	Camera OFFSET
;//;	0x01f	Camera 4 Screeen Matrix
;//;	0x023	Camera 4 Clipping Matrix
;//;	0x027	Camera SCISSOR
;//;	0x028	Camera OFFSET
;//;	0x029	Camera Setup Gif Tag
;//;	0x02a		Camera SCISSOR
;//;	0x02b		Camera OFFSET

;//;
;//;	Buffer layout during rendering :
;//;	--------------------------------
;//;	0x02c
;//;			Input Data 0 : RGBA, Vertex 0, Vertex 2, STQ0, STQ2
;//;	0x033
;//;			Input Data 1 : RGBA, Vertex 0, Vertex 2, STQ0, STQ2
;//;	0x040
;//;	0x079
;//;			Output Buffer 0 - (must be multiple of 6) + 1 in size for buffer end check to work
;//;	0x1A6
;//;			Output Buffer 1 - (must be multiple of 6) + 1 in size for buffer end check to work
;//;	0x2d3		
;//;			Output Buffer 2 - (must be multiple of 6) + 1 in size for buffer end check to work
;//;	0x400
;//;
;//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;// I N C L U D E S   &   D E F I N E S ////////////////////////////////////////
#include "vcl_sml.i"
#define CAMERA_DATA_SIZE		(10)
#define OUTPUT_BUFFER_START		(121)
#define OUTPUT_BUFFER_SIZE		(301)
#define CAMERA_SETUP_LIST		(41)
#define DIVISIONS				(32)
#define RECIPRICAL_DIVISIONS	(3.125e-2)
;//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

.syntax new

.init_vf_all
.init_vi_all

    --enter
    --endenter

START:
	;// Setup the output buffer pointers (also where the constant are)
	IADDIU	pThisOutputBuffer, VI00, OUTPUT_BUFFER_START
	IADDIU	pNextOutputBuffer,	pThisOutputBuffer,	OUTPUT_BUFFER_SIZE
	IADDIU	pDrawOutputBuffer,	pNextOutputBuffer,	OUTPUT_BUFFER_SIZE

;// Preload the constants
	;// Load the Drawing GifTag
	LQ	fDrawingGifTag,	0(VI00)

	;// Store the gif tag in each of the buffers now, to save on registers
	SQ		fDrawingGifTag,		0(pThisOutputBuffer)
	SQ		fDrawingGifTag,		0(pNextOutputBuffer)
	SQ		fDrawingGifTag,		0(pDrawOutputBuffer)

	;// Get the number of cameras from ITOP
	XITOP	iCameraQty

	;// Set up useful i0x8000 register
	IADDIU	i0X8000,	VI00,	0x7fff
	IADDIU	i0X8000,	i0X8000,	1

	--cont

RAIN_QUAD_RENDERER:
;// Get the input buffer address
    XTOP	pInputBuffer

;// We are not lighting the rain, but we still need an RGBA so the Q values can get set corrently (RGBA is real RGBAQ)
	LQ		fRgba,		0(pInputBuffer)

;// Get camera visibility flags
	ILW.w	iVisibility, 1(pInputBuffer)

;// Setup camera counter, data pointer & visibility mask
	IADDIU	iCameraCount,		iCameraQty, 0
	IADDIU	pCameraData,		VI00,		1
	IADDIU	iVisibilityMask,	VI00,		1

;// Per camera loop
RAIN_QUAD_CAMERA_LOOP:
	IAND	iTemp,	iVisibility, iVisibilityMask
	;// Is it visible to this camera
	IBEQ	iTemp, VI00, NEXT_CAMERA

;// It's visibility
	;// Load the first cameras World->Screen matrix
	MatrixLoad	matLocalToScreen,	0,	pCameraData

	;// Load the first cameras World->Clip matrix
	MatrixLoad	matLocalToClip,		4,	pCameraData

	;// Setup the first cameras SETUP drawlist
	LQ		fScissor,	8(pCameraData)
	LQ		fOffset,	9(pCameraData)
	SQ		fScissor,	(CAMERA_SETUP_LIST+1)(VI00)
	SQ		fOffset,	(CAMERA_SETUP_LIST+2)(VI00)

;// Clear the clipping flags
	FCSET   0

	;// Calculate the quad size
	LQ		fStartPoint,	1(pInputBuffer)
	LQ		fEndPoint,		2(pInputBuffer)
	SUB.xyz	fSize,	fEndPoint,	fStartPoint

	LQ		fSTQStart,		3(pInputBuffer)
	LQ		fSTQEnd,		4(pInputBuffer)
	SUB.xy	fSTQSize,	fSTQEnd,	fSTQStart

;// Sub divide
	IADDIU		iDivisions,	VI00, DIVISIONS
	LOI			RECIPRICAL_DIVISIONS
	MULi.xyz	fStep,	fSize,	I
;// Calculate the S and T steps
	MULi.xy		fSTQStep,	fSTQSize,	I
;// Create 2 output pointers one for this row (A) and one for the previous (B)
	IADDIU	pOutA,	pThisOutputBuffer,	1
	IADDIU	pOutB,	pThisOutputBuffer,	4
;// Create end of buffer pointer
	IADDIU	pBufferEnd,	pThisOutputBuffer,	OUTPUT_BUFFER_SIZE
;// Setup our initial vertex & ST
	ADD.xyz		fVertex,	fStartPoint, VF00
	ADD.xyz		fSTQ,		fSTQStart,	VF00
;// Keep track of how many verts we've written through output pointer a and b
	SUB.xyzw	fVertexCounters, VF00, VF00
;// Clear the clipping flags
	FCSET   0
;// a counter for the row (1 more than the number of division for start+end)
	IADDIU	iRowCount,	VI00,	(DIVISIONS+1)
;// Kick first 2 verts of row ... degerate triangles (0xc000)
	ISUBIU	iKick,	VI00,	0x4000

;// Kick the camera setup
	IADDIU pKickMe, VI00, CAMERA_SETUP_LIST
	XGKICK pKickMe

;// Create verts for the first row ( don't need to write previous row verts )
FIRST_ROW_LOOP:
	--LoopCS    2,0
	;// Light the vertex
		SQ		fRgba,		 1(pOutA) 
	;// Transform the verts to screen space
		MatrixMultiplyVertexW1	fScrnPos,	matLocalToScreen, fVertex
	;// Transform the verts into Clip Space
		MatrixMultiplyVertexW1	fClipPos,	matLocalToClip,	fVertex
	;// Perspective Divide the Vert and STQ
		DIV	Q,		VF00[w],	fScrnPos[w]
		MULq		fScrnPos,	fScrnPos,	Q
		MULq.xyz	fSTQOut,	fSTQ,		Q
	;// Convert XYZ and RGB for the GIF packet
		FTOI4	fScrnPos,	fScrnPos
	;// Output STQ, RGBA, XYZ - for this row
		SQ.xyz	fSTQOut,		 0(pOutA) 
		SQ.xyz	fScrnPos,		 2(pOutA)
	;// Reduce kick from last vert
		IADD	iKick,	iKick,	iKick
	;// Check clip flags
		CLIPw.xyz	fClipPos,	fClipPos
		FCAND		VI01,		0x3f
		IBNE		VI01,	VI00,	FIRST_ROW_LOOP_CLIPVERTEX	
FIRST_ROW_LOOP_CLIPVERTEX_DONE:
		;// Store the kick flag for this vert
		ISW.w		iKick,	2(pOutA)w
	;// Increment the vertex counter / decrement the vertex counter
		ADDw.y		fVertexCounters,	fVertexCounters,	VF00
		ISUBIU		iRowCount,	iRowCount,	1
	;// Move the output pointer along 2 verts worth to leave space for the fill in vert from the next row
		IADDIU		pOutA,	pOutA,	6
	;// Increment vertex horizontally and S check for end of row
		ADD.xz	fVertex,	fVertex,	fStep
		ADD.x	fSTQ,		fSTQ,		fSTQStep

		IBNE    iRowCount,	VI00,	FIRST_ROW_LOOP
	;// Create a column counter - 1 less than the number of divisions (start and end handled sperately)
		ISUBIU	iColumnCount,	iDivisions,	1
COLUMN_LOOP:
	--LoopCS    1,0
	;// Increment vertex vertically
		ADD.y	fVertex,	fVertex,	fStep
		ADD.y	fSTQ,		fSTQ,		fSTQStep
	;// Reset vertex horizontally
		ADD.xz	fVertex,	fStartPoint,	VF00
		ADD.x	fSTQ,		VF00,			fSTQStart
	;// Reset the row counter
		IADDIU	iRowCount,	iDivisions,	1
	;// Kick first 2 verts of row ... degerate triangles (0xc000)
		ISUBIU	iKick,	VI00,	0x4000

ROW_LOOP:
	--LoopCS    2,0
		SQ		fRgba,	 1(pOutA) 
		SQ		fRgba,	 1(pOutB) 
	;// Transform the vertex to screen space
		MatrixMultiplyVertexW1	fScrnPos,	matLocalToScreen, fVertex
	;// Transform the vertex into Clip Space
		MatrixMultiplyVertexW1	fClipPos,	matLocalToClip,	fVertex
	;// Perspective Divide the Vert and STQ
		DIV	Q,		VF00[w],	fScrnPos[w]
		MULq		fScrnPos,	fScrnPos,	Q
		MULq.xyz	fSTQOut,	fSTQ,		Q
	;// Convert XYZ and RGB for the GIF packet
		FTOI4	fScrnPos,	fScrnPos
	;// Output STQ, RGBA, XYZ - for this row and the next
		SQ.xyz	fSTQOut,		 0(pOutA) 
		SQ.xyz	fScrnPos,		 2(pOutA)
		SQ.xyz	fSTQOut,		 0(pOutB) 
		SQ.xyz	fScrnPos,		 2(pOutB)
	;// Reduce kick from last vert
		IADD	iKick,	iKick,	iKick
	;// Check clip flags
		CLIPw.xyz	fClipPos,	fClipPos
		FCAND		VI01,		0x3f
		IBNE		VI01,	VI00,	ROW_LOOP_CLIPVERTEX	
ROW_LOOP_CLIPVERTEX_DONE:

		;// Store the kick flag for this vert
		ISW.w	iKick,	2(pOutA)w
		;// combine the previous verts clip flag in the back fill
		ILW.w	iOldKick,	-1(pOutB)w
		IADD	iFillInKick,	iOldKick,	iOldKick
		IOR		iFillInKick,	iKick,		iFillInKick
		ISW.w	iFillInKick,	2(pOutB)w
		;// Update the previous vert in the back fill
		IOR		iOldKick,		iOldKick,	iKick
		ISW.w	iOldKick,	-1(pOutB)w
	;// Increment the vertex counters / decrement the vertex counter
		ADDw.yz	fVertexCounters,	fVertexCounters,	VF00
		ISUBIU	iRowCount,	iRowCount,	1
	;// Move the output pointer along 2 verts worth to leave space for the fill in vert from the next row
		IADDIU	pOutA,	pOutA,	6
		IADDIU	pOutB,	pOutB,	6
	;// Check for output buffer full
		IBEQ	pOutA, pBufferEnd, ROW_LOOP_OUPUTA_BUFFER_FULL
ROW_LOOP_OUTPUTA_BUFFER_DONE:
		IBEQ	pOutB, pBufferEnd, ROW_LOOP_OUPUTB_BUFFER_FULL
ROW_LOOP_OUTPUTB_BUFFER_DONE:
	;// Increment vertex horizontally and check for end of row
		ADD.xz	fVertex,	fVertex,	fStep
		ADD.x	fSTQ,		fSTQ,		fSTQStep
		IBNE    iRowCount,	VI00,	ROW_LOOP
	;// End of the row.. check for the end of the column
		ISUBIU	iColumnCount,	iColumnCount,	1
		IBNE    iColumnCount,	VI00,	COLUMN_LOOP

;// Now fill in the last row of verts
		IADDIU	iRowCount,	iDivisions,	1
	;// Increment vertex vertically
		ADD.y	fVertex,	fVertex,	fStep
		ADD.y	fSTQ,		fSTQ,		fSTQStep
	;// Reset vertex horizontally
		ADD.xz	fVertex,	fStartPoint,	VF00
		ADD.x	fSTQ,		VF00,			fSTQStart
	;// Kick first 2 verts of row ... degerate triangles (0xc000)
		ISUBIU	iKick,	VI00,	0x4000
LAST_ROW_LOOP:
	--LoopCS    2,0
		SQ		fRgba,		 1(pOutB) 
	;// Transform the verts to screen space
		MatrixMultiplyVertexW1	fScrnPos,	matLocalToScreen, fVertex
	;// Transform the verts into Clip Space
		MatrixMultiplyVertexW1	fClipPos,	matLocalToClip,	fVertex
	;// Perspective Divide the Vert and STQ
		DIV	Q,		VF00[w],	fScrnPos[w]
		MULq		fScrnPos,	fScrnPos,	Q
		MULq.xyz	fSTQOut,	fSTQ,		Q
	;// Convert XYZ and RGB for the GIF packet
		FTOI4	fScrnPos,	fScrnPos
	;// Output STQ, RGBA, XYZ - for this row
		SQ.xyz	fSTQOut,		 0(pOutB) 
		SQ.xyz	fScrnPos,		 2(pOutB)
	;// Reduce kick from last vert
		IADD	iKick,	iKick,	iKick
	;// Check clip flags
		CLIPw.xyz	fClipPos,	fClipPos
		FCAND		VI01,		0x3f
		IBNE		VI01,	VI00,	LAST_ROW_LOOP_CLIPVERTEX	
LAST_ROW_LOOP_CLIPVERTEX_DONE:
		;// combine the previous verts clip flag in the back fill
		ILW.w	iOldKick,		-1(pOutB)w
		IADD	iFillInKick,	iOldKick,	iOldKick
		IOR		iFillInKick,	iKick,		iFillInKick
		ISW.w	iFillInKick,	2(pOutB)w
		;// Update the previous vert in the back fill
		IOR		iOldKick,		iOldKick,	iKick
		ISW.w	iOldKick,		-1(pOutB)w
	;// Increment the vertex counter / decrement the vertex counter
		ADDw.z	fVertexCounters,	fVertexCounters,	VF00
		ISUBIU	iRowCount,	iRowCount,	1
	;// Move the output pointer along 2 verts worth to leave space for the fill in vert from the next row
		IADDIU	pOutB,	pOutB,	6
		IBEQ	pOutB, pBufferEnd, LAST_ROW_LOOP_OUPUTB_BUFFER_FULL
LAST_ROW_LOOP_OUTPUTB_BUFFER_DONE:
	;// Increment vertex horizontally and check for end of row
		ADD.xz	fVertex,	fVertex,	fStep
		ADD.x	fSTQ,		fSTQ,		fSTQStep
		IBNE    iRowCount,	VI00,	LAST_ROW_LOOP

;// All done - write out the GIF tag and then kick it
		ADDy.x	fVertexCounters,	VF00,			 fVertexCounters
		ADDz.x	fVertexCounters,	fVertexCounters, fVertexCounters
		FTOI0.x fVertexCounters,	fVertexCounters
		MTIR	iVertexCount,		fVertexCounters[X]
		IOR		iVertexCount,		iVertexCount,	i0x8000
		ISW.x	iVertexCount,		0(pThisOutputBuffer)

		--barrier

		;// And Send this strip to the GS
		XGKICK	pThisOutputBuffer

		;// Swap output buffers
		IADDIU	iTemp,				pThisOutputBuffer,	0
		IADDIU	pThisOutputBuffer,	pNextOutputBuffer,	0
		IADDIU	pNextOutputBuffer,	iTemp,				0

NEXT_CAMERA:
	;// Decrement camera count and check for all cameras done
	ISUBIU	iCameraCount, iCameraCount, 1
	IBEQ	iCameraCount, VI00, FINISHED
	;// More cameras - move the camera data pointer along
	IADDIU	pCameraData, pCameraData, CAMERA_DATA_SIZE
	;// Add make the next cameras visibility flag
	IADD	iVisibilityMask, iVisibilityMask, iVisibilityMask

FINISHED:

    --cont

    B    RAIN_QUAD_RENDERER

FIRST_ROW_LOOP_CLIPVERTEX:
		;// Set vertex kick to 0xc000
		ISUBIU	iKick,	VI00,	0x4000
		B		FIRST_ROW_LOOP_CLIPVERTEX_DONE

ROW_LOOP_OUPUTA_BUFFER_FULL:
		;// Copy Last vrtex into the start of the next buffer
		LQ		fTempSTQ,	-6(pOutA)
		LQ		fTempRGB,	-5(pOutA)
		LQ		fTempXYZ,	-4(pOutA)
		SQ		fTempSTQ,	1(pNextOutputBuffer)
		SQ		fTempRGB,	2(pNextOutputBuffer)
		SQ		fTempXYZ,	3(pNextOutputBuffer)
		;// Move pointer A into the next output buffer
		IADDIU	pOutA,	pNextOutputBuffer,	7
		;// Update end of buffer pointer ready for checking pointer B
		IADDIU	pBufferEnd,	pBufferEnd,	3
		;// Store and reset output a counter
		ADDy.x	fVertexCounters,	VF00,	fVertexCounters
		ADDw.y	fVertexCounters,	VF00,	VF00		;// Set to 1 - we copied the last vertex across
		;// Back to the loop
		B	ROW_LOOP_OUTPUTA_BUFFER_DONE

ROW_LOOP_OUPUTB_BUFFER_FULL:
		;// Copy Last vrtex into the start of the next buffer
		LQ		fTempSTQ,	-6(pOutB)
		LQ		fTempRGB,	-5(pOutB)
		LQ		fTempXYZ,	-4(pOutb)
		SQ		fTempSTQ,	4(pNextOutputBuffer)
		SQ		fTempRGB,	5(pNextOutputBuffer)
		SQ		fTempXYZ,	6(pNextOutputBuffer)
		;// Move pointer B into the next output buffer
		IADDIU	pOutB,	pNextOutputBuffer,	10
		;// Update end of buffer pointer ready for checking pointer A
		IADDIU	pBufferEnd,	pNextOutputBuffer,	OUTPUT_BUFFER_SIZE
		;// and the vert counter
		ADDz.x	fVertexCounters,	fVertexCounters, fVertexCounters
		ADDw.z	fVertexCounters,	VF00,	VF00		;// Set to 1 - we copied the last vertex across
		FTOI0.x fVertexCounters,	fVertexCounters
		MTIR	iVertexCount,		fVertexCounters[X]
		IOR		iVertexCount,	iVertexCount,	i0x8000
		ISW.x	iVertexCount,	0(pThisOutputBuffer)
		;// And Send that buffer to the GS
		XGKICK	pThisOutputBuffer
		;// Swap the output buffers
		IADD	pTemp, pThisOutputBuffer,	VI00
		IADD	pThisOutputBuffer, pNextOutputBuffer,	VI00
		IADD	pNextOutputBuffer, pDrawOutputBuffer,	VI00
		IADD	pDrawOutputBuffer, pTemp,	VI00
		;// Back to the loop
		B	ROW_LOOP_OUTPUTB_BUFFER_DONE

ROW_LOOP_CLIPVERTEX:
		;// Set vertex kick to 0xc000
		ISUBIU	iKick,	VI00,	0x4000
		B	ROW_LOOP_CLIPVERTEX_DONE

LAST_ROW_LOOP_OUPUTB_BUFFER_FULL:
		;// Copy Last vrtex into the start of the next buffer
		LQ		fTempSTQ,	-6(pOutB)
		LQ		fTempRGB,	-5(pOutB)
		LQ		fTempXYZ,	-4(pOutb)
		SQ		fTempSTQ,	4(pNextOutputBuffer)
		SQ		fTempRGB,	5(pNextOutputBuffer)
		SQ		fTempXYZ,	6(pNextOutputBuffer)
		;// Move pointer B into the next output buffer
		IADDIU	pOutB,	pNextOutputBuffer,	10
		;// Update end of buffer pointer ready for checking pointer A
		IADDIU	pBufferEnd,	pNextOutputBuffer,	OUTPUT_BUFFER_SIZE
		;// and the vert counter
		ADDz.x	fVertexCounters,	fVertexCounters, fVertexCounters
		ADDw.z	fVertexCounters,	VF00,	VF00		;// Set to 1 - we copied the last vertex across
		FTOI0.x fVertexCounters,	fVertexCounters
		MTIR	iVertexCount,		fVertexCounters[X]
		IOR		iVertexCount,	iVertexCount,	i0x8000
		ISW.x	iVertexCount,	0(pThisOutputBuffer)
		;// And Send that buffer to the GS
		XGKICK	pThisOutputBuffer
		;// Swap the output buffers
		IADD	pTemp, pThisOutputBuffer,	VI00
		IADD	pThisOutputBuffer, pNextOutputBuffer,	VI00
		IADD	pNextOutputBuffer, pDrawOutputBuffer,	VI00
		IADD	pDrawOutputBuffer, pTemp,	VI00
		;// Back to the loop
		B	LAST_ROW_LOOP_OUTPUTB_BUFFER_DONE

LAST_ROW_LOOP_CLIPVERTEX:
		;// Set vertex kick to 0xc000
		ISUBIU	iKick,	VI00,	0x4000
		B		LAST_ROW_LOOP_CLIPVERTEX_DONE

	--exit
	--endexit

.end

