;//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;//;
;//;    FILE            VU1_Skinning_Dredd.vcl
;//;
;//;    DESCRIPTION     VU1 microcode for skinning and rendering 2 pass dredd characters
;//;					in 1 to 4 cameras and rendering 0 to 4 shadows per character.
;//;					
;//;					Also contains a second quad renderer for rendering the shadow
;//;					textures to screen
;//;
;//;    AUTHORS         Pete Ellacott
;//;
;//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;//; 
;//;	The quad renderer can only be used directly after the character renderer because
;//;	it use data in memory and registers setup by the character renderer
;//;
;//;	(PSJE: 20-11-02) Extended to use variable number of bones upto a maximum of :...
;//;
;//;	The input buffer is used to hold camera data and shadow data once skinning has been performed.
;//;	This gives a minimum size for the input buffer and so a maximum number of bones.
;//;	Per camera data which is moved = 12 Qws (stored 2 Qws per vertex ) = 6 Verts per camera
;//;	Per shadow data which is moved = 5 Qws (stored 2 Qws per vertex ) = 3 Verts per camera
;//;	So in the maximum case of 4 cameras and 4 shadows = 6*4 + 3*4 = minimum of 36 verts per input buffer
;//;	This gives a maximum bone count of : 108 Bones
;//;
;//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;//;
;//;	Calculation for buffer sizes :
;//;	------------------------------
;//;	Camera + Pass 1 + Pass 2 Setup : 13QWs 
;//;	OutTags : 2QWs ( 1 gif tag per output buffer )
;//;    Bone Buffer : nBones * 4QWs
;//;	Input Vert Size = 5Qws ( Position, Normal, UV, Weight, Bone Index )
;//;	Output Vert Size = 3Qws ( Position, RGBA, UV )
;//;	VertsPerBuffer = (1024Qws - (BoneBuffer + Setup + OutTags)) / (8*2) = (1009 - BoneBuffer)/16
;//;	Input Buffer Size = 5Qws * VertsPerBuffer
;//;	Output Buffer Size = 1Qw + 3Qws * VertsPerBuffer
;//;
;//;	Constants uploaded at startup:
;//;	------------------------------
;//;    (BoneBuffer - 21)   Pass 1 TEST
;//;	(BoneBuffer - 20)	Pass 1 Setup Gif Tag
;//;	(BoneBuffer - 19)	Pass 1 MIPTBP1
;//;	(BoneBuffer - 18)	Pass 1 TEX1
;//;	(BoneBuffer - 17)	Pass 1 TEX0
;//;	(BoneBuffer - 16)	Pass 2 Setup Gif Tag
;//;	(BoneBuffer - 15)	Pass 2 Setup TEX1
;//;	(BoneBuffer - 14)	Pass 2 Setup TEX0
;//;	(BoneBuffer - 13)	Pass 2 Setup TEST
;//;	(BoneBuffer - 12)	Shadow Setup Gif Tag
;//;	(BoneBuffer - 11)	Shadow RGBAQ register
;//;	(BoneBuffer - 10)	Pass 1 Drawing Gif Tag - Camera count in NLOOP (x component)
;//;	(BoneBuffer -  9)	Pass 2 Drawing Gif Tag - Shadow casting light count in NLOOP (x component)
;//;	(BoneBuffer -  8)	Shadow Drawing Gif Tag
;//;	(BoneBuffer -  7)	Light Normals (X Components )
;//;	(BoneBuffer -  6)	Light Normals (Y Components )
;//;	(BoneBuffer -  5)	Light Normals (Y Components )
;//;	(BoneBuffer -  4)	Light Colour 0
;//;	(BoneBuffer -  3)	Light Colour 1
;//;	(BoneBuffer -  2)	Light Colour 2
;//;	(BoneBuffer -  1)	Specular Colour
;//;	(BoneBuffer +  0)	Camera 1 Screeen Matrix
;//;	(BoneBuffer +  4)	Camera 1 Clipping Matrix
;//;	(BoneBuffer +  8)	Camera 1 Specular Eye Vector
;//;	(BoneBuffer +  9)	Camera 1 Specular Half Vector
;//;	(BoneBuffer + 10)	Camera 1 Scissor Value
;//;	(BoneBuffer + 11)	Camera 1 XYOffset Value
;//;	
;//;	Optional data depending on camera count
;//;	---------------------------------------
;//;	4Qwds	Camera 2 Screeen Matrix
;//;	4Qwds	Camera 2 Clipping Matrix
;//;	1Qwds	Camera 2 Specular Eye Vector
;//;	1Qwds	Camera 2 Specular Half Vector
;//;	1Qwds	Camera 2 Scissor Value
;//;	1Qwds	Camera 2 XYOffset Value
;//;	4Qwds	Camera 3 Screeen Matrix
;//;	4Qwds	Camera 3 Clipping Matrix
;//;	1Qwds	Camera 3 Specular Eye Vector
;//;	1Qwds	Camera 3 Specular Half Vector
;//;	1Qwds	Camera 3 Scissor Value
;//;	1Qwds	Camera 3 XYOffset Value
;//;	4Qwds	Camera 4 Screeen Matrix
;//;	4Qwds	Camera 4 Clipping Matrix
;//;	1Qwds	Camera 4 Specular Eye Vector
;//;	1Qwds	Camera 4 Specular Half Vector
;//;	1Qwds	Camera 4 Scissor Value
;//;	1Qwds	Camera 4 XYOffset Value
;//;
;//;	Optional data depending on shadow count
;//;	---------------------------------------
;//;	1Qwds	Shadow 1 Frame Setting
;//;	4Qwds	Shadow 1 Screeen Matrix
;//;	1Qwds	Shadow 2 Frame Setting
;//;	4Qwds	Shadow 2 Screeen Matrix
;//;	1Qwds	Shadow 3 Frame Setting
;//;	4Qwds	Shadow 3 Screeen Matrix
;//;	1Qwds	Shadow 5 Frame Setting
;//;	4Qwds	Shadow 4 Screeen Matrix
;//;
;//;	0x000	Bone Array		: BoneMatrix[MaxBones] - Loaded at startup, after the constants have been set
;//;
;//;
;//;	Buffer layout during rendering :
;//;	--------------------------------
;//;	0x000
;//;			Bone Array		: BoneMatrix[nBones]
;//;	0xXXX
;//;			Output Buffer 0	: GifTag,  (STQ,RGB,XYZ)[VertQty]
;//;	0xXXX
;//;			Output Buffer 1	: GifTag,  (STQ,RGB,XYZ)[VertQty]
;//;	0xXXX
;//;			Input Buffer 0	: Initially - (Normal,BoneIndex,BoneWeight,Position,UV)[VertQty]
;//;			Input Buffer 0	: After Skinning -	(Normal,	Camera1 Matrix 1, Camera1 Matrix 2,	Position, UV),
;//;												(Normal,	Camera1 Matrix 3, Camera1 Matrix 4,	Position, UV),
;//;												(Normal,	Camera1 Clip 1,	  Camera1 Clip 2,	Position, UV),
;//;												(Normal,	Camera1 Clip 3,	  Camera1 Clip 4,	Position, UV),
;//;												(Normal,	Camera2 Matrix 1, Camera2 Matrix 2,	Position, UV),
;//;												(Normal,	Camera2 Matrix 3, Camera2 Matrix 4,	Position, UV),
;//;												(Normal,	Camera2 Clip 1,	  Camera2 Clip 2,	Position, UV),
;//;												(Normal,	Camera2 Clip 3,	  Camera2 Clip 4,	Position, UV),
;//;												(Normal,	Camera3 Matrix 1, Camera3 Matrix 2,	Position, UV),
;//;												(Normal,	Camera3 Matrix 3, Camera3 Matrix 4,	Position, UV),
;//;												(Normal,	Camera3 Clip 1,	  Camera3 Clip 2,	Position, UV),
;//;												(Normal,	Camera3 Clip 3,	  Camera3 Clip 4,	Position, UV),
;//;												(Normal,	Camera4 Matrix 1, Camera4 Matrix 2,	Position, UV),
;//;												(Normal,	Camera4 Matrix 3, Camera4 Matrix 4,	Position, UV),
;//;												(Normal,	Camera4 Clip 1,	  Camera4 Clip 2,	Position, UV),
;//;												(Normal,	Camera4 Clip 3,	  Camera4 Clip 4,	Position, UV),
;//;												(Normal,	unused,			  unused,			Position, UV)[VertQty-16]
;//;	0xXXX
;//;			Input Buffer 1	: See input buffer 0 for layout
;//;	0x3f3
;//;			Camera Setup + First Pass	: GifTag, Scissor, XYOffset, Tex1, Tex0, Test )
;//;	0x3f9
;//;			Second Pass setup	: GifTag, Tex1, Tex0, Test
;//;	0x3fd
;//;			Shadow Setup	: Gif Tag, RGBAQ, Frame_2
;//;	0x400
;//;
;//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;// I N C L U D E S   &   D E F I N E S ////////////////////////////////////////
#include "Asura_PS2_CharDefines.h"

;// I N C L U D E S   &   D E F I N E S ////////////////////////////////////////
#include "Asura_PS2_CharDefines.h"
#define PASS1_SETUP_BUFFER			(1024 - ASURA_PS2_SKINNING__QWS_TOTAL_SETUP)
#define PASS2_SETUP_BUFFER			(PASS1_SETUP_BUFFER + ASURA_PS2_SKINNING__QWS_PASS1_SETUP)
#define SHADOW_SETUP_BUFFER			(PASS2_SETUP_BUFFER + ASURA_PS2_SKINNING__QWS_PASS2_SETUP)

;//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

.syntax new

.init_vf_all
.init_vi VI01, VI02, VI03, VI04, VI05, VI06, VI07, VI08, VI09, VI10, VI11, VI13

#define REG_THIS_OUTPUT_BUFFER VI15
#define REG_NEXT_OUTPUT_BUFFER VI14
#define REG_NUM_CAMERAS        VI12

.macro ConvertInputNormal nrm
	ITOF4.xyz \nrm, \nrm
	LOI -1.0
	ADDAi.xyz ACC, VF00, I
	LOI 0.125
	MADDi.xyz \nrm, \nrm, I  ;// N = (1/8)N - 1
	;// Convert normal ADC degeneracy flag
	ITOF0.w \nrm, \nrm
	LOI	0.5
	ADDi.w	\nrm, \nrm, I
.endm

    --enter
    --endenter

START:
	;// Get the address of the end of the bone matrix (and the start of the constants)
	XITOP	REG_THIS_OUTPUT_BUFFER
	ISUBIU	pConstants,		REG_THIS_OUTPUT_BUFFER,	ASURA_PS2_SKINNING__QWS_CONSTANTS_GRAPHICS_NOVEL
	;// We are switching output buffers manually so we can find our camera data 
	ILW.x	iOutputBufferSize,	13(pConstants)x
	IADD	REG_NEXT_OUTPUT_BUFFER,	REG_THIS_OUTPUT_BUFFER,		iOutputBufferSize
;// Preload registers with common data
;// Pass 1 Setup List
	LQ		fPass1Test, 		0(pConstants)
    LQ		fGifTag,			1(pConstants)
    LQ      fMIPTBP1,           2(pConstants)
	LQ		fTex1,				3(pConstants)
	LQ		fTex0,				4(pConstants)
	SQ		fGifTag,			(PASS1_SETUP_BUFFER+0)(VI00)
	SQ		fMIPTBP1,		    (PASS1_SETUP_BUFFER+3)(VI00)
	SQ		fTex1,				(PASS1_SETUP_BUFFER+4)(VI00)
	SQ		fTex0,				(PASS1_SETUP_BUFFER+5)(VI00)
;//	Pass 2 Setup List
	LQ		fPass2SetupGifTag,	5(pConstants)
	LQ		fTex1,				6(pConstants)
	LQ		fTex0,				7(pConstants)
	LQ		fTest,				8(pConstants)
//	SQ		fGifTag,			(PASS2_SETUP_BUFFER+0)(VI00)
	SQ		fTex1,				(PASS2_SETUP_BUFFER+1)(VI00)
	SQ		fTex0,				(PASS2_SETUP_BUFFER+2)(VI00)
	SQ		fTest,				(PASS2_SETUP_BUFFER+3)(VI00)
// Shadow Setup List
	LQ		fGifTag,			9(pConstants)
	LQ		fRGBAQ,				10(pConstants)
	SQ		fGifTag,			(SHADOW_SETUP_BUFFER+0)(VI00)
	SQ		fRGBAQ,				(SHADOW_SETUP_BUFFER+1)(VI00)
// Load Drawing GifTags
	LQ.yzw	fPass1_DrawGIF,		11(pConstants)
	LQ.yzw	fPass2_DrawGIF,		12(pConstants)
	LQ.yzw	fShadow_DrawGIF,	13(pConstants)
;// Load the Light data
    LQ.xyz	fLightNormalXs,		14(pConstants)
	LQ.xyz	fLightNormalYs,		15(pConstants)
	LQ.xyz	fLightNormalZs,		16(pConstants)
    LQ.xyz	fLightColour1,		17(pConstants)
	LQ.xyz	fLightColour2,		18(pConstants)
	LQ.xyz	fLightColour3,		19(pConstants)
    LQ		fSpecularColour,	20(pConstants)
	;// Load camera count from Pass 1 Drawing Gif Tag - NLOOP member
	ILW.x	REG_NUM_CAMERAS,	11(pConstants)x
	;// Load shadow count from Pass 2 Drawing Gif Tag - NLOOP member
	ILW.x	iShadowQty,			12(pConstants)x
	;// Load the World->Screen matrix
    LQ		fWorldToScreen1,	21(pConstants)
    LQ		fWorldToScreen2,	22(pConstants)
	LQ		fWorldToScreen3,	23(pConstants)
    LQ		fWorldToScreen4,	24(pConstants)
	;// Load the World->Clip matrix
    LQ		fWorldToClip1,		25(pConstants)
    LQ		fWorldToClip2,		26(pConstants)
    LQ		fWorldToClip3,		27(pConstants)
    LQ		fWorldToClip4,		28(pConstants)
	;// Load first cameras Eye Vector & Half Vector
	LQ.xyz	fToEye,				29(pConstants)
	LQ.xyz	fSpecularHalf,		30(pConstants)
	;// Load first cameras Scissor and xyoffset values
	LQ		fSCISSOR,			31(pConstants)
	LQ		fXYOFFSET,			32(pConstants)
	;// Complete the Pass 1 Setup List
	SQ		fSCISSOR,			(PASS1_SETUP_BUFFER+1)(VI00)
	SQ		fXYOFFSET,			(PASS1_SETUP_BUFFER+2)(VI00)

	--cont                    

;//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

STRIP_SETUP:
;// Get which double buffer we are in (where our memory starts)
    XTOP	pInputBuffer

;// Read the vertex count from ITOP
	XITOP	iVertexQty

;// SKINNING
;//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;// Counter for vertex quantity
	IADD	iVertexCount,	iVertexQty,	VI00
;// Input and Output pointers for skinned verts and normals (These are after the uvs)
	IADD	pIn,	pInputBuffer,	VI00

SKINNING_VERTEX_LOOP:
		--LoopCS    3,0
		;// load in our data
		LQ.xyzw	fNormal,		0(pIn)
		LQ.xyzw	fBoneIds,		1(pIn)
		LQ.xyzw	fBoneWeights,	2(pIn)
		LQ.xyz	fVertexIn,		3(pIn)
		;// Unpack Normal & weight
		ConvertInputNormal	fNormal
		ITOF15.xyzw			fBoneWeights, fBoneWeights
		;// set output point to (0, 0, 0, X)
		MOVE.xyz	fVertexOut,	VF00
		;// Extract the matrix address for this bone
		MTIR    iBoneIndex,	fBoneIds[x]
		;// Load first bone matrix.
		LQ.xyz	fBoneMatrix1,	0(iBoneIndex)
		LQ.xyz	fBoneMatrix2,	1(iBoneIndex)
		LQ.xyz	fBoneMatrix3,	2(iBoneIndex)
		LQ.xyz	fBoneMatrix4,	3(iBoneIndex)
		;// transform normal to output reg
		MULAx.xyz  ACC,		fBoneMatrix1,	fNormal
		MADDAy.xyz ACC,		fBoneMatrix2,	fNormal
		MADDz.xyz  fNormal,	fBoneMatrix3,	fNormal
		;// Setup bone counter for maximum number of bones
		IADDIU	iBoneCount, VI00, 4
SKINNING_BONE_LOOP:
			--LoopCS	1,0 ;// minimum 1 time

			;// transform point to temp reg 
			MULAx.xyz	ACC,	fBoneMatrix1,	fVertexIn
			MADDAy.xyz	ACC,	fBoneMatrix2,	fVertexIn
			MADDAz.xyz	ACC,	fBoneMatrix3,	fVertexIn
			MADDw.xyz	fTemp,	fBoneMatrix4,	VF00
			;// scale transformed point by weight and add to output point
			MULx.xyz	fTemp,		fTemp,		fBoneWeights
			ADD.xyz		fVertexOut,	fVertexOut,	fTemp
			;// Rotate bone indicies and weights ready for next bone.
			MR32    fBoneIds,		fBoneIds
			MR32    fBoneWeights,	fBoneWeights
			;// Get the address of the next bone matrix address for next bone.
			MTIR    iBoneIndex,		fBoneIds[x]
			;// load next bone matrix
			LQ.xyz	fBoneMatrix1, 0(iBoneIndex)
			LQ.xyz	fBoneMatrix2, 1(iBoneIndex)
			LQ.xyz	fBoneMatrix3, 2(iBoneIndex)
			LQ.xyz	fBoneMatrix4, 3(iBoneIndex)
			;// Are we done yet?
			IBEQ    iBoneIndex,   VI00,   FINISHED_SKINNING_BONE_LOOP
			;// decrement loop counter
			IADDI   iBoneCount,	iBoneCount,	-1
			;// Are we done yet?
			IBNE    iBoneCount,   VI00,   SKINNING_BONE_LOOP
FINISHED_SKINNING_BONE_LOOP:
		;// Store Weighted vert and normal back in the input buffer ready for rendering
		SQ.xyz	fVertexOut,	3(pIn)
		SQ.xyzw	fNormal,	0(pIn)
		IADDIU	pIn,	pIn,	5
		;// Decrement counter
		IADDI	iVertexCount,	iVertexCount,	-1
		;// Are we done yet?
		IBNE    iVertexCount,   VI00,   SKINNING_VERTEX_LOOP
	;// Everythings Skinned .. now we need to move the extra camera data and shadow data, out of the output buffers and stash it in the the unused input buffer
	IADD	pOut,	pInputBuffer,		VI00
	IADD	pIn,	REG_THIS_OUTPUT_BUFFER,	VI00
	IADDIU	iCameraCount,	REG_NUM_CAMERAS,	0
MOVE_CAMERA_DATA_LOOP:
		--LoopCS    1,0
	
			LQ	temp1,   0(pIn)
			LQ	temp2,   1(pIn)
			LQ	temp3,   2(pIn)
			LQ	temp4,   3(pIn)
			LQ	temp5,   4(pIn)
			LQ	temp6,   5(pIn)
			LQ	temp7,   6(pIn)
			LQ	temp8,   7(pIn)
			LQ	temp9,   8(pIn)
			LQ	temp10,  9(pIn)
			LQ	temp11, 10(pIn)
			LQ	temp12, 11(pIn)

			SQ	temp1,   1(pOut)
			SQ	temp2,   2(pOut)
			SQ	temp3,   6(pOut)
			SQ	temp4,   7(pOut)
			SQ	temp5,  11(pOut)
			SQ	temp6,  12(pOut)
			SQ	temp7,  16(pOut)
			SQ	temp8,  17(pOut)
			SQ	temp9,  21(pOut)
			SQ	temp10, 22(pOut)
			SQ	temp11, 26(pOut)
			SQ	temp12, 27(pOut)

			IADDIU	pIn, pIn,	12
			IADDIU	pOut, pOut, 30

			IADDI	iCameraCount,	iCameraCount,	-1
			IBNE	iCameraCount,	VI00,	MOVE_CAMERA_DATA_LOOP
CAMERA_DATA_MOVED:

	;// Now move the shadow data - skip if no shadows
	IBEQ	iShadowQty,	VI00,	SHADOW_DATA_MOVED
	IADD	iShadowCount,	iShadowQty,		VI00

MOVE_SHADOW_DATA_LOOP:
		--LoopCS    1,0
	
			LQ	temp1,  0(pIn)
			LQ	temp2,  1(pIn)
			LQ	temp3,  2(pIn)
			LQ	temp4,  3(pIn)
			LQ	temp5,  4(pIn)

			SQ	temp1,  1(pOut)
			SQ	temp2,  2(pOut)
			SQ	temp3,  6(pOut)
			SQ	temp4,  7(pOut)
			SQ	temp5,  11(pOut)

			IADDIU	pIn, pIn,	5
			IADDIU	pOut, pOut, 15

			IADDI	iShadowCount,	iShadowCount,	-1
			IBNE	iShadowCount,	VI00,	MOVE_SHADOW_DATA_LOOP
SHADOW_DATA_MOVED:

	;// Write out the gif tags for both passes - They might have been overwritten by shadow Gif Tags
	SQ.yzw	fPass1_DrawGIF,		0(REG_THIS_OUTPUT_BUFFER)
	SQ.yzw	fPass2_DrawGIF,		0(REG_NEXT_OUTPUT_BUFFER)
	;// write the NLOOP in the two output GIF Tags
	IADDIU	i0x8000, VI00,	   0x7fff
	IADDIU	i0x8000, i0x8000,  1
    IOR     iEop_and_Nloop, i0x8000,	iVertexQty
    ISW.x	iEop_and_Nloop, 0(REG_THIS_OUTPUT_BUFFER)
	ISW.x	iEop_and_Nloop, 0(REG_NEXT_OUTPUT_BUFFER)
	;// Render each of the cameras first and second pass
	IADD	iCameraCount,	REG_NUM_CAMERAS, VI00
	IADDIU	pCameraData,	pInputBuffer,	 30	;// The first cameras data is already loaded so we'll start reloading data fromt he second camera
	;// Setup constants in a register for colour scaling and clamping
	LOI 128
	ADDi.x vColourScaleAndClampConstants, VF00, I
	LOI	255
	ADDi.y vColourScaleAndClampConstants, VF00, I

PER_CAMERA_RENDERING_LOOP:
	--LoopCS    1,0
	;// RENDERING - Pass 1
	;//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	
	;// Kick this camera setup drawlist - This ensures the previous cameras second render pass has completed before we overwrite XYZ values
        SQ fPass1Test, PASS2_SETUP_BUFFER(VI00)
		IADDIU	pKickMe,	VI00,	PASS1_SETUP_BUFFER
		XGKICK	pKickMe
	;// Clear clipping flags
		FCSET	0
	;// Counter for vertex quantity
		IADD	iVertexCount,	iVertexQty,	VI00
	;// Input and Output pointers for skinned verts and normals and UVs
		IADD	pIn,	pInputBuffer,		VI00
		IADDIU	pOut1,	REG_THIS_OUTPUT_BUFFER,	1
		IADDIU	pOut2,	REG_NEXT_OUTPUT_BUFFER,	1

	;// Set RGBAOut Alpha to 128
		LOI 128
		MULi.w	fRgbaOut,	VF00,	i
		FTOI0.w	fRgbaOut,	fRgbaOut

PASS1_RENDERING_VERTEX_LOOP:
			--LoopCS    3,0
        
			LQ.xyz	fVertexIn,		3(pIn)		;// Load XYZW
			LQ.xyzw	fNormalIn,		0(pIn)		;// Load Normal
			LQ.xy	fSTQ,			4(pIn)		;// Load STQ
			IADDIU	pIn,	pIn,	5
			;// Transform the vertex into Clip Space and Clip
			MULAx.xyzw	ACC,			fWorldToClip1,	fVertexIn
			MADDAy.xyzw	ACC,			fWorldToClip2,	fVertexIn
			MADDAz.xyzw	ACC,			fWorldToClip3,	fVertexIn
			MADDw.xyzw	fClipVertex,	fWorldToClip4,	VF00
			CLIPw.xyz	fClipVertex,	fClipVertex
			;// Transform the vertex to screen space
			MULAx.xyzw	ACC,		fWorldToScreen1,	fVertexIn
			MADDAy.xyzw	ACC,		fWorldToScreen2,	fVertexIn
			MADDAz.xyzw	ACC,		fWorldToScreen3,	fVertexIn
			MADDw.xyzw	fVertexOut,	fWorldToScreen4,	VF00
			;// Convert the STQs from 4.12 to floats, 
			ITOF12.xy	fSTQ,	fSTQ
			ADDw.z		fSTQ,	VF00,	VF00   ;// set Q to 1.0f
			;// Calculate 1/W
			DIV	Q,	VF00[w],	fVertexOut[w]
		;// Calculate light.normal (3 lights)
			MULAx.xyz	ACC,				fLightNormalXs,		fNormalIn
			MADDAy.xyz	ACC,				fLightNormalYs,		fNormalIn
			MADDz.xyz	fLightDotNormal,	fLightNormalZs,		fNormalIn
			;// Clamp to >= 0
			MAXx.xyz	fLightDotNormal,	fLightDotNormal,	VF00
			;// Create light colour
			MULAx.xyz	ACC,		fLightColour1,	fLightDotNormal
			MADDAy.xyz  ACC,		fLightColour2,	fLightDotNormal
			MADDz.xyz	fRgbaOut,	fLightColour3,	fLightDotNormal
			;// scale the colour and then clamp to [0, 255]
			MULx.xyz    fRgbaOut,   fRgbaOut,   vColourScaleAndClampConstants
			MINIy.xyz   fRgbaOut,   fRgbaOut,   vColourScaleAndClampConstants
		;// Perspective divide both the XYZ and ST
			MULq.xyz	fVertexOut,	fVertexOut,	Q                
			MULq.xyz	fSTQ,		fSTQ,		Q            
			;// Convert RGBA and XYZ for the GIF packet
			FTOI0.xyz	fRgbaOut,	fRgbaOut
			FTOI4.xyz	fVertexOut,	fVertexOut
			;// Write out STQ, RGBA, and XYZ for Pass 1 and STQ and XYZ for Pass 2 (only Q used from STQ in pass2 to avoid another perspective divide)
			SQ.xyz		fSTQ,			0(pOut1) ;// Output STQ
			SQ.xyz		fSTQ,			0(pOut2) ;// Output STQ
			SQ.xyzw		fRgbaOut,		1(pOut1) ;// Output RGBA
			SQ.xyz		fVertexOut,		2(pOut1) ;// Output XYZ	- only store xyz components. kick(w) stored seperately.
			SQ.xyz		fVertexOut,		2(pOut2) ;// Output XYZ	- only store xyz components. kick(w) stored seperately.
			IADDIU		pOut1,	pOut1,	3
			IADDIU		pOut2,	pOut2,	3
			;// Copy export ADC bit from normal's w to integer register.
			MTIR	iDegeneracyADCFlag,	fNormalIn[w]
			;// Test if any of the clip flags are set for last check
			FCAND	VI01,		0x3ffff				;// previous 3 . any set, then vi01=1, else 0
			IADDIU	iADCFlag,	VI01,	0x7fff		;// add 0x7fff so that iADCFlag will be 0x8000 if clip flags set or 0x7fff otherwise
			;// OR in the ADC flag from the input data.
			IOR		iADCFlag,	iADCFlag,	iDegeneracyADCFlag
			ISW.w	iADCFlag,    -1(pOut1)w
			ISW.w	iADCFlag,    -1(pOut2)w
			;// Decrement the vertex loop counter
			IADDI	iVertexCount,	iVertexCount,	-1
			IBNE    iVertexCount,	VI00,	PASS1_RENDERING_VERTEX_LOOP
		;// Kick Output Buffer 0 (First Pass Drawing)
		XGKICK	REG_THIS_OUTPUT_BUFFER

;// RENDERING - Pass 2
;//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
		;// Counter for vertex quantity
		IADDIU	iVertexCount,	iVertexQty,	0
		;// Input and Output pointers for skinned verts and normals and UVs
		IADD	pIn,	pInputBuffer,	VI00
		IADDIU	pOut2,	REG_NEXT_OUTPUT_BUFFER,	1
PASS2_RENDERING_VERTEX_LOOP:
		--LoopCS    3,0
			;// Load Normal & Load STQ - We want the Q value without doing another perspective divide.
			LQ.xyz		fNormalIn,  0(pIn)
			LQ.z		fSTQ,		0(pOut2)
			IADDIU		pIn,	pIn,	5
			;// Dredd renderer: calculate STs for outlining and specular
			MUL.xyz		fLightDotEye,	fToEye,			fNormalIn
			ADDy.x		fLightDotEye,	fLightDotEye,	fLightDotEye
			ADDz.x		fLightDotEye,	fLightDotEye,	fLightDotEye
			;// our U is dotproduct [-1 to 1] clamped to [0,1]
			MAX.x		fSTQ,	fLightDotEye,	VF00
			;// to test specular set U to 1
			;//	ADDw.x	fSTQ,	VF00,	VF00
			;// specular: V = (h.n)^5
			MUL.xyz		fHalfDotNormal,	fSpecularHalf,	fNormalIn
			ADDy.x		fHalfDotNormal,	fHalfDotNormal,	fHalfDotNormal
			ADDz.x		fHalfDotNormal,	fHalfDotNormal,	fHalfDotNormal
			;// Raise it to the power 5 to sharpen the effect
			ADDx.y		fSTQ,	VF00,	fHalfDotNormal		;// V = (h.n)
			MUL.y		fSTQ,	fSTQ,	fSTQ				;// V = (h.n)^2
			MUL.y		fSTQ,	fSTQ,	fSTQ				;// V = (h.n)^4
			MULx.y		fSTQ,	fSTQ,	fHalfDotNormal		;// V = (h.n)^5
			;// to test outline, set V to 0
			;//ADD.y	fSTQ,	VF00,	VF00
			;// our V is this [-1 to 1] clamped to [0,1]
			MAX.y		fSTQ,	fSTQ,	VF00
			;// Perspective divide the ST ( We loaded Q from Pass 1 )
			MULz.xy		fSTQ,	fSTQ,	fSTQ
			;// Write out STQ and RGBA and skip xyz
			SQ.xyz	fSTQ,			0(pOut2)
			SQ.xyzw	fSpecularColour,1(pOut2)
			IADDIU	pOut2,	pOut2,	3
			;// Decrement the vertex loop counter
			IADDI	iVertexCount,    iVertexCount,    -1
			;// Are we done yet ?
			IBNE    iVertexCount,	VI00,	PASS2_RENDERING_VERTEX_LOOP
		;// Kick the pass 2 Setup list
        SQ fPass2SetupGifTag, PASS2_SETUP_BUFFER(VI00)
		IADDIU	pKickMe,	VI00,	PASS2_SETUP_BUFFER
		XGKICK	pKickMe

		;// Kick Output Buffer 1 (Second Pass Drawing)
		XGKICK	REG_NEXT_OUTPUT_BUFFER

		;// Are there anymore cameras to come ?
		IADDI	iCameraCount,	iCameraCount,	-1
		IBEQ	iCameraCount,	VI00,	FINISHED_RENDERING_CAMERAS
		;// If we have multiple cameras most all this register shuffling time will be obsorbed by the time it take to render the second pass before the next camera
		;// Load the next cameras settings
		LQ.xyzw	fWorldToScreen1,  1(pCameraData)
		LQ.xyzw	fWorldToScreen2,  2(pCameraData)
		LQ.xyzw	fWorldToScreen3,  6(pCameraData)
		LQ.xyzw	fWorldToScreen4,  7(pCameraData)
		LQ.xyzw	fWorldToClip1,	 11(pCameraData)
		LQ.xyzw	fWorldToClip2,	 12(pCameraData)
		LQ.xyzw	fWorldToClip3,	 16(pCameraData)
		LQ.xyzw	fWorldToClip4,	 17(pCameraData)
		LQ.xyz	fToEye,			 21(pCameraData)
		LQ.xyz	fSpecularHalf,	 22(pCameraData)
		LQ.xyzw	fSCISSOR,		 26(pCameraData)
		LQ.xyzw	fXYOFFSET,		 27(pCameraData)
		;// Update the camera setup draw list
		SQ.xyzw	fSCISSOR,		(PASS1_SETUP_BUFFER+1)(VI00)
		SQ.xyzw	fXYOFFSET,		(PASS1_SETUP_BUFFER+2)(VI00)

		IADDIU	pCameraData, pCameraData, 30

		B	PER_CAMERA_RENDERING_LOOP

FINISHED_RENDERING_CAMERAS:
	;// Now render any shadows
	IBEQ	iShadowQty,	VI00,	FINISHED_RENDERING_SHADOWS
	IADD	iShadowCount,	iShadowQty,		VI00

PER_SHADOW_RENDERING_LOOP:
	--LoopCS    1,0
	;// Load the shadow matrix
		LQ	fFRAME,			   1(pCameraData)
		LQ	fWorldToScreen1,   2(pCameraData)
		LQ	fWorldToScreen2,   6(pCameraData)
		LQ	fWorldToScreen3,   7(pCameraData)
		LQ	fWorldToScreen4,  11(pCameraData)
		;// Update the shadow setup draw list
		SQ	fFRAME,	(SHADOW_SETUP_BUFFER+2)(VI00)
		;// Move the shadow data pointer on, ready for the next shadow
		IADDIU	pCameraData, pCameraData, 15
	;// Counter for vertex quantity
		IADDIU	iVertexCount,	iVertexQty,	0
	;// Input and Output pointers for skinned verts and normals and UVs
		IADD	pIn,	pInputBuffer,	VI00
		IADDIU	pOut,	REG_THIS_OUTPUT_BUFFER,	1

SHADOW_RENDERING_VERTEX_LOOP:
			--LoopCS    3,0
     
			LQ.xyz	fVertexIn,		3(pIn)		;// Load XYZ
			LQ.w	fScrnPos,		0(pIn)		;// Load degeneracy flag
			IADDIU	pIn,	pIn,	5
			;// Transform the vertex to screen space - no need to clip the matrixs ensures it's all visible
			MULAx.xyzw	ACC,		fWorldToScreen1,	fVertexIn
			MADDAy.xyzw	ACC,		fWorldToScreen2,	fVertexIn
			MADDAz.xyzw	ACC,		fWorldToScreen3,	fVertexIn
			MADDw.xyzw	fVertexOut,	fWorldToScreen4,	VF00
			;// Calculate 1/W
			DIV	Q,	VF00[w],	fVertexOut[w]
			;// Perspective divide both the XYZ
			MULq.xyz	fScrnPos,	fVertexOut,	Q                
			;// Convert XYZ for the GIF packet
			FTOI4.xyz	fScrnPos,	fScrnPos
			;// Write out XYZ
			SQ.xyzw	fScrnPos,		0(pOut)  ;// Output XYZ
			IADDIU	pOut,	pOut,	1
			;// Decrement the vertex loop counter
			IADDI	iVertexCount,	iVertexCount,	-1
			IBNE    iVertexCount,	VI00,	SHADOW_RENDERING_VERTEX_LOOP
		;// Kick the shadow setup drawlist
		IADDIU	pKickMe,	VI00,	SHADOW_SETUP_BUFFER
		XGKICK	pKickMe
		;// Write shadow drawing GifTag ( NLOOP is still ok from pass 1 and pass 2 renders )
		SQ.yzw	fShadow_DrawGIF, 0(REG_THIS_OUTPUT_BUFFER)
		;// Kick the Output Buffer
		XGKICK	REG_THIS_OUTPUT_BUFFER
		;// Swap output buffers
		IADDIU	iTemp,				REG_THIS_OUTPUT_BUFFER,	0
		IADDIU	REG_THIS_OUTPUT_BUFFER,	REG_NEXT_OUTPUT_BUFFER,	0
		IADDIU	REG_NEXT_OUTPUT_BUFFER,	iTemp,				0
		;// Are we finished with the shadows ?
		IADDI	iShadowCount,	iShadowCount,	-1
		IBNE	iShadowCount,	VI00,	PER_SHADOW_RENDERING_LOOP
FINISHED_RENDERING_SHADOWS:
	;// This strip is all rendered in each of the cameras. Put all the camera data back in the right places.. If we have shadows put that data back in the right place
	;// Set up these pointers now in case we have shadows but only one camera
	IADD	pIn,	pInputBuffer,		VI00
	IADD	pOut,	REG_THIS_OUTPUT_BUFFER,	VI00
	;// Reload first camera settings
	IADD	pCameraData,	pInputBuffer,	VI00
	LQ.xyzw	fWorldToScreen1,  1(pCameraData)
	LQ.xyzw	fWorldToScreen2,  2(pCameraData)
	LQ.xyzw	fWorldToScreen3,  6(pCameraData)
	LQ.xyzw	fWorldToScreen4,  7(pCameraData)
	LQ.xyzw	fWorldToClip1,	 11(pCameraData)
	LQ.xyzw	fWorldToClip2,	 12(pCameraData)
	LQ.xyzw	fWorldToClip3,	 16(pCameraData)
	LQ.xyzw	fWorldToClip4,	 17(pCameraData)
	LQ.xyz	fToEye,			 21(pCameraData)
	LQ.xyz	fSpecularHalf,	 22(pCameraData)
	LQ.xyzw	fSCISSOR,		 26(pCameraData)
	LQ.xyzw	fXYOFFSET,		 27(pCameraData)
	;// Update the camera setup draw list
	SQ.xyzw	fSCISSOR,		(PASS1_SETUP_BUFFER+1)(VI00)
	SQ.xyzw	fXYOFFSET,		(PASS1_SETUP_BUFFER+2)(VI00)

	;// Move the camera data back into the output buffers ready for then next strip
	IADDIU	iCameraCount,	REG_NUM_CAMERAS,	0
MOVE_CAMERA_DATA_BACK_LOOP:
	--LoopCS    2,0
		LQ	temp1,   1(pIn)
		LQ	temp2,   2(pIn)
		LQ	temp3,   6(pIn)
		LQ	temp4,   7(pIn)
		LQ	temp5,  11(pIn)
		LQ	temp6,  12(pIn)
		LQ	temp7,  16(pIn)
		LQ	temp8,  17(pIn)
		LQ	temp9,  21(pIn)
		LQ	temp10, 22(pIn)
		LQ	temp11, 26(pIn)
		LQ	temp12, 27(pIn)

		SQ	temp1,   0(pOut)
		SQ	temp2,   1(pOut)
		SQ	temp3,   2(pOut)
		SQ	temp4,   3(pOut)
		SQ	temp5,   4(pOut)
		SQ	temp6,   5(pOut)
		SQ	temp7,   6(pOut)
		SQ	temp8,   7(pOut)
		SQ	temp9,   8(pOut)
		SQ	temp10,  9(pOut)
		SQ	temp11, 10(pOut)
		SQ	temp12, 11(pOut)

		IADDIU	pIn, pIn, 30
		IADDIU	pOut, pOut, 12

		IADDI	iCameraCount,	iCameraCount,	-1
		IBNE	iCameraCount,	VI00,	MOVE_CAMERA_DATA_BACK_LOOP

CAMERA_DATA_MOVED_BACK:
	;// Now move the shadow data - skip if no shadows
	IBEQ	iShadowQty,		VI00,	SHADOW_DATA_MOVED_BACK
	IADD	iShadowCount,	iShadowQty,		VI00

MOVE_SHADOW_DATA_BACK_LOOP:
		--LoopCS    1,0
			LQ	temp1,  1(pIn)
			LQ	temp2,  2(pIn)
			LQ	temp3,  6(pIn)
			LQ	temp4,  7(pIn)
			LQ	temp5, 11(pIn)

			SQ	temp1,  0(pOut)
			SQ	temp2,  1(pOut)
			SQ	temp3,  2(pOut)
			SQ	temp4,  3(pOut)
			SQ	temp5,  4(pOut)

			IADDIU	pIn, pIn,	15
			IADDIU	pOut, pOut, 5

			IADDI	iShadowCount,	iShadowCount,	-1
			IBNE	iShadowCount,	VI00,	MOVE_SHADOW_DATA_BACK_LOOP
SHADOW_DATA_MOVED_BACK:

    --cont
    
    B    STRIP_SETUP

;// Quad renderer to draw the shadows back into the the scene once the character has finished rendering
;// It's here so we can keep the same renderer in memory, and luckily all the camera data is still present

	--enter
        .init_vi_all
		in_vi pThisOutputBuffer(VI15)
		in_vi pNextOutputBuffer(VI14)
		in_vi iCameraQty(VI12)
	--endenter

.global VU1_Micro_Skinning_GraphicNovel_QuadRender
VU1_Micro_Skinning_GraphicNovel_QuadRender:

	;// At the end of the character rendering, all the camera data we need is sitting in one of the two output buffers.
	;// No we are going to be setting up scissor and offset values and drawing quads, giving us very little ouput data
	;// So we'll leave the output buffer with the camera data alone, and split the other output buffer into 2 and use that for output

	;// Load the gif tags for the quads from 0 - the bone buffer is no longer needed
	LQ		fQuad_GIF,	0(VI00)

	;// And the output buffer size
	ILW.w	iHalfOutputBufferSize, 1(VI00)w

	// Setup the output pointer and camera data pointer
	IADD	pSkinCamData, pThisOutputBuffer, VI00
	IADD	pThisOutputBuffer, pNextOutputBuffer, iHalfOutputBufferSize

	--cont

QUAD_RENDER_SETUP:

	;// Get which double buffer we are in (where our memory starts)
    XTOP	pInputBuffer

	;// Per camera quad rendering loop
    IADDIU pCamIn,       pInputBuffer, 0
    IADDIU pGSSetupIn,   pSkinCamData, 0
    IADDIU iCameraCount, iCameraQty,   0

PER_CAMERA_QUAD_RENDERING_LOOP:
	--LoopCS    1,0

		;// Load the settings for this camera
		LQ	fWorldToScreen1,  0(pCamIn)
		LQ	fWorldToScreen2,  1(pCamIn)
		LQ	fWorldToScreen3,  2(pCamIn)
		LQ	fWorldToScreen4,  3(pCamIn)
		LQ	fWorldToClip1,	  4(pCamIn)
		LQ	fWorldToClip2,	  5(pCamIn)
		LQ	fWorldToClip3,	  6(pCamIn)
		LQ	fWorldToClip4,	  7(pCamIn)
        LQ  fRGBA,            8(pCamIn)
		LQ	fSCISSOR,		 10(pGSSetupIn)
		LQ	fXYOFFSET,		 11(pGSSetupIn)
        IADDIU pCamIn,     pCamIn,     9
		IADDIU pGSSetupIn, pGSSetupIn, 12

		;// Build the start of the draw list
		SQ	fQuad_GIF,	0(pThisOutputBuffer)
		SQ	fSCISSOR,	1(pThisOutputBuffer)
		SQ	fXYOFFSET,	2(pThisOutputBuffer)

		;// Clear clipping flags
		FCSET	0

		;// It's a quad, so it has 4 verts
		IADDIU	iVertexCount,	VI00,	4

		IADDIU	pIn,	pInputBuffer,		(4 * 9)
		IADDIU	pOut,	pThisOutputBuffer,	3

QUAD_RENDERING_VERTEX_LOOP:
			--LoopCS    4,0

			LQ.xyz		fSTQ,			0(pIn)		;// Load STQ        
			LQ.xyz		fVertexIn,		1(pIn)		;// Load XYZW

			IADDIU	pIn,	pIn,	2

			;// Transform the vertex into Clip Space and Clip
			MULAx.xyzw	ACC,			fWorldToClip1,	fVertexIn
			MADDAy.xyzw	ACC,			fWorldToClip2,	fVertexIn
			MADDAz.xyzw	ACC,			fWorldToClip3,	fVertexIn
			MADDw.xyzw	fClipVertex,	fWorldToClip4,	VF00
			CLIPw.xyz	fClipVertex,	fClipVertex
    
			;// Transform the vertex to screen space
			MULAx.xyzw	ACC,		fWorldToScreen1,	fVertexIn
			MADDAy.xyzw	ACC,		fWorldToScreen2,	fVertexIn
			MADDAz.xyzw	ACC,		fWorldToScreen3,	fVertexIn
			MADDw.xyzw	fVertexOut,	fWorldToScreen4,	VF00

			;// Test if any of the clip flags are set for last check
			FCAND	VI01,		0x3ffff				;// previous 3 . any set, then vi01=1, else 0
			IADDIU	iADCFlag,	VI01,	0x7fff		;// add 0x7fff so that iADCFlag will be 0x8000 if clip flags set or 0x7fff otherwise

			;// Calculate 1/W
			DIV	Q,	VF00[w],	fVertexOut[w]

			;// Copy clipping KICK flag into output vertex
			MFIR.w	fVertexOut, iADCFlag

			;// Perspective divide both the XYZ and ST
			MULq.xyz	fVertexOut,	fVertexOut,	Q
			MULq.xyz	fSTQ,		fSTQ,		Q            

			;// Convert XYZ for the GIF packet
			FTOI4.xyz	fVertexOut,	fVertexOut

			;// Write out STQ, RGBA, and XYZ 
			SQ.xyz	fSTQ,			0(pOut) ;// Output STQ
			SQ		fRGBA,			1(pOut) ;// Output RGBA
			SQ		fVertexOut,		2(pOut)
			IADDIU	pOut,	pOut,	3

			;// Decrement the vertex loop counter
			IADDI	iVertexCount,	iVertexCount,	-1
			IBNE    iVertexCount,	VI00,	QUAD_RENDERING_VERTEX_LOOP
    
		;// Kick Output Buffer 0 (First Pass Drawing)
		XGKICK	pThisOutputBuffer

		;// Swap output buffers
		IADDIU	iTemp,				pThisOutputBuffer,	0
		IADDIU	pThisOutputBuffer,	pNextOutputBuffer,	0
		IADDIU	pNextOutputBuffer,	iTemp,				0

		;// Have we done all the cameras ?
		IADDI	iCameraCount,	iCameraCount,	-1
		IBNE	iCameraCount,	VI00,	PER_CAMERA_QUAD_RENDERING_LOOP

	;// Ok, we are done with this quad

END:
    --cont
    
    B    QUAD_RENDER_SETUP

.end

