//////////////////////////////////////////////////////////////////////////////
//
// VU1_SmoothSkin.vcl
//
// Copyright (c) 2004 Rebellion. All rights reserved.
//
// See Asura_PS2_SmoothSkinDefines.h for memory map.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Asura_PS2_SmoothSkinDefines.h"
#include "vcl_sml.i"
#include "VU1_CommonMacros.i"

// V C L   S E T U P /////////////////////////////////////////////////////////

.syntax new

.init_vf_all
.init_vi_all

.global VU1_SS_INIT
.global VU1_SS_BEGIN_NEW_SKIN
#if ASURA_PS2_SS__REDUCE_WEIGHTINGS_WITH_DISTANCE
.global VU1_SS_SKIN_SET_MAX_WEIGHTINGS_PER_VTX_CLAMP
#endif
.global VU1_SS_LOAD_CAM_FLAGS
.global VU1_SS_SWITCH_TEX_MAT_BFR
.global VU1_SS_BEGIN_MATERIAL
.global VU1_SS_NOP
.global VU1_SS_READY_FOR_NEXT_STRIP
.global VU1_SS_SKIN_VERTS_1_BONE
.global VU1_SS_SKIN_VERTS_2_BONES
.global VU1_SS_SKIN_VERTS_3_BONES
.global VU1_SS_SKIN_VERTS_4_BONES

// C O D E ///////////////////////////////////////////////////////////////////

--enter
    in_hw_clip clip
--endenter

    // (*) Note that there are some slightly subtle ordering issues in the
    // first few branches below. We're pretending to VCL that we're MSCNTing
    // our way straight through this stuff while in reality we'll mostly be
    // MSCALing all over the place. We can let VCL know about all the
    // dependencies we require with careful organisation and fake
    // conditionals.

VU1_SS_INIT:

    // This subroutine is expected to be run exactly once, after the
    // microprogram has been uploaded but before any other part of it is run.

    // Setup initial ptr values
    IADDIU pRSInBfr,   VI00, ASURA_PS2_SS__ADDR__RENDERSTATE_BFR_0
    IADDIU pTexMatBfr, VI00, ASURA_PS2_SS__ADDR__TEX_MAT_BFR_0
    // Write to reg store.
    ISW.w pRSInBfr,   ASURA_PS2_SS__ADDR__VI_REG_STORE_3(VI00)
    ISW.y pTexMatBfr, ASURA_PS2_SS__ADDR__VI_REG_STORE_2(VI00)
    // Setup the normal unpacking register.
    MakeNormalUnpackScale fUnpackScale
    // Reset camera visibility and clipping flags so we recognise they've
    // changed the first time they're uploaded. We assume that at least one
    // camera will always be able to see transferred geometry and so 0 is a
    // suitable invalid value.
    ISW.w VI00, ASURA_PS2_SS__ADDR__VI_REG_STORE_1(VI00)
    // "Reserve" the material flags register to avoid it getting override in
    // certain execution paths.
    IADDIU iMaterialFlags, VI00, 0
    --cont

VU1_SS_BEGIN_NEW_SKIN:

    XITOP iSkinFlags
    // Extract length of longest strip.
    IADDIU iMask, VI00, ASURA_PS2_SS__MASK__LONGEST_STRIP_VTX_COUNT
    IAND iLongestStripVtxQty, iSkinFlags, iMask
    // Calculate bfr offsets. See the memory map for details of this.
    ISUB iMinusV, VI00, iLongestStripVtxQty
    IADD iMinus2V, iMinusV, iMinusV
    IADD iMinus3V, iMinus2V, iMinusV
    ISUBIU iMinus1Minus3V, iMinus3V, 1
    IADD iMinus1Minus4V, iMinus1Minus3V, iMinusV
    // And calculate bfr pointers. Note that we don't bother calculating the
    // clip bfr ptr until we require it; it's just p-(1021-p)=2p-1021, where
    // p is pPass2Bfr.
    IADDIU pPass2Bfr, iMinusV, 1021
    IADD pSecondOutputBfr, pPass2Bfr, iMinus1Minus4V
    IADD pFirstOutputBfr, pSecondOutputBfr, iMinus1Minus3V
    // Reset current output buffer to first one. We're going to have to stall
    // for the bone data upload anyway, since that's single-buffered, so this
    // doesn't really matter.
    IADDIU pThisOutputBfr, pFirstOutputBfr, 0
#if ASURA_PS2_SS__REDUCE_WEIGHTINGS_WITH_DISTANCE
    // Reset max weightings per vertex clamp.
    IADDIU iMaxWeightingsPerVtxClamp, VI00, 4
    ISW.z iMaxWeightingsPerVtxClamp, ASURA_PS2_SS__ADDR__VI_REG_STORE_4(VI00)
#endif
    // Write new values to reg store
    ISW.y pPass2Bfr,        ASURA_PS2_SS__ADDR__VI_REG_STORE_3(VI00)
    ISW.z iSkinFlags,       ASURA_PS2_SS__ADDR__VI_REG_STORE_3(VI00)
    ISW.x pFirstOutputBfr,  ASURA_PS2_SS__ADDR__VI_REG_STORE_4(VI00)
    ISW.y pSecondOutputBfr, ASURA_PS2_SS__ADDR__VI_REG_STORE_4(VI00)
    --cont

#if ASURA_PS2_SS__REDUCE_WEIGHTINGS_WITH_DISTANCE

VU1_SS_SKIN_SET_MAX_WEIGHTINGS_PER_VTX_CLAMP:

    // Read and then write to reg store.
    XITOP iMaxWeightingsPerVtxClamp
    ISW.z iMaxWeightingsPerVtxClamp, ASURA_PS2_SS__ADDR__VI_REG_STORE_4(VI00)
    --cont

#endif

VU1_SS_LOAD_CAM_FLAGS:

    XITOP iCamFlags
    // Extract components.
    IADDIU iMask, VI00, ASURA_PS2_SS__FLAG__VIS_FOR_MORE_THAN_ONE_CAM
    IAND iMoreThanOneCamCanSeeFlag, iCamFlags, iMask
    IADDIU iMask, VI00, ASURA_PS2_SS__MASK__CAM_VIS_AND_CLIP_FLAGS
    IAND iVisAndCullFlags, iCamFlags, iMask
    // Isolate vis flags, both old and new.
    ILW.w iOldVisAndCullFlags, ASURA_PS2_SS__ADDR__VI_REG_STORE_1(VI00)
    IADDIU iMask, VI00, ASURA_PS2_SS__MASK__CAM_VIS_FLAGS
    IAND iOldVisFlags, iOldVisAndCullFlags, iMask
    IAND iNewVisFlags, iVisAndCullFlags, iMask
    // Write new values to VI reg store.
    ISW.w iVisAndCullFlags,          ASURA_PS2_SS__ADDR__VI_REG_STORE_1(VI00)
    ISW.x iMoreThanOneCamCanSeeFlag, ASURA_PS2_SS__ADDR__VI_REG_STORE_2(VI00)
    // If the vis flags have changed, force RS to be sent next time. Note
    // that writing any non-zero value will do here.
    IBEQ iOldVisFlags, iNewVisFlags, VU1_SS_FINISHED_SETTING_UP_CAM_FLAGS
    ISW.w iMask, ASURA_PS2_SS__ADDR__VI_REG_STORE_4(VI00)

VU1_SS_FINISHED_SETTING_UP_CAM_FLAGS:
    --cont

VU1_SS_SWITCH_TEX_MAT_BFR:

    ILW.y pTexMatBfr, ASURA_PS2_SS__ADDR__VI_REG_STORE_2(VI00)
    SwitchBfr_Constants pTexMatBfr, ASURA_PS2_SS__ADDR__TEX_MAT_BFR_0, ASURA_PS2_SS__ADDR__TEX_MAT_BFR_1
    ISW.y pTexMatBfr, ASURA_PS2_SS__ADDR__VI_REG_STORE_2(VI00)
    --cont

VU1_SS_DUMMY: // Never executed.

    // Make VCL think we may branch to VU1_SS_READY_FOR_NEXT_STRIP at this
    // point, so that it doesn't assume iMaterialFlags will always get
    // reloaded at this point. See the (*) comment up above.
    XITOP dummy
    IBNE dummy, VI00, VU1_SS_READY_FOR_NEXT_STRIP

VU1_SS_BEGIN_MATERIAL:

    // Retrieve old RS bfr ptr from stack.
    ILW.w pRSInBfr, ASURA_PS2_SS__ADDR__VI_REG_STORE_3(VI00)
    // Load new material flags.
    XITOP iMaterialFlags
    // Switch renderstate bfrs.
    SwitchBfr_Constants pRSInBfr, ASURA_PS2_SS__ADDR__RENDERSTATE_BFR_0, ASURA_PS2_SS__ADDR__RENDERSTATE_BFR_1
    // Store new RS bfr ptr to VI reg store.
    ISW.w pRSInBfr, ASURA_PS2_SS__ADDR__VI_REG_STORE_3(VI00)
    // Set need-to-send-RS flag for next strip. Note that any non-zero value
    // will do here.
    ISW.w pRSInBfr, ASURA_PS2_SS__ADDR__VI_REG_STORE_4(VI00)
    --cont

VU1_SS_DUMMY2: // Never executed.

    // Make VCL think we may branch to VU1_SS_BEGIN_NEW_SKIN at this point.
    // See the (*) comment up above.
    XITOP dummy
    IBNE dummy, VI00, VU1_SS_BEGIN_NEW_SKIN

VU1_SS_END_STRIP:
VU1_SS_NOP:
VU1_SS_READY_FOR_NEXT_STRIP:

    --cont
    // NB: This is the one and only point we expect to do an MSCNT from.

VU1_SS_BEGIN_STRIP:

    // Get input ptr.
    XTOP pIn
    // Load max weightings per vertex for this strip and the clamp value
    // specified by EE-side code.
    ILW.w iMaxWeightingsPerVtx,      4(pIn)
#if ASURA_PS2_SS__REDUCE_WEIGHTINGS_WITH_DISTANCE
    ILW.z iMaxWeightingsPerVtxClamp, ASURA_PS2_SS__ADDR__VI_REG_STORE_4(VI00)
#endif
    // Read vertex count and start calculating end-of-input ptr.
    XITOP iVtxQty
    IADD i2xVtxQty, iVtxQty, iVtxQty
    // Load skinning subroutine for max weightings specified in strip. Note
    // that we need to offset by -1 since our lookup table is obviously
    // 0-indexed.
    ILW.w pSkinSubAddr, (ASURA_PS2_SS__ADDR__SKIN_SUB_LOOKUP_TABLE - 1)(iMaxWeightingsPerVtx)
#if ASURA_PS2_SS__REDUCE_WEIGHTINGS_WITH_DISTANCE
    // Start calculating clamp check value for max weightings.
    ISUB iMaxWeightingsDifference, iMaxWeightingsPerVtxClamp, iMaxWeightingsPerVtx
#endif
    // Bit more end-of-input ptr calculation...
    IADD i4xVtxQty, i2xVtxQty, i2xVtxQty
    IADD i5xVtxQty, i4xVtxQty, iVtxQty
#if ASURA_PS2_SS__REDUCE_WEIGHTINGS_WITH_DISTANCE
    // If max weightings clamp is lower than the specified value, override
    // the subroutine address accordingly.
    IBGEZ iMaxWeightingsDifference, VU1_SS_LOADED_SKINNING_SUBROUTINE_ADDRESS
    // Ok, override.
    ILW.w pSkinSubAddr, (ASURA_PS2_SS__ADDR__SKIN_SUB_LOOKUP_TABLE - 1)(iMaxWeightingsPerVtxClamp)
#endif

VU1_SS_LOADED_SKINNING_SUBROUTINE_ADDRESS:

    // Finish calculating end-of-input ptr.
    IADD pInEnd, pIn, i5xVtxQty
    // Reset skinning output vtx's w component. We'll maintain this at 1
    // throughout either skinning loop below.
    MOVE.w fPosOut, VF00
    // Branch to appropriate skinning subroutine.
    JR pSkinSubAddr:VU1_SS_SKIN_VERTS_1_BONE:VU1_SS_SKIN_VERTS_2_BONES:VU1_SS_SKIN_VERTS_3_BONES:VU1_SS_SKIN_VERTS_4_BONES

VU1_SS_SKIN_VERTS_1_BONE:

    VU1_SS_SKIN_VERTS_1_BONE_LOOP:

        --LoopCS  3, 0
        --LoopAbs 14

        // Load input vtx data.
        LQ     fNormal,      0(pIn)
        LQ.x   fBoneWeights, 2(pIn)
        LQ.xyz fPosIn,       3(pIn)
        // Convert normal. We might as well do the ADC at the same time, so
        // we don't need to bother when rendering.
        UnpackNormalADC fNormal, fUnpackScale
        // Convert the offset and bone weights.
        ITOF12.xyz fPosIn,       fPosIn
        ITOF15.x   fBoneWeights, fBoneWeights
        // Load bone matrix.
        ILW.x iBoneIdx, 1(pIn)
        LQ.xyz fBoneMat0, (ASURA_PS2_SS__ADDR__BONE_MATRIX_BFR + 0)(iBoneIdx)
        LQ.xyz fBoneMat1, (ASURA_PS2_SS__ADDR__BONE_MATRIX_BFR + 1)(iBoneIdx)
        LQ.xyz fBoneMat2, (ASURA_PS2_SS__ADDR__BONE_MATRIX_BFR + 2)(iBoneIdx)
        LQ.xyz fBoneMat3, (ASURA_PS2_SS__ADDR__BONE_MATRIX_BFR + 3)(iBoneIdx)
        // Rotate normal.
        MULAx.xyz  ACC,     fBoneMat0, fNormal
        MADDAy.xyz ACC,     fBoneMat1, fNormal
        MADDz.xyz  fNormal, fBoneMat2, fNormal        
        // Transform input vtx.
        MULAw.xyz  ACC,      fBoneMat3, VF00[w]
        MADDAx.xyz ACC,      fBoneMat0, fPosIn[x]
        MADDAy.xyz ACC,      fBoneMat1, fPosIn[y]
        MADDz.xyz  fPosTemp, fBoneMat2, fPosIn[z]
        // Scale transformed point by weighting.
        MULx.xyz fPosOut, fPosTemp, fBoneWeights[x]
        // Overwrite input buffer data with skinned position and normal, and
        // also the converted ADC.
        SQ fNormal, 0(pIn)
        SQ fPosOut, 3(pIn)
        // Are we done yet?
        IADDIU pIn, pIn, 5
        IBNE pIn, pInEnd, VU1_SS_SKIN_VERTS_1_BONE_LOOP

    B VU1_SS_FINISHED_SKINNING

VU1_SS_SKIN_VERTS_2_BONES:

    VU1_SS_SKIN_VERTS_2_BONES_LOOP:

        --LoopCS  3, 0
        --LoopAbs 19

        // Load input vtx data.
        LQ     fNormal,      0(pIn)
        LQ.xy  fBoneWeights, 2(pIn)
        LQ.xyz fPosIn,       3(pIn)
        // Convert normal. We might as well do the ADC at the same time, so
        // we don't need to bother when rendering.
        UnpackNormalADC fNormal, fUnpackScale
        // Convert the offset and bone weights.
        ITOF12.xyz fPosIn,       fPosIn
        ITOF15.xy  fBoneWeights, fBoneWeights
        // Bone 0.
        // Load bone matrix.
        ILW.x iBoneIdx, 1(pIn)
        LQ.xyz fBoneMat0, (ASURA_PS2_SS__ADDR__BONE_MATRIX_BFR + 0)(iBoneIdx)
        LQ.xyz fBoneMat1, (ASURA_PS2_SS__ADDR__BONE_MATRIX_BFR + 1)(iBoneIdx)
        LQ.xyz fBoneMat2, (ASURA_PS2_SS__ADDR__BONE_MATRIX_BFR + 2)(iBoneIdx)
        LQ.xyz fBoneMat3, (ASURA_PS2_SS__ADDR__BONE_MATRIX_BFR + 3)(iBoneIdx)
        // Rotate normal. We only need to do this for the first bone.
        MULAx.xyz  ACC,     fBoneMat0, fNormal
        MADDAy.xyz ACC,     fBoneMat1, fNormal
        MADDz.xyz  fNormal, fBoneMat2, fNormal        
        // Transform input vtx.
        MULAw.xyz  ACC,      fBoneMat3, VF00[w]
        MADDAx.xyz ACC,      fBoneMat0, fPosIn[x]
        MADDAy.xyz ACC,      fBoneMat1, fPosIn[y]
        MADDz.xyz  fPosTemp, fBoneMat2, fPosIn[z]
        // Scale transformed point by weighting.
        MULx.xyz fPosOut, fPosTemp, fBoneWeights[x]
        // Bone 1.
        // Load bone matrix.
        ILW.y iBoneIdx, 1(pIn)
        LQ.xyz fBoneMat0, (ASURA_PS2_SS__ADDR__BONE_MATRIX_BFR + 0)(iBoneIdx)
        LQ.xyz fBoneMat1, (ASURA_PS2_SS__ADDR__BONE_MATRIX_BFR + 1)(iBoneIdx)
        LQ.xyz fBoneMat2, (ASURA_PS2_SS__ADDR__BONE_MATRIX_BFR + 2)(iBoneIdx)
        LQ.xyz fBoneMat3, (ASURA_PS2_SS__ADDR__BONE_MATRIX_BFR + 3)(iBoneIdx)
        // Transform input vtx.
        MULAw.xyz  ACC,      fBoneMat3, VF00[w]
        MADDAx.xyz ACC,      fBoneMat0, fPosIn[x]
        MADDAy.xyz ACC,      fBoneMat1, fPosIn[y]
        MADDz.xyz  fPosTemp, fBoneMat2, fPosIn[z]
        // Scale transformed point by weighting and add to output position.
        MULy.xyz fPosTemp, fPosTemp, fBoneWeights[y]
        ADD.xyz fPosOut, fPosOut, fPosTemp
        // Finished bones.
        // Overwrite input buffer data with skinned position and normal, and
        // also the converted ADC.
        SQ fNormal, 0(pIn)
        SQ fPosOut, 3(pIn)
        // Are we done yet?
        IADDIU pIn, pIn, 5
        IBNE pIn, pInEnd, VU1_SS_SKIN_VERTS_2_BONES_LOOP

    B VU1_SS_FINISHED_SKINNING

VU1_SS_SKIN_VERTS_3_BONES:

    VU1_SS_SKIN_VERTS_3_BONES_LOOP:

        --LoopCS  3, 0
        --LoopAbs 25

        // Load input vtx data.
        LQ     fNormal,      0(pIn)
        LQ.xyz fBoneWeights, 2(pIn)
        LQ.xyz fPosIn,       3(pIn)
        // Convert normal. We might as well do the ADC at the same time, so
        // we don't need to bother when rendering.
        UnpackNormalADC fNormal, fUnpackScale
        // Convert the offset and bone weights.
        ITOF12.xyz fPosIn,       fPosIn
        ITOF15.xyz fBoneWeights, fBoneWeights
        // Bone 0.
        // Load bone matrix.
        ILW.x iBoneIdx, 1(pIn)
        LQ.xyz fBoneMat0, (ASURA_PS2_SS__ADDR__BONE_MATRIX_BFR + 0)(iBoneIdx)
        LQ.xyz fBoneMat1, (ASURA_PS2_SS__ADDR__BONE_MATRIX_BFR + 1)(iBoneIdx)
        LQ.xyz fBoneMat2, (ASURA_PS2_SS__ADDR__BONE_MATRIX_BFR + 2)(iBoneIdx)
        LQ.xyz fBoneMat3, (ASURA_PS2_SS__ADDR__BONE_MATRIX_BFR + 3)(iBoneIdx)
        // Rotate normal. We only need to do this for the first bone.
        MULAx.xyz  ACC,     fBoneMat0, fNormal
        MADDAy.xyz ACC,     fBoneMat1, fNormal
        MADDz.xyz  fNormal, fBoneMat2, fNormal        
        // Transform input vtx.
        MULAw.xyz  ACC,      fBoneMat3, VF00[w]
        MADDAx.xyz ACC,      fBoneMat0, fPosIn[x]
        MADDAy.xyz ACC,      fBoneMat1, fPosIn[y]
        MADDz.xyz  fPosTemp, fBoneMat2, fPosIn[z]
        // Scale transformed point by weighting.
        MULx.xyz fPosOut, fPosTemp, fBoneWeights[x]
        // Bone 1.
        // Load bone matrix.
        ILW.y iBoneIdx, 1(pIn)
        LQ.xyz fBoneMat0, (ASURA_PS2_SS__ADDR__BONE_MATRIX_BFR + 0)(iBoneIdx)
        LQ.xyz fBoneMat1, (ASURA_PS2_SS__ADDR__BONE_MATRIX_BFR + 1)(iBoneIdx)
        LQ.xyz fBoneMat2, (ASURA_PS2_SS__ADDR__BONE_MATRIX_BFR + 2)(iBoneIdx)
        LQ.xyz fBoneMat3, (ASURA_PS2_SS__ADDR__BONE_MATRIX_BFR + 3)(iBoneIdx)
        // Transform input vtx.
        MULAw.xyz  ACC,      fBoneMat3, VF00[w]
        MADDAx.xyz ACC,      fBoneMat0, fPosIn[x]
        MADDAy.xyz ACC,      fBoneMat1, fPosIn[y]
        MADDz.xyz  fPosTemp, fBoneMat2, fPosIn[z]
        // Scale transformed point by weighting and add to output position.
        MULy.xyz fPosTemp, fPosTemp, fBoneWeights[y]
        ADD.xyz fPosOut, fPosOut, fPosTemp
        // Bone 2.
        // Load bone matrix.
        ILW.z iBoneIdx, 1(pIn)
        LQ.xyz fBoneMat0, (ASURA_PS2_SS__ADDR__BONE_MATRIX_BFR + 0)(iBoneIdx)
        LQ.xyz fBoneMat1, (ASURA_PS2_SS__ADDR__BONE_MATRIX_BFR + 1)(iBoneIdx)
        LQ.xyz fBoneMat2, (ASURA_PS2_SS__ADDR__BONE_MATRIX_BFR + 2)(iBoneIdx)
        LQ.xyz fBoneMat3, (ASURA_PS2_SS__ADDR__BONE_MATRIX_BFR + 3)(iBoneIdx)
        // Transform input vtx.
        MULAw.xyz  ACC,      fBoneMat3, VF00[w]
        MADDAx.xyz ACC,      fBoneMat0, fPosIn[x]
        MADDAy.xyz ACC,      fBoneMat1, fPosIn[y]
        MADDz.xyz  fPosTemp, fBoneMat2, fPosIn[z]
        // Scale transformed point by weighting and add to output position.
        MULz.xyz fPosTemp, fPosTemp, fBoneWeights[z]
        ADD.xyz fPosOut, fPosOut, fPosTemp
        // Finished bones.
        // Overwrite input buffer data with skinned position and normal, and
        // also the converted ADC.
        SQ fNormal, 0(pIn)
        SQ fPosOut, 3(pIn)
        // Are we done yet?
        IADDIU pIn, pIn, 5
        IBNE pIn, pInEnd, VU1_SS_SKIN_VERTS_3_BONES_LOOP

    B VU1_SS_FINISHED_SKINNING

VU1_SS_SKIN_VERTS_4_BONES:

    VU1_SS_SKIN_VERTS_4_BONES_LOOP:

        --LoopCS  3, 0
        --LoopAbs 32

        // Load input vtx data.
        LQ     fNormal,      0(pIn)
        LQ     fBoneWeights, 2(pIn)
        LQ.xyz fPosIn,       3(pIn)
        // Convert normal. We might as well do the ADC at the same time, so
        // we don't need to bother when rendering.
        UnpackNormalADC fNormal, fUnpackScale
        // Convert the offset and bone weights.
        ITOF12.xyz fPosIn,       fPosIn
        ITOF15     fBoneWeights, fBoneWeights
        // Bone 0.
        // Load bone matrix.
        ILW.x iBoneIdx, 1(pIn)
        LQ.xyz fBoneMat0, (ASURA_PS2_SS__ADDR__BONE_MATRIX_BFR + 0)(iBoneIdx)
        LQ.xyz fBoneMat1, (ASURA_PS2_SS__ADDR__BONE_MATRIX_BFR + 1)(iBoneIdx)
        LQ.xyz fBoneMat2, (ASURA_PS2_SS__ADDR__BONE_MATRIX_BFR + 2)(iBoneIdx)
        LQ.xyz fBoneMat3, (ASURA_PS2_SS__ADDR__BONE_MATRIX_BFR + 3)(iBoneIdx)
        // Rotate normal. We only need to do this for the first bone.
        MULAx.xyz  ACC,     fBoneMat0, fNormal
        MADDAy.xyz ACC,     fBoneMat1, fNormal
        MADDz.xyz  fNormal, fBoneMat2, fNormal        
        // Transform input vtx.
        MULAw.xyz  ACC,      fBoneMat3, VF00[w]
        MADDAx.xyz ACC,      fBoneMat0, fPosIn[x]
        MADDAy.xyz ACC,      fBoneMat1, fPosIn[y]
        MADDz.xyz  fPosTemp, fBoneMat2, fPosIn[z]
        // Scale transformed point by weighting.
        MULx.xyz fPosOut, fPosTemp, fBoneWeights[x]
        // Bone 1.
        // Load bone matrix.
        ILW.y iBoneIdx, 1(pIn)
        LQ.xyz fBoneMat0, (ASURA_PS2_SS__ADDR__BONE_MATRIX_BFR + 0)(iBoneIdx)
        LQ.xyz fBoneMat1, (ASURA_PS2_SS__ADDR__BONE_MATRIX_BFR + 1)(iBoneIdx)
        LQ.xyz fBoneMat2, (ASURA_PS2_SS__ADDR__BONE_MATRIX_BFR + 2)(iBoneIdx)
        LQ.xyz fBoneMat3, (ASURA_PS2_SS__ADDR__BONE_MATRIX_BFR + 3)(iBoneIdx)
        // Transform input vtx.
        MULAw.xyz  ACC,      fBoneMat3, VF00[w]
        MADDAx.xyz ACC,      fBoneMat0, fPosIn[x]
        MADDAy.xyz ACC,      fBoneMat1, fPosIn[y]
        MADDz.xyz  fPosTemp, fBoneMat2, fPosIn[z]
        // Scale transformed point by weighting and add to output position.
        MULy.xyz fPosTemp, fPosTemp, fBoneWeights[y]
        ADD.xyz fPosOut, fPosOut, fPosTemp
        // Bone 2.
        // Load bone matrix.
        ILW.z iBoneIdx, 1(pIn)
        LQ.xyz fBoneMat0, (ASURA_PS2_SS__ADDR__BONE_MATRIX_BFR + 0)(iBoneIdx)
        LQ.xyz fBoneMat1, (ASURA_PS2_SS__ADDR__BONE_MATRIX_BFR + 1)(iBoneIdx)
        LQ.xyz fBoneMat2, (ASURA_PS2_SS__ADDR__BONE_MATRIX_BFR + 2)(iBoneIdx)
        LQ.xyz fBoneMat3, (ASURA_PS2_SS__ADDR__BONE_MATRIX_BFR + 3)(iBoneIdx)
        // Transform input vtx.
        MULAw.xyz  ACC,      fBoneMat3, VF00[w]
        MADDAx.xyz ACC,      fBoneMat0, fPosIn[x]
        MADDAy.xyz ACC,      fBoneMat1, fPosIn[y]
        MADDz.xyz  fPosTemp, fBoneMat2, fPosIn[z]
        // Scale transformed point by weighting and add to output position.
        MULz.xyz fPosTemp, fPosTemp, fBoneWeights[z]
        ADD.xyz fPosOut, fPosOut, fPosTemp
        // Bone 3.
        // Load bone matrix.
        ILW.w iBoneIdx, 1(pIn)
        LQ.xyz fBoneMat0, (ASURA_PS2_SS__ADDR__BONE_MATRIX_BFR + 0)(iBoneIdx)
        LQ.xyz fBoneMat1, (ASURA_PS2_SS__ADDR__BONE_MATRIX_BFR + 1)(iBoneIdx)
        LQ.xyz fBoneMat2, (ASURA_PS2_SS__ADDR__BONE_MATRIX_BFR + 2)(iBoneIdx)
        LQ.xyz fBoneMat3, (ASURA_PS2_SS__ADDR__BONE_MATRIX_BFR + 3)(iBoneIdx)
        // Transform input vtx.
        MULAw.xyz  ACC,      fBoneMat3, VF00[w]
        MADDAx.xyz ACC,      fBoneMat0, fPosIn[x]
        MADDAy.xyz ACC,      fBoneMat1, fPosIn[y]
        MADDz.xyz  fPosTemp, fBoneMat2, fPosIn[z]
        // Scale transformed point by weighting and add to output position.
        MULw.xyz fPosTemp, fPosTemp, fBoneWeights[w]
        ADD.xyz fPosOut, fPosOut, fPosTemp
        // Finished bones.
        // Overwrite input buffer data with skinned position and normal, and
        // also the converted ADC.
        SQ fNormal, 0(pIn)
        SQ fPosOut, 3(pIn)
        // Are we done yet?
        IADDIU pIn, pIn, 5
        IBNE pIn, pInEnd, VU1_SS_SKIN_VERTS_4_BONES_LOOP

VU1_SS_FINISHED_SKINNING:

    // Reset remaining cam flags.
    ILW.w iRemainingVisAndCullFlags, ASURA_PS2_SS__ADDR__VI_REG_STORE_1(VI00)
    // Reset cam data ptr.
    IADDIU pCurCamData, VI00, ASURA_PS2_SS__ADDR__CAM_DATA

VU1_SS_NEXT_CAM:

    // Prepare mask value for loop and setup first iteration test.
    IADDIU iVisBitMask, VI00, 0x80
    IAND iRenderForThisCam, iRemainingVisAndCullFlags, iVisBitMask
    // Any cameras left to render this strip for?
    IBEQ iRemainingVisAndCullFlags, VI00, VU1_SS_FINISHED_FOR_ALL_CAMS

    VU1_SS_FIND_NEXT_CAM_LOOP:

        // Render for this cam?
        IBNE iRenderForThisCam, VI00, VU1_SS_FOUND_NEXT_CAM
        // No. Shift flags up 2.
        IADD iRemainingVisAndCullFlags, iRemainingVisAndCullFlags, iRemainingVisAndCullFlags
        IADD iRemainingVisAndCullFlags, iRemainingVisAndCullFlags, iRemainingVisAndCullFlags
        // Setup next iteration test.
        IAND iRenderForThisCam, iRemainingVisAndCullFlags, iVisBitMask
        // And move ptr on.
        IADDIU pCurCamData, pCurCamData, ASURA_PS2_SS__OFFSET__CAM_DATA_NEXT_CAM
        // Go on to check for next cam.
        B VU1_SS_FIND_NEXT_CAM_LOOP

VU1_SS_FOUND_NEXT_CAM:

    // Check if we need to do any fiddling with the vertex data.
    IADDIU iMask, VI00, ASURA_PS2_SS__MASK__ANY_VTX_MANIPULATIONS
    IAND iAnyVtxManipulations, iMaterialFlags, iMask
    // Load in O->GBC matrix and GBC->S scale and translation vectors.
    MatrixLoad mO2GBC, ASURA_PS2_SS__OFFSET__CAM_DATA_O2GBC, pCurCamData
    LQ fGBC2SScale, ASURA_PS2_SS__OFFSET__CAM_DATA_GBC2S_SCALE(pCurCamData)
    LQ fGBC2STrans, ASURA_PS2_SS__OFFSET__CAM_DATA_GBC2S_TRANS(pCurCamData)
    // Any vertex manipulations flags required?
    IBEQ iAnyVtxManipulations, VI00, VU1_SS_POST_VTX_MANIPULATIONS

VU1_SS_VTX_MANIPULATIONS:

    // Setup branch conditions.
    IADDIU iMask, VI00, ASURA_PS2_SS__FLAG__TEX_ANIM
    IAND iTexAnim, iMaterialFlags, iMask
    IADDIU iMask, VI00, ASURA_PS2_SS__MASK__PASS_2
    IAND iPass2, iMaterialFlags, iMask
    // Any texture animation? If not, branch straight to pass 2 stuff - 
    // currently it's one or the other.
    IBEQ iTexAnim, VI00, VU1_SS_PASS_2_BFR_CREATE

VU1_SS_TEX_ANIM:

    // Reset input pointer.
    XTOP pIn
    // Load offset data.
    LQ.xy fSTOffset, ASURA_PS2_SS__ADDR__TEX_ANIM_DATA(VI00)
    
    VU1_SS_TEX_ANIM_LOOP:

        --LoopCS  3, 0
        --LoopAbs 5

        // Load and convert the input ST.
        LQ.xy     fSTQIn, 4(pIn)
        ITOF12.xy fSTQIn, fSTQIn
        // Apply ST offset.
        ADD.xy fSTQOut, fSTQIn, fSTOffset
        // Convert back and overwrite input.
        FTOI12.xy fSTQOut, fSTQOut
        SQ.xy     fSTQOut, 4(pIn)
        // Update ptr and check if we're done yet.
        IADDIU pIn, pIn, 5
        IBNE pIn, pInEnd, VU1_SS_TEX_ANIM_LOOP

    // Any pass 2 stuff as well?
    IBEQ iPass2, VI00, VU1_SS_POST_VTX_MANIPULATIONS

VU1_SS_PASS_2_BFR_CREATE:

    // NB: The pass 2 buffer is setup to allow an overrun of up to three
    // vertices, meaning we can deal with them up to four at a time, to
    // reduce latencies and allow us to do some stuff in parallel. This can
    // substantially decrease the number of cycles per vertex.

    // Ok, are we after a specular or reflective pass? Note that these are
    // mutually exclusive.
    IADDIU iMask, VI00, ASURA_PS2_SS__FLAG__REFLECTIVE
    IAND iPass2, iPass2, iMask
    // Setup input and output ptrs.
    XTOP pIn
    ILW.y pOut, ASURA_PS2_SS__ADDR__VI_REG_STORE_3(VI00)
    // And branch to appropriate pass 2 code.
    IBEQ iPass2, VI00, VU1_SS_SPECULAR_VALUE_CREATE

VU1_SS_REFLECTIVE_ST_CREATE:

    // Load cur tex mat bfr ptr.
    ILW.y pTexMat, ASURA_PS2_SS__ADDR__VI_REG_STORE_2(VI00)
    // Initialise ptr for search loop.
    IADDIU pCamDataSearch, VI00, ASURA_PS2_SS__ADDR__CAM_DATA

    // Here we figure out which tex mat to use based on the current main cam
    // data ptr. This is to save maintaining a ptr elsewhere that's only used
    // for stuff with more complex materials. There's only really any penalty
    // when multiple cameras are involved, and even then it's only a few
    // cycles.
    VU1_SS_CALC_TEX_MAT_PTR_LOOP:

        // Done?
        IBEQ pCamDataSearch, pCurCamData, VU1_SS_GOT_TEX_MAT_PTR
        // Nope. Keep searching.
        IADDIU pCamDataSearch, pCamDataSearch, ASURA_PS2_SS__OFFSET__CAM_DATA_NEXT_CAM
        IADDIU pTexMat, pTexMat, ASURA_PS2_SS__OFFSET__TEX_MAT_NEXT_CAM
        B VU1_SS_CALC_TEX_MAT_PTR_LOOP

VU1_SS_GOT_TEX_MAT_PTR:

    // Read in the texture matrix from the calculated address.
    LQ.xy mTex[0], 0(pTexMat)
    LQ.xy mTex[1], 1(pTexMat)
    LQ.xy mTex[2], 2(pTexMat)
    LQ.xy mTex[3], 3(pTexMat)
    // And the O->V matrix.
    LQ.xyz mO2V[0], (ASURA_PS2_SS__OFFSET__CAM_DATA_O2V + 0)(pCurCamData)
    LQ.xyz mO2V[1], (ASURA_PS2_SS__OFFSET__CAM_DATA_O2V + 1)(pCurCamData)
    LQ.xyz mO2V[2], (ASURA_PS2_SS__OFFSET__CAM_DATA_O2V + 2)(pCurCamData)
    LQ.xyz mO2V[3], (ASURA_PS2_SS__OFFSET__CAM_DATA_O2V + 3)(pCurCamData)

    VU1_SS_REFLECTIVE_ST_CREATE_LOOP:

        --LoopCS  3, 0
        --LoopAbs 21

        // Load objectspace normal and position.
        LQ.xyz fNrm, 0(pIn)
        LQ.xyz fPos, 3(pIn)
        // Transform position to viewspace.
        MULAw.xyz  ACC,  mO2V[3], VF00[w]
        MADDAx.xyz ACC,  mO2V[0], fPos[x]
        MADDAy.xyz ACC,  mO2V[1], fPos[y]
        MADDz.xyz  fPos, mO2V[2], fPos[z]
        // And the normal.
        MULAx.xyz  ACC,  mO2V[0], fNrm[x] 
        MADDAy.xyz ACC,  mO2V[1], fNrm[y]        
        MADDz.xyz  fNrm, mO2V[2], fNrm[z]
        // Calculate a viewspace cam->vertex direction. We can do this by 
        // just normalising the viewspace position.
        VectorNormalizeXYZ fDir, fPos
        // Calculate 2(D.N).
        VectorDotProduct fDot, fDir, fNrm
        ADD.x fDot, fDot, fDot
        // Calculate R = D - 2(D.N)N
        MULx.xyz fR, fNrm, fDot
        SUB.xyz  fR, fDir, fR
        // Apply texture coordinate matrix.
        MULAw.xy  ACC, mTex[3], VF00[w]
        MADDAx.xy ACC, mTex[0], fR[x]
        MADDAy.xy ACC, mTex[1], fR[y]
        MADDz.xy  fST, mTex[2], fR[z]
        // Output.
        SQI.xy fST, (pOut++)
        // Update loop state.
        IADDIU pIn, pIn, 5
        // Done yet?
        IBNE pIn, pInEnd, VU1_SS_REFLECTIVE_ST_CREATE_LOOP

    B VU1_SS_POST_VTX_MANIPULATIONS

VU1_SS_SPECULAR_VALUE_CREATE:

    // Determine which specular flag is set. They're mutually exclusive.
    IADDIU iMask, VI00, ASURA_PS2_SS__FLAG__SPECULAR_SOFT
    IAND iSoftSpecular, iMaterialFlags, iMask
    // Load objectspace camera position.
    LQ.xyz fCamPos, ASURA_PS2_SS__OFFSET__CAM_DATA_OBJ_SPACE_POS(pCurCamData)
    // Branch to appropriate loop.
    IBEQ iSoftSpecular, VI00, VU1_SS_SPECULAR_VALUE_SHARP_CREATE_LOOP

    // These loops deal with the vertices three at a time. This reduced the
    // cost per cycle from 13 to 8.7 and 15 to 9.3 respectively. It's worth
    // noting that both results were substantially better than a previous
    // effort using approximate normalisation and power calculations, which
    // produced a loop costing 12 cycles per vertex.

    VU1_SS_SPECULAR_VALUE_SOFT_CREATE_LOOP:

        --LoopCS  1, 0
        --LoopAbs 26

        // Load objectspace normal and position for next three vertices.
        LQ.xyz fNrm0, (0*5 + 0)(pIn)
        LQ.xyz fPos0, (0*5 + 3)(pIn)
        LQ.xyz fNrm1, (1*5 + 0)(pIn)
        LQ.xyz fPos1, (1*5 + 3)(pIn)
        LQ.xyz fNrm2, (2*5 + 0)(pIn)
        LQ.xyz fPos2, (2*5 + 3)(pIn)
        // Calculate V, the direction to the camera.
        SUB.xyz fV0, fCamPos, fPos0
        SUB.xyz fV1, fCamPos, fPos1
        SUB.xyz fV2, fCamPos, fPos2
        // Normalise. Use EFU for one...
        ERLENG P, fV0
        MFP.x fInvLen0, P
        MULx.xyz fV0, fV0, fInvLen0
        // ... and FDIV for the other two. The latency on RSQRT is such that
        // we can more or less fit in two while a single ERLENG completes.
        VectorNormalize fV1, fV1
        VectorNormalize fV2, fV2
        // Calculate N.V for each.
        MUL.xyz fTmp0, fNrm0, fV0
        ADDy.x fNdotV, fTmp0, fTmp0[y]
        ADDz.x fNdotV, fNdotV, fTmp0[z]
        MUL.xyz fTmp1, fNrm1, fV1
        ADDx.y fNdotV, fTmp1, fTmp1[x]
        ADDz.y fNdotV, fNdotV, fTmp1[z]
        MUL.xyz fTmp2, fNrm2, fV2
        ADDx.z fNdotV, fTmp2, fTmp2[x]
        ADDy.z fNdotV, fNdotV, fTmp2[y]
        // Clamp to >= 0.
        MAXx.xyz fNdotV, fNdotV, VF00
        // Raise to the 8th power.
        MUL.xyz fSpecularValues, fNdotV, fNdotV
        MUL.xyz fSpecularValues, fSpecularValues, fSpecularValues
        MUL.xyz fSpecularValues, fSpecularValues, fSpecularValues
        // Better clamp to <= 1 as well. May have slight inaccuracies above.
        MINIw.xyz fSpecularValues, fSpecularValues, VF00[w]
        // And output.
        SQI.x fSpecularValues, (pOut++)
        MR32.xy fSpecularValues, fSpecularValues
        SQI.x fSpecularValues, (pOut++)
        MR32.x fSpecularValues, fSpecularValues
        SQI.x fSpecularValues, (pOut++)
        // Update loop state.
        IADDIU pIn, pIn, (3*5)
        // Done yet?
        ISUB iRemainder, pInEnd, pIn
        IBGTZ iRemainder, VU1_SS_SPECULAR_VALUE_SOFT_CREATE_LOOP

    B VU1_SS_POST_VTX_MANIPULATIONS

    VU1_SS_SPECULAR_VALUE_SHARP_CREATE_LOOP:

        --LoopCS  1, 0
        --LoopAbs 28

        // Load objectspace normal and position for next three vertices.
        LQ.xyz fNrm0, (0*5 + 0)(pIn)
        LQ.xyz fPos0, (0*5 + 3)(pIn)
        LQ.xyz fNrm1, (1*5 + 0)(pIn)
        LQ.xyz fPos1, (1*5 + 3)(pIn)
        LQ.xyz fNrm2, (2*5 + 0)(pIn)
        LQ.xyz fPos2, (2*5 + 3)(pIn)
        // Calculate V, the direction to the camera.
        SUB.xyz fV0, fCamPos, fPos0
        SUB.xyz fV1, fCamPos, fPos1
        SUB.xyz fV2, fCamPos, fPos2
        // Normalise. Use EFU for one...
        ERLENG P, fV0
        MFP.x fInvLen0, P
        MULx.xyz fV0, fV0, fInvLen0
        // ... and FDIV for the other two. The latency on RSQRT is such that
        // we can more or less fit in two while a single ERLENG completes.
        VectorNormalize fV1, fV1
        VectorNormalize fV2, fV2
        // Calculate N.V for each.
        MUL.xyz fTmp0, fNrm0, fV0
        ADDy.x fNdotV, fTmp0, fTmp0[y]
        ADDz.x fNdotV, fNdotV, fTmp0[z]
        MUL.xyz fTmp1, fNrm1, fV1
        ADDx.y fNdotV, fTmp1, fTmp1[x]
        ADDz.y fNdotV, fNdotV, fTmp1[z]
        MUL.xyz fTmp2, fNrm2, fV2
        ADDx.z fNdotV, fTmp2, fTmp2[x]
        ADDy.z fNdotV, fNdotV, fTmp2[y]
        // Clamp to >= 0.
        MAXx.xyz fNdotV, fNdotV, VF00
        // Raise to the 32nd power.
        MUL.xyz fSpecularValues, fNdotV, fNdotV
        MUL.xyz fSpecularValues, fSpecularValues, fSpecularValues
        MUL.xyz fSpecularValues, fSpecularValues, fSpecularValues
        MUL.xyz fSpecularValues, fSpecularValues, fSpecularValues
        MUL.xyz fSpecularValues, fSpecularValues, fSpecularValues
        // Better clamp to <= 1 as well. May have slight inaccuracies above.
        MINIw.xyz fSpecularValues, fSpecularValues, VF00[w]
        // And output.
        SQI.x fSpecularValues, (pOut++)
        MR32.xy fSpecularValues, fSpecularValues
        SQI.x fSpecularValues, (pOut++)
        MR32.x fSpecularValues, fSpecularValues
        SQI.x fSpecularValues, (pOut++)
        // Update loop state.
        IADDIU pIn, pIn, (3*5)
        // Done yet?
        ISUB iRemainder, pInEnd, pIn
        IBGTZ iRemainder, VU1_SS_SPECULAR_VALUE_SHARP_CREATE_LOOP

VU1_SS_POST_VTX_MANIPULATIONS:

    // Reset input ptr.
    XTOP pIn
    // Setup output ptr.
    IADDIU pOut, pThisOutputBfr, ASURA_PS2_SS__OFFSET__GEOM_OUT_FIRST_VERTEX
    // Determine whether or not culling will be required.
    IADDIU iMask, VI00, 0x40
    IAND iCullReqd, iRemainingVisAndCullFlags, iMask
    // Fix up NLOOP in the GIF tag in the output buffer with the correct
    // vertex count.
    IADDIU iNLOOPandEOP, iVtxQty,      0x7FFF
    IADDIU iNLOOPandEOP, iNLOOPandEOP, 0x0001
    ISW.x  iNLOOPandEOP, ASURA_PS2_SS__OFFSET__GEOM_OUT_DRAW_GIF_TAG(pThisOutputBfr)
    // Load light data.
    LQ     afSHMatrix[0], (ASURA_PS2_SS__ADDR__LIGHT_DATA_BFR + 0)(VI00)
    LQ     afSHMatrix[1], (ASURA_PS2_SS__ADDR__LIGHT_DATA_BFR + 1)(VI00)
    LQ.xyz afSHMatrix[2], (ASURA_PS2_SS__ADDR__LIGHT_DATA_BFR + 2)(VI00)
    LQ.xyz afSHMatrix[3], (ASURA_PS2_SS__ADDR__LIGHT_DATA_BFR + 3)(VI00)
    // Fix Q component of texture QW at 1. This will be maintained through
    // either T&L loop below.
    MAXw.z fSTQIn, VF00, VF00
    // Prepare value ready in I register for clamping RGBAs.
    LOI 255.0

VU1_SS_T_AND_L:

    // Branch to faster path if no culling is required.
    IBEQ iCullReqd, VI00, VU1_SS_T_AND_L_NO_CULL
    // Reset all-clipped flag.
    IADDIU iAllClipped, VI00, 0x3F

    VU1_SS_T_AND_L_LOOP:

        --LoopCS  3, 0
        --LoopAbs 18

        // Load the vertex.
        LQ     fNrmAndADC, 0(pIn)
        LQ.xyz fOPos,      3(pIn)
        LQ.xy  fSTQIn,     4(pIn)
        // Convert input data. Note that we've already taken care of
        // unpacking the normal and ADC bit in the skinning loop above.
        ITOF12.xy fSTQIn, fSTQIn
        // Transform to GB clip space.
        MatrixMultiplyVertexW1 fGBCPos, mO2GBC, fOPos
        // Perform clipping judgement.
        CLIPw.xyz fGBCPos, fGBCPos
        // Scale and translate to screenspace.
        MULA  ACC,         fGBCPos,     fGBC2SScale
        MADDw fSPosPreDiv, fGBC2STrans, fGBCPos[w]
        // Perspective divide XYZ and STs.
        DIV      Q,     VF00[w],     fSPosPreDiv[w]
        MULq.xyz fSPos, fSPosPreDiv, Q
        MULq.xyz fSTQ,  fSTQIn,      Q
        // Light vertex.
        LightVtxWithSH fRGBA, fNrmAndADC, afSHMatrix
        // Clamp result to [0, 255].
        MINIi fRGBA, fRGBA, I
        // Get clip flags and AND them with iAllClipped. If iAllClipped is
        // anything other than 0 at the end, then all verts were outside one
        // of the planes and we can reject the entire strip. Note that this
        // isn't actually likely to allow us to reject much in general but we
        // have plenty of spare upper instructions in this loop anyway.
        FCGET iClipFlags
        IAND iAllClipped, iAllClipped, iClipFlags
        // Get clipping result.
        FCAND  VI01, 0x3FFFF
        IADDIU iClipADC, VI01, 0x7FFF
        // Combine the clip ADC with the degeneracy ADC.
        MTIR   iInADC,  fNrmAndADC[w]
        IOR    iOutADC, iInADC, iClipADC
        MFIR.w fSPos,   iOutADC 
        // Convert output data.
        FTOI4.xyz fSPos, fSPos
        FTOI0     fRGBA, fRGBA
        // Store.
        SQ.xyz fSTQ,  0(pOut)
        SQ     fRGBA, 1(pOut)
        SQ     fSPos, 2(pOut)
        // Update ptrs.
        IADDIU pIn,  pIn,  5
        IADDIU pOut, pOut, 3
        // Done yet?
        IBNE pIn, pInEnd, VU1_SS_T_AND_L_LOOP

    // Trivial rejection for entire strip being outside a clip plane.
    IBNE iAllClipped, VI00, VU1_SS_FINISHED_FOR_CUR_CAM
    // Otherwise head off to send it all to the GS.
    B VU1_SS_POST_T_AND_L

VU1_SS_T_AND_L_NO_CULL:

    // This is a slightly faster loop produced by concatenating the O->GBC->S
    // transforms into one matrix and skipping the CLIP check.
    MULA  ACC,     fGBC2SScale, mO2GBC[0]
    MADDw mO2S[0], fGBC2STrans, mO2GBC[0][w]
    MULA  ACC,     fGBC2SScale, mO2GBC[1]
    MADDw mO2S[1], fGBC2STrans, mO2GBC[1][w]
    MULA  ACC,     fGBC2SScale, mO2GBC[2]
    MADDw mO2S[2], fGBC2STrans, mO2GBC[2][w]
    MULA  ACC,     fGBC2SScale, mO2GBC[3]
    MADDw mO2S[3], fGBC2STrans, mO2GBC[3][w]

    VU1_SS_T_AND_L_NO_CULL_LOOP:

        --LoopCS  3, 0
        --LoopAbs 15

        // Load the vertex. Might as well load ADC bit directly to output
        // position QW.
        LQ.xyz fNrm,   0(pIn)
        LQ.w   fSPos,  0(pIn)
        LQ.xyz fOPos,  3(pIn)
        LQ.xy  fSTQIn, 4(pIn)
        // Convert input data. Note that we've already taken care of
        // unpacking the normal and ADC bit in the skinning loop above.
        ITOF12.xy fSTQIn, fSTQIn
        // Transform to GB clip space.
        MatrixMultiplyVertexW1 fSPosPreDiv, mO2S, fOPos
        // Perspective divide XYZ and STs.
        DIV      Q,     VF00[w],     fSPosPreDiv[w]
        MULq.xyz fSPos, fSPosPreDiv, Q
        MULq.xyz fSTQ,  fSTQIn,      Q
        // Light vertex.
        LightVtxWithSH fRGBA, fNrm, afSHMatrix
        // Clamp result to [0, 255].
        MINIi fRGBA, fRGBA, I
        // Convert output data.
        FTOI4.xyz fSPos, fSPos
        FTOI0     fRGBA, fRGBA
        // Store.
        SQ.xyz fSTQ,  0(pOut)
        SQ     fRGBA, 1(pOut)
        SQ     fSPos, 2(pOut)
        // Update ptrs.
        IADDIU pIn,  pIn,  5
        IADDIU pOut, pOut, 3
        // Done yet?
        IBNE pIn, pInEnd, VU1_SS_T_AND_L_NO_CULL_LOOP

VU1_SS_POST_T_AND_L:

    // Render.
    ILW.y pPass2Bfr, ASURA_PS2_SS__ADDR__VI_REG_STORE_3(VI00)
    BAL pRenderAllPassesRA, VU1_SS_RENDER_ALL_PASSES

VU1_SS_FINISHED_FOR_CUR_CAM:
        
    // Mask top two bits off cam flags so that next time we run the
    // VU1_SS_FIND_NEXT_CAM_LOOP we'll be forced to move ptrs on, etc.
    IADDIU iMask, VI00, 0x3F
    IAND iRemainingVisAndCullFlags, iRemainingVisAndCullFlags, iMask
    // And continue on to the next camera, if there is one.
    B VU1_SS_NEXT_CAM
    
VU1_SS_FINISHED_FOR_ALL_CAMS:

    // Shadow required?
    ILW.z iSkinFlags, ASURA_PS2_SS__ADDR__VI_REG_STORE_3(VI00)
    IADDIU iMask, VI00, ASURA_PS2_SS__FLAG__SHADOW
    IAND iShadowReqd, iSkinFlags, iMask
    IBEQ iShadowReqd, VI00, VU1_SS_END_STRIP

VU1_SS_RENDER_TO_SHADOW_BFR:

    // Load projection matrix.
    LQ mShadowProjMat[0], (ASURA_PS2_SS__ADDR__SHADOW_DATA_BFR + ASURA_PS2_SS__OFFSET__SHADOW_PROJ_MAT + 0)(VI00)
    LQ mShadowProjMat[1], (ASURA_PS2_SS__ADDR__SHADOW_DATA_BFR + ASURA_PS2_SS__OFFSET__SHADOW_PROJ_MAT + 1)(VI00)
    LQ mShadowProjMat[2], (ASURA_PS2_SS__ADDR__SHADOW_DATA_BFR + ASURA_PS2_SS__OFFSET__SHADOW_PROJ_MAT + 2)(VI00)
    LQ mShadowProjMat[3], (ASURA_PS2_SS__ADDR__SHADOW_DATA_BFR + ASURA_PS2_SS__OFFSET__SHADOW_PROJ_MAT + 3)(VI00)
    // Re-read vertex count from ITOP. Loading it again, rather than forcing
    // VCL to preserve the value, will help with register pressure.
    XITOP iVtxQty
    // Prepare input and output ptrs.
    XTOP pIn
    IADDIU pOut, pThisOutputBfr, ASURA_PS2_SS__OFFSET__SHADOW_OUT_FIRST_VTX
    // And an end-of-output ptr. It's easier to calculate end-of-output here,
    // rather than the usual end-of-input, since we're only writing 1 QW per
    // vtx.
    IADD pOutEnd, pOut, iVtxQty

    // Deal with the vertices 3 at a time. This allows us to amortise some of
    // the latencies involved in the 1/w operations. The output buffer has
    // plenty of space for this since we're only writing 1 QW per vertex, as
    // opposed to 3 in the other render loops. This brought the cost per
    // vertex down to 6, from 8. (8 is, in fact, the lower limit when
    // processing one vertex per iteration, due to the DIV instruction's
    // latency.)
    VU1_SS_RENDER_TO_SHADOW_BFR_LOOP:

        --LoopCS  1, 0
        --LoopABS 18

        // Load next three input positions.
        LQ.xyz fOPos0, (0*5 + 3)(pIn)
        LQ.xyz fOPos1, (1*5 + 3)(pIn)
        LQ.xyz fOPos2, (2*5 + 3)(pIn)
        // Transform each.
        MatrixMultiplyVertexW1 fSPosPreDiv0, mShadowProjMat, fOPos0
        MatrixMultiplyVertexW1 fSPosPreDiv1, mShadowProjMat, fOPos1
        MatrixMultiplyVertexW1 fSPosPreDiv2, mShadowProjMat, fOPos2
        // Calculate 1/w and perform perspective divide. Use EFU for 1 and
        // FDIV for the other.
        ERCPR P, fSPosPreDiv0[w]
        DIV Q, VF00[w], fSPosPreDiv1[w]
        MULq.xyz fSPosPreDiv1, fSPosPreDiv1, Q
        MFP.w fOneOverSPos0W, P
        MULw.xyz fSPosPreDiv0, fSPosPreDiv0, fOneOverSPos0W[w]
        DIV Q, VF00[w], fSPosPreDiv2[w]
        MULq.xyz fSPosPreDiv2, fSPosPreDiv2, Q
        // Convert positions for GS.
        FTOI4.xyz fSPos0, fSPosPreDiv0
        FTOI4.xyz fSPos1, fSPosPreDiv1
        FTOI4.xyz fSPos2, fSPosPreDiv2
        // Load ADC bits.
        LQ.w fSPos0, (0*5 + 0)(pIn)
        LQ.w fSPos1, (1*5 + 0)(pIn)
        LQ.w fSPos2, (2*5 + 0)(pIn)
        // Output.
        SQ fSPos0, 0(pOut)
        SQ fSPos1, 1(pOut)
        SQ fSPos2, 2(pOut)
        // Update loop state.
        IADDIU pIn,  pIn,  (3*5)
        IADDIU pOut, pOut, (3*1)
        // Done yet?
        ISUB iRemainder, pOutEnd, pOut
        IBGTZ iRemainder, VU1_SS_RENDER_TO_SHADOW_BFR_LOOP

    // Load GIF tags and GS setup QWs.
    LQ     fSetupGIFTag, (ASURA_PS2_SS__ADDR__GIF_TAGS + ASURA_PS2_SS__OFFSET__GIF_TAGS_SHADOW_SETUP)(VI00)
    LQ     fRGBAQ,       (ASURA_PS2_SS__ADDR__CONST_SHADOW_DATA + ASURA_PS2_SS__OFFSET__CONST_SHADOW_RGBAQ)(VI00)
    LQ.yzw fDrawGIFTag,  (ASURA_PS2_SS__ADDR__GIF_TAGS + ASURA_PS2_SS__OFFSET__GIF_TAGS_SHADOW_DRAW)(VI00)
    // Write out to output bfr.
    SQ     fSetupGIFTag, ASURA_PS2_SS__OFFSET__SHADOW_OUT_SETUP_GIF_TAG(pThisOutputBfr)
    SQ     fRGBAQ,       ASURA_PS2_SS__OFFSET__SHADOW_OUT_RGBAQ(pThisOutputBfr)
    SQ.yzw fDrawGIFTag,  ASURA_PS2_SS__OFFSET__SHADOW_OUT_DRAW_GIF_TAG(pThisOutputBfr)
    // Load output buffer addresses - we'll need them in a moment.
    ILW.x pFirstOutputBfr,  ASURA_PS2_SS__ADDR__VI_REG_STORE_4(VI00)
    ILW.y pSecondOutputBfr, ASURA_PS2_SS__ADDR__VI_REG_STORE_4(VI00)
    // Fix up NLOOP in draw GIF tag.
    IADDIU iNLOOPandEOP, iVtxQty, 0x7FFF
    IADDIU iNLOOPandEOP, iNLOOPandEOP, 0x1
    ISW.x iNLOOPandEOP, ASURA_PS2_SS__OFFSET__SHADOW_OUT_DRAW_GIF_TAG(pThisOutputBfr)
    // Send that off to the GS.
    XGKICK pThisOutputBfr
    // Switch output bfrs ready for next time.
    SwitchBfr_Regs pThisOutputBfr, pFirstOutputBfr, pSecondOutputBfr
    // And we're done.
    B VU1_SS_END_STRIP

VU1_SS_RENDER_ALL_PASSES:

VU1_SS_RENDER_PASS_0:

    // Load visible-for-more-than-one-cam and need-to-send-RS flags.
    ILW.x iMoreThanOneCamCanSeeFlag, ASURA_PS2_SS__ADDR__VI_REG_STORE_2(VI00)
    ILW.w iNeedToSendRSFlag,         ASURA_PS2_SS__ADDR__VI_REG_STORE_4(VI00)
    // Load output buffer addresses; they'll be required below.
    ILW.x pFirstOutputBfr,  ASURA_PS2_SS__ADDR__VI_REG_STORE_4(VI00)
    ILW.y pSecondOutputBfr, ASURA_PS2_SS__ADDR__VI_REG_STORE_4(VI00)
    // Update the need-to-send-RS flag. We want to reset it to 0 unless there
    // are multiple cameras (or if we discover below that there are multiple
    // passes required but we'll deal with that at that point.)
    ISW.w iMoreThanOneCamCanSeeFlag, ASURA_PS2_SS__ADDR__VI_REG_STORE_4(VI00)
    // Setup RS ptrs, in case they're required.
    ILW.w pRSInBfr, ASURA_PS2_SS__ADDR__VI_REG_STORE_3(VI00)
    IADDIU pRSOutBfr, VI00, ASURA_PS2_SS__ADDR__RS_OUTPUT_BFR
    // Skip sending RS if it's not required.
    IBEQ iNeedToSendRSFlag, VI00, VU1_SS_RENDER_PASS_0_KICK

VU1_SS_RENDER_PASS_0_RS:

    // Build the base pass GS setup list. Note that the input setup GIF tag
    // is assumed to have the correct NLOOP for the base pass setup.
    LQ fSetupGIFTag, (ASURA_PS2_SS__ADDR__GIF_TAGS + ASURA_PS2_SS__OFFSET__GIF_TAGS_GEOM_SETUP)(VI00)
    LQ fSCISSOR,     ASURA_PS2_SS__OFFSET__CAM_DATA_SCISSOR(pCurCamData)
    LQ fXYOFFSET,    ASURA_PS2_SS__OFFSET__CAM_DATA_XYOFFSET(pCurCamData)
    LQ fALPHA,       ASURA_PS2_SS__OFFSET__RS_IN_PASS_0_ALPHA(pRSInBfr)
    LQ fTEX1,        ASURA_PS2_SS__OFFSET__RS_IN_PASS_0_TEX1(pRSInBfr)
    LQ fTEX0,        ASURA_PS2_SS__OFFSET__RS_IN_PASS_0_TEX0(pRSInBfr)
    LQ fMIPTBP1,     ASURA_PS2_SS__OFFSET__RS_IN_PASS_0_MIPTBP1(pRSInBfr)
    LQ fTEST,        ASURA_PS2_SS__OFFSET__RS_IN_PASS_0_TEST(pRSInBfr)
    // Copy to RS output bfr.
    SQ fSetupGIFTag, ASURA_PS2_SS__OFFSET__RS_OUT_0_SETUP_GIF_TAG(pRSOutBfr)
    SQ fSCISSOR,     ASURA_PS2_SS__OFFSET__RS_OUT_0_SCISSOR(pRSOutBfr)
    SQ fXYOFFSET,    ASURA_PS2_SS__OFFSET__RS_OUT_0_XYOFFSET(pRSOutBfr)
    SQ fALPHA,       ASURA_PS2_SS__OFFSET__RS_OUT_0_ALPHA(pRSOutBfr)
    SQ fTEX1,        ASURA_PS2_SS__OFFSET__RS_OUT_0_TEX1(pRSOutBfr)
    SQ fMIPTBP1,     ASURA_PS2_SS__OFFSET__RS_OUT_0_MIPTBP1(pRSOutBfr)
    SQ fTEX0,        ASURA_PS2_SS__OFFSET__RS_OUT_0_TEX0(pRSOutBfr)
    SQ fTEST,        ASURA_PS2_SS__OFFSET__RS_OUT_0_TEST(pRSOutBfr)
    // Send it.
    XGKICK pRSOutBfr

VU1_SS_RENDER_PASS_0_KICK:

    // Copy last 3/4 of draw GIF tag to output bfr. The first word (NLOOP +
    // EOP) will already have been dealt with.
    LQ.yzw fDrawGIFTag, (ASURA_PS2_SS__ADDR__GIF_TAGS + ASURA_PS2_SS__OFFSET__GIF_TAGS_GEOM_DRAW)(VI00)
    SQ.yzw fDrawGIFTag, ASURA_PS2_SS__OFFSET__GEOM_OUT_DRAW_GIF_TAG(pThisOutputBfr)
    // Keep ptr to current output bfr and then switch.
    IADDIU pOtherOutputBfr, pThisOutputBfr, 0
    SwitchBfr_Regs pThisOutputBfr, pFirstOutputBfr, pSecondOutputBfr
    // Determine whether or not we have further passes to do.
    IADDIU iMask, VI00, ASURA_PS2_SS__MASK__MULTIPASS
    IAND iMultipass, iMaterialFlags, iMask
    // Send the pending bfr.
    XGKICK pOtherOutputBfr
    // Continue with other passes if required.
    IBNE iMultipass, VI00, VU1_SS_MULTIPASS_REQUIRED
    // Otherwise we're done.
    JR pRenderAllPassesRA

VU1_SS_MULTIPASS_REQUIRED:

    // Copy draw GIF tag to new output bfr. We'll then have it ready it in
    // both buffers, regardless of how many more passes we end up doing.
    // Unlike above, we now want to copy the NLOOP+EOP word as well now.
    LQ fDrawGIFTag, ASURA_PS2_SS__OFFSET__GEOM_OUT_DRAW_GIF_TAG(pOtherOutputBfr)
    SQ fDrawGIFTag, ASURA_PS2_SS__OFFSET__GEOM_OUT_DRAW_GIF_TAG(pThisOutputBfr)
    // Setup 0x7FFF and 0x8000 in registers. We'll need them for setting up
    // further passes' GIF tags.
    IADDIU i0x7FFF, VI00,    0x7FFF
    IADDIU i0x8000, i0x7FFF, 0x0001
    // Copy NLOOP+EOP from draw GIF tag to integer register and mask off EOP,
    // to get vertex count. Note that we can't simply re-read the vtx count
    // from ITOP since we may come in here from clipping code, in which case
    // the vtx count could be anything.
    MTIR iVtxQty, fDrawGIFTag[x]
    IAND iVtxQty, iVtxQty, i0x7FFF
    // Ensure the need-to-send-RS is set for next time we render a base pass.
    // Note that writing any non-zero value will do here.
    ISW.w iVtxQty, ASURA_PS2_SS__ADDR__VI_REG_STORE_4(VI00)

VU1_SS_RENDER_PASS_1:

    // Check if we need the self illuminated pass. If not then we must need
    // pass 3 since otherwise none of the multipass flags would have been on.
    IADDIU iMask, VI00, ASURA_PS2_SS__FLAG__SELFILLUMINATED
    IAND iSelfIllum, iMaterialFlags, iMask
    IBEQ iSelfIllum, VI00, VU1_SS_RENDER_PASS_2
    // Build the self-illuminated pass GS setup list.
    LQ fALPHA, ASURA_PS2_SS__OFFSET__RS_IN_PASS_1_ALPHA(pRSInBfr)
    SQ fALPHA, ASURA_PS2_SS__OFFSET__RS_OUT_1_ALPHA(pRSOutBfr)
    // Correct NLOOP+EOP word for pass 1 RS QW count.
    IADDIU iNLOOPandEOP, i0x8000, ASURA_PS2_SS__RS_OUT_1_NLOOP
    ISW.x iNLOOPandEOP, ASURA_PS2_SS__OFFSET__RS_OUT_1_SETUP_GIF_TAG(pRSOutBfr)
    // Setup input and output ptrs.
    IADDIU pCopyIn,  pOtherOutputBfr, ASURA_PS2_SS__OFFSET__GEOM_OUT_FIRST_VERTEX
    IADDIU pCopyOut, pThisOutputBfr,  ASURA_PS2_SS__OFFSET__GEOM_OUT_FIRST_VERTEX
    // Setup end-of-input ptr.
    IADD i2xVtxQty,  iVtxQty,   iVtxQty
    IADD i3xVtxQty,  i2xVtxQty, iVtxQty
    IADD pCopyInEnd, pCopyIn,   i3xVtxQty
    // Setup fullbright colour. Note that we'll grab the alpha from the first
    // pass vertices.
    LOI 128.0
    MAXi.xyz fRGBA, VF00, I
    FTOI0.xyz fRGBA, fRGBA

    VU1_SS_PASS_1_LOOP:

        --LoopCS  3, 0
        --LoopAbs 7

        // Get data from other buffer.
        LQI.xyz fSTQ,     (pCopyIn++)
        LQI.w   fRGBA,    (pCopyIn++)
        LQI     fScrnPos, (pCopyIn++)
        // Copy to new buffer, with modified RGB.
        SQI.xyz fSTQ,     (pCopyOut++)
        SQI     fRGBA,    (pCopyOut++)
        SQI     fScrnPos, (pCopyOut++)
        // Done yet?
        IBNE pCopyIn, pCopyInEnd, VU1_SS_PASS_1_LOOP

    // Send the prepared RS bfr.
    XGKICK pRSOutBfr
    // Swap output bfrs.
    Swap pThisOutputBfr, pOtherOutputBfr
    // Determine whether or not we have further passes to do.
    IADDIU iMask, VI00, ASURA_PS2_SS__MASK__PASS_2
    IAND iNeedPass2, iMaterialFlags, iMask
    // Send pass 1 geometry.
    XGKICK pOtherOutputBfr
    // Continue with other passes if required.
    IBNE iNeedPass2, VI00, VU1_SS_RENDER_PASS_2
    // Otherwise we're done.
    JR pRenderAllPassesRA

VU1_SS_RENDER_PASS_2:

    // Reflective flag present?
    IADDIU iMask, VI00, ASURA_PS2_SS__FLAG__REFLECTIVE
    IAND iReflective, iMaterialFlags, iMask
    // Setup input and output ptrs.
    IADDIU pCopyIn,  pOtherOutputBfr, ASURA_PS2_SS__OFFSET__GEOM_OUT_FIRST_VERTEX
    IADDIU pCopyOut, pThisOutputBfr,  ASURA_PS2_SS__OFFSET__GEOM_OUT_FIRST_VERTEX
    // Setup end-of-input ptr. Easiest to do it with pass 2 bfr ptr, since
    // that's just 1 QW per vertex.
    IADD pPass2BfrEnd, pPass2Bfr, iVtxQty
    // Branch to appropriate render path.
    IBEQ iReflective, VI00, VU1_SS_RENDER_PASS_2_SPECULAR

VU1_SS_RENDER_PASS_2_REFLECTIVE:

    // Build the reflective pass setup list.
    LQ fALPHA,   ASURA_PS2_SS__OFFSET__RS_IN_PASS_2_ALPHA(pRSInBfr)
    LQ fTEX1,    ASURA_PS2_SS__OFFSET__RS_IN_PASS_2_TEX1(pRSInBfr)
    LQ fTEX0,    ASURA_PS2_SS__OFFSET__RS_IN_PASS_2_TEX0(pRSInBfr)
    LQ fMIPTBP1, ASURA_PS2_SS__OFFSET__RS_IN_PASS_2_MIPTBP1(pRSInBfr)
    SQ fALPHA,   ASURA_PS2_SS__OFFSET__RS_OUT_2_REFL_ALPHA(pRSOutBfr)
    SQ fTEX1,    ASURA_PS2_SS__OFFSET__RS_OUT_2_REFL_TEX1(pRSOutBfr)
    SQ fMIPTBP1, ASURA_PS2_SS__OFFSET__RS_OUT_2_REFL_MIPTBP1(pRSOutBfr)
    SQ fTEX0,    ASURA_PS2_SS__OFFSET__RS_OUT_2_REFL_TEX0(pRSOutBfr)
    // Correct NLOOP+EOP word for RS QW count.
    IADDIU iNLOOPandEOP, i0x8000, ASURA_PS2_SS__RS_OUT_2_REFL_NLOOP
    ISW.x iNLOOPandEOP, ASURA_PS2_SS__OFFSET__RS_OUT_2_REFL_GIF_TAG(pRSOutBfr)

    VU1_SS_PASS_2_REFLECTIVE_LOOP:

        --LoopCS  3, 0
        --LoopAbs 8

        // Load data from pass 2 bfr and other output buffer.
        LQI.xy fSTQ,     (pPass2Bfr++)
        LQI.z  fSTQ,     (pCopyIn++)
        LQI    fRGBA,    (pCopyIn++)
        LQI    fScrnPos, (pCopyIn++)
        // Perspective divide texture coordinates.
        MULz.xy fSTQ, fSTQ, fSTQ[z]
        // Copy to new output buffer.
        SQI.xyz fSTQ,     (pCopyOut++)
        SQI     fRGBA,    (pCopyOut++)
        SQI     fScrnPos, (pCopyOut++)
        // Done yet?
        IBNE pPass2Bfr, pPass2BfrEnd, VU1_SS_PASS_2_REFLECTIVE_LOOP

    B VU1_SS_RENDER_PASS_2_KICK

VU1_SS_RENDER_PASS_2_SPECULAR:

    // Build the specular pass setup list.
    LQ fALPHA, ASURA_PS2_SS__OFFSET__RS_IN_PASS_2_ALPHA(pRSInBfr)
    SQ fALPHA, ASURA_PS2_SS__OFFSET__RS_OUT_2_SPEC_ALPHA(pRSOutBfr)
    // Correct NLOOP+EOP word for RS QW count.
    IADDIU iNLOOPandEOP, i0x8000, ASURA_PS2_SS__RS_OUT_2_SPEC_NLOOP
    ISW.x iNLOOPandEOP, ASURA_PS2_SS__OFFSET__RS_OUT_2_SPEC_GIF_TAG(pRSOutBfr)
    // Overwrite draw GIF tag with special specular pass one. We don't need
    // to send STs and the PRIM setting is different. Note that NLOOP and EOP
    // are still correct from previous pass(es).
    LQ.yzw fDrawGIFTag, (ASURA_PS2_SS__ADDR__GIF_TAGS + ASURA_PS2_SS__OFFSET__GIF_TAGS_GEOM_DRAW_SPEC)(VI00)
    SQ.yzw fDrawGIFTag, ASURA_PS2_SS__OFFSET__GEOM_OUT_DRAW_GIF_TAG(pThisOutputBfr)

    VU1_SS_PASS_2_SPECULAR_LOOP:

        --LoopCS  3, 0
        --LoopAbs 7

        // Load data from pass 2 bfr and other output buffer.
        LQI.x   fSpecValue, (pPass2Bfr++)
        LQ.xyz  fRGBA,      1(pCopyIn)
        LQ      fScrnPos,   2(pCopyIn)
        // Convert RGBA from GS format.
        ITOF0.xyz fRGBA, fRGBA
        // Scale by specular value.
        MULx.xyz fRGBA, fRGBA, fSpecValue[x]
        // Convert back to GS format.
        FTOI0.xyz fRGBA, fRGBA
        // And store to new output buffer.
        SQI.xyz fRGBA,    (pCopyOut++)
        SQI     fScrnPos, (pCopyOut++)
        // Update loop state.
        IADDIU pCopyIn, pCopyIn, 3
        // Done yet?
        IBNE pPass2Bfr, pPass2BfrEnd, VU1_SS_PASS_2_SPECULAR_LOOP

VU1_SS_RENDER_PASS_2_KICK:

    // Send the prepared RS bfr.
    XGKICK pRSOutBfr
    // Swap output bfrs.
    Swap pThisOutputBfr, pOtherOutputBfr
    // Send pass 2 geometry.
    XGKICK pOtherOutputBfr
    // And we're done.
    JR pRenderAllPassesRA

.end
