//////////////////////////////////////////////////////////////////////////////
//
// Asura_PS2_ShellMCUI.h
//
// Copyright (c) 2003 Rebellion. All rights reserved.
//
// Description: Asura Interface to the Sony Memory Card User Interface 
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef ___ASURA_PS2_SHELLMCUI_H___
#define ___ASURA_PS2_SHELLMCUI_H___ 1

// asura includes
#include "Core/Asura_PlatformBasics.h"
#include "Core/Asura_Maths.h"

// middleware includes
#include "Sony/PS2Shell/shellMCUI.h"


// Allow other projects to override this define in EngineConfiguration if needed

#ifndef ASURA_PS2_SHELLMCUI_MAX_NUM_DATA_FILES
	#define ASURA_PS2_SHELLMCUI_MAX_NUM_DATA_FILES		(5)
#endif


class Asura_PS2_ShellMCUI_LoadParams
{
	public:

		char 	m_szDirNameExtension[SHELLMEMCARD_MAX_LEN_FILENAME];

		//Data file info:
		int		m_iNumDataFiles;
		char	m_szDataFileNames[ASURA_PS2_SHELLMCUI_MAX_NUM_DATA_FILES][SHELLMEMCARD_MAX_LEN_FILENAME];

};


class Asura_PS2_ShellMCUI_SaveParams
{
	public:

		char 	m_szDirNameExtension[SHELLMEMCARD_MAX_LEN_FILENAME];
		
		char	m_szTitleName[64];
		
		// User file info
		int		m_iUserFileSize;
		char 	*m_pcUserFileData;

		// Data file info:
		int		m_iNumDataFiles;
		char	m_szDataFileNames[ASURA_PS2_SHELLMCUI_MAX_NUM_DATA_FILES][SHELLMEMCARD_MAX_LEN_FILENAME];
		char*	m_pcDataFileData [ASURA_PS2_SHELLMCUI_MAX_NUM_DATA_FILES];
		int		m_iDataFileSizes [ASURA_PS2_SHELLMCUI_MAX_NUM_DATA_FILES];
};



class Asura_PS2_ShellMCUI
{
	public:

		//Main User screens:
		enum eUserScreen
		{
			SCREEN_NULL		= -1,
			SCREEN_MAIN		= 0,
			SCREEN_LIST_SAVES,
			SCREEN_LOAD,
			SCREEN_SAVE,
	
			SCREEN_MAX	
		}; 

		enum eAsura_PS2_ShellMCUI_Icon_BackgroundPosition
		{
			TOP_LEFT = 0,
			TOP_RIGHT,
			BOTTOM_LEFT,
			BOTTOM_RIGHT,
			
			NUMBER_POSITIONS,
		};

		enum eStatusMessageType
		{
			STATUS_MSG_TYPE_CARD = SHELLMCUI_STATUS_TYPE_CARD,	// "PS2 Card inserted, not inserted, unformatted" etc
			STATUS_MSG_TYPE_OPERATION,							// "Loading", "Load failed", etc
			STATUS_MSG_TYPE_REQUEST_USER,						// "Format Card?", "Overwrite existing save?", etc  
			STATUS_MSG_TYPE_USER_INPUT_INFO,					// "X to continue", "Triangle to cancel", etc
			
			STATUS_MSG_MAX,
		};

		static bool Initialise( void );
		static void Shutdown( void );

		static void Update( void );

		static bool SetMemoryCardIcon( const char* szIconName );

		static bool IsMainScreen	( void )	{ return s_eScreen == SHELLMCUI_SCREEN_MAIN; }
		static bool	HasScreenChanged( void )	{ return s_ePrevScreen == s_eScreen;		 }

		static bool IsMainMenuLoadOptionAvailable( void );
		static bool IsMainMenuSaveOptionAvailable( void );

		static bool IsCardDetecting		( void );
		static bool IsCardPresent		( void );
		static bool IsCardUnformatted	( void );

		static bool CardChanged			( void )	{ return s_bCardChanged; }
		static void ResetCardChangedFlag( void )	{ s_bCardChanged = false; }

		static const int GetNumberOfSavesFound( void );

		static const int GetFreeSpaceOnCard( void );

		static const char* GetSaveNameForIndex( const int iIndex ); 

		static bool IsLoading		( void );
		static bool LoadSuccessful	( void );
		static bool LoadFailed		( void );

		static bool IsSaving		( void );
		static bool SaveSuccessful	( void );
		static bool SaveFailed		( void );

		static bool IsFormatting	( void );
		static bool FormatSuccessful( void );
		static bool FormatFailed	( void );

		static const char* GetLoadedDataPtrForIndex( const int iIndex ); 
		static void	FreeLoadedDataPtrForIndex( const int iIndex ); 

		static const int GetLoadedDataSizeForIndex	( const int iIndex );

		static void ResetAllDataPtrs( void );

		static bool IsOverwriteRequested( void );
		static bool IsFormatRequested	( void );
		
		static bool IsInitialised( void ) { return s_bInitialised; }

		static const Asura_UnicodeChar* GetStatusMsg(const eStatusMessageType eStatusMsgType);

		// Background transparency range is 0 (fully transparent) to 0x80 (fully opaque)
		static void SetBackgroundTransparency(const unsigned char ucTrans);

		static void SetBackgroundColour(const eAsura_PS2_ShellMCUI_Icon_BackgroundPosition eBackPos, 
										const unsigned char ucR, 
										const unsigned char ucG, 
										const unsigned char ucB);

		// 3 lightsources allowed - colour in 0..1
		static void SetLightSource(	const int iLightSourceNumber, 
									const Asura_Vector_3& xLightSourcePosition,
									const Asura_Vector_3& xLightSourceColour);

		// Ambient colour / light
		static void SetAmbient(	const float fR, 
								const float fG, 
								const float fB);


	private:

		static void SetCurrentUserScreen(EShellMCUIScreen eMCUIScreen); 

		static void UpdateMainScreen( void ); 
		static void UpdateLoadScreen( void ); 
		static void UpdateSaveScreen( void); 
		static void UpdateListSavesScreen ( void ); 

		static void UpdateUserInput ( void ); 

		static bool SetSaveParams( const Asura_PS2_ShellMCUI_SaveParams &xSaveParams );
		static bool SetLoadParams( const Asura_PS2_ShellMCUI_LoadParams &xLoadParams );

		static void SetIconSys( const char *szTitleName );

		static void CheckForCardChanged( void );

		// project-supplied functions
		static const char* Project_GetProductCode( void );
		static int Project_GetMaxNumSavesPerCard( void );

		static bool Project_LoadScreenRequested( void );
		static bool Project_SaveScreenRequested( void );

		static bool Project_ContinueToLoading( void );
		static bool Project_ContinueToSaving( void );

		static void Project_GetLoadParams( Asura_PS2_ShellMCUI_LoadParams &xLoadParams );
		static void Project_GetSaveParams( Asura_PS2_ShellMCUI_SaveParams &xSaveParams );

		static void Project_RequestInputUserContinue( void );
		static bool Project_GetInputUserContinue( void );

		static void Project_RequestInputUserCancel( void );
		static bool Project_GetInputUserCancel( void );

		static void Project_RequestInputUserYesNo( void );
		static bool Project_GetInputUserYes( void );
		static bool Project_GetInputUserNo( void );
		
		// data
		static bool s_bInitialised;
		
		static char s_acKeyCode[2];
		static char s_acProductCode[11];

		static char	s_acFileName[7];		// first six characters of save name


		static char*				s_pcIconBuffer; 

		static sceMcIconSys			s_xIconSysFile;

		static ShellMCUIOutputs		s_xOutputParms;
		static ShellMCUIInputs 		s_xInputParms;

		static EShellMCUIScreen 	s_eScreen;
		static EShellMCUIScreen 	s_ePrevScreen;

		static bool					s_bSavesFound;

		static bool					s_bCardChanged;
		static bool					s_bCardWasInserted;

};

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

#endif // #ifndef ___ASURA_GAMESPY_QR2_H___
