//////////////////////////////////////////////////////////////////////////////
//
// Asura_PS2_CoronaDefines.h
//
// Copyright (c) 2003 Rebellion. All rights reserved.
//
// Constants for use by Asura_Platform_Corona.cpp and VU1_Coronas.vcl.
//
// (MP - 03.09.2002) NB: Don't use the string <hash>define in comments in 
// this file because VCL chokes on it.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef ___ASURA_PS2_CORONADEFINES_H___
#define ___ASURA_PS2_CORONADEFINES_H___ 1

// M A C R O S ///////////////////////////////////////////////////////////////

// We have 1024 QWs to play with. We have the following overhead:
//
//     cam data (52 QWs currently)
//     input buffer overhead (2 QWs per buffer = 4 QWs currently)
//     output buffer overhead (4 QWs per buffer = 8 QWs currently)
//
// That's a total of 64 QWs currently. We require our max particles per batch
// M to satisfy
//
//     2 * 3 * M + 2 * 9 * M <= 1024 - 64
//
// since input coronas currently use 3 QWs and output ones 9.
//
// Hence
//
//     M <=   1024 - 64   == 40
//          -------------
//          2 * 3 + 2 * 9
//
// Obviously this needs to be kept up to date with changes to the defines
// below. The #if check at the bottom of this section will enforce this.
#define ASURA_PS2_CORONA_MAX_CORONAS_PER_BATCH (40)

// Cam data.
#define ASURA_PS2_CORONA_NUM_QWS_PER_VIEWPORT (2)
#define ASURA_PS2_CORONA_NUM_QWS_PER_CAM      (ASURA_PS2_CORONA_NUM_QWS_PER_VIEWPORT + 2 * 4 + 1 + 2)
                                                  // Viewport setup + matrices + pos + corona axes.
#define ASURA_PS2_CORONA_MAX_CAM_QWS          (4 * ASURA_PS2_CORONA_NUM_QWS_PER_CAM)

// Input coronas.
#define ASURA_PS2_CORONA_NUM_QWS_PER_INPUT_CORONA (3) // Pos & size (in w), RGBA, dir & use dir flag (in w).
#define ASURA_PS2_CORONA_INPUT_BUFFERS_SIZE       (ASURA_PS2_CORONA_MAX_CORONAS_PER_BATCH * ASURA_PS2_CORONA_NUM_QWS_PER_INPUT_CORONA) // Just corona data in input.

// Output coronas.
#define ASURA_PS2_CORONA_NUM_QWS_PER_OUTPUT_CORONA (1 + 4 * 2) // RGBA + UV & XYZ per corner.
#define ASURA_PS2_CORONA_OUTPUT_BUFFERS_SIZE       (1 + ASURA_PS2_CORONA_NUM_QWS_PER_VIEWPORT + 1 + ASURA_PS2_CORONA_MAX_CORONAS_PER_BATCH * ASURA_PS2_CORONA_NUM_QWS_PER_OUTPUT_CORONA)
                                                       // GIF tag, viewport setup, GIF tag, corona data.

// Memory map.
#define ASURA_PS2_CORONA_CAM_DATA_BASE       (0)
#define ASURA_PS2_CORONA_INPUT_BUFFERS_BASE  (ASURA_PS2_CORONA_CAM_DATA_BASE + ASURA_PS2_CORONA_MAX_CAM_QWS)
#define ASURA_PS2_CORONA_OUTPUT_BUFFERS_BASE (ASURA_PS2_CORONA_INPUT_BUFFERS_BASE + 2 * ASURA_PS2_CORONA_INPUT_BUFFERS_SIZE)

// Check it all adds up...
#if ASURA_PS2_CORONA_OUTPUT_BUFFERS_BASE + 2 * ASURA_PS2_CORONA_OUTPUT_BUFFERS_SIZE > 1024
#error Need to reduce ASURA_PS2_CORONA_MAX_CORONAS_PER_BATCH.
#endif

// Rendering parameters.
#define ASURA_PS2_CORONA_FADE_DIST_BEGIN 60.0 // NB: This and _END need to be kept in sync with independent code.
#define ASURA_PS2_CORONA_FADE_DIST_END   75.0

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

#endif // #ifndef ___ASURA_PS2_CORONADEFINES_H___
