//////////////////////////////////////////////////////////////////////////////
//
// Asura_PS2_RenderParticle.cpp
//
// Copyright (c) 2002-2004 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header.
#include "SpecialFX/Asura_PS2_RenderParticle.h"

// Engine independent.
#include "Core/Asura_TextureManagement.h"
#include "Core/Asura_Timers.h"
#include "Light/Asura_GI_Solution.h"
#include "Rendering/Asura_Colour.h"

#ifdef ASURA_USE_ORIGINAL_PARTICLE_SYSTEM
#error ASURA_USE_ORIGINAL_PARTICLE_SYSTEM is not supported anymore
#endif //ASURA_USE_ORIGINAL_PARTICLE_SYSTEM

#include "SpecialFX/pfx/Asura_PFX_Classes.h"
#include "SpecialFX/PFX/Asura_PFX_Effect.h"
#include "SpecialFX/PFX/Asura_PFX_System.h"
#include "SpecialFX/PFX/Asura_PFX_Templates.h"

// Engine platform-specific.
#include "Core/Asura_PS2_DMA.h"
#include "Profiling/Asura_PS2_Profiling.h"
#include "Rendering/Asura_PlatformRendering.h"
#include "Rendering/Asura_PlatformRenderStates.h"
#include "Rendering/Asura_PlatformTextureManagement.h"
#include "Rendering/Asura_PS2_DeferredRendering.h"
#include "Rendering/Asura_PS2_GS.h"
#include "Rendering/Asura_PS2_GS_Util.h"
#include "Rendering/Asura_PS2_RenderingOptions.h"
#include "Rendering/Asura_PS2_VU1.h"
#include "SpecialFX/Asura_PS2_ParticleDefines.h"

// JAT 05/06/06 this code needs updating to work with streaming
#if defined( ASURA_USE_STREAMING )
#error THE PS2 PARTICLE CODE NEEDS UPDATING TO WORK WITH STREAMING
#endif // #if defined( ASURA_USE_STREAMING )

#define ASURA_PS2_PFX_SUPPORT_MULTIPLE_TEXTURES

// M A C R O S ///////////////////////////////////////////////////////////////

#define PARTICLES_MMAG (dbg_bParticlesBilinear ? SCE_GS_LINEAR : SCE_GS_NEAREST)
#define PARTICLES_MMIN (dbg_bParticlesBilinear ? SCE_GS_LINEAR_MIPMAP_NEAREST : SCE_GS_NEAREST_MIPMAP_NEAREST)

// C O N S T A N T S /////////////////////////////////////////////////////////

static const u_int NUM_QWS_FOR_PACKET_SETUP = 3;
static const u_int MAX_PARTICLE_DATA_QWS    = ASURA_PS2_PARTICLE_MAX_PARTICLES * ASURA_PS2_PARTICLE_INPUT_QWS_PER_PARTICLE;
static const u_int MAX_PACKET_QWC           = NUM_QWS_FOR_PACKET_SETUP + MAX_PARTICLE_DATA_QWS + 1;
static const u_int MAX_PACKET_TOTAL_QWS     = 1 + MAX_PACKET_QWC;

// G L O B A L S /////////////////////////////////////////////////////////////

DEBUGVAR bool dbg_bParticlesBilinear = true;

DEBUGVAR int dbg_iHeatHazeURandom = 21;
DEBUGVAR int dbg_iHeatHazeVRandom = 10;
DEBUGVAR float dbg_fHeatHazeSinAmplitudeU = 21.0f;
DEBUGVAR float dbg_fHeatHazeSinAmplitudeV = 10.0f;
DEBUGVAR float dbg_fHeatHazeSinRateU1 = 5.0f;
DEBUGVAR float dbg_fHeatHazeSinRateV1 = 6.0f;
DEBUGVAR float dbg_fHeatHazeSinAngleScaleUU1 = Asura_Maths::Deg360/(256.0f*16.0f);// input is shifted left by 4 i.e. 16.
DEBUGVAR float dbg_fHeatHazeSinAngleScaleUV1 = Asura_Maths::Deg360/(256.0f*16.0f);
DEBUGVAR float dbg_fHeatHazeSinAngleScaleVU1 = Asura_Maths::Deg360/(256.0f*16.0f);
DEBUGVAR float dbg_fHeatHazeSinAngleScaleVV1 = Asura_Maths::Deg360/(256.0f*16.0f);
DEBUGVAR float dbg_fHeatHazeSinRateU2 = 21.0f;
DEBUGVAR float dbg_fHeatHazeSinRateV2 = 22.0f;
DEBUGVAR float dbg_fHeatHazeSinAngleScaleUU2 = Asura_Maths::Deg360/(256.0f*16.0f);// input is shifted left by 4 i.e. 16.
DEBUGVAR float dbg_fHeatHazeSinAngleScaleUV2 = Asura_Maths::Deg360/(256.0f*16.0f);
DEBUGVAR float dbg_fHeatHazeSinAngleScaleVU2 = Asura_Maths::Deg360/(256.0f*16.0f);
DEBUGVAR float dbg_fHeatHazeSinAngleScaleVV2 = Asura_Maths::Deg360/(256.0f*16.0f);

DEBUGVAR bool dbg_bTestHeathazeFirstPass = false;	// disables second pass
DEBUGVAR bool dbg_bTestHeathazeFirstPass2 = false;	// sets normal translucency instead of writing just alpha
DEBUGVAR bool dbg_bTestHeathazeSecondPass = false;	// uses the normal texture instead of heathaze one and prevents setting of flag that causes STs to be calculated on VU
DEBUGVAR bool dbg_bOnlyHeathaze = false;
DEBUGVAR bool dbg_bDisplayHeathazeTexture = false;
DEBUGVAR u_int dbg_uDisplayHeathazeTextureX = 0;
DEBUGVAR u_int dbg_uDisplayHeathazeTextureY = 0;
DEBUGVAR u_int dbg_uDisplayHeathazeTextureW = 1<<uASURA_PS2_DEFORMMESH_SIZE_LOG2;
DEBUGVAR u_int dbg_uDisplayHeathazeTextureH = 1<<uASURA_PS2_DEFORMMESH_SIZE_LOG2;

static bool                           g_bInitialised = false;
#ifndef ASURA_PS2_PFX_SUPPORT_MULTIPLE_TEXTURES
static int                            g_iParticleTexture = ASURA_TEXTURE_NONE;
#endif
static Asura_PS2_GifTag               g_xParticleGifTag;
static Asura_PS2_GifTag               g_xClippedGifTag;
static Asura_PS2_GifTag               g_xCameraSettingsGifTag;
static u_int                          g_uNumParticlesToAdd;
static u_int                          g_uNumParticlesInThisPacket;
static const Asura_Particle_Template* g_pxCurParticleTemplate;
static u_int                          g_uFlagsForCurTemplate;
static u_int                          g_uCurCamFlags;
static bool                           g_bAnyRotations;
static u_int                          g_uPacketBufferID;
static Asura_PS2_QW*                  g_pxPacketBase;
static Asura_PS2_QW*                  g_pxPacket;
static ASURA_TRANSLUCENCY_MODE        g_eLastTranslucencyMode;
static bool                           g_bIsHeathazePass;
static int                            g_iHeathazePassNumber;
static float                          g_fEffectOffsetScale;
static bool							  g_bSendCameraVectors;
static u_int						  g_uNumActiveCameras;
static Asura_Vector_3				  g_xCameraVectors[ASURA_PS2_PARTICLE_MAX_CAMERAS*2];

extern u_int VU1_PARTICLERENDERER_NEW_BILLBOARD[] __attribute__(( section(".vudata") ));

// P R O T O T Y P E S   A N D   I N L I N E   F U N C T I O N S /////////////

static void InitVU1( void );
static void SendNewAlignmentVectors( const Asura_Vector_3& xXalignment, const Asura_Vector_3& xYalignment );
static void SendNewCameraVectors( void );

static void StartNewPacket( void );
static void EndPacket( float fTopLeftU, float fTopLeftV, float fBottomRightU, float fBottomRightV );

static void CreateHeatHazeTexture( void );
static void SetRenderstateForHeathazeFirstPass( const Asura_PFX_Particle_Template* pxTemplate );
static void SetRenderstateForHeathazeSecondPass( const Asura_PFX_Particle_Template* pxTemplate );

static void PFX_BeginEffectRender( const Asura_PFX_Particle_Template* pxTemplate, bool bAlwaysOnTop );
static void PFX_AddParticles_NoPrevPos( const Asura_PFX_Particle* pxParticles,
                                        u_int uNumParticles,
                                        const Asura_PFX_Effect* pxEffect,
                                        const Asura_PFX_Particle_Template* pxTemplate,
                                        const Asura_RGBA& xLighting );
static void PFX_AddParticles_WithPrevPos( const Asura_PFX_Particle* pxParticles,
                                          u_int uNumParticles,
                                          const Asura_PFX_Effect* pxEffect,
                                          const Asura_PFX_Particle_Template* pxTemplate,
                                          const Asura_RGBA& xLighting );

// C L A S S   I M P L E M E N T A T I O N S /////////////////////////////////

//
// Asura_PS2_ParticleSystem.
//

void Asura_PS2_ParticleSystem::Initialise( void )
{
    Asura_Assert( !g_bInitialised, "" );
    g_bInitialised = true;

#ifndef ASURA_PS2_PFX_SUPPORT_MULTIPLE_TEXTURES
    // Sort out texture.
    {
        Asura_Assert( ASURA_TEXTURE_NONE == g_iParticleTexture, "" );

        bool bWasNew;
        g_iParticleTexture = Asura_Texture_Management::AddTexture( "\\specialfx\\partclfx.tm2", &bWasNew );
        if( bWasNew ) { Asura_Texture_Management::LoadTextures(); }
    }
#endif

    // setup the camera settings gif tag
    g_xCameraSettingsGifTag.Set(
        2,                  // Loop count
        false,              // End of packet flag.
        false,              // Primitive field enable flag.
        0,                  // Value to be written to PRIM register, if PRE is true.
        SCE_GIF_PACKED,     // Data format.
        1,                  // Number of registers set per loop.
        SCE_GIF_PACKED_AD   // register description
    );

    // setup the gif tag used for every particle
    // Setup prim value.
    u_int uPRIM;
    uPRIM = SCE_GS_SET_PRIM(
                    SCE_GS_PRIM_TRISTRIP,   // PRIM (Type of drawing primitive)
                    0,                      // IIP (Flat Shading/Gouraud Shading)
                    1,                      // TME (Texture Mapping Enabled)
                    0,                      // FGE (Fogging Enabled)
                    1,                      // ABE (Alpha Blending Enabled)
                    0,                      // AA1 (1 Pass Anti-Aliasing Enabled)
                    0,                      // FST (Method of specifying Texture Coords)
                    0,                      // CTXT (Context)
                    0                       // FIX (Fragment Value Control)
    );

    g_xParticleGifTag.Set(
        0,                  // Loop count - will be setup on VU for each strip.
        true,               // End of packet flag.
        true,               // Primitive field enable flag.
        uPRIM,              // Value to be written to PRIM register, if PRE is true.
        SCE_GIF_PACKED,     // Data format.
		12,                 // Number of registers set per loop.
		SCE_GS_ST,          // 1st register written: UV0
		SCE_GS_RGBAQ,       // 2nd register written: colour
		SCE_GS_XYZ2,        // 3rd register written: vertex pos 0
		SCE_GS_ST,          // 4th register written: UV1
		SCE_GS_RGBAQ,       // 5th register written: colour
		SCE_GS_XYZ2,        // 6th register written: vertex pos 1
		SCE_GS_ST,          // 7th register written: UV2
		SCE_GS_RGBAQ,       // 8th register written: colour
		SCE_GS_XYZ2,        // 9th register written: vertex pos 2
		SCE_GS_ST,          // 10th register written: UV3
		SCE_GS_RGBAQ,       // 11th register written: colour
		SCE_GS_XYZ2         // 12th register written: vertex pos 3
    );

	g_xClippedGifTag.Set(
		0,                  // Loop count - will be setup on VU for each strip.
		true,               // End of packet flag.
		true,               // Primitive field enable flag.
		uPRIM,              // Value to be written to PRIM register, if PRE is true.
		SCE_GIF_PACKED,     // Data format.
		3,					// Number of registers set per loop.
		SCE_GS_ST,          // 1st register written: UV0
		SCE_GS_RGBAQ,       // 2nd register written: colour
		SCE_GS_XYZ2         // 3rd register written: vertex pos
		);
}


void Asura_PS2_ParticleSystem::Reset( void )
{
#ifndef ASURA_PS2_PFX_SUPPORT_MULTIPLE_TEXTURES
    g_iParticleTexture = ASURA_TEXTURE_NONE;
#endif
    g_bInitialised = false;
}


void Asura_PS2_ParticleSystem::Shutdown( void )
{
    Asura_Assert( g_bInitialised, "" );
    g_bInitialised = false;
}

void Asura_PS2_ParticleSystem::BeginFrame( bool bHeatHaze )
{
    if( !Asura_PS2_RenderingOptions::GetOption(Asura_PS2_RenderingOptions::OPTION_DRAW_PARTICLES) )  { return; }

    g_bIsHeathazePass = bHeatHaze;

    // (MH - 25.06.2002) do this init here rather than at startup
    // so that texture management is initialised
    if( !g_bInitialised )
    {
        Initialise();
    }

    ASURA_PS2_PROFILE__ADD_VIF_START( ASURA_PS2_PROFILE__PARTICLE_MARKER );

#ifdef ASURA_PS2_PFX_SUPPORT_MULTIPLE_TEXTURES

	Asura_PS2_TextureManagement::SetBufferLayout( ASURA_PS2_TEXBFRLAYOUT_2_LARGE );

#else
    // Set texture buffer layout.
    // (MP - 29.10.2003) For now, just disable auto buffer switching here. We
    // only allow a single particle texture so we can just restrict that to
    // buffer 0 to ensure it's safe for the heat haze stuff to use 1.
    Asura_PS2_TextureManagement::SetBufferLayout( ASURA_PS2_TEXBFRLAYOUT_2_LARGE, false );    

    //blocking upload
    bool bTextureUploaded = Asura_PS2_TextureManagement::UploadTexture( g_iParticleTexture, 0 );

    // setup the texture registers
    Asura_PS2_TextureManagement::SetupTextureEnvironment( bTextureUploaded,
                                                          ASURA_PS2_TEXREGFLAG_ALL,
                                                          g_iParticleTexture,
                                                          0,
                                                          PARTICLES_MMAG,
                                                          PARTICLES_MMIN );
#endif

    if (g_bIsHeathazePass)
    {
        // make the wobbly texture in 2nd texture buffer
        CreateHeatHazeTexture();

        // set renderstate for drawing particles after drawing that texture
        Asura_PlatformRenderStates::CreateDefaultRenderStatePacket( Asura_PlatformRenderStates::CONTEXT_1 );
        Asura_PlatformRenderStates::SetZBufferWriteEnabled(false);
        Asura_PlatformRenderStates::ApplyRenderStates();
    }


    // upload microprogram
    Asura_PS2_VU1::LoadRenderer( Asura_PS2_VU1::RENDERER_PARTICLES );

    // setup VU1 double buffer, upload camera data, shared GIF tags etc., kick VU
    InitVU1();

    g_eLastTranslucencyMode = ASURA_TRANSLUCENCY_NOT_SET;
}

void Asura_PS2_ParticleSystem::EndFrame( void )
{
    if( !Asura_PS2_RenderingOptions::GetOption(Asura_PS2_RenderingOptions::OPTION_DRAW_PARTICLES) ) { return; }

    // Need a FLUSH before any other renderer starts fiddling with stuff.
    {
        Asura_PS2_DmaTag* pxTag = reinterpret_cast<Asura_PS2_DmaTag*>( Asura_PS2_DeferredRendering::GetVifBuffer(1) );
        pxTag->Set( 0,
                    Asura_PS2_DmaTag::PCE_NOCHANGE,
                    Asura_PS2_DmaTag::ID_CNT,
                    false,
                    0,
                    false,
                    SCE_VIF1_SET_NOP(false),
                    SCE_VIF1_SET_FLUSH(false) );
    }

	if (g_bIsHeathazePass && dbg_bDisplayHeathazeTexture)
	{
	    Asura_PS2_GS_Util::RenderArea( dbg_uDisplayHeathazeTextureX, dbg_uDisplayHeathazeTextureY, dbg_uDisplayHeathazeTextureW, dbg_uDisplayHeathazeTextureH,
                                       0, 0, 1 << uASURA_PS2_DEFORMMESH_SIZE_LOG2, 1 << uASURA_PS2_DEFORMMESH_SIZE_LOG2, 
                                       Asura_PS2_GS::GetDrawBfrFBP(), Asura_PS2_GS::GetScreenWidth(),  
                                       Asura_PS2_TextureManagement::GetBfrTBP(1), 1 << uASURA_PS2_DEFORMMESH_SIZE_LOG2, 1 << uASURA_PS2_DEFORMMESH_SIZE_LOG2, 
                                       0, 0x80808080, SCE_GS_SET_ALPHA( SCE_GS_ALPHA_CS, SCE_GS_ALPHA_CS, SCE_GS_ALPHA_FIX, SCE_GS_ALPHA_CS, 0),
                                       SCE_GS_PSMCT32, 0 );
	}

    if( g_bIsHeathazePass ) { Asura_PS2_DeferredRendering::AddPath3Gate(); }

    ASURA_PS2_PROFILE__ADD_VIF_END( ASURA_PS2_PROFILE__PARTICLE_MARKER );
}

// should be static functions but they're not so project code can extern and use them for debugging.
u_int Asura_PS2_ParticleHeatHazeGetU(u_int uU, u_int uV)
{
	int iRandomOffset;
    if (Asura_Timers::GetGameTimeScale() == 0.0f)
    {
        iRandomOffset = 0;
    }
	else
	{
		iRandomOffset = Asura_Random::GetInt(-dbg_iHeatHazeURandom, dbg_iHeatHazeURandom);
	}
	float fAngle1 = (dbg_fHeatHazeSinRateU1 * Asura_Timers::GetLocalScaledTime()) + 
		(dbg_fHeatHazeSinAngleScaleUU1 * static_cast<float>(static_cast<int>(uU))) + 
		(dbg_fHeatHazeSinAngleScaleUV1 * static_cast<float>(static_cast<int>(uV)));
	float fAngle2 = (dbg_fHeatHazeSinRateU2 * Asura_Timers::GetLocalScaledTime()) + 
		(dbg_fHeatHazeSinAngleScaleUU2 * static_cast<float>(static_cast<int>(uU))) + 
		(dbg_fHeatHazeSinAngleScaleUV2 * static_cast<float>(static_cast<int>(uV)));
	int iSinOffset = Asura_Maths::FloatToInt(dbg_fHeatHazeSinAmplitudeU * (Asura_Maths::Sine(fAngle1) + Asura_Maths::Sine(fAngle2)) );
    int iResult = static_cast<int>(uU) + iRandomOffset + iSinOffset;
	return (iResult > 0) ? static_cast<u_int>(iResult) : 0;
}
u_int Asura_PS2_ParticleHeatHazeGetV(u_int uU, u_int uV)
{
	int iRandomOffset;
    if (Asura_Timers::GetGameTimeScale() == 0.0f)
    {
        iRandomOffset = 0;
    }
	else
	{
		iRandomOffset = Asura_Random::GetInt(-dbg_iHeatHazeVRandom, dbg_iHeatHazeVRandom);
	}

	float fAngle1 = (dbg_fHeatHazeSinRateV1 * Asura_Timers::GetLocalScaledTime()) + 
		(dbg_fHeatHazeSinAngleScaleVU1 * static_cast<float>(static_cast<int>(uU))) + 
		(dbg_fHeatHazeSinAngleScaleVV1 * static_cast<float>(static_cast<int>(uV)));
	float fAngle2 = (dbg_fHeatHazeSinRateV2 * Asura_Timers::GetLocalScaledTime()) + 
		(dbg_fHeatHazeSinAngleScaleVU2 * static_cast<float>(static_cast<int>(uU))) + 
		(dbg_fHeatHazeSinAngleScaleVV2 * static_cast<float>(static_cast<int>(uV)));
	int iSinOffset = Asura_Maths::FloatToInt(dbg_fHeatHazeSinAmplitudeV * (Asura_Maths::Sine(fAngle1) + Asura_Maths::Sine(fAngle2)) );
    int iResult = static_cast<int>(uV) + iRandomOffset + iSinOffset;
	return (iResult > 0) ? static_cast<u_int>(iResult) : 0;
}

//DEBUGVAR u_int dbg_uHeathazeQuad[2][2] = { { 96<<4, 250<<4 }, { 150<<4, 340<<4 } };
//DEBUGVAR u_int dbg_uHeathazeQuadZ = 1000000;
//DEBUGVAR u_int dbg_uUVOffset = 8;
//static void RenderHeathazeParticles()
//{
//  u_int uNumVerts = 2;
//  u_int uNLOOP = 4 + (2 * uNumVerts);// contexts regs plus vertex regs
//
//  // Create a dma tag for all this
//  Asura_PS2_QW *pxVifBuffer = Asura_PS2_DeferredRendering::GetVifBuffer(uNLOOP+2);
//  pxVifBuffer->u32s[0] = PY_DMA_CNT | (uNLOOP+1);
//  pxVifBuffer->u32s[1] = 0;
//  pxVifBuffer->u32s[2] = SCE_VIF1_SET_NOP( false );
//  pxVifBuffer->u32s[3] = SCE_VIF1_SET_DIRECT( uNLOOP+1, false );
//  pxVifBuffer++;
//
//  // Create a gif tag for it
//  pxVifBuffer->u64s[0] = SCE_GIF_SET_TAG( uNLOOP,         // NLOOP
//                                          1,                  // EOP
//                                          NULL,               // Prim Enable
//                                          NULL,               // Prim
//                                          SCE_GIF_PACKED, // PACKET type
//                                          1 );                // Register quantity
//  pxVifBuffer->u64s[1] = SCE_GIF_PACKED_AD;
//  pxVifBuffer++;
//
//  // Set the alpha blend mode
//  pxVifBuffer->u64s[0] = SCE_GS_SET_ALPHA(0, 1, 2, 1, dbg_ucHeatHazeAlphaValue); //(SRC-FB)*FIX + FB;
//  pxVifBuffer->u64s[1] = SCE_GS_ALPHA_1;
//  pxVifBuffer++;
//
//  // Set PRIM for sprite drawing with alpha blend
//  pxVifBuffer->u64s[0] = SCE_GS_SET_PRIM( SCE_GS_PRIM_SPRITE,     // PRIM Type
//                                          0,                      // IIP (Flat/Gouraud Shading)
//                                          1,                      // TME (Texture Mapping Enabled)
//                                          0,                      // FGE (Fogging Enabled)
//                                          1,                      // ABE (Alpha Blending Enabled)
//                                          0,                      // AA1 (1 Pass Anti-Aliasing Enabled)
//                                          1,                      // FST (Method of specifying Texture Coords)
//                                          0,                      // CTXT (Context)
//                                          0 );                    // FIX (Fragment Value Control)
//  pxVifBuffer->u64s[1] = SCE_GS_PRIM;
//  pxVifBuffer++;
//
//  // Set Test register to use Z buffer test
//  pxVifBuffer->u64s[0] = SCE_GS_SET_TEST_1( 0,                    // ATE
//                                            0,                    // ATST  Alpha test method
//                                            0,                    // AREF  Alpha Comparison
//                                            0,                    // AFAIL When failed
//                                            0,                    // DATE  Destination alpha test
//                                            0,                    // DATM  Dest. Alpha test mode
//                                            1,                    // ZTE   Depth Test enable
//                                            SCE_GS_ZGREATER );   // ZTST  Depth Test Method
//  pxVifBuffer->u64s[1] = SCE_GS_TEST_1;
//  pxVifBuffer++;
//
//  // Disable Z writes.
//  pxVifBuffer->u64s[0] = SCE_GS_SET_ZBUF( ASURA_PS2_ZBFR_BASE_PTR, Asura_PS2_GS::GetZBfrPSM(), 1 );
//  pxVifBuffer->u64s[1] = SCE_GS_ZBUF_1;
//  pxVifBuffer++;
//
//  u_int uTopLeftX = (2048-256)<<4;
//  u_int uTopLeftY = (2048-224)<<4;
//  // top-left
//  pxVifBuffer->u64s[0] = SCE_GS_SET_UV(dbg_uHeathazeQuad[0][0] + dbg_uUVOffset, dbg_uHeathazeQuad[0][1] + dbg_uUVOffset);
//  pxVifBuffer->u64s[1] = SCE_GS_UV;
//  pxVifBuffer++;
//
//  pxVifBuffer->u64s[0] = SCE_GS_SET_XYZ(uTopLeftX + dbg_uHeathazeQuad[0][0], uTopLeftY + dbg_uHeathazeQuad[0][1], dbg_uHeathazeQuadZ);
//  pxVifBuffer->u64s[1] = SCE_GS_XYZ2;
//  pxVifBuffer++;
//
//  // bottom-right
//  pxVifBuffer->u64s[0] = SCE_GS_SET_UV(dbg_uHeathazeQuad[1][0] + dbg_uUVOffset, dbg_uHeathazeQuad[1][1] + dbg_uUVOffset);
//  pxVifBuffer->u64s[1] = SCE_GS_UV;
//  pxVifBuffer++;
//
//  pxVifBuffer->u64s[0] = SCE_GS_SET_XYZ(uTopLeftX + dbg_uHeathazeQuad[1][0], uTopLeftY + dbg_uHeathazeQuad[1][1], dbg_uHeathazeQuadZ);
//  pxVifBuffer->u64s[1] = SCE_GS_XYZ2;
//  pxVifBuffer++;
//
//}


void Asura_PFX_System::Platform_BeginRender( void )
{
    Asura_PS2_ParticleSystem::BeginFrame( s_bIsRenderingHeatHazeSources );
}

void Asura_PFX_System::Platform_EndRender( void )
{
    Asura_PS2_ParticleSystem::EndFrame();
}

bool Asura_PFX_System::Platform_SupportsHeatHaze( void )
{
#ifdef ASURA_PS2_PFX_SUPPORT_MULTIPLE_TEXTURES
	return false;
#endif

	// (MH - 04.03.2005) disabled in split-screen until it gets fixed.
	if (Asura_Camera::GetNumberOfCameras() != 1)
	{
		return false;
	}
    return true;
}

//
// Asura_PFX_Effect.
//

void Asura_PFX_Effect::Platform_RenderParticles( const Asura_PFX_Particle* pxParticles,
                                                 u_int uNumParticles,
                                                 const Asura_PFX_Particle_Template* pxTemplate,
                                                 const Asura_RGBA& xLightColour ) const
{
	Asura_Assert( m_uCameraFlagsFrameStamp == Asura_Timers::GetFrameNumber(), "This code assumes camera flags have been calculated this frame" );

    if( !Asura_PS2_RenderingOptions::GetOption(Asura_PS2_RenderingOptions::OPTION_DRAW_PARTICLES) ) { return; }

    if( dbg_bOnlyHeathaze && !g_bIsHeathazePass ) { return; }

	// It is valid to pass in zero particles (source could just have started or ended. Simply return)
	if(!uNumParticles) return;

	Asura_Assert(pxParticles, "Null particle pointer given");
	Asura_Assert(pxTemplate, "Null particle template pointer given");

    Asura_Assert( m_ucCameraFlags, "Shouldn't call this function unless there are cameras to render to" );
    g_uCurCamFlags = m_ucCameraFlags;

#ifdef ASURA_PS2_PFX_SUPPORT_MULTIPLE_TEXTURES
	// load our texture...
	bool bTextureUploaded = Asura_PS2_TextureManagement::UploadTexture( pxTemplate->m_iTextureHandle );

	// setup the texture registers
	Asura_PS2_TextureManagement::SetupTextureEnvironment( bTextureUploaded,
		ASURA_PS2_TEXREGFLAG_ALL,
		pxTemplate->m_iTextureHandle,
		Asura_PS2_TextureManagement::BFR_AUTO,
		PARTICLES_MMAG,
		PARTICLES_MMIN );
#endif

	Asura_Vector_3 xDeltaX;
	Asura_Vector_3 xDeltaY;
	bool bSendAlignVectors = false;
	if( pxTemplate->m_bAlignEffectX )
	{
		const Asura_Matrix_3x3* pxOrientation = GetOrientation();
		if ( pxOrientation )
		{
			xDeltaX = Asura_Vector_3( pxOrientation->e12, pxOrientation->e22, pxOrientation->e32 );
			xDeltaY = Asura_Vector_3( pxOrientation->e13, pxOrientation->e23, pxOrientation->e33 );
			bSendAlignVectors = true;
		}
	}
	else if ( pxTemplate->m_bAlignEffectY )
	{
		const Asura_Matrix_3x3* pxOrientation = GetOrientation();
		if ( pxOrientation )
		{
			xDeltaX = Asura_Vector_3( pxOrientation->e11, pxOrientation->e21, pxOrientation->e31 );
			xDeltaY = Asura_Vector_3( pxOrientation->e13, pxOrientation->e23, pxOrientation->e33 );
			bSendAlignVectors = true;
		}
	}
	else if ( pxTemplate->m_bAlignEffectZ )
	{
		const Asura_Matrix_3x3* pxOrientation = GetOrientation();
		if ( pxOrientation )
		{
			xDeltaX = Asura_Vector_3( pxOrientation->e11, pxOrientation->e21, pxOrientation->e31 );
			xDeltaY = Asura_Vector_3( pxOrientation->e12, pxOrientation->e22, pxOrientation->e32 );
			bSendAlignVectors = true;
		}
	}

	// Send the alignment vectors
	if ( bSendAlignVectors )
	{
		SendNewAlignmentVectors( xDeltaX, xDeltaY );
	}
	else if ( g_bSendCameraVectors )
	{
		SendNewCameraVectors();
	}

    if( g_bIsHeathazePass )
    {
        // first pass uses normal renderer
        g_iHeathazePassNumber = 0;

        // (MP - 14.07.2005) Don't think we need to support heathaze skybox effects...
        g_fEffectOffsetScale = 0.0f;

        // 2 passes - would be better done on VU1 to save uploading and transforming
        // all the particles twice!
        SetRenderstateForHeathazeFirstPass(pxTemplate);

        // init the effect render
        PFX_BeginEffectRender( pxTemplate, false );


        PFX_AddParticles_NoPrevPos( pxParticles, uNumParticles, this, pxTemplate, xLightColour );

		// (AW - 17.09.2004) - TODO, handle texture frame animation properly.
		const Asura_PFX_TextureFrame* pxTextureFrame = pxTemplate->GetTextureFrameAtTime(0.0f);
		EndPacket( pxTextureFrame->m_afTextureUVs[0],
                   pxTextureFrame->m_afTextureUVs[1],
                   pxTextureFrame->m_afTextureUVs[2],
                   pxTextureFrame->m_afTextureUVs[3] );

        g_iHeathazePassNumber = 1;

        if (!dbg_bTestHeathazeFirstPass)
        {
			// need to flush so we don't trash the first pass renderstates
			{
				Asura_PS2_DmaTag* pxTag = reinterpret_cast<Asura_PS2_DmaTag*>( Asura_PS2_DeferredRendering::GetVifBuffer(1) );
				pxTag->Set( 0,
							Asura_PS2_DmaTag::PCE_NOCHANGE,
							Asura_PS2_DmaTag::ID_CNT,
							false,
							0,
							false,
							SCE_VIF1_SET_NOP(false),
							SCE_VIF1_SET_FLUSH(false) );
			}

            SetRenderstateForHeathazeSecondPass(pxTemplate);

            // init the effect render
            PFX_BeginEffectRender( pxTemplate, false );

            PFX_AddParticles_NoPrevPos( pxParticles, uNumParticles, this, pxTemplate, xLightColour );

			// (AW - 17.09.2004) - TODO, handle texture frame animation properly.
			const Asura_PFX_TextureFrame* pxTextureFrame = pxTemplate->GetTextureFrameAtTime(0.0f);
			EndPacket( pxTextureFrame->m_afTextureUVs[0],
                       pxTextureFrame->m_afTextureUVs[1],
                       pxTextureFrame->m_afTextureUVs[2],
                       pxTextureFrame->m_afTextureUVs[3] );
        }
    }
    else
    {
        g_fEffectOffsetScale = ( (m_usPFXFlags & ASURA_PFX_EFFECT_SKYBOX) ? 1.0f : 0.0f );

		// init the effect render
		PFX_BeginEffectRender( pxTemplate, (m_usPFXFlags & ASURA_PFX_EFFECT_ALWAYS_ON_TOP) );

		if( pxTemplate->m_bUseMotionBlur )
		{
    		PFX_AddParticles_WithPrevPos( pxParticles, uNumParticles, this, pxTemplate, xLightColour );
		}
		else
		{
			PFX_AddParticles_NoPrevPos( pxParticles, uNumParticles, this, pxTemplate, xLightColour );
		}

		// (AW - 17.09.2004) - TODO, handle texture frame animation properly.
		const Asura_PFX_TextureFrame* pxTextureFrame = pxTemplate->GetTextureFrameAtTime(0.0f);
		EndPacket( pxTextureFrame->m_afTextureUVs[0],
                   pxTextureFrame->m_afTextureUVs[1],
                   pxTextureFrame->m_afTextureUVs[2],
                   pxTextureFrame->m_afTextureUVs[3] );
	}

#if ASURA_PS2_COLLECT_RENDER_STATS
    // update triangle count
    u_int uNumCameras = 0;
    for(int iCamID = 0; iCamID < Asura_Camera::GetNumberOfCameras(); iCamID++)
    {
		if (m_ucCameraFlags & (1 << iCamID))
		{
			uNumCameras++;
		}
	}
    Asura_PS2_Render::UpdateTriStats( (uNumParticles * uNumCameras) << 1, 1 );
#endif //ASURA_PS2_COLLECT_RENDER_STATS
}

void Asura_PFX_System::Platform_Initialise( void )
{
}

void Asura_PFX_System::Platform_Reset( void )
{
}

void Asura_PFX_System::Platform_Shutdown( void )
{
}


// F U N C T I O N   I M P L E M E N T A T I O N S ///////////////////////////

static void InitVU1( void )
{
    int num_active_cameras = 0;
    for(int iCamID = 0; iCamID < Asura_Camera::GetNumberOfCameras(); iCamID++)
    {
        Asura_Camera::SetCameraIndex(iCamID);

        if (Asura_Camera::IsCameraActive())
        {
            num_active_cameras++;
        }
    }
	g_uNumActiveCameras = num_active_cameras;

    // (MH - 27.06.2002) could save a few QWs by using normal mode transfer but I doubt it's worth it.
    int num_qwords_for_camera_data = num_active_cameras * ASURA_PS2_PARTICLE_NUM_QWS_PER_CAMERA;
    int num_qwords_for_giftags_and_screensize = 4;
    int num_qwords = num_qwords_for_camera_data + num_qwords_for_giftags_and_screensize + 3;  // extra 3 for VIFtag and commands

    Asura_PS2_QW* const pxBase = Asura_PS2_DeferredRendering::GetVifBuffer(num_qwords);
    Asura_PS2_QW* pPacket = pxBase;
    
    // upload camera data
    pPacket->u32s[0] = PY_DMA_CNT | num_qwords_for_camera_data;
    pPacket->u32s[1] = 0;
    pPacket->u32s[2] = SCE_VIF1_SET_STCYCL( 1, 1, false );
    pPacket->u32s[3] = SCE_VIF1_SET_UNPACK(0, num_qwords_for_camera_data, CMD_V4_32, 0); // address in VU mem
    pPacket++;

    // unpack camera setting data, matrices and deltas to start of VU mem
	int iAlignVectorIndex = 0;
    for(int iCamID = 0; iCamID < Asura_Camera::GetNumberOfCameras(); iCamID++)
    {
        Asura_Camera::SetCameraIndex(iCamID);

        if (Asura_Camera::IsCameraActive())
        {
            // add viewport commands
            pPacket = Asura_PS2_GS::GetViewportSetupGifCmds(iCamID, Asura_PS2_GS::CONTEXT_1, pPacket);
            
            // add world-to-clip matrix and clip to screen scale/transform
			const sceVu0FMATRIX& xW2GBC = Asura_PS2_VU1::GetWorldToClipMatGuardband( iCamID );
			Asura_PS2_VU1::CopyPS2Mat( pPacket, xW2GBC );
			pPacket += 4;
			(pPacket++)->u128 = reinterpret_cast<const u_long128&>( Asura_PS2_VU1::GetGBClipToScrScaleVec(iCamID) );
			(pPacket++)->u128 = reinterpret_cast<const u_long128&>( Asura_PS2_VU1::GetGBClipToScrTransVec(iCamID) );
			(pPacket++)->u128 = reinterpret_cast<const u_long128&>( Asura_PS2_VU1::GetGBClipToScrClipScaleVec(iCamID) );

            // add deltas
            const Asura_Matrix_3x3 * cameraOrientation = Asura_Camera::GetOrientation();

            pPacket->f32s[0] = cameraOrientation->e11;
            pPacket->f32s[1] = cameraOrientation->e21;
            pPacket->f32s[2] = cameraOrientation->e31;
            pPacket->u32s[3] = num_active_cameras;
            pPacket++;


            pPacket->f32s[0] = cameraOrientation->e12;
            pPacket->f32s[1] = cameraOrientation->e22;
            pPacket->f32s[2] = cameraOrientation->e32;
            pPacket->f32s[3] = 0.0f;
            pPacket++;

			// Cache the camera vectors for later resending
			g_xCameraVectors[iAlignVectorIndex].x = cameraOrientation->e11;
			g_xCameraVectors[iAlignVectorIndex].y = cameraOrientation->e21;
			g_xCameraVectors[iAlignVectorIndex].z = cameraOrientation->e31;
			iAlignVectorIndex++;

			g_xCameraVectors[iAlignVectorIndex].x = cameraOrientation->e12;
			g_xCameraVectors[iAlignVectorIndex].y = cameraOrientation->e22;
			g_xCameraVectors[iAlignVectorIndex].z = cameraOrientation->e32;
			iAlignVectorIndex++;

            // Add particle offset for this camera. This is to support skybox
            // effects and therefore it's just the camera position.
            const Asura_Vector_3* pxCamPos = Asura_Camera::GetPosition();
            pPacket->f32s[0] = pxCamPos->x;
            pPacket->f32s[1] = pxCamPos->y;
            pPacket->f32s[2] = pxCamPos->z;
            ++pPacket;
        }
    }

    // upload GIF tags for camera settings and quads to start of input buffer
    pPacket->u32s[0] = PY_DMA_CNT | num_qwords_for_giftags_and_screensize;
    pPacket->u32s[1] = 0;
    pPacket->u32s[2] = SCE_VIF1_SET_BASE(ASURA_PS2_PARTICLE_INPUT_BUFFERS_START, 0);
    pPacket->u32s[3] = SCE_VIF1_SET_UNPACK(ASURA_PS2_PARTICLE_INPUT_BUFFERS_START, num_qwords_for_giftags_and_screensize, CMD_V4_32, 0); // address in VU mem
    pPacket++;
    
    (pPacket++)->u128 = g_xCameraSettingsGifTag.u128;
    (pPacket++)->u128 = g_xParticleGifTag.u128;
	(pPacket++)->u128 = g_xClippedGifTag.u128;
	// set up the screen size constants (used by heathaze)
	pPacket->f32s[0] = static_cast<float>(1.0f/Asura_PS2_GS::GetScreenWidth());
	pPacket->f32s[1] = static_cast<float>(1.0f/Asura_PS2_GS::GetScreenHeight());
	pPacket->f32s[2] = static_cast<float>(ASURA_PS2_SCREEN_CENTRE_X - (Asura_PS2_GS::GetScreenWidth() >> 1));
	pPacket->f32s[3] = static_cast<float>(ASURA_PS2_SCREEN_CENTRE_Y - (Asura_PS2_GS::GetScreenHeight() >> 1));
	pPacket++;
    
    // setup double buffer, 
    pPacket->u32s[0] = PY_DMA_CNT | 0;                  // CNT tag to send some commands
    pPacket->u32s[1] = 0;                               // null address, since not REF tag
    pPacket->u32s[2] = SCE_VIF1_SET_OFFSET(ASURA_PS2_PARTICLE_INPUT_BUFFER_SIZE, 0);     
    pPacket->u32s[3] = SCE_VIF1_SET_MSCALF(0, 0);   // equivalent of FLUSH then MSCAL
    pPacket++;

	g_bSendCameraVectors = false;
}

// Send new billboard camera vectors
static void SendNewCameraVectors( void )
{
	const u_int uNumCameras = g_uNumActiveCameras;
	const u_int uNumQWCamData = uNumCameras * ASURA_PS2_PARTICLE_NUM_QWS_PER_ALIGN_VECTORS;
	const u_int uNumQWs = uNumQWCamData + 1;
	const u_int uTotalQWs = 1 + uNumQWs;

	Asura_PS2_QW* const pxBase = Asura_PS2_DeferredRendering::GetVifBuffer( uTotalQWs );
	Asura_PS2_QW* pxOut = pxBase;

	// upload camera data
	(reinterpret_cast<Asura_PS2_DmaTag*>(pxOut++))->Set( uNumQWs,
		Asura_PS2_DmaTag::PCE_NOCHANGE,
		Asura_PS2_DmaTag::ID_CNT,
		false,
		0,
		false,
		SCE_VIF1_SET_STCYCL( 1, 1, false ),
		SCE_VIF1_SET_UNPACK( ASURA_PS2_PARTICLE_ALIGN_VECTORS_START, uNumQWCamData, Asura_PS2_VU1::VIFPK_V4_32, false ) );

	// unpack camera billboard vectors
	int iAlignVectorIndex = 0;
	for( u_int uI = 0; uI < uNumCameras; uI++ )
	{
		pxOut->f32s[0] = g_xCameraVectors[iAlignVectorIndex].x;
		pxOut->f32s[1] = g_xCameraVectors[iAlignVectorIndex].y;
		pxOut->f32s[2] = g_xCameraVectors[iAlignVectorIndex].z;
		pxOut->u32s[3] = uNumCameras;
		++pxOut;
		++iAlignVectorIndex;

		pxOut->f32s[0] = g_xCameraVectors[iAlignVectorIndex].x;
		pxOut->f32s[1] = g_xCameraVectors[iAlignVectorIndex].y;
		pxOut->f32s[2] = g_xCameraVectors[iAlignVectorIndex].z;
		pxOut->f32s[3] = 0.0f;
		++pxOut;
		++iAlignVectorIndex;
	}

	// Run program 
	pxOut->u32s[0] = ASURA_PS2_MAKE_MSCAL_FROM_VU_PTR( VU1_PARTICLERENDERER_NEW_BILLBOARD );
	pxOut->u32s[1] = SCE_VIF1_SET_NOP( false );
	pxOut->u32s[2] = SCE_VIF1_SET_NOP( false );
	pxOut->u32s[3] = SCE_VIF1_SET_NOP( false );
	++pxOut;

	Asura_Assert( static_cast<u_int>(pxOut - pxBase) == uTotalQWs, "" );
	g_bSendCameraVectors = false;
}

// Send new effect orientation alignment vectors
// This is the same orientation for each camera
static void SendNewAlignmentVectors( const Asura_Vector_3& xXalignment, const Asura_Vector_3& xYalignment )
{
	const u_int uNumCameras = g_uNumActiveCameras;
	const u_int uNumQWCamData = uNumCameras * ASURA_PS2_PARTICLE_NUM_QWS_PER_ALIGN_VECTORS;
	const u_int uNumQWs = uNumQWCamData + 1; 
	const u_int uTotalQWs = 1 + uNumQWs;

	Asura_PS2_QW* const pxBase = Asura_PS2_DeferredRendering::GetVifBuffer( uTotalQWs );
	Asura_PS2_QW* pxOut = pxBase;

	// upload camera data
	(reinterpret_cast<Asura_PS2_DmaTag*>(pxOut++))->Set( uNumQWs,
		Asura_PS2_DmaTag::PCE_NOCHANGE,
		Asura_PS2_DmaTag::ID_CNT,
		false,
		0,
		false,
		SCE_VIF1_SET_STCYCL( 1, 1, false ),
		SCE_VIF1_SET_UNPACK( ASURA_PS2_PARTICLE_ALIGN_VECTORS_START, uNumQWCamData, Asura_PS2_VU1::VIFPK_V4_32, false ) );

	// unpack camera billboard vectors
	int iAlignVectorIndex = 0;
	for( u_int uI = 0; uI < uNumCameras; uI++ )
	{
		pxOut->f32s[0] = xXalignment.x;
		pxOut->f32s[1] = xXalignment.y;
		pxOut->f32s[2] = xXalignment.z;
		pxOut->u32s[3] = uNumCameras;
		pxOut++;
		iAlignVectorIndex++;

		pxOut->f32s[0] = xYalignment.x;
		pxOut->f32s[1] = xYalignment.y;
		pxOut->f32s[2] = xYalignment.z;
		pxOut->f32s[3] = 0.0f;
		pxOut++;
		iAlignVectorIndex++;
	}

	// Run program 
	pxOut->u32s[0] = ASURA_PS2_MAKE_MSCAL_FROM_VU_PTR( VU1_PARTICLERENDERER_NEW_BILLBOARD );
	pxOut->u32s[1] = SCE_VIF1_SET_NOP( false );
	pxOut->u32s[2] = SCE_VIF1_SET_NOP( false );
	pxOut->u32s[3] = SCE_VIF1_SET_NOP( false );
	++pxOut;

	Asura_Assert( static_cast<u_int>(pxOut - pxBase) == uTotalQWs, "" );
	g_bSendCameraVectors = true;
}

static void StartNewPacket( void )
{
    g_uNumParticlesInThisPacket = 0;
    
    // Setup output ptrs. Allow for maximum number of QWs. We'll fix it up as
    // necessary in EndPacket().
    g_pxPacketBase = Asura_PS2_DeferredRendering::GetVifBuffer( MAX_PACKET_TOTAL_QWS );
    g_pxPacket = g_pxPacketBase;

    // Leave gap for DMA tag and packet setup.
    g_pxPacket += 1 + NUM_QWS_FOR_PACKET_SETUP;
}

static void EndPacket( float fTopLeftU, float fTopLeftV, float fBottomRightU, float fBottomRightV )
{
    // If we didn't actually end up rendering any particles for some reason,
    // cancel the whole thing.
    if( !g_uNumParticlesInThisPacket )
    {
        Asura_PS2_DeferredRendering::ReturnVifBuffer( g_pxPacketBase, MAX_PACKET_TOTAL_QWS );
        return;
    }

    // Otherwise, calculate actual num QWs used.
    const u_int uActualQWsForParticleData = g_uNumParticlesInThisPacket * ASURA_PS2_PARTICLE_INPUT_QWS_PER_PARTICLE;
    const u_int uActualQWC = NUM_QWS_FOR_PACKET_SETUP + uActualQWsForParticleData + 1;
    const u_int uActualTotalQWs = 1 + uActualQWC;

    // Go back and fill in DMA tag and packet setup.
    {
        Asura_PS2_QW* pxOut = g_pxPacketBase;

        (reinterpret_cast<Asura_PS2_DmaTag*>(pxOut))->Set( uActualQWC,
                                                           Asura_PS2_DmaTag::PCE_NOCHANGE,
                                                           Asura_PS2_DmaTag::ID_CNT,
                                                           false,
                                                           0,
                                                           false );

        pxOut->u32s[ 2] = SCE_VIF1_SET_UNPACKR( 0, ASURA_PS2_PARTICLE_HEADER_QWS, Asura_PS2_VU1::VIFPK_V3_32, false );
        pxOut->f32s[ 3] = fTopLeftU;
        pxOut->f32s[ 4] = fTopLeftV;
        pxOut->u32s[ 5] = g_uNumParticlesInThisPacket;
        pxOut->f32s[ 6] = fBottomRightU;
        pxOut->f32s[ 7] = fTopLeftV;
        pxOut->u32s[ 8] = g_uFlagsForCurTemplate;
        pxOut->f32s[ 9] = fTopLeftU;
        pxOut->f32s[10] = fBottomRightV;
        pxOut->f32s[11] = g_fEffectOffsetScale;
        pxOut->f32s[12] = fBottomRightU;
        pxOut->f32s[13] = fBottomRightV;
        pxOut->u32s[14] = g_uCurCamFlags;

        pxOut->u32s[15] = SCE_VIF1_SET_UNPACKR( ASURA_PS2_PARTICLE_HEADER_QWS, uActualQWsForParticleData, Asura_PS2_VU1::VIFPK_V4_32, false );
    }

    // Add an MSCNT on the end.
    g_pxPacket->u128 = ASURA_PS2_INIT_ULONG128( 0uLL ); // NOPs.
    (g_pxPacket++)->u32s[0] = SCE_VIF1_SET_MSCNT( false );

    // Return the difference.
    if( uActualTotalQWs < MAX_PACKET_TOTAL_QWS )
    {
        Asura_PS2_DeferredRendering::ReturnVifBuffer( g_pxPacket, MAX_PACKET_TOTAL_QWS - uActualTotalQWs );
    }
}


static void PFX_BeginEffectRender( const Asura_PFX_Particle_Template* pxTemplate, bool bAlwaysOnTop )
{
	if (!g_bIsHeathazePass)
	{
		const ASURA_TRANSLUCENCY_MODE eTranslucencyMode = pxTemplate->m_eTranslucency;
		if( eTranslucencyMode != g_eLastTranslucencyMode )
		{
            const bool bIsSolid = ( eTranslucencyMode == ASURA_TRANSLUCENCY_OFF );

			// (MH - 27.10.2004) need to flush here, to ensure the new render states don't affect old particles
			{
				Asura_PS2_DmaTag* pxTag = reinterpret_cast<Asura_PS2_DmaTag*>( Asura_PS2_DeferredRendering::GetVifBuffer(1) );
				pxTag->Set( 0,
							Asura_PS2_DmaTag::PCE_NOCHANGE,
							Asura_PS2_DmaTag::ID_CNT,
							false,
							0,
							false,
							SCE_VIF1_SET_NOP(false),
							SCE_VIF1_SET_FLUSH(false) );
			}

			Asura_PlatformRenderStates::CreateDefaultRenderStatePacket( Asura_PlatformRenderStates::CONTEXT_1 );

            // Only solid particles want z buffer writes.
			Asura_PlatformRenderStates::SetZBufferWriteEnabled( bIsSolid );

            bool bATE;
			if( ASURA_TRANSLUCENCY_NORMAL == eTranslucencyMode )
            {
                bATE = true;
            }
            else
			{
                // Leave alpha test on for solid particles but turn it off
                // for all other non-translucent effects.
                bATE = bIsSolid;

				Asura_PlatformRenderStates::SetTranslucencyMode( eTranslucencyMode );
			}

            if( !bATE || bAlwaysOnTop )
            {
                Asura_PlatformRenderStates::SetPixelTest( bATE, !bAlwaysOnTop );
            }

			Asura_PlatformRenderStates::ApplyRenderStates();

			g_eLastTranslucencyMode = eTranslucencyMode;
		}
	}

    // Calculate flags for this template. 
    {
        u_int uFlags = 0;

        if( g_bIsHeathazePass )
        {
            // first pass uses normal renderer
            if( g_iHeathazePassNumber == 1 ) { uFlags |= ASURA_PS2_PARTICLE_FLAG_HEATHAZE; }
        }
        else if( pxTemplate->m_bUseMotionBlur )
        {
            uFlags |= ASURA_PS2_PARTICLE_FLAG_USEMOTIONBLUR;
        }
        else
		{
			// PFX always uses rotation, if not using motion blur
			uFlags |= ASURA_PS2_PARTICLE_FLAG_USEROTATION;
		}

		if( pxTemplate->m_bCanUseExpensiveClipping)
		{
			Asura_Assert(uFlags == ASURA_PS2_PARTICLE_FLAG_USEROTATION, "Motion blurred or heat haze particles cannot be clipped on PS2.");
			// TODO - only set this flag if the effect BB crosses the guardband?
			// (fairly moot given that the BB is usually absurdly big - see Mantis 19534)
			uFlags |= ASURA_PS2_PARTICLE_FLAG_CLIPPED;
		}

        g_uFlagsForCurTemplate = uFlags;
    }

    StartNewPacket();
}

DEBUGVAR float dbg_fFadeoutHeathazeDist = 50.0f;
DEBUGVAR bool  dbg_bFadeoutHeathazeWithDist = true;

static void PFX_AddParticles_NoPrevPos( const Asura_PFX_Particle* pxParticles,
                                        u_int uNumParticles,
                                        const Asura_PFX_Effect* pxEffect,
                                        const Asura_PFX_Particle_Template* pxTemplate,
                                        const Asura_RGBA& xLighting )
{
	Asura_Camera::SetCameraIndex(0);
	const Asura_Vector_3& xCamera0Pos = *Asura_Camera::GetPosition();
    for( u_int uParticleIdx = 0; uParticleIdx < uNumParticles; ++uParticleIdx )
    {
        const Asura_PFX_Particle* pxParticle = &pxParticles[uParticleIdx];

        const float fAge  = pxParticle->m_fAge;
        const float fSeed = pxParticle->m_fSeed;

        Asura_RGBA xColour = pxTemplate->m_xColour.GetValueAtTime_Discrete( fAge );

        const Asura_Vector_3& xPos = pxParticle->m_xPosition;

        // alpha out the heathaze effect with distance
        // Note that in pass 2 of the heathaze
        // alpha has no effect but we still do the calculation so tiny alphas are rejected.
        if (dbg_bFadeoutHeathazeWithDist && g_bIsHeathazePass)
        {
            float fMinSqDist = Asura_Maths::LargeFloat;
            for (int iCamera = 0; iCamera < Asura_Camera::GetNumberOfCameras(); ++iCamera)
            {
                Asura_Camera::SetCameraIndex(iCamera);
                if (Asura_Camera::IsCameraActive())
                {
                    float fDistSquared = (xPos - *Asura_Camera::GetPosition()).MagnitudeSquared();
                    if (fDistSquared < fMinSqDist)
                    {
                        fMinSqDist = fDistSquared;
                    }
                }
            }

            float fDiff = (dbg_fFadeoutHeathazeDist * dbg_fFadeoutHeathazeDist) - fMinSqDist;
            float fScale = Asura_Maths::Max( 0.0f, fDiff / (dbg_fFadeoutHeathazeDist * dbg_fFadeoutHeathazeDist) );

            xColour.m_fA *= fScale;
        }
    
        if( pxTemplate->m_bNearCameraFade || pxTemplate->m_bFarCameraFade)
        {
            // (MP - 07.01.2005) TODO: This assumes camera 0 is the only relevant
            // one here. This is a temporary implementation until we hopefully
            // get this all implemented on VU1.
            Asura_Camera::SetCameraIndex( 0 );
            float fDist = (*Asura_Camera::GetPosition() - xPos).Magnitude();
	        if( pxTemplate->m_bNearCameraFade)
			{
				float fNearFadeDist = pxTemplate->m_fNearFadeDistance;
            	if( fDist < fNearFadeDist )
            	{
            	    float fNearZeroDist = pxTemplate->m_fNearZeroDistance;
            	    xColour.m_fA *= (fDist - fNearZeroDist) / (fNearFadeDist - fNearZeroDist);
			        if( xColour.m_fA <= 0.01f ) { continue; }
            	}
			}
	        if( pxTemplate->m_bFarCameraFade)
			{
				float fFarFadeDist = pxTemplate->m_fFarFadeDistance;
            	if( fDist > fFarFadeDist )
            	{
            	    float fFarZeroDist = pxTemplate->m_fFarZeroDistance;
            	    xColour.m_fA *= (fDist - fFarZeroDist) / (fFarFadeDist - fFarZeroDist);
			        if( xColour.m_fA <= 0.01f ) { continue; }

            	}
			}
        }
    
        if( xColour.m_fA <= 0.01f ) { continue; }

        if( pxTemplate->m_bUseLighting )
        {
            xColour.m_fR *= 2.0f * xLighting.m_fR;
            xColour.m_fG *= 2.0f * xLighting.m_fG;
            xColour.m_fB *= 2.0f * xLighting.m_fB;
        }
    
		const Asura_PFX_TextureFrame* pxBaseTextureFrame = pxTemplate->GetTextureFrameAtTime(0.0f);
        if( g_uNumParticlesInThisPacket == ASURA_PS2_PARTICLE_MAX_PARTICLES )
        {
            EndPacket( pxBaseTextureFrame->m_afTextureUVs[0], pxBaseTextureFrame->m_afTextureUVs[1], pxBaseTextureFrame->m_afTextureUVs[2], pxBaseTextureFrame->m_afTextureUVs[3] );
            StartNewPacket();
        }

        float fSize = pxTemplate->m_xSize.GetValueAtTime_Discrete( fAge );
        if( pxTemplate->m_bUseSizeScale )
        {
            fSize *= pxEffect->GetSizeScale();
        }
        fSize *= 1.0f - fSeed * pxTemplate->m_xSizeVariation.GetValueAtTime_Discrete( fAge );
    
        g_pxPacket->f32s[0] = Asura_Maths::Min( 128.0f * xColour.m_fR, 255.0f );
        g_pxPacket->f32s[1] = Asura_Maths::Min( 128.0f * xColour.m_fG, 255.0f );
        g_pxPacket->f32s[2] = Asura_Maths::Min( 128.0f * xColour.m_fB, 255.0f );
        g_pxPacket->f32s[3] = Asura_Maths::Min( 128.0f * xColour.m_fA, 128.0f );
        ++g_pxPacket;

        g_pxPacket->f32s[0] = xPos.x;
        g_pxPacket->f32s[1] = xPos.y;
        g_pxPacket->f32s[2] = xPos.z;
        g_pxPacket->f32s[3] = fSize;
        ++g_pxPacket;

        float fRotation;
        if( pxTemplate->m_bFixedRotation )
        {
            // Fixed rotation is inexplicably stored in degrees...
            fRotation = Asura_Maths::DegToRad( pxTemplate->m_fFixedRotation );

            int iNumFixedRotations = pxTemplate->m_uNumFixedRotations;
            if( iNumFixedRotations > 1 )
            {
                float fNumFixedRotations = iNumFixedRotations;
                int iStep = Asura_Maths::FloatToInt_Chop( fSeed * fNumFixedRotations );
                fRotation += (iStep / fNumFixedRotations) * 2.0f * Asura_Maths::Pi;
            }
        }
        else
        {
            if( pxTemplate->m_bUseRandomRotation )
            {
                fRotation = ( Asura_Timers::GetGameTimeScale() ? Asura_Random::GetFloat() * 2.0f * Asura_Maths::Pi : 0.0f );
            }
            else
            {
                fRotation = (fSeed + pxTemplate->m_xRotation.GetValueAtTime_Discrete(fAge)) * 2.0f * Asura_Maths::Pi;
            }

            if( pxTemplate->m_bMirrorRotation && (reinterpret_cast<const u_int&>(fSeed) & 1) )
            {
                fRotation = -fRotation;
            }
        }
        g_pxPacket->f32s[3] = fRotation;

		float fVOffset = 0.f;
		if(pxTemplate->m_bSpriteRotation)
		{
			Asura_Assert(Asura_Camera::GetNumberOfCameras() == 1, "Sprite rotation not handled for multiple cameras");
			Asura_Vector_3 xParticleDirection = pxParticle->m_xVelocity;
			Asura_Vector_3 xCameraDirection = pxParticle->m_xPosition - xCamera0Pos;
			// Flatten it out to the X-Z plane.
			xParticleDirection.y = 0.f;
			xCameraDirection.y = 0.f;
			if(xParticleDirection.Normalise() > 0.f && xCameraDirection.Normalise() > 0.f)
			{
				const float fParticleYAngle = Asura_Maths::ArcTangent(xParticleDirection.x, xParticleDirection.z);
				const float fCameraYAngle = Asura_Maths::ArcTangent(xCameraDirection.x, xCameraDirection.z);
				const float fAngleDiff = Asura_Maths::RadiansToZeroMinimumRange(fParticleYAngle + Asura_Maths::DegToRad(22.5f) - fCameraYAngle);
				const u_int uFrameOffset = Asura_Maths::FloatToInt_Near(fAngleDiff / Asura_Maths::DegToRad(45.f));
				fVOffset = uFrameOffset * pxBaseTextureFrame->m_afTextureUVs[3] - pxBaseTextureFrame->m_afTextureUVs[1];
			}
		}

		// KOR - 07/02/2008 - texture animation
		const Asura_PFX_TextureFrame* pxCurrTextureFrame = pxTemplate->GetTextureFrameAtTime(fAge);
		if(pxCurrTextureFrame != pxBaseTextureFrame || fVOffset != 0.f)
		{
#ifdef ASURA_ASSERT
			const float fBaseUSize = pxBaseTextureFrame->m_afTextureUVs[2] - pxBaseTextureFrame->m_afTextureUVs[0];
			const float fBaseVSize = pxBaseTextureFrame->m_afTextureUVs[3] - pxBaseTextureFrame->m_afTextureUVs[1];
			const float fCurrUSize = pxCurrTextureFrame->m_afTextureUVs[2] - pxCurrTextureFrame->m_afTextureUVs[0];
			const float fCurrVSize = pxCurrTextureFrame->m_afTextureUVs[3] - pxCurrTextureFrame->m_afTextureUVs[1];
			Asura_Assert(fBaseUSize == fCurrUSize && fBaseVSize == fCurrVSize, "PS2 cannot handle PFX texture frames with different sizes");
#endif
			// U offset
			g_pxPacket->f32s[0] = pxCurrTextureFrame->m_afTextureUVs[0] - pxBaseTextureFrame->m_afTextureUVs[0];
			// V offset
			g_pxPacket->f32s[1] = (pxCurrTextureFrame->m_afTextureUVs[1] - pxBaseTextureFrame->m_afTextureUVs[1]) + fVOffset;
		}
		else
		{
			g_pxPacket->f32s[0] = 0.f;
			// V offset
			g_pxPacket->f32s[1] = 0.f;
		}

		
		++g_pxPacket;

        ++g_uNumParticlesInThisPacket;
    }
}

static void PFX_AddParticles_WithPrevPos( const Asura_PFX_Particle* pxParticles,
                                          u_int uNumParticles,
                                          const Asura_PFX_Effect* pxEffect,
                                          const Asura_PFX_Particle_Template* pxTemplate,
                                          const Asura_RGBA& xLighting )
{
    for( u_int uParticleIdx = 0; uParticleIdx < uNumParticles; ++uParticleIdx )
    {
        const Asura_PFX_Particle* pxParticle = &pxParticles[uParticleIdx];

        const float fAge  = pxParticle->m_fAge;
        const float fSeed = pxParticle->m_fSeed;

        Asura_RGBA xColour = pxTemplate->m_xColour.GetValueAtTime_Discrete( fAge );

        const Asura_Vector_3& xPos = pxParticle->m_xPosition;

        if( pxTemplate->m_bNearCameraFade || pxTemplate->m_bFarCameraFade)
        {
            // (MP - 07.01.2005) TODO: This assumes camera 0 is the only relevant
            // one here. This is a temporary implementation until we hopefully
            // get this all implemented on VU1.
            Asura_Camera::SetCameraIndex( 0 );
            float fDist = (*Asura_Camera::GetPosition() - xPos).Magnitude();
	        if( pxTemplate->m_bNearCameraFade)
			{
            	float fNearFadeDist = pxTemplate->m_fNearFadeDistance;
            	if( fDist < fNearFadeDist )
            	{
            	    float fNearZeroDist = pxTemplate->m_fNearZeroDistance;
            	    xColour.m_fA *= (fDist - fNearZeroDist) / (fNearFadeDist - fNearZeroDist);
			        if( xColour.m_fA <= 0.01f ) { continue; }
            	}
			}
	        if( pxTemplate->m_bFarCameraFade)
			{
            	float fFarFadeDist = pxTemplate->m_fFarFadeDistance;
            	if( fDist > fFarFadeDist )
            	{
            	    float fFarZeroDist = pxTemplate->m_fFarZeroDistance;
            	    xColour.m_fA *= (fDist - fFarZeroDist) / (fFarFadeDist - fFarZeroDist);
			        if( xColour.m_fA <= 0.01f ) { continue; }
            	}
			}
        }

        if( xColour.m_fA <= 0.01f ) { continue; }

        if( pxTemplate->m_bUseLighting )
        {
            xColour.m_fR *= 2.0f * xLighting.m_fR;
            xColour.m_fG *= 2.0f * xLighting.m_fG;
            xColour.m_fB *= 2.0f * xLighting.m_fB;
        }

        if( g_uNumParticlesInThisPacket == ASURA_PS2_PARTICLE_MAX_PARTICLES )
        {
            // (AW - 17.09.2004) - TODO, handle texture frame animation properly.
            const Asura_PFX_TextureFrame* pxTextureFrame = pxTemplate->GetTextureFrameAtTime(0.0f);
            EndPacket( pxTextureFrame->m_afTextureUVs[0], pxTextureFrame->m_afTextureUVs[1], pxTextureFrame->m_afTextureUVs[2], pxTextureFrame->m_afTextureUVs[3] );
            StartNewPacket();
        }

        float fSize = pxTemplate->m_xSize.GetValueAtTime_Discrete( fAge );
        if( pxTemplate->m_bUseSizeScale )
        {
            fSize *= pxEffect->GetSizeScale();
        }
        fSize *= 1.0f - fSeed * pxTemplate->m_xSizeVariation.GetValueAtTime_Discrete( fAge );
    
        g_pxPacket->f32s[0] = Asura_Maths::Min( 128.0f * xColour.m_fR, 255.0f );
        g_pxPacket->f32s[1] = Asura_Maths::Min( 128.0f * xColour.m_fG, 255.0f );
        g_pxPacket->f32s[2] = Asura_Maths::Min( 128.0f * xColour.m_fB, 255.0f );
        g_pxPacket->f32s[3] = Asura_Maths::Min( 128.0f * xColour.m_fA, 128.0f );
        ++g_pxPacket;

        g_pxPacket->f32s[0] = xPos.x;
        g_pxPacket->f32s[1] = xPos.y;
        g_pxPacket->f32s[2] = xPos.z;
        g_pxPacket->f32s[3] = fSize;
        ++g_pxPacket;

        const float fBlurLength = Asura_Maths::Min( pxTemplate->m_fBlurLength, fAge * pxTemplate->GetLifetimeFromSeed(fSeed) );
        const Asura_Vector_3& xVel = pxParticle->m_xVelocity;

        g_pxPacket->f32s[0] = xPos.x - xVel.x * fBlurLength;
        g_pxPacket->f32s[1] = xPos.y - xVel.y * fBlurLength;
        g_pxPacket->f32s[2] = xPos.z - xVel.z * fBlurLength;
        ++g_pxPacket;

        ++g_uNumParticlesInThisPacket;
    }
}


static void CreateHeatHazeTexture( void )
{
	Asura_PS2_GS_Util::ShrinkAndDeformFB( Asura_PS2_ParticleHeatHazeGetU, Asura_PS2_ParticleHeatHazeGetV );
}

static void SetRenderstateForHeathazeFirstPass( const Asura_PFX_Particle_Template* pxTemplate )
{
#ifdef ASURA_PS2_PFX_SUPPORT_MULTIPLE_TEXTURES
	const int iTexture = pxTemplate->m_iTextureHandle;
#else
	const int iTexture = g_iParticleTexture;
#endif

    const u_int uNLOOP = 3;

    // Create a dma tag for all this
    Asura_PS2_QW *pxVifBuffer = Asura_PS2_DeferredRendering::GetVifBuffer(uNLOOP+2);
    pxVifBuffer->u32s[0] = PY_DMA_CNT | (uNLOOP+1);
    pxVifBuffer->u32s[1] = 0;
    pxVifBuffer->u32s[2] = SCE_VIF1_SET_NOP( false );
    pxVifBuffer->u32s[3] = SCE_VIF1_SET_DIRECT( uNLOOP+1, false );
    pxVifBuffer++;

    // Create a gif tag for it
    pxVifBuffer->u64s[0] = SCE_GIF_SET_TAG( uNLOOP,         // NLOOP
                                            1,                  // EOP
                                            NULL,               // Prim Enable
                                            NULL,               // Prim
                                            SCE_GIF_PACKED, // PACKET type
                                            1 );                // Register quantity
    pxVifBuffer->u64s[1] = SCE_GIF_PACKED_AD;
    pxVifBuffer++;

    // Mip-map setup.
    extern unsigned short g_uMipMapK;
    const u_int num_mipmap_levels = Asura_PS2_TextureManagement::GetNumMipmapLevels( iTexture );
    pxVifBuffer->u64s[0] = SCE_GS_SET_TEX1( 0,                      // lcm - LOD calculation method: 0: ((log2(1/|q|)) << L) + K, 1: fixed value K
                                            num_mipmap_levels-1,    // mxl - maximum mipmap level
                                            PARTICLES_MMAG,     // mmag - filter when texture is expanded (LOD < 0)
                                            PARTICLES_MMIN,     // mmin - filter when texture is reduced (LOD >= 0)
                                            1,                      // mtba - automatic calculation of mip map address 1-3
                                            0,                      // L
                                            g_uMipMapK );           // K (see sce.dev.prog.gs "MipMap L & K") this is based on texture size?
    pxVifBuffer->u64s[1] = SCE_GS_TEX1_1;
    pxVifBuffer++;

    // Texture address and format - Texture position in frame buffer.
    Asura_PS2_TextureManagement::GetTEX0Setting( &pxVifBuffer->u64s[0], iTexture, 0 );
    pxVifBuffer->u64s[1] = SCE_GS_TEX0_1;
    pxVifBuffer++;

    // Set the alpha blend mode - write alpha source only
    if (dbg_bTestHeathazeFirstPass2)
    {
        pxVifBuffer->u64s[0] = SCE_GS_SET_ALPHA(SCE_GS_ALPHA_CS, SCE_GS_ALPHA_CD, SCE_GS_ALPHA_AS, SCE_GS_ALPHA_CD, 0); 
    }
    else
    {
        pxVifBuffer->u64s[0] = SCE_GS_SET_ALPHA(SCE_GS_ALPHA_ZERO, SCE_GS_ALPHA_ZERO, SCE_GS_ALPHA_FIX, SCE_GS_ALPHA_CD, 0); //(0-0)*0 + FB; note Alpha source is still written.
    }
    pxVifBuffer->u64s[1] = SCE_GS_ALPHA_1;
    pxVifBuffer++;
}

static void SetRenderstateForHeathazeSecondPass( const Asura_PFX_Particle_Template* pxTemplate  )
{
#ifdef ASURA_PS2_PFX_SUPPORT_MULTIPLE_TEXTURES
	const int iTexture = pxTemplate->m_iTextureHandle;
#else
	const int iTexture = g_iParticleTexture;
#endif

    // Set Tex register to point to where we just rendered (texture address, size, format) Decal Mode
    const u_int uBaseTexAddress = Asura_PS2_TextureManagement::GetBfrTBP( 1 );
    const u_int uPixelFormat    = Asura_PS2_GS::GetDrawBfrPSM();

    const u_int uNLOOP = 3;

    // Create a dma tag for all this
    Asura_PS2_QW *pxVifBuffer = Asura_PS2_DeferredRendering::GetVifBuffer(uNLOOP+2);
    pxVifBuffer->u32s[0] = PY_DMA_CNT | (uNLOOP+1);
    pxVifBuffer->u32s[1] = 0;
    pxVifBuffer->u32s[2] = SCE_VIF1_SET_NOP( false );
    pxVifBuffer->u32s[3] = SCE_VIF1_SET_DIRECT( uNLOOP+1, false );
    pxVifBuffer++;

    // Create a gif tag for it
    pxVifBuffer->u64s[0] = SCE_GIF_SET_TAG( uNLOOP,         // NLOOP
                                            1,                  // EOP
                                            NULL,               // Prim Enable
                                            NULL,               // Prim
                                            SCE_GIF_PACKED, // PACKET type
                                            1 );                // Register quantity
    pxVifBuffer->u64s[1] = SCE_GIF_PACKED_AD;
    pxVifBuffer++;

	if (dbg_bTestHeathazeSecondPass)
	{
		// Mip-map setup.
		extern unsigned short g_uMipMapK;
		const u_int num_mipmap_levels = Asura_PS2_TextureManagement::GetNumMipmapLevels( iTexture );
		pxVifBuffer->u64s[0] = SCE_GS_SET_TEX1( 0,                      // lcm - LOD calculation method: 0: ((log2(1/|q|)) << L) + K, 1: fixed value K
												num_mipmap_levels-1,    // mxl - maximum mipmap level
												PARTICLES_MMAG,     // mmag - filter when texture is expanded (LOD < 0)
												PARTICLES_MMIN,     // mmin - filter when texture is reduced (LOD >= 0)
												1,                      // mtba - automatic calculation of mip map address 1-3
												0,                      // L
												g_uMipMapK );           // K (see sce.dev.prog.gs "MipMap L & K") this is based on texture size?
		pxVifBuffer->u64s[1] = SCE_GS_TEX1_1;
		pxVifBuffer++;

		// Texture address and format - Texture position in frame buffer.
		Asura_PS2_TextureManagement::GetTEX0Setting( &pxVifBuffer->u64s[0], iTexture, 0 );
		pxVifBuffer->u64s[1] = SCE_GS_TEX0_1;
		pxVifBuffer++;
	}
	else
	{
		pxVifBuffer->u64s[0] = SCE_GS_SET_TEX1_1( 1, 0, // LOD=K, Max MIP level=0
												  SCE_GS_LINEAR,        // expand filter
												  SCE_GS_LINEAR,        // reduce filter 
												  1, 
												  0, 1);    // L, K
		pxVifBuffer->u64s[1] = SCE_GS_TEX1_1;
		pxVifBuffer++;

		pxVifBuffer->u64s[0] = SCE_GS_SET_TEX0_1( uBaseTexAddress,
												  ((1<<uASURA_PS2_DEFORMMESH_SIZE_LOG2)>>6),
												  uPixelFormat, 
												  uASURA_PS2_DEFORMMESH_SIZE_LOG2, uASURA_PS2_DEFORMMESH_SIZE_LOG2,// width, height (256x256)
												  1,                                                // 0=RGB(no texture alpha), 1=RGBA(Alpha texA for <32bit), 
												  1,                                                // 0=Modulate, 1=Decal
												  0, 0, 0, 0, 0 );                              // CLUT
		pxVifBuffer->u64s[1] = SCE_GS_TEX0_1;
		pxVifBuffer++;
	}

    // Set the alpha blend mode - wrote dest alpha in first pass
    pxVifBuffer->u64s[0] = SCE_GS_SET_ALPHA(SCE_GS_ALPHA_CS, SCE_GS_ALPHA_CD, SCE_GS_ALPHA_AD, SCE_GS_ALPHA_CD, 0); //(CS-FB)*AD + FB
    pxVifBuffer->u64s[1] = SCE_GS_ALPHA_1;
    pxVifBuffer++;
}
