/**
*
* \file Asura_Platform_WeatherFX.cpp
*
* Created by Kevin Lea - 07/12/2002 17:16:04
*
* Copyright (C) 2002 Rebellion, All Rights Reserved.
*
* \brief
*
**/

#include "Core/Asura.h"

#include "SpecialFX/Asura_Weather_System.h"

#include "Camera/Asura_Camera.h"
#include "Core/Asura_PS2_DMA.h"
#include "Core/Asura_Timers.h"
#include "Rendering/Asura_Render.h"
#include "Rendering/Asura_PlatformRenderStates.h"
#include "Rendering/Asura_PlatformTextureManagement.h"
#include "Rendering/Asura_PS2_GS.h"
#include "Rendering/Asura_PS2_VU1.h"

#ifdef ASURA_PS2_DEFERRED_RENDERING
#include "Rendering/Asura_PS2_DeferredRendering.h"
#endif

//// L O C A L   D E F I N E S //////////////////////////////
#define ASURA_PS2_RAINQUAD_CAMERADATA_SIZE		(10)
#define ASURA_PS2_RAINQUAD_SETUPDATA_SIZE		((4*ASURA_PS2_RAINQUAD_CAMERADATA_SIZE)+2)
#define ASURA_PS2_RAINQUAD_QUADDATA_SIZE		(5)
#define ASURA_PS2_RAINQUAD_INPUTBUFFER_START	(0x2c)
#define ASURA_PS2_RAINQUAD_INPUTBUFFER_SIZE		(5)

//// C L A S S   F U N C T I O N S //////////////////////////////
void Asura_Weather_RainQuad::Platform_BeginRender( int iTextureIndex )
{
    // Load the Rain Quad Renderer
	Asura_PS2_VU1::LoadRenderer( Asura_PS2_VU1::RENDERER_RAINQUAD );

    // Load the rain texture
	Asura_PS2_TextureManagement::SetBufferLayout( ASURA_PS2_TEXBFRLAYOUT_2_LARGE );
	bool bTextureUploaded = Asura_PS2_TextureManagement::UploadTexture( iTextureIndex );

    Asura_PS2_TextureManagement::SetupTextureEnvironment( bTextureUploaded,
                                                          ASURA_PS2_TEXREGFLAG_ALL,
                                                          iTextureIndex,
                                                          Asura_PS2_TextureManagement::BFR_AUTO,
                                                          SCE_GS_LINEAR,
                                                          SCE_GS_LINEAR_MIPMAP_NEAREST );

	// Setup the render state
    Asura_PlatformRenderStates::CreateDefaultRenderStatePacket( Asura_PlatformRenderStates::CONTEXT_1 );
	Asura_PlatformRenderStates::SetPixelTest( true, true );
	Asura_PlatformRenderStates::SetTranslucencyMode( ASURA_TRANSLUCENCY_GLOWING );
    Asura_PlatformRenderStates::ApplyRenderStates();

	// Start up the renderer
	const u_int uSetupQWords = ASURA_PS2_RAINQUAD_SETUPDATA_SIZE;
	const u_int uTotalQWords = uSetupQWords	+ 1;		// Camera data + kick

	// Create the upload data
#ifdef ASURA_PS2_DEFERRED_RENDERING
	// Create the upload data
	Asura_PS2_QW *pxOut = Asura_PS2_DeferredRendering::GetVifBuffer(uTotalQWords+1);
	const u_int uTagType = PY_DMA_CNT;
#else
	// setup the texture for this mesh
	Asura_PS2_QW* const pxBase = Asura_PS2_SPR_Alloc( 0, uTotalQWords+1 );
	Asura_PS2_QW* pxOut = pxBase;
	const u_int uTagType = PY_DMA_END;
#endif

	pxOut->u32s[0] = uTagType | uTotalQWords;
	pxOut->u32s[1] = 0;
	pxOut->u32s[2] = SCE_VIF1_SET_STCYCL( 1, 1, 0 ),
	pxOut->u32s[3] = SCE_VIF1_SET_UNPACK(0, uSetupQWords, CMD_V4_32, 0);
	pxOut++;

	// Add the drawing GIF tag
	const u_int uPrimType = (Asura_Render::GetWireframe()) ? SCE_GS_PRIM_LINESTRIP : SCE_GS_PRIM_TRISTRIP;
	pxOut->u64s[0] = SCE_GIF_SET_TAG( 0,					// NLOOP - Filled in by microcode
									  0,					// EOP - Filled in by microcode
									  1,					// PRE (PRIM field Enable)
									  SCE_GS_SET_PRIM( uPrimType,	// PRIM (Type of drawing primitive)
													   0,			// IIP (Flat Shading/Gouraud Shading)
													   1,			// TME (Texture Mapping Enabled)
													   0,			// FGE (Fogging Enabled)
													   1,			// ABE (Alpha Blending Enabled)
													   0,			// AA1 (1 Pass Anti-Aliasing Enabled)
													   0,			// FST (Method of specifying Texture Coords)
													   0,			// CTXT (Context)
													   0 ),			// FIX (Fragment Value Control)
										   SCE_GIF_PACKED,	// FLG (Data format, 0 = PACKED, 1 = REGLIST, 2 = IMAGE)
										   3 );				// NREG (Number of registers)
	pxOut->u64s[1] = (SCE_GS_XYZ2<<8) | (SCE_GS_RGBAQ<<4) | (SCE_GS_ST<<0);
	pxOut++;

	// Add the camera data
	const u_int uCameraQty = Asura_Camera::GetNumberOfCameras();
	for(u_int u=0; u<uCameraQty; u++)
	{
		Asura_Camera::SetCameraIndex(u);
	
		// Add the camera world->screen matrix
		sceVu0FMATRIX &aafWorld2Screen = Asura_PS2_VU1::GetWorldToScreenMat();
		for (u_int v=0; v<4; v++)
		{
			for (u_int w=0; w<4; w++)
			{
				pxOut->f32s[w] = aafWorld2Screen[v][w];
			}
			pxOut++;
		}

		// Add the camera world->clip matrix
		sceVu0FMATRIX &aafWorld2Clip = Asura_PS2_VU1::GetWorldToClipMatGuardband();
		for (u_int v=0; v<4; v++)
		{
			for (u_int w=0; w<4; w++)
			{
				pxOut->f32s[w] = aafWorld2Clip[v][w];
			}
			pxOut++;
		}

		// Add the SCISSOR and OFFSET settings
		pxOut = Asura_PS2_GS::GetViewportSetupGifCmds( u, Asura_PS2_GS::CONTEXT_1, pxOut );
	}

	// If we added less than 4 camertas, pad out the upload data
	pxOut += (4 - uCameraQty) * ASURA_PS2_RAINQUAD_CAMERADATA_SIZE;

	// Add the camera setup gif tag
	pxOut->u64s[0] = SCE_GIF_SET_TAG( 2, 1, 0, 0, SCE_GIF_PACKED, 1 );
	pxOut->u64s[1] = SCE_GIF_PACKED_AD;
	pxOut++;

	// Kick of the setup code and set the double buffer layout
	pxOut->u32s[0] = SCE_VIF1_SET_ITOP(uCameraQty, 0);
	pxOut->u32s[1] = SCE_VIF1_SET_MSCAL(0,0);
	pxOut->u32s[2] = SCE_VIF1_SET_BASE( ASURA_PS2_RAINQUAD_INPUTBUFFER_START, 0 );
	pxOut->u32s[3] = SCE_VIF1_SET_OFFSET( ASURA_PS2_RAINQUAD_INPUTBUFFER_SIZE, 0 );
	pxOut++;

#ifndef ASURA_PS2_DEFERRED_RENDERING
	// Send it off
	DPUT_D1_QWC(0);

	// source address
	DPUT_D1_TADR((u_int)pxBase | ASURA_PS2_DMA_SPR_FLAG);   // when transferring from the scratchpad, set the top bit to 1.

	// for polling of completion
    DPUT_D_PCR(D_PCR_CPC1_M);
    DPUT_D_STAT(D_STAT_CIS1_M);

    asm __volatile__("sync.l");

    // start actual kick
    DPUT_D1_CHCR(1  // from memory
        | (1 << 2)  // chain mode
        | (1 << 6)  // transfer tag on
        | (1 << 8));    // dma start

    asm __volatile__("sync.l");
	asm __volatile__("sync.p");
    asm __volatile__("DMAGLASSRENDER:");
    asm __volatile__("bc0f DMAGLASSRENDER");
    asm __volatile__("nop");
#endif
}


void Asura_Weather_RainQuad::Platform_Render(void)
{
	if (m_fUScale > 14.0f)
		m_fUScale = 14.0f;

	if (m_fVScale > 14.0f)
		m_fVScale = 14.0f;

	float fUOffset = (Asura_Timers::GetGameTimeScale() == 0.0f) ? 0.0f : Asura_Random::GetFloat();
	fUOffset -= (m_fUScale*0.5f);
	float fVOffset = (Asura_Timers::GetGameTimeScale() == 0.0f) ? 0.0f : Asura_Random::GetFloat();
	fVOffset -= (m_fVScale*0.5f);

	// Calculate the total amount of data we are going to DMA, then we can put it all in 1 DMA Block
	const u_int uTotalQWords = ASURA_PS2_RAINQUAD_QUADDATA_SIZE;					// Quad data
							   
	// Create the upload data
#ifdef ASURA_PS2_DEFERRED_RENDERING
	// Create the upload data
	Asura_PS2_QW *pxOut = Asura_PS2_DeferredRendering::GetVifBuffer(uTotalQWords+1);
	const u_int uTagType = PY_DMA_CNT;
#else
	// setup the texture for this mesh
	Asura_PS2_QW* const pxBase = Asura_PS2_SPR_Alloc( 0, uTotalQWords+1 );
	Asura_PS2_QW* pxOut = pxBase;
	const u_int uTagType = PY_DMA_END;
#endif
	// Unpack the quad data
	pxOut->u32s[0] = uTagType | uTotalQWords;
	pxOut->u32s[1] = 0;
    pxOut->u32s[2] = SCE_VIF1_SET_UNPACKR( 0, ASURA_PS2_RAINQUAD_QUADDATA_SIZE, Asura_PS2_VU1::VIFPK_V4_32, false );
	
	//	RGBA
static int iRainAlpha = 32;
	pxOut->u32s[3] = 128;		// Red
	pxOut++;
	pxOut->u32s[0] = 128;		// Green
	pxOut->u32s[1] = 128;		// Blue
	pxOut->u32s[2] = iRainAlpha;		// Alpha

	// Start
	pxOut->f32s[3] = m_xVertices[0].x;
	pxOut++;
	pxOut->f32s[0] = m_xVertices[0].y;
	pxOut->f32s[1] = m_xVertices[0].z;
	pxOut->u32s[2] = m_uCamVisFlags;
	
	// End
	pxOut->f32s[3] = m_xVertices[2].x;
	pxOut++;
	pxOut->f32s[0] = m_xVertices[2].y;
	pxOut->f32s[1] = m_xVertices[2].z;
	pxOut->f32s[2] = 1.0f;

	// STQ Start 
	pxOut->f32s[3] = fUOffset;
	pxOut++;
	pxOut->f32s[0] = fVOffset;
	pxOut->f32s[1] = 1.0f;
	pxOut->f32s[2] = 0.0f;

	// STQ End
	pxOut->f32s[3] = fUOffset + m_fUScale;
	pxOut++;
	pxOut->f32s[0] = fVOffset + m_fVScale;
	pxOut->f32s[1] = 1.0f;
	pxOut->f32s[2] = 0.0f;

	// Add the kick to get it rendering - and a flush so we can wait for it to finish
	pxOut->u32s[3] = SCE_VIF1_SET_MSCNT(0);
	pxOut++;

#ifndef ASURA_PS2_DEFERRED_RENDERING
	// Send it off
	DPUT_D1_QWC(0);

	// source address
	DPUT_D1_TADR((u_int)pxBase | ASURA_PS2_DMA_SPR_FLAG);   // when transferring from the scratchpad, set the top bit to 1.

	// for polling of completion
    DPUT_D_PCR(D_PCR_CPC1_M);
    DPUT_D_STAT(D_STAT_CIS1_M);

    asm __volatile__("sync.l");

    // start actual kick
    DPUT_D1_CHCR(1  // from memory
        | (1 << 2)  // chain mode
        | (1 << 6)  // transfer tag on
        | (1 << 8));    // dma start

    asm __volatile__("sync.l");
	asm __volatile__("sync.p");
    asm __volatile__("DMARAINQUADRENDER:");
    asm __volatile__("bc0f DMARAINQUADRENDER");
    asm __volatile__("nop");
#endif
}


void Asura_Weather_RainQuad::Platform_EndRender(void)
{
#ifdef ASURA_PS2_DEFERRED_RENDERING
	// Add a flush to wait for rendering to finish
	Asura_PS2_QW *pxVifBuffer = Asura_PS2_DeferredRendering::GetVifBuffer(1);
	pxVifBuffer->u32s[0] = PY_DMA_CNT | 0;
	pxVifBuffer->u32s[1] = 0;
	pxVifBuffer->u32s[2] = SCE_VIF1_SET_NOP(0);
	pxVifBuffer->u32s[3] = SCE_VIF1_SET_FLUSH(0);
#endif
}
