//////////////////////////////////////////////////////////////////////////////
//
// Asura_PS3_BotanicalsLODBilboard.cpp
//
// Copyright (c) 2006 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

/******************************************************************************/
/* Included Files                                                             */
/******************************************************************************/

#include "Core/Asura.h"

#include "Botanicals/Asura_PS3_Botanicals.h"

// Asura Platform-independent
#include "Camera/Asura_Camera.h"
#include "Core/Memory/Asura_Mem.h"
#include "Core/Asura_MaterialFlags.h"
#include "Core/Asura_Material_System.h"
#include "Rendering/Asura_Module.h"
#include "Rendering/Asura_Render.h"
#include "Core/Asura_MaterialResponse_System.h"

// Asura PS3
#include "Rendering/Asura_PS3_Graphics.h"
#include "Rendering/Asura_PS3_TextureManagement.h"
#include "Rendering/Shaders/Asura_PS3_GCM_Shader.h"
#include "Rendering/Asura_PS3_FrameBuffer.h"
#include "Rendering/Asura_PS3_RenderingOptions.h"
#include "Rendering/Shaders/Asura_PS3_GCM_Shader.h"

#include "Rendering/ShadersDefinition/VD/BotanicalsInstace_VD.h"
#include "Rendering/ShadersDefinition/VD/BotanicalsBillboards_VD.h"

#include "Rendering/ShadersDefinition/FP/Botanicals_FP.h"

#include "Rendering/ShadersDefinition/VP/BotanicalsBillboards_VP.h"

static Asura_PS3_VertexBufferDeclaration g_axVertexDeclarationBotanicalBillboardInstances [] =
{
	//	m_szName, m_uOffset, m_uType, m_uNumberOfCoords, m_ucAttributeIndex
	{ "xIn.m_xPositionOffsetUV", 0,	CELL_GCM_VERTEX_SF,		4, 2 }, //ATTR2

	{ "xIn.m_xPosition",		 8,	CELL_GCM_VERTEX_SF,		4, 3 }, //ATTR3

//	{ "xIn.m_xNormal",			16,	CELL_GCM_VERTEX_CMP,	1, 4 }, //ATTR4
	{ "xIn.m_xTangent",			20,	CELL_GCM_VERTEX_CMP,	1, 5 }, //ATTR5
	{ "xIn.m_xBinormal",		24,	CELL_GCM_VERTEX_CMP,	1, 6 }, //ATTR6

	{ "xIn.m_fBoneWeight",		28,	CELL_GCM_VERTEX_F,		1, 7 },	//ATTR7
};

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_PS3_Botanicals_LOD_Billboards::Asura_PS3_Botanicals_LOD_Billboards
// Author       : Catalin Arsenescu
// Date         : 02-August-2007
////////////////////////////////////////////////////////////////////////////////
Asura_PS3_Botanicals_LOD_Billboards::Asura_PS3_Botanicals_LOD_Billboards()
: m_uMaterialResponseHashID( ASURA_HASH_ID_UNSET )
, m_iNumBillboards( 0 )
{
}
/// End of function 'Asura_PS3_Botanicals_LOD_Billboards::Asura_PS3_Botanicals_LOD_Billboards'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_PS3_Botanicals_LOD_Billboards::~Asura_PS3_Botanicals_LOD_Billboards
// Author       : Catalin Arsenescu
// Date         : 02-August-2007
////////////////////////////////////////////////////////////////////////////////
Asura_PS3_Botanicals_LOD_Billboards::~Asura_PS3_Botanicals_LOD_Billboards()
{
	m_xBillboardVertexBuffer.Shutdown();
}
/// End of function 'Asura_PS3_Botanicals_LOD_Billboards::~Asura_PS3_Botanicals_LOD_Billboards'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_PS3_Botanicals_LOD_Billboards::Render
// Author       : Catalin Arsenescu
// Date         : 06-September-2007
////////////////////////////////////////////////////////////////////////////////
void Asura_PS3_Botanicals_LOD_Billboards::Render( u_int uRenderStartInstance, u_int uNumVisibleInstances )
{
	Asura_Vector_3 xCameraUp, xCameraRight, xCameraForwards;
	Asura_Camera::GetUpVector( xCameraUp);
	Asura_Camera::GetRightVector( xCameraRight);
	Asura_Camera::GetForwardVector( xCameraForwards);

	xCameraUp.Normalise();
	xCameraRight.Normalise();
	xCameraForwards.Normalise();

	Asura_PS3_GCM_VertexShader* g_pxVertexShader = Asura_PS3_Botanicals_System::GetBotanicalsBillboardVP();
	Asura_PS3_FragmentShaderCopy* g_pxFragmentShader = Asura_PS3_GCM_ShadersLibrary::GetResettableFP( static_cast<Asura_Hash_ID> ( ASURA_SHADER_HASH_BOTANICALS ), m_uFPHashID );
	const Asura_Vector_3* const pxCamPos = Asura_Camera::GetPosition();
	float afCamPos[ 4 ]	= { pxCamPos->x, pxCamPos->y, pxCamPos->z, 1.0f };
	if( !g_pxFragmentShader ) return;

	Asura_PS3_DeferredRendering::Bind();
	g_pxVertexShader->Bind();

	const Asura_MaterialResponse* const pxMaterial1 = Asura_MaterialResponse_System::GetMaterialByHashID( m_uMaterialResponseHashID );
	Asura_TextureAnimationOutput xTexAnim1;
	Asura_TextureAnimation_System::GetAnimationState( pxMaterial1->GetTextureAnim(), NULL, &xTexAnim1 );
	const float afTexCoordOffset[4] = { xTexAnim1.m_fUOffset, xTexAnim1.m_fVOffset, 0.0f, 0.0f };

	const Asura_AMatrix4x4& xProjectionMatrix = Asura_PS3_Graphics::GetProjectionMatrix();
	const Asura_AMatrix4x4& xViewMatrix = Asura_PS3_Graphics::GetViewMatrix();
	g_pxVertexShader->SetUniformParameter( VSB_ProjectionMatrix, xProjectionMatrix.GetElements() );
	g_pxVertexShader->SetUniformParameter( VSB_ViewMatrix, xViewMatrix.GetElements() );
	g_pxVertexShader->SetUniformParameter( VSB_CameraPosition, afCamPos );

	g_pxVertexShader->SetUniformParameter( VSB_CameraRightVector, &xCameraRight.x );
	g_pxVertexShader->SetUniformParameter( VSB_CameraUpVector, &xCameraUp.x );
	g_pxVertexShader->SetUniformParameter( VSB_CameraForwardsVector, &xCameraForwards.x );
	g_pxVertexShader->SetUniformParameter( VSB_UV_Offsets, &afTexCoordOffset[ 0 ] );

	Asura_PS3_Botanicals_System::SetMaterial( m_uMaterialResponseHashID, g_pxFragmentShader );
	Asura_PS3_RenderingStates::RequestZBufferCompareMode( ASURA_ZBUFFERCMP_LESSEQUAL );
	Asura_PS3_RenderingStates::RequestZBufferEnabled( true );
	Asura_PS3_RenderingStates::RequestZBufferWriteEnabled( true );
	Asura_PS3_RenderingStates::RequestTranslucencyMode( ASURA_TRANSLUCENCY_OFF );
	Asura_PS3_RenderingStates::SetBlendEnableMrt( false, false, false );
	Asura_PS3_RenderingStates::SetAlphaBlending( false );

	u_short uiStride = m_iNumBillboards * 4;
	Asura_PS3_Botanicals_System::BindInstanceData( uRenderStartInstance, uiStride );
	bool abKeep[16] = {false, false, false, false, false, false, false, false, true, true, true, true, true, true, true, true};
	m_xBillboardVertexBuffer.Bind( g_axVertexDeclarationBotanicalBillboardInstances, VDBILLI_ELEMENTS, uiStride, true, abKeep );
	g_pxFragmentShader->Bind();

	u_int uiNumVertices = uNumVisibleInstances * uiStride;
	cellGcmSetFrequencyDividerOperationUnsafeInline( Asura_PS3_Graphics::GetGcmCurrentContext(),
		CELL_GCM_FREQUENCY_MODULO << 2 |
		CELL_GCM_FREQUENCY_MODULO << 3 |
		CELL_GCM_FREQUENCY_MODULO << 4 |
		CELL_GCM_FREQUENCY_MODULO << 5 |
		CELL_GCM_FREQUENCY_MODULO << 6 |
		CELL_GCM_FREQUENCY_MODULO << 7 );

	m_xBillboardVertexBuffer.RenderQuads( uiNumVertices );
	Asura_PS3_Graphics::Flush();
	cellGcmSetFrequencyDividerOperationUnsafeInline( Asura_PS3_Graphics::GetGcmCurrentContext(), 0 );
}
/// End of function 'Asura_PS3_Botanicals_LOD_Billboards::Render'. ///
