//
// Asura_Chunk_ResourceFile_PS3.cpp
//
// Created by KJFL - 27 June 2005
//
// Copyright (C) 2005 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

// This file's header.
#include "ChunkSystem/Asura_Chunk_ResourceFile_PS3.h"

#include "Debugging/Asura_Assert.h"
#include "Debugging/Asura_Logfile.h"
#include "ChunkSystem/Asura_Chunk_ResourceFile.h"
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "Core/Asura_Handle.h"
#include "FileAccess/Asura_FileAccess.h"

#include "ChunkSystem/Asura_Chunk_ResourceFile_PS3_Environment.h"
#include "ChunkSystem/Asura_Chunk_ResourceFile_PS3_Character.h"
#include "ChunkSystem/Asura_Chunk_ResourceFile_PS3_ObjectHierarchy.h"
#include "ChunkSystem/Asura_Chunk_ResourceFile_PS3_MaterialResponse.h"
#include "ChunkSystem/Asura_Chunk_ResourceFile_PS3_StreamedSounds.h"
#if defined ASURA_USE_BOTANICALS
#include "ChunkSystem/Asura_Chunk_ResourceFile_PS3_Botanicals.h"
#endif
#if defined ASURA_USE_CARPET
#include "ChunkSystem/Asura_Chunk_ResourceFile_PS3_Carpet.h"
#endif
#if defined ASURA_USE_STATIC_DECAL_SYSTEM
	#include "ChunkSystem/Asura_Chunk_ResourceFile_PS3_StaticDecals.h"
#endif
#include "ChunkSystem/Asura_Chunk_ResourceFile_PS3_Shaders.h"

// C L A S S   I M P L E M E N T A T I O N S /////////////////////////////////



bool Asura_Chunk_ResourceFile::Platform_Process( Asura_Handle __hFile, const char* szOriginalFileName )
{
	Asura_Chunk_File_Stream	xStream (__hFile);

	// (MP - 16.09.2002) Note that I've omitted the semi-colon at the end of
	// this macro; this stops Lint complaining.
	#define PROCESS( type, class_for_type ) \
		case type: \
		reinterpret_cast<class_for_type*>(this)->Process( szOriginalFileName, xStream ); \
		break

	switch( this->SubType )
	{
		// Process known subtypes.
		PROCESS( ASURA_RESOURCEFILE_TYPE_PS3_ENVIRONMENT, Asura_Chunk_ResourceFile_PS3_Environment);
		PROCESS( ASURA_RESOURCEFILE_TYPE_PS3_CHARACTER, Asura_Chunk_ResourceFile_PS3_Character);
		PROCESS( ASURA_RESOURCEFILE_TYPE_PS3_OBJECTHIERARCHY, Asura_Chunk_ResourceFile_PS3_ObjectHierarchy);
		PROCESS( ASURA_RESOURCEFILE_TYPE_PS3_MATERIALRESPONSE, Asura_Chunk_ResourceFile_PS3_MaterialResponse );
#if defined ASURA_USE_BOTANICALS
		PROCESS( ASURA_RESOURCEFILE_TYPE_PS3_BOTANICALS, Asura_Chunk_ResourceFile_PS3_Botanicals );
#endif

#if defined ASURA_USE_CARPET
		PROCESS( ASURA_RESOURCEFILE_TYPE_PS3_CARPET, Asura_Chunk_ResourceFile_PS3_Carpet );
#endif

#if defined ASURA_USE_STATIC_DECAL_SYSTEM
		PROCESS( ASURA_RESOURCEFILE_TYPE_PS3_STATICDECALS, Asura_Chunk_ResourceFile_PS3_StaticDecals );
#endif
		PROCESS( ASURA_RESOURCEFILE_TYPE_PS3_SHADERS, Asura_Chunk_ResourceFile_PS3_Shaders );

#ifndef ASURA_USE_AUDIO_SYSTEM
	case ASURA_RESOURCEFILE_TYPE_PS3_STREAMINGSOUNDS:
			reinterpret_cast<Asura_Chunk_ResourceFile_PS3_StreamedSounds*>(this)->Process( szOriginalFileName, xStream );
			return false;
		break;
#endif

		default:
		{
            Asura_Assert( false, "Unrecognised resource file subtype." );
			
			// skip the data to keep things happy
			Asura_FileAccess::SkipData( __hFile, this->FileSize );
			return false;
		}
    }

	return true;
}


