///////////////////////////////////////////////////////////////////////////////
//
// File      : Asura_Chunk_ResourceFile_PS3_StreamedSounds.cpp
// Author    : Philip Hall
// Created   : 14/11/2007
//
// Copyright (C) 2007 Rebellion. All Rights Reserved
//
///////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

#ifndef ASURA_USE_AUDIO_SYSTEM

// This file's header
#include "ChunkSystem/Asura_Chunk_ResourceFile_PS3_StreamedSounds.h"

// Asura headers
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "FileAccess/Asura_FileAccess.h"

// Asura PS3 headers
#include "ChunkSystem/Asura_Chunk_ResourceFile_PS3.h"
#include "FileAccess/Asura_PS3_FileSystem.h"
#include "Sound/Asura_PS3_Sound_System.h"



////PGH////////////////////////////////////////////////////////////////////////
// Function    : Asura_Chunk_ResourceFile_PS3_StreamedSounds::Process
// Description : Process the streamed sounds chunks
//
// Parameters  : 
//   const char* szSection  - Section name 
//   Asura_Chunk_File_Stream& xStream - Source stream
//
// Returns     : nothing
///////////////////////////////////////////////////////////////////////////////
void Asura_Chunk_ResourceFile_PS3_StreamedSounds::Process( const char* szSection, Asura_Chunk_File_Stream& xStream)
{
	int iVersion;
	xStream >> iVersion;
	
    // Check the chunk version isn't too recent
	if( iVersion > ASURA_PS3_STREAMEDSOUNDS_CHUNK_VERSION )
    {
        Asura_Assertf( false, ("[Asura_Chunk_ResourceFile_PS3_StreamedSounds::Process] - Chunk version %d is too recent, expected %d\n", iVersion, ASURA_PS3_STREAMEDSOUNDS_CHUNK_VERSION) );
    }

	int iSoundCount;
	xStream >> iSoundCount;

	if ( iSoundCount > 0 )
	{
		const char *szFilename = Asura_FileAccess::GetFilename( xStream.GetFileHandle() );

		// Store a record of which file we're streaming the sounds from
		int iCurrentFile = Asura_PS3_Sound_System::GetAPI().AddStreamingSoundSourceFile(szFilename);
		if( iCurrentFile < 0 )
		{
			Asura_Assertf( false, ("[Asura_Chunk_ResourceFile_PS3_StreamedSounds::Process] - No space for more streaming source files\n"));
		}

		bool bSoundIDsAreDynamic = (this->Flags & uASURA_PS3_STREAMEDSOUNDS_FLAG_SOUNDIDS_ARE_DYNAMIC) != 0;

		// Attempt to load each sound (and its information) in turn
		while (iSoundCount > 0)
		{
			// Read sound header from file
			HdrEntry xHdrEntry;
			xStream >> xHdrEntry.iSoundID;
			xStream >> xHdrEntry.uOffset;
			xStream >> xHdrEntry.uSize;

			// Add streaming sound data
			Asura_PS3_Sound_System::GetAPI().AddStreamingSoundData(iCurrentFile, 
			                                                       bSoundIDsAreDynamic ? ASURA_SID_DYNAMIC : xHdrEntry.iSoundID, 
			                                                       xHdrEntry.uOffset, 
			                                                       xHdrEntry.uSize,
			                                                       bSoundIDsAreDynamic ? xHdrEntry.iSoundID : ASURA_HASH_ID_UNSET);

			iSoundCount--;
		}
	}
}

#endif // #ifndef ASURA_USE_AUDIO_SYSTEM

//eof
