///////////////////////////////////////////////////////////////////////////////
//
// File      : Asura_PS3_Semaphore.h
// Author    : Philip Hall
// Created   : 19/10/2007
//
// Copyright (C) 2007 Rebellion. All Rights Reserved
//
///////////////////////////////////////////////////////////////////////////////

#if !defined __ASURA_PS3_SEMAPHORE_H__
#define __ASURA_PS3_SEMAPHORE_H__

// Asura headers
#include "Debugging/Asura_Assert.h"
#include "Debugging/Asura_LogFile.h"

#include <sys/synchronization.h>


////PGH/////////////////////////////////////////
// Semaphore wrapper
////////////////////////////////////////////////									  
class Asura_PS3_Semaphore
{
private:
	Asura_PS3_Semaphore(const Asura_PS3_Semaphore& );

public:
	Asura_PS3_Semaphore()
	{
		sys_semaphore_attribute_t xAttr;
		sys_semaphore_attribute_initialize(xAttr);

		int iResult = sys_semaphore_create(&m_xSemaphore, &xAttr, 0, 256);	  // Arbitrary max limit
		if (iResult < CELL_OK)
		{
			Asura_Assert(false, "[Asura_PS3_Semaphore] - Unable to initialise semaphore\n");
		}
	}

	~Asura_PS3_Semaphore()
	{
		Destroy();
	}

	void Destroy()
	{
		if ( m_xSemaphore != 0 )
		{
			sys_semaphore_destroy(m_xSemaphore);
			m_xSemaphore = 0;
		}
	}

	// Signal (increment) this semaphore
	bool Signal()
	{
		int iResult = sys_semaphore_post(m_xSemaphore, 1);
		return (iResult == CELL_OK);
	}

	// Quick check for semaphore non-zero
	bool TryWait()
	{
		int iResult = sys_semaphore_trywait(m_xSemaphore);
		return (iResult == CELL_OK);
	}

	// Wait (blocking) for semaphore non-zero
	bool Wait(usecond_t uTimeoutUS)
	{
		int iResult = sys_semaphore_wait(m_xSemaphore, uTimeoutUS);
		return (iResult == CELL_OK);
	}

	// Get the current semaphore counter
	int GetValue()
	{
		sys_semaphore_value_t iValue;
		sys_semaphore_get_value(m_xSemaphore, &iValue);
		return iValue;
	}

protected:
	sys_semaphore_t m_xSemaphore;
};



#endif






