///////////////////////////////////////////////////////////////////////////////
//
// File      : Asura_PS3_FMV_Control.cpp
// Author    : Philip Hall
// Created   : 19/10/2007
//
// Copyright (C) 2007 Rebellion. All Rights Reserved
//
///////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header
#include "FMV/Asura_PS3_FMV_dmux.h"

// Asura headers
#include "Debugging/Asura_Assert.h"
#include "Debugging/Asura_LogFile.h"




Asura_PS3_FMV_Control::Asura_PS3_FMV_Control()
{
	m_bComplete       = false;
	m_bEndOfAudioData = false;
	m_bEndOfVideoData = false;
	m_bEndOfStream    = false;
	m_bEndOfDMux      = false;
	m_iErrorCode      = 0;
}


Asura_PS3_FMV_Control::~Asura_PS3_FMV_Control()
{
	m_xControlLock.Destroy();
}


void Asura_PS3_FMV_Control::Begin()
{
	m_xATime.upper = m_xATime.lower = 0;
	m_bAborted        = false;
	m_bComplete       = false;
	m_bEndOfAudioData = false;
	m_bEndOfVideoData = false;
	m_bEndOfStream    = false;
	m_bEndOfDMux      = false;
}




void Asura_PS3_FMV_Control::Abort(int iError)
{
	if ( m_xControlLock.Lock(0) )
	{
		if ( !m_bComplete )
		{
		    m_bAborted        = true;
			m_bComplete       = true;
			m_bEndOfDMux      = true;
			m_bEndOfStream    = false;
			m_bEndOfAudioData = false;
			m_bEndOfVideoData = false;
			m_iErrorCode = iError;
			Asura_OutputToDebugger(("[Asura_PS3_FMV_Control::Abort] - Aborted code = 0x%X\n", iError));
		}
		m_xControlLock.Unlock();
	}
}


void Asura_PS3_FMV_Control::Complete()
{
	if ( m_xControlLock.Lock(0) )
	{
		if ( !m_bComplete )
		{
			m_bComplete = true;
			Asura_OutputToDebugger(("[Asura_PS3_FMV_Control::End] - Ended\n"));
		}
		m_xControlLock.Unlock();
	}
}


void Asura_PS3_FMV_Control::EndStream()
{
	if ( m_xControlLock.Lock(0) )
	{
		m_bEndOfStream = true;
		Asura_OutputToDebugger(("[Asura_PS3_FMV_Control::EndStream] - Streamer Finished\n"));
		m_xControlLock.Unlock();
	}
}

void Asura_PS3_FMV_Control::EndVideoData()
{
	if ( m_xControlLock.Lock(0) )
	{
		m_bEndOfVideoData = true;
		Asura_OutputToDebugger(("[Asura_PS3_FMV_Control::EndVideoData] - Data Finished\n"));
		m_xControlLock.Unlock();
	}
}

void Asura_PS3_FMV_Control::EndAudioData()
{
	if ( m_xControlLock.Lock(0) )
	{
		m_bEndOfAudioData = true;
		Asura_OutputToDebugger(("[Asura_PS3_FMV_Control::EndAudioData] - Data Finished\n"));
		m_xControlLock.Unlock();
	}
}

void Asura_PS3_FMV_Control::EndDMux()
{
	if ( m_xControlLock.Lock(0) )
	{
		m_bEndOfDMux = true;
		Asura_OutputToDebugger(("[Asura_PS3_FMV_Control::EndDMux] - DMux Finished\n"));
		m_xControlLock.Unlock();
	}
}
