///////////////////////////////////////////////////////////////////////////////
//
// File      : Asura_PS3_FMV_adec.h
// Author    : Philip Hall
// Created   : 18/10/2007
//
// Copyright (C) 2007 Rebellion. All Rights Reserved
//
///////////////////////////////////////////////////////////////////////////////

#if !defined __ASURA_PS3_FMV_ADEC_H__
#define __ASURA_PS3_FMV_ADEC_H__


// Asura PS3 headers
#include "Core/Asura_PS3_Mutex.h"
#include "FMV/Asura_PS3_FMV_dmux.h"

#include <cell/codec.h>
#include <cell/codec/adec.h>


////PGH/////////////////////////////////////////
// Structures from PAMF sample           
////////////////////////////////////////////////
union UAdecParam
{
	CellAdecParamAc3	ac3;
	CellAdecParamLpcm	lpcm;
	CellAdecParamAtracX	at3plus;
};

#define ASURA_PS3_AUDIO_DATA_SIZE (53284)


////PGH/////////////////////////////////////////
// Audio storage block
////////////////////////////////////////////////
struct Asura_PS3_AudioBlock
{
	u_char acData[ASURA_PS3_AUDIO_DATA_SIZE]; // Buffer at top to ensure initial alignment
	u_int  uSize;
	CellAdecTimeStamp xTimestamp;
};

// Aligned audio buffer ring
typedef Asura_RingBuffer<Asura_PS3_AudioBlock, ASURA_SIZE_TO_128_BYTE(sizeof(Asura_PS3_AudioBlock))>  Asura_PS3_AudioBuffer;


////PGH/////////////////////////////////////////
// libadec wrapper
////////////////////////////////////////////////
class Asura_PS3_FMV_adec
{
public:
	Asura_PS3_FMV_adec();
	~Asura_PS3_FMV_adec();

	bool Initialise(Asura_PS3_FMV_Control* pControl);
	bool Open(Asura_PS3_FMV_ESInfo* pEsInfo, u_int uChNum);
	void Close();

	Asura_PS3_FMV_DecodeResult StartProcessing();
	bool StartAudio();
	void StopAudio();
	bool IsRunning() { return m_iMSStreamHandle != ASURA_INVALID_HANDLE; }
											
	Asura_PS3_AudioBuffer& Buffer() { return m_xAudioBuffer; }


protected:
	static void    AudioStreamCallback(int iStreamHandle, void* pUserData, int iCallbackType, void* pWriteBuffer, int nBufferSize);
	static int32_t CallbackProxy(CellAdecHandle handle, CellAdecMsgType msgType, int32_t msgData, void *cbArg) 
		{ return reinterpret_cast<Asura_PS3_FMV_adec*>(cbArg)->Callback(handle, msgType, msgData);}

	int  MakeSeqParam();
	int  GetNextPCMBlock();
	int32_t Callback(CellAdecHandle handle, CellAdecMsgType msgType, int32_t msgData);
	
private:
	Asura_PS3_FMV_Control*  m_pControl;
	Asura_PS3_FMV_ESInfo*   m_pESInfo;
	const CellDmuxAuInfo*   m_pAUInfo;
	Asura_PS3_AudioBlock    m_xHeldBlock;
	bool                    m_bHoldingBlock;
    bool                    m_bFlushEs;
    uint16_t                m_uPRXModule;
    CellAdecType            m_decType;
    CellAdecHandle          m_decHandle;
    UAdecParam              m_decParam;
    uint32_t                m_chNum;
    uint32_t                m_uSampleBits;
    size_t                  m_uMemSize;
    void*                   m_pWorkMemory;
	int                     m_iRunningFrameCount;

	// Audio buffering/rendering
	int                    m_iMSStreamHandle;
	Asura_PS3_Mutex        m_xBufferLock;
	Asura_PS3_AudioBuffer  m_xAudioBuffer;
};


#endif
