
///////////////////////////////////////////////////////////////////////////////
//
// File   : Asura_PS3_FileSystem.h
// Author : Andy Tate
// Date   : 23-Sep-2005
//
// Copyright (C) 2005 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __ASURA_PS3_FILESYSTEM_H__
#define __ASURA_PS3_FILESYSTEM_H__

#include "Core/Asura_List.h"
#include "Core/Asura_String.h"

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////


#if !defined ASURA_PS3_MAX_PATH
	#define ASURA_PS3_MAX_PATH 256	// Seems to be missing on PS3
#endif

#if !defined ASURA_PS3_CACHE_ID
	#define ASURA_PS3_CACHE_ID	"DEFAULT_GAME"	// Maximum 32 character or NULL;
#endif

////////////////////////////////////////////////////////////////////////////////
// Forward class declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Class declaration...
////////////////////////////////////////////////////////////////////////////////

enum PS3_TYPE_OF_FILE
{
	PS3_TOF_Directory,
	PS3_TOF_File,
	PS3_TOF_SymbolicLink,
	PS3_TOF_Unknown
};

struct PS3_DirectoryEntry
{
	PS3_TYPE_OF_FILE	ucType;
	char				szName[ASURA_PS3_MAX_PATH];
};

class Asura_PS3_FileSystem
{
public:
	// Public typedefs...

	typedef int Handle;


	// Public enumerated types...

	enum
	{
		INVALID_HANDLE	= -1,
	};


	// Public functions...

	// Static initialisation/shutdown functions...
	static	void	Initialise					(const char* szBootFile, bool bForceDiscAccess);
	static	void	Shutdown					(void);

	// Reading/writing functions...
	static bool OpenForAppending	(const char* szFilename, Handle* pxHandle, bool bForceHOSTFS = false, bool bOpenInCache = false );
	static bool OpenForReading		(const char* szFilename, Handle* pxHandle, bool bForceHOSTFS = false, bool bOpenInCache = false );
	static bool	OpenForWriting		(const char* szFilename, Handle* pxHandle, bool bForceHOSTFS = false, bool bOpenInCache = false );
	static bool	OpenForReadWrite	(const char* szFilename, Handle* pxHandle, bool bForceHOSTFS = false, bool bOpenInCache = false );

	static	u_int	ReadData					(const Handle xHandle, const u_int uNumberOfBytes, void* pData);
	static	u_int	SkipData					(const Handle xHandle, const u_int uNumberOfBytes);
	static	u_int	WriteData					(const Handle xHandle, const u_int uNumberOfBytes, const void* pData);

	static	bool	SetCurrentPosition			(const Handle xHandle, const u_int uPosition);
	static	u_int	GetCurrentPosition			(const Handle xHandle);

	static	u_int	GetFileSize					(const Handle xHandle);

	static	bool	Close						(Handle* pxHandle);
	static	void	GenerateFullFilename		(const char* szInput, char* szOutputBfr, u_int uOutputBfrSize, bool bForceHOSTFS = false);
	static const char* GetMountPrefix(bool bForceHOSTFS = false);

	static	int		OpenDir						(const char* szFullPath); //returns the file descriptor
	static	int		ReadDir						(int iFileDescriptor, PS3_DirectoryEntry* pxDirectoryEntry ); // returns <0 in case of error; 0 if no more entries; !=0 read succeeded
	static	void	CloseDir					(int iFileDescriptor );

	static bool		CreateDir					( const char* const szDir );

	static bool		DiscExists					( void ) { return true; }

	static void		GenerateFilenameInCache		(const char* szInput, char* szOutputBfr, u_int uOutputBfrSize);
	static void		ClearCache					( void );

#ifdef ASURA_SUPPORT_DOWNLOADABLE_CONTENT
	static void		GenerateDLCFilename(const char* szInput, char* szOutputBfr, u_int uOutputBfrSize);
#endif //#ifdef ASURA_SUPPORT_DOWNLOADABLE_CONTENT

	// Moving the main folder search functionality here so we can search both our main data directory and our DLC directory, as well as allowing the DLC code to find what files have been downloaded.
	static void FindFiles(Asura_List<Asura_String>& xFiles, const char* szMountPrefix, const char* szDirectory, bool bRecursive = true);

private:

	// Since all the generate filename functions basically do the same thing, this function merges the functionality they share.
	static void GenerateFilenameFromPrefix(const char* szPrefix, const char* szInput, char* szOutputBfr, u_int uOutputBfrSize);

	// static data
	static bool			s_bInitialised;
	static const char*	s_szMountPrefix;

	// Private functions...

	Asura_PS3_FileSystem  (void);
	Asura_PS3_FileSystem  (const Asura_PS3_FileSystem& xOther);
	~Asura_PS3_FileSystem ();

			Asura_PS3_FileSystem&	operator=	(const Asura_PS3_FileSystem& xOther);

};   // End of class 'Asura_PS3_FileSystem'.


#endif   // ifndef __ASURA_PS3_FILESYSTEM_H__


/// End of file. ///
