//////////////////////////////////////////////////////////////////////////////
//
// Asura_Platform_NetLinkManager.cpp
//
// Copyright (c) 2007 Rebellion. All rights reserved.
//
// Description:
//
//     PS3 implementation of network link class functions.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

#ifdef ASURA_SUPPORT_EXTERNAL_NETWORK

// this file's header
#include "Network/Asura_NetLinkManager.h"

// Asura independent
#include "Debugging/Asura_Assert.h"

// PS3 headers
#include "Network/Asura_PS3_Network.h"
#ifdef ASURA_SUPPORT_LOBBY
#include "Network/Online/Asura_PS3_NP.h"
#endif

enum ASURA_PS3_NETLINK_PHASE
{
	ASURA_PS3_NETLINK_PHASE_IDLE,
	ASURA_PS3_NETLINK_PHASE_STARTING_NETWORK,
#ifdef ASURA_SUPPORT_LOBBY
	ASURA_PS3_NETLINK_PHASE_INIT_NP,
#endif
};

static ASURA_PS3_NETLINK_PHASE g_eNetlinkPhase;

// Platform_UpdateXXX() functions should call FinishedPlatformXXX() when finished.
void Asura_NetLinkManager::Platform_UpdateConnecting()
{
	switch (g_eNetlinkPhase)
	{
		case ASURA_PS3_NETLINK_PHASE_STARTING_NETWORK:
		{
			if (!Asura_PS3_Network::IsConnecting())
			{
				if (Asura_PS3_Network::GetConnectionError())
				{
					FinishedPlatformConnecting(false);
					Asura_PS3_Network::Shutdown();
				}
				else if (Asura_PS3_Network::IsConnected())
				{
#ifdef ASURA_SUPPORT_LOBBY
					g_eNetlinkPhase = ASURA_PS3_NETLINK_PHASE_INIT_NP;
					Asura_PS3_NP::Initialise();
#else
					FinishedPlatformConnecting(true);
					g_eNetlinkPhase = ASURA_PS3_NETLINK_PHASE_IDLE;
#endif
				}
			}
			break;
		}

#ifdef ASURA_SUPPORT_LOBBY
		case ASURA_PS3_NETLINK_PHASE_INIT_NP:
		{
			if (Asura_PS3_NP::UpdateInitialisation())
			{
				if (Asura_PS3_NP::IsInErrorState())
				{
					FinishedPlatformConnecting(false);
					Asura_PS3_NP::Shutdown();
					Asura_PS3_Network::Shutdown();
					g_eNetlinkPhase = ASURA_PS3_NETLINK_PHASE_IDLE;
				}
				else
				{
					FinishedPlatformConnecting(true);
					g_eNetlinkPhase = ASURA_PS3_NETLINK_PHASE_IDLE;
				}
			}
			break;
		}
#endif
		default:
		{
			Asura_Assert(false, "unexpected connection phase");
			break;
		}
	}
}

void Asura_NetLinkManager::Platform_UpdateConnecting_PostRender()
{
}

void Asura_NetLinkManager::Platform_UpdateDisconnecting()
{
#ifdef ASURA_SUPPORT_LOBBY
	Asura_PS3_NP::Shutdown();
#endif
	Asura_PS3_Network::Shutdown();

	FinishedPlatformDisconnecting();
}

void Asura_NetLinkManager::Platform_UpdateConnected()
{
	if (Asura_PS3_Network::GetConnectionError())
	{
		ConnectionDropped();
	}
#ifdef ASURA_SUPPORT_LOBBY
	else if (Asura_PS3_NP::IsInErrorState())
	{
		ConnectionDropped();
	}
	else
	{
		Asura_PS3_NP::Update();
	}
#endif
}

// Platform_InitiateConnection() function should return true iff requested operation was started.
bool Asura_NetLinkManager::Platform_InitiateConnection()
{
	g_eNetlinkPhase = ASURA_PS3_NETLINK_PHASE_STARTING_NETWORK;
	return Asura_PS3_Network::Initialise();
}

void Asura_NetLinkManager::Platform_InitiateDisconnection()
{
}


#endif // #ifdef ASURA_SUPPORT_EXTERNAL_NETWORK

