//////////////////////////////////////////////////////////////////////////////
//
// Asura_PS3_NP_Basic.h
//
// Copyright (c) 2008 Rebellion. All rights reserved.
//
// Description:
//
//     wrapper for PS3 NP Basic API, which provides friends list, block list,
//     recent players etc.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef ASURA_PS3_NP_BASIC_H___
#define ASURA_PS3_NP_BASIC_H___ 1

// I N C L U D E S ///////////////////////////////////////////////////////////
#include "Core/Asura_RingBuffer.h"
#include <np.h>

// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////////
class Asura_Mutex;


// C L A S S   D E C L A R A T I O N S ///////////////////////////////////////
class Asura_PS3_NP_Basic_Event
{
public:
	int m_iEvent;
	int m_iRetCode;
	// not needed? 
	// u_int m_uReqId;
};

class Asura_PS3_InvitationInfo 
{
public:
	SceNpMatching2RoomId m_xRoomId;
	// if we want to use passwords:
	//bool m_bWithPassword;
	//SceNpMatching2SessionPassword m_xRoomPassword;
};


class Asura_PS3_NP_Basic
{
public:

	static bool IsInitialised() { return s_pxFriends != NULL; }
	static void Initialise();
	static void Update();
	static void Shutdown();
	static void AddConsoleCmdsAndVars();

	static bool IsPlayerAFriend(const SceNpId& xNpId);
	static void AddPlayerToRecentPlayersList( const SceNpId& xID );
	static void AddPlayersToRecentPlayersList( const SceNpId* pxIDs, u_int uNumPlayers );
	static const SceNpId* GetFriendsList() { return s_pxFriends; }
	static u_int GetFriendsListSize() { return s_uNumFriends; }
	static const SceNpId* GetFriendsListIncludingSelf() { return s_pxFriends; }// TODO.
	static u_int GetFriendsListSizeIncludingSelf() { return s_uNumFriends; }

	static bool IsPlayerOnBlockList(const SceNpId& xNpId);
	static const SceNpId* GetBlockList() { return s_pxBlockList; }
	static u_int GetBlockListSize() { return s_uBlockListSize; }

	static void Refresh();

	static bool IsInvitePending() { return s_bInvitePending; }	// returns true when invite has been accepted in the gui
	static const Asura_PS3_InvitationInfo& GetInviteInfo();		// only valid when IsInvitePending() has returned true

	static void StartInvitationGui();	// display the PS3 UI which lets you choose an invitation and choose join/don't join.
	static u_int GetNumInvitations()	{ return s_uInvitationCount; }
	static bool GetNewInviteFlag()		{ return s_bNewInvitation; }
	static void ResetNewInviteFlag()	{ s_bNewInvitation = false; }
	// console interface for use in GUIMenus
	static bool ConsoleCmd_StartInvitationGui(bool bHelp);
	static bool ConsoleCmd_ResetNewInviteFlag(bool bHelp);

private:

	// callback
	static int EventHandler( int iEvent, int iRetCode, u_int uReqId, void* pArg );
	// function to process events in main thread
	static void ProcessEvent(const Asura_PS3_NP_Basic_Event* pxEvent);

	// helper functions for friends added after initial fetch
	static void AddFriend(const SceNpId& xNpId);
	static void RemoveFriend(const SceNpId& xNpId);

	static SceNpId* s_pxFriends;
	static u_int s_uNumFriends;
	static u_int s_uFriendlistArraySize;
	static SceNpId* s_pxBlockList;
	static u_int s_uBlockListSize;
	static u_int s_uInvitationCount;
	static bool s_bNewInvitation;
	static Asura_PS3_InvitationInfo s_xInviteInfo;
	static bool s_bInvitePending;

	static Asura_RingBuffer<Asura_PS3_NP_Basic_Event>* s_pxEventBuffer;
	static Asura_Mutex* s_pxMutex;
};

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

#endif // #ifndef ASURA_PS3_NP_BASIC_H___
