//////////////////////////////////////////////////////////////////////////////
//
// Asura_Platform_FriendsList.cpp
//
// Copyright (c) 2008 Rebellion. All rights reserved.
//
// Description:
//
//     PS3 implementation of Asura_FriendsList functions
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

#ifdef ASURA_SUPPORT_LOBBY

// Asura independent includes
#include "Debugging/Asura_Assert.h"
#include "Network/Online/Asura_FriendsList.h"
#include "Network/Online/Asura_Lobby.h"

// Asura PS3 includes
#include "Network/Online/Asura_PS3_NP_Basic.h"

// I N T E R N A L   F U N C T I O N S ///////////////////////////////////////
static int FriendlistClosedCallback(int iRetCode, void* pArg)
{
	Asura_FriendsList::FriendsGUIDeactivated();
	// friends list GUI doesn't cause the XMB closed callback to happen so
	// we need to refresh our list now.
	Asura_FriendsList::Refresh();
	return 0;
}

// C L A S S   I M P L E M E N T A T I O N ///////////////////////////////////
void Asura_FriendsList::Platform_Initialise()
{
}

void Asura_FriendsList::Platform_Shutdown()
{
}

void Asura_FriendsList::Platform_Refresh()
{
	Asura_PS3_NP_Basic::Refresh();
}


bool Asura_FriendsList::Platform_ShowFriendsGUI()
{
	// with firmware >= 240 you don't need a memory container any more.
	int iRet = sceNpFriendlist(FriendlistClosedCallback, NULL, SYS_MEMORY_CONTAINER_ID_INVALID);
	Asura_Assertf(iRet == CELL_OK, ("sceNpFriendlist failed (0x%x)", iRet));
	return (iRet == CELL_OK);
}

bool Asura_FriendsList::Platform_IsPlayerAFriend(const Asura_Lobby_Player& xPlayer)
{
	return Asura_PS3_NP_Basic::IsPlayerAFriend(xPlayer.GetPlatformData().m_xNpID);
}

#endif // #ifdef ASURA_SUPPORT_LOBBY
