// SCE CONFIDENTIAL
// PLAYSTATION(R)3 Programmer Tool Runtime Library 250.001
// Copyright (C) 2008 Sony Computer Entertainment Inc.
// All Rights Reserved.

#ifndef __CELL_UTIL_SPURS_COMMAND_LIST_DISPATCHER_H__
#define __CELL_UTIL_SPURS_COMMAND_LIST_DISPATCHER_H__ 1

#include <cellUtilSpursJobChain.h>
#include <cellUtilSpursJobCommand.h>
#include <cellUtilSpursEventFlag.h>
#include <cellUtilSpursJobGuard.h>
#include <cellUtilSpursJobNotification.h>
#include <cell/atomic.h>

__CELL_UTIL_SPURS_BEGIN

class CommandListDispatcher : public JobChain {
	CommandListDispatcher(const CommandListDispatcher&);
	CommandListDispatcher& operator =(const CommandListDispatcher&);

//	uint64_t __reserved__[14]; // reserved for future update.
	CellSpursJobList mJobList;
	uint32_t mHasSet;
	uint32_t __reserved__;
	Job::Command mCommand[11];
	// aligned to 128 bytes
	EventFlag mEventFlag;
	JobGuard mGuard;
	Job::Notification mNotification;
	uint64_t __aligned_to_128bytes__[8];

public:
	static int create(CommandListDispatcher* jobChain,
					  const char *name,
					  Spurs* spurs,
					  uint16_t sizeJobDescriptor,
					  uint8_t priority = CELL_SPURS_MAX_PRIORITY - 1,
					  uint8_t spuMask = 0xff);

	int dispatch(const Job::Command* command)
		{
			if (command == 0) {
				return CELL_SPURS_JOB_ERROR_NULL_POINTER;
			}
			bool hadSet = cellAtomicStore32(&mHasSet, true);

			if (hadSet) {
				return CELL_SPURS_JOB_ERROR_BUSY;
			} else {
				mCommand[1] = Job::Call(command);
				mGuard.notify();
				return cellSpursRunJobChain(this);
			}
		}

	int wait()
		{
			if (mHasSet) {
				int ret = mNotification.wait();
				if (ret == CELL_OK) {
					mHasSet = false;
				}
				return ret;
			} else {
				return CELL_SPURS_JOB_ERROR_STAT;
			}
		}

} __attribute__((aligned(CELL_SPURS_JOBCHAIN_ALIGN)));

__CELL_UTIL_SPURS_END

#endif // __CELL_UTIL_SPURS_COMMAND_LIST_DISPATCHER_H__

// Local Variables:
// mode: C++
// c-file-style: "stroustrup"
// tab-width: 4
// End:
// vim:sw=4:sts=4:ts=4
