// SCE CONFIDENTIAL
// PLAYSTATION(R)3 Programmer Tool Runtime Library 250.001
// Copyright (C) 2008 Sony Computer Entertainment Inc.
// All Rights Reserved.

#ifndef __CELL_UTIL_SPURS_JOB_COMMAND_H__
#define __CELL_UTIL_SPURS_JOB_COMMAND_H__ 1

#include <cell/spurs/job_commands.h>
#include <cell/spurs/job_descriptor.h>	// CellSpursJobList
#include <cellUtilSpursDefine.h>
#include <cellUtilSpursJobGuard.h>
#include <cellUtilSpursJobDescriptor.h>

__CELL_UTIL_SPURS_BEGIN

namespace Job {
	class Command {
		uint64_t m_command;
	public:
		Command(uint64_t command) : m_command(command) {}
		operator uint64_t() { return m_command; }
	};
	class Nop : public Command {
	public:
		Nop() : Command(CELL_SPURS_JOB_COMMAND_NOP) {}
	};
	class Job : public Command {
	public:
		Job(Descriptor& jd) : Command(CELL_SPURS_JOB_COMMAND_JOB(&jd)) {}
		Job(Descriptor* jd) : Command(CELL_SPURS_JOB_COMMAND_JOB(jd)) {}
	};
	class JobList : public Command {
	public:
		JobList(CellSpursJobList& jl) : Command(CELL_SPURS_JOB_COMMAND_JOBLIST(&jl)) {}
		JobList(CellSpursJobList* jl) : Command(CELL_SPURS_JOB_COMMAND_JOBLIST(jl)) {}
	};
	class Guard : public Command {
	public:
		Guard(JobGuard& guard) : Command(CELL_SPURS_JOB_COMMAND_GUARD(&guard)) {}
		Guard(JobGuard* guard) : Command(CELL_SPURS_JOB_COMMAND_GUARD(guard)) {}
	};
	class Sync : public Command {
	public:
		Sync() : Command(CELL_SPURS_JOB_COMMAND_SYNC) {}
	};
	class LWSync : public Command {
	public:
		LWSync() : Command(CELL_SPURS_JOB_COMMAND_LWSYNC) {}
	};
	class SetLabel : public Command {
	public:
		SetLabel(unsigned label = 0) : Command(CELL_SPURS_JOB_COMMAND_SET_LABEL(label)) {}
	};
	class SyncLabel : public Command {
		SyncLabel(unsigned label = 0) : Command(CELL_SPURS_JOB_COMMAND_SYNC_LABEL(label)) {}
	};
	class LWSyncLabel : public Command {
		LWSyncLabel(unsigned label = 0) : Command(CELL_SPURS_JOB_COMMAND_LWSYNC_LABEL(label)) {}
	};
	class ResetPC : public Command {
	public:
		ResetPC(const uint64_t* command) : Command(CELL_SPURS_JOB_COMMAND_RESET_PC(command)) {}
		ResetPC(const Command* command) : Command(CELL_SPURS_JOB_COMMAND_RESET_PC(command)) {}
	};
	class Next : public Command {
	public:
		Next(const uint64_t* command) : Command(CELL_SPURS_JOB_COMMAND_NEXT(command)) {}
		Next(const Command* command) : Command(CELL_SPURS_JOB_COMMAND_NEXT(command)) {}
	};
	class Call : public Command {
	public:
		Call(const uint64_t* command) : Command(CELL_SPURS_JOB_COMMAND_CALL(command)) {}
		Call(const Command* command) : Command(CELL_SPURS_JOB_COMMAND_CALL(command)) {}
	};
	class Ret : public Command {
	public:
		Ret() : Command(CELL_SPURS_JOB_COMMAND_RET) {}
	};
	class Abort : public Command {
	public:
		Abort() : Command(CELL_SPURS_JOB_COMMAND_ABORT) {}
	};
	class End : public Command {
	public:
		End() : Command(CELL_SPURS_JOB_COMMAND_END) {}
	};
	class Flush : public Command {
	public:
		Flush() : Command(CELL_SPURS_JOB_COMMAND_FLUSH) {}
	};
}; // namespace Job {

__CELL_UTIL_SPURS_END

#endif // __CELL_UTIL_SPURS_JOB_COMMAND_H__

// Local Variables:
// mode: C++
// c-file-style: "stroustrup"
// tab-width: 4
// End:
// vim:sw=4:sts=4:ts=4
