// SCE CONFIDENTIAL
// PLAYSTATION(R)3 Programmer Tool Runtime Library 250.001
// Copyright (C) 2008 Sony Computer Entertainment Inc.
// All Rights Reserved.

#ifndef __CELL_UTIL_SPURS_JOB_NOTIFICATION_H__
#define __CELL_UTIL_SPURS_JOB_NOTIFICATION_H__ 1

#define __CELL_UTIL_SPURS_JOB_NOTIFICATION_BITS 1

#include <cellUtilSpursJobDescriptor.h>
#include <cellUtilSpursEventFlag.h>

extern const char _binary_job_notify_bin_start[];
extern const char _binary_job_notify_bin_size[];

__CELL_UTIL_SPURS_BEGIN

namespace Job {
	class Notification : public Descriptor {
		Notification(const Notification&);
		Notification& operator =(const Notification&);

		union {
			uint32_t mEaEventFlag;
			EventFlag* mEventFlag;
		};
	public:
#ifdef __PPU__
		Notification() : mEventFlag(0) {}
		Notification(EventFlag* eventflag) : mEventFlag(eventflag)
			{
				setBinary(_binary_job_notify_bin_start,
						  _binary_job_notify_bin_size);
			}

		static int initialize(Notification* self, EventFlag* eventflag)
			{
				if (self == 0) {
					return CELL_SPURS_JOB_ERROR_NULL_POINTER;
				}
				self->mEventFlag = eventflag;
				self->setBinary(_binary_job_notify_bin_start,
								_binary_job_notify_bin_size);
				return CELL_OK;
			}

		int tryWait()
			{
				uint16_t bits = __CELL_UTIL_SPURS_JOB_NOTIFICATION_BITS;
				return mEventFlag->tryWait(&bits, CELL_SPURS_EVENT_FLAG_OR);
			}

		int wait()
			{
				uint16_t bits = __CELL_UTIL_SPURS_JOB_NOTIFICATION_BITS;
				if (CELL_OK == mEventFlag->tryWait(&bits, CELL_SPURS_EVENT_FLAG_OR)) {
					return CELL_OK;
				} else {
					__RETURN_IF(mEventFlag->attachLv2EventQueue());
					int ret = mEventFlag->wait(&bits, CELL_SPURS_EVENT_FLAG_OR);
					__RETURN_IF(mEventFlag->detachLv2EventQueue());
					return ret;
				}
			}
#endif // __PPU__

#ifdef __SPU__
		int set()
			{
				return cellSpursEventFlagSet(mEaEventFlag, __CELL_UTIL_SPURS_JOB_NOTIFICATION_BITS);
			}

		int tryWait()
			{
				uint16_t bits = __CELL_UTIL_SPURS_JOB_NOTIFICATION_BITS;
				return cellSpursEventFlagTryWait(mEaEventFlag, &bits, CELL_SPURS_EVENT_FLAG_OR);
			}

		// only for tasks
		int wait()
			{
				uint16_t bits = __CELL_UTIL_SPURS_JOB_NOTIFICATION_BITS;
				return cellSpursEventFlagWait(mEaEventFlag, &bits, CELL_SPURS_EVENT_FLAG_OR);
			}
#endif // __SPU__
	};
}; // namespace Job {

__CELL_UTIL_SPURS_END

#endif // __CELL_UTIL_SPURS_JOB_NOTIFICATION_H__

// Local Variables:
// mode: C++
// c-file-style: "stroustrup"
// tab-width: 4
// End:
// vim:sw=4:sts=4:ts=4
