//
// Asura_PS3_DeferredRendering.h
//
// Created by Catalin Arsenescu - 20 November 2007
//
// Copyright (C) 2005 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif //#ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __ASURA_PS3_DEFERRED_RENDERING_H__
#define __ASURA_PS3_DEFERRED_RENDERING_H__


enum ASURA_DEFERRED_RENDER_TARGET_ENUM
{
	ASURA_DEFERRED_RENDER_TARGET_CURRENT_FRONT_BUFFER			= 0,
	ASURA_DEFERRED_RENDER_TARGET_NORMAL_MATERIAL				= 1,
	ASURA_DEFERRED_RENDER_TARGET_ALBEDO_SPECULAR				= 2,
};


class Asura_PS3_FrameBuffer;
class Asura_PS3_Texture;

class Asura_PS3_DeferredRendering
{
public:
	static void						Initialise				( void );
	static void						Shutdown				( void );

	static void						BeginFrame				( void );
	static void						EndFrame				( void );
	static void						EndMrt					( void );

	static void						Bind					( void );
	static void						BindAsSingleRT			( void );
	static void						BindAsZPassOnly			( void );
	static void						RestoreBind				( void );
	static bool						IsBindAsSingleRT		( void );

	//static Asura_PS3_FrameBuffer*	GetFrameBuffer			( void ); //BW - removed this because people were using it instead of the Bind functions
	static Asura_PS3_FrameBuffer*	GetDeferredFrameBuffer	( void ) { return s_pxDeferredBuffer; }
	static Asura_PS3_FrameBuffer*	GetOutputBuffer			( void ) { return s_pxFinalBuffer; }
	static Asura_PS3_FrameBuffer*	GetTempFrameBuffer		( void ) { return s_pxTempBuffer; }
#ifdef ASURA_USE_HDR
	static Asura_PS3_FrameBuffer*	GetHDRBuffer			( void ) { return s_pxResolvedBuffer; }
	static void						SetAfterToneMapping		( void ) { s_bAfterToneMapping = true; }
#endif
#if defined(ASURA_USE_CSM)
	static Asura_PS3_FrameBuffer*	GetCSMDepthBuffer		( void ) { return s_pxCSMDepthBuffer; }
	static Asura_PS3_FrameBuffer*	GetCSMVarianceBuffer	( void ) { return s_pxCSMVarianceBuffer; }
#endif

	static void						ResolveBackBuffer		( void );
	static Asura_PS3_Texture*		GetResolvedTexture		( void );
	static Asura_PS3_Texture*		GetRTAsTexture( u_int uRenderTarget );
	
	static void						SwitchToFrontBuffer(void);
	static bool						IsRenderingToFrontBuffer() { return s_bUsingFrontBuffer; }


#ifdef ASURA_ASSERT
	static void						AssertFlip				( void );
#endif

protected:
	Asura_PS3_DeferredRendering( void );
	static void						RenderMagnifier			( float fMagnification, u_int uWidth, u_int uHeight );

	static Asura_PS3_FrameBuffer*	s_pxTempBuffer;
	static Asura_PS3_FrameBuffer*	s_pxDeferredBuffer;
	static Asura_PS3_FrameBuffer*	s_pxResolvedBuffer;
	static Asura_PS3_FrameBuffer*	s_pxFinalBuffer;
#if defined(ASURA_USE_CSM)
	static Asura_PS3_FrameBuffer*	s_pxCSMDepthBuffer;
	static Asura_PS3_FrameBuffer*	s_pxCSMVarianceBuffer;
#endif
	static bool						s_bAfterToneMapping;
	static bool						s_bUseResolvedBuffer;

	// true if we have already switched over the the front buffer, false if we are still rendering to the back buffer
	static bool						s_bUsingFrontBuffer;

};

#endif //#ifndef __ASURA_PS3_DEFERRED_RENDERING_H__
