//
// Asura_PS3_EnvironmentRenderer.cpp
//
// Created by KJFL - 22 June 2005
//
// Copyright (C) 2005 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

// This file's header.
#include "Rendering/Asura_PS3_EnvironmentRenderer.h"

// Stdlib platform-specific.
#include <stdlib.h>

// Asura independent headers
#include "Animation/Asura_TextureAnimation.h"
#include "Core/Asura_Material_System.h"
#include "Core/Asura_Maths.h"
#include "Camera/Asura_Camera.h"
#include "Debugging/Asura_CPUProfile.h"
#include "Light/Asura_Photon.h"
#include "Rendering/Asura_Module.h"
#include "Rendering/Asura_Colour.h"
#include "Rendering/Asura_Environment_Renderer.h"
#include "Rendering/Asura_Render.h"
#include "Light/Asura_GI_Solution.h"
#include "Core/Asura_MaterialResponse_System.h"
#include "Core/Asura_Timers.h"
#include "Rendering/Asura_DeferredShading.h"
#include "Rendering/Asura_Viewport.h"
#include "SpecialFX/PFX/Asura_PFX_System.h"
#include "Visibility/Asura_Visibility_System.h"
#include "Interface/Asura_CommandConsole.h"

// Asura PS3 headers
#include "ChunkSystem/Asura_Chunk_ResourceFile_PS3.h"
#include "Rendering/Shaders/Asura_PS3_GCM_Shader.h"
#include "Rendering/Asura_PS3_Fog.h"
#include "Rendering/Asura_PS3_FrameBuffer.h"
#include "Rendering/Asura_PS3_Graphics.h"
#include "Rendering/Asura_PS3_MaterialResponse_System.h"
#include "Rendering/Asura_PS3_QuadOverlayRenderer.h"
#include "Rendering/Asura_PS3_RenderingStates.h"
#include "Rendering/Asura_PS3_RSXProfiling.h"
#include "Rendering/Asura_PS3_TextureManagement.h"
#include "Rendering/Asura_PS3_EdgeAntiAliasing.h"

//Shaders definition
#include "Rendering/ShadersDefinition/VD/Env_VD.h"
#include "Rendering/ShadersDefinition/VP/Env_VP.h"
#include "Rendering/ShadersDefinition/FP/Env_FP.h"
#include "Rendering/ShadersDefinition/FP/Env_Forward_FP.h"

#include "Rendering/ShadersDefinition/VP/Env_Shadow_VP.h"

#include "Rendering/ShadersDefinition/VP/Env_ZPass_VP.h"
#include "Rendering/ShadersDefinition/FP/ClearAlpha_FP.h"

#include "Rendering/ShadersDefinition/VD/Water_VD.h"
#include "Rendering/ShadersDefinition/VP/Water_VP.h"
#include "Rendering/ShadersDefinition/FP/Water_FP.h"

// G L O B A L S /////////////////////////////////////////////////////////////
u_int									Asura_PS3_EnvironmentRenderer::s_uNumberOfStrips			= 0;
int										Asura_PS3_EnvironmentRenderer::s_iNumSolidStrips			= 0;
Asura_PS3_EnvironmentRenderer_Strip*	Asura_PS3_EnvironmentRenderer::s_pxStrips					= NULL;
Asura_PS3_EnvironmentRenderer_Buffer*	Asura_PS3_EnvironmentRenderer::s_pxBuffers					= NULL;
int*									Asura_PS3_EnvironmentRenderer::s_aiQueuedWaterStrips		= NULL;
int*									Asura_PS3_EnvironmentRenderer::s_aiSolidAlphaTestStrips		= NULL;
int*									Asura_PS3_EnvironmentRenderer::s_aiSolidStrips				= NULL;
int*									Asura_PS3_EnvironmentRenderer::s_aiTransparentStrips		= NULL;
bool*									Asura_PS3_EnvironmentRenderer::s_abStripInRenderList		= NULL;

u_int									Asura_PS3_EnvironmentRenderer::s_uNumVertexBuffers			= 0;
Asura_PS3_GCM_VertexBuffer*				Asura_PS3_EnvironmentRenderer::s_pxVertexBuffer				= NULL;
Asura_PS3_GCM_IndexBuffer				Asura_PS3_EnvironmentRenderer::s_xAntiAliasEdges;

Asura_Hash_ID							Asura_PS3_EnvironmentRenderer::s_uStripMaterialHash1		= ASURA_HASH_ID_UNSET;
Asura_Hash_ID							Asura_PS3_EnvironmentRenderer::s_uStripMaterialHash2		= ASURA_HASH_ID_UNSET;
const Asura_MaterialResponse*			Asura_PS3_EnvironmentRenderer::s_pxStripMaterial1			= NULL;
const Asura_MaterialResponse*			Asura_PS3_EnvironmentRenderer::s_pxStripMaterial2			= NULL;
Asura_PS3_FragmentShaderCopy*			Asura_PS3_EnvironmentRenderer::s_pxStripShader				= NULL;
float									Asura_PS3_EnvironmentRenderer::s_fStripAmbientContribution	= 0.0f;
float									Asura_PS3_EnvironmentRenderer::s_afStripPrelightColour[4]	= {0.0f,0.0f,0.0f,0.0f};
u_int									Asura_PS3_EnvironmentRenderer::s_uStripVertexBufferId		= 0xffffffff;

static Asura_PS3_GCM_VertexShader*		g_pxVertexShaderPL				= NULL;
static Asura_PS3_GCM_VertexShader*		g_pxVertexShaderShadowCubicMap	= NULL;
static Asura_PS3_GCM_VertexShader*		g_pxVertexShader				= NULL;
static Asura_PS3_GCM_VertexShader*		g_pxVertexShaderWater			= NULL;

static Asura_PS3_FragmentShader*		g_pxFragmentShaderZ				= NULL;
static Asura_PS3_GCM_VertexShader*		g_pxVertexShaderZ				= NULL;

//debugging - commmand console values:
//BW to help the compiler optimise I've set these to const if they're not currently being used.
static bool								g_bUseSorting			= true;
static const bool						g_bCacheSort			= true;
static const bool						g_bInsertionSort		= true;
static bool								g_bCacheStripMaterial	= true;
static const bool						g_bSortMatFirst			= true; // true = help PPU, false = help RSX (slightly better ZCULL)

//
// Asura_PS3_EnvironmentRenderer
//

void Asura_PS3_EnvironmentRenderer::Initialise()
{
#if defined( ASURA_SUPPORT_COMMAND_CONSOLE )
	Asura_CommandConsole::AddVar( L"Render.Environment.CacheMaterial", &g_bCacheStripMaterial, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::AddVar( L"Render.Environment.SortStrips", &g_bUseSorting, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
//	Asura_CommandConsole::AddVar( L"Render.Environment.CacheSortOrder", &g_bCacheSort, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
//	Asura_CommandConsole::AddVar( L"Render.Environment.InsertionSort", &g_bInsertionSort, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
//	Asura_CommandConsole::AddVar( L"Render.Environment.SortMaterialFirst", &g_bSortMatFirst, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
#endif

	g_pxFragmentShaderZ				= new Asura_PS3_FragmentShader( (unsigned char *)&_binary_PS3_DeferredPhoton_P0_FP_bin_start, (unsigned char *)&_binary_PS3_DeferredPhoton_P0_FP_bin_end, NULL );
	g_pxVertexShaderZ				= new Asura_PS3_VP_Env_ZPass();
	g_pxVertexShaderShadowCubicMap	= new Asura_PS3_VP_Env_Shadow_CubicMap();
	g_pxVertexShaderPL				= new Asura_PS3_VP_Env_Prelighting();
	g_pxVertexShaderWater			= new Asura_PS3_VP_Water();

	g_pxVertexShaderPL->InitVertexBufferDeclaration( g_axVertexBufferDeclarationEnv, VD_ELEMENTS );
	g_pxVertexShaderWater->InitVertexBufferDeclaration( g_axVertexDeclarationWater, VDW_ELEMENTS );
}

int Asura_PS3_EnvironmentRenderer::GetVertexSize( void )
{
	return ENVIRONMENT_VERTEX_SIZE;
}

static int SortStripsFrontToBack( const void *pC1, const void *pC2 )
{
	const u_int uStrip1 = *static_cast<const u_int*>(pC1);
	const u_int uStrip2 = *static_cast<const u_int*>(pC2);

	Asura_Bounding_Box &xBoundingBox1 = Asura_PS3_EnvironmentRenderer::GetStrip( uStrip1 )->m_xBoundingBox;
	Asura_Bounding_Box &xBoundingBox2 = Asura_PS3_EnvironmentRenderer::GetStrip( uStrip2 )->m_xBoundingBox;
	Asura_Vector_3 xCameraPosition = *Asura_Camera::GetPosition();
	Asura_Vector_3 xCameraForward;
	float fDist1, fDist2;

	Asura_Camera::GetForwardVector( xCameraForward );
	fDist1 = ( ( xBoundingBox1.CalculateCentre() - xCameraPosition ) * xCameraForward ) - xBoundingBox1.CalculateRadius();
	fDist2 = ( ( xBoundingBox2.CalculateCentre() - xCameraPosition ) * xCameraForward ) - xBoundingBox2.CalculateRadius();

	if ( g_bSortMatFirst )
	{
		if ( (int)Asura_PS3_EnvironmentRenderer::GetStrip( uStrip1 )->m_auMaterialResponseHashIDs[0] != (int)Asura_PS3_EnvironmentRenderer::GetStrip( uStrip2 )->m_auMaterialResponseHashIDs[0] )
		{
			return (int)Asura_PS3_EnvironmentRenderer::GetStrip( uStrip1 )->m_auMaterialResponseHashIDs[0] - (int)Asura_PS3_EnvironmentRenderer::GetStrip( uStrip2 )->m_auMaterialResponseHashIDs[0];
		}
	}
	else
	{
		int iDist1 = (int)fDist1;
		int iDist2 = (int)fDist2;

		if ( iDist1 == iDist2 )
		{
			//BW secondary sort by material, it helps GPU & CPU render times
			return (int)Asura_PS3_EnvironmentRenderer::GetStrip( uStrip1 )->m_auMaterialResponseHashIDs[0] - (int)Asura_PS3_EnvironmentRenderer::GetStrip( uStrip2 )->m_auMaterialResponseHashIDs[0];
		}
	}
	return (int)(fDist1 - fDist2);
}

static int SortStripsBackToFront( const void *pC1, const void *pC2 )
{
	const u_int uStrip1 = *static_cast<const u_int*>(pC1);
	const u_int uStrip2 = *static_cast<const u_int*>(pC2);
	Asura_Bounding_Box &xBoundingBox1 = Asura_PS3_EnvironmentRenderer::GetStrip( uStrip1 )->m_xBoundingBox;
	Asura_Bounding_Box &xBoundingBox2 = Asura_PS3_EnvironmentRenderer::GetStrip( uStrip2 )->m_xBoundingBox;
	Asura_Vector_3 xCameraPosition = *Asura_Camera::GetPosition();
	Asura_Vector_3 xCameraForward;
	float fDist1, fDist2;

	Asura_Camera::GetForwardVector( xCameraForward );
	fDist1 = ( ( xBoundingBox1.CalculateCentre() - xCameraPosition ) * xCameraForward ) - xBoundingBox1.CalculateRadius();
	fDist2 = ( ( xBoundingBox2.CalculateCentre() - xCameraPosition ) * xCameraForward ) - xBoundingBox2.CalculateRadius();

	//BW not doing a material sort in here, because transparency needs strict depth sorting

	return (int)(fDist2 - fDist1);
}

//SortStrips
// Ben Weston 12/3/08
// strip-order is cached, so optimize for a roughly in-order list - using an insertion sort.
// this goes mega slow for a frame when a new module appears, so use a different sort for that situation
static void SortStrips( int* piIndexArray, int iNumStrips )
{
	Asura_Vector_3 xCameraPosition = *Asura_Camera::GetPosition();
	Asura_Vector_3 xCameraForward;
	float fDistI, fDistJ;
	u_int uMatI, uMatJ;

	if ( iNumStrips <= 1 ) return;

	Asura_Camera::GetForwardVector( xCameraForward );

	//compute distance for last item in sorted list (sorted list is initially 1 item long)
	const Asura_PS3_EnvironmentRenderer_Strip* pxStripJ = Asura_PS3_EnvironmentRenderer::GetStrip( piIndexArray[0] );
	const Asura_Bounding_Box* pxBoundingBox = &pxStripJ->m_xBoundingBox;
	fDistJ = ( ( pxBoundingBox->CalculateCentre() - xCameraPosition ) * xCameraForward ) - pxBoundingBox->CalculateRadius();
	uMatJ = pxStripJ->m_auMaterialResponseHashIDs[0];

	//insertion sort the list, re-using values so that a mostly-sorted list goes much quicker
	for ( int i=1; i < iNumStrips; i++ )
	{
		const Asura_PS3_EnvironmentRenderer_Strip* pxStripI = Asura_PS3_EnvironmentRenderer::GetStrip( piIndexArray[i] );
		pxBoundingBox = &pxStripI->m_xBoundingBox;
		fDistI = ( ( pxBoundingBox->CalculateCentre() - xCameraPosition ) * xCameraForward ) - pxBoundingBox->CalculateRadius();
		uMatI = pxStripI->m_auMaterialResponseHashIDs[0];
		
		//search the sorted list (everything below i) for the right place for this strip
		int j=i-1;
		while ( j > 0 )
		{
			if ( g_bSortMatFirst )
			{
				if ( uMatJ < uMatI || ( uMatJ == uMatI && fDistJ < fDistI ) )
					break;
			}
			else
			{
				if ( ( (int)fDistJ == (int)fDistI && uMatJ <= uMatI ) || fDistJ < fDistI )
					break;
			}
			j--;
			//compute distance for next item
			pxStripJ = Asura_PS3_EnvironmentRenderer::GetStrip( piIndexArray[j] );
			pxBoundingBox = &pxStripJ->m_xBoundingBox;
			fDistJ = ( ( pxBoundingBox->CalculateCentre() - xCameraPosition ) * xCameraForward ) - pxBoundingBox->CalculateRadius();
			uMatJ = pxStripJ->m_auMaterialResponseHashIDs[0];
		}

		if ( j != i-1 )
		{
			//insert after j
			int t = piIndexArray[i];
			//can't memcpy overlapping areas so:
			for ( int k=i; k>j+1; k-- )
			{
				piIndexArray[k]=piIndexArray[k-1];
			}
			piIndexArray[j+1] = t;
			
			//compute distance for last item in sorted list
			pxStripJ = Asura_PS3_EnvironmentRenderer::GetStrip( piIndexArray[i] );
			pxBoundingBox = &pxStripJ->m_xBoundingBox;
			fDistJ = ( ( pxBoundingBox->CalculateCentre() - xCameraPosition ) * xCameraForward ) - pxBoundingBox->CalculateRadius();
			uMatJ = pxStripJ->m_auMaterialResponseHashIDs[0];
		}
		else
		{
			//compute distance for last item in sorted list, which is i in this case
			fDistJ = fDistI;
			uMatJ = uMatI;
		}
	}
}

Asura_PS3_EnvironmentRenderer_Strip* Asura_PS3_EnvironmentRenderer::GetStrip( u_int uStrip )
{
	if( uStrip < s_uNumberOfStrips )
	{
		return &s_pxStrips[ uStrip ];
	}
	Asura_Assert( false, "Request of an invalid strip" );
	return &s_pxStrips[ 0 ];
}

void Asura_PS3_EnvironmentRenderer::RenderZPassOnly( void )
{
	ASURA_PS3_RSX_PROFILE__BEGIN( ASURA_PS3_RSX_PROFILE__ENV, "Environment Render Z Pass" );
	const Asura_AMatrix4x4& xViewProjectionMatrix = Asura_PS3_Graphics::GetViewProjectionMatrix();
	g_pxVertexShaderZ->SetUniformParameter( VIEW_PROJECTION_MATRIX_ZPASS, xViewProjectionMatrix.GetElements() );
	g_pxVertexShaderZ->Bind();
	g_pxFragmentShaderZ->Bind();

#ifdef ASURA_USE_NEW_VISIBILITY_SYSTEM
	bool	bXRayMode = Asura_Visibility_System::ShouldRenderInXRayMode();
	if ( !bXRayMode )
#endif
	{
		Asura_PS3_DeferredRendering::BindAsZPassOnly();
	}
	
	Asura_PS3_RenderingStates::SetColorMaskMrtState( ASURA_PS3_MRT_NONE );

	Asura_PS3_RenderingStates::RequestAlphaTest( false );
	Asura_PS3_RenderingStates::RequestCullMode( ASURA_CULLMODE_CW );
	Asura_PS3_RenderingStates::RequestZBufferCompareMode( ASURA_ZBUFFERCMP_LESSEQUAL );
	Asura_PS3_RenderingStates::RequestZBufferEnabled( true );
	Asura_PS3_RenderingStates::RequestZBufferWriteEnabled( true );

	bool bFlushSortCache = ( !g_bUseSorting || !g_bCacheSort ); //obviously a cached sort is very misleading if we turn off sorting

#ifdef ASURA_USE_NEW_VISIBILITY_SYSTEM
	bFlushSortCache = (bFlushSortCache || bXRayMode);
#endif
	
	if ( bFlushSortCache ) 
	{
		Asura_Mem::Set(s_abStripInRenderList,false,s_uNumberOfStrips);
	}

	// Ensure that our strip-visibility is up-to date...
	Asura_Environment_Renderer::UpdateVisibilityOfStrips();

	//check the strips we've already listed to see if their modules have stopped being visible
	//pack together the list using some nice memcpys (which assume most things from a module will be concurrent)
	int iCpyOffset = 0;
	int iCpyStart = s_iNumSolidStrips;
	int iNewNum = s_iNumSolidStrips;
	if ( bFlushSortCache )
	{
		s_iNumSolidStrips = 0;
	}
	else
	{
		for( int i=0; i < (int)s_iNumSolidStrips; i++ )
		{
			bool	bStripVisible	= Asura_Environment_Renderer::IsStripVisibleFromCurrentCamera(s_aiSolidStrips[i]);

			// Only bother with the material tests on strips we know we need to render...
			if(bStripVisible)
			{
				const Asura_MaterialResponse* const	pxMaterial1	= Asura_MaterialResponse_System::GetMaterialByHashID( s_pxStrips[s_aiSolidStrips[i]].m_auMaterialResponseHashIDs[0] );

				if((pxMaterial1 != NULL) && pxMaterial1->GetDontRender())
				{
					bStripVisible	= false;
				}
			}

			if(!bStripVisible)
			{
				//strip was rendered last frame but its module is no longer visible
				//sanity check: Asura_Assert(s_bStripInRenderList[s_aiSolidStrips[i]],"strip cache - trying to delete a deleted strip!");

				s_abStripInRenderList[s_aiSolidStrips[i]] = false;
				if ( iCpyStart < i )
				{
					//copy the preceding block of contiguous data (since this is being removed the data won't be contiguous any more)
					int iSize=i-iCpyStart;
					for ( int c=0; c<iSize; c+=iCpyOffset )
					{
						//memcpy doesn't like copying overlapping memory so move non-overlapping chunks
						Asura_Mem::Copy(&s_aiSolidStrips[iCpyStart-iCpyOffset+c],&s_aiSolidStrips[iCpyStart+c],4*(((iSize-c)<iCpyOffset)?iSize-c:iCpyOffset));
					}
				}
				//remove from list
				iNewNum--;
				//setup the next copy
				iCpyOffset++;
				iCpyStart = i+1;
			}
		}
		if ( iCpyStart < s_iNumSolidStrips )
		{
			//memcpy( &s_aiSolidStrips[iCpyStart-iCpyOffset], &s_aiSolidStrips[iCpyStart], 4*(s_iNumSolidStrips-iCpyStart) );
			int iSize=s_iNumSolidStrips-iCpyStart;
			for ( int c=0; c<iSize; c+=iCpyOffset )
			{
				//memcpy doesn't like copying overlapping memory
				memcpy(&s_aiSolidStrips[iCpyStart-iCpyOffset+c],&s_aiSolidStrips[iCpyStart+c],4*(((iSize-c)<iCpyOffset)?iSize-c:iCpyOffset));
			}
		}
		s_iNumSolidStrips = iNewNum;
	}

	//add new strips from modules that have become visible
	int iTransparentStrip	= 0;
	int iAlphaTest			= 0;
	int iWaterStrips		= 0;
	bool bSolidStripsAdded = false;

	for( u_int u = 0; u < s_uNumberOfStrips; ++u )
	{
		Asura_PS3_EnvironmentRenderer_Strip& xStrip = s_pxStrips[u];

		if ( !Asura_Environment_Renderer::IsStripVisibleFromCurrentCamera(u) || s_abStripInRenderList[u] )
			continue;

		const Asura_MaterialResponse* const pxMaterial1 = Asura_MaterialResponse_System::GetMaterialByHashID( xStrip.m_auMaterialResponseHashIDs[0] );
		if( pxMaterial1 )
		{
			if( pxMaterial1->GetDontRender() ) continue;
			if( pxMaterial1->GetSpecialFXMode() == Asura_MaterialResponse::SPECIALFX_MODE_WATER )
			{
				s_aiQueuedWaterStrips[ iWaterStrips ] = u;
				++iWaterStrips;
				continue;
			}
#ifdef ASURA_USE_NEW_VISIBILITY_SYSTEM
			if ( pxMaterial1->IsTransparent() || bXRayMode )			
#else
			if( pxMaterial1->IsTransparent() )
#endif
			{
			
				s_aiTransparentStrips[ iTransparentStrip ] = u;
				++iTransparentStrip;
				continue;
			}
			if( pxMaterial1->GetTexture( Asura_MaterialResponse::TRANSPARENCY_MAP ) != ASURA_TEXTURE_NONE )
			{
				s_aiSolidAlphaTestStrips[ iAlphaTest ] = u;
				++iAlphaTest;
				continue;
			}
		}
		s_aiSolidStrips[ s_iNumSolidStrips ] = u;
		++s_iNumSolidStrips;
		s_abStripInRenderList[u] = true;
		bSolidStripsAdded = true;
	}

	if( g_bUseSorting )
	{
		if ( bSolidStripsAdded || !g_bInsertionSort )
		{
			//qsort faster on unsorted list (e.g. when a new module has become visible so lost of new strips have been added)
			qsort( s_aiSolidStrips, s_iNumSolidStrips, sizeof(u_int), SortStripsFrontToBack );
		}
		else
		{
			//insertion sort faster on a mostly sorted list
			SortStrips( s_aiSolidStrips, s_iNumSolidStrips );
		}

		//these other things are so quick it's not worth caching their sorts, so they are always sorting form an unsorted list
		qsort( s_aiSolidAlphaTestStrips,	iAlphaTest,	sizeof(u_int), SortStripsFrontToBack );
	}
	qsort( s_aiTransparentStrips,		iTransparentStrip,	sizeof(u_int), SortStripsBackToFront );
	qsort( s_aiQueuedWaterStrips,		iWaterStrips,		sizeof(u_int), SortStripsBackToFront );

	u_int uBoundId = 0xffffffff;
	for( int i = 0; i < s_iNumSolidStrips; ++i )
	{
		int u = s_aiSolidStrips[ i ];
		const Asura_PS3_EnvironmentRenderer_Strip& xStrip = s_pxStrips[ u ];

		if ( xStrip.m_uVertexBufferId != uBoundId )
		{
			s_pxVertexBuffer[xStrip.m_uVertexBufferId].Bind( g_axVertexBufferDeclarationEnv, 1 );
			uBoundId = xStrip.m_uVertexBufferId;
		}
		s_pxBuffers[ u ].m_xIndexBuffer.RenderTriangleStrip( xStrip.m_uNumberOfIndices, 0 );
	}

	s_aiTransparentStrips[ iTransparentStrip ]	= -1;
	s_aiSolidAlphaTestStrips[ iAlphaTest ]		= -1;
	s_aiSolidStrips[ s_iNumSolidStrips ]		= -1;
	s_aiQueuedWaterStrips[ iWaterStrips ]		= -1;
	ASURA_PS3_RSX_PROFILE__END( ASURA_PS3_RSX_PROFILE__ENV );
}

//clear the cached data, call this before rendering using RenderStrip
void Asura_PS3_EnvironmentRenderer::FlushCachedStrip( void )
{
	//set to things that will not match what is in the current strip
	s_uStripMaterialHash1 = ASURA_HASH_ID_UNSET; //BW apparently this is actually valid sometimes! so need to check NULL anyway
	s_uStripMaterialHash2 = ASURA_HASH_ID_UNSET;
	s_pxStripMaterial1 = NULL;
	s_pxStripMaterial2 = NULL;

	//just clear these for tidyness, shouldn't be accessed without the above
	s_fStripAmbientContribution = 0.0f;
	s_pxStripShader = NULL;

	s_uStripVertexBufferId = 0xffffffff;
}

void Asura_PS3_EnvironmentRenderer::RenderStrip( const int iStripIdx, const bool bTransparentStrips )
{
	int u = iStripIdx;
	const Asura_PS3_EnvironmentRenderer_Strip& xStrip = s_pxStrips[ u ];
	bool bSetStates = false;
	bool bForceDoOver = bTransparentStrips;

	//BW use cached strip material so we can skip a lot of work if it's the same in the next strip
	if ( !g_bCacheStripMaterial ) bForceDoOver = true;

	if ( s_pxStripMaterial1 == NULL || xStrip.m_auMaterialResponseHashIDs[0] != s_uStripMaterialHash1 || bForceDoOver )
	{
		s_uStripMaterialHash1 = xStrip.m_auMaterialResponseHashIDs[0];
		s_pxStripMaterial1 = Asura_MaterialResponse_System::GetMaterialByHashID( s_uStripMaterialHash1 );

		// Set prelighting colour parameters
		s_fStripAmbientContribution = s_pxStripMaterial1->GetAmbientContribution();
		const Asura_RGBA& s_fStripPrelightColour = s_pxStripMaterial1->GetAmbientColour();
		s_afStripPrelightColour[0] = s_fStripPrelightColour.m_fR;
		s_afStripPrelightColour[1] = s_fStripPrelightColour.m_fG;
		s_afStripPrelightColour[2] = s_fStripPrelightColour.m_fB;
		s_afStripPrelightColour[3] = 0.0f;

		bSetStates = true;
	}
	if ( s_pxStripMaterial2 == NULL || xStrip.m_auMaterialResponseHashIDs[1] != s_uStripMaterialHash2 || bForceDoOver )
	{
		s_uStripMaterialHash2 = xStrip.m_auMaterialResponseHashIDs[1];
		s_pxStripMaterial2 = Asura_MaterialResponse_System::GetMaterialByHashID( s_uStripMaterialHash2 );

		bSetStates = true;
	}

	if ( bSetStates ) //BW reinitialise shader if material has changed (even if shader hasn't), shader init does some stuff based on the "current" material (tut-tut)
	{
		Asura_Hash_ID uHashID = s_pxStrips[ u ].m_uFPHashID;
		if( Asura_MaterialResponse_System::GetCurrentOverrideMapHash() != ASURA_HASH_ID_UNSET )
		{
			uHashID = Asura_PS3_MaterialResponse_System::CreateShaderHashID( s_pxStrips[ u ].m_auMaterialResponseHashIDs[0], s_pxStrips[ u ].m_auMaterialResponseHashIDs[1] );
		}
#ifdef ASURA_USE_NEW_VISIBILITY_SYSTEM
		bool	bXRayMode = Asura_Visibility_System::ShouldRenderInXRayMode();
		if( !bXRayMode && bTransparentStrips && Asura_PS3_Fog::GetFogParameters().m_bFogEnabled )
		{
			uHashID += ASURA_PS3_SHADER_FOG;
		}		
#else
		if( bTransparentStrips && Asura_PS3_Fog::GetFogParameters().m_bFogEnabled )
		{
			uHashID += ASURA_PS3_SHADER_FOG;
		}
#endif

		s_pxStripShader = Asura_PS3_GCM_ShadersLibrary::GetResettableFP( ASURA_SHADER_HASH_ENVIRONMENT, uHashID );

		bSetStates = true;
	}

	if( !s_pxStripShader )
	{
		Asura_Assert( false, "Error: No shader set for this strip" );
		return;
	}

	if ( bSetStates )
	{
		g_pxVertexShader->SetUniformParameter( VP_PrelightingColourFilter, ( float* ) &s_afStripPrelightColour[0] );
		g_pxVertexShader->SetUniformParameter( VP_PrelightingScale, ( float* ) &s_fStripAmbientContribution );

		Asura_PS3_MaterialResponse_System::SetEffectParameters( s_pxStripMaterial1, s_pxStripMaterial2, NULL, s_pxStripShader );
		Asura_PS3_MaterialResponse_System::SetProjectContext( s_pxStripMaterial1, s_pxStripMaterial2 );
		Asura_PS3_MaterialResponse_System::SetBlendStates( s_pxStripMaterial1 );

		Asura_TextureAnimationOutput xTexAnim1, xTexAnim2;
		Asura_TextureAnimation_System::GetAnimationState( s_pxStripMaterial1->GetTextureAnim(), NULL, &xTexAnim1 );
		Asura_TextureAnimation_System::GetAnimationState( s_pxStripMaterial2 ? s_pxStripMaterial2->GetTextureAnim() : ASURA_HASH_ID_UNSET, NULL, &xTexAnim2 );
		const float afTexCoordOffset[4] = { xTexAnim1.m_fUOffset, xTexAnim1.m_fVOffset, xTexAnim2.m_fUOffset, xTexAnim2.m_fVOffset };
		g_pxVertexShader->SetUniformParameter( VP_UV_Offsets, afTexCoordOffset );
	}

	if( bTransparentStrips )
	{
		Asura_SphericalHarmonics_Order2_RGB& xSH = Asura_GI_Solution::s_xDynamicSHForCurrentRender;

		const Asura_Vector_3 xStripCentre = xStrip.m_xBoundingBox.CalculateCentre();

		Asura_GI_Solution::GetDynamicIrradiance( xSH, xStripCentre );

		// Static SH will come from the per-vertex values
		Asura_GI_Solution::s_xStaticSHForCurrentRender.Zero();
		Asura_PS3_MaterialResponse_System::SetSHParameters( s_pxStripMaterial1, s_pxStripShader );
		bSetStates = true;
	}

	if ( bSetStates )
	{
		s_pxStripShader->Bind();
	}


	// Render
	if ( xStrip.m_uVertexBufferId != s_uStripVertexBufferId )
	{
		s_pxVertexBuffer[xStrip.m_uVertexBufferId].Bind( g_axVertexBufferDeclarationEnv, VD_ELEMENTS );
		s_uStripVertexBufferId = xStrip.m_uVertexBufferId;
	}
	s_pxBuffers[ u ].m_xIndexBuffer.RenderTriangleStrip( xStrip.m_uNumberOfIndices, 0 );

#ifdef ASURA_EDGE_ANTIALIASING
	// Add Anti-Alias edges to list to render later
	if ( Asura_PS3_EnvironmentRenderer::s_xAntiAliasEdges.IsInitialized() && xStrip.m_uAntiAliasEdgeIdxNum > 0 )
	{
		Asura_PS3_EdgeAntiAliasing::AddEdgeList( &Asura_PS3_EnvironmentRenderer::s_xAntiAliasEdges, &Asura_PS3_EnvironmentRenderer::s_pxVertexBuffer[xStrip.m_uVertexBufferId], &Asura_PS3_Graphics::GetViewProjectionMatrix(), xStrip.m_uAntiAliasEdgeIdxStart, xStrip.m_uAntiAliasEdgeIdxNum );
	}
#endif
}

void Asura_PS3_EnvironmentRenderer::RenderUsingZPrePass( const bool bTransparentStrips )
{
	if( bTransparentStrips )
	{
		if( s_aiTransparentStrips[ 0 ] == -1 ) return;

		ASURA_PS3_RSX_PROFILE__BEGIN( ASURA_PS3_RSX_PROFILE__ENV, "Environment Render Transparent" );

		const Asura_Vector_3* pxCamPos = Asura_Camera::GetPosition();

		const Asura_AMatrix4x4& xViewProjectionMatrix = Asura_PS3_Graphics::GetViewProjectionMatrix();
		Asura_PS3_DeferredRendering::Bind();

		//Prelighting
		g_pxVertexShader	= g_pxVertexShaderPL;
		g_pxVertexShader->Bind();

		//Set view matrix and camera pos
		{
			g_pxVertexShader->SetUniformParameter( VP_ViewProjectionMatrix, xViewProjectionMatrix.GetElements() );
			g_pxVertexShader->SetUniformParameter( VP_CameraPosition, &pxCamPos->x );
		}

		Asura_PS3_RenderingStates::SetColorMaskMrtState( ASURA_PS3_MRT_MRT0123_ALL );
		Asura_PS3_RenderingStates::RequestCullMode( ASURA_CULLMODE_CW );
		Asura_PS3_RenderingStates::RequestZBufferEnabled( true );

		Asura_PS3_RenderingStates::RequestAlphaTest( false );
		Asura_PS3_RenderingStates::RequestZBufferCompareMode( ASURA_ZBUFFERCMP_LESSEQUAL );
		Asura_PS3_RenderingStates::RequestZBufferWriteEnabled( false );
		
		FlushCachedStrip();

		for( int i = 0; s_aiTransparentStrips[ i ] >= 0; ++i )
		{
			RenderStrip( s_aiTransparentStrips[ i ], bTransparentStrips );
		}

        ASURA_PS3_RSX_PROFILE__END( ASURA_PS3_RSX_PROFILE__ENV );
		Asura_PS3_Graphics::Flush();
    }
	else
	{
		if( s_aiSolidAlphaTestStrips[ 0 ] == -1 && s_aiSolidStrips[ 0 ] == -1 ) return;

        const Asura_Vector_3* pxCamPos = Asura_Camera::GetPosition();

		const Asura_AMatrix4x4& xViewProjectionMatrix = Asura_PS3_Graphics::GetViewProjectionMatrix();
		Asura_PS3_DeferredRendering::Bind();

		//Prelighting
		g_pxVertexShader	= g_pxVertexShaderPL;
		g_pxVertexShader->Bind();

		//Set view matrix and camera pos
		{
			g_pxVertexShader->SetUniformParameter( VP_ViewProjectionMatrix, xViewProjectionMatrix.GetElements() );
			g_pxVertexShader->SetUniformParameter( VP_CameraPosition, &pxCamPos->x );
		}

		Asura_PS3_RenderingStates::SetColorMaskMrtState( ASURA_PS3_MRT_MRT0123_ALL );
		Asura_PS3_RenderingStates::RequestCullMode( ASURA_CULLMODE_CW );
		Asura_PS3_RenderingStates::RequestZBufferEnabled( true );

		if( s_aiSolidAlphaTestStrips[ 0 ] != -1 )
		{
			ASURA_PS3_RSX_PROFILE__BEGIN( ASURA_PS3_RSX_PROFILE__ENV, "Environment Render Alpha test" );

			Asura_PS3_RenderingStates::RequestAlphaTest( false );
			Asura_PS3_RenderingStates::RequestZBufferCompareMode( ASURA_ZBUFFERCMP_LESSEQUAL );
			Asura_PS3_RenderingStates::RequestZBufferWriteEnabled( true );

			FlushCachedStrip();

			for( int i = 0; s_aiSolidAlphaTestStrips[ i ] >= 0; ++i )
			{
				RenderStrip( s_aiSolidAlphaTestStrips[ i ], bTransparentStrips );
			}
			
			ASURA_PS3_RSX_PROFILE__END( ASURA_PS3_RSX_PROFILE__ENV );
		}

		if( s_aiSolidStrips[ 0 ] != -1 )
		{
			ASURA_PS3_RSX_PROFILE__BEGIN( ASURA_PS3_RSX_PROFILE__ENV, "Environment Render Opaque" );

			Asura_PS3_RenderingStates::RequestAlphaTest( false );
			Asura_PS3_RenderingStates::RequestZBufferCompareMode( ASURA_ZBUFFERCMP_EQUALS );
			Asura_PS3_RenderingStates::RequestZBufferWriteEnabled( false );

			FlushCachedStrip();

			for( int i = 0; s_aiSolidStrips[ i ] >= 0; ++i )
			{
				RenderStrip( s_aiSolidStrips[ i ], bTransparentStrips );
			}

			ASURA_PS3_RSX_PROFILE__END( ASURA_PS3_RSX_PROFILE__ENV );
		}

		Asura_PS3_Graphics::Flush();
	}
}

void Asura_PS3_EnvironmentRenderer::Render( const bool bTransparentStrips )
{
	if( s_uNumberOfStrips == 0 ) return;

	Asura_Render::ApplyCameraSettings();

#ifdef ASURA_USE_ZPREPASS
	if( Asura_Render::IsZPrepass() )
	{
		RenderZPassOnly();
		RenderUsingZPrePass( false );
	}
	else
	{
		if( bTransparentStrips )
		{
			RenderUsingZPrePass( true );
		}
	}
#else
	if( bTransparentStrips )
	{
		RenderUsingZPrePass( true );
	}
	else
	{
		RenderZPassOnly();
		RenderUsingZPrePass( false );
	}
#endif
}

void Asura_PS3_EnvironmentRenderer::Reset()
{
	if ( s_pxBuffers )
	{
		for( u_int u = 0; u < s_uNumberOfStrips; ++u )
		{
			s_pxBuffers[u].m_xIndexBuffer.Shutdown();
		}
		delete[] s_pxBuffers;
		s_pxBuffers					= NULL;
	}
	
	if ( s_pxVertexBuffer )
	{
		for ( u_int u=0; u < s_uNumVertexBuffers; u++ )
		{
			s_pxVertexBuffer[u].Shutdown();
		}
		delete[] s_pxVertexBuffer;
		s_pxVertexBuffer = NULL;
	}
	
	Asura_PS3_EnvironmentRenderer::s_xAntiAliasEdges.Shutdown();

	delete[] s_pxStrips;
	delete[] s_aiQueuedWaterStrips;
	delete[] s_aiSolidAlphaTestStrips;
	delete[] s_aiTransparentStrips;
	delete[] s_aiSolidStrips;
	delete[] s_abStripInRenderList;

	s_pxStrips					= NULL;
	s_aiQueuedWaterStrips		= NULL;
	s_aiSolidAlphaTestStrips	= NULL;
	s_aiTransparentStrips		= NULL;
	s_aiSolidStrips				= NULL;
	s_abStripInRenderList		= NULL;
	s_uNumberOfStrips			= 0;
	s_iNumSolidStrips			= 0;
}

void Asura_PS3_EnvironmentRenderer::Shutdown()
{
    Reset();

	delete g_pxFragmentShaderZ;
	delete g_pxVertexShaderZ;
	delete g_pxVertexShaderPL;
	delete g_pxVertexShaderWater;
	delete g_pxVertexShaderShadowCubicMap;

	g_pxVertexShaderPL				= NULL;
	g_pxVertexShaderWater			= NULL;
	g_pxFragmentShaderZ				= NULL;
	g_pxVertexShaderZ				= NULL;
	g_pxVertexShaderShadowCubicMap	= NULL;
}

void Asura_PS3_EnvironmentRenderer::RenderWater( void )
{
	if ( s_aiQueuedWaterStrips && s_aiQueuedWaterStrips[ 0 ] < 0 )
	{
		return;
	}

	if (!Asura_Render::IsSystemEnabled(ASURA_RENDER_SYSTEM_WATER))
	{
		return;
	}

	ASURA_PS3_RSX_PROFILE__BEGIN( ASURA_PS3_RSX_PROFILE__ENV, "Environment Render Water Pass Start" );


	Asura_PS3_DeferredRendering::BindAsSingleRT();

	// Clear the dest alpha to 0 first
	float fWidth =  (float)Asura_Viewport::GetWidth();		
	float fHeight = (float)Asura_Viewport::GetHeight();			
	Asura_Overlay_Quad xQuad;
	xQuad.SetRectXY(0,0, fWidth, fHeight);
	xQuad.Colour = 0x00FFFFFF;
	xQuad.TextureIndex = ASURA_TEXTURE_NONE;
	xQuad.TranslucencyMode = ASURA_TRANSLUCENCY_OFF;	
	Asura_PS3_RenderingStates::RequestAlphaTest( false , 0x00);
	Asura_PS3_RenderingStates::SetColorMaskMrtState( ASURA_PS3_MRT_MRT0_ALPHA );
	bool bStencilEnabled = Asura_PS3_RenderingStates::GetStencilTestEnabled();
	Asura_PS3_RenderingStates::SetStencilTestEnabled( false );
	Asura_PS3_QuadOverlayRenderer::RenderCurrentRenderTarget( xQuad );
	Asura_PS3_RenderingStates::SetColorMaskMrtState( ASURA_PS3_MRT_MRT0_ALL );
	Asura_PS3_Graphics::Flush();

	// Water ripples - esentially a heightmap - gets rendered into the the dest. alpha channel
	Asura_PFX_System::Render( ASURA_PFX_RENDER_WATER_DISTORTER );

	Asura_Render::ApplyCameraSettings();

	Asura_PS3_DeferredRendering::ResolveBackBuffer();
	Asura_PS3_DeferredRendering::BindAsSingleRT();

	// Set render states
	Asura_PS3_RenderingStates::RequestAlphaTest( false );
	Asura_PS3_RenderingStates::RequestCullMode( ASURA_CULLMODE_NONE );		
	Asura_PS3_RenderingStates::RequestZBufferWriteEnabled( false ); //BW we're reading at the same time so writing it causes glitches.
	Asura_PS3_RenderingStates::RequestZBufferCompareMode( ASURA_ZBUFFERCMP_LESSEQUAL );
	Asura_PS3_RenderingStates::RequestTranslucencyMode( ASURA_TRANSLUCENCY_NORMAL );
	Asura_PS3_RenderingStates::RequestZBufferEnabled( true );


	g_pxVertexShaderWater->Bind();
	const Asura_Vector_3* pxCamPos = Asura_Camera::GetPosition();
	//Set view matrix and camera pos
	{

		const Asura_AMatrix4x4& xViewProjectionMatrix = Asura_PS3_Graphics::GetViewProjectionMatrix();
		g_pxVertexShaderWater->SetUniformParameter( VPW_ViewProjectionMatrix, xViewProjectionMatrix.GetElements() );
		g_pxVertexShaderWater->SetUniformParameter( VPW_CameraPosition, &pxCamPos->x );
	}

	float faOneOverScreenWidthHeight[ 2 ] = { 1.0f / Asura_PS3_Graphics::GetScreenWidth(), 1.0f / Asura_PS3_Graphics::GetScreenHeight() };

	Asura_PS3_FP_List_Tree* fpTree = Asura_PS3_GCM_ShadersLibrary::GetResettableFPTree( ASURA_SHADER_HASH_ENVIRONMENT );
	if( !fpTree || s_aiQueuedWaterStrips == NULL ) return;

	u_int uBoundId = 0xffffffff;
	for(int i = 0; s_aiQueuedWaterStrips[ i ] >= 0; ++i )
	{
		int iStripIdx = s_aiQueuedWaterStrips[ i ];

		Asura_PS3_EnvironmentRenderer_Strip *pxStrip = &s_pxStrips[ iStripIdx ];

		const Asura_MaterialResponse* pxMaterial1 = Asura_MaterialResponse_System::GetMaterialByHashID( pxStrip->m_auMaterialResponseHashIDs[0] );
		const Asura_MaterialResponse* pxMaterial2 = Asura_MaterialResponse_System::GetMaterialByHashID( pxStrip->m_auMaterialResponseHashIDs[1] );

		if( pxMaterial1 )
		{
			Asura_PS3_FragmentShaderCopy* pxShader = NULL;
			Asura_Hash_ID uHashID = pxStrip->m_uFPHashID;
			Asura_PS3_FogParameters &xFogParams = Asura_PS3_Fog::GetFogParameters();
			if( Asura_MaterialResponse_System::GetCurrentOverrideMapHash() != ASURA_HASH_ID_UNSET )
			{
				uHashID = Asura_PS3_MaterialResponse_System::CreateShaderHashID( pxStrip->m_auMaterialResponseHashIDs[0], pxStrip->m_auMaterialResponseHashIDs[1] );
			}
			if( xFogParams.m_bFogEnabled )
			{
				uHashID += ASURA_PS3_SHADER_FOG;
			}
			pxShader = fpTree->GetShader( uHashID );
			if( pxShader == NULL )
			{
				Asura_Assert( false, "Unable to find the water shader" );
				continue;
			}

			pxShader->SetTexture( FPW_DepthTextureSampler, Asura_PS3_DeferredRendering::GetDeferredFrameBuffer()->GetZBufferAsTexture(), true );
			pxShader->SetTexture( FPW_BackBufferTextureSampler, Asura_PS3_DeferredRendering::GetResolvedTexture(), true );
			pxShader->SetTexture( FPW_TransparencyMapSampler, pxMaterial1->GetTexture( Asura_MaterialResponse::TRANSPARENCY_MAP ) );
			pxShader->SetTexture( FPW_NormalMapSampler, pxMaterial1->GetTexture( Asura_MaterialResponse::NORMAL_MAP ) );
			pxShader->SetTexture( FPW_ReflectionTextureSampler, pxMaterial1->GetTexture( Asura_MaterialResponse::ENVIRONMENT_MAP ) );
			if( pxMaterial2 )
			{
				pxShader->SetTexture( FPW_NormalMap2Sampler, pxMaterial2->GetTexture( Asura_MaterialResponse::NORMAL_MAP ) );
			}

			if( xFogParams.m_bFogEnabled )
			{
				const float afFogProperties [ 2 ] = { xFogParams.m_fFogNearPlane, xFogParams.m_fOneOverFogDepth };
				pxShader->SetTexture( FPW_FogTexture, xFogParams.m_pxFogTexture, true );

				pxShader->SetFloats( FPW_FogColour,				3, xFogParams.m_afFogColour );
				pxShader->SetFloats( FPW_FogProperties,			2, afFogProperties );
			}

			pxShader->SetFloats( FPW_OneOverScreenWidthHeight, 2, faOneOverScreenWidthHeight );
			pxShader->SetFloats( FPW_CameraPosition, 3, &pxCamPos->x );

			float f = Asura_Camera::GetFarClip(), n = Asura_Camera::GetNearClip();
			const float afDepthAdjustment[2] = { (f*n)/(f-n), f/(f-n) };
			pxShader->SetFloats( FPW_DepthAdjustment, 2, afDepthAdjustment );

			float afTemp[4];
			afTemp[ 0 ] = pxMaterial1->GetWaterFogStrength();
			afTemp[ 1 ] = 5.0f / pxMaterial1->GetWaterEdgeSoftness();

			afTemp[ 2 ] = pxMaterial1->GetWaterRippleScale();
			afTemp[ 3 ] = pxMaterial1->GetWaterDistortionScale();
			pxShader->SetFloats( FPW_WaterProperties, 4, afTemp );

			afTemp[0] = pxMaterial1->GetWaterReflectionColour().m_fR;
			afTemp[1] = pxMaterial1->GetWaterReflectionColour().m_fG;
			afTemp[2] = pxMaterial1->GetWaterReflectionColour().m_fB;
			afTemp[3] = pxMaterial1->GetWaterReflectionColour().m_fA;			
			pxShader->SetFloats( FPW_WaterReflectionColour, 4, afTemp );

			afTemp[0] = pxMaterial1->GetWaterRefractionColour().m_fR;
			afTemp[1] = pxMaterial1->GetWaterRefractionColour().m_fG;
			afTemp[2] = pxMaterial1->GetWaterRefractionColour().m_fB;
			afTemp[3] = pxMaterial1->GetWaterRefractionColour().m_fA;			
			pxShader->SetFloats( FPW_WaterRefractionColour, 4, afTemp );

			afTemp[0] = pxMaterial1->GetWaterFogColour().m_fR;
			afTemp[1] = pxMaterial1->GetWaterFogColour().m_fG;
			afTemp[2] = pxMaterial1->GetWaterFogColour().m_fB;
			afTemp[3] = pxMaterial1->GetWaterFogColour().m_fA;			
			pxShader->SetFloats( FPW_WaterFogColour, 4, afTemp );

			afTemp[0] = pxMaterial1->GetWaterUVControl(0) * Asura_Timers::GetLocalScaledTime();
			afTemp[1] = pxMaterial1->GetWaterUVControl(1) * Asura_Timers::GetLocalScaledTime();
			afTemp[2] = pxMaterial1->GetWaterUVControl(2);			
			pxShader->SetFloats( FPW_UVScroll, 3, afTemp );

			afTemp[0] = pxMaterial1->GetWaterUVControl(3) * Asura_Timers::GetLocalScaledTime();
			afTemp[1] = pxMaterial1->GetWaterUVControl(4) * Asura_Timers::GetLocalScaledTime();
			afTemp[2] = pxMaterial1->GetWaterUVControl(5);			
			pxShader->SetFloats( FPW_UVScroll2, 3, afTemp );

			afTemp[0] = pxMaterial1->GetWaterWaveControl(0);
			afTemp[1] = pxMaterial1->GetWaterWaveControl(1);
			afTemp[2] = pxMaterial1->GetWaterWaveControl(2) * Asura_Timers::GetLocalScaledTime();
			g_pxVertexShaderWater->SetUniformParameter( VPW_WavesX, afTemp );

			afTemp[0] = pxMaterial1->GetWaterWaveControl(3);
			afTemp[1] = pxMaterial1->GetWaterWaveControl(4);
			afTemp[2] = pxMaterial1->GetWaterWaveControl(5) * Asura_Timers::GetLocalScaledTime();
			g_pxVertexShaderWater->SetUniformParameter( VPW_WavesZ, afTemp );
			
			Asura_TextureAnimationOutput xTexAnim1, xTexAnim2;
			Asura_TextureAnimation_System::GetAnimationState( pxMaterial1->GetTextureAnim(), NULL, &xTexAnim1 );
			Asura_TextureAnimation_System::GetAnimationState( pxMaterial2 ? pxMaterial2->GetTextureAnim() : ASURA_HASH_ID_UNSET, NULL, &xTexAnim2 );
			const float afTexCoordOffset[4] = { xTexAnim1.m_fUOffset, xTexAnim1.m_fVOffset, xTexAnim2.m_fUOffset, xTexAnim2.m_fVOffset };
			g_pxVertexShader->SetUniformParameter( VPW_UV_Offsets, afTexCoordOffset );

			pxShader->Bind();
			if ( pxStrip->m_uVertexBufferId != uBoundId )
			{
				s_pxVertexBuffer[pxStrip->m_uVertexBufferId].Bind( g_axVertexDeclarationWater, VDW_ELEMENTS );
				uBoundId = pxStrip->m_uVertexBufferId;
			}
			s_pxBuffers[ iStripIdx ].m_xIndexBuffer.RenderTriangleStrip( pxStrip->m_uNumberOfIndices, 0 );
		}
	}

	Asura_PS3_RenderingStates::RequestTranslucencyMode( ASURA_TRANSLUCENCY_OFF );
	Asura_PS3_RenderingStates::RequestCullMode( ASURA_CULLMODE_CW );		
	Asura_PS3_RenderingStates::RequestZBufferWriteEnabled( true );
	Asura_PS3_RenderingStates::SetStencilTestEnabled( bStencilEnabled );

    ASURA_PS3_RSX_PROFILE__END( ASURA_PS3_RSX_PROFILE__ENV );
}


void Asura_PS3_EnvironmentRenderer::RenderGeometry(Asura_AMatrix4x4& xViewProjectionMatrix, Asura_Vector_3& xClipSphereCentre, float fClipSphereRadius, const Asura_OBB* pxClipOBB)
{
	Asura_PS3_RenderingStates::RequestZBufferWriteEnabled( true );
	Asura_PS3_RenderingStates::RequestZBufferEnabled( true );
	Asura_PS3_RenderingStates::RequestZBufferCompareMode(ASURA_ZBUFFERCMP_LESSEQUAL);		
	Asura_PS3_RenderingStates::RequestTranslucencyMode( ASURA_TRANSLUCENCY_OFF );
	Asura_PS3_RenderingStates::SetBlendEnableMrt( false, false, false );
	Asura_PS3_RenderingStates::SetAlphaBlending( false );
	Asura_PS3_RenderingStates::SetColorMaskMrtState( ASURA_PS3_MRT_MRT0_ALL );
//	Asura_PS3_RenderingStates::SetUseDepthBias( false );
	Asura_PS3_RenderingStates::RequestCullMode( ASURA_CULLMODE_CW );

	// render in 2 passes, first just for opaque stuff that we can draw with double-speed z-writes, then things with transparency masks
	for ( int iPass=0; iPass < 2; iPass++ )
	{
		bool bTranspPass = (iPass!=0);
		Asura_PS3_FragmentShaderCopy* pxFragmentShader = NULL;

		if ( bTranspPass )
		{
			// use a proper vertex shader, so we get UV's
			g_pxVertexShader = g_pxVertexShaderPL;

			pxFragmentShader = Asura_PS3_GCM_ShadersLibrary::GetPermanentFP( ASURA_SHADER_HASH_GLOBAL, ASURA_GLOBAL_SHADER_ALPHA_TEST );
			pxFragmentShader->Bind();
		}
		else
		{
			// double-speed z-writes
			g_pxVertexShader = g_pxVertexShaderShadowCubicMap;

			// set pixel shader, this doesn't have to do anything clever.
			pxFragmentShader = Asura_PS3_GCM_ShadersLibrary::GetPermanentFP( ASURA_SHADER_HASH_GLOBAL, ASURA_GLOBAL_SHADER_BLANK );
			pxFragmentShader->Bind();
		}

		g_pxVertexShader->SetUniformParameter( VIEW_PROJECTION_MATRIX_SHADOW, xViewProjectionMatrix.GetElements() );
		g_pxVertexShader->Bind();

		u_int uBoundId = 0xffffffff;

		for( u_int u = 0; u < s_uNumberOfStrips; ++u )
		{
			Asura_PS3_EnvironmentRenderer_Strip& xStrip = s_pxStrips[ u ];

			if ( !xStrip.m_xBoundingBox.IntersectsSphere(xClipSphereCentre, fClipSphereRadius) ) continue;		//BW this test is inaccurate.

			if ( pxClipOBB && !Asura_OBB(xStrip.m_xBoundingBox).Intersects(*pxClipOBB) ) continue;

	// doesn't work, points aren't enough need edges.
	//  		if( !IsAABBInClip( xViewProjectionMatrix, xStrip.m_xBoundingBox ) ) continue;

			const Asura_MaterialResponse* const pxMaterial1 = Asura_MaterialResponse_System::GetMaterialByHashID( xStrip.m_auMaterialResponseHashIDs[0] );
			if( pxMaterial1 )
			{
				if( pxMaterial1->GetDontCastShadow() ) continue;
				if( pxMaterial1->GetSpecialFXMode() == Asura_MaterialResponse::SPECIALFX_MODE_WATER ) continue;
				if( pxMaterial1->IsTransparent() ) continue; // never draw alpha-blended transparency, alpha-tested transparency is ok

				// test if this has a transparency map, and only draw it in the transparent pass (and only draw non-transparent things in the main pass)
				if ( ( pxMaterial1->GetTexture( Asura_MaterialResponse::TRANSPARENCY_MAP ) != ASURA_TEXTURE_NONE) // true = it has transparency
					!= bTranspPass ) //mmm... logic.
				{
					continue;
				}
			}
			
			if ( bTranspPass )
			{
				// use a proper vertex shader, so we get UV's
				g_pxVertexShader = g_pxVertexShaderPL;

				// texture animation for alpha-masked textures
				Asura_TextureAnimationOutput xTexAnim1, xTexAnim2;
				Asura_TextureAnimation_System::GetAnimationState( pxMaterial1->GetTextureAnim(), NULL, &xTexAnim1 );
				const float afTexCoordOffset[4] = { xTexAnim1.m_fUOffset, xTexAnim1.m_fVOffset, 0, 0 }; // we only draw the base texture in the shader
				g_pxVertexShader->SetUniformParameter( VP_UV_Offsets, afTexCoordOffset );

				// only use the transparency map
				pxFragmentShader->SetTexture( TU_MR_TransparencyMap, pxMaterial1->GetTexture( Asura_MaterialResponse::TRANSPARENCY_MAP ) );
			}

			if ( xStrip.m_uVertexBufferId != uBoundId )
			{
				s_pxVertexBuffer[xStrip.m_uVertexBufferId].Bind( g_axVertexBufferDeclarationEnv, bTranspPass?VD_ELEMENTS:1 );
				uBoundId = xStrip.m_uVertexBufferId;
			}
			s_pxBuffers[ u ].m_xIndexBuffer.RenderTriangleStrip( xStrip.m_uNumberOfIndices, 0 );
		} // for ( strips )
	} // for ( iPass )
}


void Asura_PS3_EnvironmentRenderer::RenderToShadow( float fNear, float fFar, bool bCubeMap )
{
	Asura_PS3_Graphics::ApplyCameraSettings();

	const Asura_Vector_3* pxCamPos = Asura_Camera::GetPosition();

	ASURA_PS3_RSX_PROFILE__BEGIN( ASURA_PS3_RSX_PROFILE__ENV, "Environment Render to Shadow" );

	const Asura_AMatrix4x4& xViewProjectionMatrix = Asura_PS3_Graphics::GetViewProjectionMatrix();

	g_pxVertexShader	= g_pxVertexShaderShadowCubicMap;
	float xShadowFrustum[ 2 ] = { fNear, 1.0f / ( fFar - fNear ) };

	g_pxVertexShader->SetUniformParameter( CAMERA_POSITION_SHADOW, &pxCamPos->x );
	g_pxVertexShader->SetUniformParameter( VIEW_PROJECTION_MATRIX_SHADOW, xViewProjectionMatrix.GetElements() );
	g_pxVertexShader->SetUniformParameter( SHADOW_FRUSTUM_NEAR_PLANE_SHADOW, xShadowFrustum );
	g_pxVertexShader->Bind();

	Asura_PS3_RenderingStates::RequestZBufferWriteEnabled( true );
	Asura_PS3_RenderingStates::RequestZBufferEnabled( true );
	Asura_PS3_RenderingStates::RequestZBufferCompareMode(ASURA_ZBUFFERCMP_LESSEQUAL);		
	Asura_PS3_RenderingStates::RequestTranslucencyMode( ASURA_TRANSLUCENCY_OFF );
	Asura_PS3_RenderingStates::SetBlendEnableMrt( false, false, false );
	Asura_PS3_RenderingStates::SetAlphaBlending( false );
	Asura_PS3_RenderingStates::SetColorMaskMrtState( ASURA_PS3_MRT_MRT0_ALL );
	Asura_PS3_RenderingStates::SetUseDepthBias( false );
	Asura_PS3_RenderingStates::RequestCullMode( ASURA_CULLMODE_CW );

	// make a rough bounding box around the light, for trivial rejection of env strips
	Asura_Bounding_Box xLightAABB;
	xLightAABB.SetToPoint(*Asura_Camera::GetPosition());
	xLightAABB.Grow( fFar );


	// render in 2 passes, first just for opaque stuff that we can draw with double-speed z-writes, then things with transparency masks
	for ( int iPass=0; iPass < 2; iPass++ )
	{
		bool bTranspPass = (iPass!=0);
		Asura_PS3_FragmentShaderCopy* pxFragmentShader = NULL;

		if ( bTranspPass )
		{
			// use a proper vertex shader, so we get UV's
			g_pxVertexShader = g_pxVertexShaderPL;

			pxFragmentShader = Asura_PS3_GCM_ShadersLibrary::GetPermanentFP( ASURA_SHADER_HASH_GLOBAL, ASURA_GLOBAL_SHADER_ALPHA_TEST );
			pxFragmentShader->Bind();
		}
		else
		{
			// double-speed z-writes
			g_pxVertexShader = g_pxVertexShaderShadowCubicMap;

			// set pixel shader, this doesn't have to do anything clever.
			pxFragmentShader = Asura_PS3_GCM_ShadersLibrary::GetPermanentFP( ASURA_SHADER_HASH_GLOBAL, ASURA_GLOBAL_SHADER_BLANK );
			pxFragmentShader->Bind();
		}

		g_pxVertexShader->SetUniformParameter( VIEW_PROJECTION_MATRIX_SHADOW, xViewProjectionMatrix.GetElements() );
		g_pxVertexShader->Bind();

		u_int uBoundId = 0xffffffff;

		for( u_int u = 0; u < s_uNumberOfStrips; ++u )
		{
			Asura_PS3_EnvironmentRenderer_Strip& xStrip = s_pxStrips[ u ];

			if (!xLightAABB.Intersects(xStrip.m_xBoundingBox)) continue;
			if( !Asura_Camera::IsAABBInFrustum( xStrip.m_xBoundingBox ) ) continue;


			const Asura_MaterialResponse* const pxMaterial1 = Asura_MaterialResponse_System::GetMaterialByHashID( xStrip.m_auMaterialResponseHashIDs[0] );
			if( pxMaterial1 )
			{
				if( pxMaterial1->GetDontCastShadow() ) continue;
				if( pxMaterial1->GetSpecialFXMode() == Asura_MaterialResponse::SPECIALFX_MODE_WATER ) continue;
				if( pxMaterial1->IsTransparent() ) continue; // never draw alpha-blended transparency, alpha-tested transparency is ok

				// test if this has a transparency map, and only draw it in the transparent pass (and only draw non-transparent things in the main pass)
				if ( ( pxMaterial1->GetTexture( Asura_MaterialResponse::TRANSPARENCY_MAP ) != ASURA_TEXTURE_NONE) // true = it has transparency
					!= bTranspPass ) //mmm... logic.
				{
					continue;
				}
			}
			
			if ( bTranspPass )
			{
				// use a proper vertex shader, so we get UV's
				g_pxVertexShader = g_pxVertexShaderPL;

				// texture animation for alpha-masked textures
				Asura_TextureAnimationOutput xTexAnim1, xTexAnim2;
				Asura_TextureAnimation_System::GetAnimationState( pxMaterial1->GetTextureAnim(), NULL, &xTexAnim1 );
				const float afTexCoordOffset[4] = { xTexAnim1.m_fUOffset, xTexAnim1.m_fVOffset, 0, 0 }; // we only draw the base texture in the shader
				g_pxVertexShader->SetUniformParameter( VP_UV_Offsets, afTexCoordOffset );

				// only use the transparency map
				pxFragmentShader->SetTexture( TU_MR_TransparencyMap, pxMaterial1->GetTexture( Asura_MaterialResponse::TRANSPARENCY_MAP ) );
			}

			if ( xStrip.m_uVertexBufferId != uBoundId )
			{
				s_pxVertexBuffer[xStrip.m_uVertexBufferId].Bind( g_axVertexBufferDeclarationEnv, bTranspPass?VD_ELEMENTS:1 );
				uBoundId = xStrip.m_uVertexBufferId;
			}
			s_pxBuffers[ u ].m_xIndexBuffer.RenderTriangleStrip( xStrip.m_uNumberOfIndices, 0 );
		} // for ( strips )
	} // for ( iPass )

	ASURA_PS3_RSX_PROFILE__END( ASURA_PS3_RSX_PROFILE__ENV );
}


u_int Asura_Environment_Renderer::Platform_GetNumberOfStrips( void )
{
	return Asura_PS3_EnvironmentRenderer::s_uNumberOfStrips;
}

void Asura_Environment_Renderer::Platform_PrefetchStrip(u_int uStrip)
{
	const Asura_PS3_EnvironmentRenderer_Strip&	xStrip	= Asura_PS3_EnvironmentRenderer::s_pxStrips[uStrip];

	Asura_Mem::PreFetchData(&xStrip, sizeof(xStrip));
}

const Asura_Bounding_Box& Asura_Environment_Renderer::Platform_GetBoundingBox( u_int uStrip )
{
	return Asura_PS3_EnvironmentRenderer::s_pxStrips[uStrip].m_xBoundingBox;
}

const Asura_MaterialResponse* Asura_Environment_Renderer::Platform_GetMaterial( u_int uStrip, u_int uPass )
{
	const Asura_Hash_ID uMaterialHash = Asura_PS3_EnvironmentRenderer::s_pxStrips[uStrip].m_auMaterialResponseHashIDs[ uPass ];
	const Asura_MaterialResponse* pxMaterial = Asura_MaterialResponse_System::GetMaterialByHashID( uMaterialHash );

	return pxMaterial;
}

u_int Asura_Environment_Renderer::Platform_GetModuleIndex( u_int uStrip )
{
	return Asura_PS3_EnvironmentRenderer::s_pxStrips[uStrip].m_iModuleIndex;
}


/// End of file. ///
