//
// Asura_PS3_Graphics.h
//
// Created by KJFL - 21 June 2005
// Modified Catalin Arsenescu - 27 March 2007
//
// Copyright (C) 2005 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __ASURA_PS3_GRAPHICS_H__
#define __ASURA_PS3_GRAPHICS_H__

#if _DEBUG
//#define ASURA_USE_GCM_DEBUG			1
#endif

#ifdef ASURA_USE_GCM_DEBUG
#include <stdio.h>
#define CELL_GCM_DEBUG
#endif

// Middleware
#ifdef ASURA_USE_GCM_REPLAY
#include "Sony/PS3/GcmReplay/gcmReplay.h"
#endif

#include <cell/gcm.h>

#include "Rendering/Asura_PS3_RenderingStates.h"

#if !defined(ASURA_PS3_INTERNAL_SCREEN_HEIGHT)
#define ASURA_PS3_INTERNAL_SCREEN_HEIGHT	720		// Default to 720p
#endif	//!defined(ASURA_PS3_INTERNAL_SCREEN_HEIGHT)

class Asura_AMatrix4x4;
class Asura_Mutex;

class Asura_PS3_Graphics
{

public:

    static void Initialise( const u_char ucHostMainMemorySizeInMB = 128, unsigned char ucCommandBufferSizeInMB = 2, unsigned char ucRenderTargets = 3 );
    static void Shutdown( void );
    static void Reset( void );

	static CellGcmContextData*	GetGcmCurrentContext(void) { return gCellGcmCurrentContext; }

    static void BeginFrame( void );
    static void EndFrame( void );

	static void ApplyCameraSettings( void );
	static void ClearZBuffer( void );

	static void Flush( void ) {}
	static bool IsInitialised( void ) { return s_bIsInitialised; }

    static float GetInternalScreenAspectRatio    (void)  { return s_fAspectRatio; }
    static float GetScreenAspectRatio( void ) { return s_fAspectRatio; }
    static u_int GetScreenWidth( void ) { return s_uScreenWidth; }
    static u_int GetScreenHeight( void ) { return s_uScreenHeight; }
    static u_int GetScreenFormat( void ) { return s_uScreenFormat; }

	static u_int GetOutputWidth( void ) { return s_uOutputWidth; }
	static u_int GetOutputHeight( void ) { return s_uOutputHeight; }

	static const Asura_AMatrix4x4& GetViewMatrix( void ) { return s_xViewMatrix; }
	static const Asura_AMatrix4x4& GetViewProjectionMatrix( void ) { return s_xViewProjectionMatrix; }
	static const Asura_AMatrix4x4& GetProjectionMatrix( void ) { return s_xProjectionMatrix; }
	static const Asura_AMatrix4x4& GetWorldViewProjectionMatrix( const Asura_AMatrix4x4* xObjectMatrix = NULL );

    static void SetViewportToDefaults( bool bOverrun = true );

    static void SetViewProjMatricesAndViewport( float fViewportOutputMinZ = fASURA_PS3_DEFAULT_VIEWPORT_MIN_Z, float fViewportOutputMaxZ = fASURA_PS3_DEFAULT_VIEWPORT_MAX_Z );

    static void LockMutex();
	static bool LockMutexTimeOut( float fTimeoutInSeconds );
    static void UnlockMutex();

	static void SetTranslucencyModeMRT( bool bTarget1, bool bTarget2, bool bTarget3 );

	static bool IsInSecondaryDrawThread( void ) { return s_bSecondaryDrawThread; }
	static void SetIsInSecondaryDrawThread( bool bInSecondaryDrawThread ) { s_bSecondaryDrawThread = bInSecondaryDrawThread; }

	static bool	IsInShutdown( void ) { return s_bInShutdown; }
	static void SetIsInShutdown( void ) { CheckExitCallBack(); s_bInShutdown = true; }

	static void CheckExitCallBack( void );
	
	static bool UseGCmMacros( void )
	{
#if defined( ASURA_DEBUG_VARIABLE_SYSTEM )
		return s_bUseGCmMacros;
#else
		return false;
#endif
	}

	static bool UseGammaCorrection( void )
	{
#if defined( ASURA_DEBUG_VARIABLE_SYSTEM )
		return s_bUseGammaCorrection;
#else
		return true;
#endif
	}
	
	static void BeginRenderToSkybox();
	static void EndRenderToSkybox();
	static bool IsRenderingToSkybox() { return s_RenderingToSkybox; }

	static void SwitchStencilCullMode	( /*ASURA_PS3_STENCIL_FUNCTION*/u_int eStencilFunction, u_int uiRef, u_int uiMask );
	static void ReloadStencilCull		( void );
	
	static float GetGammaSetting() { return s_fGamma; }
	static void SetGammaSetting(float fGamma);
	
private:

	//Private members variables
	static Asura_AMatrix4x4			s_xViewMatrix;
	static Asura_AMatrix4x4			s_xProjectionMatrix;
	static Asura_AMatrix4x4			s_xViewProjectionMatrix;
	static Asura_AMatrix4x4			s_xWorldViewProjectionMatrix;

	static u_int*					s_pxMainMemoryAddr;
    static unsigned int				s_uScreenWidth;
    static unsigned int				s_uScreenHeight;
    static unsigned int				s_uScreenFormat;
	static unsigned int				s_uiFlushReference;
	static float					s_fAspectRatio;

	static unsigned int				s_uOutputWidth;
	static unsigned int				s_uOutputHeight;

	static bool						s_bInShutdown;
	static bool						s_bIsInitialised;
	static bool						s_bSecondaryDrawThread;
#if defined( ASURA_DEBUG_VARIABLE_SYSTEM )
	static bool						s_bUseGCmMacros;
	static bool						s_bUseGammaCorrection;
#endif

	static float					s_fGamma;

	static Asura_Mutex				s_xMutex;
	
	static u_int s_uSCullModeHash;

	static bool s_RenderingToSkybox;

	//Private Constructor & Destructor
	Asura_PS3_Graphics( void );
	~Asura_PS3_Graphics( void );

	//Private functions
	static	void	SetGraphicMode( void ); //!< Get the best graphic mode based on the user selection
	static	void	SetRenderTarget( void );
	static	void	ResetEnvironment( void );

};

////////////////////////////////////////////////////////////////////////////////
//
// Stencil Masks
//
// Keep track of who is using which bits of the stencil buffer to ensure the play
// nicely with each other
//
////////////////////////////////////////////////////////////////////////////////

enum {
	ASURA_STENCIL_BIT_PHOTON = 0,
	ASURA_STENCIL_BIT_SKY,
	ASURA_STENCIL_BIT_FIRST_PROJECT_SPECIFIC
};

#define ASURA_STENCIL_MASK_PHOTON				(1<<ASURA_STENCIL_BIT_PHOTON) 
#define ASURA_STENCIL_MASK_SKY					(1<<ASURA_STENCIL_BIT_SKY)
#define ASURA_STENCIL_MASK_PROJECT_SPECIFIC		(0xFF -  ((1<<ASURA_STENCIL_BIT_FIRST_PROJECT_SPECIFIC) - 1))


// Labels for synchronizing with RSX. (cellGcmSetWaitLabelInline & cellGcmGetLabelAddress)
// range 64-255
enum ASURA_GCM_LABELS {
	ASURA_GCM_LABEL_SPUSKINNING_WAITFORSPU_0 = 64,
	ASURA_GCM_LABEL_SPUSKINNING_WAITFORSPU_1,
	ASURA_GCM_LABEL_SPUSKINNING_WAITFORRSX_0,
	ASURA_GCM_LABEL_SPUSKINNING_WAITFORRSX_1,
	
	ASURA_GCM_LABEL_RSX_PROFILING = 255
};


#endif//__ASURA_PS3_GRAPHICS_H__
