//
// Asura_PS3_Helper.cpp
//
// Created by Catalin Arsenescu - 20 November 2007
//
// Copyright (C) 2005 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

// This file's header.
#include "Rendering/Asura_PS3_Helper.h"

// Engine independent.
#include "Debugging/Asura_Assert.h"
#include "Debugging/Asura_Logfile.h"

// Engine platform-specific.
#include "Rendering/Asura_PS3_Graphics.h"
#include "Rendering/Asura_PS3_SwapChain.h"

u_int				Asura_PS3_Helper::s_uiRefVal				= 1;


////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_PS3_Helper::WaitForIdleState
// Author       : Catalin Arsenescu (20-November-2007)
////////////////////////////////////////////////////////////////////////////////
void Asura_PS3_Helper::WaitForIdleState( void )
{
	Asura_PS3_SwapChain::BlockUntilIdle();
}


void Asura_PS3_Helper::WriteBlockingWaitForBackEndCmds( CellGcmContextData* pxContext )
{
    // Since this is a blocking wait with a varying value there's no need
    // to change the counter index.
    static const u_int uCounterIndex = ASURA_GCM_LABEL_RSX_PROFILING;

    // (MP - 27.11.2007) TODO: Are there any guarantees about initial value
    // here? We don't want the first wait we ever do to immediately pass if
    // the counter happens to already be set to that value... Choose an
    // arbitrary value for now.
    static u_int uNextCounterValue = 0x12345678;

    cellGcmSetWriteBackEndLabelInline( pxContext, uCounterIndex, uNextCounterValue );
    cellGcmSetWaitLabelInline( pxContext, uCounterIndex, uNextCounterValue );

    uNextCounterValue++;
}


////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_PS3_Helper::GetPitchSize
// Author       : Catalin Arsenescu (20-November-2007)
////////////////////////////////////////////////////////////////////////////////
uint32 Asura_PS3_Helper::GetPitchSize( uint32 uiSize )
{
	uint32 uRet = cellGcmGetTiledPitchSize( uiSize ); // 32Bits color format
	if( uRet == 0 )
	{
		Asura_OutputToDebugger( ("cellGcmGetTiledPitchSize failed (game will crash)\n") );
	}

	return uRet;
}

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_PS3_Helper::AddressToOffset
// Author       : Catalin Arsenescu (20-November-2007)
////////////////////////////////////////////////////////////////////////////////
uint32 Asura_PS3_Helper::AddressToOffset( const void* pxAddress )
{
	uint32 uOffset = 0;

	int iRet = cellGcmAddressToOffset( pxAddress, &uOffset );
	Asura_Assert( iRet == CELL_OK, "cellGcmAddressToOffset failed (game will crash)\n" );

	return uOffset;
}
