
///////////////////////////////////////////////////////////////////////////////
//
// File		: Asura_PS3_LineRenderer.h
// Author	: Andy Tate ( 8-Sep-2005 )
// Modified	: Catalin Arsenescu ( 19-June-2007 )
//
// Copyright (C) 2005 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "Rendering/Asura_PS3_LineRenderer.h"

// Asura independent header-files...
#include "Camera/Asura_Camera.h"
#include "Core/Asura_TiledTextureManagement.h"
#include "Rendering/Asura_Colour.h"
#include "Rendering/Asura_Render.h"
#include "Rendering/Asura_Viewport.h"

// Asura platform header-files...
#include "Rendering/Shaders/Asura_PS3_GCM_Shader.h"
#include "Rendering/Asura_PS3_GCM_VertexBuffer.h"
#include "Rendering/Asura_PS3_Graphics.h"
#include "Rendering/Asura_PS3_FrameBuffer.h"
#include "Rendering/Asura_PS3_GCM_IndexBuffer.h"
#include "Rendering/Asura_PS3_RenderingOptions.h"
#include "Rendering/Asura_PS3_TextureManagement.h"
#include "Rendering/Asura_PS3_RenderingStates.h"

//Shaders definition
#include "Rendering/ShadersDefinition/VD/Lines_VD.h"
#include "Rendering/ShadersDefinition/VP/2DLines_VP.h"
#include "Rendering/ShadersDefinition/VP/3DLines_VP.h"
#include "Rendering/ShadersDefinition/FP/Lines_FP.h"

////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////

struct LineVertex
{
	float	x, y, z;
	u_int	xDiffuse;
};

////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////

// Vertex declarations...

#define ASURA_PS3_2D_LINES_CACHE	1500
#define ASURA_PS3_3D_LINES_CACHE	1500
#define ASURA_PS3_NUMBER_OF_BUFFERS	4

static	Asura_PS3_GCM_VertexShader*		g_pxVertexShader2D;
static	Asura_PS3_GCM_VertexShader*		g_pxVertexShader3D;
static	Asura_PS3_FragmentShader*		g_pxFragmentShader;
static	Asura_PS3_Lines_Handles*		g_pxHandles;
static	Asura_PS3_GCM_VertexBuffer		g_xVertexBuffer2D[ ASURA_PS3_NUMBER_OF_BUFFERS ];
static	Asura_PS3_GCM_VertexBuffer		g_xVertexBuffer3D[ ASURA_PS3_NUMBER_OF_BUFFERS ];
u_int	Asura_PS3_LineRenderer::s_u2DLinesToRender;
u_int	Asura_PS3_LineRenderer::s_u3DLinesToRender;
u_int	Asura_PS3_LineRenderer::s_u2DBuffer	= 0;
u_int	Asura_PS3_LineRenderer::s_u3DBuffer	= 0;


////////////////////////////////////////////////////////////////////////////////
// Private functions...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Function	: Asura_PS3_LineRenderer::Initialise
// Author	: Andy Tate (8-Sep-2005)
// Modified	: Catalin Arsenescu (19-June-2007)
//
// Description  : 
//
// Return value : None.
////////////////////////////////////////////////////////////////////////////////
void Asura_PS3_LineRenderer::Initialise (void)
{
	g_pxVertexShader2D	= new Asura_PS3_VP_2DLines();
	g_pxVertexShader3D	= new Asura_PS3_VP_3DLines();
	g_pxHandles			= new Asura_PS3_Lines_Handles();
	g_pxFragmentShader	= new Asura_PS3_FragmentShader( (unsigned char *)&_binary_PS3_Lines_FP_bin_start, (unsigned char *)&_binary_PS3_Lines_FP_bin_end, g_pxHandles );

	g_pxVertexShader3D->InitVertexBufferDeclaration( g_axVertexDeclaration, VD_ELEMENTS );

	for( int i = 0; i < ASURA_PS3_NUMBER_OF_BUFFERS; ++i )
	{
		g_xVertexBuffer2D[ i ].Initialise( LINE_VERTEX_SIZE, ASURA_PS3_2D_LINES_CACHE * 2, CELL_GCM_LOCATION_MAIN );
		g_xVertexBuffer3D[ i ].Initialise( LINE_VERTEX_SIZE, ASURA_PS3_3D_LINES_CACHE * 2, CELL_GCM_LOCATION_MAIN );
	}
}

/// End of function 'Asura_PS3_LineRenderer::Initialise'. ///

////////////////////////////////////////////////////////////////////////////////
// Function	: Asura_PS3_LineRenderer::Reset
// Author	: Andy Tate (8-Sep-2005)
// Modified	: Catalin Arsenescu (19-June-2007)
//
// Description  : 
//
// Return value : None.
////////////////////////////////////////////////////////////////////////////////
void Asura_PS3_LineRenderer::Reset(void)
{
}

/// End of function 'Asura_PS3_LineRenderer::Reset'. ///


////////////////////////////////////////////////////////////////////////////////
// Function	: Asura_PS3_LineRenderer::Shutdown
// Author	: Andy Tate (8-Sep-2005)
// Modified	: Catalin Arsenescu (19-June-2007)
//
// Description  : 
//
// Return value : None.
////////////////////////////////////////////////////////////////////////////////
void Asura_PS3_LineRenderer::Shutdown (void)
{
	delete g_pxVertexShader2D;
	delete g_pxVertexShader3D;
	delete g_pxFragmentShader;
	delete g_pxHandles;

	g_pxVertexShader2D	= NULL;
	g_pxVertexShader3D	= NULL;
	g_pxFragmentShader	= NULL;
	g_pxHandles			= NULL;
	for( int i = 0; i < ASURA_PS3_NUMBER_OF_BUFFERS; ++i )
	{
		g_xVertexBuffer2D[ i ].Shutdown();
		g_xVertexBuffer3D[ i ].Shutdown();
	}
}

/// End of function 'Asura_PS3_LineRenderer::Shutdown'. ///


////////////////////////////////////////////////////////////////////////////////
// Function	: Asura_PS3_LineRenderer::Render2DLine
// Author	: Andy Tate (8-Sep-2005)
// Modified	: Catalin Arsenescu (19-June-2007)
//
// Description  : 
//
// Return value : None.
////////////////////////////////////////////////////////////////////////////////
void Asura_PS3_LineRenderer::Render2DLine( const Asura_Vector_2& xP1, const Asura_Vector_2& xP2, const u_int uColour )
{
	if( s_u2DLinesToRender >= ASURA_PS3_2D_LINES_CACHE )
	{
		Flush2DLines();
	}

	LineVertex* pxVertices = static_cast<LineVertex*>( g_xVertexBuffer2D[ s_u2DBuffer ].LockForWriting() );

	u_int uPosInBuffer = s_u2DLinesToRender * 2;
	u_int xDiffuse =	( ( uColour & 0x00FF0000 ) <<  8 ) + 
						( ( uColour & 0x0000FF00 ) <<  8 ) +
						( ( uColour & 0x000000FF ) <<  8 ) + 
						( ( uColour & 0xFF000000 ) >> 24 );
	pxVertices[ uPosInBuffer ].x = ( ( xP1.x * 2.0f ) / Asura_Viewport::GetWidth() ) - 1.0f;
	pxVertices[ uPosInBuffer ].y = ( ( xP1.y * 2.0f ) / Asura_Viewport::GetHeight() ) - 1.0f;
	pxVertices[ uPosInBuffer ].z = 0;
	pxVertices[ uPosInBuffer ].xDiffuse	= xDiffuse;
	++uPosInBuffer;
	pxVertices[ uPosInBuffer ].x = ( ( xP2.x * 2.0f ) / Asura_Viewport::GetWidth() ) - 1.0f;
	pxVertices[ uPosInBuffer ].y = ( ( xP2.y * 2.0f ) / Asura_Viewport::GetHeight() ) - 1.0f;
	pxVertices[ uPosInBuffer ].z = 0;
	pxVertices[ uPosInBuffer ].xDiffuse	= xDiffuse;
	++s_u2DLinesToRender;
	return;
#if 0
	g_pxVertexShader2D->Bind();

	Asura_PS3_DeferredRendering::BindAsSingleRT();
	Asura_PS3_RenderingStates::RequestAlphaTest( false );

	Asura_PS3_RenderingStates::RequestTranslucencyMode( ASURA_TRANSLUCENCY_NORMAL );
	
	const float fAlpha = static_cast< float >( (uColour & 0xFF000000) >> 24 ) / 255.0f;
	const float fRed   = static_cast< float >( (uColour & 0x00FF0000) >> 16 ) / 255.0f;
	const float fGreen = static_cast< float >( (uColour & 0x0000FF00) >>  8 ) / 255.0f;
	const float fBlue  = static_cast< float >( (uColour & 0x000000FF)       ) / 255.0f;

	LineVertex* pxVertices = (LineVertex*) g_xVertexBuffer.LockForWriting();

	if( pxVertices )
	{
		g_pxFragmentShader->Bind();
		g_xVertexBuffer.Bind( g_axVertexDeclaration, VD_ELEMENTS );
		g_xVertexBuffer.RenderLinesInline( 2, pxVertices );
	}
#endif
}

/// End of function 'Asura_PS3_LineRenderer::Render2DLine'. ///

////////////////////////////////////////////////////////////////////////////////
// Function	: Asura_PS3_LineRenderer::Render3DLine
// Author	: Andy Tate (8-Sep-2005)
// Modified	: Catalin Arsenescu (19-June-2007)
//
// Description  : 
//
// Return value : None.
////////////////////////////////////////////////////////////////////////////////
void Asura_PS3_LineRenderer::Render3DLine( const Asura_Vector_3& xP1, const Asura_Vector_3& xP2, const u_int uColour )
{
	if( s_u3DLinesToRender >= ASURA_PS3_3D_LINES_CACHE )
	{
		Flush3DLines();
	}

	LineVertex* pxVertices = static_cast<LineVertex*>( g_xVertexBuffer3D[ s_u3DBuffer ].LockForWriting() );

	u_int uPosInBuffer = s_u3DLinesToRender * 2;
	u_int xDiffuse =	( ( uColour & 0x00FF0000 ) <<  8 ) + 
		( ( uColour & 0x0000FF00 ) <<  8 ) +
		( ( uColour & 0x000000FF ) <<  8 ) + 
		( ( uColour & 0xFF000000 ) >> 24 );
	pxVertices[ uPosInBuffer ].x = xP1.x;
	pxVertices[ uPosInBuffer ].y = xP1.y;
	pxVertices[ uPosInBuffer ].z = xP1.z;
	pxVertices[ uPosInBuffer ].xDiffuse	= xDiffuse;
	++uPosInBuffer;
	pxVertices[ uPosInBuffer ].x = xP2.x;
	pxVertices[ uPosInBuffer ].y = xP2.y;
	pxVertices[ uPosInBuffer ].z = xP2.z;
	pxVertices[ uPosInBuffer ].xDiffuse	= xDiffuse;
	++s_u3DLinesToRender;
	return;
#if 0
	g_pxVertexShader3D->Bind();

	const Asura_AMatrix4x4& xViewProjectionMatrix = Asura_PS3_Graphics::GetViewProjectionMatrix();
	g_pxVertexShader3D->SetUniformParameter( VP_ViewProjectionMatrix, xViewProjectionMatrix.GetElements() );

	Asura_PS3_DeferredRendering::BindAsSingleRT();
	Asura_PS3_RenderingStates::RequestAlphaTest( false );

	Asura_PS3_RenderingStates::RequestTranslucencyMode( ASURA_TRANSLUCENCY_NORMAL );

	const float fAlpha = static_cast< float >( (uColour & 0xFF000000) >> 24 ) / 255.0f;
	const float fRed   = static_cast< float >( (uColour & 0x00FF0000) >> 16 ) / 255.0f;
	const float fGreen = static_cast< float >( (uColour & 0x0000FF00) >>  8 ) / 255.0f;
	const float fBlue  = static_cast< float >( (uColour & 0x000000FF)       ) / 255.0f;

	LineVertex* pxVertices = (LineVertex*) g_xVertexBuffer.LockForWriting();

	if( pxVertices )
	{
		if( !s_bColorSet || uColour != s_uLastColour )
		{
			float afColour[4] = { fRed, fGreen, fBlue, fAlpha };
			g_pxFragmentShader = Asura_PS3_GCM_ShadersLibrary::GetPermanentFP( ASURA_SHADER_HASH_SPECIALFX, ASURA_SPECIALFX_LINES_HASH );
			g_pxFragmentShader->SetFloats( FP_Colour, 4, afColour );
			s_bColorSet = true;
			s_uLastColour = uColour;
		}

		pxVertices[0].x = xP1.x;
		pxVertices[0].y = xP1.y;
		pxVertices[0].z = xP1.z;

		pxVertices[1].x = xP2.x;
		pxVertices[1].y = xP2.y;
		pxVertices[1].z = xP2.z;

		g_pxFragmentShader->Bind();
		g_xVertexBuffer.Bind( g_axVertexDeclaration, VD_ELEMENTS );
		g_xVertexBuffer.RenderLinesInline( 2, pxVertices );
		Asura_PS3_Graphics::Flush();
	}
#endif
}

void Asura_PS3_LineRenderer::BeginFrame( void )
{
	s_u2DLinesToRender	= 0;
	s_u3DLinesToRender	= 0;
}

void Asura_PS3_LineRenderer::EndFrame( void )
{
	Flush2DLines();
	Flush3DLines();
}

void Asura_PS3_LineRenderer::Flush2DLines( void )
{
	if( s_u2DLinesToRender > 0 )
	{

		bool bOldSRGBWrite = Asura_PS3_RenderingStates::GetSRGBWriteEnabled();
		Asura_PS3_RenderingStates::SetSRGBWriteEnabled( false );
		g_pxVertexShader2D->Bind();

		Asura_PS3_DeferredRendering::BindAsSingleRT();
		Asura_PS3_RenderingStates::RequestAlphaTest( true );
		Asura_PS3_RenderingStates::RequestZBufferEnabled( false );

		Asura_PS3_RenderingStates::RequestTranslucencyMode( ASURA_TRANSLUCENCY_NORMAL );
		Asura_PS3_RenderingStates::SetLineWidth(1.0f);

		g_pxFragmentShader->Bind();
		g_xVertexBuffer2D[ s_u2DBuffer ].Bind( g_axVertexDeclaration, VD_ELEMENTS );
		g_xVertexBuffer2D[ s_u2DBuffer ].RenderLines( s_u2DLinesToRender * 2 );
		s_u2DLinesToRender = 0;
		++s_u2DBuffer;
		s_u2DBuffer %= ASURA_PS3_NUMBER_OF_BUFFERS;
		Asura_PS3_RenderingStates::SetSRGBWriteEnabled( bOldSRGBWrite );
	}
}

void Asura_PS3_LineRenderer::Flush3DLines( void )
{
	if( s_u3DLinesToRender > 0 )
	{
		g_pxVertexShader3D->Bind();

		bool bOldSRGBWrite = Asura_PS3_RenderingStates::GetSRGBWriteEnabled();
		Asura_PS3_RenderingStates::SetSRGBWriteEnabled( false );
		const Asura_AMatrix4x4& xViewProjectionMatrix = Asura_PS3_Graphics::GetViewProjectionMatrix();
		g_pxVertexShader3D->SetUniformParameter( VP_ViewProjectionMatrix, xViewProjectionMatrix.GetElements() );

		Asura_PS3_DeferredRendering::BindAsSingleRT();
		Asura_PS3_RenderingStates::RequestAlphaTest( true );
		Asura_PS3_RenderingStates::RequestZBufferEnabled( true );

		Asura_PS3_RenderingStates::RequestTranslucencyMode( ASURA_TRANSLUCENCY_NORMAL );
		Asura_PS3_RenderingStates::SetLineWidth(1.0f);

		g_pxFragmentShader->Bind();
		g_xVertexBuffer3D[ s_u3DBuffer ].Bind( g_axVertexDeclaration, VD_ELEMENTS );
		g_xVertexBuffer3D[ s_u3DBuffer ].RenderLines( s_u3DLinesToRender * 2 );
		s_u3DLinesToRender = 0;
		++s_u3DBuffer;
		s_u3DBuffer %= ASURA_PS3_NUMBER_OF_BUFFERS;
		Asura_PS3_RenderingStates::SetSRGBWriteEnabled( bOldSRGBWrite );
	}
}

/// End of function 'Asura_PS3_LineRenderer::Render3DLine'. ///

/// End of file. ///
