//////////////////////////////////////////////////////////////////////////////
//
// Asura_PS3_ObjectHierarchyRenderer.cpp
//
// Copyright (c) 2006 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header.
#include "Asura_PS3_ObjectHierarchyRenderer.h"

// Engine independent.
#include "Animation/Asura_Animation.h"
#include "Animation/Asura_Hierarchy.h"
#include "Animation/Asura_TextureAnimation.h"
#include "Camera/Asura_Camera.h"
#include "Core/Asura_MaterialResponse_System.h"
#include "Rendering/Asura_SortedRender.h"
#include "Rendering/Asura_DeferredShading.h"

// Engine platform-specific.
#include "ChunkSystem/Asura_Chunk_ResourceFile_PS3.h"
#include "Rendering/Shaders/Asura_PS3_GCM_Shader.h"
#include "Rendering/Asura_PS3_Fog.h"
#include "Rendering/Asura_PS3_FrameBuffer.h"
#include "Rendering/Asura_PS3_Graphics.h"
#include "Rendering/Asura_PS3_MaterialResponse_System.h"
#include "Rendering/Asura_PS3_RenderingStates.h"
#include "Rendering/Asura_PS3_RSXProfiling.h"
#include "Rendering/Asura_PS3_EdgeAntiAliasing.h"

//Shaders definition
#include "Rendering/ShadersDefinition/VD/Object_VD.h"
#include "Rendering/ShadersDefinition/VD/ObjectShadow_VD.h"
#include "Rendering/ShadersDefinition/VP/Object_VP.h"
#include "Rendering/ShadersDefinition/VP/ObjectShadow_VP.h"
#include "Rendering/ShadersDefinition/FP/Object_FP.h"
#include "Rendering/ShadersDefinition/FP/Object_Forward_FP.h"

////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////


// G L O B A L S /////////////////////////////////////////////////////////////
static	Asura_PS3_GCM_VertexShader*	g_pxVertexShader			= NULL;
static	Asura_PS3_GCM_VertexShader*	g_pxVertexShaderShadow		= NULL;
bool	Asura_PS3_ObjectHierarchyRenderer::s_bSetVBData			= false;

DVSVAR bool dvs_bDrawObjHier = true;

void Asura_PS3_ObjectHierarchyRenderer::Initialise (void)
{
	g_pxVertexShader			= new Asura_PS3_VP_ObjectH();
	g_pxVertexShaderShadow		= new Asura_PS3_VP_ObjectHShadow();

	g_pxVertexShader->InitVertexBufferDeclaration( g_axVertexDeclaration, VD_ELEMENTS );
	g_pxVertexShaderShadow->InitVertexBufferDeclaration( g_axVertexDeclarationShadow, VDS_ELEMENTS );
	
#if defined( ASURA_DEBUG_VARIABLE_SYSTEM)
	Asura_CommandConsole::AddVar( L"Render.Enable.ObjHier",	&dvs_bDrawObjHier,	Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
#endif
}

void Asura_PS3_ObjectHierarchyRenderer::BeginRender( const Asura_Animation& xAnim, const Asura_Hierarchy_Skin& xSkin )
{
	s_pxCurrentHierarchy = GetHierarchyTree().GetDataByHash( xSkin.m_uNameHashID );

	if( !s_pxCurrentHierarchy )
	{
		Asura_Assertf( false, ("Unable to find rendering data for hierarchy %s.", xSkin.Name.GetString() ));
		return;
	}

    ASURA_PS3_RSX_PROFILE__BEGIN( ASURA_PS3_RSX_PROFILE__OBJHIER, "Object Hierarchy" );

	s_bSetVBData = true;
}

void Asura_PS3_ObjectHierarchyRenderer::RenderStrip( const u_int uSectionIndex, const Asura_Vector_3& xPosition, const Asura_Matrix_3x3& xOrientation, const Asura_Animation& xAnim )
{
	if( s_pxCurrentHierarchy->m_uNumberOfStrips == 0 || !s_pxCurrentHierarchy->m_pxStrips ) return;

	Strip& xStrip = s_pxCurrentHierarchy->m_pxStrips[uSectionIndex];
	if( xStrip.m_uNumberOfTriangles == 0 ) return;

	const bool bTransparentPass = Asura_SortedRender::IsTransparentPass();

	const Asura_MaterialResponse* const pxMaterial1 = Asura_MaterialResponse_System::GetMaterialByHashID( xStrip.m_auMaterialResponseHashIDs[0] );
	const Asura_MaterialResponse* const pxMaterial2 = Asura_MaterialResponse_System::GetMaterialByHashID( xStrip.m_auMaterialResponseHashIDs[1] );
	const Asura_TextureAnimationController* pxTextureAnimationController = xAnim.GetTextureAnimController();

	if( pxMaterial1 )
	{
		const bool bTransparentStrip = pxMaterial1->IsTransparent();
		if( ( pxMaterial1->GetDontRender() ) || ( bTransparentStrip != bTransparentPass ) ) return;

		// Render the strip

		Asura_Hash_ID uHashID = xStrip.m_uFPHashID;
		if( Asura_MaterialResponse_System::GetCurrentOverrideMapHash() != ASURA_HASH_ID_UNSET )
		{
			uHashID = Asura_PS3_MaterialResponse_System::CreateShaderHashID( xStrip.m_auMaterialResponseHashIDs[0], xStrip.m_auMaterialResponseHashIDs[1] );
		}
		if( bTransparentPass && Asura_PS3_Fog::GetFogParameters().m_bFogEnabled )
		{
			uHashID += ASURA_PS3_SHADER_FOG;
		}

		Asura_PS3_FragmentShaderCopy* pxShader = Asura_PS3_GCM_ShadersLibrary::GetResettableFP( ASURA_SHADER_HASH_OBJECTS, uHashID );
		if( !pxShader )
		{
			Asura_Assert( false, "Error: No shader set for this strip" );
			return;
		}

		Asura_PS3_MaterialResponse_System::SetBlendStates( pxMaterial1 );

		//BW moved this from inside SetBlendStates, because character and environment renderers don't want this being changed
		if ( pxMaterial1->GetBlendMode() == Asura_MaterialResponse::BLEND_MODE_OPAQUE )
		{
			Asura_PS3_RenderingStates::RequestZBufferWriteEnabled( true );
		}
		else
		{
			Asura_PS3_RenderingStates::RequestZBufferWriteEnabled( false );
		}

		if( s_bSetVBData )
		{
			g_pxVertexShader->Bind();

			if( bTransparentPass )
			{
				Asura_PS3_DeferredRendering::BindAsSingleRT();
			}
			else
			{
				Asura_PS3_DeferredRendering::Bind();
			}

			// Set render states
			{
				Asura_PS3_RenderingStates::RequestCullMode( ASURA_CULLMODE_CW );
				Asura_PS3_RenderingStates::RequestZBufferCompareMode( ASURA_ZBUFFERCMP_LESSEQUAL );
				Asura_PS3_RenderingStates::RequestZBufferEnabled( true );
				if( bTransparentPass )
				{
					Asura_PS3_RenderingStates::RequestZBufferWriteEnabled( false );
					Asura_PS3_DeferredRendering::BindAsSingleRT();
					Asura_PS3_RenderingStates::RequestAlphaTest( false );
				}
				else
				{
					if ( !Asura_PS3_Graphics::IsRenderingToSkybox() )
					{
						Asura_PS3_RenderingStates::RequestZBufferWriteEnabled( true );
					}
					Asura_PS3_DeferredRendering::RestoreBind();
					Asura_PS3_RenderingStates::RequestAlphaTest( false );
				}
			}

			s_pxCurrentHierarchy->m_xVertexBuffer.Bind( g_axVertexDeclaration, VD_ELEMENTS );
			s_bSetVBData = false;
		}

		// Set world matrix
		Asura_AMatrix4x4 xObjectMatrix( xOrientation, xPosition );
		const Asura_AMatrix4x4& xWorldProjectionView = Asura_PS3_Graphics::GetWorldViewProjectionMatrix( &xObjectMatrix );

		g_pxVertexShader->SetUniformParameter( VS_WorldViewProjectionMatrix, xWorldProjectionView.GetElements() );
		g_pxVertexShader->SetUniformParameter( VS_WorldMatrix, xObjectMatrix.GetElements() );
		Asura_TextureAnimationOutput xTexAnim1, xTexAnim2;

		Asura_TextureAnimation_System::GetAnimationState( pxMaterial1->GetTextureAnim(), pxTextureAnimationController, &xTexAnim1 );
		Asura_TextureAnimation_System::GetAnimationState( pxMaterial2 ? pxMaterial2->GetTextureAnim() : ASURA_HASH_ID_UNSET, pxTextureAnimationController, &xTexAnim2 );

		const float afTexCoordOffset[4] = { xTexAnim1.m_fUOffset, xTexAnim1.m_fVOffset, xTexAnim2.m_fUOffset, xTexAnim2.m_fVOffset };
		g_pxVertexShader->SetUniformParameter( VS_UV_Offsets, afTexCoordOffset );

		const Asura_Vector_3* const pxCamPos = Asura_Camera::GetPosition();
		float afCamPos[4] = { pxCamPos->x, pxCamPos->y, pxCamPos->z, 1.0f };

		g_pxVertexShader->SetUniformParameter( VS_CameraPosition, afCamPos );

		Asura_PS3_MaterialResponse_System::SetEffectParameters( pxMaterial1, pxMaterial2, &xAnim, pxShader );
		Asura_PS3_MaterialResponse_System::SetProjectContext( pxMaterial1, pxMaterial2 );
		Asura_PS3_MaterialResponse_System::SetSHParameters( pxMaterial1, pxShader );
		pxShader->Bind();
		s_pxCurrentHierarchy->m_xIndexBuffer.RenderTriangleStrip( xStrip.m_uNumberOfIndices, xStrip.m_uStartIndex );
		Asura_PS3_Graphics::Flush();
	}

}

void Asura_PS3_ObjectHierarchyRenderer::EndRender()
{
	if( !s_bSetVBData )
	{
		Asura_PS3_RenderingStates::SetUseDepthBias( false );
		Asura_PS3_RenderingStates::SetBlendEnableMrt( false, false, false );
		Asura_PS3_RenderingStates::SetAlphaBlending( false );
	}

    ASURA_PS3_RSX_PROFILE__END( ASURA_PS3_RSX_PROFILE__OBJHIER );
}

void Asura_PS3_ObjectHierarchyRenderer::RenderShadow (const Asura_Animation& xAnim, const Asura_Hierarchy_Skin& xSkin, const Asura_Shadow& xShadow)
{
	BeginShadowRender( xSkin, xShadow );

	if( s_pxCurrentHierarchy == NULL ) return;
	const Hierarchy* const pxHierarchy = s_pxCurrentHierarchy;

	for( u_int u = 0; u < pxHierarchy->m_uNumberOfStrips; ++u )
	{
		const u_int uBoneIndex = pxHierarchy->m_pxStrips[u].m_uBoneIndex;

		const Asura_Vector_3& xBonePos = Asura_Hierarchy_System::GetWorldspaceBonePos( uBoneIndex );
		const Asura_Matrix_3x3& xBoneOri = Asura_Hierarchy_System::GetWorldspaceBoneOri( uBoneIndex );

		RenderShadowStrip( u, xBonePos, xBoneOri, xAnim, xShadow );
	}

	EndShadowRender();
}

void Asura_PS3_ObjectHierarchyRenderer::BeginShadowRender( const Asura_Hierarchy_Skin& xSkin, const Asura_Shadow& xShadow )
{
	s_pxCurrentHierarchy = s_xHierarchyTree.GetDataByHash( xSkin.m_uNameHashID );
	Asura_Assertf( s_pxCurrentHierarchy, ("Failed to get hierarchy data for %s.", xSkin.Name.GetString()) );
	if( s_pxCurrentHierarchy == NULL ) return;

	Asura_PS3_RenderingStates::RequestZBufferWriteEnabled( true );
	Asura_PS3_RenderingStates::RequestZBufferEnabled( true );
	Asura_PS3_RenderingStates::RequestTranslucencyMode( ASURA_TRANSLUCENCY_OFF );
	Asura_PS3_RenderingStates::SetBlendEnableMrt( false, false, false );
	Asura_PS3_RenderingStates::SetAlphaBlending( false );
	Asura_PS3_RenderingStates::SetColorMaskMrtState( ASURA_PS3_MRT_MRT0_ALL );
	Asura_PS3_RenderingStates::SetUseDepthBias( false );
	Asura_PS3_RenderingStates::RequestZBufferCompareMode( ASURA_ZBUFFERCMP_LESSEQUAL );
	Asura_PS3_RenderingStates::RequestCullMode( ASURA_CULLMODE_CW );

	g_pxVertexShaderShadow->Bind();
	s_pxCurrentHierarchy->m_xVertexBuffer.Bind( g_axVertexDeclarationShadow, VDS_ELEMENTS );
}

void Asura_PS3_ObjectHierarchyRenderer::RenderShadowStrip( const u_int uStripIndex, const Asura_Vector_3& xPosition, const Asura_Matrix_3x3& xOrientation, const Asura_Animation& xAnim, const Asura_Shadow& xShadow )
{
	if( s_pxCurrentHierarchy == NULL ) return;
	if( s_pxCurrentHierarchy->m_uNumberOfStrips == 0 || !s_pxCurrentHierarchy->m_pxStrips ) return;

	const Strip& xStrip = s_pxCurrentHierarchy->m_pxStrips[uStripIndex];
	if( xStrip.m_uNumberOfTriangles == 0 ) return;

	const Asura_Hash_ID uHash = xStrip.m_auMaterialResponseHashIDs[0];
	const Asura_MaterialResponse* const pxMaterial = Asura_MaterialResponse_System::GetMaterialByHashID( uHash );

	if( pxMaterial && (pxMaterial->GetRenderType() ==  Asura_MaterialResponse::RENDER_TYPE_RENDER_BUT_DONT_CAST_SHADOW) ) return;

	Asura_AMatrix4x4 xWorldProjectionView;
	Asura_AMatrix4x4 xObjectMatrix( xOrientation, xPosition );

	Asura_Matrix_3x3 xLightOri;
	xShadow.GetLightOrientation(xLightOri);
	const Asura_Vector_3& xLightPos = xShadow.GetLightPosition();

	Asura_AMatrix4x4 xPerspective;
	Asura_AMatrix4x4 xLight( xLightOri, xLightPos );
	xLight.InvertOrthogonal();

	xPerspective.SetPerspectiveForLH( Asura_Maths::DegToRad( xShadow.GetFrustum().GetVerticalFOVInDegrees() ), 1.0f, xShadow.GetFrustum().GetNearPlane(), xShadow.GetFrustum().GetFarPlane() );

	xWorldProjectionView = xObjectMatrix * xLight * xPerspective;

	g_pxVertexShaderShadow->SetUniformParameter( VSS_WorldViewProjectionMatrix, xWorldProjectionView.GetElements() );

	s_pxCurrentHierarchy->m_xIndexBuffer.RenderTriangleStrip( xStrip.m_uNumberOfIndices, xStrip.m_uStartIndex );
	Asura_PS3_Graphics::Flush();
}

void Asura_PS3_ObjectHierarchyRenderer::EndShadowRender()
{
}

int Asura_PS3_ObjectHierarchyRenderer::GetVertexSize( void )
{
	return OBJECT_HIERARCHY_VERTEX_SIZE;
}

////////////////////////////////////////////////////////////////////////////////
//
// Asura_PS3_ObjectHierarchyRenderer::IsHierarchyOpaque
//
// Queries whether a hierarchy is entirely opaque
//
// Parameters:		uHash - Hash ID of hierarchy name
// Return value:	true if all strips are opaque
//
////////////////////////////////////////////////////////////////////////////////

bool Asura_PS3_ObjectHierarchyRenderer::IsHierarchyOpaque( const Asura_Hash_ID uHash )
{
	const Hierarchy* const pxHierarchy = s_xHierarchyTree.GetDataByHash( uHash );
	Asura_Assert( pxHierarchy, "Failed to get object hierarchy data." );

	if( !pxHierarchy ) return false;

	for( u_int u = 0; u < pxHierarchy->GetNumberOfStrips(); ++u )
	{
		const Asura_Hash_ID uMaterialHash = pxHierarchy->GetStrip(u).m_auMaterialResponseHashIDs[0];
		const Asura_MaterialResponse* const pxMaterial = Asura_MaterialResponse_System::GetMaterialByHashID( uMaterialHash );

		// Decal-type materials count as opaque
		if( pxMaterial && pxMaterial->IsTransparent() ) 
		{
			return false;
		}
	}

	return true;
}

/******************************************************************************/
/* Asura_PS3_ObjectHierarchyRenderer::Hierarchy Class Functions           */
/******************************************************************************/

Asura_PS3_ObjectHierarchyRenderer::Hierarchy::Hierarchy()
	: m_pxAntiAliasEdges( NULL )
	, m_uNumAABuffers(0)
	, m_uNumberOfStrips(0)
	, m_pxStrips( NULL )
{
}

Asura_PS3_ObjectHierarchyRenderer::Hierarchy::~Hierarchy()
{
	m_xVertexBuffer.Destroy();
	m_xIndexBuffer.Shutdown();
 	for ( u_int uBone=0; uBone < m_uNumAABuffers; uBone++ )
 	{
 		m_pxAntiAliasEdges[uBone].Shutdown();
 	}
	delete[] m_pxAntiAliasEdges;

	delete[] m_pxStrips;
}

/******************************************************************************/
/* Asura_PS3_ObjectHierarchyRenderer Static Class Data                    */
/******************************************************************************/

Asura_TemplateHashTree< Asura_PS3_ObjectHierarchyRenderer::Hierarchy > Asura_PS3_ObjectHierarchyRenderer::s_xHierarchyTree;
const Asura_PS3_ObjectHierarchyRenderer::Hierarchy* Asura_PS3_ObjectHierarchyRenderer::s_pxCurrentHierarchy = NULL;

/******************************************************************************/
/* Asura_PS3_ObjectHierarchyRenderer Static Class Functions               */
/******************************************************************************/

////////////////////////////////////////////////////////////////////////////////
//
// Asura_PS3_ObjectHierarchyRenderer::Reset()
//
// Resets the PS3 object hierarchy renderer
//
// Parameters:		None
// Return value:	None
//
////////////////////////////////////////////////////////////////////////////////
void Asura_PS3_ObjectHierarchyRenderer::Reset (void)
{
	s_xHierarchyTree.Reset();
}

////////////////////////////////////////////////////////////////////////////////
//
// Asura_PS3_ObjectHierarchyRenderer::Shutdown()
//
// Shuts the PS3 object hierarchy renderer down
//
// Parameters:		None
// Return value:	None
//
////////////////////////////////////////////////////////////////////////////////
void Asura_PS3_ObjectHierarchyRenderer::Shutdown (void)
{
    Reset();
	delete g_pxVertexShader;
	delete g_pxVertexShaderShadow;

	g_pxVertexShader			= NULL;
	g_pxVertexShaderShadow		= NULL;
}

////////////////////////////////////////////////////////////////////////////////
//
// Asura_PS3_ObjectHierarchyRenderer::CreateHierarchy()
//
// Creates a new PS3 object hierarchy
//
// Parameters:		uHashID - Hash ID of hierarchy's name
// Return value:	Pointer to the new object, or NULL if unable to allocate
//
////////////////////////////////////////////////////////////////////////////////
Asura_PS3_ObjectHierarchyRenderer::Hierarchy* Asura_PS3_ObjectHierarchyRenderer::CreateHierarchy( const Asura_Hash_ID uHashID )
{
	Hierarchy* const pxNewHierarchy = s_xHierarchyTree.AddNode( uHashID );

	return pxNewHierarchy;
}


////////////////////////////////////////////////////////////////////////////////
//
// Asura_PS3_ObjectHierarchyRenderer::RenderHierarchy()
//
// Renders a complete object hierarchy on PS3
//
// Parameters:		xAnim - Reference to animation instance to render
//					xSkin - Reference to hierarchy skin
//
// Return value:	None
//
////////////////////////////////////////////////////////////////////////////////
void Asura_PS3_ObjectHierarchyRenderer::RenderHierarchy( const Asura_Animation& xAnim, const Asura_Hierarchy_Skin& xSkin )
{
#if 0 //def ASURA_USE_ZPREPASS
	if( !Asura_Render::IsZPrepass() ) return;
#endif
	
	if ( !dvs_bDrawObjHier ) return;

	BeginRender( xAnim, xSkin );

	const Hierarchy* const pxHierarchy = s_pxCurrentHierarchy;
    if( !pxHierarchy ) { return; }

	for( u_int u = 0; u < pxHierarchy->m_uNumberOfStrips; ++u )
	{
		const u_int				uBoneIndex	= pxHierarchy->m_pxStrips[u].m_uBoneIndex;
		const Asura_Vector_3&	xBonePos	= Asura_Hierarchy_System::GetWorldspaceBonePos( uBoneIndex );
		const Asura_Matrix_3x3&	xBoneOri	= Asura_Hierarchy_System::GetWorldspaceBoneOri( uBoneIndex );

		RenderStrip( u, xBonePos, xBoneOri, xAnim );
	}

	if ( !Asura_SortedRender::IsTransparentPass() && !Asura_PS3_Graphics::IsRenderingToSkybox() && s_pxCurrentHierarchy->m_pxAntiAliasEdges != NULL )
	{
		for ( u_int uBoneIndex = 0; uBoneIndex < (u_int)xSkin.NumberOfBones; uBoneIndex++ )
		{
			Asura_AMatrix4x4 xObjectMatrix( Asura_Hierarchy_System::GetWorldspaceBoneOri( uBoneIndex ), Asura_Hierarchy_System::GetWorldspaceBonePos( uBoneIndex ) );
			const Asura_AMatrix4x4& xWorldProjectionView = Asura_PS3_Graphics::GetWorldViewProjectionMatrix( &xObjectMatrix );

#ifdef ASURA_EDGE_ANTIALIASING
			if ( uBoneIndex < s_pxCurrentHierarchy->m_uNumAABuffers && s_pxCurrentHierarchy->m_pxAntiAliasEdges[uBoneIndex].IsInitialized() )
			{
				Asura_PS3_EdgeAntiAliasing::AddEdgeList( &s_pxCurrentHierarchy->m_pxAntiAliasEdges[uBoneIndex], &s_pxCurrentHierarchy->m_xVertexBuffer, &xWorldProjectionView );
			}
#endif
		}

	}
	EndRender();
}
