
///////////////////////////////////////////////////////////////////////////////
//
// File		: Asura_Platform_RenderStates.cpp
// Author	: Andy Tate (1-Jul-2005)
// Modified	: Catalin Arsenescu (27-March-2007)
//
// Copyright (C) 2005 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "Rendering/Asura_RenderStates.h"

// Engine independent.
#include "Debugging/Asura_Logfile.h"

// Engine platform-specific.



////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////

// These functions are not implemented, all rendering code on PS3 should use Asura_PS3_RenderStates
// Assert if something tries to set these states to anything other than the defaults in Asura_Render_States::ResetAll()
#ifdef ASURA_ASSERT
const bool g_bAssertsEnabled = true;
#endif

////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Private functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Protected functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Public functions...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Function		: Asura_Render_States::SetTextureIndex
// Author		: Andy Tate (1-Jul-2005)
// Modified		: Catalin Arsenescu (27-March-2007)
//
// Description  : Sets the specified texture on the specified channel.
//
// Parameters   : The channel to set the texture on AND the texture to be set.
//
// Return value : None.
////////////////////////////////////////////////////////////////////////////////

void Asura_Render_States::SetTextureIndex (int iTextureChannel, int iTextureIndex)
{
	Asura_Assert(!g_bAssertsEnabled || iTextureIndex == ASURA_TEXTURE_NONE,"Asura_Render_States::SetTextureIndex - removed function called");
#if 0
	Asura_PS3_Texture* pxTexture = Asura_PS3_TextureManagement::GetTexture( iTextureIndex );
	pxTexture->SetAsTextureIndex( iTextureChannel );
	// Finally, store the new texture index...
	TextureIndex [iTextureChannel]	= iTextureIndex;
#endif
}
/// End of function 'Asura_Render_States::SetTextureIndex'. ///

////////////////////////////////////////////////////////////////////////////////
// Function		: Asura_Render_States::SetWireFrameMode
// Author		: Andy Tate (1-Jul-2005)
// Modified		: Catalin Arsenescu (27-March-2007)
//
// Description  : 
//
// Parameters   : "true" to set the mode, "false" to unset it.
//
// Return value : None.
////////////////////////////////////////////////////////////////////////////////
void Asura_Render_States::SetWireFrameMode (bool bSet)
{
	Asura_Assert(!g_bAssertsEnabled || !bSet,"Asura_Render_States::SetWireFrameMode - removed function called");
#if 0
	// Finally, set the current mode...

	WireFrameMode	= bSet;
	if( bSet )
	{
		cellGcmSetFrontPolygonModeInline	( Asura_PS3_Graphics::GetGcmCurrentContext(), CELL_GCM_POLYGON_MODE_LINE );
		cellGcmSetBackPolygonModeInline	( Asura_PS3_Graphics::GetGcmCurrentContext(), CELL_GCM_POLYGON_MODE_LINE );
	}
	else
	{
		cellGcmSetFrontPolygonModeInline	( Asura_PS3_Graphics::GetGcmCurrentContext(), CELL_GCM_POLYGON_MODE_FILL );
		cellGcmSetBackPolygonModeInline	( Asura_PS3_Graphics::GetGcmCurrentContext(), CELL_GCM_POLYGON_MODE_FILL );
	}
#endif
}
/// End of function 'Asura_Render_States::SetWireFrameMode'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Render_States::SetDitheringMode
// Author		: Andy Tate (1-Jul-2005)
// Modified		: Catalin Arsenescu (27-March-2007)
//
// Description  : 
//
// Parameters   : "true" to set the mode, "false" to unset it.
//
// Return value : None.
////////////////////////////////////////////////////////////////////////////////
void Asura_Render_States::SetDitheringMode (bool bSet)
{
	Asura_Assert(!g_bAssertsEnabled || bSet,"Asura_Render_States::SetDitheringMode - removed function called");
#if 0
	// Are we to enable dithering?
#ifndef ASURA_PS3_SONY_SUBMISSION
	static bool bDisplayWarning = true;
#endif

	if (bSet)
	{
#ifndef ASURA_PS3_SONY_SUBMISSION
		if( bDisplayWarning )
		{
			Asura_OutputToDebugger ( ("Dithering mode supported by the hardware only in: CELL_GCM_SURFACE_X1R5G5B5_Z1R5G5B5, CELL_GCM_SURFACE_X1R5G5B5_O1R5G5B5, CELL_GCM_SURFACE_R5G6B5\n") );
			bDisplayWarning = false;
		}
#endif
		//cellGcmSetDitherEnable	( Asura_PS3_Graphics::GetGcmCurrentContext(), CELL_GCM_TRUE );
	}
	else
	{
#ifndef ASURA_PS3_SONY_SUBMISSION
		bDisplayWarning = true;
#endif
		cellGcmSetDitherEnableInline	( Asura_PS3_Graphics::GetGcmCurrentContext(), CELL_GCM_FALSE );
	}

	// Finally, set the current mode...

	DitheringMode	= bSet;
#endif
}
/// End of function 'Asura_Render_States::SetDitheringMode'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Render_States::SetZBufferEnabled
// Author		: Andy Tate (1-Jul-2005)
// Modified		: Catalin Arsenescu (27-March-2007)
//
// Description  : 
//
// Parameters   : "true" to set the mode, "false" to unset it.
//
// Return value : None.
////////////////////////////////////////////////////////////////////////////////
void Asura_Render_States::SetZBufferEnabled (bool bSet)
{
	Asura_Assert(!g_bAssertsEnabled || bSet,"Asura_Render_States::SetZBufferEnabled - removed function called");
#if 0
	// Do we need to enable the Z-buffer test?

	if (bSet)
	{
		cellGcmSetDepthTestEnableInline ( Asura_PS3_Graphics::GetGcmCurrentContext(), CELL_GCM_TRUE );
	}
	else													  
	{
		cellGcmSetDepthTestEnableInline ( Asura_PS3_Graphics::GetGcmCurrentContext(), CELL_GCM_FALSE );
	}

	// Finally, set the current mode...
	ZBufferEnabled	= bSet;
#endif
}
/// End of function 'Asura_Render_States::SetZBufferEnabled'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Render_States::SetZBufferCompareMode
// Author		: Andy Tate (1-Jul-2005)
// Modified		: Catalin Arsenescu (27-March-2007)
//
// Description  : 
//
// Parameters   : 
//
// Return value : None.
////////////////////////////////////////////////////////////////////////////////
void Asura_Render_States::SetZBufferCompareMode (enum ASURA_ZBUFFERCMP_ID mode)
{
	Asura_Assert(!g_bAssertsEnabled || mode == ASURA_ZBUFFERCMP_LESSEQUAL,"Asura_Render_States::SetZBufferCompareMode - removed function called");
#if 0

	// Which Z-buffer mode do we wish to set?

	switch (mode)
	{
	case ASURA_ZBUFFERCMP_LESSEQUAL:
		{
			cellGcmSetDepthFuncInline ( Asura_PS3_Graphics::GetGcmCurrentContext(), CELL_GCM_LEQUAL );
			break;		
		}
	case ASURA_ZBUFFERCMP_ALWAYS:
		{
			cellGcmSetDepthFuncInline ( Asura_PS3_Graphics::GetGcmCurrentContext(), CELL_GCM_ALWAYS );
			break;		
		}
	case ASURA_ZBUFFERCMP_EQUALS:
		{
			cellGcmSetDepthFuncInline ( Asura_PS3_Graphics::GetGcmCurrentContext(), CELL_GCM_EQUAL );
			break;
		}
	case ASURA_ZBUFFERCMP_GREATER:
		{
			cellGcmSetDepthFuncInline ( Asura_PS3_Graphics::GetGcmCurrentContext(), CELL_GCM_GREATER );
			break;		
		}
	}		// End of switch.

	// Finally, set the current mode...
	ZBufferCompareMode = mode;
#endif
}
/// End of function 'Asura_Render_States::SetZBufferCompareMode'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Render_States::SetZBufferWriteEnabled
// Author		: Andy Tate (1-Jul-2005)
// Modified		: Catalin Arsenescu (27-March-2007)
//
// Description  : 
//
// Parameters   : "true" to set the mode, "false" to unset it.
//
// Return value : None.
////////////////////////////////////////////////////////////////////////////////
void Asura_Render_States::SetZBufferWriteEnabled (bool bSet)
{
	Asura_Assert(!g_bAssertsEnabled || bSet,"Asura_Render_States::SetZBufferCompareMode - removed function called");
#if 0
	// Do we wish to enable Z-buffer writes?

	if( bSet )
	{
		cellGcmSetDepthMaskInline ( Asura_PS3_Graphics::GetGcmCurrentContext(), CELL_GCM_TRUE );
	}
	else
	{
		cellGcmSetDepthMaskInline ( Asura_PS3_Graphics::GetGcmCurrentContext(), CELL_GCM_FALSE );
	}


	// Finally, set the current mode...
	ZBufferWriteEnabled	= bSet;
#endif
}
/// End of function 'Asura_Render_States::SetZBufferWriteEnabled'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Render_States::SetAlphaBlending
// Author		: Andy Tate (1-Jul-2005)
// Modified		: Catalin Arsenescu (27-March-2007)
//
// Description  : 
//
// Parameters   : "true" to set the mode, "false" to unset it.
//
// Return value : None.
////////////////////////////////////////////////////////////////////////////////
void Asura_Render_States::SetAlphaBlending (bool bSet)
{
	Asura_Assert(!g_bAssertsEnabled || !bSet,"Asura_Render_States::SetAlphaBlending - removed function called");
#if 0
	// Do we wish to enable alpha-blending?

	if (bSet)
	{
		cellGcmSetBlendEnableInline ( Asura_PS3_Graphics::GetGcmCurrentContext(), CELL_GCM_TRUE );
	}
	else
	{
		cellGcmSetBlendEnableInline ( Asura_PS3_Graphics::GetGcmCurrentContext(), CELL_GCM_FALSE );
	}

	// Finally, set the current mode...
	AlphaBlendingEnabled	= bSet;
#endif
}
/// End of function 'Asura_Render_States::SetAlphaBlending'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Render_States::SetTranslucencyMode
// Author		: Andy Tate (1-Jul-2005)
// Modified		: Catalin Arsenescu (27-March-2007)
//
// Description  : 
//
// Parameters   : The translucency mode to be set.
//
// Return value : None.
////////////////////////////////////////////////////////////////////////////////
void Asura_Render_States::SetTranslucencyMode (enum ASURA_TRANSLUCENCY_MODE mode)
{
	Asura_Assert(!g_bAssertsEnabled || mode == ASURA_TRANSLUCENCY_OFF,"Asura_Render_States::SetTranslucencyMode - removed function called");
#if 0
	const bool	 bAlphaBlending	= (ASURA_TRANSLUCENCY_OFF != mode);


	// Do we actually need to set alpha-blending?
	if (bAlphaBlending != AlphaBlendingEnabled)
	{
		SetAlphaBlending	(bAlphaBlending);
	}

	// Which translucency mode do we wish to set?
	switch (mode)
	{
 	case ASURA_TRANSLUCENCY_OFF:
		{
			break;
		}
 	case ASURA_TRANSLUCENCY_NORMAL:
		{
			cellGcmSetBlendFuncInline( Asura_PS3_Graphics::GetGcmCurrentContext(), CELL_GCM_SRC_ALPHA, CELL_GCM_ONE_MINUS_SRC_ALPHA, CELL_GCM_SRC_ALPHA, CELL_GCM_ONE_MINUS_SRC_ALPHA );
			break;
		}
 	case ASURA_TRANSLUCENCY_COLOUR:
		{
			cellGcmSetBlendFuncInline( Asura_PS3_Graphics::GetGcmCurrentContext(), CELL_GCM_DST_COLOR, CELL_GCM_ZERO, CELL_GCM_DST_COLOR, CELL_GCM_ZERO );
			break;
		}
	case ASURA_TRANSLUCENCY_DOUBLECOLOUR:
		{
			cellGcmSetBlendFuncInline( Asura_PS3_Graphics::GetGcmCurrentContext(), CELL_GCM_DST_COLOR, CELL_GCM_SRC_COLOR, CELL_GCM_DST_COLOR, CELL_GCM_SRC_COLOR );
			break;
		}
 	case ASURA_TRANSLUCENCY_INVCOLOUR:
		{
			cellGcmSetBlendFuncInline( Asura_PS3_Graphics::GetGcmCurrentContext(), CELL_GCM_ZERO, CELL_GCM_ONE_MINUS_SRC_COLOR, CELL_GCM_ZERO, CELL_GCM_ONE_MINUS_SRC_COLOR );
			break;
		}
	case ASURA_TRANSLUCENCY_GLOWING:
		{
			cellGcmSetBlendFuncInline( Asura_PS3_Graphics::GetGcmCurrentContext(), CELL_GCM_SRC_ALPHA, CELL_GCM_ONE, CELL_GCM_SRC_ALPHA, CELL_GCM_ONE );
			break;
		}
	case ASURA_TRANSLUCENCY_DARKENINGCOLOUR:
		{
			cellGcmSetBlendFuncInline( Asura_PS3_Graphics::GetGcmCurrentContext(), CELL_GCM_ONE_MINUS_DST_COLOR, CELL_GCM_ZERO, CELL_GCM_ONE_MINUS_DST_COLOR, CELL_GCM_ZERO );
			break;
		}
	case ASURA_TRANSLUCENCY_JUSTSETZ:
		{
			cellGcmSetBlendFuncInline( Asura_PS3_Graphics::GetGcmCurrentContext(), CELL_GCM_ZERO, CELL_GCM_ONE, CELL_GCM_ZERO, CELL_GCM_ONE );
			break;
		}
	case ASURA_TRANSLUCENCY_ALPHACOMPARE:
		{
			cellGcmSetBlendFuncInline( Asura_PS3_Graphics::GetGcmCurrentContext(), CELL_GCM_DST_ALPHA, CELL_GCM_ZERO, CELL_GCM_DST_ALPHA, CELL_GCM_ZERO );
			break;
		}
	case ASURA_TRANSLUCENCY_UGH:
		{
			cellGcmSetBlendFuncInline( Asura_PS3_Graphics::GetGcmCurrentContext(), CELL_GCM_DST_COLOR, CELL_GCM_ONE, CELL_GCM_DST_COLOR, CELL_GCM_ONE );
			break;
		}
	default:
		{
			break;
		}
	}		// End of switch.

	// Finally, set the current mode...
	TranslucencyMode	= mode;
#endif
}
/// End of function 'Asura_Render_States::SetTranslucencyMode'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Render_States::SetTextureOpMode
// Author		: Andy Tate (1-Jul-2005)
// Modified		: Catalin Arsenescu (27-March-2007)
//
// Description  : 
//
// Parameters   : The texture-operation to be used.
//
// Return value : None.
////////////////////////////////////////////////////////////////////////////////
void Asura_Render_States::SetTextureOpMode (enum ASURA_TEXTUREOP_ID mode)
{
	Asura_Assert(!g_bAssertsEnabled || mode == ASURA_TEXTUREOP_SINGLE_PASS,"Asura_Render_States::SetTextureOpMode - removed function called");
#if 0
	// Finally, set the current mode...
	TextureOpMode = mode;
#endif
}
/// End of function 'Asura_Render_States::SetTextureOpMode'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Render_States::SetAlphaTest
// Author		: Andy Tate (1-Jul-2005)
// Modified		: Catalin Arsenescu (27-March-2007)
//
// Description  : 
//
// Parameters   : "true" to set the mode, "false" to unset it.
//
// Return value : None.
////////////////////////////////////////////////////////////////////////////////

void Asura_Render_States::SetAlphaTest (bool bSet)
{
	Asura_Assert(!g_bAssertsEnabled || bSet,"Asura_Render_States::SetAlphaTest - removed function called");
#if 0
	// Do we wish to enable alpha-testing?
	if (bSet)
	{
		if( Asura_PS3_DeferredRendering::IsBindAsSingleRT() )
		{
			cellGcmSetAlphaTestEnableInline( Asura_PS3_Graphics::GetGcmCurrentContext(), CELL_GCM_TRUE );
			cellGcmSetAlphaFuncInline( Asura_PS3_Graphics::GetGcmCurrentContext(), CELL_GCM_GREATER, m_iAlphaTestRef);
		}
		else
		{
			bSet = false;
			Asura_Assert( false, "Alpha testing is not available when using multiple render targets" );
		}
	}
	else
	{
		cellGcmSetAlphaTestEnableInline( Asura_PS3_Graphics::GetGcmCurrentContext(), CELL_GCM_FALSE );
	}

	// Finally, set the current mode...
	AlphaTest	= bSet;
#endif
}
/// End of function 'Asura_Render_States::SetAlphaTest'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Render_States::SetDynamicLights
// Author		: Andy Tate (1-Jul-2005)
// Modified		: Catalin Arsenescu (27-March-2007)
//
// Description  : 
//
// Parameters   : "true" to set the mode, "false" to unset it.
//
// Return value : None.
////////////////////////////////////////////////////////////////////////////////

void Asura_Render_States::SetDynamicLights (bool bSet)
{
	Asura_Assert(!g_bAssertsEnabled || !bSet,"Asura_Render_States::SetDynamicLights - removed function called");
#if 0
	// Finally, set the current mode...
	DynamicLights	= bSet;
#endif
}
/// End of function 'Asura_Render_States::SetDynamicLights'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Render_States::SetCullMode
// Author		: Andy Tate (1-Jul-2005)
// Modified		: Catalin Arsenescu (27-March-2007)
//
// Description  : 
//
// Parameters   : The cull-mode to be used.
//
// Return value : None.
////////////////////////////////////////////////////////////////////////////////
void Asura_Render_States::SetCullMode (enum ASURA_CULLMODE_ID eMode)
{
	Asura_Assert(!g_bAssertsEnabled || eMode == ASURA_CULLMODE_NONE,"Asura_Render_States::SetCullMode - removed function called");
#if 0
	// Which cull-mode do we wish to use?

	switch (eMode)
	{
	case ASURA_CULLMODE_CW:
		{
			// Ensure that cull-mode is enabled...
			cellGcmSetCullFaceEnableInline ( Asura_PS3_Graphics::GetGcmCurrentContext(), CELL_GCM_TRUE );

			// Are we to mirror the image?
			if (Asura_Render::ShouldRenderMirrored())
			{
				cellGcmSetFrontFaceInline( Asura_PS3_Graphics::GetGcmCurrentContext(), CELL_GCM_CCW );
			}
			else
			{
				cellGcmSetFrontFaceInline( Asura_PS3_Graphics::GetGcmCurrentContext(), CELL_GCM_CW );
			}
			break;
		}
	case ASURA_CULLMODE_CCW:
		{
			// Ensure that cull-mode is enabled...
			cellGcmSetCullFaceEnableInline ( Asura_PS3_Graphics::GetGcmCurrentContext(), CELL_GCM_TRUE );

			// Are we to mirror the image?
			if (Asura_Render::ShouldRenderMirrored())
			{
				cellGcmSetFrontFaceInline( Asura_PS3_Graphics::GetGcmCurrentContext(), CELL_GCM_CW );
			}
			else
			{
				cellGcmSetFrontFaceInline( Asura_PS3_Graphics::GetGcmCurrentContext(), CELL_GCM_CCW );
			}
			break;
		}
	default:
		{
			// Diable cull-mode...
			cellGcmSetCullFaceEnableInline ( Asura_PS3_Graphics::GetGcmCurrentContext(), CELL_GCM_FALSE );
			break;
		}
	}		// End of switch.


	// Finally, set the current mode...

	m_eCullMode = eMode;
#endif
}
/// End of function 'Asura_Render_States::SetCullMode'. ///

void Asura_Render_States::SetStencilEnabled( bool bSet ) 
{ 
	Asura_Assert(!g_bAssertsEnabled || !bSet,"Asura_Render_States::SetStencilEnabled - removed function called");
#if 0
	StencilEnabled = b;
#endif
}

/// End of file. ///
