//
// Asura_PS3_GCM_Shader.h
//
// Created by Catalin Arsenescu - 29 March 2007
//
// Copyright (C) 2005 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif

#ifndef __ASURA_PS3_GCM_SHADER_H__
#define __ASURA_PS3_GCM_SHADER_H__

// I N C L U D E S ///////////////////////////////////////////////////////////
//Sony
#include <cell/cgb.h>

// Engine platform independent
#include "Core/Memory/Asura_Mem.h"
#include "Core/Asura_String.h"
#include "Core/Asura_Memheap.h"
#include "Core/Asura_HashTree.h"

// Engine platform-specific.
#include "Rendering/Asura_PS3_Graphics.h"
#include "Rendering/Asura_PS3_RenderingStates.h"
#include "Rendering/Asura_PS3_TextureManagement.h"
#include "Rendering/Asura_PS3_VideoMemoryManager.h"

#define ASURA_MAXIMUM_FRAGMENT_SHADER_OFFSETS	17
#define ASURA_MAXIMUM_NUMBER_OF_POOLS			2048
#define ASURA_SHADERS_LIBRARY_BUFFERS			2

#ifndef ASURA_ASSERT
#include "Debugging/Asura_Logfile.h"
#endif

class Asura_PS3_GCM_Shader
{
public:
	Asura_PS3_GCM_Shader( unsigned char* uStartProgram, unsigned char* uEndProgram )
	{
		m_aiVertexInputIndex	= NULL;
		m_aiTextureUnitIndex	= NULL;

		u_int	uProgramSize	= uEndProgram - uStartProgram;
		int		res				= cellCgbRead( uStartProgram, uProgramSize, &m_xProgram );

		if (res != CELL_CGB_OK)
		{
#ifdef ASURA_ASSERT
			Asura_Assert( false, "Error: couldn't read vertex program binary\n" );
#else
			Asura_OutputToDebugger( ("Error: couldn't read vertex program binary\n") );
#endif
		}
	}

	virtual ~Asura_PS3_GCM_Shader()
	{
		m_aiVertexInputIndex	= NULL;
		m_aiTextureUnitIndex	= NULL;
	}

protected:
	CellCgbProgram						m_xProgram;
	int*								m_aiVertexInputIndex;
	int*								m_aiTextureUnitIndex;
}; //Asura_PS3_GCM_Shader

#include "Rendering/Shaders/Asura_PS3_Shader_Elements.h"
#include "Rendering/Shaders/Asura_PS3_VertexShader.h"
#include "Rendering/Shaders/Asura_PS3_FP_Handles.h"
#include "Rendering/Shaders/Asura_PS3_FragmentShaderBase.h"
#include "Rendering/Shaders/Asura_PS3_FragmentShaderCopy.h"
#include "Rendering/Shaders/Asura_PS3_FP_List.h"
#include "Rendering/Shaders/Asura_PS3_FP_List_Tree.h"
#include "Rendering/Shaders/Asura_PS3_GCM_ShadersLibrary.h"

#endif //#ifndef __ASURA_PS3_GCM_SHADER_H__
