#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif

#ifndef __ASURA_PS3_VERTEX_SHADER_CG_H__
#define __ASURA_PS3_VERTEX_SHADER_CG_H__

#include "Core/Memory/Asura_Mem.h"

// C L A S S E S /////////////////////////////////////////////////////////////
class Asura_PS3_VertexBufferDeclaration
{
public:
	// id from shader
	const char*		m_szName;
	u_int			m_uOffset;
	u_char			m_uType;
	u_char			m_uNumberOfCoords;
	u_char			m_ucAttributeIndex; //Automatically set by the Vertex/Fragment Program
}; //class Asura_PS3_VertexBufferDeclaration

class Asura_PS3_GCM_VertexShader : public Asura_PS3_GCM_Shader
{
public:
	Asura_PS3_GCM_VertexShader( unsigned char* uStartProgram, unsigned char* uEndProgram, Asura_PS3_GCM_VertexShaderUniformParameter* pxVertexShaderUniform, char cVertexShaderUniformElements, Asura_PS3_GCM_TextureUnitElement* pxTextureUnit = NULL, char cTextureUnitsElements = 0 );

	void InitVertexBufferDeclaration( Asura_PS3_VertexBufferDeclaration* pxVertexDeclaration, u_char cVertexDeclarationElements );

	void SetUniformParameter( int iParameter, const float* data )
	{
		//TODO: Modify to support setting a individual parameter when the compiler will support getting individual parameters.
		SetUniformParameter( iParameter, data, m_pxVertexShaderUniform[ iParameter ].m_usArraySize );
	}

	void SetUniformParameter( int iParameter, const float* data, unsigned short usElements )
	{
		//TODO: Modify to support setting a individual parameter when the compiler will support getting individual parameters.
		SetRegisterValue( m_pxVertexShaderUniform[ iParameter ].m_usRegister[ 0 ], data, m_pxVertexShaderUniform[ iParameter ].m_sizeInFloat4ForOneElement * usElements );
	}

	void SetBoolParameter( int iParameter, bool bValue );

	void SetBranchBits( const uint32_t uiBranchBits )
	{
		cellGcmSetTransformBranchBitsUnsafeInline( Asura_PS3_Graphics::GetGcmCurrentContext(), uiBranchBits );
	}

	void Bind( void )
	{
		Asura_PS3_RenderingStates::SetVertexShader( &m_xProgramConfig, m_pxProgramAddr, this );
	}

	void LoadVertexProgramConstants( void );

	static void ResetToDefault( void )
	{
		Asura_Mem::Set( &s_afRegisters[ 0 ][ 0 ], 0xFF, sizeof( s_afRegisters ) );
		s_pxIgnoreRegisterCache = NULL;
	}

	virtual ~Asura_PS3_GCM_VertexShader()
	{
	}
	
	int GetRegisterIndex( int iParameter )
	{
		return m_pxVertexShaderUniform[ iParameter ].m_usRegister[ 0 ];
	}
	
	void InvalidateRegisterCache()
	{
		// some of this shader's registers are being set directly, make sure we clean up our cache of registers before the next shader relies on it
		s_pxIgnoreRegisterCache = this;
	}
	
protected:
	void SetRegisterValue( int iRegister, const float* pfData, unsigned short usFloat4 );
		
	CellCgbVertexProgramConfiguration	m_xProgramConfig;
	Asura_PS3_GCM_VertexShaderUniformParameter*	m_pxVertexShaderUniform;
	const void*							m_pxProgramAddr;
	static float						s_afRegisters[ 468 ][ 4 ]; //468 float4 registers in RSX
	static Asura_PS3_GCM_VertexShader	*s_pxIgnoreRegisterCache;
}; //class Asura_PS3_GCM_VertexShader

#endif //#ifndef __ASURA_PS3_VERTEX_SHADER_CG_H__
