//
// AmbientOcclusion_FP.h
//
// Ben Weston 14/11/08
//
// This is a non-standard way of declaring a shader, see my notes in EdgeAntiAlias_FP.h
//

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __ASURA_PS3_AMBIENTOCCLUSION_FP_H__
#define __ASURA_PS3_AMBIENTOCCLUSION_FP_H__

// The number of ambient occlusion blurred levels, not counting sampling direct from the depth buffer for the first (sharpest) level (if we're doing that)
// i.e. this is the number of textures that need to be generated by pre-processing before the actual SSAO pass.
#define NUM_AMBIENT_OCCLUSION_LEVELS			2
#define NUM_AMBIENT_OCCLUSION_SHADER_SAMPLERS	2 //corresponding to declaration in AmbientOcclusion_FP.fpx


extern u_int _binary_PS3_AmbientOcclusion_FP_bin_start;
extern u_int _binary_PS3_AmbientOcclusion_FP_bin_end;

class Asura_PS3_AmbientOcclusion_FP : public Asura_PS3_FragmentShader
{
public:
	enum {
		FP_ViewMatrix,
		FP_TwoOverScreen,
		FP_CameraTanHalfFOV,
		FP_DepthAdjustment,
		FP_BrightnessContrast,
		FP_DistanceFade,
		FP_SamplePositions,
		FP_ELEMENTS
	};

	enum {
		FP_NormalMaterialBufferSampler,
		FP_DepthBlurSamplers,
		FP_DepthBufferSampler = FP_DepthBlurSamplers + NUM_AMBIENT_OCCLUSION_SHADER_SAMPLERS,
		TU_ELEMENTS
	};

	class Handles : public Asura_PS3_FP_Handles
	{
	public:
		Handles() : Asura_PS3_FP_Handles( FP_ELEMENTS, TU_ELEMENTS )
		{}

		void InitialiseFromShader( Asura_PS3_FragmentShaderBase* pxShader )
		{
			//Uniform parameters
			pxShader->InitUniformElement( FP_ViewMatrix,			"g_xViewMatrix",			4 );
			pxShader->InitUniformElement( FP_TwoOverScreen,			"g_xTwoOverScreen",			1 );
			pxShader->InitUniformElement( FP_CameraTanHalfFOV,		"g_xCameraTanHalfFOV",		1 );
			pxShader->InitUniformElement( FP_DepthAdjustment,		"g_xDepthAdjustment",		1 );
			pxShader->InitUniformElement( FP_BrightnessContrast,	"g_xBrightnessContrast",	1 );
			pxShader->InitUniformElement( FP_DistanceFade,			"g_xDistanceFade",			1 );
			pxShader->InitUniformElement( FP_SamplePositions,		"g_axSamplePositions",		2 );
			

			//Texture units
			pxShader->InitTextureUnit( FP_NormalMaterialBufferSampler, "g_xNormalMaterialBufferSampler", CELL_GCM_TEXTURE_CLAMP, CELL_GCM_TEXTURE_CLAMP, CELL_GCM_TEXTURE_NEAREST, CELL_GCM_TEXTURE_NEAREST );

			for ( int i=0; i < NUM_AMBIENT_OCCLUSION_SHADER_SAMPLERS; i++ )
			{
				char szName[64];
				sprintf( szName, "g_axDepthDownsampleSampler[%d]", i );

				//cannot bilinear filter FP32, the filter is only 16 precision (which may be good enough, but it refuses to use it & crashes)
				pxShader->InitTextureUnit( FP_DepthBlurSamplers + i, szName, CELL_GCM_TEXTURE_CLAMP, CELL_GCM_TEXTURE_CLAMP, CELL_GCM_TEXTURE_NEAREST, CELL_GCM_TEXTURE_NEAREST );
			}

			pxShader->InitTextureUnit( FP_DepthBufferSampler, "g_xDepthBufferSampler", CELL_GCM_TEXTURE_CLAMP, CELL_GCM_TEXTURE_CLAMP, CELL_GCM_TEXTURE_NEAREST, CELL_GCM_TEXTURE_NEAREST );
		}
	};

	Asura_PS3_AmbientOcclusion_FP() :
		Asura_PS3_FragmentShader( (unsigned char *)&_binary_PS3_AmbientOcclusion_FP_bin_start,
								  (unsigned char *)&_binary_PS3_AmbientOcclusion_FP_bin_end,
								  new Handles ) // create handles
	{
	}

	~Asura_PS3_AmbientOcclusion_FP()
	{
		delete m_pxHandles;	// delete the handles we created in the constructor (though it's not obvious thanks to C++'s obfuscated syntax)
	}

	// alternative to creating an instance of this shader, hand it over to the library.
	static void AddToShaderLibrary(Asura_Hash_ID uHashTree, Asura_Hash_ID uHashFP)
	{
		Asura_PS3_GCM_ShadersLibrary::AddFP( uHashTree, uHashFP, (unsigned char *)&_binary_PS3_AmbientOcclusion_FP_bin_start, (unsigned char *)&_binary_PS3_AmbientOcclusion_FP_bin_end - (unsigned char *)&_binary_PS3_AmbientOcclusion_FP_bin_start,
											new Handles, 1, true );
	}
};


#endif //#ifndef __ASURA_PS3_AMBIENTOCCLUSION_FP_H__
