//
// DepthDownsample_FP.h
//
// Ben Weston 14/11/08
//
// This is a non-standard way of declaring a shader, see my notes in EdgeAntiAlias_FP.h
//

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __ASURA_PS3_DEPTHDOWNSAMPLE_FP_H__
#define __ASURA_PS3_DEPTHDOWNSAMPLE_FP_H__


extern u_int _binary_PS3_DepthDownsample_FP_bin_start;
extern u_int _binary_PS3_DepthDownsample_FP_bin_end;

class Asura_PS3_DepthDownsample_FP : public Asura_PS3_FragmentShader
{
public:
	enum {
		FP_Parameters,
		FP_DepthProjection,
		FP_ELEMENTS
	};

	enum {
		FP_DepthBufferSampler,
		TU_ELEMENTS
	};

	class Handles : public Asura_PS3_FP_Handles
	{
	public:
		Handles() : Asura_PS3_FP_Handles( FP_ELEMENTS, TU_ELEMENTS )
		{}

		void InitialiseFromShader( Asura_PS3_FragmentShaderBase* pxShader )
		{
			//Uniform parameters
			pxShader->InitUniformElement( FP_Parameters,		"g_afParams",			1 );
			pxShader->InitUniformElement( FP_DepthProjection,	"g_xDepthProjection",	1 );
		
			//Texture units
			pxShader->InitTextureUnit( FP_DepthBufferSampler, "g_xDepthBufferSampler", CELL_GCM_TEXTURE_CLAMP, CELL_GCM_TEXTURE_CLAMP, CELL_GCM_TEXTURE_NEAREST, CELL_GCM_TEXTURE_NEAREST );
		}
	};

	Asura_PS3_DepthDownsample_FP() :
		Asura_PS3_FragmentShader( (unsigned char *)&_binary_PS3_DepthDownsample_FP_bin_start,
								  (unsigned char *)&_binary_PS3_DepthDownsample_FP_bin_end,
								  new Handles ) // create handles
	{
	}

	~Asura_PS3_DepthDownsample_FP()
	{
		delete m_pxHandles;	// delete the handles we created in the constructor (though it's not obvious thanks to C++'s obfuscated syntax)
	}
	
	// alternative to creating an instance of this shader, hand it over to the library.
	static void AddToShaderLibrary(Asura_Hash_ID uHashTree, Asura_Hash_ID uHashFP)
	{
		Asura_PS3_GCM_ShadersLibrary::AddFP( uHashTree, uHashFP, (unsigned char *)&_binary_PS3_DepthDownsample_FP_bin_start, (unsigned char *)&_binary_PS3_DepthDownsample_FP_bin_end - (unsigned char *)&_binary_PS3_DepthDownsample_FP_bin_start,
											new Handles, 1, true );
	}
};



#endif //#ifndef __ASURA_PS3_DEPTHDOWNSAMPLE_FP_H__
