
#include "Shared.FXH"
#include "MaterialResponse_Shared.ps3FXH"

const float4x4	g_xInvViewMatrix;

const float4	g_xTwoOverScreen;		// dimensions of screen, in pixels. Used to convert VPOS pixel shader input into normalised coordinates.
const float4	g_xCameraTanHalfFOV;	// Slopes of camera frustum. Used to reconstruct viewspace positions from screen coordinates and depth.

// Omni light parameters
const float4	g_xFalloff;
const float4	g_xLightPosScale;		// World-space light pos in xyz, 1/range in w
const float4	g_xLightColour;			// Colour in rgb, intensity in a

// Directional light parameters
const float4	g_xLightDirection;
const float3x3	g_xLightOrientation;

const float		g_fProjCubeLerpDist;

const sampler	g_xNormalMaterialBufferSampler		:register( s0 );
const sampler	g_xAlbedoSpecularBufferSampler		:register( s1 );
const sampler	g_xDepthBufferSampler				:register( s2 );

const samplerCUBE g_xProjCubeMapSampler;
const samplerCUBE g_xProjCubeMapBlurredSampler;
/******************************************************************************/
/* Types                                                                      */
/******************************************************************************/

struct VS_Output
{
	float4 xPosition  : POSITION;
};

// -----------------------------------------------------------------------------
// Pixel Shader (ClearDestAlpha)
// Just output 0
// -----------------------------------------------------------------------------

#ifdef SHADER_CLEAR_ALPHA
float4 main( void ) : COLOR
{
	return 0.0f;
}
#endif


// -----------------------------------------------------------------------------
// Pixel Shader (Photon)
// Material response lighting from a point light
// -----------------------------------------------------------------------------
#ifdef SHADER_PHOTON
float4 main( const VS_Output xIn, half2 xScreenPos : WPOS ) : COLOR
{
	const float4 xBufferNormalMaterial		= GetScreenColour( g_xNormalMaterialBufferSampler,		xScreenPos );
	const float4 xBufferAlbedoSpecular		= GetScreenColour( g_xAlbedoSpecularBufferSampler,		xScreenPos );
	const float4 xDepthBufferValue			= GetScreenColour( g_xDepthBufferSampler,				xScreenPos );

	float3	xNormal;
	float	fScreenDepth;
	float3	xAlbedoColour;
	float	hSpecularStrength;
	float	fMaterial;

	Deferred_GetNormalDepthAlbedoSpecularMaterial( xBufferNormalMaterial, xBufferAlbedoSpecular, xDepthBufferValue, xNormal, fScreenDepth, xAlbedoColour, hSpecularStrength, fMaterial);

//return float4(float3(fScreenDepth),1);


	// Reconstruct the camera view space position. I'm not 100% sure I understand why we can't construct the clip space position
	// and then push it through the inverse projection matrix, but I think it's because we don't have a value for w and so can't
	// reverse the transform correctly. Note that the y coordinate gets flipped; this is because Asura's Y coordinates are upside down
	xScreenPos -= 0.5h;
//	const float4 xCameraViewSpacePosition = float4(	g_xCameraTanHalfFOV.x * fScreenDepth *  ( xScreenPos.x * g_xTwoOverScreen.x - 1.0h ),
//													g_xCameraTanHalfFOV.y * fScreenDepth * -( xScreenPos.y * g_xTwoOverScreen.y - 1.0h ),
//													fScreenDepth, 1.0h );
	const float4 xCameraViewSpacePosition = float4(	g_xCameraTanHalfFOV.xy * fScreenDepth * ( xScreenPos * g_xTwoOverScreen.xy - 1.0h ),
													fScreenDepth, 1.0h );

	float4 xOutput;

	float4	xWorldPosition	= mul( xCameraViewSpacePosition, g_xInvViewMatrix );
	float3	xLightDelta		= g_xLightPosScale.xyz - xWorldPosition.xyz;
	float3	xLightStrength	= xLightDelta.xyz * g_xLightPosScale.w;
	float	fLightDistance	= sqrt( dot( xLightStrength.xyz, xLightStrength.xyz ) );
	float3	xCameraDir		= normalize( g_xCameraPosition.xyz - xWorldPosition.xyz );

	// Early-out if not in range of the light
	clip( 0.5f - abs( fLightDistance - 0.5f ) );

	// Full material response (slow)
	float	fLightAttenuation		= min( 1.0h, pow( ( 1.0h - fLightDistance ) * g_xFalloff.x, g_xFalloff.y ) );
	float3	xLightAttenuatedColour	= g_xLightColour.rgb * fLightAttenuation;
	xOutput.rgb = MaterialResponse_Basic( fMaterial, xAlbedoColour, hSpecularStrength,
		float4( xNormal, 1.0h ), xCameraDir, normalize( xLightDelta ), xLightAttenuatedColour,
		xLightAttenuatedColour,  dot( xLightAttenuatedColour, float3( 0.333h, 0.333h, 0.333h ) ) );
	xOutput.a = 1.0h;

	return xOutput;
}
#endif


// -----------------------------------------------------------------------------
// Pixel Shader (Limited Photon)
// Material response lighting from a point light - with angular constraints
// and cubemap projection
// -----------------------------------------------------------------------------
#ifdef SHADER_LIMITED_PHOTON

#ifdef SHADER_USE_ANGULAR_RANGE
static const bool g_bUseAngularRange = true;
#else
static const bool g_bUseAngularRange = false;
#endif

#ifdef SHADER_USE_PROJ_CUBE
static const bool g_bUseProjCube = true;
#else
static const bool g_bUseProjCube = false;
#endif

#ifdef SHADER_USE_DUAL_PROJ_CUBE
static const bool g_bUseDualProjCube = true;
#else
static const bool g_bUseDualProjCube = false;
#endif

float4 main( const VS_Output xIn, float2 xScreenPos : WPOS ) : COLOR
{
	const float4 xBufferNormalMaterial		= GetScreenColour( g_xNormalMaterialBufferSampler,		xScreenPos );
	const float4 xBufferAlbedoSpecular		= GetScreenColour( g_xAlbedoSpecularBufferSampler,		xScreenPos );
	const float4 xDepthBufferValue			= GetScreenColour( g_xDepthBufferSampler,				xScreenPos );

	float3	xNormal;
	float	fScreenDepth;
	float3	xAlbedoColour;
	float	hSpecularStrength;
	float	fMaterial;

	Deferred_GetNormalDepthAlbedoSpecularMaterial( xBufferNormalMaterial, xBufferAlbedoSpecular, xDepthBufferValue, xNormal, fScreenDepth, xAlbedoColour, hSpecularStrength, fMaterial);

	// Reconstruct the camera view space position. I'm not 100% sure I understand why we can't construct the clip space position
	// and then push it through the inverse projection matrix, but I think it's because we don't have a value for w and so can't
	// reverse the transform correctly. Note that the y coordinate gets flipped; this is because Asura's Y coordinates are upside down
	xScreenPos -= 0.5h;
//	const float4 xCameraViewSpacePosition = float4(	g_xCameraTanHalfFOV.x * fScreenDepth *  ( xScreenPos.x * g_xTwoOverScreen.x - 1.0h ),
//													g_xCameraTanHalfFOV.y * fScreenDepth * -( xScreenPos.y * g_xTwoOverScreen.y - 1.0h ),
//													fScreenDepth, 1.0h );
	const float4 xCameraViewSpacePosition = float4(	g_xCameraTanHalfFOV.xy * fScreenDepth * ( xScreenPos * g_xTwoOverScreen.xy - 1.0h ),
													fScreenDepth, 1.0h );

	float4 xOutput;

	float4	xWorldPosition	= mul( xCameraViewSpacePosition, g_xInvViewMatrix );
	float3	xLightDelta		= g_xLightPosScale.xyz - xWorldPosition.xyz;
	float3	xLightStrength	= xLightDelta.xyz * g_xLightPosScale.w;
	float	fLightDistance	= sqrt( dot( xLightStrength.xyz, xLightStrength.xyz ) );
	float3	xCameraDir		= normalize( g_xCameraPosition.xyz - xWorldPosition.xyz );

	// Early-out if not in range of the light
	clip( 0.5f - abs( fLightDistance - 0.5f ) );

	// Full material response (slow)
	float	fLightAttenuation = min( 1.0h, pow( ( 1.0h - fLightDistance ) * g_xFalloff.x, g_xFalloff.y ) );
	float3	xOfs = normalize( g_xLightPosScale.xyz - xWorldPosition.xyz );

	if( g_bUseAngularRange )
	{
		// Handle the angular falloff		
		float fDot = dot( xOfs, g_xLightDirection.xyz );
		fLightAttenuation *= saturate( ( fDot + g_xFalloff.z ) * g_xFalloff.w );
	}

	float3 xLightAttenuatedColour = g_xLightColour.rgb * fLightAttenuation;

	if( g_bUseProjCube )
	{
		float3 xCol;
				
		// Cubemap projector
		float3 xDir = normalize( mul( xOfs, g_xLightOrientation ) );
		xCol = h3texCUBE( g_xProjCubeMapSampler, xDir);		

		if( g_bUseDualProjCube )
		{
			float fLerp = min( fLightDistance / g_fProjCubeLerpDist, 1.0h );
			xCol = lerp( xCol, h3texCUBE( g_xProjCubeMapBlurredSampler, xDir ), fLerp );
		}

		xLightAttenuatedColour *= xCol;
	}			

	xOutput.rgb = MaterialResponse_Basic( fMaterial, xAlbedoColour, hSpecularStrength,
		float4( xNormal, 1.0h ), xCameraDir, normalize( xLightDelta ), xLightAttenuatedColour,
		xLightAttenuatedColour, dot( xLightAttenuatedColour, float3( 0.333h, 0.333h, 0.333h ) ) );
	xOutput.a = 1;

	return xOutput;
}
#endif

// -----------------------------------------------------------------------------
// Pixel Shader (Infinite)
// Material response lighting from a parallel light
// -----------------------------------------------------------------------------
#ifdef SHADER_INFINITE
float4 main( const VS_Output xIn, float2 xScreenPos : WPOS ) : COLOR
{
	const float4 xBufferNormalMaterial		= GetScreenColour( g_xNormalMaterialBufferSampler,		xScreenPos );
	const float4 xBufferAlbedoSpecular		= GetScreenColour( g_xAlbedoSpecularBufferSampler,		xScreenPos );
	const float4 xDepthBufferValue			= GetScreenColour( g_xDepthBufferSampler,				xScreenPos );

	float3	xNormal;
	float	fScreenDepth;
	float3	xAlbedoColour;
	float	hSpecularStrength;
	float	fMaterial;

	Deferred_GetNormalDepthAlbedoSpecularMaterial( xBufferNormalMaterial, xBufferAlbedoSpecular, xDepthBufferValue, xNormal, fScreenDepth, xAlbedoColour, hSpecularStrength, fMaterial);

	// Reconstruct the camera view space position. I'm not 100% sure I understand why we can't construct the clip space position
	// and then push it through the inverse projection matrix, but I think it's because we don't have a value for w and so can't
	// reverse the transform correctly. Note that the y coordinate gets flipped; this is because Asura's Y coordinates are upside down
	xScreenPos -= 0.5h;
//	const float4 xCameraViewSpacePosition = float4(	g_xCameraTanHalfFOV.x * fScreenDepth *  ( xScreenPos.x * g_xTwoOverScreen.x - 1.0h ),
//													g_xCameraTanHalfFOV.y * fScreenDepth * -( xScreenPos.y * g_xTwoOverScreen.y - 1.0h ),
//													fScreenDepth, 1.0h );
	const float4 xCameraViewSpacePosition = float4(	g_xCameraTanHalfFOV.xy * fScreenDepth * ( xScreenPos * g_xTwoOverScreen.xy - 1.0h ),
													fScreenDepth, 1.0h );
		   														 
	float4	xOutput;
	float4	xWorldPosition  = mul( xCameraViewSpacePosition, g_xInvViewMatrix );
	float3	xLightDelta		= g_xLightPosScale.xyz - xWorldPosition.xyz;
	float3	xLightStrength	= xLightDelta.xyz * ( g_xLightPosScale.w );
	float	fLightDistance	= sqrt( dot( xLightStrength.xyz, xLightStrength.xyz ) );
	float3	xCameraDir		= normalize( g_xCameraPosition.xyz - xWorldPosition.xyz );
	
	clip( 0.5f - abs( fLightDistance - 0.5f ) );
	// Handle the falloff
	float	fLightAttenuation		= min( 1.0h, pow( ( ( 1.0h - fLightDistance ) * g_xFalloff.x ), g_xFalloff.y ) );
	float3	xLightAttenuatedColour	= g_xLightColour.rgb * fLightAttenuation;

	xOutput.rgb	= MaterialResponse_Basic( fMaterial, xAlbedoColour, hSpecularStrength,
		float4( xNormal, 1.0h ), xCameraDir, g_xLightDirection.xyz, xLightAttenuatedColour,
		xLightAttenuatedColour, dot( xLightAttenuatedColour, float3( 0.333h, 0.333h, 0.333h ) ) );
	xOutput.a	= 1.0h;

	return xOutput;
}
#endif
