
/******************************************************************************/
/* Xbox360 version of the 'Advanced Bloom/SM3.0 Bloom' shader				  */
/* Dave Reed - 24/01/07														  */
/*																			  */
/* Basically a port of the PC version, but updated to work the 360 way, with  */
/* just one render target which gets resolved to a texture.					  */
/*																			  */
/* I've not attempted to optimize this yet, in case we decide to change it's  */
/* behaviour																  */
/******************************************************************************/

/******************************************************************************/
/* Parameters                                                                 */
/******************************************************************************/

const sampler2D g_xTexture		: register(s0);
const sampler2D g_xAccumTexture	: register(s1);
const sampler2D g_xDepthTexture	: register(s2);

#ifndef SHADER_USE_BLUR
#define SHADER_USE_BLUR false
#endif

#ifndef SHADER_USE_EXTRACT_BRIGHT_BITS
#define SHADER_USE_EXTRACT_BRIGHT_BITS false
#endif

#ifndef SHADER_USE_ACCUMULATE
#define SHADER_USE_ACCUMULATE false
#endif

#ifndef SHADER_USE_DEPTH_OF_FIELD
#define SHADER_USE_DEPTH_OF_FIELD false
#endif

static const bool   g_bBlur						= SHADER_USE_BLUR;
static const bool   g_bExtractBrightBits		= SHADER_USE_EXTRACT_BRIGHT_BITS;
static const bool   g_bAccumulate				= SHADER_USE_ACCUMULATE;
static const bool   g_bExtractNearDepthOfField	= SHADER_USE_DEPTH_OF_FIELD;

const float3	g_xOutputColour;
const float3	g_xInputThreshold;
const float3	g_xFocalDistances;
const float2	g_xUVCorrection;
const float		g_fSaturation;
const float		g_fInputScale;
const float		g_fAccumFade;

/******************************************************************************/
/* Types                                                                      */
/******************************************************************************/

struct OutputVertex
{
	float4 m_xPosition	: POSITION;
	float4 m_xUVs01		: TEXCOORD0;
	float4 m_xUVs23		: TEXCOORD1;
	float2 m_xUVs4		: TEXCOORD2;
};

/******************************************************************************/
/* Functions                                                                  */
/******************************************************************************/

half4 main( const OutputVertex xIn,  const half2 xScreenPos : WPOS  ) : COLOR
{
	half4 xCol;

	if( g_bBlur )
	{
		xCol.rgb = 0.25h * (	saturate( h3tex2D( g_xTexture, xIn.m_xUVs01.zw ) ) +
								saturate( h3tex2D( g_xTexture, xIn.m_xUVs23.xy ) ) +
								saturate( h3tex2D( g_xTexture, xIn.m_xUVs23.zw ) ) +
								saturate( h3tex2D( g_xTexture, xIn.m_xUVs4.xy  ) ) );
		xCol.a = g_xFocalDistances.z;
	}
	else
	{
		xCol.rgb = saturate( h3tex2D( g_xTexture, xIn.m_xUVs01.xy ) );
		xCol.a = 1.0h;
	}

	if( g_bExtractBrightBits )
	{
	     xCol.rgb = saturate( xCol.rgb - g_xInputThreshold ) * g_fInputScale;
  
	     half fMono = 0.3333h * ( xCol.r + xCol.g + xCol.b );
	     xCol.rgb = saturate( lerp( fMono.xxx, xCol.rgb, g_fSaturation ) );

	     if( g_bAccumulate )
	     {
			// Hacky - this will need improving, especially if the system is updated to handle split-screen...
			// The UVs passed in are correct for sampling from the width x height buffer - this correction is so
			// we sample correctly from the half-dimensions buffer used to accumulate the glow
			//
			// (Values are 0.5/screen width and 0.5/screen height)

			 xCol.rgb += saturate( g_fAccumFade * saturate( h3tex2D( g_xAccumTexture, xIn.m_xUVs01.xy ) ) );
	     }
	}
	else
	{
		
	    xCol.rgb *= g_xOutputColour;
	}
	
	if( g_bExtractNearDepthOfField )
	{
		const half2 xScaledScreenPos = xScreenPos * 2.0h;
		const half4 xDepth = GetScreenColour( g_xDepthTexture, xScaledScreenPos );
		const half fDepth = Deferred_GetDepth( xDepth );

		half fBlurNear = saturate( 1.0h - ( fDepth - g_xFocalDistances.x ) / ( g_xFocalDistances.y - g_xFocalDistances.x ) );
		xCol.a = fBlurNear;
	}

	return saturate( xCol );
}
