const	float4		g_xUV_Offsets;
// Texture-related variables...
const	float4		g_xTextureCoordOffsetAndScale		= {0.0, 0.0, 1.0f, 1.0f};
// High Dynamic Range stuff
const	float 		g_fHighDynamicRangeExposureScale	= 4.0f;

// Texture co-ordinate calculation function...
// Camera-related variables...
const	float4		g_xCameraPosition;

// View-related variables...
const	float4x4	g_xViewProjectionMatrix;
float2 CalculateTextureCoordinate (float2 xUV)
{
	// Simply scale the existing co-ordinates and add the offset...
	return	((xUV * g_xTextureCoordOffsetAndScale.zw) + g_xTextureCoordOffsetAndScale.xy);
}

// Tangent-space rotation function...
float3 RotateIntoTangentSpace (float3 xVector,
							   float3 xTangent,
							   float3 xBinormal,
							   float3 xNormal)
{
	float3	xRotatedVector;

	// Rotate the x, y and z components into tangent-space...
	xRotatedVector.x	= dot(xVector, xTangent);
	xRotatedVector.y	= dot(xVector, xBinormal);
	xRotatedVector.z	= dot(xVector, xNormal);

	// Return the rotated vector...
	return	(xRotatedVector);
}

float3 RotateFromTangentSpace (float3 xVector,
							  float3 xTangent,
							  float3 xBinormal,
							  float3 xNormal)
{
	float3	xRotatedVector;

	// Rotate the x, y and z components into tangent-space...
	xRotatedVector.xyz	= xVector.x * xTangent;
	xRotatedVector.xyz	+= xVector.y * xBinormal;
	xRotatedVector.xyz	+= xVector.z * xNormal;

	// Return the rotated vector...
	return	(xRotatedVector);
}

// High-dynamic range calculation function...
float3 CalculateHighDynamicRangeRGB (float3 xColour)
{
	// Move High Dynamic Range lighting into display range...
	xColour	*= g_fHighDynamicRangeExposureScale;
	xColour	 = (xColour) / (xColour + 1.0);
	
	// Return the colour...
	return	(xColour);
}

float4 Project(	float4 vPosObject )
{
	return mul( vPosObject, g_xViewProjectionMatrix );
}

float3 CalculateBrightestSHLightDirection (float4 xRedSH,
										   float4 xGreenSH,
										   float4 xBlueSH)
{
	float3	xBrightestLightDirection;
	
	// Calculate the brightest light direction from the spherical harmonic lighting...
	xBrightestLightDirection.x	= xRedSH.x + xGreenSH.x + xBlueSH.x;
	xBrightestLightDirection.y	= xRedSH.y + xGreenSH.y + xBlueSH.y;
	xBrightestLightDirection.z	= xRedSH.z + xGreenSH.z + xBlueSH.z;
	
	// Return the light direction...
	return	(xBrightestLightDirection);
}

float3 Bot_CalcNewBonedPosition( float3 xOldPosition, float fBoneWeight, float3 xBone)
{
//	float fOldPositionLength = sqrt( dot( xOldPosition.xyz, xOldPosition.xyz));
//	float3 xBonedPosition = xOldPosition + (fBoneWeight * xBone);
//	float3 xBoneDirection = normalize( xBonedPosition);
//	float3 xNewPosition = xBoneDirection * fOldPositionLength;
//	return( xNewPosition);

	//BW simple tweak to the maths, saved an instruction! just do 1 sqrt, and make it /sqrt since the instruction is actually RSQRT
	float3 xBonedPosition = xOldPosition + (fBoneWeight * xBone);
	float3 xNewPosition = xBonedPosition / sqrt( dot(xBonedPosition,xBonedPosition) / dot(xOldPosition.xyz,xOldPosition.xyz) );
	return( xNewPosition);
}

float4 MaterialResponse_AnimateUVs( const float4 xUVPair )
{
	return xUVPair + g_xUV_Offsets;
}

/// End of file. ///
