// smooth skin shader - position only
// unlike SmoothSkin_VP.fxh this does not define main(), because it's used by some shaders that have different output vertex formats which want some additional data passed through.

//This shader uses a lot of constants, so don't allow compiler to use it's own ones, and explicitly define all global constants

/******************************************************************************/
/* Parameters                                                                 */
/******************************************************************************/

const float4x4	g_xWorldViewProjectionMatrix : C0;

//this is the transpose of how we do it in the engine (converted when uploaded), to save constants (only need 3 constants not 4)
const float3x4	g_axTransforms[150] : C10; // * 3

/******************************************************************************/
/* Types                                                                      */
/******************************************************************************/

/******************************************************************************/
/* Functions                                                                  */
/******************************************************************************/

float4 SkinPosition( const float3 xInPosition, const int4 aiBoneIndices, const float4 afBoneWeights )
{
	// Position and UVs
	float4	xObjectSpacePosition; 
	float4	xScreenPosition;

	xObjectSpacePosition.xyz  = mul( g_axTransforms[ aiBoneIndices[0] ], float4( xInPosition, 1.0f ) ) * afBoneWeights[0];
	xObjectSpacePosition.xyz += mul( g_axTransforms[ aiBoneIndices[1] ], float4( xInPosition, 1.0f ) ) * afBoneWeights[1];
	xObjectSpacePosition.xyz += mul( g_axTransforms[ aiBoneIndices[2] ], float4( xInPosition, 1.0f ) ) * afBoneWeights[2];
	xObjectSpacePosition.xyz += mul( g_axTransforms[ aiBoneIndices[3] ], float4( xInPosition, 1.0f ) ) * (1-dot(afBoneWeights.xyz,float3(1)));
	xObjectSpacePosition.w  = 1.0f;

	return mul( xObjectSpacePosition, g_xWorldViewProjectionMatrix );
}


// slower calculation that matches the method used in the main shader
// the main shader blends matrices because it's faster. Though perhaps, since z-prepass uses this and is more vertex-bound, we should change the main shader.
float4 SkinPositionUsingBlendedMatrices( const float3 xInPosition, const int4 aiBoneIndices, const float4 afBoneWeightsIn, bool bDamageMap )
{
	float4			xObjectSpacePosition = float4( 0.0f, 0.0f, 0.0f, 1.0f ); 
	float4			afBoneWeights = float4 ( afBoneWeightsIn.xyz, 1-dot(afBoneWeightsIn.xyz,float3(1)) );   //this takes 2 extra instructions, not too bad

	float3x4 axBoneMatrix;

	axBoneMatrix  = g_axTransforms[ aiBoneIndices[0] ] * afBoneWeights[0];
	axBoneMatrix += g_axTransforms[ aiBoneIndices[1] ] * afBoneWeights[1];
	axBoneMatrix += g_axTransforms[ aiBoneIndices[2] ] * afBoneWeights[2];
	axBoneMatrix += g_axTransforms[ aiBoneIndices[3] ] * afBoneWeights[3];
	
	if ( bDamageMap )
	{
		// now re-compute the [2][2] value in the matrix, because that's what the damage map shader does and we must exactly match it.
		float2 xZ0 = g_axTransforms[ aiBoneIndices[0] ]._m00_m01 * g_axTransforms[ aiBoneIndices[0] ]._m11_m10;
		float2 xZ1 = g_axTransforms[ aiBoneIndices[1] ]._m00_m01 * g_axTransforms[ aiBoneIndices[1] ]._m11_m10;
		float2 xZ2 = g_axTransforms[ aiBoneIndices[2] ]._m00_m01 * g_axTransforms[ aiBoneIndices[2] ]._m11_m10;
		float2 xZ3 = g_axTransforms[ aiBoneIndices[3] ]._m00_m01 * g_axTransforms[ aiBoneIndices[3] ]._m11_m10;
		float2 xZ = xZ0*afBoneWeights[0]
				   +xZ1*afBoneWeights[1]
				   +xZ2*afBoneWeights[2]
				   +xZ3*afBoneWeights[3];

		axBoneMatrix[2][2] = xZ.x - xZ.y;
	}

	xObjectSpacePosition.xyz = mul( axBoneMatrix, float4( xInPosition, 1.0f ) );
	
	return mul( xObjectSpacePosition, g_xWorldViewProjectionMatrix );
}