// Asura_PS3_Multistream_Audio.h
// Thomas Young - 15.09.08
// Copyright (C) 2008 Rebellion, All Rights Reserved.

// F I L E   W R A P P E R ///////////////////////////////////////////////////

#if defined ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __ASURA_PS3_MULTISTREAM_AUDIO_H_
#define __ASURA_PS3_MULTISTREAM_AUDIO_H_

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

#ifdef ASURA_USE_AUDIO_SYSTEM

// Asura
#include "Core/Asura_List.h"
#include "Core/Asura_Multithread.h"
#include "Core/Resources/Asura_ResourceSet.h"
#include "Sound/Asura_Audio_DSP.h"

// PS3
#include "Sound/Asura_PS3_Multistream_Voice.h"
#include "Sound/Asura_PS3_Multistream_File_Loading.h"

// SDK
#include <cell/spurs/control.h>
#include <cell/mstream.h>

// D E F I N E S /////////////////////////////////////////////////////////////

#define		ASURA_MS_INVALID_SUBBUS		-1

// C O N S T A N T S /////////////////////////////////////////////////////////

static const int	iASURA_MS_MAX_SUBBUS		= 31;
static const int	iASURA_MS_MAX_VOICES		= 128;

// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////////

class Asura_MemHeap;

// E N U M E R A T I O N S ///////////////////////////////////////////////////

enum ASURA_MS_DSP_EFFECT_HANDLE
{
	ASURA_MS_DSP_EFFECT_HANDLE_ADD_NOISE		= ASURA_DSP_ADD_NOISE,			// Match the independent enum (Asura_Audio_DSP.h)
	ASURA_MS_DSP_EFFECT_HANDLE_SIMPLE_FILTER	= ASURA_DSP_SIMPLE_FILTER,

	ASURA_MS_DSP_EFFECT_HANDLE_REVERB			= ASURA_DSP_NUM_TYPES,
	ASURA_MS_DSP_EFFECT_HANDLE_LPF				,
	ASURA_MS_DSP_EFFECT_HANDLE_AUX_SEND			,
	ASURA_MS_DSP_EFFECT_HANDLE_AUX_RETURN		,
	
	ASURA_MS_DSP_EFFECT_NUM_HANDLES
};


// C L A S S   D E C L A R A T I O N S ///////////////////////////////////////

class Asura_MultiStream_Audio
{
	friend class Asura_MultiStream_Voice;
	friend class Asura_MultiStream_Buffer_Voice;
	friend class Asura_MultiStream_MP3_Streaming_Voice;
	friend class Asura_MultiStream_Custom_Streaming_Voice;

	public:

		//------------- Functions ----------------

		Asura_MultiStream_Audio		()	{};
		~Asura_MultiStream_Audio	()	{};

		static void							Initialise					( );
		static void							Update						( );
		static void							Shutdown					( );

		static Asura_Handle					CreateVoice					( const Asura_Hash_ID uWave );
		static Asura_Handle					CreateStreamingVoice		( );
		static Asura_Handle					CreateCustomStreamingVoice	( );
		static Asura_Handle					CreateSubmixVoice			( );
		static void							DestroyVoice				( const Asura_Handle xHandle );
		static Asura_Audio_Voice*			GetVoice					( const Asura_Handle xHandle );

		static Asura_MultiStream_Buffer*	CreateBuffer				( const char* const szFilename, bool& bAlreadyExists );
		static Asura_MultiStream_Buffer*	FindBuffer					( Asura_Hash_ID xHash );

		static char*						GetWavFilename				( Asura_Hash_ID xFileHash );

		static void							CreateDSPChain				( const Asura_Hash_ID xHash, const Asura_Audio_DSPChain* pxChain );
		static void							SetMasterDSPChain			( const Asura_Hash_ID xHash );

		#ifdef ASURA_ASSERT
		static void							PauseAllStreams				( );	// These functions are for asserts, to prevent streams starving
		static void							UnpauseAllStreams			( );
		#endif
		
	private:

		//------------- Enumerations -------------

		enum ASURA_MS_BUS_NUMBERS
		{
			iASURA_MS_REVERB_DSP_BUS_NUMBER		= 1,														// The reverb DSP bus is for reverbed sounds which are affected by
			iASURA_MS_REVERB_DSP_BUS			= CELL_MS_BUS_FLAG | iASURA_MS_REVERB_DSP_BUS_NUMBER,		// the DSP chain on the master voice

			iASURA_MS_REVERB_NODSP_BUS_NUMBER	= 2,														// The reverb No DSP bus is for reverbed sounds which are not affected
			iASURA_MS_REVERB_NODSP_BUS			= CELL_MS_BUS_FLAG | iASURA_MS_REVERB_NODSP_BUS_NUMBER,		// by the DSP chain on the master voice

			iAUSRA_MS_DEAD_BUS_NUMBER			= 3,														// The dead bus just allows us to route from streams to 
			iASURA_MS_DEAD_BUS					= CELL_MS_BUS_FLAG | iAUSRA_MS_DEAD_BUS_NUMBER,				// master with zero gain https://ps3.scedev.net/forums/thread/47205

			iAUSRA_MS_MASTER_DSP1_BUS_NUMBER	= 4,														// The Master DSP bus performs DSP effects for the master voice
			iASURA_MS_MASTER_DSP1_BUS			= CELL_MS_BUS_FLAG | iAUSRA_MS_MASTER_DSP1_BUS_NUMBER,

			iAUSRA_MS_MASTER_DSP2_BUS_NUMBER	= 5,														// The Master DSP bus performs DSP effects for the master voice, we
			iASURA_MS_MASTER_DSP2_BUS			= CELL_MS_BUS_FLAG | iAUSRA_MS_MASTER_DSP2_BUS_NUMBER,		// adjust the volume of this and the above bus to crossfade DSP chains
		};
		

		//------------- Functions ----------------
		
		static bool 						InitialiseAudio				( );
		static bool 						InitialiseSPURS				( );
		static bool 						InitialiseMultiStream		( );
		static bool 						InitialiseSulpha			( );
		static bool 						InitialiseDSPEffects		( );
		static bool							InitialiseSubbusses		( );

		static void							UpdateReverb				( );
		static void							PopulateCellReverbParams	( CellMSFXI3DL2Params* pCellParams, Asura_Audio_Reverb* pAsuraParams );

		static void 						MultiStreamUpdateThread		( uint64_t param );

		static Asura_Handle					NextVoiceHandle				( );

		static int							LoadDSPFromMemory			( void* pDSPMemory );
		static int							GetDSPHandle				( const ASURA_MS_DSP_EFFECT_HANDLE eHandle ) { return s_iDSPHandle[ eHandle ]; }
		static void*						AllocateFromDSPHeap			( const int iSize, const int iAlignment );
		static void							DeallocateFromDSPHeap		( void* pMemory );

		static int							GetNextFreeSubbus			( );
		static void							ReleaseSubbus				( const int iBusNumber );

		static void*						GetReverbAuxRetBuffer		( const bool bDSPReverb);

		//------------- Variables ----------------

		static void*											s_pSulphaMemory;
		static void*											s_pMSSystemMemory;
		static void*											s_pMP3Memory;

		static u_int											s_uMSPortNum;
		static sys_ppu_thread_t									s_xMultiStreamPuThread;
		static volatile bool									s_bMSThreadExitFlag;

		static Asura_ResourceSet <Asura_MultiStream_Buffer>		s_xBuffers;
		static char*											s_pVoiceMemory;
		static Asura_MemHeap*									s_pxVoiceHeap;
		static Asura_MultiStream_Voice*							s_pxVoices					[ iASURA_MS_MAX_VOICES ];
		static Asura_Handle										s_iNextVoiceHandle;

		static int												s_iDSPHandle				[ ASURA_MS_DSP_EFFECT_NUM_HANDLES ];
		static Asura_MemHeap*									s_pxDSPHeap;
		static void*											s_pDSPMemory;

		static void*											s_pReverbBus_ReverbDSP;
		static void*											s_pReverbBus_AuxReturnDSP;
		static void*											s_pReverbBus_ReverbNoDSP;
		static void*											s_pReverbBus_AuxReturnNoDSP;

		static int												s_iActiveMasterDSPBus;
		static float											s_fMasterDSPBusTransition;
};

#endif //__ASURA_PS3_MULTISTREAM_AUDIO_H_

#endif //ASURA_USE_AUDIO_SYSTEM


//eof