// Asura_PS3_Multistream_File_Loading.h
// Thomas Young - 18.09.08
// Copyright (C) 2008 Rebellion, All Rights Reserved.

// F I L E   W R A P P E R ///////////////////////////////////////////////////

#if defined ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __ASURA_PS3_MULTISTREAM_BUFFER_H_
#define __ASURA_PS3_MULTISTREAM_BUFFER_H_

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

#ifdef ASURA_USE_AUDIO_SYSTEM

#include "Core/Asura_String.h"
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "Debugging/Asura_Assert.h"
#include "FileAccess/Asura_PS3_FileSystem.h"
#include "Sound/Asura_PS3_Multistream_Voice.h"

// D E F I N E S /////////////////////////////////////////////////////////////

#define ASURA_PS3_MULTISTREAM_MP3_STREAM_BUFFERS	3
#define ASURA_PS3_MULTISTREAM_CUSTOM_STREAM_BUFFERS	2

// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////////

class Asura_MemHeap;

// F U N C T I O N   P R O T O T Y P E S /////////////////////////////////////

namespace Asura_Multistream_Streaming
{
	void InitialiseStreaming	();
	void ShutdownStreaming		();
}

// C L A S S   D E C L A R A T I O N S ///////////////////////////////////////

class Asura_PS3_Sound_Info
{
	public:

		// (MH - 27.03.2008) reduced size as mush as I could. With 2750 loaded sounds, 
		// and the structure using 36 bytes, that came to nearly 99kB! Now 8 bytes => more like 22kB total.
		// TY 02.10.08 - changed samples per mp3 frame to a bool (was short?) and added bitrate

		Asura_PS3_Sound_Info()
		{
			m_usMP3FrameSize	= 0;	
			m_usSampleRate		= 0;		
			m_uSampleFormat		= 0;	
			m_uChannelCount		= 0;	
			m_uTotalDataBytes	= 0;
		}

		int GetSamplesPerMP3Frame() const { return m_bSamplesPerMP3FrameIs576 ? 576 : 1152; }
		void SetSamplesPerMP3Frame(int iValue) 
		{ 
			Asura_Assert(iValue == 1152 || iValue == 576, "unexpected samples per MP3 frame");
			m_bSamplesPerMP3FrameIs576 = static_cast<bool>(iValue == 576);
		}

		int GetMP3FrameSize() const { return static_cast<int>(m_usMP3FrameSize); }
		void SetMP3FrameSize(int iValue)
		{
			Asura_Assert(iValue >= 0 && iValue < (1 << 15), "unexpected MP3FrameSize");
			m_usMP3FrameSize = static_cast<u_short>(iValue);
		}

		int GetSampleRate() const { return static_cast<int>(m_usSampleRate); }
		void SetSampleRate(int iValue)
		{
			Asura_Assert(iValue >= 0 && iValue < 65536, "unexpected sample rate");
			m_usSampleRate = static_cast<u_short>(iValue);
		}

		int GetSampleFormat() const { return static_cast<int>(m_uSampleFormat); }
		void SetSampleFormat(int iValue)
		{
			Asura_Assert(iValue >= 0 && iValue < (1<<3), "unexpected sample format");
			m_uSampleFormat = static_cast<u_int>(iValue);
		}

		int GetSampleBits() const { return 16; }
		void SetSampleBits(int iValue) { Asura_Assert(iValue == 16, "unexpected bits per sample"); }

		int GetChannelCount() const { return static_cast<int>(m_uChannelCount); }
		void SetChannelCount(int iValue)
		{
			Asura_Assert(iValue >= 0 && iValue < (1<<3), "unexpected channel count");
			m_uChannelCount = static_cast<u_int>(iValue);
		}

		int GetLoopOffset() const { return 0; }
		void SetLoopOffset(int iValue) { Asura_Assert(iValue == 0, "unexpected loop offset"); }

		int GetDataOffset() const { return 64; }
		void SetDataOffset(int iValue) { Asura_Assert(iValue == 64, "unexpected data offset"); }

		int GetTotalDataBytes() const { return static_cast<int>(m_uTotalDataBytes); }
		void SetTotalDataBytes(int iValue)
		{
			Asura_Assert(iValue >= 0 && iValue < (1<<26), "unexpected channel count");
			m_uTotalDataBytes = static_cast<u_int>(iValue);
		}

		int GetBitRate() const { return m_uBitRate; }
		void SetBitRate( const u_int uBitRate )
		{
			Asura_Assert( uBitRate <= 511, "unexpected bitrate" );
			m_uBitRate = uBitRate;
		}

		int GetBytesPerSample () const
		{
			return (GetSampleBits() >> 3);
		}

	private:

		bool	m_bSamplesPerMP3FrameIs576;	//	: 1		;	// 576 or 1152.
		u_short	m_usMP3FrameSize				: 15	;	// packet size. dependent on bit-rate. Typically around 420.
		u_short	m_usSampleRate;				//	: 16	;	// 0-48000 (or a bit more once we've accounted for looping MP3s adjusting sample rate)
		u_int	m_uSampleFormat					: 3		;	// 0-7
		u_int	m_uChannelCount					: 3		;	// 0-7
		u_int	m_uTotalDataBytes				: 26	;
		u_int	m_uBitRate						: 9		;	// up to 320kbps
};

//___________________________________________________________________________________________________

class Asura_MultiStream_Buffer
{
	public:

		Asura_MultiStream_Buffer		();
		~Asura_MultiStream_Buffer		();

		bool	Load					( Asura_Chunk_Stream& xStream, const char* const szFilename, const u_int uFilesize );
		
		void	SetAsLooping			()			{ m_bLooping = true;	}

		bool	IsLoaded				() const	{ return (m_pcWaveData != NULL);			}
		bool	IsLooping				() const	{ return m_bLooping;						}
		int		GetSampleRate			() const	{ return m_xSoundInfo.GetSampleRate();		}
		char*	GetSampleData			() const	{ return m_pcWaveData;						}
		u_int	GetSampleDataSize		() const	{ return m_xSoundInfo.GetTotalDataBytes();	}
		int		GetNumChannels			() const	{ return m_xSoundInfo.GetChannelCount();	}
		char*	GetFilename				() const	{ return m_szFilename;						}
		int		GetSampleFormat			() const	{ return m_xSoundInfo.GetSampleFormat();	}
		int		GetNumSamples			() const	{ return m_xSoundInfo.GetTotalDataBytes() / (m_xSoundInfo.GetSampleBits() >> 3); } 
		
	private:

		char*					m_pcWaveData;  // Stored waveform samples
		Asura_PS3_Sound_Info	m_xSoundInfo;
		
		bool					m_bLooping;

		char*					m_szFilename;
		Asura_Hash_ID			m_xFilenameHash;

		Asura_MultiStream_Buffer(const Asura_MultiStream_Buffer &xCopy);
		Asura_MultiStream_Buffer& operator=(const Asura_MultiStream_Buffer &xCopy);
};

//___________________________________________________________________________________________________

class Asura_MultiStream_MP3_Stream
{
	public:

		// ------------------ Enumerations ----------------------

		enum  StreamState
		{
			STREAM_STATE_NEED_PREPARE	= 0,	// Waiting to be primed
			STREAM_STATE_PREPARING		,		// In the process of priming
			STREAM_STATE_IDLE			,		// Doing nothing
			STREAM_STATE_READING		,		// Reading from file
			STREAM_STATE_LOADED			,		// Finished reading, active buffer contains new data
			STREAM_STATE_COMPLETED		,		// Reached the end of the file

			STREAM_STATE_ERROR					// There was an error
		};

		// ------------------- Functions ------------------------

		Asura_MultiStream_MP3_Stream		( const Asura_Hash_ID xWavHash );
		~Asura_MultiStream_MP3_Stream		();

		bool			GetBuffer			( char* &pcBuffer, u_int &uBufferSize, bool &bDiscontinuous, float &fBufferStartTime );
		
		void			SetAsLooping		()			{ m_bLooping = true;	}

		bool			IsLooping			() const	{ return m_bLooping;							}
		u_int			GetSampleDataSize	() const	{ return m_xSoundInfo.GetTotalDataBytes();		}
		u_int			GetSampleDataOffset	() const	{ return m_xSoundInfo.GetDataOffset();			}
		int				GetNumChannels		() const	{ return m_xSoundInfo.GetChannelCount();		}
		u_int			GetBytesPerSample	() const	{ return m_xSoundInfo.GetBytesPerSample();		}
		const char*		GetFilename			() const	{ return m_szFilename;							}
		int				GetSampleFormat		() const	{ return m_xSoundInfo.GetSampleFormat();		}
		int				GetSampleRate		() const	{ return m_xSoundInfo.GetSampleRate();			}
		StreamState		GetState			() const	{ return m_eState;								}
		int				GetBitRate			() const	{ return m_xSoundInfo.GetBitRate();				}
		bool			IsPreparing			() const	{ return m_eState == STREAM_STATE_PREPARING;	}
		int				GetNumSamples		() const	{ return m_xSoundInfo.GetTotalDataBytes() / (m_xSoundInfo.GetSampleBits() >> 3); } 

		bool			Prepare				();
		void			Read				( const bool bThreaded = true );
		void			SeekNextRead		( const float fSeekTime );
		void			SetNextReadSize		( const float fDuration );
		void			CleanNextRead		();
		bool			SeekToTimeInFile	( const float fSeekTime );

		static int		GetDefaultBufferSize	();

		float			GetReadPosition			( int iBufferOffset );

		void			IncrementProcessingBuffer ()	{ m_iProcessingBuffer = ( m_iProcessingBuffer + 1 ) % ASURA_PS3_MULTISTREAM_MP3_STREAM_BUFFERS; }
		
	private:

		// ------------------- Variables ------------------------

		int						m_iFilehandle;
		volatile StreamState	m_eState;
		
		char*					m_apStreamBuffer			[ASURA_PS3_MULTISTREAM_MP3_STREAM_BUFFERS];	// Audio data
		u_int					m_auStreamBufferSize		[ASURA_PS3_MULTISTREAM_MP3_STREAM_BUFFERS];	// Audio data size
		bool					m_bStreamBufferDiscontinuous[ASURA_PS3_MULTISTREAM_MP3_STREAM_BUFFERS];	// Does audio data follow from previous buffer?
		float					m_afStreamBufferTimes		[ASURA_PS3_MULTISTREAM_MP3_STREAM_BUFFERS];	// What time in the MP3 is each buffer
		int						m_iActiveBuffer;													// Which buffer are we writing to?
		int						m_iProcessingBuffer;												// Which buffer is being processed by multistream?
		int						m_iBufferReadSize;

		char*					m_szFilename;
		Asura_Hash_ID			m_xFilenameHash;
		Asura_PS3_Sound_Info	m_xSoundInfo;
		u_int					m_uFrameSyncBytes;

		bool					m_bShouldSeek;
		float					m_fSeekTime;
		bool					m_bShouldCleanRead;

		bool					m_bLooping;
		bool					m_bCompleted;

		Asura_Audio_StreamInfo*	m_pxStreamInfo;

		// ------------------- Functions ------------------------
		
		inline bool IsMP3FrameStart( const u_int uBytes );

		float GetMP3PlayPosFromFileOffset ( const int iFileOffset ) const;

		static void ThreadFunc_LoadData ( void* pxData, u_int uDataSize );

		Asura_MultiStream_MP3_Stream(const Asura_MultiStream_MP3_Stream &xCopy);
		Asura_MultiStream_MP3_Stream& operator=(const Asura_MultiStream_MP3_Stream &xCopy);
};

//___________________________________________________________________________________________________

class Asura_MultiStream_Custom_Stream
{
	// This class isn't strictly speaking 'file loading' but it is analogous to it

	public:

		// ------------------ Enumerations ----------------------

		enum  StreamState
		{
			STREAM_STATE_IDLE			,		// Doing nothing
			STREAM_STATE_READING		,		// Grabbing data
			STREAM_STATE_LOADED			,		// Finished grabbing data, active buffer contains new data
			STREAM_STATE_COMPLETED				// No more data
		};

		// ------------------- Functions ------------------------

		Asura_MultiStream_Custom_Stream			( const Asura_PS3_Audio_MoreDataFunc pfnMoreDataCallback, void* pMoreDataCallbackArg );
		~Asura_MultiStream_Custom_Stream		();

		bool			GetBuffer				( char* &pcBuffer, u_int &uBufferSize );

		void			Read					( const bool bThreaded = true );
		
		static int		GetDefaultBufferSize	();

		StreamState		GetState				() const	{ return m_eState; }

	private:

		// ------------------- Variables ------------------------

		Asura_PS3_Audio_MoreDataFunc	m_pfnMoreDataCallback;
		void*							m_pMoreDataCallbackArg;

		volatile StreamState			m_eState;

		char*							m_apStreamBuffer			[ASURA_PS3_MULTISTREAM_CUSTOM_STREAM_BUFFERS];	// Audio data
		u_int							m_auStreamBufferSize		[ASURA_PS3_MULTISTREAM_CUSTOM_STREAM_BUFFERS];	// Audio data size
		int								m_iActiveBuffer;															// Which buffer are we writing to?
		
		bool							m_bCompleted;

		// ------------------- Functions ------------------------

		static void ThreadFunc_GetData ( void* pxData, u_int uDataSize );

		Asura_MultiStream_Custom_Stream(const Asura_MultiStream_MP3_Stream &xCopy);
		Asura_MultiStream_Custom_Stream& operator=(const Asura_MultiStream_MP3_Stream &xCopy);
};

#endif	//ASURA_USE_AUDIO_SYSTEM

#endif	//__ASURA_PS3_MULTISTREAM_BUFFER_H_

//eof
