///////////////////////////////////////////////////////////////////////////////
//
// File      : Asura_PS3_ReplaceableSounds.h
// Author    : Philip Hall
// Created   : 17/09/2007
//
// Copyright (C) 2007 Rebellion. All Rights Reserved
//
///////////////////////////////////////////////////////////////////////////////

#ifndef __ASURA_PS3_REPLACEABLESOUNDS_H__
#define __ASURA_PS3_REPLACEABLESOUNDS_H__

#ifndef ASURA_USE_AUDIO_SYSTEM

// Asura headers
#include "Core/Asura_PlatformBasics.h"

// Asura PS3 headers
#include "Sound/Asura_PS3_Sound_Common.h"


static const u_int ASURA_PS3_SOUND_INVALID_BYTE_OFFSET = ~0;

////PGH/////////////////////////////////////////
// PS3 replaceable sound stub             
////////////////////////////////////////////////
class Asura_PS3_ReplaceableSound
{
public:
	Asura_PS3_ReplaceableSound();

	void	Create(u_int uOffset, u_int uSize) { m_uByteOffset = uOffset; m_uSoundSize = uSize; }
	bool	IsLoaded() const { return m_bLoaded; }
	bool	IsValid() const  { return (m_uSoundSize != 0) && (m_uByteOffset != ASURA_PS3_SOUND_INVALID_BYTE_OFFSET); }

	void	SetLoaded(bool bLoaded) { m_bLoaded = bLoaded; }
	u_int	GetOffset() const { return m_uByteOffset; }
	u_int	GetSize() const { return m_uSoundSize;  }

protected:
	u_int   m_uByteOffset;
	u_int   m_uSoundSize;
	bool	m_bLoaded;
	
	ASURA_DECLARE_NO_COPYING(Asura_PS3_ReplaceableSound)
};
				   

////PGH/////////////////////////////////////////
// PS3 replaceable sound subsystem                                          
////////////////////////////////////////////////
class Asura_PS3_ReplaceableSound_System
{
public:
	Asura_PS3_ReplaceableSound_System();

	bool Initialise( const char* szFilename );
	void Shutdown();
	void Reset();
	bool LoadReplaceableSound( int iLoadedSoundID );
	bool ReplaceSound( int iExistingSoundLoadedID, int iNewSoundLoadedID );
	bool AddReplaceableSoundInfo( int iSoundId, u_int uiByteOffset, u_int uiSize );

	int	GetMaxActiveReplaceableSounds() const { return ASURA_MAX_NUMBER_OF_LOADED_SOUNDS; }
	int	GetReplaceableSoundSize(int iLoadedSoundID) { return m_axReplaceableSounds[iLoadedSoundID].GetSize(); }

	static Asura_PS3_ReplaceableSound_System& GetAPI() { return s_xAPI; }

protected:
	char						m_szReplaceableSoundFile[ASURA_PS3_MAX_PATH];
	Asura_PS3_ReplaceableSound	m_axReplaceableSounds[ASURA_MAX_NUMBER_OF_LOADED_SOUNDS];

	static Asura_PS3_ReplaceableSound_System s_xAPI;
	
	ASURA_DECLARE_NO_COPYING(Asura_PS3_ReplaceableSound_System)
};

#endif // #ifndef ASURA_USE_AUDIO_SYSTEM

#endif

//eof
