///////////////////////////////////////////////////////////////////////////////
//
// File      : Asura_PS3_Sound_Common.h
// Author    : Philip Hall
// Created   : 12/09/2007
// Revision  : $Revision: #4 $
// Updated   : $Date: 2007/10/01 $  by  $Author: Philip.Hall $
//
// Copyright (C) 2007 Rebellion. All Rights Reserved
//
///////////////////////////////////////////////////////////////////////////////
#ifndef __ASURA_PS3_SOUND_COMMON_H__
#define __ASURA_PS3_SOUND_COMMON_H__

#ifndef ASURA_USE_AUDIO_SYSTEM

// Asura headers
#include "Core/Memory/Asura_Mem.h"


namespace Asura_Maths
{
	static inline bool Equal( float fA, float fB, float fTolerance=0.0000001f)
	{
		return Asura_Maths::Abs(fA - fB) < fTolerance;
	}
}

////PGH/////////////////////////////////////////
// PS3 active sound common data (derive from this)
////////////////////////////////////////////////
class Asura_PS3_ActiveSound_Common
{
public:
	float			m_fVolume;
	float			m_fDryVolumeScale;
	float			m_fWetVolumeScale;
	float			m_fPitch;
	float			m_fPan;
	float			m_fFade;
	bool			m_bUseEQ;
	int				m_iLoadedSoundIndex;
	bool			m_bLooping;
	mutable bool	m_bPlaying;	 // Cached 'playing' state
	bool			m_bDecaying;
	bool			m_bPending;
	bool            m_bChanged;
	bool            m_bSpatial;
	Asura_Vector_3  m_xPosition;
	float			m_fInnerRadius;
	float			m_fOuterRadius;
	bool			m_bUseDistanceFilter;

	Asura_PS3_ActiveSound_Common()
	{
		m_iLoadedSoundIndex = ASURA_INVALID_HANDLE;
		m_bPlaying = false;
		m_bDecaying = false;
		m_bLooping = false;
		m_fVolume  = 1.0f;
		m_fDryVolumeScale = 1.0f;
		m_fWetVolumeScale = 1.0f;
		m_fPitch = 1.0f;
		m_fPan = 0.0f;
		m_fFade = 1.0f;
		m_bSpatial = false;
		m_bUseEQ = false;
		m_bPending = false;
		m_bUseDistanceFilter = false;
		m_fInnerRadius = m_fOuterRadius = 0.0f;
	}

	void SetRadii( float fInner, float fOuter )
	{
		m_fInnerRadius = fInner;
		m_fOuterRadius = fOuter;
		m_bChanged = true;
	}

	void SetVolume( float fVolume )
	{
		m_bChanged = m_bChanged || !Asura_Maths::Equal(m_fVolume, fVolume);
		m_fVolume = fVolume;
	}

	void SetPan( float fPan )
	{
		m_bChanged = m_bChanged || !Asura_Maths::Equal(m_fPan, fPan);
		m_fPan = fPan;
	}

	void SetFade( float fFade )
	{
		m_bChanged = m_bChanged || !Asura_Maths::Equal(m_fFade, fFade);
		m_fFade = fFade;
	}

	void SetPitch( float fPitch )
	{
		m_bChanged = m_bChanged || !Asura_Maths::Equal(m_fPitch, fPitch);
		m_fPitch = fPitch;
	}

	void SetPosition( const Asura_Vector_3& xPosition )
	{
		m_xPosition = xPosition;
		m_bChanged = true;
	}

	void SetSpatial( bool bIsSpatial )
	{
		m_bSpatial = bIsSpatial;
		m_bChanged = true;
	}

	void SetUseDistanceFilter( bool bUseFilter )
	{
		m_bUseDistanceFilter = bUseFilter;
		m_bChanged = true;
	}

	void SetPending(bool bPending)
	{
		m_bPending = bPending;
	}
	
	void SetWetDryVolume(float fWetVolumeScale, float fDryVolumeScale)
	{
		m_bChanged = m_bChanged || !Asura_Maths::Equal(m_fDryVolumeScale, fDryVolumeScale) || !Asura_Maths::Equal(m_fWetVolumeScale, fWetVolumeScale);
		m_fDryVolumeScale = fDryVolumeScale;
		m_fWetVolumeScale = fWetVolumeScale;
	}

	void SetLooping(bool bLooping)
	{
		m_bLooping = bLooping;
	}
};


// Convenience anti-copying macro
#define ASURA_DECLARE_NO_COPYING(__class__) \
	private: __class__(const __class__ &second); \
	private: __class__& operator=(const __class__ &second);


#endif

#endif // #ifndef ASURA_USE_AUDIO_SYSTEM

//eof
