////////////////////////////////////////////////////////////////////////////////
//
// File   : Asura_PS3_GameContent.cpp
// Author : Anthony Gregson
//
// Copyright (C) 2009 Rebellion, All Rights Reserved.
//
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include Files...
////////////////////////////////////////////////////////////////////////////////
#include "Core/Asura.h"

// Own header
#include "Utility/Asura_PS3_GameContent.h"

// Asura independent
#include "Core/Asura_String.h"
#include "Debugging/Asura_Assert.h"

// Asura PS3
#include "Utility/Asura_PS3_DiscBootGame.h"

// Library headers
#include <cell/sysmodule.h>
#include <sysutil/sysutil_gamecontent.h>

static const u_int uASURA_PS3_GAMECONTENT_DIRECTORY_NAME_MAX_LENGTH = 32;

bool Asura_PS3_GameContent::s_bInitialised = false;

// Initialisation functions - handles loading and unloading the module.
void Asura_PS3_GameContent::Initialise()
{
	int iRet = cellSysmoduleLoadModule(CELL_SYSMODULE_SYSUTIL_GAME);
	if (iRet == CELL_OK)
	{
		s_bInitialised = true;
	}
	else
	{
		Asura_Assertf(false, ("cellSysmoduleLoadModule failed to load sysutil_game (0x%x)", iRet));
	}
}

void Asura_PS3_GameContent::Shutdown()
{
	if (s_bInitialised)
	{
		int iRet = cellSysmoduleUnloadModule(CELL_SYSMODULE_SYSUTIL_GAME);
		Asura_Assertf(iRet == CELL_OK, ("cellSysmoduleUnloadModule failed to unload sysutil_game (0x%x)", iRet));

		s_bInitialised = false;
	}
}

// Static creation functions
Asura_PS3_GameContent* Asura_PS3_GameContent::CreateDiscContent()
{
	Asura_Assert(false, "Disc content creation not implemented.");
	return NULL;
}

Asura_PS3_GameContent* Asura_PS3_GameContent::CreateHDDContent()
{
	Asura_Assert(false, "HDD content creation not implemented.");
	return NULL;
}

Asura_PS3_GameContent* Asura_PS3_GameContent::CreateGameDataContent(const char* szDirectoryNameSuffix/* = NULL*/)
{
	char acBuffer[uASURA_PS3_GAMECONTENT_DIRECTORY_NAME_MAX_LENGTH];
	if (szDirectoryNameSuffix)
	{
		Asura_PrintfToBfr(acBuffer, "%s%s", uASURA_PS3_GAMECONTENT_DIRECTORY_NAME_MAX_LENGTH, true, Asura_PS3_DiscBootGame::GetLeadProductCode(), szDirectoryNameSuffix);
	}
	else
	{
		Asura_PrintfToBfr(acBuffer, "%s", uASURA_PS3_GAMECONTENT_DIRECTORY_NAME_MAX_LENGTH, true, Asura_PS3_DiscBootGame::GetLeadProductCode());
	}

	// Do the check before we construct the instance, in case there's no reason to create it.
	u_int uType = CELL_GAME_GAMETYPE_GAMEDATA;
	CellGameContentSize xSize;
	memset(&xSize, 0, sizeof(CellGameContentSize));
	int iRet = cellGameDataCheck(uType, acBuffer, &xSize);
	Asura_Assertf((iRet == CELL_GAME_RET_OK) || (iRet == CELL_GAME_RET_NONE), ("cellGameDataCheck failed (0x%x)", iRet)); // TODO - handle broken data case.

	Asura_PS3_GameContent* pxContent = NULL;

	if (iRet == CELL_GAME_RET_OK)
	{
		pxContent = new Asura_PS3_GameContent(uType);
	}
	else if (iRet == CELL_GAME_RET_NONE)
	{
		// Not sure this is needed if the content isn't there, but to be on the safe side...
		char acTempBuffer1[ASURA_PS3_CONTENT_DIRECTORY_BUFFER_SIZE];
		char acTempBuffer2[ASURA_PS3_CONTENT_DIRECTORY_BUFFER_SIZE];
		iRet = cellGameContentPermit(acTempBuffer1, acTempBuffer2);
		Asura_Assertf(iRet == CELL_OK, ("cellGameContentPermit failed (0x%x)", iRet));
	}

	return pxContent;
}

// Static destruction function (to match the constructor being wrapped by creation functions).
void Asura_PS3_GameContent::DestroyContent(Asura_PS3_GameContent* pxContentToDestroy)
{
	delete pxContentToDestroy;
}

Asura_PS3_GameContent::Asura_PS3_GameContent(u_int uType)
{
	m_acContentInformationDirectoryBuffer[0] = '\0';
	m_acUsrDirectoryBuffer[0] = '\0';

	if (uType == CELL_GAME_GAMETYPE_GAMEDATA)
	{
		// The check has been done by the create call, just do the permit call.
		int iRet = cellGameContentPermit(m_acContentInformationDirectoryBuffer, m_acUsrDirectoryBuffer);
		Asura_Assertf(iRet == CELL_OK, ("cellGameContentPermit failed (0x%x)", iRet));

		if (iRet != CELL_OK)
		{
			// Make sure we don't try to access anything through this content (TODO - this may not be needed as the directories may not be changed if the function fails).
			m_acContentInformationDirectoryBuffer[0] = '\0';
			m_acUsrDirectoryBuffer[0] = '\0';
		}
	}
}

Asura_PS3_GameContent::~Asura_PS3_GameContent()
{
}
