////////////////////////////////////////////////////////////////////////////////
//
// File   : Asura_PS3_SaveLoad.cpp
// Author : Neil Young
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include Files...
////////////////////////////////////////////////////////////////////////////////
#include "Core/Asura.h"

#include "Utility/Asura_PS3_SaveLoad.h"

#ifdef ASURA_PS3_SAVEGAMES

#include "ChunkSystem/Asura_SaveBlock.h"
#include "Core/Asura_Maths.h"
#include "Core/Asura_String.h"
#include "Core/Memory/Asura_Mem.h"
#include "Core/Threads/Asura_ProducerConsumerQueue.h"
#include "Debugging/Asura_Logfile.h"
#include "FileAccess/Asura_FileAccess.h"
#include "FileAccess/Asura_PS3_FileSystem.h"
#include "Rendering/Asura_PS3_Graphics.h"

char Asura_PS3_SaveLoad::s_szProjectDirName[uPROJECTDIRNAMELENGTH] = "";
char Asura_PS3_SaveLoad::s_szAutosaveFilename[CELL_SAVEDATA_DIRNAME_SIZE] = "Test string";

u_int Asura_PS3_SaveLoad::s_uSizeOfSaveBlock = 0;
Asura_ProtoSaveBlock* Asura_PS3_SaveLoad::s_pxSaveBlock = 0;

const Asura_PS3_SaveLoad::FILE_INDEX Asura_PS3_SaveLoad::s_aeFileIndexForSaveStep[NUM_SAVE_OPERATION_STEPS] =
{
		FILE_INDEX_ICON0,			//SAVE_OPERATION_STEP_ICON0
		FILE_INDEX_ICON1,			//SAVE_OPERATION_STEP_ICON1
		FILE_INDEX_PIC1,			//SAVE_OPERATION_STEP_PIC1
		FILE_INDEX_SND0,			//SAVE_OPERATION_STEP_SND0
		FILE_INDEX_PROFILE,			//SAVE_OPERATION_STEP_PROFILE
		FILE_INDEX_SAVEBLOCKHEADER,	//SAVE_OPERATION_STEP_SAVEBLOCKHEADER
		FILE_INDEX_SAVEBLOCK,		//SAVE_OPERATION_STEP_SAVEBLOCK
		// NY - 13 May 2008 - return a valid file index for the end step, although it never accesses a file
		FILE_INDEX_PROFILE,			//SAVE_OPERATION_STEP_END
};

const Asura_PS3_SaveLoad::FILE_INDEX Asura_PS3_SaveLoad::s_aeFileIndexForLoadStep[NUM_LOAD_OPERATION_STEPS] =
{
		FILE_INDEX_PROFILE,			//LOAD_OPERATION_STEP_PROFILE
		FILE_INDEX_SAVEBLOCKHEADER,	//LOAD_OPERATION_STEP_SAVEBLOCKHEADER
		FILE_INDEX_SAVEBLOCK,		//LOAD_OPERATION_STEP_SAVEBLOCK
		// NY - 13 May 2008 - return a valid file index for the end step, although it never accesses a file
		FILE_INDEX_PROFILE,			//LOAD_OPERATION_STEP_END
};

const CellSaveDataFileType Asura_PS3_SaveLoad::s_aeFileType[FILE_INDEX_TOTAL] =
{
	CELL_SAVEDATA_FILETYPE_CONTENT_ICON0,	//FILE_INDEX_ICON0
	CELL_SAVEDATA_FILETYPE_CONTENT_ICON1,	//FILE_INDEX_ICON1
	CELL_SAVEDATA_FILETYPE_CONTENT_PIC1,	//FILE_INDEX_PIC1
	CELL_SAVEDATA_FILETYPE_CONTENT_SND0,	//FILE_INDEX_SND0
	CELL_SAVEDATA_FILETYPE_SECUREFILE,		//FILE_INDEX_PROFILE
	CELL_SAVEDATA_FILETYPE_SECUREFILE,		//FILE_INDEX_SAVEBLOCKHEADER
	CELL_SAVEDATA_FILETYPE_SECUREFILE,		//FILE_INDEX_SAVEBLOCK
};

void* Asura_PS3_SaveLoad::s_apFileBuffers[FILE_INDEX_TOTAL] =
{
	0,	//FILE_INDEX_ICON0
	0,	//FILE_INDEX_ICON1
	0,	//FILE_INDEX_PIC1
	0,	//FILE_INDEX_SND0
	0,	//FILE_INDEX_PROFILE
	0,	//FILE_INDEX_SAVEBLOCKHEADER
	0,	//FILE_INDEX_SAVEBLOCK
};
u_int Asura_PS3_SaveLoad::s_auFileBufferSizes[FILE_INDEX_TOTAL] =
{
	0,	//FILE_INDEX_ICON0
	0,	//FILE_INDEX_ICON1
	0,	//FILE_INDEX_PIC1
	0,	//FILE_INDEX_SND0
	0,	//FILE_INDEX_PROFILE
	0,	//FILE_INDEX_SAVEBLOCKHEADER
	0,	//FILE_INDEX_SAVEBLOCK
};
bool Asura_PS3_SaveLoad::s_abExistingFiles[FILE_INDEX_TOTAL] =
{
	false,	//FILE_INDEX_ICON0
	false,	//FILE_INDEX_ICON1
	false,	//FILE_INDEX_PIC1
	false,	//FILE_INDEX_SNDfalse
	false,	//FILE_INDEX_PROFILE
	false,	//FILE_INDEX_SAVEBLOCKHEADER
	false,	//FILE_INDEX_SAVEBLOCK
};

void* Asura_PS3_SaveLoad::s_pNewIconBuffer = 0;
u_int Asura_PS3_SaveLoad::s_uNewIconBufferSize = 0;

bool Asura_PS3_SaveLoad::s_bBlankSlotOnLoad = false;
bool Asura_PS3_SaveLoad::s_bDontOverwriteSaveBlock = false;

bool Asura_PS3_SaveLoad::s_bFreeSpaceFailOnLastSave = false;
bool Asura_PS3_SaveLoad::s_bErrorOnLastOperation = false;
bool Asura_PS3_SaveLoad::s_bLastOperationCanceled = false;

u_int Asura_PS3_SaveLoad::s_uLastProfileCountResult = 0;

char Asura_PS3_SaveLoad::s_szNewDataTitle[CELL_SAVEDATA_SYSP_TITLE_SIZE] = "";
CellSaveDataListNewData Asura_PS3_SaveLoad::s_xNewData;
CellSaveDataNewDataIcon Asura_PS3_SaveLoad::s_xNewDataIcon;
char Asura_PS3_SaveLoad::s_szNewDataDirName[CELL_SAVEDATA_DIRNAME_SIZE] = "";

u_int Asura_PS3_SaveLoad::s_uSaveStep = 0;
u_int Asura_PS3_SaveLoad::s_uLoadStep = 0;
bool Asura_PS3_SaveLoad::s_bCanAutosave = false;
bool Asura_PS3_SaveLoad::s_bIsAutosave = false;
Asura_PS3_SaveLoad::SAVELOAD_STATUS Asura_PS3_SaveLoad::s_eStatus = STATUS_IDLE;

bool (*Asura_PS3_SaveLoad::s_pfnCallback)(const void* pData) = 0;

const u_int Asura_PS3_SaveLoad::s_uMaxFiles = FILE_INDEX_TOTAL;
void* Asura_PS3_SaveLoad::s_pListBuffer = 0;
u_int Asura_PS3_SaveLoad::s_uListBufferSize = 0;

char Asura_PS3_SaveLoad::s_szTitle[CELL_SAVEDATA_SYSP_TITLE_SIZE] = "";
char Asura_PS3_SaveLoad::s_szSubTitle[CELL_SAVEDATA_SYSP_SUBTITLE_SIZE] = "";
char Asura_PS3_SaveLoad::s_szDetail[CELL_SAVEDATA_SYSP_DETAIL_SIZE] = "";

// the PS3 is able to create secure files to prevent the files being read on non PS3 systems and 
// to detect unauthorised edits. This is the Id used to encrypt the files. There is a cap of 113 encrypted files (CELL_SAVEDATA_SECUREFILE_MAX)
// in any one save, but I doubt we'll ever hit that; 
const char* Asura_PS3_SaveLoad::s_szSecureFileId = "1234567890123456";// NY - 13 March 2008 - this is 17 bytes long (16+terminator), but copying from it will ignore the terminator
u_int Asura_PS3_SaveLoad::s_uNumCallBacksForBar = 1;//for progress bar maths
Asura_ProducerConsumerQueue* Asura_PS3_SaveLoad::s_pxProducerConsumerQueue = 0;

// (NY - 18.02.2008) I would love to make these pointers to const but sony want non const, and this seems better than const casting...
static char* szASURA_PS3_NEWDATA_NAME = "/misc/NEWDATA.png";
static char* szASURA_PS3_ICON0_NAME = "/misc/ICON0.png";
static char* szASURA_PS3_ICON1_NAME = "/misc/ICON1.pam";
static char* szASURA_PS3_PIC1_NAME = "/misc/PIC1.png";
static char* szASURA_PS3_SND0_NAME = "/misc/SND0.at3";
static char* szASURA_PS3_PROFILE_NAME = "PROFILE";
static char* szASURA_PS3_SAVEBLOCKHEADER_NAME = "SAVEBLKH";
static char* szASURA_PS3_SAVEBLOCK_NAME = "SAVEBLK";
char* Asura_PS3_SaveLoad::s_aszFileNames[FILE_INDEX_TOTAL] =
{
	szASURA_PS3_ICON0_NAME,	//FILE_INDEX_ICON0
	szASURA_PS3_ICON1_NAME,	//FILE_INDEX_ICON1
	szASURA_PS3_PIC1_NAME,	//FILE_INDEX_PIC1
	szASURA_PS3_SND0_NAME,	//FILE_INDEX_SND0
	szASURA_PS3_PROFILE_NAME,	//FILE_INDEX_PROFILE
	szASURA_PS3_SAVEBLOCKHEADER_NAME,	//FILE_INDEX_SAVEBLOCKHEADER
	szASURA_PS3_SAVEBLOCK_NAME,	//FILE_INDEX_SAVEBLOCK
};

void Asura_PS3_SaveLoad::Initialise()
{
	s_pxProducerConsumerQueue = new Asura_ProducerConsumerQueue(1, 1, ASURA_MULTITHREAD_STACK_SIZE_SMALL);
	LoadCIFiles();
	SetupNewDataIcon();
	s_uListBufferSize = Asura_Maths::Max( MAX_SAVE_GAMES * sizeof( CellSaveDataDirList ), s_uMaxFiles * sizeof( CellSaveDataFileStat ) );
	s_pListBuffer = Asura_Allocate( s_uListBufferSize );
}

void Asura_PS3_SaveLoad::Shutdown()
{
	Asura_Assert(!IsBusy(), "shutting down PS3 save data while it's busy");
	delete s_pxProducerConsumerQueue;
	s_pxProducerConsumerQueue = 0;
	UnloadCIFiles();
	UnloadNewDataIcon();
	Asura_Deallocate(s_pListBuffer);
}

// SaveProfile saves a buffer of size uSizeOfData starting at address pData. The created files in the
// Save Data location will be the profile file, and any content files (icons etc.) specified. It will attempt to
// autosave if a save data has been specified by a previous save or load operation, otherwise it
// will ask the user to choose a save data to save to. Returns false if save thread cannot be created.
bool Asura_PS3_SaveLoad::SaveProfile( void* pData, u_int uSizeOfData, Asura_ProtoSaveBlock* pxSaveBlock /*= 0*/ )
{
	if (!s_pxProducerConsumerQueue)
	{
		Asura_Assert(0, "Asura_PS3_SaveLoad is uninitialised");
		return false;
	}
	if (s_szProjectDirName[0] == 0)
	{
		Asura_Assert(0, "s_szProjectDirName is unset");
		return false;
	}
	// make sure we are not busy at the moment
	if(IsBusy())
	{
		Asura_OutputToDebugger_Retail( ("Save Utility is busy\n") );
		return false;
	}

	s_auFileBufferSizes[FILE_INDEX_PROFILE] = uSizeOfData;
	s_apFileBuffers[FILE_INDEX_PROFILE] = pData;

	s_uSizeOfSaveBlock = 0;
	s_pxSaveBlock = pxSaveBlock;

	s_bFreeSpaceFailOnLastSave = false;
	
	// reset SaveStep, used to keep track of how far through the save operation we are
	s_uSaveStep = 0;

	s_eStatus = STATUS_SAVING;
	// do we have an autosave slot chosen
	if( s_bCanAutosave )
	{
		s_pxProducerConsumerQueue->AddToQueue(Thread_Autosave, 0, 0);
	}
	else
	{
		s_pxProducerConsumerQueue->AddToQueue(Thread_Save, 0, 0);
	}

	return true;
}

// ManualSaveProfile saves a buffer of size uSizeOfData starting at address pData to 
// a location chosen by the user. The created files in the Save Data location will be 
// the profile file, and any content files (icons etc.) specified.Returns false if save
// thread cannot be created.
bool Asura_PS3_SaveLoad::ManualSaveProfile( void* pData, u_int uSizeOfData, Asura_ProtoSaveBlock* pxSaveBlock /*= 0*/ )
{
	if (!s_pxProducerConsumerQueue)
	{
		Asura_Assert(0, "Asura_PS3_SaveLoad is uninitialised");
		return false;
	}
	if (s_szProjectDirName[0] == 0)
	{
		Asura_Assert(0, "s_szProjectDirName is unset");
		return false;
	}
	// make sure we are not busy at the moment
	if(IsBusy())
	{
		Asura_OutputToDebugger_Retail( ("Save Utility is busy\n") );
		return false;
	}

	s_auFileBufferSizes[FILE_INDEX_PROFILE] = uSizeOfData;
	s_apFileBuffers[FILE_INDEX_PROFILE] = pData;

	s_uSizeOfSaveBlock = 0;
	s_pxSaveBlock = pxSaveBlock;

	s_bFreeSpaceFailOnLastSave = false;

	// reset SaveStep, which keeps track of how far through the save operation we are
	s_eStatus = STATUS_SAVING;
	s_uSaveStep = 0;
	s_pxProducerConsumerQueue->AddToQueue(Thread_Save, 0, 0);

	return true;
}

void Asura_PS3_SaveLoad::SetProfileCheckCallback(bool (*pfnCallback)(const void* pData))
{
	s_pfnCallback = pfnCallback;
}

// LoadProfile loads a buffer of size uSizeOfData starting at address pData to 
// a location chosen by the user. The loaded file will be the profile file.
// Returns false if load thread cannot be created.
bool Asura_PS3_SaveLoad::LoadProfile( void* pData, u_int uSizeOfData, Asura_ProtoSaveBlock* pxSaveBlock /*= 0*/ )
{
	if (!s_pxProducerConsumerQueue)
	{
		Asura_Assert(0, "Asura_PS3_SaveLoad is uninitialised");
		return false;
	}
	if (s_szProjectDirName[0] == 0)
	{
		Asura_Assert(0, "s_szProjectDirName is unset");
		return false;
	}
	// make sure we are not busy at the moment
	if(IsBusy())
	{
		return false;
	}

	s_uSizeOfSaveBlock = 0;
	s_pxSaveBlock = pxSaveBlock;

	// reset LoadStep, which keeps track of how far through the load operation we are
	s_uLoadStep = 0;
	s_uSaveStep = 0;

	s_uLastProfileCountResult = 0;

	s_apFileBuffers[FILE_INDEX_PROFILE] = pData;
	s_auFileBufferSizes[FILE_INDEX_PROFILE] = uSizeOfData;

	s_eStatus = STATUS_LOADING;

	s_pxProducerConsumerQueue->AddToQueue(Thread_Load, 0, 0);

	return true;
}

// Returns false if load thread cannot be created.
bool Asura_PS3_SaveLoad::CountProfiles()
{
	if (!s_pxProducerConsumerQueue)
	{
		Asura_Assert(0, "Asura_PS3_SaveLoad is uninitialised");
		return false;
	}
	if (s_szProjectDirName[0] == 0)
	{
		Asura_Assert(0, "s_szProjectDirName is unset");
		return false;
	}
	// make sure we are not busy at the moment
	if(IsBusy())
	{
		return false;
	}

	s_eStatus = STATUS_COUNTING;

	s_uLastProfileCountResult = 0;

	s_pxProducerConsumerQueue->AddToQueue(Thread_CountProfiles, 0, 0);

	return true;
}

// DeleteProfiles presents the user with a list of save data on the system (from all
// games) and allows them to choose save data to delete.
bool Asura_PS3_SaveLoad::DeleteProfiles()
{
	if (!s_pxProducerConsumerQueue)
	{
		Asura_Assert(0, "Asura_PS3_SaveLoad is uninitialised");
		return false;
	}
	// check we are not busy
	if(IsBusy())
	{
		Asura_OutputToDebugger_Retail( ("Save Utility Busy") );
		return false;
	}

	s_eStatus = STATUS_DELETING;

	s_pxProducerConsumerQueue->AddToQueue(Thread_DeleteProfiles, 0, 0);

	return true;
}

bool Asura_PS3_SaveLoad::IsBusy()
{
	if( s_eStatus == STATUS_SAVING
		|| s_eStatus == STATUS_LOADING
		|| s_eStatus == STATUS_DELETING
		|| s_eStatus == STATUS_COUNTING)
	{
		return true;
	}
	return false;
}

//helper functions
//convert bytes to k, rounding up - sony's official maths for this, ported from their #define
u_int Asura_PS3_SaveLoad::SizeInK(u_int uBytes)
{
	return (uBytes + 1023) / 1024;
}

//number of percentage points to bump progress bar by for one file
u_int Asura_PS3_SaveLoad::ProgressPercent()
{
	return Asura_Maths::Max(1, Asura_Maths::FloatToInt_Near(static_cast<float>(100) / static_cast<float>(s_uNumCallBacksForBar)));
}

//sets up blank/new data slot for list save (and in theory load)
void Asura_PS3_SaveLoad::SetupBlankSlot(CellSaveDataListGet* pxListGet, CellSaveDataListSet* pxListSet)
{
	// we do have space for a new save game, now we need to settle on a directory name for it
	// we have up to MAX_SAVE_GAMES games in total, so we should loop over the directories we have found and 
	// find a free number to use as the directory name
	// loop over all possible names for directories
	for(u_int u = 0; u < MAX_SAVE_GAMES; ++u )
	{
		// construct possible name from iterator value
		Asura_PrintfToBfr( s_szNewDataDirName, "%s_DIR%u", CELL_SAVEDATA_DIRNAME_SIZE, true, s_szProjectDirName, u );

		bool bFound = false;
		
		// loop over all actually found directories to see if current possible name is being used
		for(u_int v = 0; v < pxListGet->dirListNum; ++v )
		{
			if( strcmp( s_szNewDataDirName, pxListGet->dirList[v].dirName ) == 0 )
			{
				bFound = true;
				break;
			}
		}
		if( !bFound )
		{
			break;
		}
	}

	// set our newly created free title as the directory name to be used by the new save if the user selects it
	s_xNewData.dirName = s_szNewDataDirName;
	// put the create new save option at the top of the list of saved data
	s_xNewData.iconPosition = CELL_SAVEDATA_ICONPOS_HEAD;
	s_xNewData.reserved = NULL;

	SetupNewDataIcon();

	pxListSet->newData = &s_xNewData;
}

//load Content info files - return false for failures as these tend to indicate bd removal, which we need to abort on
bool Asura_PS3_SaveLoad::LoadCIFiles()
{
	for (u_int u = 0; u <= FILE_INDEX_LAST_CIF; ++u)
	{
		if( Asura_FileAccess::FileExists( s_aszFileNames[u] ) )
		{
			Asura_PS3_FileSystem::Handle xHandle;
			Asura_PS3_FileSystem::OpenForReading( s_aszFileNames[u], &xHandle );
			s_auFileBufferSizes[u] = Asura_PS3_FileSystem::GetFileSize( xHandle );
			s_apFileBuffers[u] = Asura_Allocate( s_auFileBufferSizes[u] );
			u_int uBytes = Asura_PS3_FileSystem::ReadData( xHandle, s_auFileBufferSizes[u], s_apFileBuffers[u] );
			Asura_PS3_FileSystem::Close( &xHandle );
			if (uBytes != s_auFileBufferSizes[u])
			{
				return false;
			}
		}
		else if (Asura_PS3_Graphics::IsInShutdown())
		{
			return false;
		}
		else if (u == FILE_INDEX_ICON0)
		{
			Asura_Assert(0, "can't find ICON0.png");
		}
	}
	return true;
}

void Asura_PS3_SaveLoad::SetupNewDataIcon()
{
	s_xNewData.icon = &s_xNewDataIcon;
	s_xNewDataIcon.title = s_szNewDataTitle;
	if (!s_pNewIconBuffer)
	{
		// get hold of the icon data from file
		Asura_PS3_FileSystem::Handle xHandle;
		if (Asura_PS3_FileSystem::OpenForReading( szASURA_PS3_NEWDATA_NAME, &xHandle ))
		{
			s_uNewIconBufferSize = Asura_PS3_FileSystem::GetFileSize( xHandle );
			s_pNewIconBuffer = Asura_Allocate( s_uNewIconBufferSize );
			Asura_PS3_FileSystem::ReadData( xHandle, s_uNewIconBufferSize, s_pNewIconBuffer );
			Asura_PS3_FileSystem::Close( &xHandle );
		}
	}

	Asura_Assert(s_pNewIconBuffer, "unable to load NEWDATA.PNG");

	s_xNewDataIcon.iconBuf = s_pNewIconBuffer;
	s_xNewDataIcon.iconBufSize = s_pNewIconBuffer?s_uNewIconBufferSize:0;
	s_xNewDataIcon.reserved = NULL;
}

void Asura_PS3_SaveLoad::UnloadCIFiles()
{
	for (u_int u = 0; u <= FILE_INDEX_LAST_CIF; ++u)
	{
		Asura_Deallocate( s_apFileBuffers[u] );
		s_apFileBuffers[u] = NULL;
	}
}

void Asura_PS3_SaveLoad::UnloadNewDataIcon()
{
	Asura_Deallocate( s_pNewIconBuffer );
	s_pNewIconBuffer = NULL;
}

//determine how much space in k is needed (relative to what's available) for a new save
//values less than zero mean not enough free space
int Asura_PS3_SaveLoad::CalculateSizeNewData(CellSaveDataStatGet* pxStatGet)
{
	int iNewSize = SizeInK(s_auFileBufferSizes[FILE_INDEX_PROFILE]);
	for (u_int u = 0; u <= FILE_INDEX_LAST_CIF; ++u)
	{
		if( s_apFileBuffers[u] )
		{
			iNewSize += SizeInK(s_auFileBufferSizes[u]);
		}
	}
	if (s_pxSaveBlock)
	{
		iNewSize += SizeInK(s_pxSaveBlock->GetSpaceUsed() + s_pxSaveBlock->GetSpaceLeft()) + SizeInK(sizeof(u_int));
	}

	iNewSize += pxStatGet->sysSizeKB;
	return pxStatGet->hddFreeSizeKB - iNewSize;
}

//determine how much space in k is needed (relative to what's available) for overwriting
//values less than zero mean not enough free space
//this also flags which files current exist on the HDD
int Asura_PS3_SaveLoad::CalculateSizeOverwrite(CellSaveDataStatGet* pxStatGet)
{
	int iRunningSizeChange = 0;
	int iMaxSizeChange = 0;
	for (u_int u = 0; u < SAVE_OPERATION_STEP_END; ++u)
	{
		u_int uSize = 0;
		u_int uType = CELL_SAVEDATA_FILETYPE_SECUREFILE;
		const char* szName = 0;

		switch (u)
		{
			case SAVE_OPERATION_STEP_ICON0:
			{
				uType = CELL_SAVEDATA_FILETYPE_CONTENT_ICON0;
				szName = szASURA_PS3_ICON0_NAME;
				if (s_apFileBuffers[FILE_INDEX_ICON0])
				{
					uSize = s_auFileBufferSizes[FILE_INDEX_ICON0];
				}
				break;
			}
			case SAVE_OPERATION_STEP_ICON1:
			{
				uType = CELL_SAVEDATA_FILETYPE_CONTENT_ICON1;
				szName = szASURA_PS3_ICON1_NAME;
				if (s_apFileBuffers[FILE_INDEX_ICON1])
				{
					uSize = s_auFileBufferSizes[FILE_INDEX_ICON1];
				}
				break;
			}
			case SAVE_OPERATION_STEP_PIC1:
			{
				uType = CELL_SAVEDATA_FILETYPE_CONTENT_PIC1;
				szName = szASURA_PS3_PIC1_NAME;
				if (s_apFileBuffers[FILE_INDEX_PIC1])
				{
					uSize = s_auFileBufferSizes[FILE_INDEX_PIC1];
				}
				break;
			}
			case SAVE_OPERATION_STEP_SND0:
			{
				uType = CELL_SAVEDATA_FILETYPE_CONTENT_SND0;
				szName = szASURA_PS3_SND0_NAME;
				if (s_apFileBuffers[FILE_INDEX_SND0])
				{
					uSize = s_auFileBufferSizes[FILE_INDEX_SND0];
				}
				break;
			}
			case SAVE_OPERATION_STEP_PROFILE:
			{
				szName = szASURA_PS3_PROFILE_NAME;
				uSize = s_auFileBufferSizes[FILE_INDEX_PROFILE];
				break;
			}
			case SAVE_OPERATION_STEP_SAVEBLOCKHEADER:
			{
				szName = szASURA_PS3_SAVEBLOCKHEADER_NAME;
				if (s_pxSaveBlock)
				{
					uSize = sizeof(u_int);
				}
				break;
			}
			case SAVE_OPERATION_STEP_SAVEBLOCK:
			{
				szName = szASURA_PS3_SAVEBLOCK_NAME;
				if (s_pxSaveBlock)
				{
					uSize = s_pxSaveBlock->GetSpaceUsed() + s_pxSaveBlock->GetSpaceLeft();
				}
				break;
			}
			default:
			{
				Asura_Assert(0, "unhandled ps3 save step");
				break;
			}
		}

		int iSizeChange = SizeInK(uSize);

		for (u_int v = 0; v < pxStatGet->fileListNum; ++v)
		{
			if (pxStatGet->fileList[v].fileType == uType
				&& (uType != CELL_SAVEDATA_FILETYPE_SECUREFILE || Asura_StringUtil::CompareAgainstWildcardString(szName, pxStatGet->fileList[v].fileName, true)))
			{
				s_abExistingFiles[s_aeFileIndexForSaveStep[u]] = true;
				iSizeChange -= SizeInK(pxStatGet->fileList[v].st_size);
				break;
			}
		}
		// NY - 13 May 2008 - don't perform the size change for existing CIF files on an autosave, as we won't overwrite
		// same for save block and header if flagged to not overwrite
		if (!(s_bIsAutosave
			&& s_abExistingFiles[s_aeFileIndexForSaveStep[u]]
			&& (s_aeFileIndexForSaveStep[u] <= FILE_INDEX_LAST_CIF
				|| (s_bDontOverwriteSaveBlock && u >= SAVE_OPERATION_STEP_SAVEBLOCKHEADER))))
		{
			iRunningSizeChange += iSizeChange;
			iMaxSizeChange = Asura_Maths::Max(iMaxSizeChange, iRunningSizeChange);
		}
	}
	return pxStatGet->hddFreeSizeKB - iMaxSizeChange;
}

// SaveDataListCallback is called when a list of files in the project save directory has
// been compiled. It receives the list as input and uses it to build the list that will be
// displayed to the user. In this case it simply copies the list into the output list and 
// then checks to see if the current number of save data is less than the maximum allowed.
// If it is then it creates a New Save Data option and adds it to the list.
void Asura_PS3_SaveLoad::SaveDataListCallback( CellSaveDataCBResult* pxResult, CellSaveDataListGet* pxListGet, CellSaveDataListSet* pxListSet )
{
	Asura_OutputToDebugger_Retail( ("We have found %i files in total\n", pxListGet->dirListNum) );

	if( pxListGet->dirNum > pxListGet->dirListNum )
	{
		Asura_OutputToDebugger( ("There are more save games in the file than have been put in the save game list\n") );
	}

	// just put everything we find on the users list
	// this simplification is only valid if we're just doing a single type of list load - if we add other save data for the title, we'll need to filter it out here
	pxListSet->fixedList = pxListGet->dirList;
	pxListSet->fixedListNum = pxListGet->dirListNum;
	//if we have an autosave, focus on that; otherwise focus on the top item
	if (s_bCanAutosave)
	{
		pxListSet->focusPosition = CELL_SAVEDATA_FOCUSPOS_DIRNAME;
		pxListSet->focusDirName = s_szAutosaveFilename;
	}
	else
	{
		pxListSet->focusPosition = CELL_SAVEDATA_FOCUSPOS_LISTHEAD;
		pxListSet->focusDirName = NULL;
	}
	pxListSet->reserved = NULL;

	// do we have space for a new save game
	if( pxListGet->dirListNum < MAX_SAVE_GAMES )
	{
		SetupBlankSlot(pxListGet, pxListSet);
	}
	else
	{
		pxListSet->newData = NULL;
	}

	pxResult->result = CELL_SAVEDATA_CBRESULT_OK_NEXT;
}

//takes the place of the list callback when doing an autosave - just tells the util which slot to save to
void Asura_PS3_SaveLoad::AutoSaveDataFixedCallback( CellSaveDataCBResult* pxResult, CellSaveDataListGet* pxListGet, CellSaveDataFixedSet* pxFixedSet )
{
	pxFixedSet->dirName = s_szAutosaveFilename;
	pxFixedSet->newIcon = NULL;
	pxFixedSet->option = CELL_SAVEDATA_OPTION_NOCONFIRM;
	pxResult->result = CELL_SAVEDATA_CBRESULT_OK_NEXT;
}

// LoadDataListCallback is called when a list of files in the project save directory has been
// compiled. It very simply passes the list on to the user to choose which file to load, as there
// is no option for new data
void Asura_PS3_SaveLoad::LoadDataListCallback( CellSaveDataCBResult* pxResult, CellSaveDataListGet* pxListGet, CellSaveDataListSet* pxListSet )
{
	// just copy the saves we have found into the user list
	// this simplification is only valid if we're just doing a single type of list load - if we add other save data for the title, we'll need to filter it out here
	pxListSet->fixedList = pxListGet->dirList;
	pxListSet->fixedListNum = pxListGet->dirListNum;
	//store the count result since we're just got it for free
	s_uLastProfileCountResult = pxListGet->dirListNum;
	pxListSet->reserved = NULL;

	if(s_bBlankSlotOnLoad && pxListGet->dirListNum < MAX_SAVE_GAMES )
	{
		// NY - 10 July 2008 - remove this assert if/when sony fix the util
		Asura_Assert(0, "blank slot on load not properly supported by util");
		pxListSet->focusPosition = CELL_SAVEDATA_FOCUSPOS_LATEST;
		SetupBlankSlot(pxListGet, pxListSet);
	}
	else
	{
		pxListSet->focusPosition = CELL_SAVEDATA_FOCUSPOS_LISTHEAD;
		pxListSet->newData = NULL;
	}

	pxResult->result = CELL_SAVEDATA_CBRESULT_OK_NEXT;
}

// acts like the list callback when counting, only it just records the count and informs the util to early out
void Asura_PS3_SaveLoad::CountDataFixedCallback( CellSaveDataCBResult* pxResult, CellSaveDataListGet* pxListGet, CellSaveDataFixedSet* pxFixedSet )
{
	s_uLastProfileCountResult = pxListGet->dirListNum;

	// NY - 10 July 2008 - despite the fact we don't want user confirmation, we have to return the standard last enum
	// the utility only accepts the noconfirm form from certain callbacks, and interprets the standard one as noconfirm for others
	pxResult->result = CELL_SAVEDATA_CBRESULT_OK_LAST;//CELL_SAVEDATA_CBRESULT_OK_LAST_NOCONFIRM;
}

// It is the job of the status callback to confirm that the save data the user has chosen is suitable for the operation
// being carried out on it. we need to check free space here - sony kindly provide a step by step guide for this.
void Asura_PS3_SaveLoad::SaveDataStatusCallback( CellSaveDataCBResult* pxResult, CellSaveDataStatGet* pxStatGet, CellSaveDataStatSet* pxStatSet )
{
	pxStatSet->reCreateMode = CELL_SAVEDATA_RECREATE_NO;
	//set->reserved = NULL;

	// update the parameters for the save data. If we are overwriting something, or even autosaving, we don't want to be left with the
	// old list display parameters
	pxStatSet->setParam = &pxStatGet->getParam;
	Asura_PrintfToBfr( pxStatSet->setParam->title,s_szTitle, CELL_SAVEDATA_SYSP_TITLE_SIZE, true );
	Asura_PrintfToBfr( pxStatSet->setParam->subTitle, s_szSubTitle, CELL_SAVEDATA_SYSP_SUBTITLE_SIZE, true );
	Asura_PrintfToBfr( pxStatSet->setParam->detail, s_szDetail, CELL_SAVEDATA_SYSP_DETAIL_SIZE, true );
	pxStatSet->setParam->attribute = CELL_SAVEDATA_ATTR_NORMAL;
	// listParam is an internal name for the list in use - it can used for filtering lists
	Asura_PrintfToBfr( pxStatSet->setParam->listParam, "LIST", sizeof( pxStatSet->setParam->listParam ), true );

	for (u_int u = 0; u < FILE_INDEX_TOTAL; ++u)
	{
		s_abExistingFiles[u] = false;
	}

	// NY - 09 May 2008 - this should be 2 (this call plus minimum 1 call to file callback HOWEVER the progress bar seems to be currently broken and ignore the value from this call.
	// setting it to 1 for now makes the rest of maths work correctly
	// NY - 08 July 2008 - sony have confirmed this as a bug and are working on it
	s_uNumCallBacksForBar = 1;//2;
	for (u_int u = 0; u <= FILE_INDEX_LAST_CIF; ++u)
	{
		if( s_apFileBuffers[u] )
		{
			++s_uNumCallBacksForBar;
		}
	}
	if (s_pxSaveBlock)
	{
		s_uNumCallBacksForBar += 2;
	}

	int iNeedSize = 0;
	if( pxStatGet->isNewData )
	{
		iNeedSize = CalculateSizeNewData(pxStatGet);
	}
	else
	{
		iNeedSize = CalculateSizeOverwrite(pxStatGet);
	}
	if( iNeedSize < 0 )
	{
		s_bCanAutosave = false;
		s_bFreeSpaceFailOnLastSave = true;
		// we don't have space for this
		pxResult->errNeedSizeKB = iNeedSize;
		pxResult->result = CELL_SAVEDATA_CBRESULT_ERR_NOSPACE;
		return;
	}

	// we must be ok
	pxResult->result = CELL_SAVEDATA_CBRESULT_OK_NEXT;

	// set the autosave filename so next time we can autosave directly into this save data next time
	Asura_PrintfToBfr( s_szAutosaveFilename, "%s", CELL_SAVEDATA_DIRNAME_SIZE, true, pxStatGet->dir.dirName );
	Asura_OutputToDebugger_Retail( ("Autosave directory is now %s\n", s_szAutosaveFilename) );
	s_bCanAutosave = true;

	pxResult->progressBarInc = ProgressPercent();
}

// This status callback is very simple. As the file already exists and doesn't need to be modified in any way, it should be fine to proceed
// with saving. It also updates the autosave directory.
void Asura_PS3_SaveLoad::LoadDataStatusCallback( CellSaveDataCBResult* pxResult, CellSaveDataStatGet* pxStatGet, CellSaveDataStatSet* pxStatSet )
{
	// NY - 10 July 2008 - this catches where we're "loading" a blank slot, and reroutes to the save functions
	if (Asura_StringUtil::CompareAgainstWildcardString(s_szNewDataDirName, s_szAutosaveFilename, true))
	{
		SaveDataStatusCallback(pxResult, pxStatGet, pxStatSet);
		return;
	}

	pxStatSet->reCreateMode = CELL_SAVEDATA_RECREATE_NO_NOBROKEN;

	pxStatSet->setParam = 0;

	for (u_int u = 0; u < FILE_INDEX_TOTAL; ++u)
	{
		s_abExistingFiles[u] = false;
		for (u_int v = 0; v < pxStatGet->fileListNum; ++v)
		{
			if (pxStatGet->fileList[v].fileType == static_cast<u_int>(s_aeFileType[u])
				&& (s_aeFileType[u] != CELL_SAVEDATA_FILETYPE_SECUREFILE || Asura_StringUtil::CompareAgainstWildcardString(s_aszFileNames[u], pxStatGet->fileList[v].fileName, true)))
			{
				s_abExistingFiles[u] = true;
			}
		}
	}

	// NY - 09 May 2008 - this should be 2 (this call plus minimum 1 call to file callback HOWEVER the progress bar seems to be currently broken and ignore the value from this call.
	// setting it to 1 for now makes the rest of maths work correctly
	// NY - 08 July 2008 - sony have confirmed this as a bug and are working on it
	s_uNumCallBacksForBar = 1;//2;
	if (s_pxSaveBlock && s_abExistingFiles[FILE_INDEX_SAVEBLOCK])
	{
		s_uNumCallBacksForBar += 2;
	}

	// set the autosave filename so next time we can autosave directly into this save data next time
	Asura_PrintfToBfr( s_szAutosaveFilename, "%s", CELL_SAVEDATA_DIRNAME_SIZE, true, pxStatGet->dir.dirName );
	Asura_OutputToDebugger_Retail( ("Autosave directory is now %s\n", s_szAutosaveFilename) );
	s_bCanAutosave = true;

	pxResult->result = CELL_SAVEDATA_CBRESULT_OK_NEXT;
	pxResult->progressBarInc = ProgressPercent();
}

// The file callback specifies the operations to perform on the hard disk. Each operation corresponds to one callback, with it being called
// repeatedly until cbResult->result is set to CELL_SAVEDATA_CBRESULT_OK_LAST. The easiest way to write a file callback then is to have a
// static counter and perform different operations based on the value of the counter. 
void Asura_PS3_SaveLoad::SaveDataFileCallback( CellSaveDataCBResult* pxResult, CellSaveDataFileGet* pxFileGet, CellSaveDataFileSet* pxFileSet )
{
	Asura_OutputToDebugger_Retail( ("File Callback called\n") );

	pxResult->result = CELL_SAVEDATA_CBRESULT_OK_NEXT;

	bool bNextStepNeed = true;
	while (bNextStepNeed)
	{
		// (NY - 18.02.2008) set this flag back to true if a step has done nothing, and we'll move onto the next one.
		bNextStepNeed = false;

		switch( s_uSaveStep )
		{
			case SAVE_OPERATION_STEP_ICON0:
			case SAVE_OPERATION_STEP_ICON1:
			case SAVE_OPERATION_STEP_PIC1:
			case SAVE_OPERATION_STEP_SND0:
			{
				FILE_INDEX eFile = s_aeFileIndexForSaveStep[s_uSaveStep];
				if (s_apFileBuffers[eFile] && !(s_bIsAutosave && s_abExistingFiles[eFile]))
				{
					pxFileSet->fileOperation = CELL_SAVEDATA_FILEOP_WRITE;
					pxFileSet->fileBuf = s_apFileBuffers[eFile];
					pxFileSet->fileSize = s_auFileBufferSizes[eFile];
					pxFileSet->fileBufSize = s_auFileBufferSizes[eFile];
					pxFileSet->fileType = s_aeFileType[eFile];
					pxResult->progressBarInc = ProgressPercent();
					Asura_OutputToDebugger_Retail(("writing file %s\n", s_aszFileNames[eFile]));
				}
				else
				{
					//no need to check for one to delete, as we can't go (in a released game) from having one to not having one
					//just skip writing if we don't have one to write, or don't need to overwrite
					bNextStepNeed = true;
				}
				break;
			}
			case SAVE_OPERATION_STEP_PROFILE:
			{
				// let's save the file
				pxFileSet->fileOperation = CELL_SAVEDATA_FILEOP_WRITE;
				pxFileSet->fileBuf = s_apFileBuffers[FILE_INDEX_PROFILE];
				pxFileSet->fileBufSize = s_auFileBufferSizes[FILE_INDEX_PROFILE];
				pxFileSet->fileSize = s_auFileBufferSizes[FILE_INDEX_PROFILE];
				pxFileSet->fileType = CELL_SAVEDATA_FILETYPE_SECUREFILE;
				pxFileSet->fileName = szASURA_PS3_PROFILE_NAME;
				Asura_Mem::Copy( pxFileSet->secureFileId, s_szSecureFileId, CELL_SAVEDATA_SECUREFILEID_SIZE );
				pxResult->progressBarInc = ProgressPercent();
				Asura_OutputToDebugger_Retail(("writing file %s\n", s_aszFileNames[FILE_INDEX_PROFILE]));
				break;
			}
			case SAVE_OPERATION_STEP_SAVEBLOCKHEADER:
			{
				if (s_pxSaveBlock)
				{
					if (!(s_bIsAutosave && s_bDontOverwriteSaveBlock))
					{
						// let's save the file
						pxFileSet->fileOperation = CELL_SAVEDATA_FILEOP_WRITE;
						s_uSizeOfSaveBlock = s_pxSaveBlock->GetSpaceUsed();
						pxFileSet->fileBuf = &s_uSizeOfSaveBlock;
						pxFileSet->fileBufSize = sizeof(s_uSizeOfSaveBlock);
						pxFileSet->fileSize = sizeof(s_uSizeOfSaveBlock);
						pxFileSet->fileType = CELL_SAVEDATA_FILETYPE_SECUREFILE;
						pxFileSet->fileName = szASURA_PS3_SAVEBLOCKHEADER_NAME;
						Asura_Mem::Copy( pxFileSet->secureFileId, s_szSecureFileId, CELL_SAVEDATA_SECUREFILEID_SIZE );
						pxResult->progressBarInc = ProgressPercent();
						Asura_OutputToDebugger_Retail(("writing file %s\n", s_aszFileNames[FILE_INDEX_SAVEBLOCKHEADER]));
					}
					else
					{
						bNextStepNeed = true;
					}
				}
				else if (s_abExistingFiles[FILE_INDEX_SAVEBLOCK])
				{
					pxFileSet->fileOperation = CELL_SAVEDATA_FILEOP_DELETE;
					pxFileSet->fileType = CELL_SAVEDATA_FILETYPE_SECUREFILE;
					pxFileSet->fileName = szASURA_PS3_SAVEBLOCKHEADER_NAME;
					Asura_Mem::Copy( pxFileSet->secureFileId, s_szSecureFileId, CELL_SAVEDATA_SECUREFILEID_SIZE );
					pxResult->progressBarInc = 1;
					Asura_OutputToDebugger_Retail(("deleting file %s\n", s_aszFileNames[FILE_INDEX_SAVEBLOCKHEADER]));
				}
				else
				{
					//file neither exists nor is needed, so do nothing
					bNextStepNeed = true;
				}
				break;
			}
			case SAVE_OPERATION_STEP_SAVEBLOCK:
			{
				if (s_pxSaveBlock)
				{
					if (!(s_bIsAutosave && s_bDontOverwriteSaveBlock))
					{
						// let's save the file
						pxFileSet->fileOperation = CELL_SAVEDATA_FILEOP_WRITE;
						// (NY - 18.02.2008) sony want a non-const pointer for data to *write* to a file. huh?
						pxFileSet->fileBuf = const_cast<char*>(s_pxSaveBlock->GetStartPtr());
						pxFileSet->fileBufSize = s_pxSaveBlock->GetSpaceUsed() + s_pxSaveBlock->GetSpaceLeft();
						pxFileSet->fileSize = s_pxSaveBlock->GetSpaceUsed() + s_pxSaveBlock->GetSpaceLeft();
						pxFileSet->fileType = CELL_SAVEDATA_FILETYPE_SECUREFILE;
						pxFileSet->fileName = szASURA_PS3_SAVEBLOCK_NAME;
						Asura_Mem::Copy( pxFileSet->secureFileId, s_szSecureFileId, CELL_SAVEDATA_SECUREFILEID_SIZE );
						pxResult->progressBarInc = ProgressPercent();
						Asura_OutputToDebugger_Retail(("writing file %s\n", s_aszFileNames[FILE_INDEX_SAVEBLOCK]));
					}
					else
					{
						bNextStepNeed = true;
					}
				}
				else if (s_abExistingFiles[FILE_INDEX_SAVEBLOCK])
				{
					pxFileSet->fileOperation = CELL_SAVEDATA_FILEOP_DELETE;
					pxFileSet->fileType = CELL_SAVEDATA_FILETYPE_SECUREFILE;
					pxFileSet->fileName = szASURA_PS3_SAVEBLOCK_NAME;
					Asura_Mem::Copy( pxFileSet->secureFileId, s_szSecureFileId, CELL_SAVEDATA_SECUREFILEID_SIZE );
					pxResult->progressBarInc = 1;
					Asura_OutputToDebugger_Retail(("deleting file %s\n", s_aszFileNames[FILE_INDEX_SAVEBLOCK]));
				}
				else
				{
					//file neither exists nor is needed, so do nothing
					bNextStepNeed = true;
				}
				break;
			}
			default:
			{
				Asura_Assert(0, "unexpected save step");
			}//fallthrough to end step so at least we stop!
			case SAVE_OPERATION_STEP_END:
			{
				Asura_OutputToDebugger_Retail( ("That is the end of the save operation\n") );
				pxResult->result = CELL_SAVEDATA_CBRESULT_OK_LAST;
				pxResult->progressBarInc = 1;
				break;
			}
		}

		s_uSaveStep++;
	}
}

// similar to the save file callback, but only care about actual data files.
void Asura_PS3_SaveLoad::LoadDataFileCallback( CellSaveDataCBResult* pxResult, CellSaveDataFileGet* pxFileGet, CellSaveDataFileSet* pxFileSet )
{
	// NY - 10 July 2008 - if we're "loading" from a blank slot then reroute to the save functions
	if (Asura_StringUtil::CompareAgainstWildcardString(s_szNewDataDirName, s_szAutosaveFilename, true))
	{
		SaveDataFileCallback(pxResult, pxFileGet, pxFileSet);
		return;
	}

	Asura_OutputToDebugger_Retail( ("File Callback called\n") );
	pxResult->result = CELL_SAVEDATA_CBRESULT_OK_NEXT;

	if (s_pfnCallback && (s_uLoadStep - 1) == LOAD_OPERATION_STEP_PROFILE)// NY - 09 May 2008 - I know this can lead to doing 0-1 unsigned, but the maths still works.
	{
		if (!s_pfnCallback(s_apFileBuffers[FILE_INDEX_PROFILE]))
		{
			//project code has rejected the profile data, so tell the util the save is corrupt
			pxResult->result = CELL_SAVEDATA_CBRESULT_ERR_BROKEN;
			return;
		}
	}
	

	bool bNextStepNeed = true;
	while (bNextStepNeed)
	{
		// (NY - 18.02.2008) set this flag back to true if a step has done nothing, and we'll move onto the next one.
		bNextStepNeed = false;

		switch( s_uLoadStep )
		{
			case LOAD_OPERATION_STEP_PROFILE:
			{
				pxFileSet->fileOperation = CELL_SAVEDATA_FILEOP_READ;
				pxFileSet->fileBuf = s_apFileBuffers[FILE_INDEX_PROFILE];
				pxFileSet->fileBufSize = s_auFileBufferSizes[FILE_INDEX_PROFILE];
				pxFileSet->fileName = szASURA_PS3_PROFILE_NAME;
				pxFileSet->fileSize = s_auFileBufferSizes[FILE_INDEX_PROFILE];
				pxFileSet->fileOffset = 0;
				pxFileSet->fileType = CELL_SAVEDATA_FILETYPE_SECUREFILE;
				Asura_Mem::Copy( pxFileSet->secureFileId, s_szSecureFileId, CELL_SAVEDATA_SECUREFILEID_SIZE );
				pxResult->progressBarInc = ProgressPercent();
				break;
			}
			case LOAD_OPERATION_STEP_SAVEBLOCKHEADER:
			{
				if (s_pxSaveBlock && s_abExistingFiles[FILE_INDEX_SAVEBLOCK])
				{
					pxFileSet->fileOperation = CELL_SAVEDATA_FILEOP_READ;
					pxFileSet->fileBuf = &s_uSizeOfSaveBlock;
					pxFileSet->fileBufSize = sizeof(s_uSizeOfSaveBlock);
					pxFileSet->fileName = szASURA_PS3_SAVEBLOCKHEADER_NAME;
					pxFileSet->fileSize = sizeof(s_uSizeOfSaveBlock);
					pxFileSet->fileType = CELL_SAVEDATA_FILETYPE_SECUREFILE;
					Asura_Mem::Copy( pxFileSet->secureFileId, s_szSecureFileId, CELL_SAVEDATA_SECUREFILEID_SIZE );
					pxResult->progressBarInc = ProgressPercent();
				}
				else
				{
					bNextStepNeed = true;
				}
				break;
			}
			case LOAD_OPERATION_STEP_SAVEBLOCK:
			{
				if (s_pxSaveBlock && s_abExistingFiles[FILE_INDEX_SAVEBLOCK])
				{
					s_pxSaveBlock->Clear(false);
					s_apFileBuffers[FILE_INDEX_SAVEBLOCK] = s_pxSaveBlock->GetAccessPointer(s_uSizeOfSaveBlock);
					s_auFileBufferSizes[FILE_INDEX_SAVEBLOCK] = s_pxSaveBlock->GetSpaceUsed() + s_pxSaveBlock->GetSpaceLeft();

					pxFileSet->fileOperation = CELL_SAVEDATA_FILEOP_READ;
					pxFileSet->fileBuf = s_apFileBuffers[FILE_INDEX_SAVEBLOCK];
					pxFileSet->fileBufSize = s_auFileBufferSizes[FILE_INDEX_SAVEBLOCK];
					pxFileSet->fileName = szASURA_PS3_SAVEBLOCK_NAME;
					pxFileSet->fileSize = s_auFileBufferSizes[FILE_INDEX_SAVEBLOCK];
					pxFileSet->fileOffset = 0;

					pxFileSet->fileType = CELL_SAVEDATA_FILETYPE_SECUREFILE;
					Asura_Mem::Copy( pxFileSet->secureFileId, s_szSecureFileId, CELL_SAVEDATA_SECUREFILEID_SIZE );
					pxResult->progressBarInc = ProgressPercent();
				}
				else
				{
					bNextStepNeed = true;
				}
				break;
			}
			default:
			{
				Asura_Assert(0, "unexpected save step");
			}//fallthrough to end step so at least we stop!
			case LOAD_OPERATION_STEP_END:
			{
				pxResult->result = CELL_SAVEDATA_CBRESULT_OK_LAST;
				pxResult->progressBarInc = 1;
				break;
			}
		}

		s_uLoadStep++;
	}
}

// Thread_Save is called by both SaveProfile and ManualSaveProfile. It prepares the system utility for a call to cellSaveDataListSave2
// which will present the user with a list of possible save data to save to.
void Asura_PS3_SaveLoad::Thread_Save( void* pData, u_int uSize )
{
	s_bIsAutosave = false;
	// settings for the list of SaveData we will construct
	CellSaveDataSetList xSetList;
	// settings for the buffer which will hold the list
	CellSaveDataSetBuf xSetBuf;

	s_szNewDataDirName[0] = 0;

	// how do we want the list to be sorted
	xSetList.sortType = CELL_SAVEDATA_SORTTYPE_MODIFIEDTIME;
	xSetList.sortOrder = CELL_SAVEDATA_SORTORDER_DESCENT;
	// the directory to look in for Save Data
	xSetList.dirNamePrefix = s_szProjectDirName;
	xSetList.reserved = NULL;

	xSetBuf.dirListMax = MAX_SAVE_GAMES;
	xSetBuf.fileListMax = s_uMaxFiles;
	Asura_Mem::Set( xSetBuf.reserved, 0, sizeof( xSetBuf.reserved ) );

	xSetBuf.bufSize = s_uListBufferSize;
	xSetBuf.buf = s_pListBuffer;

	// now we can actually make the call to save
	int iRet = cellSaveDataListSave2(
		CELL_SAVEDATA_VERSION_CURRENT,
		&xSetList,
		&xSetBuf,
		SaveDataListCallback,
		SaveDataStatusCallback,
		SaveDataFileCallback,
		SYS_MEMORY_CONTAINER_ID_INVALID,
		NULL
		);

	s_bLastOperationCanceled = (iRet == CELL_SAVEDATA_RET_CANCEL);
	s_bErrorOnLastOperation = (iRet != CELL_SAVEDATA_RET_OK && iRet != CELL_SAVEDATA_RET_CANCEL);
	if (s_bErrorOnLastOperation)
	{
		s_bCanAutosave = false;
	}

	switch( iRet )
	{
	case CELL_SAVEDATA_ERROR_CBRESULT:
		Asura_OutputToDebugger_Retail( ("Error in cellSaveDataListSave2 - Callback function returned an error\n") );
		break;
	case CELL_SAVEDATA_ERROR_ACCESS_ERROR:
		Asura_OutputToDebugger_Retail( ("Error in cellSaveDataListSave2 - HDD access error\n") );
		break;
	case CELL_SAVEDATA_ERROR_INTERNAL:
		Asura_OutputToDebugger_Retail( ("Error in cellSaveDataListSave2 - Fatal Internal Error\n") );
		break;
	case CELL_SAVEDATA_ERROR_PARAM:
		Asura_OutputToDebugger_Retail( ("Error in cellSaveDataListSave2 - Error in parameter to be set to utility (application bug)\n") );
		break;
	case CELL_SAVEDATA_ERROR_NOSPACE:
		Asura_OutputToDebugger_Retail( ("Error in cellSaveDataListSave2 - Insufficient free space (bug in SaveDataStatusCallback)\n") );
		break;
	case CELL_SAVEDATA_ERROR_BROKEN:
		Asura_OutputToDebugger_Retail( ("Error in cellSaveDataListSave2 - Save data corrupted (modification detected, etc.)\n") );
		break;
	case CELL_SAVEDATA_ERROR_FAILURE:
		Asura_OutputToDebugger_Retail( ("Error in cellSaveDataListSave2 - Save/Load of save data failed (file could not be found, etc.)\n") );
		break;
	case CELL_SAVEDATA_ERROR_BUSY:
		Asura_OutputToDebugger_Retail( ("Error in cellSaveDataListSave2 - Save data utility function was called simultaneously\n") );
		break;
	}

	s_apFileBuffers[FILE_INDEX_PROFILE] = NULL;
	s_apFileBuffers[FILE_INDEX_SAVEBLOCK] = NULL;

	s_eStatus = STATUS_SAVED;
}

// Thread_Autosave is called by SaveProfile if an autosave save data is already specified. In this case, there is no need to display a list
// of save data to the user so we call cellSaveDataAutoSave2 which moves straight to the Status callback stage.
void Asura_PS3_SaveLoad::Thread_Autosave( void* pData, u_int uSize )
{
	s_bIsAutosave = true;
	CellSaveDataSetList xSetList;
	// settings for the buffer which will hold the list
	CellSaveDataSetBuf xSetBuf;

	s_szNewDataDirName[0] = 0;

	// how do we want the list to be sorted
	xSetList.sortType = CELL_SAVEDATA_SORTTYPE_MODIFIEDTIME;
	xSetList.sortOrder = CELL_SAVEDATA_SORTORDER_DESCENT;
	// the directory to look in for Save Data
	xSetList.dirNamePrefix = s_szProjectDirName;
	xSetList.reserved = NULL;

	xSetBuf.fileListMax = s_uMaxFiles;
	Asura_Mem::Set( xSetBuf.reserved, 0, sizeof( xSetBuf.reserved ) );

	xSetBuf.bufSize = s_uListBufferSize;
	xSetBuf.buf = s_pListBuffer;

	// now we can actually make the call to save
	int iRet = cellSaveDataListAutoSave(
		CELL_SAVEDATA_VERSION_CURRENT,
		CELL_SAVEDATA_ERRDIALOG_ALWAYS,
		&xSetList,
		&xSetBuf,
		AutoSaveDataFixedCallback,
		SaveDataStatusCallback,
		SaveDataFileCallback,
		SYS_MEMORY_CONTAINER_ID_INVALID,
		NULL
		);

	s_bLastOperationCanceled = (iRet == CELL_SAVEDATA_RET_CANCEL);
	s_bErrorOnLastOperation = (iRet != CELL_SAVEDATA_RET_OK && iRet != CELL_SAVEDATA_RET_CANCEL);
	if (s_bErrorOnLastOperation)
	{
		s_bCanAutosave = false;
	}

	switch( iRet )
	{
	case CELL_SAVEDATA_ERROR_CBRESULT:
		Asura_OutputToDebugger_Retail( ("Error in cellSaveDataListAutoSave - Callback function returned an error\n") );
		break;
	case CELL_SAVEDATA_ERROR_ACCESS_ERROR:
		Asura_OutputToDebugger_Retail( ("Error in cellSaveDataListAutoSave - HDD access error\n") );
		break;
	case CELL_SAVEDATA_ERROR_INTERNAL:
		Asura_OutputToDebugger_Retail( ("Error in cellSaveDataListAutoSave - Fatal Internal Error\n") );
		break;
	case CELL_SAVEDATA_ERROR_PARAM:
		Asura_OutputToDebugger_Retail( ("Error in cellSaveDataListAutoSave - Error in parameter to be set to utility (application bug)\n") );
		break;
	case CELL_SAVEDATA_ERROR_NOSPACE:
		Asura_OutputToDebugger_Retail( ("Error in cellSaveDataListAutoSave - Insufficient free space (bug in SaveDataStatusCallback)\n") );
		break;
	case CELL_SAVEDATA_ERROR_BROKEN:
		Asura_OutputToDebugger_Retail( ("Error in cellSaveDataListAutoSave - Save data corrupted (modification detected, etc.)\n") );
		break;
	case CELL_SAVEDATA_ERROR_FAILURE:
		Asura_OutputToDebugger_Retail( ("Error in cellSaveDataListAutoSave - Save/Load of save data failed (file could not be found, etc.)\n") );
		break;
	case CELL_SAVEDATA_ERROR_BUSY:
		Asura_OutputToDebugger_Retail( ("Error in cellSaveDataListAutoSave - Save data utility function was called simultaneously\n") );
		break;
	}

	s_apFileBuffers[FILE_INDEX_PROFILE] = NULL;
	s_apFileBuffers[FILE_INDEX_SAVEBLOCK] = NULL;

	s_eStatus = STATUS_SAVED;
}

// Thread_Load is called by LoadProfile and presents the user with a list of save data to choose which one to load. It behaves exactly
// similarly to the Thread_Save, but using slightly different callbacks
void Asura_PS3_SaveLoad::Thread_Load( void* pData, u_int uSize )
{
	CellSaveDataSetList xSetList;
	CellSaveDataSetBuf xSetBuf;

	s_szNewDataDirName[0] = 0;

	xSetList.sortType = CELL_SAVEDATA_SORTTYPE_MODIFIEDTIME;
	xSetList.sortOrder = CELL_SAVEDATA_SORTORDER_DESCENT;
	xSetList.dirNamePrefix = s_szProjectDirName;
	xSetList.reserved = NULL;

	xSetBuf.dirListMax = MAX_SAVE_GAMES;
	xSetBuf.fileListMax = s_uMaxFiles;
	Asura_Mem::Set( xSetBuf.reserved, 0, sizeof( xSetBuf.reserved ) );

	xSetBuf.bufSize = s_uListBufferSize;
	xSetBuf.buf = s_pListBuffer;

	int iRet = cellSaveDataListLoad2(
		CELL_SAVEDATA_VERSION_CURRENT,
		&xSetList,
		&xSetBuf,
		LoadDataListCallback,
		LoadDataStatusCallback,
		LoadDataFileCallback,
		SYS_MEMORY_CONTAINER_ID_INVALID,
		NULL
		);

	s_bLastOperationCanceled = (iRet == CELL_SAVEDATA_RET_CANCEL);
	s_bErrorOnLastOperation = (iRet != CELL_SAVEDATA_RET_OK && iRet != CELL_SAVEDATA_RET_CANCEL);
	if (s_bErrorOnLastOperation)
	{
		s_bCanAutosave = false;
	}

	switch( iRet )
	{
	case CELL_SAVEDATA_ERROR_CBRESULT:
		Asura_OutputToDebugger_Retail( ("Error in cellSaveDataListLoad2 - Callback function returned an error\n") );
		break;
	case CELL_SAVEDATA_ERROR_ACCESS_ERROR:
		Asura_OutputToDebugger_Retail( ("Error in cellSaveDataListLoad2 - HDD access error\n") );
		break;
	case CELL_SAVEDATA_ERROR_INTERNAL:
		Asura_OutputToDebugger_Retail( ("Error in cellSaveDataListLoad2 - Fatal Internal Error\n") );
		break;
	case CELL_SAVEDATA_ERROR_PARAM:
		Asura_OutputToDebugger_Retail( ("Error in cellSaveDataListLoad2 - Error in parameter to be set to utility (application bug)\n") );
		break;
	case CELL_SAVEDATA_ERROR_NOSPACE:
		Asura_OutputToDebugger_Retail( ("Error in cellSaveDataListLoad2 - Insufficient free space (bug in SaveDataStatusCallback)\n") );
		break;
	case CELL_SAVEDATA_ERROR_BROKEN:
		Asura_OutputToDebugger_Retail( ("Error in cellSaveDataListLoad2 - Save data corrupted (modification detected, etc.)\n") );
		break;
	case CELL_SAVEDATA_ERROR_FAILURE:
		Asura_OutputToDebugger_Retail( ("Error in cellSaveDataListLoad2 - Save/Load of save data failed (file could not be found, etc.)\n") );
		break;
	case CELL_SAVEDATA_ERROR_BUSY:
		Asura_OutputToDebugger_Retail( ("Error in cellSaveDataListLoad2 - Save data utility function was called simultaneously\n") );
		break;
	}

	s_apFileBuffers[FILE_INDEX_PROFILE] = NULL;
	s_apFileBuffers[FILE_INDEX_SAVEBLOCK] = NULL;

	s_eStatus = STATUS_LOADED;
}

//thread for counting profiles. it's actually and autoload that never gets as far as loading the data, or displaying any dialogs
void Asura_PS3_SaveLoad::Thread_CountProfiles( void* pData, u_int uSize )
{
	CellSaveDataSetList xSetList;
	CellSaveDataSetBuf xSetBuf;

	s_szNewDataDirName[0] = 0;

	xSetList.sortType = CELL_SAVEDATA_SORTTYPE_MODIFIEDTIME;
	xSetList.sortOrder = CELL_SAVEDATA_SORTORDER_DESCENT;
	xSetList.dirNamePrefix = s_szProjectDirName;
	xSetList.reserved = NULL;

	xSetBuf.dirListMax = MAX_SAVE_GAMES;
	xSetBuf.fileListMax = s_uMaxFiles;
	Asura_Mem::Set( xSetBuf.reserved, 0, sizeof( xSetBuf.reserved ) );

	xSetBuf.bufSize = s_uListBufferSize;
	xSetBuf.buf = s_pListBuffer;

	int iRet = cellSaveDataListAutoLoad(
		CELL_SAVEDATA_VERSION_CURRENT,
		CELL_SAVEDATA_ERRDIALOG_NONE,
		&xSetList,
		&xSetBuf,
		CountDataFixedCallback,
		LoadDataStatusCallback,//just pass load, will never be called
		LoadDataFileCallback,//just pass load, will never be called
		SYS_MEMORY_CONTAINER_ID_INVALID,
		NULL
		);

	s_bLastOperationCanceled = (iRet == CELL_SAVEDATA_RET_CANCEL);
	s_bErrorOnLastOperation = (iRet != CELL_SAVEDATA_RET_OK && iRet != CELL_SAVEDATA_RET_CANCEL);

	switch( iRet )
	{
	case CELL_SAVEDATA_ERROR_CBRESULT:
		Asura_OutputToDebugger_Retail( ("Error in cellSaveDataListLoad2 - Callback function returned an error\n") );
		break;
	case CELL_SAVEDATA_ERROR_ACCESS_ERROR:
		Asura_OutputToDebugger_Retail( ("Error in cellSaveDataListLoad2 - HDD access error\n") );
		break;
	case CELL_SAVEDATA_ERROR_INTERNAL:
		Asura_OutputToDebugger_Retail( ("Error in cellSaveDataListLoad2 - Fatal Internal Error\n") );
		break;
	case CELL_SAVEDATA_ERROR_PARAM:
		Asura_OutputToDebugger_Retail( ("Error in cellSaveDataListLoad2 - Error in parameter to be set to utility (application bug)\n") );
		break;
	case CELL_SAVEDATA_ERROR_NOSPACE:
		Asura_OutputToDebugger_Retail( ("Error in cellSaveDataListLoad2 - Insufficient free space (bug in SaveDataStatusCallback)\n") );
		break;
	case CELL_SAVEDATA_ERROR_BROKEN:
		Asura_OutputToDebugger_Retail( ("Error in cellSaveDataListLoad2 - Save data corrupted (modification detected, etc.)\n") );
		break;
	case CELL_SAVEDATA_ERROR_FAILURE:
		Asura_OutputToDebugger_Retail( ("Error in cellSaveDataListLoad2 - Save/Load of save data failed (file could not be found, etc.)\n") );
		break;
	case CELL_SAVEDATA_ERROR_BUSY:
		Asura_OutputToDebugger_Retail( ("Error in cellSaveDataListLoad2 - Save data utility function was called simultaneously\n") );
		break;
	}

	s_eStatus = STATUS_COUNTED;
}

// Thread_DeleteProfiles is called by DeleteProfiles and simply displays a list of save data on the PS3 hard disk and allows the user to 
// delete them.
void Asura_PS3_SaveLoad::Thread_DeleteProfiles( void* pData, u_int uSize )
{
	int iRet = cellSaveDataDelete2( SYS_MEMORY_CONTAINER_ID_INVALID );

	s_bLastOperationCanceled = (iRet == CELL_SAVEDATA_RET_CANCEL);
	s_bErrorOnLastOperation = (iRet != CELL_SAVEDATA_RET_OK && iRet != CELL_SAVEDATA_RET_CANCEL);

	s_eStatus = STATUS_DELETED;
}


//functions to set the various names and descriptions for a save
void Asura_PS3_SaveLoad::SetSaveDataTitle( const Asura_UnicodeChar* wszTitle )
{
	Asura_UnicodeToUTF8(wszTitle, s_szTitle, CELL_SAVEDATA_SYSP_TITLE_SIZE);
}

void Asura_PS3_SaveLoad::SetSaveDataSubTitle( const Asura_UnicodeChar* wszSubTitle )
{
	Asura_UnicodeToUTF8(wszSubTitle, s_szSubTitle, CELL_SAVEDATA_SYSP_TITLE_SIZE);
}

void Asura_PS3_SaveLoad::SetSaveDataDetail( const Asura_UnicodeChar* wszDetail )
{
	Asura_UnicodeToUTF8(wszDetail, s_szDetail, CELL_SAVEDATA_SYSP_TITLE_SIZE);
}

void Asura_PS3_SaveLoad::SetNewDataTitle( const Asura_UnicodeChar* wszTitle )
{
	Asura_UnicodeToUTF8(wszTitle, s_szNewDataTitle, CELL_SAVEDATA_SYSP_TITLE_SIZE);
}

void Asura_PS3_SaveLoad::SetProjectDirName( const char* szProjectDirName )
{
	Asura_PrintfToBfr(s_szProjectDirName, "%s", uPROJECTDIRNAMELENGTH, true, szProjectDirName);
}

#endif // #ifdef ASURA_PS3_SAVEGAMES
