//////////////////////////////////////////////////////////////////////////////
//
// File   : Asura_PS3_Corona_System.cpp
// Author : Catalin Arsenescu
// Date   : 12-March-2008
//
// Copyright (C) 2005 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "SpecialFX/Asura_PS3_Corona_System.h"

// Asura header-files...
#include "Camera/Asura_Camera.h"
#include "Core/Asura_TiledTextureManagement.h"
#include "Rendering/Asura_Colour.h"
#include "Rendering/Asura_Render.h"

#include "Rendering/Shaders/Asura_PS3_GCM_Shader.h"
#include "Rendering/Asura_PS3_GCM_VertexBuffer.h"
#include "Rendering/Asura_PS3_Graphics.h"
#include "Rendering/Asura_PS3_FrameBuffer.h"
#include "Rendering/Asura_PS3_GCM_IndexBuffer.h"
#include "Rendering/Asura_PS3_MaterialResponse_System.h"
#include "Rendering/Asura_PS3_RenderingOptions.h"
#include "Rendering/Asura_PS3_TextureManagement.h"
#include "Rendering/Asura_PS3_RenderingStates.h"
#include "Rendering/Asura_PS3_RSXProfiling.h"

//Shaders definition
#include "Rendering/ShadersDefinition/VD/Corona_VD.h"
#include "Rendering/ShadersDefinition/VP/Corona_VP.h"
#include "Rendering/ShadersDefinition/FP/Corona_FP.h"
// Game header-files...


////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////
// Vertex declarations...

static Asura_PS3_GCM_VertexShader*		g_pxVertexShader;
static Asura_PS3_FragmentShader*		g_pxFragmentShaderCorona;
static Asura_PS3_FragmentShader*		g_pxFragmentShaderCoronaSoft;

////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////

static Asura_PS3_GCM_VertexBuffer		g_xVertexBuffer;
static Asura_PS3_Corona_Handles*		g_pxHandlesCorona;
static Asura_PS3_Corona_Handles*		g_pxHandlesCoronaSoft;

////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////
int						Asura_PS3_CoronaSystem::s_aiBatchedTextures  [iASURA_MAX_BATCHABLE_CORONA_TEXTURES];
int						Asura_PS3_CoronaSystem::s_aiNumBatchedCoronas[iASURA_MAX_BATCHABLE_CORONA_TEXTURES];
const Asura_Corona*		Asura_PS3_CoronaSystem::s_apxBatchedCoronas  [iASURA_MAX_BATCHABLE_CORONA_TEXTURES][iASURA_MAX_BATCHABLE_CORONAS_PER_TEXTURE];
uint32					Asura_PS3_CoronaSystem::s_auBatchedVisFlags  [iASURA_MAX_BATCHABLE_CORONA_TEXTURES][iASURA_MAX_BATCHABLE_CORONAS_PER_TEXTURE];
CoronaVertex			Asura_PS3_CoronaSystem::s_axVertices		 [iASURA_MAX_BATCHABLE_CORONAS_PER_TEXTURE*4];


////////////////////////////////////////////////////////////////////////////////
// Private functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Protected functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Public functions...
////////////////////////////////////////////////////////////////////////////////

void Asura_PS3_CoronaSystem::Initialise()
{
	g_pxVertexShader				= new Asura_PS3_VP_Corona();
	g_pxHandlesCorona				= new Asura_PS3_Corona_Handles();
	g_pxFragmentShaderCorona		= new Asura_PS3_FragmentShader( (unsigned char *)&_binary_PS3_Corona_FP_bin_start, (unsigned char *)&_binary_PS3_Corona_FP_bin_end, g_pxHandlesCorona );
	g_pxHandlesCoronaSoft			= new Asura_PS3_Corona_Handles();
	g_pxFragmentShaderCoronaSoft	= new Asura_PS3_FragmentShader( (unsigned char *)&_binary_PS3_Corona_Soft_FP_bin_start, (unsigned char *)&_binary_PS3_Corona_Soft_FP_bin_end, g_pxHandlesCoronaSoft );

	g_pxVertexShader->InitVertexBufferDeclaration( g_axVertexDeclaration, VD_ELEMENTS );

	g_xVertexBuffer.Initialise( CORONA_VERTEX_SIZE, 4, CELL_GCM_LOCATION_LOCAL );

	for(int i=0; i<iASURA_MAX_BATCHABLE_CORONA_TEXTURES; i++)
	{
		s_aiNumBatchedCoronas[i] = 0;
		s_aiBatchedTextures[i] = -1;

		for(int j=0; j<iASURA_MAX_BATCHABLE_CORONAS_PER_TEXTURE; j++)
		{
			s_apxBatchedCoronas[i][j]=0;
		}
	}

	for(int i=0; i<iASURA_MAX_BATCHABLE_CORONAS_PER_TEXTURE; i++)
	{
		// UVs are constant
		s_axVertices[i*4+0].U = 0.0f;
		s_axVertices[i*4+0].V = 0.0f;
		s_axVertices[i*4+1].U = 1.0f;
		s_axVertices[i*4+1].V = 0.0f;
		s_axVertices[i*4+2].U = 1.0f;
		s_axVertices[i*4+2].V = 1.0f;
		s_axVertices[i*4+3].U = 0.0f;
		s_axVertices[i*4+3].V = 1.0f;
	}	
}

void Asura_PS3_CoronaSystem::Shutdown()
{
	g_xVertexBuffer.Destroy();
	delete g_pxVertexShader;
	delete g_pxHandlesCorona;
	delete g_pxHandlesCoronaSoft;
	delete g_pxFragmentShaderCorona;
	delete g_pxFragmentShaderCoronaSoft;

	g_pxVertexShader				= NULL;
	g_pxHandlesCorona				= NULL;
	g_pxHandlesCoronaSoft			= NULL;
	g_pxFragmentShaderCorona		= NULL;
	g_pxFragmentShaderCoronaSoft	= NULL;
}

void Asura_PS3_CoronaSystem::RenderStart (void)
{
	if( !Asura_PS3_RenderingOptions::GetOption( Asura_PS3_RenderingOptions::OPTION_DRAW_CORONAS ) ) return;
	ASURA_PS3_RSX_PROFILE__BEGIN( ASURA_PS3_RSX_PROFILE__MISC, "Asura_Corona_System" );

	Asura_PS3_RenderingStates::RequestZBufferCompareMode(ASURA_ZBUFFERCMP_LESSEQUAL);		
	Asura_PS3_RenderingStates::RequestTranslucencyMode( ASURA_TRANSLUCENCY_GLOWING );
	Asura_PS3_RenderingStates::RequestZBufferWriteEnabled( false );
	Asura_PS3_RenderingStates::RequestZBufferEnabled( true );
	Asura_PS3_RenderingStates::SetAlphaBlending( true );

	g_pxVertexShader->Bind();

	const Asura_AMatrix4x4& xViewProjectionMatrix = Asura_PS3_Graphics::GetViewProjectionMatrix();
	g_pxVertexShader->SetUniformParameter( VP_ViewProj, xViewProjectionMatrix.GetElements() );

	Asura_PS3_DeferredRendering::BindAsSingleRT();
	Asura_PS3_RenderingStates::RequestAlphaTest( false );

	Asura_PS3_RenderingStates::RequestCullMode( ASURA_CULLMODE_NONE );
}

void Asura_PS3_CoronaSystem::RenderCorona (const Asura_Corona* pxCorona, u_int uCamVisFlags)
{
	if( !Asura_PS3_RenderingOptions::GetOption( Asura_PS3_RenderingOptions::OPTION_DRAW_CORONAS ) ) return;

	Asura_Assert( pxCorona, "NULL parameter passed to Asura_Corona_System::Platform_RenderCorona()." );

	if (pxCorona->m_iTextureIndex<0)
	{
		// Invalid texture
		return;
	}

	// Do we have a batch for this texture?
	int iTextureSlot = -1;
	for(int i=0; i<iASURA_MAX_BATCHABLE_CORONA_TEXTURES; i++)
	{							
		if (s_aiBatchedTextures[i] == pxCorona->m_iTextureIndex)
		{
			iTextureSlot = i;
		}
	}

	if (iTextureSlot>=0)
	{
		// If we've got a batch, but it's full, we need to flush it
		if (s_aiNumBatchedCoronas[iTextureSlot]>=iASURA_MAX_BATCHABLE_CORONAS_PER_TEXTURE)
		{
			FlushBatch(iTextureSlot);
		}		
	}
	else
	{
		// No - Do we have an empty slot for a new texture batch?		
		int iBestToFlush  = 0;		
		int iBiggestBatch = 0;		

		for(int i=0; i<iASURA_MAX_BATCHABLE_CORONA_TEXTURES; i++)
		{					
			// Find the first empty slot, if we have one
			if (s_aiBatchedTextures[i]<0)
			{
				if (iTextureSlot<0)
				{
					iTextureSlot = i;
				}				
			}

			// Update best candidate to flush (the one with the biggest batch already)
			if (s_aiNumBatchedCoronas[i]>iBiggestBatch)
			{
				iBiggestBatch = s_aiNumBatchedCoronas[i];
				iBestToFlush = i;
			}
		}

		if (iTextureSlot<0)
		{
			FlushBatch(iBestToFlush);
			iTextureSlot = iBestToFlush;
		}
	}

	s_aiBatchedTextures  [iTextureSlot] = pxCorona->m_iTextureIndex;	
	s_apxBatchedCoronas  [iTextureSlot][s_aiNumBatchedCoronas[iTextureSlot]] = pxCorona;	
	s_auBatchedVisFlags  [iTextureSlot][s_aiNumBatchedCoronas[iTextureSlot]] = uCamVisFlags;
	s_aiNumBatchedCoronas[iTextureSlot]++;
}

void Asura_PS3_CoronaSystem::RenderEnd (void)
{
	if( !Asura_PS3_RenderingOptions::GetOption( Asura_PS3_RenderingOptions::OPTION_DRAW_CORONAS ) ) return;
	for(int i=0; i<iASURA_MAX_BATCHABLE_CORONA_TEXTURES; i++)
	{	
		FlushBatch(i);
	}
	Asura_PS3_Graphics::Flush();
	ASURA_PS3_RSX_PROFILE__END( ASURA_PS3_RSX_PROFILE__MISC );

}

// ---------------------------------------------------------------------------------
// CalcLightScale() - Calculate the intensity of the corona - handle the distance fade
//                    and direction effect
// ---------------------------------------------------------------------------------

float Asura_PS3_CoronaSystem::CalcLightScale(const Asura_Corona *pxCorona, const Asura_Vector_3 *pxCameraPosition,
											 const Asura_Vector_3 &xShiftForward, const Asura_Matrix_3x3 *pxCameraOrientation)
{
	float fLightScale = 255.0f; 

	Asura_Vector_3 xDelta =  *pxCameraPosition - (pxCorona->m_xPosition + xShiftForward);
	float fDistance = xDelta.Magnitude();

	if(pxCorona->m_bUseDirectionEffect)
	{
		if (fDistance>0.0f)
		{
			fLightScale = (pxCorona->m_xDirection * xDelta) / fDistance;

			if (fLightScale<0)
			{
				fLightScale = 64.0f;
			}
			else
			{
				fLightScale = 191.0f*fLightScale+64.0f;
			}
		}
		else
		{
			fLightScale = 0.0f;
		}
	}

	if( !pxCorona->m_bDoNotFadeOutWithDistance )
	{
		float fDistanceScale = fDistance - ASURA_CORONA_FADE_DIST_BEGIN;

		if (fDistanceScale > 0.0f)
		{
			fDistanceScale *= ( 1.0f / (ASURA_CORONA_FADE_DIST_END - ASURA_CORONA_FADE_DIST_BEGIN) );

			fLightScale *= (1.0f - fDistanceScale);
		}
	}

	const float fNearFadeDistance = pxCorona->GetNearFadeDistance();
	if(fNearFadeDistance)
	{
		Asura_Vector_3 xZDir = Asura_Vector_3(pxCameraOrientation->e13,pxCameraOrientation->e23,pxCameraOrientation->e33);
		float fZDistance = -(xDelta*xZDir);

		if(fZDistance<fNearFadeDistance)
		{
			fLightScale *= fZDistance/fNearFadeDistance;
			if(fLightScale<0) fLightScale = 0;
		}
	}

	return fLightScale;
}

// ---------------------------------------------------------------------------------
// FlushBatch() - Render a batch of coronas that share the same texture
// ---------------------------------------------------------------------------------

void Asura_PS3_CoronaSystem::FlushBatch(int iTextureSlot)
{
	// Have we got something to render?
	if (s_aiBatchedTextures[iTextureSlot]>=0 && s_aiNumBatchedCoronas[iTextureSlot]>0)
	{
		for(int iCamera=0; iCamera<Asura_Camera::GetNumberOfCameras(); iCamera++)
		{			
			Asura_Camera::SetCameraIndex(iCamera);
			Asura_Render::ApplyCameraSettings();

			const Asura_Vector_3*	pxCameraPosition = Asura_Camera::GetPosition();
			const Asura_Matrix_3x3* pxCameraOrientation = Asura_Camera::GetOrientation();
			Asura_Vector_3 xDeltaX = Asura_Vector_3(pxCameraOrientation->e11,pxCameraOrientation->e21,pxCameraOrientation->e31);
			Asura_Vector_3 xDeltaY = Asura_Vector_3(pxCameraOrientation->e12,pxCameraOrientation->e22,pxCameraOrientation->e32);

			// 2 passes through the coronas - first the soft ones, then the non-soft ones?
			for(int iPass=0; iPass<2; iPass++)
			{
				int iNumCoronas = 0;
				int iNumBatched = s_aiNumBatchedCoronas[iTextureSlot];

				CoronaVertex *pxVtx = &s_axVertices[0];

				for(int i=0; i<iNumBatched; i++)
				{
					const Asura_Corona *pxCorona = s_apxBatchedCoronas[iTextureSlot][i];

					if (iPass==0 && pxCorona->m_fVolumetricDepthScale<=0.0f || 
						iPass==1 && pxCorona->m_fVolumetricDepthScale>0.0f)
					{
						if((s_auBatchedVisFlags[iTextureSlot][i] & (1<<iCamera))) 
						{
							Asura_Vector_3 xForward = pxCorona->m_xPosition - *pxCameraPosition;
							xForward.Normalise();
							Asura_Vector_3 xShiftForward =- ( xForward * pxCorona->m_fRadius * pxCorona->m_fVolumetricDepthScale );

							// Calculate the colour
							float fLightScale = CalcLightScale(pxCorona, pxCameraPosition, xShiftForward, pxCameraOrientation);

							if (fLightScale>0.0f)
							{							
								// Calculate the billboarded vert positions
								float fSize = pxCorona->m_fRadius;
								float afDeltaXScale[4] = {-fSize, fSize, fSize,-fSize};
								float afDeltaYScale[4] = {-fSize,-fSize, fSize, fSize};

								for(int j=0; j<4; j++)
								{
									pxVtx->X = pxCorona->m_xPosition.x + (afDeltaXScale[j] * xDeltaX.x) + (afDeltaYScale[j] * xDeltaY.x) + xShiftForward.x;
									pxVtx->Y = pxCorona->m_xPosition.y + (afDeltaXScale[j] * xDeltaX.y) + (afDeltaYScale[j] * xDeltaY.y) + xShiftForward.y;
									pxVtx->Z = pxCorona->m_xPosition.z + (afDeltaXScale[j] * xDeltaX.z) + (afDeltaYScale[j] * xDeltaY.z) + xShiftForward.z;
									pxVtx->Diffuse[0] = fLightScale * pxCorona->m_fRed / 255.0f;
									pxVtx->Diffuse[1] = fLightScale * pxCorona->m_fGreen / 255.0f;
									pxVtx->Diffuse[2] = fLightScale * pxCorona->m_fBlue / 255.0f;
									pxVtx->U2 = fSize * pxCorona->m_fVolumetricDepthScale * 2.0f;
									pxVtx++;
								}

								// Add the corona to the VB
								iNumCoronas++;
							}
						}						
					}				
				}

				// Render 'em now
				if (iNumCoronas>0)
				{
					if( iPass == 0 )
					{
						g_pxFragmentShaderCorona->SetTexture( FP_Sampler, s_aiBatchedTextures[iTextureSlot] );
						g_pxFragmentShaderCorona->Bind();
					}
					else
					{
//						Asura_PS3_MaterialResponse_System::SetDepthParameters( g_pxFragmentShaderCoronaSoft, FP_DepthAdjustment );
						// can't call that function because this isn't a copy, which is going to cause problems if we want to draw 2 shafts with different textures (though this parameter will be fine since it's not instance-dependent)
						float f = Asura_Camera::GetFarClip(), n = Asura_Camera::GetNearClip();
						const float afDepthAdjustment[2] = { (f*n)/(f-n), f/(f-n) };
						g_pxFragmentShaderCoronaSoft->SetFloats( FP_DepthAdjustment, 2, afDepthAdjustment );

						g_pxFragmentShaderCoronaSoft->SetTexture( FP_DepthBufferSampler, Asura_PS3_DeferredRendering::GetDeferredFrameBuffer()->GetZBufferAsTexture(), false );
						g_pxFragmentShaderCoronaSoft->SetTexture( FP_Sampler, s_aiBatchedTextures[iTextureSlot] );
						g_pxFragmentShaderCoronaSoft->Bind();
					}

					g_xVertexBuffer.Bind( g_axVertexDeclaration, VD_ELEMENTS );
					g_xVertexBuffer.RenderQuadsInline( iNumCoronas * 4, s_axVertices );
				}
			}
		}
	}

	s_aiBatchedTextures[iTextureSlot]   = -1;
	s_aiNumBatchedCoronas[iTextureSlot] = 0;
}

/// End of file. ///
