///////////////////////////////////////////////////////////////////////////////
//
// File   : Asura_PS3_DecalSystem.cpp
// Author : Catalin Arsenescu
// Date   : 02-September-2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "SpecialFX/Asura_PS3_DecalSystem.h"

// Asura header-files...
#include "Core/Asura.h"
#include "Core/Asura_Timers.h"
#include "Core/Asura_HalfFloat.h"
#include "Core/Asura_TextureManagement.h"
#include "Core/Asura_MaterialResponse_System.h"

#include "Light/Asura_GI_Solution.h"
#include "SpecialFX/Asura_Decal_System.h"
#include "SpecialFX/Asura_Decal_Template.h"
#include "SpecialFX/Asura_Decal_Manager.h"
#include "SpecialFX/Asura_Decal_System.h"
#include "Rendering/Asura_Colour.h"
#include "Camera/Asura_Camera.h"
#include "Rendering/Asura_Module.h"

#include "Rendering/Shaders/Asura_PS3_GCM_Shader.h"
#include "Rendering/Asura_PS3_Graphics.h"
#include "Rendering/Asura_PS3_FrameBuffer.h"
#include "Rendering/Asura_PS3_RenderingOptions.h"
#include "Rendering/Asura_PS3_AABBRenderer.h"
#include "Rendering/Asura_PS3_Graphics.h"
#include "Rendering/Asura_PS3_RenderingOptions.h"
#include "Rendering/Asura_PS3_TextureManagement.h"
#include "Rendering/Asura_PS3_MaterialResponse_System.h"
#include "Rendering/Asura_PS3_RenderingStates.h"
#include "Rendering/Asura_PS3_RSXProfiling.h"
#include "Rendering/Asura_PS3_GCM_IndexBuffer.h"
#include "Rendering/Asura_PS3_GCM_VertexBuffer.h"

//Shaders definition
#include "Rendering/ShadersDefinition/VD/Decals_VD.h"
#include "Rendering/ShadersDefinition/VD/Decals_Spec_VD.h"
#include "Rendering/ShadersDefinition/VP/Decals_VP.h"
#include "Rendering/ShadersDefinition/FP/Decals_FP.h"
#include "Rendering/ShadersDefinition/VP/Decals_Spec_VP.h"
#include "Rendering/ShadersDefinition/FP/Decals_Specular_FP.h"

/******************************************************************************/
/* Macros                                                                     */
/******************************************************************************/

#define VECTOR_TO_UINT( xVector )						\
(														\
  (static_cast< u_int >( xVector.x * 511.0f ) <<  0)	\
+ (static_cast< u_int >( xVector.y * 511.0f ) << 11)	\
+ (static_cast< u_int >( xVector.z * 511.0f ) << 22)	\
)

// The effect passes
enum
{
	Decal_MR_Pass	= 0,
	Decal_Spec_Pass	= 1,
};

// ************************************************************************************************
// Asura_PS3_DecalSystem                                                 
// ************************************************************************************************

// Asura_Decal_System - Platform functions redirected to Asura_Xbox360_Decal_System
void Asura_Decal_System::Platform_Initialise( void )			{ Asura_PS3_DecalSystem::Initialise();			}
void Asura_Decal_System::Platform_Shutdown( void )				{ Asura_PS3_DecalSystem::Shutdown();			}
void Asura_Decal_System::Platform_Reset( void )					{ Asura_PS3_DecalSystem::Reset();				}

#ifdef ASURA_USE_STATIC_DECAL_SYSTEM
void Asura_Decal_System::Platform_RenderStaticDecals( void )    { Asura_PS3_DecalSystem::RenderStaticDecals();	}
#endif

static Asura_PS3_GCM_VertexShader*		g_apxVP[2];

Asura_PS3_StaticDecalBuffer*			Asura_PS3_DecalSystem::s_pxStaticDecalBuffers	= NULL;
int										Asura_PS3_DecalSystem::s_iNumStaticDecalBuffers	= 0;

#if defined(ASURA_PS3_ALLOW_SHADER_RELOAD)
static bool ReloadShaders( bool bHelp );
#endif
// ------------------------------------------------------------------------------------------------
// Asura_PS3_DecalSystem::Initialise( )
// ------------------------------------------------------------------------------------------------

void Asura_PS3_DecalSystem::Initialise( void )
{		
	g_apxVP[Decal_MR_Pass]		= new PS3_Decals_VP::Asura_PS3_VP_Decals();
	g_apxVP[Decal_Spec_Pass]	= new PS3_Decals_Spec_VP::Asura_PS3_VP_Decals_Spec();

	Asura_PS3_GCM_ShadersLibrary::AddFP( static_cast<Asura_Hash_ID> ( ASURA_SHADER_HASH_DECALS ), Decal_MR_Pass, (unsigned char *)&_binary_PS3_Decals_FP_bin_start, ( (unsigned char *)&_binary_PS3_Decals_FP_bin_end - (unsigned char *)&_binary_PS3_Decals_FP_bin_start), new Asura_PS3_MaterialResponse_Handles(), 1, true );
	Asura_PS3_GCM_ShadersLibrary::AddFP( static_cast<Asura_Hash_ID> ( ASURA_SHADER_HASH_DECALS ), Decal_Spec_Pass, (unsigned char *)&_binary_PS3_Decals_Spec_FP_bin_start, ( (unsigned char *)&_binary_PS3_Decals_Spec_FP_bin_end - (unsigned char *)&_binary_PS3_Decals_Spec_FP_bin_start), new Asura_PS3_MaterialResponse_Handles(), 1, true );


	g_apxVP[Decal_MR_Pass]->InitVertexBufferDeclaration	( g_axVertexDeclaration,		sizeof(g_axVertexDeclaration)/sizeof(g_axVertexDeclaration[0])  );
	g_apxVP[Decal_Spec_Pass]->InitVertexBufferDeclaration	( g_axVertexDeclarationSpec,		sizeof(g_axVertexDeclarationSpec)/sizeof(g_axVertexDeclarationSpec[0])  );

#if defined(ASURA_PS3_ALLOW_SHADER_RELOAD)
	Asura_CommandConsole::AddCmd(L"Render.ReloadShaders.Decals", ReloadShaders, Asura_CommandConsole::REMOVE_AT_SHUTDOWN);
#endif
}

// ------------------------------------------------------------------------------------------------
// Asura_PS3_DecalSystem::Reset( )
// ------------------------------------------------------------------------------------------------

void Asura_PS3_DecalSystem::Reset( void ) 
{
	delete[] s_pxStaticDecalBuffers;		
	s_pxStaticDecalBuffers = NULL;
	s_iNumStaticDecalBuffers = 0;
}

// ------------------------------------------------------------------------------------------------
// Asura_PS3_DecalSystem::Shutdown( )
// ------------------------------------------------------------------------------------------------

void Asura_PS3_DecalSystem::Shutdown( void ) 
{
	if( s_pxStaticDecalBuffers )
	{
		delete [] s_pxStaticDecalBuffers;		
		s_pxStaticDecalBuffers = NULL;
		s_iNumStaticDecalBuffers = 0;
	}

	delete g_apxVP[Decal_MR_Pass];
	delete g_apxVP[Decal_Spec_Pass];

	g_apxVP[Decal_MR_Pass]			= NULL;
	g_apxVP[Decal_Spec_Pass]		= NULL;
}

// ------------------------------------------------------------------------------------------------
// RenderStaticDecals( )
// ------------------------------------------------------------------------------------------------
#ifdef ASURA_USE_STATIC_DECAL_SYSTEM
void Asura_PS3_DecalSystem::RenderStaticDecals(void)
{
	ASURA_PS3_RSX_PROFILE__BEGIN( ASURA_PS3_RSX_PROFILE__MISC, "Render Static decals" );

	PrepareRenderStates();

	float afFadeValues[4] = {5,1,1,10};

	for(int i=0; i<s_iNumStaticDecalBuffers; i++)
	{
		Asura_PS3_StaticDecalBuffer *pxBuffer = &s_pxStaticDecalBuffers[i];

		if (pxBuffer->m_iDecalMaterialIndex<0)
		{			
			pxBuffer->m_iDecalMaterialIndex = Asura_Decal_Manager::GetMaterialIndex(pxBuffer->m_uDecalMaterialHash);						
		}
	
		if (pxBuffer->m_iDecalMaterialIndex>=0)
		{		
			const Asura_Decal_Material *pxMtl = Asura_Decal_System::Project_GetDecalMaterial(pxBuffer->m_iDecalMaterialIndex+1);
			const Asura_DecalManager_Material *pxMgrMtl = Asura_Decal_Manager::GetDecalMaterialByIndex(pxBuffer->m_iDecalMaterialIndex);

			if (pxBuffer->m_iNumVertices>0 && ((pxBuffer->m_iModuleIndex<0 || Asura_Module_System::IsModuleVisible(pxBuffer->m_iModuleIndex))))
			{
				bool abAABBInFrustum[ASURA_MAX_NO_OF_CAMERAS];
				int  iNumCamerasVisibleFrom = 0;

				for(int j=0; j<Asura_Camera::GetNumberOfCameras(); j++)
				{
					Asura_Camera::SetCameraIndex(j);
					abAABBInFrustum[j] = Asura_Camera::IsAABBInFrustum(pxBuffer->m_xAABB);

					if (abAABBInFrustum[j])
					{					
						iNumCamerasVisibleFrom++;
					}
				}

				if (iNumCamerasVisibleFrom>0)
				{
					if (pxMgrMtl->m_bStaticDecalBackFaceCull)
					{
						Asura_PS3_RenderingStates::RequestCullMode( ASURA_CULLMODE_CCW ); 
					}
					else
					{				
						Asura_PS3_RenderingStates::RequestCullMode( ASURA_CULLMODE_NONE ); 
					}


					for( int iPass=0; iPass < 2; ++iPass )
					{
						if (iPass == 0)
							pxBuffer->m_xVertexBuffer.Bind( g_axVertexDeclaration, sizeof(g_axVertexDeclaration) / sizeof(g_axVertexDeclarationSpec[0]) );
						else
							pxBuffer->m_xVertexBuffer.Bind( g_axVertexDeclarationSpec, sizeof(g_axVertexDeclarationSpec) / sizeof(g_axVertexDeclarationSpec[0]) );

						// Set the shader parameters
						SetVertexShader( iPass == 1, pxMtl, afFadeValues );
						SetupMaterial( iPass, pxMtl);

						// Render them
						for(int j=0; j<Asura_Camera::GetNumberOfCameras(); j++)
						{
							Asura_Camera::SetCameraIndex(j);

							if (Asura_Camera::IsCameraActive() && abAABBInFrustum[j])
							{
								Asura_Render::ApplyCameraSettings();					
								pxBuffer->m_xIndexBuffer.RenderTriangles( pxBuffer->m_iNumIndices );
							}
						}					
					}
				}
			}
		}
	}

	RestoreRenderStates();
	ASURA_PS3_RSX_PROFILE__END( ASURA_PS3_RSX_PROFILE__MISC );

}
#endif

void Asura_PS3_DecalSystem::SetVertexShader( bool bSpecularPass, const Asura_Decal_Material *pxMtl, float *pfFadeValues )
{
	// UV scrolling
	float afUVScroll[4]						= { 0.0f, 0.0f, 0.0f, 0.0f };
	const Asura_Vector_3* const pxCamPos	= Asura_Camera::GetPosition();
	float afCamPos[4]						= { pxCamPos->x, pxCamPos->y, pxCamPos->z, 1.0f };
	const Asura_AMatrix4x4& xViewProjection	= Asura_PS3_Graphics::GetViewProjectionMatrix();

	if( pxMtl->m_fUScrollSpeed > 0.0f || pxMtl->m_fVScrollSpeed > 0.0f )
	{
		float fTimer = Asura_Timers::GetLocalScaledTime();
		afUVScroll[ 0 ] = Asura_Maths::Sine( fTimer * pxMtl->m_fUScrollSpeed ) * pxMtl->m_fUScrollRange;
		afUVScroll[ 1 ] = Asura_Maths::Cosine( fTimer * pxMtl->m_fVScrollSpeed ) * pxMtl->m_fVScrollRange;
	}

	g_apxVP[bSpecularPass]->Bind();
	g_apxVP[bSpecularPass]->SetUniformParameter( PS3_Decals_VP::VS_ViewProjectionMatrix, xViewProjection.GetElements() );
	g_apxVP[bSpecularPass]->SetUniformParameter( PS3_Decals_VP::VS_CameraPosition, afCamPos );
	g_apxVP[bSpecularPass]->SetUniformParameter( PS3_Decals_VP::VS_FadeValues, pfFadeValues );
	g_apxVP[bSpecularPass]->SetUniformParameter( PS3_Decals_VP::VS_UVScroll, afUVScroll );

	 if (!bSpecularPass)
	 {
		 const Asura_MaterialResponse* pxMaterial
			 = Asura_MaterialResponse_System::GetMaterialByHashID( pxMtl->m_uMaterialResponseHash );
		 Asura_RGBA xPrelightingColour = pxMaterial->GetAmbientColour();
		 xPrelightingColour *= pxMaterial->GetAmbientContribution();

		 float afPrelightingColourVector [4]	= { xPrelightingColour.m_fR, xPrelightingColour.m_fG, xPrelightingColour.m_fB, 0.0f };

		 g_apxVP[bSpecularPass]->SetUniformParameter( PS3_Decals_VP::VS_PrelightingColourFilter, afPrelightingColourVector );
	 }

}

void Asura_PS3_DecalSystem::SetupMaterial( int iPass, const Asura_Decal_Material *pxMtl )
{
	const Asura_MaterialResponse* pxMaterial
		= Asura_MaterialResponse_System::GetMaterialByHashID( pxMtl->m_uMaterialResponseHash );

	const Asura_MaterialResponse::BLEND_MODE eBlendMode = pxMaterial ? pxMaterial->GetBlendMode() : Asura_MaterialResponse::BLEND_MODE_OPAQUE;

	Asura_PS3_FragmentShaderCopy* pxFPC = Asura_PS3_GCM_ShadersLibrary::GetPermanentFP( ASURA_SHADER_HASH_DECALS, iPass );

	if (pxMaterial)
	{
		Asura_PS3_MaterialResponse_System::SetEffectParameters( pxMaterial, NULL, NULL, pxFPC );
		Asura_PS3_MaterialResponse_System::SetProjectContext( pxMaterial, NULL );

		Asura_PS3_MaterialResponse_System::SetSHParameters( pxMaterial, pxFPC );
	}

	cellGcmSetBlendEquationUnsafeInline(Asura_PS3_Graphics::GetGcmCurrentContext(), CELL_GCM_FUNC_ADD, CELL_GCM_FUNC_ADD);

	if (iPass == 0)
	{
		if (eBlendMode == Asura_MaterialResponse::BLEND_MODE_ADDITIVE)
		{
			cellGcmSetBlendFuncUnsafeInline( Asura_PS3_Graphics::GetGcmCurrentContext(), CELL_GCM_SRC_ALPHA, CELL_GCM_ONE, CELL_GCM_ZERO, CELL_GCM_ONE_MINUS_SRC_ALPHA );
		} 
		else
		{
			cellGcmSetBlendFuncUnsafeInline( Asura_PS3_Graphics::GetGcmCurrentContext(), CELL_GCM_SRC_ALPHA, CELL_GCM_ONE_MINUS_SRC_ALPHA, CELL_GCM_ZERO, CELL_GCM_ONE_MINUS_SRC_ALPHA );
		}

		if (pxMaterial->GetTexture(Asura_MaterialResponse::NORMAL_MAP) != -1)
		{
			Asura_PS3_RenderingStates::SetColorMaskMrtState( 0x0fef ); // write all but material
		} 
		else 
		{
			Asura_PS3_RenderingStates::SetColorMaskMrtState( 0x0f0f ); // everything but materials and normals
		}
	} 
	else
	{
		cellGcmSetBlendFuncUnsafeInline( Asura_PS3_Graphics::GetGcmCurrentContext(), CELL_GCM_ONE, CELL_GCM_ONE, CELL_GCM_ONE, CELL_GCM_ONE );
		Asura_PS3_RenderingStates::SetColorMaskMrtState( ASURA_PS3_MRT_MRT2_ALPHA ); // write specular only
	}


	// Diffuse texture
	pxFPC->Bind();
}

void Asura_PS3_DecalSystem::PrepareRenderStates( void )
{
	Asura_PS3_RenderingStates::RequestAlphaTest(false);
	Asura_PS3_RenderingStates::RequestTranslucencyMode(ASURA_TRANSLUCENCY_NORMAL);		
	Asura_Render::CurrentStates.RequestCullMode(ASURA_CULLMODE_NONE);	
	Asura_PS3_RenderingStates::RequestZBufferWriteEnabled(false);
	Asura_PS3_RenderingStates::RequestZBufferCompareMode(ASURA_ZBUFFERCMP_LESSEQUAL);		

	// Z-bias them forwards a bit
	Asura_PS3_RenderingStates::SetUseDepthBias( true );
	Asura_PS3_RenderingStates::SetDepthBias( -75.0f, 0.0f );

 	Asura_PS3_RenderingStates::SetAlphaBlending(true);
 	Asura_PS3_RenderingStates::SetBlendEnableMrt( true, true, false );

}

void Asura_PS3_DecalSystem::RestoreRenderStates( void )
{
	Asura_PS3_RenderingStates::RequestZBufferWriteEnabled(true);	
	Asura_PS3_RenderingStates::SetUseDepthBias( false );
	Asura_PS3_RenderingStates::SetColorMaskMrtState( ASURA_PS3_MRT_MRT0123_ALL );
	Asura_PS3_RenderingStates::InvalidateTranslucencyMode();
}

// ------------------------------------------------------------------------------------------------
// AllocateStaticDecalBuffers() - Called from the resource chunk loader
// ------------------------------------------------------------------------------------------------

void Asura_PS3_DecalSystem::AllocateStaticDecalBuffers(int iNumBuffers)
{
	if( s_pxStaticDecalBuffers )
	{
		delete [] s_pxStaticDecalBuffers;
		s_pxStaticDecalBuffers = NULL;
		s_iNumStaticDecalBuffers = 0;
	}

	s_pxStaticDecalBuffers = new Asura_PS3_StaticDecalBuffer[iNumBuffers];
	s_iNumStaticDecalBuffers = iNumBuffers;
}

// ------------------------------------------------------------------------------------------------
// GetStaticDecalBuffer()
// ------------------------------------------------------------------------------------------------

Asura_PS3_StaticDecalBuffer* Asura_PS3_DecalSystem::GetStaticDecalBuffer(int iIndex)
{
	Asura_Assert(iIndex>=0 && iIndex<s_iNumStaticDecalBuffers, "Invalid staticdecal buffer index!");
	return &s_pxStaticDecalBuffers[iIndex];
}

// ------------------------------------------------------------------------------------------------
// Platform_AddStrip( )
// ------------------------------------------------------------------------------------------------

void Asura_Decal_Handler::Platform_AddStrip (const Asura_Decal_Vertex* pxStrip, u_int uNumVertices, const Asura_RGBA& xRGBA, const Asura_Decal_Template* pxDecalTemplate)
{
	Asura_PS3_DecalBuffer* pxDecalBuffer = static_cast<Asura_PS3_DecalBuffer*>( m_pxFirstDecalBuffer );

	// Do we have a decal buffer for the current decal material?
	while( pxDecalBuffer )
	{
		if( pxDecalBuffer->m_iDecalMaterialIndex == pxDecalTemplate->m_iDecalMaterialIndex )
		{
			pxDecalBuffer->AddStrip( pxStrip, uNumVertices, pxDecalTemplate, xRGBA );
			return;
		}
		pxDecalBuffer = pxDecalBuffer->m_pxNextDecalBuffer;
	}

	// Need to create a new buffer
	pxDecalBuffer = new Asura_PS3_DecalBuffer( pxDecalTemplate );
	pxDecalBuffer->m_pxNextDecalBuffer = static_cast< Asura_PS3_DecalBuffer* >( m_pxFirstDecalBuffer );
	m_pxFirstDecalBuffer = pxDecalBuffer;
	pxDecalBuffer->AddStrip( pxStrip, uNumVertices, pxDecalTemplate, xRGBA );
}

// ************************************************************************************************
// Asura_PS3_Buffer
// This stores all decals for one material type (which may be more than one decal template)
// Has a circular buffer of verts, and an index buffer that gets shifted down when full
// ************************************************************************************************
Asura_PS3_DecalBuffer::Asura_PS3_DecalBuffer(const Asura_Decal_Template* pxDecalTemplate)
{
	const Asura_Decal_Material *pxMtl = Asura_Decal_System::Project_GetDecalMaterial( pxDecalTemplate->m_iDecalMaterialIndex );

	m_iDecalMaterialIndex = pxDecalTemplate->m_iDecalMaterialIndex;
	m_iMaxDecals		  = pxMtl->m_iMaxDecals;
	m_apxDecals			  = new Decal[ m_iMaxDecals ];
	m_iNumVerts			  = m_iMaxDecals * 8;
	m_iNumIndices		  = m_iMaxDecals * 18;
	m_apxIndices		  = new uint16[ m_iNumIndices ]; // Worst-case is 18 indices for a decal  (if all 8 verts are used)
	m_iFirstDecal		  = 0;
	m_iNumDecals		  = 0;
	m_iNextVertex		  = 0;
	m_iNumDecalTris		  = 0;
	m_pxNextDecalBuffer   = NULL;
	m_iCurrentIndexBuffer = 0;
	m_bIndexBufferInvalid = false;

	m_pxVB.Initialise( sizeof(Asura_PS3_Decal_Vertex), m_iNumVerts, CELL_GCM_LOCATION_MAIN );

	for(int i=0;i<2;i++)
	{
		m_apxIB[ i ].Initialise( m_iNumIndices );
	}
}

Asura_PS3_DecalBuffer::~Asura_PS3_DecalBuffer()
{
	delete m_pxNextDecalBuffer;

	m_pxVB.Shutdown();
	m_apxIB[ 0 ].Shutdown();
	m_apxIB[ 1 ].Shutdown();

	delete [] m_apxDecals;		
	delete [] m_apxIndices;
}

void Asura_PS3_DecalBuffer::AddStrip(const Asura_Decal_Vertex* pxStrip, const u_int uNumberOfVertices, const Asura_Decal_Template* pxDecalTemplate, const Asura_RGBA& xRGBA )
{
	if (m_iNumDecals>=m_iMaxDecals)
	{
		// No room - flush the first one
		m_iNumDecalTris -= ( m_apxDecals[ m_iFirstDecal ].iNumIndices / 3 );
		++m_iFirstDecal;
		if( m_iFirstDecal >= m_iMaxDecals )
		{
			m_iFirstDecal = 0;
		}
		--m_iNumDecals;
	}

	// Is there space in the index buffer?
	int iFirstIndex = 0;
	if (m_iNumDecals>0)
	{
		int iLastDecal = (m_iFirstDecal+m_iNumDecals-1);
		if (iLastDecal>=m_iMaxDecals)
		{
			iLastDecal-=m_iMaxDecals;
		}

		iFirstIndex = m_apxDecals[iLastDecal].iFirstIndex+
			m_apxDecals[iLastDecal].iNumIndices;

		// De we need to shift the index buffer down
		if (iFirstIndex + ((int)((uNumberOfVertices-2)*3)) >= m_iNumIndices)
		{					
			int iShiftDown = m_apxDecals[m_iFirstDecal].iFirstIndex;			
			int iSrc	   = iShiftDown;
			int iDst	   = 0;
			int iDecal	   = m_iFirstDecal;

			for(int i=0;i<m_iNumDecalTris*3;i++)
			{				
				m_apxIndices[iDst++]=m_apxIndices[iSrc++];
			}

			for(int i=0; i<m_iNumDecals; i++)
			{										
				m_apxDecals[iDecal].iFirstIndex-=iShiftDown;										
				iDecal++;
				if (iDecal>=m_iMaxDecals)
				{
					iDecal-=m_iMaxDecals;
				}
			}	

			iFirstIndex = m_apxDecals[iLastDecal].iFirstIndex+
				m_apxDecals[iLastDecal].iNumIndices;
		}
	}

	// Fill in the decal struct
	int iDecal = m_iFirstDecal+m_iNumDecals;
	if (iDecal>=m_iMaxDecals)
	{
		iDecal-=m_iMaxDecals;
	}
	m_apxDecals[iDecal].iFirstIndex		= iFirstIndex;
	m_apxDecals[iDecal].iNumIndices		= (uNumberOfVertices-2)*3;
	m_apxDecals[iDecal].fCreationTime	= Asura_Timers::GetLocalScaledTime();
	m_iNumDecals++;

	// Fill in the verts+indices		
	if ((m_iNextVertex+(int)uNumberOfVertices)>m_iNumVerts)
	{
		m_iNextVertex = 0;
	}

	// Compute the tangent/binormal
	// We take a single normal for the decal now, to avoid a discontinuity when the
	// decal is 'bent' over a polygon edge.
	Asura_Vector_3 xNormal,xTangent,xBinormal;
	xNormal = Asura_Decal_System::GetCurrentDecalNormal();
	xNormal.Normalise();
	{		
		// There's room for some optimization here...
		Asura_Vector_3 axPositions[3];
		Asura_Vector_2 axUVs[3];
		for(int i=0;i<3;i++)
		{			
			axPositions[i] = pxStrip[i].m_xPosition;
			axUVs[i].x	   = pxStrip[i].m_fU;
			axUVs[i].y     = 1.0f - pxStrip[i].m_fV;
		}
		Asura_PS3_DecalSystem::CalculateTangentAndBinormal( axPositions, axUVs, xTangent, xBinormal );

		// The calculation above is incomplete, as it doesn't take the normal into account... so finish the job:
		xBinormal.Normalise();
		xTangent  = xBinormal ^  xNormal;
		xTangent.Normalise();
		xBinormal = xNormal ^ xTangent;
		xBinormal.Normalise();
	}

	// Converting from a strip to an indexed list for now - avoids messing with degens, and
	// may actually be faster?
	if (m_iNextVertex+((int)uNumberOfVertices) >= m_iNumVerts)
	{
		m_iNextVertex = 0;
	}

	uint16 *pxIndices = &m_apxIndices[iFirstIndex];
	int     iIndex = 0;

	for(u_int i=2; i<uNumberOfVertices; i++)
	{	
		// Make it back-face-culling-friendly
		if (i&1)
		{
			pxIndices[iIndex++] = (uint16)(m_iNextVertex+i);			
			pxIndices[iIndex++] = (uint16)(m_iNextVertex+i-1);			
			pxIndices[iIndex++] = (uint16)(m_iNextVertex+i-2);			
		}
		else
		{			
			pxIndices[iIndex++] = (uint16)(m_iNextVertex+i-2);			
			pxIndices[iIndex++] = (uint16)(m_iNextVertex+i-1);			
			pxIndices[iIndex++] = (uint16)(m_iNextVertex+i);			
		}
	}

	// Lock the VB with NoOverwrite flag - as the VB is a circular buffer with space for the max number
	// of decals, so we'll never overwrite verts that the GPU hasn't yet rendered.
	Asura_PS3_Decal_Vertex *pxVert = ( Asura_PS3_Decal_Vertex * )m_pxVB.LockForWriting();
	if( pxVert == NULL ) return;
	pxVert = &pxVert[ m_iNextVertex ];

	// Fill in the verts
	for(u_int i=0; i<uNumberOfVertices; ++i )
	{
		pxVert->fX = pxStrip[i].m_xPosition.x;
		pxVert->fY = pxStrip[i].m_xPosition.y;
		pxVert->fZ = pxStrip[i].m_xPosition.z;
		pxVert->fU = pxStrip[i].m_fU;
		pxVert->fV = pxStrip[i].m_fV;

		// (packing some of these values would be a good idea - at least on 360 - as PC support for packed verts seems poor)
		// Tangent frame

		pxVert->uNormal   = VECTOR_TO_UINT(xNormal);
		pxVert->uTangent  = VECTOR_TO_UINT(xTangent);
		pxVert->uBinormal = VECTOR_TO_UINT(xBinormal);

		pxVert->fAlpha = xRGBA.m_fA;
		pxVert->xSHRed[3] = xRGBA.m_fR;
		pxVert->xSHRed[0] = pxVert->xSHRed[1] = pxVert->xSHRed[2] = 0.0f;
		pxVert->xSHGreen[3] = xRGBA.m_fG;
		pxVert->xSHGreen[0] = pxVert->xSHGreen[1] = pxVert->xSHGreen[2] = 0.0f;
		pxVert->xSHBlue[3] = xRGBA.m_fB;
		pxVert->xSHBlue[0] = pxVert->xSHBlue[1] = pxVert->xSHBlue[2] = 0.0f;


		pxVert->fCreationTime = m_apxDecals[iDecal].fCreationTime;

		pxVert++;			
		m_iNextVertex++;						
	}	

	m_pxVB.Unlock();

	m_iNumDecalTris+= (uNumberOfVertices-2);

	// Index buffer needs repopulating
	m_bIndexBufferInvalid=true;
}

void Asura_PS3_DecalBuffer::Render(void)
{
	// Set up the fade values		
	const Asura_Decal_Material *pxMtl = Asura_Decal_System::Project_GetDecalMaterial(m_iDecalMaterialIndex);				
	float fCurrentTime = Asura_Timers::GetLocalScaledTime();
	float afFadeValues[4] = {fCurrentTime, pxMtl->m_fFadeInTime,pxMtl->m_fFadeOutTime, pxMtl->m_fDecalStayTime};
	if (pxMtl->m_fDecalStayTime<0)
	{
		// Not 'forever', but should be long enough?
		// (AAM - 17.01.2007) If this value is too large, PC cards
		// don't render properly. Some register overflow perhaps?
		afFadeValues[3] = 100000.0f;
	}

	// Remove faded-out decals here - so it doesn't keep rendering them invisibly.
	while (m_iNumDecals>0 && (fCurrentTime > m_apxDecals[m_iFirstDecal].fCreationTime + (afFadeValues[3]+pxMtl->m_fFadeOutTime)))
	{
		// Decal is faded out, remove it
		m_iNumDecalTris -= (m_apxDecals[m_iFirstDecal].iNumIndices/3);
		m_iFirstDecal++;
		if (m_iFirstDecal>=m_iMaxDecals)
		{
			m_iFirstDecal=0;
		}
		m_iNumDecals--;			
		m_bIndexBufferInvalid = true;
	}			

	// Repopulate the index buffer. We do it here, not in the add decal function, as that can often occur more than
	// once in a frame, and doing this more than once would be bad (cause lock stalls).
	//
	// It's a bit annoying having to do it like this, but the 360 doesn't support range-locking of vert/index buffers		
	if ( m_bIndexBufferInvalid )
	{		
		m_iCurrentIndexBuffer=1-m_iCurrentIndexBuffer;
		u_short *pxIndices = (u_short *)m_apxIB[m_iCurrentIndexBuffer].LockForWriting();
		if (pxIndices==NULL) return;
		int iFirstIndex = m_apxDecals[m_iFirstDecal].iFirstIndex;
		memcpy(&pxIndices[0], &m_apxIndices[iFirstIndex], m_iNumDecalTris*3*2);
		m_apxIB[m_iCurrentIndexBuffer].Unlock();
		m_bIndexBufferInvalid = false;
	}

	m_pxVB.Bind( g_axVertexDeclaration, VD_ELEMENTS );
	for( int iPass = 0; iPass < 2; ++iPass )
	{
		if( m_iNumDecals > 0 )
		{
			// Set the shader parameters
			Asura_PS3_DecalSystem::SetVertexShader( iPass == 1, pxMtl, afFadeValues );
			Asura_PS3_DecalSystem::SetupMaterial( iPass, pxMtl);

			// Render them
			for( int i = 0; i < Asura_Camera::GetNumberOfCameras(); ++i )
			{
				Asura_Camera::SetCameraIndex( i );

				if( Asura_Camera::IsCameraActive() )
				{
					Asura_Render::ApplyCameraSettings();					
					m_apxIB[ m_iCurrentIndexBuffer ].RenderTriangles( m_iNumDecalTris * 3 );
				}
			}
		}
	}

	if (m_pxNextDecalBuffer)
	{
		m_pxNextDecalBuffer->Render();
	}
}

// ------------------------------------------------------------------------------------------------
// Platform_Render( )
// ------------------------------------------------------------------------------------------------

void Asura_Decal_Handler::Platform_Render (void)
{
	if (m_pxFirstDecalBuffer == 0) return;

	ASURA_PS3_RSX_PROFILE__BEGIN( ASURA_PS3_RSX_PROFILE__MISC, "Render Decals" );

	Asura_PS3_DecalSystem::PrepareRenderStates();
	Asura_PS3_RenderingStates::RequestCullMode( ASURA_CULLMODE_NONE ); 
	static_cast<Asura_PS3_DecalBuffer*>(m_pxFirstDecalBuffer)->Render();
	Asura_PS3_DecalSystem::RestoreRenderStates();	
	ASURA_PS3_RSX_PROFILE__END( ASURA_PS3_RSX_PROFILE__MISC );

}

// Copied straight from Tools_Tangentspace.
static void Swap( float& xValue1, float& xValue2 ) { float xTemp = xValue1; xValue1 = xValue2; xValue2 = xTemp; }
static void Swap( Asura_Vector_3& xValue1, Asura_Vector_3& xValue2 ) { Asura_Vector_3 xTemp = xValue1; xValue1 = xValue2; xValue2 = xTemp; }

void Asura_PS3_DecalSystem::CalculateTangentAndBinormal( const Asura_Vector_3* pxPositions, const Asura_Vector_2* pxTextureCoords, Asura_Vector_3& rxTangent, Asura_Vector_3& rxBinormal )
{
	float			afU [3];
	float			afV [3];
	Asura_Vector_3	axPositions [3];

	// Initialise the local data...
	for (int i = 0; i < 3; i++)
	{
		axPositions [i]	= pxPositions [i];
		afU [i]			= pxTextureCoords [i].x;
		afV [i]			= pxTextureCoords [i].y;
	}

	// *** calc the tangent vector ***
	{
		// first sort vertices based on v coord
		if (afV [0] < afV [1])
		{
			Swap	(afU [0],			afU [1]);
			Swap	(afV [0],			afV [1]);
			Swap	(axPositions [0],	axPositions [1]);
		}
		if (afV [0] < afV [2])
		{
			Swap	(afU [0],			afU [2]);
			Swap	(afV [0],			afV [2]);
			Swap	(axPositions [0],	axPositions [2]);
		}
		if (afV [1] < afV [2])
		{
			Swap	(afU [1],			afU [2]);
			Swap	(afV [1],			afV [2]);
			Swap	(axPositions [1],	axPositions [2]);
		}

		// find offset along edge2 to the 'middle' v coord
		float fLambda;
		float fDivisor = (afV [2] - afV [0]);

		if (Asura_Maths::Abs(fDivisor) < 0.0001f)
		{
			fLambda = 1.0f;
		}
		else
		{
			fLambda = (afV [1] - afV [0]) / fDivisor;
		}

		// find the vertex position associated with this
		Asura_Vector_3 xPos = (axPositions [2] - axPositions [0]) * fLambda + axPositions [0];

		// tangent is the ray from the middle vertex to the lerped vertex
		rxTangent = xPos - axPositions [1];

		// make sure tangent points in the right direction
		float fInterpolatedU = (afU [2] - afU [0]) * fLambda + afU [0];

		if (fInterpolatedU < afU [1])
		{
			rxTangent = -rxTangent;
		}
	}

	// *** calc the binormal vector ***
	{
		// first sort vertices based on u coord
		if (afU [0] < afU [1])
		{
			Swap	(afU [0],			afU [1]);
			Swap	(afV [0],			afV [1]);
			Swap	(axPositions [0],	axPositions [1]);
		}
		if (afU [0] < afU [2])
		{
			Swap	(afU [0],			afU [2]);
			Swap	(afV [0],			afV [2]);
			Swap	(axPositions [0],	axPositions [2]);
		}
		if (afU [1] < afU [2])
		{
			Swap	(afU [1],			afU [2]);
			Swap	(afV [1],			afV [2]);
			Swap	(axPositions [1],	axPositions [2]);
		}

		// find offset along edge2 to the 'middle' v coord
		float fLambda;
		float fDivisor = (afU [2] - afU [0]);

		if (Asura_Maths::Abs(fDivisor) < 0.0001f)
		{
			fLambda = 1.0f;
		}
		else
		{
			fLambda = (afU [1] - afU [0]) / fDivisor;
		}

		// find the vertex position associated with this
		Asura_Vector_3 xPos = (axPositions [2] - axPositions [0]) * fLambda + axPositions [0];

		// binormal is the ray form the middle vertex to the lerped vertex
		rxBinormal = xPos - axPositions [1];

		// make sure binormal points in the right direction
		float fInterpolatedV = (afV [2] - afV [0]) * fLambda + afV [0];

		if (fInterpolatedV > afV [1])
		{
			rxBinormal = -rxBinormal;
		}
	}
}

#if defined(ASURA_PS3_ALLOW_SHADER_RELOAD)
bool ReloadShaders( bool bHelp )
{
	if ( bHelp )
	{
		Asura_CommandConsole::Printf("recompile & load decal shader");
		return true;
	}

	const char *aszIncludePath[]= { 
		"~/Asura/PS3/Shaders/Headers/", 
		NULL };
	Asura_PS3_GCM_ShadersLibrary::LoadAndCompileShader( "~/Asura/PS3/Shaders/FragmentPrograms/Decals/Decals_FP.fpx",
		ASURA_SHADER_HASH_DECALS, 0, NULL, aszIncludePath, new Asura_PS3_MaterialResponse_Handles );
	Asura_PS3_GCM_ShadersLibrary::LoadAndCompileShader( "~/Asura/PS3/Shaders/FragmentPrograms/Decals/Decals_Spec_FP.fpx",
		ASURA_SHADER_HASH_DECALS, 1, NULL, aszIncludePath, new Asura_PS3_MaterialResponse_Handles );

	return true;
}
#endif
