///////////////////////////////////////////////////////////////////////////////
//
// File   : Asura_Platform_Botanical_Game_Render_Instance_Inline.h
// Author : Andy Tate
// Date   : 9-Feb-2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifdef ASURA_USE_BOTANICALS
#ifndef __ASURA_PLATFORM_BOTANICAL_GAME_RENDER_INSTANCE_INLINE_H__
#define __ASURA_PLATFORM_BOTANICAL_GAME_RENDER_INSTANCE_INLINE_H__

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////
#include "Core/Asura_Maths.h"

////////////////////////////////////////////////////////////////////////////////
// Class declaration...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_BGR_Instance::GetInstancePosition
// Author       : Catalin Arsenescu
// Date         : 12-March-2007
////////////////////////////////////////////////////////////////////////////////
inline void Asura_BGR_Instance::GetInstancePosition( Asura_AVector3& xInstancePosition) const
{
	xInstancePosition.SetX( afCull[0] );
	xInstancePosition.SetY( afCull[1] );
	xInstancePosition.SetZ( afCull[2] );
}
/// End of function 'Asura_BGR_Instance::GetInstancePosition'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_BGR_Instance::GetInstancePosition
// Author       : Catalin Arsenescu
// Date         : 12-March-2007
////////////////////////////////////////////////////////////////////////////////
inline void Asura_BGR_Instance::GetInstancePosition( Asura_Vector_3& xInstancePosition) const
{
	xInstancePosition.SetX( afCull[0] );
	xInstancePosition.SetY( afCull[1] );
	xInstancePosition.SetZ( afCull[2] );
}
/// End of function 'Asura_BGR_Instance::GetInstancePosition'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_BGR_Instance::GetOrientation
// Author       : Catalin Arsenescu
// Date         : 12-March-2007
////////////////////////////////////////////////////////////////////////////////
inline void Asura_BGR_Instance::GetOrientation( Asura_AMatrix3x3& xInstanceOrientation) const
{
	xInstanceOrientation.SetColumn0( Asura_AVector3( fOrientation[0], fOrientation[3], fOrientation[6]));
	xInstanceOrientation.SetColumn1( Asura_AVector3( fOrientation[1], fOrientation[4], fOrientation[7]));
	xInstanceOrientation.SetColumn2( Asura_AVector3( fOrientation[2], fOrientation[5], fOrientation[8]));
}
/// End of function 'Asura_BGR_Instance::GetOrientation'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_BGR_Instance::GetOrientationTransposed
// Author       : Catalin Arsenescu
// Date         : 12-March-2007
////////////////////////////////////////////////////////////////////////////////
inline void Asura_BGR_Instance::GetOrientationTransposed( Asura_AMatrix3x3& xInstanceOrientation) const
{
	xInstanceOrientation.SetColumn0( Asura_AVector3( fOrientation[0], fOrientation[1], fOrientation[2]));
	xInstanceOrientation.SetColumn1( Asura_AVector3( fOrientation[3], fOrientation[4], fOrientation[5]));
	xInstanceOrientation.SetColumn2( Asura_AVector3( fOrientation[6], fOrientation[7], fOrientation[8]));
}
/// End of function 'Asura_BGR_Instance::GetOrientationTransposed'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_BGR_Instance::GetOrientation
// Author       : Catalin Arsenescu
// Date         : 12-March-2007
////////////////////////////////////////////////////////////////////////////////
inline void Asura_BGR_Instance::GetOrientation( Asura_Matrix_3x3& xInstanceOrientation) const
{
	xInstanceOrientation.e11 = fOrientation[0];
	xInstanceOrientation.e12 = fOrientation[1];
	xInstanceOrientation.e13 = fOrientation[2];
						   
	xInstanceOrientation.e21 = fOrientation[3];
	xInstanceOrientation.e22 = fOrientation[4];
	xInstanceOrientation.e23 = fOrientation[5];
						   
	xInstanceOrientation.e31 = fOrientation[6];
	xInstanceOrientation.e32 = fOrientation[7];
	xInstanceOrientation.e33 = fOrientation[8];
}
/// End of function 'Asura_BGR_Instance::GetOrientation'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_BGR_Instance::GetOrientationTransposed
// Author       : Catalin Arsenescu
// Date         : 12-March-2007
////////////////////////////////////////////////////////////////////////////////
inline void Asura_BGR_Instance::GetOrientationTransposed( Asura_Matrix_3x3& xInstanceOrientation) const
{
	xInstanceOrientation.e11 = fOrientation[0];
	xInstanceOrientation.e21 = fOrientation[3];
	xInstanceOrientation.e31 = fOrientation[6];
							   
	xInstanceOrientation.e12 = fOrientation[1];
	xInstanceOrientation.e22 = fOrientation[4];
	xInstanceOrientation.e32 = fOrientation[7];
							   
	xInstanceOrientation.e13 = fOrientation[2];
	xInstanceOrientation.e23 = fOrientation[5];
	xInstanceOrientation.e33 = fOrientation[8];
}
/// End of function 'Asura_BGR_Instance::GetOrientationTransposed'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_BGR_Instance::GetOrientationInversed
// Author       : Catalin Arsenescu
// Date         : 12-March-2007
////////////////////////////////////////////////////////////////////////////////
inline void Asura_BGR_Instance::GetOrientationInversed( Asura_Matrix_3x3& xInstanceOrientation) const
{
	const float fLength = Asura_Vector_3( fOrientation[0], fOrientation[3], fOrientation[6]).Magnitude();
	const float fScale = 1.0f / (fLength * fLength);
	
	xInstanceOrientation.e11 = fScale * fOrientation[0];
	xInstanceOrientation.e12 = fScale * fOrientation[3];
	xInstanceOrientation.e13 = fScale * fOrientation[6];
																			   
	xInstanceOrientation.e21 = fScale * fOrientation[1];
	xInstanceOrientation.e22 = fScale * fOrientation[4];
	xInstanceOrientation.e23 = fScale * fOrientation[7];
																			   
	xInstanceOrientation.e31 = fScale * fOrientation[2];
	xInstanceOrientation.e32 = fScale * fOrientation[5];
	xInstanceOrientation.e33 = fScale * fOrientation[8];
}
/// End of function 'Asura_BGR_Instance::GetOrientationInversed'. ///

#endif	// ifndef __ASURA_PLATFORM_BOTANICAL_GAME_RENDER_INSTANCE_INLINE_H__
#endif	//#ifdef ASURA_USE_BOTANICALS

/// End of file. ///
