//
// Asura_Chunk_ResourceFile_PSP_Character.cpp
//
// Created by KJFL - Wednesday, March 03, 2004
//
// Copyright (C) 2004 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

// This file's header.
#include "ChunkSystem/Asura_Chunk_ResourceFile_PSP_Character.h"

#include "Core/Asura_Hash.h"
#include "Core/Asura_Material_System.h"
#include "FileAccess/Asura_FileAccess.h"

#include "Rendering/Asura_PSP_CharacterRenderer.h"
#include "Rendering/Asura_PSP_CharacterRenderer_Internal.h"


void Asura_Chunk_ResourceFile_PSP_Character::Process( Asura_Handle hFile )
{
	// In version 2 and up colours can replace normals in the exported data
	const bool bColoursReplaceNormals = (Flags & ColoursReplaceNormals) && Version >= 2;

	Asura_ResourcePushType(ASURA_RESOURCE_SKINNED_MESH);
	{
		u_int uReadBytes = Asura_FileAccess::GetFilePosition( hFile );
		const Asura_Char* szName = Asura_FileAccess::ReadString( hFile );
		uReadBytes = Asura_FileAccess::GetFilePosition( hFile ) - uReadBytes;

		Asura_PSP_CharacterRender_Handler&	xHandler	= Asura_PSP_CharacterRenderer::GetCurrentHandler();
		Asura_PSP_SmoothSkin*				pxSkin		= xHandler.AddCharacter( Asura_GetHashID(szName) );

		if (pxSkin)
		{
			u_int uTotalNumberOfVertices;
			u_int uNumberOfGroups;

			Asura_FileAccess::ReadData( hFile, &uTotalNumberOfVertices, sizeof(u_int) );
			Asura_FileAccess::ReadData( hFile, &uNumberOfGroups, sizeof(u_int) );

			pxSkin->m_bColoursReplaceNormals = bColoursReplaceNormals;

			// allocate space for groups, vertices & indices
			pxSkin->m_uNumberOfGroups	= uNumberOfGroups;
			pxSkin->m_xVertexSize = (bColoursReplaceNormals ? sizeof(Asura_PSP_SmoothSkin_Vertex_Coloured) : sizeof(Asura_PSP_SmoothSkin_Vertex_Normal));
			Asura_ResourceAllocateArray(pxSkin->m_pxGroups, uNumberOfGroups);
			Asura_ResourceAllocateArray(pxSkin->m_pxVertices, uTotalNumberOfVertices * pxSkin->m_xVertexSize);
			Asura_FileAccess::ReadData( hFile, pxSkin->m_pxGroups, sizeof(Asura_PSP_SmoothSkin_Group) * uNumberOfGroups );

			if (bColoursReplaceNormals)
			{
				Asura_FileAccess::ReadData( hFile, pxSkin->m_pxVertices, uTotalNumberOfVertices * pxSkin->m_xVertexSize );
			}
			else
			{
				Asura_FileAccess::ReadData( hFile, pxSkin->m_pxVertices, uTotalNumberOfVertices * pxSkin->m_xVertexSize );

				// (NY - 01.03.2007) don't bother on submission builds (to save on code bloat!), but if we're loading old data, bit shift the vert data to correct for the change in fixed point precision
				#ifndef ASURA_PSP_SONY_SUBMISSION
				if (Version == 0)
				{
					for (u_int u = 0; u < uTotalNumberOfVertices; ++u)
					{
						for (u_int v = 0; v < 3; ++v)
						{
							reinterpret_cast<Asura_PSP_SmoothSkin_Vertex_Normal*>(pxSkin->m_pxVertices)[u].m_asPosition[v] = ((Asura_PSP_SmoothSkin_Vertex_Normal*)pxSkin->m_pxVertices)[u].m_asPosition[v] >> 1;
						}
					}
				}
				#endif // #ifndef ASURA_PSP_SONY_SUBMISSION

			}

			// now go through groups and patch up material ids
			for (u_int u=0; u<uNumberOfGroups; u++)
			{
				pxSkin->m_pxGroups[u].m_iMaterial = Asura_Material_System::ConvertIndex( pxSkin->m_pxGroups[u].m_iMaterial );
			}
		}
		else
		{
			// we've ran out of strip space or the character already existed
			// either way, let's skip the data
			Asura_FileAccess::SkipData( hFile, this->FileSize - uReadBytes);
		}
	}
	Asura_ResourcePopType();
}
