
///////////////////////////////////////////////////////////////////////////////
//
// File   : Asura_PSP_KernelManagement.h
// Author : Andy Tate
// Date   : 18-May-2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __ASURA_PSP_KERNELMANAGEMENT_H__
#define __ASURA_PSP_KERNELMANAGEMENT_H__

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

// Asura
#include "Core/Asura_Multithread.h"

// Library
#include <kerneltypes.h>
#include <psptypes.h>


////////////////////////////////////////////////////////////////////////////////
// Forward class declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Class declaration...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Class        : Asura_PSP_KernelManagement
// Author       : Andy Tate
// Date         : 18-May-2006
//
// Description  : Class containing interface functions between Asura and the
//				  PSP kernel.
////////////////////////////////////////////////////////////////////////////////

class Asura_PSP_KernelManagement
{
public:
	// Public enumerated types...
	enum SuspendReason
	{
		NONE = -1,

		POWER_OFF,
		UTILITY_LIBRARY,

		MAX_SUSPENSION_REASONS
	};

	enum ESpeed
	{
		SPEED_SLOW		= 0,	// Menus etc...
		SPEED_STANDARD,			// Multi-player etc...
		SPEED_FULL				// Single-player etc...
	};

	// Public functions...

	// Initialisation/shutdown functions...
	static	void	EarlyInitialise	(void);
	static	void	Initialise		(void);
	static	void	Shutdown		(void);

	// Suspend/wake functions...
	static	void	Suspend			(const SuspendReason eReason);
	static	void	Resume			(const SuspendReason eReason);

	static	bool	IsUMDReady		();
	static	bool	IsSuspended		() { return s_uSuspensionReasonBitfield != 0; }

	static	int		GetMainThreadID	() { return s_iMainThreadID;}
	static	void	StoreMainThreadID();

	static	bool	SetCPUSpeed		( ESpeed eSpeed ); 	// Set the PSP CPU (and bus) clock speed

protected:
	// Protected variables...


	// Protected functions...


private:
	// Private variables...
	static  int			    s_iMainThreadID;
	static  volatile bool	s_bUMDReady;

	static 	SceUID			s_iUmdCallBackId;				// ID for the UMD callback in operation.
	static 	SceUID			s_iPowerCallBackId;				// ID for the Power callback in operation.
	static 	SceUID			s_iExitGameCallBackId;			// ID for the Exit-game callback in operation.
	static 	SceUID			s_iMemoryStickCallBackId;		// ID for the memory stick callback in operation.

	static	volatile u_int	s_uSuspensionReasonBitfield;	// Bitfield used to say why we've suspended operation.
	static  Asura_Mutex		s_xCPUSpeedMutex;				// Mutex to prevent sceKernelExitGame being called during CPU speed manipulation
	
	// Private functions...
	Asura_PSP_KernelManagement  (void);
	Asura_PSP_KernelManagement  (const Asura_PSP_KernelManagement& xOther);
	~Asura_PSP_KernelManagement ();

	Asura_PSP_KernelManagement&	operator=	(const Asura_PSP_KernelManagement& xOther);

	// Callback functions...
	static 	int		UMDCallback(int count, int arg, void *param);
	static 	int		ExitgameCallback(int count, int arg, void *param);
	static 	int		PowerCallback(int count, int arg, void *param);
	static 	int		MemoryStickCallback(int count, int arg, void *param);
	static 	int		CallbackThread(SceSize args, ScePVoid argp);

	// Project-specific functions...
	static	void	Project_Suspend(void);
	static	void	Project_Resume(void);

};   // End of class 'Asura_PSP_KernelManagement'.


#endif   // ifndef __ASURA_PSP_KERNELMANAGEMENT_H__


/// End of file. ///
