//
// Asura_PSP_GPIO.h
//
// Created by KOR - 06/12/2005 14:31:19
//
// Copyright (C) 2005 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif

#ifndef __ASURA_PSP_GPIO_H__
#define __ASURA_PSP_GPIO_H__

#include <kernelutils.h>

#include "Debugging/Asura_Assert.h"


// KOR - 06/12/2005 - when changing this, please update the wiki at:
// http://ASURAWIKI/twiki/bin/viewauth/Programming/HowToUsePSPTestKitSwitches
enum ASURA_PSP_GPI_SWITCH
{
	ASURA_PSP_GPI_FPS = 0,
	ASURA_PSP_GPI_PROFILING,
	ASURA_PSP_GPI_DEBUG_INFO,
	ASURA_PSP_GPI_QA,
	ASURA_PSP_GPI_UNTEXTURED,
	ASURA_PSP_GPI_WIREFRAME_ENV,
	ASURA_PSP_GPI_SCREENSHOT,
	ASURA_PSP_GPI_DISABLE_VSYNC,
	// All GPI switches now used.

	ASURA_PSP_GPI_NUM_SWITCHES
};

// This class allows the user to use the following hardware functionality of the PSP devkit/testkit:
// 1) retrieve the status of the GPI switches
// 2) set the status of the GPO LEDs
class Asura_PSP_GPIO
{
public:
	// GPI caching to save frame-rate...
	static inline void CacheGPI (void)
	{
#if !defined(ASURA_PSP_SONY_SUBMISSION)
		s_ucGPICache	= sceKernelGetGPI();
#endif	// !defined(ASURA_PSP_SONY_SUBMISSION)
	}

	// GPI retrieval
	static inline bool GetGPI(ASURA_PSP_GPI_SWITCH eSwitch)
	{
#if defined(ASURA_PSP_SONY_SUBMISSION)
		return(false);
#else
		Asura_Assert(eSwitch < ASURA_PSP_GPI_NUM_SWITCHES, "Switch number must be in the range [0, 7]");
		return (s_ucGPICache & (1 << eSwitch));
#endif	// defined(ASURA_PSP_SONY_SUBMISSION)
	}

	// GPO manipulation
	static inline void SetGPO(u_char ucAll)
	{
		sceKernelSetGPO(ucAll);
	}


private:
#if !defined(ASURA_PSP_SONY_SUBMISSION)
	static u_char s_ucGPICache;		// Bits used to cache the state of the GPI switches.
#endif	// !defined(ASURA_PSP_SONY_SUBMISSION)
};

#endif//__ASURA_PSP_GPIO_H__
