//////////////////////////////////////////////////////////////////////////////
//
// Asura_PSP_Profiling.cpp
//
// Created by: Anthony Gregson - 20/09/04
//
// Copyright (c) 2004 Rebellion. All rights reserved.
//
// First version of PSP profiling. Uses Asura_Profiling_System, and probably
// isn't the most efficient of systems at this point. Needs proper work on
// making a system that impacts the running of the system less by using more
// PSP-specific calls.
//
//////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_PSP_PROFILING

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef ___ASURA_PSP_PROFILING_H___
#define ___ASURA_PSP_PROFILING_H___ 1

class Asura_PSP_Profiling
{

public:

	enum Asura_PSP_Signals
	{
		ASURA_PSP_SIGNAL_IDLE_START,
		ASURA_PSP_SIGNAL_IDLE_END,
		ASURA_PSP_SIGNAL_CLEAR_START,
		ASURA_PSP_SIGNAL_CLEAR_END,
		ASURA_PSP_SIGNAL_SKYBOX_START,
		ASURA_PSP_SIGNAL_SKYBOX_END,
		ASURA_PSP_SIGNAL_ENVIRONMENT_START,
		ASURA_PSP_SIGNAL_ENVIRONMENT_END,
		ASURA_PSP_SIGNAL_BOTANICALS_START, 
		ASURA_PSP_SIGNAL_BOTANICALS_END,
		ASURA_PSP_SIGNAL_DECAL_START,
		ASURA_PSP_SIGNAL_DECAL_END,
		ASURA_PSP_SIGNAL_ENTITY_START,
		ASURA_PSP_SIGNAL_ENTITY_END,
		ASURA_PSP_SIGNAL_GLOW_START,
		ASURA_PSP_SIGNAL_GLOW_END,
		ASURA_PSP_SIGNAL_PFX_START,
		ASURA_PSP_SIGNAL_PFX_END,
		ASURA_PSP_SIGNAL_OTHERS_START,
		ASURA_PSP_SIGNAL_OTHERS_END,
		ASURA_PSP_SIGNAL_MOTIONBLUR_START,
		ASURA_PSP_SIGNAL_MOTIONBLUR_END,
		ASURA_PSP_SIGNAL_FOG_START,
		ASURA_PSP_SIGNAL_FOG_END,
		ASURA_PSP_SIGNAL_FILTER_START,
		ASURA_PSP_SIGNAL_FILTER_END,
		ASURA_PSP_SIGNAL_CORONAS_START,
		ASURA_PSP_SIGNAL_CORONAS_END,
		ASURA_PSP_SIGNAL_OUTLINE_START, 
		ASURA_PSP_SIGNAL_OUTLINE_END, 
		ASURA_PSP_SIGNAL_PROFILE_START, 
		ASURA_PSP_SIGNAL_PROFILE_END, 

		ASURA_PSP_SIGNAL_MAX,

		ASURA_PSP_SIGNAL_SYNC,
	};

	static void Initialise();
	static void Shutdown();

	static void RegisterLabel(int iProfileType, const char* szName);
	static void StartProfile(int iProfileType);
	static void StopProfile(int iProfileType);

	static void Render();
	static void ClearValues();

	static void InstallSignalHandler(); 
	static void Signal(int iSignalID); 

	// Prints <szEvent> at time: <current time in microseconds>. Useful for profiling slow stuff like file reads.
	static void LogEventTime(const char* szEvent);
private:

	static void SignalHandler(int iSignalID);
};

#endif //#ifdef ___ASURA_PSP_PROFILING_H___

#endif //#ifdef ASURA_PSP_PROFILING
