//
// Asura_Assert.h
//
// Created by Kevin Lea - 12/06/2001 11:14:30
// 
// Copyright (C) 2001 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#define ASURA_PLATFORM_ASSERT_OVERRIDES_INDEPENDENT_CODE

#ifndef __ASURA_PLATFORM_ASSERT_H__
#define __ASURA_PLATFORM_ASSERT_H__

// Stdlib independent.
#include <stdlib.h>

extern void Asura_AssertfSetup( const char* szFile, int iLine, const char* szCondition, u_char* pucIgnore);

// Both of these functions return true iff we should break into the debugger.
// They are assumed to dealing with exiting the application internally.
extern bool Asura_AssertFired( const char* szFile, int iLine, const char* szCondition, const char* szDesc, u_char* pucIgnore );
extern bool Asura_AssertfFired( const char* szDescFormat, ... );

#ifdef _lint
// (MP - 16.09.2002) Shut Lint up about code like Asura_Assert( false, "" );
// and similar.
//lint -emacro(774,  Asura_Assert ) // Boolean always evaluates to true / false.
//lint -emacro(778,  Asura_Assert ) // Constant expression evaluates to 0.
//lint -emacro(1534, Asura_Assert ) // Static variable 'ignore' found within inline function in header.
//lint -emacro(774,  Asura_Assertf) // Boolean always evaluates to true / false.
//lint -emacro(778,  Asura_Assertf) // Constant expression evaluates to 0.
//lint -emacro(1534, Asura_Assertf) // Static variable 'ignore' found within inline function in header.
#endif

#ifdef ASURA_ASSERT

extern u_char g_ucAssertIgnoreCounter;

#if (defined __GNUC__ && !defined __SNC__)
	#define Asura_Assert(cond,desc) \
	do { \
		static u_char _ucIgnore = 0; \
		static const char* const _szCond = #cond; \
		auto const char* const _szDesc = (desc); \
		auto const bool _bCond = (cond); \
        \
		if ( (_ucIgnore != g_ucAssertIgnoreCounter) && !_bCond  \
			&& Asura_AssertFired(__FILE__,__LINE__, \
								 _szCond,_szDesc,&_ucIgnore)) \
        { asm("break 1"); asm("nop"); } \
	} while(0)

	#define Asura_Assertf(cond,params) \
    do { \
		static u_char _ucIgnore = 0; \
		static const char* const _szCond = #cond; \
		auto const bool _bCond = (cond); \
        \
	    if ( (_ucIgnore != g_ucAssertIgnoreCounter) && !_bCond  \
            && (Asura_AssertfSetup(__FILE__,__LINE__,\
								   _szCond,&_ucIgnore), \
                Asura_AssertfFired params)) \
        { asm("break 1"); asm("nop"); } \
    } while (0)
#else
    #define Asura_Assert(cond,desc) \
    do { \
		static u_char _ucIgnore = 0; \
		if (_ucIgnore != g_ucAssertIgnoreCounter) \
	    { \
		    if(!(cond)) \
		    { \
			    if (Asura_AssertFired(__SN_FILE__,__LINE__,#cond,desc,&_ucIgnore) ) \
			    { \
				    asm("break 1"); asm("nop");\
			    } \
		    } \
	    } \
    } while (0)

    // (MH - 01.11.2002) paramaterised assert. Usage example (note the brackets around arg 2):
    // Asura_Assertf(false, ("%s is invalid", szString));
    #define Asura_Assertf( __cond, __params ) \
    do { \
		static u_char _ucIgnore = 0; \
		if (_ucIgnore != g_ucAssertIgnoreCounter) \
	    { \
		    if( !(__cond) ) \
		    { \
                Asura_AssertfSetup(__SN_FILE__, __LINE__, #__cond, &_ucIgnore); \
			    if( Asura_AssertfFired __params ) \
			    { \
					asm("break 1"); asm("nop");\
				} \
		    } \
	    } \
    } while (0)
#endif
    
	//supresses all the assert dialogs , just leaving a single entry in the logfile per assert RAR 02/06/03
    void AsuraAssert_SetIgnoreAll(bool bIgnore);
	bool AsuraAssert_GetIgnoreAll();

	void AsuraAssert_ReEnableAll();	// Turn all asserts which have been skipped with "Ignore All" back on
	bool AsuraAssert_ConsoleCmd_EnableAsserts( bool bHelp );

    // (MH - 05.02.2007) PSP-specific function to allow notification that global mutex instance
	// has been destroyed. Asserts firing during shutdown can skip the mutex lock/unlock.
	void Asura_PSP_AssertShutdown(void);
	
#else // not def ASURA_ASSERT

#define Asura_Assert(ignore1,ignore2)
#define Asura_Assertf(ignore1,ignore2)

#define AsuraAssert_SetIgnoreAll(bIgnore)
#define AsuraAssert_GetIgnoreAll(bIgnore) true
#define AsuraAssert_ReEnableAll()
#define AsuraAssert_ConsoleCmd_EnableAsserts(bHelp) true

#endif // else not def ASURA_ASSERT


// KJFL - 22 March 2006 - Static, i.e. compile-time, assertions
// For documentation see http://asurawiki.rebellion.co.uk/twiki/bin/viewauth/Programming/AsuraStaticAssert

template<bool b> struct ASURA_STATIC_ASSERTION_FAILURE;
template <> struct ASURA_STATIC_ASSERTION_FAILURE<true> {};

#define ASURA_COMBINE_LITERALS( a, b ) a ## b
#define ASURA_STATIC_ASSERT_UNIQUENAME( a ) ASURA_COMBINE_LITERALS( ASURA_STATIC_ASSERT_FAILED_AT_LINE_, a )

#define ASURA_STATIC_ASSERT( bCondition ) static ASURA_STATIC_ASSERTION_FAILURE<(bCondition != 0)> ASURA_STATIC_ASSERT_UNIQUENAME(__LINE__) 


#endif // __ASURA_PLATFORM_ASSERT_H__
