//////////////////////////////////////////////////////////////////////////
// tgIoOpen.h
// PSP(TM) Programmer Tool Runtime Library Release 2.6.0
// cached high-speed sector based sceIoOpen() replacement function
//
// Copyright (C) 2004-2005 Sony Computer Entertainment Europe
// All Rights Reserved.
//////////////////////////////////////////////////////////////////////////

#ifndef __TGIOOPEN_H__
#define __TGIOOPEN_H__

// undefine if you know you have no hash collisions
#define TGIO_CHECK_COLLISIONS

// define if you want the filenames to be stored in the cache as well
#define TGIO_CACHE_STORE_FILENAME

// undefine if you only use it for UMD access
//#define TGIO_CHECK_FATDIRS

// define if you know how many files you'll have to cache
//#define TGIO_CACHE_NUM_FILES	(0)

// undefine if you don't want TTY output
#define	TGIO_REPORT_ERROR

//
#define TGIO_STRINGBUFFER_SIZE	(48)

// structure used to cache filedata
typedef struct tagTGIO_CACHE_ENTRY{
	unsigned int hash;
	#ifdef TGIO_CACHE_STORE_FILENAME
		char *name;
	#endif
	unsigned int lba;
	unsigned int size;
} TGIO_CACHE_ENTRY;


typedef struct tagTGIO_CACHE {
	int numFiles;
	TGIO_CACHE_ENTRY *cache;
} TGIO_CACHE;


// the prototypes
#ifdef __cplusplus
extern "C" {
#endif

// callback setting functions
void tgIoSetMemoryManager( void *(*alloc_func)( size_t size ), void (*free_func)( void *addr ) );
void tgIoSetHashFunction( unsigned int (*hash_func)( const char *string ) );

// hashing functions
unsigned int tgIoHash( const char *cdata );
TGIO_CACHE_ENTRY *tgIoFindHashB( unsigned int hash, TGIO_CACHE_ENTRY *walkList );
TGIO_CACHE_ENTRY *tgIoFindHash( unsigned int hash );
TGIO_CACHE_ENTRY *tgIoFindFile( const char *filename );
TGIO_CACHE_ENTRY *tgIoGetCachedName( const char *filename, char *outname );
const char *tgIoGetCachedNameA( const char *filename );

// fill and clear cache functions
int tgIoCountFiles( const char *dname, int recursive );
TGIO_CACHE *tgIoFillCache( const char *dname , TGIO_CACHE *cache, int recursive );
void tgIoClearCache( TGIO_CACHE *cache );

// functions to get some info from the cache
int tgIoGetNumFiles( void );
int tgIoFindCacheCollide( void );
unsigned int tgIoGetAllocatedBytes( void );

TGIO_CACHE *tgIoGetCache(void);
void tgIoSetCache(TGIO_CACHE *cache);

void tgIoSetCacheSort( void (*sort_func)( TGIO_CACHE *cache ) );
void tgIoSortCache(TGIO_CACHE *cache);

// the sceIoOpen() replacements itself
SceUID tgIoOpen( const char *filename, int flag, SceMode mode );
SceUID tgIoOpenAsync( const char *filename, int flag, SceMode mode );

// KOR - 25/02/2006 - extending provided sample with get size functionality
int tgIoGetFileSize( const char* szFilename );

#ifdef __cplusplus
}
#endif

#endif
