// MV_PSP_Main.cpp
//
// Created by KOR - 13/10/2005
// 
// Copyright (C) 2005 Rebellion, All Rights Reserved.

#include "Core/Asura.h"

// Asura headers
#include "Core/Asura_Core.h"
#include "Core/Asura_PSP_KernelManagement.h"
#include "Debugging/Asura_Assert.h"
#include "Debugging/Asura_Logfile.h"

// Project headers
#include "Main/Asura_Project_Main.h"

#include <stdio.h>
#include <kernel.h>
#include <moduleexport.h>
#ifdef ASURA_PSP_DEMO
#include <pspdemo.h>
#endif//def ASURA_PSP_DEMO

SCE_MODULE_INFO( Asura, 0, 1, 1 );

// KOR - 13/10/2005 - Urgh. Die die die.
extern char* DefaultFileToLoad;

// KOR - 13/10/2005 - Enable VFPU for main thread
#if (SCE_DEVKIT_VERSION >= 0x02080000)
int sce_user_main_thread_attribute = SCE_KERNEL_TH_USE_VFPU;
#else
int sce_newlib_attribute = SCE_KERNEL_TH_USE_VFPU;
#endif

// AJG - 31/10/05 - Following MH's suggestion, turned this off in
// submission builds as it adds code that almost certainly isn't
// needed then.
#if !(defined ASURA_PSP_SONY_SUBMISSION \
	|| (defined __GNUC__ && !defined __SNC__))
#define ASURA_PSP_MAIN_DO_GLOBAL_DTORS (1)
#endif

#ifdef ASURA_PSP_MAIN_DO_GLOBAL_DTORS
extern "C" void __do_global_dtors();

// JGP Project specific command line
#ifdef ASURA_USE_PROJECT_COMMAND_LINE
extern const bool Project_ProcessCmdLineToken( const char* const szToken );
#endif

void Asura_PSP_CallGlobalDestructors(void)
{
	__do_global_dtors();
}
#endif//def ASURA_PSP_MAIN_DO_GLOBAL_DTORS

int main(int argc,char **argv)
{
	// KOR - 17/10/2005 - don't want fp exceptions in art builds.
	#if !defined ASURA_ENABLE_FLOATING_POINT_EXCEPTIONS \
			|| defined ASURA_PSP_ARTIST_BUILD \
			|| defined ASURA_PSP_SONY_SUBMISSION

		// Disable fp exceptions if not requested
		__asm__ volatile("ctc1 %0, $31" : : "r" (0));
	#endif

#ifdef ASURA_PSP_DEMO
	sceDemoStart(argc, argv);
#endif//def ASURA_PSP_DEMO

	Asura_PSP_KernelManagement::EarlyInitialise();

	Asura_Core::EarlyInitialise();
	Asura_Project::EarlyInitialise();

	Asura_Core::LateInitialise();
	Asura_Project::LateInitialise();

#if !defined ASURA_PSP_SONY_SUBMISSION
	{
		fputs("Command line:",stdout);
		if (argc == 0)
		{
			fputs(" <empty>",stdout);
		}
		else
		{
			for (int i = 0; i < argc; i++)
			{
				sceKernelPrintf(" '%s'",argv[i]);				

// JGP Project specific command line
#ifdef ASURA_USE_PROJECT_COMMAND_LINE
				if ( i > 0 )
				{
					Project_ProcessCmdLineToken( argv[ i ] );
				}
#endif
			}
		}
		fputc('\n',stdout);
	}
#endif

    while (Asura_Project::MainLoop())
	{
		continue;
	}

#ifdef ASURA_PSP_DEMO
	Asura_OutputToDebugger_Retail(("Informing demo service we're quitting.\n"));
	sceDemoService( SCE_PSPDEMO_SERVICE_STATE_DEMO_QUIT, 0, NULL );
#endif //ASURA_PSP_DEMO

	Asura_Project::Shutdown();
	Asura_Core::Shutdown();

#ifdef ASURA_ASSERT		
	Asura_PSP_AssertShutdown();
#endif

#ifdef ASURA_PSP_MAIN_DO_GLOBAL_DTORS
	// AJG - 31/10/05 - Manually call the destructor function, as
	// this doesn't seem to be added automatically.
	Asura_PSP_CallGlobalDestructors();
#endif//def ASURA_PSP_MAIN_DO_GLOBAL_DTORS

#ifdef ASURA_PSP_DEMO
	Asura_OutputToDebugger_Retail(("Shutting down demo service.\n"));
	sceDemoEnd();
#endif //ASURA_PSP_DEMO

	return 0;
}
