//
// Asura_PSP_Adhoc.cpp
//
// Created by KOR - 22/02/2005
//
// Copyright (C) 2005 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

#ifdef ASURA_SUPPORT_EXTERNAL_NETWORK
#include "ProjectSettings/Asura_PSPConfiguration.h" // for product code

// This file's header.
#include "Network/Asura_PSP_Adhoc.h"

// Asura independent headers
#include "Core/Memory/Asura_Mem.h"

// Asura PSP headers
#include "Core/Asura_PlatformBasics.h"
#include "Core/Asura_PSP_KernelManagement.h"
#include "Core/Asura_PSP_Thread_Priorities.h"
#include "Debugging/Asura_Assert.h"
#include "Debugging/Asura_Logfile.h"

// PSP library headers
#include <pspnet.h>
#include <pspnet_adhoc.h>
#include <pspnet_adhocctl.h>
#include <thread.h>
#include <wlan.h>

#define PSPNET_POOLSIZE		128*1024	/* 128KB */
#define ADHOCCTL_STACKSIZE 	(8*1024)	/* 8KB */

#define CALLOUT_STACKSIZE	(0)	/* Now fixed to 4KB, should pass in zero according to docs */
#define NETINTR_STACKSIZE	(0)	/* Now fixed to 4KB, should pass in zero according to docs */

// Static members
ASURA_PSP_ADHOC_STATE	Asura_PSP_Adhoc::s_eState = ASURA_PSP_ADHOC_UNINITIALISED;
int						Asura_PSP_Adhoc::s_iHandlerID = 0;
// KOR - this should be moved into project code
SceNetAdhocctlAdhocId	Asura_PSP_Adhoc::s_xAdhocID =
{
    // GCC does not support named initializers in C++ source, only in C
#ifdef ASURA_PSP_SONY_SUBMISSION
    /* .type = */ SCE_NET_ADHOCCTL_ADHOCTYPE_PRODUCT_ID,	
#else
    /* .type = */ SCE_NET_ADHOCCTL_ADHOCTYPE_RESERVED,	
#endif 
    /* .data = */ ASURA_PSP_PRODUCT_CODE_UNTERMINATED,	// For example: ULUS99999
};
int						Asura_PSP_Adhoc::s_iNumScanResults = -1;
SceNetAdhocctlScanInfo*	Asura_PSP_Adhoc::s_pxScanResults = NULL;
int						Asura_PSP_Adhoc::s_iSemaphoreID = -1;
Asura_Mutex				Asura_PSP_Adhoc::s_xAdhocMutex;


void Asura_PSP_Adhoc::Initialise()
{
	Asura_OutputToDebugger(("Initialising Adhoc...\n"));

	Asura_Assert(!IsInitialised(), "Already initialised");
	if (IsInitialised()) 
	{
		return;
	}

	s_eState = ASURA_PSP_ADHOC_DISCONNECTED;
	int iRet;
	sceKernelDelayThread(30000);
	iRet = sceNetInit(PSPNET_POOLSIZE, iASURA_THREAD_PRIORITY_ADHOC_CALLOUT, CALLOUT_STACKSIZE, iASURA_THREAD_PRIORITY_NETINTR, NETINTR_STACKSIZE);
	if (iRet == SCE_KERNEL_ERROR_NO_MEMORY)
	{
		Asura_Assertf(false, ("not enough memory for sceNetInit() (need at least %d, max/total available %d/%d)", 
			PSPNET_POOLSIZE, sceKernelMaxFreeMemSize(), sceKernelTotalFreeMemSize()));
	}
	else
	{
		Asura_Assertf(iRet >= 0, ("sceNetInit(): Failed %08X", iRet));
	}
	if(iRet < 0) return;
	sceKernelDelayThread(30000);
	iRet = sceNetAdhocInit();
	Asura_Assertf(iRet >= 0, ("sceNetAdhocInit(): Failed %08X", iRet));
	if(iRet < 0) return;
	sceKernelDelayThread(30000);
	iRet = sceNetAdhocctlInit(ADHOCCTL_STACKSIZE, iASURA_THREAD_PRIORITY_ADHOCCTL, &s_xAdhocID);
	Asura_Assertf(iRet >= 0, ("sceNetAdhocctlInit(): Failed %08X", iRet));
	if(iRet < 0) return;
	sceKernelDelayThread(30000);
	iRet = sceNetAdhocctlAddHandler(&Handler, NULL);
	Asura_Assertf(iRet >= 0, ("sceNetAdhocctlAddHandler(): Failed %08X", iRet));
	if(iRet < 0) return;
	s_iHandlerID = iRet;

	sceKernelDelayThread(30000);
	sceKernelChangeThreadPriority(Asura_PSP_KernelManagement::GetMainThreadID(), iASURA_THREAD_PRIORITY_MAIN_THREAD_DURING_ADHOC);

	sceKernelDelayThread(30000);
	// Init semaphore
	if (s_iSemaphoreID < 0)
	{
		if ((s_iSemaphoreID = sceKernelCreateSema("ASURA_ADHOC_SEMA", SCE_KERNEL_SA_THFIFO, 1, 1, NULL)) < 0)
		{
			sceKernelPrintf("*** ERROR: %s(0x%08x)\n", __FUNCTION__, s_iSemaphoreID);
			return;
		}
	}
}

void Asura_PSP_Adhoc::Shutdown()
{
	Asura_OutputToDebugger(("Shutting down Adhoc...\n"));
	bool bCanShutdown = IsDisconnected() || IsWLANSwitchedOff();
	Asura_Assert(bCanShutdown, "Must be disconnected / WLAN off to shut down");
	if(!bCanShutdown) 
	{
		return;
	}

	if(s_pxScanResults) Asura_Deallocate(s_pxScanResults);
	s_pxScanResults		= NULL;
	s_iNumScanResults	= -1;

	sceKernelDelayThread(30000);
	int iRet = sceNetAdhocctlDelHandler(s_iHandlerID);
	Asura_Assertf(iRet >= 0, ("sceNetAdhocctlDelHandler(): Failed %08X", iRet));
	if(iRet < 0) return;
	sceKernelDelayThread(30000);
	iRet = sceNetAdhocctlTerm();
	Asura_Assertf(iRet >= 0, ("sceNetAdhocctlTerm(): Failed %08X", iRet));
	if(iRet < 0) return;
	sceKernelDelayThread(30000);
	iRet = sceNetAdhocTerm();
	Asura_Assertf(iRet >= 0, ("sceNetAdhocTerm(): Failed %08X", iRet));
	if(iRet < 0) return;
	sceKernelDelayThread(30000);
	iRet = sceNetTerm();
	Asura_Assertf(iRet >= 0, ("sceNetTerm(): Failed %08X", iRet));
	if(iRet < 0) return;

	sceKernelDelayThread(30000);
	sceKernelChangeThreadPriority(Asura_PSP_KernelManagement::GetMainThreadID(), iASURA_THREAD_PRIORITY_MAIN_THREAD_NORMAL);
	sceKernelDelayThread(30000);

	// Release the semaphore
	if (s_iSemaphoreID >= 0)
	{
		WaitSemaphoreCB();

		int uid = s_iSemaphoreID;
		s_iSemaphoreID = -1;

		sceKernelDeleteSema(uid);
	}
	s_eState			= ASURA_PSP_ADHOC_UNINITIALISED;
}

void Asura_PSP_Adhoc::Suspend (void)
{
	Asura_OutputToDebugger(("Suspending Adhoc...\n"));
	if(s_eState != ASURA_PSP_ADHOC_UNINITIALISED)
	{
		s_xAdhocMutex.Lock();

		if(s_pxScanResults) Asura_Deallocate(s_pxScanResults);
		s_pxScanResults = NULL;
		s_iNumScanResults	= -1;

		sceKernelDelayThread(30000);
		int iRet = sceNetAdhocctlDelHandler(s_iHandlerID);
		Asura_Assertf(iRet >= 0, ("sceNetAdhocctlDelHandler(): Failed %08X", iRet));
		if(iRet < 0)
		{
			s_xAdhocMutex.Unlock();
			return;
		}
		sceKernelDelayThread(30000);
		iRet = sceNetAdhocctlTerm();
		Asura_Assertf(iRet >= 0, ("sceNetAdhocctlTerm(): Failed %08X", iRet));
		if(iRet < 0)
		{
			s_xAdhocMutex.Unlock();
			return;
		}
		sceKernelDelayThread(30000);
		iRet = sceNetAdhocTerm();
		Asura_Assertf(iRet >= 0, ("sceNetAdhocTerm(): Failed %08X", iRet));
		if(iRet < 0)
		{
			s_xAdhocMutex.Unlock();
			return;
		}
		sceKernelDelayThread(30000);
		iRet = sceNetTerm();
		Asura_Assertf(iRet >= 0, ("sceNetTerm(): Failed %08X", iRet));
		if(iRet < 0)
		{
			s_xAdhocMutex.Unlock();
			return;
		}

		sceKernelDelayThread(30000);
		sceKernelChangeThreadPriority(Asura_PSP_KernelManagement::GetMainThreadID(), iASURA_THREAD_PRIORITY_MAIN_THREAD_NORMAL);
		sceKernelDelayThread(30000);

		// Release the semaphore
		if (s_iSemaphoreID >= 0)
		{
			WaitSemaphoreCB();

			int uid = s_iSemaphoreID;
			s_iSemaphoreID = -1;

			sceKernelDeleteSema(uid);
		}
		s_eState	= ASURA_PSP_ADHOC_UNINITIALISED;

		s_xAdhocMutex.Unlock();
	}
}

void Asura_PSP_Adhoc::Resume (void)
{
}

void Asura_PSP_Adhoc::Connect(const char* szGroupName)
{
	Asura_Assert(strlen(szGroupName) <= 8, "Group name too long");
	SceNetAdhocctlGroupName xName;
	Asura_Mem::Copy(xName.data, szGroupName, 8);
	int iRet = sceNetAdhocctlConnect(&xName);
	Asura_Assertf(iRet >= 0, ("Error 0x%x returned from sceNetAdhocctlConnect", iRet));
	if(iRet < 0) return;
	WaitSemaphore();
	s_eState = ASURA_PSP_ADHOC_CONNECTING;
	SignalSemaphore();
}

void Asura_PSP_Adhoc::Disconnect()
{
	int iRet = sceNetAdhocctlDisconnect();
	Asura_Assertf(iRet >= 0, ("Error 0x%x returned from sceNetAdhocctlDisconnect", iRet));
	if(iRet < 0) return;	
	WaitSemaphore();
	s_eState			= ASURA_PSP_ADHOC_DISCONNECTING;
	SignalSemaphore();
	s_iNumScanResults	= -1;
}

void Asura_PSP_Adhoc::Scan()
{
	int iRet = sceNetAdhocctlScan();
	Asura_Assertf(iRet >= 0, ("Error 0x%x returned from sceNetAdhocctlScan", iRet));
	if(iRet < 0) return;	
	WaitSemaphore();
	s_eState = ASURA_PSP_ADHOC_SCANNING;
	SignalSemaphore();
}

void Asura_PSP_Adhoc::Create(const char* szGroupName)
{
	Asura_Assert(strlen(szGroupName) <= 8, "Group name too long");
	SceNetAdhocctlGroupName xName;
	Asura_Mem::Copy(xName.data, szGroupName, 8);
	int iRet = sceNetAdhocctlCreate(&xName);
	Asura_Assertf(iRet >= 0, ("Error 0x%x returned from sceNetAdhocctlCreate", iRet));
	if(iRet < 0) return;	
	WaitSemaphore();
	s_eState = ASURA_PSP_ADHOC_CONNECTING;
	SignalSemaphore();
}

bool Asura_PSP_Adhoc::Join(int iGroup)
{
	Asura_Assert((s_iNumScanResults > 0) && s_pxScanResults, "No scan results to join");
	if((s_iNumScanResults <= 0) || !s_pxScanResults) return false;

	Asura_Assert(0 <= iGroup && iGroup < s_iNumScanResults, "Group index out of range");
	if(0 > iGroup || iGroup >= s_iNumScanResults) return false;

	int iRet = sceNetAdhocctlJoin(&s_pxScanResults[iGroup]);
	Asura_Assertf(iRet >= 0, ("Error 0x%x returned from sceNetAdhocctlJoin", iRet));
	if(iRet < 0) return false;	
	WaitSemaphore();
	s_eState = ASURA_PSP_ADHOC_CONNECTING;
	SignalSemaphore();

	return true;
}

int Asura_PSP_Adhoc::GetNumScanResults()
{
	return s_iNumScanResults;
}

const char* Asura_PSP_Adhoc::GetScanResult(int iGroup)
{
	Asura_Assert((s_iNumScanResults >0) && s_pxScanResults, "No scan results to return");
	if((s_iNumScanResults <= 0) || !s_pxScanResults) return NULL;

	Asura_Assert(0 <= iGroup && iGroup < s_iNumScanResults, "Group index out of range");
	if(0 > iGroup || iGroup >= s_iNumScanResults) return NULL;
	static char szGroupName[9];
	Asura_Mem::Copy(szGroupName, &s_pxScanResults[iGroup].group_name.data, 8);
	// Make sure it's terminated
	szGroupName[8] = 0;
	return szGroupName;
}

void Asura_PSP_Adhoc::Update()
{
	int iSwitchState = sceWlanGetSwitchState();
	if(iSwitchState == SCE_WLAN_SWITCH_STATE_OFF)
	{
		WaitSemaphore();
		s_eState = ASURA_PSP_ADHOC_WLAN_SWITCH_OFF;
		SignalSemaphore();
		return;
	}
	Asura_Assert(iSwitchState == SCE_WLAN_SWITCH_STATE_ON, "Unexpected switch state");

	s_xAdhocMutex.Lock();

	int iAdhocctlState;
	int iRet = sceNetAdhocctlGetState(&iAdhocctlState);
	Asura_Assert(iRet >= 0, "Error returned from sceNetAdhocctlGetState");
	
	
	if(iRet < 0)
	{
		s_xAdhocMutex.Unlock();
		return;
	}

	Asura_Assert(iAdhocctlState != SCE_NET_ADHOCCTL_STATE_GAMEMODE, "Should not be using game mode");

	WaitSemaphore();
	switch(s_eState)
	{
		case ASURA_PSP_ADHOC_UNINITIALISED: 
		{
			Asura_Assert(0, "Should not update while uninitialised"); 
			break;
		}
		case ASURA_PSP_ADHOC_DISCONNECTED: // Nothing to do while disconnected
		{
			Asura_Assert(iAdhocctlState == SCE_NET_ADHOCCTL_STATE_DISCONNECTED, "State mismatch");
			break;
		}
		case ASURA_PSP_ADHOC_CREATING:
		case ASURA_PSP_ADHOC_JOINING:
		case ASURA_PSP_ADHOC_CONNECTING:
		{
			if(iAdhocctlState == SCE_NET_ADHOCCTL_STATE_CONNECTED)
			{
				s_eState = ASURA_PSP_ADHOC_CONNECTED;
			}
			break;
		}
		case ASURA_PSP_ADHOC_CONNECTED:
		// If connected, a library error could send us to disconnected. Handle in fall-through:
		case ASURA_PSP_ADHOC_DISCONNECTING:
		{
			if(iAdhocctlState == SCE_NET_ADHOCCTL_STATE_DISCONNECTED)
			{
				s_eState			= ASURA_PSP_ADHOC_DISCONNECTED;
				s_iNumScanResults	= -1;
			}
			break;
		}
		case ASURA_PSP_ADHOC_SCANNING:
		{
			if(iAdhocctlState == SCE_NET_ADHOCCTL_STATE_DISCONNECTED)
			{
				// get scan results
				int iBufferLength;
				int iRet;
				iRet = sceNetAdhocctlGetScanInfo(&iBufferLength, NULL);
				Asura_Assertf(iRet >= 0, ("Error 0x%x returned from sceNetAdhocctlGetScanInfo", iRet));
				if(iRet < 0) 
				{
					break;
				}
				if(iBufferLength)
				{
					Asura_Assert(!(iBufferLength % sizeof(SceNetAdhocctlScanInfo)), "Unexpected buffer size");
					s_iNumScanResults = iBufferLength / sizeof(SceNetAdhocctlScanInfo);
					if(s_pxScanResults) 
					{
						Asura_Deallocate(s_pxScanResults);
					}
					s_pxScanResults = reinterpret_cast<SceNetAdhocctlScanInfo*>(Asura_Allocate(iBufferLength));
					Asura_Assert(s_pxScanResults, "Failed to allocate buffer for scan results");
					iRet = sceNetAdhocctlGetScanInfo(&iBufferLength, s_pxScanResults);
					Asura_Assertf(iRet >= 0, ("Error 0x%x returned from sceNetAdhocctlGetScanInfo", iRet));
					if(iRet < 0) 
					{
						break;
					}
				}
				else
				{
					s_iNumScanResults = 0;
				}
				s_eState = ASURA_PSP_ADHOC_DISCONNECTED;
			}
			break;
		}
		case ASURA_PSP_ADHOC_WLAN_SWITCH_OFF:
		{
			// If we've got this far, then the switch is on. Move back to disconnected state.
			// (KOR - 13/04/2005 - TODO - confirm that it is not necessary to re-initialise the libraries when the switch is turned off)
			s_eState = ASURA_PSP_ADHOC_DISCONNECTED;
			break;
		}
		default: 
			Asura_Assert(0, "Unknown state");
			break;
	}
	SignalSemaphore();
	s_xAdhocMutex.Unlock();
}

void Asura_PSP_Adhoc::Handler(int iEvent, int iError, void *pvArg)
{
	Asura_Assert(IsInitialised(), "Why have we got an event to handle when we're uninitialised?");
	if(iError)
	{
		switch(s_eState)
		{
		case ASURA_PSP_ADHOC_CREATING:
		case ASURA_PSP_ADHOC_JOINING:
		case ASURA_PSP_ADHOC_CONNECTING:
		case ASURA_PSP_ADHOC_SCANNING:
		{
			// (AW - 8/12/05) - I think that most errors during connection will result in us never successfully connecting.
			// Go into our disconnected state so that we don't wait forever for a connection that'll never happen.
			WaitSemaphore();
			s_eState = ASURA_PSP_ADHOC_DISCONNECTED;
			SignalSemaphore();
			break;
		}
		default:
			break;
		}
		sceKernelPrintf("Asura_PSP_Adhoc error: 0x%x (event code 0x%x)\n", iError, iEvent);
	}
	else
	{
		sceKernelPrintf("Asura_PSP_Adhoc event: 0x%x\n", iEvent);
	}
}

// Semaphore
int	Asura_PSP_Adhoc::PollSemaphore()
{
	return sceKernelPollSema(s_iSemaphoreID, 1);
}

int	Asura_PSP_Adhoc::WaitSemaphore()
{
	return sceKernelWaitSema(s_iSemaphoreID, 1, NULL);
}

int	Asura_PSP_Adhoc::WaitSemaphoreCB()
{
	return sceKernelWaitSemaCB(s_iSemaphoreID, 1, NULL);
}

int	Asura_PSP_Adhoc::SignalSemaphore()
{
	return sceKernelSignalSema(s_iSemaphoreID, 1);
}
#endif//ASURA_SUPPORT_EXTERNAL_NETWORK
