//
// Asura_PSP_Matching.cpp
//
// Created by KOR - 22/02/2005
//
// Copyright (C) 2005 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

#ifdef ASURA_SUPPORT_EXTERNAL_NETWORK

// This file's header.
#include "Network/Asura_PSP_Matching.h"
// Asura headers
#include "Debugging/Asura_Assert.h"
#include "Network/Asura_PSP_Adhoc.h"
// PSP headers
#include <pspnet_adhoc_matching.h>

// Static members
bool	Asura_PSP_Matching::s_bInitialised = false;
int		Asura_PSP_Matching::s_iContextID = -1;
bool	Asura_PSP_Matching::s_bParent = false;

#define MATCHING_EVENT_TPL			54
#define MATCHING_EVENT_STACKSIZE	(8*1024)	/* 8KB */
#define MATCHING_INPUT_TPL			54
#define MATCHING_INPUT_STACKSIZE	(8*1024)	/* 8KB */

#define	MATCHING_POOLSIZE	32*1024		/* 32KB */
#define MATCHING_PORT		1
#define MATCHING_RXBUFLEN	1024
#define HELLO_INTERVAL		1*1000*1000		/* 5sec */
#define KEEPALIVE_INTERVAL	2*1000*1000		/* 2sec */
#define KEEPALIVE_COUNT		3
#define REXMT_INTERVAL		500*1000		/* 500msec */

#define HELLO_PORT			7
#define HELLO_TPL			32
#define HELLO_STACKSIZE		(1024 * 8)

void Asura_PSP_Matching::Initialise(bool bParent)
{
	Asura_Assert(Asura_PSP_Adhoc::IsConnected(), "Asura_PSP_Adhoc must be connected before Asura_PSP_Matching is initialised");
	if(!Asura_PSP_Adhoc::IsConnected()) return;
	Asura_Assert(!s_bInitialised, "Already initialised");
	if(s_bInitialised) return;

	// Create the matching instance
	int iRet;
	iRet = sceNetAdhocMatchingCreate(s_bParent ? SCE_NET_ADHOC_MATCHING_MODE_PARENT : SCE_NET_ADHOC_MATCHING_MODE_CHILD,
									4, // TODO  - correct number of players
									MATCHING_PORT, MATCHING_RXBUFLEN, HELLO_INTERVAL,
									KEEPALIVE_INTERVAL, KEEPALIVE_COUNT, REXMT_INTERVAL,
									s_bParent ? ParentHandler : ChildHandler);
	Asura_Assert(iRet >= 0, "Error returned from sceNetAdhocMatchingCreate");
	if(iRet < 0) return;
	s_iContextID = iRet;

	iRet = sceNetAdhocMatchingStart(s_iContextID, MATCHING_EVENT_TPL, MATCHING_EVENT_STACKSIZE,
									MATCHING_INPUT_TPL, MATCHING_INPUT_STACKSIZE, 100, NULL); // TODO - hello options
	Asura_Assert(iRet >= 0, "Error returned from sceNetAdhocMatchingStart");
	if(iRet < 0) return;

	s_bParent = bParent;
	s_bInitialised = true;
}

void Asura_PSP_Matching::Shutdown()
{
	Asura_Assert(s_bInitialised , "Not initialised");
	if(!s_bInitialised) return;

	sceNetAdhocMatchingStop(s_iContextID);
	sceNetAdhocMatchingDelete(s_iContextID);

	s_bInitialised = false;
}

void Asura_PSP_Matching::Update()
{
}

void Asura_PSP_Matching::ParentHandler(int iID, int iEvent, SceNetEtherAddr *pxPeer, int iOptlen, void *pvOpt)
{
	Asura_Assert(s_bParent, "Should not call parent handler as child");
}

void Asura_PSP_Matching::ChildHandler(int iID, int iEvent, SceNetEtherAddr *pxPeer, int iOptlen, void *pvOpt)
{
	Asura_Assert(!s_bParent, "Should not call child handler as parent");
}

#endif//ASURA_SUPPORT_EXTERNAL_NETWORK
