//////////////////////////////////////////////////////////////////////////////
//
// Asura_Platform_NetLinkManager.cpp
//
// Copyright (c) 2007 Rebellion. All rights reserved.
//
// Description:
//
//     PSP implementation of network link class functions.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

#ifdef ASURA_SUPPORT_EXTERNAL_NETWORK

// Asura
#include "Core/Asura_Timers.h"
#include "Network/Asura_NetLinkManager.h"

// PSP
#include "Network/Asura_PSP_Adhoc.h"
#include "Network/Asura_PSP_InfrastructureMode.h"

// libraries
#include <wlan.h>

// D E B U G V A R S /////////////////////////////////////////////////////////
DEBUGVAR u_int dbg_uStartNetTimeout = 10000;// 10 secs

// G L O B A L S /////////////////////////////////////////////////////////////
static u_int g_uStartNetworkTimeStamp = 0;

// C L A S S   I M P L E M E N T A T I O N ///////////////////////////////////

// Platform_UpdateXXX() functions should set status directly when finished
void Asura_NetLinkManager::Platform_UpdateConnecting()
{
	if (s_uType == ASURA_NETLINK_LAN)
	{
		int iSwitchState = sceWlanGetSwitchState();
		s_bWLANSwitchedOff = (iSwitchState == SCE_WLAN_SWITCH_STATE_OFF);

		Asura_PSP_Adhoc::Update();

		if (s_bWLANSwitchedOff || ( (Asura_Timers::GetMSecsSinceStart() - g_uStartNetworkTimeStamp) > dbg_uStartNetTimeout) )
		{
			// timeout
			Asura_PSP_Adhoc::Shutdown();
			FinishedPlatformConnecting(false);
		}
		else if (Asura_PSP_Adhoc::IsConnected())
		{
			FinishedPlatformConnecting(true);
		}
	}
}

void Asura_NetLinkManager::Platform_UpdateConnecting_PostRender()
{
	if (s_uType == ASURA_NETLINK_INTERNET)
	{
#ifdef ASURA_PSP_SUPPORT_INFRASTRUCTURE_NETWORK
		Asura_PSP_InfrastructureMode::UpdateConnecting();
		Asura_PSP_InfrastructureMode::RenderConnecting();	// does SCEutility rendering
		if (!Asura_PSP_InfrastructureMode::IsConnecting())
		{
			bool bSuccess = Asura_PSP_InfrastructureMode::IsConnected();
			if (!bSuccess)
			{
				Asura_PSP_InfrastructureMode::CloseConnection();
			}
			FinishedPlatformConnecting(bSuccess);
		}
#else
		Asura_Assert(false, "not supported");
#endif// #ifdef ASURA_PSP_SUPPORT_INFRASTRUCTURE_NETWORK
	}
}

void Asura_NetLinkManager::Platform_UpdateDisconnecting()
{
	if (s_uType == ASURA_NETLINK_INTERNET)
	{
		FinishedPlatformDisconnecting();
	}
	else
	{
		Asura_PSP_Adhoc::Update();
		if (Asura_PSP_Adhoc::IsDisconnected() || Asura_PSP_Adhoc::IsWLANSwitchedOff())
		{
			Asura_PSP_Adhoc::Shutdown();
			FinishedPlatformDisconnecting();
		}
	}
}

void Asura_NetLinkManager::Platform_UpdateConnected()
{
#ifdef ASURA_PSP_SUPPORT_INFRASTRUCTURE_NETWORK
	if (s_uType == ASURA_NETLINK_INTERNET)
	{
		int iSwitchState = sceWlanGetSwitchState();
		if(iSwitchState == SCE_WLAN_SWITCH_STATE_OFF)
		{
			s_bWLANSwitchedOff = true;
			ConnectionDropped();
		}
		else if (!Asura_PSP_InfrastructureMode::IsConnected())
		{
			ConnectionDropped();
		}
	}
	else
#endif
	{
		Asura_PSP_Adhoc::Update();
		ASURA_PSP_ADHOC_STATE eState = Asura_PSP_Adhoc::GetState();
		if (eState == ASURA_PSP_ADHOC_WLAN_SWITCH_OFF)
		{
			s_bWLANSwitchedOff = true;
			ConnectionDropped();
		}
		else if (eState == ASURA_PSP_ADHOC_DISCONNECTED)
		{
			ConnectionDropped();
		}
	}
}

// Platform_InitiateConnection() function should return true iff requested operation was started.
bool Asura_NetLinkManager::Platform_InitiateConnection()
{
	int iSwitchState = sceWlanGetSwitchState();
	s_bWLANSwitchedOff = (iSwitchState == SCE_WLAN_SWITCH_STATE_OFF);

	g_uStartNetworkTimeStamp = Asura_Timers::GetMSecsSinceStart();

	if (s_uType == ASURA_NETLINK_INTERNET)
	{
#ifdef ASURA_PSP_SUPPORT_INFRASTRUCTURE_NETWORK
		Asura_PSP_InfrastructureMode::BeginConnecting();
		return true;
#else
		Asura_Assert(false, "not supported");
		return false;
#endif// #ifdef ASURA_PSP_SUPPORT_INFRASTRUCTURE_NETWORK
	}
	else
	{
		Asura_PSP_Adhoc::Initialise();
		Asura_PSP_Adhoc::Connect("PSPGame");	// TODO specify per game before connecting.
	}
	return true;
}

void Asura_NetLinkManager::Platform_InitiateDisconnection()
{
#ifdef ASURA_PSP_SUPPORT_INFRASTRUCTURE_NETWORK
	if (s_uType == ASURA_NETLINK_INTERNET)
	{
		Asura_PSP_InfrastructureMode::CloseConnection();
	}
	else
#endif // #ifdef ASURA_PSP_SUPPORT_INFRASTRUCTURE_NETWORK
	if (s_uType == ASURA_NETLINK_LAN)
	{
		Asura_PSP_Adhoc::Disconnect();
	}
}


#endif // #ifdef ASURA_SUPPORT_EXTERNAL_NETWORK

