//
// Asura_Platform_NetSocket.cpp
//
// PSP version
//
// Created by KOR - 18.02.2005
// On the PSP, this contains both Platform and API implementations for Asura_NetSocket
// (on other platforms, API stuff goes in the independent Asura_NetSocket_BSD.cpp)
//
// Copyright (C) 2005 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

#ifdef ASURA_SUPPORT_EXTERNAL_NETWORK

// This file's header.
#include "Network/Asura_NetSocket.h"

// Asura headers
#include "Network/Asura_PSP_Adhoc.h"
#include "Network/Asura_Platform_NetSocket_PSPAdhoc.h"

// PSP library headers
#include <pspnet_adhoc.h>
#include <pspnet_error.h>
#include <thread.h>

bool Asura_NetSocket::Platform_SendTo(const Asura_NetSocket_MessageInfo* pxMessageInfo)
{
	// We ignore pxMessageInfo->m_bEncrypted
	//char szAddress[24];
	//API_GetStringFromAddress(szAddress, pxMessageInfo->m_xAddress, pxMessageInfo->m_usPort);
	//printf("Sending message to %s.\n", szAddress);

	int iRet = -1;
	if (Asura_PSP_Adhoc::IsInitialised())
	{
		iRet = sceNetAdhocPdpSend(m_xSocket, &pxMessageInfo->m_xAddress.GetAdhocAddress(), pxMessageInfo->m_usPort, 
			pxMessageInfo->m_pcMessage, pxMessageInfo->m_iLength, 0, SCE_NET_ADHOC_F_NONBLOCK);
		Asura_Assertf(iRet >= 0, ("Error 0x%x returned from sceNetAdhocPdpSend", iRet));
	}
#ifdef ASURA_PSP_SUPPORT_INFRASTRUCTURE_NETWORK
	else
	{
		SOCKADDR_IN xAddress;
		memset(&xAddress, 0, sizeof(SOCKADDR_IN));
		xAddress.sin_family = AF_INET;
		xAddress.sin_addr.s_addr = pxMessageInfo->m_xAddress.GetBSDAddress();
		xAddress.sin_port = htons(pxMessageInfo->m_usPort);

		iRet = sceNetInetSendto(m_xSocket, pxMessageInfo->m_pcMessage, pxMessageInfo->m_iLength, 0, (SOCKADDR *)&xAddress, sizeof(SOCKADDR_IN)) != SOCKET_ERROR;
		Asura_Assertf(iRet >= 0, ("Error 0x%x returned from sceNetInetSendto", iRet));
	}
#endif // #ifdef ASURA_PSP_SUPPORT_INFRASTRUCTURE_NETWORK
	return (iRet >= 0);
}

bool Asura_NetSocket::Platform_GetData(const char*& pcMessage, int& iLength)
{
	return true;
}

bool Asura_NetSocket::Platform_SetToNonBlocking()
{
	return true;
}

// needed for ps2 only.
void Asura_NetSocket::Platform_Update()
{
}


// Platform_GetINetAddress() - static function
u_int Asura_NetSocket::Platform_GetINetAddress( const Asura_NetAddress& xAddress )
{
#ifdef ASURA_PSP_SUPPORT_INFRASTRUCTURE_NETWORK
	if (!Asura_PSP_Adhoc::IsInitialised())
	{
		return xAddress.GetBSDAddress();
	}
	else
#endif
	{
		Asura_Assert(false, "shouldn't be called in adhoc mode");
		return 0;
	}
}



// API implementations
bool Asura_NetSocket_PSPAdhoc::API_Bind(Asura_NetAddress xAddress, u_short &usPort)
{
/*
	SOCKADDR_IN xAddress;
	int iLength;

	bool bAllowPortScan = false;
	if (usPort == 0)
	{
		usPort = 
#ifdef ASURA_NETSOCK_ALTERNATIVE_PROTOCOL
			(m_bUseAlternativeProtocol) ? ASURA_NETSOCKET_ALTERNATIVE_PORT : 
#endif
			ASURA_NETSOCKET_DEFAULTPORT;
		Asura_Assert(usPort < ASURA_NETSOCKET_FIRST_DYNAMIC_PORT, "first dynamic port must be higher than default port");
		bAllowPortScan = true;
	}
	
	int iErrorCode;
	do
	{
		iErrorCode = 0;
		// Set up default options
		m_uIP = uIP;
		m_usPort = usPort;
		m_xSocket = INVALID_SOCKET;
		m_bCloseRequested = false;
		m_bError = false;
		m_bConnected = false;

		// RJF - 28 January 2005 - Select protocol to use. If a broadcast protocol is not defined this will always evaluate to ASURA_NETSOCKET_PROTOCOL
		// otherwise it chooses between PROTOCOL and PROTOCOL_BROADCAST based on m_bBroadcastFlag
		int iProtocol =
#ifdef ASURA_NETSOCKET_ALTERNATIVE_PROTOCOL
			(m_bUseAlternativeProtocol) ? ASURA_NETSOCKET_ALTERNATIVE_PROTOCOL :
#endif
		ASURA_NETSOCKET_PROTOCOL;

		// Attempt to create socket
		m_xSocket = socket(AF_INET, SOCK_DGRAM, iProtocol);
		if(m_xSocket == INVALID_SOCKET)
		{
			int iErrorCode = WSAGetLastError();
			switch(iErrorCode)
			{
				// (MH - 20.07.2004) sadly there is no equivalent to WSANOTINITIALISED on PS2
				// case WSANOTINITIALISED:		Asura_Assert(0, "Failed to create socket: WSANOTINITIALISED");		break;
				case WSAENETDOWN:			Asura_Assert(0, "Failed to create socket: WSAENETDOWN");			break;
				case WSAEAFNOSUPPORT:		Asura_Assert(0, "Failed to create socket: WSAEAFNOSUPPORT");		break;
				case WSAENOBUFS:			Asura_Assert(0, "Failed to create socket: WSAENOBUFS");				break;
				case WSAEPROTONOSUPPORT:	Asura_Assert(0, "Failed to create socket: WSAEPROTONOSUPPORT");		break;
				default:					Asura_Assert(0, "Failed to create socket: Unknown error");			break;
			}		
			return false;
		}


		// Bind socket
		memset(&xAddress, 0, sizeof(SOCKADDR_IN));
		xAddress.sin_family = AF_INET;
		xAddress.sin_addr.s_addr = INADDR_ANY;
		xAddress.sin_port = htons(usPort);
		
		if(bind(m_xSocket, (SOCKADDR *)&xAddress, sizeof(SOCKADDR_IN)) == SOCKET_ERROR)
		{
			iErrorCode = WSAGetLastError();
			switch(iErrorCode)
			{
				// (MH - 20.07.2004) sadly there is no equivalent to WSANOTINITIALISED on PS2
				// case WSANOTINITIALISED:		Asura_Assert(0, "Failed to create socket: WSANOTINITIALISED");		break;
				case WSAEADDRINUSE:
					{
						//Asura_CommandConsole::Printf(L"Port %d already in use.", usPort);
						break;
					}
				case WSAENETDOWN:			Asura_Assert(0, "Failed to create socket: WSAENETDOWN");			break;
				case WSAEAFNOSUPPORT:		Asura_Assert(0, "Failed to create socket: WSAEAFNOSUPPORT");		break;
				case WSAENOBUFS:			Asura_Assert(0, "Failed to create socket: WSAENOBUFS");				break;
				case WSAEPROTONOSUPPORT:	Asura_Assert(0, "Failed to create socket: WSAEPROTONOSUPPORT");		break;
				default:					Asura_Assert(0, "Failed to create socket: Unknown error");			break;
			}		
			Asura_Assert(iErrorCode == WSAEADDRINUSE, "Failed to bind socket.");
			closesocket(m_xSocket);
			m_xSocket = INVALID_SOCKET;

			if ( (!bAllowPortScan) || (usPort >= (ASURA_NETSOCKET_FIRST_DYNAMIC_PORT + 64)) || (iErrorCode != WSAEADDRINUSE) )
			{
				// only allow scan of up to 64 ports to find a free one
				return false;
			}

			// if default port failed, use dynamic port range.
			usPort = (usPort == ASURA_NETSOCKET_DEFAULTPORT) ? ASURA_NETSOCKET_FIRST_DYNAMIC_PORT : (++usPort);
		}
	}
	while (iErrorCode != 0);

	//Asura_CommandConsole::Printf(L"Network socket binded. Using Port %d.", usPort);

	// Set up members
	iLength = sizeof(SOCKADDR_IN);
	getsockname(m_xSocket, (SOCKADDR *)&xAddress, &iLength);
	m_uIP = xAddress.sin_addr.s_addr;
	m_usPort = ntohs(xAddress.sin_port);

*/
	if(usPort)
	{
		m_usPort = usPort;
	}
	else
	{
		m_usPort = ASURA_NETSOCKET_DEFAULTPORT;
	}
	int iRet;
	Asura_Assert(Asura_PSP_Adhoc::IsInitialised(), "Asura_PSP_Adhoc should be initialised before attempting to create the socket");
	// Set address
	SceNetEtherAddr xAdhocAddress;
	iRet = sceNetGetLocalEtherAddr(&xAdhocAddress);
	Asura_Assertf(iRet >= 0, ("Error 0x%x returned from sceNetGetLocalEtherAddr", iRet));
	if(iRet < 0) return false;
	m_xAddress = xAdhocAddress;

	// Create & bind socket
	iRet = sceNetAdhocPdpCreate(&m_xAddress.GetAdhocAddress(), m_usPort, ASURA_RECEIVE_BUFFER_SIZE, 0);
	Asura_Assertf(iRet >= 0, ("Error 0x%x returned from sceNetAdhocPdpCreate", iRet));
	if(iRet < 0) return false;
	m_xSocket = iRet;

	char szEtherAddr[18];
	sceNetEtherNtostr(&m_xAddress.GetAdhocAddress(), szEtherAddr);
	sceKernelPrintf("Socket opened at %s, port %d\n", szEtherAddr, m_usPort);

	m_bConnected = true;
	m_bListening = false;
	m_uNumConnections = 0;

	// PSP non-blocking is done as a parameter in each function call
	// Set as non blocking
	//Platform_SetToNonBlocking();

// (MH - 21.11.2005) connections created/destroyed by ctor/dtor
//	// Zero connections
//	for(int i = 0; i < ASURA_NETSOCKET_MAX_CONNECTIONS; i++)
//	{
//		m_apxConnections[i] = 0;
//	}

	return true;
}

void Asura_NetSocket_PSPAdhoc::API_Close()
{
	sceKernelPrintf("Socket closed.\n");
	if(m_bConnected)
	{
		if(m_xSocket > 0)
		{
			sceNetAdhocPdpDelete(m_xSocket, 0);
		}
		m_bConnected = false;
	}
}

bool Asura_NetSocket_PSPAdhoc::API_ReceiveMessages()
{
	int iLength, iRet;
	SceNetEtherAddr xFromAddress;
	u_short usFromPort;
	char acBuffer[ASURA_RECEIVE_BUFFER_SIZE];
	sceKernelDelayThread(500);
	while(1)
	{
		iLength = ASURA_RECEIVE_BUFFER_SIZE;
		iRet = sceNetAdhocPdpRecv(m_xSocket, &xFromAddress, &usFromPort, acBuffer, &iLength, 0, SCE_NET_ADHOC_F_NONBLOCK);
		if(iRet == SCE_ERROR_NET_ADHOC_WOULD_BLOCK)
		{
			// No more messages
			break;
		}
		else if(iRet < 0)
		{
			// Error
			Asura_Assertf(0, ("Error 0x%xreturned from sceNetAdhocPdpRecv", iRet));
			return false;
		}
		//printf("Socket received message.\n");
		if(!HandleMessage(acBuffer, iLength, Asura_NetAddress(xFromAddress), usFromPort))
		{
			return false;
		}
	}
	return true;
/*
	fd_set xFDSet;
	struct timeval timeout = {0, 0};
	int iNumSockets;

	FD_ZERO(&xFDSet);
	FD_SET(m_xSocket, &xFDSet);

	while((iNumSockets = select(FD_SETSIZE, &xFDSet, NULL, NULL, &timeout)) > 0)
	{
		SOCKADDR_IN xAddress;
		int iAddressLength;
		int iBytesReceived;
		char pcBuffer[ASURA_RECEIVE_BUFFER_SIZE];

		iAddressLength = sizeof(xAddress);
		iBytesReceived = recvfrom(m_xSocket, pcBuffer, sizeof(pcBuffer), 0, (SOCKADDR *)&xAddress, &iAddressLength);
		if(iBytesReceived == SOCKET_ERROR)
		{
			int iLastError = WSAGetLastError();
			if(iLastError == WSAECONNRESET)
			{
				if(!ResetConnection(xAddress.sin_addr.s_addr, ntohs(xAddress.sin_port)))
				{
					return false;
				}
			}
			else if(iLastError != WSAEMSGSIZE)
			{
				FatalError();
				return false;
			}
		}
		else
		{
			Asura_Assert(iBytesReceived < ASURA_RECEIVE_BUFFER_SIZE, "Message too large - buffer over-run. Will probably crash soon");

#ifdef ASURA_ENABLE_INTERNET_SIMULATION
			if (!Asura_InternetSimulation::AddToBuffer( pcBuffer, iBytesReceived, xAddress.sin_addr.s_addr, ntohs(xAddress.sin_port )))
#endif
			{
				if(!HandleMessage(pcBuffer, iBytesReceived, xAddress.sin_addr.s_addr, ntohs(xAddress.sin_port)))
				{
					return false;
				}
			}
		}
	}
	Asura_Assert(iNumSockets != SOCKET_ERROR, "Error selecting socket.");
	return true;
*/
}

bool Asura_NetSocket_PSPAdhoc::API_SendTo(const Asura_NetSocket_MessageInfo* pxMessageInfo)
{
	return Platform_SendTo(pxMessageInfo);
}

void Asura_NetSocket_PSPAdhoc::API_SetBroadcastEnabled(bool bEnabled)
{
	// Broadcast enabled by default on PSP
}

int Asura_NetSocket_PSPAdhoc::API_GetLastError()
{
/*
	return WSAGetLastError();
*/
	return 0;
}

void Asura_NetSocket_PSPAdhoc::GetAddressFromString(const char* szAddress, Asura_NetAddress* pxAddress, unsigned short* pusPort)
{
	// Passing in a null address is valid - simply return 0
	Asura_NetAddress xAddress;
	unsigned short usPort;

	// Check whether the string is null or empty
	if(!szAddress || !szAddress[0])
	{
		memset(&xAddress, 0, sizeof(Asura_NetAddress));
		usPort = 0;
	}
	else
	{
		char szAddressCopy[32];
		strcpy(szAddressCopy, szAddress);
		// Find the port
		static const u_int uBytesToSkip = 17;
		Asura_Assert(strlen("xx:xx:xx:xx:xx:xx") == uBytesToSkip, "miscalculated address size");
		// if :port follows the adhoc address then use it, else leave port untouched
		char* szPort = strchr(szAddressCopy + uBytesToSkip, ':');
		if(!szPort)
		{
			usPort = 0;
		}
		else
		{
			int iPort = atoi(szPort + 1);
			if(iPort < 0 || iPort > 0xffff)
			{
				usPort = 0;
			}
			else
			{
				usPort = (unsigned short)iPort;
			}
			szPort[0] = 0;
		}
		// Use sceNetEtherStrton to get the address
		SceNetEtherAddr xAdhocAddress;
		sceNetEtherStrton(szAddressCopy, &xAdhocAddress);
		xAddress = xAdhocAddress;
	}
	// Copy values into pointers
	if(pxAddress) *pxAddress = xAddress;
	if(pusPort) *pusPort = usPort;
}


#endif //#ifdef ASURA_SUPPORT_EXTERNAL_NETWORK
