//
// Asura_Platform_NetTransport_System.cpp
//
// Created by KOR - 18/02/2005 15:31:08
//
// Copyright (C) 2005 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

#ifdef ASURA_SUPPORT_EXTERNAL_NETWORK

// This file's header.
#include "Network/Asura_NetTransport_System.h"

// Asura includes
#include "Core/Asura_Maths.h"
#include "Core/Asura_Timers.h"
#include "Core/Memory/Asura_Mem.h"
#include "Debugging/Asura_Logfile.h"
#include "Network/Asura_LanGameFinder.h"
#include "Network/Asura_NetConnection.h"
#include "Network/Asura_NetSocket.h"
#include "Network/Asura_NetSocket_BSD.h"
#include "Network/Asura_Network.h"
#include "Network/Asura_Network_Session.h"

// Asura PSP includes
#include "Network/Asura_Platform_NetSocket_PSPAdhoc.h"
#include "Network/Asura_PSP_Adhoc.h"
#include "Network/Asura_PSP_InfrastructureMode.h"

struct Asura_Network_LANBroadcastServerRequest
{
	Asura_NTS_PacketHeader m_xHeader;
	u_int		m_uApplicationGuid;
};

class Asura_Network_LANBroadcastServerReplyHeader
{
	public:

		Asura_NTS_PacketHeader m_xHeader;
		u_int m_uApplicationGuid;
		u_int m_uServerSessionID;

		u_short m_usDataSize;		// this data is followed by this many bytes of user data
		Asura_UnicodeChar m_wszSessionName[uASURA_NETWORK_MAX_SESSION_NAME + 1];
		u_char m_ucNumPlayers;
		u_char m_ucMaxPlayers;
};


void Asura_NetTransport_System::Platform_Initialise()
{
}

void Asura_NetTransport_System::Platform_Shutdown()
{
}

void Asura_NetTransport_System::Platform_Update()
{
}

void Asura_NetTransport_System::Platform_StartServer(bool bInternetGame)
{
}

void Asura_NetTransport_System::Platform_StopServer()
{
}

bool Asura_NetTransport_System::Platform_SendEnumHostsBroadcast()
{
	return Asura_LanGameFinder::SendEnumHostsBroadcast();
}

void Asura_NetTransport_System::Platform_ProcessEnumHostsRequest( Asura_NetAddress xAddress, u_short usPort, const char* pcData, u_int uDataLength )
{
	Asura_LanGameFinder::ProcessEnumHostsRequest(xAddress, usPort, pcData, uDataLength);
}

void Asura_NetTransport_System::Platform_SendEnumHostsResponse( Asura_NetAddress xAddress, u_short usPort)
{
	Asura_LanGameFinder::SendEnumHostsResponse(xAddress, usPort);
}

void Asura_NetTransport_System::Platform_ProcessEnumHostsResponse( Asura_NetAddress xAddress, u_short usPort, const char* pcData, int iDataSize )
{
	Asura_LanGameFinder::ProcessEnumHostsResponse(xAddress, usPort, pcData, iDataSize);
}

void Asura_NetTransport_System::Platform_Client_ClientConnected(Asura_NetConnection* pxConnection)
{
}

void Asura_NetTransport_System::Platform_Client_ClientDisconnected(Asura_NetConnection* pxConnection)
{
}

void Asura_NetTransport_System::Platform_StartGame()
{
}

bool Asura_NetTransport_System::Platform_HandleMessage( Asura_NetAddress xAddress, u_short usPort, const char* pcMessage, int iLength)
{
	return false;
}

u_int Asura_NetTransport_System::Platform_GetNetAdaptorIP( int iAdapter ) 
{ 	
	return 0;
}

const char* Asura_NetTransport_System::Platform_GetStringFromAddress(Asura_NetAddress xAddress, unsigned short usPort)
{
#ifdef ASURA_PSP_SUPPORT_INFRASTRUCTURE_NETWORK
	if (Asura_PSP_InfrastructureMode::IsConnected())
	{
		return Asura_NetSocket_BSD::GetStringFromAddress( xAddress.GetBSDAddress(), usPort );
	}
	else
#endif
	{
		static char acAddress[sizeof("XX:XX:XX:XX:XX:XX:XXXXX")];
		const SceNetEtherAddr& xAdhocAddress = xAddress.GetAdhocAddress();
		Asura_PrintfToBfr(acAddress, "%02x:%02x:%02x:%02x:%02x:%02x:%u", sizeof(acAddress), true,
						xAdhocAddress.data[0], xAdhocAddress.data[1], 
						xAdhocAddress.data[2], xAdhocAddress.data[3], 
						xAdhocAddress.data[4], xAdhocAddress.data[5], usPort);
		return acAddress;
	}
}

void Asura_NetTransport_System::Platform_GetAddressFromString(const char* szAddress, Asura_NetAddress* pxAddress, unsigned short* pusPort)
{
#ifdef ASURA_PSP_SUPPORT_INFRASTRUCTURE_NETWORK
	if (Asura_PSP_InfrastructureMode::IsConnected())
	{
		u_int uAddress;
		Asura_NetSocket_BSD::GetAddressFromString( szAddress, &uAddress, pusPort );
		*pxAddress = uAddress;
	}
	else
#endif // ASURA_PSP_SUPPORT_INFRASTRUCTURE_NETWORK
	{
		Asura_NetSocket_PSPAdhoc::GetAddressFromString( szAddress, pxAddress, pusPort );
	}
}

Asura_NetSocket* Asura_NetTransport_System::Platform_AllocateNetSocket( u_int uMaxMachines )
{
#ifdef ASURA_PSP_SUPPORT_INFRASTRUCTURE_NETWORK
	if (Asura_PSP_InfrastructureMode::IsConnected())
	{
		return new Asura_NetSocket_BSD( uMaxMachines );
	}
#endif
	// must be adhoc mode
	return new Asura_NetSocket_PSPAdhoc( uMaxMachines );
}

bool Asura_Network_SessionInfo::Platform_IsSameHost(Asura_Network_SessionInfo* pxSession)
{
	return ((m_xHostAddr == pxSession->m_xHostAddr) && (m_usHostPort == pxSession->m_usHostPort));
}

#endif//ASURA_SUPPORT_EXTERNAL_NETWORK
