//
// Asura_PSP_Blur.cpp
//
// Created by JG - 10/10/2005 14:31:19
//
// Copyright (C) 2005 Rebellion, All Rights Reserved.
//

#include "Core\Asura.h"

// This file's header
#include "Rendering\Asura_PSP_Blur.h"

// Asura independent renderer
#include "Rendering/Asura_Render.h"

// PSP headers
#include "Debugging/Asura_PSP_GPIO.h"
#include "Debugging/Asura_PSP_Profiling.h"
// New fast full screen effects copy functions
#include "Rendering/Asura_PSP_Filter.h"
#include "Rendering/Asura_PSP_Graphics.h"
#include "Rendering/Asura_PSP_Stencil_Enums.h"
#include "Rendering/Asura_PSP_RenderStates.h"
#include "Rendering/Asura_PSP_StencilBufferManager.h"

//#define ASURA_PSP_BLUR_BW

// Various parameters for different effects
DEBUGVAR bool dbg_bBlendBlur = true; 
DEBUGVAR bool dbg_bEarlyMask = true; 
DEBUGVAR bool dbg_bLaterMask = true; 
DEBUGVAR bool dbg_bUpScale = true; 
DEBUGVAR bool dbg_bScreen = false; 

bool bGlowEffect = false; 

DEBUGVAR unsigned int dbg_uAlphaBlur = 0x80; 
DEBUGVAR unsigned int dbg_uClear = 0x00000000; 

const unsigned int uWidthBlur = 128;
const unsigned int uHeightBlur = 128;
const unsigned int uWidthDepthField = 128;
const unsigned int uHeightDepthField = 128;
const unsigned int uSizeBlur = uWidthBlur*uHeightBlur*4;
const unsigned int uSizeDepthField = uWidthDepthField*uHeightDepthField*4;
unsigned int uGlowPaletteR[256] __attribute__((aligned(16))); 
unsigned int uGlowPaletteG[256] __attribute__((aligned(16))); 
unsigned int uGlowPaletteB[256] __attribute__((aligned(16))); 
unsigned int uGrayScalePalette[256] __attribute__((aligned(16))); 
static unsigned int uDownWidthBlur = 16;
static unsigned int uDownWidthDepthField = 32;
static unsigned int uLimit = 64;

unsigned int* uOutBufferReadBlur = NULL; 
unsigned int* uOutBufferWriteBlur = NULL; 
unsigned int* uFilterBufferReadBlur = NULL; 
unsigned int* uFilterBufferWriteBlur = NULL; 

// Static members
u_int Asura_PSP_Blur::s_uBrightness = 64;
bool Asura_PSP_Blur::s_bEnabled = false;

void Asura_PSP_Blur::Initialise( void )
{
}

void Asura_PSP_Blur::Render()
{
	Render( NULL, Asura_PSP_Graphics::GetCurrentRenderingBuffer() );
}

void Asura_PSP_Blur::Render( const char* acDisplayList, const unsigned int uCurrentRenderingBuffer )
{
#ifdef ASURA_PSP_PROFILING
	Asura_PSP_Profiling::Signal( Asura_PSP_Profiling::ASURA_PSP_SIGNAL_GLOW_START ); 
#endif // #ifdef ASURA_PSP_PROFILING

	// Are we rendering this effect?
	if (s_bEnabled)
	{
		Asura_PSP_Graphics::SetMaterialType(ASURA_PSP_GRAPHICS_MATERIALTYPE_ENTITY);

		Asura_PSP_Graphics::SetUseColourDoubling(false);

		// Get a couple of pointers to the back buffer, one to read from it, another to write on it
		unsigned int* pReadBackBuffer = (unsigned int*)((unsigned int)sceGeEdramGetAddr() + uCurrentRenderingBuffer);
		unsigned int* pWriteBackBuffer = (unsigned int*)(uCurrentRenderingBuffer);
		
		// Pointer to intermediate buffers

		// Read and write access to down copy of the back buffer
		uOutBufferWriteBlur	= (unsigned int*)((unsigned int)SCEGU_VRAM_BP32_2 + SCEGU_VRAM_BUFSIZE); 
		uOutBufferReadBlur	= (unsigned int*)((unsigned int)sceGeEdramGetAddr() + (unsigned int)SCEGU_VRAM_BP32_2 + SCEGU_VRAM_BUFSIZE); 

		// Read and write access to the filtered buffer
		// JG - Due to a error before library 2.5.0, had to use a bigger buffer, seem to be fixed now...
		//uFilterBufferWriteBlur	= (unsigned int*)((unsigned int)SCEGU_VRAM_BP32_2 + SCEGU_VRAM_BUFSIZE32 + SCEGU_VRAM_BUFSIZE); 
		//uFilterBufferReadBlur	= (unsigned int*)((unsigned int)sceGeEdramGetAddr() + (unsigned int)SCEGU_VRAM_BP32_2 + SCEGU_VRAM_BUFSIZE32 + SCEGU_VRAM_BUFSIZE); 	
		
		uFilterBufferWriteBlur	= (unsigned int*)((unsigned int)SCEGU_VRAM_BP32_2 + SCEGU_VRAM_BUFSIZE + uSizeBlur);
		uFilterBufferReadBlur	= (unsigned int*)((unsigned int)sceGeEdramGetAddr() + (unsigned int)SCEGU_VRAM_BP32_2 + SCEGU_VRAM_BUFSIZE + uSizeBlur); 
		
		// Create a gradient-per-component palette
		static bool bInitialised = false; 
		if (!bInitialised)
		{
			for (int i = 0; i < 256; i++)
			{
				// Shift our gradient to the desired component
				unsigned int Byte = 0;

				// Current funtion: pretty simple, but seems to work quite well
				// Anything under the limit get clamped to zero
				// Values over limit get interpolated from zero to the real color
				if (i > uLimit)
				{
					Byte = s_uBrightness + ((i - uLimit) / (255.0f - uLimit)) * (0xff - s_uBrightness); 
				}
				else
				{
					Byte = (i  / (float)(uLimit) * s_uBrightness); 
				}


				// Right now every palette component is identical, but can prime glows/blur per component 
				// if we need to
				uGlowPaletteR[i] = dbg_uAlphaBlur << 24 | Byte; 
				uGlowPaletteG[i] = dbg_uAlphaBlur << 24 | Byte << 8; 
				uGlowPaletteB[i] = dbg_uAlphaBlur << 24 | Byte << 16; 

			}

			bInitialised = true; 

			// Flush the data write cache
            sceKernelDcacheWritebackRange( uGlowPaletteR, sizeof(uGlowPaletteR) );
            sceKernelDcacheWritebackRange( uGlowPaletteG, sizeof(uGlowPaletteG) );
            sceKernelDcacheWritebackRange( uGlowPaletteB, sizeof(uGlowPaletteB) );
		}

		//GU_Sync(SCEGU_SYNC_FINISH, SCEGU_SYNC_WAIT);

		// Start drawing in immediate mode
		//GU_Start(SCEGU_IMMEDIATE, (void*)acDisplayList, sizeof(acDisplayList));

		// Draw the current frame buffer into our intermediate buffer
		unsigned int uCurrentWidth = uWidthBlur; 
		unsigned int uCurrentHeight = uHeightBlur; 

		// Texture, no depth test and no blending
		GU_Enable(SCEGU_TEXTURE); 
//		Asura_Render::CurrentStates.RequestAlphaTest(false); // To make this works all the states must use the manager
//		Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_OFF);  // To make this works all the states must use the manager
		Asura_PSP_StencilBufferManager::SetStencilEnabled(false);
//		Asura_Render::CurrentStates.RequestAlphaTest(false);  // To make this works all the states must use the manager

		GU_Disable(SCEGU_DEPTH_TEST); 
		GU_Disable(SCEGU_BLEND);
//		GU_Disable(SCEGU_STENCIL_TEST);
		GU_Disable(SCEGU_ALPHA_TEST);

		//GU_DrawBuffer(SCEGU_PF8888, (void*)uOutBufferWriteBlur, uCurrentWidth);
		//GU_Offset(0, 0);
		//GU_Viewport(uWidthBlur / 2, uHeightBlur / 2, uWidthBlur, uHeightBlur);

		//GU_TexImage(0,512,512,512,((char*)pReadBackBuffer));
		//GU_TexMode(SCEGU_PF8888, 0, SCEGU_SINGLE_CLUT, SCEGU_TEXBUF_NORMAL);
		//GU_TexFilter(SCEGU_LINEAR, SCEGU_LINEAR);
		//GU_TexWrap(SCEGU_CLAMP, SCEGU_CLAMP); 
		//GU_TexFlush();

		// Copy our back buffer down into the intermediate buffer
		Asura_PSP_Filter::SetDestinationBuffer(uOutBufferWriteBlur, uCurrentWidth, uCurrentHeight); 
		Asura_PSP_Filter::SetSourceScreen(pReadBackBuffer); 

		// JG - Mask glows on specific parts of the screen
		// Mask on the first screen grab 
		if (dbg_bEarlyMask)
		{
			// Clear the buffer, colour is colour and stencil will be our alpha when we draw
			GU_ClearColor(dbg_uClear);
			Asura_PSP_StencilBufferManager::SetStencilBufferClearBits(dbg_uAlphaBlur);
			GU_Clear(SCEGU_CLEAR_COLOR | SCEGU_CLEAR_STENCIL); 

			GU_Enable(SCEGU_ALPHA_TEST); 
			GU_AlphaFunc(SCEGU_EQUAL, ASURA_PSP_STENCIL_SKYBOX, 0xff); 

		}

		//GU_SpriteMode(480, 272, uCurrentWidth, uCurrentHeight);
		//GU_DrawSprite( 0, 0, 0, 0, 0, SCEGU_NOFLIP, SCEGU_NOROTATE );

		Asura_PSP_Filter::Copy(480, 272, uCurrentWidth, uCurrentHeight);

		GU_Disable(SCEGU_ALPHA_TEST); 

		if (dbg_bScreen)
		{
			// Draw the full screen pass onto the back buffer
			GU_Disable(SCEGU_BLEND); 
			GU_Enable(SCEGU_TEXTURE); 
			GU_Disable(SCEGU_DEPTH_TEST); 
			GU_Disable(SCEGU_ALPHA_TEST); 

			//GU_DrawBuffer(SCEGU_PF8888, pWriteBackBuffer, SCEGU_VRAM_WIDTH);
			//GU_Offset(SCEGU_SCR_OFFSETX, SCEGU_SCR_OFFSETY);
			//GU_Viewport(2048, 2048, SCEGU_SCR_WIDTH, SCEGU_SCR_HEIGHT);

			//GU_TexImage(0,uCurrentWidth,uCurrentHeight,uCurrentWidth,uOutBufferReadBlur);
			//GU_TexMode(SCEGU_PF8888, 0, SCEGU_SINGLE_CLUT, SCEGU_TEXBUF_NORMAL);
			//GU_TexFilter(SCEGU_LINEAR, SCEGU_LINEAR);
			//GU_TexFlush();

			//GU_SpriteMode( uCurrentWidth, uCurrentHeight, 480, 272 );
			//GU_DrawSprite( dbg_uHorizontalShift, dbg_uVerticalShift, 0, 0, 0, SCEGU_NOFLIP, SCEGU_NOROTATE ); 

			Asura_PSP_Filter::SetDestinationScreen(pWriteBackBuffer); 
			Asura_PSP_Filter::SetSourceBuffer(uOutBufferReadBlur, uCurrentWidth, uCurrentHeight); 
			Asura_PSP_Filter::Copy(uCurrentWidth, uCurrentHeight, 480, 272); 

			// JG - End of specific regions of the screen masking
			Asura_PSP_StencilBufferManager::SetStencilEnabled(false);
			GU_AlphaFunc(SCEGU_NOTEQUAL, 0x0, 0xff); 

			// Deactivate colour test
			GU_Disable(SCEGU_COLOR_TEST);

			// Go back to previous texture filter 
			GU_TexFilter(SCEGU_LINEAR_MIPMAP_LINEAR, SCEGU_LINEAR);

			Asura_Render::CurrentStates.ResetAll(); 

			return;
		}

		// Repeat scaling down the frame buffer half size every time to average pixel values
		// until we get our target size

		while (uCurrentWidth != uDownWidthBlur)
		{

			// Next image size will be half of our current size
			unsigned int uHalfWidth = uCurrentWidth / 2; 
			unsigned int uHalfHeight = uCurrentHeight / 2; 

			//GU_DrawBuffer(SCEGU_PF8888, (void*)uFilterBufferWriteBlur, uHalfWidth);
			//GU_Offset(0, 0);
			//GU_Viewport(uHalfWidth / 2, uHalfHeight / 2, uHalfWidth, uHalfHeight);

			//GU_TexImage(0, uCurrentWidth, uCurrentHeight, uCurrentWidth,((char*)uOutBufferReadBlur));
			//GU_TexMode(SCEGU_PF8888, 0, SCEGU_SINGLE_CLUT, SCEGU_TEXBUF_NORMAL);
			//GU_TexFilter(SCEGU_LINEAR, SCEGU_LINEAR);
			//GU_TexFlush();

			//GU_SpriteMode(uCurrentWidth, uCurrentHeight, uHalfWidth, uHalfHeight);
			//GU_DrawSprite( 0, 0, 0, 0, 0, SCEGU_NOFLIP, SCEGU_NOROTATE ); 
			
			// Draw the current frame buffer into our intermediate buffer
			Asura_PSP_Filter::SetDestinationBuffer(uFilterBufferWriteBlur, uHalfWidth, uHalfHeight); 
			Asura_PSP_Filter::SetSourceBuffer(uOutBufferReadBlur, uCurrentWidth, uCurrentHeight); 
			Asura_PSP_Filter::Copy(uCurrentWidth, uCurrentHeight, uHalfWidth, uHalfHeight);

			// Swap intermediate and output buffer
			unsigned int* uTmp = NULL; 

			uTmp = uOutBufferWriteBlur; 
			uOutBufferWriteBlur = uFilterBufferWriteBlur; 
			uFilterBufferWriteBlur = uTmp; 

			uTmp = uOutBufferReadBlur; 
			uOutBufferReadBlur = uFilterBufferReadBlur; 
			uFilterBufferReadBlur = uTmp; 

			// Our current size is half
			uCurrentWidth = uHalfWidth; 
			uCurrentHeight = uHalfHeight; 

		}

		// Texture, no depth test and no blending
		GU_Enable(SCEGU_TEXTURE); 
		GU_Disable(SCEGU_DEPTH_TEST); 
		GU_Disable(SCEGU_BLEND);
		Asura_PSP_StencilBufferManager::SetStencilEnabled(false);

		//GU_DrawBuffer(SCEGU_PF8888, (void*)uFilterBufferWriteBlur, uCurrentWidth);
		//GU_Offset(0, 0);
		//GU_Viewport(uCurrentWidth / 2, uCurrentHeight / 2, uCurrentWidth, uCurrentHeight);

		// Copy our intermediate buffer onto the filter buffer
		Asura_PSP_Filter::SetDestinationBuffer(uFilterBufferWriteBlur, uCurrentWidth, uCurrentHeight);

		// Clear the buffer, colour is colour and stencil will be our alpha when we draw
		GU_ClearColor(0xff000000);
		Asura_PSP_StencilBufferManager::SetStencilBufferClearBits(dbg_uAlphaBlur);
		GU_Clear(SCEGU_CLEAR_COLOR | SCEGU_CLEAR_STENCIL); 

		//GU_TexImage(0, uCurrentWidth, uCurrentHeight, uCurrentWidth,((char*)uOutBufferReadBlur));
		//GU_TexMode(SCEGU_PFIDX32, 0, SCEGU_SINGLE_CLUT, SCEGU_TEXBUF_NORMAL);
		//GU_TexFilter(SCEGU_LINEAR, SCEGU_LINEAR);
		//GU_TexFlush();

		// Filter the three component using a gradient function
		Asura_PSP_Filter::SetSourceBuffer(uOutBufferReadBlur, uCurrentWidth, uCurrentHeight, true); 

		// Combine RGB channels back
		GU_Enable(SCEGU_BLEND);
		GU_BlendFunc(SCEGU_ADD, SCEGU_FIX_VALUE, SCEGU_FIX_VALUE, 0x00FFffFF, 0x00FFffFF); 

		// Remap colours through red palette
		GU_ClutMode(SCEGU_PF8888, 0, 0xff, 0); 
		GU_ClutLoad(32, uGlowPaletteR);

		//GU_SpriteMode(uCurrentWidth, uCurrentHeight, uCurrentWidth, uCurrentHeight);
		//GU_DrawSprite( 0, 0, 0, 0, 0, SCEGU_NOFLIP, SCEGU_NOROTATE ); 

		Asura_PSP_Filter::Copy(uCurrentWidth, uCurrentHeight, uCurrentWidth, uCurrentHeight); 

		// Remap colours through green palette
		GU_ClutMode(SCEGU_PF8888, 8, 0xff, 0); 
		GU_ClutLoad(32, uGlowPaletteG);

		//GU_SpriteMode(uCurrentWidth, uCurrentHeight, uCurrentWidth, uCurrentHeight);
		//GU_DrawSprite( 0, 0, 0, 0, 0, SCEGU_NOFLIP, SCEGU_NOROTATE ); 

		Asura_PSP_Filter::Copy(uCurrentWidth, uCurrentHeight, uCurrentWidth, uCurrentHeight); 

		// Remap colours through blue palette
		GU_ClutMode(SCEGU_PF8888, 16, 0xff, 0); 
		GU_ClutLoad(32, uGlowPaletteB);

		//GU_SpriteMode(uCurrentWidth, uCurrentHeight, uCurrentWidth, uCurrentHeight);
		//GU_DrawSprite( 0, 0, 0, 0, 0, SCEGU_NOFLIP, SCEGU_NOROTATE ); 

		Asura_PSP_Filter::Copy(uCurrentWidth, uCurrentHeight, uCurrentWidth, uCurrentHeight); 

		if (dbg_bUpScale)
		{
			// Swap intermediate and output buffer
			unsigned int* uTmp = NULL; 

			uTmp = uOutBufferWriteBlur; 
			uOutBufferWriteBlur = uFilterBufferWriteBlur; 
			uFilterBufferWriteBlur = uTmp; 

			uTmp = uOutBufferReadBlur; 
			uOutBufferReadBlur = uFilterBufferReadBlur; 
			uFilterBufferReadBlur = uTmp; 

			GU_Disable(SCEGU_BLEND);

			while (uCurrentWidth != uWidthBlur)
			{

				// Next image size will be half of our current size
				unsigned int uDoubleWidth = uCurrentWidth * 2; 
				unsigned int uDoubleHeight = uCurrentHeight * 2; 

				// Draw the current frame buffer into our intermediate buffer
				//GU_DrawBuffer(SCEGU_PF8888, (void*)uFilterBufferWriteBlur, uDoubleWidth);
				//GU_Offset(0, 0);
				//GU_Viewport(uDoubleWidth / 2, uDoubleHeight / 2, uDoubleWidth, uDoubleHeight);

				Asura_PSP_Filter::SetDestinationBuffer(uFilterBufferWriteBlur, uDoubleWidth, uDoubleHeight); 

				//GU_TexImage(0, uCurrentWidth, uCurrentHeight, uCurrentWidth,((char*)uOutBufferReadBlur));
				//GU_TexMode(SCEGU_PF8888, 0, SCEGU_SINGLE_CLUT, SCEGU_TEXBUF_NORMAL);
				//GU_TexFilter(SCEGU_LINEAR, SCEGU_LINEAR);
				//GU_TexFlush();

				// Copy our back buffer down into the intermediate buffer
				Asura_PSP_Filter::SetSourceBuffer(uOutBufferReadBlur, uCurrentWidth, uCurrentHeight); 

				//GU_SpriteMode(uCurrentWidth, uCurrentHeight, uDoubleWidth, uDoubleHeight);
				//GU_DrawSprite( 0, 0, 0, 0, 0, SCEGU_NOFLIP, SCEGU_NOROTATE ); 

				Asura_PSP_Filter::Copy(uCurrentWidth, uCurrentHeight, uDoubleWidth, uDoubleHeight); 

				// Swap intermediate and output buffer
				unsigned int* uTmp = NULL; 

				uTmp = uOutBufferWriteBlur; 
				uOutBufferWriteBlur = uFilterBufferWriteBlur; 
				uFilterBufferWriteBlur = uTmp; 

				uTmp = uOutBufferReadBlur; 
				uOutBufferReadBlur = uFilterBufferReadBlur; 
				uFilterBufferReadBlur = uTmp; 

				// Our current size is half
				uCurrentWidth = uDoubleWidth; 
				uCurrentHeight = uDoubleHeight; 
			}
		}

		// JG - Mask glows on specific parts of the screen
		// Mask on the pass back to the screen
		if (dbg_bLaterMask)
		{
			Asura_PSP_StencilBufferManager::SetStencilOperation(ASURA_PSP_STENCIL_OPERATION_KEEP, ASURA_PSP_STENCIL_OPERATION_KEEP, ASURA_PSP_STENCIL_OPERATION_REPLACE);
			Asura_PSP_StencilBufferManager::SetStencilFunction(ASURA_PSP_STENCIL_FUNCTION_NOT_EQUAL, ASURA_PSP_STENCIL_SKYBOX, 0xFF);
			Asura_PSP_StencilBufferManager::SetStencilEnabled(true);
		}

		// Only draw pixels different from black
		GU_Enable(SCEGU_COLOR_TEST);
		GU_ColorFunc(SCEGU_NOTEQUAL, 0, 0x00FFffFF); 

		//GU_DrawBuffer(SCEGU_PF8888, pWriteBackBuffer, SCEGU_VRAM_WIDTH);
		//GU_Offset(SCEGU_SCR_OFFSETX, SCEGU_SCR_OFFSETY);
		//GU_Viewport(2048, 2048, SCEGU_SCR_WIDTH, SCEGU_SCR_HEIGHT);

		// Draw the full screen pass onto the back buffer
		Asura_PSP_Filter::SetDestinationScreen(pWriteBackBuffer); 


		//if (dbg_bUpScale)
		//{
		//	GU_TexImage(0,uCurrentWidth,uCurrentHeight,uCurrentWidth,uOutBufferReadBlur);
		//}
		//else
		//{
		//	GU_TexImage(0,uCurrentWidth,uCurrentHeight,uCurrentWidth,uFilterBufferReadBlur);
		//}
		//GU_TexMode(SCEGU_PF8888, 0, SCEGU_SINGLE_CLUT, SCEGU_TEXBUF_NORMAL);
		//GU_TexFilter(SCEGU_LINEAR, SCEGU_LINEAR);
		//GU_TexFlush();

		if (dbg_bUpScale)
		{
			Asura_PSP_Filter::SetSourceBuffer(uOutBufferReadBlur, uCurrentWidth,uCurrentHeight);
		}
		else
		{
			Asura_PSP_Filter::SetSourceBuffer(uFilterBufferReadBlur, uCurrentWidth,uCurrentHeight);
		}

		if (dbg_bBlendBlur)
		{
			GU_Enable(SCEGU_BLEND);
			GU_BlendFunc(SCEGU_ADD, SCEGU_FIX_VALUE, SCEGU_FIX_VALUE, 0xffFFffFF, 0xffFFffFF); 
		}
		else
		{
			GU_Disable(SCEGU_BLEND); 
		}

		GU_Enable(SCEGU_TEXTURE); 
		GU_Disable(SCEGU_DEPTH_TEST); 
		GU_Disable(SCEGU_ALPHA_TEST); 

		//GU_SpriteMode( uCurrentWidth, uCurrentHeight, 480, 272 );
		//GU_DrawSprite( dbg_uHorizontalShift, dbg_uVerticalShift, 0, 0, 0, SCEGU_NOFLIP, SCEGU_NOROTATE ); 

		Asura_PSP_Filter::Copy( uCurrentWidth, uCurrentHeight, 480, 272 );

		// JG - End of specific regions of the screen masking
		Asura_PSP_StencilBufferManager::SetStencilEnabled(false);
		GU_AlphaFunc(SCEGU_NOTEQUAL, 0x0, 0xff); 

		// Deactivate colour test
		GU_Disable(SCEGU_COLOR_TEST);

		// Go back to previous texture filter 
		GU_TexFilter(SCEGU_LINEAR_MIPMAP_LINEAR, SCEGU_LINEAR);

		// Set textures to repeat by default
		GU_TexWrap(SCEGU_REPEAT, SCEGU_REPEAT); 

		Asura_Render::CurrentStates.ResetAll(); 

		//GU_Finish();
	}

#ifdef ASURA_PSP_PROFILING
	Asura_PSP_Profiling::Signal( Asura_PSP_Profiling::ASURA_PSP_SIGNAL_GLOW_END ); 
#endif // #ifdef ASURA_PSP_PROFILING

	return;

}

void Asura_PSP_Blur::RenderMaterialGlow( const char* acDisplayList, const unsigned int uCurrentRenderingBuffer )
{
#ifdef ASURA_PSP_PROFILING
	Asura_PSP_Profiling::Signal( Asura_PSP_Profiling::ASURA_PSP_SIGNAL_GLOW_START ); 
#endif // #ifdef ASURA_PSP_PROFILING

	// Are we rendering this effect?
	if (s_bEnabled)
	{
		Asura_PSP_Graphics::SetMaterialType(ASURA_PSP_GRAPHICS_MATERIALTYPE_ENTITY);

		Asura_PSP_Graphics::SetUseColourDoubling(false);

		// Get a couple of pointers to the back buffer, one to read from it, another to write on it
		unsigned int* pReadBackBuffer = (unsigned int*)((unsigned int)sceGeEdramGetAddr() + uCurrentRenderingBuffer);
		unsigned int* pWriteBackBuffer = (unsigned int*)(uCurrentRenderingBuffer);
		
		// Pointer to intermediate buffers

		// Read and write access to down copy of the back buffer
		uOutBufferWriteBlur	= (unsigned int*)((unsigned int)SCEGU_VRAM_BP32_2 + SCEGU_VRAM_BUFSIZE); 
		uOutBufferReadBlur	= (unsigned int*)((unsigned int)sceGeEdramGetAddr() + (unsigned int)SCEGU_VRAM_BP32_2 + SCEGU_VRAM_BUFSIZE); 

		// Read and write access to the filtered buffer
		// JG - Due to a error before library 2.5.0, had to use a bigger buffer, seem to be fixed now...
		//uFilterBufferWriteBlur	= (unsigned int*)((unsigned int)SCEGU_VRAM_BP32_2 + SCEGU_VRAM_BUFSIZE32 + SCEGU_VRAM_BUFSIZE); 
		//uFilterBufferReadBlur	= (unsigned int*)((unsigned int)sceGeEdramGetAddr() + (unsigned int)SCEGU_VRAM_BP32_2 + SCEGU_VRAM_BUFSIZE32 + SCEGU_VRAM_BUFSIZE); 	
		
		uFilterBufferWriteBlur	= (unsigned int*)((unsigned int)SCEGU_VRAM_BP32_2 + SCEGU_VRAM_BUFSIZE + uSizeBlur);
		uFilterBufferReadBlur	= (unsigned int*)((unsigned int)sceGeEdramGetAddr() + (unsigned int)SCEGU_VRAM_BP32_2 + SCEGU_VRAM_BUFSIZE + uSizeBlur); 

		// Draw the current frame buffer into our intermediate buffer
		unsigned int uCurrentWidth = uWidthBlur; 
		unsigned int uCurrentHeight = uHeightBlur; 

		// Texture, no depth test and no blending
		GU_Enable(SCEGU_TEXTURE); 
		GU_Disable(SCEGU_DEPTH_TEST); 
		GU_Disable(SCEGU_BLEND);
		Asura_PSP_StencilBufferManager::SetStencilEnabled(false);
		GU_Disable(SCEGU_ALPHA_TEST);

		// Copy our back buffer down into the intermediate buffer
		Asura_PSP_Filter::SetDestinationBuffer(uOutBufferWriteBlur, uCurrentWidth, uCurrentHeight); 
		Asura_PSP_Filter::SetSourceScreen(pReadBackBuffer); 

		// JG - Mask glows on specific parts of the screen
		// Mask on the first screen grab 
		if (dbg_bEarlyMask)
		{
			// Clear the buffer, colour is colour and stencil will be our alpha when we draw
			GU_ClearColor(dbg_uClear);
			Asura_PSP_StencilBufferManager::SetStencilBufferClearBits(dbg_uAlphaBlur);
			GU_Clear(SCEGU_CLEAR_COLOR | SCEGU_CLEAR_STENCIL); 

			GU_Enable(SCEGU_ALPHA_TEST); 
			GU_AlphaFunc(SCEGU_EQUAL, ASURA_PSP_STENCIL_GLOW, 0xff); 

		}

		Asura_PSP_Filter::Copy(480, 272, uCurrentWidth, uCurrentHeight);

		GU_Disable(SCEGU_ALPHA_TEST); 

		// Repeat scaling down the frame buffer half size every time to average pixel values
		// until we get our target size

		while (uCurrentWidth != uDownWidthBlur)
		{

			// Next image size will be half of our current size
			unsigned int uHalfWidth = uCurrentWidth / 2; 
			unsigned int uHalfHeight = uCurrentHeight / 2; 
	
			// Draw the current frame buffer into our intermediate buffer
			Asura_PSP_Filter::SetDestinationBuffer(uFilterBufferWriteBlur, uHalfWidth, uHalfHeight); 
			Asura_PSP_Filter::SetSourceBuffer(uOutBufferReadBlur, uCurrentWidth, uCurrentHeight); 
			Asura_PSP_Filter::Copy(uCurrentWidth, uCurrentHeight, uHalfWidth, uHalfHeight);

			// Swap intermediate and output buffer
			unsigned int* uTmp = NULL; 

			uTmp = uOutBufferWriteBlur; 
			uOutBufferWriteBlur = uFilterBufferWriteBlur; 
			uFilterBufferWriteBlur = uTmp; 

			uTmp = uOutBufferReadBlur; 
			uOutBufferReadBlur = uFilterBufferReadBlur; 
			uFilterBufferReadBlur = uTmp; 

			// Our current size is half
			uCurrentWidth = uHalfWidth; 
			uCurrentHeight = uHalfHeight; 

		}

		if (dbg_bScreen)
		{
			// Draw the full screen pass onto the back buffer
			GU_Disable(SCEGU_BLEND); 
			GU_Enable(SCEGU_TEXTURE); 
			GU_Disable(SCEGU_DEPTH_TEST); 
			GU_Disable(SCEGU_ALPHA_TEST); 

			Asura_PSP_Filter::SetDestinationScreen(pWriteBackBuffer); 
			Asura_PSP_Filter::SetSourceBuffer(uOutBufferReadBlur, uCurrentWidth, uCurrentHeight); 
			Asura_PSP_Filter::Copy(uCurrentWidth, uCurrentHeight, 480, 272); 

			// JG - End of specific regions of the screen masking
			Asura_PSP_StencilBufferManager::SetStencilEnabled(false);
			GU_AlphaFunc(SCEGU_NOTEQUAL, 0x0, 0xff); 

			// Deactivate colour test
			GU_Disable(SCEGU_COLOR_TEST);

			// Go back to previous texture filter 
			GU_TexFilter(SCEGU_LINEAR_MIPMAP_LINEAR, SCEGU_LINEAR);

			Asura_Render::CurrentStates.ResetAll(); 

			return;
		}

		// Texture, no depth test and no blending
		GU_Enable(SCEGU_TEXTURE); 
		GU_Disable(SCEGU_DEPTH_TEST); 
		GU_Disable(SCEGU_BLEND);
		Asura_PSP_StencilBufferManager::SetStencilEnabled(false);

		// Copy our intermediate buffer onto the filter buffer
		Asura_PSP_Filter::SetDestinationBuffer(uFilterBufferWriteBlur, uCurrentWidth, uCurrentHeight);
		Asura_PSP_Filter::SetSourceBuffer(uOutBufferReadBlur, uCurrentWidth, uCurrentHeight); 
		Asura_PSP_Filter::Copy(uCurrentWidth, uCurrentHeight, uCurrentWidth, uCurrentHeight); 

		if (dbg_bUpScale)
		{
			// Swap intermediate and output buffer
			unsigned int* uTmp = NULL; 

			uTmp = uOutBufferWriteBlur; 
			uOutBufferWriteBlur = uFilterBufferWriteBlur; 
			uFilterBufferWriteBlur = uTmp; 

			uTmp = uOutBufferReadBlur; 
			uOutBufferReadBlur = uFilterBufferReadBlur; 
			uFilterBufferReadBlur = uTmp; 

			GU_Disable(SCEGU_BLEND);

			while (uCurrentWidth != uWidthBlur)
			{

				// Next image size will be half of our current size
				unsigned int uDoubleWidth = uCurrentWidth * 2; 
				unsigned int uDoubleHeight = uCurrentHeight * 2; 

				// Draw the current frame buffer into our intermediate buffer
				Asura_PSP_Filter::SetDestinationBuffer(uFilterBufferWriteBlur, uDoubleWidth, uDoubleHeight); 

				// Copy our back buffer down into the intermediate buffer
				Asura_PSP_Filter::SetSourceBuffer(uOutBufferReadBlur, uCurrentWidth, uCurrentHeight); 

				Asura_PSP_Filter::Copy(uCurrentWidth, uCurrentHeight, uDoubleWidth, uDoubleHeight); 

				// Swap intermediate and output buffer
				unsigned int* uTmp = NULL; 

				uTmp = uOutBufferWriteBlur; 
				uOutBufferWriteBlur = uFilterBufferWriteBlur; 
				uFilterBufferWriteBlur = uTmp; 

				uTmp = uOutBufferReadBlur; 
				uOutBufferReadBlur = uFilterBufferReadBlur; 
				uFilterBufferReadBlur = uTmp; 

				// Our current size is double
				uCurrentWidth = uDoubleWidth; 
				uCurrentHeight = uDoubleHeight; 
			}
		}

		// Only draw pixels different from black
		GU_Enable(SCEGU_COLOR_TEST);
		GU_ColorFunc(SCEGU_NOTEQUAL, 0, 0x00FFffFF); 

		// Draw the full screen pass onto the back buffer
		Asura_PSP_Filter::SetDestinationScreen(pWriteBackBuffer); 

		if (dbg_bUpScale)
		{
			Asura_PSP_Filter::SetSourceBuffer(uOutBufferReadBlur, uCurrentWidth,uCurrentHeight);
		}
		else
		{
			Asura_PSP_Filter::SetSourceBuffer(uFilterBufferReadBlur, uCurrentWidth,uCurrentHeight);
		}

		if (dbg_bBlendBlur)
		{
			GU_Enable(SCEGU_BLEND);
			GU_BlendFunc(SCEGU_ADD, SCEGU_FIX_VALUE, SCEGU_FIX_VALUE, 0xffFFffFF, 0xffFFffFF); 
		}
		else
		{
			GU_Disable(SCEGU_BLEND); 
		}

		GU_Enable(SCEGU_TEXTURE); 
		GU_Disable(SCEGU_DEPTH_TEST); 
		GU_Disable(SCEGU_ALPHA_TEST); 

		Asura_PSP_Filter::Copy( uCurrentWidth, uCurrentHeight, 480, 272 );

		// JG - End of specific regions of the screen masking
		Asura_PSP_StencilBufferManager::SetStencilEnabled(false);
		GU_AlphaFunc(SCEGU_NOTEQUAL, 0x0, 0xff); 

		// Deactivate colour test
		GU_Disable(SCEGU_COLOR_TEST);

		// Go back to previous texture filter 
		GU_TexFilter(SCEGU_LINEAR_MIPMAP_LINEAR, SCEGU_LINEAR);

		// Set textures to repeat by default
		GU_TexWrap(SCEGU_REPEAT, SCEGU_REPEAT); 

		Asura_Render::CurrentStates.ResetAll(); 

	}

#ifdef ASURA_PSP_PROFILING
	Asura_PSP_Profiling::Signal( Asura_PSP_Profiling::ASURA_PSP_SIGNAL_GLOW_END ); 
#endif // #ifdef ASURA_PSP_PROFILING

	return;

}
DEBUGVAR bool dbg_bGrayScale = false; 
static unsigned int s_uCameraDistance = 0; 
static unsigned int s_uOldCameraDistance = 0; 

void Asura_PSP_Blur::RenderDepthField( const char* acDisplayList, const unsigned int uCurrentRenderingBuffer )
{
#ifdef ASURA_PSP_PROFILING
	Asura_PSP_Profiling::Signal( Asura_PSP_Profiling::ASURA_PSP_SIGNAL_GLOW_START ); 
#endif // #ifdef ASURA_PSP_PROFILING

	// Are we rendering this effect?
	if (s_bEnabled)
	{
		Asura_PSP_Graphics::SetMaterialType(ASURA_PSP_GRAPHICS_MATERIALTYPE_ENTITY);
	
		// Create a grey-scale palette
		static bool bInitialised = false; 
		if (!bInitialised || (s_uOldCameraDistance != s_uCameraDistance))
		{
			for (unsigned int u = 0; u < 256; u++)
			{
				unsigned int uByte = u;

				if (u < s_uCameraDistance)
				{
					uByte = 0; 
				}
				else
				{
					uByte = 255 * ((u - s_uCameraDistance) / (256.f - s_uCameraDistance));
				}

				// generate a grey scale palette
				uGrayScalePalette[u] = uByte << 24 | uByte << 16 | uByte << 8 | uByte; 
			}

			s_uOldCameraDistance = s_uCameraDistance; 

			bInitialised = true; 

			// Flush the data write cache
            sceKernelDcacheWritebackRange( uGrayScalePalette, sizeof(uGrayScalePalette) );
		}

		Asura_PSP_Graphics::SetUseColourDoubling(false);

		// Get a couple of pointers to the back buffer, one to read from it, another to write on it
		unsigned int* pReadBackBuffer = (unsigned int*)((unsigned int)sceGeEdramGetAddr() + uCurrentRenderingBuffer);
		unsigned int* pWriteBackBuffer = (unsigned int*)(uCurrentRenderingBuffer);
		
		// Pointer to intermediate buffers

		// Read and write access to down copy of the back buffer
		uOutBufferWriteBlur	= (unsigned int*)((unsigned int)SCEGU_VRAM_BP32_2 + SCEGU_VRAM_BUFSIZE); 
		uOutBufferReadBlur	= (unsigned int*)((unsigned int)sceGeEdramGetAddr() + (unsigned int)SCEGU_VRAM_BP32_2 + SCEGU_VRAM_BUFSIZE); 

		// Read and write access to the filtered buffer	
		uFilterBufferWriteBlur	= (unsigned int*)((unsigned int)SCEGU_VRAM_BP32_2 + SCEGU_VRAM_BUFSIZE + uSizeDepthField);
		uFilterBufferReadBlur	= (unsigned int*)((unsigned int)sceGeEdramGetAddr() + (unsigned int)SCEGU_VRAM_BP32_2 + SCEGU_VRAM_BUFSIZE + uSizeDepthField); 

		// Draw the current frame buffer into our intermediate buffer
		unsigned int uCurrentWidth = uWidthDepthField; 
		unsigned int uCurrentHeight = uHeightDepthField; 

		// Texture, no depth test and no blending
		GU_Enable(SCEGU_TEXTURE); 
		GU_Disable(SCEGU_DEPTH_TEST); 
		GU_Disable(SCEGU_BLEND);
		Asura_PSP_StencilBufferManager::SetStencilEnabled(false);
		GU_Disable(SCEGU_ALPHA_TEST);

		// Copy our back buffer down into the intermediate buffer
		Asura_PSP_Filter::SetDestinationBuffer(uOutBufferWriteBlur, uCurrentWidth, uCurrentHeight); 
		Asura_PSP_Filter::SetSourceScreen(pReadBackBuffer); 
		Asura_PSP_Filter::Copy(480, 272, uCurrentWidth, uCurrentHeight);

		GU_Disable(SCEGU_ALPHA_TEST); 

		// Repeat scaling down the frame buffer half size every time to average pixel values
		// until we get our target size

		while (uCurrentWidth != uDownWidthDepthField)
		{

			// Next image size will be half of our current size
			unsigned int uHalfWidth = uCurrentWidth / 2; 
			unsigned int uHalfHeight = uCurrentHeight / 2; 
	
			// Draw the current frame buffer into our intermediate buffer
			Asura_PSP_Filter::SetDestinationBuffer(uFilterBufferWriteBlur, uHalfWidth, uHalfHeight); 
			Asura_PSP_Filter::SetSourceBuffer(uOutBufferReadBlur, uCurrentWidth, uCurrentHeight); 
			Asura_PSP_Filter::Copy(uCurrentWidth, uCurrentHeight, uHalfWidth, uHalfHeight);

			// Swap intermediate and output buffer
			unsigned int* uTmp = NULL; 

			uTmp = uOutBufferWriteBlur; 
			uOutBufferWriteBlur = uFilterBufferWriteBlur; 
			uFilterBufferWriteBlur = uTmp; 

			uTmp = uOutBufferReadBlur; 
			uOutBufferReadBlur = uFilterBufferReadBlur; 
			uFilterBufferReadBlur = uTmp; 

			// Our current size is half
			uCurrentWidth = uHalfWidth; 
			uCurrentHeight = uHalfHeight; 

		}

		// Texture, no depth test and no blending
		GU_Enable(SCEGU_TEXTURE); 
		GU_Disable(SCEGU_DEPTH_TEST); 
		GU_Disable(SCEGU_BLEND);
		Asura_PSP_StencilBufferManager::SetStencilEnabled(false);

		// Copy our intermediate buffer onto the filter buffer
		Asura_PSP_Filter::SetDestinationBuffer(uFilterBufferWriteBlur, uCurrentWidth, uCurrentHeight);
		Asura_PSP_Filter::SetSourceBuffer(uOutBufferReadBlur, uCurrentWidth, uCurrentHeight); 
		Asura_PSP_Filter::Copy(uCurrentWidth, uCurrentHeight, uCurrentWidth, uCurrentHeight); 

		if (dbg_bUpScale)
		{
			// Swap intermediate and output buffer
			unsigned int* uTmp = NULL; 

			uTmp = uOutBufferWriteBlur; 
			uOutBufferWriteBlur = uFilterBufferWriteBlur; 
			uFilterBufferWriteBlur = uTmp; 

			uTmp = uOutBufferReadBlur; 
			uOutBufferReadBlur = uFilterBufferReadBlur; 
			uFilterBufferReadBlur = uTmp; 

			GU_Disable(SCEGU_BLEND);

			while (uCurrentWidth != uWidthDepthField)
			{

				// Next image size will be half of our current size
				unsigned int uDoubleWidth = uCurrentWidth * 2; 
				unsigned int uDoubleHeight = uCurrentHeight * 2; 

				// Draw the current frame buffer into our intermediate buffer
				Asura_PSP_Filter::SetDestinationBuffer(uFilterBufferWriteBlur, uDoubleWidth, uDoubleHeight); 

				// Copy our back buffer down into the intermediate buffer
				Asura_PSP_Filter::SetSourceBuffer(uOutBufferReadBlur, uCurrentWidth, uCurrentHeight); 

				Asura_PSP_Filter::Copy(uCurrentWidth, uCurrentHeight, uDoubleWidth, uDoubleHeight); 

				// Swap intermediate and output buffer
				unsigned int* uTmp = NULL; 

				uTmp = uOutBufferWriteBlur; 
				uOutBufferWriteBlur = uFilterBufferWriteBlur; 
				uFilterBufferWriteBlur = uTmp; 

				uTmp = uOutBufferReadBlur; 
				uOutBufferReadBlur = uFilterBufferReadBlur; 
				uFilterBufferReadBlur = uTmp; 

				// Our current size is double
				uCurrentWidth = uDoubleWidth; 
				uCurrentHeight = uDoubleHeight; 
			}
		}

		// Draw the full screen pass onto the back buffer
		GU_Disable(SCEGU_BLEND); 
		GU_Enable(SCEGU_TEXTURE); 
		GU_Disable(SCEGU_DEPTH_TEST); 
		GU_Disable(SCEGU_ALPHA_TEST); 

		
		if (dbg_bGrayScale)
		{
			Asura_PSP_Filter::SetDestinationScreen(pWriteBackBuffer); 
	
			const bool _32bitframe = true;
			const unsigned int vram_top = (unsigned int)sceGeEdramGetAddr();
			const unsigned int zread = vram_top + (_32bitframe ? SCEGU_VRAM_BUFSIZE32*2 + 0x600000 : (SCEGU_VRAM_BUFSIZE*2)+0x200000);

			GU_ClutLoad( 32, uGrayScalePalette );
			GU_ClutMode( SCEGU_PF8888,8,0xff,0);

//			GU_Enable(SCEGU_BLEND); 
//			GU_BlendFunc( SCEGU_ADD, SCEGU_SRC_ALPHA, SCEGU_ONE_MINUS_SRC_ALPHA, 0, 0 );

			Asura_PSP_Filter::SetSourceBuffer((void*)zread, 512, 512, true, true); 
			Asura_PSP_Filter::Copy(480, 272, 480, 272); 
		}
		else
		{
			Asura_PSP_Filter::SetDestinationBuffer(uOutBufferWriteBlur, uCurrentWidth, uCurrentHeight ); 

			const bool _32bitframe = true;
			const unsigned int vram_top = (unsigned int)sceGeEdramGetAddr();
			const unsigned int zread = vram_top + (_32bitframe ? SCEGU_VRAM_BUFSIZE32*2 + 0x600000 : (SCEGU_VRAM_BUFSIZE*2)+0x200000);

			GU_ClutLoad( 32, uGrayScalePalette );
			GU_ClutMode( SCEGU_PF8888,8,0xff,0);

			GU_Enable(SCEGU_BLEND); 

			Asura_PSP_Filter::SetSourceBuffer((void*)zread, 512, 512, true, true); 

			GU_BlendFunc( SCEGU_ADD, SCEGU_FIX_VALUE, SCEGU_SRC_COLOR, 0, 0);
			Asura_PSP_Filter::Copy(480, 272, uCurrentWidth, uCurrentHeight); 

			Asura_PSP_Filter::SetDestinationScreen(pWriteBackBuffer); 

			GU_BlendFunc( SCEGU_ADD, SCEGU_FIX_VALUE, SCEGU_ONE_MINUS_SRC_COLOR, 0, 0);
			Asura_PSP_Filter::Copy(480, 272, 480, 272); 

			GU_BlendFunc( SCEGU_ADD, SCEGU_FIX_VALUE, SCEGU_FIX_VALUE, 0x00ffffff, 0x00ffffff );

			Asura_PSP_Filter::SetSourceBuffer(uOutBufferReadBlur, uCurrentWidth, uCurrentHeight); 
			Asura_PSP_Filter::Copy(uCurrentWidth, uCurrentHeight, 480, 272); 

		}

		// JG - End of specific regions of the screen masking
		Asura_PSP_StencilBufferManager::SetStencilEnabled(false);
		GU_AlphaFunc(SCEGU_NOTEQUAL, 0x0, 0xff); 

		// Deactivate colour test
		GU_Disable(SCEGU_COLOR_TEST);

		// Go back to previous texture filter 
		GU_TexFilter(SCEGU_LINEAR_MIPMAP_LINEAR, SCEGU_LINEAR);

		// Set textures to repeat by default
		GU_TexWrap(SCEGU_REPEAT, SCEGU_REPEAT); 

		Asura_Render::CurrentStates.ResetAll(); 
	}

#ifdef ASURA_PSP_PROFILING
	Asura_PSP_Profiling::Signal( Asura_PSP_Profiling::ASURA_PSP_SIGNAL_GLOW_END ); 
#endif // #ifdef ASURA_PSP_PROFILING

	return;

}
