//////////////////////////////////////////////////////////////////////////////
//
// Asura_PSP_Fog.cpp
//
// Copyright (c) 2004 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// AD - To enable ZBuffer fog, use the #define ASURA_PSP_USE_IMAGE_BASED_FOG in ProjectConfiguration.h

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

// Engine independent.
#include "Camera/Asura_Camera.h"
#include "Rendering/Asura_Fog.h"
#include "Rendering/Asura_Colour.h"
#include "Rendering/Asura_Render.h"

// PSP headers
#include "Debugging/Asura_PSP_Profiling.h"
#include "Rendering/Asura_PSP_Fog.h"
#include "Rendering/Asura_PSP_Graphics.h"
#include "Rendering/Asura_PSP_RenderStates.h"

#include <geman.h>

// Local data

#ifdef ASURA_PSP_USE_IMAGE_BASED_FOG

static u_int32 _zclutA[256] __attribute__((aligned(16)));
//////////////////////////////////////////////////////////////////////////////////
// AJE - 21Mar2007
// ** PLEASE DO NOT DELETE THE COMMENTED LINES DIRECTLY BELOW **
// The commented line of code is for the original implementation of the image-based fog.
// However, the near clut was never actually used...
// If the newer version of the fogging is preferred then the commented line can be removed.

//static u_int32 _zclutB[256] __attribute__((aligned(16)));
//////////////////////////////////////////////////////////////////////////////////
#endif

// C L A S S   I M P L E M E N T A T I O N S /////////////////////////////////

void Asura_Fog::Platform_UpdateForNewParameters( void )
{

}

void Asura_Fog::Platform_Render( void )
{

}


static inline u_int32 MakeRGBA8888(float rf,float gf,float bf, float af)
{
	u_int32 result;
	__asm__(
		"lv.s		s000, %1		\n"
		"lv.s		s001, %2		\n"
		"lv.s		s002, %3		\n"
		"lv.s		s003, %4		\n"
		"vsat0.q	c000, c000		\n"	// rgba: force 0->1
		"vf2iz.q	c000, c000, 31	\n"	// rgba: convert to integer
		"vi2uc.q	s000, c000		\n"	// rgba: charpack
		"sv.s		s000, %0		\n"
		: "=m" (result)
		: "m" (rf)
		, "m" (gf)
		, "m" (bf)
		, "m" (af)
		);
	return result;
}

void Asura_PSP_Fog::OnBeginFrame( void )
{
#ifndef ASURA_PSP_USE_IMAGE_BASED_FOG				// Not using Z buffer fog? Use HW fog (if fog enabled)
	{
		const Asura_RGBA& xColour = Asura_Fog::GetFogColour();
		u_int uFogColour = MakeRGBA8888( xColour.m_fR, xColour.m_fG, xColour.m_fB, 0.0f );

		float fFogValueAtRange = Asura_Fog::GetFogValueAtFarPlane();

		if ( Asura_Fog::IsFogEnabled() && fFogValueAtRange > 0.0f )
		{
			// PSPTODO - hacked nearplane to zero cos it looks better
			//GU_Fog( 0.0f/* Asura_Fog::GetFogNearPlane()*/, Asura_Fog::GetFogFarPlane(), uFogColour );

			// We can't have a value of non-100% at the far plane with the HW fog. Try recalculating the far plane
			// such that we have FogValueAtFarPlane at the range requested. It won't be capped there, but it'd
			// still be closer to the intended effect.

			float fFogRange = Asura_Fog::GetFogFarPlane() - Asura_Fog::GetFogNearPlane();

			float fModifiedFarPlane = Asura_Fog::GetFogNearPlane() + (fFogRange / fFogValueAtRange);
			GU_Fog( 0.0f/* Asura_Fog::GetFogNearPlane()*/, fModifiedFarPlane, uFogColour );

		}

		// default to fog off; renderers that need it can turn the state on as necessary by calling
		// void TurnOnFogIfNecessary( void );
		// void TurnOffFogIfNecessary( void );
		GU_Disable( SCEGU_FOG );
	}
#endif
}

void Asura_PSP_Fog::OnEndFrame( void )
{
	#ifdef ASURA_PSP_PROFILING
	Asura_PSP_Profiling::Signal( Asura_PSP_Profiling::ASURA_PSP_SIGNAL_FOG_START ); 
	#endif // #ifdef ASURA_PSP_PROFILING

	#ifdef ASURA_PSP_USE_IMAGE_BASED_FOG

	if ( Asura_Fog::IsFogEnabled() )
	{
		const Asura_RGBA& xColour = Asura_Fog::GetFogColour();
		u_int uFogColour = MakeRGBA8888( xColour.m_fR, xColour.m_fG, xColour.m_fB, 0.0f );

		//////////////////////////////////////////////////////////////////////////////////
		// AJE - 21Mar2007
		// ** PLEASE DO NOT DELETE THE COMMENTED LINES DIRECTLY BELOW **
		// The lines below were from the original implementation of the image-based fog.
		// I'm leaving these in for now should anyone want to cast an eye over the fog code.
		// If the newer version of the fogging is preferred then the two lines can be removed.
//		const float fFogNear   = Asura_Fog::GetFogNearPlane();
//		const float fFogFar    = Asura_Fog::GetFogFarPlane();
		// The commented line below is only used by the original implementation.
//		const float frNearClip  = 1.0f / Asura_Camera::GetNearClip();

		// These are the 'tweaked' values used to produce a nicer and more uniform looking fog
		const float fFogNear   = Asura_Fog::GetFogNearPlane() * 4.0f;
		const float fFogFar    = Asura_Fog::GetFogFarPlane() * 4.0f;
		//////////////////////////////////////////////////////////////////////////////////

		const float frFogDiff  = 1.0f / (fFogFar - fFogNear);
		const float fFogFarScale = Asura_Fog::GetFogValueAtFarPlane();
		const float fFogSkyVal   = Asura_Fog::GetSkyboxFogBlendFactor();

		//+================================-//
		// Generate far clut				//
		//+================================-//

		u_int* pfClut = (u_int*)(((u_int)_zclutA) | 0x00000000);

		//////////////////////////////////////////////////////////////////////////////////
		// AJE - 21Mar2007
		// ** PLEASE DO NOT DELETE THE COMMENTED LINES DIRECTLY BELOW **
		// The commented block of code is the original implementation of the image-based fog.
		// I'm leaving this in for now should anyone want to cast an eye over the fog code.
		// If the newer version of the fogging is preferred then the commented block can be removed.
//		for (int i=0; i<255; i++) 
//		{
//			float fZ = ( 65536.0f / ((float)((256-i)<<4)) ) * frNearClip;
//
//			if (fZ < 0.0f)	
//			{
//				fZ = 0.0f;
//			}
//			else if (fZ > fFogFar)
//			{
//				fZ = fFogFar;
//			}
//
//			float fD = ((fZ - fFogNear) * frFogDiff) * fFogFarScale;
//
//			if (i < 64)										// Ramped palette bodge to blend in near plane as it is
//			{												// rendered 'pushed forward' to get round the near plane
//				fD *= ((float)(i>>1)) / 32.0f;				// palette index wrap problem.
//			}
//
//			*pfClut++ = uFogColour | ( ((u_int)(fD * 255.0f)) << 24 );
//		}
//
//		_zclutA[255] = uFogColour | ( ((u_int)(fFogSkyVal * fFogFarScale * 255.0f)) << 24 );		// Set up skybox blend factor
		/////////////////////////////////////////////////////////////

		//////////////////////////////////////////////////////////////////////////////////
		// AJE - 21Mar2007
		// The following block is the newer fog code - literally a linear blend with 'caps' for the near / far planes.
		int iMinFogIndex		= static_cast<int>(fFogNear);
		int iMaxFogIndex		= static_cast<int>(fFogFar);

		float fMinFogLevel		= 0.0f;
		float fMaxFogLevel		= 65536.0f * fFogFarScale;
		float fFogDelta			= (fMaxFogLevel - fMinFogLevel) * frFogDiff;
		float fOneOverMaxFog	= 1.0f / 65536.0f;
		float fZ				= 0.0f;

		for (int i=0; i<255; i++) 
		{
			if (i < iMinFogIndex)
			{
				// we are 'behind' the near fog plane
				fZ = fMinFogLevel;
			}
			else if (i > iMaxFogIndex)
			{
				// we are 'beyond' the far fog plane
				fZ = fMaxFogLevel;
			}
			else
			{
				// we are in-between the near and far fog planes
				fZ = (fFogDelta * static_cast<float>(i - iMinFogIndex));
			}

			float fD = fZ * fOneOverMaxFog;

			*pfClut++ = uFogColour | (((u_int)(fD * 255.0f)) << 24);
		}

		_zclutA[255] = uFogColour | ( ((u_int)(fFogSkyVal * fFogFarScale * 255.0f)) << 24 );		// Set up skybox blend factor
		//////////////////////////////////////////////////////////////////////////////////

		//////////////////////////////////////////////////////////////////////////////////
		// AJE - 21Mar2007
		// ** PLEASE DO NOT DELETE THE COMMENTED LINES DIRECTLY BELOW **
		// The commented block of code is the original implementation of the image-based fog.
		// However, the near clut was never used...
		// If the newer version of the fogging is preferred then the commented block can be removed.
//
//		//+================================-//
//		// Generate near clut				//
//		//+================================-//
//
//		/*u_int uNearFogVal =  _zclutA[1];	// uFogColour | ( ((u_int)(fD * 128.0f)) << 24 );
//
//		pfClut = (u_int*)(((u_int)_zclutB) | 0x00000000);
//
//		for (int i=0; i<256; i++) 
//		{
//			*pfClut++ = uNearFogVal;
//		}*/
		//////////////////////////////////////////////////////////////////////////////////

		//+================================-//
		// Setup render states				//
		//+================================-//

		Asura_PSP_Graphics::SetUseLighting(false);

		const unsigned int vramTop = (unsigned int)sceGeEdramGetAddr();
		const unsigned int zBuffer = (u_int)SCEGU_VRAM_BP32_2 | 0x00600000;

		GU_ClutMode( SCEGU_PF8888,4,0xff,0);						// For 16 bit clut indexed texture, index shift of 4

		GU_TexMode(SCEGU_PFIDX16, 0, SCEGU_SINGLE_CLUT, SCEGU_TEXBUF_NORMAL);
		GU_TexFunc( SCEGU_TEX_REPLACE, SCEGU_RGBA );
		GU_TexScale( 1.0f, 1.0f );
		GU_TexWrap( SCEGU_CLAMP, SCEGU_CLAMP );
		GU_TexOffset( 0.0f, 0.0f );
		GU_TexLevelMode (SCEGU_LOD_CONSTANT,0.0f);

		GU_TexFilter( SCEGU_LINEAR, SCEGU_LINEAR );
		GU_TexImage( 0, SCEGU_VRAM_WIDTH, 512, SCEGU_VRAM_WIDTH, (void *)(vramTop + zBuffer));
		GU_Enable(SCEGU_TEXTURE);
		GU_TexFlush();

		GU_TexLevelMode( SCEGU_LOD_CONSTANT, 0.0f );
		GU_TexFunc( SCEGU_TEX_REPLACE, SCEGU_RGBA );
		GU_TexWrap( SCEGU_CLAMP, SCEGU_CLAMP );

		Asura_PSP_RenderStates::RequestTexScale(  1.f, 1.f );
		Asura_PSP_RenderStates::RequestTexOffset( 0.f, 0.f );

		GU_Enable( SCEGU_BLEND );
		GU_BlendFunc( SCEGU_ADD , SCEGU_SRC_ALPHA,SCEGU_ONE_MINUS_SRC_ALPHA,0,0);
		GU_AlphaFunc( SCEGU_ALWAYS,0,0xff );
		GU_Color(0xffffffff);

		//+================================-//
		// Draw fog images					//
		//+================================-//

		GU_SpriteMode(64, 272, 64, 272);

		//////////////////////////////////////////////////////////////////////////////////
		// AJE - 21Mar2007
		// ** PLEASE DO NOT DELETE THE COMMENTED LINES DIRECTLY BELOW **
		// The commented block of code is the original implementation of the image-based fog.
		// However, the near clut was never used and the render calls were commented out.
		// If the newer version of the fogging is preferred then the commented block can be removed.
//		// TODO: When drawing 'near' fog sprites (using near clut to avoid palette indices wrapping round),
//		//		 artifacts are visible because of the overdraw. If the z buffer was inverted this problem would occur
//		//		 at the far plane, and be virtually unnoticeable. In the meantime, the ramped palette bodge
//		//		 works fine, and avoid the need for a second full screen pass..
//
//		/*GU_ClutLoad( 32, (void*)_zclutB );
//
//		GU_DrawSprite (  0, 0, 0x00000fff,   0, 0, SCEGU_NOFLIP, SCEGU_NOROTATE );
//		GU_DrawSprite ( 64, 0, 0x00000fff,  64, 0, SCEGU_NOFLIP, SCEGU_NOROTATE );
//		GU_DrawSprite (128, 0, 0x00000fff, 128, 0, SCEGU_NOFLIP, SCEGU_NOROTATE );
//		GU_DrawSprite (192, 0, 0x00000fff, 192, 0, SCEGU_NOFLIP, SCEGU_NOROTATE );
//		GU_DrawSprite (256, 0, 0x00000fff, 256, 0, SCEGU_NOFLIP, SCEGU_NOROTATE );
//		GU_DrawSprite (320, 0, 0x00000fff, 320, 0, SCEGU_NOFLIP, SCEGU_NOROTATE );
//		GU_DrawSprite (384, 0, 0x00000fff, 384, 0, SCEGU_NOFLIP, SCEGU_NOROTATE );
//		GU_DrawSprite (448, 0, 0x00000fff, 448, 0, SCEGU_NOFLIP, SCEGU_NOROTATE );*/
		//////////////////////////////////////////////////////////////////////////////////

		// Draw 'far' fog sprites
		
		GU_ClutLoad( 32, (void*)_zclutA );

		GU_DrawSprite (  0, 0, 0xf000,   0, 0, SCEGU_NOFLIP, SCEGU_NOROTATE );
		GU_DrawSprite ( 64, 0, 0xf000,  64, 0, SCEGU_NOFLIP, SCEGU_NOROTATE );
		GU_DrawSprite (128, 0, 0xf000, 128, 0, SCEGU_NOFLIP, SCEGU_NOROTATE );
		GU_DrawSprite (192, 0, 0xf000, 192, 0, SCEGU_NOFLIP, SCEGU_NOROTATE );
		GU_DrawSprite (256, 0, 0xf000, 256, 0, SCEGU_NOFLIP, SCEGU_NOROTATE );
		GU_DrawSprite (320, 0, 0xf000, 320, 0, SCEGU_NOFLIP, SCEGU_NOROTATE );
		GU_DrawSprite (384, 0, 0xf000, 384, 0, SCEGU_NOFLIP, SCEGU_NOROTATE );
		GU_DrawSprite (448, 0, 0xf000, 448, 0, SCEGU_NOFLIP, SCEGU_NOROTATE );

		//+================================-//
		// Reset render states				//
		//+================================-//

		GU_ClutMode( SCEGU_PF8888,0,0xff,0);
		GU_TexMode (SCEGU_PFIDX8, 0, SCEGU_SINGLE_CLUT, SCEGU_TEXBUF_NORMAL);
		GU_Enable( SCEGU_CULL_FACE );
		GU_Enable( SCEGU_DEPTH_TEST );

		Asura_Render::CurrentStates.RequestZBufferWriteEnabled( 0 );

		GU_TexFunc( SCEGU_TEX_MODULATE, SCEGU_RGBA );
		GU_TexLevelMode(SCEGU_LOD_SLOPE, 0.0f);
		GU_AlphaFunc(SCEGU_GREATER, 0x04, 0xff);

		// Go back to previous texture filter 
		GU_TexFilter(SCEGU_LINEAR_MIPMAP_LINEAR, SCEGU_LINEAR);

		// Set textures to repeat by default
		GU_TexWrap(SCEGU_REPEAT, SCEGU_REPEAT); 

		Asura_Render::CurrentStates.ResetAll();
	}

	#endif			// ASURA_PSP_USE_IMAGE_BASED_FOG

	#ifdef ASURA_PSP_PROFILING
	Asura_PSP_Profiling::Signal( Asura_PSP_Profiling::ASURA_PSP_SIGNAL_FOG_END ); 
	#endif // #ifdef ASURA_PSP_PROFILING
}

void Asura_PSP_Fog::TurnOnFogIfNecessary( void )
{
#ifndef ASURA_PSP_USE_IMAGE_BASED_FOG

	float fFogValueAtFarPlane = Asura_Fog::GetFogValueAtFarPlane();

	if ( Asura_Fog::IsFogEnabled() && fFogValueAtFarPlane > 0.0f )
	{
		GU_Enable( SCEGU_FOG );
	}
#endif
}
void Asura_PSP_Fog::TurnOffFogIfNecessary( void )
{
#ifndef ASURA_PSP_USE_IMAGE_BASED_FOG
	if ( Asura_Fog::IsFogEnabled() )
	{
		GU_Disable( SCEGU_FOG );
	}
#endif
}

