//
// Asura_PSP_Graphics.h
//
// Created by KJFL - Friday, February 06, 2004
//
// Copyright (C) 2004 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __ASURA_PSP_GRAPHICS_H__
#define __ASURA_PSP_GRAPHICS_H__

#include "Rendering/Asura_PSP_GU.h"

#define ASURA_PSP_DEFAULT_DISPLAY_LIST_SIZE 0x10000

typedef ScePspFMatrix4 Asura_PSP_Matrix4;
typedef ScePspFVector3 Asura_PSP_Vector3;

class Asura_Vector_3;

enum ASURA_PSP_GRAPHICS_MATERIALTYPE
{
	ASURA_PSP_GRAPHICS_MATERIALTYPE_NONE,
	ASURA_PSP_GRAPHICS_MATERIALTYPE_ENTITY,
	ASURA_PSP_GRAPHICS_MATERIALTYPE_ENV,
	ASURA_PSP_GRAPHICS_MATERIALTYPE_SHADOW,
	ASURA_PSP_GRAPHICS_MATERIALTYPE_SKYBOX,
	ASURA_PSP_GRAPHICS_MATERIALTYPE_MANUAL,
};

class Asura_PSP_Graphics
{
	public:
		static void Initialise(void);
		static void Shutdown(void);

		static void BeginFrame(void);
		static void EndFrame(void);
		static void VSync();
		static void	SwapBuffers(); 

		static void SetUseLighting(bool bUseLighting);
		static void SetUseVsync(bool bEnable) { s_bVsyncOn = bEnable; }
		static bool GetUseVsync() { return s_bVsyncOn; }
		static void SetNumberOfLights(u_int uNumLights);
		// Sets up lighting according to values in Asura_GI_Solution::s_xSHForCurrentRender
		static void SetupEntityLights( const float fAlpha , bool bObjectInWorldSpace = false);
		// KOR - 31/01/2006 - sets the translucency mode, z-writes, alpha test, etc. according to the given flags.
		static void SetupEntityAlphaStates(u_int uMaterialFlags, u_int uTextureFlags, bool bForceTranslucent = false, uint32 uProjectMaterialFlags = 0);
		static void SetMaterialType( ASURA_PSP_GRAPHICS_MATERIALTYPE eType, u_int uAmbientColour = 0xffffffff );
		// (MH - 19.12.2006) some code was setting GU_Material directly, breaking the cached render state. 
		// That code can now call InvalidateMaterialType() to fix the problem.
		static void InvalidateMaterialType() { s_eMaterialType = ASURA_PSP_GRAPHICS_MATERIALTYPE_NONE; }

		static void SetUseColourDoubling(bool bDouble);
		static void SetTransparentEntityWriteToZBuffer(bool bTransparentEntityWriteToZBuffer); 

		static void SetupPSPProjectionMatrix(const float fFOVInRadians, const float fNearPlaneOffset);
		static void SetupPSPProjectionMatrix(const float fNearPlaneOffset);
		static void SetupPSPMatFromAsuraData( const class Asura_Matrix_3x3& xOri, const class Asura_Vector_3& xPos );

		/*!
			The function accept a matrix as a parameter. If you specify to use projection the matrix provided is loaded
			to the texture matrix (or identity if no matrix is provided) and then we apply the perspective.
			If no parameter is specify we load the identity to the texture matrix.
		*/
		static void SetupPSPTextureMatrix(const Asura_PSP_Matrix4* pxMatrix = NULL, bool bUseProjectionMatrix = false);

		static bool s_bRecordImages;
		static u_int s_uCurrentImage;

		static u_int GetCurrentRenderingBuffer( void )
		{
			return s_uCurrentRenderingBuffer;
		}

		static u_int GetCurrentDisplayedBuffer( void )
		{
			return s_uCurrentDisplayedBuffer;
		}

		static uint32 GetCurrentDisplayList() { return s_uCurrentDisplayList; }
		static void SetEnableDeferredImmediateDraw( bool bEnable );

		static void SetUseHighContrastEntityLighting(bool bUseHighContrastEntityLighting) {s_bUseHighContrastEntityLighting = bUseHighContrastEntityLighting;}

		// State override function pointer
		typedef void ( fnStateOverride( uint32 uFlags ) );

		// Set a state override callback
		static void SetStateOverride( fnStateOverride *pfnOverride )		{ s_pfnStateOverride = pfnOverride; }

		// If a state override callback exists then call it
		static void DoStateOverride( uint32 uFlags = 0 );

#ifndef ASURA_PSP_SONY_SUBMISSION
		static void BeginFrame_ForAssert();
		static void EndFrame_ForAssert();
		static void VSync_ForAssert();
		static bool IsInFrameForAssert() { return s_bInFrameForAssert; }
		static u_int GetMaxDisplayList() { return s_uMaxDisplayList; }
#endif //#ifndef ASURA_PSP_SONY_SUBMISSION

		// mutex to control access to rendering. Maybe needed in non-assert builds for other purposes?
		static void LockMutex();
		static void UnlockMutex();

#ifdef ASURA_PSP_ENABLE_SCREENSHOTS
		// PSP only screenshot functions...
		static void SaveCurrentViewAsTarga(void);
		static void SaveTarga(char* szFileName, int iWidth, int iHeight, int iBitsPerPixel, void *pData);

		static void TakeManualScreenshot(void)	{ s_bTakeManualScreenshot = true; }
#endif
	private:
		static bool								s_bInitialised;
		static bool								s_bUseColourDoubling;
		static bool								s_bTransparentEntityWriteToZBuffer; 
		static bool								s_bUseHighContrastEntityLighting;
		static bool								s_bUseLighting;
		static bool								s_bVsyncOn;
		static u_int							s_uNumLights;
		static ASURA_PSP_GRAPHICS_MATERIALTYPE	s_eMaterialType;

		static u_int							s_uCurrentRenderingBuffer;
		static u_int							s_uCurrentDisplayedBuffer;

#ifndef ASURA_PSP_SONY_SUBMISSION
		static bool								s_bInFrameForAssert;
		static u_int							s_uMaxDisplayList;
#endif	//#ifndef ASURA_PSP_SONY_SUBMISSION

#if defined(ASURA_PSP_ENABLE_SCREENSHOTS)
		static bool								s_bTakeManualScreenshot;
#endif	// defined(ASURA_PSP_ENABLE_SCREENSHOTS)

		static u_int							s_uCurrentDisplayList;
		static bool								s_bDeferredImmediateDraw;
		static fnStateOverride					*s_pfnStateOverride;
};

#endif//__ASURA_PSP_GRAPHICS_H__
