//
// Asura_PSP_ObjectHierarchyRenderer.cpp
//
// Created by KJFL - Thursday, April 08, 2004
//
// Copyright (C) 2004 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

#include "Animation/Asura_Hierarchy.h"

// This file's header.
#include "Rendering/Asura_PSP_ObjectHierarchyRenderer.h"
#include "Rendering/Asura_PSP_ObjectHierarchyRenderer_Internal.h"

#include "Animation/Asura_Animation.h"
#include "Animation/Asura_TextureAnimation.h"
#include "Core/Asura_HashTree.h"
#include "Core/Asura_Material_System.h"
#include "Core/Asura_Stream.h"
#include "Core/Asura_TextureManagement.h"
#include "Light/Asura_Light.h"
#include "Light/Asura_Photon.h"
#include "Rendering/Asura_Render.h"

#include "Rendering/Asura_PSP_Graphics.h"
#include "Rendering/Asura_PSP_TextureManagement.h"
#include "Rendering/Asura_PSP_RenderStates.h"



Asura_PSP_ObjectHierarchyRenderer::FnBeginRendering*		Asura_PSP_ObjectHierarchyRenderer::s_pxBeginRenderingHierarchy		= NULL;
Asura_PSP_ObjectHierarchyRenderer::FnSetRenderingStates*	Asura_PSP_ObjectHierarchyRenderer::s_pxSetRenderingStatesHierarchy	= NULL;
Asura_PSP_ObjectHierarchyRenderer::FnEndRendering*			Asura_PSP_ObjectHierarchyRenderer::s_pxEndRenderingHierarchy		= NULL;

#ifdef ASURA_HIERARCHY_QUAD
	Asura_AMatrix4x4 Asura_PSP_ObjectHierarchyRenderer::s_xView;
#endif

////////////////////////////////////////////////////////////////////////////////
//
// Asura_PSP_ObjectHierarchyRender_Handler
//
////////////////////////////////////////////////////////////////////////////////

void Asura_PSP_ObjectHierarchyRender_Handler::Reset (void)
{
	m_xHashTree.Reset();
}


int Asura_PSP_ObjectHierarchyRender_Handler::GetMaterialIndex (const Asura_Hierarchy_Skin* pxSkin) const
{
	const Asura_PSP_ObjectHierarchy* pxHierarchy = GetHierarchy(pxSkin->m_uNameHashID);
	if(!pxHierarchy || !pxHierarchy->m_pxHierarchyStrips)
	{
		return ASURA_MATERIAL_NONE;
	}

	//find the first section that has a material , and return that
	for(u_int i=0;i<pxHierarchy->m_uNumberOfStrips;i++)
	{
		if(pxHierarchy->m_pxHierarchyStrips[i].m_iMaterialIndex != ASURA_MATERIAL_NONE)
		{
			return pxHierarchy->m_pxHierarchyStrips[i].m_iMaterialIndex;
		}
	}

	return ASURA_MATERIAL_NONE;
}


Asura_PSP_ObjectHierarchy* Asura_PSP_ObjectHierarchyRender_Handler::AddHierarchy( const Asura_Hash_ID uHash )
{
	bool bDidExist = true;
	Asura_PSP_ObjectHierarchy* pxObjHier = m_xHashTree.AddOrFindNode(uHash,&bDidExist,true);

	Asura_Assert(pxObjHier,"Out of memory creating new hash tree node");
	Asura_Assertf(!(pxObjHier && bDidExist),("Duplicate item 0x08%x",uHash));

	return (bDidExist) ? NULL : pxObjHier;
}

////////////////////////////////////////////////////////////////////////////////
//
// Asura_PSP_ObjectHierarchyRenderer
//
////////////////////////////////////////////////////////////////////////////////

#if defined(ASURA_USE_MULTIPLE_HANDLERS) || defined(ASURA_USE_STREAMING)
Asura_TemplateHashTree<Asura_PSP_ObjectHierarchyRender_Handler>	Asura_PSP_ObjectHierarchyRenderer::s_xHandlerTree;
Asura_PSP_ObjectHierarchyRender_Handler*						Asura_PSP_ObjectHierarchyRenderer::s_pxCurrentHandler	= NULL;
#else
Asura_PSP_ObjectHierarchyRender_Handler							Asura_PSP_ObjectHierarchyRenderer::s_xHandler;
#endif

// data on current status of drawing
bool									Asura_PSP_ObjectHierarchyRenderer::s_bNormalRendering;
bool									Asura_PSP_ObjectHierarchyRenderer::s_bShadowRendering;
const Asura_PSP_ObjectHierarchy*		Asura_PSP_ObjectHierarchyRenderer::s_pxCurrentHierarchyObject;


void Asura_PSP_ObjectHierarchyRenderer::Initialise( void )
{
	s_bShadowRendering = false;
	s_bNormalRendering = false;
	s_pxCurrentHierarchyObject = NULL;
}

void Asura_PSP_ObjectHierarchyRenderer::Reset()
{
#if defined(ASURA_USE_STREAMING)
	s_xHandlerTree.Reset();
	s_pxCurrentHandler	= NULL;
#else
	GetCurrentHandler().Reset();
#endif

	s_pxCurrentHierarchyObject = NULL;
}

void Asura_PSP_ObjectHierarchyRenderer::Shutdown( void )
{
#if defined(ASURA_USE_MULTIPLE_HANDLERS) || defined(ASURA_USE_STREAMING)
	s_xHandlerTree.Reset();
	s_pxCurrentHandler	= NULL;
#else
	s_xHandler.Reset();
#endif

	s_bShadowRendering = false;
	s_bNormalRendering = false;
	s_pxCurrentHierarchyObject = NULL;
}

#if defined(ASURA_USE_MULTIPLE_HANDLERS) || defined(ASURA_USE_STREAMING)

void Asura_PSP_ObjectHierarchyRenderer::ReparentHandler(Asura_PSP_ObjectHierarchyRender_Handler& xSourceHandler, const Asura_Hash_ID uNewParentHash)
{
	Asura_PSP_ObjectHierarchyRender_Handler*	pxParentHandler	= s_xHandlerTree.GetDataByHash(uNewParentHash);

	if(pxParentHandler != NULL)
	{
		xSourceHandler.m_xHashTree.SetParent(&(pxParentHandler->m_xHashTree));
	}
}

void Asura_PSP_ObjectHierarchyRenderer::ReparentHandler(const Asura_Hash_ID uSourceHash, const Asura_Hash_ID uNewParentHash)
{
	Asura_PSP_ObjectHierarchyRender_Handler*	pxHandler	= s_xHandlerTree.GetDataByHash(uSourceHash);

	if(pxHandler != NULL)
	{
		ReparentHandler(*pxHandler, uNewParentHash);
	}
}

Asura_PSP_ObjectHierarchyRender_Handler* Asura_PSP_ObjectHierarchyRenderer::CreateHandler (const Asura_Hash_ID uHash, const Asura_Hash_ID uParentHandlerHash)
{
	Asura_PSP_ObjectHierarchyRender_Handler*	pxHandler	= s_xHandlerTree.GetDataByHash(uHash);

	// Do we already have this zone?
	if(pxHandler == NULL)
	{
		// No - create one...
		pxHandler	= s_xHandlerTree.AddNode(uHash);

		if(pxHandler != NULL)
		{
			ReparentHandler( *pxHandler, uParentHandlerHash );
		}
	}
	else
	{
		// Yes - this shouldn't happen...
		Asura_Assert(false, "Already have an object heirarchy handler with this hash-ID.");
	}

	// Return the zone...
	return(pxHandler);
}


bool Asura_PSP_ObjectHierarchyRenderer::DestroyHandler (const Asura_Hash_ID uHash)
{
	// Call the delete function...
	return(s_xHandlerTree.DeleteNodeByHash(uHash));
}

void Asura_PSP_ObjectHierarchyRenderer::WriteHandlerReport(u_int uHandler, Asura_Stream& xLog)
{
	const u_int uLOGBUFFERSIZE = 64;
	char szLogBuffer[uLOGBUFFERSIZE];

	Asura_PSP_ObjectHierarchyRender_Handler* pxHandler = s_xHandlerTree.GetDataByHash(uHandler);
	if (pxHandler)
	{
		if (!pxHandler->m_xHashTree.GetNumberOfNodes())
		{
			xLog << "PSP ObjHier:\t\t\tNone\n";
		}
		else
		{
			Asura_PrintfToBfr( szLogBuffer, "PSP ObjHier:\t\t\t%d Item(s)\n", uLOGBUFFERSIZE, true, pxHandler->m_xHashTree.GetNumberOfNodes() );
			xLog << szLogBuffer;
		}
	}
	else
	{
		xLog << "PSP ObjHier:\t\t\tNo Handler\n";
	}
}

#endif	// defined(ASURA_USE_MULTIPLE_HANDLERS) || defined(ASURA_USE_STREAMING)


void Asura_PSP_ObjectHierarchyRenderer::BeginRender( const class Asura_Animation* pxAnim, const class Asura_Hierarchy_Skin* pxSkin )
{
	Asura_PSP_ObjectHierarchyRender_Handler&	xHandler	= GetCurrentHandler();

	s_bNormalRendering = false;
	s_pxCurrentHierarchyObject = xHandler.GetHierarchy(pxSkin->m_uNameHashID);

	if(s_pxCurrentHierarchyObject == NULL)
	{
		Asura_Assert(0,"no Hierarchy strips to render");
		return;
	}

	if (s_pxCurrentHierarchyObject->m_uTotalNumberOfVertices == 0 || s_pxCurrentHierarchyObject->m_uTotalNumberOfIndices == 0)
	{
		Asura_Assert(false, "This hierarchy has no polys!");
		return;
	}

	if (s_pxCurrentHierarchyObject->m_pxHierarchyStrips==0)
	{
		Asura_Assert(false, "No strips?");
		return;
	}

	// KJFL - 2004-02-20 11:11:18 - okay, we think we have a valid hierarchy
	s_bNormalRendering = true;
/*
	s_uAmbientARGB = 0xff000000;
	s_uAmbientARGB += static_cast<int>(Asura_Lighting_System::s_fEntityAmbientR * 0xff) << 16;
	s_uAmbientARGB += static_cast<int>(Asura_Lighting_System::s_fEntityAmbientG * 0xff) << 8;
	s_uAmbientARGB += static_cast<int>(Asura_Lighting_System::s_fEntityAmbientB * 0xff);
*/
	
	// (JG - 13.04.2007): Removed redundant lighting setup, as now is done earlier on; 
	// also this call was ignoring the animation / animation attachment render flags
	/*if(pxAnim->GetParent())
	{
		//This is an attachment , so just take lighting from parent.
		//Do it now , since it will be the same for each section RAR 02/04/2004
		pxAnim->UpdateLighting();
	}*/

	// Set fixed states here
	if( !s_pxSetRenderingStatesHierarchy )
	{
		Asura_Render::CurrentStates.RequestZBufferEnabled(true);
		Asura_Render::CurrentStates.RequestZBufferWriteEnabled(true);
		Asura_Render::CurrentStates.RequestCullMode(ASURA_CULLMODE_CW);
		Asura_PSP_Graphics::SetUseColourDoubling(true);
	}

	if(s_pxBeginRenderingHierarchy)
	{
		s_pxBeginRenderingHierarchy();
	}
}

DEBUGVAR float dbg_fNearTexSlopeObject = 0.3f; 
DEBUGVAR float dbg_fDistantTexSlopeObject = 0.125f; 

bool Asura_PSP_ObjectHierarchyRenderer::SetDefaultRenderingStates(const Asura_Animation* pxAnim, const Asura_PSP_ObjectHierarchy_Strip& xStrip, const float fAlpha, const int iTextureIDOverride )
{
	const int iTexture =  (iTextureIDOverride != ASURA_TEXTURE_NONE) ? iTextureIDOverride : Asura_Material_System::GetTextureID( xStrip.m_iMaterialIndex );
	const u_int uMaterialFlags = Asura_Material_System::GetMaterialFlags(xStrip.m_iMaterialIndex);
	const u_int uTextureFlags = Asura_Texture_Management::GetTextureFlags(iTexture);

	Asura_TextureAnimationOutput xTextureAnim;
	if(Asura_Material_System::GetTextureAnimationState(xStrip.m_iMaterialIndex, pxAnim->GetTextureAnimController(), &xTextureAnim))
	{
		Asura_PSP_RenderStates::RequestTexOffset(xTextureAnim.m_fUOffset, xTextureAnim.m_fVOffset);
	}
	else
	{
		Asura_PSP_RenderStates::RequestTexOffset(0.f, 0.f);
	}

	// JG - 15/11/06: Hack for mipmapping problems, reusing AsuraTextureFlag_PS2DistantTexture to flag textures applied to larger than normal objects
	Asura_PSP_RenderStates::RequestTexSlope( ( uTextureFlags & AsuraTextureFlag_PS2DistantTexture ) ? dbg_fDistantTexSlopeObject : dbg_fNearTexSlopeObject );

	Asura_PSP_Graphics::SetupEntityLights( fAlpha );
	Asura_PSP_Graphics::SetupEntityAlphaStates(uMaterialFlags, uTextureFlags, (fAlpha < 0.999f), Asura_Material_System::GetProjectFlags( xStrip.m_iMaterialIndex ));
	
	Asura_PSP_TextureManagement::RequestTextureIndex( iTexture );

	return (uMaterialFlags & AsuraMaterialFlag_SelfIlluminate);
}

#ifndef ASURA_HIERARCHY_QUAD
void Asura_PSP_ObjectHierarchyRenderer::Render( const Asura_Animation* pxAnim, const class Asura_Hierarchy_Skin* pxSkin, const int iSectionDataIndex, const Asura_Vector_3& xPosition, const Asura_Matrix_3x3& xOrientation, 
											    const int iTextureIDOverride )
{
	Asura_Assert(pxAnim && pxSkin, "No skin or anim to render.");
	if(!pxAnim || !pxSkin) { return; }

	if (!s_bNormalRendering || !s_pxCurrentHierarchyObject)
	{
		Asura_Assert( false, "No object setup, will skip");
		return;
	}

	if(s_pxCurrentHierarchyObject->m_uNumberOfStrips <= (u_int) iSectionDataIndex)
	{
		Asura_Assert(0,"number of bones and objects in hierarchy do not match");
		return;
	}

	const Asura_PSP_ObjectHierarchy_Strip& xStrip = (s_pxCurrentHierarchyObject->m_pxHierarchyStrips[iSectionDataIndex]);

	if (xStrip.m_uNumberOfTriangles==0)
	{
		// KJFL - 2004-02-20 11:12:22 - no triangles in this strip. No assert since some hierarchies may have valid 'dummy' bones.
		return;
	}

	const bool bColoursReplaceNormals = s_pxCurrentHierarchyObject->m_bColoursReplaceNormals;	
	const bool bShortVerts = s_pxCurrentHierarchyObject->m_bShortVerts;

	Asura_Assert(bShortVerts, "Short Verts support untested (this code appears deprecated; should never reach here)");

	if (bShortVerts)
	{
		if (bColoursReplaceNormals)
		{
			xVertexSize = sizeof(Asura_PSP_ObjectHierarchy_ShortVertex_Colour);
		}
		else
		{
			xVertexSize = sizeof(Asura_PSP_ObjectHierarchy_ShortVertex_Normal);
		}	

	}
	else
	{
		if (bColoursReplaceNormals)
		{
			xVertexSize = sizeof(Asura_PSP_ObjectHierarchy_Vertex_Colour);
		}
		else
		{
			xVertexSize = sizeof(Asura_PSP_ObjectHierarchy_Vertex_Normal);
		}
	}

	unsigned char* pucVertices = s_pxCurrentHierarchyObject->m_pucVertices + (xStrip.m_uStartVertex * xVertexSize);
	u_short* pxIndices = s_pxCurrentHierarchyObject->m_pusIndices + xStrip.m_uStartIndex;

	// KOR - 3/3/2006 - entity renderers now premultiply view matrix
	// into world matrix, to get around PSP's 24-bit precision rubbishness.
	Asura_PSP_Matrix4 xViewMatrix;
	GUM_MatrixMode(SCEGU_MATRIX_VIEW);
	GUM_PushMatrix();
	GUM_StoreMatrix(&xViewMatrix);
	GUM_LoadIdentity();

	Asura_PSP_Matrix4 xMatrix;

	xMatrix.x.x = xOrientation.e11;
	xMatrix.x.y = xOrientation.e21;
	xMatrix.x.z = xOrientation.e31;
	xMatrix.y.x = xOrientation.e12;
	xMatrix.y.y = xOrientation.e22;
	xMatrix.y.z = xOrientation.e32;
	xMatrix.z.x = xOrientation.e13;
	xMatrix.z.y = xOrientation.e23;
	xMatrix.z.z = xOrientation.e33;

	xMatrix.w.x = xPosition.x;
	xMatrix.w.y = xPosition.y;
	xMatrix.w.z = xPosition.z;
	xMatrix.w.w = 1.0f;

	xMatrix.x.w = 0.f;
	xMatrix.y.w = 0.f;
	xMatrix.z.w = 0.f;

//return;
	GUM_MatrixMode(SCEGU_MATRIX_WORLD);
	GUM_PushMatrix();

	GUM_LoadMatrix(&xViewMatrix);
	GUM_MultMatrix(&xMatrix);

	bool bSelfIlluminate;

	if(s_pxSetRenderingStatesHierarchy)
	{
		bSelfIlluminate = s_pxSetRenderingStatesHierarchy(pxAnim, &xViewMatrix, &xMatrix, xStrip, pxAnim->GetAlpha(), iTextureIDOverride);
	}
	else
	{
		bSelfIlluminate = SetDefaultRenderingStates(pxAnim, xStrip, pxAnim->GetAlpha(), iTextureIDOverride);
	}

	u_int uVertexFormat;
	if (bShortVerts)
	{
		if (bColoursReplaceNormals)
		{
			uVertexFormat = SCEGU_COLOR_PF8888 | SCEGU_TEXTURE_USHORT | SCEGU_NORMAL_NONE | SCEGU_VERTEX_SHORT | SCEGU_INDEX_USHORT;
		}
		else
		{
			uVertexFormat = SCEGU_COLOR_NONE | SCEGU_TEXTURE_USHORT | SCEGU_NORMAL_SHORT | SCEGU_VERTEX_SHORT | SCEGU_INDEX_USHORT;
		}	
	}
	else
	{
		if (bColoursReplaceNormals)
		{
			uVertexFormat = SCEGU_COLOR_PF8888 | SCEGU_TEXTURE_FLOAT | SCEGU_NORMAL_NONE | SCEGU_VERTEX_FLOAT | SCEGU_INDEX_USHORT;
		}
		else
		{
			uVertexFormat = SCEGU_COLOR_NONE | SCEGU_TEXTURE_FLOAT | SCEGU_NORMAL_FLOAT | SCEGU_VERTEX_FLOAT | SCEGU_INDEX_USHORT;
		}
	}

	GUM_DrawArray
	(
		SCEGU_PRIM_TRIANGLE_STRIP,
		uVertexFormat,
		xStrip.m_uNumberOfTriangles+2,
		pxIndices,
		pucVertices
	);

	if(bSelfIlluminate)
	{
		Asura_Render::CurrentStates.RequestTranslucencyMode( ASURA_TRANSLUCENCY_GLOWING );
		// Disable all lights
		Asura_PSP_Graphics::SetNumberOfLights(0);
		Asura_PSP_Graphics::SetUseLighting(false);

		GUM_DrawArray
		(
			SCEGU_PRIM_TRIANGLE_STRIP,
			uVertexFormat,
			xStrip.m_uNumberOfTriangles+2,
			pxIndices,
			pucVertices
		);
	#if defined ASURA_RENDER_TTYPES
		Asura_Render::AddToTrianglesRendered(xStrip.m_uNumberOfTriangles,1,
			ASURA_RENDER_TTYPE_SELF);
	#else
		Asura_Render::AddToTrianglesRendered(xStrip.m_uNumberOfTriangles);
	#endif
	}
	GUM_PopMatrix();
	GUM_MatrixMode(SCEGU_MATRIX_VIEW);
	GUM_PopMatrix();

#if defined ASURA_RENDER_TTYPES
	Asura_Render::AddToTrianglesRendered(xStrip.m_uNumberOfTriangles,1,
		ASURA_RENDER_TTYPE_HIER);
#else
	Asura_Render::AddToTrianglesRendered(xStrip.m_uNumberOfTriangles);
#endif

	// JG - 11/09/07: Set UV offsets to zero by default
	Asura_PSP_RenderStates::RequestTexOffset( 0.0f, 0.0f ); 

	if(s_pxEndRenderingHierarchy)
	{
		s_pxEndRenderingHierarchy();
	}
}

#else	// ASURA_HIERARCHY_QUAD

void Asura_PSP_ObjectHierarchyRenderer::SetViewMatrix( const Asura_Matrix_3x3& rxOri, const Asura_Vector_3& rxPos )
{
	// RJT TODO: Could save a few cycles here
	s_xView.SetColumn0(Asura_AVector4( rxOri.e11,	-rxOri.e12, -rxOri.e13, 0.0f ) );
	s_xView.SetColumn1(Asura_AVector4( rxOri.e21,	-rxOri.e22, -rxOri.e23, 0.0f ) );
	s_xView.SetColumn2(Asura_AVector4( rxOri.e31,	-rxOri.e32, -rxOri.e33, 0.0f ) );
	s_xView.SetColumn3(Asura_AVector4( 0.0f,			0.0f,		0.0f,		1.0f ) );
	s_xView.SetColumn3(Asura_AVector4( -rxPos.x,		-rxPos.y,	-rxPos.z,	1.0f ) * s_xView );

	// RJT TODO: Find static form of this (and move to higher level?)
	Asura_AMatrix4x4 xIdentity; xIdentity.Identity();
	GU_SetMatrix( SCEGU_MATRIX_VIEW, reinterpret_cast< const ScePspFMatrix4 * >( &xIdentity ) );
}

void Asura_PSP_ObjectHierarchyRenderer::Render(	const class Asura_Animation* pxAnim, const class Asura_Hierarchy_Skin* pxSkin, const int iSectionDataIndex, Asura_AMatrix4x4& xTransform,
												const int iTextureIDOverride )
{
	Asura_Assert(pxAnim && pxSkin, "No skin or anim to render.");
	if(!pxAnim || !pxSkin) { return; }

	if (!s_bNormalRendering || !s_pxCurrentHierarchyObject)
	{
		Asura_Assert( false, "No object setup, will skip");
		return;
	}

	if(s_pxCurrentHierarchyObject->m_uNumberOfStrips <= (u_int) iSectionDataIndex)
	{
		Asura_Assert(0,"number of bones and objects in hierarchy do not match");
		return;
	}

	const Asura_PSP_ObjectHierarchy_Strip& xStrip = (s_pxCurrentHierarchyObject->m_pxHierarchyStrips[iSectionDataIndex]);
	if (xStrip.m_uNumberOfTriangles==0)
	{
		// KJFL - 2004-02-20 11:12:22 - no triangles in this strip. No assert since some hierarchies may have valid 'dummy' bones.
		return;
	}

	const bool bColoursReplaceNormals = s_pxCurrentHierarchyObject->m_bColoursReplaceNormals;
	const bool bShortVerts = s_pxCurrentHierarchyObject->m_bShortVerts;

	size_t xVertexSize;
	if (bShortVerts)
	{
		if (bColoursReplaceNormals)
		{
			xVertexSize = sizeof(Asura_PSP_ObjectHierarchy_ShortVertex_Colour);
		}
		else
		{
			xVertexSize = sizeof(Asura_PSP_ObjectHierarchy_ShortVertex_Normal);
		}	

		// Also, add a scaling factor to the transformations to compensate for adjusting to fit within -16384,16384		
		for(u_int uColumn = 0; uColumn < 3; uColumn++)
		{		
			Asura_AVector4	xColumn = xTransform.GetColumn(uColumn);
			xColumn	*= 2.0f;
			xTransform.SetColumn(uColumn, xColumn);
		}
	}
	else
	{
		if (bColoursReplaceNormals)
		{
			xVertexSize = sizeof(Asura_PSP_ObjectHierarchy_Vertex_Colour);
		}
		else
		{
			xVertexSize = sizeof(Asura_PSP_ObjectHierarchy_Vertex_Normal);
		}
	}

	unsigned char* pucVertices = s_pxCurrentHierarchyObject->m_pucVertices + (xStrip.m_uStartVertex * xVertexSize);
	u_short* pxIndices = s_pxCurrentHierarchyObject->m_pusIndices + xStrip.m_uStartIndex;

	// KOR - 3/3/2006 - entity renderers now premultiply view matrix
	// into world matrix, to get around PSP's 24-bit precision rubbishness.
	Asura_AMatrix4x4	xWorldView;

//	xWorldView	= xTransform * s_xView;
	s_xView.Apply(xTransform, xWorldView);
	GU_SetMatrix( SCEGU_MATRIX_WORLD, reinterpret_cast< const ScePspFMatrix4 * >( &xWorldView ) );

	bool bSelfIlluminate;
	if( s_pxSetRenderingStatesHierarchy )
	{
		// RJT TODO: Cleanup
		bSelfIlluminate = s_pxSetRenderingStatesHierarchy( pxAnim,
			reinterpret_cast< Asura_PSP_Matrix4 * >( &s_xView ),
			const_cast< Asura_PSP_Matrix4 * >( reinterpret_cast< const Asura_PSP_Matrix4 * >( &xTransform ) ),
			xStrip, pxAnim->GetAlpha(), iTextureIDOverride
		);
	}
	else
	{
		// RJT TODO: Cache state
		bSelfIlluminate = SetDefaultRenderingStates(pxAnim, xStrip, pxAnim->GetAlpha(), iTextureIDOverride );
	}

	u_int uVertexFormat;
	if (bShortVerts)
	{
		if (bColoursReplaceNormals)
		{
			uVertexFormat = SCEGU_COLOR_PF8888 | SCEGU_TEXTURE_USHORT | SCEGU_NORMAL_NONE | SCEGU_VERTEX_SHORT | SCEGU_INDEX_USHORT;
		}
		else
		{
			uVertexFormat = SCEGU_COLOR_NONE | SCEGU_TEXTURE_USHORT | SCEGU_NORMAL_SHORT | SCEGU_VERTEX_SHORT | SCEGU_INDEX_USHORT;
		}	
	}
	else
	{
		if (bColoursReplaceNormals)
		{
			uVertexFormat = SCEGU_COLOR_PF8888 | SCEGU_TEXTURE_FLOAT | SCEGU_NORMAL_NONE | SCEGU_VERTEX_FLOAT | SCEGU_INDEX_USHORT;
		}
		else
		{
			uVertexFormat = SCEGU_COLOR_NONE | SCEGU_TEXTURE_FLOAT | SCEGU_NORMAL_FLOAT | SCEGU_VERTEX_FLOAT | SCEGU_INDEX_USHORT;
		}
	}

	GU_DrawArray(
		SCEGU_PRIM_TRIANGLE_STRIP,
		uVertexFormat,
		xStrip.m_uNumberOfTriangles + 2,
		pxIndices,
		pucVertices
	);

	if( bSelfIlluminate )
	{
		Asura_Render::CurrentStates.RequestTranslucencyMode( ASURA_TRANSLUCENCY_GLOWING );

		// Disable all lights
		Asura_PSP_Graphics::SetNumberOfLights(0);
		Asura_PSP_Graphics::SetUseLighting(false);

		GU_DrawArray(
			SCEGU_PRIM_TRIANGLE_STRIP,
			uVertexFormat,
			xStrip.m_uNumberOfTriangles + 2,
			pxIndices,
			pucVertices
		);

#if defined ASURA_RENDER_TTYPES
		Asura_Render::AddToTrianglesRendered(xStrip.m_uNumberOfTriangles,1,ASURA_RENDER_TTYPE_SELF);
#else
		Asura_Render::AddToTrianglesRendered(xStrip.m_uNumberOfTriangles);
#endif
	}

#if defined ASURA_RENDER_TTYPES
	Asura_Render::AddToTrianglesRendered(xStrip.m_uNumberOfTriangles,1,ASURA_RENDER_TTYPE_HIER);
#else
	Asura_Render::AddToTrianglesRendered(xStrip.m_uNumberOfTriangles);
#endif

	// JG - 11/09/07: Set UV offsets to zero by default
	Asura_PSP_RenderStates::RequestTexOffset( 0.0f, 0.0f ); 

	if( s_pxEndRenderingHierarchy )
	{
		s_pxEndRenderingHierarchy();
	}
}
#endif	// ASURA_HIERARCHY_QUAD

void Asura_PSP_ObjectHierarchyRenderer::EndRender(void)
{
	Asura_PSP_RenderStates::RequestTexSlope( dbg_fNearTexSlopeObject );

	s_pxCurrentHierarchyObject = 0;
	s_bNormalRendering = false;

#ifdef ASURA_HIERARCHY_QUAD
	// RJT TODO: Get rid (requires complete removal of GUM)
	GUM_MatrixMode( SCEGU_MATRIX_WORLD );
	GUM_PushMatrix();
	GUM_PopMatrix();
	GUM_MatrixMode( SCEGU_MATRIX_VIEW );
	GUM_PushMatrix();
	GUM_PopMatrix();
#endif
}
 
void Asura_PSP_ObjectHierarchyRenderer::BeginShadowRender( const class Asura_Hierarchy_Skin* pxSkin, const Asura_Vector_3& xShadowPosition )
{
	Asura_PSP_ObjectHierarchyRender_Handler&	xHandler	= GetCurrentHandler();

	s_bShadowRendering = false;
	s_pxCurrentHierarchyObject = xHandler.GetHierarchy(pxSkin->m_uNameHashID);

	if(s_pxCurrentHierarchyObject == NULL)
	{
		Asura_Assert(0,"no Hierarchy strips to render");
		return;
	}

	if (s_pxCurrentHierarchyObject->m_uTotalNumberOfVertices == 0 || s_pxCurrentHierarchyObject->m_uTotalNumberOfIndices == 0)
	{
		Asura_Assert(false, "This hierarchy has no polys!");
		return;
	}

	if (s_pxCurrentHierarchyObject->m_pxHierarchyStrips==0)
	{
		Asura_Assert(false, "No strips?");
		return;
	}

	// KJFL - 2004-02-20 11:11:18 - okay, we think we have a valid hierarchy
	s_bShadowRendering = true;

	Asura_Render::CurrentStates.RequestZBufferWriteEnabled(false);
	
}

void Asura_PSP_ObjectHierarchyRenderer::RenderShadow( const int iSectionDataIndex, const Asura_Vector_3& xPosition, const Asura_Matrix_3x3& xOrientation )
{
	Asura_PSP_Graphics::SetUseColourDoubling(true);

	if (!s_bShadowRendering || !s_pxCurrentHierarchyObject)
	{
		Asura_Assert( false, "No object setup, will skip");
		return;
	}

	if(s_pxCurrentHierarchyObject->m_uNumberOfStrips <= (u_int) iSectionDataIndex)
	{
		Asura_Assert(0,"number of bones and objects in hierarchy do not match");
		return;
	}

	Asura_PSP_ObjectHierarchy_Strip* pxStrip = &(s_pxCurrentHierarchyObject->m_pxHierarchyStrips[iSectionDataIndex]);

	if (pxStrip->m_uNumberOfTriangles==0)
	{
		// KJFL - 2004-02-20 11:12:22 - no triangles in this strip. No
		// assert since some hierarchies may have valid 'dummy' bones.
		return;
	}
#if 0
	float afTransformationData[] = 
	{
		// x rotation 
		xOrientation.e11, xOrientation.e12, xOrientation.e13, xPosition.x,

		// z rotation
		xOrientation.e31, xOrientation.e32, xOrientation.e33, xPosition.z,
	};
#endif

#if defined ASURA_RENDER_TTYPES
	Asura_Render::AddToTrianglesRendered(pxStrip->m_uNumberOfTriangles,1,
		ASURA_RENDER_TTYPE_SHA);
#else
	Asura_Render::AddToTrianglesRendered(pxStrip->m_uNumberOfTriangles);
#endif
}

void Asura_PSP_ObjectHierarchyRenderer::EndShadowRender(void)
{
	s_pxCurrentHierarchyObject = 0;
	s_bShadowRendering = false;
}

/// End of file. ///