//
// Asura_PSP_ObjectRenderer.h
//
// Created by KJFL - Wednesday, April 07, 2004
//
// Copyright (C) 2004 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __ASURA_PSP_OBJECTRENDERER_H__
#define __ASURA_PSP_OBJECTRENDERER_H__

#include "Core/Asura_HashTree.h"
#include "Rendering/Asura_PSP_ObjectRenderer_Internal.h"

class Asura_Stream;

class Asura_PSP_ObjectRender_Handler
{
	friend class Asura_Chunk_ResourceFile_PSP_Object;
	friend class Asura_PSP_ObjectRenderer;

public:
	// Public functions...
	Asura_PSP_ObjectRender_Handler  ()	{}
//	~Asura_PSP_ObjectRender_Handler ()	{}

	void							Reset				(void);
	int								GetMaterialIndex	(const class Asura_Shape& xShape);

private:
	// Private variables...
	Asura_TemplateHashTree<Asura_PSP_ObjectStrip>	m_xHashTree;

	// Private functions...
	Asura_PSP_ObjectStrip*			AddObjectStrip		(const Asura_Hash_ID uHash);
	const Asura_PSP_ObjectStrip*	GetObjectStrip		(const Asura_Hash_ID uHash) const	{ return(m_xHashTree.GetDataByHash(uHash)); }
};


class Asura_PSP_ObjectRenderer
{
	friend class Asura_PSP_ObjectRender_Handler;

public:
	static void Initialise( void );
	static void Reset( void );
	static void Shutdown( void );

	static void Render( const class Asura_Shape* pxShape, const class Asura_Shape_Instance* pxShapeInst, const u_int uARGB );

	static int GetMaterialIndex( const class Asura_Shape& xShape ) { return(GetCurrentHandler().GetMaterialIndex(xShape)); }

#if defined(ASURA_USE_MULTIPLE_HANDLERS) || defined(ASURA_USE_STREAMING)
	// Handler functions...
	static	void							ReparentHandler		(Asura_PSP_ObjectRender_Handler& xSourceHandler, const Asura_Hash_ID uNewParentHash);
	static	void							ReparentHandler		(const Asura_Hash_ID uSourceHash, const Asura_Hash_ID uNewParentHash);	

	static	Asura_PSP_ObjectRender_Handler*	CreateHandler		(const Asura_Hash_ID uHash, const Asura_Hash_ID uParentHandlerHash);
	static	bool							DestroyHandler		(const Asura_Hash_ID uHash);
	static	void							WriteHandlerReport	(u_int uHandler, Asura_Stream& xLog);

	static	void							CreateAndSetHandler	(const Asura_Hash_ID uHash, const Asura_Hash_ID uParentHandlerHash)	{ s_pxCurrentHandler = CreateHandler(uHash, uParentHandlerHash); }
	static	void							SetCurrentHandler	(Asura_PSP_ObjectRender_Handler* pxHandler)							{ s_pxCurrentHandler = pxHandler; }
	static	void							SetCurrentHandler	(const Asura_Hash_ID uHash)											{ s_pxCurrentHandler = s_xHandlerTree.GetDataByHash(uHash); }
	static	Asura_PSP_ObjectRender_Handler&	GetCurrentHandler	(void)																{ return(*s_pxCurrentHandler); }
	static	Asura_PSP_ObjectRender_Handler*	GetHandler			(const Asura_Hash_ID uHash)											{ return(s_xHandlerTree.GetDataByHash(uHash)); }
#else
	// Handler functions...
	static	Asura_PSP_ObjectRender_Handler&	GetCurrentHandler	(void)																{ return(s_xHandler); }
#endif

private:
#if defined(ASURA_USE_MULTIPLE_HANDLERS) || defined(ASURA_USE_STREAMING)
	// Private variables...
	static	Asura_PSP_ObjectRender_Handler*							s_pxCurrentHandler;		// Currently active handler.
	static	Asura_TemplateHashTree<Asura_PSP_ObjectRender_Handler>	s_xHandlerTree;			// Tree of all of the handlers available.
#else
	// Private variables...
	static	Asura_PSP_ObjectRender_Handler							s_xHandler;				// Single static-instance of a zone.
#endif
};

#endif//__ASURA_PSP_OBJECTRENDERER_H__


/// End of file. ///
