//
// Asura_PSP_RenderEnvironment.h
//
// Created by KJFL - Thursday, February 05, 2004
//
// Copyright (C) 2004 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __ASURA_PSP_RENDERENVIRONMENT_H__
#define __ASURA_PSP_RENDERENVIRONMENT_H__

#define PSP_NEW_ENV_RENDER 1

// RJT NOTE: Clipping optimisation making use of quad aligned data/SIMD
// RJT TODO: Move to project settings?
#define ASURA_PSP_RENDERENVIRONMENT_QUADCLIPPING
#ifdef ASURA_PSP_RENDERENVIRONMENT_QUADCLIPPING
	// RJT NOTE: This define enables extra use of the VFPU (write-back cache etc...) with inline assembly, without this just intrinsics are used
	#define ASURA_PSP_RENDERENVIRONMENT_QUADCLIPPING_VFPU
#endif

#include "Core/Asura_HashTree.h"
#include "Core/Memory/Asura_ResourceAllocator.h"
#include "Rendering/Asura_PSP_Graphics.h"

#include "Rendering/Asura_PSP_VertexFormat.h"

#include "Core/Asura_Maths.h"

enum ASURA_PSP_MODULE_CLIP_FLAGS
{
	ASURA_PSP_MODULE_OUTSIDE_FRUSTUM_U		= 0x01,
	ASURA_PSP_MODULE_OUTSIDE_FRUSTUM_D		= 0x02,
	ASURA_PSP_MODULE_OUTSIDE_FRUSTUM_L		= 0x04,
	ASURA_PSP_MODULE_OUTSIDE_FRUSTUM_R		= 0x08,
	ASURA_PSP_MODULE_OUTSIDE_FRUSTUM_ANY	= 0x0F,
	ASURA_PSP_MODULE_CLIPS_GUARDBAND_U		= 0x10,
	ASURA_PSP_MODULE_CLIPS_GUARDBAND_D		= 0x20,
	ASURA_PSP_MODULE_CLIPS_GUARDBAND_L		= 0x40,
	ASURA_PSP_MODULE_CLIPS_GUARDBAND_R		= 0x80,
	ASURA_PSP_MODULE_CLIPS_GUARDBAND_ANY	= 0xF0,
};

// do not change this as it's used in asm code
struct Asura_PSP_Plane
{
#ifdef ASURA_PSP_RENDERENVIRONMENT_QUADCLIPPING
#pragma inline
	float DistanceTo( const Asura_AVector4& rxPoint ) const
	{
		float fResult;
#if defined ASURA_PSP_RENDERENVIRONMENT_QUADCLIPPING_VFPU && defined ASURA_PSP_ASM_USE_SN_SYNTAX_NEW
		const SNVectorType& rxVFPUPoint = reinterpret_cast< const SNVectorType& >( rxPoint );
		SNScalarType fVFPUResult;
		__asm {
			vdot.t*		fVFPUResult, xVFPUPlane, rxVFPUPoint
			vadd.s		fVFPUResult, fVFPUResult, xVFPUPlane( w )
		};
		fResult = __builtin_vstof( fVFPUResult );
#elif defined ASURA_PSP_RENDERENVIRONMENT_QUADCLIPPING_VFPU && defined ASURA_PSP_USE_VFPU_ASSEMBLER
		__asm__ (
			".set		push\n"							// Save assembler option
			".set		noreorder\n"					// Suppress reordering
			"lv.q		c000, 0 + %1\n"
			"lv.q		c010, 0 + %2\n"
			"vdot.t		s000, c000, c010\n"
			"vadd.s		s000, s000, s003\n"
			"mfv		$8, s000\n"
			"mtc1		$8, %0\n"
			".set		pop\n"							// Restore assembler option
			: "=f"( fResult )
			: "m"( *this ), "m"( rxPoint )
			: "$8"
		);		
#else
		fResult = ( m_xNormal * *reinterpret_cast< const Asura_Vector_3 * >( &rxPoint ) ) + m_fPosition;
#endif
		return fResult;
	}
#endif

#if defined ASURA_PSP_RENDERENVIRONMENT_QUADCLIPPING_VFPU && defined ASURA_PSP_ASM_USE_SN_SYNTAX_NEW
	union
	{
		SNVectorType xVFPUPlane;
		struct
		{
			Asura_Vector_3	m_xNormal;
			float			m_fPosition;
		};
	};
#else
	Asura_Vector_3	m_xNormal;
	float			m_fPosition;
#endif
}__attribute__((aligned(16)));

// do not change this as it's used in asm code
struct Asura_PSP_FrustumLimits
{
	Asura_PSP_Plane		m_xLeft;
	Asura_PSP_Plane		m_xRight;
	Asura_PSP_Plane		m_xUp;
	Asura_PSP_Plane		m_xDown;
}__attribute__((aligned(16)));

struct Asura_PSP_RenderEnvironment_VertexClip
{
	// Uncompressed vertex, to get more accuracy on interpolation 
	u_short			m_usU; 
	u_short			m_usV; 
	int				m_iRGBA8888; 
	short			m_asNormal[3+1]; // Added extra short for padding
	float			m_afPosition[3]; 

	// Uncompress from flexible format
	inline void Uncompress(const unsigned char* pucVertexStart, unsigned short usFormat)
	{
		Asura_PSP_VertexFormat::ShortVector2 xTexCoord = Asura_PSP_VertexFormat::GetTexCoordsFromVertex(pucVertexStart, usFormat);
		Asura_PSP_VertexFormat::ShortVector3 xNormal = Asura_PSP_VertexFormat::GetNormalFromVertex(pucVertexStart, usFormat);
		Asura_Vector_3 xPosition = Asura_PSP_VertexFormat::GetPositionFromVertex(pucVertexStart, usFormat);
		m_usU = xTexCoord.x;
		m_usV = xTexCoord.y;
		m_iRGBA8888 = Asura_PSP_VertexFormat::GetColourFromVertex(pucVertexStart, usFormat);
		m_asNormal[0] = xNormal.x;
		m_asNormal[1] = xNormal.y;
		m_asNormal[2] = xNormal.z;
		m_afPosition[0] = xPosition.x;
		m_afPosition[1] = xPosition.y;
		m_afPosition[2] = xPosition.z;
	}
};

#ifdef ASURA_PSP_RENDERENVIRONMENT_QUADCLIPPING
struct Asura_PSP_RenderEnvironment_VertexClipQuad
{
	// VFPU friendly components
	Asura_AVector4 xRGBA;
	Asura_AVector4 xXYZU;
	Asura_AVector4 xXYZV;

	// Assignment operator
#ifdef ASURA_PSP_RENDERENVIRONMENT_QUADCLIPPING_VFPU
#pragma inline
	Asura_PSP_RenderEnvironment_VertexClipQuad& operator =( const Asura_PSP_RenderEnvironment_VertexClipQuad& rxSource )
	{
#ifdef ASURA_PSP_ASM_USE_SN_SYNTAX_NEW
		SNVectorType xA, xB, xC;
		__asm {
			lv.q		xA, 0( &rxSource )
			lv.q		xB, 16( &rxSource )
			lv.q		xC, 32( &rxSource )
			sv.q		xA, 0( this ), wb
			sv.q		xB, 16( this ), wb
			sv.q		xC, 32( this ), wb
		}
#elif ASURA_PSP_USE_VFPU_ASSEMBLER
		__asm__ (
			".set		push\n"							// Save assembler option
			".set		noreorder\n"					// Suppress reordering
			"lv.q		c000,	0 + %1\n"
			"lv.q		c010,	16 + %1\n"
			"lv.q		c020,	32 + %1\n"
			"sv.q		c000,	0 + %0, wb\n"
			"sv.q		c010,	16 + %0, wb\n"
			"sv.q		c020,	32 + %0, wb\n"
			".set		pop\n"							// Restore assembler option
			: "=m"( *this )
			: "m"( rxSource )
		);
#endif
		
		return *this;
	}
#endif	// ASURA_PSP_RENDERENVIRONMENT_QUADCLIPPING_VFPU

	// Generate clipped vertex
#pragma inline
	void Clipped( const Asura_PSP_RenderEnvironment_VertexClipQuad& rxSrcA, const Asura_PSP_RenderEnvironment_VertexClipQuad& rxSrcB, const float kfTA, const float kfTB, const Asura_AMatrix4x4& rxProjMat )
	{
		// RJT TODO:
//#if defined ASURA_PSP_RENDERENVIRONMENT_QUADCLIPPING_VFPU && defined ASURA_PSP_ASM_USE_SN_SYNTAX_NEW
//#elif defined ASURA_PSP_RENDERENVIRONMENT_QUADCLIPPING_VFPU && defined ASURA_PSP_USE_VFPU_ASSEMBLER
//#else	// #elif defined ASURA_PSP_RENDERENVIRONMENT_QUADCLIPPING_VFPU && defined ASURA_PSP_USE_VFPU_ASSEMBLER
		float fDelta = kfTA / ( kfTA - kfTB );
		float fOneMinusDelta = 1.0f - fDelta;
		xXYZU = ( rxSrcA.xXYZU * fOneMinusDelta ) + ( rxSrcB.xXYZU * fDelta );
		xXYZV = ( rxSrcA.xXYZV * fOneMinusDelta ) + ( rxSrcB.xXYZV * fDelta );
#ifdef ASURA_PSP_NO_CLIPPER_COLOUR_INTERPOLATION
		xRGBA = ( rxSrcA.xRGBA * fOneMinusDelta ) + ( rxSrcB.xRGBA * fDelta );
#else
		Asura_AVector4 xA = rxSrcA.xXYZV; xA.SetW( 1.0f );
		Asura_AVector4 xB = xXYZV; xB.SetW( 1.0f );
		Asura_AVector4 xC = rxSrcB.xXYZV; xC.SetW( 1.0f );

		xA = xA * rxProjMat;
		xB = xB * rxProjMat;
		xC = xC * rxProjMat;

		float fInverseA = 1.0f / ( xA.GetW() + 0.001f );
		float fInverseB = 1.0f / ( xB.GetW() + 0.001f );
		float fInverseC = 1.0f / ( xC.GetW() + 0.001f );

		const Asura_AVector4 xDimensions( 240.0f, 136.0f, 0.0f, 0.0f );
		xA = xA.ComponentwiseProduct( xDimensions );
		xB = xB.ComponentwiseProduct( xDimensions );
		xC = xC.ComponentwiseProduct( xDimensions );

		xA = xA * fInverseA;
		xB = xB * fInverseB;
		xC = xC * fInverseC;

		float f = ( xC - xA ).Magnitude();
		fDelta = f ? ( xB - xA ).Magnitude() / f : 0.0f;
//		fDelta = ( xB - xA ).Magnitude() / f;
		fDelta = Asura_Maths::ClampToRange( fDelta, 0.0f, 1.0f );
//		fDelta = ( xB - xA ).Magnitude() / ( xC - xA ).Magnitude();
		fOneMinusDelta = 1.0f - fDelta;

		xRGBA = ( rxSrcA.xRGBA * fOneMinusDelta ) + ( rxSrcB.xRGBA * fDelta );
#endif
//#endif
	}

	// Uncompress from flexible format
#pragma inline
	void Uncompress( const unsigned char *pucVertexStart, unsigned short usFormat )
	{
		uint32 uRGBA = Asura_PSP_VertexFormat::GetColourFromVertex( pucVertexStart, usFormat );
#if defined ASURA_PSP_RENDERENVIRONMENT_QUADCLIPPING_VFPU && defined ASURA_PSP_ASM_USE_SN_SYNTAX_NEW
		const SNVectorType& rxVFPURGBA = reinterpret_cast< const SNVectorType& >( xRGBA );
		__asm {
			lv.s		s000, 0( &uRGBA )
			vuc2ifs.s	c000, s000
			vi2f.q		rxVFPURGBA, c000, 23
		}
#elif defined ASURA_PSP_RENDERENVIRONMENT_QUADCLIPPING_VFPU && defined ASURA_PSP_USE_VFPU_ASSEMBLER
		__asm__ (
			".set		push\n"							// Save assembler option
			".set		noreorder\n"					// Suppress reordering
			"lv.s		s000, 0 + %1\n"
			"vuc2ifs.s	c000, s000\n"
			"vi2f.q		c000, c000, 23\n"
			"sv.q		c000, 0 + %0\n"
			".set		pop\n"							// Restore assembler option
			: "=m"( xRGBA )
			: "m"( uRGBA )
		);
#else
		xRGBA.SetX( uRGBA & 0xFF );
		xRGBA.SetY( ( uRGBA >> 8 ) & 0xFF );
		xRGBA.SetZ( ( uRGBA >> 16 ) & 0xFF );
		xRGBA.SetW( ( uRGBA >> 24 ) & 0xFF );
#endif
		Asura_PSP_VertexFormat::ShortVector2 vTexCoord = Asura_PSP_VertexFormat::GetTexCoordsFromVertex( pucVertexStart, usFormat );

		Asura_PSP_VertexFormat::ShortVector3 vNormal = Asura_PSP_VertexFormat::GetNormalFromVertex( pucVertexStart, usFormat );
		xXYZU.SetX( vNormal.x );
		xXYZU.SetY( vNormal.y );
		xXYZU.SetZ( vNormal.z );

		xXYZU.SetW( vTexCoord.x );

		Asura_Vector_3 vPosition = Asura_PSP_VertexFormat::GetPositionFromVertex( pucVertexStart, usFormat );
		xXYZV.SetX( vPosition.x );
		xXYZV.SetY( vPosition.y );
		xXYZV.SetZ( vPosition.z );

		xXYZV.SetW( vTexCoord.y );
	}

	// Compress to flexible format
#pragma inline
	void Compress( Asura_PSP_RenderEnvironment_VertexClip *pxClippedPackedVertex )
	{
#if defined ASURA_PSP_RENDERENVIRONMENT_QUADCLIPPING_VFPU && defined ASURA_PSP_ASM_USE_SN_SYNTAX_NEW
		const SNVectorType& rxVFPURGBA = reinterpret_cast< const SNVectorType& >( xRGBA );
		__asm {
			vf2iz.q		c000, rxVFPURGBA, 23
			vi2uc.q		s000, c000
			sv.s		s000, 0( &pxClippedPackedVertex->m_iRGBA8888 )
		}
#elif defined ASURA_PSP_RENDERENVIRONMENT_QUADCLIPPING_VFPU && defined ASURA_PSP_USE_VFPU_ASSEMBLER
		__asm__ (
			".set		push\n"							// Save assembler option
			".set		noreorder\n"					// Suppress reordering
			"lv.q		c000, 0 + %1\n"
			"vf2iz.q	c000, c000, 23\n"
			"vi2uc.q	s000, c000\n"
			"sv.s		s000, 0 + %0\n"
			".set		pop\n"							// Restore assembler option
			: "=m"( pxClippedPackedVertex->m_iRGBA8888 )
			: "m"( xRGBA )
		);
#else
		pxClippedPackedVertex->m_iRGBA8888 = 
			static_cast< uint32 >( xRGBA.GetX() ) |
			( static_cast< uint32 >( xRGBA.GetY() ) << 8 ) |
			( static_cast< uint32 >( xRGBA.GetZ() ) << 16 ) |
			( static_cast< uint32 >( xRGBA.GetW() ) << 24 );
#endif
		pxClippedPackedVertex->m_asNormal[ 0 ] = xXYZU.GetX();
		pxClippedPackedVertex->m_asNormal[ 1 ] = xXYZU.GetY();
		pxClippedPackedVertex->m_asNormal[ 2 ] = xXYZU.GetZ();

		pxClippedPackedVertex->m_usU = xXYZU.GetW();

		pxClippedPackedVertex->m_afPosition[ 0 ] = xXYZV.GetX();
		pxClippedPackedVertex->m_afPosition[ 1 ] = xXYZV.GetY();
		pxClippedPackedVertex->m_afPosition[ 2 ] = xXYZV.GetZ();

		pxClippedPackedVertex->m_usV = xXYZV.GetW();
	}
};
#endif	// ASURA_PSP_RENDERENVIRONMENT_QUADCLIPPING

struct Asura_PSP_Render_ModuleLightingCache_Photon
{
	Asura_PSP_Vector3	m_xPhotonPosition;
	u_int				m_uPhotonColour;
	float				m_fOneOverRange;
};

class Asura_PSP_Render_ModuleLightingCache
{
	public:
	Asura_PSP_Render_ModuleLightingCache_Photon m_axPhoton[4];
};
class Asura_PSP_RenderEnvironment_Strip
{
	public:
		u_short m_uNumberOfIndices;
		u_short m_usVertexFormat;
		u_int	m_uStartIndex;
		u_int	m_uStartVertex;
		int		m_iMaterialIndex;
#ifdef PSP_NEW_ENV_RENDER
		u_int	m_uModule;
#endif
		Asura_Bounding_Box m_xBB;

		u_int	m_uNumberOfStrips; 
		u_int*	m_auStartIndex; 
		Asura_Bounding_Box* m_axBB;

		Asura_PSP_RenderEnvironment_Strip()
		{
			m_auStartIndex = NULL;
			m_axBB = NULL; 
		}

		~Asura_PSP_RenderEnvironment_Strip()
		{
			Asura_ResourcePushType(ASURA_RESOURCE_ENVIRONMENT_MESH);
			{
				Asura_ResourceDeallocateArray(m_auStartIndex);
				Asura_ResourceDeallocateArray(m_axBB);
			}
			Asura_ResourcePopType();
		}
};

class Asura_PSP_RenderEnvironment_Module
{
	public:
		
		u_int m_uNumberOfStrips;
		u_int m_uStartStripIndex;
};

class Asura_PSP_Plane;
class Asura_PSP_FrustumLimits;

enum ASURA_PSP_ENV_STRIP_TYPE
{
	ASURA_PSP_ENV_STRIP_FIRST_PASS,
	ASURA_PSP_ENV_STRIP_SECOND_PASS,
	ASURA_PSP_ENV_STRIP_REFLECTED,
	ASURA_PSP_ENV_STRIP_WATER,
};

union Asura_Frustum_Test_Result_Bitfield;

class Asura_PSP_RenderEnvironment_Zone
{
	friend class Asura_Chunk_ResourceFile_PSP_Environment;

public:
	Asura_PSP_RenderEnvironment_Zone  (void);
	~Asura_PSP_RenderEnvironment_Zone ();

	// (SB - 23/02/07) Render state override function typedef
	typedef void (RenderStateOverride)(Asura_PSP_RenderEnvironment_Zone& xZone, Asura_PSP_RenderEnvironment_Strip& xStrip);
	// (SB - 27/04/07) Early out hook typedef
	typedef bool (EarlyOutCallback)(Asura_PSP_RenderEnvironment_Zone& xZone, Asura_PSP_RenderEnvironment_Strip& xStrip);

	// Reset function...
	void	Reset		(void);

	// Render functions...
	void	Render		(void);
	void	RenderStrip	(u_int uStripIndex, ASURA_PSP_ENV_STRIP_TYPE eType);
	
	void	DrawStripComplete(Asura_PSP_RenderEnvironment_Strip& xStrip, const size_t xVertexSize); 
	void	DrawInternalStripComplete(Asura_PSP_RenderEnvironment_Strip& xStrip, u_int uInternalStripIndex, const size_t xVertexSize); 
	void	DrawInternalStripPartial(Asura_PSP_RenderEnvironment_Strip& xStrip, u_int uInternalStripIndex, u_int uFirst, u_int uLast, const size_t xVertexSize);

	void inline DrawArray(unsigned int uPrimType, unsigned int uVertexFormat, unsigned int uVertices, void* pIndices, void* pBuffer); 

	// Clipping functions...
	void	SetupFrusta							(void);
	void	SetModuleClipFlags					(bool bReflected);
	void	CalculateNumberOfStrips				(Asura_PSP_RenderEnvironment_Strip& xStrip); 
#ifdef ASURA_PSP_RENDERENVIRONMENT_QUADCLIPPING
	bool	DegeneratePlaneClip					( const Asura_PSP_RenderEnvironment_VertexClipQuad *axSrcVertices, Asura_PSP_RenderEnvironment_VertexClipQuad *axDstVertices, uint32& ruNumberOfVertices, const Asura_PSP_Plane& rxPlane ) const;
#endif
	void	ClipStrip							(Asura_PSP_RenderEnvironment_Strip& xStrip, const bool bSecondPass);
	void	ClipStripSplitThroughDegenerates	(Asura_PSP_RenderEnvironment_Strip& xStrip, const bool bReflected, const bool bSecondPass, const size_t& xVertexSize, const size_t& xVertexPosOffset);
	u_int	ClipPolyToPlane						(const Asura_PSP_RenderEnvironment_VertexClip* pxInVertices, u_int& uNumberOfVertices, Asura_PSP_RenderEnvironment_VertexClip* pxOutVertices, const Asura_PSP_Plane& xPlane);

	const Asura_PSP_FrustumLimits& GetFrustumLimits() const { return m_xFrustum; }
	const Asura_PSP_FrustumLimits& GetGuardbandLimits() const { return m_xGuardband; }

	// (SB - 23/02/07) Render state override setter
	static void SetRenderStateOverride(RenderStateOverride* const pfnRenderStateOverride = NULL) { s_pfnRenderStateOverride = pfnRenderStateOverride; };
	static void SetEarlyOutCallback(EarlyOutCallback* const pfnEarlyOutCallback = NULL) { s_pfnEarlyOutCallback = pfnEarlyOutCallback; };

	// Clipping variables...
	u_int									m_uNumModuleTests;
	u_int									m_uNumModuleAccepts;
	u_int									m_uNumStripTests;
	u_int									m_uNumStripAccepts;
	u_int									m_uNumStripBehinds;
	u_int									m_uNumTriangleTests;
	u_int									m_uNumTriangleAccepts;
	u_int									m_uMaxStripLength;

	// (SB - 23/01/07) For variable vertex format - to calculate the pointer to a specified vertex
	inline unsigned char* const GetVertexPtr(const Asura_PSP_RenderEnvironment_Strip& xStrip, const unsigned int uIndex) const
	{
		const unsigned int uStripIndex = &xStrip - m_pxStrips; // Nasty way of getting the strip index by pointer arithmetic
		const bool bSecondPass = &xStrip - m_pxStrips >= m_uTotalNumberOfStrips;

		//return m_pucVertexBuffer + (xStrip.m_uStartVertex + uIndex) * Asura_PSP_VertexFormat::CalculateVertexSize(bSecondPass ? m_usSecondPassFormat : m_usFirstPassFormat);
		return m_pucVertexBuffer + (xStrip.m_uStartIndex + uIndex) * (bSecondPass ? m_xSecondPassVertexSize : m_xFirstPassVertexSize);
	}

private:
	// Private variables...

	// Current environment mesh...
			u_int									m_uTotalNumberOfVertexBuffers;
			u_int									m_uTotalNumberOfStrips;
			u_int									m_uTotalNumberOfDualPassStrips;
			u_int									m_uNumberOfModules;
			Asura_PSP_RenderEnvironment_Module*		m_pxModules;
			Asura_PSP_RenderEnvironment_Strip*		m_pxStrips;
			unsigned char*							m_pucVertexBuffer;
			u_short*								m_pusIndexBuffer;

			unsigned short							m_usFirstPassFormat;
			unsigned short							m_xFirstPassVertexSize;
			unsigned short							m_xFirstPassVertexPosOffset;
			unsigned short							m_usSecondPassFormat;
			unsigned short							m_xSecondPassVertexSize;
			unsigned short							m_xSecondPassVertexPosOffset;

	// Per frame variables...
			Asura_PSP_Render_ModuleLightingCache*	m_pxModuleLightingCache;
			float									m_fCurrentWaterfallPosition;
			bool									m_bAnyWaterThisFrame;
			int										m_eRenderMode;
			u_int									m_uVertexFormat;
			u_int*									m_puModuleClipFlags;
			u_int*									m_puReflectedModuleClipFlags;
			Asura_PSP_FrustumLimits					m_xFrustum;
			Asura_PSP_FrustumLimits					m_xGuardband;
#ifdef ASURA_PSP_RENDERENVIRONMENT_QUADCLIPPING
			Asura_PSP_FrustumLimits					m_xClipFrustum;
			Asura_AMatrix4x4						m_xAProjMat;
#endif
			Asura_PSP_Matrix4						m_xProjMat;
	static	int										s_iDepthBufferOffset;

	// (SB - 23/02/07) Render state override hook
	static RenderStateOverride* s_pfnRenderStateOverride;
	static EarlyOutCallback* s_pfnEarlyOutCallback;
};


class Asura_PSP_RenderEnvironment
{
public:
	// Public functions...
	static	void								Initialise						(void);
	static	void								Reset							(void);
	static	void								Shutdown						(void);

	static	void								Render							(void);

#if defined(ASURA_USE_STREAMING)					
	// Zone functions...
	static	Asura_PSP_RenderEnvironment_Zone*	CreateZone						(const Asura_Hash_ID uHash);
	static	bool								DestroyZone						(const Asura_Hash_ID uHash);

	static	void								CreateAndSetZone				(const Asura_Hash_ID uHash)					{ s_pxCurrentZone = CreateZone(uHash); }
	static	void								SetCurrentZone					(Asura_PSP_RenderEnvironment_Zone* pxZone)	{ s_pxCurrentZone = pxZone; }
	static	void								SetCurrentZone					(const Asura_Hash_ID uHash)					{ s_pxCurrentZone = s_xZoneTree.GetDataByHash(uHash); }
	static	Asura_PSP_RenderEnvironment_Zone&	GetCurrentZone					(void)										{ return(*s_pxCurrentZone); }
	static	Asura_PSP_RenderEnvironment_Zone*	GetZone							(const Asura_Hash_ID uHash)					{ return(s_xZoneTree.GetDataByHash(uHash)); }
#else
	// Zone functions...
	static	Asura_PSP_RenderEnvironment_Zone&	GetCurrentZone					(void)										{ return(s_xZone); }
#endif	// defined(ASURA_USE_STREAMING)

	// Project specific functions
	static 	inline bool							Project_IsStripReflected		(const unsigned int uMaterialProjectFlags); 
	static 	inline bool							Project_IsStripReflectedOnly	(const unsigned int uMaterialProjectFlags); 


private:
#if defined(ASURA_USE_STREAMING)
	// Private variables...
	static	Asura_PSP_RenderEnvironment_Zone*							s_pxCurrentZone;		// Currently active handler.
	static	Asura_TemplateHashTree<Asura_PSP_RenderEnvironment_Zone>	s_xZoneTree;			// Tree of all of the handlers available.
#else
	// Private variables...
	static	Asura_PSP_RenderEnvironment_Zone							s_xZone;				// Single static-instance of a zone.
#endif	// defined(ASURA_USE_STREAMING)
};

#endif//__ASURA_PSP_RENDERENVIRONMENT_H__


/// End of file. ///
