//
// Asura_PSP_Atrac.h
//
// Created by Kim Randell - 07/10/2004
// 
// Copyright (C) 2004 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef __ASURA_PSP_ATRAC_H__
#define __ASURA_PSP_ATRAC_H__

#if !defined( ASURA_PSP_MAX_NUM_ATRAC_STREAMS )
#define ASURA_PSP_MAX_NUM_ATRAC_STREAMS 1
#endif // #if !defined( ASURA_PSP_MAX_NUM_ATRAC_STREAMS )

#if ( ( ASURA_PSP_MAX_NUM_ATRAC_STREAMS < 0 ) || ( ASURA_PSP_MAX_NUM_ATRAC_STREAMS > 2 ) )
#error ASURA_PSP_MAX_NUM_ATRAC_STREAMS must be 1 or 2
#endif // #if ( ( ASURA_PSP_MAX_NUM_ATRAC_STREAMS < 0 ) || ( ASURA_PSP_MAX_NUM_ATRAC_STREAMS > 2 ) )

// PSP headers
#include <kerneltypes.h>
#include <psptypes.h>

#include "Sound/Asura_Sound.h" // for Asura_Sound_System::SoundGroup

// KOR - 28/02/2006
// **************** WARNING! ************************
// The atrac code here is not yet thread safe.
// This means there is the possibility of a hang.
// It is *strongly* advised that a semaphore is added
// before shipping a game using this class.
// **************************************************
class Asura_PSP_Atrac
{
public:
	static void Initialise( void );
	static void Reset( void );
	static void Shutdown( void );

	// Needed for pressing "Home" and using Sony's utilities...
	static void Suspend( void );
	static void Resume( void );

	// set to true to pause all tracks, call again with false to resume all tracks
	static void PauseAll( const bool bPauseOn );

	// set to true to pause all tracks, call again with false to resume all tracks
	static void StopAll( const bool bFadeOut = false );

	// Loads the given file and creates the thread to play it, but leaves it paused. To actually play, call Play()
	// iDataSize = -1 means use the file size
	static bool Create( const Asura_Handle xHandle, const char* const szFilename, const u_int uStartOffset, const int iDataSize);

	// Plays a track if it has been created.
	static void Play( const Asura_Handle xHandle, const bool bLooping = true );

	// Stops track dead.
	static void Stop( const Asura_Handle xHandle )	{ StopTrack( xHandle ); }

	static void SetSoundGroup( const Asura_Handle xHandle);

	// set to true to pause track, call again with false to resume track
	static void PauseTrack ( const Asura_Handle xHandle, const bool bPauseOn );

	// Stop current track (if there is one)
	// If bFadeOut is false, track stops dead
	// Otherwise track fades out asynchronously 
	// KOR - 12/05/2006 - keeping only as legacy support. Not sure if fadeout code is safe;
	// thread needs to be released after fadeout has completed.
	static void StopTrack( const Asura_Handle iHandle, const bool bFadeOut = false );

	// Stops track if necessary, then kills thread.
	static void Release( const Asura_Handle xHandle );

	// returns true if inside playing loop (i.e. after track loaded)
	// ignores pause status (i.e. designed to tell if play thread is active)
	static bool IsPlaying( const Asura_Handle xHandle );

	// returns internal state caused by PlayTrack/PauseTrack calls
	static bool IsPaused( const Asura_Handle xHandle );

	// Gets the length of the current track in seconds
	static float GetLength( const Asura_Handle xHandle );

	static void SetVolume( const Asura_Handle xHandle, const float fVolume );

	static void SetMasterVolume( const float fVolume ) { s_fMasterVolume = fVolume; }

	static bool Decode( const int iStreamID, short* psCurrentBuffer, short* psNextBuffer, float& fVolume );

	// Gets the play position of the current track in seconds
	static float GetPosition( const Asura_Handle xHandle );

private:
	// Internal functions
	// Play the given track 
	// (automatically stops any currently playing track, and sets pause to false)
	static bool PlayTrack( const Asura_Handle xhandle, const char* const szFilename );

	// returns true iff file is trying to get started but hasn't started playing yet.
	static bool IsPreloading( const Asura_Handle xHandle );

	static bool IsInitialised( void ) { return s_bInitialised; }

	// thread to do actual loading/streaming and playing of track
	static int PlayThread( SceSize args, ScePVoid argp );

	static bool		s_bInitialised;
	static float	s_fMasterVolume;

	// Internal functions...
	static int UpdateThread( SceSize args, void* argp );
	// iDataSize = -1 means use the file size
	static bool	OpenFile( const int iStreamID, const char* const szFilename, const u_int uStartOffset, const int iDataSize );
	static bool CloseFile( const int iStreamID );

	static void Initialise( const int iStreamID );
	static void Suspend( const int iStreamID );
	static void Resume( const int iStreamID );
	static void Reset( const int iStreamID );
	static int FindStreamID( const Asura_Handle xHandle );
	static int FindFreeStreamID( const Asura_Handle xHandle );

	// Internal helper functions - stops a streamID
	static void StopStream( const int iStreamID, const bool bFadeOut );
	static void ReleaseStream( const int iStreamID );
};

#endif//__ASURA_PSP_ATRAC_H__
