//
// Asura_PlatformHierarchy.cpp
//
// Created by KOR - 09 October 2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

// This file's header.
#include "Animation/Asura_Hierarchy.h"

// Asura
#include "Animation/Asura_Animation.h"
#include "Camera/Asura_Camera.h"
#include "Core/Asura_Material_System.h"
#include "Rendering/Asura_Render.h"

// Asura Wii
#include "Rendering/Asura_GX_Hierarchy.h"
#include "Rendering/Asura_GX_Shadows.h"
#include "Rendering/Asura_GX_SmoothSkin.h"

void Asura_Hierarchy_System::Platform_Initialise() {};
void Asura_Hierarchy_System::Platform_Reset() {};
void Asura_Hierarchy_System::Platform_Shutdown() {};

void RenderSimpleShadow(u_int uCamVisFlags, float fAlphaScale = 1.f);

DEBUGVAR float dbg_fNearShadowLODDistance = 10.f;
DEBUGVAR float dbg_fFarShadowLODDistance = 20.f;
DEBUGVAR float dbg_fMinShadowComponentLength = 0.2f;
DEBUGVAR bool dbg_bDebugShadows = false;
DEBUGVAR float dbg_fComplexShadowAlpha = 0.375f;
DEBUGVAR float dbg_fSimpleShadowAlpha = 0.5f;
DEBUGVAR float dbg_fSimpleShadowSize = 0.6f;
DEBUGVAR bool dbg_bDisableAnimations = false;

void RenderGXShadow(unsigned int uCamVisFlags, float fAlphaScale = 1.f)
{
	if(Asura_Camera::GetNumberOfCameras() > 1)
	{
		// Do a simple shadow instead
		RenderSimpleShadow(uCamVisFlags, fAlphaScale);
	}
	else
	{
		Asura_Camera::SetCameraIndex(0);
		const Asura_Vector_3& xPos = Asura_Hierarchy_System::GetWorldspaceBonePos(0);
		const float fDistance = (xPos - *Asura_Camera::GetPosition()).Magnitude();
		if(fDistance < dbg_fFarShadowLODDistance)
		{
			float fAlpha = dbg_fComplexShadowAlpha * fAlphaScale;
			if(fDistance > dbg_fNearShadowLODDistance)
			{
				const float fFade = (fDistance - dbg_fNearShadowLODDistance) / (dbg_fFarShadowLODDistance - dbg_fNearShadowLODDistance);
				// Fade in the simple shadow
				RenderSimpleShadow(uCamVisFlags, fFade);
				fAlpha *= (1.f - fFade);
			}
			Asura_GX_ShadowRenderer::BeginRender();
			const Asura_Hierarchy_Skin* pxSkin = Asura_Hierarchy_System::s_pxSkin;
			for (u_int u = 1; u < pxSkin->NumberOfBones; ++u)
			{
				const u_int uParentBone = pxSkin->ParentBone[u];
				const Asura_Hierarchy_SectionBoundingBox* pxSectionBBData = &pxSkin->m_pxSectionBoundingBoxData[uParentBone];
				if(pxSectionBBData->m_uValid)
				{
					const Asura_Vector_3 xBonePos1 = Asura_Hierarchy_System::GetWorldspaceBonePos(u);
					const Asura_Vector_3 xBonePos2 = Asura_Hierarchy_System::GetWorldspaceBonePos(uParentBone);
					const float fBoneLengthSq = (xBonePos1 - xBonePos2).MagnitudeSquared();
					const bool bLongEnough = (fBoneLengthSq > dbg_fMinShadowComponentLength * dbg_fMinShadowComponentLength);
					if(dbg_bDebugShadows)
					{
						Asura_Render::RenderLine(xBonePos1, xBonePos2, bLongEnough ? 0xff00ff00 : 0xffff0000);
					}
					if(bLongEnough)
					{
						const Asura_Bounding_Box* pxBox = &pxSectionBBData->m_xBB;
						// Arbitrarily pick z axis for thickness
						const float fSize = pxBox->MaxZ - pxBox->MinZ;
						Asura_Assert(fSize > 0.f && fSize < 100.f, "");
						Asura_GX_ShadowRenderer::AddShadow(xBonePos1, xBonePos2, fSize, fAlpha);
					}
				}
			}
			Asura_GX_ShadowRenderer::EndRender(uCamVisFlags, false);
		}
		else
		{
			RenderSimpleShadow(uCamVisFlags, fAlphaScale);
		}
	}
}

void RenderSimpleShadow(u_int uCamVisFlags, float fAlphaScale /* = 1.f */)
{
	Asura_GX_ShadowRenderer::BeginRender();
	const Asura_Vector_3 xBonePos = Asura_Hierarchy_System::GetWorldspaceBonePos(0);
	Asura_GX_ShadowRenderer::AddShadow(xBonePos, xBonePos, dbg_fSimpleShadowSize, dbg_fSimpleShadowAlpha * fAlphaScale);
	const Asura_Vector_3& xRenderPos = Asura_Hierarchy_System::s_bBonesInWorldspace ? xBonePos : *Asura_Hierarchy_System::s_pxPos;
	Asura_GX_ShadowRenderer::EndRender(uCamVisFlags, true, &xRenderPos);
}

void Asura_Hierarchy_System::Platform_RenderSkin( const Asura_Animation* pxAnim, 
												 float fAlpha, int iTexIDOverride,
												 u_int uCamVisFlags, u_int uRenderFlags,
												const Asura_Vector_3& xBSCentre, float fBSRadius )
{ 
	if(dbg_bDisableAnimations) { return; }

	if(s_bJustRenderForCurCam)
	{
		uCamVisFlags = (1 << Asura_Camera::GetCameraIndex());
	}
	else
	{
		if(!(uRenderFlags & ASURA_HIERARCHY_RENDER_NO_SHADOW))
		{
			RenderGXShadow(uCamVisFlags, fAlpha);
			//RenderSimpleShadow(uCamVisFlags);
		}
	}
	if(!dbg_bDebugShadows)
	{
		Asura_GX_SmoothSkinRenderer::RenderSmoothSkin(s_pxSkin->m_uNameHashID, uCamVisFlags, iTexIDOverride, fAlpha);
	}
}

void Asura_Hierarchy_System::Platform_RenderHierarchy( const Asura_Animation* pxAnim,
													  u_int uCamVisFlags, u_int uRenderFlags,
														const Asura_Vector_3& xBSCentre, float fBSRadius, 
														const int iTexIDOverride /*=ASURA_TEXTURE_NONE*/ )
{
	if(dbg_bDisableAnimations) { return; }

	if(s_bJustRenderForCurCam)
	{
		uCamVisFlags = (1 << Asura_Camera::GetCameraIndex());
	}
	Asura_GX_HierarchyRenderer::RenderHierarchy(s_pxSkin->m_uNameHashID, uCamVisFlags, pxAnim->GetAlpha());
}

static inline int GetFirstMaterialIndexForSkin( const Asura_Hierarchy_Skin* pxSkin )
{
	if(pxSkin->IsSkinnedModel())
	{
		return Asura_GX_SmoothSkinRenderer::GetFirstMaterialIndex(pxSkin->m_uNameHashID);
	}
	else
	{
		return Asura_GX_HierarchyRenderer::GetMaterialIndex(pxSkin->m_uNameHashID);
	}
}

int Asura_Hierarchy_Skin::Platform_GetTextureIndex() const
{
	const int iMaterialIndex = GetFirstMaterialIndexForSkin(this);
	return Asura_Material_System::GetTextureID(iMaterialIndex);
}

bool Asura_Hierarchy_Skin::Platform_IsOpaque( void ) const
{
	if(IsSkinnedModel())
	{
		return Asura_GX_SmoothSkinRenderer::IsSkinOpaque(m_uNameHashID);
	}
	else
	{
		return Asura_GX_HierarchyRenderer::IsSkinOpaque(m_uNameHashID);
	}
}

void Asura_Hierarchy_System::Platform_RenderSkinShadow( const Asura_Animation& xAnim, const Asura_Shadow& xShadow, const u_int uCamVisFlags, const u_int uRenderFlags )
{
}

void Asura_Hierarchy_System::Platform_RenderHierarchyShadow( const Asura_Animation& xAnim, const Asura_Shadow& xShadow, const u_int uCamVisFlags, const u_int uRenderFlags )
{
}

