//
// Asura_Bink.h
//
// Created by KOR - 09 October 2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __ASURA_BINK_H__
#define __ASURA_BINK_H__

#ifdef ASURA_USE_BINK

#include "Core/Asura_Multithread.h"
#include "Sound/Asura_Sound.h"

#include <bink.h>
#include <revolution/dvd.h>

const u_int uASURA_WII_BINK_MAX_QUEUE_LENGTH = 8;
const u_int uASURA_WII_BINK_OPENTHREAD_STACK_SIZE = 8 * 1024;

class Asura_Wii_Bink_Open_Task
{
public:
	Asura_Wii_Bink_Open_Task()
	: m_pxBinkHandle(NULL)
	, m_bCloseOnComplete(false)
	, m_bTaskComplete(false)
	{}

	HBINK m_pxBinkHandle;
	DVDFileInfo m_xFileInfo;
	Asura_Mutex m_xMutex;
	bool m_bCloseOnComplete;
	bool m_bTaskComplete;
};

class Asura_Wii_Bink
{
public:
	struct Movie_Render_Params
	{
		bool bWideScreen;
		// from bink
		float fSrcWidth;
		float fSrcHeight;
		// Pass 0 is a nearest pixel pass due to the decompression of the bin textures
		// Pass 1 is a full linearly interpolated pass so gives better results
		struct PassInfo
		{
			float fX, fY;
			float fDstWidth, fDstHeight;
		} Pass[2];
	};

	typedef void (*CBRenderChange)( Asura_Wii_Bink::Movie_Render_Params& pxMovieRenderParams );
	typedef void (*RENDER_SUBTITLE_CB_FN)(float fTime);


    Asura_Wii_Bink()
	: m_pxBinkHandle(NULL)
	, m_fVolume(0.f)
	, m_fLength(0.f)
	, m_eSoundGroup(Asura_Sound_System::SoundGroup_SoundFX)
	, m_uFlags(0)
	, m_pxOpenTask(NULL)
    {
    }

    void Update();
    bool Create(const char* szFilename);
	bool CreateFromResource(const char* szResourceFilename, int iOffset);
	// Size is ignored for Bink, but we are maintaining compatibility
	// with streamed ADPCM interface for simplicity.
	bool CreateFromResource(int iDVDEntryNum, int iOffset, int iSize = -1);
    void Play(bool bLooping);
    void Stop();
    void Release();
    void Pause();
    void Resume();
    void SetVolume(float fVol);
	void UpdateVolume();
	float GetLength();
	float GetPosition();
	bool IsFinished();
	void CalculateLength();
	void SetSoundGroup(Asura_Sound_System::SoundGroup eGroup) { m_eSoundGroup = eGroup; }
	
	static void Initialise();
	static bool s_bInitialised;
	static int PlayFMV(const char* szFilename, unsigned int uTrackNo = 0, bool bSkipAllowed = true, CBRenderChange pCB = NULL, RENDER_SUBTITLE_CB_FN pfnRenderSubtitleCB = NULL);

	static void* OpenThreadFunc(void* pvBink);
private:
	HBINK m_pxBinkHandle;
	DVDFileInfo m_xFileInfo;
	float m_fVolume;
	float m_fLength;
	Asura_Sound_System::SoundGroup m_eSoundGroup;
	Movie_Render_Params m_xParams;
	union
	{
		u_int m_uFlags;
		struct
		{
			u_int m_bLooping		:1;
			u_int m_bFinished		:1;
			u_int m_bPendingPlay	:1;
		};
	};
	Asura_Wii_Bink_Open_Task* m_pxOpenTask;

	// Asynchronous loading queue
	static Asura_Wii_Bink_Open_Task s_axOpenQueue[uASURA_WII_BINK_MAX_QUEUE_LENGTH];
	static u_int s_uQueueHead;
	static u_int s_uQueueTail;
	static OSThread s_xOpenThread;
	static char s_acOpenThreadStack[uASURA_WII_BINK_OPENTHREAD_STACK_SIZE];
	static Asura_Mutex s_xQueueMutex;
};

#endif//def ASURA_USE_BINK

#endif//__ASURA_BINK_H__