//
// Asura_Wii_MEM2.h
//
// Created by KOR - 09 February 2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __ASURA_WII_MEM2_H__
#define __ASURA_WII_MEM2_H__

#include "Core/Asura_MemoryManagement_Enums.h"
#include "Core/Asura_HashTree.h"

const int iASURA_MEM2_STATE_UNINITIALISED = -1;

class Asura_MEM2_State
{
public:
	Asura_MEM2_State() 
	: m_iTotalMemoryCurrentlyAlloced(iASURA_MEM2_STATE_UNINITIALISED)
	, m_iNumberOfAllocations(iASURA_MEM2_STATE_UNINITIALISED)
	{
	}

	int m_iTotalMemoryCurrentlyAlloced;
	int m_iNumberOfAllocations;
};

class Asura_MEM2_Node
{
public:
	Asura_MEM2_Node()
	: m_szFile(NULL)
	, m_uLine(0)
	, m_uSize(0)
	, m_uFrame(0)
	{}

	const char* m_szFile;
	u_int m_uLine;
	u_int m_uSize;
	u_int m_uFrame;
};

#ifdef ASURA_MEMORYMANAGEMENT_ENABLED
#define ASURA_WII_MEM2_ALLOC(__uSize)	Asura_Wii_MEM2::Allocate(__uSize, __FILE__, __LINE__)
#define ASURA_WII_MEM2_ALLOC_SILENTFAIL(__uSize)	Asura_Wii_MEM2::Allocate(__uSize, __FILE__, __LINE__, false)
#else
#define ASURA_WII_MEM2_ALLOC(__uSize)	Asura_Wii_MEM2::Allocate(__uSize)
#define ASURA_WII_MEM2_ALLOC_SILENTFAIL(__uSize)	Asura_Wii_MEM2::Allocate(__uSize, false)
#endif

#define ASURA_WII_MEM2_ALLOC_ARRAY(__xType, __uNumElements) static_cast<__xType*>(ASURA_WII_MEM2_ALLOC(sizeof(__xType) * __uNumElements))
#define ASURA_WII_MEM2_FREE(__pvData)	Asura_Wii_MEM2::Free(__pvData);

class Asura_Wii_MEM2
{
public:
	static void Initialise();
	static void Shutdown();

#ifdef ASURA_MEMORYMANAGEMENT_ENABLED
	static void DumpMallocInfo(const char* szFilenameNoPath = NULL, const char* szPath = NULL);
	static void DumpMallocInfoLine(Asura_Hash_ID uHash, const Asura_MEM2_Node* pxNode);

	static void* Allocate(size_t uSize, const char* szFile, u_int uLine, bool bOutOfMemoryIsFatal = true);
#else
	static void* Allocate(size_t uSize, bool bOutOfMemoryIsFatal = true);
#endif
	static void Free(void* pvData);

	static u_int GetNumAllocations() { return s_uNumAllocations; }
	static u_int GetAllocatedBytes();
	static u_int GetFreeBytes();

	static void Checkpoint(int iState);
private:
	static bool s_bInitialised;
	static u_int s_uNumAllocations;

	static Asura_MEM2_State m_axPreviousStates[ASURA_MM_STATE_NUM_TYPES];
#ifdef ASURA_MEMORYMANAGEMENT_ENABLED
	static Asura_TemplateHashTree<Asura_MEM2_Node> s_xAllocationNodeTree;
#endif
};

#endif//__ASURA_WII_MEM2_H__