//
// Asura_Platform_Logfile.cpp
//
// Created by KOR - 09 October 2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

// This file's header.
#include "Debugging/Asura_Logfile.h"

// Asura headers
#include "Core/Asura_PlatformBasics.h"
#include "Core/Asura_MultiThread.h"

#include <stdio.h>

static const int OUTPUT_BUFFER_SIZE = 1024;

static Asura_Mutex			g_xOSReportMutex;

static bool Asura_LogfileClear_Internal(const Asura_Char* szFilename)
{
	FILE *pxFileHandle;
	pxFileHandle = fopen(szFilename, "w");
	if(!pxFileHandle)
	{
		return false;
	}
	fclose(pxFileHandle);
	return true;
}

static bool Asura_LogfilePrint_Internal(const Asura_Char* szFilename, const Asura_Char* szOut)
{
	FILE *pxFileHandle;
	pxFileHandle = fopen(szFilename, "a");
	if(!pxFileHandle)
	{
		return false;
	}
	fprintf(pxFileHandle, szOut);
	fclose(pxFileHandle);
	return true;
}

void __Asura_LogfileDebugData(const Asura_Char* file, unsigned int line)
{
}

void __Asura_LogfileOutput(const Asura_Char* szFilename, const Asura_Char* string, ...)
{
	int iNumChars;
	char szBuffer[OUTPUT_BUFFER_SIZE];
	{
		va_list ap;
		va_start( ap, string );
		iNumChars = vsnprintf( szBuffer, OUTPUT_BUFFER_SIZE - 1, string, ap );
		va_end( ap );

        if( -1 == iNumChars )
        {
            szBuffer[OUTPUT_BUFFER_SIZE - 1] = '\0';
        }
	}
    // Output.
	Asura_LogfilePrint_Internal(szFilename, szBuffer);
}

// This version of the log clear function should not make use of any globals
// with constructors or any functions that use some. In particular, this
// function can't assert or use the OpenedLogfiles tree to check if the file's
// been used before or not.
void __Asura_SimpleLogfileClear(const Asura_Char* file)
{
    Asura_LogfileClear_Internal( file );
}

// This version of the logging function should not make use of any globals
// with constructors or any functions that use some. In particular, this
// function can't assert or use the OpenedLogfiles tree to check if the file's
// been used before or not.
void __Asura_SimpleLogfileOutput(const Asura_Char* szFilename, const Asura_Char* string, ...)
{
	int iNumChars;
	char szBuffer[OUTPUT_BUFFER_SIZE];
	{
		va_list ap;
		va_start( ap, string );
		iNumChars = vsnprintf( szBuffer, OUTPUT_BUFFER_SIZE - 1, string, ap );
		va_end( ap );

        if( -1 == iNumChars )
        {
            szBuffer[OUTPUT_BUFFER_SIZE - 1] = '\0';
        }
	}
    // Output.
	Asura_LogfilePrint_Internal(szFilename, szBuffer);
}

void __Asura_OutputToDebugger( const Asura_Char* __szFormat, ... )
{
	char buffer[OUTPUT_BUFFER_SIZE];

	/// Kevin Lea - 30/10/2001 16:55:50 - process the format string and any other parameters
	/// into a buffer, just like printf does.
    // (MP - 25.04.2002) Truncate it if necessary.
	{
		va_list ap;
		va_start( ap, __szFormat );
		int iNumChars = vsnprintf( buffer, OUTPUT_BUFFER_SIZE - 1, __szFormat, ap );
		va_end( ap );

        if( -1 == iNumChars )
        {
            buffer[OUTPUT_BUFFER_SIZE - 1] = '\0';
        }
	}

    // Output.
	// KOR - 29/09/2006 - OSReport is not threadsafe.
	g_xOSReportMutex.Lock();
    OSReport( buffer );
	g_xOSReportMutex.Unlock();
}


bool __Asura_Logfile_CanLogYet( void )
{
	return true;	
}

void __Asura_SimpleLogfileOutput_PreFormatted( const char* szLogFilename, const char* szStr, u_int uLen )
{
    // Output.
	Asura_LogfilePrint_Internal(szLogFilename, szStr);
//	OSReport(szStr);
}


bool __Asura_OverrideLogfilesDir( const char* szDir, bool bArchiveExisting )
{
	return false;
}
