//
// Asura_Platform_ScreenShot_System.cpp
//
// Created by KOR - 09 October 2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

#ifdef ASURA_USE_SCREENSHOT_SYSTEM

// This file's header...
#include "Debugging\Asura_ScreenShot_System.h"

// Asura header-files...
#include "Core\Asura_Stream.h"
#include "Rendering\Asura_Render.h"
#include "Rendering\Asura_GX.h"
#include <revolution/gx.h>
#include "Core/Asura_Wii_MEM2.h"
////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_ScreenShot_System::Platform_RenderProgressBar
// Author       : Andy Tate
// Date         : 16-Jun-2005
//
// Description  : 
//
// Parameters   : References to the position and size of the progress-bar and
//				  the colour with which it's to be drawn.
//
// Return value : None.
////////////////////////////////////////////////////////////////////////////////

void Asura_ScreenShot_System::Platform_RenderProgressBar (const Asura_Vector_2& xPosition,
														  const Asura_Vector_2& xSize,
														  const u_int uARGB)
{
#if 0
	struct
	{
		u_int	m_uDiffuse;
		short	m_asPosition [3];

	} axVertices [2];

	char acDisplayList [1024];


	// Set the render-states...

	Asura_Render::CurrentStates.RequestCullMode				(ASURA_CULLMODE_NONE);
	Asura_Render::CurrentStates.RequestTextureIndex			(0, ASURA_TEXTURE_NONE);
	Asura_Render::CurrentStates.RequestTranslucencyMode		(ASURA_TRANSLUCENCY_OFF);
	Asura_Render::CurrentStates.RequestZBufferEnabled		(false);
	Asura_Render::CurrentStates.RequestZBufferWriteEnabled	(false);


	// Initialise the quad...

	axVertices [0].m_uDiffuse		= uARGB;
	axVertices [0].m_asPosition [0]	= static_cast<short>(xPosition.x);
	axVertices [0].m_asPosition [1]	= static_cast<short>(xPosition.y);
	axVertices [0].m_asPosition [2]	= 32767;

	axVertices [1].m_uDiffuse		= uARGB;
	axVertices [1].m_asPosition [0]	= static_cast<short>(xPosition.x + xSize.x);
	axVertices [1].m_asPosition [1]	= static_cast<short>(xPosition.y + xSize.y);
	axVertices [1].m_asPosition [2]	= 32767;


	// Render the quad...

	sceKernelDcacheWritebackAll	();

	sceGuStart					(SCEGU_IMMEDIATE, acDisplayList, sizeof(acDisplayList));
	sceGuDrawArray				(SCEGU_PRIM_RECTANGLES, SCEGU_COLOR_PF8888 | SCEGU_VERTEX_SHORT | SCEGU_THROUGH, 2, 0, axVertices);
	sceGuFinish					();


	// Restore the render-states...

	Asura_Render::CurrentStates.RequestZBufferEnabled		(true);
	Asura_Render::CurrentStates.RequestZBufferWriteEnabled	(true);
#endif
}

/// End of function 'Asura_ScreenShot_System::Platform_RenderProgressBar'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_ScreenShot_System::Platform_GetScreenDimensions
// Author       : Andy Tate
// Date         : 10-Jun-2005
//
// Description  : Gets the specified screen dimensions.
//
// Parameters   : Pointers to the position, size and bit-depth to get, if these
//				  are NULL then don't bother fetching the data.
//
// Return value : None.
////////////////////////////////////////////////////////////////////////////////

void Asura_ScreenShot_System::Platform_GetScreenDimensions (Asura_IntVector_2* pxPosition,
															Asura_IntVector_2* pxSize,
															u_int* puFormat)
{
	// Do we need to get the position?

	if (NULL != pxPosition)
	{
		// Yes - do so...

		pxPosition->iX	= 0;
		pxPosition->iY	= 0;
	}


	// Do we need to get the size?

	if (NULL != pxSize)
	{
		// Yes - do so...

		pxSize->iX	= Asura_GX::GetScreenWidth();
		pxSize->iY	= Asura_GX::GetScreenHeight();
	}


	// Do we need to get the bit-depth?

	if (NULL != puFormat)
	{
		// Decide upon the bit-depth...

		*puFormat	= 0;
	}
}

/// End of function 'Asura_ScreenShot_System::Platform_GetScreenDimensions'. ///

DEBUGVAR bool dbg_bForceGreyScale = false;

unsigned char clip(int i) {return i > 255 ? 255 : i < 0 ? 0 : i; }
void ConvertYUVtoRGB(unsigned char Y, unsigned char U, unsigned char V, unsigned char &r, unsigned char &g, unsigned char &b)
{
	if (dbg_bForceGreyScale)
	{
		r = g = b = Y;
	} 
	else {
		int C = Y - 16;
		int D = U - 128;
		int E = V - 128;
		r = clip(( 298 * C           + 409 * E + 128) >> 8);
		g = clip(( 298 * C - 100 * D - 208 * E + 128) >> 8);
		b = clip(( 298 * C + 516 * D           + 128) >> 8);
	}
}


////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_ScreenShot_System::Platform_WriteScreenAs24Bit
// Author       : Andy Tate
// Date         : 10-Jun-2005
//
// Description  : Writes the specified portion of the screen to the specified
//				  stream as a 24-bit BMP.
//
// Parameters   : A reference to the stream to write to, along with the position
//				  size and format of the screen.
//
// Return value : None.
////////////////////////////////////////////////////////////////////////////////
#define EFB_ADDRESS 0x08000000


void Asura_ScreenShot_System::Platform_WriteScreenAs24Bit (Asura_Stream& xStream,
														   const Asura_IntVector_2& xPosition,
														   const Asura_IntVector_2& xSize,
														   const u_int uFormat)
{
#if 0
	const int				iWidth				= xSize.iX;
	const int				iHeight				= xSize.iY;

	Asura_GX::WaitForRenderDone();
	const unsigned char *pData = static_cast<const unsigned char *>(Asura_GX::GetFrameBuffer());


	// Assume that the back buffer format is in YUV 16 bit format


	unsigned char pucScanline[640*3];

	for (int y = xPosition.iY + iHeight - 1; y >= xPosition.iY; --y)
	{
		unsigned char * pucDest		= pucScanline;
		const unsigned char * pucSource   = pData + y * Asura_GX::GetScreenStride() + xPosition.iX * 2;

		// Go through each pixel on this line...
		for (int x = xPosition.iX; x < xPosition.iX + iWidth; x += 2)
		{
			unsigned char Y = *(pucSource++), U = *(pucSource++), Y2 = *(pucSource++), V = *(pucSource++);
			unsigned char r,g,b;
			ConvertYUVtoRGB(Y,U,V, r, g, b);
			*(pucDest++) = b; // blue
			*(pucDest++) = g; // green
			*(pucDest++) = r; // red
			ConvertYUVtoRGB(Y2,U,V, r, g, b);
			*(pucDest++) = b; // blue
			*(pucDest++) = g; // green
			*(pucDest++) = r; // red
		}
		// Write this line...
		xStream.WriteData(iWidth * 3, pucScanline);
	}
#elif 1
	const int				iWidth				= xSize.iX;
	const int				iHeight				= xSize.iY;


	Asura_GX::WaitForRenderDone();
	const unsigned int *pData = static_cast<const unsigned int *>(OSPhysicalToUncached(EFB_ADDRESS));


	unsigned char pucScanline[640*3];
	for (int y = xPosition.iY + iHeight - 1; y >= xPosition.iY; --y)
	{
		unsigned char * pucDest		= pucScanline;
		const unsigned int * puSource   = pData + y * 1024 + xPosition.iX;

		// Go through each pixel on this line...
		for (int x = xPosition.iX; x < xPosition.iX + iWidth; ++x)
		{
			unsigned int uRGB = *(puSource++);
			unsigned char b = uRGB & 0xFF; uRGB >>= 8;
			unsigned char g = uRGB & 0xFF; uRGB >>= 8;
			unsigned char r = uRGB & 0xFF; uRGB >>= 8;
			*(pucDest++) = b; // blue
			*(pucDest++) = g; // green
			*(pucDest++) = r; // red
		}
		// Write this line...
		xStream.WriteData(iWidth * 3, pucScanline);
	}
#else
	const int				iWidth				= xSize.iX;
	const int				iHeight				= xSize.iY;

	unsigned int *pData = static_cast<unsigned int *>(ASURA_WII_MEM2_ALLOC(Asura_GX::GetScreenStride() * OSRoundUp32B(Asura_GX::GetScreenHeight())));
	GXSetTexCopySrc( 0, 0, Asura_GX::GetScreenWidth(), Asura_GX::GetScreenHeight() );
	GXSetTexCopyDst(Asura_GX::GetScreenWidth(), Asura_GX::GetScreenHeight(), GX_TF_RGBA8, GX_DISABLE );
	GXCopyTex(pData, false);
	GXPixModeSync();
	GXInvalidateTexAll();
	Asura_GX::WaitForRenderDone();

	unsigned char pucScanline[640*3];
	for (int y = xPosition.iY + iHeight - 1; y >= xPosition.iY; --y)
	{
		unsigned char * pucDest		= pucScanline;
		const unsigned int * puSource   = pData + y * Asura_GX::GetScreenStride() + xPosition.iX;

		// Go through each pixel on this line...
		for (int x = xPosition.iX; x < xPosition.iX + iWidth; ++x)
		{
			unsigned int uRGB = *(puSource++);
			unsigned char b = uRGB & 0xFF; uRGB >>= 8;
			unsigned char g = uRGB & 0xFF; uRGB >>= 8;
			unsigned char r = uRGB & 0xFF; uRGB >>= 8;
			*(pucDest++) = b; // blue
			*(pucDest++) = g; // green
			*(pucDest++) = r; // red
		}
		// Write this line...
		xStream.WriteData(iWidth * 3, pucScanline);
	}
	Asura_Wii_MEM2::Free(pData);

#endif

}

/// End of function 'Asura_ScreenShot_System::Platform_WriteScreenAs24Bit'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_ScreenShot_System::Platform_AddScreenToFPBuffer
// Author       : Andy Tate
// Date         : 10-Jun-2005
//
// Description  : Adds the specified portion of the screen to the floating-point
//				  RGB buffer.
//
// Parameters   : 
//
// Return value : None.
////////////////////////////////////////////////////////////////////////////////


void Asura_ScreenShot_System::Platform_AddScreenToFPBuffer (const Asura_IntVector_2& xPosition,
															const Asura_IntVector_2& xSize,
															const u_int uFormat,
															float* pfFPBuffer)
{
	const int				iWidth				= xSize.iX;
	const int				iHeight				= xSize.iY;


	Asura_GX::WaitForRenderDone();

#if 0
	const unsigned char *pData = static_cast<const unsigned char *>(Asura_GX::GetFrameBuffer());


	// Assume that the back buffer format is in YUV 16 bit format

	float * pufDest		= pfFPBuffer;
	for (int y = xPosition.iY + iHeight - 1; y >= xPosition.iY; --y)
	{
		const unsigned char * pucSource   = pData + y * Asura_GX::GetScreenStride() + xPosition.iX * 2;

		// Go through each pixel on this line...
		for (int x = xPosition.iX; x < xPosition.iX + iWidth; x += 2)
		{
			unsigned char Y = *(pucSource++), U = *(pucSource++), Y2 = *(pucSource++), V = *(pucSource++);
			unsigned char r,g,b;
			ConvertYUVtoRGB(Y,U,V, r, g, b);
			*(pufDest++) += static_cast<float>(b); // blue
			*(pufDest++) += static_cast<float>(g); // green
			*(pufDest++) += static_cast<float>(r); // red
			ConvertYUVtoRGB(Y2,U,V, r, g, b);
			*(pufDest++) += static_cast<float>(b); // blue
			*(pufDest++) += static_cast<float>(g); // green
			*(pufDest++) += static_cast<float>(r); // red
		}
	}
#elif 1
	const unsigned int *pData = static_cast<const unsigned int *>(OSPhysicalToUncached(EFB_ADDRESS));
	DCInvalidateRange((void*)pData, Asura_GX::GetScreenWidth() * 1024 * 4);



	// Assume that the back buffer format is in YUV 16 bit format

	float * pufDest		= pfFPBuffer;
	for (int y = xPosition.iY, y2 = iHeight - 1; y < xPosition.iY + iHeight; ++y, --y2)
	{
		const unsigned int * puSource   = pData + y * 1024 + xPosition.iX;
		float * pufDest		= pfFPBuffer + y2 * iWidth * 3;

		// Go through each pixel on this line...
		for (int x = xPosition.iX; x < xPosition.iX + iWidth; ++x)
		{
			unsigned int uRGB = *(puSource++);
			unsigned char b = uRGB & 0xFF; uRGB >>= 8;
			unsigned char g = uRGB & 0xFF; uRGB >>= 8;
			unsigned char r = uRGB & 0xFF; uRGB >>= 8;
			*(pufDest++) += static_cast<float>(b); // blue
			*(pufDest++) += static_cast<float>(g); // green
			*(pufDest++) += static_cast<float>(r); // red
		}
	}
	DCFlushRange(pfFPBuffer, iWidth * iHeight * 3);
#else
	unsigned int *pData = static_cast<unsigned int *>(ASURA_WII_MEM2_ALLOC(Asura_GX::GetScreenStride() * OSRoundUp32B(Asura_GX::GetScreenHeight())));
	GXSetTexCopySrc( 0, 0, Asura_GX::GetScreenWidth(), Asura_GX::GetScreenHeight() );
	GXSetTexCopyDst(Asura_GX::GetScreenWidth(), Asura_GX::GetScreenHeight(), GX_TF_RGBA8, GX_DISABLE );
	GXCopyTex(pData, false);
	GXPixModeSync();
	GXInvalidateTexAll();
	Asura_GX::WaitForRenderDone();





	// Assume that the back buffer format is in YUV 16 bit format

	float * pufDest		= pfFPBuffer;
	for (int y = xPosition.iY + iHeight - 1; y >= xPosition.iY; --y)
	{
		const unsigned int * puSource   = pData + y * Asura_GX::GetScreenStride() + xPosition.iX * 2;

		// Go through each pixel on this line...
		for (int x = xPosition.iX; x < xPosition.iX + iWidth; ++x)
		{
			unsigned int uRGB = *(puSource++);
			unsigned char b = uRGB & 0xFF; uRGB >>= 8;
			unsigned char g = uRGB & 0xFF; uRGB >>= 8;
			unsigned char r = uRGB & 0xFF; uRGB >>= 8;
			*(pufDest++) += static_cast<float>(b); // blue
			*(pufDest++) += static_cast<float>(g); // green
			*(pufDest++) += static_cast<float>(r); // red
		}
	}

	Asura_Wii_MEM2::Free(pData);
#endif

}

/// End of function 'Asura_ScreenShot_System::Platform_AddScreenToFPBuffer'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_ScreenShot_System::Platform_BeginHiResRender
// Author       : Andy Tate
// Date         : 13-Jun-2005
//
// Description  : Handles the platform specific start of the high-resolution render.
//
// Return value : None.
////////////////////////////////////////////////////////////////////////////////

void Asura_ScreenShot_System::Platform_BeginHiResRender (void)
{
#if 0
	// Wait until drawing is complete...

	sceGeDrawSync	(0);


	// Disable motion-blur and mip-mapping......

	g_bMotionBlurEnabled	= Asura_PSP_MotionBlur::IsEnabled ();

	Asura_PSP_MotionBlur::Enable	(false);
	sceGuTexFilter					(SCEGU_LINEAR_MIPMAP_NEAREST, SCEGU_NEAREST);
#endif
}

/// End of function 'Asura_ScreenShot_System::Platform_BeginHiResRender'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_ScreenShot_System::Platform_EndHiResRender
// Author       : Andy Tate
// Date         : 13-Jun-2005
//
// Description  : Handles the platform specific end of the high-resolution render.
//
// Return value : None.
////////////////////////////////////////////////////////////////////////////////

void Asura_ScreenShot_System::Platform_EndHiResRender (void)
{
#if 0
	// Restore the motion-blur and mip-mapping...

	Asura_PSP_MotionBlur::Enable	(g_bMotionBlurEnabled);
	sceGuTexFilter					(SCEGU_LINEAR_MIPMAP_LINEAR, SCEGU_LINEAR);
#endif
}

/// End of function 'Asura_ScreenShot_System::Platform_EndHiResRender'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_ScreenShot_System::Platform_TakeScreenShot
// Author       : Andy Tate
// Date         : 10-Jun-2005
//
// Description  : Handles the taking of screen-shots.
//
// Parameters   : The name of the file to be saved.
//
// Return value : The result of the screen shot function.
////////////////////////////////////////////////////////////////////////////////

int Asura_ScreenShot_System::Platform_TakeScreenShot (const char* szFilename)
{
	Asura_IntVector_2	xScreenPosition;
	Asura_IntVector_2	xScreenSize;
	u_int				uScreenFormat;


	// Take the normal screen-shot as a BMP...

	Platform_GetScreenDimensions	(&xScreenPosition, &xScreenSize, &uScreenFormat);
	return							(BMPTakeScreenShot(szFilename,  xScreenPosition, xScreenSize, uScreenFormat));
}

/// End of function 'Asura_ScreenShot_System::Platform_TakeScreenShot'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_ScreenShot_System::Platform_NotTakingScreenShot
// Author       : Andy Tate
// Date         : 10-Jun-2005
//
// Description  : Handles the case of being in screen-shot mode but not actually
//				  taking a screen-shot.
//
// Return value : None.
////////////////////////////////////////////////////////////////////////////////

void Asura_ScreenShot_System::Platform_NotTakingScreenShot (void)
{
	// Nothing to do...
}

/// End of function 'Asura_ScreenShot_System::Platform_NotTakingScreenShot'. ///

float*	Asura_ScreenShot_System::Platform_AllocateFloatBuffer	(u_int uNumFloats) 
{ 
	return (float*)ASURA_WII_MEM2_ALLOC_SILENTFAIL(uNumFloats * 4);
}
void	Asura_ScreenShot_System::Platform_FreeFloatBuffer		(float* pfBuffer) 
{
	return ASURA_WII_MEM2_FREE(pfBuffer);

}
void	Asura_ScreenShot_System::Platform_RenderPrompt			(void) {}

#endif