//
// Asura_Platform_Rumble.cpp
//
// Created by KOR - 09 October 2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

// This file's header.
#include "Input/Asura_Rumble_System.h"

#if (defined ASURA_SUPPORT_CONTROLLER_RUMBLE) && (ASURA_SUPPORT_CONTROLLER_RUMBLE != 0)

#include <revolution/wpad.h>

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

Asura_Platform_Rumble* Asura_Rumble_System::ASURA_STANDARD_RUMBLE_EXPLOSION;
Asura_Platform_Rumble* Asura_Rumble_System::ASURA_STANDARD_RUMBLE_SINGLESHOT;
Asura_Platform_Rumble* Asura_Rumble_System::ASURA_STANDARD_RUMBLE_HEARTBEAT;
Asura_Platform_Rumble* Asura_Rumble_System::ASURA_STANDARD_RUMBLE_HIT;

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

static float g_fRumbleAccumulators[PLATFORM_NUM_CONTROLLERS] = {0.0f, 0.0f, 0.0f, 0.0f};
static bool g_bWasPaused = false;


void Asura_Rumble_System::Platform_PlayCurrentRumbles()
{
	if(s_bPaused)
	{
		if(!g_bWasPaused)
		{
			for ( int iPadIndex = 0; iPadIndex < PLATFORM_NUM_CONTROLLERS; ++iPadIndex )
			{
				WPADStopMotor( iPadIndex );
			}
			g_bWasPaused = true;
		}
		return;
	}
	else
	{
		g_bWasPaused = false;
	}

    for ( int iPadIndex = 0; iPadIndex < PLATFORM_NUM_CONTROLLERS; ++iPadIndex )
    {
        g_fRumbleAccumulators[iPadIndex] += s_fCurrentMotorMagnitudes[ iPadIndex ][ ASURA_WII_RUMBLE_MOTOR ];
		if(g_fRumbleAccumulators[iPadIndex] > 1.0f)
		{
            WPADStartMotor( iPadIndex );
			g_fRumbleAccumulators[iPadIndex] = 0.0f;
        }
        else
        {
            WPADStopMotor( iPadIndex );
        }
    }
}

void Asura_Rumble_System::Platform_Initialise()
{
	for(int i = 0; i < PLATFORM_NUM_CONTROLLERS; i++)
	{
		g_fRumbleAccumulators[i] = 0.0f;
	}
    // (NJD - 21.08.2003) This has been copied, pasted and slightly edited from the PS2 version
	Asura_Handle iHeartbeatMotor = Asura_Rumble::CreateAsuraRumble(RUMBLE_BASE_CONSTANT, RUMBLE_ADJUST_NONE, 0.0f, 0.0f, 5.0f, 1);
	Asura_Rumble::AddRumbleStep(iHeartbeatMotor, RUMBLE_BASE_CONSTANT, RUMBLE_ADJUST_NONE, 1.0f, 0.0f, 3.0f, 1);
	Asura_Rumble::AddRumbleStep(iHeartbeatMotor, RUMBLE_BASE_CONSTANT, RUMBLE_ADJUST_NONE, 0.0f, 0.0f, 1.0f, 1);
	Asura_Handle iHeartbeatRumble = s_pxDynamicPlatformRumbleList->Allocate();
	ASURA_STANDARD_RUMBLE_HEARTBEAT = s_pxDynamicPlatformRumbleList->GetPointer(iHeartbeatRumble);
	ASURA_STANDARD_RUMBLE_HEARTBEAT->m_apxMotorRumbles[ASURA_WII_RUMBLE_MOTOR] = Asura_Rumble::GetRumblePointer(iHeartbeatMotor);

	Asura_Handle iHitMotor = Asura_Rumble::CreateAsuraRumble(RUMBLE_BASE_LINEAR, RUMBLE_ADJUST_REVERSE, 1.0f, 0.0f, 1.0f, 1);
	Asura_Handle iHitRumble = s_pxDynamicPlatformRumbleList->Allocate();
	ASURA_STANDARD_RUMBLE_HIT = s_pxDynamicPlatformRumbleList->GetPointer(iHitRumble);
	ASURA_STANDARD_RUMBLE_HIT->m_apxMotorRumbles[ASURA_WII_RUMBLE_MOTOR] = Asura_Rumble::GetRumblePointer(iHitMotor);

	Asura_Handle iExplosionMotor = Asura_Rumble::CreateAsuraRumble(RUMBLE_BASE_CONSTANT, RUMBLE_ADJUST_NONE, 1.0f, 0.0f, 2.5f, 1);
	Asura_Rumble::AddRumbleStep(iExplosionMotor, RUMBLE_BASE_LINEAR, RUMBLE_ADJUST_REVERSE, 1.0f, 0.0f, 1.0f, 1);
	Asura_Rumble::AddRumbleStep(iExplosionMotor, RUMBLE_BASE_LINEAR, RUMBLE_ADJUST_REVERSE, 1.0f, 0.0f, 0.5f, 1);
	Asura_Handle iExplosionRumble = s_pxDynamicPlatformRumbleList->Allocate();
	ASURA_STANDARD_RUMBLE_EXPLOSION = s_pxDynamicPlatformRumbleList->GetPointer(iExplosionRumble);
	ASURA_STANDARD_RUMBLE_EXPLOSION->m_apxMotorRumbles[ASURA_WII_RUMBLE_MOTOR] = Asura_Rumble::GetRumblePointer(iExplosionMotor);

	Asura_Handle iSingleShotMotor = Asura_Rumble::CreateAsuraRumble(RUMBLE_BASE_CONSTANT, RUMBLE_ADJUST_NONE, 1.0f, 0.0f, 2.5f, 1);
	Asura_Handle iSingleShotRumble = s_pxDynamicPlatformRumbleList->Allocate();
	ASURA_STANDARD_RUMBLE_SINGLESHOT = s_pxDynamicPlatformRumbleList->GetPointer(iSingleShotRumble);
	ASURA_STANDARD_RUMBLE_SINGLESHOT->m_apxMotorRumbles[ASURA_WII_RUMBLE_MOTOR] = Asura_Rumble::GetRumblePointer(iSingleShotMotor);
}
void Asura_Rumble_System::Platform_StopAllRumbles()
{
	for ( int iPadIndex = 0; iPadIndex < PLATFORM_NUM_CONTROLLERS; ++iPadIndex )
	{
		WPADStopMotor( iPadIndex );
	}
}


void Asura_Rumble_System::Platform_Shutdown()
{
	Platform_StopAllRumbles();
}


void Asura_Rumble_System::Platform_Reset()
{
	Platform_Shutdown();
	Platform_Initialise();
}

#endif	// (defined ASURA_SUPPORT_CONTROLLER_RUMBLE) && (ASURA_SUPPORT_CONTROLLER_RUMBLE != 0)
