//////////////////////////////////////////////////////////////////////////////
//
// Asura_Platform_CommandConsole.cpp
//
// Created by ARC - 27 November 2006 15:56:35
// Copyright (c) 2006 Rebellion. All rights reserved.
//
// Description:
//
//     Wii-specific implementations of command console functionality.
//     Just dummy functions to allow the console to be instantiated so the
//     DVS can be used
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

#ifdef ASURA_SUPPORT_COMMAND_CONSOLE

// Engine independent.
#include "Core/Asura_TextureFlags.h"
#include "Input/Asura_ControlMap.h"
#include "Interface/Asura_CommandConsole.h"

// Engine platform-specific.
#include "Input/Asura_Wii_Input.h"

// C O N S T A N T S /////////////////////////////////////////////////////////

// S T A T I C   F U N C T I O N S ///////////////////////////////////////////

// C L A S S   I M P L E M E N T A T I O N S /////////////////////////////////

void Asura_CommandConsole::Platform_Initialise( void )
{
    SetBorderX(7.5f);
    SetFontScale(0.55f);

    SetTexture( ASURA_TEXTURE_NONE );
    SetBackgroundColour( 0xFF000000 );

#ifdef ASURA_DEBUG_VARIABLE_SYSTEM
	// put DVS away from the edges.
	SetDVSBorder(0.05f);
#endif
}

#ifdef ASURA_DEBUG_VARIABLE_SYSTEM

void Asura_DebugVariableSystem::Platform_InitialiseDefaultControlMapping( void )
{
	s_xControlMap.SetControllerIndex(0);

	s_xControlMap.AddKeyMapping( KEYMAP_MOVERIGHT,              ASURA_WII_INPUT_RIGHT,		Asura_ControlMapping::ON_PRESS );
	s_xControlMap.AddKeyMapping( KEYMAP_MOVELEFT,               ASURA_WII_INPUT_LEFT,		Asura_ControlMapping::ON_PRESS );
	s_xControlMap.AddKeyMapping( KEYMAP_MOVEUP,                 ASURA_WII_INPUT_UP,			Asura_ControlMapping::ON_PRESS );
	s_xControlMap.AddKeyMapping( KEYMAP_MOVEDOWN,               ASURA_WII_INPUT_DOWN,		Asura_ControlMapping::ON_PRESS );
	s_xControlMap.AddKeyMapping( KEYMAP_INCREASEVALUE,          ASURA_WII_INPUT_PLUS,		Asura_ControlMapping::ON_PRESS );
	s_xControlMap.AddKeyMapping( KEYMAP_DECREASEVALUE,          ASURA_WII_INPUT_MINUS,		Asura_ControlMapping::ON_PRESS );
	s_xControlMap.AddKeyMapping( KEYMAP_INCREASEVALUEINCREMENT, ASURA_WII_INPUT_A,			Asura_ControlMapping::ON_PRESS );
	s_xControlMap.AddKeyMapping( KEYMAP_DECREASEVALUEINCREMENT, ASURA_WII_INPUT_B,			Asura_ControlMapping::ON_PRESS );

#ifdef ASURA_USE_CONTROLMAP_MANAGER
	s_xControlMap.AddKeyMapping( KEYMAP_CLOSE,					ASURA_WII_INPUT_1,			Asura_ControlMapping::ON_DEBOUNCED_PRESS );
#endif 

}

#endif

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

#endif // #ifdef ASURA_SUPPORT_COMMAND_CONSOLE
