//
// Asura_GX_SmoothSkin.h
//
// Created by KOR - 09 October 2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __ASURA_GX_SMOOTHSKIN_H__
#define __ASURA_GX_SMOOTHSKIN_H__

#include "Core/Asura_HashTree.h"
#include "Core/Asura_Maths.h"
#include "Rendering/Asura_GX.h"
#include <Revolution/gx.h>


//#define USE_SMOOTH_SKIN_ADAPTIVE_RENDERBUFFER	(1)

#include "Rendering/Asura_GX_DisplayList.h"

const u_int uASURA_WII_MAX_SMOOTHSKIN_VERTICES = 40000;
const u_int uASURA_WII_MAX_SMOOTHSKINS = 128;

// A character vertex is now split into three sets of information:
// -Position and normal (one for each vertex)
//   These are multiplied by the appropriate bone matrices and weights, given by:
// -Bone index and weight (between one and four for each vertex)
//   These are used for the skinning; they are read in until a total weight of 1 is reached,
//   at which point the next vertex is calculated
// -Texture co-ordinates (one for each vertex)
//   Array and index passed directly to GPU for rendering
struct Asura_GX_SmoothSkin_Vertex
{
	// Must cast to float during skinning
	short			m_sX;				// 2
	short			m_sY;				// 2
	short			m_sZ;				// 2
	// Must cast to float during skinning
	char			m_cNX;				// 1
	char			m_cNY;				// 1
	char			m_cNZ;				// 1
	char			m_cNPad;			// 1
										// 10
};

struct Asura_GX_SmoothSkin_BoneInfo
{
	unsigned char	m_ucBoneIndex;		// 1
	unsigned char	m_ucBoneWeight;		// 1
										// 2
};

struct Asura_GX_SmoothSkin_TexInfo
{
	// Use as is
	short			m_sU;				// 2
	short			m_sV;				// 2
										// 4
};

struct Asura_GX_SmoothSkin_Output_Vertex
{
	Asura_Vector_3	m_xPosition;
	Asura_Vector_3	m_xNormal;
};

struct Asura_GX_SmoothSkin_Material
{
	int		m_iMaterialID;
	u_int	m_uNumTris;
	Asura_GX_DisplayListBlock m_xDisplayListBlock;
};

class Asura_GX_SmoothSkin_RenderBuffer
{
#ifdef USE_SMOOTH_SKIN_ADAPTIVE_RENDERBUFFER
	static u_int s_uRenderVertexSize;
#else
	static const u_int s_uRenderVertexSize = uASURA_WII_MAX_SMOOTHSKIN_VERTICES;
#endif
	static Asura_GX_SmoothSkin_Output_Vertex* s_pxRenderVertices;
	static unsigned int s_auRenderVertexPtr[uASURA_WII_MAX_SMOOTHSKINS];
	static volatile u_int s_pxRenderVertexGPUIndex;
	static u_int s_pxRenderVertexCPUIndex;

public:
	static void Initialise();
	static void Shutdown();

	static void IncrementGPUPtr();
	static void PostCreateVerts();
	static void PostRenderVerts();
	static Asura_GX_SmoothSkin_Output_Vertex *GetRenderVertices();

	static void AllocateRenderVertices(u_int uNumVerts);
	static void SetupArrays(bool bIncludeNormals);
};

class Asura_GX_SmoothSkin
{
public:
	Asura_GX_SmoothSkin();
	~Asura_GX_SmoothSkin();
	void LoadFromChunk(Asura_Handle fileHandle, u_int uChunkVersion, u_int uChunkFlags);
	void LoadFromStream(Asura_Chunk_Stream& xStream, u_int uChunkVersion, u_int uChunkFlags);
	void BeginRender(u_int uCamVisFlags) const;
	void Render(u_int uCamVisFlags, int iTexIDOverride = -1, bool bDepthOnly = false, float fAlpha = 1.f) const;
    void CalculateAndConvertVertexPositions( const bool bRenderSimple ) const;
	bool ShouldRenderSimple(u_int uCamVisFlags) const;
	void SetupArrays() const;
	void CalculateBoneOffsets() const;

	// Get methods
	int GetMaterialID(u_int uMaterialIndex) const { return m_pxMaterials[uMaterialIndex].m_iMaterialID; }
	u_int GetNumberOfMaterials() const { return m_uNumberOfMaterials; }
	const Asura_GX_SmoothSkin_Material* GetMaterial(u_int uIndex) const { return &m_pxMaterials[uIndex]; }
	u_char* GetColours() const { return m_pucColours; }
	const Asura_GX_DisplayList &GetDisplayList(u_int uMaterialIndex) const { return m_pxMaterials[uMaterialIndex].m_xDisplayListBlock.DisplayList(); }
	bool GetIncludeNormals() const { return m_bIncludeNormals; }
private:
	u_int							m_uNumberOfVertices;
	u_int							m_uNumberOfBoneInfos;
	u_int							m_uNumberOfMaterials;
	u_int							m_uColourDepth;
	Asura_GX_SmoothSkin_Vertex*		m_pxVertices;
	Asura_GX_SmoothSkin_TexInfo*	m_pxTexInfo;
	Asura_GX_SmoothSkin_BoneInfo*	m_pxBoneInfo;
	Asura_GX_SmoothSkin_Material*	m_pxMaterials;
	u_char*							m_pucColours;
	bool							m_bIncludeNormals;
};

class Asura_GX_SmoothSkinRenderer
{
public:
	static void Initialise();
	static void Update();
	static void Reset();
	static void Shutdown();
	static bool AddSmoothSkin(Asura_Handle fileHandle, u_int uChunkVersion, u_int uChunkFlags);
	static void RenderSmoothSkin(Asura_Hash_ID uHashID, u_int uCamVisFlags, int iTexIDOverride, float fAlpha = 1.f);
	static int  GetFirstMaterialIndex(Asura_Hash_ID uHashID);
	static bool IsSkinOpaque(Asura_Hash_ID uHashID);
	static const Asura_GX_SmoothSkin* GetSkin(Asura_Hash_ID uHashID);
private:
	static Asura_TemplateHashTree<Asura_GX_SmoothSkin>	s_xHashTree;
};

#endif//__ASURA_GX_SMOOTHSKIN_H__