//
// Asura_Platform_Shape.cpp
//
// Created by KOR - 10 October 2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

// This file's header.
#include "Shapes/Asura_Shapes.h"

// Wii
#include "Shapes/Asura_Platform_Shape.h"
#include "Rendering/Asura_GX_Shape.h"

Asura_Platform_Shape_RenderData::Asura_Platform_Shape_RenderData( const char*& pcChunkBuffer )
{
	// Ooooh! Magic number!
	pcChunkBuffer += 260;
}

void Asura_Shape_Instance::Platform_SetAmbientARGB( const unsigned int uARGB )
{
	m_uAmbientARGB = uARGB;
}

void Asura_Shape_Instance::Platform_SetAmbientRGB( const unsigned char ucR, const unsigned char ucG, const unsigned char ucB, const unsigned char ucA)
{
	m_uAmbientARGB = (ucA << 24) | (ucR << 16) | (ucG << 8) | ucB;
}

/////////////////////////////////////////////////////////////////////////////////////////
// FUNCTION		Asura_Shape_Instance::Platform_SetDefaultAmbientRGB
// PARAMETERS	None
// RETURNS		None
// DESCRIPTION	
// AUTHOR		Richard M. Smith (9 August 2002)
// MODIFIED     Mike Healey (26.9.02) now calling the appropriate function to set up
//                          ambient to correct values for each platform.
// MODIFIED		Andrew Weinkove (01.03.04) Now just set a flag to say it uses default ambient
/////////////////////////////////////////////////////////////////////////////////////////
void Asura_Shape_Instance::Platform_SetDefaultAmbientRGB( void )
{
	m_uFlags |= ASURA_SHAPE_INSTANCE_USES_DEFAULT_AMBIENT;
} 

int Asura_Shape::Platform_GetMaterialIdx() const
{
	return Asura_GX_ShapeRenderer::GetShapeMaterial(m_uHashID);
}

bool Asura_Shape::Platform_IsOpaque( void ) const
{
	const int iMaterialIndex = GetMaterialIdx();
	return !(Asura_Material_System::IsMaterialTransparent( iMaterialIndex ) || Asura_Material_System::IsMaterialPS2AdditiveAlpha( iMaterialIndex ));
}


DEBUGVAR bool dbg_bDisableShapes = false;
void Asura_Shape::Platform_Render(const Asura_Shape_Instance* pxShapeInst, u_int uAmbientARGB, u_int uCamVisFlags) const
{
	if(dbg_bDisableShapes) { return; }

	pxShapeInst->UpdateLighting();

	Asura_GX_ShapeRenderer::RenderShape(m_uHashID,uCamVisFlags, pxShapeInst->GetPosition(), pxShapeInst->GetOrientation(), pxShapeInst->m_fAlpha);
}

void Asura_Shape::Platform_RenderShadow( const Asura_Shape_Instance* pxShapeInst, const Asura_Shadow& xShadow, u_int uCamVisFlags ) const
{
	Asura_Assert(0, "Shape shadows not implemented on GameCube");
}

////////////////////////////////////// End of file //////////////////////////////////////
