//
// Asura_Wii_Sound.cpp
//
// Created by KOR - 09 July 2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

// This file's header.
#include "Sound/Asura_Wii_Sound.h"

// Asura
#include "Core/Asura_String.h"
#include "Core/Memory/Asura_Mem.h"
#include "FileAccess/Asura_FileAccess.h"

// Asura Wii
#include "FileAccess/Asura_PlatformFileAccess.h"
#include "FileAccess/Asura_PlatformFileAccessUtils.h"
#include "HomeButton/Asura_Wii_HomeButtonMenu.h"
#include "Sound/Asura_Wii_StreamSound.h"

Asura_Wii_Sound_Filename_Entry Asura_Wii_Sound::s_axSoundFilenames[uASURA_WII_SOUND_MAX_FILES];
u_int						Asura_Wii_Sound::s_uNumFilenames = 0;
u_int						Asura_Wii_Sound::s_uPermanentFilenames = 0;
Asura_Wii_Sound_LoadStatus	Asura_Wii_Sound::s_axSoundLoadStatus[ASURA_MAX_NUMBER_OF_LOADED_SOUNDS];
Asura_Wii_Sound_File_Data	Asura_Wii_Sound::s_axSoundFileData[ASURA_MAX_NUMBER_OF_LOADED_SOUNDS];
// DSP-ADPCM in ARAM = true, stream = false
bool						Asura_Wii_Sound::s_abActiveSoundInMemory[ASURA_WII_MAX_NUMBER_OF_ACTIVE_SOUNDS];
Asura_Wii_Streaming_Sound	Asura_Wii_Sound::s_axActiveStreams[ASURA_WII_MAX_NUMBER_OF_ACTIVE_SOUNDS];

Asura_Wii_Sound_Filename_Entry::Asura_Wii_Sound_Filename_Entry()
: m_iDVDEntryNum(-1)
{
	Asura_Mem::Set(m_szFilename, 0, uASURA_WII_SOUND_MAX_FILENAME);
}

void Asura_Wii_Sound_Filename_Entry::SetFilename(const char* szFilename)
{
	Asura_PrintfToBfr(m_szFilename, "%s", uASURA_WII_SOUND_MAX_FILENAME, true, szFilename);
	REPLACE_SLASHES(m_szFilename);
	m_iDVDEntryNum = DVDConvertPathToEntrynum(m_szFilename);
}

int Asura_Wii_Sound::Initialise()
{
	Asura_AX::Initialise();

	// AX has been initialised.  Time to initialise the home button.
	Asura_Wii_HomeButtonMenu::InitialiseSound();

	Asura_Wii_Streaming_Sound_System::Initialise();

	for(u_int u = 0; u < ASURA_MAX_NUMBER_OF_LOADED_SOUNDS; u++)
	{
		s_axSoundLoadStatus[u] = Asura_Wii_Sound_Not_Loaded;
		s_axSoundFileData[u].m_uOffset = 0;
		s_axSoundFileData[u].m_uSize = 0;
		s_axSoundFileData[u].m_iFileIndex = -1;
	}
	return ASURA_WII_MAX_NUMBER_OF_ACTIVE_SOUNDS;
}

void Asura_Wii_Sound::Shutdown()
{
	Asura_Wii_Streaming_Sound_System::Shutdown();
	Asura_AX::Shutdown();
}

void Asura_Wii_Sound::Update()
{
	Asura_AX::Update();
	Asura_PlatformFileAccessUtils::CheckDiscStatus();
}

void Asura_Wii_Sound::Reset()
{
	Asura_AX::Reset();
#ifdef ASURA_ASSERT
	for(u_int u = 0; u < ASURA_MAX_NUMBER_OF_LOADED_SOUNDS; u++)
	{
		const Asura_Wii_Sound_LoadStatus& eStatus = s_axSoundLoadStatus[u];
		Asura_Assert(eStatus == Asura_Wii_Sound_Loaded_Perm ||
					 eStatus == Asura_Wii_Sound_Not_Loaded ||
					 eStatus == Asura_Wii_Sound_Stream_Offset_Loaded,
					 "Unexpected sound status at reset");
	}
#endif
	s_uNumFilenames = s_uPermanentFilenames;
}

void Asura_Wii_Sound::SetResetPoint()
{
	for(u_int u = 0; u < ASURA_MAX_NUMBER_OF_LOADED_SOUNDS; u++)
	{
		Asura_Wii_Sound_LoadStatus& eStatus = s_axSoundLoadStatus[u];
		if(eStatus == Asura_Wii_Sound_Loaded_Temp)
		{
			eStatus = Asura_Wii_Sound_Loaded_Perm;
		}
	}
	s_uPermanentFilenames = s_uNumFilenames;
}

bool Asura_Wii_Sound::LoadSound(const char* szFilename, int iIndex)
{
	return Asura_AX::LoadSound(szFilename, iIndex);
}

bool Asura_Wii_Sound::LoadSoundFromResource(Asura_Handle iFileHandle, int iFileSize, int iLoadedSoundIndex)
{
	Asura_Assert( iLoadedSoundIndex >= 0 && iLoadedSoundIndex < ASURA_MAX_NUMBER_OF_LOADED_SOUNDS, "invalid index");
	if(s_axSoundLoadStatus[iLoadedSoundIndex] != Asura_Wii_Sound_Not_Loaded)
	{
		Asura_Assert(0, "Sound already loaded at this index.");
		return false;
	}
	if(Asura_AX::LoadSoundFromResource(iFileHandle, iFileSize, iLoadedSoundIndex))
	{
		s_axSoundLoadStatus[iLoadedSoundIndex] = Asura_Wii_Sound_Loaded_Temp;
		return true;
	}
	return false;
}

bool Asura_Wii_Sound::UnloadSound(const int iLoadedSoundIndex)
{
	Asura_Wii_Sound_LoadStatus& eStatus = s_axSoundLoadStatus[iLoadedSoundIndex];
	if(eStatus == Asura_Wii_Sound_Loaded_Temp)
	{
		if(Asura_AX::UnloadSound(iLoadedSoundIndex))
		{
			eStatus = Asura_Wii_Sound_Not_Loaded;
			return true;
		}
	}
	else if(eStatus == Asura_Wii_Sound_Stream_Offset_Loaded)
	{
		eStatus = Asura_Wii_Sound_Not_Loaded;
		return true;
	}
	else
	{
		Asura_Assert(0, "Unloading unexpected sound");
	}
	return false;
}

bool Asura_Wii_Sound::CreateSound(int iLoadedSoundIndex, Asura_Handle iSoundHandle, bool bUseEQ, bool bUseHardware3dPan)
{
    Asura_Assert(iSoundHandle != ASURA_INVALID_HANDLE, "invalid handle");
	if(iSoundHandle == ASURA_INVALID_HANDLE)
	{
		Asura_Assert(0, "Failed to create sound...invalid handle\n");
		return false;
	}
	switch(s_axSoundLoadStatus[iLoadedSoundIndex])
	{
		case Asura_Wii_Sound_Loaded_Perm:
		case Asura_Wii_Sound_Loaded_Temp:
			s_abActiveSoundInMemory[iSoundHandle] = true;
			return Asura_AX::CreateSound(iSoundHandle, iLoadedSoundIndex);
		case Asura_Wii_Sound_Stream_Offset_Loaded:
			s_abActiveSoundInMemory[iSoundHandle] = false;
			// Create stream
			{
				// Load it
				const Asura_Wii_Sound_File_Data& xFileData = s_axSoundFileData[iLoadedSoundIndex];
				const int iEntry = s_axSoundFilenames[xFileData.m_iFileIndex].GetDVDEntryNum();
				const bool bSuccess = s_axActiveStreams[iSoundHandle].CreateFromResource(iEntry, xFileData.m_uOffset, xFileData.m_uSize);
				return bSuccess;
			}
			return false;
		case Asura_Wii_Sound_Not_Loaded:
		default:
			Asura_Assert(0, "Tried to create unknown sound");
			return false;
	}
}

bool Asura_Wii_Sound::PlaySound(Asura_Handle iSoundHandle, bool bLooping)
{
	Asura_Assert(iSoundHandle != ASURA_INVALID_HANDLE, "invalid handle");
	if(s_abActiveSoundInMemory[iSoundHandle])
	{
		return Asura_AX::PlaySound(iSoundHandle, bLooping);
	}
	else
	{
		// Play stream
		s_axActiveStreams[iSoundHandle].Play(bLooping);
		return true;
	}
}

bool Asura_Wii_Sound::StopSound(Asura_Handle iSoundHandle)
{
	Asura_Assert(iSoundHandle != ASURA_INVALID_HANDLE, "invalid handle");
	if(s_abActiveSoundInMemory[iSoundHandle])
	{
		return Asura_AX::StopSound(iSoundHandle);
	}
	else
	{
		// Stop stream
		s_axActiveStreams[iSoundHandle].Stop();
		return true;
	}
}

bool Asura_Wii_Sound::DestroySound(Asura_Handle iSoundHandle)
{
	Asura_Assert(iSoundHandle != ASURA_INVALID_HANDLE, "invalid handle");
	if(s_abActiveSoundInMemory[iSoundHandle])
	{
		return Asura_AX::DestroySound(iSoundHandle);
	}
	else
	{
		// Destroy stream
		s_axActiveStreams[iSoundHandle].Release();
		return true;
	}
}

bool Asura_Wii_Sound::IsSoundPlaying(Asura_Handle iSoundHandle)
{
	Asura_Assert(iSoundHandle != ASURA_INVALID_HANDLE, "invalid handle");
	if(s_abActiveSoundInMemory[iSoundHandle])
	{
		return Asura_AX::IsSoundPlaying(iSoundHandle);
	}
	else
	{
		return !s_axActiveStreams[iSoundHandle].IsFinished();
	}
}

bool Asura_Wii_Sound::SetSoundVolumePanAndFade(Asura_Handle iSoundHandle, float fVolume, float fPan, float fFade)
{
	// volume scale from 0 to 1
	Asura_Assert(iSoundHandle != ASURA_INVALID_HANDLE, "invalid handle");
	Asura_Assert(fVolume >= 0.0f && fVolume <= 1.0f, "volume out of range");

	if(s_abActiveSoundInMemory[iSoundHandle])
	{
		bool bOK = Asura_AX::SetVolume(iSoundHandle, fVolume);
		bOK = bOK && Asura_AX::SetPan(iSoundHandle, fPan);
		bOK = bOK && Asura_AX::SetFade(iSoundHandle, fFade);
		return bOK;
	}
	else
	{
		s_axActiveStreams[iSoundHandle].SetVolume(fVolume);
		return true;
	}
}

bool Asura_Wii_Sound::SetSoundPitch(Asura_Handle iSoundHandle, float fPitch)
{
	if(s_abActiveSoundInMemory[iSoundHandle])
	{
		Asura_Assert(iSoundHandle != ASURA_INVALID_HANDLE, "invalid handle");
		Asura_Assert(fPitch >= 0.0f, "pitch must be positive");
		return Asura_AX::SetPitch(iSoundHandle, fPitch);
	}
	else
	{
		Asura_Assert(fPitch == 1.f, "Cannot change pitch of streams");
		return true;
	}
}

bool Asura_Wii_Sound::PauseSound(Asura_Handle iSoundHandle)
{
	if(s_abActiveSoundInMemory[iSoundHandle])
	{
		Asura_Assert(iSoundHandle != ASURA_INVALID_HANDLE, "invalid handle");
		return Asura_AX::PauseSound(iSoundHandle);
	}
	else
	{
		s_axActiveStreams[iSoundHandle].Pause();
		return true;
	}
}

bool Asura_Wii_Sound::UnpauseSound(Asura_Handle iSoundHandle)
{
	if(s_abActiveSoundInMemory[iSoundHandle])
	{
		Asura_Assert(iSoundHandle != ASURA_INVALID_HANDLE, "invalid handle");
		return Asura_AX::UnpauseSound(iSoundHandle);
	}
	else
	{
		s_axActiveStreams[iSoundHandle].Resume();
		return true;
	}
}

void Asura_Wii_Sound::InitStreamedSounds(const char* szFilename)
{
	Asura_Assert(s_uNumFilenames < uASURA_WII_SOUND_MAX_FILES, "Too many streamed sound files");

	s_axSoundFilenames[s_uNumFilenames++].SetFilename(szFilename);
}

void Asura_Wii_Sound::AddStreamingSoundData(int iSoundID, int iOffset, int iSize, Asura_Hash_ID uSoundHash)
{
	Asura_Assert(s_uNumFilenames, "no filename for streamed sound");
	// flag sound as loaded in Asura_Sound_System and get loaded index.
	int iLoadedIndex = Asura_Sound_System::SoundLoadedByPlatformCode(iSoundID, uSoundHash);
	if (iLoadedIndex < 0)
	{
		Asura_Assert(false, "failed to add streamed sound to list of loaded sounds");
		return;
	}

	Asura_Assert( iLoadedIndex >= 0 && iLoadedIndex < ASURA_MAX_NUMBER_OF_LOADED_SOUNDS, "invalid index");
	Asura_Wii_Sound_File_Data& xFileData = s_axSoundFileData[iLoadedIndex];
	xFileData.m_iFileIndex = s_uNumFilenames - 1;
	xFileData.m_uOffset = iOffset;
	xFileData.m_uSize = iSize;
	s_axSoundLoadStatus[iLoadedIndex] = Asura_Wii_Sound_Stream_Offset_Loaded;
}

void Asura_Wii_Sound::StreamingSoundCallback()
{
	Asura_Wii_Streaming_Sound_System::Update();
	for(int i = 0; i < ASURA_WII_MAX_NUMBER_OF_ACTIVE_SOUNDS; i++)
	{
		s_axActiveStreams[i].Update();
	}
}
