//
// Asura_Wii_Skybox.cpp
//
// Created by KOR - 08 January 2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

// This file's header.
#include "SpecialFX/Asura_Wii_Skybox.h"

// Asura headers
#include "Camera/Asura_Camera.h"
#include "Core/Asura_TextureManagement.h"
#include "Rendering/Asura_Render.h"
#include "SpecialFX/Asura_Skybox.h"

// Platform headers
#include "Rendering/Asura_GX.h"
#include "Rendering/Asura_GX_TextureManagement.h"

// Library headers
#include <revolution/gx.h>

const u_int uASURA_WII_SKYBOX_NUM_VERTICES = 8;

const float afXCoords[uASURA_WII_SKYBOX_NUM_VERTICES] = {-1.0f,-1.0f,+1.0f,+1.0f,-1.0f,-1.0f,+1.0f,+1.0f};
const float afYCoords[uASURA_WII_SKYBOX_NUM_VERTICES] = {+1.0f,+1.0f,+1.0f,+1.0f,-1.0f,-1.0f,-1.0f,-1.0f};
const float afZCoords[uASURA_WII_SKYBOX_NUM_VERTICES] = {-1.0f,+1.0f,+1.0f,-1.0f,-1.0f,+1.0f,+1.0f,-1.0f};
const float fASURA_WII_SKYBOX_SIZE = 1.0f;


int				Asura_Wii_Skybox::s_aiSkyBoxTextureIndex[6];
Asura_Vector_3	Asura_Wii_Skybox::s_axVertices[8];

void Asura_Wii_Skybox::DoRenderSetup()
{
	for (int iFace = 0; iFace < 6; ++iFace)
	{
		s_aiSkyBoxTextureIndex[iFace] = Asura_Texture_Management::AddTexture(Asura_Skybox::GetTexture(iFace));
	}
	float fOrient;
	Asura_Skybox::GetOrientationAroundYAxis(fOrient);
	const float fSin = Asura_Maths::Sine(fOrient);
	const float fCos = Asura_Maths::Cosine(fOrient);

	for(int iVertex = 0; iVertex < uASURA_WII_SKYBOX_NUM_VERTICES; ++iVertex)
	{
		const float fX  = afXCoords[iVertex]*fASURA_WII_SKYBOX_SIZE;
		const float fZ = afZCoords[iVertex]*fASURA_WII_SKYBOX_SIZE;
		s_axVertices[iVertex].x = fX * fCos + fZ * fSin;
		s_axVertices[iVertex].y = afYCoords[iVertex] * fASURA_WII_SKYBOX_SIZE;
		s_axVertices[iVertex].z = fZ * fCos - fX * fSin;
	}
	DCFlushRange(s_axVertices, uASURA_WII_SKYBOX_NUM_VERTICES * sizeof(Asura_Vector_3));
}

DEBUGVAR bool dbg_bForceInvert = false;
void Asura_Wii_Skybox::Render(bool bInvert /* = false */)
{
	if(dbg_bForceInvert)
	{
		bInvert = true;
	}
	int aaiFaceIndices[6][4] =
	{
		{1,0,3,2},
		{0,4,7,3},
		{2,6,5,1},
		{3,7,6,2},
		{1,5,4,0},
		{4,5,6,7}
	};
	float aafTexCoords[4][2] =
	{
		{ 1.f, 1.f, },
		{ 1.f, 0.f, },
		{ 0.f, 0.f, },
		{ 0.f, 1.f, },
	};

	Asura_Render::CurrentStates.RequestCullMode(ASURA_CULLMODE_NONE);
	Asura_Render::CurrentStates.RequestZBufferEnabled(true);
	Asura_Render::CurrentStates.RequestZBufferWriteEnabled(false);
	Asura_Render::CurrentStates.RequestZBufferCompareMode(ASURA_ZBUFFERCMP_LESSEQUAL);
	// Use offset to ensure that skybox is always rendered at farthest Z.
	GXSetZScaleOffset(1.f, 1.f);
	Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_OFF);

	GXClearVtxDesc();
	GXSetVtxDesc(GX_VA_POS,		GX_INDEX8);
	GXSetVtxDesc(GX_VA_CLR0,	GX_DIRECT);
	GXSetVtxDesc(GX_VA_TEX0,	GX_DIRECT);

	GXSetArray(GX_VA_POS,	(u32*)s_axVertices,	3*sizeof(f32));
	GXInvalidateVtxCache();

	GXSetNumChans(1);
	GXSetNumTexGens(1);
	GXSetNumTevStages(1);
	GXSetTevOrder(GX_TEVSTAGE0, GX_TEXCOORD0, GX_TEXMAP0, GX_COLOR0A0);
	GXSetTevOp(GX_TEVSTAGE0, GX_MODULATE);

	GXSetTevSwapMode(GX_TEVSTAGE0, GX_TEV_SWAP0, GX_TEV_SWAP0);
	// No lighting
	GXSetChanCtrl(GX_COLOR0, GX_DISABLE, GX_SRC_REG, GX_SRC_VTX, 0, GX_DF_CLAMP, GX_AF_NONE);
	GXSetChanCtrl(GX_ALPHA0, GX_DISABLE, GX_SRC_REG, GX_SRC_REG, 0, GX_DF_NONE, GX_AF_NONE);
	GXColor xBlack = {0, 0, 0, 0};
	GXColor xWhite = {0xff, 0xff, 0xff, 0xff};
	GXSetChanAmbColor(GX_COLOR0A0, xBlack);
	GXSetChanMatColor(GX_COLOR0A0, xWhite);	
	
	GXSetTexCoordGen(GX_TEXCOORD0, GX_TG_MTX2x4, GX_TG_TEX0, GX_IDENTITY);
	GXSetTevDirect(GX_TEVSTAGE0);

	// Use position matrix 1
	GXSetCurrentMtx(GX_PNMTX1);

	const Asura_RGBA xColour(Asura_Skybox::GetR() * (1.f / 255.f), Asura_Skybox::GetG() * (1.f / 255.f), Asura_Skybox::GetB() * (1.f / 255.f), 1.f);
	const u_int uColour = xColour.ToUIntRGBA(true);
	for(int iCam = 0; iCam < Asura_Camera::GetNumberOfCameras(); ++iCam)
	{
		if(Asura_Camera::IsCameraActive(iCam))
		{
			Asura_Camera::SetCameraIndex(iCam);
			Asura_Render::ApplyCameraSettings();

			const Mtx& xViewMatrix = Asura_GX::GetViewMatrix();
			Mtx xNewPosMtx;
			// Remove the translation component
			MTXCopy(xViewMatrix, xNewPosMtx);
			xNewPosMtx[0][3] = xNewPosMtx[1][3] = xNewPosMtx[2][3] = 0.f;
			if(bInvert)
			{
				// Flip Y.
				xNewPosMtx[0][1] = -xNewPosMtx[0][1];
				xNewPosMtx[1][1] = -xNewPosMtx[1][1];
				xNewPosMtx[2][1] = -xNewPosMtx[2][1];
			}
			GXLoadPosMtxImm(xNewPosMtx, GX_PNMTX1);

			for (int iFace = 0; iFace < 6; ++iFace)
			{
				if (s_aiSkyBoxTextureIndex[iFace] > 0)
				{
					Asura_Render::CurrentStates.RequestTextureIndex(0, s_aiSkyBoxTextureIndex[iFace]);
					if(!Asura_GX_Texture_Management::IsColourTextureLoaded()) { continue; }

					const bool bFlipTexCoords =
						(iFace == Asura_Skybox::ASURA_SKYBOX_BK && Asura_Skybox::GetBackTextureIsFrontUpsideDown()) ||
						(iFace == Asura_Skybox::ASURA_SKYBOX_RT && Asura_Skybox::GetRightTextureIsLeftUpsideDown());
					GXBegin(GX_TRIANGLEFAN, GX_VTXFMT3, 4);
					for( int iVertex = 0; iVertex < 4; ++iVertex )
					{
						GXPosition1x8(aaiFaceIndices[iFace][iVertex]);
						GXColor1u32(uColour);
						const float fV = aafTexCoords[iVertex][1];
						GXTexCoord2f32(aafTexCoords[iVertex][0], bFlipTexCoords ? (1.0f - fV) : fV);
					}
					GXEnd();
				}
			}
		}
	}
	// Reset position matrix.
	GXSetCurrentMtx(GX_PNMTX0);
	// Reset z offset.
	GXSetZScaleOffset(1.f, 0.f);
}