//
// Asura_Wii_THP.cpp
//
// Created by KOR - 09 October 2006
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

// This file's header
#include "THP/Asura_Wii_THP.h"

// Asura
#include "Debugging/Asura_Logfile.h"
#include "FileAccess/Asura_FileAccess.h"
#include "Rendering/Asura_Render.h"
#include "Sound/Asura_Sound.h"

// Asura Wii
#include "Core/Asura_Wii_MEM2.h"
#include "FileAccess/Asura_PlatformFileAccess.h"
#include "FileAccess/Asura_PlatformFileAccessUtils.h"
#include "HomeButton/Asura_Wii_HomeButtonMenu.h"
#include "Input/Asura_Wii_Input.h"
#include "Rendering/Asura_GX.h"
#include "THP/THPSimple.h"

#if defined(_DEBUG)
#include "Debugging/Asura_ScreenShot_System.h"
#endif

bool Asura_Wii_THP::s_bInitialised = false;

void Asura_Wii_THP::Initialise()
{
	Asura_Assert(!s_bInitialised, "THP system already initialised.");
	if(s_bInitialised) { return; }

	s_bInitialised = true;
}

int Asura_Wii_THP::PlayFMV(const char* szFilename, unsigned int uTrackNo /* = 0 */, bool bSkipAllowed /* = true */)
{
	if (!Asura_GX::ReadyToDraw())
	{
		Asura_OutputToDebugger(("Reached video while strap reminder still up, waiting.\n"));
	}

	while (!Asura_GX::ReadyToDraw() || Asura_Wii_HomeButtonMenu::IsActive())
	{
		Asura_Input::Update();
		Asura_Render::BeginFrame();
		Asura_Render::EndFrame();
	}

	if(!s_bInitialised) Initialise();

	char szFullFilename[128];
	strcpy( szFullFilename, "FMV/" );
	strcat( szFullFilename, szFilename );
	strcat( szFullFilename, ".thp" );

	if(!Asura_FileAccess::FileExists(szFullFilename))
	{
		return -1;
	}

	// Stop any music
	Asura_Sound_System::PauseAllSystems();
	Asura_Wii_HomeButtonMenu::Allow(false);
	
	// Render a couple of blank frames
	bool bPrevClrScr = Asura_Render::GetEnableScreenClear();
	Asura_Render::SetEnableScreenClear(true);
    Asura_Render::Overlay.Colour           = 0x00000000;
    Asura_Render::Overlay.TranslucencyMode = ASURA_TRANSLUCENCY_OFF;
    Asura_Render::Overlay.TextureIndex     = ASURA_TEXTURE_NONE;
    Asura_Render::Overlay.SetRectXY( 0.0f, 0.0f, 640.0f, 528.0f );
	Asura_Render::BeginFrame();
    Asura_Render::Overlay.Render();   	
	Asura_Render::EndFrame();
	Asura_Render::BeginFrame();
	Asura_Render::Overlay.Render();   	
	Asura_Render::EndFrame();

	//THPSimpleInit(THP_MODE_WITH_AX);
	// KOR - get a lot of glitches with AX, plus we only ever play standalone FMVs.
	THPSimpleInit(THP_MODE_ALONE);
	if (!THPSimpleOpen(szFullFilename))
	{
		Asura_Assertf(0, ("THPSimpleOpen failed to open %s", szFullFilename));
		return -1;
	}

	THPVideoInfo xVideoInfo;
	THPAudioInfo xAudioInfo;
	THPSimpleGetVideoInfo(&xVideoInfo);
	THPSimpleGetAudioInfo(&xAudioInfo);
	// Don't bother asserting if there's no soundtrack
	Asura_Assert(!xAudioInfo.sndNumTracks || uTrackNo < xAudioInfo.sndNumTracks, "Chosen track out of range");

	const GXRenderModeObj& xRenderMode = Asura_GX::GetRenderModeObject();

	const bool bWidescreen = (SCGetAspectRatio() == SC_ASPECT_RATIO_16x9);
	const u_int uHeight = bWidescreen ? xVideoInfo.ySize : (xVideoInfo.ySize * .75f);

	const u_int uX = (xRenderMode.fbWidth   - xVideoInfo.xSize) / 2;
	const u_int uY = (xRenderMode.efbHeight - uHeight) / 2;

	// Set aside work area
	const u_int uBufferSize = THPSimpleCalcNeedMemory();

	u_char* pcBuffer = static_cast<u_char*>(ASURA_WII_MEM2_ALLOC(uBufferSize));

	THPSimpleSetBuffer(pcBuffer);

	if (!THPSimplePreLoad(THP_PLAY_ONESHOT))
	{
		Asura_Assertf(0, ("THPSimplePreLoad failed to load %s", szFullFilename));
		return -1;
	}

	bool bFirstFrame = true;
	// Render loop
	while(1)
	{
		Asura_Input::Update();
		Asura_GX::BeginFrame();

		// Draw the next frame.
		if (THPSimpleDecode(uTrackNo) == THP_VIDEO_DECODE_FAIL)
		{
			Asura_Assert(0, "Failed to decode audio");
		}

		const u_int uFrameDrawn = THPSimpleDrawCurrentFrame(&xRenderMode, uX, uY, xVideoInfo.xSize, uHeight);

		Asura_Wii_HomeButtonMenu::Render();

#if defined(_DEBUG) && defined(ASURA_USE_SCREENSHOT_SYSTEM)
		if(Asura_Input::IsDebouncedKeyPressed(0, ASURA_WII_INPUT_2))
		{
			Asura_ScreenShot_System::TakeNormalScreenShot();
		}
#endif

		Asura_GX::EndFrame();

		// Now do render again so that we are locked to 30/25fps
		Asura_GX::BeginFrame();
		THPSimpleDrawCurrentFrame(&xRenderMode, uX, uY, xVideoInfo.xSize, uHeight);
		Asura_Wii_HomeButtonMenu::Render();
		Asura_GX::EndFrame();

		if(bFirstFrame)
		{
			// Allow audio output
			THPSimpleAudioStart();
			bFirstFrame = false;
		}

		// Always allow skip in debug, because it gets irritating waiting.
		bool bSkip = false;
#ifndef _DEBUG
		if(bSkipAllowed)
#endif
		{
			// Check the input status
			if(Asura_Input::IsDebouncedKeyPressed(0, ASURA_WII_INPUT_A))
			{
				// User pressed the skip button.
				bSkip = true;
			}
		}
		const u_int uTotalFrames = THPSimpleGetTotalFrame();
		if(bSkip || uFrameDrawn >= uTotalFrames - 1)
		{
			THPSimpleAudioStop();
			THPSimpleLoadStop();
			THPSimpleClose();
			break;
		}
		if(Asura_PlatformFileAccessUtils::CheckDiscStatus())
		{
			// Restart video?
		}
	}

	ASURA_WII_MEM2_FREE(pcBuffer);

	THPSimpleQuit();

	Asura_Render::SetEnableScreenClear(bPrevClrScr);
	Asura_Sound_System::UnpauseAllSystems();
	Asura_Wii_HomeButtonMenu::Allow(true);

//	GXSetDrawSyncCallback(pOldDrawSyncCB);
	GXSetTevDirect(GX_TEVSTAGE0);
	GXSetTevDirect(GX_TEVSTAGE1);
	GXSetTevDirect(GX_TEVSTAGE2);
	GXSetTevDirect(GX_TEVSTAGE3);

	Asura_GX::InitVertexFormats();

	return 0;
}
