//////////////////////////////////////////////////////////////////////////////
//
// Asura_Xbox360_Botanicals.cpp
//
// Copyright (c) 2006 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

/******************************************************************************/
/* Included Files                                                             */
/******************************************************************************/

#include "Core/Asura.h"

#include "Botanicals/Asura_Xbox360_Botanicals.h"

// Asura Platform-independent
#include "Camera/Asura_Camera.h"
#include "Core/Memory/Asura_Mem.h"
#include "Core/Asura_MaterialFlags.h"
#include "Core/Asura_Material_System.h"
#include "Core/Asura_MaterialResponse_System.h"

#if defined( ASURA_USE_DEBUGOVERLAY )
#include "Debugging/Asura_DebugOverlay.h"
#endif / /#if defined( ASURA_USE_DEBUGOVERLAY )

#if defined( ASURA_SUPPORT_COMMAND_CONSOLE)
#include "Interface/Asura_CommandConsole.h"
#endif // #if defined( ASURA_SUPPORT_COMMAND_CONSOLE)

#include "Rendering/Asura_Module.h"
#include "Debugging/Asura_Platform_GPUProfile.h"
#include "Rendering/Asura_Render.h"

// Asura Xbox 360
#include "Rendering/Asura_Xbox360_Effect_System.h"
#include "Rendering/Asura_Xbox360_Graphics.h"
#include "Rendering/Asura_Xbox360_TextureManagement.h"

// Asura Xbox 360 Shaders
#include "Botanicals_Geometry.h"
#include "Botanicals_Billboards.h"
#include "Botanicals_Imposters.h"

// Misc
#include <xtl.h>

/******************************************************************************/
/* Constants                                                                  */
/******************************************************************************/

#if 1
#define INSTANCE_VD																														\
	{ 2,  0, D3DDECLTYPE_DEC3N,		D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_TEXCOORD, 1 },	/* Instance world transform orientation */		\
	{ 2,  4, D3DDECLTYPE_DEC3N,		D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_TEXCOORD, 2 },													\
	{ 2,  8, D3DDECLTYPE_DEC3N,		D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_TEXCOORD, 3 },													\
	{ 2, 12, D3DDECLTYPE_FLOAT16_4, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_TEXCOORD, 4 },	/* Instance world transform position + alpha */	\
	{ 2, 20, D3DDECLTYPE_FLOAT16_4, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_TEXCOORD, 5 },	/* Red spherical harmonic */					\
	{ 2, 28, D3DDECLTYPE_FLOAT16_4, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_TEXCOORD, 6 },	/* Green spherical harmonic */					\
	{ 2, 36, D3DDECLTYPE_FLOAT16_4,	D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_TEXCOORD, 7 },	/* Blue spherical harmonic */					\
	{ 2, 44, D3DDECLTYPE_FLOAT16_4,	D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_TEXCOORD, 8 },	/* Bone Position Offset + scale */
#else
#define INSTANCE_VD																													\
	{ 2,  0, D3DDECLTYPE_FLOAT16_4, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_TEXCOORD, 1 }, 	/* Instance world transformation matrix */	\
	{ 2,  8, D3DDECLTYPE_FLOAT16_4, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_TEXCOORD, 2 },												\
	{ 2, 16, D3DDECLTYPE_FLOAT16_4, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_TEXCOORD, 3 },												\
	{ 2, 24, D3DDECLTYPE_FLOAT16_4, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_TEXCOORD, 4 }, 	/* Red spherical harmonic */				\
	{ 2, 32, D3DDECLTYPE_FLOAT16_4, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_TEXCOORD, 5 }, 	/* Green spherical harmonic */				\
	{ 2, 40, D3DDECLTYPE_FLOAT16_4, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_TEXCOORD, 6 }, 	/* Blue spherical harmonic */				\
	{ 2, 48, D3DDECLTYPE_FLOAT1,    D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_TEXCOORD, 7 }, 	/* Instance alpha */						\
	{ 2, 52, D3DDECLTYPE_FLOAT1,    D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_TEXCOORD, 8 }, 	/* Instance scale */						\
	{ 2, 56, D3DDECLTYPE_FLOAT3,    D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_TEXCOORD, 9 }, 	/* Bone Position Offset */
#endif

const Direct3D_VertexElement axGEOMETRY_VERTEX_DECLARATION[] =
{
	// Geometry data - remember to update uGEOMETRY_VERTEX_SIZE in Asura_Chunk_ResourceFile_Xbox360_Botanicals.cpp
	// if this changes
	{ 0,  0, D3DDECLTYPE_FLOAT3,    D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_POSITION, 0 },
	{ 0, 12, D3DDECLTYPE_FLOAT16_2, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_TEXCOORD, 0 },
	{ 0, 16, D3DDECLTYPE_DEC3N,     D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_NORMAL,   0 },
	{ 0, 20, D3DDECLTYPE_DEC3N,     D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_TANGENT,  0 },
	{ 0, 24, D3DDECLTYPE_DEC3N,     D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_BINORMAL, 0 },
	{ 0, 28, D3DDECLTYPE_FLOAT1,	D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_BLENDWEIGHT, 0 }, // Bome blend Weight

	// Geometry indices
	{ 1,  0, D3DDECLTYPE_UINT1,     D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_POSITION, 1 }, // TODO: can we get 16-bit indices here?

	// Instance data
	INSTANCE_VD

	D3DDECL_END()
};

const Direct3D_VertexElement axBILLBOARDS_VERTEX_DECLARATION[] =
{
	// Per-vertex data - remember to update uBILLBOARDS_VERTEX_SIZE in Asura_Chunk_ResourceFile_Xbox360_Botanicals.cpp
	// if this changes
	{ 0, 0, D3DDECLTYPE_FLOAT16_2, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_POSITION, 0 }, // XY offset of each vertex from billboard centre
	{ 0, 4, D3DDECLTYPE_FLOAT16_2, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_TEXCOORD, 0 }, // Tex coord

	// Per-billboard data
	{ 1, 0, D3DDECLTYPE_FLOAT16_4, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_POSITION, 1 }, // XYZ offset of each billboard from cluster location
	{ 1, 8, D3DDECLTYPE_DEC3N,     D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_NORMAL,   0 }, // Normal for this billboard in object space	
	{ 1, 12,D3DDECLTYPE_DEC3N,     D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_TANGENT,  0 },
	{ 1, 16,D3DDECLTYPE_DEC3N,     D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_BINORMAL, 0 },
	{ 1, 20,D3DDECLTYPE_FLOAT1,	   D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_BLENDWEIGHT, 0 }, // Bome blend Weight

	// Per-cluster instance data
	INSTANCE_VD

	D3DDECL_END()
};



const Direct3D_VertexElement axIMPOSTERS_VERTEX_DECLARATION[] =
{
	//	Asura_HalfFloat afPos[4];						// relative to instance, xyz
	//	Asura_HalfFloat	afAngleFade[2];					// opaque range, fade range
	//	Asura_HalfFloat afWidthHeight[2];				// w,h		
	//	Asura_HalfFloat afUVSet[4];						// U0,V0,U1,V1
	//	uint32			uNormal;	
	//	uint32			uTangent;	
	//	uint32			uBinormal;	

	// Per-imposter-angle data	
	{ 0, 0, D3DDECLTYPE_FLOAT16_4, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_POSITION, 0 }, // XY offset of each vertex from billboard centre
	{ 0, 8, D3DDECLTYPE_FLOAT16_2, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_POSITION, 1 }, // OpaqueAngle, FadeAngle
	{ 0,12, D3DDECLTYPE_FLOAT16_2, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_POSITION, 2 }, // W,H
	{ 0,16, D3DDECLTYPE_FLOAT16_4, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_TEXCOORD, 0 }, // U0,V0,U1,V1
	{ 0,24, D3DDECLTYPE_DEC3N,     D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_NORMAL,   0 }, // Normal for calculating angle fades
	{ 0,28, D3DDECLTYPE_DEC3N,     D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_TANGENT,  0 },
	{ 0,32, D3DDECLTYPE_DEC3N,     D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_BINORMAL, 0 },

	// Per-cluster instance data
	INSTANCE_VD

	D3DDECL_END()
};



const char* const aszELEMENT_TYPE_NAMES[ Asura_Xbox360_Botanicals_Element::TYPE_NUM_TYPES ] =
{
	"1-sided Geometry",
	"2-sided Geometry",
	"Billboards",
	"Imposter",
};

#if defined( ASURA_SUPPORT_COMMAND_CONSOLE)
static int s_iOnlyRenderTemplate = -1;
#endif // #if defined( ASURA_SUPPORT_COMMAND_CONSOLE)
#if defined( ASURA_USE_DEBUGOVERLAY )
static int s_iDebugDrawInstances = -1;
#endif // #if defined( ASURA_USE_DEBUGOVERLAY )

/******************************************************************************/
/* Asura_Xbox360_Botanicals_LOD_Geometry Class Functions                      */
/******************************************************************************/

Asura_Xbox360_Botanicals_LOD_Geometry::Asura_Xbox360_Botanicals_LOD_Geometry()
: m_uMaterialResponseHashID( ASURA_HASH_ID_UNSET )
, m_uNumIndices(0)
, m_uNumTriangles(0)
, m_uNumVertices(0)
{
}

Asura_Xbox360_Botanicals_LOD_Geometry::~Asura_Xbox360_Botanicals_LOD_Geometry()
{
	m_xVB.Destroy();
	m_xIB.Destroy();
}

/******************************************************************************/
/* Asura_Xbox360_Botanicals_LOD_Billboards Class Functions                    */
/******************************************************************************/

Asura_Xbox360_Botanicals_LOD_Billboards::Asura_Xbox360_Botanicals_LOD_Billboards()
: m_uMaterialResponseHashID( ASURA_HASH_ID_UNSET )
, m_uNumVertices(0)
{
}

Asura_Xbox360_Botanicals_LOD_Billboards::~Asura_Xbox360_Botanicals_LOD_Billboards()
{
	m_xBillboardVB.Destroy();
	m_xVertexVB.Destroy();
}


/******************************************************************************/
/* Asura_Xbox360_Botanicals_LOD_Imposter Class Functions					  */
/******************************************************************************/

Asura_Xbox360_Botanicals_LOD_Imposter::Asura_Xbox360_Botanicals_LOD_Imposter()
: m_apxAngles( NULL )
, m_iNumImposterAngles(0)
{
}

Asura_Xbox360_Botanicals_LOD_Imposter::~Asura_Xbox360_Botanicals_LOD_Imposter()
{
	if (m_apxAngles)
	{	
		delete [] m_apxAngles;
	}
	m_xImposterAngleDataVB.Destroy();
}



/******************************************************************************/
/* Asura_Xbox360_Botanicals_Element Class Functions                           */
/******************************************************************************/

Asura_Xbox360_Botanicals_Element::Asura_Xbox360_Botanicals_Element()
: m_pxGeometryLODs( NULL )
, m_pxLODData( NULL )
, m_eType( TYPE_GEOMETRY_1SIDED )
, m_iNumLODs(0)
{
}

Asura_Xbox360_Botanicals_Element::~Asura_Xbox360_Botanicals_Element()
{
	switch( m_eType )
	{
	case TYPE_GEOMETRY_1SIDED:
	case TYPE_GEOMETRY_2SIDED:
		{
			delete[] m_pxGeometryLODs;
			break;
		}

	case TYPE_BILLBOARDS:
		{
			delete[] m_pxBillboardLODs;
			break;
		}

	case TYPE_IMPOSTER:
		{
			delete[] m_pxImposterLODs;
			break;
		}

	default:
		{
			Asura_Assert( false, "Unknown botanicals element type." );
			break;
		}
	}

	delete[] m_pxLODData;
}

/******************************************************************************/
/* Asura_Xbox360_Botanicals_Shape Class Functions                             */
/******************************************************************************/

Asura_Xbox360_Botanicals_Shape::Asura_Xbox360_Botanicals_Shape()
: m_pxElements( NULL )
, m_uHashID( ASURA_HASH_ID_UNSET )
, m_iNumElements(0)
#if defined( ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS) && 1 == ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS
	, m_uSectionCount( 0)
#endif
#if defined( ASURA_BOTANICAL_SUPPORT_INTERACTIVE)
	, m_pxBasicInteractive( NULL)
#endif

{
}

Asura_Xbox360_Botanicals_Shape::~Asura_Xbox360_Botanicals_Shape()
{
	delete[] m_pxElements;
}

/******************************************************************************/
/* Asura_Xbox360_Botanicals_Template Class Functions                          */
/******************************************************************************/

Asura_Xbox360_Botanicals_Template::Asura_Xbox360_Botanicals_Template()
: m_pxShape( NULL )
, m_iUID(0)
, m_iNumInstances(0)
, m_pxVisibilityQueries( NULL )
{
}

Asura_Xbox360_Botanicals_Template::~Asura_Xbox360_Botanicals_Template()
{
	delete[] m_pxInstances;
	delete[] m_pxVisibilityQueries;
}

/******************************************************************************/
/* Asura_Xbox360_Botanicals_System Static Class Data                          */
/******************************************************************************/

// Instance buffer
Asura_Xbox360_VertexBuffer Asura_Xbox360_Botanicals_System::s_axInstanceVBs[NUM_INSTANCE_BUFFERS];
int						   Asura_Xbox360_Botanicals_System::s_iCurrentInstanceBuffer = 0;
// Geometry

Asura_Xbox360_Effect*					Asura_Xbox360_Botanicals_System::s_pxGeometryEffect = NULL;
Asura_Xbox360_Effect_System::Handle		Asura_Xbox360_Botanicals_System::s_xGeometryInstanceSizeHandle = 0;
Asura_Xbox360_Effect_System::Handle		Asura_Xbox360_Botanicals_System::s_xGeometryFirstInstanceHandle = 0;
Asura_Xbox360_Effect_System::Handle		Asura_Xbox360_Botanicals_System::s_xGeometryCullModeHandle = 0;
Asura_Xbox360_Effect_System::Handle		Asura_Xbox360_Botanicals_System::s_xGeometryDualSidedLightHandle = 0;
Direct3D_VertexDeclaration*				Asura_Xbox360_Botanicals_System::s_pxGeometryVD = NULL;
Asura_Xbox360_MaterialResponse_Handles	g_xGeometryMaterialResponseHandles;


// Billboards

Asura_Xbox360_Effect*					Asura_Xbox360_Botanicals_System::s_pxBillboardsEffect = NULL;
Asura_Xbox360_Effect_System::Handle		Asura_Xbox360_Botanicals_System::s_xBillboardsInstanceSizeHandle = 0;
Asura_Xbox360_Effect_System::Handle		Asura_Xbox360_Botanicals_System::s_xBillboardsFirstInstanceHandle = 0;
Asura_Xbox360_Effect_System::Handle		Asura_Xbox360_Botanicals_System::s_xBillboardsCameraUpHandle = 0;
Asura_Xbox360_Effect_System::Handle		Asura_Xbox360_Botanicals_System::s_xBillboardsCameraRightHandle = 0;
Asura_Xbox360_Effect_System::Handle		Asura_Xbox360_Botanicals_System::s_xBillboardsCameraForwardsHandle = 0;
Direct3D_VertexDeclaration*				Asura_Xbox360_Botanicals_System::s_pxBillboardsVD = NULL;
Asura_Xbox360_MaterialResponse_Handles	g_xBillboardsMaterialResponseHandles;

// Imposters

Asura_Xbox360_Effect*					Asura_Xbox360_Botanicals_System::s_pxImpostersEffect = NULL;
Asura_Xbox360_Effect_System::Handle		Asura_Xbox360_Botanicals_System::s_xImpostersNumAnglesHandle = 0;
Asura_Xbox360_Effect_System::Handle		Asura_Xbox360_Botanicals_System::s_xImpostersFirstInstanceHandle = 0;
Asura_Xbox360_Effect_System::Handle		Asura_Xbox360_Botanicals_System::s_xImpostersCameraUpHandle = 0;
Asura_Xbox360_Effect_System::Handle		Asura_Xbox360_Botanicals_System::s_xImpostersCameraRightHandle = 0;
Asura_Xbox360_Effect_System::Handle		Asura_Xbox360_Botanicals_System::s_xImpostersCameraForwardsHandle = 0;
Direct3D_VertexDeclaration*				Asura_Xbox360_Botanicals_System::s_pxImpostersVD = NULL;
Asura_Xbox360_MaterialResponse_Handles	g_xImpostersMaterialResponseHandles;

// Shape and template lists

Asura_Xbox360_Botanicals_Shape* Asura_Xbox360_Botanicals_System::s_pxShapes = NULL;
Asura_Xbox360_Botanicals_Template* Asura_Xbox360_Botanicals_System::s_pxTemplates = NULL;

u_int Asura_Xbox360_Botanicals_System::s_uNumShapes = 0;
u_int Asura_Xbox360_Botanicals_System::s_uNumTemplates = 0;

// Render data
u_int Asura_Xbox360_Botanicals_System::s_uNumInstancesRendered = 0;
u_int Asura_Xbox360_Botanicals_System::s_uNumTrianglesRendered = 0;

/******************************************************************************/
/* Asura_Xbox360_Botanicals_System Static Class Functions                     */
/******************************************************************************/

////////////////////////////////////////////////////////////////////////////////
//
// Asura_Xbox360_Botanicals_System::Initialise()
//
// Initialises the botanicals system on Xbox 360. Called at program startup
//
// Parameters:		None
// Return value:	None
//
////////////////////////////////////////////////////////////////////////////////



void Asura_Xbox360_Botanicals_System::Initialise()
{
	// Normal geometry
	// ----------------------------------------------------------------------------------------------------------------
	s_pxGeometryEffect = Asura_Xbox360_Effect_System::CreateEffectFromMemory( g_auBotanicals_Geometry );
	s_xGeometryInstanceSizeHandle = s_pxGeometryEffect->GetParameterHandle( "g_fInstanceSize" );
	s_xGeometryFirstInstanceHandle = s_pxGeometryEffect->GetParameterHandle( "g_fFirstInstance" );	
	s_xGeometryFirstInstanceHandle = s_pxGeometryEffect->GetParameterHandle( "g_fFirstInstance" );
	s_xGeometryFirstInstanceHandle = s_pxGeometryEffect->GetParameterHandle( "g_fFirstInstance" );
	s_xGeometryCullModeHandle = s_pxGeometryEffect->GetParameterHandle( "g_eCullMode" );
	s_xGeometryDualSidedLightHandle = s_pxGeometryEffect->GetParameterHandle( "g_fDualSidedLightingLerp" );
	g_xGeometryMaterialResponseHandles.InitialiseFromEffect(*s_pxGeometryEffect);

	Asura_Xbox360_Graphics::GetDevice()->CreateVertexDeclaration( axGEOMETRY_VERTEX_DECLARATION, &s_pxGeometryVD );
	Asura_Assert( s_pxGeometryVD, "Failed to create botanicals vertex declaration." );


	// Billboard clusters
	// ----------------------------------------------------------------------------------------------------------------
	s_pxBillboardsEffect			  = Asura_Xbox360_Effect_System::CreateEffectFromMemory( g_auBotanicals_Billboards );
	s_xBillboardsInstanceSizeHandle   = s_pxBillboardsEffect->GetParameterHandle( "g_fInstanceSize" );
	s_xBillboardsFirstInstanceHandle  = s_pxBillboardsEffect->GetParameterHandle( "g_fFirstInstance" );	
	s_xBillboardsCameraUpHandle		  = s_pxBillboardsEffect->GetParameterHandle( "g_xCameraUpVector" );	
	s_xBillboardsCameraRightHandle	  = s_pxBillboardsEffect->GetParameterHandle( "g_xCameraRightVector" );	
	s_xBillboardsCameraForwardsHandle = s_pxBillboardsEffect->GetParameterHandle( "g_xCameraForwardsVector" );	
	g_xBillboardsMaterialResponseHandles.InitialiseFromEffect(*s_pxBillboardsEffect);

	Asura_Xbox360_Graphics::GetDevice()->CreateVertexDeclaration( axBILLBOARDS_VERTEX_DECLARATION, &s_pxBillboardsVD );
	Asura_Assert( s_pxBillboardsVD, "Failed to create botanicals vertex declaration." );


	// Imposters
	// ----------------------------------------------------------------------------------------------------------------
	s_pxImpostersEffect				  = Asura_Xbox360_Effect_System::CreateEffectFromMemory( g_auBotanicals_Imposters );
	s_xImpostersNumAnglesHandle       = s_pxImpostersEffect->GetParameterHandle( "g_fNumAngles" );
	s_xImpostersFirstInstanceHandle   = s_pxImpostersEffect->GetParameterHandle( "g_fFirstInstance" );	
	s_xImpostersCameraUpHandle		  = s_pxImpostersEffect->GetParameterHandle( "g_xCameraUpVector" );	
	s_xImpostersCameraRightHandle	  = s_pxImpostersEffect->GetParameterHandle( "g_xCameraRightVector" );	
	s_xImpostersCameraForwardsHandle  = s_pxImpostersEffect->GetParameterHandle( "g_xCameraForwardsVector" );	
	g_xImpostersMaterialResponseHandles.InitialiseFromEffect(*s_pxImpostersEffect);

	Asura_Xbox360_Graphics::GetDevice()->CreateVertexDeclaration( axIMPOSTERS_VERTEX_DECLARATION, &s_pxImpostersVD );
	Asura_Assert( s_pxImpostersVD, "Failed to create botanicals vertex declaration." );

#if defined( ASURA_SUPPORT_COMMAND_CONSOLE)
	Asura_CommandConsole::AddVar( L"Render.Botanical.OnlyShowTemplate", &s_iOnlyRenderTemplate, -1, 128, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
#endif // #if defined( ASURA_SUPPORT_COMMAND_CONSOLE)

#if defined( ASURA_USE_DEBUGOVERLAY )
	Asura_DebugOverlay::RegisterMode( "Botanicals", ASURA_DO_BOTANICALS, DoDebugOverlayCallback, NULL );
#endif // #if defined( ASURA_USE_DEBUGOVERLAY )
}


////////////////////////////////////////////////////////////////////////////////
//
// Asura_Xbox360_Botanicals_System::Reset()
//
// Resets the botanicals system on Xbox 360. Called between level loads
//
// Parameters:		None
// Return value:	None
//
////////////////////////////////////////////////////////////////////////////////

void Asura_Xbox360_Botanicals_System::Reset()
{
	for(int i=0;i<NUM_INSTANCE_BUFFERS;i++)
	{	
		s_axInstanceVBs[i].Destroy();
	}

	delete[] s_pxShapes;
	delete[] s_pxTemplates;

	s_pxShapes = NULL;
	s_pxTemplates = NULL;

	s_uNumShapes = 0;
	s_uNumTemplates = 0;
}

////////////////////////////////////////////////////////////////////////////////
//
// Asura_Xbox360_Botanicals_System::Shutdown()
//
// Shuts down the botanicals system on Xbox 360. Called at program end (i.e. never)
//
// Parameters:		None
// Return value:	None
//
////////////////////////////////////////////////////////////////////////////////

void Asura_Xbox360_Botanicals_System::Shutdown()
{
	Reset();

	Asura_Xbox360_Effect_System::Destroy( s_pxGeometryEffect );

	s_pxGeometryVD->Release();
	s_pxGeometryVD = NULL;

	Asura_Xbox360_Effect_System::Destroy( s_pxBillboardsEffect );

	s_pxBillboardsVD->Release();
	s_pxBillboardsVD = NULL;
}

////////////////////////////////////////////////////////////////////////////////
//
// Asura_Xbox360_Botanicals_System::CalculateLODs()
//
// Fills out the instance VB with info on each LOD to render for the current camera
// position. TODO: do this in a separate thread
//
// Parameters:		None
// Return value:	None
//
////////////////////////////////////////////////////////////////////////////////

class Asura_Xbox360_Botanicals_InstanceCache
{
public:
	float m_fDistance;
	const Asura_Xbox360_Botanicals_Template* m_pxCurrTemplate;
	const Asura_Xbox360_Botanicals_Instance* m_pxCurrInstance;
#if defined( ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA) && 1 == ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA
	Asura_BGR_DynamicInstance* m_pxDynamicInstanceData;
#endif
};

Asura_Xbox360_Botanicals_Instance::RenderData* Asura_Xbox360_Botanicals_System::CalculateLODsCache( Asura_Xbox360_Botanicals_Element& xCurrElement,
                                                                                                    const Asura_Xbox360_Botanicals_InstanceCache* paxInstanceCache,
                                                                                                    const u_int uInstanceCacheMax,
                                                                                                    Asura_Xbox360_Botanicals_Instance::RenderData* pxCurrRenderData,
                                                                                                    Asura_Xbox360_Botanicals_Instance::RenderData* const pxRenderDataEnd )
{
#if defined( ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS) && 1 == ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS
	const Asura_BGR_Section_ID xCurrElementSectionID = xCurrElement.m_xSectionID;
	const Asura_BGR_Version_ID xCurrElementVersionID = xCurrElement.m_xVersionID;
#endif
	float fDistanceMin = -1e30;

	for( int j = xCurrElement.m_iNumLODs - 1; 0 <= j; --j )
		//	for( int j = 0; j < xCurrElement.m_iNumLODs; ++j )
	{
		Asura_Xbox360_Botanicals_LODData& xCurrLODData = xCurrElement.m_pxLODData[j];
		xCurrLODData.m_uNumVisibleInstances = 0;

		if (fDistanceMin > xCurrLODData.m_fMinDistance && uInstanceCacheMax > 0)
		{	
			// Assert now gives name of offending template
			Asura_Assertf( false, ("LOD Data out of order in botanical template '%s'. Please correct the data!", paxInstanceCache[0].m_pxCurrTemplate->m_xName.GetString()));
		}

		fDistanceMin = Asura_Maths::Max( fDistanceMin, xCurrLODData.m_fMinDistance);

		for( u_int uInstanceCacheIndex = 0; uInstanceCacheIndex < uInstanceCacheMax; uInstanceCacheIndex += 1)
		{
            if( pxCurrRenderData >= pxRenderDataEnd )
            {
                Asura_Assert( false, "Rendering too many botanicals this frame!" );
                return pxCurrRenderData;
            }

			const Asura_Xbox360_Botanicals_InstanceCache& xInstanceCache = paxInstanceCache[ uInstanceCacheIndex];

#if (defined( ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS) && 1 == ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS) \
	||  (defined( ASURA_BOTANICAL_SUPPORT_INTERACTIVE) && 1 == ASURA_BOTANICAL_SUPPORT_INTERACTIVE)
			const Asura_Xbox360_Botanicals_Template& xCurrTemplate = *(xInstanceCache.m_pxCurrTemplate);
#endif

			{
				const Asura_Xbox360_Botanicals_Instance& xCurrInstance = *(xInstanceCache.m_pxCurrInstance);

#if defined( ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS) && 1 == ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS
				Asura_BGR_DynamicInstance* pxDynamicInstanceData = xInstanceCache.m_pxDynamicInstanceData;
				if( true == xCurrTemplate.HasDynamicSectionVersionData() && NULL != pxDynamicInstanceData)
				{
					const Asura_BGR_Version_ID xInstanceVersionID = xCurrTemplate.GetDynamicSectionVersion( pxDynamicInstanceData, xCurrElementSectionID);
					if( xCurrElementVersionID != xInstanceVersionID)
					{
						continue;
					}
				}
#endif
				float fDist = xInstanceCache.m_fDistance;

				// Imposters don't have a draw-distance limit (no module system to limit them...)
				if (xCurrElement.m_eType == Asura_Xbox360_Botanicals_Element::TYPE_IMPOSTER)
				{					
					if (fDist > xCurrLODData.m_fMaxDistance )
					{
						fDist = xCurrLODData.m_fMaxDistance;
					}
				}

				if( xCurrLODData.m_fMinDistance <= fDist && fDist <= xCurrLODData.m_fMaxDistance )
				{
					// Calculate the fade value. This isn't a simple alpha blend; we fade botanicals
					// in by increasing their alpha level against the alpha test, combined with a
					// speckled texture alpha channel

					float fAlpha = 0.0f;
					if( fDist < xCurrLODData.m_fMinFullAlphaDistance )
					{
						fAlpha = (fDist - xCurrLODData.m_fMinDistance) / (xCurrLODData.m_fMinFullAlphaDistance - xCurrLODData.m_fMinDistance);
						fAlpha = 1.0f - 0.5f * fAlpha;
					}
					else if( fDist <= xCurrLODData.m_fMaxFullAlphaDistance )
					{
						fAlpha = 0.5f;
					}
					else
					{
						fAlpha = (xCurrLODData.m_fMaxDistance - fDist) / (xCurrLODData.m_fMaxDistance - xCurrLODData.m_fMaxFullAlphaDistance);
						fAlpha = 1.0f - 0.5f * fAlpha;
					}

					// Create the data in normal memory first, then copy the whole thing to write-combined
					Asura_Xbox360_Botanicals_Instance::RenderData xInstanceRenderData;
					Asura_Mem::Copy( &xInstanceRenderData, &xCurrInstance.m_xRenderData, sizeof( xCurrInstance.m_xRenderData ) );

					xInstanceRenderData.m_fAlpha = fAlpha;

#if defined( ASURA_BOTANICAL_SUPPORT_INTERACTIVE) && 1 == ASURA_BOTANICAL_SUPPORT_INTERACTIVE
					if( true == xCurrTemplate.HasDynamicInteractiveData())
					{
						const Asura_BGR_InteractiveBasic* pxInteractiveBasic = xCurrTemplate.GetDynamicInteractiveData( xInstanceCache.m_pxDynamicInstanceData);
						xInstanceRenderData.m_afBoneOffset[0] = pxInteractiveBasic->m_afPosition[0];
						xInstanceRenderData.m_afBoneOffset[1] = pxInteractiveBasic->m_afPosition[1];
						xInstanceRenderData.m_afBoneOffset[2] = pxInteractiveBasic->m_afPosition[2];
					}
					else
					{
						xInstanceRenderData.m_afBoneOffset[0] = 0.0f;
						xInstanceRenderData.m_afBoneOffset[1] = 0.0f;
						xInstanceRenderData.m_afBoneOffset[2] = 0.0f;
					}
#endif						

					// N.B.: since vertex buffers are allocated in write-combined memory (i.e. uncached), we can't
					// use the faster XMemCpy(), which is what we'll get if we use Asura_Mem::Copy()
					memcpy( pxCurrRenderData, &xInstanceRenderData, sizeof( xInstanceRenderData ) );

					++pxCurrRenderData;
					++xCurrLODData.m_uNumVisibleInstances;
				}
			}
		}
	}

    return pxCurrRenderData;
}

float Asura_Xbox360_Botanicals_System_Distance_Scale = 1.0f;

void Asura_Xbox360_Botanicals_System::CalculateLODs()
{
	Asura_Xbox360_Botanicals_Instance::RenderData* const pxRenderDataStart = static_cast< Asura_Xbox360_Botanicals_Instance::RenderData* >( s_axInstanceVBs[s_iCurrentInstanceBuffer].Lock() );
	Asura_Xbox360_Botanicals_Instance::RenderData* pxCurRenderData = pxRenderDataStart;
	Asura_Xbox360_Botanicals_Instance::RenderData* const pxRenderDataEnd = pxRenderDataStart + ASURA_BOTANICALS_MAX_ELEMENT_INSTANCES_PER_FRAME;

	// Don't assert here since there won't be an instance buffer if there aren't any botanicals in the level
	if( !pxCurRenderData ) return;

	PIXBeginNamedEvent( D3DCOLOR_XRGB( 0x55, 0x55, 0x55 ), "CalculateLODs" );

	Asura_Vector_3 xCameraPosition = *Asura_Camera::GetPosition();

	int iTemplateOffsetToNextBase = 1;
#if defined( ASURA_SUPPORT_COMMAND_CONSOLE)
	int iThisTemplate = 0;
#endif // #if defined( ASURA_SUPPORT_COMMAND_CONSOLE)
	for( u_int u = 0; u < s_uNumTemplates; u += iTemplateOffsetToNextBase)
	{
		Asura_Xbox360_Botanicals_Template& xCurrTemplateBase = s_pxTemplates[u];

		iTemplateOffsetToNextBase = xCurrTemplateBase.m_iOffsetToNextBase;

		if( !xCurrTemplateBase.m_pxShape ) continue;

#if defined( ASURA_SUPPORT_COMMAND_CONSOLE)
		if ( ( s_iOnlyRenderTemplate != -1 ) && ( iThisTemplate != s_iOnlyRenderTemplate ) )
		{
			iThisTemplate++;
			continue;
		}
		iThisTemplate++;
#endif // #if defined( ASURA_SUPPORT_COMMAND_CONSOLE)

		Asura_Xbox360_Botanicals_Shape& xCurrShape = *xCurrTemplateBase.m_pxShape;

		float fShapeRadius = xCurrShape.GetBoundingRadius();

		const u_int uInstanceCacheMax = 2 * 1024;
		static Asura_Xbox360_Botanicals_InstanceCache axInstanceCache[ uInstanceCacheMax];

		u_int uInstanceCacheIndex = 0;

		{
			struct TemplateCache
			{
			public:
				u_int m_uTemplateIndex;
				float m_fDistSqd;
			};
			const u_int uTemlateCacheSizeMax = 64;
			TemplateCache axTemplateCache[ uTemlateCacheSizeMax];
			u_int uTemplateCacheSize = 0;

			{
				for( int iTemplateOffset = 0; iTemplateOffset < iTemplateOffsetToNextBase; iTemplateOffset += 1)	
				{
					const u_int uTemplateIndex = u + iTemplateOffset;
					Asura_Xbox360_Botanicals_Template& xCurrTemplate = s_pxTemplates[ uTemplateIndex];

					Asura_Assert( xCurrTemplateBase.m_pxShape == xCurrTemplate.m_pxShape, "Current Template does not match Base template type");

#if defined(ASURA_USE_NEW_VISIBILITY_SYSTEM)
					if(!xCurrTemplate.IsVisibleFromCurrentCamera())
					{
						continue;
					}

					const Asura_Bounding_Box&	xTemplateBoundingBox	= xCurrTemplate.GetBoundingBox();
					const Asura_Vector_3		xDirection(xTemplateBoundingBox.CalculateCentre() - xCameraPosition);
					const float					fDistSqd				= xDirection.MagnitudeSquared();
#else
					{
						const Asura_Xbox360_Botanicals_Instance& xCurrInstance = xCurrTemplate.m_pxInstances[0];

						const int iModuleIndex = xCurrInstance.m_sModuleIndex;
						Asura_Assertf( iModuleIndex >= 0, ( "Template %d Instance %d Invalid botanical module index %d", u, iTemplateOffset, iModuleIndex ) );
						if( (0 <= iModuleIndex && iModuleIndex < Asura_Module_System::GetNumberOfModules()) && !Asura_Module_System::IsModuleVisible( iModuleIndex) )
						{
							continue;
						}
					}

					float fDistSqd = 0;

					{
						const Asura_Bounding_Box& xTemplateBoundingBox = xCurrTemplate.GetBoundingBox();
						if( !Asura_Camera::IsAABBInFrustum( xTemplateBoundingBox) ) 
						{
							continue;
						}

						const Asura_Vector_3 xDirection = xTemplateBoundingBox.CalculateCentre() - xCameraPosition;
						fDistSqd = xDirection.MagnitudeSquared();
					}
#endif	//!defined(ASURA_USE_NEW_VISIBILITY_SYSTEM)

					Asura_Assert( uTemplateCacheSize < uTemlateCacheSizeMax, "Not enough template cache space");
					if( uTemlateCacheSizeMax <= uTemplateCacheSize)
					{
						break;
					}

					{
						TemplateCache& xTemplateCache = axTemplateCache[ uTemplateCacheSize];
						xTemplateCache.m_uTemplateIndex = uTemplateIndex;
						xTemplateCache.m_fDistSqd = fDistSqd;
					}
					uTemplateCacheSize += 1;
				}
			}

			// DLH - 11 May 2007 11:12:42 - Sort required Modules into depth order to allow the Z buffer earn its keep
			// DLH - 11 May 2007 11:13:09 - Nasty Bubble sort, please improve if you get time.
			if( 1 < uTemplateCacheSize)
			{
				for( u_int uTemplateIndexA = 0; uTemplateIndexA < (uTemplateCacheSize - 1); uTemplateIndexA += 1)
				{
					for( u_int uTemplateIndexB = (uTemplateIndexA + 1); uTemplateIndexB < (uTemplateCacheSize); uTemplateIndexB += 1)
					{
						TemplateCache& xTemplateA = axTemplateCache[ uTemplateIndexA];
						TemplateCache& xTemplateB = axTemplateCache[ uTemplateIndexB];

						if( xTemplateA.m_fDistSqd < xTemplateB.m_fDistSqd )
						{
							continue;
						}
						TemplateCache xTemplateTemp = xTemplateA;
						xTemplateA = xTemplateB;
						xTemplateB = xTemplateTemp;
					}
				}
			}


			{
				for( u_int uTemplateCacheIndex = 0; uTemplateCacheIndex < uTemplateCacheSize; uTemplateCacheIndex += 1)
				{
					u_int uTemplateIndex = axTemplateCache[ uTemplateCacheIndex].m_uTemplateIndex;
					Asura_Xbox360_Botanicals_Template& xCurrTemplate = s_pxTemplates[ uTemplateIndex];

					for( int k = 0; k < xCurrTemplate.m_iNumInstances; ++k )
					{
						Asura_Xbox360_Botanicals_Instance& xCurrInstance = xCurrTemplate.m_pxInstances[k];

						Asura_Mem::PreFetchData( &xCurrInstance.m_xRenderData.m_fScale, sizeof( float ) );

						// Do frustum test
						const float fBoundingSphereRadius = fShapeRadius * xCurrInstance.m_xRenderData.m_fScale;
						const Asura_Vector_3& xBoundingSphereCentre = xCurrInstance.m_xPosition;

#if defined(ASURA_USE_NEW_VISIBILITY_SYSTEM)
						if(!xCurrTemplate.m_pxVisibilityQueries[k].IsSphereVisibleFromCurrentCamera(xBoundingSphereCentre, fBoundingSphereRadius))
						{
							continue;
						}
#else
						if( !Asura_Camera::IsSphereInFrustum( xBoundingSphereCentre, fBoundingSphereRadius ) ) 
						{
							continue;
						}
#endif	//defined(ASURA_USE_NEW_VISIBILITY_SYSTEM)

#if defined( ASURA_USE_DEBUGOVERLAY )
						if ( s_iDebugDrawInstances >= 0 )
						{
							Asura_DebugOverlay::DrawCrossWorld( ASURA_DO_BOTANICALS, xBoundingSphereCentre, 0xFFFF8080, 0.2f );
						}
						if ( s_iDebugDrawInstances >= 2 )
						{
							Asura_DebugOverlay::DrawSphereWorld( ASURA_DO_BOTANICALS, xBoundingSphereCentre, fBoundingSphereRadius, 0xFF80FF80, 4, 4 );
						}
#endif // #if defined( ASURA_USE_DEBUGOVERLAY )

#if defined( ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA) && 1 == ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA
						Asura_BGR_DynamicInstance* pxDynamicInstanceData = xCurrTemplate.GetDynamicInstanceData( k);
#endif
						// Check that we're in range
						// DLH - This could be changed for a dp4 plane against position to remove teh square root!

						Asura_Vector_3 xInstanceDisplacement = xCurrInstance.m_xPosition;
						xInstanceDisplacement -= *Asura_Camera::GetPosition();

						float fDist = xInstanceDisplacement.Magnitude();
						fDist *= Asura_Xbox360_Botanicals_System_Distance_Scale;

						if( uInstanceCacheMax <= uInstanceCacheIndex)
						{
							Asura_Assert(false, "To many instances to cache");
							break;
						}

						{
							Asura_Xbox360_Botanicals_InstanceCache& xInstanceCache = axInstanceCache[ uInstanceCacheIndex];

							xInstanceCache.m_fDistance = fDist;
							xInstanceCache.m_pxCurrTemplate = &xCurrTemplate;
							xInstanceCache.m_pxCurrInstance = &xCurrInstance;

#if defined( ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA) && 1 == ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA
							xInstanceCache.m_pxDynamicInstanceData = pxDynamicInstanceData;
#endif

							xCurrInstance.m_bVisible = true;

							uInstanceCacheIndex += 1;
						}
					}
				}
			}
		}

#if 0
		// DLH - 11 May 2007 11:12:42 - Sort required Instances into depth order to allow the Z buffer earn its keep
		// DLH - 11 May 2007 11:13:09 - Nasty Bubble sort, please improve if you get time.
		if( 1 < uInstanceCacheIndex)
		{
			for( u_int uTemplateIndexA = 0; uTemplateIndexA < (uInstanceCacheIndex - 1); uTemplateIndexA += 1)
			{
				for( u_int uTemplateIndexB = (uTemplateIndexA + 1); uTemplateIndexB < (uInstanceCacheIndex); uTemplateIndexB += 1)
				{
					Asura_Xbox360_Botanicals_InstanceCache& xTemplateA = axInstanceCache[ uTemplateIndexA];
					Asura_Xbox360_Botanicals_InstanceCache& xTemplateB = axInstanceCache[ uTemplateIndexB];

					if( xTemplateA.m_fDistance < xTemplateB.m_fDistance)
					{
						continue;
					}
					Asura_Xbox360_Botanicals_InstanceCache xTemplateTemp = xTemplateA;
					xTemplateA = xTemplateB;
					xTemplateB = xTemplateTemp;
				}
			}
		}
#endif

		for( int i = 0; i < xCurrShape.m_iNumElements; ++i )
		{
			Asura_Xbox360_Botanicals_Element& xCurrElement = xCurrShape.m_pxElements[i];

			pxCurRenderData = CalculateLODsCache( xCurrElement, axInstanceCache, uInstanceCacheIndex, pxCurRenderData, pxRenderDataEnd );
		}
	}

	PIXEndNamedEvent(); // "CalculateLODs"

//  static int iMaxInstances = 0;
//  iMaxInstances = Asura_Maths::Max( iMaxInstances, pxCurRenderData - pxRenderDataStart );

	s_axInstanceVBs[s_iCurrentInstanceBuffer].Unlock();
}

////////////////////////////////////////////////////////////////////////////////
//
// Asura_Xbox360_Botanicals_System::Update()
//
// Updates the botanicals system on Xbox 360. Not called at the moment, but may
// be later when the interactivity goes in
//
// Parameters:		fTime - Frame time interval
// Return value:	None
//
////////////////////////////////////////////////////////////////////////////////

void Asura_Xbox360_Botanicals_System::Update( const float fTime )
{
}

////////////////////////////////////////////////////////////////////////////////
//
// Asura_Xbox360_Botanicals_System::Render()
//
// Renders the botanicals system on Xbox 360
//
// Parameters:		fFade - Global transparency level for all botanicals
// Return value:	None
//
////////////////////////////////////////////////////////////////////////////////

void Asura_Xbox360_Botanicals_System::Render( const float fFade )
{
	s_uNumTrianglesRendered = 0;
	s_uNumInstancesRendered = 0;

#if defined( ASURA_SUPPORT_COMMAND_CONSOLE)
	if( false == Asura_BGR_Manager::s_bDebugVarShow)
	{
		return;
	}
#endif

	PIXBeginNamedEvent( D3DCOLOR_XRGB( 0x55, 0x55, 0x55 ), "Botanicals" );		

	ASURA_GPU_PROFILE_START(ASURA_GPUPROFILE_BOTANICALS);

	// We dont want blending on botanicals, and we need to do albedo too!
	Asura_Xbox360_Graphics::SetTranslucencyModeMRT( 2, ASURA_TRANSLUCENCY_OFF );


	CalculateLODs();

	// Set the instance data to come through stream 2
	s_axInstanceVBs[s_iCurrentInstanceBuffer].Bind(2);

	u_int uRenderStartInstance = 0;

	// Sort render by template

	struct LODCache
	{
	public:
		const Asura_Xbox360_Botanicals_Element* m_pxElement;
		u_int m_uLODIndex;
		float m_fDistance;
		u_int m_uRenderStartInstance;
	};
	const u_int uLODCacheSizeMax = 256;
	LODCache axLODCache[ uLODCacheSizeMax];
	u_int uLODCacheSize = 0;

	int iTemplateOffsetToNextBase = 1;
#if defined( ASURA_SUPPORT_COMMAND_CONSOLE)
	int iThisTemplate = 0;
#endif // #if defined( ASURA_SUPPORT_COMMAND_CONSOLE)
	for( u_int u = 0; u < s_uNumTemplates; u += iTemplateOffsetToNextBase )
	{
		const Asura_Xbox360_Botanicals_Template& xCurrTemplateBase = s_pxTemplates[u];

		iTemplateOffsetToNextBase = xCurrTemplateBase.m_iOffsetToNextBase;

		const Asura_Xbox360_Botanicals_Shape* const pxCurrShape = xCurrTemplateBase.m_pxShape;
		if( !pxCurrShape ) continue;

		//PIXBeginNamedEvent( D3DCOLOR_XRGB( 0x40, 0x40, 0x40 ), "Template %s", xCurrTemplateBase.m_xName.GetString() );

		// Loop over each shape element

#if defined( ASURA_SUPPORT_COMMAND_CONSOLE)
		if ( ( s_iOnlyRenderTemplate != -1 ) && ( iThisTemplate != s_iOnlyRenderTemplate ) )
		{
			iThisTemplate++;
			continue;
		}
		iThisTemplate++;
#endif // #if defined( ASURA_SUPPORT_COMMAND_CONSOLE)

		for( int i = 0; i < pxCurrShape->m_iNumElements; ++i )
		{
			const Asura_Xbox360_Botanicals_Element& xCurrElement = pxCurrShape->m_pxElements[i];

			//PIXBeginNamedEvent( D3DCOLOR_XRGB( 0x30, 0x30, 0x30 ), "%s Element %i", aszELEMENT_TYPE_NAMES[ xCurrElement.m_eType ], i );

			// Render as many of each level of detail as are in range

//			float fDistanceMin = -1e30;

			for( int j = xCurrElement.m_iNumLODs - 1; 0 <= j; --j )
			{
				const Asura_Xbox360_Botanicals_LODData& xCurrLODData = xCurrElement.m_pxLODData[j];

//				This assert is a duplicate of the one in CalculateLods (which now gives the template name)
//				Asura_Assert( fDistanceMin <= xCurrLODData.m_fMinDistance, "LOD Data Out of Order");
//				fDistanceMin = Asura_Maths::Max( fDistanceMin, xCurrLODData.m_fMinDistance);

				if( xCurrLODData.m_uNumVisibleInstances == 0 ) 
				{
					continue;
				}

				Asura_Assert( uLODCacheSize < uLODCacheSizeMax, "Not enough LOD cache space");
				if( uLODCacheSizeMax <= uLODCacheSize)
				{
					break;
				}

				{
					LODCache& xLODCache = axLODCache[ uLODCacheSize];
					xLODCache.m_pxElement = &xCurrElement;
					xLODCache.m_uLODIndex = j;
					xLODCache.m_fDistance = xCurrLODData.m_fMaxDistance;
					xLODCache.m_uRenderStartInstance = uRenderStartInstance;
				}
				uLODCacheSize += 1;
				uRenderStartInstance += xCurrLODData.m_uNumVisibleInstances;
			}

			//PIXEndNamedEvent(); // Element
		}

		//PIXEndNamedEvent(); // Template
	}

	// DLH - 11 May 2007 11:12:42 - Sort required LODs into depth order to allow the Z buffer earn its keep
	// DLH - 11 May 2007 11:13:09 - Nasty Bubble sort, please improve if you get time.
	if( 1 < uLODCacheSize)
	{
		for( u_int uTemplateIndexA = 0; uTemplateIndexA < (uLODCacheSize - 1); uTemplateIndexA += 1)
		{
			for( u_int uTemplateIndexB = (uTemplateIndexA + 1); uTemplateIndexB < (uLODCacheSize); uTemplateIndexB += 1)
			{
				LODCache& xTemplateA = axLODCache[ uTemplateIndexA];
				LODCache& xTemplateB = axLODCache[ uTemplateIndexB];

				int eTypeA = xTemplateA.m_pxElement->m_eType;
				int eTypeB = xTemplateB.m_pxElement->m_eType;

				if( eTypeA < eTypeB)
				{
					continue;
				}
				if( eTypeA == eTypeB)
				{
					if( xTemplateA.m_fDistance < xTemplateB.m_fDistance)
					{
						continue;
					}
				}
				LODCache xTemplateTemp = xTemplateA;
				xTemplateA = xTemplateB;
				xTemplateB = xTemplateTemp;
			}
		}
	}

#if defined( ASURA_SUPPORT_COMMAND_CONSOLE)
	RECT xOldScissorRect = { 0};
	if( false == Asura_BGR_Manager::s_bDebugVarShowPixels)
	{
		Asura_Xbox360_Graphics::GetDevice()->GetScissorRect( &xOldScissorRect);

		RECT xScissorRect = { 1, 1, 2, 2};
		Asura_Xbox360_Graphics::GetDevice()->SetScissorRect( &xScissorRect);
		Asura_Xbox360_Graphics::GetDevice()->SetRenderState( D3DRS_SCISSORTESTENABLE, TRUE);
	}
#endif

	Direct3D_Device* const pxDevice = Asura_Xbox360_Graphics::GetDevice();

	pxDevice->SetRenderState_Inline( D3DRS_COLORWRITEENABLE,  D3DCOLORWRITEENABLE_ALL );
	pxDevice->SetRenderState_Inline( D3DRS_COLORWRITEENABLE1, D3DCOLORWRITEENABLE_ALL );
	pxDevice->SetRenderState_Inline( D3DRS_COLORWRITEENABLE2, D3DCOLORWRITEENABLE_ALL );
	pxDevice->SetRenderState_Inline( D3DRS_COLORWRITEENABLE3, D3DCOLORWRITEENABLE_ALL );

	for( u_int uLODCacheIndex = 0; uLODCacheIndex < uLODCacheSize; uLODCacheIndex += 1)
	{
		LODCache& xLODCache = axLODCache[ uLODCacheIndex];
		const Asura_Xbox360_Botanicals_Element& xCurrElement = *xLODCache.m_pxElement;
		const u_int j = xLODCache.m_uLODIndex;
		const Asura_Xbox360_Botanicals_LODData& xCurrLODData = xCurrElement.m_pxLODData[ j];
		const u_int uRenderStartInstance = xLODCache.m_uRenderStartInstance;

		PIXBeginNamedEvent( D3DCOLOR_XRGB( 0x20, 0x20, 0x20 ), "%s LOD %i (%u instances)", aszELEMENT_TYPE_NAMES[ xCurrElement.m_eType ], j, xCurrLODData.m_uNumVisibleInstances );

		switch( xCurrElement.m_eType )
		{
		case Asura_Xbox360_Botanicals_Element::TYPE_GEOMETRY_1SIDED:
			{
				RenderGeometry( xCurrElement.m_pxGeometryLODs[j], uRenderStartInstance, xCurrLODData.m_uNumVisibleInstances, false );
				break;
			}

		case Asura_Xbox360_Botanicals_Element::TYPE_GEOMETRY_2SIDED:
			{
				RenderGeometry( xCurrElement.m_pxGeometryLODs[j], uRenderStartInstance, xCurrLODData.m_uNumVisibleInstances, true );
				break;
			}

		case Asura_Xbox360_Botanicals_Element::TYPE_BILLBOARDS:
			{
				RenderBillboards( xCurrElement.m_pxBillboardLODs[j], uRenderStartInstance, xCurrLODData.m_uNumVisibleInstances );
				break;
			}

		case Asura_Xbox360_Botanicals_Element::TYPE_IMPOSTER:
			{
				RenderImposters( xCurrElement.m_pxImposterLODs[j], uRenderStartInstance, xCurrLODData.m_uNumVisibleInstances );
				break;
			}

		default:
			{
				Asura_Assert( false, "Unknown botanicals element type." );
				break;
			}
		}

		PIXEndNamedEvent(); // LOD
	}

#if defined( ASURA_SUPPORT_COMMAND_CONSOLE)
	if( false == Asura_BGR_Manager::s_bDebugVarShowPixels)
	{
		Asura_Xbox360_Graphics::GetDevice()->SetScissorRect( &xOldScissorRect);
		Asura_Xbox360_Graphics::GetDevice()->SetRenderState( D3DRS_SCISSORTESTENABLE, FALSE);
	}
#endif

	Asura_Xbox360_Graphics::GetDevice()->SetStreamSource( 1, NULL, 0, 0 );
	Asura_Xbox360_Graphics::GetDevice()->SetStreamSource( 2, NULL, 0, 0 );

	s_uNumInstancesRendered = uRenderStartInstance;

	// Double-buffering: Swap buffers
	s_iCurrentInstanceBuffer++;
	if (s_iCurrentInstanceBuffer>= NUM_INSTANCE_BUFFERS)
	{
		s_iCurrentInstanceBuffer = 0;
	}

	ASURA_GPU_PROFILE_END(ASURA_GPUPROFILE_BOTANICALS);
	PIXEndNamedEvent(); // Botanicals	
}



////////////////////////////////////////////////////////////////////////////////
//
// Asura_Xbox360_Botanicals_System::SetMaterial()
//
// Sets up shader params for a given effect+material
//
// Supports a fair a bit of current (soon-obsolete) Asura Materials - but not
// everything. In particular, no dual-texturing or self-illumination
// 
// Return value:	None
//
////////////////////////////////////////////////////////////////////////////////

void Asura_Xbox360_Botanicals_System::SetMaterial(Asura_Xbox360_Effect *pxEffect, Asura_Xbox360_MaterialResponse_Handles *pxHandles, uint32 uMaterialResponseHashID)
{
	const Asura_MaterialResponse *pxMaterial = Asura_MaterialResponse_System::GetMaterialByHashID(uMaterialResponseHashID);

	if (pxMaterial)
	{
		Asura_Xbox360_MaterialResponse_System::SetEffectParameters(pxMaterial, NULL, NULL, *pxHandles, *pxEffect);
	}
	else
	{
		Asura_Assert(false, "Botanical has missing material response!");
	}
}

////////////////////////////////////////////////////////////////////////////////
//
// Asura_Xbox360_Botanicals_System::RenderGeometry()
//
// Renders a batch of geometry-type botanicals instances. Bind a stream of instances
// before calling this function
//
// Parameters:		xLOD - Specific LOD to render
//					uFirstInstance - Index of first instance in the stream
//					uNumInstances - Number of instances in the stream
//
// Return value:	None
//
////////////////////////////////////////////////////////////////////////////////

void Asura_Xbox360_Botanicals_System::RenderGeometry( const Asura_Xbox360_Botanicals_LOD_Geometry& xLOD, const u_int uFirstInstance, const u_int uNumInstances, bool bDoubleSided )
{	
	SetMaterial(s_pxGeometryEffect, &g_xGeometryMaterialResponseHandles, xLOD.m_uMaterialResponseHashID);

	s_pxGeometryEffect->GetInterface()->SetFloat( s_xGeometryInstanceSizeHandle, static_cast< float >( xLOD.m_uNumIndices ) );
	s_pxGeometryEffect->GetInterface()->SetFloat( s_xGeometryFirstInstanceHandle, static_cast< float >( uFirstInstance ) );
	s_pxGeometryEffect->GetInterface()->SetInt(   s_xGeometryCullModeHandle, bDoubleSided ? D3DCULL_NONE : D3DCULL_CW );

	const float fDualSidedLightingLerp = (bDoubleSided ? Asura_BGR_Manager::s_fDualSidedLightingLerp : 0.0f);
	s_pxGeometryEffect->GetInterface()->SetFloat(  s_xGeometryDualSidedLightHandle, fDualSidedLightingLerp );

	Asura_Xbox360_Graphics::GetDevice()->SetVertexDeclaration( s_pxGeometryVD );
	Asura_Render::CurrentStates.RequestTranslucencyMode( ASURA_TRANSLUCENCY_OFF );

	xLOD.m_xVB.Bind(0);
	xLOD.m_xIB.Bind(1);

	s_pxGeometryEffect->Begin();
	s_pxGeometryEffect->Commit();

	Asura_Xbox360_Graphics::GetDevice()->DrawVertices( D3DPT_TRIANGLELIST, 0, xLOD.m_uNumIndices * uNumInstances );
	Asura_Render::AddToTrianglesRendered( xLOD.m_uNumTriangles * uNumInstances );
	s_uNumTrianglesRendered += xLOD.m_uNumTriangles * uNumInstances;

	s_pxGeometryEffect->End();
}

////////////////////////////////////////////////////////////////////////////////
//
// Asura_Xbox360_Botanicals_System::RenderBillboards()
//
// Renders a batch of billboards-type botanicals instances. Bind a stream of instances
// before calling this function
//
// Parameters:		xLOD - Specific LOD to render
//					uFirstInstance - Index of first instance in the stream
//					uNumInstances - Number of instances in the stream
//
// Return value:	None
//
////////////////////////////////////////////////////////////////////////////////

void Asura_Xbox360_Botanicals_System::RenderBillboards( const Asura_Xbox360_Botanicals_LOD_Billboards& xLOD, const u_int uFirstInstance, const u_int uNumInstances )
{		
	SetMaterial(s_pxBillboardsEffect, &g_xBillboardsMaterialResponseHandles, xLOD.m_uMaterialResponseHashID);

	Asura_Vector_3 xCameraUp, xCameraRight, xCameraForwards;
	Asura_Camera::GetUpVector( xCameraUp);
	Asura_Camera::GetRightVector( xCameraRight);
	Asura_Camera::GetForwardVector( xCameraForwards);

	xCameraUp.Normalise();
	xCameraRight.Normalise();
	xCameraForwards.Normalise();

	s_pxBillboardsEffect->GetInterface()->SetVectorF( s_xBillboardsCameraUpHandle,      (float *)&xCameraUp );
	s_pxBillboardsEffect->GetInterface()->SetVectorF( s_xBillboardsCameraRightHandle,   (float *)&xCameraRight );
	s_pxBillboardsEffect->GetInterface()->SetVectorF( s_xBillboardsCameraForwardsHandle,(float *)&xCameraForwards );	

	s_pxBillboardsEffect->GetInterface()->SetFloat( s_xBillboardsInstanceSizeHandle, static_cast< float >( xLOD.m_uNumVertices ) );
	s_pxBillboardsEffect->GetInterface()->SetFloat( s_xBillboardsFirstInstanceHandle, static_cast< float >( uFirstInstance ) );

	Asura_Xbox360_Graphics::GetDevice()->SetVertexDeclaration( s_pxBillboardsVD );

	Asura_Render::CurrentStates.RequestTranslucencyMode( ASURA_TRANSLUCENCY_OFF );

	xLOD.m_xVertexVB.Bind(0);
	xLOD.m_xBillboardVB.Bind(1);

	s_pxBillboardsEffect->Begin();
	s_pxBillboardsEffect->Commit();

	Asura_Xbox360_Graphics::GetDevice()->DrawVertices( D3DPT_QUADLIST, 0, xLOD.m_uNumVertices * uNumInstances );
	Asura_Render::AddToTrianglesRendered( (xLOD.m_uNumVertices / 2) * uNumInstances );
	s_uNumTrianglesRendered += (xLOD.m_uNumVertices / 2) * uNumInstances;

	s_pxBillboardsEffect->End();
}

////////////////////////////////////////////////////////////////////////////////
//
// Asura_Xbox360_Botanicals_System::RenderImposters()
//
// Renders a batch of imposter-type botanicals instances. Bind a stream of instances
// before calling this function
//
// Parameters:		xLOD - Specific LOD to render
//					uFirstInstance - Index of first instance in the stream
//					uNumInstances - Number of instances in the stream
//
// Return value:	None
//
////////////////////////////////////////////////////////////////////////////////

void Asura_Xbox360_Botanicals_System::RenderImposters( const Asura_Xbox360_Botanicals_LOD_Imposter& xLOD, const u_int uFirstInstance, const u_int uNumInstances )
{	
	SetMaterial(s_pxImpostersEffect, &g_xImpostersMaterialResponseHandles, xLOD.m_uMaterialResponseHashID);

	s_pxImpostersEffect->GetInterface()->SetFloat( s_xImpostersFirstInstanceHandle, static_cast< float >( uFirstInstance ) );
	s_pxImpostersEffect->GetInterface()->SetFloat( s_xImpostersNumAnglesHandle, static_cast< float >( xLOD.m_iNumImposterAngles ) );

	Asura_Vector_3 xCameraUp, xCameraRight, xCameraForwards;
	Asura_Camera::GetUpVector( xCameraUp);
	Asura_Camera::GetRightVector( xCameraRight);
	Asura_Camera::GetForwardVector( xCameraForwards);
	
	xCameraUp.Normalise();
	xCameraRight.Normalise();
	xCameraForwards.Normalise();

	s_pxImpostersEffect->GetInterface()->SetVectorF( s_xImpostersCameraUpHandle,      (float *)&xCameraUp );
	s_pxImpostersEffect->GetInterface()->SetVectorF( s_xImpostersCameraRightHandle,   (float *)&xCameraRight );
	s_pxImpostersEffect->GetInterface()->SetVectorF( s_xImpostersCameraForwardsHandle,(float *)&xCameraForwards );	

	Asura_Xbox360_Graphics::GetDevice()->SetVertexDeclaration( s_pxImpostersVD );

	Asura_Render::CurrentStates.RequestTranslucencyMode( ASURA_TRANSLUCENCY_OFF );

	xLOD.m_xImposterAngleDataVB.Bind(0);

	s_pxImpostersEffect->Begin();
	s_pxImpostersEffect->Commit();

	Asura_Xbox360_Graphics::GetDevice()->DrawVertices( D3DPT_QUADLIST, 0, 4 * xLOD.m_iNumImposterAngles * uNumInstances );
	Asura_Render::AddToTrianglesRendered( 2* 4 * xLOD.m_iNumImposterAngles * uNumInstances );
	s_uNumTrianglesRendered += 2* 4 * xLOD.m_iNumImposterAngles * uNumInstances;

	s_pxImpostersEffect->End();
}

#if defined( ASURA_USE_DEBUGOVERLAY )
void Asura_Xbox360_Botanicals_System::DoDebugOverlayCallback( void* const pxIgnore )
{
	// 'r' - reset and turn off the debug rendering
	if ( Asura_UserInput::DebouncedKeyPressed( 'r' ) )
	{
		s_iDebugDrawInstances = -1;
#ifdef ASURA_SUPPORT_COMMAND_CONSOLE
		s_iOnlyRenderTemplate = -1;
#endif // #ifdef ASURA_SUPPORT_COMMAND_CONSOLE
	}
#ifdef ASURA_SUPPORT_COMMAND_CONSOLE
	if ( Asura_UserInput::DebouncedKeyPressed( 't' ) )
	{
		if ( Asura_UserInput::KeyPressed( ASURA_KEYID_SHIFT ) )
		{
			s_iOnlyRenderTemplate--;
		}
		else
		{
			s_iOnlyRenderTemplate++;
		}
	}
#endif // #ifdef ASURA_SUPPORT_COMMAND_CONSOLE
	if ( Asura_UserInput::DebouncedKeyPressed( 'd' ) )
	{
		if ( Asura_UserInput::KeyPressed( ASURA_KEYID_SHIFT ) )
		{
			s_iDebugDrawInstances--;
		}
		else
		{
			s_iDebugDrawInstances++;
		}
	}
	Asura_DebugOverlay::DrawConsoleText( ASURA_DO_BOTANICALS, "Only %d Draw %d", s_iOnlyRenderTemplate, s_iDebugDrawInstances );
}
#endif // #if defined( ASURA_USE_DEBUGOVERLAY )
