//////////////////////////////////////////////////////////////////////////////
//
// Asura_Xbox360_Botanicals.h
//
// Copyright (c) 2006 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

#if defined( ASURA_USE_BOTANICALS )

#if defined ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#if !defined __ASURA_XBOX360_BOTANICALS_H__
#define __ASURA_XBOX360_BOTANICALS_H__

/******************************************************************************/
/* Included Files                                                             */
/******************************************************************************/

// Asura BGR
#include "Botanicals/Asura_Botanical_Game_Render_Manager.h"
#include "Botanicals/Asura_Botanical_Game_Render_Instance.h"
#include "Botanicals/Asura_Botanical_Game_Render_Instance_Header.h"
#include "Botanicals/Asura_Botanical_Game_Render_Type.h"

// Asura Platform-independent
#include "Core/Asura.h"
#include "Core/Asura_HalfFloat.h"
#include "Core/Asura_Hash.h"
#include "Core/Asura_Maths.h"

// Asura Xbox 360
#include "Rendering/Asura_Xbox360_Effect_System.h"
#include "Rendering/Asura_Xbox360_Graphics.h"
#include "Rendering/Asura_Xbox360_VertexBuffer.h"
#include "Rendering/Asura_Xbox360_MaterialResponse_System.h"

/******************************************************************************/
/* Classes                                                                    */
/******************************************************************************/

class Asura_Xbox360_Botanicals_InstanceCache;

class Asura_Xbox360_Botanicals_LOD_Geometry
{
public:

	Asura_Xbox360_Botanicals_LOD_Geometry();
	virtual ~Asura_Xbox360_Botanicals_LOD_Geometry();

	int   m_uMaterialResponseHashID;
	u_int m_uNumIndices;
	u_int m_uNumTriangles;
	u_int m_uNumVertices;

	Asura_Xbox360_VertexBuffer m_xVB;
	Asura_Xbox360_VertexBuffer m_xIB; // NB not actually an index buffer
};

class Asura_Xbox360_Botanicals_LOD_Billboards
{
public:

	Asura_Xbox360_Botanicals_LOD_Billboards();
	virtual ~Asura_Xbox360_Botanicals_LOD_Billboards();

	int   m_uMaterialResponseHashID;
	u_int m_uNumVertices; // Num billboards = num vertices / 4

	Asura_Xbox360_VertexBuffer m_xBillboardVB;	// Stream of billboard xyz offsets
	Asura_Xbox360_VertexBuffer m_xVertexVB;		// Stream of vertex xy offsets & uvs
};



// DaveR's notes on Xbox360 imposters
//
// We use the regular instance RenderData struct in stream 0. That's about
// as compact as it's going to get.
// 
// In a second stream, we pass a set of ImposterAngleData structs. For each
// RenderData instance, we generate a quad for each ImposterAngleData, with
// a bit of vfetch indexing.
//
// The vertex shader/fetching will be rather expensive. But there's very few
// verts per instance...

struct Asura_Xbox360_Botanicals_ImposterAngleData
{
	Asura_HalfFloat afPos[4];							// relative to instance, xyz
	Asura_HalfFloat afAngleFade[2];						// opaque range, fade range
	Asura_HalfFloat afWidthHeight[2];					// w,h		
	Asura_HalfFloat afUVSet[4];							// U0,V0,U1,V1
	uint32			uNormal;							// forward normal for this billboard (packed)
	uint32			uTangent;							
	uint32			uBinormal;							
};

class Asura_Xbox360_Botanicals_ImposterAngle
{
public:
	Asura_Xbox360_Botanicals_ImposterAngleData m_xData;	// The bit that goes in the VB

};

class Asura_Xbox360_Botanicals_LOD_Imposter
{
public:
	Asura_Xbox360_Botanicals_LOD_Imposter();
	virtual ~Asura_Xbox360_Botanicals_LOD_Imposter();

	int    m_iNumImposterAngles;						// Number of billboard for different orientations
	uint32 m_uMaterialResponseHashID;					// Texture ID - for all billboards

	Asura_Xbox360_Botanicals_ImposterAngle *m_apxAngles;		 
	Asura_Xbox360_VertexBuffer				m_xImposterAngleDataVB;
};




struct Asura_Xbox360_Botanicals_LODData
{
	float m_fMinDistance;
	float m_fMinFullAlphaDistance;
	float m_fMaxFullAlphaDistance;
	float m_fMaxDistance;

	u_int m_uNumVisibleInstances; // Calculated at runtime
};


class Asura_Xbox360_Botanicals_Element
{
public:

	// Needs to match TOOLS_BOTANICALS_ELEMENTTYPE. Update aszELEMENT_TYPE_NAMES in the .cpp if this changes
	enum Type
	{
		TYPE_GEOMETRY_1SIDED	= 0,
		TYPE_GEOMETRY_2SIDED	= 1,
		TYPE_BILLBOARDS			= 2,
		TYPE_IMPOSTER			= 3,
		TYPE_NUM_TYPES
	};

	Asura_Xbox360_Botanicals_Element();
	~Asura_Xbox360_Botanicals_Element();

	union
	{
		Asura_Xbox360_Botanicals_LOD_Geometry* m_pxGeometryLODs;
		Asura_Xbox360_Botanicals_LOD_Billboards* m_pxBillboardLODs;
		Asura_Xbox360_Botanicals_LOD_Imposter* m_pxImposterLODs;
	};

	Asura_Xbox360_Botanicals_LODData* m_pxLODData;

	Type m_eType;
	int m_iNumLODs;

	float m_fBSCentreHeight;
	float m_fBSRadius;
	
#if defined( ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS) && 1 == ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS
	Asura_BGR_Section_ID m_xSectionID;
	Asura_BGR_Version_ID m_xVersionID;
#endif
};

class Asura_Xbox360_Botanicals_Shape : public Asura_BGR_Type
{
public:

	Asura_Xbox360_Botanicals_Shape();
	~Asura_Xbox360_Botanicals_Shape();

	Asura_Xbox360_Botanicals_Element* m_pxElements;

#if defined( ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS) && 1 == ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS
	virtual u_int GetSectionCount() const;
#endif
	
public:
	Asura_Hash_ID m_uHashID;
	int m_iNumElements;
	
#if defined( ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS) && 1 == ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS
	u_int m_uSectionCount;
#endif
#if defined( ASURA_BOTANICAL_SUPPORT_INTERACTIVE)
	Asura_BGR_Type_BasicInteractive* m_pxBasicInteractive;
#endif
};

#if 0
struct Asura_Xbox360_Botanicals_Instance
{
	// This struct is what's copied into the instance buffer
	struct RenderData
	{
		Asura_HalfFloat m_axTransformMatrix[12];
		Asura_HalfFloat m_axRedSH[4];
		Asura_HalfFloat m_axGreenSH[4];
		Asura_HalfFloat m_axBlueSH[4];
		float m_fAlpha;
		float m_fScale;
	} m_xRenderData;

	// Float position to avoid casting halves to floats CPU-side for frustum test
	Asura_Vector_3	m_xPosition;
	int				m_iModuleIndex;
};
#endif

class Asura_Xbox360_Botanicals_Instance : public Asura_BGR_Instance
{
public:
};



class Asura_Static_Visibility_Query;
class Asura_Xbox360_Botanicals_Template : public Asura_BGR_Instances_Header
{
public:
	// BGR Header functions
	virtual u_int GetInstanceCount() const;
	virtual const Asura_BGR_Instance* GetInstance( const u_int uInstanceIndex) const;
	

	// Own functions
	Asura_Xbox360_Botanicals_Template();
	~Asura_Xbox360_Botanicals_Template();

	Asura_Xbox360_Botanicals_Shape* m_pxShape;
	Asura_Xbox360_Botanicals_Instance* m_pxInstances;

	Asura_String m_xName;
	int m_iUID;
	int m_iNumInstances;
	int m_iOffsetToNextBase;

	// Array of visibility queries, one for each instance
	Asura_Static_Visibility_Query*	m_pxVisibilityQueries;
};

class Asura_Xbox360_Botanicals_System : public Asura_BGR_Manager
{
	friend class Asura_Chunk_ResourceFile_Xbox360_Botanicals;

public:

	// BGR Manager functions
	virtual u_int GetInstanceHeaderCount( void) const;
	virtual const Asura_BGR_Instances_Header* GetInstanceHeader( const u_int uInstanceHeaderIndex) const;
	virtual Asura_BGR_Instances_Header* GetInstanceHeader( const u_int uInstanceHeaderIndex);
	virtual Asura_BGR_Type* GetType( const Asura_BGR_TypeID xTypeID);

	// Static functions

	static void Initialise();
	static void Reset();
	static void Shutdown();

	static void Update( const float fTime );
	static void Render( const float fFade );

	static u_int GetNumInstancesRendered() { return s_uNumInstancesRendered; }
	static u_int GetNumTrianglesRendered() { return s_uNumTrianglesRendered; }

	enum
	{
		NUM_INSTANCE_BUFFERS = 2
	};

protected:

	static void CalculateLODs();
    static Asura_Xbox360_Botanicals_Instance::RenderData* CalculateLODsCache( Asura_Xbox360_Botanicals_Element& xCurrElement,
                                                                              const Asura_Xbox360_Botanicals_InstanceCache* paxInstanceCache,
                                                                              const u_int uInstanceCacheMax,
                                                                              Asura_Xbox360_Botanicals_Instance::RenderData* pxCurrRenderData,
                                                                              Asura_Xbox360_Botanicals_Instance::RenderData* const pxRenderDataEnd );
	
	static void SetMaterial(Asura_Xbox360_Effect *pxEffect, Asura_Xbox360_MaterialResponse_Handles *pxHandles, uint32 uMaterialResponeHashID);

	static void RenderGeometry( const Asura_Xbox360_Botanicals_LOD_Geometry& xLOD, const u_int uFirstInstance, const u_int uNumInstances, bool bDoubleSided );
	static void RenderBillboards( const Asura_Xbox360_Botanicals_LOD_Billboards& xLOD, const u_int uFirstInstance, const u_int uNumInstances );
	static void RenderImposters( const Asura_Xbox360_Botanicals_LOD_Imposter& xLOD, const u_int uFirstInstance, const u_int uNumInstances );

	// Instance buffer. Now double-buffered to avoid lock stalls (DMR 21/11/06)
	static int						  s_iCurrentInstanceBuffer;
	static Asura_Xbox360_VertexBuffer s_axInstanceVBs[NUM_INSTANCE_BUFFERS];

	// Geometry
	static Asura_Xbox360_Effect* s_pxGeometryEffect;
	static Asura_Xbox360_Effect_System::Handle				s_xGeometryInstanceSizeHandle;
	static Asura_Xbox360_Effect_System::Handle				s_xGeometryFirstInstanceHandle;
	static Asura_Xbox360_Effect_System::Handle				s_xGeometryCullModeHandle;
	static Asura_Xbox360_Effect_System::Handle				s_xGeometryDualSidedLightHandle;
	static Direct3D_VertexDeclaration* s_pxGeometryVD;


	// Billboards
	static Asura_Xbox360_Effect* s_pxBillboardsEffect;
	static Asura_Xbox360_Effect_System::Handle				s_xBillboardsInstanceSizeHandle;
	static Asura_Xbox360_Effect_System::Handle				s_xBillboardsFirstInstanceHandle;
	static Asura_Xbox360_Effect_System::Handle				s_xBillboardsCameraUpHandle;
	static Asura_Xbox360_Effect_System::Handle				s_xBillboardsCameraRightHandle;
	static Asura_Xbox360_Effect_System::Handle				s_xBillboardsCameraForwardsHandle;
	static Direct3D_VertexDeclaration* s_pxBillboardsVD;


	// Imposters
	static Asura_Xbox360_Effect* s_pxImpostersEffect;	
	static Asura_Xbox360_Effect_System::Handle				s_xImpostersNumAnglesHandle;
	static Asura_Xbox360_Effect_System::Handle				s_xImpostersFirstInstanceHandle;	
	static Asura_Xbox360_Effect_System::Handle				s_xImpostersCameraUpHandle;
	static Asura_Xbox360_Effect_System::Handle				s_xImpostersCameraRightHandle;
	static Asura_Xbox360_Effect_System::Handle				s_xImpostersCameraForwardsHandle;
	static Direct3D_VertexDeclaration* Asura_Xbox360_Botanicals_System::s_pxImpostersVD;



	// Shape and template lists

	static Asura_Xbox360_Botanicals_Shape* s_pxShapes;
	static Asura_Xbox360_Botanicals_Template* s_pxTemplates;

	static u_int s_uNumShapes;
	static u_int s_uNumTemplates;

	static u_int s_uNumInstancesRendered;
	static u_int s_uNumTrianglesRendered;
private:
#if defined( ASURA_USE_DEBUGOVERLAY )
	static void DoDebugOverlayCallback( void* const pxIgnore );
#endif // #if defined( ASURA_USE_DEBUGOVERLAY )

};

#endif // !defined __ASURA_XBOX360_BOTANICALS_H__

#endif // #if defined( ASURA_USE_BOTANICALS )