//*******************************************************************
// File: Asura_XBox360_Carpet.cpp
//
// Created: 26 January 2007 DLH
//
// Description: 
//
//*******************************************************************

// system libraries
#include "Core/Asura.h"

#if defined( ASURA_USE_CARPET)

// main header

// other libraries
#include "Camera/Asura_Camera.h"

#include "Carpet/Asura_Carpet_MeshManager.h"
#include "Carpet/Asura_XBox360_Carpet.h"

#include "Core/Asura_Mempool.h"
#include "Core/Asura_TextureFlags.h"
#include "Core/Asura_TextureManagement.h"
#include "Core/Asura_Timers.h"

#include "Light/Asura_Photon.h"

#include "Rendering/Asura_DIPCache.h"
#include "Rendering/Asura_Render.h"
#include "Rendering/Asura_Xbox360_Graphics.h"
#include "Rendering/Asura_Xbox360_TextureManagement.h"

#include "Debugging/Asura_Platform_GPUProfile.h"

// Asura Xbox 360 Shaders
#include "Carpet.h"


// internal dependencies

//*******************************************************************
// Constants
//*******************************************************************

#if !defined( ASURA_CGR_MESH_VB_SIZE)
	#define ASURA_CGR_MESH_VB_SIZE 0x20000
#endif

#define SAFE_DELETE_ARRAY( X) { if( NULL != (X)){ delete[] (X); (X) = NULL;} }
#define ASURA_D3D_OK( _HRESULT) SUCCEEDED( (_HRESULT))
#define ASURA_D3D_FAILED( _HRESULT) FAILED( (_HRESULT))

//#define TOOLS_CARPET_MESH_DRAW_CACHE_QUADS (uint32( 0x10000 / 4))

#if defined( ASURA_TOOLS)
#if ASURA_DIRECT3D_CGR_POLYGONSTYLE_LOOKUP_LENGTH != ASURA_CARPET_BILLBOARD_SEQUENCE_LENGTH
	#error ASURA_DIRECT3D_CGR_POLYGONSTYLE_LOOKUP_LENGTH != ASURA_CARPET_BILLBOARD_SEQUENCE_LENGTH
#endif
#endif

#define MANGLED_SB_TEXTURE_COUNT 0x1000
#define MANGLED_SB_STYLE_COUNT 0x1000
#define MANGLED_SB_BILLBOARD_COUNT 0x1000

#define MANGLED_SB_TEXTURE_OFFSET 1
#define MANGLED_SB_STYLE_OFFSET  (MANGLED_SB_TEXTURE_COUNT * MANGLED_SB_TEXTURE_OFFSET)
#define MANGLED_SB_BILLBOARD_OFFSET (MANGLED_SB_STYLE_COUNT * MANGLED_SB_STYLE_OFFSET)

#define ASURA_DIRECT3D_CARPET_GAME_RENDER_VS_CONST_MOVEMENTSCALE_INDEX 35
#define ASURA_DIRECT3D_CARPET_GAME_RENDER_VS_CONST_STYLE_START 36

//*******************************************************************
// Local Classes
//*******************************************************************

class Asura_XBox360_CGR_VSConstants_Wave
{
public:
	float m_afWave[4];
};

class Asura_XBox360_CGR_VSConstants_Base
{
public:
	float m_afTransform[4][4];
	float m_afBillboardRight[4];
	float m_afBillboardUp[4];
	float m_afCameraPlane[4];
	float m_afConstants2[4];
	float m_afCorners[4][4];
	
	Asura_XBox360_CGR_VSConstants_Wave m_axWaves[16];
};

class Asura_XBox360_CGR_VSConstants_Style
{
public:
	float m_afWidth, m_afHeight, m_afFadeStart, m_afFadeScale;
	float m_afUV[4];
};



static bool Asura_XBox360_DIPCache_Render( const Asura_DIPCache_DIPData& xDIPData, const uint32 uCallbackData);

//*******************************************************************
// Local Data
//*******************************************************************
const uint32 Asura_XBox360_CGR_PolygonStyle::uiInvalidStyleIndex = uint32( -1);

uint32 Asura_XBox360_Carpet_Game_Render::s_uMeshesCount = 0;
Asura_XBox360_CGR_Mesh* Asura_XBox360_Carpet_Game_Render::s_pxMeshes = NULL;

uint32 Asura_XBox360_Carpet_Game_Render::s_uTimeStampLast = 0;
Asura_XBox360_Carpet_Game_Render::VertexMode Asura_XBox360_Carpet_Game_Render::s_eVertexMode = Asura_XBox360_Carpet_Game_Render::SHADER_SIMPLE;
Direct3D_VertexDeclaration* Asura_XBox360_Carpet_Game_Render::s_pxVertexDeclarationSimple[ 1] = { 0};

int Asura_XBox360_Carpet_Game_Render::s_iDrawCacheTextureID = -1;
int Asura_XBox360_Carpet_Game_Render::s_iAttenuationMap = ASURA_TEXTURE_INVALID;

//uint32 Asura_XBox360_Carpet_Game_Render::s_uBatchIndexBufferSize = 6 * (0x10000 / 4);
uint32 Asura_XBox360_Carpet_Game_Render::s_uBatchIndexBufferSize = 6 * u_int(0x10000 / 6);
Direct3D_IndexBuffer* Asura_XBox360_Carpet_Game_Render::s_pxBatchIndexBuffer = NULL;
//uint32 Asura_XBox360_Carpet_Game_Render::s_uBatchVertexBufferSize = 0x10000;
//Direct3D_VertexBuffer* Asura_XBox360_Carpet_Game_Render::s_pxBatchVertexBuffer = NULL;
#undef TOOLS_CARPET_MESH_DRAW_CACHE_QUADS

Asura_DIPCache_Static< 100> s_xDIPCache( Asura_XBox360_DIPCache_Render, 0);

static Direct3D_VertexElement g_xaVSD_Carpet_Simple[] =
{
	// Vertex position and colour...
	{0,	0,		D3DDECLTYPE_FLOAT3,		D3DDECLMETHOD_DEFAULT,	D3DDECLUSAGE_POSITION,	0},
	{0,	12,		D3DDECLTYPE_FLOAT4,		D3DDECLMETHOD_DEFAULT,	D3DDECLUSAGE_COLOR,		0},
	{0,	28,		D3DDECLTYPE_D3DCOLOR,	D3DDECLMETHOD_DEFAULT,	D3DDECLUSAGE_COLOR,		1},
	D3DDECL_END()
};

static Asura_Xbox360_Effect* g_pxCarpetStaticEffect = NULL;
static Asura_Xbox360_Effect* g_pxCarpetDynamicEffect = NULL;
static Direct3D_VertexDeclaration* g_pxCarpetVD = NULL;

//*******************************************************************
// Function Definitions
//*******************************************************************

bool Asura_XBox360_Carpet_Game_Render::RenderSetShaderConstantsStyles( const uint32 uiStyleIndex, const Asura_XBox360_CGR_Style& xStyle)
{
	HRESULT xResult;
	Asura_XBox360_CGR_VSConstants_Style xVSConstantsStyle;
	
	xVSConstantsStyle.m_afWidth = xStyle.m_fWidth;
	xVSConstantsStyle.m_afHeight = xStyle.m_fHeight;
	xVSConstantsStyle.m_afFadeScale = -1.0f / xStyle.m_fLodFadeDelta;
	xVSConstantsStyle.m_afFadeStart = xStyle.m_fLodFadeFar / xStyle.m_fLodFadeDelta ;
	xVSConstantsStyle.m_afUV[0] = xStyle.m_afU[0];
	xVSConstantsStyle.m_afUV[1] = xStyle.m_afV[0];
	xVSConstantsStyle.m_afUV[2] = xStyle.m_afU[1] - xStyle.m_afU[0];
	xVSConstantsStyle.m_afUV[3] = xStyle.m_afV[1] - xStyle.m_afV[0];

	const uint32 uiConstantCount = sizeof( xVSConstantsStyle) / sizeof( float[4]);
	const uint32 uiConstantIndex = ASURA_DIRECT3D_CARPET_GAME_RENDER_VS_CONST_STYLE_START + (2 * uiStyleIndex);
	Asura_Assert( (uiConstantIndex + 2) <= 256, "This gfx card does not have enough constants");
	xResult = Asura_Xbox360_Graphics::GetDevice()->SetVertexShaderConstantF( uiConstantIndex, (float*)&xVSConstantsStyle, uiConstantCount);
	ASURA_D3D_OK( xResult);	
  
	float afMovementScale[4] = {xStyle.m_fMovementScale,0,0,0};
	xResult = Asura_Xbox360_Graphics::GetDevice()->SetVertexShaderConstantF( ASURA_DIRECT3D_CARPET_GAME_RENDER_VS_CONST_MOVEMENTSCALE_INDEX, afMovementScale, 1 );
	ASURA_D3D_OK( xResult);	
	
	return( true);
}

bool Asura_XBox360_Carpet_Game_Render::RenderSetShaderConstantsStatic( const Asura_Carpet_Mesh_Data_Frame& xDataFrame, const Asura_Photon* const pxPhoton)
{
	HRESULT xResult;
	Asura_XBox360_CGR_VSConstants_Base xVSConstantsBase;
	
#if 0
	{
		D3DXMATRIX& xMat = *((D3DXMATRIX*)( xVSConstantsBase.m_afTransform));
		D3DXMATRIX xMatView, xMatProj;

		Asura_Xbox360_Graphics::GetDevice()->GetTransform( D3DTS_VIEW, &xMatView );
		Asura_Xbox360_Graphics::GetDevice()->GetTransform( D3DTS_PROJECTION, &xMatProj );

		D3DXMatrixMultiply( &xMat, &xMatView, &xMatProj );
		D3DXMatrixTranspose( &xMat, &xMat );
		
	}
#endif

#if 1
	{
		D3DXMATRIX& xMat = *((D3DXMATRIX*)( xVSConstantsBase.m_afTransform));
		D3DXMATRIX xMatView, xMatProj;

		xMat = Asura_Xbox360_Graphics::GetViewProjectionMatrix();
		D3DXMatrixTranspose( &xMat, &xMat );
		
	}
#endif

	{
//		D3DXMATRIX xTransformViewProjection = Asura_Xbox360_Graphics::GetViewProjectionMatrix();
//		*( (D3DXMATRIX*) & xVSConstantsBase.m_afTransform) = xTransformViewProjection;
	}

	xVSConstantsBase.m_afBillboardRight[0] = xDataFrame.m_xBillboardRight[0];
	xVSConstantsBase.m_afBillboardRight[1] = xDataFrame.m_xBillboardRight[1];
	xVSConstantsBase.m_afBillboardRight[2] = xDataFrame.m_xBillboardRight[2];
	xVSConstantsBase.m_afBillboardRight[3] = 0.5f;
	
	xVSConstantsBase.m_afBillboardUp[0] = xDataFrame.m_xBillboardUp[0];
	xVSConstantsBase.m_afBillboardUp[1] = xDataFrame.m_xBillboardUp[1];
	xVSConstantsBase.m_afBillboardUp[2] = xDataFrame.m_xBillboardUp[2];
	xVSConstantsBase.m_afBillboardUp[3] = 1.0f;
	
	
	xVSConstantsBase.m_afCameraPlane[0] = xDataFrame.m_xCameraForwards[0];
	xVSConstantsBase.m_afCameraPlane[1] = xDataFrame.m_xCameraForwards[1];
	xVSConstantsBase.m_afCameraPlane[2] = xDataFrame.m_xCameraForwards[2];
	xVSConstantsBase.m_afCameraPlane[3] = xDataFrame.m_fCameraPlaneDistance;
	
	xVSConstantsBase.m_afConstants2[0] = 0x100;
	xVSConstantsBase.m_afConstants2[1] = 2 * 0x100;
	xVSConstantsBase.m_afConstants2[2] = ASURA_DIRECT3D_CARPET_GAME_RENDER_VS_CONST_STYLE_START;
	xVSConstantsBase.m_afConstants2[3] = 0;
	
	xVSConstantsBase.m_afCorners[0][0] = -0.5;
	xVSConstantsBase.m_afCorners[0][1] = +1.0;
	xVSConstantsBase.m_afCorners[0][2] = +0.0;
	xVSConstantsBase.m_afCorners[0][3] = +0.0;
	
	xVSConstantsBase.m_afCorners[1][0] = +0.5;
	xVSConstantsBase.m_afCorners[1][1] = +1.0;
	xVSConstantsBase.m_afCorners[1][2] = +1.0;
	xVSConstantsBase.m_afCorners[1][3] = +0.0;
	
	xVSConstantsBase.m_afCorners[2][0] = +0.5;
	xVSConstantsBase.m_afCorners[2][1] = +0.0;
	xVSConstantsBase.m_afCorners[2][2] = +1.0;
	xVSConstantsBase.m_afCorners[2][3] = +1.0;
	
	xVSConstantsBase.m_afCorners[3][0] = -0.5;
	xVSConstantsBase.m_afCorners[3][1] = +0.0;
	xVSConstantsBase.m_afCorners[3][2] = +0.0;
	xVSConstantsBase.m_afCorners[3][3] = +1.0;
	
	{
		const uint32 uiWavesCount = sizeof( xVSConstantsBase.m_axWaves) / sizeof( xVSConstantsBase.m_axWaves[ 0]);
		for( uint32 uiWaveIndex = 0; uiWaveIndex < uiWavesCount; uiWaveIndex += 1)
		{
			Asura_XBox360_CGR_VSConstants_Wave& xWave = xVSConstantsBase.m_axWaves[ uiWaveIndex];

			xWave.m_afWave[0] = xDataFrame.m_afWaveX[ uiWaveIndex];
			xWave.m_afWave[1] = xDataFrame.m_afWaveY[ uiWaveIndex];
			xWave.m_afWave[2] = xDataFrame.m_afWaveZ[ uiWaveIndex];
			xWave.m_afWave[3] = 0;
		}
	}

	
	const uint32 uiConstantCount = sizeof( xVSConstantsBase) / sizeof( float[4]);
	xResult = Asura_Xbox360_Graphics::GetDevice()->SetVertexShaderConstantF( 0, (float*)&xVSConstantsBase, uiConstantCount);
	ASURA_D3D_OK( xResult);	
	
	return( true);
}

bool Asura_XBox360_Carpet_Game_Render::CreateVertexFormat()
{
	s_eVertexMode;
	
	{
		//Asura_Xbox360_Graphics::GetDevice()->CreateVertexDeclaration( auVSD, &(s_pxVertexDeclarationSimple[0]) );
		
	}
	
	return( true);
}

bool Asura_XBox360_Carpet_Game_Render::CreateVertexBufferMesh( Asura_XBox360_CGR_Mesh& xCGRMesh)
{
	HRESULT xResult;
	
	Asura_Assert( NULL == xCGRMesh.m_pxBatchVertexBuffer, "Vertex Buffer already setup!");

	const uint32 uiVertexDataSize = xCGRMesh.m_uBatchVertexBufferSize * sizeof( Asura_XBox360_CGR_StreamVertexSimple);
	
	if( NULL != xCGRMesh.m_pxBatchVertexBuffer)
	{
		return( false);
	}
	
	xResult = Asura_Xbox360_Graphics::GetDevice()->CreateVertexBuffer
	(
		uiVertexDataSize,
		0, //D3DUSAGE_WRITEONLY | D3DUSAGE_DYNAMIC,
		0,
		D3DPOOL_DEFAULT,
		&xCGRMesh.m_pxBatchVertexBuffer,
		0
	);
	if( ASURA_D3D_FAILED( xResult))
	{
		return( false);
	}
	
	xCGRMesh.m_xVBRangeManager.SetMaxRange( xCGRMesh.m_uBatchVertexBufferSize);
	return( true);
}

bool Asura_XBox360_Carpet_Game_Render::CreateVertexBuffer()
{
	for( uint32 uiMeshIndex = 0; uiMeshIndex < s_uMeshesCount; uiMeshIndex += 1)
	{
		Asura_XBox360_CGR_Mesh& xCGRMesh = s_pxMeshes[ uiMeshIndex];
		
		CreateVertexBufferMesh( xCGRMesh);
		
	}
	return( true);
}

bool Asura_XBox360_Carpet_Game_Render::CreateIndexBuffer()
{
	HRESULT xResult;
	
	{
		const uint32 uiIndexDataSize = s_uBatchIndexBufferSize * sizeof( uint32);
		
		if( NULL == s_pxBatchIndexBuffer)
		{
			xResult = Asura_Xbox360_Graphics::GetDevice()->CreateIndexBuffer
			(
				uiIndexDataSize,
				D3DUSAGE_WRITEONLY,
				D3DFMT_INDEX32,
				D3DPOOL_MANAGED,
				&s_pxBatchIndexBuffer,
				0
			);
			if( ASURA_D3D_FAILED( xResult))
			{
				return( false);
			}
		}
		uint32* pauVertexIndexData = NULL;
		xResult = s_pxBatchIndexBuffer->Lock( 0, uiIndexDataSize, (void**)&pauVertexIndexData, 0);
		if( ASURA_D3D_FAILED( xResult))
		{
			return( false);
		}
	
		// DLH - 12 January 2006 16:07:27 - This should really be in an index buffer
		// DLH - 12 January 2006 16:07:13 - If this array has not been initialise then fill it with indices
		const uint32 uQuadCount = s_uBatchIndexBufferSize / 6;
		Asura_Assert( (6 * uQuadCount) == s_uBatchIndexBufferSize, "CGR Number of Indices is not a multiple of 6");
		for( uint32 uIndexQuad = 0; uIndexQuad < uQuadCount; uIndexQuad += 1)
		{
			const uint32 uIndexVertex = 4 * uIndexQuad;
			const uint32 uIndexIndex = 6 * uIndexQuad;			

			uint32* paxIndex = &pauVertexIndexData[ uIndexIndex];
			
			paxIndex[ 0] = uIndexVertex + 0;
			paxIndex[ 1] = uIndexVertex + 1;
			paxIndex[ 2] = uIndexVertex + 3;
			paxIndex[ 3] = uIndexVertex + 1;
			paxIndex[ 4] = uIndexVertex + 3;
			paxIndex[ 5] = uIndexVertex + 2;
		}
	
		xResult = s_pxBatchIndexBuffer->Unlock();
		ASURA_D3D_OK( xResult);
	}
	return( true);
}


bool Asura_XBox360_Carpet_Game_Render::CreateDirect3DData()
{
	ClearDirect3DData();
	
	CreateVertexFormat();
	CreateIndexBuffer();
	CreateVertexBuffer();

	return( true);
}

bool Asura_XBox360_Carpet_Game_Render::ClearDirect3DData_AllBuffers()
{
	for( uint32 uiMeshIndex = 0; uiMeshIndex < s_uMeshesCount; uiMeshIndex += 1)
	{
		Asura_XBox360_CGR_Mesh& xCGRMesh = s_pxMeshes[ uiMeshIndex];
		
		const uint32 uBatchCount = xCGRMesh.m_uPolygonBatchCount;
		for( uint32 uBatchIndex = 0; uBatchIndex < uBatchCount; uBatchIndex += 1)
		{
			Asura_XBox360_CGR_PolygonBatch& xBatch = xCGRMesh.m_pxPolygonBatches[ uBatchIndex];
			
			BatchRemoveFromBuffers( xCGRMesh, xBatch);
		}
		
		Asura_Assert( 0 == xCGRMesh.m_xVBRangeManager.m_uRangeUsedCount, "Not all the vertex Batchs where deallocated");
		Asura_Assert( 0 == xCGRMesh.m_xVBRangeManager.m_uBlockUsedCount, "Not all the Batchs where deallocated");
	}

	return( true);	
}

bool Asura_XBox360_Carpet_Game_Render::ClearDirect3DData()
{
	ClearDirect3DData_AllBuffers();
	
	for( uint32 uiMeshIndex = 0; uiMeshIndex < s_uMeshesCount; uiMeshIndex += 1)
	{
		Asura_XBox360_CGR_Mesh& xCGRMesh = s_pxMeshes[ uiMeshIndex];
		
		SAFE_RELEASE( xCGRMesh.m_pxBatchVertexBuffer);
	}
	
	SAFE_RELEASE( s_pxBatchIndexBuffer);
	
	SAFE_RELEASE_ARRAY( s_pxVertexDeclarationSimple, sizeof( s_pxVertexDeclarationSimple) / sizeof( s_pxVertexDeclarationSimple[0]));
	
	s_iAttenuationMap = ASURA_TEXTURE_INVALID;
	
	return( true);
}

bool Asura_XBox360_Carpet_Game_Render::ClearDataSet()
{
	Asura_XBox360_Carpet_Game_Render::ClearDirect3DData();

	SAFE_DELETE_ARRAY( s_pxMeshes);
	s_uMeshesCount = 0;
	
	return( true);
}

bool Asura_XBox360_Carpet_Game_Render::BatchGetRequiredBufferSize( const Asura_XBox360_CGR_Mesh& xCGRMesh, const Asura_XBox360_CGR_PolygonBatch& xCGRBatch, uint32& uIndexCount, uint32& uVertexCount)
{
	uIndexCount = 0;	
	uVertexCount = 0;	
	
	const uint32 uPolygonFinish = xCGRBatch.m_uDataOffset + xCGRBatch.m_uDataSize;
	for( uint32 uPolygonIndex = xCGRBatch.m_uDataOffset; uPolygonIndex < uPolygonFinish; uPolygonIndex += 1)
	{
		Asura_Assert( uPolygonIndex <= xCGRMesh.m_uPolygonCount, "CGRMesh Polygon Index not in range");
		const Asura_XBox360_CGR_Polygon& xPolygon = xCGRMesh.m_pxPolygons[ uPolygonIndex];
		
		const uint32 uInstanceCount = xPolygon.m_uInstanceCount;
		uIndexCount += 6 * uInstanceCount;
		uVertexCount += 4 * uInstanceCount;
	}
	
	return( true);
}

bool Asura_XBox360_Carpet_Game_Render::BatchCopyToBuffers( Asura_XBox360_CGR_Mesh& xCGRMesh, const Asura_XBox360_CGR_PolygonBatch& xCGRBatch)
{
	HRESULT xResult;
	
	const uint32 uLockSize = xCGRBatch.m_uVertexBufferSize;
	if( 0 == uLockSize)
	{
		return( true);
	}
	
	if( NULL == xCGRMesh.m_pxBatchVertexBuffer)
	{
		CreateVertexBufferMesh( xCGRMesh);
	}
	
	Direct3D_VertexBuffer* const pxVertexBuffer = xCGRMesh.m_pxBatchVertexBuffer;
	Asura_XBox360_CGR_StreamVertexSimple* pxVertex00 = NULL;
	xResult = pxVertexBuffer->Lock( 0, 0, (void**)&pxVertex00, D3DLOCK_NOOVERWRITE);
	if( ASURA_D3D_FAILED( xResult))
	{
		return( false);
	}
	
	const uint32 uLockOffset = xCGRBatch.m_uVertexBufferOffset;
	Asura_XBox360_CGR_StreamVertexSimple* pxVertex0 = pxVertex00 + uLockOffset;
	
	uint32 uVertexSpace = xCGRBatch.m_uVertexBufferSize;
	Asura_XBox360_CGR_StreamVertexSimple* pxVertex = pxVertex0;
	const uint32 uPolygonCount = xCGRBatch.m_uDataSize;
	Asura_XBox360_CGR_Polygon* pxCGRPolygon = &xCGRMesh.m_pxPolygons[ xCGRBatch.m_uDataOffset];
	for( uint32 uPolygonIndex = 0; uPolygonIndex < uPolygonCount; uPolygonIndex += 1, pxCGRPolygon += 1)
	{
		uint32 uVertexCount = uVertexSpace;
		
		CopyMeshPolygonToVertex( xCGRMesh, *pxCGRPolygon, pxVertex, uVertexCount);
		
		Asura_Assert( uVertexCount <= uVertexSpace, "CGR Too many vertices have been used");
		pxVertex += uVertexCount;
		uVertexSpace -= uVertexCount;
	}
	Asura_Assert( (pxVertex0 + xCGRBatch.m_uVertexBufferSize) == pxVertex, "CGR Vertex pointer anommily");
	Asura_Assert( 0 == uVertexSpace, "CGR Vertex count anommily!");
	
	xResult = pxVertexBuffer->Unlock();
	if( ASURA_D3D_FAILED( xResult))
	{
		return( false);
	}
	
	return( true);
}

bool Asura_XBox360_Carpet_Game_Render::BatchCopyAllToBuffers()
{
	const uint32 uMeshCount = s_uMeshesCount;
	for( uint32 uMeshIndex = 0; uMeshIndex < uMeshCount; uMeshIndex += 1)
	{
		Asura_XBox360_CGR_Mesh& xCGRMesh	= s_pxMeshes[ uMeshIndex];
		const uint32 uBatchCount = xCGRMesh.m_uPolygonBatchCount;
		for( uint32 uBatchIndex = 0; uBatchIndex < uBatchCount; uBatchIndex += 1)	
		{
			const Asura_XBox360_CGR_PolygonBatch& xCGRBatch = xCGRMesh.m_pxPolygonBatches[ uBatchIndex];
			
			BatchCopyToBuffers( xCGRMesh, xCGRBatch);
		}
	}
	return( true);
}

bool Asura_XBox360_Carpet_Game_Render::BatchAddToBuffers( Asura_XBox360_CGR_Mesh& xCGRMesh, Asura_XBox360_CGR_PolygonBatch& xCGRBatch)
{
	xCGRBatch.m_uTimeStampLastUsed = s_uTimeStampLast;
	if( 0 != xCGRBatch.m_uVertexBufferSize)
	{
		return( true);
	}
	
	// DLH - 03 February 2006 12:38:54 - Find Required Buffer Sizes
	if( false == xCGRMesh.m_xVBRangeManager.AddBatch( s_uTimeStampLast, xCGRMesh, xCGRBatch))
	{
		return( false);
	}
	
	// DLH - 03 February 2006 12:39:16 - Copy Data To Buffers

	BatchCopyToBuffers( xCGRMesh, xCGRBatch);
	
	return( true);
}

bool Asura_XBox360_Carpet_Game_Render::BatchRemoveFromBuffers( Asura_XBox360_CGR_Mesh& xCGRMesh, Asura_XBox360_CGR_PolygonBatch& xCGRBatch)
{
	xCGRMesh.m_xVBRangeManager.RemoveBatch( xCGRBatch);
	
	return( true);
}

bool Asura_XBox360_CGR_RangeManager::AddBatch( const uint32 uTimeStamp, Asura_XBox360_CGR_Mesh& xCGRMesh, Asura_XBox360_CGR_PolygonBatch& xCGRBatch)
{
	// DLH - 03 February 2006 12:38:54 - Find Required Buffer Sizes
	uint32 uIndexCount = 0;
	uint32 uVertexCount = 0;
	
	Asura_XBox360_Carpet_Game_Render::BatchGetRequiredBufferSize( xCGRMesh, xCGRBatch, uIndexCount, uVertexCount);
	
	if( 0 == uVertexCount)
	{
		return( true);
	}
	
	// DLH - 03 February 2006 12:39:07 - Allocate Space
	uint32 uVertexOffset;
	
	const bool bFoundSpace = FindFreeRange( xCGRMesh, &xCGRBatch, uVertexCount, uVertexOffset);
	if( false == bFoundSpace)
	{
		DeleteOldBatches( uTimeStamp);
		
		const bool bFoundSpace = FindFreeRange( xCGRMesh, &xCGRBatch, uVertexCount, uVertexOffset);
		if( false == bFoundSpace)
		{
			Asura_Assert( false, "Failed to find room for Carpet Batch in Vertex Buffer. This will cause slown down and render problems. Increase the Size of ASURA_CGR_MESH_VB_SIZE.");
			return( false);
		}
	}
	
	xCGRBatch.m_uVertexBufferSize = uVertexCount;
	xCGRBatch.m_uVertexBufferOffset = uVertexOffset;
	xCGRBatch.m_uTimeStampLastUsed = uTimeStamp;
	
	this->m_uRangeUsedCount += xCGRBatch.m_uVertexBufferSize;
	this->m_uBlockUsedCount += 1;
	
	return( true);
}

bool Asura_XBox360_CGR_RangeManager::RemoveBatch( Asura_XBox360_CGR_PolygonBatch& xCGRBatch)
{
	if( 0 != xCGRBatch.m_uVertexBufferSize)
	{
		Asura_ListElt< Asura_XBox360_CGR_PolygonBatch* >* pxIteratorBatch = this->m_xListBatches.Find( &xCGRBatch);
		Asura_Assert( NULL != pxIteratorBatch, "Missing Batch from Range Manager");
		if( NULL != pxIteratorBatch)
		{
			this->m_xListBatches.Remove( pxIteratorBatch);
		}
		
		Asura_Assert( 0 < this->m_uBlockUsedCount, "Trying to delete too many blocks");
		Asura_Assert( xCGRBatch.m_uVertexBufferSize <= this->m_uRangeUsedCount, "Trying to delete too many ranges");
		this->m_uBlockUsedCount -= 1;
		this->m_uRangeUsedCount -= xCGRBatch.m_uVertexBufferSize;
		
	}
	
	xCGRBatch.m_uVertexBufferSize = 0;
	xCGRBatch.m_uVertexBufferOffset = 0;
	
	
	return( true);
}

bool Asura_XBox360_CGR_RangeManager::FindFreeRange( const Asura_XBox360_CGR_Mesh& xCGRMesh, Asura_XBox360_CGR_PolygonBatch* pxCGRBatch, const uint32 uRequiredSize, uint32& uFounfOffset)
{
	uint32 uOffsetLast = 0;
	
    for (Asura_ListElt< Asura_XBox360_CGR_PolygonBatch*>* pxIteratorBatch = this->m_xListBatches.Front(); pxIteratorBatch; pxIteratorBatch = pxIteratorBatch->Next())
    {
		Asura_XBox360_CGR_PolygonBatch* pxBatchSearch = pxIteratorBatch->Data();		
		const uint32 uOffsetPresent = pxBatchSearch->m_uVertexBufferOffset;
		
		const uint32 uPresentSized = uOffsetPresent - uOffsetLast;
		if( uRequiredSize <= uPresentSized)
		{
			uFounfOffset = uOffsetLast;
			this->m_xListBatches.Insert_Before( pxIteratorBatch, pxCGRBatch);
			return( true);
		}
		
		uOffsetLast = uOffsetPresent + pxBatchSearch->m_uVertexBufferSize;
	}
	
	{
		const uint32 uPresentSized = this->m_uRangeMaxCount - uOffsetLast;
		if( uRequiredSize <= uPresentSized)
		{
			uFounfOffset = uOffsetLast;
			this->m_xListBatches.Push_Back( pxCGRBatch);
			return( true);
		}
	}
	
	return( false);
}

bool Asura_XBox360_CGR_RangeManager::DeleteOldBatches( const uint32 uTimeStampLast)
{
	uint32 const uTimeStampDelete = uTimeStampLast - 2;
	
	Asura_LinkList< Asura_XBox360_CGR_PolygonBatch*> xListDeleteBatches;
	
	{
		for (Asura_ListElt< Asura_XBox360_CGR_PolygonBatch*>* pxIteratorBatch = this->m_xListBatches.Front(); pxIteratorBatch; pxIteratorBatch = pxIteratorBatch->Next())
		{
			Asura_XBox360_CGR_PolygonBatch* pxBatchSearch = pxIteratorBatch->Data();		
			
			if( 0 == pxBatchSearch->m_uVertexBufferSize)
			{
				continue;
			}
			
			const int32 uTimeStampDelta = pxBatchSearch->m_uTimeStampLastUsed - uTimeStampDelete;
			if( uTimeStampDelta < 0)
			{
				xListDeleteBatches.Push_Front( pxBatchSearch);
			}
		}
	}
	
	{
		for (Asura_ListElt< Asura_XBox360_CGR_PolygonBatch*>* pxIteratorBatch = xListDeleteBatches.Front(); pxIteratorBatch; pxIteratorBatch = pxIteratorBatch->Next())
		{
			Asura_XBox360_CGR_PolygonBatch* pxBatchSearch = pxIteratorBatch->Data();		
			
			RemoveBatch( *pxBatchSearch);
		}
	}
	
	return( true);
}

bool Asura_XBox360_CGR_RangeManager::DeleteAllBatches()
{
	for (Asura_ListElt< Asura_XBox360_CGR_PolygonBatch*>* pxIteratorBatch = this->m_xListBatches.Front(); NULL != pxIteratorBatch; pxIteratorBatch = this->m_xListBatches.Front())
	{
		Asura_XBox360_CGR_PolygonBatch* pxBatch = pxIteratorBatch->Data();
		RemoveBatch( *pxBatch);
	}
	
	return( true);
}


#if 1
static bool Asura_XBox360_DIPCache_Render( const Asura_DIPCache_DIPData& xDIPData, const uint32 uCallbackData)
{
	const static uint32 uIndexMaxCount = Asura_XBox360_Carpet_Game_Render::GetBatchIndexBufferSize();
	
	HRESULT xResult;

	uint32 uIndexTotalCount = xDIPData.GetIndexBufferStop() - xDIPData.GetIndexBufferStart();
	uint32 uVertexBufferOffset = 0;
	while( 0 < uIndexTotalCount)
	{
		const uint32 uIndexCount = Asura_Maths::Min( uIndexMaxCount, uIndexTotalCount);
		const uint32 uiCountPrimitives = uIndexCount / 3;
		const uint32 uVertexCount = 2 * uiCountPrimitives;
		xResult = Asura_Xbox360_Graphics::GetDevice()->DrawIndexedPrimitive( 
			D3DPT_TRIANGLELIST, 
			xDIPData.GetVertexBufferStart() + uVertexBufferOffset, 0, uVertexCount, 
			0, uiCountPrimitives);
		if( ASURA_D3D_FAILED( xResult))
		{
			return( false);
		}
		
		#if defined( ASURA_TOOLS)
			TOOLS_CARPET_STATISTICS_ADD( s_uDIPCount, 1);
			TOOLS_CARPET_STATISTICS_ADD( s_uDIPIndexCount, uIndexCount);
			TOOLS_CARPET_STATISTICS_ADD( s_uDIPVertexCount, uVertexCount);
			TOOLS_CARPET_STATISTICS_ADD( s_uDIPPrimitiveCount, uiCountPrimitives);
		#endif
		
		uIndexTotalCount -= uIndexCount;
		uVertexBufferOffset += uVertexCount;
	}
	
	return( true);
}
#endif

bool Asura_XBox360_CGR_Camera::GetCurrentCamera( const float fFarPlane)
{
	const Asura_CameraData& xCurrentCamera = Asura_Camera::GetCameraData( 0 );
	
	this->m_fFarPlane = fFarPlane;
	this->m_xOrientation = xCurrentCamera.m_xOrientation;
	this->m_xPosition = xCurrentCamera.m_xPosition;
	this->m_fOneOverCosHalfFovX = xCurrentCamera.m_fOneOverCosHalfFovX;
	this->m_fOneOverCosHalfFovY = xCurrentCamera.m_fOneOverCosHalfFovY;
	this->m_fTanHalfFovX = xCurrentCamera.m_fTanHalfFovX;
	this->m_fYOverXAspectRatio = xCurrentCamera.m_fYOverXAspectRatio;
	
	this->m_fFarPlane = fFarPlane;
	
	return( true);
}

Asura_XBox360_CGR_RangeManager::Asura_XBox360_CGR_RangeManager()
{
	this->m_uRangeMaxCount = 0;
	this->m_uRangeUsedCount = 0;
	this->m_uBlockUsedCount = 0;
	//this->m_uBlockMaxCount = 0;
}

void Asura_XBox360_CGR_RangeManager::Clear()
{
	this->m_uRangeUsedCount = 0;
	this->m_uBlockUsedCount = 0;
}

void Asura_XBox360_CGR_RangeManager::SetMaxRange( const uint32 uMaxRange)
{
	this->m_uRangeMaxCount = uMaxRange;
}

Asura_XBox360_CGR_RTreeNode::Asura_XBox360_CGR_RTreeNode()
{
	this->m_xBoundingBox.Zero();
	this->m_pxChild = NULL;
	this->m_pxSibling = NULL;
	this->m_uData = 0;
}

Asura_XBox360_CGR_Vertex::Asura_XBox360_CGR_Vertex()
{
	//this->m_xPosition;
	//this->m_aucColourARGB[ 4];
};

Asura_XBox360_CGR_Polygon::Asura_XBox360_CGR_Polygon()
{
	this->m_fLodFadeFar = 0;
	this->m_fDenisty = 0;
	this->m_uInstanceCount = 0;
	this->m_uRandomSeed = 0;
	
	this->m_uVertexIndex[0] = 0;
	this->m_uVertexIndex[1] = 0;
	this->m_uVertexIndex[2] = 0;
};

Asura_XBox360_CGR_PolygonBatch::Asura_XBox360_CGR_PolygonBatch()
{
	this->m_uDataOffset = 0;
	this->m_uDataSize = 0;
	this->m_uVertexBufferOffset = 0;
	this->m_uVertexBufferSize = 0;
	this->m_uTimeStampLastUsed = 0;
}

Asura_XBox360_CGR_PolygonStyle::Asura_XBox360_CGR_PolygonStyle()
{
#if defined( ASURA_TOOLS)
	this->m_uStyle = 0;
	this->m_fLodFadeFar = 0;
#endif
};

Asura_XBox360_CGR_Style::Asura_XBox360_CGR_Style()
{
	this->m_fWidth = 0;
	this->m_fHeight = 0;
	this->m_afU[0] = 0;
	this->m_afV[0] = 0;
	this->m_afU[1] = 0;
	this->m_afV[1] = 0;
	this->m_fScaleMin = 0; // DLH - 0-255 encoded 0.0f-1.0f
	this->m_fLodFadeFar = 0;
	this->m_fLodFadeNear = 0;
	this->m_fLodFadeDelta = 1e-5f;
	
#if defined( ASURA_TOOLS)
	this->m_uTriangleCount = 0;
#endif
}

Asura_XBox360_CGR_Mesh::Asura_XBox360_CGR_Mesh()
{
	this->m_iTexture = 0;
	
	this->m_uVertexCount = 0;
	this->m_uPolygonCount = 0;
	this->m_uPolygonStyleCount = 0;
	this->m_uStyleCount = 0;
	this->m_uRTreeCount = 0;
	this->m_uPolygonBatchCount = 0;
	
	this->m_pxVertices = NULL;
	this->m_pxPolygons = NULL;
	this->m_pxPolygonStyles = NULL;
	this->m_pxStyles = NULL;
	this->m_pxRTree = NULL;
	this->m_pxPolygonBatches = NULL;
	
	this->m_uBatchVertexBufferSize = ASURA_CGR_MESH_VB_SIZE;
	this->m_pxBatchVertexBuffer = NULL;
}

Asura_XBox360_CGR_Mesh::~Asura_XBox360_CGR_Mesh()
{
	SAFE_RELEASE( this->m_pxBatchVertexBuffer);
	
	SAFE_DELETE_ARRAY( this->m_pxVertices);
	SAFE_DELETE_ARRAY( this->m_pxPolygons);
	SAFE_DELETE_ARRAY( this->m_pxPolygonStyles);
	SAFE_DELETE_ARRAY( this->m_pxStyles);
	SAFE_DELETE_ARRAY( this->m_pxRTree);
	SAFE_DELETE_ARRAY( this->m_pxPolygonBatches);
	
	this->m_uVertexCount = 0;
	this->m_uPolygonCount = 0;
	this->m_uPolygonStyleCount = 0;
	this->m_uStyleCount = 0;
	this->m_uRTreeCount = 0;
	this->m_uPolygonBatchCount = 0;
	this->m_uBatchVertexBufferSize = 0;
//	this->m_uBatchVertexBufferPresentSize = 0;
}

//-------------------------------------------------------------------
// Function:    Asura_XBox360_Carpet_Game_Render::GetCullStateOfSphereInFrustum
//
// Description: 
//
// Parameters:  xCentre         - 
//              fRadius         - 
//
// Returns:     Inside, outside or Touching the view frustum extents
//-------------------------------------------------------------------
Asura_XBox360_Carpet_Game_Render::CullState Asura_XBox360_Carpet_Game_Render::GetCullStateOfSphereInFrustum( const Asura_XBox360_CGR_Camera& xCamera, const Asura_Vector_3& xCentre, const float fRadius)
{			
	//const Asura_CameraData* pxCurrentCamera = Asura_Camera::GetCameraData();
	const Asura_Matrix_3x3& xOrientation = xCamera.m_xOrientation;

	const Asura_Vector_3 xViewSpacePos = xOrientation.ApplyTranspose( xCentre - xCamera.m_xPosition );

	const float fZFar = xViewSpacePos.z + fRadius;
	// is the sphere outside of the view frustum?
	if( fZFar < 0)
	{
		return( CULL_OUTSIDE);
	}
	
	const float fZNear = xViewSpacePos.z - fRadius;
	if( xCamera.m_fFarPlane < fZNear)
	{
		return( CULL_OUTSIDE);
	}
	
	const float fRX = fRadius * xCamera.m_fOneOverCosHalfFovX;
	const float fZX = xViewSpacePos.z * xCamera.m_fTanHalfFovX;

	// is the sphere outside of the left and right planes?
	if( fRX < (xViewSpacePos.x - fZX) || fRX < (-xViewSpacePos.x - fZX))
	{
		return( CULL_OUTSIDE);
	}
	
	const float fZY = fZX * xCamera.m_fYOverXAspectRatio;
	const float fRY = fRadius * xCamera.m_fOneOverCosHalfFovY;

	// is the sphere outside of the top and bottom planes?
	if ( fRY < (xViewSpacePos.y-fZY) || fRY < (-xViewSpacePos.y-fZY))
	{
		return( CULL_OUTSIDE);
	}
	
	
	// is the sphere inside of all the planes?
	if( 
			0 <= fZNear
		&&	
			fRX <= (fZX - xViewSpacePos.x)
		&& 
			fRX <= (xViewSpacePos.x - (-fZX))
		&&	
			fRY <= (fZY - xViewSpacePos.y)
		&& 
			fRY <= (xViewSpacePos.y - (-fZY))
		&&
			fZFar < xCamera.m_fFarPlane
		)
	{
		return( CULL_INSIDE);
	}
	
	
	return( CULL_TOUCHING);
}

Asura_XBox360_Carpet_Game_Render::CullState Asura_XBox360_Carpet_Game_Render::GetCullStateOfAABBInFrustum( const Asura_XBox360_CGR_Camera& xCamera, const Asura_Bounding_Box& xBB )
{
	// KJFL - 09 May 2005 - overly conservative placeholder test 
	return( GetCullStateOfSphereInFrustum( xCamera, xBB.CalculateCentre(), xBB.CalculateRadius() ));
}

Asura_XBox360_Carpet_Game_Render::CullState Asura_XBox360_Carpet_Game_Render::GetCullStateOfAABBInFrustum( const Asura_XBox360_CGR_Camera_And_Box& xCamera, const Asura_Bounding_Box& xBB )
{
	const bool bBoxIntersects = xCamera.m_xBoundingBox.Intersects( xBB);
	if( false == bBoxIntersects)
	{
		return( CULL_OUTSIDE);
	}
	
	const Asura_XBox360_Carpet_Game_Render::CullState eCullStateFrustrum = GetCullStateOfSphereInFrustum( xCamera, xBB.CalculateCentre(), xBB.CalculateRadius() );
	if( CULL_OUTSIDE == eCullStateFrustrum)
	{
		return( CULL_OUTSIDE);
	}
	
	const bool bBoxContains = xCamera.m_xBoundingBox.Contains( xBB);
	
	if( CULL_INSIDE == eCullStateFrustrum)
	{
		if( true == bBoxContains)
		{
			return( CULL_INSIDE);
		}
	}
	
	return( CULL_TOUCHING);
}

//-------------------------------------------------------------------
// Function:    Asura_XBox360_Carpet_Game_Render::CalcVisibleBatches
//
// Description: Fill in a list of uint32 values that are with in the view frustum
//
// Parameters:  
//				xDataFrame      - Not really used
//              pxRTree         - Root of RTree to search
//              puiBatchIndices - Pointer of array to place values into
//              uiBatchCount    - in: Size of Pointer array
//								- out: Count of uint32 values in the array
//
// Returns:     sucees == true
//				fail == false
//-------------------------------------------------------------------
template< class TvisibleBounds, bool bCheckVisability> bool Asura_XBox360_Carpet_Game_Render::CalcVisibleBatches( const TvisibleBounds& xCamera, const Asura_XBox360_CGR_RTreeNode* pxRTree, uint32* puiBatchIndices, uint32& uiBatchCount)
{
	const uint32 uiBatchMaxCount = uiBatchCount;
	
	uiBatchCount = 0;
	
	if( NULL == pxRTree)
	{
		return( true);
	}
	
	// DLH - 01 February 2006 16:27:49 - A stack of nodes that have to be processed
	const uint32 uiRTreeNodeToProcessMaxCount = 50;
	const Asura_XBox360_CGR_RTreeNode* apxRTreeNodesToProcess[ 1 + uiRTreeNodeToProcessMaxCount];

#if defined ASURA_ASSERT || defined ASURA_TOOLS
	const Asura_XBox360_CGR_RTreeNode** ppxRTreeNodeToProcessMax = &apxRTreeNodesToProcess[ uiRTreeNodeToProcessMaxCount];
#endif

	const Asura_XBox360_CGR_RTreeNode** ppxRTreeNodeToProcess = apxRTreeNodesToProcess;
	
	// DLH - 01 February 2006 16:28:16 - Insert the root node for processing
	(*ppxRTreeNodeToProcess) = pxRTree;
	
	while( apxRTreeNodesToProcess <= ppxRTreeNodeToProcess)
	{
		// DLH - 01 February 2006 16:28:34 - Pop top of stack
		const Asura_XBox360_CGR_RTreeNode* pxRTreeNodeProcess = *(ppxRTreeNodeToProcess);
		ppxRTreeNodeToProcess -= 1;
		
		if( NULL == pxRTreeNodeProcess)
		{
			continue;
		}
		
		Asura_Assert( (2 + ppxRTreeNodeToProcess) <= ppxRTreeNodeToProcessMax, "CGR Batch Node Buffer Overrun");
#if defined( ASURA_TOOLS)
		if( ppxRTreeNodeToProcessMax < (2 + ppxRTreeNodeToProcess))
		{
			continue;
		}
#endif

		// DLH - 01 February 2006 16:29:48 - Add the Next sibling for processing.
		ppxRTreeNodeToProcess += 1;
		(*ppxRTreeNodeToProcess) = pxRTreeNodeProcess->m_pxSibling;
		
		// DLH - 01 February 2006 16:30:06 - If this node is not visible then stop processing
		const CullState eIsAABBInFrustum = (true == bCheckVisability) ? GetCullStateOfAABBInFrustum( xCamera, pxRTreeNodeProcess->m_xBoundingBox) : CULL_TOUCHING;
		if( true == bCheckVisability)
		{
			if( CULL_OUTSIDE == eIsAABBInFrustum)
			{
				continue;
			}
			if( CULL_INSIDE == eIsAABBInFrustum)
			{
			#if 1
				uint32 uBatchNewCount = uiBatchMaxCount - uiBatchCount;
				uint32* puiBatchNewIndices = puiBatchIndices + uiBatchCount;
				CalcVisibleBatches< TvisibleBounds, false>( xCamera, pxRTreeNodeProcess->m_pxChild, puiBatchNewIndices, uBatchNewCount);
				
				uiBatchCount += uBatchNewCount;
			#endif
			}
		}
		
		if( false == bCheckVisability || CULL_TOUCHING == eIsAABBInFrustum)
		{
			ppxRTreeNodeToProcess += 1;
			(*ppxRTreeNodeToProcess) = pxRTreeNodeProcess->m_pxChild;
		}
			
		const uint32 uiMeshBatchIndex = pxRTreeNodeProcess->m_uData;
		if( uint32(-1) == uiMeshBatchIndex)
		{
			continue;
		}
		
		// DLH - 01 February 2006 16:30:25 - This node has data to batch it
		Asura_Assert( uiBatchCount < uiBatchMaxCount, "CGR Batch List Overrun");
#if defined( ASURA_TOOLS)
		if( uiBatchMaxCount <= uiBatchCount)
		{
			return( false);
		}
#endif
		puiBatchIndices[ uiBatchCount] = uiMeshBatchIndex;
		uiBatchCount += 1;
	}
	
	return( true);	
}

bool Asura_XBox360_Carpet_Game_Render::CopyMeshPolygonToVertex( const Asura_XBox360_CGR_Mesh& xCGRMesh, const Asura_XBox360_CGR_Polygon& xCGRPolygon, Asura_XBox360_CGR_StreamVertexSimple* pxVertex0, uint32& uVertexOriginalMaxCount)
{
	uVertexOriginalMaxCount = 0;
	
	Asura_XBox360_CGR_StreamVertexSimple* pxVerts = pxVertex0;
	const uint32 uiCGRPolygonStyle = xCGRPolygon.m_uPolygonStyle;
	Asura_Assertf( uiCGRPolygonStyle < xCGRMesh.m_uPolygonStyleCount, ("Polygon Style index (%d) is too high", uiCGRPolygonStyle));
	const Asura_XBox360_CGR_PolygonStyle* pxCGRPolygonStyle = &xCGRMesh.m_pxPolygonStyles[ uiCGRPolygonStyle];

	// Reseed the random number generator with any value that's constant for this triangle.
	srand( xCGRPolygon.m_uRandomSeed);

	const uint32 uiCGRInstanceCount = xCGRPolygon.m_uInstanceCount;
	
	for( uint32 uiCGRInstanceIndex = 0; uiCGRInstanceIndex < uiCGRInstanceCount; uiCGRInstanceIndex += 1)
	{
		const uint32 uiStyleLookupIndex = uiCGRInstanceIndex % ASURA_DIRECT3D_CGR_POLYGONSTYLE_LOOKUP_LENGTH;
		const uint32 uiStyleIndex = pxCGRPolygonStyle->m_aucStyleLookup[ uiStyleLookupIndex];
		Asura_Assertf( uiStyleIndex < xCGRMesh.m_uStyleCount, ("Style index (%d) is out of range", uiStyleIndex));
		const Asura_XBox360_CGR_Style* pxStyle = &xCGRMesh.m_pxStyles[ uiStyleIndex];
		
		// Pick a random point within the triangle (barycentric coords)
		float fEdgeA = ( (rand() & 255) / 255.0f);
		float fEdgeB = ( (rand() & 255) / 255.0f);

		if( (fEdgeA + fEdgeB) > 1.0f)
		{
			fEdgeA = 1.0f - fEdgeA;
			fEdgeB = 1.0f - fEdgeB;
		}

		float fEdgeC = 1.0f - (fEdgeA + fEdgeB);

		const uint32 uiCGRVertexIndexA = xCGRPolygon.m_uVertexIndex[ 0];
		const uint32 uiCGRVertexIndexB = xCGRPolygon.m_uVertexIndex[ 1];
		const uint32 uiCGRVertexIndexC = xCGRPolygon.m_uVertexIndex[ 2];
		
		const Asura_XBox360_CGR_Vertex* pxVertices = xCGRMesh.m_pxVertices;
		const Asura_XBox360_CGR_Vertex& xVertexA = pxVertices[ uiCGRVertexIndexA];
		const Asura_XBox360_CGR_Vertex& xVertexB = pxVertices[ uiCGRVertexIndexB];
		const Asura_XBox360_CGR_Vertex& xVertexC = pxVertices[ uiCGRVertexIndexC];
		Asura_Vector_3 xPositionCenter( (xVertexA.m_xPosition * fEdgeA) + (xVertexB.m_xPosition * fEdgeB) + (xVertexC.m_xPosition * fEdgeC));		


		{				
			// Add some motion
			const uint32 uiWaveIndex = (uiCGRInstanceIndex + xCGRPolygon.m_uRandomSeed) % ASURA_CARPET_MESH_DATA_FRAME_SIZE_WAVEX;

			Asura_Vector_3 xColour = (xVertexA.m_xColourRGB * fEdgeA) + (xVertexB.m_xColourRGB * fEdgeB) + (xVertexC.m_xColourRGB * fEdgeC);

			// Calc the scale for this 
			float fScale = (xVertexA.m_fDensity * fEdgeA) + (xVertexB.m_fDensity * fEdgeB) + (xVertexC.m_fDensity * fEdgeC);
			// We only scale then down to fMinScale, not 0
			fScale = ((1.0f - fScale) * pxStyle->m_fScaleMin) + fScale;
			const uint32 uiScale = uint32( 255.0f * fScale);

			// Position and colour
			for( uint32 uiCornerIndex = 0; uiCornerIndex < 4; uiCornerIndex += 1, pxVerts += 1, uVertexOriginalMaxCount += 1)
			{		
				Asura_XBox360_CGR_StreamVertexSimple& xVertex = *pxVerts;
					
				xVertex.m_fX = xPositionCenter[0];
				xVertex.m_fY = xPositionCenter[1];
				xVertex.m_fZ = xPositionCenter[2];

				xVertex.m_afDiffuse[0] = xColour.x;
				xVertex.m_afDiffuse[1] = xColour.y;
				xVertex.m_afDiffuse[2] = xColour.z;
				xVertex.m_afDiffuse[3] = 1.0f;

				xVertex.m_ucScale = static_cast<uint8>( uiScale );
				xVertex.m_ucStyleIndex = static_cast<uint8>( uiStyleIndex );
				xVertex.m_ucWaveTableIndex = static_cast<uint8>( uiWaveIndex );
				xVertex.m_ucCornerIndex = static_cast<uint8>( uiCornerIndex );
			}
		}
		
	}
		
	return( true);
}

bool Asura_XBox360_Carpet_Game_Render::RenderMeshBatch( const Asura_XBox360_CGR_Mesh& xCGRMesh, const Asura_XBox360_CGR_PolygonBatch& xCGRBatch)
{
	if( 0 == xCGRBatch.m_uVertexBufferSize)
	{
		return( true);
	}
	
#if 1
	{
		uint32 uVertexBufferStart = xCGRBatch.m_uVertexBufferOffset;
		uint32 uVertexBufferStop = xCGRBatch.m_uVertexBufferOffset + xCGRBatch.m_uVertexBufferSize;
		uint32 uIndexBufferStart = (6 * (xCGRBatch.m_uVertexBufferOffset / 4));
		uint32 uIndexBufferStop = uIndexBufferStart + (6 * (xCGRBatch.m_uVertexBufferSize / 4));
		
		Asura_DIPCache_DIPData xData;
		
		xData.Set( uIndexBufferStart, uIndexBufferStop, uVertexBufferStart, uVertexBufferStop);
		
		s_xDIPCache.AddDIP( xData);
	}
#endif
	
	return( true);
}

template< class Tcamera> bool Asura_XBox360_Carpet_Game_Render::RenderMesh( const Asura_Carpet_Mesh_Data_Frame& xDataFrame, const Tcamera& tCamera, Asura_XBox360_CGR_Mesh& xCGRMesh)
{
	HRESULT xResult;
	
	// DLH - 03 February 2006 15:25:06 - Draw a batch at a time using the tree culling and Batching
#if defined( ASURA_CARPET_MESHMANAGER_PROFILE)	
	int iOverallTimer	= Asura_Profiling_CreateTimer();

	Asura_Profiling_StartTiming( iOverallTimer );
#endif

	const uint32 uiBatchMaxCount = 0x200;
	uint32 uiBatchCount = uiBatchMaxCount;
	uint32 auiBatchIndices[ uiBatchMaxCount];
	
	{
		CalcVisibleBatches< Tcamera, true>( tCamera, xCGRMesh.m_pxRTree, auiBatchIndices, uiBatchCount);
		
		#if defined( ASURA_CARPET_MESHMANAGER_PROFILE)	
			{
				const float fOverallTime = Asura_Timers::TicksToSeconds( Asura_Profiling_StopTiming( iOverallTimer ) );

				TOOLS_CARPET_STATISTICS_SET( s_fTimeRTree, fOverallTime);
				
				Asura_Profiling_DestroyTimer(iOverallTimer);
			}
		#endif
		if( 0 == uiBatchCount)
		{
			return( true);
		}

		SetTextureID( xCGRMesh.m_iTexture);
		
		if( NULL == xCGRMesh.m_pxBatchVertexBuffer)
		{
			CreateVertexBufferMesh( xCGRMesh);
		}
		
		s_xDIPCache.Clear();
			
		xResult = Asura_Xbox360_Graphics::GetDevice()->SetStreamSource( 0, xCGRMesh.m_pxBatchVertexBuffer, 0, sizeof(Asura_XBox360_CGR_StreamVertexSimple));
		ASURA_D3D_OK( xResult);	
		
		for( uint32 uiBatchIndicesIndex = 0; uiBatchIndicesIndex < uiBatchCount; uiBatchIndicesIndex += 1)
		{
			const uint32 uiBatchIndex = auiBatchIndices[ uiBatchIndicesIndex];
			Asura_Assert( uiBatchIndex < xCGRMesh.m_uPolygonBatchCount, "CGR Polygon Batch out of range");
			
			Asura_XBox360_CGR_PolygonBatch& xCGRBatch = xCGRMesh.m_pxPolygonBatches[ uiBatchIndex];
			
			if( false == BatchAddToBuffers( xCGRMesh, xCGRBatch))
			{
				// DLH - Batch not empty, assume that there is not enough room in the VB so clear it out and continue
				s_xDIPCache.Render();
				s_xDIPCache.Clear();
				
				xCGRMesh.m_xVBRangeManager.DeleteAllBatches();
				
				if( false == BatchAddToBuffers( xCGRMesh, xCGRBatch))
				{
					continue;
				}
			}
			
			RenderMeshBatch( xCGRMesh, xCGRBatch);
		}

		s_xDIPCache.Render();
	}
		
	return( true);
}

bool Asura_XBox360_Carpet_Game_Render::Render()
{
	PIXBeginNamedEvent( D3DCOLOR_XRGB( 0x55, 0x55, 0x55 ), "Carpet" );
	ASURA_GPU_PROFILE_START(ASURA_GPUPROFILE_CARPET);
	
	const Asura_Carpet_Mesh_Data_Frame xDataFrame( Asura_Timers::GetLocalScaledTime());
	
	bool bResetRenderStates = false;
	
	{
		bool bRenderStatesInitialised = false;
					
		for( uint32 uiMeshIndex = 0; uiMeshIndex < s_uMeshesCount; uiMeshIndex += 1)
		{
			Asura_XBox360_CGR_Mesh& xCGRMesh = s_pxMeshes[ uiMeshIndex];
			
			const bool bIsMeshVisible = Asura_Camera::IsAABBInFrustum( xCGRMesh.m_xBoundingBox);
			if( false == bIsMeshVisible)
			{
				continue;
			}
	
			if( false == bRenderStatesInitialised)
			{
				if( BeginRenderStatic( xDataFrame))
				{
					bRenderStatesInitialised = true;
					bResetRenderStates = true;
				}
			}
		
			if( false == bRenderStatesInitialised)
			{
				break;
			}
			
			for( uint32 uiStyleIndex = 0; uiStyleIndex < xCGRMesh.m_uStyleCount; uiStyleIndex += 1)
			{
				const Asura_XBox360_CGR_Style& xStyle = xCGRMesh.m_pxStyles[ uiStyleIndex];
				
				RenderSetShaderConstantsStyles( uiStyleIndex, xStyle);
			}
			
			Asura_XBox360_CGR_Camera xCamera;
			
			xCamera.GetCurrentCamera( xCGRMesh.m_fLodFadeFar);
			
			RenderMesh( xDataFrame, xCamera, xCGRMesh);
		}
		
		if( true == bRenderStatesInitialised)
		{
			EndRenderStatic();
		}
	}

#if 0	
	{
		Asura_TrackingHandlePool<Asura_Photon>* pxPhotonList = Asura_Photon_System::GetDynamicPhotonList();

		if( NULL != pxPhotonList)
		{
			// DLH - 09 June 2006 16:03:54 - Find out if there are any Dynamic Photon Emitters
			Asura_TrackingHandlePool<Asura_Photon>::Iterator xIterator = pxPhotonList->GetIterator();
				
			for (xIterator.Begin(); !xIterator.End(); xIterator.Next())
			{
				Asura_Photon* pxPhoton = xIterator.GetCurrentPointer();

				if( false == pxPhoton->IsActive()) 
				{
					continue;
				}
				
				const float fRange = pxPhoton->GetRange();
				if( fRange <= 0.0f) 
				{
					continue;
				}

// DLH - 27 June 2006 17:01:02 - Does not return a valid AABB
//				const Asura_Bounding_Box& xPhotonBoundingBox = pxPhoton->GetBoundingBox();
				const Asura_Vector_3& xPhotonPosition = pxPhoton->GetPosition();
				Asura_Bounding_Box xPhotonBoundingBox;
				xPhotonBoundingBox.SetToPoint( xPhotonPosition);
				xPhotonBoundingBox.Grow( fRange);
				
				const bool bIsPhotonVisible = Asura_Camera::IsAABBInFrustum( xPhotonBoundingBox);
				if( false == bIsPhotonVisible)
				{
					continue;
				}
				
				{
					bool bRenderStatesInitialised = false;
					
					for( uint32 uiMeshIndex = 0; uiMeshIndex < s_uMeshesCount; uiMeshIndex += 1)
					{
						Asura_XBox360_CGR_Mesh& xCGRMesh = s_pxMeshes[ uiMeshIndex];
						
						
						const bool bIsMeshInPhotonBox = xPhotonBoundingBox.Intersects( xCGRMesh.m_xBoundingBox);
						if( false == bIsMeshInPhotonBox)
						{
							continue;
						}
						
						
						const bool bIsMeshVisible = Asura_Camera::IsAABBInFrustum( xCGRMesh.m_xBoundingBox);
						if( false == bIsMeshVisible)
						{
							continue;
						}
	
						// DLH - 27 June 2006 16:52:40 - Only Initialise the render states if we have something to draw
						if( false == bRenderStatesInitialised)
						{					
							if( true == BeginRenderDynamic( xDataFrame, *pxPhoton))
							{
								bRenderStatesInitialised = true;
								bResetRenderStates = true;
							}
						}
						
						if( false == bRenderStatesInitialised)
						{					
							bresk;
						}
						
						for( uint32 uiStyleIndex = 0; uiStyleIndex < xCGRMesh.m_uStyleCount; uiStyleIndex += 1)
						{
							const Asura_XBox360_CGR_Style& xStyle = xCGRMesh.m_pxStyles[ uiStyleIndex];
							
							RenderSetShaderConstantsStyles( uiStyleIndex, xStyle);
						}
					
						Asura_XBox360_CGR_Camera_And_Box xCamera;
						
						xCamera.GetCurrentCamera( xCGRMesh.m_fLodFadeFar);
						xCamera.m_xBoundingBox = xPhotonBoundingBox;
						
						RenderMesh( xDataFrame, xCamera, xCGRMesh);
					}
					
					if( true == bRenderStatesInitialised)
					{
						EndRenderDynamic();
					}
				}
				
			}
		}

	}
#endif
	
	if( true == bResetRenderStates)
	{
		EndRender();
	}

//	D3DPERFCALL(EndEvent());
	PIXEndNamedEvent();
	ASURA_GPU_PROFILE_END(ASURA_GPUPROFILE_CARPET);
	
	return( true);
}

void Asura_XBox360_Carpet_Game_Render::EndRenderStatic()
{
	g_pxCarpetStaticEffect->End();
}

bool Asura_XBox360_Carpet_Game_Render::BeginRenderStatic( const Asura_Carpet_Mesh_Data_Frame& xDataFrame)
{
	HRESULT xResult;
	// Temporary render function, using the rendering code from the tools, until we get the 
	// proper optimized renderer (with scene management, batching, and using VU/shaders)

	// Set some renderstates
	xResult = Asura_Xbox360_Graphics::GetDevice()->SetRenderState(D3DRS_CULLMODE, D3DCULL_NONE);
	ASURA_D3D_OK( xResult);	
	Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_OFF);
	Asura_Render::CurrentStates.RequestTextureOpMode(ASURA_TEXTUREOP_SINGLE_PASS_2X);
	Asura_Render::CurrentStates.RequestAlphaTest( true , 0x00);
	Asura_Render::CurrentStates.RequestZBufferEnabled(true);
	Asura_Render::CurrentStates.RequestZBufferWriteEnabled(true);

	Asura_Xbox360_Graphics::GetDevice()->SetRenderState( D3DRS_COLORWRITEENABLE1, D3DCOLORWRITEENABLE_ALL );		
	Asura_Xbox360_Graphics::GetDevice()->SetRenderState( D3DRS_COLORWRITEENABLE2, D3DCOLORWRITEENABLE_ALL );		

	xResult = Asura_Xbox360_Graphics::GetDevice()->SetSamplerState(0, D3DSAMP_ADDRESSU, D3DTADDRESS_CLAMP );
	ASURA_D3D_OK( xResult);	
	xResult = Asura_Xbox360_Graphics::GetDevice()->SetSamplerState(0, D3DSAMP_ADDRESSV, D3DTADDRESS_CLAMP );
	ASURA_D3D_OK( xResult);	
	
	Asura_Xbox360_Graphics::GetDevice()->SetVertexDeclaration( g_pxCarpetVD );
	
	xResult = Asura_Xbox360_Graphics::GetDevice()->SetIndices( s_pxBatchIndexBuffer);
	ASURA_D3D_OK( xResult);	
	
	g_pxCarpetStaticEffect->Begin();
	
	g_pxCarpetStaticEffect->Commit();
	
	RenderSetShaderConstantsStatic( xDataFrame, NULL);
	
	Asura_Render::CurrentStates.ResetTextureIndexToLast( 1);
	
	s_iDrawCacheTextureID = -1;
	s_uTimeStampLast += 1;
	
	return( true);
}

bool Asura_XBox360_Carpet_Game_Render::BeginRenderDynamic( const Asura_Carpet_Mesh_Data_Frame& xDataFrame, const Asura_Photon& xPhoton)
{
#if 0
	// Temporary render function, using the rendering code from the tools, until we get the 
	// proper optimized renderer (with scene management, batching, and using VU/shaders)

	if( ASURA_TEXTURE_INVALID == s_iAttenuationMap)
	{
		s_iAttenuationMap = Asura_Texture_Management::AddTextureWithResetGroup("\\graphicnovel\\expattenuation.dds", ASURA_TEXTURE_LOADED_PERMANENT);
	}
	
	HRESULT xResult;
	
	// Set some renderstates
	xResult = Asura_Xbox360_Graphics::GetDevice()->SetRenderState(D3DRS_CULLMODE, D3DCULL_NONE);
	ASURA_D3D_OK( xResult);	
	Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_ADD);
	Asura_Render::CurrentStates.RequestAlphaTest( true , 0x00);
	Asura_Render::CurrentStates.RequestZBufferEnabled(true);
	Asura_Render::CurrentStates.RequestZBufferWriteEnabled(false);

	xResult = Asura_Xbox360_Graphics::GetDevice()->SetSamplerState(0, D3DSAMP_ADDRESSU, D3DTADDRESS_CLAMP );
	ASURA_D3D_OK( xResult);	
	xResult = Asura_Xbox360_Graphics::GetDevice()->SetSamplerState(0, D3DSAMP_ADDRESSV, D3DTADDRESS_CLAMP );
	ASURA_D3D_OK( xResult);	
	
	//xResult = Asura_Xbox360_Graphics::GetDevice()->SetVertexDeclaration( s_pxVertexDeclarationSimple[ 0]);
	//ASURA_D3D_OK( xResult);	
	
	if( true == Asura_Fog::IsGraphFogEnabled())
	{
		Asura_Direct3D_Shaders::RequestVertexShader( ASURA_D3D_VS_CARPET_FOGG_LIGHTD);
	}
	else
	{
		Asura_Direct3D_Shaders::RequestVertexShader( ASURA_D3D_VS_CARPET_FOGL_LIGHTD);
	}
	
	Asura_Direct3D_Shaders::RequestPixelShader( ASURA_D3D_PS_CARPET_LIGHTD);
	
	xResult = Asura_Xbox360_Graphics::GetDevice()->SetIndices( s_pxBatchIndexBuffer);
	ASURA_D3D_OK( xResult);	
	
	// turn on fog if we have to...
	if (Asura_Fog::IsFogRenderable())
	{
		// Set the fog color.
		xResult = Asura_Xbox360_Graphics::GetDevice()->SetRenderState(D3DRS_FOGENABLE, TRUE);
		ASURA_D3D_OK( xResult);
		xResult = Asura_Xbox360_Graphics::GetDevice()->SetRenderState(D3DRS_FOGCOLOR, 0x00000000);
		ASURA_D3D_OK( xResult);
	}
	
	Asura_Render::CurrentStates.RequestZBufferWriteEnabled(false);
	//Asura_Render::CurrentStates.RequestZBufferCompareMode(ASURA_ZBUFFERCMP_EQUALS);

	Asura_Xbox360_Graphics::GetDevice()->SetSamplerState(1, D3DSAMP_ADDRESSU, D3DTADDRESS_CLAMP );
	Asura_Xbox360_Graphics::GetDevice()->SetSamplerState(1, D3DSAMP_ADDRESSV, D3DTADDRESS_CLAMP );

	Asura_Xbox360_Graphics::GetDevice()->SetSamplerState(2, D3DSAMP_ADDRESSU, D3DTADDRESS_CLAMP );
	Asura_Xbox360_Graphics::GetDevice()->SetSamplerState(2, D3DSAMP_ADDRESSV, D3DTADDRESS_CLAMP );
	
	Asura_Render::CurrentStates.RequestTextureIndex( 1, s_iAttenuationMap);
	Asura_Render::CurrentStates.RequestTextureIndex( 2, s_iAttenuationMap);
	
	Asura_Render::CurrentStates.ResetTextureIndexToLast( 3);
	
	RenderSetShaderConstantsStatic( xDataFrame, &xPhoton);
	
	s_iDrawCacheTextureID = -1;
	s_uTimeStampLast += 1;
	
	return( true);
#endif

	return( false);
}

void Asura_XBox360_Carpet_Game_Render::EndRender()
{
#if 1
	HRESULT xResult;

	Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_OFF);
//	Asura_Xbox360_Graphics::GetDevice()->SetRenderState( D3DRS_FOGENABLE, FALSE);
	Asura_Render::CurrentStates.RequestAlphaTest( false);
	Asura_Render::CurrentStates.RequestZBufferEnabled(true);
	Asura_Render::CurrentStates.RequestZBufferWriteEnabled(true);
	
	Asura_Render::CurrentStates.ResetTextureIndexToLast( 1);

	// DaveR - 20/02/06
	//		   These states are unsupported by Asura, so they need restoring here, otherwise
	//         it messes up platform-independent HUD/FE rendering, where there is no
	//		   platform-independent way to restore them to Asura's assumed defaults.	
	xResult = Asura_Xbox360_Graphics::GetDevice()->SetSamplerState(0, D3DSAMP_ADDRESSU, D3DTADDRESS_WRAP );
	ASURA_D3D_OK( xResult);	
	xResult = Asura_Xbox360_Graphics::GetDevice()->SetSamplerState(0, D3DSAMP_ADDRESSV, D3DTADDRESS_WRAP );
	ASURA_D3D_OK( xResult);	
	xResult = Asura_Xbox360_Graphics::GetDevice()->SetSamplerState(1, D3DSAMP_ADDRESSU, D3DTADDRESS_WRAP );
	ASURA_D3D_OK( xResult);	
	xResult = Asura_Xbox360_Graphics::GetDevice()->SetSamplerState(1, D3DSAMP_ADDRESSV, D3DTADDRESS_WRAP );
	ASURA_D3D_OK( xResult);	
	xResult = Asura_Xbox360_Graphics::GetDevice()->SetSamplerState(2, D3DSAMP_ADDRESSU, D3DTADDRESS_WRAP );
	ASURA_D3D_OK( xResult);	
	xResult = Asura_Xbox360_Graphics::GetDevice()->SetSamplerState(2, D3DSAMP_ADDRESSV, D3DTADDRESS_WRAP );
	ASURA_D3D_OK( xResult);	
#endif
}

void Asura_XBox360_Carpet_Game_Render::SetTextureID( const int iTextureID)
{
	if( iTextureID != s_iDrawCacheTextureID)
	{
		Asura_Render::CurrentStates.RequestTextureIndex( 0, iTextureID);					
		s_iDrawCacheTextureID = iTextureID;
	}
}

Asura_Carpet_Mesh_Data_Frame::Asura_Carpet_Mesh_Data_Frame( const float fTime) :
m_fTime( fTime)
{
	Asura_Vector_3 xRight;
	Asura_Camera::GetRightVector(xRight);	
	xRight.y = 0;
	xRight.Normalise();

	float fScaledTime = fTime * Asura_Carpet_MeshManager::GetMovementSpeed();

	const u_int uiSizeWaveX = ASURA_CARPET_MESH_DATA_FRAME_SIZE_WAVEX;
	for ( u_int i = 0; i < uiSizeWaveX; i++)
	{		
		float fS = (float)i;
		// DLH - 01 February 2006 16:21:02 - Added .0f to all constant values, requested by JGP
		// DLH - 01 February 2006 16:21:27 - Changed sin to Asura_Maths::Sine(), requested by JGP
		float fAmount = 0.1f  * Asura_Maths::Sine( (1.0f * fS) + fScaledTime * (1.0f + (fS * 0.05f)) )
			+ 0.05f * Asura_Maths::Sine( (2.0f * fS) + fScaledTime * (1.0f + ((16.0f - fS) * 0.05f)));		

		this->m_afWaveX[i] = fAmount * xRight.x;
		this->m_afWaveY[i] = fAmount * xRight.y;
		this->m_afWaveZ[i] = fAmount * xRight.z;
	}

	Asura_Camera::GetForwardVector( this->m_xCameraForwards);
	this->m_xCameraPosition = *Asura_Camera::GetPosition();

	// Use camera pos/fwd to get plane eq, and use this to get distance to centre 
	// of triangle
	this->m_fCameraPlaneDistance = -(this->m_xCameraForwards * this->m_xCameraPosition);

	// JAT 13 Feb 2006 - you don't need to do all this maths to get a view horizontal vector
	/*
	Asura_Matrix_3x3 xMat = *Asura_Camera::GetOrientation();
	Asura_Euler_3 xOri;
	xOri.FromMatrix(xMat);
	xOri.z = xOri.x = 0.0f;
	xOri.ToMatrix(xMat);
	*/
	// JAT 13 Feb 2006 - get the view x vector from the view matrix
	// might go wrong if the camera has roll on it
	const Asura_Matrix_3x3* const xView = Asura_Camera::GetOrientation();
	this->m_xBillboardRight.x = xView->e11;
	this->m_xBillboardRight.y = xView->e21;
	this->m_xBillboardRight.z = xView->e31;

	// DLH - 01 February 2006 16:21:02 - Added .0f to all constant values, requested by JGP
	this->m_xBillboardUp = Asura_Vector_3(0.0f,-1.0f,0.0f);

}

void Asura_Carpet_MeshManager::Platform_Initialise( void)
{
	g_pxCarpetStaticEffect = Asura_Xbox360_Effect_System::CreateEffectFromMemory( g_auCarpet );
	Asura_Assert( NULL != g_pxCarpetStaticEffect, "");
	
	Asura_Xbox360_Graphics::GetDevice()->CreateVertexDeclaration( g_xaVSD_Carpet_Simple, &g_pxCarpetVD );
	Asura_Assert( NULL != g_pxCarpetVD, "Failed to create botanicals vertex declaration." );
}

void Asura_Carpet_MeshManager::Platform_Shutdown( void)
{
	Asura_Xbox360_Effect_System::Destroy( g_pxCarpetStaticEffect);
	SAFE_RELEASE( g_pxCarpetVD);

	Asura_XBox360_Carpet_Game_Render::ClearDataSet();
}

void Asura_Carpet_MeshManager::Platform_Reset()
{
	Asura_XBox360_Carpet_Game_Render::ClearDataSet();
}

void Asura_Carpet_MeshManager::Platform_Render( float)
{
	if( Asura_XBox360_Carpet_Game_Render::HasDataSet())
	{
		Asura_XBox360_Carpet_Game_Render::Render();
	}
}


//*******************************************************************
//
// (c) 2006 Rebellion
//
// Warning: This computer program is protected by copyright law and 
// international treaties. Unauthorized reproduction or distribution
// of this program, or any portion of it, may result in severe civil and
// criminal penalties, and will be prosecuted to the maximum extent 
// possible under the law.
//
//*******************************************************************

#endif //defined( ASURA_USE_CARPET)
