//
// Asura_Chunk_ResourceFile_Xbox360_Environment.cpp
//
// Created by KJFL - 27 June 2005
//
// Copyright (C) 2005 Rebellion, All Rights Reserved.
//

/******************************************************************************/
/* Included Files                                                             */
/******************************************************************************/

#include "Core/Asura.h"

#include "ChunkSystem/Asura_Chunk_ResourceFile_Xbox360_Environment.h"

// Asura Platform-independent
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "Core/Asura_Material_System.h"
#include "Debugging/Asura_Logfile.h"
#include "Rendering/Asura_Xbox360_EnvironmentRenderer.h"
#include "Rendering/Asura_Xbox360_IndexBuffer.h"
#include "Rendering/Asura_Xbox360_VertexBuffer.h"

/******************************************************************************/
/* Constants                                                                  */
/******************************************************************************/

const u_int uVERTEX_SIZE = 68;

/******************************************************************************/
/* Asura_Chunk_Xbox360_Environment Class Functions                            */
/******************************************************************************/

bool Asura_Chunk_ResourceFile_Xbox360_Environment::Process( const char* szFilename, Asura_Chunk_Stream& xStream )
{
	if( Version != iASURA_CHUNK_XBOX360_ENVIRONMENT_VERSION )
	{
		Asura_Assertf( false, ("Invalid environment chunk version (%d). Re-export needed.", Version) );
		return false;
	}

	u_int	uNumberOfStrips;

	// Read the data...

	xStream >> uNumberOfStrips;

	Asura_Xbox360_EnvironmentRenderer::s_uNumberOfStrips = uNumberOfStrips;
	
	// allocate storage space 
	Asura_Xbox360_EnvironmentRenderer::s_pxStrips = new Asura_Xbox360_EnvironmentRenderer_Strip[ uNumberOfStrips ];

	// load in strip descriptions, all in one go
	xStream.Read( uNumberOfStrips * sizeof( Asura_Xbox360_EnvironmentRenderer_Strip ), Asura_Xbox360_EnvironmentRenderer::s_pxStrips );

	// Allocate one buffer containing all the vertices
	{
		u_int uNumberOfVertices;
		xStream >> uNumberOfVertices;

		const u_int uSize = uVERTEX_SIZE * uNumberOfVertices;

		Asura_Xbox360_EnvironmentRenderer::s_pVertexData = Asura_Xbox360_MemoryManagement::AllocatePhysical(Asura_Xbox360_MemoryManagement::MT_READ_WRITE_COMBINE, uSize, XALLOC_PHYSICAL_ALIGNMENT_DEFAULT);

		if( Asura_Xbox360_EnvironmentRenderer::s_pVertexData )
		{
			xStream.Read( uSize, Asura_Xbox360_EnvironmentRenderer::s_pVertexData );
		}
		else
		{
			Asura_Assert( false, "Couldn't alloc vertex data. Normally an 'out of memory' sign." );
			xStream.Skip( uSize );
		}
	}

	Asura_Xbox360_EnvironmentRenderer::s_pxVBs = new Asura_Xbox360_VertexBuffer[ uNumberOfStrips ];

	// Create a VB per strip, pointing into our allocated memory
	for( u_int u = 0; u < uNumberOfStrips; ++u )
	{
		const Asura_Xbox360_EnvironmentRenderer_Strip& xStrip =
			Asura_Xbox360_EnvironmentRenderer::s_pxStrips[u];

		// Each VB starts at some point within the allocation
		const u_int uDataOffset = uVERTEX_SIZE * xStrip.m_uBaseVertexIndex;

		Asura_Xbox360_EnvironmentRenderer::s_pxVBs[u].CreateStatic( uVERTEX_SIZE,
			xStrip.m_uNumberOfVertices, Asura_Xbox360_EnvironmentRenderer::s_pVertexData, uDataOffset );
	}

	// Create one IB containing all the indices
	{
		u_int uNumberOfIndices;
		xStream >> uNumberOfIndices;

		const u_int uIBSize = sizeof( u_short ) * uNumberOfIndices;

		Asura_Xbox360_EnvironmentRenderer::s_xIB.CreateStatic( uIBSize );

		void* const pIB = Asura_Xbox360_EnvironmentRenderer::s_xIB.Lock();
		if( pIB )
		{
			xStream.Read( uIBSize, pIB );
		}
		else
		{
			Asura_Assert( false, "Couldn't lock index buffer. Normally an 'out of memory' sign." );
			xStream.Skip( uIBSize );
		}

		Asura_Xbox360_EnvironmentRenderer::s_xIB.Unlock();
	}

	// read in the Anti-Alias edges.
	u_int uTotalNumEdgeIndices;
	xStream >> uTotalNumEdgeIndices;
	if ( uTotalNumEdgeIndices > 0 )
	{
		Asura_Xbox360_EnvironmentRenderer::s_xAntiAliasEdges.CreateStatic( uTotalNumEdgeIndices * 2 );

		u_short* puIdxOut = static_cast<u_short*>(Asura_Xbox360_EnvironmentRenderer::s_xAntiAliasEdges.Lock());
		xStream.Read( uTotalNumEdgeIndices*sizeof(u_short), puIdxOut );
		Asura_Xbox360_EnvironmentRenderer::s_xAntiAliasEdges.Unlock();
	}

	return(true);
}
