//////////////////////////////////////////////////////////////////////////////
//
// Asura_Platform_MultithreadTypes.h
//
// Copyright (c) 2006 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

#if defined ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#if !defined __ASURA_PLATFORM_MULTITHREADTYPES_H__
#define __ASURA_PLATFORM_MULTITHREADTYPES_H__

/******************************************************************************/
/* Included Files                                                             */
/******************************************************************************/

#include <xtl.h>

/******************************************************************************/
/* Classes                                                                    */
/******************************************************************************/

typedef unsigned Asura_ThreadID;

class Asura_Platform_MutexData
{
public:

	Asura_Platform_MutexData() : m_bInitialised(false)	{}
	~Asura_Platform_MutexData()							{ DeleteCriticalSection(&m_xCriticalSection); }

protected:

	bool				m_bInitialised;
	CRITICAL_SECTION	m_xCriticalSection;
};


class Asura_Platform_SyncEvent
{
public:

	Asura_Platform_SyncEvent( const char* szName, bool bStartsSignalled )
	{
		// KJFL - 22 April 2008 - as on PC, we don't pass the name to OS as
		// it assumes all names are unique, and it won't create a new event
		// if the name matches an existing one.
		m_xEventHandle = CreateEvent( NULL, true, bStartsSignalled, NULL );
	}

	~Asura_Platform_SyncEvent()
	{
		CloseHandle( m_xEventHandle );
	}

protected:

	HANDLE m_xEventHandle;
};

class Asura_Platform_Semaphore
{
public:

	Asura_Platform_Semaphore( const char* szName, u_int uInitialCount, u_int uMaxCount )
	{
		// KJFL - 22 April 2008 - as with the SyncEvent we don't pass the name as
		// the OS thinks instances sharing names to be the same semaphore - even
		// if the returned handle is different.
		m_xSemaphoreHandle = CreateSemaphore( NULL, uInitialCount, uMaxCount, NULL );
	}

	~Asura_Platform_Semaphore()
	{
		CloseHandle( m_xSemaphoreHandle );
	}

protected:

	HANDLE m_xSemaphoreHandle;
};

/******************************************************************************/
/* Functions                                                                  */
/******************************************************************************/

void Asura_Xbox360_SetThreadName( const char* const szThreadName,
	const DWORD uThreadID = static_cast< DWORD >(-1) );

#endif // !defined __ASURA_PLATFORM_MULTITHREADTYPES_H__
