//Asura_XBox360_Notifications.cpp
//Copyright (C) 2005 Rebellion, All Rights Reserved.
//Thomas Young - 27/09/07

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __ASURA_XBOX360_NOTIFICATIONS_H__
#define __ASURA_XBOX360_NOTIFICATIONS_H__

#include <xonline.h>

typedef void(*Asura_Notify_Function)(u_long);

//structure to hold the functions
struct Asura_Notification_Structure
{
	DWORD						uID;
	Asura_Notify_Function		pfnProjectFunction;
	Asura_Notify_Function		pfnSystemFunction;
};

class Asura_XBox360_Notifications
{
	public:

		static void Initialise	();
		static void	Update		();
		static void	Shutdown	();

		//Call RegisterNotificationCallback passing in a Notification Identifier and function you wish to have called when the notification is received
		static void RegisterNotificationCallback	(DWORD uID, Asura_Notify_Function	pfnFunction);
		static void	ClearNotificationCallback		(DWORD uID);

		//call flush to check for any notifications and clear them
		static void Flush ();

		// Default Functions, these are functions which are called if the notification is not overridden
		static void Default_XN_SYS_UI (u_long uData);

		// System Functions, these are functions which require the notifications to call some asura code
		static void System_XN_SYS_UI( u_long uData );
		static void System_XN_SYS_SIGNINCHANGED( u_long uData );
		static void System_XN_SYS_STORAGEDEVICESCHANGED( u_long uData );
		static void	System_XN_LIVE_CONNECTIONCHANGED( u_long uData );
		static void	System_XN_LIVE_LINK_STATE_CHANGED( u_long uData );
		#ifdef ASURA_SUPPORT_LOBBY
		static void System_XN_FRIENDS_FRIEND_ADDED(u_long uData);
		static void System_XN_FRIENDS_FRIEND_REMOVED(u_long uData);
		static void System_XN_LIVE_INVITE_ACCEPTED(u_long uData);
		
		static bool IsGameInvitePending() { return s_bGameInvitePending; }
		static const XINVITE_INFO& GetGameInviteDetails();
		#endif
		

	private:
	
		static void RegisterSystemNotificationCallback (DWORD uID, Asura_Notify_Function pfnFunction);

		static HANDLE s_pNotificationHandle;
		#ifdef ASURA_SUPPORT_LOBBY
		static XINVITE_INFO s_xInviteInfo;
		static bool s_bGameInvitePending;
		#endif
};

#endif

//eof
