//Asura_XUI.cpp
//Copyright (c) 2008 Rebellion

// I N C L U D E S ///////////////////////////////////////////////////////////

//Main headers
#include "Core/Asura.h"

//Other asura headers
#include "Core/Memory/Asura_Mem.h"
#include "FileAccess/Asura_FileAccess.h"

//Platform headers
#include "Input/Asura_PlatformInput.h"
#include "Interface/Asura_Xbox360_XUI_System.h"
#include "Rendering/Asura_Xbox360_Graphics.h"

// C L A S S   D E C L A R A T I O N S //////////////////////////////////////

class ASURA_XBOX360_XUI_KEY_COMPARISON
{
public:

	WORD	uVirtualKey;	//I'm relying on none of these being zero
	u_int	uAsuraKey;
};

// G L O B A L   V A R I A B L E S ///////////////////////////////////////////

HXUIDC		Asura_Xbox360_XUI_System::s_sxDeviceContext	= 0;
HXUIOBJ		Asura_Xbox360_XUI_System::s_xCanvas			= 0;
HXUIOBJ		Asura_Xbox360_XUI_System::s_xScene			= 0;

u_int		Asura_Xbox360_XUI_System::s_uPreparedFlags	= 0;
bool		Asura_Xbox360_XUI_System::s_bEnabled		= false;

u_int		Asura_Xbox360_XUI_System::s_uSceneWidth		= 0;
u_int		Asura_Xbox360_XUI_System::s_uSceneHeight	= 0;

u_int		Asura_Xbox360_XUI_System::s_uBackBufferWidth  = 0;
u_int		Asura_Xbox360_XUI_System::s_uBackBufferHeight = 0;

static ASURA_XBOX360_XUI_KEY_COMPARISON	s_xKeyList[] =			//Build a list of asura keys vs virtual key codes
{
	{ VK_PAD_DPAD_DOWN		, ASURA_XBOX360_KEYID_PAD1_DOWN		},
	{ VK_PAD_DPAD_UP		, ASURA_XBOX360_KEYID_PAD1_UP		},
	{ VK_PAD_DPAD_LEFT		, ASURA_XBOX360_KEYID_PAD1_LEFT		},
	{ VK_PAD_DPAD_RIGHT		, ASURA_XBOX360_KEYID_PAD1_RIGHT	},
	{ VK_PAD_A				, ASURA_XBOX360_KEYID_PAD1_A		},
	{ VK_PAD_B				, ASURA_XBOX360_KEYID_PAD1_B		},
	{ VK_PAD_X				, ASURA_XBOX360_KEYID_PAD1_X		},
	{ VK_PAD_Y				, ASURA_XBOX360_KEYID_PAD1_Y		},
	{ VK_PAD_START			, ASURA_XBOX360_KEYID_PAD1_START	},
	{ VK_PAD_BACK			, ASURA_XBOX360_KEYID_PAD1_BACK		},
	//Add more here if you need them
	{ 0						, 0									}	//NULL terminate the list
};

// E N U M E R A T I O N S ///////////////////////////////////////////////////

enum ASURA_XBOX360_XUI_PREPARED_STATE
{
	XUI_PREPARED_STATE_XUI_INITED		= 0x00000001,

	XUI_PREPARED_STATE_CLASS_REGISTERED	= 0x00000010,

	XUI_PREPARED_STATE_TYPEFACE_LOADED	= 0x00000100,
	XUI_PREPARED_STATE_SKIN_LOADED		= 0x00000200,
	XUI_PREPARED_STATE_SCENE_LOADED		= 0x00000400,
};



// F U N C T I O N   D E F I N I T I O N S ///////////////////////////////////

void Asura_Xbox360_XUI_System::Initialise	()
{
	if(s_uPreparedFlags & XUI_PREPARED_STATE_XUI_INITED)
	{
		Asura_Assert(false,"XUI already initialised");
	}

	s_uPreparedFlags = 0;
	
	//Call after D3D is initialised
	SCOPED_DEVICE_OWNERSHIP();

	// Initialize the XUI runtime.
	XUIInitParams xParams;
	XUI_INIT_PARAMS( xParams );
	HRESULT xResult = XuiInit( &xParams );
	if( FAILED(xResult) ) 
	{
		Asura_Assert(false, "Failed to init XUI");
		return;
	}

	D3DPRESENT_PARAMETERS xD3DParams;
	Asura_Mem::Set( &xParams, 0, sizeof( xD3DParams ) );
	Asura_Xbox360_Graphics::GetDefaultParams( &xD3DParams );

	xResult = XuiRenderInitShared(Asura_Xbox360_Graphics::GetDevice(), &xD3DParams, XuiD3DXTextureLoader);
	if( FAILED ( xResult ) )
	{
		Asura_Assert(false, "Failed to init XUI render");
		return;
	}
	
	// Create a display context.
	xResult = XuiRenderCreateDC( &s_sxDeviceContext );
	if( FAILED( xResult ) )
	{
		Asura_Assert(false, "Failed to get XUI DC");
		return;
	}
	
	XuiCreateObject( XUI_CLASS_CANVAS, &s_xCanvas );
	//Get the back buffer and store it for later so we can centralise our xui scene
	XuiRenderGetBackBufferSize(s_sxDeviceContext, &s_uBackBufferWidth, &s_uBackBufferHeight );
	xResult = XuiElementSetBounds( s_xCanvas, (float)s_uBackBufferWidth,  (float)s_uBackBufferHeight );
	if( FAILED(xResult) ) 
	{
		Asura_Assert(false,"Failed to set XUI canvas size");
	}
	else
	{
		s_uPreparedFlags |= XUI_PREPARED_STATE_XUI_INITED;
	}
}


void Asura_Xbox360_XUI_System::Render()
{
	if(!s_bEnabled)
	{
		return;
	}
	if( !( s_uPreparedFlags & XUI_PREPARED_STATE_SCENE_LOADED ) )
	{
		Asura_Assert(false,"Attempting to render an XUI scene but no scene loaded");
		return;
	}

	SCOPED_DEVICE_OWNERSHIP();

	XUIMessageRender xMessageRender;
	XUIMessage xMessage;
	HRESULT xResult;

	xResult = XuiRenderBegin( s_sxDeviceContext, D3DCOLOR_ARGB(255, 0, 0, 0) );
	if( FAILED ( xResult ) ) 
	{
		Asura_Assert(false,"Failed to render XUI");
		return;
	}

	D3DXMATRIX xOriginal;
	XuiRenderGetViewTransform( s_sxDeviceContext, &xOriginal );
	
	if( (s_uBackBufferWidth > s_uSceneWidth) && (s_uBackBufferHeight > s_uSceneHeight))
	{
		// centralise the dialog
		D3DXMATRIX xNew;
		D3DXMatrixTranslation( &xNew, (static_cast<float>(s_uBackBufferWidth - s_uSceneWidth) * 0.5f), static_cast<float>(s_uBackBufferHeight - s_uSceneHeight) * 0.5f, 0 );
		XuiRenderSetViewTransform( s_sxDeviceContext, &xNew );
	}

	XuiMessageRender( &xMessage, &xMessageRender, s_sxDeviceContext, 0xFFFFFFFF, XUI_BLEND_NORMAL );
	XuiSendMessage( s_xCanvas, &xMessage );

	XuiRenderSetViewTransform( s_sxDeviceContext, &xOriginal );

	XuiRenderEnd( s_sxDeviceContext );
	
	XuiRenderPresent( s_sxDeviceContext, NULL, NULL, NULL );
	
}


void Asura_Xbox360_XUI_System::Shutdown	()
{
	XuiDestroyObject(s_xScene);
	XuiDestroyObject(s_xCanvas);
	XuiRenderDestroyDC( s_sxDeviceContext );
	XuiRenderUninit();
	XuiUninit();
}


bool Asura_Xbox360_XUI_System::Initialised	()
{
	if(s_uPreparedFlags & XUI_PREPARED_STATE_XUI_INITED)
	{
		return true;
	}
	return false;
}


bool Asura_Xbox360_XUI_System::RegisterClass (	Asura_UnicodeChar*					pxClass, 
												Asura_XUI_ObjectProc_Callback		pfnObjectProc, 
												Asura_XUI_CreateInstance_Callback	pfnCreate /*= 0*/, 
												Asura_XUI_DestroyInstance_Callback	pfnDestroy /*= 0*/ )
{
	if( !( s_uPreparedFlags & XUI_PREPARED_STATE_XUI_INITED ) )
	{
		Asura_Assert(false,"Calling XUI function, but XUI did not initialise");
		return false;
	}

	XUIClass xXUIClass;
	HXUICLASS xHXUIClassHandle;

	Asura_Mem::Set( &xXUIClass, 0, sizeof(xXUIClass) );
	
	xXUIClass.szClassName				= pxClass;
	xXUIClass.szBaseClassName			= XUI_CLASS_SCENE;
	xXUIClass.Methods.CreateInstance	= pfnCreate;
	xXUIClass.Methods.DestroyInstance	= pfnDestroy;
	xXUIClass.Methods.ObjectProc		= pfnObjectProc;
	xXUIClass.pPropDefs					= NULL;
	xXUIClass.dwPropDefCount			= 0;

	HRESULT xResult = XuiRegisterClass( &xXUIClass, &xHXUIClassHandle );
	if( FAILED ( xResult ) ) 
	{
		Asura_Assert(false,"Failed to create XUI class");
		return false;
	}
	s_uPreparedFlags |= XUI_PREPARED_STATE_CLASS_REGISTERED;
	return true;
}


bool Asura_Xbox360_XUI_System::LoadSkin	(Asura_UnicodeChar* pxSkinFile)
{
	if( !( s_uPreparedFlags & XUI_PREPARED_STATE_XUI_INITED ) )
	{
		Asura_Assert(false,"Calling XUI function, but XUI did not initialise");
		return false;
	}

	HRESULT xResult = XuiLoadVisualFromBinary( pxSkinFile, NULL );
	if( FAILED(xResult) ) 
	{
		Asura_Assertf(false,("Failed to load skin file '%s'", pxSkinFile));
		return false;
	}
	s_uPreparedFlags |= XUI_PREPARED_STATE_SKIN_LOADED;
	return true;
}


bool Asura_Xbox360_XUI_System::LoadTypeface (Asura_UnicodeChar* pxTypeface, Asura_UnicodeChar* pxLocator)
{
	if( !( s_uPreparedFlags & XUI_PREPARED_STATE_XUI_INITED ) )
	{
		Asura_Assert(false,"Calling XUI function, but XUI did not initialise");
		return false;
	}

	TypefaceDescriptor xTypeface;

	xTypeface.szTypeface = pxTypeface;
	xTypeface.szLocator = pxLocator;
	xTypeface.szReserved1 = NULL;
	xTypeface.fBaselineAdjust = 0.0;
	xTypeface.szFallbackTypeface = NULL;
	HRESULT xResult = XuiRegisterTypeface( &xTypeface, TRUE );

	if( FAILED ( xResult ) )
	{
		Asura_Assertf(false,("Failed to register typeface '%s'",pxLocator));
		return false;
	}
	s_uPreparedFlags |= XUI_PREPARED_STATE_TYPEFACE_LOADED;
	return true;
}


bool Asura_Xbox360_XUI_System::LoadScene (Asura_UnicodeChar* pxScene)
{
	if( !( s_uPreparedFlags & XUI_PREPARED_STATE_XUI_INITED ) )
	{
		Asura_Assert(false,"Calling XUI function, but XUI did not initialise");
		return false;
	}

	HRESULT xResult = XuiSceneCreate( NULL, pxScene, NULL, &s_xScene );
	if( FAILED( xResult ) ) 
	{
		Asura_Assertf(false,("Failed to load XUI scene '%s'", pxScene));
		return false;
	}
	else
	{
		//Save the width and height
		s_uSceneWidth = 1024;
		s_uSceneHeight = 720;

		DWORD xSizeResult;
		if( SUCCEEDED ( XuiObjectGetPropertyId (s_xScene, L"Width", &xSizeResult) && xSizeResult) )
		{
			XUIElementPropVal xProp;
			if( SUCCEEDED ( XuiObjectGetProperty (s_xScene, xSizeResult, 0, &xProp)))
			{
				s_uSceneWidth = static_cast<u_int>(xProp.fVal);
			}
		}

		if( SUCCEEDED ( XuiObjectGetPropertyId (s_xScene, L"Height", &xSizeResult) && xSizeResult) )
		{
			XUIElementPropVal xProp;
			if( SUCCEEDED ( XuiObjectGetProperty (s_xScene, xSizeResult, 0, &xProp)))
			{
				s_uSceneHeight = static_cast<u_int>(xProp.fVal);
			}
		}
	}
	s_uPreparedFlags |= XUI_PREPARED_STATE_SCENE_LOADED;
	return true;
}


bool Asura_Xbox360_XUI_System::ShowFirstScene	()
{
	if( !( s_uPreparedFlags & XUI_PREPARED_STATE_XUI_INITED ) )
	{
		Asura_Assert(false,"Calling XUI function, but XUI did not initialise");
		return false;
	}
	if( !( s_uPreparedFlags & XUI_PREPARED_STATE_SCENE_LOADED ) )
	{
		Asura_Assert(false,"Attempting to show an XUI scene but no scene loaded");
		return false;
	}
	// Activate the first scene.
	HRESULT xResult = XuiSceneNavigateFirst( s_xCanvas, s_xScene, XUSER_INDEX_ANY );
	if( FAILED ( xResult ) )
	{
		Asura_Assert(false, "Failed to navigate to first XUI scene");
		return false;
	}
	HXUIOBJ	xObject;
	HRESULT xRes = XuiElementGetChildById(s_xScene,L"XuiText1",&xObject);
	if(FAILED(xRes))
	{
		Asura_Assert(false, "faield");
		return true;
	}
	XuiTextElementSetText(xObject,L"This is a test");

	return true;
}

void Asura_Xbox360_XUI_System::Update (float fFrameLength)
{
	if(!s_bEnabled)
	{
		return;
	}

	//Update our animations
	XuiAnimRun(fFrameLength);

	//Update any timers
	XuiTimersRun();

	//Update the user input, note that Asura_UserInput needs updating before this
	XINPUT_KEYSTROKE xInput;
	u_int uKeyIndex = 0;
	while(s_xKeyList[uKeyIndex].uVirtualKey)
	{
		Asura_Mem::Set(&xInput,0,sizeof(XINPUT_KEYSTROKE));

		if(Asura_UserInput::DebouncedKeyPressed(s_xKeyList[uKeyIndex].uAsuraKey))
		{
			xInput.VirtualKey	= s_xKeyList[uKeyIndex].uVirtualKey;
			xInput.Flags		= XINPUT_KEYSTROKE_KEYDOWN;
			XuiProcessInput(&xInput);
		}
		++uKeyIndex;
	}
}


//eof

