
///////////////////////////////////////////////////////////////////////////////
//
// File   : Asura_Platform_NetTransport_System.cpp
// Author : Andy Tate
// Date   : 16-Oct-2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "Network/Asura_NetTransport_System.h"

#if defined(ASURA_SUPPORT_EXTERNAL_NETWORK)


// Asura independent includes
#include "Core/Asura_Maths.h"
#include "Core/Asura_Timers.h"
#include "Core/Asura_Maths.h"
#include "Debugging/Asura_Logfile.h"
#include "Network/Asura_LanGameFinder.h"
#include "Network/Asura_Network_Session.h"
#include "Network/Asura_Network.h"
#include "Network/Asura_NetSocket.h"
#include "Network/Asura_NetConnection.h"

// Asura 360 includes
#include "Network/Asura_NetSocket_Xbox360.h"
#include "Network/Online/Asura_Xbox360_Voice.h"


#ifdef ASURA_XBOX360_BYPASS_NETWORK_SECURITY

void Asura_NetTransport_System::Platform_Initialise()
{
}

void Asura_NetTransport_System::Platform_Update()
{
}

void Asura_NetTransport_System::Platform_Shutdown()
{
}

void Asura_NetTransport_System::Platform_StartServer(bool bInternetGame)
{
}

void Asura_NetTransport_System::Platform_StopServer()
{
}

bool Asura_NetTransport_System::Platform_SendEnumHostsBroadcast()
{
	return Asura_LanGameFinder::SendEnumHostsBroadcast();
}

void Asura_NetTransport_System::Platform_ProcessEnumHostsRequest( Asura_NetAddress xAddress, u_short usPort, const char* pcData, u_int uDataLength )
{
	Asura_LanGameFinder::ProcessEnumHostsRequest(xAddress, usPort, pcData, uDataLength);
}

void Asura_NetTransport_System::Platform_SendEnumHostsResponse( Asura_NetAddress xAddress, u_short usPort)
{
	Asura_LanGameFinder::SendEnumHostsResponse(xAddress, usPort);
}

void Asura_NetTransport_System::Platform_ProcessEnumHostsResponse( Asura_NetAddress xAddress, u_short usPort, const char* pcData, int iDataSize )
{
	Asura_LanGameFinder::ProcessEnumHostsResponse(xAddress, usPort, pcData, iDataSize);
}


void Asura_NetTransport_System::Platform_Client_ClientConnected(Asura_NetConnection* pxConnection)
{
	// KOR - This empty function was originally used for initiating the peer-to-peer voice message
	// exchange, which was moved to the Platform_StartGame (see below) to stop it overflowing during load.
	// It may come in useful later or on other platforms though.
}

void Asura_NetTransport_System::Platform_Client_ClientDisconnected(Asura_NetConnection* pxConnection)
{
	// Do nothing
}

void Asura_NetTransport_System::Platform_StartGame()
{
}

bool Asura_NetTransport_System::Platform_HandleMessage( Asura_NetAddress xAddress, u_short usPort, const char* pcMessage, int iLength)
{
	return false;
}

u_int Asura_NetTransport_System::Platform_GetNetAdaptorIP( int iAdapter ) 
{ 	
	// TODO
	return 0;
}

const char* Asura_NetTransport_System::Platform_GetStringFromAddress(Asura_NetAddress xAddress, unsigned short usPort)
{
	return Asura_NetSocket_Xbox360::GetStringFromAddress( xAddress, usPort );
}

void Asura_NetTransport_System::Platform_GetAddressFromString(const char* szAddress, Asura_NetAddress* pxAddress, unsigned short* pusPort)
{
	Asura_NetSocket_Xbox360::GetAddressFromString( szAddress, pxAddress, pusPort );
}

Asura_NetSocket* Asura_NetTransport_System::Platform_AllocateNetSocket( u_int uMaxMachines )
{
	return new Asura_NetSocket_Xbox360( uMaxMachines );
}

bool Asura_Network_SessionInfo::Platform_IsSameHost(Asura_Network_SessionInfo* pxSession)
{
	return ((m_xHostAddr == pxSession->m_xHostAddr) && (m_usHostPort == pxSession->m_usHostPort));
}

#else

DEBUGVAR int dbg_iNumPortsToScan = 0;

struct Asura_Network_LANBroadcastServerRequest
{
	u_short		m_usSequenceID;		// = 0
	u_short		m_usChecksum;
	u_short		m_usMagicID;		// = ASURA_NETTRANSPORT_MSG_ENUMHOSTS
	u_short		m_usGameVersion;
	u_int		m_uApplicationGuid;
	u_int		m_uNonce;
};

struct Asura_Network_LANBroadcastServerReplyHeader
{
	u_short				m_usSequenceID;		// = 0
	u_short				m_usChecksum;
	u_short				m_usMagicID;		// = ASURA_NETTRANSPORT_MSG_HOSTREPLY
	u_short				m_usGameVersion;
	u_int				m_uApplicationGuid;
	u_int				m_uNonce;
	u_int				m_uServerSessionID;
	XNADDR				m_xXNetAddress;
	XNKID				m_xXNetKeyID;
	XNKEY				m_xXNetKey;
	u_short				m_usServerPort;		// not the same as the broadcast port, which is where this message will come in
	u_short				m_usDataSize;		// this data is followed by this many bytes of user data
	Asura_UnicodeChar	m_wszSessionName[uASURA_NETWORK_MAX_SESSION_NAME + 1];
	u_char				m_ucNumPlayers;
	u_char				m_ucMaxPlayers;

};

static XNADDR	g_xXNetAddress;
static XNKID	g_xXNetKeyID;
static XNKEY	g_xXNetKey;
static u_int	g_uNonce;
static bool		g_bHaveLocalAddress = false;

static Asura_NetSocket* g_pxBroadcastSocket = NULL;


static void CreateBroadcastSocket(u_int uMaxNumberOfMachines)
{
	if(g_pxBroadcastSocket) return;

	Asura_OutputToDebugger(("XANTS: Creating broadcast socket\n"));

	g_pxBroadcastSocket = new Asura_NetSocket_Xbox360(uMaxNumberOfMachines);
	g_pxBroadcastSocket->UseAlternativeProtocol();
	u_short uPort = ASURA_NETSOCKET_ALTERNATIVE_PORT;
	g_pxBroadcastSocket->Bind(0, uPort);
	g_pxBroadcastSocket->SetBroadcastEnabled(true);
}

static void DestroyBroadcastSocket()
{
	if(g_pxBroadcastSocket)
	{
		Asura_OutputToDebugger(("XANTS: Destroying broadcast socket\n"));
		g_pxBroadcastSocket->Close();
		Asura_NetSocket::Destroy(g_pxBroadcastSocket);
	}
	g_pxBroadcastSocket = NULL;
}





void Asura_NetTransport_System::Platform_Initialise (void)
{


	while( !g_bHaveLocalAddress )
	{
		DWORD dwStatus = XNetGetTitleXnAddr( &g_xXNetAddress );
		Asura_Assert( !(dwStatus & XNET_GET_XNADDR_NONE), "Error getting XNet address");
#if _DEBUG
		// XNET_GET_XNADDR_PENDING = 0, so bitwise and fails miserably.
		//		if(dwStatus == XNET_GET_XNADDR_PENDING) Asura_OutputToDebugger(("The address aquisition is not yet complete.\n"));
		if(dwStatus & XNET_GET_XNADDR_DHCP) Asura_OutputToDebugger(("The host has a dynamic IP address via DHCP.\n"));
		if(dwStatus & XNET_GET_XNADDR_DNS) Asura_OutputToDebugger(("The host has one or more DNS servers configured.\n"));
		if(dwStatus & XNET_GET_XNADDR_ETHERNET) Asura_OutputToDebugger(("The host has an Ethernet address.\n"));
		if(dwStatus & XNET_GET_XNADDR_GATEWAY) Asura_OutputToDebugger(("The host has one or more Internet gateways configured.\n"));
		if(dwStatus & XNET_GET_XNADDR_NONE) Asura_OutputToDebugger(("Either the network stack is not initialized or no debugger was found.\n"));
		if(dwStatus & XNET_GET_XNADDR_ONLINE) Asura_OutputToDebugger(("The host is currently connected to the online service.\n"));
		if(dwStatus & XNET_GET_XNADDR_PPPOE) Asura_OutputToDebugger(("Host has a PPPoE assigned IP address.\n"));
		if(dwStatus & XNET_GET_XNADDR_STATIC) Asura_OutputToDebugger(("The host has a static IP address.\n"));
		if(dwStatus & XNET_GET_XNADDR_TROUBLESHOOT) Asura_OutputToDebugger(("The network configuration requires troubleshooting\n"));
#endif
		if( dwStatus != XNET_GET_XNADDR_PENDING )
		{
			Asura_OutputToDebugger(("Got an address, at last.\n"));
			g_bHaveLocalAddress = true;
		}
	}
	XNetGetBroadcastVersionStatus( TRUE );
	g_uNonce = Asura_Random::GetIntU();
}

void Asura_NetTransport_System::Platform_Update (void)
{
	if(g_pxBroadcastSocket)
	{
		g_pxBroadcastSocket->Update();
	}
}

void Asura_NetTransport_System::Platform_Shutdown (void)
{
	DestroyBroadcastSocket();


}

void Asura_NetTransport_System::Platform_StartServer (bool bInternetGame)
{
	if(!bInternetGame)
	{
		CreateBroadcastSocket(Asura_NetTransport_System::s_uMaxNumberOfMachines);
		g_pxBroadcastSocket->SetListening(true);

		// Create encryption key
#ifdef ASURA_ASSERT
		int iKeyCreated = 
#endif
			XNetCreateKey( &g_xXNetKeyID, &g_xXNetKey );
		Asura_Assert(iKeyCreated == NO_ERROR, "Error creating XNetkey.");
		// Register the session using the key
#ifdef ASURA_ASSERT
		int iKeyRegistered = 
#endif
			XNetRegisterKey( &g_xXNetKeyID, &g_xXNetKey );
		Asura_Assert(iKeyRegistered == NO_ERROR, "Error registering XNetkey.");
	}
	else
	{
//		PlatformProject_StartInternetSession();
	}
}

void Asura_NetTransport_System::Platform_StopServer (void)
{
	// session deletion takes care of unregistering our key for us when we're on the 'net
	if(!Asura_Network::IsPlayingOnInternet())
	{
		XNetUnregisterKey(&g_xXNetKeyID);
		DestroyBroadcastSocket();
	} else {
		//	PlatformProject_StopInternetSession();
	}

}

bool Asura_NetTransport_System::Platform_SendEnumHostsBroadcast (void)
{
	CreateBroadcastSocket(Asura_NetTransport_System::s_uMaxNumberOfMachines);
	bool iResult = false;
	u_int uBroadcastAddr = INADDR_BROADCAST;	// 255.255.255.255
	for(int i = -1; i < dbg_iNumPortsToScan; ++i)
	{
		u_short usPort = (i == -1) ? ASURA_NETSOCKET_ALTERNATIVE_PORT : static_cast<u_short>(ASURA_NETSOCKET_FIRST_DYNAMIC_PORT + i);

		Asura_Network_LANBroadcastServerRequest xBroadcastMsg;
		xBroadcastMsg.m_usSequenceID = 0;
		xBroadcastMsg.m_usMagicID = ASURA_NETTRANSPORT_MSG_ENUMHOSTS;
		xBroadcastMsg.m_usGameVersion = Project_GetVersionID();
		xBroadcastMsg.m_uApplicationGuid = Project_GetApplicationGuid();
		xBroadcastMsg.m_uNonce = g_uNonce;

		xBroadcastMsg.m_usChecksum = ComputeChecksum( reinterpret_cast<const char*>(&xBroadcastMsg.m_usMagicID), sizeof(xBroadcastMsg) - 4 );

		Asura_Assert(g_pxBroadcastSocket, "No socket");
		if(g_pxBroadcastSocket)
		{
			iResult = g_pxBroadcastSocket->Send(uBroadcastAddr, usPort, reinterpret_cast<const char *>(&xBroadcastMsg), sizeof(xBroadcastMsg));
			Asura_Assert(iResult, "failed to send broadcast message");

			AddDirectSendBytes(sizeof(xBroadcastMsg));
		}
	}

	s_fLastSessionRequestBroadcastTime = Asura_Timers::GetLocalUnscaledTime();
	return iResult;
}

void Asura_NetTransport_System::Platform_SendEnumHostsResponse (Asura_NetAddress xAddress, u_short usPort)
{
	// (MH - 01.07.2004) ensure the session data is fresh
	Project_RefreshSessionData();

	u_int uSizeOfReplyData = sizeof(Asura_Network_LANBroadcastServerReplyHeader) + s_uSessionDataSize;
	char* const pcReplyData = new char[uSizeOfReplyData];

	Asura_Network_LANBroadcastServerReplyHeader* pxHeader = reinterpret_cast
		<Asura_Network_LANBroadcastServerReplyHeader*>(pcReplyData);
	pxHeader->m_usSequenceID = 0;
	// checksum filled out later
	pxHeader->m_usMagicID = ASURA_NETTRANSPORT_MSG_HOSTREPLY;
	pxHeader->m_usGameVersion = Project_GetVersionID();
	pxHeader->m_uApplicationGuid = Project_GetApplicationGuid();
	pxHeader->m_uServerSessionID = s_uServerSessionID;
	pxHeader->m_usServerPort = s_usPort;
	pxHeader->m_xXNetAddress = g_xXNetAddress;
	pxHeader->m_xXNetKey = g_xXNetKey;
	pxHeader->m_xXNetKeyID = g_xXNetKeyID;
	pxHeader->m_uNonce = xAddress;
	pxHeader->m_usDataSize = static_cast<u_short>(s_uSessionDataSize);
	pxHeader->m_ucNumPlayers = static_cast<u_char>(Asura_Network::GetNumberOfPlayers());
	pxHeader->m_ucMaxPlayers = static_cast<u_char>(Asura_Network::GetMaxNumberOfPlayers());
	Asura_PrintfToBfr(pxHeader->m_wszSessionName, L"%s", uASURA_NETWORK_MAX_SESSION_NAME + 1, true, Asura_Network::GetServerName());

	u_char* pucSessionData = reinterpret_cast<u_char*>(pxHeader + 1);
	memcpy(pucSessionData, s_pucSessionData, s_uSessionDataSize);

	// checksum on length of data after SeqNum and checksum
	pxHeader->m_usChecksum = ComputeChecksum( reinterpret_cast<const char*>(&pxHeader->m_usMagicID), uSizeOfReplyData - 4 );

#if defined ASURA_ASSERT
	const bool bResult =
#endif
		g_pxBroadcastSocket->Send(INADDR_BROADCAST, usPort, pcReplyData, uSizeOfReplyData);

#if defined ASURA_ASSERT
	if (!bResult)
	{
		int Err = WSAGetLastError();

		Asura_Assertf(0, ("failed to send broadcast message %d", Err));


	}
#endif

	Asura_NetTransport_System::AddDirectSendBytes(uSizeOfReplyData);

	delete[] pcReplyData;

}

void Asura_NetTransport_System::Platform_ProcessEnumHostsRequest (Asura_NetAddress xAddress, u_short usPort, const char* pcData, u_int uDataLength)
{
	const Asura_Network_LANBroadcastServerRequest* pxBroadcastRequestMsg = reinterpret_cast
		<const Asura_Network_LANBroadcastServerRequest*>(pcData);

	// Ignore requests from other games or versions
	if (pxBroadcastRequestMsg->m_uApplicationGuid != Project_GetApplicationGuid())
	{
		return;
	}

	if (pxBroadcastRequestMsg->m_usGameVersion != Project_GetVersionID())
	{
		return;
	}

	if (uDataLength != sizeof(Asura_Network_LANBroadcastServerRequest))
	{
		Asura_Assert(false, "bad data length in enum hosts");
		return;
	}

	// check the checksum
	if (pxBroadcastRequestMsg->m_usChecksum != 
		ComputeChecksum( reinterpret_cast<const char*>(&pxBroadcastRequestMsg->m_usMagicID), uDataLength - 4 ))
	{
		Asura_Assert(false, "bad checksum in enum hosts");
		return;
	}

	Asura_OutputToDebugger(("Received enum hosts request from %0X:%i with %i bytes of data.\n", xAddress, usPort, uDataLength));
	Platform_SendEnumHostsResponse( pxBroadcastRequestMsg->m_uNonce, usPort );

}

void Asura_NetTransport_System::Platform_ProcessEnumHostsResponse (Asura_NetAddress xAddress, u_short usPort, const char* pcData, int iDataSize)
{
	const Asura_Network_LANBroadcastServerReplyHeader* pxHeader = reinterpret_cast
		<const Asura_Network_LANBroadcastServerReplyHeader*>(pcData);
	const u_char* pucSessionData = reinterpret_cast<const u_char*>(pxHeader + 1);

	// check project and version
	if (pxHeader->m_uApplicationGuid != Project_GetApplicationGuid()) return;
	if (pxHeader->m_usGameVersion != Project_GetVersionID()) return;

	// check to see if we sent it!
	if (pxHeader->m_uNonce != g_uNonce) return;

	// check data length
	if (static_cast<u_int>(iDataSize) != sizeof(Asura_Network_LANBroadcastServerReplyHeader) + pxHeader->m_usDataSize)
	{
		Asura_Assert(false, "wrong data length in enum hosts response");
		return;
	}

	// check the checksum
	if (pxHeader->m_usChecksum != ComputeChecksum( reinterpret_cast<const char*>(&pxHeader->m_usMagicID), iDataSize - 4 ))
	{
		Asura_Assert(false, "bad checksum in enum hosts response");
		return;
	}

	if (!s_ppxSessionsStorage) return;

	// work out ping based on time since message was sent. Possible problem of
	// overlapping response and request...
	u_int uPing = Asura_Maths::FloatToInt(1000.0f * (Asura_Timers::GetLocalUnscaledTime() - s_fLastSessionRequestBroadcastTime));


	// RJF - 16 February 2005 - this connection code has been removed because it (a) was actually connecting to all hosts that respond to us and
	// (b) wasn't saving security keys anywhere, which meant that we were running out of key slots (we can only have four keys registered at once).
	// The connection data (XNADDR, XNKID, XNKEY) is now saved into the session's platform data.

	// IT IS THE PROJECT'S RESPONSIBLITY TO ESTABLISH THE SECURE CONNECTION ON A SESSION ONCE A SESSION HAS BEEN SELECTED. Connections to a session
	// that have no secure connection WILL NOT WORK.

	xAddress = 0; // IP address is invalid because it's not secure
	usPort = pxHeader->m_usServerPort;

	// pack the Xbox-specific data into a buffer to be glued to the session object
	u_char uBuffer[sizeof(XNADDR) + sizeof(XNKID) + sizeof(XNKEY)];
	memcpy(uBuffer, &pxHeader->m_xXNetAddress, sizeof(XNADDR));
	memcpy(uBuffer + sizeof(XNADDR), &pxHeader->m_xXNetKeyID, sizeof(XNKID));
	memcpy(uBuffer + sizeof(XNADDR) + sizeof(XNKID), &pxHeader->m_xXNetKey, sizeof(XNKEY));

	Asura_Network_SessionInfo* pxNewSession = new Asura_Network_SessionInfo(xAddress, 
		usPort, uPing, pxHeader->m_uServerSessionID, uBuffer, sizeof(uBuffer), pucSessionData, pxHeader->m_usDataSize, pxHeader->m_wszSessionName, 
		pxHeader->m_ucNumPlayers, pxHeader->m_ucMaxPlayers);

	for (u_int u = 0; u < s_uNumberOfActiveSessions; ++u)
	{
		if (s_ppxSessionsStorage[u]->Platform_IsSameHost(pxNewSession))
		{
			// SNH: copied from xbox version. Replace the session details.
			delete s_ppxSessionsStorage[u];
			s_ppxSessionsStorage[u] = pxNewSession;
			return;
		}
	}

	// this is a new server
	if (s_uNumberOfActiveSessions < uASURA_NETWORK_MAX_NUMBER_OF_SESSIONS)
	{
		s_ppxSessionsStorage[s_uNumberOfActiveSessions++] = pxNewSession;
	}
	else
	{
		delete pxNewSession;
	}

}

void Asura_NetTransport_System::Platform_Client_ClientConnected (Asura_NetConnection* pxConnection)
{
	// KOR - This empty function was originally used for initiating the peer-to-peer voice message
	// exchange, which was moved to the Platform_StartGame (see below) to stop it overflowing during load.
	// It may come in useful later or on other platforms though.

}

void Asura_NetTransport_System::Platform_Client_ClientDisconnected (Asura_NetConnection* pxConnection)
{
	// possibly temporary until xbox live is up and running but
	// needs to be done for LAN play at a minimum
	if (!Asura_Network::IsPlayingOnInternet())
	{
		IN_ADDR xInAddr;
		xInAddr.S_un.S_addr = pxConnection->GetAddress();
		XNetUnregisterInAddr(xInAddr);
	}
}

bool Asura_NetTransport_System::Platform_HandleMessage (Asura_NetAddress xAddress, u_short usPort, const char* pcMessage, int iLength)
{
#ifdef ASURA_SUPPORT_LOBBY
	const u_short usMessageID = *reinterpret_cast<const u_short*>(pcMessage);
	if (usMessageID == ASURA_NETTRANSPORT_MSG_VOICE_DATA)
	{
		return Asura_Xbox360_Voice::HandleMessage(pcMessage, iLength);
	}
#endif
	return false;
}

void Asura_NetTransport_System::Platform_StartGame (void)
{
}

u_int Asura_NetTransport_System::Platform_GetNetAdaptorIP (int iAdapter)
{
	return 0;
}

void Asura_NetTransport_System::Platform_GetAddressFromString (const char* szAddress, Asura_NetAddress* xAddress, unsigned short* pusPort)
{
	Asura_NetSocket_Xbox360::GetAddressFromString( szAddress, xAddress, pusPort );
}

const char* Asura_NetTransport_System::Platform_GetStringFromAddress (Asura_NetAddress xAddress, unsigned short usPort)
{
	return Asura_NetSocket_Xbox360::GetStringFromAddress(xAddress, usPort);
}

Asura_NetSocket* Asura_NetTransport_System::Platform_AllocateNetSocket (u_int uMaxMachines)
{
	return new Asura_NetSocket_Xbox360( uMaxMachines );
}

bool Asura_Network_SessionInfo::Platform_IsSameHost(Asura_Network_SessionInfo* pxSession)
{
	// compare XNADDRs
	const XNADDR* pxMyAddr	= reinterpret_cast<const XNADDR*>(GetPlatformData());
	const XNADDR* pxTheirAddr = reinterpret_cast<const XNADDR*>(pxSession->GetPlatformData());

	return memcmp(pxMyAddr, pxTheirAddr, sizeof(XNADDR)) == 0;
}


#endif // ndef ASURA_XBOX360_BYPASS_NETWORK_SECURITY

#endif


/// End of file. ///
