//////////////////////////////////////////////////////////////////////////////
//
// Asura_Xbox360_LiveSession.h
//
// Copyright (c) 2008 Rebellion. All rights reserved.
//
// Description:
//
//     class to wrap Live! sessions. Based on CSession from Sessions sample
//     code. This class is intended to be completely independent of 
//     Asura_Lobby.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef ASURA_XBOX360_LIVESESSION_H___
#define ASURA_XBOX360_LIVESESSION_H___ 1

// do not include Asura_Lobby.h etc.
#include <xbox.h>

// E N U M S /////////////////////////////////////////////////////////////////
enum ASURA_XBOX360_LIVESESSION_STATE
{
	ASURA_XBOX360_LIVESESSION_STATE_NONE,
	ASURA_XBOX360_LIVESESSION_STATE_CREATING,
	ASURA_XBOX360_LIVESESSION_STATE_IDLE,
	ASURA_XBOX360_LIVESESSION_STATE_REGISTERING,
	ASURA_XBOX360_LIVESESSION_STATE_REGISTERED,
	ASURA_XBOX360_LIVESESSION_STATE_STARTING,
	ASURA_XBOX360_LIVESESSION_STATE_IN_GAME,
	ASURA_XBOX360_LIVESESSION_STATE_ENDING,
	ASURA_XBOX360_LIVESESSION_STATE_FINISHED,
	ASURA_XBOX360_LIVESESSION_STATE_DELETING,
	ASURA_XBOX360_LIVESESSION_STATE_MIGRATING,
	ASURA_XBOX360_LIVESESSION_STATE_ERROR
};

// C L A S S   D E C L A R A T I O N /////////////////////////////////////////
class Asura_Xbox360_LiveSession
{
public:

	Asura_Xbox360_LiveSession( u_int uOwnerController, bool bIsHost,
		bool bIsRanked, bool bAllowJoinInProgress, u_int uPublicSlots, u_int uPrivateSlots );
	~Asura_Xbox360_LiveSession();

	void CreateSession();
	void MigrateSession( const XSESSION_INFO* pxNewSession = NULL );

	void ModifySessionFlags( u_int uFlags );

	HANDLE GetSessionHandle() const  { return m_xSession; }

	bool IsSameSession( const XSESSION_INFO& sessionInfo ) const;

	const XSESSION_INFO& GetSessionInfo() const                   { return m_xSessionInfo; }
	void SetSessionInfo( const XSESSION_INFO& sessionInfo )
	{
		memcpy( &m_xSessionInfo, &sessionInfo, sizeof( m_xSessionInfo ) );
	}

	u_int GetSessionFlags() const                   { return m_uSessionFlags; }
	bool HasSessionFlags( u_int uFlags ) const      { return (m_uSessionFlags & uFlags) != 0; }
	void    SetSessionFlags( u_int uFlags, bool clear = FALSE )
	{
		if( clear )
			m_uSessionFlags = uFlags;
		else
			m_uSessionFlags |= uFlags;
	}
	void    FlipSessionFlags( u_int uFlags )     { m_uSessionFlags ^= uFlags; }
	void    ClearSessionFlags( u_int uFlags )    { m_uSessionFlags &= ~uFlags; }

	bool IsHost() const                { return m_bIsHost; }
	void SetHost( bool bIsHost )     { m_bIsHost = bIsHost; }

	bool IsRanked() const                { return m_bIsRanked; }
	void SetRanked( bool bIsRanked )     { m_bIsRanked = bIsRanked; }

	uint64		GetSessionNonce() const                  { return m_ulSessionNonce; }
	void        SetSessionNonce( uint64 ulNonce )  { m_ulSessionNonce = ulNonce; }
	u_int       GetSessionOwner() const                  { return m_uOwnerController; }
	void        SetSessionOwner( u_int uOwnerController )      { m_uOwnerController = uOwnerController; }

	const XSESSION_REGISTRATION_RESULTS* GetRegistrationResults() const	{ return m_pxRegistrationResults; }

	void StartQoSListener( u_char* pucData, u_int uDataLen, u_int uBitsPerSec );
	void StopQoSListener();

	void Update();

	ASURA_XBOX360_LIVESESSION_STATE GetSessionState() const { return m_eSessionState; }
	void SwitchToState( ASURA_XBOX360_LIVESESSION_STATE eNewState );

	void AddLocalPlayer(bool bInvited);
	void AddRemotePlayers(u_int uNumPlayers, XUID* pxXUIDs, BOOL* pxInvited);
	void RemoveLocalPlayer();
	void RemoveRemotePlayers(u_int uNumPlayers, XUID* pxXUIDs);
	bool IsPlayerInSession(XUID xXUID) const;
	u_int GetNumPublicSlotsFilled() const { return m_uNumPublicSlotsFilled; }
	u_int GetNumPrivateSlotsFilled() const { return m_uNumPrivateSlotsFilled; }
	XUID GetXUIDForPrivateSlot(u_int uSlot) const;
	XUID GetXUIDForPublicSlot(u_int uSlot) const;

	void RegisterForArbitration();

private:

	void UpdateCreating();
	void UpdateRegistering();
	void UpdateStarting();
	void UpdateEnding();
	void UpdateDeleting();
	void UpdateMigrating();

private:

	HANDLE             m_xSession;                  // Session handle
	bool               m_bIsHost;                   // Is hosting
	bool               m_bIsRanked;                   // Is a ranked match
	bool               m_bUsingQoS;                 // Is the QoS listener enabled
	XSESSION_INFO      m_xSessionInfo;               // Session ID, key, and host address
	XSESSION_INFO	   m_xMigratingSessionInfo;		// New session info for a migrated lobby, while the migration is occurring
	uint64             m_ulSessionNonce;              // Nonce of the session
	u_int              m_uSessionFlags;            // Session creation flags
	u_int              m_uOwnerController;          // Which controller created the session
	XOVERLAPPED        m_xOverlapped;                // Overlapped task data

	// public/private slots for the session
	u_int m_uNumPrivateSlotsFilled;
	u_int m_uTotalPrivateSlots;
	u_int m_uNumPublicSlotsFilled;
	u_int m_uTotalPublicSlots;
	XUID* m_pxPublicXUIDs; // list of XUIDs of players occupying public slots.
	XUID* m_pxPrivateXUIDs; // list of XUIDs of players occupying private slots.

	// Arbitration registration
	XSESSION_REGISTRATION_RESULTS* m_pxRegistrationResults;

	ASURA_XBOX360_LIVESESSION_STATE      m_eSessionState;


};

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

#endif // #ifndef ASURA_XBOX360_LIVESESSION_H___
