//////////////////////////////////////////////////////////////////////////////
//
// Asura_Xbox360_Stats.h
//
// Copyright (c) 2008 Rebellion. All rights reserved.
//
// Description:
//
//     classes for reading/writing stats to 360 Live! leaderboards
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef ASURA_XBOX360_STATS_H___
#define ASURA_XBOX360_STATS_H___ 1

#include <Xbox.h>
#include <Xonline.h>

class Asura_Xbox360_Stats
{
public:

#ifdef ASURA_DEBUG_VARIABLE_SYSTEM
	static void ClearAllStats();	// for clearing leaderboards during development - has no effect in production environment
#endif

	// project functions.
	static u_int Project_GetLeaderboardID( u_int uLeaderboard );	// returns ID from project spa.h file
	static u_int Project_GetTrueSkillLeaderboardID( bool bRanked );
	static u_short Project_GetColumnID( u_int uLeaderboard, u_int uColumn );	// returns ID from project spa.h file
	// Project_GetStat() must fill out the XUSER_PROPERTY with the property ID from the spa.h file
	// and the XUSER_DATA value as appropriate. If it fails (e.g. player wasn't found or stat doesn't
	// exist then it returns false and the xStat data must be ignored). Otherwise returns true.
	static bool Project_GetStat( XUID xXuid, bool bDisconnectedEarly, u_int uGuid, 
								 u_int uLeaderboard, u_int uStat, XUSER_PROPERTY& xStat );

	// Project_GetTrueSkillRelativeScore() and Project_GetTrueSkillTeam should return values appropriate
	// for TrueSkill, see "Writing to TrueSkill Leaderboards" in the XDK docs.
	static u_int Project_GetTrueSkillRelativeScore( bool bDisconnectedEarly, u_int uGuid );
	static u_int Project_GetTrueSkillTeam( u_int uGuid );
};

class Asura_Xbox360_StatsWriter
{
public:

	Asura_Xbox360_StatsWriter(HANDLE xSession, bool bRanked); // unranked sessions only write trueskill
	~Asura_Xbox360_StatsWriter();

	void WriteStatsForPlayer( bool bDisconnectedEarly, u_int uGuid, XUID xXuid );

private:

	// private data
	HANDLE m_xSession;
	bool m_bRanked;
	u_int m_uNumLeaderboards;
	u_int m_uTotalUserProperties;
	XSESSION_VIEW_PROPERTIES* m_pxViewProperties;
	XUSER_PROPERTY* m_pxUserProperties;
};

class Asura_Xbox360_StatsReader
{
public:
	Asura_Xbox360_StatsReader();
	~Asura_Xbox360_StatsReader();

	const XUSER_STATS_VIEW* GetData();

	bool Update();			// returns true on completion of a pending request.
	void FreeData();		// destroys the buffer after data has been read out of it.
	bool HasErrorOccurred() const { return m_bError; }

	bool FetchStats( u_int uStart, u_int uNumRows, const XUSER_STATS_SPEC& xStatsSpec );
	// NB: xStatsSpec must not be created on the stack when calling FetchStatsForXuids()
	bool FetchStatsForXuids( const XUID* pxXuids, u_int uNumXuids, const XUSER_STATS_SPEC& xStatsSpec );

private:

	// project functions to convert between enum from 0-n leaderboards/columns 
	// and the IDs in the project.spa.h file
	static u_int Project_ConvertLeaderboardIndex(u_int uLeaderboardIndex);
	static u_short Project_ConvertColumnIndex(u_int uLeaderboardIndex, u_int uColumnIndex);

	HANDLE m_xEnumeratorHandle;
	XOVERLAPPED m_xOverlapped;
	void* m_pBuffer;
	u_int m_uTotalEntries;
	bool m_bDataIsReady;
	bool m_bError;
	bool m_bFetchInProgress;
};

class Asura_Xbox360_Stats_PreGameReader
{
public:

	static void Initialise( const XUID* pxXuids, u_int uNumXuids );
	static bool IsInitialised() { return s_pxInstance != NULL; }
	static bool Update();	// returns true on completion
	static void Shutdown();
	static const XUSER_STATS_ROW* GetDataForXuid( const XUID& xXuid ); // may return NULL

private:

	// private ctor/dtor
	Asura_Xbox360_Stats_PreGameReader( const XUID* pxXuids, u_int uNumXuids );
	~Asura_Xbox360_Stats_PreGameReader();
	const XUSER_STATS_ROW* GetDataForXuid_Internal( const XUID& xXuid ) const; // may return NULL

	// Project functions - assuming all the data we need is in one leaderboard.
	// Can probably ensure this by creating a new leaderboard if necessary.
	// Alternatively this class could iterate over more than one leaderboard 
	// using an array of stats readers.
	static u_int Project_GetPreGameLeaderboardID();
	static u_int Project_GetNumColumns();
	static u_short Project_GetColumnID( u_int uColumnIndex );

	// private member data
	XUID* m_pxXuids;
	u_int m_uNumXuids;
	Asura_Xbox360_StatsReader* m_pxReader;
	XUSER_STATS_SPEC* m_pxStatsSpec;

	// static class data
	static Asura_Xbox360_Stats_PreGameReader* s_pxInstance;
};

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

#endif // #ifndef ASURA_XBOX360_STATS_H___
