//////////////////////////////////////////////////////////////////////////////
//
// Asura_Xbox360_Voice.h
//
// Copyright (c) 2008 Rebellion. All rights reserved.
//
// Description:
//
//     wrapper for XHV/XHV2 - deals with online voice comms
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef ASURA_XBOX360_VOICE_H___
#define ASURA_XBOX360_VOICE_H___ 1

#ifdef ASURA_SUPPORT_LOBBY

// library includes
#include <xbox.h>
#include <xaudio2.h>
#include <xhv2.h>

// constants
static const u_int uASURA_XBOX360_MAX_VOICE_PAYLOAD = XHV_VOICECHAT_MODE_PACKET_SIZE * XHV_MAX_VOICECHAT_PACKETS;

// classes 
class Asura_Xbox360_Voice_MsgHeader
{
public:
	Asura_Xbox360_Voice_MsgHeader(const XUID& xXuid);

	unsigned short	m_usMsgID; // must be ASURA_NETTRANSPORT_MSG_VOICE_DATA
	unsigned short	m_usDataSize;	// how much of payload is valid.
	XUID			m_xXuidSrc;
};

class Asura_Xbox360_Voice_Msg : public Asura_Xbox360_Voice_MsgHeader
{
public:
	Asura_Xbox360_Voice_Msg(const XUID& xXuid)
		:Asura_Xbox360_Voice_MsgHeader(xXuid)
	{}

	u_char m_aucPayload[uASURA_XBOX360_MAX_VOICE_PAYLOAD];
};

class Asura_Xbox360_Voice
{
public:

	static void Initialise(u_int uMaxRemoteTalkers);
	static bool Update(); // returns true if data should be sent now
	static void Shutdown();

	static void RegisterRemotePlayer(const XUID& xXUID);
	static void UnregisterRemotePlayer(const XUID& xXUID);

	static u_int GetVoiceMsgDataSize() { return s_pxVoiceDataMsg ? sizeof(Asura_Xbox360_Voice_MsgHeader) + s_pxVoiceDataMsg->m_usDataSize : 0; }
	static const void* GetVoiceMsgData() { return s_pxVoiceDataMsg; }
	static void VoiceMsgSent();

	static bool HandleMessage( const char* pcMessage, int iLength );

	static bool IsPlayerTalking(const XUID& xXUID);
	static bool IsHeadsetPresent();

private:

	// CheckVoicePrivileges() returns false if voice cannot be used. Otherwise
	// it sets s_bFriendsOnly appropriately.
	static bool CheckVoicePrivileges(u_int uPlayerIndex); 

	static IXHV2Engine* s_pxXHVEngine;
	static bool s_bFriendsOnly;
	static float s_fForceSendTimer;
	static Asura_Xbox360_Voice_Msg* s_pxVoiceDataMsg;
};

#endif // #ifdef ASURA_SUPPORT_LOBBY

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

#endif // #ifndef ASURA_XBOX360_VOICE_H___
