//////////////////////////////////////////////////////////////////////////////
//
// Asura_Direct3D_AmbientOcclusion.cpp
//
// Created by Filip Strugar 17/03/2009
//
// Copyright (c) 2009 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////


// INCLUDES

#include "Core/Asura.h"
 
#ifdef ASURA_USE_AMBIENT_OCCLUSION

// this file's header
#include "Rendering/Asura_Xbox360_AmbientOcclusion.h"

// independent
#include "Camera/Asura_Camera.h"
#include "Interface/Asura_CommandConsole.h"
#include "Rendering/Asura_AmbientOcclusion.h"
#include "Rendering/Asura_Viewport.h"
#include "Rendering/Asura_Font_System.h"
#include "Rendering/Asura_Render.h"
#include "Rendering/Asura_Viewport.h"

// Asura Xbox 360
#include "Rendering/Asura_Xbox360_Effect_System.h"
#include "Rendering/Asura_Xbox360_Graphics.h"

// Misc
#include <xtl.h>

// Asura Shader header
#include "ScreenSpaceAmbientOcclusion.h"


// The number of ambient occlusion blurred levels, not counting sampling direct from the depth buffer for the first (sharpest) level (if we're doing that)
// i.e. this is the number of textures that need to be generated by pre-processing before the actual SSAO pass.
#define NUM_AMBIENT_OCCLUSION_SHADER_SAMPLERS	2 //corresponding to declaration in AmbientOcclusion_FP.fpx

// Globals

static Asura_Xbox360_Effect*			g_pxSSAOEffect				= NULL;

// DEBUG

enum SSAO_DEV_MODE {
	SSAO_OFF,
	SSAO_ON,
	SSAO_OPAQUE,	// replace screen colour with ambient occlusion result
	NUM_SSAO_MODES
};
//
char* g_aszSSAOModeName[] = {
	"OFF",
	"ON",
	"OPAQUE",
};

static int		g_iSSAOMode = SSAO_ON;

static int		g_iSampleDistance		= 1;		// 0 = sample adjacent pixels, bigger values artificially expand the sample range
static int		g_iReadBackScale		= 2;		// factor of g_iSampleDistance to use when reading the blurred buffers
static int		g_iSampleTightness		= 0;
static float	g_fDepthThreshold		= 0.3f;		// depth-difference between pixels at which they are treated as seperate objects
static float	g_fDistanceThreshold	= 0.8f;		// a similar value to the depth difference, but applied in the final stage to fade the edge. The interaction between these two vals is unfortunate but with tweaking can yield good results
static float	g_fBrightness			= -1.0f;
static float	g_fContrast				= 2.4f;

static bool		g_fDebugSwitchValue		= false;

// Variable Handles

//static Asura_Xbox360_Effect_System::Handle	g_xRTTextureResolution				= NULL;
//static Asura_Xbox360_Effect_System::Handle	g_xRTPixelSize						= NULL;
static Asura_Xbox360_Effect_System::Handle	g_xFocalLenHandle					= NULL;
static Asura_Xbox360_Effect_System::Handle	g_xScreenResolutionHandle			= NULL;
static Asura_Xbox360_Effect_System::Handle	g_xDepthBufferHandle				= NULL;
static Asura_Xbox360_Effect_System::Handle	g_xParamsHandle						= NULL;
static Asura_Xbox360_Effect_System::Handle	g_xDepthProjectionHandle			= NULL;
static Asura_Xbox360_Effect_System::Handle	g_xViewMatrixHandle					= NULL;
static Asura_Xbox360_Effect_System::Handle	g_xCameraTanHalfFOVHandle			= NULL;
static Asura_Xbox360_Effect_System::Handle	g_xTwoOverScreenHandle				= NULL;
static Asura_Xbox360_Effect_System::Handle	g_xBrightnessContrastHandle			= NULL;
static Asura_Xbox360_Effect_System::Handle	g_xDistanceFadeHandle				= NULL;
static Asura_Xbox360_Effect_System::Handle	g_xSamplePositionsHandle			= NULL;
static Asura_Xbox360_Effect_System::Handle	g_xDepthAdjustmentHandle			= NULL;

static Asura_Xbox360_Effect_System::Handle	g_xDfrDepthMtlBufferHandle			= NULL;
static Asura_Xbox360_Effect_System::Handle	g_xDfrNormalBufferHandle			= NULL;

static Asura_Xbox360_Effect_System::Handle	g_fDebugSwitchHandle				= NULL;

//static Asura_Xbox360_Effect_System::Handle	g_xInvViewMatrix					= NULL;
//static Asura_Xbox360_Effect_System::Handle	g_xCameraTanHalfVertFOV				= NULL;
//static Asura_Xbox360_Effect_System::Handle	g_xCameraTanHalfHorizFOV			= NULL;
//static Asura_Xbox360_Effect_System::Handle	g_xTwoOverScreenWidth				= NULL;
//static Asura_Xbox360_Effect_System::Handle	g_xTwoOverScreenHeight				= NULL;

static Asura_Xbox360_Effect_System::Handle	g_axDepthDownsampleTextureHandle[NUM_AMBIENT_OCCLUSION_SHADER_SAMPLERS]	= { NULL };

static Direct3D_Texture*					g_apxDepthTextures[NUM_AMBIENT_OCCLUSION_SHADER_SAMPLERS]	= { NULL };
static Direct3D_Surface*					g_pxWorkingDepthRT						= NULL;


// Helper class used to store/restore various dx states
class Asura_Xbox360_StoreRestoreDxSettings
{
public:
	Asura_Xbox360_StoreRestoreDxSettings()
	{
		Asura_Xbox360_Graphics::GetDevice()->GetRenderTarget( 0, &m_pxOldRT );
		Asura_Xbox360_Graphics::GetDevice()->GetDepthStencilSurface( &m_pxOldDepthStencil );
		for( u_int u = 0; u < 3; ++u )
		{
			Asura_Xbox360_Graphics::GetDevice()->GetSamplerState( u, D3DSAMP_ADDRESSU, &m_auAddressU[u] );
			Asura_Xbox360_Graphics::GetDevice()->GetSamplerState( u, D3DSAMP_ADDRESSV, &m_auAddressV[u] );
		}
	}

	~Asura_Xbox360_StoreRestoreDxSettings()
	{
		Asura_Xbox360_Graphics::GetDevice()->SetRenderTarget( 0, m_pxOldRT );
		Asura_Xbox360_Graphics::GetDevice()->SetDepthStencilSurface( m_pxOldDepthStencil );
		SAFE_RELEASE( m_pxOldRT );
		SAFE_RELEASE( m_pxOldDepthStencil );
		for( u_int u = 0; u < 3; ++u )
		{
			Asura_Xbox360_Graphics::GetDevice()->SetSamplerState( u, D3DSAMP_ADDRESSU, m_auAddressU[u] );
			Asura_Xbox360_Graphics::GetDevice()->SetSamplerState( u, D3DSAMP_ADDRESSV, m_auAddressV[u] );
		}
	}

private:
	Direct3D_Surface*	m_pxOldRT;
	Direct3D_Surface*	m_pxOldDepthStencil;
	DWORD				m_auAddressU[3], m_auAddressV[3];

};


// FUNCTIONS

void Asura_AmbientOcclusion::Platform_Initialise()			{ Asura_Direct3D_AmbientOcclusion::Initialise(); }
void Asura_AmbientOcclusion::Platform_Shutdown()			{ Asura_Direct3D_AmbientOcclusion::Shutdown(); }
void Asura_AmbientOcclusion::Platform_Reset()				{ }
void Asura_AmbientOcclusion::Platform_Render()				{ Asura_Direct3D_AmbientOcclusion::Render(); }
void Asura_AmbientOcclusion::Platform_PrepareDepthData()	{ Asura_Direct3D_AmbientOcclusion::PrepareDepthData(); }

#ifdef ASURA_SUPPORT_COMMAND_CONSOLE
static bool ChangeMode ( bool bInc, bool bInterrogate, bool bHelp )
{
	if ( bHelp || bInterrogate )
	{
		Asura_CommandConsole::Printf( "%s", g_aszSSAOModeName[g_iSSAOMode] );
		return true;
	}

	g_iSSAOMode += bInc?1:-1;
	if ( g_iSSAOMode < 0 ) g_iSSAOMode = NUM_SSAO_MODES-1;
	if ( g_iSSAOMode >= NUM_SSAO_MODES ) g_iSSAOMode = 0;

	return true;
}
#endif

static void InitialiseEffectVariableHandles();

void Asura_Direct3D_AmbientOcclusion::Initialise()
{
	// Debugging stuff

	#if defined(ASURA_SUPPORT_COMMAND_CONSOLE)
		Asura_CommandConsole::AddCmd( L"Render.SSAO.Mode", ChangeMode, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	//#if defined(ASURA_PS3_ALLOW_SHADER_RELOAD)
	//	Asura_CommandConsole::AddCmd( L"Render.SSAO.ReloadAmbOccShader", ReloadAmbientOcclusionShader, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	//	Asura_CommandConsole::AddCmd( L"Render.SSAO.ReloadDepthShader", ReloadDepthDownsampleShader, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	//#endif
		Asura_CommandConsole::AddVar( L"Render.SSAO.SampleDistance", &g_iSampleDistance, 0, INT_MAX, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
		Asura_CommandConsole::AddVar( L"Render.SSAO.SampleReadScale", &g_iReadBackScale, 0, INT_MAX, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
		Asura_CommandConsole::AddVar( L"Render.SSAO.TightnessTweak", &g_iSampleTightness, 0, INT_MAX, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
		Asura_CommandConsole::AddVar( L"Render.SSAO.DepthThreshold", &g_fDepthThreshold, 0, FLT_MAX, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
		Asura_CommandConsole::AddVar( L"Render.SSAO.DistanceThreshold", &g_fDistanceThreshold, 0, FLT_MAX, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
		Asura_CommandConsole::AddVar( L"Render.SSAO.Contrast", &g_fContrast, -FLT_MAX, FLT_MAX, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
		Asura_CommandConsole::AddVar( L"Render.SSAO.Brightness", &g_fBrightness, -FLT_MAX, FLT_MAX, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
		Asura_CommandConsole::AddVar( L"Render.SSAO.DebugSwitch", &g_fDebugSwitchValue, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
		
	#endif

	g_pxSSAOEffect = Asura_Xbox360_Effect_System::CreateEffectFromMemory( g_auScreenSpaceAmbientOcclusion );

	// Create downsampled and blurred depth buffers
	for( int i = 0; i < NUM_AMBIENT_OCCLUSION_SHADER_SAMPLERS; i++ )
	{
		Asura_Xbox360_Graphics::GetDevice()->CreateTexture( Asura_Xbox360_Graphics::GetScreenWidth() / 2, Asura_Xbox360_Graphics::GetScreenHeight() / 2,
			1, 0, D3DFMT_G16R16F, D3DPOOL_DEFAULT, &g_apxDepthTextures[i], NULL );
	}

	// Create render target used for downsampling and blurring depths
	const D3DSURFACE_PARAMETERS& xSSAODepthSurfaceParams =
		Asura_Xbox360_Graphics::GetSurfaceParams( Asura_Xbox360_Graphics::SP_SSAO_DEPTH );

	Asura_Xbox360_Graphics::GetDevice()->CreateRenderTarget( Asura_Xbox360_Graphics::GetScreenWidth() / 2, Asura_Xbox360_Graphics::GetScreenHeight() / 2,
		D3DFMT_G16R16F, D3DMULTISAMPLE_NONE, 0, FALSE, &g_pxWorkingDepthRT, &xSSAODepthSurfaceParams );

	InitialiseEffectVariableHandles();
}

static void InitialiseEffectVariableHandles()
{
	Asura_Assert( g_pxSSAOEffect != NULL, "The SSAO effect is not initialised!" );
	if( g_pxSSAOEffect == NULL )
		return;

	g_xDfrDepthMtlBufferHandle			= g_pxSSAOEffect->GetParameterHandle( "g_xDfrDepthMtlBufferSampler" );
	g_xDfrNormalBufferHandle			= g_pxSSAOEffect->GetParameterHandle( "g_xDfrNormalBufferSampler" );

	g_fDebugSwitchHandle				= g_pxSSAOEffect->GetParameterHandle( "g_fDebugSwitch" );

	//g_xRTTextureResolution				= g_pxSSAOEffect->GetParameterHandle( "g_xRTTextureResolution" );
	//g_xRTPixelSize						= g_pxSSAOEffect->GetParameterHandle( "g_xRTPixelSize" );
	g_xFocalLenHandle					= g_pxSSAOEffect->GetParameterHandle( "g_xFocalLen" );
	g_xScreenResolutionHandle			= g_pxSSAOEffect->GetParameterHandle( "g_xScreenResolution" );
	g_xDepthBufferHandle				= g_pxSSAOEffect->GetParameterHandle( "g_xDepthBufferSampler" );
	g_xParamsHandle						= g_pxSSAOEffect->GetParameterHandle( "g_xParams" );
	g_xDepthProjectionHandle			= g_pxSSAOEffect->GetParameterHandle( "g_xDepthProjection" );
	g_xViewMatrixHandle					= g_pxSSAOEffect->GetParameterHandle( "g_xViewMatrix" );
	g_xCameraTanHalfFOVHandle			= g_pxSSAOEffect->GetParameterHandle( "g_xCameraTanHalfFOV" );
	g_xTwoOverScreenHandle				= g_pxSSAOEffect->GetParameterHandle( "g_xTwoOverScreen" );
	g_xBrightnessContrastHandle			= g_pxSSAOEffect->GetParameterHandle( "g_xBrightnessContrast" );
	g_xDistanceFadeHandle				= g_pxSSAOEffect->GetParameterHandle( "g_xDistanceFade" );
	g_xSamplePositionsHandle			= g_pxSSAOEffect->GetParameterHandle( "g_axSamplePositions" );
	g_xDepthAdjustmentHandle			= g_pxSSAOEffect->GetParameterHandle( "g_xDepthAdjustment" );
	// g_xInvViewMatrix					= g_pxSSAOEffect->GetParameterHandle( "g_xInvViewMatrix" );
	// g_xCameraTanHalfVertFOV			= g_pxSSAOEffect->GetParameterHandle( "g_fCameraTanHalfVertFOV" );
	// g_xCameraTanHalfHorizFOV			= g_pxSSAOEffect->GetParameterHandle( "g_fCameraTanHalfHorizFOV" );
	// g_xTwoOverScreenWidth			= g_pxSSAOEffect->GetParameterHandle( "g_fTwoOverScreenWidth" );
	// g_xTwoOverScreenHeight			= g_pxSSAOEffect->GetParameterHandle( "g_fTwoOverScreenHeight" );

	for( int i = 0; i < NUM_AMBIENT_OCCLUSION_SHADER_SAMPLERS; i++ )
	{
		Asura_Char szBuff[1024]; Asura_PrintfToBfr( szBuff, "g_axDepthDownsampleSampler%d", sizeof(szBuff), true, i );
		g_axDepthDownsampleTextureHandle[i] = g_pxSSAOEffect->GetParameterHandle( szBuff );
	}
}

void Asura_Direct3D_AmbientOcclusion::Shutdown()
{
}

static void DrawFullscreenQuad( )
{
	Direct3D_Viewport xViewport;
	xViewport.X = 0;
	xViewport.Y = 0;
	xViewport.MinZ = 0.0f;
	xViewport.MaxZ = 1.0f;
	//
	{
		IDirect3DSurface9 * pxRenderTarget = NULL;
		Asura_Xbox360_Graphics::GetDevice()->GetRenderTarget( 0, &pxRenderTarget );
		D3DSURFACE_DESC xSurfDesc;
		pxRenderTarget->GetDesc(&xSurfDesc);
		SAFE_RELEASE( pxRenderTarget );
		xViewport.Width		= xSurfDesc.Width;
		xViewport.Height	= xSurfDesc.Height;

		float xFltArr[4];
		//render target and screen size are not the same when using tiling!
		//xFltArr[0]	= (float)xSurfDesc.Width;
		//xFltArr[1]	= (float)xSurfDesc.Height;
		//g_pxSSAOEffect->SetVector( g_xRTTextureResolution, xFltArr );
		//xFltArr[0]	= 1.0f / (float)xSurfDesc.Width;
		//xFltArr[1]	= 1.0f / (float)xSurfDesc.Height;
		//g_pxSSAOEffect->SetVector( g_xRTPixelSize, xFltArr );

		xFltArr[0]  = (float)Asura_Viewport::GetWidth();
		xFltArr[1]  = (float)Asura_Viewport::GetHeight();
		xFltArr[2]  = 2.0f / (float)Asura_Viewport::GetWidth();
		xFltArr[3]  = -2.0f / (float)Asura_Viewport::GetHeight();
		g_pxSSAOEffect->SetVector( g_xScreenResolutionHandle, xFltArr );

		xFltArr[0]      = Asura_Camera::GetOneOverTanHalfFovY() *  (float)xSurfDesc.Height / (float)xSurfDesc.Width;
		xFltArr[1]      = Asura_Camera::GetOneOverTanHalfFovY();
		g_pxSSAOEffect->SetVector( g_xFocalLenHandle, xFltArr );

		g_pxSSAOEffect->SetFloat( g_fDebugSwitchHandle, (g_fDebugSwitchValue)?(1.0f):(0.0f) );
	}

	Asura_Xbox360_Graphics::GetFullScreenVB().Bind();
	Asura_Xbox360_Graphics::GetDevice()->SetVertexDeclaration( Asura_Xbox360_Graphics::GetFullScreenVD() );

	g_pxSSAOEffect->Commit();
	Asura_Xbox360_Graphics::GetDevice()->DrawPrimitive( D3DPT_RECTLIST, 0, 1 );
}

static void ProcessDepthBuffers( )
{
	SCOPEDPIXEVENT((0xff0080ff, "SSAO Depth Buffers Processing"));

	Asura_Render::CurrentStates.RequestAlphaTest( false );
	Asura_Render::CurrentStates.RequestTranslucencyMode( ASURA_TRANSLUCENCY_OFF );
	Asura_Render::CurrentStates.RequestDynamicLights( false );
	Asura_Render::CurrentStates.RequestZBufferEnabled( false );
	Asura_Render::CurrentStates.RequestZBufferWriteEnabled( false );

	Asura_Xbox360_Graphics::GetDevice()->SetRenderTarget( 0, g_pxWorkingDepthRT );

	// Smart-blur the depth buffer as a Z24S8 texture
	// The "smart" algorithm tries to represent the pixels that will cause occlusion, ignoring ones too far back to be part of the same object
	// the stencil value stores a confidence value, represent what percentage of the pixels that were sampled by the blur were actually used in creating this pixel's averaged (blurred) value.
	// when we read this back we use confidence to reduce occlusion, simulating occlusion from a smaller object (e.g. you get less occlusion from 1 far away pixel than a 32x32 box)

	g_pxSSAOEffect->Begin( 0 );

	int uNumPasses = NUM_AMBIENT_OCCLUSION_SHADER_SAMPLERS;
	for ( int i=0; i < uNumPasses; i++ )
	{
		if ( i == 0 )
		{
			// DepthDownsample
			g_pxSSAOEffect->BeginPass( 0 );


			//special case, reading the actual (MRT) depth buffer
			//g_pxSSAOEffect->SetParameterTexture( g_xDepthBufferHandle, pxRenderTarget->GetTexture( ASURA_DIRECT3D_DEFFEDRENDERTARGET_NORMAL_INDEX ) );
		}
		else
		{
			// DepthBlur
			g_pxSSAOEffect->BeginPass( 1 );

			//read texture from the previous step
			g_pxSSAOEffect->SetTexture( g_xDepthBufferHandle, g_apxDepthTextures[i-1] );
		}

		// set the threshold - perhaps this should change with each level? Not sure if it should get smaller (because 2D range is bigger), or bigger (because legitimate depth-difference caused by gentle slope is bigger)
		float fNear = Asura_Camera::GetNearClip();
		float fFar = Asura_Camera::GetFarClip();
		float fDepthProject1 = (fNear*fFar) / (fFar-fNear);	// imitate the calculation done when creating the projection matrix
		float fDepthProject2 = (fFar) / (fFar-fNear);

		float fFactor = 1;//i+1;//1<<i; // the acceptible error must increase with each level or the blurriest level will fail on steep planar surfaces, however this causes echoes
		float fThresholdFactor = fFactor * g_fDepthThreshold;

		int iSampleExp = i;
		if ( /*g_b16TapFilter && */ i > 0 ) iSampleExp = i*2-1; //0,1,3,5,... because 16-tap counts for 2 passes (first pass is only 4-tap)
		if ( i > 0 ) iSampleExp -= 1; //the first pass downsamples so halve all later passes
		float fSampleDistance = 0.5f * (float)(1<<iSampleExp);	// 0.5, 1/2, 4/2, 16/2,...
		fSampleDistance += (float)(g_iSampleDistance<<iSampleExp);

		float afParams[4] = { fSampleDistance, fThresholdFactor, (float)i, 0 };
		g_pxSSAOEffect->SetVector( g_xParamsHandle, afParams );

		float afDepthProjection[4] = { fDepthProject1, fDepthProject2, fNear, fFar };
		g_pxSSAOEffect->SetVector( g_xDepthProjectionHandle, afDepthProjection );

		DrawFullscreenQuad();

		Asura_Xbox360_Graphics::GetDevice()->Resolve( D3DRESOLVE_RENDERTARGET0, NULL, g_apxDepthTextures[i], NULL, 0, 0, NULL, 1.0f, 0, NULL );

		// Remove textures from samplers for textures that are later used as render targets
		g_pxSSAOEffect->SetTexture( g_xDepthBufferHandle, NULL );
		g_pxSSAOEffect->Commit();
	}

	g_pxSSAOEffect->End();
}

static void ComputeSamplePositions( float* pfSamplePositions )
{
	// compute the relative placement of the samples from the blurred buffers.
	// output an array of x,y,x,y,x,y with 1 sample position for each buffer.


	// the shader samples each buffer 4 times, rotating this sample point by 90 degrees for each sample
	// we pick coordinates to give this kind of arrangement:
	//
	//   +---+
	//   |	  +---+
	// +-+-+-+	  |
	// |   +-+-+-+
	// +---+   |
	//     +---+
	//
	// with the offsets each successively blurred buffer arranging it's boxes to cover the area beyond the previous one.
	// the buffers are blurred to to represent averages of 2x2, 8x8, 32x32, etc. pixels
	
	// A couple of modifications to that:
	//	- we downsample the first buffer by 1/2, so it's boxes can't be neatly arranged around our pixel. Instead the first level just uses 4 blocks arranged in a square,
	//	  which represents the smart-blurred average of 4x4 pixels, including the pixel we're shading! Fortunately there aren't any artefacts.
	//	- g_iSampleTightness slides those boxes against each other to tighten the iris in the middle, partially/totally overlapping the boxes of later levels with the earlier
	//	  levels. This is a touchy-feely tweak to help improve the visual quality (neither average perfectly represents all of it's constituent pixels, so overlaps might help)
	//	- g_iSampleDistance gives a result sparsely sampled, so rather than the first level representing 2x2 adjacent pixels it represents 4 pixels that were further apart
	//	  obviously this can cause artefacts, but within this function what we need to do is treat it as if it's been sampled from a larger area. Again this is a tweak to
	//	  trade the visual quality of larger blurs vs the artefacts, without incurring any performance hit (from doing more blur stages).

//notes on coordinates:
//
//  full-size (not downsampled) buffers:
// 	.5, 1.5, 5.5
// 	1.5, 6.5, 26.5
// 
// 	i = 0,1,2
// 	.5+[1<<(i-1)*2+[1<<(i-2)*2]]		 //[] - if it shifts right too we're good. (i.e. 1<<-1 = 0, 2<<-1*2 = 0)
// 	.5+1<<i*2+[2<<(i-1)*2+[2<<(i-2)*2]]
// 
// 	downsampled 1/2 (positions in full-screen pixels, for comparison):
// 	basepos rounded to nearest 2x2 corner, (-1,1)
// 	basepos rounded to nearest 2x2 corner, (-2,6)
// 	basepos rounded to nearest 2x2 corner, (-6,26)
// 
// 	1+[1<<(i-1)*2+[1<<(i-2)*2]]
// 	1<<i*2+[2<<(i-1)*2+[2<<(i-2)*2]]
// 
// 	iris:
// 	offset (-1,-1)*i*g_fIris. this screws it in, overlapping the inner layer
// 
// 	sample spacings:
// 	pretty sure this is just a scale applied to the offsets
// 	in the case of non-aligned ones we would add .5 afterwards

	// sample positions are in screen pixels, the shader re-scales to the pixels of the buffer.
	float fX = -1;
	float fY = 1;

	for ( int i=0; i < NUM_AMBIENT_OCCLUSION_SHADER_SAMPLERS; i++ )
	{
		pfSamplePositions[2*i+0] = fX*(1+g_iReadBackScale*g_iSampleDistance) - g_iSampleTightness*2*i; // *2 to force symmetry in 1/2 downsampled buffers (otherwise amb-occ gets some directionality)
		pfSamplePositions[2*i+1] = fY*(1+g_iReadBackScale*g_iSampleDistance) - g_iSampleTightness*2*i;
		
		fX -= 1<<(i*2);
		fY += (1<<((i+1)*2)) + (1<<(i*2));
	}
}

static void ApplyShadows( )
{
	SCOPEDPIXEVENT((0xff0080ff, "SSAO Apply Shadows"));

	//Asura_Direct3D_RenderTarget* pxRenderTarget = Asura_Direct3D::GetDeferredRenderingTarget();

	Asura_Render::CurrentStates.RequestAlphaTest( false );
	Asura_Render::CurrentStates.RequestTranslucencyMode( (g_iSSAOMode == SSAO_OPAQUE)?(ASURA_TRANSLUCENCY_OFF):(ASURA_TRANSLUCENCY_COLOUR) );
	Asura_Render::CurrentStates.RequestDynamicLights( false );
	Asura_Render::CurrentStates.RequestZBufferEnabled( false );
	Asura_Render::CurrentStates.RequestZBufferWriteEnabled( false );

	g_pxSSAOEffect->Begin( 0 );

	D3DXMATRIX xViewMatrix;
	xViewMatrix = Asura_Xbox360_Graphics::GetViewMatrix();
	g_pxSSAOEffect->SetMatrix( g_xViewMatrixHandle, xViewMatrix );

	////this is required to perform final occlusion computation in world space
	//{
	//	D3DXMATRIX xViewMatrix;
	//	D3DXMATRIX xViewMatrixInv;
	//	xViewMatrix = Asura_Xbox360_Graphics::GetViewMatrix();
	//	D3DXMatrixInverse( &xViewMatrixInv, NULL, &xViewMatrix );
	//	g_pxSSAOEffect->GetD3DEffect()->SetMatrix( g_xInvViewMatrix, &xViewMatrixInv );
	//	float z_fCameraTanHalfVertFOV	= Asura_Maths::Tangent( Asura_Camera::GetVerticalFieldOfView() * 0.5f );
	//	float z_fCameraTanHalfHorizFOV  = z_fCameraTanHalfVertFOV * Asura_Camera::GetXOverYAspectRatio();
	//	float z_fTwoOverScreenWidth		= 2.0f / Asura_Viewport::GetWidth();
	//	float z_fTwoOverScreenHeight	= 2.0f / Asura_Viewport::GetHeight();	
	//	g_pxSSAOEffect->GetD3DEffect()->SetFloat( g_xCameraTanHalfVertFOV, z_fCameraTanHalfVertFOV );
	//	g_pxSSAOEffect->GetD3DEffect()->SetFloat( g_xCameraTanHalfHorizFOV, z_fCameraTanHalfHorizFOV);
	//	g_pxSSAOEffect->GetD3DEffect()->SetFloat( g_xTwoOverScreenWidth, z_fTwoOverScreenWidth );
	//	g_pxSSAOEffect->GetD3DEffect()->SetFloat( g_xTwoOverScreenHeight, z_fTwoOverScreenHeight );
	//}

	float afParams[4];
	float fCameraTanHalfVertFOV	= Asura_Maths::Tangent( Asura_Camera::GetVerticalFieldOfView() * 0.5f );
	float fCameraTanHalfHorizFOV= fCameraTanHalfVertFOV * Asura_Camera::GetXOverYAspectRatio();
	afParams[0] = fCameraTanHalfHorizFOV;
	afParams[1] = fCameraTanHalfVertFOV;
	g_pxSSAOEffect->SetVector( g_xCameraTanHalfFOVHandle, afParams );
	
	// !!!!!! <-> !!!!!! Asura_PS3_MaterialResponse_System::SetDepthParameters( pxAmbientOcclusionFP, Asura_PS3_AmbientOcclusion_FP::FP_DepthAdjustment ); !!

	afParams[0] = 2.0f / Asura_Viewport::GetWidth();
	afParams[1] = 2.0f / Asura_Viewport::GetHeight();	
	g_pxSSAOEffect->SetVector( g_xTwoOverScreenHandle, afParams );
	
	afParams[0] = g_fContrast;
	afParams[1] = g_fBrightness;
	g_pxSSAOEffect->SetVector( g_xBrightnessContrastHandle, afParams );
	
	afParams[0] = g_fDistanceThreshold;
	g_pxSSAOEffect->SetVector( g_xDistanceFadeHandle, afParams );

	float afSamplePositions[8];
	ComputeSamplePositions(afSamplePositions);
	g_pxSSAOEffect->GetInterface()->SetVectorArrayF( g_xSamplePositionsHandle, afSamplePositions, 2 );

	//////////////////////////////////////////////////////////////////////////
	// use this in the ProcessDepthBuffers too, instead of the existing stuff, to 
	// make it more readable
	float f = Asura_Camera::GetFarClip(), n = Asura_Camera::GetNearClip();
	const float afDepthAdjustment[4] = { (f*n)/(f-n), f/(f-n), 0.0f, 0.0f };
	g_pxSSAOEffect->SetVector( g_xDepthAdjustmentHandle, afDepthAdjustment );
	//////////////////////////////////////////////////////////////////////////

	for ( int i=0; i < NUM_AMBIENT_OCCLUSION_SHADER_SAMPLERS; i++ )
	{
		g_pxSSAOEffect->SetTexture( g_axDepthDownsampleTextureHandle[i], g_apxDepthTextures[i] );
	}

	Asura_Xbox360_Graphics::GetDevice()->SetRenderTarget( 0, Asura_Xbox360_Graphics::GetFinalRT() );

	g_pxSSAOEffect->BeginPass(2);

	for (int iTiling = 0; iTiling < Asura_Xbox360_Graphics::GetNumberOfPredicatedTilePasses(); ++iTiling)
	{
		if (Asura_Xbox360_Graphics::IsTiling2())
		{
			Asura_Xbox360_Graphics::GetDevice()->SetPredication(D3DPRED_TILE(iTiling));
			Asura_Xbox360_Graphics::CorrectVPOSForTiling(iTiling);
		}
		DrawFullscreenQuad();
	}

	// Remove textures from samplers for textures that are later used as render targets
	for ( int i=0; i < NUM_AMBIENT_OCCLUSION_SHADER_SAMPLERS; i++ )
	{
		g_pxSSAOEffect->SetTexture( g_axDepthDownsampleTextureHandle[i], NULL );
	}

	g_pxSSAOEffect->Commit();
	g_pxSSAOEffect->End();

	if (Asura_Xbox360_Graphics::IsTiling2())
	{
		Asura_Xbox360_Graphics::GetDevice()->SetPredication(0);
		Asura_Xbox360_Graphics::CorrectVPOSForTiling(0);
	}
}

void Asura_Direct3D_AmbientOcclusion::PrepareDepthData()
{
	if ( g_iSSAOMode == SSAO_OFF )
	{
		return;
	}

	Asura_Assert( ! (Asura_Xbox360_Graphics::IsTiling() || Asura_Xbox360_Graphics::IsTiling2()), "Asura_Direct3D_AmbientOcclusion::PrepareDepthData must be called when predicated tiling is not enabled" );

	//SCOPEDPIXEVENT((0xff0080ff, "SSAO - Prepare depth buffers"));

	{
		Asura_Xbox360_StoreRestoreDxSettings srrt;

		//Asura_Xbox360_Graphics::GetDevice()->SetRenderTarget( 1, NULL );
		//Asura_Xbox360_Graphics::GetDevice()->SetRenderTarget( 2, NULL );
		//Asura_Xbox360_Graphics::GetDevice()->SetRenderTarget( 3, NULL );
		Asura_Xbox360_Graphics::GetDevice()->SetDepthStencilSurface( NULL );

		g_pxSSAOEffect->SetTexture( g_xDfrDepthMtlBufferHandle,	Asura_Xbox360_Graphics::GetDepthMtlTexture() );

		ProcessDepthBuffers( );
	}
}

void Asura_Direct3D_AmbientOcclusion::Render()
{
	if ( g_iSSAOMode == SSAO_OFF )
	{
		return;
	}
	//SCOPEDPIXEVENT((0xff0080ff, "SSAO - Apply"));

	{
		Asura_Xbox360_StoreRestoreDxSettings srrt;

		//Asura_Xbox360_Graphics::GetDevice()->SetRenderTarget( 1, NULL );
		//Asura_Xbox360_Graphics::GetDevice()->SetRenderTarget( 2, NULL );
		//Asura_Xbox360_Graphics::GetDevice()->SetRenderTarget( 3, NULL );
		Asura_Xbox360_Graphics::GetDevice()->SetDepthStencilSurface( NULL );

		g_pxSSAOEffect->SetTexture( g_xDfrNormalBufferHandle,	Asura_Xbox360_Graphics::GetNormalTexture() );
		g_pxSSAOEffect->SetTexture( g_xDfrDepthMtlBufferHandle,	Asura_Xbox360_Graphics::GetDepthMtlTexture() );

		ApplyShadows( );

	}
}

#endif // #if defined(ASURA_USE_AMBIENT_OCCLUSION)
