//////////////////////////////////////////////////////////////////////////////
//
// Asura_Xbox360_CharacterRenderer.h
//
// Copyright (c) 2006 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

#if defined ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#if !defined __ASURA_XBOX360_CHARACTERRENDERER_H__
#define __ASURA_XBOX360_CHARACTERRENDERER_H__

/******************************************************************************/
/* Included Files                                                             */
/******************************************************************************/

// Asura Platform-independent
#include "Core/Asura_Hash.h"

// Asura Xbox 360
#include "Rendering/Asura_Xbox360_IndexBuffer.h"
#include "Rendering/Asura_Xbox360_VertexBuffer.h"

/******************************************************************************/
/* Classes                                                                    */
/******************************************************************************/

class Asura_Animation;
class Asura_Hierarchy_Skin;
class Asura_Matrix_3x3;
class Asura_Shadow;
class Asura_Vector_3;

// DaveR 06/12/06 - This is called a 'strip' now, not a material, to match the
// PC version, and because we'll soon support multiple skin elements
struct Asura_Xbox360_CharacterRenderer_Strip
{
	Asura_Hash_ID	m_auMaterialResponseHashIDs[2];
	u_int			m_uNumIndices;
	int				m_iElementID;
	u_int			m_uBoneMap;
};

class Asura_Xbox360_CharacterRenderer_Skin
{
	friend class Asura_Chunk_ResourceFile_Xbox360_Character;
	friend class Asura_Xbox360_CharacterRenderer;

public:
	struct BoneMap
	{
		u_int  m_uSize;
		u_int* m_puEntries;

		BoneMap() : m_uSize(0), m_puEntries(0) {}
		~BoneMap() {delete[] m_puEntries;}
	};

	// Constructor / destructor
	Asura_Xbox360_CharacterRenderer_Skin();
	~Asura_Xbox360_CharacterRenderer_Skin();

	// Functions
	u_int GetNumStrips() const { return m_uNumStrips; }
	u_int GetNumMaterials() const { return m_uNumStrips; }
	u_int GetNumTriangles() const { return m_uNumTriangles; }

	const Asura_Xbox360_CharacterRenderer_Strip& GetStrip( const u_int uStripIndex ) const { return m_pxStrips[ uStripIndex ]; }

	Asura_Xbox360_VertexBuffer& GetVB() { return m_xVertexBuffer; }
	Asura_Xbox360_IndexBuffer& GetIB() { return m_xIndexBuffer; }

	// Gets the number of strips in this smooth skin
	u_int GetNumBoneMaps() const { return m_uNumBoneMaps; }

	// Gets the strip at the given index
	BoneMap& GetBoneMap(u_int uMapIndex) { Asura_Assert(uMapIndex < m_uNumBoneMaps, "Invalid Bonemap index"); return m_pxBoneMaps[uMapIndex];}
	const BoneMap& GetBoneMap(u_int uMapIndex) const { Asura_Assert(uMapIndex < m_uNumBoneMaps, "Invalid Bonemap index"); return m_pxBoneMaps[uMapIndex];}

protected:

	// Data

	Asura_Xbox360_IndexBuffer			   m_xIndexBuffer;
	Asura_Xbox360_VertexBuffer			   m_xVertexBuffer;
	Asura_Xbox360_CharacterRenderer_Strip* m_pxStrips;
	u_int								   m_uNumStrips;
	u_int								   m_uNumTriangles;
	u_int								   m_uNumBoneMaps;

	BoneMap *							   m_pxBoneMaps;
};

class Asura_Xbox360_CharacterRenderer
{
public:

	static void Initialise();
	static void Reset();
	static void Shutdown();

	static void ResetCount(void)	{ s_uNumRendered = 0; }

	static Asura_Xbox360_CharacterRenderer_Skin* CreateCharacter( const Asura_Hash_ID uHash, const u_int uNumStrips );
	static Asura_Xbox360_CharacterRenderer_Skin* GetCharacter( const Asura_Hash_ID uHash );
	static bool IsCharacterOpaque( const Asura_Hash_ID uHash );

	static Direct3D_VertexDeclaration* GetVertexDeclaration();

#if defined(ASURA_HIERARCHY_QUAD)
	static void RenderCharacter(const Asura_Animation& xAnimation,
								const Asura_Hierarchy_Skin& xSkin,
								Asura_AMatrix4x4_InParam xTransform,
								Asura_AMatrix4x4_InParam xInverseTransform,
								const Asura_AMatrix4x4* const pxBones);
	static void RenderShadow(const Asura_Animation& xAnimation,
							 const Asura_Hierarchy_Skin& xSkin,
							 const Asura_Shadow& xShadow,
							 Asura_AMatrix4x4_InParam xTransform,
							 const Asura_AMatrix4x4* const pxBones);
#else
	static void RenderCharacter( const Asura_Animation& xAnimation,
								 const Asura_Hierarchy_Skin& xSkin,
								 const Asura_Vector_3& xPos,
								 const Asura_Matrix_3x3& xOri,
								 const Asura_Vector_3* const pxBonePos,
								 const Asura_Matrix_3x3* const pxBoneOri );

	static void RenderShadow( const Asura_Animation& xAnimation,
							  const Asura_Hierarchy_Skin& xSkin,
							  const Asura_Shadow& xShadow,
							  const Asura_Vector_3& xPos,
							  const Asura_Matrix_3x3& xOri,
							  const Asura_Vector_3* const pxBonePos,
							  const Asura_Matrix_3x3* const pxBoneOri );
#endif	// defined(ASURA_HIERARCHY_QUAD)

protected:

	static u_int					s_uNumRendered;
	static Asura_Xbox360_Effect*	s_pxSkinEffect;
};

#endif // !defined __ASURA_XBOX360_CHARACTERRENDERER_H__
