//
// Asura_Xbox360_Effect_System.h
//
// Created by KJFL - 30 September 2005
//
// Copyright (C) 2005 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#if !defined __ASURA_XBOX360_EFFECT_SYSTEM_H__
#define __ASURA_XBOX360_EFFECT_SYSTEM_H__

/******************************************************************************/
/* Included Files                                                             */
/******************************************************************************/

// Misc
#include <fxl.h>

/******************************************************************************/
/* Classes                                                                    */
/******************************************************************************/

class Asura_Xbox360_Effect;

////////////////////////////////////////////////////////////////////////////////
//
// Asura_Xbox360_Effect_System
//
// Handles effect management on 360
//
////////////////////////////////////////////////////////////////////////////////

class Asura_Xbox360_Effect_System
{
public:

	typedef FXLHANDLE Handle;

	static void Initialise();
	static void Reset();
	static void Shutdown();

	static Asura_Xbox360_Effect* CreateEffectFromMemory( const void* const pvData );
	static Asura_Xbox360_Effect* CreateEffectFromFile( const char* szFilename );
	static Asura_Xbox360_Effect* CreateEffectFromCompiledFile( const char* const szFilename );

	static void Destroy( Asura_Xbox360_Effect* pxEffect );

	static FXLEffectPool* GetPool() { return s_pxEffectPool; }
	static void SetSharedParameters();
	static void SetPredicationVPosOffset(u_int uOffsetX, u_int uOffsetY);

private:

	static FXLEffectPool* s_pxEffectPool;

	static Handle s_xCameraPositionHandle;
	static Handle s_xViewMatrixHandle;
	static Handle s_xProjectionMatrixHandle;
	static Handle s_xViewProjectionMatrixHandle;

	static bool s_bHasRetrievedPoolParameters;
};

////////////////////////////////////////////////////////////////////////////////
//
// Asura_Xbox360_Effect
//
// Wraps a single loaded D3D effect
//
////////////////////////////////////////////////////////////////////////////////

class Asura_Xbox360_Effect
{
	friend class Asura_Xbox360_Effect_System;

public:

	void Begin( u_int uTechniqueIndex = 0,  u_int uPassIndex = 0  );
	void BeginPass( const u_int uPassIndex );
	void End();

	// NB: Commit() must be called before rendering anything
	void Commit();

	void SetBool( const Asura_Xbox360_Effect_System::Handle xParamHandle, const bool bParamValue );
	void SetFloat( const Asura_Xbox360_Effect_System::Handle xParamHandle, const float fParamValue );
	void SetInt( const Asura_Xbox360_Effect_System::Handle xParamHandle, const int iParamValue );
	void SetMatrix( const Asura_Xbox360_Effect_System::Handle xParamHandle, FXLMATRIX& xMatrix );
	void SetMatrix( const Asura_Xbox360_Effect_System::Handle xParamHandle, const float* const pfMatrix );
	void SetTexture( const Asura_Xbox360_Effect_System::Handle xParamHandle, IDirect3DBaseTexture9* const pxTexture );
	void SetTexture( const Asura_Xbox360_Effect_System::Handle xParamHandle, const int iTextureID );
	void SetVector( const Asura_Xbox360_Effect_System::Handle xParamHandle, const FXLVECTOR xVector );
	void SetVector( const Asura_Xbox360_Effect_System::Handle xParamHandle, const float* const pfVector );
	void SetVector( const Asura_Xbox360_Effect_System::Handle xParamHandle, const int* const piVector );

	FXLEffect* GetInterface() { return m_pxEffect; }

	Asura_Xbox360_Effect_System::Handle GetParameterHandle( const char* const szParameterName ) const;

private: 

	Asura_Xbox360_Effect() : m_pxEffect( NULL ) {}
	~Asura_Xbox360_Effect();

	FXLEffect* m_pxEffect;
};

#endif // !defined __ASURA_XBOX360_EFFECT_SYSTEM_H__
