//
// Asura_Xbox360_Graphics.h
//
// Created by KJFL - 21 June 2005
//
// Copyright (C) 2005 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __ASURA_XBOX360_GRAPHICS_H__
#define __ASURA_XBOX360_GRAPHICS_H__

/******************************************************************************/
/* Included Files                                                             */
/******************************************************************************/

// Asura Platform-independent
#include "Core/Asura_FixedArray.h"
#include "Core/Asura_String.h"
#include "Rendering/Asura_RenderStates.h"

// Asura Xbox 360
#include "Rendering/Asura_Xbox360_Effect_System.h"
#include "Rendering/Asura_Xbox360_VertexBuffer.h"

// Misc
#include <stdarg.h>
#include "Core/Asura_MemoryManagement_DebugOff.h"
#include <xtl.h>
#include "Core/Asura_MemoryManagement_DebugOn.h"

/******************************************************************************/
/* Constants                                                                  */
/******************************************************************************/

#if !defined ASURA_XBOX360_GRAPHICS_NEAR_CLIP_PLANE
	#define ASURA_XBOX360_GRAPHICS_NEAR_CLIP_PLANE 0.1f
#endif

#if !defined ASURA_XBOX360_GRAPHICS_FAR_CLIP_PLANE
	#define ASURA_XBOX360_GRAPHICS_FAR_CLIP_PLANE 2000.0f
#endif

#if !defined ASURA_XBOX360_MULTISAMPLE_TYPE
	#define ASURA_XBOX360_MULTISAMPLE_TYPE D3DMULTISAMPLE_NONE
#endif

// See notes in the sample framework. When sampling, use ExpAdjust of -15 to convert into [-1,1]
const D3DFORMAT D3DFMT_G16R16_SIGNED_INTEGER = (D3DFORMAT) MAKED3DFMT2( GPUTEXTUREFORMAT_16_16,
	GPUENDIAN_8IN16, TRUE, GPUSIGN_SIGNED, GPUSIGN_SIGNED, GPUSIGN_SIGNED, GPUSIGN_SIGNED,
	GPUNUMFORMAT_INTEGER, GPUSWIZZLE_X, GPUSWIZZLE_Y, GPUSWIZZLE_0, GPUSWIZZLE_0 );

/******************************************************************************/
/* Types                                                                      */
/******************************************************************************/

// KJFL - 29 September 2005 - typedefs of Direct3D classes on the Xbox 360, to cope
// with possible SDK changes and to simplify name conventions.
// (A lesson learned from PC development.)

typedef struct IDirect3D9					Direct3D_Object;
typedef struct IDirect3DDevice9				Direct3D_Device;

typedef D3DVERTEXELEMENT9					Direct3D_VertexElement;
typedef struct IDirect3DVertexDeclaration9	Direct3D_VertexDeclaration;
typedef struct IDirect3DVertexBuffer9		Direct3D_VertexBuffer;
typedef struct IDirect3DIndexBuffer9		Direct3D_IndexBuffer;

typedef struct IDirect3DPixelShader9		Direct3D_PixelShader;
typedef struct IDirect3DVertexShader9		Direct3D_VertexShader;

typedef struct IDirect3DConstantBuffer9		Direct3D_ConstantBuffer;

typedef struct IDirect3DTexture9			Direct3D_Texture;
typedef struct IDirect3DLineTexture9		Direct3D_LineTexture;
typedef struct IDirect3DCubeTexture9		Direct3D_CubeTexture;
typedef struct IDirect3DArrayTexture9		Direct3D_ArrayTexture;

typedef struct IDirect3DSurface9			Direct3D_Surface;
typedef struct IDirect3DBaseTexture9		Direct3D_BaseTexture;

typedef D3DVIEWPORT9						Direct3D_Viewport;

typedef struct IDirect3DQuery9				Direct3D_Query;

/******************************************************************************/
/* Classes                                                                    */
/******************************************************************************/

////////////////////////////////////////////////////////////////////////////////
//
// Asura_D3D_PIXWrapper
//
// DMR - 14/11/06 - Included DLH's PIX wrappers, hoping that we could use the
// same functions on PC and 360. Unfortunately unicode screws things up as PC
// PIX takes unicode strings and 360 PIX doesn't...
//
////////////////////////////////////////////////////////////////////////////////

#if defined ASURA_XBOX360_ENABLE_PIX

	class Asura_D3D_PIXWrapper
	{
	public:

		Asura_D3D_PIXWrapper(void)	{}
		~Asura_D3D_PIXWrapper()		{ PIXEndNamedEvent(); }

		static void Monitor( const char* pszArgs);
	};

	#define SCOPEDPIXEVENT(args) Asura_D3D_PIXWrapper xScopedPIXEvent; PIXBeginNamedEvent##args##;

#else
	#define SCOPEDPIXEVENT(args)
#endif

////////////////////////////////////////////////////////////////////////////////
//
// Asura_Xbox360_Graphics
//
// Controls all Xbox 360 rendering
//
////////////////////////////////////////////////////////////////////////////////

class Asura_Xbox360_Graphics
{
public:

	enum SurfaceParams
	{
		SP_COLOUR_TILE,
		SP_NORMAL_TILE,
		SP_ALBEDO_TILE,
		SP_DEPTHMTL_TILE,
		SP_DEPTH_TILE,
		SP_SRT_COLOUR_TILE,
		SP_SRT_DEPTH_TILE,
		SP_FINAL_RT,
		SP_FINAL_DEPTH,
		SP_SHADOW_RT,
		SP_SHADOW_DEPTH,
		SP_BLOOM,
		SP_ADV_BLOOM,
		SP_HDR,
		SP_CSM,
		SP_CSM_DEPTH,
		SP_SSAO_DEPTH,

		SP_NUM_SURFACE_PARAMS,
	};

	static void InitialiseSurfaceParams();
	static const D3DSURFACE_PARAMETERS& GetSurfaceParams( const SurfaceParams eParams )
	{
		return s_axSurfaceParams[ eParams ];
	}
	
	static void SetViewportMaxZ(float fMaxZ)	{ s_fViewportMaxZ = fMaxZ; }
	static void SetViewportMinZ(float fMinZ)	{ s_fViewportMinZ = fMinZ; }
	static void ApplyCameraSettings();

	// (AS - 25/10/2007) 
	// To use the device outside of begin/end frame you must wrap your code
	// in AcquireThreadOwnership and ReleaseThreadOwnership calls
	// if AcquireThreadOwnership returns false then your thread already
	// had ownership and you should NOT call ReleaseThreadOwnership
	static bool AcquireThreadOwnership();
	static void ReleaseThreadOwnership();
	static bool HasThreadOwnership();
	static void ExclusiveThreadOwnership();
	static void ReleaseExclusiveThreadOwnership();
	
	static void Initialise();
	static void Reset();
	static void Shutdown();

	static void BeginFrame();
    static void EndFrame();
	static void PresentFrame(bool bNeedToSync=true);

	static void BeginTiling();
	static void EndTiling();
	static void BeginTiling2();
	static void EndTiling2();
	static bool IsTiling() { return s_bTiling; }
	static bool IsTiling2() { return s_bTiling2; }

	static void CopySceneToEDRAM(bool s_bCopyColourBuffer = true);

	static void Resolve( Direct3D_Texture* const pxTexture );

	static void ClearZBuffer();

	static Direct3D_Device* GetDevice() { return s_pxD3DDevice; }

	static int GetScreenWidth()	{ return s_iScreenWidth; }
	static int GetScreenHeight() { return s_iScreenHeight; }

	static const D3DXMATRIX& GetViewMatrix() { return s_xViewMatrix; }
	static const D3DXMATRIX& GetProjectionMatrix() { return s_xProjectionMatrix; }
	static const D3DXMATRIX& GetViewProjectionMatrix() { return s_xViewProjectionMatrix; }

#if defined ASURA_XBOX360_ALLOW_STATS
	static bool AreStatsEnabled() { return s_bShowStats; }
	static void EnableStats( const bool bVal ) { s_bShowStats = bVal; }
#endif

	static Direct3D_Texture* GetSceneResolveTexture() { return s_pxSceneResolveTexture; }
	static Direct3D_Texture* GetPostToneMapTexture()  { return &s_xPostToneMapResolveTexture; }
	static Direct3D_Texture* GetPostToneMapLinearTexture()  { return &s_xPostToneMapLinearResolveTexture; }
	static Direct3D_Texture* GetNormalTexture()       { return s_pxNormalTexture; }
	static Direct3D_Texture* GetDepthMtlTexture()     { return s_pxDepthMtlTexture; }
	static Direct3D_Texture* GetAlbedoTexture()       { return s_pxAlbedoTexture; }
	static Direct3D_Texture* GetFrontBuffer()         { return s_pxFrontBuffer; }

	static Direct3D_Surface* GetFinalRT() { 
		return IsTiling2() ? s_pxSRTColourTile : s_pxFinalRT; 
	}
	static Direct3D_Surface* GetFinalDepth() { 
		return IsTiling2() ? s_pxSRTDepthTile : s_pxFinalDepth; 
	}

	static Direct3D_Surface* GetPostToneMapRT() { return s_pxPostToneMapRT; }
	static Direct3D_Surface* GetPostToneMapLinearRT() { return s_pxPostToneMapLinearRT; }

	static Direct3D_VertexDeclaration* GetFullScreenVD() { return s_pxFullScreenVD; }
	static Asura_Xbox360_VertexBuffer& GetFullScreenVB() { return s_xFullScreenVB; }

	// For setting translucency modes for multiple render targets. Ideally this would
	// be moved into Asura_Renderstates, if PC and PS3 have similar functionality?
	static void SetTranslucencyModeMRT(int iRenderTarget,  ASURA_TRANSLUCENCY_MODE eMode);

	// Depth-bias functions...
	static void ResetDepthBias(void);
	static void SetDecalDepthBias(void);
	static void SetSkyboxDepthBias(void);

	static void GetDefaultParams(D3DPRESENT_PARAMETERS* pxParams);
	
	static bool SuspendDevice ();
	static bool ResumeDevice  ();

	static bool IsDeviceSuspended ()	{ return s_bDeviceSuspended; }
	static bool									s_bIsCurrentlyRenderingInSkybox;

	static int GetNumberOfPredicatedTilePasses()
	{
		return IsTiling() ? uNUM_TILES : IsTiling2() ? 2 : 1;
	}
	static void  CorrectVPOSForTiling( const int iTile );

	static float GetGammaSetting() { return s_fGamma; }
	static void SetGammaSetting( const float fGamma );

private:

#if defined ASURA_SUPPORT_COMMAND_CONSOLE
	static bool									UseNullHardware( bool bHelp );
#endif

	static float								s_fViewportMaxZ;
	static float								s_fViewportMinZ;

	static int									s_iScreenWidth;
	static int									s_iScreenHeight;

	static Direct3D_Object*						s_pxD3DObject;
	static Direct3D_Device*						s_pxD3DDevice;

	static D3DXMATRIX							s_xViewMatrix;
	static D3DXMATRIX							s_xProjectionMatrix;
	static D3DXMATRIX							s_xViewProjectionMatrix;

#if defined ASURA_XBOX360_ALLOW_STATS
	static bool									s_bShowStats;
#endif

	static bool									s_bTiling;
	static bool									s_bTiling2;

	static Direct3D_Surface*					s_pxColourTile;
	static Direct3D_Surface*					s_pxNormalTile;
	static Direct3D_Surface*					s_pxDepthMtlTile;
	static Direct3D_Surface*					s_pxAlbedoTile;
	
	static Direct3D_Surface*					s_pxDepthTile;

	static Direct3D_Surface*					s_pxSRTColourTile;
	static Direct3D_Surface*					s_pxSRTDepthTile;


	static Direct3D_Surface*					s_pxFinalRT;
	static Direct3D_Surface*					s_pxFinalDepth;
	static Direct3D_Surface*					s_pxHiZRestore;
	static Direct3D_Surface*					s_pxHiZRestoreSRT;
	static Direct3D_Surface*					s_pxPostToneMapRT;
	static Direct3D_Surface*					s_pxPostToneMapLinearRT;

	// Don't need to ping-pong between two front buffers since we're doing post-processing
	static Direct3D_Texture*					s_pxSceneResolveTexture;
	static Direct3D_Texture*					s_pxFrontBuffer;

	static Direct3D_Texture*					s_pxNormalTexture;
	static Direct3D_Texture*					s_pxDepthMtlTexture;
	static Direct3D_Texture*					s_pxAlbedoTexture;
	static Direct3D_Texture*					s_pxDepthTexture;

	static Direct3D_Texture						s_xPostToneMapResolveTexture;
	static Direct3D_Texture						s_xPostToneMapLinearResolveTexture;

	static Asura_Xbox360_Effect*				s_pxEDRAMCopyEffect;

	static Direct3D_VertexDeclaration*			s_pxFullScreenVD;
	static Asura_Xbox360_VertexBuffer			s_xFullScreenVB;

	static Direct3D_VertexDeclaration*			s_pxCopyToEDRAMVD;

	static Asura_Xbox360_Effect*				s_pxStencilRestoreEffect;
	static Asura_Xbox360_Effect_System::Handle	s_xStencilRestoreTextureHandle;
	static Direct3D_Texture						s_xStencilRestoreTexture;
	static Direct3D_Surface*					s_pxStencilRestoreRT;
	static Direct3D_Surface*					s_pxStencilRestoreSRT_RT;
	static Direct3D_VertexDeclaration*			s_pxStencilRestoreVD;
	static Asura_Xbox360_VertexBuffer			s_xStencilRestoreVB;
	static u_int								s_uNumStencilRestoreStrips;

	static const u_int							uNUM_TILES =
		ASURA_XBOX360_MULTISAMPLE_TYPE == D3DMULTISAMPLE_2_SAMPLES ? 4 :
		ASURA_XBOX360_MULTISAMPLE_TYPE == D3DMULTISAMPLE_4_SAMPLES ? 8 : 2;

	static Asura_FixedArray< D3DRECT, uNUM_TILES >	s_axTilingRects;
	static Asura_FixedArray< D3DRECT, 2 >			s_axTilingRectsSRT;

	static Asura_FixedArray< D3DSURFACE_PARAMETERS, SP_NUM_SURFACE_PARAMS > s_axSurfaceParams;

	static void*								s_pExtraMem;
	static bool									s_bDeviceSuspended;

	static float								s_fGamma;
};

class Asura_Xbox360_DeviceOwnershipObject
{
	bool m_bDidntHaveOwnership;
public:
	Asura_Xbox360_DeviceOwnershipObject() : m_bDidntHaveOwnership(Asura_Xbox360_Graphics::AcquireThreadOwnership())
	{
	}
	~Asura_Xbox360_DeviceOwnershipObject()
	{
		if (m_bDidntHaveOwnership)
			Asura_Xbox360_Graphics::ReleaseThreadOwnership();
	}

};

#define SCOPED_DEVICE_OWNERSHIP()	Asura_Xbox360_DeviceOwnershipObject xScopedDeviceOwnershipObject;

////////////////////////////////////////////////////////////////////////////////
//
// Stencil Masks
//
// Keep track of who is using which bits of the stencil buffer to ensure the play
// nicely with each other
//
////////////////////////////////////////////////////////////////////////////////

enum {
	ASURA_STENCIL_BIT_PHOTON = 0,
	ASURA_STENCIL_BIT_FIRST_PROJECT_SPECIFIC = 1,
};
#define ASURA_STENCIL_MASK_PHOTON				(1<<ASURA_STENCIL_BIT_PHOTON) 
#define ASURA_STENCIL_MASK_PROJECT_SPECIFIC					(0xFF -  ((1<<ASURA_STENCIL_BIT_FIRST_PROJECT_SPECIFIC) - 1))

#endif//__ASURA_XBOX360_GRAPHICS_H__
