//
// Asura_Xbox360_ObjectHierarchyRenderer.cpp
//
// Created by KJFL - 09 November 2005
//
// Copyright (C) 2005 Rebellion, All Rights Reserved.
//

/******************************************************************************/
/* Included Files                                                             */
/******************************************************************************/

#include "Core/Asura.h"

#include "Rendering/Asura_Xbox360_ObjectHierarchyRenderer.h"

// Asura Platform-independent
#include "Animation/Asura_Animation.h"
#include "Animation/Asura_Hierarchy.h"
#include "Animation/Asura_TextureAnimation.h"
#include "Camera/Asura_Camera.h"
#include "Core/Asura_HashTree.h"
#include "Core/Asura_Material_System.h"
#include "Core/Asura_MaterialResponse_System.h"
#include "Core/Memory/Asura_Mem.h"
#include "Light/Asura_GI_Solution.h"
#include "Rendering/Asura_DeferredShading.h"
#include "Rendering/Asura_Render.h"
#include "Rendering/Asura_SortedRender.h"
#include "Shadows/Asura_Shadow_System.h"

// Asura Xbox 360
#include "Debugging/Asura_Platform_GPUProfile.h"
#include "Rendering/Asura_Xbox360_Effect_System.h"
#include "Rendering/Asura_Xbox360_Graphics.h"
#include "Rendering/Asura_Xbox360_MaterialResponse_System.h"
#include "Rendering/Asura_Xbox360_TextureManagement.h"

// Shader Include Files
#include "Object_MaterialResponse.h"
#include "Object_Shadow.h"


// Misc
#include <fxl.h>
#include <xtl.h>

/******************************************************************************/
/* Constants                                                                  */
/******************************************************************************/

// Remember to update vertex size in Asura_Chunk_ResourceFile_Xbox360_ObjectHierarchy.cpp
// if this changes

const Direct3D_VertexElement axVERTEX_DECLARATION[] =
{
	{ 0,  0, D3DDECLTYPE_FLOAT3,    D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_POSITION, 0 },
	{ 0, 12, D3DDECLTYPE_FLOAT16_2, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_TEXCOORD, 0 },
	{ 0, 16, D3DDECLTYPE_FLOAT16_2, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_TEXCOORD, 1 },
	{ 0, 20, D3DDECLTYPE_FLOAT1,	D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_TEXCOORD, 2 },
	{ 0, 24, D3DDECLTYPE_DEC3N,     D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_NORMAL,   0 },
	{ 0, 28, D3DDECLTYPE_DEC3N,     D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_TANGENT,  0 },
	{ 0, 32, D3DDECLTYPE_DEC3N,     D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_BINORMAL, 0 },
	D3DDECL_END()
};

/******************************************************************************/
/* Global Variables                                                           */
/******************************************************************************/

// Standard object effect

static Asura_Xbox360_Effect* g_pxObjectEffect = NULL;

static Asura_Xbox360_Effect_System::Handle g_xWorldMatrixHandle = 0;

static Asura_Xbox360_Effect_System::Handle g_xTexCoordOffsetAndScaleHandle = 0;

static Asura_Xbox360_MaterialResponse_Handles g_xMaterialResponseHandles;

static Asura_Xbox360_Effect_System::Handle g_xWorldViewProjHandle = 0;

// Object shadow effect

static Asura_Xbox360_Effect* g_pxObjectShadowEffect = NULL;

static Asura_Xbox360_Effect_System::Handle g_xShadowWorldViewProjHandle = 0;
static Asura_Xbox360_Effect_System::Handle g_xShadowFrustumNearPlaneHandle = 0;
static Asura_Xbox360_Effect_System::Handle g_xOneOverShadowFrustumDepthHandle = 0;

// Vertex declaration

static Direct3D_VertexDeclaration* g_pxHierarchyVertexDeclaration = NULL;

/******************************************************************************/
/* Asura_Xbox360_ObjectHierarchyRenderer::Hierarchy Class Functions           */
/******************************************************************************/

Asura_Xbox360_ObjectHierarchyRenderer::Hierarchy::Hierarchy()
	: m_uNumberOfStrips(0)
	, m_pxStrips( NULL )
{
}

Asura_Xbox360_ObjectHierarchyRenderer::Hierarchy::~Hierarchy()
{
	m_xVertexBuffer.Destroy();
	m_xIndexBuffer.Destroy();

	delete[] m_pxStrips;
}

/******************************************************************************/
/* Asura_Xbox360_ObjectHierarchyRenderer Static Class Data                    */
/******************************************************************************/

u_int	Asura_Xbox360_ObjectHierarchyRenderer::s_uNumRendered	= 0;
Asura_TemplateHashTree< Asura_Xbox360_ObjectHierarchyRenderer::Hierarchy > Asura_Xbox360_ObjectHierarchyRenderer::s_xHierarchyTree;
Asura_Xbox360_ObjectHierarchyRenderer::Hierarchy* Asura_Xbox360_ObjectHierarchyRenderer::s_pxCurrentHierarchy = NULL;

/******************************************************************************/
/* Asura_Xbox360_ObjectHierarchyRenderer Static Class Functions               */
/******************************************************************************/

////////////////////////////////////////////////////////////////////////////////
//
// Asura_Xbox360_ObjectHierarchyRenderer::Initialise()
//
// Initialises the Xbox 360 object hierarchy renderer
//
// Parameters:		None
// Return value:	None
//
////////////////////////////////////////////////////////////////////////////////

void Asura_Xbox360_ObjectHierarchyRenderer::Initialise()
{
	// Standard object effect

	g_pxObjectEffect = Asura_Xbox360_Effect_System::CreateEffectFromMemory(g_auObject_MaterialResponse);	

	g_xWorldViewProjHandle = g_pxObjectEffect->GetParameterHandle( "g_xWorldViewProjectionMatrix" );

	g_xMaterialResponseHandles.InitialiseFromEffect( *g_pxObjectEffect );

	g_xWorldMatrixHandle = g_pxObjectEffect->GetParameterHandle( "g_xWorldMatrix" );

	// Object shadow effect

	g_pxObjectShadowEffect = Asura_Xbox360_Effect_System::CreateEffectFromMemory(g_auObject_Shadow);

	g_xShadowWorldViewProjHandle = g_pxObjectShadowEffect->GetParameterHandle( "g_xWorldViewProj" );
	g_xShadowFrustumNearPlaneHandle = g_pxObjectShadowEffect->GetParameterHandle( "g_fShadowFrustumNearPlane" );
	g_xOneOverShadowFrustumDepthHandle = g_pxObjectShadowEffect->GetParameterHandle( "g_fOneOverShadowFrustumDepth" );

	// Vertex declaration

	Asura_Xbox360_Graphics::GetDevice()->CreateVertexDeclaration( axVERTEX_DECLARATION, &g_pxHierarchyVertexDeclaration );
	Asura_Assert( g_pxHierarchyVertexDeclaration, "Failed to create hierarchy vertex declaration." );
}

////////////////////////////////////////////////////////////////////////////////
//
// Asura_Xbox360_ObjectHierarchyRenderer::Reset()
//
// Resets the Xbox 360 object hierarchy renderer
//
// Parameters:		None
// Return value:	None
//
////////////////////////////////////////////////////////////////////////////////

void Asura_Xbox360_ObjectHierarchyRenderer::Reset()
{
	s_xHierarchyTree.Reset();
}

////////////////////////////////////////////////////////////////////////////////
//
// Asura_Xbox360_ObjectHierarchyRenderer::Shutdown()
//
// Shuts the Xbox 360 object hierarchy renderer down
//
// Parameters:		None
// Return value:	None
//
////////////////////////////////////////////////////////////////////////////////

void Asura_Xbox360_ObjectHierarchyRenderer::Shutdown()
{
	Reset();

	Asura_Xbox360_Effect_System::Destroy( g_pxObjectEffect );
	g_pxObjectEffect = NULL;

	Asura_Xbox360_Effect_System::Destroy( g_pxObjectShadowEffect );
	g_pxObjectShadowEffect = NULL;

	g_pxHierarchyVertexDeclaration->Release();
	g_pxHierarchyVertexDeclaration = NULL;
}

////////////////////////////////////////////////////////////////////////////////
//
// Asura_Xbox360_ObjectHierarchyRenderer::CreateHierarchy()
//
// Creates a new Xbox 360 object hierarchy
//
// Parameters:		uHashID - Hash ID of hierarchy's name
// Return value:	Pointer to the new object, or NULL if unable to allocate
//
////////////////////////////////////////////////////////////////////////////////

Asura_Xbox360_ObjectHierarchyRenderer::Hierarchy* Asura_Xbox360_ObjectHierarchyRenderer::CreateHierarchy( const Asura_Hash_ID uHashID )
{
#if defined(ASURA_DONT_ASSERT_ON_DUPLICATE_RESOURCE_DATA)
	const Hierarchy* const	pxExistingHierarchy	= s_xHierarchyTree.GetDataByHash(uHashID);

	if(pxExistingHierarchy == NULL)
	{
		return(s_xHierarchyTree.AddNode(uHashID));
	}
	else
	{
		return(NULL);
	}
#else
	#if defined ASURA_ASSERT
		const Hierarchy* const pxExistingHierarchy = s_xHierarchyTree.GetDataByHash( uHashID );
		Asura_Assertf( !pxExistingHierarchy, ("A hierarchy with this hash (0x%X) has already been created.", uHashID) );
	#endif

	Hierarchy* const pxNewHierarchy = s_xHierarchyTree.AddNode( uHashID );
	Asura_Assert( pxNewHierarchy, "Unable to create new hierarchy." );

	return pxNewHierarchy;
#endif	// defined(ASURA_DONT_ASSERT_ON_DUPLICATE_RESOURCE_DATA)
}

////////////////////////////////////////////////////////////////////////////////
//
// Asura_Xbox360_ObjectHierarchyRenderer::RenderHierarchy()
//
// Renders a complete object hierarchy on Xbox 360
//
// Parameters:		xAnim - Reference to animation instance to render
//					xSkin - Reference to hierarchy skin
//
// Return value:	None
//
////////////////////////////////////////////////////////////////////////////////

void Asura_Xbox360_ObjectHierarchyRenderer::RenderHierarchy( const Asura_Animation& xAnim, const Asura_Hierarchy_Skin& xSkin )
{
	for (int iZPrePassMode = !Asura_SortedRender::IsTransparentPass(); iZPrePassMode < 2; ++iZPrePassMode)
	{
		BeginRender( xAnim, xSkin );

		const Hierarchy* const pxHierarchy = s_xHierarchyTree.GetDataByHash( xSkin.m_uNameHashID );
		Asura_Assertf( pxHierarchy, ("Failed to get hierarchy data for %s.", xSkin.Name.GetString()) );

		for( u_int u = 0; u < pxHierarchy->m_uNumberOfStrips; ++u )
		{
			const u_int uBoneIndex = pxHierarchy->m_pxStrips[u].m_uBoneIndex;

			const Asura_Vector_3& xBonePos = Asura_Hierarchy_System::GetWorldspaceBonePos( uBoneIndex );
			const Asura_Matrix_3x3& xBoneOri = Asura_Hierarchy_System::GetWorldspaceBoneOri( uBoneIndex );

			RenderStrip( u, xBonePos, xBoneOri, xAnim, iZPrePassMode == 0 );
		}

		EndRender();
	}
}

void Asura_Xbox360_ObjectHierarchyRenderer::BeginRender( const Asura_Animation& xAnim, const Asura_Hierarchy_Skin& xSkin )
{
	s_pxCurrentHierarchy = GetHierarchyTree().GetDataByHash( xSkin.m_uNameHashID );
	if( !s_pxCurrentHierarchy )
	{
		Asura_Assertf( false, ("Unable to find rendering data for hierarchy %s.", xSkin.Name.GetString() ));
		return;
	}

	ASURA_GPU_PROFILE_START(ASURA_GPUPROFILE_HIERARCHIES);
	PIXBeginNamedEvent( 0xFF837B4A, "Hierarchy %u - '%s'", ++s_uNumRendered, xSkin.Name.GetString() );

	{
		SCOPEDPIXEVENT(( 0xFF837B4A, "Begin hierarchy '%s' (material response system)", xSkin.Name.GetString() ));

		const u_int uTechnique = Asura_SortedRender::IsTransparentPass() ? 1 : 0;
		g_pxObjectEffect->Begin( uTechnique );

		// Set up streams
		{
			Asura_Xbox360_Graphics::GetDevice()->SetVertexDeclaration( g_pxHierarchyVertexDeclaration );
			s_pxCurrentHierarchy->m_xVertexBuffer.Bind();
			s_pxCurrentHierarchy->m_xIndexBuffer.Bind();
		}

		// Set render states
		{
			Asura_Render::CurrentStates.RequestAlphaTest( false );
			Asura_Render::CurrentStates.RequestCullMode( ASURA_CULLMODE_CW );
			Asura_Render::CurrentStates.RequestDitheringMode( true );
			Asura_Render::CurrentStates.RequestDynamicLights( false );
			Asura_Render::CurrentStates.RequestZBufferCompareMode( ASURA_ZBUFFERCMP_LESSEQUAL );
			Asura_Render::CurrentStates.RequestZBufferEnabled( true );

			// Leave wireframe as it was set elsewhere
		}
	}
}

void Asura_Xbox360_ObjectHierarchyRenderer::RenderStrip( const u_int uSectionIndex, const Asura_Vector_3& xPosition, const Asura_Matrix_3x3& xOrientation, const Asura_Animation& xAnim, bool bZPrePass )
{
	SCOPEDPIXEVENT(( 0xFF837B4A, "Hierarchy '%s' section %u (material response system)", xAnim.GetSkin()->Name.GetString(), uSectionIndex ));

	if( s_pxCurrentHierarchy->m_uNumberOfStrips == 0 || !s_pxCurrentHierarchy->m_pxStrips ) return;

	{
		D3DXMATRIX xWorldViewProj;

		// Rotation
		Asura_Mem::Set( &xWorldViewProj, 0, sizeof( xWorldViewProj ) );
		for( u_int u = 0; u < 3; ++u )
		{
			for( u_int v = 0; v < 3; ++v )
			{
				xWorldViewProj.m[u][v] = xOrientation.Element[v][u];
			}
		}

		xWorldViewProj._44 = 1.0f;

		// Translation
		D3DXMATRIX xTemp;
		D3DXMatrixTranslation( &xTemp, xPosition.x, xPosition.y, xPosition.z );
		xWorldViewProj *= xTemp;

		g_pxObjectEffect->SetMatrix( g_xWorldMatrixHandle, xWorldViewProj );

		// View-projection
		xWorldViewProj *= Asura_Xbox360_Graphics::GetViewProjectionMatrix();

		g_pxObjectEffect->SetMatrix( g_xWorldViewProjHandle, xWorldViewProj );
	}

	const bool bTransparentPass = Asura_SortedRender::IsTransparentPass();


	if (!bTransparentPass)
	{
		Asura_Xbox360_MaterialResponse_System::SetFogParameters( false, NULL, g_xMaterialResponseHandles, *g_pxObjectEffect  );	
	} 
	else 
	{
		Asura_Render::CurrentStates.RequestZBufferWriteEnabled( false );
	}

	const Strip& xStrip = s_pxCurrentHierarchy->m_pxStrips[uSectionIndex];
	if( xStrip.m_uNumberOfTriangles != 0 )
	{
		const Asura_MaterialResponse* const pxMaterial1
			= Asura_MaterialResponse_System::GetMaterialByHashID( xStrip.m_auMaterialResponseHashIDs[0] );

		if( pxMaterial1 )
		{
			if( pxMaterial1->GetDontRender() || bZPrePass && !pxMaterial1->GetZPrePass() ) return;

			const bool bTransparentStrip = pxMaterial1->IsTransparent();
			if( bTransparentStrip != bTransparentPass ) return;
		}

		SCOPEDPIXEVENT(( 0xFF837B4A, "Strip %u", uSectionIndex ));

		// Set material data
		{
			const Asura_MaterialResponse* const pxMaterial2 = Asura_MaterialResponse_System::GetMaterialByHashID( xStrip.m_auMaterialResponseHashIDs[1] );

			Asura_Xbox360_MaterialResponse_System::SetEffectParameters( pxMaterial1, pxMaterial2, &xAnim, g_xMaterialResponseHandles, *g_pxObjectEffect );
			Asura_Xbox360_MaterialResponse_System::SetBlendStates( pxMaterial1 );
			Asura_Xbox360_MaterialResponse_System::SetSHParameters( pxMaterial1, g_xMaterialResponseHandles, *g_pxObjectEffect );

			if (bTransparentPass)
			{
				Asura_Xbox360_MaterialResponse_System::SetFogParameters( true, pxMaterial1, g_xMaterialResponseHandles, *g_pxObjectEffect  );	
			}

			if (Asura_MaterialResponse_System::GetMaterialProjectContextCallback())
			{
				Asura_MaterialResponse_System::GetMaterialProjectContextCallback() (pxMaterial1->GetProjectContext(),
					pxMaterial2 ? pxMaterial2->GetProjectContext() : ASURA_MATERIAL_RESPONSE_PROJECT_CONTEXT_INVALID);
			}
		}

		g_pxObjectEffect->Commit();

		if (bZPrePass)
		{
			Asura_Render::CurrentStates.RequestZBufferWriteEnabled( true );
			Asura_Xbox360_Graphics::GetDevice()->SetRenderState(D3DRS_COLORWRITEENABLE, 0);
			Asura_Xbox360_Graphics::GetDevice()->SetPixelShader( NULL);
		}


		Asura_Xbox360_Graphics::GetDevice()->DrawIndexedVertices( D3DPT_TRIANGLESTRIP, 0, xStrip.m_uStartIndex, xStrip.m_uNumberOfIndices );

		if (bZPrePass)
		{
			Asura_Xbox360_MaterialResponse_System::SetBlendStates( pxMaterial1 );
			Asura_Xbox360_Graphics::GetDevice()->SetRenderState(D3DRS_COLORWRITEENABLE, D3DCOLORWRITEENABLE_RED | D3DCOLORWRITEENABLE_GREEN | D3DCOLORWRITEENABLE_BLUE | D3DCOLORWRITEENABLE_ALPHA);
		}

		Asura_Render::AddToTrianglesRendered( xStrip.m_uNumberOfTriangles );
	}

	if (bTransparentPass)
	{
		Asura_Render::CurrentStates.RequestZBufferWriteEnabled( true );
	}
	if (bZPrePass)
	{
		Asura_Xbox360_Graphics::GetDevice()->SetRenderState(D3DRS_COLORWRITEENABLE, D3DCOLORWRITEENABLE_RED | D3DCOLORWRITEENABLE_GREEN | D3DCOLORWRITEENABLE_BLUE | D3DCOLORWRITEENABLE_ALPHA);
	}
}

void Asura_Xbox360_ObjectHierarchyRenderer::EndRender()
{

	g_pxObjectEffect->End();

	{
		SCOPEDPIXEVENT(( 0xFF837B4A, "End hierarchy (material response system)" ));

		Asura_Xbox360_Graphics::ResetDepthBias();

		Asura_Xbox360_Graphics::GetDevice()->SetRenderState_Inline( D3DRS_COLORWRITEENABLE1, D3DCOLORWRITEENABLE_ALL );				
		Asura_Xbox360_Graphics::GetDevice()->SetRenderState_Inline( D3DRS_COLORWRITEENABLE2, D3DCOLORWRITEENABLE_ALL );				
		Asura_Xbox360_Graphics::SetTranslucencyModeMRT( 2, ASURA_TRANSLUCENCY_OFF );
	}

	PIXEndNamedEvent(); // Hierarchy
	ASURA_GPU_PROFILE_END(ASURA_GPUPROFILE_HIERARCHIES);
}

void Asura_Xbox360_ObjectHierarchyRenderer::RenderShadow (const Asura_Animation& xAnim,
														  const Asura_Hierarchy_Skin& xSkin,
														  const Asura_Shadow& xShadow)
{
	BeginShadowRender( xSkin, xShadow );

	const Hierarchy* const pxHierarchy = s_xHierarchyTree.GetDataByHash( xSkin.m_uNameHashID );
	Asura_Assertf( pxHierarchy, ("Failed to get hierarchy data for %s.", xSkin.Name.GetString()) );

	for( u_int u = 0; u < pxHierarchy->m_uNumberOfStrips; ++u )
	{
		const u_int uBoneIndex = pxHierarchy->m_pxStrips[u].m_uBoneIndex;

		const Asura_Vector_3& xBonePos = Asura_Hierarchy_System::GetWorldspaceBonePos( uBoneIndex );
		const Asura_Matrix_3x3& xBoneOri = Asura_Hierarchy_System::GetWorldspaceBoneOri( uBoneIndex );

		RenderShadowStrip( u, xBonePos, xBoneOri, xAnim, xShadow );
	}

	EndShadowRender();
}

void Asura_Xbox360_ObjectHierarchyRenderer::BeginShadowRender( const Asura_Hierarchy_Skin& xSkin, const Asura_Shadow& xShadow )
{
	PIXBeginNamedEvent( D3DCOLOR_XRGB( 0xFF, 0xFF, 0 ), "Hierarchy Shadow %s", xSkin.Name.GetString() );

	s_pxCurrentHierarchy = s_xHierarchyTree.GetDataByHash( xSkin.m_uNameHashID );
	Asura_Assertf( s_pxCurrentHierarchy, ("Failed to get hierarchy data for %s.", xSkin.Name.GetString()) );

	// Bind buffers and begin the effect

	s_pxCurrentHierarchy->m_xIndexBuffer.Bind();
	s_pxCurrentHierarchy->m_xVertexBuffer.Bind();

	Asura_Xbox360_Graphics::GetDevice()->SetVertexDeclaration( g_pxHierarchyVertexDeclaration );

	g_pxObjectShadowEffect->Begin();
}

void Asura_Xbox360_ObjectHierarchyRenderer::RenderShadowStrip( const u_int uStripIndex, const Asura_Vector_3& xPosition, const Asura_Matrix_3x3& xOrientation, const Asura_Animation& xAnim, const Asura_Shadow& xShadow )
{
	Asura_Assert( s_pxCurrentHierarchy, "Current hierarchy isn't valid." );
	Asura_Assertf( uStripIndex < s_pxCurrentHierarchy->m_uNumberOfStrips, ("Invalid strip index (%u) passed to Asura_Xbox360_ObjectHierarchyRenderer::RenderShadowStrip().", uStripIndex) );

	const Strip& xStrip	= s_pxCurrentHierarchy->m_pxStrips[ uStripIndex ];
	if( xStrip.m_uNumberOfIndices == 0 ) return;

	const Asura_Hash_ID uHash = xStrip.m_auMaterialResponseHashIDs[0];
	const Asura_MaterialResponse* const pxMaterial =
		Asura_MaterialResponse_System::GetMaterialByHashID( uHash );

	if( pxMaterial && (pxMaterial->GetRenderType() == Asura_MaterialResponse::RENDER_TYPE_RENDER_BUT_DONT_CAST_SHADOW) ) return;

	PIXBeginNamedEvent( D3DCOLOR_XRGB( 0xC0, 0xC0, 0 ), "Strip %u", uStripIndex );

	// Bone orientation

	D3DXMATRIX xWorldViewProj;
	memset( &xWorldViewProj, 0, sizeof( D3DXMATRIX ) );

	for( u_int u = 0; u < 3; ++u )
	{
		for( u_int v = 0; v < 3; ++v )
		{
			xWorldViewProj.m[u][v] = xOrientation.Element[v][u];
		}
	}

	xWorldViewProj._44 = 1.0f;

	// Bone position

	D3DXMATRIX xTemp;
	D3DXMatrixTranslation( &xTemp, xPosition.x, xPosition.y, xPosition.z );
	xWorldViewProj *= xTemp;

	// Light position

	const Asura_Vector_3& xLightPos = xShadow.GetLightPosition();
	D3DXMatrixTranslation( &xTemp, -xLightPos.x, -xLightPos.y, -xLightPos.z );
	xWorldViewProj *= xTemp;

	// Light orientation

	Asura_Matrix_3x3 xLightOri;
	xShadow.GetLightOrientation(xLightOri);
	memset( &xTemp, 0, sizeof( D3DXMATRIX ) );

	for( u_int u = 0; u < 3; ++u )
	{
		for( u_int v = 0; v < 3; ++v )
		{
			xTemp.m[u][v] = xLightOri.Element[u][v];

			if( v == 1 )
			{
				xTemp.m[u][v] = -xTemp.m[u][v];
			}
		}
	}

	xTemp._44 = 1.0f;

	xWorldViewProj *= xTemp;

	// Projection

	const float fVerticalFOV = Asura_Maths::DegToRad( xShadow.GetFrustum().GetVerticalFOVInDegrees() );
	D3DXMatrixPerspectiveFovLH( &xTemp, fVerticalFOV, 1.0f, xShadow.GetFrustum().GetNearPlane(), xShadow.GetFrustum().GetFarPlane() );

	xWorldViewProj *= xTemp;

	g_pxObjectShadowEffect->GetInterface()->SetMatrixF( g_xShadowWorldViewProjHandle, xWorldViewProj );

	// Frustum parameters

	const float fShadowFrustumNearPlane = xShadow.GetFrustum().GetNearPlane();
	g_pxObjectShadowEffect->GetInterface()->SetFloat( g_xShadowFrustumNearPlaneHandle, fShadowFrustumNearPlane );

	const float fOneOverShadowFrustumDepth = 1.0f / (xShadow.GetFrustum().GetFarPlane() - fShadowFrustumNearPlane);
	g_pxObjectShadowEffect->GetInterface()->SetFloat( g_xOneOverShadowFrustumDepthHandle, fOneOverShadowFrustumDepth );

	g_pxObjectShadowEffect->Commit();

	// Render the strip

	Asura_Xbox360_Graphics::GetDevice()->DrawIndexedVertices( D3DPT_TRIANGLESTRIP, 0, xStrip.m_uStartIndex, xStrip.m_uNumberOfIndices );

	Asura_Render::AddToTrianglesRendered( xStrip.m_uNumberOfTriangles );

	PIXEndNamedEvent(); // Strip
}

void Asura_Xbox360_ObjectHierarchyRenderer::EndShadowRender()
{
	g_pxObjectShadowEffect->End();
	s_pxCurrentHierarchy = NULL;

	PIXEndNamedEvent(); // Hierarchy Shadow
}

////////////////////////////////////////////////////////////////////////////////
//
// Asura_Xbox360_ObjectHierarchyRenderer::IsHierarchyOpaque
//
// Queries whether a hierarchy is entirely opaque
//
// Parameters:		uHash - Hash ID of hierarchy name
// Return value:	true iff all strips are opaque
//
////////////////////////////////////////////////////////////////////////////////

bool Asura_Xbox360_ObjectHierarchyRenderer::IsHierarchyOpaque( const Asura_Hash_ID uHash )
{
	const Hierarchy* const pxHierarchy = s_xHierarchyTree.GetDataByHash( uHash );
	Asura_Assert( pxHierarchy, "Failed to get object hierarchy data." );

	for( u_int u = 0; u < pxHierarchy->GetNumberOfStrips(); ++u )
	{
		const Asura_Hash_ID uMaterialHash = pxHierarchy->GetStrip(u).m_auMaterialResponseHashIDs[0];
		const Asura_MaterialResponse* const pxMaterial
			= Asura_MaterialResponse_System::GetMaterialByHashID( uMaterialHash );

		// Decal-type materials count as opaque

		if( pxMaterial && pxMaterial->IsTransparent() ) 
		{
			return false;
		}
	}

	return true;
}
