//
// Asura_Xbox360_TextureManagement.h
//
// Created by KJFL - 27 June 2005
//
// Copyright (C) 2005 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif

#ifndef __ASURA_XBOX360_TEXTUREMANAGEMENT_H__
#define __ASURA_XBOX360_TEXTUREMANAGEMENT_H__

/******************************************************************************/
/* Included Files                                                             */
/******************************************************************************/

// Asura Platform-independent
#include "Core/Asura_TextureFlags.h"

// Asura Xbox 360
#include "Rendering/Asura_Xbox360_Graphics.h"

/******************************************************************************/
/* Classes                                                                    */
/******************************************************************************/

class Asura_Chunk_Stream;
class Asura_Xbox360_XPR;

////////////////////////////////////////////////////////////////////////////////
//
// Asura_Xbox360_Texture
//
// A texture loaded on 360. The actual image data might be in a bundled XPR,
// or managed by the texture if loaded directly from image file
//
////////////////////////////////////////////////////////////////////////////////

class Asura_Xbox360_Texture
{
	// Friends
	friend class Asura_Texture_Management;
	friend class Asura_Xbox360_TextureManagement;

public:

	// Constructor / destructor
	Asura_Xbox360_Texture();
	~Asura_Xbox360_Texture();

	// Functions
	Direct3D_BaseTexture* GetD3DTexture() { return m_pxD3DTexture; }

	// Need ability to replace placeholder textures with our own interface
	void SetD3DTexture(Direct3D_BaseTexture* pTex) { m_pxD3DTexture = pTex; }

protected:

	// Functions
	void Destroy();
	void SetupPlaceholderTexture( const D3DCOLOR uColour );

	void InitFromFileInMemory( const char* const szName, const void* const pData,
		const int iDataSize, const bool bSRGB );

	void InitFromXPR( const char* const szName, Direct3D_Texture* const pxD3DTexture,
		void* const pData, Asura_Xbox360_XPR& xXPR );

	// Data
	Direct3D_BaseTexture* m_pxD3DTexture;
	Asura_Xbox360_XPR*    m_pxXPR;
	const char*           m_szName;
};

////////////////////////////////////////////////////////////////////////////////
//
// Asura_Xbox360_TextureManagement
//
// Handles resource management for textures on 360
//
////////////////////////////////////////////////////////////////////////////////

class Asura_Xbox360_TextureManagement
{
public:

	// Static functions
	static Asura_Xbox360_Texture* GetTexture( const int iTexID );
	static void CreateTextureFromFileInMemory( const int iTexID, const void* const pData, const int iDataSize, const bool bSRGB, bool bStreamedTexture = false );
	static void CreateTextureFromXPR( const char* const szName, Direct3D_Texture* const pxD3DTexture, void* const pData, Asura_Xbox360_XPR& xXPR, bool bStreamedTexture = false );
	static void Initialise();
	static void Reset();
	static void Shutdown();

	static bool TextureBudgetEnabled()						{return s_bTextureBudgetEnabled;}
	static void SetTextureBudget(bool bEnable, int iBytes)  {s_bTextureBudgetEnabled = bEnable; s_iTextureBudget = iBytes;}
	static int  GetTextureBudget()							{return s_iTextureBudget;}
	static void DecrementTextureBudget(int iBytes)			{s_iTextureBudget-=iBytes;}
	static void SetTextureBudgetExceededAmount(int iBytes)  {s_iTextureBudgetExceededBy = iBytes;}
	static int  GetTextureBudgetExceededAmount(void)		{return s_iTextureBudgetExceededBy;}

	static void CreatePlaceholderTexture( const char* const szName);

protected:

	static bool s_bTextureBudgetEnabled;
	static int	s_iTextureBudget;
	static int  s_iTextureBudgetExceededBy;
};

#endif // !defined __ASURA_XBOX360_TEXTUREMANAGEMENT_H__
