//////////////////////////////////////////////////////////////////////////////
//
// Asura_Xbox360_VertexBuffer.cpp
//
// Copyright (c) 2006 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

/******************************************************************************/
/* Included Files                                                             */
/******************************************************************************/

#include "Core/Asura.h"

#include "Rendering/Asura_Xbox360_VertexBuffer.h"

// Asura Xbox 360
#include "Rendering/Asura_Xbox360_Graphics.h"

// Misc
#include <xgraphics.h>
#include <xtl.h>

/******************************************************************************/
/* Asura_Xbox360_VertexBuffer Class Functions                                 */
/******************************************************************************/

Asura_Xbox360_VertexBuffer::Asura_Xbox360_VertexBuffer()
	: m_pxBuffer( NULL )
	, m_uVertexSize(0)
	, m_bManuallyManaged( false )
{
}

Asura_Xbox360_VertexBuffer::~Asura_Xbox360_VertexBuffer()
{
	Destroy();
}

// Create a static (i.e. write to once) vertex buffer, of the given size. Other methods to cope with
// more 'usage' options/combinations e.g. dynamic, patches, etc. may be required later.
void Asura_Xbox360_VertexBuffer::CreateStatic( const u_int uVertexSize,
	const u_int uNumberOfVertices, void* pData /*= NULL*/, const u_int uDataOffset /*=0*/ )
{
	m_uVertexSize = uVertexSize;

	Asura_Assert( !m_pxBuffer, "Vertex buffer being created twice." );

	const u_int uBufferSize = uVertexSize * uNumberOfVertices;
	Asura_Assert( uBufferSize > 0, "Trying to create zero-sized vertex buffer." );

	if( uBufferSize > 0 )
	{
		if( pData )
		{
			m_pxBuffer = new Direct3D_VertexBuffer;

			if( m_pxBuffer )
			{
				XGSetVertexBufferHeader( uBufferSize, 0, D3DPOOL_DEFAULT, uDataOffset, m_pxBuffer );
				XGOffsetResourceAddress( m_pxBuffer, pData );
			}
		}
		else
		{
			// Usage virtually ignored on 360
			Asura_Xbox360_Graphics::GetDevice()->CreateVertexBuffer( uBufferSize, 0, 0,
				D3DPOOL_DEFAULT, &m_pxBuffer, NULL );
		}

		Asura_Assert( m_pxBuffer,
			"Failed to create vertex buffer. This is normally an 'out of memory' condition." );
	}

	m_bManuallyManaged = (pData != NULL);
}

void Asura_Xbox360_VertexBuffer::Destroy()
{
	if( m_bManuallyManaged )
	{
		// Owner's responsibility to free whatever memory we were pointing at
		delete m_pxBuffer;
	}
	else if( m_pxBuffer )
	{
		m_pxBuffer->Release();
	}

	m_pxBuffer = NULL;
}

void* Asura_Xbox360_VertexBuffer::Lock()
{
	void* pData = NULL;

	if( m_pxBuffer )
	{
		m_pxBuffer->Lock( 0, 0, &pData, 0 );
		Asura_Assert( pData, "Failed to lock vertex buffer." );
	}

	return pData;
}

void Asura_Xbox360_VertexBuffer::Unlock()
{
	if( m_pxBuffer )
	{
		m_pxBuffer->Unlock();
	}
}

// set this vertex buffer as the active one for rendering
void Asura_Xbox360_VertexBuffer::Bind( const u_int uStream /*= 0*/ ) const
{
	Asura_Xbox360_Graphics::GetDevice()->SetStreamSource( uStream, m_pxBuffer, 0, m_uVertexSize );
}

bool Asura_Xbox360_VertexBuffer::IsEmpty() const
{
	return !m_pxBuffer;
}
