
/********************************************************************************/
/* Botanicals_Common.fxh                                                        */
/*																				*/
/* As we seem to be wanting full material support on the bots, I've rearranged	*/
/* this to be more like the PC version - with a common pixel shader shared by	*/
/* all botanical elements.														*/
/*																				*/				
/* Although I'm not doing it for imposters yet, as tangent space isn't exactly  */
/* straightforward for those...													*/
/*																				*/
/********************************************************************************/

#include "Shared.fxh"
#include "MaterialResponse_Shared.fxh"

float   g_fDualSidedLightingLerp = 0.35f;

/******************************************************************************/
/* Types                                                                      */
/******************************************************************************/

struct OutputVertex_Botanicals_Prelighting
{
	float4 m_xPosition			: POSITION;
	
	float3 m_xColour            : COLOR;
	float3 m_xBackColour        : TEXCOORD1;

	float4 m_xUVPair            : TEXCOORD0;
	
	float4 m_xCameraDirDistance : TEXCOORD4; // Camera direction, and distance from view plane in w

	float4 m_afAlphaCut			: TEXCOORD5;
	
	float3 m_xTangent           : TANGENT;
	float3 m_xBinormal          : BINORMAL;
	float3 m_xNormal            : NORMAL;		
};

/******************************************************************************/
/* Pixel Shader                                                               */
/******************************************************************************/

float3 Bot_CalcNewBonedPosition( float3 xOldPosition, float fBoneWeight, float3 xBone)
{
#if 0
	float3 xNewPosition = xOldPosition + (fBoneWeight * xBone);
#endif
	
#if 1
	float fOldPositionLength = sqrt( dot( xOldPosition.xyz, xOldPosition.xyz));
	float3 xBonedPosition = xOldPosition + (fBoneWeight * xBone);
	float3 xBoneDirection = normalize( xBonedPosition);
	float3 xNewPosition = xBoneDirection * fOldPositionLength;
#endif
	
	return( xNewPosition);
}

DeferredOutputPixel PShader_Botanicals_Prelighting( const OutputVertex_Botanicals_Prelighting xIn, float fFace:VFace )
{
	const float3x3 xTangentToWorldMatrix = float3x3( normalize( xIn.m_xTangent ),
													 normalize( xIn.m_xBinormal ),
													 normalize( xIn.m_xNormal ) );

	const float4 xUVPair = MaterialResponse_AnimateUVs( xIn.m_xUVPair );
	
	float4 xWorldSpaceNormal = NormalLookup( xUVPair, 0, xTangentToWorldMatrix, xTangentToWorldMatrix);

	half4 xColour;		
	if( 0.0f > fFace)
	{
		xWorldSpaceNormal.xyz = -xWorldSpaceNormal.xyz;
		xColour.rgb = lerp(xIn.m_xBackColour.rgb, xIn.m_xColour.rgb, g_fDualSidedLightingLerp);
	}
	else
	{
		xColour.rgb = lerp(xIn.m_xColour.rgb, xIn.m_xBackColour.rgb, g_fDualSidedLightingLerp);
	}
									
	
	// -----------------------------------------------------------------
	// Diffuse-only SH prelighting 
	// (moved to VS, now per-vert as we're doing it for both sides)
	// -----------------------------------------------------------------
	
	const float fMaterial = g_xMaterialProperties.x;
	half3 xAlbedo;
	const float3 xAlbedoTimes2 = AlbedoLookup( xUVPair,
											   0.0,
											   xWorldSpaceNormal,
											   normalize( xIn.m_xCameraDirDistance.xyz ),
											   xAlbedo );
/*	
	xColour.r	= dot (xWorldSpaceNormal, xIn.m_xRedSH);
	xColour.g	= dot (xWorldSpaceNormal, xIn.m_xGreenSH);
	xColour.b	= dot (xWorldSpaceNormal, xIn.m_xBlueSH);
	xColour.rgb = max (xColour.rgb, 0) * xAlbedoTimes2 + SelfIlluminationLookup( xUVPair, 0.0, xAlbedo );
*/
	xColour.rgb = xColour.rgb * xAlbedoTimes2;
	xColour.a = TransparencyLookup( xUVPair, 0.0 );
	
	xColour.a = xColour.a - xIn.m_afAlphaCut.x;

	half hSpecular = SpecularLookup( xUVPair, 0.0 );
	
	DeferredOutputPixel xOut = Deferred_FillOutputPixel(
										xColour,
										xWorldSpaceNormal.xyz,
										xIn.m_xCameraDirDistance.w,
										xAlbedo,
										hSpecular,
										fMaterial
										);
	

																		
	return( xOut);
}
