/******************************************************************************/
/* Constants                                                                  */
/******************************************************************************/


#include "Botanicals_Common.fxh"


const float   g_fInstanceSize; // In indices. Ints are cast to floats anyway outside of loop and vertex indices
const float   g_fFirstInstance;
const int     g_eCullMode;


/******************************************************************************/
/* Functions                                                                  */
/******************************************************************************/

OutputVertex_Botanicals_Prelighting VShader( const int iIndex : INDEX )
{
	// Integer arithmetic has some rounding errors; see help topic "Vertex Fetching in HLSL"
	// and thread "Hardware instancing oddness" in xds360.graphics

	int iInstanceIndex = (iIndex + 0.5) / g_fInstanceSize;
	const int iIndexIndex = iIndex - iInstanceIndex * g_fInstanceSize; // The real indices sit in a vertex stream

	iInstanceIndex += g_fFirstInstance;

	float  fVertexIndex;
	float4 xPosition;
	float2 xUV;
	float3 xNormal;
	float3 xTangent;
	float3 xBinormal;
	float3 xInstanceOrientation0;
	float3 xInstanceOrientation1;
	float3 xInstanceOrientation2;
	float4 xInstancePositionAndAlpha;
	float4 xRedSH;
	float4 xGreenSH;
	float4 xBlueSH;	
	float  fBoneWeight;
	float4 xBoneOffsetAndScale;

	asm
	{
		// Find the real vertex index
		vfetch fVertexIndex.x, iIndexIndex, position1

		// Get the mesh vertex attributes
		vfetch xPosition, fVertexIndex, position0
		vfetch xUV.xy, fVertexIndex, texcoord0
		vfetch xNormal.xyz, fVertexIndex, normal
		vfetch xTangent.xyz, fVertexIndex, tangent
		vfetch xBinormal.xyz, fVertexIndex, binormal
		vfetch fBoneWeight.x, fVertexIndex, blendweight

		// Grab the instance's world matrix
		vfetch xInstanceOrientation0.xyz, iInstanceIndex, texcoord1
		vfetch xInstanceOrientation1.xyz, iInstanceIndex, texcoord2
		vfetch xInstanceOrientation2.xyz, iInstanceIndex, texcoord3
		vfetch xInstancePositionAndAlpha, iInstanceIndex, texcoord4

		// Grab the spherical harmonic
		vfetch xRedSH,   iInstanceIndex, texcoord5
		vfetch xGreenSH, iInstanceIndex, texcoord6
		vfetch xBlueSH,  iInstanceIndex, texcoord7

		// Bone Position Offset
		vfetch xBoneOffsetAndScale, iInstanceIndex, texcoord8
	};

	//xPosition.xyz += xBoneOffsetAndScale.xyz * fBoneWeight.x;
	xPosition.xyz = Bot_CalcNewBonedPosition( xPosition.xyz, fBoneWeight.x, xBoneOffsetAndScale.xyz);

	const float3x4	xWorldMatrix	= float3x4(float4(xInstanceOrientation0.xyz * xBoneOffsetAndScale.w, xInstancePositionAndAlpha.x),
											   float4(xInstanceOrientation1.xyz * xBoneOffsetAndScale.w, xInstancePositionAndAlpha.y),
											   float4(xInstanceOrientation2.xyz * xBoneOffsetAndScale.w, xInstancePositionAndAlpha.z));
	float4 xWorldPosition;
	xWorldPosition.xyz = mul( xWorldMatrix, xPosition ); 
	xWorldPosition.w = 1.0;
	
	OutputVertex_Botanicals_Prelighting xOut;
		
	xOut.m_xPosition	= mul( xWorldPosition, g_xViewProjectionMatrix );	

	xOut.m_xUVPair.xyzw = xUV.xyxy;	
   	
   	
	// Lighting calculations all done in world space now
	// --------------------------------------------------------------------------------------
			   	
   	xOut.m_afAlphaCut   = xInstancePositionAndAlpha.w;
	
	xOut.m_xTangent     = mul(  xWorldMatrix, xTangent );
	xOut.m_xBinormal    = mul(  xWorldMatrix, xBinormal );
	xOut.m_xNormal      = mul(  xWorldMatrix, xNormal );
	
	// Light it - from front and back, then lerp
	half3 xColour, xBackColour;		
	half4 xNorm = half4(xOut.m_xNormal,1);						
	xOut.m_xColour.rgb = SHLookup( xNorm, xRedSH, xGreenSH, xBlueSH );
		
	xNorm.rgb=-xNorm.rgb;	
	xOut.m_xBackColour.rgb =  SHLookup( xNorm, xRedSH, xGreenSH, xBlueSH );
	 
	
	xOut.m_xCameraDirDistance.xyz = g_xCameraPosition.xyz - xWorldPosition.xyz;
	xOut.m_xCameraDirDistance.w   = xOut.m_xPosition.w;
	
	return xOut;
		
}




/******************************************************************************/
/* Techniques                                                                 */
/******************************************************************************/

technique
{
	pass
	{
		AlphaTestEnable = TRUE;
		AlphaFunc = GREATER;
		AlphaRef = 0;
		CullMode = <g_eCullMode>;
		ZEnable = TRUE;
		ZWriteEnable = TRUE;

		VertexShader = compile vs_3_0 VShader();
		PixelShader = compile ps_3_0 PShader_Botanicals_Prelighting();
	}
}