
// Decals.fx - Decal shader - based on the environment shader

#include "Shared.fxh"
#include "MaterialResponse_Shared.fxh"

const float3 g_xPrelightingColourFilter : register(c0);

float4 g_xFadeValues;		// .x = current time, .y = fade in length, .z = fade out length, .w = fade out start
float4 g_xUVScroll;
 

//------------------------------------
struct VS_Input
{
	float4	m_xPosition					: POSITION0;
	float4	m_axUVTimeAlpha				: TEXCOORD0;
	float3	m_xNormal					: NORMAL;
	float3	m_xTangent					: TANGENT;
	float3	m_xBinormal					: BINORMAL;

	float4 m_xRedSH     : TEXCOORD1;
	float4 m_xGreenSH   : TEXCOORD2;
	float4 m_xBlueSH    : TEXCOORD3;
};

struct VS_Output
{
    float4 m_xPosition					: POSITION0;    
	float4 m_xUVPair					: TEXCOORD0;
	float4 m_xTangentSpaceCameraDir		: TEXCOORD1;	// w is actually the alpha value

	float4 m_xCameraDirDistance			: TEXCOORD2;	// Camera direction, and distance from view plane in w

	float4 m_xRedSH						: TEXCOORD3;
	float4 m_xGreenSH					: TEXCOORD4;
	float4 m_xBlueSH					: TEXCOORD5;

	float3 m_xNormal					: NORMAL;
	float3 m_xTangent					: TANGENT0;
	float3 m_xBinormal					: BINORMAL0;


};

struct VS_Output_Specular
{
    float4	m_xPosition					: POSITION0;    
	float4	m_xUVPair					: TEXCOORD0;

	float4 m_xTangentSpaceCameraDir		: TEXCOORD5;
};


struct PS_Output
{
	half4 m_xBufferColour         : COLOR0;
	half4 m_xBufferNormal		  : COLOR1;
	half4 m_xBufferAlbedo		  : COLOR2;
};


//------------------------------------
VS_Output VS_Decals_MaterialResponse (VS_Input xIn)
{
	VS_Output	xOut;

    // Calculate the vertex position...
    xOut.m_xPosition	= mul( xIn.m_xPosition, g_xViewProjectionMatrix );
    
	xOut.m_xUVPair = MaterialResponse_AnimateUVs( xIn.m_axUVTimeAlpha.xyxy + g_xUVScroll.xyxy);
    
	xOut.m_xTangent = xIn.m_xTangent;
	xOut.m_xBinormal = xIn.m_xBinormal;
	xOut.m_xNormal = xIn.m_xNormal;

	xOut.m_xRedSH = xIn.m_xRedSH * g_xPrelightingColourFilter.r;
	xOut.m_xGreenSH = xIn.m_xGreenSH * g_xPrelightingColourFilter.g;
	xOut.m_xBlueSH = xIn.m_xBlueSH * g_xPrelightingColourFilter.b;

	xOut.m_xCameraDirDistance.xyz = g_xCameraPosition.xyz - xIn.m_xPosition.xyz;
	xOut.m_xCameraDirDistance.w = xOut.m_xPosition.w;

	const float3x3 xTangentToWorldMatrix = float3x3( xIn.m_xTangent.xyz,
													 xIn.m_xBinormal.xyz,
													 xIn.m_xNormal
													 );

	// Note implicit transpose in the mul, by swapping operand order
	xOut.m_xTangentSpaceCameraDir.xyz = mul( xTangentToWorldMatrix, xOut.m_xCameraDirDistance.xyz );

	float fCreationTime = xIn.m_axUVTimeAlpha.z;
	
	// Handle the fading
	if (g_xFadeValues.x < fCreationTime+g_xFadeValues.w)
	{
		// Fade in
		xOut.m_xTangentSpaceCameraDir.w = min((g_xFadeValues.x - fCreationTime) / g_xFadeValues.y , 1);
	}
	else
	{
		// Fade out
		float fZeroTime = fCreationTime + g_xFadeValues.z+g_xFadeValues.w;		
		xOut.m_xTangentSpaceCameraDir.w = max((fZeroTime - g_xFadeValues.x)/g_xFadeValues.z,0);
	}

	xOut.m_xTangentSpaceCameraDir.w *= xIn.m_axUVTimeAlpha.w; 

    // Return the vertex...
    return	(xOut);

}


//------------------------------------
VS_Output_Specular VS_Decals_MaterialResponse_Specular (VS_Input xIn)
{
	VS_Output_Specular	xOut;

    // Calculate the vertex position...
    xOut.m_xPosition	= mul( xIn.m_xPosition, g_xViewProjectionMatrix );
    
	xOut.m_xUVPair = MaterialResponse_AnimateUVs( xIn.m_axUVTimeAlpha.xyxy + g_xUVScroll.xyxy);
    

	float3 xCameraDirDistance = g_xCameraPosition.xyz - xIn.m_xPosition.xyz;

	const float3x3 xTangentToWorldMatrix = float3x3( xIn.m_xTangent.xyz,
													 xIn.m_xBinormal.xyz,
													 xIn.m_xNormal
													 );

	// Note implicit transpose in the mul, by swapping operand order
	xOut.m_xTangentSpaceCameraDir.xyz = mul( xTangentToWorldMatrix, xCameraDirDistance.xyz );

	float fCreationTime = xIn.m_axUVTimeAlpha.z;
	
	// Handle the fading
	if (g_xFadeValues.x < fCreationTime+g_xFadeValues.w)
	{
		// Fade in
		xOut.m_xTangentSpaceCameraDir.w = min((g_xFadeValues.x - fCreationTime) / g_xFadeValues.y , 1);
	}
	else
	{
		// Fade out
		float fZeroTime = fCreationTime + g_xFadeValues.z+g_xFadeValues.w;		
		xOut.m_xTangentSpaceCameraDir.w = max((fZeroTime - g_xFadeValues.x)/g_xFadeValues.z,0);
	}

	xOut.m_xTangentSpaceCameraDir.w *= xIn.m_axUVTimeAlpha.w; 

    // Return the vertex...
    return	(xOut);

}


PS_Output PS_Decals_MaterialResponse (VS_Output xIn)
{	
	const half3 xInterpolatedNormal    = normalize(xIn.m_xNormal.xyz);
 	const half3 xInterpolatedTangent   = normalize(xIn.m_xTangent.xyz);
	const half3 xInterpolatedBinormal  = normalize(xIn.m_xBinormal.xyz);

	const half3x3 xTangentToWorldMatrix
		= { xInterpolatedTangent, xInterpolatedBinormal, xInterpolatedNormal };

	const half3 xViewDir = normalize( xIn.m_xCameraDirDistance.xyz );
	const half3 xTangentSpaceViewDir = normalize( xIn.m_xTangentSpaceCameraDir );

	ApplyParallaxMapping( xTangentSpaceViewDir, 1.0f, xIn.m_xUVPair );

	const half4 xWorldSpaceNormal = NormalLookup( xIn.m_xUVPair, 1.0f, xTangentToWorldMatrix, xTangentToWorldMatrix );

	half3 xAlbedo;
	half hSpecular;
	float fMaterialsTextureY;
	half4 xColour;

	const float3 xLightColour = SHLookup( half4( xInterpolatedNormal, 1.0h ), xIn.m_xRedSH, xIn.m_xGreenSH, xIn.m_xBlueSH ) * 2.0f;

	const float fLightStrength = GetLuminance( xLightColour );

	xColour = MaterialResponse( xIn.m_xUVPair, 1.0f, xWorldSpaceNormal, xViewDir,
		xInterpolatedNormal, xLightColour, xLightColour, fLightStrength, false, xAlbedo,
		hSpecular, fMaterialsTextureY );

	xColour.rgb += SelfIlluminationLookup( xIn.m_xUVPair, 1.0h, xAlbedo );

	// include fade value
	xColour.a *= xIn.m_xTangentSpaceCameraDir.w;

	float fMaterial =  xColour.a;	


	PS_Output xOut;
	xOut.m_xBufferColour = xColour;
	xOut.m_xBufferNormal = float4((xWorldSpaceNormal.xyz * 0.5f) + 0.5f, xColour.a);
	xOut.m_xBufferAlbedo = float4(xAlbedo, xColour.a);

	return( xOut);
 }

PS_Output PS_Decals_MaterialResponse_Specular (VS_Output_Specular xIn)
{	
	const half3 xTangentSpaceViewDir = normalize( xIn.m_xTangentSpaceCameraDir.xyz );

	ApplyParallaxMapping( xTangentSpaceViewDir, 1.0h, xIn.m_xUVPair );


	half hSpecular = SpecularLookup(xIn.m_xUVPair, 1.0h);
	half hAlpha = TransparencyLookup(xIn.m_xUVPair, 1.0h );

	hSpecular *= hAlpha *  xIn.m_xTangentSpaceCameraDir.w;

	PS_Output xOut;
	xOut.m_xBufferColour = float4(0,0,0, 0);
	xOut.m_xBufferNormal = float4(0,0,0, 0);
	xOut.m_xBufferAlbedo = float4(0,0,0, hSpecular);

	return xOut;
 }




/******************************************************************************/
/* Techniques                                                                 */
/******************************************************************************/

technique Decal_MaterialResponse
{
    pass p0
    {
		VertexShader	= compile vs_3_0 VS_Decals_MaterialResponse();
		PixelShader		= compile ps_3_0 PS_Decals_MaterialResponse();
    }
    pass specular
    {
		VertexShader	= compile vs_3_0 VS_Decals_MaterialResponse_Specular();
		PixelShader		= compile ps_3_0 PS_Decals_MaterialResponse_Specular();
    }
}