/******************************************************************************/
/* Parameters                                                                 */
/******************************************************************************/

const float2 g_xCoefficients; // = { 0.5 * (brightness - contrast), 0.5 * (contrast + 1) }

shared const sampler2D g_xInitialImage = sampler_state
{
	AddressU = CLAMP;
	AddressV = CLAMP;
};

/******************************************************************************/
/* Functions                                                                  */
/******************************************************************************/

float4 VShader( const float4 xIn : POSITION ) : POSITION
{
	return xIn;
}

float4 PShader( const float2 xScreenPos : VPOS ) : COLOR
{
	float4 xInitialColour;
	asm
	{
		tfetch2D xInitialColour, xScreenPos.xy, g_xInitialImage, UnnormalizedTextureCoords=true, MinFilter=point, MagFilter=point, MipFilter=point, AnisoFilter=disabled
	};

    // Gamma to linear space
	xInitialColour.rgb = pow(xInitialColour.rgb, 2.2);

	float4 xOut;
	xOut.rgb = g_xCoefficients.x + g_xCoefficients.y * xInitialColour.rgb;
	xOut.a = xInitialColour.a;

	// Linear to gamma space
	xOut.rgb = pow(xOut.rgb, 1.0f/2.2);

	return xOut;
}

/******************************************************************************/
/* Techniques                                                                 */
/******************************************************************************/

technique
{
	pass
	{
		AlphaTestEnable = FALSE;
		CullMode = NONE;
		ZEnable = FALSE;
		ZWriteEnable = FALSE;

		VertexShader = compile vs_3_0 VShader();
		PixelShader = compile ps_3_0 PShader();
	}
}