/******************************************************************************/
/* Parameters                                                                 */
/******************************************************************************/

#include "Shared.fxh"

const float3 g_xFogColour;

const float g_fFogNearPlane;
const float g_fOneOverFogVolumeDepth; // = 1 / (fog far plane - fog near plane)

const float g_fFarClipPlane;

const sampler1D g_xFogTexture = sampler_state
{
	AddressU = CLAMP;
};

const sampler g_xDepthTexture = sampler_state
{
	AddressU = CLAMP;
	AddressV = CLAMP;
};

/******************************************************************************/
/* Functions                                                                  */
/******************************************************************************/

float4 VShader( const float4 xIn : POSITION ) : POSITION
{
	return xIn;
}

float4 PShader( float2 xScreenPos : VPOS ) : COLOR
{
	float4 xDepthMtl;

	xScreenPos += g_xVPOSOffsetVector;

	asm
	{
		tfetch2D xDepthMtl, xScreenPos, g_xDepthTexture, UnnormalizedTextureCoords=true, MinFilter=point, MagFilter=point, MipFilter=point, AnisoFilter=disabled
	};

	// Kill pixel if the distance is beyond the far clip plane (assume it's skybox)
	clip( g_fFarClipPlane - xDepthMtl.r );

	const float fLookup = (xDepthMtl.r - g_fFogNearPlane) * g_fOneOverFogVolumeDepth;
	float4 fFogTextureColour;

	asm
	{
		tfetch1D fFogTextureColour, fLookup, g_xFogTexture, MinFilter=linear, MagFilter=linear, MipFilter=point, AnisoFilter=disabled
	};

	return float4( g_xFogColour, fFogTextureColour.a );
}

/******************************************************************************/
/* Techniques                                                                 */
/******************************************************************************/

technique
{
	pass
	{
		AlphaTestEnable = FALSE;
		CullMode = NONE;
		VertexShader = compile vs_3_0 VShader();
		PixelShader = compile ps_3_0 PShader();
	}
}