
// NormalBufferOverlayRenderer.fx

//------------------------------------
const float4	g_xFlatColour;


//------------------------------------
struct VS_Input
{
	float2 Position		: POSITION0;
	float2 UV			: TEXCOORD0;
};

struct VS_Output
{
	float4 Position		: POSITION0;
	float2 UV			: TEXCOORD0;
};

uniform sampler xSampler = sampler_state
{
	AddressU = WRAP;
	AddressV = WRAP;
};

//------------------------------------
VS_Output VS_Overlay(VS_Input IN) 
{
	VS_Output OUT;

	// Mainly a straight pass-through job, but
	// we want to make z & w represent a polygon
	// right in front of the camera, i.e. a z
	// distance of zero. This makes w = 1.0f.

	OUT.Position.xy = IN.Position.xy;
	
	OUT.Position.z = 0.0f;
	OUT.Position.w = 1.0f;

	OUT.UV = IN.UV;
	

	return OUT;
}

//------------------------------------
float4 PS_Overlay (VS_Output IN) : COLOR
{

	// The nasty encoding is gone now... no need for this shader any more, really...
	
	float4 xSample = tex2D ( xSampler, IN.UV );
	
	xSample.rgb = xSample.rgb; 
	xSample.a = 1.0f;
	
	return xSample * g_xFlatColour;

}


//-----------------------------------
technique OverlayQuad
{
	pass Pass1
	{
		AlphaTestEnable = FALSE;
		CullMode = NONE;
		ZEnable = FALSE;
		ZWriteEnable = FALSE;

		VertexShader = compile vs_2_0 VS_Overlay();

		PixelShader = compile ps_2_0 PS_Overlay();
	}
}