/******************************************************************************/
/* Parameters                                                                 */
/******************************************************************************/

const float4x4 g_xWorldViewProj;

const float g_fShadowFrustumNearPlane;		// Distance to near plane of shadow frustum from light position
const float g_fOneOverShadowFrustumDepth;	// 1 / (frustum far plane - frustum near plane)

/******************************************************************************/
/* Types                                                                      */
/******************************************************************************/

struct InputVertex
{
	float4 Position : POSITION;
};

struct OutputVertex
{
	float4 Position          : POSITION;
	float LightSpaceDistance : TEXCOORD;
};

/******************************************************************************/
/* Functions                                                                  */
/******************************************************************************/

OutputVertex VShader( const InputVertex xIn )
{
	OutputVertex xOut;

	xOut.Position = mul( xIn.Position, g_xWorldViewProj );
	xOut.LightSpaceDistance = (xOut.Position.w - g_fShadowFrustumNearPlane) * g_fOneOverShadowFrustumDepth;

	return xOut;
}

float4 PShader( const OutputVertex xIn ) : COLOR
{
	// Just output the iterated distance value
	return xIn.LightSpaceDistance.xxxx;
}

/******************************************************************************/
/* Techniques                                                                 */
/******************************************************************************/

technique
{
	pass
	{
		AlphaTestEnable = FALSE;
		CullMode = CW;
		ZEnable = TRUE;
		ZWriteEnable = TRUE;

		VertexShader = compile vs_3_0 VShader();
		PixelShader = compile ps_3_0 PShader();
	}
}