/******************************************************************************/
/* Parameters                                                                 */
/******************************************************************************/

#include "Fog.fxh"
#include "Shared.fxh"

const sampler g_xTexture = sampler_state
{
	AddressU = CLAMP;
	AddressV = CLAMP;
};

const bool g_bZReadEnable;

const float4 g_xOneOverScreenDimensions;

const sampler g_xHeatHazeCaptureTexture = sampler_state
{	
	AddressU = CLAMP;
	AddressV = CLAMP;
};


/******************************************************************************/
/* Types                                                                      */
/******************************************************************************/

struct InputVertex
{
	float4 Position : POSITION;
	float4 Diffuse  : COLOR;
	float2 UV1      : TEXCOORD0;
	float2 UV2      : TEXCOORD1;
};

struct OutputVertex
{
	float4 Position						 : POSITION;
	float4 Diffuse						 : COLOR;
	float4 UVPair						 : TEXCOORD0;
	float2 xCrossFadeBlendWeightAndDepth : TEXCOORD2;
};

/******************************************************************************/
/* Functions                                                                  */
/******************************************************************************/

OutputVertex VShader( const InputVertex xIn )
{
	OutputVertex xOut;

	float4 xTmp = xIn.Position;
	xTmp.w = 1.0f;
		
	xOut.Position = mul( xTmp, g_xViewProjectionMatrix );
	xOut.Diffuse = xIn.Diffuse;
	xOut.UVPair.xy = xIn.UV1;
	xOut.UVPair.zw = xIn.UV2;
	xOut.xCrossFadeBlendWeightAndDepth.x = xIn.Position.w;
	xOut.xCrossFadeBlendWeightAndDepth.y = xOut.Position.w;
	
	return xOut;
}


float4 PShader( const OutputVertex xIn, uniform bool bCrossfadeTextureFrames ) : COLOR
{
	float4 xColour	= TexFetch2D_LinearLinearLinear( g_xTexture, xIn.UVPair.xy );

	if( bCrossfadeTextureFrames )
	{
		xColour = lerp( TexFetch2D_LinearLinearLinear( g_xTexture, xIn.UVPair.zw ), xColour, xIn.xCrossFadeBlendWeightAndDepth.x );
	}

	xColour *= xIn.Diffuse;

	if (g_bFogEnabled)
	{
		return ApplyFog(xColour,xIn.xCrossFadeBlendWeightAndDepth.y);
	}
	else
	{	
		return xColour;
	}
}




float4 PShader_HeatHaze( const OutputVertex xIn, float2 xScreenPos : VPOS, uniform bool bCrossfadeTextureFrames ) : COLOR
{
	xScreenPos += g_xVPOSOffsetVector;

	float4 xColour = TexFetch2D_LinearLinearLinear( g_xTexture, xIn.UVPair.xy );;

	if( bCrossfadeTextureFrames )
	{
		xColour = lerp( TexFetch2D_LinearLinearLinear( g_xTexture, xIn.UVPair.zw ), xColour, xIn.xCrossFadeBlendWeightAndDepth.x );
	}
	
	xColour.a *= xIn.Diffuse.a;
	
	// Sample the heathaze texture, Scale to get an offset	
	const float2 xNormalizedCoords = (xScreenPos.xy + float2(0.5, 0.5)) * g_xOneOverScreenDimensions.xy;	
	
	float2 xOffset = (xIn.Diffuse.rg * (xColour.rg-0.5f)) / (10.0f+xIn.xCrossFadeBlendWeightAndDepth.y);		
	
			
	// Do the offset lookup from the frame capture
#if 0
	xColour.rgb = 0.25f *
				  (
				  TexFetch2D_LinearLinearLinear(g_xHeatHazeCaptureTexture, xNormalizedCoords + 0.3f * xOffset).rgb + 
				  TexFetch2D_LinearLinearLinear(g_xHeatHazeCaptureTexture, xNormalizedCoords + 0.6f * xOffset).rgb +
				  TexFetch2D_LinearLinearLinear(g_xHeatHazeCaptureTexture, xNormalizedCoords + 0.8f * xOffset).rgb +
				  TexFetch2D_LinearLinearLinear(g_xHeatHazeCaptureTexture, xNormalizedCoords +        xOffset).rgb
				  );
#else
	xColour.rgb = 0.5f *
				  (
				  TexFetch2D_LinearLinearLinear(g_xHeatHazeCaptureTexture, xNormalizedCoords + 0.5f * xOffset).rgb +
				  TexFetch2D_LinearLinearLinear(g_xHeatHazeCaptureTexture, xNormalizedCoords +        xOffset).rgb
				  );
#endif
	
	return xColour;	
}



/******************************************************************************/
/* Techniques                                                                 */
/******************************************************************************/


technique PFX_Standard
{
	pass SingleTexture
	{
		AlphaTestEnable = FALSE;		
		CullMode = NONE;
		ZEnable = TRUE;

		VertexShader = compile vs_3_0 VShader();
		PixelShader = compile ps_3_0 PShader(false);
	}

	pass DualTexture
	{
		AlphaTestEnable = FALSE;		
		CullMode = NONE;
		ZEnable = TRUE;

		VertexShader = compile vs_3_0 VShader();
		PixelShader = compile ps_3_0 PShader(true);
	}
}



technique PFX_HeatHaze
{
	pass SingleTexture
	{
		AlphaTestEnable = FALSE;		
		CullMode = NONE;
		ZEnable = TRUE;

		VertexShader = compile vs_3_0 VShader();
		PixelShader = compile ps_3_0 PShader_HeatHaze(false);
	}

	pass DualTexture
	{
		AlphaTestEnable = FALSE;		
		CullMode = NONE;
		ZEnable = TRUE;

		VertexShader = compile vs_3_0 VShader();
		PixelShader = compile ps_3_0 PShader_HeatHaze(true);
	}
}