
// YUV_to_RGB.fx
//
// This takes 3 8-bit YUV textures and converts/blends them into one RGBA texture
//

#include <Shared.fxh>

//------------------------------------
struct VS_Input
{
	float4 Position		: POSITION0;
	float2 UV			: TEXCOORD0;
};

struct VS_Output
{
	float4 Position		: POSITION0;
	float2 UV			: TEXCOORD0;
};

struct PS_Output
{
	float4 Colour		: COLOR;
};

uniform sampler g_xYMap = sampler_state
{
	AddressU  = CLAMP;
	AddressV  = CLAMP;
};

uniform sampler g_xUMap = sampler_state
{
	AddressU  = CLAMP;
	AddressV  = CLAMP;
};

uniform sampler g_xVMap = sampler_state
{
	AddressU  = CLAMP;
	AddressV  = CLAMP;
};

//------------------------------------
VS_Output VS_YUV_to_RGB (VS_Input IN) 
{
	//
	// Pass-through vertex shader
	//
	
	// IN.Position promoted from float2 to float4
	// Ie. z = 0. Right in front of the camera (zero distance)
	return IN;
}

PS_Output PS_YUV_to_RGB (VS_Output IN)
{
	PS_Output    OUT;

	//
	// Texture lookup the YUV colour and calculate ARGB.
	//
	float4 f4d_Y = TexFetch2D_LinearLinearLinear ( g_xYMap, IN.UV );
	float4 f4d_U = TexFetch2D_LinearLinearLinear ( g_xUMap, IN.UV );
	float4 f4d_V = TexFetch2D_LinearLinearLinear ( g_xVMap, IN.UV );

    float4 ARGB;                             
    ARGB.r = 1.164 * ( f4d_Y.r - 0.0625 ) + 1.596 * ( f4d_V.r - 0.5 ); 
    ARGB.g = 1.164 * ( f4d_Y.r - 0.0625 ) - 0.391 * ( f4d_U.r - 0.5 ) - 0.813 * ( f4d_V.r - 0.5 ); 
    ARGB.b = 1.164 * ( f4d_Y.r - 0.0625 ) + 2.018 * ( f4d_U.r - 0.5 );                          
    ARGB.a = 1.0;                            
                                             
	OUT.Colour = ARGB;

	return OUT;
}

//-----------------------------------
technique YUV_to_RGB
{
	pass Pass1
	{
		VertexShader = compile vs_3_0 VS_YUV_to_RGB();
		PixelShader = compile ps_3_0 PS_YUV_to_RGB();
	}
}
