///////////////////////////////////////////////////////////////////////////////
//
// File   : Asura_Xbox360_Sound.h
// Author : Andy Patrick
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif

#ifndef __ASURA_XBOX360_SOUND_H__
#define __ASURA_XBOX360_SOUND_H__

#ifndef ASURA_USE_AUDIO_SYSTEM

#include "Sound/Asura_OpenAL.h"
#include "Sound/Asura_Sound.h"
#include <xact.h>

#if !defined ASURA_XBOX360_SOUND_MAX_VOICES
	#define ASURA_XBOX360_SOUND_MAX_VOICES		64
#endif

/* ------------------------------------------------------------------------- */

class Asura_Chunk_Stream;
class Asura_XACT_WaveBank;

/* ------------------------------------------------------------------------- */

#if defined( ASURA_USE_OPENAL ) && ( ASURA_USE_OPENAL == 1 )

/* ------------------------------------------------------------------------- */

class Asura_Xbox360_ActiveSound
{
public:

	Asura_Xbox360_ActiveSound();

	Asura_OpenAL_ActiveSound_API& GetAPI()		{ return m_xAPI; }

	void	SetLooping( bool bLooping )			{ m_bIsLooping = bLooping; }
	bool	IsLooping() const					{ return m_bIsLooping; }

	void	SetPausedByPitch( bool bPaused )	{ m_bPausedByPitch = bPaused; }
	bool	IsPausedByPitch() const				{ return m_bPausedByPitch; }

private:

	Asura_OpenAL_ActiveSound_API				m_xAPI;
	bool 										m_bIsLooping;
	bool 										m_bPausedByPitch;
};

/* ------------------------------------------------------------------------- */

struct RawXMASample
{
	RawXMASample() :
		m_pucXMAData(NULL),
		m_uXMADataSize(0),
		m_pucFormatBlock(NULL)
	{ }

	u_char*				m_pucXMAData;			// The XMA2 raw data
	u_int				m_uXMADataSize;			// Size of the XMA data in memory
	u_char*				m_pucFormatBlock;		// The XMA2 format block we must give to OpenAL
};

/* ------------------------------------------------------------------------- */

class Asura_Xbox360_Sound
{
public:	

	static Asura_OpenAL_Sound_API& GetAPI()		{ return s_xAPI; }

	static RawXMASample& GetSample( int iSample )				{ return s_axXMASamples[iSample]; }
	static Asura_Xbox360_ActiveSound& GetSound( int iSound )	{ return s_axActiveSounds[iSound]; }

	static void UpdateSound( Asura_Handle iActiveSoundIndex );

private:

	static Asura_OpenAL_Sound_API				s_xAPI;
	static RawXMASample							s_axXMASamples[ASURA_MAX_NUMBER_OF_LOADED_SOUNDS];
	static Asura_Xbox360_ActiveSound			s_axActiveSounds[ASURA_XBOX360_SOUND_MAX_VOICES];
};

/* ------------------------------------------------------------------------- */

#endif	// defined( ASURA_USE_OPENAL ) && ( ASURA_USE_OPENAL == 1 )

// --------- Forward declarations ---------------------------------------------

extern void Asura_Xbox360_LoadChunkedSoundData (Asura_Chunk_Stream& xStream);

// ----------- Asura_Xbox360_XACT ---------------------------------------------

// XACT management class
class Asura_Xbox360_XACT
{
public:

	static void Initialise();
	static void Shutdown();	
	static void Update();

	static bool BuildWavDetails							(const char* szFilename, int iFileOffset, Asura_XACT_WaveBank* pxWaveBank);
};

/******************************************************************************/
/* Constants                                                                  */
/******************************************************************************/

// I3DL2 reverb presets. The XDK provides the ones from the I3DL2 spec, but Creative added a load
// more in later EAX revisions. This list provides those, sans the extra EAX parameters

// Room, RoomHF, RRlOff, DecTm, DcHF, Refl, RefDel, Revb, RevDel, Diffus, Density, HFRef

#define I3DL2_ENVIRONMENT_PRESET_DRUGGED \
 -1000,    0, 0.00f,  8.39f, 1.39f,  -115, 0.002f,  985, 0.030f, 50.0f, 100.0f, 5000.0f
#define I3DL2_ENVIRONMENT_PRESET_DIZZY \
 -1000, -400, 0.00f, 17.23f, 0.56f, -1713, 0.020f, -613, 0.030f, 60.0f, 100.0f, 5000.0f
#define I3DL2_ENVIRONMENT_PRESET_PSYCHOTIC \
 -1000, -151, 0.00f,  7.56f, 0.91f,  -626, 0.020f,  774, 0.030f, 50.0f, 100.0f, 5000.0f

// CASTLE PRESETS
#define I3DL2_ENVIRONMENT_PRESET_CASTLE_SMALLROOM \
 -1000,  -800, 0.00f, 1.22f, 0.83f,  -100, 0.022f,  600, 0.011f, 89.0f, 100.0f, 5168.6f
#define I3DL2_ENVIRONMENT_PRESET_CASTLE_SHORTPASSAGE \
 -1000, -1000, 0.00f, 2.32f, 0.83f,  -100, 0.007f,  200, 0.023f, 89.0f, 100.0f, 5168.6f
#define I3DL2_ENVIRONMENT_PRESET_CASTLE_MEDIUMROOM \
 -1000, -1100, 0.00f, 2.04f, 0.83f,  -400, 0.022f,  400, 0.011f, 93.0f, 100.0f, 5168.6f
#define I3DL2_ENVIRONMENT_PRESET_CASTLE_LONGPASSAGE \
 -1000,  -800, 0.00f, 3.42f, 0.83f,  -100, 0.007f,  300, 0.023f, 89.0f, 100.0f, 5168.6f
#define I3DL2_ENVIRONMENT_PRESET_CASTLE_LARGEROOM \
 -1000, -1100, 0.00f, 2.53f, 0.83f,  -700, 0.034f,  200, 0.016f, 82.0f, 100.0f, 5168.6f
#define I3DL2_ENVIRONMENT_PRESET_CASTLE_HALL \
 -1000, -1100, 0.00f, 3.14f, 0.79f, -1500, 0.056f,  100, 0.024f, 81.0f, 100.0f, 5168.6f
#define I3DL2_ENVIRONMENT_PRESET_CASTLE_CUPBOARD \
 -1000, -1100, 0.00f, 0.67f, 0.87f,   300, 0.010f, 1100, 0.007f, 89.0f, 100.0f, 5168.6f
#define I3DL2_ENVIRONMENT_PRESET_CASTLE_COURTYARD \
 -1000,  -700, 0.00f, 2.13f, 0.61f, -1300, 0.160f, -300, 0.036f, 42.0f, 100.0f, 5000.0f
#define I3DL2_ENVIRONMENT_PRESET_CASTLE_ALCOVE \
 -1000,  -600, 0.00f, 1.64f, 0.87f,     0, 0.007f,  300, 0.034f, 89.0f, 100.0f, 5168.6f

// FACTORY PRESETS
#define I3DL2_ENVIRONMENT_PRESET_FACTORY_ALCOVE \
 -1200,  -200, 0.00f, 3.14f, 0.65f,   300, 0.010f,    0, 0.038f, 59.0f, 100.0f, 3762.6f
#define I3DL2_ENVIRONMENT_PRESET_FACTORY_SHORTPASSAGE \
 -1200,  -200, 0.00f, 2.53f, 0.65f,     0, 0.010f,  200, 0.038f, 64.0f, 100.0f, 3762.6f
#define I3DL2_ENVIRONMENT_PRESET_FACTORY_MEDIUMROOM \
 -1200,  -200, 0.00f, 2.76f, 0.65f, -1100, 0.022f,  300, 0.023f, 82.0f, 100.0f, 3762.6f
#define I3DL2_ENVIRONMENT_PRESET_FACTORY_LONGPASSAGE \
 -1200,  -200, 0.00f, 4.06f, 0.65f,     0, 0.020f,  200, 0.037f, 64.0f, 100.0f, 3762.6f
#define I3DL2_ENVIRONMENT_PRESET_FACTORY_LARGEROOM \
 -1200,  -300, 0.00f, 4.24f, 0.51f, -1500, 0.039f,  100, 0.023f, 75.0f, 100.0f, 3762.6f
#define I3DL2_ENVIRONMENT_PRESET_FACTORY_HALL \
 -1000,  -300, 0.00f, 7.43f, 0.51f, -2400, 0.073f, -100, 0.027f, 75.0f, 100.0f, 3762.6f
#define I3DL2_ENVIRONMENT_PRESET_FACTORY_CUPBOARD \
 -1200,  -200, 0.00f, 0.49f, 0.65f,   200, 0.010f,  600, 0.032f, 63.0f, 100.0f, 3762.6f
#define I3DL2_ENVIRONMENT_PRESET_FACTORY_COURTYARD \
 -1000, -1000, 0.00f, 2.32f, 0.29f, -1300, 0.140f, -800, 0.039f, 57.0f, 100.0f, 3762.6f
#define I3DL2_ENVIRONMENT_PRESET_FACTORY_SMALLROOM \
 -1000,  -200, 0.00f, 1.72f, 0.65f,  -300, 0.010f,  500, 0.024f, 82.0f, 100.0f, 3762.6f

// ICE PALACE PRESETS
#define I3DL2_ENVIRONMENT_PRESET_ICEPALACE_ALCOVE \
 -1000,  -500, 0.00f, 2.76f, 1.46f,   100, 0.010f,  -100, 0.030f, 84.0f, 100.0f, 12428.5f
#define I3DL2_ENVIRONMENT_PRESET_ICEPALACE_SHORTPASSAGE \
 -1000,  -500, 0.00f, 1.79f, 1.46f,  -600, 0.010f,   100, 0.019f, 75.0f, 100.0f, 12428.5f
#define I3DL2_ENVIRONMENT_PRESET_ICEPALACE_MEDIUMROOM \
 -1000,  -500, 0.00f, 2.22f, 1.53f,  -800, 0.039f,   100, 0.027f, 87.0f, 100.0f, 12428.5f
#define I3DL2_ENVIRONMENT_PRESET_ICEPALACE_LONGPASSAGE \
 -1000,  -500, 0.00f, 3.01f, 1.46f,  -200, 0.012f,   200, 0.025f, 77.0f, 100.0f, 12428.5f
#define I3DL2_ENVIRONMENT_PRESET_ICEPALACE_LARGEROOM \
 -1000,  -500, 0.00f, 3.14f, 1.53f, -1200, 0.039f,     0, 0.027f, 81.0f, 100.0f, 12428.5f
#define I3DL2_ENVIRONMENT_PRESET_ICEPALACE_HALL \
 -1000,  -700, 0.00f, 5.49f, 1.53f, -1900, 0.054f,  -400, 0.052f, 76.0f, 100.0f, 12428.5f
#define I3DL2_ENVIRONMENT_PRESET_ICEPALACE_CUPBOARD \
 -1000,  -600, 0.00f, 0.76f, 1.53f,   100, 0.012f,   600, 0.016f, 83.0f, 100.0f, 12428.5f
#define I3DL2_ENVIRONMENT_PRESET_ICEPALACE_COURTYARD \
 -1000, -1100, 0.00f, 2.04f, 1.20f, -1000, 0.173f, -1000, 0.043f, 59.0f, 100.0f, 12428.5f
#define I3DL2_ENVIRONMENT_PRESET_ICEPALACE_SMALLROOM \
 -1000,  -500, 0.00f, 1.51f, 1.53f,  -100, 0.010f,   300, 0.011f, 84.0f, 100.0f, 12428.5f

// SPACE STATION PRESETS
#define I3DL2_ENVIRONMENT_PRESET_SPACESTATION_ALCOVE \
 -1000, -300, 0.00f, 1.16f, 0.81f,   300, 0.007f,    0, 0.018f, 78.0f, 100.0f, 3316.1f
#define I3DL2_ENVIRONMENT_PRESET_SPACESTATION_MEDIUMROOM \
 -1000, -400, 0.00f, 3.01f, 0.50f,  -800, 0.034f,  100, 0.035f, 75.0f, 100.0f, 3316.1f
#define I3DL2_ENVIRONMENT_PRESET_SPACESTATION_SHORTPASSAGE \
 -1000, -400, 0.00f, 3.57f, 0.50f,     0, 0.012f,  100, 0.016f, 87.0f, 100.0f, 3316.1f
#define I3DL2_ENVIRONMENT_PRESET_SPACESTATION_LONGPASSAGE \
 -1000, -400, 0.00f, 4.62f, 0.62f,     0, 0.012f,  200, 0.031f, 82.0f, 100.0f, 3316.1f
#define I3DL2_ENVIRONMENT_PRESET_SPACESTATION_LARGEROOM \
 -1000, -400, 0.00f, 3.89f, 0.38f, -1000, 0.056f, -100, 0.035f, 81.0f, 100.0f, 3316.1f
#define I3DL2_ENVIRONMENT_PRESET_SPACESTATION_HALL \
 -1000, -400, 0.00f, 7.11f, 0.38f, -1500, 0.100f, -400, 0.047f, 87.0f, 100.0f, 3316.1f
#define I3DL2_ENVIRONMENT_PRESET_SPACESTATION_CUPBOARD \
 -1000, -300, 0.00f, 0.79f, 0.81f,   300, 0.007f,  500, 0.018f, 56.0f, 100.0f, 3316.1f
#define I3DL2_ENVIRONMENT_PRESET_SPACESTATION_SMALLROOM \
 -1000, -300, 0.00f, 1.72f, 0.82f,  -200, 0.007f,  300, 0.013f, 70.0f, 100.0f, 3316.1f

// WOODEN GALLEON PRESETS
#define I3DL2_ENVIRONMENT_PRESET_WOODEN_ALCOVE \
 -1000, -1800, 0.00f, 1.22f, 0.62f,  100, 0.012f,  -300, 0.024f, 100.0f, 100.0f, 4705.0f
#define I3DL2_ENVIRONMENT_PRESET_WOODEN_SHORTPASSAGE \
 -1000, -1800, 0.00f, 1.75f, 0.50f, -100, 0.012f,  -400, 0.024f, 100.0f, 100.0f, 4705.0f
#define I3DL2_ENVIRONMENT_PRESET_WOODEN_MEDIUMROOM \
 -1000, -2000, 0.00f, 1.47f, 0.42f, -100, 0.049f,  -100, 0.029f, 100.0f, 100.0f, 4705.0f
#define I3DL2_ENVIRONMENT_PRESET_WOODEN_LONGPASSAGE \
 -1000, -2000, 0.00f, 1.99f, 0.40f,    0, 0.020f,  -700, 0.036f, 100.0f, 100.0f, 4705.0f
#define I3DL2_ENVIRONMENT_PRESET_WOODEN_LARGEROOM \
 -1000, -2100, 0.00f, 2.65f, 0.33f, -100, 0.066f,  -200, 0.049f, 100.0f, 100.0f, 4705.0f
#define I3DL2_ENVIRONMENT_PRESET_WOODEN_HALL \
 -1000, -2200, 0.00f, 3.45f, 0.30f, -100, 0.088f,  -200, 0.063f, 100.0f, 100.0f, 4705.0f
#define I3DL2_ENVIRONMENT_PRESET_WOODEN_CUPBOARD \
 -1000, -1700, 0.00f, 0.56f, 0.46f,  100, 0.012f,   100, 0.028f, 100.0f, 100.0f, 4705.0f
#define I3DL2_ENVIRONMENT_PRESET_WOODEN_SMALLROOM \
 -1000, -1900, 0.00f, 0.79f, 0.32f,    0, 0.032f,  -100, 0.029f, 100.0f, 100.0f, 4705.0f
#define I3DL2_ENVIRONMENT_PRESET_WOODEN_COURTYARD \
 -1000, -2200, 0.00f, 1.79f, 0.35f, -500, 0.123f, -2000, 0.032f,  65.0f, 100.0f, 4705.0f

// SPORTS PRESETS
#define I3DL2_ENVIRONMENT_PRESET_SPORT_EMPTYSTADIUM \
 -1000,  -700, 0.00f, 6.26f, 0.51f, -2400, 0.183f,  -800, 0.038f, 100.0f, 100.0f, 5000.0f
#define I3DL2_ENVIRONMENT_PRESET_SPORT_SQUASHCOURT \
 -1000, -1000, 0.00f, 2.22f, 0.91f,  -700, 0.007f,  -200, 0.011f,  75.0f, 100.0f, 7176.9f
#define I3DL2_ENVIRONMENT_PRESET_SPORT_SMALLSWIMMINGPOOL \
 -1000,  -200, 0.00f, 2.76f, 1.25f,  -400, 0.020f,  -200, 0.030f,  70.0f, 100.0f, 5000.0f
#define I3DL2_ENVIRONMENT_PRESET_SPORT_LARGESWIMMINGPOOL \
 -1000,  -200, 0.00f, 5.49f, 1.31f,  -700, 0.039f,  -600, 0.049f,  82.0f, 100.0f, 5000.0f
#define I3DL2_ENVIRONMENT_PRESET_SPORT_GYMNASIUM \
 -1000,  -700, 0.00f, 3.14f, 1.06f,  -800, 0.029f,  -500, 0.045f,  81.0f, 100.0f, 7176.9f
#define I3DL2_ENVIRONMENT_PRESET_SPORT_FULLSTADIUM \
 -1000, -2300, 0.00f, 5.25f, 0.17f, -2000, 0.188f, -1100, 0.038f, 100.0f, 100.0f, 5000.0f
#define I3DL2_ENVIRONMENT_PRESET_SPORT_STADIUMTANNOY \
 -1000,  -500, 0.00f, 2.53f, 0.88f, -1100, 0.230f,  -600, 0.063f,  78.0f, 100.0f, 5000.0f

// PREFAB PRESETS
#define I3DL2_ENVIRONMENT_PRESET_PREFAB_WORKSHOP \
 -1000, -1700, 0.00f, 0.76f, 1.00f,    0, 0.012f,  100, 0.012f, 100.0f, 100.0f, 5000.0f
#define I3DL2_ENVIRONMENT_PRESET_PREFAB_SCHOOLROOM \
 -1000,  -400, 0.00f, 0.98f, 0.45f,  300, 0.017f,  300, 0.015f,  69.0f, 100.0f, 7176.9f
#define I3DL2_ENVIRONMENT_PRESET_PREFAB_PRACTISEROOM \
 -1000,  -800, 0.00f, 1.12f, 0.56f,  200, 0.010f,  300, 0.011f,  87.0f, 100.0f, 7176.9f
#define I3DL2_ENVIRONMENT_PRESET_PREFAB_OUTHOUSE \
 -1000, -1900, 0.00f, 1.38f, 0.38f, -100, 0.024f, -400, 0.044f,  82.0f, 100.0f, 2854.4f
#define I3DL2_ENVIRONMENT_PRESET_PREFAB_CARAVAN \
 -1000, -2100, 0.00f, 0.43f, 1.50f,    0, 0.012f,  600, 0.012f, 100.0f, 100.0f, 5000.0f

// DOME AND PIPE PRESETS
#define I3DL2_ENVIRONMENT_PRESET_DOME_TOMB \
 -1000, -900, 0.00f,  4.18f, 0.21f,  -825, 0.030f,  450, 0.022f,  79.0f, 100.0f, 2854.4f
#define I3DL2_ENVIRONMENT_PRESET_PIPE_SMALL \
 -1000, -900, 0.00f,  5.04f, 0.10f,  -600, 0.032f,  800, 0.015f, 100.0f, 100.0f, 2854.4f
#define I3DL2_ENVIRONMENT_PRESET_DOME_SAINTPAULS \
 -1000, -900, 0.00f, 10.48f, 0.19f, -1500, 0.090f,  200, 0.042f,  87.0f, 100.0f, 2854.4f
#define I3DL2_ENVIRONMENT_PRESET_PIPE_LONGTHIN \
 -1000, -700, 0.00f,  9.21f, 0.18f,  -300, 0.010f, -300, 0.022f,  91.0f, 100.0f, 2854.4f
#define I3DL2_ENVIRONMENT_PRESET_PIPE_LARGE \
 -1000, -900, 0.00f,  8.45f, 0.10f,  -800, 0.046f,  400, 0.032f, 100.0f, 100.0f, 2854.4f
#define I3DL2_ENVIRONMENT_PRESET_PIPE_RESONANT \
 -1000, -700, 0.00f,  6.81f, 0.18f,  -300, 0.010f,    0, 0.022f,  91.0f, 100.0f, 2854.4f

// OUTDOORS PRESETS
#define I3DL2_ENVIRONMENT_PRESET_OUTDOORS_BACKYARD \
 -1000, -1200, 0.00f, 1.12f, 0.34f,  -700, 0.069f,  -300, 0.023f, 45.0f, 100.0f, 4399.1f
#define I3DL2_ENVIRONMENT_PRESET_OUTDOORS_ROLLINGPLAINS \
 -1000, -3900, 0.00f, 2.13f, 0.21f, -1500, 0.300f,  -700, 0.019f,  0.0f, 100.0f, 4399.1f
#define I3DL2_ENVIRONMENT_PRESET_OUTDOORS_DEEPCANYON \
 -1000, -1500, 0.00f, 3.89f, 0.21f, -1000, 0.223f,  -900, 0.019f, 74.0f, 100.0f, 4399.1f
#define I3DL2_ENVIRONMENT_PRESET_OUTDOORS_CREEK \
 -1000, -1500, 0.00f, 2.13f, 0.21f,  -800, 0.115f, -1400, 0.031f, 35.0f, 100.0f, 4399.1f
#define I3DL2_ENVIRONMENT_PRESET_OUTDOORS_VALLEY \
 -1000, -3100, 0.00f, 2.88f, 0.26f, -1700, 0.263f,  -800, 0.100f, 28.0f, 100.0f, 2854.4f

// MOOD PRESETS
#define I3DL2_ENVIRONMENT_PRESET_MOOD_HEAVEN \
 -1000, -200, 0.00f, 5.04f, 1.12f,  -1230, 0.020f, 200, 0.029f, 94.0f, 100.0f, 5000.0f
#define I3DL2_ENVIRONMENT_PRESET_MOOD_HELL \
 -1000, -900, 0.00f, 3.57f, 0.49f, -10000, 0.020f, 300, 0.030f, 57.0f, 100.0f, 5000.0f
#define I3DL2_ENVIRONMENT_PRESET_MOOD_MEMORY \
 -1000, -400, 0.00f, 4.06f, 0.82f,  -2800, 0.000f, 100, 0.000f, 85.0f, 100.0f, 5000.0f

// DRIVING SIMULATION PRESETS
#define I3DL2_ENVIRONMENT_PRESET_DRIVING_COMMENTATOR \
     0,  -500, 0.00f, 2.42f, 0.88f, -1400, 0.093f, -1200, 0.017f,   0.0f, 100.0f,  5000.0f
#define I3DL2_ENVIRONMENT_PRESET_DRIVING_PITGARAGE \
 -1000,  -300, 0.00f, 1.72f, 0.93f,  -500, 0.000f,   200, 0.016f,  59.0f, 100.0f,  5000.0f
#define I3DL2_ENVIRONMENT_PRESET_DRIVING_INCAR_RACER \
 -1000,     0, 0.00f, 0.17f, 2.00f,   500, 0.007f,  -300, 0.015f,  80.0f, 100.0f, 10268.2f
#define I3DL2_ENVIRONMENT_PRESET_DRIVING_INCAR_SPORTS \
 -1000,  -400, 0.00f, 0.17f, 0.75f,     0, 0.010f,  -500, 0.000f,  80.0f, 100.0f, 10268.2f
#define I3DL2_ENVIRONMENT_PRESET_DRIVING_INCAR_LUXURY \
 -1000, -2000, 0.00f, 0.13f, 0.41f,  -200, 0.010f,   400, 0.010f, 100.0f, 100.0f, 10268.2f
#define I3DL2_ENVIRONMENT_PRESET_DRIVING_FULLGRANDSTAND \
 -1000, -1100, 0.00f, 3.01f, 1.37f,  -900, 0.090f, -1500, 0.049f, 100.0f, 100.0f, 10420.2f
#define I3DL2_ENVIRONMENT_PRESET_DRIVING_EMPTYGRANDSTAND \
 -1000,     0, 0.00f, 4.62f, 1.75f, -1363, 0.090f, -1200, 0.049f, 100.0f, 100.0f, 10420.2f
#define I3DL2_ENVIRONMENT_PRESET_DRIVING_TUNNEL \
 -1000,  -800, 0.00f, 3.42f, 0.94f,  -300, 0.051f,  -300, 0.047f,  81.0f, 100.0f,  5000.0f

// CITY PRESETS
#define I3DL2_ENVIRONMENT_PRESET_CITY_STREETS \
 -1000,  -300, 0.00f, 1.79f, 1.12f, -1100, 0.046f, -1400, 0.028f, 78.0f, 100.0f, 5000.0f
#define I3DL2_ENVIRONMENT_PRESET_CITY_SUBWAY \
 -1000,  -300, 0.00f, 3.01f, 1.23f,  -300, 0.046f,   200, 0.028f, 74.0f, 100.0f, 5000.0f
#define I3DL2_ENVIRONMENT_PRESET_CITY_MUSEUM \
 -1000, -1500, 0.00f, 3.28f, 1.40f, -1200, 0.039f,  -100, 0.034f, 82.0f, 100.0f, 2854.4f
#define I3DL2_ENVIRONMENT_PRESET_CITY_LIBRARY \
 -1000, -1100, 0.00f, 2.76f, 0.89f,  -900, 0.029f,  -100, 0.020f, 82.0f, 100.0f, 2854.4f
#define I3DL2_ENVIRONMENT_PRESET_CITY_UNDERPASS \
 -1000,  -700, 0.00f, 3.57f, 1.12f,  -800, 0.059f,  -100, 0.037f, 82.0f, 100.0f, 5000.0f
#define I3DL2_ENVIRONMENT_PRESET_CITY_ABANDONED \
 -1000,  -200, 0.00f, 3.28f, 1.17f,  -700, 0.044f, -1100, 0.024f, 69.0f, 100.0f, 5000.0f

// MISC ROOMS
#define I3DL2_ENVIRONMENT_PRESET_DUSTYROOM \
 -1000, -200, 0.00f, 1.79f, 0.38f, -600, 0.002f,  200, 0.006f, 56.0f, 100.0f, 13046.0f
#define I3DL2_ENVIRONMENT_PRESET_CHAPEL \
 -1000, -500, 0.00f, 4.62f, 0.64f, -700, 0.032f, -200, 0.049f, 84.0f, 100.0f,  5000.0f
#define I3DL2_ENVIRONMENT_PRESET_SMALLWATERROOM \
 -1000, -698, 0.00f, 1.51f, 1.25f, -100, 0.020f,  300, 0.030f, 70.0f, 100.0f,  5000.0f

#endif

#endif	// !defined __ASURA_XBOX360_SOUND_H__