//////////////////////////////////////////////////////////////////////////////
//
// Asura_Xbox360_Sound_LowPassFilter.h
//
// Copyright (c) 2007 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

#if defined ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#if !defined __ASURA_XBOX360_SOUND_LOWPASSFILTER_H__
#define __ASURA_XBOX360_SOUND_LOWPASSFILTER_H__

#ifndef ASURA_USE_AUDIO_SYSTEM

/******************************************************************************/
/* Included Files                                                             */
/******************************************************************************/

#include <xaudio.h>

// Need to disable Asura's 'new' macro so XAUDIO_USE_BATCHALLOC_NEWDELETE works
#include "Core/Asura_MemoryManagement_DebugOff.h"

/******************************************************************************/
/* Classes                                                                    */
/******************************************************************************/

////////////////////////////////////////////////////////////////////////////////
//
// Asura_Xbox360_Sound_LowPassFilter
//
// Implements a low-pass filter as an effect for XAudio
//
////////////////////////////////////////////////////////////////////////////////

class Asura_Xbox360_Sound_LowPassFilter : public IXAudioEffect
{
	// Grr, XAUDIO_USE_BATCHALLOC_NEWDELETE needs ASSERT which isn't actually defined
	#define ASSERT( expr ) Asura_Assert( expr, "" )
	XAUDIO_USE_BATCHALLOC_NEWDELETE();
	#undef ASSERT

public:

	enum PARAM_ID
	{
		PID_FREQUENCY,
		PID_GAIN,
	};

	Asura_Xbox360_Sound_LowPassFilter( void* pContext );

	STDMETHOD_( ULONG, AddRef )( void );
	STDMETHOD_( ULONG, Release )( void );

	STDMETHOD( GetContext )( LPVOID* ppContext );
	STDMETHOD( GetInfo )( XAUDIOFXINFO* pxInfo );
	STDMETHOD( Process )( IXAudioFrameBuffer* pxInputBuffer, IXAudioFrameBuffer* pxOutputBuffer );

	STDMETHOD( GetParam )( XAUDIOFXPARAMID ucParamID, XAUDIOFXPARAMTYPE ucParamType,
						   XAUDIOFXPARAM* pxParam );
	STDMETHOD( SetParam )( XAUDIOFXPARAMID ucParamID, XAUDIOFXPARAMTYPE ucParamType,
						   const XAUDIOFXPARAM* pxParam );

	static HRESULT QuerySize( LPCXAUDIOFXINIT pxInit, LPDWORD puEffectSize );
	static HRESULT Create( LPCXAUDIOFXINIT pxInit, IXAudioBatchAllocator* pxAllocator,
						   IXAudioEffect** ppxEffect );

protected:

	void* m_pContext;
	u_int m_uRefCount;

	// User controls
	float m_fFrequency;
	float m_fGain;

	// Internal gubbins
	float m_fk;
	float m_fv0;
	float m_fh02;
	float m_fa;
	float m_fDelay;
	bool m_bActive;
};

#include "Core/Asura_MemoryManagement_DebugOn.h"

#endif // #ifndef ASURA_USE_AUDIO_SYSTEM

#endif // !defined __ASURA_XBOX360_SOUND_LOWPASSFILTER_H__

//eof