

//////////////////////////////////////////////////////////////////////////////
//
// Asura_Platform_LightShaft_System.cpp
//
// Copyright (c) 2007 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

#ifdef ASURA_TARGET_SM3

// Asura Platform-independent

#include "Core/Asura_Maths.h"
#include "Core/Asura_Timers.h"
#include "Core/Asura_TextureManagement.h"
#include "Camera/Asura_Camera.h"
#include "Rendering/Asura_Render.h"
#include "Rendering/Asura_Viewport.h"
#include "Rendering/Asura_Colour.h"
#include "Rendering/Asura_Viewport.h"
#include "Rendering/Asura_Xbox360_Graphics.h"
#include "Rendering/Asura_Xbox360_Effect_System.h"
#include "SpecialFX/Asura_LightShaft_System.h"
#include "Debugging/Asura_Platform_GPUProfile.h"

// Shader Include Files
#include "LightShaft.h"

class Asura_Xbox360_LightShaft_System
{
public:
	static void Initialise();
	static void Shutdown();
	static void Reset();
	static void RenderStart();
	static void RenderEnd();
	static void RenderLightShaft(Asura_LightShaft *pxLightShaft);

private:

	static Direct3D_VertexDeclaration*			s_pxLightShaftVertexDecl;
	static Direct3D_VertexDeclaration*			s_pxLightShaftCoronaVertexDecl;
	static Asura_Xbox360_Effect*				s_pxLightShaftEffect;

	static Asura_Xbox360_Effect_System::Handle	s_xTextureHandle;
	static Asura_Xbox360_Effect_System::Handle	s_xTexture2Handle;
	static Asura_Xbox360_Effect_System::Handle	s_xDepthTextureHandle;
	static Asura_Xbox360_Effect_System::Handle	s_xPosOrientHandle;
	static Asura_Xbox360_Effect_System::Handle	s_xConeGeometryHandle;
	static Asura_Xbox360_Effect_System::Handle	s_xNearColourHandle;
	static Asura_Xbox360_Effect_System::Handle	s_xFarColourHandle;
	static Asura_Xbox360_Effect_System::Handle	s_xTextureParamsHandle;

	static Asura_Xbox360_VertexBuffer* 			s_apxVertexBuffers[4];
	static int									s_aiNumVertices[4];

};


Direct3D_VertexDeclaration*				Asura_Xbox360_LightShaft_System::s_pxLightShaftVertexDecl			= 0;
Direct3D_VertexDeclaration*				Asura_Xbox360_LightShaft_System::s_pxLightShaftCoronaVertexDecl		= 0;
Asura_Xbox360_Effect*					Asura_Xbox360_LightShaft_System::s_pxLightShaftEffect				= 0;

Asura_Xbox360_Effect_System::Handle		Asura_Xbox360_LightShaft_System::s_xTextureHandle					= 0;
Asura_Xbox360_Effect_System::Handle		Asura_Xbox360_LightShaft_System::s_xTexture2Handle					= 0;
Asura_Xbox360_Effect_System::Handle		Asura_Xbox360_LightShaft_System::s_xDepthTextureHandle				= 0;
Asura_Xbox360_Effect_System::Handle		Asura_Xbox360_LightShaft_System::s_xPosOrientHandle					= 0;
Asura_Xbox360_Effect_System::Handle		Asura_Xbox360_LightShaft_System::s_xConeGeometryHandle				= 0;
Asura_Xbox360_Effect_System::Handle		Asura_Xbox360_LightShaft_System::s_xNearColourHandle				= 0;
Asura_Xbox360_Effect_System::Handle		Asura_Xbox360_LightShaft_System::s_xFarColourHandle					= 0;
Asura_Xbox360_Effect_System::Handle		Asura_Xbox360_LightShaft_System::s_xTextureParamsHandle				= 0;

Asura_Xbox360_VertexBuffer*				Asura_Xbox360_LightShaft_System::s_apxVertexBuffers[4]				= {0,0,0,0};			// 4 LODs
int										Asura_Xbox360_LightShaft_System::s_aiNumVertices[4];


void Asura_LightShaft_System::Platform_Initialise(void)									{ Asura_Xbox360_LightShaft_System::Initialise();	}
void Asura_LightShaft_System::Platform_Shutdown(void)									{ Asura_Xbox360_LightShaft_System::Shutdown();		}
void Asura_LightShaft_System::Platform_Reset(void)										{ Asura_Xbox360_LightShaft_System::Reset();		}
void Asura_LightShaft_System::Platform_RenderStart(void)								{ Asura_Xbox360_LightShaft_System::RenderStart();	}
void Asura_LightShaft_System::Platform_RenderEnd(void)									{ Asura_Xbox360_LightShaft_System::RenderEnd();	}
void Asura_LightShaft_System::Platform_RenderLightShaft(Asura_LightShaft *pxLightShaft) { Asura_Xbox360_LightShaft_System::RenderLightShaft(pxLightShaft); }



void Asura_Xbox360_LightShaft_System::Initialise()
{
	Direct3D_VertexElement auVSD[] =
	{
		{	0,	0,	D3DDECLTYPE_FLOAT3,		D3DDECLMETHOD_DEFAULT,	D3DDECLUSAGE_POSITION,	0 },		
		{	0, 12,	D3DDECLTYPE_FLOAT4,		D3DDECLMETHOD_DEFAULT,	D3DDECLUSAGE_TEXCOORD,	0 },		
		D3DDECL_END()
	};

	Asura_Xbox360_Graphics::GetDevice()->CreateVertexDeclaration( auVSD, &(s_pxLightShaftVertexDecl) );

	Direct3D_VertexElement auVSD2[] =
	{
		{	0,	0,	D3DDECLTYPE_FLOAT3,		D3DDECLMETHOD_DEFAULT,	D3DDECLUSAGE_POSITION,	0 },		
		{	0, 12,	D3DDECLTYPE_FLOAT4,		D3DDECLMETHOD_DEFAULT,	D3DDECLUSAGE_COLOR,		0 },
		{	0, 28,	D3DDECLTYPE_FLOAT4,		D3DDECLMETHOD_DEFAULT,	D3DDECLUSAGE_TEXCOORD,	0 },
		D3DDECL_END()
	};

	Asura_Xbox360_Graphics::GetDevice()->CreateVertexDeclaration( auVSD2, &(s_pxLightShaftCoronaVertexDecl) );


	for (int iLOD = 0; iLOD<4; iLOD++)
	{	
		int iRadialSegments = 32;
		int iLateralSegments = 16;

		switch (iLOD)
		{
		case 1:
			iRadialSegments = 28;
			iLateralSegments = 12;
			break;
		case 2:
			iRadialSegments = 24;
			iLateralSegments = 8;
			break;
		case 3:
			iRadialSegments = 16;
			iLateralSegments = 2;
			break;
		default:
			break;
		}

		int iNumVerts = ((iRadialSegments+2) * 2) * iLateralSegments;

		s_aiNumVertices[iLOD] = iNumVerts;

		s_apxVertexBuffers[iLOD] = new Asura_Xbox360_VertexBuffer();
		s_apxVertexBuffers[iLOD]->CreateStatic(sizeof(Asura_LightShaftVertex), iNumVerts);

		Asura_LightShaftVertex *pxVerts = (Asura_LightShaftVertex*)s_apxVertexBuffers[iLOD]->Lock();		
		Asura_LightShaft_System::CreateConeVertices(iRadialSegments, iLateralSegments, pxVerts, iNumVerts);

		s_apxVertexBuffers[iLOD]->Unlock();
	}

	s_pxLightShaftEffect = Asura_Xbox360_Effect_System::CreateEffectFromMemory(g_auLightshaft);
	Asura_Assert( s_pxLightShaftEffect, "Failed to load corona effect." );

	if (s_pxLightShaftEffect)
	{	
		s_xTextureHandle					= s_pxLightShaftEffect->GetParameterHandle( "g_xTextureSampler" );
		s_xTexture2Handle					= s_pxLightShaftEffect->GetParameterHandle( "g_xTextureSampler2" );
		s_xDepthTextureHandle				= s_pxLightShaftEffect->GetParameterHandle( "g_xDepthTextureSampler" );		
		s_xPosOrientHandle					= s_pxLightShaftEffect->GetParameterHandle( "g_xPosOrient" );		
		s_xConeGeometryHandle				= s_pxLightShaftEffect->GetParameterHandle( "g_xConeGeometry" );
		s_xNearColourHandle					= s_pxLightShaftEffect->GetParameterHandle( "g_xNearColour" );
		s_xFarColourHandle					= s_pxLightShaftEffect->GetParameterHandle( "g_xFarColour" );
		s_xTextureParamsHandle				= s_pxLightShaftEffect->GetParameterHandle( "g_xTextureParams" );
	}
}


void Asura_Xbox360_LightShaft_System::Shutdown()
{

	for(int i=0; i<4; i++)
	{
		s_apxVertexBuffers[i]->Destroy();
		delete s_apxVertexBuffers[i];
	}

	s_pxLightShaftVertexDecl->Release();
	s_pxLightShaftVertexDecl = NULL;

	s_pxLightShaftCoronaVertexDecl->Release();
	s_pxLightShaftCoronaVertexDecl = NULL;


}


void Asura_Xbox360_LightShaft_System::Reset()
{

}


void Asura_Xbox360_LightShaft_System::RenderStart()
{
	ASURA_GPU_PROFILE_START(ASURA_GPUPROFILE_LIGHTSHAFTS);
	PIXBeginNamedEvent( D3DCOLOR_XRGB( 0xA0, 0xFF, 0x20 ), "LightShafts" );

	// Setup initial renderstates
	Asura_Render::CurrentStates.RequestCullMode(ASURA_CULLMODE_CCW);
	Asura_Render::CurrentStates.RequestZBufferEnabled(true);
	Asura_Render::CurrentStates.RequestZBufferWriteEnabled(false);	
	Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_GLOWING);
	Asura_Render::CurrentStates.RequestAlphaTest(false);

	// (AAM - 26.07.2007) Need to use high-precision blending in case HDR is on
	Asura_Xbox360_Graphics::GetDevice()->SetRenderState_Inline( D3DRS_HIGHPRECISIONBLENDENABLE, TRUE );

	// Constant shader params
	s_pxLightShaftEffect->GetInterface()->SetSampler(s_xDepthTextureHandle, Asura_Xbox360_Graphics::GetDepthMtlTexture());	

	s_pxLightShaftEffect->GetInterface()->BeginTechniqueFromIndex( 0, 0 );
}


void Asura_Xbox360_LightShaft_System::RenderEnd()
{
	Asura_Xbox360_Graphics::GetDevice()->SetRenderState_Inline( D3DRS_HIGHPRECISIONBLENDENABLE, FALSE );

	s_pxLightShaftEffect->GetInterface()->SetSampler(s_xDepthTextureHandle, NULL);	
	s_pxLightShaftEffect->GetInterface()->EndTechnique();

	ASURA_GPU_PROFILE_END(ASURA_GPUPROFILE_LIGHTSHAFTS);
	PIXEndNamedEvent();
}


void Asura_Xbox360_LightShaft_System::RenderLightShaft(Asura_LightShaft *pxLightShaft)
{

	//  (warning - turning this on on 360 buggers up all the renderstates....)
	//	Asura_Matrix_3x3 xTmp;
	//	xTmp.Identity();
	//	Asura_Render::RenderBB(pxLightShaft->GetBoundingBox(),Asura_Vector_3(0,0,0),xTmp);
	const Asura_Vector_3&	xPos	= pxLightShaft->GetPosition();
	Asura_Vector_3			xOfs	= *Asura_Camera::GetPosition() - pxLightShaft->GetPosition();
	float fDist = xOfs.Magnitude();

	float fDistanceFade = 1.0f;
	if (fDist < pxLightShaft->GetCloseFadeStart())
	{
		fDistanceFade = 1.0f - ((pxLightShaft->GetCloseFadeStart() - fDist)/pxLightShaft->GetCloseFadeDist());
	}
	else if (fDist > pxLightShaft->GetDistanceFadeStart())
	{
		fDistanceFade = 1.0f - ((fDist - pxLightShaft->GetDistanceFadeStart())/pxLightShaft->GetDistanceFadeDist());
	}
	if (fDistanceFade<=0.0f)
	{
		return;
	}

	SCOPEDPIXEVENT((D3DCOLOR_XRGB(0xA0, 0xFF, 0x20), "LightShaft @ (%.1f, %.1f, %.1f)", xPos.GetX(), xPos.GetY(), xPos.GetZ()));

	D3DXMATRIX xPosOrient;
	Asura_Vector_3 xDir = Asura_Vector_3(0,0,1)*pxLightShaft->GetOrient();

	// Rotation	
	memset( &xPosOrient, 0, sizeof( xPosOrient ) );
	for( u_int u = 0; u < 3; ++u )
	{ 
		for( u_int v = 0; v < 3; ++v )
		{
			xPosOrient.m[u][v] = pxLightShaft->GetOrient().Element[v][u];
		}
	}
	xPosOrient._44 = 1.0f;

	// Translation
	D3DXMATRIX xTemp;
	D3DXMatrixTranslation( &xTemp, xPos.x, xPos.y, xPos.z );
	xPosOrient *= xTemp;
	s_pxLightShaftEffect->GetInterface()->SetMatrixF(s_xPosOrientHandle, xPosOrient);

	// Near/Far colours
	Asura_RGBA xInheritedColour = pxLightShaft->GetInheritedColour();


	bool bShaftVisible = true;

	if  ((pxLightShaft->GetNearRadius() + pxLightShaft->GetFarRadius() <=0.0f) ||
		(pxLightShaft->GetNearDistance()) == pxLightShaft->GetFarDistance() ||
		((pxLightShaft->GetNearColour().ToUIntARGB()&0xffffff) == 0  && (pxLightShaft->GetFarColour().ToUIntARGB()&0xffffff) == 0))
	{
		//	bShaftVisible = false;
	}

	if (bShaftVisible)
	{
		// Cone geometry
		FXLVECTOR xConeParams;
		xConeParams.x = pxLightShaft->GetNearRadius();
		xConeParams.y = pxLightShaft->GetFarRadius();		
		xConeParams.z = pxLightShaft->GetNearDistance();
		xConeParams.w = pxLightShaft->GetFarDistance();
		s_pxLightShaftEffect->SetVector(s_xConeGeometryHandle, xConeParams);


		FXLVECTOR xColour;	
		Asura_RGBA  xRGBA = pxLightShaft->GetNearColour();
		xColour.x = xRGBA.m_fR*fDistanceFade*xInheritedColour.m_fR;
		xColour.y = xRGBA.m_fG*fDistanceFade*xInheritedColour.m_fG;
		xColour.z = xRGBA.m_fB*fDistanceFade*xInheritedColour.m_fB;
		xColour.w = xRGBA.m_fA*fDistanceFade*xInheritedColour.m_fA;
		s_pxLightShaftEffect->SetVector(s_xNearColourHandle, xColour);

		xRGBA = pxLightShaft->GetFarColour();
		xColour.x = xRGBA.m_fR*fDistanceFade*xInheritedColour.m_fR;
		xColour.y = xRGBA.m_fG*fDistanceFade*xInheritedColour.m_fG;
		xColour.z = xRGBA.m_fB*fDistanceFade*xInheritedColour.m_fB;
		xColour.w = xRGBA.m_fA*fDistanceFade*xInheritedColour.m_fA;
		s_pxLightShaftEffect->SetVector(s_xFarColourHandle, xColour);

		// Texture params
		FXLVECTOR xTextureParams;
		xTextureParams.x = pxLightShaft->GetUScale(0);
		xTextureParams.y = pxLightShaft->GetUScale(1);
		xTextureParams.z = Asura_Maths::FloatMod(pxLightShaft->GetUScrollRate(0) * Asura_Timers::GetLocalScaledTime(), 1.0f);
		xTextureParams.w = Asura_Maths::FloatMod(pxLightShaft->GetUScrollRate(1) * Asura_Timers::GetLocalScaledTime(), 1.0f);
		s_pxLightShaftEffect->SetVector(s_xTextureParamsHandle, xTextureParams);	

		// Textures	
		s_pxLightShaftEffect->SetTexture(s_xTextureHandle,  pxLightShaft->GetTextureIndex(0));
		s_pxLightShaftEffect->SetTexture(s_xTexture2Handle, pxLightShaft->GetTextureIndex(1));


		// Draw the shaft
		// --------------------------------------------------
		Asura_Xbox360_Graphics::GetDevice()->SetVertexDeclaration(s_pxLightShaftVertexDecl);	



		s_pxLightShaftEffect->GetInterface()->BeginPassFromIndex(0);
		s_pxLightShaftEffect->Commit();

		// Pick a LOD
		float fSize = Asura_Maths::Max(pxLightShaft->GetFarDistance(), 2.0f*pxLightShaft->GetFarRadius());
		int iLOD = (int)(fDist*0.25f/fSize);
		iLOD = Asura_Maths::ClampToRange(iLOD,0,3);

		s_apxVertexBuffers[iLOD]->Bind(0);

		for (int iTiling = 0; iTiling < Asura_Xbox360_Graphics::GetNumberOfPredicatedTilePasses(); ++iTiling)
		{
			if (Asura_Xbox360_Graphics::IsTiling2())
			{
				Asura_Xbox360_Graphics::GetDevice()->SetPredication(D3DPRED_TILE(iTiling));
				Asura_Xbox360_Graphics::CorrectVPOSForTiling(iTiling);
			}
			s_pxLightShaftEffect->GetInterface()->Commit();
			Asura_Xbox360_Graphics::GetDevice()->DrawPrimitive(D3DPT_TRIANGLESTRIP, 0, s_aiNumVertices[iLOD]-2);
		}

		s_pxLightShaftEffect->GetInterface()->EndPass();
	}


	// Now draw the corona
	// --------------------------------------------------
	int iTex = pxLightShaft->GetCoronaTextureIndex();

	if (iTex>=0)
	{	
		Asura_LightShaftCoronaVertex xVerts[4];

		float fSize;
		float fForwards;

		Asura_Vector_3 xFwd;
		xOfs.Normalise();
		float fDot = xDir*xOfs;

		float fAngleRange = pxLightShaft->GetCoronaAngleRange();
		float fAmount = fDistanceFade * ((fDot - (1.0f-fAngleRange))/fAngleRange);

		// Special feature to use these coronas for fog planes... set angle range to max,
		// and it's 'always on' with no fade up/down
		if (fAngleRange>=0.99f)
		{
			fAmount = 1.0f;
		}

		if (fAmount>0)
		{		
			Asura_Xbox360_Graphics::GetDevice()->SetVertexDeclaration(s_pxLightShaftCoronaVertexDecl);

			s_pxLightShaftEffect->SetTexture(s_xTextureHandle, iTex);

			fSize = pxLightShaft->GetCoronaSize();
			fForwards = pxLightShaft->GetCoronaDepth();

			fSize *= (0.666f * (0.5f+fAmount));
			fForwards *= fAmount;

			Asura_RGBA xCol = pxLightShaft->GetCoronaColour();
			xCol.m_fR *= xInheritedColour.m_fR;
			xCol.m_fG *= xInheritedColour.m_fG;
			xCol.m_fB *= xInheritedColour.m_fB;
			xCol.m_fA = fAmount;

			for(int i=0;i<4;i++)
			{
				const float afLeft[4] = {-1,-1,1,1};
				const float afUp[4]   = {-1,1,1,-1};
				const float afU[4]    = { 0,0,1,1};
				const float afV[4]    = { 0,1,1,0};

				xVerts[i].xPos.x = afLeft[i] * fSize;
				xVerts[i].xPos.y = afUp[i] * fSize;
				xVerts[i].xPos.z = fForwards;
				xVerts[i].fU	 = afU[i];
				xVerts[i].fV	 = afV[i];
				xVerts[i].fDepth = fForwards;
				xVerts[i].fR	 = xCol.m_fR;
				xVerts[i].fG	 = xCol.m_fG;
				xVerts[i].fB	 = xCol.m_fB;
				xVerts[i].fA	 = xCol.m_fA;
			}


			for (int iTiling = 0; iTiling < Asura_Xbox360_Graphics::GetNumberOfPredicatedTilePasses(); ++iTiling)
			{
				if (Asura_Xbox360_Graphics::IsTiling2())
				{
					Asura_Xbox360_Graphics::GetDevice()->SetPredication(D3DPRED_TILE(iTiling));
					Asura_Xbox360_Graphics::CorrectVPOSForTiling(iTiling);
				}

				s_pxLightShaftEffect->GetInterface()->BeginPassFromIndex(1);
				s_pxLightShaftEffect->Commit();
				Asura_Xbox360_Graphics::GetDevice()->DrawPrimitiveUP(D3DPT_TRIANGLEFAN, 2, (void *)&xVerts[0], sizeof(Asura_LightShaftCoronaVertex));

				s_pxLightShaftEffect->GetInterface()->EndPass();
			}

		}
	}
	if (Asura_Xbox360_Graphics::IsTiling2())
	{
		Asura_Xbox360_Graphics::GetDevice()->SetPredication(0);
		Asura_Xbox360_Graphics::CorrectVPOSForTiling(0);
	}
}



#endif // ASURA_TARGET_SM3