
//
// Asura_Platform_PFX_FastPath.h
//
// Created by DMR 15/11/06
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
// Faster renderer for most PFX types, using hardware instancing
//

#ifndef __ASURA_PFX_SYSTEM_FASTPATH_H__
#define __ASURA_PFX_SYSTEM_FASTPATH_H__

 #ifdef ASURA_USE_FASTPATH_PFX

// Asura Xbox 360
#include "Rendering/Asura_Xbox360_Effect_System.h"
#include "Rendering/Asura_Xbox360_Graphics.h"

// Forward declares
class Asura_PFX_Effect;
class Asura_PFX_Particle;
class Asura_PFX_Particle_Template;
class Asura_RGBA;
class Asura_Xbox360_VertexBuffer;


struct Asura_Platform_PFXInstanceData
{
	float X, Y, Z, Orientation;					
	float UVSet0[4];							
	float UVSet1[4];							
	float Width, Height;						
	float TextureAlignment, CrossFadeBlendWeight;
	float Diffuse[4];					
};


class Asura_Platform_PFX_System_FastPath
{
public:

	static void Initialise();
	static void Shutdown();

	static bool IsEnabled(void) {return s_bIsEnabled;}
	static void SetEnabled(bool bEnabled) {s_bIsEnabled = bEnabled;}

	static void UpdateInstanceBuffersForNewFrame();
	static void *GetInstanceBufferArea(uint32 uNumParticles);

	static uint32 GetNumFastPathParticlesRenderedThisFrame() {return s_uFastPathParticlesRenderedThisFrame;}
	static bool   WasVertexBufferFull()	{return s_bVertexBufferOverflow;}
	
	static void RenderParticles(				
		const  Asura_PFX_Effect *pxEffect,
		const  Asura_PFX_Particle* pxParticles,
		u_int  uNumParticles,
		const  Asura_PFX_Particle_Template* pxTemplate,
		const  Asura_RGBA& xLightColour,
		const  uint16 usCameraFlags, 
		const  uint16 usPFXFlags,														 
		void* pPlatformRenderData);


	static void Asura_Platform_PFX_System_FastPath::UpdateInstanceBuffer(const  Asura_PFX_Particle_Template *pxTemplate, 
																		 const  Asura_PFX_Particle *pxParticles,
															  			 Asura_Platform_PFXInstanceData *pxParticleInstanceData, 
																		 uint32 uNumParticles);

private:
	static bool									s_bIsEnabled;
	static Direct3D_VertexDeclaration*		    s_pxPFXVertexDecl;
	static Asura_Xbox360_Effect*			    s_pxPFXEffect;

	static Asura_Xbox360_Effect_System::Handle  s_xTextureHandle;
	static Asura_Xbox360_Effect_System::Handle  s_xDepthTextureHandle;
	static Asura_Xbox360_Effect_System::Handle  s_xFirstInstanceHandle;
	static Asura_Xbox360_Effect_System::Handle  s_xZReadEnableHandle;
	static Asura_Xbox360_Effect_System::Handle  s_xXDeltaHandle;
	static Asura_Xbox360_Effect_System::Handle  s_xYDeltaHandle;		
	static Asura_Xbox360_Effect_System::Handle  s_xLightColourHandle;
	static Asura_Xbox360_Effect_System::Handle  s_xVolumetricDepthScaleHandle;
	static Asura_Xbox360_Effect_System::Handle  s_xPosOffsetAndSizeScaleHandle;
	static Asura_Xbox360_Effect_System::Handle  s_xNearFarFadeHandle;
	static Asura_Xbox360_Effect_System::Handle  s_xTwoOverScreenWidthHandle;
	static Asura_Xbox360_Effect_System::Handle  s_xTwoOverScreenHeightHandle;


	static int									s_iCurrentVertexBuffer;
	static bool									s_bVertexBufferLocked;
	static bool									s_bVertexBufferOverflow;
	static int						 			s_iCurrentVertexBufferPosition;
	static Asura_Platform_PFXInstanceData	   *s_pxCurrentVertexBuffer;

	static uint32								s_uFastPathParticlesRenderedThisFrame;
	

	enum 
	{
#ifdef ASURA_FASTPATH_PFX_VERTEXBUFFER_SIZE		    
		VERTEXBUFFER_SIZE   = ASURA_FASTPATH_PFX_VERTEXBUFFER_SIZE
#else
		VERTEXBUFFER_SIZE   = 7500
#endif
	};

	enum
	{
#ifdef ASURA_FASTPATH_PFX_VERTEXBUFFER_COUNT		
		VERTEXBUFFER_COUNT  = ASURA_FASTPATH_PFX_VERTEXBUFFER_COUNT
#else
		VERTEXBUFFER_COUNT  = 2
#endif		
	};

	static Asura_Xbox360_VertexBuffer			s_axVertexBuffers[VERTEXBUFFER_COUNT];

	
};

#endif 

#endif // __ASURA_PFX_SYSTEM_FASTPATH_H__