// DWBS ------------------------------------------------------------------ DWBS
//
// PLATFORM		: ALL
// PRODUCT		: BIT_DEMON
// VISIBILITY	: PUBLIC
//
// DWBS ------------------------------------------------------------------ DWBS

// PURPOSE: A fixed size data buffer.

#ifndef BD_BYTE_BUFFER_H
#define BD_BYTE_BUFFER_H

#include <bdCore/bdMemory/bdMemory.h>
#include <bdCore/bdReference/bdReferencable.h>
#include <bdCore/bdReference/bdReference.h>
#include <bdCore/bdContainers/bdBitBuffer.h>

BD_REFERENCE(bdByteBuffer);

class bdByteBuffer : public bdReferencable
{
	public:

		BD_DECLARE_NEW_AND_DELETE_OPERATORS

		/// Specific size constructor.
		/// \param size[in]	The amount of memory to allocate to 
		///					this buffer.
		inline bdByteBuffer(const bdUInt size);

		/// Destructor.
		virtual ~bdByteBuffer();

		inline bdUByte8& operator[] (const bdUInt i);

		inline bdUByte8 operator[] (const bdUInt i) const;

		inline bdUInt getSize() const;

		inline bdUByte8* getData();

		inline const bdUByte8* getData() const;

		inline bdUInt getMaxWriteSize() const;

		inline bdUInt getMaxReadSize() const;

		template <typename U>
		bdBool write(const U &var)
		{
			U evar;
			bdBitBuffer::endianSwap(var, evar);
			return write(&evar, sizeof(evar));
		}

		bdBool write(const void *data, bdUInt size);
		
		template <typename U>
		bdBool read(U &var)
		{
			const bdBool result = read(&var, sizeof(var));
			U temp;
			bdBitBuffer::endianSwap(var, temp);
			return result;
		}

		bdBool read(void *data, bdUInt size);
		
	protected:

		bdUInt m_size;
		bdUByte8 *m_data;
		bdUByte8 *m_readPtr;
		bdUByte8 *m_writePtr;

};

#include <bdCore/bdContainers/bdByteBuffer.inl>

#endif // BD_BYTE_BUFFER_H
