// DWBS ------------------------------------------------------------------ DWBS
//
// PLATFORM		: ALL
// PRODUCT		: BIT_DEMON
// VISIBILITY	: PUBLIC
//
// DWBS ------------------------------------------------------------------ DWBS

#include <bdCore/bdLog/bdLogChannel.h>
#include <bdCore/bdLog/bdLogSubscriber.h>

bdLogChannel::bdLogChannel(const bdNChar8 *const channelName)
: m_channelName(channelName)
{
}

bdLogChannel::~bdLogChannel()
{
	BD_LOCK_COMPONENTS
	bdHashMap<bdString, bdLogChannel *>::Iterator it = m_components.getIterator();

	while(it)
	{
		bdLogChannel *channel = m_components.getValue(it);
		delete channel;
		m_components.next(it);
	}

	m_components.releaseIterator(it);
	BD_UNLOCK_COMPONENTS
}

void bdLogChannel::log(const bdNChar8 *const file, 
					   const bdNChar8 *const function, 
					   const bdUInt line, 
					   const bdNChar8 *const channelName, 
					   const bdNChar8 *const msg) 
{
	for(bdUInt i = 0; i < m_subscribers.getSize(); i++)
	{
		m_subscribers[i]->publish(m_channelName.getBuffer(), file, function, line, msg);
	}

	const bdNChar8 *pos = bdStrchr( channelName, '/' );
	const bdUWord offset = pos ? 1 : 0;
	bdUWord len = pos ? pos - channelName : bdStrlen( channelName );

	if(len > 0)
	{
		bdLogChannel *channel;

		bdNChar8 str[100];
		bdMemcpy(str, channelName, len);
		str[len] = 0;
		const bdString cname(str);

		BD_LOCK_COMPONENTS
		bdHashMap<bdString, bdLogChannel *>::Iterator it = m_components.getIterator(cname);

		if(it)
		{
			channel = m_components.getValue(it);

			channel->log(file, function, line, channelName + len + offset, msg);
		}

		m_components.releaseIterator(it);
		BD_UNLOCK_COMPONENTS
	}
}

void bdLogChannel::addSubscriber(const bdNChar8 *const fullChannelName,
								 const bdNChar8 *const channelName, 
								 bdLogSubscriber *const subscriber)
{
	const bdNChar8 *pos = bdStrchr( channelName, '/' );
	const bdUWord offset = pos ? 1 : 0;
	bdUWord len = pos ? pos - channelName : bdStrlen( channelName );

	if(len > 0)
	{
		bdLogChannel *channel;

		bdNChar8 str[100];
		bdMemcpy(str, channelName, len);
		str[len] = 0;
		const bdString cname(str);

		BD_LOCK_COMPONENTS
		bdHashMap<bdString, bdLogChannel *>::Iterator it = m_components.getIterator(cname);

		if(it)
		{
			channel = m_components.getValue(it);
		}
		else
		{
			channel = new bdLogChannel(fullChannelName);
			m_components.put(cname, channel);
		}

		m_components.releaseIterator(it);
		BD_UNLOCK_COMPONENTS

		channel->addSubscriber(fullChannelName, channelName + len + offset, subscriber);
	}
	else
	{
		m_subscribers.pushBack(subscriber);
	}
}


void bdLogChannel::removeSubscriber(const bdNChar8 *const fullChannelName, 
									const bdNChar8 *const channelName, 
									bdLogSubscriber *const subscriber)
{
	const bdNChar8 *pos = bdStrchr( channelName, '/' );
	const bdUWord offset = pos ? 1 : 0;
	bdUWord len = pos ? pos - channelName : bdStrlen( channelName );

	if(len > 0)
	{
		bdNChar8 str[100];
		bdMemcpy(str, channelName, len);
		str[len] = 0;
		const bdString cname(str);

		BD_LOCK_COMPONENTS
		bdHashMap<bdString, bdLogChannel *>::Iterator it = m_components.getIterator(cname);

		if(it)
		{
			bdLogChannel *const channel = m_components.getValue(it);			

			channel->removeSubscriber(fullChannelName, channelName + len + offset, subscriber);

			if(channel->m_subscribers.isEmpty())
			{
				BD_ASSERT(m_components.remove(it), "recently retrieved map element now missing");
				delete channel;
			}

			m_components.releaseIterator(it);
		}
		BD_UNLOCK_COMPONENTS

	}
	else
	{
		m_subscribers.removeAll(subscriber);		
	}

}

void bdLogChannel::removeSubscriber(bdLogSubscriber *const subscriber)
{
	m_subscribers.removeAll(subscriber);

	BD_LOCK_COMPONENTS
	bdHashMap<bdString, bdLogChannel *>::Iterator it = m_components.getIterator();

	while(it)
	{
		bdLogChannel *const channel = m_components.getValue(it);			

		channel->removeSubscriber(subscriber);

		if(channel->m_subscribers.isEmpty())
		{
			BD_ASSERT(m_components.remove(it), "recently retrieved map element now missing");
			delete channel;
		}
		else
		{
			m_components.next(it);
		}
	}
	BD_UNLOCK_COMPONENTS
}

