// DWBS ------------------------------------------------------------------ DWBS
//
// PLATFORM		: ALL
// PRODUCT		: BIT_DEMON
// VISIBILITY	: PUBLIC
//
// DWBS ------------------------------------------------------------------ DWBS

#include <bdPlatform/bdPlatformMemory/bdPlatformMemory.h>
#include <bdPlatform/bdPlatformString/bdPlatformString.h>
#include <bdPlatform/bdPlatformError/bdPlatformError.h>

#include <bdCore/bdStreams/bdIStream.h>
#include <bdCore/bdLog/bdLog.h>

#define IS_SPACE(c)			( ((c) == ' ' ) || ((c) == '\t') )
#define IS_EOL(c)			( ((c) == '\r') || ((c) == '\n') )
#define IS_INTEGER(c)		( ((c) >= '0' ) && ((c) <= '9')  )
#define IS_SIGN(c)			( ((c) == '+' ) || ((c) == '-')  )
#define IS_ZERO(c)			  ((c) == '0' ) 

#define IS_EXP(c)			( ((c) == 'E' ) || ((c) == 'e')  )
#define IS_DECIMALPOINT(c)	( ((c) == '.' ) || ((c) == ',')  )

#define IS_WHITESPACE(c)	( (IS_SPACE(c)) || (IS_EOL(c)) )

bdIStream::bdIStream(bdStreambuf* streambuf)
: m_streambuf(streambuf),
  m_flags(0)
{
}

bdBool bdIStream::isOk() const
{
	bdBool ok = false;
	if(m_streambuf)
	{
		ok = m_streambuf->isOkToRead();	
	}

	return (ok && (m_flags == 0));
}

bdIStream& bdIStream::operator>> (bdNChar8& c)
{
	if (isOk())
	{
		m_streambuf->getFrom(reinterpret_cast<bdNChar8*>(&c), 1);		
	}
	return *this;
}

bdIStream& bdIStream::operator>> (bdInt16& i)
{
	if (isOk())
	{
		bdNChar8 buf[BD_MAX_INT16_DIGITS];
		getIntegerString(buf, BD_MAX_INT16_DIGITS);
		bdNChar8* end = BD_NULL;
		bdInt value = bdStrtoi32(buf, &end, 10);

		// errno is from #include <errno.h> and set by ::strtol
		if ( (bdGetErrno() != 0) || (end == buf) )
		{
			// error
			m_flags = m_flags | ERROR_BIT;
		}
	
		if ( (value > BD_INT16_MAX) || (value < BD_INT16_MIN) )
		{
			// error
			m_flags = m_flags | INPUT_VALUE_OUT_OF_RANGE;
		}
		else
		{
			i = static_cast<bdInt16>(value);
		}
	}

	return *this;
}

bdIStream& bdIStream::operator>> (bdUInt16& u)
{
	if (isOk())
	{
		bdNChar8 buf[BD_MAX_INT16_DIGITS];
		getIntegerString(buf, BD_MAX_INT16_DIGITS);
		bdNChar8* end = BD_NULL;
		bdUInt value = bdStrtoui32(buf, &end, 10);
		
		// errno is from #include <errno.h> and set by ::strtoul
		if ( (bdGetErrno() != 0) || (end == buf) )
		{
			// error
			m_flags = m_flags | ERROR_BIT;
		}

		if (value > BD_UINT16_MAX)
		{
			// error
			m_flags = m_flags | INPUT_VALUE_OUT_OF_RANGE;
		}
		else
		{
			u = static_cast<bdUInt16>(value);
		}
	}

	return *this;
}

bdIStream& bdIStream::operator>> (bdInt& i)
{
	if (isOk())
	{
		bdNChar8 buf[BD_MAX_INT32_DIGITS];
		getIntegerString(buf, BD_MAX_INT32_DIGITS);
		bdNChar8* end = BD_NULL;
		bdInt value = bdStrtoi32(buf, &end, 10);

		// errno is from #include <errno.h> and set by ::strtol
		if ( (bdGetErrno() != 0) || (end == buf) )
		{
			// error
			m_flags = m_flags | ERROR_BIT;
		}
		else
		{
			i = value;
		}		
	}

	return *this;
}

bdIStream& bdIStream::operator>> (bdUInt& u)
{
	if (isOk())
	{
		bdNChar8 buf[BD_MAX_INT32_DIGITS];
		getIntegerString(buf, BD_MAX_INT32_DIGITS);
		bdNChar8* end = BD_NULL;
		bdUInt value = bdStrtoui32(buf, &end, 10);

		// errno is from #include <errno.h> and set by ::strtoul
		if ( (bdGetErrno() != 0) || (end == buf) )
		{
			// error
			m_flags = m_flags | ERROR_BIT;
		}
		else
		{
			u = value;
		}		
	}

	return *this;
}

bdIStream& bdIStream::operator>> (bdInt64& i)
{
	if (isOk())
	{
		bdNChar8 buf[BD_MAX_INT64_DIGITS];
		getIntegerString(buf, BD_MAX_INT64_DIGITS);
		bdNChar8* end = BD_NULL;
		bdInt64 value = bdStrtoi64(buf, &end, 10);
	
		if ( (bdGetErrno() != 0) || (end == buf) )
		{
			// error
			m_flags = m_flags | ERROR_BIT;
		}
		else
		{
			i = value;
		}		
	}

	return *this;
}

bdIStream& bdIStream::operator>> (bdUInt64& u)
{
	if (isOk())
	{
		bdNChar8 buf[BD_MAX_INT64_DIGITS];
		getIntegerString(buf, BD_MAX_INT64_DIGITS);
		bdNChar8* end = BD_NULL;
		bdUInt64 value = bdStrtoui64(buf, &end, 10);

		if ( (bdGetErrno() != 0) || (end == buf) )
		{
			// error
			m_flags = m_flags | ERROR_BIT;
		}
		else
		{
			u = value;
		}		
	}

	return *this;
}

bdIStream& bdIStream::operator>> (bdFloat32& f)
{
	if(isOk())
	{
		bdNChar8 buf[BD_MAX_FLOAT32_DIGITS];
		getFloatString(buf, BD_MAX_FLOAT32_DIGITS);
		bdNChar8* end = BD_NULL;
		bdFloat32 value = bdStrtof32(buf, &end);

		if ( (bdGetErrno() != 0) || (end == buf) )
		{
			// error
			m_flags = m_flags | ERROR_BIT;
		}
		else
		{
			f = value;
		}
	}

	return *this;
}

bdIStream& bdIStream::operator>> (bdFloat64& f)
{
	if(isOk())
	{
		bdNChar8 buf[BD_MAX_FLOAT64_DIGITS];
		getFloatString(buf, BD_MAX_FLOAT64_DIGITS);
		bdNChar8* end = BD_NULL;
		bdFloat64 value = bdStrtof64(buf, &end);

		if ( (bdGetErrno() != 0) || (end == buf) )
		{
			// error
			m_flags = m_flags | ERROR_BIT;
		}
		else
		{
			f = value;
		}
	}

	return *this;
}

bdIStream& bdIStream::operator>> (bdNChar8 *s)
{
	if(isOk())
	{
		bdNChar8 c = '\0';
		skipWhiteSpace(c);

		while ( !(IS_WHITESPACE(c) || (c == '\0')) )
		{
			*s = c;
			s++;
			c = '\0';
			m_streambuf->getFrom(&c, 1);
		}

		*s = '\0';
	}

	return *this;
}
//
//bdIStream& bdIStream::operator>> (bdUChar8 *const s)
//{
//	return (*this) >> reinterpret_cast<bdChar8*>(s);
//}

bdIStream& bdIStream::operator>> (bdString& s)
{
	if(isOk())
	{
		bdFastArray<bdNChar8> array(32);
		bdNChar8 c = '\0';
		skipWhiteSpace(c);

		while ( !(IS_WHITESPACE(c) || (c == '\0')) )
		{
			array.pushBack(c);
			c = '\0';
			m_streambuf->getFrom(&c, 1);
		}

		array.pushBack('\0');
		s = array.begin();
	}

	return *this;
}

bdIStream& bdIStream::operator>> (bdBool& b)
{
	if(isOk())
	{
		bdNChar8 c = '\0';
		skipWhiteSpace(c);
		m_streambuf->getFrom(&c, 1);
		if (c == '0')
		{
			b = false;
		}
		else if (c == '1')
		{
			b = true;
		}
	}

	return *this;
}

bdIStream& bdIStream::getLine(bdNChar8 *const buf,
							  const bdUInt bufSize,
							  const bdNChar8 *delimiters)
{
	if(bufSize > 0)
	{
		const bdUInt maxLength = bufSize-1;
		bdUInt numWritten = 0;
		
		if (!delimiters)
		{
			delimiters = "\n";
		}		

		bdNChar8 c = '\0';
		while ((numWritten < maxLength) && isOk() && m_streambuf->getFrom(&c, 1))
		{
			bdUInt i = 0;
			bdBool foundDelimiter = false;
			while (*(delimiters+i) != '\0')
			{
				if(c == *(delimiters+i))
				{
					foundDelimiter = true;
					break;
				}
				i++;
			}
			
			if(foundDelimiter)
			{
				break;
			}
			else
			{
				buf[numWritten] = c;
				numWritten++;
			}
		}

		buf[numWritten] = '\0';
	}

	return *this;
}

bdUInt bdIStream::read(bdNChar8 *const buf, 
						 const bdUInt nbytes)
			
{
	if(isOk())
	{
		return m_streambuf->getFrom(buf, nbytes);
	}

	return 0;
}
	
bdBool bdIStream::seek(const bdUInt offset,
					   const bdStreambuf::SeekFrom from)
{
	if(isOk())
	{
		return m_streambuf->seek(offset, from);
	}

	return false;
}

bdBool bdIStream::tell(bdUInt &offset) const
{
	if(isOk())
	{
		return m_streambuf->tell(offset);
	}

	return false;
}

bdStreambuf* bdIStream::getStreambuf()
{
	return m_streambuf;
}



void bdIStream::skipWhiteSpace(bdNChar8 &c)
{	
	// get the next character
	while (m_streambuf->getFrom(&c, 1) != 0)
	{
		if (!IS_WHITESPACE(c))
		{
			break;
		}
	}
}

bdUInt bdIStream::getIntegerString(bdNChar8* s,
									 const bdUInt maxlen)
{
	BD_ASSERT(maxlen >= 2, "Reading an integer string requires at least 2 characters.");

	bdUInt charsLeft = maxlen;
	bdMemset(s, '\0', charsLeft);

	// the things that can make up an integer
	bdBool countedNullTerminator = false;
	bdBool haveSign = false;
	bdBool haveFirstDigit = false;

	bdNChar8 c = '\0';
	bdNChar8 sign = '\0';
	bdUInt i = 0;

	// skip any while space and get the first character
	skipWhiteSpace(c);

	// be carefull when calling charsLeft-- and it's unsigned and
	// you don't want it to underflow.
	while( charsLeft && ( IS_INTEGER(c) || IS_SIGN(c) ) )
	{
		if (!countedNullTerminator)
		{
			charsLeft--;

			// count the null terminator
			countedNullTerminator = true;			
			continue;
		}
		else if ( IS_SIGN(c) && !haveSign )
		{
			charsLeft--;

			// found a sign character
			sign = c;
			haveSign = true;
			
		}
		else if ( IS_INTEGER(c) && !haveFirstDigit )
		{
			charsLeft--;

			// found the first digit
			haveFirstDigit = true;
			
			// skip leading zeros
			while (IS_ZERO(c))
			{	
				c = '\0';
				m_streambuf->getFrom(&c, 1);
			}

			// write out the sign (if any)
			if (haveSign)
			{
				s[i] = sign;
				i++;
			}

			// no longer care about the sign so pretend we have one
			haveSign = true;

			if (IS_INTEGER(c))
			{
				s[i] = c;
				i++;
			}
			else
			{
				s[i] = '0';
				i++;
				// already have the next character in c so don't complete 
				// the loop.
				continue;
			}
		
		}
		else if ( IS_INTEGER(c))
		{
			charsLeft--;

			// the leading zeros have been stripped so just add the digit
			s[i] = c;
			i++;
		}
		else
		{
			break;
		}

		c = '\0';
		m_streambuf->getFrom(&c, 1);
	}

	return i;
}

bdUInt bdIStream::getExpString(bdNChar8* s,
								 const bdUInt maxlen,
								 bdNChar8 expChar = 'e')
{
	bdUInt charsLeft = maxlen;
	bdBool haveSign = false;
	bdBool haveFirstDigit = false;

	bdNChar8 c = '\0';
	bdNChar8 sign = '\0';
	bdUInt i = 0;

	m_streambuf->getFrom(&c, 1);

	while( charsLeft && ( IS_INTEGER(c) || IS_SIGN(c) ) )
	{
		if ( IS_SIGN(c) && !haveSign )
		{
			charsLeft--;

			// found a sign character
			sign = c;
			haveSign = true;
			
		}
		else if ( IS_INTEGER(c) && !haveFirstDigit )
		{
			charsLeft--;

			// found the first digit
			haveFirstDigit = true;
			
			// skip leading zeros
			while (IS_ZERO(c))
			{	
				c = '\0';
				m_streambuf->getFrom(&c, 1);
			}

			// write out the exp character
			s[i] = expChar;
			i++;

			// write out the sign (if any)
			if (haveSign)
			{
				s[i] = sign;
				i++;
			}

			// no longer care about the sign so pretend we have one
			haveSign = true;

			if (IS_INTEGER(c))
			{
				s[i] = c;
				i++;
			}
			else
			{
				s[i] = '0';
				i++;
				// already have the next character in c so don't complete 
				// the loop.
				continue;
			}
		
		}
		else if ( IS_INTEGER(c))
		{
			charsLeft--;

			// the leading zeros have been stripped so just add the digit
			s[i] = c;
			i++;
		}
		else
		{
			break;
		}

		c = '\0';
		m_streambuf->getFrom(&c, 1);
	}

	return i;
}


bdUInt bdIStream::getFloatString(bdNChar8* s,
								   const bdUInt maxlen)
{
	BD_ASSERT(maxlen >= 2, "Reading an float string requires at least 2 characters.");

	bdUInt i = 0;

	bdNChar8 c;
	bdNChar8 sign = '\0';
	bdNChar8 decimalPoint = '\0';

	bdUInt charsLeft = maxlen;
	
	bdBool countedNullTerminator = false;
	bdBool haveSign = false;
	bdBool haveDecimalPoint = false;
	bdBool haveFirstDigit = false;

	bdMemset(s, '\0', charsLeft);

	// skip any while space and get the first character
	skipWhiteSpace(c);

	while ( charsLeft && (IS_INTEGER(c) || IS_SIGN(c) || IS_DECIMALPOINT(c) || IS_EXP(c) ) )
	{
		if (!countedNullTerminator)
		{
			charsLeft--;
			// count the null terminator
			countedNullTerminator = true;			
			continue;
		}
		else if ( IS_SIGN(c) && !haveSign )
		{
			charsLeft--;
			// found a sign character
			sign = c;
			haveSign = true;
		}
		else if ( IS_DECIMALPOINT(c) && !haveDecimalPoint )
		{
			charsLeft--;
			// found a sign character
			decimalPoint = c;
			haveDecimalPoint = true;

			// If we have already written some digits then write the decimal 
			// point. 
			// If not we postpone the write until we are sure there are some 
			// digits eg ".x" should write nothing.
			if (haveFirstDigit)
			{
				s[i] = decimalPoint;
				i++;
			}
		}
		else if ( IS_EXP(c) && haveFirstDigit)
		{
			const bdUInt numWritten = getExpString(s+i, charsLeft);
			charsLeft -= numWritten;
			i += numWritten;
			break;
		}
		else if ( IS_INTEGER(c) && !haveFirstDigit )
		{
			charsLeft--;
			// found the first digit
			haveFirstDigit = true;

			// skip leading zeros only if we are before the decimal point
			if(!haveDecimalPoint)
			{				
				while (IS_ZERO(c))
				{	
					c = '\0';
					m_streambuf->getFrom(&c, 1);
				}
			}

			// write out the sign (if any)
			if (haveSign)
			{
				s[i] = sign;
				i++;
			}

			// no longer care about the sign so pretend we have one
			haveSign = true;

			// write out the decimal point (if any)
			if (haveDecimalPoint)
			{
				s[i] = decimalPoint;
				i++;
			}

			// If the first non zero character is a valid integer write it.
			// Otherwise just write '0'.
			if (IS_INTEGER(c))
			{
				s[i] = c;
				i++;
			}
			else
			{
				s[i] = '0';
				i++;
				// already have the next character in c so don't complete 
				// the loop.
				continue;
			}
		}
		else if (IS_INTEGER(c))
		{
			charsLeft--;
			s[i] = c;
			i++;
		}
		else
		{
			break;
		}

		c = '\0';
		m_streambuf->getFrom(&c, 1);

	}

	return i;
}
