// DWBS ------------------------------------------------------------------ DWBS
//
// PLATFORM		: ALL
// PRODUCT		: BIT_DEMON
// VISIBILITY	: PUBLIC
//
// DWBS ------------------------------------------------------------------ DWBS

#include <bdPlatform/bdPlatformString/bdPlatformString.h>

#include <bdCore/bdStreams/bdOStream.h>

void bdOStreamManipulator::endl(bdOStream& os)
{
	os << '\n';
	os.flush();
}

void bdOStreamManipulator::flush(bdOStream& os)
{
	os.flush();
}

bdOStream::bdOStream(bdStreambuf* streambuf)
: m_streambuf(streambuf)
{
}

bdBool bdOStream::isOk() const
{
	bdBool ok = false;
	if (m_streambuf)
	{
		ok = m_streambuf->isOk();
	}

	return ok;
}

bdOStream& bdOStream::operator<< (const bdNChar8& c)
{
	m_streambuf->putTo(reinterpret_cast<const bdNChar8*>(&c), 1);
	return *this;
}

//bdOStream& bdOStream::operator<< (const bdUChar8& c)
//{
//	m_streambuf->putTo(reinterpret_cast<const bdNChar8*>(&c), 1);
//	return *this;
//}

bdOStream& bdOStream::operator<< (const bdInt16& i)
{
	bdNChar8 buf[BD_MAX_INT16_DIGITS];
	bdSnprintf(buf, BD_MAX_INT16_DIGITS, "%ih", i);	
	return *this << buf;
}

bdOStream& bdOStream::operator<< (const bdUInt16& u)
{
	bdNChar8 buf[BD_MAX_INT16_DIGITS];
	bdSnprintf(buf, BD_MAX_INT16_DIGITS, "%uh", u);
	return *this << buf;
}

bdOStream& bdOStream::operator<< (const bdInt& i)
{
	bdNChar8 buf[BD_MAX_INT32_DIGITS];
	bdSnprintf(buf, BD_MAX_INT32_DIGITS, "%i", i);
	return *this << buf;
}

bdOStream& bdOStream::operator<< (const bdUInt& u)
{
	bdNChar8 buf[BD_MAX_INT32_DIGITS];
	bdSnprintf(buf, BD_MAX_INT32_DIGITS, "%u", u);
	return *this << buf;
}

bdOStream& bdOStream::operator<< (const bdInt64& i)
{
	bdNChar8 buf[BD_MAX_INT64_DIGITS];
	bdSnprintf(buf, BD_MAX_INT64_DIGITS, BD_INT_64_FORMAT, i);
	return *this << buf;
}

bdOStream& bdOStream::operator<< (const bdUInt64& u)
{
	bdNChar8 buf[BD_MAX_INT64_DIGITS];
	bdSnprintf(buf, BD_MAX_INT64_DIGITS, BD_UINT_64_FORMAT, u);
	return *this << buf;
}

bdOStream& bdOStream::operator<< (const bdFloat32& f)
{
	bdNChar8 buf[BD_MAX_FLOAT32_DIGITS];
	bdSnprintf(buf, BD_MAX_FLOAT32_DIGITS, "%f", f);
	return *this << buf;
}

bdOStream& bdOStream::operator<< (const bdFloat64& f)
{
	bdNChar8 buf[BD_MAX_FLOAT64_DIGITS];
	bdSnprintf(buf, BD_MAX_FLOAT64_DIGITS, "%fl", f);
	return *this << buf;
}

bdOStream& bdOStream::operator<< (const bdNChar8 *const s)
{
	if (s)
	{
		m_streambuf->putTo(reinterpret_cast<const bdNChar8*>(s),
						   static_cast<bdUInt>(bdStrlen(reinterpret_cast<const bdNChar8*>(s))));
	}
	return *this;
}

//bdOStream& bdOStream::operator<< (const bdUChar8 *const s)
//{
//	return *this << reinterpret_cast<const bdChar8*>(s);
//}

bdOStream& bdOStream::operator<< (const bdString &s)
{
	return *this << s.getBuffer();
}

bdOStream& bdOStream::operator<< (const bdBool& b)
{
	bdNChar8 c = '0';
	if (b)
	{
		c = '1';
	}

	return *this << c;
}

bdOStream& bdOStream::operator<< (bdOStreamManipulator::function f)
{
	(*f)(*this);
	return *this;
}

void bdOStream::flush()
{
	m_streambuf->flush();
}

bdUInt bdOStream::write(const bdNChar8 *const buf, 
						  const bdUInt nbytes)
			
{
	return m_streambuf->putTo(buf, nbytes);
}
	
bdBool bdOStream::seek(const bdUInt offset,
					   const bdStreambuf::SeekFrom from)
{
	return m_streambuf->seek(offset, from);
}

bdBool bdOStream::tell(bdUInt &offset) const
{
	return m_streambuf->tell(offset);
}

bdStreambuf* bdOStream::getStreambuf()
{
	return m_streambuf;
}
