// DWBS ------------------------------------------------------------------ DWBS
//
// PLATFORM		: ALL
// PRODUCT		: BIT_DEMON
// VISIBILITY	: PUBLIC
//
// DWBS ------------------------------------------------------------------ DWBS

#include <bdCore/bdCore.h>

#include <Composite/bdUserInfo.h>
#include <Composite/bdTeamInfo.h>
#include <Composite/Simulation.h>

#include <bdLobbyExampleCommon/LobbyStrings.h>

#include <bdLobby/bdRemoteTaskManager/bdRemoteTask.h>
#include <bdLobby/bdTeams/bdCreateTeamResult.h>
#include <bdLobby/bdTeams/bdCheckTeamMembershipResult.h>
#include <bdLobby/bdTeams/bdGetTeamMembersResult.h>

#include <bdLobby/bdLobbyService.h>
#include <bdLobby/bdAuthService.h>
#include <bdLobby/bdLobbyCommon/bdAuthCommon.h>
#include <bdLobby/bdAuthTicket.h>

#define BD_TITLE_ID 0x55 // Composite Demo
#define BD_NEXT_KEY BD_BUTTON1

#define FRIENDS_DEMO_ACCEPT BD_MOVE_UP
#define FRIENDS_DEMO_DENY BD_MOVE_DOWN

char* Simulation::m_screenInstructionsHome = 
"Goto Teams (1) : " BD_BUTTON1_DESC "\n"
"Goto Friends (1): " BD_BUTTON2_DESC "\n"
"Goto Profiles (1): " BD_BUTTON3_DESC "\n"
"Goto Teams (2): " BD_BUTTON4_DESC "\n"
"Goto Messaging (1): " BD_BUTTON5_DESC "\n"
"Goto Teams (3): " BD_BUTTON6_DESC "\n";

// Teams 1
char* Simulation::m_screenInstructions1 = 
"Return to main screen: " BD_BUTTON3_DESC "\n\n"
"Create Team: " BD_BUTTON1_DESC "\n"
"Invite member: " BD_MOVE_UP_DESC "\n"
"Accept membership: " BD_MOVE_DOWN_DESC "\n"
"Send Admin msg: " BD_BUTTON2_DESC "\n"
"Send Member msg: " BD_BUTTON6_DESC "\n"
"Get Membership: " BD_BUTTON5_DESC "\n"
"Get Members: " BD_BUTTON4_DESC "\n"
"Get Private Info: " BD_MOVE_LEFT_DESC "\n"
"Update name: " BD_MOVE_RIGHT_DESC "\n";

// Teams 2
char* Simulation::m_screenInstructions4 = 
"Return to main screen: " BD_BUTTON3_DESC "\n\n"
"promoteMemberToAdmin: " BD_BUTTON1_DESC "\n"
"kickMember: " BD_MOVE_UP_DESC "\n"
"leaveTeam: " BD_MOVE_DOWN_DESC "\n"
"leaveTeam and promote: " BD_BUTTON2_DESC "\n"
"reject membership: " BD_BUTTON6_DESC "\n"
"Invite To Session: " BD_BUTTON5_DESC "\n"
"Accept Invite: " BD_BUTTON4_DESC "\n"
"Refuse Invite: " BD_MOVE_LEFT_DESC "\n"
"Get Public Info: " BD_MOVE_RIGHT_DESC "\n";

// Teams 3
char* Simulation::m_screenInstructions6 = 
"Return to main screen: " BD_BUTTON3_DESC "\n\n"
"set public team info: " BD_BUTTON1_DESC "\n"
"set private team info: " BD_MOVE_UP_DESC "\n"
"set public member info: " BD_MOVE_DOWN_DESC "\n"
"set private member info: " BD_BUTTON2_DESC "\n"
"get public team info: " BD_BUTTON6_DESC "\n"
"get private team info: " BD_BUTTON5_DESC "\n"
"get public member info: " BD_BUTTON4_DESC "\n"
"get private member info: " BD_MOVE_LEFT_DESC "\n";

// Friends
char* Simulation::m_screenInstructions2 = 
"Return to main screen: " BD_BUTTON3_DESC "\n\n"
"Invite To Session: " BD_BUTTON1_DESC "\n"
"Accept Invite: " BD_MOVE_UP_DESC "\n"
"Refuse Invite: " BD_MOVE_DOWN_DESC "\n"
"Block: " BD_BUTTON2_DESC "\n"
"Propose Friendship: " BD_BUTTON6_DESC "\n"
"Accept Proposal: " BD_BUTTON5_DESC "\n"
"Refuse Proposal: " BD_BUTTON4_DESC "\n"
"Send Msg To Friend: " BD_MOVE_LEFT_DESC "\n";

// Profiles
char* Simulation::m_screenInstructions3 = 
"Return to main screen: " BD_BUTTON3_DESC "\n\n"
"Create Profile: " BD_BUTTON1_DESC "\n"
"Delete Profile: " BD_MOVE_UP_DESC "\n"
"Set Public Info: " BD_MOVE_DOWN_DESC "\n"
"Get Public Info: " BD_BUTTON2_DESC "\n"
"Set Private Info: " BD_BUTTON6_DESC "\n";

// Messages
char* Simulation::m_screenInstructions5 = 
"Return to main screen: " BD_BUTTON3_DESC "\n\n"
"Get Messages and delete: " BD_BUTTON1_DESC "\n"
"Get Messages but don't delete: " BD_MOVE_UP_DESC "\n"
"Delete Local Messages: " BD_BUTTON2_DESC "\n"
"Consume One Local Message: " BD_BUTTON5_DESC "\n"
"Cycle remote' user: " BD_BUTTON4_DESC "\n";

enum CmdScreen
{
	HOME,
	SCREEN1,
	SCREEN2,
	SCREEN3,
	SCREEN4,
	SCREEN5,
	SCREEN6,
	SCREEN7
};

Simulation::~Simulation()
{
	delete m_lobbyStrings;
}

bdBool Simulation::init(bdInt32 argc,
						bdNChar8* argv[])
{
	// Initialize member variables
	m_loggedOn = false;
	m_screenIndex = 0;
	m_remoteUserIndex = 0;

	pumpTaskScreen1 = false;
	pumpTaskScreen2 = false;
	pumpTaskScreen3 = false;
	pumpTaskScreen4 = false;
	pumpTaskScreen5 = false;
	pumpTaskScreen6 = false;

	pumpCreateTeam = false;
	pumpGetTeamMembership = false;
	pumpGetMembers = false;

	m_userResult = BD_NULL;
	m_friendResult = BD_NULL;
	m_teamResult = BD_NULL;
	m_profileResult = BD_NULL;

	m_friendsManager = BD_NULL;
	m_msgManager = BD_NULL;
	m_userManager = BD_NULL;
	m_teamManager = BD_NULL;

	m_teamTask = BD_NULL;
	m_teamResult = BD_NULL;

	m_lobbyStrings = new LobbyStrings();
		
	m_screenIndex = HOME;
	m_screenText = m_screenInstructionsHome;
	m_screenInstructions = m_screenInstructionsHome;

	bdLogImpl *log = bdLog::getInstance();

	// we want to hear about all warnings and errors.
	log->subscribe("warn", &m_logSubscriber);
	log->subscribe("err", &m_logSubscriber);
	log->subscribe("info", &m_logSubscriber);

	bdNetImpl *const net = bdNet::getInstance();

	// try to start bdNet
	bdNetStartParams params;
	params.m_onlineGame = true;

	// platformParams is used to store platform specific startup parameters
	bdPlatformParams platformParams;

	bdExamplePlatformInit::init(params, &platformParams);

	if(net->start(params))
	{
		// Wait for bdNetImpl::start to complete.
		// For code clarity we do an active wait.
		while(net->getStatus() == bdNetImpl::BD_NET_PENDING)
		{
			net->pump();
		}	
	}

	m_screenText = m_screenInstructions; 
	
	// Check bdNet has been started successfully.
	bdBool ok = (net->getStatus() == bdNetImpl::BD_NET_DONE);

	if(true == ok)
	{
		//
		// Authenticate ourselves with the Authentication Server (AS)
		//

		bdLobbyServiceImpl *lobbyService = bdLobbyService::getInstance();

		bdInAddr authAddr;
		bdInt numAddrs = bdPlatformSocket::getHostByName("auth.eval.mmp.demonware.net", &authAddr, 1);
		
		bdInetAddr inAddr(authAddr.m_inAddri);
		bdAddr authServerAddr(inAddr, 3074);

		bdAuthService *authService = new bdAuthService(BD_TITLE_ID,authServerAddr);
		
		bdAuthInfoRef authInfo = new bdAuthInfo();

		int clientNum = 1;

		// The first command line argument can be used to specify a different client
		// This allows testing with different user IDs
		if (argc > 1)
		{
			clientNum = atoi(argv[1]);
		}

		bdSnprintf(m_userName,14,"testusername%i",clientNum);

		// create account if necessary
		authService->createAccount(m_userName,"testpassword"); 

		while(authService->getStatus() != bdAuthService::BD_READY)
		{
			bdPlatformTiming::sleep(500);
		}

		// Use a pre-existing account intended for testing
		authService->authorizeAccount(m_userName,"testpassword", authInfo); 

		while(authService->getStatus() != bdAuthService::BD_READY)
		{
			bdPlatformTiming::sleep(500);
		}

		bdLobbyErrorCode err = authService->getErrorCode();	
		if ( err != BD_AUTH_NO_ERROR)
		{
			return false;
		}
	
		// get the local users id and username
		m_myId = bdAuthUtility::getUserID(m_userName);

		// get the remote users id and username
		if (clientNum % 2 == 0)
			m_remoteUserIndex = clientNum + 1;
		else
			m_remoteUserIndex = clientNum - 1;

		bdSnprintf(m_otherUserName,14,"testusername%i",m_remoteUserIndex);
		m_otherUserId = bdAuthUtility::getUserID(m_otherUserName);

		// Connect to MatchMaking+
		{
			bdInAddr lsgAddr;
	
			numAddrs = bdPlatformSocket::getHostByName("lobby.eval.mmp.demonware.net", &lsgAddr, 1);

			bdInetAddr inLSGAddr(lsgAddr.m_inAddri);
			bdAddr LSGServerAddr(inLSGAddr, 3074);
			ok = lobbyService->connect(LSGServerAddr, authInfo); 
		}

		delete authService;
	}

	bdConnectionStore *store = net->getConnectionStore();
	store->registerListener(this);

	return ok;
}

void Simulation::getManagers()
{
	bdLobbyServiceImpl *lobbyService = bdLobbyService::getInstance();

	if (lobbyService->getStatus() == bdConnection::BD_CONNECTED)
	{
		if(BD_NULL == m_friendsManager)
		{
			m_friendsManager = lobbyService->getFriends();
		}

		if(BD_NULL == m_msgManager)
		{
			m_msgManager = lobbyService->getMessageStore();
			lobbyService->setMessagingMode(bdLobbyServiceImpl::BD_RETAIN_PUSH_MESSAGES);
		}

		if(BD_NULL == m_teamManager)
		{
			m_teamManager = lobbyService->getTeams();
		}

		if(BD_NULL == m_userManager)
		{
			m_userManager = lobbyService->getProfilesManager();
		}
	}
}

void Simulation::waitForTwoSeconds()
{
	bdUInt64 timestamp = bdPlatformTiming::getHiResTimeStamp();

	bdBool waitForTwoSecs = true;
	while(waitForTwoSecs)
	{
		if (bdPlatformTiming::getElapsedTime(timestamp,bdPlatformTiming::getHiResTimeStamp()) > 1.0f)
		{
			waitForTwoSecs = false;
		}
	}
}

bdBool Simulation::changeScreens()
{
	// screen switching
	//	char* Simulation::m_screenInstructionsHome = 
	//		"Goto Teams (1) : " BD_BUTTON1_DESC "\n"
	//		"Goto Friends (1): " BD_MOVE_UP_DESC "\n"
	//		"Goto Profiles (1): " BD_MOVE_DOWN_DESC "\n"
	//		"Goto Teams (2): " BD_BUTTON2_DESC "\n";

	if (getScreen().wasKeyPressed(BD_BUTTON1) && (m_screenIndex == HOME))
	{
		m_screenIndex = SCREEN1;	
		m_screenText = m_screenInstructions1;
		m_screenInstructions = m_screenInstructions1;

		waitForTwoSeconds();
		return true;
	}

	if (getScreen().wasKeyPressed(BD_BUTTON2) && (m_screenIndex == HOME))
	{
		m_screenIndex = SCREEN2;
		m_screenText = m_screenInstructions2;
		m_screenInstructions = m_screenInstructions2;

		waitForTwoSeconds();
		return true;
	}

	if (getScreen().wasKeyPressed(BD_BUTTON3) && (m_screenIndex == HOME))
	{
		m_screenIndex = SCREEN3;
		m_screenText = m_screenInstructions3;
		m_screenInstructions = m_screenInstructions3;

		waitForTwoSeconds();
		return true;
	}

	if (getScreen().wasKeyPressed(BD_BUTTON4) && (m_screenIndex == HOME))
	{
		m_screenIndex = SCREEN4;
		m_screenText = m_screenInstructions4;
		m_screenInstructions = m_screenInstructions4;

		waitForTwoSeconds();
		return true;
	}

	if (getScreen().wasKeyPressed(BD_BUTTON5) && (m_screenIndex == HOME))
	{
		m_screenIndex = SCREEN5;
		m_screenText = m_screenInstructions5;
		m_screenInstructions = m_screenInstructions5;

		waitForTwoSeconds();
		return true;
	}

	if (getScreen().wasKeyPressed(BD_BUTTON6) && (m_screenIndex == HOME))
	{
		m_screenIndex = SCREEN6;
		m_screenText = m_screenInstructions6;
		m_screenInstructions = m_screenInstructions6;

		waitForTwoSeconds();
		return true;
	}

	if (getScreen().wasKeyPressed(BD_BUTTON3) && (m_screenIndex != HOME))
	{
		m_screenIndex = HOME;
		m_screenText = m_screenInstructionsHome;
		m_screenInstructions = m_screenInstructionsHome;

		waitForTwoSeconds();
		return true;
	}

	return false;
}

bdBool Simulation::step(const bdFloat32)
{
	// sample the inputs
	const bdBool quitKey   = getScreen().wasKeyPressed(BD_QUIT);
	
	bdLobbyServiceImpl *lobbyService = bdLobbyService::getInstance();

	bdNetImpl *const net = bdNet::getInstance();
	// receive messages from the network and dispatch them
	net->receiveAndDispatchAll();

	if (lobbyService->getStatus() == bdConnection::BD_CONNECTED)
	{
		if (changeScreens())
			return true;

		getManagers();

		updateBannerText();
		
		//
		// Step the appropriate sub-screen
		//

		//static bdBool res;

		if (m_screenIndex == SCREEN1)
		{
			stepScreen1();
		}

		if (m_screenIndex == SCREEN2)
		{
			stepScreen2();
		} 

		if (m_screenIndex == SCREEN3)
		{
			stepScreen3();
		}

		if (m_screenIndex == SCREEN4)
		{
			stepScreen4();
		} 

		if (m_screenIndex == SCREEN5)
		{
			stepScreen5();
		}

		if (m_screenIndex == SCREEN6)
		{
			stepScreen6();
		} 
	}

	// send all messages
	net->sendAll();

	// Display any screen messages.
	getScreen().printToScreen(m_screenTextStatic + m_screenText);

	return !quitKey;
}

void Simulation::stepScreen4()
{

	if (getScreen().wasKeyPressed(BD_BUTTON1))
	{
		m_screenText = "Calling bdTeams::promoteMember()\n";

		m_teamTask = m_teamManager->promoteMember(0,m_teamId,m_otherUserId);
		pumpTaskScreen4 = true;
	}
	else if (getScreen().wasKeyPressed(BD_MOVE_UP))
	{
		m_screenText = "Calling bdTeams::kickMember()\n";

		m_teamTask = m_teamManager->kickMember(0,m_teamId,m_otherUserId);
		pumpTaskScreen4 = true;
	}
	else if (getScreen().wasKeyPressed(BD_MOVE_DOWN))
	{
		m_screenText = "Calling bdTeams::leaveTeam()\n";

		m_teamTask = m_teamManager->leaveTeam(0,m_teamId);
		pumpTaskScreen4 = true;
	}
	else if (getScreen().wasKeyPressed(BD_BUTTON2))
	{
		m_screenText = "Calling bdTeams::leaveTeam() (and promote)\n";

		m_teamTask = m_teamManager->leaveTeam(0,m_teamId,m_otherUserId);
		pumpTaskScreen4 = true;
	}
	else if (getScreen().wasKeyPressed(BD_BUTTON6))
	{
		m_screenText = "Calling bdMessageStore::getNumMessages()\n";

		// REM: m_otherTeamId must be set to the appropriate teamId for this to suceed.
		// so we iterate through the messages that we have received to find that id
		bdUInt numMsgs = m_msgManager->getNumMessages();
		for (bdUInt i=0;i<numMsgs;i++)
		{
			bdLobbyMessageRef msg = m_msgManager->getAt(i);

			if (msg->m_msgType == BD_TEAM_INVITATION_MSG)
			{
				bdTeamInvitationMessage* inviteMsg = static_cast<bdTeamInvitationMessage*>(&*msg);
				m_otherTeamId = inviteMsg->m_teamID;
			}
		}

		m_teamTask = m_teamManager->rejectMembership(0,m_otherTeamId,m_otherUserId);
		pumpTaskScreen4 = true;
		m_teamResult = BD_NULL; 
	}
	else if (getScreen().wasKeyPressed(BD_BUTTON5))
	{
		m_screenText = "Calling bdTeams::inviteMember()\n";

		// invite a member to a (fake) onlinegame session
		bdSecurityID secID;
		bdMemcpy(secID.ab, static_cast<const void* >("UNBREAKABLE CODE"), BD_SECURITY_ID_LENGTH);

		m_teamTask = m_teamManager->inviteMember(0,m_otherUserId,secID);
		pumpTaskScreen4 = true;
		m_teamResult = BD_NULL; 
	}
	else if (getScreen().wasKeyPressed(BD_BUTTON4))
	{
		m_screenText = "Calling bdTeams::acceptInvitation()\n";

		// accept the invitation of an onlinegame session
		m_teamTask = m_teamManager->acceptInvitation(0,m_otherUserId);
		pumpTaskScreen4 = true;
		m_teamResult = BD_NULL; 
	}
	else if (getScreen().wasKeyPressed(BD_MOVE_LEFT))
	{
		m_screenText = "Calling bdTeams::rejectInvitation()\n";

		// reject the invitation of an onlinegame session
		m_teamTask = m_teamManager->rejectInvitation(0,m_otherUserId);
		pumpTaskScreen4 = true;
		m_teamResult = BD_NULL; 
	}
	else if (getScreen().wasKeyPressed(BD_MOVE_RIGHT))
	{
		m_screenText = "Calling bdTeams::getPublicInfo()\n";

		m_teamTask = m_teamManager->getPublicInfo(0,m_teamId);
		pumpTaskScreen4 = true;
		m_teamResult = new GetPublicTeamInfoResult(); 
	}

	//.pump.//
	if (pumpTaskScreen4)
	{
		pumpTeamTask(m_teamTask,m_teamResult);

		if (m_teamTask->getStatus() == bdRemoteTask::BD_EMPTY) 
		{
			pumpTaskScreen4 = false;

			if (m_teamResult != BD_NULL)
			{
				delete m_teamResult;
				m_teamResult = BD_NULL;
			}
		}
	}
}

void Simulation::stepScreen1()
{
	//pump
	if (pumpTaskScreen1)
	{
		pumpTeamTask(m_teamTask,m_teamResult);

		if (m_teamTask->getStatus() == bdRemoteTask::BD_EMPTY) 
		{
			pumpTaskScreen1 = false;

			if (m_teamResult != BD_NULL)
			{
				delete m_teamResult;
				m_teamResult = BD_NULL;
			}
		}
	}

	if (pumpCreateTeam)
	{
		bdBool res = pumpTeamTask(m_teamTask,m_teamResult);
		if (res == true || m_teamTask->getStatus() == 0) 
		{
			pumpCreateTeam = false;

			m_teamId = static_cast<bdCreateTeamResult*>(m_teamResult)->m_teamID;
			m_lobbyStrings->appendTeamCreationText(*(static_cast<bdCreateTeamResult*>(m_teamResult)),&m_screenText);

			delete m_teamResult;
			m_teamResult = BD_NULL;
		}
	}

	if (pumpGetTeamMembership)
	{
		bdBool res = pumpTeamTask(m_teamTask,m_teamResult);
		if (res == true || m_teamTask->getStatus() == 0) 
		{
			pumpGetTeamMembership = false;

			bdUInt numTeams = static_cast<bdCheckTeamMembershipResult*>(m_teamResult)->m_teams.getSize();
			if ( numTeams > 0)
			{
				m_teamId = static_cast<bdCheckTeamMembershipResult*>(m_teamResult)->m_teams[numTeams-1].m_teamID;
			}

			m_lobbyStrings->appendTeamMembershipText(*(static_cast<bdCheckTeamMembershipResult*>(m_teamResult)),&m_screenText);

			delete m_teamResult;
			m_teamResult = BD_NULL;
		}
	}

	if (pumpGetMembers)
	{
		bdBool res = pumpTeamTask(m_teamTask,m_teamResult);
		if (res == true || m_teamTask->getStatus() == 0) 
		{
			pumpGetMembers = false;

			m_lobbyStrings->appendTeamMembersText(*(static_cast<bdGetTeamMembersResult*>(m_teamResult)),&m_screenText);

			delete m_teamResult;
			m_teamResult = BD_NULL;
		}
	}

	if (m_teamTask == BD_NULL || m_teamTask->getStatus() == bdRemoteTask::BD_EMPTY)
	{
		if (getScreen().wasKeyPressed(BD_BUTTON1))
		{
			m_screenText = "Calling bsTeams::createTeam()\n";

			m_teamTask = m_teamManager->createTeam(0,"composite team");
			m_teamResult = new bdCreateTeamResult();
			pumpCreateTeam = true;
		}

		if (getScreen().wasKeyPressed(BD_MOVE_UP))
		{
			if (m_teamId == 14829735431805717965u)
			{
				return;
			}

			m_screenText = "Calling bdTeams::proposeMembership()\n";

			m_teamTask = m_teamManager->proposeMembership(0,m_teamId,m_otherUserId);
			m_teamResult = BD_NULL;

			pumpTaskScreen1 = true;
		}

		if (getScreen().wasKeyPressed(BD_MOVE_DOWN))
		{

			// REM: m_otherTeamId must be set to the appropriate teamId for this to suceed.
			// so we iterate through the messages that we have received to find that id
			bdUInt numMsgs = m_msgManager->getNumMessages();
			for (bdUInt i=0;i<numMsgs;i++)
			{
				bdLobbyMessageRef msg = m_msgManager->getAt(i);

				if (msg->m_msgType == BD_TEAM_INVITATION_MSG)
				{
					bdTeamInvitationMessage* inviteMsg = static_cast<bdTeamInvitationMessage*>(&*msg);
					m_otherTeamId = inviteMsg->m_teamID;
				}
			}

			m_screenText = "Calling bdTeams::acceptMembership()\n";

			// accept membership
			m_teamTask = m_teamManager->acceptMembership(0,m_otherTeamId,m_otherUserId);
			m_teamResult = BD_NULL;

			pumpTaskScreen1 = true;
		}

		if (getScreen().wasKeyPressed(BD_BUTTON2))
		{
			m_screenText = "Calling bdTeams::sendAdminMessage()\n";

			// send admin message
			bdNChar8 const * const msg = "Who's down for a 10 min fragfest?\n";
			bdUInt msgsize = static_cast<bdUInt>(bdStrlen(msg))+1;

			bdByteBuffer payload(msgsize);
			bdMemcpy(payload.getData(),msg,msgsize);
			bdTeams::bdMessageDeliveryMethod delivery = bdTeams::BD_PUSH_AND_PULL;

			m_teamTask = m_teamManager->sendAdminMessage(0,m_teamId,payload,delivery);
			m_teamResult = BD_NULL;

			pumpTaskScreen1 = true;
		}

		if (getScreen().wasKeyPressed(BD_BUTTON6))
		{
			m_screenText = "Calling bdTeams::sendMemberMessage()\n";

			// send member message
			bdNChar8 const * const msg = "Hey, can you promote me?\n";
			bdUInt msgsize = static_cast<bdUInt>(bdStrlen(msg))+1;

			bdByteBuffer payload(msgsize);
			bdMemcpy(payload.getData(),msg,msgsize);
			bdTeams::bdMessageDeliveryMethod delivery = bdTeams::BD_PUSH_AND_PULL;

			m_teamTask = m_teamManager->sendMemberMessage(0,m_otherUserId,payload,delivery);
			m_teamResult = BD_NULL;

			pumpTaskScreen1 = true;
		}
		else if (getScreen().wasKeyPressed(BD_BUTTON5))
		{
			m_screenText = "Calling bdTeams::checkMemberships()\n";

			m_teamTask = m_teamManager->checkMemberships(0);
			m_teamResult = new bdCheckTeamMembershipResult();

			pumpGetTeamMembership = true;
		}
		else if (getScreen().wasKeyPressed(BD_BUTTON4))
		{
			m_screenText = "Calling bdTeams::getMembers()\n";

			m_teamTask = m_teamManager->getMembers(0,m_teamId);
			m_teamResult = new bdGetTeamMembersResult();

			pumpGetMembers = true;
		}
		else if (getScreen().wasKeyPressed(BD_MOVE_LEFT))
		{
			m_screenText = "Calling bdTeams::getPrivateInfo()\n";

			m_teamTask = m_teamManager->getPrivateInfo(0,m_teamId);
			m_teamResult = new GetPrivateTeamInfoResult(); 

			pumpTaskScreen1 = true;
		}
		else if (getScreen().wasKeyPressed(BD_MOVE_RIGHT))
		{
			m_screenText = "Calling bdTeams::updateTeam()\n";

			m_teamTask = m_teamManager->updateTeam(0,m_teamId,"new team name");
			m_teamResult = BD_NULL;

			pumpTaskScreen1 = true;
		}
	}
}

void Simulation::stepScreen2()
{
	//pump
	if (pumpTaskScreen2)
	{
		/*bdBool res =*/ pumpFriendTask(m_friendTask,m_friendResult);

		if (m_friendTask->getStatus() == bdRemoteTask::BD_EMPTY)
		{
			pumpTaskScreen2 = false;

			if (m_teamResult != BD_NULL)
			{
				delete m_teamResult;
				m_teamResult = BD_NULL;
			}
		}
	}

	if (getScreen().wasKeyPressed(BD_BUTTON1))
	{
		// Invite our imaginary friend to an imaginary game session whose securityId 
		// is set to 'SECRETI'.  The imaginary friend will receive a session invite msg
		// and can respond by calling bdFriendManagement::handleInvitation
		m_screenText = "Calling bdFriends::inviteFriend()";

		bdSecurityID secID;
		bdMemcpy(secID.ab, static_cast<const void* >("SECRETI"), BD_SECURITY_ID_LENGTH);
		m_friendTask = m_friendsManager->inviteFriend(0,m_otherUserId,secID);

		pumpTaskScreen2 = true;
		m_friendResult = BD_NULL;
	}

	if (getScreen().wasKeyPressed(BD_MOVE_UP))
	{
		m_screenText = "Calling bdFriends::handleInvitation(true) 'accept session'\n";
		m_friendTask = m_friendsManager->acceptInvitation(0,m_otherUserId);
		m_friendResult = BD_NULL;

		pumpTaskScreen2 = true;
	}

	if (getScreen().wasKeyPressed(BD_MOVE_DOWN))
	{
		m_screenText = "Calling bdFriends::handleInvitation(false) 'refuse session'\n";
		m_friendTask = m_friendsManager->rejectInvitation(0,m_otherUserId);
		m_friendResult = BD_NULL;

		pumpTaskScreen2 = true;
	}

	if (getScreen().wasKeyPressed(BD_BUTTON2))
	{
		// Block a specified user from contacting the user associated with
		// controller 0 in any way (refuse friend and session invites)
		m_screenText = "Calling bdFriends::setBlockedStatus()\n";
		m_friendTask = m_friendsManager->setBlockedStatus(0,m_otherUserId,bdFriends::BD_INVITES_BLOCK);
		m_friendResult = BD_NULL;

		pumpTaskScreen2 = true;
	}

	if (getScreen().wasKeyPressed(BD_BUTTON6))
	{
		m_screenText = "Calling bdFriends::proposeFriendship()\n";
		m_friendTask = m_friendsManager->proposeFriendship(0,m_otherUserId);
		m_friendResult = BD_NULL;

		pumpTaskScreen2 = true;
	}

	if (getScreen().wasKeyPressed(BD_BUTTON5))
	{
		m_screenText = "Calling bdFriends::acceptProposal()\n";
		m_friendTask = m_friendsManager->acceptProposal(0,m_otherUserId);
		m_friendResult = BD_NULL;

		pumpTaskScreen2 = true;
	}

	if (getScreen().wasKeyPressed(BD_BUTTON4))
	{
		m_screenText = "Calling bdFriends::rejectProposal()\n";
		m_friendTask = m_friendsManager->rejectProposal(0,m_otherUserId);
		m_friendResult = BD_NULL;

		pumpTaskScreen2 = true;
	}

	if (getScreen().wasKeyPressed(BD_MOVE_LEFT))
	{
		m_screenText = "Calling bdFriends::sendMsgToFriend()\n";
		
		bdNChar8 const * const msg = "Hey there, up for a co-op game?\n";
		bdUInt msgsize = static_cast<bdUInt>(bdStrlen(msg))+1;

		bdByteBuffer payload(msgsize);
		bdMemcpy(payload.getData(),msg,msgsize);

		bdFriends::bdMessageDeliveryMethod delivery = bdFriends::BD_PUSH_AND_PULL;
		m_friendTask = m_friendsManager->sendFriendMessage(0,m_otherUserId,payload,delivery);
		m_friendResult = BD_NULL;

		pumpTaskScreen2 = true;
	}
}

void Simulation::stepScreen3()
{
	//			"Create Profile: " BD_BUTTON1_DESC "\n"
	//				"Delete Profile: " BD_MOVE_UP_DESC "\n"
	//				"Set Public Info: " BD_MOVE_DOWN_DESC "\n"
	//				"Get Public Info: " BD_BUTTON2_DESC "\n"
	//				"Set Private Info: " BD_BUTTON6_DESC "\n";

	//pump
	if (pumpTaskScreen3)
	{
		/*bdBool res =*/ pumpUserTask(m_userTask,m_userResult);

		if (m_userTask->getStatus() == bdRemoteTask::BD_EMPTY)
		{
			pumpTaskScreen3 = false;

			if (m_userResult != BD_NULL)
			{
				delete m_userResult;
				m_userResult = BD_NULL;
			}
		}
	}

	if (getScreen().wasKeyPressed(BD_BUTTON1))
	{
		m_screenText = "Calling bdProfiles::createProfile()\n";

		PublicProfileInfo info;
		info.m_experiencePoints = 555;

		m_userTask = m_userManager->createProfile(0,&info);
		m_userResult = BD_NULL;

		pumpTaskScreen3 = true;
	}

	if (getScreen().wasKeyPressed(BD_MOVE_UP))
	{
		m_screenText = "Calling bdProfiles::deleteProfile()\n";

		m_userTask = m_userManager->deleteProfile(0);
		m_userResult = BD_NULL;

		pumpTaskScreen3 = true;
	}

	if (getScreen().wasKeyPressed(BD_MOVE_DOWN))
	{
		m_screenText = "Calling bdProfiles::setPublicInfo()\n";

		PublicProfileInfo info;
		info.m_experiencePoints = 666;

		m_userTask = m_userManager->setPublicInfo(0,&info);
		m_userResult = BD_NULL;

		pumpTaskScreen3 = true;
	}

	if (getScreen().wasKeyPressed(BD_BUTTON2))
	{
		m_screenText = "Calling bdProfiles::getPublicInfo()\n";

		m_userTask = m_userManager->getPublicInfo(0,m_myId);
		m_userResult = new GetPublicProfileResult();

		pumpTaskScreen3 = true;
	}

	if (getScreen().wasKeyPressed(BD_BUTTON6))
	{
		m_screenText = "Calling bdProfiles::getPrivateInfo()\n";

		m_userTask = m_userManager->getPrivateInfo(0);
		m_userResult = new GetPrivateProfileResult();

		pumpTaskScreen3 = true;
	}
}

void Simulation::stepScreen5()
{
	//pump
	if (pumpTaskScreen5)
	{
		pumpMessageTask(m_msgTask);

		if (m_msgTask->getStatus() == bdRemoteTask::BD_EMPTY) 
		{
			pumpTaskScreen5 = false;
		}
	}

	if (getScreen().wasKeyPressed(BD_BUTTON1))
	{
		// Retrieve the messages that are currently in the database for the user associated with
		// controller 0.  At this point we don't expect any messages to be present for this user
		// and as such expect an empty result set
		m_screenText = "Calling bdMessageStore::getMessages()\n";
		bdBool deleteMessagesOnGet = true;
		m_msgTask = this->m_msgManager->getMessages(0,deleteMessagesOnGet);

		pumpTaskScreen5 = true;
	}

	if (getScreen().wasKeyPressed(BD_MOVE_UP))
	{
		// Retrieve the messages that are currently in the database for the user associated with
		// controller 0.  At this point we don't expect any messages to be present for this user
		// and as such expect an empty result set
		m_screenText = "Calling bdMessageStore::getMessages()";
		bdBool deleteMessagesOnGet = false;
		m_msgTask = this->m_msgManager->getMessages(0,deleteMessagesOnGet);

		pumpTaskScreen5 = true;
	}

	if (getScreen().wasKeyPressed(BD_BUTTON2))
	{
		m_screenText = "Deleting local messages";

		int numMessages = m_msgManager->getNumMessages(); 
		if (numMessages > 0)
		{
			for (int i=numMessages-1;i>=0;i--)
				m_msgManager->removeAt(0);
		}

		pumpTaskScreen5 = false;
	}

	if (getScreen().wasKeyPressed(BD_BUTTON5))
	{
		m_screenText = m_screenInstructions5;
		m_screenText += "Reading one message\n";

		m_lobbyStrings->consumeMessage(m_msgManager,&m_screenText);

		pumpTaskScreen5 = false;
	}
	if (getScreen().wasKeyPressed(BD_BUTTON4))
	{
		// Toggle 'recipient'
		m_remoteUserIndex += 1;
		if (m_remoteUserIndex==4)
		{
			m_remoteUserIndex = 0;
		}

		bdSnprintf(m_otherUserName,14,"testusername%i/n",m_remoteUserIndex);
		m_otherUserId = bdAuthUtility::getUserID(m_otherUserName);

		updateBannerText();
	}
}

void Simulation::stepScreen6()
{
	if (getScreen().wasKeyPressed(BD_BUTTON1))
	{
		m_screenText = "Calling bdTeams::setPublicInfo()\n";

		PublicTeamInfo pubInfo;
		pubInfo.m_int1 = 11;
		pubInfo.m_int2 = 22;
		pubInfo.m_int3 = 33;
		pubInfo.m_int4 = 44;
		pubInfo.m_int5 = 55;
		pubInfo.m_int6 = 66;
		pubInfo.m_string1 = "string1";
		pubInfo.m_string2 = "string2";

		m_teamTask = m_teamManager->setPublicInfo(0,m_teamId,&pubInfo);
		pumpTaskScreen6 = true;
		m_teamResult = BD_NULL; 
	}
	else if (getScreen().wasKeyPressed(BD_MOVE_UP))
	{
		m_screenText = "Calling bdTeams::setPrivateInfo()\n";

		PrivateTeamInfo privInfo;

		privInfo.m_string1 = "private string1";
		privInfo.m_string2 = "private string2";

		m_teamTask = m_teamManager->setPrivateInfo(0,m_teamId,&privInfo);
		pumpTaskScreen6 = true;
		m_teamResult = BD_NULL; 
	}
	else if (getScreen().wasKeyPressed(BD_MOVE_DOWN))
	{
		m_screenText = "Calling bdTeams::setPublicMemberInfo()\n";

		PublicTeamMemberInfo pubInfo;
		pubInfo.m_int1 = 11;
		pubInfo.m_int2 = 22;
		pubInfo.m_int3 = 33;
		pubInfo.m_string1 = "string1";
		pubInfo.m_string2 = "string2";

		m_teamTask = m_teamManager->setPublicMemberInfo(0,m_teamId,&pubInfo);
		pumpTaskScreen6 = true;
		m_teamResult = BD_NULL; 
	}
	else if (getScreen().wasKeyPressed(BD_BUTTON2))
	{
		m_screenText = "Calling bdTeams::setPrivateMemberInfo()\n";

		PrivateTeamMemberInfo privInfo;
		privInfo.m_int1 = 111;
		privInfo.m_string1 = "private string1";
		privInfo.m_string2 = "private string2";

		m_teamTask = m_teamManager->setPrivateMemberInfo(0,m_teamId,&privInfo);
		pumpTaskScreen6 = true;
		m_teamResult = BD_NULL; 
	}
	else if (getScreen().wasKeyPressed(BD_BUTTON6))
	{
		m_screenText = "Calling bdTeams::getPublicInfo()\n";

		m_teamTask = m_teamManager->getPublicInfo(0,m_teamId);
		pumpTaskScreen6 = true;
		m_teamResult = new GetPublicTeamInfoResult(); 
	}
	else if (getScreen().wasKeyPressed(BD_BUTTON5))
	{
		m_screenText = "Calling bdTeams::getPrivateInfo()\n";

		m_teamTask = m_teamManager->getPrivateInfo(0,m_teamId);
		pumpTaskScreen6 = true;
		m_teamResult = new GetPrivateTeamInfoResult();
	}
	else if (getScreen().wasKeyPressed(BD_BUTTON4))
	{
		m_screenText = "Calling bdTeams::getPublicMemberInfo()\n";

		m_teamTask = m_teamManager->getPublicMemberInfo(0,m_teamId,m_myId);
		pumpTaskScreen6 = true;
		m_teamResult = new GetPublicTeamMemberInfoResult(); 
	}
	else if (getScreen().wasKeyPressed(BD_MOVE_LEFT))
	{
		m_screenText = "Calling bdTeams::getPrivateMemberInfo()\n";

		m_teamTask = m_teamManager->getPrivateMemberInfo(0,m_teamId,m_myId);
		pumpTaskScreen6 = true;
		m_teamResult = new GetPrivateTeamMemberInfoResult();
	}

	static bdBool res;

	//.pump.//
	if (pumpTaskScreen6)
	{
		res = pumpTeamTask(m_teamTask,m_teamResult);

		if (m_teamTask->getStatus() == bdRemoteTask::BD_EMPTY) 
		{
			pumpTaskScreen6 = false;

			if (m_teamResult != BD_NULL)
			{
				delete m_teamResult;
				m_teamResult = BD_NULL;
			}
		}
	}
}

void Simulation::quit()
{
	// Remove any session hosted by this client from the MatchMaking service
	bdLobbyServiceImpl *const lobbyService = bdLobbyService::getInstance();
	bdNetImpl *const net = bdNet::getInstance();
	if(net->getStatus() == bdNetImpl::BD_NET_DONE)
	{
		if (lobbyService->getStatus() == bdConnection::BD_CONNECTED)
		{
			lobbyService->disconnect();
		}

		net->sendAll();
		net->stop();
	}
	
	bdExamplePlatformInit::quit();
}

void Simulation::onConnect(bdConnectionRef connection)
{
	const bdUInt addrSize = 100;
	bdNChar8 addr[addrSize];

	bdAddressMap::getInstance()->addrToString(connection->getAddressHandle(), addr, addrSize);

	m_screenText += "\nAccepted connection from ";
	m_screenText += addr;
}

bool Simulation::pumpFriendTask( bdRemoteTask * friendTask, bdQueryResult *const infoResults )
{
	bdBool success = false;
	if(friendTask && (friendTask->getStatus() == bdRemoteTask::BD_DONE))
	{
		if (this->m_friendsManager->handleTask(friendTask, infoResults))
		{
			m_screenText = m_screenInstructions; 
			m_screenText += "\nFriends task completed successfully\n";
			success = true;
		}
		else
		{
			m_screenText = m_screenInstructions; 
			m_screenText += "\nFriends task failed:\n";
			
			m_lobbyStrings->formatErrorString(m_friendsManager->getErrorCode());
			m_screenText += m_lobbyStrings->getLastError();
		}

		friendTask->close();
		friendTask = BD_NULL;

	}
	return success;
}

bool Simulation::pumpMessageTask( bdRemoteTask * task )
{
	bdBool success = false;
	if(task && (task->getStatus() == bdRemoteTask::BD_DONE))
	{
		if (m_msgManager->handleTask(task))
		{
			m_screenText = m_screenInstructions; 
			m_screenText += "\ntask completed successfully\n";
			success = true;
		}
		else
		{
			m_screenText = m_screenInstructions; 
			m_screenText += "\ntask failed:\n";
	
			m_lobbyStrings->formatErrorString(m_msgManager->getErrorCode());
			m_screenText += m_lobbyStrings->getLastError();
		}

		task->close();
		task = BD_NULL;
	}
	return success;
}

bool Simulation::pumpUserTask( bdRemoteTask * profileTask, bdQueryResult *const infoResults )
{
	bdBool success = false;
	if(profileTask && (profileTask->getStatus() == bdRemoteTask::BD_DONE))
	{
		if (this->m_userManager->handleTask(profileTask, infoResults))
		{
			m_screenText = m_screenInstructions; 
			m_screenText += "\nUser task completed successfully\n";
			success = true;
		}
		else
		{
			m_screenText = m_screenInstructions; 
			m_screenText += "\nUser task failed:\n";

			m_lobbyStrings->formatErrorString(m_userManager->getErrorCode());
			m_screenText += m_lobbyStrings->getLastError();
		}
		profileTask->close();
		profileTask = BD_NULL;
	}
	return success;
}

bool Simulation::pumpTeamTask( bdRemoteTask * task, bdQueryResult *const infoResults )
{
	bdBool success = false;
	if(task && (task->getStatus() == bdRemoteTask::BD_DONE))
	{
		if (m_teamManager->handleTask(task, infoResults))
		{
			m_screenText = m_screenInstructions; 
			m_screenText += "\ntask completed successfully\n";
			success = true;
		}
		else
		{
			m_screenText = m_screenInstructions; 
			m_screenText += "\ntask failed:\n";

			m_lobbyStrings->formatErrorString(m_teamManager->getErrorCode());
			m_screenText += m_lobbyStrings->getLastError();
		}

		task->close();
		task = BD_NULL;
	}
	return success;
}

// update the top line of screen text - 
// userId (local) numMessages lastMsgType
void Simulation::updateBannerText()
{
	m_screenTextStatic = m_userName;
	m_screenTextStatic += " (local) ";
	m_screenTextStatic += m_otherUserName;
	m_screenTextStatic += " (remote)\n";

	char buffer1[8];
	bdUInt numMsgs = m_msgManager->getNumMessages();
	bdSnprintf(buffer1,16,"%x",numMsgs);
	m_screenTextStatic += "numMessages: ";	
	m_screenTextStatic += buffer1;

	if ( numMsgs > 0 )
	{
		m_lobbyStrings->formatMessageString(m_msgManager->getAt(numMsgs-1)->m_msgType);
		m_screenTextStatic += " ";
		m_screenTextStatic += m_lobbyStrings->getLastMessage();
	}

	m_screenTextStatic += "\n";
}
