// DWBS ------------------------------------------------------------------ DWBS
//
// PLATFORM		: ALL
// PRODUCT		: BIT_DEMON
// VISIBILITY	: PUBLIC
//
// DWBS ------------------------------------------------------------------ DWBS

#ifndef SIMULATION_H
#define SIMULATION_H

#include <bdExample/bdExampleCommonLobby/bdExampleCommonLobby.h>

#include <bdLobby/bdKeyArchive/bdKeyArchive.h>
#include <bdLobby/bdKeyArchive/bdKeyPairDict.h>

#define MAX_NUM_ENTITYIDS (10u)

class Simulation : public bdDemo
{
	public:

		BD_DECLARE_NEW_AND_DELETE_OPERATORS

		Simulation();

		~Simulation();

		virtual bdBool init(bdInt argc,
							bdNChar8* argv[]);

		virtual bdBool step(const bdFloat32 time);
		
		virtual void quit();

	protected:

		enum Task
		{
			START_NETWORK,			
			CHECK_DNS,
			CREATE_ACCOUNT,			
			AUTHENTICATE_ACCOUNT,
			LOBBY_CONNECT,

			//WRITE_KEY_PAIRS,
			READ_PAIRS,
			READ_ALL_PAIRS,
			//DELETE_PAIRS,
			USER_WRITE_PAIRS,
			USER_DELETE_PAIRS,
			USER_DELETE_ALL_USERS_PAIRS,

			LOBBY_DISCONNECT,
			STOP_NETWORK
		};

		void startTask();
		void pumpTask();
		void displayTaskResult();

		// Network startup
		void startNetworkDescription(bdString& description);
		void startNetwork();
		void pumpStartNetwork();

		// DNS lookup
		void checkDNSDescription(bdString& description);
		void startCheckDNS();
		void pumpCheckDNS();

		// Authentication service 	
		bdAuthService* getAuthService();
		const bdAuthInfo *getAuthInfo();
		void checkAuthServiceError();
		void pumpAuthServiceTask();
	
		// Create account
		void createAccountDescription(bdString& description);
		void startCreateAccount();
		void pumpCreateAccount();

		// Authenticate account
		void authenticateAccountDescription(bdString& description);
		void startAuthenticateAccount();
		void pumpAuthenticateAccount();

		// Lobby service connect
		void lobbyServiceConnectDescription(bdString& description);
		void startLobbyServiceConnect();
		void pumpLobbyServiceConnect();


		// bdKeyArchive
		void checkKeyArchiveError();
		void printDictList(bdString& string) const;


		// Read Specific KeyPairs
		void readKeyPairsDescription(bdString& description);
		void startReadKeyPairs();
		void pumpReadKeyPairs();

		// read all key pairs belonging to an entityID
		void readAllKeyPairsDescription(bdString& description);
		void startReadAllKeyPairs();
		void pumpReadAllKeyPairs();

		// user Writes own KeyPairs
		void writeKeyPairsDescription(bdString& description);
		void startWriteKeyPairs();
		void pumpWriteKeyPairs();
		
		// User Deletes own specific KeyPairs
		void deleteKeyPairsDescription(bdString& description);
		void startDeleteKeyPairs();
		void pumpDeleteKeyPairs();

		// User Deletes all of own KeyPairs
		void deleteUsersKeyPairsDescription(bdString& description);
		void startDeleteUsersKeyPairs();
		void pumpDeleteUsersKeyPairs();

		// Lobby disconnect
		void lobbyServiceDisconnectDescription(bdString& description);
		void startLobbyServiceDisconnect();
		void pumpLobbyServiceDisconnect();

		// Network stop
		void stopNetworkDescription(bdString& description);
		void stopNetwork();
		
	protected:

		// receives log messages
		bdLogSubscriber m_logSubscriber;

		TaskStatus m_taskStatus;
		Task m_task;
		bdString m_lastTaskDescription;

		bdBool m_changedScreen;
		bdString m_screenText;

		bdInetAddr m_authAddr;
		bdGetHostByName *m_authGetHostByName;

		bdAuthService *m_authService;

		bdInetAddr m_lsgAddr;
		bdGetHostByName *m_lsgGetHostByName;

		bdRemoteTask *m_remoteTask;

		bdBool m_pumpLobbyConnection;
		bdLobbyErrorCode m_lobbyErrorCode;
};

#endif // SIMULATION_H
