// DWBS ------------------------------------------------------------------ DWBS
//
// PLATFORM		: ALL
// PRODUCT		: BIT_DEMON
// VISIBILITY	: PUBLIC
//
// DWBS ------------------------------------------------------------------ DWBS

#ifndef SIMULATION_H
#define SIMULATION_H

#include <bdExample/bdExampleCommon/bdExampleCommon.h>

#include <bdExample/bdExampleCommonLobby/bdExampleCommonLobby.h>
#include <bdLobby/bdLobbyCommon/bdLobbyServicesCommon.h>
#include <bdCore/bdCrypto/bdHashTiger192.h>

#define STORAGE_EXAMPLE_FILE_SIZE 24

class Simulation : public bdDemo
{
	public:

		BD_DECLARE_NEW_AND_DELETE_OPERATORS

		Simulation();

		~Simulation();

		virtual bdBool init(bdInt argc,
							bdNChar8* argv[]);

		virtual bdBool step(const bdFloat32 time);
		
		virtual void quit();

	protected:

		enum Task
		{
			START_NETWORK,			
			CHECK_DNS,
			CREATE_ACCOUNT,			
			AUTHENTICATE_ACCOUNT,
			LOBBY_CONNECT,

			// storage tasks
			UPLOAD_FILE,
			UPDATE_FILE,
			UPDATE_FILE_PROPERTIES,
			GET_FILE,
			LIST_FILES_BY_OWNER,
			LIST_ALL_PUBLISHER_FILES,
			REMOVE_FILE,

			LOBBY_DISCONNECT,
			STOP_NETWORK
		};

		void startTask();
		void pumpTask();
		void displayTaskResult();

		// Network startup
		void startNetworkDescription(bdString& description);
		void startNetwork();
		void pumpStartNetwork();

		// DNS lookup
		void checkDNSDescription(bdString& description);
		void startCheckDNS();
		void pumpCheckDNS();

		// Authentication service 	
		bdAuthService* getAuthService();
		const bdAuthInfo *getAuthInfo();
		void checkAuthServiceError();
		void pumpAuthServiceTask();

		// Create account
		void createAccountDescription(bdString& description);
		void startCreateAccount();
		void pumpCreateAccount();

		// Authenticate account
		void authenticateAccountDescription(bdString& description);
		void startAuthenticateAccount();
		void pumpAuthenticateAccount();

		// Lobby service connect
		void lobbyServiceConnectDescription(bdString& description);
		void startLobbyServiceConnect();
		void pumpLobbyServiceConnect();
		
		// upload file
		void generateUploadFileDescription(bdString& description);
		void startUploadFile();
		void pumpUploadFile();
		
		// update file
		void generateUpdateFileDescription(bdString& description);
		void startUpdateFile();
		void pumpUpdateFile();
	
		// update file properties
		void generateUpdateFilePropertiesDescription(bdString& description);
		void startUpdateFileProperties();
		void pumpUpdateFileProperties();
		
		// get file description
		void generateGetFileDescription(bdString& description);
		void startGetFile();
		void pumpGetFile();
	
		// list files by owner
		void generateListFilesByOwnerDescription(bdString& description);
		void startListFilesByOwner();
		void pumpListFilesByOwner();
	
		// list all publisher files
		void generateListAllPublisherFilesDescription(bdString& description);
		void startListAllPublisherFiles();
		void pumpListAllPublisherFiles();
	
		// remove file
		void generateRemoveFileDescription(bdString& description);
		void startRemoveFile();
		void pumpRemoveFile();

		// Lobby disconnect
		void lobbyServiceDisconnectDescription(bdString& description);
		void startLobbyServiceDisconnect();
		void pumpLobbyServiceDisconnect();

		// Network stop
		void stopNetworkDescription(bdString& description);
		void stopNetwork();
		
	protected:

		// receives log messages
		bdLogSubscriber m_logSubscriber;

		TaskStatus m_taskStatus;
		Task m_task;
		bdString m_lastTaskDescription;

		bdBool m_changedScreen;
		bdString m_screenText;

		bdInetAddr m_authAddr;
		bdGetHostByName *m_authGetHostByName;

		bdAuthService *m_authService;

		bdInetAddr m_lsgAddr;
		bdGetHostByName *m_lsgGetHostByName;

		bdRemoteTask *m_remoteTask;

		bdBool m_pumpLobbyConnection;
		bdLobbyErrorCode m_lobbyErrorCode;

		// storage specific data
		bdHashTiger192 m_tigerHash;
		bdUByte8 m_hashFile[STORAGE_EXAMPLE_FILE_SIZE];
		bdUInt64 m_fileId;
		bdStorage *m_storageManager;
		bdOnlineUserID m_UserId;
		bdString m_fileListString;
};

#endif // SIMULATION_H
