// DWBS ------------------------------------------------------------------ DWBS
//
// PLATFORM		: ALL
// PRODUCT		: BIT_DEMON
// VISIBILITY	: PUBLIC
//
// DWBS ------------------------------------------------------------------ DWBS

#ifndef SIMULATION_H
#define SIMULATION_H

#include "TeamInfo.h"

#include <bdExample/bdExampleCommonLobby/bdExampleCommonLobby.h>
#include <bdLobby/bdTeams/bdCheckTeamMembershipResult.h>
#include <bdLobby/bdTeams/bdGetTeamMembersResult.h>
#include <bdLobby/bdTeams/bdGetTeamsProposalListResult.h>
#include <bdLobby/bdTeams/bdCreateTeamResult.h>
#include <bdLobby/bdTeams/bdTeamsResult.h>


class Simulation : public bdDemo
{
	public:

		BD_DECLARE_NEW_AND_DELETE_OPERATORS

		Simulation();

		~Simulation();

		virtual bdBool init(bdInt argc,
							bdNChar8* argv[]);

		virtual bdBool step(const bdFloat32 time);
		
		virtual void quit();

	protected:

		enum Task
		{
			START_NETWORK,			
			CHECK_DNS,
			CHOOSE_USER_ACCOUNT_1,
			CHOOSE_USER_ACCOUNT_2,
			CREATE_ACCOUNT,
			AUTHENTICATE_ACCOUNT,
			LOBBY_CONNECT,
			CREATE_PROFILE,
			GET_MESSAGES,
			ACCEPT_MEMBERSHIP,
			REJECT_MEMBERSHIP,
			ACCEPT_INVITATION,
			REJECT_INVITATION,
			CHECK_MEMBERSHIPS,
			CREATE_TEAM,
			UPDATE_TEAM,
			GET_MEMBERS,
			PROPOSE_MEMBERSHIP,
			GET_PROPOSALS,
			WITHDRAW_PROPOSAL,
			INVITE_TO_SESSION,
			PROMOTE_MEMBER,
			DEMOTE_MEMBER,
			KICK_MEMBER,
			SET_PUBLIC_TEAM_INFO,
			GET_PUBLIC_TEAM_INFO,
			SET_PRIVATE_TEAM_INFO,
			GET_PRIVATE_TEAM_INFO,
			SET_PUBLIC_MEMBER_INFO,
			GET_PUBLIC_MEMBER_INFO,
			SET_PRIVATE_MEMBER_INFO,
			GET_PRIVATE_MEMBER_INFO,
			SEND_MEMBER_MESSAGE,
			SEND_ADMIN_MESSAGE,
			LOBBY_DISCONNECT,
			STOP_NETWORK
		};

		void startTask();

		void pumpTask();

		void displayTaskResult();

		// Network startup

		void startNetworkDescription(bdString& description);

		void startNetwork();

		void pumpStartNetwork();

		// DNS lookup

		void checkDNSDescription(bdString& description);

		void startCheckDNS();

		void pumpCheckDNS();

		// Authentication service 	

		bdAuthService* getAuthService();

		const bdAuthInfo *getAuthInfo();

		void checkAuthServiceError();

		void pumpAuthServiceTask();

		// Choose User Account 1

		void chooseUserAccount1Description(bdString& description);

		void startChooseUserAccount1();

		void pumpChooseUserAccount1();


		// Choose User Account 2

		void chooseUserAccount2Description(bdString& description);

		void startChooseUserAccount2();

		void pumpChooseUserAccount2();


		// Create account

		void createAccountDescription(bdString& description);

		void startCreateAccount();

		void pumpCreateAccount();

		// Authenticate account

		void authenticateAccountDescription(bdString& description);

		void startAuthenticateAccount();

		void pumpAuthenticateAccount();

		// Lobby service connect

		void lobbyServiceConnectDescription(bdString& description);

		void startLobbyServiceConnect();

		void pumpLobbyServiceConnect();

		// Create profile

		void createProfileDescription(bdString& description);

		void startCreateProfile();

		void pumpCreateProfile();

		// Teams

		// Get Messages
		void getMessagesDescription(bdString& description);

		void startGetMessages();

		void pumpGetMessages();

		// Accept membership
		void acceptMembershipDescription(bdString& description);

		void startAcceptMembership();

		void pumpAcceptMembership();

		// Reject membership
		void rejectMembershipDescription(bdString& description);

		void startRejectMembership();

		void pumpRejectMembership();

		// Accept Invitation
		void acceptInvitationDescription(bdString& description);

		void startAcceptInvitation();

		void pumpAcceptInvitation();

		// Reject Invitation
		void rejectInvitationDescription(bdString& description);

		void startRejectInvitation();

		void pumpRejectInvitation();

		// Check memberships
		void checkMembershipsDescription(bdString& description);

		void startCheckMemberships();

		void pumpCheckMemberships();



		// Create Team

		void createTeamDescription(bdString& description);

		void startCreateTeam();

		void pumpCreateTeam();


		// Update Team

		void updateTeamDescription(bdString& description);

		void startUpdateTeam();

		void pumpUpdateTeam();


		// Get Members

		void getMembersDescription(bdString& description);

		void startGetMembers();

		void pumpGetMembers();


		// Propose Membership

		void proposeMembershipDescription(bdString& description);

		void startProposeMembership();

		void pumpProposeMembership();


		// Get Proposals

		void getProposalsDescription(bdString& description);

		void startGetProposals();

		void pumpGetProposals();


		// Withdraw Proposal

		void withdrawProposalDescription(bdString& description);

		void startWithdrawProposal();

		void pumpWithdrawProposal();


		// Invite To Session

		void inviteToSessionDescription(bdString& description);

		void startInviteToSession();

		void pumpInviteToSession();


		// Promote Member

		void promoteMemberDescription(bdString& description);

		void startPromoteMember();

		void pumpPromoteMember();


		// Demote Member

		void demoteMemberDescription(bdString& description);

		void startDemoteMember();

		void pumpDemoteMember();


		// Kick Member

		void kickMemberDescription(bdString& description);

		void startKickMember();

		void pumpKickMember();

		// Set Public Team Info

		void setPublicTeamInfoDescription(bdString& description);

		void startSetPublicTeamInfo();

		void pumpSetPublicTeamInfo();

		// Get Public Team Info

		void getPublicTeamInfoDescription(bdString& description);

		void startGetPublicTeamInfo();

		void pumpGetPublicTeamInfo();

		// Set Private Team Info

		void setPrivateTeamInfoDescription(bdString& description);

		void startSetPrivateTeamInfo();

		void pumpSetPrivateTeamInfo();

		// Get Private Team Info

		void getPrivateTeamInfoDescription(bdString& description);

		void startGetPrivateTeamInfo();

		void pumpGetPrivateTeamInfo();

		// Set Public Member Info

		void setPublicMemberInfoDescription(bdString& description);

		void startSetPublicMemberInfo();

		void pumpSetPublicMemberInfo();

		// Get Public Member Info

		void getPublicMemberInfoDescription(bdString& description);

		void startGetPublicMemberInfo();

		void pumpGetPublicMemberInfo();

		// Set Private Member Info

		void setPrivateMemberInfoDescription(bdString& description);

		void startSetPrivateMemberInfo();

		void pumpSetPrivateMemberInfo();

		// Get Private Member Info

		void getPrivateMemberInfoDescription(bdString& description);

		void startGetPrivateMemberInfo();

		void pumpGetPrivateMemberInfo();

		// Send Member Message

		void sendMemberMessageDescription(bdString& description);

		void startSendMemberMessage();

		void pumpSendMemberMessage();


		// Send Admin Message

		void sendAdminMessageDescription(bdString& description);

		void startSendAdminMessage();

		void pumpSendAdminMessage();


		// Lobby disconnect

		void lobbyServiceDisconnectDescription(bdString& description);

		void startLobbyServiceDisconnect();

		void pumpLobbyServiceDisconnect();

		// Network stop

		void stopNetworkDescription(bdString& description);

		void stopNetwork();

		// Error checking
		void checkProfilesError();

		void checkTeamsError();

		// A generic pump method, for all channel chat tasks that do not return results.
		void pumpGenericTeamsTask();
		
		// A method to determine the next Task, based on any proposals or invites
		// present in the message store
		Task determineNextTask(Task currentTask);

		// A method to consume and display any messages that have arrived in the
		// message store.
		void checkMessageStore();

	protected:

		// receives log messages
		bdLogSubscriber m_logSubscriber;

		TaskStatus m_taskStatus;
		Task m_task;
		bdString m_lastTaskDescription;

		bdBool m_changedScreen;
		bdString m_screenText;

		bdInetAddr m_authAddr;
		bdGetHostByName *m_authGetHostByName;

		bdAuthService *m_authService;
	
		bdInetAddr m_lsgAddr;
		bdGetHostByName *m_lsgGetHostByName;

		bdString m_userName;

		bdRemoteTask *m_remoteTask;

		bdCheckTeamMembershipResult m_memberships;

		bdGetTeamMembersResult m_teamMembers;

		bdCheckTeamMembershipResult::bdTeam m_myTeam;

		bdTeamsResult<PublicTeamInfo> m_publicTeamInfo;

		bdTeamsResult<PrivateTeamInfo> m_privateTeamInfo;

		bdTeamsResult<PublicTeamMemberInfo> m_publicTeamMemberInfo;

		bdTeamsResult<PrivateTeamMemberInfo> m_privateTeamMemberInfo;

		bdGetTeamsProposalListResult m_proposals;

		bdCreateTeamResult m_createResult;

		bdString m_messageText;
		bdStopwatch m_messageTimer;

		bdBool m_pumpLobbyConnection;
		bdLobbyErrorCode m_lobbyErrorCode;
};

#endif // SIMULATION_H
