	// DWBS ------------------------------------------------------------------ DWBS
//
// PLATFORM		: ALL
// PRODUCT		: BIT_DEMON
// VISIBILITY	: PUBLIC
//
// DWBS ------------------------------------------------------------------ DWBS

#include <bdLobby/bdKeyArchive/bdKeyArchive.h>
#include <bdLobby/bdRemoteTaskManager/bdRemoteTaskManager.h>

#define BD_LOG_LEVEL ("lobby/bdKeyArchive")

bdKeyArchive::bdKeyArchive(bdRemoteTaskManager *const remoteTaskManager)
: m_remoteTaskManager(remoteTaskManager),
  m_errorCode(BD_NO_ERROR)
{
}

bdKeyArchive::~bdKeyArchive()
{
	m_remoteTaskManager = BD_NULL;
}

bdRemoteTask* bdKeyArchive::write(const bdKeyPairDict &entries)
{
	bdBitBufferRef buffer = new bdBitBuffer(0, true);
	bdUByte8 controllerIndex = 0;
	buffer->writeUByte8(controllerIndex);

	entries.serialize(buffer, false);

	// start task
	bdRemoteTask *task = BD_NULL;

	m_errorCode = m_remoteTaskManager->startTask(&task,
												 BD_KEY_ARCHIVE_SERVICE,
												 BD_KEYARCHIVE_WRITE_PAIRS,
												 *buffer);
	return task;
}

bdRemoteTask* bdKeyArchive::read(const bdKeyPairDict &entries,
								 const bdBool readDedicated)
{
	bdBitBufferRef buffer = new bdBitBuffer(0, true);
	bdUByte8 controllerIndex = 0;
	buffer->writeUByte8(controllerIndex);

	buffer->writeBool(readDedicated);

	entries.serialize(buffer, true);

	// start task
	bdRemoteTask *task = BD_NULL;

	m_errorCode = m_remoteTaskManager->startTask(&task,
												 BD_KEY_ARCHIVE_SERVICE,
												 BD_KEYARCHIVE_READ_PAIRS,
												 *buffer);
	return task;
}

bdRemoteTask* bdKeyArchive::readAll(const bdEntityID entityID,
									const bdBool isTeam,
									const bdBool readDedicated)
{
	bdBitBuffer buffer(0, true);
	bdUByte8 controllerIndex = 0;
	buffer.writeUByte8(controllerIndex);

	buffer.writeBool(readDedicated);
	buffer.writeBool(isTeam);

	buffer.writeUInt64(entityID);

	// start task
	bdRemoteTask *task = BD_NULL;

	m_errorCode = m_remoteTaskManager->startTask(&task,
												 BD_KEY_ARCHIVE_SERVICE,
												 BD_KEYARCHIVE_READ_ALL_PAIRS,
												 buffer);
	return task;
}

bdRemoteTask* bdKeyArchive::remove(const bdKeyPairDict &entries)
{
	bdBitBufferRef buffer = new bdBitBuffer(0, true);
	bdUByte8 controllerIndex = 0;
	buffer->writeUByte8(controllerIndex);

	entries.serialize(buffer, true);

	// start task
	bdRemoteTask *task = BD_NULL;

	m_errorCode = m_remoteTaskManager->startTask(&task,
												 BD_KEY_ARCHIVE_SERVICE,
												 BD_KEYARCHIVE_DELETE_PAIRS,
												 *buffer);
	return task;
}

bdRemoteTask* bdKeyArchive::removeAll(const bdEntityID entityID,
									  const bdBool isTeam)
{
	bdBitBufferRef buffer = new bdBitBuffer(0, true);
	bdUByte8 controllerIndex = 0;
	buffer->writeUByte8(controllerIndex);

	buffer->writeUInt64(entityID);
	buffer->writeBool(isTeam);

	// start task
	bdRemoteTask *task = BD_NULL;

	m_errorCode = m_remoteTaskManager->startTask(&task,
												 BD_KEY_ARCHIVE_SERVICE,
												 BD_KEYARCHIVE_DELETE_USERS_PAIRS,
												 *buffer);
	return task;
}

bdBool bdKeyArchive::handleTask(const bdRemoteTask *remoteTask,
							    bdQueryResult *queryResult)
{
	bdBool ok = false;
/*
#define BD_KEYARCHIVE_WRITE_PAIRS			(1u)
#define BD_KEYARCHIVE_DELETE_PAIRS			(2u)
#define BD_KEYARCHIVE_READ_PAIRS			(3u)
#define BD_KEYARCHIVE_ENUMERATE_KEY_PAIRS	(4u)
#define BD_KEYARCHIVE_DELETE_USERS_PAIRS	(5u)
*/

	bdBitBufferRef buffer = remoteTask->getResults();
	bdUInt32 taskErrorCode = BD_MAX_ERROR_CODE;
	ok = buffer->readUInt32(taskErrorCode);
	if (ok)
	{
		m_errorCode = static_cast<bdLobbyErrorCode>(taskErrorCode);
		if (m_errorCode == BD_NO_ERROR)
		{
			bdUByte8 operation;
			ok = buffer->readUByte8(operation);

			if(ok)
			{
				switch (operation)
				{
					case BD_KEYARCHIVE_WRITE_PAIRS:
					case BD_KEYARCHIVE_DELETE_PAIRS:
					case BD_KEYARCHIVE_DELETE_USERS_PAIRS:
					{
						// no results here, just task completed
					}
					break;
					case BD_KEYARCHIVE_READ_ALL_PAIRS:
					{
						if (ok
							&& queryResult != BD_NULL)
						{
							ok = queryResult->handleResult(1, buffer);
						}
					}
					break;
					case BD_KEYARCHIVE_READ_PAIRS:
					{
						// the number of entityID's for which keypairs have been returned
						bdUInt32 numEntityIDs = 0;
						ok = buffer->readUInt32(numEntityIDs);
						if (ok
							&& numEntityIDs > 0
							&& queryResult != BD_NULL)
						{
							ok = queryResult->handleResult(numEntityIDs, buffer);
						}
					}
					break;
					default:
					{
						BD_WARN("warn", "returned operations opID doesn't match existing opIDs");
					}
				}
			}
		}
#if defined BD_DEBUG
		else
		{
			bdUInt64 transactionID;
			bdUInt64 connectionID;

			connectionID = m_remoteTaskManager->getConnectionID();
			transactionID = remoteTask->getTransactionID();

			// here we originally called bdRemoteTaskManager::obtainTransactionID
			BD_WARN(BD_LOG_LEVEL, "Transaction ID " BD_UINT_64_FORMAT ":" BD_UINT_64_FORMAT " failed.",
				connectionID, transactionID);
		}
#endif // BD_DEBUG
	}
	return (m_errorCode == BD_NO_ERROR && ok);
}

bdLobbyErrorCode bdKeyArchive::getErrorCode() const
{
	return m_errorCode;
}
