// DWBS ------------------------------------------------------------------ DWBS
//
// PLATFORM		: ALL
// PRODUCT		: BIT_DEMON
// VISIBILITY	: PUBLIC
//
// DWBS ------------------------------------------------------------------ DWBS
// PURPOSE:

BD_LOBBY_ERROR_CODES_BEGIN

	// Indicates a successful operation.
	BD_LOBBY_ERROR_CODE(BD_NO_ERROR, 					0)

	//
	// Client-side-only errors
	//

	/// Maximum number of concurrent remote tasks exceeded
	BD_LOBBY_ERROR_CODE(BD_TOO_MANY_TASKS, 				1)

	/// Not connected to the Lobby Service
	BD_LOBBY_ERROR_CODE(BD_NOT_CONNECTED,  				2)

	/// Sending of message failed
	BD_LOBBY_ERROR_CODE(BD_SEND_FAILED,    				3)

	/// The handling of the task failed
	BD_LOBBY_ERROR_CODE(BD_HANDLE_TASK_FAILED,			4)
	
	//
	// General lobby service gateway errors
	//

	/// The result returned by the lobby server exceeds the client's buffer size
	/// The default size of the receive buffer used by bdLobbyService is 64K. 
	/// Larger buffer sizes can be specified using bdLobbyService::connect().
	BD_LOBBY_ERROR_CODE(BD_RESULT_EXCEEDS_BUFFER_SIZE,	100)
	
	/// Access to requested resource denied.
	BD_LOBBY_ERROR_CODE(BD_ACCESS_DENIED,				101)
	
	/// General Exception. Please contact Demonware support
	BD_LOBBY_ERROR_CODE(BD_EXCEPTION_IN_DB,				102)
	
	/// Tried to read/write to a non-existent row in the DB.
	BD_LOBBY_ERROR_CODE(BD_INVALID_ROW,					104)

	/// The parameters passed to the Lobby Service are not valid.
    /// Please ensure that you are serializing the correct number and types 
	/// of arguments for any customized services
	BD_LOBBY_ERROR_CODE(BD_PARAM_PARSE_ERROR,	 		106)

	/// Param of incorrect type sent. For example sending a bdInt32 to a task
	/// that expects a bdUInt32.
	BD_LOBBY_ERROR_CODE(BD_PARAM_MISMATCHED_TYPE,		107)

	/// Service not available for this titleID. Please contact Demonware support
	BD_LOBBY_ERROR_CODE(BD_SERVICE_NOT_AVAILABLE, 		108)
	
	/// The connection to the Authentication Service failed
	BD_LOBBY_ERROR_CODE(BD_CONNECTION_RESET,		 	109)

	/// LSG rejected the authentication ticket, because it was out of date, or invalid.
	BD_LOBBY_ERROR_CODE(BD_INVALID_USER_ID,			 	110)

	/// LSG does not understand this client's protocol version. Can indicate a mismatch
	/// between versions of LSG and client code.
	BD_LOBBY_ERROR_CODE(BD_LOBBY_PROTOCOL_VERSION_FAILURE, 111)

	/// LSG encountered an unexpected error. Please contact DemonWare support if the error
	/// persists.
	BD_LOBBY_ERROR_CODE(BD_LOBBY_INTERNAL_FAILURE,		112)

	/// Protocol error occurred.
	BD_LOBBY_ERROR_CODE(BD_LOBBY_PROTOCOL_ERROR,		113)


	//
	// Teams service errors
	//

	/// No outstanding proposal exists for team membership for this user 
	/// Returned by bdTeams::acceptMembership or bdTeams::rejectMembership
	/// This can be caused by supplying incorrect parameters or when an existing
	/// proposal has been withdrawn using bdTeams::withdrawProposal
	BD_LOBBY_ERROR_CODE(BD_MEMBER_NO_PROPOSAL,			300)
	
	/// The team name supplied is already in use
	/// Returned by bdTeams::createTeam or bdTeams::updateTeam
	BD_LOBBY_ERROR_CODE(BD_TEAMNAME_ALREADY_EXISTS,		301)

	/// Multiple team membership is not supported for this titleID
	/// Returned by bdTeams::createTeam or bdTeams::acceptMembership
	/// Multiple team membership can be enabled or disabled by contacting Demonware support
	BD_LOBBY_ERROR_CODE(BD_NO_MULTIPLE_TEAM_MEMBERSHIP,	302)

	/// Multiple team ownership is not supported for this titleID
	/// Returned by bdTeams::createTeam
	/// Multiple team ownership can be enabled or disabled by contacting Demonware support
	BD_LOBBY_ERROR_CODE(BD_NO_MULTIPLE_TEAM_OWNERSHIP,	303)

	/// Operation not permitted by this user
	/// Returned by bdTeams::leaveTeam, bdTeams::getPrivateInfo, bdTeams::getPrivateMemberInfo 
	/// bdTeams::sendTeamMemberMessage, bdTeams::inviteToSession, bdTeams::respondToInvite
	BD_LOBBY_ERROR_CODE(BD_NOT_A_TEAM_MEMBER,			304)

	/// Incorrect TeamID supplied
	/// Returned by bdTeams::updateTeam bdTeams::promoteMemberToAdmin bdTeams::demoteMember
	/// bdTeams::kickMember bdTeams::leaveTeam bdTeams::proposeMembership 
	/// bdTeams::rejectMembership bdTeams::acceptMembership
	BD_LOBBY_ERROR_CODE(BD_INVALID_TEAM_ID,				306)

	/// Incorrect Team Name supplied
	/// Returned by bdTeams::getTeamID
	BD_LOBBY_ERROR_CODE(BD_INVALID_TEAM_NAME,			307)
	
	/// No session invite exists for this user
	/// Returned by bdTeams::acceptInvitation
	BD_LOBBY_ERROR_CODE(BD_MEMBER_NOT_INVITED,			309)
	
	/// Operation not permitted by this user
	/// Returned by bdTeams::promoteMemberToAdmin, bdTeams::demoteMember
	BD_LOBBY_ERROR_CODE(BD_NOT_A_TEAM_OWNER,			310)
	
	/// Operation not permitted by this user
	/// Returned by bdTeams::proposeMembership bdTeams::sendTeamAdminMessage
	BD_LOBBY_ERROR_CODE(BD_NOT_AN_ADMIN_OR_OWNER,		311)
	
	/// A membership proposal already exists for this user
	/// Returned by bdTeams::proposeMembership
	BD_LOBBY_ERROR_CODE(BD_MEMBER_PROPOSAL_EXISTS,		312)
	
	/// The proposed user is already a team member
	/// Returned by bdTeams::proposeMembership
	BD_LOBBY_ERROR_CODE(BD_MEMBER_EXISTS,				313)

	/// The team is full and no additional members can join.
	/// The option can be configured. Default: 500 members.
	BD_LOBBY_ERROR_CODE(BD_TEAM_FULL,					314)

	//
	// Stats service errors
	//

	/// Invalid Leaderboard ID supplied
	BD_LOBBY_ERROR_CODE(BD_INVALID_LEADERBOARD_ID,		400)

	/// Invalid Stats Query ID supplied
	BD_LOBBY_ERROR_CODE(BD_INVALID_STATS_QUERY_ID,		401)
	
	/// Stats Values and data do not match.
	/// total length(bdStatsInfo **params) must = numPlayers * numStatsPerPlayer
	BD_LOBBY_ERROR_CODE(BD_INVALID_STATS_SET,			402)
		
	// When arbitrated stats write uses a nonce we don't recognise
	BD_LOBBY_ERROR_CODE(BD_INVALID_SESSION_NONCE,		403)
	
	// The arbitrated session is over, but user tries to write stats.
	BD_LOBBY_ERROR_CODE(BD_SESSION_ENDED,				404)


	/// Title ID supplied is not configured on the LSG.
	BD_LOBBY_ERROR_CODE(BD_INVALID_TITLE_ID,			500)

	

	//
	// Matchmaking service errors
	//

	/// Invalid MatchMaking Query ID supplied.
	/// Returned by bdMatchMaking::executeCustomQuery
	BD_LOBBY_ERROR_CODE(BD_INVALID_QUERY_ID,			501)

	/// Invalid session specified 
	/// Returned by bdMatchMaking::updateSession
	BD_LOBBY_ERROR_CODE(BD_NO_ENTRY_TO_UPDATE,			502)

	//
	// Authentication service errors.
	//
	
	/// No error occured.
	/// This is returned when the authentication operation succeeded.
	BD_LOBBY_ERROR_CODE(BD_AUTH_NO_ERROR,					700)

	/// The authentication request was malformed.
	/// This is returned when the authentication service could not parse the 
	/// request correctly.
	/// This may occur if an incorrect version of the client code is used
	/// to access the authentication service.
	BD_LOBBY_ERROR_CODE(BD_AUTH_BAD_REQUEST,				701)

	/// A generic authentication service error.
	/// Please contact DemonWare support.
	BD_LOBBY_ERROR_CODE(BD_AUTH_SERVER_CONFIG_ERROR,		702)

	/// The authentication service did not recognize the title ID that was 
	/// used to access it with.
	BD_LOBBY_ERROR_CODE(BD_AUTH_BAD_TITLE_ID,				703)

	/// The authentication service did not recognize the user ID, username, or 
	/// license ID that was used.
	/// The most common cause of this is an unknown username being passed to
	/// bdAuthService::authorizeAccount.
	BD_LOBBY_ERROR_CODE(BD_AUTH_BAD_ACCOUNT,				704)

	/// An attempt was made to perform an authentication operation that is
	/// not configured for the given title ID.
	/// For example, it is not possible to use licenses with the authentication
	/// service if they are not configured for the title ID. Similarly, use
	/// of bdAuthService::authorizeAnonymous requires special configuration.
	BD_LOBBY_ERROR_CODE(BD_AUTH_ILLEGAL_OPERATION,			705)

	/// An incorrect license code was passed to the 
	/// bdAuthService::authorizeDedicatedHost function.
	BD_LOBBY_ERROR_CODE(BD_AUTH_INCORRECT_LICENSE_CODE,		706)

	/// The username that was passed to bdAuthService::createAccount already exists.
	BD_LOBBY_ERROR_CODE(BD_AUTH_CREATE_USERNAME_EXISTS,		707)

	/// The username that was passed to bdAuthService::createAccount had an 
	/// invalid format.
	/// By default, usernames must be between 6 and 20 characters long, and only
	/// contain alphanumeric characters and the characters ".-_".
	/// This can be modified on a per-title ID basis, in consultation with 
	/// DemonWare support. If you would like to modify this, please submit a 
	/// regular expression for the format you require. The default regular
	/// expression is as follows:  '^([A-Za-z0-9_\\.\\-]{6,20})$'
	BD_LOBBY_ERROR_CODE(BD_AUTH_CREATE_USERNAME_ILLEGAL,	708)

	/// The username that was passed to bdAuthService::createAccount did not
	/// pass the authentication service vulgarity filter.
	/// The authentication service performs a sub-string match for common
	/// vulgarities on the username. 
	/// This can be modified on a per-title ID basis, in consultation with 
	/// DemonWare support. To obtain the default vulgarity file, please contact
	/// DemonWare support.
	BD_LOBBY_ERROR_CODE(BD_AUTH_CREATE_USERNAME_VULGAR,		709)

	/// A call to bdAuthService::createAccount failed because the maximum 
	/// number of accounts for the given license code already exist.
	/// This is specific to titles that implement license management.
	/// The default maximum number of accounts per license code is 4. 
	/// This can be modified on a per-title ID basis, in consultation with 
	/// DemonWare support.
	BD_LOBBY_ERROR_CODE(BD_AUTH_CREATE_MAX_ACC_EXCEEDED,	710)

	/// An attempt to migrate accounts from one title ID to another failed
	/// because the title IDs are not configured to support migration.
	/// Account migration is a feature available to titles that implement
	/// license management (account sharing is available for those that don't).
	/// Account migration is only available between title IDs that have been
	/// explicitly configured to support migration of each other's accounts.
	BD_LOBBY_ERROR_CODE(BD_AUTH_MIGRATE_NOT_SUPPORTED,		711)

	/// An attempt was made to access a title ID that has been disabled on the
	/// authentication service.
	/// This will occur when the online service for a title ID has been
	/// discontinued.
	BD_LOBBY_ERROR_CODE(BD_AUTH_TITLE_DISABLED,				712)

	/// A license code that was used to perform an authentication operation has
	/// expired and cannot be used.
	/// This error may occur if license management is being used and time-limited
	/// licenses has been provided to users, for example, with time-limited demo
	/// versions of the game.
	BD_LOBBY_ERROR_CODE(BD_AUTH_ACCOUNT_EXPIRED,			713)

	/// A license code that was used to perform an authentication operation has
	/// been locked and cannot be used.
	BD_LOBBY_ERROR_CODE(BD_AUTH_ACCOUNT_LOCKED,				714)

	/// A generic authentication service error.
	/// This may occur if an incorrect version of the client code is used
	/// to access the authentication service.
	/// Please contact DemonWare support.
	BD_LOBBY_ERROR_CODE(BD_AUTH_UNKNOWN_ERROR,				715)

	/// The password that was passed to bdAuthService::authorizeAccount is
	/// incorrect.
	BD_LOBBY_ERROR_CODE(BD_AUTH_INCORRECT_PASSWORD,			716)

	//
	// Profile service errors
	//
	
	/// Profile for this user already exists.
	/// Returned by bdProfiles::createProfile
	BD_LOBBY_ERROR_CODE(BD_PROFILE_ALREADY_EXISTS,	800)

	// Returned by get(Public/Private)Info
	BD_LOBBY_ERROR_CODE(BD_NO_PROFILE_INFO_EXISTS,	801)

	//
	// Friends service errors
	//

	/// No outstanding friendship proposal exists for this user
	/// Returned by bdFriends::acceptProposal bdFriends::rejectProposal
	BD_LOBBY_ERROR_CODE(BD_FRIENDSHIP_NOT_REQUSTED,		900)
	
	/// The specified user is not a friend
	/// Returned by bdFriends::removeFriend bdFriends::inviteToSession bdFriends::sendFriendMessage
	BD_LOBBY_ERROR_CODE(BD_NOT_A_FRIEND,				901)
	
	/// No outstanding session invite exists for this user
	/// Returned by bdFriends::respondToInvite
	BD_LOBBY_ERROR_CODE(BD_NOT_INVITED_TO_SESSION,		902)
	
	/// User attempted to form friendship with themselves
	/// Returned by bdFriends::proposeFriendship
	BD_LOBBY_ERROR_CODE(BD_SELF_FRIENDSHIP_NOT_ALLOWED,	903)
	
	/// Friendship already exists
	/// Returned by bdFriends::proposeFriendship bdFriends::acceptProposal
	BD_LOBBY_ERROR_CODE(BD_FRIENDSHIP_EXISTS,			904)
	
	/// Friendship proposal already exists
	/// Returned by bdFriends::proposeFriendship
	BD_LOBBY_ERROR_CODE(BD_PENDING_FRIENDSHIP_EXISTS,	905)
	
	/// Operation blocked by the recipient
	/// Returned by bdFriends::proposeFriendship bdFriends::inviteToSession bdFriends::sendFriendMessage
	/// Returned by bdTeams::inviteToSession
	BD_LOBBY_ERROR_CODE(BD_USERID_BANNED,				906)
	
	/// User attempted to interact with a user that has yet to create a profile
	/// A user without a profile will not appear in friends lists or in team member lists
	/// and therefore it is not appropriate that they be allowed to form friendships or join
	/// teams until they have created a profile
	/// Returned by bdFriends::proposeFriendship
	/// Returned by bdTeams::createTeam bdTeams::proposeMembership
	BD_LOBBY_ERROR_CODE(BD_NO_PROFILE_MATCHES_USERID,	907)
	
	/// User attempted to block themselves
	/// Returned by bdFriends::setBlockedStatus
	BD_LOBBY_ERROR_CODE(BD_SELF_BLOCK_NOT_ALLOWED,		908)

	/// The user has the max amount of friends allowed. No additional friendships can be made.
	/// The option can be configured. Default: 500 friends.
	BD_LOBBY_ERROR_CODE(BD_FRIENDS_FULL,				909)

	//
	// Storage service errors
	//

	/// The file specified does not exist
	/// Returned by bdStorage::deleteFile bdStorage::getFile
	BD_LOBBY_ERROR_CODE(BD_NO_FILE,					1000)
	
	/// User attempted to download a private file of another user
	/// Returned by bdStorage::getFile
	BD_LOBBY_ERROR_CODE(BD_PERMISSION_DENIED,		1001)

	/// Uploaded file exceeds the maximum size allowed 
	/// Returned by bdStorage::uploadFile
	BD_LOBBY_ERROR_CODE(BD_FILESIZE_LIMIT_EXCEEDED,	1002)

	//
	// Channel chat service errors
	//
	
	/// The specified channel does not exist
	/// Returned by bdChannelChat::subscribeToChannel bdChannelChat::sendChannelMessage 
	/// bdChannelChat::sendWhisperMessage
	BD_LOBBY_ERROR_CODE(BD_CHANNEL_DOES_NOT_EXIST,		1101)

	/// User already subscribed to this channel
	/// Returned by bdChannelChat::subscribeToChannel
	BD_LOBBY_ERROR_CODE(BD_CHANNEL_ALREADY_SUBSCRIBED,	1102)

	/// User not subscribed to this channel
	/// Returned by bdChannelChat::unsubscribeToChannel bdChannelChat::sendChannelMessage 
	/// bdChannelChat::sendWhisperMessage bdChannelChat::getChannelMembers
	BD_LOBBY_ERROR_CODE(BD_CHANNEL_NOT_SUBSCRIBED,		1103)

	/// Channel has exceeded maximum number of members
	/// Returned by bdChannelChat::subscribeToChannel	
	BD_LOBBY_ERROR_CODE(BD_CHANNEL_FULL,				1104)

	//
	// Title Utilities service errors
	//

	/// No ad matched the specified search criteria
	/// Returned by bdTitleUtilities::getAdUnit
	BD_LOBBY_ERROR_CODE(BD_UTILITY_NO_MATCHING_UNIT,	1200)


	//
	// MessageChannel chat service errors
	//
	
	/// Attemped to send a message to a teamID which doesn't exist.
	BD_LOBBY_ERROR_CODE(BD_MESSAGING_NO_TEAM,			  1300)
	
	/// Tried to send a message to more than 10 userIDs in the same call.
	BD_LOBBY_ERROR_CODE(BD_MESSAGING_SEND_QUOTA_EXCEEDED, 1301)

	/// Attempted to send to a team (isTeam specified as True)
	/// and provided more than one teamID.
	/// Only one teamID may be sent to per sendMessage call.
	BD_LOBBY_ERROR_CODE(BD_MESSAGING_INVALID_TEAM,		  1302)

BD_LOBBY_ERROR_CODES_END(BD_MAX_ERROR_CODE)
