// DWBS ------------------------------------------------------------------ DWBS
//
// PLATFORM		: ALL
// PRODUCT		: BIT_DEMON
// VISIBILITY	: PUBLIC
//
// DWBS ------------------------------------------------------------------ DWBS

// PURROSE: A singleton to create instances of received lobby messages

#ifndef BD_LOBBY_MESSAGE_FACTORY_H
#define BD_LOBBY_MESSAGE_FACTORY_H

#include <bdCore/bdCore.h>
#include <bdCore/bdUtilities/bdFactory.h>

#include <bdLobby/bdMessageStore/bdLobbyMessage.h>

/// A factory class to create instances of a class derived from bdLobbyMessage.
class bdLobbyMessageFactoryImpl : public bdTypedFactory<bdLobbyMessage>
{
	protected:

		BD_DECLARE_NEW_AND_DELETE_OPERATORS

		/// Grant friendship to bdSingleton<bdLobbyMessageFactoryImpl> so that it 
		/// can construct the singleton instance of this class.
		friend class bdSingleton<bdLobbyMessageFactoryImpl>;

		/// Default constructor. 
		/// This is protected so that this class cannot be instantiated, except
		/// as a singleton.
		bdLobbyMessageFactoryImpl();

		/// Destructor.
		~bdLobbyMessageFactoryImpl();
};

/// This class provides a point of access to the singleton instance
/// of bdLobbyMessageFactoryImpl. Implementation details can be found in 
/// bdLobbyMessageFactoryImpl.
class bdLobbyMessageFactory : public bdSingleton<bdLobbyMessageFactoryImpl>
{
	protected:

		BD_DECLARE_NEW_AND_DELETE_OPERATORS

		/// Protected constructor.
		bdLobbyMessageFactory();
};

//bdLobbyMessageFactory::getInstance()->registerClass(TYPE_ID, new bdCreator< CLASS, bdLobbyMessage >);
/// Macro to register a class derived from bdLobbyMessage with the bdLobbyMessageFactory.

#define BD_REGISTER_LOBBY_MESSAGE_TYPE(MSG_TYPE, DERIVED_CLASS) \
{ \
	bdTypedCreator< DERIVED_CLASS, bdLobbyMessage > *const creator = new bdTypedCreator< DERIVED_CLASS, bdLobbyMessage >; \
	if(creator) \
	{ \
		if(!bdLobbyMessageFactory::getInstance()->registerClass(MSG_TYPE, creator)) \
		{ \
			delete creator; \
		} \
	} \
}

#endif // BD_LOBBY_MESSAGE_FACTORY_H
