// DWBS ------------------------------------------------------------------ DWBS
//
// PLATFORM		: ALL
// PRODUCT		: BIT_DEMON
// VISIBILITY	: PUBLIC
//
// DWBS ------------------------------------------------------------------ DWBS

// PURPOSE:

#include <bdLobby/bdRemoteTaskManager/bdRemoteTaskManager.h>

#include <bdLobby/bdLobbyCommon/bdLobbyServicesCommon.h>

#include <bdLobby/bdLobbyConnection.h>

#define BD_LOG_LEVEL "lobby/remotetaskmanager"


bdRemoteTaskManager::bdRemoteTaskManager(bdLobbyConnectionRef connection,
										 const bdBool useEncryption )
: m_lobbyConnection(connection),
  m_encryptedConnection(useEncryption),
  m_connectionID(0)
{
}

bdRemoteTaskManager::~bdRemoteTaskManager()
{
}

bdLobbyErrorCode bdRemoteTaskManager::startTask(bdRemoteTask **task,
												const bdUByte8 serviceID,
												const bdUByte8 taskID,
												bdBitBuffer &queryParams,
												const bdFloat32 timeout)
{
	bdLobbyErrorCode error = BD_NO_ERROR;
	bdRemoteTask *newTask = BD_NULL;

	newTask = new bdRemoteTask();
	

	// All messages sent from the task manager are type checked
	//messageBuffer = new bdBitBuffer(1, true);
	bdBitBufferRef messageBuffer = new bdBitBuffer(&serviceID, 8, true); //msg->getPayload();
	messageBuffer->setTypeCheck(false);
	messageBuffer->writeBool(true);
	messageBuffer->setTypeCheck(true);


	// Write the ID of the remote operation
	messageBuffer->writeUByte8(taskID);
	// serialize params in queryParam
	messageBuffer->append(queryParams);

	// FIXME This is a workaround for an issue where a blob as the last item
	// in the buffer cause problems on the server.
	const bdUInt32 x = 0;
	messageBuffer->writeBits(&x,32);

	const bdBool sent = m_lobbyConnection->send(messageBuffer->getData(),messageBuffer->getDataSize(), m_encryptedConnection);
	if(!sent)
	{
		error = BD_SEND_FAILED;
	}

	if(error == BD_NO_ERROR)
	{
		m_tasks.addTail(newTask);
		newTask->start(timeout);
		*task = newTask;
	}

	return error;
}




void bdRemoteTaskManager::handleTaskReply(bdBitBufferRef buffer)
{
	
	// get the transactionID 
	bdUInt64 transactionID = 0;
	bdBool ok = buffer->readUInt64(transactionID);

	bdRemoteTask *task = m_tasks.getHead();

	if(BD_NULL != task && ok)
	{
		m_tasks.removeHead();
		task->stop(buffer, transactionID);
	}
	else
	{
		BD_WARN(BD_LOG_LEVEL,"Task retrieval failed.");
	}

	BD_INFO(BD_LOG_LEVEL, "Received reply from transaction ID " BD_UINT_64_FORMAT ":" BD_UINT_64_FORMAT, m_connectionID, transactionID);
}


bdUInt64 bdRemoteTaskManager::getConnectionID() const
{
	return m_connectionID;
}


