// DWBS ------------------------------------------------------------------ DWBS
//
// PLATFORM		: ALL
// PRODUCT		: BIT_DEMON
// VISIBILITY	: PUBLIC
//
// DWBS ------------------------------------------------------------------ DWBS


#ifndef BD_STAT_WRITE_MULTI_STATS_RESULT
#define BD_STAT_WRITE_MULTI_STATS_RESULT

#include <bdCore/bdCore.h>
#include <bdLobby/bdLobbyCommon/bdQueryParams.h>
#include <bdLobby/bdLobbyCommon/bdQueryResult.h>
#include <bdLobby/bdLobbyCommon/bdLobbyServicesCommon.h>



/// Maximum number of entityIDs expected from the server in writeMultiStats result.
#define BD_STATS_MAX_WRITE_ENTITIES		(1000u)

/// Result of a \a bdStats::writeMultipleStats() call.
/// This lists all bdOnlineUserIDs and bdTeamIDs for which statistics
/// were written. Both lists never contain duplicate entries.
class bdWriteMultiStatsResult : public bdQueryResult
{
	public:

		BD_DECLARE_NEW_AND_DELETE_OPERATORS

		/// Default constructor
		bdWriteMultiStatsResult();

		/// Virtual destructor.
		virtual ~bdWriteMultiStatsResult();

		/// Handles a result set for a title specific query.
		/// \param numRows[in] : The number of rows in the result set.
		/// \param values[in] : The actual values of each data field.
		virtual bdBool handleResult(const bdUInt /*numResults*/, 
			bdBitBufferRef buffer);

		/// Non-duplicate list of users whose stats were successfully written.
		bdFastArray<bdOnlineUserID> m_usersWritten;

		/// Non-duplicate list of teams whose stats were successfully written.
		bdFastArray<bdEntityID> m_teamsWritten;
};


#endif // BD_STAT_WRITE_MULTI_STATS_RESULT
