// DWBS ------------------------------------------------------------------ DWBS
//
// PLATFORM		: ALL
// PRODUCT		: BIT_DEMON
// VISIBILITY	: PUBLIC
//
// DWBS ------------------------------------------------------------------ DWBS

#include <bdLobby/bdTeams/bdTeams.h>
#include <bdLobby/bdTeams/bdTeamInfo.h>
#include <bdLobby/bdTeams/bdTeamMemberInfo.h>
#include <bdLobby/bdRemoteTaskManager/bdRemoteTaskManager.h>
#include <bdLobby/bdLobbyCommon/bdQueryResult.h>

#define BD_LOG_LEVEL ("lobby/teams")

bdTeams::bdTeams(bdRemoteTaskManager *const remoteTaskManager)
: m_remoteTaskManager(remoteTaskManager),
  m_errorCode(BD_NO_ERROR)
{
}


bdTeams::~bdTeams()
{
	m_remoteTaskManager = BD_NULL;
}


bdRemoteTask* bdTeams::getTeamID(bdUByte8 /*controllerIndex*/, 
								 const bdNChar8 teamName[BD_MAX_TEAMNAME_LENGTH])
{
	bdRemoteTask *remoteTask = BD_NULL;
	bdBitBuffer buffer (0,true);
	buffer.writeUByte8(0);
	buffer.writeString(teamName, BD_MAX_TEAMNAME_LENGTH);

	m_errorCode = m_remoteTaskManager->startTask(&remoteTask,
												BD_TEAM_MANAGEMENT_SERVICE,
												BD_TEAM_GET_TEAMID,
												buffer);

	return remoteTask;
}

bdRemoteTask* bdTeams::createTeam(bdUByte8 /*controllerIndex*/, 
								  const bdNChar8 *teamName)
{
	bdRemoteTask *remoteTask = BD_NULL;
	bdBitBuffer buffer (0,true);
	buffer.writeUByte8(0);
	buffer.writeString(teamName, BD_MAX_TEAMNAME_LENGTH);

	m_errorCode = m_remoteTaskManager->startTask(&remoteTask,
												BD_TEAM_MANAGEMENT_SERVICE,
												BD_TEAM_CREATE,
												buffer);

	return remoteTask;
}

bdRemoteTask* bdTeams::updateTeam(bdUByte8 /*controllerIndex*/, 
								  bdTeamID teamID, 
								  const bdNChar8 *teamName)
{
	bdRemoteTask *remoteTask = BD_NULL;
	bdBitBuffer buffer (0,true);
	buffer.writeUByte8(0);
	buffer.writeUInt64(teamID);
	buffer.writeString(teamName, BD_MAX_TEAMNAME_LENGTH);

	m_errorCode = m_remoteTaskManager->startTask(&remoteTask,
												BD_TEAM_MANAGEMENT_SERVICE,
												BD_TEAM_UPDATE,
												buffer);
	return remoteTask;
}

bdRemoteTask* bdTeams::promoteMember(bdUByte8 /*controllerIndex*/, 
									 bdTeamID teamID, 
									 bdOnlineUserID userID)
{
	bdRemoteTask *remoteTask = BD_NULL;
	bdBitBuffer buffer (0,true);
	buffer.writeUByte8(0);
	buffer.writeUInt64(teamID);
	buffer.writeUInt64(userID);

	m_errorCode = m_remoteTaskManager->startTask(&remoteTask,
												BD_TEAM_MANAGEMENT_SERVICE,
												BD_TEAM_PROMOTE_MEMBER_TO_ADMIN,
												buffer);
	return remoteTask;

}


bdRemoteTask* bdTeams::promoteMemberToOwner(bdUByte8 /*controllerIndex*/, 
											bdTeamID teamID, 
											bdOnlineUserID userID)
{
	bdRemoteTask *remoteTask = BD_NULL;
	bdBitBuffer buffer (0,true);
	buffer.writeUByte8(0);
	buffer.writeUInt64(teamID);
	buffer.writeUInt64(userID);

	m_errorCode = m_remoteTaskManager->startTask(&remoteTask,
												BD_TEAM_MANAGEMENT_SERVICE,
												BD_TEAM_PROMOTE_MEMBER_TO_OWNER,
												buffer);
	return remoteTask;

}

bdRemoteTask* bdTeams::demoteMember(bdUByte8 /*controllerIndex*/, 
									 bdTeamID teamID, 
									 bdOnlineUserID userID)
{
	bdRemoteTask *remoteTask = BD_NULL;
	bdBitBuffer buffer (0,true);
	buffer.writeUByte8(0);
	buffer.writeUInt64(teamID);
	buffer.writeUInt64(userID);

	m_errorCode = m_remoteTaskManager->startTask(&remoteTask,
		BD_TEAM_MANAGEMENT_SERVICE,
		BD_TEAM_DEMOTE_MEMBER,
		buffer);
	return remoteTask;

}

bdRemoteTask* bdTeams::kickMember(bdUByte8 /*controllerIndex*/, 
										   bdTeamID teamID, 
										   bdOnlineUserID userID)
{
	bdRemoteTask *remoteTask = BD_NULL;
	bdBitBuffer buffer (0,true);
	buffer.writeUByte8(0);
	buffer.writeUInt64(teamID);
	buffer.writeUInt64(userID);

	m_errorCode = m_remoteTaskManager->startTask(&remoteTask,
												BD_TEAM_MANAGEMENT_SERVICE,
												BD_TEAM_KICK_MEMBER,
												buffer);
	return remoteTask;
}

bdRemoteTask* bdTeams::leaveTeam(bdUByte8 /*controllerIndex*/, 
								 bdTeamID teamID,
								 bdOnlineUserID newOwnerID)
{
	bdRemoteTask *remoteTask = BD_NULL;
	bdBitBuffer buffer (0, true);
	buffer.writeUByte8(0);
	buffer.writeUInt64(teamID);
	buffer.writeUInt64(newOwnerID);

	m_errorCode = m_remoteTaskManager->startTask(&remoteTask,
												BD_TEAM_MANAGEMENT_SERVICE,
												BD_TEAM_LEAVE,
												buffer);
	return remoteTask;
}

bdRemoteTask* bdTeams::proposeMembership(bdUByte8 /*controllerIndex*/, 
										bdTeamID teamID, 
										bdOnlineUserID newUserID,
										bdByteBuffer * const attachment)
{
	bdRemoteTask *remoteTask = BD_NULL;
	bdBitBuffer buffer (0,true);
	buffer.writeUByte8(0);
	buffer.writeUInt64(teamID);
	buffer.writeUInt64(newUserID);

	if ( attachment != BD_NULL )
	{
		buffer.writeBlob(attachment->getData(),attachment->getSize());
	}

	m_errorCode = m_remoteTaskManager->startTask(&remoteTask,
												BD_TEAM_MANAGEMENT_SERVICE,
												BD_TEAM_PROPOSE_MEMBERSHIP,
												buffer);
	return remoteTask;
}

bdRemoteTask* bdTeams::rejectMembership(bdUByte8 /*controllerIndex*/, 
										bdTeamID teamID, 
										bdOnlineUserID inviter)
{
	bdRemoteTask *remoteTask = BD_NULL;
	bdBitBuffer buffer (0,true);
	buffer.writeUByte8(0);
	buffer.writeUInt64(teamID);
	buffer.writeUInt64(inviter);

	m_errorCode = m_remoteTaskManager->startTask(&remoteTask,
												BD_TEAM_MANAGEMENT_SERVICE,
												BD_TEAM_REJECT_MEMBERSHIP,
												buffer);
	return remoteTask;
}

bdRemoteTask* bdTeams::acceptMembership(bdUByte8 /*controllerIndex*/, 
										bdTeamID teamID, 
										bdOnlineUserID inviter)
{
	bdRemoteTask *remoteTask = BD_NULL;
	bdBitBuffer buffer (0,true);
	buffer.writeUByte8(0);
	buffer.writeUInt64(teamID);
	buffer.writeUInt64(inviter);

	m_errorCode = m_remoteTaskManager->startTask(&remoteTask,
												BD_TEAM_MANAGEMENT_SERVICE,
												BD_TEAM_ACCEPT_MEMBERSHIP,
												buffer);
	return remoteTask;
}

bdRemoteTask *bdTeams::withdrawProposal(bdUByte8 /*controllerIndex*/, 
										bdOnlineUserID userID,
										bdTeamID teamID)
{
	bdRemoteTask *remoteTask = BD_NULL;

	bdBitBuffer buffer(0, true);
	buffer.writeUByte8(0);
	buffer.writeUInt64(userID);
	buffer.writeUInt64(teamID);

	m_errorCode = m_remoteTaskManager->startTask(&remoteTask,
		BD_TEAM_MANAGEMENT_SERVICE,
		BD_TEAM_WITHDRAW_PROPOSAL,
		buffer);

	return remoteTask;
}

bdRemoteTask *bdTeams::getProposalList(bdUByte8 /*controllerIndex*/)
{
	bdRemoteTask *remoteTask = BD_NULL;

	bdBitBuffer buffer(0, true);
	buffer.writeUByte8(0);
	m_errorCode = m_remoteTaskManager->startTask(&remoteTask,
		BD_TEAM_MANAGEMENT_SERVICE,
		BD_TEAM_GET_PROPOSALS,
		buffer);

	return remoteTask;
}

bdRemoteTask *bdTeams::inviteMember(bdUByte8 /*controllerIndex*/, 
									  bdOnlineUserID memberID,
									  const bdSecurityID &securityID,
									  bdByteBuffer * const attachment)
{
	bdRemoteTask *remoteTask = BD_NULL;
	bdBitBuffer buffer(0, true);
	buffer.writeUByte8(0);
	buffer.writeUInt64(memberID);
	buffer.writeBlob(securityID.ab, BD_SECURITY_ID_LENGTH);
	
	if ( attachment != BD_NULL )
	{
		buffer.writeBlob(attachment->getData(),attachment->getSize());
	}

	m_errorCode = m_remoteTaskManager->startTask(&remoteTask,
		BD_TEAM_MANAGEMENT_SERVICE,
		BD_TEAM_INVITE_TO_SESSION,
		buffer);

	return remoteTask;
}

bdRemoteTask* bdTeams::acceptInvitation(bdUByte8 /*controllerIndex*/, 
										  bdOnlineUserID inviter)
{
	bdRemoteTask *remoteTask = BD_NULL;
	bdBitBuffer buffer(0, true);
	buffer.writeUByte8(0);
	buffer.writeUInt64(inviter);
	m_errorCode = m_remoteTaskManager->startTask(&remoteTask,
		BD_TEAM_MANAGEMENT_SERVICE,
		BD_TEAM_ACCEPT_INVITE,
		buffer);
	return remoteTask;
}

bdRemoteTask* bdTeams::rejectInvitation(bdUByte8 /*controllerIndex*/, 
										  bdOnlineUserID inviter)
{
	bdRemoteTask *remoteTask = BD_NULL;
	bdBitBuffer buffer(0, true);
	buffer.writeUByte8(0);
	buffer.writeUInt64(inviter);
	m_errorCode = m_remoteTaskManager->startTask(&remoteTask,
		BD_TEAM_MANAGEMENT_SERVICE,
		BD_TEAM_REJECT_INVITE,
		buffer);
	return remoteTask;
}


bdRemoteTask* bdTeams::getPublicInfo(bdUByte8 /*controllerIndex*/, 
									 bdTeamID teamID)
{
	bdRemoteTask *remoteTask = BD_NULL;
	bdBitBuffer buffer (0,true);
	buffer.writeUByte8(0);
	buffer.writeUInt64(teamID);

	m_errorCode = m_remoteTaskManager->startTask(&remoteTask,
												BD_TEAM_MANAGEMENT_SERVICE,
												BD_TEAM_GET_PUBLIC_INFO,
												buffer);
	return remoteTask;
}

bdRemoteTask* bdTeams::getPrivateInfo(bdUByte8 /*controllerIndex*/, 
									  bdTeamID teamID)
{
	bdRemoteTask *remoteTask = BD_NULL;
	bdBitBuffer buffer (0,true);
	buffer.writeUByte8(0);
	buffer.writeUInt64(teamID);

	m_errorCode = m_remoteTaskManager->startTask(&remoteTask,
												BD_TEAM_MANAGEMENT_SERVICE,
												BD_TEAM_GET_PRIVATE_INFO,
												buffer);
	return remoteTask;
}

bdRemoteTask* bdTeams::setPublicInfo(bdUByte8 /*controllerIndex*/, 
									 bdTeamID teamID, 
									 bdTeamInfo *const teamInfo)
{
	bdRemoteTask *remoteTask = BD_NULL;
	bdBitBuffer buffer (0,true);
	buffer.writeUByte8(0);
	buffer.writeUInt64(teamID);
	teamInfo->serialize(buffer);

	m_errorCode = m_remoteTaskManager->startTask(&remoteTask,
												BD_TEAM_MANAGEMENT_SERVICE,
												BD_TEAM_SET_PUBLIC_INFO,
												buffer);
	return remoteTask;
}

bdRemoteTask* bdTeams::setPrivateInfo(bdUByte8 /*controllerIndex*/, 
									  bdTeamID teamID, 
									  bdTeamInfo *const teamInfo)
{
	bdRemoteTask *remoteTask = BD_NULL;
	bdBitBuffer buffer (0,true);
	buffer.writeUByte8(0);
	buffer.writeUInt64(teamID);
	teamInfo->serialize(buffer);

	m_errorCode = m_remoteTaskManager->startTask(&remoteTask,
												BD_TEAM_MANAGEMENT_SERVICE,
												BD_TEAM_SET_PRIVATE_INFO,
												buffer);
	return remoteTask;
}

bdRemoteTask* bdTeams::setPublicMemberInfo(bdUByte8 /*controllerIndex*/, 
										   bdTeamID teamID, 
										   bdTeamMemberInfo *const teamMemberInfo)
{
	bdRemoteTask *remoteTask = BD_NULL;
	bdBitBuffer buffer (0,true);
	buffer.writeUByte8(0);
	buffer.writeUInt64(teamID);
	teamMemberInfo->serialize(buffer);

	m_errorCode = m_remoteTaskManager->startTask(&remoteTask,
												BD_TEAM_MANAGEMENT_SERVICE,
												BD_TEAM_SET_PUBLIC_MEMBER_INFO,
												buffer);
	return remoteTask;
}

bdRemoteTask* bdTeams::setPrivateMemberInfo(bdUByte8 /*controllerIndex*/, 
											bdTeamID teamID, 
											bdTeamMemberInfo *const teamMemberInfo)
{
	bdRemoteTask *remoteTask = BD_NULL;
	bdBitBuffer buffer (0,true);
	buffer.writeUByte8(0);
	buffer.writeUInt64(teamID);
	teamMemberInfo->serialize(buffer);

	m_errorCode = m_remoteTaskManager->startTask(&remoteTask,
												BD_TEAM_MANAGEMENT_SERVICE,
												BD_TEAM_SET_PRIVATE_MEMBER_INFO,
												buffer);
	return remoteTask;
}


bdRemoteTask* bdTeams::getPublicMemberInfo(bdUByte8 /*controllerIndex*/, 
										   bdTeamID teamID, 
										   bdOnlineUserID userID)
{
	bdRemoteTask *remoteTask = BD_NULL;
	bdBitBuffer buffer (0,true);
	buffer.writeUByte8(0);
	buffer.writeUInt64(teamID);
	buffer.writeUInt64(userID);

	m_errorCode = m_remoteTaskManager->startTask(&remoteTask,
												BD_TEAM_MANAGEMENT_SERVICE,
												BD_TEAM_GET_PUBLIC_MEMBER_INFO,
												buffer);
	return remoteTask;
}

bdRemoteTask* bdTeams::getPrivateMemberInfo(bdUByte8 /*controllerIndex*/, 
											bdTeamID teamID, 
											bdOnlineUserID userID)
{
	bdRemoteTask *remoteTask = BD_NULL;
	bdBitBuffer buffer (0,true);
	buffer.writeUByte8(0);
	buffer.writeUInt64(teamID);
	buffer.writeUInt64(userID);

	m_errorCode = m_remoteTaskManager->startTask(&remoteTask,
												BD_TEAM_MANAGEMENT_SERVICE,
												BD_TEAM_GET_PRIVATE_MEMBER_INFO,
												buffer);
	return remoteTask;
}

bdRemoteTask* bdTeams::checkMemberships(bdUByte8 /*controllerIndex*/)
{
	bdRemoteTask *remoteTask = BD_NULL;
	bdBitBuffer buffer (0,true);
	buffer.writeUByte8(0);

	m_errorCode = m_remoteTaskManager->startTask(&remoteTask,
												BD_TEAM_MANAGEMENT_SERVICE,
												BD_TEAM_CHECK_MEMBERSHIP,
												buffer);
	return remoteTask;
}

bdRemoteTask*  bdTeams::getMembers(bdUByte8 /*controllerIndex*/,
								   bdUInt64 teamID)
{
	bdRemoteTask *remoteTask = BD_NULL;
	bdBitBuffer buffer (0,true);
	buffer.writeUByte8(0);
	buffer.writeUInt64(teamID);

	m_errorCode = m_remoteTaskManager->startTask(&remoteTask,
												BD_TEAM_MANAGEMENT_SERVICE,
												BD_TEAM_GET_MEMBERS,
												buffer);
	return remoteTask;
}

bdRemoteTask *bdTeams::sendMemberMessage(bdUByte8 /*controllerIndex*/, 
										 bdOnlineUserID teamMemberID,
										 bdByteBuffer& payload,
										 bdMessageDeliveryMethod deliveryMethod)
{
	bdRemoteTask *remoteTask = BD_NULL;
	bdBitBuffer buffer(0, true);
	buffer.writeUByte8(0);
	buffer.writeUInt64(teamMemberID);

	buffer.writeUInt32(static_cast<bdUInt32>(deliveryMethod));

	buffer.writeBlob(payload.getData(),payload.getSize());

	m_errorCode = m_remoteTaskManager->startTask(&remoteTask,
												BD_TEAM_MANAGEMENT_SERVICE,
												BD_TEAM_SEND_MEMBER_MSG,
												buffer);
	return remoteTask;
}

bdRemoteTask *bdTeams::sendAdminMessage(bdUByte8 /*controllerIndex*/, 
										bdTeamID teamID,
										bdByteBuffer& payload,
										bdMessageDeliveryMethod deliveryMethod)
{
	bdRemoteTask *remoteTask = BD_NULL;
	bdBitBuffer buffer(0, true);
	buffer.writeUByte8(0);
	buffer.writeUInt64(teamID);

	buffer.writeUInt32(static_cast<bdUInt32>(deliveryMethod));

	buffer.writeBlob(payload.getData(),payload.getSize());

	m_errorCode = m_remoteTaskManager->startTask(&remoteTask,
												BD_TEAM_MANAGEMENT_SERVICE,
												BD_TEAM_SEND_ADMIN_MSG,
												buffer);
	return remoteTask;
}

bdBool bdTeams::handleTask(const bdRemoteTask *const remoteTask, 
						   bdQueryResult *const queryResult)
{
	bdBool ok;
	bdBitBufferRef buffer = remoteTask->getResults();
	bdUInt32 taskErrorCode = BD_MAX_ERROR_CODE;
	ok = buffer->readUInt32(taskErrorCode);

	if(ok)
	{
		m_errorCode = static_cast<bdLobbyErrorCode>(taskErrorCode);
		if (m_errorCode == BD_NO_ERROR)
		{
			bdUByte8 operation;
			ok = buffer->readUByte8(operation);
			if(ok)
			{
				switch (operation)
				{
					case BD_TEAM_CREATE:
					case BD_TEAM_GET_PUBLIC_INFO:
					case BD_TEAM_GET_PRIVATE_INFO:
					case BD_TEAM_GET_PUBLIC_MEMBER_INFO:
					case BD_TEAM_GET_PRIVATE_MEMBER_INFO:
					case BD_TEAM_GET_TEAMID:
					{
						if(queryResult)
						{
							ok = queryResult->handleResult(1, buffer);
						}
					}
					break;
					case BD_TEAM_CHECK_MEMBERSHIP:
					case BD_TEAM_GET_MEMBERS:
					case BD_TEAM_GET_PROPOSALS:
					{
						if (queryResult)
						{
							bdUInt32 numReplies = 0;
							ok = buffer->readUInt32(numReplies);
							if (ok && numReplies > 0)
							{
								ok = queryResult->handleResult(numReplies, buffer);
							}
						}
					}
					break;
					default:
					{
					}
					break;
				}
			}
		}
#if defined BD_DEBUG
		else
		{

			bdUInt64 transactionID;
			bdUInt64 connectionID;

			connectionID = m_remoteTaskManager->getConnectionID();
			transactionID = remoteTask->getTransactionID();

			// here we originally called bdRemoteTaskManager::obtainTransactionID
			BD_WARN(BD_LOG_LEVEL, "Transaction ID " BD_UINT_64_FORMAT ":" BD_UINT_64_FORMAT " failed.",
				connectionID, transactionID);
		}
#endif // BD_DEBUG
	}

	return (m_errorCode == BD_NO_ERROR && ok);
}

bdLobbyErrorCode bdTeams::getErrorCode() const
{
	return m_errorCode;
}

