// DWBS ------------------------------------------------------------------ DWBS
//
// PLATFORM		: PS2
// PRODUCT		: BIT_DEMON
// VISIBILITY	: PUBLIC
//
// DWBS ------------------------------------------------------------------ DWBS

#include <bdPlatform/bdPlatformSocket/bdPlatformSocket.h>

#if defined BD_PLATFORM_PS2 && defined BD_PS2_USE_NDK

#include <bdPlatform/bdPlatformString/bdPlatformString.h>

#  include <eetypes.h>
#  include <eekernel.h>

#include "snsocket.h"
#  include <sntypes.h>       // SN Systems types
#  include <snsocket.h>      // SN Systems socket API header file

// only protocol supported by ::socket in NDK
#  define IPPROTO_UDP AF_INET

bdInt BD_CALL bdPlatformSocket::create(const bdBool /*broadcast*/)
{
	// create the socket handle
	sockAPIregthr();
	bdInt handle = ::socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);
	return handle;
}

bdSocketStatusCode BD_CALL bdPlatformSocket::bind(bdInt &handle,
												  const bdInAddr &addr,
												  const bdUInt16 port)
{
	if(handle == BD_INVALID_SOCKET_HANDLE)
	{
		return BD_NET_INVALID_HANDLE;
	}

	struct sockaddr_in localAddr;
	// Fill in address for binding locally
	bdMemset(&localAddr, 0, sizeof(localAddr));
	localAddr.sin_family = AF_INET;
	localAddr.sin_addr.s_addr = addr.m_inAddri;
	localAddr.sin_port = ::htons(port);


	if ( ::bind( handle, (struct sockaddr *)&localAddr, sizeof(localAddr) )  == BD_INVALID_SOCKET_HANDLE )
	{
		return BD_NET_ERROR;
	}

	struct sockaddr_in bindAddr;
	// Fill in address for binding locally
	bdMemset(&bindAddr, 0, sizeof(bindAddr));
	bindAddr.sin_family = AF_INET;
	localAddr.sin_addr.s_addr = addr.m_inAddri;
	bindAddr.sin_port = ::htons(port);

	return BD_NET_SUCCESS;
}


bdInt BD_CALL bdPlatformSocket::sendTo(bdInt &handle,
									   const bdInAddr &addr,
									   const bdUInt16 port,
									   const void *data,
									   const bdUInt length)
{
	if(handle == BD_INVALID_SOCKET_HANDLE)
	{
		return BD_NET_INVALID_HANDLE;
	}

	struct sockaddr_in recpt;

	bdMemset(&recpt, 0, sizeof(recpt));
	recpt.sin_family = AF_INET;
	recpt.sin_addr.s_addr = addr.m_inAddri;
	recpt.sin_port = ::htons(port);
	int result = sockAPIregthr();
	bdInt sent = ::sendto(handle,
							reinterpret_cast<const bdChar8 *>(data),
							length,
							0,
							reinterpret_cast<struct sockaddr *>(&recpt),
							sizeof(recpt));

	if(sent < 0)
	{
		const bdInt error = getErrno(handle);
		switch(error)
		{
			case EWOULDBLOCK:
				return BD_NET_WOULD_BLOCK;
			case EINVAL:
				return BD_NET_ADDRESS_INVALID;
			case ECONNRESET:
			case ETIMEDOUT:
				return BD_NET_CONNECTION_RESET;
			case EMSGSIZE:
				return BD_NET_MSG_SIZE;
			default:
				return BD_NET_ERROR;
		}
	}
	else
	{
		m_totalBytesSent += sent + BD_UDP_IP_OVERHEAD;
	}

	return sent;
}

bdInt BD_CALL bdPlatformSocket::receiveFrom(bdInt &handle,
											bdInAddr &addr,
											bdUInt16 &port,
											void *data,
											const bdUInt size)
{
	if(handle == BD_INVALID_SOCKET_HANDLE)
	{
		return BD_NET_INVALID_HANDLE;
	}

	struct sockaddr_in remoteAddr;
	bdMemset(&remoteAddr, 0, sizeof(remoteAddr));

	bdInt remoteAddrLen = sizeof(struct sockaddr_in);

	int result = sockAPIregthr();
	bdInt received = ::recvfrom(handle,
								reinterpret_cast<bdChar8 *>(data),
								size,
								0,
								reinterpret_cast<struct sockaddr *>(&remoteAddr),
								&remoteAddrLen);


	if(received < 0)
	{
		const bdInt error = getErrno(handle);

		switch(error)
		{
			case EWOULDBLOCK:
				return BD_NET_WOULD_BLOCK;
				break;
			case ECONNRESET:
			case ETIMEDOUT:
				{
					port = ::ntohs(remoteAddr.sin_port);
					addr.m_inAddri = remoteAddr.sin_addr.s_addr;
					return BD_NET_CONNECTION_RESET;
				}
				break;
			case EMSGSIZE:
				return BD_NET_MSG_SIZE;
				break;

			case EINVAL:
				// This indicates the socket has not yet been bound (either
				// through bind or a call to sendTo) and we therefore can't
				// receive.
				// This happens as we have scheduled a packet to be sent (causing
				// a bind) but this thread has run before the send thread.
				// return a
				return BD_NET_NOT_BOUND;
				break;
			default:
				return BD_NET_ERROR;
		}
	}
	else
	{
		port = ::ntohs(remoteAddr.sin_port);
		addr.m_inAddri = remoteAddr.sin_addr.s_addr;
		m_totalBytesRecvd += received + BD_UDP_IP_OVERHEAD;
	}

	return received;
}

bdBool BD_CALL bdPlatformSocket::close(bdInt &handle)
{
	bdBool success = false;
	if (handle != BD_INVALID_SOCKET_HANDLE)
	{
		int result = sockAPIregthr();
		success = ::closesocket(handle) == 0;
		handle = BD_INVALID_SOCKET_HANDLE;
	}
	return success;
}

void BD_CALL bdPlatformSocket::unregisterThread()
{
	sockAPIderegthr();
}

bdBool BD_CALL bdPlatformSocket::setBlocking(bdInt &handle, 
											 const bdBool blocking)
{
	// Only allow non-blocking mode in single-threaded environment
	int res = sockAPIregthr();
	if(handle == BD_INVALID_SOCKET_HANDLE)
	{
		return false;
	}

	if(!blocking)
	{
		// Change socket to nonblocking mode
		bdULong32 nonblocking = 1;
		int res = ::setsockopt(handle, SOL_SOCKET, SO_NONBLOCK, &nonblocking, sizeof(nonblocking));
		return res == 0;
	}
	else
	{
		// Change socket to blocking mode
		bdULong32 nonblocking = 0;
		int res = ::setsockopt(handle, SOL_SOCKET, SO_NONBLOCK, &nonblocking, sizeof(nonblocking));
		return res == 0;

//		return ::ioctl(handle, FIONBIO, &nonblocking) == 0;
	}

	// think we need to use setsockopt or sceInsockSetsockopt
	/*
	if(!blocking)
	{
		// Change socket to nonblocking mode
		bdULong32 nonblocking = 1;
		return ::ioctl(handle, FIONBIO, &nonblocking) == 0;
	}
	else
	{
		// Change socket to blocking mode
		bdULong32 nonblocking = 0;
		return ::ioctl(handle, FIONBIO, &nonblocking) == 0;
	}*/
	return true;
}

bdUInt BD_CALL bdPlatformSocket::getHostByName(const bdNChar8* const name,
											   bdInAddr *adresses,
											   bdUInt numAdresses)
{
	bdUInt numAddrs = 0;

	hostent* remoteHost;

	// If the user input is an alpha name for the host, use gethostbyname()
	// If not, get host by addr (assume IPv4)
	if (isalpha(name[0]))
	{   
		//host address is a name
		remoteHost = gethostbyname(name);

		if(remoteHost)
		{
			bdUInt index = 0;
			while (index < numAdresses && remoteHost->h_addr_list[index])
			{
				struct in_addr inAddr;
				bdMemcpy(&inAddr, remoteHost->h_addr_list[index], sizeof(struct in_addr));
				adresses[index].inUn.m_iaddr = inAddr.s_addr;
				index ++;
			}
			numAddrs = index;
		}
	}
	else
	{ 
        // FIXME: Fix this to correctly return zero addresses when it fails. Use inet_aton
		adresses[0].inUn.m_iaddr = inet_addr(name);
		numAddrs = 1;
	}

	return numAddrs;
}

void BD_CALL bdPlatformSocket::abortGetHostByName()
{
	//TODO: implement this
}

#endif // BD_PLATFORM_PS2 && BD_PS2_USE_NDK
