// DWBS ------------------------------------------------------------------ DWBS
//
// PLATFORM		: PS2
// PRODUCT		: BIT_DEMON
// VISIBILITY	: PUBLIC
//
// DWBS ------------------------------------------------------------------ DWBS


#include <bdPlatform/bdPlatformSocket/bdPlatformStreamSocket.h>
#include <bdPlatform/bdPlatformString/bdPlatformString.h>

#if defined BD_PLATFORM_PS2 && defined BD_PS2_USE_LIBINSOCK

#include <bdPlatform/bdPlatformString/bdPlatformString.h>

#  include <libinsck.h>
#  include <errno.h>

#  undef bind
#  undef send
#  undef connect
#  undef accept

bdInt BD_CALL bdPlatformStreamSocket::create()
{
	// create the socket handle
	bdInt handle = ::sceInsockSocket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
	
	return handle;
}

bdSocketStatusCode BD_CALL bdPlatformStreamSocket::bind(bdInt &handle, 
														const bdInAddr addr, 
														const bdUInt16 port)
{
	if(handle == BD_INVALID_SOCKET_HANDLE)
	{
		return BD_NET_INVALID_HANDLE;
	}

	struct sockaddr_in localAddr;
	// Fill in address for binding locally
	bdMemset(&localAddr, 0, sizeof(localAddr));
	localAddr.sin_family = AF_INET;
	localAddr.sin_addr.s_addr = addr.m_inAddri;
	localAddr.sin_port = htons(port);

	if ( ::sceInsockBind( handle, (struct sockaddr *)&localAddr, sizeof(localAddr) )  == BD_INVALID_SOCKET_HANDLE )
	{
		// couldn't bind
		return BD_NET_ERROR;
	}

	::sceInsockListen(handle, 5);

	return BD_NET_SUCCESS;
}

bdSocketStatusCode BD_CALL bdPlatformStreamSocket::connect(bdInt &handle, 
														   const bdInAddr addr, 
														   const bdUInt16 port)
{
	struct sockaddr_in remoteAddr;
	// Fill in address for binding locally
	bdMemset(&remoteAddr, 0, sizeof(remoteAddr));
	remoteAddr.sin_family = AF_INET;
	remoteAddr.sin_addr.s_addr = addr.m_inAddri;
	remoteAddr.sin_port = htons(port);

	const bdInt retval = ::sceInsockConnect(handle, (struct sockaddr *)&remoteAddr, sizeof(remoteAddr));

	if(retval < 0)
	{
		const bdInt error = getErrno(handle);

		switch(error)
		{
			case EINPROGRESS:
			case EWOULDBLOCK:
				return BD_NET_WOULD_BLOCK;
				break;
			case ECONNRESET:
			case ETIMEDOUT:
			case ENETUNREACH:
			case ECONNABORTED:
			case EHOSTUNREACH:
			case ENETDOWN:
				return BD_NET_CONNECTION_RESET;
				break;
			case EMSGSIZE:
				return BD_NET_MSG_SIZE;
				break;
			case EINTR:
				return BD_NET_BLOCKING_CALL_CANCELLED;
				break;
			case EINVAL:
				// This indicates the socket has not yet been bound (either
				// through bind or a call to sendTo) and we therefore can't
				// receive.
				// This happens as we have scheduled a packet to be sent (causing
				// a bind) but this thread has run before the send thread.
				// return a
				return BD_NET_NOT_BOUND;
				break;
			default:
				return BD_NET_ERROR;
		}
	}
	return BD_NET_SUCCESS;
}

bdBool BD_CALL bdPlatformStreamSocket::accept(bdInt &handle,
											  bdInt &newHandle, 
											  bdInAddr &addr, 
											  bdUInt16 &port)
{
	struct sockaddr_in remoteAddr;
	bdMemset(&remoteAddr, 0, sizeof(remoteAddr));

	sceInsockSocklen_t remoteAddrLen = sizeof(struct sockaddr_in);

	bdInt sd = static_cast<bdInt>(::sceInsockAccept(handle, reinterpret_cast<sockaddr*>(&remoteAddr), &remoteAddrLen));

	if(sd != BD_INVALID_SOCKET_HANDLE)
	{
		port = ntohs(remoteAddr.sin_port);
		addr.m_inAddri = remoteAddr.sin_addr.s_addr;

		newHandle = sd;

		return true;
	}

	return false;
}


bdInt BD_CALL bdPlatformStreamSocket::send(bdInt &handle,
										   const void *data, 
										   const bdUInt length)
{
	if(handle == BD_INVALID_SOCKET_HANDLE)
	{
		return BD_NET_INVALID_HANDLE;
	}

	bdInt sent = ::sceInsockSend(handle,
							reinterpret_cast<const bdChar8 *>(data),
							length,
							0);

	if(sent < 0)
	{
		const bdInt error = getErrno(0);
		switch(error)
		{
			case ECONNRESET:
			case ETIMEDOUT:
			case ENETUNREACH:
			case ECONNABORTED:
			case EHOSTUNREACH:
			case ENETDOWN:	
				return BD_NET_CONNECTION_RESET;
				break;
			case EWOULDBLOCK:
			//case EAGAIN:
				return BD_NET_WOULD_BLOCK;
			break;
			case ENOTCONN:
				if (!checkSocketException(handle))
				{
					return BD_NET_CONNECTION_RESET;
				}
				return BD_NET_NOT_CONNECTED;
			default:
				return BD_NET_ERROR;
		}
	}
	else
	{
		m_totalBytesSent += sent;
	}

	return sent;
}


bdInt BD_CALL bdPlatformStreamSocket::receive(bdInt &handle,
											  void *data,
											  const bdUInt size)
{
	if(handle == BD_INVALID_SOCKET_HANDLE)
	{
		return BD_NET_INVALID_HANDLE;
	}


	bdInt received = 0;


	received = ::sceInsockRecv(handle,
		                       reinterpret_cast<bdChar8 *>(data),
                               size,
                               0);


	if(received < 0)
	{
		const bdInt error = getErrno(handle);
		switch(error)
		{
			case EWOULDBLOCK:
				return BD_NET_WOULD_BLOCK;
				break;
			case ECONNRESET:
			case ETIMEDOUT:
			case ENETUNREACH:
			case ECONNABORTED:
			case EHOSTUNREACH:
			case ENETDOWN:	
				return BD_NET_CONNECTION_RESET;
				break;
			case EMSGSIZE:
				return BD_NET_MSG_SIZE;
				break;
			case EINTR:
				return BD_NET_BLOCKING_CALL_CANCELLED;
				break;
			case EINVAL:
				// This indicates the socket has not yet been bound (either
				// through bind or a call to sendTo) and we therefore can't
				// receive.
				// This happens as we have scheduled a packet to be sent (causing
				// a bind) but this thread has run before the send thread.
				// return a
				return BD_NET_NOT_BOUND;
				break;
			case ENOTCONN:
				if (!checkSocketException(handle))
				{
					return BD_NET_CONNECTION_RESET;
				}
				return BD_NET_NOT_CONNECTED;
			default:
				return BD_NET_ERROR;
		}
	}
	else
	{
		m_totalBytesRecvd += received;
	}

	return received;
}

bdBool BD_CALL bdPlatformStreamSocket::close(bdInt &handle)
{
	if (handle != BD_INVALID_SOCKET_HANDLE)
	{
		bdInt realHandle = handle & 0x7FFFFFFF;
		::sceInsockAbort(realHandle,0);
		const bdBool success = ::sceInsockShutdown(realHandle, SHUT_RDWR) == 0;
		handle = BD_INVALID_SOCKET_HANDLE;

		return success;
	}

	return true;
}

void BD_CALL bdPlatformStreamSocket::unregisterThread()
{
}

bdBool BD_CALL bdPlatformStreamSocket::setBlocking(bdInt &handle, 
												   const bdBool blocking)
{
	if(handle == BD_INVALID_SOCKET_HANDLE)
	{
		return false;
	}


	int argp;
	if( blocking )
	{
		argp = 0;
	}
	else
	{

		argp = 1;
	}

	bdBool success = ::sceInsockSetsockopt(handle, SOL_SOCKET, SO_NBIO, &argp, sizeof(argp)) == 0;;
	return success;

}

bdInt BD_CALL bdPlatformStreamSocket::getErrno(bdInt handle)
{
	return  sceInsockErrno;
}

bdBool BD_CALL bdPlatformStreamSocket::checkSocketException(bdInt handle)
{
	return true;
}

#endif // BD_PLATFORM_PS2 && BD_PS2_USE_LIBINSOCK
