// DWBS ------------------------------------------------------------------ DWBS
//
// PLATFORM		: WIN32 PS2 UNIX OSX PSP-INFRA PS3
// PRODUCT		: BIT_DEMON
// VISIBILITY	: PUBLIC
//
// DWBS ------------------------------------------------------------------ DWBS

#include <bdSocket/bdNAT/bdIPDiscoveryClient.h>
#include <bdSocket/bdNAT/bdPacket/bdIPDiscoveryPacket.h>
#include <bdSocket/bdNAT/bdPacket/bdIPDiscoveryPacketReply.h>

// The number of retransmissions before we give up
#define BD_IP_DISC_NUM_RETRANSMISSIONS (5u)
// Time in seconds before we timeout
#define BD_IP_DISC_REQUEST_TIMEOUT	(.5f)

#define BD_LOG_LEVEL "bdSocket/nat"

bdIPDiscoveryClient::bdIPDiscoveryClient(bdDefaultSocket *socket, const bdAddr &server)
: m_socket(socket),
  m_serverAddr(server),
  m_success(false),
  m_retries(0)
{
}

bdIPDiscoveryClient::~bdIPDiscoveryClient()
{
	m_socket = BD_NULL;
}

bdBool bdIPDiscoveryClient::startDiscovery()
{
	m_timer.start();

	bdIPDiscoveryPacket packet;
	const bdUInt bufferSize = BD_MAX_DATAGRAM_SIZE;
	bdUByte8 buffer[bufferSize];
	bdUInt len;

	BD_ASSERT(packet.serialize(buffer, bufferSize, 0, len), "Failed to serialize packet.");

	return m_socket->sendTo(m_serverAddr, buffer, len) == static_cast<bdInt>(len);
}

bdBool bdIPDiscoveryClient::isPending() const
{
	return !m_success && m_retries < BD_IP_DISC_NUM_RETRANSMISSIONS;
}

bdBool bdIPDiscoveryClient::wasSuccess() const
{
	return m_success;
}

const bdAddr& bdIPDiscoveryClient::getPublicAddress() const
{
	BD_ASSERT(wasSuccess(), "bdIPDiscoveryClient::getPublicAddress() called, but "
			"address resolution was not successful (or not yet finished).");
	return m_publicAddr;
}

void bdIPDiscoveryClient::pump()
{
	const bdUInt dataSize = BD_MAX_DATAGRAM_SIZE;
	bdUByte8 data[dataSize];

	if(!m_success)
	{ 
		bdAddr fromAddr;

		bdBool receiving = true;

		while(receiving)
		{
			bdInt received = m_socket->receiveFrom(fromAddr, data, dataSize);
			bdIPDiscoveryPacketReply packet;
			bdUInt tmpUInt;

			//if(received == sizeof(bdIPDiscoveryPacketReply) && fromAddr == m_serverAddr)
			if (received > 0 && fromAddr == m_serverAddr &&
				packet.deserialize(data, static_cast<bdUInt>(received), 0, tmpUInt))
			{
				m_publicAddr = packet.getAddr();
				m_success = true;

				const bdUWord addrBufferSize = 100;
				bdNChar8 addrBuffer[addrBufferSize];
				m_publicAddr.toString(addrBuffer, addrBufferSize);
				BD_INFO(BD_LOG_LEVEL, "Public IP discovered: %s", addrBuffer);
			}
			else if(received < 0)
			{
				switch(received)
				{
					case BD_NET_WOULD_BLOCK:
					case BD_NET_BLOCKING_CALL_CANCELLED:
						receiving = false;
						break;
					case BD_NET_CONNECTION_RESET:
						// Received ICMP connection reset. Only happens once.
						break;
					case BD_NET_MSG_SIZE:
						break;
					case BD_NET_NOT_BOUND:
					default:
						receiving = false;
						BD_WARN(BD_LOG_LEVEL, "net error.");
						break;
				}
			}
		}
	}

	if(!m_success)
	{
		if(m_timer.getElapsedTimeInSeconds() > BD_IP_DISC_REQUEST_TIMEOUT &&
			m_retries++ < BD_IP_DISC_NUM_RETRANSMISSIONS)
		{
			BD_INFO(BD_LOG_LEVEL, "IP Discovery failed. Retrying. (%u/%u)", m_retries, BD_IP_DISC_NUM_RETRANSMISSIONS);
			bdIPDiscoveryPacket packet;
			bdUInt len;
			BD_ASSERT(packet.serialize(data, dataSize, 0, len), "Failed to serialize the packet.");

			m_socket->sendTo(m_serverAddr, data, len);
			m_timer.start();
		}
	}
}


