
#ifndef _MArgDatabase
#define _MArgDatabase
//
//-
// ==========================================================================
// Copyright (C) 1995 - 2006 Autodesk, Inc., and/or its licensors.  All 
// rights reserved.
// 
// The coded instructions, statements, computer programs, and/or related 
// material (collectively the "Data") in these files contain unpublished 
// information proprietary to Autodesk, Inc. ("Autodesk") and/or its 
// licensors,  which is protected by U.S. and Canadian federal copyright law 
// and by international treaties.
// 
// The Data may not be disclosed or distributed to third parties or be 
// copied or duplicated, in whole or in part, without the prior written 
// consent of Autodesk.
// 
// The copyright notices in the Software and this entire statement, 
// including the above license grant, this restriction and the following
// disclaimer, must be included in all copies of the Software, in whole 
// or in part, and all derivative works of the Software, unless such copies 
// or derivative works are solely in the form of machine-executable object 
// code generated by a source language processor.

// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND. 
// AUTODESK DOES NOT MAKE AND HEREBY DISCLAIMS ANY EXPRESS OR IMPLIED 
// WARRANTIES INCLUDING, BUT NOT LIMITED TO, THE WARRANTIES OF 
// NON-INFRINGEMENT, MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE, 
// OR ARISING FROM A COURSE OF DEALING, USAGE, OR TRADE PRACTICE. IN NO 
// EVENT WILL AUTODESK AND/OR ITS LICENSORS BE LIABLE FOR ANY LOST 
// REVENUES, DATA, OR PROFITS, OR SPECIAL, DIRECT, INDIRECT, OR 
// CONSEQUENTIAL DAMAGES, EVEN IF AUTODESK AND/OR ITS LICENSORS HAS 
// BEEN ADVISED OF THE POSSIBILITY OR PROBABILITY OF SUCH DAMAGES. 
// ==========================================================================
//+
//
// CLASS:    MArgDatabase
//
// *****************************************************************************
//
// CLASS DESCRIPTION (MArgDatabase)
//
// This class is used to parse and store all of the flags, arguments
// and objects which are passed to a command.  The MArgDatabase accepts
// an MSyntax object, which describes the format for a command, and uses
// it to parse the command arguments into a form which is easy to query.
//
// *****************************************************************************

#if defined __cplusplus

// *****************************************************************************

// INCLUDED HEADER FILES


#include <maya/MStatus.h>
#include <maya/MTypes.h>
#include <maya/MArgParser.h>

// *****************************************************************************

// DECLARATIONS

class MSyntax;
class MSelectionList;
class MStringArray;
class MArgList;
class MDistance;
class MAngle;
class MTime;

// *****************************************************************************

// CLASS DECLARATION (MArgDatabase)

/// Command argument list parser. (OpenMaya) (OpenMaya.py)
/**
This class parses argument lists based on a syntax object (MSyntax) 
which describes the format for a command.
*/
#ifdef _WIN32
#pragma warning(disable: 4522)
#endif // _WIN32

class OPENMAYA_EXPORT MArgDatabase : public MArgParser
{
public:
	///
			MArgDatabase		();
	///
			MArgDatabase		(const MSyntax &syntax,
								 const MArgList &argList,
								 MStatus *ReturnStatus = NULL);
	///
	virtual ~MArgDatabase();

	///
	MStatus getFlagArgument		(const char *flag, unsigned int index, 
								 MSelectionList &result) const;
	///
	MStatus	getCommandArgument	(unsigned int index, MSelectionList &result) const;

	///
	MStatus	getObjects			(MSelectionList &result) const;

BEGIN_NO_SCRIPT_SUPPORT:
	// Redeclarations of overloaded methods from MArgParser
	//     NO SCRIPT SUPPORT, USE BASE CLASS
	MStatus	getFlagArgument		(const char *flag, unsigned int index, 
								 bool &result) const;
	//     NO SCRIPT SUPPORT, USE BASE CLASS
	MStatus	getFlagArgument		(const char *flag, unsigned int index,
								 int &result) const;
	//     NO SCRIPT SUPPORT, USE BASE CLASS
	MStatus	getFlagArgument		(const char *flag, unsigned int index, 
								 double &result) const;
	//     NO SCRIPT SUPPORT, USE BASE CLASS
	MStatus	getFlagArgument		(const char *flag, unsigned int index, 
								 MString &result) const;
	//     NO SCRIPT SUPPORT, USE BASE CLASS
	MStatus getFlagArgument		(const char *flag, unsigned int index, 
								 unsigned int &result) const;
	//     NO SCRIPT SUPPORT, USE BASE CLASS
	MStatus	getFlagArgument		(const char *flag, unsigned int index, 
								 MDistance &result) const;
	//     NO SCRIPT SUPPORT, USE BASE CLASS
	MStatus	getFlagArgument		(const char *flag, unsigned int index, 
								 MAngle &result) const;
	//     NO SCRIPT SUPPORT, USE BASE CLASS
	MStatus	getFlagArgument		(const char *flag, unsigned int index, 
								 MTime &result) const;
	//     NO SCRIPT SUPPORT, USE BASE CLASS
	MStatus	getCommandArgument	(unsigned int index, bool &result) const;
	//     NO SCRIPT SUPPORT, USE BASE CLASS
	MStatus	getCommandArgument	(unsigned int index, int &result) const;
	//     NO SCRIPT SUPPORT, USE BASE CLASS
	MStatus	getCommandArgument	(unsigned int index, double &result) const;
	//     NO SCRIPT SUPPORT, USE BASE CLASS
	MStatus	getCommandArgument	(unsigned int index, MString &result) const;
	//     NO SCRIPT SUPPORT, USE BASE CLASS
	MStatus	getCommandArgument	(unsigned int index, MDistance &result) const;
	//     NO SCRIPT SUPPORT, USE BASE CLASS
	MStatus	getCommandArgument	(unsigned int index, MAngle &result) const;
	//     NO SCRIPT SUPPORT, USE BASE CLASS
	MStatus	getCommandArgument	(unsigned int index, MTime &result) const;
END_NO_SCRIPT_SUPPORT:

	MStatus	getObjects			(MStringArray &result) const;

protected:
// No protected members

private:
	const char *className() const;
};

#ifdef _WIN32
#pragma warning(default: 4522)
#endif // _WIN32

// *****************************************************************************
#endif /* __cplusplus */
#endif /* _MArgDatabase */

