
#ifndef _MFnSpotLight
#define _MFnSpotLight
//
//-
// ==========================================================================
// Copyright (C) 1995 - 2006 Autodesk, Inc., and/or its licensors.  All 
// rights reserved.
// 
// The coded instructions, statements, computer programs, and/or related 
// material (collectively the "Data") in these files contain unpublished 
// information proprietary to Autodesk, Inc. ("Autodesk") and/or its 
// licensors,  which is protected by U.S. and Canadian federal copyright law 
// and by international treaties.
// 
// The Data may not be disclosed or distributed to third parties or be 
// copied or duplicated, in whole or in part, without the prior written 
// consent of Autodesk.
// 
// The copyright notices in the Software and this entire statement, 
// including the above license grant, this restriction and the following 
// disclaimer, must be included in all copies of the Software, in whole 
// or in part, and all derivative works of the Software, unless such copies 
// or derivative works are solely in the form of machine-executable object 
// code generated by a source language processor.

// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND. 
// AUTODESK DOES NOT MAKE AND HEREBY DISCLAIMS ANY EXPRESS OR IMPLIED 
// WARRANTIES INCLUDING, BUT NOT LIMITED TO, THE WARRANTIES OF 
// NON-INFRINGEMENT, MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE, 
// OR ARISING FROM A COURSE OF DEALING, USAGE, OR TRADE PRACTICE. IN NO 
// EVENT WILL AUTODESK AND/OR ITS LICENSORS BE LIABLE FOR ANY LOST 
// REVENUES, DATA, OR PROFITS, OR SPECIAL, DIRECT, INDIRECT, OR 
// CONSEQUENTIAL DAMAGES, EVEN IF AUTODESK AND/OR ITS LICENSORS HAS 
// BEEN ADVISED OF THE POSSIBILITY OR PROBABILITY OF SUCH DAMAGES. 
// ==========================================================================
//+
//
// CLASS:    MFnSpotLight
//
// *****************************************************************************
//
// CLASS DESCRIPTION (MFnSpotLight)
//
// MFnSpotLight facilitates creation and manipulation of dependency graph nodes
// representing spotlights.
//
// *****************************************************************************

#if defined __cplusplus

// *****************************************************************************

// INCLUDED HEADER FILES


#include <maya/MFnDagNode.h>
#include <maya/MObject.h>
#include <maya/MStatus.h>
#include <maya/MFnNonExtendedLight.h>

// *****************************************************************************

// DECLARATIONS

class MFltMatrix;

// *****************************************************************************

// CLASS DECLARATION (MFnSpotLight)

/// Manage Spot Light dependency Nodes. (OpenMaya) (OpenMaya.py)
/**
  Facilitate the creation and manipulation of spot light nodes
*/
#ifdef _WIN32
#pragma warning(disable: 4522)
#endif // _WIN32

class OPENMAYA_EXPORT MFnSpotLight : public MFnNonExtendedLight 
{

	declareDagMFn(MFnSpotLight,MFnNonExtendedLight);
public:
	///
	enum MBarnDoor { kLeft,
					 kRight,
					 kTop,
					 kBottom };
	///
	enum MDecayRegion { kFirst,
						kSecond,
						kThird };
	///
	MObject     create( bool UIvisible = true, MStatus * ReturnStatus = NULL );
	///
	MObject     create( const MObject& parent, bool UIvisible = true,
				  bool wantTransform = false, MStatus * ReturnStatus = NULL );
	///
	double      coneAngle( MStatus * ReturnStatus = NULL ) const;
	///
    MStatus     setConeAngle( const double& coneAngle );
	///
	double      penumbraAngle( MStatus * ReturnStatus = NULL ) const;
	///
	MStatus     setPenumbraAngle( const double& penumbraAngle );
	///
	double      dropOff( MStatus * ReturnStatus = NULL ) const;
	///
	MStatus     setDropOff( const double& dropOff );
	///
	bool        barnDoors( MStatus * ReturnStatus = NULL ) const;
	///
	MStatus     setBarnDoors( const bool& barnDoors );
	///
	double      barnDoorAngle( MBarnDoor which_door,
						MStatus * ReturnStatus = NULL ) const;
	///
	MStatus     setBarnDoorAngle( MBarnDoor which_door,
						const double& barn_door_angle );
	///
	bool        useDecayRegions( MStatus * ReturnStatus = NULL ) const;
	///
	MStatus     setUseDecayRegions( const bool& use_decay_regions );
	///
	float       startDistance( MDecayRegion which_region,
						MStatus * ReturnStatus = NULL ) const;
	///
	MStatus     setStartDistance( MDecayRegion which_region,
						const float& start_distance );
	///
	float       endDistance( MDecayRegion which_region,
						MStatus * ReturnStatus = NULL ) const;
	///
	MStatus     setEndDistance( MDecayRegion which_region,
						const float& end_distance );

BEGIN_NO_SCRIPT_SUPPORT:

 	declareDagMFnConstConstructor( MFnSpotLight,MFnNonExtendedLight );

	/// NO SCRIPT SUPPORT
	MObject     create( const MObject& parent, bool UIvisible = true,
						MStatus * ReturnStatus = NULL );
	
END_NO_SCRIPT_SUPPORT:

protected:
// No protected members

private:
// No private members

};

#ifdef _WIN32
#pragma warning(default: 4522)
#endif // _WIN32

// *****************************************************************************
#endif /* __cplusplus */
#endif /* _MFnSpotLight */



