#ifndef _MLightLinks
#define _MLightLinks
//
//-
// ==========================================================================
// Copyright (C) 1995 - 2006 Autodesk, Inc., and/or its licensors.  All 
// rights reserved.
// 
// The coded instructions, statements, computer programs, and/or related 
// material (collectively the "Data") in these files contain unpublished 
// information proprietary to Autodesk, Inc. ("Autodesk") and/or its 
// licensors,  which is protected by U.S. and Canadian federal copyright law 
// and by international treaties.
// 
// The Data may not be disclosed or distributed to third parties or be 
// copied or duplicated, in whole or in part, without the prior written 
// consent of Autodesk.
// 
// The copyright notices in the Software and this entire statement, 
// including the above license grant, this restriction and the following 
// disclaimer, must be included in all copies of the Software, in whole 
// or in part, and all derivative works of the Software, unless such copies 
// or derivative works are solely in the form of machine-executable object 
// code generated by a source language processor.

// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND. 
// AUTODESK DOES NOT MAKE AND HEREBY DISCLAIMS ANY EXPRESS OR IMPLIED 
// WARRANTIES INCLUDING, BUT NOT LIMITED TO, THE WARRANTIES OF 
// NON-INFRINGEMENT, MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE, 
// OR ARISING FROM A COURSE OF DEALING, USAGE, OR TRADE PRACTICE. IN NO 
// EVENT WILL AUTODESK AND/OR ITS LICENSORS BE LIABLE FOR ANY LOST 
// REVENUES, DATA, OR PROFITS, OR SPECIAL, DIRECT, INDIRECT, OR 
// CONSEQUENTIAL DAMAGES, EVEN IF AUTODESK AND/OR ITS LICENSORS HAS 
// BEEN ADVISED OF THE POSSIBILITY OR PROBABILITY OF SUCH DAMAGES. 
// ==========================================================================
//+
//
// CLASS:    MLightLinks
//
// ***************************************************************************
//
// CLASS DESCRIPTION (MLightLinks)
//
// MLightLinks is a class which provides access to Maya's light linking 
// information.  It parses the information expressed by the light linker nodes
// in the scene, and provides a simple queryable interface for it.
//
// ***************************************************************************

#if defined __cplusplus

// ***************************************************************************
// INCLUDED HEADER FILES

#include <maya/MDagPath.h>
#include <maya/MSelectionList.h>
#include <maya/MIOStream.h>

// ***************************************************************************
// CLASS DECLARATION (MLightLinks)

/// Class providing read-only Light Linking API functions (OpenMayaRender) (OpenMayaRender.py)
/**
*/
class OPENMAYARENDER_EXPORT MLightLinks
{
public:

	///
	MLightLinks();

	///
	~MLightLinks();

	///
	MStatus parseLinks( const MObject& linkNode = MObject::kNullObj, 
						bool verbose = false, 
						std::ostream *stream = NULL,
						bool useIgnore = false );

	///
	MStatus getLinkedLights( const MDagPath& path, 
							 const MObject& component, 
							 MDagPathArray& lights );

	///
	MStatus getLinkedObjects( const MDagPath& light, 
							  MSelectionList& objects );

	///
	MStatus getIgnoredLights( const MDagPath& path, 
							  const MObject& component, 
							  MDagPathArray& lights );

	///
	MStatus getIgnoredObjects( const MDagPath& light, 
							   MSelectionList& objects );

	///
	MStatus getShadowLinkedLights( const MDagPath& path, 
							 const MObject& component, 
							 MDagPathArray& lights );

	///
	MStatus getShadowLinkedObjects( const MDagPath& light, 
							  MSelectionList& objects );

	///
	MStatus getShadowIgnoredLights( const MDagPath& path, 
							  const MObject& component, 
							  MDagPathArray& lights );

	///
	MStatus getShadowIgnoredObjects( const MDagPath& light, 
							   MSelectionList& objects );

private:
	void *f_linkUtil;
};


// ***************************************************************************
#endif /* __cplusplus */
#endif /* _MLightLinks */
