#ifndef _MPoint
#define _MPoint
//
//-
// ==========================================================================
// Copyright (C) 1995 - 2006 Autodesk, Inc., and/or its licensors.  All 
// rights reserved.
// 
// The coded instructions, statements, computer programs, and/or related 
// material (collectively the "Data") in these files contain unpublished 
// information proprietary to Autodesk, Inc. ("Autodesk") and/or its 
// licensors,  which is protected by U.S. and Canadian federal copyright law 
// and by international treaties.
// 
// The Data may not be disclosed or distributed to third parties or be 
// copied or duplicated, in whole or in part, without the prior written 
// consent of Autodesk.
// 
// The copyright notices in the Software and this entire statement, 
// including the above license grant, this restriction and the following 
// disclaimer, must be included in all copies of the Software, in whole 
// or in part, and all derivative works of the Software, unless such copies 
// or derivative works are solely in the form of machine-executable object 
// code generated by a source language processor.

// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND. 
// AUTODESK DOES NOT MAKE AND HEREBY DISCLAIMS ANY EXPRESS OR IMPLIED 
// WARRANTIES INCLUDING, BUT NOT LIMITED TO, THE WARRANTIES OF 
// NON-INFRINGEMENT, MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE, 
// OR ARISING FROM A COURSE OF DEALING, USAGE, OR TRADE PRACTICE. IN NO 
// EVENT WILL AUTODESK AND/OR ITS LICENSORS BE LIABLE FOR ANY LOST 
// REVENUES, DATA, OR PROFITS, OR SPECIAL, DIRECT, INDIRECT, OR 
// CONSEQUENTIAL DAMAGES, EVEN IF AUTODESK AND/OR ITS LICENSORS HAS 
// BEEN ADVISED OF THE POSSIBILITY OR PROBABILITY OF SUCH DAMAGES. 
// ==========================================================================
//+
//
// CLASS:    MPoint
//
// *****************************************************************************
//
// CLASS DESCRIPTION (MPoint)
//
//	This class provides an implementation of a point.  Numerous convienence
//	operators are provided to help with the manipulation of points. This
//	includes operators that work with the MVector and MMatrix classes.
//
// *****************************************************************************

#if defined __cplusplus

// *****************************************************************************

// INCLUDED HEADER FILES



#include <maya/MTypes.h>
#include <maya/MStatus.h>

// *****************************************************************************

// DECLARATIONS

class MMatrix;
class MVector;
class MFloatVector;

#define MPoint_kTol	1.0e-10

// *****************************************************************************

// CLASS DECLARATION (MPoint)

/// Implementation of a point. (OpenMaya) (OpenMaya.py)
/**
  This class implements the Maya representation of a point
*/
#ifdef _WIN32
#pragma warning(disable: 4522)
#endif // _WIN32

class OPENMAYA_EXPORT MPoint  
{
public:
	///
					MPoint();	// defaults to 0,0,0,1
	///
					MPoint( const MPoint & srcpt );  
	///
					MPoint( const MVector & src );  
	///
					MPoint( const MFloatVector & src );  
	///
					MPoint( double xx, double yy,
							double zz = 0.0, double ww = 1.0 );
	///
					MPoint( const double[4] );
	///
					MPoint( const float[4] );
	///
					~MPoint();
	///
	MStatus			get( double[4] ) const;
	///
	MStatus			get( float[4] ) const;
	///
	double  		operator[](unsigned int i) const;
	///
	double  		operator()(unsigned int i) const;
	///
	MPoint & 		operator=( const MPoint & src );
	///
	MVector   		operator-( const MPoint & other ) const;
	///
	MPoint   		operator+( const MVector & other ) const;
	///
	MPoint   		operator-( const MVector & other ) const;
	///
	MPoint & 		operator+=( const MVector & vector );
	///
	MPoint & 		operator-=( const MVector & vector );
	///
	MPoint			operator*(const double scale) const;
	///
	MPoint			operator/(const double scale) const;
	///
	MPoint   		operator*(const MMatrix &) const;
	///
	MPoint & 		operator*=(const MMatrix &);
	///
	bool           	operator==( const MPoint & other ) const;
	///
	bool           	operator!=( const MPoint & other ) const;
	///
	MPoint & 		cartesianize();
	///
	MPoint & 		rationalize();
	///
	MPoint & 		homogenize();
	///
	double     		distanceTo( const MPoint & other ) const;
	///
	bool           	isEquivalent( const MPoint & other,
								  double tolerance = MPoint_kTol) const;

BEGIN_NO_SCRIPT_SUPPORT:

	///	NO SCRIPT SUPPORT
	double &        operator[](unsigned int i);

	///	NO SCRIPT SUPPORT
	double &        operator()(unsigned int i);

	///	NO SCRIPT SUPPORT
	friend OPENMAYA_EXPORT MPoint operator*(const MMatrix &, const MPoint &);
	///	NO SCRIPT SUPPORT
	friend OPENMAYA_EXPORT std::ostream& operator<<(std::ostream& os, const MPoint& p);

END_NO_SCRIPT_SUPPORT:

	///
	static const MPoint origin;
public:
	/// the x component of the point
	double			x;
	/// the y component of the point
	double			y;
	/// the z component of the point
	double			z;
	/// the w component of the point
	double			w;

protected:
// No protected members

private:

	static const char* className();
};

#ifdef _WIN32
#pragma warning(default: 4522)
#endif // _WIN32

// *****************************************************************************
#endif /* __cplusplus */
#endif /* _MPoint */
