/*
* SCE CONFIDENTIAL
* PSP(TM) Graphic Library Samples
* Copyright (C) 2004-2006 Sony Computer Entertainment Europe
* All Rights Reserved.
*
* sceePspGu - sceePspGu.h
*   library header file
*
*   Date            Design      Log
*  ----------------------------------------------------
*   2006-01-06      srubens     version 1.6.1
*   2005-12-12      srubens     version 1.6.0
*   2005-10-31      srubens     version 1.5.0
*   2005-09-27      srubens     version 1.4.1
*   2005-09-23      srubens     version 1.4.0
*   2005-08-23      srubens     version 1.3.0
*   2005-08-12      srubens     version 1.2.1b
*   2005-07-29      srubens     version 1.2.0
*   2005-07-21      srubens     version 1.1.1
*   2005-07-20      srubens     version 1.1.0
*   2005-04-05      srubens     version 1.0.0
*   2004-04-23      srubens     initial implementation
*/

// ------------------------------------------------------------------------------
// sceePspGu library
// Copyright (C) 2004-2006 Sony Computer Entertainment Europe.
// All Rights Reserved.
// ------------------------------------------------------------------------------
// FILE:			$File: sceePspGu.h $
// FIRST VERSION:	23/04/2004
// FIRST AUTHOR:	Sebastien Rubens
// LAST AUTHOR:		$Author: srubens  $
// VERSION:			$DateTime:  $
// REVISION:		$Revision:  $
// CHANGE:			$Change:  $
//
// DESCRIPTION:
//	This library is a libgu-like library with many improvements.
// ------------------------------------------------------------------------------


#ifndef	SCEEPSPGU_H
#define	SCEEPSPGU_H
// ------------------------------------------------------------------------------
// COMPILATION FLAGS
// ------------------------------------------------------------------------------
// TRC library usage check
#include <devkit_version.h>
#if		((SCE_DEVKIT_VERSION&0xFFFFFF00) < 0x02080000)
#error You are not authorized to use this version of the libraries for submission to SCE QA after the 9th of November 2006.
#endif


// ------------------------------------------------------------------------------
// INCLUDES
// ------------------------------------------------------------------------------
#include <psptypes.h>
#include <geman.h>

#include "sceeTypes.h"


namespace scee {
namespace Psp {
namespace Gu {
// ------------------------------------------------------------------------------
// CONSTANTS
// ------------------------------------------------------------------------------
//static const f32	PI				=	3.1415926535897932384626433832f;
static const u32	SCR_WIDTH		=	480;
static const u32	SCR_HEIGHT		=	272;
static const f32	SCR_ASPECT		=	SCR_WIDTH / static_cast<f32>(SCR_HEIGHT);
static const u32	SCR_OFFSETX     =	((4096 - SCR_WIDTH) / 2);
static const u32	SCR_OFFSETY     =	((4096 - SCR_HEIGHT) / 2);
// Frame buffer
static const u32	VRAM_TOP		=	0x00000000;
static const u32	VRAM_WIDTH		=	512;
// 16bit mode
static const u32	VRAM_BUFSIZE16	=	(VRAM_WIDTH*SCR_HEIGHT*2);
static void* const	VRAM_BP16_0		=	reinterpret_cast<void*>(VRAM_TOP);
static void* const	VRAM_BP16_1		=	reinterpret_cast<void*>(VRAM_TOP+VRAM_BUFSIZE16);
static void* const	VRAM_BP16_2		=	reinterpret_cast<void*>(VRAM_TOP+VRAM_BUFSIZE16*2);
// 32bit mode
static const u32	VRAM_BUFSIZE32	=	(VRAM_WIDTH*SCR_HEIGHT*4);
static void* const	VRAM_BP32_0		=	reinterpret_cast<void*>(VRAM_TOP);
static void* const	VRAM_BP32_1		=	reinterpret_cast<void*>(VRAM_TOP+VRAM_BUFSIZE32);
static void* const	VRAM_BP32_2		=	reinterpret_cast<void*>(VRAM_TOP+VRAM_BUFSIZE32*2);


// ------------------------------------------------------------------------------
// TYPEDEF ENUM
// ------------------------------------------------------------------------------
typedef enum
{
	SRC_COLOR					=	0,
	ONE_MINUS_SRC_COLOR			=	1,
	DST_COLOR					=	0,
	ONE_MINUS_DST_COLOR			=	1,
	SRC_ALPHA					=	2,
	ONE_MINUS_SRC_ALPHA			=	3,
	DST_ALPHA					=	4,
	ONE_MINUS_DST_ALPHA			=	5,
	DOUBLE_SRC_ALPHA			=	6,
	ONE_MINUS_DOUBLE_SRC_ALPHA	=	7,
	DOUBLE_DST_ALPHA			=	8,
	ONE_MINUS_DOUBLE_DST_ALPHA	=	9,
	FIX_VALUE					=	10
}	EBlendFunction;

typedef enum
{
	ADD							=	0,
	SUBTRACT					=	1,
	REVERSE_SUBTRACT			=	2,
	MIN							=	3,
	MAX							=	4,
	ABS							=	5
}	EBlendOperation;

typedef enum
{
	BREAK_PAUSE					=	0,
	BREAK_CANCEL				=	1
}	EBreakMode;

typedef enum
{
	NORMAL_CALL					=	0,
	SIGNAL_CALL					=	1
}	ECallMode;

typedef enum
{
	CLEAR_COLOR					=	0x0001,
	CLEAR_STENCIL				=	0x0002,
	CLEAR_DEPTH					=	0x0004,
	CLEAR_ALL					=	0x0007,
	CLEAR_FAST					=	0x0010
}	EClearMask;

typedef enum
{
	CF5650						=	0,
	CF5551						=	1,
	CF4444						=	2,
	CF8888						=	3
}	EColorFormat;

typedef enum
{
	IMMEDIATE					=	0,
	CALL						=	1,
	LIST						=	2,
	CALL_CACHED					=	3,
	LIST_CACHED					=	4
}	EDisplayListMode;

typedef enum
{
	DISPLAY_OFF					=	0,
	DISPLAY_ON					=	1
}	EDisplayStatus;

typedef enum
{
	CW							=	0,
	CCW							=	1
}	EFrontFaceMode;

typedef enum
{
	ALPHA_TEST					=	0,
	DEPTH_TEST					=	1,
	SCISSOR_TEST				=	2,
	STENCIL_TEST				=	3,
	BLEND						=	4,
	CULL_FACE					=	5,
	DITHER						=	6,
	FOG							=	7,
	CLIP_PLANE					=	8,
	TEXTURE						=	9,
	LIGHTING					=	10,
	LIGHT0						=	11,
	LIGHT1						=	12,
	LIGHT2						=	13,
	LIGHT3						=	14,
	AALINE						=	15,
	CULL_PATCH					=	16,
	COLOR_TEST					=	17,
	LOGICAL_OPERATION			=	18,
	NORMAL_REVERSE_PRIM			=	19,
	NORMAL_REVERSE_PATCH		=	20,
	COLOR_DOUBLE				=	21
}	EFunctionName;

typedef enum
{
	INT_SIGNAL					=	1,
	INT_END						=	2,
	INT_FINISH					=	4
}	EInterruptType;

typedef enum
{
	AMBIENT						=	1,
	DIFFUSE						=	2,
	SPECULAR					=	4,
	AMBIENT_AND_DIFFUSE			=	AMBIENT|DIFFUSE,
	DIFFUSE_AND_SPECULAR		=	DIFFUSE|SPECULAR,
	POWERED_DIFFUSE				=	8
}	ELightComp;

typedef enum
{
	SINGLE_COLOR				=	0,
	SEPARATE_SPECULAR_COLOR		=	1
}	ELightMode;

typedef enum
{
	LIGHT_DIRECTION				=	0,
	LIGHT_POINT					=	1,
	LIGHT_SPOT					=	2
}	ELightType;

typedef enum
{
	LOP_CLEAR					=	0,
	LOP_AND						=	1,
	LOP_AND_REVERSE				=	2,
	LOP_COPY					=	3,
	LOP_AND_INVERTED			=	4,
	LOP_NOOP					=	5,
	LOP_XOR						=	6,
	LOP_OR						=	7,
	LOP_NOR						=	8,
	LOP_EQUIV					=	9,
	LOP_INVERT					=	10,
	LOP_OR_REVERSE				=	11,
	LOP_COPY_INVERTED			=	12,
	LOP_OR_INVERTED				=	13,
	LOP_NAND					=	14,
	LOP_SET						=	15
}	ELogicalOperation;

typedef enum
{
	MATRIX_PROJECTION			=	0,
	MATRIX_VIEW					=	1,
	MATRIX_WORLD				=	2,
	MATRIX_TEXTURE				=	3
}	EMatrixType;

typedef enum
{
	ABSOLUTE_ADDRESS			=	0,
	RELATIVE_ADDRESS			=	1
}	EOrigin;

typedef enum
{
	PATCHPRIM_TRIANGLES			=	0,
	PATCHPRIM_LINES				=	1,
	PATCHPRIM_POINTS			=	2
}	EPatchPrimitiveType;

typedef enum
{
	PRIM_POINTS					=	0,
	PRIM_LINES					=	1,
	PRIM_LINE_STRIP				=	2,
	PRIM_TRIANGLES				=	3,
	PRIM_TRIANGLE_STRIP			=	4,
	PRIM_TRIANGLE_FAN			=	5,
	PRIM_RECTANGLES				=	6
}	EPrimitiveType;

typedef enum
{
	QUEUE_TAIL					=	0,
	QUEUE_HEAD					=	1
}	EQueueType;

typedef enum
{
	FLAT						=	0,
	SMOOTH						=	1
}	EShadingType;

typedef enum
{
	SIGNAL_WAIT					=	0x01,		// Generates signal interrupt (With Wait)
	SIGNAL_NOWAIT				=	0x02,		// Generates signal interrupt (Without Wait)
	SIGNAL_PAUSE				=	0x03,		// Generates signal interrupt (Pause)
	GE_SIGNAL_WAIT				=	0x01,		// Generates signal interrupt (With Wait)
	GE_SIGNAL_NOWAIT			=	0x02,		// Generates signal interrupt (Without Wait)
	GE_SIGNAL_PAUSE				=	0x03,		// Generates signal interrupt (Pause)
	GE_SIGNAL_DRAWSYNC			=	0x08,		// Drawing synchronization
	GE_SIGNAL_JUMP				=	0x10,		// Display list absolute jump
	GE_SIGNAL_CALL				=	0x11,		// Display list absolute call
	GE_SIGNAL_RET				=	0x12,		// Display list return
	GE_SIGNAL_RJUMP				=	0x13,		// Display list jump (relative from MADR)
	GE_SIGNAL_RCALL				=	0x14,		// Display list call (relative from MADR)
	GE_SIGNAL_OJUMP				=	0x15,		// Display list jump (relative from OADR)
	GE_SIGNAL_OCALL				=	0x16,		// Display list call (relative from OADR)
	GE_SIGNAL_RTBP0				=	0x20,		// Fixup to CMD_TBP0 + CMD_TBW0 (relative from MADR)
	GE_SIGNAL_RTBP1				=	0x21,		// Fixup to CMD_TBP1 + CMD_TBW1 (relative from MADR)
	GE_SIGNAL_RTBP2				=	0x22,		// Fixup to CMD_TBP2 + CMD_TBW2 (relative from MADR)
	GE_SIGNAL_RTBP3				=	0x23,		// Fixup to CMD_TBP3 + CMD_TBW3 (relative from MADR)
	GE_SIGNAL_RTBP4				=	0x24,		// Fixup to CMD_TBP4 + CMD_TBW4 (relative from MADR)
	GE_SIGNAL_RTBP5				=	0x25,		// Fixup to CMD_TBP5 + CMD_TBW5 (relative from MADR)
	GE_SIGNAL_RTBP6				=	0x26,		// Fixup to CMD_TBP6 + CMD_TBW6 (relative from MADR)
	GE_SIGNAL_RTBP7				=	0x27,		// Fixup to CMD_TBP7 + CMD_TBW7 (relative from MADR)
	GE_SIGNAL_OTBP0				=	0x28,		// Fixup to CMD_TBP0 + CMD_TBW0 (relative from OADR)
	GE_SIGNAL_OTBP1				=	0x29,		// Fixup to CMD_TBP1 + CMD_TBW1 (relative from OADR)
	GE_SIGNAL_OTBP2				=	0x2A,		// Fixup to CMD_TBP2 + CMD_TBW2 (relative from OADR)
	GE_SIGNAL_OTBP3				=	0x2B,		// Fixup to CMD_TBP3 + CMD_TBW3 (relative from OADR)
	GE_SIGNAL_OTBP4				=	0x2C,		// Fixup to CMD_TBP4 + CMD_TBW4 (relative from OADR)
	GE_SIGNAL_OTBP5				=	0x2D,		// Fixup to CMD_TBP5 + CMD_TBW5 (relative from OADR)
	GE_SIGNAL_OTBP6				=	0x2E,		// Fixup to CMD_TBP6 + CMD_TBW6 (relative from OADR)
	GE_SIGNAL_OTBP7				=	0x2F,		// Fixup to CMD_TBP7 + CMD_TBW7 (relative from OADR)
	GE_SIGNAL_RCBP				=	0x30,		// Fixup to CMD_CBP + CMD_CBW   (relative from MADR)
	GE_SIGNAL_OCBP				=	0x38,		// Fixup to CMD_CBP + CMD_CBW   (relative from OADR)
	GE_SIGNAL_BREAK				=	0xFF		// Display breakpoint (TOOL only)
}	ESignalMode;

typedef enum
{
	CLOSE_CLOSE					=	0,
	OPEN_CLOSE					=	1,
	CLOSE_OPEN					=	2,
	OPEN_OPEN					=	3
}	ESplineParameter;

typedef enum
{
	NOFLIP						=	0,
	FLIP_U						=	1,
	FLIP_V						=	2,
	FLIP_UV						=	3
}	ESpriteFlip;

typedef enum
{
	NOROTATE					=	0,
	ROTATE_90					=	1,
	ROTATE_180					=	2,
	ROTATE_270					=	3
}	ESpriteRotate;

typedef enum
{
	KEEP						=	0,
	ZERO						=	1,
	REPLACE						=	2,
	INVERT						=	3,
	INCR						=	4,
	DECR						=	5
}	EStencilOperation;

typedef enum
{
	SYNC_WAIT					=	0,
	SYNC_NOWAIT					=	1
}	ESyncMode;

typedef enum
{
	SYNC_FINISH					=	0,
	SYNC_SIGNAL					=	1,
	SYNC_END					=	2,
	SYNC_LIST					=	3,
	SYNC_SENDLIST				=	4
}	ESyncType;

typedef enum
{
	NEVER						=	0,
	ALWAYS						=	1,
	EQUAL						=	2,
	NOTEQUAL					=	3,
	LESS						=	4,
	LEQUAL						=	5,
	GREATER						=	6,
	GEQUAL						=	7
}	ETestFunction;

typedef enum
{
	TEXT_COPY					=	0,
	TEXT_XOR					=	1
}	ETextMode;

typedef enum
{
	TEXBUF_NORMAL				=	0,
	TEXBUF_FAST					=	1
}	ETextureBufferMode;

typedef enum
{
	RGB							=	0,
	RGBA						=	1
}	ETextureColorComponent;

typedef enum
{
	NEAREST						=	0,		// 000
	LINEAR						=	1,		// 001
	NEAREST_MIPMAP_NEAREST		=	4,		// 100
	LINEAR_MIPMAP_NEAREST		=	5,		// 101
	NEAREST_MIPMAP_LINEAR		=	6,		// 110
	LINEAR_MIPMAP_LINEAR		=	7		// 111
}	ETextureFilterMode;

typedef enum
{
	TF5650						=	0,
	TF5551						=	1,
	TF4444						=	2,
	TF8888						=	3,
	TFIDX4						=	4,
	TFIDX8						=	5,
	TFIDX16						=	6,
	TFIDX32						=	7,
	TFDXT1						=	8,
	TFDXT3						=	9,
	TFDXT5						=	10,
	TFDXT1EXT					=	256+TFDXT1,
	TFDXT3EXT					=	256+TFDXT3,
	TFDXT5EXT					=	256+TFDXT5
}	ETextureFormat;

typedef enum
{
	TEX_MODULATE				=	0,
	TEX_DECAL					=	1,
	TEX_BLEND					=	2,
	TEX_REPLACE					=	3,
	TEX_ADD						=	4
}	ETextureFunction;

typedef enum
{
	LOD_AUTO					=	0,
	LOD_CONSTANT				=	1,
	LOD_SLOPE					=	2
}	ETextureLevelMode;

typedef enum
{
	PROJ_XYZ					=	0,
	PROJ_UV						=	1,
	PROJ_NORMALIZE_NORMAL		=	2,
	PROJ_NORMAL					=	3
}	ETextureMapInput;

typedef enum
{
	UV_MAP						=	0,
	PROJECTION_MAP				=	1,
	SHADE_MAP					=	2
}	ETextureMapMode;

typedef enum
{
	SINGLE_CLUT					=	0,
	MULTI_CLUT					=	1
}	ETextureClutMode;

typedef enum
{
	REPEAT						=	0,
	CLAMP						=	1
}	ETextureWrapMode;

typedef enum
{
	TEXTURE_NONE				=	0 <<  0,
	TEXTURE_UBYTE				=	1 <<  0,
	TEXTURE_USHORT				=	2 <<  0,
	TEXTURE_FLOAT				=	3 <<  0,

	COLOR_NONE					=	0 <<  2,
	COLOR_CF5650				=	4 <<  2,
	COLOR_CF5551				=	5 <<  2,
	COLOR_CF4444				=	6 <<  2,
	COLOR_CF8888				=	7 <<  2,

	NORMAL_NONE					=	0 <<  5,
	NORMAL_BYTE					=	1 <<  5,
	NORMAL_SHORT				=	2 <<  5,
	NORMAL_FLOAT				=	3 <<  5,

	VERTEX_NONE					=	0 <<  7,
	VERTEX_BYTE					=	1 <<  7,
	VERTEX_SHORT				=	2 <<  7,
	VERTEX_FLOAT				=	3 <<  7,

	WEIGHT_NONE					=	0 <<  9,
	WEIGHT_UBYTE				=	1 <<  9,
	WEIGHT_USHORT				=	2 <<  9,
	WEIGHT_FLOAT				=	3 <<  9,

	INDEX_NONE					=	0 << 11,
	INDEX_UBYTE					=	1 << 11,
	INDEX_USHORT				=	2 << 11,

	WEIGHT_1					=	0 << 14,
	WEIGHT_2					=	1 << 14,
	WEIGHT_3					=	2 << 14,
	WEIGHT_4					=	3 << 14,
	WEIGHT_5					=	4 << 14,
	WEIGHT_6					=	5 << 14,
	WEIGHT_7					=	6 << 14,
	WEIGHT_8					=	7 << 14,

	VERTEX_1					=	0 << 18,
	VERTEX_2					=	1 << 18,
	VERTEX_3					=	2 << 18,
	VERTEX_4					=	3 << 18,
	VERTEX_5					=	4 << 18,
	VERTEX_6					=	5 << 18,
	VERTEX_7					=	6 << 18,
	VERTEX_8					=	7 << 18,

	THROUGH						=	1 << 23
}	EVertexFormat;


// ------------------------------------------------------------------------------
// TYPEDEFS
// ------------------------------------------------------------------------------
typedef u32					ColorRGB;
typedef u32					ColorRGBA;
typedef u32					ClearMask;
typedef u32					LightComp;
typedef u32					VertexFormat;
typedef void				InterruptCallback(	int );
typedef SceGeContext		GeContext;


// ------------------------------------------------------------------------------
// INLINE FUNCTIONS
// ------------------------------------------------------------------------------
inline f32				DegreeToRadian(	f32		x )
{
	return	x*(3.1415926535897932384626433832f/180.0f);
}


// ------------------------------------------------------------------------------
// FUNCTIONS
// ------------------------------------------------------------------------------
// Initialization/Termination functions
int					Init(				void );
int					Term(				void );

// Display list functions
int					Start(				EDisplayListMode		mode,
										const void*				p,
										u32						size );
int					Finish(				void );
int					FinishId(			u16						id );
void				Signal(				ESignalMode				mode,
										u16						id	);
int					Break(				EBreakMode				mode );
int					Continue(			void );
void				CallMode(			ECallMode				mode );
int					CallList(			const void*				p );
void*				JumpList(			const void*				p );
int					SendList(			EQueueType				mode,
										const void*				p,
										GeContext*				ctx = 0,
										int						stackDepth = 0,
										SceGeStack*				pStack = 0 );
u32					CheckList(			void );
int					Flush(				void );
int					Sync(				ESyncType				type,
										ESyncMode				mode );
//void				SignalSync(			u16						id );		// sceePspGu do not manage SignalSync -> user responsability in Signal interrupt!
void*				BeginPacket(		u32*					pSize );
void				EndPacket(			void*					pEnd );
void*				GetAlignedMemory(	u32						align,		// power of 2
										u32						size );
void*				GetMemory(			u32						size );
void*				GetMemoryEnd(		u32						size );
EDisplayStatus		Display(			EDisplayStatus			sw );
InterruptCallback*	SetCallback(		EInterruptType			id,
										InterruptCallback*		cb );

// Commands funtions
void				SendCommandf(		u8						cmd,
										f32						val );
void				SendCommandi(		u8						cmd,
										u32						val );

// Mode functions
void				Enable(				EFunctionName			cap );
void				Disable(			EFunctionName			cap );
u32					GetStatus(			EFunctionName			cap );
void				SetStatus(			EFunctionName			cap,
										bool					flag );
u32					GetAllStatus(		void );
void				SetAllStatus(		u32						status );

// Object culling functions
void				BeginObject(		VertexFormat			vf,
										u8						count,
										const void*				index,
										const void*				p );
int					EndObject(			void );

// Primitive functions
void				DrawArray(			EPrimitiveType			prim,
										VertexFormat			vf,
										u16						count,
										const void*				i,
										const void*				p );
void				DrawArrayN(			EPrimitiveType			prim,
										VertexFormat			vf,
										u16						nverts,
										u32						nprims,
										const void*				i,
										const void*				p );
void				DrawBezier(			VertexFormat			vf,
										u8						ucount,
										u8						vcount,
										const void*				i,
										const void*				p );
void				DrawSpline(			VertexFormat			vf,
										u8						ucount,
										u8						vcount,
										ESplineParameter		uflag,
										ESplineParameter		vflag,
										const void*				i,
										const void*				p );
void				DrawSprite(			s16						x,
										s16						y,
										s16						z,
										s16						u,
										s16						v,
										ESpriteFlip				flip,
										ESpriteRotate			rotate );
void				PatchDivide(		u8						udiv,
										u8						vdiv );
void				PatchPrim(			EPatchPrimitiveType		prim );
void				PatchFrontFace(		EFrontFaceMode			mode );
void				SpriteMode(			s16						sw,
										s16						sh,
										s16						dw,
										s16						dh );

// Clearing functions
void				ClearColor(			ColorRGB				col );
void				ClearStencil(		u8						stencil );
void				ClearDepth(			u16						depth );
void				Clear(				ClearMask				mask );

// Buffer Transfer functions
void				CopyImage(			EColorFormat			mode,
										u16						sx,
										u16						sy,
										u16						width,
										u16						height,
										u32						sbw,
										const void*				sbp,
										u16						dx,
										u16						dy,
										u32						dbw,
										const void*				dbp );

// Buffer Specification functions
void				DrawBufferC(		const void*				fbp,
										u16						fbw,
										EColorFormat			fpf,
										u16						width,
										u16						height );
void				DrawBuffer(			EColorFormat			fpf,
										const void*				fbp,
										u16						fbw );
void				DrawBufferList(		EColorFormat			fpf,
										const void*				fbp,
										u16						fbw );
void				DepthBuffer(		const void*				zbp,
										u16						zbw );
void				DispBufferC(		const void*				dbp,
										u16						dbw,
										EColorFormat			dpf,
										u16						width,
										u16						height );
void				DispBuffer(			u16						width,
										u16						height,
										const void*				dbp,
										u16						dbw );
void*				SwapBuffers(		void );

// Viewport Transform functions
void				Viewport(			u16						x,
										u16						y,
										u16						width,
										u16						height );
void				DepthRange(			u16						znear,
										u16						zfar );
void				Offset(				u16						x,
										u16						y );
void				DepthOffset(		s32						offset );

// Color & Lighting functions
void				FrontFace(			EFrontFaceMode			mode );
void				ShadeModel(			EShadingType			model );
	// Scene
void				Ambient(			ColorRGBA				ac );
	// Material
void				ColorMaterial(		LightComp				comp );
void				Emission(			ColorRGB				mec );
void				Material(			LightComp				comp,
										ColorRGBA				col );
void				Color(				ColorRGBA				col );
void				AmbientColor(		ColorRGBA				mac );
void				ModelColor(			ColorRGB				mec,
										ColorRGBA				mac,
										ColorRGB				mdc,
										ColorRGB				msc );
void				Specular(			f32						power );
	// Light
void				LightMode(			ELightMode				ssc );
void				Light(				u32						n,
										ELightType				type,
										LightComp				comp,
										const ScePspFVector3*	vec );
void				LightAtt(			u32						n,
										f32						a,
										f32						b,
										f32						c );
void				LightSpot(			u32						n,
										f32						s,
										f32						c,
										const ScePspFVector3*	vec );
void				LightColor(			u32						n,
										LightComp				comp,
										ColorRGB				col );

// Mipmapping functions
void				TexLevelMode(		ETextureLevelMode		calc,
										f32						offset );
void				TexSlope(			f32						tslope );

// Texture Mapping functions
void				TexMode(			ETextureFormat			tpf,
										u32						maxlod,
										ETextureClutMode		mc,
										ETextureBufferMode		hsm );
void				TexFilter(			ETextureFilterMode		min,
										ETextureFilterMode		mag );
void				TexFunc(			ETextureFunction		func,
										ETextureColorComponent	component );
void				TexEnvColor(		ColorRGB				col );
void				TexWrap(			ETextureWrapMode		uwrap,
										ETextureWrapMode		vwrap );
void				TexMapMode(			ETextureMapMode			map,
										u32						lu,
										u32						lv,
										ETextureMapInput		tmi = PROJ_XYZ );
//void				TexProjMapMode(		ETextureMapInput		tmi );		// sceePspGu doesn't manage TexProjMapMode
void				TexSync(			void );
void				TexFlush(			void );
void				TexImage(			u32						level,
										u16						width,
										u16						height,
										u16						w,
										const void*				tbp );
void				TexScale(			f32						su,
										f32						sv );
void				TexOffset(			f32						tu,
										f32						tv );

// CLUT functions
void				ClutMode(			EColorFormat			cpf,
										u8						shift,
										u8						mask,
										u8						csa );
void				ClutLoad(			u8						np,
										const void*				cbp );

// Fog
void				Fog(				f32						dfs,
										f32						dfe,
										ColorRGB				col );

// Pixel Operations
void				Scissor(			u16						x,
										u16						y,
										u16						width,
										u16						height );
void				ColorFunc(			ETestFunction			func,			// 4 tests availables!
										ColorRGB				cref,
										ColorRGB				cmsk );
void				AlphaFunc(			ETestFunction			func,
										u8						aref,
										u8						amsk );
void				StencilFunc(		ETestFunction			func,
										u8						sref,
										u8						smsk );
void				StencilOp(			EStencilOperation		sfail,
										EStencilOperation		zfail,
										EStencilOperation		zpass );
void				DepthFunc(			ETestFunction			func );
void				DepthMask(			bool					flag );
void				BlendFunc(			EBlendOperation			equ,
										EBlendFunction			asel,
										EBlendFunction			bsel,
										ColorRGB				afix,
										ColorRGB				bfix );
void				SetDither(			const ScePspIMatrix4*	d );
void				LogicalOp(			ELogicalOperation		lop );
void				PixelMask(			ColorRGBA				pixelMask );

// Deform Parameter
void				BoneMatrix(			u32						n,
										const ScePspFMatrix4*	matrix );
void				MorphWeight(		u32						n,
										f32						f );

// Matrix Set
void				SetMatrix(			EMatrixType				mode,
										const ScePspFMatrix4*	matrix );

// DebugPrint
void				DebugBegin(			void );
void				DebugEnd(			void );
void				DebugPrint(			s32						x,
										s32						y,
										ColorRGB				col,
										const char*				str,
										ETextMode				mode = TEXT_COPY );
void				DebugFlush(			void );
// Origin management
void				SetOffset(			const void*				p );
void*				GetOffset(			void );
void				Origin(				EOrigin					org =	ABSOLUTE_ADDRESS );
// DisplayList context
u32					GetContextSize(		void );
void*				SetContextPtr(		void*					p );
// 64 bits compatibility
void				SetBase64(			void*					p );
// Sync sceGeEdramSetAddrTranslation with display list
void				AddrTranslation(	int						width );


// ------------------------------------------------------------------------------
}	// namespace Gu
}	// namespace Psp
}	// namespace scee


// ------------------------------------------------------------------------------
// EXTRA LIBGU FUNCTIONS
// ------------------------------------------------------------------------------
#if		defined(__cplusplus)
extern "C" {
#endif	// defined(__cplusplus)

void				sceGuDrawBufferC(	const void*				fbp,
										int						fbw,
										int						fpf,
										int						width,
										int						height );
void				sceGuDispBufferC(	const void*				dbp,
										int						dbw,
										int						dpf,
										int						width,
										int						height );
void*				sceGuGetMemoryEnd(	int						size );
void				sceGuEmission(		int						mec );
void				sceGuDebugBegin(	void );
void				sceGuDebugEnd(		void );

#if		defined(__cplusplus)
}	// extern "C"
#endif	// defined(__cplusplus)


// ------------------------------------------------------------------------------
#endif	// SCEEPSPGU_H
