//////////////////////////////////////////////////////////////////////////////
//
// AVP_Behaviour_AVP3Alien_Wander.cpp
//
// Copyright (C) 2007 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// Core
#include "Core/Asura.h"
// Header
#include "Behaviour/BehaviourTypes/AVP3Alien/AVP_Behaviour_AVP3Alien_Flee.h"
// Asura
#include "AI/Asura_AI_Navigation.h"
#include "Core/Asura_HashedList.h"
#ifndef ASURA_NO_DEBUG_OUTPUT
#include "Rendering/Asura_Font_System.h"
#endif

// AVP Axon
#include "Core/BrainTypes/Axon_Project_Brain.h"
#include "Behaviour/BehaviourTypes/AVP3Alien/AVP_Behaviour_AVP3Alien_Movement.h"
#include "Behaviour/BehaviourTypes/AVP_Behaviour_Types.h"

// AVP
#include "Actor/Vertebrates/Alien/AVP_Alien_Description.h"
#include "Animation/Asura_AnimationController_Animation.h"
#include "Entity/PhysicalObject/Actor/AVP_ServerEntity_Actor.h"
//#include "Entity/Spawners/AVP_ServerEntity_SpawnPoint.h"



//////////////////////////////////////////////////////////////////////////////
// AVP_Behaviour_AVP3Alien_Flee :: Constructor
//////////////////////////////////////////////////////////////////////////////
AVP_Behaviour_AVP3Alien_Flee::AVP_Behaviour_AVP3Alien_Flee(Axon_Project_Brain* pxBrain)
: PARENT(pxBrain, AXON_BEHAVIOUR_TYPE_AVP3ALIEN_FLEE)
, m_eCurrentState(WAITING)
, m_fTimeInState(0.0f)
, m_fWaitTime(0.0f)
{

}

//////////////////////////////////////////////////////////////////////////////
// AVP_Behaviour_AVP3Alien_Flee :: Deactivate
//////////////////////////////////////////////////////////////////////////////
void AVP_Behaviour_AVP3Alien_Flee::Deactivate()
{
	// Destroy any children.
	Axon_Behaviour* pxChildBehaviour = GetChild();
	while ( pxChildBehaviour )
	{
		UnlinkChild( pxChildBehaviour );
		GetProjectBrain()->DestroyBehaviour( pxChildBehaviour );
		pxChildBehaviour = GetChild();
	}

	PARENT::Deactivate();
}

//////////////////////////////////////////////////////////////////////////////
// AVP_Behaviour_AVP3Alien_Flee :: Update
//////////////////////////////////////////////////////////////////////////////
void AVP_Behaviour_AVP3Alien_Flee::Update(const float fTimeslice)
{
	PARENT::Update(fTimeslice);

	m_fTimeInState += fTimeslice;

	switch (m_eCurrentState)
	{
		//-----------------------------------------------------------
		case WAITING:
		{
			if (m_fTimeInState > m_fWaitTime)
			{
				SwitchToState(FLEE);
			}
			return;
		}

		//-----------------------------------------------------------
		case FLEE:
		{
			AVP_Behaviour_AVP3Alien_Movement* pxGTPBehaviour = static_cast<AVP_Behaviour_AVP3Alien_Movement*>(GetChildWithType(this, AXON_BEHAVIOUR_TYPE_AVP3ALIEN_MOVEMENT));
			
			if (!pxGTPBehaviour || pxGTPBehaviour->IsInError()) 
			{
				SwitchToState(WAITING);
				return;
			}

			if (pxGTPBehaviour->IsFinished())
			{
				SwitchToState(REMOVE_FROM_WORLD);
				return;
			}

			return;
		}

		//-----------------------------------------------------------
		case REMOVE_FROM_WORLD:
		{
			return;
		}

		//-----------------------------------------------------------
		case WAIT_TO_RETURN:
		{
			return;
		}
	}
}

//////////////////////////////////////////////////////////////////////////////
// AVP_Behaviour_AVP3Alien_Flee :: FindDespawnPosition
//////////////////////////////////////////////////////////////////////////////
Asura_Vector_3 AVP_Behaviour_AVP3Alien_Flee::FindDespawnPosition()
{
#if 0
	AVP_ServerEntity_Actor*			pxOurServerActor	= m_pxBrain->GetActor();
	AVP_ServerEntity_SpawnPoint*	pxBestSpawnPoint	= NULL;
	float							fBestDistance		= Asura_Maths::LargeFloat;

	for (Asura_Hashed_List_It<AVP_ServerEntity_SpawnPoint> it(AVP_ServerEntity_SpawnPoint::GetList()); !it.Done(); it.Next())
	{
		AVP_ServerEntity_SpawnPoint* pxSpawnPoint = it.GetCurrent();

		if (pxSpawnPoint)
		{
			if (pxSpawnPoint->GetSpawnType() == AVP_ServerEntity_SpawnPoint::SPECIES_ALIEN)
			{
				float fDistance = (pxSpawnPoint->GetPosition() - pxOurServerActor->GetPosition()).MagnitudeSquared();

				// find the closest spawn point
				if (fDistance < fBestDistance)
				{
					fBestDistance = fDistance;
					pxBestSpawnPoint = pxSpawnPoint;
				}
			}
		}
	}

	if (!pxBestSpawnPoint)
	{
		Asura_Assert(false, "AVP_Behaviour_AVP3Alien_Flee::FindDespawnPosition - No Spawn points anywhere! can't flee, put some spawn points in, this behaviour will not run again");
		return Asura_Maths::ZeroVector3;
	}
	else
	{
		return pxBestSpawnPoint->GetPosition();
	}
#endif

	return Asura_Maths::ZeroVector3;
}

//////////////////////////////////////////////////////////////////////////////
// AVP_Behaviour_AVP3Alien_Flee :: SwitchToState
//////////////////////////////////////////////////////////////////////////////
void AVP_Behaviour_AVP3Alien_Flee::SwitchToState(const State eNextState)
{
	// Do any work we need to exit our current state.
	switch (m_eCurrentState)
	{
		//-----------------------------------------------------------
		case FLEE:
		{
			AVP_Behaviour_AVP3Alien_Movement* pxGTPBehaviour = static_cast<AVP_Behaviour_AVP3Alien_Movement*>(GetChildWithType(this, AXON_BEHAVIOUR_TYPE_AVP3ALIEN_MOVEMENT));

			if (pxGTPBehaviour)
			{
				DestroyChildWithType(AXON_BEHAVIOUR_TYPE_AVP3ALIEN_MOVEMENT);
			}

		}break;

		//-----------------------------------------------------------
		case WAITING:
		{
			m_fWaitTime = Asura_Random::GetFloat(1.0f, 3.0f);
		}break;

		//-----------------------------------------------------------
		case REMOVE_FROM_WORLD:
		{

		}break;

		//-----------------------------------------------------------
		case WAIT_TO_RETURN:
		{

		}break;
	}

	// Do any work we need to change into our next state.
	m_eCurrentState = eNextState;
	m_fTimeInState  = 0.0f;

	switch (m_eCurrentState)
	{
		//-----------------------------------------------------------
		case FLEE:
		{
			// Find a destination
			Asura_Vector_3 xDestination = FindDespawnPosition();

			// Zero vector received, so there isn't a spawn point available.
			if (xDestination == Asura_Maths::ZeroVector3)
			{
				// switch to waiting and then set behaviour as finished
				SwitchToState(WAITING);
				SetFinished();
				return;
			}

			//-----------------------------------------------------------
			// Navigation Data
			//-----------------------------------------------------------
			const AVP_Navigation::NavData& xNavData = GetProjectBrain()->GetNavData();
			AVP_Navigation::TestParams xTestParams;
			xTestParams.m_uPathfinderFlags	= xNavData.m_uPathfinderFlags;
			xTestParams.m_ucZone			= xNavData.m_ucCurrentZone;
			AVP_Navigation::ResolveToValidPosition(xDestination, xTestParams);

			//-----------------------------------------------------------
			// Create the movement child behaviour
			//-----------------------------------------------------------
			AVP_Behaviour_AVP3Alien_Movement* pxGTPBehaviour = static_cast<AVP_Behaviour_AVP3Alien_Movement*>(CreateChildWithType(AXON_BEHAVIOUR_TYPE_AVP3ALIEN_MOVEMENT));

			if ((!pxGTPBehaviour) || (!pxGTPBehaviour->GetToPosition(xDestination)))
			{
				SwitchToState(WAITING);
				SetFinished();
				return;
			}

			return;
		}

		//-----------------------------------------------------------
		case REMOVE_FROM_WORLD:
		{
			// Code to get in a spawner
			AVP_ServerEntity_Actor* pxServerActor = m_pxBrain->GetActor();

			if (pxServerActor)
			{
				// Hide the Alien and turn off their physics
				pxServerActor->HideForCutscene( true );	
				pxServerActor->SetNeverUpdates( true );

				pxServerActor->GetPhysicsObject()->ChangeType(ASURA_PHYSICS_OBJECT_EXTERNALCONTROL);
				pxServerActor->SetPhysicsObjectIgnore(true);

				SwitchToState(WAIT_TO_RETURN);
			}

			return;
		}

		//-----------------------------------------------------------
		case WAITING:
		{

			return;	
		}
		default:
		{
			return;
		}
	}
}

#ifndef ASURA_NO_DEBUG_OUTPUT
//////////////////////////////////////////////////////////////////////////////
// AVP_Behaviour_AVP3Alien_Flee :: RenderDebugInfo
//////////////////////////////////////////////////////////////////////////////
static const char* s_aszStateStrings[] = { /*0*/"WAITING", /*1*/"FLEE", /*2*/"REMOVE_FROM_WORLD", /*3*/"WAIT_TO_RETURN" };
//////////////////////////////////////////////////////////////////////////////
void AVP_Behaviour_AVP3Alien_Flee::RenderDebugInfo()
{
	PARENT::RenderDebugInfo();

	Asura_Font_System::Print
	(
		m_pxBrain->GetDebugOutputPos().x,
		m_pxBrain->GetDebugOutputPos().y,
		0.5f,
		0xFFFFFF08,
		ASURA_TEXTFORMAT_CENTREJUSTIFIED|ASURA_TEXTFORMAT_FORMATSTRING,
		"%s", s_aszStateStrings[m_eCurrentState]
	);

	m_pxBrain->GetDebugOutputPos().y += 10.0f;
}
#endif
