//////////////////////////////////////////////////////////////////////////////
//
// AVP_Behaviour_AVP3Alien_KeepDistance.h
//
// Copyright (C) 2007 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __AVP_BEHAVIOUR_AVP3ALIEN_KEEPDISTANCE_H__
#define __AVP_BEHAVIOUR_AVP3ALIEN_KEEPDISTANCE_H__ 1

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Behaviour/Axon_Project_Behaviour.h"
#include "Core/AVP_Navigation.h"

// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////////

class Axon_Project_Brain;

//////////////////////////////////////////////////////////////////////////////
//
// AVP_Behaviour_AVP3Alien_KeepDistance
//
// Low-level behaviour that follows a moving target at a given distance
// (with tolerance)
//
//////////////////////////////////////////////////////////////////////////////

class AVP_Behaviour_AVP3Alien_KeepDistance : public Axon_Project_Behaviour
{
    typedef Axon_Project_Behaviour PARENT;

	static const float fDEFAULT_TOLERANCE;

	enum
	{
		STATE_IDLE,
		STATE_GO_CLOSER,
		STATE_ORIENT_TO,
		STATE_GET_FARTHER
	};

public:
    AVP_Behaviour_AVP3Alien_KeepDistance( Axon_Project_Brain* pxBrain );

    virtual bool Activate();
    virtual void Deactivate();
    virtual void Update( const float fTimeslice );
    virtual void PerFrameUpdate( const float fTimeslice );

    void SetTargetGuid( Asura_Guid uTargetGuid );
    void SetFollowDist( float fFollowDist, float fTolerance = fDEFAULT_TOLERANCE ) { m_fFollowDist = fFollowDist; m_fTolerance = fTolerance; }

#ifndef ASURA_NO_DEBUG_OUTPUT
    virtual void RenderDebugInfo();
#endif

private:
	void SwitchToState( int iNextState );

	Asura_Guid m_uTargetGuid;
	Asura_Vector_3 m_xTargetPosition;

	float m_fFollowDist;
    float m_fTolerance;
};

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

#endif //__AVP_BEHAVIOUR_AVP3ALIEN_KEEPDISTANCE_H__
