//////////////////////////////////////////////////////////////////////////////
//
// AVP_Behaviour_Panic_Mechanic_Trigger.cpp
//
// Created by NJD - 22 December 2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"

// header
#include "Behaviour/BehaviourTypes/AVP_Behaviour_Panic_Mechanic_Trigger.h"

// asura
#include "Network/Asura_Network.h"

// axon
#include "Behaviour/BehaviourTypes/AVP_Behaviour_Types.h"
#include "Core/BrainTypes/Axon_Project_Brain.h"

// avp
#include "Entity/AVP_Messages.h"
#include "Moves/AVP_Movement_System.h"
#include "Entity/PhysicalObject/Actor/AVP_ServerEntity_Actor.h"
#include "Entity/AVP_Entity_Classification.h"
#include "Moves/Grapple/AVP_Grapple_Messages.h"

//////////////////////////////////////////////////////////////////////////////
//
// AVP_Behaviour_Panic_Mechanic_Trigger
//
//////////////////////////////////////////////////////////////////////////////

AVP_Behaviour_Panic_Mechanic_Trigger::AVP_Behaviour_Panic_Mechanic_Trigger( Axon_Project_Brain* pxBrain )
: PARENT( pxBrain, AXON_BEHAVIOUR_TYPE_PANIC_MECHANIC_TRIGGER )
, m_eUpdateState( US_UNSET )
{
}

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

void AVP_Behaviour_Panic_Mechanic_Trigger::Update( const float fTimeslice )
{
    PARENT::Update( fTimeslice );

    if ( m_eUpdateState == US_UNSET )
    {
		// We know we are going to grapple but we don't have a clue who!?!
		// Find a likely candidate here:
		Asura_Guid uVictimGuid = ASURA_GUID_UNREGISTERED;

		uVictimGuid = FindLikeleyTarget();

		if(uVictimGuid == ASURA_GUID_UNREGISTERED)
		{
			m_eUpdateState = US_TRIGGERED_PANIC_MECHANIC;
			SetFinished();
			return;
		}

        // Send the panic mechanic message trigger and switch state.
        Asura_Message* pxMessage = Asura_Network::CreateMessage( ASURA_MSG_MOVE_START, sizeof( AVP_Msg_Grapple_JumpMove_Start ) );
        if ( pxMessage )
        {
            AVP_Msg_Grapple_JumpMove_Start* pxData = static_cast< AVP_Msg_Grapple_JumpMove_Start* >( pxMessage->GetData() );
            pxData->SetMoveID(Asura_GetHashID( "AVP_Attacker_MovesGrapple_Jump" ));
			pxData->SetAttackerGuid( m_pxBrain->GetOwnerGuid() );
			pxData->SetVictimGuid( uVictimGuid );
			pxData->SetCanDodge(false);
     
            pxMessage->SetToGuid( m_pxBrain->GetOwnerGuid() );
            pxMessage->SetFromGuid( m_pxBrain->GetOwnerGuid() );
            pxMessage->UseGuaranteedDelivery();

            Asura_Network::Send( pxMessage );
        }

        m_eUpdateState = US_TRIGGERED_PANIC_MECHANIC;
    }
	else if(m_eUpdateState == US_TRIGGERED_PANIC_MECHANIC )
	{
		SetFinished();
	}
}

Asura_Guid AVP_Behaviour_Panic_Mechanic_Trigger::FindLikeleyTarget( void )
{
	Asura_Guid uClosest		= ASURA_GUID_UNREGISTERED;

	// This function returns the guid of the most likely target to grapple
	AVP_ServerEntity_Actor* pxServer = m_pxBrain->GetActor();
	if(pxServer)
	{
		// We have a server doing the grapple, get some nearby enemies
		float fClosestDistance	= Asura_Maths::LargeFloat;
		
		const Asura_Hashed_List<AVP_ServerEntity_Actor>* pxList = AVP_ServerEntity_Actor::GetList();
		Asura_Hashed_List_It<AVP_ServerEntity_Actor> xIterator(pxList);
		while(!xIterator.Done())
		{
			AVP_ServerEntity_Actor* pxVictim = xIterator.GetCurrent();
			float fDist = (pxVictim->GetPosition() - pxServer->GetPosition()).Magnitude();
			if( (fDist < fClosestDistance) &&  // Only if closer that current closest
				(pxServer->GetClassification() != pxVictim->GetClassification()) && // Can't grapple members of our class
				(pxServer->GetClassification() != AVP_ENTITYCLASS_ALIENFACEHUGGER) &&	// Can't grapple facehuggers
				(!pxVictim->IsDead()) // Can't grapple to dead!
				)
			{
				fClosestDistance = fDist;
				uClosest = pxVictim->GetGuid();
			}
			xIterator.Next();
		}
	}
	return(uClosest);
}

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////
