//////////////////////////////////////////////////////////////////////////////
//
// AVP_Behaviour_Alien_Ambush.cpp
//
// Copyright (C) 2008 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"

// header
#include "Behaviour/BehaviourTypes/Alien/AVP_Behaviour_Alien_Ambush.h"

// asura
#include "Core/Asura_LOS.h"
#include "Entity/PhysicalObject/Moves/Asura_Moves.h"
#include "Network/Asura_Message.h"
#ifndef ASURA_NO_DEBUG_OUTPUT
#include "Debugging/Asura_DebugLineRender.h"
#include "Rendering/Asura_Font_System.h"
#endif


// avp
#include "Entity/Controller/AVP_ServerEntity_Team_Alien.h"
#include "Entity/Controller/AVP_ServerEntity_Squad_Alien.h"
#include "Entity/Damage/AVP_EntityDamage_Request.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/AVP_Vertebrate_Messages.h"
#include "Moves/Vertebrates/Alien/AVP_Alien_MovesJump.h"

// axon
#include "Behaviour/BehaviourTypes/AVP_Behaviour_Types.h"
#include "Behaviour/BehaviourTypes/Alien/AVP_Behaviour_Alien_Attack2.h"
#include "Core/AVP_Navigation_Types.h"
#include "Core/AVP_Waypoint_Manager.h"
#include "Core/BrainTypes/AVP_Brain_Actor_Alien.h"
#include "Knowledge/AVP_Knowledge_System_AlienTeam.h"

DEBUGVAR float dbg_fLOSTestDistance = 12.0f;
DEBUGVAR float dbg_fPreAttackTime = 0.5f;

//////////////////////////////////////////////////////////////////////////////
//
// AVP_Behaviour_Alien_Ambush
//
//////////////////////////////////////////////////////////////////////////////

AVP_Behaviour_Alien_Ambush::AVP_Behaviour_Alien_Ambush( AVP_Brain_Actor_Alien* pxBrain )
: PARENT( pxBrain, AVP_BEHAVIOUR_TYPE_ALIEN_AMBUSH )
, m_fLastHealth(pxBrain->GetActor()->GetHealth())
, m_uSeenTarget(ASURA_GUID_UNREGISTERED)
, m_xDelayedMsgList()
{
	SetCurrentState(UNSET);
}

bool AVP_Behaviour_Alien_Ambush::Activate()
{
	return PARENT::Activate();
}

void AVP_Behaviour_Alien_Ambush::Deactivate()
{
	SwitchToState( UNSET );
	PARENT::Deactivate();
}


void AVP_Behaviour_Alien_Ambush::PerFrameUpdate( const float fTimeslice )
{
	PARENT::PerFrameUpdate( fTimeslice );

	if (fTimeslice == 0.0f)
	{
		return;
	}

	UpdateStateTimer(fTimeslice);

	if (GetAlienBrain()->CanUpdateMelee())
	{
		LateHandleMessage(); 
	}

	if (!GetAlienBrain()->CanPlanMove())
	{
		return;
	}

	switch (GetCurrentState())
	{
	case UNSET:
		SwitchToState(AMBUSH);
		break;
	case AMBUSH:
		UpdateAmbush();
		break;
	case HISS:
		UpdateHiss();
		break;
	case PREATTACK:
		UpdatePreAttack();
		break;
	case ATTACK:
		UpdateAttack();
	case FINISHED:
		break;
	default:
		Asura_Assert(false, "Unexpected state");
		break;
	}
}

bool AVP_Behaviour_Alien_Ambush::HandleMessage( const Asura_Message* pxMessage )
{
	switch ( GetCurrentState() )
	{
	case HISS:
		if ( pxMessage->GetMsgID() == ASURA_MSG_MOVECHAINCOMPLETE )
		{
			GetAlienBrain()->SetMelee(false);
			m_xDelayedMsgList.Push_Back(ASURA_MSG_MOVECHAINCOMPLETE);
			return true;
		}
		break;
	default:
		break;
	}

	return PARENT::HandleMessage( pxMessage );
}


#ifndef ASURA_NO_DEBUG_OUTPUT

static const char* s_aszStateStrings[] =
{
	"UNSET",
	"AMBUSH",
	"PREATTACK",
	"HISS",
	"ATTACK",
	"FINISH",
};

void AVP_Behaviour_Alien_Ambush::RenderDebugInfo()
{
	PARENT::RenderDebugInfo();	

	Asura_Font_System::Print( GetProjectBrain()->GetDebugOutputPos().x, GetProjectBrain()->GetDebugOutputPos().y, 0.5f, 0xffffffff,
		ASURA_TEXTFORMAT_CENTREJUSTIFIED | ASURA_TEXTFORMAT_FORMATSTRING, "Ambush: %s", s_aszStateStrings[ GetCurrentState() ] );
	GetProjectBrain()->GetDebugOutputPos().y += 10.0f;


}
#endif

AVP_Brain_Actor_Alien* AVP_Behaviour_Alien_Ambush::GetAlienBrain() const
{
	return static_cast< AVP_Brain_Actor_Alien* >( GetBrain() );
}

void AVP_Behaviour_Alien_Ambush::SwitchToState( const State eState )
{

	int eLastState = GetCurrentState();
	switch ( eLastState )
	{
	case UNSET:
		break;
	case AMBUSH:
		break;
	case PREATTACK:
		break;
	case HISS:
		GetAlienBrain()->GetAlienSquad()->ReleaseAmbushHiss();
		break;
	case ATTACK:
		DestroyChildWithType(AVP_BEHAVIOUR_TYPE_ALIEN_ATTACK2);
		break;
	case FINISHED:
		break;
	default:
		Asura_Assert(false, "Unexpected state");
		break;
	}

	SetCurrentState( eState );
	ResetStateTimer();
	switch ( GetCurrentState() )
	{
	case UNSET:
		break;
	case AMBUSH:
		break;
	case PREATTACK:
		break;
	case HISS:
		GetAlienBrain()->GetAlienSquad()->LockAmbushHiss();
		GetAlienBrain()->SetMelee(true);
		RequestHiss();
		break;
	case ATTACK:
		CreateChildWithType(AVP_BEHAVIOUR_TYPE_ALIEN_ATTACK2);
		break;
	case FINISHED:
		SetFinished(true);
		break;
	default:
		Asura_Assert(false, "Unexpected state");
		break;
	}
}

void AVP_Behaviour_Alien_Ambush::UpdateAmbush()
{
	if (TookDamage())
	{
		SwitchToState(ATTACK);
		return;
	}

	const AVP_Knowledge_System_AlienTeam* pxKnowledgeSystem = GetAlienBrain()->GetAlienTeam()->GetKnowledgeSystem();
	Asura_Assert(pxKnowledgeSystem, "Alien team knowledge not found");
	if (!pxKnowledgeSystem)
	{
		SetInError(true);
		SwitchToState(FINISHED);
		return;
	}

	Asura_Vector_3 xActorHeadPosition;
	GetAlienBrain()->GetActor()->GetHeadPosition(xActorHeadPosition);

	for ( Asura_Hashed_List_ConstIt< AVP_AlienTeam_Knowledge > xKnowIter( &pxKnowledgeSystem->GetAllKnowledge() );
		!xKnowIter.Done();
		xKnowIter.Next())
	{
		const AVP_AlienTeam_Knowledge* pxKnowledge = xKnowIter.GetCurrent();
		if (!pxKnowledge)
		{
			continue;			
		}

		const AVP_ServerEntity_Actor* pxTarget = AVP_ServerEntity_Actor::FindEntity( pxKnowledge->GetGuid() );
		if (!pxTarget || pxTarget->IsDead() || pxTarget->IsDestroyed())
		{
			continue;
		}

		switch (pxTarget->GetClassification()) // targets that are enemies but not attackable by alien AI
		{
		case AVP_ENTITYCLASS_SENTRYGUN: 
			continue;
		default:
			break;
		}

		int iTargetPriority;
		if ( !GetAlienBrain()->CalcTargetingPriority( pxKnowledge->m_iThreatClassification, pxKnowledge->GetGuid(), iTargetPriority ) )
		{
			iTargetPriority = AVP_AI_TargetPriorityData::NORMAL;
		}
		if (iTargetPriority == AVP_AI_TargetPriorityData::IGNORE_P)
		{
			continue;
		}

		Asura_Vector_3 xTargetHead;
		pxTarget->GetHeadPosition(xTargetHead);


		Asura_Vector_3 xTargetDirection = (xTargetHead - xActorHeadPosition);
		const float fDistance = xTargetDirection.Normalise();
		if (fDistance > dbg_fLOSTestDistance)
		{
			continue;
		}

		Asura_LOS_InputData xLosInput(xActorHeadPosition, xTargetDirection, GetAlienBrain()->GetActor()->GetGuid(), fDistance);
		const Asura_LOS_OutputData* pxHeadLosOutput = Asura_LOS::TestAgainstEverything(&xLosInput);

		if (pxHeadLosOutput->m_uHitEntity == pxKnowledge->GetGuid())
		{
			m_uSeenTarget = pxKnowledge->GetGuid();
			if (GetAlienBrain()->GetAlienSquad()->CanLockAmbushHiss())
			{
				SwitchToState(HISS);
			}
			else
			{
				SwitchToState(PREATTACK);
			}
		}		
	}
}

void AVP_Behaviour_Alien_Ambush::UpdatePreAttack()
{
	if (TookDamage() || GetStateTimer() > dbg_fPreAttackTime)
	{
		SwitchToState(ATTACK);
		return;
	}

	const AVP_Knowledge_System_AlienTeam* pxKnowledgeSystem = GetAlienBrain()->GetAlienTeam()->GetKnowledgeSystem();
	Asura_Assert(pxKnowledgeSystem, "Alien team knowledge not found");
	if (!pxKnowledgeSystem)
	{
		SetInError(true);
		SwitchToState(FINISHED);
		return;
	}

	Asura_Vector_3 xActorHeadPosition;
	GetAlienBrain()->GetActor()->GetHeadPosition(xActorHeadPosition);
	const AVP_AlienTeam_Knowledge* pxKnowledge = pxKnowledgeSystem->GetKnowledge(m_uSeenTarget);
	if (!pxKnowledge)
	{
		SwitchToState(AMBUSH);
		return;
	}
	
	const AVP_ServerEntity_Actor* pxTarget = AVP_ServerEntity_Actor::FindEntity( pxKnowledge->GetGuid() );
	if (!pxTarget || pxTarget->IsDead() || pxTarget->IsDestroyed())
	{
		SwitchToState(AMBUSH);
		return;
	}

	Asura_Vector_3 xTargetHead;
	pxTarget->GetHeadPosition(xTargetHead);


	Asura_Vector_3 xTargetDirection = (xTargetHead - xActorHeadPosition);
	const float fDistance = xTargetDirection.Normalise();

	Asura_LOS_InputData xLosInput(xActorHeadPosition, xTargetDirection, GetAlienBrain()->GetActor()->GetGuid(), fDistance);
	const Asura_LOS_OutputData* pxHeadLosOutput = Asura_LOS::TestAgainstEverything(&xLosInput);

	if (pxHeadLosOutput->m_uHitEntity != pxKnowledge->GetGuid())
	{
		SwitchToState(AMBUSH);
	}
}

void AVP_Behaviour_Alien_Ambush::UpdateHiss()
{
}


void AVP_Behaviour_Alien_Ambush::UpdateAttack()
{
	AVP_Behaviour_Alien_Attack2* pxBehaviour = static_cast< AVP_Behaviour_Alien_Attack2* >( GetChildWithType( this, AVP_BEHAVIOUR_TYPE_ALIEN_ATTACK2 ) );
	if ( !pxBehaviour || pxBehaviour->IsInError() || pxBehaviour->IsFinished() )
	{
		SwitchToState( ATTACK );
	}
}

void AVP_Behaviour_Alien_Ambush::RequestHiss()
{
	Asura_Message* pxMessage = Asura_Network::CreateMessage( ASURA_MSG_MOVE_START, sizeof( AVP_Move_Alien_Hiss ) );
	if ( pxMessage != NULL )
	{
		Asura_Msg_Move* pxData = static_cast< Asura_Msg_Move* >( pxMessage->GetData() );

		pxMessage->SetFromGuid( GetAlienBrain()->GetOwnerGuid() );
		pxMessage->SetToGuid( GetAlienBrain()->GetOwnerGuid() );

		pxData->ClearFlags();		
		pxData->SetIgnoredByOwner( false );
		pxData->SetMoveID( AVP_Move_Alien_Hiss::GetStaticHash() );

		Asura_Network::Send(pxMessage);
	}
	else
	{
		SwitchToState(PREATTACK);
	}

}

void AVP_Behaviour_Alien_Ambush::LateHandleMessage()
{
	Asura_LinkList_It< int > xIter(m_xDelayedMsgList);
	for (; !xIter.Done(); xIter.Next())
	{
		int eMsgId = *xIter.GetCurrent();
		switch (eMsgId)
		{
		case ASURA_MSG_MOVECHAINCOMPLETE:
			SwitchToState( ATTACK );
			break;
		default:
			Asura_Assert(false, "Unexpected message");
		}
	}
	m_xDelayedMsgList.Clear();
}


bool AVP_Behaviour_Alien_Ambush::TookDamage()
{
	bool bResult = GetAlienBrain()->GetActor()->GetHealth() < m_fLastHealth;
	m_fLastHealth = GetAlienBrain()->GetActor()->GetHealth();
	return bResult;
}


//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////
