//////////////////////////////////////////////////////////////////////////////
//
// AVP_Behaviour_Alien_Attack2.h
//
// Copyright (C) 2008 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __AVP_BEHAVIOUR_ALIEN_ATTACK2_H__
#define __AVP_BEHAVIOUR_ALIEN_ATTACK2_H__ 1

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// asura
#include "Core/Asura_LinkList.h"
#include "Network/Asura_Guid.h"

// axon
#include "Behaviour/Axon_Project_Behaviour.h"

// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////////

class Asura_Message;
class AVP_Brain_Actor_Alien;
struct AVP_AlienTeam_Knowledge;

//////////////////////////////////////////////////////////////////////////////
//
// AVP_Behaviour_Alien_Attack2
//
//////////////////////////////////////////////////////////////////////////////

class AVP_Behaviour_Alien_Attack2 : public Axon_Project_Behaviour
{
	typedef Axon_Project_Behaviour PARENT;

public:
	AVP_Behaviour_Alien_Attack2( AVP_Brain_Actor_Alien* pxBrain );
	void StopAfterAttack();
	bool IsAttacking() const;

protected:
	virtual bool Activate();
	virtual void Deactivate();
	virtual void PerFrameUpdate( const float fTimeslice );
	virtual bool HandleMessage( const Asura_Message* pxMessage );

#ifndef ASURA_NO_DEBUG_OUTPUT
	virtual void RenderDebugInfo();
#endif

private:
	enum State
	{
		UNSET,
		RUN_IN,
		CHARGING,
		JUMP_ATTACK,
		FAKE_JUMP_ATTACK,
		STAND_ATTACK,
		POUNCE_GRAPPLE,
		COUNTERED,
		RUN_OUT,
		WANDER,
		FINISHED
	};

	enum StandMeleeMode
	{
		STAND_MELEE_TO_STAND,
		STAND_MELEE_STAND,
		STAND_MELEE_WALK
	};

	enum AttackType
	{
		ATTACK_TYPE_STAND,
		ATTACK_TYPE_JUMP,
		ATTACK_TYPE_POUNCE_GRAPPLE,
		ATTACK_TYPE_UNDECIDED
	};


	AVP_Brain_Actor_Alien* GetAlienBrain() const;
	void SwitchToState( const State eState );

	const AVP_AlienTeam_Knowledge* ChooseNewTarget(bool bForceChooseNow, bool bTryToChargeImmediately, bool &bChargeImmediatly) const;
	void UpdateRunIn(float fTimeslice);
	void UpdateCharge();
	void UpdateStandAttack();
	void UpdateRunOut();
	void UpdateWander();
	void PlanPathIn(bool bIsReplan);
	void PlanCharge();
	void PlanPathOut(bool bIsReplan, bool bUseMyOrientation);
	void PlanWander();

	bool EndOfRouteAllowsStandAttack() const;
	bool OnFloorRouteTillEnd() const;
	bool StandAttackWillBeAllowed() const;
	bool StandAttackAllowed() const; 
	bool JumpAttackAllowed(bool bFake) const; 
	bool PounceGrappleAllowed() const;
	void RequestJumpAttack(bool bFake, bool bReducedDmg);
	void RequestMeleeAttack();
	void RequestPounceGrapple();
	void LateHandleMessage();	

	void ChoosePreferedAttack(bool bTakeLastGrappleTimeIntoAccount);

	bool CanLockAttack(AttackType eAttackType) const;
	void LockAttack(AttackType eAttackType);
	void UnlockAttack();

	void UpdateSprint(float fTimeslice, bool bCanStartSprint);

	bool IsReducedJumpAttackDamage() const;
	bool TargetIsPlayerAndDidntSeeMe() const;

private:

	Asura_Vector_3 m_fTargetPositionAtPlan;
	Asura_Vector_3 m_fTargetHeadingAtPlan;
	float m_fMinDistaneToReplan;
	bool m_bAttackLockedByUs;
	AttackType m_eCurrentAttackType;
	AttackType m_ePreferedAttackType;
	bool m_bLeftStandAttack;
	bool m_bLastAttackHit;
	StandMeleeMode m_eStandMeleeMode;
	u_int m_uNumberOfStandAttacksLeft;
	Asura_LinkList< int > m_xDelayedMsgList;
	bool m_bStopAttack;
	mutable float m_fTimeSinceLastChoosingTarget;
	bool m_bSprintOn;
	float m_fSprintTimer;
	float m_fSprintTestTimer;
	bool m_bTargetIsPlayerAndDidntSeenMe;
};

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

#endif //__AVP_BEHAVIOUR_ALIEN_ATTACK2_H__
