//////////////////////////////////////////////////////////////////////////////
//
// AVP_Behaviour_Alien_GrapplePuppet.cpp
//
// Copyright (C) 2008 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"

// header
#include "Behaviour/BehaviourTypes/Alien/AVP_Behaviour_Alien_GrapplePuppet.h"

// asura
#ifndef ASURA_NO_DEBUG_OUTPUT
#include "Rendering/Asura_Font_System.h"
#endif

// axon
#include "Behaviour/BehaviourTypes/AVP_Behaviour_Types.h"
#include "Behaviour/BehaviourTypes/Alien/AVP_Behaviour_Alien_Movement.h"
#include "Core/BrainTypes/AVP_Brain_Actor_Alien.h"

// avp
#include "Actor/Vertebrates/Alien/AVP_Alien_MeleeCounter_System.h"
#include "Animation/AVP_AnimFlags.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/AVP_Vertebrate_Messages.h"
#include "Entity/AVP_Messages.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Alien/AVP_ClientActor_Alien.h"
#include "Moves/AVP_Movement_System.h"
#include "Moves/AVP_Moves.h"

//////////////////////////////////////////////////////////////////////////////
//
// AVP_Behaviour_Alien_GrapplePuppet
//
//////////////////////////////////////////////////////////////////////////////

static const float fMIN_RESPONSE_TIME = 0.125f;
static const float fMAX_RESPONSE_TIME = 0.5f;

AVP_Behaviour_Alien_GrapplePuppet::AVP_Behaviour_Alien_GrapplePuppet( AVP_Brain_Actor_Alien* pxBrain )
: PARENT( pxBrain, AVP_BEHAVIOUR_TYPE_ALIEN_GRAPPLEPUPPET )
, m_bResponding( false )
, m_fResponseTimer( 0.0f )
, m_uGrapplerGuid( ASURA_GUID_UNREGISTERED )
{
}

bool AVP_Behaviour_Alien_GrapplePuppet::Activate()
{
	const bool bParentOk = PARENT::Activate();
	GetAlienBrain()->GetMovement().Stop();
	return bParentOk;
}

void AVP_Behaviour_Alien_GrapplePuppet::PerFrameUpdate( const float fTimeslice )
{
	PARENT::PerFrameUpdate( fTimeslice );

	if ( IsFinished() || IsInError() )
	{
		return;
	}

	UpdateStateTimer( fTimeslice );
	if ( GetStateTimer() > 10.0f )
	{
		// Hardcoded timeout in case, for whatever reason, our anims break.
		SetInError( true );
		SetFinished( true );
	}

	// Look for the counter anim flag, then implement our response time and try and counter.
	if ( !m_bResponding )
	{
		if ( GetAlienBrain()->GetClientAlienActor()->GetAnimationHandler()->GetAnimation()->GetFlagsForAllAnimsLastFrame() & AVP_ANIMFLAG_ALLOW_NEW_COUNTER )
		{
			m_bResponding = true;
			m_fResponseTimer = Asura_Random::GetFloat( fMIN_RESPONSE_TIME, fMAX_RESPONSE_TIME );
		}
	}
	else if ( m_fResponseTimer > 0.0f )
	{
		m_fResponseTimer -= fTimeslice;
		if ( m_fResponseTimer < 0.0f )
		{
			// Request a melee. This will be interpreted as us trying to counter the grapple we're in.
			Asura_Message* pxMessage = Asura_Network::CreateMessage( AVP_MSGID_REQUEST_MELEE, sizeof( AVP_Message_Vertebrate_RequestMelee ) );
			if ( pxMessage )
			{
				AVP_Message_Vertebrate_RequestMelee* pxData = static_cast< AVP_Message_Vertebrate_RequestMelee* >( pxMessage->GetData() );

				pxData->SetType( AVP_MELEEATTACK_TYPE_NORMAL );
				pxData->SetTargetGuid( m_uGrapplerGuid );

				pxMessage->SetToGuid( GetAlienBrain()->GetOwnerGuid() );
				pxMessage->SetFromGuid( GetAlienBrain()->GetOwnerGuid() );
				Asura_Network::Send( pxMessage );
			}
		}
	}
}

bool AVP_Behaviour_Alien_GrapplePuppet::HandleMessage( const Asura_Message* pxMessage )
{
	if ( pxMessage->GetMsgID() == ASURA_MSG_MOVECHAINCOMPLETE )
	{
		// If our chain is complete then so are we.
		SetFinished( true );
		return true;
	}
	else if ( pxMessage->GetMsgID() == ASURA_MSG_MOVE_ABORT	|| pxMessage->GetMsgID() == ASURA_MSG_MOVE_FINISHED )
	{
		const AVP_Msg_Move* pxMoveMsg = static_cast< const AVP_Msg_Move* >( pxMessage->GetData() );
		if ( pxMoveMsg->GetMoveID() == AVP_Move_Puppet::GetStaticHash() )
		{
			SetFinished( true );
			return true;
		}
		else
		{
			return false;
		}
	}
	else
	{
		return PARENT::HandleMessage( pxMessage );
	}
}

#ifndef ASURA_NO_DEBUG_OUTPUT
void AVP_Behaviour_Alien_GrapplePuppet::RenderDebugInfo()
{
	PARENT::RenderDebugInfo();

	Asura_Font_System::Print( GetProjectBrain()->GetDebugOutputPos().x, GetProjectBrain()->GetDebugOutputPos().y, 0.5f, 0xffffffff,
		ASURA_TEXTFORMAT_CENTREJUSTIFIED, "BEHAVIOUR_ALIEN_GRAPPLEPUPPET" );
	GetProjectBrain()->GetDebugOutputPos().y += 10.0f;
}
#endif

AVP_Brain_Actor_Alien* AVP_Behaviour_Alien_GrapplePuppet::GetAlienBrain()
{
	return static_cast< AVP_Brain_Actor_Alien* >( GetBrain() );
}

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////
