//////////////////////////////////////////////////////////////////////////////
//
// AVP_Behaviour_Alien_HideInShadows.cpp
//
// Copyright (C) 2008 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"

// header
#include "Behaviour/BehaviourTypes/Alien/AVP_Behaviour_Alien_HideInShadows.h"

// asura
#ifndef ASURA_NO_DEBUG_OUTPUT
#include "Debugging/Asura_DebugLineRender.h"
#include "Rendering/Asura_Font_System.h"
#endif
// avp
#include "Actor/Vertebrates/Alien/AVP_Alien_MeleeCounter_System.h"
#include "Blueprint/AVP_Blueprint_HashIDs.h"
#include "Core/AVP_Plane.h"
#include "Entity/AVP_Messages.h"
#include "Entity/Controller/AVP_ServerEntity_Team_Alien.h"
#include "Entity/Damage/AVP_EntityDamage_Request.h"
#include "Entity/GamePlayVolume/AVP_ServerEntity_GamePlayVolume.h"
#include "Entity/Locations/VantagePoint/AVP_ServerNode_VantagePoint_Predator.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/AVP_ServerActor_Vertebrate.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/AVP_Vertebrate_Messages.h"
#include "Moves/Vertebrates/Alien/AVP_Alien_MovesJump.h"
#include "Network/AVP_Player.h"


// axon
#include "Behaviour/BehaviourTypes/Alien/AVP_Behaviour_Alien_Attack2.h"
#include "Behaviour/BehaviourTypes/Alien/AVP_Behaviour_Alien_Wander.h"
#include "Behaviour/BehaviourTypes/AVP_Behaviour_Types.h"
#include "Core/AVP_Game_Core.h"
#include "Core/AVP_Navigation_Types.h"
#include "Core/AVP_Waypoint_Manager.h"
#include "Core/BrainTypes/AVP_Brain_Actor_Alien.h"
#include "Knowledge/AVP_Knowledge_System_AlienTeam.h"


//////////////////////////////////////////////////////////////////////////////
//
// AVP_AlienShadowsAroundTargetFunctor - used to find a point away from a target in shadows
//
//////////////////////////////////////////////////////////////////////////////

class AVP_AlienShadowsAroundTargetFunctor : public AVP_Navigation::BreadthFirstSearch
{
public:
	struct WPInfo
	{
		u_short m_usWPID;
		float m_fDistanceFromCenterSq;
		float m_fDistanceFromStartSq;
		float m_fScore;
	};


	virtual bool LinkFlagsTest( const Asura_AI_NavConnection* pxLink, const u_int uPathfinderFlags );
	virtual bool VisitNode( const int iWaypointIndex );

	void AddGPV(const AVP_ServerEntity_GamePlayVolume* pxGPV);
	void Filter(float fRangeBand);
	u_short GetWPID() const;

	AVP_AlienShadowsAroundTargetFunctor( const Asura_Vector_3& xCenter, const Asura_Vector_3& xStart, u_short usWPID, const float fMaxSearchRange, float fMinRangeFromCenter, float fMinRangeFromStart ) 
		: AVP_Navigation::BreadthFirstSearch(usWPID, fMaxSearchRange)
		, m_usNumGPV(0)
		, m_usNumWPInfo(0)
		, m_fMinRangeFromCenterSq(fMinRangeFromCenter * fMinRangeFromCenter)
		, m_fMinRangeFromStartSq(fMinRangeFromStart * fMinRangeFromStart)
		, m_xCenter(xCenter)
		, m_xStart(xStart)
		, m_fMinRangeFromCenterSqFound(Asura_Maths::LargeFloat)
		, m_fTotalScore(0.0f)
	{}

	enum { MAX_GPV = 8 };

protected:
	
	u_short m_usNumGPV;
	const AVP_ServerEntity_GamePlayVolume* m_apxGPV[MAX_GPV];

	enum { MAX_WPID = 64 };
	u_short m_usNumWPInfo;
	WPInfo m_axWPInfo[MAX_WPID];
	const float m_fMinRangeFromCenterSq;
	const float m_fMinRangeFromStartSq;
	const Asura_Vector_3 m_xCenter;
	const Asura_Vector_3 m_xStart;
	float m_fMinRangeFromCenterSqFound;
	float m_fTotalScore;

	typedef AVP_Navigation::BreadthFirstSearch PARENT;
};

bool AVP_AlienShadowsAroundTargetFunctor::LinkFlagsTest( const Asura_AI_NavConnection* pxLink, const u_int uPathfinderFlags )
{
	return AVP_Navigation::AlienLinkFlagsTest( pxLink, uPathfinderFlags );
}

bool AVP_AlienShadowsAroundTargetFunctor::VisitNode( const int iWaypointIndex )
{
	const Asura_AI_NavVertex* pxWP = Asura_AI_Navigation_System::GetWaypoint( iWaypointIndex );
	Asura_Assert(pxWP , "Way point not found during search");
	if (!pxWP)
	{
		return false;
	}

	if (pxWP->GetFlags() & AVP_Navigation::AVP_AI_NAVVERTEX_FLAG_GS2) // ignore GS2 nodes
	{
		return false;
	}

	float fDistanceFromCenterSq = (pxWP->GetPosition()-m_xCenter).MagnitudeSquared();

	if ( fDistanceFromCenterSq < m_fMinRangeFromCenterSq )
	{
		return false;
	}

	float fDistanceFromStartSq = (pxWP->GetPosition()-m_xStart).MagnitudeSquared();

	const float fLuminaceFactor = AVP_Waypoint_Manager::GetWPLuminance( iWaypointIndex );
	if (fLuminaceFactor > AVP_Waypoint_Manager::GetDefaultMinHideLuminance())
	{
		return false;
	}

	if (m_usNumGPV != 0)
	{
		bool bFound = false;
		for (u_short u = 0; u < m_usNumGPV && !bFound; ++u)
		{
			if (m_apxGPV[u]->Contains(pxWP->GetPosition()))
			{
				bFound = true;
			}
		}

		if (!bFound)
		{
			return false;
		}
	}

	if (m_usNumWPInfo < MAX_WPID)
	{
		m_axWPInfo[m_usNumWPInfo].m_usWPID = static_cast<u_short>(iWaypointIndex);
		m_axWPInfo[m_usNumWPInfo].m_fDistanceFromCenterSq = fDistanceFromCenterSq;
		m_axWPInfo[m_usNumWPInfo].m_fDistanceFromStartSq = fDistanceFromStartSq;
		++m_usNumWPInfo;
		if (m_fMinRangeFromCenterSqFound > fDistanceFromCenterSq)
		{
			m_fMinRangeFromCenterSqFound = fDistanceFromCenterSq;
		}		
	}

	return false;
}

void AVP_AlienShadowsAroundTargetFunctor::AddGPV(const AVP_ServerEntity_GamePlayVolume* pxGPV)
{
	if (m_usNumGPV < MAX_GPV)
	{
		m_apxGPV[m_usNumGPV] = pxGPV;
		++m_usNumGPV;
	}
}

DEBUGVAR float dbg_fGoodCloseToCenterPointScore = 5.0f;
DEBUGVAR float dbg_fGoodCloseToStartPointScore = 2.0f;
void AVP_AlienShadowsAroundTargetFunctor::Filter(float fRangeBand)
{
	float fMaxDistanceFromCenterSq = Asura_Maths::Sqrt(m_fMinRangeFromCenterSqFound) + fRangeBand;
	fMaxDistanceFromCenterSq *= fMaxDistanceFromCenterSq;

	m_fTotalScore = 0.0f;

	for (u_short u = 0; u < m_usNumWPInfo; ++u)
	{
		m_axWPInfo[u].m_fScore = 0.0f;
		if (m_axWPInfo[u].m_fDistanceFromCenterSq  < fMaxDistanceFromCenterSq)
		{
			m_axWPInfo[u].m_fScore += dbg_fGoodCloseToCenterPointScore * ( fMaxDistanceFromCenterSq - m_axWPInfo[u].m_fDistanceFromCenterSq) / (fMaxDistanceFromCenterSq - m_fMinRangeFromCenterSq) ;
		}
		else
		{
			m_axWPInfo[u].m_fScore += 0.0f;
		}

		if (m_axWPInfo[u].m_fDistanceFromStartSq  < m_fMinRangeFromStartSq)
		{
			m_axWPInfo[u].m_fScore += 0.0f;
		}
		else
		{
			m_axWPInfo[u].m_fScore += dbg_fGoodCloseToStartPointScore;
		}

		m_fTotalScore += m_axWPInfo[u].m_fScore;

	}
}

u_short AVP_AlienShadowsAroundTargetFunctor::GetWPID() const
{
	if (m_fTotalScore == 0.0f)
	{
		return ASURA_INVALID_WAYPOINT_ID;
	}

	float fDestScore = Asura_Random::GetFloat(0.0f, m_fTotalScore - 1e-6f);


	for (u_short u = 0; u < m_usNumWPInfo; ++u)
	{
		if (m_axWPInfo[u].m_fScore == 0.0f)
		{
			continue;
		}

		if (fDestScore < m_axWPInfo[u].m_fScore)
		{
			return m_axWPInfo[u].m_usWPID;
		}

		fDestScore -= m_axWPInfo[u].m_fScore;
	}

	return ASURA_INVALID_WAYPOINT_ID;

}

//////////////////////////////////////////////////////////////////////////////
//
// AVP_AleinMoveAvoidingTargetInShadowsFunctor
//
//////////////////////////////////////////////////////////////////////////////
DEBUGVAR float dbg_fCloseMultiplier = 100.0f;
DEBUGVAR float dbg_fTransitionMultiplier = 0.5f;
DEBUGVAR float dbg_fMismatchedMobilityAdditionalCostFactor  = 1.0;

class AVP_AlienMoveAvoidingTargetInShadowsFunctor : public AVP_Navigation::AlienAStarFunctor 
{
public:

	virtual float EstimateTraversalCost( const int iWaypointIndex, const int iLinkIndex, const u_int uPathfinderFlags );
	virtual float EstimateCostToGoal( const int iWaypointIndex );

	AVP_AlienMoveAvoidingTargetInShadowsFunctor(const Asura_Vector_3& xCenter, float fMinDistance) 
		: AVP_Navigation::AlienAStarFunctor() 
		, m_xCenter(xCenter)
		, m_fMinDistanceSq(fMinDistance * fMinDistance) 
	{}
private:
	Asura_Vector_3 m_xCenter;
	float m_fMinDistanceSq;

	typedef AVP_Navigation::AlienAStarFunctor PARENT;
};

float AVP_AlienMoveAvoidingTargetInShadowsFunctor::EstimateTraversalCost( const int iWaypointIndex, const int iLinkIndex, const u_int uPathfinderFlags )
{
	const float fBaseCost = Asura_AI_Navigation_System::DefaultTraversalCostFunction( iWaypointIndex, iLinkIndex, uPathfinderFlags, NULL );

	const Asura_AI_NavVertex* pxFromWP = Asura_AI_Navigation_System::GetWaypoint( iWaypointIndex );
	const Asura_AI_NavVertex* pxToWP   = Asura_AI_Navigation_System::GetWaypoint( pxFromWP->m_pxConnections[ iLinkIndex ].m_usIndex );

	// care a bit but not really.
	bool bForceLowMobility = m_eMovementLimitation == LIMIT_LOW_MOBILITY && (pxToWP->GetPosition() - m_xStartPosition).MagnitudeSquared() < m_fDistanceSq;
	float fMismatchedAgilityCost = fBaseCost * AVP_Navigation::AlienCalculateCostMultiplierDueToMobility(pxToWP, uPathfinderFlags, bForceLowMobility) * dbg_fMismatchedMobilityAdditionalCostFactor;

	// Add a cost for transition links to stop unnecessary hopping on and off walls, etc.
	float fTransitionCost = 0.0f;
	const u_short usLinkFlags = Asura_AI_Navigation_System::GetWaypoint( iWaypointIndex )->m_pxConnections[ iLinkIndex ].m_usFlags;
	if ( AVP_Navigation::IsTransitionLink( usLinkFlags ) )
	{
		fTransitionCost = fBaseCost * dbg_fTransitionMultiplier;

		if ( m_eMovementLimitation == LIMIT_AVOID_JUMPING && (pxToWP->GetPosition() - m_xStartPosition).MagnitudeSquared() < m_fDistanceSq )
		{
			fTransitionCost *= 8.0f;
		}
	}

	float fCloseToTargetCost = 0.0f;

	Asura_Vector_3 xNearestPoint;
	Asura_Maths::GetNearestPointOnLineSegment(pxFromWP->GetPosition(), pxToWP->GetPosition(), m_xCenter, xNearestPoint);
	float fDistanceSq = (m_xCenter - xNearestPoint).MagnitudeSquared();

	if (  fDistanceSq < m_fMinDistanceSq ) 
	{
		fCloseToTargetCost = fBaseCost * dbg_fCloseMultiplier * (m_fMinDistanceSq - fDistanceSq)/m_fMinDistanceSq;
	}

	// hate lite
	float fPrevLuminace = AVP_Waypoint_Manager::GetWPLuminance( iWaypointIndex );
	float fLuminace = AVP_Waypoint_Manager::GetWPLuminance( pxFromWP->m_pxConnections[ iLinkIndex ].m_usIndex );
	float fLuminaceFactor = 1.0f;

	if (fLuminace < AVP_Waypoint_Manager::GetDefaultMaxHideLuminance()) // moving into shadow
	{
		if (fPrevLuminace < AVP_Waypoint_Manager::GetDefaultMaxHideLuminance()) // was already in shadow
		{
			fLuminaceFactor = fPrevLuminace > fLuminace ? 0.1f : 0.5f;
		}
		else // moving away from the light
		{
			fLuminaceFactor = 0.0f;
		}

	}
	else // moving into the light
	{
		if (fPrevLuminace < AVP_Waypoint_Manager::GetDefaultMaxHideLuminance()) // was already in shadow
		{
			fLuminaceFactor = 10.0f;
		}
		else // was already in the light 
		{
			fLuminaceFactor = fPrevLuminace > fLuminace ? 5.0f : 8.0f;
		}

	}


	return ( fLuminaceFactor * (fBaseCost + fMismatchedAgilityCost + fTransitionCost ) + fCloseToTargetCost);

}

float AVP_AlienMoveAvoidingTargetInShadowsFunctor::EstimateCostToGoal( const int iWaypointIndex )
{
	float fLuminace = AVP_Waypoint_Manager::GetWPLuminance( iWaypointIndex );

	if (fLuminace > AVP_Waypoint_Manager::GetDefaultMaxHideLuminance())
	{
		return PARENT::EstimateCostToGoal(iWaypointIndex);
	}
	else
	{
		return 0.5f * PARENT::EstimateCostToGoal(iWaypointIndex);
	}

}

//////////////////////////////////////////////////////////////////////////////
//
// AVP_Behaviour_Alien_HideInShadows
//
//////////////////////////////////////////////////////////////////////////////

DEBUGVAR float dbg_fRangeBand = 8.0f;
DEBUGVAR float dbg_fMaxRangeSq = 30.0f * 30.0f;
DEBUGVAR float dbg_fMinTimeTotallyInvisible = 0.5f;
DEBUGVAR float dbg_fMinTimeTotallyInvisibleVariance = 1.0f;
DEBUGVAR float dbg_fForgetMarineLightTime = 0.5f;

DEBUGVAR float dbg_fLargeArriveRange = 1.0f;

DEBUGVAR float dbg_fMinTimeToStayIdleInShadows = 5.0f;
DEBUGVAR float dbg_fMaxTimeToStayIdleInShadows = 10.0f;
DEBUGVAR float dbg_fTimeTillStopAllowedAfterAttack = 2.0f;
DEBUGVAR float dbg_fMaxTimeTillStopAllowed = 0.0f; // for now don't force movement after in shadow since the env is very bright and finding dark areas is hard.
DEBUGVAR float dbg_fMinMoveTimeDuringRehide = 2.0f;
DEBUGVAR float dbg_fMinMoveDistance = 10.0f;
DEBUGVAR float dbg_fTimeWhenDamageCauseSprint = 7.0f;
DEBUGVAR float dbg_fMaxDistanceFromVP = 3.0f;

DEBUGVAR float dbg_fSrpintDuration = 1.5f;
AVP_Behaviour_Alien_HideInShadows::AVP_Behaviour_Alien_HideInShadows( AVP_Brain_Actor_Alien* pxBrain )
: PARENT( pxBrain, AVP_BEHAVIOUR_TYPE_ALIEN_HIDE_IN_SHADOWS )
, m_uLightGeneration(0)
, m_fTimeTotallyVisible(0.0f)
, m_fTimeTotallyInvisible(0.0f)
, m_fTimeVisibleByMarineLight(0.0f)
, m_fTimeSinceVisibleByMarineLight(0.0f)
, m_fMinTimeTotallyInvisible(dbg_fMinTimeTotallyInvisible + Asura_Random::GetFloat(0.0f, dbg_fMinTimeTotallyInvisibleVariance))
, m_fMaxDistanceSq(0.0f)
, m_fLastHealth(pxBrain->GetActor()->GetHealth())
, m_bNeedToChooseToRehideAfterAttack(false)
, m_fTimeTillStopAllowed(0.0f)
, m_fTimeToStayHiddenInShadows(0.0f)
, m_xTargetPositionDuringPlan(Asura_Maths::ZeroVector3)
, m_uCurrentVP(ASURA_GUID_UNREGISTERED)
, m_bDestinationIsVP(false)
, m_xDestination(Asura_Maths::ZeroVector3)
, m_usDestinationWP(ASURA_INVALID_WAYPOINT_ID)
, m_xDelayedMsgList()
, m_uPlayerGuid(ASURA_GUID_UNREGISTERED)
, m_fSprintTimer(0.0f)
{
	SetCurrentState( UNSET );
}

bool AVP_Behaviour_Alien_HideInShadows::Activate()
{
	GetAlienBrain()->GetMovement().SetShouldJumpFromDanger( false );
	return PARENT::Activate();
}

void AVP_Behaviour_Alien_HideInShadows::Deactivate()
{
	SwitchToState( UNSET );
	PARENT::Deactivate();
}


void AVP_Behaviour_Alien_HideInShadows::PerFrameUpdate( const float fTimeslice )
{
	PARENT::PerFrameUpdate( fTimeslice );

	UpdateStateTimer(fTimeslice);

	UpdateHiddenStatus(fTimeslice);
	m_fTimeTillStopAllowed = Asura_Maths::Min(m_fTimeTillStopAllowed, dbg_fMaxTimeTillStopAllowed);
	m_fSprintTimer = (m_fSprintTimer < 0.0f) ? 0.0f : m_fSprintTimer - fTimeslice;

	if (GetAlienBrain()->CanUpdateMelee())
	{
		LateHandleMessage(); 
	}

	if (GetAlienBrain()->CanUpdateMelee())
	{
		switch ( GetCurrentState() )
		{
		case UNSET:
			break;
		case MOVING:
			break;
		case HIDE_IN_SHADOWS:
			break;
		case ATTACK:		
			{
				AVP_Behaviour_Alien_Attack2* pxBehaviour = static_cast< AVP_Behaviour_Alien_Attack2* >( GetChildWithType( this, AVP_BEHAVIOUR_TYPE_ALIEN_ATTACK2 ) );
				if ( !pxBehaviour || pxBehaviour->IsInError() )
				{		
					SwitchToState(ATTACK);
				}
				else if (pxBehaviour->IsFinished())
				{
					SwitchToState(MOVING);
				}
				else if (m_bNeedToChooseToRehideAfterAttack && pxBehaviour->IsAttacking())
				{
					m_bNeedToChooseToRehideAfterAttack = false;
					if (NeedToRehideAfterATtack())
					{
						pxBehaviour->StopAfterAttack();
					}
				}
				else if (!m_bNeedToChooseToRehideAfterAttack && !pxBehaviour->IsAttacking())
				{
					m_bNeedToChooseToRehideAfterAttack = true;
				}
			}
			break;
		case HISS_AND_ATTACK:
		case HISS_AND_REHIDE:
			break;
		case FINISHED:
			break;
		case WANDER:
			break;
		default:
		case NUM_STATES:
			Asura_Assert(false, "Unexpected state");
		}
	}


	if (!GetAlienBrain()->CanPlanMove())
	{
		return;
	}

	switch ( GetCurrentState() )
	{
	case UNSET:
		{
			const AVP_Player* pxPlayer = AVP_Game_Core::GetPlayer();
			if (pxPlayer && pxPlayer->IsEntityActive() && !pxPlayer->IsDestroyed())
			{
				m_uPlayerGuid = pxPlayer->GetPuppetGuid();
				if (GetAlienBrain()->GetAlienTeam()->GetKnowledgeSystem()->GetKnowledge(m_uPlayerGuid))
				{
					SwitchToState( MOVING );
				}
				else
				{
					SwitchToState( WANDER );
				}
			}
			else
			{
				SwitchToState( WANDER );
			}
		}
		break;
	case MOVING:
		UpdateMoving(fTimeslice);
		break;
	case HIDE_IN_SHADOWS:
		UpdateHideInShadows(fTimeslice);
		break;
	case ATTACK:		
		{
			AVP_Behaviour_Alien_Attack2* pxBehaviour = static_cast< AVP_Behaviour_Alien_Attack2* >( GetChildWithType( this, AVP_BEHAVIOUR_TYPE_ALIEN_ATTACK2 ) );
			if ( !pxBehaviour || pxBehaviour->IsInError() || pxBehaviour->IsFinished() )
			{
				SwitchToState( MOVING );
			}		
		}
		break;
	case WANDER:
		UpdateWander();
		break;
	case FINISHED:
		break;
	default:
	case NUM_STATES:
		Asura_Assert(false, "Unexpected state");

	}
}

bool AVP_Behaviour_Alien_HideInShadows::HandleMessage( const Asura_Message* pxMessage )
{
	switch ( GetCurrentState() )
	{
	case HISS_AND_ATTACK:
	case HISS_AND_REHIDE:
		if ( pxMessage->GetMsgID() == ASURA_MSG_MOVECHAINCOMPLETE )
		{
			GetAlienBrain()->SetMelee(false);
			m_xDelayedMsgList.Push_Back(ASURA_MSG_MOVECHAINCOMPLETE);
			return true;
		}
		break;
	default:
		break;
	}

	return PARENT::HandleMessage( pxMessage );
}

#ifndef ASURA_NO_DEBUG_OUTPUT

static const char* s_aszStateStrings[] =
{
	"UNSET",
	"MOVING",
	"HIDE_IN_SHADOWS",
	"HISS_AND_ATTACK",
	"HISS_AND_REHIDE",
	"ATTACK",
	"WANDER",
	"FINISHED",
	"NUM_STATES"
};

void AVP_Behaviour_Alien_HideInShadows::RenderDebugInfo()
{
	PARENT::RenderDebugInfo();	
	float fTime = 0.0f;

	char cHiddenValue = ' ';

	if (m_fTimeTotallyInvisible > 0)
	{
		cHiddenValue = 'I';
		fTime = m_fTimeTotallyInvisible;
	}

	if (m_fTimeTotallyVisible > 0)
	{
		cHiddenValue = 'V';
		fTime = m_fTimeTotallyVisible;
	}


	Asura_Font_System::Print( GetProjectBrain()->GetDebugOutputPos().x, GetProjectBrain()->GetDebugOutputPos().y, 0.5f, 0xffffffff,
		ASURA_TEXTFORMAT_CENTREJUSTIFIED | ASURA_TEXTFORMAT_FORMATSTRING, "HideInShadows: %s  %c %.3f %.3f", s_aszStateStrings[ GetCurrentState() ], 
		cHiddenValue, fTime, m_fTimeVisibleByMarineLight );
	GetProjectBrain()->GetDebugOutputPos().y += 10.0f;

}
#endif

AVP_Brain_Actor_Alien* AVP_Behaviour_Alien_HideInShadows::GetAlienBrain() const
{
	return static_cast< AVP_Brain_Actor_Alien* >( GetBrain() );
}

void AVP_Behaviour_Alien_HideInShadows::SwitchToState( const State eState )
{
	switch ( GetCurrentState() )
	{
	case ATTACK:
		DestroyAllChildren();
		m_fTimeVisibleByMarineLight = 0.0f;
		m_fTimeTotallyVisible = 0.0f;
		m_fTimeTotallyInvisible = 0.0f;
		m_fTimeTillStopAllowed += dbg_fTimeTillStopAllowedAfterAttack;
		break;
	case WANDER:
		DestroyChildWithType(AVP_BEHAVIOUR_TYPE_ALIEN_WANDER);
		break;
	case MOVING:
	case HIDE_IN_SHADOWS:
	case HISS_AND_REHIDE:
	case HISS_AND_ATTACK:
	case NUM_STATES:
	case FINISHED:
	case UNSET:
		break;
	default:
		Asura_Assert(false, "Unexpected state");
		break;
	}

	SetCurrentState( eState );
	ResetStateTimer();
	switch ( GetCurrentState() )
	{
		case MOVING:
			PlanPathToShadow();
			break;
		case ATTACK:		
			m_bNeedToChooseToRehideAfterAttack = true;
			CreateChildWithType( AVP_BEHAVIOUR_TYPE_ALIEN_ATTACK2 );
			break;
		case HIDE_IN_SHADOWS:
			m_fTimeToStayHiddenInShadows = Asura_Random::GetFloat(dbg_fMinTimeToStayIdleInShadows, dbg_fMaxTimeToStayIdleInShadows);
			GetAlienBrain()->GetMovement().Stop();
			break;
		case HISS_AND_REHIDE:
		case HISS_AND_ATTACK:
			GetAlienBrain()->SetMelee(true);
			RequestHiss();
			break;
		case WANDER:
			CreateChildWithType(AVP_BEHAVIOUR_TYPE_ALIEN_WANDER);
			break;
		case FINISHED:
			SetFinished(true);
			break;
		case UNSET:
			GetAlienBrain()->StoppedTargetingCurrentTarget();
			break;
		case NUM_STATES:
		default:
			Asura_Assert(false, "Unexpected state");
			break;
	}
}

void AVP_Behaviour_Alien_HideInShadows::PlanPathToShadow(bool bUseVantagePoint)
{

	const AVP_AlienTeam_Knowledge* pxTargetKnowledge = GetAlienBrain()->GetAlienTeam()->GetKnowledgeSystem()->GetKnowledge(m_uPlayerGuid);
	if (!pxTargetKnowledge)
	{
		SetInError(true);
		SwitchToState(FINISHED);
		return;
	}

	m_xTargetPositionDuringPlan = pxTargetKnowledge->GetPosition();
	bool bVPUsed = false;

	if (bUseVantagePoint)
	{
		bVPUsed = SelectVPDestination();
	}

	if (!bVPUsed)
	{
		if (!SelectNonVPDestination())
		{		
			return;
		}
	}
	

	m_fMaxDistanceSq = (m_xDestination - m_xTargetPositionDuringPlan).Magnitude() + dbg_fRangeBand;
	m_fMaxDistanceSq *= m_fMaxDistanceSq;
	AVP_AlienMoveAvoidingTargetInShadowsFunctor xMoveInShadowsFunctor(m_xTargetPositionDuringPlan, GetAlienBrain()->GetBehaviourData().m_fMinHideDistance);

	if ( !GetAlienBrain()->GetMovement().GetToPosition( m_xDestination, m_usDestinationWP, dbg_fLargeArriveRange, xMoveInShadowsFunctor, AVP_Alien_Movement::EMM_MEDIUM, false ) )
	{
		if (bVPUsed)
		{
			PlanPathToShadow(false);
		}
		else
		{
			SwitchToState( WANDER );
		}
		return;
	}
	else
	{
		GetAlienBrain()->GetMovement().SetAvoidLight();
	}

	m_uLightGeneration = GetAlienBrain()->GetAlienTeam()->GetLightGeneration();
}

void AVP_Behaviour_Alien_HideInShadows::UpdateMoving(float fTimeslice)
{

	const AVP_AlienTeam_Knowledge* pxTargetKnowledge = GetAlienBrain()->GetAlienTeam()->GetKnowledgeSystem()->GetKnowledge(m_uPlayerGuid);
	if (!pxTargetKnowledge)
	{
		SwitchToState(WANDER);
		return;
	}

	if (TookDamage())
	{
		if (GetStateTimer() > dbg_fTimeWhenDamageCauseSprint)
		{
			SwitchToState(ATTACK);
			return;
		}
		else
		{
			m_fSprintTimer = dbg_fSrpintDuration;
		}
	}

	u_int uLightGeneration = GetAlienBrain()->GetAlienTeam()->GetLightGeneration();

	AVP_Alien_Movement& xMovement = GetAlienBrain()->GetMovement();

	float fDistanceToTargetSq = (pxTargetKnowledge->GetPosition() - GetAlienBrain()->GetActor()->GetPosition()).MagnitudeSquared();
	
	bool bInRange = fDistanceToTargetSq < m_fMaxDistanceSq && fDistanceToTargetSq > GetAlienBrain()->GetBehaviourData().m_fMinHideDistance * GetAlienBrain()->GetBehaviourData().m_fMinHideDistance;

	if ( m_fTimeTotallyInvisible >  0.0f)
	{
		if ( !m_bDestinationIsVP && bInRange && IsInAdajcentGPV(pxTargetKnowledge->m_uCurrentGPV) && ( m_fTimeTillStopAllowed <= 0.0f || xMovement.IsFinished()) && GetAlienBrain()->IsInCurrentWaypoint())
		{
			SwitchToState(HIDE_IN_SHADOWS);
			return;
		}
		else if (m_bDestinationIsVP && bInRange && IsInAdajcentGPV(pxTargetKnowledge->m_uCurrentGPV) && m_fTimeTillStopAllowed <= 0.0f && ( xMovement.IsFinished() || GetAlienBrain()->GetMovement().GetLengthLeftOnPath() < dbg_fMaxDistanceFromVP ) && GetAlienBrain()->IsInCurrentWaypoint()  )
		{
			SwitchToState(HIDE_IN_SHADOWS);
			return;
		}
		else if (m_fSprintTimer > 0.0f)
		{
			xMovement.SetMovementMode(AVP_Alien_Movement::EMM_FAST);
		}
		else
		{
			xMovement.SetMovementMode(AVP_Alien_Movement::EMM_MEDIUM);
		}
	}
	else if (m_fTimeTotallyVisible >  0.0f )
	{
		if ( bInRange && GetAlienBrain()->GetBehaviourData().m_bAttackIfInMarineLight && m_fTimeVisibleByMarineLight >  GetAlienBrain()->GetBehaviourData().m_fTimeInLightBeforeAttack)
		{
			SwitchToState(ATTACK);
			return;
		}
		else
		{
			GetAlienBrain()->GetMovement().SetMovementMode(AVP_Alien_Movement::EMM_FAST);
		}
	}
	else // not totally hidden and not totally visible
	{
		GetAlienBrain()->GetMovement().SetMovementMode(AVP_Alien_Movement::EMM_MEDIUM);
	}
	
	if (uLightGeneration != m_uLightGeneration || xMovement.IsInError() || GetAlienBrain()->ShouldRefreshPath((pxTargetKnowledge->GetPosition()- m_xTargetPositionDuringPlan).MagnitudeSquared()))
	{
			PlanPathToShadow();
			return;
	}

	if (xMovement.IsFinished())
	{
		PlanPathToShadow();
		return;
	}
}

DEBUGVAR float dbg_fMinTimeBeforeHiss = 1.5f;

void AVP_Behaviour_Alien_HideInShadows::UpdateHideInShadows(float fTimeslice)
{
	const AVP_AlienTeam_Knowledge* pxTargetKnowledge = GetAlienBrain()->GetAlienTeam()->GetKnowledgeSystem()->GetKnowledge(m_uPlayerGuid);
	if (!pxTargetKnowledge)
	{
		SwitchToState(WANDER);
		return;
	}

	if (TookDamage())
	{
		SwitchToState(ATTACK);
		return;
	}

	float fDistanceToTargetSq = (pxTargetKnowledge->GetPosition() - GetAlienBrain()->GetActor()->GetPosition()).MagnitudeSquared();

	bool bTooFar = fDistanceToTargetSq > m_fMaxDistanceSq;
	bool bTooClose = 
		fDistanceToTargetSq < GetAlienBrain()->GetBehaviourData().m_fMinHideDistance * GetAlienBrain()->GetBehaviourData().m_fMinHideDistance ||
		TooCloseToAnyTarget(); 

	if ( m_fTimeTotallyInvisible >  0.0f)
	{
		if (bTooFar || !IsInAdajcentGPV(pxTargetKnowledge->m_uCurrentGPV))
		{
			SwitchToState(MOVING);	
			return;
		}
		else if (bTooClose)
		{
			if (GetAlienBrain()->GetBehaviourData().m_fChaneToRehideIfCloseFraction == 0.0f)
			{
				SwitchToState(HISS_AND_ATTACK);
				return;
			}
			else if (GetAlienBrain()->GetBehaviourData().m_fChaneToRehideIfCloseFraction == 1.0f)
			{
				SwitchToState(MOVING);
				return;
			}
			else if (Asura_Random::GetFloat(0.0f, 1.0f) < GetAlienBrain()->GetBehaviourData().m_fChaneToRehideIfCloseFraction)
			{				
				SwitchToState(MOVING);	
				return;
			}
			else 
			{
				SwitchToState(HISS_AND_ATTACK);
				return;
			}
		}
		else if (GetStateTimer() > m_fTimeToStayHiddenInShadows) 
		{
			m_fTimeTillStopAllowed += dbg_fMinMoveTimeDuringRehide;
			SwitchToState(MOVING); // should re-hide
		}
		else
		{
			// do nothing ...
		}
	}
	else if ( m_fTimeVisibleByMarineLight > 0.0f )
	{
		if (GetAlienBrain()->GetBehaviourData().m_fChaneToRehideIfInLightFraction == 0.0f)
		{
			SwitchToState(HISS_AND_ATTACK);
			return;
		}
		else if (GetAlienBrain()->GetBehaviourData().m_fChaneToRehideIfInLightFraction == 1.0f)
		{
			if (GetStateTimer() > dbg_fMinTimeBeforeHiss)
			{
				SwitchToState(HISS_AND_REHIDE);
			}
			else
			{
				SwitchToState(MOVING);
			}
			return;
		}
		else if (Asura_Random::GetFloat(0.0f, 1.0f) < GetAlienBrain()->GetBehaviourData().m_fChaneToRehideIfInLightFraction)
		{				
			SwitchToState(HISS_AND_ATTACK);	
			return;
		}
		else 
		{
			SwitchToState(HISS_AND_ATTACK);
			return;
		}
	}
	else // should move
	{
		SwitchToState(MOVING);
		return;
	}
}

DEBUGVAR float dbg_fRetryTime = 1.0f;

void AVP_Behaviour_Alien_HideInShadows::UpdateWander()
{
	if (GetStateTimer() > dbg_fRetryTime)
	{
		SwitchToState(MOVING);
	}

	AVP_Behaviour_Alien_Wander* pxBehaviour = static_cast< AVP_Behaviour_Alien_Wander* >( GetChildWithType( this, AVP_BEHAVIOUR_TYPE_ALIEN_WANDER ) );
	if ( !pxBehaviour || pxBehaviour->IsInError() )
	{
		SwitchToState(MOVING);
		return;
	}
}

DEBUGVAR float dbg_fTourchMinMultiplier = 1.0f;
DEBUGVAR float dbg_fTourchMaxMultiplier = 1.2f;
DEBUGVAR float dbg_fTourchDelta = 1.0f;
void AVP_Behaviour_Alien_HideInShadows::UpdateHiddenStatus(float fTimeSlice) 
{
	AVP_Brain_Actor_Alien::InLightStatus eInLightStatus = GetAlienBrain()->GetLightStatus();
	AVP_Brain_Actor_Alien::InMarineLightStatus eInMarineLightStatus = GetAlienBrain()->GetInMarineLight();

	switch(eInMarineLightStatus)
	{
	case AVP_Brain_Actor_Alien::MARINE_LIGHT_NO:
		m_fTimeSinceVisibleByMarineLight += fTimeSlice;
		if (m_fTimeSinceVisibleByMarineLight > dbg_fForgetMarineLightTime)
		{
			m_fTimeVisibleByMarineLight = 0.0f;
		}
		break;
	case AVP_Brain_Actor_Alien::MARINE_LIGHT_CLOSE:
		m_fTimeVisibleByMarineLight += fTimeSlice;
		m_fTimeSinceVisibleByMarineLight = 0.0f;
		break;
	case AVP_Brain_Actor_Alien::MARINE_LIGHT_YES:
		m_fTimeVisibleByMarineLight += fTimeSlice;
		m_fTimeSinceVisibleByMarineLight = 0.0f;
		break;
	default:
		Asura_Assert(false, "Unexpected marine light status");
	}
	
	switch (eInLightStatus)
	{
	case AVP_Brain_Actor_Alien::IN_DARK:
		m_fTimeTillStopAllowed = m_fTimeTillStopAllowed < fTimeSlice ? 0.0f : m_fTimeTillStopAllowed - fTimeSlice;
		m_fTimeTotallyInvisible += fTimeSlice;
		m_fTimeTotallyVisible = 0.0f;

		break;
	case AVP_Brain_Actor_Alien::BETWEEN_LIGHT_AND_DARK:
		m_fTimeTillStopAllowed += 0.25f * fTimeSlice;
		m_fTimeTotallyVisible = 0.0f;
		m_fTimeTotallyInvisible = 0.0f;
		break;
	case AVP_Brain_Actor_Alien::IN_LIGHT:
		m_fTimeTillStopAllowed += 0.5f * fTimeSlice;
		m_fTimeTotallyVisible += fTimeSlice;
		m_fTimeTotallyInvisible = 0.0f;
		break;
	default:
		Asura_Assert(false, "Unexpected light status");
	}
}

bool AVP_Behaviour_Alien_HideInShadows::TookDamage()
{
	bool bResult = GetAlienBrain()->GetActor()->GetHealth() < m_fLastHealth;
	m_fLastHealth = GetAlienBrain()->GetActor()->GetHealth();
	return bResult;
}

bool AVP_Behaviour_Alien_HideInShadows::NeedToRehideAfterATtack() const
{
	if (GetAlienBrain()->GetBehaviourData().m_fChaneToRehideAfterAttackFraction == 0.0f)
	{
		return false;
	}
	else if (GetAlienBrain()->GetBehaviourData().m_fChaneToRehideAfterAttackFraction == 1.0f)
	{
		return true;
	}
	else
	{
		return Asura_Random::GetFloat(0.0f, 1.0f) < GetAlienBrain()->GetBehaviourData().m_fChaneToRehideAfterAttackFraction;
	}
}


bool AVP_Behaviour_Alien_HideInShadows::IsInAdajcentGPV(Asura_Guid uTargetGPV) const
{
	Asura_Guid uMyGPV = GetAlienBrain()->GetCurrentGPV();
	if (uMyGPV == ASURA_GUID_UNREGISTERED)
	{
		return true;
	}

	if (uMyGPV == uTargetGPV)
	{
		return true;
	}

	const AVP_ServerEntity_GamePlayVolume* pxTargetGPV = AVP_ServerEntity_GamePlayVolume::FindEntity(uTargetGPV);
	if (pxTargetGPV == NULL)
	{
		return true;
	}

	for (u_short u = 0; u < pxTargetGPV->GetNumLinkedGamePlayVolumes(); ++u)
	{
		if (uMyGPV == pxTargetGPV->GetLinkedGamePlayVolumeGuid(u))
		{
			return true;
		}
	}

	return false;
}

bool AVP_Behaviour_Alien_HideInShadows::TooCloseToAnyTarget() const
{

	const float fMinDistanceSq = GetAlienBrain()->GetBehaviourData().m_fMinHideDistance * GetAlienBrain()->GetBehaviourData().m_fMinHideDistance;
	const Asura_Vector_3& xOurPosition = GetAlienBrain()->GetActor()->GetPosition();

	for ( Asura_Hashed_List_ConstIt< AVP_AlienTeam_Knowledge > xKnowIter( &GetAlienBrain()->GetAlienTeam()->GetKnowledgeSystem()->GetAllKnowledge() );
		!xKnowIter.Done();
		xKnowIter.Next())
	{
		const AVP_AlienTeam_Knowledge* pxKnowledge = xKnowIter.GetCurrent();
		if ( !pxKnowledge )
		{
			continue;
		}

		if ( (pxKnowledge->m_xPosition - xOurPosition).MagnitudeSquared() < fMinDistanceSq	)
		{
			return true;
		}

	}

	return false;
}



bool AVP_Behaviour_Alien_HideInShadows::SelectVPDestination()
{

	u_int uCandidatesCount = 0;
	struct VPData
	{
		Asura_Guid  m_uGuid;
		Asura_Vector_3 m_xPosition;
		u_short m_usWPID;
	};
	VPData axCandidates[MAX_CANDIDATES];
	u_int uMaxGPV = 0;
	const AVP_ServerEntity_GamePlayVolume* apxGPV[MAX_GVP];	
	AVP_Navigation::NavData& xNavData  = GetAlienBrain()->GetNavData();

	const AVP_AlienTeam_Knowledge* pxTargetKnowledge = GetAlienBrain()->GetAlienTeam()->GetKnowledgeSystem()->GetKnowledge(m_uPlayerGuid);
	if (!pxTargetKnowledge)
	{
		SwitchToState(WANDER);
		return false;
	}

	

	const AVP_ServerEntity_GamePlayVolume* pxTargetGPV = AVP_ServerEntity_GamePlayVolume::FindEntity(pxTargetKnowledge->m_uCurrentGPV);
	if (pxTargetGPV)
	{
		apxGPV[uMaxGPV++] = pxTargetGPV;
		
		for (u_short u = 0; u < MAX_GVP-1 && u < pxTargetGPV->GetNumLinkedGamePlayVolumes(); ++u)
		{
			const AVP_ServerEntity_GamePlayVolume* pxAdjGPV = AVP_ServerEntity_GamePlayVolume::FindEntity(pxTargetGPV->GetLinkedGamePlayVolumeGuid(u));
			if (pxAdjGPV)
			{
				apxGPV[uMaxGPV++] = pxTargetGPV;
			}
		}
	}

	Asura_Hashed_List_ConstIt< AVP_ServerNode_VantagePoint_Predator > xVPIt(AVP_ServerNode_VantagePoint_Predator::GetList());
	Asura_Guid uLastGPV = m_uCurrentVP;
	m_uCurrentVP = ASURA_GUID_UNREGISTERED;

	for(;!xVPIt.Done() && uCandidatesCount <  MAX_CANDIDATES; xVPIt.Next())
	{
		const AVP_ServerNode_VantagePoint_Predator* pxVP = xVPIt.GetCurrent();
		if (!pxVP->IsEntityActive() || pxVP->GetGuid() == uLastGPV)
		{
			continue;
		}
		const Asura_Vector_3& xVPPosition = pxVP->GetPosition();

		// VP must be in the distance
		float fDistanceToTargetSq = (xVPPosition-pxTargetKnowledge->m_xPosition).MagnitudeSquared();
		if ( (fDistanceToTargetSq > dbg_fMaxRangeSq)  ||
			 (fDistanceToTargetSq < GetAlienBrain()->GetBehaviourData().m_fMinHideDistance * GetAlienBrain()->GetBehaviourData().m_fMinHideDistance) )
		{
			continue;
		}

		// VP must be in the GPV
		if (uMaxGPV > 0)
		{
			bool bFound = false;
			for (u_short u = 0; u < uMaxGPV && !bFound; ++u)
			{
				if (apxGPV[u]->Contains(xVPPosition))
				{
					bFound = true;
				}
			}
			if (!bFound)
			{
				continue;
			}
		}

		// VP must be in the shadows.
		AVP_Navigation::TestParams xTestParams( xNavData.m_uPathfinderFlags, ASURA_NAVVERTEX_MAX_ZONE_ID );
		xTestParams.m_bRejectNonalignedWPs = true;
		Asura_Matrix_3x3 xVPOri;
		pxVP->GetOrientation(xVPOri);
		const Asura_Vector_3 xVPUp(-xVPOri.e12, -xVPOri.e22, -xVPOri.e32);
		xTestParams.m_xNormal = xVPUp;
		u_short usWP;
		if ( !AVP_Navigation::GetNearestWaypointID( pxVP->GetPosition(), usWP, xTestParams ) )
		{
			continue;
		}

		const float fLuminaceFactor = AVP_Waypoint_Manager::GetWPLuminance( usWP );
		if (fLuminaceFactor > AVP_Waypoint_Manager::GetDefaultMinHideLuminance())
		{
			continue;
		}
			
		axCandidates[uCandidatesCount].m_uGuid = pxVP->GetGuid();
		axCandidates[uCandidatesCount].m_xPosition = xVPPosition;
		axCandidates[uCandidatesCount].m_usWPID = usWP;
		++uCandidatesCount;
	}

	if (uCandidatesCount == 0)
	{
		return false;
	}

	u_int uSelected = Asura_Random::GetInt(0, uCandidatesCount-1);

	m_uCurrentVP = axCandidates[uSelected].m_uGuid;
	m_xDestination = axCandidates[uSelected].m_xPosition;
	m_usDestinationWP = axCandidates[uSelected].m_usWPID;
	m_bDestinationIsVP = true;
	return true;
}

bool AVP_Behaviour_Alien_HideInShadows::SelectNonVPDestination()
{
	const AVP_AlienTeam_Knowledge* pxTargetKnowledge = GetAlienBrain()->GetAlienTeam()->GetKnowledgeSystem()->GetKnowledge(m_uPlayerGuid);
	if (!pxTargetKnowledge)
	{		
		SwitchToState(WANDER);
		return false;
	}

	AVP_Navigation::NavData xNavData;
	GetAlienBrain()->GetNavData().CopyConfigTo(xNavData);

	xNavData.m_xStartPosition = m_xTargetPositionDuringPlan;
	const AVP_Navigation::TestParams xTestParams( xNavData.m_uPathfinderFlags, ASURA_NAVVERTEX_MAX_ZONE_ID );
	if ( !AVP_Navigation::GetNearestWaypointID( m_xTargetPositionDuringPlan, xNavData.m_usCurrentWPID, xTestParams ) )
	{
		SwitchToState(WANDER);
		return false;
	}

	AVP_AlienShadowsAroundTargetFunctor xShadowAroundTargetFunctor(m_xTargetPositionDuringPlan, GetAlienBrain()->GetActor()->GetPosition(), xNavData.m_usCurrentWPID, dbg_fMaxRangeSq, GetAlienBrain()->GetBehaviourData().m_fMinHideDistance, dbg_fMinMoveDistance );
	const AVP_ServerEntity_GamePlayVolume* pxTargetGPV = AVP_ServerEntity_GamePlayVolume::FindEntity(pxTargetKnowledge->m_uCurrentGPV);
	if (pxTargetGPV)
	{
		xShadowAroundTargetFunctor.AddGPV(pxTargetGPV);
		for (u_short u = 0; u < AVP_AlienShadowsAroundTargetFunctor::MAX_GPV-1 && u < pxTargetGPV->GetNumLinkedGamePlayVolumes(); ++u)
		{
			const AVP_ServerEntity_GamePlayVolume* pxAdjGPV = AVP_ServerEntity_GamePlayVolume::FindEntity(pxTargetGPV->GetLinkedGamePlayVolumeGuid(u));
			if (pxAdjGPV)
			{
				xShadowAroundTargetFunctor.AddGPV(pxAdjGPV);
			}
		}
	}

	AVP_Navigation::FindRouteAwayFrom( xNavData, &xShadowAroundTargetFunctor, false );

	xShadowAroundTargetFunctor.Filter(dbg_fRangeBand);
	u_short usWPID = xShadowAroundTargetFunctor.GetWPID();
	if (usWPID == ASURA_INVALID_WAYPOINT_ID) // can not find a place to hide around so attack instead
	{
		SwitchToState(ATTACK);
		return false;
	}

	m_xDestination = Asura_AI_Navigation_System::GetWaypoint( usWPID )->GetPosition();
	m_usDestinationWP = usWPID;
	m_bDestinationIsVP = false;
	return true;
}

void AVP_Behaviour_Alien_HideInShadows::LateHandleMessage()
{
	Asura_LinkList_It< int > xIter(m_xDelayedMsgList);
	for (; !xIter.Done(); xIter.Next())
	{
		int eMsgId = *xIter.GetCurrent();
		switch (eMsgId)
		{
		case ASURA_MSG_MOVECHAINCOMPLETE:
			switch (GetCurrentState())
			{
			case HISS_AND_ATTACK:
				SwitchToState( ATTACK );
				break;
			case HISS_AND_REHIDE:
				if (TookDamage())
				{
					SwitchToState( ATTACK );
					break;
				}
				else
				{
					SwitchToState(MOVING);
				}
				break;
			default:
				SwitchToState(MOVING);
				Asura_Assert(false, "Unexpected State");
			}
			break;
		default:
			Asura_Assert(false, "Unexpected message");
		}
	}
	m_xDelayedMsgList.Clear();
}

void AVP_Behaviour_Alien_HideInShadows::RequestHiss()
{
	Asura_Message* pxMessage = Asura_Network::CreateMessage( ASURA_MSG_MOVE_START, sizeof( AVP_Move_Alien_Hiss ) );
	if ( pxMessage != NULL )
	{
		Asura_Msg_Move* pxData = static_cast< Asura_Msg_Move* >( pxMessage->GetData() );

		pxMessage->SetFromGuid( GetAlienBrain()->GetOwnerGuid() );
		pxMessage->SetToGuid( GetAlienBrain()->GetOwnerGuid() );

		pxData->ClearFlags();		
		pxData->SetIgnoredByOwner( false );
		pxData->SetMoveID( AVP_Move_Alien_Hiss::GetStaticHash() );

		Asura_Network::Send(pxMessage);
	}
	else
	{
		SwitchToState(MOVING);
	}
}


//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////
