//////////////////////////////////////////////////////////////////////////////
//
// AVP_Behaviour_Alien_Root.cpp
//
// Copyright (C) 2008 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"

// header
#include "Behaviour/BehaviourTypes/Alien/AVP_Behaviour_Alien_Root.h"

// asura
#include "ChunkSystem/Asura_Chunk_Stream.h"
#ifndef ASURA_NO_DEBUG_OUTPUT
#include "Rendering/Asura_Font_System.h"
#endif

// avp
#include "Entity/PhysicalObject/Actor/AVP_ServerEntity_Actor.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Alien/AVP_ServerActor_Alien.h"

// axon
#include "Behaviour/BehaviourTypes/Alien/AVP_Behaviour_Alien_Attack2.h"
#include "Behaviour/BehaviourTypes/Alien/AVP_Behaviour_Alien_HideInShadows.h"
#include "Behaviour/BehaviourTypes/Alien/AVP_Behaviour_Alien_CycleRoundDelayed.h"
#include "Behaviour/BehaviourTypes/AVP_Behaviour_Types.h"
#include "Core/BrainTypes/AVP_Brain_Actor_Alien.h"

//////////////////////////////////////////////////////////////////////////////
//
// AVP_Behaviour_Alien_Root
//
//////////////////////////////////////////////////////////////////////////////

AVP_Behaviour_Alien_Root::AVP_Behaviour_Alien_Root( Axon_Project_Brain* pxBrain )
: PARENT( pxBrain, AVP_BEHAVIOUR_TYPE_ALIEN_ROOT )
{
	SetCurrentState( UNSET );
}

void AVP_Behaviour_Alien_Root::Deactivate()
{
	AVP_Brain_Actor_Alien* pxBrain = GetAlienBrain();
	AVP_ServerActor_Alien* pxActor = pxBrain ? pxBrain->GetAlienActor() : NULL;
	if (pxActor)
	{
		pxActor->EndAcidTrail();
	}

	SwitchToState( UNSET );
	PARENT::Deactivate();
}


DEBUGVAR float dbg_fInjureTimeToStartAcidTrail = 1.5f;

void AVP_Behaviour_Alien_Root::PerFrameUpdate( const float fTimeslice )
{
	PARENT::PerFrameUpdate( fTimeslice );

	if ( fTimeslice == 0.0f ) { return; }
	UpdateStateTimer( fTimeslice );

	if (!GetAlienBrain()->CanPlanMove())
	{
		return;
	}

	const AVP_AlienBehaviourData& xBehaviourData = GetAlienBrain()->GetBehaviourData();

	//## Potential problem here: What if we're trying to disappear/destroy and we get injured?
	if ( GetAlienBrain()->IsInjured() && GetCurrentState() != INJURED )
	{
		SwitchToState( INJURED );
	}

	switch ( GetCurrentState() )
	{
		default:
		case UNSET:
		{
			SwitchToCorrectBehaviourState( xBehaviourData.m_iBehaviour );			
			//SwitchToState(HIDE_IN_SHADOWS);
		}
		break;

		case STANDARD:
		{
			// These behaviour types are all implemented in the one 'standard' behaviour.
			if ( xBehaviourData.m_iBehaviour != AVP_AlienBehaviourData::CUSTOM &&
				xBehaviourData.m_iBehaviour != AVP_AlienBehaviourData::ATTACK &&
				xBehaviourData.m_iBehaviour != AVP_AlienBehaviourData::STALK &&
				xBehaviourData.m_iBehaviour != AVP_AlienBehaviourData::STANDARD )
			{
				SwitchToCorrectBehaviourState( xBehaviourData.m_iBehaviour );
			}
		}
		break;

		case SWARM:
		{
			if ( xBehaviourData.m_iBehaviour != AVP_AlienBehaviourData::SWARM )
			{
				SwitchToCorrectBehaviourState( xBehaviourData.m_iBehaviour );
			}
		}
		break;

		case DISAPPEAR:
		{
			if ( xBehaviourData.m_iBehaviour != AVP_AlienBehaviourData::DISAPPEAR )
			{
				SwitchToCorrectBehaviourState( xBehaviourData.m_iBehaviour );
			}
		}
		break;

		case DESTROY:
		{
			if ( xBehaviourData.m_iBehaviour != AVP_AlienBehaviourData::DESTROY )
			{
				SwitchToCorrectBehaviourState( xBehaviourData.m_iBehaviour );
			}
		}
		break;

		case HIDE:
		{
			if ( xBehaviourData.m_iBehaviour != AVP_AlienBehaviourData::HIDE )
			{
				SwitchToCorrectBehaviourState( xBehaviourData.m_iBehaviour );
			}
		}
		break;

		case INJURED:
		{
			if (GetStateTimer() > dbg_fInjureTimeToStartAcidTrail)
			{
				GetAlienBrain()->GetAlienActor()->StartAcidTrail();
			}

			// We never switch out of this once we're in it.
			// The injured behaviour ensures that it can never be in error or finished.
		}
		break;
		
		case AGGRESIVE:
		{
			if ( xBehaviourData.m_iBehaviour != AVP_AlienBehaviourData::AGGRESIVE)
			{
				SwitchToCorrectBehaviourState( xBehaviourData.m_iBehaviour );
			}
			else
			{
				AVP_Behaviour_Alien_Attack2* pxBehaviour = static_cast< AVP_Behaviour_Alien_Attack2* >( GetChildWithType( this, AVP_BEHAVIOUR_TYPE_ALIEN_ATTACK2 ) );
				if ( !pxBehaviour || pxBehaviour->IsInError() || pxBehaviour->IsFinished() )
				{
					SwitchToState( UNSET );
				}
			}
		}
		break;

		case HIDE_IN_SHADOWS:
		{
			if ( xBehaviourData.m_iBehaviour != AVP_AlienBehaviourData::HIDE_IN_SHADOWS)
			{
				SwitchToCorrectBehaviourState( xBehaviourData.m_iBehaviour );
			}
			else
			{
				AVP_Behaviour_Alien_HideInShadows* pxBehaviour = static_cast< AVP_Behaviour_Alien_HideInShadows* >( GetChildWithType( this, AVP_BEHAVIOUR_TYPE_ALIEN_HIDE_IN_SHADOWS ) );
				if ( !pxBehaviour || pxBehaviour->IsInError() || pxBehaviour->IsFinished() )
				{
					SwitchToState( UNSET );
				}
			}
		}
		break;

		case AMBUSH:
		{
			if ( xBehaviourData.m_iBehaviour != AVP_AlienBehaviourData::AMBUSH)
			{
				SwitchToCorrectBehaviourState( xBehaviourData.m_iBehaviour );
			}
			else
			{
				AVP_Behaviour_Alien_HideInShadows* pxBehaviour = static_cast< AVP_Behaviour_Alien_HideInShadows* >( GetChildWithType( this, AVP_BEHAVIOUR_TYPE_ALIEN_AMBUSH ) );
				if ( !pxBehaviour || pxBehaviour->IsInError() || pxBehaviour->IsFinished() )
				{
					SwitchToState( UNSET );
				}
			}
		}

		break;
	}
}

void AVP_Behaviour_Alien_Root::ReadFromChunkStream( Asura_Chunk_Stream& xStream )
{
	u_int uVersion;
	xStream >> uVersion;

	PARENT::ReadFromChunkStream( xStream );

	if ( uVersion == uCURRENT_CHUNK_VERSION )
	{
	}
	else
	{
		Asura_Assertf( false, ( "AVP_Behaviour_Alien_Root::ReadFromChunkStream - unhandled chunk version (%u)", uVersion ) );
	}
}

void AVP_Behaviour_Alien_Root::WriteToChunkStream( Asura_Chunk_Stream& xStream ) const
{
	xStream << uCURRENT_CHUNK_VERSION;

	PARENT::WriteToChunkStream( xStream );
}

#ifndef ASURA_NO_DEBUG_OUTPUT
void AVP_Behaviour_Alien_Root::RenderDebugInfo()
{
	const char* s_aszStateStrings[] =
	{
		"UNSET",
		"STANDARD",
		"SWARM",
		"DISAPPEAR",
		"HIDE",
		"INJURED",
		"DESTROY",
		"AGGRESIVE",
		"HIDE_IN_SHADOWS",
		"AMBUSH"

	};

	PARENT::RenderDebugInfo();

	Asura_Font_System::Print( GetProjectBrain()->GetDebugOutputPos().x, GetProjectBrain()->GetDebugOutputPos().y, 0.5f, 0xffffffff,
		ASURA_TEXTFORMAT_CENTREJUSTIFIED | ASURA_TEXTFORMAT_FORMATSTRING, "Root: %s %c %d", s_aszStateStrings[ GetCurrentState() ], 
		(GetAlienBrain()->CanPlanMove() ? ' ' : '*'), GetAlienBrain()->GetCurrentTargetGuid() );
	GetProjectBrain()->GetDebugOutputPos().y += 10.0f;
}
#endif

void AVP_Behaviour_Alien_Root::SwitchToCorrectBehaviourState( const int iBehaviour )
{
	switch ( iBehaviour )
	{
		case AVP_AlienBehaviourData::STANDARD:  // fallthrough
		case AVP_AlienBehaviourData::ATTACK:    // fallthrough
		case AVP_AlienBehaviourData::STALK:     // fallthrough
		case AVP_AlienBehaviourData::CUSTOM:    SwitchToState( STANDARD );  break;

		case AVP_AlienBehaviourData::DISAPPEAR: SwitchToState( DISAPPEAR ); break;

		case AVP_AlienBehaviourData::DESTROY:   SwitchToState( DESTROY );	break;

		case AVP_AlienBehaviourData::SWARM:     SwitchToState( SWARM );     break;

		case AVP_AlienBehaviourData::HIDE:      SwitchToState( HIDE );      break;

		case AVP_AlienBehaviourData::AGGRESIVE:      SwitchToState( AGGRESIVE );      break;
		case AVP_AlienBehaviourData::HIDE_IN_SHADOWS:      SwitchToState( HIDE_IN_SHADOWS );      break;
		case AVP_AlienBehaviourData::AMBUSH:      SwitchToState( AMBUSH );      break;

		default:
		case AVP_AlienBehaviourData::GUERILLA:
			Asura_Assert( false, "AVP_Behaviour_Alien_Root::SwitchToCorrectBehaviourState - unhandled behaviour" );
			break;
	}
}

void AVP_Behaviour_Alien_Root::SwitchToState( const int iNewState )
{
	switch ( GetCurrentState() )
	{
		case STANDARD:
		case SWARM:
		case DISAPPEAR:
		case HIDE:
		case DESTROY:
			DestroyAllChildren();
			break;
		case INJURED:
			DestroyAllChildren();
			GetAlienBrain()->GetAlienActor()->EndAcidTrail();
			break;
		case AGGRESIVE:
		case HIDE_IN_SHADOWS:
		case AMBUSH:
			DestroyAllChildren();
			break;
		default:
		case UNSET:
		{
			break;
		}
	}

	SetCurrentState( iNewState );
	ResetStateTimer();
	switch ( GetCurrentState() )
	{
		case STANDARD:
		{
			CreateChildWithType( AVP_BEHAVIOUR_TYPE_ALIEN_STANDARD );
			break;
		}

		case SWARM:
		{
			CreateChildWithType( AVP_BEHAVIOUR_TYPE_ALIEN_SWARM );
			break;
		}

		case DISAPPEAR:
		{
			CreateChildWithType( AVP_BEHAVIOUR_TYPE_ALIEN_CYCLEROUNDDELAYED );
			break;
		}

		case DESTROY:
		{
			AVP_Behaviour_Alien_CycleRoundDelayed* pxCRDBehaviour =
				static_cast< AVP_Behaviour_Alien_CycleRoundDelayed* >( CreateChildWithType( AVP_BEHAVIOUR_TYPE_ALIEN_CYCLEROUNDDELAYED ) );
			if ( pxCRDBehaviour )
			{
				pxCRDBehaviour->SetDestroyNotUnspawn( true );
			}

			break;
		}

		case HIDE:
		{
			CreateChildWithType( AVP_BEHAVIOUR_TYPE_ALIEN_MOVEBETWEENCOVER );
			break;
		}

		case INJURED:
		{
			GetAlienBrain()->GetMovement().Fall();
			CreateChildWithType( AVP_BEHAVIOUR_TYPE_ALIEN_INJURED );
			break;
		}

		case AGGRESIVE:
		{
			CreateChildWithType( AVP_BEHAVIOUR_TYPE_ALIEN_ATTACK2 );
			break;
		}

		case HIDE_IN_SHADOWS:
		{
			CreateChildWithType( AVP_BEHAVIOUR_TYPE_ALIEN_HIDE_IN_SHADOWS );
			break;
		}

		case AMBUSH:
		{
			CreateChildWithType( AVP_BEHAVIOUR_TYPE_ALIEN_AMBUSH );
			break;
		}

		default:
		case UNSET:
		{
			break;
		}
	}
}

AVP_Brain_Actor_Alien* AVP_Behaviour_Alien_Root::GetAlienBrain() const
{
	return static_cast< AVP_Brain_Actor_Alien* >( GetBrain() );
}

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////
