//////////////////////////////////////////////////////////////////////////////
//
// AVP_Behaviour_Alien_Swarm.cpp
//
// Copyright (C) 2008 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"

// header
#include "Behaviour/BehaviourTypes/Alien/AVP_Behaviour_Alien_Swarm.h"

// asura
#ifndef ASURA_NO_DEBUG_OUTPUT
#include "Rendering/Asura_Font_System.h"
#endif
#include "Entity/Controller/AVP_ServerEntity_Squad_Alien.h"

// axon
#include "Behaviour/BehaviourTypes/AVP_Behaviour_Types.h"
#include "Core/BrainTypes/AVP_Brain_Actor_Alien.h"
#include "Knowledge/AVP_Knowledge_System_AlienTeam.h"

//////////////////////////////////////////////////////////////////////////////
//
// AVP_Behaviour_Alien_Swarm
//
//////////////////////////////////////////////////////////////////////////////

AVP_Behaviour_Alien_Swarm::AVP_Behaviour_Alien_Swarm( Axon_Project_Brain* pxBrain )
: PARENT( pxBrain, AVP_BEHAVIOUR_TYPE_ALIEN_SWARM )
{
}

void AVP_Behaviour_Alien_Swarm::Deactivate()
{
	SwitchToState( UNSET );
	PARENT::Deactivate();
}

void AVP_Behaviour_Alien_Swarm::PerFrameUpdate( const float fTimeslice )
{
	PARENT::PerFrameUpdate( fTimeslice );

	if ( fTimeslice == 0.0f ) { return; }
	UpdateStateTimer( fTimeslice );

	if (!GetAlienBrain()->CanPlanMove())
	{
		return;
	}

	switch ( GetCurrentState() )
	{
		default:
		case UNSET:
		case NUM_STATES:
		{
			// Pick our best threat.
			const AVP_AlienTeam_Knowledge* pxBestTarget = GetAlienBrain()->ChooseNewTarget();
			if ( pxBestTarget )
			{
				SwitchToState( SWARMING_IN, pxBestTarget->GetGuid() );
			}

			break;
		}

		case SWARMING_IN:
		{
			const AVP_AlienTeam_Knowledge* pxTargetKnowledge = GetAlienBrain()->GetCurrentTargetKnowledge();
			if ( !pxTargetKnowledge )
			{
				SwitchToState( UNSET );
			}
			else if ( GetAlienBrain()->GetCurrentTargetRange() < AVP_AR_OK )
			{
				SwitchToState( ATTACKING );
			}
			else
			{
				if (GetAlienBrain()->GetCurrentTargetRange() <= AVP_AR_FARISH &&  !GetAlienBrain()->GetMovement().IsBursting() &&
					GetAlienBrain()->GetAlienSquad()->CanBurst(GetAlienBrain()->GetCurrentTargetKnowledge()->GetGuid()))
				{
					GetAlienBrain()->GetMovement().SetBurst(true);
				}

				const Axon_Behaviour* pxSwarmChild = GetChildWithType( this, AVP_BEHAVIOUR_TYPE_ALIEN_SWARMIN );
				if ( !pxSwarmChild || pxSwarmChild->IsFinished() || pxSwarmChild->IsInError() )
				{
					// We have the opportunity to switch threats now.
					const AVP_AlienTeam_Knowledge* pxBestTarget = GetAlienBrain()->ChooseNewTarget();
					if ( pxBestTarget )
					{
						SwitchToState( SWARMING_IN, pxBestTarget->GetGuid() );
					}
					else
					{
						SwitchToState( UNSET );
					}
				}
			}

			break;
		}

		case ATTACKING:
		{
			const AVP_AlienTeam_Knowledge* pxTargetKnowledge = GetAlienBrain()->GetCurrentTargetKnowledge();
			if ( !pxTargetKnowledge )
			{
				SwitchToState( UNSET );
			}
			else
			{

				// If we've finished attacking we should cycle.
				const Axon_Behaviour* pxAttackChild = GetChildWithType( this, AVP_BEHAVIOUR_TYPE_ALIEN_ATTACK );
				if ( !pxAttackChild || pxAttackChild->IsFinished() || pxAttackChild->IsInError() )
				{
					// We have the opportunity to switch threats now.
					const AVP_AlienTeam_Knowledge* pxBestTarget = GetAlienBrain()->ChooseNewTarget();
					if ( pxBestTarget )
					{
						SwitchToState( SWARMING_IN, pxBestTarget->GetGuid() );
					}
					else
					{
						SwitchToState( UNSET );
					}
				}
			}

			break;
		}

		case CYCLING_ROUND:
		{
			break;
		}
	}
}

#ifndef ASURA_NO_DEBUG_OUTPUT
void AVP_Behaviour_Alien_Swarm::RenderDebugInfo()
{
	static const char* s_aszStateStrings[ NUM_STATES ] =
	{
		"UNSET",
		"SWARMING_IN",
		"ATTACKING",
		"CYCLING_ROUND"
	};

	PARENT::RenderDebugInfo();

	Asura_Font_System::Print( GetProjectBrain()->GetDebugOutputPos().x, GetProjectBrain()->GetDebugOutputPos().y, 0.5f, 0xffffffff,
		ASURA_TEXTFORMAT_CENTREJUSTIFIED | ASURA_TEXTFORMAT_FORMATSTRING, "Swarm: %s", s_aszStateStrings[ GetCurrentState() ] );
	GetProjectBrain()->GetDebugOutputPos().y += 10.0f;
}
#endif

void AVP_Behaviour_Alien_Swarm::SwitchToState( const int iNewState, Asura_Guid uGuid /*= ASURA_GUID_UNREGISTERED*/ )
{
	switch ( GetCurrentState() )
	{
		case SWARMING_IN:
		{
			DestroyChildWithType( AVP_BEHAVIOUR_TYPE_ALIEN_SWARMIN );
			break;
		}

		case ATTACKING:
		{
			GetAlienBrain()->GetMovement().SetBurst(false);
			DestroyChildWithType( AVP_BEHAVIOUR_TYPE_ALIEN_ATTACK );
			GetAlienBrain()->StoppedAttackingCurrentTarget();
			break;
		}

		case CYCLING_ROUND:
		{
			DestroyChildWithType( AVP_BEHAVIOUR_TYPE_ALIEN_CYCLEROUNDIMMEDIATE );
			break;
		}

		default:
		case UNSET:
		case NUM_STATES:
		{
			break;
		}
	}

	SetCurrentState( iNewState );
	ResetStateTimer();
	switch ( GetCurrentState() )
	{
		case SWARMING_IN:
		{
			GetAlienBrain()->GetMovement().SetBurst(false);
			GetAlienBrain()->StartedTargeting( uGuid );
			CreateChildWithType( AVP_BEHAVIOUR_TYPE_ALIEN_SWARMIN );
			break;
		}

		case ATTACKING:
		{
			GetAlienBrain()->StartedAttackingCurrentTarget();
			CreateChildWithType( AVP_BEHAVIOUR_TYPE_ALIEN_ATTACK );
			break;
		}

		case CYCLING_ROUND:
		{
			CreateChildWithType( AVP_BEHAVIOUR_TYPE_ALIEN_CYCLEROUNDIMMEDIATE );
			break;
		}

		default:
		case UNSET:
		case NUM_STATES:
		{
			GetAlienBrain()->StoppedTargetingCurrentTarget();
			break;
		}
	}
}

AVP_Brain_Actor_Alien* AVP_Behaviour_Alien_Swarm::GetAlienBrain() const
{
	return static_cast< AVP_Brain_Actor_Alien* >( GetBrain() );
}

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////
