///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_Behaviour_AlienQueen_Movement.h
// Author : David Wooldridge
// Date   : 13/11/2008
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __AVP_BEHAVIOUR_ALIENQUEEN_MOVEMENT_H__
#define __AVP_BEHAVIOUR_ALIENQUEEN_MOVEMENT_H__ 1

// avp axon
#include "Behaviour/Axon_Project_Behaviour.h"
#include "Core/AVP_Navigation.h"

// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////////

class AVP_Brain_Actor;
class AVP_Brain_Actor_AlienQueen;
using namespace AVP_Navigation;


class AVP_Behaviour_AlienQueen_Movement : public Axon_Project_Behaviour
{
	typedef Axon_Project_Behaviour PARENT;

public:
	AVP_Behaviour_AlienQueen_Movement( AVP_Brain_Actor* pxBrain );

	void			SetDestinationGS2NodeGuid( Asura_Guid uGS2NodeGuid );
	bool			GetToPosition(const Asura_Vector_3& xDestination, MovementMode m_eMovementMode = AVP_Navigation::MM_MEDIUM, bool bOverrideMovementMode = false);

	virtual bool	HandleMessage( const Asura_Message* pxMessage );
	virtual bool	Activate();
	virtual void	Deactivate();
	virtual void	Update( const float fTimeslice );
	virtual void	PerFrameUpdate( const float fTimeslice );

#ifndef ASURA_NO_DEBUG_OUTPUT
	virtual void	RenderDebugInfo();
#endif

	AVP_Brain_Actor_AlienQueen*	GetAlienQueenBrain() const;

private:
	enum State
	{
		UNSET,
		TRAVERSING_STANDARD_LINK,
		MOVING_TO_DESTINATION,
		FINISHED,
		NUM_STATES
	};

	void			SwitchToState( const State eNextState );
	bool			UpdateFollowPoint();
	void			UpdatePositionOnRoute();
	void			UpdateMovementSpeed();
	void			UpdateStuckDetection( const float fTimeslice );
	bool			IsTargetInFrontOfQueen();
	bool			IsQueenPerformingMeleeMove();

	State			m_eCurrentState;
	float			m_fTimeInState;
	float			m_fArrivalRange;				// The min distance from destination at which we say we have arrived. 
	Asura_Vector_3	m_xPrevPositionOnRoute;			// Used for detecting if we get stuck.
	float			m_fSmoothedPORVelocity;

	Asura_Vector_3	m_xPositionOnRoute;
	Asura_Vector_3	m_xFollowPoint;

	Asura_Guid		m_uGS2NodeGuid;

};


#endif
