//////////////////////////////////////////////////////////////////////////////
//
// AVP_Behaviour_FaceHugger.cpp
//
// Copyright (C) 2008 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"

// header
#include "Behaviour/BehaviourTypes/FaceHugger/AVP_Behaviour_FaceHugger.h"

// asura
#ifndef ASURA_NO_DEBUG_OUTPUT
#include "Rendering/Asura_Font_System.h"
#endif //ASURA_NO_DEBUG_OUTPUT

// axon
#include "Behaviour/BehaviourTypes/AVP_Behaviour_Types.h"
#include "Behaviour/BehaviourTypes/FaceHugger/AVP_Behaviour_FaceHugger_Attack.h"
#include "Behaviour/BehaviourTypes/FaceHugger/AVP_Behaviour_FaceHugger_HideInShadows.h"
#include "Behaviour/BehaviourTypes/FaceHugger/AVP_Behaviour_FaceHugger_Pet.h"
#include "Core/AxonAVP_Messages.h"
#include "Core/BrainTypes/AVP_Brain_Actor_FaceHugger.h"
#include "Knowledge/AVP_Knowledge_System_AlienTeam.h"

// game
#include "Entity/PhysicalObject/Actor/AVP_ServerEntity_Actor.h"
#include "Entity/Controller/AVP_ServerEntity_Team_Alien.h"

//////////////////////////////////////////////////////////////////////////////
//
// AVP_Behaviour_FaceHugger
//
//////////////////////////////////////////////////////////////////////////////

AVP_Behaviour_FaceHugger::AVP_Behaviour_FaceHugger( AVP_Brain_Actor_FaceHugger* pxBrain )
: PARENT( pxBrain, AVP_BEHAVIOUR_TYPE_FACEHUGGER )
, m_eCurrentState( UNSET )
, m_fTimeInState( 0.0f )
{
}

AVP_Behaviour_FaceHugger::~AVP_Behaviour_FaceHugger()
{
	SwitchToState( UNSET );
}

void AVP_Behaviour_FaceHugger::Deactivate()
{
	SwitchToState( UNSET );
	PARENT::Deactivate();

}


void AVP_Behaviour_FaceHugger::Update( const float fTimeslice )
{
	// GBT :: for when Face Huggers are being tested in a template
	if ( !AVP_ServerEntity_Team_Alien::GetInstance() )
	{
		Asura_Assert( false, "AVP_Behaviour_FaceHugger :: Update - No Alien Team Instance... Is this a Template? You should really be testing in a level.");
		return;
	}

	PARENT::Update( fTimeslice );

	// m_fTimeInState += fTimeslice;

	if (!GetFaceHuggerBrain()->CanPlanMove())
	{
		return;
	}

	switch ( m_eCurrentState )
	{
	case UNSET:
		if (GetFaceHuggerBrain()->IsPet())
		{
			SwitchToState(PET);
		}
		else
		{
			SwitchToStateFromBehaviourData();
		}
		break;
	case PET:
		{
			AVP_Behaviour_FaceHugger_Pet* pxBehaviour = static_cast< AVP_Behaviour_FaceHugger_Pet* >( GetChildWithType( this, AVP_BEHAVIOUR_TYPE_FACEHUGGER_PET ) );
			if ( !pxBehaviour || pxBehaviour->IsInError() || pxBehaviour->IsFinished() )
			{
				SwitchToState(PET);
			}
		}
		break;
	case ATTACK:
		{
			AVP_Behaviour_FaceHugger_Attack* pxBehaviour = static_cast< AVP_Behaviour_FaceHugger_Attack* >( GetChildWithType( this, AVP_BEHAVIOUR_TYPE_FACEHUGGER_ATTACK ) );
			if (SwitchToStateFromBehaviourData())
			{
			}
			else if ( !pxBehaviour || pxBehaviour->IsInError() || pxBehaviour->IsFinished() )
			{
				SwitchToState(ATTACK);
			}
		}
		break;
	case HIDE_IN_SHADOWS:
		{
			AVP_Behaviour_FaceHugger_HideInShadows* pxBehaviour = static_cast< AVP_Behaviour_FaceHugger_HideInShadows* >( GetChildWithType( this, AVP_BEHAVIOUR_TYPE_FACEHUGGER_HIDE_IN_SHADOWS ) );
			if (SwitchToStateFromBehaviourData())
			{
			}
			else if ( !pxBehaviour || pxBehaviour->IsInError() || pxBehaviour->IsFinished() )
			{
				SwitchToState(HIDE_IN_SHADOWS);
			}
		}
		break;
	default:
		Asura_Assert(false, "Unexpected state ");
		break;
	}
}

#ifndef ASURA_NO_DEBUG_OUTPUT
static const char* s_aszStateStrings[] =
{
	"UNSET",
	"PET",
	"ATTACK",
	"HIDE_IN_SHADOWS"
};

void AVP_Behaviour_FaceHugger::RenderDebugInfo()
{
	PARENT::RenderDebugInfo();

	Asura_Font_System::Print( GetProjectBrain()->GetDebugOutputPos().x, GetProjectBrain()->GetDebugOutputPos().y, 0.5f, 0xffffffff,
		ASURA_TEXTFORMAT_CENTREJUSTIFIED | ASURA_TEXTFORMAT_FORMATSTRING, "FaceHugger %s", s_aszStateStrings[ m_eCurrentState ] );
	GetProjectBrain()->GetDebugOutputPos().y += 10.0f;
}
#endif // ASURA_NO_DEBUG_OUTPUT
void AVP_Behaviour_FaceHugger::SwitchToState( const State eNextState )
{
	// Do any work we need to exit our current state.
	switch ( m_eCurrentState )
	{
	case UNSET:
		break;
	case PET:
		DestroyChildWithType( AVP_BEHAVIOUR_TYPE_FACEHUGGER_PET );
		break;
	case ATTACK:
		DestroyChildWithType( AVP_BEHAVIOUR_TYPE_FACEHUGGER_ATTACK );
		break;
	case HIDE_IN_SHADOWS:
		DestroyChildWithType( AVP_BEHAVIOUR_TYPE_FACEHUGGER_HIDE_IN_SHADOWS );
		break;
	default:
		Asura_Assert(false, "Unexpected state ");
		break;
	}

	m_eCurrentState = eNextState;
	m_fTimeInState  = 0.0f;
	switch ( m_eCurrentState )
	{
	case PET:
		{
			AVP_Behaviour_FaceHugger_Pet * pxBehaviour = static_cast< AVP_Behaviour_FaceHugger_Pet* >( CreateChildWithType( AVP_BEHAVIOUR_TYPE_FACEHUGGER_PET ) );
			if ( !pxBehaviour || pxBehaviour->IsInError() || pxBehaviour->IsFinished())
			{
				SwitchToState( UNSET );
			}
		}
		break;
	case ATTACK:
		{
			AVP_Behaviour_FaceHugger_Attack * pxBehaviour = static_cast< AVP_Behaviour_FaceHugger_Attack* >( CreateChildWithType( AVP_BEHAVIOUR_TYPE_FACEHUGGER_ATTACK ) );
			if ( !pxBehaviour || pxBehaviour->IsInError() || pxBehaviour->IsFinished())
			{
				SwitchToState( UNSET );
			}
		}
		break;
	case HIDE_IN_SHADOWS:
		{
			AVP_Behaviour_FaceHugger_HideInShadows * pxBehaviour = static_cast< AVP_Behaviour_FaceHugger_HideInShadows* >( CreateChildWithType( AVP_BEHAVIOUR_TYPE_FACEHUGGER_HIDE_IN_SHADOWS ) );
			if ( !pxBehaviour || pxBehaviour->IsInError() || pxBehaviour->IsFinished())
			{
				SwitchToState( UNSET );
			}
		}
		break;
	case UNSET:
		break;
	default:
		Asura_Assert(false, "Unexpected state ");
		break;
	}
}

bool AVP_Behaviour_FaceHugger::SwitchToStateFromBehaviourData()
{

	const AVP_FaceHuggerBehaviourData& xBehaviourData = GetFaceHuggerBrain()->GetBehaviourData();
	switch (xBehaviourData.m_iBehaviour)
	{
	case AVP_FaceHuggerBehaviourData::ATTACK:
		if (m_eCurrentState != ATTACK) 
		{
			SwitchToState(ATTACK);
			return true;
		}
		break;
	case AVP_FaceHuggerBehaviourData::HIDE_IN_SHADOWS:
		if (m_eCurrentState != HIDE_IN_SHADOWS) 
		{
			SwitchToState(HIDE_IN_SHADOWS);
			return true;
		}
		break;
	default:
		Asura_Assert(false, "Unexpected state");
	}

	return false;
}

AVP_Brain_Actor_FaceHugger* AVP_Behaviour_FaceHugger::GetFaceHuggerBrain() const
{
	return static_cast< AVP_Brain_Actor_FaceHugger* >( GetProjectBrain() );
}


//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////
