//////////////////////////////////////////////////////////////////////////////
//
// AVP_Behaviour_FaceHugger_AttackPrey.cpp
//
// Copyright (C) 2007 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"

// header
#include "Behaviour/BehaviourTypes/FaceHugger/AVP_Behaviour_FaceHugger_AttackPrey.h"

// asura
#include "AI/Asura_AI_Navigation.h"
#include "Network/Asura_Message.h"
#ifndef ASURA_NO_DEBUG_OUTPUT
#include "Rendering/Asura_Font_System.h"
#endif

// avp
#include "Actor/Vertebrates/FaceHugger/AVP_FaceHugger_AIAnimStateMachine.h"
#include "Entity/PhysicalObject/Actor/AVP_ServerEntity_Actor.h"
#include "Knowledge/AVP_Knowledge_System_AlienTeam.h"

// axon
#include "Behaviour/BehaviourTypes/FaceHugger/AVP_Behaviour_FaceHugger_Movement.h"
#include "Behaviour/BehaviourTypes/AVP_Behaviour_Types.h"
#include "Core/AVP_Navigation.h"
#include "Core/BrainTypes/AVP_Brain_Actor_FaceHugger.h"

// avp
#include "Entity/AVP_Messages.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/AVP_Vertebrate_Messages.h"
#include "Actor/Vertebrates/AVP_Vertebrate_MeleeCounter_System.h"

//////////////////////////////////////////////////////////////////////////////
//
// AVP_Behaviour_FaceHugger_AttackPrey
//
//////////////////////////////////////////////////////////////////////////////

AVP_Behaviour_FaceHugger_AttackPrey::AVP_Behaviour_FaceHugger_AttackPrey( AVP_Brain_Actor_FaceHugger* pxBrain )
: PARENT( pxBrain, AVP_BEHAVIOUR_TYPE_FACEHUGGER_ATTACKPREY )
, m_eCurrentState( UNSET )
, m_fTimeInState( 0.0f )
, m_fInsertTimer( 0.0f )
,m_xDestination()
, m_fArrivalRange( 0.75f )
, m_fTimeInJumpRange(0.0f)
, m_bTargetMarked(true)
, m_bAllowAttackPreyFromBehind(false)
{
	GetFaceHuggerBrain()->MarkTargetAttacked();
	GetFaceHuggerBrain()->SetMovementMode(AVP_Navigation::MM_FAST);
}

AVP_Behaviour_FaceHugger_AttackPrey::~AVP_Behaviour_FaceHugger_AttackPrey()
{
	if (m_bTargetMarked)
	{
		GetFaceHuggerBrain()->MarkTargetNotAttacked(true);
	}
}


void AVP_Behaviour_FaceHugger_AttackPrey::AllowAttackPreyFromBehind()
{
	m_bAllowAttackPreyFromBehind = true;
}

void AVP_Behaviour_FaceHugger_AttackPrey::Deactivate()
{
	// Destroy any children.
	Axon_Behaviour* pxChildBehaviour = GetChild();
	while ( pxChildBehaviour )
	{
		UnlinkChild( pxChildBehaviour );
		GetProjectBrain()->DestroyBehaviour( pxChildBehaviour );
		pxChildBehaviour = GetChild();
	}

	PARENT::Deactivate();
}

void AVP_Behaviour_FaceHugger_AttackPrey::PerFrameUpdate( const float fTimeslice )
{
	PARENT::PerFrameUpdate( fTimeslice );

	m_fTimeInState += fTimeslice;

	if (!GetFaceHuggerBrain()->CanPlanMove())
	{
		return;
	}

	switch ( m_eCurrentState )
	{
	case UNSET:
		SwitchToState(RUNNING_IN);
		break;
	case RUNNING_IN:
		UpdateMove(fTimeslice);
		break;
	case ATTACK_JUMP:
		break;
	case FINISHED:
		break;
	default:
		Asura_Assert(false, "unexpected state");
		break;
	}
}

bool AVP_Behaviour_FaceHugger_AttackPrey::HandleMessage( const Asura_Message* pxMessage )
{
	if ( m_eCurrentState == ATTACK_JUMP )
	{
		switch (pxMessage->GetMsgID())
		{
		case ASURA_MSG_MOVECHAINCOMPLETE:
		case ASURA_MSG_MOVE_ABORT:
			SwitchToState( FINISHED );
			GetFaceHuggerBrain()->SetJumping(false);
			return true;
		default:
			break;
		}
	}
	return PARENT::HandleMessage( pxMessage );
}


#ifndef ASURA_NO_DEBUG_OUTPUT
static const char* s_aszStateStrings[] =
{
	"UNSET",	
	"RUNNING IN",
	"ATTACK JUMP", 
	"FINISHED"
};

void AVP_Behaviour_FaceHugger_AttackPrey::RenderDebugInfo()
{
	PARENT::RenderDebugInfo();

	Asura_Font_System::Print( GetProjectBrain()->GetDebugOutputPos().x, GetProjectBrain()->GetDebugOutputPos().y, 0.5f, 0xffffffff,
		ASURA_TEXTFORMAT_CENTREJUSTIFIED | ASURA_TEXTFORMAT_FORMATSTRING, "FaceHugger AttackPrey: %s", s_aszStateStrings[ m_eCurrentState ] );
	GetProjectBrain()->GetDebugOutputPos().y += 10.0f;
}
#endif

void AVP_Behaviour_FaceHugger_AttackPrey::SwitchToState( const State eNextState )
{
	// Do any work we need to exit our current state.
	switch ( m_eCurrentState )
	{
	case UNSET:
		break;
	case RUNNING_IN:
		DestroyChildWithType( AVP_BEHAVIOUR_TYPE_FACEHUGGER_MOVEMENT );
		break;
	case ATTACK_JUMP:
		if (m_bTargetMarked == true)
		{
			m_bTargetMarked = false;
			GetFaceHuggerBrain()->MarkTargetNotAttacked(false);
		}
		break;
	case FINISHED:
	default:
		Asura_Assert(false, "unexpected state" );
		break;
	}

	// Do any work we need to change into our next state.
	m_eCurrentState = eNextState;
	m_fTimeInState  = 0.0f;
	switch ( m_eCurrentState )
	{
	case RUNNING_IN:
		InitiateMove();
		break;
	case ATTACK_JUMP:
		RequestAttackPrey();
		break;
	case FINISHED:
		if (m_bTargetMarked == true)
		{
			m_bTargetMarked = false;
			GetFaceHuggerBrain()->MarkTargetNotAttacked(true);
		}
		SetFinished(true);
		break;
	default:
		Asura_Assert(false, "unexpected state" );
		break;
	}
}

AVP_Brain_Actor_FaceHugger* AVP_Behaviour_FaceHugger_AttackPrey::GetFaceHuggerBrain() const
{
	return static_cast< AVP_Brain_Actor_FaceHugger* >( GetProjectBrain() );
}

DEBUGVAR float dbg_fIgnoreTargetTime = 2.0f;

void AVP_Behaviour_FaceHugger_AttackPrey::InitiateMove()
{
	const AVP_AlienTeam_Knowledge* pxTarget = GetFaceHuggerBrain()->GetTarget();
	if (!pxTarget)
	{
		SetInError(true);
		SwitchToState(FINISHED);
		return;
	}

	m_xDestination = pxTarget->GetPosition();

	AVP_Navigation::NavData& xNavData = GetFaceHuggerBrain()->GetNavData();

	AVP_Navigation::TestParams xTestParams;
	xTestParams.m_uPathfinderFlags = xNavData.m_uPathfinderFlags;
	xTestParams.m_usZone           = xNavData.m_usCurrentZone;
	AVP_Navigation::ResolveToValidPosition( m_xDestination, xTestParams );

	AVP_Behaviour_FaceHugger_Movement * pxBehaviour = static_cast< AVP_Behaviour_FaceHugger_Movement* >( CreateChildWithType( AVP_BEHAVIOUR_TYPE_FACEHUGGER_MOVEMENT ) );
	if ( pxBehaviour && pxBehaviour->GetToPosition( m_xDestination, true ) )
	{
		pxBehaviour->SetMovementMode(AVP_Behaviour_FaceHugger_Movement::EMM_FAST);
	}
	else
	{
		GetFaceHuggerBrain()->IgnoreCurrentTarget(dbg_fIgnoreTargetTime);
		SetInError(true);
		SwitchToState(FINISHED);
	}
}

DEBUGVAR float dbg_fMaxTimeInJumpRange = 0.5f;
void AVP_Behaviour_FaceHugger_AttackPrey::UpdateMove(float fTimeslice)
{
	AVP_Brain_Actor_FaceHugger* pxBrain =  GetFaceHuggerBrain();
	AVP_Behaviour_FaceHugger_Movement* pxBehaviour = static_cast< AVP_Behaviour_FaceHugger_Movement* >( GetChildWithType( this, AVP_BEHAVIOUR_TYPE_FACEHUGGER_MOVEMENT ) );
	if ( pxBrain->IsTargetInJumpRange() )
	{
		m_fTimeInJumpRange += fTimeslice;
		if (m_fTimeInJumpRange > dbg_fMaxTimeInJumpRange)
		{
			SetInError(true);
			SwitchToState(FINISHED);
		}
		else if ((m_bAllowAttackPreyFromBehind || pxBrain->IsTargetFacingMe()))
		{
			if (pxBrain->IsTargetVisible())
			{
				SwitchToState( ATTACK_JUMP );
			}
		}
		else
		{
			SwitchToState(FINISHED);
		}
	}
	else if ( !pxBehaviour || pxBehaviour->IsInError() || !pxBrain->IsTargetValid() )
	{
		SetInError(true);
		SwitchToState(FINISHED);
	}
	else if( pxBehaviour->IsFinished() )
	{
		SwitchToState(RUNNING_IN);
	}
	else
	{
		m_fTimeInJumpRange = 0.0f;

		const AVP_AlienTeam_Knowledge* pxTarget = pxBrain->GetTarget();

		if (!pxTarget)
		{
			SetInError(true);
			SwitchToState(FINISHED);
			return;
		}

		Asura_Vector_3 xDestination = pxTarget->GetPosition();

		AVP_Navigation::NavData& xNavData = pxBrain->GetNavData();

		AVP_Navigation::TestParams xTestParams;
		xTestParams.m_uPathfinderFlags = xNavData.m_uPathfinderFlags;
		xTestParams.m_usZone           = xNavData.m_usCurrentZone;
		AVP_Navigation::ResolveToValidPosition( m_xDestination, xTestParams );

		if ( (m_xDestination - xDestination).MagnitudeSquared() > m_fArrivalRange * m_fArrivalRange )
		{
			m_xDestination = xDestination;

			if (!pxBehaviour->GetToPosition( m_xDestination ))
			{
				GetFaceHuggerBrain()->IgnoreCurrentTarget(dbg_fIgnoreTargetTime);
				SetInError(true);
				SwitchToState(FINISHED);
			}
		}

	}
}
void AVP_Behaviour_FaceHugger_AttackPrey::RequestAttackPrey()
{
	AVP_Brain_Actor_FaceHugger* pxBrain =  GetFaceHuggerBrain();

	if ( !pxBrain->IsTargetValid() )
	{
		Asura_Assert( false, "AVP_Behaviour_FaceHugger_AttackPrey::RequestAttackPrey target become invalid!" );
		return;
	}

	// Send a request melee message.
	Asura_Message* pxMessage = Asura_Network::CreateMessage( AVP_MSGID_REQUEST_MELEE, sizeof( AVP_Message_Vertebrate_RequestMelee ) );
	if ( pxMessage )
	{
		AVP_Message_Vertebrate_RequestMelee* pxData = static_cast< AVP_Message_Vertebrate_RequestMelee* >( pxMessage->GetData() );

		pxData->SetType( AVP_MELEEATTACK_TYPE_NORMAL );
		pxData->SetTargetGuid( pxBrain->GetTarget()->GetGuid() );
		pxMessage->SetToGuid( pxBrain->GetOwnerGuid() );
		pxMessage->SetFromGuid( pxBrain->GetOwnerGuid() );
		Asura_Network::Send( pxMessage );
		GetFaceHuggerBrain()->SetJumping(true);
	}
	else
	{
		Asura_Assert( false, "AVP_Behaviour_FaceHugger_AttackPrey::RequestAttackPrey message not created" );

	}
}


//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////
