//////////////////////////////////////////////////////////////////////////////
//
// AVP_Behaviour_FaceHugger_Pet.cpp
//
// Copyright (C) 2007 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"

// header
#include "Behaviour/BehaviourTypes/FaceHugger/AVP_Behaviour_FaceHugger_Pet.h"

// asura
#include "AI/Asura_AI_Navigation.h"
#ifndef ASURA_NO_DEBUG_OUTPUT
#include "Rendering/Asura_Font_System.h"
#endif

// avp
#include "Actor/Vertebrates/FaceHugger/AVP_FaceHugger_AIAnimStateMachine.h"
#include "Blueprint/AVP_Blueprint_HashIDs.h"
#include "Entity/Controller/AVP_ServerEntity_Squad_FaceHugger.h"
#include "Entity/Damage/AVP_EntityDamage_Request.h"
#include "Entity/PhysicalObject/Actor/AVP_ServerEntity_Actor.h" 
#include "Entity/Spawners/AVP_ServerEntity_SpawnController.h"
#include "Entity/Spawners/AVP_ServerEntity_SpawnPoint2.h"

// axon
#include "Behaviour/BehaviourTypes/FaceHugger/AVP_Behaviour_FaceHugger_AttackPrey.h"
#include "Behaviour/BehaviourTypes/FaceHugger/AVP_Behaviour_FaceHugger_Movement.h"
#include "Behaviour/BehaviourTypes/FaceHugger/AVP_Behaviour_FaceHugger_MoveToPrey.h"
#include "Behaviour/BehaviourTypes/AVP_Behaviour_Types.h"
#include "Core/AVP_Navigation.h"
#include "Core/BrainTypes/AVP_Brain_Actor_FaceHugger.h"
#include "Knowledge/AVP_Knowledge_System_AlienTeam.h"


//////////////////////////////////////////////////////////////////////////////
//
// AVP_Behaviour_FaceHugger_Pet
//
//////////////////////////////////////////////////////////////////////////////

AVP_Behaviour_FaceHugger_Pet::AVP_Behaviour_FaceHugger_Pet( AVP_Brain_Actor_FaceHugger* pxBrain )
: PARENT( pxBrain, AVP_BEHAVIOUR_TYPE_FACEHUGGER_PET )
, m_eCurrentState( WAIT )
, m_fTimeInState( 0.0f )
, m_xMasterLastPosition( Asura_Maths::ZeroVector3 )
, m_xMasterLastForwardDirection( Asura_Maths::ZeroVector3 )
, m_bBeOnRightSide(Asura_Random::GetInt(0,1) > 0)
, m_fTimeSinceLastFindTarget(0.0f)
{
}

void AVP_Behaviour_FaceHugger_Pet::Deactivate()
{
	// Destroy any children.
	Axon_Behaviour* pxChildBehaviour = GetChild();
	while ( pxChildBehaviour )
	{
		UnlinkChild( pxChildBehaviour );
		GetProjectBrain()->DestroyBehaviour( pxChildBehaviour );
		pxChildBehaviour = GetChild();
	}

	PARENT::Deactivate();
}

DEBUGVAR float dbg_fMaxTimeInWait = 5.0f;
DEBUGVAR float dbg_fMaxTimeInForceWait = 0.5f;
DEBUGVAR float dbg_fMaxTimeSinceLastFindTargetPetMode = 1.0f;

DEBUGVAR float dbg_fSmallDistanceFromEndPath = 2.0f;
DEBUGVAR float dbg_fMediumDistanceFromEndPath = 5.0f;
DEBUGVAR float dbg_fLongIgnoreTime = 30.0f;


void AVP_Behaviour_FaceHugger_Pet::PerFrameUpdate( const float fTimeslice )
{
	PARENT::PerFrameUpdate( fTimeslice );

	m_fTimeInState += fTimeslice;
	m_fTimeSinceLastFindTarget += fTimeslice;

	if (!GetFaceHuggerBrain()->CanPlanMove())
	{
		return;
	}

	switch ( m_eCurrentState )
	{
	case FORCE_WAIT:
		break;
	case WAIT:
		break;
	case GET_TO_MASTER:
		{
			AVP_Behaviour_FaceHugger_Movement* pxBehaviour = static_cast< AVP_Behaviour_FaceHugger_Movement* >( GetChildWithType( this, AVP_BEHAVIOUR_TYPE_FACEHUGGER_MOVEMENT ) );
			if (!pxBehaviour || pxBehaviour->IsInError() )
			{
				SwitchToState(GET_TO_MASTER);
			}
			else if (pxBehaviour->IsFinished())
			{
				SwitchToState(WAIT);
			}
			else
			{
				float fDistanceToTravel = pxBehaviour->GetLengthLeftOnPath();
				if (fDistanceToTravel < dbg_fSmallDistanceFromEndPath )
				{
					GetFaceHuggerBrain()->SetMovementMode(AVP_Navigation::MM_SLOW);
				}
				else if (fDistanceToTravel < dbg_fMediumDistanceFromEndPath )
				{
					GetFaceHuggerBrain()->SetMovementMode(AVP_Navigation::MM_MEDIUM);
				}
				else
				{
					GetFaceHuggerBrain()->SetMovementMode(AVP_Navigation::MM_FAST);
				}
			}
		}
		break;
	case MOVE_TO_PRAY:
		{
			AVP_Behaviour_FaceHugger_MoveToPrey* pxBehaviour = static_cast< AVP_Behaviour_FaceHugger_MoveToPrey* >( GetChildWithType( this, AVP_BEHAVIOUR_TYPE_FACEHUGGER_MOVETOPREY ) );
			if (!GetFaceHuggerBrain()->IsAggressive())
			{
				SwitchToState(FORCE_WAIT);
			}
			else if ( !pxBehaviour || pxBehaviour->IsInError() || pxBehaviour->IsFinished() )
			{
				GetFaceHuggerBrain()->IgnoreCurrentTarget(dbg_fLongIgnoreTime);
				SwitchToState(WAIT);
			}
			else if ( ShouldAttack(pxBehaviour) )
			{
				SwitchToState(ATTACKING);
			}
		}
		break;
	case ATTACKING:
		{
			AVP_Behaviour_FaceHugger_AttackPrey* pxBehaviour = static_cast< AVP_Behaviour_FaceHugger_AttackPrey* >( GetChildWithType( this, AVP_BEHAVIOUR_TYPE_FACEHUGGER_ATTACKPREY ) );
			if (!GetFaceHuggerBrain()->IsAggressive())
			{
				SwitchToState(FORCE_WAIT);
			}
			else if ( !pxBehaviour || pxBehaviour->IsInError() || pxBehaviour->IsFinished() )
			{
				SwitchToState(WAIT);
			}
		}
		break;
	case DESPAWN:
		break;
	case DIE:
		break;
	default:
		Asura_Assert(false, "Unexpected state");
	}
}

DEBUGVAR float fDistanceMediumDistanceToTarget = 10.0f; 
DEBUGVAR float fDistanceSmallDistanceToTarget = 3.0f; 


void AVP_Behaviour_FaceHugger_Pet::Update( const float fTimeslice )
{
	PARENT::Update( fTimeslice );

	m_fTimeInState += fTimeslice;
	m_fTimeSinceLastFindTarget += fTimeslice;

	if (!GetFaceHuggerBrain()->CanPlanMove())
	{
		return;
	}

	switch ( m_eCurrentState )
	{
	case FORCE_WAIT:
		if (m_fTimeInState > dbg_fMaxTimeInForceWait )
		{
			SwitchToState(WAIT);
		}
		else if (GetFaceHuggerBrain()->IsAlwaysAggressive())
		{
			SwitchToState(WAIT);
		}
		break;
	case WAIT:
		if (GetFaceHuggerBrain()->IsAggressive() && m_fTimeSinceLastFindTarget > dbg_fMaxTimeSinceLastFindTargetPetMode)
		{
			m_fTimeSinceLastFindTarget = 0.0f;
			if(GetFaceHuggerBrain()->FindTargetPetMode())
			{
				SwitchToState(MOVE_TO_PRAY);
				break;
			}
			else if (GetFaceHuggerBrain()->IsAlwaysAggressive())
			{
				SwitchToState(DESPAWN);
				break;
			}
		}
		
		if (!GetFaceHuggerBrain()->IsAlwaysAggressive() && (GetFaceHuggerBrain()->IsTooFarOrToCloseToMaster() || GetFaceHuggerBrain()->IsInMasterFoV() || m_fTimeInState > dbg_fMaxTimeInWait) )
		{
			SwitchToState(GET_TO_MASTER);
		}
		break;
	case GET_TO_MASTER:
		{
			if (GetFaceHuggerBrain()->IsAggressive() && m_fTimeSinceLastFindTarget > dbg_fMaxTimeSinceLastFindTargetPetMode)
			{
				m_fTimeSinceLastFindTarget = 0.0f;
				if(GetFaceHuggerBrain()->FindTargetPetMode())
				{
					SwitchToState(MOVE_TO_PRAY);
					break;
				}
			}
			
			if (NeedToRecalculateDestination())
			{
				SwitchToState(GET_TO_MASTER);
				break;
			}

			AVP_Behaviour_FaceHugger_Movement* pxMovementChild = static_cast<AVP_Behaviour_FaceHugger_Movement*>(GetChildWithType( this, AVP_BEHAVIOUR_TYPE_FACEHUGGER_MOVEMENT ));
			if (!pxMovementChild || pxMovementChild->IsInError())
			{
				SwitchToState(GET_TO_MASTER);
				break;
			}
			else if (pxMovementChild->IsFinished())
			{
				SwitchToState(WAIT);
				break;
			}
			else
			{
				float fLengthOnPath = pxMovementChild->GetLengthLeftOnPath();
				if (fLengthOnPath < fDistanceSmallDistanceToTarget )
				{
					pxMovementChild->SetMovementMode(AVP_Behaviour_FaceHugger_Movement::EMM_SLOW);
				}
				else if (fLengthOnPath < fDistanceMediumDistanceToTarget )
				{
					pxMovementChild->SetMovementMode(AVP_Behaviour_FaceHugger_Movement::EMM_MEDIUM);
				}
				else
				{
					pxMovementChild->SetMovementMode(AVP_Behaviour_FaceHugger_Movement::EMM_FAST);
				}
			}
		}
		break;
	case MOVE_TO_PRAY:
		{
			if (m_fTimeSinceLastFindTarget > dbg_fMaxTimeSinceLastFindTargetPetMode || !GetFaceHuggerBrain()->IsTargetValid() )
			{
				const AVP_AlienTeam_Knowledge* pxTarget = GetFaceHuggerBrain()->GetTarget();
				Asura_Guid uLastGuid = pxTarget ? pxTarget->GetGuid() : ASURA_GUID_UNREGISTERED;
				m_fTimeSinceLastFindTarget = 0.0f;
				GetFaceHuggerBrain()->FindTargetPetMode();
				const AVP_AlienTeam_Knowledge* pxNewTarget = GetFaceHuggerBrain()->GetTarget();
				Asura_Guid uNewGuid = pxNewTarget ? pxNewTarget->GetGuid() : ASURA_GUID_UNREGISTERED;
				if (uNewGuid != uLastGuid)
				{
					SwitchToState(MOVE_TO_PRAY);
				}	
			}
		}
		break;
	case DESPAWN:
		{
			AVP_Behaviour_FaceHugger_Movement* pxMovementChild = static_cast<AVP_Behaviour_FaceHugger_Movement*>(GetChildWithType( this, AVP_BEHAVIOUR_TYPE_FACEHUGGER_MOVEMENT ));
			if (!pxMovementChild || pxMovementChild->IsInError())
			{
				SwitchToState(DESPAWN);
				break;
			}
			else if (pxMovementChild->IsFinished())
			{
				GetFaceHuggerBrain()->GetActor()->SetThisAndClientsAsDestroyed();
				// Actors register themselves as out of play when they are killed or deactivated.
				// We're skipping all of this, so we have to do it ourselves.
				AVP_ServerEntity_SpawnController::ActorOutOfPlay( GetProjectBrain()->GetOwnerGuid() );
				AVP_ServerEntity_SpawnPoint2::ActorOutOfPlay( GetProjectBrain()->GetOwnerGuid() );
				break;
			}
		}
		break;
	case DIE:
	case ATTACKING:
		break;
	default:
		Asura_Assert(false, "Unexpected state");
	}
}

#ifndef ASURA_NO_DEBUG_OUTPUT
static const char* s_aszStateStrings[] =
{
	"WAIT",
	"FORCE_WAIT",
	"GET_TO_MASTER",
	"MOVE_TO_PRAY",
	"ATTACKING",
	"DESPAWN",
	"DIE",
	"NUM_STATES"
};

void AVP_Behaviour_FaceHugger_Pet::RenderDebugInfo()
{
	PARENT::RenderDebugInfo();

	Asura_Font_System::Print( GetProjectBrain()->GetDebugOutputPos().x, GetProjectBrain()->GetDebugOutputPos().y, 0.5f, 0xffffffff,
		ASURA_TEXTFORMAT_CENTREJUSTIFIED | ASURA_TEXTFORMAT_FORMATSTRING, "FaceHugger Pet: %s", s_aszStateStrings[ m_eCurrentState ] );
	GetProjectBrain()->GetDebugOutputPos().y += 10.0f;
}
#endif


void AVP_Behaviour_FaceHugger_Pet::SwitchToState( const State eNextState )
{
	// Do any work we need to exit our current state.
	switch ( m_eCurrentState )
	{
	case FORCE_WAIT:
	case WAIT:
		break;
	case GET_TO_MASTER:
	case DESPAWN:
		DestroyChildWithType( AVP_BEHAVIOUR_TYPE_FACEHUGGER_MOVEMENT );
		break;
	case ATTACKING:
		GetFaceHuggerBrain()->SetNearbyActorGuidToIgnore(ASURA_GUID_UNREGISTERED);
		DestroyChildWithType( AVP_BEHAVIOUR_TYPE_FACEHUGGER_ATTACK );
		break;
	case MOVE_TO_PRAY:
		DestroyChildWithType( AVP_BEHAVIOUR_TYPE_FACEHUGGER_MOVETOPREY );
		break;
	case DIE:
	default:
		Asura_Assert(false, "Unexpected state");
	}

	// Do any work we need to change into our next state.
	m_eCurrentState = eNextState;
	m_fTimeInState  = 0.0f;
	switch ( m_eCurrentState )
	{
	case FORCE_WAIT:
	case WAIT:
		break;
	case GET_TO_MASTER:
		CalculateAndMoveToDestination();
		break;
	case MOVE_TO_PRAY:
		{
			AVP_Behaviour_FaceHugger_MoveToPrey * pxBehaviour = static_cast< AVP_Behaviour_FaceHugger_MoveToPrey* >( CreateChildWithType( AVP_BEHAVIOUR_TYPE_FACEHUGGER_MOVETOPREY ) );
			if ( !pxBehaviour || pxBehaviour->IsInError() || pxBehaviour->IsFinished())
			{
				GetFaceHuggerBrain()->IgnoreCurrentTarget(dbg_fLongIgnoreTime);
				SwitchToState( FORCE_WAIT );
			}
		}
		break;
	case ATTACKING:
		{
			GetFaceHuggerBrain()->SetNearbyActorGuidToIgnore(GetFaceHuggerBrain()->GetTarget()->GetGuid());
			AVP_Behaviour_FaceHugger_AttackPrey * pxBehaviour = static_cast< AVP_Behaviour_FaceHugger_AttackPrey* >( CreateChildWithType( AVP_BEHAVIOUR_TYPE_FACEHUGGER_ATTACKPREY ) );
			if ( !pxBehaviour || pxBehaviour->IsInError() || pxBehaviour->IsFinished())
			{
				SwitchToState( FORCE_WAIT );
			}
		}
		break;
	case DESPAWN:
		CalculateAndMoveToSpawnpoint();
		break;
	case DIE: // force death since we die not near our spawn point - so maybe infront of the player
		{
			AVP_EntityDamage_RequestVictim xDamage( uAVP_BLUEPRINT_NAME_DAMAGE_MELEE,
				GetBrain()->GetOwnerGuid(),
				ASURA_HASH_ID_UNSET,
				GetBrain()->GetOwnerGuid(),
				1000000.0f,
				GetFaceHuggerBrain()->GetActor()->GetPosition(),
				Asura_Vector_3( 0.0f, 1.0f, 0.0f ),
				Asura_Vector_3( 0.0f, 1.0f, 0.0f ) );
			GetFaceHuggerBrain()->GetActor()->TakeDamageExtended( xDamage, true );
		}
		break;
	default:
		Asura_Assert(false, "Unexpected state");
	}
}

AVP_Brain_Actor_FaceHugger* AVP_Behaviour_FaceHugger_Pet::GetFaceHuggerBrain() const
{
	return static_cast< AVP_Brain_Actor_FaceHugger* >( GetProjectBrain() );
}


DEBUGVAR float dbg_fMinDirectionAngle = Asura_Maths::DegToRad(60.0f);
DEBUGVAR float dbg_fMaxDirectionAngle = Asura_Maths::DegToRad(120.0f);
DEBUGVAR float dbg_fDesiredDistance = 6.0f;
DEBUGVAR float dbg_fDesiredDistanceVarianceMax = 3.0f;
DEBUGVAR float dbg_fDesiredDistanceVarianceMin = -1.0f;
DEBUGVAR float dbg_fMinDistance = 3.0f;
DEBUGVAR float dbg_fMaxDistance = 15.0f;
DEBUGVAR float dbg_fChanceToSwitchSides = 0.1;

void AVP_Behaviour_FaceHugger_Pet::CalculateAndMoveToDestination()
{
	AVP_ServerEntity_Actor* pxMaster = GetFaceHuggerBrain()->GetMaster();

	if (!pxMaster)
	{
		SwitchToState(FORCE_WAIT);
		return;
	}

	// update master position and forward direction and position.
	m_xMasterLastPosition = pxMaster->GetPosition();
	const Asura_Matrix_3x3& xMasterOrientation = pxMaster->GetOrientation();

	m_xMasterLastForwardDirection = pxMaster->GetAimDirection();

	Asura_Vector_3 xUp(-xMasterOrientation.e12, -xMasterOrientation.e22, -xMasterOrientation.e32);
	Asura_Vector_3 xForward(xMasterOrientation.e13, xMasterOrientation.e23, xMasterOrientation.e33);

	Asura_Quat xTurnQuat;

	float fRotateAngle = Asura_Random::GetFloat(dbg_fMinDirectionAngle, dbg_fMaxDirectionAngle);
	if (!m_bBeOnRightSide)
	{
		fRotateAngle = Asura_Maths::Pi * 2.0f - fRotateAngle;
	}

	if (Asura_Random::GetFloat(0,1) < dbg_fChanceToSwitchSides)
	{
		m_bBeOnRightSide = !m_bBeOnRightSide;
	}

	xTurnQuat.FromRotation(fRotateAngle, xUp);

	Asura_Vector_3 xDestDirecion;
	xDestDirecion = xTurnQuat.RotateVector(xForward);


	AVP_Navigation::FaceHuggerAwayFromInDirFunctor xFunctor(m_xMasterLastPosition, dbg_fDesiredDistance + Asura_Random::GetFloat(dbg_fDesiredDistanceVarianceMin,dbg_fDesiredDistanceVarianceMax) , xDestDirecion);

	AVP_Navigation::NavData& xOrigNavData = GetFaceHuggerBrain()->GetNavData();
	AVP_Navigation::NavData xNavData;
	xOrigNavData.CopyConfigTo(xNavData);

	xNavData.m_xStartPosition = m_xMasterLastPosition;

	AVP_Navigation::TestParams xTestParams;
	xTestParams.m_uPathfinderFlags		= xNavData.m_uPathfinderFlags;
	xTestParams.m_bRejectNonalignedWPs	= true;
	xTestParams.m_xNormal				= xUp;

	if ( !AVP_Navigation::GetNearestWaypointID(m_xMasterLastPosition, xNavData.m_usCurrentWPID, xTestParams))
	{
		SwitchToState(FORCE_WAIT);
		return;
	}

	// Clear out the old route and find the new one.
	xNavData.ResetRoute();

	if (!AVP_Navigation::FindRouteAwayFrom( xNavData, &xFunctor ) || xNavData.m_usRouteLength == 0)
	{
		SwitchToState( FORCE_WAIT );
		return;
	}

	const Asura_AI_NavVertex *pxDestVertex = Asura_AI_Navigation_System::GetWaypoint(xNavData.m_pusRoute[xNavData.m_usRouteLength-1]);

	Asura_Vector_3 xDestination = pxDestVertex->GetPosition();

	AVP_Behaviour_FaceHugger_Movement * pxBehaviour = static_cast< AVP_Behaviour_FaceHugger_Movement* >( CreateChildWithType( AVP_BEHAVIOUR_TYPE_FACEHUGGER_MOVEMENT ) );

	if (!pxBehaviour)
	{
		SwitchToState( FORCE_WAIT );
		return;
	}

	pxBehaviour->SetMovementMode(AVP_Behaviour_FaceHugger_Movement::EMM_FAST);
	if (!pxBehaviour->GetToPosition(xDestination, m_xMasterLastPosition, dbg_fMinDistance, dbg_fMaxDistance, -m_xMasterLastForwardDirection ))
	{
		SwitchToState( FORCE_WAIT );
		return;
	}
}

void AVP_Behaviour_FaceHugger_Pet::CalculateAndMoveToSpawnpoint()
{
	const Asura_Vector_3& xSpawnPoint = GetFaceHuggerBrain()->GetSpawnPoint();

	AVP_Behaviour_FaceHugger_Movement * pxBehaviour = static_cast< AVP_Behaviour_FaceHugger_Movement* >( CreateChildWithType( AVP_BEHAVIOUR_TYPE_FACEHUGGER_MOVEMENT ) );

	if (!pxBehaviour)
	{
		SwitchToState( DIE );
		return;
	}

	pxBehaviour->SetMovementMode(AVP_Behaviour_FaceHugger_Movement::EMM_FAST);
	if (!pxBehaviour->GetToPosition(xSpawnPoint))
	{
		SwitchToState( DIE );
		return;
	}
}


DEBUGVAR float dbg_fMediumDeltaSquared = 5.0f*5.0f;
DEBUGVAR float dbg_fLargeDeltaSquared = 15.0f*15.0f;
DEBUGVAR float dbg_fVeryLargeDeltaSquared = 25.0f*25.0f;

DEBUGVAR float dbg_fVerySmallTurnCosDelta = Asura_Maths::Cosine(Asura_Maths::DegToRad(30.0f));
DEBUGVAR float dbg_fSmallTurnCosDelta = Asura_Maths::Cosine(Asura_Maths::DegToRad(45.0f));
DEBUGVAR float dbg_fMediumTurnCosDelta = Asura_Maths::Cosine(Asura_Maths::DegToRad(60.0f));

DEBUGVAR float dbg_fVerySmallPathDistance = 7.0f;
DEBUGVAR float dbg_fSmallPathDistance = 15.0f;
DEBUGVAR float dbg_fMediumPathDistance = 30.0f;
DEBUGVAR float dbg_fLargePathDistance = 60.0f;

bool AVP_Behaviour_FaceHugger_Pet::NeedToRecalculateDestination() const
{
	
	AVP_ServerEntity_Actor* pxMaster = GetFaceHuggerBrain()->GetMaster();

	if (!pxMaster)
	{
		return false;
	}

	float fDeltaMasterMovementSq = (pxMaster->GetPosition()-m_xMasterLastPosition).MagnitudeSquared();
	Asura_Vector_3 xForward;
	xForward = pxMaster->GetAimDirection();
	float fdeltaMasterTurnCos = m_xMasterLastForwardDirection * xForward;

	AVP_Behaviour_FaceHugger_Movement* pxBehaviour = static_cast< AVP_Behaviour_FaceHugger_Movement* >( GetChildWithType( this, AVP_BEHAVIOUR_TYPE_FACEHUGGER_MOVEMENT ) );
	if (pxBehaviour && pxBehaviour->IsInError() )
	{
		return true;
	}

	float fPathLength = pxBehaviour->GetLengthLeftOnPath();

	if (fPathLength < dbg_fVerySmallPathDistance)
	{
		if (fdeltaMasterTurnCos < dbg_fVerySmallTurnCosDelta)
		{
			return true;
		}
		else
		{
			return false;
		}
	}
	else if (fPathLength < dbg_fSmallPathDistance)
	{
		if (fdeltaMasterTurnCos < dbg_fSmallTurnCosDelta)
		{
			return true;
		}
		else
		{
			return false;
		}
	}
	else if (fPathLength < dbg_fMediumPathDistance)
	{
		if (fDeltaMasterMovementSq > dbg_fMediumDeltaSquared || dbg_fMediumTurnCosDelta < dbg_fMediumTurnCosDelta)
		{
			return true;
		}
		else
		{
			return false;
		}
	}
	else if (fPathLength < dbg_fLargePathDistance)
	{
		if (fDeltaMasterMovementSq > dbg_fLargeDeltaSquared)
		{
			return true;
		}
		else
		{
			return false;
		}
	}
	else
	{
		if (fDeltaMasterMovementSq > dbg_fVeryLargeDeltaSquared)
		{
			return true;
		}
		else
		{
			return false;
		}
	}
}


bool AVP_Behaviour_FaceHugger_Pet::ShouldAttack(const AVP_Behaviour_FaceHugger_MoveToPrey* pxBehaviour) const
{
	const AVP_Brain_Actor_FaceHugger *pxBrain = GetFaceHuggerBrain();
	// if there is an attack
	if (pxBrain->IsTargetUnderAttack())
	{
		return false;
	}

	// else attackablity depend on the orientation of the target to me
	if (pxBrain->IsTargetFacingMe())
	{
		return pxBrain->ShouldAttackFromTheFront();
	}
	else
	{
		return pxBrain->ShouldAttackFromTheBack();
	}
}

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////
