//-------------------------------------------------------------------------------------------------
//	File	:	AVP_Behaviour_FaceHugger_StalkPrey.cpp
//	Created	:	02 / 07 / 07
//	Author	:	Gaz Thomas
//	Copyright (C) 2006 Rebellion, All Rights Reserved.
//-------------------------------------------------------------------------------------------------

//-------------------------------------------------------------------------------------------------
//	INCLUDES
//-------------------------------------------------------------------------------------------------
//	Core
#include "Core/Asura.h"
#include "Core/Asura_Maths.h"
//	This files header
#include "Behaviour\BehaviourTypes\FaceHugger\AVP_Behaviour_FaceHugger_StalkPrey.h"
// Asura
#ifndef ASURA_NO_DEBUG_OUTPUT
#include "Rendering/Asura_Font_System.h"	
#endif
//	AVP
#include "Actor/Vertebrates/Alien/AVP_FaceHugger_Description.h"
//	Axon
#include "Behaviour/BehaviourTypes/FaceHugger/AVP_Behaviour_FaceHugger.h"
#include "Behaviour/BehaviourTypes/AVP_Behaviour_Types.h"
#include "Core/BrainTypes/Axon_Project_Brain.h"
#include "Knowledge/SenseTypes/AVP_Sense_Vision.h"


//-------------------------------------------------------------------------------------------------
//	Function	:	AVP_Behaviour_FaceHugger_StalkPrey::AVP_Behaviour_FaceHugger_StalkPrey
//	Created		:	29 / 06 / 07
//	Author		:	Gaz Thomas
//-------------------------------------------------------------------------------------------------
AVP_Behaviour_FaceHugger_StalkPrey::AVP_Behaviour_FaceHugger_StalkPrey( Axon_Project_Brain* pxBrain )
: PARENT(pxBrain, AXON_BEHAVIOUR_TYPE_FACEHUGGER_STALK_PREY)
, m_uUpdateState(US_UNSET)
{	
}/// End of function 'AVP_Behaviour_FaceHugger_StalkPrey::AVP_Behaviour_FaceHugger_StalkPrey'. ///

//--------------------------------------------------------------------------------------
//	Function	:	AVP_Behaviour_FaceHugger_StalkPrey::~AVP_Behaviour_FaceHugger_StalkPrey
//	Created		:	29 / 06 / 07
//	Author		:	Gaz Thomas
//--------------------------------------------------------------------------------------
AVP_Behaviour_FaceHugger_StalkPrey::~AVP_Behaviour_FaceHugger_StalkPrey()
{
}/// End of function 'AVP_Behaviour_FaceHugger_StalkPrey::~AVP_Behaviour_FaceHugger_StalkPrey'. ///

//--------------------------------------------------------------------------------------
//	Function	:	AVP_Behaviour_FaceHugger_StalkPrey::Activate
//	Created		:	29 / 06 / 07
//	Author		:	Gaz Thomas
//--------------------------------------------------------------------------------------
bool AVP_Behaviour_FaceHugger_StalkPrey::Activate()
{
	bool bOk = PARENT::Activate();

	//	Set up the default distance we will try to find a position in front of the target
	m_fOffsetDistance = 5.0f;

	//	Set the update state
	SetUpdateState(US_FIND_ROUTE_TO_TARGET);

	return bOk;

}/// End of function 'AVP_Behaviour_FaceHugger_StalkPrey::Activate'. ///

//--------------------------------------------------------------------------------------
//	Function	:	AVP_Behaviour_FaceHugger_StalkPrey::Deactivate
//	Created		:	29 / 06 / 07
//	Author		:	Gaz Thomas
//--------------------------------------------------------------------------------------
void AVP_Behaviour_FaceHugger_StalkPrey::Deactivate()
{   
	PARENT::Deactivate();

}/// End of function 'AVP_Behaviour_FaceHugger_StalkPrey::Deactivate'. ///

//--------------------------------------------------------------------------------------
//	Function	:	AVP_Behaviour_FaceHugger_StalkPrey::PerFrameUpdate
//	Created		:	29 / 06 / 07
//	Author		:	Gaz Thomas
//--------------------------------------------------------------------------------------
void AVP_Behaviour_FaceHugger_StalkPrey::PerFrameUpdate(const float fTimeslice)
{
	//	Check we aren't in an error state, or that we haven't finished the behaviour
	if (IsInError() || IsFinished())
		return;

	switch ( m_uUpdateState )
	{
		case US_FIND_ROUTE_TO_TARGET:
		{
			FindRouteToTarget_PerFrameUpdate(fTimeslice);
			return;
		}
		case US_MOVE_TO_TARGET:
		{
			MoveToTarget_PerFrameUpdate(fTimeslice);
			return;
		}
		case US_UNSET:
		default:
		{
			SetInError();
			return;
		}
	}

}/// End of function 'AVP_Behaviour_FaceHugger_StalkPrey::PerFrameUpdate'. ///

//--------------------------------------------------------------------------------------
//	Function	:	AVP_Behaviour_FaceHugger_StalkPrey::FindRouteToTarget_PerFrameUpdate
//	Created		:	02 / 07 / 07
//	Author		:	Gaz Thomas
//--------------------------------------------------------------------------------------
void AVP_Behaviour_FaceHugger_StalkPrey::FindRouteToTarget_PerFrameUpdate(const float fTimeslice)
{
	AVP_Behaviour_FaceHugger* pxParentBehaviour = static_cast<AVP_Behaviour_FaceHugger*>(m_pxParentBehaviour);
	
	//	Validate the pointer
	if (pxParentBehaviour)
	{
		//	Check we have a target
		if (pxParentBehaviour->GetTargetGUID() != ASURA_GUID_UNREGISTERED)
		{
			//	Get The target actor's pointer
			AVP_ServerEntity_Actor* pxTarget	= AVP_ServerEntity_Actor::FindEntity(pxParentBehaviour->GetTargetGUID());
			
			//	Validate the pointer
			if (pxTarget)
			{
				//	Get The Target Actors Orientation
				Asura_Matrix_3x3		xTargetOri	= pxTarget->GetOrientation();

				//	The Face hug animation only works from the front, so we need to find a position to move to that's in front of the target
				Asura_Vector_3 xTargetForward = Asura_Vector_3(xTargetOri.e13, xTargetOri.e23, xTargetOri.e33);
				xTargetForward.Normalise();

				//	Find a position in front of the target
				m_xMoveToPos = pxTarget->GetPosition() + (xTargetForward * m_fOffsetDistance);

				//	Get a route to the target
				if (GetRouteTo(m_xMoveToPos))
				{
					// Decrease the distance in front of the target we want to move to
					m_fOffsetDistance -= 1.0f;

					//	don't let the offset distance go below zero
					if (m_fOffsetDistance <= 1.0f)
					{
						m_fOffsetDistance = 1.0f;
					}

					//	Change state!
					SetUpdateState(US_MOVE_TO_TARGET);
				}
			}
		}
		else
		{
			//	No target set finsihed so we can find a new one
			SetFinished();
		}
	}

}/// End of function 'AVP_Behaviour_FaceHugger_StalkPrey::FindRouteToTarget_PerFrameUpdate'. ///

//--------------------------------------------------------------------------------------
//	Function	:	AVP_Behaviour_FaceHugger_StalkPrey::MoveToTarget_PerFrameUpdate
//	Created		:	02 / 07 / 07
//	Author		:	Gaz Thomas
//--------------------------------------------------------------------------------------
void AVP_Behaviour_FaceHugger_StalkPrey::MoveToTarget_PerFrameUpdate(const float fTimeslice)
{
	AVP_Behaviour_FaceHugger* pxParentBehaviour = static_cast<AVP_Behaviour_FaceHugger*>(m_pxParentBehaviour);

	//	Validate the pointer
	if (pxParentBehaviour)
	{
		//	Check we have a target
		if (pxParentBehaviour->GetTargetGUID() != ASURA_GUID_UNREGISTERED)
		{
			// Orient Along Route and play a movement animation
			m_pxBrain->RequestAnimation(AVP_FaceHugger_Description::FACEHUGGER_ANIM_RUN);
			OrientAlongRoute(fTimeslice);

			// Check if we are at the target position
			if (AtTargetPosition(m_pxBrain->GetActor()->GetPosition(), m_xMoveToPos))
			{
				//	if we are lets find a closer position in front of the target
				SetUpdateState(US_FIND_ROUTE_TO_TARGET);
			}
		}
		else
		{
			SetFinished();
		}
	}

}/// End of function 'AVP_Behaviour_FaceHugger_StalkPrey::MoveToTarget_PreFrameUpdate'. ///

#ifndef ASURA_NO_DEBUG_OUTPUT
static const char* s_aszStateStrings[] =
{
	"CHILD STATE - UNSET",
	"CHILD STATE - FIND ROUTE TO TARGET",
	"CHILD STATE - MOVE TO TARGET",
};

//--------------------------------------------------------------------------------------
//	Function	:	AVP_Behaviour_FaceHugger_StalkPrey::RenderDebugInfo
//	Created		:	02 / 07 / 07
//	Author		:	Gaz Thomas
//--------------------------------------------------------------------------------------
void AVP_Behaviour_FaceHugger_StalkPrey::RenderDebugInfo()
{
	PARENT::RenderDebugInfo();

	Asura_Font_System::Print
	(
		m_pxBrain->GetDebugOutputPos().x, 
		m_pxBrain->GetDebugOutputPos().y, 
		0.5f, 
		0xff0000ff,
		ASURA_TEXTFORMAT_CENTREJUSTIFIED | ASURA_TEXTFORMAT_FORMATSTRING, 
		"%s", s_aszStateStrings[ m_uUpdateState ] 
	);
	m_pxBrain->GetDebugOutputPos().y += 10.0f;

}/// End of function 'AVP_Behaviour_FaceHugger_StalkPrey::RenderDebugInfo'. ///
#endif
