//---------------------------------------------------------------------------
// File			: AVP_Behaviour_Civilian_alert.h
// Created		: 04 / 03 / 2008
// Created By	: Gareth Thomas
// Copyright (C) 2007 Rebellion. All Rights Reserved.
//---------------------------------------------------------------------------

// BEGIN FILE WRAPPER ///////////////////////////////////////
#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif
#ifndef __AVP_BEHAVIOUR_CIVILIAN_ATTACK_H__
#define __AVP_BEHAVIOUR_CIVILIAN_ATTACK_H__ 1
// END FILE WRAPPER ///////////////////////////////////////////

// INCLUDES ///////////////////////////////////////////////////////////
#include "Behaviour/Axon_Project_Behaviour.h"

// AVP Axon
#include "Core/BrainTypes/AVP_Brain_Actor_Civilian.h"

// FORWARD DECLARATIONS ///////////////////////////////////
class AVP_ServerActor_Civilian;
class AVP_Knowledge_System_Actor;
class AVP_Tactical_System;

//---------------------------------------------------------------------------
// Class		: AVP_Behaviour_Civilian_Attack
//---------------------------------------------------------------------------
class AVP_Behaviour_Civilian_Attack : public Axon_Project_Behaviour
{
	typedef Axon_Project_Behaviour PARENT;

public:

	// Attack Update State Enum
	enum AttackUpdateState
	{
		UNSET,
		CHECK_TO_SHOOT,
		SHOOT,
		WAIT_TO_SHOOT,
		RELOAD,
		MAX_STATES
	};

	// Construction / Deconstruction
	AVP_Behaviour_Civilian_Attack						( AVP_Brain_Actor_Civilian* pxBrain );
	virtual ~AVP_Behaviour_Civilian_Attack				( void ) {}

	// Public Member Virtual Functions...
	virtual bool 					Activate			( void );
	virtual void 					Deactivate			( void );
	virtual void					PerFrameUpdate		( const float fTimeslice );

#ifndef ASURA_NO_DEBUG_OUTPUT
	virtual void					RenderDebugInfo		( void );
#endif

	// Public Member Functions...
	AttackUpdateState				GetUpdateState		( void ) const					{ return m_eUpdateState; }
	AVP_Brain_Actor_Civilian*		GetCivilianBrain	( void )							{ return m_pxCivilianBrain; }
	AVP_ServerActor_Civilian*		GetOurActor			( void )							{ return m_pxCivilianBrain->GetCivilianActor();	}
	AVP_Knowledge_System_Actor*		GetKnowledgeSystem	( void )							{ return m_pxCivilianBrain->GetKnowledgeSystem(); }
	AVP_Tactical_System*			GetTacticalSystem	( void )							{ return m_pxCivilianBrain->GetTacticalSystem(); }

protected:

	void							SetUpdateState				( AttackUpdateState eState )				{ m_eUpdateState = eState; }
	void							CreateMovementBehaviour		( AVP_ServerEntity_Actor* pxAttackActor );
	void							SwitchToState				( AttackUpdateState eState );
	bool							CheckForFire				( void );
	float							GetWeaponAmmo				( void );
	bool							AllyToCloseToFire			( const Asura_Vector_3 &xStart, const Asura_Vector_3& xEnd );
	bool							GetWeaponMuzzlePositionAndOrientation( Asura_Vector_3* xPosition, Asura_Matrix_3x3* xOrientation );
	bool							IsWeaponPositionInsideFriendlyBoundingBox( const Asura_Vector_3& xTestPos );

	void							Shoot(void);
	void							StopShooting(void);
	float							GetWeaponsMaxRange(void);

private:

	// Private Member Variables...
	AttackUpdateState				m_eUpdateState;
	AVP_Brain_Actor_Civilian*		m_pxCivilianBrain;
	float							m_fShootTimer;
	float							m_fDontShootTimer;
	Asura_Guid						m_uEnemyGuid;
	bool							m_bReloading;

	enum { CURRENT_CHUNK_VERSION = 1 };
};

#endif //__AVP_BEHAVIOUR_CIVILIAN_ATTACK_H__

/// End Of File 'AVP_Behaviour_Civilian_Attack.h'
