//////////////////////////////////////////////////////////////////////////////
//
// AVP_Behaviour_GS2Node_Marine_Combat.h
//
// Copyright (C) 2008 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// Core
#include "Core/Asura.h"

// This files header
#include "Behaviour/BehaviourTypes/Human/Marine/AVP_Behaviour_GS2Spline_Marine_Combat.h"

// Asura Includes
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "Network/Asura_Message.h"

// AVP
#include "Actor/AVP_Actor_AnimStateData.h"
#include "Actor/Vertebrates/Human/AVP_Human_AIAnimStateData.h"
#include "Actor/Vertebrates/Human/AVP_Human_AIAnimStateMachine.h"
#include "Entity/Locations/OrientedNode/GamesceneNode/AVP_ServerNode_GS2Node.h"
#include "Entity/Locations/Spline/GamesceneSpline/AVP_ServerNode_GS2Spline.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Human/AVP_Messages_Human.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Human/AVP_ServerActor_Human.h"
#include "Inventory/AVP_Inventory_Group.h"
#include "Inventory/AVP_Inventory_Item.h"
#include "Inventory/AVP_Inventory_System.h"
#include "Inventory/AVP_Inventory_HashIDs.h"
#include "Moves/Vertebrates/Human/AVP_Human_MovesGamescene.h"

// AVP Axon
#include "Behaviour/AVP_BehaviourParams.h"
#include "Behaviour/BehaviourTypes/AVP_Behaviour_Types.h"
#include "Behaviour/BehaviourTypes/Axon_Behaviour_GS2Actor.h"
#include "Knowledge/AVP_Knowledge_System_Actor.h"
#include "Tactical/AVP_Tactical_System.h"
#include "Behaviour/BehaviourParamsTypes/Axon_BehaviourParams_GS2Node.h"



//-------------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------------
AVP_Behaviour_GS2Spline_Marine_Combat::AVP_Behaviour_GS2Spline_Marine_Combat( AVP_Brain_Actor_Marine* pxBrain )
: PARENT( AVP_BEHAVIOUR_TYPE_GS2_MARINE_SPLINE_COMBAT, pxBrain )
, m_uGS2NodeGuid( ASURA_GUID_UNREGISTERED )
, m_bWaiting( false )
, m_eUpdateState( LOOK_N_AIM_DOWN_ROUTE )
, m_bStoppingAtDestination( true )
{

}

//-------------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------------
bool AVP_Behaviour_GS2Spline_Marine_Combat::Activate( void )
{
	AVP_Brain_Actor_Marine* pxMarineBrain = GetMarineBrain();

	if ( pxMarineBrain )
	{
		AVP_ServerNode_GS2Spline* pxGS2Spline = static_cast<AVP_ServerNode_GS2Spline*>( AVP_ServerNode_GS2Spline::FindEntity( GetGSActorBehaviour()->GetCurrentEntityGuid() ) );

		if ( pxGS2Spline )
		{
			m_uGS2NodeGuid = pxGS2Spline->GetTargetGuid();
		}

		AVP_Actor_PhysicsObjectHandler* pxPhysicsHandler = pxMarineBrain->GetHumanActor()->GetPhysicsObjectHandler();

		if ( pxPhysicsHandler )
		{
			pxPhysicsHandler->DisableExternalControlPhysics( AVP_PhysicsObjectHandler::ECREQ_GAMESCENE );
		}

		CreateChildWithType( AVP_BEHAVIOUR_TYPE_MARINE_TRAVEL );
		CreateChildWithType( AVP_BEHAVIOUR_TYPE_MARINE_WEAPON );

		AVP_Tactical_System*		pxTacticalSystem = pxMarineBrain->GetTacticalSystem();
		AVP_ServerNode_GS2Node*		pxGS2Node = AVP_ServerNode_GS2Node::FindEntity( GetNextGamesceneEntityGuid() );

		pxTacticalSystem->SetDestination( pxGS2Node->GetPosition() );
		pxTacticalSystem->OverrideStartMovementDelay();
		pxTacticalSystem->OverrideSlowDownForThisDestination();
		pxTacticalSystem->SetIsOnGameScene( false );
		pxTacticalSystem->SetMovementSpeed( pxMarineBrain->GetHumanActor()->GetGS2MoveSpeed() );

		if ( pxGS2Node )
		{
			if ( pxGS2Node->GetBehaviourParams() ) 
			{
				if (pxGS2Node->GetBehaviourParams()->GetBehaviourType() == AXON_BEHAVIOUR_TYPE_GS2_NODE )
				{
					const Axon_BehaviourParams_GS2Node* pxGS2BehaviourParams = static_cast< const Axon_BehaviourParams_GS2Node*>(pxGS2Node->GetBehaviourParams());

					if ( pxGS2BehaviourParams && pxGS2BehaviourParams->m_bStopHere == false )
					{
						m_bStoppingAtDestination = false;
					}
				}
			}
		}


		return true;
	}

	return false;
}

//-------------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------------
void AVP_Behaviour_GS2Spline_Marine_Combat::Deactivate( void )
{
	DestroyAllChildren();

	AVP_Brain_Actor_Marine* pxMarineBrain = GetMarineBrain();

	if ( pxMarineBrain && pxMarineBrain->IsActive() )
	{
		AVP_Tactical_System* pxTacticalSystem = pxMarineBrain->GetTacticalSystem();

		if ( !pxMarineBrain->DoesHaveAlert() && !pxMarineBrain->DoesHaveThreat() )
		{
			pxTacticalSystem->SetIsOnCombatGS2( false );
		}
		else
		{
			pxTacticalSystem->SetIsOnCombatGS2( true );
		}
	}

	PARENT::Deactivate();
}

//-------------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------------
void AVP_Behaviour_GS2Spline_Marine_Combat::WriteToChunkStream(Asura_Chunk_Stream& xStream) const
{
	xStream << static_cast<int>(CURRENT_CHUNK_VERSION);

	PARENT::WriteToChunkStream(xStream);
}

//-------------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------------
void AVP_Behaviour_GS2Spline_Marine_Combat::ReadFromChunkStream(Asura_Chunk_Stream& xStream)
{
	int iVersion;
	xStream >> iVersion;

	PARENT::ReadFromChunkStream(xStream);
}

//-------------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------------
bool AVP_Behaviour_GS2Spline_Marine_Combat::HasNextGuidControl()
{
	return true;
}


//-------------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------------
Asura_Guid AVP_Behaviour_GS2Spline_Marine_Combat::GetNextGamesceneEntityGuid()
{
	return m_uGS2NodeGuid;
}

//-------------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------------
float AVP_Behaviour_GS2Spline_Marine_Combat::GetDistanceToStop(void)
{
	AVP_ServerActor_Human&			xOurActor			= *GetMarineBrain()->GetHumanActor();
	AVP_Tactical_System*			pxTactical			= GetMarineBrain()->GetTacticalSystem();
	AVP_Human_AIAnimStateMachine*	pxHumanStateMachine	= static_cast<AVP_Human_AIAnimStateMachine*>(xOurActor.GetAIAnimStateMachine());

	if (pxTactical->ShouldMoveCrouched())
	{
		return pxHumanStateMachine->GetDistanceRequired(AVP_Human_AIAnimStateMachine::HUMAN_STATE_CROUCH_MOVE, AVP_Human_AIAnimStateMachine::HUMAN_STATE_CROUCH_IDLE);
	}
	else
	{
		switch(pxTactical->GetMovementSpeed())
		{
			case AVP_Navigation::MM_SLOW:
			{
				return pxHumanStateMachine->GetDistanceRequired(AVP_Human_AIAnimStateMachine::VERTEBRATE_STATE_MOVE_SLOW, AVP_Human_AIAnimStateMachine::ACTOR_STATE_IDLE);
			}

			case AVP_Navigation::MM_FAST:
			{
				return pxHumanStateMachine->GetDistanceRequired(AVP_Human_AIAnimStateMachine::VERTEBRATE_STATE_MOVE_FAST, AVP_Human_AIAnimStateMachine::ACTOR_STATE_IDLE);
			}

			default:
			case AVP_Navigation::MM_MEDIUM:
			{
				return pxHumanStateMachine->GetDistanceRequired(AVP_Human_AIAnimStateMachine::VERTEBRATE_STATE_MOVE_MEDIUM, AVP_Human_AIAnimStateMachine::ACTOR_STATE_IDLE);
			}
		}
	}

}

//-------------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------------
void AVP_Behaviour_GS2Spline_Marine_Combat::SwitchToState( UpdateState eState )
{
	m_eUpdateState = eState;
}

//-------------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------------
void AVP_Behaviour_GS2Spline_Marine_Combat::PerFrameUpdate( const float fTimeslice )
{
	AVP_Brain_Actor_Marine*		pxMarineBrain		= GetMarineBrain();
	AVP_ServerActor_Human*		pxOurActor			= pxMarineBrain->GetHumanActor();
	AVP_Tactical_System*		pxTacticalSystem	= pxMarineBrain->GetTacticalSystem();
	AVP_ServerNode_GS2Node*		pxGS2Node			= AVP_ServerNode_GS2Node::FindEntity( m_uGS2NodeGuid );

	if ( !pxMarineBrain || !pxOurActor || !pxTacticalSystem || !pxGS2Node || pxMarineBrain->GetKnowledgeSystem()->DoesHaveReconKnowledge() )
	{
		SetFinished();
		return;
	}

	PARENT::PerFrameUpdate( fTimeslice );

	switch ( m_eUpdateState )
	{
		//-------------------------------------------------------------------

		case LOOK_N_AIM_DOWN_ROUTE:
		{
			if ( pxMarineBrain->DoesHaveThreat() )
			{
				SwitchToState( LOOK_N_AIM_AT_THREAT );
			}
			else if ( pxMarineBrain->DoesHaveAlert() )
			{
				SwitchToState( LOOK_N_AIM_AT_ALERT );
			}
			else
			{
				pxTacticalSystem->LetRouteSetLookAt( true );
			}

		}break;

		//-------------------------------------------------------------------

		case LOOK_N_AIM_AT_THREAT:
		{
			if ( !pxMarineBrain->DoesHaveThreat() && pxMarineBrain->DoesHaveAlert() )
			{
				SwitchToState( LOOK_N_AIM_AT_ALERT );
			}
			else if ( !pxMarineBrain->DoesHaveThreat() && !pxMarineBrain->DoesHaveAlert())
			{
				SwitchToState( LOOK_N_AIM_DOWN_ROUTE );
			}
			else
			{
				pxMarineBrain->LookAtThreat();
				pxMarineBrain->AimAtThreat();
				pxTacticalSystem->LetRouteSetLookAt( false );
			}

		}break;

		//-------------------------------------------------------------------

		case LOOK_N_AIM_AT_ALERT:
		{
			if ( pxMarineBrain->DoesHaveThreat() )
			{
				SwitchToState( LOOK_N_AIM_AT_THREAT );
			}
			else if ( !pxMarineBrain->DoesHaveAlert())
			{
				SwitchToState( LOOK_N_AIM_DOWN_ROUTE );
			}
			else
			{
				Asura_Vector_3 xAlertPos( pxMarineBrain->GetKnowledgeSystem()->GetBestAlertKnowledge().m_xPosition );
				pxMarineBrain->AimAtPosition( xAlertPos );
				pxMarineBrain->SetLookAtPosition( xAlertPos );
				pxTacticalSystem->LetRouteSetLookAt( false );
			}

		}break;

		//-------------------------------------------------------------------

		case FINISHING:
		default: {}
		break;
	}

	if ( m_bStoppingAtDestination && m_eUpdateState != FINISHING )
	{
		float fDistanceSq = ( pxGS2Node->GetPosition() - pxOurActor->GetPosition() ).MagnitudeSquared();
		float fStopDistSq = 0.0f;
		float fTurnDistanceSq = 0.0f;
		float fDistanceToStop = GetDistanceToStop();

		if ( fDistanceToStop == 0.0f )
		{
			fDistanceToStop = 2.6f;
		}

		fStopDistSq = fDistanceToStop * fDistanceToStop;
		fTurnDistanceSq = ( fDistanceToStop + 2.0f ) * ( fDistanceToStop + 2.0f );

		if ( fDistanceSq < fStopDistSq )
		{
			SendMountGamesceneMessage();
			SwitchToState( FINISHING );
		}
		// turn to face nodes orientation if we have no threta or alert.
		else if ( fDistanceSq < fTurnDistanceSq && !pxMarineBrain->DoesHaveThreat() && !pxMarineBrain->DoesHaveAlert() )
		{
			Asura_Matrix_3x3 xNodeOri; pxGS2Node->GetOrientation( xNodeOri );
			Asura_Vector_3 xNodeFwd( xNodeOri.e13, xNodeOri.e23, xNodeOri.e33 );

			pxMarineBrain->GetTacticalSystem()->LetRouteSetLookAt( false );
			GetMarineBrain()->SetAimDirection( xNodeFwd );
			GetMarineBrain()->SetLookAtPosition( pxOurActor->GetPosition() + xNodeFwd );
		}

		pxTacticalSystem->SetShouldStopAtDestination( true );
	}
	else if ( m_eUpdateState != FINISHING )
	{
		pxTacticalSystem->SetShouldStopAtDestination( false );

		if ( pxTacticalSystem->HasArrivedAtDestination() )
		{
			SetFinished();
		}
	}

	pxTacticalSystem->SetMovementSpeed( pxOurActor->GetGS2MoveSpeed() );
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine :: SendMountGamesceneMessage
//---------------------------------------------------------------------------
void AVP_Behaviour_GS2Spline_Marine_Combat::SendMountGamesceneMessage()
{
	Asura_Message* pxMessage = Asura_Network::CreateMessage( ASURA_MSG_MOVE_START, sizeof( AVP_Message_Human_MountGamescene ) );

	if ( pxMessage )
	{
		AVP_Message_Human_MountGamescene*	pxData		= static_cast< AVP_Message_Human_MountGamescene* >( pxMessage->GetData() );
		AVP_ServerNode_GS2Node*				pxGS2Node	= AVP_ServerNode_GS2Node::FindEntity( m_uGS2NodeGuid );
		AVP_ServerActor_Human*				pxOurActor	= GetMarineBrain()->GetHumanActor();

		if ( pxGS2Node )
		{
			Asura_Matrix_3x3 xNodeOri;
			pxGS2Node->GetOrientation(xNodeOri);

			pxMessage->SetFromGuid( GetMarineBrain()->GetOwnerGuid() );
			pxMessage->SetToGuid( GetMarineBrain()->GetOwnerGuid() );
			pxData->ClearFlags();
			pxData->SetMoveID( AVP_Move_Human_MountGamescene::GetStaticHash() );
			pxData->SetDestinationPosition( pxGS2Node->GetPosition() );

			if ( GetMarineBrain()->DoesHaveAlert() || GetMarineBrain()->DoesHaveThreat() )
			{
				pxData->SetDestinationOrientation( pxOurActor->GetOrientation() );
			}
			else
			{
				pxData->SetDestinationOrientation( xNodeOri );
			}
			
			pxData->SetAnimBlendTemplateBlueprintHash( GetMountGamesceneAnimBlendTemplateBlueprintHash() );

			AVP_ServerActor_Human* xOurActor = GetMarineBrain()->GetHumanActor();

			if ( xOurActor->GetAIAnimStateMachine() )
			{
				xOurActor->GetAIAnimStateMachine()->RequestInMove( pxMessage, AVP_Human_AIAnimStateMachine::ACTOR_STATE_IDLE );
				GetMarineBrain()->GetTacticalSystem()->StopMoving();
			}
			else
			{
				Asura_Network::DestroyMessage(pxMessage);
			}
		}
		else
		{
			Asura_Network::DestroyMessage(pxMessage);
		}
	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine :: GetMountGamesceneAnimation
//---------------------------------------------------------------------------
Asura_Hash_ID AVP_Behaviour_GS2Spline_Marine_Combat::GetMountGamesceneAnimBlendTemplateBlueprintHash(void)
{
	const AVP_ServerActor_Human* pxHuman = GetMarineBrain()->GetHumanActor();

	const AVP_Inventory_System* pxInvSys = pxHuman->GetInventorySystem();
	const AVP_Inventory_Group* pxWeaponGroup = pxInvSys != NULL ? pxInvSys->GetGroup( uAVP_INVENTORY_GROUPHASH_WEAPON ) : NULL;
	Asura_Assert( pxWeaponGroup != NULL, "Unable to get wepaon group" );
	if ( pxWeaponGroup != NULL )
	{
		const AVP_Inventory_Item* pxItem = pxWeaponGroup->GetCurrentInventoryItem();
		if ( pxItem )
		{
			if ( GetMarineBrain()->GetTacticalSystem()->GetMovementSpeed() == AVP_Navigation::MM_SLOW )
			{
				const AVP_Actor_AnimStateData* pxStateData = AVP_Human_AIAnimStateData::GetAnimStateData( pxHuman->GetAnimStateBlueprintType(), pxItem->GetBlueprintHash(), AVP_Human_AIAnimStateMachine::VERTEBRATE_STATE_MOVE_SLOW_TO_IDLE, pxHuman->IsAiming(), pxHuman->GetCoverType() );
				if ( pxStateData )
				{
					return pxStateData->GetAnimBlendTemplateBlueprintHash();
				}
			}
			else
			{
				const AVP_Actor_AnimStateData* pxStateData = AVP_Human_AIAnimStateData::GetAnimStateData( pxHuman->GetAnimStateBlueprintType(), pxItem->GetBlueprintHash(), AVP_Human_AIAnimStateMachine::VERTEBRATE_STATE_MOVE_MEDIUM_TO_IDLE, pxHuman->IsAiming(), pxHuman->GetCoverType() );
				if ( pxStateData )
				{
					return pxStateData->GetAnimBlendTemplateBlueprintHash();
				}
			}
		}
	}

	// else
	return ASURA_HASH_ID_UNSET;
}

//-------------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------------
void AVP_Behaviour_GS2Spline_Marine_Combat::HandleParams( const Axon_BehaviourParams* pxParams )
{
	PARENT::HandleParams( pxParams );

	if ( pxParams->GetParamsType() == AVP_BP_TYPE_MARINE_SPLINE_COMBAT )
	{
		//const AVP_BehaviourParams_GS2Spline_Marine_Combat* pxMrnCmbtParams = static_cast< const AVP_BehaviourParams_GS2Spline_Marine_Combat* >( pxParams );
	}
}

//-------------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------------
void AVP_Behaviour_GS2Spline_Marine_Combat::LeaveNode(void)
{

}

//-------------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------------
bool AVP_Behaviour_GS2Spline_Marine_Combat::HandleMessage( const Asura_Message* pxMessage )
{
	switch ( pxMessage->GetMsgID() )
	{
		case ASURA_MSG_AXON_GS2_LEAVE_NODE:
		{
			LeaveNode();
			return true;
		}

		case ASURA_MSG_MOVECHAINCOMPLETE:
		{
			const Asura_Msg_MoveChainComplete* pxData = static_cast< const Asura_Msg_MoveChainComplete* >( pxMessage->GetData() );

			if ( pxData && pxData->GetInitialMoveID() == AVP_Move_Human_MountGamescene::GetStaticHash() )
			{
				SetFinished();
			}

			return true;
		}

		default:
		{
			return PARENT::HandleMessage(pxMessage);
		}
	}
}

//-------------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------------
AVP_Brain_Actor_Marine* AVP_Behaviour_GS2Spline_Marine_Combat::GetMarineBrain( void ) const
{
	return static_cast< AVP_Brain_Actor_Marine* >( GetBrain() );
}

//-------------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------------
#ifndef ASURA_NO_DEBUG_OUTPUT
void AVP_Behaviour_GS2Spline_Marine_Combat::RenderDebugInfo( void )
{
	PARENT::RenderDebugInfo();

	Asura_Font_System::Print( GetDbgOutputPosX(), GetDbgOutputPosY(), 0.5f, 0xffffffff, ASURA_TEXTFORMAT_CENTREJUSTIFIED, "AVP MARINE SPLINE COMBAT" );

	GetMarineBrain()->GetDebugOutputPos().y += 10.0f;
}
#endif

//*********************************************************************************************************************
//*********************************************************************************************************************
//*********************************************************************************************************************
//*********************************************************************************************************************
//*********************************************************************************************************************
//*********************************************************************************************************************
//*********************************************************************************************************************
//*********************************************************************************************************************

//////////////////////////////////////////////////////////////////////////////
AVP_BehaviourParams_GS2Spline_Marine_Combat::AVP_BehaviourParams_GS2Spline_Marine_Combat( const u_int uBehaviourType )
: PARENT( AVP_BP_TYPE_MARINE_SPLINE_COMBAT, uBehaviourType )
{

}

//////////////////////////////////////////////////////////////////////////////
bool AVP_BehaviourParams_GS2Spline_Marine_Combat::IsBaseOrDerivativeOf( const u_int uParamsType ) const
{
	return ( uParamsType == AVP_BP_TYPE_MARINE_SPLINE_COMBAT ) ? true : PARENT::IsBaseOrDerivativeOf( uParamsType );
}

//////////////////////////////////////////////////////////////////////////////
void AVP_BehaviourParams_GS2Spline_Marine_Combat::ReadFromChunkStream( Asura_Chunk_Stream& xStream )
{
	int iVersion;
	xStream >> iVersion;

	PARENT::ReadFromChunkStream( xStream );
}

//////////////////////////////////////////////////////////////////////////////
void AVP_BehaviourParams_GS2Spline_Marine_Combat::WriteToChunkStream( Asura_Chunk_Stream& xStream ) const
{
	xStream << CURRENT_CHUNK_VERSION;

	PARENT::WriteToChunkStream( xStream );
}

#ifdef ASURA_TOOLS
//////////////////////////////////////////////////////////////////////////////
void AVP_BehaviourParams_GS2Spline_Marine_Combat::DoAddOverlayControls( Asura_Tabs_Dialog* pxTabsDialog, Asura_Vector_2& xOffset )
{
	AddOverlayControls( pxTabsDialog, xOffset );
}

//////////////////////////////////////////////////////////////////////////////
void AVP_BehaviourParams_GS2Spline_Marine_Combat::AddOverlayControls(Asura_Tabs_Dialog* pxTabsDialog, Asura_Vector_2& xOffset)
{
	PARENT::AddOverlayControls( pxTabsDialog, xOffset );
}

//////////////////////////////////////////////////////////////////////////////
void AVP_BehaviourParams_GS2Spline_Marine_Combat::UpdateWhileSelected()
{
	PARENT::UpdateWhileSelected();
}

//////////////////////////////////////////////////////////////////////////////
bool AVP_BehaviourParams_GS2Spline_Marine_Combat::GetParameter( const Asura_Hash_ID uParamID, Tools_Parameter* pxParam ) const
{
	return PARENT::GetParameter( uParamID, pxParam );
}

//////////////////////////////////////////////////////////////////////////////
bool AVP_BehaviourParams_GS2Spline_Marine_Combat::SetParameter( const Asura_Hash_ID uParamID, const Tools_Parameter* pxParam )
{
	return PARENT::SetParameter( uParamID, pxParam );
}

//////////////////////////////////////////////////////////////////////////////
Axon_BehaviourParams* AVP_BehaviourParams_GS2Spline_Marine_Combat::Clone() const
{
	return new AVP_BehaviourParams_GS2Spline_Marine_Combat( *this );
}
#endif
