//---------------------------------------------------------------------------
// File			: AVP_Behaviour_Marine.cpp
// Created		: 29 / 10 / 07
// Created By	: Gareth Thomas
// Copyright (C) 2007 Rebellion. All Rights Reserved.
//---------------------------------------------------------------------------

// INCLUDES ///////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"

// header
#include "Behaviour/BehaviourTypes/Human/Marine/AVP_Behaviour_Marine.h"

// asura
#include "Blueprint/Asura_Blueprint_System.h"
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "Network/Asura_Message.h"
#ifndef ASURA_NO_DEBUG_OUTPUT
#include "Rendering/Asura_Font_System.h"
#include "Debugging/Asura_DebugLineRender.h"
#endif

// axon
#include "Knowledge/Axon_Knowledge.h"

// AVP
#include "Actor/Vertebrates/Human/AVP_Human_AIAnimStateData.h"
#include "Actor/Vertebrates/Human/AVP_Human_AIAnimStateMachine.h"
#include "Actor/Vertebrates/AVP_Vertebrate_MeleeCounter_System.h"
#include "Entity/AVP_Entity_Classification.h"
#include "Entity/Controller/AVP_Controller_Messages.h"
#include "Entity/Controller/AVP_ServerEntity_Squad_Marine.h"
#include "Entity/GamePlayVolume/AVP_ServerEntity_GamePlayVolume.h"
#include "Entity/Locations/Objective/Marine/AVP_ServerEntity_Objective_Marine.h"
#include "Entity/Locations/OrientedNode/CoverpointNode/Asura_ServerNode_Coverpoint.h"
#include "Entity/Locations/OrientedNode/GamesceneNode/AVP_ServerNode_GS2Node.h"
#include "Entity/PhysicalObject/Actor/AVP_ServerEntity_Actor.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Human/Android/AVP_ServerActor_Android.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Human/AVP_Messages_Human.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Human/AVP_ServerActor_Human.h"
#include "Entity/PlayerInfo/AVP_ServerEntity_PlayerInfo.h"
#include "Entity/AVP_Messages.h"
#include "FacialAnimation/Asura_FacialAnimation_AnimationHandler.h"
#include "FacialAnimation/Asura_FacialAnimation_System.h"
#include "Inventory/AVP_Inventory_Group.h"
#include "Inventory/AVP_Inventory_HashIDs.h"
#include "Inventory/AVP_Inventory_Item.h"
#include "Inventory/AVP_Inventory_System.h"
#include "Inventory/Items/Weapons/AVP_InventoryWeapon_RangedWeapon.h"
#include "Moves/AVP_Movement_System.h"
#include "Moves/Vertebrates/Human/AVP_Human_MovesGamescene.h"

// AVP axon
#include "Behaviour/BehaviourTypes/AVP_Behaviour_Types.h"
#include "Behaviour/BehaviourTypes/Human/AVP_Behaviour_Human_Movement.h"
#include "Behaviour/BehaviourTypes/Human/Marine/AVP_Behaviour_Marine_Alert.h"
#include "Behaviour/BehaviourTypes/Human/Marine/AVP_Behaviour_Marine_Attack.h"
#include "Behaviour/BehaviourTypes/Human/Marine/AVP_Behaviour_Marine_MeleePlayer.h"
#include "Behaviour/BehaviourTypes/Human/Marine/AVP_Behaviour_Marine_Patrol.h"
#include "Behaviour/BehaviourTypes/Human/Marine/AVP_Behaviour_Marine_SquadMember.h"
#include "Behaviour/BehaviourTypes/Human/Marine/AVP_Behaviour_Marine_Travel.h"
#include "Behaviour/BehaviourTypes/Human/Marine/AVP_Behaviour_Marine_Weapon.h"
#include "Core/AVP_Waypoint_Manager.h"
#include "Core/BrainTypes/AVP_Brain_Actor_Marine.h"
#include "Knowledge/AVP_Knowledge_System_Actor.h"
#include "Tactical/AVP_Tactical_System.h"
#include "Knowledge/SenseTypes/AVP_Sense_Vision_Actor.h"

//---------------------------------------------------------------------------
//***************************************************************************
//---------------------------------------------------------------------------
// Class		: AVP_Behaviour_Marine
// Description	: Function Implementations for the AVP_Beahviour_Marine class
//---------------------------------------------------------------------------
//***************************************************************************
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine :: Constructor
//---------------------------------------------------------------------------
AVP_Behaviour_Marine::AVP_Behaviour_Marine(AVP_Brain_Actor_Marine* pxBrain)
: PARENT(pxBrain, AVP_BEHAVIOUR_TYPE_MARINE)
, m_eUpdateState(IDLE)
, m_pxMarineBrain(pxBrain)
, m_bHaveOrders(false)
, m_bHaveCountered(false)
, m_fCounterTimer(0.0f)
, m_fMeleeTimer( 0.0f )
, m_uFaceType( 0 )
, m_iLastAlertLevel( -1 )
, m_bFaceTypeSet( false )
, m_fFaceTimer( 0.0f )
, m_pxTravellingBehaviour(NULL)
, m_pxWeaponBehaviour(NULL)
{

}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine :: Activate
//---------------------------------------------------------------------------
bool AVP_Behaviour_Marine::Activate()
{
	const bool bParentOk = PARENT::Activate();

	if ( !GetMarineBrain()->GetHumanActor()->GetUseTestBehaviour() ) 
	{
		CreateTravellingBehaviour();
		CreateWeaponBehaviour();
	}

	AVP_Human_AIAnimStateMachine* pxStateMachine = (AVP_Human_AIAnimStateMachine*)GetMarineBrain()->GetHumanActor()->GetAIAnimStateMachine();

	if (pxStateMachine)
	{
		pxStateMachine->RequestIdle();
	}

	return bParentOk;
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine :: PostLoadInitialise
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine::PostLoadInitialise()
{
// 	if ( GetMarineBrain()->GetHumanActor()->IsEntityActive() )
// 	{
// 		const AVP_Navigation::NavData& xNavData = GetMarineBrain()->GetNavData();
// 
// 		if ( xNavData.m_usCurrentWPID != ASURA_INVALID_WAYPOINT_ID )
// 		{
// 			Asura_Vector_3		xLookDirection( Asura_Maths::ZeroVector3 );
// 			Asura_Matrix_3x3	xOri;
// 
// 			AVP_Waypoint_Manager::GetMaxLOFDirection(xNavData.m_usCurrentWPID, xLookDirection);
// 
// 			Asura_Maths::OrientationFromDirection(xOri, xLookDirection);
// 
// 			GetMarineBrain()->GetHumanActor()->TeleportTo( GetMarineBrain()->GetHumanActor()->GetPosition(), xOri ,true );
// 		}
// 	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine :: Deactivate
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine::Deactivate()
{
	DestroyAllChildren( false );

	PARENT::Deactivate();
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine :: Pause
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine::Pause()
{
	PARENT::Pause();
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine :: Unpause
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine::Unpause()
{
	PARENT::Unpause();
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine :: SetFacialExpression
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine::SetFacialExpression( const float fTimeslice )
{
	AVP_ServerActor_Human*	pxOurActor = GetMarineBrain()->GetHumanActor();

	Asura_FacialAnimation_AnimationHandler* pxFacialAnimHandler = Asura_FacialAnimation_System::FindAnimationHandler( pxOurActor->GetGuid() );

	if ( pxFacialAnimHandler )
	{
		AVP_ServerActor_Human::ExpressionType uFaceExpression = static_cast< AVP_ServerActor_Human::ExpressionType >( GetMarineBrain()->GetKnowledgeSystem()->GetAlertLevel() );

		m_fFaceTimer += fTimeslice;

		if ( m_fFaceTimer > 10.0f )
		{
			m_fFaceTimer = 0.0f;
		}

		if ( !m_bFaceTypeSet || m_iLastAlertLevel == -1 || m_iLastAlertLevel != (int)uFaceExpression || m_fFaceTimer == 0.0f )
		{
			if ( IsAllowedToPlayAIFaceAnim() )
			{
				m_bFaceTypeSet			= true;
				m_iLastAlertLevel		= (int)uFaceExpression;
				u_int uNumFaceAnims		= pxOurActor->GetNumFaceAnimsForExpression(uFaceExpression);

				if ( uNumFaceAnims > 0 )
				{
					m_uFaceType				= Asura_Random::GetIntU( 0, pxOurActor->GetNumFaceAnimsForExpression( uFaceExpression ) );

					pxFacialAnimHandler->StopAllFaceAnimations();
					pxFacialAnimHandler->PlayFaceAnimation( pxOurActor->GetFacialAnimation( uFaceExpression, m_uFaceType ), true );
				}
			}
		}
	}
}

bool AVP_Behaviour_Marine::IsAllowedToPlayAIFaceAnim()
{
	AVP_ServerActor_Human*						pxOurActor				= GetMarineBrain()->GetHumanActor();
	Asura_FacialAnimation_AnimationHandler*		pxFacialAnimHandler		= Asura_FacialAnimation_System::FindAnimationHandler( pxOurActor->GetGuid() );

	u_int uNumFound = 0;

	if ( pxFacialAnimHandler )
	{
		Asura_FacialAnimation_AnimationHandler::FaceAnimationInstanceContainer* pxContainer = pxFacialAnimHandler->GetFaceAnimInstanceContainer();

		if ( pxContainer )
		{
			u_int uNumExpressions = (u_int)AVP_ServerActor_Human::MAX_FACE_EXPRESSION;

			for ( u_int u = 0; u < uNumExpressions; ++u )
			{
				for ( u_int v = 0; v < 4; ++v )
				{
					if ( pxContainer->GetDataByHash( pxOurActor->GetFacialAnimation( (AVP_ServerActor_Human::ExpressionType)u, v ) ) )
					{
						++uNumFound;
					}
				}
			}
		}

		if ( pxFacialAnimHandler->GetNumPlayingFaceAnimations() > uNumFound )
		{
			return false;
		}
		else
		{
			return true;
		}
	}

	return true;
}


//---------------------------------------------------------------------------
// AVP_Behaviour_Marine :: PerFrameUpdate
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine::PerFrameUpdate(const float fTimeslice)
{
	AVP_ServerActor_Human*	pxOurActor = GetMarineBrain()->GetHumanActor();
	AVP_Tactical_System*	pxTactical = GetMarineBrain()->GetTacticalSystem();

	pxTactical->SetIsOnCombatGS2( false );
	pxTactical->SetShouldStopAtDestination( true );

	// Always keep our threat up to date
	if ( !pxOurActor->GetUseTestBehaviour() ) 
	{
		UpdateThreat();
		CheckForAiming();
	}

	if ( GetMarineBrain()->IsInjured() && GetUpdateState() != INJURED && GetUpdateState() != GRAPPLE && GetUpdateState() != TEQUIILA )
	{
		SwitchToState( INJURED );
	}

	if ( pxOurActor->GetClassification() != AVP_ENTITYCLASS_ANDROID )
	{
		SetFacialExpression(fTimeslice);
	}

	switch (GetUpdateState())
	{
		//---------------------------------------------------------------------------
		case IDLE:
		{
			pxTactical->StopMoving(); // We are Idle Stop Moving!

			Idle_Update();

			GetMarineBrain()->GetTacticalSystem()->SetIsInGrapple( false );

		}break;
		//---------------------------------------------------------------------------
		case IN_MELEE:
		{
			pxTactical->StopMoving();

			m_fMeleeTimer -= fTimeslice;

			if ( m_fMeleeTimer <= 0.0f )
			{
				SetUpdateState( IDLE );
			}

			GetMarineBrain()->GetTacticalSystem()->SetIsInGrapple( false );

		}break;
		//---------------------------------------------------------------------------
		case INJURED:
		case GRAPPLE:
		case TEQUIILA:
		{
				CheckForChildFinished(GetChildTypeFromUpdateState());
		}break;
		case MELEE_COUNTER:
		case HANDLE_OBJECTIVE:
		case SQUAD_MEMBER:
		case SQUAD_LEADER:
		case MELEE_PLAYER:
		case ON_LIFT:
		{
			GetMarineBrain()->GetTacticalSystem()->SetIsInGrapple( false );
			
			CheckForChildFinished( GetChildTypeFromUpdateState() );

		}break;
		//---------------------------------------------------------------------------
		case TEST_BEHAVIOUR:
		{
			GetMarineBrain()->GetTacticalSystem()->SetIsInGrapple( false );

		}break;
		//---------------------------------------------------------------------------

		//---------------------------------------------------------------------------
	
		default:
		{
			Asura_Assert(false, "AVP_Behaviour_Marine :: Update - Serious Problem. Update State Is Not Recognised!");
		}
		break;
	}

	// Always check for melee counter
	CheckForMeleeCounter();

	// At the end check for a new squad leader
	CheckForNewSquadLeader();

	// Keep the counter timer up to date
	UpdateCounterTimer(fTimeslice);
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine :: SwitchToState
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine::SwitchToState( MarineUpdateState eState )
{
	// Do anything to come out of last state
	switch ( GetUpdateState() )
	{
		case IDLE:
		{
		}break;
		case IN_MELEE:
		{
		}break;
		case GRAPPLE:
		{
			if ( GetChildWithType( this, AVP_BEHAVIOUR_TYPE_HUMAN_GRAPPLE ) )
			{
				DestroyChildWithType( AVP_BEHAVIOUR_TYPE_HUMAN_GRAPPLE );
			}

		}break;
		case HANDLE_OBJECTIVE:
		{
			if ( GetChildWithType( this, AVP_BEHAVIOUR_TYPE_MARINE_HANDLEOBJECTIVE ) )
			{
				DestroyChildWithType( AVP_BEHAVIOUR_TYPE_MARINE_HANDLEOBJECTIVE );
			}

		}break;
		case MELEE_COUNTER:
		{
			GetMarineBrain()->GetTacticalSystem()->SetStopShooting( false );

			if ( GetChildWithType( this, AVP_BEHAVIOUR_TYPE_MARINE_MELEE_COUNTER ) )
			{
				DestroyChildWithType( AVP_BEHAVIOUR_TYPE_MARINE_MELEE_COUNTER );
			}

		}break;
		case SQUAD_MEMBER:
		{
			if ( GetChildWithType( this, AVP_BEHAVIOUR_TYPE_MARINE_SQUADMEMBER ) )
			{
				DestroyChildWithType( AVP_BEHAVIOUR_TYPE_MARINE_SQUADMEMBER );
			}

		}break;
		case SQUAD_LEADER:
		{
			if ( GetChildWithType( this, AVP_BEHAVIOUR_TYPE_MARINE_SQUADLEADER ) )
			{
				DestroyChildWithType( AVP_BEHAVIOUR_TYPE_MARINE_SQUADLEADER );
			}

		}break;
		case TEST_BEHAVIOUR:
		{
			if ( GetChildWithType( this, AVP_BEHAVIOUR_TYPE_MARINE_TESTBEHAVIOUR ) )
			{
				DestroyChildWithType( AVP_BEHAVIOUR_TYPE_MARINE_TESTBEHAVIOUR );
			}

		}break;
		case MELEE_PLAYER:
		{
			if ( GetChildWithType( this, AVP_BEHAVIOUR_TYPE_MARINE_MELEE_PLAYER ) )
			{
				DestroyChildWithType( AVP_BEHAVIOUR_TYPE_MARINE_MELEE_PLAYER );
			}

		}break;

		case INJURED:
		{
			if ( GetChildWithType( this, AVP_BEHAVIOUR_TYPE_MARINE_INJURED ) )
			{
				DestroyChildWithType( AVP_BEHAVIOUR_TYPE_MARINE_INJURED );
			}
		}break;

		case TEQUIILA:
		{
			if ( GetChildWithType( this, AVP_BEHAVIOUR_TYPE_MARINE_TEQUIILACARRIED ) )
			{
				DestroyChildWithType( AVP_BEHAVIOUR_TYPE_MARINE_TEQUIILACARRIED );
			}
		}break;

		case ON_LIFT:
		{
			if ( GetChildWithType( this, AVP_BEHAVIOUR_TYPE_HUMAN_LIFT ) )
			{
				DestroyChildWithType( AVP_BEHAVIOUR_TYPE_HUMAN_LIFT );
			}
		}break;

		default:
		{
		}break;
	}

	SetUpdateState( eState );

	// Do anything to come go into next state
	switch ( GetUpdateState() )
	{
		case IDLE:
		{
		}break;
		case IN_MELEE:
		{
			GetMarineBrain()->GetTacticalSystem()->StopMoving();

		}break;
		case GRAPPLE:
		{
			if ( GetChildWithType( this, AVP_BEHAVIOUR_TYPE_HUMAN_GRAPPLE ) )
			{
				DestroyChildWithType( AVP_BEHAVIOUR_TYPE_HUMAN_GRAPPLE );
			}

			CreateAndLinkChild( AVP_BEHAVIOUR_TYPE_HUMAN_GRAPPLE );

		}break;
		case HANDLE_OBJECTIVE:
		{
			if ( GetChildWithType( this, AVP_BEHAVIOUR_TYPE_MARINE_HANDLEOBJECTIVE ) )
			{
				DestroyChildWithType( AVP_BEHAVIOUR_TYPE_MARINE_HANDLEOBJECTIVE );
			}

			CreateAndLinkChild( AVP_BEHAVIOUR_TYPE_MARINE_HANDLEOBJECTIVE );

		}break;
		case MELEE_COUNTER:
		{
			if ( GetChildWithType( this, AVP_BEHAVIOUR_TYPE_MARINE_MELEE_COUNTER ) )
			{
				DestroyChildWithType( AVP_BEHAVIOUR_TYPE_MARINE_MELEE_COUNTER );
			}

			CreateChildWithType( AVP_BEHAVIOUR_TYPE_MARINE_MELEE_COUNTER );

		}break;
		case SQUAD_MEMBER:
		{
			if ( GetChildWithType( this, AVP_BEHAVIOUR_TYPE_MARINE_SQUADMEMBER ) )
			{
				DestroyChildWithType( AVP_BEHAVIOUR_TYPE_MARINE_SQUADMEMBER );
			}

			CreateAndLinkChild( AVP_BEHAVIOUR_TYPE_MARINE_SQUADMEMBER );

		}break;
		case SQUAD_LEADER:
		{
			if ( GetChildWithType( this, AVP_BEHAVIOUR_TYPE_MARINE_SQUADLEADER ) )
			{
				DestroyChildWithType( AVP_BEHAVIOUR_TYPE_MARINE_SQUADLEADER );
			}

			CreateAndLinkChild( AVP_BEHAVIOUR_TYPE_MARINE_SQUADLEADER );

		}break;
		case TEST_BEHAVIOUR:
		{
			if ( GetChildWithType( this, AVP_BEHAVIOUR_TYPE_MARINE_TRAVEL ) )
			{
				DestroyChildWithType( AVP_BEHAVIOUR_TYPE_MARINE_TRAVEL );
			}

			if ( GetChildWithType( this, AVP_BEHAVIOUR_TYPE_MARINE_WEAPON ) )
			{
				DestroyChildWithType( AVP_BEHAVIOUR_TYPE_MARINE_WEAPON );
			}

			if ( GetChildWithType( this, AVP_BEHAVIOUR_TYPE_MARINE_TESTBEHAVIOUR ) )
			{
				DestroyChildWithType( AVP_BEHAVIOUR_TYPE_MARINE_TESTBEHAVIOUR );
			}

			CreateAndLinkChild( AVP_BEHAVIOUR_TYPE_MARINE_TESTBEHAVIOUR );

		}break;

		case MELEE_PLAYER:
		{
			if ( GetChildWithType( this, AVP_BEHAVIOUR_TYPE_MARINE_MELEE_PLAYER ) )
			{
				DestroyChildWithType( AVP_BEHAVIOUR_TYPE_MARINE_MELEE_PLAYER );
			}

			CreateAndLinkChild( AVP_BEHAVIOUR_TYPE_MARINE_MELEE_PLAYER );

		}break;

		case INJURED:
		{
			if ( GetChildWithType( this, AVP_BEHAVIOUR_TYPE_MARINE_INJURED ) )
			{
				DestroyChildWithType( AVP_BEHAVIOUR_TYPE_MARINE_INJURED );
			}

			CreateAndLinkChild( AVP_BEHAVIOUR_TYPE_MARINE_INJURED );
		}break;

		case TEQUIILA:
		{
			if ( GetChildWithType( this, AVP_BEHAVIOUR_TYPE_MARINE_TEQUIILACARRIED ) )
			{
				DestroyChildWithType( AVP_BEHAVIOUR_TYPE_MARINE_TEQUIILACARRIED );
			}

			CreateAndLinkChild( AVP_BEHAVIOUR_TYPE_MARINE_TEQUIILACARRIED );
		}break;

		case ON_LIFT:
		{
			if ( GetChildWithType( this, AVP_BEHAVIOUR_TYPE_HUMAN_LIFT ) )
			{
				DestroyChildWithType( AVP_BEHAVIOUR_TYPE_HUMAN_LIFT );
			}

			CreateAndLinkChild( AVP_BEHAVIOUR_TYPE_HUMAN_LIFT );

		}break;

		default:
		{
		}break;
	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine :: Idle_Update
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine::Idle_Update()
{
	AVP_ServerActor_Human* pxOurActor = GetMarineBrain()->GetHumanActor();

	if ( pxOurActor->GetUseTestBehaviour() )
	{
		SwitchToState( TEST_BEHAVIOUR );
	}
	else
	{
		AVP_ServerEntity_Squad_Marine*	pxSquad = NULL;

		if ( pxOurActor->GetInitialSquadGuid() != ASURA_GUID_UNREGISTERED )
		{
			pxSquad = AVP_ServerEntity_Squad_Marine::FindEntity( pxOurActor->GetInitialSquadGuid() );
		}
		else if ( DoWeHaveOurOwnValidObjective() )
		{
			SwitchToState( HANDLE_OBJECTIVE );
			return;
		}
		else if ( pxOurActor->GetInitialSquadGuid() == ASURA_GUID_UNREGISTERED )
		{
			SwitchToState( SQUAD_LEADER );
			return;
		}

		Asura_Assert( pxSquad, "AVP_Behaviour_Marine :: Idle_Update - We have a Squad Guid but the squad is not valid Marine / Android will do nothing." );

		if (pxSquad)
		{
			AVP_ServerEntity_Squad_Marine* pxSquad = AVP_ServerEntity_Squad_Marine::FindEntity(pxOurActor->GetInitialSquadGuid());

			if ( pxSquad->IsSquadSetup() )
			{
				if ( pxSquad->IsPlayersSquad() )
				{
					if ( DoesSquadHaveValidObjective() || DoWeHaveOurOwnValidObjective() )
					{
						SwitchToState( HANDLE_OBJECTIVE );
					}
					else if ( pxOurActor->GetGuid() == pxSquad->GetSquadLeaderGuid() )
					{
						SwitchToState( SQUAD_LEADER );
					}
					else
					{
						SwitchToState( SQUAD_MEMBER );
					}
				}
				else
				{
					if ( DoesSquadHaveValidObjective() || DoWeHaveOurOwnValidObjective() )
					{
						SwitchToState( HANDLE_OBJECTIVE );
					}
					else if ( pxOurActor->GetCurrentObjective() && pxOurActor->GetCurrentObjectiveThreatAction() == AVP_ServerNode_Objective_Marine::NO_ACTION )
					{
						SwitchToState( HANDLE_OBJECTIVE );
					}
					else if ( !pxOurActor->GetCurrentObjective() && pxSquad->GetCurrentObjective() && pxSquad->GetCurrentObjectiveThreatAction() == AVP_ServerNode_Objective_Marine::NO_ACTION )
					{
						SwitchToState( HANDLE_OBJECTIVE );
					}
					else if ( pxOurActor->GetGuid() == pxSquad->GetSquadLeaderGuid() )
					{
						SwitchToState( SQUAD_LEADER );
					}
					else
					{
						SwitchToState( SQUAD_MEMBER );
					}
				}
			}
		}
	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine :: DoWeHaveOurOwnValidObjective
//---------------------------------------------------------------------------
bool AVP_Behaviour_Marine::DoWeHaveOurOwnValidObjective()
{
	AVP_ServerActor_Human* pxOurActor = GetMarineBrain()->GetHumanActor();

	if ( pxOurActor->GetCurrentObjective() && !GetMarineBrain()->DoesHaveThreat() && !GetMarineBrain()->DoesHaveAlert() && 
		GetMarineBrain()->GetKnowledgeSystem()->GetAlertLevel() == AVP_Knowledge_System_Actor::PASSIVE_ALERTNESS )
	{
		return true;		
	}

	if ( pxOurActor->GetCurrentObjective() &&
		pxOurActor->GetCurrentObjectiveThreatAction() == AVP_ServerNode_Objective_Marine::NO_ACTION && 
		pxOurActor->GetCurrentObjectiveState() != AVP_ServerNode_Objective_Marine::COMPLETE &&
		pxOurActor->GetCurrentObjectiveState() != AVP_ServerNode_Objective_Marine::ARRIVED_AT_OBJECTIVE )
	{
		return true;
	}

	return false;
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine :: DoesSquadHaveValidObjective
//---------------------------------------------------------------------------
bool AVP_Behaviour_Marine::DoesSquadHaveValidObjective()
{
	AVP_ServerActor_Human*			pxOurActor = GetMarineBrain()->GetHumanActor();
	AVP_ServerEntity_Squad_Marine*	pxSquad = AVP_ServerEntity_Squad_Marine::FindEntity(pxOurActor->GetInitialSquadGuid());

	if ( pxSquad->IsSquadSetup() )
	{
		if ( pxSquad->IsPlayersSquad() )
		{
			if ( pxSquad->GetCurrentObjective() && !GetMarineBrain()->DoesHaveThreat() && !GetMarineBrain()->DoesHaveAlert() && 
				( GetMarineBrain()->GetKnowledgeSystem()->GetAlertLevel() == AVP_Knowledge_System_Actor::PASSIVE_ALERTNESS || pxSquad->ShouldIgnoreAlertState() ) &&
				pxSquad->GetCurrentObjectiveState() != AVP_ServerNode_Objective_Marine::NOT_STARTED &&
				pxSquad->GetCurrentObjectiveState() != AVP_ServerNode_Objective_Marine::MOVING_TO_OBJECTIVE )
			{
				return true;
			}
		}
		else
		{
			if ( pxSquad->GetCurrentObjective() && !GetMarineBrain()->DoesHaveThreat() && !GetMarineBrain()->DoesHaveAlert() && 
				( GetMarineBrain()->GetKnowledgeSystem()->GetAlertLevel() == AVP_Knowledge_System_Actor::PASSIVE_ALERTNESS || pxSquad->ShouldIgnoreAlertState() ) &&
				pxSquad->GetCurrentObjectiveState() != AVP_ServerNode_Objective_Marine::COMPLETE )
			{
				return true;
			}

		}
	}

	return false;
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine :: CheckForChildFinished
//---------------------------------------------------------------------------
bool AVP_Behaviour_Marine::CheckForChildFinished( u_int uBehaviourType )
{
	Axon_Behaviour* pxChild = GetChildWithType( this, uBehaviourType );

	if ( pxChild )
	{
		if ( pxChild->IsFinished() )
		{
			SwitchToState( IDLE );
			return true;
		}
	}

	return false;
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine :: UpdateCounterTimer
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine::UpdateCounterTimer(const float fTimeslice)
{
	m_fCounterTimer += fTimeslice;

	if ( m_fCounterTimer > 25.0f )
	{
		m_fCounterTimer = 25.0f;
	}

	if ( m_fCounterTimer > 20.0f && m_bHaveCountered == true )
	{
		m_bHaveCountered = false;
	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine :: Update
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine::CheckForMeleeCounter(void)
{
	AVP_ServerEntity_PlayerInfo* pxPlayerInfo = AVP_ServerEntity_PlayerInfo::FindEntity( 1000 );

	if ( pxPlayerInfo )
	{
		AVP_ServerActor_Vertebrate* pxVertPlayer = AVP_ServerActor_Vertebrate::FindEntity( pxPlayerInfo->GetPuppetGuid() );

		if ( pxVertPlayer )
		{
			bool bProcess = true;

			if ( pxVertPlayer->IsCloaked(true) && GetMarineBrain()->GetKnowledgeSystem()->GetAlertLevel() == AVP_Knowledge_System_Actor::PASSIVE_ALERTNESS )
			{
				bProcess = false;
			}

			if ( bProcess )
			{
				AVP_Vertebrate_MeleeCounter_System* pxMeleeCounter = pxVertPlayer->GetMeleeSystem();

				// Check if we are the current target
				if ( pxMeleeCounter && pxMeleeCounter->GetCurrentTarget() == GetMarineBrain()->GetHumanActor()->GetGuid() )
				{
					float fAttackDistanceSq = pxMeleeCounter->GetCurrentMaxCounterDistance();
					fAttackDistanceSq = fAttackDistanceSq * fAttackDistanceSq;
					float fDistanceSq = (GetMarineBrain()->GetHumanActor()->GetPosition() - pxVertPlayer->GetPosition()).MagnitudeSquared();

					// we want to counter
					if ( GetUpdateState() != MELEE_COUNTER && pxMeleeCounter->GetCurrentAttackType() != AVP_MELEEATTACK_TYPE_NONE && pxMeleeCounter->GetCurrentAttackCanBeCountered() && fDistanceSq <= fAttackDistanceSq )
					{
						Asura_Vector_3 xDirToCounterTarget = pxVertPlayer->GetPosition() - GetMarineBrain()->GetActor()->GetPosition();
						xDirToCounterTarget.Normalise();

						const Asura_Matrix_3x3&	xOurOri = GetMarineBrain()->GetActor()->GetOrientation();
						Asura_Vector_3			xFwd( xOurOri.e13, xOurOri.e23, xOurOri.e33 );

						float fDot = xFwd * xDirToCounterTarget;

						if ( fDot < 0.2f )
						{
							bProcess = false;
						}

						if ( bProcess && Asura_Random::GetFloat( 0.0f, 1.0f ) > 0.75f )
						{
							AVP_ServerActor_Human* pxOurActor = GetMarineBrain()->GetHumanActor();
							AVP_ClientEntity_Actor* pxOurClientActor = AVP_ClientEntity_Actor::FindEntity( pxOurActor->GetGuid() );

							if (pxOurClientActor)
							{
								AVP_Inventory_System*	pxInventorySystem	= pxOurActor->GetInventorySystem();
								AVP_Inventory_Item*		pxCurrentWeapon		= pxInventorySystem ? pxInventorySystem->GetCurrentItem(uAVP_INVENTORY_GROUPHASH_WEAPON) : NULL;

								if ((pxCurrentWeapon != NULL) && pxCurrentWeapon->IsUsing(true) && ( pxCurrentWeapon->GetCurrentUsageType() == AVP_InventoryWeapon_RangedWeapon::UT_FULL_AUTO_PRIMARY || pxCurrentWeapon->GetCurrentUsageType() == AVP_InventoryWeapon_RangedWeapon::UT_N_BULLET_BURST_PRIMARY ) )
								{
									pxCurrentWeapon->RequestEndUsage();
								}
							}

							GetMarineBrain()->GetTacticalSystem()->SetStopShooting( true );
							SwitchToState( MELEE_COUNTER );
						}
					}
				}
			}
		}
	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine :: CheckForAiming
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine::CheckForAiming(void)
{
	AVP_ServerActor_Human* pxOurActor = GetMarineBrain()->GetHumanActor();

	if ( GetMarineBrain()->GetKnowledgeSystem()->ShootAtScriptedTarget() || GetMarineBrain()->GetTacticalSystem()->GetForceAiming() || GetMarineBrain()->GetTacticalSystem()->GetMessageForceAiming() )
	{
		pxOurActor->SetAiming( true );
		return;
	}

	AVP_Human_AIAnimStateMachine*	pxStateMacine	= ( AVP_Human_AIAnimStateMachine* )pxOurActor->GetAIAnimStateMachine();

	if ( !pxStateMacine ) { return; }

//	AVP_ServerEntity_Squad_Marine* pxSquad = AVP_ServerEntity_Squad_Marine::FindEntity( pxOurActor->GetInitialSquadGuid() );

// 	if ( pxSquad )
// 	{
// 		if ( pxSquad->GetCurrentObjective() )
// 		{
// 			if ( pxSquad->GetCurrentObjectiveType() == AVP_ServerNode_Objective_Marine::MOVE_TO_OBJECTIVE_SLOW_AND_SCAN_AREA_STOP_TO_ENGAGE )
// 			{
// 				pxOurActor->SetAiming( true );
// 				return;
// 			}
// 		}
// 	}
	
	if ( GetMarineBrain()->GetKnowledgeSystem()->GetAlertLevel() == AVP_Knowledge_System_Actor::AGGRESSIVE_ALERTNESS || pxStateMacine->IsCrouchIdle() || pxStateMacine->IsCrouchMove() )
	{
		pxOurActor->SetAiming( true );
	}
	else
	{
		pxOurActor->SetAiming( false );
	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine :: SendMountGamesceneMessage
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine::SendMountGamesceneMessage()
{
	Asura_Message* pxMessage = Asura_Network::CreateMessage( ASURA_MSG_MOVE_START, sizeof( AVP_Message_Human_MountGamescene ) );

	if ( pxMessage )
	{
		AVP_Message_Human_MountGamescene*	pxData		= static_cast< AVP_Message_Human_MountGamescene* >( pxMessage->GetData() );
		AVP_ServerNode_GS2Node*				pxGS2Node	= AVP_ServerNode_GS2Node::FindEntity( GetMarineBrain()->GetHumanActor()->GetActionGamesceneGuid() );

		if ( pxGS2Node )
		{
			Asura_Matrix_3x3 xNodeOri;
			pxGS2Node->GetOrientation(xNodeOri);

			pxMessage->SetFromGuid( GetMarineBrain()->GetOwnerGuid() );
			pxMessage->SetToGuid( GetMarineBrain()->GetOwnerGuid() );
			pxData->ClearFlags();
			pxData->SetMoveID( AVP_Move_Human_MountGamescene::GetStaticHash() );
			pxData->SetDestinationPosition( pxGS2Node->GetPosition() );
			pxData->SetDestinationOrientation( xNodeOri );
			pxData->SetAnimBlendTemplateBlueprintHash( GetMountGamesceneAnimBlendTemplateBlueprintHash() );

			AVP_ServerActor_Human* xOurActor = GetMarineBrain()->GetHumanActor();

			if ( xOurActor->GetAIAnimStateMachine() )
			{
				xOurActor->GetAIAnimStateMachine()->RequestInMove( pxMessage, AVP_Human_AIAnimStateMachine::ACTOR_STATE_IDLE );
			}
			else
			{
				Asura_Network::DestroyMessage(pxMessage);
			}
		}
	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine :: GetMountGamesceneAnimation
//---------------------------------------------------------------------------
Asura_Hash_ID AVP_Behaviour_Marine::GetMountGamesceneAnimBlendTemplateBlueprintHash(void)
{
	const AVP_ServerActor_Human* pxHuman = GetMarineBrain()->GetHumanActor();

	const AVP_Inventory_System* pxInvSys = pxHuman->GetInventorySystem();
	const AVP_Inventory_Group* pxWeaponGroup = pxInvSys != NULL ? pxInvSys->GetGroup( uAVP_INVENTORY_GROUPHASH_WEAPON ) : NULL;
	Asura_Assert( pxWeaponGroup != NULL, "Unable to get wepaon group" );
	if ( pxWeaponGroup != NULL )
	{
		const AVP_Inventory_Item* pxItem = pxWeaponGroup->GetCurrentInventoryItem();
		if ( pxItem )
		{
			const AVP_Actor_AnimStateData* pxStateData = AVP_Human_AIAnimStateData::GetAnimStateData( pxHuman->GetAnimStateBlueprintType(), pxItem->GetBlueprintHash(), AVP_Human_AIAnimStateMachine::VERTEBRATE_STATE_MOVE_MEDIUM_TO_IDLE, pxHuman->IsAiming(), pxHuman->GetCoverType() );
			if ( pxStateData )
			{
				return pxStateData->GetAnimBlendTemplateBlueprintHash();
			}
		}
	}

	// else
	return ASURA_HASH_ID_UNSET;
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine :: PerFrameUpdate
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine::UpdateThreat(void)
{
	AVP_ServerActor_Human*			pxOurActor		= GetMarineBrain()->GetHumanActor();
	const Axon_Knowledge&			xThreat			= GetMarineBrain()->GetKnowledgeSystem()->GetBestThreatKnowledge();
	const Axon_Knowledge&			xAlert			= GetMarineBrain()->GetKnowledgeSystem()->GetBestAlertKnowledge();

	if ( xThreat.GetGuid() != ASURA_GUID_UNREGISTERED || xAlert.GetGuid() != ASURA_GUID_UNREGISTERED )
	{
		for ( Asura_Hashed_List_It < AVP_ServerActor_Human > it( AVP_ServerActor_Human::GetList() ); !it.Done(); it.Next() )
		{
			AVP_ServerActor_Human* pxHuman = it.GetCurrent();

			if ( pxHuman && pxOurActor->GetGuid() != pxHuman->GetGuid() && !pxHuman->IsDead() && !pxHuman->IsPlayer() && !pxHuman->IsADeadBody() && pxHuman->IsEntityActive() && ( AVP_Tactical_System::GetAllegiance( *pxOurActor, *pxHuman ) != AVP_ALLEGIANCE_ENEMY ) )
			{
				float fDistanceSq = ( pxOurActor->GetPosition() - pxHuman->GetPosition() ).MagnitudeSquared();

				// For Alert do a Distance check or Check that it is our squad.
				if ( fDistanceSq < 225.0f || pxHuman->GetInitialSquadGuid() == pxOurActor->GetInitialSquadGuid() ) // 15.0f meters
				{
					AVP_Brain_Actor_Marine* pxMarineBrain = ( AVP_Brain_Actor_Marine* )pxHuman->GetActorBrain();

					if ( pxMarineBrain && pxMarineBrain->IsActive() )
					{
						if ( xAlert.GetGuid() != ASURA_GUID_UNREGISTERED )
						{
							m_bHasAlert = true;
							pxMarineBrain->GetKnowledgeSystem()->AddAlertKnowledgeIfNeeded( xAlert );
						}
					}
				}

				// For Threat Do a Distance check and make sure it is local to GPV / Linked GPVs
				if ( fDistanceSq < 225.0f ) // 15.0f meters
				{
					bool bFound = false;
					AVP_ServerEntity_GamePlayVolume* pxGPV = pxOurActor->GetCurrentGamePlayVolume();

					if ( pxGPV )
					{
						int iNumVolumes = pxGPV->GetNumLinkedGamePlayVolumes();												

						AVP_ServerEntity_GamePlayVolume* pxTestGPV = NULL;

						for ( int i = -1; i < iNumVolumes; ++i )
						{
							if ( i == -1 )
								pxTestGPV = pxGPV;
							else
								pxTestGPV = AVP_ServerEntity_GamePlayVolume::FindEntity( pxGPV->GetLinkedGamePlayVolumeGuid( i ) );

							if ( pxTestGPV && pxTestGPV->Contains( pxHuman->GetPosition() ) )
							{
								bFound = true;
								break;
							}
						}
					}

					if ( bFound )
					{
						AVP_Brain_Actor_Marine* pxMarineBrain = ( AVP_Brain_Actor_Marine* )pxHuman->GetActorBrain();

						if ( pxMarineBrain && pxMarineBrain->IsActive() )
						{
							if ( xThreat.GetGuid() != ASURA_GUID_UNREGISTERED )
							{
								m_bHasThreat = true;
								pxMarineBrain->GetKnowledgeSystem()->AddKnowledgeIfNeeded( xThreat );
							}
						}
					}
				}
			}
		}
	}

	if ( xThreat.GetGuid() == ASURA_GUID_UNREGISTERED )
	{
		m_bHasThreat = false;
	}

	if ( xAlert.GetGuid() == ASURA_GUID_UNREGISTERED )
	{
		m_bHasAlert = false;
	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine :: CreateTravellingBehaviour
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine::CreateTravellingBehaviour()
{
	if (!GetTravellingBehaviour())
	{
		m_pxTravellingBehaviour = (AVP_Behaviour_Marine_Travel*)GetMarineBrain()->CreateBehaviour(AVP_BEHAVIOUR_TYPE_MARINE_TRAVEL);

		if (!GetTravellingBehaviour())
		{
			Asura_Assert(GetTravellingBehaviour(), "AVP_Behaviour_Marine::Activate - *ERROR* Could not create Travelling Behaviour");
		}
		else
		{
			m_pxTravellingBehaviour->SetParentBehaviour(this);
			LinkChild(m_pxTravellingBehaviour);
		}
	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine :: CreateWeaponBehaviour
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine::CreateWeaponBehaviour()
{
	if (!GetWeaponBehaviour())
	{
		m_pxWeaponBehaviour = (AVP_Behaviour_Marine_Weapon*)GetMarineBrain()->CreateBehaviour(AVP_BEHAVIOUR_TYPE_MARINE_WEAPON);

		if (!GetWeaponBehaviour())
		{
			Asura_Assert(GetWeaponBehaviour(), "AVP_Behaviour_Marine::Activate - *ERROR* Could not create Weapon Handling Behaviour");
		}
		else
		{
			m_pxWeaponBehaviour->SetParentBehaviour(this);
			LinkChild(m_pxWeaponBehaviour);
		}
	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine :: CreateAndLinkChild
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine::CreateAndLinkChild(u_int uBehaviourType)
{
	Axon_Project_Behaviour* pxBehaviour = NULL;

	pxBehaviour = (Axon_Project_Behaviour*)CreateChildWithType(uBehaviourType);

	if (pxBehaviour)
	{
		pxBehaviour->SetParentBehaviour(this);
	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine :: CheckForNewSquadLeader
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine::CheckForNewSquadLeader(void)
{
// 	AVP_ServerActor_Human* pxOurActor = GetMarineBrain()->GetHumanActor();
// 
// 	if (pxOurActor->GetInitialSquadGuid() != ASURA_GUID_UNREGISTERED && !pxOurActor->GetUseTestBehaviour())
// 	{
// 		AVP_ServerEntity_Squad_Marine* pxSquad = AVP_ServerEntity_Squad_Marine::FindEntity(pxOurActor->GetInitialSquadGuid());
// 
// 		if (pxSquad && !pxSquad->IsPlayersSquad())
// 		{
// 			if (pxSquad->GetSquadLeaderGuid() == pxOurActor->GetGuid())
// 			{
// 				u_int uChildType = GetChildTypeFromUpdateState();
// 
// 				// If the squad leader isn't in the correct
// 				if (
// 					(uChildType != AVP_BEHAVIOUR_TYPE_MARINE_SQUADLEADER) && 
// 					(uChildType != AVP_BEHAVIOUR_TYPE_MARINE_GRAPPLE) && 
// 					(uChildType != AVP_BEHAVIOUR_TYPE_MARINE_MELEE_COUNTER) && 
// 					)
// 				{
// 					Axon_Behaviour* pxChild = GetChildWithType(this, uChildType);
// 
// 					if (pxChild)
// 					{
// 						SwitchToState( IDLE );
// 					}
// 				}
// 			}
// 		}
// 	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine :: WriteToChunkStream
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine::WriteToChunkStream(Asura_Chunk_Stream& xStream) const
{
	xStream << static_cast<int>(CURRENT_CHUNK_VERSION);

	PARENT::WriteToChunkStream(xStream);
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine :: ReadFromChunkStream
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine::ReadFromChunkStream(Asura_Chunk_Stream& xStream)
{
	int iVersion;
	xStream >> iVersion;

	PARENT::ReadFromChunkStream(xStream);

	if (HasActivateBeenCalled())
	{
		if ( !GetMarineBrain()->GetHumanActor()->GetUseTestBehaviour() ) 
		{
			if (!GetTravellingBehaviour())
			{
				m_pxTravellingBehaviour = (AVP_Behaviour_Marine_Travel*)CreateChildWithType(AVP_BEHAVIOUR_TYPE_MARINE_TRAVEL);

				if (IsPaused())
				{
					m_pxTravellingBehaviour->DoPause();
				}
			}

			if (!GetWeaponBehaviour())
			{
				m_pxWeaponBehaviour = (AVP_Behaviour_Marine_Weapon*)CreateChildWithType(AVP_BEHAVIOUR_TYPE_MARINE_WEAPON);

				if (IsPaused())
				{
					m_pxWeaponBehaviour->DoPause();
				}
			}
		}
	}
}


//---------------------------------------------------------------------------
// AVP_Behaviour_Marine :: HandleMessage
//---------------------------------------------------------------------------
bool AVP_Behaviour_Marine::HandleMessage(const Asura_Message* pxMessage)
{
	switch (pxMessage->GetMsgID())
	{
		case AVP_MSGID_MOUNT_LIFT:
		{
			SwitchToState( ON_LIFT );
			return true;
		}

		//-----------------------------------------------------------------

		case AVP_MSGID_INCOMING_MELEE_FINISHED:
		{
			SwitchToState( IDLE );
			return true;
		}

		case AVP_MSGID_PICK_TEQUILA_UP:
		{
			u_int uChildType = GetChildTypeFromUpdateState();

			if ( uChildType != AVP_BEHAVIOUR_TYPE_MARINE_TEQUIILACARRIED )
			{
				SwitchToState( TEQUIILA );
			}

			return true;
		}

		case AVP_MSGID_PUT_TEQUILA_DOWN:
		{
			u_int uChildType = GetChildTypeFromUpdateState();

			if ( uChildType == AVP_BEHAVIOUR_TYPE_MARINE_TEQUIILACARRIED )
			{
				SwitchToState( IDLE );
			}

			return true;
		}

		//-----------------------------------------------------------------

		case ASURA_MSG_MOVE_START:
		case AVP_MSGID_MOVESTART_SERVER:
		{
			const AVP_Msg_Move*	pxData = static_cast<const AVP_Msg_Move*>(pxMessage->GetData());

			if ( pxData && pxData->GetMoveID() == AVP_Move_Puppet::GetStaticHash() )
			{
				GetMarineBrain()->GetTacticalSystem()->SetIsInGrapple(true);

				u_int uChildType = GetChildTypeFromUpdateState();

				if (uChildType != AVP_BEHAVIOUR_TYPE_HUMAN_GRAPPLE)
				{
					SwitchToState( GRAPPLE );
				}
			}
			return true;
		}

		case AVP_MSGID_MELEE_PLAYER_BACK:
		{
			if ( !GetMarineBrain()->DoesHaveThreat() )
			{
				u_int uChildType = GetChildTypeFromUpdateState();

				if ( uChildType != AVP_BEHAVIOUR_TYPE_MARINE_MELEE_PLAYER )
				{
					SwitchToState( MELEE_PLAYER );
				}
			}

			return true;
		}

		//-----------------------------------------------------------------

		case ASURA_MSG_MOVECHAINCOMPLETE:
		{
			const Asura_Msg_MoveChainComplete* pxData = static_cast< const Asura_Msg_MoveChainComplete* >( pxMessage->GetData() );

			if ( pxData && pxData->GetInitialMoveID() == AVP_Move_Human_MountGamescene::GetStaticHash() )
			{
				AVP_Behaviour_Marine_SquadMember* pxObjectiveBehaviour = ( AVP_Behaviour_Marine_SquadMember* )GetChildWithType( this, AVP_BEHAVIOUR_TYPE_MARINE_HANDLEOBJECTIVE );

				if ( pxObjectiveBehaviour )
				{
					pxObjectiveBehaviour->DoHandleMessage( pxMessage );
				}
			}
			else if ( pxData && pxData->GetInitialMoveID() == AVP_Move_Flinch::GetStaticHash() && GetChildTypeFromUpdateState() == AVP_BEHAVIOUR_TYPE_MARINE_MELEE_PLAYER )
			{
				AVP_Behaviour_Marine_MeleePlayer* pxBehaviour = ( AVP_Behaviour_Marine_MeleePlayer* )GetChildWithType( this, AVP_BEHAVIOUR_TYPE_MARINE_MELEE_PLAYER );

				if ( pxBehaviour )
				{
					pxBehaviour->DoHandleMessage( pxMessage );
				}
			}
			else
			{

				u_int uChildType = GetChildTypeFromUpdateState();

				switch (uChildType)
				{
					case AVP_BEHAVIOUR_TYPE_MARINE_GRAPPLE:
					{
						GetMarineBrain()->GetTacticalSystem()->SetIsInGrapple( false );
						SwitchToState( IDLE );
					}break;
					
					case AVP_BEHAVIOUR_TYPE_MARINE_MELEE_COUNTER:
					{
						SwitchToState( IDLE );
					}break;

					default:
					{

					}break;
				}
			}

			return true;
		}

		//-----------------------------------------------------------------

		case AVP_MSGID_LEADER_MOVING_TO_AI_OBJECTIVE:
		{
			AVP_Behaviour_Marine_SquadMember* pxSquadMemberBehaviour = ( AVP_Behaviour_Marine_SquadMember* )GetChildWithType( this, AVP_BEHAVIOUR_TYPE_MARINE_SQUADMEMBER );

			if ( pxSquadMemberBehaviour )
			{
				pxSquadMemberBehaviour->DoHandleMessage( pxMessage );
			}

			return true;
		}

		//-----------------------------------------------------------------

		default:
		{
			return PARENT::HandleMessage(pxMessage);
		}
	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine :: GetChildTypeFromUpdateState
//---------------------------------------------------------------------------
u_int AVP_Behaviour_Marine::GetChildTypeFromUpdateState(void)
{
	switch (GetUpdateState())
	{
		case SQUAD_MEMBER:		return AVP_BEHAVIOUR_TYPE_MARINE_SQUADMEMBER;
		case SQUAD_LEADER:		return AVP_BEHAVIOUR_TYPE_MARINE_SQUADLEADER;
		case GRAPPLE:			return AVP_BEHAVIOUR_TYPE_MARINE_GRAPPLE;
		case TEST_BEHAVIOUR:	return AVP_BEHAVIOUR_TYPE_MARINE_TESTBEHAVIOUR;
		case MELEE_COUNTER:		return AVP_BEHAVIOUR_TYPE_MARINE_MELEE_COUNTER;
		case HANDLE_OBJECTIVE:	return AVP_BEHAVIOUR_TYPE_MARINE_HANDLEOBJECTIVE;
		case MELEE_PLAYER:		return AVP_BEHAVIOUR_TYPE_MARINE_MELEE_PLAYER;
		case TEQUIILA:			return AVP_BEHAVIOUR_TYPE_MARINE_TEQUIILACARRIED;
		case ON_LIFT:			return AVP_BEHAVIOUR_TYPE_HUMAN_LIFT;
		case IDLE:
		default:
		{
			return 0xffffffff;
		}
	}

}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine :: RenderDebugInfo
//---------------------------------------------------------------------------
#ifndef ASURA_NO_DEBUG_OUTPUT
static const char* s_aszMarineStateStrings[AVP_Behaviour_Marine::MAX_STATES] =
{
	"MARINE - UNSET",
	"MARINE - IDLE",
	"MARINE - IN_MELEE",
	"MARINE - GRAPPLE",
	"MARINE - HANDLE_OBJECTIVE",
	"MARINE - MELEE_COUNTER",
	"MARINE - SQUAD_MEMBER",
	"MARINE - SQUAD_LEADER",
	"MARINE - TEST_BEHAVIOUR",
};

static const char* s_aszAndroidStateStrings[AVP_Behaviour_Marine::MAX_STATES] =
{
	"ANDROID - UNSET",
	"ANDROID - IDLE",
	"ANDROID - IN_MELEE",
	"ANDROID - GRAPPLE",
	"ANDROID - HANDLE_OBJECTIVE",
	"ANDROID - MELEE_COUNTER",
	"ANDROID - SQUAD_MEMBER",
	"ANDROID - SQUAD_LEADER",
	"ANDROID - TEST_BEHAVIOUR",
};

static bool g_bDrawHighLevelInfo = false;

static const char* s_aszAlertStrings[AVP_Knowledge_System_Actor::MAX_ALERT_LEVELS] =
{
	"Alert Level - PASSIVE",
	"Alert Level - ACTIVE",
	"Alert Level - AGGRESSIVE",
};

void AVP_Behaviour_Marine::RenderDebugInfo()
{
	PARENT::RenderDebugInfo();

	if (g_bDrawHighLevelInfo)
	{
		GetProjectBrain()->GetDebugOutputPos().y += 2.0f;

		bool	bIsAndroid = false;

		if (GetMarineBrain()->GetHumanActor()->GetClassification() == AVP_ENTITYCLASS_ANDROID)
		{
			bIsAndroid = true;
		}

		if (!bIsAndroid)
		{
			Asura_Font_System::Print
			(
				GetProjectBrain()->GetDebugOutputPos().x,
				GetProjectBrain()->GetDebugOutputPos().y,
				0.6f,
				AVP_Brain_Actor_Marine::s_uHighLevelBehaviourDebugColour,
				ASURA_TEXTFORMAT_CENTREJUSTIFIED, 
				s_aszMarineStateStrings[GetUpdateState()]
			);
		}
		else
		{
			Asura_Font_System::Print
			(
				GetProjectBrain()->GetDebugOutputPos().x,
				GetProjectBrain()->GetDebugOutputPos().y,
				0.6f,
				0xff00ffff,
				ASURA_TEXTFORMAT_CENTREJUSTIFIED, 
				s_aszAndroidStateStrings[GetUpdateState()]
			);
		}

		// Offset the debug output position for other behaviours to render there own text
		GetProjectBrain()->GetDebugOutputPos().y += 14.0f;

		u_int uColour = 0xFF00FF00;
		switch (GetMarineBrain()->GetKnowledgeSystem()->GetAlertLevel())
		{
			case AVP_Knowledge_System_Actor::AGGRESSIVE_ALERTNESS:	uColour = 0xffff0000; break;
			case AVP_Knowledge_System_Actor::ACTIVE_ALERTNESS:		uColour = 0xffffff00; break;
			case AVP_Knowledge_System_Actor::PASSIVE_ALERTNESS:		uColour = 0xff00ff00; break;
			default: {} break;
		}

		Asura_Font_System::Print
		(
			GetProjectBrain()->GetDebugOutputPos().x,
			GetProjectBrain()->GetDebugOutputPos().y,
			0.5f,
			uColour,
			ASURA_TEXTFORMAT_CENTREJUSTIFIED, 
			s_aszAlertStrings[GetMarineBrain()->GetKnowledgeSystem()->GetAlertLevel()]
		);

		GetProjectBrain()->GetDebugOutputPos().y += 10.0f;
	}

	if ( GetMarineBrain()->GetHumanActor()->GetCurrentObjective() )
	{
		Asura_Font_System::Print
		(
			GetProjectBrain()->GetDebugOutputPos().x,
			GetProjectBrain()->GetDebugOutputPos().y,
			0.5f,
			0xffffff00,
			ASURA_TEXTFORMAT_CENTREJUSTIFIED | ASURA_TEXTFORMAT_FORMATSTRING, 
			"Current Objective : INVIDIVDUAL : GUID = %d", GetMarineBrain()->GetHumanActor()->GetCurrentObjectiveGuid()
		);

		GetProjectBrain()->GetDebugOutputPos().y += 10.0f;
	}
	else
	{
		AVP_ServerEntity_Squad_Marine* pxSquad = GetMarineBrain()->GetMarineSquad();

		if ( pxSquad && pxSquad->GetCurrentObjective() )
		{
			Asura_Font_System::Print
			(
				GetProjectBrain()->GetDebugOutputPos().x,
				GetProjectBrain()->GetDebugOutputPos().y,
				0.5f,
				0xffffff00,
				ASURA_TEXTFORMAT_CENTREJUSTIFIED | ASURA_TEXTFORMAT_FORMATSTRING, 
				"Current Objective : SQUAD : GUID = %d", pxSquad->GetCurrentObjectiveGuid()
			);

			GetProjectBrain()->GetDebugOutputPos().y += 10.0f;
		}
	}

	if ( GetMarineBrain()->GetKnowledgeSystem() )
	{
		if ( GetMarineBrain()->GetKnowledgeSystem()->GetAlertLevel() == AVP_Knowledge_System_Actor::PASSIVE_ALERTNESS )
		{
			Asura_Font_System::Print
			(
				GetProjectBrain()->GetDebugOutputPos().x,
				GetProjectBrain()->GetDebugOutputPos().y,
				0.5f,
				0xff00ff00,
				ASURA_TEXTFORMAT_CENTREJUSTIFIED, 
				"PASSIVE ALERTNESS"
			);

			GetProjectBrain()->GetDebugOutputPos().y += 10.0f;
		}
		else if ( GetMarineBrain()->GetKnowledgeSystem()->GetAlertLevel() == AVP_Knowledge_System_Actor::ACTIVE_ALERTNESS )
		{
			Asura_Font_System::Print
			(
				GetProjectBrain()->GetDebugOutputPos().x,
				GetProjectBrain()->GetDebugOutputPos().y,
				0.5f,
				0xff00ff00,
				ASURA_TEXTFORMAT_CENTREJUSTIFIED, 
				"ACTIVE ALERTNESS"
			);

			GetProjectBrain()->GetDebugOutputPos().y += 10.0f;

			Asura_Font_System::Print
			(
				GetProjectBrain()->GetDebugOutputPos().x,
				GetProjectBrain()->GetDebugOutputPos().y,
				0.5f,
				0xff00ff00,
				ASURA_TEXTFORMAT_CENTREJUSTIFIED | ASURA_TEXTFORMAT_FORMATSTRING, 
				"Alert Timer = %0.2f / %0.2f", GetMarineBrain()->GetKnowledgeSystem()->GetAlertTimer(), GetMarineBrain()->GetKnowledgeSystem()->GetActiveAlertTimOut()
			);

			GetProjectBrain()->GetDebugOutputPos().y += 10.0f;
		}
		else
		{
			Asura_Font_System::Print
			(
				GetProjectBrain()->GetDebugOutputPos().x,
				GetProjectBrain()->GetDebugOutputPos().y,
				0.5f,
				0xff00ff00,
				ASURA_TEXTFORMAT_CENTREJUSTIFIED | ASURA_TEXTFORMAT_FORMATSTRING, 
				"AGGRESSIVE ALERTNESS"
			);

			GetProjectBrain()->GetDebugOutputPos().y += 10.0f;

			Asura_Font_System::Print
			(
				GetProjectBrain()->GetDebugOutputPos().x,
				GetProjectBrain()->GetDebugOutputPos().y,
				0.5f,
				0xff00ff00,
				ASURA_TEXTFORMAT_CENTREJUSTIFIED | ASURA_TEXTFORMAT_FORMATSTRING, 
				"Alert Timer = %0.2f / %0.2f", GetMarineBrain()->GetKnowledgeSystem()->GetAlertTimer(), GetMarineBrain()->GetKnowledgeSystem()->GetAggressiveAlertTimOut()
			);

			GetProjectBrain()->GetDebugOutputPos().y += 10.0f;
		}
	}

	
}
#endif

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////
