//---------------------------------------------------------------------------
// File			: AVP_Behaviour_Marine.h
// Created		: 29 / 10 / 07
// Created By	: Gareth Thomas
// Copyright (C) 2007 Rebellion. All Rights Reserved.
//---------------------------------------------------------------------------

// BEGIN FILE WRAPPER ///////////////////////////////////////
#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif
#ifndef __AVP_BEHAVIOUR_MARINE_H__
#define __AVP_BEHAVIOUR_MARINE_H__ 1
// END FILE WRAPPER ///////////////////////////////////////////

// INCLUDES ///////////////////////////////////////////////////////////
#include "Behaviour/Axon_Project_Behaviour.h"

// FORWARD DECLARATIONS ///////////////////////////////////
class AVP_Brain_Actor_Marine;
class AVP_Behaviour_Marine_InCover;
class AVP_Behaviour_Marine_Look;
class AVP_Behaviour_Marine_Travel;
class AVP_Behaviour_Marine_Weapon;
class AVP_Behaviour_Marine_Attack;
class AVP_Behaviour_Marine_Alert;
class AVP_Behaviour_Marine_PositionEvaluation;
class AVP_ServerEntity_GamePlayVolume;

//---------------------------------------------------------------------------
// Class		: AVP_Behaviour_Marine
// Description	: High Level Behaviour for the Marine AI Actors
//---------------------------------------------------------------------------
class AVP_Behaviour_Marine : public Axon_Project_Behaviour
{
	typedef Axon_Project_Behaviour PARENT;

public:

	// Marine Update State Enum
	enum MarineUpdateState
	{
		UNSET,
		IDLE,
		IN_MELEE,
		GRAPPLE,
		HANDLE_OBJECTIVE,
		MELEE_COUNTER,
		SQUAD_MEMBER,
		SQUAD_LEADER,
		MELEE_PLAYER,
		TEST_BEHAVIOUR,
		INJURED,
		TEQUIILA,
		ON_LIFT,
		MAX_STATES
	};

	// Construction / Deconstruction
	AVP_Behaviour_Marine								(AVP_Brain_Actor_Marine* pxBrain);
	virtual ~AVP_Behaviour_Marine						(void)								{}

	// Public Member Functions...
	MarineUpdateState				GetUpdateState				(void) const												{return m_eUpdateState;}
	AVP_Behaviour_Marine_Travel*	GetTravellingBehaviour		(void)														{return m_pxTravellingBehaviour;}
	AVP_Behaviour_Marine_Weapon*	GetWeaponBehaviour			(void)														{return m_pxWeaponBehaviour;}

protected:
	virtual bool 					Activate					(void);
	virtual void 					PostLoadInitialise			(void);
	virtual void 					Deactivate					(void);
	virtual void 					Pause						(void);
	virtual void 					Unpause						(void);
	virtual void					PerFrameUpdate				(const float fTimeslice);
	virtual void					ReadFromChunkStream			(Asura_Chunk_Stream& xStream);
	virtual void					WriteToChunkStream			(Asura_Chunk_Stream& xStream) const;
	virtual bool					HandleMessage				(const Asura_Message* pxMessage);
#ifndef ASURA_NO_DEBUG_OUTPUT
	virtual void					RenderDebugInfo				(void);
#endif

	// Protected Member Functions...
	AVP_Brain_Actor_Marine*				GetMarineBrain				(void)													{return m_pxMarineBrain;}
	void								CreateTravellingBehaviour	(void);
	void								CreateLookBehaviour			(void);
	void								CreateWeaponBehaviour		(void);
	u_int								GetChildTypeFromUpdateState	(void);
	void								UpdateThreat				(void);
	void								SetUpdateState				(MarineUpdateState eState)								{m_eUpdateState = eState;}
	bool								DoWeHaveOrders				(void)													{return m_bHaveOrders;}
	bool								CheckForChildFinished		(u_int uBehaviourType);
	void								CreateAndLinkChild			(u_int uBehaviourType);
	void								CreateFormationvBeahviour	(void);
	void								CheckForNewSquadLeader		(void);
	void								CheckForMeleeCounter		(void);
	void								MoveToAction				(void);
	void								CheckForAiming				(void);

	// Update state update functions
	void								Idle_Update					(void);
	void								Formation_Update			(void);
	void								SendMountGamesceneMessage	(void);
	Asura_Hash_ID						GetMountGamesceneAnimBlendTemplateBlueprintHash	(void);
	void								LookAround					(const float fTimeslice);

	void								SwitchToState( MarineUpdateState eState );
	void								UpdateCounterTimer(const float fTimeslice);

	bool								DoWeHaveOurOwnValidObjective();
	bool								DoesSquadHaveValidObjective();

	void								SetFacialExpression(const float fTimeslice);

	bool								IsAllowedToPlayAIFaceAnim(void);

private:

	// Private Member Variables...
	MarineUpdateState					m_eUpdateState;
	AVP_Brain_Actor_Marine*				m_pxMarineBrain;
	bool								m_bHaveOrders;
	bool								m_bHaveNewOrders;
	bool								m_bIsPatrolling;
	bool								m_bHasThreat;
	bool								m_bHasAlert;
	bool								m_bHaveCountered;
	float								m_fCounterTimer;
	Asura_Guid							m_uActionGuid;
	float								m_fMeleeTimer;
	bool								m_bNewLookSet;
	float								m_fLookTimer;
	float								m_fLookTimeout;
	bool								m_bDirection;
	u_int								m_uLastRouteSegment;

	u_int								m_uFaceType;
	int									m_iLastAlertLevel;
	bool								m_bFaceTypeSet;
	float								m_fFaceTimer;

	// Behaviour Pointers...
	AVP_Behaviour_Marine_Travel*		m_pxTravellingBehaviour;				// Travelling
	AVP_Behaviour_Marine_Weapon*		m_pxWeaponBehaviour;

	enum { CURRENT_CHUNK_VERSION = 0 };

};

#endif //__AVP_BEHAVIOUR_MARINE_H__

/// End Of File 'AVP_Behaviour_Marine.h'
