//---------------------------------------------------------------------------
// File			: AVP_Behaviour_Marine_Attack_SmartGun.cpp
// Created		: 
// Created By	: Gareth Thomas
// Copyright (C) 2007 Rebellion. All Rights Reserved.
//---------------------------------------------------------------------------

// core
#include "Core/Asura.h"
// header
#include "Behaviour/BehaviourTypes/Human/Marine/AVP_Behaviour_Marine_Attack_SmartGun.h"
// asura
#include "Entity/Locations/OrientedNode/CoverpointNode/Asura_ServerNode_Coverpoint.h"
#ifndef ASURA_NO_DEBUG_OUTPUT
#include "Rendering/Asura_Font_System.h"
#endif

// axon
// AVP
#include "Entity/GamePlayVolume/AVP_ServerEntity_GamePlayVolume.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Human/AVP_ServerActor_Human.h"
#include "Inventory/AVP_Inventory_HashIDs.h"
#include "Inventory/AVP_Inventory_Item.h"
#include "Inventory/AVP_Inventory_System.h"

// AVP axon
#include "Behaviour/BehaviourTypes/AVP_Behaviour_Types.h"
#include "Core/AVP_Navigation.h"
#include "Core/BrainTypes/AVP_Brain_Actor_Marine.h"
#include "Core/AVP_PositionEvaluation.h"
#include "Knowledge/AVP_Knowledge_System_Actor.h"
#include "Knowledge/SenseTypes/AVP_Sense_Vision_Actor.h"
#include "Tactical/AVP_Tactical_System.h"

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_Attack_SmartGun :: Constructor
//---------------------------------------------------------------------------
AVP_Behaviour_Marine_Attack_SmartGun::AVP_Behaviour_Marine_Attack_SmartGun( AVP_Brain_Actor_Marine* pxBrain )
: PARENT( pxBrain, AVP_BEHAVIOUR_TYPE_MARINE_ATTACK_SMARTGUN )
, m_pxMarineBrain( pxBrain )
, m_eUpdateState( SMARTGUN_CHOOSE )
, m_fTimer( 0.0f )
, m_uThreatGuid( ASURA_GUID_UNREGISTERED )
{

}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_Attack_SmartGun :: Deconstructor
//---------------------------------------------------------------------------
AVP_Behaviour_Marine_Attack_SmartGun::~AVP_Behaviour_Marine_Attack_SmartGun( void )
{

}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_Attack_SmartGun :: Activate
//---------------------------------------------------------------------------
bool AVP_Behaviour_Marine_Attack_SmartGun::Activate( void )
{
	const bool bParentOk = PARENT::Activate();
	// Put Code Below This Comment

	GetMarineBrain()->GetTacticalSystem()->SetAimLerpModifier( 2.5f );

	// Put Code Above This Comment
	return bParentOk;
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_Attack_SmartGun :: Deactivate
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_Attack_SmartGun::Deactivate( void )
{
	if ( GetMarineBrain()->IsActive() ) 
	{
		GetMarineBrain()->GetTacticalSystem()->SetAimLerpModifier( 1.0f );
	}
	// Put Code Above This Comment
	DestroyAllChildren();
	PARENT::Deactivate();
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_Attack_SmartGun :: PerFrameUpdate
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_Attack_SmartGun::PerFrameUpdate( const float fTimeslice )
{
	switch ( GetUpdateState() )
	{
		case SMARTGUN_CHOOSE:
		{
			u_int uNumThreats		= GetKnowledgeSystem()->GetNumSecondaryThreats() + 1;
			u_int uCurrentThreat	= Asura_Random::GetIntU( 0, uNumThreats - 1 );

			if ( uCurrentThreat == 0 || GetMarineBrain()->GetKnowledgeSystem()->ShootAtScriptedTarget() )
			{
				m_uThreatGuid = GetKnowledgeSystem()->GetBestThreatKnowledge().GetGuid();
			}
			else
			{
				m_uThreatGuid = GetKnowledgeSystem()->GetSecondaryThreatKnowledge( uCurrentThreat - 1 ).GetGuid();
			}

			SwitchToState( SMARTGUN_AIM );

		}break;

		case SMARTGUN_AIM:
		{
			AVP_ServerEntity_Actor*	pxThreatActor	= AVP_ServerEntity_Actor::FindEntity( m_uThreatGuid );
			AVP_ServerActor_Human*	pxOurActor		= GetMarineBrain()->GetHumanActor();

			if ( pxThreatActor && pxOurActor )
			{
				Asura_Vector_3 xThreatChestPos( Asura_Maths::ZeroVector3 );
				Asura_Vector_3 xThreatEyePos( Asura_Maths::ZeroVector3 );
				Asura_Vector_3 xOurEyePos( Asura_Maths::ZeroVector3 );

				pxThreatActor->GetChestPosition( xThreatChestPos );
				pxThreatActor->GetHeadPosition( xThreatEyePos );

				GetMarineBrain()->AimAtPosition( xThreatChestPos );
				GetMarineBrain()->SetLookAtPosition( xThreatEyePos );

				pxOurActor->GetHeadPosition( xOurEyePos );

				Asura_Vector_3 xDir( xThreatEyePos - xOurEyePos );
				xDir.Normalise();

				Asura_Vector_3 xLookDir( GetMarineBrain()->GetLookAtPosition() - xOurEyePos );
				xLookDir.Normalise();

				float fDot = xDir * xLookDir;

				if ( fDot > 0.95f )
				{
					AVP_Inventory_System*	pxInventorySystem	= pxOurActor->GetInventorySystem();
					AVP_Inventory_Item*		pxCurrentWeapon		= pxInventorySystem ? pxInventorySystem->GetCurrentItem(uAVP_INVENTORY_GROUPHASH_WEAPON) : NULL;

					if ( ( pxCurrentWeapon != NULL ) && !pxCurrentWeapon->IsIdling( true ) )
					{
						m_fTimer += fTimeslice;

						if ( m_fTimer > 0.5f )
						{
							SwitchToState( SMARTGUN_CHOOSE );
						}
					}
				}

				if ( fDot < 0.0f )
				{
					GetMarineBrain()->GetTacticalSystem()->SetAimLerpModifier( 2.5f );
				}
				else
				{
					GetMarineBrain()->GetTacticalSystem()->SetAimLerpModifier( 0.5f );
				}
			}

		}break;

		default:
		{

		}break;
	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_Attack_SmartGun :: GetAttackingScoreForWaypoint
//---------------------------------------------------------------------------
u_int AVP_Behaviour_Marine_Attack_SmartGun::EvaluationFunction( AVP_Navigation::AVP_PosEvalParams xPosEvalParams )
{
	AVP_ServerActor_Human* pxHuman = AVP_ServerActor_Human::FindEntity( xPosEvalParams.m_uActorGuid );

	u_int uScore = 0;

	// this should always be valid as he has just requested a position evaluation
	if ( !pxHuman ) { return uScore; }

	// Waypoint Proximity
	uScore += AVP_PositionEvaluation::GetProximityScore( pxHuman, xPosEvalParams.m_uTestWPID, xPosEvalParams.m_fEvaluationRange * 2.0f );

	AVP_Brain_Actor_Marine* pxBrain = (AVP_Brain_Actor_Marine*)pxHuman->GetActorBrain();

	if ( pxBrain )
	{
		if ( !pxBrain->GetKnowledgeSystem()->ShootAtScriptedTarget() || ( pxBrain->GetKnowledgeSystem()->ShootAtScriptedTarget() && !pxBrain->GetKnowledgeSystem()->ScriptedTargetIsShootAtPoint() ) )
		{
			// Line Of Fire To The Primary Threat
			uScore += AVP_PositionEvaluation::GetWaypointLOFtoPrimaryThreatScore( pxHuman, xPosEvalParams.m_uTestWPID, 60 );
		}
		else
		{
			uScore += AVP_PositionEvaluation::GetWaypointLOFToScriptedTarget( pxHuman, xPosEvalParams.m_uTestWPID, 40 );
		}
	}

	// Preferred Fighting Range
	uScore += AVP_PositionEvaluation::GetPreferredFightingRangeScore( pxHuman, xPosEvalParams.m_uTestWPID, xPosEvalParams.m_fMinFightingRange, xPosEvalParams.m_fMaxFightingRange, 40 );

	// Waypoint Occupied by ally
	uScore += AVP_PositionEvaluation::GetHumanoidPositionOccupiedByAllyScore( pxHuman, xPosEvalParams.m_uTestWPID, 30 );

	return uScore;
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_Attack_SmartGun :: FindBestWaypoint
//---------------------------------------------------------------------------
bool AVP_Behaviour_Marine_Attack_SmartGun::FindBestWaypoint( AVP_Navigation::PosEvalFunctor* pxPosEval )
{
	u_int uBestWaypointID = pxPosEval->GetBestWaypointID();

	if ( uBestWaypointID != ASURA_INVALID_WAYPOINT_ID )
	{
		Asura_AI_NavVertex*	pxWaypoint	= Asura_AI_Navigation_System::GetWaypoint( uBestWaypointID );
		AVP_Tactical_System* pxTactical	= GetMarineBrain()->GetTacticalSystem();

		if ( pxWaypoint )
		{
			float fDistanceSq = ( GetOurActor()->GetPosition() - pxWaypoint->m_xPosition ).MagnitudeSquared();

			if ( fDistanceSq > 9.0f )
			{
				pxTactical->SetDestination( pxWaypoint->m_xPosition );
				pxTactical->SetAtDestinationRange( ONE_METER );
				pxTactical->SetMovementSpeed( AVP_Navigation::MM_SLOW );
				pxTactical->LetRouteSetLookAt( false );
				return true;
			}
		}
	}

	return false;
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_Attack_SmartGun :: SwitchToState
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_Attack_SmartGun::SwitchToState( UpdateState eState )
{
	switch (GetUpdateState())
	{
		case SMARTGUN_CHOOSE:
		{

		}break;
		case SMARTGUN_AIM:
		{

		}break;
		default:
		{

		}
	}

	SetUpdateState( eState );

	switch (GetUpdateState())
	{
		case SMARTGUN_CHOOSE:
		{
		}break;
		case SMARTGUN_AIM:
		{
			m_fTimer = 0.0f;

		}break;
		default:
		{

		}
	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_Attack_SmartGun :: RenderDebugInfo
//---------------------------------------------------------------------------
#ifndef ASURA_NO_DEBUG_OUTPUT
static const char* s_aszStateStrings[ AVP_Behaviour_Marine_Attack_SmartGun::MAX_STATES ] =
{
	"Attack_SmartGun - UNSET",
	"Attack_SmartGun - SMARTGUN_CHOOSE",
	"Attack_SmartGun - SMARTGUN_AIM",
};

void AVP_Behaviour_Marine_Attack_SmartGun::RenderDebugInfo( void )
{
	PARENT::RenderDebugInfo();

	Asura_Font_System::Print
	(
		GetProjectBrain()->GetDebugOutputPos().x,
		GetProjectBrain()->GetDebugOutputPos().y,
		0.5f, 
		0XFFFFFFFF,
		ASURA_TEXTFORMAT_CENTREJUSTIFIED, 
		s_aszStateStrings[GetUpdateState()]
	);

	// Offset the debug output position for other behaviours to render there own text
	GetProjectBrain()->GetDebugOutputPos().y += 10.0f;

	Asura_Font_System::Print
	(
		GetProjectBrain()->GetDebugOutputPos().x,
		GetProjectBrain()->GetDebugOutputPos().y,
		0.5f, 
		0XFFFFFFFF,
		ASURA_TEXTFORMAT_CENTREJUSTIFIED | ASURA_TEXTFORMAT_FORMATSTRING, 
		"AIM TIME: %0.2f", m_fTimer
	);

	// Offset the debug output position for other behaviours to render there own text
	GetProjectBrain()->GetDebugOutputPos().y += 10.0f;
}
#endif

/// End Of File 'AVP_behaviour_Marine_Attack_SmartGun.cpp'
