//---------------------------------------------------------------------------
// File			: AVP_Behaviour_Marine_HandleObjective.cpp
// Created		: 
// Created By	: Gareth Thomas
// Copyright (C) 2007 Rebellion. All Rights Reserved.
//---------------------------------------------------------------------------

// INCLUDES ///////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"
// header
#include "Behaviour/BehaviourTypes/Human/Marine/AVP_Behaviour_Marine_HandleObjective.h"
// asura
#ifndef ASURA_NO_DEBUG_OUTPUT
#include "Rendering/Asura_Font_System.h"
#endif

#include "Blueprint/Asura_Blueprint_System.h"
#include "Entity/PhysicalObject/Moves/Asura_Movement_System.h"
#include "Network/Asura_Message.h"

// axon
#include "Core/Axon_Messages.h"

// AVP
#include "Actor/Vertebrates/Human/AVP_Human_AIAnimStateData.h"
#include "Entity/AVP_Messages.h"
#include "Entity/Controller/AVP_ServerEntity_Squad_Marine.h"
#include "Entity/Locations/Objective/Marine/AVP_ServerEntity_Objective_Marine.h"
#include "Entity/Locations/OrientedNode/GamesceneNode/AVP_ServerNode_GS2Node.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Human/AVP_ServerActor_Human.h"
#include "Entity/PlayerInfo/AVP_ServerEntity_PlayerInfo.h"
#include "Inventory/AVP_Inventory_Group.h"
#include "Inventory/AVP_Inventory_HashIDs.h"
#include "Inventory/AVP_Inventory_Item.h"
#include "Inventory/AVP_Inventory_System.h"
#include "Moves/Vertebrates/Human/AVP_Human_MovesGamescene.h"

// AVP axon
#include "Behaviour/BehaviourTypes/AVP_Behaviour_Types.h"
#include "Core/AVP_PositionEvaluation.h"
#include "Core/AVP_Waypoint_Manager.h"
#include "Core/BrainTypes/AVP_Brain_Actor_Marine.h"
#include "Knowledge/AVP_Knowledge_System_Actor.h"
#include "Knowledge/SenseTypes/AVP_Sense_Vision_Actor.h"
#include "Tactical/AVP_Tactical_System.h"

#include "Entity/GamePlayVolume/AVP_ServerEntity_GamePlayVolume.h"
#include "Entity/Locations/LookAt/AVP_ServerEntity_LookAt.h"

DEBUGVAR float dbg_fLongTimeOut = 5.0f;
DEBUGVAR u_int NUMBER_OF_REAR_LOOKS = 3;

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_HandleObjective :: Constructor
//---------------------------------------------------------------------------
AVP_Behaviour_Marine_HandleObjective::AVP_Behaviour_Marine_HandleObjective( AVP_Brain_Actor_Marine* pxBrain )
: PARENT( pxBrain, AVP_BEHAVIOUR_TYPE_MARINE_HANDLEOBJECTIVE )
, m_pxMarineBrain( pxBrain )
, m_eUpdateState( UNSET )
, m_uObjectiveGuid( ASURA_GUID_UNREGISTERED )
, m_uActionGuid( ASURA_GUID_UNREGISTERED )
, m_bHasStopped( false )
, m_fWaitForPlayerDistance( 20.0f )
, m_bIsOwnActorsObjective( false )
, m_bPlayerArrived( false )
, m_xPosOnRoute( Asura_Maths::ZeroVector3 )
, m_uLeadMarine( ASURA_GUID_UNREGISTERED )
, m_uPosition( 0 )
, m_uRearGuardCounter( 0 )
, m_bDirectionLeft( false )
, m_bOverrideMovementMode( false )
, m_fScanTimer( 0.0f )
, m_fCorpseReactionTimer( 0.0f )
, m_fCorpseCheckTimer( 0.0f )
{
	m_fLookTimeOut = Asura_Random::GetFloat( 1.9f, 3.1f );
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_HandleObjective :: Deconstructor
//---------------------------------------------------------------------------
AVP_Behaviour_Marine_HandleObjective::~AVP_Behaviour_Marine_HandleObjective( void )
{

}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_HandleObjective :: Activate
//---------------------------------------------------------------------------
bool AVP_Behaviour_Marine_HandleObjective::Activate( void )
{
	const bool bParentOk = PARENT::Activate();
	// Put Code Below This Comment

	AVP_ServerActor_Human* pxOurActor = GetOurActor();

	if ( pxOurActor->GetCurrentObjective() )
	{
		m_uObjectiveGuid = pxOurActor->GetCurrentObjectiveGuid();
		m_bIsOwnActorsObjective = true;
	}
	else
	{
		AVP_ServerEntity_Squad_Marine* pxSquad = GetSquad();

		if ( pxSquad && pxSquad->GetCurrentObjective() )
		{
			m_uObjectiveGuid = pxSquad->GetCurrentObjectiveGuid();
			m_bIsOwnActorsObjective = false;
		}
		else
		{
			Asura_Assert( false, "AVP_Behaviour_Marine_HandleObjective :: Activate - we have no Objective." );
			SetFinished();
		}
	}

	m_fWaitForPlayerDistance = 20.0f;

	// Put Code Above This Comment
	return bParentOk;
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_HandleObjective :: Deactivate
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_HandleObjective::Deactivate( void )
{
	if ( !m_bIsOwnActorsObjective )
	{
		AVP_ServerEntity_Squad_Marine* pxSquad = GetSquad();

		if ( pxSquad && ( m_uActionGuid != ASURA_GUID_UNREGISTERED ) )
		{
			pxSquad->ReleaseActionGuid( m_uActionGuid );
		}
	}

	if ( GetMarineBrain()->IsActive() )
	{
		GetTacticalSystem()->SetForceAiming( false );
		GetMarineBrain()->GetTacticalSystem()->SetIsPerformingAction( false );
	}

	// Put Code Above This Comment
	DestroyAllChildren();
	PARENT::Deactivate();
}

DEBUGVAR float dbg_fTurnOffset = 2.0f;

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_HandleObjective :: PerFrameUpdate
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_HandleObjective::PerFrameUpdate( const float fTimeslice )
{
	// first things first - Check The validity of our current objective
	if ( !ValidateObjective() )
	{
		SetFinished();
		return;
	}

	CheckForEarlyOut();
	CheckObjectiveAction();
	CheckForAimingAtThreat();

	switch ( GetUpdateState() )
	{
		//--------------------------------------------------------------

		case MOVING_TO_OBJECTIVE:
		{
			AVP_ServerEntity_Squad_Marine* pxSquad = GetSquad();

			if ( !m_bIsOwnActorsObjective )
			{
				if ( pxSquad && pxSquad->IsAILedPlayersSquad() )
				{
					if ( WaitForPlayer() )
					{
						SwitchToState( WAIT_FOR_PLAYER );
						return;
					}
				}

				if ( pxSquad->GetSquadLeaderGuid() != GetOurActor()->GetGuid() )
				{
					AVP_ServerNode_Objective_Marine* pxObjective = AVP_ServerNode_Objective_Marine::FindEntity( pxSquad->GetCurrentObjectiveGuid() );

					if ( pxObjective )
					{
						if ( pxObjective->Contains( GetOurActor()->GetPosition() ) )
						{
							SwitchToState( WAIT_FOR_OBJECTIVE_COMPLETE );
							return;
						}
					}
				}
			}

			if ( CheckForCorpseResponse( fTimeslice ) )
			{
				return;
			}

			if ( pxSquad && pxSquad->AllMembersHaveValidRoute() && pxSquad->GetCurrentObjective() && !DoesHaveThreat() )
			{
				ScanArea( fTimeslice );

				TestForPause();
			}
			else
			{
				GetMarineBrain()->GetTacticalSystem()->LetRouteSetLookAt( true );
			}

			if ( m_bOverrideMovementMode )
			{
				GetTacticalSystem()->SetMovementSpeed( AVP_Navigation::MM_FAST );
			}
			else
			{
				if ( !m_bIsOwnActorsObjective )
				{
					GetTacticalSystem()->SetMovementSpeed( pxSquad->GetCurrentObjectiveMovementMode() );
				}
				else
				{
					GetTacticalSystem()->SetMovementSpeed( GetOurActor()->GetCurrentObjectiveMovementMode() );
				}
			}

			const AVP_Navigation::NavData& xNavData = GetMarineBrain()->GetNavData();

			if ( !xNavData.m_pusRoute )
			{
				SwitchToState( MOVING_TO_OBJECTIVE );
			}
			else
			{
				if ( GetKnowledgeSystem()->GetAlertLevel() == AVP_Knowledge_System_Actor::PASSIVE_ALERTNESS )
				{
					GetTacticalSystem()->SetIsAllowedToReact( false );
				}
			}

		}break;

		//--------------------------------------------------------------

		case WAIT_FOR_PLAYER:
		{
			if ( !WaitForPlayer() )
			{
				SwitchToState( MOVING_TO_OBJECTIVE );
			}
			else
			{
				AVP_ServerEntity_Squad_Marine* pxSquad = GetSquad();

				if ( pxSquad )
				{
					if ( GetOurActor()->GetGuid() != pxSquad->GetSquadLeaderGuid() )
					{
						// Do some position evaluation
						FormationPositionEvaluation( fTimeslice );
					}
					else
					{
						GetTacticalSystem()->StopMoving();
					}

					if ( GetTacticalSystem()->HasArrivedAtDestination() || GetTacticalSystem()->DidRouteFindFail() )
					{
						AVP_ServerActor_Human* pxSquadMember = GetClosestSquadMemberToPlayer();

						if ( pxSquadMember && pxSquadMember->GetGuid() == GetOurActor()->GetGuid() )
						{
							AVP_ServerActor_Human* pxPlayer = GetPlayer();

							if ( pxPlayer )
							{
								Asura_Vector_3 xDirToPlayer = pxPlayer->GetPosition() - pxSquadMember->GetPosition();
								GetMarineBrain()->SetAimDirection( xDirToPlayer );
								GetMarineBrain()->SetLookAtPosition( pxSquadMember->GetPosition() + ( xDirToPlayer * 5.0f ) );
							}
						}
						else
						{
							GetMarineBrain()->AimForward();
							GetMarineBrain()->LookForward();
						}
					}
				}
			}

		}break;

		//--------------------------------------------------------------

		case WAIT_TO_PERFROM_ACTION:
		{
			// do nothing

		}break;

		//--------------------------------------------------------------

		case MOVE_TO_ACTION:
		{
			//GetMarineBrain()->GetTacticalSystem()->SetIsPerformingAction( true );

			AVP_ServerNode_GS2Node* pxGS2Node = AVP_ServerNode_GS2Node::FindEntity( m_uActionGuid );

			if ( pxGS2Node )
			{
				float fDistanceSq = ( pxGS2Node->GetPosition() - GetOurActor()->GetPosition() ).MagnitudeSquared();
				
				float fStopDistSq = 0.0f;
				float fTurnDistanceSq = 0.0f;
				float fDistanceToStop = GetDistanceToStop();

				if ( fDistanceToStop == 0.0f )
				{
					fDistanceToStop = 2.6f;
				}
				
				fStopDistSq = fDistanceToStop * fDistanceToStop;
				fTurnDistanceSq = ( fDistanceToStop + dbg_fTurnOffset ) * ( fDistanceToStop + dbg_fTurnOffset );

				if ( fDistanceSq < fStopDistSq )
				{
					SwitchToState( IN_MOVE );
				}
				else if ( fDistanceSq < fTurnDistanceSq )
				{
					Asura_Matrix_3x3 xNodeOri; pxGS2Node->GetOrientation( xNodeOri );
					Asura_Vector_3 xNodeFwd( xNodeOri.e13, xNodeOri.e23, xNodeOri.e33 );

					GetTacticalSystem()->LetRouteSetLookAt( false );
					GetMarineBrain()->SetAimDirection( xNodeFwd );
					GetMarineBrain()->SetLookAtPosition( GetOurActor()->GetPosition() + xNodeFwd );
				}
			}

			if ( !m_bIsOwnActorsObjective )
			{
				AVP_ServerEntity_Squad_Marine* pxSquad = GetSquad();

				if ( pxSquad )
				{
					AVP_Navigation::MovementMode uMoveMode = (AVP_Navigation::MovementMode)pxSquad->GetCurrentObjectiveMovementMode();

					GetTacticalSystem()->SetMovementSpeed( uMoveMode );
					GetTacticalSystem()->OverrideSlowDownForThisDestination();
				}
			}
			else
			{
				GetTacticalSystem()->SetMovementSpeed( GetOurActor()->GetCurrentObjectiveMovementMode() );
				GetTacticalSystem()->OverrideSlowDownForThisDestination();
			}

			const AVP_Navigation::NavData& xNavData = GetMarineBrain()->GetNavData();

			if ( !xNavData.m_pusRoute && m_uActionGuid != ASURA_GUID_UNREGISTERED )
			{
				SwitchToState( MOVE_TO_ACTION );
			}

		}break;

		//--------------------------------------------------------------

		case PERFROM_ACTION:
		{
			CheckForCompletion();

		}break;

		//--------------------------------------------------------------

		case ACTION_COMPLETE:
		{
			SetFinished();

		}break;

		//--------------------------------------------------------------

		case WAIT_FOR_OBJECTIVE_COMPLETE:
		{
			AVP_ServerEntity_Squad_Marine* pxSquad = GetSquad();

			if ( pxSquad && pxSquad->GetNumSquadMembers() > 1 )
			{
				FormationPositionEvaluation( fTimeslice );
			}

		}break;

		//--------------------------------------------------------------

		case FORMATION_ADVANCE:
		{

		}break;

		//--------------------------------------------------------------

		case FORMATION_RETREAT:
		{

		}break;

		case CORPSE_REACTION:
		{
			GetTacticalSystem()->StopMoving();
			GetTacticalSystem()->SetIsAllowedToReact( true );

			m_fCorpseReactionTimer += fTimeslice;

			if ( m_fCorpseReactionTimer > 10.0f )
			{
				SwitchToState( RESTART );
			}

		}break;

		//--------------------------------------------------------------

		case RESTART:
		{

		}break;

		default:
		{

		}
	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_HandleObjective :: CheckForCorpseResponse
//---------------------------------------------------------------------------
bool AVP_Behaviour_Marine_HandleObjective::CheckForCorpseResponse( const float fTimeSlice )
{
	AVP_ServerActor_Human* pxOurActor = GetOurActor();

	m_fCorpseCheckTimer -= fTimeSlice;

	if ( m_fCorpseCheckTimer <= 0.0f )
	{
		m_fCorpseCheckTimer = 12.0f;

		bool bValidToCheck = true;

		if ( !m_bIsOwnActorsObjective )
		{
			if ( pxOurActor->GetCurrentObjectiveThreatAction() == AVP_ServerNode_Objective_Marine::NO_ACTION )
			{
				bValidToCheck = false;
			}
		}
		else
		{
			AVP_ServerEntity_Squad_Marine* pxSquad = GetSquad();

			if ( pxSquad && pxSquad->GetCurrentObjectiveThreatAction() == AVP_ServerNode_Objective_Marine::NO_ACTION )
			{
				bValidToCheck = false;
			}
		}

		if ( bValidToCheck )
		{
			for ( Asura_Hashed_List_It < AVP_ServerActor_Human > it( AVP_ServerActor_Human::GetList() ); !it.Done(); it.Next() )
			{
				AVP_ServerActor_Human* pxHuman = it.GetCurrent();

				if ( pxHuman && pxOurActor->GetGuid() != pxHuman->GetGuid() && pxHuman->IsDead() && !pxHuman->IsPlayer() && !pxHuman->IsADeadBody() && pxHuman->IsEntityActive() && ( AVP_Tactical_System::GetAllegiance( *pxOurActor, *pxHuman ) != AVP_ALLEGIANCE_ENEMY ) )
				{
					float fDistanceSq = ( pxOurActor->GetPosition() - pxHuman->GetPosition() ).MagnitudeSquared();

					if ( fDistanceSq < 10.0f )
					{
						if ( GetKnowledgeSystem()->GetAlertLevel() != AVP_Knowledge_System_Actor::AGGRESSIVE_ALERTNESS )
						{
							GetKnowledgeSystem()->SetAlertLevel( AVP_Knowledge_System_Actor::AGGRESSIVE_ALERTNESS );
							GetKnowledgeSystem()->ResetAlertTimer();
							SwitchToState( CORPSE_REACTION );
							return true;
						}
					}
				}
			}
		}
	}

	return false;
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_HandleObjective :: GetDistanceToStop
//---------------------------------------------------------------------------
float AVP_Behaviour_Marine_HandleObjective::GetDistanceToStop(void)
{
	AVP_ServerActor_Human&			xOurActor			= *GetMarineBrain()->GetHumanActor();
	AVP_Tactical_System*			pxTactical			= GetMarineBrain()->GetTacticalSystem();
	AVP_Human_AIAnimStateMachine*	pxHumanStateMachine	= static_cast<AVP_Human_AIAnimStateMachine*>(xOurActor.GetAIAnimStateMachine());

	if (pxTactical->ShouldMoveCrouched())
	{
		return pxHumanStateMachine->GetDistanceRequired(AVP_Human_AIAnimStateMachine::HUMAN_STATE_CROUCH_MOVE, AVP_Human_AIAnimStateMachine::HUMAN_STATE_CROUCH_IDLE);
	}
	else
	{
		switch(pxTactical->GetMovementSpeed())
		{
			case AVP_Navigation::MM_SLOW:
			{
				return pxHumanStateMachine->GetDistanceRequired(AVP_Human_AIAnimStateMachine::VERTEBRATE_STATE_MOVE_SLOW, AVP_Human_AIAnimStateMachine::ACTOR_STATE_IDLE);
			}

			case AVP_Navigation::MM_FAST:
			{
				return pxHumanStateMachine->GetDistanceRequired(AVP_Human_AIAnimStateMachine::VERTEBRATE_STATE_MOVE_FAST, AVP_Human_AIAnimStateMachine::ACTOR_STATE_IDLE);
			}

			default:
			case AVP_Navigation::MM_MEDIUM:
			{
				return pxHumanStateMachine->GetDistanceRequired(AVP_Human_AIAnimStateMachine::VERTEBRATE_STATE_MOVE_MEDIUM, AVP_Human_AIAnimStateMachine::ACTOR_STATE_IDLE);
			}
		}
	}

}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_HandleObjective :: SwitchToState
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_HandleObjective::TestForPause( void )
{
	bool bShouldStop = false;

	if ( m_uPosition == MIDDLE_MAN )
	{
		AVP_ServerEntity_Squad_Marine* pxSquad = GetSquad();

		if ( pxSquad )
		{
			for ( u_int u = 0; u < pxSquad->GetNumSquadMembers(); ++u )
			{
				AVP_ServerActor_Human* pxSquadMember = AVP_ServerActor_Human::FindEntity( pxSquad->GetMemberGuid( u ) );

				if ( pxSquadMember && !pxSquadMember->IsDead() && pxSquadMember->IsEntityActive() && pxSquadMember->GetGuid() != GetMarineBrain()->GetOwnerGuid() )
				{
					float fDistanceSq = ( GetOurActor()->GetPosition() - pxSquadMember->GetPosition() ).MagnitudeSquared();

					if ( fDistanceSq < 6.0f && !pxSquadMember->IsIdle() )
					{						
						bShouldStop = true;
					}
				}
			}
		}
	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_HandleObjective :: SwitchToState
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_HandleObjective::SwitchToState( UpdateState eState )
{
	switch (GetUpdateState())
	{
		//--------------------------------------------------------------
		case MOVING_TO_OBJECTIVE:
		case WAIT_TO_PERFROM_ACTION:
		case MOVE_TO_ACTION:
		case IN_MOVE:
		{
			GetMarineBrain()->GetTacticalSystem()->SetIsPerformingAction( false );
		}break;
		case PERFROM_ACTION:
		case ACTION_COMPLETE:
		case UNSET:
		case WAIT_FOR_PLAYER:
		case WAIT_FOR_OBJECTIVE_COMPLETE:
		case FORMATION_ADVANCE:
		case FORMATION_RETREAT:
		case CORPSE_REACTION:
		case RESTART:
		default:
		{

		}
	}

	SetUpdateState( eState );

	switch (GetUpdateState())
	{
		//--------------------------------------------------------------

		case MOVING_TO_OBJECTIVE:
		{
			SetRouteToObjective();
			m_fWaitForPlayerDistance = 20.0f;
			m_fScanTimer = 0.0f;
			m_uRearGuardCounter = NUMBER_OF_REAR_LOOKS + 1;
			m_bOverrideMovementMode = false;
			m_fCorpseCheckTimer = 30.0f;

		}break;

		//--------------------------------------------------------------

		case WAIT_FOR_PLAYER:
		{
			GetTacticalSystem()->StopMoving();
			m_fPosEvalUpdateTimer = dbg_fLongTimeOut;
			m_fWaitForPlayerDistance = 15.0f;
		}break;

		//--------------------------------------------------------------

		case WAIT_TO_PERFROM_ACTION:
		{
			RequestAction();

		}break;

		//--------------------------------------------------------------

		case MOVE_TO_ACTION:
		{
			GetMarineBrain()->GetTacticalSystem()->SetIsPerformingAction( true );
			SetRouteToActionGameScene();

		}break;

		//--------------------------------------------------------------

		case IN_MOVE:
		{
			AVP_Human_AIAnimStateMachine* pxStateMachine = static_cast<AVP_Human_AIAnimStateMachine*>( GetOurActor()->GetAIAnimStateMachine() );

			if ( pxStateMachine )
			{
				if ( !pxStateMachine->IsInMove() )
				{
					GetMarineBrain()->GetTacticalSystem()->SetIsPerformingAction( true );
					SendMountGamesceneMessage();
				}
			}

		}break;

		//--------------------------------------------------------------

		case PERFROM_ACTION:
		{
			
		}break;
		
		//--------------------------------------------------------------

		case ACTION_COMPLETE:
		{

		}break;

		//--------------------------------------------------------------

		case WAIT_FOR_OBJECTIVE_COMPLETE:
		{
			m_fPosEvalUpdateTimer = dbg_fLongTimeOut;
		}break;

		//--------------------------------------------------------------

		case FORMATION_ADVANCE:
		{

		}break;

		//--------------------------------------------------------------
		case CORPSE_REACTION:
		{
			m_fCorpseReactionTimer = 0.0f;

		}break;

		//--------------------------------------------------------------
		case FORMATION_RETREAT:
		{

		}break;

		//--------------------------------------------------------------

		case RESTART:
		{

		}break;

		default:
		{

		}
	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_HandleObjective :: HandleMessage
//---------------------------------------------------------------------------
bool AVP_Behaviour_Marine_HandleObjective::HandleMessage( const Asura_Message* pxMessage )
{
	switch ( pxMessage->GetMsgID() )
	{
		case AVP_MSGID_PERFORM_AI_OBJECTIVE_ACTION:
		{
			const AVP_Message_PerformObjectiveAction* pxMsgData = static_cast<const AVP_Message_PerformObjectiveAction*>(pxMessage->GetData());

			if ( pxMsgData )
			{
				m_uActionGuid = pxMsgData->GetActionGuid();
				SwitchToState( MOVE_TO_ACTION );
			}

			return true;
		}

		case ASURA_MSG_MOVECHAINCOMPLETE:
		{
			const Asura_Msg_MoveChainComplete* pxData = static_cast< const Asura_Msg_MoveChainComplete* >( pxMessage->GetData() );

			if ( pxData && pxData->GetInitialMoveID() == AVP_Move_Human_MountGamescene::GetStaticHash() )
			{
				GetMarineBrain()->GetHumanActor()->SetInitialGamesceneGuid( m_uActionGuid );

				Asura_Message* pxMessage = Asura_Network::CreateMessage( ASURA_MSG_AXON_GS2_TELEPORT_TO_NODE, sizeof(Axon_Msg_GS2_Teleport_To_Node) );

				if ( pxMessage )
				{
					Axon_Msg_GS2_Teleport_To_Node* pxData = static_cast<Axon_Msg_GS2_Teleport_To_Node*>( pxMessage->GetData() );

					if ( pxData )
					{
						pxData->m_uNodeGuid = m_uActionGuid;
					}

					pxMessage->SetToGuid( GetMarineBrain()->GetOwnerGuid() );
					pxMessage->SetFromGuid( GetMarineBrain()->GetOwnerGuid() );
					
					Asura_Network::Send( pxMessage );
				}

				SwitchToState( PERFROM_ACTION );
			}

			return true;
		}

		default:
		{
			return PARENT::HandleMessage( pxMessage );
		}
	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_HandleObjective :: GetClosestSquadMemberToPlayer
//---------------------------------------------------------------------------
AVP_ServerActor_Human* AVP_Behaviour_Marine_HandleObjective::GetClosestSquadMemberToPlayer( void )
{
	AVP_ServerEntity_Squad_Marine*	pxSquad		= GetSquad();
	AVP_ServerActor_Human*			pxPlayer	= GetPlayer();

	float					fBestDistance	= Asura_Maths::LargeFloat;
	AVP_ServerActor_Human*	pxBestActor		= NULL;

	if ( pxSquad && pxPlayer )
	{
		for ( u_int u = 0; u < pxSquad->GetNumSquadMembers(); ++u )
		{
			AVP_ServerActor_Human* pxMember = AVP_ServerActor_Human::FindEntity( pxSquad->GetMemberGuid( u ) );

			if ( pxMember )
			{
				float fDistance = ( pxPlayer->GetPosition() - pxMember->GetPosition() ).MagnitudeSquared();

				if ( fDistance < fBestDistance )
				{
					fBestDistance = fDistance;
					pxBestActor = pxMember;
				}
			}
		}
	}

	return pxBestActor;
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_HandleObjective :: PositionEvaluation
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_HandleObjective::FormationPositionEvaluation( const float fTimeslice )
{
	m_fPosEvalUpdateTimer += fTimeslice;

	if ( m_fPosEvalUpdateTimer > dbg_fLongTimeOut )
	{
		Asura_Guid uCombatZoneGuid = GetMarineBrain()->GetCombatZoneGuid();

		AVP_ServerActor_Human* pxLeader = GetSquadLeader();

		if ( pxLeader && !pxLeader->IsDead() )
		{
			const AVP_Navigation::NavData& xNavData = GetMarineBrain()->GetNavData();

			u_short usNearestWPID = ASURA_INVALID_WAYPOINT_ID;
			AVP_Navigation::TestParams xTestparams;
			xTestparams.m_xNormal			= -pxLeader->GetCurrentGravityDirection();
			xTestparams.m_uPathfinderFlags	= AVP_Navigation::AVP_AI_PATHFINDER_FLAG_HUMANOID;
			xTestparams.m_usZone			= xNavData.m_usCurrentZone;
			
			AVP_Navigation::GetNearestWaypointID( pxLeader->GetPosition(), usNearestWPID, xTestparams );

			if ( usNearestWPID == ASURA_INVALID_WAYPOINT_ID )
			{
				return;
			}

			AVP_Navigation::AVP_PosEvalParams	xPosEvalParams( GetMarineBrain()->GetOwnerGuid(), 0, 30.0f, 10.0f, 15.0f, uCombatZoneGuid );
			AVP_Navigation::PosEvalFunctor		pxPosEval( usNearestWPID, 25.0f, xPosEvalParams, GetFormationScoreForWaypoint);
			AVP_Navigation::SearchWaypoints(AVP_Navigation::AVP_AI_PATHFINDER_FLAG_HUMANOID, &pxPosEval);
			FindBestWaypoint(&pxPosEval);
		}
	}

	m_fPosEvalUpdateTimer = 0.0f;
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_SquadMember :: GetFormationScoreForWaypoint
//---------------------------------------------------------------------------
u_int AVP_Behaviour_Marine_HandleObjective::GetFormationScoreForWaypoint( AVP_Navigation::AVP_PosEvalParams xPosEvalParams )
{
	AVP_ServerActor_Human* pxHuman = AVP_ServerActor_Human::FindEntity( xPosEvalParams.m_uActorGuid );

	u_int uScore = 0;

	// this should always be valid as he has just requested a position evaluation
	if ( !pxHuman ) { return uScore; }

	// Waypoint Proximity
	uScore += AVP_PositionEvaluation::GetProximityScore( pxHuman, xPosEvalParams.m_uTestWPID, xPosEvalParams.m_fEvaluationRange );

	// Waypoint Occupied by ally
	uScore += AVP_PositionEvaluation::GetHumanoidPositionOccupiedByAllyScore( pxHuman, xPosEvalParams.m_uTestWPID, 60 );

	// Always want a position close to the squad leader
	uScore += AVP_PositionEvaluation::GetPositionCloseToSquadLeaderScore( pxHuman, xPosEvalParams.m_uTestWPID, 60 );

	// Stops marines moving around to much.
	uScore += AVP_PositionEvaluation::GetPositionIsInFrontOfSquadScore(pxHuman, xPosEvalParams.m_uTestWPID, 40 );

	return uScore;
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_SquadMember :: FindBestWaypoint
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_HandleObjective::FindBestWaypoint( AVP_Navigation::PosEvalFunctor* pxPosEval )
{
	u_int uMovementMode = AVP_Navigation::MM_SLOW;

	if ( m_bIsOwnActorsObjective )
	{
		if ( GetOurActor()->GetCurrentObjective() )
		{
			uMovementMode = GetOurActor()->GetCurrentObjectiveMovementMode();
		}
	}
	else
	{
		AVP_ServerEntity_Squad_Marine* pxSquad = GetSquad();

		if ( pxSquad && pxSquad->GetCurrentObjective() )
		{
			uMovementMode = pxSquad->GetCurrentObjectiveMovementMode();
		}
	}

	u_int uBestWaypointID = pxPosEval->GetBestWaypointID();

	if ( uBestWaypointID != ASURA_INVALID_WAYPOINT_ID )
	{
		Asura_AI_NavVertex*	pxWaypoint	= Asura_AI_Navigation_System::GetWaypoint( uBestWaypointID );

		if ( pxWaypoint )
		{
			GetTacticalSystem()->SetDestination( pxWaypoint->m_xPosition );
			GetTacticalSystem()->SetAtDestinationRange( ONE_METER );
			GetTacticalSystem()->LetRouteSetLookAt( true );
			GetTacticalSystem()->SetMovementSpeed( uMovementMode );
		}
	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_SquadMember :: GetSquadLeader
//---------------------------------------------------------------------------
AVP_ServerActor_Human* AVP_Behaviour_Marine_HandleObjective::GetSquadLeader()
{
	AVP_ServerEntity_Squad_Marine* pxSquad = GetSquad();

	if ( !pxSquad ) { return NULL; }

	Asura_Guid uLeaderGuid = pxSquad->GetSquadLeaderGuid();

	if ( uLeaderGuid == ASURA_GUID_UNREGISTERED ) { return NULL; }

	AVP_ServerActor_Human* pxSquadLeader = AVP_ServerActor_Human::FindEntity( uLeaderGuid );

	return pxSquadLeader;
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_HandleObjective :: WaitForPlayer
//---------------------------------------------------------------------------
bool AVP_Behaviour_Marine_HandleObjective::WaitForPlayer( void )
{
	AVP_ServerEntity_Squad_Marine* pxSquad = GetSquad();

	if ( pxSquad && pxSquad->IsAILedPlayersSquad() )
	{
		AVP_ServerEntity_PlayerInfo* pxPlayerInfo = AVP_ServerEntity_PlayerInfo::FindEntity( 1000 );

		if ( pxPlayerInfo )
		{
			if ( pxPlayerInfo->HasPlayerArrivedAtCurrentAIObjective() )
			{
				return false;
			}

			const AVP_Navigation::NavData& xPlayersNavData = pxPlayerInfo->GetAINavData();

			if ( xPlayersNavData.m_pusRoute && xPlayersNavData.m_usDestinationWPID != ASURA_INVALID_WAYPOINT_ID )
			{
				Asura_Vector_3			xPositionOnRoute( Asura_Maths::ZeroVector3 );
				u_short					usLeadersRouteIndex	= 0;
				const Asura_Vector_3&	xOurPosition( GetOurActor()->GetPosition() );

				AVP_Navigation::FindNearestPointOnOtherRoute( xPlayersNavData, xOurPosition, xPositionOnRoute, usLeadersRouteIndex );

				float fDistToRoute			= ( xPositionOnRoute - xOurPosition ).Magnitude();
				float fOurRouteLen			= AVP_Navigation::CalculateRouteDistance( xPlayersNavData, xPositionOnRoute, usLeadersRouteIndex ) + fDistToRoute;
				float fPlayersRouteLen		= pxPlayerInfo->GetCurrentAIRouteDistance();
				
				if ( usLeadersRouteIndex <= 1 )
				{
					if ( xPlayersNavData.m_pusRoute )
					{
						Asura_Vector_3 xFristWpPos( Asura_AI_Navigation_System::GetWaypoint( xPlayersNavData.m_pusRoute[ usLeadersRouteIndex ] )->GetPosition() );
						float fDistanceToFirstWP = ( xFristWpPos - GetOurActor()->GetPosition() ).Magnitude();

						if ( fDistanceToFirstWP > 10.0f )
						{
							fOurRouteLen += fDistanceToFirstWP;
						}
					}
				}

				if ( fOurRouteLen > fPlayersRouteLen )
				{
					if (fOurRouteLen - fPlayersRouteLen > 30.0f )
					{
						m_bOverrideMovementMode = true;
					}
					else
					{
						m_bOverrideMovementMode = false;
					}

					return false;
				}

				m_bOverrideMovementMode = false;

				AVP_ServerEntity_Actor* pxPlayer		= AVP_ServerEntity_Actor::FindEntity( pxPlayerInfo->GetPuppetGuid() );
				AVP_ServerActor_Human*	pxSquadLeader	= GetSquadLeader();

				if (pxPlayer && pxSquadLeader)
				{
					float fDistanceSq = ( pxPlayer->GetPosition() - pxSquadLeader->GetPosition() ).MagnitudeSquared();

					if (fDistanceSq >= ( m_fWaitForPlayerDistance * m_fWaitForPlayerDistance) )
					{
						return true;
					}
				}
			}
		}
	}

	return false;
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_HandleObjective :: GetPlayer
//---------------------------------------------------------------------------
AVP_ServerActor_Human* AVP_Behaviour_Marine_HandleObjective::GetPlayer( void )
{
	AVP_ServerEntity_PlayerInfo* pxPlayerInfo = AVP_ServerEntity_PlayerInfo::FindEntity( 1000 );

	if ( pxPlayerInfo )
	{
		AVP_ServerActor_Human* pxPlayer = AVP_ServerActor_Human::FindEntity( pxPlayerInfo->GetPuppetGuid() );

		if (pxPlayer)
		{
			return pxPlayer;
		}
	}

	return NULL;
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_HandleObjective :: ValidateObjective
//---------------------------------------------------------------------------
bool AVP_Behaviour_Marine_HandleObjective::ValidateObjective( void )
{
	if ( m_bIsOwnActorsObjective )
	{
		if ( GetOurActor()->GetCurrentObjective() )
		{
			if ( GetOurActor()->GetCurrentObjectiveGuid() == m_uObjectiveGuid )
			{
				return true;
			}
		}
	}
	else
	{
		AVP_ServerEntity_Squad_Marine* pxSquad = GetSquad();

		if ( pxSquad && pxSquad->GetCurrentObjective() )
		{
			if ( pxSquad->GetCurrentObjectiveGuid() == m_uObjectiveGuid )
			{
				return true;
			}
		}
	}

	return false;
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_HandleObjective :: CheckObjectiveAction
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_HandleObjective::CheckObjectiveAction( void )
{
	int		uObjectiveState = -1;
	u_int	uObjectiveType	= AVP_ServerNode_Objective_Marine::MOVE_SLOW_STOP_TO_ENGAGE;
	u_int	uNumMembers		= 0;

	if (m_bIsOwnActorsObjective)
	{
		if ( GetOurActor()->GetCurrentObjective() )
		{
			uObjectiveState = GetOurActor()->GetCurrentObjectiveState();
			uObjectiveType = GetOurActor()->GetCurrentObjectiveType();
			uNumMembers = 1;
		}
	}
	else
	{
		AVP_ServerEntity_Squad_Marine*	pxSquad	= GetSquad();

		if ( pxSquad && pxSquad->GetCurrentObjective() )
		{
			uObjectiveState = pxSquad->GetCurrentObjectiveState();
			uObjectiveType = pxSquad->GetCurrentObjectiveType();
			uNumMembers = pxSquad->GetNumSquadMembers();
		}
	}

	if ( GetUpdateState() == CORPSE_REACTION )
	{
		return;
	}

	switch ( uObjectiveState )
	{
		//--------------------------------------------------------------

		case AVP_ServerNode_Objective_Marine::NOT_STARTED:
		{
			if ( uNumMembers > 1 && ( uObjectiveType == AVP_ServerNode_Objective_Marine::ADVANCE_MEDIUM || uObjectiveType == AVP_ServerNode_Objective_Marine::ADVANCE_SLOW ) )
			{
				SwitchToState( FORMATION_ADVANCE );
			}
			else if ( uNumMembers > 1 && ( uObjectiveType == AVP_ServerNode_Objective_Marine::RETREAT_MEDIUM || uObjectiveType == AVP_ServerNode_Objective_Marine::RETREAT_SLOW ) )
			{
				SwitchToState( FORMATION_RETREAT );
			}
			else
			{
				SwitchToState( MOVING_TO_OBJECTIVE );
			}

		}break;

		//--------------------------------------------------------------

		case AVP_ServerNode_Objective_Marine::MOVING_TO_OBJECTIVE:
		{
			if ( GetUpdateState() != MOVING_TO_OBJECTIVE && GetUpdateState() != WAIT_FOR_PLAYER && GetUpdateState() != WAIT_FOR_OBJECTIVE_COMPLETE )
			{
				SwitchToState( MOVING_TO_OBJECTIVE );
			}

		}break;

		//--------------------------------------------------------------

		case AVP_ServerNode_Objective_Marine::ARRIVED_AT_OBJECTIVE:
		{
			if ( GetUpdateState() == MOVING_TO_OBJECTIVE )
			{
				SwitchToState( WAIT_TO_PERFROM_ACTION );
			}
			else if ( GetUpdateState() == UNSET || GetUpdateState() == WAIT_FOR_OBJECTIVE_COMPLETE )
			{
				RequestAction();
			}

		}break;

		//--------------------------------------------------------------

		case AVP_ServerNode_Objective_Marine::WAITING_FOR_COMPLETION:
		{
			if ( m_uActionGuid == ASURA_GUID_UNREGISTERED && GetUpdateState() != WAIT_FOR_OBJECTIVE_COMPLETE )
			{
				RequestAction();
			}

		}break;

		//--------------------------------------------------------------

		case AVP_ServerNode_Objective_Marine::COMPLETE:
		{
			SetFinished();

		}break;

		//--------------------------------------------------------------

		default:
		{
			Asura_Assert( false, " AVP_Behaviour_Marine_HandleObjective::CheckObjectiveAction - Objective state is invalid, things are not going to work!" );

		}break;
	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_HandleObjective :: SetRouteToObjective
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_HandleObjective::SetRouteToObjective( bool bIgnoreSettingRouteLookAt /*= false*/, bool bSetOverrideMoverDelay /*= false*/ )
{
	u_int uMovementMode		= AVP_Navigation::MM_SLOW;

	if ( m_bIsOwnActorsObjective )
	{
		if ( GetOurActor()->GetCurrentObjective() )
		{
			uMovementMode = GetOurActor()->GetCurrentObjectiveMovementMode();

			if ( GetOurActor()->GetCurrentObjectiveState() == AVP_ServerNode_Objective_Marine::NOT_STARTED )
			{
				GetOurActor()->GetCurrentObjective()->SetObjectiveState(AVP_ServerNode_Objective_Marine::MOVING_TO_OBJECTIVE);
			}
		}
	}
	else
	{
		AVP_ServerEntity_Squad_Marine* pxSquad = GetSquad();

		if ( pxSquad && pxSquad->GetCurrentObjective() )
		{
			uMovementMode = pxSquad->GetCurrentObjectiveMovementMode();
			pxSquad->MemberIsMovingToObjective();
		}
	}


	AVP_ServerNode_Objective_Marine* pxObjective = AVP_ServerNode_Objective_Marine::FindEntity( m_uObjectiveGuid );

	if ( pxObjective )
	{
		if ( bSetOverrideMoverDelay )
		{
			GetTacticalSystem()->OverrideStartMovementDelay();
		}

		if (bIgnoreSettingRouteLookAt)
		{
			GetTacticalSystem()->LetRouteSetLookAt( true );
		}

		GetTacticalSystem()->SetMovementSpeed( uMovementMode );
		GetTacticalSystem()->SetDestination( pxObjective->GetPosition() );
		GetTacticalSystem()->SetAtDestinationRange( 1.0f );
		GetTacticalSystem()->OverrideSlowDownForThisDestination();
	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_HandleObjective :: RequestAction
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_HandleObjective::RequestAction( void )
{
	if ( m_bIsOwnActorsObjective )
	{
		// Request Action
		Asura_Guid uLastActionGuid = m_uActionGuid;
		m_uActionGuid = ASURA_GUID_UNREGISTERED;
		
		GetOurActor()->RequestAction( uLastActionGuid );
	}
	else
	{
		AVP_ServerEntity_Squad_Marine* pxSquad = GetSquad();

		if ( pxSquad )
		{
			Asura_Guid uLastActionGuid = m_uActionGuid;
			m_uActionGuid = ASURA_GUID_UNREGISTERED;
			if (pxSquad->RequestAction( GetMarineBrain()->GetOwnerGuid(), uLastActionGuid ))
			{
				pxSquad->MemberHasRequestedAction();
			}
			else
			{
				SwitchToState(WAIT_FOR_OBJECTIVE_COMPLETE);
			}
		}
	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_HandleObjective :: SetRouteToActionGameScene
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_HandleObjective::SetRouteToActionGameScene( void )
{
	u_int uMovementMode = AVP_Navigation::MM_SLOW;

	if ( m_bIsOwnActorsObjective )
	{
		if ( GetOurActor()->GetCurrentObjective() )
		{
			uMovementMode = GetOurActor()->GetCurrentObjectiveMovementMode();
		}
	}
	else
	{
		AVP_ServerEntity_Squad_Marine* pxSquad = GetSquad();

		if ( pxSquad && pxSquad->GetCurrentObjective() )
		{
			uMovementMode = pxSquad->GetCurrentObjectiveMovementMode();
		}
	}

	if ( m_uActionGuid != ASURA_GUID_UNREGISTERED )
	{
		AVP_ServerNode_GS2Node* pxGS2Node = AVP_ServerNode_GS2Node::FindEntity( m_uActionGuid );

		if ( pxGS2Node )
		{
			GetTacticalSystem()->OverrideSlowDownForThisDestination();
			GetTacticalSystem()->SetMovementSpeed( uMovementMode );
			GetTacticalSystem()->SetDestination( pxGS2Node->GetPosition() );
			GetTacticalSystem()->LetRouteSetLookAt( true );
			GetTacticalSystem()->SetAtDestinationRange( 0.75f );
		}
	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_HandleObjective :: SetRouteToActionGameScene
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_HandleObjective::SendMountGamesceneMessage()
{
	Asura_Message* pxMessage = Asura_Network::CreateMessage( ASURA_MSG_MOVE_START, sizeof( AVP_Message_Human_MountGamescene ) );

	if ( pxMessage )
	{
		AVP_Message_Human_MountGamescene*	pxData		= static_cast< AVP_Message_Human_MountGamescene* >( pxMessage->GetData() );
		AVP_ServerNode_GS2Node*				pxGS2Node	= AVP_ServerNode_GS2Node::FindEntity( m_uActionGuid );

		if ( pxGS2Node )
		{
			Asura_Matrix_3x3 xNodeOri;
			pxGS2Node->GetOrientation(xNodeOri);

			pxMessage->SetFromGuid( GetMarineBrain()->GetOwnerGuid() );
			pxMessage->SetToGuid( GetMarineBrain()->GetOwnerGuid() );
			pxData->ClearFlags();
			pxData->SetMoveID( AVP_Move_Human_MountGamescene::GetStaticHash() );
			pxData->SetDestinationPosition( pxGS2Node->GetPosition() );
			pxData->SetDestinationOrientation( xNodeOri );
			pxData->SetAnimBlendTemplateBlueprintHash( GetMountGamesceneAnimBlendTemplateBlueprintHash() );

			AVP_ServerActor_Human* xOurActor = GetMarineBrain()->GetHumanActor();

			if ( xOurActor->GetAIAnimStateMachine() )
			{
				xOurActor->GetAIAnimStateMachine()->RequestInMove( pxMessage, AVP_Human_AIAnimStateMachine::ACTOR_STATE_IDLE );
				GetTacticalSystem()->StopMoving();
			}
			else
			{
				Asura_Network::DestroyMessage(pxMessage);
			}
		}
		else
		{
			Asura_Network::DestroyMessage(pxMessage);
		}
	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine :: GetMountGamesceneAnimation
//---------------------------------------------------------------------------
Asura_Hash_ID AVP_Behaviour_Marine_HandleObjective::GetMountGamesceneAnimBlendTemplateBlueprintHash(void)
{
	const AVP_ServerActor_Human* pxHuman = GetMarineBrain()->GetHumanActor();
	
	const AVP_Inventory_System* pxInvSys = pxHuman->GetInventorySystem();
	const AVP_Inventory_Group* pxWeaponGroup = pxInvSys != NULL ? pxInvSys->GetGroup( uAVP_INVENTORY_GROUPHASH_WEAPON ) : NULL;
	Asura_Assert( pxWeaponGroup != NULL, "Unable to get wepaon group" );
	if ( pxWeaponGroup != NULL )
	{
		const AVP_Inventory_Item* pxItem = pxWeaponGroup->GetCurrentInventoryItem();
		if ( pxItem )
		{
			if ( GetTacticalSystem()->GetMovementSpeed() == AVP_Navigation::MM_SLOW )
			{
				const AVP_Actor_AnimStateData* pxStateData = AVP_Human_AIAnimStateData::GetAnimStateData( pxHuman->GetAnimStateBlueprintType(), pxItem->GetBlueprintHash(), AVP_Human_AIAnimStateMachine::VERTEBRATE_STATE_MOVE_SLOW_TO_IDLE, pxHuman->IsAiming(), pxHuman->GetCoverType() );
				if ( pxStateData )
				{
					return pxStateData->GetAnimBlendTemplateBlueprintHash();
				}
			}
			else
			{
				const AVP_Actor_AnimStateData* pxStateData = AVP_Human_AIAnimStateData::GetAnimStateData( pxHuman->GetAnimStateBlueprintType(), pxItem->GetBlueprintHash(), AVP_Human_AIAnimStateMachine::VERTEBRATE_STATE_MOVE_MEDIUM_TO_IDLE, pxHuman->IsAiming(), pxHuman->GetCoverType() );
				if ( pxStateData )
				{
					return pxStateData->GetAnimBlendTemplateBlueprintHash();
				}
			}
		}
	}

	// else
	return ASURA_HASH_ID_UNSET;
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_HandleObjective :: CheckForCompletion
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_HandleObjective::CheckForCompletion( void )
{
	bool bIgnoreThreat = false;

	if ( m_bIsOwnActorsObjective )
	{
		bIgnoreThreat = ( GetOurActor()->GetCurrentObjectiveThreatAction() == AVP_ServerNode_Objective_Marine::NO_ACTION ) ? true : false;
	}
	else
	{
		AVP_ServerEntity_Squad_Marine* pxSquad = GetSquad();

		if ( pxSquad )
		{
			bIgnoreThreat = ( pxSquad->GetCurrentObjectiveThreatAction() == AVP_ServerNode_Objective_Marine::NO_ACTION ) ? true : false;
		}
	}

	if ( !bIgnoreThreat && ( DoesHaveThreat() || DoesHaveAlert() ) )
	{
		return;
	}

	if (m_bIsOwnActorsObjective)
	{
		if ( GetOurActor()->GetCurrentObjective() )
		{
			AVP_ServerNode_Objective_Marine* pxObjective = AVP_ServerNode_Objective_Marine::FindEntity( GetOurActor()->GetCurrentObjectiveGuid() );

			if ( pxObjective )
			{
				if ( pxObjective->GetObjectiveCompleteOnMessageOnly() == 0 )
				{
					// no looping
					GetOurActor()->GetCurrentObjective()->SetObjectiveState( AVP_ServerNode_Objective_Marine::COMPLETE );
					SetFinished();
				}
				else
				{
					// looping
					RequestAction();
				}
			}
		}
	}
	else
	{
		AVP_ServerEntity_Squad_Marine* pxSquad = GetSquad();

		if ( !pxSquad ) { return; }

		if ( pxSquad->GetCurrentObjective() )
		{
			AVP_ServerNode_Objective_Marine* pxObjective = AVP_ServerNode_Objective_Marine::FindEntity( pxSquad->GetCurrentObjectiveGuid() );

			if ( pxObjective )
			{
				if ( pxObjective->GetObjectiveCompleteOnMessageOnly() == 0)
				{
					pxSquad->MemberHasCompletedAction();
					SwitchToState(ACTION_COMPLETE);
				}
				else
				{
					pxSquad->ReleaseActionGuid( m_uActionGuid );
					RequestAction();
				}
			}
		}
	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_HandleObjective :: CheckForEarlyOut
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_HandleObjective::CheckForEarlyOut( void )
{
	if ( !m_bIsOwnActorsObjective )
	{
		if ( GetOurActor()->GetCurrentObjective() )
		{
			SetFinished();
			return;
		}

		AVP_ServerEntity_Squad_Marine* pxSquad = GetSquad();

		if ( !pxSquad ) { return; }
		if ( !pxSquad->GetCurrentObjective() ) { return; }

		if ( pxSquad->GetCurrentObjectiveThreatAction() != AVP_ServerNode_Objective_Marine::NO_ACTION && ( DoesHaveThreat() || DoesHaveAlert() ) )
		{
			SetFinished();
			SwitchToState( ACTION_COMPLETE );
		}

		if ( pxSquad->IsPlayersSquad() && pxSquad->GetCurrentObjectiveState() == AVP_ServerNode_Objective_Marine::MOVING_TO_OBJECTIVE )
		{
			SetFinished();
		}

		if ( pxSquad->GetCurrentObjectiveThreatAction() == AVP_ServerNode_Objective_Marine::NO_ACTION )
		{
			if ( GetKnowledgeSystem()->DoesHaveReconKnowledge() )
			{
				GetOurActor()->PlayDialogue( AVP_ServerActor_Human::AVP_DLG_HUMAN_KEEP_FORMATION ); // RECON FAILURE
			}
		}
	}
	else
	{
		if ( GetOurActor()->GetCurrentObjectiveThreatAction() != AVP_ServerNode_Objective_Marine::NO_ACTION && ( DoesHaveThreat() || DoesHaveAlert() ) )
		{
			SetFinished();
			SwitchToState( ACTION_COMPLETE );
		}

		if ( GetOurActor()->GetCurrentObjectiveThreatAction() == AVP_ServerNode_Objective_Marine::NO_ACTION )
		{
			if ( GetKnowledgeSystem()->DoesHaveReconKnowledge() )
			{
				GetOurActor()->PlayDialogue( AVP_ServerActor_Human::AVP_DLG_HUMAN_KEEP_FORMATION ); // RECON FAILURE
			}
		}
	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_HandleObjective :: CheckForAimingAtThreat
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_HandleObjective::CheckForAimingAtThreat( void )
{
	u_int uThreatAction = AVP_ServerNode_Objective_Marine::NO_ACTION;

	if (m_bIsOwnActorsObjective)
	{
		uThreatAction = GetOurActor()->GetCurrentObjectiveThreatAction();
	}
	else
	{
		AVP_ServerEntity_Squad_Marine* pxSquad = GetSquad();

		if ( !pxSquad ) { return; }
		
		uThreatAction = pxSquad->GetCurrentObjectiveThreatAction();
	}

	if ( uThreatAction == AVP_ServerNode_Objective_Marine::NO_ACTION && DoesHaveThreat() )
	{
		if ( GetKnowledgeSystem()->GetBestThreatKnowledge().m_uTypeInformation == AVP_Sense_Vision::TYPE_INFO_CAN_SEE )
		{
			GetTacticalSystem()->LetRouteSetLookAt( false );
			GetMarineBrain()->AimAtThreat();
			GetMarineBrain()->LookAtThreat();
		}
		else
		{
			GetTacticalSystem()->LetRouteSetLookAt( true );
		}
	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_HandleObjective :: IsLeadMarine
//---------------------------------------------------------------------------
bool AVP_Behaviour_Marine_HandleObjective::IsLeadMarine()
{
	AVP_ServerActor_Human*				pxOurActor	= GetOurActor();
	AVP_ServerEntity_Squad_Marine*		pxSquad		= GetSquad();
	const AVP_Navigation::NavData&		xNavData	= GetMarineBrain()->GetNavData();

	u_short			uOurRouteSegmentIndex = 0;
	Asura_Vector_3	xOurNearestPos( Asura_Maths::ZeroVector3 );

	AVP_Navigation::FindNearestPointOnOtherRoute( xNavData, pxOurActor->GetPosition(), xOurNearestPos, uOurRouteSegmentIndex );
	float fBestRouteDistance	= AVP_Navigation::CalculateRouteDistance( xNavData, xOurNearestPos, uOurRouteSegmentIndex );

	if ( pxSquad )
	{
		for ( u_int u = 0; u < pxSquad->GetNumSquadMembers(); ++u )
		{
			AVP_ServerEntity_Actor* pxActor = AVP_ServerEntity_Actor::FindEntity( pxSquad->GetMemberGuid( u ) );

			// verify
			if ( pxActor && !pxActor->IsDead() && pxActor->IsEntityActive() && pxActor->GetGuid() != GetMarineBrain()->GetOwnerGuid() )
			{
				u_short			uRouteSegmentIndex = 0;
				Asura_Vector_3	xNearestPos( Asura_Maths::ZeroVector3 );

				AVP_Navigation::FindNearestPointOnOtherRoute( xNavData, pxActor->GetPosition(), xNearestPos, uRouteSegmentIndex );
				float fRouteDistance = AVP_Navigation::CalculateRouteDistance( xNavData, xNearestPos, uRouteSegmentIndex );

				if ( fRouteDistance < fBestRouteDistance )
				{
					m_uLeadMarine = pxActor->GetGuid();
					return false;
				}
			}
		}
	}

	m_uLeadMarine = GetMarineBrain()->GetOwnerGuid();
	return true;
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_HandleObjective :: IsRearGuard
//---------------------------------------------------------------------------
bool AVP_Behaviour_Marine_HandleObjective::IsRearGuard()
{
	AVP_ServerActor_Human*				pxLeadActor	= AVP_ServerActor_Human::FindEntity( m_uLeadMarine );
	AVP_ServerEntity_Squad_Marine*		pxSquad		= GetSquad();
	const AVP_Navigation::NavData&		xNavData	= pxLeadActor->GetActorBrain()->GetNavData();

	u_short			uOurRouteSegmentIndex = 0;
	Asura_Vector_3	xOurNearestPos( Asura_Maths::ZeroVector3 );

	AVP_Navigation::FindNearestPointOnRoute( xNavData, pxLeadActor->GetPosition(), 3.0f, m_xPosOnRoute, uOurRouteSegmentIndex );
	float fBestRouteDistance	= AVP_Navigation::CalculateRouteDistance( xNavData, xOurNearestPos, uOurRouteSegmentIndex );

	Asura_Guid uBestGuid = ASURA_GUID_UNREGISTERED;

	if ( pxSquad )
	{
		for ( u_int u = 0; u < pxSquad->GetNumSquadMembers(); ++u )
		{
			AVP_ServerEntity_Actor* pxActor = AVP_ServerEntity_Actor::FindEntity( pxSquad->GetMemberGuid( u ) );

			// verify
			if ( pxActor && !pxActor->IsDead() && pxActor->IsEntityActive() && pxActor->GetGuid() != pxLeadActor->GetGuid() )
			{
				u_short			uRouteSegmentIndex = 0;
				Asura_Vector_3	xNearestPos( Asura_Maths::ZeroVector3 );

				AVP_Navigation::FindNearestPointOnOtherRoute( xNavData, pxActor->GetPosition(), xNearestPos, uRouteSegmentIndex );
				float fRouteDistance = AVP_Navigation::CalculateRouteDistance( xNavData, xNearestPos, uRouteSegmentIndex );

				if ( uRouteSegmentIndex < xNavData.m_usRouteLength - 2 )
				{
					Asura_AI_NavVertex* pxWaypoint = Asura_AI_Navigation_System::GetWaypoint( xNavData.m_pusRoute[uRouteSegmentIndex + 1] );
					fRouteDistance += ( pxActor->GetPosition() - pxWaypoint->GetPosition() ).Magnitude();
				}

				if ( fRouteDistance > fBestRouteDistance )
				{
					uBestGuid = pxActor->GetGuid();
					fBestRouteDistance = fRouteDistance;
				}
			}
		}
	}

	if ( GetMarineBrain()->GetOwnerGuid() == uBestGuid )
	{
		return true;
	}
	else
	{
		return false;
	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_HandleObjective :: ScanAreaInFront
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_HandleObjective::ScanAreaInFront( const float fTimeslice )
{
	m_fScanTimer += fTimeslice;

	if ( m_fScanTimer > m_fLookTimeOut )
	{
		AVP_ServerActor_Human* pxOurActor = GetOurActor();

		const AVP_Navigation::NavData&	xNavData			= GetMarineBrain()->GetNavData();
		u_short							uRouteSegmentIndex	= 0;
		Asura_Vector_3					xNearestPos( Asura_Maths::ZeroVector3 );
		
		AVP_Navigation::FindNearestPointOnRoute( xNavData, pxOurActor->GetPosition(), 3.0f, xNearestPos, uRouteSegmentIndex );

		Asura_Vector_3 xStartDirection( Asura_Maths::ZeroVector3 );

		if ( xNearestPos == Asura_Maths::ZeroVector3 )
		{
			Asura_Matrix_3x3 xOri = pxOurActor->GetOrientation();
			xStartDirection = Asura_Vector_3( xOri.e13, xOri.e23, xOri.e33 );
		}
		else
		{
			xStartDirection = ( xNearestPos - pxOurActor->GetPosition() );
			xStartDirection.Normalise();
		}

		Asura_Matrix_3x3 xOri;
		Asura_Maths::OrientationFromDirection( xOri, xStartDirection );
		Asura_Angle		f45Degrees = Asura_Maths::Deg90 / 2.0f;
		Asura_Angle		fAngle	= Asura_Random::GetFloat(-f45Degrees, f45Degrees);
		Asura_Vector_3	xUp( xOri.e12, xOri.e22, xOri.e32 );
		Asura_Quat		xQuat;
		
		xQuat.FromMatrix( xOri );
		xQuat.FromRotation(fAngle, xUp);
		xStartDirection = xQuat.RotateVector(xStartDirection);
		
		m_fScanTimer = 0.0f;
		m_fLookTimeOut = Asura_Random::GetFloat( 1.9f, 3.1f );

		Asura_Vector_3 xHeadPos( Asura_Maths::ZeroVector3 );

		pxOurActor->GetHeadPosition( xHeadPos );
		GetMarineBrain()->SetLookAtPosition( xHeadPos + xStartDirection );
		GetMarineBrain()->SetAimDirection( xStartDirection );
		GetTacticalSystem()->LetRouteSetLookAt( false );
		GetTacticalSystem()->SetForceAiming( false );
	}

	
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_HandleObjective :: ScanAreaBehind
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_HandleObjective::ScanAreaBehind( const float fTimeslice )
{
	m_fScanTimer += fTimeslice;

	if ( m_fScanTimer > m_fLookTimeOut )
	{
		AVP_ServerActor_Human* pxOurActor = GetOurActor();

		const AVP_Navigation::NavData&	xNavData			= GetMarineBrain()->GetNavData();
		u_short							uRouteSegmentIndex	= 0;
		Asura_Vector_3					xNearestPos( Asura_Maths::ZeroVector3 );

		AVP_Navigation::FindNearestPointOnRoute( xNavData, pxOurActor->GetPosition(), 3.0f, xNearestPos, uRouteSegmentIndex );

		Asura_Vector_3 xStartDirection( Asura_Maths::ZeroVector3 );

		if ( m_uRearGuardCounter < NUMBER_OF_REAR_LOOKS )
		{
			Asura_Vector_3 xStartDirection( Asura_Maths::ZeroVector3 );

			if ( xNearestPos == Asura_Maths::ZeroVector3 )
			{
				Asura_Matrix_3x3 xOri = pxOurActor->GetOrientation();
				xStartDirection = Asura_Vector_3( xOri.e13, xOri.e23, xOri.e33 );
			}
			else
			{
				xStartDirection = ( xNearestPos - pxOurActor->GetPosition() );
				xStartDirection.Normalise();
			}

			Asura_Quat		xQuat;
			Asura_Matrix_3x3 xOri;
			Asura_Maths::OrientationFromDirection( xOri, xStartDirection );

			Asura_Vector_3	xUp( xOri.e12, xOri.e22, xOri.e32 );
			Asura_Angle		f45Degrees = Asura_Maths::Deg90 / 2.0f;
			Asura_Angle		fQuaterOf90Degrees = Asura_Maths::Deg90 / 4.0f;
			Asura_Angle		fAngle	= Asura_Random::GetFloat( fQuaterOf90Degrees, f45Degrees );

			m_bDirectionLeft = !m_bDirectionLeft;

			if ( m_bDirectionLeft )
				fAngle = -fAngle;

			xQuat.FromMatrix( xOri );
			xQuat.FromRotation(fAngle, xUp);
			xStartDirection = xQuat.RotateVector(xStartDirection);

			m_fLookTimeOut = Asura_Random::GetFloat( 0.8f, 1.2f );
		}
		else
		{
			Asura_Vector_3 xStartDirection( Asura_Maths::ZeroVector3 );

			if ( xNearestPos == Asura_Maths::ZeroVector3 )
			{
				Asura_Matrix_3x3 xOri = pxOurActor->GetOrientation();
				xStartDirection = Asura_Vector_3( xOri.e13, xOri.e23, xOri.e33 );
			}
			else
			{
				xStartDirection = ( xNearestPos - pxOurActor->GetPosition() );
				xStartDirection.Normalise();
			}

			m_fLookTimeOut = Asura_Random::GetFloat( 2.1f, 3.5f );
		}
		
		Asura_Vector_3 xHeadPos( Asura_Maths::ZeroVector3 );

		pxOurActor->GetHeadPosition( xHeadPos );

		if ( xStartDirection == Asura_Maths::ZeroVector3 )
		{
			GetMarineBrain()->AimForward();
			GetMarineBrain()->LookForward();
		}
		else
		{
			GetMarineBrain()->SetLookAtPosition( xHeadPos + xStartDirection );
			GetMarineBrain()->SetAimDirection( xStartDirection );
		}
		
		GetTacticalSystem()->LetRouteSetLookAt( false );
		GetTacticalSystem()->SetForceAiming( false );

		m_fScanTimer = 0.0f;
		
		++m_uRearGuardCounter;

		if ( m_uRearGuardCounter > NUMBER_OF_REAR_LOOKS )
		{
			m_uRearGuardCounter = 0;
		}
	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_HandleObjective :: ScanAnyLookAtNodes
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_HandleObjective::ScanAnyLookAtNodes( const float fTimeslice )
{
	AVP_ServerActor_Human*				pxOurActor = GetOurActor();
	AVP_ServerEntity_GamePlayVolume*	pxGPVolume = GetOurActor()->GetCurrentGamePlayVolume();

	m_fScanTimer += fTimeslice;

	if ( pxGPVolume )
	{
		if ( m_fScanTimer > m_fLookTimeOut )
		{
			bool bDontUseLookAt = false;

			if ( pxGPVolume->GetNumLookAtPointGuids() > 0 )
			{
				
				AVP_ServerEntity_LookAt*	pxLookAt		= NULL;
				u_int						uLookAtIndex	= 0;

				if ( pxGPVolume->GetNumLookAtPointGuids() > 1 )
				{
					uLookAtIndex = Asura_Random::GetIntU( 0, pxGPVolume->GetNumLookAtPointGuids() - 1 );
				}

				pxLookAt = AVP_ServerEntity_LookAt::FindEntity( pxGPVolume->GetLookAtPointGuid( uLookAtIndex ) );

				if ( pxLookAt )
				{
					Asura_Vector_3 xHeadPos( Asura_Maths::ZeroVector3 );
					pxOurActor->GetHeadPosition( xHeadPos );

					Asura_Vector_3 xToLookAt = pxLookAt->GetPosition() - xHeadPos;
					float fDistanceToLookAt = xToLookAt.Normalise();
					
					// Do a LOS test
					const Asura_LOS_InputData	xInput = Asura_LOS_InputData( xHeadPos, xToLookAt, pxOurActor->GetGuid(), fDistanceToLookAt, false);
					const Asura_LOS_OutputData*	pxOutput = Asura_LOS::TestAgainstEnvironment( &xInput );

					if ( pxOutput->m_fDistanceToIntersection == fDistanceToLookAt )
					{
						// look at it!
		 				GetMarineBrain()->SetLookAtPosition( xHeadPos + xToLookAt );
		 				GetMarineBrain()->SetAimDirection( xToLookAt );
		 				GetTacticalSystem()->LetRouteSetLookAt( false );

						GetTacticalSystem()->SetForceAiming( true );

						m_fScanTimer = 0.0f;
						m_fLookTimeOut = Asura_Random::GetFloat( 1.9f, 3.1f );
					}
					else
					{
						bDontUseLookAt = true;
					}
				}
			}

			if ( bDontUseLookAt || pxGPVolume->GetNumLookAtPointGuids() == 0 )
			{
				AVP_ServerActor_Human* pxOurActor = GetOurActor();

				const AVP_Navigation::NavData&	xNavData			= GetMarineBrain()->GetNavData();
				u_short							uRouteSegmentIndex	= 0;
				Asura_Vector_3					xNearestPos( Asura_Maths::ZeroVector3 );
				
				AVP_Navigation::FindNearestPointOnRoute( xNavData, pxOurActor->GetPosition(), 3.0f, xNearestPos, uRouteSegmentIndex );

				Asura_Vector_3 xStartDirection( Asura_Maths::ZeroVector3 );

				if ( xNearestPos == Asura_Maths::ZeroVector3 )
				{
					Asura_Matrix_3x3 xOri = pxOurActor->GetOrientation();
					xStartDirection = Asura_Vector_3( xOri.e13, xOri.e23, xOri.e33 );
				}
				else
				{
					xStartDirection = ( xNearestPos - pxOurActor->GetPosition() );
					xStartDirection.Normalise();
				}

				Asura_Matrix_3x3 xOri;
				Asura_Maths::OrientationFromDirection( xOri, xStartDirection );
				Asura_Angle		f45Degrees = Asura_Maths::Deg90 / 2.0f;
				Asura_Angle		fAngle	= Asura_Random::GetFloat(-f45Degrees, f45Degrees);
				Asura_Vector_3	xUp( xOri.e12, xOri.e22, xOri.e32 );
				Asura_Quat		xQuat;
				
				xQuat.FromMatrix( xOri );
				xQuat.FromRotation(fAngle, xUp);
				xStartDirection = xQuat.RotateVector(xStartDirection);
				
				m_fScanTimer = 0.0f;
				m_fLookTimeOut = Asura_Random::GetFloat( 1.9f, 3.1f );

				Asura_Vector_3 xHeadPos( Asura_Maths::ZeroVector3 );

				pxOurActor->GetHeadPosition( xHeadPos );
				GetMarineBrain()->LookForward();
				GetMarineBrain()->SetAimDirection( xStartDirection );
				GetTacticalSystem()->LetRouteSetLookAt( false );
				GetTacticalSystem()->SetForceAiming( false );
			}
		}
	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_HandleObjective :: ScanArea
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_HandleObjective::ScanArea( const float fTimeslice )
{
	AVP_ServerEntity_Squad_Marine* pxSquad = GetSquad();

	if ( !pxSquad ) { return; }
	if ( !pxSquad->GetCurrentObjective() ) { return; }

	if ( GetMarineBrain()->DoesHaveThreat() && pxSquad->GetCurrentObjective()->GetObjectiveThreatAction() == AVP_ServerNode_Objective_Marine::NO_ACTION )
	{
		if ( GetKnowledgeSystem()->GetBestThreatKnowledge().m_uTypeInformation == AVP_Sense_Vision::TYPE_INFO_CAN_SEE )
		{
			return;
		}
	}
	
	if ( pxSquad )
	{
		if ( pxSquad->GetNumSquadMembers() == 1 )
		{
			GetTacticalSystem()->LetRouteSetLookAt( true );
		}
		else if ( GetMarineBrain()->GetLookAtGuid() != ASURA_GUID_UNREGISTERED )
		{
			GetMarineBrain()->LookAt_LookAtEntity();
		}
		else if ( pxSquad->GetLeadGuid() == GetMarineBrain()->GetOwnerGuid() )
		{
			m_uPosition = LEAD;
			ScanAreaInFront( fTimeslice );
		}
		else if ( pxSquad->GetRearGuardGuid() == GetMarineBrain()->GetOwnerGuid() )
		{
			m_uPosition = REAR_GUARD;
			ScanAreaBehind( fTimeslice );
		}
		else
		{
			m_uPosition = MIDDLE_MAN;
			ScanAnyLookAtNodes(fTimeslice);
		}
	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_HandleObjective :: RenderDebugInfo
//---------------------------------------------------------------------------w
#ifndef ASURA_NO_DEBUG_OUTPUT
static const char* s_aszStateStrings[ AVP_Behaviour_Marine_HandleObjective::MAX_STATES ] =
{
	"HANDLE GOAL - UNSET",
	"HANDLE GOAL - MOVING TO GOAL",
	"HANDLE GOAL - WAIT TO PERFROM ACTION",
	"HANDLE GOAL - MOVE TO ACTION",
	"HANDLE GOAL - IN MOVE",
	"HANDLE GOAL - PERFROM ACTION",
	"HANDLE GOAL - ACTION COMPLETE",
	"HANDLE GOAL - WAIT FOR PLAYER",
	"HANDLE GOAL - WAIT FOR GOAL COMPLETE",
};

static const char* s_aszPositionStrings[ AVP_Behaviour_Marine_HandleObjective::MAX_POSITIONS ] =
{
	"POSITION : LEAD",
	"POSITION : REAR GUARD",
	"POSITION : MIDDLE MAN",
};

static const char* s_aszSpeedStrings[ AVP_Navigation::NUM_MOVEMENT_MODES ] =
{
	"MOVEMENT SPEED - Slow",
	"MOVEMENT SPEED - Medium",
	"MOVEMENT SPEED - Fast",
};

static const char* s_aszThreatActionStrings[ AVP_ServerNode_Objective_Marine::MAX_THREAT_ACTIONS ] =
{
	"DONT STOP TO ENGAGE",
	"STOP TO ENGAGE",
};

DEBUGVAR bool dbg_uRenderAdditionalInfo = false;

void AVP_Behaviour_Marine_HandleObjective::RenderDebugInfo( void )
{
	PARENT::RenderDebugInfo();

	Asura_Font_System::Print
	(
		GetProjectBrain()->GetDebugOutputPos().x,
		GetProjectBrain()->GetDebugOutputPos().y,
		0.5f, 
		0xffffff00,
		ASURA_TEXTFORMAT_CENTREJUSTIFIED, 
		s_aszStateStrings[GetUpdateState()]
	);

	// Offset the debug output position for other behaviours to render there own text
	GetProjectBrain()->GetDebugOutputPos().y += 10.0f;

	if ( dbg_uRenderAdditionalInfo )
	{
		Asura_Font_System::Print
		(
			GetProjectBrain()->GetDebugOutputPos().x,
			GetProjectBrain()->GetDebugOutputPos().y,
			0.5f, 
			0xffffffff,
			ASURA_TEXTFORMAT_CENTREJUSTIFIED,
			s_aszPositionStrings[m_uPosition]
		);

		// Offset the debug output position for other behaviours to render there own text
		GetProjectBrain()->GetDebugOutputPos().y += 10.0f;

		if ( GetSquad() )
		{
			if ( GetSquad()->GetRearGuardGuid() == GetMarineBrain()->GetOwnerGuid() )
			{
				Asura_Font_System::Print
				(
					GetProjectBrain()->GetDebugOutputPos().x,
					GetProjectBrain()->GetDebugOutputPos().y,
					0.5f, 
					0xffffffff,
					ASURA_TEXTFORMAT_CENTREJUSTIFIED | ASURA_TEXTFORMAT_FORMATSTRING,
					"RGC : %d", m_uRearGuardCounter
				);

				// Offset the debug output position for other behaviours to render there own text
				GetProjectBrain()->GetDebugOutputPos().y += 10.0f;
			}
		}
	}

	if ( m_bIsOwnActorsObjective )
	{
		Asura_Font_System::Print
		(
			GetProjectBrain()->GetDebugOutputPos().x,
			GetProjectBrain()->GetDebugOutputPos().y,
			0.5f, 
			0xff88ff88,
			ASURA_TEXTFORMAT_CENTREJUSTIFIED,
			s_aszSpeedStrings[ GetMarineBrain()->GetHumanActor()->GetCurrentObjectiveMovementMode() ]
		);

		GetProjectBrain()->GetDebugOutputPos().y += 10.0f;

		Asura_Font_System::Print
		(
			GetProjectBrain()->GetDebugOutputPos().x,
			GetProjectBrain()->GetDebugOutputPos().y,
			0.5f, 
			0xff88ff88,
			ASURA_TEXTFORMAT_CENTREJUSTIFIED,
			s_aszThreatActionStrings[ GetMarineBrain()->GetHumanActor()->GetCurrentObjectiveThreatAction() ]
		);
	}
	else
	{
		Asura_Font_System::Print
		(
			GetProjectBrain()->GetDebugOutputPos().x,
			GetProjectBrain()->GetDebugOutputPos().y,
			0.5f, 
			0xff88ff88,
			ASURA_TEXTFORMAT_CENTREJUSTIFIED,
			s_aszSpeedStrings[ GetMarineBrain()->GetMarineSquad()->GetCurrentObjectiveMovementMode() ]
		);

		GetProjectBrain()->GetDebugOutputPos().y += 10.0f;

		Asura_Font_System::Print
		(
			GetProjectBrain()->GetDebugOutputPos().x,
			GetProjectBrain()->GetDebugOutputPos().y,
			0.5f, 
			0xff88ff88,
			ASURA_TEXTFORMAT_CENTREJUSTIFIED,
			s_aszThreatActionStrings[ GetMarineBrain()->GetMarineSquad()->GetCurrentObjectiveThreatAction() ]
		);
	}

	GetProjectBrain()->GetDebugOutputPos().y += 10.0f;
}
#endif

/// End Of File 'AVP_behaviour_Marine_HandleObjective.cpp'
