//---------------------------------------------------------------------------
// File			: AVP_Behaviour_Marine_HideFromCloakedPredator.cpp
// Created		: 02 / 07 / 2008
// Created By	: Gareth Thomas
// Copyright (C) 2007 Rebellion. All Rights Reserved.
//---------------------------------------------------------------------------

// INCLUDES ///////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"
// header
#include "Behaviour/BehaviourTypes/Human/Marine/AVP_Behaviour_Marine_HideFromCloakedPredator.h"
// asura
#ifndef ASURA_NO_DEBUG_OUTPUT
#include "Rendering/Asura_Font_System.h"
#include "Debugging/Asura_DebugLineRender.h"
#endif

// axon
// AVP
#include "Entity/PhysicalObject/Actor/Vertebrates/Human/AVP_ServerActor_Human.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Predator/AVP_ServerActor_Predator.h"
#include "Inventory/AVP_Inventory_HashIDs.h"
#include "Inventory/AVP_Inventory_System.h"
#include "Inventory/Items/AVP_InventoryItem_PredatorCloak.h"
#include "Inventory/Items/Weapons/AVP_RangedWeapon_SubType.h"
#include "Inventory/Items/Weapons/AVP_InventoryWeapon_RangedWeapon.h"

// AVP axon
#include "Behaviour/BehaviourTypes/AVP_Behaviour_Types.h"
#include "Core/BrainTypes/AVP_Brain_Actor_Marine.h"
#include "Core/AVP_PositionEvaluation.h"
#include "Knowledge/AVP_Knowledge_System_Actor.h"
#include "Tactical/AVP_Tactical_System.h"

DEBUGVAR float dbg_fMinFireTime = 0.5f;
DEBUGVAR float dbg_fMaxFireTime = 0.8f;

DEBUGVAR float dbg_fMinStoppedFireTime = 0.3f;
DEBUGVAR float dbg_fMaxStoppedFireTime = 0.5f;

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_HideFromCloakedPredator :: Constructor
//---------------------------------------------------------------------------
AVP_Behaviour_Marine_HideFromCloakedPredator::AVP_Behaviour_Marine_HideFromCloakedPredator( AVP_Brain_Actor_Marine* pxBrain )
: PARENT( pxBrain, AVP_BEHAVIOUR_TYPE_MARINE_HIDEFROMCLOAKEDPREDATOR )
, m_eUpdateState( UNSET )
, m_pxMarineBrain( pxBrain )
, m_xLastSeenPosition( Asura_Maths::ZeroVector3 )
, m_fShootTimer( 0.0f )
, m_fAimTimer( 0.0f )
, m_fHideTimer( 0.0f )
, m_fHideTimout( 0.0f )
{

}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_HideFromCloakedPredator :: Deconstructor
//---------------------------------------------------------------------------
AVP_Behaviour_Marine_HideFromCloakedPredator::~AVP_Behaviour_Marine_HideFromCloakedPredator( void )
{

}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_HideFromCloakedPredator :: Activate
//---------------------------------------------------------------------------
bool AVP_Behaviour_Marine_HideFromCloakedPredator::Activate()
{
	const bool bParentOk = PARENT::Activate();
	// Put Code Below This Comment

	SwitchToState( POSITION_EVALUATION );

	StopShooting();

	GetTacticalSystem()->UseExternalShooting( true );

	if ( GetKnowledgeSystem()->GetAlertLevel() != AVP_Knowledge_System_Actor::AGGRESSIVE_ALERTNESS )
	{
		GetKnowledgeSystem()->SetAlertLevel( AVP_Knowledge_System_Actor::ACTIVE_ALERTNESS );
	}
	GetKnowledgeSystem()->ResetAlertTimer();

	GetMarineBrain()->GetHumanActor()->PlayDialogue( AVP_ServerActor_Human::AVP_DLG_HUMAN_AGGRESSIVE_TO_ALERT );

	// Put Code Above This Comment
	return bParentOk;
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_HideFromCloakedPredator :: Deactivate
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_HideFromCloakedPredator::Deactivate()
{
	StopShooting();

	if ( GetMarineBrain()->IsActive() )
	{
		GetTacticalSystem()->UseExternalShooting( false );

		if ( GetKnowledgeSystem()->GetAlertLevel() != AVP_Knowledge_System_Actor::AGGRESSIVE_ALERTNESS )
		{
			GetKnowledgeSystem()->SetAlertLevel( AVP_Knowledge_System_Actor::ACTIVE_ALERTNESS );
		}

		GetKnowledgeSystem()->ResetAlertTimer();
	}

	// Put Code Above This Comment
	DestroyAllChildren();
	PARENT::Deactivate();
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_HideFromCloakedPredator :: PerFrameUpdate
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_HideFromCloakedPredator::PerFrameUpdate( const float fTimeslice )
{
	if ( DoesHaveThreat() && GetKnowledgeSystem()->GetBestThreatKnowledge().m_uSourceClassification != AVP_ENTITYCLASS_PREDATOR )
	{
		SetFinished();
		return;
	}
	else if ( DoesHaveThreat() && GetKnowledgeSystem()->GetBestThreatKnowledge().m_uSourceClassification == AVP_ENTITYCLASS_PREDATOR )
	{
		AVP_ServerActor_Predator* pxPredator = AVP_ServerActor_Predator::FindEntity( GetKnowledgeSystem()->GetBestThreatKnowledge().GetGuid() );

		if ( pxPredator )
		{
			AVP_InventoryItem_Cloak* pxCloak = static_cast< AVP_InventoryItem_Cloak* >( pxPredator->GetInventorySystem()->GetItem( uAVP_INVENTORY_ITEMHASH_CLOAK ) );

			if ( pxCloak && !pxCloak->IsUsing( false ) )
			{
				SetFinished();
				return;
			}
			else
			{
				float fDistanceSq = ( GetOurActor()->GetPosition() - pxPredator->GetPosition() ).MagnitudeSquared();

				if ( fDistanceSq < GetKnowledgeSystem()->GetCloakedVisibleRangeSq() )
				{
					SetFinished();
					return;
				}
			}
		}
	}

	switch ( GetUpdateState() )
	{
		case POSITION_EVALUATION:
		{
			Asura_Guid							uCombatZoneGuid = GetMarineBrain()->GetCombatZoneGuid();
			AVP_Navigation::NavData&			xNavData = GetMarineBrain()->GetNavData();
			AVP_Navigation::AVP_PosEvalParams	xPosEvalParams( GetMarineBrain()->GetOwnerGuid(), 0, 10.0f, m_xLastSeenPosition, uCombatZoneGuid );
			AVP_Navigation::PosEvalFunctor		xPosEval( xNavData.m_usCurrentWPID, 10.0f, xPosEvalParams, GetHideFromPredatorScoreForWaypoint );
			AVP_Navigation::SearchWaypoints( AVP_Navigation::AVP_AI_PATHFINDER_FLAG_HUMANOID, &xPosEval );
			FindBestWaypoint( &xPosEval );
			SwitchToState( MOVE_TO_HIDE_POSITION );

		}break;

		case MOVE_TO_HIDE_POSITION:
		{
			if ( GetTacticalSystem()->HasArrivedAtDestination() )
			{
				SwitchToState( AT_HIDE_POSITION );
			}

			if (!IsFinished())
			{
				m_fShootTimer += fTimeslice;
				m_fAimTimer += fTimeslice;

				DoAimingAndFiring();
			}

		}break;

		case AT_HIDE_POSITION:
		{
			if ( !IsFinished() )
			{
				m_fShootTimer += fTimeslice;
				m_fAimTimer += fTimeslice;

				DoAimingAndFiring();
			}

			m_fHideTimer += fTimeslice;

			if (m_fHideTimer > m_fHideTimout)
			{
				SetFinished();
			}
		}

		case UNSET:
		default:
		{

		}break;
	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_HideFromCloakedPredator :: SwitchToState
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_HideFromCloakedPredator::SwitchToState( UpdateState eState )
{
	switch ( GetUpdateState() )
	{
		case POSITION_EVALUATION:
		case MOVE_TO_HIDE_POSITION:
		case UNSET:
		default:
		{

		}
	}

	SetUpdateState( eState );

	switch ( GetUpdateState() )
	{
		case POSITION_EVALUATION:
		{

		}break;
		case MOVE_TO_HIDE_POSITION:
		{
			m_fShootTimer = 0.0f;
			m_fAimTimer = 0.0f;
			
			m_fStopShootTimeOut = Asura_Random::GetFloat( dbg_fMinFireTime, dbg_fMaxFireTime );
			m_fShootTimeOut = Asura_Random::GetFloat( m_fStopShootTimeOut + dbg_fMinStoppedFireTime, m_fStopShootTimeOut + dbg_fMaxStoppedFireTime );
		}break;

		case AT_HIDE_POSITION:
		{
			m_fHideTimer = 0.0f;
			m_fHideTimout = Asura_Random::GetFloat( 3.5f, 6.0f );

		}
		case UNSET:
		default:
		{

		}
	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_HideFromCloakedPredator :: FindBestWaypoint
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_HideFromCloakedPredator::FindBestWaypoint( AVP_Navigation::PosEvalFunctor*	pxPosEval )
{
	u_int uBestWaypointID = pxPosEval->GetBestWaypointID();

	if ( uBestWaypointID != ASURA_INVALID_WAYPOINT_ID )
	{
		Asura_AI_NavVertex*	pxWaypoint	= Asura_AI_Navigation_System::GetWaypoint( uBestWaypointID );
		const Asura_Vector_3&	xOurPos		= GetMarineBrain()->GetHumanActor()->GetPosition();

		if ( pxWaypoint )
		{
			float fDistanceSq = ( pxWaypoint->m_xPosition - xOurPos ).MagnitudeSquared();
			if ( fDistanceSq >= 9.0f )
			{
				GetTacticalSystem()->SetDestination( pxWaypoint->m_xPosition );
				GetTacticalSystem()->SetAtDestinationRange( ONE_METER );
				GetTacticalSystem()->SetMovementSpeed( AVP_Navigation::MM_MEDIUM );
				GetTacticalSystem()->LetRouteSetLookAt( false );
				GetTacticalSystem()->OverrideSlowDownForThisDestination();
			}
		}
	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_HideFromCloakedPredator :: GetHideFromPredatorScoreForWaypoint
//---------------------------------------------------------------------------
u_int AVP_Behaviour_Marine_HideFromCloakedPredator::GetHideFromPredatorScoreForWaypoint( AVP_Navigation::AVP_PosEvalParams xPosEvalParams )
{
	AVP_ServerActor_Human* pxHuman = AVP_ServerActor_Human::FindEntity( xPosEvalParams.m_uActorGuid );

	u_int uScore = 0;

	// this should always be valid as he has just requested a position evaluation
	if ( !pxHuman ) { return uScore; }

	// Waypoint Proximity
	uScore += AVP_PositionEvaluation::GetWaypointNoLOFToPositionScore( pxHuman, xPosEvalParams.m_uTestWPID, xPosEvalParams.m_xPosition, 60 );

	// Waypoint Occupied by ally
	uScore += AVP_PositionEvaluation::GetHumanoidPositionOccupiedByAllyScore( pxHuman, xPosEvalParams.m_uTestWPID, 40 );

	// Always want a position close to the squad leader
	uScore += AVP_PositionEvaluation::GetPositionCloseToSquadLeaderScore( pxHuman, xPosEvalParams.m_uTestWPID, 40 );

	uScore += AVP_PositionEvaluation::GetWaypointProvidesCoverFromPosition( xPosEvalParams.m_xPosition, xPosEvalParams.m_uTestWPID, 60 );

	int iScore = uScore - AVP_PositionEvaluation::GetProximityToPositionScore( xPosEvalParams.m_uTestWPID, xPosEvalParams.m_fEvaluationRange, xPosEvalParams.m_xPosition, 2.0f );

	if ( iScore < 0 )
	{
		return 0;
	}
	else
	{
		uScore = ( u_int )iScore;
	}

	return uScore;
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_HideFromCloakedPredator :: DoAimingAndFiring
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_HideFromCloakedPredator::DoAimingAndFiring()
{
	AimAroundLastSeenPos();

	AVP_ServerActor_Human*	pxOurActor = GetMarineBrain()->GetHumanActor();
	Asura_Vector_3			xGunPos( Asura_Maths::ZeroVector3 );
	Asura_Matrix_3x3		xGunOri;

	pxOurActor->SetAiming( true );

	pxOurActor->GetWorldSpaceMarkerPoint(0x0001A7D2, &xGunPos, &xGunOri); // Asura_GetHashID("muz")

	Asura_Vector_3	xTestDirection(xGunOri.e13, xGunOri.e23, xGunOri.e33);

	const Asura_LOS_InputData	xInput = Asura_LOS_InputData(xGunPos, xTestDirection, pxOurActor->GetGuid(), 100.0f, true);
	const Asura_LOS_OutputData*	pxOutput = Asura_LOS::TestAgainstEverything(&xInput);

	if ( pxOutput->m_uHitEntity != 0 )
	{
		AVP_ServerEntity_Actor* pxHitActor = AVP_ServerEntity_Actor::FindEntity( pxOutput->m_uHitEntity );

		if ( pxHitActor )
		{
			u_int uAllegiance = AVP_Tactical_System::GetAllegiance( *pxOurActor, *pxHitActor );

			if (uAllegiance != AVP_ALLEGIANCE_ENEMY)
			{
				StopShooting();
				return;
			}
		}
	}

	if ( pxOutput->m_fDistanceToIntersection <= 5.0f )
	{
		StopShooting();
	}
	else
	{
		if (m_fShootTimer < m_fShootTimeOut)
		{
			FireAtAimPoint();
		}
		else
		{
			StopShooting();

			if (m_fShootTimer >= m_fStopShootTimeOut)
			{
				m_fShootTimer = 0.0f;
				m_fStopShootTimeOut = Asura_Random::GetFloat( dbg_fMinFireTime, dbg_fMaxFireTime );
				m_fShootTimeOut = Asura_Random::GetFloat( m_fStopShootTimeOut + dbg_fMinStoppedFireTime, m_fStopShootTimeOut + dbg_fMaxStoppedFireTime );
			}
		}
	}
}

DEBUGVAR float dbg_fSpreadX = 3.5f;
DEBUGVAR float dbg_fSpreadZ = 3.5f;
DEBUGVAR float dbg_fSpreadY = 0.4f;
DEBUGVAR float dbg_fAimTimeout = 2.0f;

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_HideFromCloakedPredator :: AimAroundLastSeenPos
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_HideFromCloakedPredator::AimAroundLastSeenPos( void )
{
	if (m_fAimTimer > dbg_fAimTimeout)
	{
		Asura_Vector_3 xOffset( Asura_Random::GetFloat(-dbg_fSpreadX, dbg_fSpreadX), Asura_Random::GetFloat(-dbg_fSpreadY, dbg_fSpreadY), Asura_Random::GetFloat(-dbg_fSpreadZ, dbg_fSpreadZ) );
		Asura_Vector_3 xAimAt = m_xLastSeenPosition + xOffset;

		GetMarineBrain()->AimAtPosition( xAimAt );

		m_fAimTimer = 0.0f;
	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_HideFromCloakedPredator :: FireAtAimPoint
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_HideFromCloakedPredator::FireAtAimPoint( void )
{
	AVP_ServerActor_Human* pxOurActor = GetMarineBrain()->GetHumanActor();

	if (pxOurActor)
	{
		AVP_ClientEntity_Actor* pxOurClientActor = AVP_ClientEntity_Actor::FindEntity(pxOurActor->GetGuid());

		if (pxOurClientActor)
		{
			AVP_Inventory_System*		pxInventorySystem	= pxOurActor->GetInventorySystem();
			AVP_InventoryItem_Weapon*	pxCurrentWeapon		= static_cast<AVP_InventoryItem_Weapon*>(pxInventorySystem ? pxInventorySystem->GetCurrentItem(uAVP_INVENTORY_GROUPHASH_WEAPON) : NULL);

			if (pxCurrentWeapon != NULL)
			{
				if (pxCurrentWeapon->GetSubType(0)->GetCurrentQuantity() > 0)
				{
					if (pxCurrentWeapon->IsIdling(true))
					{
						pxCurrentWeapon->RequestBeginUsage(AVP_InventoryWeapon_RangedWeapon::UT_FULL_AUTO_PRIMARY);
					}
				}
			}
		}
	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_HideFromCloakedPredator :: StopShooting
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_HideFromCloakedPredator::StopShooting()
{
	AVP_ServerActor_Human* pxOurActor = GetMarineBrain()->GetHumanActor();

	if (pxOurActor)
	{
		AVP_ClientEntity_Actor* pxOurClientActor = AVP_ClientEntity_Actor::FindEntity(pxOurActor->GetGuid());

		if (pxOurClientActor)
		{
			AVP_Inventory_System*	pxInventorySystem	= pxOurActor->GetInventorySystem();
			AVP_Inventory_Item*		pxCurrentWeapon		= pxInventorySystem ? pxInventorySystem->GetCurrentItem(uAVP_INVENTORY_GROUPHASH_WEAPON) : NULL;

			if((pxCurrentWeapon != NULL) && pxCurrentWeapon->IsUsing(true) && pxCurrentWeapon->GetCurrentUsageType() != AVP_InventoryWeapon_RangedWeapon::UT_RELOAD_ALL)
			{
				pxCurrentWeapon->RequestEndUsage(AVP_InventoryWeapon_RangedWeapon::UT_FULL_AUTO_PRIMARY);
			}
		}
	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_HideFromCloakedPredator :: RenderDebugInfo
//---------------------------------------------------------------------------
#ifndef ASURA_NO_DEBUG_OUTPUT
static const char* s_aszStateStrings[ AVP_Behaviour_Marine_HideFromCloakedPredator::MAX_STATES ] =
{
	"HIDE FROM PRED - UNSET",
	"HIDE FROM PRED - POSITION_EVALUATION",
	"HIDE FROM PRED - MOVETO_HIDE_POSITION",
};

void AVP_Behaviour_Marine_HideFromCloakedPredator::RenderDebugInfo()
{
	PARENT::RenderDebugInfo();

	Asura_Font_System::Print
	(
		GetProjectBrain()->GetDebugOutputPos().x,
		GetProjectBrain()->GetDebugOutputPos().y,
		0.5f, 
		AVP_Brain_Actor_Marine::s_uTemporaryChildDebugColour,
		ASURA_TEXTFORMAT_CENTREJUSTIFIED, 
		s_aszStateStrings[ GetUpdateState() ]
	);

	// Offset the debug output position for other behaviours to render there own text
	GetProjectBrain()->GetDebugOutputPos().y += 10.0f;
}
#endif

/// End Of File 'AVP_behaviour_Marine_HideFromCloakedPredator.cpp'
