//---------------------------------------------------------------------------
// File			: AVP_Behaviour_Marine_InCover.cpp
// Created		: 02 / 11 / 07
// Created By	: Gareth Thomas
// Copyright (C) 2007 Rebellion. All Rights Reserved.
//---------------------------------------------------------------------------

// INCLUDES ///////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"
// header
#include "Behaviour/BehaviourTypes/Human/Marine/AVP_Behaviour_Marine_InCover.h"
// asura
#include "Blueprint/Asura_Blueprint_System.h"
#ifndef ASURA_NO_DEBUG_OUTPUT
#include "Rendering/Asura_Font_System.h"
#include "Debugging/Asura_DebugLineRender.h"
#endif

// AVP
#include "Actor/Vertebrates/Human/AVP_Human_AIAnimStateMachine.h"
#include "Blueprint/AVP_Blueprint_HashIDs.h"
#include "Entity/GamePlayVolume/AVP_ServerEntity_GamePlayVolume.h"
#include "Entity/Locations/OrientedNode/CoverpointNode/AVP_ServerNode_CoverpointNode.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Human/AVP_ClientActor_Human.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Human/AVP_ServerActor_Human.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Human/AVP_Messages_Human.h"
#include "Inventory/AVP_Inventory_HashIDs.h"
#include "Moves/Vertebrates/Human/AVP_Human_MovesCover.h"
#include "Network/Asura_Message.h"

// axon
#include "Behaviour/BehaviourTypes/AVP_Behaviour_Types.h"
#include "Core/BrainTypes/AVP_Brain_Actor_Marine.h"
#include "Knowledge/AVP_Knowledge_System_Actor.h"
#include "Tactical/AVP_Tactical_System.h"

DEBUGVAR float dbg_fMinInCoverPopupTime		= 0.9f;
DEBUGVAR float dbg_fMaxInCoverPopupTime		= 1.2f;
DEBUGVAR float dbg_fMinInCoverStepOutTime	= 1.0f;
DEBUGVAR float dbg_fMaxInCoverStepOutTime	= 2.0f;

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_InCover :: Constructor
//---------------------------------------------------------------------------
AVP_Behaviour_Marine_InCover::AVP_Behaviour_Marine_InCover( AVP_Brain_Actor_Marine* pxBrain )
: PARENT( pxBrain, AVP_BEHAVIOUR_TYPE_MARINE_INCOVER )
, m_eUpdateState( UNSET )
, m_pxMarineBrain( pxBrain )
, m_uCoverPoint( ASURA_GUID_UNREGISTERED )
, m_fIdleTimer( 0.0f )
, m_fPeekTimer( 0.0f )
, m_uEmergePointIndex( 0 )
, m_bAdvanceState( false )
#ifndef ASURA_NO_DEBUG_OUTPUT
, m_fDebugRenderTimer( 0.0f )
#endif
{

}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_InCover :: Activate
//---------------------------------------------------------------------------
bool AVP_Behaviour_Marine_InCover::Activate( void )
{
	const bool bParentOk = PARENT::Activate();

	GetMarineBrain()->GetTacticalSystem()->SetIsUsingCover( true );
	GetMarineBrain()->GetHumanActor()->SetIsAllowedToFlinch( false );

	return bParentOk;
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_InCover :: Deactivate
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_InCover::Deactivate( void )
{
	PARENT::Deactivate();
	DestroyAllChildren();

	if ( m_uCoverPoint != ASURA_GUID_UNREGISTERED )
	{
		Asura_ServerNode_Coverpoint* pxCoverPoint = Asura_ServerNode_Coverpoint::FindEntity( m_uCoverPoint );

		if ( pxCoverPoint )
		{
			pxCoverPoint->SetIsOccupied( false );
		}
	}

	if ( GetMarineBrain()->IsActive() )
	{
		GetMarineBrain()->GetTacticalSystem()->SetIsUsingCover( false );
		GetMarineBrain()->GetTacticalSystem()->SetStopShooting( false );
		GetMarineBrain()->GetHumanActor()->SetIsAllowedToFlinch( true );

		AVP_ClientActor_Human* pxHuman = AVP_ClientActor_Human::FindEntity( GetMarineBrain()->GetOwnerGuid() );

		if ( pxHuman )
		{
			if ( pxHuman->GetMovementSystem() && pxHuman->GetMovementSystem()->GetCurrentMove() )
			{
				Asura_Hash_ID uMoveHash = pxHuman->GetMovementSystem()->GetCurrentMove()->GetHash();

				if ( uMoveHash == AVP_Move_Human_EnterCover::GetStaticHash() ||
					uMoveHash == AVP_Move_Human_IntoPeekEmerge::GetStaticHash() ||
					uMoveHash == AVP_Move_Human_PeekEmerge::GetStaticHash() ||
					uMoveHash == AVP_Move_Human_StepOutOfCover::GetStaticHash() )
				{
					Asura_Message* pxMessage = Asura_Network::CreateMessage( ASURA_MSG_MOVE_ABORT );

					if ( pxMessage )
					{
						pxMessage->SetToGuid( GetMarineBrain()->GetOwnerGuid() );
						pxMessage->SetFromGuid( GetMarineBrain()->GetOwnerGuid() );

						Asura_Network::Send( pxMessage );
					}
				}
			}
		}
	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_InCover :: HandleMessage
//---------------------------------------------------------------------------
bool AVP_Behaviour_Marine_InCover::HandleMessage( const Asura_Message* pxMessage )
{
	switch ( pxMessage->GetMsgID() )
	{
		case ASURA_MSG_MOVECHAINCOMPLETE:
		{
			const Asura_Msg_MoveChainComplete* pxData = static_cast< const Asura_Msg_MoveChainComplete* >( pxMessage->GetData() );
			
			if ( pxData && pxData->GetInitialMoveID() == AVP_Move_Human_IntoPeekEmerge::GetStaticHash() )
			{
				if ( GetUpdateState() != STEP_OUT )
				{
					SwitchToState( STEP_OUT );
				}
			}

			if ( pxData && pxData->GetInitialMoveID() == AVP_Move_Human_EnterCover::GetStaticHash() )
			{
				if ( GetUpdateState() == ENTER_COVER )
				{
					m_bAdvanceState = true;
				}
			}

			return true;
		}

		default:
		{
			return PARENT::HandleMessage( pxMessage );
		}
	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_InCover :: PerFrameUpdate
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_InCover::PerFrameUpdate( const float fTimeslice )
{
	AVP_Tactical_System*	pxTactical = GetMarineBrain()->GetTacticalSystem();
	AVP_ServerActor_Human*	pxOurActor = GetMarineBrain()->GetHumanActor();

	switch (GetUpdateState())
	{
		case UNSET:
		{
			if ( IsInError() )
			{
				SetFinished();
				return;
			}

		}break;

		case ENTER_COVER:
		{
			m_fIdleTimer -= fTimeslice;

#ifndef ASURA_NO_DEBUG_OUTPUT
			m_fDebugRenderTimer = m_fIdleTimer;
#endif

			if ( m_fIdleTimer <= 0.0f )
			{
				if ( pxOurActor->GetCoverType() == AVP_Human_AIAnimStateMachine::COVER_TYPE_LOW && m_bAdvanceState )
				{
					SwitchToState( POPUP_FROM_COVER );
				}
				else if ( pxOurActor->GetCoverType() == AVP_Human_AIAnimStateMachine::COVER_TYPE_MID && m_bAdvanceState )
				{
					SelectPeekOrMove();
				}
				else if ( pxOurActor->GetCoverType() == AVP_Human_AIAnimStateMachine::COVER_TYPE_HIGH && m_bAdvanceState )
				{
					SelectEmergePoint();
				}
			}

			GetMarineBrain()->AimAtThreat();
			GetMarineBrain()->LookAtThreat();

		}break;

		case POPUP_FROM_COVER:
		{
			if ( IsInCoverPopup() )
			{
				pxTactical->SetStopShooting( false );

				GetMarineBrain()->AimAtThreat();
				GetMarineBrain()->LookAtThreat();

				m_fPeekTimer -= fTimeslice;

				if ( m_fPeekTimer <= 0.0f || 
					( pxOurActor->GetTimeSinceLastDamage() < 0.3f && pxOurActor->GetTimeSinceLastDamage() != 0.0f ) )// && Asura_Random::GetFloat(0.0f, 1.0f) > 0.9f ) )
				{
					SwitchToState( BACK_INTO_COVER );
				}
			}

#ifndef ASURA_NO_DEBUG_OUTPUT
			m_fDebugRenderTimer = m_fPeekTimer;
#endif

		}break;

		case BACK_INTO_COVER:
		{
			pxTactical->SetStopShooting( true );

			m_fIdleTimer -= fTimeslice;

			if ( m_fIdleTimer <= 0.0f )
			{
				if ( pxOurActor->GetCoverType() == AVP_Human_AIAnimStateMachine::COVER_TYPE_LOW )
				{
					SwitchToState( POPUP_FROM_COVER );
				}
				else if ( pxOurActor->GetCoverType() == AVP_Human_AIAnimStateMachine::COVER_TYPE_MID )
				{
					SelectPeekOrMove();
				}
				else if ( pxOurActor->GetCoverType() == AVP_Human_AIAnimStateMachine::COVER_TYPE_HIGH )
				{
					SelectEmergePoint();
				}
			}

#ifndef ASURA_NO_DEBUG_OUTPUT
			m_fDebugRenderTimer = m_fIdleTimer;
#endif

		}break;

		case PEEK_EMEREGE_FROM_COVER:
		{
			m_fIdleTimer -= fTimeslice;

			if ( m_fIdleTimer <= 0.0f )
			{
				SwitchToState( STEP_OUT );
			}

			if ( !pxTactical->GetStopShooting() )
			{
				pxTactical->SetStopShooting( true );
			}

#ifndef ASURA_NO_DEBUG_OUTPUT
			m_fDebugRenderTimer = m_fIdleTimer;
#endif

		}break;

		case STEP_OUT:
		{
			if ( GetMarineBrain()->GetHumanActor()->GetAIAnimStateMachine() )
			{
				if (GetMarineBrain()->GetHumanActor()->GetAIAnimStateMachine()->IsIdle() )
				{
					if ( pxTactical->GetStopShooting() )
					{
						pxTactical->SetStopShooting( false );
					}

					GetMarineBrain()->AimAtThreat();
					GetMarineBrain()->LookAtThreat();

					m_fPeekTimer -= fTimeslice;

					if ( m_fPeekTimer <= 0.0f )
					{
						SwitchToState( STEP_IN );
					}
				}
			}

#ifndef ASURA_NO_DEBUG_OUTPUT
			m_fDebugRenderTimer = m_fPeekTimer;
#endif
		}break;

		case STEP_IN:
		{
			if ( !pxTactical->GetStopShooting() )
			{
				pxTactical->SetStopShooting( true );
			}

			if ( IsInCoverIdle() )
			{
				SwitchToState( IN_COVER );
			}

		}break;

		case IN_COVER:
		{
			m_fIdleTimer -= fTimeslice;

			if ( m_fIdleTimer <= 0.0f )
			{
				SelectEmergePoint();
			}

#ifndef ASURA_NO_DEBUG_OUTPUT
			m_fDebugRenderTimer = m_fIdleTimer;
#endif

		}break;

		default:
		{

		}break;
	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_InCover :: SwitchToState
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_InCover::SwitchToState( UpdateState eState )
{
	AVP_ServerActor_Human*			pxOurActor		= GetMarineBrain()->GetHumanActor();
	Asura_ServerNode_Coverpoint*	pxCoverPoint	= Asura_ServerNode_Coverpoint::FindEntity( m_uCoverPoint );

	if ( !pxCoverPoint )
	{
		SetFinished();
		return;
	}

	switch ( GetUpdateState() )
	{
		case UNSET:
		case ENTER_COVER:
		{
			m_bAdvanceState = false;
		}break;
		case POPUP_FROM_COVER:
		case PEEK_EMEREGE_FROM_COVER:
		case STEP_OUT:
		case STEP_IN:
		case BACK_INTO_COVER:
		case IN_COVER:
		default:
		{

		}break;
	}

	SetUpdateState( eState );

	switch ( GetUpdateState() )
	{
		case UNSET:
		{

		}break;

		case ENTER_COVER:
		{
			if ( pxOurActor->GetCoverType() == AVP_Human_AIAnimStateMachine::COVER_TYPE_LOW )
			{
				m_fIdleTimer = Asura_Random::GetFloat( dbg_fMinInCoverPopupTime, dbg_fMaxInCoverPopupTime );
			}
			else if ( pxOurActor->GetCoverType() == AVP_Human_AIAnimStateMachine::COVER_TYPE_MID )
			{
				if ( pxCoverPoint->GetEmergePointCount() == 0 )
				{
					m_fIdleTimer = Asura_Random::GetFloat( dbg_fMinInCoverPopupTime, dbg_fMaxInCoverPopupTime );
				}
				else
				{
					m_fIdleTimer = Asura_Random::GetFloat( dbg_fMinInCoverStepOutTime, dbg_fMaxInCoverStepOutTime );
				}
			}
			else if ( pxOurActor->GetCoverType() == AVP_Human_AIAnimStateMachine::COVER_TYPE_HIGH )
			{
				m_fIdleTimer = Asura_Random::GetFloat( dbg_fMinInCoverStepOutTime, dbg_fMaxInCoverStepOutTime );
			}
			
			SendEnterCoverMoveMessage();

			m_bAdvanceState = false;
		
		}break;

		case POPUP_FROM_COVER:
		{
			GetMarineBrain()->GetTacticalSystem()->SetResetShootTimers( true );

			if ( pxOurActor->GetCoverType() == AVP_Human_AIAnimStateMachine::COVER_TYPE_LOW )
			{
				if ( GetMarineBrain()->GetCurrentWeaponType() == AVP_WEAPON_BEHAVIOUR_TYPE_SCOPED_RIFLE )
				{
					m_fPeekTimer = Asura_Random::GetFloat( 0.35f, 0.55f );
				}
				else
				{
					m_fPeekTimer = Asura_Random::GetFloat( 1.0f, 5.0f );
				}
			}
			else if ( pxOurActor->GetCoverType() == AVP_Human_AIAnimStateMachine::COVER_TYPE_MID )
			{
				if ( GetMarineBrain()->GetCurrentWeaponType() == AVP_WEAPON_BEHAVIOUR_TYPE_SCOPED_RIFLE )
				{
					m_fPeekTimer = Asura_Random::GetFloat( 0.35f, 0.55f );
				}
				else
				{
					m_fPeekTimer = Asura_Random::GetFloat( 1.0f, 5.0f );
				}
			}
			else
			{
				if ( GetMarineBrain()->GetCurrentWeaponType() == AVP_WEAPON_BEHAVIOUR_TYPE_SCOPED_RIFLE )
				{
					m_fPeekTimer = Asura_Random::GetFloat( 0.35f, 0.55f );
				}
				else
				{
					m_fPeekTimer = Asura_Random::GetFloat( 1.0f, 5.0f );
				}
			}

			RequestCoverAnimState( COVER_POPUP );

		}break;

		case PEEK_EMEREGE_FROM_COVER:
		{
			if ( pxOurActor->GetCoverType() == AVP_Human_AIAnimStateMachine::COVER_TYPE_LOW )
			{
				m_fIdleTimer = Asura_Random::GetFloat( dbg_fMinInCoverPopupTime, dbg_fMaxInCoverPopupTime );
			}
			else if ( pxOurActor->GetCoverType() == AVP_Human_AIAnimStateMachine::COVER_TYPE_MID )
			{
				if ( pxCoverPoint->GetEmergePointCount() == 0 )
				{
					if ( GetMarineBrain()->GetCurrentWeaponType() == AVP_WEAPON_BEHAVIOUR_TYPE_SCOPED_RIFLE )
					{
						m_fIdleTimer = Asura_Random::GetFloat( 0.35f, 0.55f );
					}
					else
					{
						m_fIdleTimer = Asura_Random::GetFloat( 1.0f, 5.0f );
					}
				}
				else
				{
					if ( GetMarineBrain()->GetCurrentWeaponType() == AVP_WEAPON_BEHAVIOUR_TYPE_SCOPED_RIFLE )
					{
						m_fIdleTimer = Asura_Random::GetFloat( 0.35f, 0.55f );
					}
					else
					{
						m_fIdleTimer = Asura_Random::GetFloat( 1.0f, 5.0f );
					}
				}
			}
			else if ( pxOurActor->GetCoverType() == AVP_Human_AIAnimStateMachine::COVER_TYPE_HIGH )
			{
				if ( GetMarineBrain()->GetCurrentWeaponType() == AVP_WEAPON_BEHAVIOUR_TYPE_SCOPED_RIFLE )
				{
					m_fIdleTimer = Asura_Random::GetFloat( 0.35f, 0.55f );
				}
				else
				{
					m_fIdleTimer = Asura_Random::GetFloat( 1.0f, 5.0f );
				}
			}

			SendPeekEmergeCoverMessage();
		}break;

		case STEP_OUT:
		{
			GetMarineBrain()->GetTacticalSystem()->SetResetShootTimers( true );

			if ( GetMarineBrain()->GetCurrentWeaponType() == AVP_WEAPON_BEHAVIOUR_TYPE_SCOPED_RIFLE )
			{
				m_fPeekTimer = Asura_Random::GetFloat( 0.35f, 0.55f );
			}
			else
			{
				m_fPeekTimer = Asura_Random::GetFloat( 1.0f, 5.0f );
			}
			SendStepOutOfCoverMessage();
		}break;

		case STEP_IN:
		{
			SendStepIntoCoverMessage();
		}break;

		case BACK_INTO_COVER:
		{
			if ( pxOurActor->GetCoverType() == AVP_Human_AIAnimStateMachine::COVER_TYPE_LOW )
			{
				m_fIdleTimer = Asura_Random::GetFloat( dbg_fMinInCoverPopupTime, dbg_fMaxInCoverPopupTime );
			}
			else if ( pxOurActor->GetCoverType() == AVP_Human_AIAnimStateMachine::COVER_TYPE_MID )
			{
				if ( pxCoverPoint->GetEmergePointCount() == 0 )
				{
					m_fIdleTimer = Asura_Random::GetFloat( dbg_fMinInCoverPopupTime, dbg_fMaxInCoverPopupTime );
				}
				else
				{
					m_fIdleTimer = Asura_Random::GetFloat( dbg_fMinInCoverStepOutTime, dbg_fMaxInCoverStepOutTime );
				}
			}
			else if ( pxOurActor->GetCoverType() == AVP_Human_AIAnimStateMachine::COVER_TYPE_HIGH )
			{
				m_fIdleTimer = Asura_Random::GetFloat( dbg_fMinInCoverStepOutTime, dbg_fMaxInCoverStepOutTime );
			}

			RequestCoverAnimState( COVER_IDLE );
		}break;

		case IN_COVER:
		{
			if ( pxOurActor->GetCoverType() == AVP_Human_AIAnimStateMachine::COVER_TYPE_LOW )
			{
				m_fIdleTimer = Asura_Random::GetFloat( dbg_fMinInCoverPopupTime, dbg_fMaxInCoverPopupTime );
			}
			else if ( pxOurActor->GetCoverType() == AVP_Human_AIAnimStateMachine::COVER_TYPE_MID )
			{
				if ( pxCoverPoint->GetEmergePointCount() == 0 )
				{
					m_fIdleTimer = Asura_Random::GetFloat( dbg_fMinInCoverPopupTime, dbg_fMaxInCoverPopupTime );
				}
				else
				{
					m_fIdleTimer = Asura_Random::GetFloat( dbg_fMinInCoverStepOutTime, dbg_fMaxInCoverStepOutTime );
				}
			}
			else if ( pxOurActor->GetCoverType() == AVP_Human_AIAnimStateMachine::COVER_TYPE_HIGH )
			{
				m_fIdleTimer = Asura_Random::GetFloat( dbg_fMinInCoverStepOutTime, dbg_fMaxInCoverStepOutTime );
			}

			LookInCoversDirection();
		}

		default:
		{

		}break;
	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_InCover :: SetCoverPoint
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_InCover::SetCoverPoint( Asura_Guid uGuid )
{
	// GBT : This function should be called right after creating the behaviour. you have just created it 
	// and got a pointer to it so set this so it has knows which cover point to use!

	AVP_ServerNode_CoverpointNode* pxCoverPoint = static_cast< AVP_ServerNode_CoverpointNode* >( AVP_ServerNode_CoverpointNode::FindEntity( uGuid ) );

	// Validate the cover point and set up data for the behaviour.
	if ( pxCoverPoint )
	{
		pxCoverPoint->SetIsOccupied( true );

		m_uCoverPoint = uGuid;

		if ( pxCoverPoint->GetIsLowCover() )
		{
			if (pxCoverPoint->GetLowCoverType() == AVP_ServerNode_CoverpointNode::MID_COVER)
			{
				GetMarineBrain()->GetHumanActor()->SetCoverType(AVP_Human_AIAnimStateMachine::COVER_TYPE_MID);
			}
			else
			{
				GetMarineBrain()->GetHumanActor()->SetCoverType(AVP_Human_AIAnimStateMachine::COVER_TYPE_LOW);
			}
		}
		else
		{
			GetMarineBrain()->GetHumanActor()->SetCoverType(AVP_Human_AIAnimStateMachine::COVER_TYPE_HIGH);
		}

		SwitchToState( ENTER_COVER );
	}
	else
	{
		SetInError();
	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_InCover :: RequestAnimStateCover
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_InCover::RequestCoverAnimState( AnimState eState )
{
	AVP_ServerActor_Human*			pxOurActor			= GetMarineBrain()->GetHumanActor();
	AVP_Human_AIAnimStateMachine*	pxAnimStateMachine	= ( AVP_Human_AIAnimStateMachine* )pxOurActor->GetAIAnimStateMachine();

	if ( !pxAnimStateMachine )
	{
		SetInError();
		SwitchToState( UNSET );
		return;
	}

	switch ( eState )
	{
		case COVER_IDLE:		if ( !pxAnimStateMachine->IsCoverIdle() )		{ pxAnimStateMachine->RequestCoverIdle(); }			return;
		case COVER_POPUP:		if ( !pxAnimStateMachine->IsCoverPopup() )		{ pxAnimStateMachine->RequestCoverPopup(); }		return;
		case IDLE:				if ( !pxAnimStateMachine->IsIdle() )			{ pxAnimStateMachine->RequestIdle(); }				return;
		default:				SetInError(); SetUpdateState( UNSET ); break;
	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_InCover :: IsInCoverIdle
//---------------------------------------------------------------------------
bool AVP_Behaviour_Marine_InCover::IsInCoverIdle()
{
	AVP_ServerActor_Human*			pxOurActor			= GetMarineBrain()->GetHumanActor();
	AVP_Human_AIAnimStateMachine*	pxAnimStateMachine	= ( AVP_Human_AIAnimStateMachine* )pxOurActor->GetAIAnimStateMachine();

	if ( !pxAnimStateMachine )
	{
		SetInError();
		SwitchToState( UNSET );
		return false;
	}

	if ( pxAnimStateMachine->IsCoverIdle() )
	{
		return true;
	}
	else
	{
		return false;
	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_InCover :: IsInCoverPopup
//---------------------------------------------------------------------------
bool AVP_Behaviour_Marine_InCover::IsInCoverPopup( void )
{
	AVP_ServerActor_Human*			pxOurActor			= GetMarineBrain()->GetHumanActor();
	AVP_Human_AIAnimStateMachine*	pxAnimStateMachine	= ( AVP_Human_AIAnimStateMachine* )pxOurActor->GetAIAnimStateMachine();

	if ( !pxAnimStateMachine )
	{
		SetInError();
		SwitchToState( UNSET );
		return false;
	}

	if ( pxAnimStateMachine->IsCoverPopup() )
	{
		return true;
	}
	else
	{
		return false;
	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_InCover :: LookIncoversDirection
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_InCover::LookInCoversDirection( void )
{
	if ( m_uCoverPoint == ASURA_GUID_UNREGISTERED )
	{
		SetInError();
		SwitchToState( UNSET );
		return;
	}

	Asura_ServerNode_Coverpoint* pxCoverPoint = Asura_ServerNode_Coverpoint::FindEntity( m_uCoverPoint );

	if ( !pxCoverPoint )
	{
		SetInError();
		SwitchToState( UNSET );
		return;
	}

	Asura_Matrix_3x3 xOri; 
	pxCoverPoint->GetOrientation( xOri );

	GetMarineBrain()->GetHumanActor()->TeleportTo( pxCoverPoint->GetPosition(), xOri, true );
}


//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_InCover :: SelectPeekOrMove
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_InCover::SelectPeekOrMove( void )
{
	Asura_ServerNode_Coverpoint* pxCoverPoint = Asura_ServerNode_Coverpoint::FindEntity( m_uCoverPoint );

	if ( !pxCoverPoint )
	{
		SetInError();
		SwitchToState( UNSET );
		return;
	}

	if ( pxCoverPoint->GetEmergePointCount() <= 0 )
	{
		SwitchToState( POPUP_FROM_COVER );
	}
	else
	{
		SelectEmergePoint();
	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_InCover :: SelectEmergePoint
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_InCover::SelectEmergePoint( void )
{
	Asura_ServerNode_Coverpoint* pxCoverPoint = Asura_ServerNode_Coverpoint::FindEntity( m_uCoverPoint );

	if ( !pxCoverPoint || pxCoverPoint->GetEmergePointCount() <= 0 )
	{
		SetInError();
		SwitchToState( UNSET );
		return;
	}

	u_int uNumCoverFirePositions = pxCoverPoint->GetEmergePointCount();
	m_uEmergePointIndex = Asura_Random::GetIntU( 0, uNumCoverFirePositions - 1 );
	SwitchToState( PEEK_EMEREGE_FROM_COVER );
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_InCover :: SendEnterCoverMoveMessage
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_InCover::SendEnterCoverMoveMessage( void )
{
	Asura_Message* pxMessage = Asura_Network::CreateMessage( ASURA_MSG_MOVE_START, sizeof( AVP_Message_Human_EnterCover ) );

	if ( pxMessage )
	{
		AVP_Message_Human_EnterCover* pxData = static_cast< AVP_Message_Human_EnterCover* >( pxMessage->GetData() );
		Asura_ServerNode_Coverpoint* pxCoverPoint = Asura_ServerNode_Coverpoint::FindEntity( m_uCoverPoint );

		if ( pxCoverPoint )
		{
			Asura_Matrix_3x3 xCoverOri;
			pxCoverPoint->GetOrientation( xCoverOri );

			pxMessage->SetFromGuid( GetMarineBrain()->GetOwnerGuid() );
			pxMessage->SetToGuid( GetMarineBrain()->GetOwnerGuid() );
			pxData->ClearFlags();
			pxData->SetMoveID( AVP_Move_Human_EnterCover::GetStaticHash() );
			pxData->SetDestinationPosition( pxCoverPoint->GetPosition() );
			pxData->SetDestinationOrientation( xCoverOri );
			pxData->SetAnimationHash( GetEnterCoverAnimHash() );

			AVP_ServerActor_Human* xOurActor = GetMarineBrain()->GetHumanActor();

			if ( xOurActor->GetAIAnimStateMachine() )
			{
				xOurActor->GetAIAnimStateMachine()->RequestInMove( pxMessage, AVP_Human_AIAnimStateMachine::HUMAN_STATE_COVER_IDLE );
			}
		}
		else
		{
			Asura_Network::DestroyMessage(pxMessage);
		}

	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_InCover :: GetEnterCoverAnimHash
//---------------------------------------------------------------------------
Asura_Hash_ID AVP_Behaviour_Marine_InCover::GetEnterCoverAnimHash()
{
	AVP_ServerNode_CoverpointNode* pxCoverPoint = static_cast< AVP_ServerNode_CoverpointNode* >( Asura_ServerNode_Coverpoint::FindEntity( m_uCoverPoint ) );
	
	if ( !pxCoverPoint )
	{
		return ASURA_HASH_ID_UNSET;
	}

	// First get the Actor blueprint
	AVP_ServerActor_Human*			pxOurActor			= GetMarineBrain()->GetHumanActor();
	AVP_Human_AIAnimStateMachine*	pxAnimStateMachine	= ( AVP_Human_AIAnimStateMachine* )pxOurActor->GetAIAnimStateMachine();

	if ( !pxAnimStateMachine )
	{
		Asura_Assert( pxAnimStateMachine, "AVP_Behaviour_Marine_InCover :: GetEnterCoverAnimHash - Human AI Anim State Machine is Invalid!" );
		return ASURA_HASH_ID_UNSET;
	}

	Asura_Blueprint* pxCoverBlueprint = GetCoverBlueprint( pxCoverPoint );

	if ( !pxCoverBlueprint )
	{
		Asura_Assert( pxCoverBlueprint, "AVP_Behaviour_Marine_InCover :: GetStepInAnimHash - Cover Blueprint is Invalid!" );
		return ASURA_HASH_ID_UNSET;
	}

	Asura_Value xValue;

	//-------------------------------------------------------------------------------------------------
	// From Idle / Walk
	//-------------------------------------------------------------------------------------------------
	if ( pxAnimStateMachine->IsMoveSlow() || pxAnimStateMachine->IsIdle() )
	{
		if ( pxCoverBlueprint )
		{
			if ( pxCoverBlueprint->GetParameterValueByHashID( 0x61a5d11a, xValue ) ) //"Move.Slow->Cover.Idle"
			{
				return xValue.GetHashIDValue();
			}
		}
	}

	//-------------------------------------------------------------------------------------------------
	// From Running / Sprinting
	//-------------------------------------------------------------------------------------------------
	if ( pxAnimStateMachine->IsMoveMedium() || pxAnimStateMachine->IsMoveFast() )
	{
		if ( pxCoverBlueprint )
		{
			if ( pxCoverBlueprint->GetParameterValueByHashID( 0x07e6bd4e, xValue ) ) //"Move.Medium->Cover.Idle"
			{
				return xValue.GetHashIDValue();
			}
		}
	}

	return ASURA_HASH_ID_UNSET;
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_InCover :: SendPeekEmergeCoverMessage
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_InCover::SendPeekEmergeCoverMessage( void )
{
	Asura_Message* pxMessage = Asura_Network::CreateMessage( ASURA_MSG_MOVE_START, sizeof( AVP_Message_Human_PeekEmerge ) );

	if ( pxMessage )
	{
		AVP_Message_Human_PeekEmerge* pxData = static_cast< AVP_Message_Human_PeekEmerge* >( pxMessage->GetData() );
		AVP_ServerNode_CoverpointNode* pxCoverPoint = static_cast< AVP_ServerNode_CoverpointNode* >( Asura_ServerNode_Coverpoint::FindEntity( m_uCoverPoint ) );

		if ( pxCoverPoint )
		{
			bool bIsLeft = IsPeekDirectionLeft(pxCoverPoint);

			pxMessage->SetFromGuid( GetMarineBrain()->GetOwnerGuid() );
			pxMessage->SetToGuid( GetMarineBrain()->GetOwnerGuid() );
			pxData->ClearFlags();
			pxData->SetMoveID( AVP_Move_Human_IntoPeekEmerge::GetStaticHash() );
			pxData->SetIntoPeekAnimationHash( GetIntoPeekAnimHash( bIsLeft ) );
			pxData->SetPeekAnimationHash( GetPeekAnimHash( bIsLeft ) );

			AVP_ServerActor_Human* xOurActor = GetMarineBrain()->GetHumanActor();

			if ( xOurActor->GetAIAnimStateMachine() )
			{
				Asura_Vector_3 xDir = pxCoverPoint->GetEmergePointPosition( m_uEmergePointIndex ) - pxCoverPoint->GetPosition();
				// get xDir in object space
				xDir = xOurActor->GetOrientation().ApplyTranspose( xDir );
				xOurActor->GetAIAnimStateMachine()->RequestInMove( pxMessage );
			}
			else
			{
				Asura_Network::DestroyMessage(pxMessage);
			}
		}
		else
		{
			Asura_Network::DestroyMessage(pxMessage);
		}
	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_InCover :: RenderDebugInfo
//---------------------------------------------------------------------------
Asura_Hash_ID AVP_Behaviour_Marine_InCover::GetIntoPeekAnimHash( bool bIsLeftPeek )
{
	AVP_ServerNode_CoverpointNode* pxCoverPoint = static_cast< AVP_ServerNode_CoverpointNode* >( Asura_ServerNode_Coverpoint::FindEntity( m_uCoverPoint ) );
	
	if ( !pxCoverPoint )
	{
		return ASURA_HASH_ID_UNSET;
	}

	Asura_Blueprint* pxCoverBlueprint = GetCoverBlueprint( pxCoverPoint );

	if ( !pxCoverBlueprint )
	{
		Asura_Assert( pxCoverBlueprint, "AVP_Behaviour_Marine_InCover :: GetStepInAnimHash - Cover Blueprint is Invalid!" );
		return ASURA_HASH_ID_UNSET;
	}

	Asura_Value xValue;

	//-------------------------------------------------------------------------------------------------
	// Left Peek-ing
	//-------------------------------------------------------------------------------------------------
	if ( bIsLeftPeek )
	{
		if ( pxCoverBlueprint )
		{
			if ( pxCoverBlueprint->GetParameterValueByHashID( 0xa210add5 , xValue ) ) //"Cover.Idle->Cover.PeekLeft"
			{
				return xValue.GetHashIDValue();
			}
		}
	}
	//-------------------------------------------------------------------------------------------------
	// right Peek-ing
	//-------------------------------------------------------------------------------------------------
	else
	{
		if ( pxCoverBlueprint )
		{
			if ( pxCoverBlueprint->GetParameterValueByHashID( 0xa05b6e0e, xValue ) ) //"Cover.Idle->Cover.PeekRight"
			{
				return xValue.GetHashIDValue();
			}
		}
	}

	return ASURA_HASH_ID_UNSET;
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_InCover :: RenderDebugInfo
//---------------------------------------------------------------------------
Asura_Hash_ID AVP_Behaviour_Marine_InCover::GetPeekAnimHash( bool bIsLeftPeek )
{
	AVP_ServerNode_CoverpointNode* pxCoverPoint = static_cast< AVP_ServerNode_CoverpointNode* >( Asura_ServerNode_Coverpoint::FindEntity( m_uCoverPoint ) );
	
	if ( !pxCoverPoint )
	{
		return ASURA_HASH_ID_UNSET;
	}

	Asura_Blueprint* pxCoverBlueprint = GetCoverBlueprint( pxCoverPoint );

	if ( !pxCoverBlueprint )
	{
		Asura_Assert( pxCoverBlueprint, "AVP_Behaviour_Marine_InCover :: GetStepInAnimHash - Cover Blueprint is Invalid!" );
		return ASURA_HASH_ID_UNSET;
	}

	Asura_Value xValue;

	//-------------------------------------------------------------------------------------------------
	// Left Peek-ing
	//-------------------------------------------------------------------------------------------------
	if ( bIsLeftPeek )
	{
		if ( pxCoverBlueprint )
		{
			if ( pxCoverBlueprint->GetParameterValueByHashID( 0x7e7ac839 , xValue ) ) //"Cover.PeekLeft"
			{
				return xValue.GetHashIDValue();
			}
		}
	}
	//-------------------------------------------------------------------------------------------------
	// right Peek-ing
	//-------------------------------------------------------------------------------------------------
	else
	{
		if ( pxCoverBlueprint )
		{
			if ( pxCoverBlueprint->GetParameterValueByHashID( 0x5134a02a, xValue ) ) //"Cover.PeekRight"
			{
				return xValue.GetHashIDValue();
			}
		}
	}

	return ASURA_HASH_ID_UNSET;
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_InCover :: IsPeekDirectionLeft
//---------------------------------------------------------------------------
bool AVP_Behaviour_Marine_InCover::IsPeekDirectionLeft( AVP_ServerNode_CoverpointNode* pxCoverPoint )
{
	const Asura_Vector_3&	xEmergePos			= pxCoverPoint->GetEmergePointPosition( m_uEmergePointIndex );
	const Asura_Vector_3&	xCoverPos			= pxCoverPoint->GetPosition();
	Asura_Matrix_3x3		xCoverOri;			pxCoverPoint->GetOrientation(xCoverOri);
	Asura_Vector_3			xLeft(xCoverOri.e11, xCoverOri.e21, xCoverOri.e31);
	Asura_Vector_3			xDirToEmergePoint	= xEmergePos - xCoverPos;
	float					fDistOrigToPlane	= 0.0f;
	float					fDistToPlane		= 0.0f;

	xDirToEmergePoint.Normalise();
	fDistOrigToPlane = xLeft * xCoverPos;
	fDistToPlane = fDistOrigToPlane - (xLeft * xEmergePos);

	if (fDistToPlane <= 0.0f)
	{
		return true; // is to the left
	}
	else
	{
		return false; // is to the right
	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_InCover :: SendStepOutOfCoverMessage
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_InCover::SendStepOutOfCoverMessage( void )
{
	Asura_Message* pxMessage = Asura_Network::CreateMessage( ASURA_MSG_MOVE_START, sizeof( AVP_Message_Human_StepOutOfCover ) );

	if ( pxMessage )
	{
		AVP_Message_Human_StepOutOfCover* pxData = static_cast< AVP_Message_Human_StepOutOfCover* >( pxMessage->GetData() );
		AVP_ServerNode_CoverpointNode* pxCoverPoint = static_cast< AVP_ServerNode_CoverpointNode* >( Asura_ServerNode_Coverpoint::FindEntity( m_uCoverPoint ) );

		if ( pxCoverPoint )
		{
			bool bIsLeft = IsPeekDirectionLeft(pxCoverPoint);

			pxMessage->SetFromGuid( GetMarineBrain()->GetOwnerGuid() );
			pxMessage->SetToGuid( GetMarineBrain()->GetOwnerGuid() );
			pxData->ClearFlags();
			pxData->SetMoveID( AVP_Move_Human_StepOutOfCover::GetStaticHash() );
			pxData->SetStepOutAnimHash( GetStepOutAnimHash( bIsLeft ) );

			AVP_ServerActor_Human* xOurActor = GetMarineBrain()->GetHumanActor();

			if ( xOurActor->GetAIAnimStateMachine() )
			{
				xOurActor->GetAIAnimStateMachine()->RequestInMove( pxMessage, AVP_Human_AIAnimStateMachine::ACTOR_STATE_IDLE );
			}
			else
			{
				Asura_Network::DestroyMessage(pxMessage);
			}

		}
		else
		{
			Asura_Network::DestroyMessage(pxMessage);
		}
	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_InCover :: GetStepOutAnimHash
//---------------------------------------------------------------------------
Asura_Hash_ID AVP_Behaviour_Marine_InCover::GetStepOutAnimHash( bool bIsLeftPeek )
{
	AVP_ServerNode_CoverpointNode* pxCoverPoint = static_cast< AVP_ServerNode_CoverpointNode* >( Asura_ServerNode_Coverpoint::FindEntity( m_uCoverPoint ) );
	
	if ( !pxCoverPoint )
	{
		return ASURA_HASH_ID_UNSET;
	}

	Asura_Blueprint* pxCoverBlueprint = GetCoverBlueprint( pxCoverPoint );

	if ( !pxCoverBlueprint )
	{
		Asura_Assert( pxCoverBlueprint, "AVP_Behaviour_Marine_InCover :: GetStepInAnimHash - Cover Blueprint is Invalid!" );
		return ASURA_HASH_ID_UNSET;
	}

	Asura_Value xValue;

	//-------------------------------------------------------------------------------------------------
	// Left Peek-ing
	//-------------------------------------------------------------------------------------------------
	if ( bIsLeftPeek )
	{
		if ( pxCoverBlueprint )
		{
			if ( pxCoverBlueprint->GetParameterValueByHashID( 0x1808344d , xValue ) ) //"Cover.PeekLeft.StepOut"
			{
				return xValue.GetHashIDValue();
			}
		}
	}
	//-------------------------------------------------------------------------------------------------
	// right Peek-ing
	//-------------------------------------------------------------------------------------------------
	else
	{
		if ( pxCoverBlueprint )
		{
			if ( pxCoverBlueprint->GetParameterValueByHashID( 0x2f678b3e, xValue ) ) //"Cover.PeekRight.StepOut"
			{
				return xValue.GetHashIDValue();
			}
		}
	}

	return ASURA_HASH_ID_UNSET;
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_InCover :: SendStepIntoCoverMessage
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_InCover::SendStepIntoCoverMessage( void )
{
	Asura_Message* pxMessage = Asura_Network::CreateMessage( ASURA_MSG_MOVE_START, sizeof( AVP_Message_Human_StepOutOfCover ) );

	if ( pxMessage )
	{
		AVP_Message_Human_StepOutOfCover* pxData = static_cast< AVP_Message_Human_StepOutOfCover* >( pxMessage->GetData() );
		AVP_ServerNode_CoverpointNode* pxCoverPoint = static_cast< AVP_ServerNode_CoverpointNode* >( Asura_ServerNode_Coverpoint::FindEntity( m_uCoverPoint ) );

		if ( pxCoverPoint )
		{
			bool bIsLeft = IsPeekDirectionLeft(pxCoverPoint);

			pxMessage->SetFromGuid( GetMarineBrain()->GetOwnerGuid() );
			pxMessage->SetToGuid( GetMarineBrain()->GetOwnerGuid() );
			pxData->ClearFlags();
			pxData->SetMoveID( AVP_Move_Human_StepOutOfCover::GetStaticHash() );
			pxData->SetStepOutAnimHash( GetStepInAnimHash( bIsLeft ) );

			AVP_ServerActor_Human* xOurActor = GetMarineBrain()->GetHumanActor();

			if ( xOurActor->GetAIAnimStateMachine() )
			{
				xOurActor->GetAIAnimStateMachine()->RequestInMove( pxMessage, AVP_Human_AIAnimStateMachine::HUMAN_STATE_COVER_IDLE );
			}
			else
			{
				Asura_Network::DestroyMessage(pxMessage);
			}
		}
		else
		{
			Asura_Network::DestroyMessage(pxMessage);
		}
	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_InCover :: GetStepInAnimHash
//---------------------------------------------------------------------------
Asura_Hash_ID AVP_Behaviour_Marine_InCover::GetStepInAnimHash( bool bIsLeftPeek )
{
	AVP_ServerNode_CoverpointNode* pxCoverPoint = static_cast< AVP_ServerNode_CoverpointNode* >( Asura_ServerNode_Coverpoint::FindEntity( m_uCoverPoint ) );
	
	if ( !pxCoverPoint )
	{
		return ASURA_HASH_ID_UNSET;
	}

	Asura_Blueprint* pxCoverBlueprint = GetCoverBlueprint( pxCoverPoint );

	if ( !pxCoverBlueprint )
	{
		Asura_Assert( pxCoverBlueprint, "AVP_Behaviour_Marine_InCover :: GetStepInAnimHash - Cover Blueprint is Invalid!" );
		return ASURA_HASH_ID_UNSET;
	}

	Asura_Value xValue;

	//-------------------------------------------------------------------------------------------------
	// Left Peek-ing
	//-------------------------------------------------------------------------------------------------
	if ( bIsLeftPeek )
	{
		if ( pxCoverBlueprint && pxCoverBlueprint->GetParameterValueByHashID( 0x7b6657ad , xValue ) ) //"Cover.StepBackLeft->Cover.Idle"
		{
			return xValue.GetHashIDValue();
		}
	}
	//-------------------------------------------------------------------------------------------------
	// right Peek-ing
	//-------------------------------------------------------------------------------------------------
	else
	{
		if ( pxCoverBlueprint && pxCoverBlueprint->GetParameterValueByHashID( 0x6359276e, xValue ) ) //"Cover.StepBackRight->Cover.Idle"
		{
			return xValue.GetHashIDValue();
		}
	}

	return ASURA_HASH_ID_UNSET;
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_InCover :: GetWeaponBlueprint
//---------------------------------------------------------------------------
Asura_Blueprint* AVP_Behaviour_Marine_InCover::GetWeaponBlueprint()
{
	// First get the Actor blueprint
	AVP_ServerActor_Human*			pxOurActor			= GetMarineBrain()->GetHumanActor();
	Asura_Blueprint*				pxActorBlueprint	= Asura_Blueprint_System::GetBlueprintByHashID( pxOurActor->GetBlueprintTypeHashID(), pxOurActor->GetBlueprintHashID() );

	if ( !pxActorBlueprint )
	{
		Asura_Assert( pxActorBlueprint, "AVP_Behaviour_Marine_InCover :: GetWeaponBlueprint - Actor Blueprint is Invalid!" );
		return NULL;
	}

	Asura_Value xValue;
	
	if ( pxActorBlueprint->GetParameterValueByHashID( uAVP_INVENTORY_WEAPONHASH_WEAPON01, xValue ) )
	{
		// Now get the weapon blueprint from the actor blueprint
		Asura_Hash_ID		uWeaponBPTypeHash	= uAVP_BLUEPRINT_TYPE_INVENTORY_ITEMS;
		Asura_Hash_ID		uWeaponBPHash		= xValue.GetHashIDValue();
		Asura_Blueprint*	pxWeaponBlueprint	= Asura_Blueprint_System::GetBlueprintByHashID( uWeaponBPTypeHash, uWeaponBPHash );

		if ( pxWeaponBlueprint )
		{
			return pxWeaponBlueprint;
		}
	}

	Asura_Assert( false, "AVP_Behaviour_Marine_InCover :: GetWeaponBlueprint - Weapon Blueprint is Invalid!" );
	return NULL;
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_InCover :: GetCoverBlueprint
//---------------------------------------------------------------------------
Asura_Blueprint* AVP_Behaviour_Marine_InCover::GetCoverBlueprint( AVP_ServerNode_CoverpointNode* pxCoverPoint )
{
	Asura_Blueprint* pxWeaponBlueprint = GetWeaponBlueprint();

	if ( !pxCoverPoint )
	{
		Asura_Assert( pxCoverPoint, "AVP_Behaviour_Marine_InCover :: GetCoverBlueprint - Cover Pointer is Invalid!" );
		return NULL;
	}

	if ( !pxWeaponBlueprint )
	{
		Asura_Assert( pxWeaponBlueprint, "AVP_Behaviour_Marine_InCover :: GetCoverBlueprint - Weapon Blueprint is Invalid!" );
		return NULL;
	}

	Asura_Hash_ID		uAnimStateMarineBPTypeHash	= GetMarineBrain()->GetActor()->GetAnimStateBlueprintType();
	Asura_Hash_ID		uAnimStateCoverBPHash		= ASURA_HASH_ID_UNSET;
	Asura_Blueprint*	pxCoverBlueprint			= NULL;
	Asura_Value			xValue;

	//-------------------------------------------------------------------------------------------------
	// HIGH COVER
	//-------------------------------------------------------------------------------------------------
	if ( !pxCoverPoint->GetIsLowCover() )
	{
		if ( pxWeaponBlueprint->GetParameterValueByHashID( 0x1e54a115, xValue ) ) //"AnimState.ThirdPerson.HighCover"
		{
			uAnimStateCoverBPHash = xValue.GetHashIDValue();
			pxCoverBlueprint = Asura_Blueprint_System::GetBlueprintByHashID( uAnimStateMarineBPTypeHash, uAnimStateCoverBPHash );

			if ( pxCoverBlueprint )
			{
				return pxCoverBlueprint;
			}
		}
	}
	//-------------------------------------------------------------------------------------------------
	// MEDIUM COVER
	//-------------------------------------------------------------------------------------------------
	else if ( pxCoverPoint->GetLowCoverType() == AVP_ServerNode_CoverpointNode::MID_COVER )
	{
		if ( pxWeaponBlueprint->GetParameterValueByHashID( 0x51d5ab22, xValue ) ) //"AnimState.ThirdPerson.MediumCover"
		{
			uAnimStateCoverBPHash = xValue.GetHashIDValue();
			pxCoverBlueprint = Asura_Blueprint_System::GetBlueprintByHashID( uAnimStateMarineBPTypeHash, uAnimStateCoverBPHash );

			if ( pxCoverBlueprint )
			{
				return pxCoverBlueprint;
			}
		}				
	}
	//-------------------------------------------------------------------------------------------------
	// LOW COVER
	//-------------------------------------------------------------------------------------------------
	else if ( pxCoverPoint->GetLowCoverType() == AVP_ServerNode_CoverpointNode::LOW_COVER )
	{
		if ( pxWeaponBlueprint->GetParameterValueByHashID( 0x5c09f723, xValue ) ) //"AnimState.ThirdPerson.LowCover"
		{
			uAnimStateCoverBPHash = xValue.GetHashIDValue();
			pxCoverBlueprint = Asura_Blueprint_System::GetBlueprintByHashID( uAnimStateMarineBPTypeHash, uAnimStateCoverBPHash );

			if ( pxCoverBlueprint )
			{
				return pxCoverBlueprint;
			}
		}
	}

	Asura_Assert( pxWeaponBlueprint, "AVP_Behaviour_Marine_InCover :: GetCoverBlueprint - Cover Blueprint is Invalid!" );
	return NULL;
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_InCover :: RenderDebugInfo
//---------------------------------------------------------------------------
#ifndef ASURA_NO_DEBUG_OUTPUT
static const char* s_aszStateStrings[ AVP_Behaviour_Marine_InCover::MAX_STATES ] =
{
	"COVER - UNSET",
	"COVER - ENTER COVER",
	"COVER - POPUP FROM COVER",
	"COVER - PEEK EMEREGE FROM COVER",
	"COVER - STEP OUT",
	"COVER - STEP IN",
	"COVER - BACK INTO COVER",
	"COVER - IN COVER"
};

void AVP_Behaviour_Marine_InCover::RenderDebugInfo()
{
	PARENT::RenderDebugInfo();

	Asura_Font_System::Print
	(
		GetProjectBrain()->GetDebugOutputPos().x,
		GetProjectBrain()->GetDebugOutputPos().y,
		0.5f, 
		AVP_Brain_Actor_Marine::s_uTemporaryChildDebugColour,
		ASURA_TEXTFORMAT_CENTREJUSTIFIED, 
		s_aszStateStrings[GetUpdateState()]
	);

	// Offset the debug output position for other behaviours to render there own text
	GetProjectBrain()->GetDebugOutputPos().y += 10.0f;

	Asura_Font_System::Print
	(
		GetProjectBrain()->GetDebugOutputPos().x,
		GetProjectBrain()->GetDebugOutputPos().y,
		0.5f, 
		0xffffffff,
		ASURA_TEXTFORMAT_CENTREJUSTIFIED | ASURA_TEXTFORMAT_FORMATSTRING, 
		"Timer : %.2f ", m_fDebugRenderTimer
	);
	// Offset the debug output position for other behaviours to render there own text
	GetProjectBrain()->GetDebugOutputPos().y += 30.0f;

	AVP_ClientActor_Human* pxHuman = AVP_ClientActor_Human::FindEntity( GetMarineBrain()->GetOwnerGuid() );

	if ( pxHuman )
	{
		if ( pxHuman->GetMovementSystem() )
		{
			if ( pxHuman->GetMovementSystem()->GetCurrentMove() )
			{
				Asura_Font_System::Print
				(
					GetProjectBrain()->GetDebugOutputPos().x,
					GetProjectBrain()->GetDebugOutputPos().y,
					1.0f, 
					0xff00ff00,
					ASURA_TEXTFORMAT_CENTREJUSTIFIED, 
					"IN MOVE"
				);
			}
		}
	}

}
#endif

/// End Of File 'AVP_behaviour_Marine_InCover.cpp'
