//---------------------------------------------------------------------------
// File			: AVP_Behaviour_Marine_Injured.cpp
// Created		: 
// Created By	: Gareth Thomas
// Copyright (C) 2007 Rebellion. All Rights Reserved.
//---------------------------------------------------------------------------

// INCLUDES ///////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"
// header
#include "Behaviour\BehaviourTypes\Human\Marine\AVP_Behaviour_Marine_Injured.h"
// asura
#ifndef ASURA_NO_DEBUG_OUTPUT
#include "Rendering/Asura_Font_System.h"
#endif
// axon
// AVP
#include "Entity/PhysicalObject/Actor/Vertebrates/Human/AVP_Messages_Human.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Human/AVP_ServerActor_Human.h"
#include "Moves/Vertebrates/Human/AVP_Human_Moves_MoveToFloor.h"

// AVP axon
#include "Behaviour/BehaviourTypes/AVP_Behaviour_Types.h"
#include "Core/BrainTypes/AVP_Brain_Actor_Marine.h"

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_Injured :: Constructor
//---------------------------------------------------------------------------
AVP_Behaviour_Marine_Injured::AVP_Behaviour_Marine_Injured( AVP_Brain_Actor_Marine* pxBrain )
: PARENT( pxBrain, AVP_BEHAVIOUR_TYPE_MARINE_INJURED )
, m_pxMarineBrain( pxBrain )
, m_eUpdateState( UNSET )
{

}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_Injured :: Deconstructor
//---------------------------------------------------------------------------
AVP_Behaviour_Marine_Injured::~AVP_Behaviour_Marine_Injured( void )
{

}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_Injured :: Activate
//---------------------------------------------------------------------------
bool AVP_Behaviour_Marine_Injured::Activate( void )
{
	const bool bParentOk = PARENT::Activate();
	// Put Code Below This Comment

	Asura_Guid uDamagGuid = GetMarineBrain()->GetHumanActor()->GetGuidToLastDamageUs();
	AVP_ServerEntity_Actor* pxDamageActor = AVP_ServerEntity_Actor::FindEntity( uDamagGuid );
	AVP_ServerActor_Human* pxOurActor = GetMarineBrain()->GetHumanActor();

	if ( pxOurActor->GetStartsInjured() )
	{
		(static_cast<AVP_Human_AIAnimStateMachine*>(pxOurActor->GetAIAnimStateMachine()))->RequestSittingIdle();

		SwitchToState( IDLE );
	}
	else
	{
		if ( pxDamageActor )
		{
			Asura_Vector_3 xDir( pxDamageActor->GetPosition() - pxOurActor->GetPosition() );
			xDir.Normalise();
			Asura_Matrix_3x3 xOri;
			Asura_Maths::OrientationFromDirection( xOri, xDir );
			pxOurActor->TeleportTo( pxOurActor->GetPosition(), xOri, true );
		}

		Asura_Message* pxMessage = Asura_Network::CreateMessage( ASURA_MSG_MOVE_START, sizeof( AVP_Message_Human_MoveToFloor ) );

		if ( pxMessage )
		{
			// Send move msg
			AVP_Message_Human_MoveToFloor* pxData = static_cast< AVP_Message_Human_MoveToFloor* >( pxMessage->GetData() );
			pxData->m_uAnimHash = 0xc722af58; //"Android_thwa_LooseLeg_to_sit"
			pxData->SetMoveID( AVP_Move_Human_MoveToFloor::GetStaticHash() );
			pxMessage->SetToGuid( GetMarineBrain()->GetOwnerGuid() );
			pxMessage->SetFromGuid( GetMarineBrain()->GetOwnerGuid() );
			pxOurActor->GetAIAnimStateMachine()->RequestInMove( pxMessage, AVP_Human_AIAnimStateMachine::HUMAN_STATE_SITTING_IDLE );
		}

		SwitchToState( IN_MOVE );
	}

	

	// Put Code Above This Comment
	return bParentOk;
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_Injured :: Deactivate
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_Injured::Deactivate( void )
{
	// Put Code Above This Comment
	DestroyAllChildren();
	PARENT::Deactivate();
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_Injured :: HandleMessage
//---------------------------------------------------------------------------
bool AVP_Behaviour_Marine_Injured::HandleMessage( const Asura_Message* pxMessage )
{
	switch ( pxMessage->GetMsgID() )
	{
		case ASURA_MSG_MOVECHAINCOMPLETE:
		{
			const Asura_Msg_MoveChainComplete* pxData = static_cast< const Asura_Msg_MoveChainComplete* >( pxMessage->GetData() );

			if ( pxData && pxData->GetInitialMoveID() == AVP_Move_Human_MoveToFloor::GetStaticHash() )
			{
				SwitchToState( IDLE );
			}

			return true;
		}

		default:
		{
			return PARENT::HandleMessage( pxMessage );
		}
	}
}


//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_Injured :: PerFrameUpdate
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_Injured::PerFrameUpdate( const float fTimeslice )
{
	switch (GetUpdateState())
	{
		case UNSET:
		{

		}break;

		case IDLE:
		{
			if ( GetMarineBrain()->DoesHaveThreat() )
			{
				GetMarineBrain()->AimAtThreat();
				GetMarineBrain()->LookAtThreat();
				return;
			}
			else if ( GetMarineBrain()->DoesHaveAlert() )
			{
				AVP_ServerEntity_Actor* pxActor = AVP_ServerEntity_Actor::FindEntity( GetMarineBrain()->GetAlertGuid() );

				if ( pxActor && pxActor->GetGuid() != GetMarineBrain()->GetOwnerGuid() )
				{
					Asura_Vector_3 xChest( Asura_Maths::ZeroVector3 );
					Asura_Vector_3 xEyes( Asura_Maths::ZeroVector3 );
					pxActor->GetChestPosition( xChest );
					pxActor->GetHeadPosition( xEyes );
					GetMarineBrain()->AimAtPosition( xChest );
					GetMarineBrain()->SetLookAtPosition( xEyes );
					return;
				}
			}

			GetMarineBrain()->AimForward();
			GetMarineBrain()->LookForward();

		}break;

		case IN_MOVE:
		{

		}break;

		default:
		{

		}
	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_Injured :: SwitchToState
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_Injured::SwitchToState( UpdateState eState )
{
	switch (GetUpdateState())
	{
		case UNSET:
		case IDLE:
		case IN_MOVE:
		default:
		{

		}
	}

	SetUpdateState( eState );

	switch (GetUpdateState())
	{
		case UNSET:
		case IDLE:
		case IN_MOVE:
		default:
		{

		}
	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_Injured :: RenderDebugInfo
//---------------------------------------------------------------------------
#ifndef ASURA_NO_DEBUG_OUTPUT
static const char* s_aszStateStrings[ AVP_Behaviour_Marine_Injured::MAX_STATES ] =
{
	"Injured - UNSET",
	"Injured - IDLE",
	"Injured - IN_MOVE",
};

void AVP_Behaviour_Marine_Injured::RenderDebugInfo( void )
{
	PARENT::RenderDebugInfo();

	Asura_Font_System::Print
	(
		GetProjectBrain()->GetDebugOutputPos().x,
		GetProjectBrain()->GetDebugOutputPos().y,
		0.5f, 
		AVP_Brain_Actor_Marine::s_uTemporaryChildDebugColour,
		ASURA_TEXTFORMAT_CENTREJUSTIFIED, 
		s_aszStateStrings[GetUpdateState()]
	);

	// Offset the debug output position for other behaviours to render there own text
	GetProjectBrain()->GetDebugOutputPos().y += 10.0f;
}
#endif

/// End Of File 'AVP_behaviour_Marine_Injured.cpp'
