//---------------------------------------------------------------------------
// File			: AVP_Behaviour_Marine_Investigate.cpp
// Created		: 04 / 02 / 08
// Created By	: Gareth Thomas
// Copyright (C) 2007 Rebellion. All Rights Reserved.
//---------------------------------------------------------------------------

// INCLUDES ///////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"
// header
#include "Behaviour/BehaviourTypes/Human/Marine/AVP_Behaviour_Marine_Investigate.h"
// asura
#ifndef ASURA_NO_DEBUG_OUTPUT
#include "Rendering/Asura_Font_System.h"
#endif
// axon
// AVP
#include "Entity/PhysicalObject/Actor/Vertebrates/Human/AVP_ServerActor_Human.h"
// AVP axon
#include "Behaviour/BehaviourTypes/AVP_Behaviour_Types.h"
#include "Core/BrainTypes/AVP_Brain_Actor_Marine.h"
#include "Knowledge/AVP_Knowledge_System_Actor.h"
#include "Tactical/AVP_Tactical_System.h"


//---------------------------------------------------------------------------------
//*********************************************************************************
//---------------------------------------------------------------------------------
// Class		: AVP_Behaviour_Marine_Investigate
// Description	: 
//---------------------------------------------------------------------------------
//*********************************************************************************
//---------------------------------------------------------------------------------

Asura_Guid AVP_Behaviour_Marine_Investigate::s_uInvestigatorGuid = ASURA_GUID_UNREGISTERED;

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_Investigate :: Constructor
//---------------------------------------------------------------------------
AVP_Behaviour_Marine_Investigate::AVP_Behaviour_Marine_Investigate(AVP_Brain_Actor_Marine* pxBrain)
: PARENT(pxBrain, AVP_BEHAVIOUR_TYPE_MARINE_INVESTIGATE)
, m_eUpdateState(UNSET)
, m_pxMarineBrain(pxBrain)
, m_xAlertPosition(Asura_Maths::ZeroVector3)
, m_fInvestigateTimer(0.0f)
, m_uSquadGuid(ASURA_GUID_UNREGISTERED)
, m_xLookAt(Asura_Maths::ZeroVector3)
, m_bGetNewLookAtPos(false)
, m_uOurActorGuid(ASURA_GUID_UNREGISTERED)
, m_fTimeInBehaviour( 0.0f )
, m_bIsRecon( false )
{

}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_Investigate :: Activate
//---------------------------------------------------------------------------
bool AVP_Behaviour_Marine_Investigate::Activate()
{
	const bool bParentOk = PARENT::Activate();

	SetUpdateState(FIND_ROUTE_TO_ALERT);

	m_uSquadGuid	= GetMarineBrain()->GetHumanActor()->GetInitialSquadGuid();
	m_uOurActorGuid	= GetMarineBrain()->GetHumanActor()->GetGuid();

	s_uInvestigatorGuid = m_uOurActorGuid;

	return bParentOk;
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_Investigate :: Deactivate
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_Investigate::Deactivate()
{
	DestroyAllChildren();

	s_uInvestigatorGuid = ASURA_GUID_UNREGISTERED;

	if ( GetMarineBrain()->IsActive() && m_fTimeInBehaviour > 0.5f )
	{
		GetMarineBrain()->GetKnowledgeSystem()->ResetReconKnowledge();
		GetMarineBrain()->GetTacticalSystem()->SetAtDestinationRange( 1.0f );
		GetMarineBrain()->GetTacticalSystem()->SetIsAllowedToReact( false );
	}

	PARENT::Deactivate();
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_Investigate :: Pause
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_Investigate::Pause()
{
	PARENT::Pause();
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_Investigate :: Unpause
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_Investigate::Unpause()
{
	PARENT::Unpause();
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_Investigate :: PerFrameUpdate
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_Investigate::PerFrameUpdate(const float fTimeslice)
{
	AVP_Knowledge_System_Actor*		pxKnowledgeSystem	= GetMarineBrain()->GetKnowledgeSystem();
	AVP_Tactical_System*			pxTactical			= GetMarineBrain()->GetTacticalSystem();
	AVP_ServerActor_Human*			pxOurActor			= GetMarineBrain()->GetHumanActor();
	AVP_ServerEntity_Squad_Marine*	pxSquad				= AVP_ServerEntity_Squad_Marine::FindEntity(pxOurActor->GetInitialSquadGuid());
	bool							bHaveThreat			= false;
	bool							bIsSquadLeader		= false;

	m_fTimeInBehaviour += fTimeslice;

	if (pxSquad)
	{
		bIsSquadLeader = (pxSquad->GetSquadLeaderGuid() == pxOurActor->GetGuid());
	}

	if (pxKnowledgeSystem->GetBestThreatKnowledge().GetGuid() != ASURA_GUID_UNREGISTERED)
	{
		bHaveThreat = true;
	}

	if ( IsFinished() )
	{
		return;
	}

	switch (GetUpdateState())
	{
		//-------------------------------------------------------------
		case FIND_ROUTE_TO_ALERT:
		{
			if ( pxKnowledgeSystem->DoesHaveReconKnowledge() )
			{
				m_bIsRecon = true;
			}

			m_xAlertPosition = pxKnowledgeSystem->GetBestAlertKnowledge().m_xPosition;

			m_xAlertPosition.x += Asura_Random::GetFloat(-1.0f, +1.0f);
			m_xAlertPosition.z += Asura_Random::GetFloat(-1.0f, +1.0f);

			pxTactical->SetDestination(m_xAlertPosition);
			pxTactical->LetRouteSetLookAt(true);
			pxTactical->SetMovementSpeed(AVP_Navigation::MM_SLOW);
			pxTactical->SetAtDestinationRange( 1.0f );
			pxTactical->UseAcurateDistanceCheck( false );
			pxTactical->SetShouldStopAtDestination( true );
			SetUpdateState(MOVE_TO_ALERT);

		}break;
		//-------------------------------------------------------------
		case MOVE_TO_ALERT:
		{

			if ( pxTactical->DidRouteFindFail() )
			{
				m_fInvestigateTimer = 0.0f;
				m_bGetNewLookAtPos = true;
				GetMarineBrain()->GetKnowledgeSystem()->SetAlertLevel( AVP_Knowledge_System_Actor::AGGRESSIVE_ALERTNESS );
				GetMarineBrain()->GetKnowledgeSystem()->ResetAlertTimer();
				SetUpdateState(INVESTIGATE);
				return;
			}

			if (pxTactical->HasArrivedAtDestination())
			{
				m_fInvestigateTimer = 0.0f;
				m_bGetNewLookAtPos = true;
				GetMarineBrain()->GetKnowledgeSystem()->SetAlertLevel( AVP_Knowledge_System_Actor::AGGRESSIVE_ALERTNESS );
				GetMarineBrain()->GetKnowledgeSystem()->ResetAlertTimer();
				SetUpdateState(INVESTIGATE);
			}

			if ((bHaveThreat) && (!bIsSquadLeader))
			{
				SetUpdateState(FIND_ROUTE_TO_SQUAD);
			}
			else if ((bHaveThreat) && (bIsSquadLeader))
			{
				SetFinished();
				return;
			}

		}break;
		//-------------------------------------------------------------
		case INVESTIGATE:
		{
			m_fInvestigateTimer += fTimeslice;

			GetMarineBrain()->GetTacticalSystem()->SetIsAllowedToReact( true );

			if ( m_fInvestigateTimer > 10.0f )
			{
				if (!bIsSquadLeader)
				{
					GetMarineBrain()->GetHumanActor()->PlayDialogue( AVP_ServerActor_Human::AVP_DLG_HUMAN_ALERT_TO_PASSIVE );

					SetUpdateState(FIND_ROUTE_TO_SQUAD);
				}
				else
				{
					GetMarineBrain()->GetHumanActor()->PlayDialogue( AVP_ServerActor_Human::AVP_DLG_HUMAN_ALERT_TO_PASSIVE );

					SetFinished();
					return;
				}
			}

			if (bHaveThreat)
			{
				SetUpdateState(FIND_ROUTE_TO_SQUAD);
			}
		}break;
		//-------------------------------------------------------------
		case FIND_ROUTE_TO_SQUAD:
		{
			GetMarineBrain()->GetTacticalSystem()->SetIsAllowedToReact( false );

			if ( pxSquad )
			{
				AVP_ServerActor_Human* pxSquadLeader = AVP_ServerActor_Human::FindEntity(pxSquad->GetSquadLeaderGuid());

				if (pxSquadLeader)
				{
					pxTactical->SetDestination(pxSquadLeader->GetPosition());

					if (!bHaveThreat)
					{
						pxTactical->LetRouteSetLookAt(true);
					}
					else
					{
						pxTactical->LetRouteSetLookAt(false);
						GetMarineBrain()->AimAtThreat();
						GetMarineBrain()->LookAtThreat();
						pxTactical->SetMovementSpeed(AVP_Navigation::MM_MEDIUM);
					}

					pxTactical->SetAtDestinationRange(7.0f);

					SetUpdateState(MOVE_TO_SQUAD);
				}
			}
			else
			{
				SetFinished();
			}

		}break;
		//-------------------------------------------------------------
		case MOVE_TO_SQUAD:
		{
			if (pxTactical->HasArrivedAtDestination())
			{
				SetFinished();
				return;
			}

			if (!bHaveThreat)
			{
				pxTactical->LetRouteSetLookAt(true);
			}
			else
			{
				pxTactical->LetRouteSetLookAt(false);
				GetMarineBrain()->AimAtThreat();
				GetMarineBrain()->LookAtThreat();
			}
		}break;
		//-------------------------------------------------------------
		case ALERT:
		{
			Axon_Behaviour* pxAlertBeahviour = GetChildWithType(this, AVP_BEHAVIOUR_TYPE_MARINE_ALERT);

			if (pxAlertBeahviour)
			{
				if (pxAlertBeahviour->IsFinished())
				{
					DestroyChildWithType(AVP_BEHAVIOUR_TYPE_MARINE_ALERT);

					if (!bIsSquadLeader)
					{
						SetUpdateState(FIND_ROUTE_TO_SQUAD);
					}
					else
					{
						SetFinished();
						return;
					}
				}
			}
		}break;
		//-------------------------------------------------------------
		case UNSET:
		default:
		{

		}
	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_Investigate :: RenderDebugInfo
//---------------------------------------------------------------------------
#ifndef ASURA_NO_DEBUG_OUTPUT
static const char* s_aszStateStrings[AVP_Behaviour_Marine_Investigate::MAX_STATES] =
{
	"INVESTIGATE - UNSET",
	"INVESTIGATE - FIND_ROUTE_TO_ALERT",
	"INVESTIGATE - MOVE_TO_ALERT",
	"INVESTIGATE - INVESTIGATE",
	"INVESTIGATE - FIND_ROUTE_TO_SQUAD",
	"INVESTIGATE - MOVE_TO_SQUAD",
	"INVESTIGATE - ALERT",
};

void AVP_Behaviour_Marine_Investigate::RenderDebugInfo()
{
	PARENT::RenderDebugInfo();

	Asura_Font_System::Print
	(
		GetProjectBrain()->GetDebugOutputPos().x,
		GetProjectBrain()->GetDebugOutputPos().y,
		0.5f, 
		AVP_Brain_Actor_Marine::s_uTemporaryChildDebugColour,
		ASURA_TEXTFORMAT_CENTREJUSTIFIED, 
		s_aszStateStrings[GetUpdateState()]
	);

	// Offset the debug output position for other behaviours to render there own text
	GetProjectBrain()->GetDebugOutputPos().y += 10.0f;
}
#endif

/// End Of File 'AVP_behaviour_Marine_Investigate.cpp'
