//---------------------------------------------------------------------------
// File			: AVP_Behaviour_Marine_MeleePlayer.cpp
// Created		: 
// Created By	: Gareth Thomas
// Copyright (C) 2007 Rebellion. All Rights Reserved.
//---------------------------------------------------------------------------

// INCLUDES ///////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"
// header
#include "Behaviour/BehaviourTypes/Human/Marine/AVP_Behaviour_Marine_MeleePlayer.h"
// asura
#ifndef ASURA_NO_DEBUG_OUTPUT
#include "Rendering/Asura_Font_System.h"
#endif
// axon
// AVP
#include "Actor/Vertebrates/AVP_Vertebrate_MeleeCounter_System.h"
#include "Actor/Vertebrates/Human/AVP_Human_MeleeCounter_System.h"
#include "Entity/AVP_Messages.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/AVP_Vertebrate_Messages.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Human/AVP_ServerActor_Human.h"
#include "Entity/PlayerInfo/AVP_ServerEntity_PlayerInfo.h"
#include "Moves/AVP_Moves.h"
#include "Moves/Vertebrates/Human/AVP_Human_MovesMelee.h"

#include "Entity/PhysicalObject/Actor/Vertebrates/Human/AVP_ClientActor_Human.h"

// AVP axon
#include "Behaviour/BehaviourTypes/AVP_Behaviour_Types.h"
#include "Core/BrainTypes/AVP_Brain_Actor_Marine.h"
#include "Tactical/AVP_Tactical_System.h"

DEBUGVAR float fAttckRngSq = 2.5f;

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_MeleePlayer :: Constructor
//---------------------------------------------------------------------------
AVP_Behaviour_Marine_MeleePlayer::AVP_Behaviour_Marine_MeleePlayer( AVP_Brain_Actor_Marine* pxBrain )
: PARENT( pxBrain, AVP_BEHAVIOUR_TYPE_MARINE_MELEE_PLAYER )
, m_eUpdateState( UNSET )
, m_pxMarineBrain( pxBrain )
, m_uPlayerGuid( ASURA_GUID_UNREGISTERED )
, m_bMeleeRequested( false )
, m_fWaitTimer( 0.0f )
, m_bFlinchFinished( false )
{

}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_MeleePlayer :: Deconstructor
//---------------------------------------------------------------------------
AVP_Behaviour_Marine_MeleePlayer::~AVP_Behaviour_Marine_MeleePlayer( void )
{

}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_MeleePlayer :: Activate
//---------------------------------------------------------------------------
bool AVP_Behaviour_Marine_MeleePlayer::Activate( void )
{
	const bool bParentOk = PARENT::Activate();
	// Put Code Below This Comment

	AVP_ServerEntity_PlayerInfo* pxPlayerInfo = AVP_ServerEntity_PlayerInfo::FindEntity( 1000 );

	if ( pxPlayerInfo )
	{
		m_uPlayerGuid = pxPlayerInfo->GetPuppetGuid();
	}

	if ( m_uPlayerGuid != ASURA_GUID_UNREGISTERED )
	{
		if ( IsInAttackRange() )
		{
			SwitchToState( TURN_TO_FACE );
		}
		else
		{
			// move to player
			SwitchToState( MOVE_TO_PLAYER );
		}
	}

	GetTacticalSystem()->SetForceAiming( true );

	// Put Code Above This Comment
	return bParentOk;
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_MeleePlayer :: Deactivate
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_MeleePlayer::Deactivate( void )
{
	if ( GetMarineBrain()->IsActive() )
	{
		GetTacticalSystem()->SetForceAiming( false );
	}

	// Put Code Above This Comment
	DestroyAllChildren();
	PARENT::Deactivate();
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_MeleePlayer :: PerFrameUpdate
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_MeleePlayer::PerFrameUpdate( const float fTimeslice )
{
	GetMarineBrain()->LookAtPlayerHeadPosition();

	switch (GetUpdateState())
	{
		case MOVE_TO_PLAYER:
		{
			LookAtPlayer();

			if ( !IsInAttackRange() )
			{
				SwitchToState( MOVE_TO_PLAYER );
			}
			else
			{
				if ( GetTacticalSystem()->HasArrivedAtDestination() )
				{
					SwitchToState( TURN_TO_FACE );
				}
			}

		}break;
		case TURN_TO_FACE:
		{
			if ( IsInAttackRange() )
			{
				if ( IsLookingAtPlayer() )
				{
					m_fWaitTimer -= fTimeslice;

					if ( m_fWaitTimer <= 0.0f )
					{
						SwitchToState( MELEE_REQUESTED );
					}
				}
				else
				{
					LookAtPlayer();
				}
			}
			else
			{
				LookAtPlayer();
				SwitchToState( MOVE_TO_PLAYER );
			}
		}break;
		case MELEE_REQUESTED:
		{
			if ( m_fWaitTimer > 0.0f )
			{
				m_fWaitTimer -= fTimeslice;
			}
			else if ( m_fWaitTimer <= 0.0f && m_bMeleeRequested == false && m_bFlinchFinished == true )
			{
				if ( IsInAttackRange() )
				{
					m_bMeleeRequested = true;

					Asura_Message* pxMessage = Asura_Network::CreateMessage(AVP_MSGID_REQUEST_MELEE, sizeof(AVP_Message_Vertebrate_RequestMelee));
				
					if (pxMessage)
					{
						AVP_Message_Vertebrate_RequestMelee* pxData = static_cast<AVP_Message_Vertebrate_RequestMelee*>(pxMessage->GetData());

						pxData->SetType( AVP_HUMAN_MELEEATTACK_TYPE_RETALIATION );
						pxData->SetTargetGuid( m_uPlayerGuid );

						pxMessage->SetToGuid( GetMarineBrain()->GetOwnerGuid() );
						pxMessage->SetFromGuid( GetMarineBrain()->GetOwnerGuid() );
						Asura_Network::Send( pxMessage );
						
						GetOurActor()->PlayDialogue( AVP_ServerActor_Human::AVP_DLG_HUMAN_SCARED_REACTION );
					}
				}
				else
				{
					SwitchToState( MOVE_TO_PLAYER );
				}
			}

		}break;

		case POST_ATTACK:
		{
			GetMarineBrain()->LookAtPlayerHeadPosition();
			LookAtPlayer();

			m_fWaitTimer -= fTimeslice;

			if ( m_fWaitTimer <= 0.0f )
			{
				SetFinished();
			}
		}
		case UNSET:
		default:
		{

		}
	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_MeleePlayer :: HandleMessage
//---------------------------------------------------------------------------
bool AVP_Behaviour_Marine_MeleePlayer::HandleMessage(const Asura_Message* pxMessage)
{
	switch ( pxMessage->GetMsgID() )
	{
		case ASURA_MSG_MOVECHAINCOMPLETE:
		{
			const Asura_Msg_MoveChainComplete* pxData = static_cast< const Asura_Msg_MoveChainComplete* >( pxMessage->GetData() );

			if ( pxData && pxData->GetInitialMoveID() == AVP_Move_Flinch::GetStaticHash() )
			{
				m_fWaitTimer = 0.0f;
				m_bFlinchFinished = true;
			}
			else if (pxData && pxData->GetInitialMoveID() == AVP_Human_Move_Melee_Retaliation_Start::GetStaticHash() )
			{
				SwitchToState( POST_ATTACK );
			}

			return true;
		}

		default:
		{
			return PARENT::HandleMessage( pxMessage );
		}
	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_MeleePlayer :: IsPlayerToFarAwayFromOurDestination
//---------------------------------------------------------------------------
bool AVP_Behaviour_Marine_MeleePlayer::IsPlayerToFarAwayFromOurDestination( void )
{
	const AVP_Navigation::NavData& xNavData = GetMarineBrain()->GetNavData();

	AVP_ServerActor_Human* pxPlayer = AVP_ServerActor_Human::FindEntity( m_uPlayerGuid );

	if ( pxPlayer )
	{
		float fDistanceSq = ( xNavData.m_xDestinationPosition - pxPlayer->GetPosition() ).MagnitudeSquared();

		if ( fDistanceSq > fAttckRngSq )
		{
			return true;
		}
	}

	return false;
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_MeleePlayer :: LookAtPlayer
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_MeleePlayer::LookAtPlayer()
{
	AVP_ServerActor_Human* pxPlayerActor = AVP_ServerActor_Human::FindEntity( m_uPlayerGuid );

	if ( pxPlayerActor )
	{
		Asura_Vector_3 xPlayerHeadPos( Asura_Maths::ZeroVector3 );
		pxPlayerActor->GetHeadPosition( xPlayerHeadPos );
		GetMarineBrain()->AimAtPosition( xPlayerHeadPos );
	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_MeleePlayer :: IsInAttackRange
//---------------------------------------------------------------------------
bool AVP_Behaviour_Marine_MeleePlayer::IsInAttackRange()
{
	AVP_ServerActor_Human* pxPlayerActor = AVP_ServerActor_Human::FindEntity( m_uPlayerGuid );

	if ( pxPlayerActor )
	{
		float					fDistanceSq			= ( pxPlayerActor->GetPosition() - GetOurActor()->GetPosition() ).MagnitudeSquared();

		if ( fDistanceSq < fAttckRngSq )
		{
			return true;
		}
	}

	return false;
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_MeleePlayer :: IsLookingAtPlayer
//---------------------------------------------------------------------------
bool AVP_Behaviour_Marine_MeleePlayer::IsLookingAtPlayer()
{
	AVP_ServerActor_Human* pxPlayerActor = AVP_ServerActor_Human::FindEntity( m_uPlayerGuid );

	if ( pxPlayerActor )
	{
		AVP_ClientActor_Human* pxOurClient = AVP_ClientActor_Human::FindEntity( GetOurActor()->GetGuid() );

		if (pxOurClient && pxOurClient->GetAnimationHandler())
		{
			if ( !pxOurClient->GetAnimationHandler()->GetBodyRig()->IsTurning() )
			{

				Asura_Vector_3			xDirectionToPlayer( pxPlayerActor->GetPosition() - GetOurActor()->GetPosition() );
				xDirectionToPlayer.Normalise();

				const Asura_Matrix_3x3& xOri = GetOurActor()->GetOrientation();
				Asura_Vector_3 xFwd( xOri.e13, xOri.e23, xOri.e33 );
				float fDot = xFwd * xDirectionToPlayer;

				if ( fDot > 0.5f )
				{
					// angle is good enough
					return true;
				}
			}
		}
	}

	return false;
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_MeleePlayer :: SwitchToState
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_MeleePlayer::SwitchToState( UpdateState eState )
{
	bool bComeFromOtherState = false;

	switch (GetUpdateState())
	{
		case MOVE_TO_PLAYER:
		{
			if ( eState != MOVE_TO_PLAYER )
			{
				GetTacticalSystem()->StopMoving();
			}
			
			bComeFromOtherState = true;

		}break;
		case TURN_TO_FACE:
		{
			bComeFromOtherState = true;

		}break;
		case MELEE_REQUESTED:
		{
		}break;
		case POST_ATTACK:
		{
		}break;
		case UNSET:
		default:
		{

		}
	}

	SetUpdateState( eState );

	switch (GetUpdateState())
	{
		case MOVE_TO_PLAYER:
		{
			AVP_ServerActor_Human* pxPlayer = AVP_ServerActor_Human::FindEntity( m_uPlayerGuid );

			if (pxPlayer)
			{
				GetTacticalSystem()->SetDestination( pxPlayer->GetPosition() );
				GetTacticalSystem()->SetMovementSpeed( AVP_Navigation::MM_MEDIUM );
				GetTacticalSystem()->LetRouteSetLookAt( false );
				GetTacticalSystem()->SetAtDestinationRange( 1.5f );
				GetTacticalSystem()->OverrideStartMovementDelay();
			}
			
		}break;
		case TURN_TO_FACE:
		{
			m_fWaitTimer = 0.0f;
			
		}break;
		case MELEE_REQUESTED:
		{
			GetTacticalSystem()->StopMoving();

			m_bMeleeRequested = false;

			if ( bComeFromOtherState )
			{
				m_fWaitTimer = 0.0f;
			}
			else
			{
				m_fWaitTimer = 0.5f;
			}
			
		}break;
		case POST_ATTACK:
		{
			m_fWaitTimer = 2.0f;
		}break;
		case UNSET:
		default:
		{

		}
	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_MeleePlayer :: RenderDebugInfo
//---------------------------------------------------------------------------
#ifndef ASURA_NO_DEBUG_OUTPUT
static const char* s_aszStateStrings[ AVP_Behaviour_Marine_MeleePlayer::MAX_STATES ] =
{
	"MELEE PLAYER - UNSET",
	"MELEE PLAYER - MOVE_TO_PLAYER",
	"MELEE PLAYER - TURN_TO_FACE",
	"MELEE PLAYER - MELEE_REQUESTED",
};

void AVP_Behaviour_Marine_MeleePlayer::RenderDebugInfo( void )
{
	PARENT::RenderDebugInfo();

	Asura_Font_System::Print
	(
		GetProjectBrain()->GetDebugOutputPos().x,
		GetProjectBrain()->GetDebugOutputPos().y,
		0.5f, 
		AVP_Brain_Actor_Marine::s_uTemporaryChildDebugColour,
		ASURA_TEXTFORMAT_CENTREJUSTIFIED, 
		s_aszStateStrings[GetUpdateState()]
	);

	// Offset the debug output position for other behaviours to render there own text
	GetProjectBrain()->GetDebugOutputPos().y += 10.0f;
}
#endif

/// End Of File 'AVP_behaviour_Marine_MeleePlayer.cpp'
