//////////////////////////////////////////////////////////////////////////////
//
// AVP_Behaviour_Predalien.cpp
//
// Copyright (C) 2007 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"

// header
#include "Behaviour/BehaviourTypes/Predalien/AVP_Behaviour_Predalien.h"

// asura
#include "Entity/PhysicalObject/Moves/Asura_Moves.h"
#include "Physics/Asura_Physics_MovementQuery.h"
#ifndef ASURA_NO_DEBUG_OUTPUT
#include "Rendering/Asura_Font_System.h"
#endif

// axon
#include "Behaviour/BehaviourTypes/AVP_Behaviour_Types.h"
#include "Behaviour/BehaviourTypes/Predalien/AVP_Behaviour_Predalien_Movement.h"
#include "Core/BrainTypes/AVP_Brain_Actor_Predalien.h"
#include "Core/AVP_Navigation_Types.h"

// avp
#include "Animation/AVP_AnimFlags.h"
#include "Actor/Vertebrates/AVP_Vertebrate_AIAnimStateMachine.h"
#include "Actor/Vertebrates/Predalien/AVP_Predalien_MeleeCounter_System.h"
#include "Core/AVP_Game_Core.h"
#include "Entity/AVP_Messages.h"
#include "Entity/Locations/VantagePoint/AVP_ServerNode_VantagePoint_Predator.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/AVP_Vertebrate_Messages.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Predalien/AVP_ServerActor_Predalien.h"
#include "Moves/Vertebrates/Predalien/AVP_Predalien_MovesMelee.h"
#include "Network/AVP_Player.h"


//////////////////////////////////////////////////////////////////////////////
// AVP_PredalienChargeFunctor
//////////////////////////////////////////////////////////////////////////////

class AVP_PredalienChargeFunctor : public AVP_Navigation::AStarFunctorBase
{
public:
	AVP_PredalienChargeFunctor(const Asura_Vector_3& xDestination)
		: m_xDestination(xDestination)
	{}

	virtual float EstimateTraversalCost(const int iWaypointIndex, const int iLinkIndex, const u_int uPathfinderFlags);
	virtual bool LinkFlagsTest( const Asura_AI_NavConnection* pxLink, const u_int uPathfinderFlags );

private:

	const Asura_Vector_3 m_xDestination;
};

float AVP_PredalienChargeFunctor::EstimateTraversalCost(const int iWaypointIndex, const int iLinkIndex, const u_int uPathfinderFlags)
{
	//float fCost = Asura_AI_Navigation_System::DefaultTraversalCostFunction( iWaypointIndex, iLinkIndex, uPathfinderFlags, NULL );

	// calculate the horizontal distance - ignore vertical cost

	const Asura_AI_NavVertex* pxFromWP = Asura_AI_Navigation_System::GetWaypoint( iWaypointIndex );
	const Asura_AI_NavConnection& xLink = pxFromWP->GetLink(iLinkIndex);
	const Asura_AI_NavVertex* pxToWP = Asura_AI_Navigation_System::GetWaypoint( xLink.m_usIndex );

	Asura_Vector_3 xFromToHor = pxToWP->GetPosition() - pxFromWP->GetPosition();
	xFromToHor.y = 0.0f;
	float fCost = xFromToHor.Magnitude();

	float fCostDueTojumping = 0.0f;

	if ( (xLink.GetFlags() & AVP_Navigation::AVP_AI_CONNECTION_FLAG_TRANSITION_HIGH_MOBILITY) )
	{
		fCostDueTojumping = - 0.5f * fCost;
	}

	// prefer to be in the same hight as the target - this will allow more chance to do pounce attacks
	float fCostDueToHightDiff = 0.0f;
	if (Asura_Maths::Abs(m_xDestination.y - pxToWP->GetPosition().y) > 1.5f)
	{
		fCostDueToHightDiff = .35f * fCost;
	}
	return fCost + fCostDueTojumping + fCostDueToHightDiff;
}

bool AVP_PredalienChargeFunctor::LinkFlagsTest( const Asura_AI_NavConnection* pxLink, const u_int uPathfinderFlags )
{
	return AVP_Navigation::PredatorLinkFlagsTest(pxLink, uPathfinderFlags);
}


//////////////////////////////////////////////////////////////////////////////
// AVP_PredalienSneakFunctor
//////////////////////////////////////////////////////////////////////////////

DEBUGVAR float dbg_fMinDistanceSquared = 6.0f;

class AVP_PredalienSneakFunctor : public AVP_Navigation::AStarFunctorBase
{
public:
	AVP_PredalienSneakFunctor(const Asura_Vector_3& xAvoidCentre,  const Asura_Vector_3& xDestination)
		: m_xAvoidCentre(xAvoidCentre)
		, m_xDestination(xDestination)
	{}

	virtual float EstimateTraversalCost(const int iWaypointIndex, const int iLinkIndex, const u_int uPathfinderFlags);
	virtual bool LinkFlagsTest( const Asura_AI_NavConnection* pxLink, const u_int uPathfinderFlags );

private:

	const Asura_Vector_3 m_xAvoidCentre;
	const Asura_Vector_3 m_xDestination;
};

float AVP_PredalienSneakFunctor::EstimateTraversalCost(const int iWaypointIndex, const int iLinkIndex, const u_int uPathfinderFlags)
{
	//float fCost = Asura_AI_Navigation_System::DefaultTraversalCostFunction( iWaypointIndex, iLinkIndex, uPathfinderFlags, NULL );

	// calculate the horizontal distance - ignore vertical cost

	const Asura_AI_NavVertex* pxFromWP = Asura_AI_Navigation_System::GetWaypoint( iWaypointIndex );
	const Asura_AI_NavConnection& xLink = pxFromWP->GetLink(iLinkIndex);
	const Asura_AI_NavVertex* pxToWP = Asura_AI_Navigation_System::GetWaypoint( xLink.m_usIndex );

	Asura_Vector_3 xFromToHor = pxToWP->GetPosition() - pxFromWP->GetPosition();
	xFromToHor.y = 0.0f;
	float fCost = xFromToHor.Magnitude();

	float fCostDueTojumping = 0.0f;

	if ( (xLink.GetFlags() & AVP_Navigation::AVP_AI_CONNECTION_FLAG_TRANSITION_HIGH_MOBILITY) )
	{
		fCostDueTojumping = - 0.5f * fCost;
	}

	// prefer to be in the above as the avoid point
	float fCostDueToHightDiff = 0.0f;

	if ((pxToWP->GetPosition().y - m_xAvoidCentre.y) > 1.0f)
	{
		fCostDueToHightDiff = -0.25f * fCost;
	}

	float fCostDueToDistanceFromAvoidCentre = 0.0f;
	Asura_Vector_3 xNearestPoint;
	Asura_Maths::GetNearestPointOnLineSegment(pxFromWP->GetPosition(), pxToWP->GetPosition(), m_xAvoidCentre, xNearestPoint);
	float fDistanceSquared = (m_xAvoidCentre - xNearestPoint).MagnitudeSquared();
	if (fDistanceSquared < 1.0f * 1.0f)
	{
		fCostDueToDistanceFromAvoidCentre = fCost * 5.0f;
	}
	else if(fDistanceSquared < dbg_fMinDistanceSquared)
	{
		fCostDueToDistanceFromAvoidCentre = fCost * (5.0f * (dbg_fMinDistanceSquared - fDistanceSquared)/ dbg_fMinDistanceSquared);
	}
	return fCost + fCostDueTojumping + fCostDueToHightDiff + fCostDueToDistanceFromAvoidCentre;
}

bool AVP_PredalienSneakFunctor::LinkFlagsTest( const Asura_AI_NavConnection* pxLink, const u_int uPathfinderFlags )
{
	return AVP_Navigation::PredatorLinkFlagsTest(pxLink, uPathfinderFlags);
}

//////////////////////////////////////////////////////////////////////////////
//
// AVP_Behaviour_Predalien
//
//////////////////////////////////////////////////////////////////////////////

DEBUGVAR float dbg_fDistanceToWalkMelee = 7.0f;
DEBUGVAR float dbg_fDistanceToStandMelee = 2.5f;
DEBUGVAR float dbg_fTimeToWalkMelee = 0.5f;
DEBUGVAR float dbg_fMaxDistanceToTrackTarget = 10.0f;
DEBUGVAR float dbg_fMaxDistanceToLeapAttack = 13.0f;
DEBUGVAR float dbg_fMinDistanceToLeapAttack = 8.0f;
DEBUGVAR float dbg_fLongTimeSinceDamage = 8.0f;
DEBUGVAR float dbg_fLongTimeSinceStalking = 30.0f;
DEBUGVAR float dbg_fLongTimeInVentagePoint = 10.0f;

AVP_Behaviour_Predalien::AVP_Behaviour_Predalien( AVP_Brain_Actor_Predalien* pxBrain )
: PARENT( pxBrain, AVP_BEHAVIOUR_TYPE_PREDALIEN )
, m_eCurrentState( UNSET )
, m_fTimeInState( 0.0f )
, m_fTimeToIgnoreTarget(0.0f)
, m_xDestinationPosition(Asura_Maths::ZeroVector3)
, m_xDelayedMsgList()
, m_uVantagePointGuid(ASURA_GUID_UNREGISTERED)
, m_fTargetHealth(0.0f)
, m_fTimeSinceHealthChanged(0.0f)
, m_fTimeSinceStalking(0.0f)
, m_bWasKnockedDown(false)
{
}

void AVP_Behaviour_Predalien::KnockDown()
{
	m_bWasKnockedDown = true;
}

void AVP_Behaviour_Predalien::Deactivate()
{
	Axon_Behaviour* pxChild = GetChild();
	while ( pxChild )
	{
		UnlinkChild( pxChild );
		GetProjectBrain()->DestroyBehaviour( pxChild );
		pxChild = GetChild();
	}

    PARENT::Deactivate();
}

void AVP_Behaviour_Predalien::PerFrameUpdate( const float fTimeslice )
{
    PARENT::PerFrameUpdate( fTimeslice );

	
    if ( IsFinished() || IsInError() )
    {
        return;
    }

    m_fTimeInState += fTimeslice;
	m_fTimeSinceStalking += fTimeslice;
	m_fTimeToIgnoreTarget = m_fTimeToIgnoreTarget > fTimeslice ? m_fTimeToIgnoreTarget -= fTimeslice : 0.0f;

	if (GetTarget())
	{
		float fCurrentHealth = GetTarget()->GetHealth();
		if (fCurrentHealth < m_fTargetHealth)
		{
			m_fTimeSinceHealthChanged = 0.0f;
		}
		else
		{
			m_fTimeSinceHealthChanged += fTimeslice;
		}

		m_fTargetHealth = fCurrentHealth;
	}

	if (m_bWasKnockedDown && m_eCurrentState != KNOCKED_DOWN) // for now no limit on when that can happen
	{
		SwitchToState(KNOCKED_DOWN);		
	}


	// wait for the fall or jump to finish...
	if (!GetPredalienBrain()->CanPlanMove())
	{
		return;
	}

	LateHandleMessage();

	switch ( m_eCurrentState )
	{
	case UNSET:
		if ( GetPredalienBrain()->GetPredalienActor()->GetUseTestBehaviour())
		{
			SwitchToState( TEST_BEHAVIOUR );
		}
		else
		{
			SwitchToState(IDLE);
		}
		break;
	case IDLE:
		if (GetTarget())
		{
			SwitchToState(RUNNING_IN);
		}
		break;
	case RUNNING_IN:
		UpdateMoveToMelee(fTimeslice);
		break;
	case RUNNING_TO_VANTAGE_POINT:
		UpdateMoveToVantagePoint(fTimeslice);
		break;
	case AT_VANTAGE_POINT:
		UpdateAtVantagePoint(fTimeslice);
	case FLINCHED:
	case STAND_CLOSE_MELEE:
	case WALK_CLOSE_MELEE:
	case TAIL_CLOSE_MELEE:
	case LEAP_ATTACK:
	case JUMP_DOWN_ATTACK:
	case KNOCKED_DOWN:
	case BEING_GRAPPLED:
		break;
	case TEST_BEHAVIOUR:
		break;
	default:
		Asura_Assert(false, "Unexpected state");
	}
}

bool AVP_Behaviour_Predalien::HandleMessage( const Asura_Message* pxMessage )
{
	switch (pxMessage->GetMsgID())
	{
	case ASURA_MSG_MOVECHAINCOMPLETE:
		if (m_eCurrentState != RUNNING_IN && m_eCurrentState != RUNNING_TO_VANTAGE_POINT && m_eCurrentState != AT_VANTAGE_POINT)
		{
			if (GetPredalienBrain()->IsMelee())
			{
				GetPredalienBrain()->SetMelee(false);
			}
			m_xDelayedMsgList.Push_Back(ASURA_MSG_MOVECHAINCOMPLETE);
			return true;
		}
		break;
	case ASURA_MSG_MOVE_ABORT:
		if (m_eCurrentState != RUNNING_IN)
		{
			if (GetPredalienBrain()->IsMelee())
			{
				GetPredalienBrain()->SetMelee(false);
			}
			m_xDelayedMsgList.Push_Back(ASURA_MSG_MOVE_ABORT);
			return true;
		}
		break;
	case AVP_MSGID_GRAPPLE_START: // switching to being grappled from knocked down is allowed 
		{
			if (m_eCurrentState == KNOCKED_DOWN)
			{
				SwitchToState(BEING_GRAPPLED);
			}
			return true;
		}
	case AVP_MSGID_FLINCH: // check if a flinch started
		{
			const AVP_Msg_Flinch* pxData = static_cast<const AVP_Msg_Flinch*>(pxMessage->GetData());
			// Only bother to pay attention to the flinch if it is going to start a move.
			if (pxData && pxData->GetUseFlinchMove())
			{
				m_xDelayedMsgList.Push_Back(AVP_MSGID_FLINCH);
			}
		}
		return true;
	default:
		break;
	}

	return PARENT::HandleMessage( pxMessage );
}


#ifndef ASURA_NO_DEBUG_OUTPUT
static const char* s_aszStateStrings[] =
{
    "UNSET",
	"IDLE",
	"RUNNIG_IN",
	"STAND_CLOSE_MELEE",
	"WALK_CLOSE_MELEE",
	"TAIL_CLOSE_MELEE",
	"LEAP_ATTACK",
	"FLINCHED",
	"RUNNING_TO_VANTAGE_POINT",
	"AT_VANTAGE_POINT",
	"JUMP_DOWN_ATTACK",
	"KNOCKED_DOWN",
	"BEING_GRAPPLED",
	"TEST_BEHAVIOUR",
	"NUM_STATES"
};

void AVP_Behaviour_Predalien::RenderDebugInfo()
{
    PARENT::RenderDebugInfo();

    Asura_Font_System::Print( GetProjectBrain()->GetDebugOutputPos().x, GetProjectBrain()->GetDebugOutputPos().y, 0.5f, 0xffffffff,
        ASURA_TEXTFORMAT_CENTREJUSTIFIED | ASURA_TEXTFORMAT_FORMATSTRING, "%s", s_aszStateStrings[ m_eCurrentState ] );
    GetProjectBrain()->GetDebugOutputPos().y += 10.0f;
}
#endif

AVP_Brain_Actor_Predalien* AVP_Behaviour_Predalien::GetPredalienBrain() const
{
    return static_cast< AVP_Brain_Actor_Predalien* >( GetProjectBrain() );
}

void AVP_Behaviour_Predalien::SwitchToState( const State eNextState )
{
	
	switch ( m_eCurrentState )
	{
	case UNSET:
		break;
	case IDLE:
		break;
	case RUNNING_IN:
	case RUNNING_TO_VANTAGE_POINT:
		DestroyChildWithType( AVP_BEHAVIOUR_TYPE_PREDALIEN_MOVEMENT );
		break;
	case AT_VANTAGE_POINT:
		m_fTimeSinceStalking = 0.0f;
		break;
	case STAND_CLOSE_MELEE:
	case WALK_CLOSE_MELEE:
	case TAIL_CLOSE_MELEE:
	case LEAP_ATTACK:
	case FLINCHED:
	case JUMP_DOWN_ATTACK:
		break;
	case TEST_BEHAVIOUR:
		DestroyChildWithType( AVP_BEHAVIOUR_TYPE_PREDALIEN_TEST );
		break;
	case KNOCKED_DOWN: 
		m_bWasKnockedDown = false;		
		break;
	case BEING_GRAPPLED:
		break;
	default:
		Asura_Assert(false, "Unexpected state");
	}

	m_eCurrentState = eNextState;
	m_fTimeInState  = 0.0f;
	switch ( m_eCurrentState )
	{
	case UNSET:
	case IDLE:	
		break;
	case RUNNING_IN:
		PlanMeleePath();
		break;
	case RUNNING_TO_VANTAGE_POINT:
		PlanVantagePath();
		break;
	case STAND_CLOSE_MELEE:
		RequestMelee( AVP_PREDPREDALIEN_MELEEATTACK_TYPE_CLOSE_STAND_ATTACK, true );
		break;
	case WALK_CLOSE_MELEE:
		RequestMelee( AVP_PREDPREDALIEN_MELEEATTACK_TYPE_CLOSE_WALK_ATTACK, false );
		break;
	case TAIL_CLOSE_MELEE:
		RequestMelee( AVP_PREDPREDALIEN_MELEEATTACK_TYPE_CLOSE_TAIL_ATTACK, false );
		break;
	case LEAP_ATTACK:
		RequestMelee( AVP_PREDPREDALIEN_MELEEATTACK_TYPE_LEAP_ATTACK, true );
		break;
	case FLINCHED:
		break;
	case JUMP_DOWN_ATTACK:
		RequestMelee( AVP_PREDPREDALIEN_MELEEATTACK_TYPE_JUMP_DOWN_ATTACK, true );
		break;
	case AT_VANTAGE_POINT:
		{ // make sure we are idle
			AVP_Vertebrate_AIAnimStateMachine* pxVertStateMachine = static_cast< AVP_Vertebrate_AIAnimStateMachine* >( GetPredalienBrain()->GetActor()->GetAIAnimStateMachine() );
			pxVertStateMachine->RequestIdle();
		}
		break;
	case KNOCKED_DOWN:
		if (GetPredalienBrain()->IsJumping()) // fix the jump / melee flags that
		{
			GetPredalienBrain()->SetJumping(false);
		}
		if (!GetPredalienBrain()->IsMelee())
		{
			GetPredalienBrain()->SetMelee(false);
		}
		RequestKnockedDownMove();
		break;
	case BEING_GRAPPLED:
		GetPredalienBrain()->GetPredalienActor()->SetGrapple();
		break;
	case TEST_BEHAVIOUR:
		DestroyAllChildren();
		CreateChildWithType( AVP_BEHAVIOUR_TYPE_PREDALIEN_TEST );
		break;
	default:
		Asura_Assert(false, "Unexpected state");
	}
}

const AVP_ServerActor_Vertebrate* AVP_Behaviour_Predalien::GetTarget() const // predalien only deals with the player
{
	return AVP_ServerActor_Vertebrate::FindEntity(GetTargetGuid());
}

Asura_Guid AVP_Behaviour_Predalien::GetTargetGuid() const // predalien only deals with the player
{
	if (m_fTimeToIgnoreTarget > 0.0f)
	{
		return ASURA_GUID_UNREGISTERED;
	}

	const AVP_Player* pxPlayer = AVP_Game_Core::GetPlayer();
	if (pxPlayer && pxPlayer->IsEntityActive() && !pxPlayer->IsDestroyed())
	{
		return pxPlayer->GetPuppetGuid();
	}

	return  ASURA_GUID_UNREGISTERED;
}

void AVP_Behaviour_Predalien::UpdateMoveToMelee(float fTimeslice)
{

	const AVP_ServerEntity_Actor* pxTarget = GetTarget();
	if (!pxTarget)
	{
		SwitchToState(IDLE);
		return;
	}

	Asura_Vector_3 xOurPosition = GetPredalienBrain()->GetActor()->GetPosition();
	if ( (m_fTimeSinceStalking > dbg_fLongTimeSinceStalking || 
		  m_fTimeSinceHealthChanged > dbg_fLongTimeSinceDamage) && SelectVantagePoint(xOurPosition, pxTarget->GetPosition()))
	{
		SwitchToState(RUNNING_TO_VANTAGE_POINT);
		return;
	}

	AVP_Behaviour_Predalien_Movement* pxBehaviour = static_cast< AVP_Behaviour_Predalien_Movement* >( GetChildWithType( this, AVP_BEHAVIOUR_TYPE_PREDALIEN_MOVEMENT ) );
	if ( !pxBehaviour || pxBehaviour->IsInError() )
	{
		PlanMeleePath();
		return;
	}


	Asura_Vector_3 xToTargetPosition = pxTarget->GetPosition() - xOurPosition;
	float fDistanceToTarget = xToTargetPosition.Normalise();

	bool bCanMeleeFromFront, bCanMeleeFromAnyDirection;
	bool bUseTail;
	CanMelee(pxTarget, bCanMeleeFromFront, bCanMeleeFromAnyDirection, bUseTail);
	
	if (bCanMeleeFromFront && CanDoLeapAttack(fDistanceToTarget, xToTargetPosition))
	{
		SwitchToState( LEAP_ATTACK );
		return;
	}
	else if (bCanMeleeFromAnyDirection && CanDoCloseStandMelee(fDistanceToTarget) )
	{
		if (bUseTail)
		{
			SwitchToState( TAIL_CLOSE_MELEE );
		}
		else
		{
			SwitchToState( STAND_CLOSE_MELEE );
		}
		return;
	}
	else if ( bCanMeleeFromFront && CanDoCloseWalkMelee(fDistanceToTarget, pxBehaviour) )
	{
		SwitchToState( WALK_CLOSE_MELEE );
		return;
	}
	else if( pxBehaviour->IsFinished() ||
			 GetPredalienBrain()->ShouldRefreshPath(pxBehaviour->GetLengthLeftOnPath(), (pxTarget->GetPosition() - m_xDestinationPosition).MagnitudeSquared()) )
	{
		PlanMeleePath();
	}
	else 
	{
		pxBehaviour->SetMovementMode(AVP_Behaviour_Predalien_Movement::EMM_MEDIUM);
		if( fDistanceToTarget < dbg_fMaxDistanceToTrackTarget)
		{
			AimAtTarget(true);
		}
		else
		{
			pxBehaviour->AimWhereYouMove();
		}
	}
}

void AVP_Behaviour_Predalien::UpdateMoveToVantagePoint(float fTimeslice)
{
	const AVP_ServerEntity_Actor* pxTarget = GetTarget();
	if (!pxTarget)
	{
		SwitchToState(IDLE);
		return;
	}

	AVP_Behaviour_Predalien_Movement* pxBehaviour = static_cast< AVP_Behaviour_Predalien_Movement* >( GetChildWithType( this, AVP_BEHAVIOUR_TYPE_PREDALIEN_MOVEMENT ) );
	if ( !pxBehaviour || pxBehaviour->IsInError() )
	{
		PlanVantagePath();
		return;
	}
	else if ( pxBehaviour->IsFinished())
	{
		SwitchToState(AT_VANTAGE_POINT);
		return;
	}
	else
	{
		pxBehaviour->SetMovementMode(AVP_Behaviour_Predalien_Movement::EMM_MEDIUM);
		pxBehaviour->AimWhereYouMove();
	}

	// check if the vantage point still valid.
	if (!IsVantagePointValid(m_xDestinationPosition, pxTarget->GetPosition(), true) )
	{
		SwitchToState(RUNNING_IN);
	}
}

void AVP_Behaviour_Predalien::UpdateAtVantagePoint(float fTimeslice)
{
	const AVP_ServerEntity_Actor* pxTarget = GetTarget();
	if (!pxTarget)
	{
		SwitchToState(IDLE);
		return;
	}

	Asura_Vector_3 xToTarget = pxTarget->GetPosition() - GetPredalienBrain()->GetActor()->GetPosition();
	xToTarget.y = 0.0f;
	if (!xToTarget.ApproximatelyEquals(Asura_Maths::ZeroVector3))
	{
		xToTarget.Normalise();

		GetPredalienBrain()->SetLookAtPosition(xToTarget);
		GetPredalienBrain()->SetAimDirection(xToTarget);
	}

	// check if the vantage point still valid.
	if (!IsVantagePointValid(m_xDestinationPosition, pxTarget->GetPosition(), true))
	{
		SwitchToState(RUNNING_IN);
		return;
	}
	else if (m_fTimeInState > dbg_fLongTimeInVentagePoint)
	{
		SwitchToState(RUNNING_IN);
		return;
	}
	else if (CanDoJumpDownAttack(pxTarget->GetPosition()))
	{
		SwitchToState(JUMP_DOWN_ATTACK);
		return;
	}
}

void AVP_Behaviour_Predalien::PlanMeleePath()
{

	DestroyChildWithType( AVP_BEHAVIOUR_TYPE_PREDALIEN_MOVEMENT );

	const AVP_ServerEntity_Actor* pxTarget = GetTarget();
	if (!pxTarget)
	{
		SwitchToState(IDLE);
		return;
	}

	m_xDestinationPosition = pxTarget->GetPosition();	

	AVP_PredalienChargeFunctor xFunctor(m_xDestinationPosition);
	if (!PlanPathToDestination(&xFunctor))
	{
		IgnoreTarget();
		SwitchToState(IDLE);
		return;
	}

	GetPredalienBrain()->SetNearbyActorGuidToIgnore(pxTarget->GetGuid());

	AVP_Behaviour_Predalien_Movement* pxMovementBehaviour =
		static_cast< AVP_Behaviour_Predalien_Movement* >( CreateChildWithType( AVP_BEHAVIOUR_TYPE_PREDALIEN_MOVEMENT ) );
	if ( !pxMovementBehaviour )
	{
		IgnoreTarget();
		SwitchToState(IDLE);
	}
}


void AVP_Behaviour_Predalien::PlanVantagePath()
{

	DestroyChildWithType( AVP_BEHAVIOUR_TYPE_PREDALIEN_MOVEMENT );

	const AVP_ServerNode_VantagePoint_Predator* pxVP = AVP_ServerNode_VantagePoint_Predator::FindEntity(m_uVantagePointGuid);
	if (!pxVP)
	{
		SwitchToState(IDLE);
		return;
	}

	const AVP_ServerEntity_Actor* pxTarget = GetTarget();
	if (!pxTarget)
	{
		SwitchToState(IDLE);
		return;
	}

	m_xDestinationPosition = pxVP->GetPosition();	

	AVP_PredalienSneakFunctor xFunctor(pxTarget->GetPosition(), m_xDestinationPosition);
	if (!PlanPathToDestination(&xFunctor))
	{
		IgnoreVentagePoint(m_uVantagePointGuid);
		SwitchToState(RUNNING_IN);
		return;
	}


	AVP_Behaviour_Predalien_Movement* pxMovementBehaviour =
		static_cast< AVP_Behaviour_Predalien_Movement* >( CreateChildWithType( AVP_BEHAVIOUR_TYPE_PREDALIEN_MOVEMENT ) );
	if ( !pxMovementBehaviour )
	{
		IgnoreVentagePoint(m_uVantagePointGuid);
		SwitchToState(RUNNING_IN);
	}
}



bool AVP_Behaviour_Predalien::PlanPathToDestination(AVP_Navigation::AStarFunctorBase* xFunctor)
{
	AVP_Navigation::NavData& xNavData = GetPredalienBrain()->GetNavData();

	xNavData.m_xStartPosition = GetPredalienBrain()->GetActor()->GetPosition();
	xNavData.m_eMovementMode = AVP_Navigation::MM_FAST;
	xNavData.m_xDestinationPosition = m_xDestinationPosition;
	const AVP_Navigation::TestParams xTargetTestParams(xNavData.m_uPathfinderFlags, ASURA_NAVVERTEX_MAX_ZONE_ID);

	if ( !AVP_Navigation::GetNearestWaypointID( xNavData.m_xDestinationPosition, xNavData.m_usDestinationWPID, xTargetTestParams ) )
	{
		return false;
	}

	// Clear out the old route and find the new one.
	xNavData.ResetRoute();
	if ( !AVP_Navigation::FindRoute( xNavData, xFunctor, true ) )
	{
		return false;
	}

	return true;
}



DEBUGVAR float dbg_fIgnoreTime = 1.0f;

void AVP_Behaviour_Predalien::IgnoreTarget()
{
	m_fTimeToIgnoreTarget = dbg_fIgnoreTime;
}

DEBUGVAR float dbg_fMaxAboveToMelee = -2.5f;
DEBUGVAR float dbg_fMaxBelowToMelee = 0.5f;
DEBUGVAR float dbg_fBigHeadingDifferenceCos = Asura_Maths::Cosine(Asura_Maths::DegToRad(70.0f));
DEBUGVAR float dbg_fMediumHeadingDifferenceCos = Asura_Maths::Cosine(Asura_Maths::DegToRad(35.0f));
DEBUGVAR float dbg_fClawAttackMaxAngleCos = Asura_Maths::Cosine(Asura_Maths::DegToRad(120.0f));

void AVP_Behaviour_Predalien::CanMelee(const AVP_ServerEntity_Actor* pxTarget, bool& bFromFront, bool& bFromAnyDirection, bool& bUseTail)  const
{

	bUseTail = true;
	bFromFront = bFromAnyDirection = false;

	const AVP_ServerEntity_Actor* pxActor = GetPredalienBrain()->GetActor();

	Asura_Vector_3 xTargetUp(-pxTarget->GetCurrentGravityDirection());
	Asura_Vector_3 xUp(0.0f, -1.0f, 0.0f);

	Asura_Vector_3 xToTarget = pxTarget->GetPosition() - pxActor->GetPosition();
	float fDeltaY = xToTarget.y;
	xToTarget.y = 0.0f;
	xToTarget.Normalise();

	const Asura_Matrix_3x3& xOrientation = pxActor->GetOrientation();
	const Asura_Vector_3 xFront(xOrientation.e13, xOrientation.e23, xOrientation.e33);

	if (fDeltaY < dbg_fMaxAboveToMelee || fDeltaY > dbg_fMaxBelowToMelee)
	{
		return;
	}

	bFromAnyDirection = true;

	float fDot = xToTarget * xFront;

	if ( fDot >=  dbg_fBigHeadingDifferenceCos )
	{
		bFromFront = true;
	}

	if ( fDot >= dbg_fClawAttackMaxAngleCos)
	{
		bUseTail = false;
	}
}

void AVP_Behaviour_Predalien::RequestMelee(Asura_Hash_ID uMeleeType, bool bStopBeforeMelee)
{
	AVP_Brain_Actor_Predalien* pxBrain =  GetPredalienBrain();
	const AVP_ServerEntity_Actor* pxTarget = GetTarget();
	if (!pxTarget)
	{
		SwitchToState(RUNNING_IN);
		return;
	}

	Asura_Message* pxMessage = Asura_Network::CreateMessage( AVP_MSGID_REQUEST_MELEE, sizeof( AVP_Message_Vertebrate_RequestMelee ) );
	if ( !pxMessage )
	{
		SwitchToState(RUNNING_IN);
		return;
	}

	AVP_Message_Vertebrate_RequestMelee* pxData = static_cast< AVP_Message_Vertebrate_RequestMelee* >( pxMessage->GetData() );

	pxData->SetType( uMeleeType );
	pxData->SetTargetGuid( pxTarget->GetGuid() );

	pxMessage->SetToGuid( pxBrain->GetOwnerGuid() );
	pxMessage->SetFromGuid( pxBrain->GetOwnerGuid() );

	if (bStopBeforeMelee)
	{
		AVP_Vertebrate_AIAnimStateMachine* pxVertStateMachine = static_cast< AVP_Vertebrate_AIAnimStateMachine* >( GetPredalienBrain()->GetActor()->GetAIAnimStateMachine() );
		pxVertStateMachine->RequestIdle();
	}

	Asura_Network::Send( pxMessage );
	GetPredalienBrain()->SetMelee(true);
}

void AVP_Behaviour_Predalien::RequestKnockedDownMove()
{
	Asura_Message* pxMessage = Asura_Network::CreateMessage( ASURA_MSG_MOVE_START, sizeof( Asura_Msg_Move ) );

	Asura_Assert(pxMessage, "Unable to create message");
	if ( pxMessage != NULL )
	{

		Asura_Msg_Move* pxData = static_cast< Asura_Msg_Move* >( pxMessage->GetData() );
		// Fill-in the message-body...
		pxMessage->SetFromGuid(GetPredalienBrain()->GetActor()->GetGuid());
		pxMessage->SetToGuid(GetPredalienBrain()->GetActor()->GetGuid());

		pxData->ClearFlags();
		pxData->SetMoveID( AVP_Predalien_Move_KnockedDown_Start::GetStaticHash() );
		//pxData->SetForceAbort( true );
		pxData->SetIgnoredByOwner( false );
		AVP_Vertebrate_AIAnimStateMachine* pxVertStateMachine = static_cast< AVP_Vertebrate_AIAnimStateMachine* >( GetPredalienBrain()->GetActor()->GetAIAnimStateMachine() );
		pxVertStateMachine->RequestInMove(pxMessage, AVP_Actor_AIAnimStateMachine::ACTOR_STATE_IDLE);
	}
}


void AVP_Behaviour_Predalien::AimAtTarget(bool m_bUpdateMoveToo)
{
	const AVP_ServerEntity_Actor* pxTarget = GetTarget();
	if (!pxTarget)
	{
		return;
	}

	Asura_Vector_3 xTargetHead;
	pxTarget->GetHeadPosition(xTargetHead);
	Asura_Vector_3 xOurHeadPosition;
	GetPredalienBrain()->GetActor()->GetHeadPosition(xOurHeadPosition);
	Asura_Vector_3 xToTargetHead = xTargetHead - xOurHeadPosition;
	xToTargetHead.Normalise();
	GetPredalienBrain()->SetAimDirection( xToTargetHead );

	AVP_Behaviour_Predalien_Movement* pxBehaviour = static_cast< AVP_Behaviour_Predalien_Movement* >( GetChildWithType( this, AVP_BEHAVIOUR_TYPE_PREDALIEN_MOVEMENT ) );

	pxBehaviour->AimAt(xToTargetHead);
}


void AVP_Behaviour_Predalien::LateHandleMessage()
{
	Asura_LinkList_It< int > xIter(m_xDelayedMsgList);
	for (; !xIter.Done(); xIter.Next())
	{
		int eMsgId = *xIter.GetCurrent();
		switch (eMsgId)
		{
		case ASURA_MSG_MOVECHAINCOMPLETE:
			switch (m_eCurrentState)
			{
			case STAND_CLOSE_MELEE:				
			case WALK_CLOSE_MELEE:
			case TAIL_CLOSE_MELEE:
			case LEAP_ATTACK:
			case JUMP_DOWN_ATTACK:
				SwitchToState( RUNNING_IN );
				break;
			case KNOCKED_DOWN:
			case BEING_GRAPPLED:
				SwitchToState( RUNNING_TO_VANTAGE_POINT ); // Placeholder - probalby will want to make him run away for a few seconds ?
				break;
			case FLINCHED:
				SwitchToState( RUNNING_IN );
				break;
			case IDLE:
				break;
			default:
				Asura_Assert(false, "Unexpected state");
				break;
			}
			break;
		case ASURA_MSG_MOVE_ABORT:
			if (m_eCurrentState != RUNNING_IN)
			{
				SwitchToState( RUNNING_IN );
			}
			else
			{
				Asura_Assert(false, "Unexpected state");
			}
			break;
		case AVP_MSGID_FLINCH: // check if a flinch started
			SwitchToState(FLINCHED);
			break;		
		case IDLE:
			break;
		default:
			Asura_Assert(false, "Unexpected state");
			break;
		}
	}
	m_xDelayedMsgList.Clear();
}

bool AVP_Behaviour_Predalien::CanDoCloseStandMelee(float fDistanceToTarget) const
{
	return fDistanceToTarget < dbg_fDistanceToStandMelee;
}

bool AVP_Behaviour_Predalien::CanDoCloseWalkMelee(float fDistanceToTarget, const AVP_Behaviour_Predalien_Movement *pxBehaviour) const
{
	return fDistanceToTarget < dbg_fDistanceToWalkMelee && m_fTimeInState > dbg_fTimeToWalkMelee && pxBehaviour->NoJumpsTillEndOfPath();
}

DEBUGVAR float dbg_fDisplacementEpsilon = 0.1f;

bool AVP_Behaviour_Predalien::CanDoLeapAttack(float fDistanceToTarget, const Asura_Vector_3& xToTargetDirection) const
{
	if (fDistanceToTarget < dbg_fMinDistanceToLeapAttack || fDistanceToTarget > dbg_fMaxDistanceToLeapAttack)
	{
		return false;
	}

	AVP_ServerEntity_Actor* pxActor = GetPredalienBrain()->GetActor();
	AVP_Actor_PhysicsObjectHandler* pxPhyHandler = pxActor ? pxActor->GetPhysicsObjectHandler() : NULL;  
	const Asura_Physics_Object* pxPhyObject =  pxPhyHandler ? pxPhyHandler->GetPhysicsObject() : NULL;
	const Asura_Collision_Object* pxCollObj = pxPhyObject ? pxPhyObject->GetCollisionObject() : NULL;
	Asura_Assert(pxCollObj, "Collision not found");
	if (!pxCollObj)
	{
		return false;
	}

	const Asura_Matrix_3x3& xOurOrientation = pxActor->GetOrientation();
	const Asura_AVector3 xStartPosition = Asura_AVector3(pxActor->GetPosition());
	Asura_AVector3 xDisplacement = Asura_AVector3(xToTargetDirection) * fDistanceToTarget;

	// Proceed with the movement query
	Asura_OBB xOurOBB(pxCollObj->GetBoundingBox());
	xOurOBB.SetOrientation( Asura_AMatrix3x3(xOurOrientation) );

	Asura_Physics_MovementQuery xMovementQuery( xStartPosition, xDisplacement, 
												xOurOBB, 
												Asura_Physics_MovementQuery::ASURA_PMQCP_SPHERESTACK,
												AsuraPolygonFlag_NoCollisions,
												false,// don't stop at first collision 
												true // collide with static object
											); 

	const Asura_Physics_MovementQuery_Results xQueryResults( xMovementQuery ); 

	Asura_AVector3 xStartToEndPosVec = xQueryResults.GetEndPosition() - xStartPosition;

	if (xStartToEndPosVec.MagnitudeSquared() < fDistanceToTarget * fDistanceToTarget - dbg_fDisplacementEpsilon)
	{
		return false;
	}

	return true;
}

DEBUGVAR float dbg_fMaxHorizontalDistanceSq = 10.0f * 10.0f;

bool AVP_Behaviour_Predalien::CanDoJumpDownAttack(const Asura_Vector_3& xTargetPosition) const
{
	AVP_ServerEntity_Actor* pxActor = GetPredalienBrain()->GetActor();

	const AVP_ClientEntity_Actor* pxClientTarget = AVP_ClientEntity_Actor::FindEntity(GetTargetGuid()); // the target is always the player - his handler is on the client side 
	const AVP_Actor_PhysicsObjectHandler* pxPhyHandler = pxClientTarget ? pxClientTarget->GetPhysicsObjectHandler() : NULL;  
	const Asura_Physics_Object* pxPhyObject =  pxPhyHandler ? pxPhyHandler->GetPhysicsObject() : NULL;
	const Asura_Collision_Object* pxCollObj = pxPhyObject ? pxPhyObject->GetCollisionObject() : NULL;
	if (!pxCollObj) { return false; }
	const Asura_Bounding_Box& xBB = pxCollObj->GetBoundingBox();

	Asura_Vector_3 xTargetTopPosition = xTargetPosition;
	xTargetTopPosition.y -= Asura_Maths::Abs(xBB.GetMax().y - xBB.GetMin().y );

	Asura_Vector_3 xHorForward(GetPredalienBrain()->GetAimDirection());
	xHorForward.y = 0.0f;
	xHorForward.Normalise();
	const Asura_Vector_3& xMyPosition = pxActor->GetPosition();
	Asura_Vector_3 xToTargetHor = xTargetTopPosition - xMyPosition;
	Asura_Vector_3 xToTargetVer(Asura_Maths::ZeroVector3);
	xToTargetVer.y = xToTargetHor.y;
	xToTargetHor.y = 0.0f;

	if (xHorForward.ApproximatelyEquals(Asura_Maths::ZeroVector3))
	{
		return false;
	}

	xHorForward.Normalise();
	Asura_Vector_3 xToTargetHorDirection = xToTargetHor;
	xToTargetHorDirection.Normalise();
	if (xHorForward * xToTargetHorDirection < dbg_fMediumHeadingDifferenceCos)
	{
		return false;
	}

	if (xToTargetHor.MagnitudeSquared() > dbg_fMaxHorizontalDistanceSq)
	{
		return false;
	}

	// do movement query to verify the jump is possible.

	pxPhyHandler = pxActor ? pxActor->GetPhysicsObjectHandler() : NULL;  
	pxPhyObject =  pxPhyHandler ? pxPhyHandler->GetPhysicsObject() : NULL;
	pxCollObj = pxPhyObject ? pxPhyObject->GetCollisionObject() : NULL;
	Asura_Assert(pxCollObj, "Collision not found");
	if (!pxCollObj)
	{
		return false;
	}

	// first the horizontal move
	{
		const Asura_Matrix_3x3& xOurOrientation = pxActor->GetOrientation();
		Asura_OBB xOurOBB(pxCollObj->GetBoundingBox());
		xOurOBB.SetOrientation( Asura_AMatrix3x3(xOurOrientation) );

		const Asura_AVector3 xStartPosition(xMyPosition);
		Asura_AVector3 xDisplacement(xToTargetHor);

		Asura_Physics_MovementQuery xMovementQuery( xStartPosition, xDisplacement, 
													xOurOBB, 
													Asura_Physics_MovementQuery::ASURA_PMQCP_SPHERESTACK,
													AsuraPolygonFlag_NoCollisions,
													false, // don't stop at first collision 
													true // collide with static object
			); 

		const Asura_Physics_MovementQuery_Results xQueryResults( xMovementQuery ); 

		Asura_AVector3 xStartToEndPosVec = xQueryResults.GetEndPosition() - xStartPosition;

		if (xStartToEndPosVec.MagnitudeSquared() < xDisplacement.MagnitudeSquared() - dbg_fDisplacementEpsilon)
		{
			return false;
		}
	}

	// next the vertical move
	{

		const Asura_Matrix_3x3& xOurOrientation = pxActor->GetOrientation();
		Asura_OBB xOurOBB(pxCollObj->GetBoundingBox());
		xOurOBB.SetOrientation( Asura_AMatrix3x3(xOurOrientation) );

		const Asura_AVector3 xStartPosition(xMyPosition + xToTargetHor);
		Asura_AVector3 xDisplacement(xToTargetVer);

		Asura_Physics_MovementQuery xMovementQuery( xStartPosition, xDisplacement, 
			xOurOBB, 
			Asura_Physics_MovementQuery::ASURA_PMQCP_SPHERESTACK,
			AsuraPolygonFlag_NoCollisions,
			false, // don't stop at first collision 
			true // collide with static object
			); 

		const Asura_Physics_MovementQuery_Results xQueryResults( xMovementQuery ); 

		Asura_AVector3 xStartToEndPosVec = xQueryResults.GetEndPosition() - xStartPosition;

		if (xStartToEndPosVec.MagnitudeSquared() < xDisplacement.MagnitudeSquared() - dbg_fDisplacementEpsilon)
		{
			return false;
		}
	}

	return true;
}



DEBUGVAR float dbg_fMaxVPDistanceFromMeSq = 30.0f * 30.0f;
DEBUGVAR float dbg_fMinVerticalDistance = 4.0f;
DEBUGVAR float dbg_fMaxVPDistanceFromTargetSq = 10.0f * 10.0f;

bool AVP_Behaviour_Predalien::SelectVantagePoint(const Asura_Vector_3& xOurPosition, const Asura_Vector_3& xTargetPosition)
{
	Asura_Hashed_List_ConstIt< AVP_ServerNode_VantagePoint_Predator > xVPIt(AVP_ServerNode_VantagePoint_Predator::GetList());
	m_uVantagePointGuid = ASURA_GUID_UNREGISTERED;

	float fBestDistanceSq = Asura_Maths::LargeFloat;

	for(;!xVPIt.Done(); xVPIt.Next())
	{
		const AVP_ServerNode_VantagePoint_Predator* pxVP = xVPIt.GetCurrent();
		if (!pxVP->IsEntityActive())
		{
			continue;
		}
		const Asura_Vector_3& xVPPosition = pxVP->GetPosition();

		if (!IsVantagePointValid(xVPPosition, xTargetPosition, false))
		{
			continue;
		}

		float fVPDistanceSq = (xOurPosition - xVPPosition).MagnitudeSquared();

		if (fVPDistanceSq > dbg_fMaxVPDistanceFromMeSq || fVPDistanceSq > fBestDistanceSq)
		{
			continue;
		}
		
		m_uVantagePointGuid = pxVP->GetGuid();
	}

	return m_uVantagePointGuid != ASURA_GUID_UNREGISTERED;
}

bool AVP_Behaviour_Predalien::IsVantagePointValid( const Asura_Vector_3& xVPPosition, const Asura_Vector_3& xTargetPosition, bool bIgnoreHorizontalDistance)
{
	if (xTargetPosition.y - xVPPosition.y < dbg_fMinVerticalDistance )
	{
		return false;
	}


	Asura_Vector_3 xVPToTargetHorizontal = xTargetPosition - xVPPosition;
	xVPToTargetHorizontal.y = 0.0f;
	if (!bIgnoreHorizontalDistance && xVPToTargetHorizontal.MagnitudeSquared() > dbg_fMaxVPDistanceFromTargetSq)
	{
		return false;
	}

	return true;
}


DEBUGVAR float dbg_fIgnoreVPDuration = 5.0f;

void AVP_Behaviour_Predalien::IgnoreVentagePoint(Asura_Guid uGuid)
{
	IgnoreVPData xData;
	xData.m_uGuid = uGuid;
	xData.m_fTimeToStopIgnore = Asura_Timers::GetLocalScaledTime() + dbg_fIgnoreVPDuration;
	m_xIgnoredVentagePoints.Push_Back(xData);
}



bool AVP_Behaviour_Predalien::IsVentagePointIgnored( Asura_Guid uGuid ) const
{
	for (Asura_LinkList_ConstIt<IgnoreVPData> xIt(m_xIgnoredVentagePoints); !xIt.Done(); xIt.Next())
	{
		if (xIt.GetCurrent()->m_uGuid == uGuid )
		{
			return true;
		}
	}

	return false;
}

void AVP_Behaviour_Predalien::UpdateIgnoreList()
{
	float fNow = Asura_Timers::GetLocalScaledTime();

	while (m_xIgnoredVentagePoints.Size() > 0 )
	{
		if (m_xIgnoredVentagePoints.Front()->Data().m_fTimeToStopIgnore > fNow )
		{
			return;
		}

		m_xIgnoredVentagePoints.PopFront();
	}
}


//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////
