//////////////////////////////////////////////////////////////////////////////
//
// AVP_Behaviour_Predalien_GS2Move.cpp
//
// Copyright (C) 2007 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"

// header
#include "Behaviour/BehaviourTypes/Predalien/AVP_Behaviour_Predalien_GS2Move.h"

// asura
#ifndef ASURA_NO_DEBUG_OUTPUT
#include "Rendering/Asura_Font_System.h"
#endif

// axon
#include "Behaviour/BehaviourTypes/AVP_Behaviour_Types.h"
#include "Behaviour/BehaviourTypes/Predalien/AVP_Behaviour_Predalien_Movement.h"
#include "Core/BrainTypes/AVP_Brain_Actor_Predalien.h"

// avp
#include "Entity/Locations/VantagePoint/AVP_ServerNode_VantagePoint_Predator.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Predalien/AVP_ServerActor_Predalien.h"

//////////////////////////////////////////////////////////////////////////////
//
// AVP_Behaviour_Predalien_GS2Move
//
//////////////////////////////////////////////////////////////////////////////

AVP_Behaviour_Predalien_GS2Move::AVP_Behaviour_Predalien_GS2Move( AVP_Brain_Actor_Predalien* pxBrain )
: PARENT( pxBrain, AVP_BEHAVIOUR_TYPE_PREDALIEN_GS2MOVE )
, m_eCurrentState( UNSET )
, m_fTimeInState( 0.0f )
, m_fArrivalRange( 0.75f )
, m_xDestination( Asura_Maths::ZeroVector3 )
, m_bDestinationSet( false )
{
}


bool AVP_Behaviour_Predalien_GS2Move::Activate()
{
#ifndef ASURA_NO_DEBUG_OUTPUT
	GetPredalienBrain()->SetRenderRoute( true );
#endif

	return PARENT::Activate();
}

void AVP_Behaviour_Predalien_GS2Move::Deactivate()
{
	Axon_Behaviour* pxChild = GetChild();
	while ( pxChild )
	{
		UnlinkChild( pxChild );
		GetProjectBrain()->DestroyBehaviour( pxChild );
		pxChild = GetChild();
	}

    PARENT::Deactivate();
}


bool AVP_Behaviour_Predalien_GS2Move::GetToPosition( const Asura_Vector_3& xDestination )
{
	SetInError( false );
	SetFinished( false );
	SwitchToState(UNSET);

	m_xDestination = xDestination;
	m_bDestinationSet = true;

	return true;
}


void AVP_Behaviour_Predalien_GS2Move::Update( const float fTimeslice )
{
    PARENT::Update( fTimeslice );

    if ( IsFinished() || IsInError() )
    {
        return;
    }

    m_fTimeInState += fTimeslice;
	switch ( m_eCurrentState )
	{
	case UNSET:
		Asura_Assert(m_bDestinationSet, "GetToPosition not called");
		if (GetPredalienBrain()->CanPlanMove() &&  m_bDestinationSet)
		{
			CalculateMove();
		}
		break;
	case MOVE_TO_POSITION:
		{
			AVP_Navigation::NavData& xNavData = GetPredalienBrain()->GetNavData();
			Axon_Behaviour* pxMovementChild = GetChildWithType( this, AVP_BEHAVIOUR_TYPE_PREDALIEN_MOVEMENT );
			if ( !pxMovementChild || pxMovementChild->IsInError() )
			{
				if (!GetToPosition(xNavData.m_xDestinationPosition))
				{
					SetInError(true);
					SwitchToState(FINISHED);
				}
			}
			else if (pxMovementChild->IsFinished())
			{
				SwitchToState(FINISHED);
			}
		}
		break;
	case FINISHED:
		break;
	default:
		Asura_Assert(false, "unexpected state");
	}
}

#ifndef ASURA_NO_DEBUG_OUTPUT
static const char* s_aszStateStrings[] =
{
    "UNSET",
	"GET TO POSITION",
	"FINISHED"
};

void AVP_Behaviour_Predalien_GS2Move::RenderDebugInfo()
{
    PARENT::RenderDebugInfo();

    Asura_Font_System::Print( GetProjectBrain()->GetDebugOutputPos().x, GetProjectBrain()->GetDebugOutputPos().y, 0.5f, 0xffffffff,
        ASURA_TEXTFORMAT_CENTREJUSTIFIED | ASURA_TEXTFORMAT_FORMATSTRING, "%s", s_aszStateStrings[ m_eCurrentState ] );
    GetProjectBrain()->GetDebugOutputPos().y += 10.0f;
}
#endif

AVP_Brain_Actor_Predalien* AVP_Behaviour_Predalien_GS2Move::GetPredalienBrain() const
{
    return static_cast< AVP_Brain_Actor_Predalien* >( GetProjectBrain() );
}

void AVP_Behaviour_Predalien_GS2Move::SwitchToState( const State eNextState )
{
	
	switch ( m_eCurrentState )
	{
	case UNSET:
		break;
	case MOVE_TO_POSITION:
		if ( GetChildWithType( this, AVP_BEHAVIOUR_TYPE_PREDALIEN_MOVEMENT ) )
		{
			DestroyChildWithType( AVP_BEHAVIOUR_TYPE_PREDALIEN_MOVEMENT );
		}
		break;
	case FINISHED:
	default:
		Asura_Assert(false, "unexpected state");
	}

	m_eCurrentState = eNextState;
	m_fTimeInState  = 0.0f;
	switch ( m_eCurrentState )
	{
	case MOVE_TO_POSITION:
		{
			AVP_Behaviour_Predalien_Movement* pxMovementBehaviour =
				static_cast< AVP_Behaviour_Predalien_Movement* >( CreateChildWithType( AVP_BEHAVIOUR_TYPE_PREDALIEN_MOVEMENT ) );
			if ( !pxMovementBehaviour )
			{
				SetInError(true);
				SwitchToState( FINISHED );
			}
			else
			{
				pxMovementBehaviour->SetMovementMode(AVP_Behaviour_Predalien_Movement::EMM_MEDIUM);
			}
		}
		break;
	case FINISHED:
		m_bDestinationSet = false;
		SetFinished(true);
		break;
	case UNSET:
		break;
	default:
		Asura_Assert(false, "unexpected state");
		break;
	}
}

void AVP_Behaviour_Predalien_GS2Move::CalculateMove()
{

	AVP_Navigation::NavData& xNavData = GetPredalienBrain()->GetNavData();

	xNavData.m_xStartPosition = GetPredalienBrain()->GetActor()->GetPosition();
	xNavData.m_xDestinationPosition = m_xDestination;
	// Our current waypoint ID should already be up to date, as we keep it up to date when moving.
	// Update our destination waypoint ID.
	const AVP_Navigation::TestParams xTestParams( xNavData.m_uPathfinderFlags, ASURA_NAVVERTEX_MAX_ZONE_ID );
	if ( !AVP_Navigation::GetNearestWaypointID( m_xDestination, xNavData.m_usDestinationWPID, xTestParams ) )
	{
		SetInError();
		SwitchToState( FINISHED );
		return;
	}

	if ( ( xNavData.m_xStartPosition - xNavData.m_xDestinationPosition ).MagnitudeSquared() < m_fArrivalRange * m_fArrivalRange )
	{
		// If we're already near enough the destination then there's nothing to do.
		SwitchToState( FINISHED );
	}
	else
	{
		// Clear out the old route and find the new one.
		xNavData.ResetRoute();
		if ( !AVP_Navigation::FindRoute( xNavData ) )
		{
			SetInError();
			SwitchToState( FINISHED );
		}

		SwitchToState( MOVE_TO_POSITION );
	}

	return;
}


//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////
