//////////////////////////////////////////////////////////////////////////////
//
// AVP_Behaviour_Predator_Melee.cpp
//
// Copyright (C) 2007 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"

// header
#include "Behaviour/BehaviourTypes/Predator/AVP_Behaviour_Predator_Melee.h"

// asura
#include "AI/Asura_AI_Navigation.h"
#include "Network/Asura_Message.h"
#ifndef ASURA_NO_DEBUG_OUTPUT
#include "Rendering/Asura_Font_System.h"
#endif


// axon
#include "Behaviour/BehaviourTypes/Predator/AVP_Behaviour_Predator_Movement.h"
#include "Behaviour/BehaviourTypes/AVP_Behaviour_Types.h"
#include "Core/AVP_Navigation.h"
#include "Core/BrainTypes/AVP_Brain_Actor_Predator.h"
#include "Knowledge/AVP_Knowledge_System_Predator.h"


// avp
#include "Actor/Vertebrates/AVP_Vertebrate_MeleeCounter_System.h"
#include "Actor/Vertebrates/Predator/AVP_Predator_AIAnimStateMachine.h"
#include "Actor/Vertebrates/Predator/AVP_Predator_MeleeCounter_System.h"
#include "Animation/AVP_AnimFlags.h"
#include "Entity/AVP_Messages.h"
#include "Entity/PhysicalObject/Actor/AVP_ServerEntity_Actor.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/AVP_Vertebrate_Messages.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/FaceHugger/AVP_ServerActor_FaceHugger.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Predator/AVP_ServerActor_Predator.h"
#include "Entity/PhysicalObject/Moves/Asura_Movement_System.h"
#include "Moves/AVP_Moves.h"


//////////////////////////////////////////////////////////////////////////////
//
// AVP_PredatorChargeFunctor
// Class that calculate cost for A* algorithm trying to avoid moving visibly to all the threats in the knowledge system
//
//////////////////////////////////////////////////////////////////////////////

DEBUGVAR float dbg_fMinDistanceToSuppressJumpSq = 10.0f * 10.0f;
DEBUGVAR float dbg_fDistanceToMeleeSq = 2.2f * 2.2f;
DEBUGVAR float dbg_fDistanceToGrappleSq = 2.2f * 2.2f;
DEBUGVAR float dbg_fMinDistanceToSpeedSq = 6.0f * 6.0f;
DEBUGVAR float dbg_fMaxDistanceToSpeedSq = 20.0f * 20.0f;

DEBUGVAR float dbg_fMaxDistanceToPounceSq = 14.0f * 14.0f;
DEBUGVAR float dbg_fMinDistanceToPounceSq = 3.0f * 3.0f;
DEBUGVAR float dbg_fHightToJumpUp = 4.0f;
DEBUGVAR float dbg_fHightToFollowUp = 25.0f;
DEBUGVAR float dbg_fMinTimeBetweenPounces = 2.0f;
DEBUGVAR float dbg_fMaxIdleTime = 0.5f;
DEBUGVAR float dbg_fMaxFireDistanceToTargetSq = 30.0f * 30.0f;
DEBUGVAR float dbg_fMaxFireDistanceToPositionSq = 30.0f * 30.0f;
//DEBUGVAR float dbg_fDesiredFireDistanceToPositionSq = 25.0f * 25.0f;
DEBUGVAR float dbg_fMinFireDistanceToPositionSq = 15.0f * 15.0f;

DEBUGVAR float dbg_fMaxDistanceToReplanMeleeSq = 2.0f * 2.0f;
DEBUGVAR float dbg_fMaxDistanceToReplanFollowSq = 4.0f * 4.0f;
DEBUGVAR float dbg_fSpeedStabilizationDistanceSq = 2.0f;
DEBUGVAR float dbg_fMaxTimeToStandInFollow = 4.0f;

DEBUGVAR float dbg_fOldKnowledgeAge = 6.0f;
DEBUGVAR float dbg_fAncientKnowledgeAge = 12.0f;


//////////////////////////////////////////////////////////////////////////////
// AVP_PredatorChargeFunctor
//////////////////////////////////////////////////////////////////////////////

class AVP_PredatorChargeFunctor : public AVP_Navigation::AStarFunctorBase
{
public:
	AVP_PredatorChargeFunctor(const Asura_Vector_3& xDestination)
		: m_xDestination(xDestination)
	{}

	virtual float EstimateTraversalCost(const int iWaypointIndex, const int iLinkIndex, const u_int uPathfinderFlags);
	virtual bool LinkFlagsTest( const Asura_AI_NavConnection* pxLink, const u_int uPathfinderFlags );

private:

	const Asura_Vector_3 m_xDestination;
};

float AVP_PredatorChargeFunctor::EstimateTraversalCost(const int iWaypointIndex, const int iLinkIndex, const u_int uPathfinderFlags)
{
	float fCost = Asura_AI_Navigation_System::DefaultTraversalCostFunction( iWaypointIndex, iLinkIndex, uPathfinderFlags, NULL );

	const Asura_AI_NavVertex* pxFromWP = Asura_AI_Navigation_System::GetWaypoint( iWaypointIndex );
	const Asura_AI_NavConnection& xLink = pxFromWP->GetLink(iLinkIndex);
	const Asura_AI_NavVertex* pxToWP = Asura_AI_Navigation_System::GetWaypoint( xLink.GetIndex() );

	float fCostDueTojumping = 0.0f;

	if ( (xLink.GetFlags() & AVP_Navigation::AVP_AI_CONNECTION_FLAG_TRANSITION_HIGH_MOBILITY) &&
		(m_xDestination - pxToWP->GetPosition()).MagnitudeSquared() < dbg_fMinDistanceToSuppressJumpSq )
	{
		fCostDueTojumping = fCost * 4.0f;
	}

	return fCost + fCostDueTojumping;
}

bool AVP_PredatorChargeFunctor::LinkFlagsTest( const Asura_AI_NavConnection* pxLink, const u_int uPathfinderFlags )
{
	return AVP_Navigation::PredatorLinkFlagsTest(pxLink, uPathfinderFlags);
}


//////////////////////////////////////////////////////////////////////////////
// AVP_PredatorAwayAvoidJumpFunctor
//////////////////////////////////////////////////////////////////////////////

class AVP_PredatorInDirectionFunctor : public AVP_Navigation::AStarFunctorBase
{
public:
	AVP_PredatorInDirectionFunctor(const Asura_Vector_3& xCentre, float fRange, const Asura_Vector_3& xDirection)
		:  m_xCentre(xCentre),  m_fRange(fRange),  m_xDirection(xDirection)
	{}

	virtual float EstimateTraversalCost(const int iWaypointIndex, const int iLinkIndex, const u_int uPathfinderFlags);
	virtual bool LinkFlagsTest( const Asura_AI_NavConnection* pxLink, const u_int uPathfinderFlags );
	virtual bool IsGoalAchieved( const int iWaypointIndex );
	virtual float EstimateCostToGoal( const int iWaypointIndex );

private:

	const Asura_Vector_3 m_xCentre;
	const float m_fRange;
	const Asura_Vector_3 m_xDirection;
};

float AVP_PredatorInDirectionFunctor::EstimateTraversalCost(const int iWaypointIndex, const int iLinkIndex, const u_int uPathfinderFlags)
{
	float fCost = Asura_AI_Navigation_System::DefaultTraversalCostFunction( iWaypointIndex, iLinkIndex, uPathfinderFlags, NULL );

	const Asura_AI_NavVertex* pxFromWP = Asura_AI_Navigation_System::GetWaypoint( iWaypointIndex );
	const Asura_AI_NavConnection& xLink = pxFromWP->GetLink(iLinkIndex);

	float fCostDueTojumping = 0.0f;

	if ( (xLink.GetFlags() & AVP_Navigation::AVP_AI_CONNECTION_FLAG_TRANSITION_HIGH_MOBILITY) ) // just a little bit of avoid jumping
	{
		fCostDueTojumping = fCost * 0.1f;
	}

	return fCost + fCostDueTojumping;
}

bool AVP_PredatorInDirectionFunctor::LinkFlagsTest( const Asura_AI_NavConnection* pxLink, const u_int uPathfinderFlags )
{
	return AVP_Navigation::PredatorLinkFlagsTest(pxLink, uPathfinderFlags);
}

bool AVP_PredatorInDirectionFunctor::IsGoalAchieved( const int iWaypointIndex )
{
	const Asura_AI_NavVertex* pxWP = Asura_AI_Navigation_System::GetWaypoint( iWaypointIndex );

	return (pxWP->GetPosition() - m_xCentre).MagnitudeSquared() > m_fRange*m_fRange;

}
float AVP_PredatorInDirectionFunctor::EstimateCostToGoal( const int iWaypointIndex )
{
	const Asura_AI_NavVertex* pxWP = Asura_AI_Navigation_System::GetWaypoint( iWaypointIndex );

	float fRange = (pxWP->GetPosition() - m_xCentre).Magnitude();

	if (fRange > m_fRange)
	{
		return 0.0f;
	}
	else
	{

		Asura_Vector_3 xToWP = pxWP->GetPosition() - m_xCentre;
		if (xToWP.ApproximatelyEquals(Asura_Maths::ZeroVector3))
		{
			return 10.0f;
		}

		xToWP.Normalise();

		float fCostDirection =  10*(1.0f - xToWP * m_xDirection);

		return fCostDirection; 
	}
}


//////////////////////////////////////////////////////////////////////////////
// AVP_Behaviour_Predator_Melee
//////////////////////////////////////////////////////////////////////////////

AVP_Behaviour_Predator_Melee::AVP_Behaviour_Predator_Melee( AVP_Brain_Actor_Predator* pxBrain )
: PARENT( pxBrain, AVP_BEHAVIOUR_TYPE_PREDATOR_MELEE )
, m_eCurrentState( UNSET )
, m_fTimeInState( 0.0f )
, m_bNeedToAimAtTarget(false)
, m_fTimeSinceLastPounce(100.0f)
, m_xDestination(Asura_Maths::ZeroVector3)
, m_xTargetPosition(Asura_Maths::ZeroVector3)
, m_usDestinationWPIndex(ASURA_INVALID_WAYPOINT_ID)
, m_xDelayedMsgList()
, m_bWasTargetKilled(false)
, m_uCurrentMeleeTarget(ASURA_GUID_UNREGISTERED)
, m_uCurrentMeleedTarget(ASURA_GUID_UNREGISTERED)
, m_uCurrentFollowTarget(ASURA_GUID_UNREGISTERED)
, m_xIgnoreTargets()
, m_bCurrentGrappleSucceed(true)
, m_bAbort(false)
, m_uNumberOfAttacksSincePowerAttack(0)
, m_uNumberOfAttackers(0)
, m_bIsCharging(false)
, m_fTimeUntilFire(0.0f)
, m_eFollowMode(FOLLOW_STAND)
, m_bShouldFlee(false)
, m_xAlertPosition(Asura_Maths::ZeroVector3)
, m_fAlertAge(dbg_fOldKnowledgeAge +1.0f)
{
}

AVP_Behaviour_Predator_Melee::~AVP_Behaviour_Predator_Melee()
{
}

void AVP_Behaviour_Predator_Melee::Deactivate()
{
	// Destroy any children.
	Axon_Behaviour* pxChildBehaviour = GetChild();
	while ( pxChildBehaviour )
	{
		UnlinkChild( pxChildBehaviour );
		GetProjectBrain()->DestroyBehaviour( pxChildBehaviour );
		pxChildBehaviour = GetChild();
	}

	PARENT::Deactivate();
}

void AVP_Behaviour_Predator_Melee::PerFrameUpdate( const float fTimeslice )
{
	PARENT::PerFrameUpdate( fTimeslice );

	m_fTimeInState += fTimeslice;
	m_fTimeSinceLastPounce += fTimeslice;
	if(m_fTimeUntilFire >= 0.0f)
	{
		m_fTimeUntilFire -= fTimeslice;
	}

	if (m_bNeedToAimAtTarget)
	{
		AimAtTarget();
	}

	// update melee first - might cause early exit from melee mode.
	if (GetPredatorBrain()->CanUpdateMelee())
	{
		switch ( m_eCurrentState )
		{
		case MELEE:
		case GRAPPLE:
		case POUNCE:
			UpdateMelee();
			break;
		default:
			break;
		}
	}

	if (!GetPredatorBrain()->CanPlanMove())
	{
		return;
	}

	LateHandleMessage();

	switch ( m_eCurrentState )
	{
	case UNSET:
		if (m_bAbort)
		{
			SwitchToState(FINISHED);
			return;
		}
		else if (GetPredatorBrain()->WasAnyTargetSeenRecently(dbg_fOldKnowledgeAge))
		{
			SwitchToState(IDLE);
		}
		else
		{
			SwitchToState(FINISHED);
		}
		break;
	case IDLE:
		if (GetPredatorBrain()->IsCloaked())
		{
			GetPredatorBrain()->Decloak();
		}
		
		if (m_fTimeInState > dbg_fMaxIdleTime)
		{
			SwitchToState(FINISHED);
		}
		else if (GetCurrentMeleeTargetKnowledge())
		{
			SwitchToState(RUNNING_IN);
		}
		else if (GetCurrentFollowTargetKnowledge())
		{
			SwitchToState(FOLLOW);
		}
		else
		{
			UpdateIdle(fTimeslice);
		}
		break;
	case RUNNING_IN:
		if (m_bAbort)
		{
			SwitchToState(FINISHED);
			return;
		}
		UpdateMove(fTimeslice);
		break;
	case MELEE:
	case GRAPPLE:
	case POUNCE:
//		UpdateMelee();
		break;
	case FLINCHED:
		break;
	case FOLLOW:
		if (m_bAbort)
		{
			SwitchToState(FINISHED);
			return;
		}
		UpdateFollow(fTimeslice);
		break;
	case FINISHED:
		break;
	default:
		Asura_Assert(false, "unexpected state");
		break;
	}
}

void AVP_Behaviour_Predator_Melee::Update( const float fTimeslice )
{
	PARENT::Update(fTimeslice);

	UpdateIgnoreTarget(fTimeslice);
	SelectCurrentMeleeTarget();
	SelectCurrentFollowTarget();
	UpdateNumberOfAttackers();
}

bool AVP_Behaviour_Predator_Melee::HandleMessage( const Asura_Message* pxMessage )
{

	switch (pxMessage->GetMsgID())
	{
	case AVP_MSGID_ACTIVEGRAPPLE_RESULT:
		{
			const AVP_Message_Vertebrate_ActiveGrappleResult* pxData = static_cast<const AVP_Message_Vertebrate_ActiveGrappleResult*>(pxMessage->GetData());
			m_bCurrentGrappleSucceed = pxData->GetAttackerVictorious();
		}
		break;
	case ASURA_MSG_MOVECHAINCOMPLETE:
		if (m_eCurrentState != RUNNING_IN && m_eCurrentState != FOLLOW)
		{
			if (GetPredatorBrain()->IsMelee())
			{
				GetPredatorBrain()->SetMelee(false);
			}
			m_xDelayedMsgList.Push_Back(ASURA_MSG_MOVECHAINCOMPLETE);
			return true;
		}
		break;
	case ASURA_MSG_MOVE_ABORT:
		if (m_eCurrentState != RUNNING_IN && m_eCurrentState != FOLLOW)
		{
			if (GetPredatorBrain()->IsMelee())
			{
				GetPredatorBrain()->SetMelee(false);
			}
			m_xDelayedMsgList.Push_Back(ASURA_MSG_MOVE_ABORT);
			return true;
		}
		break;
	case AVP_MSGID_FLINCH: // check if a flinch started
		{
			const AVP_Msg_Flinch* pxData = static_cast<const AVP_Msg_Flinch*>(pxMessage->GetData());
			// Only bother to pay attention to the flinch if it is going to start a move.
			if (pxData && pxData->GetUseFlinchMove())
			{
				m_xDelayedMsgList.Push_Back(AVP_MSGID_FLINCH);
			}
		}
		return true;
	default:
		break;
	}

	return PARENT::HandleMessage( pxMessage );
}


#ifndef ASURA_NO_DEBUG_OUTPUT
static const char* s_aszStateStrings[] =
{
	"UNSET",
	"IDLE",
	"RUNNING_IN",
	"MELEE",
	"FLINCHED",
	"GRAPPLE",
	"POUNCE",
	"FOLLOW",
	"FINISHED"
};

static const char* s_aszFollowModeStrings[] =
{
	"TOWARDS",
	"AWAY",
	"RANDOM",
	"STAND"
};

void AVP_Behaviour_Predator_Melee::RenderDebugInfo()
{
	PARENT::RenderDebugInfo();

	if (m_eCurrentState != FOLLOW)
	{
		Asura_Font_System::Print( GetProjectBrain()->GetDebugOutputPos().x, GetProjectBrain()->GetDebugOutputPos().y, 0.5f, 0xffffffff,
			ASURA_TEXTFORMAT_CENTREJUSTIFIED | ASURA_TEXTFORMAT_FORMATSTRING, "Predator Melee: %s", s_aszStateStrings[ m_eCurrentState ] );
	}
	else
	{
		Asura_Font_System::Print( GetProjectBrain()->GetDebugOutputPos().x, GetProjectBrain()->GetDebugOutputPos().y, 0.5f, 0xffffffff,
			ASURA_TEXTFORMAT_CENTREJUSTIFIED | ASURA_TEXTFORMAT_FORMATSTRING, "Predator Melee: %s %s", s_aszStateStrings[ m_eCurrentState ], s_aszFollowModeStrings[m_eFollowMode] );

	}

	GetProjectBrain()->GetDebugOutputPos().y += 10.0f;
}
#endif

DEBUGVAR u_int dbg_uMinAttacksBetweenPowerAttack = 2;

void AVP_Behaviour_Predator_Melee::SwitchToState( const State eNextState )
{
	// Do any work we need to exit our current state.
	switch ( m_eCurrentState )
	{
	case FOLLOW:
		m_bNeedToAimAtTarget = false;
		DestroyChildWithType( AVP_BEHAVIOUR_TYPE_PREDATOR_MOVEMENT );
		if (m_bIsCharging && eNextState != FOLLOW)
		{
			GetPredatorBrain()->AbortPlasmaCasterCharge();
			m_bIsCharging = false;
		}
		break;
	case RUNNING_IN:
		DestroyChildWithType( AVP_BEHAVIOUR_TYPE_PREDATOR_MOVEMENT );
		break;
	case IDLE:
	case UNSET:
	case FLINCHED:
		break;
	case MELEE:
	case GRAPPLE:
		m_uCurrentMeleedTarget = ASURA_GUID_UNREGISTERED;
		m_bNeedToAimAtTarget = false;
		break;
	case POUNCE:
		m_uCurrentMeleedTarget = ASURA_GUID_UNREGISTERED;
		m_fTimeSinceLastPounce = 0.0f;
		m_bNeedToAimAtTarget = false;
		break;
	case FINISHED:
	default:
		Asura_Assert(false, "unexpected state" );
		break;
	}

	// Do any work we need to change into our next state.
	m_eCurrentState = eNextState;
	m_fTimeInState  = 0.0f;
	switch ( m_eCurrentState )
	{
	case RUNNING_IN:
		if (PlanMeleePath())
		{
			AVP_Behaviour_Predator_Movement* pxMovementBehaviour =
				static_cast< AVP_Behaviour_Predator_Movement* >( CreateChildWithType( AVP_BEHAVIOUR_TYPE_PREDATOR_MOVEMENT ) );
			if ( !pxMovementBehaviour )
			{
				SetInError(true);
				SwitchToState( FINISHED );
			}
			GetPredatorBrain()->EquipWristBlades();
		}
		else
		{
			SwitchToState( IDLE );
		}
		break;
	case FOLLOW:
		if (PlanFollowPath())
		{
			if (m_eFollowMode != FOLLOW_STAND)
			{
				AVP_Behaviour_Predator_Movement* pxMovementBehaviour =
					static_cast< AVP_Behaviour_Predator_Movement* >( CreateChildWithType( AVP_BEHAVIOUR_TYPE_PREDATOR_MOVEMENT ) );
				if ( !pxMovementBehaviour )
				{
					SetInError(true);
					SwitchToState( FINISHED );
				}
				else
				{
					pxMovementBehaviour->SetMovementMode(AVP_Behaviour_Predator_Movement::EMM_MEDIUM);
				}
			}
			else
			{
				AVP_Vertebrate_AIAnimStateMachine* pxVertStateMachine = static_cast< AVP_Vertebrate_AIAnimStateMachine* >( GetPredatorBrain()->GetActor()->GetAIAnimStateMachine() );
				pxVertStateMachine->RequestIdle();
			}
		}
		else
		{
			SwitchToState( IDLE );
		}
		break;
	case MELEE:
		{
			bool bRequestResult = false;
			if (++m_uNumberOfAttacksSincePowerAttack > dbg_uMinAttacksBetweenPowerAttack)
			{
				bRequestResult = RequestMelee( AVP_PREDATOR_MELEEATTACK_TYPE_POWERATTACK );
				m_uNumberOfAttacksSincePowerAttack = 0;
			}
			else
			{
				bRequestResult = RequestMelee( AVP_PREDATOR_MELEEATTACK_TYPE_PRIMARYRELEASE );
			}
			if(bRequestResult)
			{
				GetPredatorBrain()->SetMelee(true);
				m_uCurrentMeleedTarget = m_uCurrentMeleeTarget;
				m_bNeedToAimAtTarget = true;
			}
			else
			{

			}
		}
		break;
	case GRAPPLE:
		if(RequestMelee( AVP_PREDATOR_MELEEATTACK_TYPE_PICKUPENEMY ))
		{
			GetPredatorBrain()->SetMelee(true);
			m_uCurrentMeleedTarget = m_uCurrentMeleeTarget;
			m_bCurrentGrappleSucceed = true;
			m_bNeedToAimAtTarget = true;
		}
		break;
	case POUNCE:
		if(RequestMelee( AVP_PREDATOR_MELEEATTACK_TYPE_FOCUSPOUNCE ))
		{
			GetPredatorBrain()->SetMelee(true);
			m_uCurrentMeleedTarget = m_uCurrentMeleeTarget;
			m_bNeedToAimAtTarget = true;
		}
		break;
	case IDLE:
		ResetAlert();
		GetPredatorBrain()->Decloak();
		break;
	case FINISHED:
		GetPredatorBrain()->SetNearbyActorGuidToIgnore(ASURA_GUID_UNREGISTERED);
		SetFinished(true);
		break;
	case UNSET:
		GetPredatorBrain()->SetNearbyActorGuidToIgnore(ASURA_GUID_UNREGISTERED);
		break;
	case FLINCHED:
		break;
	default:
		Asura_Assert(false, "unexpected state" );
		break;
	}
}

AVP_Brain_Actor_Predator* AVP_Behaviour_Predator_Melee::GetPredatorBrain() const
{
	return static_cast< AVP_Brain_Actor_Predator* >( GetProjectBrain() );
}

bool AVP_Behaviour_Predator_Melee::PlanMeleePath()
{

	const Axon_Knowledge* pxTargetKnowledge = GetCurrentMeleeTargetKnowledge();
	if (!pxTargetKnowledge)
	{
		return false;
	}

	m_xDestination = pxTargetKnowledge->m_xPosition;
	m_xTargetPosition = pxTargetKnowledge->m_xPosition;

	if (!IsTargetReachable(pxTargetKnowledge, dbg_fHightToJumpUp, m_usDestinationWPIndex, m_xDestination))
	{
		//IgnoreTarget(pxTargetKnowledge->GetGuid());
		return false;
	}

	if (!PlanPath())
	{
		IgnoreTarget(pxTargetKnowledge->GetGuid(), pxTargetKnowledge->m_xPosition);
		return false;
	}

	GetPredatorBrain()->SetNearbyActorGuidToIgnore(pxTargetKnowledge->GetGuid());

	return true;

}

bool AVP_Behaviour_Predator_Melee::PlanFollowPath()
{

	const Axon_Knowledge* pxTargetKnowledge = GetCurrentFollowTargetKnowledge();
	if (!pxTargetKnowledge)
	{
		return false;
	}

	m_xDestination = pxTargetKnowledge->m_xPosition;
	m_xTargetPosition = pxTargetKnowledge->m_xPosition;

	float fDistanceToTargetSq = (GetPredatorBrain()->GetPredatorActor()->GetPosition() - m_xDestination).MagnitudeSquared();

	const bool bIsTargetReachable = IsTargetReachable(pxTargetKnowledge, dbg_fHightToFollowUp, m_usDestinationWPIndex, m_xDestination);

	if ( fDistanceToTargetSq < dbg_fMinFireDistanceToPositionSq )
	{
		m_eFollowMode = FOLLOW_AWAY;
	}
	else if (fDistanceToTargetSq > dbg_fMaxFireDistanceToPositionSq)
	{
		m_eFollowMode = FOLLOW_TOWARDS;
	}
	else
	{
		if (Asura_Random::GetInt(0, 10) < 3)
		{
			m_eFollowMode = FOLLOW_STAND;
		}
		else
		{
			m_eFollowMode = FOLLOW_RANDOM;
		}
	}

	if (m_eFollowMode == FOLLOW_TOWARDS)
	{
		if (!bIsTargetReachable || !PlanPath())
		{
			if (!PlanPathInDirection())
			{
				return false;
			}
		}
	}
	else if (!PlanPathInDirection())
	{
		return false;
	}

	return true;
	 
}


bool AVP_Behaviour_Predator_Melee::PlanPath()
{

	AVP_Navigation::NavData& xNavData = GetPredatorBrain()->GetNavData();

	xNavData.m_xStartPosition = GetPredatorBrain()->GetActor()->GetPosition();
	xNavData.m_eMovementMode = AVP_Navigation::MM_FAST;
	xNavData.m_xDestinationPosition = m_xDestination;
	xNavData.m_usDestinationWPID = m_usDestinationWPIndex;

	AVP_PredatorChargeFunctor xFunctor(m_xDestination);

	// Clear out the old route and find the new one.
	xNavData.ResetRoute();
	if ( !AVP_Navigation::FindRoute( xNavData, &xFunctor, true ) )
	{
		return false;
	}

	return true;
}

DEBUGVAR float dbg_fRangeToMoveInDirection = 25.0f;
DEBUGVAR float dbg_fAngleVariation = Asura_Maths::DegToRad(45.0f);

bool AVP_Behaviour_Predator_Melee::PlanPathInDirection()
{

	AVP_Navigation::NavData& xNavData = GetPredatorBrain()->GetNavData();

	const Asura_Vector_3& xOurPosition = GetPredatorBrain()->GetPredatorActor()->GetPosition();

	xNavData.m_xStartPosition = xOurPosition;
	xNavData.m_eMovementMode = AVP_Navigation::MM_FAST;
	xNavData.ResetRoute();

	if (m_eFollowMode == FOLLOW_STAND)
	{
		return true;
	}

	Asura_Vector_3 xDirectionToTarget =  m_xDestination - xOurPosition;
	xDirectionToTarget.y = 0.0f; // ignore hight difference
	if (xDirectionToTarget.MagnitudeSquared() < 0.00001)
	{
		xDirectionToTarget.x = Asura_Random::GetFloat(-1.0f, 1.0f); 
		xDirectionToTarget.z = 1.0f -  xDirectionToTarget.x; 
	}

	xDirectionToTarget.Normalise();

	Asura_Vector_3 xDirection = Asura_Maths::ZeroVector3;
	float xRotationAngle = 0.0f;

	switch (m_eFollowMode)
	{
	case FOLLOW_AWAY:
		xDirection = -xDirectionToTarget;
		xRotationAngle = Asura_Random::GetFloat(-1.0f, 1.0f) * dbg_fAngleVariation;
		break;
	case FOLLOW_TOWARDS:
		xDirection = xDirectionToTarget;
		xRotationAngle = Asura_Random::GetFloat(-1.0f, 1.0f) * dbg_fAngleVariation;
		break;
	case FOLLOW_RANDOM:
		xDirection = xDirectionToTarget;
		xRotationAngle =  (Asura_Random::GetInt(0,1) == 0 ? 1.0f : -1.0f) * Asura_Maths::DegToRad(90.0f) + Asura_Random::GetFloat(-1.0f, 1.0f) * dbg_fAngleVariation;
		break;
	case FOLLOW_STAND:
	default:
		Asura_Assert(false, "Unexpected follow mode");
		return false;
	}

	Asura_Quat xRotate;
	xRotate.FromRotation(xRotationAngle, Asura_AVector3(0.0f,-1.0f, 0.0f));
	xDirection = xRotate.RotateVector(xDirection);

	AVP_PredatorInDirectionFunctor xFunctor(xOurPosition, dbg_fRangeToMoveInDirection, xDirection);

	// Clear out the old route and find the new one.
	if ( !AVP_Navigation::FindRouteAwayFrom( xNavData, &xFunctor, true ) )
	{
		return false;
	}

	if (xNavData.m_usRouteLength < 1)
	{
		Asura_Assert(false, "PlanPathInDirection failed");
		return false;
	}

	const Asura_AI_NavVertex* pxFromWP = Asura_AI_Navigation_System::GetWaypoint( xNavData.m_pusRoute[xNavData.m_usRouteLength-1] );

	xNavData.m_xDestinationPosition = pxFromWP->GetPosition();

	return true;
}

DEBUGVAR float dbg_fPounceMaxDirDiffCos = Asura_Maths::Cosine(Asura_Maths::DegToRad(45.0f));

void AVP_Behaviour_Predator_Melee::UpdateMove(float fTimeslice)
{
	if (ShouldAbortAttack())
	{
		m_bShouldFlee = true;
		SwitchToState(FINISHED);
		return;
	}

	const Axon_Knowledge* pxTargetKnowledge = GetCurrentMeleeTargetKnowledge();
	if (!pxTargetKnowledge)
	{
		SwitchToState(IDLE);
		return;
	}

	AVP_ServerEntity_Actor* pxTarget = AVP_ServerEntity_Actor::FindEntity(pxTargetKnowledge->GetGuid());
	if (!pxTarget)
	{
		SwitchToState(IDLE);
		return;
	}	

	bool bIsHoldingWristBlades = GetPredatorBrain()->IsHoldingWristBlades();

	if (!bIsHoldingWristBlades)
	{
		GetPredatorBrain()->EquipWristBlades();
	}

	Asura_Vector_3 xOurPosition = GetPredatorBrain()->GetActor()->GetPosition();

	AVP_Behaviour_Predator_Movement* pxBehaviour = static_cast< AVP_Behaviour_Predator_Movement* >( GetChildWithType( this, AVP_BEHAVIOUR_TYPE_PREDATOR_MOVEMENT ) );
	if ( !pxBehaviour || pxBehaviour->IsInError() )
	{
		SwitchToState(RUNNING_IN);
		return;
	}
	Asura_Vector_3 xToTargetPosition = pxTarget->GetPosition() - xOurPosition;
	Asura_Vector_3 xToTargetMeleeOrigin; 
	pxTarget->GetMeleeOrigin(xToTargetMeleeOrigin);
	xToTargetMeleeOrigin -= xOurPosition;
	float fDistanceSq = Asura_Maths::Min(xToTargetPosition.MagnitudeSquared(), xToTargetMeleeOrigin.MagnitudeSquared());

	Asura_Vector_3 xToTargetMeleeOriginDir = xToTargetMeleeOrigin;
	xToTargetMeleeOriginDir.Normalise();
	bool bFacingTargetToPounce = GetPredatorBrain()->GetAimDirection()*xToTargetMeleeOriginDir > dbg_fPounceMaxDirDiffCos;

	bool bMustGrapple = MustGrapple(pxTarget);
	bool bCanGrapple = CanGrapple(pxTarget);
	bool bCanMelee = CanMelee(pxTarget);

	if ( bIsHoldingWristBlades && bMustGrapple && fDistanceSq < dbg_fDistanceToGrappleSq)
	{
		if (bCanGrapple)
		{
			if (pxBehaviour)
			{
				pxBehaviour->Stop();
			}
			SwitchToState(GRAPPLE);
		}
	}
	else if (bIsHoldingWristBlades && !bMustGrapple && fDistanceSq < dbg_fDistanceToGrappleSq && bCanGrapple )
	{
		if (pxBehaviour)
		{
			pxBehaviour->Stop();
		}
		SwitchToState(GRAPPLE);
	}
	else if (bIsHoldingWristBlades && !bMustGrapple && fDistanceSq < dbg_fDistanceToMeleeSq && !bCanGrapple && bCanMelee )
	{

		if (pxBehaviour)
		{
			pxBehaviour->Stop();
		}
		SwitchToState( MELEE );
	}
	else if ( bIsHoldingWristBlades && !bMustGrapple && bFacingTargetToPounce && 
			  fDistanceSq < dbg_fMaxDistanceToPounceSq && 
			  fDistanceSq > dbg_fMinDistanceToPounceSq && 
			  m_fTimeSinceLastPounce > dbg_fMinTimeBetweenPounces &&
			  pxTargetKnowledge->m_fAge == 0.0f &&
			  static_cast<const AVP_Predator_MeleeCounter_System*>(GetPredatorBrain()->GetPredatorActor()->GetMeleeSystem())->GetCanPounce(pxTarget) )
	{
		if (pxBehaviour)
		{
			pxBehaviour->Stop();
		}
		SwitchToState(POUNCE);
	}	
	else if( pxBehaviour->IsFinished() || (pxTargetKnowledge->m_xPosition - m_xTargetPosition).MagnitudeSquared() > dbg_fMaxDistanceToReplanMeleeSq )
	{
		// make update to aim direction since if we are too close the move behaviour will always be in finished state.

		Asura_Vector_3 xTargetHead;
		pxTarget->GetHeadPosition(xTargetHead);
		Asura_Vector_3 xOurHeadPosition;
		GetPredatorBrain()->GetPredatorActor()->GetHeadPosition(xOurHeadPosition);
		xToTargetPosition = xTargetHead - xOurHeadPosition;
		xToTargetPosition.Normalise();
		GetPredatorBrain()->SetAimDirection( xToTargetPosition );

		SwitchToState(RUNNING_IN);
	}
	else if( fDistanceSq > dbg_fMaxDistanceToSpeedSq)
	{
		pxBehaviour->SetMovementMode(AVP_Behaviour_Predator_Movement::EMM_MEDIUM);
	}
	else 
	{

		float fDistanceToPositionSq = pxBehaviour->GetLengthLeftOnPath();
		fDistanceToPositionSq *= fDistanceToPositionSq;
		switch (pxBehaviour->GetMovementMode() )
		{
		case AVP_Behaviour_Predator_Movement::EMM_IDLE:
		case AVP_Behaviour_Predator_Movement::EMM_SLOW:
			pxBehaviour->SetMovementMode(AVP_Behaviour_Predator_Movement::EMM_MEDIUM);
			break;
		case AVP_Behaviour_Predator_Movement::EMM_MEDIUM:
			if (fDistanceToPositionSq < dbg_fMaxDistanceToSpeedSq - dbg_fSpeedStabilizationDistanceSq || 
				fDistanceToPositionSq > dbg_fMinDistanceToSpeedSq + dbg_fSpeedStabilizationDistanceSq)
			{
				pxBehaviour->SetMovementMode(AVP_Behaviour_Predator_Movement::EMM_FAST);
			}
			break;
		case AVP_Behaviour_Predator_Movement::EMM_FAST:
			if (fDistanceToPositionSq > dbg_fMaxDistanceToSpeedSq || 
				fDistanceToPositionSq < dbg_fMinDistanceToSpeedSq )
			{
				pxBehaviour->SetMovementMode(AVP_Behaviour_Predator_Movement::EMM_MEDIUM);
			}
			break;
		}

		if( fDistanceToPositionSq < dbg_fMinDistanceToSpeedSq)
		{
			Asura_Vector_3 xTargetHead;
			pxTarget->GetHeadPosition(xTargetHead);
			Asura_Vector_3 xOurHeadPosition;
			GetPredatorBrain()->GetPredatorActor()->GetHeadPosition(xOurHeadPosition);
			xToTargetPosition = xTargetHead - xOurHeadPosition;

			pxBehaviour->SetMovementMode(AVP_Behaviour_Predator_Movement::EMM_MEDIUM);
			pxBehaviour->AimAt(xToTargetPosition);
		}
		else
		{
			pxBehaviour->AimWhereYouMove();
		}
	}
}

void AVP_Behaviour_Predator_Melee::UpdateFollow(float fTimeslice)
{
	if (ShouldAbortAttack())
	{
		m_bShouldFlee = true;
		SwitchToState(FINISHED);
		return;
	}

	if ( GetCurrentMeleeTargetKnowledge() )
	{
		SwitchToState(IDLE);
		return;
	}

	const Axon_Knowledge* pxTargetKnowledge = GetCurrentFollowTargetKnowledge();
	if (!pxTargetKnowledge)
	{
		SwitchToState(IDLE);
		return;
	}

	AVP_ServerEntity_Actor* pxTarget = AVP_ServerEntity_Actor::FindEntity(pxTargetKnowledge->GetGuid());
	if (!pxTarget)
	{
		SwitchToState(IDLE);
		return;
	}	

	Asura_Vector_3 xOurPosition = GetPredatorBrain()->GetActor()->GetPosition();

	const bool bIsStand = m_eFollowMode == FOLLOW_STAND;

	AVP_Behaviour_Predator_Movement* pxBehaviour = bIsStand ? NULL : static_cast< AVP_Behaviour_Predator_Movement* >( GetChildWithType( this, AVP_BEHAVIOUR_TYPE_PREDATOR_MOVEMENT ) );
	Asura_Vector_3 xToTargetPosition = pxTarget->GetPosition() - xOurPosition;
	float fDistanceToTargetSq = xToTargetPosition.MagnitudeSquared();

	Asura_Vector_3 xOurMuzzlePosition;		
	if (!GetPredatorBrain()->GetPredatorActor()->GetMuzzlePosition(xOurMuzzlePosition)) // we can not get muzzle position if weapon is not equipped - get next best thing and get head
	{
		GetPredatorBrain()->GetPredatorActor()->GetHeadPosition(xOurMuzzlePosition);
	}

	Asura_Vector_3 xFromMuzzleToPosition = pxTarget->GetPosition() - xOurMuzzlePosition;
	float fFromMuzzleToPositionDistance = xFromMuzzleToPosition.Normalise();

	if (bIsStand)
	{
		if (m_fTimeInState > dbg_fMaxTimeToStandInFollow)
		{
			SwitchToState(FOLLOW);
		}
		else
		{
			GetPredatorBrain()->SetAimDirection( xFromMuzzleToPosition);
			TryToFire(xFromMuzzleToPosition, fFromMuzzleToPositionDistance);
		}
	}
	else if ( !pxBehaviour || pxBehaviour->IsInError() )
	{
		SwitchToState(FOLLOW);
	}
	else if( pxBehaviour->IsFinished() || (pxTargetKnowledge->m_xPosition - m_xTargetPosition).MagnitudeSquared() > dbg_fMaxDistanceToReplanFollowSq )
	{
		SwitchToState(FOLLOW);
	}
	else if (fDistanceToTargetSq < dbg_fMaxFireDistanceToTargetSq)
	{


// 		GetPredatorBrain()->EquipPlasmaCaster();
// 
// 		float fDistanceToPositionSq = pxBehaviour->GetLengthLeftOnPath();
// 		fDistanceToPositionSq *= fDistanceToPositionSq;
// 		switch (pxBehaviour->GetMovementMode() )
// 		{
// 		case AVP_Behaviour_Predator_Movement::EMM_IDLE:
// // 			if (fDistanceToPositionSq > dbg_fMinFireDistanceToPositionSq + dbg_fSpeedStabilizationDistanceSq)
// 			{
// 				pxBehaviour->SetMovementMode(AVP_Behaviour_Predator_Movement::EMM_SLOW);
// 			}
// 			break;
// 		case AVP_Behaviour_Predator_Movement::EMM_SLOW:
// 			if (fDistanceToPositionSq < dbg_fMinFireDistanceToPositionSq)
// 			{
// 				// pxBehaviour->SetMovementMode(AVP_Behaviour_Predator_Movement::EMM_IDLE);
// 			}
// 			else if (fDistanceToPositionSq > dbg_fDesiredFireDistanceToPositionSq + dbg_fSpeedStabilizationDistanceSq)
// 			{
// 				pxBehaviour->SetMovementMode(AVP_Behaviour_Predator_Movement::EMM_MEDIUM);
// 			}
// 			break;
// 		case AVP_Behaviour_Predator_Movement::EMM_MEDIUM:
// 			if (fDistanceToPositionSq < dbg_fDesiredFireDistanceToPositionSq)
// 			{
// 				pxBehaviour->SetMovementMode(AVP_Behaviour_Predator_Movement::EMM_SLOW);
// 			}
// 			break;
// 		case AVP_Behaviour_Predator_Movement::EMM_FAST:
// 			pxBehaviour->SetMovementMode(AVP_Behaviour_Predator_Movement::EMM_MEDIUM);
// 			break;
// 		}

		pxBehaviour->AimAt(xFromMuzzleToPosition);
		TryToFire(xFromMuzzleToPosition, fFromMuzzleToPositionDistance);
	}
	else if (m_eFollowMode != FOLLOW_TOWARDS)
	{
		SwitchToState(FOLLOW);
	}
	else
	{
		pxBehaviour->SetMovementMode(AVP_Behaviour_Predator_Movement::EMM_MEDIUM);
		pxBehaviour->AimWhereYouMove();
	}
}


void AVP_Behaviour_Predator_Melee::UpdateIdle(float fTimeslice)
{
	UpdateAlert(fTimeslice);
	if (m_fAlertAge < dbg_fOldKnowledgeAge)
	{
		AimAtPosition(m_xAlertPosition);
	}
}

void AVP_Behaviour_Predator_Melee::UpdateMelee()
{
	const Asura_Move* pxMove = GetPredatorBrain()->GetClientActor()->GetMovementSystem()->GetCurrentMove();
	const u_int uMoveAnimFlags = pxMove ? pxMove->GetAnimFlagsForLastFrame() : 0;
	if ( ( uMoveAnimFlags & AVP_ANIMFLAG_ALLOW_NEW_MELEE ) != 0 )
	{
		if (GetPredatorBrain()->IsMelee())
		{
			GetPredatorBrain()->SetMelee(false);
		}
		SwitchToState(RUNNING_IN);
	}
}

bool AVP_Behaviour_Predator_Melee::RequestMelee(Asura_Hash_ID uMeleeType)
{
	AVP_Brain_Actor_Predator* pxBrain =  GetPredatorBrain();

	const Axon_Knowledge* pxTargetKnowledge = GetCurrentMeleeTargetKnowledge();
	if (!pxTargetKnowledge)
	{
		SetInError(true);
		SwitchToState(UNSET);
		return false;
	}

	Asura_Message* pxMessage = Asura_Network::CreateMessage( AVP_MSGID_REQUEST_MELEE, sizeof( AVP_Message_Vertebrate_RequestMelee ) );
	if ( pxMessage )
	{
		AVP_Message_Vertebrate_RequestMelee* pxData = static_cast< AVP_Message_Vertebrate_RequestMelee* >( pxMessage->GetData() );

		pxData->SetType( uMeleeType );
		pxData->SetTargetGuid( pxTargetKnowledge->GetGuid() );

		pxMessage->SetToGuid( pxBrain->GetOwnerGuid() );
		pxMessage->SetFromGuid( pxBrain->GetOwnerGuid() );
		Asura_Network::Send( pxMessage );
		return true;
	}
	else
	{
		SetInError(true);
		SwitchToState(UNSET);
		return false;
	}
}

void AVP_Behaviour_Predator_Melee::AimAtTarget()
{
	const Axon_Knowledge* pxMeleeTargetKnowldeg = GetCurrentMeleeTargetKnowledge();
	const Axon_Knowledge* pxFollowTargetKnowldeg = GetCurrentFollowTargetKnowledge();

	const Axon_Knowledge* pxTargetKnowledge = pxMeleeTargetKnowldeg ? pxMeleeTargetKnowldeg : pxFollowTargetKnowldeg;
	if (!pxTargetKnowledge)
	{
		return;
	}
	AVP_ServerEntity_Actor* pxTarget = AVP_ServerEntity_Actor::FindEntity(pxTargetKnowledge->GetGuid());

	if (!pxTarget)
	{
		return;
	}

	Asura_Vector_3 xTargetHead;
	pxTarget->GetHeadPosition(xTargetHead);

	AimAtPosition(xTargetHead);
}

void AVP_Behaviour_Predator_Melee::AimAtPosition(const Asura_Vector_3& xPosition)
{
	Asura_Vector_3 xOurHeadPosition;
	GetPredatorBrain()->GetPredatorActor()->GetHeadPosition(xOurHeadPosition);
	Asura_Vector_3 xToTarget = xPosition - xOurHeadPosition;
	GetPredatorBrain()->SetAimDirection( xToTarget );
}


void AVP_Behaviour_Predator_Melee::LateHandleMessage()
{
	Asura_LinkList_It< int > xIter(m_xDelayedMsgList);
	for (; !xIter.Done(); xIter.Next())
	{
		int eMsgId = *xIter.GetCurrent();
		switch (eMsgId)
		{
		case ASURA_MSG_MOVECHAINCOMPLETE:
			switch (m_eCurrentState)
			{
			case MELEE:
				{
					const AVP_ServerEntity_Actor* pxTarget = AVP_ServerEntity_Actor::FindEntity(m_uCurrentMeleedTarget);
					if (!pxTarget || pxTarget->IsDead())
					{
						m_bWasTargetKilled = true;
					}
				}
				SwitchToState( RUNNING_IN );
				break;
			case FLINCHED:
				SwitchToState( RUNNING_IN );
				break;
			case GRAPPLE:
				{
					const AVP_ServerEntity_Actor* pxTarget = AVP_ServerEntity_Actor::FindEntity(m_uCurrentMeleedTarget);
					if (!pxTarget || pxTarget->IsDead())
					{
						m_bWasTargetKilled = true;
					}
				}
				if (m_bCurrentGrappleSucceed)
				{
					SwitchToState( RUNNING_IN );
				}
				else
				{ 
					SwitchToState( FINISHED );
				}
				break;
			case POUNCE:
				{
					const AVP_ServerEntity_Actor* pxTarget = AVP_ServerEntity_Actor::FindEntity(m_uCurrentMeleedTarget);
					if (!pxTarget || pxTarget->IsDead())
					{
						m_bWasTargetKilled = true;
					}
				}
				SwitchToState( RUNNING_IN );
				break;
			case IDLE:
				break;
			default:
				Asura_Assert(false, "Unexpected state");
				break;
			}
			break;
		case ASURA_MSG_MOVE_ABORT:
			if (m_eCurrentState == GRAPPLE)
			{
				SwitchToState( FINISHED );
			}
			if (m_eCurrentState != RUNNING_IN)
			{
				SwitchToState( RUNNING_IN );
			}
			else
			{
				Asura_Assert(false, "Unexpected state");
			}
			break;
		case AVP_MSGID_FLINCH: // check if a flinch started
			SwitchToState(FLINCHED);
			break;
		case IDLE:
			break;
		default:
			Asura_Assert(false, "Unexpected state");
			break;
		}
	}
	m_xDelayedMsgList.Clear();
}

DEBUGVAR float dbg_fBigOrientDifferenceCos = Asura_Maths::Cosine(Asura_Maths::DegToRad(45.0f));
DEBUGVAR float dbg_fBigHeadingDifferenceCos = Asura_Maths::Cosine(Asura_Maths::DegToRad(70.0f));
DEBUGVAR u_int dbg_uMaxInGroupToGrapple = 2;
bool AVP_Behaviour_Predator_Melee::CanGrapple(const AVP_ServerEntity_Actor* pxTarget)  const
{

	const AVP_ServerActor_Predator* pxActor = GetPredatorBrain()->GetPredatorActor();

	// no grapple if target can take more damage without dieing
	if (pxTarget->GetHealth() > pxActor->GetMeleeSystem()->GetModerateDamageAmount())
	{
		return false;
	}

	// no grapple if too many potenial attackers
	if (GetNumberOfAttackers() > dbg_uMaxInGroupToGrapple)
	{
		return false;
	}

	Asura_Vector_3 xTargetUp(-pxTarget->GetCurrentGravityDirection());
	Asura_Vector_3 xUp(0.0f, -1.0f, 0.0f);

	// only grapple if the  target is close to our orientation
	if ( xTargetUp * xUp < dbg_fBigOrientDifferenceCos )
	{
		return false;
	}

	// for now ignore front-back as we have grapple from behind but not for injured aliens
// 	// grapple allowed on player even if not facing so he can break free.
// 	if (pxTarget->IsPlayer())
// 	{
// 		return true;
// 	}
// 
// 	// dont grapple if not facing the target more or less 
// 	Asura_Vector_3 xTargetBackward;
// 	pxTarget->GetHeadFowardVector(xTargetBackward);
// 	xTargetBackward -= xTargetBackward;
// 	Asura_Vector_3 xFront;
// 	pxActor->GetHeadFowardVector(xFront);
// 
// 	if ( xTargetBackward * xFront < dbg_fBigHeadingDifferenceCos )
// 	{
// 		return false;
// 	}

	return CanMelee(pxTarget);

}

DEBUGVAR float dbg_fMaxAboveToMelee = -2.5f;
DEBUGVAR float dbg_fMaxBelowToMelee = 0.5f;

bool AVP_Behaviour_Predator_Melee::CanMelee(const AVP_ServerEntity_Actor* pxTarget)  const
{

	const AVP_ServerActor_Predator* pxActor = GetPredatorBrain()->GetPredatorActor();
	
	Asura_Vector_3 xTargetUp(-pxTarget->GetCurrentGravityDirection());
	Asura_Vector_3 xUp(0.0f, -1.0f, 0.0f);

	Asura_Vector_3 xToTarget = pxTarget->GetPosition() - pxActor->GetPosition();
	float fDeltaY = xToTarget.y;
	xToTarget.y = 0.0f;
	xToTarget.Normalise();

	const Asura_Matrix_3x3& xOrientation = pxActor->GetOrientation();
	const Asura_Vector_3 xFront(xOrientation.e13, xOrientation.e23, xOrientation.e33);
	
	if (fDeltaY < dbg_fMaxAboveToMelee || fDeltaY > dbg_fMaxBelowToMelee)
	{
		return false;
	}

 	if ( xToTarget * xFront < dbg_fBigHeadingDifferenceCos )
 	{
 		return false;
 	}

	return true;
}


bool AVP_Behaviour_Predator_Melee::MustGrapple(const AVP_ServerEntity_Actor* pxTarget)  const
{

	if (!pxTarget->IsPlayer())
	{
		return false;
	}

	const AVP_ServerActor_Predator* pxActor = GetPredatorBrain()->GetPredatorActor();

	// no grapple if target can take more damage without dieing
	if (pxTarget->GetHealth() > pxActor->GetMeleeSystem()->GetModerateDamageAmount())
	{
		return false;
	}

	return true;
}


DEBUGVAR float dbg_fIgnoreDistanceSq = 2.0f * 2.0f;


void AVP_Behaviour_Predator_Melee::UpdateIgnoreTarget(float fTimeslice)
{
	for ( Asura_LinkList_It< IgnoreTargetData > xIgnoreIt( &m_xIgnoreTargets ); !xIgnoreIt.Done(); )
	{
		IgnoreTargetData *pxIgnoreTarget = xIgnoreIt.GetCurrent();

		const AVP_ServerEntity_Actor* pxActor = AVP_ServerEntity_Actor::FindEntity(pxIgnoreTarget->m_uTarget);
		if (!pxActor || pxActor->IsDead() || (pxActor->GetPosition()-pxIgnoreTarget->m_xPosition).MagnitudeSquared() > dbg_fIgnoreDistanceSq)
		{
			xIgnoreIt.DeleteCurrent();
		}
		else
		{
			xIgnoreIt.Next();
		}
	}
}

bool AVP_Behaviour_Predator_Melee::IsTargetIgnored(Asura_Guid uGuid) const
{
	for ( Asura_LinkList_ConstIt< IgnoreTargetData > xIgnoreIt( &m_xIgnoreTargets ); !xIgnoreIt.Done();  xIgnoreIt.Next() )
	{
		const IgnoreTargetData *pxIgnoreTarget = xIgnoreIt.GetCurrent();
		if (pxIgnoreTarget->m_uTarget == uGuid)
		{
			return true;
		}
	}
	return false;
}

DEBUGVAR float dbg_fDistanceForMeleeAttackerSq = 12.0f * 12.0f;
DEBUGVAR float dbg_fDistanceForRangedAttackerSq = 40.0f * 40.0f;

void AVP_Behaviour_Predator_Melee::UpdateNumberOfAttackers()
{
	m_uNumberOfAttackers = 0;
	const AVP_ServerActor_Predator* pxOurActor = GetPredatorBrain()->GetPredatorActor();
	const Asura_Vector_3& xOurPosition = pxOurActor->GetPosition();

	for ( Asura_Hashed_List_ConstIt< Axon_Knowledge > xVisionIt( GetPredatorBrain()->GetKnowledgeSystem()->GetPredatorVision()->GetList() ); !xVisionIt.Done(); xVisionIt.Next() )
	{
		const Axon_Knowledge* pxTargetKnowledge = xVisionIt.GetCurrent();
		const AVP_ServerActor_Vertebrate* pxTarget = pxTargetKnowledge == NULL ? NULL : AVP_ServerActor_Vertebrate::FindEntity( pxTargetKnowledge->GetGuid() );
		if (!pxTarget || pxTarget->IsDead() || pxTargetKnowledge->m_fAge > dbg_fAncientKnowledgeAge )
		{
			continue;		
		}

		bool bIsMeleeOnly = false;

		switch (pxTargetKnowledge->m_uSourceClassification)
		{
		default:
			continue;
		case AVP_ENTITYCLASS_ALIEN:
		case AVP_ENTITYCLASS_ALIENQUEEN:
		case AVP_ENTITYCLASS_ALIENCHESTBURSTER:
		case AVP_ENTITYCLASS_ALIEN_HYBRID:
			bIsMeleeOnly = true;
			break;
		case AVP_ENTITYCLASS_HUMAN:
		case AVP_ENTITYCLASS_ANDROID:
		case AVP_ENTITYCLASS_MARINE:
		case AVP_ENTITYCLASS_CIVILIAN:
		case AVP_ENTITYCLASS_SENTRYGUN:
			break;
		case AVP_ENTITYCLASS_ALIENFACEHUGGER: // ignore hugging facehuggers as they already dead;
			bIsMeleeOnly = true;
			{
				const AVP_ServerActor_FaceHugger* pxTarget = AVP_ServerActor_FaceHugger::FindEntity( pxTargetKnowledge->GetGuid() );
				if ( !pxTarget || pxTarget->IsHugging() )
				{
					continue;
				}
			}
			break;
		}

		float fTargetDistanceSq = (pxTargetKnowledge->m_xPosition - xOurPosition).MagnitudeSquared();


		if ( (bIsMeleeOnly && fTargetDistanceSq < dbg_fDistanceForMeleeAttackerSq ) || 
			(!bIsMeleeOnly && fTargetDistanceSq < dbg_fDistanceForRangedAttackerSq ) )
		{
				++m_uNumberOfAttackers;
		}

	}
	
}

void AVP_Behaviour_Predator_Melee::IgnoreTarget(Asura_Guid uGuid, const Asura_Vector_3& xPosition)
{
	IgnoreTargetData xData;
	xData.m_uTarget = uGuid;
	xData.m_xPosition = xPosition;
	m_xIgnoreTargets.Push_Back(xData);

	if (uGuid == m_uCurrentMeleeTarget)
	{
		SelectCurrentMeleeTarget();
	}
}


bool AVP_Behaviour_Predator_Melee::IsTargetReachable(const Axon_Knowledge* pxTargetKnowledge, float fDistance, u_short& usDestWPID, Asura_Vector_3& xPosition ) const
{

	const AVP_Navigation::TestParams xTargetTestParams(GetPredatorBrain()->GetPathFinderFlagsFromClassification(pxTargetKnowledge->m_uSourceClassification), ASURA_NAVVERTEX_MAX_ZONE_ID);

	usDestWPID = ASURA_INVALID_WAYPOINT_ID;

	if ( !AVP_Navigation::GetNearestWaypointID( pxTargetKnowledge->m_xPosition, usDestWPID, xTargetTestParams ) )
	{
		usDestWPID = ASURA_INVALID_WAYPOINT_ID;
	}

	// if the nearest waypoint is not a waypoint we can get to - i.e. it is not predator walkable - look for a waypoint straight below the head position
	const Asura_AI_NavVertex* pxDestWP = usDestWPID == ASURA_INVALID_WAYPOINT_ID ? NULL : Asura_AI_Navigation_System::GetWaypoint( usDestWPID );

	if ( pxDestWP && (pxDestWP->GetFlags() & AVP_Navigation::AVP_AI_PATHFINDER_FLAG_PREDATOR) != 0)
	{
		xPosition = pxTargetKnowledge->m_xPosition;
		return true;
	}

	AVP_ServerEntity_Actor* pxTarget = AVP_ServerEntity_Actor::FindEntity(pxTargetKnowledge->GetGuid());
	if (!pxTarget)
	{
		return false;
	}	

	Asura_Vector_3 xTargetHead;
	pxTarget->GetHeadPosition(xTargetHead);
	const Asura_Vector_3 xDown(0.0f, 1.0f, 0.0f);

	Asura_LOS_InputData xLosInput(xTargetHead, xDown, ASURA_GUID_UNREGISTERED,  fDistance);
	const Asura_LOS_OutputData* pxLosOutput = Asura_LOS::TestAgainstEnvironment(&xLosInput);

	if (!pxLosOutput->m_bHitEnvironment)
	{
		return false;
	}


	xPosition = xTargetHead + xDown * pxLosOutput->m_fDistanceToIntersection;
	
	const AVP_Navigation::TestParams xMyTestParams(GetPredatorBrain()->GetNavData().m_uPathfinderFlags, ASURA_NAVVERTEX_MAX_ZONE_ID);

	if ( !AVP_Navigation::GetNearestWaypointID( xPosition, usDestWPID, xMyTestParams ) )
	{
		return false;
	}

	return true;
}


DEBUGVAR float dng_fAbortAttackPercentage = 25.0f; 


bool AVP_Behaviour_Predator_Melee::ShouldAbortAttack() const
{

	if (!m_bWasTargetKilled)
	{
		return false;
	}

	const AVP_ServerEntity_Actor* pxActor = GetPredatorBrain()->GetActor();

	float fMaxHealth = pxActor->GetMaxHealth();
	float fCurrentHealth = pxActor->GetHealth();

	if (fCurrentHealth / fMaxHealth > dng_fAbortAttackPercentage / 100.0f || GetNumberOfAttackers() < 2)
	{
		return false;
	}
	return true;
}


DEBUGVAR float dbg_fLargeTurnThreatCos = Asura_Maths::Cosine(Asura_Maths::DegToRad(60.0f));
DEBUGVAR float dbg_fTargetTurnMutiplier = Asura_Maths::Pi * 0.5f; // 0.5PI in a sec
DEBUGVAR float dbg_fMyTurnMutiplier = Asura_Maths::Pi * 2.0f; // 0.5PI in a sec
DEBUGVAR float dbg_fDistanceMutiplier = 3.0f; // move about 3.0f in a sec
DEBUGVAR float dbg_fFlinchMutiplier = 0.5f; 
DEBUGVAR float dbg_fStablizer = 0.2f;
DEBUGVAR float dbg_fOutOfFightTime = 3.0f;


bool AVP_Behaviour_Predator_Melee::SelectCurrentMeleeTarget()
{
	const AVP_ServerActor_Predator* pxOurActor = GetPredatorBrain()->GetPredatorActor();
	const Asura_Vector_3& xOurPosition = pxOurActor->GetPosition();
	Asura_Vector_3 xOurForward;
	pxOurActor->GetHeadFowardVector(xOurForward);

	Asura_Guid uLastTarget = m_uCurrentMeleeTarget;
	m_uCurrentMeleeTarget = ASURA_GUID_UNREGISTERED;

 	bool bIsCurrentTargetOutOfFight = false;
	int iCurrentTargetPriority =  AVP_AI_TargetPriorityData::IGNORE_P;
	float fCurrentTargetAvarageThreat = -Asura_Maths::LargeFloat;
	bool bCurrentTargetOld = true;

	for ( Asura_Hashed_List_ConstIt< Axon_Knowledge > xVisionIt( GetPredatorBrain()->GetKnowledgeSystem()->GetPredatorVision()->GetList() ); !xVisionIt.Done(); xVisionIt.Next() )
	{
		const Axon_Knowledge* pxTargetKnowledge = xVisionIt.GetCurrent();
		const AVP_ServerActor_Vertebrate* pxTarget = pxTargetKnowledge == NULL ? NULL : AVP_ServerActor_Vertebrate::FindEntity( pxTargetKnowledge->GetGuid() );
		if (!pxTarget || pxTarget->IsDead() || IsTargetIgnored(pxTargetKnowledge->GetGuid()) || pxTargetKnowledge->m_fAge > dbg_fAncientKnowledgeAge)
		{
			continue;		
		}

		switch (pxTargetKnowledge->m_uSourceClassification)
		{
		default:
			continue;
		case AVP_ENTITYCLASS_ALIEN:
		case AVP_ENTITYCLASS_ALIENQUEEN:
		case AVP_ENTITYCLASS_ALIENCHESTBURSTER:
		case AVP_ENTITYCLASS_HUMAN:
		case AVP_ENTITYCLASS_ALIEN_HYBRID:
		case AVP_ENTITYCLASS_ANDROID:
		case AVP_ENTITYCLASS_MARINE:
		case AVP_ENTITYCLASS_CIVILIAN:
		case AVP_ENTITYCLASS_SENTRYGUN:
			break;
		case AVP_ENTITYCLASS_ALIENFACEHUGGER: // ignore hugging facehuggers as they already dead;
			{
				const AVP_ServerActor_FaceHugger* pxTarget = AVP_ServerActor_FaceHugger::FindEntity( pxTargetKnowledge->GetGuid() );
				if ( !pxTarget || pxTarget->IsHugging() )
				{
					continue;
				}
			}
			break;
		}

		int iPriority;
		if(!GetPredatorBrain()->CalcTargetingPriority(pxTargetKnowledge->m_uSourceClassification, pxTargetKnowledge->m_uSourceGuid, iPriority))
		{
			iPriority = AVP_AI_TargetPriorityData::NORMAL;
		}

		if (iPriority == AVP_AI_TargetPriorityData::IGNORE_P || iCurrentTargetPriority > iPriority)
		{
			continue;
		}

		float fFlinchTime = pxTarget->GetTimeLeftInFlinching();
		bool bIsTargetOutOfFight = fFlinchTime > dbg_fOutOfFightTime;
		if (!bIsCurrentTargetOutOfFight && bIsTargetOutOfFight)
		{
			continue;
		}

		float fTargetDistance = (pxTargetKnowledge->m_xPosition - xOurPosition).Magnitude();

		const Asura_Vector_3& xMeToTarget = pxTargetKnowledge->m_xDirection;
		const Asura_Vector_3& xTargetDirection = pxTarget->GetAimDirection();
		const Asura_Vector_3& xMyDirection = pxOurActor->GetAimDirection();
		float fTargetTurn = Asura_Maths::ArcCosine(- (xTargetDirection * xMeToTarget));
		float fMyTurn = Asura_Maths::ArcCosine( xMyDirection * xMeToTarget);
		float fAvarageThreat =
			(Asura_Maths::Pi - fTargetTurn) / dbg_fTargetTurnMutiplier  - fTargetDistance / dbg_fDistanceMutiplier - fMyTurn / dbg_fMyTurnMutiplier - fFlinchTime * dbg_fFlinchMutiplier;
		if (pxTargetKnowledge->GetGuid() == uLastTarget)
		{
			fAvarageThreat += dbg_fStablizer;
		}

		bool bTargetOld = pxTargetKnowledge->m_fAge > dbg_fOldKnowledgeAge; 

		if (bTargetOld && !bCurrentTargetOld)
		{
			continue;
		}

		bool bIsCandidate = (iPriority > iCurrentTargetPriority) || 
							(bCurrentTargetOld && !bTargetOld) ||
							(bIsCurrentTargetOutOfFight && !bIsTargetOutOfFight) ||
							(fAvarageThreat > fCurrentTargetAvarageThreat);

		if (bIsCandidate)
		{
			u_short usDummy;
			Asura_Vector_3 xDummy;
			if (!IsTargetReachable(pxTargetKnowledge, dbg_fHightToJumpUp, usDummy, xDummy))
			{
				//IgnoreTarget(pxTargetKnowledge->GetGuid());
				continue;
			}
			m_uCurrentMeleeTarget = pxTargetKnowledge->GetGuid();
			iCurrentTargetPriority = iPriority;
			fCurrentTargetAvarageThreat = fAvarageThreat;
			bIsCurrentTargetOutOfFight = bIsTargetOutOfFight;
			bCurrentTargetOld = bTargetOld;
		}
	}

	if (m_uCurrentMeleeTarget == uLastTarget)
	{
		return false;
	}
	else
	{
		m_uNumberOfAttacksSincePowerAttack = 0; // new target - reset power attack counter
	}

	return true;
}

bool AVP_Behaviour_Predator_Melee::SelectCurrentFollowTarget()
{
	const AVP_ServerActor_Predator* pxOurActor = GetPredatorBrain()->GetPredatorActor();
	const Asura_Vector_3& xOurPosition = pxOurActor->GetPosition();
	Asura_Vector_3 xOurForward;
	pxOurActor->GetHeadFowardVector(xOurForward);

	Asura_Guid uLastTarget = m_uCurrentFollowTarget;
	m_uCurrentFollowTarget = ASURA_GUID_UNREGISTERED;

	int iCurrentTargetPriority =  AVP_AI_TargetPriorityData::IGNORE_P;
	float fCurrentTargetAvarageThreat = -Asura_Maths::LargeFloat;
	bool bCurrentTargetOld = true;

	for ( Asura_Hashed_List_ConstIt< Axon_Knowledge > xVisionIt( GetPredatorBrain()->GetKnowledgeSystem()->GetPredatorVision()->GetList() ); !xVisionIt.Done(); xVisionIt.Next() )
	{
		const Axon_Knowledge* pxTargetKnowledge = xVisionIt.GetCurrent();
		const AVP_ServerActor_Vertebrate* pxTarget = pxTargetKnowledge == NULL ? NULL : AVP_ServerActor_Vertebrate::FindEntity( pxTargetKnowledge->GetGuid() );
		if (!pxTarget || pxTarget->IsDead() || pxTargetKnowledge->m_fAge > dbg_fOldKnowledgeAge )
		{
			continue;		
		}

		switch (pxTargetKnowledge->m_uSourceClassification)
		{
		default:
			continue;
		case AVP_ENTITYCLASS_ALIEN:
		case AVP_ENTITYCLASS_ALIENQUEEN:
		case AVP_ENTITYCLASS_ALIENCHESTBURSTER:
		case AVP_ENTITYCLASS_HUMAN:
		case AVP_ENTITYCLASS_ALIEN_HYBRID:
		case AVP_ENTITYCLASS_ANDROID:
		case AVP_ENTITYCLASS_MARINE:
		case AVP_ENTITYCLASS_CIVILIAN:
		case AVP_ENTITYCLASS_SENTRYGUN:
			break;
		case AVP_ENTITYCLASS_ALIENFACEHUGGER: // ignore hugging facehuggers as they already dead;
			{
				const AVP_ServerActor_FaceHugger* pxTarget = AVP_ServerActor_FaceHugger::FindEntity( pxTargetKnowledge->GetGuid() );
				if ( !pxTarget || pxTarget->IsHugging() )
				{
					continue;
				}
			}
			break;
		}

		int iPriority;
		if(!GetPredatorBrain()->CalcTargetingPriority(pxTargetKnowledge->m_uSourceClassification, pxTargetKnowledge->m_uSourceGuid, iPriority))
		{
			iPriority = AVP_AI_TargetPriorityData::NORMAL;
		}

		if (iPriority == AVP_AI_TargetPriorityData::IGNORE_P || iCurrentTargetPriority > iPriority)
		{
			continue;
		}

		bool bTargetOld = pxTargetKnowledge->m_fAge > dbg_fOldKnowledgeAge; 
		if (bTargetOld && !bCurrentTargetOld)
		{
			continue;
		}

		float fTargetDistance = (pxTargetKnowledge->m_xPosition - xOurPosition).Magnitude();

		const Asura_Vector_3& xMeToTarget = pxTargetKnowledge->m_xDirection;
		const Asura_Vector_3& xTargetDirection = pxTarget->GetAimDirection();
		const Asura_Vector_3& xMyDirection = pxOurActor->GetAimDirection();
		float fTargetTurn = Asura_Maths::ArcCosine(- (xTargetDirection * xMeToTarget));
		float fMyTurn = Asura_Maths::ArcCosine( xMyDirection * xMeToTarget);
		float fAvarageThreat =
			(Asura_Maths::Pi - fTargetTurn) / dbg_fTargetTurnMutiplier  - fTargetDistance / dbg_fDistanceMutiplier - fMyTurn / dbg_fMyTurnMutiplier;
		if (pxTargetKnowledge->GetGuid() == uLastTarget)
		{
			fAvarageThreat += dbg_fStablizer;
		}

		if ( (iPriority > iCurrentTargetPriority) || 
			 (bCurrentTargetOld && !bTargetOld) ||
			 (fAvarageThreat > fCurrentTargetAvarageThreat))
		{
			m_uCurrentFollowTarget = pxTargetKnowledge->GetGuid();
			iCurrentTargetPriority = iPriority;
			fCurrentTargetAvarageThreat = fAvarageThreat;
			bCurrentTargetOld = bTargetOld;

		}
	}

	if (m_uCurrentFollowTarget == uLastTarget)
	{
		return false;
	}
	
	return true;
}

DEBUGVAR float dbg_fTimeBetweenFire = 1.5f;
DEBUGVAR float dbg_fTimeBetweenFireAfterAbort = 0.5f;
DEBUGVAR float dbg_fSafeDistance = 8.0f;
DEBUGVAR float dbg_fSmallDiffCos = Asura_Maths::Cosine(Asura_Maths::DegToRad(5.0f));

void AVP_Behaviour_Predator_Melee::TryToFire(const Asura_Vector_3& xDirection, float fDistance)
{
	if (m_fTimeUntilFire >= 0.0f)
	{
		return;
	}

	const Axon_Knowledge* pxTargetKnowledge = GetCurrentFollowTargetKnowledge();

	Asura_AVector3 xCurrecntAimDirection;
	GetPredatorBrain()->GetClientActor()->GetAimDirection(xCurrecntAimDirection);


	bool bIsInCorrectDirection = xDirection * Asura_Vector_3(xCurrecntAimDirection) > dbg_fSmallDiffCos;

	if(!m_bIsCharging)
	{

		if (GetPredatorBrain()->IsHoldingPlasmaCaster())
		{
			if (pxTargetKnowledge && pxTargetKnowledge->m_fAge == 0.0f && bIsInCorrectDirection && fDistance > dbg_fSafeDistance )
			{
				m_bIsCharging = GetPredatorBrain()->StartPlasmaCasterCharge(GetPredatorBrain()->GetConfigInfo().m_xWeaponConfig.m_fPlasmaCasterDamage);
			}
		}
		else
		{
			GetPredatorBrain()->EquipPlasmaCaster();
		}
	}
	else
	{
		if (GetPredatorBrain()->IsPlasmaCasterCharged())
		{
			m_bIsCharging = false;			
			if (!pxTargetKnowledge || pxTargetKnowledge->m_fAge > 0.0f || !bIsInCorrectDirection || fDistance < dbg_fSafeDistance)
			{
				m_fTimeUntilFire += dbg_fTimeBetweenFireAfterAbort;
				GetPredatorBrain()->AbortPlasmaCasterCharge();
				return;
			}

			Asura_Vector_3 xSafeTarget;
			GetPredatorBrain()->GetActor()->GetMuzzlePosition(xSafeTarget);
			xSafeTarget += Asura_Vector_3(xCurrecntAimDirection) * dbg_fSafeDistance;

			if(!GetPredatorBrain()->CanFireAt(Asura_Vector_3(xSafeTarget), true))
			{
				m_fTimeUntilFire += dbg_fTimeBetweenFireAfterAbort;
				GetPredatorBrain()->AbortPlasmaCasterCharge();
				return;
			}

			m_fTimeUntilFire += dbg_fTimeBetweenFire;
			GetPredatorBrain()->ReleasePlasmaCasterCharge();
		}
	}
}


void AVP_Behaviour_Predator_Melee::UpdateAlert(float fTimeslice)
{
	Axon_Knowledge xKnowledge = GetPredatorBrain()->GetKnowledgeSystem()->GetBestAlertKnowledge();

	if (xKnowledge.GetGuid() != ASURA_GUID_UNREGISTERED)
	{
		m_xAlertPosition = xKnowledge.m_xPosition;
		m_fAlertAge = 0.0f;
	}
	else
	{
		m_fAlertAge += fTimeslice;
	}	
}

void AVP_Behaviour_Predator_Melee::ResetAlert()
{
	m_fAlertAge = dbg_fOldKnowledgeAge + 1.0f;
}


const Axon_Knowledge* AVP_Behaviour_Predator_Melee::GetCurrentMeleeTargetKnowledge() const
{
	return GetTargetKnowledge(m_uCurrentMeleeTarget);
}

const Axon_Knowledge* AVP_Behaviour_Predator_Melee::GetCurrentFollowTargetKnowledge() const
{
	return GetTargetKnowledge(m_uCurrentFollowTarget);
}


const Axon_Knowledge* AVP_Behaviour_Predator_Melee::GetTargetKnowledge(Asura_Guid uGuid) const
{
	return GetPredatorBrain()->GetKnowledgeSystem()->GetPredatorVision()->GetList()->FindEntity(uGuid);
}


//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////
