//////////////////////////////////////////////////////////////////////////////
//
// AVP_Behaviour_Predator_Movement.h
//
// Copyright (C) 2008 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __AVP_BEHAVIOUR_PREDATOR_MOVEMENT_H__
#define __AVP_BEHAVIOUR_PREDATOR_MOVEMENT_H__ 1

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// asura
#include "Core/Asura_TemplateTree.h"

// avp axon
#include "Behaviour/Axon_Project_Behaviour.h"
#include "Core/AVP_Navigation_Types.h"

// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////////

class AVP_Brain_Actor_Predator;

//////////////////////////////////////////////////////////////////////////////
//
// AVP_Behaviour_Predator_Movement
//
//////////////////////////////////////////////////////////////////////////////

class AVP_Behaviour_Predator_Movement : public Axon_Project_Behaviour
{
	typedef Axon_Project_Behaviour PARENT;

public:
	AVP_Behaviour_Predator_Movement( AVP_Brain_Actor_Predator* pxBrain );
	~AVP_Behaviour_Predator_Movement();

	enum ExMovementMode
	{
		EMM_IDLE = -1,
		EMM_SLOW = 0,
		EMM_MEDIUM,
		EMM_FAST,		
	};

	void SetMovementMode(ExMovementMode eMove) { m_eRequestedMoveMode = eMove; }
	ExMovementMode GetMovementMode() const { return m_eRequestedMoveMode; }

	void Stop();

	void AimAt(const Asura_Vector_3& xAimTarget);
	void AimWhereYouMove();

	float GetLengthLeftOnPath() const;

protected:
	virtual bool HandleMessage( const Asura_Message* pxMessage );
	virtual bool Activate();
	virtual void Deactivate();
	virtual void PerFrameUpdate( const float fTimeslice );

#ifndef ASURA_NO_DEBUG_OUTPUT
	virtual void RenderDebugInfo();
#endif

	AVP_Brain_Actor_Predator* GetPredatorBrain() const;

private:

	enum State
	{
		UNSET,
		TRAVERSING_STANDARD_LINK,
		PREPARE_TO_JUMP,
		TRAVERSING_JUMP_LINK,
		TRAVERSING_MOVE_AND_JUMP_LINK,
		TURN_AFTER_JUMP,
		MOVING_TO_DESTINATION,
		RESOLVE_STUCK_BY_JUMPING,
		FINISHED,
		NUM_STATES
	};

	void SwitchToState( const State eNextState );
	void UpdateFollowPoint();
	void UpdatePositionOnRoute();
	void UpdateCurrentWaypoint();
	void UpdateStandardLinkTraverse(float fTimeslice);
	void UpdateMoveToDestination(float fTimeslice);
	//void UpdateTurnToJump(float fTimeslice);
	void UpdateTurnAfterJump(float fTimeslice);
	void UpdateJump(float fTimeslice);
	void UpdatePrepareToJump(float fTimeslice);

	void RequestJump();
	void RequestMoveAndJump();
	bool GetJumpAnimVariation(const Asura_Vector_3& xFromPosition, const Asura_Vector_3& xToPosition, int& iVariation ) const;

	void CalculateAvoidanceSteering( Asura_Vector_3& xSteering ) const;
	Asura_Vector_3 MergeSteering(const Asura_Vector_3& xToDirection, const Asura_Vector_3& xAvoidanceDirection) const;
	void LookToNextWaypoint();

	void FindNextJumpIndex(); 
	u_short FindJumpIndexStartingAt(u_short uStartIndex); 
	void SetDirectionAndSpeed(const Asura_Vector_3& xMoveDirection, const Asura_Vector_3& xAimDirection);

	void UpdateStuckDetection( const float fTimeslice, bool bLargeAvoidanceSteering );
	bool AttemptJumpToUnStuck();

	bool NeedToRestartAfterFalling( float fTimeslice );

	void ClearWPDistanceCache();
	float FindDistanceOfWPFromStart(u_short usWPNumber);
	bool FindNearestPointOnRoute( const AVP_Navigation::NavData& xNavData, const Asura_Vector_3& xCurrentPosition,
								  u_short usBlockIndex, float fLookAheadDistance, Asura_Vector_3& xNearestPosition, u_short& usNearestRouteSegmentIndex );

	ExMovementMode CalcLimitDueToTurn(const Asura_Vector_3& xDesiredDirection) const;
	ExMovementMode CalcPostTurnMovementMode();

	State m_eCurrentState;
	float m_fTimeInState;
	float m_fArrivalRange;
	// Used for detecting if we get stuck.
	// If our follow point stops moving we're obviously making no headway along the route.
	Asura_Vector_3 m_xPrevPositionOnRoute;

	Asura_Vector_3 m_xPositionOnRoute;
	Asura_Vector_3 m_xFollowPoint;
	u_short m_usNextNextValidForJumpIndex; // keep track the next jump WP.
	u_short m_usNextValidForJumpIndex; // keep track the next jump WP.
	Asura_Vector_3 m_xNextJumpPoint;
	u_short m_usLastValidForJumpIndex; // keep track the last jump WP.
	Asura_Hash_ID m_uMoveHash;
	bool m_bStuck;
	Asura_TemplateTree< u_short, float > m_xWPDistanceFromStartCache;
	float m_fRouteTraversedDistance;
	float m_fRouteTotalDistance;

	bool m_bFreeAim;
	Asura_Vector_3 m_xAimTarget;

	ExMovementMode m_eLimitMoveMode;
	ExMovementMode m_eRequestedMoveMode;
	ExMovementMode m_eActualMoveMode;

	float m_fTimeInStuckOnPath;
	float m_fFramesInStuckOnPath;
	float m_fTimeCheckOrientationAndPositionStuck;
	float m_fFramesCheckOrientationAndPositionStuck;
	float m_fMaxRouteTraversedDistance;
	float m_fLastMaxRouteFraction;	
	Asura_Matrix_3x3 m_xLastOrientation;
	Asura_Vector_3 m_xLastPosition;
	bool m_bTryToUnstuckByJumping;
	int m_iNextJumpVariation;
	ExMovementMode m_ePostJumpMoveMode; 
#ifndef ASURA_NO_DEBUG_OUTPUT
	Asura_Vector_3 m_xFollowSteering;
	Asura_Vector_3 m_xAvoidanceSteering;
	Asura_Vector_3 m_xFinalSteering;
#endif

	static const float s_fCollisionAvoidMinimumAngleDeg;	
	static const float s_fCollisionAvoidLocalScalar;
	static const float s_fCollisionAvoidRangePower;
	static const float s_fCollisionAvoidOverallScalar;
};

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

#endif //__AVP_BEHAVIOUR_PREDATOR_MOVEMENT_H__
