//////////////////////////////////////////////////////////////////////////////
//
// AVP_Waypoint_Manager.h
//
// Copyright (C) 2008 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __AVP_WAYPOINT_MANAGER_H__
#define __AVP_WAYPOINT_MANAGER_H__ 1

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// asura
#include "Core/Asura_Maths.h"
#include "Core/Asura_Multithread.h"
#include "Network/Asura_Guid.h"
#include "Entity/PhysicalObject/Asura_ServerEntity_PhysicalObject.h"

// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////////

struct Asura_AI_NavConnection;
class Asura_AI_NavVertex;
class Asura_Chunk_Stream;
class Asura_ClientEntity_DynamicLight;
class Asura_HalfFloat;
template< typename T > class Asura_Hashed_List;
template< typename T > class Asura_List;
class Asura_Message;
class Asura_Projector;
class Asura_ResizingMempool;
class Asura_ServerEntity_DynamicLight;
class AVP_ServerProjectile_HumanFlare;
class AVP_ServerEntity_Container;

//#define AVP_INCLUDE_LOF_DEBUG_INFO (1)

//////////////////////////////////////////////////////////////////////////////
//
// AVP_Waypoint_Manager
//
//////////////////////////////////////////////////////////////////////////////

class AVP_Waypoint_Manager
{
public:
	static void Initialise();
	static void Update();
	static void Reset();
	static void Shutdown();
	static void HandleMessage( const Asura_Message* pxMessage );

	static void CreateConsoleVars();

	static void ReadFromChunkStream( Asura_Chunk_Stream& xChunkStream );

	static float GetWPLuminance( const u_int uWPID );

	static bool GetGS2NodeGuidsFromWPIndex( const u_short usWPIndex, const Asura_Guid*& puNodeGuids, int& iNumNodesGuids );

	static bool  HasCoverFrom( const u_int uBaseWPID, const u_int uThreatWPID );
	static float GetMinLOFValue( const u_int uWaypointID, const Asura_Vector_3& xDirection );
	static float GetMaxLOFValue( const u_int uWaypointID, const Asura_Vector_3& xDirection );
	static void  GetMinLOFDirection( const u_int uWaypointID, Asura_Vector_3& xDirection );
	static void  GetMaxLOFDirection( const u_int uWaypointID, Asura_Vector_3& xDirection );

	// Exrta links data is defined as waypoint blocking per species.
	static void InitialiseLinkExtraData();
	static void ShutdownLinkExtraData();
	static bool IsLinkBlockedForHumans(const Asura_AI_NavConnection* pxLink);
	static bool IsLinkBlockedForAliens(const Asura_AI_NavConnection* pxLink);
	static bool IsLinkBlockedForPredator(const Asura_AI_NavConnection* pxLink);
	static void SetBlockParams(bool bHuman, bool bAlien, bool bPredator);
	static void RestoreBlockParamsDefault();
	static void AvpBlockLinkCallback(u_int uWaypointIndex, u_int uConnectionIndex, const bool bBlock);

	static float GetDefaultMinHideLuminance() { return s_fMinHideLum; }
	static float GetDefaultMaxHideLuminance() { return s_fMaxHideLum; }
	static float GetDefaultMaxHideLuminanceForMarineAI() { return s_fMaxHideLumForMarineAI; }

#ifdef ASURA_TOOLS

	static void VerifyWaypointsForLOF();

	static void WriteToChunkStream( Asura_Chunk_Stream& xChunkStream );

	// Extra waypointing data is defined as waypoint luminance factors, GS2 lookup data, and LOF data.
	static bool ExtraWaypointingDataValid();
	static void DestroyExtraWaypointingData();
	static bool CalculateExtraWaypointingData( bool& bTouchedEntityData );

	static bool CalculateWaypointLuminanceFactors();
	static void ResetUserLuminanceFactors();
// 	static void IncrementLuminanceFactor( const int iWaypointID, const float fTimeslice );
// 	static void DecrementLuminanceFactor( const int iWaypointID, const float fTimeslice );
// 
	// Distance is stored internally in a u_char, so we should only test up to (and just past) 256m.
	static float GetMaxLOFTestDistance() { return 260.0f; }
	static u_int GetNumSectors() { return NUM_SECTORS; }
	static void  GetSectorLimits( const u_int uSector, Asura_Vector_3& xLimitOne, Asura_Vector_3& xLimitTwo );
	static float GetMinLOFValue( const u_int uWaypointID, const u_int uSector );
	static float GetMaxLOFValue( const u_int uWaypointID, const u_int uSector );
#ifdef AVP_INCLUDE_LOF_DEBUG_INFO
	static u_short GetMinLOFWPID( const u_int uWaypointID, const u_int uSector );
	static u_short GetMaxLOFWPID( const u_int uWaypointID, const u_int uSector );
#endif
#endif // ASURA_TOOLS

#if !defined(ASURA_NO_DEBUG_OUTPUT)
	static void RenderDebugInfo();
#endif

private:
#ifdef ASURA_TOOLS
	static void DestroyWaypointLuminanceFactors();
	static void DestroyGS2LookupData();
	static bool CalculateGS2LookupData( bool& bTouchedEntityData );

	static void DestroyLOFLookUpTableData();
	static bool CalculateLOFLookUpTableData();
	static void BuildStaticObjectsLOSList();
	static void DestroyStaticObjectsLOSList();

	static const float EnvironmentAndStaticObjectsLOSTest( const Asura_Vector_3& xOrigin, const Asura_Vector_3& xDirection, float fDistance );
#endif


	static void CollectWPForBoundingBox( const Asura_Bounding_Box& xBB );
	static bool CalculateLuminanceForWaypoint( u_short usWaypointID );
	static bool AddWaypointForLuminanceCalculation( u_short usWaypointID );

	static float GetProjectorLuminance( const Asura_Projector& xProjector, const Asura_AI_NavVertex& xWaypoint );
	static float GetLightLuminance( const Asura_ClientEntity_DynamicLight& xLight, const Asura_AI_NavVertex& xWaypoint );
	static float GetFlareLuminance( const AVP_ServerProjectile_HumanFlare& xFlare, const Asura_AI_NavVertex& xWaypoint );
	static float GetContainerLuminance( const AVP_ServerEntity_Container& xContainerEntity, const Asura_AI_NavVertex& xWaypoint );
	static void SetBBFromLight( const Asura_ServerEntity_DynamicLight& xLight, Asura_Bounding_Box& xBB );
	static void SetBBFromFlare( const AVP_ServerProjectile_HumanFlare& xFlare, Asura_Bounding_Box& xBB );
	static void SetBBFromContainer( const AVP_ServerEntity_Container& xContainerEntity, Asura_Bounding_Box& xBB );
	static u_int GetSectorFromDirection( const Asura_Vector_3& xDirection );
	static u_int GetOppositeSector( const u_int uSector );
	static void CalcDirectionFromSector( const u_int uSector, Asura_Vector_3& xDirection );
	// Task array callback function - tests against all waypoints and sets the polar LOF data.
	static void CalcLOFForWaypointCallback( const void* pData );
private:
	// Luminance stuff.
	static Asura_LinkList< u_short >	 s_xPendingWPIDList;
	static u_char*					 s_pucPendingWPIDMaker;

	static u_int	s_uNumEntries;
	static Asura_HalfFloat*	s_pxWaypointLuminance;
	static u_char*	s_pucWaypointUserFactors;

	static float	s_fMinHideLum;
	static float	s_fMaxHideLum;
	static float    s_fMaxHideLumForMarineAI;

	// LOF Look Up table
	enum { NUM_SECTORS = 8 };
	struct WaypointLOFData
	{
	public:
		void Reset();
#ifdef ASURA_TOOLS
		void SetMinLOFValue( const u_int uSector, const float fDistance );
		void SetMaxLOFValue( const u_int uSector, const float fDistance );
#endif
		float GetMinLOFValue( const u_int uSector ) const;
		float GetMaxLOFValue( const u_int uSector ) const;

#ifdef AVP_INCLUDE_LOF_DEBUG_INFO
		void SetMinWP( const u_int uSector, const u_short usWPID ) { m_usMinWP[ uSector ] = usWPID; }
		void SetMaxWP( const u_int uSector, const u_short usWPID ) { m_usMaxWP[ uSector ] = usWPID; }
		u_short GetMinWP( const u_int uSector ) const { return m_usMinWP[ uSector ]; }
		u_short GetMaxWP( const u_int uSector ) const { return m_usMaxWP[ uSector ]; }
#endif

	private:
		u_char m_ucMinData[ NUM_SECTORS ];
		u_char m_ucMaxData[ NUM_SECTORS ];

#ifdef AVP_INCLUDE_LOF_DEBUG_INFO
		u_short m_usMinWP[ NUM_SECTORS ];
		u_short m_usMaxWP[ NUM_SECTORS ];
#endif

		friend class AVP_Waypoint_Manager;
	};
	static WaypointLOFData* s_pxWaypointLOFData;
	static Asura_Mutex     s_xRequestMutex;

	// GS2 node/waypoint mapping stuff.
	struct WaypointAndNodes
	{
		u_short	    m_usWPID;
		u_short     m_usNumNodes;
		Asura_Guid* m_puGS2NodeGuids;

		// We want to look up the node guid from the waypoint ID.
		Asura_Guid GetGuid() { return static_cast< Asura_Guid >( m_usWPID ); }
		bool	   IsDestroyed() const { return false; }
	};
	static Asura_Hashed_List< WaypointAndNodes > s_xWaypointNodesLookupList;
#ifdef ASURA_TOOLS
	static bool* s_bLOFWaypointShouldWriteToLog;
	static Asura_Guid s_uLastGamePlayVolumeGuid;
#endif

	static const u_int uCURRENT_CHUNK_VERSION = 8;

#if !defined(ASURA_NO_DEBUG_OUTPUT)
	static bool s_bRenderDebugInfo;
#endif

	// extra links data
	class ExtraLinksData
	{
	public:
		ExtraLinksData();
		~ExtraLinksData();

		void Initialise();
		void Reset();

		bool IsLinkBlockedForHumans(const Asura_AI_NavConnection* pxLink) const;
		bool IsLinkBlockedForAliens(const Asura_AI_NavConnection* pxLink) const;
		bool IsLinkBlockedForPredator(const Asura_AI_NavConnection* pxLink) const;

		// callback part
		void operator()(u_int uWaypointIndex, u_int uConnectionIndex, const bool bBlock);
		void SetBlockParams(bool bHuman, bool bAlien, bool bPredator);
		void RestoreDefaultBlockParams();

	private:

		void BlockLink(u_int uLinkIndex);
		void UnblockLink(u_int uLinkIndex);
		u_int GetLinkIndex(u_int uWapointID, u_int uLinkNumber) const;
		u_int GetLinkIndex(const Asura_AI_NavConnection* pxLink) const;

		enum 
		{
			BLOCK_HUMAN_FLAG = 0x01,
			BLOCK_ALIEN_FLAG = 0x02,
			BLOCK_PREDATOR_FLAG = 0x04
		};

		struct LinkBlockDescriptor // Link index is used for hashing
		{			
			u_char m_ucHumanBlockCount;
			u_char m_ucAlienBlockCount;
			u_char m_ucPredatorBlockCount;
		};

		typedef Asura_TemplateHashTree< LinkBlockDescriptor > HashTree;
		typedef Asura_TemplateHashTree_Iterator< LinkBlockDescriptor > HashTreeIter;

		HashTree m_xLinkDescriptors;
		u_int m_uNumLinks;
		u_char *m_aucLinksBlockFlags;

		// call back configuration part
		bool m_bHuman;
		bool m_bAlien;
		bool m_bPredator;


	};

	static ExtraLinksData s_xExtraLinkData;
};

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

#endif //__AVP_WAYPOINT_MANAGER_H__
