///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_Brain_Actor_AlienQueen.h
// Author : David Wooldridge
// Date   : 11/11/2008
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __AVP_BRAIN_NPC_ALIENQUEEN_H__
#define __AVP_BRAIN_NPC_ALIENQUEEN_H__ 1


// asura
#include "Core/Asura_LinkList.h"

// axon
#include "Behaviour/BehaviourTypes/Alien/AVP_Behaviour_Alien_Movement.h"
#include "Core/AxonAVP_Messages.h"
#include "Core/BrainTypes/AVP_Brain_Actor.h"
#include "Knowledge/AVP_Knowledge_System_AlienQueen.h"

// avp
#include "Entity/Controller/AVP_Controller_Messages.h"

class Asura_Blueprint;
class AVP_ServerActor_AlienQueen;
class AVP_Knowledge_System_AlienQueen;


enum AVP_AlienQueenRange
{
	AVP_AR_NEAR,
	AVP_AR_NEARISH,
	AVP_AR_OK,
	AVP_AR_FARISH,
	AVP_AR_FAR,

	NUM_AVP_ALIEN_RANGES
};

enum AVP_AlienQueen_AttackType
{
	AVP_AQ_ATTACK_CHARGE,
	AVP_AQ_ATTACK_CLAW_SWIPE,
	AVP_AQ_ATTACK_SPIT_ACID,
	
	NUM_AVP_AQ_ATTACKS
};

enum AVP_AlienQueen_LookAtType
{
	AVP_AQ_LOOK_AT_TYPE_NONE,
	AVP_AQ_LOOK_AT_TYPE_PLAYER,
	AVP_AQ_LOOK_AT_TYPE_ACTOR,
	AVP_AQ_LOOK_AT_TYPE_NODE,

	AVP_AQ_MAX_LOOK_AT_TYPES
};

class AVP_Brain_Actor_AlienQueen : public AVP_Brain_Actor
{
	typedef AVP_Brain_Actor PARENT;

public:
								AVP_Brain_Actor_AlienQueen( AVP_ServerActor_AlienQueen* pxAlienQueen );
	virtual						~AVP_Brain_Actor_AlienQueen();

	virtual void				Initialise	(void);
	virtual void				Shutdown	(void);
	virtual void				InitialiseFromBlueprint( const Asura_Blueprint& xBlueprint );
	virtual void				PostLoadInitialise();
	virtual void				DoPerFrameUpdate( const float fTimeslice );

#ifndef ASURA_NO_DEBUG_OUTPUT
	virtual void				DoRenderDebugInfo();
#endif

	virtual void				ReadFromChunkStream( Asura_Chunk_Stream& xStream );
	virtual void				WriteToChunkStream( Asura_Chunk_Stream& xStream ) const;

	virtual Axon_Behaviour*		CreateBehaviour( const u_int uBehaviourType, const Asura_Message* pxMessage = NULL );

	virtual Asura_Guid			GetPrimaryThreatGuid( void );
	virtual Asura_Guid			GetAlertGuid( void );

	virtual void				EstablishNavData();

	void						SetMovementMode(AVP_Navigation::MovementMode uMode);

	void						SetNextAttackType(const AVP_AlienQueen_AttackType eAttackType) { m_eAttackType = eAttackType; }
	AVP_AlienQueen_AttackType	GetNextAttackType() const { return m_eAttackType; }

	static u_int				GetLargestBehaviourSize();

	AVP_ServerActor_AlienQueen* GetQueenActor() const;

	Axon_Knowledge*				GetTargetKnowledge()	{return (m_pxKnowledgeSystem) ? m_pxKnowledgeSystem->GetTargetKnowledge() : NULL;}

	bool						HaveLOSToCurrentTarget();

	const AVP_AlienQueenBehaviourData* GetBehaviourData() const {return &m_xBehaviourData;}

	Asura_Guid					GetCurrentTargetGuid() const { return m_uCurrentTargetGuid; }
	bool						HasCurrentTarget() const { return m_uCurrentTargetGuid != ASURA_GUID_UNREGISTERED; }

	float						GetHealth() const { return m_fHealthLastFrame; }

	bool						CanSpitAcid() const { return m_bCanSpitAcid; }
	bool						CanPlanMove() const;

	float						GetSpitAcidMinRange() const { return m_fSpitAcidMinRange; }
	float						GetSpitAcidMaxRange() const { return m_fSpitAcidMaxRange; }

	float						GetSlashAttackMinRange() const { return m_fMinSlashAttackDistance; }

	float						GetMeleeChargeMinDistance() const { return m_fMeleeChargeMinDistance; }
	float						GetMeleeChargeMaxDistance() const { return m_fMeleeChargeMaxDistance; }

	void						SetInGameScene( bool bInGS2 ); 
	bool						IsInGameScene() { return m_bIsGS2; }

	Asura_Guid					GetLookAtGuid() const { return m_uLookAtGuid; }
	void						LookAtEntityFromMsgData();
	void						LookAndAimForward();

	void						AimAtPosition( const Asura_Vector_3& xPosition );

	float						GetMaxHeadTurnCosine() const { return m_fMaxHeadTurn_Cosine; }

protected:
	virtual bool HandleMessage( const Asura_Message* pxMessage );

	virtual bool RegisterWithTeam();
	virtual bool UnregisterWithTeam();
	virtual void RegisterCallbacks();
	virtual void UnregisterCallbacks();

	void DoKnowledgeUpdate( const float fTimeslice );


#ifndef ASURA_NO_DEBUG_OUTPUT
	virtual bool Verify();
#endif


private:
	Asura_Guid m_uCurrentTargetGuid;

	Asura_Handle					m_iKnowledgeUpdateHandle;
	AVP_Knowledge_System_AlienQueen* m_pxKnowledgeSystem;

	AVP_AlienQueen_AttackType		m_eAttackType;

	u_int	m_uLOSToCurrentTargetTestFrame;
	bool	m_bHaveLOSToCurrentTarget;

	bool	m_bIsGS2;
	bool	m_bCanSpitAcid;
	
	float	m_fSpitAcidMinRange;
	float	m_fSpitAcidMaxRange;

	float	m_fMeleeChargeMinDistance;
	float	m_fMeleeChargeMaxDistance;
	
	float	m_fMinSlashAttackDistance;
	
	float	m_fHealthLastFrame;

	Asura_Guid	m_uLookAtGuid;
	u_int		m_uLookAtType;

	float	m_fMaxHeadTurn_Cosine;

	AVP_AlienQueenBehaviourData m_xBehaviourData;

	enum { CURRENT_CHUNK_VERSION = 2 };

};


#endif