//-----------------------------------------------------------------------------
//	File		: AVP_Brain_Actor_Civilian.h
//	Created		: 12 / 02 / 2008
//	Author		: Gareth Thomas
//	Copyright (C) 2007 Rebellion. All Rights Reserved.
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
//	BEGIN FILE WRAPPER
#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif
#ifndef __AVP_BRAIN_ACTOR_CIVILIAN_H__
#define __AVP_BRAIN_ACTOR_CIVILIAN_H__	1
//	END FILE WRAPPER
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
//	INCLUDES
//-----------------------------------------------------------------------------
#include "Core/BrainTypes/AVP_Brain_Actor.h"

//-----------------------------------------------------------------------------
// FORWARD DECLARATIONS
//-----------------------------------------------------------------------------
class AVP_ServerActor_Civilian;
class AVP_Behaviour_Civilian;
class AVP_Tactical_System;

//////////////////////////////////////////////////////////////////////////////
//
// AVP_Brain_Actor_Civilian
//
//////////////////////////////////////////////////////////////////////////////
class AVP_Brain_Actor_Civilian : public AVP_Brain_Actor
{
	typedef AVP_Brain_Actor PARENT;

public:

	AVP_Brain_Actor_Civilian(AVP_ServerActor_Civilian* pxHuman);

	// Public Member Virtual Functions...
	virtual void					ReadFromChunkStream					(Asura_Chunk_Stream& xStream);
	virtual void					WriteToChunkStream					(Asura_Chunk_Stream& xStream) const;
	virtual void					Initialise							(void);
	virtual void					Shutdown							(void);
	virtual void					InitialiseFromBlueprint				(const Asura_Blueprint& xBlueprint);
	virtual Axon_Behaviour* 		CreateBehaviour						(const u_int uBehaviourID, const Asura_Message* pxMessage = NULL);
	virtual void					PostLoadInitialise					(void);

	virtual Asura_Guid				GetPrimaryThreatGuid( void );
	virtual Asura_Guid				GetAlertGuid( void );

	// Public Member Functions...
	AVP_ServerActor_Civilian*		GetCivilianActor					(void) const;
	AVP_Knowledge_System_Actor*		GetKnowledgeSystem					(void) const		{return m_pxKnowledgeSystem;}
	AVP_Tactical_System*			GetTacticalSystem					(void) const		{return m_pxTacticalSystem;}
	void							UpdateKnowledge						(const float fTimeslice);
	bool							DoesHaveThreat						(void);
	bool							DoesHaveAlert						(void);
	u_int							GetCurrentWeaponType				(void);

	void							AimAtPosition( const Asura_Vector_3& xPosition );

	// Public Statics
	static u_int					GetLargestBehaviourSize				(void);
	void							GetThreatPos						( Asura_Vector_3& xPosition );
	
protected:
	virtual void 					RegisterCallbacks					(void);
	virtual void 					UnregisterCallbacks					(void);
	virtual bool 					HandleMessage						(const Asura_Message* pxMessage);
	virtual float					GetNearbyRange						() const;

#ifndef ASURA_NO_DEBUG_OUTPUT
	virtual bool Verify();
#endif

	// Protected Member functions...
	bool							IsCivilianBehaviour					(const u_int uBehaviourType);
	Axon_Behaviour*					CreateCivilianBehaviour				(const u_int uBehaviourType);
	bool							CreateDefaultBehaviour				(void);
	void							CleanUpBehaviours					(void);

	bool	IsTorchOn(void) const;
	void	TorchOn(void);
	void	TorchOff(void);



private:

	// Private member Functions
	void							DoKnowledgeUpdate					(const float fTimeslice)		{UpdateKnowledge(fTimeslice);}

	// Note: Parent function is public
	virtual void					EstablishNavData					(void);

	Asura_Handle                    m_iKnowledgeUpdateHandle;
	AVP_Knowledge_System_Actor*		m_pxKnowledgeSystem;
	AVP_Tactical_System*			m_pxTacticalSystem;

	enum { CURRENT_CHUNK_VERSION = 0 };
};

#endif //__AVP_BRAIN_ACTOR_CIVILIAN_H__

/// end of file 'AVP_Brain_Actor_Civilian.h'
