//////////////////////////////////////////////////////////////////////////////
//
// AVP_Brain_Actor_Dropship.cpp
//
// Copyright (C) 2007 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"

// header
#include "Core/BrainTypes/AVP_Brain_Actor_Dropship.h"

// asura
#include "Entity/Locations/OrientedNode/GamesceneNode/Asura_ServerNode_GS2Node.h"
#include "Entity/Locations/Spline/GamesceneSpline/Asura_ServerNode_GS2Spline.h"


// Axon headers
#include "Behaviour/BehaviourTypes/AVP_Behaviour_Types.h"
#include "Behaviour/BehaviourTypes/Dropship/AVP_Behaviour_Dropship.h"
#include "Behaviour/BehaviourTypes/Dropship/AVP_Behaviour_Dropship_FireMissiles.h"
#include "Behaviour/BehaviourTypes/Dropship/AVP_Behaviour_Dropship_Movement.h"

// AVP
#include "Debugging/Asura_DebugLineRender.h"

#include "Entity/PhysicalObject/Actor/Vehicles/Dropship/AVP_ClientVehicle_Dropship.h"
#include "Entity/PhysicalObject/Actor/Vehicles/Dropship/AVP_ServerVehicle_Dropship.h"
#include "Inventory/AVP_Inventory_Group.h"
#include "Inventory/AVP_Inventory_HashIDs.h"
#include "Inventory/AVP_Inventory_Item.h"
#include "Inventory/AVP_Inventory_System.h"
#include "Inventory/Items/Weapons/AVP_InventoryWeapon_DropshipMissile.h"


//////////////////////////////////////////////////////////////////////////////
// Constants
//////////////////////////////////////////////////////////////////////////////
static const Asura_Hash_ID s_uLauncher0HashID = uAVP_INVENTORY_WEAPONHASH_WEAPON01;	// Weapon01
//static const Asura_Hash_ID s_uLauncher1HashID = s_uLauncher0HashID + 1;	// Weapon02
//static const Asura_Hash_ID s_uLauncher2HashID = s_uLauncher1HashID + 1;	// Weapon02
//static const Asura_Hash_ID s_uLauncher3HashID = s_uLauncher2HashID + 1;	// Weapon03
//static const Asura_Hash_ID s_uMachineGunHashID = s_uLauncher3HashID + 1; // Weapon04

//////////////////////////////////////////////////////////////////////////////
//
// AVP_Brain_Actor_Dropship
//
//////////////////////////////////////////////////////////////////////////////

AVP_Brain_Actor_Dropship::AVP_Brain_Actor_Dropship( AVP_ServerVehicle_Dropship* pxDropship )
: PARENT( pxDropship )
, m_uEndSplineNodeID(ASURA_GUID_UNREGISTERED)
, m_uTargetID(ASURA_GUID_UNREGISTERED)
, m_fTimeSinceTargetSelected(Asura_Maths::LargeFloat)
, m_eDesiredWeaponType(WEAPON_LAUNCHER)
, m_uLauncherNumber(0)
, m_bWeaponStatusNeedUpdate(false)
, m_bUseMissiles(false)
, m_bUseMachinegun(false)
, m_xMissDelta(Asura_Maths::ZeroVector3)
, m_bDoesMiss(false)
{
}

void AVP_Brain_Actor_Dropship::Initialise()
{
	PARENT::Initialise();
}

void AVP_Brain_Actor_Dropship::Shutdown()
{
	PARENT::Shutdown();
}

void AVP_Brain_Actor_Dropship::InitialiseFromBlueprint( const Asura_Blueprint& xBlueprint )
{
    PARENT::InitialiseFromBlueprint( xBlueprint );

	Asura_Value xValue;

	DestroyAllBehaviours();
	if ( IsActive() )
	{
		// Create our initial behaviours based on our blueprint and any relevant information in our actor.
		bool bCreatedFromBP = false;
		if ( xBlueprint.GetParameterValueByHashID( 0xae7a48c3, xValue ) ) // Asura_GetHashID( "AIStartBehaviour" )
		{
			if ( xValue.GetIntValue() >= AXON_BEHAVIOUR_TYPE_FIRST_PROJECT_SPECIFIC )
			{
				CreateActivateAndPushBehaviour( xValue.GetIntValue() );
				bCreatedFromBP = true;
			}
		}
		if ( !bCreatedFromBP )
		{
			CreateActivateAndPushBehaviour( AVP_BEHAVIOUR_TYPE_DROPSHIP );
		}
		CreateGS2BehaviourIfNecessary();
	}

	EstablishNavData();
}

void AVP_Brain_Actor_Dropship::ReadFromChunkStream( Asura_Chunk_Stream& xStream )
{
    int iVersion;
    xStream >> iVersion;

    PARENT::ReadFromChunkStream( xStream );

    if ( iVersion == 0 )
    {
    }
    else
    {
        Asura_Assertf( false, ( "AVP_Brain_Actor_Dropship::ReadFromChunkStream - unhandled chunk version (%d)", iVersion ) );
    }
	
}

void AVP_Brain_Actor_Dropship::PostLoadInitialise()
{
	PARENT::PostLoadInitialise();
	m_uEndSplineNodeID = GetDropshipActor()->GetStartNode();
}


void AVP_Brain_Actor_Dropship::WriteToChunkStream( Asura_Chunk_Stream& xStream ) const
{
    xStream << CURRENT_CHUNK_VERSION;

    PARENT::WriteToChunkStream( xStream );
}


void AVP_Brain_Actor_Dropship::DoPerFrameUpdate( const float fTimeslice )
{
	PARENT::DoPerFrameUpdate( fTimeslice );
	FindTarget();
	UpdateWeaponState();
}


#include "Core/Asura_MemoryManagement_DebugOff.h"
Axon_Behaviour* AVP_Brain_Actor_Dropship::CreateBehaviour( const u_int uBehaviourType, const Asura_Message* pxMessage /*= NULL*/ )
{
    void* pxMemory = s_pxBehaviourMempool->Allocate();
    if ( !pxMemory )
    {
        Asura_Assert( false, "Axon_Project_Brain::CreateBehaviour - could not allocate behaviour memory" );
        return NULL;
    }

	enum { // there was a problem in the avp behaviour types enum - added this here to fix.
		AVP_BEHAVIOUR_TYPE_DROPSHIP_BUG = 4011,
		AVP_BEHAVIOUR_TYPE_DROPSHIP_MOVEMENT_BUG = 4012,
		AVP_BEHAVIOUR_TYPE_DROPSHIP_FIRE_MISSILES_BUG = 4013,
	};

    Axon_Behaviour* pxBehaviour = NULL;
    switch ( uBehaviourType )
    {
        case AVP_BEHAVIOUR_TYPE_DROPSHIP:
		case AVP_BEHAVIOUR_TYPE_DROPSHIP_BUG:
        {
            pxBehaviour = new ( pxMemory ) AVP_Behaviour_Dropship( this );
            break;
        }

		case AVP_BEHAVIOUR_TYPE_DROPSHIP_MOVEMENT:
		case AVP_BEHAVIOUR_TYPE_DROPSHIP_MOVEMENT_BUG:
			{
				pxBehaviour = new ( pxMemory ) AVP_Behaviour_Dropship_Movement( this );
				break;
			}
		
		case AVP_BEHAVIOUR_TYPE_DROPSHIP_FIRE_MISSILES:
		case AVP_BEHAVIOUR_TYPE_DROPSHIP_FIRE_MISSILES_BUG:
			{
				pxBehaviour = new ( pxMemory ) AVP_Behaviour_Dropship_FireMissiles( this );
				break;
			}


		default:
        {
            break;
        }
    }

    if ( !pxBehaviour )
    {
        // Couldn't create the behaviour - try the parent.
        s_pxBehaviourMempool->Deallocate( pxMemory );
        pxBehaviour = PARENT::CreateBehaviour( uBehaviourType, pxMessage );
    }
    else if ( pxMessage )
    {
        pxBehaviour->DoHandleMessage( pxMessage, true );
    }

    return pxBehaviour;
}
#include "Core/Asura_MemoryManagement_DebugOn.h"

Asura_Guid AVP_Brain_Actor_Dropship::GetPrimaryThreatGuid()
{
	return ASURA_GUID_UNREGISTERED;
}

Asura_Guid AVP_Brain_Actor_Dropship::GetAlertGuid( void )
{
	return ASURA_GUID_UNREGISTERED;
}



AVP_ServerVehicle_Dropship* AVP_Brain_Actor_Dropship::GetDropshipActor() const
{
    return static_cast< AVP_ServerVehicle_Dropship* >( GetActor() );
}

AVP_ClientVehicle_Dropship* AVP_Brain_Actor_Dropship::GetDropshipClientActor() const
{
		return static_cast< AVP_ClientVehicle_Dropship* >( GetClientActor() );
}


void AVP_Brain_Actor_Dropship::EquipMissile(u_int uLauncherNumber)
{
	if (m_eDesiredWeaponType != WEAPON_LAUNCHER || m_uLauncherNumber != uLauncherNumber)
	{
		m_eDesiredWeaponType = WEAPON_LAUNCHER;
		m_uLauncherNumber = uLauncherNumber;
		m_bWeaponStatusNeedUpdate = true;
		UpdateWeaponState();
	}

}

bool AVP_Brain_Actor_Dropship::IsMissileReady(u_int uLauncherNumber) const
{
	AVP_Inventory_System* pxInventorySystem	= GetDropshipActor()->GetInventorySystem();
	AVP_Inventory_Group*  pxWeaponGroup = pxInventorySystem ? pxInventorySystem->GetGroup( uAVP_INVENTORY_GROUPHASH_WEAPON ) : NULL;
	Asura_Assert(pxWeaponGroup, "Weapon group not found");

	return pxWeaponGroup ? pxWeaponGroup->GetCurrentItem() == s_uLauncher0HashID + uLauncherNumber : false ;
}


bool AVP_Brain_Actor_Dropship::FireMissile()
{
	AVP_ServerVehicle_Dropship* pxActor = GetDropshipActor();
	AVP_Inventory_System* pxInventorySystem	= pxActor ? pxActor->GetInventorySystem() : NULL;
	AVP_Inventory_Group*  pxWeaponGroup	= pxInventorySystem ? pxInventorySystem->GetGroup( uAVP_INVENTORY_GROUPHASH_WEAPON ) : NULL;
	Asura_Assert(pxWeaponGroup, "Weapon group not found");
	if (!pxWeaponGroup) { return false; }
	Asura_Assert(pxWeaponGroup->GetCurrentItem() == s_uLauncher0HashID + m_uLauncherNumber , "Trying to launch a missile but launcher not equipped");

	AVP_InventoryWeapon_DropshipMissile* pxWeaponItem	= 
		static_cast<AVP_InventoryWeapon_DropshipMissile*>((pxInventorySystem == NULL) ? NULL : pxInventorySystem->GetItem(s_uLauncher0HashID + m_uLauncherNumber));
	Asura_Assert(pxWeaponItem, "Weapon not found");
	if (!pxWeaponItem) { return false; }

	return pxWeaponItem->RequestBeginUsage( AVP_InventoryWeapon_RangedWeapon::UT_N_BULLET_BURST_PRIMARY);
}

bool AVP_Brain_Actor_Dropship::AreFrontPodsOpened() const
{
	return GetDropshipActor()->AreFrontPodsOpened();
}

bool AVP_Brain_Actor_Dropship::AreFrontPodsClosed() const
{
	return GetDropshipActor()->AreFrontPodsClosed();
}

bool AVP_Brain_Actor_Dropship::AreBackPodsOpened() const
{
	return GetDropshipActor()->AreBackPodsOpened();
}

bool AVP_Brain_Actor_Dropship::AreBackPodsClosed() const
{
	return GetDropshipActor()->AreBackPodsClosed();
}

bool AVP_Brain_Actor_Dropship::AreBackPodsDoorsOpened() const
{
	return GetDropshipActor()->AreBackPodsDoorsOpened();
}

bool AVP_Brain_Actor_Dropship::AreBackPodsDoorsClosed() const
{
	return GetDropshipActor()->AreBackPodsDoorsClosed();
}

void AVP_Brain_Actor_Dropship::OpenFrontPods()
{
	GetDropshipActor()->OpenFrontPods();
}

void AVP_Brain_Actor_Dropship::CloseFrontPods()
{
	GetDropshipActor()->CloseFrontPods();
}

void AVP_Brain_Actor_Dropship::OpenBackPods()
{
	GetDropshipActor()->OpenBackPods();
}

void AVP_Brain_Actor_Dropship::CloseBackPods()
{
	GetDropshipActor()->CloseBackPods();
}

void AVP_Brain_Actor_Dropship::OpenBackPodsDoors()
{
	GetDropshipActor()->OpenBackPodsDoors();
}

void AVP_Brain_Actor_Dropship::CloseBackPodsDoors()
{
	GetDropshipActor()->CloseBackPodsDoors();
}

u_int AVP_Brain_Actor_Dropship::GetLargestBehaviourSize()
{
	u_int uLargestBehaviourSize = 0;

	AXON_CALC_MAX_BEHAVIOUR_SIZE( uLargestBehaviourSize, AVP_Behaviour_Dropship );
	AXON_CALC_MAX_BEHAVIOUR_SIZE( uLargestBehaviourSize, AVP_Behaviour_Dropship_Movement );
	AXON_CALC_MAX_BEHAVIOUR_SIZE( uLargestBehaviourSize, AVP_Behaviour_Dropship_FireMissiles );
	
	return uLargestBehaviourSize;
}

bool AVP_Brain_Actor_Dropship::HandleMessage( const Asura_Message* pxMessage )
{
	return PARENT::HandleMessage( pxMessage );
}

#ifndef ASURA_NO_DEBUG_OUTPUT
bool AVP_Brain_Actor_Dropship::Verify()
{
	return true;
}
#endif

void AVP_Brain_Actor_Dropship::EstablishNavData()
{
	// Dropship does not use navdata
}


float AVP_Brain_Actor_Dropship::ExpandPath(Asura_LinkList<Asura_Vector_3>& xPathSegments)
{
	const Asura_ServerNode_GS2Node* pxSplineStart = Asura_ServerNode_GS2Node::FindEntity(m_uEndSplineNodeID);
	Asura_Assert(pxSplineStart, "Spline node not found");

	if (!pxSplineStart)
	{
		return -1.0f;
	}


	u_int uNumberOfLinks = pxSplineStart->GetNumberLinks();
	u_int uNumberOfActiveLinks = 0;
	for (u_int u = 0; u< uNumberOfLinks; ++u)
	{
		const Asura_ServerNode_GS2Spline* pxSpline = Asura_ServerNode_GS2Spline::FindEntity( pxSplineStart->GetLink( u ) );
		if (pxSpline->IsAvailable())
		{
			++uNumberOfActiveLinks;
		}
	}
	Asura_Assert(uNumberOfActiveLinks > 0, "Spline hit a dead end");
	if (uNumberOfActiveLinks == 0)
	{
		return -1.0f;
	}

	u_int uSelectedLink = Asura_Random::GetIntU(0, uNumberOfActiveLinks-1);

	const Asura_ServerNode_GS2Spline* pxSelectedSpline = NULL;

	for (u_int u = 0; u< uNumberOfLinks && !pxSelectedSpline; ++u)
	{
		const Asura_ServerNode_GS2Spline* pxSpline = Asura_ServerNode_GS2Spline::FindEntity( pxSplineStart->GetLink( u ) );
		if (pxSpline->IsAvailable())
		{
			if (uSelectedLink == 0)
			{
				pxSelectedSpline = pxSpline;
			}
			else
			{
				--uSelectedLink;
			}

		}
	}

	Asura_Assert(pxSelectedSpline , "Spline selection bug");

	if (!pxSelectedSpline)
	{
		return -1.0f;
	}

	float fTotalDistanceAdded(0.0f);
	Asura_Vector_3 xLastPosition;
	if (xPathSegments.Back())
	{
		xLastPosition = xPathSegments.Back()->Data();
	}
	else
	{
		pxSelectedSpline->GetPositionFromProportion(0.0, xLastPosition);
		xPathSegments.Push_Back(xLastPosition);
	}

	// push 4 poisitions to approximate the spline - .25, .50, .75, 1.0 - position 0.0 is assumed to already be there from previous expansions
	Asura_Vector_3 xPosition;
	pxSelectedSpline->GetPositionFromProportion(0.25, xPosition);
	xPathSegments.Push_Back(xPosition);
	fTotalDistanceAdded += (xPosition - xLastPosition).Magnitude();
	xLastPosition = xPosition;

	pxSelectedSpline->GetPositionFromProportion(0.5, xPosition);
	xPathSegments.Push_Back(xPosition);
	fTotalDistanceAdded += (xPosition - xLastPosition).Magnitude();
	xLastPosition = xPosition;

	pxSelectedSpline->GetPositionFromProportion(0.75, xPosition);
	xPathSegments.Push_Back(xPosition);
	fTotalDistanceAdded += (xPosition - xLastPosition).Magnitude();
	xLastPosition = xPosition;

	pxSelectedSpline->GetPositionFromProportion(1.0, xPosition);
	xPathSegments.Push_Back(xPosition);
	fTotalDistanceAdded += (xPosition - xLastPosition).Magnitude();
	xLastPosition = xPosition;

	m_uEndSplineNodeID = pxSelectedSpline->GetTargetGuid();

	return fTotalDistanceAdded;
}


bool AVP_Brain_Actor_Dropship::GetTarget(Asura_Vector_3& xTarget)
{
	if(m_uTargetID == ASURA_GUID_UNREGISTERED)
	{
		return false;
	}
	else
	{
		const AVP_ServerEntity_Actor* pxTarget = AVP_ServerEntity_Actor::FindEntity(m_uTargetID);
		if (pxTarget && !pxTarget->IsDead())
		{
			xTarget = pxTarget->GetPosition() + m_xMissDelta;
			return true;
		}
		else
		{
			m_uTargetID = ASURA_GUID_UNREGISTERED;
			return false;
		}
	}
}

DEBUGVAR float dbg_fMissDeltaDistance = 10.f;
DEBUGVAR float dbg_fMaxMissRotation = Asura_Maths::DegToRad(60.0f);

void AVP_Brain_Actor_Dropship::RefreshMissValue()
{
	m_xMissDelta = Asura_Maths::ZeroAVector3;
	if (!m_bDoesMiss || m_uTargetID == ASURA_GUID_UNREGISTERED)
	{
		return;
	}
	else
	{
		const AVP_ServerEntity_Actor* pxTarget = AVP_ServerEntity_Actor::FindEntity(m_uTargetID);
		if (!pxTarget || pxTarget->IsDead())
		{
			return;
		}			

		Asura_Vector_3 xTargetPosition = pxTarget->GetPosition();
		Asura_Vector_3 xMyPosition = GetActor()->GetPosition();

		Asura_Vector_3 xDirToMeOnPlane = xMyPosition - xTargetPosition;
		xDirToMeOnPlane.y = 0.0f;
		if (xDirToMeOnPlane.ApproximatelyEquals(Asura_Maths::ZeroVector3))
		{
			xDirToMeOnPlane.x = Asura_Random::GetFloat(-1.0f, 1.0f);
			xDirToMeOnPlane.z = Asura_Random::GetFloat(-1.0f, 1.0f);
		}
		xDirToMeOnPlane.Normalise();
		Asura_Quat xTuran;
		xTuran.FromRotation(Asura_Random::GetFloat(-dbg_fMaxMissRotation, dbg_fMaxMissRotation), Asura_Vector_3(0.0f, -1.0f, 0.0f));
		xDirToMeOnPlane = xTuran.RotateVector(xDirToMeOnPlane);

		m_xMissDelta = xDirToMeOnPlane * dbg_fMissDeltaDistance;
	}
}

DEBUGVAR float dbg_fMinDistanceToAquireSq = 150.0f * 150.0f;
DEBUGVAR float dbg_fMinTimeToSwapTargets = 3.0f;
DEBUGVAR float dbg_fMinTimeToLook = 0.1f;

void AVP_Brain_Actor_Dropship::FindTarget()
{
	const Asura_Vector_3 xOurPosition = GetActor()->GetPosition();
	m_fTimeSinceTargetSelected += Asura_Timers::GetGameTimeScale();

	// dont look every frame
	if (m_fTimeSinceTargetSelected < dbg_fMinTimeToLook)
	{
		return;
	}


	// if we have a target and its close enough and "new" enough just keep it as our target.
	Asura_Vector_3 xCurrentTargetPosition;
	if (GetTarget(xCurrentTargetPosition) && 
		(xCurrentTargetPosition-xOurPosition).MagnitudeSquared() < dbg_fMinDistanceToAquireSq &&
		m_fTimeSinceTargetSelected < dbg_fMinTimeToSwapTargets)
	{
		return;
	}
	
	// no target or target got out of range - try to pick a new target.
	float fClosestDistanceSq = Asura_Maths::LargeFloat;
	int iBestPriority = AVP_AI_TargetPriorityData::IGNORE_P;

	m_uTargetID = ASURA_GUID_UNREGISTERED;
	m_fTimeSinceTargetSelected = 0.0f;

	Asura_Hashed_List_ConstIt<AVP_ServerEntity_Actor> xIter(AVP_ServerEntity_Actor::GetList());
	for (; !xIter.Done(); xIter.Next(true))
	{
		const AVP_ServerEntity_Actor* pxTarget = xIter.GetCurrent();
		if (pxTarget->IsDead())
		{
			continue;
		}

		// ignore targets that are not enemies
		switch (pxTarget->GetClassification())
		{
		case AVP_ENTITYCLASS_ALIEN:
		case AVP_ENTITYCLASS_ALIENCHESTBURSTER:
		case AVP_ENTITYCLASS_ALIENFACEHUGGER:
		case AVP_ENTITYCLASS_ALIENQUEEN:
		case AVP_ENTITYCLASS_PREDATOR:
		case AVP_ENTITYCLASS_ALIEN_HYBRID:
		case AVP_ENTITYCLASS_PREDALIEN:
			break;
		default: // everything else is not an enemy
			continue;
		}
		const Asura_Vector_3 xTargetPosition = pxTarget->GetPosition();

		float fCurrentDistance = (xTargetPosition-xOurPosition).MagnitudeSquared();

		// ignore targets that are too far
		if (fCurrentDistance > dbg_fMinDistanceToAquireSq)
		{
			continue;
		}

		int iPriority;

		if(!CalcTargetingPriority( pxTarget->GetClassification(), pxTarget->GetGuid(), iPriority ))
		{
			iPriority = AVP_AI_TargetPriorityData::NORMAL;
		}

		// ignore targets with ignore priority or with lower priority then best one.
		if (iPriority == AVP_AI_TargetPriorityData::IGNORE_P || iPriority < iBestPriority)
		{
			continue;
		}



		if (iPriority > iBestPriority || ((iPriority == iBestPriority) && fClosestDistanceSq > fCurrentDistance))
		{
			iBestPriority = iPriority;
			m_uTargetID = pxTarget->GetGuid();
			fClosestDistanceSq = fCurrentDistance;
		}
	}

}

void AVP_Brain_Actor_Dropship::UpdateWeaponState()
{
	if (m_bWeaponStatusNeedUpdate)
	{
		AVP_ServerVehicle_Dropship* pxActor = GetDropshipActor();
		AVP_Inventory_System* pxInventorySystem	= pxActor ? pxActor->GetInventorySystem() : NULL;
		AVP_Inventory_Group*  pxWeaponGroup	= pxInventorySystem ? pxInventorySystem->GetGroup( uAVP_INVENTORY_GROUPHASH_WEAPON ) : NULL;
		Asura_Assert(pxWeaponGroup, "Weapon group not found");
		if ( pxWeaponGroup )
		{
			switch (m_eDesiredWeaponType)
			{
				case WEAPON_LAUNCHER:
					if (pxWeaponGroup->GetCurrentItem() == s_uLauncher0HashID + m_uLauncherNumber || 
						( (!pxWeaponGroup->GetCurrentInventoryItem() || pxWeaponGroup->GetCurrentInventoryItem()->IsIdling(true)) &&
						pxWeaponGroup->SetCurrentItem( s_uLauncher0HashID + m_uLauncherNumber, false ) ) )
					{
						m_bWeaponStatusNeedUpdate = false;
					}
					break;
				case WEAPON_NONE:
					if (pxWeaponGroup->GetCurrentItem() == ASURA_GUID_UNREGISTERED || 
						(pxWeaponGroup->GetCurrentInventoryItem()->IsIdling(true) &&
						pxWeaponGroup->SetCurrentItem( ASURA_GUID_UNREGISTERED, false ) ) )
					{
						m_bWeaponStatusNeedUpdate = false;
					}
					break;
				default:
					Asura_Assert(false, "Unknown desired weapon");
					break;
			}
		}
	}
}




//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////
