//////////////////////////////////////////////////////////////////////////////
//
// AVP_Brain_Actor_Predator.h
//
// Copyright (C) 2007 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __AVP_BRAIN_ACTOR_PREDATOR_H__
#define __AVP_BRAIN_ACTOR_PREDATOR_H__ 1

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////


// asura
#include "Core/Asura_HashedList.h"
#include "Core/AVP_Navigation_Types.h"

// axon
#include "Core/BrainTypes/AVP_Brain_Actor.h"

// avp

// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////////

class AVP_Knowledge_System_Actor;
class AVP_ServerActor_Predator;
class AVP_ClientActor_Predator;
class AVP_ServerNode_VantagePoint_Predator;
struct Axon_Knowledge;
class AVP_Brain_Actor_Predator;
class AVP_Knowledge_System_Predator;

//////////////////////////////////////////////////////////////////////////////
//
// AVP_Predator_VentagePointEvaluator
//
//////////////////////////////////////////////////////////////////////////////

class AVP_Predator_VentagePointEvaluator
{
public:
	AVP_Predator_VentagePointEvaluator();
	~AVP_Predator_VentagePointEvaluator();
	void SetBrain(const AVP_Brain_Actor_Predator* pxBrain);

	void UpdateMyPosition(const Asura_Vector_3 &xMyPosition);
	void Start(Asura_Guid uCurrentPreferedVP, bool bFast);
	void Tick(float fTimeslice);
	bool HasREsult();
	const AVP_ServerNode_VantagePoint_Predator* GetResult() const;
	void Cancel();

	void IgnoreVentagePoint(Asura_Guid uGuid, float fDuration);

#ifndef ASURA_NO_DEBUG_OUTPUT
	void RenderDebugInfo(Asura_Vector_2 &xDebugOutputPos);
#endif


private:

	bool IsVentagePointIgnored( Asura_Guid uGuid );
	void UpdateIgnoreList();

	void FindNextVentagePoint(bool bSkipCurrent);
	void CheckVentagePoint();

	const AVP_Brain_Actor_Predator* m_pxBrain;
	bool m_bCalculating;
	Asura_Vector_3 m_xMyPosition;
	Asura_Vector_3 m_xMyPositionUsed;
	Asura_Hashed_List_ConstIt< AVP_ServerNode_VantagePoint_Predator > m_xVPIt;
	int m_iVPListSize;
	const AVP_ServerNode_VantagePoint_Predator* m_pxBestVP;
	const AVP_ServerNode_VantagePoint_Predator* m_pxCurrentPreferedVP;
	float m_fDistanceToCurrentPreferedVPSqrd;
	float m_fCurrentScore;
	float m_fCurrentDenum;
	float m_fBestScore;
	Asura_LinkList<const Asura_AI_NavVertex*> m_xTargetWPList;
	int m_iWPListSize;
	Asura_LinkList_ConstIt<const Asura_AI_NavVertex*> m_xTargetWPListIt;

	struct  IgnoreVPData
	{
		Asura_Guid m_uGuid;
		float m_fTimeToStopIgnore;
	};

	Asura_LinkList<IgnoreVPData> m_xIgnoredVentagePoints;

	int m_iLOSTestCount;
	bool m_bFastSearch;

#ifndef ASURA_NO_DEBUG_OUTPUT
	float m_fRunTime;
	int m_iNumberOfVP;
	int m_iNumberOfVPTested;
	int m_iNumberOfLOSTests;
#endif

};

//////////////////////////////////////////////////////////////////////////////
//
// AVP_Predator_FindATargetToAimAt
//
//////////////////////////////////////////////////////////////////////////////

class AVP_Predator_FindATargetToAimAt
{
public:

	AVP_Predator_FindATargetToAimAt();
	void SetBrain(const AVP_Brain_Actor_Predator* pxBrain);

	void Start(bool bForgetLastTarget);
	void Tick(float fTimeslice);
	bool Finished() const;
	bool GetResult(Asura_Vector_3 &xResult) const;
	void Cancel();


#ifndef ASURA_NO_DEBUG_OUTPUT
	void RenderDebugInfo(Asura_Vector_2 &xDebugOutputPos);
#endif

private:

	enum Status
	{
		STATUS_SUCCESS,
		STATUS_FAIL,
		STATUS_CONTINUE
	};

	enum TargetType 
	{
		TT_MAIN_TARGET,
		TT_MAIN_TARGET_LAST_POSITION,
		TT_SECONDARY_TARGET,
		TT_SECONDARY_TARGET_LAST_POSITION,
		TT_MAIN_TARGET_LAST_POSITION_AREA,
		TT_NO_TARGET
	};

	enum State
	{
		IDLE,
		CHECK_MAIN_TARGET_ACTUAL_POSITION,
		CHECK_SECONDARY_TARGETS_ACTUAL_POSITION,
		CHECK_MAIN_TARGET_LAST_KNOWN_POSITION,
		CHECK_SECONDARY_TARGETS_LAST_KNOWN_POSITION,
		CHECK_AROUND_MAIN_TARGET
	};

	void ConditionalSwitchToState(Status eStatus);
	void SwitchToState(State eState);

	void BuildCheckList();

	Status CheckMainTargetActualPosition();
	Status CheckSecondaryTargetsActualPosition(u_int uIndex);
	Status CheckAllSecondaryTargetsActualPosition();
	Status CheckMainTargetLastPosition();
	Status CheckSecondaryTargetsLastPosition(u_int uIndex);
	Status CheckAllSecondaryTargetsLastPosition();
	Status CheckAroundMainTargetLastPosition();

	bool GenerateWaypoints();

	bool CanFireAt(const Asura_Vector_3& xTarget);

	State m_eState;
	Asura_Vector_3 m_xMuzzlePosition;
	const AVP_Brain_Actor_Predator* m_pxBrain;
	Asura_LinkList<const Asura_AI_NavVertex*> m_xTargetWPList;
	float m_fWPListSize;
	Asura_LinkList_ConstIt<const Asura_AI_NavVertex*> m_xTargetWPListIt;

	int m_iAimTestCount;

	struct  
	{
		TargetType m_eType;
		Asura_Vector_3 m_xPosition;
		u_int m_uSecondaryIndex;
	} m_xLastResults;

	Asura_LinkList<State> m_xStateList; 

	bool m_bTargetFound;

	u_int m_uTemporaryIndex;

	float m_fRunTime;

#ifndef ASURA_NO_DEBUG_OUTPUT
	int m_iWPCounter;
#endif

};



//////////////////////////////////////////////////////////////////////////////
//
// AVP_Brain_Actor_Predator
//
//////////////////////////////////////////////////////////////////////////////

// blueprint configuration
struct AVP_Brain_Actor_Predator_ConfigurationData
{
	// weapon config
	struct 
	{
		float m_fAimSearchRadius;
		float m_fPlasmaCasterDamage;
		float m_fPlasmaCasterDelayBetweenShots;
		int m_iPlasmaCasterMaximumNumberOfShots;
		int m_iPlasmaCasterMiniumNumberOfShots;
	} m_xWeaponConfig;

	// ventage point configuration
	struct 
	{
		float m_fMaximumDistanceToTarget;
		float m_fMaximumMoveDistance;
		float m_fMinimumDistanceToTarget;
		float m_fMoveFactor;
		float m_fRadiusAroundTarget;
		float m_fSmallMoveDistance;
	} m_xVentargePointEvalConfig;

	// Behaviour config
	struct 
	{
		float m_fAimDelay;
	} m_xAtVPBehaviour;

	struct 
	{
		float m_fFleeTime;
		float m_fFleeMinDistance;
		float m_fFleeSearchRadius;
	} m_xFleeBehaviour;

	struct 
	{
		int m_iNumberOfDamagePercentPoint;
		float m_fDamagePercentPoint[3];
		float m_fAvoidDistance;
		float m_fSeenRecentlyTime;
		float m_fNotSeenRecentlyTime;
	} m_xHighLevelBehaviour;

	struct 
	{
		float m_fLookAroundMaxTime;
		float m_fSeekRadiusNoTarget;
		float m_fSeekRadiusAroundTarget;
	} m_xSeekBehaviour;
};


class AVP_Brain_Actor_Predator : public AVP_Brain_Actor
{
    typedef AVP_Brain_Actor PARENT;

public:
    AVP_Brain_Actor_Predator( AVP_ServerActor_Predator* pxPredator );

	virtual void Initialise();
	virtual void Shutdown();
    virtual void InitialiseFromBlueprint( const Asura_Blueprint& xBlueprint );
    virtual void ReadFromChunkStream( Asura_Chunk_Stream& xStream );
    virtual void WriteToChunkStream( Asura_Chunk_Stream& xStream ) const;

	virtual void EndGS2();

	virtual void DoPerFrameUpdate(const float fTimeslice);

    virtual Axon_Behaviour* CreateBehaviour( const u_int uBehaviourType, const Asura_Message* pxMessage = NULL );

    // must implement 
	virtual Asura_Guid GetPrimaryThreatGuid( void );
	virtual Asura_Guid GetAlertGuid( void );

    void Cloak();
    void Decloak();
	bool IsCloaked() const;

	void EquipPlasmaCaster();
	void EquipWristBlades();
	bool IsHoldingPlasmaCaster() const;
	bool IsHoldingWristBlades() const;

	bool StartPlasmaCasterCharge(float fDamage);
	void ReleasePlasmaCasterCharge();
	void AbortPlasmaCasterCharge();
	bool IsPlasmaCasterCharged();

#ifndef ASURA_NO_DEBUG_OUTPUT
	void GetCloakDebugString(const u_int uMaxChars, char* szText) const;
	void GetWeaponDebugString(const u_int uMaxChars, char* szText) const;
#endif

	void SetFastTurnSpeed();
	void SetNormalTurnSpeed();
	void SetNormalAimSpeed();


	AVP_ServerActor_Predator* GetPredatorActor() const;
	AVP_ClientActor_Predator* GetPredatorClientActor() const;
	AVP_Knowledge_System_Predator* GetKnowledgeSystem() const { return m_pxKnowledgeSystem; }

	void SetCrouched( const bool bCrouched );

	void IgnoreVentagePoint(Asura_Guid uGuid, float fDuration);

	void StartVPTest(Asura_Guid uCurrentPreferedVP, bool bFast);
	bool IsVPTestFinished();
	const AVP_ServerNode_VantagePoint_Predator* GetVPTestResult() const;
	void CancelVPTest();

	void StartAimAtTest(bool bForgetLastTarget);
	bool IsAimAtTestFinished() const;
	bool GetAimAtTestResult(Asura_Vector_3& xResult) const;
	void CancelAimAtTest();
	bool CanFireAt(const Asura_Vector_3& xTarget, bool bMustBeExact) const;

	float GetHealthPercentage() const;
	float GetDamageTakenInLast() const;

	bool WasAnyTargetSeenRecently(float fRecentTime) const;
	bool AreAllTargetsTooOld() const;

	const AVP_Brain_Actor_Predator_ConfigurationData& GetConfigInfo() const { return m_xConfig; }

	const Asura_Vector_3& GetSpawnPoint() const { return m_xSpawnPoint; } 

	bool CanPlanMove() const { return !m_bIsFalling && !m_bIsJumping && !m_bIsMelee; }
	bool CanUpdateMelee() const { return !m_bIsFalling && !m_bIsJumping; }

	bool IsFalling() const { return m_bIsFalling; }
	bool IsJumping() const { return m_bIsJumping; }
	bool IsMelee() const { return m_bIsMelee; }
	void SetJumping( bool bIsJumping );
	bool JustFinishedFalling() const { return m_bJustFinishedFalling; }
	void SetMelee( bool bIsMelee );

    static u_int GetLargestBehaviourSize ();
	static u_int GetPathFinderFlagsFromClassification(u_int uClassification);

#ifndef ASURA_NO_DEBUG_OUTPUT
	virtual void DoRenderDebugInfo();
#endif

protected:
	virtual void RegisterCallbacks();
	virtual void UnregisterCallbacks();
    virtual bool HandleMessage( const Asura_Message* pxMessage );
	virtual float GetNearbyRange() const;


#ifndef ASURA_NO_DEBUG_OUTPUT
	virtual bool Verify();
#endif

private:
	// Note: Parent function is public
	virtual void EstablishNavData();
	void DoKnowledgeUpdate( const float fTimeslice );

	void UpdateFalling(float fTimeslice );

private:
    union
    {
        u_int m_uFlags;
        struct 
        {
            u_int m_bUnused : 1;
        };
    };

	Asura_Handle m_iKnowledgeUpdateHandle;
	AVP_Knowledge_System_Predator*	m_pxKnowledgeSystem;

    enum { CURRENT_CHUNK_VERSION = 1 };

	AVP_Predator_VentagePointEvaluator m_xVentagePointEvaluator;
	AVP_Predator_FindATargetToAimAt m_xFindTargetToAimAt;
	const Asura_Vector_3 m_xSpawnPoint;

	bool m_bIsFalling; // true while falling
	bool m_bIsJumping; // true from the start of requesting a jump till the animation finished
	bool m_bJustFinishedFalling; // true for one frame right after detecting that a fall finished.
	bool m_bIsMelee; // true from the start of requesting a melee move till it is finished or stopped.

	float m_fNotSupportedTime;

	AVP_Brain_Actor_Predator_ConfigurationData m_xConfig;
};

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

#endif //__AVP_BRAIN_ACTOR_PREDATOR_H__
