//----------------------------------------------------------------------------
// File			: AVP_Brain_Actor_Sentrygun.cpp
// Created		: 29 / 10 / 07
// Created By	: Gareth Thomas
// Copyright (C) 2007 Rebellion. All Rights Reserved.
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// INCLUDES
//----------------------------------------------------------------------------
// Core
#include "Core/Asura.h"
// This files Header
#include "Core/BrainTypes/AVP_Brain_Actor_Sentrygun.h"
// Asura
#include "Network/Asura_Message.h"
#include "Network/Asura_Network.h"
#include "Core/Asura_Scheduler.h"
#ifndef ASURA_NO_DEBUG_OUTPUT
#include "Camera/Asura_Camera.h"
#include "Rendering/Asura_Font_System.h"
#include "Interface/Asura_CommandConsole.h"
#endif

// AVP
#include "Actor/Weapons/Sentrygun/AVP_Sentrygun_AIAnimStateMachine.h"
#include "Entity/AVP_Entity_Classification.h"
#include "Entity/AVP_Messages.h"
#include "Entity/PhysicalObject/Actor/Weapons/Sentrygun/AVP_ServerWeapon_Sentrygun.h"

// AVP Axon
#include "Behaviour/BehaviourTypes/AVP_Behaviour_Types.h"
#include "Core/Axon_Project_System.h"
#include "Knowledge/AVP_Knowledge_System_Sentrygun.h"
#include "Tactical/AVP_Tactical_System.h"

// Behaviour Header files includes
#include "Behaviour/BehaviourTypes/Sentrygun/AVP_Behaviour_Sentrygun.h"

AVP_Brain_Actor_Sentrygun::AVP_Brain_Actor_Sentrygun(AVP_ServerWeapon_Sentrygun* pxSentrygun)
: PARENT(pxSentrygun)
, m_iKnowledgeUpdateHandle(ASURA_INVALID_HANDLE)
{
}

void AVP_Brain_Actor_Sentrygun::Initialise(void)
{
	PARENT::Initialise();

	m_pxSentryGunKnowledgeSystem = new AVP_Knowledge_System_Sentrygun(this);

	if (m_pxSentryGunKnowledgeSystem)
	{
		m_pxSentryGunKnowledgeSystem->Initialise();
	}
}

void AVP_Brain_Actor_Sentrygun::Shutdown(void)
{
	if (m_pxSentryGunKnowledgeSystem)
	{
		m_pxSentryGunKnowledgeSystem->Shutdown();
	}

	delete m_pxSentryGunKnowledgeSystem;
	m_pxSentryGunKnowledgeSystem = NULL;

	PARENT::Shutdown();
}

void AVP_Brain_Actor_Sentrygun::InitialiseFromBlueprint(const Asura_Blueprint& xBlueprint)
{
	PARENT::InitialiseFromBlueprint(xBlueprint);

	if ( m_pxSentryGunKnowledgeSystem )
	{
		m_pxSentryGunKnowledgeSystem->InitialiseFromBlueprint( xBlueprint );
	}

	CleanUpBehaviours();
	if ( IsActive() )
	{
		// create the default high level behaviour
		if (!CreateDefaultBehaviour())
		{
			Asura_Assert(false, "AVP_Brain_Actor_Sentrygun :: InitialiseFromBlueprint - Could not create / push default sentrygun behaviour");
		}
		CreateGS2BehaviourIfNecessary();
	}

	EstablishNavData();
}

void AVP_Brain_Actor_Sentrygun::CleanUpBehaviours()
{
	DestroyAllBehaviours();
}

Axon_Behaviour* AVP_Brain_Actor_Sentrygun::CreateBehaviour(const u_int uBehaviourType, const Asura_Message* pxMessage)
{
	bool				bIsSentrygunBehaviour	= IsSentrygunBehaviour(uBehaviourType);
	Axon_Behaviour*		pxBehaviour			= NULL;

	if (bIsSentrygunBehaviour)
	{
		pxBehaviour = CreateSentrygunBehaviour(uBehaviourType);

		if (pxBehaviour && pxMessage)
		{
			pxBehaviour->DoHandleMessage(pxMessage, true);
		}
	}
	else
	{
		pxBehaviour = PARENT::CreateBehaviour(uBehaviourType, pxMessage);
	}

	return pxBehaviour;
}

Asura_Guid AVP_Brain_Actor_Sentrygun::GetPrimaryThreatGuid()
{
	AVP_Knowledge_System_Sentrygun*	pxKnowledgeSystem = GetKnowledgeSystem();

	if ( !pxKnowledgeSystem ) 
	{
		return ASURA_GUID_UNREGISTERED;
	}
	else
	{
		return  pxKnowledgeSystem->GetBestThreatKnowledge().GetGuid();
	}
}

Asura_Guid AVP_Brain_Actor_Sentrygun::GetAlertGuid()
{
	return ASURA_GUID_UNREGISTERED;
}

void AVP_Brain_Actor_Sentrygun::RegisterCallbacks()
{
	PARENT::RegisterCallbacks();

	if (m_iKnowledgeUpdateHandle == ASURA_INVALID_HANDLE)
	{
		Asura_Scheduler& xKnowledgeUpdateScheduler = Axon_System::GetKnowledgeUpdateScheduler();
		Asura_Callback_MemberFunction_1Arg<void, float, AVP_Brain_Actor_Sentrygun> xKnowledgeCallback( this, &AVP_Brain_Actor_Sentrygun::DoKnowledgeUpdate );
		m_iKnowledgeUpdateHandle = xKnowledgeUpdateScheduler.RegisterCallback( 0.5f, &xKnowledgeCallback );
	}
}

void AVP_Brain_Actor_Sentrygun::UnregisterCallbacks()
{
	PARENT::UnregisterCallbacks();

	if (m_iKnowledgeUpdateHandle != ASURA_INVALID_HANDLE)
	{
		Axon_System::GetKnowledgeUpdateScheduler().UnregisterCallback( m_iKnowledgeUpdateHandle );
		m_iKnowledgeUpdateHandle = ASURA_INVALID_HANDLE;
	}
}

#include "Core/Asura_MemoryManagement_DebugOff.h"
Axon_Behaviour* AVP_Brain_Actor_Sentrygun::CreateSentrygunBehaviour(u_int uBehaviourType)
{
	void*				pxMemory			= s_pxBehaviourMempool->Allocate();
	Axon_Behaviour*		pxBehaviour			= NULL;

	if (!pxMemory)
	{
		Asura_Assert(false, "Axon_Project_Brain::CreateBehaviour - could not allocate behaviour memory");
		return NULL;
	}

	switch (uBehaviourType)
	{
		case AVP_BEHAVIOUR_TYPE_SENTRYGUN:
		{
			pxBehaviour = new (pxMemory) AVP_Behaviour_Sentrygun(this);
		}
		break;
		
		default:
		{
			Asura_Assert(false, "AVP_Brain_Actor_Sentrygun :: CreateSentrygunBeahviour - Unknown Behaviour Type");
		}
		break;
	}

	if (!pxBehaviour)
	{
		s_pxBehaviourMempool->Deallocate(pxMemory);
	}

	return pxBehaviour;
}
#include "Core/Asura_MemoryManagement_DebugOn.h"

bool AVP_Brain_Actor_Sentrygun::IsSentrygunBehaviour(const u_int uBehaviourType)
{
	switch (uBehaviourType)
	{
		case AVP_BEHAVIOUR_TYPE_SENTRYGUN:
		{
			return true;
		}
		
		default:
		{
			return false;
		}
	}
}

bool AVP_Brain_Actor_Sentrygun::CreateDefaultBehaviour()
{
	return CreateActivateAndPushBehaviour(AVP_BEHAVIOUR_TYPE_SENTRYGUN) != NULL;
}

void AVP_Brain_Actor_Sentrygun::ReadFromChunkStream(Asura_Chunk_Stream& xStream)
{
	int iVersion;
	xStream >> iVersion;

	PARENT::ReadFromChunkStream( xStream );

	if ( iVersion == 0 )
	{
		//CreateDefaultBehaviour(); // we don't chunk out our behaviours so we'll re-create our default here
	}
	else
	{
		Asura_Assertf( false, ( "AVP_Brain_Actor_Sentrygun::ReadFromChunkStream - unhandled chunk version (%d)", iVersion ) );
	}
}

void AVP_Brain_Actor_Sentrygun::WriteToChunkStream(Asura_Chunk_Stream& xStream) const
{
	xStream << CURRENT_CHUNK_VERSION;

	PARENT::WriteToChunkStream( xStream );
}

AVP_ServerWeapon_Sentrygun* AVP_Brain_Actor_Sentrygun::GetSentrygunActor() const
{
	return static_cast<AVP_ServerWeapon_Sentrygun* >(GetActor());
}

void AVP_Brain_Actor_Sentrygun::UpdateKnowledge(const float fTimeslice)
{
	if (m_pxSentryGunKnowledgeSystem)
	{ 
		m_pxSentryGunKnowledgeSystem->Update(fTimeslice); 
	}
}

u_int AVP_Brain_Actor_Sentrygun::GetLargestBehaviourSize()
{
	u_int uLargestBehaviourSize = 0;

	AXON_CALC_MAX_BEHAVIOUR_SIZE(uLargestBehaviourSize, AVP_Behaviour_Sentrygun);

	return uLargestBehaviourSize;
}

#ifndef ASURA_NO_DEBUG_OUTPUT
void AVP_Brain_Actor_Sentrygun::DoRenderDebugInfo()
{
	Asura_Vector_3 xOutputPos = GetSentrygunActor()->GetPosition();

	if (!Asura_Camera::ProjectWorldCoordsToScreenSpace( xOutputPos, true ) || xOutputPos.z <= 0.0f)
	{
		return;
	}

	PARENT::DoRenderDebugInfo();

	Asura_Font_System::Print
	(
		GetDebugOutputPos().x,
		GetDebugOutputPos().y,
		0.5f,
		0xffffffff,
		ASURA_TEXTFORMAT_CENTREJUSTIFIED,
		AVP_Sentrygun_AIAnimStateMachine::GetStateDescriptionStatic( GetActor()->GetAIAnimStateMachine()->GetCurrentState() )
	);
	GetDebugOutputPos().y += 10.0f;
}

bool AVP_Brain_Actor_Sentrygun::Verify()
{
	Asura_Assert( m_pxSentryGunKnowledgeSystem, "AVP_Brain_Actor_Sentrygun::Verify - no knowledge system" );
	Asura_Assert( GetSentrygunActor(), "AVP_Brain_Actor_Sentrygun::Verify - no civilian actor" );

	return ( PARENT::Verify() && m_pxSentryGunKnowledgeSystem && m_pxSentryGunKnowledgeSystem->Verify() && GetSentrygunActor() );
}
#endif
