//////////////////////////////////////////////////////////////////////////////
//
// AVP_Brain_Team_Alien.h
//
// Copyright (C) 2007 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __AVP_BRAIN_TEAM_ALIEN_H__
#define __AVP_BRAIN_TEAM_ALIEN_H__ 1

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// asura
#include "Core/Asura_Handle.h"
#include "ChunkSystem/Asura_Chunk_Stream.h"

// axon
#include "Core/BrainTypes/AVP_Brain_Team.h"

// avp
#include "Rendering/Asura_Project_Skin_Elements.h"

// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////////

class AVP_Knowledge_System_AlienTeam;
class AVP_ServerActor_Alien;
class AVP_ServerEntity_AlienSpawnPoint;
class AVP_ServerEntity_Team_Alien;

//////////////////////////////////////////////////////////////////////////////
//
// AVP_Brain_Team_Alien
//
//////////////////////////////////////////////////////////////////////////////

class AVP_Brain_Team_Alien : public AVP_Brain_Team
{
    typedef AVP_Brain_Team PARENT;

public:
	enum Behaviour
	{
		BEHAVIOUR_INACTIVE = 0,
		BEHAVIOUR_STALK,
		BEHAVIOUR_ATTACK,
		// ...

		NUM_VALID_BEHAVIOURS,

		BEHAVIOUR_NONE
	};

	AVP_Brain_Team_Alien( AVP_ServerEntity_Team_Alien* pxAlienTeam );

    virtual void Initialise();
    virtual void Shutdown();
	virtual void DoPerFrameUpdate( const float fTimeslice );
	virtual void ReadFromChunkStream( Asura_Chunk_Stream& xStream );
    virtual void WriteToChunkStream( Asura_Chunk_Stream& xStream ) const;
#if !defined( ASURA_NO_DEBUG_OUTPUT )
	virtual void DoRenderDebugInfo();
#endif

	virtual Axon_Behaviour* CreateBehaviour( const u_int uBehaviourType, const Asura_Message* pxMessage = NULL ) { return NULL; }
	virtual void            DestroyBehaviour( Axon_Behaviour*& pxBehaviour ) {}

    AVP_ServerEntity_Team_Alien* GetAlienTeam() const;
	AVP_Knowledge_System_AlienTeam* GetKnowledgeSystem() const { return m_pxAlienTeamKnowledgeSystem; }

protected:
    virtual void RegisterCallbacks();
    virtual void UnregisterCallbacks();
    virtual void UpdateKnowledge( const float fTimeslice );

#ifndef ASURA_NO_DEBUG_OUTPUT
	virtual bool Verify();
#endif

private:
    void DoKnowledgeUpdate( const float fTimeslice ) { UpdateKnowledge( fTimeslice ); }

private:
    Asura_Handle                    m_iKnowledgeUpdateHandle;
    AVP_Knowledge_System_AlienTeam* m_pxAlienTeamKnowledgeSystem;

	enum { CURRENT_CHUNK_VERSION = 5 };
};

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

#endif //__AVP_BRAIN_TEAM_ALIEN_H__
