//////////////////////////////////////////////////////////////////////////////
//
// AVP_Knowledge_System_AlienTeam.cpp
//
// Copyright (C) 2007 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"

// header
#include "Knowledge/AVP_Knowledge_System_AlienTeam.h"

// asura
#include "ChunkSystem/Asura_Chunk_Stream.h"

#ifndef ASURA_NO_DEBUG_OUTPUT
#include "Rendering/Asura_Font_System.h"
#endif

// avp
#include "Entity/AVP_Entity_Classification.h"
#include "Entity/Controller/AVP_ServerEntity_Team_Alien.h"
#include "Entity/GamePlayVolume/AVP_ServerEntity_GamePlayVolume.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Alien/AVP_ServerActor_Alien.h"
#include "Inventory/AVP_Inventory_HashIDs.h"
#include "Inventory/AVP_Inventory_Item.h"
#include "Inventory/AVP_Inventory_System.h"
#include "Inventory/Items/Weapons/AVP_InventoryWeapon_RangedWeapon.h"
#include "Tactical/AVP_Tactical_System.h"

// Axon
#include "Behaviour/BehaviourTypes/AVP_Behaviour_Types.h"
#include "Core/BrainTypes/AVP_Brain_Actor_Alien.h"
#include "Core/BrainTypes/AVP_Brain_Team_Alien.h"
#include "Knowledge/SenseTypes/AVP_Sense_Vision_AlienTeam.h"

//////////////////////////////////////////////////////////////////////////////
//
// AVP_AlienTeam_Knowledge
//
//////////////////////////////////////////////////////////////////////////////

void AVP_AlienTeam_Knowledge::UpdateWeaponData() const
{
	const u_int uFrameNumber = Asura_Timers::GetFrameNumber();
	if ( uFrameNumber == m_uWeaponFrameLastUpdated )
	{
		return;
	}

	const AVP_ServerEntity_Actor* pxThreatActor = AVP_ServerEntity_Actor::FindEntity( m_uThreatGuid );
	if ( !pxThreatActor )
	{
		Asura_Assert( false, "AVP_AlienTeam_Knowledge::UpdateWeaponData - couldn't find threat actor" );
		return;
	}

	m_bWeaponFiring = false;
	const AVP_Inventory_System* pxInventorySystem = pxThreatActor->GetInventorySystem();
	if ( pxInventorySystem )
	{
		const AVP_Inventory_Item* pxPlasmaCaster = pxInventorySystem->GetItem( uAVP_INVENTORY_WEAPONHASH_WEAPON01 );
		const AVP_Inventory_Item* pxCurrentWeapon = pxInventorySystem->GetCurrentItem( uAVP_INVENTORY_GROUPHASH_WEAPON );
		if ( pxCurrentWeapon && pxCurrentWeapon != pxPlasmaCaster )
		{
			if ( pxCurrentWeapon->GetItemType() == AVP_INVENTORYITEMTYPE_WEAPON_RANGED || pxCurrentWeapon->GetItemType() == AVP_INVENTORYITEMTYPE_WEAPON_FLAMETHROWER )
			{
				if ( pxCurrentWeapon->GetCurrentUsageType() == AVP_InventoryWeapon_RangedWeapon::UT_FULL_AUTO_PRIMARY ||
					pxCurrentWeapon->GetCurrentUsageType() == AVP_InventoryWeapon_RangedWeapon::UT_N_BULLET_BURST_PRIMARY )
				{
					m_bWeaponFiring = true;
				}
			}
		}
	}

	// We could use the muzzle marker point, but not all skins have this so we'll have to approximate.
	pxThreatActor->GetHeadPosition( m_xWeaponPosition );
	m_xWeaponDirection = pxThreatActor->GetAimDirection();

	// Now fire a ray and see how far we go. This isn't going to be deathly accurate, but it should do.
	Asura_LOS_InputData xInputData( m_xWeaponPosition, m_xWeaponDirection, m_uThreatGuid, 250.0f );
	const Asura_LOS_OutputData* pxOutputData = Asura_LOS::TestAgainstEverything( &xInputData );
	m_fWeaponLOSLength = pxOutputData->m_fDistanceToIntersection;

	m_uWeaponFrameLastUpdated = uFrameNumber;
}

//////////////////////////////////////////////////////////////////////////////
//
// AVP_Knowledge_System_AlienTeam
//
//////////////////////////////////////////////////////////////////////////////

AVP_Knowledge_System_AlienTeam::AVP_Knowledge_System_AlienTeam( AVP_Brain_Team_Alien* pxAlienTeamBrain )
: m_pxBrain( pxAlienTeamBrain )
, m_xSenseList()
{
}

AVP_Knowledge_System_AlienTeam::~AVP_Knowledge_System_AlienTeam()
{
	DestroyAllKnowledge();
}

void AVP_Knowledge_System_AlienTeam::ReadFromChunkStream( Asura_Chunk_Stream& xStream )
{
    int iVersion;
    xStream >> iVersion;

	// No parent functions to call.

	if ( iVersion == CURRENT_CHUNK_VERSION )
	{
	}
	else if ( iVersion == 2 )
	{
		float fDummy;
		// Load class multipliers
		for( int i = 0; i < 11; i++ )
		{
			xStream >> fDummy;
		}
	}
	else if ( iVersion == 1 )
	{
		float fDummy;
		// Load class multipliers
		for( int i = 0; i < 11; i++ )
		{
			xStream >> fDummy;
		}

		xStream >> fDummy;
		xStream >> fDummy;
		xStream >> fDummy;

		// vision sense was just an int
		int iVisionSystemVersion;
		xStream >> iVisionSystemVersion;
	}
	else if ( iVersion == 0 )
	{
		// vision sense was just an int
		int iVisionSystemVersion;
		xStream >> iVisionSystemVersion;
	} 
	else
	{
		Asura_Assertf( false, ( "AVP_Knowledge_System_AlienTeam::ReadFromChunkStream - unhandled version (%d)", iVersion ) );
	}
}

void AVP_Knowledge_System_AlienTeam::WriteToChunkStream( Asura_Chunk_Stream& xStream ) const
{
    xStream << CURRENT_CHUNK_VERSION;

	// No parent functions to call.
}

void AVP_Knowledge_System_AlienTeam::Update( float fTimeslice )
{
	UpdateSense();
}

void AVP_Knowledge_System_AlienTeam::PerFrameUpdate( float fTimeslice )
{
	PerFrameUpdateSense(fTimeslice);
}

void AVP_Knowledge_System_AlienTeam::Shutdown()
{
	DestroyAllKnowledge();
}

void AVP_Knowledge_System_AlienTeam::AddAttacker( Asura_Guid uAttackerGuid, const Asura_Guid uTargetGuid )
{
	for ( Asura_Hashed_List_It< AVP_AlienTeam_Knowledge > xVisionIt( &m_xSenseList ); !xVisionIt.Done(); xVisionIt.Next() )
	{
		AVP_AlienTeam_Knowledge* pxKnowledge = xVisionIt.GetCurrent();
		if ( pxKnowledge && pxKnowledge->GetGuid() == uTargetGuid )
		{
			++pxKnowledge->m_uNumAttackers;
			return;
		}
	}
}

void AVP_Knowledge_System_AlienTeam::RemoveAttacker( Asura_Guid uAttackerGuid, const Asura_Guid uTargetGuid, bool bAttackAborted )
{
	for ( Asura_Hashed_List_It< AVP_AlienTeam_Knowledge > xVisionIt( &m_xSenseList ); !xVisionIt.Done(); xVisionIt.Next() )
	{
		AVP_AlienTeam_Knowledge* pxKnowledge = xVisionIt.GetCurrent();
		if ( pxKnowledge && pxKnowledge->GetGuid() == uTargetGuid )
		{
			--pxKnowledge->m_uNumAttackers;
			if (!bAttackAborted)
			{
				pxKnowledge->m_fLastAttackFinishTime = Asura_Timers::GetLocalScaledTime();
			}
			return;
		}
	}
}

void AVP_Knowledge_System_AlienTeam::AddTargeter( Asura_Guid uTargeterGuid, const Asura_Guid uTargetGuid )
{
	for ( Asura_Hashed_List_It< AVP_AlienTeam_Knowledge > xVisionIt( &m_xSenseList ); !xVisionIt.Done(); xVisionIt.Next() )
	{
		AVP_AlienTeam_Knowledge* pxKnowledge = xVisionIt.GetCurrent();
		if ( pxKnowledge && pxKnowledge->GetGuid() == uTargetGuid )
		{
			++pxKnowledge->m_uNumTargeters;
			return;
		}
	}
}

void AVP_Knowledge_System_AlienTeam::RemoveTargeter( Asura_Guid uTargeterGuid, const Asura_Guid uTargetGuid )
{
	for ( Asura_Hashed_List_It< AVP_AlienTeam_Knowledge > xVisionIt( &m_xSenseList ); !xVisionIt.Done(); xVisionIt.Next() )
	{
		AVP_AlienTeam_Knowledge* pxKnowledge = xVisionIt.GetCurrent();
		if ( pxKnowledge && pxKnowledge->GetGuid() == uTargetGuid )
		{
			--pxKnowledge->m_uNumTargeters;
			return;
		}
	}
}


void AVP_Knowledge_System_AlienTeam::AddCharger( Asura_Guid uTargetGuid )
{
	for ( Asura_Hashed_List_It< AVP_AlienTeam_Knowledge > xVisionIt( &m_xSenseList ); !xVisionIt.Done(); xVisionIt.Next() )
	{
		AVP_AlienTeam_Knowledge* pxKnowledge = xVisionIt.GetCurrent();
		if ( pxKnowledge && pxKnowledge->GetGuid() == uTargetGuid )
		{
			++pxKnowledge->m_uNumChargers;
			return;
		}
	}
}

void AVP_Knowledge_System_AlienTeam::RemoveCharger( Asura_Guid uTargetGuid )
{
	for ( Asura_Hashed_List_It< AVP_AlienTeam_Knowledge > xVisionIt( &m_xSenseList ); !xVisionIt.Done(); xVisionIt.Next() )
	{
		AVP_AlienTeam_Knowledge* pxKnowledge = xVisionIt.GetCurrent();
		if ( pxKnowledge && pxKnowledge->GetGuid() == uTargetGuid )
		{
			--pxKnowledge->m_uNumChargers;
			return;
		}
	}

}
u_int AVP_Knowledge_System_AlienTeam::GetNumChargers( Asura_Guid uTargetGuid ) const
{
	for ( Asura_Hashed_List_It< AVP_AlienTeam_Knowledge > xVisionIt( &m_xSenseList ); !xVisionIt.Done(); xVisionIt.Next() )
	{
		AVP_AlienTeam_Knowledge* pxKnowledge = xVisionIt.GetCurrent();
		if ( pxKnowledge && pxKnowledge->GetGuid() == uTargetGuid )
		{
			return pxKnowledge->m_uNumChargers;
		}
	}

	return 0;
}

void AVP_Knowledge_System_AlienTeam::AddAttackGrapple( const Asura_Guid uTargetGuid )
{
	AVP_AlienTeam_Knowledge *pxKnowledge = GetKnowledge(uTargetGuid);
	Asura_Assert(pxKnowledge, "AddAttackGrapple: Target Knowledge not found");
	if (pxKnowledge)
	{
		++pxKnowledge->m_uNumAttackers;
		++pxKnowledge->m_uNumAttackGrapple;
	}
}

void AVP_Knowledge_System_AlienTeam::AddAttackStand( const Asura_Guid uTargetGuid )
{
	AVP_AlienTeam_Knowledge *pxKnowledge = GetKnowledge(uTargetGuid);
	Asura_Assert(pxKnowledge, "AddAttackMelee: Target Knowledge not found");
	if (pxKnowledge)
	{
		++pxKnowledge->m_uNumAttackers;
		++pxKnowledge->m_uNumAttackStand;
	}
}

void AVP_Knowledge_System_AlienTeam::AddAttackJump( const Asura_Guid uTargetGuid )
{
	AVP_AlienTeam_Knowledge *pxKnowledge = GetKnowledge(uTargetGuid);
	Asura_Assert(pxKnowledge, "AddAttackJump: Target Knowledge not found");
	if (pxKnowledge)
	{
		++pxKnowledge->m_uNumAttackers;
		++pxKnowledge->m_uNumAttackJump;
	}
}

void AVP_Knowledge_System_AlienTeam::RemoveAttackGrapple( const Asura_Guid uTargetGuid )
{
	AVP_AlienTeam_Knowledge *pxKnowledge = GetKnowledge(uTargetGuid);
	if (pxKnowledge)
	{
		Asura_Assert(pxKnowledge->m_uNumAttackers > 0 && pxKnowledge->m_uNumAttackGrapple > 0, "RemoveAttackGrapple: attackers count mismatch");
		if (pxKnowledge->m_uNumAttackers > 0)
		{
			--pxKnowledge->m_uNumAttackers;
		}
		if (pxKnowledge->m_uNumAttackGrapple > 0)
		{
			--pxKnowledge->m_uNumAttackGrapple;
		}
		pxKnowledge->m_fTimeSinceGrapple = 0.0f;
	}
}

void AVP_Knowledge_System_AlienTeam::RemoveAttackStand( const Asura_Guid uTargetGuid )
{
	AVP_AlienTeam_Knowledge *pxKnowledge = GetKnowledge(uTargetGuid);
	if (pxKnowledge)
	{
		Asura_Assert(pxKnowledge->m_uNumAttackers > 0 && pxKnowledge->m_uNumAttackStand > 0, "RemoveAttackMelee: attackers count mismatch");
		if (pxKnowledge->m_uNumAttackers > 0)
		{
			--pxKnowledge->m_uNumAttackers;
		}
		if (pxKnowledge->m_uNumAttackStand > 0)
		{
			--pxKnowledge->m_uNumAttackStand;
		}
	}
}

void AVP_Knowledge_System_AlienTeam::RemoveAttackJump( const Asura_Guid uTargetGuid )
{
	AVP_AlienTeam_Knowledge *pxKnowledge = GetKnowledge(uTargetGuid);
	if (pxKnowledge)
	{
		Asura_Assert(pxKnowledge->m_uNumAttackers > 0 && pxKnowledge->m_uNumAttackJump > 0, "RemoveAttackJump: attackers count mismatch");
		if (pxKnowledge->m_uNumAttackers > 0)
		{
			--pxKnowledge->m_uNumAttackers;
		}
		if (pxKnowledge->m_uNumAttackJump > 0)
		{
			--pxKnowledge->m_uNumAttackJump;
		}
	}
}

bool AVP_Knowledge_System_AlienTeam::GetNumAttackers(const Asura_Guid uTargetGuid, u_int& uAttackGrapple, u_int& uAttackStand, u_int& uAttackJump) const
{
	const AVP_AlienTeam_Knowledge *pxKnowledge = GetKnowledge(uTargetGuid);
	Asura_Assert(pxKnowledge, "Target Knowledge not found");
	if (pxKnowledge)
	{
		uAttackGrapple = pxKnowledge->m_uNumAttackGrapple;
		uAttackStand = pxKnowledge->m_uNumAttackStand;
		uAttackJump = pxKnowledge->m_uNumAttackJump;
		return true;
	}
	else
	{
		uAttackGrapple = uAttackStand = 0;
		return false;
	}
}

DEBUGVAR float dbg_fMinDistance = 2.0f;

bool AVP_Knowledge_System_AlienTeam::DidTargetMovedRecently(Asura_Guid uTargetGuid) const
{
	const AVP_AlienTeam_Knowledge *pxKnowledge = GetKnowledge(uTargetGuid);
	Asura_Assert(pxKnowledge, "Target Knowledge not found");
	if (pxKnowledge)
	{
		float fDistnce = 0;
		for (u_int u = 0; u < AVP_AlienTeam_Knowledge::NUM_POSTION_SAMPLES-1; ++u)
		{
			u_int uIndex = u + pxKnowledge->m_uMHIndexStart;
			if (uIndex >= AVP_AlienTeam_Knowledge::NUM_POSTION_SAMPLES)
			{
				uIndex -= AVP_AlienTeam_Knowledge::NUM_POSTION_SAMPLES;
			}

			fDistnce += pxKnowledge->m_fMovementHistory[uIndex];
		}

		return fDistnce > dbg_fMinDistance;
	}
	else
	{
		return true;
	}
}

float AVP_Knowledge_System_AlienTeam::TimeSinceLastGrapple(Asura_Guid uTargetGuid) const
{
	const AVP_AlienTeam_Knowledge *pxKnowledge = GetKnowledge(uTargetGuid);
	Asura_Assert(pxKnowledge, "Target Knowledge not found");
	if (pxKnowledge)
	{
		return pxKnowledge->m_fTimeSinceGrapple;
	}
	else
	{
		return 0.0f;
	}
}


//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

#ifndef ASURA_NO_DEBUG_OUTPUT
bool AVP_Knowledge_System_AlienTeam::Verify()
{
	return true;
}

void AVP_Knowledge_System_AlienTeam::RenderDebugInfo()
{
	Asura_Font_System::Print( m_pxBrain->GetDebugOutputPos().x, m_pxBrain->GetDebugOutputPos().y, 0.75f, 0xffff3535,
		ASURA_TEXTFORMAT_LEFTJUSTIFIED | ASURA_TEXTFORMAT_FORMATSTRING, "Threats: %u", m_xSenseList.GetNumEntries() );
	m_pxBrain->GetDebugOutputPos().y += 10.0f;

	for ( Asura_Hashed_List_ConstIt< AVP_AlienTeam_Knowledge > xKnowIt( &m_xSenseList ); !xKnowIt.Done(); xKnowIt.Next() )
	{
		const AVP_AlienTeam_Knowledge* pxKnow = xKnowIt.GetCurrent();
		if ( !pxKnow ) { continue; }

		Asura_Font_System::Print( m_pxBrain->GetDebugOutputPos().x, m_pxBrain->GetDebugOutputPos().y, 0.75f, 0xffff3535,
			ASURA_TEXTFORMAT_LEFTJUSTIFIED | ASURA_TEXTFORMAT_FORMATSTRING,
			"%u: targeters %u, attackers %u, chargers %u", pxKnow->GetGuid(), pxKnow->m_uNumTargeters, pxKnow->m_uNumAttackers, pxKnow->m_uNumChargers );
		m_pxBrain->GetDebugOutputPos().y += 10.0f;
	}
}
#endif

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

AVP_AlienTeam_Knowledge* AVP_Knowledge_System_AlienTeam::GetKnowledge( Asura_Guid uThreatGuid )
{
	return m_xSenseList.FindEntity( uThreatGuid );
}

const AVP_AlienTeam_Knowledge* AVP_Knowledge_System_AlienTeam::GetKnowledge( Asura_Guid uThreatGuid ) const
{
	return m_xSenseList.FindEntity( uThreatGuid );
}


u_int AVP_Knowledge_System_AlienTeam::GetNumAttackers( const Asura_Guid uTargetGuid ) const
{
	for ( Asura_Hashed_List_It< AVP_AlienTeam_Knowledge > xVisionIt( &m_xSenseList ); !xVisionIt.Done(); xVisionIt.Next() )
	{
		AVP_AlienTeam_Knowledge* pxKnowledge = xVisionIt.GetCurrent();
		if ( pxKnowledge && pxKnowledge->GetGuid() == uTargetGuid )
		{
			return pxKnowledge->m_uNumAttackers;
		}
	}

	return 0;
}

float AVP_Knowledge_System_AlienTeam::GetTimeSinceLastAttack( const Asura_Guid uTargetGuid ) const
{
	for ( Asura_Hashed_List_It< AVP_AlienTeam_Knowledge > xVisionIt( &m_xSenseList ); !xVisionIt.Done(); xVisionIt.Next() )
	{
		AVP_AlienTeam_Knowledge* pxKnowledge = xVisionIt.GetCurrent();
		if ( pxKnowledge && pxKnowledge->GetGuid() == uTargetGuid )
		{
			return Asura_Timers::GetLocalScaledTime() - pxKnowledge->m_fLastAttackFinishTime;
		}
	}

	return 0.0f;
}


void AVP_Knowledge_System_AlienTeam::UpdateSense()
{
	// Invalidate all entites on the list
	for ( Asura_Hashed_List_It< AVP_AlienTeam_Knowledge > xSenseIter( &m_xSenseList ); !xSenseIter.Done(); xSenseIter.Next() )
	{
		AVP_AlienTeam_Knowledge* pxKnowledge = xSenseIter.GetCurrent();
		pxKnowledge->m_bIsDestroyed = true;
	}

	// Go through the actors and add them to our knowledge list.
	const AVP_ServerEntity_Team_Alien* pxAlienTeam = GetAlienTeamBrain()->GetAlienTeam();
	for ( Asura_Hashed_List_ConstIt< AVP_ServerEntity_Actor > xActorIter( AVP_ServerEntity_Actor::GetList() ); !xActorIter.Done(); xActorIter.Next() )
	{
		// Ignore inactive or dead actors.
		const AVP_ServerEntity_Actor* pxActor = xActorIter.GetCurrent();
		if ( !pxActor || !pxActor->IsEntityActive() || pxActor->IsDead() )
		{
			continue;
		}
	
		// Ignore friends.
		if ( AVP_Tactical_System::GetAllegiance( *pxAlienTeam, *pxActor ) != AVP_ALLEGIANCE_ENEMY )
		{
			continue;
		}

		// Add or update this knowledge.
		AVP_AlienTeam_Knowledge* pxExistingKnowledge = m_xSenseList.FindObject( pxActor->GetGuid(), false ); // Check 'destroyed' knowledge as well.
		if ( pxExistingKnowledge )
		{
			// We update the other fields on a per-frame basis, so no need to do it here.
			pxExistingKnowledge->m_bIsDestroyed	= false;
		}
		else
		{
			AVP_AlienTeam_Knowledge* pxKnowledge = CreateKnowledge();
			if ( pxKnowledge )
			{
				pxKnowledge->Reset();

				pxKnowledge->m_uThreatGuid		= pxActor->GetGuid();
				pxKnowledge->m_iThreatClassification = pxActor->GetClassification();
				pxKnowledge->m_xPosition		= pxActor->GetPosition();
				pxKnowledge->m_xVelocity		= pxActor->GetVelocity();
				pxKnowledge->m_fSpeed			= pxKnowledge->m_xVelocity.Magnitude();
				pxKnowledge->m_xOrientation		= pxActor->GetOrientation();
				const AVP_ServerEntity_GamePlayVolume* pxGVP = pxActor->GetCurrentGamePlayVolume();
				pxKnowledge->m_uCurrentGPV		= pxGVP ? pxGVP->GetGuid() : ASURA_GUID_UNREGISTERED;

				// Add it to the knowledge list.
				m_xSenseList.AddToList( pxKnowledge );
			}
		}
	}

	// Remove destroyed entites from list.
	for ( Asura_Hashed_List_Deleter_It< AVP_AlienTeam_Knowledge > xSenseIter( &m_xSenseList ); !xSenseIter.Done(); xSenseIter.Next() )
	{
		AVP_AlienTeam_Knowledge* pxKnowledge = xSenseIter.GetCurrent();
		if ( pxKnowledge->m_bIsDestroyed )
		{
			xSenseIter.DeleteCurrent();
			DestroyKnowledge( pxKnowledge );
		}
	}
}

DEBUGVAR float dbg_fDeltaTimeNextPositionUpdate = 2.0f/ static_cast<float>(AVP_AlienTeam_Knowledge::NUM_POSTION_SAMPLES);

void AVP_Knowledge_System_AlienTeam::PerFrameUpdateSense(float fTimeslice)
{
	for ( Asura_Hashed_List_It< AVP_AlienTeam_Knowledge > xSenseIter( &m_xSenseList ); !xSenseIter.Done(); xSenseIter.Next() )
	{
		AVP_AlienTeam_Knowledge* pxKnowledge = xSenseIter.GetCurrent();

		const AVP_ServerEntity_Actor* pxActor = AVP_ServerEntity_Actor::FindEntity( pxKnowledge->GetGuid() );
		if ( !pxActor ) { continue; }

		pxKnowledge->m_xPosition		= pxActor->GetPosition();
		pxKnowledge->m_xVelocity		= pxActor->GetVelocity();
		pxKnowledge->m_fSpeed			= pxKnowledge->m_xVelocity.Magnitude();
		pxKnowledge->m_xOrientation		= pxActor->GetOrientation();
		const AVP_ServerEntity_GamePlayVolume* pxGVP = pxActor->GetCurrentGamePlayVolume();
		pxKnowledge->m_uCurrentGPV		= pxGVP ? pxGVP->GetGuid() : ASURA_GUID_UNREGISTERED;

		pxKnowledge->m_fTimeTillNextUpdate -= fTimeslice;
		if (pxKnowledge->m_fTimeTillNextUpdate <= 0.0f)
		{
			pxKnowledge->m_fMovementHistory[pxKnowledge->m_uMHIndexStart] = (pxKnowledge->m_xPosition - pxKnowledge->m_xLastPosition).Magnitude();
			pxKnowledge->m_xLastPosition = pxKnowledge->m_xPosition;
			pxKnowledge->m_fTimeTillNextUpdate += dbg_fDeltaTimeNextPositionUpdate;
			++pxKnowledge->m_uMHIndexStart;
			if (pxKnowledge->m_uMHIndexStart >= AVP_AlienTeam_Knowledge::NUM_POSTION_SAMPLES)
			{
				pxKnowledge->m_uMHIndexStart = 0;
			}
		}
		pxKnowledge->m_fTimeSinceGrapple += fTimeslice;
	}
}

void AVP_Knowledge_System_AlienTeam::DestroyAllKnowledge()
{
	Asura_Hashed_List_Deleter_It< AVP_AlienTeam_Knowledge > xSenseIter( &m_xSenseList );
	while ( !xSenseIter.Done() )
	{
		AVP_AlienTeam_Knowledge* pxToDelete = xSenseIter.GetCurrent();
		if ( pxToDelete )
		{
			xSenseIter.DeleteCurrent();
			DestroyKnowledge( pxToDelete );
		}
	}
}
