//////////////////////////////////////////////////////////////////////////////
//
// AVP_Knowledge_System_AlienTeam.h
//
// Copyright (C) 2007 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __AVP_KNOWLEDGE_SYSTEM_ALIENTEAM_H__
#define __AVP_KNOWLEDGE_SYSTEM_ALIENTEAM_H__ 1

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// axon
#include "Knowledge/AVP_Knowledge_System.h"

// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////////

class AVP_Brain_Team_Alien;
class AVP_Sense_AlienTeam_Vision;
class AVP_ServerActor_Alien;

//////////////////////////////////////////////////////////////////////////////
//
// AVP_Knowledge_System_AlienTeam
//
//////////////////////////////////////////////////////////////////////////////

struct AVP_AlienTeam_Knowledge
{
	friend class AVP_Knowledge_System_AlienTeam;

	Asura_Guid			m_uThreatGuid;
	int					m_iThreatClassification;
	Asura_Vector_3		m_xPosition;
	Asura_Vector_3		m_xVelocity;
	float				m_fSpeed;
	Asura_Matrix_3x3	m_xOrientation;

	// Number of people interested in this guid.
	u_int				m_uNumTargeters;
	// Number of people attacking this guid.
	u_int				m_uNumAttackers;
	u_int				m_uNumChargers;
	bool				m_bIsDestroyed;
	float				m_fLastAttackFinishTime;
	Asura_Guid			m_uCurrentGPV;

	u_int				m_uNumAttackGrapple;
	u_int				m_uNumAttackStand;
	u_int				m_uNumAttackJump;
	float				m_fTimeSinceGrapple;

	enum { NUM_POSTION_SAMPLES = 8};
	float				m_fMovementHistory[NUM_POSTION_SAMPLES];
	u_int				m_uMHIndexStart;
	float				m_fTimeTillNextUpdate;
	Asura_Vector_3		m_xLastPosition;

private:
	void UpdateWeaponData() const;

	mutable bool		   m_bWeaponFiring;
	mutable Asura_Vector_3 m_xWeaponPosition;
	mutable Asura_Vector_3 m_xWeaponDirection;
	mutable float          m_fWeaponLOSLength;
	mutable u_int          m_uWeaponFrameLastUpdated;

public:
	void Reset()
	{
		m_uThreatGuid = ASURA_GUID_UNREGISTERED;
		m_iThreatClassification = 0;
		m_xPosition.Zero();
		m_xVelocity.Zero();
		m_fSpeed = 0.0f;
		m_xOrientation.Identity();

		m_uWeaponFrameLastUpdated = 0;

		m_uNumTargeters = 0;
		m_uNumAttackers	= 0;
		m_uNumChargers  = 0;
		m_bIsDestroyed  = false;
		m_fLastAttackFinishTime = 0.0f;
		m_uCurrentGPV = ASURA_GUID_UNREGISTERED;

		m_uNumAttackGrapple = 0;
		m_uNumAttackStand = 0;
		m_uNumAttackJump = 0;

		for (int i = 0; i < NUM_POSTION_SAMPLES; ++i)
		{
			m_fMovementHistory[i] = 0.0f;
		}
		m_uMHIndexStart = 0;
		m_fTimeTillNextUpdate = 0.0f;
		m_xLastPosition.Zero();
		m_fTimeSinceGrapple = Asura_Maths::LargeFloat;
	}

	Asura_Guid GetGuid() const { return m_uThreatGuid; }
	bool IsDestroyed() const { return m_bIsDestroyed; }

	const Asura_Vector_3&	GetPosition()	 const { return m_xPosition; }
	const Asura_Vector_3&	GetVelocity()	 const { return m_xVelocity; }
	float					GetSpeed()		 const { return m_fSpeed; }
	const Asura_Matrix_3x3& GetOrientation() const { return m_xOrientation; }

	bool IsWeaponFiring() const { UpdateWeaponData(); return m_bWeaponFiring; }
	const Asura_Vector_3& GetWeaponPosition() const { UpdateWeaponData(); return m_xWeaponPosition; }
	const Asura_Vector_3& GetWeaponDirection() const { UpdateWeaponData(); return m_xWeaponDirection; }
	float GetWeaponLOSLength() const { UpdateWeaponData(); return m_fWeaponLOSLength; }
};

class AVP_Knowledge_System_AlienTeam : public Axon_Knowledge_System< AVP_AlienTeam_Knowledge >
{
	typedef Axon_Knowledge_System< AVP_AlienTeam_Knowledge > PARENT;

public:
	AVP_Knowledge_System_AlienTeam( AVP_Brain_Team_Alien* pxAlienTeamBrain );
	virtual ~AVP_Knowledge_System_AlienTeam();

	virtual void Initialise() {}
	virtual void Update( const float fTimeslice );
	void PerFrameUpdate( const float fTimeslice );
	virtual void Shutdown();

	virtual void ReadFromChunkStream( Asura_Chunk_Stream& xStream );
	virtual void WriteToChunkStream( Asura_Chunk_Stream& xStream ) const;

	AVP_Brain_Team_Alien* GetAlienTeamBrain() const { return m_pxBrain; }

	void AddAttacker( Asura_Guid uAttackerGuid, const Asura_Guid uTargetGuid );
	void RemoveAttacker( Asura_Guid uAttackerGuid, const Asura_Guid uTargetGuid, bool bAttackAborted = false );
	void AddTargeter( Asura_Guid uTargeterGuid, const Asura_Guid uTargetGuid );
	void RemoveTargeter( Asura_Guid uTargeterGuid, const Asura_Guid uTargetGuid );
	u_int GetNumAttackers( Asura_Guid uTargetGuid ) const;
	float GetTimeSinceLastAttack( Asura_Guid uTargetGuid ) const;

	void AddCharger( Asura_Guid uTargetGuid );
	void RemoveCharger( Asura_Guid uTargetGuid );
	u_int GetNumChargers( Asura_Guid uTargetGuid ) const;

	void AddAttackGrapple( Asura_Guid uTargetGuid );
	void AddAttackStand( Asura_Guid uTargetGuid );
	void AddAttackJump( Asura_Guid uTargetGuid );
	void RemoveAttackGrapple( Asura_Guid uTargetGuid );
	void RemoveAttackStand( Asura_Guid uTargetGuid );
	void RemoveAttackJump( Asura_Guid uTargetGuid );
	bool GetNumAttackers(Asura_Guid uTargetGuid, u_int& uAttackGrapple, u_int& uAttackStand, u_int& uAttackJump) const;
	bool DidTargetMovedRecently(Asura_Guid uTargetGuid) const;
	float TimeSinceLastGrapple(Asura_Guid uTargetGuid) const;

	AVP_AlienTeam_Knowledge* GetKnowledge( Asura_Guid uThreatGuid );
	const AVP_AlienTeam_Knowledge* GetKnowledge( Asura_Guid uThreatGuid ) const;
	const Asura_Hashed_List< AVP_AlienTeam_Knowledge >& GetAllKnowledge() const { return m_xSenseList; }

#ifndef ASURA_NO_DEBUG_OUTPUT
	virtual bool Verify();
	virtual void RenderDebugInfo();
#endif

private:
	void UpdateSense();
	void PerFrameUpdateSense(float fTimeslice);
	void DestroyAllKnowledge();

protected:
	AVP_Brain_Team_Alien* m_pxBrain;

private:
	Asura_Hashed_List< AVP_AlienTeam_Knowledge > m_xSenseList;

	enum { CURRENT_CHUNK_VERSION = 3 };
};

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

#endif //__AVP_KNOWLEDGE_SYSTEM_ALIENTEAM_H__
