//////////////////////////////////////////////////////////////////////////////
//
// AVP_Knowledge_System_Predator.h
//
// Copyright (C) 2007 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __AVP_KNOWLEDGE_SYSTEM_ACTOR_PREDATOR_H__
#define __AVP_KNOWLEDGE_SYSTEM_ACTOR_PREDATOR_H__ 1

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Knowledge/AVP_Knowledge_System.h"


// axon
#include "Knowledge/SenseTypes/AVP_Sense_Vision_Predator.h"

// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////////

class Asura_Chunk_Stream;
class AVP_Brain_Predator;
class AVP_Sense_Hearing_Predator;

//////////////////////////////////////////////////////////////////////////////
//
// AVP_Knowledge_System_Predator
//
//////////////////////////////////////////////////////////////////////////////

class AVP_Knowledge_System_Predator : public AVP_Knowledge_System
{
    typedef AVP_Knowledge_System PARENT;

public:

	AVP_Knowledge_System_Predator( AVP_Brain_Actor* pxBrain );
	~AVP_Knowledge_System_Predator();

	friend class AVP_Sense_Hearing_Predator;
	friend class AVP_Sense_Vision_Predator;

#ifndef ASURA_NO_DEBUG_OUTPUT
	virtual bool Verify();
#endif
    virtual void ReadFromChunkStream			( Asura_Chunk_Stream& xStream );
    virtual void WriteToChunkStream				( Asura_Chunk_Stream& xStream ) const;
	virtual void Update							( const float fTimeslice );
	virtual void Initialise						( void );

	void UpdateBestKnowledge					( const float fTimeslice );
	static bool UpdatePlayerAlienAlertStatus	( void );

	void AddKnowledgeIfNeeded					( const Axon_Knowledge& xNewKnowledge );
	void AddAlertKnowledgeIfNeeded				( const Axon_Knowledge& xNewKnowledge );
	void DeleteCurrentAlertKnowledge			( const Asura_Guid uAlertGuid );

	virtual Axon_Sense_Vision*  CreateVisionSense();
	virtual Axon_Sense_Hearing* CreateHearingSense();
   
    const Axon_Knowledge& GetBestThreatKnowledge() const { return m_xBestThreatKnowledge; }
	const Axon_Knowledge& GetBestAlertKnowledge() const	{ return m_xBestAlertKnowledge; }

	void ResetAlertTimer() { m_fAlertLevelTimer = 0.0f; }

	u_int GetNumSecondaryThreats() const { return m_uNumSecondaryThreats; }

	const Axon_Knowledge& GetSecondaryThreatKnowledge( u_int uThreatId ) const { return m_pxSecondaryThreatKnowledge[uThreatId]; }
	
    AVP_Brain_Actor* GetActorBrain() const;

	void SetVisionFOV( const float fTanHorHalfFOV, const float fTanVertUpHalfFOV, const float fTanVertDownHalfFOV );

	void SetShootAtTarget( Asura_Guid uGuid, bool bIsShootAtNode );
	void ResetShootAtTarget();
	bool IsBestThreatAShootAtNode() const;

	const Axon_Knowledge* CanSeeNow(Asura_Guid uGuid) const;
	const Axon_Knowledge* CanSee(Asura_Guid uGuid) const;
	// const Axon_Knowledge* CanHear(Asura_Guid uGuid) const;

#ifndef ASURA_NO_DEBUG_OUTPUT
	virtual void RenderDebugInfo();
#endif

	void SetLongMemoryEntity(Asura_Guid uGuid); // mark an entity to be remembered even after the maximum time

	AVP_Sense_Vision_Predator* GetPredatorVision() { return static_cast<AVP_Sense_Vision_Predator*>(m_pxVision); }
	const AVP_Sense_Vision_Predator* GetPredatorVision() const { return static_cast<AVP_Sense_Vision_Predator const*>(m_pxVision); }

protected:

	void ClearSecondaryThreatArray();
	void BuildSecondaryThreatArray( u_int uNumEntries );

	// Strictly sounds from the actors enemies
	Axon_Knowledge      m_xBestThreatKnowledge;
	// store 4 secondary threats
	Axon_Knowledge*		m_pxSecondaryThreatKnowledge;
	u_int				m_uNumSecondaryThreats;
	// Threatening sounds from non-enemies, such as screaming, gunfire, etc
	Axon_Knowledge      m_xBestAlertKnowledge;
	float				m_fAlertLevelTimer;
	
	Asura_Guid			m_uShootAtGuid;
	bool				m_bIsTargetAShootAtNode;

	static float		s_fAggressiveToActiveTime;
	static float		s_fActiveToPassiveTime;


private:

    enum { CURRENT_CHUNK_VERSION = 0 };
};

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

#endif //__AVP_KNOWLEDGE_SYSTEM_ACTOR_PREDATOR_H__
