//////////////////////////////////////////////////////////////////////////////
//
// AVP_Knowledge_System_Sentrygun.cpp
//
// Copyright (C) 2007 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"

// header
#include "Knowledge/AVP_Knowledge_System_Sentrygun.h"

// asura
#include "ChunkSystem/Asura_Chunk_Stream.h"

// avp
#include "Core/BrainTypes/AVP_Brain_Actor.h"
// #include "Entity/AVP_Entity_Classification.h"
// #include "Entity/PhysicalObject/Actor/Vertebrates/Alien/AVP_ServerActor_Alien.h"
// #include "Entity/PhysicalObject/Actor/Vertebrates/Human/AVP_ServerActor_Human.h"

// axon
#include "Knowledge/Axon_Knowledge.h"
//#include "Knowledge/SenseTypes/AVP_Sense_Hearing_Actor.h"
// #include "Tactical/AVP_Tactical_System.h"
#include "Core/AVP_Waypoint_Manager.h"
// #include "Core/BrainTypes/AVP_Brain_Actor_Civilian.h"
// #include "Core/BrainTypes/AVP_Brain_Actor_Marine.h"
// #include "Core/BrainTypes/AVP_Brain_Actor_Predator.h"
// #include "Core/BrainTypes/AVP_Brain_Actor_Sentrygun.h"

static const u_int g_uCurrentChunkVersion = 1;

AVP_Knowledge_System_Sentrygun::AVP_Knowledge_System_Sentrygun(AVP_Brain_Actor* pxBrain)
: PARENT(pxBrain)
, m_xBestThreatKnowledge()
{
	m_xBestThreatKnowledge.Reset();
}

AVP_Knowledge_System_Sentrygun::~AVP_Knowledge_System_Sentrygun()
{

}

#ifndef ASURA_NO_DEBUG_OUTPUT
bool AVP_Knowledge_System_Sentrygun::Verify()
{
	Asura_Assert( m_pxVision, "AVP_Knowledge_System_Sentrygun::Verify - no vision" );
	return ( PARENT::Verify() && m_pxVision );
}
#endif

void AVP_Knowledge_System_Sentrygun::Initialise()
{
	PARENT::Initialise();

	m_xBestThreatKnowledge.Reset();
}

void AVP_Knowledge_System_Sentrygun::ReadFromChunkStream( Asura_Chunk_Stream& xStream )
{
	u_int uVersion;
	xStream >> uVersion;

	switch ( uVersion )
	{
		case 0:
		{
			Asura_Assert( false, "AVP_Knowledge_System_Sentrygun never had a chunk version 0" );
		}
		break;

		case 1:
		{
			PARENT::ReadFromChunkStream( xStream );
		}
		break;

		default:
		{
			Asura_Assertf( false, ( "Unhandled chunk version (%d)", uVersion ) );
		}
	}
}

void AVP_Knowledge_System_Sentrygun::WriteToChunkStream( Asura_Chunk_Stream& xStream ) const
{
	xStream << g_uCurrentChunkVersion;

	PARENT::WriteToChunkStream( xStream );
}

void AVP_Knowledge_System_Sentrygun::Update(const float fTimeslice)
{
	PARENT::Update(fTimeslice);

	UpdateBestThreatKnowledge(fTimeslice);
}

void AVP_Knowledge_System_Sentrygun::UpdateBestThreatKnowledge(const float fTimeslice)
{
	AVP_ServerEntity_Actor* pxOurActor = GetActorBrain()->GetActor();

	if (!pxOurActor)
	{
		return;
	}

	m_xBestThreatKnowledge.Reset();

	float fClosestVisionThreatDist	= Asura_Maths::LargeFloat;

	// Vision!
	for ( Asura_Hashed_List_ConstIt< Axon_Knowledge > xVisionIt( m_pxVision->GetList() ); !xVisionIt.Done(); xVisionIt.Next() )
	{
		// Use the closest enemy entity we can see as our threat for now
		const Axon_Knowledge* pxKnowledge = xVisionIt.GetCurrent();
		Asura_Assert( pxKnowledge, "AVP_Knowledge_System::UpdateBestKnowledge: invalid vision knowledge" );

		if ( pxKnowledge )
		{
			if ( pxKnowledge->m_fDistance < fClosestVisionThreatDist )
			{
				fClosestVisionThreatDist	= pxKnowledge->m_fDistance;
				m_xBestThreatKnowledge		= *pxKnowledge;
			}
		}
	}
}
