//////////////////////////////////////////////////////////////////////////////
//
// AVP_Sense_Vision.cpp
//
// Created by NJD - 06 December 2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// Core
#include "Core/Asura.h"

// This files header
#include "Knowledge/SenseTypes/AVP_Sense_Vision_Actor.h"

#include "Debugging/Asura_DebugLineRender.h"

// Asura
#include "Blueprint/Asura_Blueprint.h"
#include "Blueprint/Asura_Blueprint_System.h"
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "Entity/PhysicalObject/Moves/Asura_Movement_System.h"

#ifndef ASURA_NO_DEBUG_OUTPUT
#include "Rendering/Asura_Font_System.h"
#endif

// Axon
#include "Behaviour/BehaviourTypes/AVP_Behaviour_Types.h"
#include "Core/Axon_Project_System.h"
#include "Core/AVP_Waypoint_Manager.h"
#include "Knowledge/AVP_Knowledge_System_Actor.h"
#include "Knowledge/AVP_Knowledge_System_Sentrygun.h"
#include "Knowledge/Axon_Knowledge.h"
#include "Knowledge/SenseTypes/Axon_Sense_Types.h"
#include "Tactical/AVP_Tactical_System.h"
#include "Core/BrainTypes/AVP_Brain_Actor_Civilian.h"
#include "Core/BrainTypes/AVP_Brain_Actor_Marine.h"
#include "Core/BrainTypes/AVP_Brain_Actor_SentryGun.h"

// AVP
#include "Core/AVP_Game_Core.h"
#include "Core/AVP_Settings.h"
#include "Entity/AVP_Entity_Classification.h"
#include "Entity/Controller/AVP_ServerEntity_Team_Marine.h"
#include "Entity/PhysicalObject/Actor/AVP_ClientEntity_Actor.h"
#include "Entity/PhysicalObject/Actor/AVP_ServerEntity_Actor.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Alien/AVP_ServerActor_Alien.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Predator/AVP_ServerActor_Predator.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Human/AVP_ClientActor_Human.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Human/AVP_ServerActor_Human.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Predator/AVP_ClientActor_Predator.h"
#include "Network/AVP_Player.h"

//////////////////////////////////////////////////////////////////////////////
//
// AVP_Sense_Vision
//
//////////////////////////////////////////////////////////////////////////////

#ifndef ASURA_NO_DEBUG_OUTPUT
Asura_Guid AVP_Sense_Vision::s_uRenderGuid = 999;
float AVP_Sense_Vision::s_fTotalScore = 0.0f;
float AVP_Sense_Vision::s_fSpeedScore = 0.0f;
float AVP_Sense_Vision::s_fRangeScore = 0.0f;
float AVP_Sense_Vision::s_fFOVScore = 0.0f;
float AVP_Sense_Vision::s_fLightScore = 0.0f;
float AVP_Sense_Vision::s_fPassiveThreshold = 0.0f;
float AVP_Sense_Vision::s_fActiveThreshold = 0.0f;
float AVP_Sense_Vision::s_fAggressiveThreshold = 0.0f;
#endif

//---------------------------------------------------------------------------
// AVP_Sense_Vision :: Constructor
//---------------------------------------------------------------------------
AVP_Sense_Vision::AVP_Sense_Vision(AVP_Knowledge_System_Actor* pxAVPKnowledgeSystem)
: PARENT( pxAVPKnowledgeSystem )
, m_bEnemyLaserActive( false )
, m_xLaserPos( Asura_Maths::ZeroVector3 )
, m_xLaserDirection( Asura_Maths::ZeroVector3 )
, m_fLaserRange( 0.0f )
, m_uLaserOwnerGuid( ASURA_GUID_UNREGISTERED )
, m_bIsInShadow_LastFrame( true )
, m_fAlienPlayerScore(0.0f)
, m_fAlienPlayerScoreToBeat(200.0f) // initialise to passive score to beat
{
    m_fRange       = 200.0f;
    m_fTanFovX     = Asura_Maths::Tangent(Asura_Maths::DegToRad(170.0f) * 0.5f);
    m_fTanFovYUp   = Asura_Maths::Tangent(Asura_Maths::DegToRad(130.0f) * 0.25f);
    m_fTanFovYDown = Asura_Maths::Tangent(Asura_Maths::DegToRad(110.0f) * 0.6f);

	m_xKnowledgeSettings.m_fPassiveCloakingThreshold = 700;
	m_xKnowledgeSettings.m_fActiveCloakingThreshold = 400;
	m_xKnowledgeSettings.m_fAggressiveCloakingThreshold = 300;
}

//---------------------------------------------------------------------------
// AVP_Sense_Vision :: Deconstructor
//---------------------------------------------------------------------------
AVP_Sense_Vision::~AVP_Sense_Vision() 
{
	DestroyAllKnowledge();
}

//---------------------------------------------------------------------------
// AVP_Sense_Vision :: InitialiseFromBlueprint
//---------------------------------------------------------------------------
void AVP_Sense_Vision::InitialiseFromBlueprint(const Asura_Blueprint& xBlueprint)
{
	PARENT::InitialiseFromBlueprint(xBlueprint);

	// We also need to initialise from our AI settings blueprint currently.
	InitFromSettingsBlueprint();
}

//---------------------------------------------------------------------------
// AVP_Sense_Vision :: ReadFromChunkStream
//---------------------------------------------------------------------------
void AVP_Sense_Vision::ReadFromChunkStream(Asura_Chunk_Stream& xStream)
{
    u_int uVersion;
    xStream >> uVersion;

    PARENT::ReadFromChunkStream(xStream);

	if ( uVersion == uCURRENT_CHUNK_VERSION )
	{
		xStream >> m_xKnowledgeSettings.m_fPassiveRange;
		xStream >> m_xKnowledgeSettings.m_fActiveRange;
		xStream >> m_xKnowledgeSettings.m_fAggressiveRange;
		xStream >> m_xKnowledgeSettings.m_fPassiveThreshold;
		xStream >> m_xKnowledgeSettings.m_fActiveThreshold;
		xStream >> m_xKnowledgeSettings.m_fAggressiveThreshold;
		xStream >> m_xKnowledgeSettings.m_fCloakedVisibleRangeSq;
		xStream >> m_xKnowledgeSettings.m_fKnowledgeLifetime;
		xStream >> m_xKnowledgeSettings.m_fVisionCutOffDistanceSq;
		xStream >> m_xKnowledgeSettings.m_fDarknessLightThreshold;
		xStream >> m_xKnowledgeSettings.m_fBrightnessLightThreshold;
		xStream >> m_xKnowledgeSettings.m_fDarknessLightScore;
		xStream >> m_xKnowledgeSettings.m_fBrightnessLightScore;
		xStream >> m_xKnowledgeSettings.m_fNeutralLightScore;
		xStream >> m_xKnowledgeSettings.m_fPassiveCloakingThreshold;
		xStream >> m_xKnowledgeSettings.m_fActiveCloakingThreshold;
		xStream >> m_xKnowledgeSettings.m_fAggressiveCloakingThreshold;

	}
	else if ( uVersion == 1 )
	{
		xStream >> m_xKnowledgeSettings.m_fPassiveRange;
		xStream >> m_xKnowledgeSettings.m_fActiveRange;
		xStream >> m_xKnowledgeSettings.m_fAggressiveRange;
		xStream >> m_xKnowledgeSettings.m_fPassiveThreshold;
		xStream >> m_xKnowledgeSettings.m_fActiveThreshold;
		xStream >> m_xKnowledgeSettings.m_fAggressiveThreshold;
		xStream >> m_xKnowledgeSettings.m_fCloakedVisibleRangeSq;
		xStream >> m_xKnowledgeSettings.m_fKnowledgeLifetime;
		xStream >> m_xKnowledgeSettings.m_fVisionCutOffDistanceSq;
		xStream >> m_xKnowledgeSettings.m_fDarknessLightThreshold;
		xStream >> m_xKnowledgeSettings.m_fBrightnessLightThreshold;
		xStream >> m_xKnowledgeSettings.m_fDarknessLightScore;
		xStream >> m_xKnowledgeSettings.m_fBrightnessLightScore;
		xStream >> m_xKnowledgeSettings.m_fNeutralLightScore;

		m_xKnowledgeSettings.m_fPassiveCloakingThreshold = 500.0f;
		m_xKnowledgeSettings.m_fActiveCloakingThreshold = 350.0f;
		m_xKnowledgeSettings.m_fAggressiveCloakingThreshold = 200.0f;
	}
	else if ( uVersion == 0 )
	{
	}
	else
	{
		Asura_Assertf(false, ("AVP_Sense_Vision::ReadFromChunkStream - unhandled chunk version (%u)", uVersion));
	}

	m_fRange = m_xKnowledgeSettings.m_fPassiveRange;
}

//---------------------------------------------------------------------------
// AVP_Sense_Vision :: WriteToChunkStream
//---------------------------------------------------------------------------
void AVP_Sense_Vision::WriteToChunkStream(Asura_Chunk_Stream& xStream) const
{
    xStream << uCURRENT_CHUNK_VERSION;

    PARENT::WriteToChunkStream(xStream);

	// Could just save the blueprint that we initialised these values from. Change this if our chunks are too large.
	xStream << m_xKnowledgeSettings.m_fPassiveRange;
	xStream << m_xKnowledgeSettings.m_fActiveRange;
	xStream << m_xKnowledgeSettings.m_fAggressiveRange;
	xStream << m_xKnowledgeSettings.m_fPassiveThreshold;
	xStream << m_xKnowledgeSettings.m_fActiveThreshold;
	xStream << m_xKnowledgeSettings.m_fAggressiveThreshold;
	xStream << m_xKnowledgeSettings.m_fCloakedVisibleRangeSq;
	xStream << m_xKnowledgeSettings.m_fKnowledgeLifetime;
	xStream << m_xKnowledgeSettings.m_fVisionCutOffDistanceSq;
	xStream << m_xKnowledgeSettings.m_fDarknessLightThreshold;
	xStream << m_xKnowledgeSettings.m_fBrightnessLightThreshold;
	xStream << m_xKnowledgeSettings.m_fDarknessLightScore;
	xStream << m_xKnowledgeSettings.m_fBrightnessLightScore;
	xStream << m_xKnowledgeSettings.m_fNeutralLightScore;
	xStream << m_xKnowledgeSettings.m_fPassiveCloakingThreshold;
	xStream << m_xKnowledgeSettings.m_fActiveCloakingThreshold;
	xStream << m_xKnowledgeSettings.m_fAggressiveCloakingThreshold;
}

//---------------------------------------------------------------------------
// AVP_Sense_Vision :: Update
//---------------------------------------------------------------------------
void AVP_Sense_Vision::Update(const float fTimeslice) 
{
	// Reset these every update
	m_fAlienPlayerScore = 0.0f;
	m_fAlienPlayerScoreToBeat = GetAlertLevelScoreToBeat();

	AVP_ServerEntity_Actor* pxServerActor = GetActorKnowledgeSystem()->GetActorBrain()->GetActor();

	m_fRange		= GetAlertLevelRange();

	AVP_ClientEntity_Actor* pxClient = AVP_ClientEntity_Actor::FindEntity(pxServerActor->GetGuid());

	// For cases of spawned actors make sure we have started to render them before trying to use them for knowledge.
	if (pxClient && pxClient->GetShapeInstance().GetAnimation() && pxClient->GetShapeInstance().GetAnimation()->GetTweenStore())
	{
		if (pxClient->GetShapeInstance().GetAnimation()->GetTweenStore()->IsPositionDataValid())
		{
			pxServerActor->GetHeadPosition(m_xPosition);
			pxServerActor->GetHeadFowardVector(m_xDirection);
			DepricateCurrentKnowledge(fTimeslice);
			AddNewKnowledge();
		}
	}
}

#ifndef ASURA_NO_DEBUG_OUTPUT

void AVP_Sense_Vision::RenderKnowledgeDebugInfo(Asura_Guid uGuid)
{
	AVP_ServerEntity_Actor* pxActor = AVP_ServerEntity_Actor::FindEntity( uGuid );

	if ( pxActor )
	{
		u_int uFlags = ASURA_TEXTFORMAT_CENTREJUSTIFIED | ASURA_TEXTFORMAT_FORMATSTRING;
		Asura_Font_System::Print( 20.0f, 20.0f, 1.0f, 0xffffffff, uFlags, "Score: %0.2f", s_fTotalScore );
	}
}
#endif

//---------------------------------------------------------------------------
// AVP_Sense_Vision :: DepricateCurrentKnowledge
//---------------------------------------------------------------------------
void AVP_Sense_Vision::DepricateCurrentKnowledge(const float fTimeslice)
{
	AVP_ServerEntity_Actor* pxServerActor = GetActorKnowledgeSystem()->GetActorBrain()->GetActor();

	// Age the vision knowledge. Cull any that are too old
	Asura_Hashed_List_Deleter_It<Axon_Knowledge> xVisionIter(&m_xVisionList);

	while (!xVisionIter.Done())
	{
		Axon_Knowledge* pxKnowledge = xVisionIter.GetCurrent();
		const AVP_ServerEntity_Actor* pxActor = AVP_ServerEntity_Actor::FindEntity(pxKnowledge->GetGuid());

		bool bDeleteKnowledge = false;

		// Actor validation, pointer check, dead check is active check
		if (!pxActor || pxActor->IsDead() || !pxActor->IsEntityActive())
		{
			bDeleteKnowledge = true;
		}
		// Check if threat is cloaked and close enough to actually be seen
		else if ( pxActor->IsCloaked(true) && pxKnowledge->m_uTypeInformation != TYPE_INFO_LASER )
		{
			float fActorThreatScore = 0.0f;

			// redo the score
			// Get the score for the speed the actor is moving
			fActorThreatScore += GetScoreForSpeed(pxActor);

			// Get the score for the range of the actor
			fActorThreatScore += GetScoreForRange(pxServerActor->GetPosition(), pxActor->GetPosition());

			// Get the score for the position in the FOV
			fActorThreatScore += GetScoreForFOVPosition(pxServerActor, pxActor->GetPosition());

			// have we beaten our current alertness levels adding knowledge score
			if ( fActorThreatScore <= ( GetCloakingAlertLevelScoreToBeat() * 0.7f ) )
			{
				bDeleteKnowledge = true;
			}
		}
		else
		{
			if ( pxKnowledge->m_uTypeInformation == TYPE_INFO_LASER && !m_bEnemyLaserActive )
			{
				bDeleteKnowledge = true;
			}
			
			// Alien player in darkness cannot be seen.
			if ( pxActor->GetClassification() == AVP_ENTITYCLASS_ALIEN )
			{
				if ( IsInDarkness( pxActor->GetGuid() ) )
				{
					m_fLightKnowledgeLifetime = Axon_Project_System::GetLightCutOffTimerOut();
				}
				else
				{
					m_fLightKnowledgeLifetime = m_xKnowledgeSettings.m_fKnowledgeLifetime;
				}
			}
			else
			{
				m_fLightKnowledgeLifetime = m_xKnowledgeSettings.m_fKnowledgeLifetime;
			}

			if (AVP_Tactical_System::GetAllegiance(*pxServerActor, *pxActor) != AVP_ALLEGIANCE_ENEMY)
			{
				bDeleteKnowledge = true;
			}

			if ( GetActorKnowledgeSystem()->GetTargetPriority( pxActor->GetGuid() ) == AVP_AI_TargetPriorityData::IGNORE_P )
			{
				bDeleteKnowledge = true;
			}
		}

		// Knowledge time out
		pxKnowledge->m_fAge += fTimeslice;

		if ( pxKnowledge->m_fAge >= m_fLightKnowledgeLifetime )
		{
			bDeleteKnowledge = true;
		}

		if (pxKnowledge->m_fAge >= m_xKnowledgeSettings.m_fKnowledgeLifetime)
		{
			bDeleteKnowledge = true;
		}

		if (bDeleteKnowledge)
		{
			xVisionIter.DeleteCurrent();
			m_pxKnowledgeSystem->DestroyKnowledge(pxKnowledge);
		}
		else 
		{
			// Update stuff for alien player stealth indicator
			if ( pxActor && pxActor->GetClassification() == AVP_ENTITYCLASS_ALIEN && pxActor->IsPlayer())
			{
				m_fAlienPlayerScore = GetAlertLevelScoreToBeat();
				m_fAlienPlayerScoreToBeat = GetAlertLevelScoreToBeat();
			}

			xVisionIter.Next();
		}
	}
}

//---------------------------------------------------------------------------
// AVP_Sense_Vision :: AddNewKnowledge
//---------------------------------------------------------------------------
void AVP_Sense_Vision::AddNewKnowledge(void)
{
	AVP_ServerEntity_Actor* pxServerActor = GetActorKnowledgeSystem()->GetActorBrain()->GetActor();

	for (Asura_Hashed_List_ConstIt<AVP_ServerEntity_Actor> xIt(AVP_ServerEntity_Actor::GetList()); !xIt.Done(); xIt.Next())
	{
		const AVP_ServerEntity_Actor*	pxActor = xIt.GetCurrent();		

		// isActor Valid
		if (!pxActor)
			continue;

		// Ignore ourselves.
		if (pxServerActor->GetGuid() == pxActor->GetGuid())
			continue;

		// is the actor dead or inactive, don't remove from the list as he will be removed next time
		if (pxActor->IsDead() || !pxActor->IsEntityActive())
			continue;

		if ( GetActorKnowledgeSystem()->GetTargetPriority( pxActor->GetGuid() ) == AVP_AI_TargetPriorityData::IGNORE_P )
			continue;

		AVP_ClientEntity_Actor* pxClient = AVP_ClientEntity_Actor::FindEntity(pxActor->GetGuid());

		// For cases of spawned actors make sure we have started to render them before trying to use them for knowledge.
		if (pxClient && pxClient->GetShapeInstance().GetAnimation() && pxClient->GetShapeInstance().GetAnimation()->GetTweenStore())
		{
			if (!pxClient->GetShapeInstance().GetAnimation()->GetTweenStore()->IsPositionDataValid())
			{
				continue;
			}
		}
		else
		{
			continue;
		}

		// We need to store things specifically for the player alien
		bool bIsAlienPlayer = false;

		if ((pxActor->GetClassification() == AVP_ENTITYCLASS_ALIEN) && (pxActor->IsPlayer()))
		{
			bIsAlienPlayer = true;
		}

		// Ignore friends
		if (AVP_Tactical_System::GetAllegiance(*pxServerActor, *pxActor) != AVP_ALLEGIANCE_ENEMY)
		{
			continue;
		}

		// If it is outside our range and FOV then move on to the next
		if (ShouldCullActorOnRangeAndFOV(pxServerActor, pxActor) )
		{
			continue;
		}

		Axon_Knowledge* pxExistingKnowledge = m_xVisionList.FindObject( pxActor->GetGuid() );

		if ( !LOSCheckOnActor(pxActor) )
		{
			if ( pxExistingKnowledge )
			{
				pxExistingKnowledge->m_uTypeInformation = TYPE_INFO_CANT_SEE;
			}

			continue;
		}

		// Alien player in darkness cannot be seen.
		if ( pxActor->GetClassification() == AVP_ENTITYCLASS_ALIEN )
		{
			if ( IsInDarkness( pxActor->GetGuid() ) )
			{
				continue;
			}
		}

		if (pxExistingKnowledge)
		{
			GetActorKnowledgeSystem()->ResetAlertTimer();

			bool	bIgnoreCloakedStatus	= ShouldIgnoreCloakedStatus();
			float	fActorThreatScore		= 0.0f;

			pxExistingKnowledge->m_uTypeInformation = TYPE_INFO_CAN_SEE;

			if ( !bIgnoreCloakedStatus && pxActor->GetClassification() == AVP_ENTITYCLASS_PREDATOR && pxActor->IsCloaked( true ) )
			{
				// Get the score for the speed the actor is moving
				fActorThreatScore += GetScoreForSpeed(pxActor);

				// Get the score for the range of the actor
				fActorThreatScore += GetScoreForRange(pxServerActor->GetPosition(), pxActor->GetPosition());

				// Get the score for the position in the FOV
				fActorThreatScore += GetScoreForFOVPosition(pxServerActor, pxActor->GetPosition());

				// have we beaten our current alertness levels adding knowledge score
				if ( fActorThreatScore <= GetCloakingAlertLevelScoreToBeat() && fActorThreatScore >= ( GetCloakingAlertLevelScoreToBeat() * 0.7f ) )
				{
					pxExistingKnowledge->m_uTypeInformation = TYPE_INFO_CLOAKED_CANT_SEE;
				}

#ifndef ASURA_NO_DEBUG_OUTPUT
				s_fTotalScore = fActorThreatScore;
#endif
			}

			// Update the old one.
			if ( pxExistingKnowledge->m_fAge > 0.5f )
			{
				pxExistingKnowledge->m_fAge = 0.5f;
			}

			pxActor->GetChestPosition( pxExistingKnowledge->m_xPosition );
			pxExistingKnowledge->m_xDirection		= pxActor->GetPosition() - pxServerActor->GetPosition();
			pxExistingKnowledge->m_fDistance		= pxExistingKnowledge->m_xDirection.Normalise();

			// Alien player has been spotted so set the score to indicate as such.
			if (bIsAlienPlayer)
			{
				m_fAlienPlayerScore = 200.0f;
				m_fAlienPlayerScoreToBeat = 100.0f;
			}
		}
		else
		{
			bool	bAddNewKnowledge	= false;
			float	fActorThreatScore	= 0.0f;

			//---------------------------------------------------------------------------------------------------------------
			//	Threat Score is determined by three things
			// 
			//	1.) The speed the actor is moving, returns a value between 0.0f and 100.0f.
			//	2.) The score for the range from our actor, returns a value between 0.0f and 100.0f.
			//	3.) The position in our actors FOV returns a smaller value if the actor is out to the edges of our vision -
			//		Returns a value between 0.0f and 100.0f.
			//---------------------------------------------------------------------------------------------------------------

			bool bIgnoreCloakedStatus = ShouldIgnoreCloakedStatus();

			if ( !bIgnoreCloakedStatus && pxActor->GetClassification() == AVP_ENTITYCLASS_PREDATOR && pxActor->IsCloaked( true ) )
			{
				// do pred cloaking stuff here

				// Get the score for the speed the actor is moving
				fActorThreatScore += GetScoreForSpeed(pxActor);

				// Get the score for the range of the actor
				fActorThreatScore += GetScoreForRange(pxServerActor->GetPosition(), pxActor->GetPosition());

				// Get the score for the position in the FOV
				fActorThreatScore += GetScoreForFOVPosition(pxServerActor, pxActor->GetPosition());

				// have we beaten our current alertness levels adding knowledge score
				if ( fActorThreatScore > GetCloakingAlertLevelScoreToBeat() )
				{
					// Add the new knowledge
					bAddNewKnowledge = true;
				}

#ifndef ASURA_NO_DEBUG_OUTPUT
				s_fTotalScore = fActorThreatScore;
#endif
			}
			else
			{
				// Get the score for the speed the actor is moving
				fActorThreatScore += GetScoreForSpeed(pxActor);

				// Get the score for the range of the actor
				fActorThreatScore += GetScoreForRange(pxServerActor->GetPosition(), pxActor->GetPosition());

				// Get the score for the position in the FOV
				fActorThreatScore += GetScoreForFOVPosition(pxServerActor, pxActor->GetPosition());

				// if the actor we are checking against is the alien player update the threat scores
				if (bIsAlienPlayer)
				{
					m_fAlienPlayerScore			= fActorThreatScore;
					m_fAlienPlayerScoreToBeat	= GetAlertLevelScoreToBeat();
				}

				// have we beaten our current alertness levels adding knowledge score
				if ( fActorThreatScore > GetAlertLevelScoreToBeat() )
				{
					// Add the new knowledge
					bAddNewKnowledge = true;
				}
			}

			if (bAddNewKnowledge)
			{
				if ( GetActorKnowledgeSystem()->GetAlertLevel() != AVP_Knowledge_System_Actor::AGGRESSIVE_ALERTNESS )
				{
					GetActorKnowledgeSystem()->SetAlertLevel( AVP_Knowledge_System_Actor::AGGRESSIVE_ALERTNESS );
					PlayEnteringAggressiveDialogue();
				}

				GetActorKnowledgeSystem()->ResetAlertTimer();

				// Create a new one.
				Axon_Knowledge* pxKnowledge = m_pxKnowledgeSystem->CreateKnowledge();

				if ( pxKnowledge )
				{
					GetActorKnowledgeSystem()->ResetAlertTimer();

					pxActor->GetChestPosition( pxKnowledge->m_xPosition );

					pxKnowledge->m_uKnowledgeID          = m_pxKnowledgeSystem->GetNewKnowledgeID();
					pxKnowledge->m_uSourceGuid           = pxActor->GetGuid();
					pxKnowledge->m_uSourceClassification = pxActor->GetClassification();
					pxKnowledge->m_iSenseID              = AXON_SENSE_TYPE_VISION;
					pxKnowledge->m_uTypeInformation		 = TYPE_INFO_CAN_SEE;
					pxKnowledge->m_fAge                  = 0.0f;
					pxKnowledge->m_fThreatValue          = 0.0f;
					pxKnowledge->m_xPosition             = pxActor->GetPosition();
					pxKnowledge->m_xDirection            = pxActor->GetPosition() - pxServerActor->GetPosition();
					pxKnowledge->m_fDistance             = pxKnowledge->m_xDirection.Normalise();
					m_xVisionList.AddToList( pxKnowledge );
				}
			}
		}
	}

	ProcessPredatorLaser();
}

//---------------------------------------------------------------------------
// AVP_Sense_Vision :: PlayEnteringAggressiveDialogue
//---------------------------------------------------------------------------
void AVP_Sense_Vision::PlayEnteringAggressiveDialogue()
{
	AVP_ServerEntity_Actor* pxServerActor = GetActorKnowledgeSystem()->GetActorBrain()->GetActor();

	if ((pxServerActor->GetClassification() == AVP_ENTITYCLASS_MARINE) || (pxServerActor->GetClassification() == AVP_ENTITYCLASS_ANDROID) || (pxServerActor->GetClassification() == AVP_ENTITYCLASS_CIVILIAN))
	{
		AVP_ClientActor_Human* pxClientActor = AVP_ClientActor_Human::FindEntity( pxServerActor->GetGuid() );

		if ( pxClientActor )
		{
			if ( pxClientActor->GetMovementSystem() )
			{
				if ( pxClientActor->GetMovementSystem()->GetCurrentMove() == NULL )
				{
					((AVP_ServerActor_Human*)pxServerActor)->PlayDialogue(AVP_ServerActor_Human::AVP_DLG_HUMAN_ENTERING_AGGRESSIVE);
				}
			}
		}
	}
}

//---------------------------------------------------------------------------
// AVP_Sense_Vision :: ProcessPRedatorLaser
//---------------------------------------------------------------------------
void AVP_Sense_Vision::ProcessPredatorLaser()
{
	AVP_ServerEntity_Actor* pxServerActor = GetActorKnowledgeSystem()->GetActorBrain()->GetActor();

	m_bEnemyLaserActive	= false;
	m_xLaserPos			= Asura_Maths::ZeroVector3;
	m_xLaserDirection	= Asura_Maths::ZeroVector3;
	m_fLaserRange		= 0.0f;
	m_uLaserOwnerGuid	= ASURA_GUID_UNREGISTERED;

	AVP_Player* pxPlayer = AVP_Game_Core::GetPlayer();

	if ( pxPlayer && pxPlayer->GetPuppetSpecies() == AVP_Settings::AVP_SPECIES_PREDATOR )
	{
		AVP_ClientActor_Predator* pxPlayerPredator = AVP_ClientActor_Predator::FindEntity( pxPlayer->GetPuppetGuid() );

		if ( pxPlayerPredator && pxPlayerPredator->IsUsingLaser() )
		{
			m_bEnemyLaserActive	= true;
			m_xLaserPos			= pxPlayerPredator->GetLaserPosition();
			m_xLaserDirection	= pxPlayerPredator->GetLaserDirection();
			m_fLaserRange		= pxPlayerPredator->GetLaserRange();
			m_uLaserOwnerGuid	= pxPlayerPredator->GetGuid();
		}
	}

	if ( m_bEnemyLaserActive )
	{
		AVP_ClientActor_Predator* pxPlayerPredator = AVP_ClientActor_Predator::FindEntity( m_uLaserOwnerGuid );
		Axon_Knowledge* pxExistingKnowledge = m_xVisionList.FindObject( m_uLaserOwnerGuid );

		if ( pxPlayerPredator )
		{
			if ( !pxExistingKnowledge )
			{
				Asura_Vector_3 xDir = m_xLaserPos - pxServerActor->GetPosition();
				float fDistance = xDir.Normalise();
				Asura_Vector_3 xLaserEnd( m_xLaserPos + m_xLaserDirection * m_fLaserRange );

				float fDistToLaserEndSq = (pxServerActor->GetPosition() - xLaserEnd).MagnitudeSquared();

				if ( fDistToLaserEndSq < GetAlertLevelRange() )
				{
					Axon_Knowledge* pxNewKnowledge = m_pxKnowledgeSystem->CreateKnowledge();

					if ( pxNewKnowledge )
					{
						pxNewKnowledge->m_uKnowledgeID          = m_pxKnowledgeSystem->GetNewKnowledgeID();
						pxNewKnowledge->m_uSourceGuid           = m_uLaserOwnerGuid;
						pxNewKnowledge->m_uSourceClassification = AVP_ENTITYCLASS_PREDATOR;
						pxNewKnowledge->m_iSenseID              = AXON_SENSE_TYPE_VISION;
						pxNewKnowledge->m_uTypeInformation		= AVP_Sense_Vision::TYPE_INFO_LASER;
						pxNewKnowledge->m_fAge                  = 0.0f;
						pxNewKnowledge->m_fThreatValue          = 0.0f;
						pxNewKnowledge->m_xPosition             = m_xLaserPos;
						pxNewKnowledge->m_xDirection            = xDir;
						pxNewKnowledge->m_fDistance             = fDistance;
						m_xVisionList.AddToList(pxNewKnowledge);
					}
				}
			}
			else
			{
				pxPlayerPredator->GetChestPosition( pxExistingKnowledge->m_xPosition );
				pxExistingKnowledge->m_xDirection		= pxExistingKnowledge->m_xPosition - pxServerActor->GetPosition();
				pxExistingKnowledge->m_fDistance		= pxExistingKnowledge->m_xDirection.Normalise();
				pxExistingKnowledge->m_uTypeInformation = TYPE_INFO_LASER;
				pxExistingKnowledge->m_xPosition		= m_xLaserPos;
			}
		}
	}
}

//---------------------------------------------------------------------------
// AVP_Sense_Vision :: IsInDarkness
//---------------------------------------------------------------------------
bool AVP_Sense_Vision::IsInDarkness( const Asura_Guid uAlienGuid )
{
	const AVP_ServerActor_Alien* pxAlien = AVP_ServerActor_Alien::FindEntity( uAlienGuid );

	if ( pxAlien )
	{
		return pxAlien->IsInShadows();
	}
	else
	{
		return false;
	}

// 	AVP_ServerEntity_Actor* pxServerActor = GetActorKnowledgeSystem()->GetActorBrain()->GetActor();
// 
// 	float fDistSq = ( pxThreatActor->GetPosition() - pxServerActor->GetPosition() ).MagnitudeSquared();
// 
// 	if ( fDistSq > m_xKnowledgeSettings.m_fVisionCutOffDistanceSq )
// 	{
// 		const Asura_Vector_3&			xPlayerPos			= pxThreatActor->GetPosition();
// 		u_short							usNearestWaypoint	= ASURA_INVALID_WAYPOINT_ID;
// 		AVP_Navigation::GetNearestWaypointID( xPlayerPos, usNearestWaypoint );
// 
// 		if ( usNearestWaypoint != ASURA_INVALID_WAYPOINT_ID )
// 		{
// 			float fLightCutOffValue = 1.75f;
// 
// 			if ( pxServerActor->GetClassification() == AVP_ENTITYCLASS_ANDROID )
// 			{
// 				fLightCutOffValue = Axon_Project_System::GetAndroidLightCutOffValue();
// 			}
// 			else
// 			{
// 				fLightCutOffValue = Axon_Project_System::GetLightCutOffValue();
// 			}
// 
// 			if ( AVP_Waypoint_Manager::GetWPLuminance( usNearestWaypoint ) < fLightCutOffValue )
// 			{
// 				return true;
// 			}
// 		}
// 	}
// 
// 	return false;
}

//---------------------------------------------------------------------------
// AVP_Sense_Vision :: ShouldIgnoreCloakedStatus
//---------------------------------------------------------------------------
bool AVP_Sense_Vision::ShouldIgnoreCloakedStatus()
{
	AVP_ServerEntity_Actor* pxServerActor = GetActorKnowledgeSystem()->GetActorBrain()->GetActor();

	if ( pxServerActor->GetClassification() == AVP_ENTITYCLASS_MARINE || pxServerActor->GetClassification() == AVP_ENTITYCLASS_ANDROID )
	{
		AVP_Brain_Actor_Marine* pxMarineBrain = static_cast<AVP_Brain_Actor_Marine*>( GetActorKnowledgeSystem()->GetActorBrain() );

		if ( pxMarineBrain )
		{
			if ( pxMarineBrain->GetCurrentWeaponType() == AVP_WEAPON_BEHAVIOUR_TYPE_SMARTGUN )
			{
				return true;
			}
		}
	}

	return false;
}

//---------------------------------------------------------------------------
// AVP_Sense_Vision :: GetTargetPriority
//---------------------------------------------------------------------------
int	AVP_Sense_Vision::GetTargetPriority( Asura_Guid uThreatGuid )
{
	if ( uThreatGuid == ASURA_GUID_UNREGISTERED ) { return AVP_AI_TargetPriorityData::IGNORE_P; }

	AVP_ServerEntity_Actor*		pxThreatActor	= AVP_ServerEntity_Actor::FindEntity( uThreatGuid );

	if ( !pxThreatActor ) { return AVP_AI_TargetPriorityData::IGNORE_P; }

	AVP_ServerEntity_Actor*		pxOurActor		= GetActorKnowledgeSystem()->GetActorBrain()->GetActor();
	int							iPriority		= AVP_AI_TargetPriorityData::NORMAL;
	bool						bInValidSquad	= false;
	bool						bUseSquads		= false;
	AVP_ServerEntity_Squad*		pxSquad			= AVP_ServerEntity_Squad::FindEntity( pxOurActor->GetInitialSquadGuid() );

	if ( pxSquad )
	{
		bInValidSquad = true;
	}

	// Actors priority always overrides squads.
	if ( !GetActorKnowledgeSystem()->GetActorBrain()->CalcTargetingPriority( pxThreatActor->GetClassification(), pxThreatActor->GetGuid(), iPriority ) )
	{
		if ( bInValidSquad )
		{
			bUseSquads = true;
		}
	}
	else
	{
		return iPriority;
	}

	if ( bUseSquads )
	{
		int iSquadsPriority = AVP_AI_TargetPriorityData::NORMAL;

		if ( bInValidSquad )
		{
			if ( !pxSquad->CalcTargetingPriority( pxThreatActor->GetClassification(), pxThreatActor->GetGuid(), iSquadsPriority ) )
			{
				return AVP_AI_TargetPriorityData::NORMAL;
			}
			else
			{
				return iSquadsPriority;
			}
		}
	}

	return AVP_AI_TargetPriorityData::NORMAL;
}

//---------------------------------------------------------------------------
// AVP_Sense_Vision :: AddToVisionList
//---------------------------------------------------------------------------
void AVP_Sense_Vision::AddToVisionList(Axon_Knowledge* pxNewKnowledge)
{
	if (pxNewKnowledge)
	{
		m_xVisionList.AddToList(pxNewKnowledge);
	}
}

//---------------------------------------------------------------------------
// AVP_Sense_Vision :: LOSCheckOnActor
//---------------------------------------------------------------------------
bool AVP_Sense_Vision::LOSCheckOnActor(const AVP_ServerEntity_Actor* pxThreatActor, const bool bFOV)
{
	const AVP_ServerEntity_Actor* pxOurActor = GetActorKnowledgeSystem()->GetActorBrain()->GetActor();

	// Simple LOS check
	if (pxOurActor && pxThreatActor)
	{
		// Make sure we are all alive and active
		if (!pxOurActor->IsEntityActive() || !pxThreatActor->IsEntityActive() || pxOurActor->IsDead() || pxThreatActor->IsDead())
		{
			return false;
		}

		// Get the eye position
		Asura_Vector_3 xEyePosition = pxOurActor->GetPosition();
		pxOurActor->GetHeadPosition(xEyePosition);

		// Get the Threat chest position
		Asura_Vector_3 xThreatPosition = pxThreatActor->GetPosition();
		pxThreatActor->GetChestPosition(xThreatPosition);

		// Get the direction of the LOS
		Asura_Vector_3 xDirection = xThreatPosition - xEyePosition;
		const float    fDistance  = xDirection.Normalise();

		const Asura_LOS_InputData   xInputData(xEyePosition, xDirection, pxOurActor->GetGuid(), fDistance, true);
		const Asura_LOS_OutputData* pxOutputData = Asura_LOS::TestAgainstEverything(&xInputData);

		// if we didn't hit the environment and hit our Threat we have a sighting.
		if ((!pxOutputData->m_bHitEnvironment) && (pxOutputData->m_uHitEntity == pxThreatActor->GetGuid()))
		{
			return true;
		}

		return false;
	}

	Asura_Assertf(pxOurActor, ("Invalid source actor."));
	Asura_Assertf(pxThreatActor, ("Invalid threat actor."));
	return false;
}

//---------------------------------------------------------------------------
// AVP_Sense_Vision :: GetAVPKnowledgeSystem
//---------------------------------------------------------------------------
AVP_Knowledge_System_Actor* AVP_Sense_Vision::GetActorKnowledgeSystem()
{
    return static_cast< AVP_Knowledge_System_Actor* >( GetKnowledgeSystem() );
}

//---------------------------------------------------------------------------
// AVP_Sense_Vision :: 
//---------------------------------------------------------------------------
void AVP_Sense_Vision::InitFromSettingsBlueprint()
{
	AVP_ServerEntity_Actor*	pxOurActor				= GetActorKnowledgeSystem()->GetActorBrain()->GetActor();
	Asura_Hash_ID			uBlueprintTypeHashID	= 0x7AD7BD1B; //Asura_GetHashID("AI Settings");
	Asura_Hash_ID			uBlueprintHashID		= ASURA_HASH_ID_UNSET;

	AVP_ServerEntity_Team_Marine* pxTeam = AVP_ServerEntity_Team_Marine::GetInstance();

	if ( !pxOurActor )
	{
		Asura_Assertf( false, ( "AVP_Sense_Vision::InitFromSettingsBlueprint - this shouldn't be possible. GUID = %u", GetActorKnowledgeSystem()->GetActorBrain()->GetActor() ) );
		return;
	}

	if ( pxTeam )
	{
		switch (pxOurActor->GetClassification())
		{
			case AVP_ENTITYCLASS_ANDROID:			uBlueprintHashID = pxTeam->GetAndroidKnowledgeBlueprintHash();	break;
			case AVP_ENTITYCLASS_CIVILIAN:			uBlueprintHashID = pxTeam->GetCivilianKnowledgeBlueprintHash(); break;
			case AVP_ENTITYCLASS_MARINE:			uBlueprintHashID = pxTeam->GetMarineKnowledgeBlueprintHash();	break;
			default: Asura_Assert(false, "AVP_Sense_Vision :: InitFromSettingsBlueprint - Invalid Actor Type using default values for knowledge settings");
		}

		Asura_Blueprint*		pxBlueprint				= Asura_Blueprint_System::GetBlueprintByHashID(uBlueprintTypeHashID, uBlueprintHashID);
		if (pxBlueprint)
		{
			Asura_Value xValue;

			if (pxBlueprint->GetParameterValueByHashID(0x5739C427, xValue))	// 0x5739c427; //"PassiveSightRange"
				m_xKnowledgeSettings.m_fPassiveRange = xValue.GetFloatValue();

			if (pxBlueprint->GetParameterValueByHashID(0x510FC946, xValue))	// 0x510fc946; //"ActiveSightRange"
				m_xKnowledgeSettings.m_fActiveRange = xValue.GetFloatValue();

			if (pxBlueprint->GetParameterValueByHashID(0x196680C4, xValue))	// 0x196680c4; //"AggressiveSightRange"
				m_xKnowledgeSettings.m_fAggressiveRange = xValue.GetFloatValue();

			if (pxBlueprint->GetParameterValueByHashID(0x4c3ad6f7, xValue))	// 0x4c3ad6f7; //"PassiveThresholdToAcknowledgeThreat"
				m_xKnowledgeSettings.m_fPassiveThreshold = xValue.GetFloatValue();

			if (pxBlueprint->GetParameterValueByHashID(0xf81b7256, xValue))	// 0xf81b7256; //"ActiveThresholdToAcknowledgeThreat"
				m_xKnowledgeSettings.m_fActiveThreshold = xValue.GetFloatValue();

			if (pxBlueprint->GetParameterValueByHashID(0x9cae8654, xValue))	// 0x9cae8654; //"AggressiveThresholdToAcknowledgeThreat"
				m_xKnowledgeSettings.m_fAggressiveThreshold = xValue.GetFloatValue();

			if (pxBlueprint->GetParameterValueByHashID(0x18c61f5f, xValue))	// 0x18c61f5f; //"VisionKnowledgeLifeTime"
				m_xKnowledgeSettings.m_fKnowledgeLifetime = xValue.GetFloatValue();

			if (pxBlueprint->GetParameterValueByHashID(0xb1a0c5fa, xValue))	// 0xb1a0c5fa; //"CloakedVisibleRange"
			{
				m_xKnowledgeSettings.m_fCloakedVisibleRangeSq = xValue.GetFloatValue() * xValue.GetFloatValue();
				GetActorKnowledgeSystem()->SetCloakedVisibleRangeSq( m_xKnowledgeSettings.m_fCloakedVisibleRangeSq );
			}

			if (pxBlueprint->GetParameterValueByHashID(0xc5ee14f1, xValue))	// 0xc5ee14f1; //"VisionalwaysSeeDistance"
				m_xKnowledgeSettings.m_fVisionCutOffDistanceSq = xValue.GetFloatValue() * xValue.GetFloatValue();

			if (pxBlueprint->GetParameterValueByHashID(0x2f076c3e, xValue)) // 0x2f076c3e; //"DarknessThreshold"
				m_xKnowledgeSettings.m_fDarknessLightThreshold = xValue.GetFloatValue();

			if (pxBlueprint->GetParameterValueByHashID(0x379e5b32, xValue)) // 0x379e5b32; //"DarkLightScore"
				m_xKnowledgeSettings.m_fDarknessLightScore = xValue.GetFloatValue();

			if (pxBlueprint->GetParameterValueByHashID(0xbe2e033a, xValue)) // 0xbe2e033a; //"BrightnessThreshold"
				m_xKnowledgeSettings.m_fBrightnessLightThreshold = xValue.GetFloatValue();

			if (pxBlueprint->GetParameterValueByHashID(0x84ab85b6, xValue)) // 0x84ab85b6; //"BrightLightScore"
				m_xKnowledgeSettings.m_fBrightnessLightScore = xValue.GetFloatValue();

			if (pxBlueprint->GetParameterValueByHashID(0xf7189843, xValue)) // 0xf7189843; //"NeutralLightScore"
				m_xKnowledgeSettings.m_fNeutralLightScore = xValue.GetFloatValue();

			if (pxBlueprint->GetParameterValueByHashID(0xCA663280, xValue)) //0xCA663280 //"PassiveCloakingThresholdToSeeThreat"
				m_xKnowledgeSettings.m_fPassiveCloakingThreshold = xValue.GetFloatValue();

			if (pxBlueprint->GetParameterValueByHashID(0x7646CDDF, xValue)) //0x7646CDDF //"ActiveCloakingThresholdToSeeThreat""
				m_xKnowledgeSettings.m_fActiveCloakingThreshold = xValue.GetFloatValue();

			if (pxBlueprint->GetParameterValueByHashID(0x1AD9E1DD, xValue)) //0x1AD9E1DD //"AggressiveCloakingThresholdToSeeThreat"
				m_xKnowledgeSettings.m_fAggressiveCloakingThreshold = xValue.GetFloatValue();
		}
	}

	// Forgot to initialise these.
	m_fRange       = m_xKnowledgeSettings.m_fPassiveRange;
	m_fTanFovX     = Asura_Maths::Tangent( Asura_Maths::DegToRad( 170.0f ) * 0.5f );
	m_fTanFovYUp   = Asura_Maths::Tangent( Asura_Maths::DegToRad( 130.0f ) * 0.25f );
	m_fTanFovYDown = Asura_Maths::Tangent( Asura_Maths::DegToRad( 110.0f ) * 0.6f );
}

void AVP_Sense_Vision::UpdateKnowlegdeSettings( KnowledgeSettings* pxKnowledgeSettings )
{
	if ( pxKnowledgeSettings )
	{
		m_xKnowledgeSettings.m_fPassiveRange = pxKnowledgeSettings->m_fPassiveRange;
		m_xKnowledgeSettings.m_fActiveRange = pxKnowledgeSettings->m_fActiveRange;
		m_xKnowledgeSettings.m_fAggressiveRange = pxKnowledgeSettings->m_fAggressiveRange;
		m_xKnowledgeSettings.m_fPassiveThreshold = pxKnowledgeSettings->m_fPassiveThreshold;
		m_xKnowledgeSettings.m_fActiveThreshold = pxKnowledgeSettings->m_fActiveThreshold;
		m_xKnowledgeSettings.m_fAggressiveThreshold = pxKnowledgeSettings->m_fAggressiveThreshold;
		m_xKnowledgeSettings.m_fKnowledgeLifetime = pxKnowledgeSettings->m_fKnowledgeLifetime;
		m_xKnowledgeSettings.m_fCloakedVisibleRangeSq = pxKnowledgeSettings->m_fCloakedVisibleRangeSq;
		GetActorKnowledgeSystem()->SetCloakedVisibleRangeSq( m_xKnowledgeSettings.m_fCloakedVisibleRangeSq );
		m_xKnowledgeSettings.m_fVisionCutOffDistanceSq = pxKnowledgeSettings->m_fVisionCutOffDistanceSq;
		m_xKnowledgeSettings.m_fDarknessLightThreshold = pxKnowledgeSettings->m_fDarknessLightThreshold;
		m_xKnowledgeSettings.m_fDarknessLightScore = pxKnowledgeSettings->m_fDarknessLightScore;
		m_xKnowledgeSettings.m_fBrightnessLightThreshold = pxKnowledgeSettings->m_fBrightnessLightThreshold;
		m_xKnowledgeSettings.m_fBrightnessLightScore = pxKnowledgeSettings->m_fBrightnessLightScore;
		m_xKnowledgeSettings.m_fNeutralLightScore = pxKnowledgeSettings->m_fNeutralLightScore;
		m_xKnowledgeSettings.m_fPassiveCloakingThreshold = pxKnowledgeSettings->m_fPassiveCloakingThreshold;
		m_xKnowledgeSettings.m_fActiveCloakingThreshold = pxKnowledgeSettings->m_fActiveCloakingThreshold;
		m_xKnowledgeSettings.m_fAggressiveCloakingThreshold = pxKnowledgeSettings->m_fAggressiveCloakingThreshold;
	}
}

//---------------------------------------------------------------------------
// AVP_Sense_Vision :: GetAlertLevelRange
//---------------------------------------------------------------------------
float	AVP_Sense_Vision::GetAlertLevelRange(void)
{
	switch (GetActorKnowledgeSystem()->GetAlertLevel())
	{
		case (AVP_Knowledge_System_Actor::PASSIVE_ALERTNESS):
		{
			if (GetActorKnowledgeSystem()->EverBeenAggressive())
			{
				return m_xKnowledgeSettings.m_fActiveRange;
			}
			else
			{
				return m_xKnowledgeSettings.m_fPassiveRange;
			}
		}
		case (AVP_Knowledge_System_Actor::ACTIVE_ALERTNESS):
		{
			return m_xKnowledgeSettings.m_fActiveRange;
		}
		case (AVP_Knowledge_System_Actor::AGGRESSIVE_ALERTNESS):
		{
			return m_xKnowledgeSettings.m_fAggressiveRange;
		}
		default:
		{
			return m_fRange;
		}
	}
}

//---------------------------------------------------------------------------
// AVP_Sense_Vision :: ShouldCullActorOnRangeAndFOV
//---------------------------------------------------------------------------
bool AVP_Sense_Vision::ShouldCullActorOnRangeAndFOV(const AVP_ServerEntity_Actor* pxOurActor, const AVP_ServerEntity_Actor* pxOtherActor)
{
	// Cull on range and FOV.
	const Asura_Vector_3&	xActorPosition	= pxOtherActor->GetPosition();
	Asura_Vector_3			xDirection		= xActorPosition - pxOurActor->GetPosition();
	const float				fDistance		= xDirection.Magnitude();

	if (ShouldCullOnRange(fDistance, xDirection) || ShouldCullOnFOV(fDistance, xDirection))
	{
		return true;
	}
	else
	{
		return false;
	}
}

//---------------------------------------------------------------------------
// AVP_Sense_Vision :: GetScoreForFOVPosition
//---------------------------------------------------------------------------
float AVP_Sense_Vision::GetScoreForFOVPosition(const AVP_ServerEntity_Actor* pxOurActor, const Asura_Vector_3& xPoint)
{
	Asura_Vector_3				xDirection	= xPoint - pxOurActor->GetPosition();
	Asura_Vector_3				xHeadFwd	= Asura_Maths::ZeroVector3;

	pxOurActor->GetHeadFowardVector(xHeadFwd);

	// get the face marker point
	xDirection.Normalise();
	
	float fDot = xHeadFwd * xDirection;
	return fDot * 100.0f;
}

//---------------------------------------------------------------------------
// AVP_Sense_Vision :: GetScoreForRange
//---------------------------------------------------------------------------
float AVP_Sense_Vision::GetScoreForRange(const Asura_Vector_3& xOurPos, const Asura_Vector_3& xOtherPos)
{
	float fDistSq = (xOtherPos - xOurPos).MagnitudeSquared();
	
	// If the threat is under the always see range
	if (fDistSq <= m_xKnowledgeSettings.m_fVisionCutOffDistanceSq)
	{
		// Return a very large number as we want the threat to be detected if it is this close to the actor.
		return 1000.0f;
	}

	float	fPercentage			= (fDistSq / (m_fRange * m_fRange)) * 100.0f;
	float	fInversePercentage	= 100.0f - fPercentage;
	return	fInversePercentage;
}

//---------------------------------------------------------------------------
// AVP_Sense_Vision :: GetScoreForspeed
//---------------------------------------------------------------------------
float AVP_Sense_Vision::GetScoreForSpeed(const AVP_ServerEntity_Actor* pxActor)
{
	if (!pxActor)
	{
		return 0.0f;
	}

	float fSpeed = pxActor->GetVelocity().Magnitude();

	if ( pxActor->IsCloaked(true) )
	{
		fSpeed = fSpeed * 5.0f;

		if (fSpeed > 50.0f)
		{
			fSpeed = 50.0f;
		}
	}
	else
	{
		fSpeed = fSpeed * 10.0f;

		if (fSpeed > 100.0f)
		{
			fSpeed = 100.0f;
		}
	}

	return fSpeed;
}

//---------------------------------------------------------------------------
// AVP_Sense_Vision :: GetLightingScore
//---------------------------------------------------------------------------
float AVP_Sense_Vision::GetLightingScore(const Asura_Vector_3& xThreatPos)
{
	u_short uWPID;

	AVP_Navigation::GetNearestWaypointID(xThreatPos, uWPID);
	float fLightValue = AVP_Waypoint_Manager::GetWPLuminance((u_int)uWPID);

	if (fLightValue < m_xKnowledgeSettings.m_fDarknessLightThreshold)
	{
		return m_xKnowledgeSettings.m_fDarknessLightScore;
	}
	else if (fLightValue > m_xKnowledgeSettings.m_fBrightnessLightThreshold)
	{
		return m_xKnowledgeSettings.m_fBrightnessLightScore;
	}
	else
	{
		return m_xKnowledgeSettings.m_fNeutralLightScore;
	}
}

//---------------------------------------------------------------------------
// AVP_Sense_Vision :: GetAlertLevelScoreToBeat
//---------------------------------------------------------------------------
float AVP_Sense_Vision::GetAlertLevelScoreToBeat(void)
{
	switch (GetActorKnowledgeSystem()->GetAlertLevel())
	{
		case (AVP_Knowledge_System_Actor::AGGRESSIVE_ALERTNESS):
		{
			return m_xKnowledgeSettings.m_fAggressiveThreshold;
		}
		case (AVP_Knowledge_System_Actor::ACTIVE_ALERTNESS):
		{
			return m_xKnowledgeSettings.m_fActiveThreshold;
		}
		case (AVP_Knowledge_System_Actor::PASSIVE_ALERTNESS):
		default:
		{
			if ( GetActorKnowledgeSystem()->EverBeenAggressive() )
			{
				return m_xKnowledgeSettings.m_fActiveThreshold;
			}
			else
			{
				return m_xKnowledgeSettings.m_fPassiveThreshold;
			}
		}
	}
}

//---------------------------------------------------------------------------
// AVP_Sense_Vision :: GetCloakingAlertLevelScoreToBeat
//---------------------------------------------------------------------------
float AVP_Sense_Vision::GetCloakingAlertLevelScoreToBeat(void)
{
	switch (GetActorKnowledgeSystem()->GetAlertLevel())
	{
		case (AVP_Knowledge_System_Actor::AGGRESSIVE_ALERTNESS):
		{
			return m_xKnowledgeSettings.m_fAggressiveCloakingThreshold;
		}
		case (AVP_Knowledge_System_Actor::ACTIVE_ALERTNESS):
		{
			return m_xKnowledgeSettings.m_fActiveCloakingThreshold;
		}
		case (AVP_Knowledge_System_Actor::PASSIVE_ALERTNESS):
		default:
		{
			if ( GetActorKnowledgeSystem()->EverBeenAggressive() )
			{
				return m_xKnowledgeSettings.m_fActiveCloakingThreshold;
			}
			else
			{
				return m_xKnowledgeSettings.m_fPassiveCloakingThreshold;
			}
		}
	}
}

//---------------------------------------------------------------------------
// AVP_Sense_Vision :: UpdatePlayerAlienThreatStatus
//---------------------------------------------------------------------------
float AVP_Sense_Vision::UpdatePlayerAlienThreatStatus(AVP_ServerActor_Alien* pxPlayerAlien)
{
	float fHighestScore	= 0.0f;

	for (Asura_Hashed_List_It<AVP_ServerActor_Vertebrate> it(AVP_ServerActor_Vertebrate::GetList()); !it.Done(); it.Next())
	{
		AVP_ServerActor_Vertebrate* pxVertebrate = it.GetCurrent();

		if (pxVertebrate)
		{
			if (pxVertebrate->IsPlayer())
				continue;

			if (!pxVertebrate->GetActorBrain())
				continue;

			if ((pxVertebrate->GetClassification() != AVP_ENTITYCLASS_ALIEN) && (pxVertebrate->GetClassification() != AVP_ENTITYCLASS_ALIENFACEHUGGER) && (!pxVertebrate->IsDead()) && (pxVertebrate->IsEntityActive()))
			{
				AVP_Sense_Vision*	pxVisionSense = NULL;

				switch (pxVertebrate->GetClassification())
				{
					case AVP_ENTITYCLASS_MARINE:
					case AVP_ENTITYCLASS_ANDROID:
					{
						pxVisionSense	= static_cast<AVP_Sense_Vision*>(((AVP_Brain_Actor_Marine*)pxVertebrate->GetActorBrain())->GetKnowledgeSystem()->m_pxVision);
					}break;

					case AVP_ENTITYCLASS_CIVILIAN:
					{
						continue;
						//pxVisionSense	= static_cast<AVP_Sense_Vision*>(((AVP_Brain_Actor_Civilian*)pxVertebrate->GetActorBrain())->GetKnowledgeSystem()->m_pxVision);
					}

					case AVP_ENTITYCLASS_SENTRYGUN:
					{
						pxVisionSense	= static_cast<AVP_Sense_Vision*>(((AVP_Brain_Actor_Sentrygun*)pxVertebrate->GetActorBrain())->GetKnowledgeSystem()->m_pxVision);
					}break;

					case AVP_ENTITYCLASS_PREDATOR:
					default:
					{
						continue;
					}
				}

				if (pxVisionSense)
				{
					float				fScore			= pxVisionSense->m_fAlienPlayerScore;
					float				fScoreToBeat	= pxVisionSense->m_fAlienPlayerScoreToBeat;

					if (fScore > fScoreToBeat)
					{
						// spotted!
						return 100.0f;
					}
					else
					{
						float fPercentage = (fScore / fScoreToBeat) * 100.0f;

						if (fPercentage > fHighestScore)
						{
							fHighestScore = fPercentage;
						}
					}
				}
			}
		}
	}

	return fHighestScore;
}
