//////////////////////////////////////////////////////////////////////////////
//
// AVP_Sense_Vision_Predator.h
//
// Created by NJD - 06 December 2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __AVP_SENSE_VISION_PREDATOR_H__
#define __AVP_SENSE_VISION_PREDATOR_H__ 1

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// Axon
#include "Knowledge/SenseTypes/Axon_Sense_Vision.h"

// AVP
#include "Entity/PhysicalObject/Actor/AVP_ServerEntity_Actor.h"

// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////////

class AVP_Knowledge_System_Predator;
class AVP_ServerActor_Alien;

//////////////////////////////////////////////////////////////////////////////
//
// AVP_Sense_Vision_Predator
//
//////////////////////////////////////////////////////////////////////////////

class AVP_Sense_Vision_Predator : public Axon_Sense_Vision
{
    typedef Axon_Sense_Vision PARENT;

public:
    AVP_Sense_Vision_Predator(AVP_Knowledge_System_Predator* pxAVPKnowledgeSystem);
    virtual ~AVP_Sense_Vision_Predator();

	virtual void InitialiseFromBlueprint( const Asura_Blueprint& xBlueprint );
    virtual void ReadFromChunkStream( Asura_Chunk_Stream& xStream );
    virtual void WriteToChunkStream( Asura_Chunk_Stream& xStream ) const;
	static bool CullOnFOV(const AVP_ServerEntity_Actor* pxOurActor, const AVP_ServerEntity_Actor* pxThreatActor);
	static bool IsOtherActorInFieldOfView(const AVP_ServerEntity_Actor* pxOurActor, const AVP_ServerEntity_Actor* pxOtherActor);

    virtual void Update(const float fTimeslice);
	bool LOSCheckOnActor(const AVP_ServerEntity_Actor* pxThreatActor, const bool bFOV = false);

	bool ShouldCullActorOnRangeOrFOV(const AVP_ServerEntity_Actor* pxOurActor, const AVP_ServerEntity_Actor* pxOtherActor);
	float GetScoreForFOVPosition(const AVP_ServerEntity_Actor* pxOurActor, const Asura_Vector_3& xPoint);
	float GetScoreForRange(const Asura_Vector_3& xOurPos, const Asura_Vector_3& xOtherPos);
	float GetScoreForSpeed(const AVP_ServerEntity_Actor* pxActor);
	float GetScoreToBeat();
	Asura_Guid GetLongTermMemoryThreat() const { return m_uLongMemoryVisionGuid; }

	float GetAlienPlayerScore()			{return m_fAlienPlayerScore;}
	float GetAlienPlayerScoreToBeat()	{return m_fAlienPlayerScoreToBeat;}


	static float UpdatePlayerAlienThreatStatus(AVP_ServerActor_Alien* pxPlayerAlien);

#ifndef ASURA_NO_DEBUG_OUTPUT
	static void RenderKnowledgeDebugInfo(Asura_Guid uGuid);
#endif

	void SetVisionFOV( const float fTanHorHalfFOV, const float fTanVertUpHalfFOV, const float fTanVertDownHalfFOV ) { m_fTanFovX = fTanHorHalfFOV; m_fTanFovYUp = fTanVertUpHalfFOV; m_fTanFovYDown = fTanVertDownHalfFOV; }

	virtual void AddToVisionList(Axon_Knowledge* pxNewKnowledge);


protected:
	AVP_Knowledge_System_Predator* GetActorKnowledgeSystem();

	void InitFromSettingsBlueprint();
	void DepricateCurrentKnowledge(const float fTimeslice);
	void AddNewKnowledge();

private:

	// Blueprinted Values
	float m_fVisionCutOffDistanceSq;
	float m_fVisionMemoryTime;
	float m_fThreshold;	
	float m_fKnowledgeLifetime;

	// Alien Player Stealth Indicator Values
	float m_fAlienPlayerScore;
	float m_fAlienPlayerScoreToBeat;

	Asura_Guid m_uLongMemoryVisionGuid;

#ifndef ASURA_NO_DEBUG_OUTPUT
	// debug rendering variables
	static float s_fSpeedScore;
	static float s_fRangeScore;
	static float s_fFOVScore;
	static float s_fTotalScore;
	static float s_fThreshold;
	static Asura_Guid s_uRenderGuid;
#endif

private:

    enum { CURRENT_CHUNK_VERSION = 0 };
};

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

#endif //__AVP_SENSE_VISION_PREDATOR_H__
