//////////////////////////////////////////////////////////////////////////////
//
// AVP_AI_Navigation.cpp
//
// Copyright (C) 2009 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"

// header
#include "AI/AVP_AI_Navigation.h"

// asura
#include "ChunkSystem/Asura_Chunk_Stream.h"

// axon

// tools
#ifdef ASURA_TOOLS
#include "Waypointing/ToolsAVP_Mode_Waypointing.h"
#endif

//////////////////////////////////////////////////////////////////////////////
//
// AVP_AI_Navigation
//
//////////////////////////////////////////////////////////////////////////////

void AVP_AI_Navigation_System::AVPUserDefinedWaypoint::ReadFromChunkStream( Asura_Chunk_Stream& xStream )
{
	u_int uVersion;
	xStream >> uVersion;

	UserDefinedWaypoint::ReadFromChunkStream( xStream );

	if ( uVersion == uCURRENT_CHUNK_VERSION )
	{
		xStream >> m_fRadius;
		u_int uNumLinks;
		xStream >> uNumLinks;
		for ( u_int u = 0; u < uNumLinks; ++u )
		{
			u_int uLinkedUDW;
			xStream >> uLinkedUDW;
			m_xLinkedUDWs.Push_Back( uLinkedUDW );
		}
	} 
	else
	{
		Asura_Assertf( false, ( "AVP_AI_Navigation_System::AVPUserDefinedWaypoint::ReadFromChunkStream - bad chunk version (%u)", uVersion ) );
	}
}

void AVP_AI_Navigation_System::AVPUserDefinedWaypoint::WriteToChunkStream( Asura_Chunk_Stream& xStream ) const
{
	xStream << uCURRENT_CHUNK_VERSION;

	UserDefinedWaypoint::WriteToChunkStream( xStream );

	xStream << m_fRadius;
	xStream << m_xLinkedUDWs.Size();
	for ( Asura_LinkList_ConstIt< u_int > xLinkIter( m_xLinkedUDWs ); !xLinkIter.Done(); xLinkIter.Next() )
	{
		xStream << *xLinkIter.GetCurrent();
	}
}

#ifdef ASURA_TOOLS
bool AVP_AI_Navigation_System::AVPUserDefinedWaypoint::IsLiftWaypoint() const
{
	return ( ( m_uFlags & TOOLS_NAVVERTEX_FLAG_LIFT_UDW ) != 0 );
}

void AVP_AI_Navigation_System::AVPUserDefinedWaypoint::AddLink( const u_int uUDWID )
{
	if ( !m_xLinkedUDWs.Find( uUDWID ) )
	{
		m_xLinkedUDWs.Push_Back( uUDWID );
	}
}

void AVP_AI_Navigation_System::AVPUserDefinedWaypoint::RemoveLink( const u_int uUDWID )
{
	m_xLinkedUDWs.Remove( uUDWID );
}
#endif


//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

Asura_AI_Navigation_System* Asura_AI_Navigation_System::Project_CreateSystem()
{
	return new AVP_AI_Navigation_System;
}

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////
