
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_Actor_AIAnimStateMachine.cpp
// Author : Will Plano
// Date   : 29/02/2008
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

#include "Actor/AVP_Actor_AnimStateData.h"

#include "Animation/Asura_Animation.h"
#include "Animation/Asura_Hierarchy.h"
#include "Blueprint/Asura_Blueprint.h"
#include "Blueprint/Asura_Blueprint_System.h"
#include "ChunkSystem/Asura_Chunk_Stream.h"

#include "Blueprint/AVP_Blueprint_HashIDs.h"

static const u_int g_uChunkVersion = 4;
static const u_int g_uChunkHeaderVersion = 1;
static const u_int g_uNonDirectionalChunkVersion = 2;
static const u_int g_uDirectionalChunkVersion = 1;
static const u_int g_uDirectionalRegionChunkVersion = 1;

static const char* const g_szSecondaryString = ":Secondary";
static const char* const g_szAimDownUpString = ":AimDownToUp";
static const char* const g_szAimDownUpDragString = ":AimDownToUpDrag";
static const char* const g_szAimRightLeftString = ":AimRightToLeft";
static const char* const g_szAimRightLeftDragString = ":AimRightToLeftDrag";
static const char* const g_szStrafeDragString = ":StrafeDrag";
static const char* const g_szPostureString = ":Posture";
static const char* const g_szPrimaryActorShootBeginString = ":PrimaryActorShootBegin";
static const char* const g_szPrimaryWeaponShootBeginString = ":PrimaryWeaponShootBegin";
static const char* const g_szSecondaryActorShootBeginString = ":SecondaryActorShootBegin";
static const char* const g_szSecondaryWeaponShootBeginString = ":SecondaryWeaponShootBegin";
static const char* const g_szPrimaryActorShootLoopString = ":PrimaryActorShootLoop";
static const char* const g_szPrimaryWeaponShootLoopString = ":PrimaryWeaponShootLoop";
static const char* const g_szSecondaryActorShootLoopString = ":SecondaryActorShootLoop";
static const char* const g_szSecondaryWeaponShootLoopString = ":SecondaryWeaponShootLoop";
static const char* const g_szPrimaryActorShootEndString = ":PrimaryActorShootEnd";
static const char* const g_szPrimaryWeaponShootEndString = ":PrimaryWeaponShootEnd";
static const char* const g_szSecondaryActorShootEndString = ":SecondaryActorShootEnd";
static const char* const g_szSecondaryWeaponShootEndString = ":SecondaryWeaponShootEnd";
static const char* const g_szWeaponInActorAnimString = ":WeaponInActorAnim";
static const char* const g_szWeaponInWeaponAnimString = ":WeaponInWeaponAnim";
static const char* const g_szWeaponOutActorAnimString = ":WeaponOutActorAnim";
static const char* const g_szWeaponOutWeaponAnimString = ":WeaponOutWeaponAnim";
static const char* const g_szScopeInActorAnimString = ":ScopeInto";
static const char* const g_szScopeOutActorAnimString = ":ScopeOutof";
static const char* const g_szWeaponRestWeaponAnimString = ":WeaponRestWeaponAnim";
static const char* const g_szActorReloadString = ":ReloadActor";
static const char* const g_szWeaponReloadString = ":ReloadWeapon";
static const char* const g_szStimActorString = ":StimActorAnim";
static const char* const g_szStimItemString = ":StimItemAnim";
static const char* const g_szStimWeaponString = ":StimWeaponAnim";
static const char* const g_szFlareActorString = ":FlareActorAnim";
static const char* const g_szCloakActorString = ":CloakActorAnim";
static const char* const g_szCloakWeaponString = ":CloakWeaponAnim";
static const char* const g_szNonDirLowerBodyString = "";
static const char* const g_szTurnLeft45String = ":TurnLeft45";
static const char* const g_szTurnLeft90String = ":TurnLeft90";
static const char* const g_szTurnLeft180String = ":TurnLeft180";
static const char* const g_szTurnRight45String = ":TurnRight45";
static const char* const g_szTurnRight90String = ":TurnRight90";
static const char* const g_szTurnRight180String = ":TurnRight180";
static const char* const g_szFidgetString = ":Fidget";
static const char* const g_szDirBlendLowerBodyString = "";

AVP_Actor_AnimStateData::AVP_Actor_AnimStateData()
: m_fMoveDistance( 0.0f )
, m_uCollisionDescriptorHash( ASURA_HASH_ID_UNSET )
, m_uSecondaryAnimHash( ASURA_HASH_ID_UNSET )
, m_uAimDownUpAnimHash( ASURA_HASH_ID_UNSET )
, m_uAimDownUpDragAnimHash( ASURA_HASH_ID_UNSET )
, m_uAimRightLeftAnimHash( ASURA_HASH_ID_UNSET )
, m_uAimRightLeftDragAnimHash( ASURA_HASH_ID_UNSET )
, m_uStrafeDragAnimHash( ASURA_HASH_ID_UNSET )
, m_uPrimaryActorShootBeginHash( ASURA_HASH_ID_UNSET )
, m_uPrimaryWeaponShootBeginHash( ASURA_HASH_ID_UNSET )
, m_uSecondaryActorShootBeginHash( ASURA_HASH_ID_UNSET )
, m_uSecondaryWeaponShootBeginHash( ASURA_HASH_ID_UNSET )
, m_uPrimaryActorShootLoopHash( ASURA_HASH_ID_UNSET )
, m_uPrimaryWeaponShootLoopHash( ASURA_HASH_ID_UNSET )
, m_uSecondaryActorShootLoopHash( ASURA_HASH_ID_UNSET )
, m_uSecondaryWeaponShootLoopHash( ASURA_HASH_ID_UNSET )
, m_uPrimaryActorShootEndHash( ASURA_HASH_ID_UNSET )
, m_uPrimaryWeaponShootEndHash( ASURA_HASH_ID_UNSET )
, m_uSecondaryActorShootEndHash( ASURA_HASH_ID_UNSET )
, m_uSecondaryWeaponShootEndHash( ASURA_HASH_ID_UNSET )
, m_uWeaponInActorAnimHash( ASURA_HASH_ID_UNSET )
, m_uWeaponInWeaponAnimHash( ASURA_HASH_ID_UNSET )
, m_uWeaponOutActorAnimHash( ASURA_HASH_ID_UNSET )
, m_uWeaponOutWeaponAnimHash( ASURA_HASH_ID_UNSET )
, m_uScopeInActorAnimHash( ASURA_HASH_ID_UNSET )
, m_uScopeOutActorAnimHash( ASURA_HASH_ID_UNSET )
, m_uWeaponRestWeaponAnimHash( ASURA_HASH_ID_UNSET )
, m_uReloadActorHash( ASURA_HASH_ID_UNSET )
, m_uReloadWeaponHash( ASURA_HASH_ID_UNSET )
, m_uStimActorAnimHash( ASURA_HASH_ID_UNSET )
, m_uStimItemAnimHash( ASURA_HASH_ID_UNSET )
, m_uStimWeaponAnimHash( ASURA_HASH_ID_UNSET )
, m_uFlareActorAnimHash( ASURA_HASH_ID_UNSET )
, m_uCloakActorAnimHash( ASURA_HASH_ID_UNSET )
, m_uCloakWeaponAnimHash( ASURA_HASH_ID_UNSET )
, m_uFidgetBPHash( ASURA_HASH_ID_UNSET )
{

}

//#ifdef ASURA_TOOLS

void AVP_Actor_AnimStateData::CopyFrom( const AVP_Actor_AnimStateData& xOther )
{
	m_fMoveDistance = xOther.m_fMoveDistance;
	m_uCollisionDescriptorHash = xOther.m_uCollisionDescriptorHash;
	m_uSecondaryAnimHash = xOther.m_uSecondaryAnimHash;
	m_uAimDownUpAnimHash = xOther.m_uAimDownUpAnimHash;
	m_uAimDownUpDragAnimHash = xOther.m_uAimDownUpDragAnimHash;
	m_uAimRightLeftAnimHash = xOther.m_uAimRightLeftAnimHash;
	m_uAimRightLeftDragAnimHash = xOther.m_uAimRightLeftDragAnimHash;
	m_uStrafeDragAnimHash = xOther.m_uStrafeDragAnimHash;
	m_uPrimaryActorShootBeginHash = xOther.m_uPrimaryActorShootBeginHash;
	m_uPrimaryWeaponShootBeginHash = xOther.m_uPrimaryWeaponShootBeginHash;
	m_uSecondaryActorShootBeginHash = xOther.m_uSecondaryActorShootBeginHash;
	m_uSecondaryWeaponShootBeginHash = xOther.m_uSecondaryWeaponShootBeginHash;
	m_uPrimaryActorShootLoopHash = xOther.m_uPrimaryActorShootLoopHash;
	m_uPrimaryWeaponShootLoopHash = xOther.m_uPrimaryWeaponShootLoopHash;
	m_uSecondaryActorShootLoopHash = xOther.m_uSecondaryActorShootLoopHash;
	m_uSecondaryWeaponShootLoopHash = xOther.m_uSecondaryWeaponShootLoopHash;
	m_uPrimaryActorShootEndHash = xOther.m_uPrimaryActorShootEndHash;
	m_uPrimaryWeaponShootEndHash = xOther.m_uPrimaryWeaponShootEndHash;
	m_uSecondaryActorShootEndHash = xOther.m_uSecondaryActorShootEndHash;
	m_uSecondaryWeaponShootEndHash = xOther.m_uSecondaryWeaponShootEndHash;
	m_uWeaponInActorAnimHash = xOther.m_uWeaponInActorAnimHash;
	m_uWeaponInWeaponAnimHash = xOther.m_uWeaponInWeaponAnimHash;
	m_uWeaponOutActorAnimHash = xOther.m_uWeaponOutActorAnimHash;
	m_uWeaponOutWeaponAnimHash = xOther.m_uWeaponOutWeaponAnimHash;
	m_uScopeInActorAnimHash = xOther.m_uScopeInActorAnimHash;
	m_uScopeOutActorAnimHash = xOther.m_uScopeOutActorAnimHash;
	m_uWeaponRestWeaponAnimHash = xOther.m_uWeaponRestWeaponAnimHash;
	m_uReloadActorHash = xOther.m_uReloadActorHash;
	m_uReloadWeaponHash = xOther.m_uReloadWeaponHash;
	m_uStimActorAnimHash = xOther.m_uStimActorAnimHash;
	m_uStimItemAnimHash = xOther.m_uStimItemAnimHash;
	m_uStimWeaponAnimHash = xOther.m_uStimWeaponAnimHash;
	m_uFlareActorAnimHash = xOther.m_uFlareActorAnimHash;
	m_uCloakActorAnimHash = xOther.m_uCloakActorAnimHash;
	m_uCloakWeaponAnimHash = xOther.m_uCloakWeaponAnimHash;
	m_uFidgetBPHash = xOther.m_uFidgetBPHash;
}

void AVP_Actor_AnimStateData::InitialiseFromBlueprint( const Asura_Hash_ID uBPType, const Asura_Blueprint& xBlueprint, const char* const szStateString, const bool bAssertOnFailure )
{
	Asura_Value xValue;
	if ( GetParameterValueByString( xBlueprint, szStateString, g_szPostureString, m_uCollisionDescriptorHash == ASURA_HASH_ID_UNSET ? bAssertOnFailure : false, xValue ) )
	{
		m_uCollisionDescriptorHash = xValue.GetHashIDValue();
	}
	if ( GetParameterValueByString( xBlueprint, szStateString, g_szSecondaryString, false, xValue ) )
	{
		m_uSecondaryAnimHash = xValue.GetHashIDValue();
	}
	if ( GetParameterValueByString( xBlueprint, szStateString, g_szAimDownUpString, false, xValue ) )
	{
		m_uAimDownUpAnimHash = xValue.GetHashIDValue();
	}
	if ( GetParameterValueByString( xBlueprint, szStateString, g_szAimDownUpDragString, false, xValue ) )
	{
		m_uAimDownUpDragAnimHash = xValue.GetHashIDValue();
	}
	if ( GetParameterValueByString( xBlueprint, szStateString, g_szAimRightLeftString, false, xValue ) )
	{
		m_uAimRightLeftAnimHash = xValue.GetHashIDValue();
	}
	if ( GetParameterValueByString( xBlueprint, szStateString, g_szAimRightLeftDragString, false, xValue ) )
	{
		m_uAimRightLeftDragAnimHash = xValue.GetHashIDValue();
	}
	if ( GetParameterValueByString( xBlueprint, szStateString, g_szStrafeDragString, false, xValue ) )
	{
		m_uStrafeDragAnimHash = xValue.GetHashIDValue();
	}
	if ( GetParameterValueByString( xBlueprint, szStateString, g_szPrimaryActorShootBeginString, false, xValue ) )
	{
		m_uPrimaryActorShootBeginHash = xValue.GetHashIDValue();
	}
	if ( GetParameterValueByString( xBlueprint, szStateString, g_szPrimaryWeaponShootBeginString, false, xValue ) )
	{
		m_uPrimaryWeaponShootBeginHash = xValue.GetHashIDValue();
	}
	if ( GetParameterValueByString( xBlueprint, szStateString, g_szSecondaryActorShootBeginString, false, xValue ) )
	{
		m_uSecondaryActorShootBeginHash = xValue.GetHashIDValue();
	}
	if ( GetParameterValueByString( xBlueprint, szStateString, g_szSecondaryWeaponShootBeginString, false, xValue ) )
	{
		m_uSecondaryWeaponShootBeginHash = xValue.GetHashIDValue();
	}
	if ( GetParameterValueByString( xBlueprint, szStateString, g_szPrimaryActorShootLoopString, false, xValue ) )
	{
		m_uPrimaryActorShootLoopHash = xValue.GetHashIDValue();
	}
	if ( GetParameterValueByString( xBlueprint, szStateString, g_szPrimaryWeaponShootLoopString, false, xValue ) )
	{
		m_uPrimaryWeaponShootLoopHash = xValue.GetHashIDValue();
	}
	if ( GetParameterValueByString( xBlueprint, szStateString, g_szSecondaryActorShootLoopString, false, xValue ) )
	{
		m_uSecondaryActorShootLoopHash = xValue.GetHashIDValue();
	}
	if ( GetParameterValueByString( xBlueprint, szStateString, g_szSecondaryWeaponShootLoopString, false, xValue ) )
	{
		m_uSecondaryWeaponShootLoopHash = xValue.GetHashIDValue();
	}
	if ( GetParameterValueByString( xBlueprint, szStateString, g_szPrimaryActorShootEndString, false, xValue ) )
	{
		m_uPrimaryActorShootEndHash = xValue.GetHashIDValue();
	}
	if ( GetParameterValueByString( xBlueprint, szStateString, g_szPrimaryWeaponShootEndString, false, xValue ) )
	{
		m_uPrimaryWeaponShootEndHash = xValue.GetHashIDValue();
	}
	if ( GetParameterValueByString( xBlueprint, szStateString, g_szSecondaryActorShootEndString, false, xValue ) )
	{
		m_uSecondaryActorShootEndHash = xValue.GetHashIDValue();
	}
	if ( GetParameterValueByString( xBlueprint, szStateString, g_szSecondaryWeaponShootEndString, false, xValue ) )
	{
		m_uSecondaryWeaponShootEndHash = xValue.GetHashIDValue();
	}
	if ( GetParameterValueByString( xBlueprint, szStateString, g_szWeaponInActorAnimString, false, xValue ) )
	{
		m_uWeaponInActorAnimHash = xValue.GetHashIDValue();
	}
	if ( GetParameterValueByString( xBlueprint, szStateString, g_szWeaponInWeaponAnimString, false, xValue ) )
	{
		m_uWeaponInWeaponAnimHash = xValue.GetHashIDValue();
	}
	if ( GetParameterValueByString( xBlueprint, szStateString, g_szWeaponOutActorAnimString, false, xValue ) )
	{
		m_uWeaponOutActorAnimHash = xValue.GetHashIDValue();
	}
	if ( GetParameterValueByString( xBlueprint, szStateString, g_szWeaponOutWeaponAnimString, false, xValue ) )
	{
		m_uWeaponOutWeaponAnimHash = xValue.GetHashIDValue();
	}
	if ( GetParameterValueByString( xBlueprint, szStateString, g_szScopeInActorAnimString, false, xValue ) )
	{
		m_uScopeInActorAnimHash = xValue.GetHashIDValue();
	}
	if ( GetParameterValueByString( xBlueprint, szStateString, g_szScopeOutActorAnimString, false, xValue ) )
	{
		m_uScopeOutActorAnimHash = xValue.GetHashIDValue();
	}
	if ( GetParameterValueByString( xBlueprint, szStateString, g_szWeaponRestWeaponAnimString, false, xValue ) )
	{
		m_uWeaponRestWeaponAnimHash = xValue.GetHashIDValue();
	}
	if ( GetParameterValueByString( xBlueprint, szStateString, g_szActorReloadString, false, xValue ) )
	{
		m_uReloadActorHash = xValue.GetHashIDValue();
	}
	if ( GetParameterValueByString( xBlueprint, szStateString, g_szWeaponReloadString, false, xValue ) )
	{
		m_uReloadWeaponHash = xValue.GetHashIDValue();
	}
	if ( GetParameterValueByString( xBlueprint, szStateString, g_szStimActorString, false, xValue ) )
	{
		m_uStimActorAnimHash = xValue.GetHashIDValue();
	}
	if ( GetParameterValueByString( xBlueprint, szStateString, g_szStimItemString, false, xValue ) )
	{
		m_uStimItemAnimHash = xValue.GetHashIDValue();
	}
	if ( GetParameterValueByString( xBlueprint, szStateString, g_szStimWeaponString, false, xValue ) )
	{
		m_uStimWeaponAnimHash = xValue.GetHashIDValue();
	}
	if ( GetParameterValueByString( xBlueprint, szStateString, g_szFlareActorString, false, xValue ) )
	{
		m_uFlareActorAnimHash = xValue.GetHashIDValue();
	}
	if ( GetParameterValueByString( xBlueprint, szStateString, g_szCloakActorString, false, xValue ) )
	{
		m_uCloakActorAnimHash = xValue.GetHashIDValue();
	}
	if ( GetParameterValueByString( xBlueprint, szStateString, g_szCloakWeaponString, false, xValue ) )
	{
		m_uCloakWeaponAnimHash = xValue.GetHashIDValue();
	}
	if ( GetParameterValueByString( xBlueprint, szStateString, g_szFidgetString, false, xValue ) )
	{
		m_uFidgetBPHash = xValue.GetHashIDValue();
	}
}

#ifdef ASURA_TOOLS

void AVP_Actor_AnimStateData::WriteToChunkStream( Asura_Chunk_Stream& xStream ) const
{
	xStream << g_uChunkVersion;

	xStream << m_fMoveDistance;
	xStream << m_uCollisionDescriptorHash;
	xStream << m_uSecondaryAnimHash;
	xStream << m_uAimDownUpAnimHash;
	xStream << m_uAimDownUpDragAnimHash;
	xStream << m_uAimRightLeftAnimHash;
	xStream << m_uAimRightLeftDragAnimHash;
	xStream << m_uStrafeDragAnimHash;
	xStream << m_uPrimaryActorShootBeginHash;
	xStream << m_uPrimaryWeaponShootBeginHash;
	xStream << m_uSecondaryActorShootBeginHash;
	xStream << m_uSecondaryWeaponShootBeginHash;
	xStream << m_uPrimaryActorShootLoopHash;
	xStream << m_uPrimaryWeaponShootLoopHash;
	xStream << m_uSecondaryActorShootLoopHash;
	xStream << m_uSecondaryWeaponShootLoopHash;
	xStream << m_uPrimaryActorShootEndHash;
	xStream << m_uPrimaryWeaponShootEndHash;
	xStream << m_uSecondaryActorShootEndHash;
	xStream << m_uSecondaryWeaponShootEndHash;
	xStream << m_uWeaponInActorAnimHash;
	xStream << m_uWeaponInWeaponAnimHash;
	xStream << m_uWeaponOutActorAnimHash;
	xStream << m_uWeaponOutWeaponAnimHash;
	xStream << m_uScopeInActorAnimHash;
	xStream << m_uScopeOutActorAnimHash;
	xStream << m_uWeaponRestWeaponAnimHash;
	xStream << m_uReloadActorHash;
	xStream << m_uReloadWeaponHash;
	xStream << m_uStimActorAnimHash;
	xStream << m_uStimItemAnimHash;
	xStream << m_uStimWeaponAnimHash;
	xStream << m_uFlareActorAnimHash;
	xStream << m_uCloakActorAnimHash;
	xStream << m_uCloakWeaponAnimHash;
	xStream << m_uFidgetBPHash;
}

#endif // ASURA_TOOLS

void AVP_Actor_AnimStateData::ReadFromChunkStream( Asura_Chunk_Stream& xStream )
{
	u_int uVersion;
	xStream >> uVersion;

	Asura_Assertf( uVersion > 0 && uVersion <= g_uChunkVersion, ( "Invalid chunk version (%d)!", uVersion ) );
	if ( uVersion == 1 )
	{
		xStream >> m_fMoveDistance;
		xStream >> m_uCollisionDescriptorHash;
		xStream >> m_uSecondaryAnimHash;
		xStream >> m_uAimDownUpAnimHash;
		xStream >> m_uAimRightLeftAnimHash;
		xStream >> m_uPrimaryActorShootBeginHash;
		xStream >> m_uPrimaryWeaponShootBeginHash;
		xStream >> m_uSecondaryActorShootBeginHash;
		xStream >> m_uSecondaryWeaponShootBeginHash;
		xStream >> m_uPrimaryActorShootLoopHash;
		xStream >> m_uPrimaryWeaponShootLoopHash;
		xStream >> m_uSecondaryActorShootLoopHash;
		xStream >> m_uSecondaryWeaponShootLoopHash;
		xStream >> m_uPrimaryActorShootEndHash;
		xStream >> m_uPrimaryWeaponShootEndHash;
		xStream >> m_uSecondaryActorShootEndHash;
		xStream >> m_uSecondaryWeaponShootEndHash;
		xStream >> m_uWeaponInActorAnimHash;
		xStream >> m_uWeaponInWeaponAnimHash;
		xStream >> m_uWeaponOutActorAnimHash;
		xStream >> m_uWeaponOutWeaponAnimHash;
		xStream >> m_uWeaponRestWeaponAnimHash;
		xStream >> m_uReloadActorHash;
		xStream >> m_uReloadWeaponHash;
		xStream >> m_uStimActorAnimHash;
		xStream >> m_uStimItemAnimHash;
		xStream >> m_uStimWeaponAnimHash;
		xStream >> m_uFlareActorAnimHash;
		xStream >> m_uCloakActorAnimHash;
		xStream >> m_uCloakWeaponAnimHash;
		xStream >> m_uFidgetBPHash;
		Asura_Hash_ID uDummy;
		xStream >> uDummy; // old interaction anims blueprint

		m_uScopeInActorAnimHash = ASURA_HASH_ID_UNSET;
		m_uScopeOutActorAnimHash = ASURA_HASH_ID_UNSET;
	}
	else if ( uVersion == 2 )
	{
		xStream >> m_fMoveDistance;
		xStream >> m_uCollisionDescriptorHash;
		xStream >> m_uSecondaryAnimHash;
		xStream >> m_uAimDownUpAnimHash;
		xStream >> m_uAimRightLeftAnimHash;
		xStream >> m_uPrimaryActorShootBeginHash;
		xStream >> m_uPrimaryWeaponShootBeginHash;
		xStream >> m_uSecondaryActorShootBeginHash;
		xStream >> m_uSecondaryWeaponShootBeginHash;
		xStream >> m_uPrimaryActorShootLoopHash;
		xStream >> m_uPrimaryWeaponShootLoopHash;
		xStream >> m_uSecondaryActorShootLoopHash;
		xStream >> m_uSecondaryWeaponShootLoopHash;
		xStream >> m_uPrimaryActorShootEndHash;
		xStream >> m_uPrimaryWeaponShootEndHash;
		xStream >> m_uSecondaryActorShootEndHash;
		xStream >> m_uSecondaryWeaponShootEndHash;
		xStream >> m_uWeaponInActorAnimHash;
		xStream >> m_uWeaponInWeaponAnimHash;
		xStream >> m_uWeaponOutActorAnimHash;
		xStream >> m_uWeaponOutWeaponAnimHash;
		xStream >> m_uScopeInActorAnimHash;
		xStream >> m_uScopeOutActorAnimHash;
		xStream >> m_uWeaponRestWeaponAnimHash;
		xStream >> m_uReloadActorHash;
		xStream >> m_uReloadWeaponHash;
		xStream >> m_uStimActorAnimHash;
		xStream >> m_uStimItemAnimHash;
		xStream >> m_uStimWeaponAnimHash;
		xStream >> m_uFlareActorAnimHash;
		xStream >> m_uCloakActorAnimHash;
		xStream >> m_uCloakWeaponAnimHash;
		xStream >> m_uFidgetBPHash;
		Asura_Hash_ID uDummy;
		xStream >> uDummy; // old interaction anims blueprint
	}
	else if ( uVersion == 3 )
	{
		xStream >> m_fMoveDistance;
		xStream >> m_uCollisionDescriptorHash;
		xStream >> m_uSecondaryAnimHash;
		xStream >> m_uAimDownUpAnimHash;
		xStream >> m_uAimRightLeftAnimHash;
		xStream >> m_uPrimaryActorShootBeginHash;
		xStream >> m_uPrimaryWeaponShootBeginHash;
		xStream >> m_uSecondaryActorShootBeginHash;
		xStream >> m_uSecondaryWeaponShootBeginHash;
		xStream >> m_uPrimaryActorShootLoopHash;
		xStream >> m_uPrimaryWeaponShootLoopHash;
		xStream >> m_uSecondaryActorShootLoopHash;
		xStream >> m_uSecondaryWeaponShootLoopHash;
		xStream >> m_uPrimaryActorShootEndHash;
		xStream >> m_uPrimaryWeaponShootEndHash;
		xStream >> m_uSecondaryActorShootEndHash;
		xStream >> m_uSecondaryWeaponShootEndHash;
		xStream >> m_uWeaponInActorAnimHash;
		xStream >> m_uWeaponInWeaponAnimHash;
		xStream >> m_uWeaponOutActorAnimHash;
		xStream >> m_uWeaponOutWeaponAnimHash;
		xStream >> m_uScopeInActorAnimHash;
		xStream >> m_uScopeOutActorAnimHash;
		xStream >> m_uWeaponRestWeaponAnimHash;
		xStream >> m_uReloadActorHash;
		xStream >> m_uReloadWeaponHash;
		xStream >> m_uStimActorAnimHash;
		xStream >> m_uStimItemAnimHash;
		xStream >> m_uStimWeaponAnimHash;
		xStream >> m_uFlareActorAnimHash;
		xStream >> m_uCloakActorAnimHash;
		xStream >> m_uCloakWeaponAnimHash;
		xStream >> m_uFidgetBPHash;
	}
	else if ( uVersion == 4 )
	{
		xStream >> m_fMoveDistance;
		xStream >> m_uCollisionDescriptorHash;
		xStream >> m_uSecondaryAnimHash;
		xStream >> m_uAimDownUpAnimHash;
		xStream >> m_uAimDownUpDragAnimHash;
		xStream >> m_uAimRightLeftAnimHash;
		xStream >> m_uAimRightLeftDragAnimHash;
		xStream >> m_uStrafeDragAnimHash;
		xStream >> m_uPrimaryActorShootBeginHash;
		xStream >> m_uPrimaryWeaponShootBeginHash;
		xStream >> m_uSecondaryActorShootBeginHash;
		xStream >> m_uSecondaryWeaponShootBeginHash;
		xStream >> m_uPrimaryActorShootLoopHash;
		xStream >> m_uPrimaryWeaponShootLoopHash;
		xStream >> m_uSecondaryActorShootLoopHash;
		xStream >> m_uSecondaryWeaponShootLoopHash;
		xStream >> m_uPrimaryActorShootEndHash;
		xStream >> m_uPrimaryWeaponShootEndHash;
		xStream >> m_uSecondaryActorShootEndHash;
		xStream >> m_uSecondaryWeaponShootEndHash;
		xStream >> m_uWeaponInActorAnimHash;
		xStream >> m_uWeaponInWeaponAnimHash;
		xStream >> m_uWeaponOutActorAnimHash;
		xStream >> m_uWeaponOutWeaponAnimHash;
		xStream >> m_uScopeInActorAnimHash;
		xStream >> m_uScopeOutActorAnimHash;
		xStream >> m_uWeaponRestWeaponAnimHash;
		xStream >> m_uReloadActorHash;
		xStream >> m_uReloadWeaponHash;
		xStream >> m_uStimActorAnimHash;
		xStream >> m_uStimItemAnimHash;
		xStream >> m_uStimWeaponAnimHash;
		xStream >> m_uFlareActorAnimHash;
		xStream >> m_uCloakActorAnimHash;
		xStream >> m_uCloakWeaponAnimHash;
		xStream >> m_uFidgetBPHash;
	}
}

#ifdef ASURA_TOOLS

void AVP_Actor_AnimStateData::WriteToChunkStream( Asura_Chunk_Stream& xStream, const AVP_Actor_AnimStateData* pxStateData )
{
	xStream << g_uChunkHeaderVersion;

	bool bIsDirectional = pxStateData->IsDirectional();
	xStream << bIsDirectional;

	pxStateData->WriteToChunkStream( xStream );
}

#endif

AVP_Actor_AnimStateData* AVP_Actor_AnimStateData::CreateFromChunkStream( Asura_Chunk_Stream& xStream )
{
	AVP_Actor_AnimStateData* pxStateData = NULL;

	u_int uVersion;
	xStream >> uVersion;

	Asura_Assertf( uVersion > 0 && uVersion <= g_uChunkHeaderVersion, ( "Invalid chunk header version (%d)!", uVersion ) );
	if ( uVersion == 1 )
	{
		bool bIsDirectional;
		xStream >> bIsDirectional;

		if ( bIsDirectional )
		{
			pxStateData = new AVP_Actor_AnimStateData_Directional();
			pxStateData->ReadFromChunkStream( xStream );
		}
		else
		{
			pxStateData = new AVP_Actor_AnimStateData_NonDirectional();
			pxStateData->ReadFromChunkStream( xStream );
		}
	}

	return pxStateData;
}

//#ifdef ASURA_TOOLS

bool AVP_Actor_AnimStateData::GetParameterValueByString( const Asura_Blueprint& xBlueprint, const char* const szStateString, const char* const szParamString, const bool bAssertOnFailure, Asura_Value& xValueOut )
{
	bool bOK = true;
	Asura_String xStateString( szStateString );
	while ( !xBlueprint.GetParameterValueByHashID( ( xStateString + szParamString ).GetHash(), xValueOut ) && bOK )
	{
		// chop a bit off the string if we can( unless it's an into), otherwise we've failed.
		if ( xStateString.IsBlank() )
		{
			bOK = false;
		}
		else if ( xStateString.Contains( "->" ) )
		{
			xStateString.MakeBlank();
		}
		else
		{
			u_int uCharItr = xStateString.GetLength() - 1;
			while ( uCharItr != 0 && ( ( xStateString[ uCharItr ] >= '0' && xStateString[ uCharItr ] <= '9' ) || ( xStateString[ uCharItr ] >= 'A' && xStateString[ uCharItr ] <= 'Z' ) || ( xStateString[ uCharItr ] >= 'a' && xStateString[ uCharItr ] <= 'z' ) ) )
			{
				--uCharItr;
			}

			xStateString.Truncate( uCharItr );
		}
	}
//	Asura_Assertf( !bAssertOnFailure || bOK, ( "Couldn't get anim state BP parameter from blueprint \"%s\" with state string \"%s\", param string \"%s\"", xBlueprint.GetName().GetString(), szStateString, szParamString ) );
	return bOK;
}

//#endif // ASURA_TOOLS

AVP_Actor_AnimStateData_NonDirectional::AVP_Actor_AnimStateData_NonDirectional()
: PARENT()
, m_uAnimHash( ASURA_HASH_ID_UNSET )
, m_uTurn45LeftAnimHash( ASURA_HASH_ID_UNSET )
, m_uTurn45RightAnimHash( ASURA_HASH_ID_UNSET )
, m_uTurn90LeftAnimHash( ASURA_HASH_ID_UNSET )
, m_uTurn90RightAnimHash( ASURA_HASH_ID_UNSET )
, m_uTurn180LeftAnimHash( ASURA_HASH_ID_UNSET )
, m_uTurn180RightAnimHash( ASURA_HASH_ID_UNSET )
{
	
}

void AVP_Actor_AnimStateData_NonDirectional::InitialiseFromBlueprint( const Asura_Hash_ID uBPType, const Asura_Blueprint& xBlueprint, const char* const szStateString, const bool bAssertOnFailure )
{
	PARENT::InitialiseFromBlueprint( uBPType, xBlueprint, szStateString, bAssertOnFailure );

	Asura_Value xValue;
	if ( GetParameterValueByString( xBlueprint, szStateString, g_szNonDirLowerBodyString, m_uAnimHash == ASURA_HASH_ID_UNSET ? bAssertOnFailure : false, xValue ) )
	{
		m_uAnimHash = xValue.GetHashIDValue();
	}
	if ( GetParameterValueByString( xBlueprint, szStateString, g_szTurnLeft45String, false, xValue ) )
	{
		m_uTurn45LeftAnimHash = xValue.GetHashIDValue();
	}
	if ( GetParameterValueByString( xBlueprint, szStateString, g_szTurnLeft90String, false, xValue ) )
	{
		m_uTurn90LeftAnimHash = xValue.GetHashIDValue();
	}
	if ( GetParameterValueByString( xBlueprint, szStateString, g_szTurnLeft180String, false, xValue ) )
	{
		m_uTurn180LeftAnimHash = xValue.GetHashIDValue();
	}
	if ( GetParameterValueByString( xBlueprint, szStateString, g_szTurnRight45String, false, xValue ) )
	{
		m_uTurn45RightAnimHash = xValue.GetHashIDValue();
	}
	if ( GetParameterValueByString( xBlueprint, szStateString, g_szTurnRight90String, false, xValue ) )
	{
		m_uTurn90RightAnimHash = xValue.GetHashIDValue();
	}
	if ( GetParameterValueByString( xBlueprint, szStateString, g_szTurnRight180String, false, xValue ) )
	{
		m_uTurn180RightAnimHash = xValue.GetHashIDValue();
	}

	if ( m_uAnimHash != ASURA_HASH_ID_UNSET )
	{
		Asura_Vector_3 xOffset;
		const Asura_Hierarchy_Anim* pxHierarchyAnim	= Asura_Animation_System::GetAnimByHash( m_uAnimHash );
		if ( pxHierarchyAnim && pxHierarchyAnim->GetMovementOffset( NULL, 1.0f, &xOffset ) )
		{
			m_fMoveDistance = xOffset.Magnitude();
		}
	}
}

#ifdef ASURA_TOOLS

void AVP_Actor_AnimStateData_NonDirectional::WriteToChunkStream( Asura_Chunk_Stream& xStream ) const
{
	xStream << g_uNonDirectionalChunkVersion;

	PARENT::WriteToChunkStream( xStream );

	xStream << m_uAnimHash;
	xStream << m_uTurn45LeftAnimHash;
	xStream << m_uTurn45RightAnimHash;
	xStream << m_uTurn90LeftAnimHash;
	xStream << m_uTurn90RightAnimHash;
	xStream << m_uTurn180LeftAnimHash;
	xStream << m_uTurn180RightAnimHash;
}

#endif

void AVP_Actor_AnimStateData_NonDirectional::ReadFromChunkStream( Asura_Chunk_Stream& xStream )
{
	u_int uVersion;
	xStream >> uVersion;

	Asura_Assertf( uVersion > 0 && uVersion <= g_uNonDirectionalChunkVersion, ( "Invalid chunk version (%d)!", uVersion ) );
	if ( uVersion == 2 )
	{
		PARENT::ReadFromChunkStream( xStream );

		xStream >> m_uAnimHash;
		xStream >> m_uTurn45LeftAnimHash;
		xStream >> m_uTurn45RightAnimHash;
		xStream >> m_uTurn90LeftAnimHash;
		xStream >> m_uTurn90RightAnimHash;
		xStream >> m_uTurn180LeftAnimHash;
		xStream >> m_uTurn180RightAnimHash;
	}
	else if ( uVersion == 1 )
	{
		PARENT::ReadFromChunkStream( xStream );

		xStream >> m_uAnimHash;
		xStream >> m_uTurn45LeftAnimHash;
		xStream >> m_uTurn45RightAnimHash;
		xStream >> m_uTurn90LeftAnimHash;
		xStream >> m_uTurn90RightAnimHash;

		m_uTurn180LeftAnimHash = ASURA_HASH_ID_UNSET;
		m_uTurn180RightAnimHash = ASURA_HASH_ID_UNSET;
	}
}

AVP_Actor_AnimStateData_Directional_Region::AVP_Actor_AnimStateData_Directional_Region()
: m_fTweenAngleACW( 0.0f )
, m_fTweenAngleCW( 0.0f )
, m_uNumAnims( 0 )
{
	for ( u_int uItr = 0; uItr < AVP_ACTOR_ANIMSTATEDATA_MAX_ANIMS_PER_REGION; ++uItr )
	{
		m_afAngle[ uItr ]				= 0.0f;
		m_auTweenOnAnimFlags[ uItr ]	= 0;
	}
}


void AVP_Actor_AnimStateData_Directional_Region::Initialise( const AVP_Actor_AnimStateData& xParentStateData, const Asura_Hash_ID uBPTypeHash, const Asura_Blueprint& xBP, const u_int uRegionItr, const bool bAssertOnFailure )
{
	char szRegionNumBuffer[3];
	Asura_PrintfToBfr( szRegionNumBuffer, "%02d", 3, true, uRegionItr + 1 );
	Asura_String xRegionString = Asura_String( "Region." ) + szRegionNumBuffer;
	Asura_Value xValue;
	if ( xBP.GetParameterValueByHashID( ( xRegionString + ".TweenAngle.ACW" ).GetHash(), xValue ) )
	{
		m_fTweenAngleACW = Asura_Maths::RadiansToZeroMinimumRange( Asura_Maths::DegToRad( xValue.GetFloatValue() ) );
	}
	if ( xBP.GetParameterValueByHashID( ( xRegionString + ".TweenAngle.CW" ).GetHash(), xValue ) )
	{
		m_fTweenAngleCW = Asura_Maths::RadiansToZeroMinimumRange( Asura_Maths::DegToRad( xValue.GetFloatValue() ) );
	}

	AVP_Blueprint_ParameterEnumerator_AnimState_Directional_Region_AnimData xAnimDataEnumerator( xParentStateData, uBPTypeHash, xBP, m_axAnimData, bAssertOnFailure );
	AVP_Blueprint_ParameterEnumerator_AnimState_Directional_Region_Angle xAngleEnumerator( m_afAngle );
	m_uNumAnims = xBP.EnumerateParameters( ( xRegionString + ".Anim." ).GetString(), 2, xAnimDataEnumerator );
	xBP.EnumerateParameters( ( xRegionString + ".Angle." ).GetString(), 2, xAngleEnumerator );
}

#ifdef ASURA_TOOLS

void AVP_Actor_AnimStateData_Directional_Region::WriteToChunkStream( Asura_Chunk_Stream& xStream ) const
{
	xStream << g_uDirectionalRegionChunkVersion;

	xStream << m_fTweenAngleACW;
	xStream << m_fTweenAngleCW;
	xStream << m_uNumAnims;

	for ( u_int uItr = 0; uItr < m_uNumAnims; ++uItr )
	{
		xStream << m_afAngle[ uItr ];
		xStream << m_auTweenOnAnimFlags[ uItr ];
		m_axAnimData[ uItr ].WriteToChunkStream( xStream );
	}
}

#endif

void AVP_Actor_AnimStateData_Directional_Region::ReadFromChunkStream( Asura_Chunk_Stream& xStream )
{
	u_int uVersion;
	xStream >> uVersion;

	Asura_Assertf( uVersion > 0 && uVersion <= g_uDirectionalRegionChunkVersion, ( "Invalid chunk version (%d)!", uVersion ) );
	if ( uVersion == 1 )
	{
		xStream >> m_fTweenAngleACW;
		xStream >> m_fTweenAngleCW;
		xStream >> m_uNumAnims;

		Asura_Assert( m_uNumAnims <= AVP_ACTOR_ANIMSTATEDATA_MAX_ANIMS_PER_REGION, "Too many anims in blend region in chunk data, who's been messing with my code!" );
		for ( u_int uItr = 0; uItr < m_uNumAnims; ++uItr )
		{
			if ( m_uNumAnims <= AVP_ACTOR_ANIMSTATEDATA_MAX_ANIMS_PER_REGION )
			{
				xStream >> m_afAngle[ uItr ];
				xStream >> m_auTweenOnAnimFlags[ uItr ];
				m_axAnimData[ uItr ].ReadFromChunkStream( xStream );
			}
			else
			{
				float fDummy;
				u_int uDummy;
				AVP_Actor_AnimStateData_NonDirectional xDummy;
				xStream >> fDummy;
				xStream >> uDummy;
				xDummy.ReadFromChunkStream( xStream );
			}
		}
		if ( m_uNumAnims > AVP_ACTOR_ANIMSTATEDATA_MAX_ANIMS_PER_REGION )
		{
			m_uNumAnims = AVP_ACTOR_ANIMSTATEDATA_MAX_ANIMS_PER_REGION;
		}
	}
}

AVP_Actor_AnimStateData_Directional::AVP_Actor_AnimStateData_Directional()
: PARENT()
, m_uBlendTemplateHash( ASURA_HASH_ID_UNSET )
, m_uNumBlendRegions( 0 )
{

}

#ifdef ASURA_TOOLS

void AVP_Actor_AnimStateData_Directional::WriteToChunkStream( Asura_Chunk_Stream& xStream ) const
{
	xStream << g_uDirectionalChunkVersion;

	PARENT::WriteToChunkStream( xStream );

	xStream << m_uBlendTemplateHash;
	xStream << m_uNumBlendRegions;

	for ( u_int uItr = 0; uItr < m_uNumBlendRegions; ++uItr )
	{
		m_axRegion[ uItr ].WriteToChunkStream( xStream );
	}
}

#endif

void AVP_Actor_AnimStateData_Directional::ReadFromChunkStream( Asura_Chunk_Stream& xStream )
{
	u_int uVersion;
	xStream >> uVersion;

	Asura_Assertf( uVersion > 0 && uVersion <= g_uNonDirectionalChunkVersion, ( "Invalid chunk version (%d)!", uVersion ) );
	if ( uVersion == 1 )
	{
		PARENT::ReadFromChunkStream( xStream );

		xStream >> m_uBlendTemplateHash;
		xStream >> m_uNumBlendRegions;

		Asura_Assert( m_uNumBlendRegions <= AVP_ACTOR_ANIMSTATEDATA_MAX_BLEND_REGIONS, "Too many blend regions in chunk data, who's been messing with my code!" );
		for ( u_int uItr = 0; uItr < m_uNumBlendRegions; ++uItr )
		{
			if ( m_uNumBlendRegions <= AVP_ACTOR_ANIMSTATEDATA_MAX_BLEND_REGIONS )
			{
				m_axRegion[ uItr ].ReadFromChunkStream( xStream );
			}
			else
			{
				AVP_Actor_AnimStateData_Directional_Region xDummy;
				xDummy.ReadFromChunkStream( xStream );
			}
		}
		if ( m_uNumBlendRegions > AVP_ACTOR_ANIMSTATEDATA_MAX_BLEND_REGIONS )
		{
			m_uNumBlendRegions = AVP_ACTOR_ANIMSTATEDATA_MAX_BLEND_REGIONS;
		}
	}
}


void AVP_Actor_AnimStateData_Directional::InitialiseFromBlueprint( const Asura_Hash_ID uBPType, const Asura_Blueprint& xBlueprint, const char* const szStateString, const bool bAssertOnFailure )
{
	PARENT::InitialiseFromBlueprint( uBPType, xBlueprint, szStateString, bAssertOnFailure );

	Asura_Value xValue;
	if ( GetParameterValueByString( xBlueprint, szStateString, g_szDirBlendLowerBodyString, bAssertOnFailure, xValue ) )
	{
		m_uBlendTemplateHash = xValue.GetHashIDValue();
		Asura_Blueprint* pxTemplateBP = Asura_Blueprint_System::GetBlueprintByHashID( uBPType, m_uBlendTemplateHash );
		if ( pxTemplateBP )
		{
			AVP_Blueprint_ParameterEnumerator_AnimState_Directional_Region xRegionEnumerator( *this, uBPType, *pxTemplateBP, m_axRegion, bAssertOnFailure );
			m_uNumBlendRegions = pxTemplateBP->EnumerateParameters( "Region.", 2, xRegionEnumerator );

			if ( m_uNumBlendRegions > 0 && m_axRegion[ 0 ].GetNumAnims() > 0 && m_axRegion[ 0 ].GetAnimData( 0 ) != NULL  )
			{
				Asura_Vector_3 xOffset;
				const Asura_Hierarchy_Anim* pxHierarchyAnim	= Asura_Animation_System::GetAnimByHash( m_axRegion[ 0 ].GetAnimData( 0 )->GetAnimHash() );
				if ( pxHierarchyAnim && pxHierarchyAnim->GetMovementOffset( NULL, 1.0f, &xOffset ) )
				{
					m_fMoveDistance = xOffset.Magnitude();
				}
			}
		}
	}
}

//#ifdef ASURA_TOOLS

bool AVP_Blueprint_ParameterEnumerator_AnimState_Directional_Region::Enumerate( const u_int uIndex, char const* const szName, const Asura_Value& xValue )
{
	bool bOK = uIndex < AVP_ACTOR_ANIMSTATEDATA_MAX_BLEND_REGIONS;
	Asura_Assert( uIndex < AVP_ACTOR_ANIMSTATEDATA_MAX_BLEND_REGIONS, "Too many blend regions in state blueprint" );
	if ( bOK )
	{
		m_pxRegion[ uIndex ].Initialise( m_xParentStateData, m_uBPTypeHash, m_xBP, uIndex, m_bAssertOnFailure );
	}

	return bOK;
}

bool AVP_Blueprint_ParameterEnumerator_AnimState_Directional_Region_Angle::Enumerate( const u_int uIndex, char const* const szName, const Asura_Value& xValue )
{
	bool bOK = uIndex < AVP_ACTOR_ANIMSTATEDATA_MAX_ANIMS_PER_REGION;
	Asura_Assert( uIndex < AVP_ACTOR_ANIMSTATEDATA_MAX_ANIMS_PER_REGION, "Too many angles in blend region" );
	if ( bOK )
	{
		m_pfAngle[ uIndex ] = Asura_Maths::RadiansToZeroMinimumRange( Asura_Maths::DegToRad( xValue.GetFloatValue() ) );
	}

	return bOK;
}

bool AVP_Blueprint_ParameterEnumerator_AnimState_Directional_Region_AnimData::Enumerate( const u_int uIndex, char const* const szName, const Asura_Value& xValue )
{
	bool bOK = uIndex < AVP_ACTOR_ANIMSTATEDATA_MAX_ANIMS_PER_REGION;
	Asura_Assert( uIndex < AVP_ACTOR_ANIMSTATEDATA_MAX_ANIMS_PER_REGION, "Too many anims in blend region" );
	if ( bOK )
	{
		m_pxAnimData[ uIndex ].CopyFrom( m_xParentStateData );
		m_pxAnimData[ uIndex ].InitialiseFromBlueprint( m_uBPTypeHash, m_xBP, szName, m_bAssertOnFailure );
	}

	return bOK;
}

//#endif // ASURA_TOOLS
