///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_Actor_DamageDirectionIndicator.cpp
// Author : Chris Wallace
// Date   : 25-Sept-2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "Actor/AVP_Actor_DamageDirectionIndicator.h"

// Asura
#include "Blueprint/Asura_Blueprint_System.h"
#include "Camera/Asura_Camera.h"
#include "Core/Asura_Maths.h"
#include "Core/Asura_Timers.h"
#include "Interface/Asura_CommandConsole.h"
#include "GUIMenu/Asura_GUIMenu_Element_Image.h"
#include "GUIMenu/Asura_GUIMenu_Menu_Page.h"
#include "Rendering/Asura_Viewport.h"

// AVP
#include "Actor/Vertebrates/AVP_Vertebrate_Description.h"
#include "Blueprint/AVP_Blueprint_HashIDs.h"
#include "Entity/Damage/AVP_EntityDamage_Apply.h"
#include "Entity/PhysicalObject/Actor/AVP_ClientEntity_Actor.h"
#include "GUIMenu/AVP_GUIMenu_System.h"
#include "Rendering/AVP_Overlay_Render.h"

#define HUD_SHOWDAMAGEINDICATOR1	L"AVP.HUD.ShowDamageIndicator1"
#define HUD_SHOWDAMAGEINDICATOR2	L"AVP.HUD.ShowDamageIndicator2"

#define HUD_DAMAGEHIT_TOPLEFT1		L"AVP.HUD.DamageHit.TopLeft1"
#define HUD_DAMAGEHIT_TOPLEFT2		L"AVP.HUD.DamageHit.TopLeft2"
#define HUD_DAMAGEHIT_TOPLEFT3		L"AVP.HUD.DamageHit.TopLeft3"
#define HUD_DAMAGEHIT_TOPCENTRE1	L"AVP.HUD.DamageHit.TopCentre1"
#define HUD_DAMAGEHIT_TOPCENTRE2	L"AVP.HUD.DamageHit.TopCentre2"
#define HUD_DAMAGEHIT_TOPCENTRE3	L"AVP.HUD.DamageHit.TopCentre3"
#define HUD_DAMAGEHIT_TOPRIGHT1		L"AVP.HUD.DamageHit.TopRight1"
#define HUD_DAMAGEHIT_TOPRIGHT2		L"AVP.HUD.DamageHit.TopRight2"
#define HUD_DAMAGEHIT_TOPRIGHT3		L"AVP.HUD.DamageHit.TopRight3"
#define HUD_DAMAGEHIT_CENTRERIGHT1	L"AVP.HUD.DamageHit.CentreRight1"
#define HUD_DAMAGEHIT_CENTRERIGHT2	L"AVP.HUD.DamageHit.CentreRight2"
#define HUD_DAMAGEHIT_CENTRERIGHT3	L"AVP.HUD.DamageHit.CentreRight3"
#define HUD_DAMAGEHIT_BOTTOMRIGHT1	L"AVP.HUD.DamageHit.BottomRight1"
#define HUD_DAMAGEHIT_BOTTOMRIGHT2	L"AVP.HUD.DamageHit.BottomRight2"
#define HUD_DAMAGEHIT_BOTTOMRIGHT3	L"AVP.HUD.DamageHit.BottomRight3"
#define HUD_DAMAGEHIT_BOTTOMCENTRE1	L"AVP.HUD.DamageHit.BottomCentre1"
#define HUD_DAMAGEHIT_BOTTOMCENTRE2	L"AVP.HUD.DamageHit.BottomCentre2"
#define HUD_DAMAGEHIT_BOTTOMCENTRE3	L"AVP.HUD.DamageHit.BottomCentre3"
#define HUD_DAMAGEHIT_BOTTOMLEFT1	L"AVP.HUD.DamageHit.BottomLeft1"
#define HUD_DAMAGEHIT_BOTTOMLEFT2	L"AVP.HUD.DamageHit.BottomLeft2"
#define HUD_DAMAGEHIT_BOTTOMLEFT3	L"AVP.HUD.DamageHit.BottomLeft3"
#define HUD_DAMAGEHIT_CENTRELEFT1	L"AVP.HUD.DamageHit.CentreLeft1"
#define HUD_DAMAGEHIT_CENTRELEFT2	L"AVP.HUD.DamageHit.CentreLeft2"
#define HUD_DAMAGEHIT_CENTRELEFT3	L"AVP.HUD.DamageHit.CentreLeft3"

bool  AVP_Actor_DamageDirectionIndicator_System::s_bShowDamageIndicator[ MAX_DAMAGE_INDICATORS ];
float AVP_Actor_DamageDirectionIndicator_System::DamageHitEffect::s_TimeToFade = 15.0f;
u_int AVP_Actor_DamageDirectionIndicator_System::DamageHitEffect::s_NumOfDifferentTypes = 3;

DEBUGVAR float dbg_fDamageLifeTime		= 1.5f;
DEBUGVAR float dbg_fMinTimeBetweenAdd	= 1.0f;

static float g_afDDIRadius[ MAX_DAMAGE_INDICATORS ];


AVP_Actor_DamageDirectionIndicator_System::DamageHitEffect AVP_Actor_DamageDirectionIndicator_System::s_xDamageHitEffect[AVP_Actor_DamageDirectionIndicator_System::DAMAGEHIT_MAX];

///////////////////////////////
// Damage Indicator System
///////////////////////////////

AVP_Actor_DamageDirectionIndicator_System::AVP_Actor_DamageDirectionIndicator_System( const AVP_ClientEntity_Actor& xActor )
: m_xOwnerActor( xActor )
{
	Reset();

	// Set some static variables.
	Asura_Blueprint* pxBlueprint = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_PLAYERS, xActor.GetBlueprintHashID() );
	if( pxBlueprint )
	{
		Asura_Value xValue;

		if (pxBlueprint->GetParameterValueByHashID( 0x13DD118F, xValue))	// "DamageIndicator.NumOfDifferentTypes"
		{
			DamageHitEffect::s_NumOfDifferentTypes = xValue.GetIntValue();
		}

		if (pxBlueprint->GetParameterValueByHashID( 0x306843B2, xValue))	// "DamageIndicator.TimeToFade"
		{
			DamageHitEffect::s_TimeToFade = xValue.GetFloatValue();
		}
	}
}

void AVP_Actor_DamageDirectionIndicator_System::Add( const AVP_EntityDamage_ApplyToEntity& xDamage )
{
	if (xDamage.GetAmount() <= 0.f) return;

	Asura_Vector_3 xOurPosition = m_xOwnerActor.GetCurrentPosition();

	AVP_ClientEntity_PhysicalObject* pxSourceOfDamage = AVP_ClientEntity_PhysicalObject::FindEntity(xDamage.GetFromGuid());
	if (!pxSourceOfDamage) return; 

	Asura_Vector_3 xSourcePosition = pxSourceOfDamage->GetCurrentPosition();

	// SJL - it might be that this should be an 'eyes to eyes' direction instead of feet to feet, it should be ok
	// in most circumstances, but if the damage is unreliable with a strangley oriented aliens vs small objects we might need to 
	// revisit this.

	Asura_Vector_3 xDirectionToSource = xSourcePosition - xOurPosition;

	if (xDirectionToSource.ApproximatelyEquals(Asura_Maths::ZeroVector3))
	{
		// Hit by ourselves, or maybe something right ontop of us. Lets pretend its slightly behind us.
		Asura_Vector_3 xForward;
		Asura_Camera::GetForwardVector(xForward);
		xDirectionToSource = -xForward;
	}

	AddDamageHitEffect( xDirectionToSource );

	if ( m_fTimeSinceLastAdd > dbg_fMinTimeBetweenAdd ) //Only add if enough time has passed
	{
		m_axDirection[ m_uCurrentIndex ] = xDirectionToSource;
		m_afLifeTimer[ m_uCurrentIndex ] = dbg_fDamageLifeTime;

		m_fTimeSinceLastAdd = 0.0f;

		if ( ++m_uCurrentIndex == MAX_DAMAGE_INDICATORS )
		{
			m_uCurrentIndex = 0;
		}
	}
}

void AVP_Actor_DamageDirectionIndicator_System::Update()
{	
	for( u_int u=0; u<DAMAGEHIT_MAX; u++ )
	{
		s_xDamageHitEffect[u].Update();
	}

	m_fTimeSinceLastAdd += Asura_Timers::GetGameTimeSlice(); // Update timer

	const float fScreenWidth = Asura_Viewport::GetWidth() * Asura_Camera::GetViewingArea()->m_fWidth;
	const float fScreenHeight = Asura_Viewport::GetHeight() * Asura_Camera::GetViewingArea()->m_fHeight;
	const float fCenterPosX = fScreenWidth*0.5f;
	const float fCenterPosY = fScreenHeight*0.5f;

	Asura_Matrix_3x3 xMatrix;
	Asura_Hash_ID uCameraHashID = AVP_Vertebrate_Description::GetMarkerPointHashID( AVP_Vertebrate_Description::MPID_EYES ); 

	m_xOwnerActor.GetWorldSpaceMarkerPoint( uCameraHashID, NULL, &xMatrix ); // Camera

	for ( u_int uItr = 0; uItr < MAX_DAMAGE_INDICATORS; ++uItr )
	{
		Asura_GUIMenu_Element_Image* pxImageItem = static_cast< Asura_GUIMenu_Element_Image* >( AVP_GUIMenu_System::GetHUDElement( static_cast < AVP_GUIMenu_System::STORED_WIDGETS >( AVP_GUIMenu_System::BEGIN_DAMAGE_DIRECTION_INDICATOR + uItr ) ) );
		if ( pxImageItem )
		{
			Asura_Bounding_Rectangle xBounds = pxImageItem->GetScaledWorldBounds();
			if ( g_afDDIRadius[ uItr ] == 0.0f )
			{

				const float fDeltaX = fCenterPosX - xBounds.CalculateCentre().x;
				const float fDeltaY = fCenterPosY - xBounds.CalculateCentre().y;
				g_afDDIRadius[ uItr ] = Asura_Maths::Sqrt( fDeltaX * fDeltaX + fDeltaY * fDeltaY );
			}

			if ( s_bShowDamageIndicator[ uItr ] )
			{
				m_afLifeTimer[ uItr ] = Asura_Maths::Max( 0.0f, m_afLifeTimer[ uItr ] - Asura_Timers::GetGameTimeSlice() );

				Asura_Vector_3 xDir = xMatrix.ApplyTranspose( m_axDirection[ uItr ] );
				const float fNorm = Asura_Maths::Sqrt( xDir.x * xDir.x + xDir.z * xDir.z );
				const float fNormXPos = fNorm > 0.0f ? xDir.x / fNorm : 0.0f;
				const float fNormYPos = fNorm > 0.0f ? -xDir.z / fNorm : -1.0f;

				const float fObjectX = fCenterPosX + fNormXPos * g_afDDIRadius[ uItr ] - 0.5f * xBounds.GetWidth();
				const float fObjectY = fCenterPosY + fNormYPos * g_afDDIRadius[ uItr ] - 0.5f * xBounds.GetHeight();

				const float fAngle = Asura_Maths::ArcTangent( fNormXPos, -fNormYPos );

				xBounds.Displace(Asura_Vector_2(fObjectX, fObjectY ) - xBounds.GetTopLeft());
				pxImageItem->SetScaledWorldBounds(xBounds);
				pxImageItem->SetRotation( fAngle );
				Asura_RGBA xColour = pxImageItem->GetColour();
				xColour.m_fA = m_afLifeTimer[ uItr ] / dbg_fDamageLifeTime;
				pxImageItem->SetColour( xColour );
			}
			s_bShowDamageIndicator[ uItr ] = m_afLifeTimer[ uItr ] > 0.0f;
		}
	}
}

void AVP_Actor_DamageDirectionIndicator_System::Reset()
{
	for( u_int u=0; u<DAMAGEHIT_MAX; u++ )
	{
		s_xDamageHitEffect[u].Reset();
	}

	m_uCurrentIndex = 0;
	m_fTimeSinceLastAdd = dbg_fMinTimeBetweenAdd;
	for ( u_int uItr = 0; uItr < MAX_DAMAGE_INDICATORS; ++uItr )
	{
		g_afDDIRadius[ uItr ] = 0.0f;
		m_afLifeTimer[ uItr ] = 0.0f;
		//s_bShowDamageIndicator[ uItr ] = false;
	}
}

void AVP_Actor_DamageDirectionIndicator_System::CreateConsoleVars()
{
	Asura_CommandConsole::AddVar( HUD_SHOWDAMAGEINDICATOR1, &s_bShowDamageIndicator[ 0 ], Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::MakeVarReadOnly( HUD_SHOWDAMAGEINDICATOR1 );

	Asura_CommandConsole::AddVar( HUD_SHOWDAMAGEINDICATOR2, &s_bShowDamageIndicator[ 1 ], Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::MakeVarReadOnly( HUD_SHOWDAMAGEINDICATOR2 );

	s_xDamageHitEffect[DAMAGEHIT_TOPLEFT	].AddConsoleVar( HUD_DAMAGEHIT_TOPLEFT1,	 HUD_DAMAGEHIT_TOPLEFT2,	 HUD_DAMAGEHIT_TOPLEFT3 );
	s_xDamageHitEffect[DAMAGEHIT_TOPCENTRE	].AddConsoleVar( HUD_DAMAGEHIT_TOPCENTRE1,	 HUD_DAMAGEHIT_TOPCENTRE2,	 HUD_DAMAGEHIT_TOPCENTRE3 );
	s_xDamageHitEffect[DAMAGEHIT_TOPRIGHT	].AddConsoleVar( HUD_DAMAGEHIT_TOPRIGHT1,	 HUD_DAMAGEHIT_TOPRIGHT2,	 HUD_DAMAGEHIT_TOPRIGHT3 );
	s_xDamageHitEffect[DAMAGEHIT_CENTRERIGHT].AddConsoleVar( HUD_DAMAGEHIT_CENTRERIGHT1, HUD_DAMAGEHIT_CENTRERIGHT2, HUD_DAMAGEHIT_CENTRERIGHT3 );
	s_xDamageHitEffect[DAMAGEHIT_BOTTOMRIGHT].AddConsoleVar( HUD_DAMAGEHIT_BOTTOMRIGHT1, HUD_DAMAGEHIT_BOTTOMRIGHT2, HUD_DAMAGEHIT_BOTTOMRIGHT3 );
	s_xDamageHitEffect[DAMAGEHIT_BOTTOMCENTRE].AddConsoleVar(HUD_DAMAGEHIT_BOTTOMCENTRE1,HUD_DAMAGEHIT_BOTTOMCENTRE2,HUD_DAMAGEHIT_BOTTOMCENTRE3 );
	s_xDamageHitEffect[DAMAGEHIT_BOTTOMLEFT	].AddConsoleVar( HUD_DAMAGEHIT_BOTTOMLEFT1,  HUD_DAMAGEHIT_BOTTOMLEFT2,  HUD_DAMAGEHIT_BOTTOMLEFT3 );
	s_xDamageHitEffect[DAMAGEHIT_CENTRELEFT	].AddConsoleVar( HUD_DAMAGEHIT_CENTRELEFT1,	 HUD_DAMAGEHIT_CENTRELEFT2,  HUD_DAMAGEHIT_CENTRELEFT3 );
}



void AVP_Actor_DamageDirectionIndicator_System::AddDamageHitEffect( const Asura_Vector_3& xDirection )
{
	// Calculate correct angle/
	Asura_Matrix_3x3 xMatrix;
	Asura_Hash_ID uCameraHashID = AVP_Vertebrate_Description::GetMarkerPointHashID( AVP_Vertebrate_Description::MPID_EYES ); 

	m_xOwnerActor.GetWorldSpaceMarkerPoint( uCameraHashID, NULL, &xMatrix ); // Camera

	Asura_Vector_3 xDir = xMatrix.ApplyTranspose( xDirection );
	const float fNorm = Asura_Maths::Sqrt( xDir.x * xDir.x + xDir.z * xDir.z );
	const float fNormXPos = fNorm > 0.0f ? xDir.x / fNorm : 0.0f;
	const float fNormYPos = fNorm > 0.0f ? -xDir.z / fNorm : -1.0f;

	// Now we have final angle.
	float fAngle = Asura_Maths::ArcTangent( fNormXPos, -fNormYPos );

	// So it is time to convert it to our new table
	fAngle += (Asura_Maths::DegToRadConstant * 67.5f);

	// Make sure that angle is in radius 0 degree to 359 degree.
	while( fAngle<0.0f )					{fAngle+=Asura_Maths::Deg360;}
	while( fAngle>=Asura_Maths::Deg360 )	{fAngle-=Asura_Maths::Deg360;}

	u_int uWhichOne= static_cast<u_int>(fAngle / (Asura_Maths::DegToRadConstant * 45.0f));

	if( uWhichOne<DAMAGEHIT_MAX )
	{
		s_xDamageHitEffect[uWhichOne].HitMe();
	}
	else
	{
		Asura_Assert( false, "Should never reach this code: AVP_Actor_DamageDirectionIndicator_System::AddDamageHitEffect");	// It seems that we can't translate angle from 0 to 359 to 8 directions
	}
}

//------------------------------------------ DamageHitEffect class ------------------------------------------

void AVP_Actor_DamageDirectionIndicator_System::DamageHitEffect::AddConsoleVar( const Asura_UnicodeChar* wszVar1, const Asura_UnicodeChar* wszVar2, const Asura_UnicodeChar* wszVar3 )
{
	Asura_CommandConsole::AddVar( wszVar1, &m_uCurrentState[0], 0, INT_MAX, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::MakeVarReadOnly( wszVar1 );

	Asura_CommandConsole::AddVar( wszVar2, &m_uCurrentState[1], 0, INT_MAX, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::MakeVarReadOnly( wszVar2 );

	Asura_CommandConsole::AddVar( wszVar3, &m_uCurrentState[2], 0, INT_MAX, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::MakeVarReadOnly( wszVar3 );
}

void AVP_Actor_DamageDirectionIndicator_System::DamageHitEffect::Update()
{
	for( int i=0; i<s_iMaxStates; i++ )
	{
		if( m_fTimer[i]>0.0f )
		{
			m_fTimer[i] -= Asura_Timers::GetGameTimeSlice();

			if( m_fTimer[i]<=0.0f )
			{
				m_fTimer[i] =0.0f;
				m_uCurrentState[i] = 0;
			}
		}
	}
}

void AVP_Actor_DamageDirectionIndicator_System::DamageHitEffect::HitMe()
{
	// We should have 3 slots. Firstly check, if all of them are occupied, or if some of them are sleeping.

	int uCountSleepingSlots = 0;
	for( int i=0; i<s_iMaxStates; i++ )
	{
		if( m_fTimer[i]==0.0f )
		{
			uCountSleepingSlots++;
		}
	}

	// If we have at least one sleeping slot, that we can make it active, lets do it. However, we don't want to
	// make first slot active. We want to do it by random. So lets choose by random, some empty slot.
	if( uCountSleepingSlots )
	{
		// Choose some random sleeping slot to activate.
		int uActualEmptySlot = uCountSleepingSlots==1 ? 1 : Asura_Random::GetInt(1,uCountSleepingSlots);


		// Activate randomly selected sleeping slot
		for( int i=0; i<s_iMaxStates; i++ )
		{
			if( m_fTimer[i]==0.0f )
			{
				if( uActualEmptySlot==1 )
				{
					m_fTimer[i] = s_TimeToFade;	// Activate sleeping slot, by setting the Timer
					m_uCurrentState[i] = 1;
					break;
				}
				else
				{
					uActualEmptySlot--;
				}
			}
		}
	}
}


// End of file
