
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_Dropship_AnimationHandler.cpp
// Author : Sharon
// Date   : 6-Oct-2008
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "Actor/Vehicles/Dropship/AVP_Dropship_AnimationHandler.h"

// Asura independent header-files...
#include "Animation/Asura_Animation.h"
#include "Animation/Asura_AnimationBlend.h"
#include "Animation/Asura_AnimationController_Animation.h"
#include "Blueprint/Asura_Blueprint.h"

// Game independent header-files...
#include "Blueprint/AVP_Blueprint_HashIDs.h"
#include "Entity/PhysicalObject/Actor/Vehicles/Dropship/AVP_ClientVehicle_Dropship.h"
#include "Entity/PhysicalObject/Actor/Vehicles/Dropship/AVP_Messages_Dropship.h"


////////////////////////////////////////////////////////////////////////////////
// Constant declarations
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Static variable declarations
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// AVP_Dropship_AnimationHandler
////////////////////////////////////////////////////////////////////////////////
AVP_Dropship_AnimationHandler::AVP_Dropship_AnimationHandler(AVP_ClientVehicle_Dropship& xOwner)
: PARENT(xOwner)
, m_xOwnerDropship(xOwner)
, m_eFrontPods(OCS_CLOSED)
, m_eBackPods(OCS_CLOSED)
, m_eBackPodsDoors(OCS_CLOSED)
, m_uFrontPodsAnimIDCloseToOpen(ASURA_HASH_ID_UNSET)
, m_uBackPodsAnimIDCloseToOpen(ASURA_HASH_ID_UNSET)
, m_uBackPodsDoorsAnimIDCloseToOpen(ASURA_HASH_ID_UNSET)
, m_uFrontPodsAnimIDOpenToClose(ASURA_HASH_ID_UNSET)
, m_uBackPodsAnimIDOpenToClose(ASURA_HASH_ID_UNSET)
, m_uBackPodsDoorsAnimIDOpenToClose(ASURA_HASH_ID_UNSET)
, m_uFrontPodsControllerID(ASURA_HASH_ID_UNSET)
, m_uBackPodsControllerID(ASURA_HASH_ID_UNSET)
, m_uBackPodsDoorsControllerID(ASURA_HASH_ID_UNSET)
, m_uFrontPodsControllerGroupID(ASURA_HASH_ID_UNSET)
, m_uBackPodsControllerGroupID(ASURA_HASH_ID_UNSET)
, m_uBackPodsDoorsControllerGroupID(ASURA_HASH_ID_UNSET)
{
}

void AVP_Dropship_AnimationHandler::HandleDropshipAnimationRequest( const AVP_Message_Dropship_RequestAnimation& xAnimRequestData )
{
	if (xAnimRequestData.m_eRequestType < AVP_Message_Dropship_RequestAnimation::REQUEST_ANIM_LAST_ANIM_WITH_TIME)
	{
		HandleRequestWithTime(xAnimRequestData);
		return;
	}

	switch (xAnimRequestData.m_eRequestType)
	{
	case AVP_Message_Dropship_RequestAnimation::REQUEST_ANIM_OPEN_FRONT_MISSILE_PODS:
		OpenFrontPods();
		break;
	case AVP_Message_Dropship_RequestAnimation::REQUEST_ANIM_OPEN_BACK_MISSILE_PODS:
		OpenBackPods();
		break;
	case AVP_Message_Dropship_RequestAnimation::REQUEST_ANIM_OPEN_BACK_MISSILE_PODS_DOORS:
		OpenBackPodsDoors();
		break;
	case AVP_Message_Dropship_RequestAnimation::REQUEST_ANIM_CLOSE_FRONT_MISSILE_PODS:
		CloseFrontPods();
		break;
	case AVP_Message_Dropship_RequestAnimation::REQUEST_ANIM_CLOSE_BACK_MISSILE_PODS:
		CloseBackPods();
		break;
	case AVP_Message_Dropship_RequestAnimation::REQUEST_ANIM_CLOSE_BACK_MISSILE_PODS_DOORS:
		CloseBackPodsDoors();
		break;

	default:
		Asura_Assert(false, "unexpected request");
	}

	// .. else specific animation state machine
}

void AVP_Dropship_AnimationHandler::PostCreateInitialise(Asura_Blueprint* pxBluprint)
{
	Asura_Value xValue;
	TimeDependentAnimDescriptor xAnimDesc;
	xAnimDesc.m_fCurrentTime = xAnimDesc.m_fDesiredTime = 0.5f;

	if ( pxBluprint->GetParameterValueByHashID( uAVP_DROPSHIP_ANIM_FLAPS_DOWNUP, xValue ) ) 
	{
		xAnimDesc.m_uAnimID = xValue.GetHashIDValue();
		Asura_AnimationBlend* pxBlend = new Asura_AnimationBlend_AnimatedTween( 0.0, 0.0, false );
		Asura_AnimationController_Animation* pxAACA = GetAnimation()->CreateControllerAnimation(AAC_FLAG_ACTIVE, 0, xAnimDesc.m_uAnimID, pxBlend);
		xAnimDesc.m_uControllerID = pxAACA->GetControllerHash();
		m_xTimeDependentAnim[AVP_Message_Dropship_RequestAnimation::REQUEST_ANIM_FLAPS_DOWNUP] = xAnimDesc;
	}

	if ( pxBluprint->GetParameterValueByHashID( uAVP_DROPSHIP_ANIM_FLAPS_RIGHTLEFT, xValue ) ) 
	{
		xAnimDesc.m_uAnimID = xValue.GetHashIDValue();
		Asura_AnimationBlend* pxBlend = new Asura_AnimationBlend_AnimatedTween( 0.0, 0.0, false );
		Asura_AnimationController_Animation* pxAACA = GetAnimation()->CreateControllerAnimation(AAC_FLAG_ACTIVE, 0, xAnimDesc.m_uAnimID, pxBlend);
		xAnimDesc.m_uControllerID = pxAACA->GetControllerHash();
		m_xTimeDependentAnim[AVP_Message_Dropship_RequestAnimation::REQUEST_ANIM_FLAPS_RIGHTLEFT] = xAnimDesc;
	}

	if ( pxBluprint->GetParameterValueByHashID( uAVP_DROPSHIP_ANIM_THRUSTERS_FRONTBACK, xValue ) ) 
	{
		xAnimDesc.m_uAnimID = xValue.GetHashIDValue();
		Asura_AnimationBlend* pxBlend = new Asura_AnimationBlend_AnimatedTween( 0.0, 0.0, false );
		Asura_AnimationController_Animation* pxAACA = GetAnimation()->CreateControllerAnimation(AAC_FLAG_ACTIVE, 0, xAnimDesc.m_uAnimID, pxBlend);
		xAnimDesc.m_uControllerID = pxAACA->GetControllerHash();
		m_xTimeDependentAnim[AVP_Message_Dropship_RequestAnimation::REQUEST_ANIM_THRUSTERS_FRONTBACK] = xAnimDesc;
	}

	if ( pxBluprint->GetParameterValueByHashID( uAVP_DROPSHIP_ANIM_IDLE, xValue ) )
	{
		Asura_AnimationBlend* pxBlend = new Asura_AnimationBlend_AnimatedTween( 0.0, 0.0, false );
		GetAnimation()->CreateControllerAnimation(AAC_FLAG_ACTIVE, 100, xValue.GetHashIDValue(), pxBlend);
	}

	if ( pxBluprint->GetParameterValueByHashID( uAVP_DROPSHIP_ANIM_FRONT_PODS_CLOSE_TO_OPEN, xValue ) ) 
	{
		m_uFrontPodsAnimIDCloseToOpen = xValue.GetHashIDValue();
	}

	if ( pxBluprint->GetParameterValueByHashID( uAVP_DROPSHIP_ANIM_BACK_PODS_CLOSE_TO_OPEN, xValue ) ) 
	{
		m_uBackPodsAnimIDCloseToOpen = xValue.GetHashIDValue();
	}

	if ( pxBluprint->GetParameterValueByHashID( uAVP_DROPSHIP_ANIM_BACK_PODS_DOORS_CLOSE_TO_OPEN, xValue ) ) 
	{
		m_uBackPodsDoorsAnimIDCloseToOpen = xValue.GetHashIDValue();
	}

	if ( pxBluprint->GetParameterValueByHashID( uAVP_DROPSHIP_ANIM_FRONT_PODS_OPEN_TO_CLOSE, xValue ) ) 
	{
		m_uFrontPodsAnimIDOpenToClose = xValue.GetHashIDValue();
	}

	if ( pxBluprint->GetParameterValueByHashID( uAVP_DROPSHIP_ANIM_FRONT_PODS_OPEN_TO_CLOSE, xValue ) ) 
	{
		m_uBackPodsAnimIDOpenToClose = xValue.GetHashIDValue();
	}

	if ( pxBluprint->GetParameterValueByHashID( uAVP_DROPSHIP_ANIM_FRONT_PODS_OPEN_TO_CLOSE, xValue ) ) 
	{
		m_uBackPodsDoorsAnimIDOpenToClose = xValue.GetHashIDValue();
	}

	Asura_AnimationController_Group *pxAnimGroup;
	pxAnimGroup = GetAnimation()->CreateControllerGroup("Dropship Front Pods", AAC_FLAG_ACTIVE, 0, new Asura_AnimationBlend(1.0f) );
	m_uFrontPodsControllerGroupID = pxAnimGroup->GetControllerHash();
	pxAnimGroup = GetAnimation()->CreateControllerGroup("Dropship Back Pods", AAC_FLAG_ACTIVE, 0, new Asura_AnimationBlend(1.0f) );
	m_uBackPodsControllerGroupID = pxAnimGroup->GetControllerHash();
	pxAnimGroup = GetAnimation()->CreateControllerGroup("Dropship Back Pods Doors", AAC_FLAG_ACTIVE, 0, new Asura_AnimationBlend(1.0f) );
	m_uBackPodsDoorsControllerGroupID = pxAnimGroup->GetControllerHash();
}


bool AVP_Dropship_AnimationHandler::AreFrontPodsOpened() const
{
	Asura_Assert(m_eFrontPods == OCS_OPENED || m_eFrontPods == OCS_OPENING, "Front Pods were not asked to be opened" );
	return m_eFrontPods == OCS_OPENED;
}

bool AVP_Dropship_AnimationHandler::AreFrontPodsClosed() const
{
	Asura_Assert(m_eFrontPods == OCS_CLOSED|| m_eFrontPods == OCS_CLOSING, "Front Pods were not asked to be closed" );
	return m_eFrontPods == OCS_CLOSED;
}

bool AVP_Dropship_AnimationHandler::AreBackPodsOpened() const
{
	Asura_Assert(m_eBackPods == OCS_OPENED || m_eBackPods == OCS_OPENING, "Back Pods were not asked to be opened" );
	return m_eBackPods == OCS_OPENED;
}

bool AVP_Dropship_AnimationHandler::AreBackPodsClosed() const
{
	Asura_Assert(m_eBackPods == OCS_CLOSED || m_eBackPods == OCS_CLOSING, "Front Pods were not asked to be closed" );
	return m_eBackPods == OCS_CLOSED;
}

bool AVP_Dropship_AnimationHandler::AreBackPodsDoorsOpened() const
{
	Asura_Assert(m_eBackPodsDoors == OCS_OPENED || m_eBackPodsDoors == OCS_OPENING, "Back Pod's Doors were not asked to be opened" );
	return m_eBackPodsDoors == OCS_OPENED;
}

bool AVP_Dropship_AnimationHandler::AreBackPodsDoorsClosed() const
{
	Asura_Assert(m_eBackPodsDoors == OCS_CLOSED || m_eBackPodsDoors == OCS_CLOSING, "Back Pod's Doors were not asked to be closed" );
	return m_eBackPodsDoors == OCS_CLOSED;
}


DEBUGVAR float dbg_fTimeMultiplier = 0.5;

void AVP_Dropship_AnimationHandler::Update()
{
    PARENT::Update();

	float fTimeStep = Asura_Timers::GetGameTimeSlice();

	// update time dependend animation
	float fMaxTurn = dbg_fTimeMultiplier * fTimeStep;
	for (int i = 0; i < AVP_Message_Dropship_RequestAnimation::REQUEST_ANIM_LAST_ANIM_WITH_TIME; ++i)
	{
		Asura_AnimationController_Animation* pxAACA = GetAnimation()->GetControllerAnimationFromHash(m_xTimeDependentAnim[i].m_uControllerID);
		Asura_Assert(pxAACA, "animation controller not found");
		if (pxAACA)
		{
			if (Asura_Maths::Abs(m_xTimeDependentAnim[i].m_fDesiredTime - m_xTimeDependentAnim[i].m_fCurrentTime) < fMaxTurn)
			{
				m_xTimeDependentAnim[i].m_fCurrentTime = m_xTimeDependentAnim[i].m_fDesiredTime;
			}
			else
			{
				m_xTimeDependentAnim[i].m_fCurrentTime += fMaxTurn * Asura_Maths::Sign(m_xTimeDependentAnim[i].m_fDesiredTime - m_xTimeDependentAnim[i].m_fCurrentTime);
			}

			pxAACA->SetParametricTimer(m_xTimeDependentAnim[i].m_fCurrentTime);
		}
	}


	// update state animations
	if (m_eFrontPods == OCS_OPENING)
	{
		Asura_AnimationController_Animation* pxAACA = GetAnimation()->GetControllerAnimationFromHash(m_uFrontPodsControllerID);
		Asura_Assert(pxAACA, "Front Doors animation not found while opening");
		if (pxAACA && pxAACA->IsFinishing())
		{
			m_eFrontPods = OCS_OPENED;
		}
	}
	else if (m_eFrontPods == OCS_CLOSING)
	{
		Asura_AnimationController_Animation* pxAACA = GetAnimation()->GetControllerAnimationFromHash(m_uFrontPodsControllerID);
		if (pxAACA && pxAACA->IsFinishing())
		{
			m_eFrontPods = OCS_CLOSED;
		}
	}
	
	if (m_eBackPods == OCS_OPENING)
	{
		Asura_AnimationController_Animation* pxAACA = GetAnimation()->GetControllerAnimationFromHash(m_uBackPodsControllerID);
		Asura_Assert(pxAACA, "Front Doors animation not found while opening");
		if (pxAACA && pxAACA->IsFinishing())
		{
			m_eBackPods = OCS_OPENED;
		}
	}
	else if (m_eBackPods == OCS_CLOSING)
	{
		Asura_AnimationController_Animation* pxAACA = GetAnimation()->GetControllerAnimationFromHash(m_uBackPodsControllerID);
		if (pxAACA && pxAACA->IsFinishing())
		{
			m_eBackPods = OCS_CLOSED;
		}
	}

	if (m_eBackPodsDoors == OCS_OPENING)
	{
		Asura_AnimationController_Animation* pxAACA = GetAnimation()->GetControllerAnimationFromHash(m_uBackPodsDoorsControllerID);
		Asura_Assert(pxAACA, "Front Doors animation not found while opening");
		if (pxAACA && pxAACA->IsFinishing())
		{
			m_eBackPodsDoors = OCS_OPENED;
		}
	}
	else if (m_eBackPodsDoors == OCS_CLOSING)
	{
		Asura_AnimationController_Animation* pxAACA = GetAnimation()->GetControllerAnimationFromHash(m_uBackPodsDoorsControllerID);
		if (pxAACA && pxAACA->IsFinishing())
		{
			m_eBackPodsDoors = OCS_CLOSED;
		}
	}

}


void AVP_Dropship_AnimationHandler::HandleRequestWithTime(const AVP_Message_Dropship_RequestAnimation& xDropshiprequestData)
{
	m_xTimeDependentAnim[xDropshiprequestData.m_eRequestType].m_fDesiredTime = xDropshiprequestData.m_fDesiredTime;
	if (m_xTimeDependentAnim[xDropshiprequestData.m_eRequestType].m_fDesiredTime < 0.0f)
	{
		m_xTimeDependentAnim[xDropshiprequestData.m_eRequestType].m_fDesiredTime = 0.0f;
	}
	else if (m_xTimeDependentAnim[xDropshiprequestData.m_eRequestType].m_fDesiredTime > 1.0f)
	{
		m_xTimeDependentAnim[xDropshiprequestData.m_eRequestType].m_fDesiredTime = 1.0f;
	}
}

void AVP_Dropship_AnimationHandler::OpenFrontPods()
{
	Asura_Assert(m_eFrontPods == OCS_CLOSED, "Request to open front pods while front doors are not closed");
	m_eFrontPods = OCS_OPENING;

	Asura_AnimationBlend* pxBlend = new Asura_AnimationBlend_AnimatedTween( 0.0, 0.0, false );
	Asura_AnimationController_Group* pxAACG = GetAnimation()->GetControllerGroupFromHash(m_uFrontPodsControllerGroupID);
	Asura_Assert(pxAACG, "Front Pods Controller Group not found");
	Asura_AnimationController_Animation* pxAACA = 
		GetAnimation()->CreateControllerAnimation(AAC_FLAG_ACTIVE | AAC_FLAG_ALLOW_DESTROY_FROM_MESSAGE, 0, m_uFrontPodsAnimIDCloseToOpen, pxBlend, pxAACG);
	Asura_Assert(pxAACA, "Front Pods Controller create failed");
	if (pxAACA)
	{
		m_uFrontPodsControllerID = pxAACA->GetControllerHash();
		pxAACA->Play();
	}
}

void AVP_Dropship_AnimationHandler::CloseFrontPods()
{
	Asura_Assert(m_eFrontPods == OCS_OPENED, "Request to close front pods while front doors are not closed");
	m_eFrontPods = OCS_CLOSING;

	Asura_AnimationBlend* pxBlend = new Asura_AnimationBlend_AnimatedTween( 0.0, 0.0, false );
	Asura_AnimationController_Group* pxAACG = GetAnimation()->GetControllerGroupFromHash(m_uFrontPodsControllerGroupID);
	Asura_Assert(pxAACG, "Front Pods Controller Group not found");
	Asura_AnimationController_Animation* pxAACA = 
		GetAnimation()->CreateControllerAnimation(AAC_FLAG_ACTIVE | AAC_FLAG_ALLOW_DESTROY_FROM_MESSAGE, 0, m_uFrontPodsAnimIDCloseToOpen, pxBlend, pxAACG);
	Asura_Assert(pxAACA, "Front Pods Controller create failed");
	if (pxAACA)
	{
		m_uFrontPodsControllerID = pxAACA->GetControllerHash();
		pxAACA->Play();
	}
}

void AVP_Dropship_AnimationHandler::OpenBackPods()
{
	Asura_Assert(m_eBackPods == OCS_CLOSED, "Request to open back pods while front doors are not closed");
	m_eBackPods = OCS_OPENING;

	Asura_AnimationBlend* pxBlend = new Asura_AnimationBlend_AnimatedTween( 0.0, 0.0, false );
	Asura_AnimationController_Group* pxAACG = GetAnimation()->GetControllerGroupFromHash(m_uBackPodsControllerGroupID);
	Asura_Assert(pxAACG, "Back Pods Controller Group not found");
	Asura_AnimationController_Animation* pxAACA = 
		GetAnimation()->CreateControllerAnimation(AAC_FLAG_ACTIVE | AAC_FLAG_ALLOW_DESTROY_FROM_MESSAGE, 0, m_uBackPodsAnimIDCloseToOpen, pxBlend, pxAACG);
	Asura_Assert(pxAACA, "Back Pods Controller create failed");
	if (pxAACA)
	{
		m_uBackPodsControllerID = pxAACA->GetControllerHash();
		pxAACA->Play();
	}
}

void AVP_Dropship_AnimationHandler::CloseBackPods()
{
	Asura_Assert(m_eBackPods == OCS_OPENED, "Request to close back pods while front doors are not closed");
	m_eBackPods = OCS_CLOSING;

	Asura_AnimationBlend* pxBlend = new Asura_AnimationBlend_AnimatedTween( 0.0, 0.0, false );
	Asura_AnimationController_Group* pxAACG = GetAnimation()->GetControllerGroupFromHash(m_uBackPodsControllerGroupID);
	Asura_Assert(pxAACG, "Back Pods Controller Group not found");
	Asura_AnimationController_Animation* pxAACA = 
		GetAnimation()->CreateControllerAnimation(AAC_FLAG_ACTIVE | AAC_FLAG_ALLOW_DESTROY_FROM_MESSAGE, 0, m_uBackPodsAnimIDCloseToOpen, pxBlend, pxAACG);
	Asura_Assert(pxAACA, "Back Pods Controller create failed");
	if (pxAACA)
	{
		m_uBackPodsControllerID = pxAACA->GetControllerHash();
		pxAACA->Play();
	}
}

void AVP_Dropship_AnimationHandler::OpenBackPodsDoors()
{
	Asura_Assert(m_eBackPodsDoors == OCS_CLOSED, "Request to open back pods doors while front doors are not closed");
	m_eBackPodsDoors = OCS_OPENING;

	Asura_AnimationBlend* pxBlend = new Asura_AnimationBlend_AnimatedTween( 0.0, 0.0, false );
	Asura_AnimationController_Group* pxAACG = GetAnimation()->GetControllerGroupFromHash(m_uBackPodsDoorsControllerGroupID);
	Asura_Assert(pxAACG, "Back Pods Doors Controller Group not found");
	Asura_AnimationController_Animation* pxAACA = 
		GetAnimation()->CreateControllerAnimation(AAC_FLAG_ACTIVE | AAC_FLAG_ALLOW_DESTROY_FROM_MESSAGE, 0, m_uBackPodsDoorsAnimIDCloseToOpen, pxBlend, pxAACG);
	Asura_Assert(pxAACA, "Back Pods Doors Controller create failed");
	if (pxAACA)
	{
		m_uBackPodsDoorsControllerID = pxAACA->GetControllerHash();
		pxAACA->Play();
	}
}

void AVP_Dropship_AnimationHandler::CloseBackPodsDoors()
{
	Asura_Assert(m_eBackPodsDoors == OCS_OPENED, "Request to close back pods doors while front doors are not closed");
	m_eBackPodsDoors = OCS_CLOSING;

	Asura_AnimationBlend* pxBlend = new Asura_AnimationBlend_AnimatedTween( 0.0, 0.0, false );
	Asura_AnimationController_Group* pxAACG = GetAnimation()->GetControllerGroupFromHash(m_uBackPodsDoorsControllerGroupID);
	Asura_Assert(pxAACG, "Back Pods Doors Controller Group not found");
	Asura_AnimationController_Animation* pxAACA = 
		GetAnimation()->CreateControllerAnimation(AAC_FLAG_ACTIVE | AAC_FLAG_ALLOW_DESTROY_FROM_MESSAGE, 0, m_uBackPodsDoorsAnimIDCloseToOpen, pxBlend, pxAACG);
	Asura_Assert(pxAACA, "Back Pods Doors Controller create failed");
	if (pxAACA)
	{
		m_uBackPodsDoorsControllerID = pxAACA->GetControllerHash();
		pxAACA->Play();
	}
}

