
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_Vertebrate_AIAnimStateMachine.cpp
// Author : Will Plano
// Date   : 27/09/2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "Actor/Vertebrates/AVP_Vertebrate_AIAnimStateMachine.h"

#include "Actor/AVP_Actor_AnimationHandler.h"
#include "Animation/AVP_Animation_BodyRig.h"
#include "Entity/PhysicalObject/Actor/AVP_ClientEntity_Actor.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/AVP_ServerActor_Vertebrate.h"

static const char* g_aszStateString[ AVP_Vertebrate_AIAnimStateMachine::VERTEBRATE_STATE_END - AVP_Vertebrate_AIAnimStateMachine::VERTEBRATE_STATE_BEGIN ] = 
{
	"Move.Slow",			// VERTEBRATE_STATE_MOVE_SLOW
	"Move.Medium",			// VERTEBRATE_STATE_MOVE_MEDIUM
	"Move.Fast",			// VERTEBRATE_STATE_MOVE_FAST

	"Idle->Move.Slow",		// VERTEBRATE_STATE_IDLE_TO_MOVE_SLOW
	"Move.Slow->Idle",		// VERTEBRATE_STATE_MOVE_SLOW_TO_IDLE
	"Idle->Move.Medium",	// VERTEBRATE_STATE_IDLE_TO_MOVE_MEDIUM
	"Move.Medium->Idle",	// VERTEBRATE_STATE_MOVE_MEDIUM_TO_IDLE
	"Idle->Move.Fast",		// VERTEBRATE_STATE_IDLE_TO_MOVE_FAST
	"Move.Fast->Idle"		// VERTEBRATE_STATE_MOVE_FAST_TO_IDLE
};

static const bool g_abDirectionalState[ AVP_Vertebrate_AIAnimStateMachine::VERTEBRATE_STATE_END - AVP_Vertebrate_AIAnimStateMachine::VERTEBRATE_STATE_BEGIN ] = 
{
	true,	// VERTEBRATE_STATE_MOVE_SLOW
	true,	// VERTEBRATE_STATE_MOVE_MEDIUM
	true,	// VERTEBRATE_STATE_MOVE_FAST

	true,	// VERTEBRATE_STATE_IDLE_TO_MOVE_SLOW
	true, 	// VERTEBRATE_STATE_MOVE_SLOW_TO_IDLE
	true,	// VERTEBRATE_STATE_IDLE_TO_MOVE_MEDIUM
	true, 	// VERTEBRATE_STATE_MOVE_MEDIUM_TO_IDLE
	true,	// VERTEBRATE_STATE_IDLE_TO_MOVE_FAST
	true 	// VERTEBRATE_STATE_MOVE_FAST_TO_IDLE
};

static const bool g_abLoopedState[ AVP_Vertebrate_AIAnimStateMachine::VERTEBRATE_STATE_END - AVP_Vertebrate_AIAnimStateMachine::VERTEBRATE_STATE_BEGIN ] = 
{
	true,	// VERTEBRATE_STATE_MOVE_SLOW
	true,	// VERTEBRATE_STATE_MOVE_MEDIUM
	true,	// VERTEBRATE_STATE_MOVE_FAST

	false,	// VERTEBRATE_STATE_IDLE_TO_MOVE_SLOW
	false,	// VERTEBRATE_STATE_MOVE_SLOW_TO_IDLE
	false,	// VERTEBRATE_STATE_IDLE_TO_MOVE_MEDIUM
	false,	// VERTEBRATE_STATE_MOVE_MEDIUM_TO_IDLE
	false,	// VERTEBRATE_STATE_IDLE_TO_MOVE_FAST
	false	// VERTEBRATE_STATE_MOVE_FAST_TO_IDLE
};

AVP_Vertebrate_AIAnimStateMachine::AVP_Vertebrate_AIAnimStateMachine( AVP_ServerEntity_Actor& xServerActor )
: PARENT( xServerActor )
{

}

void AVP_Vertebrate_AIAnimStateMachine::BeginState( const u_int uState )
{
	PARENT::BeginState( uState );

	switch( uState )
	{
		case VERTEBRATE_STATE_MOVE_SLOW:
		{
			m_xServerActor.SetMoving( true );
			m_xServerActor.SetMovingSlow( true );
		}
		break;

		case VERTEBRATE_STATE_MOVE_MEDIUM:
		case VERTEBRATE_STATE_MOVE_MEDIUM_TO_IDLE:
		case VERTEBRATE_STATE_IDLE_TO_MOVE_MEDIUM:
		{
			m_xServerActor.SetMoving( true );
		}
		break;

		case VERTEBRATE_STATE_MOVE_FAST:
		{
			m_xServerActor.SetMoving( true );
			m_xServerActor.SetMovingFast( true );
		}
		break;

		default:
		{
			//	do nothing
		}
		break;
	}
}

bool AVP_Vertebrate_AIAnimStateMachine::EndState( const u_int uState )
{
	switch( uState )
	{
		case VERTEBRATE_STATE_MOVE_SLOW:
		case VERTEBRATE_STATE_MOVE_SLOW_TO_IDLE:
		case VERTEBRATE_STATE_IDLE_TO_MOVE_SLOW:
		{
			m_xServerActor.SetMoving( false );
			m_xServerActor.SetMovingSlow( false );
		}
		break;

		case VERTEBRATE_STATE_MOVE_MEDIUM:
		case VERTEBRATE_STATE_MOVE_MEDIUM_TO_IDLE:
		case VERTEBRATE_STATE_IDLE_TO_MOVE_MEDIUM:
		{
			m_xServerActor.SetMoving( false );
		}
		break;

		case VERTEBRATE_STATE_MOVE_FAST:
		case VERTEBRATE_STATE_MOVE_FAST_TO_IDLE:
		case VERTEBRATE_STATE_IDLE_TO_MOVE_FAST:
		{
			m_xServerActor.SetMoving( false );
			m_xServerActor.SetMovingFast( false );
		}
		break;

		default:
		{
			//	do nothing
		}
		break;
	}
	
	return PARENT::EndState( uState );
}

const char* AVP_Vertebrate_AIAnimStateMachine::GetStateDescriptionStatic( const u_int uState )
{
	if ( uState < VERTEBRATE_STATE_BEGIN )
	{
		return PARENT::GetStateDescriptionStatic( uState );
	}

	Asura_Assert( uState < VERTEBRATE_STATE_END, "Invalid state" );
	return g_aszStateString[ uState - VERTEBRATE_STATE_BEGIN ];
}

bool AVP_Vertebrate_AIAnimStateMachine::IsDirectionalState( const u_int uState )
{
	if ( uState < VERTEBRATE_STATE_BEGIN )
	{
		return false; // actor states have no concept of movement direction blends
	}

	Asura_Assert( uState < VERTEBRATE_STATE_END, "Invalid state" );
	return g_abDirectionalState[ uState - VERTEBRATE_STATE_BEGIN ];
}

bool AVP_Vertebrate_AIAnimStateMachine::IsLoopedStateStatic( const u_int uState )
{
	if ( uState < VERTEBRATE_STATE_BEGIN )
	{
		return PARENT::IsLoopedStateStatic( uState );
	}

	Asura_Assert( uState < VERTEBRATE_STATE_END, "Invalid state" );
	return g_abLoopedState[ uState - VERTEBRATE_STATE_BEGIN ];
}

// end of file
