
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_Vertebrate_AIAnimStateMachine.h
// Author : Will Plano
// Date   : 18/09/2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __AVP_VERTEBRATE_AIANIMSTATEMACHINE_H__
#define __AVP_VERTEBRATE_AIANIMSTATEMACHINE_H__

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura_Maths.h"

#include "Actor/AVP_Actor_AIAnimStateMachine.h"

////////////////////////////////////////////////////////////////////////////////
// Forward class declarations...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Class declaration...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_Vertebrate_AIAnimStateMachine
// Author		: Will Plano
// Date			: 18/09/2007
//
// Description  : Class to handle state changing for vertebrates.
////////////////////////////////////////////////////////////////////////////////

class AVP_Vertebrate_AIAnimStateMachine : public AVP_Actor_AIAnimStateMachine
{
public:
	enum VertebrateAnimState
	{
		VERTEBRATE_STATE_BEGIN = ACTOR_STATE_END,

		VERTEBRATE_STATE_MOVE_SLOW = VERTEBRATE_STATE_BEGIN,	// 4
		VERTEBRATE_STATE_MOVE_MEDIUM,							// 5
		VERTEBRATE_STATE_MOVE_FAST,								// 6

		// Transition states
		VERTEBRATE_STATE_IDLE_TO_MOVE_SLOW,						// 7
		VERTEBRATE_STATE_MOVE_SLOW_TO_IDLE,						// 8
		VERTEBRATE_STATE_IDLE_TO_MOVE_MEDIUM,					// 9
		VERTEBRATE_STATE_MOVE_MEDIUM_TO_IDLE,					// 10
		VERTEBRATE_STATE_IDLE_TO_MOVE_FAST,						// 11
		VERTEBRATE_STATE_MOVE_FAST_TO_IDLE,						// 12

		VERTEBRATE_STATE_END									// 13
	};

	AVP_Vertebrate_AIAnimStateMachine( AVP_ServerEntity_Actor& xServerActor );
	virtual ~AVP_Vertebrate_AIAnimStateMachine() {}

	void	RequestMoveSlow() { RequestState( VERTEBRATE_STATE_MOVE_SLOW ); }
	bool	IsMoveSlow()	const { return IsStateMoveSlow( GetCurrentState() ); }
	bool	IsStateMoveSlow( const u_int uState ) const { return ( VERTEBRATE_STATE_MOVE_SLOW == uState || VERTEBRATE_STATE_IDLE_TO_MOVE_SLOW == uState ) ; }
	bool	IsMovingSlowState( ) const { return ( GetCurrentState() == VERTEBRATE_STATE_MOVE_SLOW || GetCurrentState() == VERTEBRATE_STATE_IDLE_TO_MOVE_SLOW || GetCurrentState() == VERTEBRATE_STATE_MOVE_SLOW_TO_IDLE ) ; }

	void	RequestMoveMedium() { RequestState( VERTEBRATE_STATE_MOVE_MEDIUM ); }
	bool	IsMoveMedium()	const { return IsStateMoveMedium( GetCurrentState() ); }
	bool	IsStateMoveMedium( const u_int uState ) const { return ( VERTEBRATE_STATE_MOVE_MEDIUM == uState || VERTEBRATE_STATE_IDLE_TO_MOVE_MEDIUM == uState ) ; }
	bool	IsMovingMediumState( ) const { return ( GetCurrentState() == VERTEBRATE_STATE_MOVE_MEDIUM || GetCurrentState() == VERTEBRATE_STATE_IDLE_TO_MOVE_MEDIUM || GetCurrentState() == VERTEBRATE_STATE_MOVE_MEDIUM_TO_IDLE ) ; }

	void	RequestMoveFast() { RequestState( VERTEBRATE_STATE_MOVE_FAST ); }
	bool	IsMoveFast()	const { return IsStateMoveFast( GetCurrentState() ); }
	bool	IsStateMoveFast( const u_int uState ) const { return ( VERTEBRATE_STATE_MOVE_FAST == uState || VERTEBRATE_STATE_IDLE_TO_MOVE_FAST == uState ) ; }
	bool	IsMovingFastState( ) const { return ( GetCurrentState() == VERTEBRATE_STATE_MOVE_FAST || GetCurrentState() == VERTEBRATE_STATE_IDLE_TO_MOVE_FAST || GetCurrentState() == VERTEBRATE_STATE_MOVE_FAST_TO_IDLE ) ; }

	virtual const char* GetStateDescription( const u_int uState ) const { return GetStateDescriptionStatic( uState ); }
	static const char* GetStateDescriptionStatic( const u_int uState );

	static bool IsDirectionalState( const u_int uState );

	virtual bool IsLoopedState( const u_int uState ) const { return IsLoopedStateStatic( uState ); }
	static bool IsLoopedStateStatic( const u_int uState );

	static bool IsStateRequired( const u_int uState ) { return	uState != VERTEBRATE_STATE_IDLE_TO_MOVE_SLOW	&& 
																uState != VERTEBRATE_STATE_MOVE_SLOW_TO_IDLE	&&
																uState != VERTEBRATE_STATE_IDLE_TO_MOVE_MEDIUM	&&
																uState != VERTEBRATE_STATE_MOVE_MEDIUM_TO_IDLE	&&
																uState != VERTEBRATE_STATE_IDLE_TO_MOVE_FAST	&&	
																uState != VERTEBRATE_STATE_MOVE_FAST_TO_IDLE; }

protected:
	virtual void BeginState		( const u_int uState );
	//virtual void UpdateState	( const u_int uState ) { PARENT::UpdateState( uState ); }
	virtual bool EndState		( const u_int uState );

private:
	typedef AVP_Actor_AIAnimStateMachine PARENT;

	AVP_Vertebrate_AIAnimStateMachine(const AVP_Vertebrate_AIAnimStateMachine& xOther);
	AVP_Vertebrate_AIAnimStateMachine&	operator=(const AVP_Vertebrate_AIAnimStateMachine& xOther);

};   // End of class 'AVP_Vertebrate_AIAnimStateMachine'.

#endif   // ifndef __AVP_VERTEBRATE_AIANIMSTATEMACHINE_H__

/// End of file. ///
