
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_Alien_AIAnimStateMachine.cpp
// Author : Will Plano
// Date   : 27/02/2008
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "Actor/Vertebrates/Alien/AVP_Alien_AIAnimStateMachine.h"

#include "Rendering/Asura_Font_System.h"
#include "Network/Asura_Message.h"

#include "Actor/Vertebrates/Alien/AVP_Alien_AIAnimStateConnections.h"
#include "Actor/Vertebrates/Alien/AVP_Alien_AIAnimStateData.h"
#include "Actor/Vertebrates/Alien/AVP_Alien_JumpHandler.h"
#include "Blueprint/AVP_Blueprint_HashIDs.h"
#include "Entity/PhysicalObject/Actor/AVP_ServerEntity_Actor.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Alien/AVP_Messages_Alien.h"
#include "Inventory/AVP_Inventory_HashIDs.h"
#include "Inventory/AVP_Inventory_System.h"
#include "Moves/Vertebrates/Alien/AVP_Alien_MovesJump.h"

static const char* g_aszStateString[ AVP_Alien_AIAnimStateMachine::ALIEN_STATE_END - AVP_Alien_AIAnimStateMachine::ALIEN_STATE_BEGIN ] = 
{
	"Stand.Idle",		// ALIEN_STATE_STAND_IDLE
	"Stand.Move",		// ALIEN_STATE_STAND_MOVE
	"Idle->Stand.Idle",	// ALIEN_STATE_IDLE_TO_STAND_IDLE
	"Stand.Idle->Idle",	// ALIEN_STATE_STAND_IDLE_TO_IDLE
	"Burst",			// ALIEN_STATE_BURST
	"Hesitate",			// ALIEN_STATE_HESITATE
	"Hide.Idle"			// ALIEN_STATE_HIDE_IDLE
};

static bool g_abDirectionalState[ AVP_Alien_AIAnimStateMachine::ALIEN_STATE_END - AVP_Alien_AIAnimStateMachine::ALIEN_STATE_BEGIN ] = 
{
	false,	// ALIEN_STATE_STAND_IDLE
	true,	// ALIEN_STATE_STAND_MOVE
	false,	// ALIEN_STATE_IDLE_TO_STAND_IDLE,
	false,	// ALIEN_STATE_STAND_IDLE_TO_IDLE,
	false,	// ALIEN_STATE_BURST
	false,	// ALIEN_STATE_HESITATE
	false	// ALIEN_STATE_HIDE_IDLE

};

static const char* g_aszGroundString[ AVP_Alien_AIAnimStateMachine::GROUNDTYPE_END ] = 
{
	"OnFloor",
	"NonFloor",
};

static const char* g_aszLimbString[ AVP_Alien_AIAnimStateMachine::LIMBDMG_END ] = 
{
	"NoDamage",
	"LeftLegDamaged",
	"RightLegDamaged",
	"LeftArmDamaged",
	"RightArmDamaged",
	"BothLegsDamaged"
};

static bool g_abLoopedState[ AVP_Alien_AIAnimStateMachine::ALIEN_STATE_END - AVP_Alien_AIAnimStateMachine::ALIEN_STATE_BEGIN ] = 
{
	true,	// ALIEN_STATE_STAND_IDLE
	true,	// ALIEN_STATE_STAND_MOVE
	false,	// ALIEN_STATE_IDLE_TO_STAND_IDLE,
	false,	// ALIEN_STATE_STAND_IDLE_TO_IDLE,
	false,	// ALIEN_STATE_BURST
	false,	// ALIEN_STATE_HESITATE
	true	// ALIEN_STATE_HIDE_IDLE
};

////////////////////////////////////////////////////////////////////////////////
// Public functions...
////////////////////////////////////////////////////////////////////////////////

void AVP_Alien_AIAnimStateMachine::Initialise()
{
	AVP_Alien_AIAnimStateData::Initialise( m_xServerActor.GetAnimStateBlueprintType() );

	PARENT::Initialise();
}

const AVP_Actor_AnimStateData* AVP_Alien_AIAnimStateMachine::GetStateAnimData( const u_int uState ) const
{
	const AVP_ServerActor_Alien& xAlien = static_cast< const AVP_ServerActor_Alien& >( m_xServerActor );
	AVP_Alien_AIAnimStateMachine::LimbDamage eLimbDamage = xAlien.GetLimbDamage() ;
	AVP_Actor_PhysicsObjectHandler::GroundType eGroundType = xAlien.GetGroundType();
	return AVP_Alien_AIAnimStateData::GetAnimStateData( m_xServerActor.GetBlueprintTypeHashID(), uState, eGroundType, eLimbDamage );
}

void AVP_Alien_AIAnimStateMachine::BeginState( const u_int uState )
{
	AVP_ServerActor_Alien& xAlien = static_cast< AVP_ServerActor_Alien& >( m_xServerActor );

	if ( uState == ALIEN_STATE_STAND_MOVE )
	{
		xAlien.SetMoving( true );
		xAlien.SetStanding( true );
	}
	else if ( uState == ALIEN_STATE_STAND_IDLE || uState == ALIEN_STATE_HIDE_IDLE )
	{
		xAlien.SetStanding( true );
	}
	else if ( uState == ALIEN_STATE_BURST || uState == ALIEN_STATE_HESITATE )
	{
		xAlien.SetMoving( true );
	}

	PARENT::BeginState( uState );
}

bool AVP_Alien_AIAnimStateMachine::EndState( const u_int uState )
{
	AVP_ServerActor_Alien& xAlien = static_cast< AVP_ServerActor_Alien& >( m_xServerActor );

	if ( uState == ALIEN_STATE_STAND_MOVE )
	{
		xAlien.SetMoving( false );
		xAlien.SetStanding( false );
	}
	else if ( uState == ALIEN_STATE_STAND_IDLE || uState == ALIEN_STATE_HIDE_IDLE )
	{
		xAlien.SetStanding( false );
	}
	else if ( uState == ALIEN_STATE_BURST || uState == ALIEN_STATE_HESITATE )
	{
		xAlien.SetMoving( false );
	}

	return PARENT::EndState( uState );
}

#ifndef ASURA_NO_DEBUG_OUTPUT

void AVP_Alien_AIAnimStateMachine::RenderDebugInfo()
{
	const AVP_ServerActor_Alien& xAlien = static_cast< const AVP_ServerActor_Alien& >( m_xServerActor );
	PARENT::RenderDebugInfo();

	Asura_Vector_3 xDebugPos( m_xServerActor.GetPosition() );
	if ( Asura_Camera::ProjectWorldCoordsToScreenSpace( xDebugPos, true ) && xDebugPos.z > 0.0f )
	{
		xDebugPos.y -= 10.0f;

		if ( xAlien.IsStanding() )
		{
			xDebugPos.y -= 10.0f;

			Asura_Font_System::Print
			(
				xDebugPos.GetX(),
				xDebugPos.GetY(),
				0.6f,
				0xffffffff,
				ASURA_TEXTFORMAT_CENTREJUSTIFIED, 
				"Standing"
			);
		}
	
		xDebugPos.y -= 10.0f;

		Asura_Font_System::Print
		(
			xDebugPos.GetX(),
			xDebugPos.GetY(),
			0.6f,
			0xffffffff,
			ASURA_TEXTFORMAT_CENTREJUSTIFIED, 
			g_aszGroundString[ m_xServerActor.GetGroundType() == AVP_Actor_PhysicsObjectHandler::ON_FLOOR ? GROUNDTYPE_ON_FLOOR : GROUNDTYPE_NON_FLOOR ]
		);

		xDebugPos.y -= 10.0f;

		Asura_Font_System::Print
		(
			xDebugPos.GetX(),
			xDebugPos.GetY(),
			0.6f,
			0xffffffff,
			ASURA_TEXTFORMAT_CENTREJUSTIFIED, 
			g_aszLimbString[ xAlien.GetLimbDamage() ]
		);
	}
}
#endif

const AVP_Actor_AIAnimStateConnectionData* AVP_Alien_AIAnimStateMachine::GetConnectionData( const u_int uFromState, const u_int uToState ) const
{
	return AVP_Alien_AIAnimStateConnections::GetConnectionData( uFromState, uToState );
}

const char* AVP_Alien_AIAnimStateMachine::GetStateDescriptionStatic( const u_int uState )
{
	if ( uState < ALIEN_STATE_BEGIN )
	{
		return PARENT::GetStateDescriptionStatic( uState );
	}

	Asura_Assert( uState < ALIEN_STATE_END, "Invalid state" );
	return g_aszStateString[ uState - ALIEN_STATE_BEGIN ];
}

bool AVP_Alien_AIAnimStateMachine::IsDirectionalState( const u_int uState )
{
	if ( uState < ALIEN_STATE_BEGIN )
	{
		return PARENT::IsDirectionalState( uState );
	}

	Asura_Assert( uState < ALIEN_STATE_END, "Invalid state" );
	return g_abDirectionalState[ uState - ALIEN_STATE_BEGIN ];
}

const char* AVP_Alien_AIAnimStateMachine::GetGroundTypeDescription( const u_int uGroundType )
{
	Asura_Assert( uGroundType < GROUNDTYPE_END, "Invalid ground type" );
	return g_aszGroundString[ uGroundType ];
}

const char* AVP_Alien_AIAnimStateMachine::GetLimbDamageDescription( const u_int uLimbDamage )
{
	Asura_Assert( uLimbDamage < LIMBDMG_END, "Invalid ground type" );
	return g_aszLimbString[ uLimbDamage ];
}

bool AVP_Alien_AIAnimStateMachine::IsLoopedStateStatic( const u_int uState )
{
	if ( uState < ALIEN_STATE_BEGIN )
	{
		return PARENT::IsLoopedStateStatic( uState );
	}

	Asura_Assert( uState < ALIEN_STATE_END, "Invalid state" );
	return g_abLoopedState[ uState - ALIEN_STATE_BEGIN ];
}

bool AVP_Alien_AIAnimStateMachine::IsCurrentStateFinished() const
{
	if ( GetCurrentState() == ALIEN_STATE_BURST || GetCurrentState() == ALIEN_STATE_HESITATE )
	{
		// These states continue until either the animation is finished or a state other than move medium is in the queue.
		if ( HasStateInQueue() && !IsStateMoveMedium( GetTargetState() ) )
		{
			return true;
		}
		else
		{
			return PARENT::IsCurrentStateFinished();
		}
	}
	else
	{
		return PARENT::IsCurrentStateFinished();
	}
}

/// End of file. ///
