
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_Alien_CameraHandler.cpp
// Author : Chris Wallace
// Date   : 5-Feb-2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "AVP_Alien_CameraHandler.h"

// Asura independent header-files...
#include "Core/Asura_Timers.h"
#include "Interface/Asura_CommandConsole.h"

// Game independent header-files...
#include "Actor/Vertebrates/AVP_Vertebrate_InputHandler.h"
#include "Blueprint/AVP_Blueprint_HashIDs.h"
#include "Debugging/Asura_Logfile.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Alien/AVP_ClientActor_Alien.h"
#include "Moves/AVP_Movement_System.h"

//#define LOG(params) Asura_OutputToDebugger_Retail(params)
#define LOG(params)

DVSVAR float	dvs_fLookSmoothingAngVel	= Asura_Maths::Deg270;
#if defined(ASURA_SUPPORT_COMMAND_CONSOLE)
u_int AVP_Alien_CameraHandler::s_uConsoleVarReferenceCount = 0;
#endif

AVP_Actor_CameraProperties g_xCameraPropertiesAlien =
{
	0.2f,										// camera radius
	0.02f,										// pull-back speed

	0.55f,										// third person offset distance
	1.5f,										// the distance below which the offset is reduced
	Asura_Maths::DegToRad(85.0f),				// third person FOV
	0.5f,										// duration from third to first
	0.5f,										// duration from first to third
	8.0f,										// character turn speed when changing to first person camera

	3.5f,										// maximum angular velocity of the camera when rotating around the player
	0.1f,										// time taken for the angular velocity to half

	0.27f,										// half player width used for LOS corner tests
	1.001f,										// extra factor to ensure moving camera around corner

	Asura_Vector_3(0.0f, -2.1f, -0.1f),			// offset for the back of the head
	2.0f,										// distance before character starts to fade out

	3.2f,										// distance for camera to follow player inside
	4.5f,										// distance for camera to follow player outside
	3.0f,										// distance for camera to move back from player in open space
	3.0f,										// distance above which the camera looks at the player
	5.0f,										// distance for the grapple LOS tests to determine the camera move direction

	0.5f,										// factor for camera to follow player inside
	0.5f,										// factor for camera to follow player outside
	3.0f,										// factor for smoothing character height
	1.5f,										// factor for smoothing camera height
	7.0f,										// factor for moving the camera around walls
	7.0f,										// factor for moving the camera behind the player when lock is held
	0.2f,										// factor for moving the camera behind the player automatically after inactive period

	1.0f,										// duration to change from outside to inside camera
	1.0f,										// duration to change from inside to outside camera
	0.1f,										// duration after which the camera gives up moving behind walls and is placed in front
/*
		{											// outside camera
			0.0f,												// threshold
				Asura_Vector_3(0.0f, -2.0f, -9.0f),					// displacement (x component is to the right of the player)
				Asura_Euler_3(0.0f, 0.0f, 0.0f),					// rotation (y component is clock-wise from above)
				2.0f,												// displacement duration
				2.0f,												// rotation duration
		},
		{											// inside camera
			0.0f,												// ..
				Asura_Vector_3(0.8f, -0.5f, -2.5f),					// ...
				Asura_Euler_3(0.0f, 0.0f, 0.0f),
				2.0f,
				2.0f,
			},
			{											// grapple camera (left side; right is the mirror image of this)
				0.0f,
					Asura_Vector_3(-1.5f, 0.7f, 0.5f),	
					Asura_Euler_3(0.0f, Asura_Maths::Deg90, 0.0f),
					1.7f,
					1.7f,
			},
			{											// two-step grapple camera - step 1
				0.0f,
					Asura_Vector_3(0.0f, -0.5f, -2.5f),
					Asura_Euler_3(0.0f, 0.0f, 0.0f),
					1.0f,
					0.5f,
				},
				{											// two-step grapple camera - step 2
					0.5f,
						Asura_Vector_3(-1.5f, 0.7f, 0.5f),	
						Asura_Euler_3(0.0f, Asura_Maths::Deg90, 0.0f),
						1.7f,
						2.5f,
				},
				{
					0.0f,									// locked camera outside (left side)
						Asura_Vector_3(-10.0f, -1.5f, -0.5f),
						Asura_Euler_3(0.0f, Asura_Maths::DegToRad(50.0f), 0.0f),	// clock-wise from above
						1.0f,
						1.0f,
					},
					{
						0.0f,									// locked camera inside (left side)
							Asura_Vector_3(-5.0f, -0.7f, -1.7f),
							Asura_Euler_3(0.0f, Asura_Maths::DegToRad(50.0f), 0.0f),	// clock-wise from above
							1.0f,
							1.0f,
					},
*/
};

AVP_Alien_CameraHandler::AVP_Alien_CameraHandler (const Asura_Guid uOwnerGuid)
: PARENT(uOwnerGuid, g_xCameraPropertiesAlien)
{
#if defined(ASURA_DEBUG_VARIABLE_SYSTEM)
	if ( s_uConsoleVarReferenceCount == 0 )
	{
		if ( !Asura_CommandConsole::VarExists( L"AVP.BodyRig.LookSmoothing.MaxVelocity" ) )
		{
			Asura_CommandConsole::AddVar( L"AVP.BodyRig.LookSmoothing.MaxVelocity", &dvs_fLookSmoothingAngVel, 0.0f );
		}
	}
	++s_uConsoleVarReferenceCount;
#endif
}

AVP_Alien_CameraHandler::~AVP_Alien_CameraHandler ()
{
#if defined(ASURA_SUPPORT_COMMAND_CONSOLE)
	--s_uConsoleVarReferenceCount;
#endif
}

void AVP_Alien_CameraHandler::Reset()
{
	AVP_ClientActor_Alien* pxClient = AVP_ClientActor_Alien::FindEntity( m_uOwnerGuid );
	if ( pxClient )
	{
		m_xCurrentCameraOrientation.FromMatrix( pxClient->GetCurrentOrientation() );
	}
	PARENT::Reset();
}

void AVP_Alien_CameraHandler::RotateCurrentAimDirection( const Asura_Quat& xRot )
{
	m_xCurrentCameraOrientation = xRot * m_xCurrentCameraOrientation;
}

DEBUGVAR float dbg_fMinSmoothingAngle = Asura_Maths::DegToRad( 0.0f );
DEBUGVAR float dbg_fMaxSmoothingAngle = Asura_Maths::DegToRad( 30.0f );

void AVP_Alien_CameraHandler::UpdateCameraOrientation( AVP_ClientEntity_Actor& xOwner, Asura_Matrix_3x3& xOrientation )
{
	Asura_Quat xTargetOri;
	xTargetOri.FromMatrix( xOrientation );

	Asura_Quat xInvCurrentOri = m_xCurrentCameraOrientation;
	xInvCurrentOri.Invert();
	
	xTargetOri = xTargetOri * xInvCurrentOri;
	xTargetOri.Normalise();
	Asura_AVector3 xAxis;
	float fAng = xTargetOri.ToRotation( xAxis );
	if ( fAng < 0.0f )
	{
		fAng *= -1.0f; 
		xAxis *= -1.0f;
	}
	if ( fAng < 0.0001f )
	{
		return;
	}

	float fMaxAng = dvs_fLookSmoothingAngVel * Asura_Timers::GetGameTimeSlice();
	if ( fAng > dbg_fMinSmoothingAngle )
	{
		if ( fAng < dbg_fMaxSmoothingAngle )
		{
			fMaxAng *= ( fAng - dbg_fMinSmoothingAngle ) / ( dbg_fMaxSmoothingAngle - dbg_fMinSmoothingAngle );
		}
		const float fMinAng = fAng - Asura_Maths::Deg90;
		fMaxAng = Asura_Maths::Max( fMinAng, fMaxAng );
		const float fDeltaAng = Asura_Maths::ClampToRange( fAng, fMinAng, fMaxAng );

		xTargetOri.FromRotation( fDeltaAng, xAxis );
		m_xCurrentCameraOrientation = xTargetOri * m_xCurrentCameraOrientation;
		Asura_Assert( m_xCurrentCameraOrientation.x == m_xCurrentCameraOrientation.x, "" );
		m_xCurrentCameraOrientation.Normalise();
	}
	m_xCurrentCameraOrientation.ToMatrix( xOrientation );

	LOG( ( "Camera Fwd.Up = %.4f\n", xOrientation.e23 ) );
}

/// End of file. ///

