
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_Alien_Description.cpp
// Author : Andy Tate
// Date   : 10-Jan-2007
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "Actor/Vertebrates/Alien/AVP_Alien_Description.h"

// Asura independent header-files...
#include "Blueprint/Asura_Blueprint_System.h"
#ifdef ASURA_TOOLS
#include "Blueprint/Tools_Mode_Blueprint.h"
#endif
// Asura platform header-files...

// Game independent header-files...
#include "Blueprint/AVP_Blueprint_HashIDs.h"
#include "Rendering/Asura_Project_Skin_Elements.h"

// Game platform header-files...

#if defined(ASURA_TOOLS)
// Tools header-files...
#include "Particles/Tools_Mode_PFX.h"
#endif	// defined(ASURA_TOOLS)

// Marker-point hash-IDs...
const Asura_Hash_ID AVP_Alien_Description::s_auMarkerPointHashIDs[AVP_Alien_Description::NUM_MARKER_POINT_IDS] =
{
	0x31DA7431,		// Asura_GetHashID("innerMouth");			 	// MPID_INNERMOUTH
	0x0035F848,		// Asura_GetHashID("spit");					 	// MPID_SPIT
	0x586BCE4F,		// Asura_GetHashID("throught");				 	// MPID_THROAT

	0xF4067346,		// Asura_GetHashID("bloodairTubeBottomLeft");	// MPID_BLOOD_AIRTUBE_BOTTOMLEFT
	0x8D1E56BD,		// Asura_GetHashID("bloodairTubeBottomRight");	// MPID_BLOOD_AIRTUBE_BOTTOMRIGHT
	0xF79810C8,		// Asura_GetHashID("bloodairTubeTopLeft");	 	// MPID_BLOOD_AIRTUBE_TOPLEFT
	0xFBC0697B,		// Asura_GetHashID("bloodairTubeTopRight");	 	// MPID_BLOOD_AIRTUBE_TOPRIGHT
	0xD013C1D6,		// Asura_GetHashID("bloodBip01LUpperArm");	 	// MPID_BLOOD_UPPERARM_LEFT
	0x3F2210F0,		// Asura_GetHashID("bloodBip01Head");		 	// MPID_BLOOD_HEAD
	0x3F24CB6F,		// Asura_GetHashID("bloodBip01Neck");		 	// MPID_BLOOD_NECK
	0x49AE5BDC,		// Asura_GetHashID("bloodBip01RUpperArm");	 	// MPID_BLOOD_UPPERARM_RIGHT
	0x068BA63A,		// Asura_GetHashID("bloodBip01LThigh");		 	// MPID_BLOOD_THIGH_LEFT
	0x10C8B9F4,		// Asura_GetHashID("bloodBip01RThigh");		 	// MPID_BLOOD_THIGH_RUGHT
	0xA5C774D2,		// Asura_GetHashID("bloodBip01Tail2");		 	// MPID_BLOOD_TAILTIP
	0x3F277760,		// Asura_GetHashID("bloodBip01Tail");		 	// MPID_BLOOD_TAIL
};

const int AVP_Alien_Description::s_aiFirstPersonFootBoneID[ AVP_ClientEntity_Actor::MAX_NUM_FEET ] = 
{
	FP_BIP01_L_FOOT,
	FP_BIP01_R_FOOT,
	FP_BIP01_L_HAND,
	FP_BIP01_R_HAND
};

const int AVP_Alien_Description::s_aiThirdPersonFootBoneID[ AVP_ClientEntity_Actor::MAX_NUM_FEET ] = 
{
	TP_BIP01_L_FOOT,
	TP_BIP01_R_FOOT,
	TP_BIP01_L_HAND,
	TP_BIP01_R_HAND
};

// IK
const int AVP_Alien_Description::s_aiFirstPersonLeftArmChainBonesArray[ uAVP_ALIEN_NUM_IK_BONES_PER_LIMB + 1 ] =
{
	FP_BIP01_L_UPPERARM,
	FP_BIP01_L_FOREARM,
	FP_BIP01_L_HAND,
	-1
};

const int AVP_Alien_Description::s_aiFirstPersonLeftFinger0ChainBonesArray[ uAVP_ALIEN_NUM_IK_BONES_PER_LIMB + 1 ] =
{
	FP_BIP01_L_FINGER0,
	FP_BIP01_L_FINGER01,
	FP_BIP01_L_FINGER02,
	-1
};

const int AVP_Alien_Description::s_aiFirstPersonLeftFinger1ChainBonesArray[ uAVP_ALIEN_NUM_IK_BONES_PER_LIMB + 1 ] =
{
	FP_BIP01_L_FINGER1,
	FP_BIP01_L_FINGER11,
	FP_BIP01_L_FINGER12,
	-1
};

const int AVP_Alien_Description::s_aiFirstPersonLeftFinger2ChainBonesArray[ uAVP_ALIEN_NUM_IK_BONES_PER_LIMB + 1 ] =
{
	FP_BIP01_L_FINGER2,
	FP_BIP01_L_FINGER21,
	FP_BIP01_L_FINGER22,
	-1
};

const int AVP_Alien_Description::s_aiFirstPersonLeftFinger3ChainBonesArray[ uAVP_ALIEN_NUM_IK_BONES_PER_LIMB + 1 ] =
{
	FP_BIP01_L_FINGER3,
	FP_BIP01_L_FINGER31,
	FP_BIP01_L_FINGER32,
	-1
};

const int AVP_Alien_Description::s_aiFirstPersonRightArmChainBonesArray[ uAVP_ALIEN_NUM_IK_BONES_PER_LIMB + 1 ] =
{
	FP_BIP01_R_UPPERARM,
	FP_BIP01_R_FOREARM,
	FP_BIP01_R_HAND,
	-1
};

const int AVP_Alien_Description::s_aiFirstPersonRightFinger0ChainBonesArray[ uAVP_ALIEN_NUM_IK_BONES_PER_LIMB + 1 ] =
{
	FP_BIP01_R_FINGER0,
	FP_BIP01_R_FINGER01,
	FP_BIP01_R_FINGER02,
	-1
};

const int AVP_Alien_Description::s_aiFirstPersonRightFinger1ChainBonesArray[ uAVP_ALIEN_NUM_IK_BONES_PER_LIMB + 1 ] =
{
	FP_BIP01_R_FINGER1,
	FP_BIP01_R_FINGER11,
	FP_BIP01_R_FINGER12,
	-1
};

const int AVP_Alien_Description::s_aiFirstPersonRightFinger2ChainBonesArray[ uAVP_ALIEN_NUM_IK_BONES_PER_LIMB + 1 ] =
{
	FP_BIP01_R_FINGER2,
	FP_BIP01_R_FINGER21,
	FP_BIP01_R_FINGER22,
	-1
};

const int AVP_Alien_Description::s_aiFirstPersonRightFinger3ChainBonesArray[ uAVP_ALIEN_NUM_IK_BONES_PER_LIMB + 1 ] =
{
	FP_BIP01_R_FINGER3,
	FP_BIP01_R_FINGER31,
	FP_BIP01_R_FINGER32,
	-1
};

const int AVP_Alien_Description::s_aiFirstPersonLeftLegChainBonesArray[ uAVP_ALIEN_NUM_IK_BONES_PER_LIMB + 1 ] =
{
	FP_BIP01_L_THIGH,
	FP_BIP01_L_CALF,
	FP_BIP01_L_FOOT,
	-1
};

const int AVP_Alien_Description::s_aiFirstPersonRightLegChainBonesArray[ uAVP_ALIEN_NUM_IK_BONES_PER_LIMB + 1 ] =
{
	FP_BIP01_R_THIGH,
	FP_BIP01_R_CALF,
	FP_BIP01_R_FOOT,
	-1
};

const int AVP_Alien_Description::s_aiFirstPersonTailChainBonesArray[ uAVP_ALIEN_NUM_IK_BONES_PER_TAIL + 1 ] =
{
	FP_BIP01_TAIL,
	FP_BIP01_TAIL1,
	FP_BIP01_TAIL2,
	FP_BIP01_TAIL3,
	FP_BIP01_TAIL4,
	FP_BIP01_TAIL5,
	FP_BIP01_TAIL6,
	FP_BIP01_TAIL7,
	-1
};

const int AVP_Alien_Description::s_aiThirdPersonLeftArmChainBonesArray[ uAVP_ALIEN_NUM_IK_BONES_PER_LIMB + 1 ] =
{
	TP_BIP01_L_UPPERARM,
	TP_BIP01_L_FOREARM,
	TP_BIP01_L_HAND,
	-1
};

const int AVP_Alien_Description::s_aiThirdPersonLeftFinger0ChainBonesArray[ uAVP_ALIEN_NUM_IK_BONES_PER_LIMB + 1 ] =
{
	TP_BIP01_L_FINGER0,
	TP_BIP01_L_FINGER01,
	TP_BIP01_L_FINGER02,
	-1
};

const int AVP_Alien_Description::s_aiThirdPersonLeftFinger1ChainBonesArray[ uAVP_ALIEN_NUM_IK_BONES_PER_LIMB + 1 ] =
{
	TP_BIP01_L_FINGER1,
	TP_BIP01_L_FINGER11,
	TP_BIP01_L_FINGER12,
	-1
};

const int AVP_Alien_Description::s_aiThirdPersonLeftFinger2ChainBonesArray[ uAVP_ALIEN_NUM_IK_BONES_PER_LIMB + 1 ] =
{
	TP_BIP01_L_FINGER2,
	TP_BIP01_L_FINGER21,
	TP_BIP01_L_FINGER22,
	-1
};

const int AVP_Alien_Description::s_aiThirdPersonLeftFinger3ChainBonesArray[ uAVP_ALIEN_NUM_IK_BONES_PER_LIMB + 1 ] =
{
	TP_BIP01_L_FINGER3,
	TP_BIP01_L_FINGER31,
	TP_BIP01_L_FINGER32,
	-1
};

const int AVP_Alien_Description::s_aiThirdPersonRightArmChainBonesArray[ uAVP_ALIEN_NUM_IK_BONES_PER_LIMB + 1 ] =
{
	TP_BIP01_R_UPPERARM,
	TP_BIP01_R_FOREARM,
	TP_BIP01_R_HAND,
	-1
};

const int AVP_Alien_Description::s_aiThirdPersonRightFinger0ChainBonesArray[ uAVP_ALIEN_NUM_IK_BONES_PER_LIMB + 1 ] =
{
	TP_BIP01_R_FINGER0,
	TP_BIP01_R_FINGER01,
	TP_BIP01_R_FINGER02,
	-1
};

const int AVP_Alien_Description::s_aiThirdPersonRightFinger1ChainBonesArray[ uAVP_ALIEN_NUM_IK_BONES_PER_LIMB + 1 ] =
{
	TP_BIP01_R_FINGER1,
	TP_BIP01_R_FINGER11,
	TP_BIP01_R_FINGER12,
	-1
};

const int AVP_Alien_Description::s_aiThirdPersonRightFinger2ChainBonesArray[ uAVP_ALIEN_NUM_IK_BONES_PER_LIMB + 1 ] =
{
	TP_BIP01_R_FINGER2,
	TP_BIP01_R_FINGER21,
	TP_BIP01_R_FINGER22,
	-1
};

const int AVP_Alien_Description::s_aiThirdPersonRightFinger3ChainBonesArray[ uAVP_ALIEN_NUM_IK_BONES_PER_LIMB + 1 ] =
{
	TP_BIP01_R_FINGER3,
	TP_BIP01_R_FINGER31,
	TP_BIP01_R_FINGER32,
	-1
};

const int AVP_Alien_Description::s_aiThirdPersonLeftLegChainBonesArray[ uAVP_ALIEN_NUM_IK_BONES_PER_LIMB + 1 ] =
{
	TP_BIP01_L_THIGH,
	TP_BIP01_L_CALF,
	TP_BIP01_L_FOOT,
	-1
};

const int AVP_Alien_Description::s_aiThirdPersonRightLegChainBonesArray[ uAVP_ALIEN_NUM_IK_BONES_PER_LIMB + 1 ] =
{
	TP_BIP01_R_THIGH,
	TP_BIP01_R_CALF,
	TP_BIP01_R_FOOT,
	-1
};

const int AVP_Alien_Description::s_aiThirdPersonTailChainBonesArray[ uAVP_ALIEN_NUM_IK_BONES_PER_TAIL + 1 ] =
{
	TP_BIP01_TAIL,
	TP_BIP01_TAIL1,
	TP_BIP01_TAIL2,
	TP_BIP01_TAIL3,
	TP_BIP01_TAIL4,
	TP_BIP01_TAIL5,
	TP_BIP01_TAIL6,
	TP_BIP01_TAIL7,
	-1
};

const int AVP_Alien_Description::s_aiFirstPersonLeftArmChainTwistBonesArray[ uAVP_ALIEN_NUM_IK_BONES_PER_TAIL + 1  ] =
{
	FP_BIP01_LUPARMTWIST,
	FP_BIP01_L_FORETWIST,
	-1,
	-1
};

const int AVP_Alien_Description::s_aiFirstPersonRightArmChainTwistBonesArray[ uAVP_ALIEN_NUM_IK_BONES_PER_TAIL + 1  ] =
{
	FP_BIP01_RUPARMTWIST,
	FP_BIP01_R_FORETWIST,
	-1,
	-1
};

const int AVP_Alien_Description::s_aiFirstPersonLeftLegChainTwistBonesArray[ uAVP_ALIEN_NUM_IK_BONES_PER_TAIL + 1  ] =
{
	FP_BIP01_LTHIGHTWIST,
	-1,
	-1,
	-1
};

const int AVP_Alien_Description::s_aiFirstPersonRightLegChainTwistBonesArray[ uAVP_ALIEN_NUM_IK_BONES_PER_TAIL + 1  ] =
{
	FP_BIP01_RTHIGHTWIST,
	-1,
	-1,
	-1
};

const int AVP_Alien_Description::s_aiThirdPersonLeftArmChainTwistBonesArray[ uAVP_ALIEN_NUM_IK_BONES_PER_TAIL + 1  ] =
{
	TP_BIP01_LUPARMTWIST,
	TP_BIP01_L_FORETWIST,
	-1,
	-1
};

const int AVP_Alien_Description::s_aiThirdPersonRightArmChainTwistBonesArray[ uAVP_ALIEN_NUM_IK_BONES_PER_TAIL + 1  ] =
{
	TP_BIP01_RUPARMTWIST,
	TP_BIP01_R_FORETWIST,
	-1,
	-1
};

const int AVP_Alien_Description::s_aiThirdPersonLeftLegChainTwistBonesArray[ uAVP_ALIEN_NUM_IK_BONES_PER_TAIL + 1  ] =
{
	TP_BIP01_LTHIGHTWIST,
	-1,
	-1,
	-1
};

const int AVP_Alien_Description::s_aiThirdPersonRightLegChainTwistBonesArray[ uAVP_ALIEN_NUM_IK_BONES_PER_TAIL + 1  ] =
{
	TP_BIP01_RTHIGHTWIST,
	-1,
	-1,
	-1
};

const int* AVP_Alien_Description::s_apiFirstPersonChainsArray[ AVP_Alien_Description::NUM_IK_CHAINS ] = 
{
	s_aiFirstPersonLeftLegChainBonesArray,
	s_aiFirstPersonRightLegChainBonesArray,
	s_aiFirstPersonLeftArmChainBonesArray,
	s_aiFirstPersonRightArmChainBonesArray,
	s_aiFirstPersonLeftFinger0ChainBonesArray,
	s_aiFirstPersonLeftFinger1ChainBonesArray,
	s_aiFirstPersonLeftFinger2ChainBonesArray,
	s_aiFirstPersonLeftFinger3ChainBonesArray,
	s_aiFirstPersonRightFinger0ChainBonesArray,
	s_aiFirstPersonRightFinger1ChainBonesArray,
	s_aiFirstPersonRightFinger2ChainBonesArray,
	s_aiFirstPersonRightFinger3ChainBonesArray,
	s_aiFirstPersonTailChainBonesArray
};

const int* AVP_Alien_Description::s_apiFirstPersonTwistChainsArray[ AVP_Alien_Description::NUM_IK_CHAINS ] = 
{
	s_aiFirstPersonLeftLegChainTwistBonesArray,
	s_aiFirstPersonRightLegChainTwistBonesArray,
	s_aiFirstPersonLeftArmChainTwistBonesArray,
	s_aiFirstPersonRightArmChainTwistBonesArray,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL
};

const int* AVP_Alien_Description::s_apiThirdPersonChainsArray[ AVP_Alien_Description::NUM_IK_CHAINS ] = 
{
	s_aiThirdPersonLeftLegChainBonesArray,
	s_aiThirdPersonRightLegChainBonesArray,
	s_aiThirdPersonLeftArmChainBonesArray,
	s_aiThirdPersonRightArmChainBonesArray,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	s_aiThirdPersonTailChainBonesArray
};

const int* AVP_Alien_Description::s_apiThirdPersonTwistChainsArray[ AVP_Alien_Description::NUM_IK_CHAINS ] = 
{
	s_aiThirdPersonLeftLegChainTwistBonesArray,
	s_aiThirdPersonRightLegChainTwistBonesArray,
	s_aiThirdPersonLeftArmChainTwistBonesArray,
	s_aiThirdPersonRightArmChainTwistBonesArray,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL
};

////////////////////////////////////////////////////////////////////////////////
// Private functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Protected functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Public functions...
////////////////////////////////////////////////////////////////////////////////

u_int AVP_Alien_Description::GetElementFromBoneID(int iBoneID)
{
	switch(iBoneID)
	{
		case TP_BIP01_XTRA01:
		case TP_BIP01_XTRA02:
		{
			return(AVP_SET_ALIEN_PIPES_LEFT);
		}

		case TP_BIP01_XTRA01OPP:
		case TP_BIP01_XTRA02OPP:
		{
			return(AVP_SET_ALIEN_PIPES_RIGHT);
		}

		case TP_BIP01_NECK:
		case TP_BIP01_NECK1:
		case TP_BIP01_HEAD:
		{
			return(AVP_SET_HEAD);
		}

		case TP_INNERMOUTH:
		case TP_INNERJAWBOTTOM:
		case TP_INNERJAWTOP:
		case TP_JAW:
		case TP_BOTTOMLIPLEFT:
		case TP_BOTTOMLIPRIGHT:
		case TP_TOPLIPLEFT:
		case TP_TOPLIPRIGHT:
		{
			return(AVP_SET_ALIEN_FACE);
		}

		case TP_BIP01_LUPARMTWIST:
		{
			return(AVP_SET_LEFT_ELBOW);
		}

		case TP_BIP01_L_UPPERARM:
		case TP_BIP01_L_FOREARM:
		case TP_BIP01_L_FORETWIST:
		case TP_BIP01_L_HAND:
		case TP_BIP01_L_FINGER0:
		case TP_BIP01_L_FINGER01:
		case TP_BIP01_L_FINGER02:
		case TP_BIP01_L_FINGER1:
		case TP_BIP01_L_FINGER11:
		case TP_BIP01_L_FINGER12:
		case TP_BIP01_L_FINGER2:
		case TP_BIP01_L_FINGER21:
		case TP_BIP01_L_FINGER22:
		case TP_BIP01_L_FINGER3:
		case TP_BIP01_L_FINGER31:
		case TP_BIP01_L_FINGER32:
		{
			return(AVP_SET_LEFT_ARM);
		}

		case TP_BIP01_RUPARMTWIST:
		{
			return(AVP_SET_RIGHT_ELBOW);
		}

		case TP_BIP01_R_UPPERARM:
		case TP_BIP01_R_FOREARM:
		case TP_BIP01_R_FORETWIST:
		case TP_BIP01_R_HAND:
		case TP_BIP01_R_FINGER0:
		case TP_BIP01_R_FINGER01:
		case TP_BIP01_R_FINGER02:
		case TP_BIP01_R_FINGER1:
		case TP_BIP01_R_FINGER11:
		case TP_BIP01_R_FINGER12:
		case TP_BIP01_R_FINGER2:
		case TP_BIP01_R_FINGER21:
		case TP_BIP01_R_FINGER22:
		case TP_BIP01_R_FINGER3:
		case TP_BIP01_R_FINGER31:
		case TP_BIP01_R_FINGER32:
		{
			return(AVP_SET_RIGHT_ARM);
		}

		case TP_BIP01_LTHIGHTWIST:
		{
			return(AVP_SET_LEFT_KNEE);
		}

		case TP_BIP01_L_THIGH:
		case TP_BIP01_L_CALF:
		case TP_BIP01_L_FOOT:
		case TP_BIP01_L_TOE0:
		case TP_BIP01_L_TOE01:
		{
			return(AVP_SET_LEFT_LEG);
		}

		case TP_BIP01_RTHIGHTWIST:
		{
			return(AVP_SET_RIGHT_KNEE);
		}

		case TP_BIP01_R_THIGH:
		case TP_BIP01_R_CALF:
		case TP_BIP01_R_FOOT:
		case TP_BIP01_R_TOE0:
		case TP_BIP01_R_TOE01:
		{
			return(AVP_SET_RIGHT_LEG);
		}

		case TP_BIP01_TAIL:
		case TP_BIP01_TAIL1:
		case TP_BIP01_TAIL2:
		{
			return(AVP_SET_ALIEN_TAIL);
		}

		case TP_BIP01_TAIL3:
		case TP_BIP01_TAIL4:
		case TP_BIP01_TAIL5:
		case TP_BIP01_TAIL6:
		case TP_BIP01_TAIL7:
		{
			return(AVP_SET_ALIEN_TAIL_TIP);
		}

		case TP_BIP01_L_CLAVICLE:
		case TP_BIP01_R_CLAVICLE:
		case TP_BIP01_PELVIS:
		case TP_BIP01_SPINE:
		case TP_BIP01_SPINE1:
		case TP_BIP01_SPINE2:
		case TP_BIP01_XTRA03:
		{
			return(AVP_SET_BODY);
		}

		case TP_BIP01:
		default:
		{
			return(NUM_AVP_SKIN_ELEMENT_TYPES);
		}
	}
}

// Since bones->elements is a many->one mapping, this function will return one of the bones associated with the given element, but not necessarily the only one.
int AVP_Alien_Description::GetAnyBoneIDFromElement(u_int uElement)
{
	switch (uElement)
	{
	case AVP_SET_BODY:
		return TP_BIP01_SPINE2;

	case AVP_SET_HEAD:
	case AVP_SET_HEAD_AFTER_HEADBITE:
		return TP_BIP01_HEAD;

	case AVP_SET_LEFT_ARM:
		return TP_BIP01_L_UPPERARM;

	case AVP_SET_LEFT_ELBOW:
		return TP_BIP01_LUPARMTWIST;

	case AVP_SET_RIGHT_ARM:
		return TP_BIP01_R_UPPERARM;

	case AVP_SET_RIGHT_ELBOW:
		return TP_BIP01_RUPARMTWIST;

	case AVP_SET_LEFT_LEG:
		return TP_BIP01_L_THIGH;

	case AVP_SET_LEFT_KNEE:
		return TP_BIP01_LTHIGHTWIST;

	case AVP_SET_RIGHT_LEG:
		return TP_BIP01_R_THIGH;

	case AVP_SET_RIGHT_KNEE:
		return TP_BIP01_RTHIGHTWIST;

	case AVP_SET_ALIEN_PIPES_LEFT:
		return TP_BIP01_XTRA01;

	case AVP_SET_ALIEN_PIPES_RIGHT:
		return TP_BIP01_XTRA01OPP;

	case AVP_SET_ALIEN_FACE:
		return TP_JAW;

	case AVP_SET_ALIEN_TAIL:
		return TP_BIP01_TAIL;

	case AVP_SET_ALIEN_TAIL_TIP:
		return TP_BIP01_TAIL3;

	default:
		return -1;
	}
}

#if defined(ASURA_TOOLS)

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Alien_Description::AddFilesToExport
// Author       : Andy Tate
// Date         : 29-Jan-2007
////////////////////////////////////////////////////////////////////////////////

void AVP_Alien_Description::AddFilesToExport(const bool bFirstPerson,
											 const Asura_Hash_ID uBlueprintHash,
											 List<Asura_String>& xNameList)
{
	if(uBlueprintHash != ASURA_HASH_ID_UNSET)
	{
		Tools_Mode_Blueprint::AddBlueprintFilesToExportList(xNameList, uAVP_BLUEPRINT_TYPE_ACTORS, uBlueprintHash, false);
		if ( bFirstPerson )
		{
			Tools_Mode_Blueprint::AddBlueprintFilesToExportList(xNameList, uAVP_BLUEPRINT_TYPE_PLAYERS, uBlueprintHash, false);
		}
	}

	// Also add everything of type "Grapple Alien". This has first and third person anims, so do it regardless of skin type.
	Tools_Mode_Blueprint::AddBlueprintFilesToExportList(xNameList, uAVP_BLUEPRINT_TYPE_GRAPPLEALIEN, ASURA_HASH_ID_UNSET, true);
}

/// End of function 'AVP_Alien_Description::AddFilesToExport'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Alien_Description::CanBeImpaled
// Author       : Andy Tate
// Date         : 31-Jan-2007
////////////////////////////////////////////////////////////////////////////////

bool AVP_Alien_Description::CanBeImpaled (const u_int uBoneID) const
{
	Asura_Assert(uBoneID < MAX_THIRD_PERSON_BONES_IDS, "Unexpected bone ID.");
	
	switch(uBoneID)
	{
		case TP_BIP01:
		case TP_BIP01_PELVIS:
		case TP_BIP01_SPINE:
		case TP_BIP01_SPINE1:
		case TP_BIP01_SPINE2:
		{
			return(true);
		}

		default:
		{
			return(false);
		}
	}		// End of switch.
}

/// End of function 'AVP_ServerActor_Alien::CanBeImpaled'. ///

#endif	// defined(ASURA_TOOLS)


/// End of file. ///
