
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_Alien_Description.h
// Author : Andy Tate
// Date   : 10-Jan-2007
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __AVP_ALIEN_DESCRIPTION_H__
#define __AVP_ALIEN_DESCRIPTION_H__

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura_Hash.h"
#include "Physics/Asura_Physics_Ragdoll_Description.h"
#include "Sound/Asura_Sound_Event_System.h"

#if defined(ASURA_TOOLS)
#include "Misc/list_tem.hpp"
#endif	// defined(ASURA_TOOLS)

#include "Entity/PhysicalObject/Actor/AVP_ClientEntity_Actor.h"
#include "Physics/AVP_Collision_Descriptor.h"

////////////////////////////////////////////////////////////////////////////////
// Forward class declarations...
////////////////////////////////////////////////////////////////////////////////

// Sound Events...

static const u_int uAVP_ALIEN_NUM_IK_BONES_PER_LIMB = 3;
static const u_int uAVP_ALIEN_NUM_IK_BONES_PER_TAIL = 8;

////////////////////////////////////////////////////////////////////////////////
// Class declaration...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_Alien_Description
// Author       : Andy Tate
// Date         : 10-Jan-2007
//
// Description  : Class used to contain the appropriate descriptions for the
//				  Alien skins.
////////////////////////////////////////////////////////////////////////////////

class AVP_Alien_Description
{
public:
	// Public enumerated types...
	enum ThirdPersonBoneID
	{
		TP_BIP01,
		TP_BIP01_PELVIS,
		TP_BIP01_SPINE,
		TP_BIP01_LTHIGHTWIST,
		TP_BIP01_RTHIGHTWIST,
		TP_BIP01_SPINE1,
		TP_BIP01_SPINE2,
		TP_BIP01_NECK,
		TP_BIP01_L_CLAVICLE,
		TP_BIP01_L_UPPERARM,
		TP_BIP01_L_FOREARM,
		TP_BIP01_L_HAND,
		TP_BIP01_L_FINGER0,
		TP_BIP01_L_FINGER01,
		TP_BIP01_L_FINGER02,
		TP_BIP01_L_FINGER1,
		TP_BIP01_L_FINGER11,
		TP_BIP01_L_FINGER12,
		TP_BIP01_L_FINGER2,
		TP_BIP01_L_FINGER21,
		TP_BIP01_L_FINGER22,
		TP_BIP01_L_FINGER3,
		TP_BIP01_L_FINGER31,
		TP_BIP01_L_FINGER32,
		TP_BIP01_L_FORETWIST,
		TP_BIP01_LUPARMTWIST,
		TP_BIP01_NECK1,
		TP_BIP01_HEAD,
		TP_INNERMOUTH,
		TP_INNERJAWBOTTOM,
		TP_INNERJAWTOP,
		TP_JAW,
		TP_BOTTOMLIPLEFT,
		TP_BOTTOMLIPRIGHT,
		TP_TOPLIPLEFT,
		TP_TOPLIPRIGHT,
		TP_BIP01_R_CLAVICLE,
		TP_BIP01_R_UPPERARM,
		TP_BIP01_R_FOREARM,
		TP_BIP01_R_HAND,
		TP_BIP01_R_FINGER0,
		TP_BIP01_R_FINGER01,
		TP_BIP01_R_FINGER02,
		TP_BIP01_R_FINGER1,
		TP_BIP01_R_FINGER11,
		TP_BIP01_R_FINGER12,
		TP_BIP01_R_FINGER2,
		TP_BIP01_R_FINGER21,
		TP_BIP01_R_FINGER22,
		TP_BIP01_R_FINGER3,
		TP_BIP01_R_FINGER31,
		TP_BIP01_R_FINGER32,
		TP_BIP01_R_FORETWIST,
		TP_BIP01_RUPARMTWIST,
		TP_BIP01_XTRA03,
		TP_BIP01_XTRA01,
		TP_BIP01_XTRA01OPP,
		TP_BIP01_XTRA02,
		TP_BIP01_XTRA02OPP,
		TP_BIP01_L_THIGH,
		TP_BIP01_L_CALF,
		TP_BIP01_L_FOOT,
		TP_BIP01_L_TOE0,
		TP_BIP01_L_TOE01,
		TP_BIP01_R_THIGH,
		TP_BIP01_R_CALF,
		TP_BIP01_R_FOOT,
		TP_BIP01_R_TOE0,
		TP_BIP01_R_TOE01,
		TP_BIP01_TAIL,
		TP_BIP01_TAIL1,
		TP_BIP01_TAIL2,
		TP_BIP01_TAIL3,
		TP_BIP01_TAIL4,
		TP_BIP01_TAIL5,
		TP_BIP01_TAIL6,
		TP_BIP01_TAIL7,

		MAX_THIRD_PERSON_BONES_IDS
	};

	enum FirstPersonBoneID
	{
		FP_BIP01,
		FP_BIP01_PELVIS,
		FP_BIP01_SPINE,
		FP_BIP01_LTHIGHTWIST,
		FP_BIP01_RTHIGHTWIST,
		FP_BIP01_SPINE1,
		FP_BIP01_SPINE2,
		FP_BIP01_NECK,
		FP_BIP01_L_CLAVICLE,
		FP_BIP01_L_UPPERARM,
		FP_BIP01_L_FOREARM,
		FP_BIP01_L_HAND,
		FP_BIP01_L_FINGER0,
		FP_BIP01_L_FINGER01,
		FP_BIP01_L_FINGER02,
		FP_BIP01_L_FINGER1,
		FP_BIP01_L_FINGER11,
		FP_BIP01_L_FINGER12,
		FP_BIP01_L_FINGER2,
		FP_BIP01_L_FINGER21,
		FP_BIP01_L_FINGER22,
		FP_BIP01_L_FINGER3,
		FP_BIP01_L_FINGER31,
		FP_BIP01_L_FINGER32,
		FP_BIP01_L_FORETWIST,
		FP_BIP01_LUPARMTWIST,
		FP_BIP01_NECK1,
		FP_BIP01_HEAD,
		FP_BIP01_R_CLAVICLE,
		FP_BIP01_R_UPPERARM,
		FP_BIP01_R_FOREARM,
		FP_BIP01_R_HAND,
		FP_BIP01_R_FINGER0,
		FP_BIP01_R_FINGER01,
		FP_BIP01_R_FINGER02,
		FP_BIP01_R_FINGER1,
		FP_BIP01_R_FINGER11,
		FP_BIP01_R_FINGER12,
		FP_BIP01_R_FINGER2,
		FP_BIP01_R_FINGER21,
		FP_BIP01_R_FINGER22,
		FP_BIP01_R_FINGER3,
		FP_BIP01_R_FINGER31,
		FP_BIP01_R_FINGER32,
		FP_BIP01_R_FORETWIST,
		FP_BIP01_RUPARMTWIST,
		FP_BIP01_XTRA03,
		FP_BIP01_XTRA01,
		FP_BIP01_XTRA01OPP,
		FP_BIP01_XTRA02,
		FP_BIP01_XTRA02OPP,
		FP_BIP01_L_THIGH,
		FP_BIP01_L_CALF,
		FP_BIP01_L_FOOT,
		FP_BIP01_L_TOE0,
		FP_BIP01_L_TOE01,
		FP_BIP01_R_THIGH,
		FP_BIP01_R_CALF,
		FP_BIP01_R_FOOT,
		FP_BIP01_R_TOE0,
		FP_BIP01_R_TOE01,
		FP_BIP01_TAIL,
		FP_BIP01_TAIL1,
		FP_BIP01_TAIL2,
		FP_BIP01_TAIL3,
		FP_BIP01_TAIL4,
		FP_BIP01_TAIL5,
		FP_BIP01_TAIL6,
		FP_BIP01_TAIL7,
		FP_CAMERA_BONE,
		FP_INNER_MOUTH,
		FP_INNER_MOUTH_BOTTOM,
		FP_INNER_MOUTH_LEFT,
		FP_INNER_MOUTH_RIGHT,
		FP_INNER_MOUTH_TOP,

		MAX_FIRST_PERSON_BONES_IDS
	};

	enum MarkerPointID
	{
		MPID_INNERMOUTH,
		MPID_SPIT,
		MPID_THROAT,

		MPID_BLOOD_AIRTUBE_BOTTOMLEFT,
		MPID_BLOOD_AIRTUBE_BOTTOMRIGHT,
		MPID_BLOOD_AIRTUBE_TOPLEFT,
		MPID_BLOOD_AIRTUBE_TOPRIGHT,
		MPID_BLOOD_UPPERARM_LEFT,
		MPID_BLOOD_HEAD,
		MPID_BLOOD_NECK,
		MPID_BLOOD_UPPERARM_RIGHT,
		MPID_BLOOD_THIGH_LEFT,
		MPID_BLOOD_THIGH_RIGHT,
		MPID_BLOOD_TAILTIP,
		MPID_BLOOD_TAIL,

		NUM_MARKER_POINT_IDS
	};

	enum IKChains
	{
		LEFT_LEG,
		RIGHT_LEG,
		LEFT_ARM,
		RIGHT_ARM,
		LEFT_FINGER0,
		LEFT_FINGER1,
		LEFT_FINGER2,
		LEFT_FINGER3,
		RIGHT_FINGER0,
		RIGHT_FINGER1,
		RIGHT_FINGER2,
		RIGHT_FINGER3,
		TAIL,

		NUM_IK_CHAINS
	};

	// Public functions...
#if defined(ASURA_TOOLS)
	static void	AddFilesToExport(const bool bFirstPerson, const Asura_Hash_ID uBlueprintHash, List<Asura_String>& xNameList);
#endif	// defined(ASURA_TOOLS)

	// Accessor functions...
	static Asura_Hash_ID	GetMarkerPointHashID(const MarkerPointID eID)	{ return(s_auMarkerPointHashIDs[eID]); }
	static int				GetFootBoneID( u_int uFootIndex, bool bFirstPerson ) { Asura_Assert( uFootIndex < AVP_ClientEntity_Actor::MAX_NUM_FEET, "Bad foot index" ); return bFirstPerson ? s_aiFirstPersonFootBoneID[ uFootIndex ] : s_aiThirdPersonFootBoneID[ uFootIndex ]; }

	static const int* 		GetFirstPersonChainsArray(const u_int uIndex)		{ return(s_apiFirstPersonChainsArray[uIndex]); }
	static const int* 		GetThirdPersonChainsArray(const u_int uIndex)		{ return(s_apiThirdPersonChainsArray[uIndex]); }
	static const int* 		GetFirstPersonTwistChainsArray(const u_int uIndex)	{ return(s_apiFirstPersonTwistChainsArray[uIndex]); }
	static const int* 		GetThirdPersonTwistChainsArray(const u_int uIndex)	{ return(s_apiThirdPersonTwistChainsArray[uIndex]); }

	static u_int			GetElementFromBoneID(int iBoneID);
	static int				GetAnyBoneIDFromElement(u_int uElement); // Since bones->elements is a many->one mapping, this function will return one of the bones associated with the given element, but not necessarily the only one.

	virtual	bool			CanBeImpaled(const u_int uBoneID) const;

protected:
	static const int 			s_aiFirstPersonLeftArmChainBonesArray[ uAVP_ALIEN_NUM_IK_BONES_PER_LIMB + 1 ];
	static const int 			s_aiFirstPersonLeftFinger0ChainBonesArray[ uAVP_ALIEN_NUM_IK_BONES_PER_LIMB + 1 ];
	static const int 			s_aiFirstPersonLeftFinger1ChainBonesArray[ uAVP_ALIEN_NUM_IK_BONES_PER_LIMB + 1 ];
	static const int 			s_aiFirstPersonLeftFinger2ChainBonesArray[ uAVP_ALIEN_NUM_IK_BONES_PER_LIMB + 1 ];
	static const int 			s_aiFirstPersonLeftFinger3ChainBonesArray[ uAVP_ALIEN_NUM_IK_BONES_PER_LIMB + 1 ];
	static const int 			s_aiFirstPersonRightArmChainBonesArray[ uAVP_ALIEN_NUM_IK_BONES_PER_LIMB + 1 ];
	static const int 			s_aiFirstPersonRightFinger0ChainBonesArray[ uAVP_ALIEN_NUM_IK_BONES_PER_LIMB + 1 ];
	static const int 			s_aiFirstPersonRightFinger1ChainBonesArray[ uAVP_ALIEN_NUM_IK_BONES_PER_LIMB + 1 ];
	static const int 			s_aiFirstPersonRightFinger2ChainBonesArray[ uAVP_ALIEN_NUM_IK_BONES_PER_LIMB + 1 ];
	static const int 			s_aiFirstPersonRightFinger3ChainBonesArray[ uAVP_ALIEN_NUM_IK_BONES_PER_LIMB + 1 ];
	static const int 			s_aiFirstPersonLeftLegChainBonesArray[ uAVP_ALIEN_NUM_IK_BONES_PER_LIMB + 1 ];
	static const int 			s_aiFirstPersonRightLegChainBonesArray[ uAVP_ALIEN_NUM_IK_BONES_PER_LIMB + 1 ];
	static const int 			s_aiFirstPersonTailChainBonesArray[ uAVP_ALIEN_NUM_IK_BONES_PER_TAIL + 1 ];

	static const int 			s_aiFirstPersonLeftArmChainTwistBonesArray[ uAVP_ALIEN_NUM_IK_BONES_PER_TAIL + 1  ];
	static const int 			s_aiFirstPersonRightArmChainTwistBonesArray[ uAVP_ALIEN_NUM_IK_BONES_PER_TAIL + 1  ];
	static const int 			s_aiFirstPersonLeftLegChainTwistBonesArray[ uAVP_ALIEN_NUM_IK_BONES_PER_TAIL + 1  ];
	static const int 			s_aiFirstPersonRightLegChainTwistBonesArray[ uAVP_ALIEN_NUM_IK_BONES_PER_TAIL + 1  ];

	static const int 			s_aiThirdPersonLeftArmChainBonesArray[ uAVP_ALIEN_NUM_IK_BONES_PER_LIMB + 1 ];
	static const int 			s_aiThirdPersonLeftFinger0ChainBonesArray[ uAVP_ALIEN_NUM_IK_BONES_PER_LIMB + 1 ];
	static const int 			s_aiThirdPersonLeftFinger1ChainBonesArray[ uAVP_ALIEN_NUM_IK_BONES_PER_LIMB + 1 ];
	static const int 			s_aiThirdPersonLeftFinger2ChainBonesArray[ uAVP_ALIEN_NUM_IK_BONES_PER_LIMB + 1 ];
	static const int 			s_aiThirdPersonLeftFinger3ChainBonesArray[ uAVP_ALIEN_NUM_IK_BONES_PER_LIMB + 1 ];
	static const int 			s_aiThirdPersonRightArmChainBonesArray[ uAVP_ALIEN_NUM_IK_BONES_PER_LIMB + 1 ];
	static const int 			s_aiThirdPersonRightFinger0ChainBonesArray[ uAVP_ALIEN_NUM_IK_BONES_PER_LIMB + 1 ];
	static const int 			s_aiThirdPersonRightFinger1ChainBonesArray[ uAVP_ALIEN_NUM_IK_BONES_PER_LIMB + 1 ];
	static const int 			s_aiThirdPersonRightFinger2ChainBonesArray[ uAVP_ALIEN_NUM_IK_BONES_PER_LIMB + 1 ];
	static const int 			s_aiThirdPersonRightFinger3ChainBonesArray[ uAVP_ALIEN_NUM_IK_BONES_PER_LIMB + 1 ];
	static const int 			s_aiThirdPersonLeftLegChainBonesArray[ uAVP_ALIEN_NUM_IK_BONES_PER_LIMB + 1 ];
	static const int 			s_aiThirdPersonRightLegChainBonesArray[ uAVP_ALIEN_NUM_IK_BONES_PER_LIMB + 1 ];
	static const int 			s_aiThirdPersonTailChainBonesArray[ uAVP_ALIEN_NUM_IK_BONES_PER_TAIL + 1 ];

	static const int 			s_aiThirdPersonLeftArmChainTwistBonesArray[ uAVP_ALIEN_NUM_IK_BONES_PER_TAIL + 1  ];
	static const int 			s_aiThirdPersonRightArmChainTwistBonesArray[ uAVP_ALIEN_NUM_IK_BONES_PER_TAIL + 1  ];
	static const int 			s_aiThirdPersonLeftLegChainTwistBonesArray[ uAVP_ALIEN_NUM_IK_BONES_PER_TAIL + 1  ];
	static const int 			s_aiThirdPersonRightLegChainTwistBonesArray[ uAVP_ALIEN_NUM_IK_BONES_PER_TAIL + 1  ];

	static const Asura_Hash_ID	s_auMarkerPointHashIDs[NUM_MARKER_POINT_IDS];
	static const int			s_aiFirstPersonFootBoneID[ AVP_ClientEntity_Actor::MAX_NUM_FEET ];
	static const int			s_aiThirdPersonFootBoneID[ AVP_ClientEntity_Actor::MAX_NUM_FEET ];

	static const int* 			s_apiFirstPersonChainsArray[ NUM_IK_CHAINS ];
	static const int* 			s_apiThirdPersonChainsArray[ NUM_IK_CHAINS ];
	static const int* 			s_apiFirstPersonTwistChainsArray[ NUM_IK_CHAINS ];
	static const int* 			s_apiThirdPersonTwistChainsArray[ NUM_IK_CHAINS ];

private:
	// Private variables...

	// Private functions...
	AVP_Alien_Description(void);
	AVP_Alien_Description(const AVP_Alien_Description& xOther);
	~AVP_Alien_Description();

	AVP_Alien_Description&	operator=(const AVP_Alien_Description& xOther);

};   // End of class 'AVP_Alien_Description'.


#endif   // ifndef __AVP_ALIEN_DESCRIPTION_H__


/// End of file. ///
