////////////////////////////////////////////////////////////////
//
// File:		AVP_Alien_MeleeCounter_System.h
// Author:		Anthony Gregson
// Date:		22/02/08
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//
////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __AVP_ALIEN_MELEECOUNTER_SYSTEM_H__
#define __AVP_ALIEN_MELEECOUNTER_SYSTEM_H__

///////////////////////////////////////////////
// #includes
///////////////////////////////////////////////

#include "Actor/Vertebrates/AVP_Vertebrate_MeleeCounter_System.h"

///////////////////////////////////////////////

class AVP_ServerEntity_Actor;
class AVP_ServerActor_Alien;

/////////////////////////////////////////////////////////////////////
//
// Class		:	AVP_Alien_MeleeCounter_System
//
//////////////////////////////////////////////////////////////////////

enum AVP_ALIEN_MELEEATTACK_TYPE
{
	AVP_ALIEN_MELEEATTACK_TYPE_PRIMARYATTACK = NUM_AVP_MELEEATTACK_TYPES,
	AVP_ALIEN_MELEEATTACK_TYPE_SECONDARYATTACK,
	AVP_ALIEN_MELEEATTACK_TYPE_SECONDARYRELEASE,
	AVP_ALIEN_MELEEATTACK_TYPE_SECONDARYCANCEL,
	AVP_ALIEN_MELEEATTACK_TYPE_FOCUSATTACK, // Will probably match the pounce, but allows us to differentiate between player inputs (if they press this, but are too close, they should do a primary attack.
	AVP_ALIEN_MELEEATTACK_TYPE_FOCUSPOUNCE,
	AVP_ALIEN_MELEEATTACK_TYPE_NONFOCUSPOUNCE, // For AIs, who don't go through the focus system.
	AVP_ALIEN_MELEEATTACK_TYPE_JUMPSLASH,
	AVP_ALIEN_MELEEATTACK_TYPE_HEADBITE,
	AVP_ALIEN_MELEEATTACK_TYPE_GRAPPLEHEADBITE,
	AVP_ALIEN_MELEEATTACK_TYPE_AI_CRAWLTOSTANDSLASH_R,
	AVP_ALIEN_MELEEATTACK_TYPE_AI_CRAWLTOSTANDSLASH_L,
	AVP_ALIEN_MELEEATTACK_TYPE_AI_CRAWLTOSTANDSLASH_SHORT_R,
	AVP_ALIEN_MELEEATTACK_TYPE_AI_CRAWLTOSTANDSLASH_SHORT_L,
	AVP_ALIEN_MELEEATTACK_TYPE_AI_STANDSLASHLOOP_L, // Intended to follow on from the end of the appropriate previous stand slash.
	AVP_ALIEN_MELEEATTACK_TYPE_AI_STANDSLASHLOOP_R, // Intended to follow on from the end of the appropriate previous stand slash.
	AVP_ALIEN_MELEEATTACK_TYPE_AI_WALKSLASHLOOP_L, // Intended to follow on from the end of the appropriate previous stand slash.
	AVP_ALIEN_MELEEATTACK_TYPE_AI_WALKSLASHLOOP_R, // Intended to follow on from the end of the appropriate previous stand slash.
	AVP_ALIEN_MELEEATTACK_TYPE_PICKUPENEMY,
	AVP_ALIEN_MELEEATTACK_TYPE_PICKUPENEMY_HEADBITE,
	AVP_ALIEN_MELEEATTACK_TYPE_AI_JUMPSLASHWALL, // Jump slash attack from the wall.
	AVP_ALIEN_MELEEATTACK_TYPE_AI_INJURED,
	AVP_ALIEN_MELEEATTACK_TYPE_AI_POUNCEGRAPPLE,

	NUM_AVP_ALIEN_MELEEATTACK_TYPES,
};

class AVP_Alien_MeleeCounter_System : public AVP_Vertebrate_MeleeCounter_System
{
public: 
	AVP_Alien_MeleeCounter_System(AVP_ServerActor_Alien& xOwner);
	virtual ~AVP_Alien_MeleeCounter_System() {};

	virtual void InitialiseFromBlueprint(const Asura_Blueprint& xBlueprint);

	//virtual bool HandleMessage( const Asura_Message* pxMessage );
	//virtual void Update( void );

	virtual bool StartMeleeMove(u_int uType);

	static const char* GetStandingLeftSlashAttackBaseParameterName() { return s_szStandingLeftSlashAttackBaseParameterName; }
	static const char* GetStandingRightSlashAttackBaseParameterName() { return s_szStandingRightSlashAttackBaseParameterName; }

	virtual bool InsidePounceRange() const;
	virtual bool InsideTailAttackRange() const;

protected:
	virtual void CheckForAndDealMeleeDamage();
	virtual void ApplyGrappleDamage() const;

	//virtual bool GetAttackTypeCanCounter(u_int uType) const; // Returns true if a particular melee request can also be used to counter an incoming attack.

	// Go off and get a move hash relevant to the species and weapon
	virtual Asura_Hash_ID GetMeleeMoveHash(u_int uType) const;
	virtual u_int PickMeleeMoveVariation(Asura_Hash_ID uMoveHash) const;
	virtual Asura_Hash_ID GetMeleeMoveGrappleSetBlueprintParam(u_int uType) const;

	virtual bool GetCanBlock() const;

	// Functions for counters by each species, split up to keep the code clearer. Uses the move we're currently doing to pick the correct counter for that species.
	//virtual bool GetCanCounter_AlienCounter(const AVP_ServerEntity_Actor& xEnemy, const u_int uQueuedCounterType, Asura_Hash_ID& uMoveHash, u_int& uNewAttackType, bool& bMakeEnemyIntoPuppet, Asura_Hash_ID& uGrappleSetBlueprintParam) const;
	//virtual bool GetCanCounter_HumanCounter(const AVP_ServerEntity_Actor& xEnemy, const u_int uQueuedCounterType, Asura_Hash_ID& uMoveHash, u_int& uNewAttackType, bool& bMakeEnemyIntoPuppet, Asura_Hash_ID& uGrappleSetBlueprintParam) const;
	virtual bool GetCanCounter_PredatorCounter(const AVP_ServerEntity_Actor& xEnemy, const u_int uQueuedCounterType, Asura_Hash_ID& uMoveHash, u_int& uNewAttackType, bool& bMakeEnemyIntoPuppet, Asura_Hash_ID& uGrappleSetBlueprintParam) const;

	virtual float GetCurrentMaxCounterDistance() const;

	virtual float GetAttackDamage() const;
	virtual Asura_Hash_ID GetAttackDamageType() const;
	virtual float GetAttackImpulseStrength() const;
	virtual void GetAttackDirection(const AVP_ServerEntity_PhysicalObject* pxTarget, Asura_Vector_3& xDirection) const;
	virtual int GetAttackBoneHit(Asura_Guid uDamagedGuid) const;
	virtual bool IsAttackUnblockable() const;
	virtual bool GetAttackOnlyHitsCurrentTarget() const;

	virtual Asura_Hash_ID GetAttackGrappleSetBlueprintParamID() const;

	virtual bool GetSendIncomingMeleeMessage(u_int uAttackType) const;

	virtual void SendMeleeAttackHitMessage();

	virtual const Asura_Blueprint* GetExtraMeleeDataBlueprint() const;
	virtual bool GetQuickAttackBaseParameterNames(const char*& szLeftAttackName, const char*& szRightAttackName) const;
	virtual Asura_Hash_ID GetPounceBlueprintType() const;

	float m_fTailLashDamage;
	float m_fTailLashImpulse;
	float m_fTailStabDamage;
	float m_fTailStabImpulse;
	float m_fTailAttackRange;

	static const char* s_szStandingLeftSlashAttackBaseParameterName;
	static const char* s_szStandingRightSlashAttackBaseParameterName;

	enum
	{
		AVP_ALIEN_JUMPSLASH_VARIATION_LEFT = 0,
		AVP_ALIEN_JUMPSLASH_VARIATION_RIGHT = 1,
	};

private:
	typedef AVP_Vertebrate_MeleeCounter_System PARENT;
};

#endif //__AVP_ALIEN_MELEECOUNTER_SYSTEM_H__

/// End of file. ///
