///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_AlienQueen_AnimationHandler.cpp
// Author : David Wooldridge
// Date   : 06/11/2008
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "Actor/Vertebrates/AlienQueen/AVP_AlienQueen_AnimationHandler.h"

// Asura independent header-files...
#include "Animation/Asura_Animation.h"
#include "Animation/Asura_AnimationBlend.h"
#include "Animation/Asura_AnimationController_Animation.h"
#include "Actor/Vertebrates/AlienQueen/AVP_AlienQueen_Description.h"

// Asura platform header-files...

// Game independent header-files...
#include "Entity/PhysicalObject/Actor/Vertebrates/AlienQueen/AVP_ClientActor_AlienQueen.h"
#include "Entity/PhysicalObject/Actor/AVP_Actor_Messages.h"


AVP_AlienQueen_AnimationHandler::AVP_AlienQueen_AnimationHandler(AVP_ClientActor_AlienQueen& xOwner)
: PARENT(xOwner)
, m_xOwnerAlienQueen(xOwner)
{
}

void AVP_AlienQueen_AnimationHandler::Initialise( void )
{
	PARENT::Initialise();

	InitialiseIKSolver();
}

void AVP_AlienQueen_AnimationHandler::HandleAnimationRequest(const AVP_Message_Actor_RequestAnimation& xAnimRequestData)
{
#if 0
	if ( m_xCurrentAnimSequenceData.IsValid() || m_xPendingAnimSequenceData.IsValid() )
	{
		return;
	}

	u_int uAnimFlags = AAC_FLAG_ACTIVE | AAC_FLAG_DESTROY_WHEN_FINISHED | AAC_FLAG_ENSURE_UNIQUE_HASH | AACA_FLAG_CALCULATE_VELOCITY;

	// Work out if we should add the loop flag.
	if ( xAnimRequestData.m_eLoopStatus == AVP_Message_Actor_RequestAnimation::LOOP_ON )
	{
		uAnimFlags |= AACA_FLAG_LOOPED;
	}
	else if ( xAnimRequestData.m_eLoopStatus == AVP_Message_Actor_RequestAnimation::LOOP_STATUS_UNSET )
	{
		const bool bAnimLoopsByDefault =
			( 
			xAnimRequestData.m_uAnimation == AVP_AlienQueen_Description::ALIENQUEEN_ANIM_IDLE	||
			xAnimRequestData.m_uAnimation == AVP_AlienQueen_Description::ALIENQUEEN_ANIM_WALK	||
			xAnimRequestData.m_uAnimation == AVP_AlienQueen_Description::ALIENQUEEN_ANIM_RUN
			);

		uAnimFlags |= bAnimLoopsByDefault ? AACA_FLAG_LOOPED : 0;
	}

	u_int uHashId = AVP_FaceHugger_Description::GetAnimationHashID(static_cast<AVP_FaceHugger_Description::AnimationType>(xAnimRequestData.m_uAnimation));

	CreateRequestedAnimation(uHashId, uAnimFlags); 		
#endif
}

void AVP_AlienQueen_AnimationHandler::CreateRequestedAnimation( const Asura_Hash_ID uAnimHashID, const u_int uAnimFlags )
{
	Asura_AnimationBlend*					pxAnimBlend = new Asura_AnimationBlend_AnimatedTween(0.1f, 0.1f, false);
	m_uCurrentRequestedAnimControllerHash = PlayAnimation( uAnimFlags, AVP_AAPC_REQUEST_ANIM, uAnimHashID, pxAnimBlend );
}

void AVP_AlienQueen_AnimationHandler::InitialiseIKSolver()
{
	Asura_Assert( m_xIKSolver.GetFirstChain() == NULL, "IK Solver already initialised, clearing it..." );
	if ( m_xIKSolver.GetFirstChain() )
	{
		m_xIKSolver.Clear();
	}

	m_xTailIKChain.SetBonesArrays( AVP_AlienQueen_Description::GetTailChainArray(), NULL );
	m_xTailIKChain.SetDoJointConstraints();
	m_xTailIKChain.SetDoCollisionConstraints();
	m_xTailIKChain.SetBoneCollisionRadius( 0.05f );
	m_xIKSolver.AddChain( m_xTailIKChain );

	EnableIKSolver();
}
/// End of file. ///
