
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_Human_AIAnimStateMachine.cpp
// Author : Will Plano
// Date   : 25/02/2008
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "Actor/Vertebrates/Human/AVP_Human_AIAnimStateMachine.h"

#include "Rendering/Asura_Font_System.h"
#include "Network/Asura_Message.h"

#include "Actor/Vertebrates/Human/AVP_Human_AIAnimStateConnections.h"
#include "Actor/Vertebrates/Human/AVP_Human_AIAnimStateData.h"
#include "Core/AVP_Settings.h"
#include "Entity/PhysicalObject/Actor/AVP_ServerEntity_Actor.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Human/AVP_ClientActor_Human.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Human/AVP_Messages_Human.h"
#include "Inventory/AVP_Inventory_Group.h"
#include "Inventory/AVP_Inventory_HashIDs.h"
#include "Inventory/AVP_Inventory_Item.h"
#include "Inventory/AVP_Inventory_System.h"


static const u_int g_uAllCoverTypesValid		=	1 << AVP_Human_AIAnimStateMachine::COVER_TYPE_LOW
												|	1 << AVP_Human_AIAnimStateMachine::COVER_TYPE_MID
												|	1 << AVP_Human_AIAnimStateMachine::COVER_TYPE_HIGH;

static const u_int g_uLowAndMidCoverTypesValid	=	1 << AVP_Human_AIAnimStateMachine::COVER_TYPE_LOW
												|	1 << AVP_Human_AIAnimStateMachine::COVER_TYPE_MID;

static const char* g_aszStateString[ AVP_Human_AIAnimStateMachine::HUMAN_STATE_END - AVP_Human_AIAnimStateMachine::HUMAN_STATE_BEGIN ] = 
{
	"Crouch.Idle",						// HUMAN_STATE_CROUCH_IDLE
	"Crouch.Move",						// HUMAN_STATE_CROUCH_MOVE
	"Idle->Crouch.Idle",				// IDLE_TO_CROUCH_IDLE
	"Crouch.Idle->Idle",				// CROUCH_IDLE_TO_IDLE
	"Panic.Idle",						// PANIC_IDLE
	"Cover.Idle",						// HUMAN_STATE_COVER_IDLE
	"Cover.Popup",						// HUMAN_STATE_COVER_POPUP
	"Idle->Cover.Idle",					// IDLE_TO_COVER_IDLE
	"Cover.Idle->Idle",					// COVER_IDLE_TO_IDLE
	"Cover.Idle->Cover.PopUp",			// COVER_IDLE_TO_COVER_POPUP
	"Cover.PopUp->Cover.Idle",			// COVER_POPUP_TO_COVER_IDLE
	"Idle->Alert.Idle",					// HUMAN_STATE_IDLE_TO_ALERT_IDLE
	"Idle->Aggresive.Idle",				// HUMAN_STATE_IDLE_TO_AGGRESSIVE_IDLE
	"Alert.Idle->Aggressive.Idle",		// HUMAN_STATE_ALERT_IDLE_TO_AGGRESSIVE_IDLE
	"Aggressive.Idle->Alert.Idle",		// HUMAN_STATE_AGGRESSIVE_IDLE_TO_ALERT_IDLE
	"Alert.Idle",						// HUMAN_STATE_ALERT_IDLE
	"Aggressive.Idle",					// HUMAN_STATE_AGGRESSIVE_IDLE
	"Sitting.Idle",						// HUMAN_STATE_SITTING_IDLE
};

static const char* g_aszCover[ AVP_Human_AIAnimStateMachine::COVER_TYPE_END ] = 
{
	"LowCover~",
	"MidCover~",
	"HighCover~"
};

static bool g_abDirectionalState[ AVP_Human_AIAnimStateMachine::HUMAN_STATE_END - AVP_Human_AIAnimStateMachine::HUMAN_STATE_BEGIN ] = 
{
	false,	// HUMAN_STATE_CROUCH_IDLE
	true,	// HUMAN_STATE_CROUCH_MOVE
	false,	// IDLE_TO_CROUCH_IDLE
	false,	// CROUCH_IDLE_TO_IDLE
	false,	// PANIC_IDLE
	false,	// HUMAN_STATE_COVER_IDLE
	false,	// HUMAN_STATE_COVER_POPUP
	false,	// IDLE_TO_COVER_IDLE
	false,	// COVER_IDLE_TO_IDLE
	false,	// COVER_IDLE_TO_COVER_POPUP
	false,	// COVER_POPUP_TO_COVER_IDLE
	false,	// HUMAN_STATE_IDLE_TO_ALERT_IDLE
	false,	// HUMAN_STATE_IDLE_TO_AGGRESSIVE_IDLE
	false,	// HUMAN_STATE_ALERT_IDLE_TO_AGGRESSIVE_IDLE
	false,	// HUMAN_STATE_AGGRESSIVE_IDLE_TO_ALERT_IDLE
	false,	// HUMAN_STATE_ALERT_IDLE
	false,	// HUMAN_STATE_AGGRESSIVE_IDLE
	false,	// HUMAN_STATE_SITTING_IDLE
};

static u_int g_auCoverTypeFlags[ AVP_Human_AIAnimStateMachine::HUMAN_STATE_END - AVP_Human_AIAnimStateMachine::HUMAN_STATE_BEGIN ] = 
{
	0,								// HUMAN_STATE_CROUCH_IDLE
	0,								// HUMAN_STATE_CROUCH_MOVE
	0,								// IDLE_TO_CROUCH_IDLE
	0,								// CROUCH_IDLE_TO_IDLE
	0,								// PANIC_IDLE
	g_uAllCoverTypesValid,			// HUMAN_STATE_COVER_IDLE
	g_uLowAndMidCoverTypesValid,	// HUMAN_STATE_COVER_POPUP
	g_uAllCoverTypesValid,			// IDLE_TO_COVER_IDLE,
	g_uAllCoverTypesValid,			// COVER_IDLE_TO_IDLE,
	g_uLowAndMidCoverTypesValid,	// COVER_IDLE_TO_COVER_POPUP
	g_uLowAndMidCoverTypesValid,	// COVER_POPUP_TO_COVER_IDLE
	0,								// HUMAN_STATE_IDLE_TO_ALERT_IDLE
	0,								// HUMAN_STATE_IDLE_TO_AGGRESSIVE_IDLE
	0,								// HUMAN_STATE_ALERT_IDLE_TO_AGGRESSIVE_IDLE
	0,								// HUMAN_STATE_AGGRESSIVE_IDLE_TO_ALERT_IDLE
	0,								// HUMAN_STATE_ALERT_IDLE
	0,								// HUMAN_STATE_AGGRESSIVE_IDLE
	0,								// HUMAN_STATE_SITTING_IDLE
};

static bool g_abLoopedState[ AVP_Human_AIAnimStateMachine::HUMAN_STATE_END - AVP_Human_AIAnimStateMachine::HUMAN_STATE_BEGIN ] = 
{
	true,	// HUMAN_STATE_CROUCH_IDLE
	true,	// HUMAN_STATE_CROUCH_MOVE
	false,	// IDLE_TO_CROUCH_IDLE
	false,	// CROUCH_IDLE_TO_IDLE
	true,	// PANIC_IDLE
	true,	// HUMAN_STATE_COVER_IDLE
	true,	// HUMAN_STATE_COVER_POPUP
	false,	// IDLE_TO_COVER_IDLEa
	false,	// COVER_IDLE_TO_IDLE
	false,	// COVER_IDLE_TO_COVER_POPUP
	false,	// COVER_POPUP_TO_COVER_IDLE
	false,	// HUMAN_STATE_IDLE_TO_ALERT_IDLE
	false,	// HUMAN_STATE_IDLE_TO_AGGRESSIVE_IDLE
	false,	// HUMAN_STATE_ALERT_IDLE_TO_AGGRESSIVE_IDLE
	false,	// HUMAN_STATE_AGGRESSIVE_IDLE_TO_ALERT_IDLE
	true,	// HUMAN_STATE_ALERT_IDLE
	true,	// HUMAN_STATE_AGGRESSIVE_IDLE
	true,	// HUMAN_STATE_SITTING_IDLE
};

void AVP_Human_AIAnimStateMachine::Initialise()
{
	// We load the anim data for all our weapons here to avoid long loading sequences in game.
	const AVP_Inventory_System* pxInvSys = m_xServerActor.GetInventorySystem();
	const AVP_Inventory_Group* pxWeaponGroup = pxInvSys != NULL ? pxInvSys->GetGroup( uAVP_INVENTORY_GROUPHASH_WEAPON ) : NULL;
	if ( pxWeaponGroup != NULL )
	{
		Asura_Hash_ID uWeaponItr( pxWeaponGroup->FindLowestPriorityItem() );
		for ( ; uWeaponItr != ASURA_HASH_ID_UNSET; uWeaponItr = pxWeaponGroup->FindNextHighestPriorityItem( uWeaponItr ) )
		{
			const AVP_Inventory_Item* pxItem = pxWeaponGroup->GetItem( uWeaponItr );
			if ( pxItem )
			{
				AVP_Human_AIAnimStateData::InitialiseWeaponData( m_xServerActor.GetAnimStateBlueprintType(), pxItem->GetBlueprintHash() );
			}
		}
	}

	PARENT::Initialise();
}

const AVP_Actor_AnimStateData* AVP_Human_AIAnimStateMachine::GetStateAnimData( const u_int uState ) const
{
	const AVP_ServerActor_Human& xHuman = static_cast< const AVP_ServerActor_Human& >( m_xServerActor );
	const AVP_Inventory_System* pxInvSys = m_xServerActor.GetInventorySystem();
	const AVP_Inventory_Group* pxWeaponGroup = pxInvSys != NULL ? pxInvSys->GetGroup( uAVP_INVENTORY_GROUPHASH_WEAPON ) : NULL;
	Asura_Assert( pxWeaponGroup != NULL, "Unable to get wepaon group" );
	if ( pxWeaponGroup != NULL )
	{
		const AVP_Inventory_Item* pxItem = pxWeaponGroup->GetCurrentInventoryItem();
		if ( pxItem )
		{
			return AVP_Human_AIAnimStateData::GetAnimStateData( m_xServerActor.GetAnimStateBlueprintType(), pxItem->GetBlueprintHash(), uState, xHuman.IsAiming(), xHuman.GetCoverType() );
		}
		else
		{
			return NULL;
		}
	}
	else
	{
		return NULL;
	}
}

void AVP_Human_AIAnimStateMachine::BeginState( const u_int uState )
{
	AVP_ServerActor_Human& xHuman = static_cast< AVP_ServerActor_Human& >( m_xServerActor );
	
	if ( uState == HUMAN_STATE_CROUCH_MOVE )
	{
		xHuman.SetMoving( true );
		xHuman.SetCrouching( true );
	}
	else if ( uState == HUMAN_STATE_CROUCH_IDLE )
	{
		xHuman.SetCrouching( true );
	}

	PARENT::BeginState( uState );
}

void AVP_Human_AIAnimStateMachine::UpdateState( const u_int uState )
{
	PARENT::UpdateState( uState );
	AVP_ClientActor_Human*	pxClient = AVP_ClientActor_Human::FindEntity( m_xServerActor.GetGuid() );
	if ( pxClient )
	{
		pxClient->SetCoverType( static_cast< AVP_ServerActor_Human& >( m_xServerActor ).GetCoverType() );
	}
}

bool AVP_Human_AIAnimStateMachine::EndState( const u_int uState )
{
	AVP_ServerActor_Human& xHuman = static_cast< AVP_ServerActor_Human& >( m_xServerActor );
	
	if ( uState == HUMAN_STATE_CROUCH_MOVE )
	{
		xHuman.SetMoving( false );
		xHuman.SetCrouching( false );
	}
	else if ( uState == HUMAN_STATE_CROUCH_IDLE )
	{
		xHuman.SetCrouching( false );
	}

	return PARENT::EndState( uState );
}

#ifndef ASURA_NO_DEBUG_OUTPUT
void AVP_Human_AIAnimStateMachine::RenderDebugInfo()
{
	const AVP_ServerActor_Human& xHuman = static_cast< const AVP_ServerActor_Human& >( m_xServerActor );

	PARENT::RenderDebugInfo();

	Asura_Vector_3 xDebugPos( m_xServerActor.GetPosition() );
	if ( Asura_Camera::ProjectWorldCoordsToScreenSpace( xDebugPos, true ) && xDebugPos.z > 0.0f)
	{
		xDebugPos.y -= 20.0f;

		Asura_Font_System::Print
		(
			xDebugPos.GetX(),
			xDebugPos.GetY(),
			0.6f,
			0xffffffff,
			ASURA_TEXTFORMAT_CENTREJUSTIFIED, 
			( xHuman.IsAiming() ? "Aiming" : "Neutral" )
		);

		const AVP_Inventory_System* pxInvSys = m_xServerActor.GetInventorySystem();
		const AVP_Inventory_Group* pxWeaponGroup = pxInvSys != NULL ? pxInvSys->GetGroup( uAVP_INVENTORY_GROUPHASH_WEAPON ) : NULL;
		Asura_Assert( pxWeaponGroup != NULL, "Unable to get wepaon group" );
		if ( pxWeaponGroup != NULL )
		{
			xDebugPos.y -= 10.0f;

			Asura_Font_System::Print
			(
				xDebugPos.GetX(),
				xDebugPos.GetY(),
				0.6f,
				0xffffffff,
				ASURA_TEXTFORMAT_CENTREJUSTIFIED | ASURA_TEXTFORMAT_FORMATSTRING, 
				"Weapon: 0x%08x", pxWeaponGroup->GetCurrentItem()
			);
		}
	}
}
#endif

const AVP_Actor_AIAnimStateConnectionData* AVP_Human_AIAnimStateMachine::GetConnectionData( const u_int uFromState, const u_int uToState ) const
{
	return AVP_Human_AIAnimStateConnections::GetConnectionData( uFromState, uToState );
}

const char* AVP_Human_AIAnimStateMachine::GetStateDescriptionStatic( const u_int uState )
{
	if ( uState < HUMAN_STATE_BEGIN )
	{
		return PARENT::GetStateDescriptionStatic( uState );
	}

	Asura_Assert( uState < HUMAN_STATE_END, "Invalid state" );
	return g_aszStateString[ uState - HUMAN_STATE_BEGIN ];
}

bool AVP_Human_AIAnimStateMachine::IsDirectionalState( const u_int uState )
{
	if ( uState < HUMAN_STATE_BEGIN )
	{
		return PARENT::IsDirectionalState( uState );
	}

	Asura_Assert( uState < HUMAN_STATE_END, "Invalid state" );
	return g_abDirectionalState[ uState - HUMAN_STATE_BEGIN ];
}

const char* AVP_Human_AIAnimStateMachine::GetCoverTypeDescription( const u_int uCoverType )
{
	Asura_Assert( uCoverType < COVER_TYPE_END, "Invalid cover type" );
	return g_aszCover[ uCoverType ];
}

bool AVP_Human_AIAnimStateMachine::HasCoverTypes( const u_int uState )
{
	if ( uState < HUMAN_STATE_BEGIN )
	{
		return false; // no cover types for vertebrate or actor states
	}

	Asura_Assert( uState < HUMAN_STATE_END, "Invalid State" );
	return g_auCoverTypeFlags[ uState - HUMAN_STATE_BEGIN ] != 0;
}

bool AVP_Human_AIAnimStateMachine::IsCoverTypeValid( const u_int uCoverType, const u_int uState )
{
	Asura_Assert( uCoverType < COVER_TYPE_END, "Invalid cover type" );
	if ( uState < HUMAN_STATE_BEGIN )
	{
		return false; // no cover types for vertebrate or actor states
	}

	Asura_Assert( uState < HUMAN_STATE_END, "Invalid state" );
	return ( g_auCoverTypeFlags[ uState - HUMAN_STATE_BEGIN ] & ( 1 << uCoverType ) ) != 0;
}

bool AVP_Human_AIAnimStateMachine::IsLoopedStateStatic( const u_int uState )
{
	if ( uState < HUMAN_STATE_BEGIN )
	{
		return PARENT::IsLoopedStateStatic( uState );
	}

	Asura_Assert( uState < HUMAN_STATE_END, "Invalid state" );
	return g_abLoopedState[ uState - HUMAN_STATE_BEGIN ];
}

void AVP_Human_AIAnimStateMachine::RequestCoverPopup()
{
	const AVP_ServerActor_Human& xHuman = static_cast< const AVP_ServerActor_Human& >( m_xServerActor );
	bool bCoverValid = IsCoverTypeValid( xHuman.GetCoverType(), HUMAN_STATE_COVER_POPUP );
	Asura_Assertf( bCoverValid, ( "PopUp not valid for cover type %s", GetCoverTypeDescription( xHuman.GetCoverType() ) ) );
	if ( bCoverValid )
	{
		RequestState( HUMAN_STATE_COVER_POPUP );
	}
}
/// End of file. ///
