
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_Human_AIAnimStateMachine.h
// Author : Will Plano
// Date   : 27/02/2008
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __AVP_HUMAN_AIANIMSTATEMACHINE_H__
#define __AVP_HUMAN_AIANIMSTATEMACHINE_H__

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Actor/Vertebrates/AVP_Vertebrate_AIAnimStateMachine.h"

class AVP_Human_AIAnimStateMachine : public AVP_Vertebrate_AIAnimStateMachine
{
public:
	// Public enumerated types...

	enum HumanAnimState
	{
		HUMAN_STATE_BEGIN = VERTEBRATE_STATE_END,	
		// crouching states
		HUMAN_STATE_CROUCH_IDLE = HUMAN_STATE_BEGIN,		// 13
		HUMAN_STATE_CROUCH_MOVE,							// 14

		// transition states
		HUMAN_STATE_IDLE_TO_CROUCH_IDLE,					// 15
		HUMAN_STATE_CROUCH_IDLE_TO_IDLE,					// 16

		// panic states
		HUMAN_STATE_PANIC_IDLE,								// 17

		// cover states
		HUMAN_STATE_COVER_IDLE,								// 18
		HUMAN_STATE_COVER_POPUP,							// 19
		// transition states
		HUMAN_STATE_IDLE_TO_COVER_IDLE,						// 20
		HUMAN_STATE_COVER_IDLE_TO_IDLE,						// 21
		HUMAN_STATE_COVER_IDLE_TO_COVER_POPUP,				// 22
		HUMAN_STATE_COVER_POPUP_TO_COVER_IDLE,				// 23

		HUMAN_STATE_IDLE_TO_ALERT_IDLE,						// 24
		HUMAN_STATE_IDLE_TO_AGGRESSIVE_IDLE,				// 25
		HUMAN_STATE_ALERT_IDLE_TO_AGGRESSIVE_IDLE,			// 26

		HUMAN_STATE_AGGRESSIVE_IDLE_TO_ALERT_IDLE,			// 27

		// ALERT SATES
		HUMAN_STATE_ALERT_IDLE,								// 28
		HUMAN_STATE_AGGRESSIVE_IDLE,						// 29

		// SITTING STATES
		HUMAN_STATE_SITTING_IDLE,							// 30

		// END
		HUMAN_STATE_END										// 31
	};

	enum AimingState
	{
		AIMINGSTATE_BEGIN,

		AIMING = AIMINGSTATE_BEGIN,
		NEUTRAL,

		AIMINGSTATE_END
	};

	enum CoverType
	{
		COVER_TYPE_BEGIN,

		COVER_TYPE_LOW = COVER_TYPE_BEGIN,
		COVER_TYPE_MID,
		COVER_TYPE_HIGH,

		COVER_TYPE_END
	};

	// Public functions...
	AVP_Human_AIAnimStateMachine( AVP_ServerEntity_Actor& xServer ) : PARENT( xServer ), m_bOldAiming( false ) {}
	virtual ~AVP_Human_AIAnimStateMachine() {}

	virtual void Initialise();

	// Sitting Requests
	void	RequestSittingIdle() { RequestState( HUMAN_STATE_SITTING_IDLE ); }
	bool	IsSittingIdle() const { return IsStateSittingIdle( GetCurrentState() ); }
	bool	IsStateSittingIdle( const u_int uState ) const { return HUMAN_STATE_SITTING_IDLE == uState; }

	// Alert Requests
	void	RequestAlertIdle() { RequestState( HUMAN_STATE_ALERT_IDLE ); }
	bool	IsAlertIdle() const { return IsStateAlertIdle( GetCurrentState() ); }
	bool	IsStateAlertIdle( const u_int uState ) const { return HUMAN_STATE_ALERT_IDLE == uState || HUMAN_STATE_IDLE_TO_ALERT_IDLE == uState || HUMAN_STATE_AGGRESSIVE_IDLE_TO_ALERT_IDLE == uState; }

	// Alert Requests
	void	RequestAggressiveIdle() { RequestState( HUMAN_STATE_AGGRESSIVE_IDLE ); }
	bool	IsAggressiveIdle() const { return IsStateAlertIdle( GetCurrentState() ); }
	bool	IsStateAggressiveIdle( const u_int uState ) const { return HUMAN_STATE_AGGRESSIVE_IDLE == uState || HUMAN_STATE_IDLE_TO_AGGRESSIVE_IDLE == uState || HUMAN_STATE_ALERT_IDLE_TO_AGGRESSIVE_IDLE == uState; }

	// Crouch Requests
	void	RequestCrouchIdle() { RequestState( HUMAN_STATE_CROUCH_IDLE ); }
	bool	IsCrouchIdle() const { return IsStateCrouchIdle( GetCurrentState() ); }
	bool	IsStateCrouchIdle( const u_int uState ) const { return HUMAN_STATE_CROUCH_IDLE == uState; }

	void	RequestCrouchMove() { RequestState( HUMAN_STATE_CROUCH_MOVE ); }
	bool	IsCrouchMove() const { return IsStateCrouchMove( GetCurrentState() ); }
	bool	IsStateCrouchMove( const u_int uState ) const { return HUMAN_STATE_CROUCH_MOVE == uState; }

	// Panic Requests
	void	RequestPanicIdle() { RequestState( HUMAN_STATE_PANIC_IDLE ); }
	bool	IsPanicIdle() const { return IsStatePanicIdle( GetCurrentState() ); }
	bool	IsStatePanicIdle( const u_int uState ) const { return HUMAN_STATE_PANIC_IDLE == uState; }

	// Cover Requests
	void	RequestCoverIdle()	{ RequestState( HUMAN_STATE_COVER_IDLE); }
	bool	IsCoverIdle() const	{ return IsStateCoverIdle( GetCurrentState() ); }
	bool	IsStateCoverIdle( const u_int uState ) const { return HUMAN_STATE_COVER_IDLE == uState; }

	void	RequestCoverPopup();
	bool	IsCoverPopup() const { return IsStateCoverPopup( GetCurrentState() ); }
	bool	IsStateCoverPopup( const u_int uState ) const { return HUMAN_STATE_COVER_POPUP == uState; }

	virtual const AVP_Actor_AIAnimStateConnectionData* GetConnectionData( const u_int uFromState, const u_int uToState ) const;

	virtual const char* GetStateDescription( const u_int uState ) const { return GetStateDescriptionStatic( uState ); }
	static const char* GetStateDescriptionStatic( const u_int uState );
	
	static bool IsDirectionalState( const u_int uState );

	static const char* GetCoverTypeDescription( const u_int uCoverType );

	static bool HasCoverTypes( const u_int uState );
	static bool IsCoverTypeValid( const u_int uCoverType, const u_int uState );

	virtual bool IsLoopedState( const u_int uState ) const { return IsLoopedStateStatic( uState ); }
	static bool IsLoopedStateStatic( const u_int uState );

protected:
	virtual void BeginState		( const u_int uState );
	virtual void UpdateState	( const u_int uState );
	virtual bool EndState		( const u_int uState );

	virtual u_int GetNumStates() const { return HUMAN_STATE_END; }
	virtual const AVP_Actor_AnimStateData* GetStateAnimData( const u_int uState ) const;

#ifndef ASURA_NO_DEBUG_OUTPUT
	virtual void RenderDebugInfo();
#endif

private:
	typedef AVP_Vertebrate_AIAnimStateMachine PARENT;

	bool m_bOldAiming;

	AVP_Human_AIAnimStateMachine();
	AVP_Human_AIAnimStateMachine(const AVP_Human_AIAnimStateMachine& xOther);
	AVP_Human_AIAnimStateMachine&	operator=(const AVP_Human_AIAnimStateMachine& xOther);

}; // End of class 'AVP_Human_AIAnimStateMachine'.

#endif // ifndef __AVP_HUMAN_AIANIMSTATEMACHINE_H__

/// End of file. ///
