
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_Human_Description.cpp
// Author : Andy Tate
// Date   : 9-Nov-2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "Actor/Vertebrates/Human/AVP_Human_Description.h"

// Asura independent header-files...
#include "Blueprint/Asura_Blueprint_System.h"
#include "Blueprint/Asura_Blueprint.h"
#ifdef ASURA_TOOLS
	#include "Blueprint/Tools_Mode_Blueprint.h"
#endif

// Game header files
#include "Blueprint/AVP_Blueprint_HashIDs.h"
#include "Rendering/Asura_Project_Skin_Elements.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Human/Android/AVP_ServerActor_Android.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Human/Civilian/AVP_ServerActor_Civilian.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Human/Marine/AVP_ServerActor_Marine.h"

// Tools header-files...
#if defined(ASURA_TOOLS)
#include "Particles/Tools_Mode_PFX.h"
#endif	// defined(ASURA_TOOLS)

// Marker-point hash-IDs...
const Asura_Hash_ID AVP_Human_Description::s_auMarkerPointHashIDs[NUM_MARKER_POINT_IDS] =
{
	0xE70A498D,	//Asura_GetHashID("Bip01 Spine"),		//BLOOD_SPINE
	0x077181F5,	//Asura_GetHashID("Bip01 Neck"),		//BLOOD_NECK
	0x88CEF77E,	//Asura_GetHashID("Bip01 R UpperArm"),	//BLOOD_R_UPPERARM
	0xB7C7029C, //Asura_GetHashID("Bip01 R ForeArm"),	//BLOOD_R_FOREARM
	0xCF1650C4, //Asura_GetHashID("Bip01 L UpperArm"),	//BLOOD_L_UPPERARM
	0x3E2C6896, //Asura_GetHashID("Bip01 L ForeArm"),	//BLOOD_L_FOREARM
	0x90CC9192,	//Asura_GetHashID("Bip01 R Thigh"),		//BLOOD_R_THIGH
	0xF41FCD5C,	//Asura_GetHashID("Bip01 R Calf"),		//BLOOD_R_CALF
	0x53672E0C, //Asura_GetHashID("Bip01 L Thigh"),		//BLOOD_L_THIGH
	0xE9E2B9A2, //Asura_GetHashID("Bip01 L Calf"),		//BLOOD_L_CALF

	0x002FBC62, //Asura_GetHashID("eyes"),			//EYES_MARKER_POINT_HASH,
	0x667604F6, //Asura_GetHashID("lefthand"),		//LEFT_HAND_MARKER_POINT_HASH,
	0xA282148B, //Asura_GetHashID("righthand"),		//RIGHT_HAND_MARKER_POINT_HASH,
	0x06233516, //Asura_GetHashID("light"),			//TORCH_MARKER_POINT_HASH,
	0x3577B49A,	//Asura_GetHashID("armour_front"),	//ARMOUR_FRONT,
	0x544C1C56,	//Asura_GetHashID("armour_back"),	//ARMOUR_BACK,
	0xB73A046D,	//Asura_GetHashID("helmet"),		//HELMET,
	0x00363450, //Asura_GetHashID("Tail");			//GRENADE_TAIL
};

const int AVP_Human_Description::s_aiFirstPersonFootBoneID[ AVP_ClientEntity_Actor::MAX_NUM_FEET ] = 
{
	FP_BIP01_L_FOOT,
	FP_BIP01_R_FOOT,
	-1,
	-1
};

const int AVP_Human_Description::s_aiThirdPersonFootBoneID[ AVP_ClientEntity_Actor::MAX_NUM_FEET ] = 
{
	TP_BIP01_L_FOOT,
	TP_BIP01_R_FOOT,
	-1,
	-1
};

// IK
const int AVP_Human_Description::s_aiFirstPersonLeftArmChainBonesArray[ uAVP_HUMAN_NUM_IK_BONES_PER_LIMB + 1  ] =
{
	FP_BIP01_L_UPPERARM2,
	FP_BIP01_L_FOREARM,
	FP_BIP01_L_HAND,
	-1
};

const int AVP_Human_Description::s_aiFirstPersonLeftLegChainBonesArray[ uAVP_HUMAN_NUM_IK_BONES_PER_LIMB + 1  ] =
{
	FP_BIP01_L_THIGH,
	FP_BIP01_L_CALF,
	FP_BIP01_L_FOOT,
	-1
};

const int AVP_Human_Description::s_aiFirstPersonRightLegChainBonesArray[ uAVP_HUMAN_NUM_IK_BONES_PER_LIMB + 1  ] =
{
	FP_BIP01_R_THIGH,
	FP_BIP01_R_CALF,
	FP_BIP01_R_FOOT,
	-1
};

const int AVP_Human_Description::s_aiThirdPersonLeftArmChainBonesArray[ uAVP_HUMAN_NUM_IK_BONES_PER_LIMB + 1  ] =
{
	TP_BIP01_L_UPPERARM,
	TP_BIP01_L_FOREARM,
	TP_BIP01_L_HAND,
	-1
};

const int AVP_Human_Description::s_aiThirdPersonLeftLegChainBonesArray[ uAVP_HUMAN_NUM_IK_BONES_PER_LIMB + 1  ] =
{
	TP_BIP01_L_THIGH,
	TP_BIP01_L_CALF,
	TP_BIP01_L_FOOT,
	-1
};

const int AVP_Human_Description::s_aiThirdPersonRightLegChainBonesArray[ uAVP_HUMAN_NUM_IK_BONES_PER_LIMB + 1  ] =
{
	TP_BIP01_R_THIGH,
	TP_BIP01_R_CALF,
	TP_BIP01_R_FOOT,
	-1
};

const int AVP_Human_Description::s_aiFirstPersonLeftArmChainTwistBonesArray[ uAVP_HUMAN_NUM_IK_BONES_PER_LIMB + 1  ] =
{
	FP_BIP01_LUPARMTWIST,
	FP_BIP01_L_FORETWIST,
	-1,
	-1
};

const int AVP_Human_Description::s_aiThirdPersonLeftArmChainTwistBonesArray[ uAVP_HUMAN_NUM_IK_BONES_PER_LIMB + 1  ] =
{
	-1,
	TP_BIP01_L_FORETWIST,
	-1,
	-1
};

////////////////////////////////////////////////////////////////////////////////
// Private functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Protected functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Public functions...
////////////////////////////////////////////////////////////////////////////////

u_int AVP_Human_Description::GetElementFromBoneID(int iBoneID)
{
	switch(iBoneID)
	{
		case TP_BIP01:
		case TP_BIP01_PELVIS:
		case TP_BIP01_SPINE:
		case TP_BIP01_SPINE1:
		case TP_BIP01_SPINE2:
		case TP_TURRET:
		case TP_KNIFE:
		case TP_REARPLATE:
		case TP_SHELLS:
		case TP_FRONT_FLAP:
		case TP_LIGHTPITCH:
		case TP_LIGHTYAW:
		{
			return(AVP_SET_BODY);
		}

		case TP_BROWIN_L:
		case TP_BROWIN_R:
		case TP_BROWOUT_L:
		case TP_BROWOUT_R:
		case TP_CHEEK_BL:
		case TP_CHEEK_BR:
		case TP_CHEEK_LEFT:
		case TP_CHEEK_RIGHT:
		case TP_EYE_LEFT:
		case TP_EYE_RIGHT:
		case TP_JAW:
		case TP_MOUTH_B:
		case TP_MOUTH_BL:
		case TP_MOUTH_BR:
		case TP_LOWEREYELID_LEFT:
		case TP_LOWEREYELID_RIGHT:
		case TP_MOUTH_L:
		case TP_MOUTH_R:
		case TP_MOUTH_T:
		case TP_MOUTH_TL:
		case TP_MOUTH_TR:
		case TP_NOSE_L:
		case TP_NOSE_R:
		case TP_UPPEREYELID_LEFT:
		case TP_UPPEREYELID_RIGHT:
		case TP_BIP01_NECK:
		case TP_BIP01_HEAD:
		{
			return(AVP_SET_HEAD);
		}

		case TP_BIP01_L_CLAVICLE:
		case TP_BIP01_L_UPPERARM:
		case TP_L_SHOULDER:
		case TP_PAD_LEFT:
		{
			return(AVP_SET_LEFT_ARM);
		}

		case TP_BIP01_L_FOREARM:
		case TP_BIP01_L_HAND:
		case TP_BIP01_L_FINGER0:
		case TP_BIP01_L_FINGER01:
		case TP_BIP01_L_FINGER1:
		case TP_BIP01_L_FINGER11:
		case TP_BIP01_L_FINGER2:
		case TP_BIP01_L_FINGER21:
		case TP_BIP01_L_FINGER3:
		case TP_BIP01_L_FINGER31:
		case TP_BIP01_L_FINGER4:
		case TP_BIP01_L_FINGER41:
		case TP_BIP01_L_FORETWIST:
		{
			return(AVP_SET_LEFT_ELBOW);
		}

		case TP_BIP01_R_CLAVICLE:
		case TP_BIP01_R_UPPERARM:
		case TP_R_SHOULDER:
		case TP_PAD_RIGHT:
		{
			return(AVP_SET_RIGHT_ARM);
		}

		case TP_BIP01_R_FOREARM:
		case TP_BIP01_R_HAND:
		case TP_BIP01_R_FINGER0:
		case TP_BIP01_R_FINGER01:
		case TP_BIP01_R_FINGER1:
		case TP_BIP01_R_FINGER11:
		case TP_BIP01_R_FINGER2:
		case TP_BIP01_R_FINGER21:
		case TP_BIP01_R_FINGER3:
		case TP_BIP01_R_FINGER31:
		case TP_BIP01_R_FINGER4:
		case TP_BIP01_R_FINGER41:
		case TP_BIP01_R_FORETWIST:
		{
			return(AVP_SET_RIGHT_ELBOW);
		}

		case TP_BIP01_L_CALF:
		case TP_BIP01_L_THIGH:
		{
			return(AVP_SET_LEFT_LEG);
		}

		case TP_BIP01_L_FOOT:
		case TP_BIP01_L_TOE0:
		{
			return(AVP_SET_LEFT_KNEE);
		}

		case TP_BIP01_R_CALF:
		case TP_BIP01_R_THIGH:
		{
			return(AVP_SET_RIGHT_LEG);
		}

		case TP_BIP01_R_FOOT:
		case TP_BIP01_R_TOE0:
		{
			return(AVP_SET_RIGHT_KNEE);
		}

		default:
		{
			return(NUM_AVP_SKIN_ELEMENT_TYPES);
		}
	}	// End of switch.
}

// Since bones->elements is a many->one mapping, this function will return one of the bones associated with the given element, but not necessarily the only one.
int AVP_Human_Description::GetAnyBoneIDFromElement(u_int uElement)
{
	switch (uElement)
	{
	case AVP_SET_BODY:
		return TP_BIP01_SPINE2;

	case AVP_SET_HEAD:
	case AVP_SET_HEAD_AFTER_HEADBITE:
		return TP_BIP01_HEAD;

	case AVP_SET_LEFT_ARM:
		return TP_BIP01_L_UPPERARM;

	case AVP_SET_LEFT_ELBOW:
		return TP_BIP01_L_FOREARM;

	case AVP_SET_RIGHT_ARM:
		return TP_BIP01_R_UPPERARM;

	case AVP_SET_RIGHT_ELBOW:
		return TP_BIP01_R_FOREARM;

	case AVP_SET_LEFT_LEG:
		return TP_BIP01_L_THIGH;

	case AVP_SET_LEFT_KNEE:
		return TP_BIP01_L_FOOT;

	case AVP_SET_RIGHT_LEG:
		return TP_BIP01_R_THIGH;

	case AVP_SET_RIGHT_KNEE:
		return TP_BIP01_R_FOOT;

	default:
		return -1;
	}
}

#if defined(ASURA_TOOLS)

//------------------------------------------------------------------------------
// Function     : AVP_Human_Description::AddMarineFilesToExport
// Author       : Gareth Thomas
// Date         : 18 / 02 / 2008
//------------------------------------------------------------------------------
void AVP_Human_Description::AddMarineFilesToExport(const bool bFirstPerson, const Asura_Hash_ID uBlueprintHash, List<Asura_String>& xNameList)
{
	// we shouldn't really need this, the blueprint should link everything we use in
	// Add all of the animations in the appropriate directory...
	//if(bFirstPerson)
	//{
	//	Tools_Level_Export::AddAnimDirToExportList(xNameList, "Characters\\FirstPerson\\Marine", true);
	//}
	//else
	//{
	//	Tools_Level_Export::AddAnimDirToExportList(xNameList, "Characters\\ThirdPerson\\Marine", true);
	//}

	// Export the relevant blueprints
	if (uBlueprintHash != ASURA_HASH_ID_UNSET)
	{
		Tools_Mode_Blueprint::AddBlueprintFilesToExportList(xNameList, uAVP_BLUEPRINT_TYPE_ACTORS, uBlueprintHash, false);
		if ( bFirstPerson )
		{
			Tools_Mode_Blueprint::AddBlueprintFilesToExportList(xNameList, uAVP_BLUEPRINT_TYPE_PLAYERS, uBlueprintHash, false);
		}
	}

	AddDialogueFromBlueprint(uBlueprintHash);
}

//------------------------------------------------------------------------------
// Function     : AVP_Human_Description::AddAndroidFilesToExport
// Author       : Gareth Thomas
// Date         : 18 / 02 / 2008
//------------------------------------------------------------------------------
void AVP_Human_Description::AddAndroidFilesToExport(const bool bFirstPerson, const Asura_Hash_ID uBlueprintHash, List<Asura_String>& xNameList)
{
	// Add all of the animations in the appropriate directory...
	Asura_Assert(!bFirstPerson, "First-person android's cannot yet be exported.");
	//Tools_Level_Export::AddAnimDirToExportList(xNameList, "Characters\\ThirdPerson\\Marine", true);

	// Export the relevant blueprints
	if (uBlueprintHash != ASURA_HASH_ID_UNSET)
	{
		Tools_Mode_Blueprint::AddBlueprintFilesToExportList(xNameList, 0x766161d2, uBlueprintHash, false); //Asura_GetHashID("Actor Initialisation");
	}

	AddDialogueFromBlueprint(uBlueprintHash);
}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Human_Description::AddFilesToExport
// Author       : Gareth Thomas
// Date         : 18 / 02 / 2008
////////////////////////////////////////////////////////////////////////////////
void AVP_Human_Description::AddCivilianFilesToExport(const bool bFirstPerson, const Asura_Hash_ID uBlueprintHash, List<Asura_String>& xNameList)
{
	// Add all of the animations in the appropriate directory...
	Asura_Assert(!bFirstPerson, "First-person civilian's cannot yet be exported.");
	//Tools_Level_Export::AddAnimDirToExportList(xNameList, "Characters\\ThirdPerson\\Industrial_Worker", true);

	// Export the relevant blueprints
	if (uBlueprintHash != ASURA_HASH_ID_UNSET)
	{
		Tools_Mode_Blueprint::AddBlueprintFilesToExportList(xNameList, 0x766161d2, uBlueprintHash, false); //Asura_GetHashID("Actor Initialisation");
	}

	AddDialogueFromBlueprint(uBlueprintHash);
}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Human_Description::AddDialogueFromBlueprint
// Author       : Gareth Thomas
// Date         : 18 / 02 / 2008
////////////////////////////////////////////////////////////////////////////////
void AVP_Human_Description::AddDialogueFromBlueprint(const Asura_Hash_ID uBlueprintHash)
{
	Asura_Blueprint* pxActorBlueprint = Asura_Blueprint_System::GetBlueprintByHashID(0x766161d2, uBlueprintHash); //Asura_GetHashID("Actor Initialisation");

	// Add dialog from the dialogue blueprint
	if (pxActorBlueprint)
	{
		Asura_Value xBPValue;

		if (pxActorBlueprint->GetParameterValueByHashID(0x661d3a4b, xBPValue))	//Asura_GetHashID("Dialogue Blueprint");
		{
			Asura_Hash_ID uDialogueBlueprintHashID = xBPValue.GetHashIDValue();

			if (uDialogueBlueprintHashID != ASURA_HASH_ID_UNSET)
			{
				Asura_Blueprint* pxBlueprint = Asura_Blueprint_System::GetBlueprintByHashID(0xf20fd9d8, uDialogueBlueprintHashID); //Asura_GetHashID("Dialogue");
				if ( pxBlueprint )
				{
					// Add android dialogue to the export list!
					for (int i = 0; i < pxBlueprint->GetNumParameters(); ++i)
					{
						Asura_Value xValue;

						if (pxBlueprint->GetParameterValueByIndex(i, xValue))
						{
							Tools_Level_Export::AddDialogueEventToExportList(xValue.GetHashIDValue());
						}
					}
				}
			}
		}
	}
}

#endif	// defined(ASURA_TOOLS)

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Human_Description::CanBeImpaled
// Author       : Andy Tate
// Date         : 31-Jan-2007
////////////////////////////////////////////////////////////////////////////////

bool AVP_Human_Description::CanBeImpaled (const u_int uBoneID) const
{
	Asura_Assert(uBoneID < MAX_THIRD_PERSON_BONES_IDS, "Unexpected bone ID.");

	switch(uBoneID)
	{
		case TP_BIP01:
		case TP_BIP01_PELVIS:
		case TP_TURRET:
		case TP_BIP01_SPINE:
		case TP_BIP01_SPINE1:
		case TP_BIP01_SPINE2:
		{
			return(true);
		}

		default:
		{
			return(false);
		}
	}		// End of switch.
}

/// End of file. ///
