
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_Human_HUDHandler.cpp
// Author : Chris Wallace
// Date   : 8-Jun-2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "AVP_Human_HUDHandler.h"

// Asura header-files...
#include "Animation/Asura_Animation.h"
#include "Blueprint/Asura_Blueprint_System.h"
#include "Core/Asura_HashedLocalisedText.h"
#include "Core/Asura_TiledTextureManagement.h"
#include "Core/Asura_Timers.h"
#include "FSFX/Asura_FSFX_System.h"
#include "Interface/Asura_CommandConsole.h"
#include "Network/Asura_Message.h"
#include "Rendering/Asura_Font_System.h"

// Game header-files...
#include "Actor/Vertebrates/AVP_Vertebrate_CameraHandler.h"
#include "Actor/Vertebrates/Human/AVP_Human_Focus.h"
#include "Actor/Vertebrates/Human/AVP_Human_JumpHandler.h"
#include "Actor/Vertebrates/Human/AVP_Human_MotionTracker.h"
#include "Blueprint/AVP_Blueprint_HashIDs.h"
#include "Core/AVP_Game_Core.h"
#include "Core/AVP_Settings.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Human/AVP_ClientActor_Human.h"
#include "Entity/PlayerInfo/AVP_ClientEntity_PlayerInfo.h"
#include "Inventory/AVP_AmmoStore.h"
#include "Inventory/AVP_Inventory_HashIDs.h"
#include "Inventory/AVP_Inventory_Item.h"
#include "Inventory/AVP_InventoryItem_SubType.h"
#include "Inventory/AVP_Inventory_System.h"
#include "Inventory/Items/Weapons/AVP_InventoryWeapon_RangedWeapon.h"
#include "Inventory/Items/Weapons/AVP_InventoryWeapon_SmartGun.h"
#include "Inventory/Items/Weapons/AVP_InventoryWeapon_ScopedRifle.h"
#include "Rendering/AVP_Overlay_Render.h"
////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////

bool AVP_Human_HUDHandler::s_bGotStaticBlueprintValues = false;

int AVP_Human_HUDHandler::s_iGrenadeAmmoCounter		= 4;
int AVP_Human_HUDHandler::s_iGrenadeAmmoCounterMax	= 4;

int AVP_Human_HUDHandler::s_iStimAmmoCounter = 6;
int AVP_Human_HUDHandler::s_iAmmoCounter	= 99;
int AVP_Human_HUDHandler::s_iAmmoCounterMax = 99;

int AVP_Human_HUDHandler::s_iHeartRateCounter = 0;

int AVP_Human_HUDHandler::s_iArmourCounter = 0;

bool AVP_Human_HUDHandler::s_bMantleAvailable = false;

bool AVP_Human_HUDHandler::s_bHasSecondaryAmmo = true;
bool AVP_Human_HUDHandler::s_bLastClip = false;
bool AVP_Human_HUDHandler::s_bLowAmmoInClip = false;
bool AVP_Human_HUDHandler::s_bShowAimingHUD = false;
bool AVP_Human_HUDHandler::s_bMotionTrackerActive = false;
bool AVP_Human_HUDHandler::s_bTorchActive = false;

////////////////////////////////////////////////////////////////////////////////
// Private functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Protected functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Public functions...
////////////////////////////////////////////////////////////////////////////////

AVP_Human_HUDHandler::AVP_Human_HUDHandler (AVP_ClientEntity_Actor& xClient) 
: PARENT(xClient)
, m_fHealth(100.0f)
, m_fArmour(100.0f)
, m_fHUDScaleH(1.0f)
, m_fHUDScaleV(1.0f)
, m_bIsAiming(false)
{
	s_bShowAimingHUD = false; // new HUD, set to default

	m_pxMotionTracker = new AVP_Human_MotionTracker(xClient.GetGuid());
	
	//RefreshBlueprintValues();
}

AVP_Human_HUDHandler::~AVP_Human_HUDHandler () 
{
	delete(m_pxMotionTracker);
	m_pxMotionTracker = NULL;
}

#if defined(ASURA_SUPPORT_COMMAND_CONSOLE)
void AVP_Human_HUDHandler::CreateConsoleVars()
{
	if ( !Asura_CommandConsole::VarExists( L"AVP.Hud.Human.HasSecondaryAmmo" ) )
	{
		Asura_CommandConsole::AddVar( L"AVP.Hud.Human.HasSecondaryAmmo", &s_bHasSecondaryAmmo, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	}
	if ( !Asura_CommandConsole::VarExists( L"AVP.MarinePlayer.MantlePointInRange" ) )
	{
		Asura_CommandConsole::AddVar( L"AVP.MarinePlayer.MantlePointInRange", &s_bMantleAvailable, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	}
	if ( !Asura_CommandConsole::VarExists( L"AVP.Hud.Human.IsAiming" ) )
	{
		Asura_CommandConsole::AddVar( L"AVP.Hud.Human.IsAiming", &s_bShowAimingHUD, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	}

	if ( !Asura_CommandConsole::VarExists( L"AVP.Hud.Human.PrimaryAmmo" ) )
	{
		Asura_CommandConsole::AddVar( L"AVP.Hud.Human.PrimaryAmmo", &s_iAmmoCounter, INT_MIN, INT_MAX, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	}
	if ( !Asura_CommandConsole::VarExists( L"AVP.Hud.Human.PrimaryAmmoMax" ) )
	{
		Asura_CommandConsole::AddVar( L"AVP.Hud.Human.PrimaryAmmoMax", &s_iAmmoCounterMax, INT_MIN, INT_MAX, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	}

	if ( !Asura_CommandConsole::VarExists( L"AVP.Hud.Human.SecondaryAmmo" ) )
	{
		Asura_CommandConsole::AddVar( L"AVP.Hud.Human.SecondaryAmmo", &s_iGrenadeAmmoCounter, INT_MIN, INT_MAX, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	}
	if ( !Asura_CommandConsole::VarExists( L"AVP.Hud.Human.SecondaryAmmoMax" ) )
	{
		Asura_CommandConsole::AddVar( L"AVP.Hud.Human.SecondaryAmmoMax", &s_iGrenadeAmmoCounterMax, INT_MIN, INT_MAX, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	}

	if ( !Asura_CommandConsole::VarExists( L"AVP.Hud.Human.Stims" ) )
	{
		Asura_CommandConsole::AddVar( L"AVP.Hud.Human.Stims", &s_iStimAmmoCounter, INT_MIN, INT_MAX, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	}
	if ( !Asura_CommandConsole::VarExists( L"AVP.MarinePlayer.MotionTrackerActive" ) )
	{
		Asura_CommandConsole::AddVar( L"AVP.MarinePlayer.MotionTrackerActive", &s_bMotionTrackerActive, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	}
	if ( !Asura_CommandConsole::VarExists( L"AVP.MarinePlayer.TorchActive" ) )
	{
		Asura_CommandConsole::AddVar( L"AVP.MarinePlayer.TorchActive", &s_bTorchActive, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	}
	if ( !Asura_CommandConsole::VarExists( L"AVP.MarinePlayer.LastClip" ) )
	{
		Asura_CommandConsole::AddVar( L"AVP.MarinePlayer.LastClip", &s_bLastClip, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	}
	if ( !Asura_CommandConsole::VarExists( L"AVP.MarinePlayer.LowAmmoInClip" ) )
	{
		Asura_CommandConsole::AddVar( L"AVP.MarinePlayer.LowAmmoInClip", &s_bLowAmmoInClip, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	}
	if ( !Asura_CommandConsole::VarExists( L"AVP.HUD.ShowMotionTrackerScanner" ) )
	{
		Asura_CommandConsole::AddVar( L"AVP.HUD.ShowMotionTrackerScanner", &AVP_Human_MotionTracker::s_bShowMotionTrackerScanner, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	}
	if ( !Asura_CommandConsole::VarExists( L"AVP.HUD.ShowMotionTrackerPulse" ) )
	{
		Asura_CommandConsole::AddVar( L"AVP.HUD.ShowMotionTrackerPulse", &AVP_Human_MotionTracker::s_bShowMotionTrackerPulse, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	}
	if ( !Asura_CommandConsole::VarExists( L"AVP.HUD.ShowMotionTrackerDot" ) )
	{
		Asura_CommandConsole::AddVar( L"AVP.HUD.ShowMotionTrackerDot", &AVP_Human_MotionTracker::s_bShowMotionTrackerDot, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	}
	if ( !Asura_CommandConsole::VarExists( L"AVP.HUD.ShowMotionTrackerBase" ) )
	{
		Asura_CommandConsole::AddVar( L"AVP.HUD.ShowMotionTrackerBase", &AVP_Human_MotionTracker::s_bShowMotionTrackerBase, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	}
	if ( !Asura_CommandConsole::VarExists( L"AVP.HUD.MotionTrackerIsDisrupted" ) )
	{
		Asura_CommandConsole::AddVar( L"AVP.HUD.MotionTrackerIsDisrupted", &AVP_Human_MotionTracker::s_bMotionTrackerDisrupted, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	}
	
}
#endif
////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Human_HUDHandler::Update
// Author       : Chris Wallace
// Date         : 8-Jun-2007
////////////////////////////////////////////////////////////////////////////////

void AVP_Human_HUDHandler::Update( const AVP_ClientEntity_Actor& xClient )
{
	const AVP_ClientActor_Human& xHuman = static_cast< const AVP_ClientActor_Human& >( xClient );
	
	const AVP_Inventory_System* pxInventorySystem	= xHuman.GetInventorySystem();
	const AVP_InventoryItem_Weapon*	pxCurrentWeapon		= pxInventorySystem ? static_cast<const AVP_InventoryItem_Weapon*>(pxInventorySystem->GetCurrentItem(uAVP_INVENTORY_GROUPHASH_WEAPON)) : NULL;

	//Update health
	m_fHealth	= xHuman.GetHealth();

	// Update ammo counter
	if(pxCurrentWeapon != NULL && !pxCurrentWeapon->IsEncumberment() ) // If it is different from a tool item
	{
		const AVP_AmmoStore*			 pxAmmoStore		= pxInventorySystem->GetAmmoStore();
		const AVP_InventoryItem_SubType* pxPrimarySubType	= NULL;
		const AVP_InventoryItem_SubType* pxSecondarySubType	= NULL;
		s_bHasSecondaryAmmo = false;
		
		if(pxCurrentWeapon->GetItemType() == AVP_INVENTORYITEMTYPE_SMARTGUN)
		{
			pxPrimarySubType = pxCurrentWeapon->GetSubType(AVP_InventoryWeapon_RangedWeapon::ST_PRIMARY);
		}
		else if(pxCurrentWeapon->GetItemType() == AVP_INVENTORYITEMTYPE_SHOTGUN)
		{
			pxPrimarySubType = pxCurrentWeapon->GetSubType(AVP_InventoryWeapon_RangedWeapon::ST_PRIMARY); // Always get primary
		}
		else
		{
			if(pxCurrentWeapon->GetItemType() == AVP_INVENTORYITEMTYPE_SCOPEDRIFLE)
			{
				const AVP_InventoryWeapon_ScopedRifle* pxScopedRifle = static_cast<const AVP_InventoryWeapon_ScopedRifle*>(pxCurrentWeapon);

				bool bInvIsAimingFull = pxScopedRifle->IsInSniperMode(true);
				bool bInvIsAimingMode = pxScopedRifle->IsInSniperMode(false);
			
				if(m_bIsAiming != bInvIsAimingMode)
				{
					if(bInvIsAimingMode)
					{
						OnBeginSniperScope();
					}
					else
					{
						OnEndSniperScope();
					}
					m_bIsAiming = bInvIsAimingMode;
				}	

				s_bShowAimingHUD = bInvIsAimingFull;
			}
			else
			{
				if(s_bShowAimingHUD)
				{
					s_bShowAimingHUD = false;
					m_bIsAiming = false;
					OnEndSniperScope();
				}
			}

			pxPrimarySubType	= pxCurrentWeapon->GetSubType(AVP_InventoryWeapon_RangedWeapon::ST_PRIMARY);
		
			if ( pxCurrentWeapon->GetNumSubTypes() > 1 && !pxCurrentWeapon->GetSubtypesUseSameAmmo())
			{
				pxSecondarySubType = pxCurrentWeapon->GetSubType(AVP_InventoryWeapon_RangedWeapon::ST_SECONDARY);
				s_bHasSecondaryAmmo = true;
			}
		}

		s_iAmmoCounterMax	= (pxAmmoStore&&pxPrimarySubType)?Asura_Maths::FloatToInt(pxAmmoStore->GetAmmoQuantity(pxPrimarySubType->GetAmmoType())):0;
		s_iAmmoCounter		= (pxPrimarySubType)?Asura_Maths::FloatToInt(pxPrimarySubType->GetCurrentQuantity()):0;

		s_bLastClip = ( s_iAmmoCounterMax <= Asura_Maths::FloatToInt(pxPrimarySubType->GetMaxQuantity()) )? true : false;
		s_bLowAmmoInClip = ( s_iAmmoCounter <= Asura_Maths::FloatToInt_Ceil( pxPrimarySubType->GetMaxQuantity() * 0.1f ))? true : false;


		s_iGrenadeAmmoCounterMax	= (pxAmmoStore&&pxSecondarySubType)?Asura_Maths::FloatToInt(pxAmmoStore->GetAmmoQuantity(pxSecondarySubType->GetAmmoType())):0;
		s_iGrenadeAmmoCounter		= (pxSecondarySubType)?Asura_Maths::FloatToInt(pxSecondarySubType->GetCurrentQuantity()):0;	
		
		// Hack for infinite ammo
		if(AVP_Settings::s_bInfiniteAmmo || s_iAmmoCounterMax > 999)
		{
			s_iAmmoCounterMax = 999;
		}
	}

	const AVP_Inventory_Group* pxStimGroup = (pxInventorySystem)?pxInventorySystem->GetPermanentlyActiveItemGroup():NULL;
	if(pxStimGroup)
	{
		const AVP_Inventory_Item* pxStim = pxStimGroup->GetItem(uAVP_INVENTORY_ITEMHASH_STIM);
		if(pxStim)
		{
			const AVP_InventoryItem_SubType* pxStimSubType	= NULL;
			pxStimSubType = pxStim->GetSubType(0);
			s_iStimAmmoCounter	= (pxStimSubType)?Asura_Maths::FloatToInt(pxStimSubType->GetCurrentQuantity()):0;	
			s_iArmourCounter	= Asura_Maths::FloatToInt(xHuman.GetArmour());
		}
	}
	
	// AJK; AVP Functionality for tutorial; check if func allowed
	if (AVP_Settings::IsMotionTrackerFuncOn())
	{
		UpdateMotionTracker( xHuman );
	}

	const AVP_Human_JumpHandler* pxJumpHandler = static_cast<const AVP_Human_JumpHandler*>(xHuman.GetJumpHandler());
	if (pxJumpHandler)
	{
		// Only show the indicator when jump input is needed.
		s_bMantleAvailable = pxJumpHandler->GetJumpRequestMantleAvailable();
	}
	else
	{
		s_bMantleAvailable = false;
	}

	PARENT::Update( xClient );
}

/// End of function 'AVP_Predator_HUDHandler::Update'. ///

void AVP_Human_HUDHandler::GetReticlePosition(float& fX, float &fY)
{
	const AVP_Inventory_System* pxSystem = m_xOwner.GetInventorySystem();
	const AVP_Inventory_Group* pxGroup = (pxSystem)?pxSystem->GetGroup(uAVP_INVENTORY_GROUPHASH_WEAPON):NULL;
	const AVP_Inventory_Item* pxItem = (pxGroup)?pxGroup->GetCurrentInventoryItem():NULL;
	if(pxItem && pxItem->GetItemType() == AVP_INVENTORYITEMTYPE_SMARTGUN)
	{
		const AVP_InventoryWeapon_SmartGun* pxWeapon = static_cast<const AVP_InventoryWeapon_SmartGun*>(pxItem);
		if(pxWeapon->GetTrackingMode())
		{
			pxWeapon->GetScreenCoordsForReticle(fX,fY);
			return;
		}
	}

	// Only run if not smartgun!
	PARENT::GetReticlePosition(fX,fY);
}

void AVP_Human_HUDHandler::UpdateHUDCounter(u_int uCounterValue, int& iCounterUnits, int& iCounterTens,  int& iCounterHundreds)
{
	UpdateHUDCounter(3,uCounterValue, 999, iCounterUnits, iCounterTens, iCounterHundreds);
}

void AVP_Human_HUDHandler::UpdateHUDCounter(u_int uCounterValue, int& iCounterUnits, int& iCounterTens)
{
	//Create temp hundreds counter
	int iCounterHundreds=0;

	UpdateHUDCounter(2,uCounterValue, 99,iCounterUnits, iCounterTens, iCounterHundreds);
}

void AVP_Human_HUDHandler::UpdateHUDCounter(u_int uCounterValue, int& iCounterUnits)
{
	//Create temp hundreds counter
	int iCounterHundreds=0;
	int iCounterTens=0;

	UpdateHUDCounter(2,uCounterValue, 9,iCounterUnits, iCounterTens, iCounterHundreds);
}


void AVP_Human_HUDHandler::UpdateHUDCounter(u_int uNumCounters, u_int uCounterValue, u_int uMaxValue, int& iCounterUnits, int& iCounterTens, int& iCounterHundreds)
{
	//Calculate HUD counter values
	if ( uCounterValue <= uMaxValue )
	{
		char szHealthBuffer[4];
		u_int uNumChars = Asura_PrintfToBfr(szHealthBuffer,"%u",sizeof(szHealthBuffer),true, uCounterValue);

		if(uNumCounters >= 1)
		{
			iCounterUnits		= szHealthBuffer[uNumChars-1]-48;
			
			if(iCounterUnits > 9 || iCounterUnits < 0)
			{
				iCounterUnits=0;
			}
		}
		
		if(uNumCounters >= 2)
		{
			iCounterTens = szHealthBuffer[uNumChars-2]-48;
			
			if(iCounterTens > 9 || iCounterTens < 0)
			{
				iCounterTens=0;
			}
		}

		if(uNumCounters >= 3)
		{
			iCounterHundreds	= szHealthBuffer[uNumChars-3]-48;
			
			if(iCounterHundreds > 9 || iCounterHundreds < 0)
			{
				iCounterHundreds=0;
			}
		}
	}
	else
	{
		iCounterUnits=0;
		iCounterTens=0;
		iCounterHundreds=0;
		uCounterValue = 0;
	}
}

void AVP_Human_HUDHandler::UpdateMotionTracker( const AVP_ClientActor_Human& xClient )
{
	if(m_pxMotionTracker)
	{
		m_pxMotionTracker->Update( xClient );
	}
}

void AVP_Human_HUDHandler::DisruptMotionTracker()
{
	if(m_pxMotionTracker)
	{
		m_pxMotionTracker->Disrupt();
	}
}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Predator_CameraHandler::Render
// Author       : Chris Wallace
// Date         : 17-Oct-2006
////////////////////////////////////////////////////////////////////////////////

void AVP_Human_HUDHandler::Render( const AVP_ClientEntity_Actor& xClient )
{
	const AVP_ClientActor_Human& xHuman = static_cast< const AVP_ClientActor_Human& >( xClient );
	//Render HUD below menu
	// AJK; AVP Functionality for tutorial; check if func allowed
	if ( AVP_Settings::IsMotionTrackerFuncOn() )
	{
		RenderMotionTracker( xHuman );
	}

	//RenderText();	
	RenderPanicPrompts();

	RenderFocusTarget();
	
	RenderTargetName();

	PARENT::Render( xClient );
}

void AVP_Human_HUDHandler::RenderMotionTracker( const AVP_ClientActor_Human& xClient )
{
	if(m_pxMotionTracker)
	{
		m_pxMotionTracker->Render( xClient );
	}
}

/////////////////////////////////////////////////
// Private Render Functions
/////////////////////////////////////////////////

void AVP_Human_HUDHandler::RenderText( void )
{
 	//////////////////// TEXT ONLY STUFF /////////////
 
 	//Re-set co-ordinates
 	//Ideally this should be in the parent class, but as the parent class is called 
 	//last to keep the display order correct it is here.
 	//Should probably be moved once the HUD is complete.
	AVP_ClientActor_Human* pxHuman = AVP_ClientActor_Human::FindEntity(m_uOwnerGuid);

	if(pxHuman)
	{
		m_fY = static_cast<float>(m_fHeight) - (m_fIncrement*5.0f);
		m_fX = static_cast<float>(m_fWidth) - 300.0f;

		m_uTextColour = 0xff00ff00;

		Asura_UnicodeChar			wszText[256];

		Asura_PrintfToBfr(wszText, L"Min Heart Rate: %f", 256, true, pxHuman->GetMinHeartRate());
		Asura_Font_System::Print(m_fLeft+m_fX, m_fTop+m_fY, m_fHUDScaleH*m_fTextScale, m_uTextColour, m_uTextFormat, wszText);

		m_fY += m_fIncrement;

		Asura_PrintfToBfr(wszText, L"Max Heart Rate: %f", 256, true, pxHuman->GetMaxHeartRate());
		Asura_Font_System::Print(m_fLeft+m_fX, m_fTop+m_fY, m_fHUDScaleH*m_fTextScale, m_uTextColour, m_uTextFormat, wszText);

		m_fY += m_fIncrement;

		Asura_PrintfToBfr(wszText, L"Cur Heart Rate: %f", 256, true, pxHuman->GetCurHeartRate());
		Asura_Font_System::Print(m_fLeft+m_fX, m_fTop+m_fY, m_fHUDScaleH*m_fTextScale, m_uTextColour, m_uTextFormat, wszText);
	}
}

void AVP_Human_HUDHandler::RenderFocusTarget(void)
{
	const AVP_Human_Focus* pxFocus = static_cast<const AVP_Human_Focus*>(m_xOwner.GetFocus());
	if(pxFocus)
	{
		AVP_ClientEntity_Actor* pxTargetPlayerPuppet = AVP_ClientEntity_Actor::FindEntity( pxFocus->GetLockOnTarget() ); 
		if (pxTargetPlayerPuppet)
		{				
			// Player puppet position
			const Asura_ListElt<Focus_Target>* pxFocusTargetLE = pxFocus->GetTargetList()->Front();
			while(pxFocusTargetLE)
			{
				const Focus_Target xFocusTarget = pxFocusTargetLE->Data();

				Asura_GUIMenu_Root* pxItem = AVP_GUIMenu_System::GetHUDElement( AVP_GUIMenu_System::SMARTGUN_TARGET, AVP_GUIMenu_System::GetMenuHash( AVP_GUIMenu_System::GUIMENU_ID_HUD_ELEMENTS ), false );
				if( pxItem )
				{
					// Update pos as update would have been too inacurate for rendering
						AVP_ClientEntity_Actor* pxTarget= AVP_ClientEntity_Actor::FindEntity( xFocusTarget.m_uGuid ); 
						Asura_Vector_3 xCurrentPlayerWorldPosition; 
						if ( !pxTarget->GetMarkerLocationFromHash( 0xe5a85d03, &xCurrentPlayerWorldPosition, NULL ) ) // target_1
						{
							xCurrentPlayerWorldPosition = pxTarget->GetCurrentPosition();
							Asura_Assert(false, "Failed to find target markerpoint");
						}

						// Projected player position on screen
						Asura_Vector_3 xCurrentPlayerScreenPosition( xCurrentPlayerWorldPosition );
						Asura_Camera::ProjectWorldCoordsToScreenSpace( xCurrentPlayerScreenPosition, true ); 
						Asura_Vector_2 xInstancePosition( xCurrentPlayerScreenPosition.GetX(), xCurrentPlayerScreenPosition.GetY() ); 
					//end					

					Asura_Bounding_Rectangle xNewBoundingBox( pxItem->GetScaledWorldBounds() ); 
					Asura_Vector_2 xIconOrigin( pxItem->GetPosOrigin() );
					Asura_Vector_2 xIconSize( xNewBoundingBox.GetDimension() ); 
					xIconOrigin.x *= xIconSize.x;
					xIconOrigin.y *= xIconSize.y;
					Asura_Vector_2 xInstanceDisplacement = xInstancePosition - ( pxItem->GetScaledWorldPos() + xIconOrigin );
					xNewBoundingBox.Displace( xInstanceDisplacement );

					if(pxItem->IsWidget())
					{
						for(int i=0;i<pxItem->GetContainerCount();i++)
						{
							Asura_GUIMenu_Root* pxChild = pxItem->GetContainerItem(i);
							if(pxChild)
							{
								Asura_RGBA xColour = pxChild->GetColour();
								xColour.m_fA = 1.0f;
								u_int uColour = xColour.ToUIntARGB();
								uColour = AVP_Overlay_Render::MultiplyAlphaValue(xColour.ToUIntARGB(), xFocusTarget.m_fPriority);

								pxChild->SetColourFromARGB(uColour);
							}
						}
					}
					else
					{
						Asura_RGBA xColour = pxItem->GetColour();
						xColour.m_fA = 1.0f;
						u_int uColour = xColour.ToUIntARGB();
						uColour = AVP_Overlay_Render::MultiplyAlphaValue(xColour.ToUIntARGB(), xFocusTarget.m_fPriority);

						pxItem->SetColourFromARGB(uColour);
					}

					pxItem->SetScaledWorldBounds(xNewBoundingBox);
					pxItem->Render( ASURA_GUIMENU_ACTION_FLAG_ALL );
				}

				pxFocusTargetLE = pxFocusTargetLE->Next();
			}	
		}
	}
}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Human_HUDHandler::RenderTargetName
// Author       : AJK
// Date         : 05-Nov-2008
////////////////////////////////////////////////////////////////////////////////
void AVP_Human_HUDHandler::RenderTargetName(void) const
{
	AVP_ClientEntity_PlayerInfo* pxPlayerInfo = AVP_ClientEntity_PlayerInfo::FindEntity( AVP_Game_Core::GetPlayerGuid() ); 

	if ( pxPlayerInfo && ( pxPlayerInfo->GetTargetGuid() != ASURA_GUID_UNREGISTERED))
	{
		Asura_Vector_3 xHeadPosition;
		const AVP_ClientActor_Human* pxTarget = AVP_ClientActor_Human::FindEntity( pxPlayerInfo->GetTargetGuid() ); 

		if( pxTarget )
		{
			pxTarget->GetHeadPosition(xHeadPosition);
			if(Asura_Camera::ProjectWorldCoordsToScreenSpace(xHeadPosition, true))
			{
				Asura_Font_System::Print(xHeadPosition.x, xHeadPosition.y, 0xffffffff,
					ASURA_TEXTFORMAT_CENTREJUSTIFIED|ASURA_TEXTFORMAT_VERTICALCENTREJUSTIFIED|ASURA_TEXTFORMAT_FORMATSTRING,
					Asura_HashedLocalisedText::GetString(pxTarget->GetNameText().GetTextPageHash(), pxTarget->GetNameText().GetTextID()));
			}
		}
	}
}

void AVP_Human_HUDHandler::Reset( void )
{
	PARENT::Reset();
}

void AVP_Human_HUDHandler::RefreshBlueprintValues(void)
{
#ifndef ASURA_TOOLS
	// Only want to keep these up-to-date in a tools environment, save a little processing in retail builds
	if(s_bGotStaticBlueprintValues)
	{
		return;
	}

	s_bGotStaticBlueprintValues = true;
#endif

	{
// TODO, any blueprint values go here


// 		Asura_Value xValue;
// 		Asura_Hash_ID uBlueprintHash;
// 		bool bGotBlueprint;
// 
// 		Asura_Blueprint* pxBlueprint = NULL;
// 		Asura_Hash_ID uTypeHash = Asura_GetHashID("Settings");
// 
// 		uBlueprintHash = Asura_GetHashID("MarineHUD");
// 		bGotBlueprint = Asura_Blueprint_System::GetBlueprint(uTypeHash, uBlueprintHash, pxBlueprint);
// 
// 		if(bGotBlueprint && pxBlueprint)
// 		{
// 			if(pxBlueprint->GetParameterValueByHashID(Asura_GetHashID("HealthBarX"), xValue))
// 			{
// 				m_xHealthBarPos.x = (xValue.GetFloatValue());
// 			}
// 			if(pxBlueprint->GetParameterValueByHashID(Asura_GetHashID("HealthBarY"), xValue))
// 			{
// 				m_xHealthBarPos.y = (xValue.GetFloatValue());
// 			}
// 			if(pxBlueprint->GetParameterValueByHashID(Asura_GetHashID("ArmourBarX"), xValue))
// 			{
// 				m_xArmourBarPos.x = (xValue.GetFloatValue());
// 			}
// 			if(pxBlueprint->GetParameterValueByHashID(Asura_GetHashID("ArmourBarY"), xValue))
// 			{
// 				m_xArmourBarPos.y = (xValue.GetFloatValue());
// 			}
// 		}
	}
}

void AVP_Human_HUDHandler::OnBeginSniperScope(void)
{
	if(AVP_InventoryWeapon_ScopedRifle::s_uAimingFSFX != ASURA_HASH_ID_UNSET)
	{
		SetFSFXValue(FSFX_SCOPEDRIFLE, AVP_InventoryWeapon_ScopedRifle::s_uAimingFSFX, 1.0f, 0.5f);
		
		AVP_Vertebrate_CameraHandler* pxCameraHandler = static_cast<AVP_Vertebrate_CameraHandler*>( m_xOwner.GetCameraHandler() );
		if(pxCameraHandler)
		{
			pxCameraHandler->SetZoomLevel( static_cast<AVP_ClientActor_Vertebrate&>(m_xOwner), 1 );
		}
	}
}

void AVP_Human_HUDHandler::OnEndSniperScope(void)
{
	if(AVP_InventoryWeapon_ScopedRifle::s_uAimingFSFX != ASURA_HASH_ID_UNSET)
	{
		SetFSFXValue(FSFX_SCOPEDRIFLE, AVP_InventoryWeapon_ScopedRifle::s_uAimingFSFX, 0.0f, 0.5f);
		
		AVP_Vertebrate_CameraHandler* pxCameraHandler = static_cast<AVP_Vertebrate_CameraHandler*>( m_xOwner.GetCameraHandler() );
		if(pxCameraHandler)
		{
			pxCameraHandler->SetZoomLevel( static_cast<AVP_ClientActor_Vertebrate&>(m_xOwner), 0 );
		}
	}
}

/// End of file. ///
