
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_Human_InputHandler.h
// Author : Chris Wallace
// Date   : 5-Feb-2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __AVP_HUMAN_INPUTHANDLER_H__
#define __AVP_HUMAN_INPUTHANDLER_H__

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Actor/Vertebrates/AVP_Vertebrate_InputHandler.h"


////////////////////////////////////////////////////////////////////////////////
// Forward class declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Class declaration...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_Human_InputHandler
// Author       : Andy Tate
// Date         : 3-Oct-2006
//
// Description  : Class to handle the input for a predator client-entity and
//				  create the appropriate message data for the server-entity.
////////////////////////////////////////////////////////////////////////////////

class AVP_Human_InputHandler : public AVP_Vertebrate_InputHandler
{
public:
	// Public functions...
	AVP_Human_InputHandler             (const Asura_Guid uOwnerGuid);
	virtual ~AVP_Human_InputHandler ();

	void MapVisionAndFlare(Asura_ControlMap& xMap, const u_int uKeyVisionNext, const u_int uKeyFlare);
	void MapTorch(Asura_ControlMap& xMap, const u_int uKey);
	void MapFlare(Asura_ControlMap& xMap, const u_int uKey);
	void MapADS(Asura_ControlMap& xMap, const u_int uKey);
	

protected:
	// Protected variables...

	// Protected functions...
	virtual	Asura_Message*	Update( AVP_ClientEntity_Actor& xClient );

	virtual void			UpdateMovement						( AVP_ClientEntity_Actor& xClient );
	virtual void			UpdateWeapons						( AVP_ClientEntity_Actor& xClient );
	virtual void			UpdateZoom							( AVP_ClientActor_Vertebrate& xClient );
	virtual void			UpdateVisionModes					( AVP_ClientEntity_Actor& xClient );
	virtual void			UpdateMeleeRequests					( AVP_ClientActor_Vertebrate& xClient );

	virtual void UpdateMovementInput(AVP_ClientEntity_Actor& xClient);

	virtual	Asura_Message*	CreatePlayerCommandMessageToServer	( const AVP_ClientEntity_Actor& xClient ) const;

private:
	// Private typedefs...
	typedef AVP_Vertebrate_InputHandler PARENT;

	// Private variables...
#if defined(ASURA_SUPPORT_COMMAND_CONSOLE)
	static int	s_iConsoleVarsCount;
#endif	// defined(ASURA_SUPPORT_COMMAND_CONSOLE)

	// Private functions...
	AVP_Human_InputHandler  (const AVP_Human_InputHandler& xOther);

	AVP_Human_InputHandler&	operator=	(const AVP_Human_InputHandler& xOther);

	// Platform specific control-map creation...
	void	Platform_InitialiseControlMap	(void);

};   // End of class 'AVP_Human_InputHandler'.


#endif   // ifndef __AVP_HUMAN_INPUTHANDLER_H__


/// End of file. ///
