//-----------------------------------------------------------------------------
// File   : AVP_Human_MoraleHandler.cpp
// Author : Gareth Thomas
// Date   : 08 / 02 / 2008
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------
// includes
//-----------------------------------------------------------------------
// Core
#include "Core/Asura.h"

// This Files Header
#include "Actor/Vertebrates/Human/AVP_Human_MoraleHandler.h"

// Asura Includes
#include "Blueprint/Asura_Blueprint.h"
#include "Blueprint/Asura_Blueprint_System.h"
#include "Core/Asura_Timers.h"
#include "Core/Asura_Maths.h"

// AVP
#include "Entity/AVP_Entity_Classification.h"
#include "Entity/PhysicalObject/Actor/AVP_ServerEntity_Actor.h"


//-----------------------------------------------------------------------
// AVP_Human_Morale_Handler::Constructor
//-----------------------------------------------------------------------
AVP_Human_MoraleHandler::AVP_Human_MoraleHandler(Asura_Guid uOwnerGuid)
: m_uOwnerGuid(uOwnerGuid)
, m_fInitialValue(50.0f)
, m_fCurrentValue(m_fInitialValue)
, m_fDesiredValue(50.0f)
, m_fMoraleMoverate(1.0f)		
, m_fMinMorale(0.0f)
, m_fMaxMorale(100.0f)	
, m_fNormalMoraleLowPoint(10.0f)
, m_fNormalMoraleHighPoint(90.0f)
, m_fLowMoraleNormalPoint(20.0f)
, m_fHighMoraleNormalPoint(80.0f)
, m_eMoraleType(NORMAL_MORALE)
, m_bDegrading(false)
, m_fDegradationTimer(0.0f)
, m_fTimeBeforeDegradation(2.0f)
{
	// Initialise Our effectors
	for (u_int u = 0; u < MAX_EFFECTORS; ++u)
	{
		m_fEffectors[u] = 0.0f;
	}
}

//-----------------------------------------------------------------------
// AVP_Human_Morale_Handler::Deconstructor
//-----------------------------------------------------------------------
AVP_Human_MoraleHandler::~AVP_Human_MoraleHandler(void)
{
	
}

//-----------------------------------------------------------------------
// AVP_Human_Morale_Handler::Update
//-----------------------------------------------------------------------
void AVP_Human_MoraleHandler::InitaliseFromBlueprint(void)
{
	AVP_ServerEntity_Actor* pxActor = AVP_ServerEntity_Actor::FindEntity(m_uOwnerGuid);

	if (pxActor)
	{
		Asura_Hash_ID		uBlueprintTypeHashID	= 0x7AD7BD1B; //Asura_GetHashID("AI Settings");
		Asura_Hash_ID		uBlueprintHashID		= ASURA_HASH_ID_UNSET;

		switch (pxActor->GetClassification())
		{
			case AVP_ENTITYCLASS_MARINE:	uBlueprintHashID = 0x8bf96ccc; break; //"MarineMorale"
			default:	Asura_Assert(false, "AVP_Human_MoraleHandler :: InitaliseFromBlueprint - This actor is not a AI marine actor, only marines are meant to have a morale handler");
		}

		Asura_Blueprint*	pxBlueprint				= Asura_Blueprint_System::GetBlueprintByHashID(uBlueprintTypeHashID, uBlueprintHashID);
		if (pxBlueprint)
		{
			Asura_Value xValue;

			if (pxBlueprint->GetParameterValueByHashID(0x0615EF95, xValue)) //Asura_GetHashID("HighMorale NormalPoint"), xValue))
				m_fHighMoraleNormalPoint = xValue.GetFloatValue();

			if (pxBlueprint->GetParameterValueByHashID(0xCF976CC7, xValue)) //Asura_GetHashID("LowMorale NormalPoint"), xValue))
				m_fLowMoraleNormalPoint = xValue.GetFloatValue();

			if (pxBlueprint->GetParameterValueByHashID(0x09B2C2EE, xValue)) //Asura_GetHashID("MaxMorale"), xValue))
				m_fMaxMorale = xValue.GetFloatValue();

			if (pxBlueprint->GetParameterValueByHashID(0x37BDD95C, xValue)) //Asura_GetHashID("MinMorale"), xValue))
				m_fMinMorale = xValue.GetFloatValue();

			if (pxBlueprint->GetParameterValueByHashID(0xC47BC1CC, xValue)) //Asura_GetHashID("MoraleMoveSpeed"), xValue))
				m_fMoraleMoverate = xValue.GetFloatValue();

			if (pxBlueprint->GetParameterValueByHashID(0xD1F8F3FF, xValue)) //Asura_GetHashID("NormalMorale HighPoint"), xValue))
				m_fNormalMoraleHighPoint = xValue.GetFloatValue();

			if (pxBlueprint->GetParameterValueByHashID(0x7B4CC1CB, xValue)) //Asura_GetHashID("NormalMorale LowPoint"), xValue))
				m_fNormalMoraleLowPoint = xValue.GetFloatValue();

			if (pxBlueprint->GetParameterValueByHashID(0xEF3568CA, xValue)) //Asura_GetHashID("StartingMorale"), xValue))
				m_fInitialValue = xValue.GetFloatValue();

			if (pxBlueprint->GetParameterValueByHashID(0xe0efd1fa, xValue)) //Asura_GetHashID("TimeBeforeDegradation"), xValue))
				m_fTimeBeforeDegradation = xValue.GetFloatValue();

			// Negative Effectors
			if (pxBlueprint->GetParameterValueByHashID(0xE9657210, xValue)) //Asura_GetHashID("z_n_Effect Health Below 50%"), xValue))
				m_fEffectors[AVP_MORALE_EFFECTOR_HEALTH_DROPPING_BELOW_50_PERCENT] = xValue.GetFloatValue();

			if (pxBlueprint->GetParameterValueByHashID(0xF4448960, xValue)) //Asura_GetHashID("z_n_Effect Hit By Acid"), xValue))
				m_fEffectors[AVP_MORALE_EFFECTOR_HIT_BY_ACID] = xValue.GetFloatValue();

			if (pxBlueprint->GetParameterValueByHashID(0xB6E3C6B4, xValue)) //Asura_GetHashID("z_n_Effect Lost A Limb"), xValue))
				m_fEffectors[AVP_MORALE_EFFECTOR_LOST_A_LIMB] = xValue.GetFloatValue();

			if (pxBlueprint->GetParameterValueByHashID(0xCEB9DB12, xValue)) //Asura_GetHashID("z_n_Effect Reloading Under Threat"), xValue))
				m_fEffectors[AVP_MORALE_EFFECTOR_RELOADING_WHILE_UNDER_THREAT] = xValue.GetFloatValue();

			if (pxBlueprint->GetParameterValueByHashID(0x97848664, xValue)) //Asura_GetHashID("z_n_Effect Squad Member Killed"), xValue))
				m_fEffectors[AVP_MORALE_EFFECTOR_SQUAD_MEMBER_KILLED] = xValue.GetFloatValue();

			// Positive Effectors
			if (pxBlueprint->GetParameterValueByHashID(0xCF851500, xValue)) //Asura_GetHashID("z_p_Effect Health Above 70%"), xValue))
				m_fEffectors[AVP_MORALE_EFFECTOR_HEALTH_RECOVERED_ABOVE_70_PERCENT] = xValue.GetFloatValue();

			if (pxBlueprint->GetParameterValueByHashID(0x075DAB3F, xValue)) //Asura_GetHashID("z_p_Effect Killed Alien"), xValue))
				m_fEffectors[AVP_MORALE_EFFECTOR_KILLED_ALIEN] = xValue.GetFloatValue();

			if (pxBlueprint->GetParameterValueByHashID(0xA9C90CE7, xValue)) //Asura_GetHashID("z_p_Effect Killed Android"), xValue))
				m_fEffectors[AVP_MORALE_EFFECTOR_KILLED_ANDROID] = xValue.GetFloatValue();

			if (pxBlueprint->GetParameterValueByHashID(0x7A0C7B7F, xValue)) //Asura_GetHashID("z_p_Effect Killed Predator"), xValue))
				m_fEffectors[AVP_MORALE_EFFECTOR_KILLED_PREDATOR] = xValue.GetFloatValue();

			if (pxBlueprint->GetParameterValueByHashID(0x6AE46925, xValue)) //Asura_GetHashID("z_p_Effect Squad Killed Alien"), xValue))
				m_fEffectors[AVP_MORALE_EFFECTOR_SQUAD_MEMBER_KILLED_ALIEN] = xValue.GetFloatValue();

			if (pxBlueprint->GetParameterValueByHashID(0x4697E94D, xValue)) //Asura_GetHashID("z_p_Effect Squad Killed Android"), xValue))
				m_fEffectors[AVP_MORALE_EFFECTOR_SQUAD_MEMBER_KILLED_ANDROID] = xValue.GetFloatValue();

			if (pxBlueprint->GetParameterValueByHashID(0x77192BD9, xValue)) //Asura_GetHashID("z_p_Effect Squad Killed Predator"), xValue))
				m_fEffectors[AVP_MORALE_EFFECTOR_SQUAD_MEMBER_KILLED_PREDATOR] = xValue.GetFloatValue();
		}

		// Set our current and desired value to that of the initial value
		m_fCurrentValue = m_fDesiredValue = m_fInitialValue;
	}
}

//-----------------------------------------------------------------------
// AVP_Human_Morale_Handler::Update
//-----------------------------------------------------------------------
void AVP_Human_MoraleHandler::Update(void)
{
	float fTimeslice			= Asura_Timers::GetFrameTime();
	float fDegradationSpeed		= CalculateDegradationIfNeeded(fTimeslice);

	UpdateMorale(fTimeslice, fDegradationSpeed);
	UpdateMoraleType();
}

//-----------------------------------------------------------------------
// AVP_Human_Morale_Handler :: UpdateMoraleType
//-----------------------------------------------------------------------
void AVP_Human_MoraleHandler::UpdateMoraleType()
{
	// Set our enum
	switch (m_eMoraleType)
	{
		//--------------------------------------------------------------
		case NORMAL_MORALE:
		{
			if (m_fCurrentValue > m_fNormalMoraleHighPoint)
			{
				m_eMoraleType = HIGH_MORALE;
			}
			else if (m_fCurrentValue < m_fNormalMoraleLowPoint)
			{
				m_eMoraleType = LOW_MORALE;
			}
		}break;
		//--------------------------------------------------------------
		case LOW_MORALE:
		{
			if (m_fCurrentValue > m_fLowMoraleNormalPoint)
			{
				m_eMoraleType = NORMAL_MORALE;
			}
		}break;
		//--------------------------------------------------------------
		case HIGH_MORALE:
		{
			if (m_fCurrentValue < m_fHighMoraleNormalPoint)
			{
				m_eMoraleType = NORMAL_MORALE;
			}
		}break;
		//--------------------------------------------------------------
		default:
		{

		}break;
	}
}

//-----------------------------------------------------------------------
// AVP_Human_Morale_Handler::EffectMorale
//-----------------------------------------------------------------------
float AVP_Human_MoraleHandler::CalculateDegradationIfNeeded(const float fTimeslice)
{
	// Update the degradation timer
	m_fDegradationTimer -= fTimeslice;

	if (m_fDegradationTimer < 0.0f)
		m_fDegradationTimer = 0.0f;

	// Do some natural degradation.
	if ((m_fCurrentValue == m_fDesiredValue) && (m_fCurrentValue != m_fInitialValue) && (m_fDegradationTimer == 0.0f))
	{
		// Greater than starting morale.
		if (m_fCurrentValue > m_fInitialValue)
		{
			m_bDegrading		= true;
			m_fDesiredValue		= m_fInitialValue;
		}
		// Less than starting morale, set desired to the initial and work out a speed.
		else if (m_fCurrentValue < m_fInitialValue)
		{
			m_bDegrading		= true;
			m_fDesiredValue		= m_fInitialValue;
		}
	}

	float fDegradationSpeed = 1.0f;

	// Calculate the degradation speed if we are degrading morale
	if (m_bDegrading)
	{
		// Change the calculation depending on which side the current value is of the initial value
		if (m_fCurrentValue > m_fInitialValue)
		{
			fDegradationSpeed = (m_fCurrentValue - m_fInitialValue) / 100.0f;
		}
		else if (m_fCurrentValue < m_fInitialValue)
		{
			fDegradationSpeed = (m_fInitialValue - m_fCurrentValue) / 100.0f;
		}

		if (fDegradationSpeed < 0.2f)
		{
			fDegradationSpeed = 0.2f;
		}
	}

	return fDegradationSpeed;
}

//-----------------------------------------------------------------------
// AVP_Human_Morale_Handler :: UpdateMorale
//-----------------------------------------------------------------------
void AVP_Human_MoraleHandler::UpdateMorale(const float fTimeslice, const float fDegradationSpeed)
{
	// Update the current value
	if (m_fDesiredValue != m_fCurrentValue)
	{
		// Are we currently less than our desired value
		if (m_fDesiredValue < m_fCurrentValue)
		{
			if (m_bDegrading)
			{
				// If we are degrading do it at a different speed. morale degrades slower than it changes by effectors
				// Move the current value
				m_fCurrentValue -= (m_fMoraleMoverate * fDegradationSpeed * fTimeslice) * Asura_Timers::GetGameTimeScale();
			}
			else
			{
				// Move the current value
				m_fCurrentValue -= (m_fMoraleMoverate * fTimeslice) * Asura_Timers::GetGameTimeScale();
			}

			// We were less than desired see if changing the current value has put us over the desired. 
			// If so set it to the desired so we don't poll each side of desired
			if (m_fCurrentValue < m_fDesiredValue)
			{
				m_fCurrentValue = m_fDesiredValue;
				m_fDegradationTimer = 2.0f;
			}
			// this is unlikely to happen because of floating point accuracy, but it is here in case!
			else if ((m_fCurrentValue == m_fDesiredValue) || (m_fCurrentValue < m_fMinMorale))
			{
				m_fDegradationTimer = 2.0f;
			}
		}
		// Are we currently more than our desired value
		else if (m_fDesiredValue > m_fCurrentValue)
		{

			if (m_bDegrading)
			{
				// if we are degrading do it at a different speed. morale degrades slower than it changes by effectors
				// Move the current value
				m_fCurrentValue += (m_fMoraleMoverate * fDegradationSpeed * fTimeslice) * Asura_Timers::GetGameTimeScale();
			}
			else
			{
				// Move the current value
				m_fCurrentValue += (m_fMoraleMoverate * fTimeslice) * Asura_Timers::GetGameTimeScale();
			}

			// We were more than desired see if changing the current value has put us under the desired. 
			// If so set it to the desired so we don't poll each side of desired
			if (m_fCurrentValue > m_fDesiredValue)
			{
				m_fDegradationTimer = 2.0f;
				m_fCurrentValue = m_fDesiredValue;
			}
			// this is unlikely to happen because of floating point accuracy, but it is here in case!
			else if ((m_fCurrentValue == m_fDesiredValue) || (m_fCurrentValue > m_fMaxMorale))
			{
				m_fDegradationTimer = 2.0f;
			}
		}
	}

	// Clamp out current value to the min and max morale
	m_fCurrentValue = Asura_Maths::ClampToRange(m_fCurrentValue, m_fMinMorale, m_fMaxMorale);
}

//-----------------------------------------------------------------------
// AVP_Human_Morale_Handler::EffectMorale
//-----------------------------------------------------------------------
void AVP_Human_MoraleHandler::EffectMorale(u_int eEffectorType)
{
	if (eEffectorType >= MAX_EFFECTORS)
	{
		Asura_Assert(false, "AVP_Human_MoraleHandler::EffectMorale - Unrecognised Morale Effector");
	}
	else
	{
		// If we are degrading morale, stop it and set our desired value to our current so we can go in the opposite direction if needed.
		if (m_bDegrading)
		{
			m_fDesiredValue = m_fCurrentValue;
			m_bDegrading = false;
		}
		
		// change the desired value according to the effector
		m_fDesiredValue += m_fEffectors[eEffectorType];

		// Reset the degradation timer as we don't want any right now
		m_fDegradationTimer = 0.0f;

		// Clamp the desired to the minimum and maximum.
		m_fDesiredValue = Asura_Maths::ClampToRange(m_fDesiredValue, m_fMinMorale, m_fMaxMorale);
	}
}

/// End of file.
