///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_Human_MotionTracker.h
// Author : Chris Wallace
// Date   : 8-Jun-2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __AVP_HUMAN_MOTIONTRACKER_H__
#define __AVP_HUMAN_MOTIONTRACKER_H__

#include "Core/Asura_LinkList.h"

#include "Sound/Asura_Sound_Event_System.h"
#include "Core/Asura_Maths.h"

////////////////////////////////////////////////////////////////////////////////
// Structs and enums ...
////////////////////////////////////////////////////////////////////////////////

class AVP_ClientActor_Human;
class Asura_GUIMenu_Element_Image;
class Asura_GUIMenu_Element_Text;

// Struct for the dots
typedef struct xMTDot {
	Asura_Guid		uActorGuid;
	Asura_Vector_3	xInitialPos;
	Asura_Vector_2	xPosition;
	float			fLifeTime;
	bool			bActive;
} TRACKER_DOT;

// enum for graphics
enum {
	AVP_MOTIONTRACKER_FRAME_ID,
	AVP_MOTIONTRACKER_SCANNER_ID,
	AVP_MOTIONTRACKER_PULSE_ID,
	AVP_MOTIONTRACKER_DOT_ID,

	MAX_MOTIONTRACKER_IDS
};

////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_Human_MotionTracker
// Author       : Chris Wallace
// Date         : 8-Jun-2007
//
// Description  : Class for the marines motion tracker.
////////////////////////////////////////////////////////////////////////////////

class AVP_Human_MotionTracker
{
public:
	AVP_Human_MotionTracker(Asura_Guid uOwnerGuid);
	~AVP_Human_MotionTracker();
	
	void Update( const AVP_ClientActor_Human& xClient );
	void Render( const AVP_ClientActor_Human& xClient );
	void Disrupt();

	static bool s_bShowMotionTrackerScanner;
	static bool s_bShowMotionTrackerPulse;
	static bool s_bShowMotionTrackerDot;
	static bool s_bShowMotionTrackerBase;
	static bool s_bMotionTrackerDisrupted;

private:

	static const Asura_Hash_ID s_uTextureID[MAX_MOTIONTRACKER_IDS];

	void RefreshBlueprintValues(void);

	void RenderScanners	();
	void RenderScanner  ( Asura_GUIMenu_Element_Image& xImageScanner );
	void RenderBase 	();
	void RenderPulse	(float fAlpha);
	void RenderDots		(const AVP_ClientActor_Human& xClient, float fAlpha, Asura_LinkList<TRACKER_DOT>* pxList);
	void RenderFuzz		(float fAlpha);
	void RenderDistances();
	void RenderDistance ( Asura_GUIMenu_Element_Text& xText );

	void UpdateScanner	( const AVP_ClientActor_Human& xClient );
	void UpdatePulse	();
	void UpdateSecondaryPulse();
	void UpdateActors	( const AVP_ClientActor_Human& xClient );
	void UpdateDots		( const AVP_ClientActor_Human& xClient, Asura_LinkList<TRACKER_DOT>* );
	void UpdateFadeParametric(void);
	void UpdateDisruption();

	void PlayBleepSound	(float fParametric);
	void PlayPulseSound	();
	void SwapLists(void);

	static void RenderImage(const Asura_GUIMenu_Element_Image& xImage );

	Asura_LinkList<TRACKER_DOT> m_xDotList;
	Asura_LinkList<TRACKER_DOT> m_xSecondaryDotList;
	
	Asura_LinkList<TRACKER_DOT>* m_pxCurrentList;


	Asura_Vector_2							 m_xPosition;
	Asura_Guid								 m_uOwnerGuid;

	Asura_Angle		m_fScannerTheta;			// Angle of the trackers background
	float			m_fMotionTimer;				// Parametric time per pulse
	
	float			m_fPulseScale;				// Current scale of the pulse
	float			m_fPulseAlpha;				// Current alpha of the pulse

	float			m_fSecondaryPulseScale;				// Current scale of the pulse
	float			m_fSecondaryPulseAlpha;				// Current alpha of the pulse
	
	float			m_fFadeParametric;			// Used for when the HUD is faded in/out
	float			m_fDisruptTimer;
	float			m_fFlickerTimer;
	float           m_fClosestDistance;         // Store the closest actor distance to the player

	union
	{
		u_int		m_fFlags;
		struct
		{
			u_int	m_bDisrupted		: 1,
					m_bScannedThisPulse	: 1,
					m_bBleepedThisPulse	: 1,
										: 0;
		};
	};

	const Asura_Sound_Event*	m_pxPulseSound;
	const Asura_Sound_Event*	m_pxBleepSound;

	Asura_Handle m_iPulseHandle;
	Asura_Handle m_iBleepHandle;
};
#endif
