
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_Predalien_Description.h
// Author : Andy Tate
// Date   : 12-Feb-2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __AVP_PREDALIEN_DESCRIPTION_H__
#define __AVP_PREDALIEN_DESCRIPTION_H__

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura_Hash.h"

#if defined(ASURA_TOOLS)
#include "Misc/list_tem.hpp"
#endif	// defined(ASURA_TOOLS)

#include "Entity/PhysicalObject/Actor/AVP_ClientEntity_Actor.h"
#include "Physics/AVP_Collision_Descriptor.h"

static const u_int uAVP_PREDALIEN_NUM_IK_BONES_PER_LIMB = 3;
static const u_int uAVP_PREDALIEN_NUM_IK_BONES_PER_TAIL = 12;


////////////////////////////////////////////////////////////////////////////////
// Forward class declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Class declaration...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_Predalien_Description
// Author       : Andy Tate
// Date         : 12-Feb-2007
//
// Description  : Class used to contain the appropriate descriptions for the
//				  Predalien skins.
////////////////////////////////////////////////////////////////////////////////

class AVP_Predalien_Description
{
public:
	// Public enumerated types...
	enum MarkerPointType
	{
		EYES_MARKER_POINT_HASH,
		TARGET_MARKER_POINT_HASH,
		NUM_MARKER_POINT_IDS
	};

	enum ThirdPersonBoneID
	{	
		BIP01,					 // 0 
		BIP01_PELVIS,            // 1 
		BIP01_SPINE,             // 2 
		BIP01_SPINE1,            // 3 
		BIP01_SPINE2,            // 4 
		BIP01_NECK,              // 5 
		BIP01_HEAD,              // 6 
		CMANDIBLE_R_T_1,         // 7 
		MANDIBLE_R_T_2,          // 8 
		MANDIBLE_R_T_3,          // 9 
		MANDIBLE_R_T_4,          // 10
		INNER_MOUTH,             // 11
		INNER_JAW_BOTTOM,        // 12
		INNER_JAW_TOP,           // 13
		JAW,                     // 14
		MANDIBLE_L_B_1,          // 15
		MANDIBLE_L_B_2,          // 16
		MANDIBLE_L_B_3,          // 17
		MANDIBLE_L_B_4,          // 18
		MANDIBLE_R_B_1,          // 19
		MANDIBLE_R_B_2,          // 20
		MANDIBLE_R_B_3,          // 21
		MANDIBLE_R_B_4,          // 22
		MANDIBLE_L_T_1,          // 23
		MANDIBLE_L_T_2,          // 24
		MANDIBLE_L_T_3,          // 25
		MANDIBLE_L_T_4,          // 26
		BIP01_L_CLAVICLE,        // 27
		BIP01_L_UPPERARM,        // 28
		BIP01_L_FOREARM,         // 29
		BIP01_L_HAND,            // 30
		BIP01_L_FINGER0,         // 31
		BIP01_L_FINGER01,        // 32
		BIP01_L_FINGER02,        // 33
		BIP01_L_FINGER1,         // 34
		BIP01_L_FINGER11,        // 35
		BIP01_L_FINGER12,        // 36
		BIP01_L_FINGER2,         // 37
		BIP01_L_FINGER21,        // 38
		BIP01_L_FINGER22,        // 39
		BIP01_L_FINGER3,         // 40
		BIP01_L_FINGER31,        // 41
		BIP01_L_FINGER32,        // 42
		BIP01_L_FORETWIST,       // 43
		BIP01_LUPARMTWIST,       // 44
		BIP01_R_CLAVICLE,        // 45
		BIP01_R_UPPERARM,        // 46
		BIP01_R_FOREARM,         // 47
		BIP01_R_HAND,            // 48
		BIP01_R_FINGER0,         // 49
		BIP01_R_FINGER01,        // 50
		BIP01_R_FINGER02,        // 51
		BIP01_R_FINGER1,         // 52
		BIP01_R_FINGER11,        // 53
		BIP01_R_FINGER12,        // 54
		BIP01_R_FINGER2,         // 55
		BIP01_R_FINGER21,        // 56
		BIP01_R_FINGER22,        // 57
		BIP01_R_FINGER3,         // 58
		BIP01_R_FINGER31,        // 59
		BIP01_R_FINGER32,        // 60
		BIP01_R_FORETWIST,       // 61
		BIP01_RUPARMTWIST,       // 62
		BIP01_L_THIGH,           // 63
		BIP01_L_CALF,            // 64
		BIP01_L_FOOT,            // 65
		BIP01_L_TOE0,            // 66
		BIP01_L_TOE01,           // 67
		BIP01_R_THIGH,           // 68
		BIP01_R_CALF,            // 69
		BIP01_R_FOOT,            // 70
		BIP01_R_TOE0,            // 71
		BIP01_R_TOE01,           // 72
		BIP01_LTHIGHTWIST,       // 73
		BIP01_RTHIGHTWIST,       // 74
		BIP01_TAIL,              // 75
		BIP01_TAIL1,             // 76
		BIP01_TAIL2,             // 77
		BIP01_TAIL3,             // 78
		BIP01_TAIL4,             // 79
		BIP01_TAIL5,             // 80
		BIP01_TAIL6,             // 81
		BIP01_TAIL7,             // 82
		BIP01_TAIL8,             // 83
		BIP01_TAIL9,             // 84
		BIP01_TAIL10,            // 85
		BIP01_TAIL11,            // 86
		BACK_TUBE_LEFT01,        // 87
		BACK_TUBE_LEFT02,        // 88

		MAX_THIRD_PERSON_BONES_IDS			   
	};							

	enum FirstPersonBoneID
	{
		MAX_FIRST_PERSON_BONES_IDS
	};

	enum IKChains
	{
		LEFT_LEG,
		RIGHT_LEG,
		TAIL,

		NUM_IK_CHAINS
	};

	// Public functions...
#if defined(ASURA_TOOLS)
	static void	AddFilesToExport(const bool bFirstPerson, const Asura_Hash_ID uBlueprintHash, List<Asura_String>& xNameList);
#endif	// defined(ASURA_TOOLS)

	// Accessor functions...
	static Asura_Hash_ID	GetMarkerPointHashID(const MarkerPointType eID)	{ return(s_auMarkerPointHashIDs[eID]); }
	static int				GetFootBoneID( u_int uFootIndex, bool bFirstPerson ) { Asura_Assert( !bFirstPerson, "No FP predalien" ); Asura_Assert( uFootIndex < AVP_ClientEntity_Actor::MAX_NUM_FEET, "Bad foot index" ); return s_aiThirdPersonFootBoneID[ uFootIndex ]; }

	static const int* 		GetThirdPersonChainsArray(const u_int uIndex)		{ return(s_apiThirdPersonChainsArray[uIndex]); }
	static const int*		GetTailChainArray() { return s_aiTailChainBonesArray; }

protected:
	// Protected variables...
	static const Asura_Hash_ID	s_auMarkerPointHashIDs[NUM_MARKER_POINT_IDS];
	static const int			s_aiThirdPersonFootBoneID[ AVP_ClientEntity_Actor::MAX_NUM_FEET ];

	static const int 			s_aiThirdPersonLeftLegChainBonesArray[ uAVP_PREDALIEN_NUM_IK_BONES_PER_LIMB + 1 ];
	static const int 			s_aiThirdPersonRightLegChainBonesArray[ uAVP_PREDALIEN_NUM_IK_BONES_PER_LIMB + 1 ];
	static const int 			s_aiTailChainBonesArray[ uAVP_PREDALIEN_NUM_IK_BONES_PER_TAIL + 1  ];
	static const int* 			s_apiThirdPersonChainsArray[ NUM_IK_CHAINS ];


	// Protected functions...

private:
	// Private functions...
	AVP_Predalien_Description(void);
	AVP_Predalien_Description(const AVP_Predalien_Description& xOther);
	~AVP_Predalien_Description();

	AVP_Predalien_Description&	operator=(const AVP_Predalien_Description& xOther);

};   // End of class 'AVP_Predalien_Description'.


#endif   // ifndef __AVP_PREDALIEN_DESCRIPTION_H__


/// End of file. ///
