
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_Predator_Focus.h
// Author : Anthony Gregson
// Date   : 31/01/08
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __AVP_PREDATOR_FOCUS_H__
#define __AVP_PREDATOR_FOCUS_H__

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////
#include "Core/Asura_Maths.h"
#include "Actor/Vertebrates/AVP_Vertebrate_Focus.h"
#include "Actor/Vertebrates/Predator/AVP_Predator_LockOn.h"

////////////////////////////////////////////////////////////////////////////////
// Forward class declarations...
////////////////////////////////////////////////////////////////////////////////

class Asura_Attractor_Compiled;
class AVP_ClientActor_Predator;
class AVP_ClientEntity_Actor;

////////////////////////////////////////////////////////////////////////////////
// Class declaration...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_Predator_Focus
// Author       : Anthony Gregson
// Date         : 31/01/08
////////////////////////////////////////////////////////////////////////////////

class AVP_Predator_Focus : public AVP_Vertebrate_Focus
{
public:
	// Public functions...
	AVP_Predator_Focus(AVP_ClientActor_Predator& xOwner);
	virtual ~AVP_Predator_Focus();

	// Virtual manipulation functions...
	virtual void Update();
	virtual void Render();

	// Accessor functions...
	virtual bool HasValidLockOnTarget() const	{ return(AVP_Predator_LockOn::GetSingleton().IsLockedOn() && PARENT::HasValidLockOnTarget()); }
	virtual Asura_Guid GetMeleeTarget() const { return m_bTargetIsLockOn ? ASURA_GUID_UNREGISTERED : m_uTargetEnemyGuid; }
	const Asura_Vector_3& GetJumpDestinationPosition(void) const { return m_xJumpDestinationPosition; }
	int GetJumpVariation() const { return m_iJumpVariation; }
	bool GetJumpAvailable() const { return m_bJumpAvailable; } // In the initial design, this should be ignored if we have a valid melee target.
	bool IsJumpNoisy() const { return m_bNoisyJump; }
	virtual void PostCreateInitialise();

protected:
	// Protected functions...
	void UpdateTargets();
	void UpdateJumpTargetValues(); // updates console commands

	void UpdateEnemyTarget( Asura_Vector_3& xPosition, Asura_Matrix_3x3& xOrientation );
	void UpdateJumpTarget( Asura_Vector_3& xPosition, Asura_Matrix_3x3& xOrientation );

	// Protected variables...
	Asura_Vector_3 m_xJumpDestinationPosition;	// The destination position over which a jump takes place.
	int		m_iJumpVariation;					// If the destination is valid, this is the jump variation we need to get there.

	u_int	m_uMeleeIndicatorPFXID;
	u_int	m_uJumpIndicatorPFXID;
	u_int	m_uCantJumpIndicatorPFXID;
	u_int	m_uNoisyJumpIndicatorPFXID;

	float	m_fJumpDistance; 
	float	m_fJumpHeight; 
	float	m_fJumpAttractorStickyTimer; // Countdown for the last valid jump target.

	// following used to be flags, but need to work with command console...
	bool	m_bDestinationPositionValid;	// Set if we've got a destination position, but not at a valid jump target (i.e. out of range).
	bool	m_bJumpAvailable;				// Whether the jump position is currently valid. May need replacing with types, similar to the alien handler, if just trying to land on something isn't enough.
	bool	m_bInvalidDistance;
	bool	m_bInvalidHeight;
	bool	m_bNoisyJump;

private:
	// Private functions...
	bool TestEntityAgainstScreenArea(const AVP_ClientEntity_Actor& xEntity, float fLockRange, const Asura_Vector_3& xCameraPosition, const Asura_Vector_2& xPreviousBestScreenPos, Asura_Vector_2& xEntityScreenPos) const; // The final parameter is output only.

	// Private helper class for the jump target tests. No longer just for the attractor callback.
	class JumpTargetData
	{
	public:
		JumpTargetData(AVP_Predator_Focus& xFocus, const Asura_Vector_3& xOrigin, const Asura_Vector_3& xDirection)
			: m_xFocus(xFocus)
			, m_xOrigin(xOrigin)
			, m_xDirection(xDirection)
			, m_xJumpDestinationPosition(Asura_Maths::ZeroVector3)
			, m_iJumpVariation(0)
			, m_bDestinationPositionValid(false)
			, m_bJumpAvailable(false)
		{
		}

		AVP_Predator_Focus& m_xFocus;
		const Asura_Vector_3& m_xOrigin;
		const Asura_Vector_3& m_xDirection;
		Asura_Vector_3 m_xJumpDestinationPosition;
		int m_iJumpVariation;
		bool m_bDestinationPositionValid;
		bool m_bJumpAvailable;
	};

	// Env hook callback
	static void CheckForJump_Callback(void* pData, const Asura_Attractor_Compiled* const pxAttractor);
	void TestJumpAttractor(const Asura_Attractor_Compiled& xAttractor, JumpTargetData& xJTData);

	// Private variables...
	float m_fMaxLOSRange; // The maximum range for LOS tests.
	float m_fMinJumpDistance; // The minimum distance before a jump becomes available.
	float m_fMaxJumpDistance; // The maximum jump distance on the same level as (or lower than) the predator
	float m_fMaxJumpHeight; // The maximum jump height if jumping directly up.
	float m_fMaxJumpAttractorRenderingDistance; // The maximum distance for rendering a too-distant jump.
	float m_fJumpAttractorStickyTimeLimit; // Amount of time the last valid jump target needs to persist for.

	// Private typedefs..
	typedef AVP_Vertebrate_Focus PARENT;
};   // End of class 'AVP_Predator_Focus'.


#endif   // ifndef __AVP_PREDATOR_FOCUS_H__


/// End of file. ///
