
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_Predator_HUDHandler.h
// Author : Chris Wallace
// Date   : 2-Nov-2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __AVP_PREDATOR_HUDHANDLER_H__
#define __AVP_PREDATOR_HUDHANDLER_H__

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Actor/AVP_Actor_CameraHandler.h"
#include "Actor/Vertebrates/AVP_Vertebrate_HUDHandler.h"

#include "GUIMenu/AVP_GUIMenu_System.h"


////////////////////////////////////////////////////////////////////////////////
// Defines...
////////////////////////////////////////////////////////////////////////////////

#define	MAX_HUD_ENERGY_ELEMENTS 5


////////////////////////////////////////////////////////////////////////////////
// Forward class declarations...
////////////////////////////////////////////////////////////////////////////////

class AVP_ClientEntity_Actor;
class AVP_ClientActor_Predator;

////////////////////////////////////////////////////////////////////////////////
// Class declaration...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Variables...
////////////////////////////////////////////////////////////////////////////////
// 
// #define Tri_texture_width	40

// class AVP_HUD_BG_Node
// {
// public:
// 	float fValue;
// 	float fGlow;
// 
// 	AVP_HUD_BG_Node() 
// 		{
// 			fValue = 0.0f;
// 			fGlow = 0.0f;
// 		}
// 
// 	void RemoveGlow()			{ fGlow = 0.0f;  }
// 
// 	void AddGlow()				{ AddGlow(1.0f); }
// 	void AddGlow(float fVal)	{ fGlow = fVal; }
// };

// CAH 22/4/09
// Small data structure to hold information needed about current targets in focus mode, 
// Grouped here to save clutter in the AVP_Predator_HUDHandler class
struct PredHudTargetInfo
{
	int		m_iSpecies;
	int		m_iWeapon;
	int		m_iThreatNear;
	int		m_iThreatFar;
	bool	m_bMotionTracker;
	bool	m_bHieghtenedSences;
};

////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_Predator_HUDHandler
// Author       : Chris Wallace
// Date         : 2 Nov-2006
//
// Description  : Class to the handle the Predator specific HUD manipulation.
////////////////////////////////////////////////////////////////////////////////

class AVP_Predator_HUDHandler : public AVP_Vertebrate_HUDHandler
{
public:
	// Public functions...
	explicit AVP_Predator_HUDHandler ( AVP_ClientEntity_Actor& xClient );
	
	virtual ~AVP_Predator_HUDHandler ();

	// Overridden functions...
	virtual	void	Update	( const AVP_ClientEntity_Actor& xClient );
	// 	virtual void	Render	( const AVP_ClientEntity_Actor& xClient );
	virtual void	Reset	();

	// Event functions...
	virtual void 	OnBeginTargetedJump(void);
	virtual void 	OnEndTargetedJump(void);
		
	virtual void	OnBeginZoomingIn(const u_int uZoomLevel);
	virtual void	OnBeginZoomingOut(const u_int uZoomLevel);

	virtual void	OnBeginRecon(void);
	virtual void	OnEndRecon(void);
	virtual void	OnSelectReconTarget(void);

	static void  SetLeapPointInRange( bool bLeapPointInRange ){ s_bLeapPointInRange = bLeapPointInRange; }
	static void  SetReconTargetAvailable( bool bReconTargetAvailable ){ s_bReconTargetAvailable = bReconTargetAvailable; }
	static void  SetReconTargetSelected( bool bReconTargetSelected ){ s_bReconTargetSelected = bReconTargetSelected; }

	static bool* GetShowVisionSelectorConsoleVarPtr();	

// Create console vars
#if defined(ASURA_SUPPORT_COMMAND_CONSOLE)
	static void CreateConsoleVars();
#endif	// defined(ASURA_SUPPORT_COMMAND_CONSOLE)	

protected:
	// Protected variables...
	
	Asura_Vector_2	m_xNoEnergyPos;
	Asura_Vector_2	m_xFullEnergyPos;

	u_int			m_uMaxNumEnergyBlocks;
	float			m_fEnergyPerBlock;

	float			m_fTickerPhase;

	float			m_fPreviousEnergy; // Energy previous update

	Asura_Handle	m_iTargetedJumpSoundHandle;
	Asura_Handle	m_iTargetInfoSoundLoopHandle;

private:
	// Private typedefs...
	typedef AVP_Vertebrate_HUDHandler PARENT;

	// Private functions...
	AVP_Predator_HUDHandler  (const AVP_Predator_HUDHandler& xOther);
	AVP_Predator_HUDHandler&	operator=	(const AVP_Predator_HUDHandler& xOther);

	// 	void RenderZoomModes	( const AVP_ClientActor_Predator& xPredator );

	void UpdateZoomModes	( const AVP_ClientActor_Predator& xPredator );
	void UpdateEnergy		( const AVP_ClientActor_Predator& xPredator );
	void UpdateAmmo			( const AVP_ClientActor_Predator& xPredator );
	void UpdateTickers		();
	void UpdateVisionMode	( const AVP_ClientActor_Predator& xPredator );
	
	void UpdateTargetInfo	( Asura_Guid uGuidOfTarget );
	void SetTargetInfo		( int iSpecies, int iWeapon, int iThreatNear, int iThreatFar, bool bMotionTracker, bool bHieghtendSences );
	void OnBeginTargetInfo	();
	void OnEndTargetInfo	();

	Asura_Guid	GetPotentialPrey();
	bool		TestEntityAgainstLOS(const AVP_ClientEntity_Actor& xEntity, const Asura_Vector_3& xCameraPosition, Asura_Vector_3 xCameraDirection, Asura_Vector_2& xBestScreenPos ) const;

	static bool  s_bShowVisionSelector;
	static bool  s_bShowEnergy[ MAX_HUD_ENERGY_ELEMENTS ];
	static float s_fEnergyParametric[ MAX_HUD_ENERGY_ELEMENTS ];
	static bool  s_bLeapPointInRange;
	static bool  s_bReconTargetAvailable;
	static bool  s_bReconTargetSelected;
	static bool  s_bShowZoom[ AVP_CAMERA_NUM_ZOOM_LEVELS - 1 ];
	static bool  s_bShowRecon;
	static float s_fShowReconSound;
	static int	 s_iAmmoCounter;
	static int	 s_iCurrentVisionMode;
	static float s_fHasFocusInfoTarget;
	static PredHudTargetInfo s_xTargetInfo;

	float m_fHudInfoStickTime;
	float m_fHudInfoStickCounter;

};   // End of class 'AVP_Predator_HUDHandler'.

#endif   // ifndef __AVP_PREDATOR_HUDHANDLER_H__

/// End of file. ///
