////////////////////////////////////////////////////////////////
//
// File:		AVP_Predator_MeleeCounter_System.h
// Author:		Chris Wallace
// Date:		25-Jan-2008
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//
////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __AVP_PREDATOR_MELEECOUNTER_SYSTEM_H__
#define __AVP_PREDATOR_MELEECOUNTER_SYSTEM_H__

///////////////////////////////////////////////
// #includes
///////////////////////////////////////////////

#include "Actor/Vertebrates/AVP_Vertebrate_MeleeCounter_System.h"

///////////////////////////////////////////////

class AVP_ServerEntity_Actor;
class AVP_ServerActor_Predator;
class AVP_Predator_Focus;

/////////////////////////////////////////////////////////////////////
//
// Class		:	AVP_Predator_MeleeCounter_System
//
//////////////////////////////////////////////////////////////////////

enum AVP_PREDATOR_MELEEATTACK_TYPE
{
	AVP_PREDATOR_MELEEATTACK_TYPE_PRIMARYSLASH = NUM_AVP_MELEEATTACK_TYPES,
	AVP_PREDATOR_MELEEATTACK_TYPE_PRIMARYRELEASE,
	AVP_PREDATOR_MELEEATTACK_TYPE_PRIMARYCANCEL,
	AVP_PREDATOR_MELEEATTACK_TYPE_FOCUSATTACK, // Will probably match the pounce, but allows us to differentiate between player inputs (if they press this, but are too close, they should do a primary attack).
	AVP_PREDATOR_MELEEATTACK_TYPE_FOCUSPOUNCE,
	AVP_PREDATOR_MELEEATTACK_TYPE_PRIMARYFINISHER,
	AVP_PREDATOR_MELEEATTACK_TYPE_COUNTERGRAPPLE_FATAL,
	AVP_PREDATOR_MELEEATTACK_TYPE_COUNTERGRAPPLE_MINOR,
	AVP_PREDATOR_MELEEATTACK_TYPE_COUNTERHIT_FATAL,
	AVP_PREDATOR_MELEEATTACK_TYPE_COUNTERHIT_REMOVELIMB,
	AVP_PREDATOR_MELEEATTACK_TYPE_COUNTERTHROW,
	AVP_PREDATOR_MELEEATTACK_TYPE_PICKUPENEMY,
	AVP_PREDATOR_MELEEATTACK_TYPE_STEALTHPICKUPENEMY,
	AVP_PREDATOR_MELEEATTACK_TYPE_POWERATTACK, // AI way to trigger the power attack without charging up first.
	AVP_PREDATOR_MELEEATTACK_TYPE_GRAPPLE_PREDALIEN1, 
	AVP_PREDATOR_MELEEATTACK_TYPE_GRAPPLE_PREDALIEN2, 
	AVP_PREDATOR_MELEEATTACK_TYPE_GRAPPLE_PREDALIENKILL, 
	AVP_PREDATOR_MELEEATTACK_TYPE_PREDALIEN_KNOCKDOWN,
	AVP_PREDATOR_MELEEATTACK_TYPE_PREDALIEN_DONT_KNOCKDOWN,

	NUM_AVP_PREDATOR_MELEEATTACK_TYPES,
};

class AVP_Predator_MeleeCounter_System : public AVP_Vertebrate_MeleeCounter_System
{
public: 
	AVP_Predator_MeleeCounter_System(AVP_ServerActor_Predator& xOwner);
	virtual ~AVP_Predator_MeleeCounter_System();

	//virtual bool HandleMessage( const Asura_Message* pxMessage );
	virtual void Update( void );

	virtual bool StartMeleeMove(u_int uType);
	virtual bool PerformCounter();

	bool GetCanPounce(const AVP_ServerEntity_Actor* pxTarget) const;

	static const char* GetLeftQuickAttackBaseParameterName() { return s_szLeftQuickAttackBaseParameterName; }
	static const char* GetRightQuickAttackBaseParameterName() { return s_szRightQuickAttackBaseParameterName; }

	virtual bool InsidePounceRange() const;

#if defined(ASURA_SUPPORT_COMMAND_CONSOLE)
	static int	s_iConsoleVarsCount;
	static void CreateConsoleVars();
#endif	// defined(ASURA_SUPPORT_COMMAND_CONSOLE)

protected:
	virtual void CheckForAndDealMeleeDamage();
	virtual void ApplyMeleeDamage(Asura_Guid uDamagedGuid);
	virtual void ApplyGrappleDamage() const;

	virtual bool GetAttackTypeCanCounter(u_int uType) const; // Returns true if a particular melee request can also be used to counter an incoming attack.

	// Go off and get a move hash relevant to the species and weapon
	virtual Asura_Hash_ID GetMeleeMoveHash(u_int uType) const;
	virtual u_int PickMeleeMoveVariation(Asura_Hash_ID uMoveHash) const;

	// Functions for counters by each species, split up to keep the code clearer. Uses the move we're currently doing to pick the correct counter for that species.
	//virtual bool GetCanCounter_AlienCounter(const AVP_ServerEntity_Actor& xEnemy, const u_int uQueuedCounterType, Asura_Hash_ID& uMoveHash, u_int& uNewAttackType, bool& bMakeEnemyIntoPuppet, Asura_Hash_ID& uGrappleSetBlueprintParam) const;
	//virtual bool GetCanCounter_HumanCounter(const AVP_ServerEntity_Actor& xEnemy, const u_int uQueuedCounterType, Asura_Hash_ID& uMoveHash, u_int& uNewAttackType, bool& bMakeEnemyIntoPuppet, Asura_Hash_ID& uGrappleSetBlueprintParam) const;
	//virtual bool GetCanCounter_PredatorCounter(const AVP_ServerEntity_Actor& xEnemy, const u_int uQueuedCounterType, Asura_Hash_ID& uMoveHash, u_int& uNewAttackType, bool& bMakeEnemyIntoPuppet, Asura_Hash_ID& uGrappleSetBlueprintParam) const;

	virtual float GetCurrentMaxCounterDistance() const;

	float GetCloakedEnergyDrain( u_int uType ) const;
	virtual float GetAttackDamage() const;
	virtual Asura_Hash_ID GetAttackDamageType() const;
	virtual float GetAttackImpulseStrength() const;
	virtual void GetAttackDirection(const AVP_ServerEntity_PhysicalObject* pxTarget, Asura_Vector_3& xDirection) const;
	virtual int GetAttackBoneHit(Asura_Guid uDamagedGuid) const;
	virtual float GetGrappleDamage() const;
	virtual bool IsAttackUnblockable() const;
	virtual bool GetAttackOnlyHitsCurrentTarget() const;
	virtual bool GetAttackIsRightHanded() const;

	virtual Asura_Hash_ID GetAttackGrappleSetBlueprintParamID() const;

	virtual bool GetSendIncomingMeleeMessage(u_int uAttackType) const;

	virtual bool IsCurrentTargetInSuckRange() const;


	bool GetCanPounce() const;
	bool GetTargetTooCloseToPounce() const;

	virtual const Asura_Blueprint* GetExtraMeleeDataBlueprint() const;
	virtual bool GetQuickAttackBaseParameterNames(const char*& szLeftAttackName, const char*& szRightAttackName) const;
	virtual Asura_Hash_ID GetPounceBlueprintType() const;

	static const char* s_szLeftQuickAttackBaseParameterName;
	static const char* s_szRightQuickAttackBaseParameterName;

private:
	typedef AVP_Vertebrate_MeleeCounter_System PARENT;

	static bool s_bHackedTestCounterIndicatorOn;
};

#endif //__AVP_PREDATOR_MELEECOUNTER_SYSTEM_H__

/// End of file. ///
