
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_Sentrygun_AnimationHandler.cpp
// Author : Chris
// Date   : 16-Apr-2007
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

// Core
#include "Core/Asura.h"

// This file's header...
#include "Actor/Weapons/Sentrygun/AVP_Sentrygun_AnimationHandler.h"

// Asura independent header-files...
#include "Animation/Asura_Animation.h"
#include "Animation/Asura_AnimationBlend.h"
#include "Animation/Asura_AnimationController_Animation.h"
#include "Animation/Asura_AnimationController_Group.h"
#include "Blueprint/Asura_Blueprint.h"

// Project independent header-files...
#include "Actor/AVP_Actor_AnimStateData.h"
#include "Blueprint/AVP_Blueprint_HashIDs.h"
#include "Entity/PhysicalObject/Actor/Weapons/Sentrygun/AVP_ClientWeapon_Sentrygun.h"

#ifdef _DEBUG

static const char* s_szRootAimGroupName = "AimGroup";

#else

static const Asura_Hash_ID s_uRootAimGroupHash = 0xcea1805a; //"AimGroup"

#endif

AVP_Sentrygun_AnimationHandler::AVP_Sentrygun_AnimationHandler(AVP_ClientWeapon_Sentrygun& xOwner)
: PARENT( xOwner )
, m_xOwnerSentryGun(xOwner)
, m_uRootAimGroupHash( ASURA_HASH_ID_UNSET )
, m_uStateAnim( ASURA_HASH_ID_UNSET )
, m_uAimRightLeftAnimHash( ASURA_HASH_ID_UNSET )
, m_uAimDownUpAnimHash( ASURA_HASH_ID_UNSET )
, m_uAimRightLeftAACAHash( ASURA_HASH_ID_UNSET )
, m_uAimDownUpAACAHash( ASURA_HASH_ID_UNSET )
, m_fMaxAimRightLeft( Asura_Maths::DegToRad( 90.0f ) )
, m_fMaxAimDownUp( Asura_Maths::DegToRad( 30.0f ) )
, m_pxCurrentStateData( NULL )
{

}

void AVP_Sentrygun_AnimationHandler::UpdateAnimStateData( const AVP_Actor_AnimStateData* pxAnimDataLBSlow, const AVP_Actor_AnimStateData* pxAnimDataLBFast, const AVP_Actor_AnimStateData* pxAnimDataUBSlow, const AVP_Actor_AnimStateData* pxAnimDataUBFast, const bool bLoop, const float fBlendLBSpeed, const float fBlendUBSpeed, const float fOSMoveDirAngle, const float fTweenTime, const bool bSynch )
{
	m_pxCurrentStateData = pxAnimDataLBSlow;
	Asura_Assert( pxAnimDataLBFast == NULL, "Sentryguns don't use speed blends" );
	if ( pxAnimDataLBSlow == NULL )
	{
		m_pxAnimation->DestroyController( m_uStateAACAHash );
		SetAimAnimations( ASURA_HASH_ID_UNSET, ASURA_HASH_ID_UNSET, 0.0f );
	}
	else
	{
		Asura_Hash_ID uAnimHash = pxAnimDataLBSlow->GetAnimHash();
		if ( m_uStateAnim != uAnimHash || !bSynch )
		{
			Asura_AnimationController_Group* pxRootStatesGroup = m_pxAnimation->GetControllerGroupFromHash( m_uRootStatesGroupHash );
			if ( pxRootStatesGroup )
			{
				u_int uAnimFlags = AAC_FLAG_ACTIVE | AAC_FLAG_ENSURE_UNIQUE_HASH | AAC_FLAG_ALLOW_DESTROY_FROM_MESSAGE | AACA_FLAG_CALCULATE_VELOCITY | AACA_FLAG_CALCULATE_MOVEMENT_OFFSET | AACA_FLAG_CALCULATE_ANGULAR_VELOCITY;

				if ( bLoop )
				{
					uAnimFlags |= AACA_FLAG_LOOPED;
				}

				Asura_AnimationBlend* pxBlend = new Asura_AnimationBlend_AnimatedTween( fTweenTime, bSynch );
				
				m_uStateAACAHash	= PlayAnimation( uAnimFlags, AVP_AAPC_GENERAL_ANIM, uAnimHash, pxBlend, pxRootStatesGroup );

				if ( m_uStateAACAHash != ASURA_HASH_ID_UNSET )
				{
					m_uStateAnim = uAnimHash;
				}
				else
				{
					m_uStateAnim = ASURA_HASH_ID_UNSET;
				}
			}
			SetAimAnimations( pxAnimDataLBSlow->GetAimDownUpAnimHash(), pxAnimDataLBSlow->GetAimRightLeftAnimHash(), fTweenTime );
		}
	}
}

void AVP_Sentrygun_AnimationHandler::PauseCurrentAnimState()
{
	Asura_AnimationController_Animation* pxAACA = m_pxAnimation->GetControllerAnimationFromHash( m_uStateAACAHash );
	if ( pxAACA != NULL && pxAACA->IsPlaying() )
	{
		pxAACA->Pause();
	}
}

void AVP_Sentrygun_AnimationHandler::PlayCurrentAnimState()
{
	Asura_AnimationController_Animation* pxAACA = m_pxAnimation->GetControllerAnimationFromHash( m_uStateAACAHash );
	if ( pxAACA != NULL && !pxAACA->IsPlaying() )
	{
		pxAACA->Play();
	}
}

bool AVP_Sentrygun_AnimationHandler::IsCurrentAnimStateFinished() const
{
	Asura_AnimationController_Animation* pxAACA = m_pxAnimation->GetControllerAnimationFromHash( m_uStateAACAHash );
	return pxAACA == NULL || pxAACA->IsFinished();
}

void AVP_Sentrygun_AnimationHandler::SetCurrentAnimStateLooped( const bool bSet )
{
	Asura_AnimationController_Animation* pxAACA = m_pxAnimation->GetControllerAnimationFromHash( m_uStateAACAHash );
	if ( pxAACA != NULL )
	{
		if ( bSet )
		{
			pxAACA->SetControllerFlags( pxAACA->GetControllerFlags() | AACA_FLAG_LOOPED );
		}
		else
		{
			pxAACA->SetControllerFlags( pxAACA->GetControllerFlags() & ~AACA_FLAG_LOOPED );
		}
	}
}

bool AVP_Sentrygun_AnimationHandler::SetDamagedSkin()
{
	if(IsDamaged())
	{
		return(false);
	}
	else
	{
		SetDamaged(true);
		UpdateSkin();
		return(true);
	}
}

void AVP_Sentrygun_AnimationHandler::SetAimAnimations( Asura_Hash_ID uAimDownUpHash, Asura_Hash_ID uAimRightLeftHash, const float fTweemTime )
{
	AVP_ClientWeapon_Sentrygun* pxClient = AVP_ClientWeapon_Sentrygun::FindEntity( m_uOwnerGuid );
	Asura_Assert( pxClient, "Unable to find Sentry gun" );
	if ( pxClient )
	{
		if ( ( uAimRightLeftHash != m_uAimRightLeftAnimHash ) || ( uAimDownUpHash != m_uAimDownUpAnimHash ) )
		{
			Asura_AnimationController_Group* pxRootAimGroup = m_pxAnimation->GetControllerGroupFromHash( m_uRootAimGroupHash );
			if ( pxRootAimGroup != NULL )
			{
				const u_int								uGroupFlags			= AAC_FLAG_ACTIVE | AAC_FLAG_ALLOW_DESTROY_FROM_MESSAGE | AAC_FLAG_ENSURE_UNIQUE_HASH;
				const u_int								uPriority			= AVP_AAPC_GENERAL_ANIM;
				Asura_AnimationBlend*					pxAimGroupBlend		= new Asura_AnimationBlend_Tween( fTweemTime, false );
				pxAimGroupBlend->SetDeltasUseBlends( true );

#ifdef _DEBUG
				Asura_AnimationController_Group*		pxGroup				= m_pxAnimation->CreateControllerGroup( "Aim Group", uGroupFlags, uPriority, pxAimGroupBlend, pxRootAimGroup );
#else
				Asura_AnimationController_Group*		pxGroup				= m_pxAnimation->CreateControllerGroup( 0x8bccbbe4, uGroupFlags, uPriority, pxAimGroupBlend, pxRootAimGroup ); // Asura_GetHashID( "Aim Group" )
#endif
				const u_int								uAnimationFlags		= AAC_FLAG_ACTIVE | AAC_FLAG_ENSURE_UNIQUE_HASH | AAC_FLAG_ALLOW_DESTROY_FROM_MESSAGE | AACA_FLAG_CALCULATE_MOVEMENT_OFFSET;

				Asura_AnimationController_Animation*	pxRightLeftAACA		= NULL;
				Asura_AnimationBlend*					pxRightLeftBlend	= new Asura_AnimationBlend (1.0f);
#ifdef _DEBUG
				pxRightLeftAACA	= m_pxAnimation->CreateControllerAnimation( "Aim Right To Left", uAnimationFlags, uPriority, uAimRightLeftHash == ASURA_HASH_ID_UNSET ? m_uAimRightLeftAnimHash : uAimRightLeftHash, pxRightLeftBlend, pxGroup );
#else
				pxRightLeftAACA	= m_pxAnimation->CreateControllerAnimation( uAnimationFlags, uPriority, uAimRightLeftHash == ASURA_HASH_ID_UNSET ? m_uAimRightLeftAnimHash : uAimRightLeftHash, pxRightLeftBlend, pxGroup );
#endif
				Asura_Assertf( pxRightLeftAACA != NULL, ( "Unable to create look anim with hash ID 0x%08x", uAimRightLeftHash ) );
				if ( pxRightLeftAACA != NULL )
				{
					pxRightLeftAACA->Stop();
					if ( uAimRightLeftHash == ASURA_HASH_ID_UNSET )
					{
						pxRightLeftAACA->SetParametricTimer( 0.5f, false );
						m_uAimRightLeftAACAHash = m_uAimRightLeftAnimHash = ASURA_HASH_ID_UNSET;
					}
					else
					{
						m_uAimRightLeftAnimHash = uAimRightLeftHash;
						m_uAimRightLeftAACAHash = pxRightLeftAACA->GetControllerHash();
					}
				}
				else
				{
					m_uAimRightLeftAACAHash = m_uAimRightLeftAnimHash = ASURA_HASH_ID_UNSET;
					delete pxRightLeftBlend;
				}

				Asura_AnimationController_Animation*	pxDownUpAACA	= NULL;
				Asura_AnimationBlend*					pxDownUpBlend	= new Asura_AnimationBlend (1.0f);
#ifdef _DEBUG
				pxDownUpAACA	= m_pxAnimation->CreateControllerAnimation( "Aim Down To Up", uAnimationFlags, uPriority, uAimDownUpHash == ASURA_HASH_ID_UNSET ? m_uAimDownUpAnimHash : uAimDownUpHash, pxDownUpBlend, pxGroup );
#else
				pxDownUpAACA	= m_pxAnimation->CreateControllerAnimation( uAnimationFlags, uPriority, uAimDownUpHash == ASURA_HASH_ID_UNSET ? m_uAimDownUpAnimHash : uAimDownUpHash, pxDownUpBlend, pxGroup );
#endif
				Asura_Assertf( pxDownUpAACA != NULL, ( "Unable to create look anim with hash ID 0x%08x", uAimDownUpHash ) );
				if ( pxDownUpAACA != NULL )
				{
					pxDownUpAACA->Stop();
					if ( uAimDownUpHash == ASURA_HASH_ID_UNSET )
					{
						pxDownUpAACA->SetParametricTimer( 0.5f, false );
						m_uAimDownUpAACAHash = m_uAimDownUpAnimHash = ASURA_HASH_ID_UNSET;
					}
					else
					{
						m_uAimDownUpAnimHash = uAimDownUpHash;
						m_uAimDownUpAACAHash = pxDownUpAACA->GetControllerHash();
					}
				}
				else
				{
					m_uAimDownUpAACAHash = m_uAimDownUpAnimHash = ASURA_HASH_ID_UNSET;
					delete pxDownUpBlend;
				}
			}
		}
	}
}

void AVP_Sentrygun_AnimationHandler::Update()
{
	PARENT::Update();

	if ( m_uAimDownUpAACAHash != ASURA_HASH_ID_UNSET || m_uAimRightLeftAACAHash != ASURA_HASH_ID_UNSET )
	{
		AVP_ClientWeapon_Sentrygun* pxClient = AVP_ClientWeapon_Sentrygun::FindEntity( m_uOwnerGuid );
		Asura_Assert( pxClient, "Unable to find Sentry gun" );
		if ( pxClient )
		{
			Asura_AVector3 xAimDir;
			pxClient->GetAimDirection( xAimDir );
			// get aim dir in object space
			Asura_Vector_3 xAimDir_3( xAimDir );
			xAimDir = pxClient->GetCurrentOrientation().ApplyTranspose( xAimDir_3 );

			if ( m_uAimDownUpAACAHash != ASURA_HASH_ID_UNSET )
			{
				const float fAimX = Asura_Maths::ArcSine( xAimDir.GetY() );
				float fLookDownToUpParametric = m_fMaxAimDownUp == 0.0f ? 0.5f : Asura_Maths::ClampToRange( ( -fAimX / ( 2.0f * m_fMaxAimDownUp ) ) + 0.5f, 0.0f, 1.0f );
				Asura_AnimationController_Animation* pxAACA = m_pxAnimation->GetControllerAnimationFromHash( m_uAimDownUpAACAHash );
				if ( pxAACA != NULL )
				{
					pxAACA->SetParametricTimer( fLookDownToUpParametric, false );
				}
			}

			if ( m_uAimRightLeftAACAHash != ASURA_HASH_ID_UNSET )
			{
				const float fAimY = Asura_Maths::ArcTangent( xAimDir.GetX(), xAimDir.GetZ() );
				float fLookRightToLeftParametric = m_fMaxAimRightLeft == 0.0f ? 0.5f : Asura_Maths::ClampToRange( ( -fAimY / ( 2.0f * m_fMaxAimRightLeft ) ) + 0.5f, 0.0f, 1.0f );
				Asura_AnimationController_Animation* pxAACA = m_pxAnimation->GetControllerAnimationFromHash( m_uAimRightLeftAACAHash );
				if ( pxAACA != NULL )
				{
					pxAACA->SetParametricTimer( fLookRightToLeftParametric, false );
				}
			}
		}
	}
}

void AVP_Sentrygun_AnimationHandler::Initialise( void )
{
	PARENT::Initialise();
	
	Asura_AnimationBlend*	pxAimGroupBlend	= new Asura_AnimationBlend(1.0f);

	Asura_Assert( m_uRootAimGroupHash == ASURA_HASH_ID_UNSET, "Already got an aim root group?  Shouldn't have one.");

#ifdef _DEBUG
	Asura_AnimationController_Group* pxRootAimGroup = m_pxAnimation->CreateControllerGroup( s_szRootAimGroupName, AAC_FLAG_ACTIVE | AACG_FLAG_DONT_PASS_DESTROY_TO_CHILDREN, AVP_AAPC_LOOK_RL_GROUP_TP, pxAimGroupBlend );
#else
	Asura_AnimationController_Group* pxRootAimGroup = m_pxAnimation->CreateControllerGroup( s_uRootAimGroupHash, AAC_FLAG_ACTIVE | AACG_FLAG_DONT_PASS_DESTROY_TO_CHILDREN, AVP_AAPC_LOOK_RL_GROUP_TP, pxAimGroupBlend );
#endif

	Asura_Assert( pxRootAimGroup != NULL, "Did not create root sentrygun aim animation group" );
	if ( pxRootAimGroup == NULL )
	{
		delete pxAimGroupBlend;
	}
	else
	{
		m_uRootAimGroupHash = pxRootAimGroup->GetControllerHash();
	}
}


/// End of file. ///
