
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_Animation_BodyRig.cpp
// Author : Will Plano
// Date   : 31/01/2008
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

#include "Animation/AVP_Animation_BodyRig.h"

#include "Animation/Asura_Animation.h"
#include "Animation/Asura_AnimationBlend.h"
#include "Animation/Asura_AnimationController_Animation.h"
#include "Animation/Asura_AnimationController_Group.h"
#include "Core/Asura_Timers.h"
#include "Debugging/Asura_DebugLineRender.h"
#include "Entity/Asura_PosAndOriInterpolator.h"
#include "Entity/PhysicalObject/Moves/Asura_Moves.h"
#include "Entity/PhysicalObject/Moves/Asura_Movement_System.h"
#include "FacialAnimation/Asura_FacialAnimation_System.h"
#include "FacialAnimation/Asura_FacialAnimation_AnimationHandler.h"
#include "Interface/Asura_CommandConsole.h"
#include "Physics/Asura_Physics.h"
#include "Rendering/Asura_Render.h"

#include "Actor/AVP_Actor_AnimationHandler.h"
#include "Actor/Vertebrates/AVP_Vertebrate_Description.h"
#include "Core/AVP_Interpolator.h"
#include "Core/AVP_Game_Core.h"
#include "Core/AVP_Settings.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/AVP_ClientActor_Vertebrate.h"
#include "Moves/AVP_Moves.h"

DVSVAR bool		dvs_bRenderDebugOutput		= false; // render debug arrows
DVSVAR float	dvs_fTurnIdleTime			= 1.0f; // a second pause before turning.

DEBUGVAR float		dbg_fStartTurn90Percentage	= 0.8f; // if we're looking more than this much, start turning immediately
DEBUGVAR float		dbg_fStartTurn45Angle		= Asura_Maths::DegToRad( 30.0f );

DEBUGVAR float		dbg_fTPTurnTweenTime		= 0.2f;
DEBUGVAR float		dbg_fHUDTurnTweenTime		= 0.05f;

DEBUGVAR bool dbg_bDeactivateTurnAnims = false;

//DEBUGVAR float dbg_fHeadRandomDragX				= 2.0f;
//DEBUGVAR float dbg_fHeadRandomSpringConstantX	= 1.0f;

//DEBUGVAR float dbg_fHeadRandomDragY				= 2.0f;
//DEBUGVAR float dbg_fHeadRandomSpringConstantY	= 1.0f;

DEBUGVAR u_int dbg_uUpdateTimersPriority		= 0x00001000u;
DEBUGVAR u_int dbg_uUpdateBoneDataPriority		= 0x00001000u; // this had better be before the Asura_FacialAnimation gets done

DVSVAR float dvs_fEyeNaturalFocusAngle	= 0.0f;
DVSVAR float dvs_fEyeRestClose			= 0.0f;
DVSVAR float dvs_fEyeSquint				= 0.0f;
DVSVAR float dvs_fEyeBalance			= 0.0f;

DVSVAR float dvs_fRLDragTweenInTime		= 0.175;
DVSVAR float dvs_fDUDragTweenInTime		= 0.175;
DVSVAR float dvs_fStrafeDragTweenInTime	= 0.2f;
DVSVAR float dvs_fRLDragTweenOutTime		= 0.175;
DVSVAR float dvs_fDUDragTweenOutTime		= 0.175;
DVSVAR float dvs_fStrafeDragTweenOutTime	= 0.2f;

DVSVAR Asura_Guid dvs_uDebugEyeGuid	= ASURA_GUID_UNREGISTERED;

#if defined(ASURA_SUPPORT_COMMAND_CONSOLE)
u_int AVP_Animation_BodyRig::s_uConsoleVarReferenceCount = 0;
#endif

AVP_Animation_BodyRig::AVP_Animation_BodyRig( AVP_ClientEntity_Actor& xOwnerActor )
: PARENT( xOwnerActor.GetShapeInstance().GetAnimation(), dbg_uUpdateTimersPriority, dbg_uUpdateBoneDataPriority )
, m_xOwnerActor( xOwnerActor )
, m_pxAimDirInterpolator( NULL )
, m_pxLookAtPosInterpolator( NULL )
, m_uRLAimGroupHash( ASURA_HASH_ID_UNSET )
, m_uDUAimGroupHash( ASURA_HASH_ID_UNSET )
, m_uRLDragGroupHash( ASURA_HASH_ID_UNSET )
, m_uDUDragGroupHash( ASURA_HASH_ID_UNSET )
, m_uStrafeDragGroupHash( ASURA_HASH_ID_UNSET )
, m_uTurn45LeftAnimHash( ASURA_HASH_ID_UNSET )
, m_uTurn45RightAnimHash( ASURA_HASH_ID_UNSET )
, m_uTurn90LeftAnimHash( ASURA_HASH_ID_UNSET )
, m_uTurn90RightAnimHash( ASURA_HASH_ID_UNSET )
, m_uTurnAACAHash( ASURA_HASH_ID_UNSET )
, m_fCurrentTurnAngle( 0.0f )
, m_fCurrentTurnAngVel( 0.0f )
, m_uAimDownUpAACAHash( ASURA_HASH_ID_UNSET )
, m_uAimDownUpDragAACAHash( ASURA_HASH_ID_UNSET )
, m_uAimRightLeftAACAHash( ASURA_HASH_ID_UNSET )
, m_uAimRightLeftDragAACAHash( ASURA_HASH_ID_UNSET )
, m_uStrafeDragAACAHash( ASURA_HASH_ID_UNSET )
, m_uAimDownUpAnimHash( ASURA_HASH_ID_UNSET )
, m_uAimDownUpDragAnimHash( ASURA_HASH_ID_UNSET )
, m_uAimRightLeftAnimHash( ASURA_HASH_ID_UNSET )
, m_uAimRightLeftDragAnimHash( ASURA_HASH_ID_UNSET )
, m_uStrafeDragAnimHash( ASURA_HASH_ID_UNSET )
, m_xAimDirection( 0.0f, 0.0f, 1.0f )
, m_xLookAtPosition( 0.0f, 0.0f, 0.0f )
, m_xNeckParentRestPoseQuatOS( Asura_Maths::IdentityQuat )
, m_xEyeParentRestPoseQuatOS( Asura_Maths::IdentityQuat )
, m_xCurrentNeckEuler( 0.0f, 0.0f, 0.0f )
, m_fIdleTimer( 0.0f )
, m_fCurrentAimRightLeftDragValue( 0.0f )
, m_fCurrentAimDownUpDragValue( 0.0f )
, m_fCurrentStrafeDragValue( 0.0f )
, m_fTargetAimRightLeftDragValue( 0.0f )
, m_fTargetAimDownUpDragValue( 0.0f )
, m_fTargetStrafeDragValue( 0.0f )
, m_fLookDownToUpParametric( 0.5f )
, m_fLookDownToUpDragParametric( 0.5f )
, m_fLookRightToLeftParametric( 0.5f )
, m_fLookRightToLeftDragParametric( 0.5f )
, m_fStrafeDragParametric( 0.5f )
, m_fMaxAimDownUp( Asura_Maths::Deg90 )
, m_fMaxAimRightLeft( Asura_Maths::Deg90 )
, m_fNeckBlend( 1.0f )
, m_fMaxNeckDownUp( Asura_Maths::DegToRad( 30.0f ) )
, m_fMaxNeckRightLeft( Asura_Maths::DegToRad( 60.0f ) )
, m_fMaxNeckTilt( Asura_Maths::DegToRad( 20.0f ) )
, m_fNeckTilt( 0.0f )
, m_fHeadRightLeftAngSpeed( Asura_Maths::DegToRad( 160.0f ) )
, m_fHeadDownUpAngSpeed( Asura_Maths::DegToRad( 60.0f ) )
, m_fHeadTiltAngSpeed( Asura_Maths::DegToRad( 30.0f ) )
, m_fHeadRandomVelocityX( 0.0f )
, m_fHeadRandomVelocityY( 0.0f )
, m_fHeadRandomDisplacementX( 0.0f )
, m_fHeadRandomDisplacementY( 0.0f )
, m_fEyeNaturalFocusAngle( 0.0f )
, m_fEyeCurrentClose( 0.0f )
, m_fEyeRestClose( 0.0f )
, m_fEyeSquint( 0.0f )
, m_fEyeBalance( 0.0f )
, m_fHalfBlinkDuration( 0.075f )
, m_fMinBlinkTime( 12.0f )
, m_fMaxBlinkTime( 30.0f )
, m_fBlinkTimer( 20.0f )
, m_eBlinkState( BLINK_OPEN )
, m_fLookAtCameraParameter( 0.0f )
, m_fLookTweenTime( 0.1f )
, m_fMinLookAtCameraTime( 0.5f )
, m_fMaxLookAtCameraTime( 2.0f )
, m_fMinLookFwdTime( 10.0f )
, m_fMaxLookFwdTime( 30.0f )
, m_fLookTimer( 20.0f )
, m_eLookState( LOOKSTATE_FWD )
, m_iNeckBoneIndex( 7 )
, m_iLeftEyeBoneIndex( 64 )
, m_iRightEyeBoneIndex( 65 )
{
	bool bLocalPlayer = AVP_Game_Core::IsLocalPlayer( m_xOwnerActor.GetGuid() );
	if ( !bLocalPlayer )
	{
		m_pxAimDirInterpolator = new AVP_Direction_Interpolator();
		m_pxLookAtPosInterpolator = new Asura_PosAndOriInterpolator();
	}

	AVP_Actor_AnimationHandler* pxAnimHandler = m_xOwnerActor.GetAnimationHandler();
	Asura_Animation* pxAnimation = pxAnimHandler ? pxAnimHandler->GetAnimation() : NULL;
	if ( pxAnimation )
	{
		Asura_AnimationBlend*	pxRootRLAimGroupBlend	= new Asura_AnimationBlend(1.0f);
#ifdef _DEBUG
		Asura_AnimationController_Group* pxRootRLAimGroup = pxAnimation->CreateControllerGroup( "Aim Group Right To Left", AAC_FLAG_ACTIVE | AAC_FLAG_ENSURE_UNIQUE_HASH | AACG_FLAG_DONT_PASS_DESTROY_TO_CHILDREN, bLocalPlayer ? AVP_AAPC_LOOK_GROUP_HUD : AVP_AAPC_LOOK_RL_GROUP_TP, pxRootRLAimGroupBlend );
#else
		Asura_AnimationController_Group* pxRootRLAimGroup = pxAnimation->CreateControllerGroup( 0xe2362e26, AAC_FLAG_ACTIVE | AAC_FLAG_ENSURE_UNIQUE_HASH | AACG_FLAG_DONT_PASS_DESTROY_TO_CHILDREN, bLocalPlayer ? AVP_AAPC_LOOK_GROUP_HUD : AVP_AAPC_LOOK_RL_GROUP_TP, pxRootRLAimGroupBlend );
#endif
		if ( pxRootRLAimGroup != NULL )
		{
			m_uRLAimGroupHash = pxRootRLAimGroup->GetControllerHash();
		}

		Asura_AnimationBlend*	pxRootDUAimGroupBlend	= new Asura_AnimationBlend(1.0f);
#ifdef _DEBUG
		Asura_AnimationController_Group* pxRootDUAimGroup = pxAnimation->CreateControllerGroup( "Aim Group Down To Up", AAC_FLAG_ACTIVE | AAC_FLAG_ENSURE_UNIQUE_HASH | AACG_FLAG_DONT_PASS_DESTROY_TO_CHILDREN, bLocalPlayer ? AVP_AAPC_LOOK_GROUP_HUD : AVP_AAPC_LOOK_DU_GROUP_TP, pxRootDUAimGroupBlend );
#else
		Asura_AnimationController_Group* pxRootDUAimGroup = pxAnimation->CreateControllerGroup( 0xe2362e26, AAC_FLAG_ACTIVE | AAC_FLAG_ENSURE_UNIQUE_HASH | AACG_FLAG_DONT_PASS_DESTROY_TO_CHILDREN, bLocalPlayer ? AVP_AAPC_LOOK_GROUP_HUD : AVP_AAPC_LOOK_DU_GROUP_TP, pxRootDUAimGroupBlend );
#endif
		if ( pxRootDUAimGroup != NULL )
		{
			m_uDUAimGroupHash = pxRootDUAimGroup->GetControllerHash();
		}

		Asura_AnimationBlend*	pxRLDragGroupBlend	= new Asura_AnimationBlend(1.0f);
#ifdef _DEBUG
		Asura_AnimationController_Group* pxRLDragGroup = pxAnimation->CreateControllerGroup( "RL Drag Group", AAC_FLAG_ACTIVE | AAC_FLAG_ENSURE_UNIQUE_HASH | AACG_FLAG_DONT_PASS_DESTROY_TO_CHILDREN, AVP_AAPC_RL_DRAG_GROUP, pxRLDragGroupBlend );
#else
		Asura_AnimationController_Group* pxRLDragGroup = pxAnimation->CreateControllerGroup( 0x667a8239, AAC_FLAG_ACTIVE | AAC_FLAG_ENSURE_UNIQUE_HASH | AACG_FLAG_DONT_PASS_DESTROY_TO_CHILDREN, AVP_AAPC_RL_DRAG_GROUP, pxRLDragGroupBlend );
#endif
		if ( pxRLDragGroup != NULL )
		{
			m_uRLDragGroupHash = pxRLDragGroup->GetControllerHash();
		}

		Asura_AnimationBlend*	pxDUDragGroupBlend	= new Asura_AnimationBlend(1.0f);
#ifdef _DEBUG
		Asura_AnimationController_Group* pxDUDragGroup = pxAnimation->CreateControllerGroup( "DU Drag Group", AAC_FLAG_ACTIVE | AAC_FLAG_ENSURE_UNIQUE_HASH | AACG_FLAG_DONT_PASS_DESTROY_TO_CHILDREN, AVP_AAPC_DU_DRAG_GROUP, pxDUDragGroupBlend );
#else
		Asura_AnimationController_Group* pxDUDragGroup = pxAnimation->CreateControllerGroup( 0xa08ef782, AAC_FLAG_ACTIVE | AAC_FLAG_ENSURE_UNIQUE_HASH | AACG_FLAG_DONT_PASS_DESTROY_TO_CHILDREN, AVP_AAPC_DU_DRAG_GROUP, pxDUDragGroupBlend );
#endif
		if ( pxDUDragGroup != NULL )
		{
			m_uDUDragGroupHash = pxDUDragGroup->GetControllerHash();
		}

		Asura_AnimationBlend*	pxStrafeDragGroupBlend	= new Asura_AnimationBlend(1.0f);
#ifdef _DEBUG
		Asura_AnimationController_Group* pxStrafeDragGroup = pxAnimation->CreateControllerGroup( "Strafe Drag Group", AAC_FLAG_ACTIVE | AAC_FLAG_ENSURE_UNIQUE_HASH | AACG_FLAG_DONT_PASS_DESTROY_TO_CHILDREN, AVP_AAPC_STRAFE_DRAG_GROUP, pxStrafeDragGroupBlend );
#else
		Asura_AnimationController_Group* pxStrafeDragGroup = pxAnimation->CreateControllerGroup( 0x0da0a844, AAC_FLAG_ACTIVE | AAC_FLAG_ENSURE_UNIQUE_HASH | AACG_FLAG_DONT_PASS_DESTROY_TO_CHILDREN, AVP_AAPC_STRAFE_DRAG_GROUP, pxStrafeDragGroupBlend );
#endif
		if ( pxStrafeDragGroup != NULL )
		{
			m_uStrafeDragGroupHash = pxStrafeDragGroup->GetControllerHash();
		}
	}

	const Asura_Hierarchy_Skin* pxSkin = pxAnimation ? pxAnimation->GetSkin() : NULL;
	if ( pxSkin )
	{
		if ( m_iNeckBoneIndex > 0 && m_iNeckBoneIndex < pxSkin->NumberOfBones )
		{
			const Asura_Quat& xNeckQuat = pxSkin->BoneRestPose[ m_iNeckBoneIndex ].Quat;
			xNeckQuat.ToEuler( m_xCurrentNeckEuler );

			int iParentBone = pxSkin->ParentBone[ m_iNeckBoneIndex ];
			m_xNeckParentRestPoseQuatOS = pxSkin->BoneRestPose[ iParentBone ].Quat;
			while ( iParentBone > 0 && iParentBone < pxSkin->NumberOfBones )
			{
				iParentBone = pxSkin->ParentBone[ iParentBone ];
				m_xNeckParentRestPoseQuatOS = pxSkin->BoneRestPose[ iParentBone ].Quat * m_xNeckParentRestPoseQuatOS;
			}
		}

		if ( m_iLeftEyeBoneIndex > 0 && m_iLeftEyeBoneIndex < pxSkin->NumberOfBones )
		{
			int iParentBone = pxSkin->ParentBone[ m_iLeftEyeBoneIndex ];
			m_xEyeParentRestPoseQuatOS = pxSkin->BoneRestPose[ iParentBone ].Quat;
			while ( iParentBone > 0 && iParentBone < pxSkin->NumberOfBones )
			{
				iParentBone = pxSkin->ParentBone[ iParentBone ];
				m_xEyeParentRestPoseQuatOS = pxSkin->BoneRestPose[ iParentBone ].Quat * m_xEyeParentRestPoseQuatOS;
			}
		}
	}

	const Asura_Matrix_3x3& xOri = m_xOwnerActor.GetCurrentOrientation();
	Asura_AVector3 xInitialFwdVec( xOri.e13, xOri.e23, xOri.e33 );
	ResetAimDirection( xInitialFwdVec );
	ResetLookAtPosition( Asura_AVector3( m_xOwnerActor.GetCurrentPosition() ) + xInitialFwdVec * 10.0f );

#if defined(ASURA_DEBUG_VARIABLE_SYSTEM)
	if ( s_uConsoleVarReferenceCount == 0 )
	{
#if !defined(ASURA_NO_DEBUG_OUTPUT)
		if ( !Asura_CommandConsole::VarExists( L"AVP.BodyRig.RenderDebugOutput" ) )
		{
			Asura_CommandConsole::AddVar( L"AVP.BodyRig.RenderDebugOutput", &dvs_bRenderDebugOutput );
		}
#endif // ASURA_NO_DEBUG_OUTPUT
		if ( !Asura_CommandConsole::VarExists( L"AVP.BodyRig.TurnIdleTime" ) )
		{
			Asura_CommandConsole::AddVar( L"AVP.BodyRig.TurnIdleTime", &dvs_fTurnIdleTime, 0.0f );
		}

		if ( !Asura_CommandConsole::VarExists( L"AVP.BodyRig.EyeControlGuid" ) )
		{
			Asura_CommandConsole::AddVar( L"AVP.BodyRig.EyeControlGuid", &dvs_uDebugEyeGuid );
		}

		if ( !Asura_CommandConsole::VarExists( L"AVP.BodyRig.EyeBalance" ) )
		{
			Asura_CommandConsole::AddVar( L"AVP.BodyRig.EyeBalance", &dvs_fEyeBalance, -1.0f, 1.0f );
		}

		if ( !Asura_CommandConsole::VarExists( L"AVP.BodyRig.EyeClose" ) )
		{
			Asura_CommandConsole::AddVar( L"AVP.BodyRig.EyeClose", &dvs_fEyeRestClose, 0.0f, 1.0f );
		}

		if ( !Asura_CommandConsole::VarExists( L"AVP.BodyRig.EyeSquint" ) )
		{
			Asura_CommandConsole::AddVar( L"AVP.BodyRig.EyeSquint", &dvs_fEyeSquint, 0.0f, 1.0f );
		}

		if ( !Asura_CommandConsole::VarExists( L"AVP.BodyRig.EyeNaturalFocus" ) )
		{
			Asura_CommandConsole::AddVar( L"AVP.BodyRig.EyeNaturalFocus", &dvs_fEyeNaturalFocusAngle, -Asura_Maths::Pi, Asura_Maths::Pi );
		}

		if ( !Asura_CommandConsole::VarExists( L"AVP.BodyRig.LookRightLeftDragTweenInTime" ) )
		{
			Asura_CommandConsole::AddVar( L"AVP.BodyRig.LookRightLeftDragTweenInTime", &dvs_fRLDragTweenInTime, 0.00001f, 1.0f );
		}

		if ( !Asura_CommandConsole::VarExists( L"AVP.BodyRig.LookDownUpDragTweenInTime" ) )
		{
			Asura_CommandConsole::AddVar( L"AVP.BodyRig.LookDownUpDragTweenInTime", &dvs_fDUDragTweenInTime, 0.00001f, 1.0f );
		}

		if ( !Asura_CommandConsole::VarExists( L"AVP.BodyRig.StrafeRightLeftDragTweenInTime" ) )
		{
			Asura_CommandConsole::AddVar( L"AVP.BodyRig.StrafeRightLeftDragTweenInTime", &dvs_fStrafeDragTweenInTime, 0.00001f, 1.0f );
		}

		if ( !Asura_CommandConsole::VarExists( L"AVP.BodyRig.LookRightLeftDragTweenOutTime" ) )
		{
			Asura_CommandConsole::AddVar( L"AVP.BodyRig.LookRightLeftDragTweenOutTime", &dvs_fRLDragTweenOutTime, 0.00001f, 1.0f );
		}

		if ( !Asura_CommandConsole::VarExists( L"AVP.BodyRig.LookDownUpDragTweenOutTime" ) )
		{
			Asura_CommandConsole::AddVar( L"AVP.BodyRig.LookDownUpDragTweenOutTime", &dvs_fDUDragTweenOutTime, 0.00001f, 1.0f );
		}

		if ( !Asura_CommandConsole::VarExists( L"AVP.BodyRig.StrafeRightLeftDragTweenOutTime" ) )
		{
			Asura_CommandConsole::AddVar( L"AVP.BodyRig.StrafeRightLeftDragTweenOutTime", &dvs_fStrafeDragTweenOutTime, 0.00001f, 1.0f );
		}
	}

// 	if ( !Asura_CommandConsole::VarExists( L"AVP.BodyRig.NeckTilt" ) && !bLocalPlayer )
// 	{
// 		Asura_CommandConsole::AddVar( L"AVP.BodyRig.NeckTilt", &m_fNeckTilt, -Asura_Maths::Pi, Asura_Maths::Pi );
// 	}
//////////////////////   DEBUG   /////////////////////////////////////////

	++s_uConsoleVarReferenceCount;
#endif // ASURA_SUPPORT_COMMAND_CONSOLE
}

AVP_Animation_BodyRig::~AVP_Animation_BodyRig()
{
	delete m_pxAimDirInterpolator;
	delete m_pxLookAtPosInterpolator;

#if defined(ASURA_SUPPORT_COMMAND_CONSOLE)
	--s_uConsoleVarReferenceCount;
#endif
}

void AVP_Animation_BodyRig::Update()
{
	UpdateBlinking();
	UpdateLookAtCamera();

#if defined(ASURA_SUPPORT_COMMAND_CONSOLE)
	if ( dvs_uDebugEyeGuid == m_xOwnerActor.GetGuid() )
	{
		m_fEyeBalance	= dvs_fEyeBalance;
		m_fEyeRestClose = dvs_fEyeRestClose;
		m_fEyeSquint	= dvs_fEyeSquint;
		m_fEyeNaturalFocusAngle = dvs_fEyeNaturalFocusAngle;
	}
#endif
}

void AVP_Animation_BodyRig::UpdateTimers( const float fTime )
{
	if ( m_xOwnerActor.IsDead() )
	{
		m_xOwnerActor.GetShapeInstance().SetAnimTimestep( ANIMTIME_USEGAMETIME );
	}
	else
	{
		UpdateTurning();
		UpdateAimAnims();

		// Set the timestep for the next frame;
		float fNextTimeStep = Asura_Maths::FloatToInt_Chop( ( Asura_Physics_System::GetTimeLeft() + Asura_Timers::GetGameTimeSlice() ) / Asura_Physics_System::GetTimeSlice() ) * Asura_Physics_System::GetTimeSlice();
		// this is a copy of a condition in Asura_Physics_System::Update() required to stop animations playing if the game is paused with physics time to process
		if ( Asura_Timers::GetGameTimeSlice() < 0.00001f ) 
		{
			fNextTimeStep = 0.0f;
		}

		if ( AVP_Game_Core::IsLocalPlayer( m_xOwnerActor.GetGuid() ) )
		{
			// set the time step for the next frame's anim update
			m_xOwnerActor.GetShapeInstance().SetAnimTimestep( fNextTimeStep );
		}
		else
		{
			m_xOwnerActor.GetShapeInstance().SetAnimTimestep( ANIMTIME_USEGAMETIME );
		}
	}
}

void AVP_Animation_BodyRig::UpdateBoneData( const int iFirstBone, const int iLastBone )
{
	if ( !m_xOwnerActor.IsDead() )
	{
		if ( iFirstBone <= m_iNeckBoneIndex && m_iNeckBoneIndex <= iLastBone )
		{
			UpdateNeckBone();
		}
		if ( ( iFirstBone <= m_iLeftEyeBoneIndex && m_iLeftEyeBoneIndex <= iLastBone ) || ( iFirstBone <= m_iRightEyeBoneIndex && m_iRightEyeBoneIndex <= iLastBone ) )
		{
			UpdateEyeBones();
		}
	}

#ifndef ASURA_NO_DEBUG_OUTPUT
	RenderDebugOutput();
#endif
}

void AVP_Animation_BodyRig::SetAimAnimations( const Asura_Hash_ID uAimDownUpHash, const Asura_Hash_ID uAimDownUpDragHash, const Asura_Hash_ID uAimRightLeftHash, const Asura_Hash_ID uAimRightLeftDragHash, const Asura_Hash_ID uStrafeDragHash, const float fTweemTime )
{
	AVP_Actor_AnimationHandler* pxAnimHandler = m_xOwnerActor.GetAnimationHandler();
	Asura_Animation* pxAnimation = pxAnimHandler ? pxAnimHandler->GetAnimation() : NULL;
	if ( pxAnimation )
	{
		const u_int uAnimationFlags = AAC_FLAG_ACTIVE | AAC_FLAG_ENSURE_UNIQUE_HASH | AAC_FLAG_ALLOW_DESTROY_FROM_MESSAGE | AACA_FLAG_CALCULATE_MOVEMENT_OFFSET;
		if ( uAimRightLeftHash != m_uAimRightLeftAnimHash )
		{
			Asura_AnimationController_Group*			pxRLAimGroup		= pxAnimation->GetControllerGroupFromHash( m_uRLAimGroupHash );
			if ( pxRLAimGroup != NULL )
			{
				Asura_AnimationController_Animation*	pxRightLeftAACA		= NULL;
				Asura_AnimationBlend*					pxRightLeftBlend	= new Asura_AnimationBlend_Tween( fTweemTime, false );
				pxRightLeftBlend->SetDeltasUseBlends( true );
#ifdef _DEBUG
				pxRightLeftAACA	= pxAnimation->CreateControllerAnimation( "Aim Right To Left", uAnimationFlags, AVP_AAPC_GENERAL_ANIM, uAimRightLeftHash == ASURA_HASH_ID_UNSET ? m_uAimRightLeftAnimHash : uAimRightLeftHash, pxRightLeftBlend, pxRLAimGroup );
#else
				pxRightLeftAACA	= pxAnimation->CreateControllerAnimation( uAnimationFlags, AVP_AAPC_GENERAL_ANIM, uAimRightLeftHash == ASURA_HASH_ID_UNSET ? m_uAimRightLeftAnimHash : uAimRightLeftHash, pxRightLeftBlend, pxRLAimGroup );
#endif
				Asura_Assertf( pxRightLeftAACA != NULL, ( "Unable to create look anim with hash ID 0x%08x", uAimRightLeftHash ) );
				if ( pxRightLeftAACA != NULL )
				{
					pxRightLeftAACA->Stop();
					if ( uAimRightLeftHash == ASURA_HASH_ID_UNSET )
					{
						pxRightLeftAACA->SetParametricTimer( 0.5f, false );
						m_uAimRightLeftAACAHash = m_uAimRightLeftAnimHash = ASURA_HASH_ID_UNSET;
					}
					else
					{
						m_uAimRightLeftAnimHash = uAimRightLeftHash;
						m_uAimRightLeftAACAHash = pxRightLeftAACA->GetControllerHash();
					}
				}
				else
				{
					m_uAimRightLeftAACAHash = m_uAimRightLeftAnimHash = ASURA_HASH_ID_UNSET;
					delete pxRightLeftBlend;
				}
			}
		}

		if ( uAimDownUpHash != m_uAimDownUpAnimHash )
		{
			Asura_AnimationController_Group* pxDUAimGroup = pxAnimation->GetControllerGroupFromHash( m_uDUAimGroupHash );
			if ( pxDUAimGroup != NULL )
			{
				Asura_AnimationController_Animation*	pxDownUpAACA	= NULL;
				Asura_AnimationBlend*					pxDownUpBlend	= new Asura_AnimationBlend_Tween( fTweemTime, false );
				pxDownUpBlend->SetDeltasUseBlends( true );
#ifdef _DEBUG
				pxDownUpAACA	= pxAnimation->CreateControllerAnimation( "Aim Down To Up", uAnimationFlags, AVP_AAPC_GENERAL_ANIM, uAimDownUpHash == ASURA_HASH_ID_UNSET ? m_uAimDownUpAnimHash : uAimDownUpHash, pxDownUpBlend, pxDUAimGroup );
#else
				pxDownUpAACA	= pxAnimation->CreateControllerAnimation( uAnimationFlags, AVP_AAPC_GENERAL_ANIM, uAimDownUpHash == ASURA_HASH_ID_UNSET ? m_uAimDownUpAnimHash : uAimDownUpHash, pxDownUpBlend, pxDUAimGroup );
#endif
				Asura_Assertf( pxDownUpAACA != NULL, ( "Unable to create look anim with hash ID 0x%08x", uAimDownUpHash ) );
				if ( pxDownUpAACA != NULL )
				{
					pxDownUpAACA->Stop();
					if ( uAimDownUpHash == ASURA_HASH_ID_UNSET )
					{
						pxDownUpAACA->SetParametricTimer( 0.5f, false );
						m_uAimDownUpAACAHash = m_uAimDownUpAnimHash = ASURA_HASH_ID_UNSET;
					}
					else
					{
						m_uAimDownUpAnimHash = uAimDownUpHash;
						m_uAimDownUpAACAHash = pxDownUpAACA->GetControllerHash();
					}
				}
				else
				{
					m_uAimDownUpAACAHash = m_uAimDownUpAnimHash = ASURA_HASH_ID_UNSET;
					delete pxDownUpBlend;
				}
			}
		}

		if ( uAimRightLeftDragHash != m_uAimRightLeftDragAnimHash )
		{
			Asura_AnimationController_Group*			pxRLDragGroup		= pxAnimation->GetControllerGroupFromHash( m_uRLDragGroupHash );
			if ( pxRLDragGroup != NULL )
			{
				Asura_AnimationController_Animation*	pxRightLeftDragAACA		= NULL;
				Asura_AnimationBlend*					pxRightLeftDragBlend	= new Asura_AnimationBlend_Tween( fTweemTime, false );
				pxRightLeftDragBlend->SetDeltasUseBlends( true );
#ifdef _DEBUG
				pxRightLeftDragAACA	= pxAnimation->CreateControllerAnimation( "Aim Right To Left Drag", uAnimationFlags, AVP_AAPC_GENERAL_ANIM, uAimRightLeftDragHash == ASURA_HASH_ID_UNSET ? m_uAimRightLeftDragAnimHash : uAimRightLeftDragHash, pxRightLeftDragBlend, pxRLDragGroup );
#else
				pxRightLeftDragAACA	= pxAnimation->CreateControllerAnimation( uAnimationFlags, AVP_AAPC_GENERAL_ANIM, uAimRightLeftDragHash == ASURA_HASH_ID_UNSET ? m_uAimRightLeftDragAnimHash : uAimRightLeftDragHash, pxRightLeftDragBlend, pxRLDragGroup );
#endif
				Asura_Assertf( pxRightLeftDragAACA != NULL, ( "Unable to create look anim with hash ID 0x%08x", uAimRightLeftDragHash ) );
				if ( pxRightLeftDragAACA != NULL )
				{
					pxRightLeftDragAACA->Stop();
					if ( uAimRightLeftDragHash == ASURA_HASH_ID_UNSET )
					{
						pxRightLeftDragAACA->SetParametricTimer( 0.5f, false );
						m_uAimRightLeftDragAACAHash = m_uAimRightLeftDragAnimHash = ASURA_HASH_ID_UNSET;
					}
					else
					{
						m_uAimRightLeftDragAnimHash = uAimRightLeftDragHash;
						m_uAimRightLeftDragAACAHash = pxRightLeftDragAACA->GetControllerHash();
					}
				}
				else
				{
					m_uAimRightLeftDragAACAHash = m_uAimRightLeftDragAnimHash = ASURA_HASH_ID_UNSET;
					delete pxRightLeftDragBlend;
				}
			}
		}

		if ( uAimDownUpDragHash != m_uAimDownUpDragAnimHash )
		{
			Asura_AnimationController_Group*			pxDUDragGroup		= pxAnimation->GetControllerGroupFromHash( m_uDUDragGroupHash );
			if ( pxDUDragGroup != NULL )
			{
				Asura_AnimationController_Animation*	pxDownUpDragAACA		= NULL;
				Asura_AnimationBlend*					pxDownUpDragBlend	= new Asura_AnimationBlend_Tween( fTweemTime, false );
				pxDownUpDragBlend->SetDeltasUseBlends( true );
#ifdef _DEBUG
				pxDownUpDragAACA	= pxAnimation->CreateControllerAnimation( "Aim Down To Up Drag", uAnimationFlags, AVP_AAPC_GENERAL_ANIM, uAimDownUpDragHash == ASURA_HASH_ID_UNSET ? m_uAimDownUpDragAnimHash : uAimDownUpDragHash, pxDownUpDragBlend, pxDUDragGroup );
#else
				pxDownUpDragAACA	= pxAnimation->CreateControllerAnimation( uAnimationFlags, AVP_AAPC_GENERAL_ANIM, uAimDownUpDragHash == ASURA_HASH_ID_UNSET ? m_uAimDownUpDragAnimHash : uAimDownUpDragHash, pxDownUpDragBlend, pxDUDragGroup );
#endif
				Asura_Assertf( pxDownUpDragAACA != NULL, ( "Unable to create look anim with hash ID 0x%08x", uAimDownUpDragHash ) );
				if ( pxDownUpDragAACA != NULL )
				{
					pxDownUpDragAACA->Stop();
					if ( uAimDownUpDragHash == ASURA_HASH_ID_UNSET )
					{
						pxDownUpDragAACA->SetParametricTimer( 0.5f, false );
						m_uAimDownUpDragAACAHash = m_uAimDownUpDragAnimHash = ASURA_HASH_ID_UNSET;
					}
					else
					{
						m_uAimDownUpDragAnimHash = uAimDownUpDragHash;
						m_uAimDownUpDragAACAHash = pxDownUpDragAACA->GetControllerHash();
					}
				}
				else
				{
					m_uAimDownUpDragAACAHash = m_uAimDownUpDragAnimHash = ASURA_HASH_ID_UNSET;
					delete pxDownUpDragBlend;
				}
			}
		}

		if ( uStrafeDragHash != m_uStrafeDragAnimHash )
		{
			Asura_AnimationController_Group*			pxStrafeDragGroup		= pxAnimation->GetControllerGroupFromHash( m_uStrafeDragGroupHash );
			if ( pxStrafeDragGroup != NULL )
			{
				Asura_AnimationController_Animation*	pxStrafeDragAACA		= NULL;
				Asura_AnimationBlend*					pxStrafeDragBlend	= new Asura_AnimationBlend_Tween( fTweemTime, false );
				pxStrafeDragBlend->SetDeltasUseBlends( true );
#ifdef _DEBUG
				pxStrafeDragAACA	= pxAnimation->CreateControllerAnimation( "Strafe Drag", uAnimationFlags, AVP_AAPC_GENERAL_ANIM, uStrafeDragHash == ASURA_HASH_ID_UNSET ? m_uStrafeDragAnimHash : uStrafeDragHash, pxStrafeDragBlend, pxStrafeDragGroup );
#else
				pxStrafeDragAACA	= pxAnimation->CreateControllerAnimation( uAnimationFlags, AVP_AAPC_GENERAL_ANIM, uStrafeDragHash == ASURA_HASH_ID_UNSET ? m_uStrafeDragAnimHash : uStrafeDragHash, pxStrafeDragBlend, pxStrafeDragGroup );
#endif
				Asura_Assertf( pxStrafeDragAACA != NULL, ( "Unable to create look anim with hash ID 0x%08x", uStrafeDragHash ) );
				if ( pxStrafeDragAACA != NULL )
				{
					pxStrafeDragAACA->Stop();
					if ( uStrafeDragHash == ASURA_HASH_ID_UNSET )
					{
						pxStrafeDragAACA->SetParametricTimer( 0.5f, false );
						m_uStrafeDragAACAHash = m_uStrafeDragAnimHash = ASURA_HASH_ID_UNSET;
					}
					else
					{
						m_uStrafeDragAnimHash = uStrafeDragHash;
						m_uStrafeDragAACAHash = pxStrafeDragAACA->GetControllerHash();
					}
				}
				else
				{
					m_uStrafeDragAACAHash = m_uStrafeDragAnimHash = ASURA_HASH_ID_UNSET;
					delete pxStrafeDragBlend;
				}
			}
		}
	}
}

void AVP_Animation_BodyRig::GetAimDirection( Asura_AVector3& xOutDirection ) const
{
	if ( m_pxAimDirInterpolator != NULL )
	{
		m_pxAimDirInterpolator->GetCurrent( xOutDirection );
	}
	else
	{
		xOutDirection = m_xAimDirection;
	}
}

void AVP_Animation_BodyRig::SetAimDirection( const Asura_AVector3& xDirection, const float fTime )
{
	Asura_Assert(xDirection == xDirection, "Trying To Set AimDirection to NAN");
	if (xDirection == xDirection)
	{
		if ( m_pxAimDirInterpolator != NULL )
		{
			m_pxAimDirInterpolator->Update( xDirection, fTime );
		}
		else
		{
			Asura_Assert( Asura_Maths::Abs( 1.0f - xDirection.MagnitudeSquared() ) < 1e-5f, "Direction is not normalized, it should be" );
			m_xAimDirection = xDirection;
		}
	}
}

void AVP_Animation_BodyRig::ResetAimDirection( const Asura_AVector3& xDirection )
{
	Asura_Assert(xDirection == xDirection, "Trying To Set AimDirection to NAN");
	if (xDirection == xDirection)
	{
		if ( m_pxAimDirInterpolator != NULL )
		{
			m_pxAimDirInterpolator->Reset( xDirection );
		}
		else
		{
			m_xAimDirection = xDirection;
		}
	}
}

void AVP_Animation_BodyRig::GetLookAtPosition( Asura_AVector3& xOutPosition ) const
{
	Asura_AVector3 xRestLookAtPos;
	if ( m_pxLookAtPosInterpolator != NULL )
	{
		Asura_Vector_3 xPos;
		Asura_Vector_3 xVel;
		m_pxLookAtPosInterpolator->GetCurrentPosition( xPos, xVel );
		xRestLookAtPos = Asura_AVector3( xPos );
	}
	else
	{
		xRestLookAtPos = m_xLookAtPosition;
	}

	xOutPosition.Lerp( xRestLookAtPos, Asura_AVector3( *Asura_Camera::GetPosition() ), m_fLookAtCameraParameter );
}

void AVP_Animation_BodyRig::SetLookAtPosition( const Asura_AVector3& xPosition, const float fTime )
{
	Asura_Assert(xPosition == xPosition, "Trying To Set LookAtPosition to NAN");
	if (xPosition == xPosition)
	{
		if ( m_pxLookAtPosInterpolator != NULL )
		{
			m_pxLookAtPosInterpolator->ProcessNewData( Asura_Vector_3( xPosition ), Asura_Maths::IdentityQuat, fTime );
		}
		else
		{
			m_xLookAtPosition = xPosition;
		}
	}
}

void AVP_Animation_BodyRig::ResetLookAtPosition( const Asura_AVector3& xPosition )
{
	Asura_Assert(xPosition == xPosition, "Trying To Set LookAtPosition to NAN");
	if (xPosition == xPosition)
	{
		if ( m_pxLookAtPosInterpolator != NULL )
		{
			m_pxLookAtPosInterpolator->TeleportTo( Asura_Vector_3( xPosition ), Asura_Maths::IdentityMatrix3x3 );
		}
		else
		{
			m_xLookAtPosition = xPosition;
		}
	}
}

void AVP_Animation_BodyRig::Blink()
{
	if ( m_eBlinkState == BLINK_OPEN ) { m_eBlinkState = BLINK_CLOSING; m_fBlinkTimer = m_fHalfBlinkDuration; }
}

void AVP_Animation_BodyRig::UpdateNeckBone()
{
// 	const Asura_Hierarchy_Skin* pxSkin = GetSkin();
// 	if ( pxSkin && m_iNeckBoneIndex > 0 && m_iNeckBoneIndex < pxSkin->NumberOfBones && m_fNeckBlend > 0.0f )
// 	{
// 		Asura_Matrix_3x3 xNeckParentOriWS;
// 		GetBoneLocation( pxSkin->ParentBone[ m_iNeckBoneIndex ], NULL, &xNeckParentOriWS, true );
// 		Asura_AMatrix3x3 xNeckParentOriWSA( xNeckParentOriWS );
// 		
// 		Asura_AVector3 xLookDirInNeckParentSpace;
// 		GetLookDirection( xLookDirInNeckParentSpace );
// 		xLookDirInNeckParentSpace = xNeckParentOriWSA.ApplyTranspose( xLookDirInNeckParentSpace );
// 
// 		Asura_AVector3 xWSFwdInNeckParentRestPoseSpace;
// 		Asura_Quat xAnimOri = GetOrientQuat();
// 		xAnimOri.Invert();
// 		xAnimOri.RotateVector( m_xOSFwdInNeckParentRestPoseSpace, xWSFwdInNeckParentRestPoseSpace );
// 
// 		Asura_Euler_3 xLookEuler;
// 		xLookEuler.x = Asura_Maths::RadiansToZeroCentredRange(	Asura_Maths::ArcTangent( -xLookDirInNeckParentSpace.GetY(), xLookDirInNeckParentSpace.GetZ() )
// 															-	Asura_Maths::ArcTangent( -xWSFwdInNeckParentRestPoseSpace.GetY(), xWSFwdInNeckParentRestPoseSpace.GetZ() ) );
// 		xLookEuler.y = Asura_Maths::RadiansToZeroCentredRange(	Asura_Maths::ArcSine( xLookDirInNeckParentSpace.GetX() )
// 															-	Asura_Maths::ArcSine( xWSFwdInNeckParentRestPoseSpace.GetX() ) );
// 		xLookEuler.z = 0.0f;
// 
// 		Asura_Euler_3 xNeckEuler;
// 		pxSkin->BoneRestPose[ m_iNeckBoneIndex ].Quat.ToEuler( xNeckEuler );
// 
// 		Asura_Quat xOldNeckQuat;
// 		Asura_Euler_3 xOldNeckEuler;
// 		GetTweenStore()->GetBoneData( m_iNeckBoneIndex ).GetOrientation( xOldNeckQuat );
// 		xOldNeckQuat.ToEuler( xOldNeckEuler );
// 
// 		xNeckEuler += xLookEuler;
// 		xNeckEuler.x = Asura_Maths::ClampToRange( xNeckEuler.x, -m_fMaxNeckRightLeft, m_fMaxNeckRightLeft );
// 		float fXMod = Asura_Maths::Abs( xNeckEuler.x ) / m_fMaxNeckRightLeft;
// 		fXMod = 1.0f - 0.5f * fXMod * fXMod;
// 		xNeckEuler.y = Asura_Maths::ClampToRange( xNeckEuler.y, -m_fMaxNeckDownUp * fXMod, m_fMaxNeckDownUp * fXMod );
// 		xNeckEuler.z = xOldNeckEuler.z;
// 		if ( Asura_Maths::Abs( xOldNeckEuler.x > 0.0f ) )
// 		{
// 			xNeckEuler.z *= xNeckEuler.x / xOldNeckEuler.x;
// 		}
// 		xNeckEuler.z += m_fNeckTilt;
// 		xNeckEuler.z = Asura_Maths::ClampToRange( xNeckEuler.z, -m_fMaxNeckTilt, m_fMaxNeckTilt );
// 
// 		const float fTimeSlice = Asura_Timers::GetGameTimeSlice();
// 		Asura_Euler_3 xNeckEulerDelta = xNeckEuler - m_xCurrentNeckEuler;
// 		xNeckEulerDelta.x = Asura_Maths::ClampToRange( xNeckEulerDelta.x, -m_fHeadRightLeftAngSpeed * fTimeSlice, m_fHeadRightLeftAngSpeed * fTimeSlice );
// 		xNeckEulerDelta.y = Asura_Maths::ClampToRange( xNeckEulerDelta.y, -m_fHeadDownUpAngSpeed * fTimeSlice, m_fHeadDownUpAngSpeed * fTimeSlice );
// 		xNeckEulerDelta.z = Asura_Maths::ClampToRange( xNeckEulerDelta.z, -m_fHeadTiltAngSpeed * fTimeSlice, m_fHeadTiltAngSpeed * fTimeSlice );
// 		m_xCurrentNeckEuler += xNeckEulerDelta;
// 		xNeckEuler = m_xCurrentNeckEuler;
// 
// 		float fHeadRandomTorqueX = ( -dbg_fHeadRandomSpringConstantX * Asura_Maths::Sign( m_fHeadRandomRotationX ) * m_fHeadRandomRotationX * m_fHeadRandomRotationX ) + ( -dbg_fHeadRandomDragX * m_fHeadRandomAngVelocityX ) + Asura_Random::GetFloat( -dbg_fHeadRandomMaxTorqueX , dbg_fHeadRandomMaxTorqueX );
// 		m_fHeadRandomAngVelocityX += fHeadRandomTorqueX * fTimeSlice;
// 		m_fHeadRandomRotationX += m_fHeadRandomAngVelocityX * fTimeSlice;
// 
// 		float fHeadRandomTorqueY = ( -dbg_fHeadRandomSpringConstantY * Asura_Maths::Sign( m_fHeadRandomRotationY ) * m_fHeadRandomRotationY * m_fHeadRandomRotationY ) + ( -dbg_fHeadRandomDragY * m_fHeadRandomAngVelocityY ) + Asura_Random::GetFloat( -dbg_fHeadRandomMaxTorqueY , dbg_fHeadRandomMaxTorqueY );
// 		m_fHeadRandomAngVelocityY += fHeadRandomTorqueY * fTimeSlice;
// 		m_fHeadRandomRotationY += m_fHeadRandomAngVelocityY * fTimeSlice;
// 
// 		xNeckEuler.x += m_fHeadRandomRotationX;
// 		xNeckEuler.y += m_fHeadRandomRotationY;
// 
// 		Asura_Quat xNeckQuat;
// 		xNeckQuat.FromEuler( xNeckEuler );
// 		xNeckQuat.Slerp( xOldNeckQuat, 1.0f - m_fNeckBlend );
// 		GetTweenStore()->GetBoneData( m_iNeckBoneIndex ).SetOrientation( xNeckQuat );
// 	}
}

void AVP_Animation_BodyRig::UpdateEyeBones()
{
	Asura_FacialAnimation_AnimationHandler* pxFaceHandler = Asura_FacialAnimation_System::FindAnimationHandler( m_xOwnerActor.GetGuid() );
	if ( pxFaceHandler )
	{
		AVP_Actor_AnimationHandler* pxAnimHandler = m_xOwnerActor.GetAnimationHandler();
		Asura_Animation* pxAnimation = pxAnimHandler ? pxAnimHandler->GetAnimation() : NULL;
		const Asura_Hierarchy_Skin* pxSkin = pxAnimation ? pxAnimation->GetSkin() : NULL;
		if ( pxSkin && m_iLeftEyeBoneIndex > 0 && m_iLeftEyeBoneIndex < pxSkin->NumberOfBones && m_iRightEyeBoneIndex > 0 && m_iRightEyeBoneIndex < pxSkin->NumberOfBones )
		{
			// Eye direction stuff
			Asura_AVector3 xLookAtPosWS;
			GetLookAtPosition( xLookAtPosWS );

			Asura_Quat xClientOri;
			xClientOri.FromMatrix( m_xOwnerActor.GetCurrentOrientation() );

			Asura_AVector3 xLookAtPosOS;
			Asura_Quat xInvClientOri = xClientOri;
			xInvClientOri.Invert();
			xLookAtPosOS = xInvClientOri.RotateVector( xLookAtPosWS - Asura_AVector3( m_xOwnerActor.GetCurrentPosition() ));

			Asura_Vector_3 xEyeParentPosOS;
			Asura_Matrix_3x3 xEyeParentMatOS;
			pxAnimation->GetBoneLocation( pxSkin->ParentBone[ m_iLeftEyeBoneIndex ], &xEyeParentPosOS, &xEyeParentMatOS, false );
			Asura_Quat xEyeParentQuatOS;
			xEyeParentQuatOS.FromMatrix( xEyeParentMatOS );

			Asura_Vector_3 xLeftEyePosPS;
			pxAnimation->GetTweenStore()->GetBoneData( m_iLeftEyeBoneIndex ).GetPosition( xLeftEyePosPS );
			Asura_Vector_3 xRightEyePosPS;
			pxAnimation->GetTweenStore()->GetBoneData( m_iRightEyeBoneIndex ).GetPosition( xRightEyePosPS );
			Asura_Vector_3 xEyePosPS = ( xLeftEyePosPS + xRightEyePosPS ) * 0.5f;

			Asura_AVector3 xEyePosOS = xEyeParentQuatOS.RotateVector( Asura_AVector3( xEyePosPS ));
			xEyePosOS += Asura_AVector3( xEyeParentPosOS );
			Asura_AVector3 xLookDirOS = xLookAtPosOS - xEyePosOS;
			float fLookDistance = xLookDirOS.NormaliseReturningMagnitude();

			Asura_Quat xLookRotOS;
			xLookRotOS.RotationFromAToB( Asura_AVector3( 0.0f, 0.0f, 1.0f ), xLookDirOS );

			Asura_Quat xInvEyeParentQuatOS = xEyeParentQuatOS;
			xInvEyeParentQuatOS.Invert();
			Asura_Quat xInvEyeParentRot = m_xEyeParentRestPoseQuatOS * xInvEyeParentQuatOS;

			Asura_Quat xLookRotMinusParentRot = xLookRotOS * xInvEyeParentRot;
			Asura_Euler_3 xLookEuler;
			xLookRotMinusParentRot.ToEuler( xLookEuler );

			Asura_Vector_3 xEyeSep = xRightEyePosPS - xLeftEyePosPS;
			float fEyeSep = xEyeSep.Magnitude();
			float fFocus = m_fEyeNaturalFocusAngle + 0.5f * fEyeSep / fLookDistance;

			pxFaceHandler->SetExternalEyeData( -xLookEuler.y, xLookEuler.x, fFocus, m_fEyeCurrentClose, m_fEyeSquint, m_fEyeBalance );
		}
	}
}

void AVP_Animation_BodyRig::UpdateTurning()
{
	const Asura_Movement_System*		pxMovementSystem	= m_xOwnerActor.GetMovementSystem();

	if ( pxMovementSystem == NULL || pxMovementSystem->IsMoveInProgress() || m_xOwnerActor.GetGroundType() == AVP_Actor_PhysicsObjectHandler::OFF_GROUND || m_xOwnerActor.IsReorienting() )
	{
		StopTurnAnim();
	}
	else
	{
		const Asura_AMatrix3x3 xClientOri( m_xOwnerActor.GetCurrentOrientation() );
		Asura_AVector3 xOSAim;
		GetAimDirection( xOSAim );
		xOSAim = xClientOri.ApplyTranspose( xOSAim );
		float fOSAimPhi = Asura_Maths::ArcTangent( xOSAim.GetX(), xOSAim.GetZ() );
		float fAbsPhi = Asura_Maths::Abs( fOSAimPhi );

		if ( m_xOwnerActor.IsMoving() )
		{
			m_fIdleTimer = 0.0f;
			StopTurnAnim();
		}
		else if ( !m_xOwnerActor.IsFalling() && !m_xOwnerActor.IsLanding() && !IsTurning() )
		{
			// Then we're idle
			m_fIdleTimer += Asura_Timers::GetGameTimeSlice();

			// Work out if we should start turning
			if ( fAbsPhi > dbg_fStartTurn90Percentage * m_fMaxAimRightLeft )
			{
				// if we're approaching the limit of our look anim, start turning
				if ( fOSAimPhi < 0.0f )
				{
					m_fCurrentTurnAngle = -Asura_Maths::Deg90;
					PlayTurnAnim( m_uTurn90LeftAnimHash , AVP_Game_Core::IsLocalPlayer( m_xOwnerActor.GetGuid() ), 1.0f );
				}
				else
				{
					m_fCurrentTurnAngle = Asura_Maths::Deg90;
					PlayTurnAnim( m_uTurn90RightAnimHash, AVP_Game_Core::IsLocalPlayer( m_xOwnerActor.GetGuid() ), 1.0f );
				}
				m_fIdleTimer = 0.0f;
			}
			else if ( fAbsPhi > dbg_fStartTurn45Angle )
			{
				if ( m_fIdleTimer > dvs_fTurnIdleTime )
				{
					// or else if we've been idle for a while and we are aiming far enough from center for a turn anim to play, play one.
					if ( fOSAimPhi < 0.0f && m_uTurn45LeftAnimHash != ASURA_HASH_ID_UNSET )
					{
						m_fCurrentTurnAngle = -Asura_Maths::Deg90 * 0.5f;
						PlayTurnAnim( m_uTurn45LeftAnimHash, AVP_Game_Core::IsLocalPlayer( m_xOwnerActor.GetGuid() ), 1.0f );
					}
					else if ( m_uTurn45RightAnimHash != ASURA_HASH_ID_UNSET )
					{
						m_fCurrentTurnAngle = Asura_Maths::Deg90 * 0.5f;
						PlayTurnAnim( m_uTurn45RightAnimHash, AVP_Game_Core::IsLocalPlayer( m_xOwnerActor.GetGuid() ), 1.0f );
					}
					m_fIdleTimer = 0.0f;
				}
			}
			else
			{
				m_fIdleTimer = 0.0f;
			}
		}
	}
}

void AVP_Animation_BodyRig::UpdateAimAnims()
{
	Asura_AVector3 xAimDir;
	Asura_AMatrix3x3 xClientOri( m_xOwnerActor.GetCurrentOrientation() );
	GetAimDirection( xAimDir );

	// get aim dir in object space
	xAimDir = xClientOri.ApplyTranspose( xAimDir );
	const float fAimX = Asura_Maths::ArcSine( xAimDir.GetY() );
	const float fAimY = Asura_Maths::ArcTangent( xAimDir.GetX(), xAimDir.GetZ() );

	m_fLookDownToUpParametric = m_fMaxAimDownUp == 0.0f ? 0.5f : Asura_Maths::ClampToRange( ( -fAimX / ( 2.0f * m_fMaxAimDownUp ) ) + 0.5f, 0.0f, 1.0f );
	m_fLookRightToLeftParametric = m_fMaxAimRightLeft == 0.0f ? 0.5f : Asura_Maths::ClampToRange( ( -fAimY / ( 2.0f * m_fMaxAimRightLeft ) ) + 0.5f, 0.0f, 1.0f );

	if ( m_fLookDownToUpParametric < 0.02f || m_fLookDownToUpParametric > 0.98f )
	{
		m_fTargetAimDownUpDragValue = 0.0f;
	}

	const float fTimeSlice = Asura_Timers::GetGameTimeSlice();
	if ( m_fCurrentAimRightLeftDragValue < m_fTargetAimRightLeftDragValue )
	{
		if ( Asura_Maths::Abs( m_fTargetAimRightLeftDragValue ) < 0.1f )
		{
			m_fCurrentAimRightLeftDragValue += fTimeSlice / dvs_fRLDragTweenOutTime;
		}
		else
		{
			m_fCurrentAimRightLeftDragValue += fTimeSlice / dvs_fRLDragTweenInTime;
		}
		m_fCurrentAimRightLeftDragValue = Asura_Maths::Min( m_fCurrentAimRightLeftDragValue, m_fTargetAimRightLeftDragValue );
	}
	else
	{
		if ( Asura_Maths::Abs( m_fTargetAimRightLeftDragValue ) < 0.1f )
		{
			m_fCurrentAimRightLeftDragValue -= fTimeSlice / dvs_fRLDragTweenOutTime;
		}
		else
		{
			m_fCurrentAimRightLeftDragValue -= fTimeSlice / dvs_fRLDragTweenInTime;
		}

		m_fCurrentAimRightLeftDragValue = Asura_Maths::Max( m_fCurrentAimRightLeftDragValue, m_fTargetAimRightLeftDragValue );
	}

	if ( m_fCurrentAimDownUpDragValue < m_fTargetAimDownUpDragValue )
	{
		if ( Asura_Maths::Abs( m_fTargetAimDownUpDragValue ) < 0.1f )
		{
			m_fCurrentAimDownUpDragValue += fTimeSlice / dvs_fDUDragTweenOutTime;
		}
		else
		{
			m_fCurrentAimDownUpDragValue += fTimeSlice / dvs_fDUDragTweenInTime;
		}
		m_fCurrentAimDownUpDragValue = Asura_Maths::Min( m_fCurrentAimDownUpDragValue, m_fTargetAimDownUpDragValue );
	}
	else
	{
		if ( Asura_Maths::Abs( m_fTargetAimDownUpDragValue ) < 0.1f )
		{
			m_fCurrentAimDownUpDragValue -= fTimeSlice / dvs_fDUDragTweenOutTime;
		}
		else
		{
			m_fCurrentAimDownUpDragValue -= fTimeSlice / dvs_fDUDragTweenInTime;
		}
		m_fCurrentAimDownUpDragValue = Asura_Maths::Max( m_fCurrentAimDownUpDragValue, m_fTargetAimDownUpDragValue );
	}

	if ( m_fCurrentStrafeDragValue < m_fTargetStrafeDragValue )
	{
		if ( Asura_Maths::Abs( m_fTargetStrafeDragValue ) < 0.1f )
		{
			m_fCurrentStrafeDragValue += fTimeSlice / dvs_fStrafeDragTweenOutTime;
		}
		else
		{
			m_fCurrentStrafeDragValue += fTimeSlice / dvs_fStrafeDragTweenInTime;
		}
		m_fCurrentStrafeDragValue = Asura_Maths::Min( m_fCurrentStrafeDragValue, m_fTargetStrafeDragValue );
	}
	else
	{
		if ( Asura_Maths::Abs( m_fTargetStrafeDragValue ) < 0.1f )
		{
			m_fCurrentStrafeDragValue -= fTimeSlice / dvs_fStrafeDragTweenOutTime;
		}
		else
		{
			m_fCurrentStrafeDragValue -= fTimeSlice / dvs_fStrafeDragTweenInTime;
		}
		m_fCurrentStrafeDragValue = Asura_Maths::Max( m_fCurrentStrafeDragValue, m_fTargetStrafeDragValue );
	}

	m_fLookRightToLeftDragParametric = Asura_Maths::ClampToRange( ( 0.5f * m_fCurrentAimRightLeftDragValue ) + 0.5f, 0.0f, 1.0f );
	m_fLookDownToUpDragParametric = Asura_Maths::ClampToRange( ( -0.5f * m_fCurrentAimDownUpDragValue ) + 0.5f, 0.0f, 1.0f );
	m_fStrafeDragParametric = Asura_Maths::ClampToRange( ( -0.5f * m_fCurrentStrafeDragValue ) + 0.5f, 0.0f, 1.0f );

	AVP_Actor_AnimationHandler* pxAnimHandler = m_xOwnerActor.GetAnimationHandler();
	Asura_Animation* pxAnimation = pxAnimHandler ? pxAnimHandler->GetAnimation() : NULL;
	if ( pxAnimation )
	{
		if ( pxAnimHandler->IsThirdPerson() && m_uTurnAACAHash != ASURA_HASH_ID_UNSET )
		{
			Asura_AnimationController_Animation* pxAACA = pxAnimation->GetControllerAnimationFromHash( m_uTurnAACAHash );
			if ( pxAACA != NULL )
			{
				const float fTurnAnimParametric = pxAACA->GetParametricTimer();
				const float fTurnAnimBlendWeight = pxAACA->GetBlendToPreviousController() ? pxAACA->GetBlendToPreviousController()->GetWeight() : 1.0f;
				const float fParametricWeight = Asura_Maths::Lerp( 1.0f, fTurnAnimParametric, fTurnAnimBlendWeight );
				m_fLookRightToLeftParametric = Asura_Maths::Lerp( 0.5f, m_fLookRightToLeftParametric, fParametricWeight );
			}
		}

		Asura_AnimationController_Animation* pxAACA = pxAnimation->GetControllerAnimationFromHash( m_uAimDownUpAACAHash );
		if ( pxAACA != NULL )
		{
			pxAACA->SetParametricTimer( m_fLookDownToUpParametric );
		}

		pxAACA = pxAnimation->GetControllerAnimationFromHash( m_uAimDownUpDragAACAHash );
		if ( pxAACA != NULL )
		{
			pxAACA->SetParametricTimer( m_fLookDownToUpDragParametric );
		}

		pxAACA = pxAnimation->GetControllerAnimationFromHash( m_uAimRightLeftDragAACAHash );
		if ( pxAACA != NULL )
		{
			pxAACA->SetParametricTimer( m_fLookRightToLeftDragParametric );
		}

		pxAACA = pxAnimation->GetControllerAnimationFromHash( m_uAimRightLeftAACAHash );
		if ( pxAACA != NULL )
		{
			pxAACA->SetParametricTimer( m_fLookRightToLeftParametric );
		}

		pxAACA = pxAnimation->GetControllerAnimationFromHash( m_uStrafeDragAACAHash );
		if ( pxAACA != NULL )
		{
			pxAACA->SetParametricTimer( m_fStrafeDragParametric );
		}
	}
}

DEBUGVAR float dbg_fTurn45Limit = Asura_Maths::DegToRad(67.5f);
DEBUGVAR float dbg_fTurn90Limit = Asura_Maths::DegToRad(135.0f);

void AVP_Animation_BodyRig::RequestTurn( const float fTurnAngle )
{
	m_fCurrentTurnAngle = fTurnAngle;
	const float fAbsTurnAngle = Asura_Maths::Abs( fTurnAngle );
	float fTurnAnimAngle;
	Asura_Hash_ID uTurnAnimHash;
	
	if ( fAbsTurnAngle < dbg_fTurn45Limit )
	{
		if ( fTurnAngle < 0.0f )
		{
			if ( m_uTurn45LeftAnimHash != ASURA_HASH_ID_UNSET )
			{
				fTurnAnimAngle = Asura_Maths::Deg90 * 0.5f;
				uTurnAnimHash = m_uTurn45LeftAnimHash;
			}
			else
			{
				fTurnAnimAngle = Asura_Maths::Deg90;
				uTurnAnimHash = m_uTurn90LeftAnimHash;
			}
		}
		else
		{
			if ( m_uTurn45RightAnimHash != ASURA_HASH_ID_UNSET )
			{
				fTurnAnimAngle = Asura_Maths::Deg90 * 0.5f;
				uTurnAnimHash = m_uTurn45RightAnimHash;
			}
			else
			{
				fTurnAnimAngle = Asura_Maths::Deg90;
				uTurnAnimHash = m_uTurn90RightAnimHash;
			}
		}
	}
	else if ( fAbsTurnAngle < dbg_fTurn90Limit )
	{
		fTurnAnimAngle = Asura_Maths::Deg90;
		if ( fTurnAngle < 0.0f )
		{
			uTurnAnimHash = m_uTurn90LeftAnimHash;
		}
		else
		{
			uTurnAnimHash = m_uTurn90RightAnimHash;
		}
	}
	else
	{
		if ( fTurnAngle < 0.0f )
		{
			if ( m_uTurn180LeftAnimHash != ASURA_HASH_ID_UNSET )
			{
				fTurnAnimAngle = Asura_Maths::Deg180;
				uTurnAnimHash = m_uTurn180LeftAnimHash;
			}
			else
			{
				fTurnAnimAngle = Asura_Maths::Deg90;
				uTurnAnimHash = m_uTurn90LeftAnimHash;
			}
		}
		else
		{
			if ( m_uTurn180RightAnimHash != ASURA_HASH_ID_UNSET )
			{
				fTurnAnimAngle = Asura_Maths::Deg180;
				uTurnAnimHash = m_uTurn180RightAnimHash;
			}
			else
			{
				fTurnAnimAngle = Asura_Maths::Deg90;
				uTurnAnimHash = m_uTurn90RightAnimHash;
			}
		}
	}

	if ( uTurnAnimHash != ASURA_HASH_ID_UNSET )
	{
		PlayTurnAnim( uTurnAnimHash, AVP_Game_Core::IsLocalPlayer( m_xOwnerActor.GetGuid() ), fAbsTurnAngle / fTurnAnimAngle );
	}
}

bool AVP_Animation_BodyRig::IsTurning()
{
	AVP_Actor_AnimationHandler* pxAnimHandler = m_xOwnerActor.GetAnimationHandler();
	bool bTurning( false );
	Asura_Animation* pxAnimation = pxAnimHandler ? pxAnimHandler->GetAnimation() : NULL;
	if ( pxAnimation )
	{
		if ( m_uTurnAACAHash != ASURA_HASH_ID_UNSET )
		{
			Asura_AnimationController_Animation* pxAACA = pxAnimation->GetControllerAnimationFromHash( m_uTurnAACAHash );
			if ( pxAACA != NULL )
			{
				bTurning = true;
			}
			else
			{
				m_uTurnAACAHash = ASURA_HASH_ID_UNSET;
			}
		}
	}
	return bTurning;
}

void AVP_Animation_BodyRig::PlayTurnAnim( const Asura_Hash_ID uTurnAnimHash, const bool bHUD, const float fAngVelScale )
{
	AVP_Actor_AnimationHandler* pxAnimHandler = m_xOwnerActor.GetAnimationHandler();
	Asura_Animation* pxAnimation = pxAnimHandler ? pxAnimHandler->GetAnimation() : NULL;
	if ( uTurnAnimHash != ASURA_HASH_ID_UNSET && pxAnimation )
	{
		if (dbg_bDeactivateTurnAnims) { return; }

		const u_int uFlags = AAC_FLAG_ACTIVE | AAC_FLAG_ALLOW_DESTROY_FROM_MESSAGE | AAC_FLAG_ENSURE_UNIQUE_HASH | AACA_FLAG_DONT_TWEEN_VELOCITY | AACA_FLAG_DONT_TWEEN_ANGULAR_VELOCITY | AACA_FLAG_DONT_TWEEN_MOVEMENT_OFFSET | AACA_FLAG_CALCULATE_VELOCITY | AACA_FLAG_CALCULATE_ANGULAR_VELOCITY | AACA_FLAG_CALCULATE_MOVEMENT_OFFSET;
		const u_int uPriority = bHUD ? AVP_AAPC_TURN_ANIM_HUD : AVP_AAPC_TURN_ANIM_TP;
		float fTweenTime = AVP_Game_Core::IsLocalPlayer( m_xOwnerActor.GetGuid() ) ? dbg_fHUDTurnTweenTime : dbg_fTPTurnTweenTime;
		Asura_AnimationBlend_Tween* pxBlend  = new Asura_AnimationBlend_AnimatedTween( fTweenTime, fTweenTime, false );
		pxBlend->SetPlayOnStateIn( true );
		Asura_AnimationController_Animation* pxAACA( NULL );
#ifdef _DEBUG
		const Asura_Hierarchy_Anim*			pxHierarchyAnim	= Asura_Animation_System::GetAnimByHash(uTurnAnimHash);
		if ( pxHierarchyAnim != NULL )
		{
			const char*							szAnimName		= pxHierarchyAnim->GetName();
			pxAACA = pxAnimation->CreateControllerAnimation( szAnimName, uFlags, uPriority, uTurnAnimHash, pxBlend );
		}
		else
#endif
		{
			pxAACA = pxAnimation->CreateControllerAnimation( uFlags, uPriority, uTurnAnimHash, pxBlend );
		}

#if defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)
		Asura_FacialAnimation_AnimationHandler*	pxFacialAnimationHandler	= Asura_FacialAnimation_System::FindAnimationHandler(m_xOwnerActor.GetGuid());

		// Play the facial-animation (if there is one)...
		if(pxFacialAnimationHandler != NULL)
		{
			Asura_FacialAnimation_FaceAnimation* pxFaceAnimation	= Asura_FacialAnimation_System::FindFaceAnimation(uTurnAnimHash);

			if(pxFaceAnimation != NULL)
			{
				pxFacialAnimationHandler->PlayFaceAnimation(uTurnAnimHash);
			}
		}
#endif	//defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)

		if ( pxAACA != NULL )
		{
			m_fCurrentTurnAngVel = m_fCurrentTurnAngle / pxAACA->GetDuration();
			m_xOwnerActor.SetTurnAnimAngularVelocityMultiplier( fAngVelScale );
			if (pxAACA->GetAnimation())
			{
				Blink();
				m_uTurnAACAHash = pxAACA->GetControllerHash();
			}
			else
			{
				pxAnimation->DestroyController(pxAACA->GetControllerHash());
				m_uTurnAACAHash = ASURA_HASH_ID_UNSET;
			}
		}
		else
		{
			delete pxBlend;
			m_uTurnAACAHash = ASURA_HASH_ID_UNSET;
		}
	}
}

void AVP_Animation_BodyRig::StopTurnAnim()
{
	AVP_Actor_AnimationHandler* pxAnimHandler = m_xOwnerActor.GetAnimationHandler();
	Asura_Animation* pxAnimation = pxAnimHandler ? pxAnimHandler->GetAnimation() : NULL;
	if ( pxAnimation )
	{
		if ( m_uTurnAACAHash != ASURA_HASH_ID_UNSET )
		{
			Asura_AnimationController_Animation* pxAACA = pxAnimation->GetControllerAnimationFromHash( m_uTurnAACAHash );
			if ( pxAACA )
			{
				// force this anim to start tweening out early
				pxAACA->SetParametricTargetTime( pxAACA->GetParametricTimer() );
			}
			m_uTurnAACAHash = ASURA_HASH_ID_UNSET;
		}
	}
}

void AVP_Animation_BodyRig::UpdateBlinking()
{
	m_fBlinkTimer -= Asura_Timers::GetGameTimeSlice();
	switch ( m_eBlinkState )
	{
		case BLINK_OPEN:
		{
			m_fEyeCurrentClose = m_fEyeRestClose;
			if ( m_fBlinkTimer < 0.0f )
			{
				m_fBlinkTimer = m_fHalfBlinkDuration;
				m_eBlinkState = BLINK_CLOSING;
			}
		}
		break;

		case BLINK_CLOSING:
		{
			m_fEyeCurrentClose = Asura_Maths::Lerp( 1.0f, m_fEyeRestClose, Asura_Maths::ClampToRange( m_fBlinkTimer / m_fHalfBlinkDuration, 0.0f, 1.0f ) );
			if ( m_fBlinkTimer < 0.0f )
			{
				m_fBlinkTimer = m_fHalfBlinkDuration;
				m_eBlinkState = BLINK_OPENING;
			}
		}
		break;

		case BLINK_OPENING:
		{
			m_fEyeCurrentClose = Asura_Maths::Lerp( m_fEyeRestClose, 1.0f, Asura_Maths::ClampToRange( m_fBlinkTimer / m_fHalfBlinkDuration, 0.0f, 1.0f ) );
			if ( m_fBlinkTimer < 0.0f )
			{
				m_fBlinkTimer = Asura_Random::GetFloat( m_fMinBlinkTime, m_fMaxBlinkTime );
				m_eBlinkState = BLINK_OPEN;
			}
		}
		break;

		default:
		{
			Asura_Assert( false, "Unhandled case" );
		}
		break;
	}
}

void AVP_Animation_BodyRig::UpdateLookAtCamera()
{
	bool bCanLookAtCamera = false;
	Asura_Vector_3 xEyesPos;
	Asura_Matrix_3x3 xEyesOri;
	m_xOwnerActor.GetWorldSpaceMarkerPoint( 0x002fbc62, &xEyesPos, &xEyesOri ); // eyes
	Asura_AVector3 xToCamera = Asura_AVector3( *Asura_Camera::GetPosition() - xEyesPos );
	float fDistToCamera = xToCamera.NormaliseReturningMagnitude();
	Asura_AVector3 xEyesFwd( xEyesOri.e13, xEyesOri.e23, xEyesOri.e33 );
	if ( fDistToCamera < 5.0f && xEyesFwd * xToCamera > 0.5f ) // 60 degrees
	{
		bCanLookAtCamera = true;
	}

	m_fLookTimer -= Asura_Timers::GetGameTimeSlice();
	switch ( m_eLookState )
	{
		case LOOKSTATE_TWEENIN:
		{
			m_fLookAtCameraParameter = Asura_Maths::ClampToRange( 1.0f - ( m_fLookTimer / m_fLookTweenTime ), 0.0f, 1.0f );
			if ( m_fLookTimer < 0.0f )
			{
				m_fLookTimer = Asura_Random::GetFloat( m_fMinLookAtCameraTime, m_fMaxLookAtCameraTime );
				m_eLookState = LOOKSTATE_CAMERA;
			}
		}
		break;

		case LOOKSTATE_CAMERA:
		{
			if ( !bCanLookAtCamera || m_fLookTimer < 0.0f )
			{
				Blink();
				m_fLookTimer = m_fLookTweenTime;
				m_eLookState = LOOKSTATE_TWEENOUT;
			}
		}
		break;

		case LOOKSTATE_TWEENOUT:
		{
			m_fLookAtCameraParameter = Asura_Maths::ClampToRange( m_fLookTimer / m_fLookTweenTime, 0.0f, 1.0f );
			if ( m_fLookTimer < 0.0f )
			{
				m_fLookTimer = Asura_Random::GetFloat( m_fMinLookFwdTime, m_fMaxLookFwdTime );
				m_eLookState = LOOKSTATE_FWD;
			}
		}
		break;

		case LOOKSTATE_FWD:
		{
			if ( bCanLookAtCamera && m_fLookTimer < 0.0f )
			{
				Blink();
				m_fLookTimer = m_fLookTweenTime;
				m_eLookState = LOOKSTATE_TWEENIN;
			}
		}
		break;

		default:
		{
			Asura_Assert( false, "Unhandled case" );
		}
		break;
	}
}

#ifndef ASURA_NO_DEBUG_OUTPUT
void AVP_Animation_BodyRig::RenderDebugOutput()
{
	if ( dvs_bRenderDebugOutput )
	{
		const Asura_Matrix_3x3& xOri = m_xOwnerActor.GetCurrentOrientation();
		Asura_Vector_3 xPos( m_xOwnerActor.GetCurrentPosition() - Asura_Vector_3( xOri.e12, xOri.e22, xOri.e32 ) );
		Asura_AVector3 xAPos( xPos );
		Asura_AVector3 xAimDirection;
		Asura_AVector3 xLookAtPos;
		GetAimDirection( xAimDirection );
		GetLookAtPosition( xLookAtPos );
		Asura_DebugLineRender::AddArrow( xAPos, xAPos + xAimDirection, 0xFFFFFF00, 0.1f, 0.1f );
		Asura_DebugLineRender::AddArrow( xAPos, xLookAtPos, 0xFF00FFFF, 0.1f, 0.1f );
	}
}

#endif
