//////////////////////////////////////////////////////////////////////////////
//
// AVP_Chunk_Loaders.cpp
//
// Copyright (c) 2006 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header.
#include "ChunkSystem/AVP_Chunk_Loaders.h"

// Engine independent.
#include "ChunkSystem/Asura_Chunk_Stream.h"

// Project independent.
#include "ChunkSystem/AVP_Chunk_Alien_PlayerAnimStateData.h"
#include "ChunkSystem/AVP_Chunk_Human_PlayerAnimStateData.h"
#include "ChunkSystem/AVP_Chunk_Predator_PlayerAnimStateData.h"
#include "ChunkSystem/AVP_Chunk_Navigation.h"
#include "ChunkSystem/AVP_Chunk_SurfaceOverlay.h"
#include "ChunkSystem/AVP_Chunk_WeaponData.h"

// C L A S S   I M P L E M E N T A T I O N S /////////////////////////////////

bool AVP_ChunkLoading_System::ParseChunk( Asura_Chunk_Header* pxChunkHdr, Asura_Chunk_File_Stream& xFileStream )
{
    #define CHUNK_PROCESS( chunkID, chunkClass ) \
        case (chunkID): \
        { \
            xFileStream.ReadChunk_GetNextHeader( pcChunkData, sizeof(chunkClass) ); \
            chunkClass* pxSpecificChunkHdr = reinterpret_cast<chunkClass*>( pcChunkData ); \
            pxSpecificChunkHdr->ReadFromChunkStream( xFileStream ); \
            return true; \
        }

    char* pcChunkData = reinterpret_cast<char*>( pxChunkHdr );

    switch( static_cast<u_int>(pxChunkHdr->ID) )
    {
		CHUNK_PROCESS( AVP_CHUNKID_SURFACEEFFECTS,					AVP_Chunk_SurfaceOverlay )
        CHUNK_PROCESS( AVP_CHUNKID_NAVIGATION,						AVP_Chunk_Navigation )
		CHUNK_PROCESS( AVP_CHUNKID_WEAPONDATA,						AVP_Chunk_WeaponData )
		CHUNK_PROCESS( AVP_CHUNKID_ALIEN_PLAYERANIMSTATEDATA,		AVP_Chunk_Alien_PlayerAnimStateData )
		CHUNK_PROCESS( AVP_CHUNKID_HUMAN_PLAYERANIMSTATEDATA,		AVP_Chunk_Human_PlayerAnimStateData )
		CHUNK_PROCESS( AVP_CHUNKID_PREDATOR_PLAYERANIMSTATEDATA,	AVP_Chunk_Predator_PlayerAnimStateData )

	    default:
            return false;
    }

    #undef CHUNK_PROCESS
}
