
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_Connection.h
// Author : Anthony Gregson
// Date   : 21/11/07
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __AVP_CONNECTION_H__
#define __AVP_CONNECTION_H__



////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////
#include "Core/AVP_StateMachine.h"


////////////////////////////////////////////////////////////////////////////////
// Forward class declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Class declaration...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Class  : AVP_Connection
// Author : Anthony Gregson
// Date   : 21/11/07
////////////////////////////////////////////////////////////////////////////////

class AVP_Connection : public AVP_StateMachine
{
public:
	enum State
	{
		STATE_INVALID,
		STATE_WAITFORPLAYERS,
		STATE_LOADING,
		STATE_WAITFORGUID,
		STATE_BEGINGAMEINITIALISATION,
		STATE_ENDGAMEINITIALISATION,
		STATE_PLAYING, // This has a sub-state machine - AVP_StateMachine_Playing.
		STATE_MAPCHANGE,
		STATE_TERMINATED,

		MAX_STATES
	};

	AVP_Connection();
	virtual ~AVP_Connection() {}

	// State machine overrides
	virtual void Initialise();
	virtual void Shutdown();

	// Helper functions for states to check network status and terminate the connection with reasons.
	bool ValidateNetworkConnection();
	void Terminate(int iNetworkDisconnectReason);
	int GetNetworkDisconnectReason() const { return m_iNetworkDisconnectReason; }

protected:
#ifdef AVP_STATEMACHINE_LOGGING
	virtual const char* GetStateMachineName() { return "AVP_Connection"; }
#endif //#ifdef AVP_STATEMACHINE_LOGGING

private:
	// This value is intended to be used by the Terminated state, in order to display any required error messages.
	int m_iNetworkDisconnectReason;

	typedef AVP_StateMachine PARENT;
};   // End of class 'AVP_Connection'.


#endif   // ifndef __AVP_CONNECTION_H__


/// End of file. ///
