//////////////////////////////////////////////////////////////////////////////
//
// AVP_Core.cpp
//
// Copyright (c) 2005 Rebellion. All rights reserved.
//
// Description:
//
//     basic shell to handle switching between Game and Tools
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header.
#include "Core/AVP_Core.h"
#include "Core/Asura_Core.h"

// Asura headers
#include "Blueprint/Asura_Blueprint_System.h"
#include "Botanicals/Asura_Botanical_Game_Render_Manager.h"
#include "Botanicals/Asura_Botanicals_InstanceManager.h"
#include "Camera/Asura_Camera.h"
#include "Camera/AVP_CameraShake_System.h"
#include "Carpet/Asura_Carpet_MeshManager.h"
#include "ChunkSystem/Asura_SaveBlock.h"
#include "ChunkSystem/Asura_Chunk_Loaders.h"
#ifdef AVP_LOG_FILE_LOAD_MEMORY_USE
#include "Core/Asura_LocalisedText.h"
#endif //#ifdef AVP_LOG_FILE_LOAD_MEMORY_USE
#include "Core/Asura_MaterialResponse_System.h"
#include "Core/Asura_TextureManagement.h"
#include "Core/Asura_TiledTextureManagement.h"
#include "Core/Asura_Timers.h"
#include "Core/Asura_UserInput.h"
#include "Cutscene/Asura_Cutscene_System.h"
#include "Debugging/Asura_ScreenShot_System.h"
#include "Entity/Asura_Entity_System.h"
#include "Entity/Asura_StaticMessage_System.h"
#include "Entity/PhysicalObject/Asura_Dynamic_Extents.h"
#include "Entity/PhysicalObject/Moves/Asura_Movement_System.h"
#include "Entity/Triggers/Asura_ServerEntity_DebugMessageTrigger.h"
#include "Entity/Triggers/Asura_ServerEntity_DebugWaypointTrigger.h"
#if defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)
#include "FacialAnimation/Asura_FacialAnimation_System.h"
#endif	//defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)
#ifdef ASURA_USE_VIDEO_TO_TEXTURE_SYSTEM
#include "FMV/Asura_VideoToTexture_System.h"
#endif
#include "Input/Asura_ControlManager.h"
#include "InputSimulator/AVP_InputSimulator_System.h"
#if defined(ASURA_SUPPORT_COMMAND_CONSOLE)
#include "Interface/Asura_CommandConsole.h"
#endif	// defined(ASURA_SUPPORT_COMMAND_CONSOLE)
#include "Interface/Asura_Cursor.h"
#include "Interface/Asura_Overlay_System.h"
#include "LoadAndSave/Asura_LoadAndSave.h"
#include "Mesh/Asura_Env_System.h"
#include "Network/Asura_NetLinkManager.h"
#include "Network/Asura_NetMsg.h"
#include "Network/Asura_Network.h"
#ifdef ASURA_SUPPORT_LOBBY
#include "Network/Online/Asura_Lobby.h"
#endif
#include "Physics/Asura_Physics.h"
#include "Rendering/Asura_AmbientOcclusion.h"
#include "Rendering/Asura_DeferredShading.h"
#include "Rendering/Asura_Font_System.h"
#include "Rendering/Asura_Module.h"
#include "Rendering/Asura_Render.h"
#include "Rendering/Asura_Viewport.h"
#include "Rendering/Asura_Primitive_Render.h"
#include "Sound/Asura_Audio_System.h"
#include "Sound/Asura_Audio_Systems_Manager.h"
#include "Sound/Asura_Sound.h"
#include "Sound/Asura_Dialogue_System.h"
#include "SpecialFX/Asura_Decal_System.h"
#include "SpecialFX/Asura_Skybox.h"
#include "SpecialFX/Asura_SpecialFX.h"
#include "SpecialFX/pfx/Asura_PFX_System.h"
#include "Teleport/Asura_Teleport_System.h"
#if defined(ASURA_SUPPORT_UNITTEST)
#include "UnitTest/Asura_UnitTest.h"
#endif	// defined(ASURA_SUPPORT_UNITTEST)

#ifdef ASURA_TOOLS
#include "PC/FileSystem/Tools_FileSystem.h"
#endif

#ifdef ASURA_USE_STREAMING
#include "Streaming/Asura_Streaming_System.h"
#endif //#ifdef ASURA_USE_STREAMING

// Game headers
#if defined(ASURA_SUPPORT_COMMAND_CONSOLE)
	#include "Actor/AVP_Actor_HUDHandler.h"
	#include "Actor/AVP_Actor_InteractionHandler.h"
	#include "Actor/Vertebrates/Alien/AVP_Alien_HUDHandler.h"
	#include "Actor/Vertebrates/Alien/AVP_Alien_InputHandler.h"
	#include "Actor/Vertebrates/Alien/AVP_Alien_JumpHandler.h"
	#include "Actor/Vertebrates/Human/AVP_Human_InteractionHandler.h"
	#include "Actor/Vertebrates/Human/AVP_Human_HUDHandler.h"
	#include "Actor/Vertebrates/Predator/AVP_Predator_HUDHandler.h"
	#include "Actor/Vertebrates/Predator/AVP_Predator_InteractionHandler.h"
	#include "Actor/Vertebrates/Predator/AVP_Predator_MeleeCounter_System.h"
#endif	// defined(ASURA_SUPPORT_COMMAND_CONSOLE)	
#include "Actor/Vertebrates/AVP_Vertebrate_AnimationHandler.h"
#include "Actor/Vertebrates/Alien/AVP_Alien_AIAnimStateData.h"
#include "Actor/Vertebrates/Alien/AVP_Alien_Description.h"
#include "Actor/Vertebrates/Alien/AVP_Alien_PlayerAnimStateData.h"
#include "Actor/Vertebrates/AlienQueen/AVP_AlienQueen_AIAnimStateData.h"
#include "Actor/Vertebrates/FaceHugger/AVP_FaceHugger_AIAnimStateData.h"
#include "Actor/Vertebrates/Predator/AVP_Predator_LockOn.h"
#include "AI/Asura_AI_Navigation.h"
#include "Attractors/AVP_Attractors_System.h"
#include "Blueprint/AVP_Blueprint_HashIDs.h"
#include "ChunkSystem/AVP_Chunk_Loaders.h"
#include "Core/AVP_Game_Core.h"
#include "Core/AVP_GameStatus.h"
#include "Core/AVP_Settings.h"
#include "Core/AVP_Waypoint_Manager.h"
#include "Core/Axon_Project_System.h"
#include "Entity/AVP_Entity_Classification.h"
#include "Entity/PhysicalObject/Actor/AVP_ServerEntity_Actor.h"
#if defined(AVP_USE_DEV_LOBBY)
#include "FrontEnd/AVP_DevLobby.h"
#endif	//defined(AVP_USE_DEV_LOBBY)
#if defined(AVP_USE_GAME_LOBBY)
#include "FrontEnd/AVP_GameLobby.h"
#endif	//defined(AVP_USE_GAME_LOBBY)
#if defined(ASURA_USE_GUI_MENU_SYSTEM)
#include "GUIMenu/AVP_GUIMenu_System.h"
#endif
#include "Multiplayer/AVP_Multiplayer_GameCore.h"
#include "Network/AVP_Client.h"
#include "Network/AVP_Player.h"
#include "Network/AVP_Server.h"

#include "SpecialFX/AVP_CollisionFX_System.h"
#include "SpecialFX/AVP_Laser.h"
#include "SpecialFX/AVP_CloakEffect.h"
#include "SpecialFX/AVP_GlowOutlines.h"
#include "Rendering/AVP_Platform_Stencil_Setup.h"

#ifdef ASURA_USE_STREAMING
#include "Streaming/AVP_Streaming_MemoryBankHandler.h"
#endif //#ifdef ASURA_USE_STREAMING

#include "Core/Axon_System.h"

#include "SpecialFX/AVP_Platform_PerlinNoise.h"

// Tools headers
#ifdef ASURA_TOOLS
#include "Load&Save/LevelExport/Tools_Level_Export.h"
#include "Load&Save/Tools_OptionsFile.h"
#include "Misc/Tools_Render_Options.h"
#include "Tools_System.h"
#endif

//private static data
const char* AVP_Core::s_aszFontNames[MAX_FONT_TYPES] =
{
	"\\fonts\\Continuum Medium_14",
	"\\fonts\\Continuum Medium_14",
#if defined(ASURA_USE_DEBUG_MESSAGE_TRIGGER)
	"\\fonts\\Continuum Medium_14",
#endif	// defined(ASURA_USE_DEBUG_MESSAGE_TRIGGER)
#if defined(ASURA_SUPPORT_COMMAND_CONSOLE)
	"\\fonts\\Continuum Medium_14",
#endif	// defined(ASURA_SUPPORT_COMMAND_CONSOLE)
};

Asura_Handle					AVP_Core::s_aiFontHandles[MAX_FONT_TYPES];

AVP_StateMachine_Core			AVP_Core::s_xStateMachine;
bool							AVP_Core::s_bShouldUpdateSoundSystem = true;

bool                            AVP_Core::s_bShowDialogue			 = false;
Asura_UnicodeChar               AVP_Core::s_aswzDialogue[256]; 

#ifdef ASURA_TOOLS
bool							AVP_Core::s_bToolsRequested;
class Asura_ProtoSaveBlock*		AVP_Core::s_pxToolsQuickSaveBlock;
#else
u_int							AVP_Core::s_uDesiredScreenWidth		= 1280;
u_int							AVP_Core::s_uDesiredScreenHeight	= 720;
Asura_Render::ScreenBorderType	AVP_Core::s_eScreenBorderType		= Asura_Render::SCREEN_BORDER_DISABLED;
#endif
bool							AVP_Core::g_bRenderHUDToFrontBuffer = true;

#ifdef ASURA_SUPPORT_DOWNLOADABLE_CONTENT
bool AVP_Core::s_bDLCRequiresFilesUpdate = true;
#endif //#ifdef ASURA_SUPPORT_DOWNLOADABLE_CONTENT

// nasty globals, should use evilg_ prefix
char*		DefaultFileToLoad;	// ugh...
#ifdef ASURA_TOOLS
// this is used when saving files
Asura_String EnvironmentFilename;
#endif

#ifdef ASURA_USE_STREAMING
// Streaming zone data. Real projects may want to create these arrays as and when they're needed, so they can change the block allocations according to the mode (and current state of the game?)

static const u_int g_auNumZoneDataIndices[Asura_Streaming_Zone::MAX_DATA_STATES] =
{
	ASURA_STREAMING_MAX_RESIDENT_HIGH_DETAIL_ZONES,
#ifndef ASURA_TOOLS
	ASURA_STREAMING_MAX_RESIDENT_LOW_DETAIL_ZONES,
#endif //#ifndef ASURA_TOOLS
};

static const u_int g_auNumMissionDataIndices[Asura_Streaming_Zone::MAX_DATA_STATES] =
{
	ASURA_STREAMING_MAX_RESIDENT_MISSIONS,
#ifndef ASURA_TOOLS
	0,
#endif //#ifndef ASURA_TOOLS
};

#endif //#ifdef ASURA_USE_STREAMING

static const float fAVP_MAX_TIME_UPDATING_OCCLUSIONS = (1.0f / 30.0f) / 200.0f;	//0.5 percent of the frame @ 30fps

static float AVP_GetAudioOccl( Asura_Hash_ID uMaterialHash )
{
			float				fResult				= 0.0f;
	const	float				fDefaultOcclusion	= 0.0f;
	const	Asura_Blueprint*	pxBluePrint			= Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_SURFACEEFFECTS, uMaterialHash );
	
	fResult = pxBluePrint ? pxBluePrint->GetParameterAsFloat( 0x0B43B613 /*Asura_GetHashID( "AudioOccl" )*/, fDefaultOcclusion ) : fDefaultOcclusion;

	return	fResult;
}

static float AVP_GetOcclusionFactor_Entity(const Asura_LOS_OutputData* pxLoSData)
{
	float									fResult = 0.0f;
	AVP_ServerEntity_PhysicalObject*		pxServerEntityPhysicalObject	= AVP_ServerEntity_PhysicalObject::FindEntity( pxLoSData->m_uHitEntity );
	
	fResult = pxServerEntityPhysicalObject ? AVP_GetAudioOccl( pxServerEntityPhysicalObject->GetMaterialType() ) : 0.0f;

	return fResult;
}

static float AVP_GetOcclusionFactor_Environment(const Asura_LOS_OutputData* pxLoSData)
{
	float							fResult = 0.0f;
	const	Asura_MaterialResponse*	pxMaterial = Asura_MaterialResponse_System::GetMaterialByHashID( pxLoSData->m_uMaterialResponseHashID );
	
	fResult = pxMaterial ? AVP_GetAudioOccl( pxMaterial->GetSurfaceTypeHash() ) : 0.0f;

	return fResult;
}

//////////////////////////////////////////////////////////////////////////
// Initialise and Shutdown
//////////////////////////////////////////////////////////////////////////

void AVP_Core::EarlyInitialise(void)
{
	SetProjectSpecificChunkParser( AVP_ChunkLoading_System::ParseChunk );

#ifdef ASURA_USE_COLLISIONFX
	Asura_CollisionFX_System::SetCallback(AVP_CollisionFX_System::CreateEffect);
	Asura_CollisionFX_System::SetThresholdMinSpeed( fAVP_COLLISION_SFX_SPEED_THRESHOLD );
#endif //ASURA_USE_COLLISIONFX

	// State-machine initialisation...
	AVP_StateMachine::LogInitialise();

#ifdef ASURA_SUPPORT_NETMSG_SYSTEM
	Asura_NetMsg_System::SetPlayerEntityClassForProject(AVP_ENTITYCLASS_PLAYER);
#endif //#ifdef ASURA_SUPPORT_NETMSG_SYSTEM

	// Prevent us from resetting the sound-system too early...
	#ifndef ASURA_USE_AUDIO_SYSTEM
	Asura_Sound_System::DisableResetPointDuringInitialisation();
	#endif
#if defined(ASURA_USE_GUI_MENU_SYSTEM)
	Asura_GUIMenu_System::DisableLoadGenericAudioDataDuringInitialisation();
#endif	//defined(ASURA_USE_GUI_MENU_SYSTEM)

	Asura_MaterialResponse_System::SetDefaultSurfaceTypeHash(0x5C13D641);	//Asura_GetHashID("Default"));
	Asura_MaterialResponse_System::SetMaterialProjectContextCallback(AVP_PlatformStencilSetup::MaterialContextCallback);

	Asura_PFX_System::SetClearTemplatesOnReset(false);

#ifdef AVP_LOG_FILE_LOAD_MEMORY_USE
	ClearFileLoadMemoryUseLogfile();
#endif //#ifdef AVP_LOG_FILE_LOAD_MEMORY_USE
}

void AVP_Core::LateInitialise(void)
{
	AVP_Settings::Initialise();
    AVP_Attractors_System::Initialise();
	AVP_Waypoint_Manager::Initialise();
    Platform_Initialise();

#ifdef ASURA_TOOLS
	// Allow PSP scaling to be handled in Asura...
	Tools_Render_Options::EnableOverlayScaling(true);

	// Setup the quicksave block for tools
	s_pxToolsQuickSaveBlock = new Asura_SaveBlock();
	s_pxToolsQuickSaveBlock->Clear();
	s_pxToolsQuickSaveBlock->AddAsuraTerminator();
#else
	Asura_Timers::SetMaxFrameRate(AVP_Settings::GetMaxFrameRate());
#endif // #ifdef ASURA_TOOLS

#ifdef ASURA_USE_STREAMING
	Asura_Streaming_System::SetMissionHasPriority(false);
#ifdef ASURA_RESOURCE_ALLOCATOR_USES_MEMORY_BANK
	// Initialise streaming memory
	AVP_Streaming_MemoryBankHandler::Initialise();
	AVP_Streaming_MemoryBankHandler::InitialiseSinglePlayerGameMemoryBanks();
#endif //#ifdef ASURA_RESOURCE_ALLOCATOR_USES_MEMORY_BANK
#endif //#ifdef ASURA_USE_STREAMING
	
#ifdef ASURA_TOOLS
	// Load in the HUD files
	AVP_Core::LoadFile( "Graphics\\Marine_HUD_Elements."ASURA_STANDARD_FILE_EXTENSION, ASURA_TEXTURE_RESETTABLE_GROUP_COMMON );
	AVP_Core::LoadFile( "Graphics\\Predator_HUD_Elements."ASURA_STANDARD_FILE_EXTENSION, ASURA_TEXTURE_RESETTABLE_GROUP_COMMON );
	AVP_Core::LoadFile( "Graphics\\Alien_HUD_Elements."ASURA_STANDARD_FILE_EXTENSION, ASURA_TEXTURE_RESETTABLE_GROUP_COMMON );
#endif

    // Load Font
#if defined(ASURA_ASSERT)
	bool bOK = 
#endif
	AVP_Core::LoadFile( "Fonts\\Fonts."ASURA_STANDARD_FILE_EXTENSION, ASURA_TEXTURE_LOADED_PERMANENT );
	Asura_Assert(bOK, "failed to load fonts file");

	for(u_int u = 0; u < MAX_FONT_TYPES; u++)
	{
		const char*	szFontName	= s_aszFontNames[u];

		if(Asura_Font_System::IsFontLoaded(szFontName))
		{
			s_aiFontHandles[u]	= Asura_Font_System::GetFontHandle(szFontName);
		}
		else
		{
			s_aiFontHandles[u]	= ASURA_INVALID_HANDLE;
		}
	}

	// Tidy-up the tiled-textures...
	Asura_TiledTextureManagement::SetTextureHandlesFromNames();

	// Ensure "Asura_ServerEntity_PhysicalObject::ResetCollisionObject()" doesn't bin our collisions...
	Asura_Dynamic_Extents::SetExtentsHandler(GetPhysicalObjectCollisionObject);

#if defined(ASURA_SUPPORT_UNITTEST)
	// Time to handle the unit-tests...
	Asura_UnitTest_RunAllTests();
#endif

	AVP_Platform_PerlinNoise::Initialise();

#ifdef ASURA_MAX_NO_OF_STORED_POSITIONS
	Asura_Physics_System::SetEndOfPhysicsUpdateCallback( AVP_ServerEntity_PhysicalObject::EndOfPhysicsUpdateCallback ); 
#endif // #ifdef ASURA_MAX_NO_OF_STORED_POSITIONS

#if defined(ASURA_USE_GUI_MENU_SYSTEM)
	AVP_GUIMenu_System::Initialise();
#endif //defined(ASURA_USE_GUI_MENU_SYSTEM)

#if defined(ASURA_SUPPORT_COMMAND_CONSOLE)
	CreateConsoleVars();
	CreateConsoleCommands();
#endif	// defined(ASURA_SUPPORT_COMMAND_CONSOLE)	

#if defined(ASURA_USE_DIALOGUE_SYSTEM)
	Asura_Dialogue_System::SetProjectRenderSubtitles(RenderDialogueSubtitles);
#endif	//defined(ASURA_USE_DIALOGUE_SYSTEM)

#ifdef ASURA_SUPPORT_NETMSG_SYSTEM
	// only need to do this once at startup, not every time network is initialised.
	Asura_NetMsg_System::AllowLODUpdates(true);
	Asura_NetMsg_System::SetUseLODUpdates(true);
#endif


#if ASURA_NETWORK_LOGGING
	// (MH - 06.03.2007) check all network message IDs are registered
	for (u_int u = 0; u < ASURA_MSG_END_OF_ENGINE_MESSAGES; ++u)
	{
		const char* szDesc;
		Asura_Network::GetMsgIDDesc( u, &szDesc );
		Asura_String xString(szDesc);
		Asura_Assertf( !xString.Contains("["), ("failed to get description for message ID %u\n"
			"A programmer needs to add it to Asura_Network::GetMsgIDDesc()", u) );
	}
	for (u_int u = AVP_MSGID_START_OF_STATIC_MESSAGES + 1; u < AVP_MSG_ID_MAX; ++u)
	{
		const char* szDesc;
		Asura_Network::GetMsgIDDesc( u, &szDesc );
		Asura_String xString(szDesc);
		Asura_Assertf( !xString.Contains("["), ("failed to get description for message ID %u\n"
			"A programmer needs to add it to Asura_Network::Project_GetMsgIDDesc()", u) );
	}
	for (u_int u = 0; u < AsuraEntityClass_LastEngineClassification; ++u)
	{
		const char* szDesc = "unknown";
		Asura_Network::GetEntityClassificationDesc( static_cast<u_short>(u), &szDesc );
		Asura_String xString(szDesc);
		Asura_Assertf( !xString.Contains("unknown"), ("failed to get description for entity classification %u.\n"
			"A programmer needs to add it to Asura_Network::GetEntityClassificationDesc()", u) );
	}
	for (u_int u = AsuraEntityClass_ProjectSpecific; u < AVP_ENTITYCLASS_MAX; ++u)
	{
		const char* szDesc = "unknown";
		Asura_Network::GetEntityClassificationDesc( static_cast<u_short>(u), &szDesc );
		Asura_String xString(szDesc);
		Asura_Assertf( !xString.Contains("unknown"), ("failed to get description for entity classification %u.\n"
			"A programmer needs to add it to Asura_Network::Project_GetEntityClassificationDesc()", u) );
	}
#endif

#if !defined(ASURA_TOOLS)
	// Load the animation-templates, blueprints and loading screens....
	Asura_Resource_System::CreateResourceBank(uAVP_RESOURCE_BANK_HASH_COMMON, szAVP_RESOURCE_BANK_NAME_COMMON, uAVP_RESOURCE_BANK_TRG_COMMON);
	AVP_Core::LoadFile("Misc\\Common."ASURA_STANDARD_FILE_EXTENSION, ASURA_TEXTURE_LOADED_PERMANENT);
#endif	//!defined(ASURA_TOOLS)

	// Set up the occlusion properties in the sound system (feel free to move this)
	Asura_Audio_System::SetEntityOcclusionFactorCallback		( AVP_GetOcclusionFactor_Entity );
	Asura_Audio_System::SetEnvironmentOcclusionFactorCallback	( AVP_GetOcclusionFactor_Environment );

	Asura_Audio_System::SetOcclusionCPUTimePerFrame ( fAVP_MAX_TIME_UPDATING_OCCLUSIONS );
	AVP_CameraShake_System::Initialise();


	// Some support for a basic Debug catagory for Asura_Primive_Render, on in non-submission builds.
#ifndef AVP_SUBMISSION
	// Turn on primitive rendering
	Asura_Primitive_Render::SetActive(true);
	// Register general debug catagory
	Asura_Primitive_Render::RegisterCategory("DEBUG");
	// Enable general debug catagory
	Asura_Primitive_Render::EnableCategory(Asura_GetHashID("DEBUG"), true);
	// Set rendering parameters to default. Note you may have to call this again before any 
	// rendering call as theres no guarantee someone hasnt set something silly elsewhere.
	Asura_Primitive_Render::UseDefaultRenderParams();
#endif


#if defined(AVP_USE_DEV_LOBBY)
	AVP_DevLobby::AddConsoleCmdsAndVars();
#endif	//defined(AVP_USE_DEV_LOBBY)
#if defined(AVP_USE_GAME_LOBBY)
	AVP_GameLobby::AddConsoleCmdsAndVars();
#endif	//defined(AVP_USE_GAME_LOBBY)
#ifdef AVP_USE_INPUT_SIMULATOR
	AVP_InputSimulator_System::Initialise();
#endif
}


void AVP_Core::Reset(void)
{
	// Asura_Core::Reset() resets the timers.
	// Don't reset the timers on the server if online unless 6 hours have elapsed.
	// TODO: this needs a proper fix at some point.
	const float fMAX_SERVER_TIME = 6.0f * 60.0f * 60.0f;
	// store old game time
	float fServerAbsoluteTime = Asura_Timers::GetServerAbsoluteTime();
	
	// reset
	Asura_Core::Reset();
	
	if (Asura_Network::IsServer() && Asura_Network::IsExternalNetworkActive() && 
		fServerAbsoluteTime < fMAX_SERVER_TIME)
	{
		// restore the old game time
		Asura_Timers::UpdateServerAbsoluteTime( fServerAbsoluteTime, 0, true );
	}

	AVP_Waypoint_Manager::Reset();
	// Resetting Asura_Core may return some player-set options to default values. Reapply the settings.
	AVP_Settings::ApplyProfileSettings();
}

void AVP_Core::Shutdown(void)
{
#if defined(ASURA_USE_GUI_MENU_SYSTEM)
	AVP_GUIMenu_System::Shutdown();
#endif //defined(ASURA_USE_GUI_MENU_SYSTEM)

#ifdef ASURA_SUPPORT_LOBBY

#if defined(AVP_USE_DEV_LOBBY)
	AVP_DevLobby::Shutdown();
#endif

#if defined(AVP_USE_GAME_LOBBY)
	AVP_GameLobby::Shutdown();
#endif

	Asura_Lobby_System::Shutdown();
#endif // #ifdef ASURA_SUPPORT_LOBBY


	ShutdownNetwork();

#ifdef ASURA_TOOLS
	delete s_pxToolsQuickSaveBlock;
	s_pxToolsQuickSaveBlock = NULL;
#endif

#if defined (ASURA_USE_STREAMING) && defined (ASURA_RESOURCE_ALLOCATOR_USES_MEMORY_BANK)
	// shutdown streaming memory
	AVP_Streaming_MemoryBankHandler::ShutdownGameMemoryBanks();
	AVP_Streaming_MemoryBankHandler::Shutdown();
#endif //#if defined (ASURA_USE_STREAMING) && defined (ASURA_RESOURCE_ALLOCATOR_USES_MEMORY_BANK)

#ifdef AVP_USE_INPUT_SIMULATOR
	AVP_InputSimulator_System::Shutdown();
#endif
	AVP_Platform_PerlinNoise::Shutdown();

	Platform_Shutdown();
    AVP_Attractors_System::Shutdown();
	AVP_Settings::Shutdown();
	AVP_Waypoint_Manager::Shutdown();
	AVP_CameraShake_System::Shutdown();
}


//////////////////////////////////////////////////////////////////////////
// Load function...
//////////////////////////////////////////////////////////////////////////
bool AVP_Core::LoadFile(const char* szFilename, const int iTextureResetGroup)
{
	bool	bLoadSuccessful;

#if !defined(ASURA_NO_DEBUG_OUTPUT)
	const Asura_TickCount xStart = Asura_Timers::GetCurTicks();
	Asura_OutputToDebugger_Retail(("About to load '%s'...\n", szFilename));
#endif	//!defined(ASURA_NO_DEBUG_OUTPUT)

#if defined(AVP_LOG_FILE_LOAD_MEMORY_USE)
	AVP_SmokeTest::LogBeginFileLoad(szFilename);
#endif	//defined(AVP_LOG_FILE_LOAD_MEMORY_USE)

	if(iTextureResetGroup < 0)
	{
		bLoadSuccessful	= Asura_ChunkLoading_System::LoadAsuraFile(szFilename);
	}
	else
	{
		bLoadSuccessful	= Asura_Texture_Management::LoadAsuraFileUsingResetGroup(szFilename, iTextureResetGroup);
	}
		
#if defined(AVP_LOG_FILE_LOAD_MEMORY_USE)
	AVP_SmokeTest::LogEndFileLoad();
#endif	//defined(AVP_LOG_FILE_LOAD_MEMORY_USE)

	if(bLoadSuccessful)
	{
#ifndef ASURA_NO_DEBUG_OUTPUT
		const Asura_TickCount xTimeInTicks = Asura_Timers::GetElapsedTicks( xStart );
		const float fTimeInSecs = Asura_Timers::TicksToSeconds( xTimeInTicks );
		Asura_OutputToDebugger_Retail(( "...that took %.3f seconds.\n", fTimeInSecs ));
#endif
		return(true);
	}
	else
	{
		Asura_OutputToDebugger_Retail(( "...failed!\n"));
		return(false);
	}
}

//////////////////////////////////////////////////////////////////////////
// Update functions...
//////////////////////////////////////////////////////////////////////////
void AVP_Core::Update(const bool bUpdateScene, AVP_Player* pxPlayer)
{
	Asura_Network::Update();

	if(bUpdateScene)
	{
		Asura_Cutscene_System::Update();
		Asura_Teleport_System::Update();
	}

#ifdef ASURA_USE_VIDEO_TO_TEXTURE_SYSTEM
	Asura_VideoToTexture_System::Update();
#endif

	if(bUpdateScene)
	{
		if(pxPlayer != NULL)
		{
			AVP_Waypoint_Manager::Update();
			Axon_System::Update();
			Asura_AI_Navigation_System::Update();
		}

#ifdef ASURA_USE_STREAMING
		Asura_Streaming_System::UpdateEntities();
#else
		// Only update entities if the game is not paused
		Asura_Entity_System::Update();
#endif
		// Now update all the movement systems that have queued themselves up.
		Asura_Movement_System::DeferredUpdate();

		Asura_Physics_System::Update();

#ifdef ASURA_USE_COLLISIONFX
		Asura_CollisionFX_System::Update();
#endif //ASURA_USE_COLLISIONFX

#if defined ASURA_BOTANICAL_SUPPORT_STATES
		Asura_BGR_Manager& xBGRManager = Asura_BGR_Manager::GetBGRManager();
		xBGRManager.Update( Asura_Timers::GetGameTimeSlice());
#endif // #ifdef ASURA_USE_BOTANICALS
	}

#ifdef ASURA_TOOLS
	Asura_Cursor::OncePerFrameUpdate();
	Asura_Overlay_System::Update(false);
#endif

	if(bUpdateScene)
	{
		//RAR 26/07/2006 If any overlays are open they will be making use of any mouse/keyboard commands
		if(pxPlayer != NULL)
		{
			Asura_Network::BeginClientToServerUpdates();

			pxPlayer->Update();

			Asura_Network::EndClientToServerUpdates();
		}
	}

	// AJK; E3 Update FSFX even in pause menu
	Asura_SpecialFX_System::Update();

#if defined ASURA_SUPPORT_COMMAND_CONSOLE
	Asura_CommandConsole::Update();
#endif

#ifdef ASURA_USE_STREAMING
#ifdef ASURA_TOOLS
	Asura_Streaming_System::Update(false);
#else
	Asura_Streaming_System::Update(true);
#endif
#endif // #ifdef ASURA_USE_STREAMING

#ifdef ASURA_MAX_NO_OF_STORED_POSITIONS
	bool bIsSendingClientUpdatesThisFrame = Asura_Network::IsSendingDataAtEndOfFrame(); 

	if(Asura_Network::IsNetworkInitialised())
	{
		Asura_Network::EndOfFrameUpdate();
	}

	if (Asura_Network::IsServer() && bIsSendingClientUpdatesThisFrame)
	{
		// Data for last client update has been sent, reset it 
		AVP_ServerEntity_PhysicalObject::ResetAllStoredPositions(); 
	}
#else
	if(Asura_Network::IsNetworkInitialised())
	{
		Asura_Network::EndOfFrameUpdate();
	}
#endif // #ifdef ASURA_MAX_NO_OF_STORED_POSITIONS

#ifdef ASURA_TOOLS
	//Test for taking a hi-res screenshot. Probably not the best place to do this , so feel free to
	//relocate. RAR 03/07/2007
	if(Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_PAUSE))
	{
		if (Asura_UserInput::KeyPressed(ASURA_KEYID_SHIFT))
		{
			// Shift key creates a hi-res screenshot
			Asura_ScreenShot_System::RenderHiResScreenshot(AVP_Core::RenderForScreenshot);
		}
		else
		{
			// Otherwise, a normal res shot.
			Asura_ScreenShot_System::TakeNormalScreenShot();
		}
	}
#endif

	// The last thing to do in the update is ensure that the camera is up-to-date...
#ifdef ASURA_TOOLS
	if(!Asura_Overlay_System::AreAnyOverlaysVisible())
#endif
	{
		if((pxPlayer != NULL) && !Asura_Render::GetCutsceneHasCameraControl())
		{
			pxPlayer->SetCamera();
			AVP_CameraShake_System::Update();
			AVP_CameraShake_System::ApplyShakes();
		}
	}

	s_bShowDialogue = false;
}


//////////////////////////////////////////////////////////////////////////
// Render functions...
//////////////////////////////////////////////////////////////////////////
void AVP_Core::BeginRender( bool bRenderEnvironment )
{
	Asura_Camera::SetCameraIndex(0);
	if( bRenderEnvironment )
	{
		Asura_Module_System::Update();
		Asura_Environment_System::Update();
	}

#ifdef ASURA_USE_STREAMING
	Asura_Streaming_System::PreRenderUpdate();
#endif

	Asura_Render::BeginFrame();
	Asura_Render::CurrentStates.ResetAll();

#if !defined(ASURA_TOOLS)
// 	const u_int	uActualWidth	= Asura_Viewport::GetWidth();
// 	const u_int	uActualHeight	= Asura_Viewport::GetHeight();
// 
// 	// Calculate the type of screen-border needed...
// 	if((s_uDesiredScreenHeight > uActualHeight) || (s_uDesiredScreenWidth > uActualWidth))
// 	{
// 		s_eScreenBorderType	= Asura_Render::SCREEN_BORDER_STRETCH_TO_FIT;
// 	}
// 	else if((s_uDesiredScreenHeight == uActualHeight) && (s_uDesiredScreenWidth == uActualWidth))
// 	{
// 		s_eScreenBorderType	= Asura_Render::SCREEN_BORDER_DISABLED;
// 	}
// 	else
// 	{
// 		s_eScreenBorderType	= Asura_Render::SCREEN_BORDER_STRETCH_TO_FIT;
// 	}
// 
// 	// Do we need to change the screen-border?
// 	if(s_eScreenBorderType != Asura_Render::SCREEN_BORDER_DISABLED)
// 	{
// 		Asura_Render::ActivateScreenBorder(s_eScreenBorderType, s_uDesiredScreenWidth, s_uDesiredScreenHeight);
// 	}
#endif	// defined(ASURA_TOOLS)
}

void AVP_Core::RenderEnvironmentAndEntities(void)
{
	//////////////////////////////////////////////////////////////////////////
	// Render environment...
	//////////////////////////////////////////////////////////////////////////
#if defined(ASURA_USE_STREAMING)
	Asura_Streaming_System::RenderEnvironment();
#else
	Asura_Module_System::Render();
#endif	// defined(ASURA_USE_STREAMING)

#ifdef ASURA_USE_ZPREPASS
	if( Asura_Render::IsZPrepass() )
	{
#endif

#if defined(ASURA_USE_STREAMING)
		Asura_Streaming_System::RenderEntities();
#else
		Asura_Entity_System::Render();
#endif	// defined(ASURA_USE_STREAMING)

#ifdef ASURA_USE_ZPREPASS
	}
#endif
}


void AVP_Core::Render(AVP_Player* pxPlayer)
{
	{
#ifdef ASURA_USE_VIDEO_TO_TEXTURE_SYSTEM
		Asura_VideoToTexture_System::Render();
#endif

		Asura_Render::BeginMRT();

#if !defined(AVP_RENDER_SKYBOX_AFTER_SCENE)
		//////////////////////////////////////////////////////////////////////////
		// Skybox rendering...
		//////////////////////////////////////////////////////////////////////////
#if defined(ASURA_USE_STREAMING)
		//	Asura_Streaming_System::RenderSkybox();
#else
		Asura_Skybox::Render();
#endif	// defined(ASURA_USE_STREAMING)
		Asura_SpecialFX_System::RenderSkyboxEffects();
#endif	// defined(AVP_RENDER_SKYBOX_AFTER_SCENE)

#ifdef ASURA_USE_ZPREPASS
		Asura_Render::SetZPrepass( true );
#endif
		RenderEnvironmentAndEntities();
#ifdef ASURA_USE_ZPREPASS
		Asura_Render::SetZPrepass( false );
#endif
#ifdef ASURA_USE_MATERIAL_RESPONSE_SYSTEM
		if (Asura_MaterialResponse_System::GetMaterialProjectContextCallback())
		{
			Asura_MaterialResponse_System::GetMaterialProjectContextCallback()(0,0);
		}
#endif

		//////////////////////////////////////////////////////////////////////////
		// Render botanicals/carpet...
		//////////////////////////////////////////////////////////////////////////
#ifdef ASURA_USE_BOTANICALS
		Asura_Botanicals_InstanceManager::Render();
#endif
#ifdef ASURA_USE_CARPET
		Asura_Carpet_MeshManager::Render();
#endif
#ifdef ASURA_USE_MATERIAL_RESPONSE_SYSTEM
		if (Asura_MaterialResponse_System::GetMaterialProjectContextCallback())
		{
			Asura_MaterialResponse_System::GetMaterialProjectContextCallback()(ASURA_MATERIAL_RESPONSE_PROJECT_CONTEXT_INVALID,ASURA_MATERIAL_RESPONSE_PROJECT_CONTEXT_INVALID);
		}
#endif

#ifdef ASURA_USE_ZPREPASS
		RenderEnvironmentAndEntities();
#ifdef ASURA_USE_MATERIAL_RESPONSE_SYSTEM
		if (Asura_MaterialResponse_System::GetMaterialProjectContextCallback())
		{
			Asura_MaterialResponse_System::GetMaterialProjectContextCallback()(ASURA_MATERIAL_RESPONSE_PROJECT_CONTEXT_INVALID,ASURA_MATERIAL_RESPONSE_PROJECT_CONTEXT_INVALID);
		}
#endif
#endif

		//////////////////////////////////////////////////////////////////////////
		// Render decals...
		//////////////////////////////////////////////////////////////////////////
#ifdef ASURA_USE_STATIC_DECAL_SYSTEM
		Asura_Decal_System::RenderStaticDecals();
#endif
		Asura_SpecialFX_System::RenderDecals();


#if defined(AVP_RENDER_SKYBOX_AFTER_SCENE)
		//////////////////////////////////////////////////////////////////////////
		// Skybox rendering...
		//////////////////////////////////////////////////////////////////////////
#if defined(ASURA_USE_STREAMING)
		//	Asura_Streaming_System::RenderSkybox();
#else
		Asura_Skybox::Render();
#endif	// defined(ASURA_USE_STREAMING)
		Asura_SpecialFX_System::RenderSkyboxEffects();
#endif	// defined(AVP_RENDER_SKYBOX_AFTER_SCENE)

#ifdef ASURA_USE_MATERIAL_RESPONSE_SYSTEM
		if (Asura_MaterialResponse_System::GetMaterialProjectContextCallback())
		{
			Asura_MaterialResponse_System::GetMaterialProjectContextCallback()(ASURA_MATERIAL_RESPONSE_PROJECT_CONTEXT_INVALID,ASURA_MATERIAL_RESPONSE_PROJECT_CONTEXT_INVALID);
		}
#endif


		Asura_Render::EndMRT();

		// Prepares data used by Asura_AmbientOcclusion::Render() - needs to be where predicated tiling is not enabled on 360
		Asura_AmbientOcclusion::PrepareDepthData();

		// DMR 05/06/07 - This call now renders the photons, projectors, and shadows. There should be no other calls to render any of these
		// when deferred shading is enabled
		Asura_DeferredShading::Render();

		Asura_AmbientOcclusion::Render();

		Asura_SortedRender::RenderTransparentItems();

		// This renders the new water effect
		Asura_Module_System::RenderLate();
#ifdef ASURA_USE_MATERIAL_RESPONSE_SYSTEM
		if (Asura_MaterialResponse_System::GetMaterialProjectContextCallback())
		{
			Asura_MaterialResponse_System::GetMaterialProjectContextCallback()(ASURA_MATERIAL_RESPONSE_PROJECT_CONTEXT_INVALID,ASURA_MATERIAL_RESPONSE_PROJECT_CONTEXT_INVALID);
		}
#endif


#if defined(ASURA_USE_STREAMING)
		Asura_Streaming_System::RenderSpecialFX();
#else
		Asura_SpecialFX_System::Render();
#endif	// defined(ASURA_USE_STREAMING)

		Asura_SpecialFX_System::RenderHUDEffects();
		AVP_CloakEffect::RenderTransparentPasses();
	}

	if(pxPlayer != NULL)
	{
		if(!Asura_Render::GetCutsceneHasCameraControl() && !AVP_Game_Core::IsPaused())
		{
			pxPlayer->Render();
		}

		// Render multiplayer game system related stuff if necessary
		if (AVP_Settings::IsMultiPlayer())
		{
			AVP_Multiplayer_GameCore::Render(); 
		}

		AVP_Predator_LockOn::GetSingleton().Render();
	}

	Platform_BeginHUDRender();

	Asura_SpecialFX_System::Render2DEffects();

#ifdef ASURA_TOOLS
	if(Asura_Overlay_System::AreAnyOverlaysVisible())
	{
		Asura_Overlay_System::Render();
		Asura_Cursor::Render();
	}
#endif

	Asura_Teleport_System::Render();

	// Debug rendering after all HDR and full-screen effects.
#if defined(ASURA_USE_DEBUG_MESSAGE_TRIGGER)
	if(!AVP_Game_Core::IsPaused())
	{
		const Asura_Handle iFontHandle = AVP_Core::GetFontHandle( AVP_Core::DEBUG_MESSAGE_FONT );
		if (Asura_Network::IsServer())
		{
			// JG - 09 January 2009 - Debug message triggers don't work in multiplayer at the moment, so I'm not using this system in client machines.
			// If we want to use them in multiplayer levels we will have to fix them, but that is quite unlikely to happen
			Asura_DebugMessageTrigger_Messages::Render(iFontHandle); 
		}
		Asura_DebugWaypointTrigger_Messages::Render(iFontHandle);
	}
#endif

	Asura_Physics_System::DebuggingRender();
#if !defined(ASURA_NO_DEBUG_OUTPUT)
	Axon_Project_System::RenderDebugInfo();
	AVP_Waypoint_Manager::RenderDebugInfo();
#endif
	Asura_DeferredShading::RenderDebugOutput();

	AVP_GUIMenu_System::Render(false);
}

void AVP_Core::EndRender(void)
{
#if defined(ASURA_TOOLS)
	Asura_Render::DeactivateScreenBorder();
#else
	// Do we need to change the screen-border?
	if(s_eScreenBorderType != Asura_Render::SCREEN_BORDER_DISABLED)
	{
		Asura_Render::DeactivateScreenBorder();
	}
#endif	// defined(ASURA_TOOLS)

#if defined(ASURA_SUPPORT_COMMAND_CONSOLE)
	{
		const Asura_Handle	iFontHandle	= AVP_Core::GetFontHandle(AVP_Core::COMMAND_CONSOLE_FONT);

		Asura_Font_System::SetFont(iFontHandle);
		Asura_CommandConsole::Render();
	}
#endif	// defined(ASURA_SUPPORT_COMMAND_CONSOLE)
	Asura_Render::EndFrame();
}

void AVP_Core::RenderForScreenshot()
{
	s_xStateMachine.Render();
}

//////////////////////////////////////////////////////////////////////////
// Load and Save
//////////////////////////////////////////////////////////////////////////
#ifdef ASURA_TOOLS
bool AVP_Core::ToolsQuickSave()
{
	Asura_Assert(s_pxToolsQuickSaveBlock, "no tools quicksave block!");
	s_pxToolsQuickSaveBlock->Clear();
	return Asura_LoadAndSave::SaveToBuffer( *s_pxToolsQuickSaveBlock );
}

bool AVP_Core::ToolsQuickLoad()
{
    Asura_Assert(s_pxToolsQuickSaveBlock, "no tools quicksave block!");
    // check for empty save block (24 bytes is minimum: 8 bytes for fileID and 16 bytes for terminator)
	if( s_pxToolsQuickSaveBlock->GetSpaceUsed() <= 24 )
    {
        return false;
    }
	bool bSuccess = Asura_LoadAndSave::LoadFromBuffer( *s_pxToolsQuickSaveBlock );
	Asura_Assert(bSuccess, "Failed to load from buffer");
	return bSuccess;
}
#endif // ASURA_TOOLS


//////////////////////////////////////////////////////////////////////////////
// Network startup/shutdown
//////////////////////////////////////////////////////////////////////////////
void AVP_Core::InitialiseNetwork(const bool bExternalNetwork)
{
#ifdef ASURA_SUPPORT_EXTERNAL_NETWORK
	bool bShouldNetworkAlreadyBeInitialised = false;
	bool bNetworkIsInitialised = Asura_Network::IsNetworkInitialised();

	if (bShouldNetworkAlreadyBeInitialised)
	{
		Asura_Assert(bNetworkIsInitialised, "Network not initialised yet - Lobby should have done it!");
	}
	else
	{
		Asura_Assert(!bNetworkIsInitialised, "Network already initialised");
		if (bNetworkIsInitialised)
		{
			Asura_Network::Shutdown();
		}
		bNetworkIsInitialised = false;
	}

	// get settings and initialise network here
	bool bOnline				= AVP_Settings::GetOnlineGame();
	bool bServer				= AVP_Settings::GetIsServer();
	u_int uNumLocalPlayers		= AVP_Settings::GetNumberOfLocalPlayers();
	u_int uMaxPlayers			= bExternalNetwork ? AVP_Settings::GetMaxNetPlayers() : uNumLocalPlayers;	
	u_int uMaxMachines			= bExternalNetwork ? AVP_Settings::GetMaxNetPlayers() : 1;		// TODO: assumes 1 player per machine...
	const char* szAddress		= bServer ? NULL : AVP_Settings::GetAddressToConnectTo();

	if (bNetworkIsInitialised)
	{
		// set "ready for messages" off so we don't receive loads of messages while loading.
		// This also makes the state match the case where we init the network here.
		Asura_Network::SetReadyForMessages(false);
	}
	else
	{
		Asura_Network::Initialise(bExternalNetwork, bServer, bServer ? &AVP_Server::MessageHandler : NULL, 
			&AVP_Client::MessageHandler, uNumLocalPlayers, uMaxPlayers, uMaxMachines, szAddress);

		//TODO: options for these?
		if(bServer && bExternalNetwork)
		{
			Asura_Network::SetServerLANReporting(!bOnline);
		}	
		if(bExternalNetwork)
		{
			Asura_Network::SetPlayingOnInternet(bOnline);
		}
	}
#else //#ifdef ASURA_SUPPORT_EXTERNAL_NETWORK
	Asura_Network::Initialise(false, true, &AVP_Server::MessageHandler, &AVP_Client::MessageHandler, 1, 1, 1);
#endif //#else //#ifdef ASURA_SUPPORT_EXTERNAL_NETWORK

	if (AVP_Settings::GetIsServer())
	{
		// Wait until the server has loaded the level before trying to initialise any clients.
		AVP_Server::SetCanInitialiseClients(false);
		Asura_Network::SetClientCreateMessagesSentFunction(AVP_Server::ClientCreateMessagesSent);
	}
}

void AVP_Core::ShutdownNetwork()
{
	bool bShouldShutdownNetwork = Asura_Network::IsNetworkInitialised();
	if (bShouldShutdownNetwork)
	{
		Asura_Network::Shutdown();
	}

	AVP_Server::ClearClientInitialisationLists();
}


void AVP_Core::InitialiseEffects()
{
	AVP_Laser::Initialise();

	AVP_CloakEffect::Initialise();
	AVP_GlowOutlines::Initialise();
}

void AVP_Core::ShutdownEffects()
{
	AVP_Laser::Shutdown();
	AVP_CloakEffect::Shutdown();
	AVP_GlowOutlines::Shutdown();
}


//////////////////////////////////////////////////////////////////////////
// State handling
//////////////////////////////////////////////////////////////////////////

bool AVP_Core::MainLoop(void)
{
	Platform_Update();

#if defined(ASURA_TOOLS)
	if(!Tools_System::IsActive())
#endif	// defined(ASURA_TOOLS)
	{
		Asura_UserInput::Update();
	}
	Asura_ControlManager::Update();

#ifdef ASURA_SUPPORT_EXTERNAL_NETWORK
	Asura_NetLinkManager::Update();
#endif

#ifdef ASURA_SUPPORT_LOBBY
	Asura_Lobby_System::Update();
#endif

#if defined(ASURA_USE_GUI_MENU_SYSTEM)
#ifdef ASURA_TOOLS
	if(!Tools_System::IsActive())
#endif
	{
		AVP_GUIMenu_System::Update();
	}
#endif

#ifdef ASURA_USE_AUDIO_SYSTEM
	if(s_bShouldUpdateSoundSystem)
	{
		Asura_Audio_Systems_Manager::Update(ASURA_AUDIO_UPDATE_EVERYTHING);
	}
#else
	if (s_bShouldUpdateSoundSystem)
	{
		Asura_Sound_System::Update();
	}
#endif


#if defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)
#if defined(ASURA_TOOLS)
	if(!IsInTools())
#endif	//defined(ASURA_TOOLS)
	{
		Asura_FacialAnimation_System::Update();
	}
#endif	//defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)

	if (AVP_GameStatus::GetQuitApplication())
	{
		s_xStateMachine.RequestQuitApplication();
	}

	s_xStateMachine.Update();
	s_xStateMachine.Render();

#ifdef ASURA_SUPPORT_EXTERNAL_NETWORK
	Asura_NetLinkManager::Update_PostRender();
#endif

#ifdef ASURA_USE_STREAMING
	// Streaming EOF update, using fixed frame time (which is 30 at present)
	const float	fFramesPerSecond	= 60.0f;
	const float	fFrameDuration		= 1.0f / fFramesPerSecond;
	const float	fMaxTimeAllowed		= Asura_Timers::GetFixedFrameTime() + fFrameDuration;
	const float	fTimeInFrame		= Asura_Timers::GetTimeInFrame();
	float		fTimeAllowed		= (fTimeInFrame > fMaxTimeAllowed)
									? (fFrameDuration - Asura_Maths::FloatMod(fTimeInFrame, fFrameDuration))
									: (fMaxTimeAllowed - fTimeInFrame);

	// Update what we can at the end of the frame...
	Asura_Streaming_System::EndOfFrameUpdate(fTimeAllowed, fFrameDuration);
#endif //#ifdef ASURA_USE_STREAMING
	
#ifdef ASURA_TOOLS
	// Tools updates the timer for us in Tools Mode
	if(!IsInTools())
#endif
	{
		Asura_Timers::HandlePrimaryTimer();
	}

#ifdef AVP_LOG_FILE_LOAD_MEMORY_USE
	AVP_SmokeTest::Update();
#endif // #ifdef AVP_LOG_FILE_LOAD_MEMORY_USE

	return(s_xStateMachine.GetCurrentState() != AVP_StateMachine_Core::STATE_SHUTTINGDOWN);
}

void AVP_Core::RequestLevelComplete()
{
	AVP_GameStatus::SetLevelComplete(true);
}

void AVP_Core::RequestQuit()
{
	AVP_GameStatus::SetQuitApplication(true);
}

void AVP_Core::RequestRestartAtCheckPoint()
{
	AVP_GameStatus::SetRestoreCheckpoint(true);
}

void AVP_Core::RequestRestartLevel()
{
	AVP_GameStatus::SetRestartLevel(true);
}

void AVP_Core::RequestGameExit()
{
	AVP_GameStatus::SetExitGame(true);
}

void AVP_Core::RequestDisconnecting()
{
	AVP_GameStatus::SetExitGame(true);
}

void AVP_Core::RequestGameOver()
{
	AVP_GameStatus::SetGameOver(true);
}


#ifdef AVP_LOG_FILE_LOAD_MEMORY_USE

void AVP_Core::ClearFileLoadMemoryUseLogfile()
{
	AVP_SmokeTest::ClearLogFiles();
}

int AVP_Core::GetAvailableMemory()
{
	return Platform_GetAvailableMemory();
}

#endif //#ifdef AVP_LOG_FILE_LOAD_MEMORY_USE


//////////////////////////////////////////////////////////////////////////
// Collision object function...
//////////////////////////////////////////////////////////////////////////
const Asura_Collision_Object* AVP_Core::GetPhysicalObjectCollisionObject(int iIndex, Asura_Hash_ID uShapeHash, Asura_Guid uGuid)
{
	const AVP_ServerEntity_Actor*	pxActor	= AVP_ServerEntity_Actor::FindEntity(uGuid);

	if(pxActor == NULL)
	{
		return(NULL);
	}
	else
	{
		return(pxActor->GetActorCollisionObject());
	}
}

void AVP_Core::RenderDialogueSubtitles(const Asura_UnicodeChar* wszText)
{
	if(AVP_Settings::GetCurrentProfile()->GetSounds().GetShowSubtitles())
	{
#if defined(ASURA_USE_DIALOGUE_SYSTEM)
		Asura_PrintfToBfr(s_aswzDialogue, wszText, sizeof(s_aswzDialogue), true);
		s_bShowDialogue = true;
#endif	//defined(ASURA_USE_DIALOGUE_SYSTEM)
	}
}

#if defined(ASURA_SUPPORT_COMMAND_CONSOLE)
void AVP_Core::CreateConsoleVars()
{
	AVP_Actor_HUDHandler::CreateConsoleVars();
	AVP_Vertebrate_HUDHandler::CreateConsoleVars();
	AVP_Alien_HUDHandler::CreateConsoleVars();
	AVP_Alien_InputHandler::CreateConsoleVars();
	AVP_Alien_JumpHandler::CreateConsoleVars();
	AVP_Predator_HUDHandler::CreateConsoleVars();
	AVP_Human_HUDHandler::CreateConsoleVars();
	AVP_Actor_InteractionHandler::CreateConsoleVars();
	AVP_Predator_InteractionHandler::CreateConsoleVars();
	AVP_Human_InteractionHandler::CreateConsoleVars();
	AVP_Predator_MeleeCounter_System::CreateConsoleVars();
	AVP_Actor_DamageDirectionIndicator_System::CreateConsoleVars();
	AVP_Waypoint_Manager::CreateConsoleVars();
	AVP_ClientEntity_Actor::CreateConsoleVars();
	AVP_Vertebrate_AnimationHandler::CreateConsoleVars();

	Asura_CommandConsole::AddVar( L"AVP.HUD.ShowDialogue", &s_bShowDialogue, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
}

void AVP_Core::CreateConsoleCommands()
{
	AVP_Actor_HUDHandler::CreateConsoleCommands();
	Asura_CommandConsole::AddCmd( L"AVP.CMD.DialogueList", AVP_Core::ConsoleCmd_DialogueList, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );

}
#endif

bool AVP_Core::ConsoleCmd_DialogueList( u_int uElementIndex, bool bInterrogate, bool bHelp )
{
	if( bInterrogate || bHelp )
	{
		return true;
	}

	Asura_CommandConsole::Printf( L"%s", s_aswzDialogue );
	return true;
}

/// End of file. ///
