//////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_GameStatus.h
// Author : Anthony Gregson
// Date   : 04/12/07
//
// Copyright (c) 2007 Rebellion. All rights reserved.
//
// The companion to AVP_Multiplayer_GameRules, this is the storage for the variable data
// in a single AVP game. This, the rules, and the PlayerInfo data are used by
// the Playing states to determine whether the game should be finishing or
// not, and potentially for making various other decisions about what needs
// to update (respawn waves being one possible example). Only the server
// should be making any direct decisions on these details, the clients
// updating by message (although that isn't quite true in the initial version,
// as we're storing quit requests here).
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef ___AVP_GAMESTATUS_H___
#define ___AVP_GAMESTATUS_H___ 1

// I N C L U D E S ///////////////////////////////////////////////////////////

// C O N S T A N T S /////////////////////////////////////////////////////////

// Forward declarations
class Asura_Message;

// C L A S S E S /////////////////////////////////////////////////////////////

class AVP_GameStatus
{
public:
	// TODO - we may want to be more selective about when to send all the data over, having a set of single- (or few-) variable messages top use on changes (and actually checking if the value did change).
	static void SetGameOver(bool bSet)			{ s_bGameOver = bSet; SendGameStatusUpdateMessage(); }
	static void SetLevelComplete(bool bSet)		{ s_bLevelComplete = bSet; SendGameStatusUpdateMessage(); }
	static void SetQuitApplication(bool bSet)	{ s_bQuitApplication = bSet; SendGameStatusUpdateMessage(); }
	static void SetExitGame(bool bSet)			{ s_bExitGame = bSet; SendGameStatusUpdateMessage(); }
	static void SetLoadCheckpoint(bool bSet)	{ s_bLoadCheckpoint = bSet; SendGameStatusUpdateMessage(); }
	static void SetRestoreCheckpoint(bool bSet)	{ s_bRestoreCheckpoint = bSet; SendGameStatusUpdateMessage(); }
	static void SetRestartLevel(bool bSet)		{ s_bRestartLevel = bSet; SendGameStatusUpdateMessage(); }
	static void SetLoadMap(bool bSet)			{ s_bLoadMap = bSet; SendGameStatusUpdateMessage(); }

	static void ResetStatusChanges();

	static bool GetGameOver()			{ return s_bGameOver; }
	static bool GetLevelComplete()		{ return s_bLevelComplete; }
	static bool GetQuitApplication()	{ return s_bQuitApplication; }
	static bool GetExitGame()			{ return s_bExitGame; }
	static bool GetLoadCheckpoint()		{ return s_bLoadCheckpoint; }
	static bool GetRestoreCheckpoint()	{ return s_bRestoreCheckpoint; }
	static bool GetRestartLevel()		{ return s_bRestartLevel; }
	static bool GetLoadMap()			{ return s_bLoadMap; }

	static void SendGameStatusUpdateMessage();
	static bool HandleMessage(const Asura_Message* pxMessage);

private:
	// These values really aren't the correct ones. However, they allow us to work with roughly the gameflow we originally had before the state machine overhaul.
	// Some of these actually relate to the connection state machine rather than the playing machine.
	static bool s_bGameOver;
	static bool s_bLevelComplete;
	static bool s_bQuitApplication;
	static bool s_bExitGame;
	static bool s_bLoadCheckpoint;
	static bool s_bRestoreCheckpoint;
	static bool s_bRestartLevel;
	static bool s_bLoadMap; 

};

class AVP_Msg_GameStatus_FullUpdate
{
public:
	AVP_Msg_GameStatus_FullUpdate()
		: m_uFlags(0)
	{
	}

	void SetGameOver(bool bSet)				{ m_bGameOver = bSet; }
	void SetLevelComplete(bool bSet)		{ m_bLevelComplete = bSet; }
	void SetRestoreCheckpoint(bool bSet)	{ m_bRestoreCheckpoint = bSet; }
	void SetRestartLevel(bool bSet)			{ m_bRestartLevel = bSet; }
	void SetLoadMap(bool bSet)				{ m_bLoadMap = bSet; }

	bool GetGameOver() const			{ return m_bGameOver; }
	bool GetLevelComplete() const		{ return m_bLevelComplete; }
	bool GetRestoreCheckpoint() const	{ return m_bRestoreCheckpoint; }
	bool GetRestartLevel() const		{ return m_bRestartLevel; }
	bool GetLoadMap() const				{ return m_bLoadMap; }

private:
	union
	{
		u_int m_uFlags;
		struct
		{
			u_int m_bGameOver			: 1;
			u_int m_bLevelComplete		: 1;
			u_int m_bRestoreCheckpoint	: 1;
			u_int m_bRestartLevel		: 1;
			u_int m_bLoadMap			: 1;
		};
	};
};

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

#endif // #ifndef ___AVP_GAMESTATUS_H___
