//////////////////////////////////////////////////////////////////////////////
//
// AVP_Game_Core.cpp
//
// Copyright (c) 2005 Rebellion. All rights reserved.
//
// Description:
//
//     basic game loop.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header.
#include "Core/AVP_Game_Core.h"

// Asura headers
#include "Animation/Asura_Animation.h"
#include "Animation/Asura_Hierarchy.h"
#if defined(ASURA_TOOLS)
#include "Blueprint/Tools_Mode_Blueprint.h"
#endif
#include "Botanicals/Asura_Botanical_Game_Render_Manager.h"
#include "Botanicals/Asura_Botanicals_InstanceManager.h"
#include "Camera/Asura_Camera.h"
#include "Carpet/Asura_Carpet_MeshManager.h"
#include "ChunkSystem/Asura_Chunk_Loaders.h"
#include "Core/Asura_Maths.h"
#include "Core/Asura_Multithread.h"
#include "Core/Asura_TextureManagement.h"
#include "Core/Asura_MaterialResponse_System.h"
#include "Core/Asura_Timers.h"
#include "Core/Asura_Time_System.h"
#include "Core/Asura_UserInput.h"
#include "Cutscene/Asura_Cutscene_System.h"
#include "Debugging/Asura_CPUProfile.h"
#include "Debugging/Asura_DebugText.h"
#include "Debugging/Asura_Logfile.h"
#ifdef ASURA_TOOLS
#include "Debugging/Asura_ScreenShot_System.h"
#endif
#include "Entity/Asura_Entity.h"
#include "Entity/PhysicalObject/Asura_ServerEntity_PhysicalObject.h"
#include "Entity/PhysicalObject/Moves/Asura_Movement_System.h"
#include "Entity/Ragdoll/Asura_ClientEntity_Ragdoll.h"
#if defined(ASURA_USE_DEBUG_MESSAGE_TRIGGER)
#include "Entity/Triggers/Asura_ServerEntity_DebugMessageTrigger.h"
#include "Entity/Triggers/Asura_ServerEntity_DebugWaypointTrigger.h"
#endif	// defined(ASURA_USE_DEBUG_MESSAGE_TRIGGER)

#ifdef ASURA_USE_VIDEO_TO_TEXTURE_SYSTEM
#include "FMV/Asura_VideoToTexture_System.h"
#endif

#include "FSFX/Asura_FSFX_System.h"
#include "Interface/Asura_CommandConsole.h"
#ifdef ASURA_TOOLS
#include "Interface/Asura_Cursor.h"
#include "Interface/Asura_Overlay_System.h"
#endif
#include "LoadAndSave/Asura_LoadAndSave.h"
#include "Mesh/Asura_Env_System.h"
#include "Network/Asura_Network.h"
#include "Physics/Asura_Physics.h"
#include "Projectors/Asura_Projector_System.h" 
#include "Rendering/Asura_AmbientOcclusion.h"
#include "Rendering/Asura_DeferredShading.h"
#include "Rendering/Asura_Module.h"
#include "Rendering/Asura_Render.h"
#include "Rendering/Asura_SortedRender.h"
#if defined(ASURA_USE_INDEPENDENT_SHADOW_SYSTEM)
#include "Shadows/Asura_Shadow_System.h"
#endif	// defined(ASURA_USE_INDEPENDENT_SHADOW_SYSTEM)
#include "Sound/Asura_Audio_System.h"
#include "Sound/Asura_Sound.h"
#include "Sound/Asura_Sound_Mixer.h"
#include "SpecialFX/Asura_Decal_System.h"
#include "SpecialFX/Asura_Skybox.h"
#include "SpecialFX/Asura_SpecialFX.h"
#include "SpecialFX/PFX/Asura_PFX_System.h"
#ifdef ASURA_USE_STREAMING
#include "Streaming/Asura_Streaming_System.h"
#endif //#ifdef ASURA_USE_STREAMING
#include "Teleport/Asura_Teleport_System.h"

// Axon headers
#include "Core/AVP_Waypoint_Manager.h"
#include "Core/Axon_Project_System.h"

// Game headers
#include "Actor/AVP_Actor_InputHandler.h"
#include "Actor/AVP_Actor_HUDHandler.h"
#include "Actor/Vertebrates/Alien/AVP_Alien_AIAnimStateData.h"
#include "Actor/Vertebrates/Alien/AVP_Alien_Description.h"
#include "Actor/Vertebrates/Alien/AVP_Alien_PlayerAnimStateData.h"
#include "Actor/Vertebrates/AlienQueen/AVP_AlienQueen_AIAnimStateData.h"
#include "Actor/Vertebrates/AVP_Vertebrate_HUDHandler.h"
#include "Actor/Vertebrates/FaceHugger/AVP_FaceHugger_AIAnimStateData.h"
#include "Actor/Vertebrates/Human/AVP_Human_AIAnimStateData.h"
#include "Actor/Vertebrates/Human/AVP_Human_PlayerAnimStateData.h"
#include "Actor/Vertebrates/Predalien/AVP_Predalien_AIAnimStateData.h"
#include "Actor/Vertebrates/Predator/AVP_Predator_AIAnimStateData.h"
#include "Actor/Vertebrates/Predator/AVP_Predator_PlayerAnimStateData.h"
#include "Actor/Vertebrates/Predator/AVP_Predator_LockOn.h"
#include "Actor/Weapons/Sentrygun/AVP_Sentrygun_AIAnimStateData.h"
#include "Animation/AVP_Hierarchy_System.h"
#include "Blueprint/AVP_Blueprint_HashIDs.h"
#include "Camera/AVP_CameraShake_System.h"
#include "Core/AVP_Core.h"
#include "Entity/AVP_Messages.h"
#include "Entity/Damage/AVP_EntityDamage_Messages.h"
#include "Entity/Damage/AVP_EntityDamage_Request.h"
#include "Entity/PhysicalObject/Projectile/AVP_GenericProjectileData.h"
#include "Entity/PlayerInfo/AVP_ServerEntity_PlayerInfo.h"
#include "Input/AVP_Rumble_System.h"
#include "Inventory/Items/Weapons/AVP_RangedWeapon_ClientEffectsHandler.h"
#include "Multiplayer/AVP_Multiplayer_GameCore.h"
#include "Network/AVP_Player.h"
#include "Network/AVP_Server.h"
#include "SpecialFX/pfx/AVP_DamagePFX.h"
#include "Sound/AVP_Dialogue_System.h"

#if defined(ASURA_USE_GUI_MENU_SYSTEM)
#include "GUIMenu/AVP_GUIMenu_System.h"
#endif 
#include "SpecialFX/AVP_CollisionFX_System.h"
#include "SpecialFX/AVP_CloakEffect.h"

// Hack. These will be removed later
#ifdef ASURA_PC_TARGET_SM3
#include "Rendering/Asura_Direct3D.h"
#endif

#include "Rendering/Asura_Font_System.h"
#include "SpecialFX/AVP_GlowOutlines.h"

// Tools
#ifdef ASURA_TOOLS
#include "Misc/Tools_Render_Options.h"
#include "FSFX/Tools_FSFX_System.h"
#endif

// S T A T I C   V A R I A B L E S ///////////////////////////////////////

static const u_int	g_uSaveBlockSize	= 3 * 1024 * 1024;

AVP_Player*				AVP_Game_Core::s_pxPlayer		= NULL;

Asura_StaticSaveBlock	AVP_Game_Core::s_xLevelStartBlock(g_uSaveBlockSize);
Asura_StaticSaveBlock	AVP_Game_Core::s_xCheckpointBlock(g_uSaveBlockSize);

#if defined(AVP_ALLOW_GAME_TIME_SCALE_CHANGE)
Asura_ControlMap	AVP_Game_Core::s_xGameTimeScaleControlMap(MAX_GAME_TIME_SCALE_MAPPINGS);
bool				AVP_Game_Core::s_bAdvanceOneFrame = false;

float				AVP_Game_Core::s_fGameTimeScaleDisplay	= 0.0f;
float				AVP_Game_Core::s_fCurrentGameTimeScale	= 1.0f;
#endif	// defined(AVP_ALLOW_GAME_TIME_SCALE_CHANGE)

u_int			AVP_Game_Core::s_uRespawnSpecies		= 0;
DVSVAR	bool dvs_bRenderFrameNumber = false;

bool AVP_Game_Core::s_bReceivedInitialiseCompleteMessage = false;

Asura_FSFX_Handler* AVP_Game_Core::s_pxFSFXHandler = NULL;
Asura_CameraData::HDRSettings AVP_Game_Core::s_xVisionModeHDRSettings;
// C L A S S   I M P L E M E N T A T I O N ///////////////////////////////////

void AVP_Game_Core::Initialise()
{
	if (AVP_Settings::GetIsServer())
	{
		AVP_Server::SetCanInitialiseClients(true);
	}

 	AVP_GenericProjectileData::ResetStorage();
	AVP_RangedWeapon_ClientEffectsHandler::ResetStorage();
	AVP_Collision_Descriptor::ResetStorage();

	//////////////////////////////////////////////////////////////////////////
	// TODO get rid of this lot
	//////////////////////////////////////////////////////////////////////////
	AVP_Alien_AIAnimStateData::ResetStorage();
	AVP_Predalien_AIAnimStateData::ResetStorage();
	AVP_FaceHugger_AIAnimStateData::ResetStorage();
	AVP_Human_AIAnimStateData::ResetStorage();
	AVP_Predator_AIAnimStateData::ResetStorage();
	AVP_Sentrygun_AIAnimStateData::ResetStorage();
	//////////////////////////////////////////////////////////////////////////

	AVP_Dialogue_System::Initialise();

	// Create the player...
	// (AAM - 13.11.2007) Moving this to the top of the function, because some actors in
	// PostLoadInitialise() send client updates. In their ProcessClientUpdate(), these clients
	// check to see if they're the local player, so we get an assert if it doesn't exist yet
	Asura_Guid uPlayerGuid = ASURA_GUID_UNREGISTERED;
	Asura_Guid uPlayerPuppetGuid = ASURA_GUID_UNREGISTERED;
	if ( AVP_Settings::GetNumberOfLocalPlayers() > 0 )
	{
		s_pxPlayer = AVP_Player::Create();
		uPlayerGuid = s_pxPlayer->GetGuid();
		uPlayerPuppetGuid = s_pxPlayer->GetPuppetGuid();
	}

	// SG initialise link block extra data 
	AVP_Waypoint_Manager::InitialiseLinkExtraData();

	// Initialise Asura's game-side data...
    Asura_Entity_System::InitialiseServerEntitiesFromBlueprints();
	Asura_Entity_System::PostLoadInitialise();
	Axon_System::PostLoadInitialise();

	// Time for the network initialisation...
	Asura_Assert(!Asura_Network::IsReadyForMessages(), "network state machine error");
	Asura_Network::StartGame();

	Asura_Teleport_System::RegisterPlayerGuid(0, uPlayerGuid);
	Asura_Teleport_System::SetProjectTeleportServerEntityFunction(Project_TeleportServerEntity);

	Asura_Movement_System::SetUseDeferredUpdate(true);

	// AVP Rumble system
	AVP_Rumble_System::Initialise();

#ifdef ASURA_USE_STREAMING
	// In streaming, set a fixed frame time so we have the chance to stream some stuff in.
	Asura_Timers::SetFixedFrameTime(1.0f / 30.0f);
#endif //#ifdef ASURA_USE_STREAMING

#if defined(AVP_ALLOW_DEBUG_MOVEMENT)
	AVP_Actor_InputHandler::SetMovementType(AVP_Actor_InputHandler::MT_PLAYER_FIRST_PERSON);
#endif	// defined(AVP_ALLOW_DEBUG_MOVEMENT)

#ifdef ASURA_SUPPORT_COMMAND_CONSOLE
	Asura_CommandConsole::AddVar(L"AVP.Player.Species", &s_uRespawnSpecies, 0, AVP_Settings::AVP_NUM_SPECIES-1, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::AddCmd(L"Respawn", Respawn, Asura_CommandConsole::REMOVE_AT_SHUTDOWN ); 
#ifdef ASURA_DEBUG_VARIABLE_SYSTEM
	Asura_CommandConsole::AddVar(L"AVP.ShowFrameNumber", &dvs_bRenderFrameNumber, Asura_CommandConsole::REMOVE_AT_SHUTDOWN  );
#endif //ASURA_DEBUG_VARIABLE_SYSTEM
#endif //#ifdef ASURA_SUPPORT_COMMAND_CONSOLE

#if defined(AVP_ALLOW_GAME_TIME_SCALE_CHANGE)
	// Used to advance one frame while paused
	s_bAdvanceOneFrame = false;
#endif	//defined(AVP_ALLOW_GAME_TIME_SCALE_CHANGE)

	// Initialise multiplayer game system if necessary
	if (AVP_Settings::IsMultiPlayer())
	{
		AVP_Multiplayer_GameCore::Initialise(); 
	}

	AVP_GUIMenu_System::ClearStoredHUDWidgets();

	// Create the game FSFXHandler
	s_pxFSFXHandler =  Asura_FSFX_System::AddHandler( uPlayerPuppetGuid );
	Asura_Assertf(s_pxFSFXHandler, ("Failed to create FSFX handler for player %u.", uPlayerPuppetGuid ));

	Platform_Initialise();
}

void AVP_Game_Core::Update()
{
#ifndef ASURA_NO_DEBUG_OUTPUT
	if ( dvs_bRenderFrameNumber )
	{
		Asura_DebugTextPrint( ( 1, 1, 0xffffffff, 0, "Frame: %d", Asura_Timers::GetFrameNumber() ) );
	}
#endif

	AVP_Core::Update(!IsPaused(), s_pxPlayer);

#if defined(AVP_ALLOW_GAME_TIME_SCALE_CHANGE)
	UpdateGameTimeScale();
#endif	// defined(AVP_ALLOW_GAME_TIME_SCALE_CHANGE)

	// Update multiplayer game logic if necessary
	if (AVP_Settings::IsMultiPlayer())
	{
		AVP_Multiplayer_GameCore::Update(); 
	}

	// Finally, see if a save was triggered during this update.
	AVP_ServerEntity_PlayerInfo* pxLocalPlayerInfo = AVP_ServerEntity_PlayerInfo::FindEntity(GetPlayerGuid());
	if (pxLocalPlayerInfo)
	{
		const Asura_Guid uCheckpointGuid = pxLocalPlayerInfo->GetCheckPointGuid();
		if (uCheckpointGuid != ASURA_GUID_UNREGISTERED)
		{
			// Make sure the actor isn't dead.
			const AVP_ServerEntity_Actor* pxPlayerActor = AVP_ServerEntity_Actor::FindEntity(pxLocalPlayerInfo->GetPuppetGuid());
			if (pxPlayerActor && !pxPlayerActor->IsDead())
			{
				AVP_Actor_HUDHandler::SetCheckpointReached();
				// Save requested. Trigger it, then reset the request.
				SaveCheckPointBlock();

				pxLocalPlayerInfo->SetCheckPointGuid(ASURA_GUID_UNREGISTERED);
			}
		}
	}

	AVP_Rumble_System::Update();
	if (Asura_Network::IsServer()) AVP_DamagePFX::Update();
}

void AVP_Game_Core::Render()
{
	AVP_Core::Render(s_pxPlayer);
#if defined(AVP_ALLOW_GAME_TIME_SCALE_CHANGE)
    RenderGameTimeScale();
#endif
}

void AVP_Game_Core::Shutdown()
{
#if defined(AVP_ALLOW_GAME_TIME_SCALE_CHANGE)
	// Empty the associated control-mappings...
	s_xGameTimeScaleControlMap.DeleteMappings();
	s_xGameTimeScaleControlMap.CreateMappings(MAX_GAME_TIME_SCALE_MAPPINGS);
#endif	//defined(AVP_ALLOW_GAME_TIME_SCALE_CHANGE)

	Platform_Shutdown();

	AVP_GenericProjectileData::ResetStorage();
	AVP_RangedWeapon_ClientEffectsHandler::ResetStorage();
	AVP_Collision_Descriptor::ResetStorage();
	AVP_Alien_AIAnimStateData::ResetStorage();
	AVP_Predalien_AIAnimStateData::ResetStorage();
	AVP_AlienQueen_AIAnimStateData::ResetStorage();
	AVP_FaceHugger_AIAnimStateData::ResetStorage();
	AVP_Human_AIAnimStateData::ResetStorage();
	AVP_Predator_AIAnimStateData::ResetStorage();
	AVP_Sentrygun_AIAnimStateData::ResetStorage();
	AVP_Alien_PlayerAnimStateData::GetSingleton().ResetStorage();
	AVP_Human_PlayerAnimStateData::GetSingleton().ResetStorage();
	AVP_Predator_PlayerAnimStateData::GetSingleton().ResetStorage();

	AVP_CameraShake_System::Reset();
	
	// SG shutdown link block extra data 
	AVP_Waypoint_Manager::ShutdownLinkExtraData();


	delete s_pxPlayer;
	s_pxPlayer = NULL;

	Asura_FSFX_System::RemoveHandler( s_pxFSFXHandler );
	s_pxFSFXHandler = NULL;

	// Shutdown game system if necessary
	if (AVP_Settings::IsMultiPlayer())
	{
		AVP_Multiplayer_GameCore::Reset(); 
	}

	AVP_Rumble_System::Shutdown();

#ifdef ASURA_SUPPORT_COMMAND_CONSOLE
	Asura_CommandConsole::RemoveVar(L"AVP.Player.Species" );
	Asura_CommandConsole::RemoveCmd(L"Respawn" ); 
#ifdef ASURA_DEBUG_VARIABLE_SYSTEM
	Asura_CommandConsole::RemoveVar(L"AVP.ShowFrameNumber" );
#endif
#endif //#ifdef ASURA_SUPPORT_COMMAND_CONSOLE

#ifdef ASURA_USE_STREAMING
	// In streaming, set a fixed frame time so we have the chance to stream some stuff in.
	Asura_Timers::SetFixedFrameTime(0.0f);
#endif

	AVP_Dialogue_System::Shutdown();
	if (Asura_Network::IsServer()) AVP_DamagePFX::Shutdown();

}

bool AVP_Game_Core::IsPaused()
{
	return (Asura_Timers::GetGameTimeScale() <= 0.0f && AVP_GUIMenu_System::IsInPauseMenu());
}

bool AVP_Game_Core::SaveLevelStartBlock()
{
	s_xLevelStartBlock.Clear();
	return Asura_LoadAndSave::SaveToBuffer( s_xLevelStartBlock );
}

bool AVP_Game_Core::SaveCheckPointBlock()
{
	s_xCheckpointBlock.Clear();
	return Asura_LoadAndSave::SaveToBuffer( s_xCheckpointBlock );
}

bool AVP_Game_Core::LoadLevelStartBlock( void )
{
	return(LoadBlock(s_xLevelStartBlock));
}

bool AVP_Game_Core::LoadCheckPointBlock( void )
{
	return(LoadBlock(s_xCheckpointBlock));
}

bool AVP_Game_Core::LoadBlock(Asura_StaticSaveBlock &xSaveBlock)
{
	//Asura_Assert(s_xLevelStartBlock, "Caller should check block is available." );

	/// Richard May - 25/04/2003 18:19:21 - make sure the save contains something!
	if( xSaveBlock.GetSpaceUsed() <= 24 )
	{
		return false;
	}

	// Ensure we've got no client-initialisation messages hanging around...
	AVP_Server::ClearClientInitialisationLists();

	bool bSuccess = Asura_LoadAndSave::LoadFromBuffer( xSaveBlock );
	Asura_Assert(bSuccess, "Failed to load from buffer");

    Asura_Entity_System::PostLoadInitialise();
    Axon_System::PostLoadInitialise();

	// Grapple plays with the clipping plane, on the offchance that someone has loaded while mid grapple, reset the clip.
	Asura_Camera::ResetNearFarClip();

	return bSuccess;
}


bool AVP_Game_Core::IsLocalPlayer(const Asura_Guid uGuid)
{
	if(s_pxPlayer)
	{
		return(s_pxPlayer->IsLocalPlayer(uGuid));
	}
	else
	{
		Asura_Assert(AVP_Settings::GetNumberOfLocalPlayers() == 0,"Checking for local player with no player.");
		return(false);
	}
}

Asura_Guid AVP_Game_Core::GetPlayerGuid( void )
{
	if(s_pxPlayer)
	{
		return(s_pxPlayer->GetGuid());
	}
	else
	{
		Asura_Assert(AVP_Settings::GetNumberOfLocalPlayers() == 0,"Checking local player guid with no player.");
		return(ASURA_GUID_UNREGISTERED);
	}
}


#if defined(AVP_ALLOW_GAME_TIME_SCALE_CHANGE)

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Game_Core::UpdateGameTimeScale
// Author       : Andy Tate
// Date         : 1-Oct-2007
////////////////////////////////////////////////////////////////////////////////

void AVP_Game_Core::UpdateGameTimeScale(void)
{
	const float	fTimeScale	= Asura_Timers::GetGameTimeScale();

	float		fNewTimeScale;

	// Update the time-scale...
	s_xGameTimeScaleControlMap.Update();

	// Dont update if the DVS view is down (as it uses the same keys!)
	if (Asura_CommandConsole::IsConsoleDown() || Asura_DebugVariableSystem::IsOpen())
		return;
	
	if (s_bAdvanceOneFrame)
	{
		fNewTimeScale = 0.0f;
		s_bAdvanceOneFrame = false;
	}
	else if (s_xGameTimeScaleControlMap.GetValue(GTS_CHANGE_SPEED) != 0.0f)
	{
		fNewTimeScale	= fTimeScale + s_xGameTimeScaleControlMap.GetValue(GTS_CHANGE_SPEED);
	}
	else if (s_xGameTimeScaleControlMap.GetValue(GTS_RESET) > 0.0f)
	{
		fNewTimeScale	= 1.0f;
	}
	else if (s_xGameTimeScaleControlMap.GetValue(GTS_PAUSE) != 0.0f)
	{
		fNewTimeScale	= 0.0f;
	}
	else if ((fTimeScale == 0.0f) && (s_xGameTimeScaleControlMap.GetValue(GTS_ADVANCE_ONE_FRAME)))
	{
		fNewTimeScale = 1.0f;
		s_bAdvanceOneFrame = true;
	}
	else
	{
		fNewTimeScale	= fTimeScale;
	}

	fNewTimeScale	= Asura_Maths::ClampToRange(fNewTimeScale, 0.0f, 4.0f);

	// Check whether the time-scale has changed...
	if(fNewTimeScale == fTimeScale)
	{
		const float	fTimeStep	= Asura_Timers::GetActualFrameTime();

		s_fGameTimeScaleDisplay	= Asura_Maths::Max(0.0f, s_fGameTimeScaleDisplay - fTimeStep);
	}
	else
	{
		Asura_Timers::SetGameTimeScale(fNewTimeScale);

		s_fCurrentGameTimeScale	= fNewTimeScale;
		s_fGameTimeScaleDisplay	= 1.0f;
	}
}

/// End of function 'AVP_Game_Core::UpdateGameTimeScale'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Game_Core::RenderGameTimeScale
// Author       : Andy Tate
// Date         : 1-Oct-2007
////////////////////////////////////////////////////////////////////////////////

void AVP_Game_Core::RenderGameTimeScale(void)
{
#if defined(ASURA_USE_DEBUG_TEXT)
	if(s_fGameTimeScaleDisplay > 0.0f)
	{
		const u_int	uAlpha	= Asura_Maths::FloatToInt(s_fGameTimeScaleDisplay * 255.0f);
		const u_int	uColour	= (uAlpha << 24) | 0x00FFFFFF;

		// The debug text system asserts if you pass zero alpha, so make sure we don't
		if(uAlpha > 0)
		{
			Asura_DebugText::AddTextBlock(1, 1, uColour, 0, "Time Scale %.3f", s_fCurrentGameTimeScale);
		}
	}
#endif	// defined(ASURA_USE_DEBUG_TEXT)
}

/// End of function 'AVP_Game_Core::RenderGameTimeScale'. ///

#endif	// defined(AVP_ALLOW_GAME_TIME_SCALE_CHANGE)


////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Game_Core::GetCanMakePlayerActors
// Author       : Anthony Gregson
// Date         : 07/11/07
////////////////////////////////////////////////////////////////////////////////

bool AVP_Game_Core::GetCanMakePlayerActors()
{
	// Check if local players have puppet guids, or are happy not to have them.
	if (s_pxPlayer && (s_pxPlayer->GetPuppetGuid() == ASURA_GUID_UNREGISTERED))
	{
		return false;
	}

	return true;
}

/// End of function 'AVP_Game_Core::GetCanMakePlayerActors'. ///

////////////////////////////////////////////////////////////////////////////////
//  CLASS    :  AVP_Game_Core
//  NAME     :  ShouldCompressPlayerCommandMessages()
//  CREATED  :  JG (6.11.2008)
////////////////////////////////////////////////////////////////////////////////
bool AVP_Game_Core::ShouldCompressPlayerCommandsMessages()
{
	bool bCompress = false; 
	
	// Onmly remote clients in multiplayer games compress player command messages
	if ( AVP_Settings::IsMultiPlayer() && !Asura_Network::IsServer() )
	{
		bCompress = true; 
	}
	
	return bCompress; 
}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Game_Core::RequestPause
// Author       : Anthony Gregson
// Date         : 09/11/07
////////////////////////////////////////////////////////////////////////////////

void AVP_Game_Core::RequestPause()
{
#ifdef ASURA_USE_GUI_MENU_SYSTEM
	if (AVP_Settings::IsSinglePlayer())
	{
		if( !AVP_GUIMenu_System::IsMenuActive( AVP_GUIMenu_System::GUIMENU_ID_PAUSE ) )
		{
			AVP_GUIMenu_System::Activate( AVP_GUIMenu_System::GUIMENU_ID_PAUSE );
		}
	}
	else
	{
		if( !AVP_GUIMenu_System::IsMenuActive( AVP_GUIMenu_System::GUIMENU_ID_PAUSE ) )
		{
			AVP_Settings::Species eMenuSpecies = AVP_Settings::AVP_SPECIES_UNKNOWN;
			if (AVP_Settings::GetPlayerSpecies() != AVP_Settings::AVP_SPECIES_UNKNOWN)
			{
				eMenuSpecies = AVP_Settings::GetPlayerSpecies();
			}
			else if (GetRespawnSpecies() != AVP_Settings::AVP_SPECIES_UNKNOWN)
			{
				eMenuSpecies = GetRespawnSpecies(); 
			}
			else
			{
				Asura_Assert(false, "Couldn't find the player species, opening marine pause menu" ); 
				eMenuSpecies = AVP_Settings::AVP_SPECIES_HUMAN; 
			}
			AVP_GUIMenu_System::Activate( AVP_GUIMenu_System::GUIMENU_ID_PAUSE, eMenuSpecies );
		}
	}
#endif //#ifdef ASURA_USE_GUI_MENU_SYSTEM

	SetPausedFSFX(true);

	SetupPausedDSP( true );
}

void AVP_Game_Core::SetupPausedDSP(bool bOn)
{
	if(s_pxPlayer != NULL)
	{
		// Turn off the health DSP effect
		Asura_Guid uPuppetGuid				           = s_pxPlayer->GetPuppetGuid();
		const AVP_ClientActor_Vertebrate* pxVertebrate = AVP_ClientActor_Vertebrate::FindEntity(uPuppetGuid); 

		if ( pxVertebrate && pxVertebrate->GetHUDHandler() )
		{
			const AVP_Vertebrate_HUDHandler* pxHUDHandler = static_cast<const AVP_Vertebrate_HUDHandler*>(pxVertebrate->GetHUDHandler() );
			if (pxHUDHandler)
			{
				pxHUDHandler->SetHealthDSP( !bOn );
			}
		}
	}
}

/// End of function 'AVP_Game_Core::RequestPause'. ///
void AVP_Game_Core::SetPausedFSFX(bool bTurnOn)
{
	// AJK; E3 Set Pause FSFX
	if ( s_pxPlayer )
	{
		// Get the puppet
		Asura_Guid uPuppetGuid				  = s_pxPlayer->GetPuppetGuid();
		const AVP_ClientEntity_Actor* pxActor = AVP_ClientEntity_Actor::FindEntity(uPuppetGuid); 

		if ( pxActor )
		{
			const Asura_Blueprint*	pxBlueprint	= Asura_Blueprint_System::GetBlueprintByHashID(uAVP_BLUEPRINT_TYPE_PLAYERS, pxActor->GetBlueprintHashID());
			if( pxBlueprint )
			{
				Asura_Value	xValueFade;
				if( pxBlueprint->GetParameterValueByHashID(uAVP_BLUEPRINT_PARAM_PAUSE_FADETIME, xValueFade) )
				{
					SetFSFXValue(*pxBlueprint, uAVP_BLUEPRINT_PARAM_PAUSE_FSFX, (bTurnOn)?1.0f:0.0f, xValueFade.GetFloatValue());
				}
			}
		}
	}
}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Game_Core::RequestObjectives
// Author       : AJK
// Date         : 09/10/08
////////////////////////////////////////////////////////////////////////////////

void AVP_Game_Core::RequestObjectives()
{
#ifdef ASURA_USE_GUI_MENU_SYSTEM
	if (AVP_Settings::IsSinglePlayer())
	{
		if( !AVP_GUIMenu_System::IsMenuActive( AVP_GUIMenu_System::GUIMENU_ID_OBJECTIVES ) )
		{
			AVP_GUIMenu_System::Activate( AVP_GUIMenu_System::GUIMENU_ID_OBJECTIVES );
		}
	}
	else
	{
		if( !AVP_GUIMenu_System::IsMenuActive( AVP_GUIMenu_System::GUIMENU_ID_OBJECTIVES ) )
		{
			AVP_Settings::Species eMenuSpecies = AVP_Settings::AVP_SPECIES_UNKNOWN;
			if (AVP_Settings::GetPlayerSpecies() != AVP_Settings::AVP_SPECIES_UNKNOWN)
			{
				eMenuSpecies = AVP_Settings::GetPlayerSpecies();
			}
			else if (GetRespawnSpecies() != AVP_Settings::AVP_SPECIES_UNKNOWN)
			{
				eMenuSpecies = GetRespawnSpecies(); 
			}
			else
			{
				Asura_Assert(false, "Couldn't find the player species, opening marine objectives menu" ); 
				eMenuSpecies = AVP_Settings::AVP_SPECIES_HUMAN; 
			}
			AVP_GUIMenu_System::Activate( AVP_GUIMenu_System::GUIMENU_ID_OBJECTIVES, eMenuSpecies );
		}
	}
#endif //#ifdef ASURA_USE_GUI_MENU_SYSTEM

	SetPausedFSFX(true);

	SetupPausedDSP( true );
}

/// End of function 'AVP_Game_Core::RequestObjectives'. ///

bool AVP_Game_Core::Project_TeleportServerEntity(const Asura_Guid uGuidToTeleport, const Asura_Vector_3& xPosition, const Asura_Quat& xOrientation)
{
	// This might be the guid of a player infor entity. See if it's one of them.
	const AVP_ServerEntity_PlayerInfo* pxPlayerInfo = AVP_ServerEntity_PlayerInfo::FindEntity(uGuidToTeleport);

	if (!pxPlayerInfo) { return false; }

	Asura_ServerEntity_PhysicalObject* pxPuppet = Asura_ServerEntity_PhysicalObject::FindEntity(pxPlayerInfo->GetPuppetGuid());
	if (!pxPuppet) { return true; } // We've still handled it as best we can.

	Asura_Matrix_3x3 xOrientationMatrix;

	xOrientation.ToMatrix(xOrientationMatrix);
	pxPuppet->TeleportTo(xPosition, xOrientationMatrix, true);

	return true;
}


////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Game_Core::SetRespawnSpecies
// Author       : Andy Tate
// Date         : 18-Mar-2008
////////////////////////////////////////////////////////////////////////////////

void AVP_Game_Core::SetRespawnSpecies(const AVP_Settings::Species eSpecies)
{
	s_uRespawnSpecies			= eSpecies;
}



////////////////////////////////////////////////////////////////////////////////
//  NAME     :  GetPlayerBlueprintHashForSpecies()
//  CREATED  :  JG (14.10.2008)
//  DESC     :  Gets the blueprint name to create a player for the required species 
////////////////////////////////////////////////////////////////////////////////

Asura_Hash_ID AVP_Game_Core::GetPlayerBlueprintHashForSpecies( const AVP_Settings::Species eSpecies )
{
	if(AVP_Settings::IsSinglePlayer())
	{
		// Single-player species...
		switch(eSpecies)
		{
			case AVP_Settings::AVP_SPECIES_ALIEN:
			{
				return uAVP_BLUEPRINT_NAME_ALIENSINGLEPLAYER;
			}

			case AVP_Settings::AVP_SPECIES_HUMAN:
			{
				return uAVP_BLUEPRINT_NAME_MARINESINGLEPLAYER;
			}

			case AVP_Settings::AVP_SPECIES_PREDATOR:
			{
				return uAVP_BLUEPRINT_NAME_PREDATORSINGLEPLAYER;
			}

			default:
			{
				Asura_Assert( false, "Invalid species requested, don't know what blueprint should use" );
			}
		}	// End of switch.

		return ASURA_HASH_ID_UNSET;
	}
	else
	{
		// Multiplayer species...
		switch(eSpecies)
		{
			case AVP_Settings::AVP_SPECIES_ALIEN:
			{
				return uAVP_BLUEPRINT_NAME_ALIENMULTIPLAYER;
			}

			case AVP_Settings::AVP_SPECIES_HUMAN:
			{
				return uAVP_BLUEPRINT_NAME_MARINEMULTIPLAYER;
			}

			case AVP_Settings::AVP_SPECIES_PREDATOR:
			{
				return uAVP_BLUEPRINT_NAME_PREDATORMULTIPLAYER;
			}

			default:
			{
				Asura_Assert( false, "Invalid species requested, don't know what blueprint should use" );
			}
		}	// End of switch.

		return ASURA_HASH_ID_UNSET;
	}
}

////////////////////////////////////////////////////////////////////////////////
//  NAME     :  GetStartPointBlueprintHashForSpecies()
//  CREATED  :  JG (14.10.2008)
//  DESC     :  Gets the blueprint name for the start point of the required species
////////////////////////////////////////////////////////////////////////////////

Asura_Hash_ID AVP_Game_Core::GetStartPointBlueprintHashForSpecies( const AVP_Settings::Species eSpecies )
{
	Asura_Hash_ID  uStartPointBlueprintParameterHash = ASURA_HASH_ID_UNSET; 

	switch (eSpecies)
	{
		case AVP_Settings::AVP_SPECIES_ALIEN:
		{
			uStartPointBlueprintParameterHash = 0x592351FB; //Asura_GetHashID("AlienStart");
			break;
		}

		case AVP_Settings::AVP_SPECIES_HUMAN:
		{
			uStartPointBlueprintParameterHash = 0x04DA7FE0; //Asura_GetHashID("MarineStart");
			break;
		}

		case AVP_Settings::AVP_SPECIES_PREDATOR:
		default:
		{
			uStartPointBlueprintParameterHash = 0xEA3683CB; //Asura_GetHashID("PredatorStart");
			break;
		}
	}
	
	return uStartPointBlueprintParameterHash; 
}

////////////////////////////////////////////////////////////////////////////////
//  NAME     :  GetStartPointBlueprintHashForTeam()
//  CREATED  :  JG (14.10.2008)
//  DESC     :  Gets the blueprint name for the start point of the required team
////////////////////////////////////////////////////////////////////////////////

Asura_Hash_ID AVP_Game_Core::GetStartPointBlueprintHashForTeam( const AVP_Settings::Team eTeam )
{
	Asura_Hash_ID  uStartPointBlueprintParameterHash = ASURA_HASH_ID_UNSET; 
	
	switch (eTeam)
	{
		case AVP_Settings::AVP_TEAM_ONE:
		{
			uStartPointBlueprintParameterHash = 0x6cc8836e; //"Team1Start"
			break;
		}

		case AVP_Settings::AVP_TEAM_TWO:
		{
			uStartPointBlueprintParameterHash = 0x6e7d5c0d; //"Team2Start"
			break;
		}

		case AVP_Settings::AVP_TEAM_THREE:
		{
			uStartPointBlueprintParameterHash = 0x703234ac; //"Team3Start"
			break;
		}
		default:
		{
			Asura_Assert( false, "Invalid team, will use the species start point" ); 
			break; 
		}
	}

	return uStartPointBlueprintParameterHash;
}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Game_Core::SetFSFXValue
// Author       : Andy Tate
// Date         : 30-Jun-2008
////////////////////////////////////////////////////////////////////////////////

bool AVP_Game_Core::SetFSFXValue(const Asura_Blueprint& xBlueprint,
										const Asura_Hash_ID uParameterHash,
										const float fValue,
										const float fTime)
{
	Asura_Value	xValue;

	if(uParameterHash == ASURA_HASH_ID_UNSET)
	{
		Asura_Assert(false, "Parameter-hash not filled in when attempting to set a FSFX.");
		return(false);
	}
	else if(xBlueprint.GetParameterValueByHashID(uParameterHash, xValue))
	{
		const Asura_Hash_ID	uFSFXHash	= xValue.GetHashIDValue();

		return SetFSFXValue(uFSFXHash, fValue, fTime);
	}
	else
	{
		return(false);
	}
}

bool AVP_Game_Core::SetFSFXValue(const Asura_Hash_ID uFSFXHash, const float fValue, const float fTime)
{
	if (uFSFXHash == ASURA_HASH_ID_UNSET)
	{
		return false;
	}
	else
	{
#if defined(ASURA_TOOLS)
		if (Tools_System::IsActive())
		{
			// (AAM - 15.10.2007) Don't do this in FSFX mode, since that will be previewing
			// other templates
			if (Tools_System::GetCurrentMode() != TOOLS_MODE_FSFX)
			{
				Asura_FSFX_Template* const pxTemplate = Asura_FSFX_System::FindTemplate(uFSFXHash);

				if(pxTemplate != NULL)
				{
					Tools_FSFX_System::SetOverrideFSFX(pxTemplate, fValue);
				}
			}
		}
		else
#endif
		{
			s_pxFSFXHandler->SetValue(uFSFXHash, fValue, fTime);
		}
		return true;
	}
}


void AVP_Game_Core::SetVisionMode(const Asura_Hash_ID uNewBlueprintHash,
								  const Asura_Hash_ID uOldBlueprintHash,
								  const bool bInstant)
{
	// NB: In Tools, don't check if we're already in this vision mode. Need to do this in case user
	// changes a blueprint
#if defined(ASURA_TOOLS)
	bool bDontRedoOverrides = uNewBlueprintHash == uOldBlueprintHash;

	if(!Tools_System::IsActive())
#endif
	{
		if(uNewBlueprintHash == uOldBlueprintHash) return;
	}

	const Asura_Blueprint*	pxOldModeBlueprint	= AVP_Actor_HUDHandler::GetInventoryBlueprint(uOldBlueprintHash);
	const Asura_Blueprint*	pxNewModeBlueprint	= AVP_Actor_HUDHandler::GetInventoryBlueprint(uNewBlueprintHash);

	if( ((pxOldModeBlueprint == NULL) && (uOldBlueprintHash != ASURA_HASH_ID_UNSET)) ||
		((pxNewModeBlueprint == NULL) && (uNewBlueprintHash != ASURA_HASH_ID_UNSET)) )
	{
		Asura_Assert(false, "Failed to find vision mode blueprint.");
		return;
	}

	Asura_Value xValue;

	// Stop old FSFX, if not in Tools

#if defined(ASURA_TOOLS)
	if(!Tools_System::IsActive())
#endif
	{
		if( pxOldModeBlueprint
			&& pxOldModeBlueprint->GetParameterValueByHashID(uAVP_BLUEPRINT_PARAM_FSFX_TEMPLATE, xValue))
		{
			const Asura_Hash_ID uFSFXHash = xValue.GetHashIDValue();
			if (uFSFXHash != ASURA_HASH_ID_UNSET)
			{
				s_pxFSFXHandler->SetValue(uFSFXHash, 0.0f, 0.1f );
			}
		}
	}

	if(pxNewModeBlueprint)
	{
		// Play new FSFX. If in Tools, have to go through Tools_FSFX_System
		if(pxNewModeBlueprint->GetParameterValueByHashID(uAVP_BLUEPRINT_PARAM_FSFX_TEMPLATE, xValue))
		{
			const Asura_Hash_ID uFSFXHash = xValue.GetHashIDValue();
			if (uFSFXHash != ASURA_HASH_ID_UNSET)
			{
				float fFSFXIntensity;
				float fFSFXFadeTime;

				if(pxNewModeBlueprint->GetParameterValueByHashID(uAVP_BLUEPRINT_PARAM_FSFX_INTENSITY, xValue))
				{
					fFSFXIntensity = xValue.GetFloatValue();
				}
				else
				{
					fFSFXIntensity	= 1.0f;
				}

				if(bInstant)
				{
					fFSFXFadeTime	= 0.0f;
				}
				else
				{
					if(pxNewModeBlueprint->GetParameterValueByHashID(uAVP_BLUEPRINT_PARAM_FSFX_FADE_TIME, xValue))
					{
						fFSFXFadeTime = xValue.GetFloatValue();
					}
					else
					{
						fFSFXFadeTime = 0.35f;
					}
				}

				SetFSFXValue(*pxNewModeBlueprint, uAVP_BLUEPRINT_PARAM_FSFX_TEMPLATE, fFSFXIntensity, fFSFXFadeTime);
			}

#if defined(ASURA_TOOLS)
			else if(Tools_System::IsActive() && Tools_System::GetCurrentMode() != TOOLS_MODE_FSFX)
			{
				Tools_FSFX_System::SetOverrideFSFX(NULL, 1.0f);
			}
#endif
		}

#if defined(ASURA_TOOLS)
		if (!bDontRedoOverrides)
		{
#endif
			// Set material response (and now PFX too) override
			if(pxNewModeBlueprint->GetParameterValueByHashID(uAVP_BLUEPRINT_PARAM_MATERIAL_OVERRIDE, xValue))
			{
				const Asura_Hash_ID uOverrideHash = xValue.GetHashIDValue();
				Asura_MaterialResponse_System::SetCurrentOverrideMap(uOverrideHash);
				Asura_PFX_System::GetSourceTempResourceSet().EnableOverrideMap(uOverrideHash, true);
				Asura_PFX_System::GetParticleTempResourceSet().EnableOverrideMap(uOverrideHash, true);
				Asura_FSFX_System::EnableOverrideMap(uOverrideHash, true);
			} 
			else 
			{
				Asura_MaterialResponse_System::ResetCurrentOverrideMap();
				Asura_PFX_System::GetSourceTempResourceSet().DisableAllOverrideMaps();
				Asura_PFX_System::GetParticleTempResourceSet().DisableAllOverrideMaps();
				Asura_FSFX_System::DisableAllOverrideMaps();
			}
#if defined(ASURA_TOOLS)
		}
#endif

		// Set HDR settings
		{
			GetHDRSettingsFromBlueprint(*pxNewModeBlueprint, s_xVisionModeHDRSettings);

			if ( pxNewModeBlueprint->GetParameterValueByHashID(uAVP_BLUEPRINT_PARAM_HDR_ALLOW_TORCH_OVERRIDE, xValue))
			{
				AVP_Actor_HUDHandler::SetAllowTorchOverride(xValue.GetBoolValue());
			}

			if ( AVP_Actor_HUDHandler::IsTorchOverride() && AVP_Actor_HUDHandler::IsAllowTorchOverride() )
			{
				Asura_Camera::SetHDRSettings(AVP_Actor_HUDHandler::GetTorchHDRSettings());
			}
			else
			{
				Asura_Camera::SetHDRSettings(s_xVisionModeHDRSettings);
			}
		}

		bool bAllowLighting;
		if(pxNewModeBlueprint->GetParameterValueByHashID(uAVP_BLUEPRINT_PARAM_DISABLE_LIGHTING, xValue))
		{
			bAllowLighting = !xValue.GetBoolValue();
		}
		else
		{
			bAllowLighting = true;
		}
		Asura_Render::SetSystemEnabled(ASURA_RENDER_SYSTEM_CORONAS, bAllowLighting);
		Asura_Render::SetSystemEnabled(ASURA_RENDER_SYSTEM_LIGHTSHAFTS, bAllowLighting);
		Asura_Render::SetSystemEnabled(ASURA_RENDER_SYSTEM_PHOTONS, bAllowLighting);
		//		Asura_Render::SetSystemEnabled(ASURA_RENDER_SYSTEM_PROJECTORS, bAllowLighting);
		Asura_Render::SetSystemEnabled(ASURA_RENDER_SYSTEM_SHADOWS, bAllowLighting);
	}
	else
	{
		Asura_MaterialResponse_System::ResetCurrentOverrideMap();
		Asura_PFX_System::GetSourceTempResourceSet().DisableAllOverrideMaps();
		Asura_PFX_System::GetParticleTempResourceSet().DisableAllOverrideMaps();

#if defined(ASURA_TOOLS)
		if(Tools_System::IsActive())
		{
			Tools_Render_Options::UpdateFSFXSettings();
		}
#endif
	}
}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Game_Core::GetHDRSettingsFromBlueprint
// Author       : Andy McDonald
// Date         : 9-Sept-2008
////////////////////////////////////////////////////////////////////////////////

void AVP_Game_Core::GetHDRSettingsFromBlueprint(const Asura_Blueprint& xBlueprint, Asura_CameraData::HDRSettings& xSettings)
{
	Asura_Value xValue;

	Asura_Camera::GetHDRSettings(xSettings);

	if(xBlueprint.GetParameterValueByHashID(uAVP_BLUEPRINT_PARAM_HDR_ADAPTATION_RATE, xValue))
	{
		xSettings.m_fAdaptationRate = xValue.GetFloatValue();
	}
	if(xBlueprint.GetParameterValueByHashID(uAVP_BLUEPRINT_PARAM_HDR_MAX_LUMINANCE, xValue))
	{
		xSettings.m_fMaxLuminance = xValue.GetFloatValue();
	}
	if(xBlueprint.GetParameterValueByHashID(uAVP_BLUEPRINT_PARAM_HDR_MIN_LUMINANCE, xValue))
	{
		xSettings.m_fMinLuminance = xValue.GetFloatValue();
	}
	if(xBlueprint.GetParameterValueByHashID(uAVP_BLUEPRINT_PARAM_HDR_SCENE_LUMINANCE, xValue))
	{
		xSettings.m_fKeyValue = xValue.GetFloatValue();
	}
	if(xBlueprint.GetParameterValueByHashID(uAVP_BLUEPRINT_PARAM_HDR_WHITE_POINT, xValue))
	{
		xSettings.m_fWhitePoint = xValue.GetFloatValue();
	}
	if(xBlueprint.GetParameterValueByHashID(uAVP_BLUEPRINT_PARAM_HDR_BLACK_POINT, xValue))
	{
		xSettings.m_fBlackPoint = xValue.GetFloatValue();
	}
	if(xBlueprint.GetParameterValueByHashID(uAVP_BLUEPRINT_PARAM_HDR_DYNAMIC_EXPOSURE, xValue))
	{
		xSettings.m_bDynamicExposure = xValue.GetBoolValue();
	}
	if(xBlueprint.GetParameterValueByHashID(uAVP_BLUEPRINT_PARAM_HDR_FIXED_LUMINANCE, xValue))
	{
		xSettings.m_fFixedLuminance = xValue.GetFloatValue();
	}
	if(xBlueprint.GetParameterValueByHashID(uAVP_BLUEPRINT_PARAM_HDR_REGION_BLX, xValue))
	{
		xSettings.m_xRegion.MinX = xValue.GetFloatValue();
	}
	if(xBlueprint.GetParameterValueByHashID(uAVP_BLUEPRINT_PARAM_HDR_REGION_BLY, xValue))
	{
		xSettings.m_xRegion.MinY = xValue.GetFloatValue();
	}
	if(xBlueprint.GetParameterValueByHashID(uAVP_BLUEPRINT_PARAM_HDR_REGION_TRX, xValue))
	{
		xSettings.m_xRegion.MaxX = xValue.GetFloatValue();
	}
	if(xBlueprint.GetParameterValueByHashID(uAVP_BLUEPRINT_PARAM_HDR_REGION_TRY, xValue))
	{
		xSettings.m_xRegion.MaxY = xValue.GetFloatValue();
	}

	#if defined(ASURA_TOOLS)

		// (AAM - 18.10.2007) Allow user to force fixed exposure, makes it easier to work
		// around the black screen bug if it pops up again

		if(AVP_Actor_HUDHandler::IsCnsl_bForceFixedExposure())
		{
			xSettings.m_bDynamicExposure = false;

			Asura_CameraData::HDRSettings xUserSettings;
			Tools_Render_Options::GetHDRSettings(xUserSettings);

			xSettings.m_fFixedLuminance = xUserSettings.m_fFixedLuminance;
		}

	#endif // defined(ASURA_TOOLS)
}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Game_Core::ResetFSFXValue
// Author       : Andy Tate
// Date         : 30-Jun-2008
////////////////////////////////////////////////////////////////////////////////

bool AVP_Game_Core::ResetFSFXValue(const Asura_Blueprint& xBlueprint,
										  const Asura_Hash_ID uFSFXParameterHash)
{
	Asura_Value	xValue;

	if(uFSFXParameterHash == ASURA_HASH_ID_UNSET)
	{
		Asura_Assert(false, "Parameter-hash not filled in when attempting to reset a FSFX.");
		return(false);
	}
	else if(xBlueprint.GetParameterValueByHashID(uFSFXParameterHash, xValue))
	{
		const Asura_Hash_ID	uFSFXHash	= xValue.GetHashIDValue();

		if(uFSFXHash == ASURA_HASH_ID_UNSET)
		{
			return(false);
		}
		else
		{
			s_pxFSFXHandler->ResetValue(uFSFXHash);
			return(true);
		}
	}
	else
	{
		return(false);
	}
}

const Asura_Blueprint* AVP_Game_Core::GetLevelBlueprint( const char* szLevelFile, CharacterType eCharType )
{
	Asura_String xLevelNameWithPathAndExtension( szLevelFile );
	Asura_String xLevelNameStripped;

	// strip the path and extension of the level file then use the level name to find and set the correct environment
	xLevelNameWithPathAndExtension.SplitFilename( NULL, &xLevelNameStripped, NULL );

	const Asura_Blueprint* pxLevelBP = NULL;
	if ( eCharType >= FIRST_SINGLE_PLAYER_CHARACTER && eCharType <= LAST_SINGLE_PLAYER_CHARACTER )
	{
		pxLevelBP = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_LEVELS, xLevelNameStripped.GetHash() );
		if ( pxLevelBP )
		{
			bool bRightSpecies = false;
			AVP_Settings::Species eLevelSpecies = static_cast< AVP_Settings::Species >( pxLevelBP->GetParameterAsInt( 0x11d155d1, AVP_Settings::AVP_SPECIES_UNKNOWN ) );	// Character Type
			switch ( eLevelSpecies )
			{
				case AVP_Settings::AVP_SPECIES_ALIEN: bRightSpecies = ( eCharType == SINGLE_PLAYER_ALIEN ); break;
				case AVP_Settings::AVP_SPECIES_HUMAN: bRightSpecies = ( eCharType == SINGLE_PLAYER_MARINE ); break;
				case AVP_Settings::AVP_SPECIES_PREDATOR: bRightSpecies = ( eCharType == SINGLE_PLAYER_PREDATOR ); break;
				default: break;
			}
			if ( !bRightSpecies )
			{
				pxLevelBP = NULL;
			}
		}
	}
	if ( !pxLevelBP )
	{
		switch( eCharType )
		{
			case SINGLE_PLAYER_ALIEN:
			{
				pxLevelBP = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_LEVELS, 0x4e6a0ce5 ); // AlienMission
			}
			break;

			case SINGLE_PLAYER_MARINE:
			{
				pxLevelBP = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_LEVELS, 0xe90d558a ); // MarineMission
			}
			break;

			case SINGLE_PLAYER_PREDATOR:
			{
				pxLevelBP = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_LEVELS, 0xe7780ab5 ); // PredatorMission
			}
			break;

			case MULTIPLAYER_ALIEN:
			{
				pxLevelBP = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_LEVELS, 0x8e591833 ); // AlienMultiplayer
			}
			break;

			case MULTIPLAYER_MARINE:
			{
				pxLevelBP = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_LEVELS, 0x1f2d8658 ); // MarineMultiplayer
			}
			break;

			case MULTIPLAYER_PREDATOR:
			{
				pxLevelBP = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_LEVELS, 0xd193ae03 ); // PredatorMultiplayer
			}
			break;

			default:
			{
				Asura_Assert( false, "Unknown species" );
			}
			break;
		}
	}
	Asura_Assertf( pxLevelBP, ( "Unable to get level blueprint for level %s", xLevelNameStripped.GetString() ) );
	return pxLevelBP;
}

////////////////////////////////////////////////////////////////////////////////
//  CLASS    :  AVP_Game_Core
//  NAME     :  Respawn()
//  CREATED  :  JG (1.10.2008)
//  DESC     :  Debug console command to kill the local player
////////////////////////////////////////////////////////////////////////////////
bool AVP_Game_Core::Respawn(bool bHelp)
{
	if (bHelp)
	{
		Asura_CommandConsole::Printf(L"Kill the local player"); 
		return true; 
	}
	else
	{
		// Get the local player
		if (s_pxPlayer)
		{
			// Get the puppet
			Asura_Guid uVictimGuid = s_pxPlayer->GetPuppetGuid();
			const AVP_ClientEntity_Actor* pxVictimActor = AVP_ClientEntity_Actor::FindEntity(uVictimGuid); 
			if (pxVictimActor)
			{
				Asura_Message* pxMessage = Asura_Network::CreateMessage(AVP_MSGID_ENTITYDAMAGE_REQUEST_VICTIM, sizeof(AVP_Message_EntityDamage_RequestVictim));
				if(pxMessage)
				{
					AVP_Message_EntityDamage_RequestVictim*	pxData = static_cast<AVP_Message_EntityDamage_RequestVictim*>(pxMessage->GetData());
					const AVP_EntityDamage_RequestVictim xDamage(	uAVP_BLUEPRINT_NAME_DAMAGE_FALLING,
																	ASURA_GUID_UNREGISTERED,
																	ASURA_HASH_ID_UNSET,
																	uVictimGuid,
																	pxVictimActor->GetHealth(),
																	pxVictimActor->GetCurrentPosition(),
																	Asura_Vector_3(0.0f, 1.0f, 0.0f),
																	Asura_Vector_3(1.0f, 0.0f, 0.0f),
																	0.0f,
																	-1,
																	false,
																	ASURA_GUID_UNREGISTERED );
					pxData->SetDamage(xDamage);

					pxMessage->SetFromGuid(uVictimGuid);
					pxMessage->SetToGuid(uVictimGuid);
					Asura_Network::Send(pxMessage);
				}
			}
		}
		return true; 
	}
}
/// End of file. ///
