//////////////////////////////////////////////////////////////////////////////
//
// AVP_Profile_Graphics.cpp
//
// (AJK - 04.08.2008) Created
//
// Copyright (c) 2008 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"

// header
#include "Core/Profile/AVP_Profile_Graphics.h"

// Asura
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "Interface/Asura_CommandConsole.h"

// AVP
#include "Core/AVP_Core.h"
#include "Core/AVP_Settings.h"
#include "GUIMenu/Asura_GUIMenu_System.h"

AVP_Profile_Graphics::AVP_Profile_Graphics()
{
	Platform_SetToDefaults();
}

void AVP_Profile_Graphics::WriteToChunkStream(Asura_Chunk_Stream& xStream) 
{
	int iCurrentChunkVersion = CURRENT_CHUNK_VERSION;

	xStream << iCurrentChunkVersion;

	xStream << m_uVisualQuality;
	xStream << m_uAntiAliasing;
	xStream << m_uTextureQuality;
	xStream << m_uShadowQuality;
	xStream << m_uResolutionIndex;
	xStream << m_bTrilinearFiltering;
	xStream << m_fGamma;
	xStream << m_uAnisotropicFiltering;
}

void AVP_Profile_Graphics::ReadFromChunkStream(Asura_Chunk_Stream& xStream)
{
	int iCurrentChunkVersion;

	xStream >> iCurrentChunkVersion;

	switch ( iCurrentChunkVersion )
	{
	case 2 :
		{
			xStream >> m_uVisualQuality;
			xStream >> m_uAntiAliasing;
			xStream >> m_uTextureQuality;
			xStream >> m_uShadowQuality;
			xStream >> m_uResolutionIndex;
			xStream >> m_bTrilinearFiltering;
			xStream >> m_fGamma;
			xStream >> m_uAnisotropicFiltering;
			break;
		}
	case 1 :
		{
			//Changing the gamma settings so need to force it to 0.5f
			xStream >> m_uVisualQuality;
			xStream >> m_uAntiAliasing;
			xStream >> m_uTextureQuality;
			xStream >> m_uShadowQuality;
			xStream >> m_uResolutionIndex;
			xStream >> m_bTrilinearFiltering;
			xStream >> m_fGamma;
			xStream >> m_uAnisotropicFiltering;
			m_fGamma = 0.5f;
			break;
		}
	case 0 :
		{
			xStream >> m_uVisualQuality;
			xStream >> m_uTextureQuality;
			xStream >> m_uShadowQuality;
			xStream >> m_uResolutionIndex;
			xStream >> m_bTrilinearFiltering;
			xStream >> m_fGamma;
			xStream >> m_uAnisotropicFiltering;
			break;
		}
	default :
		{
			Asura_Assertf(false, ("Unknown chunk version (%d) in AVP_Profile_Graphics", iCurrentChunkVersion));
			break;
		}
	}
}

#if defined(ASURA_SUPPORT_COMMAND_CONSOLE)

void AVP_Profile_Graphics::SetupConsoleCommands()
{
	// Add the necessary console commands...
	Asura_CommandConsole::AddCmd( L"Profile.CMD.ResolutionsList"	  , AVP_Profile_Graphics::Platform_ConsoleCmd_ResolutionsList                , Asura_CommandConsole::REMOVE_AT_SHUTDOWN, (1 << 2));
	Asura_CommandConsole::AddCmd( L"Profile.CMD.ChangeResolution"	  , AVP_Profile_Graphics::Platform_ConsoleCmd_ChangeResolution               , Asura_CommandConsole::REMOVE_AT_SHUTDOWN, (1 << 2));
	Asura_CommandConsole::AddCmd( L"Profile.CMD.RevertToOldRes"		  , AVP_Profile_Graphics::Platform_ConsoleCmd_RevertResolution               , Asura_CommandConsole::REMOVE_AT_SHUTDOWN, (1 << 2));

	Asura_CommandConsole::AddCmd( L"Profile.CMD.AcceptNewRes"		  , AVP_Profile_Graphics::Platform_ConsoleCmd_AcceptNewRes                   , Asura_CommandConsole::REMOVE_AT_SHUTDOWN, (1 << 2));
	Asura_CommandConsole::AddCmd( L"Profile.CMD.ChangeVisualDetails"  , AVP_Profile_Graphics::Platform_ConsoleCmd_ChangeVisualDetails            , Asura_CommandConsole::REMOVE_AT_SHUTDOWN, (1 << 2));
	Asura_CommandConsole::AddCmd( L"Profile.CMD.ChangeTextureDetails" , AVP_Profile_Graphics::Platform_ConsoleCmd_ChangeTextureDetails           , Asura_CommandConsole::REMOVE_AT_SHUTDOWN, (1 << 2));
	Asura_CommandConsole::AddCmd( L"Profile.CMD.ChangeShadowDetails"  , AVP_Profile_Graphics::Platform_ConsoleCmd_ChangeShadowDetails            , Asura_CommandConsole::REMOVE_AT_SHUTDOWN, (1 << 2));
	Asura_CommandConsole::AddCmd( L"Profile.CMD.ChangeAniFilterLeft"  , AVP_Profile_Graphics::Platform_ConsoleCmd_ChangeAnisotropicFilteringLeft , Asura_CommandConsole::REMOVE_AT_SHUTDOWN, (1 << 2)); // DynLabel_AniFiltering
	Asura_CommandConsole::AddCmd( L"Profile.CMD.ChangeAniFilterRight" , AVP_Profile_Graphics::Platform_ConsoleCmd_ChangeAnisotropicFilteringRight, Asura_CommandConsole::REMOVE_AT_SHUTDOWN, (1 << 2)); // DynLabel_AniFiltering
	Asura_CommandConsole::AddCmd( L"Profile.CMD.OnInitGraphicsOptions", AVP_Profile_Graphics::Platform_ConsoleCmd_OnInitGraphicsOptions          , Asura_CommandConsole::REMOVE_AT_SHUTDOWN, (1 << 2));
}

void AVP_Profile_Graphics::SetupConsoleVariables()
{
	// Add the necessary console variables...
	Asura_CommandConsole::AddVar( L"Render.VAR.AntiAliasing"		, &m_uAntiAliasing			 , 0		, 3		, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::AddVar( L"Render.VAR.TextureDetails"      , &m_uTextureQuality         , 0		, 2		, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::AddVar( L"Render.VAR.VisualDetails"       , &m_uVisualQuality          , 0		, 2		, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::AddVar( L"Render.VAR.ResolutionIndex"     , &m_uResolutionIndex        , 0		, 999	, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::AddVar( L"Render.VAR.TrilinearFiltering"  , &m_bTrilinearFiltering						, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::AddVar( L"Render.VAR.Gamma"               , &m_fGamma                  , 0.0f		, 1.0f	, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::AddVar( L"Render.VAR.ShadowQuality"       , &m_uShadowQuality          , 0		, 2		, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::AddVar( L"Render.VAR.ConfirmNewResolution", &m_uNewResConfirmationTimer, 0		, 15	, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::MakeVarReadOnly(L"Render.VAR.ResolutionIndex");
}

void AVP_Profile_Graphics::RemoveConsoleVariables()
{
	// Add the necessary console variables...
	Asura_CommandConsole::RemoveVar( L"Render.VAR.AntiAliasing" );
	Asura_CommandConsole::RemoveVar( L"Render.VAR.TextureDetails" );
	Asura_CommandConsole::RemoveVar( L"Render.VAR.VisualDetails" );
	Asura_CommandConsole::RemoveVar( L"Render.VAR.ResolutionIndex" );
	Asura_CommandConsole::RemoveVar( L"Render.VAR.TrilinearFiltering" );
	Asura_CommandConsole::RemoveVar( L"Render.VAR.Gamma" );
	Asura_CommandConsole::RemoveVar( L"Render.VAR.ShadowQuality" );
	Asura_CommandConsole::RemoveVar( L"Render.VAR.ConfirmNewResolution" );
}

#endif	//defined(ASURA_SUPPORT_COMMAND_CONSOLE)

u_int AVP_Profile_Graphics::GetCurSpeciesConfirmResMenuHash()
{
	if( ! AVP_Core::IsInGameState() )
	{
		// if the program is not in the game state then return the hash for the front end res confirm screen as it is species independant. 
		return s_uAVP_PROFILE_FRONTEND_RES_CONFIRM_MENU;
	}

	switch(AVP_Settings::GetPlayerSpecies())
	{
		case AVP_Settings::AVP_SPECIES_HUMAN:
		{
			return s_uAVP_PROFILE_MARINE_CONFIRM_RES_MENU;
		}
		case AVP_Settings::AVP_SPECIES_ALIEN:
		{
			return s_uAVP_PROFILE_ALIEN_CONFIRM_RES_MENU;
		}
		case AVP_Settings::AVP_SPECIES_PREDATOR:
		{
			return s_uAVP_PROFILE_PREDATOR_CONFIRM_RES_MENU;
		}
		case AVP_Settings::AVP_SPECIES_UNKNOWN:
		{
			// This is valid, we don't know what species we are yet
			return ASURA_HASH_ID_UNSET; 
		}
		default :
		{
			Asura_Assert(false,"Wrong species passed in AVP_Profile_Graphics");
			return ASURA_HASH_ID_UNSET;
		}
	}
}

u_int AVP_Profile_Graphics::GetCurSpeciesResolutionsMenuHash()
{
	if( ! AVP_Core::IsInGameState() )
	{
		// if the program is not in the game state then return the hash for the front end res screen as it is species independant. 
		return s_uAVP_PROFILE_FRONTEND_RESOLUTION_MENU;
	}

	switch(AVP_Settings::GetPlayerSpecies())
	{
		case AVP_Settings::AVP_SPECIES_HUMAN :
		{
			return s_uAVP_PROFILE_MARINE_RESOLUTIONS_MENU;
		}
		case AVP_Settings::AVP_SPECIES_ALIEN :
		{
			return s_uAVP_PROFILE_ALIEN_RESOLUTIONS_MENU;
		}
		case AVP_Settings::AVP_SPECIES_PREDATOR :
		{
			return s_uAVP_PROFILE_PREDATOR_RESOLUTIONS_MENU;
		}
		case AVP_Settings::AVP_SPECIES_UNKNOWN:
		{
			// This is valid, we don't know what species we are yet
			return ASURA_HASH_ID_UNSET; 
		}
		default :
		{
			Asura_Assert(false,"Wrong species passed in AVP_Profile_Graphics");
			return ASURA_HASH_ID_UNSET;
		}
	}
}

u_int AVP_Profile_Graphics::GetCurSpeciesGraphicsMenuHash()
{
	if( ! AVP_Core::IsInGameState() )
	{
		// if the program is not in the game state then return the hash for the frontend video options screen as it is species independant. 
		return s_uAVP_PROFILE_FRONTEND_GRAPHICS_OPT_MENU;
	}

	switch(AVP_Settings::GetPlayerSpecies())
	{
		case AVP_Settings::AVP_SPECIES_HUMAN :
		{
			return s_uAVP_PROFILE_MARINE_GRAPHICS_OPT_MENU;
		}
		case AVP_Settings::AVP_SPECIES_ALIEN :
		{
			return s_uAVP_PROFILE_ALIEN_GRAPHICS_OPT_MENU;
		}
		case AVP_Settings::AVP_SPECIES_PREDATOR :
		{
			return s_uAVP_PROFILE_PREDATOR_GRAPHICS_OPT_MENU;
		}
		case AVP_Settings::AVP_SPECIES_UNKNOWN:
		{
			// This is valid, we don't know what species we are yet
			return ASURA_HASH_ID_UNSET; 
		}
		default :
		{
			Asura_Assert(false,"Wrong species passed in AVP_Profile_Graphics");
			return ASURA_HASH_ID_UNSET;
		}
	}
}

void AVP_Profile_Graphics::Update()
{
	Platform_UpdateNewResConfirmationTimer();
	
	if( ( Asura_GUIMenu_System::IsMenuPageHashActive( GetCurSpeciesGraphicsMenuHash() ) ) || ( Asura_GUIMenu_System::IsMenuPageHashActive( s_uAVP_PROFILE_FRONTEND_GRAPHICS_OPT_MENU ) ) )
	{
		ApplySettings();
	}
}

void AVP_Profile_Graphics::ApplySettings()
{
	Platform_ApplySettings();
}

bool  AVP_Profile_Graphics::operator!=(const AVP_Profile_Graphics& xOther) const
{
	return 	 !(*this == xOther) ;
}

bool  AVP_Profile_Graphics::operator==(const AVP_Profile_Graphics& xOther) const
{
	return 	( m_uVisualQuality        == xOther.m_uVisualQuality        ) &&
			( m_uAntiAliasing         == xOther.m_uAntiAliasing         ) &&
			( m_uTextureQuality       == xOther.m_uTextureQuality       ) &&
			( m_uShadowQuality        == xOther.m_uShadowQuality        ) &&
			( m_uResolutionIndex      == xOther.m_uResolutionIndex      ) &&
			( m_bTrilinearFiltering   == xOther.m_bTrilinearFiltering   ) &&
			( m_fGamma                == xOther.m_fGamma                ) &&
			( m_uAnisotropicFiltering == xOther.m_uAnisotropicFiltering );
}