//////////////////////////////////////////////////////////////////////////////
//
// AVP_Profile_Controls.h
//
// (AJK - 04.08.2008) Created
//
// Copyright (c) 2008 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

#ifndef __AVP_PROFILE_GRAPHICS__
#define __AVP_PROFILE_GRAPHICS__

//Forward declaration
class Asura_Chunk_Stream;

class AVP_Profile_Graphics
{

public:

	AVP_Profile_Graphics();

#if defined(ASURA_SUPPORT_COMMAND_CONSOLE)
	void SetupConsoleCommands();
	void SetupConsoleVariables();
	void RemoveConsoleVariables();
#endif	//defined(ASURA_SUPPORT_COMMAND_CONSOLE)

	void Platform_SetToDefaults();
    bool HasChanged();

	void WriteToChunkStream(Asura_Chunk_Stream& xStream);
	void ReadFromChunkStream(Asura_Chunk_Stream& xStream);

	static bool Platform_ConsoleCmd_ChangeVisualDetails       ( bool bHelp);
	static bool Platform_ConsoleCmd_ChangeResolution          ( u_int uResIndex, bool bInterrogate, bool bHelp );
	static bool Platform_ConsoleCmd_RevertResolution          ( bool bHelp );
	static bool Platform_ConsoleCmd_ResolutionsList           ( u_int uResIndex, bool bInterrogate, bool bHelp );
	static bool Platform_ConsoleCmd_ChangeTextureDetails      ( bool bHelp);
	static bool Platform_ConsoleCmd_ChangeShadowDetails       ( bool bHelp);
	static bool Platform_ConsoleCmd_ChangeAnisotropicFilteringRight( bool bHelp);
	static bool Platform_ConsoleCmd_ChangeAnisotropicFilteringLeft( bool bHelp);
    static bool Platform_ConsoleCmd_AcceptNewRes					   ( bool bHelp);
	static bool Platform_ConsoleCmd_OnInitGraphicsOptions( bool bHelp);
	static void Platform_UpdateNewResConfirmationTimer();

	static u_int GetCurSpeciesConfirmResMenuHash();
	static u_int GetCurSpeciesResolutionsMenuHash();
	static u_int GetCurSpeciesGraphicsMenuHash();

	//Getters
	u_int GetVisualQuality		 () const { return m_uVisualQuality       ;}
	u_int GetAntiAliasing		 () const { return m_uAntiAliasing        ;}
	u_int GetTextureQuality		 () const { return m_uTextureQuality      ;}
	u_int GetShadowQuality	     () const { return m_uShadowQuality       ;}
	u_int GetResolutionIndex	 () const { return m_uResolutionIndex     ;}
	bool  IsTrilinearFiltering	 () const { return m_bTrilinearFiltering  ;}
	float GetGamma               () const { return m_fGamma               ;}
	u_int GetAnisotropicFiltering() const { return m_uAnisotropicFiltering;}

	//Setters
	void SetVisualQuality		( u_int uVisualQuality        ) { m_uVisualQuality        = uVisualQuality       ; }
	void SetAntiAliasing		( u_int uAntiAliasing		  ) { m_uAntiAliasing         = uAntiAliasing        ; }
	void SetTextureQuality		( u_int uTextureQuality       ) { m_uTextureQuality       = uTextureQuality      ; }
	void SetShadowQuality	    ( u_int uShadowQuality        ) { m_uShadowQuality        = uShadowQuality       ; }
	void SetResolutionIndex	    ( u_int uResolutionIndex      ) { m_uResolutionIndex      = uResolutionIndex     ; }
	void SetTrilinearFiltering	( bool  bTrilinearFiltering   ) { m_bTrilinearFiltering   = bTrilinearFiltering  ; }
	void SetGamma               ( float fGamma                ) { m_fGamma                = fGamma               ; }
	void SetAnisotropicFiltering( u_int uAnisotropicFiltering ) { m_uAnisotropicFiltering = uAnisotropicFiltering; }

	bool  operator==(const AVP_Profile_Graphics& xOther) const;
	bool  operator!=(const AVP_Profile_Graphics& xOther) const;

    void Update();

	// AJG - 23/12/08 - Nothing to apply at this point. We should probably update the resolution (on PC) but since, slightly bizarrely, we're currently storing that as an index (which just refers to
	//					the resolution enumeration, not an internal AvP list, so it can change what it means between executions of the program) it doesn't seem worth doing that right now.
	void ApplySettings();
	void Platform_ApplySettings();

private:

	enum 
	{
		CURRENT_CHUNK_VERSION = 2,
		CONFIRMATION_TIME_LENGTH = 15,
	};

	static const u_int s_uAVP_PROFILE_FRONTEND_RESOLUTION_MENU		= 0xfdcfb54d; //Asura_GetHashID( "Options_Video_Resolution_FE"); 
	static const u_int s_uAVP_PROFILE_FRONTEND_RES_CONFIRM_MENU		= 0x45b402e2; //Asura_GetHashID( "Options_Video_Res_Confirm_FE"); 
	static const u_int s_uAVP_PROFILE_FRONTEND_GRAPHICS_OPT_MENU	= 0xDEA91F48; //Asura_GetHashID( "M_Options_Video");

	static const u_int s_uAVP_PROFILE_DYN_FILETRING_LBL				= 0xC02068B2; //Asura_GetHashID( "DynLabel_AniFiltering"); 
	static const u_int s_uAVP_PROFILE_MARINE_CONFIRM_RES_MENU		= 0xD5843D28; //Asura_GetHashID( "Marine_Confirm_Resolution");
	static const u_int s_uAVP_PROFILE_MARINE_RESOLUTIONS_MENU		= 0x05E5F64A; //Asura_GetHashID( "Marine_Resolutions");
	static const u_int s_uAVP_PROFILE_MARINE_GRAPHICS_OPT_MENU		= 0x9C4FA868; //Asura_GetHashID( "Marine_Graphics"); 
	static const u_int s_uAVP_PROFILE_ALIEN_CONFIRM_RES_MENU		= 0x8319C403; //Asura_GetHashID( "Alien_Confirm_Resolution");
	static const u_int s_uAVP_PROFILE_ALIEN_RESOLUTIONS_MENU		= 0x7C2C9FCF; //Asura_GetHashID( "Alien_Resolutions");
	static const u_int s_uAVP_PROFILE_ALIEN_GRAPHICS_OPT_MENU		= 0x1D5BF503; //Asura_GetHashID( "Alien_Graphics"); 
	static const u_int s_uAVP_PROFILE_PREDATOR_CONFIRM_RES_MENU		= 0x388989D3; //Asura_GetHashID( "Predator_Confirm_Resolution");
	static const u_int s_uAVP_PROFILE_PREDATOR_RESOLUTIONS_MENU		= 0xA044C3FF; //Asura_GetHashID( "Predator_Resolutions");
	static const u_int s_uAVP_PROFILE_PREDATOR_GRAPHICS_OPT_MENU	= 0xAAE1BED3; //Asura_GetHashID( "Predator_Graphics"); 
	static const u_int s_uAVP_PROFILE_OFF_CAPTION					= 0x0001AD6F; //Asura_GetHashID( "OFF"); 


	u_int m_uVisualQuality;
	u_int m_uTextureQuality;
	u_int m_uShadowQuality;
	u_int m_uResolutionIndex;
	bool  m_bTrilinearFiltering;
	float m_fGamma;
	u_int m_uAnisotropicFiltering;
	u_int m_uAntiAliasing;

    int   m_uNewResConfirmationTimer;
	u_int m_uOldResolutionIndex;

	static bool  Platform_SetResolution(u_int uResIndex);
	static void  Platform_UpdateAnisotropicFilteringDynLbl(u_int uAnisotropicFiltering);
	static void  Platform_ChangeAnisotropicFiltering(int iStep);

};



#endif // #ifndef __AVP_PROFILE_CONTROL__
