//////////////////////////////////////////////////////////////////////////////
//
// AVP_Profile_Sounds.cpp
//
// (AJK - 04.08.2008) Created
//
// Copyright (c) 2008 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"

// header
#include "Core/Profile/AVP_Profile_Sounds.h"

// Asura
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "GUIMenu/Asura_GUIMenu_System.h"
#include "Interface/Asura_CommandConsole.h"
#include "Sound/Asura_Sound_Mixer.h"

// AVP
#include "Core/AVP_Settings.h"

AVP_Profile_Sounds::AVP_Profile_Sounds()
{
	SetToDefaults();
}

void AVP_Profile_Sounds::SetToDefaults()
{
	m_fVolumeMaster = 0.75f;
	m_fVolumeMusic  = 0.5f;      
	m_fVolumeSFX    = 0.5f;      
	m_fVolumeSpeech = 0.5f;
#if defined(AVP_SUBMISSION)
	m_bSubtitles    = true;
#else
	m_bSubtitles    = true;
#endif	//defined(AVP_SUBMISSION)
	m_iSpeakerMode  = 0;
}

void AVP_Profile_Sounds::WriteToChunkStream(Asura_Chunk_Stream& xStream) 
{
	int iCurrentChunkVersion = CURRENT_CHUNK_VERSION;

	xStream << iCurrentChunkVersion;

	xStream << m_fVolumeMaster;
	xStream << m_fVolumeMusic;
	xStream << m_fVolumeSFX;
	xStream << m_fVolumeSpeech;
	xStream << m_bSubtitles;
	xStream << m_iSpeakerMode;
}

void AVP_Profile_Sounds::ReadFromChunkStream(Asura_Chunk_Stream& xStream)
{
	int iCurrentChunkVersion;

	xStream >> iCurrentChunkVersion;

	switch ( iCurrentChunkVersion )
	{
		case 2 :
		{
			xStream >> m_fVolumeMaster;
			xStream >> m_fVolumeMusic;
			xStream >> m_fVolumeSFX;
			xStream >> m_fVolumeSpeech;
			xStream >> m_bSubtitles;
			xStream >> m_iSpeakerMode;
			break;
		}
		case 1 :
		{
			xStream >> m_fVolumeMaster;
			xStream >> m_fVolumeMusic;
			xStream >> m_fVolumeSFX;
			xStream >> m_fVolumeSpeech;
			xStream >> m_bSubtitles;
			xStream >> m_iSpeakerMode;
			m_bSubtitles = true;
			break;
		}
		case 0 :
		{
			xStream >> m_fVolumeMusic;
			xStream >> m_fVolumeSFX;
			xStream >> m_fVolumeSpeech;
			xStream >> m_bSubtitles;
			m_bSubtitles = true;
			break;
		}
		default :
		{
			Asura_Assertf(false, ("Unknown chunk version (%d) in AVP_Profile_Sounds", iCurrentChunkVersion));
			break;
		}
	}
}

#if defined(ASURA_SUPPORT_COMMAND_CONSOLE)

void AVP_Profile_Sounds::SetupConsoleCommands()
{
	// Add the necessary console commands...
	Asura_CommandConsole::AddCmd( L"Profile.CMD.OnDeInitSoundsOptions", AVP_Profile_Sounds::ConsoleCmd_OnDeInitSoundsOptions          , Asura_CommandConsole::REMOVE_AT_SHUTDOWN, (1 << 2));
}

void AVP_Profile_Sounds::SetupConsoleVariables()
{
	// Add the necessary console variables...
	Asura_CommandConsole::AddVar( L"Sound.VAR.Subtitles",		&m_bSubtitles,					Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::AddVar( L"Sound.VAR.SpeakerMode",		&m_iSpeakerMode,	0,    3,	Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::AddVar( L"Sound.VAR.VolumeMaster",	&m_fVolumeMaster,	0.0f, 1.0f,	Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::AddVar( L"Sound.VAR.VolumeMusic",		&m_fVolumeMusic,	0.0f, 1.0f,	Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::AddVar( L"Sound.VAR.VolumeSFX",		&m_fVolumeSFX,		0.0f, 1.0f,	Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::AddVar( L"Sound.VAR.VolumeSpeech",	&m_fVolumeSpeech,	0.0f, 1.0f,	Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
}

void AVP_Profile_Sounds::RemoveConsoleVariables()
{
	// Remove the necessary console variables...
	Asura_CommandConsole::RemoveVar( L"Sound.VAR.Subtitles" );
	Asura_CommandConsole::RemoveVar( L"Sound.VAR.SpeakerMode" );
	Asura_CommandConsole::RemoveVar( L"Sound.VAR.VolumeMaster" );
	Asura_CommandConsole::RemoveVar( L"Sound.VAR.VolumeMusic" );
	Asura_CommandConsole::RemoveVar( L"Sound.VAR.VolumeSFX" );
	Asura_CommandConsole::RemoveVar( L"Sound.VAR.VolumeSpeech" );
}

#endif	// defined(ASURA_SUPPORT_COMMAND_CONSOLE)

void AVP_Profile_Sounds::UpdateVolumeSFX()
{
	Asura_Sound_Mixer_Channel* pxChannel = Asura_Sound_Mixer_System::FindChannel(s_uAVP_SOUNDS_SFX);
	if ( pxChannel )
	{
		// the volume to set is multiplied here by the master volume (ranged from 0.0 to 1.0) - so if SFXvol is at 80% and master is 50% SFXvol will be set to 40%
		float fMasterEqualedVol = AVP_Settings::GetCurrentProfile()->GetSounds().m_fVolumeSFX * AVP_Settings::GetCurrentProfile()->GetSounds().m_fVolumeMaster;
		pxChannel->SetVolume( fMasterEqualedVol );
	}
	else 
	{
		Asura_Assert(false, "No sfx channel");
	}
}

void AVP_Profile_Sounds::UpdateVolumeMusic()
{
	Asura_Sound_Mixer_Channel* pxChannel = Asura_Sound_Mixer_System::FindChannel(s_uAVP_SOUNDS_MUSIC);
	if ( pxChannel )
	{
		// the volume to set is multiplied here by the master volume (ranged from 0.0 to 1.0) - so if MUSICvol is at 80% and master is 50% MUSICvol will be set to 40%
		float fMasterEqualedVol = AVP_Settings::GetCurrentProfile()->GetSounds().m_fVolumeMusic * AVP_Settings::GetCurrentProfile()->GetSounds().m_fVolumeMaster;
		pxChannel->SetVolume( fMasterEqualedVol );
	}
	else 
	{
		Asura_Assert(false, "No music channel");
	}
}

void AVP_Profile_Sounds::UpdateVolumeSpeech()
{
	Asura_Sound_Mixer_Channel* pxChannel = Asura_Sound_Mixer_System::FindChannel(s_uAVP_SOUNDS_SPEECH);
	if ( pxChannel )
	{
		// the volume to set is multiplied here by the master volume (ranged from 0.0 to 1.0) - so if SPEECHvol is at 80% and master is 50% SPEECHvol will be set to 40%
		float fMasterEqualedVol = AVP_Settings::GetCurrentProfile()->GetSounds().m_fVolumeSpeech * AVP_Settings::GetCurrentProfile()->GetSounds().m_fVolumeMaster;
		pxChannel->SetVolume( fMasterEqualedVol );
	}
	else 
	{
		Asura_Assert(false, "No speech channel");
	}
}

void AVP_Profile_Sounds::Update()
{
	// if any of the pause menu sound options OR the frontend sound options are open
	if( ( Asura_GUIMenu_System::IsMenuPageHashActive( GetCurSpeciesSoundsMenuHash() ) ) || ( Asura_GUIMenu_System::IsMenuPageHashActive( s_uAVP_PROFILE_FRONTEND_SOUNDS_MENU ) ) )
	{
		ApplySettings();
	}
}

void AVP_Profile_Sounds::ApplySettings()
{
	UpdateVolumeSFX();
	UpdateVolumeMusic();
	UpdateVolumeSpeech();
}

u_int AVP_Profile_Sounds::GetCurSpeciesSoundsMenuHash()
{
	switch(AVP_Settings::GetPlayerSpecies())
	{
		case AVP_Settings::AVP_SPECIES_HUMAN :
		{
			return s_uAVP_PROFILE_MARINE_SOUNDS_MENU;
		}
		case AVP_Settings::AVP_SPECIES_ALIEN :
		{
			return s_uAVP_PROFILE_ALIEN_SOUNDS_MENU;
		}
		case AVP_Settings::AVP_SPECIES_PREDATOR :
		{
			return s_uAVP_PROFILE_PREDATOR_SOUNDS_MENU;
		}
		case AVP_Settings::AVP_SPECIES_UNKNOWN:
		{
			// This is valid, we don't know what species we are yet
			return ASURA_HASH_ID_UNSET; 
		}
		default :
		{
			Asura_Assert(false,"Wrong species passed in AVP_Profile_Sounds");
			return ASURA_HASH_ID_UNSET;
		}
	}
}

bool AVP_Profile_Sounds::ConsoleCmd_OnDeInitSoundsOptions( bool bHelp)
{
	return true;
}

bool AVP_Profile_Sounds::operator!=(const AVP_Profile_Sounds& xOther) const
{
	return 	 !(*this == xOther) ;
}

bool AVP_Profile_Sounds::operator==(const AVP_Profile_Sounds& xOther) const
{
	return 	(m_fVolumeMaster == xOther.m_fVolumeMaster ) &&
			(m_fVolumeSFX    == xOther.m_fVolumeSFX    ) &&
		    (m_fVolumeMusic  == xOther.m_fVolumeMusic  ) &&
		    (m_fVolumeSpeech == xOther.m_fVolumeSpeech ) &&
		    (m_bSubtitles    == xOther.m_bSubtitles    ) &&
			(m_iSpeakerMode  == xOther.m_iSpeakerMode  ) ;
}

void AVP_Profile_Sounds::SetSpeakerMode( int iSpeakerMode )
{
	if( ( iSpeakerMode >= 0 ) && ( iSpeakerMode < m_iNumSpeakerModes ) )
	{
		m_iSpeakerMode = iSpeakerMode;
	}
	else
	{
		Asura_Assert( 1 , "Speaker Mode Requested is not Valid" );
	}
}