
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_State.h
// Author : Andy Tate
// Date   : 12-Mar-2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __AVP_STATE_H__
#define __AVP_STATE_H__

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Forward class declarations...
////////////////////////////////////////////////////////////////////////////////

class AVP_StateMachine;


////////////////////////////////////////////////////////////////////////////////
// Class declaration...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_State
// Author       : Andy Tate
// Date         : 12-Mar-2007
//
// Description  : Abstract class used to handle the execution of a particular
//				  state.
////////////////////////////////////////////////////////////////////////////////

class AVP_State
{
public:
	// Public functions...
	AVP_State(AVP_StateMachine& xParentMachine)
		: m_xParentMachine(xParentMachine)
	{
	}
	virtual ~AVP_State() {}

	// Virtual functions...
	virtual	void Enter(const u_int uPreviousState)	{}
	virtual	void Update()							{}
	virtual	void Render() const						{}
	virtual	void Exit(const u_int uNextState)		{}

	// Abstract functions...
	virtual	const char*	GetName(void) const	= 0;

	// Accessors
	AVP_StateMachine& GetParentMachine() { return m_xParentMachine; }
	const AVP_StateMachine& GetParentMachine() const { return m_xParentMachine; }

protected:
	// Protected functions...

	// Protected variables...
	AVP_StateMachine& m_xParentMachine;

private:
	// Private variables...

	// Private functions...
	AVP_State(const AVP_State& xOther);

	AVP_State&	operator=	(const AVP_State& xOther);

};   // End of class 'AVP_State'.


#endif   // ifndef __AVP_STATE_H__


/// End of file. ///
