
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_ConnectionState_Loading.h
// Author : Anthony Gregson
// Date   : 22/11/07
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __AVP_CONNECTIONSTATE_LOADING_H__
#define __AVP_CONNECTIONSTATE_LOADING_H__

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura_Timers.h"

#include "Core/States/Connection/AVP_State_Connection.h"


////////////////////////////////////////////////////////////////////////////////
// Forward class declarations...
////////////////////////////////////////////////////////////////////////////////

class AVP_Connection;

////////////////////////////////////////////////////////////////////////////////
// Class declaration...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Class  : AVP_ConnectionState_Loading
// Author : Anthony Gregson
// Date   : 22/11/07
////////////////////////////////////////////////////////////////////////////////

class AVP_ConnectionState_Loading : public AVP_State_Connection
{
	typedef AVP_State_Connection PARENT;

public:
	// Public functions...
	AVP_ConnectionState_Loading(AVP_Connection& xParentMachine)
		: PARENT(xParentMachine)
	{
	}
//	virtual ~AVP_ConnectionState_Loading();

	// Overridden functions...
	virtual	void Enter(const u_int uPreviousState);
	virtual	void Update();
	virtual	void Render() const;
	virtual	void Exit(const u_int uNextState);

	virtual	const char*	GetName(void) const	{ return("Loading"); }

protected:
	// Protected variables...
	static volatile bool s_bLoadCompleted;	// Flag to say that we've finished loading.

	// Protected functions...
	static void PerformLoad(const void* pUserData);

private:
	// Private declarations
	struct LoadingData
	{
		const char*	m_szFilename; 
		bool		m_bOnlyLoadLevelFile; 
	};

	// Private variables...
	LoadingData		m_xLoadingData;	// The filename to load.
	Asura_TickCount	m_xStartTick;	// The tick-count when loading starts.

	// Private functions...
	static void LoadCharacterFiles();
	static bool LoadWeaponFiles();
	static void	LoadLevel(const char* szLevelFile);
	static void AddLevelBlueprintWeaponsToLoadList( const char* szLevelFile );
};   // End of class 'AVP_ConnectionState_Loading'.


#endif   // ifndef __AVP_CONNECTIONSTATE_LOADING_H__


/// End of file. ///
