
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_ConnectionState_Playing.cpp
// Author : Anthony Gregson
// Date   : 22/11/07
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "Core/States/Connection/AVP_ConnectionState_Playing.h"

// Asura independent header-files...
#include "Botanicals/Asura_Botanical_Game_Render_Manager.h"
#include "Core/Asura_UserInput.h"
#include "Entity/Asura_Entity_System.h"
#include "SpecialFX/Asura_FullScreenSFX_System.h"
#include "GUIMenu/Asura_GUIMenu_System.h"
#if defined(ASURA_SUPPORT_DYNAMIC_MUSIC_SYSTEM)
#include "Sound/Asura_Dynamic_Music_System.h"
#endif	//defined(ASURA_SUPPORT_DYNAMIC_MUSIC_SYSTEM)

// Game independent header-files...
#include "Core/AVP_Connection.h"
#include "Core/AVP_Core.h"
#include "Core/AVP_Game_Core.h"
#include "Core/AVP_GameStatus.h"
#ifdef AVP_LOG_FILE_LOAD_MEMORY_USE
#include "Core/AVP_SmokeTest.h"
#endif // #ifdef AVP_LOG_FILE_LOAD_MEMORY_USE
#include "Core/AVP_Settings.h"
#include "GUIMenu/AVP_GUIMenu_System.h"

// Asura independent header-files...
#include "Network/Asura_Network.h"

// Game independent header-files...
#include "Network/AVP_Server.h"


////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Private functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Protected functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Public functions...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Function	: AVP_ConnectionState_Playing::Enter
// Author	: Anthony Gregson
// Date		: 03/12/07
////////////////////////////////////////////////////////////////////////////////

void AVP_ConnectionState_Playing::Enter(const u_int uPreviousState)
{
	Asura_FullScreenSFX_System::CancelFade();

	m_xPlayingMachine.Initialise();

#if !defined(ASURA_TOOLS)
	Asura_Timers::SetMaxFrameRate(AVP_Settings::GetMaxInGameFrameRate());
#endif	//!defined(ASURA_TOOLS)

#ifdef AVP_LOG_FILE_LOAD_MEMORY_USE
	AVP_SmokeTest::SetPlaying( true );
#endif //#ifdef AVP_LOG_FILE_LOAD_MEMORY_USE
}

////////////////////////////////////////////////////////////////////////////////
// Function	: AVP_ConnectionState_Playing::Update
// Author	: Anthony Gregson
// Date		: 22/11/07
////////////////////////////////////////////////////////////////////////////////

void AVP_ConnectionState_Playing::Update()
{
	if (!GetParentConnection().ValidateNetworkConnection())
	{
		// If the above failed it will have called for a state change, so don't do any further processing here.
		return;
	}

	if(Asura_Network::IsServer())
	{
		AVP_Server::UpdateClientInitialisation();
	}

	AVP_Game_Core::Update();

	// Not sure at this time whether the sub-state machine needs to update before or after the core. I suspect after is correct though, as it wants to check how the game conditions have progressed.
	m_xPlayingMachine.Update();

#if defined(ASURA_TOOLS)
	if (AVP_Core::GetToolsRequested() || Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_F11))
	{
		Asura_GUIMenu_System::RestoreResizedSettings();
		AVP_Core::SetToolsRequested(true);
		GetParentConnection().Terminate(ASURA_NDR_SERVERQUIT);
	}
	else
#endif	// defined(ASURA_TOOLS)

	if (AVP_GameStatus::GetRestartLevel() || AVP_GameStatus::GetRestoreCheckpoint())
	{
		GetParentMachine().ChangeState(AVP_Connection::STATE_BEGINGAMEINITIALISATION);
	}
	else if (AVP_GameStatus::GetExitGame())
	{
		GetParentConnection().Terminate((AVP_Settings::IsSinglePlayer() || AVP_Settings::GetIsServer()) ? ASURA_NDR_SERVERQUIT : ASURA_NDR_UNKNOWN);
	}
	else if (AVP_GameStatus::GetLoadMap())
	{
		GetParentConnection().ChangeState(AVP_Connection::STATE_MAPCHANGE);
	}
}

/// End of function 'AVP_ConnectionState_Playing::Update'. ///



////////////////////////////////////////////////////////////////////////////////
// Function	: AVP_ConnectionState_Playing::Render
// Author	: Anthony Gregson
// Date		: 22/11/07
////////////////////////////////////////////////////////////////////////////////

void AVP_ConnectionState_Playing::Render() const
{
	AVP_Core::BeginRender(true);
	{
		AVP_Game_Core::Render();
	}
	AVP_Core::EndRender();

	m_xPlayingMachine.Render();
}

/// End of function 'AVP_ConnectionState_Playing::Render'. ///



////////////////////////////////////////////////////////////////////////////////
// Function	: AVP_ConnectionState_Playing::Exit
// Author	: Anthony Gregson
// Date		: 22/11/07
////////////////////////////////////////////////////////////////////////////////

void AVP_ConnectionState_Playing::Exit(const u_int uNextState)
{
#if !defined(ASURA_TOOLS)
	Asura_Timers::SetMaxFrameRate(AVP_Settings::GetMaxFrameRate());
#endif	//!defined(ASURA_TOOLS)
#if defined(ASURA_SUPPORT_DYNAMIC_MUSIC_SYSTEM)
	Asura_Dynamic_Music_System::Stop();
#endif	//defined(ASURA_SUPPORT_DYNAMIC_MUSIC_SYSTEM)
#ifdef AVP_LOG_FILE_LOAD_MEMORY_USE
	AVP_SmokeTest::SetPlaying( false );
#endif //#ifdef AVP_LOG_FILE_LOAD_MEMORY_USE

	// CW - Is this enough? What if we are in the options menu?
	// Ensure we ditch the pause menu
	if( AVP_GUIMenu_System::IsCurrentlyInNonHudOrLoadingMenu() )
	{
		AVP_GUIMenu_System::Deactivate( AVP_GUIMenu_System::GUIMENU_ID_PAUSE );
	}

	// Get out of our current sub-state.
	m_xPlayingMachine.Shutdown();
}

/// End of function 'AVP_ConnectionState_Playing::Exit'. ///


/// End of file. ///
