
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_ConnectionState_WaitForGuid.cpp
// Author : Anthony Gregson
// Date   : 22/11/07
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "Core/States/Connection/AVP_ConnectionState_WaitForGuid.h"

// Asura independent header-files...
#include "Debugging/Asura_DebugText.h"
#include "Network/Asura_Network.h"

// Game independent header-files...
#include "Core/AVP_Connection.h"
#include "Core/AVP_Core.h"
#include "GUIMenu/AVP_GUIMenu_System.h"


////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////

DEBUGVAR float	dbg_fConnectionTimeoutDuration	= 30.0f;


////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Private functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Protected functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Public functions...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Function	: AVP_ConnectionState_WaitForGuid::Update
// Author	: Anthony Gregson
// Date		: 22/11/07
////////////////////////////////////////////////////////////////////////////////

void AVP_ConnectionState_WaitForGuid::Update()
{
	if (!GetParentConnection().ValidateNetworkConnection())
	{
		// If the above failed it will have called for a state change, so don't do any further processing here.
		return;
	}

	// TODO - allow user to quit, possibly through some new gui menu (at the moment we'll still be on the loading screen as nothing else has been set up)
	Asura_Network::Update();
	Asura_Network::FlushBufferedMessages();
	if (Asura_Network::IsWaitingForClientGuid())
	{
		const float	fTimeInState = GetParentMachine().GetTimeInCurrentState();

		if (fTimeInState > dbg_fConnectionTimeoutDuration)
		{
			GetParentConnection().Terminate(ASURA_NDR_TIMEOUT);
		}
	}
	else
	{
		GetParentMachine().ChangeState(AVP_Connection::STATE_BEGINGAMEINITIALISATION);
	}
}

/// End of function 'AVP_ConnectionState_WaitForGuid::Update'. ///


////////////////////////////////////////////////////////////////////////////////
// Function	: AVP_ConnectionState_WaitForGuid::Render
// Author	: Anthony Gregson
// Date		: 22/11/07
////////////////////////////////////////////////////////////////////////////////

void AVP_ConnectionState_WaitForGuid::Render() const
{
#if !defined(ASURA_TOOLS)
	AVP_Core::BeginRender(false);
	{
		AVP_GUIMenu_System::Render(true);
#if defined(ASURA_USE_DEBUG_TEXT)
		Asura_DebugText::AddTextBlock(50, 20, 0xFFFFFFFF, 0, "Waiting: %.02f secs", GetParentMachine().GetTimeInCurrentState());
#endif	// defined(ASURA_USE_DEBUG_TEXT)
	}
	AVP_Core::EndRender();
#endif	// defined(ASURA_TOOLS)
}

/// End of function 'AVP_ConnectionState_WaitForGuid::Render'. ///


/// End of file. ///
