
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_ConnectionState_WaitForPlayers.cpp
// Author : Paul Cardy
// Date   : 25/03/09
//
// Copyright (C) 2009 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "Core/States/Connection/AVP_ConnectionState_WaitForPlayers.h"

// Asura
#include "Debugging/Asura_DebugText.h"
#ifdef ASURA_SUPPORT_LOBBY
#include "Network/Online/Asura_Lobby.h"
#endif

// AVP
#include "Core/AVP_Connection.h"
#include "Core/AVP_Core.h"
#if defined(ASURA_USE_GUI_MENU_SYSTEM)
#include "GUIMenu/AVP_GUIMenu_System.h"
#endif

// Tools header-files...

////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////

DEBUGVAR float	dbg_fConnectionTimeoutDuration	= 30.0f;

////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Private functions...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Protected functions...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Public functions...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Function	: AVP_ConnectionState_WaitForPlayers::Enter
// Author	: Paul Cardy
// Date		: 25/03/09
////////////////////////////////////////////////////////////////////////////////

void AVP_ConnectionState_WaitForPlayers::Enter(const u_int uPreviousState)
{
	if (uPreviousState == AVP_Connection::STATE_INVALID)
	{
		AVP_Core::InitialiseNetwork(AVP_Settings::GetExternalNetworkActive());
	}

	m_xStartTick = Asura_Timers::GetCurTicks();
}

/// End of function 'AVP_ConnectionState_WaitForPlayers::Enter'. ///



////////////////////////////////////////////////////////////////////////////////
// Function	: AVP_ConnectionState_WaitForPlayers::Update
// Author	: Paul Cardy
// Date		: 25/03/09
////////////////////////////////////////////////////////////////////////////////

void AVP_ConnectionState_WaitForPlayers::Update()
{
	Asura_Network::Update();

#ifdef ASURA_SUPPORT_LOBBY
	if (Asura_Lobby_System::IsInitialised() && (Asura_Lobby_System::GetErrorCode() != ASURA_RS_DISCONNECT_NOT_DISCONNECTED))
	{
		GetParentMachine().ChangeState(AVP_Connection::STATE_TERMINATED);
	}
	else if (Asura_Lobby_System::IsInitialised() && !Asura_Lobby_System::IsReadyToLoadGame() )
	{
		const float	fTimeInState = GetParentMachine().GetTimeInCurrentState();

		if (fTimeInState > dbg_fConnectionTimeoutDuration)
		{
			GetParentConnection().Terminate(ASURA_NDR_TIMEOUT);
		}
	}
	else
#endif
	{
		GetParentMachine().ChangeState(AVP_Connection::STATE_LOADING);
	}
}

/// End of function 'AVP_ConnectionState_WaitForPlayers::Update'. ///



////////////////////////////////////////////////////////////////////////////////
// Function	: AVP_ConnectionState_WaitForPlayers::Render
// Author	: Paul Cardy
// Date		: 25/03/09
////////////////////////////////////////////////////////////////////////////////

void AVP_ConnectionState_WaitForPlayers::Render() const
{
#if !defined(ASURA_TOOLS)
	AVP_Core::BeginRender(false);
	{
		AVP_GUIMenu_System::Render(true);

#if defined(ASURA_USE_DEBUG_TEXT)
		{
			const Asura_TickCount	xElapsedTicks	= Asura_Timers::GetElapsedTicks(m_xStartTick);
			const float				fElapsedSecs	= Asura_Timers::TicksToSeconds(xElapsedTicks);

			Asura_DebugText::AddTextBlock(50, 20, 0xFFFFFFFF, 0, "WaitingForPlayers: %.02f secs", fElapsedSecs);
		}
#endif	// defined(ASURA_USE_DEBUG_TEXT)
	}
	AVP_Core::EndRender();
#endif	// defined(ASURA_TOOLS)
}

/// End of function 'AVP_ConnectionState_WaitForPlayers::Render'. ///



////////////////////////////////////////////////////////////////////////////////
// Function	: AVP_ConnectionState_WaitForPlayers::Exit
// Author	: Paul Cardy
// Date		: 25/03/09
////////////////////////////////////////////////////////////////////////////////

void AVP_ConnectionState_WaitForPlayers::Exit(const u_int uNextState)
{
}

/// End of function 'AVP_ConnectionState_WaitForPlayers::Exit'. ///

/// End of file. ///
