
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_CoreState_DevelopmentFrontend.cpp
// Author : Anthony Gregson
// Date   : 21/11/07
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

#if defined(AVP_USE_DEVELOPMENT_FRONT_END)

// This file's header...
#include "Core/States/Core/AVP_CoreState_DevelopmentFrontend.h"

// Asura independent header-files...
#include "Core/Asura_Core.h"
#include "Core/Asura_MemoryManagement_Enums.h"
#include "ChunkSystem/Asura_Chunk_Loaders.h"

// Game independent header-files...
#include "Core/AVP_Core.h"
#include "Core/AVP_GameStatus.h"
#include "Core/AVP_StateMachine_Core.h"
#include "Core/States/Core/AVP_CoreState_LoadingFrontend.h"
#include "FrontEnd/AVP_DevelopmentFrontEnd.h"

#if defined(ASURA_TOOLS)
// Tools header-files...
#include "Tools_System.h"
#endif	// defined(ASURA_TOOLS)


////////////////////////////////////////////////////////////////////////////////
// Function	: AVP_CoreState_DevelopmentFrontend::Enter
// Author	: Anthony Gregson
// Date		: 21/11/07
////////////////////////////////////////////////////////////////////////////////

void AVP_CoreState_DevelopmentFrontend::Enter(const u_int uPreviousState)
{
	// Ensure we've cleared down any unwanted data...
#if !defined(ASURA_TOOLS)
	AVP_Core::Reset();
	Asura_MemoryManagement_System::Checkpoint(ASURA_MM_STATE_PRE_DEVFRONTEND);
#endif
	AVP_DevelopmentFrontEnd::Initialise();
}

/// End of function 'AVP_CoreState_DevelopmentFrontend::Enter'. ///



////////////////////////////////////////////////////////////////////////////////
// Function	: AVP_CoreState_DevelopmentFrontend::Update
// Author	: Anthony Gregson
// Date		: 21/11/07
////////////////////////////////////////////////////////////////////////////////

void AVP_CoreState_DevelopmentFrontend::Update()
{
	if(AVP_DevelopmentFrontEnd::Update())
	{
		if(AVP_DevelopmentFrontEnd::IsQuitRequested())
		{
			GetParentMachine().ChangeState(AVP_StateMachine_Core::STATE_SHUTTINGDOWN);
		}
#if defined(AVP_USE_DEV_LOBBY)
		else if ( AVP_DevelopmentFrontEnd::IsDevLobbyRequested() )
		{
			AVP_CoreState_LoadingFrontend::SetStatePostLoad( AVP_CoreState_LoadingFrontend::STATE_POSTLOAD_MPLOBBY );
			GetParentMachine().ChangeState(AVP_StateMachine_Core::STATE_GAMEFRONTEND);
		}
#endif	//defined(AVP_USE_DEV_LOBBY)
#if defined(AVP_USE_GAME_LOBBY)
		else if ( AVP_DevelopmentFrontEnd::IsGameLobbyRequested() )
		{
			AVP_CoreState_LoadingFrontend::SetStatePostLoad( AVP_CoreState_LoadingFrontend::STATE_POSTLOAD_MPLOBBY );
			GetParentMachine().ChangeState(AVP_StateMachine_Core::STATE_GAMEFRONTEND);
		}
#endif	//defined(AVP_USE_GAME_LOBBY)
#if defined(AVP_USE_GAME_FRONTEND)
		else if( AVP_DevelopmentFrontEnd::IsFrontEndRequested() )
		{
			GetParentMachine().ChangeState(AVP_StateMachine_Core::STATE_FOX_FMV);
		}
#endif //defined(AVP_USE_GAME_FRONTEND)
#if defined(ASURA_TOOLS)
		//If doing a single zone then go via tools. RAR 05/10/2006
		else if(AVP_DevelopmentFrontEnd::GetGameMode() == AVP_DFE_GAMEMODE_LOAD_SINGLE_ZONE)
		{
			Tools_System::RequestAutoQuitToGame();
			AVP_Core::SetToolsRequested(true);
			GetParentMachine().ChangeState(AVP_StateMachine_Core::STATE_LOADINGTOOLS);
		}
#endif	// defined(ASURA_TOOLS)
		else if (!AVP_GameStatus::GetQuitApplication())
		{
			GetParentMachine().ChangeState(AVP_StateMachine_Core::STATE_GAME);
		}

	}
}

/// End of function 'AVP_CoreState_DevelopmentFrontend::Update'. ///



////////////////////////////////////////////////////////////////////////////////
// Function	: AVP_CoreState_DevelopmentFrontend::Render
// Author	: Anthony Gregson
// Date		: 21/11/07
////////////////////////////////////////////////////////////////////////////////

void AVP_CoreState_DevelopmentFrontend::Render() const
{
	AVP_Core::BeginRender( false );
	{
		AVP_DevelopmentFrontEnd::Render();
	}
	AVP_Core::EndRender();
}

/// End of function 'AVP_CoreState_DevelopmentFrontend::Render'. ///



////////////////////////////////////////////////////////////////////////////////
// Function	: AVP_CoreState_DevelopmentFrontend::Exit
// Author	: Anthony Gregson
// Date		: 21/11/07
////////////////////////////////////////////////////////////////////////////////

void AVP_CoreState_DevelopmentFrontend::Exit(const u_int uNextState)
{
	AVP_DevelopmentFrontEnd::Shutdown();
}

/// End of function 'AVP_CoreState_DevelopmentFrontend::Exit'. ///

#endif	// defined(AVP_USE_DEVELOPMENT_FRONT_END)


/// End of file. ///
