
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_CoreState_Void.cpp
// Author : Anthony Gregson
// Date   : 21/11/07
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "Core/States/Core/AVP_CoreState_Void.h"

// Engine independent header-files...
#include "ChunkSystem/Asura_Chunk_Loaders.h"
#include "Core/Asura_TextureManagement.h"
#include "Core/Resources/Asura_Resource_System.h"
#include "GUIMenu/Asura_GUIMenu_System.h"
#include "Sound/Asura_Sound.h"
#include "Sound/Asura_Audio_Systems_Manager.h"

// Game independent header-files...
#include "Core/AVP_Core.h"
#include "Core/AVP_StateMachine_Core.h"
#include "Multiplayer/AVP_Multiplayer_GameRules.h"


////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Private functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Protected functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Public functions...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_CoreState_Void::Update
// Author       : Andy Tate
// Date         : 12-Mar-2007
////////////////////////////////////////////////////////////////////////////////

void AVP_CoreState_Void::Update()
{
#ifdef ASURA_TOOLS
	if(AVP_Core::GetToolsRequested())
	{
		///// Load GUI Menu Sounds
		Asura_GUIMenu_System::LoadGenericAudioData();

		// Start Tools...
		GetParentMachine().ChangeState(AVP_StateMachine_Core::STATE_LOADINGTOOLS);
	}
	else
#endif	//#ifdef ASURA_TOOLS
	{

		///// Load GUI Menu Sounds
		Asura_GUIMenu_System::LoadGenericAudioData();

#ifdef AVP_USE_DEVELOPMENT_FRONT_END
		//Can still get into game front-end from dev front-end if both are defined.
		GetParentMachine().ChangeState(AVP_StateMachine_Core::STATE_DEVELOPMENTFRONTEND);

#elif defined(AVP_USE_GAME_FRONTEND)

		//Only game front-end is defined, one assumes this is the way it'll eventually be.
		GetParentMachine().ChangeState(AVP_StateMachine_Core::STATE_FOX_FMV);

#else //defined(AVP_USE_GAME_FRONTEND)

		//Neither front-end is defined, go straight to game (don't ask me how!).
		GetParentMachine().ChangeState(AVP_StateMachine_Core::STATE_GAME);

#endif //defined(AVP_USE_GAME_FRONTEND)

	}

	// Ensure that we're now able to reset without binning any existing audio...
#ifndef ASURA_USE_AUDIO_SYSTEM
	Asura_Sound_System::SetResetPoint();
#endif
	
	AVP_Multiplayer_GameRules::Initialise(); 
	AVP_Multiplayer_GameRules::AddCommandConsoleVars();
}

/// End of function 'AVP_CoreState_Void::Update'. ///


/// End of file. ///
