
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_PlayingState_InProgress.cpp
// Author : Anthony Gregson
// Date   : 03/12/07
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "Core/States/Playing/AVP_PlayingState_InProgress.h"

// Asura independent header-files...
#include "Core/Asura_Timers.h"

// Game independent header-files...
#include "Core/AVP_Core.h"
#include "Core/AVP_Settings.h"
#include "Core/AVP_GameStatus.h"
#include "Core/AVP_StateMachine_Playing.h"
#include "Multiplayer/AVP_Multiplayer_GameCore.h"


////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Private functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Protected functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Public functions...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Function	: AVP_PlayingState_InProgress::Enter
// Author	: Anthony Gregson
// Date		: 03/12/07
////////////////////////////////////////////////////////////////////////////////

void AVP_PlayingState_InProgress::Enter(const u_int uPreviousState)
{
	// TODO - This is intended for setting up initial set of of the game entities and game situation relating to the chosen game rules.
	//		Most, if not all, of this is only relevant to the server.

	// Let the multiplayer system know that a new game is in progress
	if (AVP_Settings::IsMultiPlayer())
	{
		AVP_Multiplayer_GameCore::BeginMultiplayerMatch( Asura_Timers::GetServerAbsoluteTime() ); 
	}
}

/// End of function 'AVP_PlayingState_InProgress::Enter'. ///



////////////////////////////////////////////////////////////////////////////////
// Function	: AVP_PlayingState_InProgress::Update
// Author	: Anthony Gregson
// Date		: 03/12/07
////////////////////////////////////////////////////////////////////////////////

void AVP_PlayingState_InProgress::Update()
{
	// TODO - This is intended for updating any spawning and scoring rules dictated by the game rules and the current game situation.
	//		Most, if not all, of this is only relevant to the server.

	// Game over works fine in multiplayer games
	if (AVP_Settings::IsMultiPlayer())
	{
		if (AVP_Multiplayer_GameCore::ShouldEndMultiplayerMatch())
		{
			AVP_Core::RequestGameOver(); 
		}
	}

	if (AVP_GameStatus::GetLevelComplete() || AVP_GameStatus::GetGameOver())
	{
		GetParentMachine().ChangeState(AVP_StateMachine_Playing::STATE_GAMEOVER);
	}
}

/// End of function 'AVP_PlayingState_InProgress::Update'. ///



////////////////////////////////////////////////////////////////////////////////
// Function	: AVP_PlayingState_InProgress::Render
// Author	: Anthony Gregson
// Date		: 03/12/07
////////////////////////////////////////////////////////////////////////////////

void AVP_PlayingState_InProgress::Render() const
{
	// TODO - not sure if there's anything to this, but any additional rendering because of game situation can go here if it's not handled by the normal HUD code.
}

/// End of function 'AVP_PlayingState_InProgress::Render'. ///


////////////////////////////////////////////////////////////////////////////////
//  CLASS    :  AVP_PlayingState_InProgress
//  NAME     :  Exit()
//  CREATED  :  JG (3.7.2008)
////////////////////////////////////////////////////////////////////////////////
void AVP_PlayingState_InProgress::Exit(const u_int uNextState)
{
	// If this is multiplayer match, it is finished now
	if (AVP_Settings::IsMultiPlayer())
	{
		AVP_Multiplayer_GameCore::EndMultiplayerMatch(); 
	}
}


/// End of file. ///
