//////////////////////////////////////////////////////////////////////////////
//
// AVP_Cutscene_Event_CloakActor.cpp
//
// (Ali Jakamy - 01.06.2008) Created
//
// Copyright (c) 2008 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"

// header
#include "AVP_Cutscene_Event_CloakActor.h"

// Asura
#include "Animation/Asura_Animation.h"
#include "Cutscene/Asura_Cutscene.h"
#include "Cutscene/Asura_Cutscene_Actor.h"


// AVP
#include "Cutscene/AVP_Cutscene_Event.h"
#include "Network/AVP_Player.h"

// Tools
#ifdef ASURA_TOOLS
	#include "Tools_System.h"
	#include "ToolsAVP_System.h"
#endif // ASURA_TOOLS

AVP_Cutscene_Event_CloakActor::AVP_Cutscene_Event_CloakActor(Asura_Cutscene* pxCutscene)
: PARENT(AVP_CET_CloakActor, pxCutscene)
, m_fDuration(0.0f)
, m_fCloakingDuration(1.5f)
, m_fUncloakingDuration(1.5f)
, m_fLightningScale(0.06f)
{
}

void AVP_Cutscene_Event_CloakActor::Finish()
{
	if(m_pxTargetActor)
	{	
		if(m_pxTargetActor->IsHierarchyActor())
		{
            Asura_Cutscene_Actor_Hierarchy* pxActor = static_cast<Asura_Cutscene_Actor_Hierarchy*> (m_pxTargetActor);
			Asura_Animation* pxAnim = pxActor->GetAnimation();
			if( pxAnim )
			{
				m_xAnimProjectData.SetCloakEffect(NULL);
				m_xAVPCloakEffect.SetTransitionTimeAndState( 0.0f , AVP_CloakEffect::UNCLOAKED, *pxAnim );
			}
			else
			{
				Asura_Assert(false, "Actor has no anmiation");
			}
		}
		else
		{
			Asura_Assert(false, "Not a hierarchy actor");
		}
	}
	else
	{
		Asura_Assert(false, "No actor!");
	}
}

float AVP_Cutscene_Event_CloakActor::GetTransitionTime(float fStart, float fCurrent, float fEnd, bool bCloaking)
{
	// When cloaking
	// The length of Transition time is 1 (from 0.0 to 1.0)
	// The length of event cloaking is fEnd - fStart
	// To get the current Transition time we multiply (fCurrent - fStart) * ((Full cloak - Full unCloak)/(fEnd - fStart))
    // wich is (fCurrent - fStart) * (1 / (fEnd - fStart))
	// When uncloaking 1 - ((fCurrent - fStart) * (1 / (fEnd - fStart)))

	float fTmpTransition = ( fCurrent - fStart ) / ( fEnd - fStart ) ;
	
	return ( bCloaking ? fTmpTransition : 1.0f - fTmpTransition );
}

void AVP_Cutscene_Event_CloakActor::UpdateAfterTrigger(Asura_Cutscene* pxCutscene)
{
	UpdateCloakingTimerAndState(pxCutscene);
}

void AVP_Cutscene_Event_CloakActor::UpdateCloakingTimerAndState(Asura_Cutscene* pxCutscene)
{
	if(m_pxTargetActor)
	{	
		if(m_pxTargetActor->IsHierarchyActor())
		{
            Asura_Cutscene_Actor_Hierarchy* pxActor = static_cast<Asura_Cutscene_Actor_Hierarchy*> (m_pxTargetActor);
			Asura_Animation* pxAnim = pxActor->GetAnimation();
			if( pxAnim )
			{
				// On the cloaking path
				if( pxCutscene->Timer < m_fTriggerTime + m_fCloakingDuration )
				{
					float fTransTime = GetTransitionTime( m_fTriggerTime, pxCutscene->Timer, m_fTriggerTime + m_fCloakingDuration, true);
					m_xAVPCloakEffect.SetTransitionTimeAndState( fTransTime , AVP_CloakEffect::CLOAKING, *pxAnim );
				}
				// Fully cloaked
				else if( pxCutscene->Timer >= m_fTriggerTime + m_fCloakingDuration && pxCutscene->Timer <= m_fTriggerTime + m_fDuration - m_fUncloakingDuration)
				{
					m_xAVPCloakEffect.SetTransitionTimeAndState( 1.0f , AVP_CloakEffect::CLOAKED, *pxAnim );
				}
				// On the uncloaking path
				else if( ( pxCutscene->Timer > m_fTriggerTime + m_fDuration - m_fUncloakingDuration ) && 
						 ( pxCutscene->Timer < m_fTriggerTime + m_fDuration))
				{
					float fTransTime = GetTransitionTime( m_fTriggerTime + m_fDuration - m_fUncloakingDuration, pxCutscene->Timer, m_fTriggerTime + m_fDuration, false);
					m_xAVPCloakEffect.SetTransitionTimeAndState( fTransTime , AVP_CloakEffect::UNCLOAKING, *pxAnim );
				}
				// Uncloked
				else
				{
					m_xAVPCloakEffect.SetTransitionTimeAndState( 0.0f , AVP_CloakEffect::UNCLOAKED, *pxAnim );
				}
			}
			else
			{
				Asura_Assert(false, "Actor has no anmiation");
			}
		}
		else
		{
			Asura_Assert(false, "Not a hierarchy actor");
		}
	}
	else
	{
		Asura_Assert(false, "No actor!");
	}
}

void AVP_Cutscene_Event_CloakActor::Trigger(Asura_Cutscene* pxCutscene)
{
	PARENT::Trigger(pxCutscene);
    
	if(m_pxTargetActor)
	{	
		if(m_pxTargetActor->IsHierarchyActor())
		{
            Asura_Cutscene_Actor_Hierarchy* pxActor = static_cast<Asura_Cutscene_Actor_Hierarchy*> (m_pxTargetActor);
			Asura_Animation* pxAnim = pxActor->GetAnimation();
			if( pxAnim )
			{
				Asura_Assert(m_fDuration > m_fCloakingDuration + m_fUncloakingDuration, "Event duration must be bigger than start and end cloaking phases ");
			
				if (!pxAnim->GetProjectData())
				{
					pxAnim->SetProjectData(&m_xAnimProjectData);
				}
				m_xAnimProjectData.SetCloakEffect(&m_xAVPCloakEffect);

				m_xAVPCloakEffect.SetLightningScale(m_fLightningScale);
				UpdateCloakingTimerAndState(pxCutscene);
			}
			else
			{
				Asura_Assert(false, "Actor has no anmiation");
			}
		}
		else
		{
			Asura_Assert(false, "Not a hierarchy actor");
		}
	}
	else
	{
		Asura_Assert(false, "No actor!");
	}
}

int AVP_Cutscene_Event_CloakActor::GetChunkSize()
{
	int iChunkSize = PARENT::GetChunkSize()
		             + sizeof(int)            // Chunk version
				     + sizeof(float) * 4;     // Duration, start/end transition and scale

	return iChunkSize;
}

void AVP_Cutscene_Event_CloakActor::SaveToChunk(char*& pcChunkBuffer)
{
	int iCurrentChunkVersion = CURRENT_CHUNK_VERSION;

	Asura_Chunk_ChunkBuffer_Stream xChunkBufferStream(pcChunkBuffer);

	xChunkBufferStream << iCurrentChunkVersion;

	PARENT::WriteToChunkStream(xChunkBufferStream);

	xChunkBufferStream << m_fDuration;
	xChunkBufferStream << m_fCloakingDuration;
    xChunkBufferStream << m_fUncloakingDuration;
    xChunkBufferStream << m_fLightningScale;
}

void AVP_Cutscene_Event_CloakActor::LoadFromChunk(Asura_Chunk_CutsceneEvent* pxChunk, Asura_Handle iFileHandle)
{
	int iCurrentChunkVersion;

	Asura_Chunk_File_Stream xFstream(iFileHandle);

    xFstream >> iCurrentChunkVersion;

	switch ( iCurrentChunkVersion )
	{
		case 0 :
		{
			PARENT::LoadFromChunk(pxChunk, iFileHandle);

			xFstream >> m_fDuration;
			xFstream >> m_fCloakingDuration;
			xFstream >> m_fUncloakingDuration;
			xFstream >> m_fLightningScale;
			break;
		}
		default :
		{
			Asura_Assertf(false, ("Unknown chunk version (%d) in AVP_Cutscene_Event_CloakActor", iCurrentChunkVersion));
			break;
		}
	}
}

#ifdef ASURA_TOOLS
void AVP_Cutscene_Event_CloakActor::Copy(Asura_Cutscene_Event* pxEvent)
{
	AVP_Cutscene_Event_CloakActor* pxCloakEvent = static_cast<AVP_Cutscene_Event_CloakActor*> (pxEvent);
	PARENT::Copy(pxCloakEvent);

	m_fDuration           = pxCloakEvent->m_fDuration;
    m_fCloakingDuration   = pxCloakEvent->m_fCloakingDuration;
	m_fUncloakingDuration = pxCloakEvent->m_fUncloakingDuration;
	m_fLightningScale     = pxCloakEvent->m_fLightningScale;
}
#endif // ASURA_TOOLS