///////////////////////////////////////////////////////////////////////////////
//
// File: AVP_ServerEntity_CameraVolume.cpp
//
// Created by Martin G.Harris (6/9/2006)
// 
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"
#include "Entity/CameraVolume/AVP_ServerEntity_CameraVolume.h"

#ifdef ASURA_USE_CAMERA_VOLUME

// Includes
#include "Entity/CameraVolume/AVP_ClientEntity_CameraVolume.h"

///////////////////////////////////////
// AVP_ServerEntity_CameraVolume //
///////////////////////////////////////
AVP_ServerEntity_CameraVolume::AVP_ServerEntity_CameraVolume(bool bActive /*=true*/)
: PARENT(bActive)
{
}

AVP_ServerEntity_CameraVolume::AVP_ServerEntity_CameraVolume(Asura_Guid uGuid)
: PARENT(uGuid)
{
}

AVP_ServerEntity_CameraVolume::~AVP_ServerEntity_CameraVolume()
{
}

// Chunk buffer functions
int AVP_ServerEntity_CameraVolume::GetChunkBufferSize()
{
	u_int uSize = 0;
	const AVP_ClientEntity_CameraVolume* pxClient = AVP_ClientEntity_CameraVolume::FindEntity(Guid);
	if (pxClient)
	{
		uSize = pxClient->m_xProperty.GetChunkBufferSize();
	}
	else
	{
		// Couldn't find the client entity; fallback is to write a version-zero chunk.
		uSize = AVP_CameraVolumeData::GetChunkBufferSize_V0();
	}

	return PARENT::GetChunkBufferSize() + uSize;
}

void AVP_ServerEntity_CameraVolume::FillInChunkBuffer(char*& pcChunkBuffer)
{
	// Parent must come first.It contains the volume data.
	PARENT::FillInChunkBuffer(pcChunkBuffer);

	AVP_ClientEntity_CameraVolume* pxClient = AVP_ClientEntity_CameraVolume::FindEntity(Guid);
	if (pxClient)
	{
		// All the volume data is stored in the client entity.
		pxClient->m_xProperty.FillInChunkBuffer(pcChunkBuffer);
	}
	else
	{
		Asura_Assert(0, "Error: AVP_ServerEntity_CameraVolume couldn't find client");

		// Write a zero version number to mark that there is no volume data.
		AVP_CameraVolumeData::FillInChunkBuffer_V0(pcChunkBuffer);
	}
}

void AVP_ServerEntity_CameraVolume::InitialiseFromChunkBuffer(const char*& pcChunkBuffer)
{
	// Load the active flag.
	PARENT::InitialiseFromChunkBuffer(pcChunkBuffer);

	// Check the version and skip the appropriate number of bytes. All the property data is
	// stored in the client entity.
	AVP_CameraVolumeData::SkipChunk(pcChunkBuffer);
}

#endif //ASURA_USE_CAMERA_VOLUME
