///////////////////////////////////////////////////////////////////////////////
//
// File: AVP_ClientEntity_Container.h
//
// Created by Martin G.Harris (20/8/2008)
// 
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//
// Project-specific client entity that manages a container instance.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef __AVP_CLIENTENTITY_CONTAINER_H__
#define __AVP_CLIENTENTITY_CONTAINER_H__

#ifdef ASURA_USE_CONTAINER_SYSTEM

// Includes
#include "Entity/Container/Asura_ClientEntity_Container.h"

// Forward declarations
class AVP_EntityDamage_ApplyToEntity;

////////////////////////////////
// AVP_ClientEntity_Container //
////////////////////////////////
class AVP_ClientEntity_Container : public Asura_ClientEntity_Container
{
	typedef Asura_ClientEntity_Container PARENT;

public:
	AVP_ClientEntity_Container();
	virtual ~AVP_ClientEntity_Container();

	virtual bool HandleMessage(const Asura_Message* pxMessage);

	// Damage message received from server.
	void OnDeath		(const AVP_EntityDamage_ApplyToEntity& xDamage);
	void OnApplyDamage	(const AVP_EntityDamage_ApplyToEntity& xDamage);

	void SetIgnoredForSurfaceReorientation(bool bIgnore);
	bool IsIgnoredForSurfaceReorientation() const { return m_bIgnoreForSurfaceReorientation; }

	// Process the specified number of bytes of project-specific data.
	virtual void ProcessProjectCreateMessageData(u_int uNumBytes, const void* pData);

	// Called after the creation of the entity, and after a state change, activation, or deactivation.
	// Gives projects a chance to update any project-specific stuff.
	virtual void PostChangeInitialisation();

protected:
	void UpdateIgnoredForSurfaceReorientation();

protected:
	bool m_bIgnoreForSurfaceReorientation;

public:
	// Static members
	static AVP_ClientEntity_Container* FindEntity(Asura_Guid uGuid)
	{
		return static_cast<AVP_ClientEntity_Container*>(PARENT::FindEntity(uGuid));
	}
};

////////////////////////////////////////
// AVP_Container_Functor_GetLuminance //
////////////////////////////////////////
// Get a rough luminance value at the waypoint, given its position and normal.
class AVP_Container_Functor_GetLuminance : public Asura_Container_Functor
{
public:
	AVP_Container_Functor_GetLuminance(const Asura_Vector_3& xPosition, const Asura_Vector_3& xNormal)
		: m_fLuminance(0.0f)
		, m_xPosition(xPosition)
		, m_xNormal(xNormal)
	{}

	virtual void Process(const Asura_ClientContainerInstance* pxEntry);

	float GetLuminance() const { return m_fLuminance; }

private:
	void ProcessPhoton(const Asura_ClientContainerInstance_Photon* pxPhotonInstance);
	void ProcessProjector(const Asura_ClientContainerInstance_Projector* pxProjectorInstance);

private:
	float m_fLuminance;			// Result
	Asura_Vector_3 m_xPosition; // Waypoint position
	Asura_Vector_3 m_xNormal;	// Waypoint normal
};

/////////////////////////////////////////
// AVP_Container_Functor_SetLightingBB //
/////////////////////////////////////////
// Get a rough BB for the volume affected by the lighting, combine this volume with the given BB.
class AVP_Container_Functor_SetLightingBB : public Asura_Container_Functor
{
public:
	AVP_Container_Functor_SetLightingBB(Asura_Bounding_Box& xBB)
		: m_xBB(xBB)
		, m_iNumber(0)
	{}

	virtual void Process(const Asura_ClientContainerInstance* pxEntry);

	int GetNumber() const { return m_iNumber; }
	bool HasLights() const {  return m_iNumber > 0; }

private:
	void ProcessPhoton(const Asura_ClientContainerInstance_Photon* pxPhotonInstance);
	void ProcessProjector(const Asura_ClientContainerInstance_Projector* pxProjectorInstance);

private:
	Asura_Bounding_Box& m_xBB;
	int m_iNumber;
};


///////////////////////////////////////////////////////
// AVP_Container_Functor_SetSurfaceReorientationFlag //
///////////////////////////////////////////////////////
// Set/reset the surface reorientation flag for physics objects.
class AVP_Container_Functor_SetSurfaceReorientationFlag : public Asura_Container_Functor
{
public:
	AVP_Container_Functor_SetSurfaceReorientationFlag(bool bIgnore)
		: m_bIgnoreForSurfaceReorientation(bIgnore)
	{}

	virtual void Process(const Asura_ClientContainerInstance* pxEntry);

private:
	void ProcessMesh(const Asura_ClientContainerInstance_Mesh* pxMeshInstance);

private:
	bool m_bIgnoreForSurfaceReorientation;
};

#endif //#ifdef ASURA_USE_CONTAINER_SYSTEM

#endif //__AVP_CLIENTENTITY_CONTAINER_H__

