//------------------------------------------------------------------------------
// File   : AVP_Controller_Messages.h
// Author : Gareth Thomas
// Date   : 31 / 05 / 07
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//------------------------------------------------------------------------------
#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE
#ifndef __AVP_CONTROLLER_MESSAGES_H__
#define __AVP_CONTROLLER_MESSAGES_H__ 1

//------------------------------------------------------------------------------
// Includes
//------------------------------------------------------------------------------

#include "Network/Asura_Guid.h"

#include "Entity/AVP_Entity_Classification.h"

class Asura_Chunk_Stream;

//------------------------------------------------------------------------------
// Class        : AVP_Message_ChangeAlienSpawnSettings
//------------------------------------------------------------------------------
class AVP_Message_ChangeAlienSpawnSettings
{
public:
	void Initialise(void)
	{
		m_uVersion = CURRENT_VERSION;

		// Some arbitrary default values
		m_uMaxSpawned = 3;
		m_fSpawnDelay = 1.0f;
		m_fMaxValidThreatDist = 25.0f;
		m_fThreatDistDropoffStart = 4.0f;
		m_fSpawnDelayFromSame = 3.0f;
		m_fSpawnDelayDropoffEnd = 10.0f;

		m_uFlags = 0;
	}

	enum	{ CURRENT_VERSION = 2 };
	int		m_uVersion;

	int 	m_uMaxSpawned;
	float	m_fSpawnDelay;
	float	m_fMaxValidThreatDist;
	float	m_fThreatDistDropoffStart;
	float	m_fSpawnDelayFromSame;
	float	m_fSpawnDelayDropoffEnd; // Not used at the moment (AN 26/02/08) - TODO: Remove if not necessary

	union
	{
		u_int		m_uFlags;

		struct
		{
			u_int	m_bUpdateMaxSpawned				: 1,
					m_bUpdateSpawnDelay				: 1,
					m_bUpdateMaxSpawnDist			: 1,
					m_bUpdateSpawnDistDropoffStart	: 1,
					m_bUpdateSpawnDelayFromSame		: 1,
					m_bUpdateSpawnDelayDropoffEnd	: 1,
													: 0;
		};
	};
};
/// End of class 'AVP_Message_ChangeAlienSpawnSettings'. ///

//------------------------------------------------------------------------------
// Class        : AVP_Message_AddAlienToSpawnQueue
// Author       : Gareth Thomas
// Date         : 26-Oct-2007
//------------------------------------------------------------------------------
// Used to add Aliens to be spawned by the Team Controller
//------------------------------------------------------------------------------
class AVP_Message_AddAlienToSpawnQueue
{
public:

	// Constructor
	AVP_Message_AddAlienToSpawnQueue	(void)
	{
		m_uVersion = 1;
		m_uNumToSpawn = 1; 
		m_uBlueprintTypeHashID = ASURA_HASH_ID_UNSET;
		m_uBlueprintHashID = ASURA_HASH_ID_UNSET;
	}

	// Deconstructor
	~AVP_Message_AddAlienToSpawnQueue	(void)
	{
	}

	// Public Access Functions...
	void			SetNumToSpawn					(u_int uNum)			{m_uNumToSpawn = uNum;}
	u_int			GetNumToSpawn					(void) const			{return m_uNumToSpawn;}
	void			SetBlueprintType				(u_int uTypeHash)		{m_uBlueprintTypeHashID = uTypeHash;}
	u_int			GetBlueprintType				(void) const 			{return m_uBlueprintTypeHashID;}
	void			SetBlueprint					(u_int uHash)			{m_uBlueprintHashID = uHash;}
	u_int			GetBlueprint					(void) const			{return m_uBlueprintHashID;}

	// Public Variables...
	u_int			m_uVersion;		// MESSAGE VERSION NUMBER
	u_int			m_uNumToSpawn;
	Asura_Hash_ID	m_uBlueprintTypeHashID;
	Asura_Hash_ID	m_uBlueprintHashID;
};
/// End of class 'AVP_Message_AddAlienToSpawnQueue'. ///

//------------------------------------------------------------------------------
// Class        : AVP_Message_AddObjective
// Author       : Gareth Thomas
// Date         : 18-Nov-2007
//------------------------------------------------------------------------------
// Used to add an objective to a marine squad
//------------------------------------------------------------------------------
class AVP_Message_AddObjective
{
public:

	void Initialise()
	{
		m_uVersion = 2;
		SetObjectiveGuid(ASURA_GUID_UNREGISTERED);
	}

	~AVP_Message_AddObjective(void);

	void		SetObjectiveGuid	(Asura_Guid uObjectiveGuid)			{m_uObjectiveGuid = uObjectiveGuid;}
	Asura_Guid	GetObjectiveGuid	(void) const						{return m_uObjectiveGuid;}

	u_int		m_uVersion;
	Asura_Guid	m_uObjectiveGuid;
};
/// End of class 'AVP_Message_AddObjective'. ///

//------------------------------------------------------------------------------
// Class        : AVP_Message_AddSquadMember
// Author       : Gareth Thomas
// Date         : 21-Nov-2007
//------------------------------------------------------------------------------
// Used to add a marine to a marine squad
//------------------------------------------------------------------------------
class AVP_Message_AddSquadMember
{
public:

	AVP_Message_AddSquadMember(void)
	{
		m_uVersion = 1;
		m_uGuidToAdd = ASURA_GUID_UNREGISTERED;
	}

	~AVP_Message_AddSquadMember(void);

	void			SetGuidToAdd		(Asura_Guid uGuidToAdd)			{m_uGuidToAdd = uGuidToAdd;}
	Asura_Guid		GetGuidToAdd		(void) const					{return m_uGuidToAdd;}

	u_int			m_uVersion;
	Asura_Guid		m_uGuidToAdd;
};
/// End of class 'AVP_Message_AddSquadMember'. ///

//------------------------------------------------------------------------------
// Class        : AVP_Message_RemoveSquadMember
// Author       : Gareth Thomas
// Date         : 21-Nov-2007
//------------------------------------------------------------------------------
// Used to remove a marine from a marine squad
//------------------------------------------------------------------------------
class AVP_Message_RemoveSquadMember
{
public:

	AVP_Message_RemoveSquadMember(void)
	{
		m_uVersion = 1;
		m_uGuidToRemove = ASURA_GUID_UNREGISTERED;
	}

	~AVP_Message_RemoveSquadMember(void);

	void			SetGuidToRemove		(Asura_Guid uGuidToRemove)			{m_uGuidToRemove = uGuidToRemove;}
	Asura_Guid		GetGuidToRemove		(void) const						{return m_uGuidToRemove;}

	u_int			m_uVersion;
	Asura_Guid		m_uGuidToRemove;
};
/// End of class 'AVP_Message_RemoveSquadMember'. ///

//------------------------------------------------------------------------------
// Class        : AVP_Message_SetAlienTeamBehaviour
//------------------------------------------------------------------------------
class AVP_Message_SetAlienTeamBehaviour
{
public:

	void Initialise(void)
	{
		m_iVersion = CURRENT_VERSION;

		m_iBehaviour = 0;
		m_uBlueprintHashID = ASURA_HASH_ID_UNSET;
		m_uBlueprintTypeHashID = ASURA_HASH_ID_UNSET;
	}

	enum			{ CURRENT_VERSION = 3 }; // Changed behaviour attribute type
	int				m_iVersion;

	int				m_iBehaviour;

	// DO NOT RE-ORDER THESE ATTRIBUTES! (See ToolsAVP_Message_Editing::ChooseBlueprint() for details)
	Asura_Hash_ID	m_uBlueprintHashID;
	Asura_Hash_ID	m_uBlueprintTypeHashID;
	//
};
/// End of class 'AVP_Message_SetAlienTeamBehaviour'. ///

//------------------------------------------------------------------------------
// Class        : AVP_Message_AddAlienTeamTokens
//------------------------------------------------------------------------------
class AVP_Message_AddAlienTeamTokens
{
public:

	void Initialise(void)
	{
		m_iVersion = CURRENT_VERSION;

		m_uBlueprintHashID = ASURA_HASH_ID_UNSET;
		m_uBlueprintTypeHashID = ASURA_HASH_ID_UNSET;
		m_uTokenNum = 1;
	}

	enum			{ CURRENT_VERSION = 2 };
	int				m_iVersion;

	// DO NOT RE-ORDER THESE ATTRIBUTES! (See ToolsAVP_Message_Editing::ChooseBlueprint() for details)
	Asura_Hash_ID	m_uBlueprintHashID;
	Asura_Hash_ID	m_uBlueprintTypeHashID;
	//

	u_int			m_uTokenNum;
};
/// End of class 'AVP_Message_AddAlienTeamTokens'. ///

//------------------------------------------------------------------------------
// Class        : AVP_Message_RemoveAlienTeamTokens
//------------------------------------------------------------------------------
class AVP_Message_RemoveAlienTeamTokens
{
public:

	void Initialise(void)
	{
		m_iVersion = CURRENT_VERSION;

		m_uBlueprintHashID = ASURA_HASH_ID_UNSET;
		m_uBlueprintTypeHashID = ASURA_HASH_ID_UNSET;
		m_uTokenNum = 1;
		m_uFlags = 0;
	}

	enum			{ CURRENT_VERSION = 1 };
	int				m_iVersion;

	// DO NOT RE-ORDER THESE ATTRIBUTES! (See ToolsAVP_Message_Editing::ChooseBlueprint() for details)
	Asura_Hash_ID	m_uBlueprintHashID;
	Asura_Hash_ID	m_uBlueprintTypeHashID;
	//

	u_int			m_uTokenNum;

	union
	{
		u_int		m_uFlags;

		struct
		{
			u_int	m_bRemoveAll	: 1,
									: 0;
		};
	};
};
/// End of class 'AVP_Message_RemoveAlienTeamTokens'. ///

//------------------------------------------------------------------------------
// Class        : AVP_Message_ChangeThreatBonusValues
//------------------------------------------------------------------------------
class AVP_Message_ChangeThreatBonusValues
{
public:
	void Initialise(void)
	{
		m_iVersion = CURRENT_VERSION;

		m_fUnknownBonus = 0.0f;
		m_fCivilianBonus = 1.0f;

		m_fMarineBonus = 1.0f;
		m_fMarineShotgunBonus = 1.0f;
		m_fMarineFlameBonus = 1.0f;
		m_fMarineSmartgunBonus = 1.0f;
		m_fMarinePlayerBonus = 1.0f;

		m_fAndroidBonus = 1.0f;
		m_fSentryGunBonus = 1.0f;

		m_fPredatorBonus = 1.0f;
		m_fPredatorPlayerBonus = 1.0f;

		m_fSameTargetBonus = 1.0f;
		m_fPerAttackerBonus = 1.0f;

		m_uFlags = 0;
	}

	enum	{ CURRENT_VERSION = 1 };
	int		m_iVersion;

	float	m_fUnknownBonus;
	float	m_fCivilianBonus;

	float	m_fMarineBonus;
	float	m_fMarineShotgunBonus;
	float	m_fMarineFlameBonus;
	float	m_fMarineSmartgunBonus;
	float	m_fMarinePlayerBonus;

	float	m_fAndroidBonus;
	float	m_fSentryGunBonus;

	float	m_fPredatorBonus;
	float	m_fPredatorPlayerBonus;
		
	float	m_fSameTargetBonus;
	float	m_fPerAttackerBonus;

	union
	{
		u_int		m_uFlags;

		struct
		{
			u_int	m_bUpdateUnknownBonus			: 1,
					m_bUpdateCivilianBonus			: 1,
					m_bUpdateMarineBonus			: 1,
					m_bUpdateMarineShotgunBonus		: 1,
					m_bUpdateMarineFlameBonus		: 1,
					m_bUpdateMarineSmartgunBonus	: 1,
					m_bUpdateMarinePlayerBonus		: 1,
					m_bUpdateAndroidBonus			: 1,
					m_bUpdateSentryGunBonus			: 1,
					m_bUpdatePredatorBonus			: 1,
					m_bUpdatePredatorPlayerBonus	: 1,
					m_bUpdateSameTargetBonus		: 1,
					m_bUpdatePerAttackerBonus		: 1,
													: 0;
		};
	};
};
/// End of class 'AVP_Message_ChangeAlienSpawnSettings'. ///

//------------------------------------------------------------------------------
// Class        : AVP_Message_SpawnHumanoids
// Author       : Gareth Thomas
// Date         : 25 / 02 / 08
//------------------------------------------------------------------------------
// Spawned Humanoid Actor(s)
//------------------------------------------------------------------------------
class AVP_Message_SpawnHumanoids
{

public:

	enum HumanoidTypes
	{
		SPAWN_HUMANOID_ANDROID = 0,
		SPAWN_HUMANOID_CIVILIAN,
		SPAWN_HUMANOID_MARINE,
		MAX_HUMANOID_TYPES,
	};

	enum SpawnSettings
	{
		MIN_TO_SPAWN			= 1,
		MAX_TO_SPAWN			= 8,
	};

	enum SpawnMethods
	{
		SPAWN_METHOD_TEAM_CONTROLLED = 0,
		SPAWN_METHOD_SPECIFIC_SPAWN_POINT,
		SPAWN_METHOD_SPECIFIC_GAME_PLAY_VOLUME,
		MAX_SPAWN_METHODS,
	};

	u_int			GetVersion					(void) const	{return m_uVersion;}
	int				GetHumanoidTypeToSpawn		(void) const	{return m_iHumanoidType;}
	int				GetNumberToSpawn			(void) const	{return m_iNumToSpawn;}
	int				GetSpawnMethod				(void) const	{return m_iSpawnMethod;}
	Asura_Guid		GetSquadGuid				(void) const	{return m_uSquadControllerGuid;}
	Asura_Guid		GetGamePlayVolumeGuid		(void) const	{return m_uGamePlayVolumeGuid;}
	Asura_Guid		GetHumanoidSpawnGuid		(void) const	{return m_uSpawnPointGuid;}
	Asura_Hash_ID	GetBlueprintTypeHashID		(void) const	{return m_uBlueprintTypeHashID;}
	Asura_Hash_ID	GetBlueprintHashID			(void) const	{return m_uBlueprintHashID;}

	void Initialise(void)
	{
		m_uVersion = 1;
		
		m_iHumanoidType			= SPAWN_HUMANOID_ANDROID;
		m_iNumToSpawn			= MIN_TO_SPAWN;
		m_iSpawnMethod			= SPAWN_METHOD_TEAM_CONTROLLED;

		m_uSquadControllerGuid	= ASURA_GUID_UNREGISTERED;
		m_uGamePlayVolumeGuid	= ASURA_GUID_UNREGISTERED;
		m_uSpawnPointGuid		= ASURA_GUID_UNREGISTERED;

		m_uBlueprintTypeHashID	= ASURA_HASH_ID_UNSET;
		m_uBlueprintHashID		= ASURA_HASH_ID_UNSET;
	}

	u_int			m_uVersion;
	int				m_iHumanoidType;
	int				m_iNumToSpawn;
	int				m_iSpawnMethod;
	Asura_Guid		m_uSquadControllerGuid;
	Asura_Guid		m_uGamePlayVolumeGuid;
	Asura_Guid		m_uSpawnPointGuid;
	
	// DO NOT RE-ORDER THESE ATTRIBUTES! (See ToolsAVP_Message_Editing::ChooseBlueprint() for details)
	Asura_Hash_ID	m_uBlueprintHashID;
	Asura_Hash_ID	m_uBlueprintTypeHashID;
};
/// End of class 'AVP_Message_SpawnHumanoids'. ///

//------------------------------------------------------------------------------
// Class        : AVP_Message_ChangeHumanoidSpawnSettings
// Author       : Gareth Thomas
// Date         : 27 / 02 / 08
//------------------------------------------------------------------------------
class AVP_Message_ChangeHumanoidSpawnSettings
{
public:

	void Initialise(void)
	{
		m_uVersion = 1;
		m_fSpawnDelay = 1.0f;
	}

	u_int GetVersion()		const	{return m_uVersion;}
	float GetSpawnDealy()	const	{return m_fSpawnDelay;}

	u_int	m_uVersion;
	float	m_fSpawnDelay;
};
/// End of class 'AVP_Message_ChangeHumanoidSpawnSettings'. ///

//------------------------------------------------------------------------------
// Class        : AVP_Message_MergeSquads
// Author       : Gareth Thomas
// Date         : 27 / 02 / 08
//------------------------------------------------------------------------------
class AVP_Message_MergeSquads
{
public:

	void Initialise(void)
	{
		m_uVersion = 1;
		m_uSquadToMergeInto = ASURA_GUID_UNREGISTERED;
	}

	u_int			m_uVersion;
	Asura_Guid		m_uSquadToMergeInto;
};
/// End of class 'AVP_Message_MergeSquads'. ///

//------------------------------------------------------------------------------
// Class        : AVP_Message_SplitSquad
// Author       : Gareth Thomas
// Date         : 27 / 02 / 08
//------------------------------------------------------------------------------
class AVP_Message_SplitSquad
{
public:

	enum { MAX_SQUAD_MEMBERS = 8};

	void Initialise(void)
	{
		m_uVersion = 1;
		m_uSquadToAddTo = ASURA_GUID_UNREGISTERED;

		for (u_int u = 0; u < MAX_SQUAD_MEMBERS; ++u)
		{
			m_auSplitMembers[u] = ASURA_GUID_UNREGISTERED;
		}
	}

	u_int			m_uVersion;
	Asura_Guid		m_uSquadToAddTo;
	Asura_Guid		m_auSplitMembers[MAX_SQUAD_MEMBERS];
};
/// End of class 'AVP_Message_SplitSquad'. ///

//------------------------------------------------------------------------------
// Class        : AVP_Message_RequestPatrol
// Author       : Gareth Thomas
// Date         : 27 / 02 / 08
//------------------------------------------------------------------------------
class AVP_Message_RequestPatrol
{
public:

	void Initialise(void)
	{
		m_uVersion = 1;
		m_bFullSquad = false;
		m_uNumPatrolMembers = 0;
		m_uGuardZoneGuid = ASURA_GUID_UNREGISTERED;
	}

	u_int		m_uVersion;
	u_int		m_bFullSquad;
	u_int		m_uNumPatrolMembers;
	Asura_Guid	m_uGuardZoneGuid;

};
/// End of class 'AVP_Message_RequestPatrol'. ///

//------------------------------------------------------------------------------
// Class        : AVP_Message_ChangeMarineSquadSettings
// Author       : Gareth Thomas
// Date         : 27 / 02 / 08
//------------------------------------------------------------------------------
class AVP_Message_ChangeMarineSquadSettings
{
public:

	enum { CURRENT_MSG_VERSION = 3 };

	void Initialise()
	{
		m_uVersion = 3;
		m_bIsPlayersSquad = false;
		m_bIsAiLedPlayersSquad = false;
		m_bIsRestrictedToCombatZone = false;
		m_bIgnoreAlertState = false;
		m_uCombatZoneGPVGuid = ASURA_GUID_UNREGISTERED;
		m_iSquadCombatType = 0;
	}

	u_int			m_uVersion;
	u_int			m_bIsPlayersSquad;
	u_int			m_bIsAiLedPlayersSquad;
	u_int			m_bIsRestrictedToCombatZone;
	u_int			m_bIgnoreAlertState;
	Asura_Guid		m_uCombatZoneGPVGuid;
	int				m_iSquadCombatType;
};
/// End of class 'AVP_Message_ChangeMarineSquadSettings'. ///

//------------------------------------------------------------------------------
// Class        : AVP_Message_ChangeHumanoidMovementMode
// Author       : Gareth Thomas
// Date         : 02 / 12 / 08
//------------------------------------------------------------------------------
class AVP_Message_ChangeHumanoidMovementMode
{

public:

	// CHANGE THE DATA - CHANGE THE VERSION!
	enum { CURRENT_MSG_VERSION = 1 };

	void Initialise()
	{
		m_uVersion = CURRENT_MSG_VERSION;
		m_iMovementMode = 0;
	}

	u_int m_uVersion;
	int m_iMovementMode;
};
/// End of class 'AVP_Message_ChangeHumanoidMovementMode'. ///

//------------------------------------------------------------------------------
// Class        : AVP_Message_RequestPatrol
// Author       : Gareth Thomas
// Date         : 27 / 02 / 08
//------------------------------------------------------------------------------
class AVP_Message_PerformObjectiveAction
{
public:

	void Initialise(void)
	{
		m_uVersion = 1;
		m_uActionGuid = ASURA_GUID_UNREGISTERED;
	}

	void		SetActionGuid(Asura_Guid uGuid) {m_uActionGuid = uGuid;}
	Asura_Guid	GetActionGuid(void) const		{return m_uActionGuid;}

	u_int		m_uVersion;
	u_int		m_uActionGuid;

};
/// End of class 'AVP_Message_RequestPatrol'. ///

struct AVP_AI_TargetPriorityData
{
	enum Priority
	{
		UNSET,
		IGNORE_P,		// winbase.h: #define IGNORE 0. Genius.
		LOW,
		NORMAL,
		HIGH,
		NUM_PRIORITIES
	};
	enum TargetType
	{
		ALIEN,
		ANDROID,
		FACE_HUGGER,
		MARINE,
		PLAYER,
		PREDATOR,
		SENTRY_GUN,
		SPECIFIC_ENTITY,
		CIVILIAN,
		PREDALIEN,
		NUM_TARGET_TYPES
	};
	enum SpecificGuidType
	{
		ACTOR,
		SQUAD,
		NUM_SPECIFIC_GUID_TYPES
	};

	int m_iPriority;
	int m_iTargetType;
	Asura_Guid m_uSpecificGuid;
	int m_iSpecificGuidType;

	AVP_AI_TargetPriorityData()
	: m_iPriority( UNSET ), m_iTargetType( NUM_TARGET_TYPES ), m_uSpecificGuid( ASURA_GUID_UNREGISTERED ), m_iSpecificGuidType( NUM_SPECIFIC_GUID_TYPES )
	{
	}

	bool operator==( const AVP_AI_TargetPriorityData& xRhs ) const
	{
		return ( m_iPriority == xRhs.m_iPriority &&
			m_iTargetType == xRhs.m_iTargetType &&
			m_uSpecificGuid == xRhs.m_uSpecificGuid &&
			m_iSpecificGuidType == xRhs.m_iSpecificGuidType );
	}

	bool ClassificationMatchesTargetType( const int iClassification ) const;

	void ReadFromChunkStream( Asura_Chunk_Stream& xStream );
	void WriteToChunkStream( Asura_Chunk_Stream& xStream ) const;

#ifdef ASURA_TOOLS
	static const char* s_aszPriorityStrings[ NUM_PRIORITIES ];
	static const char* s_aszTargetTypeStrings[ NUM_TARGET_TYPES ];
#endif

private:
	static const u_int uCURRENT_CHUNK_VERSION = 0;
};

struct AVP_Msg_SetAITargetPriorities
{
	u_int m_uVersion;
	u_int m_uNumEntries;

	// There then follows m_uNumEntries lots of the following data:
	struct TPData
	{
		int m_iPriority;
		int m_iTargetType;
		Asura_Guid m_uSpecificGuid;
		int m_iSpecificGuidType;
	};
	// using the same enum values as AVP_AI_TargetPriorityData.

	void Initialise()
	{
		m_uVersion = uCURRENT_VERSION;
		m_uNumEntries = 1;
	}

	TPData* GetFirstTPDataPtr() const
	{
		Asura_Assert( m_uNumEntries, "AVP_Msg_SetAITargetPriorities::GetFirstTPDataPtr - no entries" );
		const u_int* pu = &m_uNumEntries + 1;
		return ( TPData* )pu;
	}

	static const u_int uCURRENT_VERSION = 0;
};

//------------------------------------------------------------------------------
// Class        : AVP_Message_ChangeHumanoidMovementMode
// Author       : Gareth Thomas
// Date         : 02 / 12 / 08
//------------------------------------------------------------------------------
class AVP_Message_LookAtEntity
{
public:

	void Initialise(void)
	{
		m_uVersion = 2;
		m_uLookAtGuid = ASURA_GUID_UNREGISTERED;
		m_bLookAtPlayer = 0;
		m_uLookAtMarker = ASURA_HASH_ID_UNSET;
	}

	void			SetLookAtGuid(Asura_Guid uGuid) { m_uLookAtGuid = uGuid; }
	Asura_Guid		GetLookAtGuid(void) const		{ return m_uLookAtGuid; }

	void			SetLookAtMarker(Asura_Hash_ID uMarker) { m_uLookAtMarker = uMarker; }
	Asura_Hash_ID	GetLookAtMarker(void) const		{ return m_uLookAtMarker; }

	u_int			m_uVersion;
	Asura_Guid		m_uLookAtGuid;
	u_int			m_bLookAtPlayer;
	Asura_Hash_ID	m_uLookAtMarker;

};
/// End of class 'AVP_Message_ChangeHumanoidMovementMode'. ///

class AVP_Message_ChangeHumanoidKnowledgeSettings
{

public:

	void Initialise( void )
	{
		m_uVersion = 1;
		m_uHumanoidType = AVP_ENTITYCLASS_ANDROID;
		m_uBluepprintHashID = 0x87aac6a1; //"KnowledgeSettings";
	}

	u_int			m_uVersion;
	u_int			m_uHumanoidType;
	Asura_Hash_ID	m_uBluepprintHashID;
};

#endif // __AVP_CONTROLLER_MESSAGES_H__
