//////////////////////////////////////////////////////////////////////////////
//
// AVP_ServerEntity_Squad.cpp
//
// Copyright (C) 2008 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"

// header
#include "Entity/Controller/AVP_ServerEntity_Squad.h"

// asura
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "Network/Asura_Message.h"

// axon

// AVP
#include "Entity/AVP_Entity_Classification.h"
#include "Entity/AVP_Messages.h"
#include "Entity/AVP_Entity_EventIDs.h"
#include "Entity/Controller/AVP_Controller_Messages.h"
#include "Entity/PhysicalObject/Actor/AVP_ServerEntity_Actor.h"

//////////////////////////////////////////////////////////////////////////////
//
// AVP_AI_TargetPriorityData
// It's gotta go somewhere.
//
//////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_TOOLS
const char* AVP_AI_TargetPriorityData::s_aszPriorityStrings[ NUM_PRIORITIES ] =
{
	"",
	"Ignore",
	"Low",
	"Normal",
	"High"
};
const char* AVP_AI_TargetPriorityData::s_aszTargetTypeStrings[ NUM_TARGET_TYPES ] =
{
	"Alien",
	"Android",
	"Face Hugger",
	"Marine",
	"Player",
	"Predator",
	"Sentry Gun",
	"Specific Entity",
	"Civilians",
	"Predalien",
};
#endif

bool AVP_AI_TargetPriorityData::ClassificationMatchesTargetType( const int iClassification ) const
{
	switch ( m_iTargetType )
	{
		case AVP_AI_TargetPriorityData::ALIEN:			return ( iClassification == AVP_ENTITYCLASS_ALIEN );
		case AVP_AI_TargetPriorityData::ANDROID:		return ( iClassification == AVP_ENTITYCLASS_ANDROID );
		case AVP_AI_TargetPriorityData::FACE_HUGGER:	return ( iClassification == AVP_ENTITYCLASS_ALIENFACEHUGGER );
		case AVP_AI_TargetPriorityData::MARINE:			return ( iClassification == AVP_ENTITYCLASS_MARINE );
		case AVP_AI_TargetPriorityData::PLAYER:			return ( iClassification == AVP_ENTITYCLASS_PLAYER );
		case AVP_AI_TargetPriorityData::PREDATOR:		return ( iClassification == AVP_ENTITYCLASS_PREDATOR );
		case AVP_AI_TargetPriorityData::SENTRY_GUN:		return ( iClassification == AVP_ENTITYCLASS_SENTRYGUN );
		case AVP_AI_TargetPriorityData::CIVILIAN:		return ( iClassification == AVP_ENTITYCLASS_CIVILIAN );
		case AVP_AI_TargetPriorityData::PREDALIEN:		return ( iClassification == AVP_ENTITYCLASS_PREDALIEN );
		default: return false;
	}
}

void AVP_AI_TargetPriorityData::ReadFromChunkStream( Asura_Chunk_Stream& xStream )
{
	u_int uVersion;
	xStream >> uVersion;

	if ( uVersion == uCURRENT_CHUNK_VERSION )
	{
		xStream >> m_iPriority;
		xStream >> m_iTargetType;
		xStream >> m_uSpecificGuid;
		xStream >> m_iSpecificGuidType;
	}
	else
	{
		Asura_Assertf( false, ( "AVP_AI_TargetPriorityData::ReadFromChunkStream - unhandled chunk version (%u)", uVersion ) );
	}
}

void AVP_AI_TargetPriorityData::WriteToChunkStream( Asura_Chunk_Stream& xStream ) const
{
	xStream << uCURRENT_CHUNK_VERSION;
	xStream << m_iPriority;
	xStream << m_iTargetType;
	xStream << m_uSpecificGuid;
	xStream << m_iSpecificGuidType;
}

//////////////////////////////////////////////////////////////////////////////
//
// AVP_ServerEntity_Squad
//
//////////////////////////////////////////////////////////////////////////////

Asura_Hashed_List< AVP_ServerEntity_Squad > AVP_ServerEntity_Squad::s_xList;

AVP_ServerEntity_Squad::AVP_ServerEntity_Squad( Asura_Guid uGuid )
: PARENT( uGuid )
, m_xMembers()
, m_uNumMembers( 0 )
, m_uFlags( 0 )
{
	SetNeverUpdateClient();

	s_xList.AddToList( this );
}

AVP_ServerEntity_Squad::~AVP_ServerEntity_Squad()
{
	s_xList.RemoveFromList( this );
}

void AVP_ServerEntity_Squad::ReadFromChunkStream( Asura_Chunk_Stream& xStream )
{
	u_int uVersion;
	xStream	>> uVersion;

	PARENT::ReadFromChunkStream( xStream );

	if ( uVersion == uCURRENT_CHUNK_VERSION )
	{
		xStream >> m_uNumMembers;
		for ( u_int u = 0; u < m_uNumMembers; ++u )
		{
			SquadMember xSquadMember;
			xStream >> xSquadMember.m_uMemberGuid;
			xSquadMember.m_pxMemberData = AllocateMemberData( xSquadMember.m_uMemberGuid );
			m_xMembers.Push_Back( xSquadMember );
		}

		Asura_ReadFromStream_Bitfield_Wrapper< u_int > uFlagsWrapper( m_uFlags );
		xStream >> uFlagsWrapper;

		u_int uNumTPs;
		xStream >> uNumTPs;
		for ( u_int uTP = 0; uTP < uNumTPs; ++uTP )
		{
			AVP_AI_TargetPriorityData xTP;
			xTP.ReadFromChunkStream( xStream );
			m_xTargetPriorityList.Push_Back( xTP );
		}
	}
	else if ( uVersion == 2 )
	{
		xStream >> m_uNumMembers;
		for ( u_int u = 0; u < m_uNumMembers; ++u )
		{
			SquadMember xSquadMember;
			xStream >> xSquadMember.m_uMemberGuid;
			xSquadMember.m_pxMemberData = AllocateMemberData( xSquadMember.m_uMemberGuid );
			m_xMembers.Push_Back( xSquadMember );
		}

		Asura_ReadFromStream_Bitfield_Wrapper< u_int > uFlagsWrapper( m_uFlags );
		xStream >> uFlagsWrapper;
	}
#ifdef ASURA_TOOLS
	else if ( uVersion == 1 )
	{
		Asura_ReadFromStream_Bitfield_Wrapper< u_int > uFlagsWrapper( m_uFlags );
		xStream >> uFlagsWrapper;
	}
	else if ( uVersion == 0 )
	{
	}
#endif
	else
	{
		Asura_Assertf( false, ( "AVP_ServerEntity_Squad::ReadFromChunkStream - unhandled chunk version (%u)", uVersion ) );
	}
}

void AVP_ServerEntity_Squad::WriteToChunkStream( Asura_Chunk_Stream& xStream ) const
{
	xStream << uCURRENT_CHUNK_VERSION;

	PARENT::WriteToChunkStream( xStream );

	// Save the member guids only. Derived classes are responsible for saving member data.
	xStream << m_uNumMembers;
	Asura_ListElt< SquadMember >* pxElt = m_xMembers.Front();
	while ( pxElt != NULL )
	{
		xStream << pxElt->Data().GetGuid();
		pxElt = pxElt->Next();
	}

	Asura_WriteToStream_Bitfield_Wrapper< u_int > uFlagsWrapper( m_uFlags );
	uFlagsWrapper.Add( m_bIsDefaultSquad, 1 );
	uFlagsWrapper.Add( m_bEliminatedEventSent, 1 );
	uFlagsWrapper.Add( m_bHasHadMoreThanOneMember, 1 );
	uFlagsWrapper.Add( m_bIsDefaultPetSquad, 1 );
	xStream << uFlagsWrapper;

	u_int uNumTPs = m_xTargetPriorityList.Size();
	xStream << uNumTPs;
	for ( Asura_LinkList_ConstIt< AVP_AI_TargetPriorityData > xTPIt( m_xTargetPriorityList ); !xTPIt.Done(); xTPIt.Next() )
	{
		xTPIt.GetCurrent()->WriteToChunkStream( xStream );
	}
}

bool AVP_ServerEntity_Squad::HandleMessage( const Asura_Message* pxMessage )
{
	switch ( pxMessage->GetMsgID() )
	{
		case ASURA_MSG_ACTIVATE:
		{
			bool bRet = PARENT::HandleMessage( pxMessage );

			for (Asura_Hashed_List_It<AVP_ServerEntity_Actor> it(AVP_ServerEntity_Actor::GetList()); !it.Done(); it.Next())
			{
				AVP_ServerEntity_Actor* pxActor = it.GetCurrent();
				
				if (!pxActor || pxActor->IsPlayer() || pxActor->IsDead())
				{
					continue;
				}

				if (pxActor->GetInitialSquadGuid() == GetGuid())
				{
					pxActor->HandleMessage(pxMessage);
				}
			}

			return bRet;
		}

		case ASURA_MSG_DEACTIVATE:
		{
			// Forward the message on to the members.
			for ( Asura_LinkList_It< SquadMember > xMemberIt( m_xMembers ); !xMemberIt.Done(); )
			{
				SquadMember* pxMember = xMemberIt.GetCurrent();
				AVP_ServerEntity_Actor* pxActor = AVP_ServerEntity_Actor::FindEntity( pxMember->GetGuid() );
				if ( pxActor && !pxActor->IsPlayer() )
				{
					// The only safe way we can do this, since handling the message will remove it from the list without our iterator having any way to know about it, is to get in there and delete it first.
					ReturnMemberData( pxMember->m_pxMemberData );
					xMemberIt.DeleteCurrent();
					--m_uNumMembers;
					pxActor->HandleMessage( pxMessage );
				}
				else
				{
					xMemberIt.Next();
				}
			}

			return PARENT::HandleMessage( pxMessage );
		}

		case AVP_MSGID_SET_AI_TARGET_PRIORITIES:
		{
			// Register the target priorities.
			const AVP_Msg_SetAITargetPriorities* pxMsgData = static_cast< const AVP_Msg_SetAITargetPriorities* >( pxMessage->GetData() );
			if ( pxMsgData )
			{
				for ( u_int u = 0; u < pxMsgData->m_uNumEntries; ++u )
				{
					const AVP_Msg_SetAITargetPriorities::TPData* pxMsgTPData = &pxMsgData->GetFirstTPDataPtr()[ u ];

					if ( pxMsgTPData->m_iPriority == AVP_AI_TargetPriorityData::UNSET )
					{
						continue;
					}

					// See if we already have an entry that matches.
					AVP_AI_TargetPriorityData* pxTPData = NULL;
					for ( Asura_LinkList_It< AVP_AI_TargetPriorityData > xTPIt( m_xTargetPriorityList ); !xTPIt.Done() && !pxTPData; xTPIt.Next() )
					{
						AVP_AI_TargetPriorityData* pxItTPData = xTPIt.GetCurrent();
						if ( pxItTPData->m_iTargetType == pxMsgTPData->m_iTargetType )
						{
							if ( pxItTPData->m_iTargetType == AVP_AI_TargetPriorityData::SPECIFIC_ENTITY )
							{
								if ( pxItTPData->m_uSpecificGuid == pxMsgTPData->m_uSpecificGuid )
								{
									pxTPData = pxItTPData;
								}
							}
							else
							{
								pxTPData = pxItTPData;
							}
						}
					}

					if ( !pxTPData )
					{
						// Have to add a new one.
						AVP_AI_TargetPriorityData xTP;
						m_xTargetPriorityList.Push_Front( xTP );
						pxTPData = &m_xTargetPriorityList.Front()->Data();
					}

					// Update this entry.
					// If the priority is normal then delete it.
					if ( pxMsgTPData->m_iPriority == AVP_AI_TargetPriorityData::NORMAL )
					{
						m_xTargetPriorityList.Remove( m_xTargetPriorityList.Find( *pxTPData ) );
					}
					else
					{
						pxTPData->m_iPriority = pxMsgTPData->m_iPriority;
						pxTPData->m_iTargetType = pxMsgTPData->m_iTargetType;
						pxTPData->m_iSpecificGuidType = pxMsgTPData->m_iSpecificGuidType;
						pxTPData->m_uSpecificGuid = pxMsgTPData->m_uSpecificGuid;
					}
				}
			}

			return true;
		}

		case AVP_MSGID_RESET_AI_TARGET_PRIORITIES:
		{
			m_xTargetPriorityList.Clear();
			return true;
		}

		case ASURA_MSG_AXON_GS2_EXIT_GAMESCENE:
		case ASURA_MSG_SET_DESTRUCTIBLE:
		{
			// Forward the message on to the members.
			for ( Asura_LinkList_It< SquadMember > xMemberIt( m_xMembers ); !xMemberIt.Done(); xMemberIt.Next() )
			{
				SquadMember* pxMember = xMemberIt.GetCurrent();
				AVP_ServerEntity_Actor* pxActor = AVP_ServerEntity_Actor::FindEntity( pxMember->GetGuid() );
				if ( pxActor && !pxActor->IsPlayer() )
				{
					pxActor->HandleMessage( pxMessage );
				}
			}

			return true;
		}

		default:
		{
			return PARENT::HandleMessage( pxMessage );
		}
	}
}

void AVP_ServerEntity_Squad::Update()
{
	PARENT::Update();

	if ( GetNumSquadMembers() > 0 )
	{
		SetElimintedEventSent( false );
		SetHasHadMoreThanOneMember( true );
	}

	SendEventMessages();
}

void AVP_ServerEntity_Squad::PreDestructionShutdown()
{
	PARENT::PreDestructionShutdown();

	// Chuck out any members that have not unregistered.
	Asura_ListElt< SquadMember >* pxFront = m_xMembers.Front();
	while ( pxFront )
	{
		ReturnMemberData( pxFront->Data().m_pxMemberData );
		m_xMembers.Remove( pxFront );
		pxFront = m_xMembers.Front();
	}

	m_xTargetPriorityList.Clear();
}

void AVP_ServerEntity_Squad::SendEventMessages( void )
{
	if ( GetNumSquadMembers() == 0 && GetHasHadMoreThanOneMember() && !GetElimintedEventSent() )
	{
		SetElimintedEventSent( true );
		SendMessagesForEvent( AVP_ENTITY_EVENT_SQUAD_ELIMINATED );
	}
}

void AVP_ServerEntity_Squad::SendMemberDeadEventMessage( void )
{
	SendMessagesForEvent( AVP_ENTITY_EVENT_SQUAD_MEMBER_DIED );
}

void AVP_ServerEntity_Squad::AddMember( const Asura_Guid uGuid )
{
	// Check if this is already a member.
	if ( m_xMembers.FindByGuid( uGuid ) )
	{
		return;
	}

	SquadMember xSquadMember;
    xSquadMember.m_uMemberGuid  = uGuid;
	xSquadMember.m_pxMemberData = AllocateMemberData( uGuid );
	m_xMembers.Push_Front( xSquadMember );
	++m_uNumMembers;
}

void AVP_ServerEntity_Squad::RemoveMember( const Asura_Guid uGuid )
{
	Asura_ListElt< SquadMember >* pxElt = m_xMembers.FindByGuid( uGuid );
	if ( pxElt )
	{
		ReturnMemberData( pxElt->Data().m_pxMemberData );
		m_xMembers.Remove( pxElt );
		--m_uNumMembers;
	}
}

Asura_Guid AVP_ServerEntity_Squad::GetMemberGuid( int iIndex ) const
{
	Asura_ListElt< SquadMember >* pxElt = m_xMembers.Front();
	while ( iIndex-- && pxElt != NULL )
	{
		pxElt = pxElt->Next();
	}
	return ( pxElt != NULL ) ? pxElt->Data().GetGuid() : ASURA_GUID_UNREGISTERED;
}

void* AVP_ServerEntity_Squad::GetMemberData( const Asura_Guid uGuid ) const
{
	Asura_ListElt< SquadMember >* pxMemberElt = m_xMembers.FindByGuid( uGuid );
	return pxMemberElt ? pxMemberElt->Data().m_pxMemberData : NULL;
}

bool AVP_ServerEntity_Squad::CalcTargetingPriority( const int iTargetClassification, const Asura_Guid uTargetGuid, int& iPriority ) const
{
	// See if this guid or type is down as being of a non-default targeting priority.
	const AVP_ServerEntity_Actor* pxTargetActor = AVP_ServerEntity_Actor::FindEntity( uTargetGuid );
	const bool bTargetIsPlayer = ( pxTargetActor && pxTargetActor->IsPlayer() );

	for ( Asura_LinkList_ConstIt< AVP_AI_TargetPriorityData > xTPIter( m_xTargetPriorityList ); !xTPIter.Done(); xTPIter.Next() )
	{
		const AVP_AI_TargetPriorityData* pxTP = xTPIter.GetCurrent();
		switch ( pxTP->m_iTargetType )
		{
			default:
			{
				if ( !bTargetIsPlayer && pxTP->ClassificationMatchesTargetType( iTargetClassification ) )
				{
					iPriority = pxTP->m_iPriority;
					return true;
				}
				else
				{
					break;
				}
			}
			case AVP_AI_TargetPriorityData::PLAYER:
			{
				if ( bTargetIsPlayer )
				{
					iPriority = pxTP->m_iPriority;
					return true;
				}
				else
				{
					break;
				}
			}
			case AVP_AI_TargetPriorityData::SPECIFIC_ENTITY:
			{
				if ( pxTP->m_uSpecificGuid == uTargetGuid )
				{
					iPriority = pxTP->m_iPriority;
					return true;
				}
				else
				{
					break;
				}
			}
		}
	}

	return false;
}

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////
