//////////////////////////////////////////////////////////////////////////////
//
// AVP_ServerEntity_Squad.h
//
// Copyright (C) 2008 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __AVP_SERVERENTITY_SQUAD_H__
#define __AVP_SERVERENTITY_SQUAD_H__ 1

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// asura
#include "Core/Asura_LinkList.h"
#include "Entity/Asura_Entity.h"

// avp
#include "Entity/Controller/AVP_Controller_Messages.h"

// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
//
// AVP_ServerEntity_Squad
//
//////////////////////////////////////////////////////////////////////////////

class AVP_ServerEntity_Squad : public Asura_Entity_Server
{
	typedef Asura_Entity_Server PARENT;

public:
	AVP_ServerEntity_Squad( Asura_Guid uGuid );
	virtual ~AVP_ServerEntity_Squad() = 0;

	virtual bool CanBeSavedToChunk()	   const { return true; }
	virtual bool IsFilledFromChunkBuffer() const { return false; }
	virtual void ReadFromChunkStream( Asura_Chunk_Stream& xStream );
	virtual void WriteToChunkStream( Asura_Chunk_Stream& xStream ) const;
	virtual bool HandleMessage( const Asura_Message* pxMessage );
	virtual bool HasClientEntities() const { return false; }
	virtual void Update( void );
	virtual void PreDestructionShutdown();

	void AddMember( const Asura_Guid uGuid );
	void RemoveMember( const Asura_Guid uGuid );

	Asura_Guid  GetMemberGuid( int iIndex ) const;
	void*		GetMemberData( const Asura_Guid uGuid ) const;
	u_int	    GetNumSquadMembers() const { return m_uNumMembers; }

	virtual void SendMemberDeadEventMessage( void );


	// Returns whether it has a priority for this target.
	bool CalcTargetingPriority( const int iTargetClassification, const Asura_Guid uTargetGuid, int& iPriority ) const;

#ifdef ASURA_TOOLS
	void SetIsDefaultSquad( bool bIsDefaultSquad ) { m_bIsDefaultSquad = bIsDefaultSquad; }
	void SetIsDefaultPetSquad( bool bIsDefaultPetSquad ) { m_bIsDefaultPetSquad = bIsDefaultPetSquad; }
#endif

	static AVP_ServerEntity_Squad* FindEntity( const Asura_Guid uGuid ) { return s_xList.FindEntity( uGuid ); }

protected:
	virtual void* AllocateMemberData( const Asura_Guid uMemberGuid ) { return NULL; }
	virtual void  ReturnMemberData( void* pData ) {}
	virtual void  SendEventMessages( void );

	bool IsDefaultSquad() const							{ return ( m_bIsDefaultSquad == 0 ) ? false : true; }
	bool IsDefaultPetSquad() const							{ return ( m_bIsDefaultPetSquad == 0 ) ? false : true; }
	void SetHasHadMoreThanOneMember( bool bVal )		{ m_bHasHadMoreThanOneMember = bVal; }
	bool GetHasHadMoreThanOneMember( void ) const		{ return m_bHasHadMoreThanOneMember; }
	void SetElimintedEventSent( bool bVal )				{ m_bEliminatedEventSent = bVal; }
	bool GetElimintedEventSent( void ) const			{ return m_bEliminatedEventSent; }

	struct SquadMember
	{
		Asura_Guid m_uMemberGuid;
		void*	   m_pxMemberData;

		SquadMember() : m_uMemberGuid( ASURA_GUID_UNREGISTERED ), m_pxMemberData( NULL ) {}
		Asura_Guid GetGuid() const { return m_uMemberGuid; }
	};

	const Asura_LinkList< SquadMember >& GetMemberList() const { return m_xMembers; }

private:
	Asura_LinkList< SquadMember > m_xMembers;
	u_int						  m_uNumMembers;

	union
	{
		u_int m_uFlags;
		struct
		{
			u_int	m_bIsDefaultSquad			: 1,
					m_bEliminatedEventSent		: 1,
					m_bHasHadMoreThanOneMember	: 1,
					m_bIsDefaultPetSquad		: 1,
												: 0;
		};
	};

	Asura_LinkList< AVP_AI_TargetPriorityData > m_xTargetPriorityList;

	static const u_int uCURRENT_CHUNK_VERSION = 3;
	static Asura_Hashed_List< AVP_ServerEntity_Squad > s_xList;
};

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

#endif //__AVP_SERVERENTITY_SQUAD_H__
