//////////////////////////////////////////////////////////////////////////////
//
// AVP_ServerEntity_Team.cpp
//
// Copyright (C) 2007 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"

// header
#include "Entity/Controller/AVP_ServerEntity_Team.h"

// asura
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "Core/Memory/Asura_Mem.h"

// avp
#include "Core/BrainTypes/AVP_Brain_Team.h"

//////////////////////////////////////////////////////////////////////////////
//
// AVP_ServerEntity_Team
//
//////////////////////////////////////////////////////////////////////////////

AVP_ServerEntity_Team::AVP_ServerEntity_Team( Asura_Guid uGuid )
: PARENT( uGuid )
, m_pxBrain( NULL )
, m_usNumMembers( 0 )
, m_usStorageSize( 0 )
, m_puMemberGuids( NULL )
{
}

AVP_ServerEntity_Team::~AVP_ServerEntity_Team()
{
    delete [] m_puMemberGuids;
    m_puMemberGuids = NULL;
}

void AVP_ServerEntity_Team::ReadFromChunkStream( Asura_Chunk_Stream& xStream )
{
    int iVersion;
    xStream >> iVersion;

    PARENT::ReadFromChunkStream( xStream );

    if ( iVersion == CURRENT_CHUNK_VERSION )
    {
    }
    else
    {
        Asura_Assertf( false, ( "AVP_ServerEntity_Team::ReadFromChunkStream - unhandled chunk version (%d)", iVersion ) );
    }
}

void AVP_ServerEntity_Team::WriteToChunkStream( Asura_Chunk_Stream& xStream ) const
{
    xStream << CURRENT_CHUNK_VERSION;

    PARENT::WriteToChunkStream( xStream );

    //## Do we need to save member guids into the chunk or not? Are members going to re-register automatically on a chunk load?
}

void AVP_ServerEntity_Team::Register( const Asura_Guid uGuid )
{
	for ( u_short us = 0; us < m_usNumMembers; ++us )
	{
		if ( m_puMemberGuids[ us ] == uGuid )
		{
			return;
		}
	}

	if ( !UpdateStorageSize( m_usNumMembers + 1 ) )
    {
        return;
    }

    m_puMemberGuids[ m_usNumMembers++ ] = uGuid;
}

void AVP_ServerEntity_Team::Unregister( const Asura_Guid uGuid )
{
    for ( u_short us = 0; us < m_usNumMembers; ++us )
    {
        if ( m_puMemberGuids[ us ] == uGuid )
        {
			m_puMemberGuids[ us ] = m_puMemberGuids[ --m_usNumMembers ];
            break;
        }
    }
}

bool AVP_ServerEntity_Team::HasMember( const Asura_Guid uGuid ) const
{
    for ( u_short usMemberIndex = 0; usMemberIndex < m_usNumMembers; ++usMemberIndex )
    {
        if ( m_puMemberGuids[ usMemberIndex ] == uGuid )
        {
            return true;
        }
    }

    return false;
}

void AVP_ServerEntity_Team::InitialiseFromBlueprint( const Asura_Blueprint& xBlueprint )
{
    PARENT::InitialiseFromBlueprint( xBlueprint );

    if ( m_pxBrain )
    {
        m_pxBrain->InitialiseFromBlueprint( xBlueprint );
    }
}

// TODO: switch this with an asura_hashedlist of the actual pointers (AN 10/12/07)
bool AVP_ServerEntity_Team::UpdateStorageSize( const u_short usRequiredSize )
{
    static const u_short s_usGrowSize = 16;

    if ( usRequiredSize > m_usStorageSize )
    {
        do
        {
            m_usStorageSize += s_usGrowSize;
        } while ( m_usStorageSize < usRequiredSize );

        Asura_Guid* puGuidArray = new Asura_Guid[ m_usStorageSize ];
        if ( !puGuidArray ) { return false; }

        for ( u_short usMember = 0; usMember < m_usNumMembers; ++usMember )
        {
            puGuidArray[ usMember ] = m_puMemberGuids[ usMember ];
        }

        delete [] m_puMemberGuids;
        m_puMemberGuids = puGuidArray;
    }

    return true;
}

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////
