//////////////////////////////////////////////////////////////////////////////
//
// AVP_ServerEntity_AlienTeam.cpp
//
// Copyright (C) 2007 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"

// header
#include "Entity/Controller/AVP_ServerEntity_Team_Alien.h"

// asura
#include "Blueprint/Asura_Blueprint_System.h"
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "Network/Asura_Message.h"

// avp
#include "Blueprint/AVP_Blueprint_HashIDs.h"
#include "Entity/AVP_Entity_Classification.h"
#include "Entity/AVP_Entity_EventIDs.h"
#include "Entity/AVP_Messages.h"
#include "Entity/Controller/AVP_Controller_Messages.h"
#include "Entity/Controller/AVP_ServerEntity_Squad_Alien.h"
#include "Entity/GamePlayVolume/AVP_ServerEntity_GamePlayVolume.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Human/Marine/AVP_ServerActor_Marine.h"
#include "Entity/PhysicalObject/Projectile/AVP_ServerProjectile_GenericProjectile.h"
#include "Entity/PhysicalObject/Projectile/AVP_ServerProjectile_HumanFlare.h"
#include "Entity/PlayerInfo/AVP_ServerEntity_PlayerInfo.h"
#include "Entity/Spawners/Alien/AVP_ServerEntity_AlienSpawnPoint.h"
#include "Inventory/Items/AVP_InventoryItem_Torch.h"

// axon
#include "Core/BrainTypes/AVP_Brain_Team_Alien.h"
#include "Knowledge/AVP_Knowledge_System_AlienTeam.h"

//////////////////////////////////////////////////////////////////////////////
//
// AVP_ServerEntity_AlienTeam
//
//////////////////////////////////////////////////////////////////////////////

static const float fDEFAULT_ATTACK_PERIOD = 2.5f;
static const float fBURST_CRAWL_PERIOD_MIN = 2.5f;
static const float fBURST_CRAWL_PERIOD_MAX = 2.5f;
static const float fHALT_CRAWL_PERIOD_MIN = 2.0f;
static const float fHALT_CRAWL_PERIOD_MAX = 10.0f;

AVP_ServerEntity_Team_Alien* AVP_ServerEntity_Team_Alien::s_pxAlienTeam = NULL;

AVP_ServerEntity_Team_Alien::AVP_ServerEntity_Team_Alien( Asura_Guid uGuid ) 
: PARENT( uGuid )
, m_fFlareCheckTimer( 0.0f )
, m_uNumLights( 0 )
, m_uLightGeneration( 0 )
, m_uNumFPThreats( 0 )
, m_fAttackTimer( 0.0f )
, m_fAttackPeriod( fDEFAULT_ATTACK_PERIOD )
, m_fBurstCrawlTimer( 0.0f )
, m_fHaltCrawlTimer( 0.0f )
{
    Classification = AVP_ENTITYCLASS_ALIENTEAM;

    SetNeverUpdateClient();

    m_pxBrain = new AVP_Brain_Team_Alien( this );
    m_pxBrain->Initialise();

    Asura_Assert( !s_pxAlienTeam, "AVP_ServerEntity_AlienTeam::AVP_ServerEntity_AlienTeam - there should only ever be one of these" );
    s_pxAlienTeam = this;
}

AVP_ServerEntity_Team_Alien::~AVP_ServerEntity_Team_Alien()
{
    m_pxBrain->Shutdown();
    delete m_pxBrain;
    m_pxBrain = NULL;

    s_pxAlienTeam = NULL;
}

void AVP_ServerEntity_Team_Alien::PostLoadInitialise()
{
    PARENT::PostLoadInitialise();

    m_pxBrain->PostLoadInitialise();
}

void AVP_ServerEntity_Team_Alien::ReadFromChunkStream( Asura_Chunk_Stream& xStream )
{
    int iVersion;
    xStream >> iVersion;

    PARENT::ReadFromChunkStream( xStream );

    if ( iVersion == CURRENT_CHUNK_VERSION )
    {
        m_pxBrain->ReadFromChunkStream( xStream );
    }
    else if ( iVersion == 0 )
    {
    }
    else
    {
        Asura_Assertf( false, ( "AVP_ServerEntity_AlienTeam::ReadFromChunkStream - bad chunk version (%d)", iVersion ) );
    }

	// Reset the blueprint settings.
	SetBlueprintTypeHashID( ASURA_HASH_ID_UNSET );
	SetBlueprintHashID( ASURA_HASH_ID_UNSET );
}

void AVP_ServerEntity_Team_Alien::WriteToChunkStream( Asura_Chunk_Stream& xStream ) const
{
    xStream << CURRENT_CHUNK_VERSION;

    PARENT::WriteToChunkStream( xStream );

    m_pxBrain->WriteToChunkStream( xStream );
}

bool AVP_ServerEntity_Team_Alien::HandleMessage( const Asura_Message* pxMessage )
{
	switch ( pxMessage->GetMsgID() )
	{
		case AVP_MSGID_CHANGE_ALIEN_SPAWN_SETTINGS:
		case AVP_MSGID_ADD_ALIENS_TO_SPAWN_QUEUE:
		case AVP_MSGID_ALIEN_TEAM_SET_BEHAVIOUR:
		case AVP_MSGID_ALIEN_TEAM_ADD_SPAWNTOKEN:
		case AVP_MSGID_ALIEN_TEAM_REMOVE_SPAWNTOKEN:
		case AVP_MSGID_ALIEN_TEAM_PREPARE_ROOM:
		case AVP_MSGID_ALIEN_TEAM_THREAT_BONUS:
		{
			return true;
		}

		case AVP_MSGID_SET_AI_TARGET_PRIORITIES:
		case AVP_MSGID_RESET_AI_TARGET_PRIORITIES:
		{
			for ( Asura_Hashed_List_It< AVP_ServerEntity_Squad_Alien > xASIt( AVP_ServerEntity_Squad_Alien::GetList() ); !xASIt.Done(); xASIt.Next() )
			{
				AVP_ServerEntity_Squad_Alien* pxAS = xASIt.GetCurrent();
				pxAS->HandleMessage( pxMessage );
			}
			
			return true;
		}
	}

    return PARENT::HandleMessage( pxMessage );
}

bool AVP_ServerEntity_Team_Alien::GetFPThreatData( const u_int uIndex, Asura_Vector_3& xInitialPosition, Asura_Vector_3& xFinalPosition ) const
{
	if ( uIndex >= m_uNumFPThreats )
	{
		Asura_Assert( false, "AVP_ServerEntity_Team_Alien::GetFPThreatData - bad index" );
		return false;
	}

	const FPThreatData& xThreatData = m_axFPThreats[ uIndex ];

	const u_int uFrameNumber = Asura_Timers::GetFrameNumber();
	if ( xThreatData.m_uLastFrameUpdated != uFrameNumber )
	{
		xThreatData.m_uLastFrameUpdated = uFrameNumber;

		const AVP_ServerProjectile_GenericProjectile* pxProj = AVP_ServerProjectile_GenericProjectile::FindEntity( xThreatData.m_uGuid );
		if ( !pxProj ) { return false; }

		xThreatData.m_xPosT = pxProj->GetPosition();
		xThreatData.m_xPosTPlusOne = xThreatData.m_xPosT + pxProj->GetVelocity();
	}

	xInitialPosition = xThreatData.m_xPosT;
	xFinalPosition = xThreatData.m_xPosTPlusOne;
	return true;
}

AVP_ServerEntity_Team_Alien* AVP_ServerEntity_Team_Alien::Create( const Asura_Guid uGuid )
{
    return new AVP_ServerEntity_Team_Alien( uGuid );
}

AVP_ServerEntity_Team_Alien* AVP_ServerEntity_Team_Alien::FindEntity( const Asura_Guid uGuid )
{
    if ( s_pxAlienTeam && uGuid == s_pxAlienTeam->GetGuid() )
    {
        return s_pxAlienTeam;
    }
    else
    {
        return NULL;
    }
}

void AVP_ServerEntity_Team_Alien::Update(void)
{
	PARENT::Update();

	const float fTimeSlice = Asura_Timers::GetGameTimeSlice();
	UpdateLights( fTimeSlice );
	m_fAttackTimer -= fTimeSlice;
	m_fBurstCrawlTimer -= fTimeSlice;
	m_fHaltCrawlTimer -= fTimeSlice;

	UpdateFPThreats();
}

AVP_Brain_Team_Alien* AVP_ServerEntity_Team_Alien::GetAlienTeamBrain() const
{
    return static_cast< AVP_Brain_Team_Alien* >( GetTeamBrain() );
}

AVP_Knowledge_System_AlienTeam* AVP_ServerEntity_Team_Alien::GetKnowledgeSystem() const
{
	return GetAlienTeamBrain()->GetKnowledgeSystem();
}

DEBUGVAR float dbg_fPlayerCutoffDistance = 20.0f; 
DEBUGVAR float dbg_fNonPlayerCutoffDistance = 8.0f; 
DEBUGVAR float dbg_fTourchAngleTan = Asura_Maths::Tangent(Asura_Maths::DegToRad(35.0f/2.0f)); // This is estimate taking the torch image into account

void AVP_ServerEntity_Team_Alien::UpdateLights( const float fTimeslice )
{
	m_uNumLights = 0;

	// Keep the marine players' torch calculations up to date constantly.
	for ( Asura_Hashed_List_ConstIt< AVP_ServerActor_Human > xHumanIter( AVP_ServerActor_Human::GetList() );
		!xHumanIter.Done();
		xHumanIter.Next() )
	{
		const AVP_ServerActor_Human* pxHumanActor = xHumanIter.GetCurrent();
		if ( !pxHumanActor )
		{
			continue;
		}

		// Check to see whether he has his torch switched on.
		const AVP_InventoryItem_Torch* pxTorchItem = pxHumanActor->GetTorchItem();
		if ( !pxTorchItem || !pxTorchItem->IsUsing( false ) )
		{
			continue;
		}

		// Fire a ray from the marine player's torch marker point and work out where it hits the environment.
		Asura_Vector_3   xTorchPosition;
		Asura_Matrix_3x3 xTorchOrient;
		if ( !pxHumanActor->GetWorldSpaceMarkerPoint( AVP_Human_Description::GetMarkerPointHashID( AVP_Human_Description::MPT_TORCH ),
			&xTorchPosition,
			&xTorchOrient ) )
		{
			continue;
		}
		const Asura_Vector_3 xTorchForward = Asura_Vector_3( 0.0f, 0.0f, 1.0f ) * xTorchOrient;

		float fTestDistance = (pxHumanActor->IsPlayer() ? dbg_fPlayerCutoffDistance : dbg_fNonPlayerCutoffDistance);

		Asura_LOS_InputData xInputData( xTorchPosition, xTorchForward, pxHumanActor->GetGuid(), fTestDistance + 1.0f );
		const Asura_LOS_OutputData* pxOutputData = Asura_LOS::TestAgainstEnvironment( &xInputData );
		if ( m_uNumLights < MAX_LIGHTS )
		{
			m_axLightInfo[ m_uNumLights ].m_fDistance = pxOutputData->m_bHitEnvironment ? pxOutputData->m_fDistanceToIntersection : fTestDistance;
			m_axLightInfo[ m_uNumLights ].m_fRadius = m_axLightInfo[ m_uNumLights ].m_fDistance * dbg_fTourchAngleTan;
			m_axLightInfo[ m_uNumLights ].m_xPosition = xTorchPosition + xTorchForward * pxOutputData->m_fDistanceToIntersection;
			m_axLightInfo[ m_uNumLights ].m_xSourcePosition = xTorchPosition;
			m_axLightInfo[ m_uNumLights ].m_fCutoffDistance = (pxHumanActor->IsPlayer() ? dbg_fPlayerCutoffDistance : dbg_fNonPlayerCutoffDistance);
			m_axLightInfo[ m_uNumLights ].m_eType = LightInfo::TORCH;
			++m_uNumLights;
		}
	}

	for ( Asura_Hashed_List_ConstIt< AVP_ServerProjectile_HumanFlare > xFlareIter( AVP_ServerProjectile_HumanFlare::GetList() );
		!xFlareIter.Done();
		xFlareIter.Next() )
	{
 		const AVP_ServerProjectile_HumanFlare* pxFlare = xFlareIter.GetCurrent();

		if ( m_uNumLights < MAX_LIGHTS )
		{
			m_axLightInfo[ m_uNumLights ].m_fRadius = pxFlare->GetFlareAILightRadius() ;
			m_axLightInfo[ m_uNumLights ].m_fDistance = 0.0f;
			m_axLightInfo[ m_uNumLights ].m_xPosition = pxFlare->GetPosition();
			m_axLightInfo[ m_uNumLights ].m_xSourcePosition = pxFlare->GetPosition();
			m_axLightInfo[ m_uNumLights ].m_fCutoffDistance = Asura_Maths::LargeFloat;
			if ( pxFlare->GetPhysicsObject()->GetLinVelocity().MagnitudeSquared() > 1.0f )
			{
				m_axLightInfo[ m_uNumLights ].m_eType = LightInfo::MOVING_FLARE;
			}
			else
			{
				m_axLightInfo[ m_uNumLights ].m_eType = LightInfo::IDLE_FLARE;
			}

			++m_uNumLights;
		}
	}
}

void AVP_ServerEntity_Team_Alien::UpdateFPThreats()
{
	m_uNumFPThreats = 0;

	// This function only goes through and works out what all the FP threats are.
	// The FP threat data itself is lazily-evaluated when users request it.
	for ( Asura_Hashed_List_ConstIt< AVP_ServerProjectile_GenericProjectile > xProjIter( AVP_ServerProjectile_GenericProjectile::GetList() );
		!xProjIter.Done();
		xProjIter.Next() )
	{
		const AVP_ServerProjectile_GenericProjectile* pxProj = xProjIter.GetCurrent();
		if ( !pxProj->IsAvoidedByAI() || !pxProj->IsForwardPredictedByAI() )
		{
			continue;
		}

		if ( m_uNumFPThreats < MAX_FP_THREATS )
		{
			m_axFPThreats[ m_uNumFPThreats ].m_uGuid = pxProj->GetGuid();
			m_axFPThreats[ m_uNumFPThreats ].m_uLastFrameUpdated = 0;
			++m_uNumFPThreats;
		}
		else
		{
			Asura_Assert( false, "AVP_ServerEntity_Team_Alien::UpdateFPThreats - run out of FP threat storage" );
			break;
		}
	}
}

void AVP_ServerEntity_Team_Alien::OnAlienKilled()
{
	// Trigger event
	SendMessagesForEvent( AVP_ENTITY_EVENT_ONALIENKILLED );
}

bool AVP_ServerEntity_Team_Alien::CanAttack( Asura_Guid uTargetGuid ) const
{
	// Now implementing this solely in the squads.
	return true;
	//return ( ( GetKnowledgeSystem()->GetNumAttackers( uTargetGuid ) < 2 ) && ( m_fAttackTimer <= 0.0f ) );
}

void AVP_ServerEntity_Team_Alien::AddAttacker( Asura_Guid uAttackerGuid, const Asura_Guid uTargetGuid )
{
	GetKnowledgeSystem()->AddAttacker( uAttackerGuid, uTargetGuid );
	m_fAttackTimer = m_fAttackPeriod;
}

void AVP_ServerEntity_Team_Alien::RemoveAttacker( Asura_Guid uAttackerGuid, const Asura_Guid uTargetGuid )
{
	GetKnowledgeSystem()->RemoveAttacker( uAttackerGuid, uTargetGuid );
}

void AVP_ServerEntity_Team_Alien::AddTargeter( Asura_Guid uTargeterGuid, const Asura_Guid uTargetGuid )
{
	GetKnowledgeSystem()->AddTargeter( uTargeterGuid, uTargetGuid );
}

void AVP_ServerEntity_Team_Alien::RemoveTargeter( Asura_Guid uTargeterGuid, const Asura_Guid uTargetGuid )
{
	GetKnowledgeSystem()->RemoveTargeter( uTargeterGuid, uTargetGuid );
}

void AVP_ServerEntity_Team_Alien::AddAttackGrapple( Asura_Guid uTargetGuid )
{
	GetKnowledgeSystem()->AddAttackGrapple(uTargetGuid);
}

void AVP_ServerEntity_Team_Alien::AddAttackStand( const Asura_Guid uTargetGuid )
{
	GetKnowledgeSystem()->AddAttackStand(uTargetGuid);
}

void AVP_ServerEntity_Team_Alien::AddAttackJump( Asura_Guid uTargetGuid )
{
	GetKnowledgeSystem()->AddAttackJump(uTargetGuid);
}

void AVP_ServerEntity_Team_Alien::RemoveAttackGrapple( Asura_Guid uTargetGuid )
{
	GetKnowledgeSystem()->RemoveAttackGrapple(uTargetGuid);
}

void AVP_ServerEntity_Team_Alien::RemoveAttackStand( Asura_Guid uTargetGuid )
{
	GetKnowledgeSystem()->RemoveAttackStand(uTargetGuid);
}

void AVP_ServerEntity_Team_Alien::RemoveAttackJump( Asura_Guid uTargetGuid )
{
	GetKnowledgeSystem()->RemoveAttackJump(uTargetGuid);
}

bool AVP_ServerEntity_Team_Alien::GetNumAttackers( Asura_Guid uTargetGuid, u_int& uAttackGrapple, u_int& uAttackStand, u_int& uAttackJump) const
{
	return GetKnowledgeSystem()->GetNumAttackers(uTargetGuid, uAttackGrapple, uAttackStand, uAttackJump);
}

bool AVP_ServerEntity_Team_Alien::DidTargetMovedRecently(Asura_Guid uTargetGuid) const
{
	return GetKnowledgeSystem()->DidTargetMovedRecently(uTargetGuid);
}

float AVP_ServerEntity_Team_Alien::TimeSinceLastGrapple(Asura_Guid uTargetGuid) const
{
	return GetKnowledgeSystem()->TimeSinceLastGrapple(uTargetGuid);
}

u_int AVP_ServerEntity_Team_Alien::GetNumberOfChargers( const Asura_Guid uTargetGuid ) const
{
	return GetKnowledgeSystem()->GetNumChargers(uTargetGuid);
}

void AVP_ServerEntity_Team_Alien::AddCharger( const Asura_Guid uTargetGuid )
{
	GetKnowledgeSystem()->AddCharger(uTargetGuid);
}

void AVP_ServerEntity_Team_Alien::RemoveCharger( const Asura_Guid uTargetGuid )
{
	GetKnowledgeSystem()->RemoveCharger(uTargetGuid);
}


void AVP_ServerEntity_Team_Alien::HaveBurstCrawled()
{
	m_fBurstCrawlTimer = Asura_Random::GetFloat( fBURST_CRAWL_PERIOD_MIN, fBURST_CRAWL_PERIOD_MAX );
}

void AVP_ServerEntity_Team_Alien::HaveHaltCrawled()
{
	m_fHaltCrawlTimer = Asura_Random::GetFloat( fHALT_CRAWL_PERIOD_MIN, fHALT_CRAWL_PERIOD_MAX );
}

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////
