
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_AcidDamage_Messages.h
// Author : Andy Tate
// Date   : 29-Feb-2008
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __AVP_ACIDDAMAGE_MESSAGES_H__
#define __AVP_ACIDDAMAGE_MESSAGES_H__

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Forward class declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Class declaration...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_Message_AcidDamage_Create
// Author       : Andy Tate
// Date         : 28-Feb-2008
//
// Description  : Class to inform the server that it needs to create acid-damage.
////////////////////////////////////////////////////////////////////////////////

class AVP_Message_AcidDamage_Create
{
public:
	// Public functions...
	void		SetSourceGuid(const Asura_Guid uGuid)	{ m_uSourceGuid = uGuid; }
	Asura_Guid	GetSourceGuid(void) const				{ return(m_uSourceGuid); }

	void	SetPosition(const Asura_Vector_3& xPosition)	{ m_xPosition = xPosition; }
	void	GetPosition(Asura_Vector_3& xPosition) const	{ xPosition = m_xPosition; }

	void	SetSurfaceNormal(const Asura_Vector_3& xSurfaceNormal)	{ m_xSurfaceNormal = xSurfaceNormal; }
	void	GetSurfaceNormal(Asura_Vector_3& xSurfaceNormal) const	{ xSurfaceNormal = m_xSurfaceNormal; }

	void	SetIsTrailMode(bool bIsTrail) { m_bIsTrail = bIsTrail; }
	bool	GetIsTrailMode() const { return m_bIsTrail; }

protected:
	// Protected variables...

	// Protected functions...


private:
	// Private variables...
	Asura_Guid		m_uSourceGuid;		// The GUID of the entity which caused this damaged.
	Asura_Vector_3	m_xPosition;		// The position from which the acid-damage is created.
	Asura_Vector_3	m_xSurfaceNormal;	// The surface-normal of the point at which the acid-damage is created.
	bool			m_bIsTrail;			// Use normal AcidDamage or Trail mode

	// Private functions...
	AVP_Message_AcidDamage_Create(void);
	AVP_Message_AcidDamage_Create(const AVP_Message_AcidDamage_Create& xOther);
	~AVP_Message_AcidDamage_Create();

	AVP_Message_AcidDamage_Create&	operator=(const AVP_Message_AcidDamage_Create& xOther);

};   // End of class 'AVP_Message_AcidDamage_Create'.



////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_Message_AcidDamage_ClientUpdate
// Author       : Andy Tate
// Date         : 29-Feb-2008
//
// Description  : Class used to handle the client-update message for the
//				  acid-damage entity.
////////////////////////////////////////////////////////////////////////////////

class AVP_Message_AcidDamage_ClientUpdate
{
public:
	// Public functions...

#ifdef ASURA_SUPPORT_NETMSG_SYSTEM
	static void RegisterFields(u_short usMsgID, u_short usEntityClassification) { Asura_Assert(false, "Attempting to register an empty message AVP_Message_AcidDamage_ClientUpdate"); }
	static void RegisterFields(class Asura_NetMsg* pxNetMsg) { Asura_Assert(false, "Shouldn't get here"); }; 
	static u_char GetNumFields() { return 0; }
#endif//#ifdef ASURA_SUPPORT_NETMSG_SYSTEM

protected:
	// Protected variables...

	// Protected functions...


private:
	// Private variables...

	// Private functions...
	AVP_Message_AcidDamage_ClientUpdate(void);
	AVP_Message_AcidDamage_ClientUpdate(const AVP_Message_AcidDamage_ClientUpdate& xOther);
	~AVP_Message_AcidDamage_ClientUpdate();

	AVP_Message_AcidDamage_ClientUpdate&	operator=(const AVP_Message_AcidDamage_ClientUpdate& xOther);

};   // End of class 'AVP_Message_AcidDamage_ClientUpdate'.

#endif   // ifndef __AVP_ACIDDAMAGE_MESSAGES_H__


/// End of file. ///
