
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_DamageMultiplier.cpp
// Author : Andy Tate
// Date   : 3-Sep-2008
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "Entity/Damage/AVP_DamageMultiplier.h"

// Asura independent header-files...
#include "Blueprint/Asura_Blueprint_System.h"

// Game independent header-files...
#include "Blueprint/AVP_Blueprint_HashIDs.h"


////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Private functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Protected functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Public functions...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_DamageMultiplier::AVP_DamageMultiplier
// Author       : Andy Tate
// Date         : 3-Sep-2008
////////////////////////////////////////////////////////////////////////////////

AVP_DamageMultiplier::AVP_DamageMultiplier(const Asura_Blueprint& xBlueprint)
: m_fMulitplier(1.0f)
{
	Asura_Value	xValue;

	if(xBlueprint.GetParameterValueByHashID(uAVP_BLUEPRINT_PARAM_DAMAGEMULTIPLIER, xValue))
	{
		m_fMulitplier	= xValue.GetFloatValue();
	}
}

/// End of function 'AVP_DamageMultiplier::AVP_DamageMultiplier'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_DamageMultiplier_Blueprint_ParameterEnumerator::BeginEnumeration
// Author       : Andy Tate
// Date         : 3-Sep-2008
////////////////////////////////////////////////////////////////////////////////

void AVP_DamageMultiplier_Blueprint_ParameterEnumerator::BeginEnumeration(void)
{
	// Ensure that our container is empty before enumeration...
	m_xDamageMultiplierContainer.Reset();
}

/// End of function 'AVP_DamageMultiplier_Blueprint_ParameterEnumerator::BeginEnumeration'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_DamageMultiplier_Blueprint_ParameterEnumerator::Enumerate
// Author       : Andy Tate
// Date         : 3-Sep-2008
////////////////////////////////////////////////////////////////////////////////

bool AVP_DamageMultiplier_Blueprint_ParameterEnumerator::Enumerate(const u_int uIndex,
																   char const* const szName,
																   const Asura_Value& xValue)
{
	const Asura_Hash_ID		uDamageMultiplierHash	= xValue.GetHashIDValue();

	if(uDamageMultiplierHash == ASURA_HASH_ID_UNSET)
	{
		// We may always have a default damage-multiplier - if we do, treat it as being enumerated successfully...
		return(true);
	}
	else
	{
		const Asura_Blueprint*	pxBlueprint	= Asura_Blueprint_System::GetBlueprintByHashID(uAVP_BLUEPRINT_TYPE_DAMAGE, uDamageMultiplierHash);

		if(pxBlueprint == NULL)
		{
#if defined(ASURA_TOOLS)
			Asura_Assertf(false, ("Cannot find blueprint for damage-multiplier \"%s\" during enumeration.", xValue.GetHashStringValue()));
#else
			Asura_Assertf(false, ("Cannot find blueprint for damage-multiplier %08X during enumeration.", uDamageMultiplierHash));
#endif	//defined(ASURA_TOOLS)
			return(false);
		}
		else
		{
			Asura_Value	xLocalValue;

			if(pxBlueprint->GetParameterValueByHashID(uAVP_BLUEPRINT_PARAM_DAMAGETYPE, xLocalValue))
			{
				const Asura_Hash_ID	uDamageTypeHash	= xLocalValue.GetHashIDValue();

				if(uDamageTypeHash == ASURA_HASH_ID_UNSET)
				{
#if defined(ASURA_TOOLS)
					Asura_Assertf(false, ("Damage-type is not set for damage-multiplier \"%s\".", xValue.GetHashStringValue()));
#else
					Asura_Assertf(false, ("Damage-type is not set for damage-multiplier %08X.", uDamageMultiplierHash));
#endif	//defined(ASURA_TOOLS)
					return(false);
				}
				else
				{
					AVP_DamageMultiplier*	pxDamageMultiplier	= m_xDamageMultiplierContainer.GetDataByHash(uDamageTypeHash);

					if(pxDamageMultiplier == NULL)
					{
						const AVP_DamageMultiplier	xDamageMultiplier(*pxBlueprint);

						pxDamageMultiplier	= m_xDamageMultiplierContainer.AddNode(uDamageTypeHash, xDamageMultiplier);
						return(pxDamageMultiplier != NULL);
					}
					else
					{
#if defined(ASURA_TOOLS)
						Asura_Assertf(false, ("Damage-type \"%s\" (in damage-multiplier \"%s\") has already been specified during enumeration.", xLocalValue.GetHashStringValue(), xValue.GetHashStringValue()));
#else
						Asura_Assertf(false, ("Damage-type %08X (in damage-multiplier %08X) has already been specified during enumeration.", xLocalValue.GetHashIDValue(), xValue.GetHashIDValue()));
#endif	//defined(ASURA_TOOLS)
						return(false);
					}
				}
			}
			else
			{
#if defined(ASURA_TOOLS)
				Asura_Assertf(false, ("Failed to find \"DamageType\" parameter in blueprint \"%s\".", xValue.GetHashStringValue()));
#else
				Asura_Assertf(false, ("Failed to find \"DamageType\" parameter in blueprint %08X.", xValue.GetHashIDValue()));
#endif	//defined(ASURA_TOOLS)
				return(false);
			}
		}
	}
}

/// End of function 'AVP_DamageMultiplier_Blueprint_ParameterEnumerator::Enumerate'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_DamageMultiplier_Blueprint_ParameterEnumerator::EndEnumeration
// Author       : Andy Tate
// Date         : 3-Sep-2008
////////////////////////////////////////////////////////////////////////////////

void AVP_DamageMultiplier_Blueprint_ParameterEnumerator::EndEnumeration(const bool bSuccess)
{
}

/// End of function 'AVP_DamageMultiplier_Blueprint_ParameterEnumerator::EndEnumeration'. ///


/// End of file. ///
