//--------------------------------------------------------------------------------------
//	File	:	AVP_ServerEntity_GamePlayVolume.cpp
//	Created	:	22 / 10 / 07
//	Author	:	Gaz Thomas
//	Copyright (C) 2006 Rebellion, All Rights Reserved.
//--------------------------------------------------------------------------------------

//--------------------------------------------------------------------------------------
//	INCLUDES
//--------------------------------------------------------------------------------------

// Core
#include "Core/Asura.h"

// This Files Header
#include "Entity/GamePlayVolume/AVP_ServerEntity_GamePlayVolume.h"

//	Asura
#include "Blueprint/Asura_Blueprint_System.h"
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "Entity/DynamicLight/Asura_ServerEntity_DynamicLight.h"
#include "Entity/LightShaft/Asura_ServerEntity_LightShaft.h"
#include "Entity/ParticleGenerator/Asura_ServerEntity_PFX_Effect.h"
#include "Network/Asura_Message.h"
#include "Sound/Asura_Phonon.h"
#include "Volume/Asura_Volume_Cuboid.h"
#include "Volume/Asura_Volume.h"
#include "Volume/Asura_Volume_Sphere.h"
#include "Volume/Asura_Volume_Cylinder.h"
#include "Volume/Asura_VolumeGroup.h"

//	AVP
#include "Entity/AVP_Entity_Classification.h"
#include "Entity/AVP_Messages.h"
#include "Entity/Locations/FleePoint/AVP_ServerNode_FleePoint.h"
#include "Entity/Locations/FormationPoint/AVP_ServerNode_FormationPoint.h"
#include "Entity/Locations/GuardZone/AVP_ServerNode_GuardZone.h"
#include "Entity/Locations/LookAt/AVP_ServerEntity_LookAt.h"
#include "Entity/Locations/OrientedNode/CoverpointNode/AVP_ServerNode_CoverpointNode.h"
#include "Entity/Locations/OrientedNode/GamesceneNode/AVP_ServerNode_GS2Node.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Alien/AVP_ServerActor_Alien.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/FaceHugger/AVP_ServerActor_FaceHugger.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Human/AVP_ServerActor_Human.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Human/Android/AVP_ServerActor_Android.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Human/Civilian/AVP_ServerActor_Civilian.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Human/Marine/AVP_ServerActor_Marine.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Predator/AVP_ServerActor_Predator.h"
#include "Entity/PhysicalObject/Actor/Weapons/Sentrygun/AVP_ServerWeapon_Sentrygun.h"
#include "Entity/PhysicalObject/Lift/AVP_ServerEntity_Lift.h"
#include "Entity/PhysicalObject/PhysicsObject/AVP_ServerEntity_PhysicsObject.h"
#include "Entity/Spawners/AVP_ServerEntity_SpawnPoint2.h"
#include "Entity/Spawners/Alien/AVP_ServerEntity_AlienSpawnPoint.h"
#include "Entity/Spawners/Human/AVP_ServerEntity_HumanSpawnPoint.h"
#include "Entity/Triggers/AVP_ServerEntity_MultiDamageVolume.h"
#include "Entity/Triggers/AVP_ServerEntity_VolumeTrigger.h"

#include "Entity/Container/Asura_ServerEntity_Container.h"

#ifdef ASURA_TOOLS
#include "EntityTemplate/Tools_EntityTemplate_System.h"
#include "Misc/Tools_Render.h"
#endif

//--------------------------------------------------------------------------------------
//	Statics
//--------------------------------------------------------------------------------------
Asura_Hashed_List<AVP_ServerEntity_GamePlayVolume> AVP_ServerEntity_GamePlayVolume::s_xList;

//--------------------------------------------------------------------------------------
//	Function	:	AVP_ServerEntity_GamePlayVolume
//	Created		:	28 / 11 / 07
//	Author		:	Gareth Thomas
//--------------------------------------------------------------------------------------
const char* AVP_ServerEntity_GamePlayVolume::s_aszAreaTypeNames[MAX_AREA_TYPES]=
{
	"NONE ASSIGNED",
	"TIGHT CORRIDOR",
	"LARGE CORRIDOR",
	"SMALL ROOM",
	"LARGE ROOM",
	"OPEN SPACE",
	"IGNORE",
};

//--------------------------------------------------------------------------------------
//	Function	:	AVP_ServerEntity_GamePlayVolume
//	Created		:	26 / 10 / 07
//	Author		:	Gareth Thomas
//--------------------------------------------------------------------------------------
AVP_ServerEntity_GamePlayVolume::AVP_ServerEntity_GamePlayVolume(const Asura_Guid uGuid)
: PARENT(uGuid)
, m_pxVolumeGroup(NULL), m_iNumVolumes(0)
, m_pxAlienSpawnerGuids(NULL), m_uNumAlienSpawnerGuids(0)
, m_pxHumanoidSpawnerGuids(NULL), m_uNumHumanoidSpawnerGuids(0)
, m_pxSpawnPointGuids(NULL), m_uNumSpawnPointGuids(0)
, m_pxCoverPointGuids(NULL), m_uNumCoverPointGuids(0)
, m_pxGuardPointGuids(NULL), m_uNumGuardPointGuids(0)
, m_pxFormationPointGuids(NULL), m_uNumFormationPointGuids(0)
, m_pxLookAtPointGuids(NULL), m_uNumLookAtPointGuids(0)
, m_pxFleePointGuids(NULL), m_uNumFleePointGuids(0)
, m_pxGS2Guids(NULL) , m_uNumGS2Guids(0)
, m_xPosition(Asura_Maths::ZeroVector3)
, m_eGamePlayAreaType(LARGE_ROOM)
, m_iBitFlags(0)
, m_uNumLinkedGamePlayVolumes(0)
, m_uFlareBlueprintHash( 0x2c8dd037 ) //"FlareProjectile"
{
	Classification = AVP_ENTITYCLASS_GAMEPLAYVOLUME;

    // Doesn't have a client and doesn't need to update.
	SetNeverUpdateClient();
    SetNeverUpdates(true);

	m_iNumVolumes = 0;

	m_pxVolumeGroup = new Asura_VolumeGroup;

	for ( int i = 0; i < MAX_LINKED_GAMEPLAYVOLUMES; ++i )
	{
		m_puLinkedGamePlayVolumes[ i ] = ASURA_GUID_UNREGISTERED;
	}

	s_xList.AddToList(this);
}


//--------------------------------------------------------------------------------------
//	Function	:	AVP_ServerEntity_GamePlayVolume
//	Created		:	26 / 10 / 07
//	Author		:	Gareth Thomas
//--------------------------------------------------------------------------------------
AVP_ServerEntity_GamePlayVolume::~AVP_ServerEntity_GamePlayVolume()
{
	delete m_pxVolumeGroup;
	m_pxVolumeGroup = NULL;
	delete [] m_pxAlienSpawnerGuids;
	m_pxAlienSpawnerGuids = NULL;
	delete [] m_pxHumanoidSpawnerGuids;
	m_pxHumanoidSpawnerGuids = NULL;
	delete [] m_pxSpawnPointGuids;
	m_pxSpawnPointGuids = NULL;
	delete [] m_pxCoverPointGuids;
	m_pxCoverPointGuids = NULL;
	delete [] m_pxGuardPointGuids;
	m_pxGuardPointGuids = NULL;
	delete [] m_pxFormationPointGuids;
	m_pxFormationPointGuids = NULL;
	delete [] m_pxLookAtPointGuids;
	m_pxLookAtPointGuids = NULL;
	delete [] m_pxFleePointGuids;
	m_pxFleePointGuids = NULL;
	delete [] m_pxGS2Guids;
	m_pxGS2Guids = NULL;

	s_xList.RemoveFromList(this);
}


//--------------------------------------------------------------------------------------
//	Function	:	AVP_ServerEntity_GamePlayVolume
//	Created		:	26 / 10 / 07
//	Author		:	Gareth Thomas
//--------------------------------------------------------------------------------------
AVP_ServerEntity_GamePlayVolume* AVP_ServerEntity_GamePlayVolume::Create(const Asura_Guid uGuid)
{
	return(new AVP_ServerEntity_GamePlayVolume(uGuid));
}


//--------------------------------------------------------------------------------------
//	Function	:	AVP_ServerEntity_GamePlayVolume
//	Created		:	26 / 10 / 07
//	Author		:	Gareth Thomas
//--------------------------------------------------------------------------------------
void AVP_ServerEntity_GamePlayVolume::WriteToChunkStream(Asura_Chunk_Stream& xStream) const
{
	//Version
	xStream << CURRENT_CHUNK_VERSION;

	//Parent
	PARENT::WriteToChunkStream(xStream);

	//our data
	xStream << m_xPosition;
	xStream << m_iNumVolumes;

	m_pxVolumeGroup->WriteToChunkStream(xStream);

	xStream << m_uNumLinkedGamePlayVolumes;

	for (u_int i = 0; i < m_uNumLinkedGamePlayVolumes; i += 1)
	{
		xStream << m_puLinkedGamePlayVolumes[i];
	}

	xStream << m_eGamePlayAreaType;

	xStream << m_iBitFlags;
}


//--------------------------------------------------------------------------------------
//	Function	:	AVP_ServerEntity_GamePlayVolume
//	Created		:	26 / 10 / 07
//	Author		:	Gareth Thomas
//--------------------------------------------------------------------------------------
void AVP_ServerEntity_GamePlayVolume::ReadFromChunkStream(Asura_Chunk_Stream& xStream)
{
	//Version
	u_int uVersion;
	xStream	>> uVersion;

	//Parent
	PARENT::ReadFromChunkStream(xStream);

	// OLD VERSION
    if (uVersion == 0)
    {
		//our data
		xStream >> m_xPosition;
		xStream >> m_iNumVolumes;

		Asura_OrientedVolume* pxOldVolumes = new Asura_OrientedVolume[m_iNumVolumes];

		// Need to covert the old volumes to the new volumes
		for (int i = 0; i < m_iNumVolumes; ++i)
		{
			// Get the old data
			xStream >> pxOldVolumes[i].m_xBox;
			xStream >> pxOldVolumes[i].m_xOrient;

			// Create a New Cuboid Volume Asura_Volume_Cuboid
			class Asura_Volume_Cuboid* pxNewVolume = new class Asura_Volume_Cuboid(pxOldVolumes[i].m_xBox, pxOldVolumes[i].m_xOrient);

			m_pxVolumeGroup->AddVolume(pxNewVolume);
		}

		delete [] pxOldVolumes;

		xStream >> m_uNumLinkedGamePlayVolumes;

		for (u_int i = 0; i < m_uNumLinkedGamePlayVolumes; ++i)
		{
			xStream >> m_puLinkedGamePlayVolumes[i];
		}

		xStream >> m_eGamePlayAreaType;

		m_iBitFlags = 0;
	}
	else if (uVersion == 1)
	{
		//our data
		xStream >> m_xPosition;
		xStream >> m_iNumVolumes;

		m_pxVolumeGroup->ReadFromChunkStream(xStream);

		xStream >> m_uNumLinkedGamePlayVolumes;

		for (u_int i = 0; i < m_uNumLinkedGamePlayVolumes; ++i)
		{
			xStream >> m_puLinkedGamePlayVolumes[i];
		}

		xStream >> m_eGamePlayAreaType;

		m_iBitFlags = 0;
	}
	else if (uVersion == 2)
	{
		//our data
		xStream >> m_xPosition;
		xStream >> m_iNumVolumes;

		m_pxVolumeGroup->ReadFromChunkStream(xStream);

		xStream >> m_uNumLinkedGamePlayVolumes;

		for (u_int i = 0; i < m_uNumLinkedGamePlayVolumes; ++i)
		{
			xStream >> m_puLinkedGamePlayVolumes[i];
		}

		xStream >> m_eGamePlayAreaType;

		xStream >> m_iBitFlags;
	}
	else if (uVersion == 3)
	{
		//our data
		xStream >> m_xPosition;
		xStream >> m_iNumVolumes;

		m_pxVolumeGroup->ReadFromChunkStream(xStream);

		xStream >> m_uNumLinkedGamePlayVolumes;

		for (u_int i = 0; i < m_uNumLinkedGamePlayVolumes; ++i)
		{
			xStream >> m_puLinkedGamePlayVolumes[i];
		}

		xStream >> m_eGamePlayAreaType;

		xStream >> m_iBitFlags;

		Asura_Hash_ID uDummyFlareBPHash;
		xStream >> uDummyFlareBPHash;
	}
	else if (uVersion == 4)
	{
		//our data
		xStream >> m_xPosition;
		xStream >> m_iNumVolumes;

		m_pxVolumeGroup->ReadFromChunkStream(xStream);

		xStream >> m_uNumLinkedGamePlayVolumes;

		for (u_int i = 0; i < m_uNumLinkedGamePlayVolumes; ++i)
		{
			xStream >> m_puLinkedGamePlayVolumes[i];
		}

		xStream >> m_eGamePlayAreaType;

		xStream >> m_iBitFlags;
	}
	else if ((uVersion > CURRENT_CHUNK_VERSION))
    {
        Asura_Assertf(false, ("AVP_ServerEntity_GamePlayVolume::ReadFromChunkStream - unhandled chunk version (%u)", uVersion));
    }
}

//--------------------------------------------------------------------------------------
//	Function	:	AVP_ServerEntity_GamePlayVolume
//	Created		:	18 / 04 / 07
//	Author		:	Gareth Thomas
//--------------------------------------------------------------------------------------
bool AVP_ServerEntity_GamePlayVolume::HandleMessage(const Asura_Message* pxMessage)
{
	switch (pxMessage->GetMsgID())
	{
		case AVP_MSGID_ACTIVATE_SELECTED_ENTITIES:
		{
			ActivateSelectedEntities(true);
			return true;
		}

		case AVP_MSGID_DEACTIVATE_SELECTED_ENTITIES:
		{
			ActivateSelectedEntities(false);
			return true;
		}

		default:
		{
			return PARENT::HandleMessage(pxMessage);
		}
	}
}

//--------------------------------------------------------------------------------------
//	Function	:	AVP_ServerEntity_GamePlayVolume
//	Created		:	18 / 04 / 07
//	Author		:	Gareth Thomas
//--------------------------------------------------------------------------------------
void AVP_ServerEntity_GamePlayVolume::ActivateSelectedEntities(bool bActivate)
{
	Asura_Message* pxMessage = NULL;

	if (bActivate)
	{
		pxMessage = Asura_Network::CreateMessage(ASURA_MSG_ACTIVATE);
	}
	else
	{
		pxMessage = Asura_Network::CreateMessage(ASURA_MSG_DEACTIVATE);
	}

	if ( IncludeContainers() || IncludeAllTypes() )
	{
		for (Asura_Hashed_List_It<Asura_ServerEntity_Container> it(Asura_ServerEntity_Container::GetList()); !it.Done(); it.Next())
		{
			Asura_ServerEntity_Container* pxEntity = it.GetCurrent();

			if (pxEntity)
			{
				if (Contains(pxEntity->GetPosition()))
				{
					pxEntity->HandleMessage(pxMessage);
				}
			}
		}
	}

	if (IncludeTriggers() || IncludeAllTypes())
	{
		for (Asura_Hashed_List_It<AVP_ServerEntity_VolumeTrigger> it(AVP_ServerEntity_VolumeTrigger::GetList()); !it.Done(); it.Next())
		{
			AVP_ServerEntity_VolumeTrigger* pxEntity = it.GetCurrent();

			if (pxEntity)
			{
				bool bIsInVolume = true;

				for (u_int u = 0; u < pxEntity->GetNumberOfVolumes(); ++u)
				{
					Asura_Bounding_Box xBB;

					pxEntity->GetVolume(u)->GetAABB(xBB);

					Asura_Vector_3 xCentre = xBB.CalculateCentre();

					if (!Contains(xCentre))
					{
						bIsInVolume = false;
						break;
					}
				}

				if (bIsInVolume)
				{
					pxEntity->HandleMessage(pxMessage);
				}
			}
		}

		for (Asura_Hashed_List_It<AVP_ServerEntity_MultiDamageVolume> it(AVP_ServerEntity_MultiDamageVolume::GetList()); !it.Done(); it.Next())
		{
			AVP_ServerEntity_MultiDamageVolume* pxEntity = it.GetCurrent();

			if (pxEntity)
			{
				bool bIsInVolume = true;

				const Asura_VolumeGroup* pxVolGroup = pxEntity->GetVolumeGroup();

				for (u_int u = 0; u < pxVolGroup->GetNumberOfVolumes(); ++u)
				{
					Asura_Bounding_Box xBB;

					pxVolGroup->GetVolume(u)->GetAABB(xBB);

					Asura_Vector_3 xCentre = xBB.CalculateCentre();

					if (!Contains(xCentre))
					{
						bIsInVolume = false;
						break;
					}
				}

				if (bIsInVolume)
				{
					pxEntity->HandleMessage(pxMessage);
				}
			}
		}
	}

	if (IncludeLights() || IncludeAllTypes())
	{
		// Asura_ServerEntity_DynamicLight
		for (Asura_Hashed_List_It<Asura_ServerEntity_DynamicLight> it(Asura_ServerEntity_DynamicLight::GetList()); !it.Done(); it.Next())
		{
			Asura_ServerEntity_DynamicLight* pxEntity = it.GetCurrent();

			if (pxEntity)
			{
				if (Contains(pxEntity->GetParams().m_xPosition))
				{
					pxEntity->HandleMessage(pxMessage);
				}
			}
		}
		
		// Asura_ServerEntity_LightShaft
		for (Asura_Hashed_List_It<Asura_ServerEntity_LightShaft> it(Asura_ServerEntity_LightShaft::GetList()); !it.Done(); it.Next())
		{
			Asura_ServerEntity_LightShaft* pxEntity = it.GetCurrent();

			if (pxEntity)
			{
				if (Contains(pxEntity->GetParams().m_xPosition))
				{
					pxEntity->HandleMessage(pxMessage);
				}
			}
		}
	}

	// PHYSICAL OBJECT
	if (IncludePhysicsObjects() || IncludeAllTypes())
	{
		for (Asura_Hashed_List_It<AVP_ServerEntity_PhysicsObject> it(AVP_ServerEntity_PhysicsObject::GetList()); !it.Done(); it.Next())
		{
			AVP_ServerEntity_PhysicsObject* pxEntity = it.GetCurrent();

			if (pxEntity)
			{
				if (Contains(pxEntity->GetPosition()))
				{
					pxEntity->HandleMessage(pxMessage);
				}
			}
		}
	}

	// PREDATORS
	if (IncludePredators() || IncludeAllTypes() || IncludeAllActors())
	{
		// AVP_ServerActor_Predator
		for (Asura_Hashed_List_It<AVP_ServerActor_Predator> it(AVP_ServerActor_Predator::GetList()); !it.Done(); it.Next())
		{
			AVP_ServerActor_Predator* pxEntity = it.GetCurrent();

			if (pxEntity && !pxEntity->IsPlayer() && !pxEntity->IsDead())
			{
				if (Contains(pxEntity->GetPosition()))
				{
					pxEntity->HandleMessage(pxMessage);
				}
			}
		}
	}

	// ALIENS
	if (IncludeAliens() || IncludeAllTypes() || IncludeAllActors())
	{
		for (Asura_Hashed_List_It<AVP_ServerActor_Alien> it(AVP_ServerActor_Alien::GetList()); !it.Done(); it.Next())
		{
			AVP_ServerActor_Alien* pxEntity = it.GetCurrent();

			if (pxEntity && !pxEntity->IsPlayer() && !pxEntity->IsDead())
			{
				if (Contains(pxEntity->GetPosition()))
				{
					pxEntity->HandleMessage(pxMessage);
				}
			}
		}
	}
	
	// ALL HUMAN TYPES
	if (IncludeHumans() || IncludeAllTypes() || IncludeAllActors())
	{
		for (Asura_Hashed_List_It<AVP_ServerActor_Human> it(AVP_ServerActor_Human::GetList()); !it.Done(); it.Next())
		{
			AVP_ServerActor_Human* pxEntity = it.GetCurrent();

			if (pxEntity && !pxEntity->IsPlayer() && !pxEntity->IsDead())
			{
				if (Contains(pxEntity->GetPosition()))
				{
					pxEntity->HandleMessage(pxMessage);
				}
			}
		}
	}
	
	// LIFTS
	if (IncludeLifts() || IncludeAllTypes())
	{
		// AVP_ServerEntity_Lift
		for (Asura_Hashed_List_It<AVP_ServerEntity_Lift> it(AVP_ServerEntity_Lift::GetList()); !it.Done(); it.Next())
		{
			AVP_ServerEntity_Lift* pxEntity = it.GetCurrent();

			if (pxEntity)
			{
				if (Contains(pxEntity->GetPosition()))
				{
					pxEntity->HandleMessage(pxMessage);
				}
			}
		}
	}

	// PFX
	if (IncludePFX() || IncludeAllTypes())
	{
		// Asura_ServerEntity_PFX_Effect
		for (Asura_Hashed_List_It<Asura_ServerEntity_PFX_Effect> it(Asura_ServerEntity_PFX_Effect::GetList()); !it.Done(); it.Next())
		{
			Asura_ServerEntity_PFX_Effect* pxEntity = it.GetCurrent();

			if (pxEntity)
			{
				if (Contains(pxEntity->GetPosition()))
				{
					pxEntity->HandleMessage(pxMessage);
				}
			}
		}
	}

	// PHONONS
	if (IncludeSounds() || IncludeAllTypes())
	{
// 		for (int i = 0; i < Asura_Phonon_System::GetNumberOfPhonons(); ++i)
// 		{
// 			Asura_Phonon* pxPhonon = Asura_Phonon_System::GetPhononByIndex( i );
// 
// 			if (pxPhonon)
// 			{
// 				if (Contains(pxPhonon->GetPosition()))
// 				{
// 					if (bActivate)
// 					{
// 						pxPhonon->Activate();
// 					}
// 					else
// 					{
// 						pxPhonon->Deactivate();
// 					}
// 				}
// 			}
// 		}
	}

	// FACE HUGGERS
	if (IncludeFaceHuggers() || IncludeAllTypes() || IncludeAllActors())
	{
		// AVP_ServerActor_FaceHugger
		for (Asura_Hashed_List_It<AVP_ServerActor_FaceHugger> it(AVP_ServerActor_FaceHugger::GetList()); !it.Done(); it.Next())
		{
			AVP_ServerActor_FaceHugger* pxEntity = it.GetCurrent();

			if (pxEntity && !pxEntity->IsPlayer() && !pxEntity->IsDead())
			{
				if (Contains(pxEntity->GetPosition()))
				{
					pxEntity->HandleMessage(pxMessage);
				}
			}
		}
	}

	// SENTRY GUNS
	if (IncludeSentryguns() || IncludeAllTypes() || IncludeAllActors())
	{
		for (Asura_Hashed_List_It<AVP_ServerWeapon_Sentrygun> it(AVP_ServerWeapon_Sentrygun::GetList()); !it.Done(); it.Next())
		{
			AVP_ServerWeapon_Sentrygun* pxEntity = it.GetCurrent();

			if (pxEntity && !pxEntity->IsPlayer() && !pxEntity->IsDead())
			{
				if (Contains(pxEntity->GetPosition()))
				{
					pxEntity->HandleMessage(pxMessage);
				}
			}
		}
	}

	// CIVILIANS
	if (IncludeCivilains() || IncludeAllTypes())
	{
		for (Asura_Hashed_List_It<AVP_ServerActor_Civilian> it(AVP_ServerActor_Civilian::GetList()); !it.Done(); it.Next())
		{
			AVP_ServerActor_Civilian* pxEntity = it.GetCurrent();

			if (pxEntity && !pxEntity->IsPlayer() && !pxEntity->IsDead())
			{
				if (Contains(pxEntity->GetPosition()))
				{
					pxEntity->HandleMessage(pxMessage);
				}
			}
		}
	}

	// ANDROIDS
	if (IncludeAndroids() || IncludeAllTypes())
	{
		for (Asura_Hashed_List_It<AVP_ServerActor_Android> it(AVP_ServerActor_Android::GetList()); !it.Done(); it.Next())
		{
			AVP_ServerActor_Android* pxEntity = it.GetCurrent();

			if (pxEntity && !pxEntity->IsPlayer() && !pxEntity->IsDead())
			{
				if (Contains(pxEntity->GetPosition()))
				{
					pxEntity->HandleMessage(pxMessage);
				}
			}
		}
	}

	// MARINES
	if (IncludeMarines() || IncludeAllTypes())
	{
		for (Asura_Hashed_List_It<AVP_ServerActor_Marine> it(AVP_ServerActor_Marine::GetList()); !it.Done(); it.Next())
		{
			AVP_ServerActor_Marine* pxEntity = it.GetCurrent();

			if (pxEntity && !pxEntity->IsPlayer() && !pxEntity->IsDead())
			{
				if (Contains(pxEntity->GetPosition()))
				{
					pxEntity->HandleMessage(pxMessage);
				}
			}
		}
	}

	Asura_Network::DestroyMessage(pxMessage);
}

//--------------------------------------------------------------------------------------
//	Function	:	AVP_ServerEntity_GamePlayVolume
//	Created		:	28 / 11 / 07
//	Author		:	Nick Davis
//--------------------------------------------------------------------------------------
void AVP_ServerEntity_GamePlayVolume::PostLoadInitialise()
{
    PARENT::PostLoadInitialise();

	// Build the game play volumes list of things we are concerned with.
    BuildAlienSpawnGuidArray();
	BuildHumanoidSpawnGuidArray();
	BuildSpawnPointArray();
    BuildCoverPointGuidArray();
    BuildGuardPointGuidArray();
	BuildFormationPointGuidArray();
	BuildLookAtPointGuidArray();
	BuildFleePointGuidArray();
	BuildGS2GuidArray();
	SetFlareBPHashFromAreaType();

	if ( GetNumberOfVolumes() == 0 )
	{
		Asura_Assertf(false, ( "Game Play Volume with Guid %u has NO VOLUME!", GetGuid() ) );
	}
}

//--------------------------------------------------------------------------------------
//	Function	:	AVP_ServerEntity_GamePlayVolume
//	Created		:	13 / 05 / 08
//	Author		:	Gareth Thomas
//--------------------------------------------------------------------------------------
void AVP_ServerEntity_GamePlayVolume::BuildGS2GuidArray(void)
{
	m_uNumGS2Guids	= 0;

	for (Asura_Hashed_List_It<AVP_ServerNode_GS2Node> it(AVP_ServerNode_GS2Node::GetList()); !it.Done(); it.Next())
	{
		AVP_ServerNode_GS2Node* pxNode = it.GetCurrent();

		if (pxNode)
		{
			if (Contains(pxNode->GetPosition()))
			{
				m_uNumGS2Guids += 1;
			}
		}
	}

	// Allocate the array
	m_pxGS2Guids = new Asura_Guid[m_uNumGS2Guids];

	u_int iGS2Index = 0;

	// Now iterate through and add the Guids to the array
	for (Asura_Hashed_List_It<AVP_ServerNode_GS2Node> it(AVP_ServerNode_GS2Node::GetList()); !it.Done(); it.Next())
	{
		AVP_ServerNode_GS2Node* pxNode = it.GetCurrent();

		if (pxNode)
		{
			// does this volume contain this node?
			if (Contains(pxNode->GetPosition()))
			{
				// Add the guid
				m_pxGS2Guids[iGS2Index] = pxNode->GetGuid();
				// Increment the array index
				iGS2Index += 1;
			}
		}
	}
}

//--------------------------------------------------------------------------------------
//	Function	:	AVP_ServerEntity_GamePlayVolume
//	Created		:	29 / 11 / 07
//	Author		:	Gareth Thomas
//--------------------------------------------------------------------------------------
void AVP_ServerEntity_GamePlayVolume::BuildFormationPointGuidArray()
{
	m_uNumFormationPointGuids	= 0;

	for (Asura_Hashed_List_It<AVP_ServerNode_FormationPoint> it(AVP_ServerNode_FormationPoint::GetList()); !it.Done(); it.Next())
	{
		AVP_ServerNode_FormationPoint* pxNode = it.GetCurrent();

		if (pxNode)
		{
			if (Contains(pxNode->GetPosition()))
			{
				m_uNumFormationPointGuids += 1;
			}
		}
	}

	// Allocate the array
	m_pxFormationPointGuids = new Asura_Guid[m_uNumFormationPointGuids];

	u_int iFormationPointIndex = 0;

	// Now iterate through and add the Guids to the array
	for (Asura_Hashed_List_It<AVP_ServerNode_FormationPoint> it(AVP_ServerNode_FormationPoint::GetList()); !it.Done(); it.Next())
	{
		AVP_ServerNode_FormationPoint* pxNode = it.GetCurrent();

		if (pxNode)
		{
			// does this volume contain this node?
			if (Contains(pxNode->GetPosition()))
			{
				// Add the guid
				m_pxFormationPointGuids[iFormationPointIndex] = pxNode->GetGuid();
				// Increment the array index
				iFormationPointIndex += 1;
			}
		}
	}
}

//--------------------------------------------------------------------------------------
//	Function	:	AVP_ServerEntity_GamePlayVolume
//	Created		:	18 / 12 / 07
//	Author		:	Gareth Thomas
//--------------------------------------------------------------------------------------
void AVP_ServerEntity_GamePlayVolume::BuildLookAtPointGuidArray()
{
	m_uNumLookAtPointGuids	= 0;

	for (Asura_Hashed_List_It<AVP_ServerEntity_LookAt> it(AVP_ServerEntity_LookAt::GetList()); !it.Done(); it.Next())
	{
		AVP_ServerEntity_LookAt* pxNode = it.GetCurrent();

		if (pxNode)
		{
			if (Contains(pxNode->GetPosition()))
			{
				m_uNumLookAtPointGuids += 1;
			}
		}
	}

	// Allocate the array
	m_pxLookAtPointGuids = new Asura_Guid[m_uNumLookAtPointGuids];

	u_int iLookAtPointIndex = 0;

	// Now iterate through and add the Guids to the array
	for (Asura_Hashed_List_It<AVP_ServerEntity_LookAt> it(AVP_ServerEntity_LookAt::GetList()); !it.Done(); it.Next())
	{
		AVP_ServerEntity_LookAt* pxNode = it.GetCurrent();

		if (pxNode)
		{
			// does this volume contain this node?
			if (Contains(pxNode->GetPosition()))
			{
				// Add the guid
				m_pxLookAtPointGuids[iLookAtPointIndex] = pxNode->GetGuid();
				// Increment the array index
				iLookAtPointIndex += 1;
			}
		}
	}
}

//--------------------------------------------------------------------------------------
//	Function	:	AVP_ServerEntity_GamePlayVolume
//	Created		:	18 / 12 / 07
//	Author		:	Gareth Thomas
//--------------------------------------------------------------------------------------
void AVP_ServerEntity_GamePlayVolume::BuildFleePointGuidArray()
{
	m_uNumFleePointGuids	= 0;

	for (Asura_Hashed_List_It<AVP_ServerNode_FleePoint> it(AVP_ServerNode_FleePoint::GetList()); !it.Done(); it.Next())
	{
		AVP_ServerNode_FleePoint* pxNode = it.GetCurrent();

		if (pxNode)
		{
			if (Contains(pxNode->GetPosition()))
			{
				m_uNumFleePointGuids += 1;
			}
		}
	}

	// Allocate the array
	m_pxFleePointGuids = new Asura_Guid[m_uNumFleePointGuids];

	u_int iFleePointIndex = 0;

	// Now iterate through and add the Guids to the array
	for (Asura_Hashed_List_It<AVP_ServerNode_FleePoint> it(AVP_ServerNode_FleePoint::GetList()); !it.Done(); it.Next())
	{
		AVP_ServerNode_FleePoint* pxNode = it.GetCurrent();

		if (pxNode)
		{
			// does this volume contain this node?
			if (Contains(pxNode->GetPosition()))
			{
				// Add the guid
				m_pxFleePointGuids[iFleePointIndex] = pxNode->GetGuid();
				// Increment the array index
				iFleePointIndex += 1;
			}
		}
	}
}


//--------------------------------------------------------------------------------------
//	Function	:	AVP_ServerEntity_GamePlayVolume
//	Created		:	30 / 10 / 07
//	Author		:	Gareth Thomas
//--------------------------------------------------------------------------------------
void AVP_ServerEntity_GamePlayVolume::BuildAlienSpawnGuidArray()
{
	m_uNumAlienSpawnerGuids = 0;

	for (Asura_Hashed_List_It<AVP_ServerEntity_AlienSpawnPoint> it(AVP_ServerEntity_AlienSpawnPoint::GetList()); !it.Done(); it.Next())
	{
		AVP_ServerEntity_AlienSpawnPoint* pxSpawner = it.GetCurrent();

		if (pxSpawner)
		{
			if (Contains(pxSpawner->GetPosition()))
			{
				m_uNumAlienSpawnerGuids += 1;
			}
		}
	}

	// Allocate the array
	m_pxAlienSpawnerGuids = new Asura_Guid[m_uNumAlienSpawnerGuids];

	u_int iIndex = 0;

	// Now iterate through and add the Guids to the array
	for (Asura_Hashed_List_It<AVP_ServerEntity_AlienSpawnPoint> it(AVP_ServerEntity_AlienSpawnPoint::GetList()); !it.Done(); it.Next())
	{
		AVP_ServerEntity_AlienSpawnPoint* pxSpawner = it.GetCurrent();

		if (pxSpawner)
		{
			// does this volume contain this spawner?
			if (Contains(pxSpawner->GetPosition()))
			{
				// Add the guid
				m_pxAlienSpawnerGuids[iIndex] = pxSpawner->GetGuid();
				// Increment the array index
				iIndex += 1;
			}
		}
	}
}

//--------------------------------------------------------------------------------------
//	Function	:	AVP_ServerEntity_GamePlayVolume :: BuildHumanoidSpawnGuidArray
//	Created		:	27 / 02 / 08
//	Author		:	Gareth Thomas
//--------------------------------------------------------------------------------------
void AVP_ServerEntity_GamePlayVolume::BuildHumanoidSpawnGuidArray()
{
	m_uNumHumanoidSpawnerGuids = 0;

	for (Asura_Hashed_List_It<AVP_ServerEntity_HumanSpawnPoint> it(AVP_ServerEntity_HumanSpawnPoint::GetList()); !it.Done(); it.Next())
	{
		AVP_ServerEntity_HumanSpawnPoint* pxSpawner = it.GetCurrent();

		if (pxSpawner)
		{
			if (Contains(pxSpawner->GetPosition()))
			{
				m_uNumHumanoidSpawnerGuids += 1;
			}
		}
	}

	// Allocate the array
	m_pxHumanoidSpawnerGuids = new Asura_Guid[m_uNumHumanoidSpawnerGuids];

	u_int iIndex = 0;

	// Now iterate through and add the Guids to the array
	for (Asura_Hashed_List_It<AVP_ServerEntity_HumanSpawnPoint> it(AVP_ServerEntity_HumanSpawnPoint::GetList()); !it.Done(); it.Next())
	{
		AVP_ServerEntity_HumanSpawnPoint* pxSpawner = it.GetCurrent();

		if (pxSpawner)
		{
			// does this volume contain this spawner?
			if (Contains(pxSpawner->GetPosition()))
			{
				// Add the guid
				m_pxHumanoidSpawnerGuids[iIndex] = pxSpawner->GetGuid();
				// Increment the array index
				iIndex += 1;
			}
		}
	}
}

void AVP_ServerEntity_GamePlayVolume::BuildSpawnPointArray()
{
	m_uNumSpawnPointGuids = 0;

	for ( Asura_Hashed_List_ConstIt< AVP_ServerEntity_SpawnPoint2 > xSPIter( AVP_ServerEntity_SpawnPoint2::GetList() ); !xSPIter.Done(); xSPIter.Next() )
	{
		const AVP_ServerEntity_SpawnPoint2* pxSP = xSPIter.GetCurrent();
		if ( Contains( pxSP->GetPosition() ) )
		{
			++m_uNumSpawnPointGuids;
		}
	}

	m_pxSpawnPointGuids = new Asura_Guid[ m_uNumSpawnPointGuids ];

	u_int uIndex = 0;
	for ( Asura_Hashed_List_ConstIt< AVP_ServerEntity_SpawnPoint2 > xSPIter( AVP_ServerEntity_SpawnPoint2::GetList() ); !xSPIter.Done(); xSPIter.Next() )
	{
		const AVP_ServerEntity_SpawnPoint2* pxSP = xSPIter.GetCurrent();
		if ( Contains( pxSP->GetPosition() ) )
		{
			m_pxSpawnPointGuids[ uIndex++ ] = pxSP->GetGuid();
		}
	}
}

//--------------------------------------------------------------------------------------
//	Function	:	BuildCoverPointGuidArray
//	Created		:	30 / 10 / 07
//	Author		:	Gareth Thomas
//--------------------------------------------------------------------------------------
void AVP_ServerEntity_GamePlayVolume::BuildCoverPointGuidArray()
{
	m_uNumCoverPointGuids = 0;

	for (Asura_Hashed_List_It<Asura_ServerNode_Coverpoint> xIt(Asura_ServerNode_Coverpoint::GetList()); !xIt.Done(); xIt.Next())
	{
		Asura_ServerNode_Coverpoint* pxCoverPoint = xIt.GetCurrent();

		if (pxCoverPoint)
		{
			if (Contains(pxCoverPoint->GetPosition()))
			{
				m_uNumCoverPointGuids += 1;
			}
		}
	}

	// Allocate the array
	m_pxCoverPointGuids = new Asura_Guid[m_uNumCoverPointGuids];

	u_int iIndex = 0;

	// Now iterate through and add the Guids to the array
	for (Asura_Hashed_List_It<Asura_ServerNode_Coverpoint> xIt(Asura_ServerNode_Coverpoint::GetList()); !xIt.Done(); xIt.Next())
	{
		Asura_ServerNode_Coverpoint* pxCoverPoint = xIt.GetCurrent();

		if (pxCoverPoint)
		{
			// does this volume contain this spawner?
			if (Contains(pxCoverPoint->GetPosition()))
			{
				// Add the guid
				m_pxCoverPointGuids[iIndex] = pxCoverPoint->GetGuid();
				// Increment the array index
				iIndex += 1;
			}
		}
	}
}

//--------------------------------------------------------------------------------------
//	Function	:	BuildGuardPointGuidArray
//	Created		:	30 / 10 / 07
//	Author		:	Gareth Thomas
//--------------------------------------------------------------------------------------
void AVP_ServerEntity_GamePlayVolume::BuildGuardPointGuidArray()
{
	m_uNumGuardPointGuids = 0;

	for (Asura_Hashed_List_It<AVP_ServerNode_GuardZone> xIt(AVP_ServerNode_GuardZone::GetList()); !xIt.Done(); xIt.Next())
	{
		AVP_ServerNode_GuardZone* pxGuardPoint = xIt.GetCurrent();

		if (pxGuardPoint)
		{
			if (Contains(pxGuardPoint->GetPosition()))
			{
				m_uNumGuardPointGuids += 1;
			}
		}
	}

	// Allocate the array
	m_pxGuardPointGuids = new Asura_Guid[m_uNumGuardPointGuids];

	u_int iIndex = 0;

	// Now iterate through and add the Guids to the array
	for (Asura_Hashed_List_It<AVP_ServerNode_GuardZone> xIt(AVP_ServerNode_GuardZone::GetList()); !xIt.Done(); xIt.Next())
	{
		AVP_ServerNode_GuardZone* pxGuardPoint = xIt.GetCurrent();

		if (pxGuardPoint)
		{
			// does this volume contain this spawner?
			if (Contains(pxGuardPoint->GetPosition()))
			{
				// Add the guid
				m_pxGuardPointGuids[iIndex] = pxGuardPoint->GetGuid();
				// Increment the array index
				iIndex += 1;
			}
		}
	}
}

#ifdef ASURA_TOOLS

void AVP_ServerEntity_GamePlayVolume::RenderForWaypointingMode( void )
{
	for (int i = 0; i < m_iNumVolumes; ++i)
	{
		switch (GetVolume(i)->GetClassification())
		{
			case ASURA_VOLUME_CLASSIFICATION_CUBOID:
			{
				Asura_Volume_Cuboid*	pxCubeVolume	= static_cast<Asura_Volume_Cuboid*>(GetVolume(i));
				Asura_Bounding_Box		xRelativeBox	= *pxCubeVolume->GetVolumeBox();
				Asura_Vector_3			xCentre			= xRelativeBox.CalculateCentre();

				xRelativeBox.Displace(-xCentre);

				Asura_Matrix_3x3 xMat;
				pxCubeVolume->GetVolumeOrient()->ToMatrix(xMat);

				u_int uEdgeColour, uBodyColour;

				switch ( m_eGamePlayAreaType )
				{
					case TIGHT_CORRIDOR:				uEdgeColour = 0xffff8888; uBodyColour = 0x66ff5555; break;
					case LARGE_CORRIDOR:				uEdgeColour = 0xff55ff55; uBodyColour = 0x6655ff55; break;
					case SMALL_ROOM:					uEdgeColour = 0xff8888ff; uBodyColour = 0x665555ff; break;
					case LARGE_ROOM:					uEdgeColour = 0xffffff00; uBodyColour = 0x66ffff55; break;
					case OPEN_SPACE:					uEdgeColour = 0xffff00ff; uBodyColour = 0x66ff55ff; break;
					case IGNORE_GPV:					uEdgeColour = 0xff000000; uBodyColour = 0x66111111; break;
					case NONE_ASSIGNED:	default:		uEdgeColour = 0xff00ffff; uBodyColour = 0x6655ffff; break;
				}

				Tools_Render::RenderCube(xRelativeBox, xCentre, xMat, uEdgeColour, true);
				Tools_Render::RenderCube(xRelativeBox, xCentre, xMat, uBodyColour, false);

			}break;
			case ASURA_VOLUME_CLASSIFICATION_SPHERE:
			{
				Asura_Volume_Sphere* pxSphereVolume = static_cast<Asura_Volume_Sphere*>(GetVolume(i));

				Asura_Matrix_3x3 xMat;
				pxSphereVolume->GetVolumeOrient()->ToMatrix(xMat);

				float			fRadius = *pxSphereVolume->GetRadiusPtr();
				Asura_Vector_3 xExtents = Asura_Vector_3(fRadius, fRadius, fRadius);

				u_int uEdgeColour, uBodyColour;

				switch ( m_eGamePlayAreaType )
				{
					case TIGHT_CORRIDOR:				uEdgeColour = 0xffff8888; uBodyColour = 0xffff5555; break;
					case LARGE_CORRIDOR:				uEdgeColour = 0xff55ff55; uBodyColour = 0xff55ff55; break;
					case SMALL_ROOM:					uEdgeColour = 0xff8888ff; uBodyColour = 0xff5555ff; break;
					case LARGE_ROOM:					uEdgeColour = 0xffffff00; uBodyColour = 0xffffff55; break;
					case OPEN_SPACE:					uEdgeColour = 0xffff00ff; uBodyColour = 0xffff55ff; break;
					case IGNORE_GPV:					uEdgeColour = 0xff222222; uBodyColour = 0x66333333; break;
					case NONE_ASSIGNED:	default:		uEdgeColour = 0xff00ffff; uBodyColour = 0xff55ffff; break;
				}

				Tools_Render::RenderSpheroid(*pxSphereVolume->GetPositionPtr(), xMat, xExtents, uEdgeColour, true);
				Tools_Render::RenderSpheroid(*pxSphereVolume->GetPositionPtr(), xMat, xExtents, uBodyColour, false);

			}break;
			case ASURA_VOLUME_CLASSIFICATION_CYLINDER:
			{
				Asura_Volume_Cylinder* pxCylinderVolume = static_cast<Asura_Volume_Cylinder*>(GetVolume(i));

				Asura_Matrix_3x3 xMat;
				pxCylinderVolume->GetVolumeOrient()->ToMatrix(xMat);

				Asura_Vector_3 xUp = Asura_Vector_3(xMat.e12, xMat.e22, xMat.e32);
				Asura_Vector_3 xStart = *pxCylinderVolume->GetPositionPtr();
				Asura_Vector_3 xEnd = *pxCylinderVolume->GetPositionPtr() + (-xUp * *pxCylinderVolume->GetHeightPtr());

				u_int uEdgeColour, uBodyColour;

				switch ( m_eGamePlayAreaType )
				{
					case TIGHT_CORRIDOR:				uEdgeColour = 0xffff8888; uBodyColour = 0x66ff5555; break;
					case LARGE_CORRIDOR:				uEdgeColour = 0xff55ff55; uBodyColour = 0x6655ff55; break;
					case SMALL_ROOM:					uEdgeColour = 0xff8888ff; uBodyColour = 0x665555ff; break;
					case LARGE_ROOM:					uEdgeColour = 0xffffff00; uBodyColour = 0x66ffff55; break;
					case OPEN_SPACE:					uEdgeColour = 0xffff00ff; uBodyColour = 0x66ff55ff; break;
					case IGNORE_GPV:					uEdgeColour = 0xff222222; uBodyColour = 0x66333333; break;
					case NONE_ASSIGNED:	default:	uEdgeColour = 0xff00ffff; uBodyColour = 0x6655ffff; break;
				}

				Tools_Render::RenderCylinder(xStart, xEnd, *pxCylinderVolume->GetRadiusPtr(), uEdgeColour, true);
				Tools_Render::RenderCylinder(xStart, xEnd, *pxCylinderVolume->GetRadiusPtr(), uBodyColour, false);

			}break;
			default:
			{
				Asura_Assert(0,"AVP_ServerEntity_GamePlayVolume :: RenderForWaypointingMode - Unkown Volume Classification");

			}break;
		}
	}
}

AVP_ServerEntity_GamePlayVolume* AVP_ServerEntity_GamePlayVolume::Tools_GetVolumeThatContainsPosition( const Asura_Vector_3& xPosition, const Asura_Guid uCurrentGPVGuid /*= ASURA_GUID_UNREGISTERED*/)
{
	AVP_ServerEntity_GamePlayVolume* pxCurrentGPV = AVP_ServerEntity_GamePlayVolume::FindEntity( uCurrentGPVGuid );
	if ( pxCurrentGPV )
	{
		// See if we're still in the current GPV.
		if ( pxCurrentGPV->Contains( xPosition ) )
		{
			return pxCurrentGPV;
		}
		else
		{
			// Search the linked GPVs to see if we've just popped into the next one.
			const u_int uNumLinked = pxCurrentGPV->GetNumLinkedGamePlayVolumes();
			for ( u_int uLinkedGPV = 0; uLinkedGPV < uNumLinked; ++uLinkedGPV )
			{
				AVP_ServerEntity_GamePlayVolume* pxLinkedGPV = AVP_ServerEntity_GamePlayVolume::FindEntity( pxCurrentGPV->GetLinkedGamePlayVolumeGuid( uLinkedGPV ) );
				if ( pxLinkedGPV && pxLinkedGPV->Contains( xPosition ) )
				{
					return pxLinkedGPV;
				}
			}
		}
	}

	// Iterate through all gameplay volumes looking for the one that contains the position passed in.
	for (AVP_GPV_NoDestroy_Hashed_List_It<AVP_ServerEntity_GamePlayVolume> xIt(AVP_ServerEntity_GamePlayVolume::GetList()); !xIt.Done(); xIt.Next())
	{
		AVP_ServerEntity_GamePlayVolume* pxGamePlayVolume = xIt.GetCurrent();

		if ( pxGamePlayVolume->GetGamePlayAreaType() == IGNORE_GPV )
		{
			continue;
		}

		if ( pxGamePlayVolume && pxGamePlayVolume->Contains( xPosition ) )
		{
			return pxGamePlayVolume;
		}
	}

	return NULL;
}
#endif

AVP_ServerEntity_GamePlayVolume* AVP_ServerEntity_GamePlayVolume::GetVolumeThatContainsPosition( const Asura_Vector_3& xPosition,
																								 const Asura_Guid uCurrentGPVGuid /*= ASURA_GUID_UNREGISTERED*/)
{
	AVP_ServerEntity_GamePlayVolume* pxCurrentGPV = AVP_ServerEntity_GamePlayVolume::FindEntity( uCurrentGPVGuid );
	if ( pxCurrentGPV )
	{
		// See if we're still in the current GPV.
		if ( pxCurrentGPV->Contains( xPosition ) )
		{
			return pxCurrentGPV;
		}
		else
		{
			// Search the linked GPVs to see if we've just popped into the next one.
			const u_int uNumLinked = pxCurrentGPV->GetNumLinkedGamePlayVolumes();
			for ( u_int uLinkedGPV = 0; uLinkedGPV < uNumLinked; ++uLinkedGPV )
			{
				AVP_ServerEntity_GamePlayVolume* pxLinkedGPV = AVP_ServerEntity_GamePlayVolume::FindEntity( pxCurrentGPV->GetLinkedGamePlayVolumeGuid( uLinkedGPV ) );
				if ( pxLinkedGPV && (pxLinkedGPV->GetGamePlayAreaType() != AVP_ServerEntity_GamePlayVolume::IGNORE_GPV) && pxLinkedGPV->Contains( xPosition ) )
				{
					return pxLinkedGPV; 
				}
			}
		}
	}

	// Iterate through all gameplay volumes looking for the one that contains the position passed in.
	for (Asura_Hashed_List_It<AVP_ServerEntity_GamePlayVolume> xIt(AVP_ServerEntity_GamePlayVolume::GetList()); !xIt.Done(); xIt.Next())
	{
		AVP_ServerEntity_GamePlayVolume* pxGamePlayVolume = xIt.GetCurrent();

		if ( pxGamePlayVolume->GetGamePlayAreaType() == IGNORE_GPV )
		{
			continue;
		}

		if ( pxGamePlayVolume && pxGamePlayVolume->Contains( xPosition ) )
		{
			return pxGamePlayVolume;
		}
	}

	return NULL;
}

#ifdef ASURA_TOOLS
void AVP_ServerEntity_GamePlayVolume::ApplyChangesForTemplateInstance(class Tools_EntityTemplateInstance* pxInstance)
{
	 PARENT::ApplyChangesForTemplateInstance(pxInstance);

	 m_xPosition = pxInstance->TransformPosition(m_xPosition);

	 for (u_int u = 0; u < MAX_LINKED_GAMEPLAYVOLUMES; ++u)
	 {
		m_puLinkedGamePlayVolumes[u] = pxInstance->ConvertGuid(m_puLinkedGamePlayVolumes[u]);
	 }
}
#endif

void AVP_ServerEntity_GamePlayVolume::ScoreSpawners( const Asura_ScoreParams* pxScoreParams, float& fHighestScore, Asura_Guid& uBestGuid ) const
{
	fHighestScore = 0.0f;
	uBestGuid	  = ASURA_GUID_UNREGISTERED;

	if ( !pxScoreParams ) { return; }

	float fLocalHighestScore = 0.0f;
	// We keep a list of the best guids and pick randomly from it if necessary.
	enum { MAX_BEST_GUIDS = 64 };
	u_int	   uNumBestGuids = 0;
	Asura_Guid auBestGuids[ MAX_BEST_GUIDS ];

	for ( u_int u = 0; u < m_uNumSpawnPointGuids; ++u )
	{
		const AVP_ServerEntity_SpawnPoint2* pxSpawnPoint = AVP_ServerEntity_SpawnPoint2::FindEntity( m_pxSpawnPointGuids[ u ] );
		if ( pxSpawnPoint )
		{
			float	   fScore = -1.0f;
			Asura_Guid uGuid  = ASURA_GUID_UNREGISTERED;
			pxSpawnPoint->ScoreSpawners( pxScoreParams, fScore, uGuid );

			if ( fScore > fLocalHighestScore )
			{
				fLocalHighestScore             = fScore;
				uNumBestGuids                  = 0;
				auBestGuids[ uNumBestGuids++ ] = uGuid;
			}
			else if ( fScore == fLocalHighestScore )
			{
				if ( uNumBestGuids == MAX_BEST_GUIDS )
				{
					uNumBestGuids = MAX_BEST_GUIDS - 1;
				}
				auBestGuids[ uNumBestGuids++ ] = uGuid;
			}
		}
	}

	// Pick from our list of best guids.
	if ( uNumBestGuids > 0 )
	{
		fHighestScore = fLocalHighestScore;
		uBestGuid	  = auBestGuids[ Asura_Random::GetIntU( 0, uNumBestGuids - 1 ) ];
	}
}

bool AVP_ServerEntity_GamePlayVolume::OwnsSpawnerGuid( const Asura_Guid uSpawnerGuid ) const
{
	for ( u_int u = 0; u < m_uNumSpawnPointGuids; ++u )
	{
		const AVP_ServerEntity_SpawnPoint2* pxSpawnPoint = AVP_ServerEntity_SpawnPoint2::FindEntity( m_pxSpawnPointGuids[ u ] );
		if ( pxSpawnPoint && pxSpawnPoint->OwnsSpawnerGuid( uSpawnerGuid ) )
		{
			return true;
		}
	}

	return false;
}

Asura_Guid AVP_ServerEntity_GamePlayVolume::Spawn( const Asura_Guid uSpawnerGuid, const Asura_SpawnParams* pxSpawnParams )
{
	for ( u_int u = 0; u < m_uNumSpawnPointGuids; ++u )
	{
		AVP_ServerEntity_SpawnPoint2* pxSpawnPoint = AVP_ServerEntity_SpawnPoint2::FindEntity( m_pxSpawnPointGuids[ u ] );
		if ( pxSpawnPoint && pxSpawnPoint->OwnsSpawnerGuid( uSpawnerGuid ) )
		{
			return pxSpawnPoint->Spawn( uSpawnerGuid, pxSpawnParams );
		}
	}

	return ASURA_GUID_UNREGISTERED;
}

void AVP_ServerEntity_GamePlayVolume::SetFlareBPHashFromAreaType()
{
	if ( m_eGamePlayAreaType < MAX_AREA_TYPES && GetAreaTypeNames()[ m_eGamePlayAreaType ] )
	{
		Asura_Hash_ID uGPVHash = Asura_GetHashID( GetAreaTypeNames()[ m_eGamePlayAreaType ] );
		const Asura_Blueprint* pxGPVBP = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_GAMEPLAYVOLUMES, uGPVHash );
		if ( pxGPVBP )
		{
			Asura_Value xVal;
			if ( pxGPVBP->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_FLARE, xVal ) )
			{
				m_uFlareBlueprintHash = xVal.GetHashIDValue();
			}
		}
	}
}
