//--------------------------------------------------------------------------------------
//	File	:	AVP_ServerNode_FleePoint.cpp
//	Author	:	Gareth Thomas
//	Created	:	06 / 03 / 08
//	Copyright (C) 2006 Rebellion, All Rights Reserved.
//--------------------------------------------------------------------------------------

// Core
#include "Core/Asura.h"

// Header
#include "Entity/Locations/FleePoint/AVP_ServerNode_FleePoint.h"

// Asura
#include "ChunkSystem/Asura_Chunk_Stream.h"

// AVP
#include "Entity/AVP_Entity_Classification.h"

//------------------------------------------------------------------------------
// Constants, Defines, Statics and Globals
//------------------------------------------------------------------------------
Asura_Hashed_List<AVP_ServerNode_FleePoint> AVP_ServerNode_FleePoint::s_xList;

//--------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------
AVP_ServerNode_FleePoint::AVP_ServerNode_FleePoint(const Asura_Guid uGuid)
: PARENT(uGuid)
, m_pxFleeActions( NULL )
, m_uNumFleeActions( 0 )
{
	Classification = AVP_ENTITYCLASS_FLEEPOINT;

	SetNeverUpdateClient();
	SetNeverUpdates(true);
	s_xList.AddToList(this);
}

//--------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------
AVP_ServerNode_FleePoint::~AVP_ServerNode_FleePoint(void)
{
	if ( m_pxFleeActions )
	{
		delete [] m_pxFleeActions;
	}

	s_xList.RemoveFromList(this);
}

//--------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------
void AVP_ServerNode_FleePoint::PostLoadInitialise(void)
{
	PARENT::PostLoadInitialise();
}

//--------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------
void AVP_ServerNode_FleePoint::WriteToChunkStream(Asura_Chunk_Stream& xStream) const
{
	//Version
	xStream << CURRENT_CHUNK_VERSION;

	//Parent
	PARENT::WriteToChunkStream(xStream);

	xStream << m_uNumFleeActions;

	for ( u_int u = 0; u < m_uNumFleeActions; ++u )
	{
		xStream << m_pxFleeActions[ u ];
	}
}

//--------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------
void AVP_ServerNode_FleePoint::ReadFromChunkStream(Asura_Chunk_Stream& xStream)
{
	//Version
	u_int uVersion;
	xStream	>> uVersion;

	//Parent
	PARENT::ReadFromChunkStream(xStream);

	if ( uVersion == CURRENT_CHUNK_VERSION )
	{
		xStream >> m_uNumFleeActions;
		m_pxFleeActions = new Asura_Guid[ m_uNumFleeActions ];

		for ( u_int u = 0; u < m_uNumFleeActions; ++u )
		{
			xStream >> m_pxFleeActions[ u ];
		}
	}
}

//--------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------
void AVP_ServerNode_FleePoint::SetActionGuids(Asura_Guid* pxGuids, u_int uNumGuids)
{
	if ( m_pxFleeActions )
	{
		delete [] m_pxFleeActions;
	}

	m_pxFleeActions = new Asura_Guid[ uNumGuids ];

	for ( u_int u = 0; u < uNumGuids; ++u )
	{
		m_pxFleeActions[u] = pxGuids[u];
	}

	m_uNumFleeActions = uNumGuids;
}

//--------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------
AVP_ServerNode_FleePoint* AVP_ServerNode_FleePoint::Create(const Asura_Guid uGuid)
{
	return(new AVP_ServerNode_FleePoint(uGuid));
}

#ifdef ASURA_TOOLS
void AVP_ServerNode_FleePoint::ApplyChangesForTemplateInstance(Tools_EntityTemplateInstance* pxInstance)
{
	PARENT::ApplyChangesForTemplateInstance(pxInstance);
}
#endif
