//------------------------------------------------------------------------------
// File		: AVP_ServerNode_GuardZone.h
// Created	: 06 / 03 / 08
// Author	: Gareth Thomas
// Copyright (c) 2007 Rebellion. All rights reserved.
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// Begin File Wrapper
#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif
#ifndef AVP_SERVERNODE_GUARDZONE_H___
#define AVP_SERVERNODE_GUARDZONE_H___ 1
// End File Wrapper
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// Includes
//------------------------------------------------------------------------------
#include "Entity/Locations/GuardZone/Asura_ServerNode_GuardZone.h"
#include "Entity/Locations/OrientedNode/AVP_ServerNode_OrientedNode.h"


//------------------------------------------------------------------------------
// Class	: AVP_ServerNode_GuardZone.
// Created	: 06 / 03 / 08
// Author	: Gareth Thomas
// Copyright (c) 2007 Rebellion. All rights reserved.
//------------------------------------------------------------------------------
class AVP_ServerNode_GuardZone : public Asura_ServerNode_GuardZone
{

public:

	enum GUARD_ZONE_PATROL_TYPES
	{		
		GUARD_ZONE_PATROL_TYPE_LOOPING		= 0,
		GUARD_ZONE_PATROL_TYPE_PING_PONG,
		GUARD_ZONE_PATROL_TYPE_RANDOM,
		NUM_GUARD_ZONE_PATROL_TYPES
	};

	// Constructor / Destructor
	AVP_ServerNode_GuardZone(Asura_Guid uGuid);
	virtual ~AVP_ServerNode_GuardZone();

	// Statics
	static  void						CreateFromChunkBuffer(Asura_Guid uGUID, const char*& pcChunkBfr);
	virtual void						InitialiseFromChunkBuffer(const char*& pcChunkBfr);
	virtual void						WriteToChunkStream(Asura_Chunk_Stream& xStream) const;
	virtual void						ReadFromChunkStream(Asura_Chunk_Stream& xStream);

	// Public Static Functions...
	static AVP_ServerNode_GuardZone*	Create(const Asura_Guid uGuid);
	static AVP_ServerNode_GuardZone*	FindEntity(Asura_Guid uGuid)			{return s_xList.FindEntity(uGuid);}
	static const Asura_Hashed_List<AVP_ServerNode_GuardZone>* GetList(void)		{return &s_xList;}

	// Public Member Functions...
	AVP_ServerNode_OrientedNode*		GetNearestGuardPoint(const Asura_Vector_3 xPosition);
	AVP_ServerNode_OrientedNode*		GetRandomGuardPoint(void);
	AVP_ServerNode_OrientedNode*		GetNextLoopingGuardPoint(const Asura_Guid uCurrentGPGuid);
	AVP_ServerNode_OrientedNode*		GetNextPingPongGuardPoint(const Asura_Guid uCurrentGPGuid);
	Asura_Guid							GetGuidFromGPPatrolOrder(const u_int uPatrolOrder) const;
	u_int								GetPatrolType(void)						{return m_iPatrolType;}

	u_int	GetNextLoopingPatrolOrderPos(u_int uPatrolOrderPos);
	u_int	GetNextPingPongPatrolOrderPos(u_int uPatrolOrderPos);

#ifdef ASURA_TOOLS
	virtual bool IsBaseOrDerivativeOf(u_short usClassification) const;
	virtual void ApplyChangesForTemplateInstance(Tools_EntityTemplateInstance* pxInstance);
	virtual bool CanBeIncludedInTemplate(void) const;
	virtual bool RemoveGuardPoint( Asura_Guid uGuid );
#endif

private:

	// Parent typedef
	typedef Asura_ServerNode_GuardZone PARENT;

	// Friends...
	friend class ToolsAVP_Node_GuardZone;
	friend class ToolsAVP_Node_OrientedNode;

	enum { CURRENT_CHUNK_VERSION = 1000 };

	// Statics
	static Asura_Hashed_List<AVP_ServerNode_GuardZone> s_xList;

	// Private variables
	u_int	m_iPatrolType;
	u_int	m_uPingPongDirection;	// 0 = positive, 1 = negative
};



#endif // #ifndef AVP_SERVERNODE_GUARDZONE_H___
