//////////////////////////////////////////////////////////////////////////////
//
// AVP_ServerNode_OrientedNode.cpp
//
// Created by Stuart Perry ( 18.04.07 )
//
// Copyright (c) 2007 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header.
#include "Entity/Locations/OrientedNode/AVP_ServerNode_OrientedNode.h"

//Asura
#include "ChunkSystem/Asura_Chunk_Stream.h"

//Project
#include "Entity/AVP_Entity_Classification.h"

// Tools
#ifdef ASURA_TOOLS
#include "EntityTemplate/Tools_EntityTemplate_System.h"
#endif

Asura_Hashed_List<AVP_ServerNode_OrientedNode> AVP_ServerNode_OrientedNode::s_xList(2);

////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
//
// AVP_ServerNode_OrientedNode
//
//////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ServerNode_OrientedNode::AVP_ServerNode_OrientedNode
// Author       : Stuart Perry
// Date         : 18-APRIL-2007
////////////////////////////////////////////////////////////////////////////////

AVP_ServerNode_OrientedNode::AVP_ServerNode_OrientedNode(Asura_Guid uGuid)
: PARENT(uGuid)
, m_uRandomWeighting(1)
, m_uPatrolOrder(1)
, m_fWaitTime(10)
, m_fWaitTimeRandomOffset(5)
, m_eNodeType(GUARD_NODE)
, m_bWaitTimeRandomOffset(false)
{
	Classification = AVP_ENTITYCLASS_ORIENTEDNODE;

	s_xList.AddToList( this );
}

/// End of function 'AVP_ServerNode_OrientedNode::AVP_ServerNode_OrientedNode'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ServerNode_OrientedNode::~AVP_ServerNode_OrientedNode
// Author       : Stuart Perry
// Date         : 18-APRIL-2007
////////////////////////////////////////////////////////////////////////////////

AVP_ServerNode_OrientedNode::~AVP_ServerNode_OrientedNode()
{
	s_xList.RemoveFromList( this );
}

/// End of function 'AVP_ServerNode_OrientedNode::~AVP_ServerNode_OrientedNode'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ServerNode_OrientedNode::Create
// Author       : Stuart Perry
// Date         : 18-APRIL-2007
////////////////////////////////////////////////////////////////////////////////

AVP_ServerNode_OrientedNode* AVP_ServerNode_OrientedNode::Create(const Asura_Guid uGuid)
{
	return(new AVP_ServerNode_OrientedNode(uGuid));
}

/// End of function 'AVP_ServerNode_OrientedNode::Create'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ServerNode_OrientedNode::CreateFromChunkBuffer
// Author       : Stuart Perry
// Date         : 18-APRIL-2007
////////////////////////////////////////////////////////////////////////////////

void AVP_ServerNode_OrientedNode::CreateFromChunkStream( Asura_Guid uGUID, Asura_Chunk_Stream& xStream )
{
	AVP_ServerNode_OrientedNode* pxNode = new AVP_ServerNode_OrientedNode( uGUID );
	if( pxNode )
	{
		pxNode->InitialiseFromChunkStream( xStream );
	}
}

/// End of function 'AVP_ServerNode_OrientedNode::CreateFromChunkBuffer'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ServerNode_OrientedNode::InitialiseFromChunkStream
// Author       : Stuart Perry
// Date         : 18-APRIL-2007
////////////////////////////////////////////////////////////////////////////////

void AVP_ServerNode_OrientedNode::InitialiseFromChunkStream( Asura_Chunk_Stream& xStream )
{
	int iVersion = xStream.PeekInt();

	if ( iVersion < 1000 )
	{
		//Just do Parent
		PARENT::ReadFromChunkStream( xStream );
	}
	else
	{
		//Version
		xStream >> iVersion;

		//Parent
		PARENT::ReadFromChunkStream( xStream );
	}
}

/// End of function 'AVP_ServerNode_OrientedNode::InitialiseFromChunkStream'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ServerNode_OrientedNode::WriteToChunkStream
// Author       : Stuart Perry
// Date         : 18-APRIL-2007
////////////////////////////////////////////////////////////////////////////////

void AVP_ServerNode_OrientedNode::WriteToChunkStream( Asura_Chunk_Stream& xStream ) const
{
	//Version
	xStream << uCURRENT_CHUNK_VERSION;

	//Parent
	PARENT::WriteToChunkStream( xStream );

	xStream << m_uRandomWeighting;
	xStream << m_uPatrolOrder;
	xStream << m_fWaitTime;
	xStream << m_fWaitTimeRandomOffset;
	xStream << m_uFlags;
	xStream << m_eNodeType;
}

/// End of function 'AVP_ServerNode_OrientedNode::WriteToChunkStream'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ServerNode_OrientedNode::WriteToChunkStream
// Author       : Stuart Perry
// Date         : 18-APRIL-2007
////////////////////////////////////////////////////////////////////////////////

void AVP_ServerNode_OrientedNode::ReadFromChunkStream( Asura_Chunk_Stream& xStream )
{
	//Version
	u_int uVersion = uCURRENT_CHUNK_VERSION + 1;
	xStream	>> uVersion;

	//Parent
	PARENT::ReadFromChunkStream( xStream );

	if ( uVersion == uCURRENT_CHUNK_VERSION )
	{
		xStream >> m_uRandomWeighting;
		xStream >> m_uPatrolOrder;
		xStream >> m_fWaitTime;
		xStream >> m_fWaitTimeRandomOffset;
		xStream >> m_uFlags;
		xStream >> m_eNodeType;
	}
	else if ( uVersion == 1 )
	{
		xStream >> m_uRandomWeighting;
		xStream >> m_uPatrolOrder;
		xStream >> m_fWaitTime;
		xStream >> m_fWaitTimeRandomOffset;
		xStream >> m_uFlags;
		Asura_Guid uDummyGuid;
		xStream >> uDummyGuid;
		xStream >> m_eNodeType;
	}
	else
	{
		Asura_Assertf( false, ( "AVP_ServerNode_OrientedNode::ReadFromChunkStream - unhandled chunk version (%u)", uVersion ) );
	}
}

/// End of function 'AVP_ServerNode_OrientedNode::WriteToChunkStream'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ServerNode_OrientedNode::IsBaseOrDerivativeOf
// Author       : Stuart Perry
// Date         : 18-APRIL-2007
////////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_TOOLS
bool AVP_ServerNode_OrientedNode::IsBaseOrDerivativeOf( u_short usClassification ) const
{
	if (usClassification == AVP_ENTITYCLASS_ORIENTEDNODE)
	{
		return true;
	}
	return PARENT::IsBaseOrDerivativeOf(usClassification);
}

/// End of function 'AVP_ServerNode_OrientedNode::IsBaseOrDerivativeOf'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ServerNode_OrientedNode::CanBeIncludedInTemplate
// Author       : Stuart Perry
// Date         : 18-APRIL-2007
////////////////////////////////////////////////////////////////////////////////

bool AVP_ServerNode_OrientedNode::CanBeIncludedInTemplate() const 
{
	return ( Classification == AVP_ENTITYCLASS_ORIENTEDNODE );
}

/// End of function 'AVP_ServerNode_OrientedNode::CanBeIncludedInTemplate'. ///
#endif // ASURA_TOOLS
//////////////////////////////////////////////////////////////////////////////
//
//
//////////////////////////////////////////////////////////////////////////////
