//////////////////////////////////////////////////////////////////////////////
//
// AVP_ServerNode_GS2Node.cpp
//
// Copyright (C) 2008 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"

// header
#include "Entity/Locations/OrientedNode/GamesceneNode/AVP_ServerNode_GS2Node.h"

// asura
#include "AI/Asura_AI_Navigation.h"
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "Network/Asura_Message.h"

// axon

// avp
#include "Entity/AVP_Entity_Classification.h"
#include "Entity/AVP_Messages.h"

//////////////////////////////////////////////////////////////////////////////
//
// AVP_ServerNode_GS2Node
//
//////////////////////////////////////////////////////////////////////////////

Asura_Hashed_List< AVP_ServerNode_GS2Node > AVP_ServerNode_GS2Node::s_xList;

AVP_ServerNode_GS2Node::AVP_ServerNode_GS2Node( Asura_Guid uGuid )
: PARENT_ASURA( uGuid )
, m_usWaypointID( ASURA_INVALID_WAYPOINT_ID )
#ifdef ASURA_TOOLS
, m_bIntegrateWithNavData( false )
, m_uTraverseHelper( 0 )
#endif
, m_bLeaveGamesceneOnKnowledge( false )
{
	Classification = AVP_ENTITYCLASS_GS2NODE;
	s_xList.AddToList( this );
}

AVP_ServerNode_GS2Node::~AVP_ServerNode_GS2Node()
{
	s_xList.RemoveFromList( this );
}

void AVP_ServerNode_GS2Node::ReadFromChunkStream( Asura_Chunk_Stream& xStream )
{
	u_int uVersion;
	xStream >> uVersion;

	// Only read in our Asura parent if we are actually a node. Classes below us will effectively read in the parent for us.
	if ( Classification == AVP_ENTITYCLASS_GS2NODE )
	{
		// Parent.
		PARENT_ASURA::ReadFromChunkStream( xStream );
	}

	// Us.
	if ( uVersion == uCURRENT_CHUNK_VERSION )
	{
		xStream >> m_usWaypointID;

		u_short usBool;
		xStream >> usBool;
#ifdef ASURA_TOOLS
		m_bIntegrateWithNavData = ( usBool ? true : false );
#endif
		xStream >> usBool;
		m_bLeaveGamesceneOnKnowledge = ( usBool? true : false );
	}
	else if ( uVersion == 2 )
	{
		xStream >> m_usWaypointID;

		u_short usBool;
		xStream >> usBool;
#ifdef ASURA_TOOLS
		m_bIntegrateWithNavData = ( usBool ? true : false );
#endif
	}
	else if ( uVersion == 1 )
	{
		int iBool;
		xStream >> iBool;
#ifdef ASURA_TOOLS
		m_bIntegrateWithNavData = ( iBool ? true : false );
#endif
	}
	else if ( uVersion == 0 )
	{
	}
	else
	{
		Asura_Assertf( false, ( "AVP_ServerNode_GS2Node::ReadFromChunkStream - unhandled chunk version (%u)", uVersion ) );
	}
}

void AVP_ServerNode_GS2Node::WriteToChunkStream( Asura_Chunk_Stream& xStream ) const
{
	// Version
	xStream << uCURRENT_CHUNK_VERSION;

	// Only chunk out our Asura parent if we are actually a node. Classes below us will effectively chunk out the parent for us.
	if ( Classification == AVP_ENTITYCLASS_GS2NODE )
	{
		// Parent.
		PARENT_ASURA::WriteToChunkStream( xStream );
	}

	// Our data
	xStream << m_usWaypointID;

#ifdef ASURA_TOOLS
	xStream << static_cast< u_short >( m_bIntegrateWithNavData ? 1 : 0 );
#else
	xStream << static_cast< u_short >( 0 );
#endif

	xStream << static_cast< u_short >( m_bLeaveGamesceneOnKnowledge ? 1 : 0 );
}

bool AVP_ServerNode_GS2Node::IsBaseOrDerivativeOf( u_short usClassification ) const
{
	if ( usClassification == AVP_ENTITYCLASS_GS2NODE )
	{
		return true;
	}
	return PARENT_ASURA::IsBaseOrDerivativeOf( usClassification );
}

#ifdef ASURA_TOOLS
bool AVP_ServerNode_GS2Node::CanBeIncludedInTemplate() const
{
	return ( Classification == AVP_ENTITYCLASS_GS2NODE );
}
#endif

void AVP_ServerNode_GS2Node::Create( const Asura_Guid uGuid, Asura_Chunk_Stream& xStream )
{
	AVP_ServerNode_GS2Node* pxNode = new AVP_ServerNode_GS2Node( uGuid );
	pxNode->ReadFromChunkStream( xStream );
}

bool AVP_ServerNode_GS2Node::HandleMessage( const Asura_Message* pxMessage )
{
	switch ( pxMessage->GetMsgID() )
	{
		case AVP_MSGID_DEACTIVATE_OCCUPIER:
		case AVP_MSGID_DESTROY_OCCUPIER:
		case AVP_MSGID_SET_HEALTH:
		{
			// Forward these messages on to our current users.
			for ( Asura_LinkList_ConstIt< Asura_Guid > xIt( GetOccupierList() ); !xIt.Done(); xIt.Next() )
			{
				Asura_Network::Forward( pxMessage, *xIt.GetCurrent(), GetGuid() );
			}

			return true;
		}

		default:
		{
			// Only pass the messge up if we are actually a node.
			// If we're an attractor the attractor HandleMessage() function will pass it up.
			if ( GetClassification() == AVP_ENTITYCLASS_GS2NODE )
			{
				return PARENT_ASURA::HandleMessage( pxMessage );
			}
			else
			{
				return false;
			}
		}
	}
}

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////
