//////////////////////////////////////////////////////////////////////////////
//
// AVP_ServerNode_GS2Spline.cpp
//
// Copyright (C) 2008 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"

// header
#include "Entity/Locations/Spline/GamesceneSpline/AVP_ServerNode_GS2Spline.h"

// asura
#include "ChunkSystem/Asura_Chunk_Stream.h"

// axon

// avp
#include "Entity/AVP_Entity_Classification.h"

//////////////////////////////////////////////////////////////////////////////
//
// AVP_ServerNode_GS2Spline
//
//////////////////////////////////////////////////////////////////////////////

AVP_ServerNode_GS2Spline::AVP_ServerNode_GS2Spline( Asura_Guid uGuid )
: PARENT( uGuid )
, m_bIntegrateWithNavData( false )
, m_bLeaveGamesceneOnKnowledge( false )
{
	Classification = AVP_ENTITYCLASS_GS2SPLINE;
}

void AVP_ServerNode_GS2Spline::ReadFromChunkStream( Asura_Chunk_Stream& xStream )
{
	u_int uVersion;
	xStream >> uVersion;

	// Parent.
	PARENT::ReadFromChunkStream( xStream );

	// Us.
	if ( uVersion == uCURRENT_CHUNK_VERSION )
	{
		int iBool;
		xStream >> iBool;
		m_bIntegrateWithNavData = ( iBool ? true : false );
		xStream >> iBool;
		m_bLeaveGamesceneOnKnowledge = ( iBool ? true : false );
	}
	else if ( uVersion == 1 )
	{
		int iBool;
		xStream >> iBool;
		m_bIntegrateWithNavData = ( iBool ? true : false );
	}
	else if ( uVersion == 0 )
	{
	}
	else
	{
		Asura_Assertf( false, ( "AVP_ServerNode_GS2Spline::ReadFromChunkStream - unhandled chunk version (%u)", uVersion ) );
	}
}

void AVP_ServerNode_GS2Spline::WriteToChunkStream( Asura_Chunk_Stream& xStream ) const
{
	// Version
	xStream << uCURRENT_CHUNK_VERSION;

	// Parent.
	PARENT::WriteToChunkStream( xStream );

	// Our data
	xStream << static_cast< int >( m_bIntegrateWithNavData ? 1 : 0 );

	xStream << static_cast< int >( m_bLeaveGamesceneOnKnowledge ? 1 : 0 );
}

bool AVP_ServerNode_GS2Spline::IsBaseOrDerivativeOf( u_short usClassification ) const
{
	if ( usClassification == AVP_ENTITYCLASS_GS2SPLINE )
	{
		return true;
	}
	return PARENT::IsBaseOrDerivativeOf( usClassification );
}

void AVP_ServerNode_GS2Spline::Create( const Asura_Guid uGuid, Asura_Chunk_Stream& xStream )
{
	AVP_ServerNode_GS2Spline* pxNode = new AVP_ServerNode_GS2Spline( uGuid );
	pxNode->ReadFromChunkStream( xStream );
}

#ifdef ASURA_TOOLS
bool AVP_ServerNode_GS2Spline::CanBeIncludedInTemplate() const
{
	return ( Classification == AVP_ENTITYCLASS_GS2SPLINE );
}
#endif

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////
