//--------------------------------------------------------------------------------------
//	File	:	AVP_ServerEntity_MotionTrackerBlip.h
//	Author	:	Gareth Thomas
//	Created	:	18 / 12 / 2007
//	Copyright (C) 2006 Rebellion, All Rights Reserved.
//--------------------------------------------------------------------------------------

//--------------------------------------------------------------------------------------
// BEGIN FILE WRAPPER
#ifdef ASURA_USE_PRAGMA_ONCE 
#pragma once
#endif
#ifndef __AVP_SERVERENTITY_MOTIONTRACKERBLIP_H__
#define __AVP_SERVERENTITY_MOTIONTRACKERBLIP_H__ 1
// END FILE WRAPPER
//--------------------------------------------------------------------------------------

//--------------------------------------------------------------------------------------
//	INCLUDES
//--------------------------------------------------------------------------------------
#include "Entity/Locations/Asura_ServerEntity_Node.h"

//--------------------------------------------------------------------------------------
//	Class	:	AVP_ServerEntity_MotionTrackerBlip
//	Created	:	18 / 12 / 07
//	Author	:	Gareth Thomas
//--------------------------------------------------------------------------------------
class AVP_ServerEntity_MotionTrackerBlip : public Asura_ServerEntity_Node
{

public:

	enum UpdateState
	{
		UNSET,
		BEGIN,
		ON_LINK,
		AT_NODE,
		WAITING,
		MAX_STATES,
	};

	// Public Overridden functions...
	virtual void						WriteToChunkStream		(Asura_Chunk_Stream& xStream) const;
	virtual void						ReadFromChunkStream		(Asura_Chunk_Stream& xStream);
	virtual bool 						CanBeSavedToChunk		(void) const							{return(true);}
	virtual bool						IsFilledFromChunkBuffer	(void) const							{return(false);}
	virtual bool						HasClientEntities		(void) const							{return (false);}
	virtual void						PostLoadInitialise		(void);
	virtual void						Update					(void);


	float								GetVelocity				(void)			{ return m_fVelocity; }

	Asura_Guid	GetStartGS2NodeGuid( void ) const		{ return m_uStartGS2NodeGuid; }
	void		SetStartGS2Node( Asura_Guid uGuid )		{ m_uStartGS2NodeGuid = uGuid; }

	// Public Static Functions...
	static AVP_ServerEntity_MotionTrackerBlip*								Create(const Asura_Guid uGuid);
	static AVP_ServerEntity_MotionTrackerBlip*								FindEntity(Asura_Guid uGuid)			{return s_xList.FindEntity(uGuid);}
	static const Asura_Hashed_List<AVP_ServerEntity_MotionTrackerBlip>*		GetList(void)							{return &s_xList;}

#ifdef ASURA_TOOLS
	virtual void ApplyChangesForTemplateInstance(class Tools_EntityTemplateInstance* pxInstance);
	virtual bool CanBeIncludedInTemplate() const { return true; }
#endif

protected:

	// Construction / Destruction
	AVP_ServerEntity_MotionTrackerBlip(const Asura_Guid uGuid);
	virtual ~AVP_ServerEntity_MotionTrackerBlip(void);

	UpdateState			GetUpdateState( void )						{ return m_eUpdateState; }
	void				SetUpdateState( UpdateState eState )		{ m_eUpdateState = eState; }
	void				SwitchToState( UpdateState eState );
	void				ChooseLinkToTake( const Asura_Guid uSourceNodeGuid, Asura_Guid& uLinkGuid );
	void				UpdateVelocity( void );

#ifndef ASURA_NO_DEBUG_OUTPUT
	void				RenderDebugInfo( void );
#endif

	

private:

	// Friend classes
	friend class ToolsAVP_Entity_MotionTrackerBlip;

	Asura_Guid		m_uStartGS2NodeGuid;
	Asura_Guid		m_uCurrentNodeGuid;
	Asura_Guid		m_uCurrentSplineGuid;
	float			m_fDistanceToTraveled;
	UpdateState		m_eUpdateState;
	Asura_Vector_3	m_xVelocity;
	Asura_Vector_3	m_xPreviousPosition;

	
	float			m_fTime;
	float			m_fVelocity;

#ifndef ASURA_NO_DEBUG_OUTPUT
	float			m_fProportion;
#endif


	// Private typedefs
	typedef Asura_ServerEntity_Node PARENT;

	// Chunk Version
	enum { CURRENT_CHUNK_VERSION = 0 };

	// Private statics
	static Asura_Hashed_List<AVP_ServerEntity_MotionTrackerBlip> s_xList;
};

#endif   // __AVP_SERVERENTITY_MOTIONTRACKERBLIP_H__
