//////////////////////////////////////////////////////////////////////////////
//
// AVP_ServerEntity_PlayerObjective.h
//
// Created by AJK 01 / 10 / 2008
//
// Copyright (c) 2009 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE 
#pragma once
#endif

#ifndef AVP_SERVERENTITY_PLAYEROBJECTIVE
#define AVP_SERVERENTITY_PLAYEROBJECTIVE

// Asura
#include "Core/Asura_StringBuffer.h"
#include "GUIMenu/Asura_GUIMenu_Element_Text.h"

// AVP
#include "Entity/AVP_Entity_Classification.h"
#include "Entity/Locations/Objective/AVP_ServerNode_Objective.h"

class AVP_ServerEntity_PlayerObjective : public AVP_ServerNode_Objective
{
public:
	// Objectives
	enum
	{
		FLAG_COMPLETED = 0x00000002,
	};

	enum
	{
		TARGET_OWN_LOCATION    ,
		TARGET_ENTITY          ,
		TARGET_LOCATION_MARKER ,
		TARGET_NOTHING         ,
		NUM_TARGET_TYPES
	};

	static AVP_ServerEntity_PlayerObjective*						  Create(Asura_Guid uGuid)	   { return new AVP_ServerEntity_PlayerObjective(uGuid); }
	static AVP_ServerEntity_PlayerObjective*						  FindEntity(Asura_Guid uGuid) { return s_xList.FindEntity(uGuid); }
	static const Asura_Hashed_List<AVP_ServerEntity_PlayerObjective>* GetList()            		   { return &s_xList; }

	// Public Overridden functions...
	virtual void						WriteToChunkStream						( Asura_Chunk_Stream& xStream ) const;
	virtual void						ReadFromChunkStream						( Asura_Chunk_Stream& xStream );
	virtual bool 						CanBeSavedToChunk						( void ) const						{ return ( true ); }
	virtual bool						IsFilledFromChunkBuffer					( void ) const						{ return ( false ); }
	virtual bool						HasClientEntities						( void ) const						{ return ( false ); }
	virtual bool						HandleMessage							( const Asura_Message* pxMessage);
	virtual void						MakeCreateMessage                       ( Asura_Guid uRequestingClient, bool bSendGuaranteed = true);
	virtual bool                        GetHasCreateMessage() const             { return true; }

	void				Reset(void);
	void				SendCompleteEvent(void);

	Asura_GUIMenu_Element_Text&                     GetObjectiveText() { return m_xObjectiveText;}
	const Asura_GUIMenu_Element_Text&               GetObjectiveText() const { return m_xObjectiveText;}
	const Asura_LinkList< Asura_Guid >& GetMarkerList() const { return m_xMarkerList; }
	Asura_LinkList< Asura_Guid >&       GetMarkerList() { return m_xMarkerList; }
	u_int                       GetFlags() const { return m_uFlags;}

	// Objectives Getters/Setters
	bool	IsCompleted()       const { return IsFlagged(FLAG_COMPLETED); }
	void	SetAsCompleted     (bool bTrue) { bTrue ? AddFlag( FLAG_COMPLETED      ) : RemoveFlag(FLAG_COMPLETED); }

	void	   SetTargetGuid (Asura_Guid uGuid)  { m_uTargetGuid = uGuid;}
	Asura_Guid GetTargetGuid () const            { return m_uTargetGuid; }

	int        GetTargetType() const { return m_iTargetType; }

#ifdef ASURA_TOOLS
	virtual bool CanBeIncludedInTemplate(void) const {return (Classification == AVP_ENTITYCLASS_PLAYER_OBJECTIVE);}
#endif

protected:

	// Construction / Destruction
	AVP_ServerEntity_PlayerObjective(const Asura_Guid uGuid);
	virtual ~AVP_ServerEntity_PlayerObjective(void);

private:

	// Flag functions
	void                    AddFlag    (u_int uFlag) { m_uFlags |= uFlag;               }
	void                    RemoveFlag (u_int uFlag) { m_uFlags &= ~uFlag;              }
	bool                    IsFlagged  (u_int uFlag) const { return ((m_uFlags & uFlag) != 0); }

	void                    OnChange();

	Asura_GUIMenu_Element_Text                     m_xObjectiveText;
	u_int                                          m_uFlags;
	Asura_Guid                                     m_uTargetGuid;
	u_int                                          m_iTargetType;
	Asura_LinkList <Asura_Guid>                    m_xMarkerList;

#ifdef ASURA_TOOLS
	// Friend classes
	friend class ToolsAVP_Entity_PlayerObjective;
#endif

	typedef AVP_ServerNode_Objective PARENT;

	// Chunk Version
	enum { CURRENT_CHUNK_VERSION = 6};

	// Private statics
	static const char* s_aszTargetTypes[];
	static Asura_Hashed_List < AVP_ServerEntity_PlayerObjective > s_xList;
};

#endif   // __AVP_ServerEntity_PlayerObjective_H__
