//////////////////////////////////////////////////////////////////////////////
//
// AVP_ServerEntity_RoomNode.h
//
// Created by AJK 15/01/2009
//
// Copyright (c) 2009 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header.
#include "Entity/ObjectiveSystem/RoomNode/AVP_ServerEntity_RoomNode.h"

// Asura
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "Core/Asura_LinkList.h"
#include "Core/Asura_OrientedVolume.h"

// AVP
#include "Entity/AVP_Entity_Classification.h"
#include "Entity/ObjectiveSystem/RoomNode/AVP_RoomNode_ExitLink.h"
#include "Entity/PhysicalObject/Actor/AVP_ServerEntity_Actor.h"

#ifdef ASURA_TOOLS
#include "EntityTemplate/Tools_EntityTemplate_System.h"
#endif

// Statics definitions
Asura_Hashed_List<AVP_ServerEntity_RoomNode> AVP_ServerEntity_RoomNode::s_xList;

AVP_ServerEntity_RoomNode::AVP_ServerEntity_RoomNode( Asura_Guid uGuid ) 
: PARENT( uGuid )
, m_bOldVersion( false ) 
{
	Classification = AVP_ENTITYCLASS_ROOMNODE;

	SetNeverUpdateClient();

	s_xList.AddToList(this);
}

AVP_ServerEntity_RoomNode::~AVP_ServerEntity_RoomNode()
{
	for ( Asura_LinkList_ConstIt< AVP_RoomNode_ExitLink* > xLinkListIter( m_xLinkList ); !xLinkListIter.Done(); xLinkListIter.Next() )
	{
		delete *xLinkListIter.GetCurrent();
	}

	s_xList.RemoveFromList(this);
}

void AVP_ServerEntity_RoomNode::ReadFromChunkStream(Asura_Chunk_Stream& xStream)
{
	//Version
	u_int uVersion;
	xStream	>> uVersion;

	//Parent
	PARENT::ReadFromChunkStream(xStream);
	if (uVersion == CURRENT_CHUNK_VERSION)
	{
		xStream >> m_xPosition;

		m_xVolumeGroup.ReadFromChunkStream( xStream );

		u_int uNumLinks = 0;
		xStream >> uNumLinks;

		u_int          uMyID;
		u_int          uLinkedExitLinkID;
		Asura_Guid     uRoom;
		Asura_Vector_3 xPosition;
		u_int          uFlags;

		for ( u_int u = 0; u < uNumLinks; ++u )
		{
			xStream >> uMyID;
			xStream >> uLinkedExitLinkID;
			xStream >> uRoom;  
			xStream >> xPosition;
			xStream >> uFlags;

			AVP_RoomNode_ExitLink* pxExitLink = new AVP_RoomNode_ExitLink( uMyID, uLinkedExitLinkID, uRoom, xPosition, uFlags );
			if ( pxExitLink )
			{
				// Add it to room list
				m_xLinkList.Push_Back( pxExitLink );
			}
			else
			{
				Asura_Assert(false, "Couldn't create exit link!");
			}
		}
	}
	else if (uVersion == 8)
	{
		xStream >> m_xPosition;

		m_xVolumeGroup.ReadFromChunkStream( xStream );

		u_int uNumLinks = 0;
		xStream >> uNumLinks;

		for ( u_int u = 0; u < uNumLinks; ++u )
		{
			u_int          uID;
			Asura_Guid     uRoom1, uRoom2;
			Asura_Vector_3 xPosition;
			u_int          uFlags;

			xStream >> uID;
			xStream >> uRoom1;  
			xStream >> uRoom2;
			xStream >> xPosition;
			xStream >> uFlags;

			if ( uRoom1 == ASURA_GUID_UNREGISTERED && uRoom2 != ASURA_GUID_UNREGISTERED )
			{
				uRoom1 = uRoom2;
				uRoom2 = ASURA_GUID_UNREGISTERED;
			}

			AVP_RoomNode_ExitLink* pxExitLink = new AVP_RoomNode_ExitLink( uID, xPosition, uRoom1, uRoom2, uFlags );
			if ( pxExitLink )
			{
				// Add it to room list
				m_xLinkList.Push_Back( pxExitLink );
			}
			else
			{
				Asura_Assert(false, "Couldn't create exit link!");
			}
		}

		m_bOldVersion = true;
		ConvertOldVersionLinks();
	}
	else if (uVersion == 7)
	{
		xStream >> m_xPosition;

		Asura_OrientedVolume xVolume;
		xStream >> xVolume.m_xBox;
		xStream >> xVolume.m_xOrient;

		bool bVolumeSet;
		xStream >> bVolumeSet;

		u_int uNumLinks = 0;
		xStream >> uNumLinks;

		for ( u_int u = 0; u < uNumLinks; ++u )
		{
			u_int          uID;
			Asura_Guid     uRoom1, uRoom2;
			Asura_Vector_3 xPosition;
			u_int          uFlags;

			xStream >> uID;
			xStream >> uRoom1;  
			xStream >> uRoom2;
			xStream >> xPosition;
			xStream >> uFlags;

			if ( uRoom1 == ASURA_GUID_UNREGISTERED && uRoom2 != ASURA_GUID_UNREGISTERED )
			{
				uRoom1 = uRoom2;
				uRoom2 = ASURA_GUID_UNREGISTERED;
			}

			AVP_RoomNode_ExitLink* pxExitLink = new AVP_RoomNode_ExitLink( uID, xPosition, uRoom1, uRoom2, uFlags );
			if ( pxExitLink )
			{
				// Add it to room list
				m_xLinkList.Push_Back( pxExitLink );
			}
			else
			{
				Asura_Assert(false, "Couldn't create exit link!");
			}
		}

		m_bOldVersion = true;
		ConvertOldVersionLinks();
	}
	else if (uVersion == 6)
	{
		xStream >> m_xPosition;

		Asura_OrientedVolume xVolume;
		xStream >> xVolume.m_xBox;
		xStream >> xVolume.m_xOrient;

		bool bVolumeSet;
		xStream >> bVolumeSet;

		u_int uNumLinks = 0;
		xStream >> uNumLinks;

		for ( u_int u = 0; u < uNumLinks; ++u )
		{
			Asura_Vector_3 xPosition;
			AVP_RoomNode_ExitLink* pxExitLink = NULL;
			if ( pxExitLink )
			{
				// Exit link already on global list
				pxExitLink->SetPosition( (xStream >> xPosition, xPosition) );
				m_xLinkList.Push_Back( pxExitLink );
			}
			else
			{
				// Create exit link
				//pxExitLink = new AVP_RoomNode_ExitLink( ( xStream >> xPosition, xPosition) , uRoom1, uRoom2 );
				if ( pxExitLink )
				{
					// Add it to room list
					m_xLinkList.Push_Back( pxExitLink );
				}
				else
				{
					Asura_Assert(false, "Couldn't create exit link!");
				}
			}
		}
	}
	else if (uVersion == 5)
	{
		xStream >> m_xPosition;

		Asura_OrientedVolume xVolume;
		xStream >> xVolume.m_xBox;
		xStream >> xVolume.m_xOrient;

		u_int uNumLinks = 0;
		xStream >> uNumLinks;

		for ( u_int u = 0; u < uNumLinks; ++u )
		{
			Asura_Guid uJunk;
			Asura_Vector_3 xJunk;
			xStream >> uJunk;
			xStream >> xJunk;
		}

		bool bVolumeSet;
		xStream >> bVolumeSet;
	}
	else if (uVersion == 4)
	{
		Asura_OrientedVolume xVolume;
		xStream >> xVolume.m_xBox;
		xStream >> xVolume.m_xOrient;

		u_int uNumLinks = 0;
		xStream >> uNumLinks;

		for ( u_int u = 0; u < uNumLinks; ++u )
		{
			Asura_Guid uJunk;
			Asura_Vector_3 xJunk;
			xStream >> uJunk;
			xStream >> xJunk;
		}

		bool bVolumeSet;
		xStream >> bVolumeSet;
	}
	else if (uVersion == 3)
	{
		Asura_OrientedVolume xVolume;
		xStream >> xVolume.m_xBox;
		xStream >> xVolume.m_xOrient;

		u_int uNumLinks = 0;
		xStream >> uNumLinks;

		bool bVolumeSet;
		xStream >> bVolumeSet;
	}
	else if (uVersion == 1)
	{
		Asura_OrientedVolume xVolume;
		xStream >> xVolume.m_xBox;
		xStream >> xVolume.m_xOrient;

		u_int uNumLinks = 0;
		xStream >> uNumLinks;

		bool bVolumeSet;
		xStream >> bVolumeSet;
	}

	// This entity doesn't need to update.
	SetNeverUpdates(true);
}

void AVP_ServerEntity_RoomNode::WriteToChunkStream(Asura_Chunk_Stream& xStream) const
{
	//Version
	xStream << CURRENT_CHUNK_VERSION;

	//Parent
	PARENT::WriteToChunkStream(xStream);

	xStream << m_xPosition;

	m_xVolumeGroup.WriteToChunkStream( xStream );

	xStream << m_xLinkList.Size();

	Asura_LinkList_ConstIt< AVP_RoomNode_ExitLink* > xLinkListIter( m_xLinkList );

	while ( !xLinkListIter.Done() )
	{
		xStream << (*xLinkListIter.GetCurrent())->GetMyID();
		xStream << (*xLinkListIter.GetCurrent())->GetLinkedExitLinkID();
		xStream << (*xLinkListIter.GetCurrent())->GetLinkedRoomGuid();
		xStream << (*xLinkListIter.GetCurrent())->GetPosition();
		xStream << (*xLinkListIter.GetCurrent())->GetFlags();
		xLinkListIter.Next();
	}
}

#ifdef ASURA_TOOLS
u_int AVP_ServerEntity_RoomNode::GetLinkedExitLinksCount() const
{
	Asura_LinkList_ConstIt< AVP_RoomNode_ExitLink* > xLinkListIter( m_xLinkList );
	u_int u = 0;
	while ( !xLinkListIter.Done() )
	{
		if ( (*xLinkListIter.GetCurrent())->GetLinkedRoomGuid() != ASURA_GUID_UNREGISTERED  )
		{
			++u;
		}
		xLinkListIter.Next();
	}
	return u;
}

AVP_RoomNode_ExitLink* AVP_ServerEntity_RoomNode::GetExitLinkTo( Asura_Guid uToRoomNode, u_int uID ) const
{
	Asura_LinkList_ConstIt< AVP_RoomNode_ExitLink* > xLinkListIter( m_xLinkList );
	while ( !xLinkListIter.Done() )
	{
		if ( (*xLinkListIter.GetCurrent())->GetLinkedRoomGuid() == uToRoomNode && (*xLinkListIter.GetCurrent())->GetLinkedExitLinkID() == uID  )
		{
			return *xLinkListIter.GetCurrent();
		}
		xLinkListIter.Next();
	}
	return NULL;
}
#endif // #ifdef ASURA_TOOLS

const AVP_RoomNode_ExitLink* AVP_ServerEntity_RoomNode::GetNearestExitLinkTo( Asura_Guid uToRoomNode, const AVP_ServerEntity_Actor& xActor ) const
{
	const AVP_ServerEntity_RoomNode* pxRoomNode        = AVP_ServerEntity_RoomNode::FindEntity( uToRoomNode );
	const AVP_RoomNode_ExitLink*     pxNearestExitLink = NULL;

	if ( pxRoomNode )
	{
	    float fShortestDistance = Asura_Maths::LargeFloat;
		float fCurrentDistance;
		
		const AVP_RoomNode_ExitLink* pxCurrentExitLink = NULL;
		Asura_LinkList_ConstIt< AVP_RoomNode_ExitLink* > xLinkListIter( m_xLinkList );
		while ( !xLinkListIter.Done() )
		{
			pxCurrentExitLink = *xLinkListIter.GetCurrent();
			if ( pxCurrentExitLink->GetLinkedRoomGuid() == uToRoomNode )
			{
				if (   xActor.GetClassification() == AVP_ENTITYCLASS_MARINE   && pxCurrentExitLink->IsUsedByMarine() 
					|| xActor.GetClassification() == AVP_ENTITYCLASS_ALIEN    && pxCurrentExitLink->IsUsedByAlien()
					|| xActor.GetClassification() == AVP_ENTITYCLASS_PREDATOR && pxCurrentExitLink->IsUsedByPredator()
					|| xActor.GetClassification() == AVP_ENTITYCLASS_PREDALIEN && pxCurrentExitLink->IsUsedByPredator())
				{
					if ( ( fCurrentDistance = ( pxCurrentExitLink->GetPosition() - xActor.GetPosition() ).MagnitudeSquared() ) < fShortestDistance )
					{
						fShortestDistance = fCurrentDistance;
						pxNearestExitLink = pxCurrentExitLink;
					}
				}
			}
			xLinkListIter.Next();
		}
	}
	return pxNearestExitLink;
}

u_int AVP_ServerEntity_RoomNode::GetMaxExitLinkID() const
{
	u_int uMaxID = 0;
	for ( Asura_LinkList_ConstIt< AVP_RoomNode_ExitLink* > xLinkListIter( m_xLinkList ); !xLinkListIter.Done(); xLinkListIter.Next() )
	{
		if ( uMaxID < (*xLinkListIter.GetCurrent())->GetMyID() ) 
		{
			uMaxID =(*xLinkListIter.GetCurrent())->GetMyID();
		}
	}
	return uMaxID;
}

bool AVP_ServerEntity_RoomNode::HasExitLinkForSpecies(const AVP_ServerEntity_Actor& xActor ) const
{
	const AVP_RoomNode_ExitLink* pxCurrentExitLink = NULL;
	for ( Asura_LinkList_ConstIt< AVP_RoomNode_ExitLink* > xLinkListIter( m_xLinkList ); !xLinkListIter.Done(); xLinkListIter.Next() )
	{
		pxCurrentExitLink = *xLinkListIter.GetCurrent();
		if (   xActor.GetClassification() == AVP_ENTITYCLASS_MARINE   && pxCurrentExitLink->IsUsedByMarine() 
			|| xActor.GetClassification() == AVP_ENTITYCLASS_ALIEN    && pxCurrentExitLink->IsUsedByAlien()
			|| xActor.GetClassification() == AVP_ENTITYCLASS_PREDATOR && pxCurrentExitLink->IsUsedByPredator()
			|| xActor.GetClassification() == AVP_ENTITYCLASS_PREDALIEN && pxCurrentExitLink->IsUsedByPredator() )
		{
			return true;
		}
	}
	return false;
}

void AVP_ServerEntity_RoomNode::ConvertOldVersionLinks()
{
	if ( m_bOldVersion )
	{
		for ( Asura_LinkList_ConstIt< AVP_RoomNode_ExitLink* > xLinkListIter1( m_xLinkList ); !xLinkListIter1.Done(); xLinkListIter1.Next() )
		{
			if ( !(*xLinkListIter1.GetCurrent())->IsConverted() )
			{
				if ( ((*xLinkListIter1.GetCurrent())->GetLinkedRoomGuid() != ASURA_GUID_UNREGISTERED) && ( (*xLinkListIter1.GetCurrent())->GetLinkedRoom2Guid() != ASURA_GUID_UNREGISTERED ) )
				{
					Asura_Hashed_List_It<AVP_ServerEntity_RoomNode> xRoomIter(AVP_ServerEntity_RoomNode::GetList());
					for ( ; !xRoomIter.Done(); xRoomIter.Next() )
					{
						if ( xRoomIter.GetCurrent()->IsOldVersion() && xRoomIter.GetCurrent() != this)
						{
							for ( Asura_LinkList_ConstIt< AVP_RoomNode_ExitLink* > xLinkListIter2( xRoomIter.GetCurrent()->m_xLinkList ); !xLinkListIter2.Done(); xLinkListIter2.Next() )
							{
								if ( !(*xLinkListIter2.GetCurrent())->IsConverted() )
								{
									if ( ((*xLinkListIter2.GetCurrent())->GetLinkedRoomGuid() != ASURA_GUID_UNREGISTERED) && ((*xLinkListIter2.GetCurrent())->GetLinkedRoom2Guid() != ASURA_GUID_UNREGISTERED ) )
									{
										if ( (*xLinkListIter1.GetCurrent())->GetMyID() == (*xLinkListIter2.GetCurrent())->GetMyID() )
										{
											(*xLinkListIter1.GetCurrent())->SetMyID( GetMaxExitLinkID() +  1);
											(*xLinkListIter2.GetCurrent())->SetMyID( xRoomIter.GetCurrent()->GetMaxExitLinkID() +  1);
											(*xLinkListIter1.GetCurrent())->SetLinkedExitLinkID( (*xLinkListIter2.GetCurrent())->GetMyID());
											(*xLinkListIter2.GetCurrent())->SetLinkedExitLinkID( (*xLinkListIter1.GetCurrent())->GetMyID());
											(*xLinkListIter1.GetCurrent())->SetLinkedRoomGuid( xRoomIter.GetCurrent()->GetGuid() );
											(*xLinkListIter2.GetCurrent())->SetLinkedRoomGuid( GetGuid() );
											(*xLinkListIter1.GetCurrent())->SetConverted( true );
											(*xLinkListIter2.GetCurrent())->SetConverted( true );
										}
									}
									else
									{
										(*xLinkListIter2.GetCurrent())->SetLinkedRoomGuid( ASURA_GUID_UNREGISTERED );
										(*xLinkListIter2.GetCurrent())->SetLinkedRoom2Guid( ASURA_GUID_UNREGISTERED );
										(*xLinkListIter2.GetCurrent())->SetMyID( xRoomIter.GetCurrent()->GetMaxExitLinkID() +  1);
										(*xLinkListIter2.GetCurrent())->SetLinkedExitLinkID( 0 );
										(*xLinkListIter2.GetCurrent())->SetConverted( true );
									}
								}
							}
						}
					}
				}
				else
				{
					(*xLinkListIter1.GetCurrent())->SetLinkedRoomGuid( ASURA_GUID_UNREGISTERED );
					(*xLinkListIter1.GetCurrent())->SetLinkedRoom2Guid( ASURA_GUID_UNREGISTERED );
					(*xLinkListIter1.GetCurrent())->SetMyID( GetMaxExitLinkID() +  1);
					(*xLinkListIter1.GetCurrent())->SetLinkedExitLinkID( 0 );
					(*xLinkListIter1.GetCurrent())->SetConverted( true );
				}
			}
		}
	}
}

#ifdef ASURA_TOOLS
void AVP_ServerEntity_RoomNode::ApplyChangesForTemplateInstance( Tools_EntityTemplateInstance* pxInstance )
{
	PARENT::ApplyChangesForTemplateInstance(pxInstance);

	m_xVolumeGroup.ApplyChangesForTemplateInstance( pxInstance );

	Asura_LinkList < AVP_RoomNode_ExitLink* > xTmpLinkList = m_xLinkList;

	m_xLinkList.Clear();

	Asura_LinkList_It< AVP_RoomNode_ExitLink* > xLinkListIter( xTmpLinkList );

	while ( !xLinkListIter.Done() )
	{
		AVP_RoomNode_ExitLink* pxLink = *xLinkListIter.GetCurrent();

		pxLink->SetLinkedRoomGuid( pxInstance->ConvertGuid( pxLink->GetLinkedRoomGuid()));
		pxLink->SetLinkedRoom2Guid( pxInstance->ConvertGuid( pxLink->GetLinkedRoom2Guid()));

		Asura_Vector_3 xPosition = pxLink->GetPosition();
		xPosition = pxInstance->TransformPosition( xPosition );
		pxLink->SetPosition( xPosition );
		m_xLinkList.Push_Back( pxLink ) ;
		xLinkListIter.Next();
	}
}

bool AVP_ServerEntity_RoomNode::CanBeIncludedInTemplate() const
{
	return (GetClassification() == AVP_ENTITYCLASS_ROOMNODE);
}
#endif // #ifdef ASURA_TOOLS
