//////////////////////////////////////////////////////////////////////////////
//
// AVP_ClientEntity_PhysicalObject.h
//
// Copyright (c) 2006 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif

#ifndef AVP_CLIENTENTITY_PHYSICALOBJECT_H___
#define AVP_CLIENTENTITY_PHYSICALOBJECT_H___ 1

// I N C L U D E S ///////////////////////////////////////////////////////////

// Engine independent.
#include "Entity/PhysicalObject/Asura_ClientEntity_PhysicalObject.h"
#include "Physics/Asura_Physics_Object.h"

// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////////

class Asura_Sound_Event;
class AVP_EntityDamage_ApplyToEntity;
class AVP_Message_PhysicalObject_ClientUpdate;
class AVP_Message_PhysicalObject_ClientCreate;

// C L A S S E S /////////////////////////////////////////////////////////////

class AVP_ClientEntity_PhysicalObject : public virtual Asura_ClientEntity_PhysicalObject
{   

public:
	static AVP_ClientEntity_PhysicalObject* FindEntity( Asura_Guid uGUID ) { return s_xList.FindEntity( uGUID ); }
	static AVP_ClientEntity_PhysicalObject* FindObject(Asura_Guid uGUID, bool bIgnoreDestroyedEntities = true ) { return s_xList.FindObject( uGUID, bIgnoreDestroyedEntities ); }
	static Asura_Classified_List<AVP_ClientEntity_PhysicalObject>* GetList( void ) { return s_xList.GetList(); }
	static void Create( const Asura_Message* pxMessage );

	virtual bool HandleMessage( const Asura_Message* pxMsg );
	virtual bool CanSendMessageToServer(void) const	{ return(true); }

	bool SendMessageToServer(Asura_Message* pxMessage) const;

	const Asura_Sound_Event* GetSoundEventFromBlueprints(const Asura_Hash_ID uBlueprintParamHash) const;

	// Physics functions...
	void ChangePhysicsObjectType(const ASURA_PHYSICS_OBJECT_TYPE eType)	{ if(m_pxDynamicObject != NULL) { m_pxDynamicObject->ChangeType(eType); } }
	void SetLinVelocity(const Asura_Vector_3& xVelocity)				{ if(m_pxDynamicObject != NULL) { m_pxDynamicObject->SetLinVelocity(xVelocity); } }
	Asura_Handle GetPhysicsObjectHandle(void) const						{ return((NULL == m_pxDynamicObject) ? ASURA_INVALID_HANDLE : m_pxDynamicObject->GetHandle()); }

	// Overrides for physics object-access...
	virtual void TeleportTo(const Asura_Vector_3& xPosition, const Asura_Matrix_3x3& xOrientation);

	virtual const Asura_Vector_3&	GetCurrentPosition		(void) const;
	virtual const Asura_Matrix_3x3&	GetCurrentOrientation	(void) const;
	virtual void					GetCurrentVelocity		(Asura_Vector_3& xVelocity) const;

	float							GetInitialHealth(void) const { return m_fInitialHealth; }

	void	SetIgnoreForSurfaceReorientation(bool bIgnore);
	bool	IsIgnoredForSurfaceReorientation(void) const	{ return(m_bIgnoreForSurfaceReorientation); }
	bool	IsDontShowUpInMotionTracker(void)         const	{ return(m_bDontShowUpInMotionTracker); }
	void	SetDontShowUpInMotionTracker(bool bTrue)		{ m_bDontShowUpInMotionTracker = bTrue; }
	bool	IsBurning()	const								{ return(m_bIsBurning);}
	void	SetBurningFlag( bool bIsBurning )				{ m_bIsBurning = bIsBurning;	}


	// Can the specified entity be used for surface reorientation? Checks physical objects and containers.
	static bool CanUseForSurfaceReorientation(Asura_Guid uGuid);

	bool	GetBoundingSphere	(Asura_Vector_3& xBSCentre, float& fBSRadius) const;

	bool	GetObjectSpaceMarkerPoint	(const Asura_Hash_ID uMarkerPointHash, Asura_Vector_3* pxPosition, Asura_Matrix_3x3* pxOrientation) const;
	bool	GetWorldSpaceMarkerPoint	(const Asura_Hash_ID uMarkerPointHash, Asura_Vector_3* pxPosition, Asura_Matrix_3x3* pxOrientation) const;

	Asura_Physics_Object*	GetPhysicsObject(void) const	{ return m_pxDynamicObject; }

	virtual void	Update();
	virtual void	Render();
	virtual void	PostCreateInitialise();

	virtual bool	MakeRagdoll() { return false; }

	bool					IsEntityCurrentlyMoving() const					{ return m_bIsEntityCurrentlyMoving; }

	virtual void	GetMeleeOrigin(Asura_Vector_3& xOrigin) const	{ xOrigin = GetCurrentPosition(); }

	virtual void	OnBeginBurning();
	virtual void	OnEndBurning();

	virtual bool    IsADeadBody()     const { return false; }
	virtual bool    IsRagDollAsleep() const { return false; }
	virtual void    WakeUpRagdoll()   const {};

	Asura_Hash_ID	GetMaterialType() const { return m_uMaterialType; }

	void ForceBoneUpdate() { m_fForcedBoneUpdateTimer = 0.0f; }

	bool AttachPFX(Asura_Hash_ID uMarkerID, Asura_Hash_ID uPFXID, bool bMoveWithEffect = false);

protected:
	AVP_ClientEntity_PhysicalObject(void);
	virtual ~AVP_ClientEntity_PhysicalObject();

	Asura_Physics_Object*	m_pxDynamicObject;	// A pointer to the physics-object to use by this actor.
	Asura_Hash_ID			m_uMaterialType;
	float					m_fInitialHealth;
	u_int					m_uBurningEffectID;

	void	ProcessClientUpdate(const AVP_Message_PhysicalObject_ClientUpdate* pxData, float fServerTime, bool bProcessPosAndOri = true);
	void	ProcessClientCreate(const AVP_Message_PhysicalObject_ClientCreate* pxData, float fServerTime, bool bProcessPosAndOri = true);

	// Virtual event functions...
	virtual	void	OnDeath			(const AVP_EntityDamage_ApplyToEntity& xDamage)	{}
	virtual void	OnApplyDamage	(const AVP_EntityDamage_ApplyToEntity& xDamage)	{}

	// Virtual blueprint functions...
	virtual void InitialiseFromBlueprint( const Asura_Blueprint& xBlueprint );

private:
	// Private typedefs...
	typedef Asura_ClientEntity_PhysicalObject PARENT_ASURA;

	// Private variables...
	union
	{
		u_int		m_uFlags;									// Single access point for the flags.

		struct
		{
			u_int	m_bIgnoreForSurfaceReorientation	: 1,	// Flag to say whether this entity is ignored when using surface-reorientation.
					m_bDontShowUpInMotionTracker        : 1,
					m_bIsBurning						: 1,	// true = object is burning. false = object isn't burning.	[EP]
														: 0;	// 31 spare-bits
		};
	};

	float m_fForcedBoneUpdateTimer;
	float m_fMaxTimeBetweenBoneUpdates;

	static Asura_Combined_List<AVP_ClientEntity_PhysicalObject> s_xList;

	// Prevent these from ever being called...
	virtual	void	MakeDead(void)	{}
};

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

#endif // #ifndef AVP_CLIENTENTITY_PHYSICALOBJECT_H___
