//////////////////////////////////////////////////////////////////////////////
//
// AVP_ServerEntity_PhysicalObject.h
//
// Copyright (c) 2006 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef AVP_SERVERENTITY_PHYSICALOBJECT_H___
#define AVP_SERVERENTITY_PHYSICALOBJECT_H___ 1

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// Engine independent.
#include "Core/Asura_HashTree.h"
#include "Debugging/Asura_Assert.h"
#include "Entity/Damage/AVP_DamageMultiplier.h"
#include "Entity/PhysicalObject/Asura_ServerEntity_PhysicalObject.h"
#include "Sound/Asura_Sound_Event_System.h"

// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////////

class AVP_EntityDamage_RequestVictim;
class AVP_EntityDamage_ApplyToEntity;
class AVP_ServerEntity_GamePlayVolume;

class AVP_ServerEntity_PhysicalObject_FlammableHandler;

// C L A S S E S /////////////////////////////////////////////////////////////

class AVP_ServerEntity_PhysicalObject : public virtual Asura_ServerEntity_PhysicalObject
{
public:
    static AVP_ServerEntity_PhysicalObject* FindEntity( Asura_Guid uGUID ) { return s_xList.FindEntity( uGUID ); }
    static const Asura_Hashed_List<AVP_ServerEntity_PhysicalObject>* GetList( void ) { return &s_xList; }    

    virtual bool	HandleMessage( const Asura_Message* pxMessage );
	virtual	void	TakeDamageExtended(const AVP_EntityDamage_RequestVictim& xDamage, const bool bForceFatal, const bool bSendMessageToClient = true );

    virtual void	WriteToChunkStream( Asura_Chunk_Stream& xStream ) const;
	virtual void	ReadFromChunkStream( Asura_Chunk_Stream& xStream );

	// Overridden position/orientation/velocity accessor functions...
	virtual void					TeleportTo(const Asura_Vector_3& xPosition, const Asura_Matrix_3x3& xOrientation, const bool bInformClients);

	virtual	void					SetLinVelocity(const Asura_Vector_3& xVelocity);
	virtual const Asura_Vector_3&	GetVelocity(void) const;

	virtual	void					SetPosition(const Asura_Vector_3& xPosition);
	virtual const Asura_Vector_3&	GetPosition(void) const;

#ifdef ASURA_MAX_NO_OF_STORED_POSITIONS
	const Asura_Vector_3*			GetStoredPositions() const					{ return &(m_axStoredPositions[0]); }
	const float*					GetTimeStampsForStoredPositions() const		{ return &(m_afTimeStampsForStoredPositions[0]); }
	u_int							GetNumberOfStoredPositions() const			{ return m_uNumberOfStoredPositions; }
	
	void							ResetStoredPositions(); 
	void							StoreCurrentPosition( float fTimeStamp ); 

	static void						ResetAllStoredPositions(); 
	static void						EndOfPhysicsUpdateCallback( float fTimeStamp ); 
#endif // #ifdef ASURA_MAX_NO_OF_STORED_POSITIONS


	virtual	void					SetOrientation(const Asura_Matrix_3x3& xOrientation);
	virtual const Asura_Matrix_3x3&	GetOrientation(void) const;

    // Legacy chunk handling, to be removed as soon as possible.
    virtual bool IsFilledFromChunkBuffer( void ) const { return false; }
    virtual void InitialiseFromChunkBuffer( const char*& pcChunkBfr ) { Asura_Assert(false, "Not supported."); }
    virtual int GetChunkBufferSize( void ) { Asura_Assert( false, "Shouldn't be here" ); return 0; }
    virtual void FillInChunkBuffer( char*& pcChunkBfr ) { Asura_Assert( false, "Shouldn't be here" ); }
	virtual bool IsChunkBufferServerOnly(void) const	{ return(true); }

	bool GetBoundingSphere(Asura_Vector_3& xBSCentre, float& fBSRadius) const;
	const Asura_Bounding_Box* GetBoundingBox() const;

	bool	GetObjectSpaceMarkerPoint(const Asura_Hash_ID uMarkerPointHash, Asura_Vector_3* pxPosition, Asura_Matrix_3x3* pxOrientation) const;
	bool	GetWorldSpaceMarkerPoint(const Asura_Hash_ID uMarkerPointHash, Asura_Vector_3* pxPosition, Asura_Matrix_3x3* pxOrientation) const;

	virtual void GetTargetPosition( Asura_Vector_3& xPosition ) const;

	void	SetIgnoreForSurfaceReorientation(bool bIgnore);
	bool	IsIgnoredForSurfaceReorientation(void) const	{ return(m_bIgnoreForSurfaceReorientation); }

	// Can the specified entity be used for surface reorientation? Checks physical objects and containers.
	static bool CanUseForSurfaceReorientation(Asura_Guid uGuid);

	Asura_Physics_Object*	GetPhysicsObject(void) const	{ return(m_pxDynamicObject); }
	bool					HasPhysicsObject(void) const	{ return(m_pxDynamicObject != NULL); }

	void		SetOwnerGuid(Asura_Guid uOwner)				{ m_uOwnerGuid = uOwner; }
	Asura_Guid	GetOwnerGuid() const						{ return m_uOwnerGuid; }

#ifdef ASURA_TOOLS
	void AddOnHealthEventData( const int iOnHealthEventID );
	void RemoveOnHealthEventData(int iEventID);
	float	GetHealthValueForOnHealthEvent( const int iOnHealthEventID );
#endif

	virtual void GetMeleeOrigin(Asura_Vector_3& xOrigin) const { xOrigin = GetPosition(); }

	AVP_ServerEntity_GamePlayVolume* GetCurrentGamePlayVolume() const;
	const AVP_DamageMultiplier*	FindDamageMultiplier(const Asura_Hash_ID uDamageTypeHash) const	{ return(m_xDamageMultiplierContainer.GetDataByHash(uDamageTypeHash)); }

	float	GetInitialHealth() const { return m_fInitialHealth; }

	virtual bool IsADeadBody() const { return  false; }

	Asura_Hash_ID GetMaterialType() const { return m_uMaterialType; }
	void SetMaterialType( const Asura_Hash_ID uMaterialType ) { m_uMaterialType = uMaterialType; }

	void SetAsFlammable( bool bIsFlammable );
	bool IsFlammable()		{ return m_bIsFlammable; }
	bool IsCurrentlyBurning();

	virtual bool IsHuman() const { return false; }
	virtual bool ExplodesOnFatalPlasmaDamage() const { return false ;}

protected:
	// Protected functions...
    AVP_ServerEntity_PhysicalObject(void);
    virtual ~AVP_ServerEntity_PhysicalObject( void );

	// Overridden event functions...
    virtual void	OnEntityActivate();
    virtual void	OnEntityDeactivate();
    virtual void	OnEntityActivateMessage();

	virtual void	Update();

	virtual void	OnApplyDamage(const AVP_EntityDamage_ApplyToEntity& xDamage)	{}
	virtual void	OnDeath(const AVP_EntityDamage_ApplyToEntity& xDamage);

	virtual void	SendDamageMessageToClient(const AVP_EntityDamage_ApplyToEntity& xDamage);

	// This returns the updated damage value based on the location hit, with the return parameters returning any limb-related details (limbs newly destroyed, limbs gibbed (should be a subset of the previous)).
	// This also updates the skin element damage values, should that be of relevance to this type of entity.
	virtual float CalculateAndUpdateLocationSpecificDamage(const AVP_EntityDamage_RequestVictim& xDamage, u_int& uDestroyedSkinElements, u_int& uGibbedSkinElements, bool bForceFatal);

    virtual void InitialiseFromBlueprint( const Asura_Blueprint& xBlueprint );

#if defined(ASURA_TOOLS)
	// Function to patch anim/skin file IDs...
	void	Tools_PatchSkinAndAnimFileIDs(void);
#endif

	// All physics are now done client-side - so we need to keep track of our own position/orientation...
	Asura_Vector_3		m_xLinVelocity;		// The linear velocity of this entity.
	Asura_Vector_3		m_xPosition;		// The position of this entity.
	Asura_Matrix_3x3	m_xOrientation;		// The orientation of this entity.
	Asura_Guid			m_uOwnerGuid;		// Who owns this object? (e.g. projectiles, who fired them...objects, who created them...etc)
#ifdef ASURA_MAX_NO_OF_STORED_POSITIONS
	u_int				m_uNumberOfStoredPositions; 
	Asura_Vector_3		m_axStoredPositions[ASURA_MAX_NO_OF_STORED_POSITIONS]; 
	float				m_afTimeStampsForStoredPositions[ASURA_MAX_NO_OF_STORED_POSITIONS]; 
#endif // #ifdef ASURA_MAX_NO_OF_STORED_POSITIONS

private:
	// Private typedefs...
	typedef Asura_TemplateHashTree<AVP_DamageMultiplier> DamageMultiplierContainer;
	typedef Asura_ServerEntity_PhysicalObject PARENT_ASURA;

	// Private variables...
	union
	{
		u_int		m_uFlags;

		struct
		{
			u_int	m_bResetNoCollFlagOnUnhide			: 1,	// ???
					m_bIgnoreForSurfaceReorientation	: 1,	// Flag to say whether this entity is ignored when using surface-reorientation.
					m_bIsFlammable						: 1,	// Flag to say whether the entity can be set on fire or not.
														: 0;	// 30 spare-bits
		};
	};

	Asura_Hash_ID m_uMaterialType;

	// Health-based static message events.
	DamageMultiplierContainer	m_xDamageMultiplierContainer;	// The container used to store ALL of the damage multipliers for this entity.
	float						m_fInitialHealth;
	u_int						m_uNumOnHealthEvents;

	struct EventIDAndHealth
	{
		int	  m_iEventID;
		float m_fHealthFraction;
	}* m_pxOnHealthEvents;

	mutable Asura_Guid m_uCurrentGamePlayVolumeID; // cache volume id - do not use directly - always call GetCurrentGamePlayVolume()

	void SetInitialHealth( float fHealth ) { m_fInitialHealth = fHealth; }

	// Prevent these from ever being called...
	virtual void	TakeDamage(float fDamage)	{ PARENT_ASURA::TakeDamage(fDamage); }
	virtual	void	MakeDead(void)				{} // Also blanked so that the version that actually does something is only ever called through OnDeath

	static Asura_Hashed_List<AVP_ServerEntity_PhysicalObject> s_xList;

	// Also this object has double use. When it is non zero, it indicates that object IsFlammable. If
	AVP_ServerEntity_PhysicalObject_FlammableHandler* m_xFlammableHandler;
};




// This class will handle all states for burning of the object.
class AVP_ServerEntity_PhysicalObject_FlammableHandler
{
public:
	void	Init( float fMaxTimeToStopBurning, float fMaxHealthPercentageToTake, float fStartBurningDamageTreshold );
	float	Update( float fMaxHealth, Asura_Guid uFromGuid );
	void	ResetReceivedFireDamage()						{m_fAlreadyReceivedFireDamage=0.0f;}
	void	AddReceivedFireDamage( float fFireDamage )	
	{
		if( fFireDamage>0.0f )
		{
			m_fAlreadyReceivedFireDamage+=fFireDamage;
			m_fCurrenTimeToStopBurning = m_fMaxTimeToStopBurning;		// each time we will receive damage, extend the time of burning.
		}
	}

	void	SendMessageToClient_StartBurning( Asura_Guid uFromGuid );
	void	SendMessageToClient_StopBurning( Asura_Guid uFromGuid );
	bool	IsCurrentlyBurning()	{return m_bIsBurning;}

	float	CalculateCurrentBurningDamage( float fMaxHealth, float fCurrentTimeSlice )
	{
		float fDamagePerSecond = (fMaxHealth * m_fMaxHealthPercentageToTake) / m_fMaxTimeToStopBurning;	// I guess that this function may be pre-calculated. It depends only on fact if the maxHealth can be changed on the fly or not.
		float fDamagePerThisFrame = fDamagePerSecond * fCurrentTimeSlice;
		return fDamagePerThisFrame;
	}

	AVP_ServerEntity_PhysicalObject_FlammableHandler();

protected:
	bool	m_bIsBurning;
	float	m_fAlreadyReceivedFireDamage;
	float	m_fCurrenTimeToStopBurning;
	float	m_fMaxTimeToStopBurning;
	float	m_fMaxHealthPercentageToTake;
	float	m_fStartBurningDamageTreshold;
};



#endif // #ifndef AVP_SERVERENTITY_PHYSICALOBJECT_H___
