//////////////////////////////////////////////////////////////////////////////
//
// AVP_Actor_Messages.cpp
//
// Created by NJD - 23 November 2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"

// header
#include "Entity/PhysicalObject/Actor/AVP_Actor_Messages.h"

// asura
#include "Entity/PhysicalObject/Asura_ServerEntity_PhysicalObject.h"
#include "Network/Asura_NetMsg.h"

// avp
#include "Actor/AVP_Actor_AIAnimStateMachine.h"
#include "Actor/AVP_Actor_PlayerAnimStateHandler.h"
#include "Entity/PhysicalObject/Actor/AVP_ClientEntity_Actor.h"
#include "Entity/PhysicalObject/Actor/AVP_ServerEntity_Actor.h"
#include "Inventory/AVP_Inventory_HashIDs.h"
#include "Inventory/AVP_Inventory_System.h"
#include "Inventory/Items/AVP_InventoryItem_Weapon.h"
#include "Network/AVP_Message_Compression.h"

//////////////////////////////////////////////////////////////////////////////
//
// AVP_Message_Actor_ClientUpdate
//
//////////////////////////////////////////////////////////////////////////////

void AVP_Message_Actor_ClientUpdate::FillActor( const AVP_ServerEntity_Actor& xActor )
{
	const AVP_Inventory_System*			pxInventorySystem	= xActor.GetInventorySystem();

    FillPhysicalObject( &xActor );
	m_xAimDirection = xActor.GetAimDirection();
	m_xLookAtPosition = xActor.GetLookAtPosition();
	m_xMovementDirection = xActor.GetMovementDirection();

    m_xAnimSequenceData	= xActor.GetAnimSequenceData();
    Asura_Assert( xActor.GetCurrentAnim() < UCHAR_MAX, "AVP_Message_Actor_ClientUpdate::FillActor - only storing current anim in a u_char" );
    m_ucCurrentAnim     = static_cast<u_char>(xActor.GetCurrentAnim());
    m_ucLoopData		= static_cast<u_char>(xActor.GetLoopData());
    m_bShouldTween      = xActor.ShouldTween();

	m_fAIStateAnimTweenTime		= xActor.GetAIAnimTweenTime();
	m_bAIStateAnimSync			= !xActor.GetPlayNextAIStateAnimFromStart();

	m_uAIAnimState	= AVP_Actor_AIAnimStateMachine::ACTOR_STATE_IDLE;
	const AVP_Actor_AIAnimStateMachine* pxStateMachine = xActor.GetAIAnimStateMachine();
	if ( pxStateMachine != NULL )
	{
		m_uAIAnimState	= pxStateMachine->GetCurrentState();
		m_uAIAnimStateAfterMove = pxStateMachine->GetStateAfterMove();
		m_fTimeInAIAnimState = pxStateMachine->GetTimeInState();
		m_bAIAnimStateLooped = pxStateMachine->IsLoopedState( m_uAIAnimState );
	}
	
    m_uActorGuidImAttacking = xActor.GetGuidOfActorBeingAttackedByUs();
	m_uPlayerInfoGuid = xActor.GetPlayerInfoGuid();

    m_eGroundType					= static_cast<u_char>(xActor.GetGroundType());
    m_bIsPlayer						= xActor.IsPlayer();
	m_bMoving						= xActor.IsMoving();
	m_bMovingSlow					= xActor.IsMovingSlow();
	m_bMovingFast					= xActor.IsMovingFast();
	m_bIsDestructible				= xActor.IsDestructible();

	m_uCurrentWeaponHash			= pxInventorySystem ? pxInventorySystem->GetCurrentItemID(uAVP_INVENTORY_GROUPHASH_WEAPON) : ASURA_HASH_ID_UNSET;
	m_uDeadAnimHash                 = xActor.GetDeadAnimHash();
	m_bDeadBody                     = xActor.IsADeadBody();
	m_fPlayerSpeedParameter			= xActor.GetPlayerSpeedParameter();
	m_fPlayerMoveAngle				= xActor.GetPlayerMoveAngle();
	
	// MGH 28/1/2009: Send the timestamp for the last pos/ori information received from the client.
	m_bClientPosAndOriTimeValid		= xActor.IsClientPosAndOriTimeValid();
	m_fClientPosAndOriTime			= xActor.GetClientPosAndOriTime();
}


#ifdef ASURA_SUPPORT_NETMSG_SYSTEM
DEBUGVAR float dbg_fBodyOrientationChangeThreshold = 0.0001f;

void AVP_Message_Actor_ClientUpdate::RegisterFields(Asura_NetMsg* pxNetMsg)
{
	const AVP_Message_Actor_ClientUpdate xDummy;
	AVP_Message_PhysicalObject_ClientUpdate::RegisterFields(pxNetMsg);

	Asura_NetMsg_BeginRegisteringSubclass(pxNetMsg, xDummy, m_xAnimSequenceData);
	Axon_Gamescene_AnimSequenceData::RegisterFields(pxNetMsg);
	Asura_NetMsg_FinishedRegisteringSubclass(pxNetMsg);

	Asura_NetMsg_RegisterField(pxNetMsg, xDummy, Asura_FieldType_vector3, m_xAimDirection, 1, Asura_Threshold_Magnitude, dbg_fBodyOrientationChangeThreshold);
	Asura_NetMsg_RegisterField(pxNetMsg, xDummy, Asura_FieldType_vector3, m_xLookAtPosition, 1, Asura_Threshold_Magnitude, dbg_fBodyOrientationChangeThreshold);
	Asura_NetMsg_RegisterField(pxNetMsg, xDummy, Asura_FieldType_vector3, m_xMovementDirection, 1, Asura_Threshold_Magnitude, dbg_fBodyOrientationChangeThreshold);
	Asura_NetMsg_RegisterField(pxNetMsg, xDummy, Asura_FieldType_uchar, m_ucCurrentAnim, 1, Asura_Threshold_AnyChange, 0.0f);
	Asura_NetMsg_RegisterField(pxNetMsg, xDummy, Asura_FieldType_uchar, m_ucLoopData, 1, Asura_Threshold_AnyChange, 0.0f);
	Asura_NetMsg_RegisterField(pxNetMsg, xDummy, Asura_FieldType_uchar, m_bShouldTween, 1, Asura_Threshold_AnyChange, 0.0f);
	Asura_NetMsg_RegisterField(pxNetMsg, xDummy, Asura_FieldType_uchar, m_eGroundType, 1, Asura_Threshold_AnyChange, 0.0f);
	Asura_NetMsg_RegisterField(pxNetMsg, xDummy, Asura_FieldType_uchar, m_bIsPlayer, 1, Asura_Threshold_AnyChange, 0.0f);
	Asura_NetMsg_RegisterField(pxNetMsg, xDummy, Asura_FieldType_uchar, m_bAIStateAnimSync, 1, Asura_Threshold_AnyChange, 0.0f);
	Asura_NetMsg_RegisterField(pxNetMsg, xDummy, Asura_FieldType_uchar, m_bMoving, 1, Asura_Threshold_AnyChange, 0.0f);
	Asura_NetMsg_RegisterField(pxNetMsg, xDummy, Asura_FieldType_uchar, m_bMovingSlow, 1, Asura_Threshold_AnyChange, 0.0f);
	Asura_NetMsg_RegisterField(pxNetMsg, xDummy, Asura_FieldType_uchar, m_bMovingFast, 1, Asura_Threshold_AnyChange, 0.0f);
	Asura_NetMsg_RegisterField(pxNetMsg, xDummy, Asura_FieldType_uchar, m_bIsDestructible, 1, Asura_Threshold_AnyChange, 0.0f);
	Asura_NetMsg_RegisterField(pxNetMsg, xDummy, Asura_FieldType_uchar, m_bDeadBody, 1, Asura_Threshold_AnyChange, 0.0f);
	Asura_NetMsg_RegisterField(pxNetMsg, xDummy, Asura_FieldType_uchar, m_bClientPosAndOriTimeValid, 1, Asura_Threshold_AnyChange, 0.0f);
	Asura_NetMsg_RegisterField(pxNetMsg, xDummy, Asura_FieldType_Guid, m_uActorGuidImAttacking, 1, Asura_Threshold_AnyChange, 0.0f);
	Asura_NetMsg_RegisterField(pxNetMsg, xDummy, Asura_FieldType_Guid, m_uPlayerInfoGuid, 1, Asura_Threshold_AnyChange, 0.0f);
	Asura_NetMsg_RegisterField(pxNetMsg, xDummy, Asura_FieldType_uint, m_uCurrentWeaponHash, 1, Asura_Threshold_AnyChange, 0.0f);
	Asura_NetMsg_RegisterField(pxNetMsg, xDummy, Asura_FieldType_uint, m_uDeadAnimHash, 1, Asura_Threshold_AnyChange, 0.0f);
	Asura_NetMsg_RegisterField(pxNetMsg, xDummy, Asura_FieldType_uint, m_uAIAnimState, 1, Asura_Threshold_AnyChange, 0.0f);
	Asura_NetMsg_RegisterField(pxNetMsg, xDummy, Asura_FieldType_float, m_fAIStateAnimTweenTime, 1, Asura_Threshold_AnyChange, 0.0f);
	Asura_NetMsg_RegisterField(pxNetMsg, xDummy, Asura_FieldType_float, m_fPlayerSpeedParameter, 1, Asura_Threshold_AnyChange, 0.0f);
	Asura_NetMsg_RegisterField(pxNetMsg, xDummy, Asura_FieldType_float, m_fPlayerMoveAngle, 1, Asura_Threshold_AnyChange, 0.0f);
	Asura_NetMsg_RegisterField(pxNetMsg, xDummy, Asura_FieldType_float, m_fClientPosAndOriTime, 1, Asura_Threshold_AnyChange, 0.0f );
}
#endif//#ifdef ASURA_SUPPORT_NETMSG_SYSTEM

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

void AVP_Message_ActorPlayerCommands::FillActor( const AVP_ClientEntity_Actor& xActor )
{
	const AVP_Inventory_System*					pxInventorySystem		= xActor.GetInventorySystem();
	const AVP_Actor_PlayerAnimStateHandler*		pxPlayerStateHandler	= xActor.GetPlayerStateHandler();
	Asura_AVector3								xAVector;

	xActor.GetAimDirection( xAVector );
	m_xAimDirection = xAVector;
	xActor.GetLookAtPosition( xAVector );
	m_xLookAtPosition = xAVector;

	xActor.GetCurrentVelocity( m_xVelocity );
	m_xPosition = xActor.GetCurrentPosition();
	m_xOrientation.FromMatrix( xActor.GetCurrentOrientation() );

	m_bMoving		= xActor.IsMoving();
	m_bMovingSlow	= xActor.IsMovingSlow();
	m_bMovingFast	= xActor.IsMovingFast();

	m_uCurrentWeaponHash = pxInventorySystem ? pxInventorySystem->GetCurrentItemID( uAVP_INVENTORY_GROUPHASH_WEAPON ) : ASURA_HASH_ID_UNSET;

	m_fSpeedParameter = pxPlayerStateHandler ? pxPlayerStateHandler->GetSpeedParametric() : 0.0f;
	m_fMoveAngle = pxPlayerStateHandler ? pxPlayerStateHandler->GetMoveAngle() : 0.0f;
}

// Normalised vector component compression
const u_int s_uBITS_PER_FLOAT_NORMALISED = 10; 
const float s_fMAX_FLOAT_NORMALISED = 1.0f; 
const float s_fMIN_FLOAT_NORMALISED = -s_fMAX_FLOAT_NORMALISED; 

// Quaternion component compression
const u_int s_uBITS_PER_FLOAT_QUATERNION = 10; 
const float s_fMAX_FLOAT_QUATERNION = 1.0f; 
const float s_fMIN_FLOAT_QUATERNION = -s_fMAX_FLOAT_QUATERNION;

// Network compressed positions
const u_int s_uBITS_PER_FLOAT_HORIZONTAL_POS = 16; 
const float s_fMAX_FLOAT_HORIZONTAL_POS = 4096.0f; 
const float s_fMIN_FLOAT_HORIZONTAL_POS = -s_fMAX_FLOAT_HORIZONTAL_POS; 

const u_int s_uBITS_PER_FLOAT_VERTICAL_POS = 14; 
const float s_fMAX_FLOAT_VERTICAL_POS = 1024.0f; 
const float s_fMIN_FLOAT_VERTICAL_POS = -s_fMAX_FLOAT_VERTICAL_POS; 

// Parametric values
const u_int s_uBITS_PER_FLOAT_PARAMETRIC = 11; 
const float s_fMAX_FLOAT_PARAMETRIC = 10.0f; 
const float s_fMIN_FLOAT_PARAMETRIC = -s_fMAX_FLOAT_PARAMETRIC; 

void AVP_Message_ActorPlayerCommands::Pack( Asura_BitPacker& xPacker ) const
{
	// Pack normalised aim direction 
	AVP_Compressed_Float xCompressedNorm( s_fMIN_FLOAT_NORMALISED, s_fMAX_FLOAT_NORMALISED, s_uBITS_PER_FLOAT_NORMALISED ); 
	
	xCompressedNorm.FromFloat( m_xAimDirection.x ); 
	xCompressedNorm.Pack( xPacker ); 
	xCompressedNorm.FromFloat( m_xAimDirection.y ); 
	xCompressedNorm.Pack( xPacker ); 
	xCompressedNorm.FromFloat( m_xAimDirection.z ); 
	xCompressedNorm.Pack( xPacker ); 

	// Pack look at position
	AVP_Compressed_Float xCompressedHorizPos( s_fMIN_FLOAT_HORIZONTAL_POS, s_fMAX_FLOAT_HORIZONTAL_POS, s_uBITS_PER_FLOAT_HORIZONTAL_POS ); 
	AVP_Compressed_Float xCompressedVertPos( s_fMIN_FLOAT_VERTICAL_POS, s_fMAX_FLOAT_VERTICAL_POS, s_uBITS_PER_FLOAT_VERTICAL_POS ); 

	xCompressedHorizPos.FromFloat( m_xLookAtPosition.x ); 
	xCompressedHorizPos.Pack( xPacker ); 
	xCompressedVertPos.FromFloat( m_xLookAtPosition.y ); 
	xCompressedVertPos.Pack( xPacker ); 
	xCompressedHorizPos.FromFloat( m_xLookAtPosition.z ); 
	xCompressedHorizPos.Pack( xPacker ); 

	// Pack velocity 
	// TODO: we really need this? We also have speed 
	xCompressedHorizPos.FromFloat( m_xVelocity.x ); 
	xCompressedHorizPos.Pack( xPacker ); 
	xCompressedVertPos.FromFloat( m_xVelocity.y ); 
	xCompressedVertPos.Pack( xPacker ); 
	xCompressedHorizPos.FromFloat( m_xVelocity.z ); 
	xCompressedHorizPos.Pack( xPacker ); 

	// Pack position
	xCompressedHorizPos.FromFloat( m_xPosition.x ); 
	xCompressedHorizPos.Pack( xPacker ); 
	xCompressedVertPos.FromFloat( m_xPosition.y ); 
	xCompressedVertPos.Pack( xPacker ); 
	xCompressedHorizPos.FromFloat( m_xPosition.z ); 
	xCompressedHorizPos.Pack( xPacker ); 

	// Pack orientation 
	AVP_Compressed_Float xQuaternion( s_fMIN_FLOAT_QUATERNION, s_fMAX_FLOAT_QUATERNION, s_uBITS_PER_FLOAT_QUATERNION ); 
	xQuaternion.FromFloat( m_xOrientation.x ); 
	xQuaternion.Pack( xPacker ); 
	xQuaternion.FromFloat( m_xOrientation.y ); 
	xQuaternion.Pack( xPacker ); 
	xQuaternion.FromFloat( m_xOrientation.z ); 
	xQuaternion.Pack( xPacker ); 
	xQuaternion.FromFloat( m_xOrientation.w ); 
	xQuaternion.Pack( xPacker ); 

	// Pack movement flags
	xPacker.PackBool( m_bMoving ); 
	xPacker.PackBool( m_bMovingSlow ); 
	xPacker.PackBool( m_bMovingFast ); 

	// Pack current weapon hash
	const bool bPackCurrentWeaponHash = ( m_uCurrentWeaponHash != ASURA_HASH_ID_UNSET ); 
	xPacker.PackBool(bPackCurrentWeaponHash);
	if ( bPackCurrentWeaponHash )
	{
		xPacker.Pack( m_uCurrentWeaponHash, sizeof(Asura_Hash_ID) * 8 ); 
	}

	// Pack speed parametric
	AVP_Compressed_Float xCompressedParametric( s_fMIN_FLOAT_PARAMETRIC, s_fMAX_FLOAT_PARAMETRIC, s_uBITS_PER_FLOAT_PARAMETRIC ); 

	xCompressedParametric.FromFloat( m_fSpeedParameter ); 
	xCompressedParametric.Pack( xPacker ); 
	
	// Pack move angle
	AVP_Compressed_Angle xAngle; 
	xAngle.FromAngle( m_fMoveAngle ); 
	xPacker.Pack( xAngle.GetData(), xAngle.GetNumberOfBits() ); 
}

void AVP_Message_ActorPlayerCommands::Unpack( Asura_BitUnpacker &xUnpacker )
{
	// Unpack normalised aim direction 
	AVP_Compressed_Float xCompressedNorm( s_fMIN_FLOAT_NORMALISED, s_fMAX_FLOAT_NORMALISED, s_uBITS_PER_FLOAT_NORMALISED ); 
	
	xCompressedNorm.Unpack( xUnpacker ); 
	xCompressedNorm.ToFloat( m_xAimDirection.x ); 
	xCompressedNorm.Unpack( xUnpacker ); 
	xCompressedNorm.ToFloat( m_xAimDirection.y ); 
	xCompressedNorm.Unpack( xUnpacker ); 
	xCompressedNorm.ToFloat( m_xAimDirection.z ); 
	// Normalise the direction to fix quantization errors
	m_xAimDirection.Normalise(); 

	// Unpack look at position
	AVP_Compressed_Float xCompressedHorizPos( s_fMIN_FLOAT_HORIZONTAL_POS, s_fMAX_FLOAT_HORIZONTAL_POS, s_uBITS_PER_FLOAT_HORIZONTAL_POS ); 
	AVP_Compressed_Float xCompressedVertPos( s_fMIN_FLOAT_VERTICAL_POS, s_fMAX_FLOAT_VERTICAL_POS, s_uBITS_PER_FLOAT_VERTICAL_POS ); 

	xCompressedHorizPos.Unpack( xUnpacker ); 
	xCompressedHorizPos.ToFloat( m_xLookAtPosition.x ); 
	xCompressedVertPos.Unpack( xUnpacker ); 
	xCompressedVertPos.ToFloat( m_xLookAtPosition.y ); 
	xCompressedHorizPos.Unpack( xUnpacker ); 
	xCompressedHorizPos.ToFloat( m_xLookAtPosition.z ); 

	// Unpack velocity
	xCompressedHorizPos.Unpack( xUnpacker ); 
	xCompressedHorizPos.ToFloat( m_xVelocity.x ); 
	xCompressedVertPos.Unpack( xUnpacker ); 
	xCompressedVertPos.ToFloat( m_xVelocity.y ); 
	xCompressedHorizPos.Unpack( xUnpacker ); 
	xCompressedHorizPos.ToFloat( m_xVelocity.z ); 

	// Unpack position
	xCompressedHorizPos.Unpack( xUnpacker ); 
	xCompressedHorizPos.ToFloat( m_xPosition.x ); 
	xCompressedVertPos.Unpack( xUnpacker ); 
	xCompressedVertPos.ToFloat( m_xPosition.y ); 
	xCompressedHorizPos.Unpack( xUnpacker ); 
	xCompressedHorizPos.ToFloat( m_xPosition.z ); 

	// Unpack orientation
	AVP_Compressed_Float xQuaternion( s_fMIN_FLOAT_QUATERNION, s_fMAX_FLOAT_QUATERNION, s_uBITS_PER_FLOAT_QUATERNION ); 
	xQuaternion.Unpack( xUnpacker ); 
	xQuaternion.ToFloat( m_xOrientation.x ); 
	xQuaternion.Unpack( xUnpacker ); 
	xQuaternion.ToFloat( m_xOrientation.y ); 
	xQuaternion.Unpack( xUnpacker ); 
	xQuaternion.ToFloat( m_xOrientation.z ); 
	xQuaternion.Unpack( xUnpacker ); 
	xQuaternion.ToFloat( m_xOrientation.w ); 
	// Normalise the quaternion after the trip
	m_xOrientation.Normalise(); 

	// Pack movement flags
	m_bMoving = xUnpacker.UnpackBool(); 
	m_bMovingSlow = xUnpacker.UnpackBool(); 
	m_bMovingFast = xUnpacker.UnpackBool(); 

	// Unpack current weapon hash
	m_uCurrentWeaponHash = ASURA_HASH_ID_UNSET; 
	const bool bUnpackCurrentWeaponHash = xUnpacker.UnpackBool(); 
	if ( bUnpackCurrentWeaponHash )
	{
		m_uCurrentWeaponHash = xUnpacker.Unpack( sizeof(Asura_Hash_ID) * 8 ); 
	}

	// Unpack speed parametric
	AVP_Compressed_Float xCompressedParametric( s_fMIN_FLOAT_PARAMETRIC, s_fMAX_FLOAT_PARAMETRIC, s_uBITS_PER_FLOAT_PARAMETRIC ); 

	xCompressedParametric.Unpack( xUnpacker ); 
	xCompressedParametric.ToFloat( m_fSpeedParameter ); 

	// Unpack move angle
	AVP_Compressed_Angle xAngle; 
	xAngle.SetData( static_cast<u_short>( xUnpacker.Unpack( xAngle.GetNumberOfBits() ))); 
	xAngle.ToAngle( m_fMoveAngle ); 
}
