//////////////////////////////////////////////////////////////////////////////
//
// AVP_ServerEntity_Actor.h
//
// Basic actor class, at this level we'll assume the shape instance is still being used.
// A subclass can set up a posable anim instead if that's needed.
//
// Copyright (c) 2006 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif

#ifndef ___AVP_SERVERENTITY_ACTOR_H___
#define ___AVP_SERVERENTITY_ACTOR_H___ 1

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// asura
#include "Core/Asura_List.h"

// avp
#include "Actor/AVP_Actor_PhysicsObjectHandler.h"
#include "Core/AVP_Settings.h"
#include "Entity/PhysicalObject/AVP_ServerEntity_PhysicalObject.h"
#include "Entity/AVP_Entity_EventIDs.h"

// axon
#include "AnimTemplates/Axon_Gamescene_AnimSequenceData.h"

// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////////

class AVP_Actor_AIAnimStateMachine;
class AVP_ClientEntity_Actor;
class AVP_Collision_Descriptor;
class AVP_Inventory_System;
class AVP_Inventory_Group;
class AVP_Message_ActorPlayerCommands;
class AVP_Brain_Actor;
class AVP_Multiplayer_DamageCounter; 
class AVP_Actor_PickupHandler;
class AVP_ServerEntity_Pickup;
class AVP_InventoryItem_Weapon;
class AVP_Actor_ObjectiveHandler;

// C L A S S E S /////////////////////////////////////////////////////////////

class AVP_ServerEntity_Actor : public AVP_ServerEntity_PhysicalObject
{
	typedef AVP_ServerEntity_PhysicalObject PARENT; 
	friend class ToolsAVP_Entity_Actor;

public:
	// Public functions...
	void			SetToStartPosition(const Asura_Hash_ID uStartPointBlueprintParameterHash);

	bool			HasEverBeenDamaged() const { return ( m_fLastDamageAmount > 0.0f ); }
	float			GetLastDamageInterval() const { return m_fLastDamageInterval; }
	float			GetTimeSinceLastDamage() const { return m_fTimeSinceDamage; }

	void			SetAIStateAnimsNeedToTween( float fAIAnimTweenTime ) { m_fAIAnimTweenTime = fAIAnimTweenTime; }
	void			SetPlayNextAIStateAnimFromStart() { m_bAIAnimPlayNextFromStart = true; }

	float 			GetAIAnimTweenTime() const { return m_fAIAnimTweenTime; }
	bool			GetPlayNextAIStateAnimFromStart() const { return m_bAIAnimPlayNextFromStart; }

	void			AwardLoadsOfDamage();

	// Virtual Overridden Public Functions...
	virtual	void	Update(void);
	virtual	bool	HandleMessage(const Asura_Message* pxMessage);
    virtual void	PostLoadInitialise(void);
	virtual void	PreDestructionShutdown();
	virtual	bool	CanBeImpaled(const u_int uBoneID) const									{ return false; }
	virtual	void	TakeDamageExtended(const AVP_EntityDamage_RequestVictim& xDamage, const bool bForceFatal, const bool bSendMessageToClient=true );

	virtual void	AddPermenantlyActiveItems(AVP_Inventory_Group* pxGroup) {};

	virtual void	MakePlayer(Asura_Guid uPlayerGuid);
	virtual void	MakeAI();
	virtual void	DisconnectPlayer(); // Similar to MakeAI, but designed just to chuck the player info out without going on to create some AI handling stuff.
	Asura_Guid		GetPlayerInfoGuid() const { return m_uPlayerInfoGuid; }
	
	AVP_Settings::Species GetSpeciesByClassification() const 	{ return AVP_Settings::GetSpeciesByClassification( static_cast<AVP_ENTITY_CLASSIFICATION>(GetClassification())); }

	// Override functions used to make sure our local-orientation is always correct in the physics object handler...
	virtual	void	SetOrientation(const Asura_Matrix_3x3& xOrientation);
	virtual void	TeleportTo(const Asura_Vector_3& xPosition, const Asura_Matrix_3x3& xOrientation, const bool bInformClients);

	// Chunk processing functions...
	virtual	void	ReadFromChunkStream(Asura_Chunk_Stream& xStream);
	virtual	void	WriteToChunkStream(Asura_Chunk_Stream& xStream) const;
	virtual void	WriteToClientCreateStream(Asura_Chunk_Memory_Stream& xStream) const;
	    
	// Legacy chunk handling, to be removed as soon as possible...
	virtual	bool	IsFilledFromChunkBuffer(void) const	{ return(false); }

    AVP_Brain_Actor* GetActorBrain() const;

	AVP_Actor_PhysicsObjectHandler::GroundType	GetGroundType(void) const	{ return(static_cast<AVP_Actor_PhysicsObjectHandler::GroundType>(m_eGroundType)); }

	bool	IsPlayer(void) const	{ return m_bIsPlayer; }
	bool	IsCloaked(const bool bFullyCloaked) const;

	bool	IsMoving()			const { return m_bMoving; }
	bool	IsMovingSlow()		const { return m_bMovingSlow; }
	bool	IsMovingMedium()	const { return IsMoving() && !m_bMovingSlow && !m_bMovingFast; }
	bool	IsMovingFast()		const { return m_bMovingFast; }

	void	SetMoving( bool bSet )	{ m_bMoving = bSet; }
	void	SetMovingSlow( bool bSet )	{ m_bMovingSlow = bSet; }
	void	SetMovingFast( bool bSet )	{ m_bMovingFast = bSet; }

	Asura_Guid	GetGuidOfActorBeingAttackedByUs(void) const 				{ return m_uGuidOfActorBeingAttackedByUs; }
	void		SetGuidOfActorBeingAttackedByUs(Asura_Guid uGuid)			{ m_uGuidOfActorBeingAttackedByUs = uGuid; }
    u_int       GetCurrentAnim(void) const									{ return m_uCurrentAnim; }
    void        SetCurrentAnim(u_int uAnim)									{ m_uCurrentAnim = uAnim; }
    u_int       GetLoopData(void) const										{ return m_uLoopData; }
    void        SetLoopData(u_int uLoopData)								{ m_uLoopData = uLoopData; }
    bool        ShouldTween(void) const										{ return m_bShouldTween; }
    void        SetShouldTween(bool bShouldTween)							{ m_bShouldTween = bShouldTween; }
	Asura_Guid	GetInitialGamesceneGuid(void) const							{ return m_uInitialGamesceneGuid; }
	void		SetInitialGamesceneGuid(Asura_Guid uGuid)					{ m_uInitialGamesceneGuid = uGuid; }
	Asura_Guid  GetInitialSquadGuid() const									{ return m_uInitialSquadGuid; }
	void		SetInitialSquadGuid( const Asura_Guid uGuid )				{ m_uInitialSquadGuid = uGuid; }
	u_int       GetCurrentWPID();
	void		SetMarkOutOfPlayOnDeactivate( const bool bMark )			{ m_bMarkOutOfPlayOnDeactivate = bMark; }

	AVP_Actor_PhysicsObjectHandler*			GetPhysicsObjectHandler(void)		{ return(m_pxPhysicsObjectHandler); }
	const AVP_Actor_PhysicsObjectHandler*	GetPhysicsObjectHandler(void) const { return(m_pxPhysicsObjectHandler); }
	AVP_Inventory_System*					GetInventorySystem(void)			{ return(m_pxInventorySystem); }
	const AVP_Inventory_System*				GetInventorySystem(void) const		{ return(m_pxInventorySystem); }
	const Axon_Gamescene_AnimSequenceData&	GetAnimSequenceData(void) const		{ return m_xAnimSequenceData; }
    void                                    SetAnimSequenceData( const Axon_Gamescene_AnimSequenceData& xData ) { m_xAnimSequenceData = xData; }
	const Asura_Collision_Object*			GetActorCollisionObject(void) const	{ return( m_pxDynamicObject->GetCollisionObject() ); }
	Asura_AVector3							GetCurrentGravityDirection(void) const;
	AVP_Actor_ObjectiveHandler*				GetObjectiveHandler(void)			{ return(m_pxObjectiveHandler); }
	const AVP_Actor_ObjectiveHandler*		GetObjectiveHandler(void) const		{ return(m_pxObjectiveHandler); }
	AVP_Actor_PickupHandler*		        GetPickupHandler(void)			    { return(m_pxPickupHandler); }
	const AVP_Actor_PickupHandler*	        GetPickupHandler(void) const		{ return(m_pxPickupHandler); }

	// Returns whether there is space to move by the required local space displacement.
	// Could potentially go into asura, but I'm not sure where.
	bool HasSpaceForAnim( const Asura_Hash_ID uAnimHashID,
						  const float fMovementScale = 1.0f, const bool bCollideWithActors = false,
						  const Asura_Vector_3& xOffset = Asura_Maths::ZeroVector3, const u_int uNumTestSegments = 4 ) const;
	bool GetAnimMovementOffset( const Asura_Hash_ID uAnimHashID, const float fParametricTime,
								Asura_Vector_3* pxPosition, Asura_Quat* pxOrientQ = NULL ) const;

	virtual void GetTargetPosition( Asura_Vector_3& xPosition ) const { GetChestPosition( xPosition ); }
	void GetHeadPosition( Asura_Vector_3& xPosition ) const;
	void GetChestPosition( Asura_Vector_3& xPosition ) const;
	void GetHeadOrientation( Asura_Matrix_3x3& xOrientation ) const;
	void GetChestOrientation( Asura_Matrix_3x3& xOrientation ) const;
	void GetHeadFowardVector( Asura_Vector_3& xForward ) const;
	// Nominal offset so that we don't return a position possibly intersecting with the ground.
	virtual float GetHeadPositionYOffset() const { return -0.5f; }
	virtual float GetChestPositionYOffset() const { return -0.5f; }

	// return valid value only if the ranged weapon is equipped
	bool GetMuzzlePosition( Asura_Vector_3& xPosition ) const;
	const AVP_InventoryItem_Weapon* GetCurrentWeapon() const;

	// Counter for number of kills performed by this actor, only relevant to players in multiplayer games
	void SetKillStreak(u_int uKillStreak)			{ m_uKillStreak = uKillStreak; }
	u_int GetKillStreak(void) const					{ return m_uKillStreak; }

	AVP_Actor_AIAnimStateMachine*		GetAIAnimStateMachine()			{ return m_pxAIAnimStateMachine; }
	const AVP_Actor_AIAnimStateMachine*	GetAIAnimStateMachine() const	{ return m_pxAIAnimStateMachine; }

	const Asura_Hashed_List<AVP_Multiplayer_DamageCounter>* GetReceivedDamageList( void ) const		{ return m_pxReceivedDamageList; }

#ifdef ASURA_TOOLS
    virtual void ApplyChangesForTemplateInstance( Tools_EntityTemplateInstance* pxInstance );
	virtual bool CanBeIncludedInTemplate() const { return true; }
	bool GetIsDeadBody() const { return m_bDeadBody; }
	void SetIsDeadBody( const bool bIsDeadBody ) { m_bDeadBody = bIsDeadBody; }
#endif

	void SetCollisionDescriptor( const Asura_Hash_ID uCollDescHash );

	// Full Body rig stuff
	const Asura_Vector_3& GetAimDirection() const { return m_xAimDirection; }
	void SetAimDirection( const Asura_Vector_3& xAimDir ) { m_xAimDirection = xAimDir; }

	const Asura_Vector_3& GetLookAtPosition() const { return m_xLookAtPosition; }
	void SetLookAtPosition( const Asura_Vector_3& xLookAtPos ) { m_xLookAtPosition = xLookAtPos; }

	const Asura_Vector_3& GetMovementDirection() const { return m_xMovementDirection; }
	void SetMovementDirection( const Asura_Vector_3& xMovementDirection ) { m_xMovementDirection = xMovementDirection; }

	void RegenHealthBlocks( u_int uNumBlocks );
	const AVP_Collision_Descriptor* GetDefaultCollisionDescriptor();
	void SetNumActiveHealthBlocks();

	float	GetPlayerSpeedParameter()	const	{ return m_fPlayerSpeedParameter; }
	float	GetPlayerMoveAngle()		const	{ return m_fPlayerMoveAngle; }

	// Pickups
	virtual void OnPickup(AVP_ServerEntity_Pickup* pxPickup, float fQuantity, float fSecondaryQuantity, bool bSetAsCurrent = true);
	void DropPickup() const;
	void DropPickup(Asura_Hash_ID uHash) const;
	void DropTool()	const;


	Asura_Hash_ID         GetDeadAnimHash() const { return m_uDeadAnimHashID; }
	virtual bool		  IsADeadBody() const { return m_bDeadBody; }

	Asura_Hash_ID GetAnimStateBlueprintType() const { return m_uAnimStateBlueprintType; }

	void SetInitiallyInactive( bool bInitiallyInactive ) { m_bNewPlayerInitiallyInactive = bInitiallyInactive; }

	Asura_Guid GetPickupGuid() const { return m_uPickupGuid; }

	bool IsRagdoll() const;

	// MGH 28/1/2009: functions for sending authoritative position updates from each client to all other clients via the server.
	bool  IsClientPosAndOriTimeValid() const	{ return m_bClientPosAndOriTimeValid; }
	float GetClientPosAndOriTime() const		{ return m_fClientPosAndOriTime; }

	bool IsReorienting() const { return m_pxPhysicsObjectHandler ? m_pxPhysicsObjectHandler->IsReorienting() : false; }

	u_short GetFlagsToIgnore() const { return m_usFlagsToIgnore; }

	virtual Asura_Hash_ID GetCurrentGrappleBlueprintID() const;

	virtual void DestroyAcidVolume() {}

	void SendGS2AnimFlagEvent()	{ SendMessagesForEvent( AVP_ENTITY_EVENT_GS2_ACTION_ANIM_FLAG_HIT ); }

	static AVP_ServerEntity_Actor*							FindEntity(const Asura_Guid uGuid)	{ return(s_xList.FindEntity(uGuid)); }
	static const Asura_Hashed_List<AVP_ServerEntity_Actor>*	GetList(void)						{ return(&s_xList); }

protected:
	AVP_ServerEntity_Actor(const u_short usFlagsToIgnore, const Asura_Guid uGUID);
	virtual ~AVP_ServerEntity_Actor(void);

	void ProcessActorPlayerCommands( const AVP_Message_ActorPlayerCommands* pxData, float fMessageSentTime );

	virtual bool InitialiseInventory(void);
	virtual void MakeCreateMessage(Asura_Guid uRequestingClient, bool bSendGuaranteed = false);
	virtual	void MakeClientUpdateMessage(void);
    virtual void InitialiseFromBlueprint(const Asura_Blueprint& xBlueprint);
	virtual void SendDamageMessageToClient(const AVP_EntityDamage_ApplyToEntity& xDamage);
	virtual void OnEntityActivateMessage();
	virtual void OnEntityActivate();
	virtual void OnEntityDeactivate();
	virtual void SendOnDestructionMessages( void );
	virtual void OnDeath(const AVP_EntityDamage_ApplyToEntity& xDamage);
	
	void 	SendClientUpdateMessage(Asura_Message* pxMessage);
    void 	QueueMessage(const Asura_Message* pxMessage );
	void	UpdateMovementSoundsForAI();
	virtual void RegisterMovementSoundsForAI();
	void	RegisterMoveStartingSounds(const Asura_Message* pxMessage);
	void	RegisterMoveFinishingSounds(const Asura_Message* pxMessage);

	void UpdateHealth(void);
	virtual void	OnApplyDamage(const AVP_EntityDamage_ApplyToEntity& xDamage);

	bool			IsClientsTweenStorePositionDataValid(void) const;
	virtual float	GetHeadPositionOffset() const  { return 0.0f; }
	virtual float	GetChestPositionOffset() const { return 0.0f; }

	virtual Asura_Hash_ID GetMovesBlueprintType() const { return ASURA_HASH_ID_UNSET; }

	virtual void UpdateMovement() {}
	virtual void HandleInteractions() {};
	void		 RestorePreviousWeapon() const;

	bool GetClientAnimVelocityOS( const AVP_ClientEntity_Actor& xClient, Asura_AVector3& xAnimVelocity ) const;
	bool GetClientAnimAngularVelocityOS( const AVP_ClientEntity_Actor& xClient, Asura_Angle& fAngleOut, Asura_AVector3& xAngVelyAxisOut ) const;

	bool IsDamageIgnoredByThisPlayer( const AVP_EntityDamage_RequestVictim& xDamage ) const; 
	void UpdateDamageCounters( const AVP_EntityDamage_RequestVictim& xDamage ); 
	
	AVP_Actor_AIAnimStateMachine*	m_pxAIAnimStateMachine;				// A pointer to our AIAnimStateMachine.
	AVP_Actor_PhysicsObjectHandler*	m_pxPhysicsObjectHandler;			// A pointer to the class used to keep track of the physics-object.
	AVP_Actor_PickupHandler*        m_pxPickupHandler;                  // Pickup handler
	AVP_Inventory_System*			m_pxInventorySystem;				// The inventory-system to use for this actor.
	Asura_Angle						m_fObjectSpaceAngVelocityAngle;		// The current angle for this actor's angular velocity.
	Asura_AVector3					m_xObjectSpaceAngVelocityAxis;		// The current axis for this actor's angular velocity.
	AVP_Actor_ObjectiveHandler*     m_pxObjectiveHandler;               // Objective handler

	Asura_Guid						m_uPickupGuid;

	float m_fClientPosAndOriTime; // Timestamp for client pos and ori forwarded from each client to other machines.

private:
	void SetPhysicsToIgnore(Asura_Guid uGuid);
	void SetObjectSpaceAngVelocity(const Asura_Vector_3& xAxis, const Asura_Angle fAngle)	{ m_xObjectSpaceAngVelocityAxis = xAxis; m_fObjectSpaceAngVelocityAngle = fAngle; }

	Axon_Gamescene_AnimSequenceData m_xAnimSequenceData;				// Gamescene animation sequence data
	u_int                           m_uCurrentAnim;						// *** Redundant after the Anim State machine is working *** GBT
	u_int                           m_uLoopData;						// *** Redundant after the Anim State machine is working *** GBT
	bool                            m_bShouldTween;						// *** Redundant after the Anim State machine is working *** GBT
	Asura_Guid						m_uGuidOfActorBeingAttackedByUs;	// Guid of the actor we are attacking
	Asura_Vector_3					m_xAimDirection;
	Asura_Vector_3					m_xLookAtPosition;
	Asura_Vector_3					m_xMovementDirection;
	u_short							m_usFlagsToIgnore;					// Flags for the physics-objects to ignore collision.
	u_short							m_usCurrentWPID;

	Asura_Vector_3					m_xDeadPos;
	Asura_Matrix_3x3				m_xDeadOri;

	Asura_Hash_ID					m_uAnimStateBlueprintType;

	Asura_Guid	m_uPlayerInfoGuid;

	u_int	m_uMaxNumHealthBlocks;
	u_int	m_uNumActiveHealthBlocks;
	float	m_fHealthPerBlock;
	float	m_fTimeSinceDamage;
	float	m_fHealthRefillSpeed;
	float	m_fTimeBeforeHealthRecharge;

	union
	{
		u_int m_uChunkingFlags;
		struct 
		{
			u_int	m_eGroundType				: 4,	// Flag to say whether we're on the ground.
					m_bIsPlayer					: 1,
					m_bDeadBody                 : 1,
												: 0;
		};
	};

	union
	{
		u_int m_uNonChunkingFlags;
		struct 
		{
			u_int	m_bSentClientUpdateMessage		: 1,
					m_bMoving						: 1,	
					m_bMovingSlow					: 1,
					m_bMovingFast					: 1,
                    m_bAIAnimPlayNextFromStart		: 1,
				    m_bMarkOutOfPlayOnDeactivate	: 1,
					m_bNewPlayerInitiallyInactive	: 1,
					m_bClientPosAndOriTimeValid	    : 1,
													: 0;
		};
	};

	Asura_Guid	m_uInitialGamesceneGuid;			// Starting gamescene guid.
	Asura_Guid	m_uInitialSquadGuid;				// Initial squad to join. If unregistered then we should join the default squad.

    float		m_fMovementNoiseUpdateTimer;
	Asura_Hashed_List<AVP_Multiplayer_DamageCounter>*	m_pxReceivedDamageList;		// Stores a list of people who damaged us, and how much 
	float		m_fLastDamageTime;		// Stores the time when we last took damage
	float		m_fLastDamageAmount;	// Stores the damage taken in the last attack
	float		m_fLastDamageInterval;	// Stores the gap between the previous & latest damage
	u_int		m_uKillStreak;			// Stores the number of kills in a row. Only players will update this. 

	float		m_fAIAnimTweenTime;

	float		m_fPlayerSpeedParameter;
	float		m_fPlayerMoveAngle;

	Asura_Hash_ID m_uDeadAnimHashID;

	// Any messages that need forwarding to the client are stored here while we do not have a client entity.
    Asura_List< Asura_Message > m_xMessageQueue;

#ifdef ASURA_ASSERT
	bool m_bInitFromBP;
#endif

#if defined(ASURA_SUPPORT_COMMAND_CONSOLE)
	static int	s_iConsoleVarsCount;
#endif	// defined(ASURA_SUPPORT_COMMAND_CONSOLE)
	
	static Asura_Hashed_List<AVP_ServerEntity_Actor>	s_xList;
	static Asura_Handle FindPhysicsObjectHandle(Asura_Guid uGuid);
	static void SetPhysicsHandleToIgnore(Asura_Guid uGuid, Asura_Handle iHandle);
	static const u_int uCURRENT_CHUNK_VERSION = 9;
};

#endif // #ifndef ___AVP_SERVERENTITY_ACTOR_H___
