//////////////////////////////////////////////////////////////////////////////
//
// AVP_ServerVehicle_Dropship.h
// Author : Sharon Galtzur
// Date   : 24-Sep-2008
//
// Copyright (c) 2007 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header.
#include "Entity/PhysicalObject/Actor/Vehicles/Dropship/AVP_ServerVehicle_Dropship.h"

// Engine independent.
#include "Affectors/Asura_Affector.h"
#include "Affectors/Asura_Affector_System.h"
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "Core/Asura_Timers.h"
#include "Mesh/Asura_Env_System.h"
#include "Network/Asura_Network.h"
#include "Network/Asura_Message.h"
#include "Shapes/Asura_Shape_System.h"
#include "Physics/Asura_Physics.h"
#include "Camera/Asura_Camera.h"


#include "Debugging/Asura_DebugLineRender.h"


// Project independent.
#include "Actor/AVP_Actor_InputHandler.h"
#include "Blueprint/AVP_Blueprint_HashIDs.h"
#include "Core/AVP_Settings.h"
#include "Entity/AVP_Entity_Classification.h"
#include "Entity/AVP_Messages.h"
#include "Entity/PhysicalObject/Actor/AVP_Actor_Messages.h"
#include "Entity/PhysicalObject/Actor/Vehicles/Dropship/AVP_Messages_Dropship.h"
#include "Entity/PhysicalObject/Actor/Vehicles/Dropship/AVP_ClientVehicle_Dropship.h"
#include "Moves/AVP_Movement_System.h"

// Axon
#include "Core/Axon_Messages.h"
#include "Core/Axon_System.h"
#include "Core/Axon_Sound_Registry.h"
#include "Core/BrainTypes/AVP_Brain_Actor_Dropship.h"
#include "Behaviour/BehaviourTypes/AVP_Behaviour_Types.h"

// Constant variables...

DEBUGVAR float dbg_fMaxVerticalAcc = 5.0f;
DEBUGVAR float dbg_fMaxHorizontalAcc = 10.0f;
DEBUGVAR float dbg_fMaxTurnAcc = Asura_Maths::Pi * 60.0f / 180.0f;
DEBUGVAR float dbg_fMaxVerticalVelocity = 40.0f;
DEBUGVAR float dbg_fMaxPlaneVelocity = 20.0f;
DEBUGVAR float dbg_fMaxTurnVelocity = Asura_Maths::Pi * 60.0f / 180.0f;

// for 0.1 sec
 DEBUGVAR float dbg_fPropVer = 3.5f;
 DEBUGVAR float dbg_fIntVer = 0.8f;
 DEBUGVAR float dbg_fDerVer = 1.5f;
 DEBUGVAR float dbg_fPropHor = 2.5f;
 DEBUGVAR float dbg_fIntHor = 1.8f;
 DEBUGVAR float dbg_fDerHor = 1.5f;
 DEBUGVAR float dbg_fPropTurn = 2.5f;
 DEBUGVAR float dbg_fIntTurn = 0.8f;
 DEBUGVAR float dbg_fDerTurn = 1.5f;

DEBUGVAR float dbg_fMaxLeanAngle = Asura_Maths::DegToRad(30.0f);
DEBUGVAR float dbg_fMaxLeanAngleSpeed = Asura_Maths::DegToRad(40.0f);
DEBUGVAR float dbg_fMaxDropAcceleration = 2.0f;

DEBUGVAR float dbg_fMaxLaserVelocity = 4.5f;

DEBUGVAR float dbg_fVelocityNoiseSize = 2.0f;
DEBUGVAR float dbg_fVelocityNoiseChangeMinTime = 1.0f;
DEBUGVAR float dbg_fVelocityNoiseChangeMaxTime = 2.0f;

// Static variables...
Asura_Hashed_List<AVP_ServerVehicle_Dropship> AVP_ServerVehicle_Dropship::s_xList(2);


// C L A S S   I M P L E M E N T A T I O N S /////////////////////////////////


AVP_ServerVehicle_Dropship::AVP_ServerVehicle_Dropship (const Asura_Guid uGUID)
: Asura_Entity_Server(uGUID)
, PARENT(0, uGUID)
, m_uGS2StartNode(ASURA_GUID_UNREGISTERED)
, m_xDestination(Asura_Maths::ZeroVector3)
, m_xCurrentVelocity(Asura_Maths::ZeroVector3)
, m_xCurrentVelocityNoise(Asura_Maths::ZeroVector3)
, m_fTimeTillVelocityNoiseChanges(0.0f)
, m_xLastVelocity(Asura_Maths::ZeroVector3)
, m_xCurrentHorizontalAcceleration(Asura_Maths::ZeroVector3)
, m_xCurrentAcceleration(Asura_Maths::ZeroVector3)
, m_xCurrentLean(0.0f,-1.0f, 0.0f)
, m_fCurrentDropAcceleration(0.0f)
, m_xCurrentAim(Asura_Maths::ZeroVector3)
, m_fCurrentTurnVelocity(0.0f)
, m_xPIDY(dbg_fPropVer, dbg_fIntVer, dbg_fDerVer, dbg_fMaxVerticalAcc)
, m_xPIDX(dbg_fPropHor, dbg_fIntHor, dbg_fDerHor, dbg_fMaxHorizontalAcc)
, m_xPIDZ(dbg_fPropHor, dbg_fIntHor, dbg_fDerHor, dbg_fMaxHorizontalAcc)
, m_xPIDAim(dbg_fPropTurn, dbg_fIntTurn, dbg_fDerTurn, dbg_fMaxTurnAcc)
, m_fTimeSinceControlUpdate(0.0)
, m_bIsLaserOn(false)
, m_xLaserTarget(Asura_Maths::ZeroVector3)
, m_xLaserActualTarget(Asura_Maths::ZeroVector3)
, m_uNumberOfMissiles(0)
, m_fVelocityFraction(0.0f)
{
	Classification = AVP_ENTITYCLASS_DROPSHIP;

	// Update the data...
	SetGravityScale(0.0f); // Floating

	Asura_Matrix_3x3 xOrientation = m_pxPhysicsObjectHandler->GetPhysicsObject()->GetOrientation();
	SetAimDirection(Asura_Vector_3(xOrientation.e13, xOrientation.e23, xOrientation.e33));

	m_xDestination = GetPosition();
	m_xCurrentAim = GetAimDirection();


	// Add to the list...
	s_xList.AddToList(this);
}

AVP_ServerVehicle_Dropship::~AVP_ServerVehicle_Dropship ()
{
	// Remove from the list...
	s_xList.RemoveFromList(this);
}

void AVP_ServerVehicle_Dropship::MakeCreateMessage( Asura_Guid uRequestingClient, bool bSendGuaranteed /*= false */ )
{
	//for now just use the client update
	Asura_Message* pxMessage = Asura_Network::CreateMessage(ASURA_MSG_CLIENTCREATE, sizeof(AVP_Message_Actor_ClientUpdate), GetClassification());
	if(pxMessage)
	{
		AVP_Message_Actor_ClientUpdate* pxData = static_cast<AVP_Message_Actor_ClientUpdate*>(pxMessage->GetData());
		pxData->FillActor(*this);

		pxMessage->SetToGuid( ASURA_GUID_MY_CLIENT_ENTITIES );
		pxMessage->SetFromGuid( GetGuid() );
		if (bSendGuaranteed)
		{
			pxMessage->UseGuaranteedDelivery();
		}
		Asura_Network::SendToSpecificMachine( pxMessage, uRequestingClient );
	}
}


void AVP_ServerVehicle_Dropship::UpdateMovement()
{

	if ( !IsDead() && m_pxPhysicsObjectHandler->GetPhysicsObject()->GetType() != ASURA_PHYSICS_OBJECT_EXTERNALCONTROL )
	{
		const float fTimeStep = Asura_Timers::GetGameTimeSlice();
		Asura_AVector3 xAnimVelocityOS = Asura_AVector3(m_xLinVelocity);


		m_pxPhysicsObjectHandler->CalculateMovementUsingOrientationAndLinVelocity(fTimeStep, 
																				  CalculateOrientation(), 
																				  CalculateVelocity(), 
																				  false, 
																				  AVP_PhysicsObjectHandler::WORLD_SPACE);
	}
}


void AVP_ServerVehicle_Dropship::MakeClientUpdateMessage(void)
{
	Asura_Message* pxMessage = Asura_Network::CreateMessage(ASURA_MSG_CLIENTUPDATE, sizeof(AVP_Message_DropshipClientUpdate), GetClassification());
	if(pxMessage != NULL)
	{
		AVP_Message_DropshipClientUpdate* pxData = static_cast<AVP_Message_DropshipClientUpdate*>(pxMessage->GetData());
		pxData->FillDropship(*this);
		SendClientUpdateMessage(pxMessage);
	}
}


const Asura_Quat AVP_ServerVehicle_Dropship::CalculateOrientation()
{
	const float fTimeStep = Asura_Timers::GetGameTimeSlice();

	Asura_Matrix_3x3 xOrigMatrix;
	Asura_Maths::OrientationFromDirection(xOrigMatrix, m_xCurrentAim);

	const Asura_Vector_3 xUp(0.0f,-1.0f,0.0f);
	Asura_Quat xRotateQuad;

	if (fTimeStep >= 0.0f)
	{
		float fDesiredLeanAngle = dbg_fMaxLeanAngle * m_xCurrentHorizontalAcceleration.Magnitude() / dbg_fMaxHorizontalAcc;

		Asura_Vector_3 xCurrentHorizontalAccelerationDir = m_xCurrentHorizontalAcceleration;
		xCurrentHorizontalAccelerationDir.Normalise();
		Asura_Vector_3 xTurnVector = xUp ^ xCurrentHorizontalAccelerationDir;
		xRotateQuad.FromRotation(fDesiredLeanAngle, xTurnVector);
		Asura_Vector_3 xDesiredLeanVector = xRotateQuad.RotateVector(xUp);

		float xMaxLeanSpeed = dbg_fMaxLeanAngleSpeed * fTimeStep;

		Asura_Vector_3 xLastLean = m_xCurrentLean;

		if (xDesiredLeanVector * m_xCurrentLean > Asura_Maths::Cosine(xMaxLeanSpeed))
		{
			m_xCurrentLean = xDesiredLeanVector;
		}
		else
		{
			Asura_Vector_3 xLeanTurnVector = m_xCurrentLean ^ xDesiredLeanVector;
			xRotateQuad.FromRotation(xMaxLeanSpeed, xLeanTurnVector);
			m_xCurrentLean = xRotateQuad.RotateVector(m_xCurrentLean);
			m_xCurrentLean.Normalise();
		}

		// drop acceleration due to leaning
		m_fCurrentDropAcceleration = dbg_fMaxDropAcceleration * ((1.0f - m_xCurrentLean * xUp)/(1.0f - Asura_Maths::Cosine(dbg_fMaxLeanAngle)));

		Asura_Vector_2 xForward(xOrigMatrix.e13, xOrigMatrix.e33);
		Asura_Vector_2 xLeanDirection(m_xCurrentLean.x, m_xCurrentLean.z);
		xLeanDirection.Normalise();
		float fLeanDelta = (xForward * xLeanDirection) * Asura_Maths::Abs(Asura_Maths::ArcCosine(xUp * m_xCurrentLean) - Asura_Maths::ArcCosine(xUp * xLastLean))/xMaxLeanSpeed;
		float fDesiredAnimTime = 0.5f * (1.0f - 2.0f * fLeanDelta);
		if (fDesiredAnimTime > 1.0f)
		{
			fDesiredAnimTime = 1.0f;
		}
		else if (fDesiredAnimTime < 0.0f)
		{
			fDesiredAnimTime = 0.0f;
		}
		SendRequestAnimation(AVP_Message_Dropship_RequestAnimation::REQUEST_ANIM_FLAPS_DOWNUP, fDesiredAnimTime);
	}

	xRotateQuad.RotationFromAToB(xUp, m_xCurrentLean);
	Asura_Matrix_3x3 xTurnMatrix;
	xRotateQuad.ToMatrix(xTurnMatrix);

	return Asura_Quat(xTurnMatrix * xOrigMatrix);

}
 
const Asura_AVector3 AVP_ServerVehicle_Dropship::CalculateVelocity() const
{
	return Asura_AVector3(m_xCurrentVelocity+m_xCurrentVelocityNoise);
}

void AVP_ServerVehicle_Dropship::PIDUpVelocity(float fDeltaTime)
{
	m_xPIDY.Update(fDeltaTime, m_pxPhysicsObjectHandler->GetPhysicsObject()->GetPosition().y, m_xDestination.y);
}

void AVP_ServerVehicle_Dropship::PIDPlaneVelocity(float fDeltaTime)
{
	m_xPIDX.Update(fDeltaTime, m_pxPhysicsObjectHandler->GetPhysicsObject()->GetPosition().x, m_xDestination.x);
	m_xPIDZ.Update(fDeltaTime, m_pxPhysicsObjectHandler->GetPhysicsObject()->GetPosition().z, m_xDestination.z);
}

void AVP_ServerVehicle_Dropship::PIDTurnVelocity(float fDeltaTime)
{

	float fCurrentAim = Asura_Maths::ArcTangent(m_pxPhysicsObjectHandler->GetPhysicsObject()->GetOrientation().e33, m_pxPhysicsObjectHandler->GetPhysicsObject()->GetOrientation().e13);
	float fDesiredAim = Asura_Maths::ArcTangent(GetAimDirection().z, GetAimDirection().x);

	if (fDesiredAim - fCurrentAim > Asura_Maths::Pi)
	{
		fDesiredAim = fDesiredAim - 2*Asura_Maths::Pi;
	}
	else if (fDesiredAim - fCurrentAim < -Asura_Maths::Pi)
	{
		fDesiredAim = fDesiredAim + 2*Asura_Maths::Pi;
	}

	m_xPIDAim.Update(fDeltaTime, fCurrentAim, fDesiredAim);

}

void AVP_ServerVehicle_Dropship::UpdateUpVelocity(float fDeltaTime)
{
	float fUpAccel = m_xPIDY.GetValue();
	fUpAccel = Asura_Maths::ClampToRange(fUpAccel, -dbg_fMaxVerticalAcc, dbg_fMaxVerticalAcc );

	m_xCurrentVelocity.y += (fUpAccel + m_fCurrentDropAcceleration) * fDeltaTime;
	m_xCurrentVelocity.y = Asura_Maths::ClampToRange(m_xCurrentVelocity.y, -dbg_fMaxVerticalVelocity , dbg_fMaxVerticalVelocity);

}

void AVP_ServerVehicle_Dropship::UpdatePlaneVelocity(float fDeltaTime)
{
	Asura_Vector_2 xPlaneAccel(m_xPIDX.GetValue(), m_xPIDZ.GetValue());

	float fMagAccel = xPlaneAccel.Magnitude();
	if (fMagAccel > dbg_fMaxHorizontalAcc)
	{
		xPlaneAccel *= dbg_fMaxHorizontalAcc / fMagAccel;
	}

	Asura_Vector_2 xPlaneVelocity(m_xCurrentVelocity.x, m_xCurrentVelocity.z);

	xPlaneVelocity += xPlaneAccel * fDeltaTime;

	float fMagVelocity = xPlaneVelocity.Magnitude();
	if (fMagVelocity > dbg_fMaxPlaneVelocity)
	{
		xPlaneVelocity *= dbg_fMaxPlaneVelocity / fMagVelocity;
	}

	m_xCurrentVelocity.x = xPlaneVelocity.x; 
	m_xCurrentVelocity.z = xPlaneVelocity.y;

	// update thrusters animation
	Asura_Vector_2 xPlaneHeading(m_pxPhysicsObjectHandler->GetPhysicsObject()->GetOrientation().e13, m_pxPhysicsObjectHandler->GetPhysicsObject()->GetOrientation().e33);
	xPlaneHeading.Normalise();
	float fDesiredAnimTime = 0.5f * ( 1.0f - (xPlaneHeading * xPlaneAccel) / dbg_fMaxHorizontalAcc);
	SendRequestAnimation(AVP_Message_Dropship_RequestAnimation::REQUEST_ANIM_THRUSTERS_FRONTBACK, fDesiredAnimTime);
}

void AVP_ServerVehicle_Dropship::UpdateTurnVelocity(float fDeltaTime)
{
	float fTurnAccel = m_xPIDAim.GetValue();
	fTurnAccel = Asura_Maths::ClampToRange(fTurnAccel, -dbg_fMaxTurnAcc, dbg_fMaxTurnAcc );

	m_fCurrentTurnVelocity += fTurnAccel * fDeltaTime;
	m_fCurrentTurnVelocity = Asura_Maths::ClampToRange(m_fCurrentTurnVelocity, -dbg_fMaxTurnVelocity, dbg_fMaxTurnVelocity);

	// update flaps and thrusters animation
	float fDesiredAnimTime = 0.5f * ( 1.0f - fTurnAccel / dbg_fMaxTurnAcc);
	SendRequestAnimation(AVP_Message_Dropship_RequestAnimation::REQUEST_ANIM_FLAPS_RIGHTLEFT, fDesiredAnimTime);
}

void AVP_ServerVehicle_Dropship::SendRequestAnimation(AVP_Message_Dropship_RequestAnimation::RequestType eRequestType, float fAnimTime, bool bIsGuaranteed)
{
	Asura_Message* pxMessage = Asura_Network::CreateMessage(AVP_MSGID_REQUEST_ANIMATION, sizeof(AVP_Message_Dropship_RequestAnimation), GetClassification());
	if(bIsGuaranteed)
	{
		pxMessage->UseGuaranteedDelivery();
	}
	if(pxMessage)
	{
		AVP_Message_Dropship_RequestAnimation* pxData = static_cast<AVP_Message_Dropship_RequestAnimation*>(pxMessage->GetData());
		*pxData = AVP_Message_Dropship_RequestAnimation(eRequestType, fAnimTime);
		Send(pxMessage, ASURA_GUID_MY_CLIENT_ENTITIES);
	}
}



void AVP_ServerVehicle_Dropship::WriteToChunkStream( Asura_Chunk_Stream& xStream ) const
{
	xStream << CURRENT_CHUNK_VERSION;
	
	PARENT::WriteToChunkStream( xStream );

	xStream << m_uGS2StartNode;

}

void AVP_ServerVehicle_Dropship::ReadFromChunkStream( Asura_Chunk_Stream& xStream )
{
	int iVersion;
	xStream >> iVersion;

	PARENT::ReadFromChunkStream( xStream );

	if (iVersion == 1 )
	{
		xStream >> m_uGS2StartNode;
	}
	else
	{
		Asura_Assertf( false, ( "AVP_ServerVehicle_Dropship::ReadFromChunkStream - unhandled chunk version (%d)", iVersion ) );
	}
}

void AVP_ServerVehicle_Dropship::SetNextDestination(const Asura_Vector_3& xDestination)
{
	m_xDestination = xDestination;
}


void AVP_ServerVehicle_Dropship::LaserOn(const Asura_Vector_3& xTarget)
{
	if (!m_bIsLaserOn)
	{
		m_xLaserActualTarget = xTarget;
	}
	m_bIsLaserOn = true;
	m_xLaserTarget = xTarget;
}

void AVP_ServerVehicle_Dropship::LaserOff()
{
	m_bIsLaserOn = false;
	m_xLaserActualTarget = Asura_Maths::ZeroVector3;
	m_xLaserTarget = Asura_Maths::ZeroVector3;	
}

bool AVP_ServerVehicle_Dropship::GetLaserTarget(Asura_Vector_3& xTarget) const
{
	xTarget = m_xLaserActualTarget;
	return m_bIsLaserOn;
}

float AVP_ServerVehicle_Dropship::GetVeloictyFraction() const
{
	return m_fVelocityFraction;
}


bool AVP_ServerVehicle_Dropship::AreFrontPodsOpened() const
{
	const AVP_ClientVehicle_Dropship* pxClient = AVP_ClientVehicle_Dropship::FindEntity(GetGuid());
	return pxClient->AreFrontPodsOpened();
}

bool AVP_ServerVehicle_Dropship::AreFrontPodsClosed() const
{
	const AVP_ClientVehicle_Dropship* pxClient = AVP_ClientVehicle_Dropship::FindEntity(GetGuid());
	return pxClient->AreFrontPodsClosed();
}

bool AVP_ServerVehicle_Dropship::AreBackPodsOpened() const
{
	const AVP_ClientVehicle_Dropship* pxClient = AVP_ClientVehicle_Dropship::FindEntity(GetGuid());
	return pxClient->AreBackPodsOpened();
}

bool AVP_ServerVehicle_Dropship::AreBackPodsClosed() const
{
	const AVP_ClientVehicle_Dropship* pxClient = AVP_ClientVehicle_Dropship::FindEntity(GetGuid());
	return pxClient->AreBackPodsClosed();
}

bool AVP_ServerVehicle_Dropship::AreBackPodsDoorsOpened() const
{
	const AVP_ClientVehicle_Dropship* pxClient = AVP_ClientVehicle_Dropship::FindEntity(GetGuid());
	return pxClient->AreBackPodsDoorsOpened();
}

bool AVP_ServerVehicle_Dropship::AreBackPodsDoorsClosed() const
{
	const AVP_ClientVehicle_Dropship* pxClient = AVP_ClientVehicle_Dropship::FindEntity(GetGuid());
	return pxClient->AreBackPodsDoorsClosed();
}

void AVP_ServerVehicle_Dropship::OpenFrontPods()
{
	SendRequestAnimation(AVP_Message_Dropship_RequestAnimation::REQUEST_ANIM_OPEN_FRONT_MISSILE_PODS, 0.0f, true);
}

void AVP_ServerVehicle_Dropship::CloseFrontPods()
{
	SendRequestAnimation(AVP_Message_Dropship_RequestAnimation::REQUEST_ANIM_CLOSE_FRONT_MISSILE_PODS, 0.0f, true);
}

void AVP_ServerVehicle_Dropship::OpenBackPods()
{
	SendRequestAnimation(AVP_Message_Dropship_RequestAnimation::REQUEST_ANIM_OPEN_BACK_MISSILE_PODS, 0.0f, true);
}

void AVP_ServerVehicle_Dropship::CloseBackPods()
{
	SendRequestAnimation(AVP_Message_Dropship_RequestAnimation::REQUEST_ANIM_CLOSE_BACK_MISSILE_PODS, 0.0f, true);
}

void AVP_ServerVehicle_Dropship::OpenBackPodsDoors()
{
	SendRequestAnimation(AVP_Message_Dropship_RequestAnimation::REQUEST_ANIM_OPEN_BACK_MISSILE_PODS_DOORS, 0.0f, true);
}

void AVP_ServerVehicle_Dropship::CloseBackPodsDoors()
{
	SendRequestAnimation(AVP_Message_Dropship_RequestAnimation::REQUEST_ANIM_CLOSE_BACK_MISSILE_PODS_DOORS, 0.0f, true);
}

void AVP_ServerVehicle_Dropship::MissileLaunched()
{
	++m_uNumberOfMissiles;
}

void AVP_ServerVehicle_Dropship::MissileDestroyed()
{
	Asura_Assert(m_uNumberOfMissiles > 0, "mismatch in number of missiles");
	if (m_uNumberOfMissiles > 0)
	{
		--m_uNumberOfMissiles;
	}
}

bool AVP_ServerVehicle_Dropship::IsAnyMissileActive()
{
	return m_uNumberOfMissiles >0;
}


void AVP_ServerVehicle_Dropship::Update()
{
    PARENT::Update();

	float fDeltaTime = Asura_Timers::GetGameTimeSlice();

	if (fDeltaTime == 0.0f)
	{
		return;
	}

	m_fTimeSinceControlUpdate += fDeltaTime;
	m_fTimeTillVelocityNoiseChanges -= fDeltaTime;

	if (m_fTimeSinceControlUpdate > 0.1f)
	{
		PIDUpVelocity(m_fTimeSinceControlUpdate);
		PIDPlaneVelocity(m_fTimeSinceControlUpdate);
		PIDTurnVelocity(m_fTimeSinceControlUpdate );
		m_fTimeSinceControlUpdate = 0.0f;
	}

	UpdateUpVelocity(fDeltaTime);
	UpdatePlaneVelocity(fDeltaTime);
	UpdateTurnVelocity(fDeltaTime );

	Asura_Vector_3 m_xCurrentPlaneVelocity = m_xCurrentVelocity;
	m_xCurrentPlaneVelocity.y = 0.0f;
	m_fVelocityFraction = m_xCurrentPlaneVelocity.Magnitude() / dbg_fMaxPlaneVelocity + Asura_Maths::Abs(m_xCurrentVelocity.y) / dbg_fMaxVerticalVelocity;
	m_fVelocityFraction = Asura_Maths::Min(m_fVelocityFraction, 1.0f);

	Asura_Quat xTurn;
	const Asura_Vector_3 xUp(0.0f,-1.0f,0.0f);
	xTurn.FromRotation(m_fCurrentTurnVelocity * fDeltaTime, xUp);
	m_xCurrentAim = xTurn.RotateVector(m_xCurrentAim);

	m_xCurrentAcceleration = (m_xCurrentVelocity - m_xLastVelocity) / fDeltaTime; 
	m_xCurrentHorizontalAcceleration = m_xCurrentAcceleration; 
	m_xCurrentHorizontalAcceleration.y = 0.0f;

	m_xLastVelocity = m_xCurrentVelocity;

	// update velocity noise
	if (m_fTimeTillVelocityNoiseChanges < 0.0f)
	{
		m_fTimeTillVelocityNoiseChanges = Asura_Random::GetFloat(dbg_fVelocityNoiseChangeMinTime, dbg_fVelocityNoiseChangeMaxTime);
		m_xCurrentVelocityNoise.RandomDirectionSeeded();
		m_xCurrentVelocityNoise *= Asura_Random::GetFloat(0, dbg_fVelocityNoiseSize);
	}

	// update Laser 
	if (m_bIsLaserOn)
	{
		float xMaxLaserDelta = dbg_fMaxLaserVelocity * fDeltaTime;
		if ((m_xLaserActualTarget - m_xLaserTarget).MagnitudeSquared() < xMaxLaserDelta * xMaxLaserDelta)
		{
			m_xLaserActualTarget = m_xLaserTarget;
		}
		else
		{
			Asura_Vector_3 xLaserDelta = m_xLaserTarget - m_xLaserActualTarget;
			xLaserDelta.Normalise(); 
			xLaserDelta *= xMaxLaserDelta;
			m_xLaserActualTarget += xLaserDelta;
		}
	}

}

Axon_Brain* AVP_ServerVehicle_Dropship::CreateBrain()
{
	return new AVP_Brain_Actor_Dropship( this );
}

bool AVP_ServerVehicle_Dropship::HandleMessage(const Asura_Message* pxMessage)
{
	// What sort of message is it?
	switch (pxMessage->GetMsgID())
	{
	case AVP_MSGID_SET_DROPSHIP_ATTACK_MODE:
		{
			const AVP_Message_DropshipAttackMode*	pxData	= static_cast<const AVP_Message_DropshipAttackMode*>(pxMessage->GetData());

			if (pxData != NULL)
			{
				AVP_Brain_Actor_Dropship *pxBrain = static_cast<AVP_Brain_Actor_Dropship *>(GetActorBrain());
				switch (pxData->m_eMode)
				{
				case AVP_Message_DropshipAttackMode::DSAM_USE_NOTHING:
					pxBrain->SetAttackMode(false,false,false);
					break;
				case AVP_Message_DropshipAttackMode::DSAM_USE_MACHINEGUN:
					pxBrain->SetAttackMode(false,true,false);
					break;
				case AVP_Message_DropshipAttackMode::DSAM_USE_MISSILES:
					pxBrain->SetAttackMode(true,false,false);
					break;
				case AVP_Message_DropshipAttackMode::DSAM_USE_MISSILES_BUT_MISS:
					pxBrain->SetAttackMode(true,false,true);
					break;
				case AVP_Message_DropshipAttackMode::DSAM_USE_MACHINEGUN_AND_USE_MISSILES:
					pxBrain->SetAttackMode(true,true,false);
					break;
				default:
					Asura_Assert(false,"unepxected mode");
					pxBrain->SetAttackMode(false,false,false);
					break;

				}
			}

			return(true);
		}

	default:
		{
			return(PARENT::HandleMessage(pxMessage));
		}
	}
}



AVP_ServerVehicle_Dropship::PIDController::PIDController(float fP, float fI, float fD, float fMaxOutput)
: m_fP(fP)
, m_fI(fI)
, m_fD(fD)
, m_fMaxOutput(fMaxOutput)
, m_fOutput(0.0f)
, m_fLastPlantValue(0.0f)
, m_fLastLastPlantValue(0.0f)      
{
}

void AVP_ServerVehicle_Dropship::PIDController::Update(float fDTime, float fPlantValue, float fDesired)
{

	m_fOutput = 
		m_fOutput
		- m_fP * (fPlantValue - m_fLastPlantValue) 
		+ m_fI * (fDesired - fPlantValue) * fDTime
		- m_fD *(fPlantValue - 2.0f * m_fLastPlantValue +  m_fLastLastPlantValue) / fDTime;
	m_fOutput = Asura_Maths::ClampToRange(m_fOutput, -m_fMaxOutput, m_fMaxOutput);

	m_fLastLastPlantValue = m_fLastPlantValue;
	m_fLastPlantValue = fPlantValue;
}



/// End of file. ///
