
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_Vertebrate_Messages.h
// Author : Chris Wallace
// Date   : 8-Aug-2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"

// header
#include "Entity/PhysicalObject/Actor/Vertebrates/AVP_Vertebrate_Messages.h"

// asura
#include "Core/Asura_BitPacker.h"
#include "Entity/PhysicalObject/Asura_ServerEntity_Actor.h"
#include "Network/Asura_NetMsg.h"

// avp
#include "Actor/Vertebrates/AVP_Vertebrate_AIAnimStateMachine.h"
#include "Actor/Vertebrates/AVP_Vertebrate_Focus.h"
#include "Actor/Vertebrates/AVP_Vertebrate_InputHandler.h"
#include "Actor/Vertebrates/AVP_Vertebrate_MeleeCounter_System.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/AVP_ServerActor_Vertebrate.h"

//////////////////////////////////////////////////////////////////////////////
//
// AVP_Message_Actor_ClientUpdate
//
//////////////////////////////////////////////////////////////////////////////

void AVP_Message_Vertebrate_ClientUpdate::FillVertebrate( const AVP_ServerActor_Vertebrate& xVertebrate )
{
    FillActor( xVertebrate );

	const AVP_Vertebrate_MeleeCounter_System* pxMeleeSystem = xVertebrate.GetMeleeSystem();
	if (pxMeleeSystem)
	{
		m_bEnemyInMeleeRange = pxMeleeSystem->GetHasAttackTarget();
	}

	m_uDestroyedSkinElementFlags = xVertebrate.GetDestroyedSkinElementFlags();
}

#ifdef ASURA_SUPPORT_NETMSG_SYSTEM
void AVP_Message_Vertebrate_ClientUpdate::RegisterFields(Asura_NetMsg* pxNetMsg)
{
	const AVP_Message_Vertebrate_ClientUpdate xDummy;
	PARENT::RegisterFields(pxNetMsg);

	Asura_NetMsg_RegisterField(pxNetMsg, xDummy, Asura_FieldType_uchar, m_bEnemyInMeleeRange, 1, Asura_Threshold_AnyChange, 0.0f);
	Asura_NetMsg_RegisterField(pxNetMsg, xDummy, Asura_FieldType_uint, m_uDestroyedSkinElementFlags, 1, Asura_Threshold_AnyChange, 0.0f);
}
#endif//#ifdef ASURA_SUPPORT_NETMSG_SYSTEM

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

void AVP_Message_VertebratePlayerCommands::FillVertebrate( const AVP_ClientActor_Vertebrate& xVert )
{
	FillActor( xVert );

	m_uCurrentMeleeTarget = xVert.GetMeleeTarget();
}

void AVP_Message_VertebratePlayerCommands::Pack( Asura_BitPacker& xPacker ) const
{
	PARENT::Pack( xPacker ); 

	const bool bPackCurrentTarget = ( GetCurrentMeleeTarget() != ASURA_GUID_UNREGISTERED );
	xPacker.PackBool( bPackCurrentTarget ); 
	if ( bPackCurrentTarget )
	{
		xPacker.Pack( GetCurrentMeleeTarget(), sizeof(Asura_Guid) * 8 ); 
	}
}

void AVP_Message_VertebratePlayerCommands::Unpack( Asura_BitUnpacker& xUnpacker )
{
	PARENT::Unpack( xUnpacker ); 

	m_uCurrentMeleeTarget = ASURA_GUID_UNREGISTERED; 
	const bool bUnpackCurrentTarget = xUnpacker.UnpackBool(); 
	if ( bUnpackCurrentTarget )
	{
		m_uCurrentMeleeTarget = xUnpacker.Unpack( sizeof(Asura_Guid) * 8 ); 
	}
}
