
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_ClientActor_Alien.cpp
// Author : Andy Tate
// Modified to Alien : Chris Wallace
// Date   : 3-Oct-2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "Entity/PhysicalObject/Actor/Vertebrates/Alien/AVP_ClientActor_Alien.h"

// Asura independent header-files...
#include "Animation/Asura_Animation.h"
#include "Animation/Asura_AnimationAttachment.h"
#include "Animation/Asura_AnimationBlend.h"
#include "Attractors/Asura_Attractor_System.h"
#include "Blueprint/Asura_Blueprint_System.h"
#include "Entity/Asura_PosAndOriInterpolator.h"
#include "Core/Asura_Timers.h"
#include "Network/Asura_Message.h"
#include "Physics/Asura_Physics.h"
#include "Rendering/Asura_Font_System.h"
#include "Rendering/Asura_Viewport.h"
#include "SpecialFX/PFX/Asura_PFX_System.h"
#include "SpecialFX/PFX/Asura_PFX_Entity_Effect.h"
#include "Debugging/Asura_DebugLineRender.h"

// Game independent header-files...
#include "Animation/AVP_AnimFlags.h"
#include "Actor/AVP_Actor_PhysicsObjectHandler.h"
#include "Actor/AVP_Actor_PickupHandler.h"
#include "Actor/Vertebrates/Alien/AVP_Alien_AIAnimStateData.h"
#include "Actor/Vertebrates/Alien/AVP_Alien_AnimationHandler.h"
#include "Actor/Vertebrates/Alien/AVP_Alien_CameraHandler.h"
#include "Actor/Vertebrates/Alien/AVP_Alien_Description.h"
#include "Actor/Vertebrates/Alien/AVP_Alien_Focus.h"
#include "Actor/Vertebrates/Alien/AVP_Alien_HUDHandler.h"
#include "Actor/Vertebrates/Alien/AVP_Alien_InputHandler.h"
#include "Actor/Vertebrates/Alien/AVP_Alien_InteractionHandler.h"
#include "Actor/Vertebrates/Alien/AVP_Alien_JumpHandler.h"
#include "Actor/Vertebrates/Alien/AVP_Alien_PlayerAnimStateData.h"
#include "Actor/Vertebrates/Alien/AVP_Alien_PlayerAnimStateHandler.h"
#include "Actor/Vertebrates/AVP_Vertebrate_MeleeCounter_System.h"
#include "Blueprint/AVP_Blueprint_HashIDs.h"
#include "Core/AVP_Game_Core.h"
#include "Entity/AVP_Entity_Classification.h"
#include "Entity/AVP_Messages.h"
#include "Entity/Damage/AVP_EntityDamage_Apply.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Alien/AVP_Messages_Alien.h"
#include "Entity/PhysicalObject/Actor/AVP_Actor_Messages.h"
#include "Entity/PhysicalObject/Gib/AVP_ClientEntity_Gib.h"
#include "Inventory/AVP_Inventory_HashIDs.h"
#include "Inventory/AVP_Inventory_System.h"
#include "Mesh/AVP_PolygonFlags.h"
#include "Moves/AVP_Movement_System.h"
#include "Moves/Vertebrates/Alien/AVP_Alien_MovesBranch.h"
#include "Moves/Vertebrates/Alien/AVP_Alien_MovesInteraction.h"
#include "Moves/Vertebrates/Alien/AVP_Alien_MovesJump.h"
#include "Moves/Vertebrates/Alien/AVP_Alien_MovesMelee.h"
#include "Moves/Vertebrates/Alien/AVP_Alien_MovesTail.h"
#include "Moves/Vertebrates/Alien/AVP_Alien_MovesVent.h"
#include "Network/AVP_Player.h"
#include "Physics/AVP_CalculateSurfaceReorientationNormal.h"
#include "SpecialFX/Asura_Decal_System.h"

#include "SpecialFX/Asura_Decal_Manager.h"

// Useful for debugging...
//#define	AVP_TEST_ALIEN_DISMEMBERMENT


static const Asura_Hash_ID s_bAliensExplodeWhenDie				= 0x7E15AC91;	// "AliensExplodeWhenDie"
static const Asura_Hash_ID s_bAliensExplodeWhenDie_PFXExplosion	= 0xE0D4FE50;	// "AliensExplodeWhenDie.PFXExplosion"

////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////

#if defined(ASURA_SUPPORT_COMMAND_CONSOLE)
u_int AVP_ClientActor_Alien::s_uConsoleVarReferenceCount = 0;
#endif

////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////

Asura_Hashed_List<AVP_ClientActor_Alien>	AVP_ClientActor_Alien::s_xList;


////////////////////////////////////////////////////////////////////////////////
// Private functions...
////////////////////////////////////////////////////////////////////////////////

bool AVP_ClientActor_Alien::InitialiseMoves(void)
{
	if(PARENT::InitialiseMoves())
	{
		Asura_Movement_System& xMovementSystem = *m_pxMovementSystem;
		Asura_Hash_ID uFirstPersonBlueprintHash, uThirdPersonBlueprintHash;
		GetCurrentMeleeBlueprintIDs(uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);

		// weapons
		new AVP_Alien_Move_Melee_CrawlToStandAttackR_Return_Hit(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);
		new AVP_Alien_Move_Melee_CrawlToStandAttackR_Return_Miss(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);
		new AVP_Alien_Move_Melee_CrawlToStandAttackR_Return_Countered(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);
		new AVP_Alien_Move_Melee_CrawlToStandAttackR_Start(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);

		new AVP_Alien_Move_Melee_CrawlToStandAttackL_Return_Hit(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);
		new AVP_Alien_Move_Melee_CrawlToStandAttackL_Return_Miss(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);
		new AVP_Alien_Move_Melee_CrawlToStandAttackL_Return_Countered(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);
		new AVP_Alien_Move_Melee_CrawlToStandAttackL_Start(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);
		new AVP_Alien_Move_Melee_CrawlToStandAttackShortL_Start(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);

		new AVP_Alien_Move_Melee_StandAttackLoopR_Return_Hit(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);
		new AVP_Alien_Move_Melee_StandAttackLoopR_Return_Miss(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);
		new AVP_Alien_Move_Melee_StandAttackLoopR_Return_Countered(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);
		new AVP_Alien_Move_Melee_StandAttackLoopR_Start(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);
		new AVP_Alien_Move_Melee_CrawlToStandAttackShortR_Start(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);

		new AVP_Alien_Move_Melee_StandAttackLoopL_Return_Hit(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);
		new AVP_Alien_Move_Melee_StandAttackLoopL_Return_Miss(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);
		new AVP_Alien_Move_Melee_StandAttackLoopL_Return_Countered(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);
		new AVP_Alien_Move_Melee_StandAttackLoopL_Start(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);

		new AVP_Alien_Move_Melee_WalkAttackLoopR_Return_Hit(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);
		new AVP_Alien_Move_Melee_WalkAttackLoopR_Return_Miss(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);
		new AVP_Alien_Move_Melee_WalkAttackLoopR_Return_Countered(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);
		new AVP_Alien_Move_Melee_WalkAttackLoopR_Start(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);

		new AVP_Alien_Move_Melee_WalkAttackLoopL_Return_Hit(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);
		new AVP_Alien_Move_Melee_WalkAttackLoopL_Return_Miss(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);
		new AVP_Alien_Move_Melee_WalkAttackLoopL_Return_Countered(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);
		new AVP_Alien_Move_Melee_WalkAttackLoopL_Start(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);


		new AVP_Alien_Move_Melee_JumpSlash_LeftClaw_Return_Hit(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);
		new AVP_Alien_Move_Melee_JumpSlash_LeftClaw_Return_Countered(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);
		new AVP_Alien_Move_Melee_JumpSlash_LeftClaw_Return_Miss(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);
		new AVP_Alien_Move_Melee_JumpSlash_LeftClaw_Start(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);

		new AVP_Alien_Move_Melee_JumpSlash_RightClaw_Return_Hit(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);
		new AVP_Alien_Move_Melee_JumpSlash_RightClaw_Return_Countered(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);
		new AVP_Alien_Move_Melee_JumpSlash_RightClaw_Return_Miss(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);
		new AVP_Alien_Move_Melee_JumpSlash_RightClaw_Start(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);

		new AVP_Alien_Move_Melee_JumpSlashWall_LeftClaw_Return_Hit(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);
		new AVP_Alien_Move_Melee_JumpSlashWall_LeftClaw_Return_Countered(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);
		new AVP_Alien_Move_Melee_JumpSlashWall_LeftClaw_Return_Miss(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);
		new AVP_Alien_Move_Melee_JumpSlashWall_LeftClaw_Start(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);

		new AVP_Alien_Move_Melee_JumpSlashWall_RightClaw_Return_Hit(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);
		new AVP_Alien_Move_Melee_JumpSlashWall_RightClaw_Return_Countered(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);
		new AVP_Alien_Move_Melee_JumpSlashWall_RightClaw_Return_Miss(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);
		new AVP_Alien_Move_Melee_JumpSlashWall_RightClaw_Start(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);

		new AVP_Alien_Move_Melee_StandingClawAttack_Return_Hit(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);
		new AVP_Alien_Move_Melee_StandingClawAttack_Return_Countered(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);
		new AVP_Alien_Move_Melee_StandingClawAttack_Return_Miss(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);
		new AVP_Alien_Move_Melee_StandingClawAttack_Start(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);

		new AVP_Alien_Move_Melee_CrouchingClawAttack_Return_Hit(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);
		new AVP_Alien_Move_Melee_CrouchingClawAttack_Return_Countered(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);
		new AVP_Alien_Move_Melee_CrouchingClawAttack_Return_Miss(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);
		new AVP_Alien_Move_Melee_CrouchingClawAttack_Start(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);

		new AVP_Alien_Move_Melee_Headbite(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);

		// drop moves
		new AVP_Move_Alien_JumpDropStart(xMovementSystem);
		new AVP_Move_Alien_FallingEnd(xMovementSystem);
		new AVP_Move_Alien_Falling(xMovementSystem);
		new AVP_Move_Alien_FallingBegin(xMovementSystem);

		// Surface interaction moves...
		new AVP_Move_Alien_JumpSurface180(xMovementSystem);
		new AVP_Move_Alien_JumpSurfaceConcave90End(xMovementSystem);
		new AVP_Move_Alien_JumpSurfaceConcave90Middle(xMovementSystem);
		new AVP_Move_Alien_JumpSurfaceConcave90Begin(xMovementSystem);
		new AVP_Move_Alien_JumpSurfaceConcave90(xMovementSystem);
		new AVP_Move_Alien_HopLeft(xMovementSystem);
		new AVP_Move_Alien_HopRight(xMovementSystem);
		new AVP_Move_Alien_WallHopLeft(xMovementSystem);
		new AVP_Move_Alien_WallHopRight(xMovementSystem);
		new AVP_Move_Alien_JumpOnToWallLeft(xMovementSystem);
		new AVP_Move_Alien_JumpOnToWallRight(xMovementSystem);
		new AVP_Move_Alien_AI_JumpSurfaceConcaveLeft(xMovementSystem);
		new AVP_Move_Alien_AI_JumpSurfaceConcaveRight(xMovementSystem);
		new AVP_Move_Alien_AI_JumpSurfaceFlat(xMovementSystem);
		new AVP_Move_Alien_AI_JumpSurfaceConcave90End(xMovementSystem);
		new AVP_Move_Alien_AI_JumpSurfaceConcave90Middle(xMovementSystem);
		new AVP_Move_Alien_AI_JumpSurfaceConcave90Begin(xMovementSystem);
		new AVP_Move_Alien_AI_MoveForwardAndJumpSurface(xMovementSystem);
		new AVP_Move_Alien_AI_Turn90LeftAndJumpSurface(xMovementSystem);
		new AVP_Move_Alien_AI_Turn90RightAndJumpSurface(xMovementSystem);
		new AVP_Move_Alien_AI_Turn180LeftAndJumpSurface(xMovementSystem);
		new AVP_Move_Alien_AI_Turn180RightAndJumpSurface(xMovementSystem);
		new AVP_Move_Alien_AI_Turn90OnSpotLeftAndJumpSurface(xMovementSystem);
		new AVP_Move_Alien_AI_Turn90OnSpotRightAndJumpSurface(xMovementSystem);
		new AVP_Move_Alien_AI_Turn180OnSpotLeftAndJumpSurface(xMovementSystem);
		new AVP_Move_Alien_AI_Turn180OnSpotRightAndJumpSurface(xMovementSystem);
		new AVP_Move_Alien_AI_Turn180Left(xMovementSystem);
		new AVP_Move_Alien_AI_Turn180Right(xMovementSystem);
		new AVP_Move_Alien_AI_FallEnd(xMovementSystem);
		new AVP_Move_Alien_AI_FallMiddle(xMovementSystem);
		new AVP_Move_Alien_AI_FallStart(xMovementSystem);

		// hiss
		new AVP_Move_Alien_Hiss(xMovementSystem);

		// Navigation moves.
		new AVP_Move_Alien_180TurnToCrawl(xMovementSystem);
		new AVP_Move_Alien_90TurnToCrawlLeft(xMovementSystem);
		new AVP_Move_Alien_90TurnToCrawlRight(xMovementSystem);

		// Vent moves
		//new AVP_Move_Alien_AI_TraverseVent_End(xMovementSystem);
		new AVP_Move_Alien_AI_TraverseVent_Begin(xMovementSystem);
		new AVP_Move_Alien_Player_TraverseVent_End(xMovementSystem);
		new AVP_Move_Alien_Player_TraverseVent_Begin(xMovementSystem);
		new AVP_Move_Alien_DuctSurfaceSwitch(xMovementSystem);

		// Branch moves
		new AVP_Move_Alien_MountBranch(xMovementSystem);
		new AVP_Move_Alien_DismountBranch(xMovementSystem);

		// Pounce moves. These moves need some additional set up from our blueprint.
		AVP_Move_Melee_Pounce_Attack_Return_Countered* pxCounteredMove = new AVP_Move_Melee_Pounce_Attack_Return_Countered(xMovementSystem);
		AVP_Move_Melee_Pounce_Attack_Return_Miss* pxMissMove = new AVP_Move_Melee_Pounce_Attack_Return_Miss(xMovementSystem);
		AVP_Move_Melee_Pounce_Attack_Return_Hit* pxHitMove = new AVP_Move_Melee_Pounce_Attack_Return_Hit(xMovementSystem);
		AVP_Move_Melee_Pounce_Attack_Start* pxStartMove = new AVP_Move_Melee_Pounce_Attack_Start(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);
		const Asura_Blueprint* pxBlueprint = GetBlueprint();
		if (pxBlueprint)
		{
			if (pxStartMove && pxHitMove && pxCounteredMove && pxMissMove)
			{
				AVP_Blueprint_ParameterEnumerator_PounceVariations_MoveData xEnumerator(uAVP_BLUEPRINT_TYPE_MOVESALIEN, *pxStartMove, *pxHitMove, *pxCounteredMove, *pxMissMove);
				pxBlueprint->EnumerateParameters(AVP_Vertebrate_MeleeCounter_System::GetPounceAttackBaseParameterName(), 2, xEnumerator);
			}
		}
		AVP_Move_Puppeteer* pxPuppeteerMove;

		new AVP_Alien_Move_Melee_TailLash_Return_Hit(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);
		new AVP_Alien_Move_Melee_TailLash_Return_Countered(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);
		new AVP_Alien_Move_Melee_TailLash_Return_Miss(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);
		new AVP_Alien_Move_Melee_TailLash_Start(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);
		new AVP_Alien_Move_Melee_TailStab_Return_Hit(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);
		new AVP_Alien_Move_Melee_TailStab_Return_Countered(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);
		new AVP_Alien_Move_Melee_TailStab_Return_Miss(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);
		new AVP_Alien_Move_Melee_TailStab_Start(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);
		new AVP_Alien_Move_Melee_TailCharge_End(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);
		new AVP_Alien_Move_Melee_TailCharge_Loop(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);
		new AVP_Alien_Move_Melee_TailCharge_Start(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);

		new AVP_Alien_Move_Melee_PickupHuman_Return_Hit(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);
		new AVP_Alien_Move_Melee_PickupHuman_Return_Countered(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);
		new AVP_Alien_Move_Melee_PickupHuman_Return_Miss(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);
		new AVP_Alien_Move_Melee_Headbite_Human_Kill(xMovementSystem);
		new AVP_Alien_Move_Melee_Headbite_Human_Countered(xMovementSystem);
		pxPuppeteerMove = new AVP_Alien_Move_Melee_Headbite_Human_Start(xMovementSystem);
		if (pxStartMove && pxPuppeteerMove)
		{
			pxStartMove->AddHumanGrapple(pxPuppeteerMove);
		}
		new AVP_Alien_Move_Melee_BackHeadbite_Human_Kill(xMovementSystem);
		new AVP_Alien_Move_Melee_BackHeadbite_Human_Countered(xMovementSystem);
		pxPuppeteerMove = new AVP_Alien_Move_Melee_BackHeadbite_Human_Start(xMovementSystem);
		if (pxStartMove && pxPuppeteerMove)
		{
			pxStartMove->AddHumanGrapple(pxPuppeteerMove);
		}
		new AVP_Alien_Move_Melee_StealthKill_Human_Kill(xMovementSystem);
		new AVP_Alien_Move_Melee_StealthKill_Human_Countered(xMovementSystem);
		new AVP_Alien_Move_Melee_StealthKill_Human_Start(xMovementSystem);
		new AVP_Alien_Move_Melee_Headbite_Predator_Kill(xMovementSystem);
		new AVP_Alien_Move_Melee_Headbite_Predator_Countered(xMovementSystem);
		pxPuppeteerMove = new AVP_Alien_Move_Melee_Headbite_Predator_Start(xMovementSystem);
		if (pxStartMove && pxPuppeteerMove)
		{
			pxStartMove->AddPredatorGrapple(pxPuppeteerMove);
		}
		new AVP_Alien_Move_Melee_BackHeadbite_Predator_Kill(xMovementSystem);
		new AVP_Alien_Move_Melee_BackHeadbite_Predator_Countered(xMovementSystem);
		pxPuppeteerMove = new AVP_Alien_Move_Melee_BackHeadbite_Predator_Start(xMovementSystem);
		if (pxStartMove && pxPuppeteerMove)
		{
			pxStartMove->AddHumanGrapple(pxPuppeteerMove);
		}
		new AVP_Alien_Move_Melee_StealthKill_Predator_Kill(xMovementSystem);
		new AVP_Alien_Move_Melee_StealthKill_Predator_Countered(xMovementSystem);
		new AVP_Alien_Move_Melee_StealthKill_Predator_Start(xMovementSystem);
		new AVP_Alien_Move_Melee_PickupHuman_Start(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);

		new AVP_Alien_Move_Melee_JumpToGrappleHuman_Return_Miss(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);
		new AVP_Alien_Move_Melee_JumpToGrappleHuman_Return_Countered(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);
		new AVP_Alien_Move_Melee_JumpToGrappleHuman_Return_Hit(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);
		new AVP_Alien_Move_Melee_JumpToGrappleHuman_Start(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);

		// Interaction moves
		new AVP_Alien_Move_Interaction_DestroyBattery(xMovementSystem);

		return(true);
	}
	else
	{
		return(false);
	}
}

void AVP_ClientActor_Alien::OnDeath( const AVP_EntityDamage_ApplyToEntity& xDamage )
{
	// For Alien, we need to modify xDamage so we can destroy its limbs, in case we want to create explosion.
	// Therefore we can't use const xDamage
	AVP_EntityDamage_ApplyToEntity xDamage2(xDamage);
	
	// Create Acid explosion when alien is killed by burn damage, plasma caster, rocket, etc...
	Asura_Blueprint* pxBlueprint = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_DAMAGE, xDamage.GetTypeHash() );
	if ( pxBlueprint )
	{
		Asura_Value xValue;
		if ( pxBlueprint->GetParameterValueByHashID( s_bAliensExplodeWhenDie, xValue ) )
		{
			bool bDoesAcidSplat = xValue.GetBoolValue();

			if( bDoesAcidSplat )
			{
				if ( pxBlueprint->GetParameterValueByHashID( s_bAliensExplodeWhenDie_PFXExplosion, xValue ) )
				{
					Asura_Hash_ID uPfxExplosion = xValue.GetHashIDValue();

					if( uPfxExplosion!=ASURA_HASH_ID_UNSET )
					{
						static const Asura_Hash_ID uChestTargetMarkerHashID = 0xe5a85d03; // Asura_GetHashID("target_1")

						CreatePFXEffect( uPfxExplosion, uChestTargetMarkerHashID, false );

						for( u_int u=0; u<NUM_AVP_SKIN_ELEMENT_TYPES; u++ )
						{
							if( IsElementUndamaged(u) )
							{
								if( u==	AVP_SET_ALIEN_TAIL_TIP )
								{
									// Alien tail tip looks terrible when it is gibbed. So don't do it.
									continue;
								}

								xDamage2.SetToDestroyAndGibBodyPart( u );
							}
						}
					}
				}
			}
		}
	}

	PARENT::OnDeath( xDamage2 );
}

void AVP_ClientActor_Alien::CreateAnimationHandler()
{
	Asura_Assert( m_pxAnimationHandler == NULL, "Already got an animation handler" );
	m_pxAnimationHandler = new AVP_Alien_AnimationHandler(*this);
	m_pxAnimationHandler->Initialise();
}

////////////////////////////////////////////////////////////////////////////////
// Protected functions...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ClientActor_Alien::AVP_ClientActor_Alien
// Author       : Andy Tate
// Date         : 3-Oct-2006
////////////////////////////////////////////////////////////////////////////////

AVP_ClientActor_Alien::AVP_ClientActor_Alien (void)
: AVP_ClientActor_Vertebrate(AVP_PolygonFlag_IgnoreForAlienCollision)
, m_bIsInShadow(false)
, m_bComboPressed(false)
, m_uDroolPFXEffect(ASURA_PFX_INVALID_EFFECT_ID)
, m_bHasBeenHeard(false)
, m_uSpottedState(1)
, m_fTimeStanding(0.0f)
, m_fLookSmoothingTransitionAngle(0.0f)
, m_bIsFaceHuggerSummonable(false)
, m_uFaceHuggerPetStatus(0)
, m_uAlienFlags(0)
{
	Classification = AVP_ENTITYCLASS_ALIEN;
	
	SetNumFeet(4);

	// Initialise the entity...
	SetTimeLagType(TimeLag_LocalPlayer);

	// Don't forget to store us in the list...
	s_xList.AddToList(this);
}

/// End of function 'AVP_ClientActor_Alien::AVP_ClientActor_Alien'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ClientActor_Alien::~AVP_ClientActor_Alien
// Author       : Andy Tate
// Date         : 3-Oct-2006
////////////////////////////////////////////////////////////////////////////////

AVP_ClientActor_Alien::~AVP_ClientActor_Alien ()
{
#if defined(ASURA_SUPPORT_COMMAND_CONSOLE)
	--s_uConsoleVarReferenceCount;
#endif
	// Don't forget to remove us from the list...
	s_xList.RemoveFromList(this);
}

/// End of function 'AVP_ClientActor_Alien::~AVP_ClientActor_Alien'. ///

bool AVP_ClientActor_Alien::InitialiseSoundEvents(void)
{
	if(PARENT::InitialiseSoundEvents())
	{
		m_pxGibSoundEvent	= GetSoundEventFromBlueprints(0x6662AF99); //AlienGib
		return(true);
	}
	else
	{
		return(false);
	}
}


////////////////////////////////////////////////////////////////////////////////
// Public functions...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ClientActor_Alien::Create
// Author       : Andy Tate
// Date         : 3-Oct-2006
////////////////////////////////////////////////////////////////////////////////

AVP_ClientActor_Alien* AVP_ClientActor_Alien::Create (const Asura_Message* pxMessage)
{
	AVP_ClientActor_Alien*	pxClient	= new AVP_ClientActor_Alien;
	
	// Initialise the entity...
	pxClient->HandleMessage(pxMessage);

	// Return the entity...
	return(pxClient);
}

/// End of function 'AVP_ClientActor_Alien::Create'. ///

void AVP_ClientActor_Alien::PostCreateInitialise(void)
{
	// If there is no animation handler yet, create it.
	if( !m_pxAnimationHandler && GetBlueprintHashID() != ASURA_HASH_ID_UNSET )
	{
		CreateAnimationHandler();
	}
	
	PARENT::PostCreateInitialise();

	if( IsPlayer() )
	{
		if ( m_pxPlayerStateHandler == NULL )
		{
			m_pxPlayerStateHandler = new AVP_Alien_PlayerAnimStateHandler( *this );
		}
		m_pxPlayerStateHandler->Initialise();
	}
}

void AVP_ClientActor_Alien::GetCurrentMeleeBlueprintIDs(Asura_Hash_ID& uFirstPersonID, Asura_Hash_ID& uThirdPersonID) const
{
	uFirstPersonID = uThirdPersonID = ASURA_HASH_ID_UNSET;

	// As the alien, we just get them from our actor blueprint.
	const Asura_Blueprint* pxBlueprint = GetBlueprint();
	if (!pxBlueprint) { return; }

	// TODO - are we better off storing these values?
	Asura_Value xValue;
	if (pxBlueprint->GetParameterValueByHashID(uAVP_BLUEPRINT_PARAM_MELEE_FIRSTPERSON, xValue))
	{
		uFirstPersonID = xValue.GetHashIDValue();
	}

	if (pxBlueprint->GetParameterValueByHashID(uAVP_BLUEPRINT_PARAM_MELEE_THIRDPERSON, xValue))
	{
		uThirdPersonID = xValue.GetHashIDValue();
	}
}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ClientActor_Alien::Update
// Author       : Andy Tate
// Date         : 3-Oct-2006
////////////////////////////////////////////////////////////////////////////////
DEBUGVAR float dbg_fAlienAutoCrouchTime = 0.5f;

void AVP_ClientActor_Alien::Update (void)
{
	// Ensure that the parent is up-to-date...
	PARENT::Update();

	if (IsStanding())
	{
		m_fTimeStanding += Asura_Timers::GetGameTimeSlice();
		if (m_fTimeStanding > dbg_fAlienAutoCrouchTime)
		{
			SetStanding(false, true);
		}
	}
}

/// End of function 'AVP_ClientActor_Alien::Update'. ///

void AVP_ClientActor_Alien::GetDesiredForwardMovementDirection(Asura_AVector3& xDirection, const Asura_AMatrix3x3& xOri) const
{
	// If we're on a branch or in a duct, orientation is relative to attractor.
	if (m_uConstrainedMovementAttractorID != uASURA_ATTRACTOR_ID_NONE)
	{
		const Asura_Attractor_Compiled* pxAttractor = Asura_Attractor_System::GetInstance()->GetAttractorByID(m_uConstrainedMovementAttractorID);
		if (pxAttractor)
		{
			Asura_Vector_3 xAttractorDir;
			pxAttractor->GetAxisDirection(xAttractorDir);
			Asura_AVector3 xAttDirAVec(xAttractorDir);

			Asura_AVector3 xForward = xOri.GetColumn2();

			if ((xForward * xAttDirAVec) < 0.0f)
			{
				xDirection = -xAttDirAVec;
			}
			else
			{
				xDirection = xAttDirAVec;
			}
		}
		else
		{
			PARENT::GetDesiredForwardMovementDirection(xDirection, xOri);
		}
	}
	else
	{
		PARENT::GetDesiredForwardMovementDirection(xDirection, xOri);
	}
}


////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ClientActor_Alien::RenderHUD
// Author       : Andy Tate
// Date         : 6-Oct-2006
////////////////////////////////////////////////////////////////////////////////

void AVP_ClientActor_Alien::RenderHUD (void) const
{
	// Render the parent...
	PARENT::RenderHUD();
}

/// End of function 'AVP_ClientActor_Alien::RenderHUD'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ClientActor_Alien::MakePlayer
// Author       : Chris Wallace
// Date         : 5-Feb-2007
////////////////////////////////////////////////////////////////////////////////

void AVP_ClientActor_Alien::MakePlayer(Asura_Guid uPlayerGuid)
{
	if (uPlayerGuid == m_uPlayerInfoGuid) { return; }
	// Check if all local players on this machine is aware of their puppets yet, since MakePlayer can fall over if they aren't.
	if (!AVP_Game_Core::GetCanMakePlayerActors()) { return; }

	const bool bLocalPlayer = AVP_Game_Core::IsLocalPlayer(uPlayerGuid);

	PARENT::MakePlayer(uPlayerGuid);

	if( m_pxPlayerStateHandler == NULL )
	{
		m_pxPlayerStateHandler = new AVP_Alien_PlayerAnimStateHandler( *this );
	}
	m_pxPlayerStateHandler->Initialise();

	if (bLocalPlayer)
	{
		if(!m_pxCameraHandler)
		{
			m_pxCameraHandler	= new AVP_Alien_CameraHandler(Guid);
			m_pxCameraHandler->Reset();
		}

		if(!m_pxHUDHandler)
		{
			m_pxHUDHandler	= new AVP_Alien_HUDHandler(*this);
		}

		if(!m_pxInputHandler)
		{
			m_pxInputHandler	= new AVP_Alien_InputHandler(Guid);
			m_pxInputHandler->Reset();
			if (m_pxInputHandler)
			{
				m_pxInputHandler->RefreshBlueprintData();
			}
		}

		if(!m_pxJumpHandler)
		{
			m_pxJumpHandler	= new AVP_Alien_JumpHandler(*this);
		}

		if (!m_pxFocus)
		{
			m_pxFocus = new AVP_Alien_Focus(*this);
		}

		if (!m_pxInteractionHandler)
		{
			m_pxInteractionHandler = new AVP_Alien_InteractionHandler(*this);
		}
	}

	if(m_pxPhysicsObjectHandler != NULL)
	{
//		AVP_CalculateSurfaceReorientationNormal*	pxCalculate;

		m_pxPhysicsObjectHandler->EnableSurfaceReorientation();

//		pxCalculate	= m_pxPhysicsObjectHandler->GetCalculateSurfaceReorientationNormal();
//
//		if(pxCalculate != NULL)
//		{
//			float fForwardsLimit	= -0.866f; // default 30 degrees
//			float fSidewaysLimit	= -0.866f;
//			float fBackwardsLimit	= -0.866f;
//
//			Asura_Blueprint* pxBP = GetBlueprint();
//			if ( pxBP )
//			{
//				Asura_Value xVal;
//				if ( pxBP->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_SRO_ANGLELIMIT_FORWARDS, xVal ) )
//				{
//					fForwardsLimit = -Asura_Maths::Cosine( Asura_Maths::DegToRad( xVal.GetFloatValue() ) );
//				}
//				if ( pxBP->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_SRO_ANGLELIMIT_SIDEWAYS, xVal ) )
//				{
//					fSidewaysLimit = -Asura_Maths::Cosine( Asura_Maths::DegToRad( xVal.GetFloatValue() ) );
//				}
//				if ( pxBP->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_SRO_ANGLELIMIT_BACKWARDS, xVal ) )
//				{
//					fBackwardsLimit = -Asura_Maths::Cosine( Asura_Maths::DegToRad( xVal.GetFloatValue() ) );
//				}
//			}
//
//			pxCalculate->SetDirectionCosLimits(fForwardsLimit, fSidewaysLimit, fBackwardsLimit);
//		}
	}
}
/// End of function 'AVP_ClientActor_Alien::MakePlayer'. ///

void AVP_ClientActor_Alien::MakeAI()
{
	PARENT::MakeAI();

	if(m_pxPhysicsObjectHandler != NULL)
	{
		m_pxPhysicsObjectHandler->EnableSurfaceReorientation();
	}
}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ClientActor_Alien::HandleMessage
// Author       : Chris Wallace
// Date         : 8-Feb-2007
////////////////////////////////////////////////////////////////////////////////

bool AVP_ClientActor_Alien::HandleMessage( const Asura_Message* pxMessage )
{
	// What sort of message is it?
	switch(pxMessage->GetMsgID())
	{
	    case ASURA_MSG_CLIENTUPDATE:
		{
			const bool	bIsLocalPlayer	= IsPlayer() && AVP_Game_Core::IsLocalPlayer(GetPlayerInfoGuid());

			ProcessClientUpdate(static_cast<const AVP_Message_AlienClientUpdate*>(pxMessage->GetData()), pxMessage->GetSentTime(), !bIsLocalPlayer);
			return(true);
		}

	    default:
		{
			return(PARENT::HandleMessage(pxMessage));
		}
	}
}

/// End of function 'AVP_ClientActor_Alien::HandleMessage'. ///


void AVP_ClientActor_Alien::SetStanding( const bool bSet, const bool bCheckIfPossible )
{
	if (IsDead()) { return; }

	bool bDoIt = true;
	if ( bCheckIfPossible && AVP_Game_Core::IsLocalPlayer( GetGuid() ) )
	{
		const AVP_Actor_AnimStateData* pxStateData = AVP_Alien_PlayerAnimStateData::GetSingleton().GetAnimStateData( GetAnimStateBlueprintType(), AVP_Game_Core::IsLocalPlayer( GetGuid() ), true, bSet, false, false, AVP_Vertebrate_PlayerAnimStateHandler::MOVESPEED_IDLE );
		const AVP_Collision_Descriptor* pxCollDesc = pxStateData ? AVP_Collision_Descriptor::GetDataByHash( pxStateData->GetCollisionDescriptorHash() ) : NULL;
		if ( pxCollDesc )
		{
			const Asura_Collision_Box* pxCollBox = pxCollDesc->GetCollisionBox();
			if ( pxCollBox )
			{
				bDoIt = CanChangeCollisionObject( *pxCollBox );
			}
		}
	}
	if ( bDoIt && IsStanding() != bSet)
	{
		m_bStanding = bSet;
		AVP_PhysicsObjectHandler*	pxPhysicsObjectHandler	= GetPhysicsObjectHandler();
		AVP_Alien_AnimationHandler* pxAnimHandler = static_cast< AVP_Alien_AnimationHandler* >( GetAnimationHandler() );
		if ( pxPhysicsObjectHandler != NULL )
		{
			if ( bSet )
			{
				/*pxPhysicsObjectHandler->DisableSurfaceReorientation();*/
				pxPhysicsObjectHandler->SetForcedTargetSurfaceNormal( -Asura_Physics_System::GetGravityDirection() );
				if ( pxAnimHandler != NULL )
				{
					pxAnimHandler->DisableArmsIK();
				}
			}
			else
			{
				pxPhysicsObjectHandler->UnsetForcedTargetSurfaceNormal();
				//pxPhysicsObjectHandler->EnableSurfaceReorientation();
				if ( pxAnimHandler != NULL )
				{
					pxAnimHandler->EnableArmsIK();
				}
			}
		}
	}

	if (bDoIt && bSet)
	{
		m_fTimeStanding = 0.0f;
	}
}

/// End of function 'AVP_ClientActor_Alien::SetStanding'. ///

void AVP_ClientActor_Alien::GetCurrentBlockBlueprintHashes(Asura_Hash_ID& uBlueprintTypeHash, Asura_Hash_ID& uFirstPersonBlueprintHash, Asura_Hash_ID& uThirdPersonBlueprintHash) const
{
	// The alien doesn't have weapon variations, so the blueprint ids come from the main actor blueprint.
	uBlueprintTypeHash = uAVP_BLUEPRINT_TYPE_MOVESALIEN;
	uFirstPersonBlueprintHash = uThirdPersonBlueprintHash = ASURA_HASH_ID_UNSET;

	const Asura_Blueprint* pxBlueprint = GetBlueprint();
	if (!pxBlueprint) { return; }

	Asura_Value xValue;
	if (pxBlueprint->GetParameterValueByHashID(uAVP_BLUEPRINT_PARAM_BLOCK_FIRSTPERSON, xValue))
	{
		uFirstPersonBlueprintHash = xValue.GetHashIDValue();
	}

	if (pxBlueprint->GetParameterValueByHashID(uAVP_BLUEPRINT_PARAM_BLOCK_THIRDPERSON, xValue))
	{
		uThirdPersonBlueprintHash = xValue.GetHashIDValue();
	}
}

bool AVP_ClientActor_Alien::IsBoneInTorso( const int iBoneID )
{
	return AVP_Alien_Description::GetElementFromBoneID( iBoneID ) == AVP_SET_BODY;
}

void AVP_ClientActor_Alien::ProcessClientUpdate( const AVP_Message_AlienClientUpdate* pxData, const float fServerTime, const bool bProcessPosAndOri /*= true*/ )
{
	PARENT::ProcessClientUpdate( pxData, fServerTime, bProcessPosAndOri );
	
	if ( !AVP_Game_Core::IsLocalPlayer( GetGuid() ))
	{
		SetStanding( pxData->IsStanding(), false );
	}

	bool bInsidePounceRange = pxData->IsEnemyInsidePounceRange();
	if( GetFocus() )
	{
		bInsidePounceRange = ( GetFocus()->GetMeleeTarget() != ASURA_GUID_UNREGISTERED )? true : bInsidePounceRange;
	}

	AVP_Vertebrate_HUDHandler::SetGrabTargetInRange( pxData->IsEnemyInsideGrabRange() );
	AVP_Vertebrate_HUDHandler::SetPounceTargetInRange( bInsidePounceRange );
	AVP_Alien_HUDHandler::SetTailstabTargetInRange( pxData->IsEnemyInsideTailAttackRange() );

	m_bIsInShadow = pxData->IsAlienInShadow();

	m_bIsFaceHuggerSummonable = pxData->IsFaceHuggerSummoningPointAvailable();
	m_uFaceHuggerPetStatus = pxData->GetFaceHuggerPetStatus();
}

const AVP_Actor_AnimStateData* AVP_ClientActor_Alien::GetStateDataFromMesssageData( const u_int uState, const AVP_Message_Actor_ClientUpdate* pxActorData )
{
	const AVP_Message_AlienClientUpdate* pxData = static_cast< const AVP_Message_AlienClientUpdate* >( pxActorData );
	return AVP_Alien_AIAnimStateData::GetAnimStateData( GetAnimStateBlueprintType(), uState, pxData->GetGroundType(), pxData->GetLimbDamage() );
}

const AVP_Actor_AnimStateData* AVP_ClientActor_Alien::GetStateData( const u_int uState, const u_int uGroundType, const u_int uLimbDamage, const bool bIsAiming, const u_int uCoverType)
{
	return AVP_Alien_AIAnimStateData::GetAnimStateData( uState, uGroundType, static_cast<AVP_Alien_AIAnimStateMachine::LimbDamage>(uLimbDamage) );
}

void AVP_ClientActor_Alien::GetFootPosition( u_int uFootItr, Asura_Vector_3& xFootPos )
{
	if ( m_pxAnimationHandler )
	{
		m_pxAnimationHandler->GetAnimation()->GetBoneLocation( AVP_Alien_Description::GetFootBoneID( uFootItr, m_pxAnimationHandler->IsFirstPerson() ), &xFootPos, NULL, true );
	}
}


void AVP_ClientActor_Alien::OnDecapitation()
{
	if (Asura_Network::IsServer())
	{
		AVP_ServerActor_Alien* pxServer = AVP_ServerActor_Alien::FindEntity(GetGuid());
		if (pxServer) pxServer->CreateAcidVolume();
	}

}


/// End of file. ///
