
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_Messages_Alien.cpp
// Author : Chris Wallace
// Date   : 7-Feb-2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "Entity/PhysicalObject/Actor/Vertebrates/Alien/AVP_Messages_Alien.h"

// Asura independent header-files...
#include "Core/Asura_BitPacker.h"
#include "Network/Asura_NetMsg.h"

// Asura platform header-files...

// Game independent header-files...
#include "Actor/Vertebrates/Alien/AVP_Alien_Focus.h"
#include "Actor/Vertebrates/Alien/AVP_Alien_InputHandler.h"
#include "Actor/Vertebrates/Alien/AVP_Alien_MeleeCounter_System.h"
#include "Entity/AVP_Entity_Classification.h"
#include "Entity/AVP_Messages.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Alien/AVP_ClientActor_Alien.h"

// Game platform header-files...

// Tools header-files...


////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Private functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Protected functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Public functions...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Message_HumanClientUpdate::FillHuman
// Author       : Andy Tate
// Date         : 6-Oct-2006
////////////////////////////////////////////////////////////////////////////////

void AVP_Message_AlienClientUpdate::FillAlien(const AVP_ServerActor_Alien& xAlien)
{
	// Fill-in the base-class...
    FillVertebrate(xAlien);

	m_bStanding = xAlien.IsStanding();
	m_bInsideGrabRange = xAlien.GetMeleeSystem()->IsCurrentTargetInSuckRange();
	m_bInsidePounceRange = static_cast<const AVP_Alien_MeleeCounter_System* >( xAlien.GetMeleeSystem() )->InsidePounceRange();
	m_bInsideTailAttackRange = static_cast<const AVP_Alien_MeleeCounter_System* >( xAlien.GetMeleeSystem() )->InsideTailAttackRange();
	m_bIsFaceHuggerSummoningPointAvailable = xAlien.IsFaceHuggerSummoningPointAvailable();
	m_bIsAlienInShadow = xAlien.IsInShadow();
	m_uFaceHuggerPetStatus = xAlien.GetFaceHuggerPetStatus();
	m_eLimbDamage = xAlien.GetLimbDamage();
}

/// End of function 'AVP_Message_AlienClientUpdate::FillHuman'. ///



#ifdef ASURA_SUPPORT_NETMSG_SYSTEM
void AVP_Message_AlienClientUpdate::RegisterFields(u_short usMsgID, u_short usEntityClassification)
{
	Asura_Assert(usMsgID == ASURA_MSG_CLIENTUPDATE, "invalid msg ID");
	Asura_Assert(!Asura_NetMsg::FindNetMsgObject(usMsgID, usEntityClassification), "already exists");
	Asura_Assert(usEntityClassification == AVP_ENTITYCLASS_ALIEN, "Unexpected entity classification.");
	
	u_char ucNumFields = GetNumFields();

	Asura_NetMsg* pxNetMsg = new Asura_NetMsg(usMsgID, usEntityClassification, ucNumFields, sizeof(AVP_Message_AlienClientUpdate));
	RegisterFields(pxNetMsg);
	pxNetMsg->EndRegistration();
}


void AVP_Message_AlienClientUpdate::RegisterFields(Asura_NetMsg* pxNetMsg)
{
	const AVP_Message_AlienClientUpdate xDummy;
	PARENT::RegisterFields(pxNetMsg);

	Asura_NetMsg_RegisterField(pxNetMsg, xDummy, Asura_FieldType_uchar, m_bStanding, 1, Asura_Threshold_AnyChange, 0.0f);
	Asura_NetMsg_RegisterField(pxNetMsg, xDummy, Asura_FieldType_uchar, m_bIsFaceHuggerSummoningPointAvailable, 1, Asura_Threshold_AnyChange, 0.0f);
	Asura_NetMsg_RegisterField(pxNetMsg, xDummy, Asura_FieldType_uchar, m_bIsAlienInShadow, 1, Asura_Threshold_AnyChange, 0.0f);
	Asura_NetMsg_RegisterField(pxNetMsg, xDummy, Asura_FieldType_uint, m_uFaceHuggerPetStatus, 1, Asura_Threshold_AnyChange, 0.0f);
	Asura_NetMsg_RegisterField(pxNetMsg, xDummy, Asura_FieldType_uint, m_eLimbDamage, 1, Asura_Threshold_AnyChange, 0.0f);

}
#endif//#ifdef ASURA_SUPPORT_NETMSG_SYSTEM

// Constructors 

AVP_Message_AlienPlayerCommands::AVP_Message_AlienPlayerCommands( const AVP_ClientActor_Alien& xAlien )
{
	FillAlien( xAlien ); 
}

AVP_Message_AlienPlayerCommands::AVP_Message_AlienPlayerCommands( Asura_BitUnpacker& xUnpacker )
{
	Unpack( xUnpacker ); 
}

void AVP_Message_AlienPlayerCommands::FillAlien( const AVP_ClientActor_Alien& xAlien )
{
	FillVertebrate( xAlien );

	m_bStanding = xAlien.IsStanding();
	const AVP_Actor_InputHandler* pxInputHandler = xAlien.GetInputHandler();
	if (pxInputHandler)
	{
		m_bFacehuggerPetRequested = pxInputHandler->GetControlMapValue(AVP_Alien_InputHandler::PET_FACEHUGGER_CONTROL) > 0.0f;
		m_bDistractionRequested = pxInputHandler->GetControlMapValue(AVP_Alien_InputHandler::CREATE_DISTRACTION) > 0.0f;
		m_bTailChargeRequested = pxInputHandler->GetControlMapValue(AVP_Alien_InputHandler::TAIL_ATTACK_HELD) > 0.5f;
	}
	else
	{
		m_bFacehuggerPetRequested = false;
		m_bDistractionRequested = false;
		m_bTailChargeRequested = false;
	}
}

void AVP_Message_AlienPlayerCommands::Pack( Asura_BitPacker &xPacker ) const
{
	PARENT::Pack( xPacker ); 

	xPacker.PackBool( m_bStanding ); 
	xPacker.PackBool( m_bFacehuggerPetRequested ); 
	xPacker.PackBool( m_bDistractionRequested ); 
	xPacker.PackBool( m_bTailChargeRequested ); 
}

void AVP_Message_AlienPlayerCommands::Unpack( Asura_BitUnpacker &xUnpacker )
{
	PARENT::Unpack( xUnpacker ); 

	m_bStanding = xUnpacker.UnpackBool();
	m_bFacehuggerPetRequested = xUnpacker.UnpackBool(); 
	m_bDistractionRequested = xUnpacker.UnpackBool(); 
	m_bTailChargeRequested = xUnpacker.UnpackBool(); 
}