
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_ServerActor_Alien.cpp
// Author : Andy Tate
// Date   : 3-Oct-2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "Entity/PhysicalObject/Actor/Vertebrates/Alien/AVP_ServerActor_Alien.h"

// Asura independent header-files...
#include "Blueprint/Asura_Blueprint_System.h"
#include "Camera/Asura_Camera.h"
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "Core/Asura_BitPacker.h"
#include "Core/Asura_Timers.h"
#include "Physics/Asura_Physics.h"
#ifndef ASURA_NO_DEBUG_OUTPUT
#include "Rendering/Asura_Font_System.h"
#endif
#include "Rendering/Asura_Viewport.h"

// Game independent header-files...
#include "Actor/AVP_Actor_InputHandler.h"
#include "Actor/AVP_Actor_InteractionHandler.h"
#include "Actor/AVP_Actor_ObjectiveHandler.h"
#include "Actor/AVP_Actor_PhysicsObjectHandler.h"
#include "Actor/AVP_Actor_PickupHandler.h"
#include "Actor/Vertebrates/Alien/AVP_Alien_AnimationHandler.h"
#include "Actor/Vertebrates/Alien/AVP_Alien_HUDHandler.h"
#include "Actor/Vertebrates/Alien/AVP_Alien_MeleeCounter_System.h"
#include "Actor/Vertebrates/Human/AVP_Human_MoraleHandler.h"
#include "Animation/AVP_AnimFlags.h"
#include "Blueprint/AVP_Blueprint_HashIDs.h"
#include "Core/AVP_Game_Core.h"
#include "Core/AVP_Settings.h"
#include "Entity/AVP_Entity_Classification.h"
#include "Entity/AVP_Entity_EventIDs.h"
#include "Entity/AVP_Messages.h"
#include "Entity/Controller/AVP_ServerEntity_Team_Alien.h"
#include "Entity/Controller/AVP_ServerEntity_Squad_Alien.h"
#include "Entity/Damage/AVP_AcidDamage_Messages.h"
#include "Entity/Damage/AVP_EntityDamage_Messages.h"
#include "Entity/GamePlayVolume/AVP_ServerEntity_GamePlayVolume.h"
#include "Entity/PhysicalObject/Actor/AVP_ServerEntity_Actor.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Alien/AVP_ClientActor_Alien.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Alien/AVP_Messages_Alien.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/FaceHugger/AVP_ServerActor_FaceHugger.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Human/AVP_ServerActor_Human.h"
#include "Entity/PhysicalObject/Actor/Weapons/Sentrygun/AVP_Sentrygun_Messages.h"
#include "Entity/PhysicalObject/Interact/InvisiSwitch/AVP_ClientEntity_InteractInvisiSwitch.h"
#include "Entity/PhysicalObject/Interact/ToolReceptacle/BatteryReceptacle/AVP_ServerEntity_InteractBatteryReceptacle.h"
#include "Entity/PlayerInfo/AVP_ServerEntity_PlayerInfo.h"
#include "Entity/Ragdoll/Asura_Ragdoll_Wrapper.h"
#include "Entity/Spawners/AVP_ServerEntity_SpawnPoint2.h"
#include "Inventory/AVP_Inventory_HashIDs.h"
#include "Inventory/AVP_Inventory_System.h"
#include "Mesh/AVP_PolygonFlags.h"
#include "Moves/AVP_Move_Interaction.h"
#include "Moves/AVP_Movement_System.h"
#include "Moves/AVP_Moves.h"
#include "Moves/Vertebrates/Alien/AVP_Alien_MovesInteraction.h"
#include "Moves/Vertebrates/Alien/AVP_Alien_MovesTail.h"
#include "Physics/Asura_Physics_Ragdoll.h"
#include "SpecialFX/pfx/AVP_DamagePFX.h"
#include "Rendering/Asura_Primitive_Render.h"
#include "Volume/Asura_VolumeGroup.h"
#include "Volume/Asura_Volume_Sphere.h"
#include "Volume/Asura_VolumeGroup.h"
// Axon
#include "Core/Axon_System.h"
#include "Core/AVP_Waypoint_Manager.h"
#include "Core/Axon_Project_System.h"
#include "Core/BrainTypes/AVP_Brain_Actor_Alien.h"
#include "Knowledge/AVP_Knowledge_System.h"
#include "Knowledge/AVP_Knowledge_System_Predator.h"
#include "Knowledge/Axon_Knowledge.h"
#include "Tactical/AVP_Tactical_System.h"
#include "Knowledge/AVP_Knowledge_System_Actor.h"
#include "Knowledge/SenseTypes/AVP_Sense_Vision_Actor.h"
#include "Knowledge/SenseTypes/AVP_Sense_Vision_Predator.h"

////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////

// Predator movement speeds...

DEBUGVAR float dbg_fStandingStateChangeTweenTime = 0.2f;
DEBUGVAR float dbg_fLimbDamageStateChangeTweenTime = 0.2f;

////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////

static const Asura_Hash_ID s_bAliensExplodeWhenDie				= 0x7E15AC91;	// "AliensExplodeWhenDie"
static const Asura_Hash_ID s_bAliensExplodeWhenDie_DamageAmount = 0x7704951A;	// "AliensExplodeWhenDie.Damage.Amount"
static const Asura_Hash_ID s_bAliensExplodeWhenDie_FarRange		= 0x59BD20C8;	// "AliensExplodeWhenDie.Damage.FarRange"
static const Asura_Hash_ID s_bAliensExplodeWhenDie_NearRange	= 0x9D2EF613;	// "AliensExplodeWhenDie.Damage.NearRange"


float AVP_ServerActor_Alien::s_fHeadsplitInitialAcidDamageDelay = 1.f;
float AVP_ServerActor_Alien::s_fHeadsplitTimeBetweenAcidDamages = 1.f;
u_int AVP_ServerActor_Alien::s_uHeadsplitNumberOfAcidDamages = 10;
float AVP_ServerActor_Alien::s_fHeadsplitAcidDamageRadius = 1.f;
float AVP_ServerActor_Alien::s_fHeadsplitAcidDamageAmount = 100.f;

Asura_Hashed_List<AVP_ServerActor_Alien>	AVP_ServerActor_Alien::s_xList;

////////////////////////////////////////////////////////////////////////////////
//  Functions...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ServerActor_Alien::CheckForEnvironmentalAcidDamage
// Author       : Andy Tate
// Date         : 3-Mar-2008
////////////////////////////////////////////////////////////////////////////////

void AVP_ServerActor_Alien::CheckForEnvironmentalAcidDamage(const Asura_Vector_3& xStartPosition,
															const Asura_Vector_3& xDirection,
															const float fRange,
															bool bIsTrailMode)
{
	const Asura_LOS_InputData	xLOSInput(xStartPosition, xDirection, GetGuid(), fRange);
	const Asura_LOS_OutputData*	pxLOSOutput	= Asura_LOS::TestAgainstEnvironment(&xLOSInput);

	// Check whether we cause acid damage to the floor...
	if(pxLOSOutput->m_fDistanceToIntersection < fRange)
	{
		Asura_Message*	pxMessage	= Asura_Network::CreateMessage(AVP_MSGID_CREATE_ACID_DAMAGE, sizeof(AVP_Message_AcidDamage_Create));

		if(pxMessage != NULL)
		{
			AVP_Message_AcidDamage_Create*	pxData	= static_cast<AVP_Message_AcidDamage_Create*>(pxMessage->GetData());

			if(pxData != NULL)
			{
				const Asura_Guid		uGuid	= GetGuid();
				const Asura_Vector_3	xPointOfImpact(xStartPosition + (xDirection * pxLOSOutput->m_fDistanceToIntersection));
				const Asura_Vector_3	xSurfaceNormal(pxLOSOutput->m_xNormal);

				pxData->SetSourceGuid(uGuid);
				pxData->SetPosition(xPointOfImpact);
				pxData->SetSurfaceNormal(xSurfaceNormal);
				pxData->SetIsTrailMode(bIsTrailMode);
			}

			Send(pxMessage, ASURA_GUID_SERVER);
		}
	}
}

/// End of function 'AVP_ServerActor_Alien::CheckForEnvironmentalAcidDamage'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ServerActor_Alien::OnApplyDamage
// Author       : Andy Tate
// Date         : 26-Feb-2008
////////////////////////////////////////////////////////////////////////////////

void AVP_ServerActor_Alien::OnApplyDamage(const AVP_EntityDamage_ApplyToEntity& xDamage)
{
	PARENT::OnApplyDamage(xDamage);
	
// 	if ( xDamage.GetAmount() > 0.0f )
// 	{
// 		Axon_System::GetSoundRegistry()->RegisterSound( GetGuid(), GetClassification(), GetPosition(), 20.0f, AVP_SOUND_TYPE_ALIEN_ROAR );
// 	}
}

/// End of function 'AVP_ServerActor_Alien::OnApplyDamage'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ServerActor_Alien::OnDeath
// Author       : Adam Law
// Date         : 04-May-2007
////////////////////////////////////////////////////////////////////////////////

void AVP_ServerActor_Alien::OnDeath(const AVP_EntityDamage_ApplyToEntity& xDamage)
{
	const Asura_Vector_3			xDirection(0.0f, 1.0f, 0.0f);
	const float						fRange	= 5.0f;

	Asura_Vector_3					xStartPosition;

	// Call the parent function...
	PARENT::OnDeath(xDamage);

	// Decide whether to create any acid-damage below this Alien. Don't create if this comes from a grapple so we don't damage our attacker
	const Asura_Hash_ID uDamageType = xDamage.GetTypeHash();
	if ((uDamageType != uAVP_BLUEPRINT_NAME_DAMAGE_DISMEMBER) && (uDamageType != uAVP_BLUEPRINT_NAME_DAMAGE_GRAPPLE))
	{
		xDamage.GetPointOfImpact(xStartPosition);
		CheckForEnvironmentalAcidDamage(xStartPosition, xDirection, fRange, false);
	}

	// Default values for acid damage.
	float fAmount					= 50.0f;
	float fRadiusFor100PercentDamage= 1.0f;
	float fRadiusFor0PercentDamage	= 2.5f;
	bool  bDoesAcidSplat			= false;

	// Now let see If we can read correct values from blueprint.
	Asura_Blueprint* pxBlueprint = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_DAMAGE, xDamage.GetTypeHash() );
	if ( pxBlueprint )
	{
		Asura_Value xValue;
		if ( pxBlueprint->GetParameterValueByHashID( s_bAliensExplodeWhenDie, xValue ) )
		{
			bDoesAcidSplat = xValue.GetBoolValue();

			if( bDoesAcidSplat )
			{
				if ( pxBlueprint->GetParameterValueByHashID( s_bAliensExplodeWhenDie_DamageAmount, xValue ) )
				{
					fAmount = xValue.GetFloatValue();
				}
				if ( pxBlueprint->GetParameterValueByHashID( s_bAliensExplodeWhenDie_FarRange, xValue ) )
				{
					fRadiusFor0PercentDamage = xValue.GetFloatValue();
				}
				if ( pxBlueprint->GetParameterValueByHashID( s_bAliensExplodeWhenDie_NearRange, xValue ) )
				{
					fRadiusFor100PercentDamage = xValue.GetFloatValue();
				}
			}
		}
	}

	if ( bDoesAcidSplat )
	{
		const Asura_Guid						uAttackerGuid				= GetGuid();
		const float								fImpulseStrength			= 0.0f;

		const AVP_EntityDamage_RequestRadial	xDamage(uAVP_BLUEPRINT_NAME_DAMAGE_ACID,
														uAttackerGuid,
														ASURA_HASH_ID_UNSET,
														fAmount,
														fImpulseStrength,
														uAttackerGuid,
														fRadiusFor100PercentDamage,
														fRadiusFor0PercentDamage,
														true);

		// Apply this damage...
		xDamage.Apply(*this);
	}

	// Send event, if AI alien
	if( ( m_bIsASpawnedAlien ) && ( AVP_ServerEntity_Team_Alien::GetInstance() ) )
	{
		AVP_ServerEntity_Team_Alien::GetInstance()->OnAlienKilled();
	}

	AVP_ServerEntity_Actor* pxKillingActor = AVP_ServerEntity_Actor::FindEntity( xDamage.GetFromGuid() );

	if( pxKillingActor )
	{
		if( ( pxKillingActor->GetClassification() == AVP_ENTITYCLASS_MARINE ) && ( !pxKillingActor->IsPlayer() ) )
		{
			EffectMorale( xDamage.GetFromGuid() );
		}

		if (!pxKillingActor->IsPlayer())
		{
			if ((pxKillingActor->GetClassification() == AVP_ENTITYCLASS_MARINE) || (pxKillingActor->GetClassification() == AVP_ENTITYCLASS_ANDROID))
			{
				if (Asura_Random::GetFloat(0.0f, 1.0f) > 0.6f)
					((AVP_ServerActor_Human*)pxKillingActor)->PlayDialogue(AVP_ServerActor_Human::AVP_DLG_HUMAN_KILLED_THREAT);
			}
		}
	}
}

//-----------------------------------------------------------------
// AVP_ServerActor_Alien :: CalculateAndUpdateLocationSpecificDamage
//-----------------------------------------------------------------

// DEBUGVAR u_int dbg_uDestroySkinElementTest = (1<< AVP_SET_RIGHT_LEG) + 
// 											 (1 << AVP_SET_RIGHT_KNEE) + 
// 											 (1 << AVP_SET_LEFT_LEG) + 
// 											 (1 << AVP_SET_LEFT_KNEE) +
// 											 (1 << AVP_SET_RIGHT_ARM) + 
// 											 (1 << AVP_SET_RIGHT_ELBOW) + 
// 											 (1 << AVP_SET_LEFT_ARM) + 
// 											 (1 << AVP_SET_LEFT_ELBOW );

DEBUGVAR u_int dbg_uDestroyArmsElementTest = (1 << AVP_SET_RIGHT_ARM) + 
											 (1 << AVP_SET_RIGHT_ELBOW) + 
											 (1 << AVP_SET_LEFT_ARM) + 
											 (1 << AVP_SET_LEFT_ELBOW );

// DEBUGVAR u_int dbg_uDestroyLegsElementTest = (1<< AVP_SET_RIGHT_LEG) + 
// 											 (1 << AVP_SET_RIGHT_KNEE) + 
// 											 (1 << AVP_SET_LEFT_LEG) + 
// 											 (1 << AVP_SET_LEFT_KNEE);

float AVP_ServerActor_Alien::CalculateAndUpdateLocationSpecificDamage(const AVP_EntityDamage_RequestVictim& xDamage, u_int& uDestroyedSkinElements, u_int& uGibbedSkinElements, bool bForceFatal)
{
	float fAmount = PARENT::CalculateAndUpdateLocationSpecificDamage(xDamage, uDestroyedSkinElements, uGibbedSkinElements, bForceFatal);

	// if the alien is about to die anyhow or damaging limbs in not allowed no need to mess with the limbs
	if (bForceFatal || (GetHealth() < fAmount) || IsDead() || !GetAllowDamageLimb())
	{
		return fAmount;
	}

	// if both arms are destroyed - kill the alien.
	if ( (!IsSkinElementAvailable(AVP_SET_RIGHT_ARM) || !IsSkinElementAvailable(AVP_SET_RIGHT_ELBOW)) &&
		 (!IsSkinElementAvailable(AVP_SET_LEFT_ARM) || !IsSkinElementAvailable(AVP_SET_LEFT_ELBOW)) )
	{
		return Asura_Maths::LargeFloat;
	}

	// if one of the legs is destroyed - do nothing
	if (!IsSkinElementAvailable(AVP_SET_LEFT_LEG) || !IsSkinElementAvailable(AVP_SET_RIGHT_LEG))
	{
		return fAmount;
	}

	// if health low enough or lost a hand or arm
	if ((GetHealth()-fAmount) / GetInitialHealth() < m_fHealthToEnterInjuredFraction || uDestroyedSkinElements & dbg_uDestroyArmsElementTest) 
	{

		if (Asura_Random::GetInt(0,1) == 0)
		{
			AVP_EntityDamage_RequestVictim xDamageLeft(xDamage.GetTypeHash(), 
				xDamage.GetFromGuid(), 
				xDamage.GetFromMarkerPointHash(), 
				xDamage.GetEntityToDamage(), 
				10000.0f, 
				Asura_Maths::ZeroVector3, 
				Asura_Maths::ZeroVector3, 
				Asura_Maths::ZeroVector3, 
				0.0f, 
				AVP_Alien_Description::TP_BIP01_L_FOOT);
			u_int uDestroyLeft, uGibbedLeft;
			PARENT::CalculateAndUpdateLocationSpecificDamage(xDamageLeft, uDestroyLeft, uGibbedLeft, bForceFatal);
			uDestroyedSkinElements |= uDestroyLeft;
			uGibbedSkinElements |= uGibbedLeft;
		}
		else
		{
			AVP_EntityDamage_RequestVictim xDamageRight(xDamage.GetTypeHash(), 
				xDamage.GetFromGuid(), 
				xDamage.GetFromMarkerPointHash(), 
				xDamage.GetEntityToDamage(), 
				10000.0f, 
				Asura_Maths::ZeroVector3, 
				Asura_Maths::ZeroVector3, 
				Asura_Maths::ZeroVector3, 
				0.0f, 
				AVP_Alien_Description::TP_BIP01_R_FOOT);
			u_int uDestroyRight, uGibbedRight;
			PARENT::CalculateAndUpdateLocationSpecificDamage(xDamageRight, uDestroyRight, uGibbedRight, bForceFatal);
			uDestroyedSkinElements |= uDestroyRight;
			uGibbedSkinElements |= uGibbedRight;
		}
	}

	return fAmount;
}


//-----------------------------------------------------------------
// AVP_ServerActor_Alien :: EffectMorale
//-----------------------------------------------------------------
void AVP_ServerActor_Alien::EffectMorale(Asura_Guid uGuidToEffect)
{
	AVP_ServerActor_Human* pxHuman = AVP_ServerActor_Human::FindEntity(uGuidToEffect);

	if (pxHuman)
	{
		if (pxHuman->GetInitialSquadGuid() != ASURA_GUID_UNREGISTERED)
		{
			AVP_ServerEntity_Squad_Marine* pxMarineSquad = AVP_ServerEntity_Squad_Marine::FindEntity(pxHuman->GetInitialSquadGuid());

			if (pxMarineSquad)
			{
				if (!pxMarineSquad->IsPlayersSquad())
				{
					if (pxHuman->GetMoraleHandler())
					{
						pxHuman->GetMoraleHandler()->EffectMorale(AVP_Human_MoraleHandler::AVP_MORALE_EFFECTOR_KILLED_ALIEN);
						pxMarineSquad->EffectSquadMorale(AVP_Human_MoraleHandler::AVP_MORALE_EFFECTOR_SQUAD_MEMBER_KILLED_ALIEN, pxHuman->GetGuid());
					}
				}
			}
		}
		else
		{
			if (!pxHuman->IsPlayer())
			{
				if (pxHuman->GetMoraleHandler())
				{
					pxHuman->GetMoraleHandler()->EffectMorale(AVP_Human_MoraleHandler::AVP_MORALE_EFFECTOR_KILLED_ALIEN);
				}
			}
		}
	}
}

void AVP_ServerActor_Alien::Create(Asura_Guid uGUID)
{
    new AVP_ServerActor_Alien(uGUID);
}

AVP_ServerActor_Alien* AVP_ServerActor_Alien::CreateAndReturn(Asura_Guid uGUID)
{
	AVP_ServerActor_Alien* pxAlien = new AVP_ServerActor_Alien(uGUID);

	return pxAlien;
}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ServerActor_Alien::AVP_ServerActor_Alien
// Author       : Andy Tate
// Date         : 3-Oct-2006
////////////////////////////////////////////////////////////////////////////////

AVP_ServerActor_Alien::AVP_ServerActor_Alien(const Asura_Guid uGUID)
: Asura_Entity_Server(uGUID)
, PARENT(AVP_PolygonFlag_IgnoreForAlienCollision, uGUID)
, m_bIsASpawnedAlien(false)
, m_bStanding(false)
, m_eLimbDamage(AVP_Alien_AIAnimStateMachine::LIMBDMG_NONE)
, m_fHealthToEnterInjuredFraction(0.0f)
, m_bIsAcidTrailOn(false)
, m_xLastAcidTrailLocation(Asura_Maths::ZeroVector3)
, m_uCurrentFacehuggerBuddy(ASURA_GUID_UNREGISTERED)
, m_uCandidateFaceHuggerPetSpawnPoint(ASURA_GUID_UNREGISTERED)
, m_uFaceHuggerPetStatus(0)
, m_bIsInShadow( false )
, m_pxAcidVolumeGroup( NULL )
, m_fTimeTillNextAcidDamage( 0.f )
, m_uAcidBurstsLeft( 0 )
{
	// Update the data...
	Classification	= AVP_ENTITYCLASS_ALIEN;

	// Ensure we can walk on walls...
	if(m_pxPhysicsObjectHandler != NULL)
	{
		m_pxPhysicsObjectHandler->EnableSurfaceReorientation();
	}

	m_pxMeleeSystem = new AVP_Alien_MeleeCounter_System(*this);

	// Don't forget to store us in the list...
	s_xList.AddToList(this);
}

/// End of function 'AVP_ServerActor_Predator::AVP_ServerActor_Predator'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ServerActor_Alien::~AVP_ServerActor_Alien
// Author       : Andy Tate
// Date         : 3-Oct-2006
////////////////////////////////////////////////////////////////////////////////

AVP_ServerActor_Alien::~AVP_ServerActor_Alien()
{
	if (m_pxAcidVolumeGroup)
	{
		delete m_pxAcidVolumeGroup;
		m_pxAcidVolumeGroup = NULL;
	}

	// Don't forget to remove us from the list...
	s_xList.RemoveFromList(this);
}

void AVP_ServerActor_Alien::Update(void)
{
	PARENT::Update();

	if (IsPlayer())
	{
		UpdateAndRenderAIThreatStatus();
		UpdateFaceHuggerPetState();
	}
	else
	{		
		if (m_bIsAcidTrailOn)
		{
			DropAcidTrail();
		}
	}

	UpdateAcidVolume();

	if (m_uCurrentFacehuggerBuddy != ASURA_GUID_UNREGISTERED)
	{
		const AVP_ServerActor_FaceHugger* pxBuddy = AVP_ServerActor_FaceHugger::FindEntity(m_uCurrentFacehuggerBuddy);
		if (!pxBuddy || pxBuddy->IsDead())
		{
			m_uCurrentFacehuggerBuddy = ASURA_GUID_UNREGISTERED;
		}
	}

	if (GetLimbDamage() == AVP_Alien_AIAnimStateMachine::LIMBDMG_NONE)
	{
		if( !IsSkinElementAvailable(AVP_SET_LEFT_ARM) || !IsSkinElementAvailable(AVP_SET_LEFT_ELBOW) )
		{
			if( !IsSkinElementAvailable(AVP_SET_RIGHT_ARM) || !IsSkinElementAvailable(AVP_SET_RIGHT_ELBOW) )
			{
				// kill alien without both arms
				AVP_EntityDamage_RequestVictim xDamage( uAVP_BLUEPRINT_NAME_DAMAGE_MELEE,
					GetGuid(),
					ASURA_HASH_ID_UNSET,
					GetGuid(),
					1000000.0f,
					GetPosition(),
					Asura_Vector_3( 0.0f, 1.0f, 0.0f ),
					Asura_Vector_3( 0.0f, 1.0f, 0.0f ) );
				TakeDamageExtended( xDamage, true );
			}
			SetLimbDamage(AVP_Alien_AIAnimStateMachine::LIMBDMG_LEFT_ARM);
		}
		else if( !IsSkinElementAvailable(AVP_SET_RIGHT_ARM) || !IsSkinElementAvailable(AVP_SET_RIGHT_ELBOW) )
		{
			SetLimbDamage(AVP_Alien_AIAnimStateMachine::LIMBDMG_RIGHT_ARM);
		}
		else if( !IsSkinElementAvailable(AVP_SET_LEFT_LEG) || !IsSkinElementAvailable(AVP_SET_LEFT_KNEE) )
		{
			if( !IsSkinElementAvailable(AVP_SET_RIGHT_LEG) || !IsSkinElementAvailable(AVP_SET_RIGHT_KNEE) )
			{
				SetLimbDamage(AVP_Alien_AIAnimStateMachine::LIMBDMG_BOTH_LEGS);
			}
			else
			{
				SetLimbDamage(AVP_Alien_AIAnimStateMachine::LIMBDMG_LEFT_LEG);
			}
		}
		else  if( !IsSkinElementAvailable(AVP_SET_RIGHT_LEG) || !IsSkinElementAvailable(AVP_SET_RIGHT_KNEE) )
		{
			SetLimbDamage(AVP_Alien_AIAnimStateMachine::LIMBDMG_LEFT_LEG);
		}
	}

}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ServerActor_Alien::UpdateAIThreatStatus
// Author       : Andy Tate
// Date         : 3-Oct-2006
////////////////////////////////////////////////////////////////////////////////
void AVP_ServerActor_Alien::UpdateAndRenderAIThreatStatus(void)
{
	if (AVP_Game_Core::IsLocalPlayer(GetGuid()))
	{
		bool bShowBeenHeard = false;

		if (AVP_Knowledge_System_Actor::UpdatePlayerAlienAlertStatus() || AVP_Knowledge_System_Predator::UpdatePlayerAlienAlertStatus())
		{
			bShowBeenHeard = true;
		}
		else
		{
			bShowBeenHeard = false;
		}

		float fThreatPercentage	= AVP_Sense_Vision::UpdatePlayerAlienThreatStatus(this);
		float fPredatorThreadPercentaget = AVP_Sense_Vision_Predator::UpdatePlayerAlienThreatStatus(this);

		if (fPredatorThreadPercentaget > fThreatPercentage)
		{
			fThreatPercentage = fPredatorThreadPercentaget;
		}

		enum SpottedState
		{
			SpottedState_NOT_SPOTTED,
			SpottedState_NOT_SPOTTED_YET,
			SpottedState_DANGER_DANGER,
			SpottedState_SPOTTED,
		};

		u_int uSpottedState = SpottedState_NOT_SPOTTED;

		if (fThreatPercentage < 33.33f)
		{
			uSpottedState = SpottedState_NOT_SPOTTED;
		}
		else if (fThreatPercentage < 66.66f)
		{
			uSpottedState = SpottedState_NOT_SPOTTED_YET;
		}
		else if  (fThreatPercentage < 100.0f)
		{
			uSpottedState = SpottedState_DANGER_DANGER;
		}
		else 
		{
			uSpottedState = SpottedState_SPOTTED;
		}

		AVP_ClientActor_Alien* pxClientAlien = AVP_ClientActor_Alien::FindEntity(GetGuid());

		if (pxClientAlien)
		{
			pxClientAlien->SetHasBeenHeard(bShowBeenHeard);
			pxClientAlien->SetSpottedState(uSpottedState);

		}

		u_short usNearestWPID = ASURA_INVALID_WAYPOINT_ID;

		AVP_Navigation::GetNearestWaypointID( GetPosition(), usNearestWPID );

		if ( usNearestWPID != ASURA_INVALID_WAYPOINT_ID )
		{
			if ( AVP_Waypoint_Manager::GetWPLuminance( usNearestWPID ) < Axon_Project_System::GetLightCutOffValue() )
			{
				m_bIsInShadow = true;
			}
			else
			{
				m_bIsInShadow = false;
			}
		}
	}
}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ServerActor_Alien::UpdateFaceHuggerPetState
////////////////////////////////////////////////////////////////////////////////

void AVP_ServerActor_Alien::UpdateFaceHuggerPetState()
{

	m_uCandidateFaceHuggerPetSpawnPoint = FindFaceHuggerPetSpawnPoint();

	enum
	{
		NO_PET = 0,
		PET_PASSIVE = 1,
		PET_AGGRESSIVE = 2
	};

	m_uFaceHuggerPetStatus = NO_PET;

	if (m_uCurrentFacehuggerBuddy != ASURA_GUID_UNREGISTERED)
	{
		AVP_ServerActor_FaceHugger* pxPet = AVP_ServerActor_FaceHugger::FindEntity(m_uCurrentFacehuggerBuddy);
		if (!pxPet || pxPet->IsHugging() || pxPet->IsDestroyed())
		{
			m_uCurrentFacehuggerBuddy = ASURA_GUID_UNREGISTERED;
		}
		else if (pxPet->IsAggressive())
		{
			m_uFaceHuggerPetStatus = PET_AGGRESSIVE;
		}
		else
		{
			m_uFaceHuggerPetStatus = PET_PASSIVE;
		}

	}
}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ServerActor_Alien::ReadFromChunkStream
// Author       : Andy Tate
// Date         : 3-Oct-2006
////////////////////////////////////////////////////////////////////////////////

void AVP_ServerActor_Alien::ReadFromChunkStream(Asura_Chunk_Stream& xStream)
{
	u_int	uVersion;

	// Read the version...
	xStream >> uVersion;
	Asura_Assert((uVersion <= CURRENT_CHUNK_VERSION), "Unsupported chunk version");

	if (uVersion > 0)
	{
		// Read the parent data...
		PARENT::ReadFromChunkStream(xStream);
	}
	else
	{
		// The vertibrate didn't have chunk data orignally, so direct subclass chunk versions have been bumped to work around it.
		AVP_ServerEntity_Actor::ReadFromChunkStream(xStream);
	}

	if (uVersion == 2)
	{
		xStream >> m_uCurrentFacehuggerBuddy;
	}
	else if (uVersion == 1)
	{
		m_uCurrentFacehuggerBuddy = ASURA_GUID_UNREGISTERED;
	}
	else
	{
		Asura_Assert(false, "Unsupported chunk version");
	}


}

/// End of function 'AVP_ServerActor_Predator::ReadFromChunkStream'. ///

void AVP_ServerActor_Alien::WriteToChunkStream(Asura_Chunk_Stream& xStream) const
{
	// Write the version...
	xStream << CURRENT_CHUNK_VERSION;

	// Write the parent data...
	PARENT::WriteToChunkStream(xStream);

	xStream << m_uCurrentFacehuggerBuddy;
}

Axon_Brain* AVP_ServerActor_Alien::CreateBrain()
{
    return new AVP_Brain_Actor_Alien( this );
}

u_int AVP_ServerActor_Alien::GetFaceHuggerPetStatus() const
{

	enum
	{
		NO_PET = 0,
		PET_PASSIVE = 1,
		PET_AGGRESSIVE = 2
	};

	if (m_uCurrentFacehuggerBuddy == ASURA_GUID_UNREGISTERED)
	{
		return NO_PET;
	}
	

	AVP_ServerActor_FaceHugger* pxPet = AVP_ServerActor_FaceHugger::FindEntity(m_uCurrentFacehuggerBuddy);
	if (!pxPet || pxPet->IsHugging() || pxPet->IsDestroyed())
	{
		return NO_PET;
	}

	if (pxPet->IsAggressive())
	{
		return PET_AGGRESSIVE;
	}
	else
	{
		return PET_PASSIVE;
	}

}

void AVP_ServerActor_Alien::TakeDamageExtended(const AVP_EntityDamage_RequestVictim& xDamage, const bool bForceFatal, const bool bSendMessageToClient )
{
	PARENT::TakeDamageExtended(xDamage, bForceFatal, bSendMessageToClient);

	bool bIsLeftHandDestroyed = !IsSkinElementAvailable(AVP_SET_LEFT_ARM) || !IsSkinElementAvailable(AVP_SET_LEFT_ELBOW);
	bool bIsRightHandDestroyed = !IsSkinElementAvailable(AVP_SET_RIGHT_ARM) || !IsSkinElementAvailable(AVP_SET_RIGHT_ELBOW);

	bool bIsLeftLegDstroyed = !IsSkinElementAvailable(AVP_SET_LEFT_LEG) || !IsSkinElementAvailable(AVP_SET_LEFT_KNEE);
	bool bIsRightLegDstroyed = !IsSkinElementAvailable(AVP_SET_RIGHT_LEG) || !IsSkinElementAvailable(AVP_SET_RIGHT_KNEE);

	if (bIsLeftHandDestroyed)
	{
		SetLimbDamage( AVP_Alien_AIAnimStateMachine::LIMBDMG_LEFT_ARM);
	}
	else if (bIsRightHandDestroyed)
	{
		SetLimbDamage( AVP_Alien_AIAnimStateMachine::LIMBDMG_RIGHT_ARM);
	}
	else if (bIsLeftLegDstroyed && bIsRightLegDstroyed)
	{
		SetLimbDamage( AVP_Alien_AIAnimStateMachine::LIMBDMG_BOTH_LEGS);
	}
	else if (bIsLeftLegDstroyed)
	{
		SetLimbDamage( AVP_Alien_AIAnimStateMachine::LIMBDMG_LEFT_LEG);
	}
	else if (bIsRightLegDstroyed)
	{
		SetLimbDamage( AVP_Alien_AIAnimStateMachine::LIMBDMG_RIGHT_LEG);
	}

	AVP_Brain_Actor_Alien *pxBrain = GetAlienActorBrain();
	AVP_ServerEntity_Squad_Alien *pxSquad = pxBrain ? pxBrain->GetAlienSquad() : NULL;
	if (pxSquad)
	{
		pxSquad->UpdateHealthFraction(GetGuid(), GetHealth()/GetMaxHealth());
	}
}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ServerActor_Alien::HandleMessage
// Author       : Chris Wallace
// Date         : 8-Feb-2007
////////////////////////////////////////////////////////////////////////////////

bool AVP_ServerActor_Alien::HandleMessage(const Asura_Message* pxMessage)
{
	// What sort of message is it?
	switch (pxMessage->GetMsgID())
	{
		case AVP_MSGID_ALIENPLAYERCOMMANDS_COMPRESSED:
		{
			// Get raw compressed data
			const u_char* pucData = static_cast<const u_char*>( pxMessage->GetData() ); 
			u_int uLen = pxMessage->GetDataSize(); 
			
			// Uncompress the data
			if (pucData && uLen)
			{
				Asura_BitUnpacker xUnpacker( pucData, uLen ); 
				AVP_Message_AlienPlayerCommands xAlienPlayerCommands( xUnpacker ); 

				ProcessAlienPlayerCommands( &xAlienPlayerCommands, pxMessage->GetSentTime() ); 
			}

			return true; 
		}

		case AVP_MSGID_ALIENPLAYERCOMMANDS:
		{
			const AVP_Message_AlienPlayerCommands*	pxData	= static_cast<const AVP_Message_AlienPlayerCommands*>(pxMessage->GetData());

			if (pxData != NULL)
			{
				ProcessAlienPlayerCommands( pxData, pxMessage->GetSentTime() );
			}

			return(true);
		}

		case AVP_MSGID_REQUEST_INTERACTION:
		{
			return true;
#if 0
			const AVP_Message_Actor_RequestInteraction*	pxData		= static_cast<const AVP_Message_Actor_RequestInteraction*>(pxMessage->GetData());
			const Asura_Guid							uGuid		= pxData->GetGuid();
			const Asura_ServerEntity_PhysicalObject*	pxServer	= Asura_ServerEntity_PhysicalObject::FindEntity(uGuid);

			if(pxServer == NULL)
			{
				Asura_Assert(false, "Cannot find entity to interact with.");
				return(true);
			}
			else
			{
				const u_int	uClassification	= pxServer->GetClassification();

				switch(uClassification)
				{
					case AVP_ENTITYCLASS_PREDATOR:
					{
						// pass messages from this entity class on towards aliens brain
						if ( m_pxBrain )
						{
							m_pxBrain->DoHandleMessage( pxMessage );
						}
					
						return(true);
					}
				
					default:
					{
						Asura_Assert(false, "Unexpected entity type for interaction.");
						return(true);
					}
				}		// End of switch.
			}
#endif
		}

		default:
		{
			return(PARENT::HandleMessage(pxMessage));
		}
	}
}

/// End of function 'AVP_ServerActor_Alien::HandleMessage'. ///

void AVP_ServerActor_Alien::MakeClientUpdateMessage(void)
{
	Asura_Message* pxMessage = Asura_Network::CreateMessage(ASURA_MSG_CLIENTUPDATE, sizeof(AVP_Message_AlienClientUpdate), GetClassification());
	if(pxMessage != NULL)
	{
		AVP_Message_AlienClientUpdate* pxData = static_cast<AVP_Message_AlienClientUpdate*>(pxMessage->GetData());
		pxData->FillAlien(*this);
		SendClientUpdateMessage(pxMessage);
	}
}

void AVP_ServerActor_Alien::MakePlayer(Asura_Guid uPlayerGuid)
{
	if (uPlayerGuid == GetPlayerInfoGuid()) { return; }

	PARENT::MakePlayer(uPlayerGuid);

	if(m_pxPhysicsObjectHandler != NULL)
	{
		m_pxPhysicsObjectHandler->EnableSurfaceReorientation();
	}
}

/// End of function 'AVP_ServerActor_Alien::MakePlayer'. ///

void AVP_ServerActor_Alien::MakeAI(void)
{
	PARENT::MakeAI();

	if(m_pxPhysicsObjectHandler != NULL)
	{
		m_pxPhysicsObjectHandler->EnableSurfaceReorientation();
	}

	if ( m_pxAIAnimStateMachine == NULL )
	{
		m_pxAIAnimStateMachine = new AVP_Alien_AIAnimStateMachine( *this );
	}
}

void AVP_ServerActor_Alien::PostLoadInitialise( void )
{
	if ( !IsPlayer() && !IsADeadBody() && !IsDead() )
	{
		if ( m_pxAIAnimStateMachine == NULL )
		{
			m_pxAIAnimStateMachine = new AVP_Alien_AIAnimStateMachine( *this );
		}
		m_pxAIAnimStateMachine->Initialise();
		m_fHealthToEnterInjuredFraction = Asura_Random::GetFloat(0.0f, 100.0f) > GetAlienActorBrain()->GetAlienSquad()->GetChanceToGetInjoredPercentage() ? 0.0f : GetAlienActorBrain()->GetAlienSquad()->GetInjurePrecentageOfHealth() / 100.0f;
	}

	PARENT::PostLoadInitialise();
}

AVP_Brain_Actor_Alien* AVP_ServerActor_Alien::GetAlienActorBrain() const
{
	return static_cast<AVP_Brain_Actor_Alien*>( GetActorBrain() );
}

void AVP_ServerActor_Alien::ProcessAlienPlayerCommands( const AVP_Message_AlienPlayerCommands* pxData, float fMessageSentTime )
{
	ProcessVertebratePlayerCommands( pxData, fMessageSentTime );

	SetStanding( pxData->IsStanding() );

	if (pxData->IsRequestingFacehuggerPetChange())
	{
		RequestFacehuggerPetChange();
	}

	if (pxData->IsRequestingDistraction())
	{
		CreateAIDistraction();
	}

	if (pxData->IsRequestingTailCharge())
	{
		// If we're not currently handling any attacks, create a request and handle it locally.
		if (m_pxMeleeSystem && m_pxMeleeSystem->GetCanStartNewAttack() && (m_pxMeleeSystem->GetCurrentAttackType() != AVP_ALIEN_MELEEATTACK_TYPE_SECONDARYATTACK))
		{
			Asura_Message* pxMessage = Asura_Network::CreateMessage(AVP_MSGID_REQUEST_MELEE, sizeof(AVP_Message_Vertebrate_RequestMelee));

			// Tell the server we want to interact...
			if(pxMessage != NULL)
			{
				AVP_Message_Vertebrate_RequestMelee* pxMeleeData = static_cast<AVP_Message_Vertebrate_RequestMelee*>(pxMessage->GetData());

				// Fill-in the message...
				pxMeleeData->SetType(AVP_ALIEN_MELEEATTACK_TYPE_SECONDARYATTACK);
				pxMeleeData->SetTargetGuid(pxData->GetCurrentMeleeTarget());
			}

			m_pxMeleeSystem->HandleMessage(pxMessage);
			Asura_Network::DestroyMessage(pxMessage);
		}
	}
	else
	{
		// If we're no longer requesting a charge, but we're still in the loop move, request an end.
		const AVP_ClientActor_Alien* pxClient = AVP_ClientActor_Alien::FindEntity(Guid);
		if (pxClient && pxClient->GetMovementSystem())
		{
			const Asura_Move* pxMove = pxClient->GetMovementSystem()->GetCurrentMove();
			if (pxMove && (pxMove->GetHash() == AVP_Alien_Move_Melee_TailCharge_Loop::GetStaticHash()))
			{
				Asura_Message* pxMessage = Asura_Network::CreateMessage(AVP_MSGID_REQUEST_MELEE, sizeof(AVP_Message_Vertebrate_RequestMelee));

				// Tell the server we want to interact...
				if(pxMessage != NULL)
				{
					AVP_Message_Vertebrate_RequestMelee* pxMeleeData = static_cast<AVP_Message_Vertebrate_RequestMelee*>(pxMessage->GetData());

					// Fill-in the message...
					pxMeleeData->SetType(AVP_ALIEN_MELEEATTACK_TYPE_SECONDARYCANCEL);
					pxMeleeData->SetTargetGuid(ASURA_GUID_UNREGISTERED);
				}

				m_pxMeleeSystem->HandleMessage(pxMessage);
				Asura_Network::DestroyMessage(pxMessage);
			}
		}
	}
}

const Asura_Blueprint* AVP_ServerActor_Alien::GetCurrentFlinchBlueprint() const
{
	// The alien doesn't have weapon variations, so the flinches come from the main actor blueprint.
	return GetBlueprint();
}

Asura_Hash_ID AVP_ServerActor_Alien::GetMovesBlueprintType() const
{
	return uAVP_BLUEPRINT_TYPE_MOVESALIEN;
}


void AVP_ServerActor_Alien::RequestFacehuggerPetChange()
{
	Asura_Assert(IsPlayer(), "Only players should be able to spawn facehuggers.");
	if (!IsPlayer())
	{
		return;
	}

	if (m_uCurrentFacehuggerBuddy == ASURA_GUID_UNREGISTERED)
	{
		RequestSpawnFacehugger();
		RequestAlwaysAggressive(); // make him aggressive immediately
	}
	else
	{
		// Facehuggers now always in aggressive mode.
		// RequestChangePetHuggerMode();
	}
}

DEBUGVAR u_int dbg_uFaceHuggerPetGuid = 0x9bcf6508; //"FaceHuggerPet"

void AVP_ServerActor_Alien::RequestSpawnFacehugger()
{

	if (m_uCandidateFaceHuggerPetSpawnPoint == ASURA_GUID_UNREGISTERED)
	{
		return;
	}

	AVP_ServerEntity_SpawnPoint2* pxSpawnPoint = AVP_ServerEntity_SpawnPoint2::FindEntity(m_uCandidateFaceHuggerPetSpawnPoint);

	if (!pxSpawnPoint)
	{
		return;
	}

	AVP_SpawnParams xSpawnParams;
	xSpawnParams.m_uBPTypeHashID = uAVP_BLUEPRINT_TYPE_ACTORS;
	xSpawnParams.m_uBPHashID = dbg_uFaceHuggerPetGuid;
	xSpawnParams.m_bUpdateStats = false;

	Asura_Guid uNewBuddy = pxSpawnPoint->Spawn(ASURA_GUID_UNREGISTERED, &xSpawnParams);
	if (uNewBuddy == ASURA_GUID_UNREGISTERED)
	{
		return;
	}

	AVP_ServerActor_FaceHugger* pxNewBuddy = AVP_ServerActor_FaceHugger::FindEntity(uNewBuddy);
	Asura_Assert(pxNewBuddy, "New Pet Not Found");
	if (pxNewBuddy)
	{
		pxNewBuddy->SetMaster(GetGuid());
	}
	else
	{
		return;
	}

	m_uCurrentFacehuggerBuddy = uNewBuddy;

}


void AVP_ServerActor_Alien::RequestChangePetHuggerMode()
{

	AVP_ServerActor_FaceHugger* pxBuddy = AVP_ServerActor_FaceHugger::FindEntity(m_uCurrentFacehuggerBuddy);

	if (!pxBuddy || pxBuddy->IsDestroyed())
	{
		return;
	}

	pxBuddy->ChangeAttackMode();
}

void AVP_ServerActor_Alien::RequestAlwaysAggressive()
{

	AVP_ServerActor_FaceHugger* pxBuddy = AVP_ServerActor_FaceHugger::FindEntity(m_uCurrentFacehuggerBuddy);

	if (!pxBuddy || pxBuddy->IsDestroyed())
	{
		return;
	}

	pxBuddy->SetAlwaysAggressive(true);
}



AVP_Alien_AIAnimStateMachine* AVP_ServerActor_Alien::GetAlienAIAnimStateMachine() const
{
	return static_cast< AVP_Alien_AIAnimStateMachine* >( m_pxAIAnimStateMachine );
}

// Temporary way of detecting downed state - is likely to see substantial changes.
bool AVP_ServerActor_Alien::IsDowned() const
{
	// Allow limb damage to count as downed.
	if (GetLimbDamage() != AVP_Alien_AIAnimStateMachine::LIMBDMG_NONE)
	{
		return true;
	}

	// The original test, check the client entity anim flags.
	const AVP_ClientActor_Alien* pxClient = AVP_ClientActor_Alien::FindEntity(Guid);
	if (!pxClient) { return false; }

	u_int uFlags = pxClient->GetFlagsForAllAnimsLastFrame();
	// The only flag is the thrash flag, although that's unlikely to remain under that name (or possibly even exist) once the downed state is done properly.

	return (uFlags & AVP_ANIMFLAG_ALIEN_THRASH) != 0;
}

void AVP_ServerActor_Alien::StartAcidTrail()
{
	if (!m_bIsAcidTrailOn)
	{
		m_bIsAcidTrailOn = true;
		m_xLastAcidTrailLocation = GetPosition();
	}
}

void AVP_ServerActor_Alien::EndAcidTrail()
{
	m_bIsAcidTrailOn = false;
}


Asura_Hash_ID AVP_ServerActor_Alien::GetCurrentGrappleBlueprintID() const
{
	Asura_Hash_ID uGrappleSetParameter;
	if (IsDowned())
	{
		uGrappleSetParameter = uAVP_BLUEPRINT_PARAM_GRAPPLESET_DOWNED;
	}
	else if (IsStanding())
	{
		uGrappleSetParameter = uAVP_BLUEPRINT_PARAM_GRAPPLESET_STANDING;
	}
	else
	{
		uGrappleSetParameter = uAVP_BLUEPRINT_PARAM_GRAPPLESET_CROUCHED;
	}

	const Asura_Blueprint* pxBlueprint = GetBlueprint();
	if (!pxBlueprint) { return ASURA_HASH_ID_UNSET; }

	Asura_Value xValue;
	if (!pxBlueprint->GetParameterValueByHashID(uGrappleSetParameter, xValue)) { return ASURA_HASH_ID_UNSET; }

	return xValue.GetHashIDValue();
}


void AVP_ServerActor_Alien::SetStanding( const bool bSet )
{
	if ( m_bStanding != bSet )
	{
		m_bStanding = bSet;
		SetAIStateAnimsNeedToTween( dbg_fStandingStateChangeTweenTime );
		SetPlayNextAIStateAnimFromStart();
		if ( m_pxAIAnimStateMachine )
		{
			m_pxAIAnimStateMachine->SetNeedToTween( dbg_fStandingStateChangeTweenTime );
			m_pxAIAnimStateMachine->SetPlayNextAnimFromStart();
		}
	}
}

void AVP_ServerActor_Alien::SetLimbDamage( const AVP_Alien_AIAnimStateMachine::LimbDamage eLimbDmg )
{
	if ( m_eLimbDamage != eLimbDmg )
	{
		m_eLimbDamage = eLimbDmg;
		SetAIStateAnimsNeedToTween( dbg_fLimbDamageStateChangeTweenTime );
		SetPlayNextAIStateAnimFromStart();
		if ( m_pxAIAnimStateMachine )
		{
			m_pxAIAnimStateMachine->SetNeedToTween( dbg_fLimbDamageStateChangeTweenTime  );
			m_pxAIAnimStateMachine->SetPlayNextAnimFromStart();
		}
	}
}

DEBUGVAR float dbg_fDistanceBetweenAcidDropsSquared = 0.5f * 0.5f;

void AVP_ServerActor_Alien::DropAcidTrail()
{
	if ((GetPosition() - m_xLastAcidTrailLocation).MagnitudeSquared() < dbg_fDistanceBetweenAcidDropsSquared )
	{
		return;
	}

	m_xLastAcidTrailLocation = GetPosition();
	const Asura_Vector_3 xDirection(0.0f, 1.0f, 0.0f);
	const float fRange = 5.0f;

	CheckForEnvironmentalAcidDamage(GetPosition(),xDirection, fRange, true );
}

DEBUGVAR float dbg_fMaxRangeToSpawnPoint = 50.0f;

Asura_Guid AVP_ServerActor_Alien::FindFaceHuggerPetSpawnPoint()
{

	Asura_Assert(IsPlayer(), "Only players should be able to spawn facehuggers.");
	if (!IsPlayer())
	{
		return ASURA_GUID_UNREGISTERED;
	}

	const AVP_ServerEntity_PlayerInfo* pxPlayerInfo = AVP_ServerEntity_PlayerInfo::FindEntity(GetPlayerInfoGuid());
	if (!pxPlayerInfo)
	{
		return ASURA_GUID_UNREGISTERED;
	}

	const AVP_ServerEntity_GamePlayVolume* pxGPV = AVP_ServerEntity_GamePlayVolume::FindEntity(pxPlayerInfo->GetPlayerGamePlayVolumeGuid());
	if (!pxGPV)
	{
		return ASURA_GUID_UNREGISTERED;
	}

	float fScore;
	Asura_Guid uSpawnPoint = ASURA_GUID_UNREGISTERED;
	AVP_ScoreParams xScoreParams;
	xScoreParams.m_iSpawnScheme = AVP_SS_CLOSEST;
	xScoreParams.m_bTypesMustMach = true;
	xScoreParams.m_xPosition = GetPosition();
	xScoreParams.m_fSpawnDistance = dbg_fMaxRangeToSpawnPoint; // 
	xScoreParams.m_uBPTypeHashID = uAVP_BLUEPRINT_TYPE_ACTORS;
	xScoreParams.m_uBPHashID = dbg_uFaceHuggerPetGuid;
	pxGPV->ScoreSpawners(&xScoreParams, fScore, uSpawnPoint);

	if (fScore == 0.0f)
	{
		return ASURA_GUID_UNREGISTERED;
	}

	return uSpawnPoint;
}

void AVP_ServerActor_Alien::HandleInteractions()
{
	AVP_ClientActor_Alien*		  pxClient          =  AVP_ClientActor_Alien::FindEntity(GetGuid());
	const Asura_Movement_System*  pxMovementSystem	= (pxClient == NULL) ? NULL : pxClient->GetMovementSystem();
	const AVP_Move*				  pxCurrentMove		= static_cast< const AVP_Move* >( (pxMovementSystem == NULL) ? NULL : pxMovementSystem->GetCurrentMove() );

	AVP_Actor_InteractionHandler* pxInteract = (pxClient == NULL) ? NULL : pxClient->GetInteractionHandler();
	if(pxInteract)
	{
		if(pxCurrentMove)
		{
			// Check if the move is an interaction one for the battery
			if(pxCurrentMove->GetHash() == AVP_Alien_Move_Interaction_DestroyBattery::GetStaticHash())
			{
				// Destroy the battery, do a mesh swap for now.
				DestroyBattery();
			}
			else if( pxCurrentMove->GetHash() == AVP_Move_Interaction_BeginWeaponAway::GetStaticHash() )
			{
				// !!! CAH !!! this will not work but not hinder anything. 
				// because the alien does not use the weapon away like the other species this is effectively defunct code. 
				// but with E3 issues I cant change this now. 
				// the functionality here needs to be added to AVP_Alien_InteractionHandler::HandleInteraction() instead, 
				// and we'll probably need a new move as well for the alien. 


				// find if the object being interacted with is an invisiswitch
				const AVP_Actor_InteractionHandler*  pxInteractionHandler = pxClient->GetInteractionHandler();
				if (pxInteractionHandler)
				{
					//Get GUID of interaction
					const Asura_Guid uGuid = pxInteractionHandler->GetGuidForInteraction( AVP_INTERACTION_TYPE_DEFAULT );
					if( uGuid != ASURA_GUID_UNREGISTERED )
					{
						AVP_ClientEntity_InteractInvisiSwitch* pxInvisiSwitch = AVP_ClientEntity_InteractInvisiSwitch::FindEntity(uGuid);
						if( pxInvisiSwitch )
						{
							Asura_Message* pxMessage = Asura_Network::CreateMessage( AVP_MSGID_SENTRYGUN_SWITCH_PLAYANIMATION, sizeof( AVP_Message_SentryGun_Switch_PlayAninmation ), GetClassification());
							if(pxMessage)
							{
								AVP_Message_SentryGun_Switch_PlayAninmation* pxData = static_cast< AVP_Message_SentryGun_Switch_PlayAninmation* >( pxMessage->GetData() );
								pxData->SetAnimHash( pxInvisiSwitch->GetAnimHashForSpeciesInteraction( AVP_Settings::AVP_SPECIES_ALIEN ) );

								pxMessage->SetToGuid( pxInvisiSwitch->GetGuid() );
								pxMessage->SetFromGuid( GetGuid() );
								Asura_Network::Send( pxMessage );
							}
						}
					}
				}
			}
		}
	}
}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ServerActor_Alien::DestroyBattery
// Author       : AJK
// Date         : 27-11-2008
////////////////////////////////////////////////////////////////////////////////
void AVP_ServerActor_Alien::DestroyBattery() const
{
	//// Get the client
	AVP_ClientActor_Alien* pxActor = AVP_ClientActor_Alien::FindEntity(GetGuid());
	if(pxActor)
	{
		const AVP_Actor_InteractionHandler*  pxInteractionHandler = pxActor->GetInteractionHandler();
		if (pxInteractionHandler)
		{
			//Get GUID of interaction
			const Asura_Guid uGuid = pxInteractionHandler->GetGuidForInteraction( AVP_INTERACTION_TYPE_DEFAULT );
			if(uGuid != ASURA_GUID_UNREGISTERED)
			{
				//Get server interaction battery
				AVP_ServerEntity_InteractBatteryReceptacle* pxBatteryReceptacle = AVP_ServerEntity_InteractBatteryReceptacle::FindEntity(uGuid);
				if(pxBatteryReceptacle)
				{
					// remove battery from case
					pxBatteryReceptacle->DestroyBattery();
				}
			}
		}
	}
}


bool AVP_ServerActor_Alien::AwardLoadsOfDamageToAllAliens( bool bHelp )
{
	if (bHelp) return true;
	for (Asura_Hashed_List_It<AVP_ServerActor_Alien> xIt(GetList()) ; !xIt.Done() ; xIt.Next())
	{
		xIt.GetCurrent()->AwardLoadsOfDamage();
	}
	return true;
}



void AVP_ServerActor_Alien::GetAcidVolumePosition(Asura_Vector_3* xPos)
{
	AVP_ClientActor_Alien* pxClient = AVP_ClientActor_Alien::FindEntity(GetGuid());
	static const Asura_Hash_ID uHeadTargetMarkerHashID = 0x002fbc62; //"eyes"

	if (pxClient)
	{
		AVP_Vertebrate_AnimationHandler * pxAnimHandler = static_cast <AVP_Vertebrate_AnimationHandler*>(pxClient->GetAnimationHandler());
		Asura_Ragdoll_Wrapper* pxRagdollWrapper = Asura_Ragdoll_Wrapper::GetRagdollWrapper(pxAnimHandler->GetRagdollHandle() );
		if ( pxRagdollWrapper )
		{
			Asura_Physics_Ragdoll* pxRagdoll = pxRagdollWrapper->GetPhysicsRagdollPointer();
			if(pxRagdoll)
			{
				Asura_Animation* pxAnim = pxRagdoll->GetAnimation();
				if (pxAnim->GetMarkerLocationFromHash(uHeadTargetMarkerHashID, xPos, NULL, true)) return; // Found done all good.
				Asura_Assert(false, "couldnt find eyes in ragdoll");
			}
		}
	} 
	//Asura_Assert(false, "dropping back to standard position");
	GetHeadPosition(*xPos);
}
void AVP_ServerActor_Alien::DestroyAcidVolume()
{
	if (m_pxAcidVolumeGroup)
	{
		delete m_pxAcidVolumeGroup;
		m_pxAcidVolumeGroup = NULL;
	}
}


void AVP_ServerActor_Alien::CreateAcidVolume()
{
	if (!m_pxAcidVolumeGroup)
	{
		Asura_Blueprint* pxBlueprint = GetBlueprint();
		Asura_Value xValue;

		if (!pxBlueprint)
		{
			Asura_Assert(false, "No blueprint, no acid volume");
			return;
		}

		m_pxAcidVolumeGroup = new Asura_VolumeGroup();
		Asura_Volume_Sphere* pxSphere = new Asura_Volume_Sphere();
		GetAcidVolumePosition(pxSphere->GetPositionPtr());
		*pxSphere->GetRadiusPtr() = s_fHeadsplitAcidDamageRadius;
		m_pxAcidVolumeGroup->AddVolume(pxSphere);

		if(pxBlueprint->GetParameterValueByHashID(uAVP_BLUEPRINT_PARAM_HEADSPLIT_INITIALDELAY, xValue))
		{
			s_fHeadsplitInitialAcidDamageDelay = xValue.GetFloatValue();
		} 
		else
		{
			Asura_Assert(false, "Warning head split initial delay not found in alien blueprint");
		}

		if(pxBlueprint->GetParameterValueByHashID(uAVP_BLUEPRINT_PARAM_HEADSPLIT_DAMAGEINTERVAL, xValue))
		{
			s_fHeadsplitTimeBetweenAcidDamages = xValue.GetFloatValue();
		} 
		else
		{
			Asura_Assert(false, "Warning head split time between acid damages not found in alien blueprint");
		}

		if(pxBlueprint->GetParameterValueByHashID(uAVP_BLUEPRINT_PARAM_HEADSPLIT_NUMDAMAGES, xValue))
		{
			s_uHeadsplitNumberOfAcidDamages = static_cast<u_int>(xValue.GetIntValue());
		}
		else
		{
			Asura_Assert(false, "Warning head split number of acid damages not found in alien blueprint");
		}


		if(pxBlueprint->GetParameterValueByHashID(uAVP_BLUEPRINT_PARAM_HEADSPLIT_RADIUS, xValue))
		{
			s_fHeadsplitAcidDamageRadius = xValue.GetFloatValue();
		}
		else
		{
			Asura_Assert(false, "Warning head split raidus of acid damages not found in alien blueprint");
		}

		if(pxBlueprint->GetParameterValueByHashID(uAVP_BLUEPRINT_PARAM_HEADSPLIT_DAMAGE, xValue))
		{
			s_fHeadsplitAcidDamageAmount = xValue.GetFloatValue();
		}
		else
		{
			Asura_Assert(false, "Warning head split damage of acid damages not found in alien blueprint");
		}

		m_fTimeTillNextAcidDamage = s_fHeadsplitInitialAcidDamageDelay;
		m_uAcidBurstsLeft = s_uHeadsplitNumberOfAcidDamages;
	}

}

bool AVP_ServerActor_Alien::IsInShadows() const
{
	return GetAlienActorBrain() ? !GetAlienActorBrain()->GetInLightForMarine() : true;
}

bool AVP_ServerActor_Alien::IsInMarineTorch() const
{ 
	return GetAlienActorBrain() ? GetAlienActorBrain()->GetInMarineLight() == AVP_Brain_Actor_Alien::MARINE_LIGHT_YES : false;
}

void AVP_ServerActor_Alien::UpdateAcidVolume()
{
	if (!m_pxAcidVolumeGroup) return;
	m_fTimeTillNextAcidDamage -= Asura_Timers::GetGameTimeSlice();

	if (AVP_DamagePFX::s_bDebugRender)
	{
		Asura_Volume_Sphere* pxSphere = static_cast<Asura_Volume_Sphere*>(m_pxAcidVolumeGroup->GetVolume(0));
		GetAcidVolumePosition(pxSphere->GetPositionPtr());
		Asura_Primitive_Render::UseDefaultRenderParams();
		Asura_Primitive_Render::DrawSphereWorld(*pxSphere->GetPositionPtr(), *(pxSphere->GetRadiusPtr()), 0xFFFFFFFF, 64, 64);
	}

	if (m_fTimeTillNextAcidDamage <= 0.f)
	{
		Asura_Volume_Sphere* pxSphere = static_cast<Asura_Volume_Sphere*>(m_pxAcidVolumeGroup->GetVolume(0));
		GetAcidVolumePosition(pxSphere->GetPositionPtr()); // sets.

		m_fTimeTillNextAcidDamage = s_fHeadsplitTimeBetweenAcidDamages;
		m_uAcidBurstsLeft--;
		Asura_Vector_3 xPosition = *pxSphere->GetPositionPtr();
		float fRadius = *pxSphere->GetRadiusPtr();

		for (Asura_Hashed_List_It<AVP_ClientEntity_Actor> xIt(AVP_ClientEntity_Actor::GetList()) ; !xIt.Done() ; xIt.Next())
		{
			AVP_ClientEntity_Actor *pxVictim = xIt.GetCurrent();
			// Dont test Aiens for speed only (should reject ourself as well thought this).
			if (pxVictim->GetClassification()!= AVP_ENTITYCLASS_ALIEN)
			{
				if (pxVictim->CoarseIntersectsSphere(xPosition ,fRadius))
				{
					if (pxVictim->IntersectsSphere(xPosition ,fRadius))
					{
						const AVP_EntityDamage_RequestVictim xDamage( uAVP_BLUEPRINT_NAME_DAMAGE_ACID, GetGuid(), ASURA_HASH_ID_UNSET,
															pxVictim->GetGuid(), s_fHeadsplitAcidDamageAmount, pxVictim->GetCurrentPosition(), 
															Asura_Maths::ZeroVector3, Asura_Maths::ZeroVector3);
						AVP_ServerEntity_Actor *pxServerVictim = AVP_ServerEntity_Actor::FindEntity( pxVictim->GetGuid());
						if (pxServerVictim) pxServerVictim->TakeDamageExtended( xDamage, false );
					}
				}
			}
		}
	}

	if (!m_uAcidBurstsLeft)
	{
		DestroyAcidVolume();
	}


}

/// End of file. ///
