
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_ServerActor_Alien.h
// Author : Andy Tate
// Date   : 3-Oct-2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __AVP_SERVERACTOR_ALIEN_H__
#define __AVP_SERVERACTOR_ALIEN_H__

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura_HashedList.h"

#include "Actor/Vertebrates/Alien/AVP_Alien_AIAnimStateMachine.h"
#include "Actor/Vertebrates/Alien/AVP_Alien_Description.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/AVP_ServerActor_Vertebrate.h"


////////////////////////////////////////////////////////////////////////////////
// Forward class declarations...
////////////////////////////////////////////////////////////////////////////////

class AVP_Brain_Actor_Alien;
class AVP_Message_AlienPlayerCommands;
class Asura_VolumeGroup;


////////////////////////////////////////////////////////////////////////////////
// Class declaration...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_ServerActor_Alien
// Author       : Andy Tate
// Date         : 3-Oct-2006
//
// Description  : This class handles the operation of the Alien actor, the
//				  Alien can be either player controlled or AI controlled.
////////////////////////////////////////////////////////////////////////////////

class AVP_ServerActor_Alien : public AVP_ServerActor_Vertebrate
{
public:
	// Overridden functions...
	virtual	bool	HandleMessage(const Asura_Message* pxMessage);
	virtual	void	ReadFromChunkStream(Asura_Chunk_Stream& xStream);
	virtual	void	WriteToChunkStream(Asura_Chunk_Stream& xStream) const;
	virtual void	Update(void);
	virtual	void	TakeDamageExtended(const AVP_EntityDamage_RequestVictim& xDamage, const bool bForceFatal, const bool bSendMessageToClient=true );

    virtual Axon_Brain* CreateBrain();

	bool			IsStanding() const { return m_bStanding; }
	void			SetStanding( const bool bSet );
	bool			IsFaceHuggerSummoningPointAvailable() const { return m_uCandidateFaceHuggerPetSpawnPoint != ASURA_GUID_UNREGISTERED; }
	u_int			GetFaceHuggerPetStatus() const;

	AVP_Alien_AIAnimStateMachine::LimbDamage GetLimbDamage() const { return m_eLimbDamage; }
	void	SetLimbDamage( const AVP_Alien_AIAnimStateMachine::LimbDamage eLimbDmg );

	// Legacy chunk handling, to be removed as soon as possible...
	virtual	bool	IsFilledFromChunkBuffer(void) const	{ return(false); }

	// Player functions...
	virtual void	MakePlayer(Asura_Guid uPlayerGuid);
	virtual void	MakeAI(void);
	virtual void	PostLoadInitialise(void);

    static	void					Create(Asura_Guid uGUID);
	static  AVP_ServerActor_Alien*	CreateAndReturn(Asura_Guid uGUID);

	bool GetIsASpawnedAlien(void)		{return m_bIsASpawnedAlien;}
	void SetIsASpawnedAlien(bool bVal)	{m_bIsASpawnedAlien = bVal;}

	bool IsInShadow() const { return m_bIsInShadow; }

	virtual bool GetHasCreateMessage() const { return !IsDead() || IsADeadBody(); }

	// Static list accessors...
	static	AVP_ServerActor_Alien*							FindEntity(const Asura_Guid uGuid)	{ return(s_xList.FindEntity(uGuid)); }
	static	const Asura_Hashed_List<AVP_ServerActor_Alien>*	GetList(void)						{ return(&s_xList); }

	virtual u_int	GetElementFromBoneID(int iBoneID) const { return AVP_Alien_Description::GetElementFromBoneID(iBoneID); }
	virtual int		GetAnyBoneIDFromElement(u_int uElement) const { return AVP_Alien_Description::GetAnyBoneIDFromElement(uElement); } // Since bones->elements is a many->one mapping, this function will return one of the bones associated with the given element, but not necessarily the only one.

	AVP_Brain_Actor_Alien* GetAlienActorBrain() const;
	AVP_Alien_AIAnimStateMachine* GetAlienAIAnimStateMachine() const;

	virtual bool IsDowned() const;
	void StartAcidTrail();
	void EndAcidTrail();

	virtual Asura_Hash_ID GetCurrentGrappleBlueprintID() const;

	virtual void DestroyAcidVolume();
	void CreateAcidVolume();
	
	virtual bool ExplodesOnFatalPlasmaDamage() const { return true ;}

	// hide in shadows queries
	bool IsInShadows() const;
	bool IsInMarineTorch() const;

	// Static functions
	static bool AwardLoadsOfDamageToAllAliens( bool bHelp );

protected:
	void	ProcessAlienPlayerCommands( const AVP_Message_AlienPlayerCommands* pxData, float fMessageSentTime );

	virtual	void	MakeClientUpdateMessage(void);

	// Overridden event functions...
	virtual void	OnApplyDamage(const AVP_EntityDamage_ApplyToEntity& xDamage);
	virtual void	OnDeath(const AVP_EntityDamage_ApplyToEntity& xDamage);

	// force alian legs loss if any limb is damaged.
	virtual float CalculateAndUpdateLocationSpecificDamage(const AVP_EntityDamage_RequestVictim& xDamage, u_int& uDestroyedSkinElements, u_int& uGibbedSkinElements, bool bForceFatal);

	void EffectMorale(Asura_Guid uGuidToEffect);

	void UpdateAndRenderAIThreatStatus(void);
	void UpdateFaceHuggerPetState();

	virtual bool GetIsInLowStance() const { return !IsStanding(); }
	void	CheckForEnvironmentalAcidDamage(const Asura_Vector_3& xStartPosition, const Asura_Vector_3& xDirection, const float fRange, bool bIsTrailMode);

	virtual const Asura_Blueprint* GetCurrentFlinchBlueprint() const;

	virtual float GetHeadPositionYOffset() const { return -1.5f; }
	virtual float GetChestPositionYOffset() const { return -1.0f; }

	virtual Asura_Hash_ID GetMovesBlueprintType() const;

	void RequestFacehuggerPetChange();
	void RequestSpawnFacehugger();
	void RequestChangePetHuggerMode();
	void RequestAlwaysAggressive();

	void UpdateAcidVolume();
	void GetAcidVolumePosition(Asura_Vector_3* xPos);

	AVP_ServerActor_Alien(const Asura_Guid uGUID);
	virtual ~AVP_ServerActor_Alien();

	bool					m_bIsASpawnedAlien;
	bool					m_bStanding;

	AVP_Alien_AIAnimStateMachine::LimbDamage m_eLimbDamage;

	float m_fHealthToEnterInjuredFraction;

private:
	typedef AVP_ServerActor_Vertebrate PARENT;

	void DropAcidTrail();
	Asura_Guid FindFaceHuggerPetSpawnPoint();
	void	   DestroyBattery() const;

	virtual void HandleInteractions();

#if defined(ASURA_TOOLS)
	friend class ToolsAVP_Entity_Alien;
#endif

	bool m_bIsAcidTrailOn;
	Asura_Vector_3 m_xLastAcidTrailLocation;

	Asura_Guid m_uCurrentFacehuggerBuddy;
	Asura_Guid m_uCandidateFaceHuggerPetSpawnPoint;
	Asura_Guid m_uFaceHuggerPetStatus;

	bool m_bIsInShadow;

	// for explosion of the alien when it is burning with fire.
	float			m_fDieWhenBurning_RadialDamageAtFarRange;
	float			m_fDieWhenBurning_RadialDamageAtNearRange;
	float			m_fDieWhenBurning_RadialFarRange;

	Asura_VolumeGroup* m_pxAcidVolumeGroup;
	float			   m_fTimeTillNextAcidDamage;
	u_int			   m_uAcidBurstsLeft;

	static float s_fHeadsplitInitialAcidDamageDelay;
	static float s_fHeadsplitTimeBetweenAcidDamages;
	static u_int s_uHeadsplitNumberOfAcidDamages;
	static float s_fHeadsplitAcidDamageRadius;
	static float s_fHeadsplitAcidDamageAmount;


    static	Asura_Hashed_List<AVP_ServerActor_Alien>	s_xList;
	
	AVP_ServerActor_Alien(void);
	AVP_ServerActor_Alien(const AVP_ServerActor_Alien& xOther);
	AVP_ServerActor_Alien&	operator=(const AVP_ServerActor_Alien& xOther);

    enum { CURRENT_CHUNK_VERSION = 2 };
};

#endif   // ifndef __AVP_SERVERACTOR_PREDATOR_H__

/// End of file. ///
