
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_ServerActor_FaceHugger.cpp
// Author : Andy Tate
// Date   : 3-Oct-2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "Entity/PhysicalObject/Actor/Vertebrates/FaceHugger/AVP_ServerActor_FaceHugger.h"

// Asura independent header-files...
#include "Camera/Asura_Camera.h"
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "Core/Asura_Timers.h"
#include "Physics/Asura_Physics.h"

// Asura platform header-files...

// Game independent header-files...
#include "Actor/AVP_Actor_InputHandler.h"
#include "Actor/AVP_Actor_PhysicsObjectHandler.h"
#include "Actor/Vertebrates/FaceHugger/AVP_FaceHugger_AIAnimStateMachine.h"
#include "Actor/Vertebrates/FaceHugger/AVP_FaceHugger_Description.h"
#include "Actor/Vertebrates/FaceHugger/AVP_FaceHugger_MeleeCounter_System.h"
#include "Entity/AVP_Entity_Classification.h"
#include "Entity/AVP_Messages.h"
#include "Entity/Damage/AVP_EntityDamage_Request.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/FaceHugger/AVP_ClientActor_FaceHugger.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/FaceHugger/AVP_Messages_FaceHugger.h"
#include "Entity/PhysicalObject/Moves/Asura_Movement_System.h"
#include "Mesh/AVP_PolygonFlags.h"
#include "Moves/AVP_Movement_System.h"
#include "Moves/Vertebrates/FaceHugger/AVP_FaceHugger_MovesMelee.h"

// axon
#include "Core/BrainTypes/AVP_Brain_Actor_FaceHugger.h"

////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
//
// AVP_ServerActor_FaceHugger
//
//////////////////////////////////////////////////////////////////////////////

Asura_Hashed_List<AVP_ServerActor_FaceHugger>	AVP_ServerActor_FaceHugger::s_xList;

AVP_ServerActor_FaceHugger::AVP_ServerActor_FaceHugger(const Asura_Guid uGUID)
: Asura_Entity_Server(uGUID)
, PARENT(AVP_PolygonFlag_IgnoreForFaceHuggerCollision, uGUID)
, m_uAttachedToGuid(ASURA_GUID_UNREGISTERED)
{
	// Update the data...
	Classification	= AVP_ENTITYCLASS_ALIENFACEHUGGER;

	if(m_pxPhysicsObjectHandler != NULL)
	{
		m_pxPhysicsObjectHandler->EnableSurfaceReorientation();
	}

	m_pxMeleeSystem = new AVP_FaceHugger_MeleeCounter_System(*this);

	// Don't forget to store us in the list...
	s_xList.AddToList(this);
}


AVP_ServerActor_FaceHugger::~AVP_ServerActor_FaceHugger()
{
	// Don't forget to remove us from the list...
	s_xList.RemoveFromList(this);
}

void AVP_ServerActor_FaceHugger::ReadFromChunkStream(Asura_Chunk_Stream& xStream)
{
	u_int	uVersion;

	// Read the version...
	xStream >> uVersion;
	Asura_Assert((uVersion <= uCURRENT_CHUNK_VERSION), "Unsupported chunk version");

	if (uVersion > 0)
	{
		// Read the parent data...
		PARENT::ReadFromChunkStream(xStream);
	}
	else
	{
		// The vertibrate didn't have chunk data orignally, so direct subclass chunk versions have been bumped to work around it.
		AVP_ServerEntity_Actor::ReadFromChunkStream(xStream);
	}
}

void AVP_ServerActor_FaceHugger::WriteToChunkStream(Asura_Chunk_Stream& xStream) const
{
	// Write the version...
	xStream << uCURRENT_CHUNK_VERSION;

	// Write the parent data...
	PARENT::WriteToChunkStream(xStream);
}

Axon_Brain* AVP_ServerActor_FaceHugger::CreateBrain()
{
	return new AVP_Brain_Actor_FaceHugger( this );
}

bool AVP_ServerActor_FaceHugger::HandleMessage(const Asura_Message* pxMessage)
{
	switch (pxMessage->GetMsgID())
	{
	case ASURA_MSG_MOVE_FINISHED: // end of jump
		{
	 		const Asura_Msg_Move* pxData = static_cast<const Asura_Msg_Move*>(pxMessage->GetData());
			Asura_Assert(pxData, "MoveComplete message with no data");
			if(pxData != NULL)
			{
				if ( pxData->GetMoveID() == AVP_FaceHugger_TP_Move_Melee_MarineGrapple::GetStaticHash() || 
					 pxData->GetMoveID() == AVP_FaceHugger_TP_Move_Melee_PredatorGrapple::GetStaticHash())
				{
 					AVP_Brain_Actor_FaceHugger* pxBrain  = static_cast<AVP_Brain_Actor_FaceHugger*>(GetActorBrain());
 					pxBrain->AttachToTarget();
					pxBrain->Shutdown();
					PARENT::HandleMessage(pxMessage);
					return true;
				}
			}

		}
		break;
	case AVP_MSGID_MOVESTART_SERVER:
	case ASURA_MSG_MOVE_START:
		{
			const Asura_Msg_Move* pxData = static_cast<const Asura_Msg_Move*>(pxMessage->GetData());
			Asura_Assert(pxData, "MoveStart message with no data");
			if(pxData != NULL)
			{
				if (pxData->GetMoveID() == AVP_FaceHugger_TP_Move_Melee_PredatorGrapple::GetStaticHash() || 
					pxData->GetMoveID() == AVP_FaceHugger_TP_Move_Melee_MarineGrapple::GetStaticHash())
				{
					AVP_Brain_Actor_FaceHugger* pxBrain  = static_cast<AVP_Brain_Actor_FaceHugger*>(GetActorBrain());
					m_uAttachedToGuid = pxBrain->GetTargetGuid();
					PARENT::HandleMessage(pxMessage);
					return true;
				}
			}
		}
		break;
	case AVP_MSGID_FACEHUGGER_HUG_ENTITY:
		{
			const AVP_FaceHugger_Hug_Entity* pxData = static_cast<const AVP_FaceHugger_Hug_Entity*>(pxMessage->GetData());
			Asura_Assert(pxData, "Hug Entity message with no data");
			if(pxData != NULL)
			{
				const AVP_ServerEntity_Actor* pxTarget = AVP_ServerEntity_Actor::FindEntity(pxData->m_uTarget);
				if (pxTarget)
				{

					Asura_Message* pxMessage = Asura_Network::CreateMessage( ASURA_MSG_MOVE_START, sizeof( AVP_Msg_Move ) );
					if ( pxMessage != NULL )
					{
						AVP_Msg_Move* pxData = static_cast< AVP_Msg_Move* >( pxMessage->GetData() );

						pxMessage->SetFromGuid( GetGuid() );
						pxMessage->SetToGuid( GetGuid() );
						pxData->ClearFlags();
						pxData->SetIgnoredByOwner( false );
						pxData->SetMoveID( AVP_FaceHugger_TP_Move_Melee_MarineDeathHug::GetStaticHash() );
						Asura_Network::Send( pxMessage );
					}

					LinkToParent(pxData->m_uTarget, 0x002fbc62 /* eyes */, false );
					if (GetActorBrain())
					{
						GetActorBrain()->Shutdown();
					}
				}
				else
				{
					Asura_Assert(false, "Entity to hug not found");
					SetThisAndClientsAsDestroyed();
				}
			}
			return true;
		}

    default:
		break;
	}

	return PARENT::HandleMessage( pxMessage );
}

void AVP_ServerActor_FaceHugger::Update()
{
	if (m_uAttachedToGuid != ASURA_GUID_UNREGISTERED)
	{
		AVP_ServerActor_Vertebrate* pxParent = AVP_ServerActor_Vertebrate::FindEntity(m_uAttachedToGuid);
		if (!pxParent || pxParent->IsDestroyed())
		{
			SetThisAndClientsAsDestroyed();
		}
		else if (!pxParent->IsSkinElementAvailable(AVP_SET_HEAD))
		{
			// kill facehugger and turn it into rugdoll
			AVP_EntityDamage_RequestVictim xDamage( uAVP_BLUEPRINT_NAME_DAMAGE_MELEE,
				GetGuid(),
				ASURA_HASH_ID_UNSET,
				GetGuid(),
				1000000.0f,
				GetPosition(),
				Asura_Vector_3( 0.0f, 1.0f, 0.0f ),
				Asura_Vector_3( 0.0f, 1.0f, 0.0f ) );
			TakeDamageExtended( xDamage, true );
		}
	}

	PARENT::Update();
}

void AVP_ServerActor_FaceHugger::SetMaster(Asura_Guid uMasterGuid)
{
	GetFaceHuggerBrain()->SetMaster(uMasterGuid);
}

void AVP_ServerActor_FaceHugger::ChangeAttackMode()
{
	GetFaceHuggerBrain()->ChangeAttackMode();
}

void AVP_ServerActor_FaceHugger::SetAlwaysAggressive( bool bAlwaysAggressive )
{
	GetFaceHuggerBrain()->SetAlwaysAggressive( bAlwaysAggressive );
}

void AVP_ServerActor_FaceHugger::PostLoadInitialise()
{
	if ( !IsPlayer() && !IsADeadBody() && !IsDead() )
	{
		if ( m_pxAIAnimStateMachine == NULL )
		{
			m_pxAIAnimStateMachine = new AVP_FaceHugger_AIAnimStateMachine( *this );
		}
		m_pxAIAnimStateMachine->Initialise();
	}

	PARENT::PostLoadInitialise();
}

void AVP_ServerActor_FaceHugger::MakeAI()
{
	PARENT::MakeAI();
	if ( m_pxAIAnimStateMachine == NULL )
	{
		m_pxAIAnimStateMachine = new AVP_FaceHugger_AIAnimStateMachine( *this );
	}
}
