//-----------------------------------------------------------------------------
// File   : AVP_ServerActor_civilian.h
// Author : Gareth Thomas
// Date   : 12 / 02 / 2008
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
//-----------------------------------------------------------------------------

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE
#ifndef __AVP_SERVERACTOR_CIVILIAN_H__
#define __AVP_SERVERACTOR_CIVILIAN_H__

//-----------------------------------------------------------------------------
// Include files...
//-----------------------------------------------------------------------------

#include "Core/Asura_HashedList.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Human/AVP_ServerActor_Human.h"

//-----------------------------------------------------------------------------
// Forward class declarations...
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Class declaration...
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Class        : AVP_ServerActor_civilian
// Author       : Gareth Thomas
// Date         : 12 / 02 / 2008
//
// Description  : This class handles the operation of the Civilian actor, the
//				  Civilian is AI controlled.
//-----------------------------------------------------------------------------
class AVP_ServerActor_Civilian : public AVP_ServerActor_Human
{
#if defined(ASURA_TOOLS)
	// Friend classes...
	friend class ToolsAVP_Entity_Civilian;
#endif//defined(ASURA_TOOLS)

public:

	enum DialogueHashIDs
	{
		CIVILIAN_DLG_HUMAN_PASSIVE_TO_ALERT = 0,
		CIVILIAN_DLG_HUMAN_ENTERING_AGGRESSIVE,
		CIVILIAN_DLG_HUMAN_AGGRESSIVE_TO_ALERT,
		CIVILIAN_DLG_HUMAN_ALERT_TO_PASSIVE,
		CIVILIAN_DLG_HUMAN_SNEAK_ATTACK_RESPONSE,
		CIVILIAN_DLG_HUMAN_TEAM_MATE_KILLED,
		CIVILIAN_DLG_HUMAN_TAUNTING,
		CIVILIAN_DLG_HUMAN_ATTACKING,
		CIVILIAN_DLG_HUMAN_KILLED_THREAT,
		CIVILIAN_DLG_HUMAN_KILLED_THREAT_RESPONSE,
		CIVILIAN_DLG_HUMAN_RELOADING,
		CIVILIAN_DLG_HUMAN_FLEEING,
		CIVILIAN_DLG_HUMAN_IDLE_CHATTER,

		CIVILIAN_DLG_MAX_DIALOGUE_HASH_IDS
	};	

	enum CivilianType
	{
		CIVILIANTYPE_BEGIN = 0,
		CIVILIANTYPE_CONSTRUCTION_WORKER = CIVILIANTYPE_BEGIN,
		CIVILIANTYPE_SCIENTIST,
		CIVILIANTYPE_END
	};

	virtual	void ReadFromChunkStream(Asura_Chunk_Stream& xStream);
	virtual	void WriteToChunkStream(Asura_Chunk_Stream& xStream) const;
	virtual void WriteToClientCreateStream(Asura_Chunk_Memory_Stream& xStream) const;

	virtual Asura_Hash_ID	GetDialogueHashID(u_int uIndex)		{return m_uCivilianDialogueHashIDs[uIndex];}
	virtual void			InitialiseDialogueFromBlueprint		(Asura_Hash_ID uBlueprintHashID);

	virtual void PostLoadInitialise();
	virtual void MakeAI();

	virtual void Update();

	virtual bool GetHasCreateMessage() const { return !IsDead() || IsADeadBody(); }

	// Static Public functions...
	static void	Create(Asura_Guid uGUID);
	static AVP_ServerActor_Civilian* CreateAndReturn(Asura_Guid uGUID);

	// Static list accessors...
	static AVP_ServerActor_Civilian* FindEntity(const Asura_Guid uGuid)				{return s_xList.FindEntity(uGuid);}
	static const Asura_Hashed_List<AVP_ServerActor_Civilian>* GetList(void)			{return &s_xList;}

	bool HasDialogueBeenInitialised()		{return m_bDialogueHashIdsInitialised;}

	CivilianType GetCivilianType() const {return m_eCivilianType;}

protected:
	// Protected functions...
	AVP_ServerActor_Civilian(const Asura_Guid uGUID);
	virtual ~AVP_ServerActor_Civilian(void);

	virtual Axon_Brain* CreateBrain();

	void SetCivilianType( CivilianType eType ) { m_eCivilianType = eType; }

	// protected dialogue functions
	virtual Asura_Hash_ID	GetDialogueHashIDFromDialoguetype	(DialogueType uDialogType);
	virtual void			InitialiseFromBlueprint				(const Asura_Blueprint& xBlueprint);

	virtual void	OnDeath(const AVP_EntityDamage_ApplyToEntity& xDamage);

private:

	enum { CURRENT_CHUNK_VERSION = 1002 };

	// Private typedefs...
	typedef AVP_ServerActor_Human PARENT;

	// Static Private variables...
	static	Asura_Hashed_List<AVP_ServerActor_Civilian>	s_xList;

	// Private Variables
	Asura_Hash_ID			m_uCivilianDialogueHashIDs[CIVILIAN_DLG_MAX_DIALOGUE_HASH_IDS];
	bool					m_bDialogueHashIdsInitialised;
	CivilianType			m_eCivilianType;
	bool					m_bAlertEventSent;

	// Private functions...
	AVP_ServerActor_Civilian(void);
	AVP_ServerActor_Civilian(const AVP_ServerActor_Civilian& xOther);
	AVP_ServerActor_Civilian& operator=(const AVP_ServerActor_Civilian& xOther);

};   // End of class 'AVP_ServerActor_Civilian'.

#endif   // ifndef __AVP_SERVERACTOR_CIVILIAN_H__

/// End of file. ///
