//------------------------------------------------------------------------------
// File   : AVP_ServerActor_Human.cpp
// Author : Gareth Thomas
// Date   : 12 / 02 / 2008
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// Include files...
//------------------------------------------------------------------------------
// Core
#include "Core/Asura.h"

// This file's header...
#include "Entity/PhysicalObject/Actor/Vertebrates/Human/Marine/AVP_ServerActor_Marine.h"

// Asura files...
#include "Blueprint/Asura_Blueprint.h"
#include "Blueprint/Asura_Blueprint_System.h"

// Game independent header-files...
#include "Actor/Vertebrates/Human/AVP_Human_MoraleHandler.h"
#include "Actor/Vertebrates/Human/AVP_Human_AIAnimStateMachine.h"
#include "Entity/AVP_Entity_Classification.h"
#include "Entity/Controller/AVP_ServerEntity_Squad_Marine.h"
#include "Inventory/AVP_Inventory_Item.h"
#include "Inventory/Items/AVP_InventoryItem_Torch.h"

// AVP axon
#include "Core/BrainTypes/AVP_Brain_Actor_Marine.h"
#include "Knowledge/AVP_Knowledge_System_Actor.h"

//------------------------------------------------------------------------------
// Static variable declarations...
//------------------------------------------------------------------------------

Asura_Hashed_List<AVP_ServerActor_Marine>	AVP_ServerActor_Marine::s_xList;

//------------------------------------------------------------------------------
// AVP_ServerActor_Marine:: Constructor
//------------------------------------------------------------------------------
AVP_ServerActor_Marine::AVP_ServerActor_Marine(const Asura_Guid uGUID)
: Asura_Entity_Server(uGUID)
, PARENT(uGUID)
, m_bDialogueHashIdsInitialised(false)
{
	// Update the data...
	Classification	= AVP_ENTITYCLASS_MARINE;

	for (int i = 0; i < MARINE_DLG_MAX_DIALOGUE_HASH_IDS; ++i)
	{
		m_uMarineDialogueHashIDs[i] = ASURA_HASH_ID_UNSET;
	}

	// Don't forget to store us in the list...
	s_xList.AddToList(this);
}

//------------------------------------------------------------------------------
// AVP_ServerActor_Marine:: Destructor
//------------------------------------------------------------------------------
AVP_ServerActor_Marine::~AVP_ServerActor_Marine()
{
	s_xList.RemoveFromList(this);
}

void AVP_ServerActor_Marine::ReadFromChunkStream(Asura_Chunk_Stream& xStream)
{
	// We were created without a chunk initially, so we must check if this is our chunk or our parent's.
	u_int uVersion = xStream.PeekUInt();

	if ( uVersion < 1000 )
	{
		PARENT::ReadFromChunkStream( xStream );
	}
	else
	{
		xStream >> uVersion;

		PARENT::ReadFromChunkStream( xStream );

		if ( uVersion == CURRENT_CHUNK_VERSION )
		{
		}
		else
		{
			Asura_Assertf( false, ( "AVP_ServerActor_Marine::ReadFromChunkStream - unhandled chunk version (%u)", uVersion ) );
		}
	}
}

void AVP_ServerActor_Marine::WriteToChunkStream(Asura_Chunk_Stream& xStream) const
{
	xStream << CURRENT_CHUNK_VERSION;

	PARENT::WriteToChunkStream( xStream );

	// No data yet.
}

void AVP_ServerActor_Marine::OnDeath(const AVP_EntityDamage_ApplyToEntity& xDamage)
{
	PARENT::OnDeath( xDamage );

	for ( Asura_Hashed_List_It < AVP_ServerActor_Marine > it( AVP_ServerActor_Marine::GetList() ); !it.Done(); it.Next() )
	{
		AVP_ServerActor_Marine* pxMarine = it.GetCurrent();

		if ( pxMarine && pxMarine->GetGuid() != GetGuid() && !pxMarine->IsPlayer() && !pxMarine->IsDead() && !pxMarine->IsADeadBody() && pxMarine->IsEntityActive() )
		{
			AVP_Brain_Actor_Marine* pxMarineBrain = static_cast<AVP_Brain_Actor_Marine*>( pxMarine->GetActorBrain() );

			if ( pxMarineBrain && pxMarineBrain->IsActive() && pxMarineBrain->GetKnowledgeSystem() )
			{
				pxMarineBrain->GetKnowledgeSystem()->SetAlertLevel( AVP_Knowledge_System_Actor::AGGRESSIVE_ALERTNESS );
				pxMarineBrain->GetKnowledgeSystem()->ResetAlertTimer();
			}
		}
	}
}

//------------------------------------------------------------------------------
// AVP_ServerActor_Marine:: Create
//------------------------------------------------------------------------------
void AVP_ServerActor_Marine::Create(Asura_Guid uGUID)
{
	new AVP_ServerActor_Marine(uGUID);
}
//------------------------------------------------------------------------------
// AVP_ServerActor_Marine:: CreateAndReturn
//------------------------------------------------------------------------------
AVP_ServerActor_Marine* AVP_ServerActor_Marine::CreateAndReturn(Asura_Guid uGUID)
{
	AVP_ServerActor_Marine* pxMarine = new AVP_ServerActor_Marine(uGUID);

	return pxMarine;
}

//-----------------------------------------------------------------
// AVP_ServerActor_Marine :: InitialiseDialogueFromBlueprint
//-----------------------------------------------------------------
void AVP_ServerActor_Marine::InitialiseDialogueFromBlueprint(Asura_Hash_ID uBlueprintHashID)
{
	if ( GetBlueprint() && !IsPlayer() )
	{
		Asura_Value xValue;

		if ( GetBlueprint()->GetParameterValueByHashID( 0x661d3a4b , xValue ) ) // 0x661d3a4b; //"Dialogue Blueprint"
		{
			Asura_Hash_ID		uBlueprintTypeHashID		= 0xf20fd9d8; //Asura_GetHashID("Dialogue");
			Asura_Hash_ID		uDialogueBlueprintHashID	= xValue.GetHashIDValue();
			Asura_Blueprint*	pxBlueprint					= Asura_Blueprint_System::GetBlueprintByHashID(uBlueprintTypeHashID, uDialogueBlueprintHashID);

			if (pxBlueprint)
			{
				// Passive State Dialogue
				if (pxBlueprint->GetParameterValueByHashID(0xd3c48c73, xValue))		//0xd3c48c73; //"AGGRESSIVE_TO_ALERT"
				{
					m_uMarineDialogueHashIDs[MARINE_DLG_HUMAN_AGGRESSIVE_TO_ALERT] = xValue.GetHashIDValue();
				}
				else
				{
					Asura_Assert( false,  "Dialogue Paramater Not Found In Blueprint." );
				}

				if (pxBlueprint->GetParameterValueByHashID(0xc95a1426, xValue))		//0xc95a1426; //"ALERT_TO_PASSIVE"
				{
					m_uMarineDialogueHashIDs[MARINE_DLG_HUMAN_ALERT_TO_PASSIVE] = xValue.GetHashIDValue();
				}
				else
				{
					Asura_Assert( false,  "Dialogue Paramater Not Found In Blueprint." );
				}

				if (pxBlueprint->GetParameterValueByHashID(0x61ebe9a0, xValue))		//0x61ebe9a0; //"ALERT_TO_PASSIVE_FROM_AGGRESSIVE"
				{
					m_uMarineDialogueHashIDs[MARINE_DLG_HUMAN_ALERT_TO_PASSIVE_FROM_AGGRESSIVE] = xValue.GetHashIDValue();
				}
				else
				{
					Asura_Assert( false,  "Dialogue Paramater Not Found In Blueprint." );
				}

				if (pxBlueprint->GetParameterValueByHashID(0xe374a49a, xValue))		//0xe374a49a; //"ATTACKING"
				{
					m_uMarineDialogueHashIDs[MARINE_DLG_HUMAN_ATTACKING] = xValue.GetHashIDValue();
				}
				else
				{
					Asura_Assert( false,  "Dialogue Paramater Not Found In Blueprint." );
				}

				if (pxBlueprint->GetParameterValueByHashID(0xe81e6559, xValue))		//0xe81e6559; //"ENTERING_AGGRESSIVE"
				{
					m_uMarineDialogueHashIDs[MARINE_DLG_HUMAN_ENTERING_AGGRESSIVE] = xValue.GetHashIDValue();
				}
				else
				{
					Asura_Assert( false,  "Dialogue Paramater Not Found In Blueprint." );
				}

				if (pxBlueprint->GetParameterValueByHashID(0xff738942, xValue))		//0xff738942; //"ENTERING_AGGRESSIVE"
				{
					m_uMarineDialogueHashIDs[MARINE_DLG_HUMAN_ENTERING_AGGRESSIVE] = xValue.GetHashIDValue();
				}
				else
				{
					Asura_Assert( false,  "Dialogue Paramater Not Found In Blueprint." );
				}

				if (pxBlueprint->GetParameterValueByHashID(0x22beef9f, xValue))		//0x22beef9f; //"FALLING_BACK"
				{
					m_uMarineDialogueHashIDs[MARINE_DLG_HUMAN_FALLING_BACK] = xValue.GetHashIDValue();
				}
				else
				{
					Asura_Assert( false,  "Dialogue Paramater Not Found In Blueprint." );
				}

				if (pxBlueprint->GetParameterValueByHashID(0xd1c5c63c, xValue))		//0xd1c5c63c; //"FLEEING"
				{
					m_uMarineDialogueHashIDs[MARINE_DLG_HUMAN_FLEEING] = xValue.GetHashIDValue();
				}
				else
				{
					Asura_Assert( false,  "Dialogue Paramater Not Found In Blueprint." );
				}

				if (pxBlueprint->GetParameterValueByHashID(0x567a794a, xValue))		//0x567a794a; //"FRIENDLY_FIRE"
				{
					m_uMarineDialogueHashIDs[MARINE_DLG_HUMAN_FRIENDLY_FIRE] = xValue.GetHashIDValue();
				}
				else
				{
					Asura_Assert( false,  "Dialogue Paramater Not Found In Blueprint." );
				}

				if (pxBlueprint->GetParameterValueByHashID(0x29bab1ae, xValue))		//0x29bab1ae; //"HURRY_UP"
				{
					m_uMarineDialogueHashIDs[MARINE_DLG_HUMAN_HURRY_UP] = xValue.GetHashIDValue();
				}
				else
				{
					Asura_Assert( false,  "Dialogue Paramater Not Found In Blueprint." );
				}

				if (pxBlueprint->GetParameterValueByHashID(0x97cc205e, xValue))		//0x97cc205e; //"IDLE_CHATTER"
				{
					m_uMarineDialogueHashIDs[MARINE_DLG_HUMAN_IDLE_CHATTER] = xValue.GetHashIDValue();
				}
				else
				{
					Asura_Assert( false,  "Dialogue Paramater Not Found In Blueprint." );
				}

				if (pxBlueprint->GetParameterValueByHashID(0xc1c4cc77, xValue))		//0xc1c4cc77; //"KEEP_FORMATION"
				{
					m_uMarineDialogueHashIDs[MARINE_DLG_HUMAN_KEEP_FORMATION] = xValue.GetHashIDValue();
				}
				else
				{
					Asura_Assert( false,  "Dialogue Paramater Not Found In Blueprint." );
				}

				if (pxBlueprint->GetParameterValueByHashID(0x385e0c1c, xValue))		//0x385e0c1c; //"KILLED_THREAT"
				{
					m_uMarineDialogueHashIDs[MARINE_DLG_HUMAN_KILLED_THREAT] = xValue.GetHashIDValue();
				}
				else
				{
					Asura_Assert( false,  "Dialogue Paramater Not Found In Blueprint." );
				}

				if (pxBlueprint->GetParameterValueByHashID(0x6bd41904, xValue))		//0x6bd41904; //"KILLED_THREAT_RESPONSE"
				{
					m_uMarineDialogueHashIDs[MARINE_DLG_HUMAN_KILLED_THREAT_RESPONSE] = xValue.GetHashIDValue();
				}
				else
				{
					Asura_Assert( false,  "Dialogue Paramater Not Found In Blueprint." );
				}

				if (pxBlueprint->GetParameterValueByHashID(0x3099d0b0, xValue))		//0x3099d0b0; //"PASSIVE_TO_ALERT"
				{
					m_uMarineDialogueHashIDs[MARINE_DLG_HUMAN_PASSIVE_TO_ALERT] = xValue.GetHashIDValue();
				}
				else
				{
					Asura_Assert( false,  "Dialogue Paramater Not Found In Blueprint." );
				}

				if (pxBlueprint->GetParameterValueByHashID(0xeb3035b4, xValue))		//0xeb3035b4; //"TAUNTING"
				{
					m_uMarineDialogueHashIDs[MARINE_DLG_HUMAN_TAUNTING] = xValue.GetHashIDValue();
				}
				else
				{
					Asura_Assert( false,  "Dialogue Paramater Not Found In Blueprint." );
				}

				if (pxBlueprint->GetParameterValueByHashID(0x5dff2dc9, xValue))		//0x5dff2dc9; //"PROVIDING_COVERING_FIRE"
				{
					m_uMarineDialogueHashIDs[MARINE_DLG_HUMAN_PROVIDING_COVERING_FIRE] = xValue.GetHashIDValue();
				}
				else
				{
					Asura_Assert( false,  "Dialogue Paramater Not Found In Blueprint." );
				}

				if (pxBlueprint->GetParameterValueByHashID(0x166103a9, xValue))		//0x166103a9; //"RELOADING"
				{
					m_uMarineDialogueHashIDs[MARINE_DLG_HUMAN_RELOADING] = xValue.GetHashIDValue();
				}
				else
				{
					Asura_Assert( false,  "Dialogue Paramater Not Found In Blueprint." );
				}

				if (pxBlueprint->GetParameterValueByHashID(0x3e71ad2d, xValue))		//0x3e71ad2d; //"SNEAK_ATTACK_RESPONSE"
				{
					m_uMarineDialogueHashIDs[MARINE_DLG_HUMAN_SNEAK_ATTACK_RESPONSE] = xValue.GetHashIDValue();
				}
				else
				{
					Asura_Assert( false,  "Dialogue Paramater Not Found In Blueprint." );
				}

				if (pxBlueprint->GetParameterValueByHashID(0x2576dc15, xValue))		//0x2576dc15; //"TEAM_MATE_KILLED"
				{
					m_uMarineDialogueHashIDs[MARINE_DLG_HUMAN_TEAM_MATE_KILLED] = xValue.GetHashIDValue();
				}
				else
				{
					Asura_Assert( false,  "Dialogue Paramater Not Found In Blueprint." );
				}

				m_bDialogueHashIdsInitialised = true;
			}
		}
	}
}




//-----------------------------------------------------------------
// AVP_ServerActor_Marine :: GetDialogueHashIDFromDialoguetype
//-----------------------------------------------------------------
Asura_Hash_ID AVP_ServerActor_Marine::GetDialogueHashIDFromDialoguetype(DialogueType uDialogType)
{
	if (!m_pxMoraleHandler)
	{
		Asura_Assert(false, "AVP_ServerActor_Marine :: GetDialogueHashIDFromDialoguetype - Morale Handler is invalid dialogue will not play");
		
		return ASURA_HASH_ID_UNSET;
	}

	switch (uDialogType)
	{
		case AVP_DLG_HUMAN_ALERT_TO_PASSIVE:					return ( m_uMarineDialogueHashIDs[ MARINE_DLG_HUMAN_ALERT_TO_PASSIVE ] );
		case AVP_DLG_HUMAN_ALERT_TO_PASSIVE_FROM_AGGRESSIVE:	return ( m_uMarineDialogueHashIDs[ MARINE_DLG_HUMAN_ALERT_TO_PASSIVE_FROM_AGGRESSIVE ] );
		case AVP_DLG_HUMAN_IDLE_CHATTER:						return ( m_uMarineDialogueHashIDs[ MARINE_DLG_HUMAN_IDLE_CHATTER ] );
		case AVP_DLG_HUMAN_ENTERING_COVER:						return ( m_uMarineDialogueHashIDs[ MARINE_DLG_HUMAN_ENTERING_COVER ] );
		case AVP_DLG_HUMAN_PASSIVE_TO_ALERT:					return ( m_uMarineDialogueHashIDs[ MARINE_DLG_HUMAN_PASSIVE_TO_ALERT ] );
		case AVP_DLG_HUMAN_AGGRESSIVE_TO_ALERT:					return ( m_uMarineDialogueHashIDs[ MARINE_DLG_HUMAN_AGGRESSIVE_TO_ALERT ] );
		case AVP_DLG_HUMAN_ENTERING_AGGRESSIVE:					return ( m_uMarineDialogueHashIDs[ MARINE_DLG_HUMAN_ENTERING_AGGRESSIVE ] );
		case AVP_DLG_HUMAN_ATTACKING:							return ( m_uMarineDialogueHashIDs[ MARINE_DLG_HUMAN_ATTACKING ] );
		case AVP_DLG_HUMAN_RELOADING:							return ( m_uMarineDialogueHashIDs[ MARINE_DLG_HUMAN_RELOADING ] );
		case AVP_DLG_HUMAN_FLEEING:								return ( m_uMarineDialogueHashIDs[ MARINE_DLG_HUMAN_FLEEING ] );
		case AVP_DLG_HUMAN_TAUNTING:							return ( m_uMarineDialogueHashIDs[ MARINE_DLG_HUMAN_TAUNTING ] );
		case AVP_DLG_HUMAN_KILLED_THREAT:						return ( m_uMarineDialogueHashIDs[ MARINE_DLG_HUMAN_KILLED_THREAT ] );
		case AVP_DLG_HUMAN_KILLED_THREAT_RESPONSE:				return ( m_uMarineDialogueHashIDs[ MARINE_DLG_HUMAN_KILLED_THREAT_RESPONSE ] );
		case AVP_DLG_HUMAN_TEAMMATE_KILLED:						return ( m_uMarineDialogueHashIDs[ MARINE_DLG_HUMAN_TEAM_MATE_KILLED ] );
		case AVP_DLG_HUMAN_SCARED_REACTION:						return ( m_uMarineDialogueHashIDs[ MARINE_DLG_HUMAN_SNEAK_ATTACK_RESPONSE ] );
		case AVP_DLG_HUMAN_FALLING_BACK:						return ( m_uMarineDialogueHashIDs[ MARINE_DLG_HUMAN_FALLING_BACK ] );
		case AVP_DLG_HUMAN_FRIENDLY_FIRE:						return ( m_uMarineDialogueHashIDs[ MARINE_DLG_HUMAN_FRIENDLY_FIRE ] );
		case AVP_DLG_HUMAN_HURRY_UP:							return ( m_uMarineDialogueHashIDs[ MARINE_DLG_HUMAN_HURRY_UP ] );
		case AVP_DLG_HUMAN_KEEP_FORMATION:						return ( m_uMarineDialogueHashIDs[ MARINE_DLG_HUMAN_KEEP_FORMATION ] );
		case AVP_DLG_HUMAN_PROVIDING_COEVRING_FIRE:				return ( m_uMarineDialogueHashIDs[ MARINE_DLG_HUMAN_PROVIDING_COVERING_FIRE ] );
		default:												return ASURA_HASH_ID_UNSET;
	}
}


void AVP_ServerActor_Marine::PostLoadInitialise()
{
	if ( !IsPlayer() && !IsADeadBody() && !IsDead() )
	{
		if ( m_pxAIAnimStateMachine == NULL )
		{
			m_pxAIAnimStateMachine = new AVP_Human_AIAnimStateMachine( *this );
		}
		m_pxAIAnimStateMachine->Initialise();
	}

	PARENT::PostLoadInitialise();

#if defined(ASURA_ASSERT)
	// Validate Our Dialogue Blueprint
	if ( GetBlueprint() && !IsPlayer() )
	{
		Asura_Value xValue;

		if ( GetBlueprint()->GetParameterValueByHashID( 0x661d3a4b, xValue ) ) // 0x661d3a4b; //"Dialogue Blueprint"
		{
			Asura_Hash_ID		uBlueprintTypeHashID		= 0xf20fd9d8; // Asura_GetHashID("Dialogue");
			Asura_Hash_ID		uDialogueBlueprintHashID	= xValue.GetHashIDValue();
			Asura_Blueprint*	pxBlueprint					= Asura_Blueprint_System::GetBlueprintByHashID(uBlueprintTypeHashID, uDialogueBlueprintHashID);
#ifdef ASURA_TOOLS
			Asura_Assertf( pxBlueprint, ( "Dialogue Blueprint Not Found For Marine Actor With GUID Of %d in blueprint %s", GetGuid(), GetBlueprint()->GetNameIncludingAllParentNames() ) );
#else
			Asura_Assertf( pxBlueprint, ( "Dialogue Blueprint Not Found For Marine Actor With GUID Of %d , Has It Been Setup in the Actors Blueprint?", GetGuid() ) );
#endif
		}
	}
#endif	//defined(ASURA_ASSERT)
}

void AVP_ServerActor_Marine::MakeAI()
{
	PARENT::MakeAI();
	if ( m_pxAIAnimStateMachine == NULL )
	{
		m_pxAIAnimStateMachine = new AVP_Human_AIAnimStateMachine( *this );
	}
}

void AVP_ServerActor_Marine::OnEntityActivate()
{
	PARENT::OnEntityActivate();

	AVP_Brain_Actor_Marine* pxMarineBrain = (AVP_Brain_Actor_Marine*)GetActorBrain();

	if (pxMarineBrain)
	{
		pxMarineBrain->RequestRegisterWithTeam();
	}
}

void AVP_ServerActor_Marine::OnEntityDeactivate()
{
	PARENT::OnEntityDeactivate();

	AVP_Brain_Actor_Marine* pxMarineBrain = (AVP_Brain_Actor_Marine*)GetActorBrain();

	if (pxMarineBrain)
	{
		pxMarineBrain->RequestUnregisterWithTeam();
	}
}

bool AVP_ServerActor_Marine::AwardLoadsOfDamageToAllMarines( bool bHelp )
{
	if (bHelp) return true;
	for (Asura_Hashed_List_It<AVP_ServerActor_Marine> xIt(GetList()) ; !xIt.Done() ; xIt.Next())
	{
		xIt.GetCurrent()->AwardLoadsOfDamage();
	}
	return true;
}