
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_ServerActor_Predalien.h
// Author : Andy Tate
// Date   : 3-Oct-2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __AVP_SERVERACTOR_PREDALIEN_H__
#define __AVP_SERVERACTOR_PREDALIEN_H__

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

// asura
#include "Core/Asura_HashedList.h"

// avp
#include "Actor/Vertebrates/Predalien/AVP_Predalien_Description.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/AVP_ServerActor_Vertebrate.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Predalien/AVP_Messages_Predalien.h"

////////////////////////////////////////////////////////////////////////////////
// Forward class declarations...
////////////////////////////////////////////////////////////////////////////////

class AVP_Message_PredalienPlayerCommands;

////////////////////////////////////////////////////////////////////////////////
// Class declaration...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_ServerActor_Predalien
////////////////////////////////////////////////////////////////////////////////

class AVP_ServerActor_Predalien : public AVP_ServerActor_Vertebrate
{
public:
	// Overridden functions...
	virtual void MakePlayer(Asura_Guid uPlayerGuid);
	virtual void MakeAI(void);
	virtual void PostLoadInitialise(void);
	virtual Axon_Brain* CreateBrain();
	virtual	void TakeDamageExtended(const AVP_EntityDamage_RequestVictim& xDamage, const bool bForceFatal, const bool bSendMessageToClient=true );
	virtual	bool HandleMessage(const Asura_Message* pxMessage);

	// Overridden chunk-handling functions...
	virtual	void ReadFromChunkStream(Asura_Chunk_Stream& xStream);
	virtual	void WriteToChunkStream(Asura_Chunk_Stream& xStream) const;

	// Legacy chunk handling, to be removed as soon as possible...
	virtual	bool IsFilledFromChunkBuffer(void) const { return(false); }

	// Public Member Functions...
	void Copy(const AVP_ServerActor_Predalien* pxFrom);

	bool GetUseTestBehaviour() const { return m_bUseTestBehaviour; }
	void SetUseTestBehaviour( bool bVal ) { m_bUseTestBehaviour = bVal; }
	void KnockDown();
	bool IsKnockedDown() const;
	void SetGrapple();
	u_int GetGrappleCount() const;

	virtual bool GetHasCreateMessage() const { return !IsDead() || IsADeadBody(); }

	// Static Public Member Functions...
	static	AVP_ServerActor_Predalien* CreateAndReturn(Asura_Guid uGUID) { return new AVP_ServerActor_Predalien(uGUID); }
	static	void Create(Asura_Guid uGUID) { new AVP_ServerActor_Predalien(uGUID); }
	static	AVP_ServerActor_Predalien* FindEntity(const Asura_Guid uGuid) { return(s_xList.FindEntity(uGuid)); }
	static	const Asura_Hashed_List<AVP_ServerActor_Predalien>*	GetList(void) { return(&s_xList); }


protected:

	// Protected functions...
	AVP_ServerActor_Predalien(const Asura_Guid uGUID);
	virtual ~AVP_ServerActor_Predalien();

	// Overridden entity update functions...
	virtual	void MakeClientUpdateMessage(void);

	// Overridden event functions...
	virtual void OnApplyDamage(const AVP_EntityDamage_ApplyToEntity& xDamage);
	virtual void OnDeath(const AVP_EntityDamage_ApplyToEntity& xDamage);

	virtual float GetHeadPositionYOffset() const { return -2.0f; }
	virtual float GetChestPositionYOffset() const { return -1.5f; }

	virtual Asura_Hash_ID GetMovesBlueprintType() const;

private:
	typedef AVP_ServerActor_Vertebrate PARENT;

	union
	{
		u_int m_uChunkedPredalienFlags;			// Single access-point for flags to be chunked.
		
		struct
		{
			u_int m_bUseTestBehaviour : 1,
				: 0;	
		};
	};

	union
	{
		u_int m_uUnchunkedPredalienFlags;			// Single access-point for flags that are not chunked.

		struct
		{
			u_int m_bCountered : 1,
				: 0;	
		};
	};

	u_int m_uMineHitsCount;
	u_int m_uGrappleCount;

#if defined(ASURA_TOOLS)
	friend class ToolsAVP_Entity_Predalien;
#endif

	static Asura_Hashed_List<AVP_ServerActor_Predalien> s_xList;

	AVP_ServerActor_Predalien(void);
	AVP_ServerActor_Predalien(const AVP_ServerActor_Predalien& xOther);
	AVP_ServerActor_Predalien&	operator=(const AVP_ServerActor_Predalien& xOther);

	static const u_int uCURRENT_CHUNK_VERSION = 1;
};

#endif   // ifndef __AVP_SERVERACTOR_PREDALIEN_H__

/// End of file. ///
