
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_ServerActor_Predator.h
// Author : Andy Tate
// Date   : 3-Oct-2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __AVP_SERVERACTOR_PREDATOR_H__
#define __AVP_SERVERACTOR_PREDATOR_H__

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

// asura
#include "Core/Asura_HashedList.h"

// avp
#include "Actor/Vertebrates/Predator/AVP_Predator_Description.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/AVP_ServerActor_Vertebrate.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Predator/AVP_Messages_Predator.h"

////////////////////////////////////////////////////////////////////////////////
// Forward class declarations...
////////////////////////////////////////////////////////////////////////////////

class AVP_Message_PredatorPlayerCommands;

////////////////////////////////////////////////////////////////////////////////
// Class declaration...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_ServerActor_Predator
// Author       : Andy Tate
// Date         : 3-Oct-2006
//
// Description  : This class handles the operation of the Predator actor, the
//				  Predator can be either player controlled or AI controlled.
////////////////////////////////////////////////////////////////////////////////

class AVP_ServerActor_Predator : public AVP_ServerActor_Vertebrate
{
public:

	enum { CURRENT_CLIENTCREATE_VERSION = 1 };

	// Overridden functions...
	virtual	void Update(void);
	virtual	bool HandleMessage(const Asura_Message* pxMessage);
	virtual void MakePlayer(Asura_Guid uPlayerGuid);
	virtual void MakeAI(void);
	virtual void PostLoadInitialise(void);
	virtual Axon_Brain* CreateBrain(void);
	virtual	void TakeDamageExtended(const AVP_EntityDamage_RequestVictim& xDamage, const bool bForceFatal, const bool bSendMessageToClient=true );

	virtual void AddPermenantlyActiveItems(AVP_Inventory_Group* pxGroup);

	// Overridden chunk-handling functions...
	virtual	void ReadFromChunkStream(Asura_Chunk_Stream& xStream);
	virtual	void WriteToChunkStream(Asura_Chunk_Stream& xStream) const;

	virtual void WriteToClientCreateStream(Asura_Chunk_Memory_Stream& xStream) const;

	// Legacy chunk handling, to be removed as soon as possible...
	virtual	bool IsFilledFromChunkBuffer(void) const { return(false); }

	// Accessors functions...
	Asura_Guid GetDiscGuid(void) const { return(m_uDiscGuid); }
	Asura_Guid GetHeldProjectileGuid(void) const { return(m_uHeldProjectileGuid); }
	void SetHeldProjectileGuid(Asura_Guid uGuid) { m_uHeldProjectileGuid = uGuid; }
	Asura_Guid GetTargetGuid(void) const { return m_uTargetGuid; }
	void SetTargetGuid(Asura_Guid uTargetGuid) { m_uTargetGuid = uTargetGuid; }

	// Public Member Functions...
	void Copy(const AVP_ServerActor_Predator* pxFrom);

	bool IsCrouching() const { return m_bCrouching; }
	void SetCrouching( const bool bSet ) { m_bCrouching = bSet; }

	bool IsLockOn(void) const { return m_bLockOn; }
	void SetLockOn(bool bLockOn) { m_bLockOn = bLockOn; }

	bool GetUseTestBehaviour() const { return m_bUseTestBehaviour; }
	void SetUseTestBehaviour( bool bVal ) { m_bUseTestBehaviour = bVal; }

	bool GetStartsCloaked() const { return m_bStartsCloaked; }
	void SetStartsCloaked(bool bVal) { m_bStartsCloaked = bVal; }

	virtual bool GetHasCreateMessage() const { return !IsDead() || IsADeadBody(); }

	virtual u_int GetElementFromBoneID(int iBoneID) const { return AVP_Predator_Description::GetElementFromBoneID(iBoneID); }
	virtual int GetAnyBoneIDFromElement(u_int uElement) const { return AVP_Predator_Description::GetAnyBoneIDFromElement(uElement); } // Since bones->elements is a many->one mapping, this function will return one of the bones associated with the given element, but not necessarily the only one.

	void Cloak();
	void Decloak(bool bManually = true);
	bool IsCloaked() const;

	void EquipPlasmaCaster();
	void EquipWristBlades();
	bool IsHoldingPlasmaCaster() const;
	bool IsHoldingWristBlades() const;

	bool StartPlasmaCasterCharge(float fDamage);
	void ReleasePlasmaCasterCharge();
	void AbortPlasmaCasterCharge();
	bool IsPlasmaCasterCharged();

#ifndef ASURA_NO_DEBUG_OUTPUT
	void GetCloakDebugString(const u_int uMaxChars, char* szText) const;
	void GetWeaponDebugString(const u_int uMaxChars, char* szText) const;
#endif


	const AVP_PredatorBehaviourData& GetBehaviourData() const { return m_xBehaviourData; }

	// Static Public Member Functions...
	static	AVP_ServerActor_Predator*							CreateAndReturn(Asura_Guid uGUID)	{ return new AVP_ServerActor_Predator(uGUID); }
	static	void												Create(Asura_Guid uGUID)			{ new AVP_ServerActor_Predator(uGUID); }
	static	AVP_ServerActor_Predator*							FindEntity(const Asura_Guid uGuid)	{ return(s_xList.FindEntity(uGuid)); }
	static	const Asura_Hashed_List<AVP_ServerActor_Predator>*	GetList(void)						{ return(&s_xList); }

	Asura_LinkList<Asura_Guid>*									GetMineGuidList(void)					{ return(&m_xMineList); }
	const Asura_LinkList<Asura_Guid>*							GetMineGuidList(void) const				{ return(&m_xMineList); }

	static bool AwardLoadsOfDamageToAllPredators( bool bHelp );

protected:

	// Protected variables...
	Asura_Guid					m_uDiscGuid;				// GUID for the Predator's Disc.
	Asura_Guid					m_uHeldProjectileGuid;		// GUID for the currently held projectile.
	Asura_Guid					m_uTargetGuid;				// GUID for the current predator target
	bool						m_bLockOn;					// Flag to say if the predator has a lock on someone

	// Energy is still handled nastily on the client machine, this is basically a copy of that value
	float						m_fPredatorEnergy;			

	// Protected functions...
	AVP_ServerActor_Predator(const Asura_Guid uGUID);
	virtual ~AVP_ServerActor_Predator();

	void	ProcessPredatorPlayerCommands( const AVP_Message_PredatorPlayerCommands* pxData, float fMessageSentTime );

	// Overridden Initialisation functions...
	virtual bool	InitialiseInventory(void);
	
	// Overridden entity update functions...
	virtual	void	MakeClientUpdateMessage(void);

	// Overridden event functions...
	virtual void	OnApplyDamage(const AVP_EntityDamage_ApplyToEntity& xDamage);
	virtual void	OnDeath(const AVP_EntityDamage_ApplyToEntity& xDamage);

	void EffectMorale(Asura_Guid uGuidToEffect);

	virtual bool GetIsInLowStance() const { return IsCrouching(); }

	virtual const Asura_Blueprint* GetCurrentFlinchBlueprint() const;

	virtual float GetHeadPositionYOffset() const { return -2.1f; }
	virtual float GetChestPositionYOffset() const { return -1.7f; }

	virtual Asura_Hash_ID GetMovesBlueprintType() const;

	virtual void HandleInteractions();

private:
	typedef AVP_ServerActor_Vertebrate PARENT;

	void UpdateCloakingAndWeaponStatus();

	enum WeaponType { WEAPON_WRISTBLADES, WEAPON_PLASMACASTER, WEAPON_UKNOWN  };
	enum PlasmacasterState { PLASMACASTER_IDLE, PLASMACASTER_CHARGING, PLASMACASTER_REQUIRE_RELEASING, PLASMACASTER_REQUIRE_ABORTING };

	bool m_bDesiredCloakingStatus;
	bool m_bDecloakManually;
	WeaponType m_eDesiredWeaponType;
	float m_fPlasmaCasterDesiredCharge;
	PlasmacasterState m_ePlasmacasterUsageState;
	Asura_LinkList<Asura_Guid> m_xMineList;

	AVP_PredatorBehaviourData m_xBehaviourData;

	union
	{
		u_int		m_uChunkedPredatorFlags;			// Single access-point for flags to be chunked.
		
		struct
		{
			u_int	m_bCrouching		: 1,	// Flag to say whether we're currently crouching.
					m_bStartsCloaked	: 1,	// Flag to say that the Predator should be cloaked when he is activated.
					m_bUseTestBehaviour : 1,
										: 0;	// 30 spare-bits.
		};
	};

#if defined(ASURA_TOOLS)
	friend class ToolsAVP_Entity_Predator;
#endif

	static Asura_Hashed_List<AVP_ServerActor_Predator> s_xList;

	AVP_ServerActor_Predator(void);
	AVP_ServerActor_Predator(const AVP_ServerActor_Predator& xOther);
	AVP_ServerActor_Predator&	operator=(const AVP_ServerActor_Predator& xOther);

	static const u_int uCURRENT_CHUNK_VERSION = 6;
};

#endif   // ifndef __AVP_SERVERACTOR_PREDATOR_H__

/// End of file. ///
