//////////////////////////////////////////////////////////////////////////////
//
// AVP_ClientWeapon_Sentrygun.h
//
// Copyright (c) 2007 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef AVP_CLIENTVEHICLE_SENTRYGUN_H___
#define AVP_CLIENTVEHICLE_SENTRYGUN_H___ 1

// I N C L U D E S ///////////////////////////////////////////////////////////

// Asura
#include "Entity/Container/Asura_ClientContainerInstance.h"

// AVP
#include "Blueprint/AVP_Blueprint_HashIDs.h"
#include "Entity/PhysicalObject/Actor/Weapons/AVP_ClientActor_Weapon.h"

// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////////

class Asura_Message;

class AVP_Message_SentryGun_ClientUpdate;

class Asura_AnimationAttachment_Projector;
class Asura_AnimationAttachment_ClientContainerInstance;

// C L A S S E S /////////////////////////////////////////////////////////////

// AVP_ClientWeapon_Sentrygun ////////////////////////////////////////////////

class AVP_ClientWeapon_Sentrygun : public AVP_ClientActor_Weapon
{
public:
	enum Sound
	{
		BEGIN_SOUND = 0,
		SOUND_INVESTIGATING = BEGIN_SOUND,
		SOUND_LOCKED_ON,
		SOUND_LOST_TARGET,
		SOUND_RESUMING_SWEEP,
		SOUND_DRY_FIRE,
		END_SOUND,
	};

	// Public functions...
	static	AVP_ClientWeapon_Sentrygun*	Create	(const Asura_Message* pxMessage);

	virtual void InitialiseFromBlueprint( const Asura_Blueprint& xBlueprint );
	virtual	bool HandleMessage(const Asura_Message* pxMessage);
	virtual void PostCreateInitialise(void);

	virtual	void OnDeath(const AVP_EntityDamage_ApplyToEntity& xDamage);

	// Static list functions...
	static AVP_ClientWeapon_Sentrygun* FindEntity( Asura_Guid uGUID )				{ return s_xList.FindEntity( uGUID ); }
	static const Asura_Hashed_List<AVP_ClientWeapon_Sentrygun>* GetList( void )		{ return &s_xList; }

protected:
	AVP_ClientWeapon_Sentrygun();
	virtual ~AVP_ClientWeapon_Sentrygun();

	void ProcessClientUpdate( const AVP_Message_SentryGun_ClientUpdate* pxData, float fServerTime, bool bProcessPosAndOri = true);
	
	void PlaySound( Sound eSound );

	virtual void CreateAnimationHandler();

	void CreateContainer();
	void DestroyContainer();
	void EnableContainer( bool bEnable );

	Asura_AnimationAttachment_ClientContainerInstance* m_pxContainerAttachment;

private:
	// Private typedefs...
	typedef AVP_ClientActor_Weapon PARENT;

	void SetContainerRange();

	virtual const AVP_Actor_AnimStateData* GetStateDataFromMesssageData( const u_int uState, const AVP_Message_Actor_ClientUpdate* pxActorData );
	virtual const AVP_Actor_AnimStateData* GetStateData( const u_int uState, const u_int uGroundType, const u_int uLimbDamage, const bool bIsAiming, const u_int uCoverType );
	const Asura_Sound_Event* m_apxSoundEvent[END_SOUND];
	Asura_Handle m_aiSoundEventHandles[END_SOUND];

	static Asura_Hashed_List<AVP_ClientWeapon_Sentrygun> s_xList;

	Asura_Hash_ID m_uDefaultBlueprint;

	// Private functions...
};

// AVP_FindSentryBeamFunctor /////////////////////////////////////////////////

class AVP_FindSentryBeamFunctor : public Asura_Container_Functor
{
public:
	const Asura_ClientContainerInstance_Projector	*GetProjector()		const {return m_pxProjector; }
	const Asura_ClientContainerInstance_LightShaft	*GetLightShaft()	const {return m_pxLightShaft; }

	AVP_FindSentryBeamFunctor() : m_pxProjector( NULL ), m_pxLightShaft( NULL ) {}
	virtual ~AVP_FindSentryBeamFunctor() {}

	virtual void Process( const Asura_ClientContainerInstance* pxEntry )
	{
		if( pxEntry->GetClassification() == AsuraContainerClass_Projector )
		{
			m_pxProjector = static_cast<const Asura_ClientContainerInstance_Projector*>( pxEntry );
		}
		else if( pxEntry->GetClassification() == AsuraContainerClass_LightShaft )
		{
			m_pxLightShaft = static_cast<const Asura_ClientContainerInstance_LightShaft*>( pxEntry );
		}
	}

private:
	const Asura_ClientContainerInstance_Projector	*m_pxProjector;
	const Asura_ClientContainerInstance_LightShaft	*m_pxLightShaft;
};

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

#endif // #ifndef AVP_CLIENTVEHICLE_SENTRYGUN_H___
