//
// AVP_ClientEntity_CommandPost.cpp
//
// Created by RDM - 21 November 2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//

/************/
/* Includes */
/************/

// main asura header
#include "Core/Asura.h"

// This file's header.
#include "Entity/PhysicalObject/CommandPost/AVP_ClientEntity_CommandPost.h"

// asura
#include "Animation/Asura_AnimationController.h"
#include "Animation/Asura_AnimationController_Animation.h"
#include "Camera/Asura_Camera.h"
#include "Core/Asura_HashedList.h"
#include "Core/Asura_HashedLocalisedText.h"
#include "Core/Asura_TiledTextureManagement.h"
#include "Core/Asura_Timers.h"
#include "Network/Asura_Message.h"
#include "Rendering/Asura_Font_System.h"
#include "Sound/Asura_Sound.h"
#include "Sound/Asura_Sound_Event.h"
#include "Sound/Asura_Sound_Event_System.h"
#include "SpecialFX/Pfx/Asura_PFX_Entity_Effect.h"
#include "SpecialFX/Pfx/Asura_PFX_System.h"

// AVP
#include "Multiplayer/AVP_Multiplayer_GameRules.h"
#include "Core/AVP_Settings.h"
#include "Core/AVP_Game_Core.h"
//#include "Core/AVP_Text.h"
//#include "Debugging/AVP_DebugInfo_System.h"
//#include "Entity/AVP_Entity_Classification.h"
//#include "Entity/AVP_Entity_System.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/AVP_ClientActor_Vertebrate.h"
#include "Entity/PlayerInfo/AVP_ClientEntity_PlayerInfo.h"
#include "Entity/PhysicalObject/CommandPost/AVP_ServerEntity_CommandPost.h"
#include "Entity/PhysicalObject//AVP_PhysicalObject_Messages.h"
#include "Multiplayer/AVP_Multiplayer_GameMessageSystem.h"
#include "Network/AVP_Player.h"

//AVP_REGISTER_ENTITY_CLASS_CLIENT( AVP_ENTITYCLASS_COMMANDPOST, AVP_ClientEntity_CommandPost::Create, NULL, NULL );

/********/
/* Data */
/********/

static const Asura_RGBA xAVP_COLOUR_FACTION_FRIENDLY = Asura_RGBA( 0x506CB3, 0xFF );
static const Asura_RGBA xAVP_COLOUR_FACTION_ENEMY    = Asura_RGBA( 0xB32F2F, 0xFF );
static const Asura_RGBA xAVP_COLOUR_FACTION_NEUTRAL  = Asura_RGBA( 0xFFFFFF, 0xFF );

// static members
Asura_Hashed_List<AVP_ClientEntity_CommandPost> AVP_ClientEntity_CommandPost::s_xList(3);
u_int AVP_ClientEntity_CommandPost::s_uInstanceCount(0);
u_int AVP_ClientEntity_CommandPost::s_auNumberOwnedByTeam [] = {0,0};
bool AVP_ClientEntity_CommandPost::s_bIsPaused = false;

const Asura_Hash_ID AVP_ClientEntity_CommandPost::s_auPulsePFX[] = 
{	
	0xb068ddfc, //"BFF_CP_Pulse_E_Red" ENEMY
	0x5cabb84f, //"BFF_CP_Pulse_E_Blue" FRIENDLY
	0xa6319b2a, //"BFF_CP_Pulse_E" NEUTRAL
};

const Asura_Hash_ID AVP_ClientEntity_CommandPost::s_auBeamPFXCaptured[] = 
{	
	0x2e2d4558, //"BFF_CP_UpLight_Fat_E_Red"
	0x97743c73, //"BFF_CP_UpLight_Fat_E_Blue"
	0x19281086, //"BFF_CP_UpLight_Fat_E"
};

const Asura_Hash_ID AVP_ClientEntity_CommandPost::s_auBeamPFXNeutral[] = 
{	
	0x9a3e312a, //"BFF_CP_UpLight_Thin_E_Red"
	0xad80cae1, //"BFF_CP_UpLight_Thin_E_Blue"
	0x241bb558, //"BFF_CP_UpLight_Thin_E"
};

const Asura_Hash_ID AVP_ClientEntity_CommandPost::s_auIconSkinIDs[] = 
{
	0xff0b09ae, // FACTION_REPUBLIC - Asura_GetHashID("Republic_Logo")
	0x77311927, // FACTION_CIS - Asura_GetHashID("Federation_Logo")
	0x98b140b3, // FACTION_ALLIANCE - Asura_GetHashID("Alliance_Logo")
	ASURA_HASH_ID_UNSET, // FACTION_NEUTRAL
};

Asura_Beam_Template AVP_ClientEntity_CommandPost::s_axBeamTemplates[] = 
{
	// Enemy
	{
		1.0f, // fInitialRadius
		1.0f, // fEndRadius
		0.0f, // fNearSpeed
		0.0f, // fFarSpeed
		0.001f, // fDecayRate
		0x8B4608A7, // Asura_GetHashID("laser2.tga")
		-1, // iTileHandle - Setup after initialisation.
		{ { 0.7f, 0.18f, 0.18f, 1.0f } }, // xInitialNearColour
		{ { 0.7f, 0.18f, 0.18f, 0.0f } }, // xInitialFarColour
		{ { 0.7f, 0.18f, 0.18f, 1.0f } }, // xEndNearColour
		{ { 0.7f, 0.18f, 0.18f, 0.0f } }, // xEndFarColour
	},

	// Friendly
	{
		1.0f, // fInitialRadius
		1.0f, // fEndRadius
		0.0f, // fNearSpeed
		0.0f, // fFarSpeed
		0.001f, // fDecayRate
		0x8B4608A7, // Asura_GetHashID("laser2.tga")
		-1, // iTileHandle - Setup after initialisation.
		{ { 0.31f, 0.42f, 0.7f, 1.0f } }, // xInitialNearColour
		{ { 0.31f, 0.42f, 0.7f, 0.0f } }, // xInitialFarColour
		{ { 0.31f, 0.42f, 0.7f, 1.0f } }, // xEndNearColour
		{ { 0.31f, 0.42f, 0.7f, 0.0f } }, // xEndFarColour
	},

	// Neutral
	{
		1.0f, // fInitialRadius
		1.0f, // fEndRadius
		0.0f, // fNearSpeed
		0.0f, // fFarSpeed
		0.001f, // fDecayRate
		0x8B4608A7, // Asura_GetHashID("laser2.tga")
		-1, // iTileHandle - Setup after initialisation.
		{ { 1.0f, 1.0f, 1.0f, 0.75f } }, // xInitialNearColour
		{ { 1.0f, 1.0f, 1.0f, 0.0f } }, // xInitialFarColour
		{ { 1.0f, 1.0f, 1.0f, 0.75f } }, // xEndNearColour
		{ { 1.0f, 1.0f, 1.0f, 0.0f } }, // xEndFarColour
	}
};

const Asura_Hash_ID AVP_ClientEntity_CommandPost::s_uIconAnimID = 0x8417bc1a; //"Logo_Spin_Move"
const Asura_Hash_ID AVP_ClientEntity_CommandPost::s_uOpenAnimID = 0xd08f411d; //"CapturePoint_Predator_Open"

//debugvars
DEBUGVAR Asura_Vector_3 dbg_xLogoPosOffset = Asura_Vector_3( 0.0f, -1.6f, 0.0f );
DEBUGVAR Asura_Vector_3 dbg_xConePosOffset = Asura_Vector_3( 0.0f, -0.3f, 0.0f );
DEBUGVAR bool  dbg_bRenderCommandPostText = false;
//DEBUGVAR float dbg_fHumSoundFadeOutTime = 1.5f;
//DEBUGVAR float dbg_fHumMinPitch = 0.6f;
//DEBUGVAR float dbg_fHumMaxPitch = 1.2f;
DEBUGVAR float dbg_fBeamLOSRange = 100.0f;
DEBUGVAR float dbg_fBeamMinRadiusScale = 0.2f;
DEBUGVAR float dbg_fBeamPulseRadiusScale = 0.2f;
DEBUGVAR float dbg_fBeamPulseDuration = 0.3f;
DEBUGVAR float dbg_fBeamLengthDelta = 60.0f;
//DEBUGVAR float dbg_fIconAlphaDelta = 0.5f; 

//sound events
ASURA_SOUND_EVENT_LOOKUP_GLOBALINIT( 0x5412293B, "command_post_change" );
ASURA_SOUND_EVENT_LOOKUP_GLOBALINIT( 0x6E926355, "command_post_hum" );

// constants
static const Asura_Hash_ID uAVP_HASHID_MARKER_PFX_PULSE = 0x06611b99; //"Pulse"
static const Asura_Hash_ID uAVP_HASHID_MARKER_PFX_BEAM  = 0x002e13af; //"Beam"
static const Asura_Hash_ID uAVP_HASHID_MARKER_BEAM		= 0x002e13af; //"Beam" 
static const Asura_Hash_ID uAVP_HASHID_MARKER_LOGO		= 0x0032c5ab; //"Logo"

/*************/
/* Functions */
/*************/

// constructor
AVP_ClientEntity_CommandPost::AVP_ClientEntity_CommandPost()
: m_iCurrentFaction( AVP_Settings::AVP_TEAM_NONE )
, m_iCapturingFaction( AVP_Settings::AVP_TEAM_NONE )
, m_uCapturingGuid(ASURA_GUID_UNREGISTERED)
, m_fCaptureTime( 1.0f )
, m_fCaptureTimer( 0.0f )
, m_fBeamLength( -1.0f )
, m_fPulseTimer( 0.0f )
, m_iHumSoundHandle( ASURA_INVALID_HANDLE )
, m_puPulsePFXHandles( NULL )
, m_puBeamPFXHandles( NULL )
, m_puBeamIDs( NULL ) 
, m_pxIconAnims( NULL )
, m_uNameHashID( ASURA_HASH_ID_UNSET )
, m_uCommandPostFlags( 0 )
{
	Classification = AVP_ENTITYCLASS_COMMANDPOST;
	s_xList.AddToList(this);
	Asura_Entity_Client::AddToUpdateableClientList(this);

	// one PFX handle for each camera
	m_puPulsePFXHandles = new u_int[Asura_Camera::GetNumberOfCameras()];
	Asura_Mem::Set( m_puPulsePFXHandles, ASURA_PFX_INVALID_EFFECT_ID, (sizeof(u_int) * Asura_Camera::GetNumberOfCameras()) );

	m_puBeamPFXHandles = new u_int[Asura_Camera::GetNumberOfCameras()];
	Asura_Mem::Set( m_puBeamPFXHandles, ASURA_PFX_INVALID_EFFECT_ID, (sizeof(u_int) * Asura_Camera::GetNumberOfCameras()) );

	// one icon anim for each camera
	m_pxIconAnims = new Asura_Animation*[Asura_Camera::GetNumberOfCameras()];
	Asura_Mem::Set( m_pxIconAnims, 0, (sizeof(Asura_Animation*) * Asura_Camera::GetNumberOfCameras()) );

	// one beam for each camera
	m_puBeamIDs = new u_int[Asura_Camera::GetNumberOfCameras()];
	Asura_Mem::Set( m_puBeamIDs, Asura_Beam_System::uINVALID_BEAM_ID, (sizeof(u_int)* Asura_Camera::GetNumberOfCameras()) );

	++s_uInstanceCount;
}

// destructor
AVP_ClientEntity_CommandPost::~AVP_ClientEntity_CommandPost()
{
	--s_uInstanceCount;

	if( m_iHumSoundHandle != ASURA_INVALID_HANDLE )
	{
		Asura_Sound_Event_System::Stop( m_iHumSoundHandle );
	}

	for( int i=0; i<Asura_Camera::GetNumberOfCameras(); ++i )
	{
		Asura_PFX_Entity_Effect* pxPulseEffect = Asura_PFX_System::GetEntityEffect( m_puPulsePFXHandles[i] );
		if( pxPulseEffect )
		{
			pxPulseEffect->Destroy();
			m_puPulsePFXHandles[i] = ASURA_PFX_INVALID_EFFECT_ID;
		}

		Asura_PFX_Entity_Effect* pxBeamEffect = Asura_PFX_System::GetEntityEffect( m_puBeamPFXHandles[i] );
		if( pxBeamEffect )
		{
			pxBeamEffect->Destroy();
			m_puBeamPFXHandles[i] = ASURA_PFX_INVALID_EFFECT_ID;
		}

		if( m_pxIconAnims[i] )
		{
			delete m_pxIconAnims[i];
			m_pxIconAnims[i] = NULL;
		}
		
		if( m_puBeamIDs[i] != Asura_Beam_System::uINVALID_BEAM_ID )
		{
			Asura_Beam_System::DeallocateBeam(m_puBeamIDs[i]);
			m_puBeamIDs[i] = Asura_Beam_System::uINVALID_BEAM_ID;
		}
	}
	delete [] m_puPulsePFXHandles;
	delete [] m_puBeamPFXHandles;
	delete [] m_pxIconAnims;
	delete [] m_puBeamIDs;

	s_xList.RemoveFromList(this);
}

void AVP_ClientEntity_CommandPost::PostCreateInitialise(void)
{
	// Register this entity with the capture point in-game HUD list
	if (AVP_Settings::IsMultiPlayer())
	{
		AVP_Multiplayer_GameMessageSystem::AddCapturePoint( GetGuid() ); 
	}
}

/*************************************************************
 *
 * Name          : AVP_ClientEntity_CommandPost::Update
 *
 * Author        : Richard May
 *
 * Description   : Handles the command post update. 
 *
 *************************************************************/
void AVP_ClientEntity_CommandPost::Update( void )
{
	if( s_bIsPaused ) return;	// We don't want the hum sounds to be recreated when the game is paused.

 	PARENT::Update();

	if( !IsMobileCommandPost() )
	{
		UpdatePFX();
		UpdateSound();
		UpdateAnims();
		UpdateBeam();		
	}
}
/* end of function Update */

Asura_PFX_Effect_Template* AVP_ClientEntity_CommandPost::CreateColourisedPFXEffectTemplate( Asura_Hash_ID uSourceEffectTempID, Asura_Hash_ID uNewEffectTempID, const Asura_RGBA& xColour )
{
	Asura_PFX_Effect_Template* pxBaseEffect = Asura_PFX_System::GetEffectTempByHash( uSourceEffectTempID );
	if( pxBaseEffect && (uNewEffectTempID != ASURA_HASH_ID_UNSET) )
	{
		// have we already made a colourised effect for this laser bolt blueprint?
		Asura_PFX_Effect_Template* pxColourisedEffect = Asura_PFX_System::GetEffectTempByHash( uNewEffectTempID );
		if( !pxColourisedEffect )
		{
			// okay, need to make a new effect and colourise it
			pxColourisedEffect = new Asura_PFX_Effect_Template( *pxBaseEffect );
			Asura_PFX_System::AddEffectTemplate( uNewEffectTempID, pxColourisedEffect );

			// copy all the sources
			for( u_int u=0; u<pxColourisedEffect->m_uNumSources; ++u )
			{
				// get the base source, generate a new hash ID and make a copy of the base source template, adding it with new hash ID
				Asura_PFX_Source_Template* pxBaseSource = Asura_PFX_System::GetSourceTempByHash( pxColourisedEffect->m_pxSources[u].m_uTemplate );				
				Asura_Assert( pxBaseSource, "Failed to find base source template." ); 
				if( !pxBaseSource ) continue; 

				Asura_Hash_ID uColourisedSourceHashID = uNewEffectTempID+u; // hash ID + u to generate unique source/particle template hash IDs
				Asura_PFX_Source_Template* pxColourisedSource = new Asura_PFX_Source_Template( *pxBaseSource );
				Asura_PFX_System::AddSourceTemplate( uColourisedSourceHashID, pxColourisedSource );	

				pxColourisedEffect->m_pxSources[u].m_uTemplate = uColourisedSourceHashID;

				// copy the particle template, generate a new hash ID and make a copy of the base particle template, adding it with new hash ID
				Asura_PFX_Particle_Template* pxBaseParticle	= Asura_PFX_System::GetParticleTempByHash( pxColourisedSource->m_uParticleTemplateHash );					
				Asura_Assert( pxBaseParticle, "Failed to find base particle template." ); 
				if( !pxBaseParticle ) continue; 

				Asura_Hash_ID uColourisedParticleHashID = uColourisedSourceHashID;
				Asura_PFX_Particle_Template* pxColourisedParticle = new Asura_PFX_Particle_Template( *pxBaseParticle );				
				Asura_PFX_System::AddParticleTemplate( uColourisedParticleHashID, pxColourisedParticle );

				pxColourisedSource->m_uParticleTemplateHash = uColourisedParticleHashID;
			}

			// now colourise the effect
			Asura_PFX_Effect_Template::ColouriseEffectTemplate( pxColourisedEffect, xColour );
		}		

		return pxColourisedEffect;
	}

	return NULL;
}

/*************************************************************
 *
 * Name          : AVP_ClientEntity_CommandPost::UpdatePFX
 *
 * Author        : Richard May
 *
 * Description   : Updates the position and effect for the PFX effects
 *
 *************************************************************/
void AVP_ClientEntity_CommandPost::UpdatePFX()
{
	Asura_Vector_3 xPosition( Asura_Maths::ZeroVector3 );
	Asura_Matrix_3x3 xOrientation( Asura_Maths::IdentityMatrix3x3 );

	// update pfx
	for( int j=0; j<2; ++j )
	{
		if( GetMarkerLocationFromHash( ((j == 0) ? uAVP_HASHID_MARKER_PFX_PULSE : uAVP_HASHID_MARKER_PFX_BEAM), &xPosition, &xOrientation ) ) 
		{				
			for( int i=0; i<Asura_Camera::GetNumberOfCameras(); ++i )
			{
				const Asura_Hash_ID* puPFXTemps = (j == 0) ? s_auPulsePFX : ((GetCurrentFaction() == AVP_Settings::AVP_TEAM_NONE) ? s_auBeamPFXNeutral : s_auBeamPFXCaptured);
				u_int* puPFXHandles = (j == 0) ? m_puPulsePFXHandles : m_puBeamPFXHandles;
			
				Asura_Hash_ID uPFXEffect = puPFXTemps[2];
				if( GetCurrentFaction() != AVP_Settings::AVP_TEAM_NONE )
				{
					AVP_Player* pxPlayer = AVP_Game_Core::GetPlayer();
					if( pxPlayer )
					{
						// If factions are opposing, enemy effect (0), otherwise friendly effect (1)
						uPFXEffect = puPFXTemps[(pxPlayer->GetMultiplayerTeam() == GetCurrentFaction())];

						// do we need to create a colourised version of this effect?
						if( !Asura_PFX_System::GetEffectTempByHash(uPFXEffect) )
						{
							const Asura_RGBA& xColour = (pxPlayer->GetMultiplayerTeam() == GetCurrentFaction()) ? xAVP_COLOUR_FACTION_FRIENDLY : xAVP_COLOUR_FACTION_ENEMY;
							if( !CreateColourisedPFXEffectTemplate( puPFXTemps[2], uPFXEffect, xColour ) )
							{
								Asura_Assert( false, "Failed to created colourised PFX effect for CP." ); 
							}
						}
					}
				}

				// update effect and position
				Asura_PFX_Entity_Effect* pxEffect = Asura_PFX_System::GetEntityEffect( puPFXHandles[i] );
				if( pxEffect )
				{
					// if the templates don't match...
					if( pxEffect->GetTemplate() != Asura_PFX_System::GetEffectTempByHash(uPFXEffect) )
					{
						// ...cast off this effect and create a new one below....
						Asura_PFX_System::DestroyEffect( puPFXHandles[i] );
					}
					else
					{
						// ...otherwise just update the pos and ori
						pxEffect->SetPosition( xPosition, xOrientation, true );
					}
				}

				// create new effect if necessary
				u_short uPFXFlags = static_cast<u_short> (1 << i); // camera index or'd with effect properties
				if( puPFXHandles[i] == ASURA_PFX_INVALID_EFFECT_ID )
				{
					puPFXHandles[i] = Asura_PFX_Entity_Effect::Create( xPosition, xOrientation, uPFXEffect, NULL, uPFXFlags );
				}
			}
		}
	}
}
/* end of function UpdatePFX */



/*************************************************************
 *
 * Name          : AVP_ClientEntity_CommandPost::UpdateAnims
 *
 * Author        : Richard May
 *
 * Description   : Updates the animations of the CP object & icon
 *
 *************************************************************/
void AVP_ClientEntity_CommandPost::UpdateAnims()
{
	static const u_int uAnimFlags = AAC_FLAG_ACTIVE;

	const float fCapturePercent = Asura_Maths::ClampToRange( 1.0f - (GetCaptureTimer() / GetCaptureTime()), 0.0f, 1.0f );

	// update main animation
	Asura_Animation* pxCPAnim = GetShapeInstance().GetAnimation();
	if( pxCPAnim ) 
	{
		// update opening anim progress
		{	
			Asura_AnimationController_Animation* pxOpenController = pxCPAnim->GetControllerAnimationFromHash(s_uOpenAnimID);
			if( !pxOpenController )
			{
				pxOpenController = pxCPAnim->CreateControllerAnimation( s_uOpenAnimID, uAnimFlags, 0, s_uOpenAnimID, NULL );
				pxOpenController->Play();
			}

			if( pxOpenController )
			{
				// 100% closed if neutral, otherwise use capture timer
				pxOpenController->SetParametricTimer( (GetCurrentFaction() == AVP_Settings::AVP_TEAM_NONE) ? fCapturePercent : (1.0f - fCapturePercent) );
			}
		}

		// update icon skin and animation
		const Asura_Hash_ID uIconSkin = s_auIconSkinIDs[GetCurrentFaction()];
		for( int i=0; i<Asura_Camera::GetNumberOfCameras(); ++i )
		{	
			// skin is always the same but colour varies depending on viewer
			Asura_Animation*& pxIconAnim = m_pxIconAnims[i];
			
			if( uIconSkin != ASURA_HASH_ID_UNSET )
			{		
				if( !pxIconAnim )
				{
					pxIconAnim = new Asura_Animation( uIconSkin );
					pxIconAnim->m_xPrimaryWrapper.StartAnim( s_uIconAnimID, true, 0.0f );					
					//pxIconAnim->SetAlpha( 0.0f, true );
				}

				if( pxIconAnim )
				{				
					const Asura_RGBA* pxColour = &xAVP_COLOUR_FACTION_NEUTRAL;
					
					if( GetCurrentFaction() != AVP_Settings::AVP_TEAM_NONE )
					{
						AVP_Player* pxPlayer = AVP_Game_Core::GetPlayer();
						if( pxPlayer )
						{
							pxColour = (pxPlayer->GetMultiplayerTeam() == GetCurrentFaction()) ? &xAVP_COLOUR_FACTION_FRIENDLY : &xAVP_COLOUR_FACTION_ENEMY;
						}
					}

					// If we're alpha'ing up, do so quickly, with a fixed timestep. If we're alpha'ing down, base alpha on capture timer
					//float fTargetAlpha  = (GetCurrentFaction() == AVP_Faction::FACTION_NEUTRAL) ? 0.0f : (1.0f - fCapturePercent);
					//float fCurrentAlpha = pxIconAnim->GetAlpha();
					//if( fTargetAlpha > fCurrentAlpha )
					//{
					//	float fDiff = fTargetAlpha - fCurrentAlpha;
					//	fCurrentAlpha += Asura_Maths::Min( fDiff, Asura_Maths::Sign(fDiff) * dbg_fIconAlphaDelta * Asura_Timers::GetGameTimeSlice() ); 
					//}
					//else
					//{
					//	fCurrentAlpha = fTargetAlpha;
					//}
					//
					//pxIconAnim->SetAlpha( fCurrentAlpha );
					pxIconAnim->SetRGB( pxColour->m_fR, pxColour->m_fG, pxColour->m_fB, true );					
				}
			}
			else
			{
				if( pxIconAnim )
				{
					delete pxIconAnim;
					pxIconAnim = NULL;
				}
			}
		}
	}
}
/* end of function UpdateAnim */



/*************************************************************
 *
 * Name          : AVP_ClientEntity_CommandPost::UpdateBeam
 *
 * Author        : Richard May
 *
 * Description   : Updates the beam for the CP object
 *
 *************************************************************/
void AVP_ClientEntity_CommandPost::UpdateBeam()
{
	Asura_Vector_3 xNearPos( Asura_Maths::ZeroVector3 );
	Asura_Matrix_3x3 xOrientation( Asura_Maths::IdentityMatrix3x3 );

	if( GetMarkerLocationFromHash( uAVP_HASHID_MARKER_BEAM, &xNearPos, &xOrientation ) )
	{
		for( int i=0; i<Asura_Camera::GetNumberOfCameras(); ++i )
		{
			// If factions are opposing, enemy effect (0), otherwise friendly effect (1)
			Asura_Beam_Template* pxBeamTemplate = &(s_axBeamTemplates[2]);
			
			if( GetCurrentFaction() != AVP_Settings::AVP_TEAM_NONE)
			{
				AVP_Player* pxPlayer = AVP_Game_Core::GetPlayer();
				if( pxPlayer )
				{			
					pxBeamTemplate = &(s_axBeamTemplates[(pxPlayer->GetMultiplayerTeam() == GetCurrentFaction())]);			
				}
			}

			if( pxBeamTemplate->iTileHandle == -1 )
			{
				pxBeamTemplate->iTileHandle = Asura_TiledTextureManagement::GetHandleOfTile( pxBeamTemplate->uTileHashID );
			}

			// allocate beam, if necessary
			Asura_Beam* pxBeam = Asura_Beam_System::GetBeamByID(m_puBeamIDs[i]);
			if( !pxBeam )
			{					
				if( m_fBeamLength == -1 )
				{
					const Asura_LOS_InputData xInput( xNearPos, Asura_Vector_3( 0.0f, -1.0f, 0.0f ), GetGuid(), dbg_fBeamLOSRange );
					const Asura_LOS_OutputData* pxOutput = Asura_LOS::TestAgainstEnvironment( &xInput );					
					m_fBeamLength = pxOutput->m_fDistanceToIntersection - pxBeamTemplate->fInitialRadius;
				}

				Asura_Vector_3 xEndPos( xNearPos );
				xEndPos.y -= m_fBeamLength;

				pxBeam = Asura_Beam_System::CreateBeam( xNearPos, xEndPos, *pxBeamTemplate );
				if( pxBeam )
				{
					//pxBeam->SetCameraFlags( 1 << i );
					m_puBeamIDs[i] = pxBeam->GetID();
				}
			}

			if( pxBeam )
			{			
				float fCapturePercent = Asura_Maths::ClampToRange( 1.0f - (GetCaptureTimer() / GetCaptureTime()), 0.0f, 1.0f );
				float fRadiusScale	= dbg_fBeamMinRadiusScale + (((GetCurrentFaction() == AVP_Settings::AVP_TEAM_NONE) ? fCapturePercent : (1.0f - fCapturePercent)) * (1.0f - dbg_fBeamMinRadiusScale));

				// template has changed, restart beam from base
				float fLength = pxBeam->GetLength();
				if( pxBeam->GetTemplate() != pxBeamTemplate )
				{
					fLength = 0.1f; 

					// trigger radius pulse					
					m_fPulseTimer = dbg_fBeamPulseDuration;
				}
				else
				{
					float fDiff = m_fBeamLength - fLength;
					fLength += Asura_Maths::Min( fDiff, dbg_fBeamLengthDelta * Asura_Timers::GetGameTimeSlice() );

					// during pulse, scale radius outward and then back again
					if( m_fPulseTimer > 0.0f )
					{
						m_fPulseTimer = Asura_Maths::ClampToRange( (m_fPulseTimer - Asura_Timers::GetGameTimeSlice()), 0.0f, dbg_fBeamPulseDuration );
						fRadiusScale  *= 1.0f + (dbg_fBeamPulseRadiusScale * Asura_Maths::Sine( (m_fPulseTimer / dbg_fBeamPulseDuration) * Asura_Maths::Pi ));
					}										
				}

				pxBeam->SetPositions( xNearPos, Asura_Vector_3( 0.0f,-1.0f,0.0f), fLength );
				pxBeam->SetTailRadiusScale( fRadiusScale );
				pxBeam->SetHeadRadiusScale( fRadiusScale );
				//pxBeam->SetTemplate( pxBeamTemplate );
				pxBeam->SetLifeTime( 1.0f );
			}
		}
	}
}
/* end of function UpdateBeam */



/*************************************************************
 *
 * Name          : AVP_ClientEntity_CommandPost::UpdateSound
 *
 * Author        : Richard May
 *
 * Description   : Updates the position and pitch/volume of CP hum 
 *
 *************************************************************/
void AVP_ClientEntity_CommandPost::UpdateSound()
{
/*
	if( m_iHumSoundHandle != ASURA_INVALID_HANDLE && !Asura_Sound_System::IsSphereInRange( GetCurrentPosition(), Asura_Sound_Event_System::GetOuterRadius( m_iHumSoundHandle ) ) )
	{
		Asura_Sound_Event_System::Stop( m_iHumSoundHandle );
	}
	if( m_fCaptureTimer < m_fCaptureTime )
	{
		if ( m_iHumSoundHandle == ASURA_INVALID_HANDLE )
		{
			const Asura_Sound_Event* pxEvent = ASURA_SOUND_EVENT_LOOKUP( 0x6E926355, "command_post_hum" );
			Asura_Assert( pxEvent, "Command post hum Sound, sound event lookup failed" );
			if ( pxEvent )
			{
				if ( Asura_Sound_System::IsSphereInRange( GetCurrentPosition(), pxEvent->GetOuterRadius() ) )
				{
					Asura_Sound_Event_System::PlayEvent( pxEvent, &m_iHumSoundHandle, &GetCurrentPosition());
				}
			}
		}	
		if( m_iHumSoundHandle != ASURA_INVALID_HANDLE )
		{		
			float fPitchScale = (m_fCaptureTime != 0.0f) ? Asura_Maths::ClampToRange( (m_fCaptureTimer / m_fCaptureTime), 0.0f, 1.0f ) : 1.0f;
			if( GetCurrentFaction() == AVP_Faction::FACTION_NEUTRAL )
			{
				fPitchScale = 1.0f - fPitchScale;
			}

			fPitchScale = dbg_fHumMinPitch + ((dbg_fHumMaxPitch - dbg_fHumMinPitch) * fPitchScale);

			Asura_Sound_Event_System::SetVolumeScale( m_iHumSoundHandle, 1.0f );
			Asura_Sound_Event_System::SetPitchScale( m_iHumSoundHandle, fPitchScale );
		}
	}
	else if( m_iHumSoundHandle != ASURA_INVALID_HANDLE )
	{
		// fade off hum sound to zero and then stop it
		float fVolumeScale = Asura_Sound_Event_System::GetVolumeScale( m_iHumSoundHandle );
		if( fVolumeScale > 0.0f )		
		{
			fVolumeScale = Asura_Maths::ClampToRange( fVolumeScale - (Asura_Timers::GetGameTimeSlice() * dbg_fHumSoundFadeOutTime), 0.0f, 1.0f );
			Asura_Sound_Event_System::SetVolumeScale( m_iHumSoundHandle, fVolumeScale );
		}
		else
		{
			Asura_Sound_Event_System::Stop( m_iHumSoundHandle );
		}		
	}
*/
}
/* end of function UpdateSound */



/*************************************************************
*
* Name          : AVP_ClientEntity_CommandPost::Render
*
* Author        : Richard May
*
* Description   : Handles any command post-specific messages
*
*************************************************************/
void AVP_ClientEntity_CommandPost::Render( void )
{
	PARENT::Render();

	Asura_Vector_3 xIconPos( GetCurrentPosition() );
	Asura_Matrix_3x3 xIconOri( GetCurrentOrientation() );

	const u_int uRenderFlags = ASURA_HIERARCHY_RENDER_NO_SHADOW | ASURA_HIERARCHY_RENDER_FORCE_AMBIENT_COLOUR_ONLY;
	if( GetMarkerLocationFromHash( uAVP_HASHID_MARKER_LOGO, &xIconPos, &xIconOri ) )
	{
		for( int i=0; i<Asura_Camera::GetNumberOfCameras(); ++i )
		{
			if( m_pxIconAnims[i] )						
			{
				Asura_Camera::SetCameraIndex(i);
				Asura_Hierarchy_System::SetJustRenderForCurCamFlag( true );
				m_pxIconAnims[i]->Update( xIconPos, xIconOri, ANIMTIME_USEGAMETIME, uRenderFlags );
				Asura_Hierarchy_System::SetJustRenderForCurCamFlag( false );
			}
		}
	}

	if( dbg_bRenderCommandPostText )
	{
		// render debug info for now
		Asura_Vector_3 xRenderPos = GetCurrentPosition();
		xRenderPos.y -= 2.0f;

		Asura_Camera::ProjectWorldCoordsToScreenSpace( xRenderPos, false );
		
		if( (xRenderPos.z > 0.0f) && (xRenderPos.z < 30.0f) )
		{		
			const u_int uFormatFlags = ASURA_TEXTFORMAT_CENTREJUSTIFIED | ASURA_TEXTFORMAT_FORMATSTRING | ASURA_TEXTFORMAT_RELATIVEPOSITION;
			const float fScale = Asura_Maths::Min( 10.0f, (6.0f / xRenderPos.z) );
			const float fAlpha = 1.0f - (Asura_Maths::ClampToRange( (xRenderPos.z - 25.0f), 0.0f, 5.0f ) / 5.0f);
			const Asura_RGBA xColour( 1.0f, 1.0f, 0.0f, fAlpha );

			if (m_iCurrentFaction == AVP_Settings::AVP_TEAM_NONE && 
				m_iCapturingFaction != AVP_Settings::AVP_TEAM_NONE)
			{
				const char* szCapturingFactionString = AVP_Settings::GetTeamNames()[m_iCapturingFaction];
				Asura_Font_System::Print( xRenderPos.x, xRenderPos.y, fScale, xColour.ToUIntARGB(), uFormatFlags, "Neutral (Capturing: %s%s)", szCapturingFactionString, IsContested() ? " *Contested*" : ""  );		
			}
			else
			{
				const char* szOwnerString = AVP_Settings::GetSpeciesNames()[m_iCurrentFaction];
				Asura_Font_System::Print( xRenderPos.x, xRenderPos.y, fScale, xColour.ToUIntARGB(), uFormatFlags, "Owner: %s%s", szOwnerString, IsContested() ? " *Contested*" : ""  );		
			}
			xRenderPos.y = Asura_Font_System::GetNextLinesYCoordinate();
			Asura_Font_System::Print( xRenderPos.x, xRenderPos.y, fScale, xColour.ToUIntARGB(), uFormatFlags, "Timer: %3.1f", Asura_Maths::FtoD(m_fCaptureTimer) );

			//if( GetName() )
			//{
			//	xRenderPos.y = Asura_Font_System::GetNextLinesYCoordinate();
			//	Asura_Font_System::Print( xRenderPos.x, xRenderPos.y, fScale, xColour.ToUIntARGB(), uFormatFlags, L"%ls", GetName() );
			//}
		}
		// JG - 03/01/07: we don't need this call now that we are keeping material set up and our internal material type in sync
		//Asura_Render::FlushDisplay();
	}
}
/* end of function Render */



/*************************************************************
 *
 * Name          : AVP_ClientEntity_CommandPost::HandleMessage
 *
 * Author        : Richard May
 *
 * Description   : Handles any command post-specific messages
 *
 *************************************************************/
bool AVP_ClientEntity_CommandPost::HandleMessage( const Asura_Message* pxMessage )
{
	switch( pxMessage->GetMsgID() )
	{
		case ASURA_MSG_CLIENTCREATE:
		{
			const AVP_Message_CommandPost_ClientCreate* pxData = static_cast<const AVP_Message_CommandPost_ClientCreate*> (pxMessage->GetData());
			if( pxData )
			{
				ProcessClientCreate( pxData, pxMessage->GetSentTime() );
			}

			return true;
		}
		case ASURA_MSG_CLIENTUPDATE:
		{
			const AVP_Message_CommandPost_ClientUpdate* pxData = static_cast<const AVP_Message_CommandPost_ClientUpdate*> (pxMessage->GetData());
			if( pxData )
			{
				PARENT::ProcessClientUpdate( pxData, pxMessage->GetSentTime() );
				pxData->ReadFromMessage( this, pxMessage->GetSentTime() );
			}

			return true;
		}
		default : break;
	}

	return PARENT::HandleMessage( pxMessage );
}
/* end of function HandleMessage */

void AVP_ClientEntity_CommandPost::ProcessClientCreate(const AVP_Message_CommandPost_ClientCreate* pxData,
														 float fServerTime,
														 bool bProcessPosAndOri)
{
	PARENT::ProcessClientCreate(pxData, fServerTime, bProcessPosAndOri);

	SetNameHashID( pxData->m_uNameHashID );
}

/*************************************************************
 *
 * Name          : AVP_ClientEntity_CommandPost::SetCurrentFaction
 *
 * Author        : Richard May
 *
 * Description   : Accessor for setting the current faction. Makes SFX, etc. 
 *				   when the faction changes. Also increments medals BJS
 *
 *************************************************************/
void AVP_ClientEntity_CommandPost::SetCurrentFaction( int iNewFaction )
{	
	int iOldFaction = m_iCurrentFaction;
	if( iOldFaction == iNewFaction ) return;

	DecrementNumberOwnedByTeam( iNewFaction );
	IncrementNumberOwnedByTeam( iNewFaction );
	
	if(iNewFaction != AVP_Settings::AVP_TEAM_NONE)
	{
		AVP_Player* pxPlayer = AVP_Game_Core::GetPlayer();
		AVP_ClientActor_Vertebrate* pxVertebrate = pxPlayer ? AVP_ClientActor_Vertebrate::FindEntity(pxPlayer->GetPuppetGuid()): NULL;
		if( pxVertebrate && m_uCapturingGuid == pxVertebrate->GetGuid())
		{
			// TODO: Scoreboard here?
			//AVP_Settings::GetPlayerProfile()->GetPlayerCareer().IncrementAttribute(AVP_CAREER_COMMAND_POSTS);
		}
	}

	m_iCurrentFaction = iNewFaction;

	// if the faction was previously 'number factions', it means this is the first ever update 
	// we've handled so don't do any SFX, etc.
	if( iOldFaction == AVP_Settings::AVP_NUM_TEAMS ) return;

	// only do SFX if the local player controller has been created and joined 
	// a team already. This should mean that you don't get spam when the objects are first created.
	// TODO: check if the player has a puppet here? Sounds like the right thing to do in AVP
	AVP_ClientEntity_PlayerInfo* pxLocalPlayerInfo = AVP_ClientEntity_PlayerInfo::FindEntity( Asura_Network::GetClientID() );
	if( pxLocalPlayerInfo )
	{
		// trigger change SFX
		Asura_Sound_Event_System::PlayEvent( ASURA_SOUND_EVENT_LOOKUP( 0x5412293B, "command_post_change" ), NULL, &GetCurrentPosition() );
	}
}
/* end of function SetCurrentFaction */



//void AVP_ClientEntity_CommandPost::GetRadarInfo(u_int uFaction, u_int uTargetGuid, AVP_Radar_Info& xRadarInfo)const
//{	
//	xRadarInfo.m_uColour = GetRadarColour(uFaction);
//	xRadarInfo.m_bCalculateAngle = false;
//	xRadarInfo.m_bFlash = false;
//	xRadarInfo.m_bRender = IsEntityActive();
//	xRadarInfo.m_bRenderAtEdge = true;
//	xRadarInfo.m_uTile = AVP_HUD_Data::HUD_RADAR_BLIP;
//}



/*************************************************************
 *
 * Name          : AVP_ClientEntity_CommandPost::OnEntityActivate
 *
 * Author        : Richard May
 *
 * Description   : Override to add from the HUD list as necessary
 *
 *************************************************************/
void AVP_ClientEntity_CommandPost::OnEntityActivate()
{
	PARENT::OnEntityActivate();
}
/* end of function OnEntityActivate */



/*************************************************************
*
* Name          : AVP_ClientEntity_CommandPost::OnEntityDeactivate
*
* Author        : Richard May
*
* Description   : Override to remove from the HUD list as necessary
*
*************************************************************/
void AVP_ClientEntity_CommandPost::OnEntityDeactivate()
{
	PARENT::OnEntityDeactivate();
}
/* end of function OnEntityDeactivate */



/*************************************************************
 *
 * Name          : AVP_ClientEntity_CommandPost::Create
 *
 * Author        : Richard May
 *
 * Description   : Static factory function for command posts
 *
 *************************************************************/
Asura_Entity* AVP_ClientEntity_CommandPost::Create( const Asura_Message* pxMessage )
{	
	AVP_ClientEntity_CommandPost* pxNewCmdPost = new AVP_ClientEntity_CommandPost();
	if( pxNewCmdPost )
	{
		pxNewCmdPost->HandleMessage( pxMessage );
	}

	return pxNewCmdPost;
}
/* end of function Create */

// (MH - 15.12.2006) only works if doing client-side imter computation
//bool AVP_ClientEntity_CommandPost::IsCaptureInProgress( void ) const
//{
//	return (m_fCaptureTimer < AVP_ServerEntity_CommandPost::GetCaptureTime());
//}

u_int AVP_ClientEntity_CommandPost::GetNumberOwnedByTeam( const AVP_Settings::Team eTeam )
{
	u_int uRtn(0);

	if( eTeam < AVP_Settings::AVP_NUM_TEAMS )
	{
		uRtn = s_auNumberOwnedByTeam[eTeam];		
	}
	else if( eTeam == AVP_Settings::AVP_TEAM_NONE )
	{
		uRtn = GetInstanceCount() - ( s_auNumberOwnedByTeam[AVP_Settings::AVP_TEAM_ONE] + 
									  s_auNumberOwnedByTeam[AVP_Settings::AVP_TEAM_TWO] +
									  s_auNumberOwnedByTeam[AVP_Settings::AVP_TEAM_THREE] );
	}

	return Asura_Maths::ClampToRange( uRtn,0u,GetInstanceCount() );
}



void AVP_ClientEntity_CommandPost::DecrementNumberOwnedByTeam( const u_int uTeam )
{
	if( uTeam < AVP_Settings::AVP_NUM_TEAMS )
	{
		--s_auNumberOwnedByTeam[uTeam];		
	}
}
	


void AVP_ClientEntity_CommandPost::IncrementNumberOwnedByTeam( const u_int uTeam )
{
	if( uTeam < AVP_Settings::AVP_NUM_TEAMS )
	{
		++s_auNumberOwnedByTeam[uTeam];
	}
}

inline void AVP_ClientEntity_CommandPost::KillHumSound()
{
	Asura_Sound_Event_System::Stop( m_iHumSoundHandle );
}

void AVP_ClientEntity_CommandPost::KillAllHumSounds()
{
	Asura_Hashed_List_It<AVP_ClientEntity_CommandPost> xIter( AVP_ClientEntity_CommandPost::GetList() );

	while( !xIter.Done() )
	{
		AVP_ClientEntity_CommandPost* pxCurrent = xIter.GetCurrent();
		pxCurrent->KillHumSound();
		xIter.Next();
	}
}


void AVP_ClientEntity_CommandPost::Pause()
{
	s_bIsPaused = true;

	KillAllHumSounds();
}

void AVP_ClientEntity_CommandPost::Unpause()
{
	s_bIsPaused = false;
}


/***************/
/* End of File */
/***************/
