//
// AVP_ServerEntity_CommandPost.h
//
// Created by RDM - 21 November 2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __AVP_SERVERENTITY_COMMANDPOST_H__
#define __AVP_SERVERENTITY_COMMANDPOST_H__

/************/
/* Includes */
/************/

// asura

// AVP
#include "Core/AVP_Settings.h"
#include "Entity/AVP_Entity_Classification.h"
#include "Entity/PhysicalObject/PhysicsObject/AVP_ServerEntity_PhysicsObject.h"
#include "Entity/PhysicalObject/PhysicsObject/AVP_PhysicsObject_Messages.h"

/*************/
/* Constants */
/*************/

static const float fAVP_COMMAND_POST_MAX_CAPTURE_TIME = 300.0f;

#define AVP_COMMANDPOST_MAX_NUM_LINKED_GUARDZONES 4

/***********/
/* Classes */
/***********/

// prototypes
class AVP_ClientEntity_CommandPost;
class AVP_ServerEntity_Actor;
class AVP_ServerActor_Vertebrate;

class AVP_CommandPost_Node_ArrayElement
{
public:
	AVP_CommandPost_Node_ArrayElement()
	{
		m_uGuid = ASURA_GUID_UNREGISTERED;
		m_uProbability = 0;
	}

	Asura_Guid m_uGuid;
	u_int m_uProbability;
};

class AVP_CommandPost_Node_Array
{
public:
	AVP_CommandPost_Node_Array();
	~AVP_CommandPost_Node_Array() {}

	static const int s_uMAX_NODES = 10;

	bool		AddNode( Asura_Guid uGuid, u_int m_uProbability );
	bool		RemoveNode( Asura_Guid uGuid );
	Asura_Guid	ValidateNodes() const;
	bool		IsEmpty() const;
	bool		IsFull() const;
	int			FindNode( Asura_Guid uGuid ) const;
	bool		HasNode( Asura_Guid uGuid ) const;
	u_int&		GetNodeProbability( u_int iIndex );
	u_int		GetSumOfProbabilities() const;
	// RWM - moved to weights rather than percentages, so validation unnecessary
	bool		HasValidProbabilities()	const { return true; } //( ( GetSumOfProbabilities() <= 100 ) || ( IsEmpty() ) ); }
	
	const AVP_CommandPost_Node_ArrayElement& GetNode( u_int iIndex ) const;

	void WriteToChunkStream( Asura_Chunk_Stream& xStream ) const;
	void ReadFromChunkStream( Asura_Chunk_Stream& xStream );

protected:
	int GetFreeElement();

	AVP_CommandPost_Node_ArrayElement m_axChildNodes[s_uMAX_NODES];
	u_int m_uFreeNodes;
};

/*************************************************************
 *
 * Class Name    : AVP_ServerEntity_CommandPost
 *
 * Author        : Richard May
 *
 * Description   : BF3's command post object for conquest mode
 *
 *************************************************************/
class AVP_ServerEntity_CommandPost : public AVP_ServerEntity_PhysicsObject
{
	friend class ToolsAVP_Entity_CommandPost;

private:
	typedef AVP_ServerEntity_PhysicsObject PARENT;

public:

	enum AVP_CommandPost_Position_Type
	{
		CP_POSITION_TYPE_HANGAR,
		CP_POSITION_TYPE_ION_CANNON,
		CP_POSITION_TYPE_STANDARD,
		CP_POSITION_TYPE_CORE,
		NUM_CP_POSITION_TYPES,
	};

	/* Public interface functions. */
	virtual void Update( void );
	virtual bool HandleMessage( const Asura_Message* pxMsg );
	virtual void MakeClientUpdateMessage( void );
	virtual void MakeCreateMessage(Asura_Guid uRequestingClient, bool bSendGuaranteed = false);

	// load and save
	virtual void WriteToChunkStream( Asura_Chunk_Stream& xStream ) const;
	virtual void ReadFromChunkStream( Asura_Chunk_Stream& xStream );
	virtual void PostLoadInitialise();

	// accessors
	bool  IsBeingCaptured( void ) const         { return (m_fCaptureTimer < m_fCaptureTime); }
	bool  IsContested( void ) const				{ return (m_fContestedTimer > 0.0f); }
	bool  IsCaptureDisabled( void ) const		{ return m_bCaptureDisabled; }
	bool  IsCaptureByBotDisabled( void ) const	{ return m_bCaptureByBotDisabled; }
	bool  IsDefaultSpawnPoint( void ) const		{ return m_bDefaultSpawnPoint; }
	bool  IsMobileCommandPost( void ) const		{ return m_bMobileCommandPost; } 
	bool  IsUsingOrientation( void ) const		{ return m_bUsesOrientation; }
	
	Asura_Hash_ID GetNameHashID( void ) const	{ return m_uLocalisedNameHashID; }

	float GetCaptureTime( void ) const			{ return m_fCaptureTime; }
	float GetCaptureTimer( void ) const			{ return m_fCaptureTimer; }
	float GetLastCapturedSpawnTime() const      { return m_fLastCapturedSpawnTime; }
	float GetSpawnProbabilityValue( void ) const{ return m_fSpawnProbabilityValue; }
	int	GetCurrentFaction( void ) const			{ return m_iCurrentFaction; }
	int GetCapturingFaction( void ) const		{ return m_iCapturingFaction; }
	u_int GetCapturingGuid(void)const			{ return m_uCapturingGuid;}
	AVP_Settings::Team GetInitialTeam(void) const { return m_eInitialTeam; }
	float GetHoldTimer( void ) const			{ return m_fHoldTimer; }
	
	bool  HasLinkedGuardZone( u_int iIndex = 0 ) const { return ( m_auLinkedGuardZoneGuids[iIndex] == ASURA_GUID_UNREGISTERED ) ? false : true;  }
	Asura_Guid GetLinkedGuardZoneGuid( u_int uIndex ) const { return m_auLinkedGuardZoneGuids[uIndex]; }

	bool GetSpawnPositionAndOri( Asura_Vector_3& xPos, Asura_Matrix_3x3& xOri, bool bIncludeChildren = false ) const;
	
	const AVP_CommandPost_Node_Array&  GetChildNodes( void ) const { return m_xChildNodes; }
	
	void  SetNameHashID( Asura_Hash_ID uNameHashID )	{ m_uLocalisedNameHashID = uNameHashID; }
	void  SetInitialTeam( AVP_Settings::Team eTeam ); // owner at start of level.
	void  SetCaptureDisabled( bool bDisable )			{ m_bCaptureDisabled = bDisable; }
	void  SetCaptureByBotDisabled( bool bDisable )		{ m_bCaptureByBotDisabled = bDisable; }
	void  SetMobileCommandPost( bool bMobile )			{ m_bMobileCommandPost = bMobile; }
	void  SetDefaultSpawnPoint( bool bDefault )			{ m_bDefaultSpawnPoint = bDefault; }
	void  SetUsesOrientation( bool bUsesOrientation )	{ m_bUsesOrientation = bUsesOrientation; }
	void  SetSpawnProbabilityValue( float fValue )		{ m_fSpawnProbabilityValue = fValue; }
	void  IncrementSpawnProbabilityValue( float fValue ){ m_fSpawnProbabilityValue += fValue; }
	void  SetHoldTimer( float fHoldTimer )				{ m_fHoldTimer = fHoldTimer; }
	void  SetLastCapturedSpawnTime( const float fTime ) { m_fLastCapturedSpawnTime = fTime; }

	/* Static functions. */
	static AVP_ServerEntity_CommandPost* FindNearestEntity( const Asura_Vector_3& xPosition, int uFaction );
	static AVP_ServerEntity_CommandPost* FindEntity( Asura_Guid uGuid )		{ return s_xList.FindEntity( uGuid ); }
	static Asura_Hashed_List<AVP_ServerEntity_CommandPost>* GetList( void )	{ return &s_xList; }
	static AVP_ServerEntity_CommandPost* Create( Asura_Guid uGuid )			{ return new AVP_ServerEntity_CommandPost(uGuid); }
	static float GetMaxHoldTime( void )										{ return s_fHoldTime; }
	static bool IsCaptureDisabledForCurrrentGametype();
	static void DebugCommandPosts( void );

	static const char*	s_apszPositionTypeNames[NUM_CP_POSITION_TYPES];

#ifdef ASURA_TOOLS
	virtual void ApplyChangesForTemplateInstance(class Tools_EntityTemplateInstance* pxInstance);
	virtual bool CanBeIncludedInTemplate() const {return Classification == AVP_ENTITYCLASS_COMMANDPOST;}
#endif

protected:
	/* Constructors/destructor. */
	AVP_ServerEntity_CommandPost( Asura_Guid uGuid = ASURA_GUID_UNREGISTERED );
	virtual ~AVP_ServerEntity_CommandPost();

	/* Helper functions. */		
	virtual void OnEntityActivate();
	virtual void OnEntityDeactivate();

	void UpdateHoldTimer();

	void Capture( int iCapturingFaction );
	bool CanCapture( const AVP_ServerActor_Vertebrate* pxVertebrate ) const;
	const Asura_Matrix_3x3 GetSpawnOrientation() const;

	/* Data. */	

	// data set in tools
	AVP_Settings::Team m_eInitialTeam;
	float m_fHoldTimer;
	float m_fCaptureTime;
	AVP_CommandPost_Node_Array m_xChildNodes;
	Asura_Guid m_auLinkedGuardZoneGuids[AVP_COMMANDPOST_MAX_NUM_LINKED_GUARDZONES];
	Asura_Hash_ID	m_uLocalisedNameHashID;

	// state data
	int m_iCurrentFaction; 
	int m_iCapturingFaction;
	float m_fContestedTimer;
	float m_fCaptureTimer;		// 0 = change state, GetCaptureTime = owned by current faction, in between = moving towards capturing faction.
	u_int m_uCapturingGuid;
	float m_fLastCapturedSpawnTime; //the time an AI spawn here while it was being captured.

	float m_fSpawnProbabilityValue; // This is the chance of an AI being spawned at this point. Controlled and set by the Commander owning this CP.
	
	union
	{
		struct
		{
			u_int m_bCaptureDisabled	  : 1,
				  m_bCaptureByBotDisabled : 1,
				  m_bMobileCommandPost	  : 1,
				  m_bDefaultSpawnPoint	  : 1,
				  m_bUsesOrientation	  : 1,
				  _CMD_POST_UNUSED		  : 25;
		};

		u_int m_uCommandPostFlags;
	};

	/* Static Data. */
	static Asura_Hashed_List<AVP_ServerEntity_CommandPost> s_xList;
	static const float s_fCaptureRadius;
	static const float s_fHoldTime;	

	#ifdef TEST_COMMAND_POST_SPAWNER
	mutable u_int m_uCount[AVP_CommandPost_Node_Array::s_uMAX_NODES];
	#endif //__TEST_COMMAND_POST_SPAWNER__
};
/* end of class AVP_ServerEntity_CommandPost */

class AVP_Message_CommandPost_ClientCreate : public AVP_Message_PhysicsObject_ClientCreate
{
private:
	typedef AVP_Message_PhysicsObject_ClientCreate PARENT;
	
public:
	void FillCreateCommandPost( const AVP_ServerEntity_CommandPost* pxCmdPost )
	{
		PARENT::FillCreatePhysicalObject( pxCmdPost ); 
		m_uNameHashID = pxCmdPost->GetNameHashID();
	}

	Asura_Hash_ID	m_uNameHashID;
};

class AVP_Message_CommandPost_ClientUpdate : public AVP_Message_PhysicsObject_ClientUpdate
{
private:
	typedef AVP_Message_PhysicsObject_ClientUpdate PARENT;
	
public:
	void WriteToMessage( const AVP_ServerEntity_CommandPost* pxCmdPost );
	void ReadFromMessage( AVP_ClientEntity_CommandPost* pxCmdPost, float fSentTime ) const;

#ifdef ASURA_SUPPORT_NETMSG_SYSTEM
	static void RegisterFields(u_short usMsgID, u_short usEntityClassification);
	static void RegisterFields(class Asura_NetMsg* pxNetMsg);
	static u_char GetNumFields()
	{
		return static_cast<u_char> (6 + PARENT::GetNumFields());
	}
#endif//#ifdef ASURA_SUPPORT_NETMSG_SYSTEM

protected:
	float m_fCaptureTime;
	float m_fCaptureTimer;
	u_int m_uCapturingGuid;
	u_int m_uCurrentFaction;
	u_int m_uCapturingFaction;

	union
	{
		struct
		{
			u_int	m_bContested			: 1,
					m_bCaptureDisabled		: 1,
					m_bDefaultSpawnPoint	: 1,
					m_bMobileCommandPost	: 1;
		};
		u_int m_uCommandPostMsgFlags;
	};
};


/***************/
/* End of File */
/***************/

#endif//__AVP_SERVERENTITY_COMMANDPOST_H__
