
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_ClientEntity_Gib.cpp
// Author : Paul Humphries
// Date   : 06-Feb-2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef AVP_CLIENTENTITY_GIB_H___
#define AVP_CLIENTENTITY_GIB_H___

// Asura Files (engine independent)
#include "Physics/Asura_Physics_Object.h"

// Project file (engine independent)
#include "Entity/PhysicalObject/AVP_ClientEntity_PhysicalObject.h"

#include "Blueprint/Asura_Blueprint.h"

// Classes
class AVP_ClientEntity_Gib : public AVP_ClientEntity_PhysicalObject
{
public:
	// These don't need to be stored, we just create and attach them immediately.
	struct PFXAttachData
	{
		Asura_Hash_ID m_uPFXID;
		Asura_Hash_ID m_uMarkerID;
	};

	static AVP_ClientEntity_Gib* Create(Asura_Guid uOwnerGuid, const Asura_Vector_3& xPosition, const Asura_Matrix_3x3& xOrientation, const Asura_Vector_3& xVelocity, const Asura_Hash_ID uSkinHash, const Asura_Vector_3& xAttackImpulse, const Asura_Hash_ID uHitPFXHash, const Asura_Hash_ID uHitDecalHash, const PFXAttachData* pxPFXAttachData, const u_int uNumPFXToAttach);

 	void SetBloodHash(Asura_Hash_ID uBlood)					{ m_uBloodHash = uBlood; }
 	void SetBloodHitPFXHash(Asura_Hash_ID uBloodHitPFXHash) { m_uBloodHitPFXHash = uBloodHitPFXHash; }

	virtual	void Update(void);
	virtual void Render(void);

	Asura_Hash_ID GetSkinHash() const { return m_uSkinHash; }
	bool IsRagdoll () const;

	virtual bool HandleMessage(const Asura_Message* pxMessage);

	// Function to get all the pfx details, without forcing other classes to do the messing about currently required.
	static void FillPFXDetailsFromBlueprint(const Asura_Blueprint* pxBlueprint, AVP_ClientEntity_Gib::PFXAttachData* pxPFXDataStorage, u_int& uNumEntries);

	static AVP_ClientEntity_Gib* FindEntity(const Asura_Guid uGuid)	{ return s_xList.FindEntity(uGuid); }
	static const Asura_Hashed_List<AVP_ClientEntity_Gib>* GetList()	{ return &s_xList; }

protected:
	void OnApplyDamage(const AVP_EntityDamage_ApplyToEntity& xDamage);
	
	AVP_ClientEntity_Gib(void);
	virtual ~AVP_ClientEntity_Gib();

	float m_fAge;

	const Asura_Collision_Box* m_pxCollisionBox;

	Asura_Hash_ID m_uBloodHash;
	Asura_Hash_ID m_uBloodHitPFXHash;
	Asura_Hash_ID m_uSkinHash;
	
	Asura_Handle m_iRagdollHandle;

private:
	typedef AVP_ClientEntity_PhysicalObject PARENT;
	static Asura_Hashed_List<AVP_ClientEntity_Gib> s_xList;
};


// Class to enumerate the PFX attachment data. Can be used to do both the effects and the marker points, but not both at once.
const u_int uAVP_VERTEBRATE_GIB_MAX_PFX_ATTACHMENTS = 16;

class AVP_Blueprint_ParameterEnumerator_EntityGibPFX : public Asura_Blueprint_ParameterEnumerator
{
	typedef Asura_Blueprint_ParameterEnumerator PARENT;

public:
	AVP_Blueprint_ParameterEnumerator_EntityGibPFX(AVP_ClientEntity_Gib::PFXAttachData* pxPFXDataStorage, u_int& uNumEntries, bool bStoringMarkerPoints);
	virtual ~AVP_Blueprint_ParameterEnumerator_EntityGibPFX() {};

	virtual void BeginEnumeration(void);
	virtual bool Enumerate(const u_int uIndex, char const* const szName, const Asura_Value& xValue);
	virtual void EndEnumeration(const bool bSuccess);

	void SetStoringMarkerPoints(bool bStoringMarkers);

protected:
	AVP_ClientEntity_Gib::PFXAttachData* m_pxPFXDataStorage;
	u_int& m_uNumEntries;
	bool m_bStoringMarkerPoints;
	bool m_bInEnumeration; // So we don't change what we're storing mid-enumeration;
};

#endif	// AVP_CLIENTENTITY_GIB_H___
