//////////////////////////////////////////////////////////////////////////////
//
// AVP_ServerEntity_InteractObject.h
//
// Copyright (c) 2006 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////


#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef AVP_SERVERENTITY_INTERACTOBJECT_H
#define AVP_SERVERENTITY_INTERACTOBJECT_H

// Project independent.
#include "Entity/PhysicalObject/AVP_ServerEntity_PhysicalObject.h"

class Asura_OBB;
class Asura_OrientedVolume;

class AVP_ServerEntity_InteractObject : public AVP_ServerEntity_PhysicalObject
{
public:
	static AVP_ServerEntity_InteractObject* Create(Asura_Guid uGuid)					{ return new AVP_ServerEntity_InteractObject(uGuid); }
    static AVP_ServerEntity_InteractObject* FindEntity(Asura_Guid uGuid)				{ return s_xList.FindEntity(uGuid); }
    static const Asura_Hashed_List<AVP_ServerEntity_InteractObject>* GetList()			{ return &s_xList; }    

	bool			IsOn() const { return m_bOn; }
	bool			IsEnabled() const { return m_bEnabled; }
	bool			IsUseByAlien() const { return m_bUseByAlien; }
	bool			IsUseByHuman() const { return m_bUseByHuman; }
	bool			IsUseByPredator() const { return m_bUseByPredator; }
    virtual bool	HandleMessage(const Asura_Message* pxMessage);
//     virtual void	Update();

    virtual void	ReadFromChunkStream(Asura_Chunk_Stream& xStream);
	virtual void	WriteToChunkStream(Asura_Chunk_Stream& xStream) const;
        
    // Legacy chunk handling, to be removed as soon as possible.
	virtual bool	IsFilledFromChunkBuffer() const	{ return false; }

	virtual void	InitialiseFromBlueprint( const Asura_Blueprint& xBlueprint );
	
	virtual bool	GetHasCreateMessage() const { return true; }
	virtual void	MakeCreateMessage( Asura_Guid uRequestingClient, bool bSendGuaranteed = false );

#ifdef ASURA_TOOLS
    static void		CreateFromTools(Asura_Guid uGuid);
    
	virtual void	ApplyChangesForTemplateInstance(Tools_EntityTemplateInstance* pxInstance);
    virtual bool	CanBeIncludedInTemplate() const;
#endif // #ifdef ASURA_TOOLS

	const Asura_OrientedVolume*	GetActorVolume() const { return m_pxActorVolume; }
	const Asura_OBB*			GetOnTargetVolume() const { return m_pxOnTargetVolume; }
	const Asura_OBB*			GetOffTargetVolume() const { return m_pxOffTargetVolume; }

	#ifdef ASURA_TOOLS
	virtual bool SupportsCustomEvents() const { return true; }
	virtual int GetNumCustomEvents() const { return NUM_MESSAGE_BLOCKS; }
	virtual unsigned short* GetCustomEvent(int iIndex);
	virtual void GetCustomEventName(int iEventIndex, Asura_String& xEventName);
	#endif

protected:
    AVP_ServerEntity_InteractObject( Asura_Guid uGuid = ASURA_GUID_UNREGISTERED );
    virtual ~AVP_ServerEntity_InteractObject();

	void		OnTurnOn();
	void		OnTurnOff();
	void		SendInteractMessage( const Asura_Guid uPlayerGuid, const bool bInitialise = false );
	void		SendClientDisableMessage();

private:
	enum
	{
		MESSAGE_BLOCK_ON,
		MESSAGE_BLOCK_OFF,
		NUM_MESSAGE_BLOCKS,
	};

    typedef AVP_ServerEntity_PhysicalObject PARENT;

	Asura_OrientedVolume*	m_pxActorVolume;
	Asura_OBB*				m_pxOnTargetVolume;
	Asura_OBB*				m_pxOffTargetVolume;

	unsigned short			m_usTurnOnMessageBlock;
	unsigned short			m_usTurnOffMessageBlock;

	// !!!WARNING!!! These flags are accessed directly by the tools entity overlay, if you change them make sure you update the dialogs in the tools entity
	union
	{
		u_int		m_uFlags;
		struct  
		{
			u_int	m_bTriggerOnce				: 1,
					m_bUseActorVolume			: 1,
					m_bUseOnTargetVolume		: 1,
					m_bUseOffTargetVolume		: 1,
					m_bOn						: 1,
					m_bUseByAlien				: 1,
					m_bUseByHuman				: 1,
					m_bUseByPredator			: 1,
					m_bEnabled					: 1,
												: 0;
		};
	};

#ifdef ASURA_TOOLS
	friend class ToolsAVP_Entity_InteractObject;
	friend class ToolsAVP_Entity_InteractObject_ActorVolumeEdit_Controller;
	friend class ToolsAVP_Entity_InteractObject_OnTargetVolumeEdit_Controller;
	friend class ToolsAVP_Entity_InteractObject_OffTargetVolumeEdit_Controller;
#endif

    static Asura_Hashed_List<AVP_ServerEntity_InteractObject> s_xList;
};

#endif // #ifndef AVP_SERVERENTITY_INTERACTOBJECT_H
