//////////////////////////////////////////////////////////////////////////////
//
// AVP_ClientEntity_InteractInvisiSwitch.h
//
// Created by CAH 25/02/2009
//
// Copyright (c) 2009 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
// INCLUDES

// CORE
#include "Core/Asura.h"

// PERSONAL
#include "Entity/PhysicalObject/Interact/InvisiSwitch/AVP_ClientEntity_InteractInvisiSwitch.h"

//ASURA
#include "Animation/Asura_AnimationController_Animation.h"
#include "Blueprint/Asura_Blueprint_System.h"
#include "Network/Asura_Message.h"

// AVP
#include "Entity/AVP_Entity_Classification.h"
#include "Entity/PhysicalObject/Actor/AVP_Actor_Messages.h"
#include "Entity/PhysicalObject/Actor/AVP_ServerEntity_Actor.h"
#include "Entity/PhysicalObject/Actor/Weapons/Sentrygun/AVP_Sentrygun_Messages.h"
#include "Entity/PhysicalObject/Interact/InvisiSwitch/AVP_Messages_InteractInvisiSwitch.h"
#include "Entity/AVP_Messages.h"
#include "Moves/AVP_Move_Interaction.h"
#include "Moves/Vertebrates/Alien/AVP_Alien_MovesInteraction.h"

//////////////////////////////////////////////////////////////////////////////
// STATIC AND GLOBAL DECLARATIONS

Asura_Hashed_List<AVP_ClientEntity_InteractInvisiSwitch> AVP_ClientEntity_InteractInvisiSwitch::s_xList;

//////////////////////////////////////////////////////////////////////////////
// CLASS FUNCTIONS
// AVP_ClientEntity_InteractInvisiSwitch

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ClientEntity_InteractInvisiSwitch - Constructor
// Author       : Callum Hope CAH
// Date         : 25/02/09
// Scope		: Protected
////////////////////////////////////////////////////////////////////////////////

AVP_ClientEntity_InteractInvisiSwitch::AVP_ClientEntity_InteractInvisiSwitch()
:	m_uMarineDisableTPAnim( ASURA_HASH_ID_UNSET )
,	m_uAlienDestroyTPAnim( ASURA_HASH_ID_UNSET )
,	m_uPredatorDestroyTPAnim( ASURA_HASH_ID_UNSET )
{
	Classification = AVP_ENTITYCLASS_INTERACT_SENTRYGUN_INVISISWITCH;

	s_xList.AddToList( this );
}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ClientEntity_InteractInvisiSwitch - Destructor
// Author       : Callum Hope CAH
// Date         : 25/02/09
// Scope		: Protected
////////////////////////////////////////////////////////////////////////////////

AVP_ClientEntity_InteractInvisiSwitch::~AVP_ClientEntity_InteractInvisiSwitch() 
{
	s_xList.RemoveFromList( this );
}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ClientEntity_InteractInvisiSwitch::Create
// Author       : Callum Hope CAH
// Date         : 25/02/09
// Scope		: Static Public
//
// Desc			: Static call to class to createa new entity (and hance add it to s_xlist for reference later) then process the creation. 
////////////////////////////////////////////////////////////////////////////////

void AVP_ClientEntity_InteractInvisiSwitch::Create( const Asura_Message* pxMsg )
{
	Asura_Assert( pxMsg->GetMsgID() == ASURA_MSG_CLIENTCREATE, "AVP_ClientEntity_InteractInvisiSwitch::Create\n we are trying to create an entity from a message that was not:\n ASURA_MSG_CLIENTCREATE\n thats a no no. " );
	Asura_Assert( pxMsg->GetEntityClassification() == AVP_ENTITYCLASS_INTERACT_SENTRYGUN_INVISISWITCH, "AVP_ClientEntity_InteractInvisiSwitch::Create\n We are tryong to create an Invisible Switch but have sent a create message with the wrong classification. " );

	const AVP_Message_InteractInvisiSwitch_ClientCreate* pxData = static_cast< const AVP_Message_InteractInvisiSwitch_ClientCreate* >( pxMsg->GetData() );

	AVP_ClientEntity_InteractInvisiSwitch* pxNew = new AVP_ClientEntity_InteractInvisiSwitch();
	pxNew->ProcessClientCreate( pxData, pxMsg->GetSentTime(), true );
}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ClientEntity_InteractInvisiSwitch::ProcessClientCreate
// Author       : Callum Hope CAH
// Date         : 25/02/09
// Scope		: Protected
//
// Desc			: Called from the static create funciton, fills all of the data needed sent in the create message. 
////////////////////////////////////////////////////////////////////////////////

void AVP_ClientEntity_InteractInvisiSwitch::ProcessClientCreate( const AVP_Message_InteractInvisiSwitch_ClientCreate *pxData, float fServerTime, bool bProcessPosAndOri )
{
	PARENT::ProcessClientCreate( pxData, fServerTime, bProcessPosAndOri );
}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ClientEntity_InteractInvisiSwitch::HandleMessage
// Author       : Callum Hope CAH
// Date         : 25/02/09
// Scope		: Virtual Public
//
// Desc			: Does what it says on the tin. 
////////////////////////////////////////////////////////////////////////////////

bool AVP_ClientEntity_InteractInvisiSwitch::HandleMessage( const Asura_Message* pxMsg )
{
	switch ( pxMsg->GetMsgID() )
	{
		//////////////////////////////////////
		case AVP_MSGID_REQUEST_INTERACTION:
		//////////////////////////////////////
		{
			const AVP_Message_Actor_RequestInteraction*	pxData = static_cast<const AVP_Message_Actor_RequestInteraction*>( pxMsg->GetData() );
			if ( pxData )
			{
				Asura_Assert( pxData->GetInteractionType() == AVP_INTERACTION_TYPE_DEFAULT, "AVP_ClientEntity_InteractInvisiSwitch::HandleMessage\n Interaction objects only handle default interactions" );

				if( pxData->GetInteractionMoveType() ==  AVP_Message_Actor_RequestInteraction::INTERACTION_INTIATED )
				{
					AVP_ServerEntity_Actor* pxServActor = AVP_ServerEntity_Actor::FindEntity( pxData->GetActorGuid() );

					if( pxServActor )
					{
						// start the move on our actor entity
						AVP_ClientEntity_Actor* pxActor = AVP_ClientEntity_Actor::FindEntity( pxData->GetActorGuid() );
						Asura_Assert( pxActor, "No actor" );
						if ( pxActor != NULL )
						{
							Asura_Message* pxMoveMessage = Asura_Network::CreateMessage( ASURA_MSG_MOVE_START, sizeof( AVP_Message_InteractionMoveStart ) );
							Asura_Assert( pxMoveMessage , "unable to create message" );
							if ( pxMoveMessage  != NULL )
							{
								AVP_Message_InteractionMoveStart* pxMoveData = static_cast< AVP_Message_InteractionMoveStart* >( pxMoveMessage->GetData() );

								// Fill-in the message-body...
								pxMoveMessage->SetFromGuid( GetGuid() );
								pxMoveMessage->SetToGuid( pxData->GetActorGuid() );

								pxMoveData->ClearFlags();
								pxMoveData->SetMoveID( AVP_Move_Interaction_BeginWeaponAway::GetStaticHash() );
								pxMoveData->SetIgnoredByOwner( false );
								pxMoveData->SetInteractionObject( GetGuid() );
								pxMoveData->SetNextMoveHash( AVP_Move_Interaction::GetStaticHash() );
								pxMoveData->SetOn( !pxData->IsOn() );
								// Forward the message directly to the client actor... we will do this on each client on reciept of the interact message
								pxActor->HandleMessage( pxMoveMessage );
								Asura_Network::DestroyMessage( pxMoveMessage );
							}
						}

						//switch( pxActor->GetClassification() )
						//{
						//	////////////////////
						//	case( AVP_ENTITYCLASS_MARINE ):
						//	////////////////////
						//	{
						//		break;
						//	}
						//	////////////////////
						//	case( AVP_ENTITYCLASS_PREDATOR ):
						//	////////////////////
						//	{
						//		PlayAnimation( m_uPredatorDestroyTPAnim );
						//		break;
						//	}
						//	////////////////////
						//	case( AVP_ENTITYCLASS_ALIEN ):
						//	////////////////////
						//	{
						//		PlayAnimation( m_uAlienDestroyTPAnim );
						//		break;
						//	}
						//	////////////////////
						//	default:
						//	////////////////////
						//	{
						//		Asura_Assert( false, "AVP_ClientEntity_InteractInvisiSwitch::HandleMessage\n entity with unknown species trying to interact with invisiswitch. " );
						//	}
						//}

						return true;
					}
				}
			}
			else
			{
				Asura_Assert( false, "AVP_ClientEntity_InteractInvisiSwitch::HandleMessage\n A message has been passed with no data" );
				return false;
			}

			return PARENT::HandleMessage(pxMsg);
		}
		//////////////////////////////////////
		case AVP_MSGID_SENTRYGUN_SWITCH_PLAYANIMATION:
		//////////////////////////////////////
		{
			const AVP_Message_SentryGun_Switch_PlayAninmation*	pxData = static_cast<const AVP_Message_SentryGun_Switch_PlayAninmation*>( pxMsg->GetData() );
			if ( pxData )
			{
				if( pxData->GetAnimHash() != ASURA_GUID_UNREGISTERED )
				{
					Asura_Animation* pxAnimation = GetShapeInstance().GetAnimation();
					if ( pxAnimation )
					{
						Asura_AnimationBlend* pxBlend = new Asura_AnimationBlend_AnimatedTween( 0.0f, false );
						Asura_AnimationController_Animation* pxAACA = pxAnimation->CreateControllerAnimation( AAC_FLAG_ACTIVE | AAC_FLAG_ALLOW_DESTROY_FROM_MESSAGE, 0, pxData->GetAnimHash(), pxBlend );
						if ( pxAACA )
						{
							pxAACA->Play();
						}
						else
						{
							delete pxBlend;
						}
					}
					//PlayAnimation( pxData->GetAnimHash() );
				}
			}
			return true;
		}
		//////////////////////////////////////
		default:
		//////////////////////////////////////
		{
			return PARENT::HandleMessage(pxMsg);
		}
	}
}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ClientEntity_InteractInvisiSwitch::InitialiseFromBlueprint
// Author       : Callum Hope CAH
// Date         : 25/02/09
// Scope		: Virtual Protected
//
// Desc			: Called from the static create funciton, retrieves initialisation data. 
////////////////////////////////////////////////////////////////////////////////

void AVP_ClientEntity_InteractInvisiSwitch::InitialiseFromBlueprint( const Asura_Blueprint& xBlueprint )
{
	PARENT::InitialiseFromBlueprint( xBlueprint );

	Asura_Value xValue;

	if ( xBlueprint.GetParameterValueByHashID( 0x3A6DCF01, xValue ) )	//Asura_GetHashID( "Anim.Interaction.DestroyGun.Alien.HUD"); 
	{
		m_uMarineDisableTPAnim = xValue.GetHashIDValue();
	}

	if ( xBlueprint.GetParameterValueByHashID( 0x757F6B32, xValue ) )	//Asura_GetHashID( "Anim.Interaction.DestroyGun.Alien.TP"); 
	{
		m_uAlienDestroyTPAnim = xValue.GetHashIDValue();
	}

	if ( xBlueprint.GetParameterValueByHashID( 0xD8850E44, xValue ) )	//Asura_GetHashID( "Anim.Interaction.DestroyGun.Predator.TP"); 
	{
		m_uPredatorDestroyTPAnim = xValue.GetHashIDValue();
	}
}

Asura_Hash_ID AVP_ClientEntity_InteractInvisiSwitch::GetAnimHashForSpeciesInteraction( AVP_Settings::Species eSpecies )
{
	switch( eSpecies )
	{
		////////////////////
		case( AVP_Settings::AVP_SPECIES_HUMAN ):
		////////////////////
		{
			return m_uMarineDisableTPAnim;
		}
		////////////////////
		case( AVP_Settings::AVP_SPECIES_PREDATOR ):
		////////////////////
		{
			return m_uPredatorDestroyTPAnim;
		}
		////////////////////
		case( AVP_Settings::AVP_SPECIES_ALIEN ):
		////////////////////
		{
			return m_uAlienDestroyTPAnim;
		}
		////////////////////
		default:
		////////////////////
		{
			break;
		}
	}
	return ASURA_GUID_UNREGISTERED;
}
