//////////////////////////////////////////////////////////////////////////////
//
// AVP_ServerEntity_InteractBatteryReceptacle.h
//
// Created by AJK 24/11/2008
//
// Copyright (c) 2008 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////


#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __AVP_SERVERENTITY_INTERACTBATTERYRECEPTACLE__
#define __AVP_SERVERENTITY_INTERACTBATTERYRECEPTACLE__

// AVP
#include "Entity/PhysicalObject/Interact/ToolReceptacle/AVP_ServerEntity_InteractToolReceptacle.h"

class AVP_ServerEntity_InteractBatteryReceptacle : public AVP_ServerEntity_InteractToolReceptacle
{
public:

	enum
	{
		FLAG_BATTERY_INSIDE       = 0x00000001,
	};

	static AVP_ServerEntity_InteractBatteryReceptacle* Create(Asura_Guid uGuid)					{ return new AVP_ServerEntity_InteractBatteryReceptacle(uGuid); }
	static AVP_ServerEntity_InteractBatteryReceptacle* FindEntity(Asura_Guid uGuid)				{ return s_xList.FindEntity(uGuid); }
	static const Asura_Hashed_List<AVP_ServerEntity_InteractBatteryReceptacle>* GetList()		{ return &s_xList; }    

	virtual bool	HandleMessage(const Asura_Message* pxMessage);
	virtual void	ReadFromChunkStream(Asura_Chunk_Stream& xStream);
	virtual void	WriteToChunkStream(Asura_Chunk_Stream& xStream) const;
	virtual bool	GetHasCreateMessage() const { return true; }
	virtual void	MakeCreateMessage( Asura_Guid uRequestingClient, bool bSendGuaranteed /*= false*/ );

	void			DestroyBattery();
	void			AttachBatteryToRightHand(Asura_Guid uActorGuid);

#ifdef ASURA_TOOLS
	static void		CreateFromTools(Asura_Guid uGuid);
	virtual bool	CanBeIncludedInTemplate() const;
	virtual void    ApplyChangesForTemplateInstance( Tools_EntityTemplateInstance* pxInstance );
#endif // #ifdef ASURA_TOOLS

	#ifdef ASURA_TOOLS
	virtual bool SupportsCustomEvents() const { return true; }
	virtual int GetNumCustomEvents() const { return NUM_MESSAGE_BLOCKS; }
	virtual unsigned short* GetCustomEvent(int iIndex);
	virtual void GetCustomEventName(int iEventIndex, Asura_String& xEventName);
	#endif

protected:

	AVP_ServerEntity_InteractBatteryReceptacle( Asura_Guid uGuid = ASURA_GUID_UNREGISTERED );
	virtual ~AVP_ServerEntity_InteractBatteryReceptacle();

	enum
	{
		MESSAGE_BLOCK_MARINE_INSERT_BATTERY,
		MESSAGE_BLOCK_MARINE_REMOVE_BATTERY,
		MESSAGE_BLOCK_PREDATOR_USE_BATTERY,
		MESSAGE_BLOCK_ALIEN_DESTROY_BATTERY,

		NUM_MESSAGE_BLOCKS,
	};

	unsigned short	m_usMarineInsertBatteryMessageBlock;
	unsigned short	m_usMarineRemoveBatteryMessageBlock;
	unsigned short	m_usPredatorUseBatteryMessageBlock;
	unsigned short	m_usAlienDestroyBatteryMessageBlock;

private:

	enum
	{
		CURRENT_CHUNK_VERSION = 0
	};

	void SendMarineInsertBatteryMessage();
	void SendMarineRemoveBatteryMessage();
	void SendAlienDestroyBatteryMessage();
	void SendPredatorUseBatteryMessage();

	typedef AVP_ServerEntity_InteractToolReceptacle PARENT;

#ifdef ASURA_TOOLS
	friend class ToolsAVP_Entity_InteractBatteryReceptacle;
#endif

	static Asura_Hashed_List<AVP_ServerEntity_InteractBatteryReceptacle> s_xList;
};

#endif // #ifndef __AVP_SERVERENTITY_INTERACTBATTERYRECEPTACLE__
