//////////////////////////////////////////////////////////////////////////////
//
// AVP_ClientEntity_DamageableObject.cpp
//
// Copyright (c) 2007 Rebellion. All rights reserved.
//
// Description:
//
//     Project override of asura lift class
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header.
#include "Entity/PhysicalObject/PhysicsObject/DamageableObject/AVP_ClientEntity_DamageableObject.h"

// Project
#include "Core/AVP_Settings.h"
#include "Entity/AVP_Entity_Classification.h"
#include "Network/Asura_Message.h"

#include "Entity/PhysicalObject/PhysicsObject/DamageableObject/AVP_ServerEntity_DamageableObject.h"
#include "Entity/PhysicalObject/PhysicsObject/DamageableObject/AVP_DamageableObject_Messages.h"

// C L A S S   I M P L E M E N T A T I O N S /////////////////////////////////

//
// AVP_ClientEntity_DamageableObject
//

Asura_Hashed_List<AVP_ClientEntity_DamageableObject> AVP_ClientEntity_DamageableObject::s_xList;

AVP_ClientEntity_DamageableObject::AVP_ClientEntity_DamageableObject(void)
{
	Classification = AVP_ENTITYCLASS_DAMAGEABLEOBJECT;
	s_xList.AddToList(this);
}

AVP_ClientEntity_DamageableObject::~AVP_ClientEntity_DamageableObject() 
{
	s_xList.RemoveFromList(this);
}

void AVP_ClientEntity_DamageableObject::Create(const Asura_Message* pxMessage)
{
	Asura_Assert( pxMessage->GetMsgID() == ASURA_MSG_CLIENTCREATE, "" );
	Asura_Assert( pxMessage->GetEntityClassification() == AVP_ENTITYCLASS_DAMAGEABLEOBJECT, "" );

	AVP_ClientEntity_DamageableObject* pxDamageableObject = new AVP_ClientEntity_DamageableObject();

	pxDamageableObject->HandleMessage(pxMessage);
	pxDamageableObject->InitialiseShapePositionAndOrientation();
}

bool AVP_ClientEntity_DamageableObject::HandleMessage( const Asura_Message* pxMessage )
{
	switch(pxMessage->GetMsgID())
	{
		case ASURA_MSG_CLIENTUPDATE:
		{
			const AVP_Message_DamageableObject_ClientUpdate*	pxData	= static_cast<const AVP_Message_DamageableObject_ClientUpdate*>(pxMessage->GetData());
			ProcessClientUpdate(pxData, pxMessage->GetSentTime());
			return true;
		}

		case ASURA_MSG_CLIENTCREATE:
		{
			const AVP_Message_DamageableObject_ClientCreate*	pxData	= static_cast<const AVP_Message_DamageableObject_ClientCreate*>(pxMessage->GetData());
			ProcessClientCreate(pxData, pxMessage->GetSentTime());
			return true;
		}
	
		default:
		return(PARENT::HandleMessage(pxMessage));
	}
}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ClientEntity_DamageableObject::ProcessClientUpdate
// Author       : Andy Tate
// Date         : 13-Sep-2007
////////////////////////////////////////////////////////////////////////////////

void AVP_ClientEntity_DamageableObject::ProcessClientUpdate(const AVP_Message_DamageableObject_ClientUpdate* pxData,
															float fServerTime,
															bool bProcessPosAndOri)
{
	PARENT::ProcessClientUpdate(pxData, fServerTime, bProcessPosAndOri);
}

void AVP_ClientEntity_DamageableObject::ProcessClientCreate( const AVP_Message_DamageableObject_ClientCreate* pxData, float fServerTime, bool bProcessPosAndOri /*= true*/ )
{
	PARENT::ProcessClientCreate(pxData, fServerTime, bProcessPosAndOri);
}
