//////////////////////////////////////////////////////////////////////////////
//
// AVP_ServerEntity_DamageableObject.h
//
// Copyright (c) 2007 Rebellion. All rights reserved.
//
// Description:
//
//     Physical object which can be affected by explosions (initially static)
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef ___AVP_SERVERENTITY_DAMAGEABLEOBJECT_H___
#define ___AVP_SERVERENTITY_DAMAGEABLEOBJECT_H___ 1

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Entity/PhysicalObject/PhysicsObject/AVP_ServerEntity_PhysicsObject.h"

// C L A S S E S /////////////////////////////////////////////////////////////

class AVP_ServerEntity_DamageableObject : public AVP_ServerEntity_PhysicsObject
{
public:

	enum {
		FACE_RIGHT,
		FACE_TOP,
		FACE_FRONT,
		FACE_LEFT,
		FACE_BOTTOM,
		FACE_BACK,
	};

	AVP_ServerEntity_DamageableObject(const Asura_Guid uGUID = ASURA_GUID_UNREGISTERED);
	
	static AVP_ServerEntity_DamageableObject* Create(Asura_Guid uGuid) { return new AVP_ServerEntity_DamageableObject(uGuid); }

	static AVP_ServerEntity_DamageableObject* FindEntity(Asura_Guid uGuid)				{ return s_xList.FindEntity(uGuid); }
	static const Asura_Hashed_List<AVP_ServerEntity_DamageableObject>* GetList(void)	{ return &s_xList; }

	virtual	void	TakeDamageExtended(const AVP_EntityDamage_RequestVictim& xDamage, const bool bForceFatal, const bool bSendMessageToClient=true);

	virtual	bool CanBeSavedToChunk(void) const			{ return true; }
	virtual	bool IsFilledFromChunkBuffer(void) const	{ return false; }
	virtual void ReadFromChunkStream(Asura_Chunk_Stream& xStream);
	virtual void WriteToChunkStream(Asura_Chunk_Stream& xStream) const;

	virtual void	MakeClientUpdateMessage(void);
	virtual void	MakeCreateMessage(Asura_Guid uRequestingClient, bool bSendGuaranteed = false);

	bool		GetBeginsStatic(void)							{ return m_bBeginsStatic; }
	void		SetBeginsStatic(bool bBeginsStatic)				{ m_bBeginsStatic = bBeginsStatic; }

	u_int	    GetDamagedSkin(u_int iIndex) const				{ return m_auDamagedSkinHashID[iIndex]; }
	void		SetDamagedSkin(u_int iIndex, u_int uHashID)		{ m_auDamagedSkinHashID[iIndex] = uHashID; }	

	u_int	    GetDamagedAnim(u_int iIndex) const				{ return m_auDamagedAnimHashID[iIndex]; }
	void		SetDamagedAnim(u_int iIndex, u_int uHashID)		{ m_auDamagedAnimHashID[iIndex] = uHashID; }

	u_int	    GetDamagedSkinFile(u_int iIndex) const			{ return m_auDamagedSkinFileHashID[iIndex]; }
	void		SetDamagedSkinFile(u_int iIndex, u_int uHashID)	{ m_auDamagedSkinFileHashID[iIndex] = uHashID; }	

	u_int	    GetDamagedAnimFile(u_int iIndex) const			{ return m_auDamagedAnimFileHashID[iIndex]; }
	void		SetDamagedAnimFile(u_int iIndex, u_int uHashID)	{ m_auDamagedAnimFileHashID[iIndex] = uHashID; }	

	float		GetDamageThresholdSkin(void) const				{ return m_fDamageThresholdSkin; }
	void		SetDamageThresholdSkin(float fDamage)			{ m_fDamageThresholdSkin = fDamage; }

	float		GetDamageThresholdDynamic(void) const			{ return m_fDamageThresholdDynamic; }
	void		SetDamageThresholdDynamic(float fDamage)		{ m_fDamageThresholdDynamic = fDamage; }

#ifdef ASURA_TOOLS
	static void CreateFromTools( Asura_Guid uGUID );

	virtual void ApplyChangesForTemplateInstance(Tools_EntityTemplateInstance* pxInstance );
	virtual bool CanBeIncludedInTemplate(void) const;
#endif // #ifdef ASURA_TOOLS

protected:
    virtual void	InitialiseFromBlueprint( const Asura_Blueprint& xBlueprint );

	virtual ~AVP_ServerEntity_DamageableObject(void);

private:
	typedef AVP_ServerEntity_PhysicsObject PARENT;

	void OrientComponent(const Asura_Vector_3& xCentre, const Asura_Vector_3& xExtents, const Asura_Matrix_3x3& xOrientation, const int iIndex, Asura_Vector_3& xResult);

	u_int			m_auDamagedSkinFileHashID[6];
	u_int			m_auDamagedSkinHashID[6];
	u_int			m_auDamagedAnimFileHashID[6];
	u_int			m_auDamagedAnimHashID[6];
	bool			m_bUsingDamagedSkin;			// Has the object been damaged yet?
	float			m_fDamageThresholdSkin;			// To change to damaged skin
	float			m_fDamageThresholdDynamic;		// To change to dynamic object
	bool			m_bBeginsStatic;

	static Asura_Hashed_List<AVP_ServerEntity_DamageableObject> s_xList;
};

#endif // #ifndef ___AVP_SERVERENTITY_DESTRUCTIBLEOBJECT_H___
