//////////////////////////////////////////////////////////////////////////////
//
// File: AVP_ClientEntity_Pickup.h
//
//(AJK - 29.08.2008) Created
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////


#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __AVP_CLIENTENTITY_PICKUP__
#define __AVP_CLIENTENTITY_PICKUP__


// ASURA
#include "Entity/PhysicalObject/AVP_ClientEntity_PhysicalObject.h"

class Asura_AnimationAttachment_ClientContainerInstance;
class AVP_Pickup;

class AVP_ClientEntity_Pickup : public AVP_ClientEntity_PhysicalObject
{
public:

	enum
	{
		FLAG_IS_DISABLED        = 0x00000001,  // This is from a static message
	};

	static void Create( const Asura_Message* pxMsg );

	static AVP_ClientEntity_Pickup* FindEntity( Asura_Guid uGUID )				       { return s_xList.FindEntity( uGUID ); }

	static const Asura_Hashed_List<AVP_ClientEntity_Pickup>* GetList( void )	       { return &s_xList;                    }
	static const Asura_Hashed_List<AVP_ClientEntity_Pickup>* GetAutoPickList (void)	   { return(&s_xAutoPickList);           }
	static const Asura_Hashed_List<AVP_ClientEntity_Pickup>* GetPromptedPickList (void){ return(&s_xPromptedPickList);       }

	virtual bool HandleMessage( const Asura_Message* pxMsg );

    void				SetPickupBlueprintHash(Asura_Hash_ID uHash) { m_uPickupBlueprintHash = uHash; }
	Asura_Hash_ID		GetPickupBlueprintHash(void) const			{ return m_uPickupBlueprintHash; }
	// Audio Dialog pickups
	void                SetDialogueBlueprintHashID(Asura_Hash_ID uBlueprint) { m_uDialogueBlueprintHash = uBlueprint;}
	Asura_Hash_ID       GetDialogueBlueprintHashID() const                   { return m_uDialogueBlueprintHash;}
	const AVP_Pickup*   GetPickupObject() const  { return m_pxPickupObject; }

	// Flag functions
	void                    AddFlag    (u_int uFlag) { m_uFlags |= uFlag;               }
	void                    RemoveFlag (u_int uFlag) { m_uFlags &= ~uFlag;              }
	bool                    IsFlagged  (u_int uFlag) const { return ((m_uFlags & uFlag) != 0); }

	bool                    IsEnabled() const     { return !IsFlagged(FLAG_IS_DISABLED); }
	void                    SetEnabed(bool bTrue) { bTrue ? RemoveFlag(FLAG_IS_DISABLED) : AddFlag(FLAG_IS_DISABLED); }

protected:

	AVP_ClientEntity_Pickup();
	virtual ~AVP_ClientEntity_Pickup();

private:

	void CreateContainer();
	void DestroyContainer();
	void UpdateContainerState();

	void CreatePickupObject();
	void PlayRestAnim();

	Asura_AnimationAttachment_ClientContainerInstance* m_pxContainerAttachment;

	Asura_Hash_ID m_uPickupBlueprintHash; // Pickup blueprint hash (Get in create message from the server)
	Asura_Hash_ID m_uDialogueBlueprintHash;   // For pickups that uses dialogues
	AVP_Pickup*   m_pxPickupObject;    	  // Actual pickup object
	u_int         m_uFlags;               // enabled;

	static Asura_Hashed_List<AVP_ClientEntity_Pickup> s_xList;
	static Asura_Hashed_List<AVP_ClientEntity_Pickup> s_xAutoPickList;
	static Asura_Hashed_List<AVP_ClientEntity_Pickup> s_xPromptedPickList;

	typedef AVP_ClientEntity_PhysicalObject PARENT;

};

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

#endif // #ifndef __AVP_CLIENTENTITY_PICKUP__
