
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_ClientEntity_Projectile.cpp
// Author : Andy Tate
// Date   : 14-Nov-2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "Entity/PhysicalObject/Projectile/AVP_ClientEntity_Projectile.h"

// Asura independent header-files...
#include "Network/Asura_Message.h"
#include "Physics/Asura_Physics_Object.h"

// Game independent header-files...
#include "Blueprint/AVP_Blueprint_HashIDs.h"
#include "Entity/PhysicalObject/Projectile/AVP_Projectile_Messages.h"


////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////
Asura_Hashed_List<AVP_ClientEntity_Projectile>	AVP_ClientEntity_Projectile::s_xList;

////////////////////////////////////////////////////////////////////////////////
// Private functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Protected functions...
////////////////////////////////////////////////////////////////////////////////

AVP_ClientEntity_Projectile::AVP_ClientEntity_Projectile()
{
	SetEntityAlwaysMoves( true );
	SetDontShowUpInMotionTracker( true );
	s_xList.AddToList( this );
}

AVP_ClientEntity_Projectile::~AVP_ClientEntity_Projectile()
{
	s_xList.RemoveFromList( this );
}

////////////////////////////////////////////////////////////////////////////////
// Public functions...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ClientEntity_Projectile::HandleMessage
// Author       : Andy Tate
// Date         : 2-Jan-2007
////////////////////////////////////////////////////////////////////////////////

bool AVP_ClientEntity_Projectile::HandleMessage (const Asura_Message* pxMessage)
{
	switch(pxMessage->GetMsgID())
	{
        case ASURA_MSG_CLIENTUPDATE:
        {
			// Catch projectile specific date
            const AVP_Message_ProjectileClientUpdate*	pxData	= static_cast<const AVP_Message_ProjectileClientUpdate*>(pxMessage->GetData());
            ProcessClientUpdate(pxData, pxMessage->GetSentTime());
			return(true);
		}

		default:
		{
			return(PARENT::HandleMessage(pxMessage));
		}
	}		// End of switch.
}

void AVP_ClientEntity_Projectile::ProcessClientUpdate( const AVP_Message_ProjectileClientUpdate* pxData, float fServerTime, const bool bProcessPosAndOri /* = true */ )
{
	m_uOwnerGuid = pxData->GetOwnerGuid();
	PARENT::ProcessClientUpdate( pxData, fServerTime, bProcessPosAndOri );
}
/// End of function 'AVP_ClientEntity_Projectile::HandleMessage'. ///


/// End of file. ///
