
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_ClientProjectile_AcidSpit.cpp
// Author : Callum Hope
// Date   : 17/4/09
//
// Copyright (C) 2009 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "Entity/PhysicalObject/Projectile/AVP_ClientProjectile_AcidSpit.h"

//// Asura independent header-files...
#include "Blueprint/Asura_Blueprint_System.h"
#include "Network/Asura_Message.h"
#include "SpecialFX/pfx/Asura_PFX_Entity_Effect.h"
//
//// Game independent header-files...
#include "Blueprint/AVP_Blueprint_HashIDs.h"
#include "Core/AVP_Settings.h"
#include "Entity/AVP_Entity_Classification.h"
#include "Entity/AVP_Messages.h"
#include "Entity/PhysicalObject/Projectile/AVP_Projectile_Messages.h"
#include "Entity/PhysicalObject/Projectile/AVP_GenericProjectileData.h"
#include "Input/AVP_Rumble_System.h"

////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////

Asura_Hashed_List<AVP_ClientProjectile_AcidSpit>	AVP_ClientProjectile_AcidSpit::s_xList;

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ClientProjectile_AcidSpit::AVP_ClientProjectile_AcidSpit
// Author       : Callum Hope
// Date         : 17/4/09
// Scope		: Protected
////////////////////////////////////////////////////////////////////////////////

AVP_ClientProjectile_AcidSpit::AVP_ClientProjectile_AcidSpit( const AVP_Message_ProjectileClientUpdate* pxData )
: PARENT( pxData )
{
	// Set the classification...
	Classification = AVP_ENTITYCLASS_PROJECTILE_ACIDSPIT;

	s_xList.AddToList( this );
}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ClientProjectile_AcidSpit::~AVP_ClientProjectile_AcidSpit
// Author       : Callum Hope
// Date         : 17/4/09
// Scope		: Virtual Protected
////////////////////////////////////////////////////////////////////////////////

AVP_ClientProjectile_AcidSpit::~AVP_ClientProjectile_AcidSpit()
{
	Asura_PFX_Entity_Effect* pxEffect = static_cast<Asura_PFX_Entity_Effect*>( Asura_PFX_System::GetEffect( m_iPFXTrailHandle ) );
	
	if( pxEffect != NULL )
	{
		pxEffect->Stop( true );
	}

	s_xList.RemoveFromList( this );
}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ClientProjectile_AcidSpit::Create
// Author       : Callum Hope
// Date         : 17/4/09
// Scope		: Static Public
////////////////////////////////////////////////////////////////////////////////

void AVP_ClientProjectile_AcidSpit::Create( const Asura_Message *pxMessage )
{
	const AVP_Message_ProjectileClientUpdate* pxData = static_cast< const AVP_Message_ProjectileClientUpdate* >( pxMessage->GetData() );
	AVP_ClientProjectile_AcidSpit*	pxClient	= new AVP_ClientProjectile_AcidSpit( pxData );

	// Pass the message to our new client...
	pxClient->HandleMessage(pxMessage);
}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ClientProjectile_AcidSpit::Update
// Author       : Callum Hope
// Date         : 17/4/09
// Scope		: Virtual Public
////////////////////////////////////////////////////////////////////////////////

void AVP_ClientProjectile_AcidSpit::Update (void)
{
	PARENT::Update();
}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ClientProjectile_AcidSpit::HandleMessage
// Author       : Callum Hope
// Date         : 17/4/09
// Scope		: Virtual Public
////////////////////////////////////////////////////////////////////////////////

bool AVP_ClientProjectile_AcidSpit::HandleMessage (const Asura_Message* pxMessage)
{
	switch(pxMessage->GetMsgID())
	{
		////////////////////
		case ASURA_MSG_CLIENTUPDATE:
		////////////////////
		{
			// ATMo we are just using the generic client update as we have nothing unique to the acid spit update. 
			const AVP_Message_GenericProjectileClientUpdate* pxData = static_cast<const AVP_Message_GenericProjectileClientUpdate*>( pxMessage->GetData() );
			ProcessClientUpdate( pxData, pxMessage->GetSentTime() );
			return PARENT::HandleMessage( pxMessage );
		}
		////////////////////
		case AVP_MSGID_ENTITYDAMAGE_APPLY_TO_ENTITY:
		////////////////////
		{
			// projectiles don't get damaged (but message handled)
			return(true);
		}
		////////////////////
		case ASURA_MSG_DESTRUCT:
		////////////////////
		{
			// carry out the impact effects for the acid projectile then forward to AVP_ClientEntity_Projectile
			// this skips out the immediate parent AVP_ClientProjectile_GenericProjectile and the explosion routein. 
			MakeImpactSplat();
			return( AVP_ClientEntity_Projectile::HandleMessage( pxMessage ) );
		}
		////////////////////
		default:
		////////////////////
		{
			return( PARENT::HandleMessage( pxMessage ) );
		}
	}
}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ClientProjectile_AcidSpit::ProcessClientUpdate
// Author       : Callum Hope
// Date         : 17/4/09
// Scope		: Private
////////////////////////////////////////////////////////////////////////////////

void AVP_ClientProjectile_AcidSpit::ProcessClientUpdate( const AVP_Message_GenericProjectileClientUpdate* pxData, const float fServerTime, const bool bProcessPosAndOri /*= true*/ )
{
}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ClientProjectile_AcidSpit::MakeImpactSplat
// Author       : Callum Hope
// Date         : 1/5/09
// Scope		: Protected
////////////////////////////////////////////////////////////////////////////////

void AVP_ClientProjectile_AcidSpit::MakeImpactSplat()
{
	//create the effect used for splat here. 
	if ( m_pxProjectileData->GetExplosionPFX() != ASURA_HASH_ID_UNSET )
	{
		int iEffect = Asura_PFX_Entity_Effect::Create( GetCurrentPosition(), GetCurrentOrientation(), m_pxProjectileData->GetExplosionPFX(), 0, ASURA_PFX_EFFECT_CAMERA_FLAGS, GetGuid() );
		Asura_PFX_Entity_Effect* pxEffect = static_cast< Asura_PFX_Entity_Effect* >( Asura_PFX_System::GetEffect( iEffect ) );
		if ( pxEffect )
		{
			pxEffect->SetSizeScale( m_fScale );
		}
	}

	// stop the effect of the projectile itself
	if ( m_iPFXTrailHandle != ASURA_INVALID_HANDLE )
	{
		Asura_PFX_Entity_Effect* pxEffect = static_cast< Asura_PFX_Entity_Effect* >( Asura_PFX_System::GetEffect( m_iPFXTrailHandle ) );
		if ( pxEffect != NULL )
		{
			// stop it immediatly because we are about to clear the handle and cant wait for the delay of a fade. 
			pxEffect->Stop( true );
			m_iPFXTrailHandle = ASURA_INVALID_HANDLE;
		}
	}

	// Rumble
	if( AVP_Settings::GetCurrentProfile()->GetControls().GetVibrate() && m_pxProjectileData->GetVibrationHash() != ASURA_HASH_ID_UNSET)
	{
		AVP_Rumble_System::AddWorldRumble( m_pxProjectileData->GetVibrationHash(), GetCurrentPosition(),  
			m_pxProjectileData->GetVibrationOuterRadius(),  m_pxProjectileData->GetVibrationInnerRadius() );
	}
}

