
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_ClientProjectile_Disc.cpp
// Author : Chris Wallace
// Date   : 27-Oct-2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "Entity/PhysicalObject/Projectile/AVP_ClientProjectile_Disc.h"

// Asura independent header-files...
#include "Network/Asura_Message.h"

// Game independent header-files...
#include "Entity/AVP_Entity_Classification.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Predator/AVP_ClientActor_Predator.h"
#include "Entity/PhysicalObject/Projectile/AVP_Projectile_Messages.h"
#include "Inventory/AVP_Inventory_System.h"
#include "Inventory/Groups/AVP_InventoryGroup_Weapons.h"
#include "Inventory/Items/Weapons/AVP_InventoryWeapon_Disc.h"

Asura_Hashed_List<AVP_ClientProjectile_Disc>	AVP_ClientProjectile_Disc::s_xList;

AVP_ClientProjectile_Disc::AVP_ClientProjectile_Disc ( const AVP_Message_ProjectileClientUpdate* pxData )
: PARENT( pxData )
{
	Classification = AVP_ENTITYCLASS_DISC;

	s_xList.AddToList(this);
}

AVP_ClientProjectile_Disc::~AVP_ClientProjectile_Disc ()
{
	AVP_ClientActor_Predator* pxClient	= AVP_ClientActor_Predator::FindEntity( m_uOwnerGuid );
	AVP_InventoryItem_Weapon* pxItem	= pxClient ? static_cast< AVP_InventoryItem_Weapon* >( pxClient->GetInventorySystem()->GetCurrentItem( uAVP_INVENTORY_GROUPHASH_WEAPON ) ) : NULL;
	if ( pxItem && pxItem->GetItemType() == AVP_INVENTORYITEMTYPE_PREDATORDISC )
	{
		AVP_InventoryWeapon_Disc* pxDiscItem	= static_cast< AVP_InventoryWeapon_Disc* >( pxItem );
		if ( pxDiscItem != NULL )
		{
			pxDiscItem->RequestEndUsage();
		}
	}

	s_xList.RemoveFromList(this);
}

void AVP_ClientProjectile_Disc::Create(const Asura_Message *pxMessage)
{
	AVP_ClientProjectile_Disc*	pxClient				= new AVP_ClientProjectile_Disc( static_cast< const AVP_Message_ProjectileClientUpdate* >( pxMessage->GetData() ) );

	pxClient->HandleMessage(pxMessage);
}

void AVP_ClientProjectile_Disc::PostCreateInitialise()
{
	AVP_ClientActor_Predator* pxPred = AVP_ClientActor_Predator::FindEntity( m_uOwnerGuid );
	if(pxPred)
	{
		AVP_Inventory_System* pxSystem = pxPred->GetInventorySystem();
		if(pxSystem)
		{
			AVP_Inventory_Item* pxItem = pxSystem->GetCurrentItem( uAVP_INVENTORY_GROUPHASH_WEAPON );
			if(pxItem)
			{
				pxItem->SetEntityGuid( GetGuid() );
			}
		}
	}
}

bool AVP_ClientProjectile_Disc::HandleMessage( const Asura_Message* pxMessage )
{
	// What sort of message is it?
	switch(pxMessage->GetMsgID())
	{
		case ASURA_MSG_DESTRUCT:
		{
			// override generic projectiles behaviour of exploding on destruct
			return(AVP_ClientEntity_Projectile::HandleMessage(pxMessage));
		}

		default:
		{
			return(PARENT::HandleMessage(pxMessage));
		}
	}		// End of switch.
}
/// End of file. ///
