///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_ClientProjectile_HumanFlare.h
// Author : Chris Wallace
// Date   : 03-May-2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __AVP_CLIENTPROJECTILE_HUMANFLARE_H__
#define __AVP_CLIENTPROJECTILE_HUMANFLARE_H__


////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Entity/PhysicalObject/Projectile/AVP_ClientProjectile_GenericProjectile.h"

#include "Rendering/Asura_Colour.h"
#include "Sound/Asura_Sound_Event_System.h"

#include "SpecialFX/PFX/Asura_PFX_Effect.h"
#include "SpecialFX/PFX/Asura_PFX_Entity_Effect.h"
#include "SpecialFX/PFX/Asura_PFX_System.h"


////////////////////////////////////////////////////////////////////////////////
// Forward class declarations...
////////////////////////////////////////////////////////////////////////////////

class Asura_Animation;
class Asura_Photon;

////////////////////////////////////////////////////////////////////////////////
// Class declaration...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_ClientProjectile_HumanFlare
// Author       : Ste Butcher
// Date         : 26-March-2007
// Description  : Class to handle a Predator's grenade
////////////////////////////////////////////////////////////////////////////////

class AVP_ClientProjectile_HumanFlare : public AVP_ClientProjectile_GenericProjectile
{
public:
	// Public functions...
	static	void		Create(const Asura_Message* pxMessage);

	virtual void InitialiseFromBlueprint( const Asura_Blueprint& xBlueprint );

	// Overridden functions...
	virtual	void		Update();

	bool IsFlareExpired() const { return m_bFlareExpired; }

	// Static list accessors...
	static	AVP_ClientProjectile_HumanFlare*							FindEntity(const Asura_Guid uGuid)		{ return s_xList.FindEntity(uGuid); }
	static	const Asura_Hashed_List<AVP_ClientProjectile_HumanFlare>*	GetList()								{ return &s_xList; }

protected:
	// Protected functions...

	virtual void UpdateProjectileSound(void);

	AVP_ClientProjectile_HumanFlare( const AVP_Message_ProjectileClientUpdate* pxData );
	virtual ~AVP_ClientProjectile_HumanFlare();

private:
	// Private typedefs...
	typedef AVP_ClientProjectile_GenericProjectile PARENT;

	// Private variables...
	Asura_Photon* m_pxFlarePhoton;

	Asura_Hash_ID	m_uBeginSoundEventHash;
	
	Asura_RGBA m_xFlareColour;

	float m_fLifeTimer;
	float m_fDuration;
	float m_fFadeOutTime;

	float m_fPhotonRange;
	float m_fPhotonIntensity;
	float m_fDistanceFadeStart;
	float m_fDistanceFadeDistance;
	float m_fFalloffPower;
	float m_fFalloffScale;

	union
	{
		u_int		m_uPhotonFlags;
		struct  
		{
			u_int	m_bDoDistanceFade			: 1,
					m_bDoCorona					: 1,
					m_bDoEnvShadows				: 1,
					m_bDoShadowNoise			: 1,
					m_bCastsOwnShadows			: 1,
					m_bContributeToAvgShadows	: 1,
					m_bScaleByBrightness		: 1,
												: 0;
		};
	};


	float m_fFlicker;

	float m_fEmbedParametric;
	float m_fPhotonOffset;
	bool m_bFlareExpired;
	
	static Asura_Hashed_List<AVP_ClientProjectile_HumanFlare>		s_xList;

	// Private functions...

	AVP_ClientProjectile_HumanFlare(const AVP_ClientProjectile_HumanFlare& xOther);
	AVP_ClientProjectile_HumanFlare& operator= (const AVP_ClientProjectile_HumanFlare& xOther);
};   // End of class 'AVP_ClientProjectile_HumanFlare'.

#endif   // ifndef __AVP_CLIENTPROJECTILE_HUMANFLARE_H__

/// End of file. ///
