///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_ServerProjectile_Disc.cpp
// Author : Chris Wallace
// Date   : 04-Dec-2007
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "Entity/PhysicalObject/Projectile/AVP_ServerProjectile_Disc.h"

// Asura independent header-files...
#include "Blueprint/Asura_Blueprint.h"
#include "Blueprint/Asura_Blueprint_System.h"
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "Core/Asura_Timers.h"
#include "Network/Asura_Network.h"
#include "Physics/Asura_Physics.h"
#include "SpecialFX/Asura_Decal_System.h"
#include "SpecialFX/pfx/Asura_PFX_Entity_Effect.h"
#include "SpecialFX/pfx/Asura_PFX_System.h"

// Game independent header-files...
#include "Actor/Vertebrates/Predator/AVP_Predator_Description.h"
#include "Blueprint/AVP_Blueprint_HashIDs.h"
#include "Entity/AVP_Entity_Classification.h"
#include "Entity/AVP_Messages.h"
#include "Entity/Damage/AVP_EntityDamage_Messages.h"
#include "Entity/PhysicalObject/Actor/AVP_Actor_Messages.h"
#include "Entity/PhysicalObject/Actor/AVP_ServerEntity_Actor.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Predator/AVP_ClientActor_Predator.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Predator/AVP_ServerActor_Predator.h"
#include "Entity/PhysicalObject/Projectile/AVP_Projectile_Messages.h"
#include "Entity/PhysicalObject/Projectile/AVP_GenericProjectileData.h"
#include "Inventory/AVP_Inventory_System.h"
#include "Inventory/AVP_Inventory_Group.h"
#include "Inventory/AVP_Inventory_HashIDs.h"
#include "Inventory/Items/Weapons/AVP_InventoryWeapon_Disc.h"

////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////

static const u_int	g_uChunkVersion	= 0;

DEBUGVAR float	dbg_fDiscCollectionDistance	= 1.0f;

Asura_Hashed_List<AVP_ServerProjectile_Disc>	AVP_ServerProjectile_Disc::s_xList;

AVP_ServerProjectile_Disc::AVP_ServerProjectile_Disc (const Asura_Guid uGuid,
													  const Asura_Message* pxMsg)
: Asura_Entity_Server( uGuid )
, PARENT( uGuid, pxMsg )
, m_fTiltAngle(0.0f)
, m_fReturnTime(2.0f)
, m_fReturnTeleportTime(5.0f)
, m_fReturnMaxBendingAngle( 2.0943951f )			// Asura_Maths::DegToRad(120.0f)
, m_fReturnTimeToReachMaxBendingAngle( 2.0f )
, m_bBounceMode(false)
, m_bLaserTargetMode(false)
, m_fLaserFollow_MinFlyDistToActivate(10.0f)
, m_fLaserFollow_MaxBendingAngle( 2.0943951f )		// Asura_Maths::DegToRad(120.0f)
, m_fSeekEnemyActors_ScanDistance(20.0f)
, m_fSeekEnemyActors_ScanAngle( 0.174532925f )		// Asura_Maths::DegToRad(10.0f)
, m_fSeekEnemyActors_MinFlyDistToActivate(10.0f)
, m_fSeekEnemyActors_MaxBendingAngle( 0.261799388 )	// Asura_Maths::DegToRad(15.0f)
, m_fTraveledDistance(0.0f)
//, m_uTargetGuid(ASURA_GUID_UNREGISTERED)
//, m_uTargetMPHash(ASURA_HASH_ID_UNSET)
, m_fTimeSinceLaunch(0.0f)
, m_fTimeSinceReturn(0.0f)
//, m_fPredictedTimeToNextTarget(0.0f)
, m_xDiscDirection(1.0f,0.0f,0.0f)
{
	// Update the data...
	Classification	= AVP_ENTITYCLASS_DISC;
	m_uDiscState	= DISC_IN_FLIGHT;

	// Set the stats...
	SetHealth(100.0f);
	SetDestructible(false);

	const AVP_Message_CreateProjectile_Disc* pxData = static_cast< const AVP_Message_CreateProjectile_Disc* >( pxMsg->GetData() );
	m_bBounceMode		= pxData->GetBounceMode();
	m_bLaserTargetMode	= pxData->GetLaserTargetMode();
//	pxData->GetTargetData( m_uTargetGuid, m_uTargetMPHash );
	
	AVP_ServerActor_Predator* pxPred = AVP_ServerActor_Predator::FindEntity( pxMsg->GetFromGuid() );
	pxPred->GetInventorySystem()->GetCurrentItem( uAVP_INVENTORY_GROUPHASH_WEAPON )->SetEntityGuid( GetGuid() );

	// Don't forget to store us in the list...
	s_xList.AddToList(this);
}


AVP_ServerProjectile_Disc::~AVP_ServerProjectile_Disc ()
{
	AVP_ServerActor_Predator* pxServer	= AVP_ServerActor_Predator::FindEntity(m_uOwnerGuid);
	AVP_InventoryItem_Weapon* pxItem	= pxServer ? static_cast< AVP_InventoryItem_Weapon* >( pxServer->GetInventorySystem()->GetCurrentItem( uAVP_INVENTORY_GROUPHASH_WEAPON ) ) : NULL;

	if ( pxItem && pxItem->GetItemType() == AVP_INVENTORYITEMTYPE_PREDATORDISC )
	{
		AVP_InventoryWeapon_Disc* pxDiscItem	= static_cast< AVP_InventoryWeapon_Disc* >( pxItem );
		if ( pxDiscItem != NULL )
		{
			pxDiscItem->RequestEndUsage();
		}
	}

	// Don't forget to remove us from the list...
	s_xList.RemoveFromList(this);
}


void AVP_ServerProjectile_Disc::InitialiseFromBlueprint( const Asura_Blueprint& xBlueprint )
{
	PARENT::InitialiseFromBlueprint( xBlueprint );

	Asura_Value xValue;

	// Read values for Disc Return Behaviour.

	if(xBlueprint.GetParameterValueByHashID( 0x105F97C8, xValue )) // Asura_GetHashID( "Return.TimeToSwitchToReturnMode" )
	{
		m_fReturnTime = xValue.GetFloatValue();
	}

	if(xBlueprint.GetParameterValueByHashID( 0xF26AFA47, xValue )) // Asura_GetHashID( "Return.TimeToTeleportToOwner" )
	{
		m_fReturnTeleportTime = xValue.GetFloatValue();
	}

	if(xBlueprint.GetParameterValueByHashID( 0xD140B510, xValue )) // Asura_GetHashID( "Return.MaxBendingAngle" )
	{
		m_fReturnMaxBendingAngle= xValue.GetFloatValue();
	}

	if(xBlueprint.GetParameterValueByHashID( 0xA2664181, xValue )) // Asura_GetHashID( "Return.TimeToReachMaxBendingAngle" )
	{
		m_fReturnTimeToReachMaxBendingAngle = xValue.GetFloatValue();
	}

	// Stuff for Laser Seek

	if(xBlueprint.GetParameterValueByHashID( 0xB1E6E227, xValue )) // Asura_GetHashID( "LaserFollow.MinFlyDistToActivate" )
	{
		m_fLaserFollow_MinFlyDistToActivate = xValue.GetFloatValue();
	}

	if(xBlueprint.GetParameterValueByHashID( 0xCB1E561C, xValue )) // Asura_GetHashID( "LaserFollow.MaxBendingAngle" )
	{
		m_fLaserFollow_MaxBendingAngle = xValue.GetFloatValue();
	}

	// Stuff for AutoCorrect or also called SeekEnemyActors

	if(xBlueprint.GetParameterValueByHashID( 0x0709EE75, xValue )) // Asura_GetHashID( "SeekEnemyActors.MinFlyDistToActivate" )
	{
		m_fSeekEnemyActors_MinFlyDistToActivate = xValue.GetFloatValue();
	}

	if(xBlueprint.GetParameterValueByHashID( 0x01DD7632, xValue )) // Asura_GetHashID( "SeekEnemyActors.ScanDistance" )
	{
		m_fSeekEnemyActors_ScanDistance = xValue.GetFloatValue();
	}

	if(xBlueprint.GetParameterValueByHashID( 0x880FFFB6, xValue )) // Asura_GetHashID( "SeekEnemyActors.ScanAngle" )
	{
		m_fSeekEnemyActors_ScanAngle = xValue.GetFloatValue();

		// Also make sure that we are using only half of this angle, because we want to check for both sides (left and right).
		// So if there is angle 30, we will store this value as 15. Then later on we will check 15 degrees in all directions, thus we make it cone with size 30.
		m_fSeekEnemyActors_ScanAngle = Asura_Maths::DegToRad( m_fSeekEnemyActors_ScanAngle*0.5f );
	}

	if(xBlueprint.GetParameterValueByHashID( 0x149C510E, xValue )) // Asura_GetHashID( "SeekEnemyActors.MaxBendingAngle" )
	{
		m_fSeekEnemyActors_MaxBendingAngle = xValue.GetFloatValue();
	}
}


AVP_ServerProjectile_Disc*	AVP_ServerProjectile_Disc::Create( const Asura_Message* pxMsg )
{
	AVP_ServerProjectile_Disc*	pxDisc = new AVP_ServerProjectile_Disc( Asura_Entity_System::GetCreatedByGameGuid(), pxMsg);
    pxDisc->InitFromBlueprint();
	const AVP_GenericProjectileData* pxProjectileData = AVP_GenericProjectileData::GetDataByHash( pxDisc->GetBlueprintHashID() );
	const AVP_Message_CreateGenericProjectile* pxData = static_cast< const AVP_Message_CreateGenericProjectile* >( pxMsg != NULL ? pxMsg->GetData() : NULL );
	if( pxData != NULL )
	{
		const float				fMuzzleVel = pxProjectileData->GetMuzzleVelocity();
		Asura_Vector_3			xVelocity( 0.0f, 0.0f, fMuzzleVel );
		const Asura_Vector_3&	xPosition = pxData->GetPosition();
		Asura_Matrix_3x3		xOrientation;
		pxData->GetOrientation().ToMatrix( xOrientation );
		xVelocity = xVelocity * xOrientation;
		float fMag = xVelocity.Normalise();
		fMag += xVelocity * pxData->GetActorVelocity();
		pxDisc->Launch( xPosition, xOrientation, xVelocity * fMag, pxDisc->m_uOwnerGuid, pxData->GetDamageScale() ); // Launch last as this creates the client entity
	}

    return pxDisc;
}


void AVP_ServerProjectile_Disc::Launch (const Asura_Vector_3 &xPosition,
										const Asura_Matrix_3x3 &xOrientation,
										const Asura_Vector_3 &xVelocity,
										const Asura_Guid uGuidToIgnore,
										float fDamageMultiplier)
{
	PARENT::Launch(xPosition, xOrientation, xVelocity, uGuidToIgnore, fDamageMultiplier);

	m_fTimeSinceReturn = 0.0f;
	//	m_fPredictedTimeToNextTarget = 0.0f;
	m_xDiscDirection = xVelocity;
	m_xDiscDirection.Normalise();
	m_fTiltAngle = Asura_Maths::Pi*0.5f;
	m_fTraveledDistance = 0.0f;
}


void AVP_ServerProjectile_Disc::MakeClientUpdateMessage (void)
{
	Asura_Message* pxMessage = Asura_Network::CreateMessage(ASURA_MSG_CLIENTUPDATE, sizeof(AVP_Message_DiscClientUpdate), GetClassification());
	
	if(pxMessage != NULL)
	{
		// Fill in the message...
		AVP_Message_DiscClientUpdate* pxData = static_cast<AVP_Message_DiscClientUpdate*>(pxMessage->GetData());
		pxData->FillDisc(this);

		// Send the message...
		Send(pxMessage, ASURA_GUID_MY_CLIENT_ENTITIES);
	}
}


void AVP_ServerProjectile_Disc::ReadFromChunkStream (Asura_Chunk_Stream& xStream)
{
	u_int	uVersion;

	// Read the version...
	xStream >> uVersion;
	Asura_Assert((uVersion <= g_uChunkVersion), "Unsupported chunk version");

	// Read the parent data...
	PARENT::ReadFromChunkStream(xStream);
}


void AVP_ServerProjectile_Disc::WriteToChunkStream (Asura_Chunk_Stream& xStream) const
{
	// Write the version...
	xStream << g_uChunkVersion;

	// Write the parent data...
	PARENT::WriteToChunkStream(xStream);
}


void AVP_ServerProjectile_Disc::FindTargetDirectionAndMaxBendingAngle( const AVP_ServerActor_Predator* pxServerPred, Asura_Vector_3& xTargetDirection, float& fMaxBendingAngle )
{
	// Then see, if we can find something that we are going to hit.
	switch( m_uDiscState )
	{
		case DISC_IN_FLIGHT:
		{
			// When there is some living target in front of the disc, lets follow it first. This following actors has bigger priority then following laser.
			Asura_Vector_3 xClosestActorDirVector;
			if( m_fTraveledDistance>m_fSeekEnemyActors_MinFlyDistToActivate && SearchForClosestTargetInSearchingCone(xClosestActorDirVector) )
			{
				xTargetDirection = xClosestActorDirVector;
				fMaxBendingAngle = m_fSeekEnemyActors_MaxBendingAngle * Asura_Timers::GetGameTimeSlice();
			}	
			// No valid target in range. So check if disk should follow laser marker point.
			else if( m_fTraveledDistance>m_fLaserFollow_MinFlyDistToActivate && GetLaserTargetMode() )
			{
				xTargetDirection = ( pxServerPred->GetLookAtPosition() - GetPosition() );
				fMaxBendingAngle = m_fLaserFollow_MaxBendingAngle * Asura_Timers::GetGameTimeSlice();
			}
			break;
		}

		case DISC_RETURNING_TO_SENDER:
		{
			// Target position should be the predators hand
			Asura_Vector_3 xHandPos;
			if(pxServerPred->GetWorldSpaceMarkerPoint(AVP_Predator_Description::GetMarkerPointHashID(AVP_Predator_Description::RIGHT_HAND_MARKER_POINT_HASH), &xHandPos, NULL))
			{
				xTargetDirection = (xHandPos - GetPosition());
			}
			else	// But if we can't find hand, lets use only predator position only.
			{
				xTargetDirection = (pxServerPred->GetPosition() - GetPosition());
			}

			fMaxBendingAngle = m_fReturnMaxBendingAngle * Asura_Timers::GetGameTimeSlice();

			// Make max bending angle time dependent. This is little bit tricky. So lets see what is happening here:
			// When the disc is flying forward and it suddenly switch to RETURN mode, them it will take 
			// very sharp turn towards you. To prevent this, new time value is introduced. This will ensure
			// That the angle will become sharp only after some time. (by interpolating from zero to max over time.)
			if( m_fReturnTimeToReachMaxBendingAngle )
			{
				fMaxBendingAngle = Asura_Maths::Lerp( 0.0f, fMaxBendingAngle, m_fTimeSinceReturn/m_fReturnTimeToReachMaxBendingAngle );
			}

			break;
		}
		default:
		{
			Asura_Assert(0, "Unhandled disc state in predator disc server update");
			break;
		}
	}
}

// #include "Debugging/Asura_DebugText.h"
//	int krak = 20;
//	u_int color = 	( m_uDiscState==DISC_RETURNING_TO_SENDER )? 0xFF00FF00 : 0xFFFFFFFF;
//	Asura_DebugText::AddTextBlock(1,krak++,color,0, "Disc Health %.2f ", GetHealth() );

void AVP_ServerProjectile_Disc::Update ( void )
{
	if( !m_pxDynamicObject )
	{
		Asura_Assert( false, "Predator Disc doesn't have Dynamic Object. Something is terrible wrong. Update failed." );
		SetThisAndClientsAsDestroyed();	// Destroy disc immediately and silently.
		return;
	}

	// Update Timers.
	float fTimeStep = Asura_Timers::GetGameTimeSlice();
	m_fTimeSinceLaunch += fTimeStep;

	if( m_uDiscState == DISC_RETURNING_TO_SENDER )
	{
		m_fTimeSinceReturn += fTimeStep;
		m_fTimeSinceReturn = Asura_Maths::ClampToRange( m_fTimeSinceReturn, 0.0f, m_fReturnTimeToReachMaxBendingAngle );
	}

	AVP_ServerActor_Predator* pxServerPred = AVP_ServerActor_Predator::FindEntity(m_uOwnerGuid);
	// If Disc owner is gone, destroy this entity.
	if(!pxServerPred || pxServerPred->IsDead())
	{
		// Slowly destroy disc. When health of disc hits zero, destroy it in next Update call.
		float fNewHealth = GetHealth() - fTimeStep * 50.0f;
		SetHealth(fNewHealth);
	}
	
	// Make sure that if disc is flying to infinity, we will return it to owner's hand after certain time.
	if( m_fTimeSinceLaunch>m_fReturnTeleportTime )
	{
		SetHealth(0.0f);	// Destroy disc in next Update call.
	}

	// Make sure that physics has been properly updated meantime. Because if physics hasn't been updated since last time we have
	// updated disc, there is no point to update it again. Because disc fly is dependent on actual and correct reading from physics
	// engine.
	if(GetHealth() > 0.0f) // Replaces the "returns" as we still wanrt to call parent. Dont update out physics if we have died.
	{
		if( Asura_Physics_System::GetPhysicsUpdateCount() > m_uPhysicsCounter )	// Don't do update if Physics is behind.
		{
			// Each frame, get new linear velocity. However, make sure that physics has been updated at least once. (Hence the test for m_uPhysicsCounter before this.)
			// Later on we will change this velocity according to flight trajectory and we will set it back to dynamic object.
			m_xDiscDirection = m_pxDynamicObject->GetLinVelocity();

			float fLinVelocityMagnitude = m_xDiscDirection.Normalise();

			if( fLinVelocityMagnitude )
			{
				// Update Travel Distance
				m_fTraveledDistance += fLinVelocityMagnitude;

				// Bug Fix. When the disc is in air, it looses health. Once the health is set to zero, it will explode. However, this is not
				// what we want. Keep disc alive, because we have other specific timers to deal with this issue. (These timers designers can set in blueprint)
				// However, keep the disc alive only when it is not stuck in the wall. If it is, it will slowly loose health and explode.
				SetHealth( GetMaxHealth() );
			}
			else
			{
				// Don't update, because disc is not moving. So it is stuck somewhere, or physics system hasn't updated it yet.
				//return;
			}

			// Function will check if the disc is in auto correct mode. If not, then it will check if the disc is in laser navigation mode.
			// Also it will check if the disc is returning to your hand. In either case, it will return direction to that target. And also
			// maximum bending angle, the disc can turn to reach it. These angles differs on target and are read from blueprint.
			Asura_Vector_3 xTargetDirection( m_xDiscDirection );			// Return value from FindTargetDirectionAndMaxBendingAngle. Must be initialized now.
			float fMaxBendingAngle = 0.0f;									// Return value from FindTargetDirectionAndMaxBendingAngle. Must be initialized now.
			if(pxServerPred)
			{
				FindTargetDirectionAndMaxBendingAngle( pxServerPred, xTargetDirection, fMaxBendingAngle );
			}
		
			float fTargetDistance = xTargetDirection.Normalise();

			if( m_uDiscState==DISC_IN_FLIGHT )
			{
				// Make sure that after some time, we will return the disc to owners hand.
				if(m_fTimeSinceLaunch >= m_fReturnTime )
				{
					ReturnToSender();
				}
			}
			else
			{
				if(fTargetDistance < dbg_fDiscCollectionDistance)
				{
					SetThisAndClientsAsDestroyed(); // this destroys the disc projectile and creates the animation in the predator's hand to replace it.
				}
			}

			// Follow some point on the screen. (Enemy Actor, Laser Marker Point, or Predators Hand...)
			if( fMaxBendingAngle!=0.0f )
			{
				Asura_Quat xTurnQuat;
				xTurnQuat.RotationFromAToB( m_xDiscDirection, xTargetDirection );

				Asura_Vector_3 xTurnAxis;
				float fTurnAngle = xTurnQuat.ToRotation( xTurnAxis );
				fTurnAngle = Asura_Maths::ClampToRange( fTurnAngle, -fMaxBendingAngle, fMaxBendingAngle );
				xTurnQuat.FromRotation( fTurnAngle, xTurnAxis );

				m_xDiscDirection = xTurnQuat.RotateVector( m_xDiscDirection );

				float fSpeed = m_pxProjectileData->GetMuzzleVelocity();	// blueprint "MuzzleVelocity"

				m_pxDynamicObject->SetLinVelocity(m_xDiscDirection * fSpeed); // Fly in the stored direction
			}

			RotateDiscToMakeItLookMoreRealisticaly( m_xDiscDirection, xTargetDirection );
		}
	}
	

	// Call the base-function... (Called late, as this updates the physics counter stored in the projectile)
	PARENT::Update();
}


void AVP_ServerProjectile_Disc::RotateDiscToMakeItLookMoreRealisticaly( const Asura_Vector_3& xDiscDirection, const Asura_Vector_3& xTargetDirection )
{
	// Calculate a tilt angle to make disc looks more realistically. It will till the angle according of the fly direction of the disc. 
	// If the disc will fly to the left, rotation of the disc will also change more to the left. And same or right direction.
	Asura_Vector_3 xRotatedTarget(-xTargetDirection.z, xTargetDirection.y, xTargetDirection.x);
	float fCosAngle = xRotatedTarget*xDiscDirection;
	float fAngle = Asura_Maths::ArcCosine(fCosAngle);
	float fTargetTiltAngle = fAngle - (Asura_Maths::Deg90);

	float fTiltDiff = fTargetTiltAngle - m_fTiltAngle;
		
//	fTiltDiff = Asura_Maths::ClampToRange(fTiltDiff, -0.5f, 0.5f);	//EP. I have disabled this clamping, so disc might rotate to all directions. Now it is navigated by laser so it looks better. But I am keeping this in case, designers would not like this.
	m_fTiltAngle += fTiltDiff;

	// Set the disc tilt
	Asura_Matrix_3x3 xOri;
	Asura_Maths::OrientationFromDirection(xOri, xDiscDirection);
	Asura_Euler_3 xEuler;
	xEuler.FromMatrix(xOri);
	xEuler.z -= m_fTiltAngle;
	xEuler.ToMatrix(xOri) ;
	m_pxDynamicObject->SetOrientation(xOri);
}

// return true if we find valid enemy target. Valid means that is close enough and seeking eye of the disc is seeing it.
bool AVP_ServerProjectile_Disc::SearchForClosestTargetInSearchingCone( Asura_Vector_3& xClosestActorDirVector )	//SearchForNearTargets
{
	const float fNearbyRangeSq = m_fSeekEnemyActors_ScanDistance * m_fSeekEnemyActors_ScanDistance;	// This value is here just to improve little bit a speed.
	Asura_Vector_3 xChestPosition;

	// Reset object. Because when range is set to this value, we can consider this as empty object.
	float fClosestActorRangeSq = Asura_Maths::LargeFloat;

	for (Asura_Hashed_List_ConstIt<AVP_ServerEntity_Actor> xActorIter(AVP_ServerEntity_Actor::GetList()); !xActorIter.Done(); xActorIter.Next())
	{
		const AVP_ServerEntity_Actor* pxActor = xActorIter.GetCurrent();

		// if the actor pointer isn't valid, or if it is our actor or the actor is dead or inactive then move onto the next actor
		if ( !pxActor || pxActor->GetGuid() == GetOwnerGuid() || !pxActor->IsDamageable() || !pxActor->IsEntityActive() )
		{
			continue;
		}

		pxActor->GetChestPosition( xChestPosition );	// GetHeadPosition
		Asura_Vector_3 xToActor =  xChestPosition - GetPosition() ;
		float fActorDistSq	= xToActor.MagnitudeSquared();

		if (fActorDistSq < fNearbyRangeSq && fActorDistSq < fClosestActorRangeSq )
		{
			Asura_Vector_3 xDesiredHeading = xToActor;
			Asura_Vector_3 xCurrentHeading = m_xDiscDirection;

			xDesiredHeading.Normalise();
			xCurrentHeading.Normalise();

			// Check that if we are in the radius of the cone.
			Asura_Quat xTurnQuat;
			xTurnQuat.RotationFromAToB(xCurrentHeading, xDesiredHeading);
			Asura_Vector_3 xTurnAxis;
			float fTurnAngle = xTurnQuat.ToRotation(xTurnAxis);

			// The angle is valid, so we can check for it.
			if( fTurnAngle<m_fSeekEnemyActors_ScanAngle )
			{
				fClosestActorRangeSq = fActorDistSq;
				xClosestActorDirVector = xToActor;
			}
		}
	}

	if( fClosestActorRangeSq < Asura_Maths::LargeFloat - 10.0f )
	{
		return true;
	}
	else
	{
		return false;
	}
}


void AVP_ServerProjectile_Disc::ReturnToSender ( void )
{
	m_fTimeSinceReturn = 0.0f;
//	m_fPredictedTimeToNextTarget = 0.0f;
//	m_uTargetGuid = ASURA_GUID_UNREGISTERED;
//	m_uTargetMPHash = ASURA_HASH_ID_UNSET;
	m_uDiscState = DISC_RETURNING_TO_SENDER;
}


void AVP_ServerProjectile_Disc::OnCollision (const Asura_LOS_InputData& xLOSInputData,
											 const Asura_LOS_OutputData& xLOSOutputData)
{
	PARENT::OnCollision( xLOSInputData, xLOSOutputData );
	if ( xLOSOutputData.m_bHitEnvironment && (!GetBounceMode() || GetLaserTargetMode() ) )	// ( xLOSOutputData.m_uHitEntity != 0 && xLOSOutputData.m_uHitEntity != m_uOwnerGuid ) ||
	{
		ReturnToSender();	
	}

	if( xLOSOutputData.m_bHitEnvironment )
	{
		// Make sure we give enough time to physics to update its states.
		m_uPhysicsCounter+=3;
	}
}


void AVP_ServerProjectile_Disc::OnImpact( const Asura_Vector_3& xImpactPoint, const Asura_Vector_3& xImpactNormal /*= Asura_Maths::ZeroVector3*/ )
{
	PARENT::OnImpact( xImpactPoint, xImpactNormal );
	if( !GetBounceMode() || GetLaserTargetMode() )
	{
		ReturnToSender();
	}

	// Make sure we give enough time to physics to update its states. Well, this OnImpact is called many times, so it should be enough to add only 1 frame for each hit. 
	// Usually it is called at least 5 times every time the disc hit something.
	m_uPhysicsCounter++;
}


bool AVP_ServerProjectile_Disc::HandleMessage( const Asura_Message* pxMessage )
{
	if(pxMessage)
	{
		if(pxMessage->GetMsgID() == AVP_MSGID_RETURN_DISC)
		{
			ReturnToSender();	
			return true;
		}
	}
	return PARENT::HandleMessage(pxMessage);
}

/// End of file. ///

