
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_ServerProjectile_Flamethrower.cpp
// Author : Emil Popovic
// Date   : 27 February 2009
//
// Copyright (c) 2009 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "Entity/PhysicalObject/Projectile/AVP_ServerProjectile_Flamethrower.h"

// Asura independent header-files...
#include "Blueprint/Asura_Blueprint.h"
#include "Blueprint/Asura_Blueprint_System.h"
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "Network/Asura_Network.h"

// Game independent header-files...
#include "Actor/Vertebrates/Predator/AVP_Predator_Description.h"
#include "Blueprint/AVP_Blueprint_HashIDs.h"
#include "Entity/AVP_Entity_Classification.h"
#include "Entity/AVP_Messages.h"
#include "Entity/Damage/AVP_EntityDamage_Messages.h"
#include "Entity/PhysicalObject/Actor/AVP_Actor_Messages.h"
#include "Entity/PhysicalObject/Actor/AVP_ServerEntity_Actor.h"
#include "Entity/PhysicalObject/Projectile/AVP_Projectile_Messages.h"
#include "Entity/PhysicalObject/Projectile/AVP_GenericProjectileData.h"

////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////

static const u_int	g_uChunkVersion	= 0;


Asura_Hashed_List<AVP_ServerProjectile_Flamethrower>	AVP_ServerProjectile_Flamethrower::s_xList;

AVP_ServerProjectile_Flamethrower::AVP_ServerProjectile_Flamethrower (const Asura_Guid uGuid, const Asura_Message* pxMsg)
: Asura_Entity_Server( uGuid )
, PARENT( uGuid, pxMsg )
{
	// Update the data...
	Classification	= AVP_ENTITYCLASS_PROJECTILE_FLAMETHROWER;

	// Don't forget to store us in the list...
	s_xList.AddToList(this);
}

AVP_ServerProjectile_Flamethrower::~AVP_ServerProjectile_Flamethrower ()
{
	// Don't forget to remove us from the list...
	s_xList.RemoveFromList(this);
}



AVP_ServerProjectile_Flamethrower* AVP_ServerProjectile_Flamethrower::Create( const Asura_Message* pxMsg )
{
	AVP_ServerProjectile_Flamethrower* pxServer = new AVP_ServerProjectile_Flamethrower( Asura_Entity_System::GetCreatedByGameGuid(), pxMsg);

	// COPIED COMPLETELY FROM AVP_ServerProjectile_GenericProjectile::Create !
	pxServer->InitFromBlueprint();
	const AVP_GenericProjectileData* pxProjectileData = AVP_GenericProjectileData::GetDataByHash( pxServer->GetBlueprintHashID() );
	const AVP_Message_CreateGenericProjectile* pxData = static_cast< const AVP_Message_CreateGenericProjectile* >( pxMsg != NULL ? pxMsg->GetData() : NULL );
	if( pxData != NULL )
	{
		const float				fMuzzleVel = pxProjectileData->GetMuzzleVelocity();
		Asura_Vector_3			xVelocity( 0.0f, 0.0f, fMuzzleVel );
		const Asura_Vector_3&	xPosition = pxData->GetPosition();
		Asura_Matrix_3x3		xOrientation;
		pxData->GetOrientation().ToMatrix( xOrientation );
		xVelocity = xVelocity * xOrientation;
		float fMag = xVelocity.Normalise();
		fMag += xVelocity * pxData->GetActorVelocity();
		if ( pxData->GetLaunch() )
		{
			pxServer->Launch( xPosition, xOrientation, xVelocity * fMag, pxServer->m_uOwnerGuid, pxData->GetDamageScale() ); // Launch last as this creates the client entity
		}
		else
		{
			pxServer->LinkToParent( pxServer->m_uOwnerGuid, pxProjectileData->GetLinkMarkerPoint() );
		}
	}
	// END OF COPIED COMPLETELY FROM AVP_ServerProjectile_GenericProjectile::Create !

	return pxServer;
}


void AVP_ServerProjectile_Flamethrower::MakeClientUpdateMessage (void)
{
	Asura_Message* pxMessage = Asura_Network::CreateMessage(ASURA_MSG_CLIENTUPDATE, sizeof(AVP_Message_GenericProjectileClientUpdate), GetClassification());
	
	if(pxMessage != NULL)
	{
		// Fill in the message...
		AVP_Message_GenericProjectileClientUpdate* pxData = static_cast<AVP_Message_GenericProjectileClientUpdate*>(pxMessage->GetData());
		pxData->FillGenericProjectile(this);

		// Send the message...
		Send(pxMessage, ASURA_GUID_MY_CLIENT_ENTITIES);
	}
}

void AVP_ServerProjectile_Flamethrower::ReadFromChunkStream (Asura_Chunk_Stream& xStream)
{
	u_int	uVersion;

	// Read the version...
	xStream >> uVersion;
	Asura_Assert((uVersion <= g_uChunkVersion), "Unsupported chunk version");

	// Read the parent data...
	PARENT::ReadFromChunkStream(xStream);
}

void AVP_ServerProjectile_Flamethrower::WriteToChunkStream (Asura_Chunk_Stream& xStream) const
{
	// Write the version...
	xStream << g_uChunkVersion;

	// Write the parent data...
	PARENT::WriteToChunkStream(xStream);
}

/*
void AVP_ServerProjectile_Flamethrower::Update( void )
{
	// Call the base-function...
	PARENT::Update();
}

void AVP_ServerProjectile_Flamethrower::Launch(const Asura_Vector_3& xPosition, const Asura_Matrix_3x3& xOrientation, const Asura_Vector_3& xVelocity, const Asura_Guid uGuidToIgnore, float fDamageMultiplier)
{
	PARENT::Launch(xPosition, xOrientation, xVelocity, uGuidToIgnore, fDamageMultiplier);
}
*/

/// End of file. ///
