
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_ServerProjectile_GenericProjectile.cpp
// Author : Andy Tate
// Date   : 27-Oct-2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "Entity/PhysicalObject/Projectile/AVP_ServerProjectile_GenericProjectile.h"

// Asura independent header-files...
#include "Blueprint/Asura_Blueprint.h"
#include "Blueprint/Asura_Blueprint_System.h"
#include "Core/Asura_Timers.h"
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "Debugging/Asura_DebugLineRender.h"
#include "Network/Asura_Network.h"
#include "Physics/Asura_Physics.h"

// Game independent header-files...
#include "Blueprint/AVP_Blueprint_HashIDs.h"
#include "Core/Axon_Sound_Registry.h"
#include "Core/Axon_System.h"
#include "Entity/AVP_Entity_Classification.h"
#include "Entity/AVP_Messages.h"
#include "Entity/Damage/AVP_EntityDamage_Apply.h"
#include "Entity/Damage/AVP_EntityDamage_Messages.h"
#include "Entity/Damage/AVP_EntityDamage_Request.h"
#include "Entity/PhysicalObject/Actor/AVP_ServerEntity_Actor.h"
#include "Entity/PhysicalObject/Projectile/AVP_ClientProjectile_GenericProjectile.h"
#include "Entity/PhysicalObject/Projectile/AVP_Projectile_Messages.h"
#include "Entity/PhysicalObject/Projectile/AVP_GenericProjectileData.h"
#include "Entity/PhysicalObject/Projectile/AVP_ServerProjectile_HumanFlare.h"
#include "Entity/PhysicalObject/Actor/AVP_ClientEntity_Actor.h"
#include "Entity/PhysicalObject/Actor/AVP_Actor_Messages.h"
#include "Inventory/Items/AVP_InventoryItem_Weapon.h"
#include "SpecialFX/Asura_Decal_System.h"
////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////

static const u_int	g_uChunkVersion	= 1;

static const Asura_Hash_ID uMARKER_EMBED = 0x05c24ed9; // "embed" 


////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////

Asura_Hashed_List<AVP_ServerProjectile_GenericProjectile>	AVP_ServerProjectile_GenericProjectile::s_xList;

////////////////////////////////////////////////////////////////////////////////
// Protected functions...
////////////////////////////////////////////////////////////////////////////////

AVP_ServerProjectile_GenericProjectile::AVP_ServerProjectile_GenericProjectile( const Asura_Guid uGuid, const Asura_Message* pxMsg )
: Asura_Entity_Server( uGuid ) 
, PARENT(uGuid, pxMsg->GetFromGuid())
, m_pxProjectileData( NULL )
, m_bEmbedded( false )
, m_uTargetGuid( ASURA_GUID_UNREGISTERED )
, m_fTargetSeekingAngle(0.0f)
, m_fTargetTurningPerSecond(0.0f)
, m_uPhysicsCounter( 0 )
{
	Classification	= AVP_ENTITYCLASS_GENERICPROJECTILE;

	const AVP_Message_CreateGenericProjectile* pxData = static_cast< const AVP_Message_CreateGenericProjectile* >( pxMsg != NULL ? pxMsg->GetData() : NULL );
	Asura_Assert( pxData != NULL, "No Data" );
	if( pxData != NULL )
	{
		SetBlueprintHashID( pxData->GetProjectileBlueprintHash() );
		SetBlueprintTypeHashID( uAVP_BLUEPRINT_TYPE_PROJECTILES );
	}
	
	// Don't forget to store us in the list...
	s_xList.AddToList( this );
}

AVP_ServerProjectile_GenericProjectile::AVP_ServerProjectile_GenericProjectile( const Asura_Guid uGuid )
: Asura_Entity_Server( uGuid ) 
, PARENT(uGuid, ASURA_GUID_UNREGISTERED)
, m_pxProjectileData( NULL )
, m_bEmbedded( false )
, m_uTargetGuid( ASURA_GUID_UNREGISTERED )
, m_fTargetSeekingAngle(0.0f)
, m_fTargetTurningPerSecond(0.0f)
, m_uPhysicsCounter( 0 )
{
	Classification	= AVP_ENTITYCLASS_GENERICPROJECTILE;

	// Don't forget to store us in the list...
	s_xList.AddToList( this );
}

AVP_ServerProjectile_GenericProjectile::~AVP_ServerProjectile_GenericProjectile()
{
	s_xList.RemoveFromList( this );
}

// Note to this function... Lot of it is also copied and duplicated inside AVP_ServerProjectile_Flamethrower::Create. So any changes there, it would
// be good if flamethrower projectile would be updated accordingly too.
AVP_ServerProjectile_GenericProjectile*	AVP_ServerProjectile_GenericProjectile::Create( const Asura_Message* pxMsg )
{
	AVP_ServerProjectile_GenericProjectile*	pxServer = new AVP_ServerProjectile_GenericProjectile( Asura_Entity_System::GetCreatedByGameGuid(), pxMsg );

	pxServer->InitFromBlueprint();
	const AVP_GenericProjectileData* pxProjectileData = AVP_GenericProjectileData::GetDataByHash( pxServer->GetBlueprintHashID() );
	const AVP_Message_CreateGenericProjectile* pxData = static_cast< const AVP_Message_CreateGenericProjectile* >( pxMsg != NULL ? pxMsg->GetData() : NULL );
	if( pxData != NULL )
	{
		const float				fMuzzleVel = pxProjectileData->GetMuzzleVelocity();
		Asura_Vector_3			xVelocity( 0.0f, 0.0f, fMuzzleVel );
		Asura_Vector_3			xLaunchPos(pxData->GetPosition());
		Asura_Matrix_3x3		xOrientation;
		pxData->GetOrientation().ToMatrix( xOrientation );
		xVelocity = xVelocity * xOrientation;
		float fMag = xVelocity.Normalise();
		fMag += xVelocity * pxData->GetActorVelocity();
		if ( pxData->GetLaunch() )
		{
			Asura_Guid uTargetGuid = pxData->GetTargetGuid();
			if(uTargetGuid != ASURA_GUID_UNREGISTERED)
			{
				Asura_Vector_3			xTargetVelocity;
				Asura_Vector_3			xTargetPosition( Asura_Maths::ZeroVector3 );
				AVP_ClientEntity_Actor*	pxTarget	= AVP_ClientEntity_Actor::FindEntity( uTargetGuid );
				Asura_Assert( pxTarget != NULL, "Unable to get tagged actor" );
				if ( pxTarget != NULL )
				{
					pxTarget->GetCurrentVelocity(xTargetVelocity);
					if ( !pxTarget->GetMarkerLocationFromHash( 0xe5a85d03, &xTargetPosition, NULL ) ) // target_1
					{
						xTargetPosition = pxTarget->GetCurrentPosition();
						Asura_Assert(false, "Failed to find target markerpoint");
					}

					pxServer->SetTargetGuid(uTargetGuid); // For any homing required, set the guid to track.
				}

				AVP_InventoryItem_Weapon::CalculateFiringDataForMovingTarget( xVelocity, xLaunchPos, fMag, xTargetPosition, xTargetVelocity, 0.0f );

				xVelocity.Normalise();
			}
			
			pxServer->Launch( xLaunchPos, xOrientation, xVelocity * fMag, pxServer->m_uOwnerGuid, pxData->GetDamageScale() ); // Launch last as this creates the client entity
		}
		else
		{
			pxServer->LinkToParent( pxServer->m_uOwnerGuid, pxProjectileData->GetLinkMarkerPoint() );
		}
	}

    return pxServer;
}


void AVP_ServerProjectile_GenericProjectile::Launch (const Asura_Vector_3 &xPosition,
										  const Asura_Matrix_3x3 &xOrientation,
										  const Asura_Vector_3 &xVelocity,
										  const Asura_Guid uGuidToIgnore,
										  float fDamageMultiplier)
{
	PARENT::Launch(xPosition,xOrientation,xVelocity,uGuidToIgnore,fDamageMultiplier);

	m_uPhysicsCounter = Asura_Physics_System::GetPhysicsUpdateCount();
}
////////////////////////////////////////////////////////////////////////////////
// Public functions...
////////////////////////////////////////////////////////////////////////////////

void AVP_ServerProjectile_GenericProjectile::ReadFromChunkStream(Asura_Chunk_Stream& xStream)
{
	u_int	uVersion;

	// Read the version...
	xStream >> uVersion;
	Asura_Assert((uVersion <= g_uChunkVersion), "Unsupported chunk version");

	xStream >> m_bEmbedded;

	// Read the parent data...
	PARENT::ReadFromChunkStream(xStream);
}

void AVP_ServerProjectile_GenericProjectile::WriteToChunkStream(Asura_Chunk_Stream& xStream) const
{
	// Write the version...
	xStream << g_uChunkVersion;

	xStream << m_bEmbedded;

	// Write the parent data...
	PARENT::WriteToChunkStream(xStream);
}

void AVP_ServerProjectile_GenericProjectile::InitialiseFromBlueprint( const Asura_Blueprint& xBlueprint )
{
	PARENT::InitialiseFromBlueprint( xBlueprint );
	m_pxProjectileData = AVP_GenericProjectileData::GetDataByHash( xBlueprint.GetHashID() );
	Asura_Assert( m_pxProjectileData != NULL, "No projectile data!" );
	if ( m_pxProjectileData != NULL )
	{
		// Set the stats...
		SetHealth( m_pxProjectileData->GetTimer() );
		SetDestructible( false );

		if ( m_pxProjectileData->GetSkin() == ASURA_HASH_ID_UNSET )
		{
			SetPhysicalObjectFlags( GetPhysicalObjectFlags() | ASURA_PHYSICALOBJECT_FLAG_NOSHADOW );
		}

		if ( m_pxDynamicObject != NULL )
		{
			const Asura_Collision_Box* pxCollisionObject( m_pxProjectileData->GetCollisionBox() );

			m_pxDynamicObject->SetCollisionProxy( m_pxProjectileData->GetCollisionProxy() );
			m_pxDynamicObject->SetCollisionObject( pxCollisionObject );
			m_pxDynamicObject->SetAxisConstraint( Asura_Physics_Object::AXIS_CONSTRAINT_NONE );
			m_pxDynamicObject->SetFeelsGravity	( true );
			m_pxDynamicObject->SetInvMass		( 1.0f / m_pxProjectileData->GetMass() );
			m_pxDynamicObject->SetGravityScale	( m_pxProjectileData->GetGravityScale() );
			m_pxDynamicObject->SetElasticity	( m_pxProjectileData->GetElasticity() );
			m_pxDynamicObject->SetDrag			( m_pxProjectileData->GetDrag() );
			m_pxDynamicObject->SetFriction		( m_pxProjectileData->GetFriction() );
			m_pxDynamicObject->SetAngularDrag	( m_pxProjectileData->GetAngularDrag() );
			m_pxDynamicObject->SetOnlyCollideWithEnvironmentAndStaticObjects( m_pxProjectileData->GetOnlyCollideWithEnvironmentAndStaticObjects() );
			m_pxDynamicObject->SetReportEnvironmentCollisions( m_pxProjectileData->GetImpactType() == AVP_GenericProjectileData::IMPACT_EMBED );
		}
	}

	Asura_Value xValue;
	if(xBlueprint.GetParameterValueByHashID( 0x652BB93E, xValue )) // Asura_GetHashID( "Target.SeekAngle" )
	{
		m_fTargetSeekingAngle = xValue.GetFloatValue();

		// Also make sure that we are using only half of this angle, because we want to check for both sides (left and right).
		// So if there is angle 30, we will store this value as 15. Then later on we will check 15 degrees in all directions, thus we make it cone with size 30.
		m_fTargetSeekingAngle = Asura_Maths::DegToRad( m_fTargetSeekingAngle*0.5f );
	}

	if(xBlueprint.GetParameterValueByHashID( 0xFB30FD98, xValue )) // Asura_GetHashID( "Target.TurnAnglePerSecond" )
	{
		m_fTargetTurningPerSecond = xValue.GetFloatValue();
		m_fTargetTurningPerSecond = Asura_Maths::DegToRad( m_fTargetTurningPerSecond );
	}
}

void AVP_ServerProjectile_GenericProjectile::Update( void )
{
	// Call the base-function...
	PARENT::Update();
	
	if ( !m_pxProjectileData )
	{
		return;
	}

	if(GetParentGuid() != ASURA_GUID_UNREGISTERED)
	{
		//We are attached to something, update position?
		Asura_Vector_3 xPos;
		AVP_ClientProjectile_GenericProjectile* pxClient = AVP_ClientProjectile_GenericProjectile::FindEntity(GetGuid());
		if(pxClient)
		{
			Asura_Animation* pxAnimation	= pxClient->GetAnimation();
			if(pxAnimation)
			{
				SetPosition(pxAnimation->GetPosition());
			}
		}
	}

	if ( !m_bEmbedded )
	{
		// Set health
		SetHealth( Asura_Maths::Max( GetHealth() - Asura_Timers::GetGameTimeSlice(), 0.0f ) );

		if(m_uTargetGuid != ASURA_GUID_UNREGISTERED && m_fTargetSeekingAngle != 0.0f ) // Dont do anything if we dont have a seek angle!
		{
			if( Asura_Physics_System::GetPhysicsUpdateCount() > m_uPhysicsCounter )	// Don't do update if Physics is behind.
			{
				AVP_ServerEntity_Actor*	pxServerTarget = AVP_ServerEntity_Actor::FindEntity( m_uTargetGuid );
				
				if(pxServerTarget)
				{
					// Check that if we are in the radius of the cone.
					Asura_Vector_3 xChestPosition;
					pxServerTarget->GetChestPosition( xChestPosition );
					
					Asura_Vector_3 xTargetVelocity = pxServerTarget->GetVelocity();
					Asura_Vector_3 xTargetDirection = GetVelocity();
					float fMag = xTargetDirection.Normalise();

					Asura_Vector_3 xCurrentDirection = xTargetDirection;

					AVP_InventoryItem_Weapon::CalculateFiringDataForMovingTarget( xTargetDirection, GetPosition(), fMag, xChestPosition, xTargetVelocity, 0.0f );
					xTargetDirection.Normalise();

					Asura_Quat xTurnQuat;
					xTurnQuat.RotationFromAToB(xCurrentDirection, xTargetDirection);
					Asura_Vector_3 xTurnAxis;
					float fTurnAngle = xTurnQuat.ToRotation(xTurnAxis);

					// The angle is valid, so we can check for it.
					if( fTurnAngle<m_fTargetSeekingAngle )
					{
						Asura_Quat xTurnQuat;
						xTurnQuat.RotationFromAToB( xCurrentDirection, xTargetDirection );

						Asura_Vector_3 xTurnAxis;
						float fTurnAngle = xTurnQuat.ToRotation( xTurnAxis );
						float fFrameTurnAngle = m_fTargetTurningPerSecond * Asura_Timers::GetGameTimeSlice();
						fTurnAngle = Asura_Maths::ClampToRange( fTurnAngle, -fFrameTurnAngle, fFrameTurnAngle );
						xTurnQuat.FromRotation( fTurnAngle, xTurnAxis );

						xCurrentDirection = xTurnQuat.RotateVector( xCurrentDirection );

						m_pxDynamicObject->SetLinVelocity(xCurrentDirection * fMag); // Fly in the stored direction
					}	
				}
			}
		}

		if ( GetParentGuid() == ASURA_GUID_UNREGISTERED ) // only do los collisions if we're not embedded or attached
		{
			if ( !CheckForCollisionsUsingLOS( GetPosition(), GetVelocity() ) )
			{
				if(m_pxDynamicObject->HasHitSomething())
				{
					if ( m_pxDynamicObject->GetEnvironmentCollisionReportList() )
					{
						OnImpact( GetPosition(), Asura_Vector_3( m_pxDynamicObject->GetEnvironmentCollisionReportList()->GetFaceNormal() ) );
					}
					else
					{
						OnImpact( GetPosition(), Asura_Maths::ZeroVector3 );
					}
				}
			}	
		}

		if ( GetHealth() <= 0.0f )
		{
			if ( m_pxProjectileData->GetImpactType() == AVP_GenericProjectileData::IMPACT_EXPLODE )
			{
				Explode();
			}
			else
			{
				SetThisAndClientsAsDestroyed();
			}
		}
	}

	m_uPhysicsCounter = Asura_Physics_System::GetPhysicsUpdateCount();
}

bool AVP_ServerProjectile_GenericProjectile::CheckForCollisionsUsingLOS( const Asura_Vector_3& xPosition, const Asura_Vector_3& xVelocity )
{
	if ( m_pxProjectileData && m_pxProjectileData->GetDoLOSCollisions() && !IsEmbedded())
	{
		return PARENT::CheckForCollisionsUsingLOS( xPosition, xVelocity );
	}
	else
	{
		return false;
	}
}

void AVP_ServerProjectile_GenericProjectile::OnImpact( const Asura_Vector_3& xImpactPoint, const Asura_Vector_3& xImpactNormal /*= Asura_Maths::ZeroVector3*/ )
{
	if ( !m_pxProjectileData )
	{
		return;
	}

	if(!IsDisarmed()) // if disarmed, don't cause damage
	{
		const float				fMaxAmount			= m_pxProjectileData->GetImpactDamageAtNearRange() * m_fDamageMultiplier;
		const float				fMaxImpulseStrength	= m_pxProjectileData->GetImpactImpulseAtNearRange() * m_fDamageMultiplier;
		Asura_Vector_3			xDirection			= xImpactPoint - GetPosition();
		const float				fMag				= xDirection.Magnitude();

		const AVP_EntityDamage_RequestLinear xDamage(	m_pxProjectileData->GetImpactDamageType(),
			GetGuid(),
			ASURA_HASH_ID_UNSET,
			fMaxAmount,
			fMaxImpulseStrength,
			m_uOwnerGuid,
			xImpactPoint, 
			xDirection,
			fMag + 10.0f,
			m_uOwnerGuid,
			ASURA_GUID_UNREGISTERED);
		xDamage.Apply( *this );
	}

	switch ( m_pxProjectileData->GetImpactType() )
	{
		case AVP_GenericProjectileData::IMPACT_EXPLODE:
		{
			Asura_Matrix_3x3 xOri;
			if ( xImpactNormal.ApproximatelyEquals( Asura_Maths::ZeroVector3 ) )
			{
				xOri = GetOrientation();
			}
			else
			{
				Asura_Maths::OrientationFromDirection( xOri, xImpactNormal );
			}
			TeleportTo( xImpactPoint, xOri, true );
			Explode();
		}
		break;

		case AVP_GenericProjectileData::IMPACT_EMBED:
		{
			if ( !m_bEmbedded )
			{
				const Asura_Matrix_3x3& xOri( GetOrientation() );
				Asura_Vector_3 xForwardDir( xOri.e13, xOri.e23, xOri.e33 );

				if ( xImpactNormal * xForwardDir < -m_pxProjectileData->GetCosEmbedAngle() )
				{
					// embed in this wall
					Asura_Vector_3 xEmbedPosition = xImpactPoint;
					const AVP_ClientProjectile_GenericProjectile* pxClient = AVP_ClientProjectile_GenericProjectile::FindEntity( GetGuid() );
					if ( pxClient != NULL )
					{
						Asura_Vector_3 xEmbedMPPos;
						if ( pxClient->GetMarkerLocationFromHash( uMARKER_EMBED, &xEmbedMPPos, NULL, false ) ) // "embed" 
						{
							Asura_AVector3 xMPPos( xEmbedMPPos );
							xMPPos = xMPPos * xOri;
							xEmbedPosition -= Asura_Vector_3( xMPPos );
						}
					}
					TeleportTo( xEmbedPosition, GetOrientation(), true );
					m_pxDynamicObject->SetIgnore( true );
					m_bEmbedded = true;
					OnEmbed( xImpactPoint, xImpactNormal );
				}
			}
		}
		break;

		case AVP_GenericProjectileData::IMPACT_NONE:
		{
			
		}
		break;

		default:
		{
			Asura_Assert( false, "Unhandled case" );
		}
		break;
	}
}

void AVP_ServerProjectile_GenericProjectile::OnCollision( const Asura_LOS_InputData& xLOSInputData, const Asura_LOS_OutputData& xLOSOutputData )
{
	if ( !m_pxProjectileData )
	{
		return;
	}
	const Asura_Guid	uVictimGuid	= xLOSOutputData.m_uHitEntity;

	if (uVictimGuid != 0)
	{
		AVP_ServerEntity_PhysicalObject* pxServer = AVP_ServerEntity_PhysicalObject::FindEntity(uVictimGuid);
		if ( pxServer != NULL )
		{
			const Asura_Vector_3&					xDirection			= xLOSInputData.m_xDirection;
			const Asura_Vector_3					xPosition			= xLOSInputData.m_xOrigin + (xDirection * xLOSOutputData.m_fDistanceToIntersection);
			if ( m_pxProjectileData->GetImpactType() == AVP_GenericProjectileData::IMPACT_EXPLODE )
			{
				TeleportTo( xPosition, GetOrientation(), true );
				Explode();
			}
			else if(m_pxProjectileData->GetImpactType() == AVP_GenericProjectileData::IMPACT_EMBED)
			{
				// Attach to bone we have hit, this allow it to work on anything that moves
				const Asura_Vector_3 xPosition(xLOSInputData.m_xOrigin + (xLOSInputData.m_xDirection * xLOSOutputData.m_fDistanceToIntersection));
				const Asura_Vector_3 xObjRelPosition(xPosition - pxServer->GetPosition());

				Asura_Matrix_3x3 xOri;
				Asura_Quat xQuat;
				Asura_Maths::OrientationFromNormal(xOri ,xLOSOutputData.m_xNormal);
				
				Asura_Matrix_3x3 xObjOri = pxServer->GetOrientation();
				xObjOri.Transpose();
				
				Asura_Quat xObjRelOri;
				xObjRelOri.FromMatrix(xObjOri*xOri);

				xQuat.FromMatrix(xOri);

				LinkToParent(uVictimGuid, xObjRelPosition*xObjOri, xObjRelOri, xLOSOutputData.m_iHitBoneID );
				OnEmbed( xPosition, xLOSOutputData.m_xNormal );
			}
			
			if ( m_pxProjectileData->GetImpactDamageType() != ASURA_HASH_ID_UNSET )
			{
				const Asura_Guid						uAttackerGuid		= GetOwnerGuid();
				const float								fMaxAmount			= m_pxProjectileData->GetImpactDamageAtNearRange() * m_fDamageMultiplier;
				const float								fMaxImpulseStrength	= m_pxProjectileData->GetImpactImpulseAtNearRange() * m_fDamageMultiplier;
				const Asura_Vector_3&					xNormal				= xLOSOutputData.m_xNormal;
				const int								iBoneID				= xLOSOutputData.m_iHitBoneID;

				const AVP_EntityDamage_RequestVictim xDamage(	m_pxProjectileData->GetImpactDamageType(),
																uAttackerGuid,
																ASURA_HASH_ID_UNSET,
																uVictimGuid,
																fMaxAmount,
																xPosition,
																xDirection,
																xNormal,
																fMaxImpulseStrength,
																iBoneID,
																false,
																m_uOwnerGuid );
				xDamage.Apply();
			}
		}
	}
	else
	{
		Asura_Assert( xLOSOutputData.m_bHitEnvironment, "This shold be true" );
		// Hit environment;
		const Asura_Vector_3 xPosition(xLOSInputData.m_xOrigin + (xLOSInputData.m_xDirection * xLOSOutputData.m_fDistanceToIntersection));
		OnImpact( xPosition, xLOSOutputData.m_xNormal );
	}
}

void AVP_ServerProjectile_GenericProjectile::Explode()
{
	if ( !m_pxProjectileData )
	{
		return;
	}
	if(!IsDisarmed()) // Only projectiles that are still armed do damage
	{
		if ( m_pxProjectileData->GetRadialDamageType() != ASURA_HASH_ID_UNSET )
		{
			Asura_Assertf(m_fDamageMultiplier <= 1.0f , ("AVP_ServerProjectile_GenericProjectile::Explode - suspiciously high damage multiplier may create quite a force. 0x%x", GetBlueprintHashID() ));
			const float								fMaxAmount					= m_pxProjectileData->GetRadialDamageAtNearRange() * m_fDamageMultiplier;
			const float								fMaxImpulseStrength			= m_pxProjectileData->GetRadialImpulseAtNearRange() * m_fDamageMultiplier;
			const float								fRadiusFor100PercentDamage	= m_pxProjectileData->GetRadialNearRange();
			const float								fRadiusFor0PercentDamage	= m_pxProjectileData->GetRadialFarRange();

			const AVP_EntityDamage_RequestRadial	xDamage(m_pxProjectileData->GetRadialDamageType(),
				GetOwnerGuid(),
				ASURA_HASH_ID_UNSET,
				fMaxAmount,
				fMaxImpulseStrength,
				GetGuid(),
				fRadiusFor100PercentDamage,
				fRadiusFor0PercentDamage,
				true,
				m_uOwnerGuid);
			xDamage.Apply(*this);
		}

		if ( m_pxProjectileData->GetSecondaryRadialDamageType() != ASURA_HASH_ID_UNSET )
		{
			const float								fMaxAmount2					= m_pxProjectileData->GetSecondaryRadialDamageAtNearRange() * m_fDamageMultiplier;
			const float								fMaxImpulseStrength2		= m_pxProjectileData->GetSecondaryRadialImpulseAtNearRange() * m_fDamageMultiplier;
			const float								fRadiusFor100PercentDamage2	= m_pxProjectileData->GetSecondaryRadialNearRange();
			const float								fRadiusFor0PercentDamage2	= m_pxProjectileData->GetSecondaryRadialFarRange();

			const AVP_EntityDamage_RequestRadial	xDamage(m_pxProjectileData->GetSecondaryRadialDamageType(),
				GetOwnerGuid(),
				ASURA_HASH_ID_UNSET,
				fMaxAmount2,
				fMaxImpulseStrength2,
				GetGuid(),
				fRadiusFor100PercentDamage2,
				fRadiusFor0PercentDamage2,
				true,
				m_uOwnerGuid);
			xDamage.Apply(*this);
		}
	}

	SetThisAndClientsAsDestroyed();
}

void AVP_ServerProjectile_GenericProjectile::MakeClientUpdateMessage( void )
{
	Asura_Message* pxMessage = Asura_Network::CreateMessage(ASURA_MSG_CLIENTUPDATE, sizeof(AVP_Message_GenericProjectileClientUpdate), GetClassification());

	if(pxMessage != NULL)
	{
		// Fill in the message...
		AVP_Message_GenericProjectileClientUpdate* pxData = static_cast<AVP_Message_GenericProjectileClientUpdate*>(pxMessage->GetData());
		pxData->FillGenericProjectile(this);

		// Send the message...
		Send(pxMessage, ASURA_GUID_MY_CLIENT_ENTITIES);
	}
}

bool AVP_ServerProjectile_GenericProjectile::IsAvoidedByAI() const
{
	return m_pxProjectileData && m_pxProjectileData->GetAvoidedByAI();
}

bool AVP_ServerProjectile_GenericProjectile::IsForwardPredictedByAI() const
{
	return m_pxProjectileData && m_pxProjectileData->GetForwardPredictedByAI();
}

/// End of file. ///
