//////////////////////////////////////////////////////////////////////////////
//
// AVP_ServerEntity_SpawnController.cpp
//
// Copyright (C) 2008 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"

// header
#include "Entity/Spawners/AVP_ServerEntity_SpawnController.h"

// asura
#include "ChunkSystem/Asura_Chunk_Stream.h"

// avp
#include "Core/Asura_Timers.h"
#include "Entity/AVP_Entity_Classification.h"
#include "Entity/AVP_Entity_EventIDs.h"
#include "Entity/AVP_Messages.h"
#include "Entity/GamePlayVolume/AVP_ServerEntity_GamePlayVolume.h"
#include "Entity/PhysicalObject/Actor/AVP_ServerEntity_Actor.h"
#include "Entity/PlayerInfo/AVP_ServerEntity_PlayerInfo.h"
#include "Entity/Spawners/AVP_ServerEntity_SpawnPoint2.h"

#ifdef ASURA_TOOLS
// tools
#include "EntityTemplate/Tools_EntityTemplate_System.h"
#endif

//////////////////////////////////////////////////////////////////////////////
//
// AVP_ServerEntity_SpawnController
//
//////////////////////////////////////////////////////////////////////////////

Asura_Hashed_List< AVP_ServerEntity_SpawnController > AVP_ServerEntity_SpawnController::s_xList;

AVP_ServerEntity_SpawnController::AVP_ServerEntity_SpawnController( const Asura_Guid uGuid )
: PARENT( uGuid )
, m_iMaxInPlay( 3 )
, m_iMaxSpawnCount( 6 )
, m_fSpawnPeriodMin( 1.0f )
, m_fSpawnPeriodMax( 1.0f )
, m_uSpawneeSquadGuid( ASURA_GUID_UNREGISTERED )
, m_uNumSpawneeTypes( 0 )
, m_pxSpawneeTypes( NULL )
, m_iSpawnScheme( AVP_SS_RANDOM )
, m_fSpawnDistance( 0.0f )
, m_uNumSpawnPoints( 0 )
, m_pxSpawnPoints( NULL )
, m_uNumGPVolumes( 0 )
, m_pxGPVolumes( NULL )
, m_fSpawnTimer( 0.0f )
, m_uTotalSpawnCount( 0 )
, m_iSpawneeCount( 0 )
, m_iSpawneeArraySize( 0 )
, m_pxSpawneeGuids( NULL )
{
	Classification = AVP_ENTITYCLASS_SPAWNCONTROLLER;

	SetNeverUpdateClient();

	s_xList.AddToList( this );
}

AVP_ServerEntity_SpawnController::~AVP_ServerEntity_SpawnController()
{
	delete [] m_pxSpawneeTypes;
	delete [] m_pxSpawnPoints;
	delete [] m_pxGPVolumes;
	delete [] m_pxSpawneeGuids;
	m_pxSpawneeTypes = NULL;
    m_pxSpawnPoints  = NULL;
    m_pxGPVolumes    = NULL;
	m_pxSpawneeGuids = NULL;
	m_iSpawneeArraySize = 0;

	s_xList.RemoveFromList( this );
}

void AVP_ServerEntity_SpawnController::WriteToChunkStream( Asura_Chunk_Stream& xStream ) const
{
	xStream << uCURRENT_CHUNK_VERSION;

	PARENT::WriteToChunkStream( xStream );

	xStream << m_iMaxInPlay;
	xStream << m_iMaxSpawnCount;
	xStream << m_fSpawnPeriodMin;
	xStream << m_fSpawnPeriodMax;
	xStream << m_uSpawneeSquadGuid;

	xStream << m_uNumSpawneeTypes;
	for ( u_int u = 0; u < m_uNumSpawneeTypes; ++u )
	{
		xStream << m_pxSpawneeTypes[ u ].m_uBPHashID;
		xStream << m_pxSpawneeTypes[ u ].m_uBPTypeHashID;
		xStream << m_pxSpawneeTypes[ u ].m_fWeight;
	}

	xStream << m_iSpawnScheme;
	xStream << m_fSpawnDistance;

	xStream << m_uNumSpawnPoints;
	for ( u_int u = 0; u < m_uNumSpawnPoints; ++u )
	{
		xStream << m_pxSpawnPoints[ u ].m_uSpawnerGuid;
		xStream << m_pxSpawnPoints[ u ].m_fWeight;
	}

	xStream << m_uNumGPVolumes;
	for ( u_int u = 0; u < m_uNumGPVolumes; ++u )
	{
		xStream << m_pxGPVolumes[ u ].m_uSpawnerGuid;
		xStream << m_pxGPVolumes[ u ].m_fWeight;
	}

	xStream << m_uTotalSpawnCount;
	xStream << m_iSpawneeCount;
	for ( int i = 0; i < m_iSpawneeCount; ++i )
	{
		xStream << m_pxSpawneeGuids[ i ];
	}
}

void AVP_ServerEntity_SpawnController::ReadFromChunkStream( Asura_Chunk_Stream& xStream )
{
	u_int uVersion;
	xStream	>> uVersion;

	PARENT::ReadFromChunkStream( xStream );

	if ( uVersion == uCURRENT_CHUNK_VERSION )
	{
		xStream >> m_iMaxInPlay;
		xStream >> m_iMaxSpawnCount;
		xStream >> m_fSpawnPeriodMin;
		xStream >> m_fSpawnPeriodMax;
		xStream >> m_uSpawneeSquadGuid;

		xStream >> m_uNumSpawneeTypes;
		if ( m_uNumSpawneeTypes > 0 )
		{
			m_pxSpawneeTypes = new BlueprintAndWeight[ m_uNumSpawneeTypes ];
			for ( u_int u = 0; u < m_uNumSpawneeTypes; ++u )
			{
				xStream >> m_pxSpawneeTypes[ u ].m_uBPHashID;
				xStream >> m_pxSpawneeTypes[ u ].m_uBPTypeHashID;
				xStream >> m_pxSpawneeTypes[ u ].m_fWeight;
			}
		}

		xStream >> m_iSpawnScheme;
		xStream >> m_fSpawnDistance;

		xStream >> m_uNumSpawnPoints;
		if ( m_uNumSpawnPoints > 0 )
		{
			m_pxSpawnPoints = new SpawnerAndWeight[ m_uNumSpawnPoints ];
			for ( u_int u = 0; u < m_uNumSpawnPoints; ++u )
			{
				xStream >> m_pxSpawnPoints[ u ].m_uSpawnerGuid;
				xStream >> m_pxSpawnPoints[ u ].m_fWeight;
			}
		}

		xStream >> m_uNumGPVolumes;
		if ( m_uNumGPVolumes > 0 )
		{
			m_pxGPVolumes = new SpawnerAndWeight[ m_uNumGPVolumes ];
			for ( u_int u = 0; u < m_uNumGPVolumes; ++u )
			{
				xStream >> m_pxGPVolumes[ u ].m_uSpawnerGuid;
				xStream >> m_pxGPVolumes[ u ].m_fWeight;
			}
		}

		xStream >> m_uTotalSpawnCount;
		xStream >> m_iSpawneeCount;
		ReallocateSpawneeArray( m_iSpawneeCount );
		for ( int i = 0; i < m_iSpawneeCount; ++i )
		{
			xStream >> m_pxSpawneeGuids[ i ];
		}
	}
	else
	{
		Asura_Assertf( false, ( "AVP_ServerEntity_SpawnController::ReadFromChunkStream - unhandled chunk version (%u)", uVersion ) );
	}
}

bool AVP_ServerEntity_SpawnController::HandleMessage( const Asura_Message* pxMessage )
{
	if ( pxMessage->GetMsgID() == AVP_MSGID_SPAWN_SET_MAX_IN_PLAY )
	{
		const AVP_Msg_Spawn_SetMaxInPlay* pxData = static_cast< const AVP_Msg_Spawn_SetMaxInPlay* >( pxMessage->GetData() );
		m_iMaxInPlay = pxData->m_iMaxInPlay;
		return true;
	}
	else if ( pxMessage->GetMsgID() == AVP_MSGID_SPAWN_SET_MAX_SPAWN_COUNT )
	{
		const AVP_Msg_Spawn_SetMaxSpawnCount* pxData = static_cast< const AVP_Msg_Spawn_SetMaxSpawnCount* >( pxMessage->GetData() );
		m_iMaxSpawnCount = pxData->m_iMaxSpawnCount;
		return true;
	}
	else if ( pxMessage->GetMsgID() == AVP_MSGID_SPAWN_RESET_SPAWN_COUNTER )
	{
		m_uTotalSpawnCount = 0;
		return true;
	}
	else
	{
		return PARENT::HandleMessage( pxMessage );
	}
}

void AVP_ServerEntity_SpawnController::Update()
{
	PARENT::Update();

	m_fSpawnTimer -= Asura_Timers::GetGameTimeSlice();
	const u_int uMaxInPlayTopUp  = static_cast< u_int >( Asura_Maths::Max( m_iMaxInPlay - m_iSpawneeCount, 0 ) );
	const u_int uMaxAllowedTotal = static_cast< u_int >( m_iMaxSpawnCount ) - m_uTotalSpawnCount;
	u_int uNumToSpawn = Asura_Maths::Min( uMaxInPlayTopUp, uMaxAllowedTotal );
	while ( m_fSpawnTimer <= 0.0f && uNumToSpawn-- > 0 )
	{
		ScoreAndSpawn( NULL, NULL );
	}
}

#ifdef ASURA_TOOLS
void AVP_ServerEntity_SpawnController::ApplyChangesForTemplateInstance( Tools_EntityTemplateInstance* pxInstance )
{
	PARENT::ApplyChangesForTemplateInstance( pxInstance );

	m_uSpawneeSquadGuid = pxInstance->ConvertGuid( m_uSpawneeSquadGuid );

	for ( u_int u = 0; u < m_uNumSpawnPoints; ++u )
	{
		m_pxSpawnPoints[ u ].m_uSpawnerGuid = pxInstance->ConvertGuid( m_pxSpawnPoints[ u ].m_uSpawnerGuid );
	}

	for ( u_int u = 0; u < m_uNumGPVolumes; ++u )
	{
		m_pxGPVolumes[ u ].m_uSpawnerGuid = pxInstance->ConvertGuid( m_pxGPVolumes[ u ].m_uSpawnerGuid );
	}

	// There shouldn't be any spawnees registered in tools.
}
#endif

void AVP_ServerEntity_SpawnController::ScoreSpawners( const Asura_ScoreParams* pxScoreParams, float& fHighestScore, Asura_Guid& uBestGuid ) const
{
	fHighestScore = 0.0f;
	uBestGuid	  = ASURA_GUID_UNREGISTERED;

	if ( !IsEntityActive() )
	{
		return;
	}
	if ( m_fSpawnTimer > 0.0f )
	{
		return;
	}
	// -1 => 0xffffffff so this test will still work.
	if ( m_uTotalSpawnCount >= static_cast< u_int >( m_iMaxSpawnCount ) )
	{
		return;
	}
	if ( m_iSpawneeCount >= m_iMaxInPlay )
	{
		return;
	}
	if ( !pxScoreParams )
	{
		return;
	}

	const AVP_ScoreParams* pxAVPScoreParams = static_cast< const AVP_ScoreParams* >( pxScoreParams );

	// Check the blueprint matches.
	if ( m_uNumSpawneeTypes )
	{
		bool bMatched = false;
		for ( u_int u = 0; u < m_uNumSpawneeTypes; ++u )
		{
			if ( pxAVPScoreParams->m_uBPTypeHashID == m_pxSpawneeTypes[ u ].m_uBPTypeHashID &&
				pxAVPScoreParams->m_uBPHashID == m_pxSpawneeTypes[ u ].m_uBPHashID )
			{
				bMatched = true;
				break;
			}
		}

		if ( !bMatched )
		{
			return;
		}
	}

	float fLocalHighestScore = 0.0f;
	// We keep a list of the best guids and pick randomly from it if necessary.
	enum { MAX_BEST_GUIDS = 64 };
	u_int	   uNumBestGuids = 0;
	Asura_Guid auBestGuids[ MAX_BEST_GUIDS ];
	float	   afWeights[ MAX_BEST_GUIDS ];

	for ( u_int u = 0; u < m_uNumSpawnPoints; ++u )
	{
		const AVP_ServerEntity_SpawnPoint2* pxSpawnPoint = AVP_ServerEntity_SpawnPoint2::FindEntity( m_pxSpawnPoints[ u ].m_uSpawnerGuid );
		if ( pxSpawnPoint )
		{
			float	   fScore = 0.0f;
			Asura_Guid uGuid  = ASURA_GUID_UNREGISTERED;
			pxSpawnPoint->ScoreSpawners( pxScoreParams, fScore, uGuid );

			if ( fScore > fLocalHighestScore )
			{
                fLocalHighestScore           = fScore;
                uNumBestGuids                = 0;
				auBestGuids[ uNumBestGuids ] = uGuid;
				afWeights[ uNumBestGuids ]   = m_pxSpawnPoints[ u ].m_fWeight;
				++uNumBestGuids;
			}
			else if ( fScore == fLocalHighestScore )
			{
				if ( uNumBestGuids == MAX_BEST_GUIDS )
				{
					uNumBestGuids = MAX_BEST_GUIDS - 1;
				}
				auBestGuids[ uNumBestGuids ] = uGuid;
				afWeights[ uNumBestGuids ]	 = m_pxSpawnPoints[ u ].m_fWeight;
				++uNumBestGuids;
			}
		}
	}

	for ( u_int u = 0; u < m_uNumGPVolumes; ++u )
	{
		const AVP_ServerEntity_GamePlayVolume* pxGPV = AVP_ServerEntity_GamePlayVolume::FindEntity( m_pxGPVolumes[ u ].m_uSpawnerGuid );
		if ( pxGPV )
		{
			float	   fScore = 0.0f;
			Asura_Guid uGuid  = ASURA_GUID_UNREGISTERED;
			pxGPV->ScoreSpawners( pxScoreParams, fScore, uGuid );

			if ( fScore > fLocalHighestScore )
			{
                fLocalHighestScore           = fScore;
                uNumBestGuids                = 0;
				auBestGuids[ uNumBestGuids ] = uGuid;
				afWeights[ uNumBestGuids ]   = m_pxGPVolumes[ u ].m_fWeight;
				++uNumBestGuids;
			}
			else if ( fScore == fLocalHighestScore )
			{
				if ( uNumBestGuids == MAX_BEST_GUIDS )
				{
					uNumBestGuids = MAX_BEST_GUIDS - 1;
				}
				auBestGuids[ uNumBestGuids ] = uGuid;
				afWeights[ uNumBestGuids ]	 = m_pxGPVolumes[ u ].m_fWeight;
				++uNumBestGuids;
			}
		}
	}

	// Pick from our list of best guids.
	if ( uNumBestGuids > 0 )
	{
		fHighestScore = fLocalHighestScore;

		float fTotalWeighting = 0.0f;
		for ( u_int u = 0; u < uNumBestGuids; ++u )
		{
			fTotalWeighting += afWeights[ u ];
		}

		const float fRandomWeight = Asura_Random::GetFloat( 0.0f, fTotalWeighting );
		float fCumulativeWeighting = 0.0f;
		for ( u_int u = 0; u < uNumBestGuids; ++u )
		{
			fCumulativeWeighting += afWeights[ u ];
			if ( fRandomWeight <= fCumulativeWeighting )
			{
				uBestGuid = auBestGuids[ u ];
				break;
			}
		}
	}
}

bool AVP_ServerEntity_SpawnController::OwnsSpawnerGuid( const Asura_Guid uSpawnerGuid ) const
{
	for ( u_int u = 0; u < m_uNumSpawnPoints; ++u )
	{
		const AVP_ServerEntity_SpawnPoint2* pxSpawnPoint = AVP_ServerEntity_SpawnPoint2::FindEntity( m_pxSpawnPoints[ u ].m_uSpawnerGuid );
		if ( pxSpawnPoint && pxSpawnPoint->OwnsSpawnerGuid( uSpawnerGuid ) )
		{
			return true;
		}
	}

	for ( u_int u = 0; u < m_uNumGPVolumes; ++u )
	{
		const AVP_ServerEntity_GamePlayVolume* pxGPV = AVP_ServerEntity_GamePlayVolume::FindEntity( m_pxGPVolumes[ u ].m_uSpawnerGuid );
		if ( pxGPV && pxGPV->OwnsSpawnerGuid( uSpawnerGuid ) )
		{
			return true;
		}
	}

	return false;
}

Asura_Guid AVP_ServerEntity_SpawnController::Spawn( const Asura_Guid uSpawnerGuid, const Asura_SpawnParams* pxSpawnParams )
{
	Asura_Guid uSpawneeGuid = ASURA_GUID_UNREGISTERED;

	for ( u_int u = 0; u < m_uNumSpawnPoints; ++u )
	{
		AVP_ServerEntity_SpawnPoint2* pxSpawnPoint = AVP_ServerEntity_SpawnPoint2::FindEntity( m_pxSpawnPoints[ u ].m_uSpawnerGuid );
		if ( pxSpawnPoint && pxSpawnPoint->OwnsSpawnerGuid( uSpawnerGuid ) )
		{
			uSpawneeGuid = pxSpawnPoint->Spawn( uSpawnerGuid, pxSpawnParams );
			goto Spawned;
		}
	}

	for ( u_int u = 0; u < m_uNumGPVolumes; ++u )
	{
		AVP_ServerEntity_GamePlayVolume* pxGPV = AVP_ServerEntity_GamePlayVolume::FindEntity( m_pxGPVolumes[ u ].m_uSpawnerGuid );
		if ( pxGPV && pxGPV->OwnsSpawnerGuid( uSpawnerGuid ) )
		{
			uSpawneeGuid = pxGPV->Spawn( uSpawnerGuid, pxSpawnParams );
			goto Spawned;
		}
	}

Spawned:
	// Remember what we've spawned and update our stats.
	if ( uSpawneeGuid != ASURA_GUID_UNREGISTERED )
	{
		Asura_Assert( m_iSpawneeCount < m_iMaxInPlay, "AVP_ServerEntity_SpawnController::Spawn - too many spawnees in play" );

		m_fSpawnTimer = Asura_Random::GetFloat( m_fSpawnPeriodMin, m_fSpawnPeriodMax );
		++m_uTotalSpawnCount;
		ReallocateSpawneeArray( m_iSpawneeCount + 1 );
		m_pxSpawneeGuids[ m_iSpawneeCount++ ] = uSpawneeGuid;

		SendMessagesForEvent( AVP_ENTITY_EVENT_SPAWN_ON_SPAWN );
		if ( m_uTotalSpawnCount == static_cast< u_int >( m_iMaxSpawnCount ) )
		{
			SendMessagesForEvent( AVP_ENTITY_EVENT_SPAWN_ON_SPAWN_COUNT_EXHAUSTED );
		}
	}

	return uSpawneeGuid;
}

void AVP_ServerEntity_SpawnController::ActorOutOfPlay( const Asura_Guid uGuid, bool bIsDespawning )
{
	// Iterate through all of the spawn controllers and make sure that this actor is removed from any stats.
	for ( Asura_Hashed_List_It< AVP_ServerEntity_SpawnController > xSCIter( &s_xList ); !xSCIter.Done(); xSCIter.Next() )
	{
		if ( xSCIter.GetCurrent()->RemoveSpawneeGuidFromStats( uGuid, bIsDespawning ) )
		{
			break;
		}
	}
}

void AVP_ServerEntity_SpawnController::ScoreAndSpawn( const Asura_ScoreParams* pxScoreParams, const Asura_SpawnParams* pxSpawnParams )
{
	// Generate/override the score params.
	AVP_ScoreParams xAVPScoreParams;
	GenerateScoreParams( static_cast< const AVP_ScoreParams* >( pxScoreParams ), xAVPScoreParams );

	float	   fHighestScore = 0.0f;
	Asura_Guid uBestGuid     = ASURA_GUID_UNREGISTERED;
	ScoreSpawners( &xAVPScoreParams, fHighestScore, uBestGuid );
	if ( fHighestScore > 0.0f )
	{
		// Generate/override the spawn params.
		AVP_SpawnParams xAVPSpawnParams;
		GenerateSpawnParams( xAVPScoreParams, static_cast< const AVP_SpawnParams* >( pxSpawnParams ), xAVPSpawnParams );

		Spawn( uBestGuid, &xAVPSpawnParams );
	}
}

void AVP_ServerEntity_SpawnController::GenerateScoreParams( const AVP_ScoreParams* pxScoreParamsIn, AVP_ScoreParams& xScoreParamsOut )
{
	// Spawn scheme.
	xScoreParamsOut.m_iSpawnScheme   = m_iSpawnScheme;
	xScoreParamsOut.m_fSpawnDistance = m_fSpawnDistance;
	if ( m_iSpawnScheme == AVP_SS_AT_DISTANCE_X )
	{
		// Calculate the centroid of the players and fill it in.
		u_int uPlayerCount = 0;
        xScoreParamsOut.m_xPosition = Asura_Maths::ZeroVector3;
		for ( Asura_Hashed_List_ConstIt< AVP_ServerEntity_PlayerInfo > xPlayerIter( AVP_ServerEntity_PlayerInfo::GetList() );
			!xPlayerIter.Done();
			xPlayerIter.Next() )
		{
			const AVP_ServerEntity_PlayerInfo* pxPlayerInfo = xPlayerIter.GetCurrent();
			const AVP_ServerEntity_Actor*	   pxActor = AVP_ServerEntity_Actor::FindEntity( pxPlayerInfo->GetPuppetGuid() );
			if ( !pxActor || pxActor->IsDead() || !pxActor->IsEntityActive() )
			{
				continue;
			}

			++uPlayerCount;
			xScoreParamsOut.m_xPosition += pxActor->GetPosition();
		}

		if ( uPlayerCount > 1 )
		{
			xScoreParamsOut.m_xPosition /= static_cast< float >( uPlayerCount );
		}
	}

	// Spawn blueprint.
	if ( !pxScoreParamsIn || !pxScoreParamsIn->BlueprintSet() )
	{
		if ( m_uNumSpawneeTypes )
		{
			// Pick a spawnee type blueprint.
			float fTotalWeighting = 0.0f;
			for ( u_int u = 0; u < m_uNumSpawneeTypes; ++u )
			{
				fTotalWeighting += m_pxSpawneeTypes[ u ].m_fWeight;
			}

			const float fRandomWeight = Asura_Random::GetFloat( 0.0f, fTotalWeighting );
			float fCumulativeWeighting = 0.0f;
			for ( u_int u = 0; u < m_uNumSpawneeTypes; ++u )
			{
				fCumulativeWeighting += m_pxSpawneeTypes[ u ].m_fWeight;
				if ( fRandomWeight <= fCumulativeWeighting )
				{
					xScoreParamsOut.m_uBPTypeHashID = m_pxSpawneeTypes[ u ].m_uBPTypeHashID;
					xScoreParamsOut.m_uBPHashID		= m_pxSpawneeTypes[ u ].m_uBPHashID;
					break;
				}
			}
		}
	}
}

void AVP_ServerEntity_SpawnController::GenerateSpawnParams( const AVP_ScoreParams& xScoreParamsIn,
														    const AVP_SpawnParams* pxSpawnParamsIn,
															AVP_SpawnParams& xSpawnParamsOut )
{
	xSpawnParamsOut.m_uBPTypeHashID = xScoreParamsIn.m_uBPTypeHashID;
	xSpawnParamsOut.m_uBPHashID		= xScoreParamsIn.m_uBPHashID;

	if ( pxSpawnParamsIn && pxSpawnParamsIn->SquadGuidSet() )
	{
		xSpawnParamsOut.m_uSquadGuid = pxSpawnParamsIn->m_uSquadGuid;
	}
	else
	{
		xSpawnParamsOut.m_uSquadGuid = m_uSpawneeSquadGuid;
	}
}

bool AVP_ServerEntity_SpawnController::RemoveSpawneeGuidFromStats( const Asura_Guid uGuid, bool bIsDespawning )
{
	for ( int i = 0; i < m_iSpawneeCount; ++i )
	{
		if ( m_pxSpawneeGuids[ i ] == uGuid )
		{
			m_pxSpawneeGuids[ i ] = m_pxSpawneeGuids[ --m_iSpawneeCount ];
			// We could reallocate the spawnee array here to save memory, but we want to avoid lots of news and deletes.
			SendMessagesForEvent( AVP_ENTITY_EVENT_SPAWN_ON_ANY_SPAWNEE_DEATH );

			if( bIsDespawning )
			{
				SendMessagesForEvent( AVP_ENTITY_EVENT_DESPAWN_ON_SPAWN );
			}

			const u_int uTotalLeftToSpawn = static_cast< u_int >( m_iMaxSpawnCount ) - m_uTotalSpawnCount;
			if ( m_iSpawneeCount == 0 && uTotalLeftToSpawn == 0 )
			{
				SendMessagesForEvent( AVP_ENTITY_EVENT_SPAWN_ON_ALL_SPAWNEES_DEAD );
			}
			return true;
		}
	}

	return false;
}

void AVP_ServerEntity_SpawnController::ReallocateSpawneeArray( const int iSize )
{
	if ( iSize == 0 )
	{
		m_iSpawneeArraySize = 0;
		delete [] m_pxSpawneeGuids;
		m_pxSpawneeGuids = NULL;
		m_iSpawneeCount = 0;
	}
	else
	{
		enum { ARRAY_ROUNDUP_SIZE = 8 };

		// Round this up to the nearest ARRAY_ROUNDUP_SIZE.
		const int iRoundedUpSize = ( ( iSize / ARRAY_ROUNDUP_SIZE ) + 1 ) * ARRAY_ROUNDUP_SIZE;
		if ( iRoundedUpSize == m_iSpawneeArraySize )
		{
			return;
		}

		Asura_Guid* pxNewArray = new Asura_Guid[ iRoundedUpSize ];
		// Copy over the old spawnee guids if they exist.
		if ( m_pxSpawneeGuids )
		{
			for ( int i = 0; i < m_iSpawneeCount && i < iRoundedUpSize; ++i )
			{
				pxNewArray[ i ] = m_pxSpawneeGuids[ i ];
			}
		}

		m_iSpawneeArraySize = iRoundedUpSize;
		delete [] m_pxSpawneeGuids;
		m_pxSpawneeGuids = pxNewArray;
		m_iSpawneeCount = Asura_Maths::Min( m_iSpawneeCount, m_iSpawneeArraySize );
	}
}

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////
