//////////////////////////////////////////////////////////////////////////////
//
// AVP_SpawnerInterface.h
//
// Copyright (C) 2008 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __AVP_SPAWNERINTERFACE_H__
#define __AVP_SPAWNERINTERFACE_H__ 1

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// Asura
#include "Core/Asura_Hash.h"
#include "Core/Asura_Maths.h"
#include "Network/Asura_Guid.h"

//////////////////////////////////////////////////////////////////////////////
//
// AVP_SpawnerInterface
//
//////////////////////////////////////////////////////////////////////////////

class Asura_ScoreParams
{
};

class Asura_SpawnParams
{
};

class AVP_SpawnerInterface
{
public:
	virtual ~AVP_SpawnerInterface() {}

	virtual void 		ScoreSpawners( const Asura_ScoreParams* pxScoreParams, float& fHighestScore, Asura_Guid& uBestGuid ) const = 0;
	virtual bool 		OwnsSpawnerGuid( const Asura_Guid uSpawnerGuid ) const = 0;
	// Returns the guid of the spawned entity.
	virtual Asura_Guid 	Spawn( const Asura_Guid uSpawnerGuid, const Asura_SpawnParams* pxSpawnParams ) = 0;
};

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

enum AVP_Spawn_Scheme
{
	AVP_SS_RANDOM = 0,
	AVP_SS_AT_DISTANCE_X,
	AVP_SS_CLOSEST,

	AVP_SS_NUM_SPAWN_SCHEMES
};

class AVP_ScoreParams : public Asura_ScoreParams
{
public:
	int				m_iSpawnScheme;
	// If the scheme specifies a distance from a position these are included here.
	float			m_fSpawnDistance;
	Asura_Vector_3  m_xPosition;
	// Blueprint to spawn. Optional.
	Asura_Hash_ID	m_uBPTypeHashID;
	Asura_Hash_ID	m_uBPHashID;
	bool			m_bTypesMustMach;

	AVP_ScoreParams()
	: m_iSpawnScheme( 0 )
	, m_fSpawnDistance( 0.0f )
	, m_xPosition( Asura_Maths::ZeroVector3 )
	, m_uBPTypeHashID( ASURA_HASH_ID_UNSET )
	, m_uBPHashID( ASURA_HASH_ID_UNSET )
	, m_bTypesMustMach(false)
	{
	}

	bool BlueprintSet() const { return ( m_uBPTypeHashID != ASURA_HASH_ID_UNSET && m_uBPHashID != ASURA_HASH_ID_UNSET ); }
};

class AVP_SpawnParams : public Asura_SpawnParams
{
public:
	// Guid of the spawnee. Optional.
	Asura_Guid    m_uSpawneeGuid;
	// Blueprint to spawn. Optional.
	Asura_Hash_ID m_uBPTypeHashID;
	Asura_Hash_ID m_uBPHashID;
	// Squad to join. Optional.
	Asura_Guid	  m_uSquadGuid;
	// Whether the spawner should keep its stats up-to-date following the spawn.
	// This is turned off if the spawn point has been forced to spawn an entity by something else (a squad for example).
	bool		  m_bUpdateStats;

	AVP_SpawnParams()
	: m_uSpawneeGuid( ASURA_GUID_UNREGISTERED )
	, m_uBPTypeHashID( ASURA_HASH_ID_UNSET )
	, m_uBPHashID( ASURA_HASH_ID_UNSET )
	, m_uSquadGuid( ASURA_GUID_UNREGISTERED )
	, m_bUpdateStats( true )
	{
	}

	bool SpawneeGuidSet() const { return ( m_uSpawneeGuid != ASURA_GUID_UNREGISTERED ); }
	bool BlueprintSet()   const { return ( m_uBPTypeHashID != ASURA_HASH_ID_UNSET && m_uBPHashID != ASURA_HASH_ID_UNSET ); }
	bool SquadGuidSet()   const { return ( m_uSquadGuid != ASURA_GUID_UNREGISTERED ); }
};

struct AVP_Msg_Spawn
{
	u_int m_uVersion;
	// Blueprint to spawn. Optional.
	Asura_Hash_ID m_uBPTypeHashID;
	Asura_Hash_ID m_uBPHashID;
	// Squad to join. Optional.
	Asura_Guid	  m_uSquadGuid;

	bool BlueprintSet() const { return ( m_uBPTypeHashID != ASURA_HASH_ID_UNSET && m_uBPHashID != ASURA_HASH_ID_UNSET ); }
	bool SquadGuidSet() const { return ( m_uSquadGuid != ASURA_GUID_UNREGISTERED ); }

	static const u_int uCURRENT_VERSION = 0;
};

struct AVP_Msg_Spawn_SetMaxSpawnCount
{
	u_int m_uVersion;
	int	  m_iMaxSpawnCount;

	static const u_int uCURRENT_VERSION = 0;
};

struct AVP_Msg_Spawn_SetMaxInPlay
{
	u_int m_uVersion;
	int	  m_iMaxInPlay;

	static const u_int uCURRENT_VERSION = 0;
};

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

#endif //__AVP_SPAWNERINTERFACE_H__
