//--------------------------------------------------------------------------------------
//	File	:	AVP_ServerEntity_AlienSpawnPoint.cpp
//	Created	:	06 / 07 / 07
//	Author	:	Gaz Thomas
//	Copyright (C) 2006 Rebellion, All Rights Reserved.
//--------------------------------------------------------------------------------------

//--------------------------------------------------------------------------------------
//	INCLUDES
//--------------------------------------------------------------------------------------

// Always include Core first
#include "Core/Asura.h"

// This Files Header
#include "Entity/Spawners/Alien/AVP_ServerEntity_AlienSpawnPoint.h"

// Asura include files
#include "Blueprint/Asura_Blueprint_System.h"
#include "ChunkSystem/Asura_Chunk_Stream.h"

// Axon Include Files

// AVP Include files
#include "Entity/AVP_Entity_Classification.h"
#include "Entity/Controller/AVP_ServerEntity_Team_Alien.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Alien/AVP_ServerActor_Alien.h"

// AVP Axon Include Files
#include "Core/BrainTypes/AVP_Brain_Actor.h"
#include "Core/BrainTypes/AVP_Brain_Actor_Alien.h"
#include "Core/BrainTypes/AVP_Brain_Team_Alien.h"

#ifdef ASURA_TOOLS
#include "EntityTemplate/Tools_EntityTemplate_System.h"
#endif

//--------------------------------------------------------------------------------------
//	Statics
//--------------------------------------------------------------------------------------
Asura_Hashed_List<AVP_ServerEntity_AlienSpawnPoint> AVP_ServerEntity_AlienSpawnPoint::s_xList;

const float AVP_ServerEntity_AlienSpawnPoint::fALIEN_SPAWNPOINT_DEFAULT_RADIUS = 1.5f;

//--------------------------------------------------------------------------------------
//	Function	:	AVP_ServerEntity_AlienSpawnPoint::AVP_ServerEntity_AlienSpawnPoint
//	Created		:	26 / 10 / 07
//	Author		:	Gareth Thomas
//--------------------------------------------------------------------------------------
AVP_ServerEntity_AlienSpawnPoint::AVP_ServerEntity_AlienSpawnPoint(const Asura_Guid uGuid)
: PARENT( uGuid )
, m_iAgilityValue( 0 )
, m_fRadius( fALIEN_SPAWNPOINT_DEFAULT_RADIUS )
, m_uNumSpawnPointLinks( 0 )
, m_puSpawnPointLinks( NULL )
, m_uGroupID( 0 )
, m_fIdleTimeLeft( 0.0f )
, m_usWaypointID( ASURA_INVALID_WAYPOINT_ID )
, m_iRealAgilityValue( 0 )
{
	Classification = AVP_ENTITYCLASS_ALIENSPAWNPOINT;

	SetNeverUpdateClient();		// Doesn't have a client

	s_xList.AddToList( this );
}
/// End of function 'AVP_ServerEntity_AlienSpawnPoint::AVP_ServerEntity_AlienSpawnPoint'. ///


//--------------------------------------------------------------------------------------
//	Function	:	AVP_ServerEntity_AlienSpawnPoint::Spawn
//	Created		:	26 / 10 / 07
//	Author		:	Gareth Thomas
//--------------------------------------------------------------------------------------
void AVP_ServerEntity_AlienSpawnPoint::Spawn( Asura_Hash_ID uBlueprintTypeHashID, Asura_Hash_ID uBlueprintHashID )
{
	if ((uBlueprintTypeHashID == ASURA_HASH_ID_UNSET) || (uBlueprintHashID == ASURA_HASH_ID_UNSET))
	{
		Asura_Assert(false, "No blueprint ID given for spawned alien");
		return;
	}

	// Get blueprint
	Asura_Blueprint* pxBlueprint = Asura_Blueprint_System::GetBlueprintByHashID( uBlueprintTypeHashID, uBlueprintHashID );
	if (!pxBlueprint)
	{
		Asura_Assert(false, "Couldn't find blueprint for spawned alien");
		return;
	}

	// Create a guid for the new entity and then create the alien
	AVP_ServerActor_Alien* pxSpawnedAlien = AVP_ServerActor_Alien::CreateAndReturn( Asura_Entity_System::GetCreatedByGameGuid() );

	// Validate the alien
	if (pxSpawnedAlien)
	{
		// Transform it into AI
		pxSpawnedAlien->SetEntityActive( true );
		pxSpawnedAlien->MakeAI();

		// Set the alien's 'is-a-spawned-alien' flag
		pxSpawnedAlien->SetIsASpawnedAlien( true );

		// Reset the physics object type
		if (pxSpawnedAlien->GetPhysicsObjectHandler())
		{
			pxSpawnedAlien->GetPhysicsObjectHandler()->Reset( AVP_Actor_PhysicsObjectHandler::FREE_AI, *pxSpawnedAlien->GetCollisionObject(), pxSpawnedAlien->GetCollisionProxy() );
		}

		// Initalise from the blueprint
		pxSpawnedAlien->SetBlueprintTypeHashID( uBlueprintTypeHashID );
		pxSpawnedAlien->SetBlueprintHashID( uBlueprintHashID );
		pxSpawnedAlien->InitFromBlueprint();

		// If this spawn point has an 'out' gamescene guid, set it as the alien's initial gamescene guid
		if (m_uGameSceneOutGuid != ASURA_GUID_UNREGISTERED)
		{
			pxSpawnedAlien->SetInitialGamesceneGuid( m_uGameSceneOutGuid );
		}

		// AJG - 20/12/07 - There's no point in trying to inform clients - we've only just created the server entity, so there's no way the client can exist yet, all we end up doing is firing an assert.
		pxSpawnedAlien->TeleportTo( GetPosition(), GetOrientation(), false );

		// Call post load initialise - this is probably the least useful comment I've ever written (AN 10/01/08)
		pxSpawnedAlien->PostLoadInitialise();

		// Set idle timer
		static const float fMAX_DELAY_FROM_SAME_DEFAULT = 5.0f;
		m_fIdleTimeLeft = fMAX_DELAY_FROM_SAME_DEFAULT;//AVP_ServerEntity_Team_Alien::GetInstance()->GetAlienTeamBrain() ?
		//	AVP_ServerEntity_Team_Alien::GetInstance()->GetAlienTeamBrain()->GetSpawnDelayFromSame() : fMAX_DELAY_FROM_SAME_DEFAULT;

		return;
	}

	Asura_Assert(false, "Couldn't create spawned alien");
}
/// End of function 'AVP_ServerEntity_AlienSpawnPoint::Spawn'. ///


//--------------------------------------------------------------------------------------
//	Function	:	AVP_ServerEntity_AlienSpawnPoint::~AVP_ServerEntity_AlienSpawnPoint
//	Created		:	26 / 10 / 07
//	Author		:	Gareth Thomas
//--------------------------------------------------------------------------------------
AVP_ServerEntity_AlienSpawnPoint::~AVP_ServerEntity_AlienSpawnPoint()
{
	delete [] m_puSpawnPointLinks;
	m_puSpawnPointLinks = NULL;

	s_xList.RemoveFromList(this);
}
/// End of function 'AVP_ServerEntity_AlienSpawnPoint::~AVP_ServerEntity_AlienSpawnPoint'. ///


//--------------------------------------------------------------------------------------
//	Function	:	AVP_ServerEntity_AlienSpawnPoint::Create
//	Created		:	26 / 10 / 07
//	Author		:	Gareth Thomas
//--------------------------------------------------------------------------------------
AVP_ServerEntity_AlienSpawnPoint* AVP_ServerEntity_AlienSpawnPoint::Create(const Asura_Guid uGuid)
{
	return(new AVP_ServerEntity_AlienSpawnPoint(uGuid));
}
/// End of function 'AVP_ServerEntity_AlienSpawnPoint::Create'. ///


//--------------------------------------------------------------------------------------
//	Function	:	AVP_ServerEntity_AlienSpawnPoint::WriteToChunkStream
//	Created		:	26 / 10 / 07
//	Author		:	Gareth Thomas
//--------------------------------------------------------------------------------------
void AVP_ServerEntity_AlienSpawnPoint::WriteToChunkStream(Asura_Chunk_Stream& xStream) const
{
	//Version
	xStream << CURRENT_CHUNK_VERSION;

	//Parent
	PARENT::WriteToChunkStream( xStream );

	// Our data.
	xStream << m_iAgilityValue;
	xStream << m_fRadius;
	xStream << m_uNumSpawnPointLinks;
	for ( u_int uLink = 0; uLink < m_uNumSpawnPointLinks; ++uLink )
	{
		xStream << m_puSpawnPointLinks[ uLink ];
	}

	xStream << m_uGroupID;

	xStream << m_fIdleTimeLeft;
}
/// End of function 'AVP_ServerEntity_AlienSpawnPoint::WriteToChunkStream'. ///

//--------------------------------------------------------------------------------------
//	Function	:	AVP_ServerEntity_AlienSpawnPoint::ReadFromChunkStream
//	Created		:	26 / 10 / 07
//	Author		:	Gareth Thomas
//--------------------------------------------------------------------------------------
void AVP_ServerEntity_AlienSpawnPoint::ReadFromChunkStream(Asura_Chunk_Stream& xStream)
{
	//Version
	u_int uVersion;
	xStream	>> uVersion;

	//Parent
	PARENT::ReadFromChunkStream( xStream );

	// Our data.
	if( uVersion >= 1 )
	{
		if( uVersion < 5 ) // Stored in parent from version 5 upwards
		{
			xStream >> m_uGameSceneInGuid;
		}

		xStream >> m_iAgilityValue;
	}

	if( uVersion >= 2 )
	{
		xStream >> m_fRadius;
	}

	if( uVersion >= 3 )
	{
		xStream >> m_uNumSpawnPointLinks;
		if ( m_uNumSpawnPointLinks > 0 )
		{
			m_puSpawnPointLinks = new Asura_Guid[ m_uNumSpawnPointLinks ];
			for ( u_int uLink = 0; uLink < m_uNumSpawnPointLinks; ++uLink )
			{
				xStream >> m_puSpawnPointLinks[ uLink ];
			}
		}
	}

	if( uVersion >= 4 )
	{
		xStream >> m_uGroupID;
		xStream >> m_fIdleTimeLeft;
	}

	if( uVersion > CURRENT_CHUNK_VERSION )
	{
		Asura_Assertf( false, ( "AVP_ServerEntity_AlienSpawnPoint::ReadFromChunkStream - unhandled chunk version (%u)", uVersion ) );
	}
}
/// End of function 'AVP_ServerEntity_AlienSpawnPoint::ReadFromChunkStream'. ///


//--------------------------------------------------------------------------------------
void AVP_ServerEntity_AlienSpawnPoint::Update()
{
	PARENT::Update();

	if( m_fIdleTimeLeft >= 0.0f )
	{
		m_fIdleTimeLeft -= Asura_Timers::GetFrameTime();
	}
}


//--------------------------------------------------------------------------------------
bool AVP_ServerEntity_AlienSpawnPoint::IsInUse() const
{
	// Can unspawn but can not spawn if spawn point is in use (== there is an alien currently inside the radius)
	Asura_Hashed_List_It<AVP_ServerActor_Alien> xpIter( AVP_ServerActor_Alien::GetList() );

	while( !xpIter.Done() )
	{
		AVP_ServerActor_Alien* pxServerAlien = xpIter.GetCurrent();
		xpIter.Next();

		if( pxServerAlien && ( !pxServerAlien->IsDead() ) && pxServerAlien->IsEntityActive() && IsInsideRadius( pxServerAlien->GetPosition() ) )
		{
			return true;
		}
	}

	return false;
}


//--------------------------------------------------------------------------------------
bool AVP_ServerEntity_AlienSpawnPoint::IsNearExitPoint( const Asura_Vector_3& xPosition ) const
{
	static const float fNEAR_DIST_SQ = ( m_uGameSceneInGuid != ASURA_GUID_UNREGISTERED ) ? 1.0f : ( GetRadius() * GetRadius() ); //
	return ( ( xPosition - GetExitPoint() ).MagnitudeSquared() <= fNEAR_DIST_SQ );
}

//--------------------------------------------------------------------------------------
const Asura_Vector_3& AVP_ServerEntity_AlienSpawnPoint::GetExitPoint() const
{
	if( m_uGameSceneInGuid != ASURA_GUID_UNREGISTERED )
	{
		//Asura_ServerNode_GamesceneNode* pxNode = Asura_ServerNode_GamesceneNode::FindEntity( m_uGameSceneInGuid );

		//if( pxNode )
		//{
		//	return pxNode->GetPosition();
		//}

		Asura_Assert( 0, "Invalid gamescene in node for alien spawn point" );
	}

	return GetPosition();
}


//--------------------------------------------------------------------------------------
void AVP_ServerEntity_AlienSpawnPoint::FindNearestWaypoint()
{
	AVP_Navigation::TestParams xParams;
    xParams.m_uPathfinderFlags     = AVP_Navigation::AVP_AI_PATHFINDER_FLAG_ALIEN_HIGH_MOBILITY | ASURA_AI_PATHFINDER_FLAGS_IGNOREZONES;
    xParams.m_usZone               = ASURA_NAVVERTEX_MAX_ZONE_ID;

	AVP_Navigation::GetNearestWaypointID( GetPosition(), m_usWaypointID, xParams );
}


//--------------------------------------------------------------------------------------
void AVP_ServerEntity_AlienSpawnPoint::CalculateRealAgility()
{
	// Set real agility value
	m_iRealAgilityValue = AVP_Navigation::AR_HIGH;

	// Find out real agility value
	const Asura_AI_NavVertex* pxWaypoint = Asura_AI_Navigation_System::GetWaypoint( m_usWaypointID );

	if( pxWaypoint )
	{
		for( u_char uIdx = 0; uIdx < pxWaypoint->m_ucNumConnections; uIdx++ )
		{
			Asura_AI_NavConnection* pxConnection = pxWaypoint->m_pxConnections + uIdx;
			if( pxConnection )
			{
				int iLinkAgility = AVP_Navigation::AR_HIGH;

				if( ( pxConnection->m_usFlags & AVP_Navigation::AVP_AI_PATHFINDER_FLAG_ALIEN_LOW_MOBILITY ) > 0 )
				{
					iLinkAgility = AVP_Navigation::AR_LOW;
				}
				else
				if( ( pxConnection->m_usFlags & AVP_Navigation::AVP_AI_CONNECTION_FLAG_TRANSITION_MED_MOBILITY ) > 0 )
				{
					iLinkAgility = AVP_Navigation::AR_MED;
				}

				// Update spawn point agility value
				m_iRealAgilityValue = Asura_Maths::Min( iLinkAgility, m_iRealAgilityValue );
			}
		}
	}
}

#ifdef ASURA_TOOLS
bool AVP_ServerEntity_AlienSpawnPoint::CanBeIncludedInTemplate() const
{
	return ( Classification == AVP_ENTITYCLASS_ALIENSPAWNPOINT );
}

void AVP_ServerEntity_AlienSpawnPoint::ApplyChangesForTemplateInstance( Tools_EntityTemplateInstance* pxInstance )
{
	PARENT::ApplyChangesForTemplateInstance( pxInstance );

	// Not sure we need to convert the links or group ID.
	delete [] m_puSpawnPointLinks;
	m_puSpawnPointLinks = NULL;
	m_uNumSpawnPointLinks = 0;
}
#endif
