/**
*
* \file AVP_ServerEntity_AdvancedVolumeTrigger.cpp
*
* Created by Anthony Gregson - 08/09/05 (from FPSDemo_ServerEntity_AdvancedVolumeTrigger by Andrew Weinkove - 14/04/2004)
*
* Copyright (C) 2005 Rebellion, All Rights Reserved.
*
* Project-implementation of advanced volume triggers
*
**/

#include "Core/Asura.h"

#include "Entity/Triggers/AVP_ServerEntity_AdvancedVolumeTrigger.h"

#include "Entity/PhysicalObject/Asura_ServerEntity_PhysicalObject.h"

#include "Entity/AVP_Messages.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Alien/AVP_ServerActor_Alien.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Human/AVP_ServerActor_Human.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Predator/AVP_ServerActor_Predator.h"
#include "Entity/PlayerInfo/AVP_ServerEntity_PlayerInfo.h"

#ifdef ASURA_TOOLS
const char* AVP_ServerEntity_AdvancedVolumeTrigger::s_aszAVPVolumeTypes[NUM_AVP_DEPRECATEDVOLUMETRIGGER_TYPES] = 
{
    "Specific Object",
	"Player Alien",
	"Player Human",
	"Player Predator",
	"AI Alien",
	"AI Human",
	"AI Predator",
};
#endif //ASURA_TOOLS


Asura_Hashed_List<AVP_ServerEntity_AdvancedVolumeTrigger> AVP_ServerEntity_AdvancedVolumeTrigger::s_xList(5);

AVP_ServerEntity_AdvancedVolumeTrigger::AVP_ServerEntity_AdvancedVolumeTrigger(Asura_Bounding_Box& xBox, bool bInitiallyActive)
: Asura_ServerEntity_AdvancedVolumeTrigger(xBox, bInitiallyActive)
{
	s_xList.AddToList(this);
}

//constructor from load
AVP_ServerEntity_AdvancedVolumeTrigger::AVP_ServerEntity_AdvancedVolumeTrigger(Asura_Guid uGuid,const char*& pcChunkBuffer)
: Asura_ServerEntity_AdvancedVolumeTrigger(uGuid, pcChunkBuffer)
{
	int iVersion;
	ASURA_CHUNK_EXTRACT(pcChunkBuffer, iVersion, int);
	Asura_Assert(iVersion == 0, "Incorrect version");

	s_xList.AddToList(this);
}


AVP_ServerEntity_AdvancedVolumeTrigger::~AVP_ServerEntity_AdvancedVolumeTrigger()
{
	s_xList.RemoveFromList(this);
}

bool AVP_ServerEntity_AdvancedVolumeTrigger::IsVolumeOccupied()
{
	for (int i=0;i<NUM_AVP_DEPRECATEDVOLUMETRIGGER_TYPES;i++)
	{
		if (m_iListFlags & (1 << i))
		{
			switch (i)
			{
                case AVP_DVTT_SPECIFICGUID:
                {
                    const Asura_ServerEntity_PhysicalObject* pxPhysicalObject = Asura_ServerEntity_PhysicalObject::FindEntity(m_uSpecificGuid);
                    if(pxPhysicalObject)
                    {
                        return Contains(pxPhysicalObject->GetPosition());
                    }
                    else
                    {
						// This might be a player guid. Find out.
						const AVP_ServerEntity_PlayerInfo* pxPlayer = AVP_ServerEntity_PlayerInfo::FindEntity(m_uSpecificGuid);
						if (pxPlayer)
						{
							pxPhysicalObject = Asura_ServerEntity_PhysicalObject::FindEntity(pxPlayer->GetPuppetGuid());
							if(pxPhysicalObject)
							{
								return Contains(pxPhysicalObject->GetPosition());
							}
						}
						Asura_Assert(0, "AVP_ServerEntity_AdvancedVolumeTrigger::IsVolumeOccupied - could not find specific guid object");
                    }

                    break;
                }

				case AVP_DVTT_AI_ALIEN:
				{
					for(Asura_Hashed_List_ConstIt<AVP_ServerActor_Alien> xIt(AVP_ServerActor_Alien::GetList()); !xIt.Done(); xIt.Next())
					{
						const AVP_ServerActor_Alien*	pxAlien		= xIt.GetCurrent();
						const bool						bIsPlayer	= pxAlien->IsPlayer();

						if(!bIsPlayer && Contains(pxAlien->GetPosition()))
						{
							return true;
						}
					}
					break;
				}

				case AVP_DVTT_PLAYER_ALIEN:
				{
					for(Asura_Hashed_List_ConstIt<AVP_ServerActor_Alien> xIt(AVP_ServerActor_Alien::GetList()); !xIt.Done(); xIt.Next())
					{
						const AVP_ServerActor_Alien*	pxAlien		= xIt.GetCurrent();
						const bool						bIsPlayer	= pxAlien->IsPlayer();

						if(bIsPlayer && Contains(pxAlien->GetPosition()))
						{
							return true;
						}
					}
					break;
				}

				case AVP_DVTT_AI_HUMAN:
				{
					for(Asura_Hashed_List_ConstIt<AVP_ServerActor_Human> xIt(AVP_ServerActor_Human::GetList()); !xIt.Done(); xIt.Next())
					{
						const AVP_ServerActor_Human*	pxHuman		= xIt.GetCurrent();
						const bool						bIsPlayer	= pxHuman->IsPlayer();

						if(!bIsPlayer && Contains(pxHuman->GetPosition()))
						{
							return true;
						}
					}
					break;
				}

				case AVP_DVTT_PLAYER_HUMAN:
				{
					for(Asura_Hashed_List_ConstIt<AVP_ServerActor_Human> xIt(AVP_ServerActor_Human::GetList()); !xIt.Done(); xIt.Next())
					{
						const AVP_ServerActor_Human*	pxHuman		= xIt.GetCurrent();
						const bool						bIsPlayer	= pxHuman->IsPlayer();

						if(bIsPlayer && Contains(pxHuman->GetPosition()))
						{
							return true;
						}
					}
					break;
				}

				case AVP_DVTT_AI_PREDATOR:
				{
					for(Asura_Hashed_List_ConstIt<AVP_ServerActor_Predator> xIt(AVP_ServerActor_Predator::GetList()); !xIt.Done(); xIt.Next())
					{
						const AVP_ServerActor_Predator*	pxPredator	= xIt.GetCurrent();
						const bool						bIsPlayer	= pxPredator->IsPlayer();

						if(!bIsPlayer && Contains(pxPredator->GetPosition()))
						{
							return true;
						}
					}
					break;
				}

				case AVP_DVTT_PLAYER_PREDATOR:
				{
					for(Asura_Hashed_List_ConstIt<AVP_ServerActor_Predator> xIt(AVP_ServerActor_Predator::GetList()); !xIt.Done(); xIt.Next())
					{
						const AVP_ServerActor_Predator*	pxPredator	= xIt.GetCurrent();
						const bool						bIsPlayer	= pxPredator->IsPlayer();

						if(bIsPlayer && Contains(pxPredator->GetPosition()))
						{
							return true;
						}
					}
					break;
				}

				default:
				{
					Asura_Assert(false, "Unknown list type for advanced volume trigger");
					break;
				}
			}		// End of switch.
		}
	}
	return false;
}

void AVP_ServerEntity_AdvancedVolumeTrigger::FillInChunkBuffer(char*& pcChunkBuffer)
{
	Asura_ServerEntity_AdvancedVolumeTrigger::FillInChunkBuffer(pcChunkBuffer);
	int iVersion = 0;
	ASURA_CHUNK_FILLANDVERIFY(pcChunkBuffer, iVersion, int);
}

int AVP_ServerEntity_AdvancedVolumeTrigger::GetChunkBufferSize()
{
	return sizeof(int) + Asura_ServerEntity_AdvancedVolumeTrigger::GetChunkBufferSize();
}

void AVP_ServerEntity_AdvancedVolumeTrigger::CreateFromChunkBuffer(Asura_Guid uGuid, const char*& pcChunkBuffer)
{
	new AVP_ServerEntity_AdvancedVolumeTrigger(uGuid, pcChunkBuffer);
}
