/**
*
* \file AVP_ServerEntity_MultiDamageVolume.h
*
* Created by Andrew Weinkove - 14/04/2004
*
* Copyright (C) 2004 Rebellion, All Rights Reserved.
*
* Project-implementation of Multi-Damage volumes
*
**/

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif

#ifndef __AVP_SERVERENTITY_MULTIDAMAGEVOLUME_H__
#define __AVP_SERVERENTITY_MULTIDAMAGEVOLUME_H__

/************/
/* Includes.*/
/************/
class AVP_ServerEntity_Actor;

enum AVP_MULTIDAMAGEVOLUME_TYPES
{
	// Preserve Order of this enum!
	// only add entries to the end as new entires in the middle will knock on all of the settings set for already saved damage volumes. 
	AVP_MDVT_PREDATOR = 0,
	AVP_MDVT_ALIEN,
	AVP_MDVT_HUMAN,
	AVP_MDVT_ALIENQUEEN,

	NUM_AVP_AIDAMAGABLE_TYPES,						

	AVP_MDVT_PLAYER = NUM_AVP_AIDAMAGABLE_TYPES,

	NUM_AVP_MULTIDAMAGEVOLUME_TYPES,
};

#include "Entity/Triggers/Asura_ServerEntity_MultiDamageVolume.h"

class AVP_ServerEntity_MultiDamageVolume : public Asura_ServerEntity_MultiDamageVolume
{
public:
    AVP_ServerEntity_MultiDamageVolume(Asura_Guid uGuid);
	virtual ~AVP_ServerEntity_MultiDamageVolume();

	static AVP_ServerEntity_MultiDamageVolume* Create( const Asura_Guid uGUID )	{ return(new AVP_ServerEntity_MultiDamageVolume (uGUID)); }

	static const Asura_Hashed_List<AVP_ServerEntity_MultiDamageVolume>* GetList() { return &s_xList; }
	static AVP_ServerEntity_MultiDamageVolume* FindEntity(Asura_Guid uGuid) { return s_xList.FindEntity(uGuid); }

	virtual void ApplyDamage(); 
	void ApplyDamageToActor(const AVP_ServerEntity_Actor* pxActor);

	virtual	bool	IsFilledFromChunkBuffer	(void) const	{ return(false); }

	Asura_VolumeGroup* GetVolumeGroup(void) const {return m_pxVolumeGroup;}

#ifdef ASURA_TOOLS
	virtual void ApplyChangesForTemplateInstance(class Tools_EntityTemplateInstance* pxInstance);
	virtual bool CanBeIncludedInTemplate() const;
#endif
	
	virtual	void ReadFromChunkStream( Asura_Chunk_Stream& xStream );
	virtual	void WriteToChunkStream( Asura_Chunk_Stream& xStream ) const;

protected:
	float m_fTimeToNextCheck;
	static float s_fTimeBetweenChecks;
#ifdef ASURA_TOOLS
	static const char* s_aszAVPVolumeTypes[NUM_AVP_MULTIDAMAGEVOLUME_TYPES];
#endif
	friend class ToolsAVP_Entity_MultiDamageVolume;
	typedef Asura_ServerEntity_MultiDamageVolume PARENT_ASURA;

private:

	static Asura_Hashed_List<AVP_ServerEntity_MultiDamageVolume> s_xList;

	
	
};


#endif//__AVP_SERVERENTITY_MULTIDAMAGEVOLUME_H__
