//////////////////////////////////////////////////////////////////////////////
//
// AVP_ClientEntity_Vent.h
//
// Copyright (c) 2008 Rebellion. All rights reserved.
//
// Description:
//
//     Vent to allow entry to/exit from ducts
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef ___AVP_CLIENTENTITY_VENT_H___
#define ___AVP_CLIENTENTITY_VENT_H___ 1

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Entity/Asura_Entity_Renderable.h"

#include "Attractors/Asura_Attractor_Data.h"

class Asura_Attractor_Compiled;
class Asura_Shape_Instance;
class Asura_ClientEntity_PhysicalObject;
class AVP_Message_Vent_ClientUpdate;
class AVP_Message_Vent_ClientCreate;

// C L A S S E S /////////////////////////////////////////////////////////////

class AVP_ClientEntity_Vent : public Asura_Entity_Renderable
{
public:
	// Players will interact with the client side vent entities through this function, AI actors with the server side
	bool GetBestExitPoint(const Asura_ClientEntity_PhysicalObject& xClient, Asura_AVector3& xExitBeginPos, Asura_AMatrix3x3& xExitBeginOri, Asura_AVector3& xExitTargetPos, Asura_AMatrix3x3& xExitTargetOri) const;
	bool GetBestEntryPoint(const Asura_ClientEntity_PhysicalObject& xClient, Asura_AVector3& xEntryBeginPos, Asura_AMatrix3x3& xEntryBeginOri, Asura_AVector3& xEntryTargetPos, Asura_AMatrix3x3& xEntryTargetOri) const;

	virtual void Render();

	void Damage(); // this will get called from the move so we don't need to worry about networking, the movement system will do it for us

	virtual bool LOSTest( const class Asura_LOS_InputData* pxInput, class Asura_LOS_OutputData* pxOutput );
	virtual void GetLineOfSightAABB( Asura_Bounding_Box& xAABB );

	bool GetUseExitPointsOnly() const { return m_bUseExitPointsOnly; }
	void SetUseExitPointsOnly(bool bExitsOnly) { m_bUseExitPointsOnly = bExitsOnly; }

	virtual void PostCreateInitialise();

	static void						Create			(const Asura_Message* pxMessage);
	static AVP_ClientEntity_Vent*	FindEntity		(Asura_Guid uGuid)			{ return s_xList.FindEntity(uGuid); }
	static const Asura_Hashed_List<AVP_ClientEntity_Vent>* GetList(void)		{ return &s_xList; }

protected:
	AVP_ClientEntity_Vent();
	virtual ~AVP_ClientEntity_Vent();

	void ProcessClientCreateMessage(const AVP_Message_Vent_ClientCreate* pxData, float fServerTime);

private:
	typedef Asura_Entity_Renderable PARENT;

	// Datatype and functions used in the search for entry points.
	struct EntryPointCallbackData
	{
		const AVP_ClientEntity_Vent& m_xVent;
		const Asura_Vector_3& m_xEntityPosition;
		const Asura_Matrix_3x3& m_xEntityOrientation;
		const Asura_Matrix_3x3& m_xEntryBeginOrientation;
		u_int m_uBestAttractorID;
		Asura_Vector_3 m_xBestPosOnAttractor;
		Asura_Matrix_3x3 m_xBestOrientationOnAttractor;

		EntryPointCallbackData(const AVP_ClientEntity_Vent& xVent, const Asura_Vector_3& xEntityPosition, const Asura_Matrix_3x3& xEntityOrientation, const Asura_Matrix_3x3& xEntryBeginOrientation)
			: m_xVent(xVent)
			, m_xEntityPosition(xEntityPosition)
			, m_xEntityOrientation(xEntityOrientation)
			, m_xEntryBeginOrientation(xEntryBeginOrientation)
			, m_uBestAttractorID(uASURA_ATTRACTOR_ID_NONE)
			, m_xBestPosOnAttractor(Asura_Maths::ZeroVector3)
			, m_xBestOrientationOnAttractor(Asura_Maths::IdentityMatrix3x3)
		{
		};
	};

	void TestDuctAttractor(EntryPointCallbackData& xData, const Asura_Attractor_Compiled& xAttractor) const;
	static void TestDuctAttractors_Callback(void* pData, const Asura_Attractor_Compiled* const pxAttractor);

	static void TestForNearbyEnvHooks_Callback(void* pData, const Asura_Attractor_Compiled* const pxAttractor);
	
	Asura_Hash_ID m_uUndamagedSkinHash;
	Asura_Hash_ID m_uDamagedSkinHash;

	struct ExitPoint
	{
		Asura_AVector3 m_xPosition;
		Asura_AVector3 m_xNormal;
	};

	u_int m_uNumExitPoints;
	ExitPoint* m_pxExitPoints;

	Asura_Shape_Instance*	m_pxShapeInstance;

	bool m_bIsDamaged;
	bool m_bUseExitPointsOnly;

	AVP_ClientEntity_Vent( const AVP_ClientEntity_Vent& xOther );
	const AVP_ClientEntity_Vent& operator=( const AVP_ClientEntity_Vent& xOther );

	static Asura_Hashed_List<AVP_ClientEntity_Vent> s_xList;
};

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

#endif // #ifndef ___AVP_CLIENTENTITY_VENT_H___
