////////////////////////////////////////////////////////////////////////////// 
//
// AVP_DevelopmentFrontEnd.cpp
//
// Development front-end. Supports quick starting of any level or game mode 
// via a text-only interface by-passing the full front-end.
//
//////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

#if defined AVP_USE_DEVELOPMENT_FRONT_END

#include "FrontEnd/AVP_DevelopmentFrontEnd.h"

// Asura headers
#include "Core/Asura_Multithread.h"
#include "Core/Asura_String.h"
#include "Core/Asura_Timers.h"
#include "Core/Asura_UserInput.h"
#ifdef ASURA_SUPPORT_COMMAND_CONSOLE
#include "Interface/Asura_CommandConsole.h"
#endif
#include "Input/Asura_ControlManager.h"
#include "Input/Asura_ControlMap.h"
#include "Network/Asura_NetLinkManager.h"
#include "Network/Asura_NetTransport_System.h"
#include "Network/Asura_Network_Session.h"
#include "Network/Asura_NetSocket.h"
#include "Rendering/Asura_Font_System.h"
#include "Rendering/Asura_Module.h"
#include "Rendering/Asura_Render.h"
#include "Rendering/Asura_Viewport.h"
#include "Sound/Asura_Audio_System.h"
#include "Sound/Asura_Music.h"
#include "Sound/Asura_Sound.h"
#include "Network/Asura_Network_Session.h"
#include "Network/Online/Asura_Lobby.h"

// Project headers
#include "Core/AVP_Core.h"
#include "Core/AVP_Game_Core.h"
#ifdef AVP_LOG_FILE_LOAD_MEMORY_USE
#include "Core/AVP_SmokeTest.h"
#endif //#ifdef AVP_LOG_FILE_LOAD_MEMORY_USE
#include "Core/AVP_Settings.h"
#include "GUIMenu/AVP_GUIMenu_System.h"
#if defined(AVP_ALLOW_MULTIPLAYER)
#include "Multiplayer/AVP_Multiplayer_GameRules.h"
#endif	//defined(AVP_ALLOW_MULTIPLAYER)


// C O N S T A N T S /////////////////////////////////////////////////////////
#ifdef ASURA_SUPPORT_EXTERNAL_NETWORK
const char* g_aszNetlinkStatus[ASURA_NETLINK_NUM_STATUS_ENTRIES] = 
{
	"disconnected", //ASURA_NETLINK_DISCONNECTED
	"connecting", //ASURA_NETLINK_CONNECTING
	"connected", //ASURA_NETLINK_CONNECTED
	"connection failed", //ASURA_NETLINK_CONNECTION_FAILED
	"disconnecting", //ASURA_NETLINK_DISCONNECTING
	"connection dropping", //ASURA_NETLINK_CONNECTION_DROPPING
	"connection dropped", //ASURA_NETLINK_CONNECTION_DROPPED
#ifdef ASURA_SUPPORT_GAMESPY
	"checking GameSpy availability", //ASURA_NETLINK_CHECKING_GAMESPY
	"Error: GameSpy backend unavailable", //ASURA_NETLINK_FAILED_GAMESPY_UNAVAILABLE
	"Error: GameSpy backend temporarily unavailable", //ASURA_NETLINK_FAILED_GAMESPY_TEMPORARILY_UNAVAILABLE
#endif
};
#endif // #ifdef ASURA_SUPPORT_EXTERNAL_NETWORK


// S T A T I C   C L A S S   D A T A /////////////////////////////////////////
bool						AVP_DevelopmentFrontEnd::s_bSetupComplete = false;
float						AVP_DevelopmentFrontEnd::s_fIgnoreInputTimer = 0.0f;
AVP_DFE_MENUMODE			AVP_DevelopmentFrontEnd::s_eMenuMode = AVP_DFE_MENUMODE_GAMESETUP;
#if defined(AVP_MULTIPLAYER_ONLY_DEVFRONTEND)
AVP_DFE_OPTIONS				AVP_DevelopmentFrontEnd::s_eCurrentlyHighlitOption = AVP_DFE_OPTION_ACCEPT;
#if defined AVP_USE_GAME_FRONTEND 
int							AVP_DevelopmentFrontEnd::s_iCurrentGameMode = AVP_DFE_GAMEMODE_GAME_FRONTEND;
#else
int							AVP_DevelopmentFrontEnd::s_iCurrentGameMode = AVP_DFE_GAMEMODE_HOST_NETWORK_GAME;
#endif	// defined AVP_USE_GAME_FRONTEND
#else
AVP_DFE_OPTIONS				AVP_DevelopmentFrontEnd::s_eCurrentlyHighlitOption = AVP_DFE_OPTION_LEVEL_SELECTION;
int							AVP_DevelopmentFrontEnd::s_iCurrentGameMode = AVP_DFE_GAMEMODE_LOAD_SINGLE_ZONE;
#endif	// defined(AVP_MULTIPLAYER_ONLY_DEVFRONTEND)
AVP_Settings::GameType		AVP_DevelopmentFrontEnd::s_eCurrentGameType = AVP_Settings::AVP_GAMETYPE_DEATH_MATCH;
AVP_Settings::Species		AVP_DevelopmentFrontEnd::s_eCurrentSpecies = AVP_Settings::AVP_SPECIES_HUMAN;
#if defined(AVP_USE_GAME_FRONTEND)
bool						AVP_DevelopmentFrontEnd::s_bFrontEndRequested = false;
#endif	//defined(AVP_USE_GAME_FRONTEND)
int							AVP_DevelopmentFrontEnd::s_iLevelNumber = 0;
int							AVP_DevelopmentFrontEnd::s_iNumberOfPlayers = 1;
int							AVP_DevelopmentFrontEnd::s_iNetworkSessionIndex = -1;
int							AVP_DevelopmentFrontEnd::s_iFMVIndex = 0;
int							AVP_DevelopmentFrontEnd::s_iFMVAudioTrack = 0;
bool						AVP_DevelopmentFrontEnd::s_bQuitRequested = false;

#if defined(AVP_USE_DEV_LOBBY)
bool						AVP_DevelopmentFrontEnd::s_bDevLobbyRequested = false;
#endif	//defined(AVP_USE_DEV_LOBBY)

#if defined(AVP_USE_GAME_LOBBY)
bool						AVP_DevelopmentFrontEnd::s_bGameLobbyRequested = false;
#endif	//defined(AVP_USE_GAME_LOBBY)

#ifdef AVP_LOG_FILE_LOAD_MEMORY_USE
bool						AVP_DevelopmentFrontEnd::s_bDoingSmokeTest = false;
bool						AVP_DevelopmentFrontEnd::s_bTestOnlyOneLevel = false;
bool						AVP_DevelopmentFrontEnd::s_bSmokeTestsFirstLevelLoaded = false;
#endif // #ifdef AVP_LOG_FILE_LOAD_MEMORY_USE

bool						AVP_DevelopmentFrontEnd::s_bUsingDevFE = false;
//my
#if defined(AVP_MULTIPLAYER_ONLY_DEVFRONTEND)
int						    AVP_DevelopmentFrontEnd::s_iCurrentWildcardChosen = 0;
Asura_UnicodeChar*			AVP_DevelopmentFrontEnd::s_acFileWildcards[iAVP_DFE_NUM_WILDCARDS] = {L"*MP_" };
int							AVP_DevelopmentFrontEnd::s_aiCurrentFileWithWildcard[iAVP_DFE_NUM_WILDCARDS] = {0};
int							AVP_DevelopmentFrontEnd::s_aiNumFilesWithWildcards[iAVP_DFE_NUM_WILDCARDS] = {0};
Asura_String*				AVP_DevelopmentFrontEnd::s_apxLevelNamesForEachWildcard[iAVP_DFE_NUM_WILDCARDS] = {NULL};
#else
int						    AVP_DevelopmentFrontEnd::s_iCurrentWildcardChosen = 0;
Asura_UnicodeChar*			AVP_DevelopmentFrontEnd::s_acFileWildcards[iAVP_DFE_NUM_WILDCARDS] = {L"", L"Colony/", L"Jungle/", L"Lab/", L"MP/", L"Pyramid/", L"Refinery/", L"Ruins/", L"Tests/", L"*/A0*", L"*/M0*", L"*/P0*", L"JustAPlane", L"*/*TestArena*" };
#ifdef AVP_LOG_FILE_LOAD_MEMORY_USE
int							AVP_DevelopmentFrontEnd::s_iCurrentSmokeTestOptionChosen = 0;
int							AVP_DevelopmentFrontEnd::s_iNumberOfFrameRates = 600;
Asura_UnicodeChar*			AVP_DevelopmentFrontEnd::s_acSmokeTestOptions[iAVP_DFE_NUM_SMOKETESTFLAGS] = {L"All Levels", L"All SP Levels", L"Alien SP Levels", L"Marine SP Levels", L"Predator SP Levels", L"MP Levels", L"Test Levels", L"Current Level"};
#endif // #ifdef AVP_LOG_FILE_LOAD_MEMORY_USE
int							AVP_DevelopmentFrontEnd::s_aiCurrentFileWithWildcard[iAVP_DFE_NUM_WILDCARDS] = {0,0};
int							AVP_DevelopmentFrontEnd::s_aiNumFilesWithWildcards[iAVP_DFE_NUM_WILDCARDS] = {0, 0};
Asura_String*				AVP_DevelopmentFrontEnd::s_apxLevelNamesForEachWildcard[iAVP_DFE_NUM_WILDCARDS] = {NULL, NULL};
#endif // #else #if defined(AVP_MULTIPLAYER_ONLY_DEVFRONTEND)
bool					    AVP_DevelopmentFrontEnd::s_bLevelNumberInitialised = false;
Asura_ControlMap*           AVP_DevelopmentFrontEnd::s_pxControlMap	= NULL;

AVP_DFE_ONLINEMODES			AVP_DevelopmentFrontEnd::s_eOnlineMode = AVP_DFE_ONLINEMODE_OFFLINE;

int AVP_DevelopmentFrontEnd::s_aaiMinMaxPlayers[NUM_AVP_DFE_GAMEMODES][2] = 
{
#if !defined(AVP_MULTIPLAYER_ONLY_DEVFRONTEND)
	{1, 1},			//AVP_DFE_GAMEMODE_LOAD_SINGLE_ZONE,
#ifdef ASURA_USE_STREAMING
	{1, 1},			//AVP_DFE_GAMEMODE_LOAD_STREAMED_ZONE,
	{1, 1},			//AVP_DFE_GAMEMODE_LOAD_STREAMED_MISSION,
#endif // #ifdef ASURA_USE_STREAMING
	{0, 0},			//AVP_DFE_GAMEMODE_PREVIEW_FMV,
#endif	// defined(AVP_MULTIPLAYER_ONLY_DEVFRONTEND)

#if defined(AVP_USE_GAME_FRONTEND)
	{0, 0},			//AVP_DFE_GAMEMODE_GAME_FRONTEND,
#endif	// defined(AVP_USE_GAME_FRONTEND)

#if defined(AVP_USE_DEV_LOBBY)
	{0, 0},			//AVP_DFE_GAMEMODE_DEV_LOBBY,
#endif	//defined(AVP_USE_DEV_LOBBY)

#if defined(AVP_USE_GAME_LOBBY)
	{0, 0},			//AVP_DFE_GAMEMODE_GAME_LOBBY,
#endif	//defined(AVP_USE_GAME_LOBBY)

#ifdef ASURA_SUPPORT_EXTERNAL_NETWORK
	{2, 8},			//AVP_DFE_GAMEMODE_HOST_NETWORK_GAME,
	{0, 0},			//AVP_DFE_GAMEMODE_JOIN_NETWORK_GAME,
#endif

#ifdef AVP_LOG_FILE_LOAD_MEMORY_USE
	{0, 0},   //AVP_DFE_GAMEMODE_SMOKE_TEST
#endif // #ifdef AVP_LOG_FILE_LOAD_MEMORY_USE

};

const Asura_Char* AVP_DevelopmentFrontEnd::s_aszGameModeNames [NUM_AVP_DFE_GAMEMODES] = 
{
#if !defined(AVP_MULTIPLAYER_ONLY_DEVFRONTEND)
	"SP - LOAD SINGLE ZONE",		//AVP_DFE_GAMEMODE_LOAD_SINGLE_ZONE,
#ifdef ASURA_USE_STREAMING
	"SP - LOAD STREAMED ZONE",		//AVP_DFE_GAMEMODE_LOAD_STREAMED_ZONE,
	"SP - LOAD STREAMED MISSION",	//AVP_DFE_GAMEMODE_LOAD_STREAMED_MISSION,
#endif // #ifdef ASURA_USE_STREAMING
	"PREVIEW FMV",					//AVP_DFE_GAMEMODE_PREVIEW_FMV,
#endif	// defined(AVP_MULTIPLAYER_ONLY_DEVFRONTEND)

#if defined( AVP_USE_GAME_FRONTEND )
	"TEST GAME FRONT END",			//AVP_DFE_GAMEMODE_GAME_FRONTEND
#endif	// defined( AVP_USE_GAME_FRONTEND )

#if defined(AVP_USE_DEV_LOBBY)
	"MP - DEVELOPMENT LOBBY",	// AVP_DFE_GAMEMODE_DEV_LOBBY
#endif	//defined(AVP_USE_DEV_LOBBY)

#if defined(AVP_USE_GAME_LOBBY)
	"MP - GAME LOBBY",			// AVP_DFE_GAMEMODE_GAME_LOBBY
#endif	//defined(AVP_USE_GAME_LOBBY)

#ifdef ASURA_SUPPORT_EXTERNAL_NETWORK
	"MP - HOST NETWORK GAME",		//AVP_DFE_GAMEMODE_HOST_NETWORK_GAME,
	"MP - JOIN NETWORK GAME",		//AVP_DFE_GAMEMODE_JOIN_NETWORK_GAME,
#endif // #ifdef ASURA_SUPPORT_EXTERNAL_NETWORK

#ifdef AVP_LOG_FILE_LOAD_MEMORY_USE
	"RUN A SMOKE TEST",
#endif // #ifdef AVP_LOG_FILE_LOAD_MEMORY_USE
};

// array to allow easy turning off of certain modes for demos etc.
const bool AVP_DevelopmentFrontEnd::s_abGameModesAvailable [NUM_AVP_DFE_GAMEMODES] = 
{
#if !defined(AVP_MULTIPLAYER_ONLY_DEVFRONTEND)
	true,	//AVP_DFE_GAMEMODE_LOAD_SINGLE_ZONE,
#ifdef ASURA_USE_STREAMING
	true,	//AVP_DFE_GAMEMODE_LOAD_STREAMED_ZONE,
	true,	//AVP_DFE_GAMEMODE_LOAD_STREAMED_MISSION,
#endif // #ifdef ASURA_USE_STREAMING
	true,	//AVP_DFE_GAMEMODE_PREVIEW_FMV,
#endif	// defined(AVP_MULTIPLAYER_ONLY_DEVFRONTEND)

#if defined AVP_USE_GAME_FRONTEND 
	true,	//AVP_DFE_GAMEMODE_FRONTEND
#endif	// defined AVP_USE_GAME_FRONTEND

#if defined(AVP_USE_DEV_LOBBY)
	true,	//AVP_DFE_GAMEMODE_DEV_LOBBY
#endif	//defined(AVP_USE_DEV_LOBBY)

#if defined(AVP_USE_GAME_LOBBY)
	true,	//AVP_DFE_GAMEMODE_GAME_LOBBY
#endif	//defined(AVP_USE_GAME_LOBBY)

#ifdef ASURA_SUPPORT_EXTERNAL_NETWORK
	true,	//AVP_DFE_GAMEMODE_HOST_NETWORK_GAME,
	true,	//AVP_DFE_GAMEMODE_JOIN_NETWORK_GAME,
#endif // #ifdef ASURA_SUPPORT_EXTERNAL_NETWORK

#ifdef AVP_LOG_FILE_LOAD_MEMORY_USE
	true,
#endif // #ifdef AVP_LOG_FILE_LOAD_MEMORY_USE
};

const bool AVP_DevelopmentFrontEnd::s_abLevelsAvailable [NUM_AVP_LEVELS] =
{
	true,
};

const Asura_Char* AVP_DevelopmentFrontEnd::s_aszMissionNames [NUM_AVP_MISSIONS] =
{
	"Test Mission 1"
};


const bool AVP_DevelopmentFrontEnd::s_abMissionsAvailable [NUM_AVP_MISSIONS] =
{
	true,
};


bool AVP_DevelopmentFrontEnd::s_abGameTypeAvailable[AVP_Settings::AVP_NUM_GAMETYPES] = 
{
#if defined(AVP_SUBMISSION)
	// Original game modes can be selected now
	true,	// AVP_DFE_MP_GAMETYPE_DISRUPTION_DOMINATION
	true,	// AVP_DFE_MP_GAMETYPE_DEATH_MATCH
	true,	// AVP_DFE_MP_GAMETYPE_HUNT
	true,	// AVP_DFE_MP_GAMETYPE_SPECIES_TEAM_DEATH_MATCH
	true,	// AVP_DFE_MP_GAMETYPE_SURVIVOR
	false,	// AVP_GAMETYPE_MIXED_SPECIES_TEAM_DEATH_MATCH
	false,	// AVP_GAMETYPE_DOMINATION
	//false,	// AVP_DFE_MP_GAMETYPE_INSTANT_ACTION
#else
	// All multiplayer game modes available 
	true,	// AVP_DFE_MP_GAMETYPE_DISRUPTION_DOMINATION
	true,	// AVP_DFE_MP_GAMETYPE_DEATH_MATCH
	true,	// AVP_DFE_MP_GAMETYPE_HUNT
	true,	// AVP_DFE_MP_GAMETYPE_SPECIES_TEAM_DEATH_MATCH
	true,	// AVP_DFE_MP_GAMETYPE_SURVIVOR
	true,	// AVP_GAMETYPE_MIXED_SPECIES_TEAM_DEATH_MATCH
	true,	// AVP_GAMETYPE_DOMINATION
	//false,	// AVP_DFE_MP_GAMETYPE_INSTANT_ACTION
#endif
};


// static function to make a string buffer with server details
const Asura_Char* AVP_DevelopmentFrontEnd::GetServerDesc(int iServerIndex)
{
#ifdef ASURA_SUPPORT_EXTERNAL_NETWORK
	static Asura_Char s_szServerDesc[256];
	//int iNumSessionsAvailable = static_cast<int>(Asura_Network_Session_System::GetNumberOfSessions());
	if (iServerIndex < 0)
	{
		Asura_PrintfToBfr(s_szServerDesc, "-No Servers Available-", 256, true);
	}
	else
	{
		const Asura_Network_Session* pxSession = Asura_Network_Session_System::GetSession(iServerIndex);
		const char* szAddress = Asura_NetTransport_System::GetStringFromAddress(pxSession->GetAddress(), pxSession->GetPort());
		if (pxSession->GetProjectSessionDataSize() != sizeof(AVP_NetworkSession) || pxSession->GetProjectSessionData() == NULL)
		{
			Asura_Assert(false, "Invalid session data");
		
			Asura_PrintfToBfr(s_szServerDesc, "%s %S (%d ms) - %d/%d", 256, true,
				pxSession->GetSessionName(),
				szAddress,
				pxSession->GetPing(),
				pxSession->GetCurrentNumberOfPlayers(),
				pxSession->GetMaxNumberOfPlayers());
		}
		else
		{
			const AVP_NetworkSession* pxData = static_cast<const AVP_NetworkSession*>(pxSession->GetProjectSessionData());
		
			Asura_String xLevelName(pxData->GetLevelName());
			Asura_String xLevelNameNoPathOrExt;
			xLevelName.SplitFilename(NULL, &xLevelNameNoPathOrExt, NULL);

			const Asura_Char* szMissionName = xLevelNameNoPathOrExt.GetString();
			const Asura_Char* szGameType    = AVP_Settings::GetGameTypeName( pxData->GetGameType() );

			Asura_PrintfToBfr(s_szServerDesc, "%s %s (%d ms) - %d/%d - %s - \"%s\"", 256, true,
				pxSession->GetSessionName(),
				szAddress,
				pxSession->GetPing(),
				pxSession->GetCurrentNumberOfPlayers(),
				pxSession->GetMaxNumberOfPlayers(),
				szGameType,
				szMissionName);
		}
	}
	return s_szServerDesc;
#else
	return NULL;
#endif //#ifdef ASURA_SUPPORT_EXTERNAL_NETWORK
}

DEBUGVAR float dbg_fIgnoreInputTime = 0.75f;
void AVP_DevelopmentFrontEnd::Initialise()
{
	s_bSetupComplete = false;
	s_fIgnoreInputTimer = dbg_fIgnoreInputTime;
	// Don't reset selections to make restarting easier.
#if defined(AVP_MULTIPLAYER_ONLY_DEVFRONTEND)
	s_eCurrentlyHighlitOption = AVP_DFE_OPTION_ACCEPT;
#else
	s_eCurrentlyHighlitOption = AVP_DFE_OPTION_LEVEL_SELECTION;
#endif	// defined(AVP_MULTIPLAYER_ONLY_DEVFRONTEND)
	s_eMenuMode = AVP_DFE_MENUMODE_GAMESETUP;
	s_bLevelNumberInitialised = false;

#if defined(AVP_USE_DEV_LOBBY)
	s_bDevLobbyRequested = false;
#endif	//defined(AVP_USE_DEV_LOBBY)

#if defined(AVP_USE_GAME_LOBBY)
	s_bGameLobbyRequested = false;
#endif	//defined(AVP_USE_GAME_LOBBY)

	s_pxControlMap	= new Asura_ControlMap (NUM_AVP_DFE_BUTTON);

	Platform_Initialise();
	Asura_ControlManager::AddDefaultControls(s_pxControlMap);

	OnEnterMode(s_iCurrentGameMode);

	// Make sure we have a valid level.
//	if (!IsLevelSelectable())
//	{
//		AlterLevelSelection(true);
//	}

#ifdef ASURA_SUPPORT_COMMAND_CONSOLE
	//AVP_CommandConsole::SetupCmdsAndVars(AVP_CommandConsole::VAR_RESTRICT_FRONTEND);
#endif //#ifdef ASURA_SUPPORT_COMMAND_CONSOLE
}

bool AVP_DevelopmentFrontEnd::Update()
{
#if defined(AVP_USE_DEV_LOBBY)
	// check for MP game invitations and automatically switch to dev lobby if one is found
	if (Asura_Lobby_System::IsInvitationPending())
	{
		// find which lobby is wanted, game has the priority
		if( s_bDevLobbyRequested && !s_bGameLobbyRequested )
		{
			// make sure we go through the "starting network mode" functionality.
			if (s_iCurrentGameMode != AVP_DFE_GAMEMODE_DEV_LOBBY)
			{
				OnLeaveMode(s_iCurrentGameMode);
				s_iCurrentGameMode = AVP_DFE_GAMEMODE_DEV_LOBBY;
				OnEnterMode(s_iCurrentGameMode);
			}
		}
		else
		{
			// make sure we go through the "starting network mode" functionality.
			if (s_iCurrentGameMode != AVP_DFE_GAMEMODE_GAME_LOBBY)
			{
				OnLeaveMode(s_iCurrentGameMode);
				s_iCurrentGameMode = AVP_DFE_GAMEMODE_GAME_LOBBY;
				OnEnterMode(s_iCurrentGameMode);
			}
		}
		// press "accept" which should do nothing until network mode is initialised.
		OnAccept();
	}
#endif

	Asura_List< Asura_String > xFileNames;
	Asura_List< Asura_String > xFileNamesP;
//	Asura_String* pxLevelNames;
	if( !s_bLevelNumberInitialised )
	{
		//we create a list of files with specific extension (levels)
		Platform_GetLevelFiles( xFileNames );
		int iInd2 = 0;
		int NumEntries;
		//we create level list for each wildcard		
		Asura_Char szBuffer[256];

		for(int iInd = 0; iInd< iAVP_DFE_NUM_WILDCARDS; iInd++)
		{

			//find file names which match specific wildcard
			Asura_PrintfToBfr(szBuffer, "%S", 256, true,  s_acFileWildcards[iInd]);		
			Asura_String xBuffer(szBuffer);
			xBuffer += "*";			
			NumEntries = 0;
			for(Asura_List_It< Asura_String > xIt(&xFileNames); !xIt.Done(); xIt.Next())
			{					
				if(MatchWithWildcard(xIt.GetCurrent()->GetString(), xBuffer.GetString()))
				{
					xFileNamesP.AddToList(xIt.GetCurrent());
					++NumEntries;
				}
			}
			//copy finded file names to array for specific wildcard
			//if(xFileNamesP.GetNumEntries()!=0)
			if(NumEntries != 0)
			{
				s_apxLevelNamesForEachWildcard[iInd] = new Asura_String[NumEntries];
				s_aiNumFilesWithWildcards[iInd] = NumEntries;
				iInd2 = 0;
				for(Asura_List_It< Asura_String > xIt(&xFileNamesP); !xIt.Done(); xIt.Next(), ++iInd2)
				{				
					s_apxLevelNamesForEachWildcard[iInd][iInd2] = *xIt.GetCurrent();		
				}
				xFileNamesP.ClearList();
			}
		
		}



		for(Asura_List_It< Asura_String > xIt(&xFileNames); !xIt.Done(); xIt.Next())
		{												
			delete xIt.GetCurrent();				
		}
		s_bLevelNumberInitialised = true;		

		// make sure we only do this the first time they enter the menu
		{
			static bool bDoOnce = false;
			extern char* DefaultFileToLoad;
			if (DefaultFileToLoad != NULL && !bDoOnce)
			{
				for(int iInd = iAVP_DFE_NUM_WILDCARDS - 1; iInd > -1; --iInd)
				{
					for(int iInd2 = 0; iInd2< s_aiNumFilesWithWildcards[iInd]; iInd2++)
					{
						if (s_apxLevelNamesForEachWildcard[iInd][iInd2].EndsWith(DefaultFileToLoad, true))
						{
							s_aiCurrentFileWithWildcard[iInd] = iInd2;
							s_iCurrentWildcardChosen = iInd;
							break;
						}
					}
				}
				if (s_apxLevelNamesForEachWildcard[s_iCurrentWildcardChosen][s_aiCurrentFileWithWildcard[s_iCurrentWildcardChosen]].EndsWith(DefaultFileToLoad, true))
				{
					OnAccept();
				}
			}
			bDoOnce = true;
		}
	}
	
	switch (s_eMenuMode)
	{
		case AVP_DFE_MENUMODE_GAMESETUP:
		{
#ifdef ASURA_SUPPORT_EXTERNAL_NETWORK
			if (s_iCurrentGameMode == AVP_DFE_GAMEMODE_JOIN_NETWORK_GAME)
			{
				if (!Asura_Network_Session_System::IsInitialised())
				{
					Asura_Network_Session_System::Initialise();
				}
				Asura_Network_Session_System::Update();

			}
#endif //#ifdef ASURA_SUPPORT_EXTERNAL_NETWORK

#ifdef ASURA_SUPPORT_COMMAND_CONSOLE
		    Asura_CommandConsole::Update();
			if (!Asura_CommandConsole::IsConsoleDown())
#endif
			{
				Update_KeyControls();
			}
			Update_SelectedServer();
			break;
		}

		case AVP_DFE_MENUMODE_STARTING_NETWORK:
		{
			bool bFinished = UpdateStartingNetworkMode();

			if (bFinished)
			{
				if (NeedToStartNetworkMode())
				{
					// failed to start online mode, switch back to single player
#if !defined(AVP_MULTIPLAYER_ONLY_DEVFRONTEND)
					s_iCurrentGameMode = AVP_DFE_GAMEMODE_LOAD_SINGLE_ZONE;
#else
	#if defined AVP_USE_GAME_FRONTEND 
					s_iCurrentGameMode = AVP_DFE_GAMEMODE_GAME_FRONTEND;
	#else
					s_iCurrentGameMode = AVP_DFE_GAMEMODE_HOST_NETWORK_GAME;
	#endif	//defined AVP_USE_GAME_FRONTEND 
#endif	// !defined(AVP_MULTIPLAYER_ONLY_DEVFRONTEND)
					s_eMenuMode = AVP_DFE_MENUMODE_GAMESETUP;
				}
				else
				{
					s_eMenuMode = AVP_DFE_MENUMODE_GAMESETUP;
				}
			}
			break;
		}

		case AVP_DFE_MENUMODE_BUILD_FMV_LIST:
		{
			bool bFinished = Platform_UpdateBuildFMVList();
			if (bFinished)
			{
				s_eMenuMode = AVP_DFE_MENUMODE_GAMESETUP;
			}
			Platform_RenderBuildFMVList();
			break;
		}

		default:
		{
			Asura_Assert(false, "invalid mode");
			break;
		}
	}
#ifdef AVP_LOG_FILE_LOAD_MEMORY_USE
	if( s_bDoingSmokeTest && s_bSmokeTestsFirstLevelLoaded )
	{
		if( !s_bTestOnlyOneLevel )
		{
			AVP_SmokeTest::PrintOutputToLog();
			AVP_SmokeTest::ResetData();

			AlterChosenWildcardLevel(true);
		
			// If s_aiCurrentFileWithWildcard[s_iCurrentWildcardChosen] = 0 it means we have done all the levels for this wildcard 
			// and then we change either wildcard or species or both depending on the wildcard / speicies we have now
			if( s_aiCurrentFileWithWildcard[s_iCurrentWildcardChosen] == 0 )
			{
				if( !MoreLevelsForTheSmokeTest() )
				{
					// end this test
					PrepareForSmokeTest();
					s_bDoingSmokeTest = false;
					s_bLevelNumberInitialised = true;
					AVP_SmokeTest::SetRunningSmokeTest( false );
				}
				else
				{
					OnAccept();
				}
			}
			else
			{
				OnAccept();
			}
		}
		else
		{
			AVP_SmokeTest::PrintOutputToLog();
			AVP_SmokeTest::ResetData();

			// end this test
			PrepareForSmokeTest();
			s_bDoingSmokeTest = false;
			s_bLevelNumberInitialised = true;
			AVP_SmokeTest::SetRunningSmokeTest( false );
		}	
	}
	else if( s_bDoingSmokeTest && !s_bSmokeTestsFirstLevelLoaded )
	{
		s_bSmokeTestsFirstLevelLoaded = true;
		OnAccept();
	}
#endif // #ifdef AVP_LOG_FILE_LOAD_MEMORY_USE

	//Asura_Timers::HandlePrimaryTimer();
	return s_bSetupComplete;
}

void AVP_DevelopmentFrontEnd::Shutdown()
{
	OnLeaveMode(s_iCurrentGameMode);
	for(int iInd = 0; iInd< iAVP_DFE_NUM_WILDCARDS; iInd++)
	{
		delete[] s_apxLevelNamesForEachWildcard[iInd];
	}

	if (!IsNetwork())
	{
		ShutdownNetworkMode();
	}

	Platform_ClearFMVList();

	Asura_ControlManager::RemoveDefaultControls(s_pxControlMap);
	Platform_Shutdown();

	delete(s_pxControlMap);
	s_pxControlMap	= NULL;
}

void AVP_DevelopmentFrontEnd::Update_SelectedServer()
{
#ifdef ASURA_SUPPORT_EXTERNAL_NETWORK
	// if we haven't got a valid server selected, select the first one, or -1.
	int iNumSessionsAvailable = static_cast<int>(Asura_Network_Session_System::GetNumberOfSessions());
	if (s_iNetworkSessionIndex < 0 || s_iNetworkSessionIndex >= iNumSessionsAvailable)
	{
		s_iNetworkSessionIndex = iNumSessionsAvailable - 1;
	}
	else
	{
		if (s_iCurrentGameMode == AVP_DFE_GAMEMODE_JOIN_NETWORK_GAME)
		{
			// set the game mode and level to match the server's
			const Asura_Network_Session* pxSession = Asura_Network_Session_System::GetSession(s_iNetworkSessionIndex);
			const AVP_NetworkSession* pxSessionData = reinterpret_cast<const AVP_NetworkSession*>(pxSession->GetProjectSessionData());
			//const char* szLevelName = pxSessionData->GetLevelName();

			//s_iLevelNumber = -1;

			//for (int i = 0; i < NUM_AVP_LEVELS; ++i)
			//{
			//	if (strncmp( szLevelName, s_aszLevelNames[i], uAVP_MAX_LEVELNAME_LEN ) == 0)
			//	{
			//		s_iLevelNumber = i;
			//		break;
			//	}
			//}

			Asura_Assert(s_iLevelNumber != -1, "server set unknown level");

			int iFirstAllowedLevel = 0;	// TODO - based on gametype.
			int iLastAllowedLevel = NUM_AVP_LEVELS; // TODO
			Asura_Assert(((s_iLevelNumber >= iFirstAllowedLevel) && (s_iLevelNumber <= iLastAllowedLevel)), "invalid level ID");
			if ((s_iLevelNumber < iFirstAllowedLevel) || (s_iLevelNumber > iLastAllowedLevel))
			{
				s_iLevelNumber = iFirstAllowedLevel;
			}

			// Better copy all the settings.
			pxSessionData->CopyToSettings();

			Asura_Assert(IsLevelSelectable(), "server set invalid level for game type!");
		}
	}
#endif //#ifdef ASURA_SUPPORT_EXTERNAL_NETWORK
}

void AVP_DevelopmentFrontEnd::Update_KeyControls()
{
	if (s_fIgnoreInputTimer >= 0.0f)
	{
		s_fIgnoreInputTimer -= Asura_Timers::GetFrameTime();
		return;
	}

	if ( s_pxControlMap->GetValue( AVP_DFE_BUTTON_QUIT ) )
	{
		AVP_Core::RequestQuit();
		s_bSetupComplete = true;
		return;
	}

#if defined(AVP_DFE_ALLOW_QUIT)
	if ( s_eCurrentlyHighlitOption == AVP_DFE_OPTION_QUIT &&
		 s_pxControlMap->GetValue( AVP_DFE_BUTTON_SELECT ) )
	{
		OnQuit();
		return;
	}
#endif // #if defined(AVP_DFE_ALLOW_QUIT)

	if ( s_eCurrentlyHighlitOption == AVP_DFE_OPTION_ACCEPT &&
		 s_pxControlMap->GetValue( AVP_DFE_BUTTON_SELECT ) )
	{
		OnAccept();
		return;
	}
	if ( s_pxControlMap->GetValue( AVP_DFE_BUTTON_UP ))
	{
		bool bFinalised = false;
		while (!bFinalised)
		{
			if (static_cast<int>(s_eCurrentlyHighlitOption) == 0)
			{
				s_eCurrentlyHighlitOption = NUM_AVP_DFE_OPTIONS;
			}
			s_eCurrentlyHighlitOption = static_cast<AVP_DFE_OPTIONS>(static_cast<int>(s_eCurrentlyHighlitOption) - 1);
			bFinalised = IsOptionSelectable(s_eCurrentlyHighlitOption);
		}
	}
	if ( s_pxControlMap->GetValue( AVP_DFE_BUTTON_DOWN ))
	{
		bool bFinalised = false;
		while (!bFinalised)
		{
			s_eCurrentlyHighlitOption = static_cast<AVP_DFE_OPTIONS>(static_cast<int>(s_eCurrentlyHighlitOption) + 1);
			if (s_eCurrentlyHighlitOption == NUM_AVP_DFE_OPTIONS)
			{
				s_eCurrentlyHighlitOption = static_cast<AVP_DFE_OPTIONS>(0);
			}
			bFinalised = IsOptionSelectable(s_eCurrentlyHighlitOption);
		}
	}
	if ( s_pxControlMap->GetValue( AVP_DFE_BUTTON_LEFT ) || s_pxControlMap->GetValue( AVP_DFE_BUTTON_RIGHT ))
	{
		bool bIncrement = s_pxControlMap->GetValue( AVP_DFE_BUTTON_LEFT ) ? false : true;
		switch (s_eCurrentlyHighlitOption)
		{
		case AVP_DFE_OPTION_GAME_MODE:
			AlterGameMode(bIncrement);
			break;
		case AVP_DFE_OPTION_GAME_TYPE:
			AlterGameType(bIncrement);
			break;
		case AVP_DFE_OPTION_CHAR_TYPE:
			AlterPlayerSpecies(bIncrement);
			break;
		case AVP_DFE_OPTION_NUM_PLAYERS:
			AlterNumPlayers(bIncrement);
			break;
		case AVP_DFE_OPTION_SERVER_TO_JOIN:
			AlterServerToJoin(bIncrement);
			break;
#if !defined(AVP_MULTIPLAYER_ONLY_DEVFRONTEND)
		case AVP_DFE_OPTION_SEARCH_WITH_WILDCARD: 
			AlterChosenWildcard( bIncrement);
			break;
#endif // !defined(AVP_MULTIPLAYER_ONLY_DEVFRONTEND)
		case AVP_DFE_OPTION_LEVEL_SELECTION:
			{
				u_int uTimes = (Asura_UserInput::KeyPressed(ASURA_KEYID_CTRL) ? 10 : 1);

				for (; uTimes; --uTimes) AlterChosenWildcardLevel(bIncrement);
			}
			break;
#if !defined(AVP_MULTIPLAYER_ONLY_DEVFRONTEND)
		case AVP_DFE_OPTION_FMV_TO_PLAY:
			AlterFMVToPlay(bIncrement);
			break;
		case AVP_DFE_OPTION_FMV_AUDIO_TRACK:
			AlterFMVAudioTrack(bIncrement);
			break;
#endif	// !defined(AVP_MULTIPLAYER_ONLY_DEVFRONTEND)
#ifdef AVP_LOG_FILE_LOAD_MEMORY_USE
		case AVP_DFE_OPTION_SMOKETEST_OPTION:
			AlterChosenSmokeTestOption( bIncrement );
			break;
		case AVP_DFE_OPTION_FRAMERATES_OPTION:
			AlterNumberOfFramerates( bIncrement );
			break;
#endif //#ifdef AVP_LOG_FILE_LOAD_MEMORY_USE
		default:
			break;
		}
	}
}

DEBUGVAR float dbg_fFontSize = 1.0f / 480.0f;
DEBUGVAR float dbg_fYPositionSpacing = 0.075f;
DEBUGVAR float dbg_fMaxNumOfLines = 2.0f;
void AVP_DevelopmentFrontEnd::Render()
{
	// Render black quad over entire screen.
	Asura_Render::Overlay.Colour = 0xff000000;
	Asura_Render::Overlay.TextureIndex = -1;
	Asura_Render::Overlay.TranslucencyMode = ASURA_TRANSLUCENCY_OFF;
	Asura_Render::Overlay.SetRectXY(0.0f, 0.0f, static_cast<float>(Asura_Viewport::GetWidth()), static_cast<float>(Asura_Viewport::GetHeight()));
	Asura_Render::Overlay.Render();
	Asura_Render::FlushDisplay();

	{

		// Setup font and constants.
		const Asura_Handle iFontHandle = AVP_Core::GetFontHandle(AVP_Core::MENU_FONT);
		Asura_Font_System::SetFont(iFontHandle);
		Asura_Font_System::SetFontScale(iFontHandle, 1.f);
		float fFontSize = Asura_Viewport::GetWidth() * dbg_fFontSize;

		// Menu title
		PrintWithDropShadow(0.75f, 
								 0.08f, 
								 fFontSize, 
								 0xffB0B0B0,
								 ASURA_TEXTFORMAT_RELATIVEPOSITION | ASURA_TEXTFORMAT_CENTREJUSTIFIED,
								 "AvP3 - DEVELOPER MENU");

		float fXPosition = 0.05f;
		float fYPosition = 0.2f;

		Asura_Char szBuffer[256];
		// Game Mode:
		Asura_PrintfToBfr(szBuffer, "Game Mode    : %s", 256, true, s_aszGameModeNames[s_iCurrentGameMode]);
		PrintWithDropShadow(fXPosition,
								 fYPosition, 
								 fFontSize, 
								 GetOptionColour(AVP_DFE_OPTION_GAME_MODE),
								 ASURA_TEXTFORMAT_RELATIVEPOSITION | ASURA_TEXTFORMAT_LEFTJUSTIFIED, 
								 szBuffer);
		fYPosition += dbg_fYPositionSpacing;

		// Game Type:
		Asura_PrintfToBfr(szBuffer, "MP Game Type : %s", 256, true, AVP_Settings::GetGameTypeName(s_eCurrentGameType));
		PrintWithDropShadow(fXPosition,
								 fYPosition, 
								 fFontSize, 
								 GetOptionColour(AVP_DFE_OPTION_GAME_TYPE),
								 ASURA_TEXTFORMAT_RELATIVEPOSITION | ASURA_TEXTFORMAT_LEFTJUSTIFIED, 
								 szBuffer);
		fYPosition += dbg_fYPositionSpacing;

		// Char Type:
		Asura_PrintfToBfr(szBuffer, "Player species : %s", 256, true, AVP_Settings::GetSpeciesName(s_eCurrentSpecies));
		PrintWithDropShadow(fXPosition,
			fYPosition, 
			fFontSize, 
			GetOptionColour(AVP_DFE_OPTION_CHAR_TYPE),
			ASURA_TEXTFORMAT_RELATIVEPOSITION | ASURA_TEXTFORMAT_LEFTJUSTIFIED, 
			szBuffer);
		fYPosition += dbg_fYPositionSpacing;

#if !defined(AVP_MULTIPLAYER_ONLY_DEVFRONTEND)
		// Wildcard:
		Asura_PrintfToBfr(szBuffer, "Search levels with wildcard  : %S", 256, true,  s_acFileWildcards[s_iCurrentWildcardChosen]);
		PrintWithDropShadow(fXPosition,
							fYPosition, 
							fFontSize, 
							GetOptionColour(AVP_DFE_OPTION_SEARCH_WITH_WILDCARD),
							ASURA_TEXTFORMAT_RELATIVEPOSITION | ASURA_TEXTFORMAT_LEFTJUSTIFIED, 
							szBuffer);
		fYPosition += dbg_fYPositionSpacing;
#endif	// !defined(AVP_MULTIPLAYER_ONLY_DEVFRONTEND)
		// Level:
		const char* szMapName;
#ifdef ASURA_USE_STREAMING
		if (s_iCurrentGameMode == AVP_DFE_GAMEMODE_LOAD_STREAMED_MISSION)
		{
			szMapName = s_aszMissionNames[s_iLevelNumber];
		}
		else
#endif
		{
			if(s_aiNumFilesWithWildcards[s_iCurrentWildcardChosen] == 0) 
			{
				szMapName = "NULL";
			}
			else
			{
				szMapName = (s_apxLevelNamesForEachWildcard[s_iCurrentWildcardChosen][s_aiCurrentFileWithWildcard[s_iCurrentWildcardChosen]]).GetString();
			}			
		}

		
		static Asura_Bounding_Rectangle xBoundingBox(CountRectangle(fXPosition, 
																fYPosition, 																	
																szBuffer));

		Asura_PrintfToBfr(szBuffer, "Map          : %s", 256, true, szMapName);
		PrintWithDropShadowInRectangle(xBoundingBox, 
											GetOptionColour(AVP_DFE_OPTION_LEVEL_SELECTION), fFontSize, 
											ASURA_TEXTFORMAT_RELATIVEPOSITION | ASURA_TEXTFORMAT_LEFTJUSTIFIED | ASURA_TEXTFORMAT_WORDWRAP | ASURA_TEXTFORMAT_RELATIVETORECT,
											szBuffer);
		fYPosition += dbg_fMaxNumOfLines*dbg_fYPositionSpacing;
		/*PrintWithDropShadow(fXPosition,
								 fYPosition, 
								 fFontSize, 
								 GetOptionColour(AVP_DFE_OPTION_LEVEL_SELECTION),
								 ASURA_TEXTFORMAT_RELATIVEPOSITION | ASURA_TEXTFORMAT_LEFTJUSTIFIED, 
								 szBuffer);
		fYPosition += dbg_fYPositionSpacing;*/
#if !defined(AVP_MULTIPLAYER_ONLY_DEVFRONTEND)
		if (s_iCurrentGameMode == AVP_DFE_GAMEMODE_PREVIEW_FMV)
		{
			// FMV to play
			Asura_PrintfToBfr(szBuffer, "FMV          : %S", 256, true, Platform_GetFMVDescription());
			PrintWithDropShadow(fXPosition,
									fYPosition, 
									fFontSize, 
									GetOptionColour(AVP_DFE_OPTION_FMV_TO_PLAY),
									ASURA_TEXTFORMAT_RELATIVEPOSITION | ASURA_TEXTFORMAT_LEFTJUSTIFIED, 
									szBuffer);
			fYPosition += dbg_fYPositionSpacing;

			// Audio track:
			Asura_PrintfToBfr(szBuffer, "Audio track  : %d", 256, true, s_iFMVAudioTrack);
			PrintWithDropShadow(fXPosition,
									fYPosition, 
									fFontSize, 
									GetOptionColour(AVP_DFE_OPTION_FMV_AUDIO_TRACK),
									ASURA_TEXTFORMAT_RELATIVEPOSITION | ASURA_TEXTFORMAT_LEFTJUSTIFIED, 
									szBuffer);
			fYPosition += dbg_fYPositionSpacing;
		}
		else
#endif	// !defined(AVP_MULTIPLAYER_ONLY_DEVFRONTEND)

#ifdef AVP_LOG_FILE_LOAD_MEMORY_USE
		if ( s_iCurrentGameMode == AVP_DFE_GAMEMODE_SMOKE_TEST )
		{
			// Choose how much you want to test with the smoketest
			Asura_PrintfToBfr(szBuffer, "Levels to load : %S", 256, true, s_acSmokeTestOptions[s_iCurrentSmokeTestOptionChosen]);
			PrintWithDropShadow(fXPosition,
									fYPosition, 
									fFontSize, 
									GetOptionColour(AVP_DFE_OPTION_SMOKETEST_OPTION),
									ASURA_TEXTFORMAT_RELATIVEPOSITION | ASURA_TEXTFORMAT_LEFTJUSTIFIED, 
									szBuffer);
			fYPosition += dbg_fYPositionSpacing;
			// Num of framerats to get per level:
			Asura_PrintfToBfr(szBuffer, "Framerate samples per level : %d", 256, true, s_iNumberOfFrameRates);
			PrintWithDropShadow(fXPosition,
									fYPosition, 
									fFontSize, 
									GetOptionColour(AVP_DFE_OPTION_FRAMERATES_OPTION),
									ASURA_TEXTFORMAT_RELATIVEPOSITION | ASURA_TEXTFORMAT_LEFTJUSTIFIED, 
									szBuffer);
			fYPosition += dbg_fYPositionSpacing;

		}
		else
#endif // #ifdef AVP_LOG_FILE_LOAD_MEMORY_USE

		{
#ifdef ASURA_SUPPORT_EXTERNAL_NETWORK
			// Num Players:
			Asura_PrintfToBfr(szBuffer, "Players      : %d", 256, true, s_iNumberOfPlayers);
			PrintWithDropShadow(fXPosition,
									fYPosition, 
									fFontSize, 
									GetOptionColour(AVP_DFE_OPTION_NUM_PLAYERS),
									ASURA_TEXTFORMAT_RELATIVEPOSITION | ASURA_TEXTFORMAT_LEFTJUSTIFIED, 
									szBuffer);
			fYPosition += dbg_fYPositionSpacing;

			// Server to join
			int iNumSessionsAvailable = static_cast<int>(Asura_Network_Session_System::GetNumberOfSessions());
			Asura_PrintfToBfr(szBuffer, "Server       : %d/%d %s", 256, true, s_iNetworkSessionIndex + 1, iNumSessionsAvailable, GetServerDesc(s_iNetworkSessionIndex));
			PrintWithDropShadow(fXPosition,
									fYPosition, 
									fFontSize, 
									GetOptionColour(AVP_DFE_OPTION_SERVER_TO_JOIN),
									ASURA_TEXTFORMAT_RELATIVEPOSITION | ASURA_TEXTFORMAT_LEFTJUSTIFIED, 
									szBuffer);
			fYPosition += dbg_fYPositionSpacing;
#endif
		}

		// Accept
		PrintWithDropShadow(fXPosition,
								 fYPosition, 
								 fFontSize, 
								 GetOptionColour(AVP_DFE_OPTION_ACCEPT),
								 ASURA_TEXTFORMAT_RELATIVEPOSITION | ASURA_TEXTFORMAT_LEFTJUSTIFIED, 
								 "ACCEPT");
		fYPosition += dbg_fYPositionSpacing;
#if defined(AVP_DFE_ALLOW_QUIT)
		// Quit
		PrintWithDropShadow(fXPosition,
								 fYPosition, 
								 fFontSize, 
								 GetOptionColour(AVP_DFE_OPTION_QUIT),
								 ASURA_TEXTFORMAT_RELATIVEPOSITION | ASURA_TEXTFORMAT_LEFTJUSTIFIED, 
								 "QUIT");
		fYPosition += dbg_fYPositionSpacing;
#endif // #if defined(AVP_DFE_ALLOW_QUIT)
	}
	
	if (s_eMenuMode == AVP_DFE_MENUMODE_STARTING_NETWORK)
	{
		RenderStartingNetworkMode();
	}
}


void AVP_DevelopmentFrontEnd::OnAccept()
{
	s_bUsingDevFE = true;

	// (MH - 20.07.2004) prevent Accept if something's gone wrong e.g. with selected server
	// Gun used to force you to wait for common data to have finished loading asynchronously.
	if (!IsOptionSelectable(AVP_DFE_OPTION_ACCEPT))
	{
		return;
	}

	switch(s_iCurrentGameMode)
	{

#if defined AVP_USE_GAME_FRONTEND
		case AVP_DFE_GAMEMODE_GAME_FRONTEND:
		{
			s_bFrontEndRequested = true;
			s_bUsingDevFE = false;
			break;
		}
#endif	// defined(AVP_USE_GAME_FRONTEND)

#if !defined(AVP_MULTIPLAYER_ONLY_DEVFRONTEND)
		case AVP_DFE_GAMEMODE_PREVIEW_FMV:
		{
			Platform_PreviewFMV();
			return;
		}

		case AVP_DFE_GAMEMODE_LOAD_SINGLE_ZONE:
		{
			//If loading a single zone in a tools build , set the filename up so that tools does the loading.
			//Somewhat ugly , but the whole DefaultFileToLoad malarky is a bit of a mess
			//RAR 05/10/2006
			static char szFilenameBuffer[256];
			Asura_PrintfToBfr(szFilenameBuffer,"%s",sizeof(szFilenameBuffer),true,(s_apxLevelNamesForEachWildcard[s_iCurrentWildcardChosen][s_aiCurrentFileWithWildcard[s_iCurrentWildcardChosen]]).GetString());
			extern char* DefaultFileToLoad;
			DefaultFileToLoad = szFilenameBuffer;
		}	break;



#ifdef ASURA_USE_STREAMING
		case AVP_DFE_GAMEMODE_LOAD_STREAMED_ZONE:
		{
			AVP_Settings::SetGameType( AVP_Settings::AVP_GAMETYPE_STORY );
			break;
		}

		case AVP_DFE_GAMEMODE_LOAD_STREAMED_MISSION:
		{
			AVP_Settings::SetGameType( AVP_Settings::AVP_GAMETYPE_STORY );
			break;
		}
#endif
#endif	// !defined(AVP_MULTIPLAYER_ONLY_DEVFRONTEND)

#ifdef ASURA_SUPPORT_EXTERNAL_NETWORK
		case AVP_DFE_GAMEMODE_HOST_NETWORK_GAME:
		{
			// Extra host-only settings, with intentional fallthrough afterwards
			AVP_Settings::SetMaxNetPlayers(s_iNumberOfPlayers);
		}
		case AVP_DFE_GAMEMODE_JOIN_NETWORK_GAME:
		{
			// Set a death match by default
			AVP_Settings::SetGameType( AVP_Settings::AVP_GAMETYPE_DEATH_MATCH );
			
			//GJDH
			//For MP games there's a this sub-type as well... perhaps could be brought together
			//It's a legacy thing...
			switch( s_eCurrentGameType )
			{
				default:
				{
					Asura_Assert( false, "AVP_DevelopmentFrontEnd -> enumeration value not handled in switch" );
					break;
				}
				case AVP_Settings::AVP_GAMETYPE_DISRUPTION_DOMINATION:
				case AVP_Settings::AVP_GAMETYPE_DEATH_MATCH:
				case AVP_Settings::AVP_GAMETYPE_HUNT:
				case AVP_Settings::AVP_GAMETYPE_SPECIES_TEAM_DEATH_MATCH:
				case AVP_Settings::AVP_GAMETYPE_SURVIVOR:
				case AVP_Settings::AVP_GAMETYPE_MIXED_SPECIES_TEAM_DEATH_MATCH:
				case AVP_Settings::AVP_GAMETYPE_DOMINATION:
				//case AVP_Settings::AVP_GAMETYPE_INSTANT_ACTION:
				{
					AVP_Settings::SetGameType(s_eCurrentGameType);
					break;
				}
			}		// End of switch.
			break;

		}

#if defined(AVP_USE_DEV_LOBBY)
		case AVP_DFE_GAMEMODE_DEV_LOBBY:
		{
			s_bDevLobbyRequested = true;
			s_bGameLobbyRequested = false;
			AVP_GUIMenu_System::SetLobbyType(AVP_GUIMenu_System::LT_DEVELOPMENT);
			break;
		}
#endif	//defined(AVP_USE_DEV_LOBBY)

#if defined(AVP_USE_GAME_LOBBY)
		case AVP_DFE_GAMEMODE_GAME_LOBBY:
		{
			s_bGameLobbyRequested = true;
			s_bDevLobbyRequested = false;
			AVP_GUIMenu_System::SetLobbyType(AVP_GUIMenu_System::LT_GAME);
			break;
		}
#endif	//defined(AVP_USE_GAME_LOBBY)

#endif // #ifdef ASURA_SUPPORT_EXTERNAL_NETWORK

		// MKC, set up the start values for the smoke test
#ifdef AVP_LOG_FILE_LOAD_MEMORY_USE
		case AVP_DFE_GAMEMODE_SMOKE_TEST:
		{
			if( !s_bDoingSmokeTest )
			{
				s_bDoingSmokeTest = true;	
				PrepareForSmokeTest();
				AVP_SmokeTest::SetRunningSmokeTest( true );
			}
			break;
		}
#endif // #ifdef AVP_LOG_FILE_LOAD_MEMORY_USE

		default:
		{
			Asura_Assertf( false, ("Invalid game mode:%d", s_iCurrentGameMode) );
		}
	}		// End of switch.

	switch(s_iCurrentGameMode)
	{
#ifdef ASURA_USE_STREAMING
		case AVP_DFE_GAMEMODE_LOAD_STREAMED_MISSION:
		{
			// TODO: set AVP_Settings mission
			Asura_Assert(false, "need mission handling code");
			break;
		}
#endif

#if !defined(AVP_MULTIPLAYER_ONLY_DEVFRONTEND)
		case AVP_DFE_GAMEMODE_PREVIEW_FMV:
#endif	//!defined(AVP_MULTIPLAYER_ONLY_DEVFRONTEND)

#if defined AVP_USE_GAME_FRONTEND
		case AVP_DFE_GAMEMODE_GAME_FRONTEND:
#endif

#if defined(AVP_USE_DEV_LOBBY)
		case AVP_DFE_GAMEMODE_DEV_LOBBY:
#endif	//defined(AVP_USE_DEV_LOBBY)

#if defined(AVP_USE_GAME_LOBBY)
		case AVP_DFE_GAMEMODE_GAME_LOBBY:
#endif	//defined(AVP_USE_GAME_LOBBY)
		{
			// We're not to start a level...
			break;
		}

		default:
		{
			//AVP_Settings::SetLevelName(s_aszLevelNames[s_iLevelNumber]);
			AVP_Settings::SetLevelName((s_apxLevelNamesForEachWildcard[s_iCurrentWildcardChosen][s_aiCurrentFileWithWildcard[s_iCurrentWildcardChosen]]).GetString());
			break;
		}
	}		// End of switch.

	AVP_Settings::SetIsServer(IsServer());
	AVP_Settings::SetExternalNetworkActive(IsNetwork());
	AVP_Settings::SetOnlineGame(false);
	AVP_Settings::SetPlayerSpecies(s_eCurrentSpecies);
	AVP_Game_Core::SetRespawnSpecies(s_eCurrentSpecies);

#ifdef ASURA_SUPPORT_EXTERNAL_NETWORK
	if (IsNetwork() && !IsServer())
	{
		Asura_Assert(s_iNetworkSessionIndex >= 0 && s_iNetworkSessionIndex < static_cast<int>
			(Asura_Network_Session_System::GetNumberOfSessions()), "invalid session index");
		const Asura_Network_Session* pxSession = Asura_Network_Session_System::GetSession(s_iNetworkSessionIndex);
		
		// 360 needs to setup the secure connection now!
		pxSession->PrepareConnection();

		AVP_Settings::SetAddressToConnectTo(pxSession->GetAddress(), pxSession->GetPort());
		Asura_NetTransport_System::SetServerSessionID( pxSession->GetServerSessionID() );
	}
#endif //#ifdef ASURA_SUPPORT_EXTERNAL_NETWORK
	s_bSetupComplete = true;
	s_bQuitRequested = false;
}

void AVP_DevelopmentFrontEnd::OnQuit()
{
	s_bSetupComplete = true;
	s_bQuitRequested = true;
}

void AVP_DevelopmentFrontEnd::AlterGameMode(bool bIncrement)
{
	int iPrevGameMode = s_iCurrentGameMode;
	
	// s_abGameModesAvailable limits the modes available, e.g. for milestone build
	do
	{
		if (bIncrement)
		{
			++s_iCurrentGameMode;
			if (s_iCurrentGameMode == NUM_AVP_DFE_GAMEMODES)
			{
				s_iCurrentGameMode = 0;
			}
		}
		else
		{
			--s_iCurrentGameMode;
			if (static_cast<int>(s_iCurrentGameMode) < 0)
			{
				s_iCurrentGameMode = NUM_AVP_DFE_GAMEMODES - 1;
			}
		}
	}
	while (!s_abGameModesAvailable[s_iCurrentGameMode]);

	// set number of players to max allowed for game mode.
	s_iNumberOfPlayers = s_aaiMinMaxPlayers[s_iCurrentGameMode][1];

	// Make sure we have a valid level.
	//if (!IsLevelSelectable())
	//{
	//	AlterLevelSelection(true);
	//}

	OnLeaveMode(iPrevGameMode);
	OnEnterMode(s_iCurrentGameMode);
}

void AVP_DevelopmentFrontEnd::AlterGameType(bool bIncrement)
{
	if (bIncrement)
	{
		s_eCurrentGameType = static_cast<AVP_Settings::GameType>(static_cast<int>(s_eCurrentGameType) + 1);
		if (s_eCurrentGameType == AVP_Settings::AVP_NUM_GAMETYPES)
		{
			s_eCurrentGameType = static_cast<AVP_Settings::GameType>(0);
		}
	}
	else
	{
		if (static_cast<int>(s_eCurrentGameType) == 0)
		{
			s_eCurrentGameType = AVP_Settings::AVP_NUM_GAMETYPES;
		}
		s_eCurrentGameType = static_cast<AVP_Settings::GameType>(static_cast<int>(s_eCurrentGameType) - 1);
	}

	if (!s_abGameTypeAvailable[s_eCurrentGameType])
	{
		AlterGameType(bIncrement);
	}

	// Reload the default game rules for the selected game mode
	AVP_Settings::SetGameType( s_eCurrentGameType ); 
#if defined(AVP_ALLOW_MULTIPLAYER)
	AVP_Multiplayer_GameRules::Refresh(); 
#endif	//defined(AVP_ALLOW_MULTIPLAYER)

	// Make sure we have a valid level.
//	if (!IsLevelSelectable())
//	{
//		AlterLevelSelection(true);
//	}
}
	
void AVP_DevelopmentFrontEnd::AlterPlayerSpecies(bool bIncrement)
{
	if (bIncrement)
	{
		s_eCurrentSpecies = static_cast<AVP_Settings::Species>(static_cast<int>(s_eCurrentSpecies) + 1);
		if (s_eCurrentSpecies == AVP_Settings::AVP_NUM_SPECIES)
		{
			s_eCurrentSpecies = static_cast<AVP_Settings::Species>(0);
		}
	}
	else
	{
		if (static_cast<int>(s_eCurrentSpecies) == 0)
		{
			s_eCurrentSpecies = static_cast<AVP_Settings::Species>(AVP_Settings::AVP_NUM_SPECIES);
		}
		s_eCurrentSpecies = static_cast<AVP_Settings::Species>(static_cast<int>(s_eCurrentSpecies) - 1);
	}
}


void AVP_DevelopmentFrontEnd::AlterChosenWildcardLevel(bool bIncrement)
{
	if(s_aiNumFilesWithWildcards[s_iCurrentWildcardChosen] != 0 )
	{
		if (bIncrement)
		{
			s_aiCurrentFileWithWildcard[s_iCurrentWildcardChosen] = (s_aiCurrentFileWithWildcard[s_iCurrentWildcardChosen] + 1) % s_aiNumFilesWithWildcards[s_iCurrentWildcardChosen];	
		}	
		else
		{
			if(s_aiCurrentFileWithWildcard[s_iCurrentWildcardChosen] ==0)
			{
				s_aiCurrentFileWithWildcard[s_iCurrentWildcardChosen] = s_aiNumFilesWithWildcards[s_iCurrentWildcardChosen]-1;
			}
			else
			{
				s_aiCurrentFileWithWildcard[s_iCurrentWildcardChosen] = (s_aiCurrentFileWithWildcard[s_iCurrentWildcardChosen] - 1) % s_aiNumFilesWithWildcards[s_iCurrentWildcardChosen];
			}
		}
	}
}

void AVP_DevelopmentFrontEnd::AlterNumPlayers(bool bIncrement)
{
	int iMinNumPlayers = s_aaiMinMaxPlayers[s_iCurrentGameMode][0];
	int iMaxNumPlayers = s_aaiMinMaxPlayers[s_iCurrentGameMode][1];
	if (bIncrement)
	{
		s_iNumberOfPlayers++;
		if (s_iNumberOfPlayers > iMaxNumPlayers)
		{
			s_iNumberOfPlayers = iMinNumPlayers;
		}
	}
	else
	{
		s_iNumberOfPlayers--;
		if (s_iNumberOfPlayers < iMinNumPlayers)
		{
			s_iNumberOfPlayers = iMaxNumPlayers;
		}
	}

}


void AVP_DevelopmentFrontEnd::AlterServerToJoin(bool bIncrement)
{
#ifdef ASURA_SUPPORT_EXTERNAL_NETWORK
	int iMin = -1;
	int iMax = Asura_Network_Session_System::GetNumberOfSessions() - 1;
	if (bIncrement)
	{
		s_iNetworkSessionIndex++;
		if (s_iNetworkSessionIndex > iMax)
		{
			s_iNetworkSessionIndex = iMin;
		}
	}
	else
	{
		s_iNetworkSessionIndex--;
		if (s_iNetworkSessionIndex < iMin)
		{
			s_iNetworkSessionIndex = iMax;
		}
	}
#endif //#ifdef ASURA_SUPPORT_EXTERNAL_NETWORK
}


void AVP_DevelopmentFrontEnd::AlterFMVToPlay(bool bIncrement)
{
	if (bIncrement)
	{
		s_iFMVIndex++;
	}
	else if (s_iFMVIndex > 0)
	{
		s_iFMVIndex--;
	}
}

void AVP_DevelopmentFrontEnd::AlterFMVAudioTrack(bool bIncrement)
{
	if (bIncrement)
	{
		s_iFMVAudioTrack++;
	}
	else if (s_iFMVAudioTrack > 0)
	{
		s_iFMVAudioTrack--;
	}
}

#ifdef AVP_LOG_FILE_LOAD_MEMORY_USE
void AVP_DevelopmentFrontEnd::AlterChosenSmokeTestOption(bool bIncrement)
{
	if (bIncrement)
	{
		s_iCurrentSmokeTestOptionChosen = (s_iCurrentSmokeTestOptionChosen + 1) % iAVP_DFE_NUM_SMOKETESTFLAGS;
	}
	else
	{
		if( s_iCurrentSmokeTestOptionChosen == 0 )
		{
			s_iCurrentSmokeTestOptionChosen = iAVP_DFE_NUM_SMOKETESTFLAGS-1;
		}
		else 
		{
			s_iCurrentSmokeTestOptionChosen = (s_iCurrentSmokeTestOptionChosen -1 ) % iAVP_DFE_NUM_SMOKETESTFLAGS;
		}
	}
}
void AVP_DevelopmentFrontEnd::AlterNumberOfFramerates(bool bIncrement)
{
	if (bIncrement)
	{
		s_iNumberOfFrameRates += 20;
	}
	else
	{
		s_iNumberOfFrameRates -= 20;
		if( s_iNumberOfFrameRates < 0 )
		{
			s_iNumberOfFrameRates = 0;
		}
	}
}
#endif // #ifdef AVP_LOG_FILE_LOAD_MEMORY_USE

bool AVP_DevelopmentFrontEnd::IsOptionSelectable(AVP_DFE_OPTIONS eOption)
{
	switch (s_iCurrentGameMode)
	{
#if !defined(AVP_MULTIPLAYER_ONLY_DEVFRONTEND)
		case AVP_DFE_GAMEMODE_LOAD_SINGLE_ZONE:
#ifdef ASURA_USE_STREAMING
		case AVP_DFE_GAMEMODE_LOAD_STREAMED_ZONE:
#endif // #ifdef ASURA_USE_STREAMING
		{
			switch(eOption)
			{
				case AVP_DFE_OPTION_GAME_TYPE:
				case AVP_DFE_OPTION_NUM_PLAYERS:
				case AVP_DFE_OPTION_SERVER_TO_JOIN:
				case AVP_DFE_OPTION_FMV_TO_PLAY:
				case AVP_DFE_OPTION_FMV_AUDIO_TRACK:
				case AVP_DFE_OPTION_SMOKETEST_OPTION:
				case AVP_DFE_OPTION_FRAMERATES_OPTION:
				{
					return(false);
				}

				default:
				{
					return(true);
				}
			}		// End of switch.
		}

#ifdef ASURA_USE_STREAMING
		case AVP_DFE_GAMEMODE_LOAD_STREAMED_MISSION:
		{
			switch(eOption)
			{
				case AVP_DFE_OPTION_GAME_TYPE:
				case AVP_DFE_OPTION_NUM_PLAYERS:
				case AVP_DFE_OPTION_SERVER_TO_JOIN:
				case AVP_DFE_OPTION_FMV_TO_PLAY:
				case AVP_DFE_OPTION_FMV_AUDIO_TRACK:
				case AVP_DFE_OPTION_SMOKETEST_OPTION:
				case AVP_DFE_OPTION_FRAMERATES_OPTION:
				{
					return(false);
				}

				default:
				{
					return(true);
				}
			}		// End of switch.
		}
#endif // #ifdef ASURA_USE_STREAMING
#endif	// !defined(AVP_MULTIPLAYER_ONLY_DEVFRONTEND)

#ifdef ASURA_SUPPORT_EXTERNAL_NETWORK
		case AVP_DFE_GAMEMODE_HOST_NETWORK_GAME:
		{
			switch(eOption)
			{
				case AVP_DFE_OPTION_SERVER_TO_JOIN:
#if !defined(AVP_MULTIPLAYER_ONLY_DEVFRONTEND)
				case AVP_DFE_OPTION_FMV_TO_PLAY:
				case AVP_DFE_OPTION_FMV_AUDIO_TRACK:
#endif	// !defined(AVP_MULTIPLAYER_ONLY_DEVFRONTEND)
				case AVP_DFE_OPTION_SMOKETEST_OPTION:
				case AVP_DFE_OPTION_FRAMERATES_OPTION:
				{
					return(false);
				}

				default:
				{
					return(true);
				}
			}		// End of switch.
		}

		case AVP_DFE_GAMEMODE_JOIN_NETWORK_GAME:
		{
			switch(eOption)
			{
				case AVP_DFE_OPTION_GAME_TYPE:
				case AVP_DFE_OPTION_NUM_PLAYERS:
				case AVP_DFE_OPTION_LEVEL_SELECTION:
#if !defined(AVP_MULTIPLAYER_ONLY_DEVFRONTEND)
				case AVP_DFE_OPTION_SEARCH_WITH_WILDCARD:
				case AVP_DFE_OPTION_FMV_TO_PLAY:
				case AVP_DFE_OPTION_FMV_AUDIO_TRACK:
#endif	// !defined(AVP_MULTIPLAYER_ONLY_DEVFRONTEND)
				case AVP_DFE_OPTION_SMOKETEST_OPTION:
				case AVP_DFE_OPTION_FRAMERATES_OPTION:
				{
					return(false);
				}

				case AVP_DFE_OPTION_ACCEPT:
				{
					return s_iNetworkSessionIndex >= 0 && s_iNetworkSessionIndex < static_cast<int>
						(Asura_Network_Session_System::GetNumberOfSessions());
				}

				default:
				{
					return(true);
				}
			}		// End of switch.
		}

#if defined(AVP_USE_DEV_LOBBY)
		case AVP_DFE_GAMEMODE_DEV_LOBBY:
		{
			switch(eOption)
			{
				case AVP_DFE_OPTION_ACCEPT:
					return s_eMenuMode != AVP_DFE_MENUMODE_STARTING_NETWORK;

				case AVP_DFE_OPTION_GAME_MODE:
#if defined(AVP_DFE_ALLOW_QUIT)
				case AVP_DFE_OPTION_QUIT:
#endif	// defined(AVP_DFE_ALLOW_QUIT)
				{
					return(true);
				}

				default:
				{
					return(false);
				}
			}
		}
#endif	//defined(AVP_USE_DEV_LOBBY)

#if defined(AVP_USE_GAME_LOBBY)
		case AVP_DFE_GAMEMODE_GAME_LOBBY:
		{
			switch(eOption)
			{
				case AVP_DFE_OPTION_ACCEPT:
					return s_eMenuMode != AVP_DFE_MENUMODE_STARTING_NETWORK;

				case AVP_DFE_OPTION_GAME_MODE:
#if defined(AVP_DFE_ALLOW_QUIT)
				case AVP_DFE_OPTION_QUIT:
#endif	// defined(AVP_DFE_ALLOW_QUIT)
				{
					return(true);
				}

				default:
				{
					return(false);
				}
			}
		}
#endif	//defined(AVP_USE_GAME_LOBBY)

#endif //#ifdef ASURA_SUPPORT_EXTERNAL_NETWORK

#if !defined(AVP_MULTIPLAYER_ONLY_DEVFRONTEND)
		case AVP_DFE_GAMEMODE_PREVIEW_FMV:
		{
			switch(eOption)
			{
				case AVP_DFE_OPTION_ACCEPT:
				case AVP_DFE_OPTION_FMV_TO_PLAY:
				case AVP_DFE_OPTION_FMV_AUDIO_TRACK:
				case AVP_DFE_OPTION_GAME_MODE:
				{
					return(true);
				}

				default:
				{
					return(false);
				}
			}		// End of switch.
		}
#endif	// !defined(AVP_MULTIPLAYER_ONLY_DEVFRONTEND)

#if defined AVP_USE_GAME_FRONTEND
		case AVP_DFE_GAMEMODE_GAME_FRONTEND:
		{
			switch(eOption)
			{
				case AVP_DFE_OPTION_GAME_MODE:
				case AVP_DFE_OPTION_ACCEPT:
#if defined(AVP_DFE_ALLOW_QUIT)
				case AVP_DFE_OPTION_QUIT:
#endif	// defined(AVP_DFE_ALLOW_QUIT)
				{
					return(true);
				}

				default:
				{
					return(false);
				}
			}		// End of switch.
		}
#endif // defined(AVP_USE_GAME_FRONTEND)

#if defined AVP_LOG_FILE_LOAD_MEMORY_USE
		case AVP_DFE_GAMEMODE_SMOKE_TEST:
		{
			switch(eOption)
			{
				case AVP_DFE_OPTION_GAME_MODE:
				case AVP_DFE_OPTION_CHAR_TYPE:
				case AVP_DFE_OPTION_LEVEL_SELECTION:
				case AVP_DFE_OPTION_SEARCH_WITH_WILDCARD:
				case AVP_DFE_OPTION_SMOKETEST_OPTION:
				case AVP_DFE_OPTION_FRAMERATES_OPTION:
				case AVP_DFE_OPTION_ACCEPT:
#if defined(AVP_DFE_ALLOW_QUIT)
				case AVP_DFE_OPTION_QUIT:
#endif	// defined(AVP_DFE_ALLOW_QUIT)
				{
					return(true);
				}

				default:
				{
					return(false);
				}
			}		// End of switch.
		}
#endif // #ifdef AVP_LOG_FILE_LOAD_MEMORY_USE

		default:
		{
			Asura_Assert(false, "Unhandled case");
			return(true);
		}
	}		// End of switch.
}

bool AVP_DevelopmentFrontEnd::IsLevelSelectable()
{
	// All levels and missions available
	return true;
}

bool AVP_DevelopmentFrontEnd::IsNetwork()
{
#if defined(AVP_MULTIPLAYER_ONLY_DEVFRONTEND)
	switch (s_iCurrentGameMode)
	{

#if defined AVP_USE_GAME_FRONTEND
		case AVP_DFE_GAMEMODE_GAME_FRONTEND:
		{
			return(false);
		}
#endif	// defined AVP_USE_GAME_FRONTEND

		default:
		{
			return(true);
		}
	}
#else
	switch (s_iCurrentGameMode)
	{
		case AVP_DFE_GAMEMODE_LOAD_SINGLE_ZONE:

#ifdef ASURA_USE_STREAMING
		case AVP_DFE_GAMEMODE_LOAD_STREAMED_ZONE:
		case AVP_DFE_GAMEMODE_LOAD_STREAMED_MISSION:
#endif
		case AVP_DFE_GAMEMODE_PREVIEW_FMV:

#if defined AVP_USE_GAME_FRONTEND
		case AVP_DFE_GAMEMODE_GAME_FRONTEND:
#endif	// defined(AVP_USE_GAME_FRONTEND)

#ifdef AVP_LOG_FILE_LOAD_MEMORY_USE
		case AVP_DFE_GAMEMODE_SMOKE_TEST:
#endif // #ifdef AVP_LOG_FILE_LOAD_MEMORY_USE
		{
			return false;
		}

		default:
			return true;
	}
#endif	// defined(AVP_MULTIPLAYER_ONLY_DEVFRONTEND)
}

bool AVP_DevelopmentFrontEnd::IsServer()
{
	switch (s_iCurrentGameMode)
	{
#ifdef ASURA_SUPPORT_EXTERNAL_NETWORK
	case AVP_DFE_GAMEMODE_JOIN_NETWORK_GAME:		//Fallthrough.
		return false;
#endif // #ifdef ASURA_SUPPORT_EXTERNAL_NETWORK
	default:
		return true;
	}
}

#ifdef AVP_LOG_FILE_LOAD_MEMORY_USE
bool AVP_DevelopmentFrontEnd::MoreLevelsForTheSmokeTest()
{
	if( s_iCurrentWildcardChosen == 9 && s_iCurrentSmokeTestOptionChosen != 2 )
	{
		s_iCurrentWildcardChosen = 10;
		s_eCurrentSpecies = AVP_Settings::AVP_SPECIES_HUMAN;
		return true;
	}
	else if( s_iCurrentWildcardChosen == 10 && s_iCurrentSmokeTestOptionChosen != 3  )
	{
		s_iCurrentWildcardChosen = 11;
		s_eCurrentSpecies = AVP_Settings::AVP_SPECIES_PREDATOR;
		return true;
	}
	else if( s_iCurrentWildcardChosen == 11 && s_iCurrentSmokeTestOptionChosen != 1 && s_iCurrentSmokeTestOptionChosen != 4 )
	{
		s_iCurrentWildcardChosen = 4;
		s_eCurrentSpecies = AVP_Settings::AVP_SPECIES_ALIEN;
		return true;
	}
	else if(  s_iCurrentWildcardChosen == 4 )
	{
		if( s_eCurrentSpecies == AVP_Settings::AVP_SPECIES_ALIEN )
		{
			s_eCurrentSpecies = AVP_Settings::AVP_SPECIES_HUMAN;
			return true;
		}
		else if( s_eCurrentSpecies == AVP_Settings::AVP_SPECIES_HUMAN )
		{
			s_eCurrentSpecies = AVP_Settings::AVP_SPECIES_PREDATOR;
			return true;
		}
		else if( s_eCurrentSpecies == AVP_Settings::AVP_SPECIES_PREDATOR && s_iCurrentSmokeTestOptionChosen != 5 )
		{
			s_eCurrentSpecies = AVP_Settings::AVP_SPECIES_ALIEN;
			s_iCurrentWildcardChosen = 8;
			return true;
		}
	}
	else if( s_iCurrentWildcardChosen == 8 )
	{
		if( s_eCurrentSpecies == AVP_Settings::AVP_SPECIES_ALIEN )
		{
			s_eCurrentSpecies = AVP_Settings::AVP_SPECIES_HUMAN;
			return true;
		}
		else if( s_eCurrentSpecies == AVP_Settings::AVP_SPECIES_HUMAN )
		{
			s_eCurrentSpecies = AVP_Settings::AVP_SPECIES_PREDATOR;
			return true;
		}
	}
	return false;
}

void AVP_DevelopmentFrontEnd::PrepareForSmokeTest()
{
	// Selection the start criteria depending of wich option we have chosen from the list
	switch( s_iCurrentSmokeTestOptionChosen )
	{
		// Complete smoke test or
		case 0:
		// All singleplayer maps or
		case 1:
		// Alien SP, all these start with the same settings
		case 2:
			{
				s_iCurrentWildcardChosen = 9;
				s_eCurrentSpecies = AVP_Settings::AVP_SPECIES_ALIEN;
				break;
			}
		// Marine SP
		case 3:
			{
				s_iCurrentWildcardChosen = 10;
				s_eCurrentSpecies = AVP_Settings::AVP_SPECIES_HUMAN;
				break;
			}
		// Predator SP
		case 4:
			{
				s_iCurrentWildcardChosen = 11;
				s_eCurrentSpecies = AVP_Settings::AVP_SPECIES_PREDATOR;
				break;
			}
		// MP Levels
		case 5:
			{
				s_iCurrentWildcardChosen = 4;
				s_eCurrentSpecies = AVP_Settings::AVP_SPECIES_ALIEN;
				break;
			}
		// Test Levels
		case 6:
			{
				s_iCurrentWildcardChosen = 8;
				s_eCurrentSpecies = AVP_Settings::AVP_SPECIES_ALIEN;
				break;
			}
		// The level we have market in the DevFrontEnd
		case 7:
			{
				s_bTestOnlyOneLevel = true;
				break;
			}
		default:
			break;
	}

	if( !s_bTestOnlyOneLevel )
	{
		//Starting with the first map for that wildcard, 0. When we wrap around to 0 again we will change wildcard
		s_aiCurrentFileWithWildcard[s_iCurrentWildcardChosen] = 0;
	}
	
	s_bSmokeTestsFirstLevelLoaded = false;
	
	//Set the number of framerates we are supposed to fetch in the Smoke Test
	AVP_SmokeTest::SetNumberOfFrameratesToGet( s_iNumberOfFrameRates );
}			
#endif // #ifdef AVP_LOG_FILE_LOAD_MEMORY_USE

DEBUGVAR u_int dbg_uSelectedColour = 0xffffffff;
DEBUGVAR u_int dbg_uUnselectedColour = 0xffB0B0B0;
DEBUGVAR u_int dbg_uUnavailableColour = 0xff505050;
u_int AVP_DevelopmentFrontEnd::GetOptionColour(AVP_DFE_OPTIONS eOption)
{
	if (s_eCurrentlyHighlitOption == eOption)
	{
		return dbg_uSelectedColour;
	}
	else if (IsOptionSelectable(eOption))
	{
		return dbg_uUnselectedColour;
	}
	return dbg_uUnavailableColour; 
}


void AVP_DevelopmentFrontEnd::OnLeaveMode(int iMode)
{
	switch (iMode)
	{
#ifdef ASURA_SUPPORT_EXTERNAL_NETWORK
		case AVP_DFE_GAMEMODE_JOIN_NETWORK_GAME:
		{
			Asura_Network_Session_System::Shutdown();
		}
		break;
#endif //#ifdef ASURA_SUPPORT_EXTERNAL_NETWORK

#if !defined(AVP_MULTIPLAYER_ONLY_DEVFRONTEND)
		case AVP_DFE_GAMEMODE_PREVIEW_FMV:
		{
			Platform_ClearFMVList();
			break;
		}
#endif	// !defined(AVP_MULTIPLAYER_ONLY_DEVFRONTEND)

		default:
		{
			// do nothing
		}
		break;
	}
}

void AVP_DevelopmentFrontEnd::OnEnterMode(int iMode)
{
	//Start music
	//Asura_Music_System::Play(AVP_MUSIC_TRACK_MENU1, 1.0f, true); 
	switch (iMode)
	{
#ifdef ASURA_SUPPORT_EXTERNAL_NETWORK
		case AVP_DFE_GAMEMODE_JOIN_NETWORK_GAME:
		case AVP_DFE_GAMEMODE_HOST_NETWORK_GAME:

#if defined(AVP_USE_DEV_LOBBY)
		case AVP_DFE_GAMEMODE_DEV_LOBBY:
#endif	//defined(AVP_USE_DEV_LOBBY)

#if defined(AVP_USE_GAME_LOBBY)
		case AVP_DFE_GAMEMODE_GAME_LOBBY:
#endif	//defined(AVP_USE_GAME_LOBBY)

		{
			if (NeedToStartNetworkMode())
			{
				StartNetworkMode();
				s_eMenuMode = AVP_DFE_MENUMODE_STARTING_NETWORK;
			}
		}
		break;
#endif // #ifdef ASURA_SUPPORT_EXTERNAL_NETWORK

#if !defined(AVP_MULTIPLAYER_ONLY_DEVFRONTEND)
		case AVP_DFE_GAMEMODE_PREVIEW_FMV:
		{
			if (Platform_NeedToBuildFMVList())
			{
				Platform_StartBuildFMVList();
				s_eMenuMode = AVP_DFE_MENUMODE_BUILD_FMV_LIST;
			}
			break;
		}
#endif	// !defined(AVP_MULTIPLAYER_ONLY_DEVFRONTEND)

		default:
		{
			// do nothing
		}
		break;
	}
}

//DEBUGVAR float fDropShadowXOffset = 0.003f;
//DEBUGVAR float fDropShadowYOffset = 0.003f;
//DEBUGVAR u_int uDropShadowColour = 0xff000000;

void AVP_DevelopmentFrontEnd::PrintWithDropShadow(const float fXPos,
												  const float fYPos,
												  const float fSize,
												  const u_int uColour,
												  const u_int uFormat,
												  const Asura_Char* szString)
{
//	Asura_Font_System::Print(fXPos + fDropShadowXOffset, fYPos + fDropShadowYOffset, fSize,
//		uDropShadowColour, uFormat, szString);
	Asura_Font_System::Print(fXPos, fYPos, fSize, uColour, uFormat, szString);
	
}

bool AVP_DevelopmentFrontEnd::MatchWithWildcard(Asura_Char* szFileName, Asura_Char* szWildcard )
{
	Asura_String xFileName( szFileName );
	Asura_String xWildcard( szWildcard );
	int iPosition;
	//cut the first Directory from the path
	xFileName.ReplaceAllChars('\\','/');
	xWildcard.ReplaceAllChars('\\','/');

	#if defined ASURA_ASSERT
		const bool bFoundSlash =
	#endif
		xFileName.Contains("/",&iPosition);

	Asura_Assert(bFoundSlash, "Something wrong with the file name");
	xFileName.Replace(0, iPosition+1, "");
	//return
	return Asura_StringUtil::CompareAgainstWildcardString(xFileName.GetString(), xWildcard.GetString(), true );
}


Asura_Bounding_Rectangle AVP_DevelopmentFrontEnd::CountRectangle(const float fCurrentPosX, 
																	const float fCurrentPosY, 																	
																	const Asura_Char* szString)
{
	float fPosYMin = fCurrentPosY;
	float fPosXMin = fCurrentPosX;
	float fPosXMax = 0.96f;
	//float fPosYMax = dbg_fYPositionSpacing*dbg_fMaxNumOfLines + fCurrentPosY;
	float fPosYMax = dbg_fYPositionSpacing*dbg_fMaxNumOfLines + fCurrentPosY;
	return Asura_Bounding_Rectangle(fPosXMin, fPosYMin, fPosXMax, fPosYMax);
}

void AVP_DevelopmentFrontEnd::PrintWithDropShadowInRectangle(Asura_Bounding_Rectangle& xRect,
																const u_int uColour,
																const float fSize,
																const u_int uFormat,
																const Asura_Char* szString)
{
	//Asura_Font_System::Print(fXPos + fDropShadowXOffset, fYPos + fDropShadowYOffset, fSize,
	//	uDropShadowColour, uFormat, szString);
	//Asura_Font_System::Print(fXPos, fYPos, fSize, uColour, uFormat, szString);
	bool bOK;
	// bool Print(const Asura_Bounding_Rectangle &xRect, const float fXOffset, const float fYOffset, const float fLineSpacing, const float fScale, const unsigned int colour, const unsigned int format, const Asura_Char* string, ...);
	bOK = Asura_Font_System::Print( xRect,  0.0f, 0.0f, 1.0f, fSize, uColour, uFormat, szString );
	Asura_Assert(bOK, "PrintWithDropShadowInRectangle failed ");
}

void AVP_DevelopmentFrontEnd::AlterChosenWildcard(bool bIncrement)
{
	if (bIncrement)
	{
		s_iCurrentWildcardChosen = (s_iCurrentWildcardChosen + 1) % iAVP_DFE_NUM_WILDCARDS;
	}
	else
	{
		if(s_iCurrentWildcardChosen ==0)
		{
			s_iCurrentWildcardChosen = iAVP_DFE_NUM_WILDCARDS-1;
		}
		else 
		{
			s_iCurrentWildcardChosen = (s_iCurrentWildcardChosen -1 ) % iAVP_DFE_NUM_WILDCARDS;
		}
	}
}

#ifdef ASURA_SUPPORT_EXTERNAL_NETWORK
// page displayed while online mode is starting up
bool AVP_DevelopmentFrontEnd::NeedToStartNetworkMode()
{
	if (!Asura_NetLinkManager::IsConnected())
	{
		return true;
	}

	// connected - correct type?
	//TODO allow selection of LAN/online here.
	ASURA_NETLINK_TYPE eType = Asura_NetLinkManager::GetType();
	ASURA_NETLINK_TYPE eRequiredType = ASURA_NETLINK_LAN;
	return eType != eRequiredType;
}

void AVP_DevelopmentFrontEnd::RenderStartingNetworkMode()
{
	// Render black quad over entire screen.
	Asura_Render::Overlay.Colour = 0xff000000;
	Asura_Render::Overlay.TextureIndex = -1;
	Asura_Render::Overlay.TranslucencyMode = ASURA_TRANSLUCENCY_OFF;
	Asura_Render::Overlay.SetRectXY(0.0f, 0.0f, static_cast<float>(Asura_Viewport::GetWidth()), static_cast<float>(Asura_Viewport::GetHeight()));
	Asura_Render::Overlay.Render();
	Asura_Render::FlushDisplay();
	{

		// Setup font and constants.
#ifdef ASURA_TOOLS
		const float fFontSize = Asura_Viewport::GetWidth() * 0.0015f;	
#else
		const float fFontSize = 1.0f;
#endif

		// Menu title
		PrintWithDropShadow(0.5f, 0.18f, fFontSize, 0xffB0B0B0,
			ASURA_TEXTFORMAT_RELATIVEPOSITION | ASURA_TEXTFORMAT_CENTREJUSTIFIED,
			"NETWORK CONNECTION STATUS");

		const char* szStatus = g_aszNetlinkStatus[Asura_NetLinkManager::GetStatus()];
		PrintWithDropShadow(0.5f, 0.28f, fFontSize, 0xffB0B0B0,
			ASURA_TEXTFORMAT_RELATIVEPOSITION | ASURA_TEXTFORMAT_CENTREJUSTIFIED,
			szStatus);

		if (Asura_NetLinkManager::IsWLANSwitchedOff())
		{
			PrintWithDropShadow(0.5f, 0.5f, fFontSize, 0xffB0B0B0,
				ASURA_TEXTFORMAT_RELATIVEPOSITION | ASURA_TEXTFORMAT_CENTREJUSTIFIED,
				"WLAN switch is off!");
		}
	}
	Asura_Render::FlushDisplay();
}

static float g_fWLANTimeout;

void AVP_DevelopmentFrontEnd::StartNetworkMode()
{
	g_fWLANTimeout = 5.0f;
	ShutdownNetworkMode();
	int iType = ASURA_NETLINK_LAN;
	Asura_NetLinkManager::Connect(iType);
}

// return true if network is ready
bool AVP_DevelopmentFrontEnd::UpdateStartingNetworkMode()
{
	ASURA_NETLINK_STATUS eStatus = Asura_NetLinkManager::GetStatus();
#ifdef ASURA_SUPPORT_GAMESPY
	bool bFinished = (eStatus != ASURA_NETLINK_CONNECTING && eStatus != ASURA_NETLINK_CHECKING_GAMESPY);
#else
	bool bFinished = (eStatus != ASURA_NETLINK_CONNECTING);
#endif
	if (bFinished && Asura_NetLinkManager::IsWLANSwitchedOff())
	{
		// return false until 5 seconds have elapsed to indicate the WLAN switch problem
		g_fWLANTimeout -= Asura_Timers::GetFrameTime();
		return (g_fWLANTimeout <= 0.0f);
	}
	return bFinished;
}

void AVP_DevelopmentFrontEnd::ShutdownNetworkMode()
{
	if (Asura_NetLinkManager::IsConnected())
	{
		Asura_NetLinkManager::Disconnect(false);
		Asura_NetLinkManager::Update();
		// a bit of a hack - ensure it's shut down.
		while (Asura_NetLinkManager::GetStatus() == ASURA_NETLINK_DISCONNECTING)
		{
			RenderStartingNetworkMode();
			Asura_Thread_System::DelayCurrentThread(0.01f);
			Asura_NetLinkManager::Update();
		}
	}
}
#else
bool AVP_DevelopmentFrontEnd::NeedToStartNetworkMode()
{
	return false;
}

void AVP_DevelopmentFrontEnd::RenderStartingNetworkMode()
{
	// Nothing to do here
}

void AVP_DevelopmentFrontEnd::StartNetworkMode()
{
	// Nothing to do here
}

// return true if network is ready
bool AVP_DevelopmentFrontEnd::UpdateStartingNetworkMode()
{
	return false;
}

void AVP_DevelopmentFrontEnd::ShutdownNetworkMode()
{
	// Nothing to do here
}
#endif // #ifdef ASURA_SUPPORT_EXTERNAL_NETWORK

#endif // defined AVP_USE_DEVELOPMENT_FRONT_END
