////////////////////////////////////////////////////////////////////////////// 
//
// AVP_GameFrontEnd.cpp
// Author : Gordon Hayes
// Date   : 02-May-2007
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

#if defined(AVP_USE_GAME_FRONTEND)

#include "FrontEnd/AVP_GameFrontEnd.h"

// Asura headers
#include "Entity/Asura_Entity_System.h"
#include "Interface/Asura_CommandConsole.h"
#include "Sound/Asura_Audio_System.h"
#include "Sound/Asura_Audio_Voice.h"

// Project headers
#include "Core/AVP_Core.h"
#include "Core/AVP_Game_Core.h"
#include "Core/AVP_Settings.h"
#include "GUIMenu/AVP_GUIMenu_System.h"

// Not sure if this is going back in after E3 so I will leave it here for now - FS 07 May 2009
//static const Asura_Hash_ID	g_uTitleScreen		= 0xa238771c;	// Asura_GetHashID("Start_Menu");

static const Asura_Hash_ID	g_uSpeciesScreen	= 0xa238771c;   // Asura_GetHashID("Start_Menu");   
static const Asura_Hash_ID	g_uPredatorScreen	= 0xACD980E5;   // Asura_GetHashID("E3_PRED_MAINMENU"); 
static const Asura_Hash_ID	g_uMarineScreen		= 0xEA605AA4;   // Asura_GetHashID("E3_MARINE_MAINMENU"); 

////////////////////////////////////////////////////////////////////////////////
// Static variables...
////////////////////////////////////////////////////////////////////////////////
bool	     AVP_GameFrontEnd::s_bSetupComplete		 = false;
bool	     AVP_GameFrontEnd::s_bDevLobbyRequested	 = false;
bool	     AVP_GameFrontEnd::s_bGameLobbyRequested = false;
u_int	     AVP_GameFrontEnd::s_uMenuPageHash		 = ASURA_HASH_ID_UNSET;
Asura_Handle AVP_GameFrontEnd::s_iMusicTrackHandle   = ASURA_INVALID_HANDLE;

#define MENU_TIME_OUT					2.0f//5.5f


void AVP_GameFrontEnd::Initialise()
{
	if(!IsInitialised())
	{
		s_bSetupComplete		= false;
		s_bDevLobbyRequested	= false;
		s_bGameLobbyRequested	= false;

		AVP_Settings::Species eSpecies = AVP_Settings::GetPlayerSpecies();

		switch( eSpecies )
		{
			default:
			case AVP_Settings::AVP_SPECIES_ALIEN:
			case AVP_Settings::AVP_SPECIES_UNKNOWN:
			{
				s_uMenuPageHash		= g_uSpeciesScreen;
				break;
			}
			case AVP_Settings::AVP_SPECIES_HUMAN:
			{
				s_uMenuPageHash		= g_uMarineScreen;
				break;
			}
			case AVP_Settings::AVP_SPECIES_PREDATOR:
			{
				s_uMenuPageHash		= g_uPredatorScreen;
				break;
			}
		}

		AVP_GUIMenu_System::Activate( s_uMenuPageHash, false, true );

		PlayMusic();
	}
}

void AVP_GameFrontEnd::PlayMusic()
{
	// AJK, E3 play music
	s_iMusicTrackHandle = Asura_Audio_System::CreateStreamingVoice( 0xECC7C79A, true, true ); // "Sounds\Music\E3_Menu_Background..wav"
	if( s_iMusicTrackHandle != ASURA_INVALID_HANDLE )
	{
		Asura_Audio_Voice* const pxVoice = Asura_Audio_System::GetVoice( s_iMusicTrackHandle );
		if ( pxVoice ) pxVoice->Start();
	}
}

void AVP_GameFrontEnd::StopMusic()
{
	// AJK, E3 stop music
	if ( s_iMusicTrackHandle != ASURA_INVALID_HANDLE )
	{
		Asura_Audio_System::DestroyVoice( s_iMusicTrackHandle );
	}
}

void AVP_GameFrontEnd::Shutdown()
{
	if(IsInitialised())
	{
		AVP_GUIMenu_System::Deactivate();

		s_uMenuPageHash	= ASURA_HASH_ID_UNSET;

		StopMusic();
	}
}

void AVP_GameFrontEnd::SetupConsoleCommands()
{
	// Add the necessary console commands...
	Asura_CommandConsole::AddCmd( L"AVP.FrontEnd.CMD.PlayMusic", AVP_GameFrontEnd::ConsoleCmd_PlayMusic, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
}

bool AVP_GameFrontEnd::ConsoleCmd_PlayMusic(u_int uPlay, bool bInterrogate, bool bHelp )
{
	if( bHelp || bInterrogate)
	{
		Asura_CommandConsole::Printf( L"Plays or stops the frontend sound/music" );
		return(true);
	}
	else
	{
		if( uPlay == 1 )
		{
			PlayMusic();
		}
		else if ( uPlay == 0 )
		{
			StopMusic();
		}
		else
		{
			Asura_Assert(false, "invalid argument in PlauMusic command!!!");
			return false ;
		}
		return true;
	}
}

#endif //defined(AVP_USE_GAME_FRONTEND)

/// End of file. ///
