//////////////////////////////////////////////////////////////////////////////
//
// AVP_GameLobby.h
//
// Description: similar to the dev front end, providing a way to knock up some
//              lobby and matchmaking functionality quickly during development.
//              Made it a separate system for clarity and maintainability.
// 
// Copyright (c) 2008 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef AVP_GAMELOBBY_H___
#define AVP_GAMELOBBY_H___ 1

#if defined( AVP_USE_GAME_LOBBY ) || defined( AVP_USE_DEV_LOBBY )

#include "Core/Asura_LinkList.h"

// E N U M S /////////////////////////////////////////////////////////////////

enum AVP_LOBBY_ATTRIBUTE
{
	AVP_LOBBY_ATTRIBUTE_MAP,
	AVP_LOBBY_ATTRIBUTE_GAMETYPE,
	AVP_LOBBY_ATTRIBUTE_GAME_VERSION,
	AVP_LOBBY_ATTRIBUTE_SKILL,
	AVP_LOBBY_ATTRIBUTE_ADVANCED_SETTINGS_A,
	AVP_LOBBY_ATTRIBUTE_ADVANCED_SETTINGS_B,

	AVP_NUM_LOBBY_ATTRIBUTES
};


#if defined(AVP_USE_GAME_LOBBY)

// C L A S S   D E C L A R A T I O N S ///////////////////////////////////////
class AVP_GameLobby_GameSettings
{
public:

	AVP_GameLobby_GameSettings();

	// public data
	u_int m_uMaxPlayers;
	int m_iMapIndex;
	int m_iGametypeIndex;
	int m_iLocalSpecies;
};

class AVP_GameLobby_MapData
{
public:
	AVP_GameLobby_MapData(const char* szFilename);

	const char* GetFilename() const { return m_acFilename; }
	const char* GetDisplayName() const { return m_acDisplayName; } // TODO - this won't be good enough for the final game.
	Asura_Hash_ID GetHash() const { return m_uHash; }

private:
	char m_acFilename[256];
	char m_acDisplayName[32]; // TODO - this won't be good enough for the final game.
	Asura_Hash_ID m_uHash;
};

class AVP_GameLobby
{
public:

	static void Initialise();
	static void AddConsoleCmdsAndVars();
	static void Shutdown();

	// Update returns true when we're finished. Then you just check whether
	// quit was requested, and if not assume that the game settings are all
	// made and you proceed into the game.
	static bool Update(); 
	static void Render();

	static bool IsQuitRequested() { return s_bQuitRequested; }

	static int GetMapIndexByHash(Asura_Hash_ID uHash);
	static Asura_Hash_ID GetMapHashByIndex(int iIndex);
	static const char* GetMapFilename(int iIndex);

private:

	static void Platform_Initialise();
	static void Platform_Shutdown();

	static void UpdateLobbySettings();
	static void InitialiseGameSettings();
	static void UpdateTeamNames();

	static void SetLobbyVoiceChannelsSettings();

	static void CreateMapList();
	static void CreateMapList_Callback(const char* szFilename);
	static void ClearMapList();

	// console commands for GUIMenu interface
	static bool RequestQuit( bool bHelp ) { if (!bHelp) { s_bQuitRequested = true; } return true; }
	static bool CreateGame( bool bHelp );
	static bool RotateMap( bool bHelp ); 
	static bool SearchForGame( bool bHelp );
	static bool StartGame( bool bHelp );
	static bool ChangeSpecies( bool bHelp );
	static bool ChangeTeam( bool bHelp );
	static bool GetLocalPlayerReady( bool bHelp );
	static bool AllowChangingTeam( bool bHelp );
	static bool AllowChangingSpecies( bool bHelp );
	static bool IsRoleBasedGame( bool bHelp );
	static bool PopulateMapSelectorName(u_int uElementIndex, bool bInterrogate, bool bHelp); // For menu items where the user can choose the map.
	static bool PopulateMapSearchName(u_int uElementIndex, bool bInterrogate, bool bHelp); // For menu items where the user can choose the map.
	static bool PopulateMapDisplayName(u_int uElementIndex, bool bInterrogate, bool bHelp); // For menu items where the user is told what map it is.

	static bool PopulateMapName(u_int uElementIndex, bool bIncludeAny, bool bIncludeUnknown);

	// internal checkings
	static bool AllowChangingTeam();
	static bool AllowChangingSpecies();

	static bool s_bSetupComplete;
	static bool s_bQuitRequested;

	static AVP_GameLobby_GameSettings s_xLobbySettings;
	static int s_aiLobbySearchFilters[AVP_NUM_LOBBY_ATTRIBUTES];

	// hack to restrict available gametypes for milestone builds
	static void CheckGametypeIsAllowed();
	static bool IsGametypeAllowed(int iGametype);
	static bool s_bGametypeChanged;
	static int s_iPrevGametype;

	static Asura_LinkList<AVP_GameLobby_MapData*> s_xMapList;
};

#endif // #if defined(AVP_USE_GAME_LOBBY)

#endif // #if defined( AVP_USE_GAME_LOBBY ) || defined( AVP_USE_DEV_LOBBY )

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

#endif // #ifndef AVP_GAMELOBBY_H___
