//////////////////////////////////////////////////////////////////////////////
//
// File: AVP_GUIMenu_AudioDiaries.cpp
//
// (AJK - 12.03.2009) Created
//
// Copyright (C) 2009 Rebellion, All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header.
#include "GUIMenu/AVP_GUIMenu_AudioDiaries.h"

// Asura
#include "Blueprint/Asura_Blueprint.h"
#include "Blueprint/Asura_Blueprint_System.h"
#include "Core/Asura_HashedLocalisedText.h"
#include "Interface/Asura_CommandConsole.h"
#include "Sound/Asura_Dialogue_System.h"

// AVP
#include "Blueprint/AVP_Blueprint_HashIDs.h"
#include "Core/AVP_Game_Core.h"
#include "Entity/PhysicalObject/Actor/AVP_ClientEntity_Actor.h"
#include "Entity/PlayerInfo/AVP_ClientEntity_PlayerInfo.h"
#include "Entity/PlayerInfo/AVP_Player_AudioDiary.h"

#if defined(ASURA_USE_DIALOGUE_SYSTEM)
Asura_Handle	AVP_GUIMenu_AudioDiaries::s_iDialogueHandle = ASURA_INVALID_HANDLE;
#endif	//defined(ASURA_USE_DIALOGUE_SYSTEM)

void AVP_GUIMenu_AudioDiaries::SetupConsoleCommands()
{
#if defined(ASURA_SUPPORT_COMMAND_CONSOLE)
	// Add the necessary console commands...
	Asura_CommandConsole::AddCmd( L"AVP.CMD.AudioDiariesList", AVP_GUIMenu_AudioDiaries::ConsoleCmd_AudioDiariesList, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::AddCmd( L"AVP.CMD.AudioDiariesDate", AVP_GUIMenu_AudioDiaries::ConsoleCmd_AudioDiariesDate, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::AddCmd( L"AVP.CMD.AudioDiariesPlay", AVP_GUIMenu_AudioDiaries::ConsoleCmd_AudioDiariesPlay, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::AddCmd( L"AVP.CMD.AudioDiariesNew" , AVP_GUIMenu_AudioDiaries::ConsoleCmd_AudioDiariesNew,  Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
#endif	// defined(ASURA_SUPPORT_COMMAND_CONSOLE)
}

bool AVP_GUIMenu_AudioDiaries::ConsoleCmd_AudioDiariesList( u_int uElementIndex, bool bInterrogate, bool bHelp )
{
	if( bInterrogate || bHelp )
	{
		return true;
	}

	AVP_ClientEntity_PlayerInfo* pxPlayerInfo = AVP_ClientEntity_PlayerInfo::FindEntity( AVP_Game_Core::GetPlayerGuid() ); 

	if( pxPlayerInfo && ( uElementIndex < pxPlayerInfo->GetAudioDiariesList().Size() ) ) 
	{
		const AVP_Player_AudioDiary* pxDiary = pxPlayerInfo->GetAudioDiary( uElementIndex );
		if ( pxDiary )
		{
			// Get name of this diary from the blueprint
			Asura_Value xValue;
			Asura_Blueprint* pxBlueprint = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_DIALOGUE, pxDiary->GetDialogBlueprintHashID());
			if(pxBlueprint)
			{
				Asura_Hash_ID uSpeakerName = ASURA_HASH_ID_UNSET;
				if(pxBlueprint->GetParameterValueByHashID(uAVP_BLUEPRINT_PARAM_AUDIODIARY_SPEAKERNAME, xValue))
				{
					uSpeakerName = xValue.GetHashIDValue();
				}

				Asura_CommandConsole::Printf( L"%s", Asura_HashedLocalisedText::GetString(Asura_GetHashID("Menu"), uSpeakerName) );
			}
		}
		return true;
	}
	return false;
}

bool AVP_GUIMenu_AudioDiaries::ConsoleCmd_AudioDiariesDate( u_int uElementIndex, bool bInterrogate, bool bHelp )
{
	if( bInterrogate || bHelp )
	{
		return true;
	}

	AVP_ClientEntity_PlayerInfo* pxPlayerInfo = AVP_ClientEntity_PlayerInfo::FindEntity( AVP_Game_Core::GetPlayerGuid() ); 

	if( pxPlayerInfo && ( uElementIndex < pxPlayerInfo->GetAudioDiariesList().Size() ) ) 
	{
		const AVP_Player_AudioDiary* pxDiary = pxPlayerInfo->GetAudioDiary( uElementIndex );
		if ( pxDiary )
		{
			// Get name of this diary from the blueprint
			Asura_Value xValue;
			Asura_Blueprint* pxBlueprint = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_DIALOGUE, pxDiary->GetDialogBlueprintHashID());
			if(pxBlueprint)
			{
				Asura_Hash_ID uDateRecorded        = ASURA_HASH_ID_UNSET;
				if(pxBlueprint->GetParameterValueByHashID(uAVP_BLUEPRINT_PARAM_AUDIODIARY_DATERECORDED, xValue))
				{
					uDateRecorded = xValue.GetHashIDValue();
				}

				Asura_CommandConsole::Printf( L"%s", Asura_HashedLocalisedText::GetString(Asura_GetHashID("Menu"), uDateRecorded) );
			}
		}
		return true;
	}
	return false;
}

bool AVP_GUIMenu_AudioDiaries::ConsoleCmd_AudioDiariesNew( u_int uElementIndex, bool bInterrogate, bool bHelp )
{
	if( bInterrogate || bHelp )
	{
		return true;
	}

	AVP_ClientEntity_PlayerInfo* pxPlayerInfo = AVP_ClientEntity_PlayerInfo::FindEntity( AVP_Game_Core::GetPlayerGuid() ); 

	if( pxPlayerInfo && ( uElementIndex < pxPlayerInfo->GetAudioDiariesList().Size() ) ) 
	{
		const AVP_Player_AudioDiary* pxDiary = pxPlayerInfo->GetAudioDiary( uElementIndex );
		if ( pxDiary && !pxDiary->GetRead())
		{
			Asura_CommandConsole::Printf( L"1" );
		}
		else
		{
			Asura_CommandConsole::Printf( L"0" );
		}
		return true;
	}
	return false;
}

bool AVP_GUIMenu_AudioDiaries::ConsoleCmd_AudioDiariesPlay( u_int uElementIndex, bool bInterrogate, bool bHelp )
{
	if (bHelp || bInterrogate)
	{
		return true;
	}
	AVP_ClientEntity_PlayerInfo* pxPlayerInfo = AVP_ClientEntity_PlayerInfo::FindEntity( AVP_Game_Core::GetPlayerGuid() ); 
	if ( pxPlayerInfo )
	{
		AVP_ClientEntity_Actor* pxPlayer = AVP_ClientEntity_Actor::FindEntity( pxPlayerInfo->GetPuppetGuid() );
		if( pxPlayer && ( uElementIndex < pxPlayerInfo->GetAudioDiariesList().Size() ) ) 
		{
			AVP_Player_AudioDiary* pxDiary = pxPlayerInfo->GetAudioDiary( uElementIndex );
			if ( pxDiary )
			{
				// Set as listened to
				pxDiary->SetRead( true );
				// Get name of this diary from the blueprint
				Asura_Value xValue;
				Asura_Blueprint* pxBlueprint = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_DIALOGUE, pxDiary->GetDialogBlueprintHashID());
				if(pxBlueprint)
				{
					Asura_Hash_ID uDialogueHash = ASURA_HASH_ID_UNSET;
					if(pxBlueprint->GetParameterValueByHashID(uAVP_BLUEPRINT_PARAM_AUDIODIARY_DIALOGUE, xValue))
					{
						uDialogueHash = xValue.GetHashIDValue();
					}

#if defined(ASURA_USE_DIALOGUE_SYSTEM)
					if((s_iDialogueHandle != ASURA_INVALID_HANDLE))
					{
						Asura_Dialogue_System::Stop(s_iDialogueHandle);
					}
					if ( ( s_iDialogueHandle == ASURA_INVALID_HANDLE ) && ( uDialogueHash != ASURA_HASH_ID_UNSET ) )
					{
						const Asura_Dialogue_Event* pxDialogue = Asura_Dialogue_System::FindEvent( uDialogueHash );
						if ( pxDialogue )
						{
							if ( Asura_Dialogue_System::GetNumActiveHandles() == 0 )
							{
								Asura_Dialogue_System::PlayEvent( pxDialogue, pxPlayer->GetCurrentPosition(), ASURA_GUID_UNREGISTERED, &s_iDialogueHandle );
							}
						}
					}
#endif	//defined(ASURA_USE_DIALOGUE_SYSTEM)
				}
			}
		}
	}
	return true;
}

void AVP_GUIMenu_AudioDiaries::StopDialogue()
{
#if defined(ASURA_USE_DIALOGUE_SYSTEM)
	Asura_Dialogue_System::Stop(s_iDialogueHandle);
#endif	//defined(ASURA_USE_DIALOGUE_SYSTEM)
}

void AVP_GUIMenu_AudioDiaries::Shutdown()
{
#if defined(ASURA_USE_DIALOGUE_SYSTEM)
	Asura_Dialogue_System::Stop(s_iDialogueHandle);
#endif	//defined(ASURA_USE_DIALOGUE_SYSTEM)
}

void AVP_GUIMenu_AudioDiaries::Update()
{
#if defined(ASURA_USE_DIALOGUE_SYSTEM)
	if((s_iDialogueHandle != ASURA_INVALID_HANDLE) && !Asura_Dialogue_System::IsPlaying(s_iDialogueHandle))
	{
		Asura_Dialogue_System::Stop(s_iDialogueHandle);
	}
#endif	//defined(ASURA_USE_DIALOGUE_SYSTEM)
}