///////////////////////////////////////////////////////////////////////////////
//
// File      : AVP_Rumble_System.cpp
// Author    : Ali Jakamy
// Created   : 25/03/2009
//
// Copyright (C) 2009 Rebellion. All Rights Reserved
//
///////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header
#include "Input/AVP_Rumble_System.h"

// Asura
#include "Blueprint/Asura_Blueprint_System.h"
#include "Core/Asura_Timers.h"
#include "Core/Memory/Asura_Mem.h"
#include "Input/Asura_Rumble_System.h"

// AVP
#include "Core/AVP_Game_Core.h"
#include "Blueprint/AVP_Blueprint_HashIDs.h"

// Statics defs
Asura_LinkList <AVP_Rumble*> AVP_Rumble_System::s_xRumbleList;
float                        AVP_Rumble_System::s_fUpdateInterval;

Asura_Hash_ID				 AVP_Rumble_System::s_uStep1Motor1 = 0x6A8C4FF7; // "Step1 Motor1"
Asura_Hash_ID				 AVP_Rumble_System::s_uStep2Motor1 = 0xD26D7CD6; // "Step2 Motor1"
Asura_Hash_ID				 AVP_Rumble_System::s_uStep3Motor1 = 0x3A4EA9B5; // "Step3 Motor1"

Asura_Hash_ID				 AVP_Rumble_System::s_uStep1Motor2 = 0x6A8C4FF8; // "Step1 Motor2"
Asura_Hash_ID				 AVP_Rumble_System::s_uStep2Motor2 = 0xD26D7CD7; // "Step2 Motor2"
Asura_Hash_ID				 AVP_Rumble_System::s_uStep3Motor2 = 0x3A4EA9B6; // "Step3 Motor2"

void AVP_Rumble_System::Initialise()
{
	s_fUpdateInterval = 0.0f;

	// Initialise the rumble system
	Asura_Rumble_System::Initialise();
}

void AVP_Rumble_System::AddPlayer(int iController, Asura_Guid uPlayer)
{
	Asura_Rumble_System::AddPlayer(iController, uPlayer);
}

void AVP_Rumble_System::InitFromBlueprint(Asura_Hash_ID uBlueprintHash)
{
	if ( uBlueprintHash != ASURA_HASH_ID_UNSET )
	{
		if ( !GetRumbleWithHash(uBlueprintHash) )
		{
			Asura_Handle iMotor1Rumble, iMotor2Rumble;
			Asura_Hash_ID uPlatformBlueprintHash = GetPlatformBuleprint( uBlueprintHash );
			iMotor1Rumble = SetupRumble ( uPlatformBlueprintHash, uBlueprintHash, AVP_RUMBLE_MOTOR1, s_uStep1Motor1, s_uStep2Motor1, s_uStep3Motor1, uBlueprintHash ); 
			
			// Create AVP rumble
			AVP_Rumble* pxRumble = NULL;
			if ( iMotor1Rumble != ASURA_INVALID_HANDLE )
			{
				pxRumble = CreateRumble( uBlueprintHash );
				if ( pxRumble )
				{
					PlatformSetupRumbleMotor( pxRumble, AVP_RUMBLE_MOTOR1, iMotor1Rumble);
				}
			}
			if ( GetPlatformSupport2Motors())
			{
				if ( !pxRumble )
				{
					pxRumble = CreateRumble( uBlueprintHash );
				}
				iMotor2Rumble = SetupRumble ( uPlatformBlueprintHash, uBlueprintHash, AVP_RUMBLE_MOTOR2, s_uStep1Motor2, s_uStep2Motor2, s_uStep3Motor2, uBlueprintHash );
				if ( iMotor2Rumble != ASURA_INVALID_HANDLE )
				{
					if ( pxRumble )
					{
						PlatformSetupRumbleMotor( pxRumble, AVP_RUMBLE_MOTOR2, iMotor2Rumble);
					}
				}
			}
			SetupRumblePriority( uPlatformBlueprintHash, uBlueprintHash );
		}
	}
}

AVP_Rumble* AVP_Rumble_System::CreateRumble( Asura_Hash_ID uBlueprintHash )
{
	AVP_Rumble* pxRumble = new AVP_Rumble;
	pxRumble->m_uRumbleHash = uBlueprintHash;
	pxRumble->PlatformSetup();
	s_xRumbleList.Push_Back( pxRumble );
	return pxRumble;
}

void AVP_Rumble_System::Reset()
{
	Asura_Rumble_System::Reset();

	// delete rumble list
	FreeRumbleList();
}

void AVP_Rumble_System::FreeRumbleList()
{
	for ( Asura_LinkList_ConstIt< AVP_Rumble* > xRumbleIter( s_xRumbleList ); !xRumbleIter.Done(); xRumbleIter.Next() )
	{
		delete *xRumbleIter.GetCurrent();
	}
	s_xRumbleList.Clear();
}

void AVP_Rumble_System::Shutdown()
{
	FreeRumbleList();
	Asura_Rumble_System::RemovePlayer(0);
	Asura_Rumble_System::Shutdown();

}

void AVP_Rumble_System::Update(bool bForced /*= false*/)
{
	Asura_Rumble_System::Update();

	// Periodic update
	if ( s_fUpdateInterval < 0.0f || bForced )
	{
		s_fUpdateInterval = 0.2f;

		// Clear exclusive play list
		for ( Asura_LinkList_ConstIt< AVP_Rumble* > xRumbleIter( s_xRumbleList ); !xRumbleIter.Done(); xRumbleIter.Next() )
		{
			if ( (*xRumbleIter.GetCurrent())->m_ePriority ==  RUMBLE_PLAY_EXCLUSIVE ) 
			{
				if ( (*xRumbleIter.GetCurrent())->iHandle != ASURA_INVALID_HANDLE )
				{
					if (  !Asura_Rumble_System::IsHUDRumbleActive( (*xRumbleIter.GetCurrent())->iHandle) )
					{
						(*xRumbleIter.GetCurrent())->iHandle = ASURA_INVALID_HANDLE;
					}
				}
			}
		}
	}
	else
	{
		s_fUpdateInterval -= Asura_Timers::GetGameTimeSlice();	
	}
}

void AVP_Rumble_System::Pause()
{
	Asura_Rumble_System::PauseAllRumbling();
}

void AVP_Rumble_System::Unpause()
{
	Asura_Rumble_System::UnpauseAllRumbling();
}

void AVP_Rumble_System::StopAll()
{
	Asura_Rumble_System::StopAllRumbling();
}

void AVP_Rumble_System::PauseRumble(Asura_Handle iHandle)
{
	Asura_Rumble_System::PauseRumble(iHandle);
}

void AVP_Rumble_System::UnpauseRumble(Asura_Handle iHandle)
{
	Asura_Rumble_System::UnpauseRumble(iHandle);
}

void AVP_Rumble_System::StopRumble(Asura_Handle iHandle)
{
	Asura_Rumble_System::StopRumble(iHandle);
}

void AVP_Rumble_System::StopHUDRumble(Asura_Handle iHandle)
{
	Asura_Rumble_System::StopHUDRumble(iHandle);
}

bool AVP_Rumble_System::IsHUDRumbleActive(Asura_Handle iHandle)
{
	return Asura_Rumble_System::IsHUDRumbleActive(iHandle);
}

bool AVP_Rumble_System::FillRumbleStepFromBlueprint(   RUMBLE_BASE_EFFECT& eEffect
	                                                 , RUMBLE_ADJUST&      eAdjust
													 , float&              fMaxMagnitude
	                                                 , float&              fMinMagnitude
											         , float&              fDecay
													 , int&                uNumPlays
													 , Asura_Hash_ID       uPlatformBlueprint )
{
	Asura_Value xValue;
	Asura_Blueprint* pxBlueprint = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_VIBRATIONS, uPlatformBlueprint );
	if ( pxBlueprint )
	{
		if(pxBlueprint->GetParameterValueByHashID(0xAB39378F, xValue)) // Adjust
		{
			eAdjust = static_cast<RUMBLE_ADJUST> (xValue.GetIntValue());
		}
		if(pxBlueprint->GetParameterValueByHashID(0x05B097BA, xValue)) // Decay
		{
			fDecay = xValue.GetFloatValue();
		}
		if(pxBlueprint->GetParameterValueByHashID(0xB226B991, xValue)) // Effect
		{
			eEffect = static_cast<RUMBLE_BASE_EFFECT> (xValue.GetIntValue());
		}
		if(pxBlueprint->GetParameterValueByHashID(0x5445AE54, xValue)) // Max Magnitude
		{
			fMaxMagnitude = xValue.GetFloatValue();
		}
		if(pxBlueprint->GetParameterValueByHashID(0xC0E3DDC2, xValue)) // Min Magnitude
		{
			fMinMagnitude = xValue.GetFloatValue();
		}
		if(pxBlueprint->GetParameterValueByHashID(0xC142A72D, xValue)) // Number of plays
		{
			uNumPlays = xValue.GetIntValue();
		}
		return true;
	}
	return false;
}

void AVP_Rumble_System::CreateRumbleStep(  RUMBLE_BASE_EFFECT eEffect
										 , RUMBLE_ADJUST      eAdjust
										 , float              fMaxMagnitude
										 , float              fMinMagnitude
										 , float              fDecay
										 , int                uNumPlays
										 , Asura_Handle&      iRumble )
{
	if ( iRumble == ASURA_INVALID_HANDLE )
	{
		iRumble = Asura_Rumble::CreateAsuraRumble(eEffect, eAdjust, fMaxMagnitude, fMinMagnitude, fDecay, uNumPlays);
	}
	else
	{
		Asura_Rumble::AddRumbleStep(iRumble, eEffect, eAdjust, fMaxMagnitude, fMinMagnitude, fDecay, uNumPlays);
	}
}

Asura_Handle AVP_Rumble_System::SetupRumble (Asura_Hash_ID uPlatformBlueprintHash, Asura_Hash_ID uBlueprintHash, AVP_Rumble_Motor eMotor, Asura_Hash_ID uStep1, Asura_Hash_ID uStep2, Asura_Hash_ID uStep3, Asura_Hash_ID uBlueprinthash )
{
	Asura_Value xValue;
	Asura_Blueprint* pxBlueprint = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_VIBRATIONS, uPlatformBlueprintHash ); // "PulseRifleFire"
	if ( pxBlueprint )
	{
		if(pxBlueprint->GetParameterValueByHashID(uStep1, xValue))
		{
			if ( xValue.GetHashIDValue() != ASURA_HASH_ID_UNSET)
			{
				RUMBLE_BASE_EFFECT  eEffect;
				RUMBLE_ADJUST       eAdjust;
				float               fMaxMagnitude;
				float               fMinMagnitude;
				float               fDecay;
				int                 uNumPlays;

				if ( FillRumbleStepFromBlueprint(eEffect, eAdjust, fMaxMagnitude, fMinMagnitude, fDecay, uNumPlays, xValue.GetHashIDValue()) )
				{
					Asura_Handle iRumble = ASURA_INVALID_HANDLE;
					CreateRumbleStep(eEffect, eAdjust, fMaxMagnitude, fMinMagnitude, fDecay, uNumPlays, iRumble);

					if(pxBlueprint->GetParameterValueByHashID(uStep2, xValue))
					{
						if ( xValue.GetHashIDValue() != ASURA_HASH_ID_UNSET)
						{
							if ( FillRumbleStepFromBlueprint(eEffect, eAdjust, fMaxMagnitude, fMinMagnitude, fDecay, uNumPlays, xValue.GetHashIDValue()) )
							{
								CreateRumbleStep(eEffect, eAdjust, fMaxMagnitude, fMinMagnitude, fDecay, uNumPlays, iRumble);
								if(pxBlueprint->GetParameterValueByHashID(uStep3, xValue))
								{
									if ( xValue.GetHashIDValue() != ASURA_HASH_ID_UNSET)
									{
										if ( FillRumbleStepFromBlueprint(eEffect, eAdjust, fMaxMagnitude, fMinMagnitude, fDecay, uNumPlays, xValue.GetHashIDValue()) )
										{
											CreateRumbleStep(eEffect, eAdjust, fMaxMagnitude, fMinMagnitude, fDecay, uNumPlays, iRumble);
										}
									}
								}
							}
						}
					}
					return iRumble;
				}
			}
		}
	}
	return ASURA_INVALID_HANDLE;
}

AVP_Rumble* AVP_Rumble_System::GetRumbleWithHash(Asura_Hash_ID uRumbleHash)
{
	for ( Asura_LinkList_ConstIt< AVP_Rumble* > xRumbleIter( s_xRumbleList ); !xRumbleIter.Done(); xRumbleIter.Next() )
	{
		if ( (*xRumbleIter.GetCurrent())->m_uRumbleHash == uRumbleHash ) 
		{
			return (*xRumbleIter.GetCurrent());
		}
	}
	return NULL;
}

void AVP_Rumble_System::SetupRumblePriority( Asura_Hash_ID uPlatformBlueprintHash, Asura_Hash_ID uBlueprintHash )
{
	Asura_Value xValue;
	Asura_Blueprint* pxBlueprint = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_VIBRATIONS, uPlatformBlueprintHash );
	if ( pxBlueprint )
	{
		if(pxBlueprint->GetParameterValueByHashID(0xBA8879A4, xValue)) // Priority
		{
			AVP_Rumble* pxRumble = GetRumbleWithHash( uBlueprintHash );
			if ( pxRumble )
			{
				pxRumble->m_ePriority = static_cast<AVP_Rumble_Priority> ( xValue.GetIntValue() );
			}
		}
	} 
}

Asura_Handle AVP_Rumble_System::AddExclusiveHUDRumble(AVP_Rumble* pxRumble, bool bPause, float fDecay)
{
	if( AVP_Game_Core::IsPaused() )
	{
		return ASURA_INVALID_HANDLE;
	}

	// Only add the rumble if the existing rumble type isn't in use
	if ( pxRumble->iHandle == ASURA_INVALID_HANDLE )
	{	
		pxRumble->iHandle = Asura_Rumble_System::AddHUDRumble(pxRumble,1, 1, 1.0f, 0.0f, fDecay, bPause);
	}

	return pxRumble->iHandle;
}

Asura_Handle AVP_Rumble_System::AddHUDRumble(Asura_Hash_ID uBlueprintHash, bool bPause, float fDecay)
{
	if( AVP_Game_Core::IsPaused() )
	{
		return ASURA_INVALID_HANDLE;
	}

	AVP_Rumble* pxRumble = GetRumbleWithHash( uBlueprintHash );
	if ( pxRumble )
	{
		if ( pxRumble->m_ePriority == RUMBLE_PLAY_ALWAYS)
		{
			return Asura_Rumble_System::AddHUDRumble(pxRumble, 1, 1, 1.0f, 0.0f, fDecay, bPause);
		}
		else if ( pxRumble->m_ePriority ==  RUMBLE_PLAY_EXCLUSIVE )
		{
			return AddExclusiveHUDRumble(pxRumble, bPause, fDecay);
		}
		else
		{
			Asura_Assert(false, "Invalid rumble play type!");
		}
	}
	return ASURA_INVALID_HANDLE;
}

Asura_Handle AVP_Rumble_System::AddWorldRumble(Asura_Hash_ID uBlueprintHash, const Asura_Vector_3& xPosition,  
											   float fOuterRadius,           float fInnerRadius, 
											   int   iEffectPlays,           float fMaxMagnitude, 
											   float fMinMagnitude,          float fDecay,  bool bPaused)
{
	if( AVP_Game_Core::IsPaused())
	{
		return ASURA_INVALID_HANDLE;
	}

	AVP_Rumble* pxRumble = GetRumbleWithHash( uBlueprintHash );
	if ( pxRumble )
	{
		return Asura_Rumble_System::AddWorldRumble(pxRumble, xPosition, fOuterRadius,  
			fInnerRadius, iEffectPlays,  fMaxMagnitude, fMinMagnitude, fDecay, bPaused );
	}
	return ASURA_INVALID_HANDLE;
}

// Asura Rumble System stubs
void Asura_Rumble_System::Project_Initialise(){}
void Asura_Rumble_System::Project_Shutdown(){}
void Asura_Rumble_System::Project_Reset(){}

