// Asura
#include "Core/Asura.h"

#ifdef AVP_USE_INPUT_SIMULATOR

// File's Header
#include "InputSimulator/AVP_InputSimulator_AutomatchTesting.h"

// Asura
#include "Core/Asura_Timers.h"
#ifdef AVP_PCCLI
#include "Core/Asura_Windows.h"
#endif //ifdef AVP_PCCLI
#include "Debugging/Asura_Logfile.h"
#include "FileAccess/Asura_FileAccess.h"
#include "GUIMenu/Asura_GUIMenu_Menu_Page.h"
#include "Network/Online/Asura_Lobby.h"
#include "Network/Online/Asura_Matchmaking_System.h"
#include "Network/Online/Asura_Merit.h"

// AVP
#include "Core/AVP_Core.h"
#include "Core/AVP_Game_Core.h"
#include "FrontEnd/AVP_GameLobby.h"
#include "GUIMenu/AVP_GUIMenu_System.h"
#include "InputSimulator/AVP_InputSimulator_System.h"

// Constants
const Asura_Hash_ID uAVP_INPUTSIMULATOR_STRINGHASH_DEVLOBBY = 0x06950657; //DevLobby_Main
const Asura_Hash_ID uAVP_INPUTSIMULATOR_STRINGHASH_DEVLOBBY_RANKEDMATCH = 0x1230747C; //DevLobby_RankedMatch
const Asura_Hash_ID uAVP_INPUTSIMULATOR_STRINGHASH_DEVLOBBY_AUTOMATCH = 0x183DE398; //DevLobby_Automatch
const Asura_Hash_ID uAVP_INPUTSIMULATOR_STRINGHASH_DEVLOBBY_AUTOMATCHING = 0x0A19ED4A; //DevLobby_Automatching
const Asura_Hash_ID uAVP_INPUTSIMULATOR_STRINGHASH_DEVLOBBY_LOBBY = 0xCC03EEF8; //DevLobby_Lobby
const Asura_Hash_ID uAVP_INPUTSIMULATOR_STRINGHASH_DEVLOBBY_STARTINGGAME = 0x0F618ED0; //DevLobby_StartingGame
const Asura_Hash_ID uAVP_INPUTSIMULATOR_STRINGHASH_DEVLOBBY_ERROR = 0xCBA2E52A; //DevLobby_Error
const Asura_Hash_ID uAVP_INPUTSIMULATOR_STRINGHASH_MP_INFO_POSTTEAM = 0xF088361C; //Multiplayer_Info_PostTeamMatch
const Asura_Hash_ID uAVP_INPUTSIMULATOR_STRINGHASH_MP_INFO_POSTNONTEAM = 0xC8FFD047; //Multiplayer_Info_PostNONTeamMatch

const Asura_Hash_ID uAVP_INPUTSIMULATOR_STRINGHASH_WIDGET_RANKEDMATCH = 0x8DD8C2DA; //RankedMatch
const Asura_Hash_ID uAVP_INPUTSIMULATOR_STRINGHASH_WIDGET_AUTOMATCH = 0x63C23176; //Automatch
const Asura_Hash_ID uAVP_INPUTSIMULATOR_STRINGHASH_WIDGET_SEARCHFORGAME = 0xC2DBB993; //SearchForGame
const Asura_Hash_ID uAVP_INPUTSIMULATOR_STRINGHASH_WIDGET_TOGGLEREADYSTATUS = 0x476D4B81; //ToggleReadyStatus
const Asura_Hash_ID uAVP_INPUTSIMULATOR_STRINGHASH_WIDGET_STARTGAME = 0x811E0F54; //StartGame
const Asura_Hash_ID uAVP_INPUTSIMULATOR_STRINGHASH_WIDGET_ERRORBUTTON = 0xA3C67850; //DevLobby_Button
const Asura_Hash_ID uAVP_INPUTSIMULATOR_STRINGHASH_WIDGET_CONTINUE = 0x9364A0A5; //BUT_Continue
const Asura_Hash_ID uAVP_INPUTSIMULATOR_STRINGHASH_WIDGET_CONTINUE_1 = 0x4CBF1717; //BUT_Continue_1
const Asura_Hash_ID uAVP_INPUTSIMULATOR_STRINGHASH_WIDGET_QUITAPP = 0x65175DE3; //BUT_QuitApplication
const Asura_Hash_ID uAVP_INPUTSIMULATOR_STRINGHASH_WIDGET_QUIT = 0xBCCE454D; //BUT_Quit

const int iAVP_INPUTSIMULATOR_LOGFILE_PATH_MAXLEN = 100;
const int iAVP_INPUTSIMULATOR_SEARCH_PRINT_INTERVAL = 60;
const int iAVP_INPUTSIMULATOR_MAX_RANDOM_MERITPOINTS = 199;
const int iAVP_INPUTSIMULATOR_MAX_LINE_READ = 200;

const char* szAVP_INPUTSIMULATOR_STATSFILE_PATH = "AutomatchTesting\\Stats.txt";
const char* szAVP_INPUTSIMULATOR_STATSFILE_CSV_PATH = "AutomatchTesting\\Stats.csv";
const char* szAVP_INPUTSIMULATOR_LOGFILE_PATH_FORMAT = "logfiles_automatchtesting_%u\\AutomatchStats.txt";

const char* szAVP_INPUTSIMULATOR_STRING_CONSOLEOUT_AQUIREDID = "Aquired log ID %i\n";
const char* szAVP_INPUTSIMULATOR_STRING_CONSOLEOUT_FAILEDTOGETID = "Failed to aquire log ID, turning off input simulation\n";
const char* szAVP_INPUTSIMULATOR_STRING_CONSOLEOUT_MERIT = "Meritpoints:%i, Meritlevel:%i\n";
const char* szAVP_INPUTSIMULATOR_STRING_CONSOLEOUT_LOOKINGFORGAME = "Looking for game...\n";
const char* szAVP_INPUTSIMULATOR_STRING_CONSOLEOUT_GAMEFOUND = "Game found, players so far: %i\n";
const char* szAVP_INPUTSIMULATOR_STRING_CONSOLEOUT_SEARCHINGGAME = "No game found so far, searching...\n";

enum AVP_INPUTSIMULATOR_AUTOMATCHTESTING_LASTMESSAGE
{
	AVP_INPUTSIMULATOR_AUTOMATCHTESTING_LASTMESSAGE_IN_UNKNOWN_MENU,
	AVP_INPUTSIMULATOR_AUTOMATCHTESTING_LASTMESSAGE_HOSTING_GAME,
	AVP_INPUTSIMULATOR_AUTOMATCHTESTING_LASTMESSAGE_FOUND_GAME,
	AVP_INPUTSIMULATOR_AUTOMATCHTESTING_LASTMESSAGE_NOT_MENU_NOT_GAME,
	AVP_INPUTSIMULATOR_AUTOMATCHTESTING_LASTMESSAGE_TRYING_START,
	AVP_INPUTSIMULATOR_AUTOMATCHTESTING_LASTMESSAGE_STARTING_GAME,
	AVP_INPUTSIMULATOR_AUTOMATCHTESTING_LASTMESSAGE_ERROR,
	AVP_INPUTSIMULATOR_AUTOMATCHTESTING_LASTMESSAGE_MP_INFO_POSTTEAM,
	AVP_INPUTSIMULATOR_AUTOMATCHTESTING_LASTMESSAGE_MP_INFO_POSTNONTEAM,
};

// Returns false to indicate it wants the automatch testing to end, otherwise returns true
bool AVP_InputSimulator_AutomatchTesting::Update( u_char* pucCurrentInput, float* pfAxisValue )
{
#ifdef AVP_PCCLI
	char szTitle[200];
	if ( Asura_Lobby_System::IsHost() )
	{
		u_int uNumPlayers = 0;
		const Asura_Lobby* pxLobby = Asura_Lobby_System::GetLobby();
		if ( pxLobby )
		{
			uNumPlayers = pxLobby->GetNumPlayers();
		}

		Asura_PrintfToBfr( szTitle, "Server, SessionID: %u, UDP Port: %u, Players: %u", 200, false, Asura_NetTransport_System::GetServerSessionID(), Asura_NetTransport_System::GetPort(), uNumPlayers );
	}
	else
	{
		Asura_PrintfToBfr( szTitle, "Client, UDP Port: %u", 200, false, Asura_NetTransport_System::GetServerPort() );
	}

	Asura_Windows::SetWindowTitle( szTitle );
#endif //ifdef AVP_PCCLI

#ifdef ASURA_GUIMENU_STORE_NAMES
	if ( AVP_GUIMenu_System::IsCurrentlyInNonHudOrLoadingMenu() )
	{
		const Asura_GUIMenu_Menu_Page* pxFocusedMenu = Asura_GUIMenu_System::GetFocusedMenu();
		if ( pxFocusedMenu )
		{
			Asura_Hash_ID uTempHash = pxFocusedMenu->GetNameHash();
			if ( uTempHash != s_uLastMenuHash )
			{
				Asura_SimpleLogfileOutput(( "logfiles\\AutomatchMenuLog.txt", "%s\n", pxFocusedMenu->GetNameAsChar() ));
				s_uLastMenuHash = uTempHash;
			}
		}
	}
#endif //ifdef ASURA_GUIMENU_STORE_NAMES

	// Only press buttons every other frame
	s_bShouldUpdate = !s_bShouldUpdate;
	if ( !s_bShouldUpdate ) return true;

	if ( !s_bOpenedNewLogFile )
	{
		Asura_SimpleLogfileClear(( "logfiles\\AutomatchStats.txt" ));
		Asura_SimpleLogfileOutput(( "logfiles\\AutomatchStats.txt", "%i\n", s_iAutomatchID ));
		s_bOpenedNewLogFile = true;
	}

	Asura_Hash_ID uMenuNameHash = ASURA_HASH_ID_UNSET;

	if ( AVP_GUIMenu_System::IsCurrentlyInNonHudOrLoadingMenu() )
	{
		const Asura_GUIMenu_Menu_Page* pxFocusedMenu = Asura_GUIMenu_System::GetFocusedMenu();
		if ( pxFocusedMenu )
		{
			uMenuNameHash = pxFocusedMenu->GetNameHash();

			//Once we know we are in a menu and have an active widget on the focused page we can start clicking our way through the menues to start an automatch game.
			if ( uMenuNameHash == uAVP_INPUTSIMULATOR_STRINGHASH_DEVLOBBY )
			{
				FindAndClickWidget( uAVP_INPUTSIMULATOR_STRINGHASH_WIDGET_RANKEDMATCH, pucCurrentInput, pfAxisValue );
			}
			else if (uMenuNameHash == uAVP_INPUTSIMULATOR_STRINGHASH_DEVLOBBY_RANKEDMATCH )
			{
				FindAndClickWidget( uAVP_INPUTSIMULATOR_STRINGHASH_WIDGET_AUTOMATCH, pucCurrentInput, pfAxisValue );
			}
			else if (uMenuNameHash == uAVP_INPUTSIMULATOR_STRINGHASH_DEVLOBBY_AUTOMATCH )
			{
				if ( s_eTestState != AVP_AUTOMATCHTESTING_STATE_PREJOIN )
				{
					s_eTestState = AVP_AUTOMATCHTESTING_STATE_PREJOIN;
				}
				FindAndClickWidget( uAVP_INPUTSIMULATOR_STRINGHASH_WIDGET_SEARCHFORGAME, pucCurrentInput, pfAxisValue );
			}
			else if (uMenuNameHash == uAVP_INPUTSIMULATOR_STRINGHASH_DEVLOBBY_AUTOMATCHING )
			{
				// We have started automatching, generate a random skill level, log that and print how the search is going every once in a while
				if ( s_eTestState != AVP_AUTOMATCHTESTING_STATE_INLOBBY )
				{
					s_eTestState = AVP_AUTOMATCHTESTING_STATE_INLOBBY;

					int iPoints = 0;
					u_int uLevel = 0;
#ifdef ASURA_USE_MERIT_SYSTEM
					int iRandomMerit = Asura_Random::GetSeededInt(0, iAVP_INPUTSIMULATOR_MAX_RANDOM_MERITPOINTS );
					Asura_Lobby_System::SetLocalPlayerMerit( Asura_Merit_Calculator::GetLevelFromMerit( iRandomMerit, 0 ), iRandomMerit );
					Asura_CommandConsole::Printf( szAVP_INPUTSIMULATOR_STRING_CONSOLEOUT_MERIT, iRandomMerit, Asura_Merit_Calculator::GetLevelFromMerit( iRandomMerit, 0 ) );
					iPoints = iRandomMerit;
					uLevel = Asura_Merit_Calculator::GetLevelFromMerit( iRandomMerit, 0 );
#else
					int iRandomSkill = Asura_Random::GetSeededInt(0, ASURA_LOBBY_MAX_SKILL );
					Asura_Lobby_System::SetLocalPlayerSkill( iRandomSkill );
					Asura_CommandConsole::Printf( szAVP_INPUTSIMULATOR_STRING_CONSOLEOUT_MERIT, 0, iRandomSkill );
					uLevel = iRandomSkill;
#endif
					Asura_SimpleLogfileOutput(( "logfiles\\AutomatchStats.txt", "#NEW#\n%S\n%i\n%u\n", Asura_Lobby_System::GetLocalPlayerName(), iPoints, uLevel ));
					s_lTicksAtAttempt = Asura_Timers::GetCurTicks();
				}
				if ( s_iFrameCounter > iAVP_INPUTSIMULATOR_SEARCH_PRINT_INTERVAL )
				{
					const Asura_Lobby* pxLobby = Asura_Lobby_System::GetLobby();
					if ( pxLobby )
					{
						if ( Asura_Lobby_System::IsHost() )
						{
							if ( s_iLastMessageDisplayed != AVP_INPUTSIMULATOR_AUTOMATCHTESTING_LASTMESSAGE_HOSTING_GAME )
							{
								Asura_CommandConsole::Printf( "Hosting game, TCP port: %u, UPD port: %u, players: %i\n", Asura_Lobby_System::GetBoundTCPPort(), Asura_NetTransport_System::GetPort(), pxLobby->GetNumPlayers() );
								s_iLastMessageDisplayed = AVP_INPUTSIMULATOR_AUTOMATCHTESTING_LASTMESSAGE_HOSTING_GAME;
							}
						}
						else
						{
							if ( s_iLastMessageDisplayed != AVP_INPUTSIMULATOR_AUTOMATCHTESTING_LASTMESSAGE_FOUND_GAME )
							{
								Asura_CommandConsole::Printf( "Found game, UPD port: %u, players: %i\n", Asura_NetTransport_System::GetServerPort(), pxLobby->GetNumPlayers() );
								s_iLastMessageDisplayed = AVP_INPUTSIMULATOR_AUTOMATCHTESTING_LASTMESSAGE_FOUND_GAME;
							}
						}
					}
					else
					{
						Asura_CommandConsole::Printf( szAVP_INPUTSIMULATOR_STRING_CONSOLEOUT_SEARCHINGGAME );
					}
					s_iFrameCounter = 0;
				}
				else
				{
					s_iFrameCounter++;
				}

			}
			else if (uMenuNameHash == uAVP_INPUTSIMULATOR_STRINGHASH_DEVLOBBY_LOBBY )
			{
				// Once we are in a lobby we log some stats on it, and make sure we are ready to start the game.
				if ( s_eTestState != AVP_AUTOMATCHTESTING_STATE_JOINING )
				{
					s_eTestState = AVP_AUTOMATCHTESTING_STATE_JOINING; 

					Asura_CommandConsole::Printf( "In lobby, waiting for players to get ready...\n" );
					int iLobbySkill = 0;
					float fLobbyMQ = 0.0f;
					const Asura_Lobby* pxLobby = Asura_Lobby_System::GetLobby();

					if ( pxLobby )
					{
						const Asura_Lobby_Attributes& xAttributes = pxLobby->GetAttributes();
						iLobbySkill = Asura_Matchmaking_System::GetLobbyAverageSkill( xAttributes );
						fLobbyMQ = Asura_Matchmaking_System::GetMatchQuality( xAttributes );
					}

					Asura_TickCount lTicksInLobby = Asura_Timers::GetCurTicks();
					Asura_TickCount lTicksToFindLobby = lTicksInLobby - s_lTicksAtAttempt;
					float fSecondsToFindLobby = Asura_Timers::TicksToSeconds( lTicksToFindLobby );

					Asura_SimpleLogfileOutput(( "logfiles\\AutomatchStats.txt", "#JOIN#\n%i\n%f\n%f\n", iLobbySkill, fLobbyMQ, fSecondsToFindLobby ));
				}
				if ( !Asura_Lobby_System::GetLocalPlayerReady() )
				{
					Asura_CommandConsole::Printf("Getting ready...\n" );
					FindAndClickWidget( uAVP_INPUTSIMULATOR_STRINGHASH_WIDGET_TOGGLEREADYSTATUS, pucCurrentInput, pfAxisValue );
				}
				if ( Asura_Lobby_System::GetLocalPlayerReady() )
				{
					if ( s_iLastMessageDisplayed != AVP_INPUTSIMULATOR_AUTOMATCHTESTING_LASTMESSAGE_TRYING_START )
					{
						Asura_CommandConsole::Printf("Trying to start game...\n" );
						s_iLastMessageDisplayed = AVP_INPUTSIMULATOR_AUTOMATCHTESTING_LASTMESSAGE_TRYING_START;
					}
					FindAndClickWidget( uAVP_INPUTSIMULATOR_STRINGHASH_WIDGET_STARTGAME, pucCurrentInput, pfAxisValue );
				}
			}
			else if (uMenuNameHash == uAVP_INPUTSIMULATOR_STRINGHASH_DEVLOBBY_STARTINGGAME )
			{
				if ( s_iLastMessageDisplayed != AVP_INPUTSIMULATOR_AUTOMATCHTESTING_LASTMESSAGE_STARTING_GAME )
				{
					Asura_CommandConsole::Printf( "Starting game...\n" );
					s_iLastMessageDisplayed = AVP_INPUTSIMULATOR_AUTOMATCHTESTING_LASTMESSAGE_STARTING_GAME;
				}
			}
			else if (uMenuNameHash == uAVP_INPUTSIMULATOR_STRINGHASH_DEVLOBBY_ERROR )
			{
				if ( s_iLastMessageDisplayed != AVP_INPUTSIMULATOR_AUTOMATCHTESTING_LASTMESSAGE_ERROR )
				{
					Asura_CommandConsole::Printf( "In error menu\n" );
					s_iLastMessageDisplayed = AVP_INPUTSIMULATOR_AUTOMATCHTESTING_LASTMESSAGE_ERROR;
				}
				FindAndClickWidget( uAVP_INPUTSIMULATOR_STRINGHASH_WIDGET_ERRORBUTTON, pucCurrentInput, pfAxisValue );
			}
			else if (uMenuNameHash == uAVP_INPUTSIMULATOR_STRINGHASH_MP_INFO_POSTTEAM )
			{
				if ( s_iLastMessageDisplayed != AVP_INPUTSIMULATOR_AUTOMATCHTESTING_LASTMESSAGE_MP_INFO_POSTTEAM )
				{
					Asura_CommandConsole::Printf( "In post team-game  menu\n" );
					s_iLastMessageDisplayed = AVP_INPUTSIMULATOR_AUTOMATCHTESTING_LASTMESSAGE_MP_INFO_POSTTEAM;
				}
				FindAndClickWidget( uAVP_INPUTSIMULATOR_STRINGHASH_WIDGET_QUITAPP, pucCurrentInput, pfAxisValue );
			}
			else if (uMenuNameHash == uAVP_INPUTSIMULATOR_STRINGHASH_MP_INFO_POSTNONTEAM )
			{
				if ( s_iLastMessageDisplayed != AVP_INPUTSIMULATOR_AUTOMATCHTESTING_LASTMESSAGE_MP_INFO_POSTNONTEAM )
				{
					Asura_CommandConsole::Printf( "In post game  menu\n" );
					s_iLastMessageDisplayed = AVP_INPUTSIMULATOR_AUTOMATCHTESTING_LASTMESSAGE_MP_INFO_POSTNONTEAM;
				}
				FindAndClickWidget( uAVP_INPUTSIMULATOR_STRINGHASH_WIDGET_QUIT, pucCurrentInput, pfAxisValue );
			}
			else
			{
				if ( s_iLastMessageDisplayed != AVP_INPUTSIMULATOR_AUTOMATCHTESTING_LASTMESSAGE_IN_UNKNOWN_MENU )
				{
#ifdef ASURA_GUIMENU_STORE_NAMES
					Asura_CommandConsole::Printf( "In unknown menu: \"%s\"\n", pxFocusedMenu->GetNameAsChar() );
#endif
					s_iLastMessageDisplayed = AVP_INPUTSIMULATOR_AUTOMATCHTESTING_LASTMESSAGE_IN_UNKNOWN_MENU;
				}
			}
		}
	}
	else
	{
		if ( AVP_Core::IsInGameState() && AVP_Game_Core::GetReceivedInitialiseCompleteMessage() )
		{
			//Once we are ingame we should log that and if we are the host we should log that we started a game, its average skill and number of players
			if ( s_eTestState != AVP_AUTOMATCHTESTING_STATE_INGAME )
			{
				s_eTestState = AVP_AUTOMATCHTESTING_STATE_INGAME;

				Asura_CommandConsole::Printf( "Im ingame!\n" );

				Asura_SimpleLogfileOutput(( "logfiles\\AutomatchStats.txt", "#GAME#\n" ));
				if ( Asura_Lobby_System::IsHost() )
				{
					int iLobbySkill = 0;
					u_int uLobbyPlayers = 0;
					const Asura_Lobby* pxLobby = Asura_Lobby_System::GetLobby();

					if ( pxLobby )
					{
						const Asura_Lobby_Attributes& xAttributes = pxLobby->GetAttributes();
						iLobbySkill = Asura_Matchmaking_System::GetLobbyAverageSkill( xAttributes );
						uLobbyPlayers = pxLobby->GetNumPlayers();
					}
					Asura_SimpleLogfileOutput(( "logfiles\\AutomatchStats.txt", "#HOST#\n%i\n%u\n", iLobbySkill, uLobbyPlayers ));
				}

				//AVP_Core::RequestQuit(); //Not sure if the clients should automaticaly quit, or when if they should.
			}
		}
		else
		{
			if ( s_iLastMessageDisplayed != AVP_INPUTSIMULATOR_AUTOMATCHTESTING_LASTMESSAGE_NOT_MENU_NOT_GAME )
			{
				Asura_CommandConsole::Printf( "Im not in a menu, and Im not ingame, dunno where I am...\n" );
				s_iLastMessageDisplayed = AVP_INPUTSIMULATOR_AUTOMATCHTESTING_LASTMESSAGE_NOT_MENU_NOT_GAME;
			}
		}
	}

	return true;
}

bool AVP_InputSimulator_AutomatchTesting::GatherStatistics( bool bHelp )
{
	int iTestID = 0;
	bool bKeepLooking = true;

	int iAutomatchAttempts = 0;
	int iAutomatchSuccesses = 0;
	int iGamesEntered = 0;
	float fMinMQ = 1.0f;
	float fMaxMQ = 0.0f;
	float fTotalMQ = 0.0f;

	int aiLevelDistribution[ASURA_LOBBY_MAX_SKILL];
	float afMQDistribution[ASURA_LOBBY_MAX_SKILL];
	int aiAutomatchSuccessesByLevel[ASURA_LOBBY_MAX_SKILL];
	int aiGamesEnteredByLevel[ASURA_LOBBY_MAX_SKILL];
	float afMinMQByLevel[ASURA_LOBBY_MAX_SKILL];
	float afMaxMQByLevel[ASURA_LOBBY_MAX_SKILL];
	int aiGamesHostedByLevel[ASURA_LOBBY_MAX_SKILL];
	u_int auLobbyPlayersByLevel[ASURA_LOBBY_MAX_SKILL];

	for (unsigned int i=0;i<ASURA_LOBBY_MAX_SKILL;i++)
	{
		aiLevelDistribution[i] = 0;
		afMQDistribution[i] = 0.0f;
		aiAutomatchSuccessesByLevel[i] = 0;
		aiGamesEnteredByLevel[i] = 0;
		afMinMQByLevel[i] = 1.0f;
		afMaxMQByLevel[i] = 0.0f;
		aiGamesHostedByLevel[i] = 0;
		auLobbyPlayersByLevel[i] = 0;
	}

	Asura_SimpleLogfileClear(( "logfiles\\AutomatchGatheredStats.txt" ));
	Asura_SimpleLogfileOutput(( "logfiles\\AutomatchGatheredStats.txt", "Results of automatch testing (stats are at the bottom)\n\n ERRORS:\n" ));

	Asura_SimpleLogfileClear(( "logfiles\\AutomatchGatheredStats.csv" ));

	Asura_Handle iInFile;

	while ( bKeepLooking )
	{
		char szFileToTest[iAVP_INPUTSIMULATOR_LOGFILE_PATH_MAXLEN];
		Asura_PrintfToBfr(szFileToTest, szAVP_INPUTSIMULATOR_LOGFILE_PATH_FORMAT, iAVP_INPUTSIMULATOR_LOGFILE_PATH_MAXLEN, false, iTestID );
		if ( Asura_FileAccess::FileExists(szFileToTest) )
		{
			iInFile = Asura_FileAccess::Open( szFileToTest );
			if ( iInFile != ASURA_INVALID_HANDLE )
			{
				int iFoundID = -1;
				int iMeritPoints = -1;
				int iMeritLevel = -1;
				int iLobbyMeritLevel = -1;
				float fLobbyMQ = 0.0f;
				float fTimeToFindLobby = 0.0f;
				int iHostedLobbyLevel = -1;
				u_int uHostedLobbyPlayers = 0;
				Asura_Char szLocalName[ASURA_LOBBY_PLAYER_NAME_BUFFER_SIZE];
				char szLine[iAVP_INPUTSIMULATOR_MAX_LINE_READ];

				Asura_FileAccess::ReadLine( iInFile, szLine, iAVP_INPUTSIMULATOR_MAX_LINE_READ );

				if ( Asura_StringUtil::StartsWith( szLine, "Asura Logfile", true ) )
				{
					Asura_FileAccess::ReadLine( iInFile, szLine, iAVP_INPUTSIMULATOR_MAX_LINE_READ );
					Asura_FileAccess::ReadLine( iInFile, szLine, iAVP_INPUTSIMULATOR_MAX_LINE_READ );

					if ( !Asura_StringUtil::GetIntValue( szLine, &iFoundID ) )
					{
						iFoundID = -1;
						Asura_SimpleLogfileOutput(( "logfiles\\AutomatchGatheredStats.txt", "[%i]ERROR: Could not find ID\n", iTestID ));
					}

					if ( iFoundID != iTestID )
					{
						Asura_SimpleLogfileOutput(( "logfiles\\AutomatchGatheredStats.txt", "[%i]ERROR: ID Missmatch, in FileName: %i, in FileHead: %i\n", iFoundID, iTestID, iFoundID ));
					}

					Asura_SimpleLogfileOutput(( "logfiles\\AutomatchGatheredStats.csv", "%i, ", iFoundID ));

					while ( szLine[0] )
					{
						Asura_FileAccess::ReadLine( iInFile, szLine, iAVP_INPUTSIMULATOR_MAX_LINE_READ );

						if ( strcmp(szLine, "#NEW#") == 0 )
						{
							Asura_FileAccess::ReadLine( iInFile, szLocalName, ASURA_LOBBY_PLAYER_NAME_BUFFER_SIZE );

							Asura_FileAccess::ReadLine( iInFile, szLine, iAVP_INPUTSIMULATOR_MAX_LINE_READ );

							if ( !Asura_StringUtil::GetIntValue( szLine, &iMeritPoints ) )
							{
								iMeritPoints = -1;
								Asura_SimpleLogfileOutput(( "logfiles\\AutomatchGatheredStats.txt", "[%i]ERROR: Could not find Merit points\n", iFoundID ));
							}

							Asura_FileAccess::ReadLine( iInFile, szLine, iAVP_INPUTSIMULATOR_MAX_LINE_READ );

							if ( !Asura_StringUtil::GetIntValue( szLine, &iMeritLevel ) )
							{
								iMeritLevel = -1;
								Asura_SimpleLogfileOutput(( "logfiles\\AutomatchGatheredStats.txt", "[%i]ERROR: Could not find Merit level\n", iFoundID ));
							}

							if ( iMeritLevel > -1 )
							{
								iAutomatchAttempts++;
								if ( iMeritLevel >= 0 && iMeritLevel <  ASURA_LOBBY_MAX_SKILL )
								{
									aiLevelDistribution[iMeritLevel]++;
								}
								else
								{
									Asura_SimpleLogfileOutput(( "logfiles\\AutomatchGatheredStats.txt", "[%i]ERROR: Merit level error: %i is not [0-%i]\n", iFoundID, iMeritLevel, ASURA_LOBBY_MAX_SKILL - 1 ));
								}
							}
							Asura_SimpleLogfileOutput(( "logfiles\\AutomatchGatheredStats.csv", "#NEW#, %s, %i, %i,", szLocalName, iMeritPoints, iMeritLevel ));
						}
						else if ( strcmp(szLine, "#JOIN#") == 0 )
						{
							Asura_FileAccess::ReadLine( iInFile, szLine, iAVP_INPUTSIMULATOR_MAX_LINE_READ );

							if ( !Asura_StringUtil::GetIntValue( szLine, &iLobbyMeritLevel ) )
							{
								iLobbyMeritLevel = -1;
								Asura_SimpleLogfileOutput(( "logfiles\\AutomatchGatheredStats.txt", "[%i]ERROR: Could not find lobby Merit level\n", iFoundID ));
							}

							Asura_FileAccess::ReadLine( iInFile, szLine, iAVP_INPUTSIMULATOR_MAX_LINE_READ );

							if ( !Asura_StringUtil::GetFloatValue( szLine, &fLobbyMQ ) )
							{
								fLobbyMQ = 0.0f;
								Asura_SimpleLogfileOutput(( "logfiles\\AutomatchGatheredStats.txt", "[%i]ERROR: Could not find lobby MQ\n", iFoundID ));
							}

							Asura_FileAccess::ReadLine( iInFile, szLine, iAVP_INPUTSIMULATOR_MAX_LINE_READ );

							if ( !Asura_StringUtil::GetFloatValue( szLine, &fTimeToFindLobby ) )
							{
								fTimeToFindLobby = 0.0f;
								Asura_SimpleLogfileOutput(( "logfiles\\AutomatchGatheredStats.txt", "[%i]ERROR: Could not find time to find lobby\n", iFoundID ));
							}

							if ( iMeritLevel >= 0 && iMeritLevel <  ASURA_LOBBY_MAX_SKILL )
							{
								afMQDistribution[iMeritLevel] += fLobbyMQ;
								aiAutomatchSuccessesByLevel[iMeritLevel]++;
								if ( afMinMQByLevel[iMeritLevel] > fLobbyMQ )
								{
									afMinMQByLevel[iMeritLevel] = fLobbyMQ;
								}
								if ( afMaxMQByLevel[iMeritLevel] < fLobbyMQ )
								{
									afMaxMQByLevel[iMeritLevel] = fLobbyMQ;
								}
							}
							else
							{
								Asura_SimpleLogfileOutput(( "logfiles\\AutomatchGatheredStats.txt", "[%i]ERROR: Merit level error: %i is not [0-%i]\n", iFoundID, iMeritLevel, ASURA_LOBBY_MAX_SKILL - 1 ));
							}

							iAutomatchSuccesses++;

							fTotalMQ += fLobbyMQ;
							if ( fMinMQ > fLobbyMQ )
							{
								fMinMQ = fLobbyMQ;
							}
							if ( fMaxMQ < fLobbyMQ )
							{
								fMaxMQ = fLobbyMQ;
							}
							Asura_SimpleLogfileOutput(( "logfiles\\AutomatchGatheredStats.csv", "#JOIN#, %i, %f, %f, ", iLobbyMeritLevel, fLobbyMQ, fTimeToFindLobby ));
						}
						else if ( strcmp(szLine, "#GAME#") == 0 )
						{
							if ( iMeritLevel >= 0 && iMeritLevel < ASURA_LOBBY_MAX_SKILL )
							{
								aiGamesEnteredByLevel[iMeritLevel]++;
							}
							iGamesEntered++;
							Asura_SimpleLogfileOutput(( "logfiles\\AutomatchGatheredStats.csv", "#GAME#, " ));
						}
						else if ( strcmp(szLine, "#HOST#") == 0 )
						{
							Asura_FileAccess::ReadLine( iInFile, szLine, iAVP_INPUTSIMULATOR_MAX_LINE_READ );

							if ( !Asura_StringUtil::GetIntValue( szLine, &iHostedLobbyLevel ) )
							{
								iHostedLobbyLevel = -1;
								Asura_SimpleLogfileOutput(( "logfiles\\AutomatchGatheredStats.txt", "[%i]ERROR: Could not find hosted lobby level\n", iFoundID ));
							}

							Asura_FileAccess::ReadLine( iInFile, szLine, iAVP_INPUTSIMULATOR_MAX_LINE_READ );

							int iTempInt;
							if ( !Asura_StringUtil::GetIntValue( szLine, &iTempInt ) )
							{
								uHostedLobbyPlayers = 0;
								Asura_SimpleLogfileOutput(( "logfiles\\AutomatchGatheredStats.txt", "[%i]ERROR: Could not find hosted lobby players\n", iFoundID ));
							}
							else
							{
								uHostedLobbyPlayers = static_cast<u_int>( iTempInt );
							}

							if ( iHostedLobbyLevel >= 0 && iHostedLobbyLevel < ASURA_LOBBY_MAX_SKILL )
							{
								aiGamesHostedByLevel[iHostedLobbyLevel]++;
								auLobbyPlayersByLevel[iHostedLobbyLevel] += uHostedLobbyPlayers;
							}
							else
							{
								Asura_SimpleLogfileOutput(( "logfiles\\AutomatchGatheredStats.txt", "[%i]ERROR: Lobby merit level error: %i is not [0-%u]\n", iFoundID, iHostedLobbyLevel, ASURA_LOBBY_MAX_SKILL - 1 ));
							}
							Asura_SimpleLogfileOutput(( "logfiles\\AutomatchGatheredStats.csv", "#HOST#, %i, %u, ", iHostedLobbyLevel, uHostedLobbyPlayers ));
						}
					}
					Asura_SimpleLogfileOutput(( "logfiles\\AutomatchGatheredStats.csv", "\n" ));
				}
				
				Asura_FileAccess::Close( iInFile );
			}
		}
		else
		{
			bKeepLooking = false;
		}
		iTestID++;
	}

	Asura_SimpleLogfileOutput(( "logfiles\\AutomatchGatheredStats.txt", "\n STATS:\n" ));

	float fAvgLobbyMQ = fTotalMQ / static_cast<float>(iAutomatchSuccesses);

	Asura_SimpleLogfileOutput(( "logfiles\\AutomatchGatheredStats.txt", "\nLevel\t#\tMinMQ\tMaxMQ\tAvgMQ\tFound\tInGame\tLobbies\tPlayers\n" ));

	for (unsigned int i=0;i<ASURA_LOBBY_MAX_SKILL;i++)
	{
		float fAvgMQAtLevel = 0.0f;
		if ( aiAutomatchSuccessesByLevel[i] > 0 )
		{
			fAvgMQAtLevel = afMQDistribution[i] / static_cast<float>(aiAutomatchSuccessesByLevel[i]);
		}
		Asura_SimpleLogfileOutput(( "logfiles\\AutomatchGatheredStats.txt", "%u\t%i\t%.2f\t%.2f\t%.2f\t%i\t%i\t%i\t%u\n", i+1, aiLevelDistribution[i], afMinMQByLevel[i], afMaxMQByLevel[i], fAvgMQAtLevel, aiAutomatchSuccessesByLevel[i], aiGamesEnteredByLevel[i], aiGamesHostedByLevel[i], auLobbyPlayersByLevel[i] ));
	}

	Asura_SimpleLogfileOutput(( "logfiles\\AutomatchGatheredStats.txt", "\n\nTotal automatching attempts: %i\n", iAutomatchAttempts ));
	Asura_SimpleLogfileOutput(( "logfiles\\AutomatchGatheredStats.txt", "\nTotal that found a game: %i\n", iAutomatchSuccesses ));
	Asura_SimpleLogfileOutput(( "logfiles\\AutomatchGatheredStats.txt", "\nTotal that entered a game: %i\n", iGamesEntered ));
	Asura_SimpleLogfileOutput(( "logfiles\\AutomatchGatheredStats.txt", "\n\nLowest lobby MQ: %.2f\n", fMinMQ ));
	Asura_SimpleLogfileOutput(( "logfiles\\AutomatchGatheredStats.txt", "\nHighest lobby MQ: %.2f\n", fMaxMQ ));
	Asura_SimpleLogfileOutput(( "logfiles\\AutomatchGatheredStats.txt", "\nAverage lobby MQ: %.2f\n", fAvgLobbyMQ ));

	return true;
}

bool AVP_InputSimulator_AutomatchTesting::ConsoleCmd_SetAutomatchID( int iNewID, bool bInterrogate, bool bHelp )
{
	if (bHelp || bInterrogate)
	{
		Asura_CommandConsole::Printf(L"Sets the Automatch ID used by the Automatch testing.");
	}
	else
	{
		s_iAutomatchID = iNewID;
	}
	return true;
}

void AVP_InputSimulator_AutomatchTesting::FindAndClickWidget( Asura_Hash_ID uWidget, u_char* pucCurrentInput, float* pfAxisValue )
{
	const Asura_GUIMenu_Menu_Page* pxFocusedMenu = Asura_GUIMenu_System::GetFocusedMenu();
	if ( !pxFocusedMenu ) return;
	const Asura_GUIMenu_Root* pxActiveWidget = pxFocusedMenu->GetActiveWidget();
	if ( !pxActiveWidget ) return;
	const Asura_Hash_ID uWidgetHash = pxActiveWidget->GetNameHash();
	if ( uWidgetHash == uWidget )
	{
		AVP_InputSimulator_System::Platform_SetAcceptButton( true, pucCurrentInput );
	}
	else
	{
		AVP_InputSimulator_System::Platform_SetDownButton( true, pucCurrentInput, pfAxisValue );
	}
}

AVP_AUTOMATCHTESTING_STATE AVP_InputSimulator_AutomatchTesting::s_eTestState = AVP_AUTOMATCHTESTING_STATE_PREJOIN;
bool AVP_InputSimulator_AutomatchTesting::s_bOpenedNewLogFile = false;
bool AVP_InputSimulator_AutomatchTesting::s_bShouldUpdate = false;
int AVP_InputSimulator_AutomatchTesting::s_iAutomatchID = -1;
int AVP_InputSimulator_AutomatchTesting::s_iFrameCounter = 0;
int AVP_InputSimulator_AutomatchTesting::s_iLastMessageDisplayed = 0;
Asura_TickCount AVP_InputSimulator_AutomatchTesting::s_lTicksAtAttempt = 0;
Asura_Hash_ID AVP_InputSimulator_AutomatchTesting::s_uLastMenuHash = ASURA_HASH_ID_UNSET;

#endif // AVP_USE_INPUT_SIMULATOR
