// Core
#include "Core/Asura.h"

#ifdef AVP_USE_INPUT_SIMULATOR

// Asura
#include "Interface/Asura_CommandConsole.h"

// AVP
#include "Core/AVP_Game_Core.h"
#include "Core/AVP_Settings.h"

// Header
#include "InputSimulator/AVP_InputSimulator_System.h"

// Systems
#include "InputSimulator/AVP_InputSimulator_Alien.h"
#include "InputSimulator/AVP_InputSimulator_Marine.h"
#include "InputSimulator/AVP_InputSimulator_Predator.h"
#include "InputSimulator/AVP_InputSimulator_AutomatchTesting.h"

static bool g_bUseInputSimulation = false;
static bool g_bUseAutomatchTesting = false;

void AVP_InputSimulator_System::Initialise()
{
	Asura_CommandConsole::AddVar( L"Input.UseInputSimulation", &g_bUseInputSimulation, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );

	Asura_CommandConsole::AddVar( L"AutomatchTesting.Active", &g_bUseAutomatchTesting, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::AddCmd( L"AutomatchTesting.SetID", AVP_InputSimulator_AutomatchTesting::ConsoleCmd_SetAutomatchID, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::AddCmd( L"AutomatchTesting.GatherStats", AVP_InputSimulator_AutomatchTesting::GatherStatistics, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );

	AVP_InputSimulator_Alien::Initialise();
	AVP_InputSimulator_Marine::Initialise();
	AVP_InputSimulator_Predator::Initialise();
}

void AVP_InputSimulator_System::Shutdown()
{
	Asura_CommandConsole::RemoveVar( L"Input.UseInputSimulation" );
	Asura_CommandConsole::RemoveVar( L"AutomatchTesting.Active" );

	AVP_InputSimulator_Alien::Shutdown();
	AVP_InputSimulator_Marine::Shutdown();
	AVP_InputSimulator_Predator::Shutdown();
}

void AVP_InputSimulator_System::Update( u_char* ucCurrentInput, float* pfAxisValue )
{	
	if ( g_bUseAutomatchTesting )
	{
		g_bUseAutomatchTesting = AVP_InputSimulator_AutomatchTesting::Update( ucCurrentInput, pfAxisValue );
	}

	if( !AVP_Game_Core::GetPlayer() || !g_bUseInputSimulation )
	{
		return;
	}

	switch( AVP_Settings::GetPlayerSpecies() )
	{	
		case AVP_Settings::AVP_SPECIES_ALIEN:
		{
			AVP_InputSimulator_Alien::Update( ucCurrentInput, pfAxisValue );
			break;
		}

		case AVP_Settings::AVP_SPECIES_HUMAN:
		{
			AVP_InputSimulator_Marine::Update( ucCurrentInput, pfAxisValue );
			break;
		}

		case AVP_Settings::AVP_SPECIES_PREDATOR:
		{
			AVP_InputSimulator_Predator::Update( ucCurrentInput, pfAxisValue );
			break;
		}

		case AVP_Settings::AVP_SPECIES_UNKNOWN:
		default:
		{
			break;
		}
	}	
}

#endif // AVP_USE_INPUT_SIMULATOR
