
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_InventoryItem_SubType.h
// Author : Andy Tate
// Date   : 21-Nov-2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __AVP_INVENTORYITEM_SUBTYPE_H__
#define __AVP_INVENTORYITEM_SUBTYPE_H__

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Forward class declarations...
////////////////////////////////////////////////////////////////////////////////

class Asura_Chunk_Stream;

////////////////////////////////////////////////////////////////////////////////
// Class declaration...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_InventoryItem_SubType
// Author       : Andy Tate
// Date         : 13-Nov-2007
//
// Description  : Class to handle the quantity of the particular inventory item.
////////////////////////////////////////////////////////////////////////////////

class AVP_InventoryItem_SubType
{
public:
	AVP_InventoryItem_SubType( const int iSubType, const Asura_Hash_ID uBlueprintHashID );
	AVP_InventoryItem_SubType(const Asura_Hash_ID uBlueprintHash);
	virtual ~AVP_InventoryItem_SubType() {}

	virtual void InitialiseFromBlueprint();

	virtual void ReadFromChunkStream( Asura_Chunk_Stream& xStream );
	virtual void WriteToChunkStream( Asura_Chunk_Stream& xStream ) const;

	virtual float	DecreaseQuantity(const float fQuantity);
	virtual float	IncreaseQuantity(const float fQuantity);

	virtual void	InitialiseQuantity(const float fInitial, const float fMax)	{ m_fCurrentQuantity = fInitial; m_fMaxQuantity = fMax; }

	virtual void	SetCurrentQuantity(const float fQuantity)	{ m_fCurrentQuantity = fQuantity; }
	virtual float	GetCurrentQuantity(void) const				{ return(m_fCurrentQuantity); }

	virtual void	SetMaxQuantity(const float fQuantity)	{ m_fMaxQuantity = fQuantity; }
	virtual float	GetMaxQuantity(void) const				{ return(m_fMaxQuantity); }

	virtual void			SetAmmoType(const Asura_Hash_ID uType)	{ m_uAmmoType = uType; }
	virtual Asura_Hash_ID	GetAmmoType(void) const					{ return(m_uAmmoType); }

	virtual bool	IsEmpty(void) const	{ return(m_fCurrentQuantity == 0.0f); }
	virtual bool	IsFull(void) const	{ return(m_fCurrentQuantity == m_fMaxQuantity); }

	int GetType() const { return m_iType; }
	Asura_Hash_ID GetBlueprintHashID() const { return m_uBlueprintHash; }

	virtual bool IsDynamic() const { return false; }

	Asura_Hash_ID GetVibrationHash()  const { return m_uVibrationHash; } 

	void          InitRumble() const;

protected:
	int				m_iType;
	Asura_Hash_ID	m_uBlueprintHash;		// The hash-ID of the blueprint used to describe this quantity.
	Asura_Hash_ID	m_uAmmoType;
	float			m_fCurrentQuantity;		// The number of items of this type we have.
	float			m_fMaxQuantity;			// The maximum number of items of this type we can have.
	Asura_Hash_ID	m_uVibrationHash;

private:
	AVP_InventoryItem_SubType(void);
	AVP_InventoryItem_SubType(const AVP_InventoryItem_SubType& xOther);
	AVP_InventoryItem_SubType&	operator=(const AVP_InventoryItem_SubType& xOther);

	static const u_int uCURRENT_CHUNK_VERSION = 1;
};

#endif   // ifndef __AVP_INVENTORYITEM_SUBTYPE_H__

/// End of file. ///
