
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_Inventory_Group.cpp
// Author : Andy Tate
// Date   : 25-Oct-2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "Inventory/AVP_Inventory_Group.h"

// Asura independent header-files...
#include "Blueprint/Asura_Blueprint_System.h"
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "Core/Asura_Timers.h"
#include "Network/Asura_Message.h"

// Game independent header-files...
#include "Blueprint/AVP_Blueprint_HashIDs.h"
#include "Core/AVP_Settings.h"
#include "Entity/AVP_Messages.h"
#include "Entity/PhysicalObject/Actor/AVP_ClientEntity_Actor.h"
#include "Entity/PhysicalObject/Actor/AVP_ServerEntity_Actor.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/AVP_ClientActor_Vertebrate.h"
#include "Entity/PhysicalObject/Pickup/AVP_Pickup_Tool.h"
#include "Inventory/AVP_Inventory_Item.h"
#include "Inventory/AVP_InventoryItem_Messages.h"
#include "Inventory/AVP_InventoryItem_SubType.h"
#include "Inventory/AVP_Inventory_System.h"
#include "Inventory/AVP_Inventory_HashIDs.h"
#include "Inventory/Items/AVP_InventoryItem_HumanFlare.h"
#include "Inventory/Items/AVP_InventoryItem_PredatorCloak.h"
#include "Inventory/Items/AVP_InventoryItem_Stim.h"
#include "Inventory/Items/AVP_InventoryItem_Torch.h"
#include "Inventory/Groups/AVP_InventoryGroup_Weapons.h" // for slot type

////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Private functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Protected functions...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Inventory_Group::OnStateChange
// Author       : Andy Tate
// Date         : 26-Nov-2007
////////////////////////////////////////////////////////////////////////////////

void AVP_Inventory_Group::OnStateChange(const u_int uOldState,
										const u_int uNewState)
{
	switch(uNewState)
	{
		case NO_ITEM_ACTIVE:
		{
			const bool	bWasActive	= (uOldState == ITEM_ACTIVE);

			if(bWasActive)
			{
				AVP_Inventory_Item*	pxCurrentItem	= GetItem(m_uCurrentItemHash);

				if(pxCurrentItem != NULL)
				{
					pxCurrentItem->RequestDeactivation(true);
				}
			}

			m_uCurrentItemHash	= m_uPendingItemHash;
			m_uPendingItemHash	= ASURA_HASH_ID_UNSET;
			ResetPickupItemData();
			break;
		}

		case DEACTIVATING_CURRENT_ITEM:
		{
			AVP_Inventory_Item*	pxCurrentItem	= GetItem(m_uCurrentItemHash);

			if(pxCurrentItem != NULL)
			{
				pxCurrentItem->RequestDeactivation(false);
			}
			break;
		}

		case SWAP_PENDING_ITEM_TO_CURRENT_ITEM:
		{
			m_uCurrentItemHash	= m_uPendingItemHash;
			m_uPendingItemHash	= ASURA_HASH_ID_UNSET;
			ResetPickupItemData();
			break;
		}

		case SWAP_PICKUP_ITEM_WITH_CURRENT_ITEM:
		{
			// Work out what type we want
			AVP_ClientActor_Vertebrate* pxClientActor = AVP_ClientActor_Vertebrate::FindEntity(GetParentSystem().GetOwnerGuid());
			if(pxClientActor)
			{
				bool bPickupFillsBothSlots = false;
				bool bPickupIsEncumberment = false;
				Asura_Blueprint* pxWeaponBP = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_INVENTORY_ITEMS, m_uPickupItemBlueprintHash );
				if ( pxWeaponBP != NULL )
				{
					int iWeaponSlot = pxWeaponBP->GetParameterAsInt( 0x86b33054, AVP_InventoryGroup_Weapons::AVP_INVENTORYWEAPON_SLOT_MIDDLE ); //WeaponSlotType
					bPickupFillsBothSlots = ( iWeaponSlot == AVP_InventoryGroup_Weapons::AVP_INVENTORYWEAPON_SLOT_TOP );
					int iItemType = pxWeaponBP->GetParameterAsInt( uAVP_BLUEPRINT_PARAM_WEAPON_TYPE, AVP_INVENTORYITEMTYPE_OTHER );
					bPickupIsEncumberment = ( iItemType == AVP_INVENTORYITEMTYPE_BATTERY || iItemType == AVP_INVENTORYITEMTYPE_TEQUILA );
				}

				// Done working out type

				Asura_Hash_ID uSlotToUse = AVP_Inventory_System::GetFirstAvailableWeaponSlotHash(&GetParentSystem());

				if(HasDualSlotWeapon() || uSlotToUse == ASURA_HASH_ID_UNSET || bPickupFillsBothSlots)
				{
					if(!GetCurrentInventoryItem()->IsEncumberment()) // Don't drop battery twice?
					{
						if(!bPickupIsEncumberment) // Don't drop when picking up battery/tequila
						{
							if(GetParentSystem().IsServerSide())
							{
								if(bPickupFillsBothSlots)
								{
									// Need to drop second weapon also!
									GetParentSystem().GetParentServer()->DropPickup(uAVP_INVENTORY_WEAPONHASH_WEAPON02); // Drop old weapon
									GetParentSystem().GetParentServer()->DropPickup(uAVP_INVENTORY_WEAPONHASH_WEAPON03); // Drop old weapon
								}
								else
								{
									GetParentSystem().GetParentServer()->DropPickup(); // Drop old weapon
								}
							}
						}
					}
				}

				AVP_Inventory_Item* pxNewItem = NULL;
				AVP_Inventory_Item*	pxCurrentItem = GetItem(m_uCurrentItemHash);

				if(!bPickupFillsBothSlots) // new item is normal
				{
					if(uSlotToUse != ASURA_HASH_ID_UNSET && !bPickupIsEncumberment && !(pxCurrentItem && pxCurrentItem->IsEncumberment())) // we think we have a slot free...
					{
						m_uCurrentItemHash = uSlotToUse;
						DestroyItemIfExists(m_uCurrentItemHash);
						pxNewItem = CreateItemFromBlueprint(m_uPickupItemBlueprintHash, m_uCurrentItemHash);
					}
					else
					{
						DestroyItemIfExists(m_uCurrentItemHash);
						pxNewItem = CreateItemFromBlueprint(m_uPickupItemBlueprintHash, m_uCurrentItemHash);
					}
				}
				else
				{
					// Clear out both weapon slots if filled, then select minigun
					DestroyItemIfExists(uAVP_INVENTORY_WEAPONHASH_WEAPON02);
					DestroyItemIfExists(uAVP_INVENTORY_WEAPONHASH_WEAPON03);

					m_uCurrentItemHash = uAVP_INVENTORY_WEAPONHASH_WEAPON02;
					pxNewItem = CreateItemFromBlueprint(m_uPickupItemBlueprintHash, m_uCurrentItemHash);
				}
				
				if(pxNewItem)
				{
					if(pxNewItem->GetNumSubTypes() > 0)
					{
						if(!pxNewItem->GetSubType(0)->IsDynamic())
						{
							pxNewItem->GetSubType(0)->SetCurrentQuantity(m_fQuantity);
						}
						
						if(pxNewItem->GetNumSubTypes() > 1)
						{
							if(!pxNewItem->GetSubType(0)->IsDynamic())
							{
								pxNewItem->GetSubType(1)->SetCurrentQuantity(m_fSecondaryQuantity);
							}
						}
					}
				}
			}
			ResetPickupItemData();
			break;
		}

		case ACTIVATING_CURRENT_ITEM:
		{
			if(m_uCurrentItemHash == ASURA_HASH_ID_UNSET)
			{
				m_uCurrentItemHash	= m_uPendingItemHash;
				m_uPendingItemHash	= ASURA_HASH_ID_UNSET;
				ResetPickupItemData();
			}

			AVP_Inventory_Item*	pxCurrentItem	= GetItem(m_uCurrentItemHash);

			if(pxCurrentItem != NULL)
			{
				pxCurrentItem->RequestActivation(false);
			}
			break;
		}

		case ITEM_ACTIVE:
		{
			const bool bWasInactive	= (uOldState == NO_ITEM_ACTIVE);

			if(bWasInactive)
			{
				AVP_Inventory_Item*	pxCurrentItem	= GetItem(m_uCurrentItemHash);

				if(pxCurrentItem != NULL)
				{
					pxCurrentItem->RequestActivation(true);
				}
			}
			break;
		}

		default:
		{
			Asura_Assert(false, "Unexpected state to change inventory-group to.");
			break;
		}
	}		// End of switch.
}

/// End of function 'AVP_Inventory_Group::OnStateChange'. ///

// if we have a smartgun, use that slot.
bool AVP_Inventory_Group::HasDualSlotWeapon(void) const
{
	const AVP_Inventory_Item* pxItem = GetItem(uAVP_INVENTORY_WEAPONHASH_WEAPON02);
	if(pxItem)
	{
		// For now there is only one type, base it on this, if more come along we will need a new blueprint parameter.
		return pxItem->GetItemType() == AVP_INVENTORYITEMTYPE_SMARTGUN;
	}
	return false;
}


////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Inventory_Group::RequestStateChange
// Author       : Andy Tate
// Date         : 25-Oct-2006
////////////////////////////////////////////////////////////////////////////////

bool AVP_Inventory_Group::RequestStateChange(const State eNewState)
{
	m_eNewState		= eNewState;
	m_bHasNewState	= true;
	return(true);
}

/// End of function 'AVP_Inventory_Group::RequestStateChange'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Inventory_Group::IsItemSelectable
// Author       : Andy Tate
// Date         : 2-Nov-2006
////////////////////////////////////////////////////////////////////////////////

bool AVP_Inventory_Group::IsItemSelectable(const Asura_Hash_ID uHash) const
{
	const AVP_Inventory_Item*	pxItem	= GetItem(uHash);

	if(pxItem == NULL)
	{
		return(false);
	}
	else if(m_bAllowSelectionOfEmptyItems)
	{
		return(true);
	}
	else
	{
		return(pxItem->IsSelectable());
	}
}

/// End of function 'AVP_Inventory_Group::IsItemSelectable'. ///

AVP_Inventory_Item* AVP_Inventory_Group::FindItemByBlueprintHash(Asura_Hash_ID uBlueprintHash) const
{
	Asura_TemplateHashTree_ConstIterator<AVP_Inventory_Item*> xIt(m_xItemTree);
	for ( ; !xIt.Done(); xIt.Next() )
	{
		AVP_Inventory_Item* pxItem = *(xIt.GetCurrent());

		Asura_Hash_ID uItBlueprintHash = pxItem->GetBlueprintHash();

		if(uItBlueprintHash == uBlueprintHash)
		{
			return pxItem;
		}
	}	
	return NULL;
}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Inventory_Group::IsPriorityGreaterThan
// Author       : Andy Tate
// Date         : 28-Nov-2007
////////////////////////////////////////////////////////////////////////////////

bool AVP_Inventory_Group::IsPriorityGreaterThan(const u_int uPriority1,
												const Asura_Hash_ID uHash1,
												const u_int uPriority2,
												const Asura_Hash_ID uHash2)
{
	return( (uPriority1 > uPriority2) || ((uPriority1 == uPriority2) && (uHash1 > uHash2)) );
}

/// End of function 'AVP_Inventory_Group::IsPriorityGreaterThan'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Inventory_Group::IsPriorityLessThan
// Author       : Andy Tate
// Date         : 28-Nov-2007
////////////////////////////////////////////////////////////////////////////////

bool AVP_Inventory_Group::IsPriorityLessThan(const u_int uPriority1,
											 const Asura_Hash_ID uHash1,
											 const u_int uPriority2,
											 const Asura_Hash_ID uHash2)
{
	return( (uPriority1 < uPriority2) || ((uPriority1 == uPriority2) && (uHash1 < uHash2)) );
}

/// End of function 'AVP_Inventory_Group::IsPriorityLessThan'. ///



////////////////////////////////////////////////////////////////////////////////
// Public functions...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Inventory_Group::AVP_Inventory_Group
// Author       : Andy Tate
// Date         : 25-Oct-2006
////////////////////////////////////////////////////////////////////////////////

AVP_Inventory_Group::AVP_Inventory_Group(AVP_Inventory_System& xParentSystem,
										 Asura_Char const* const szName,
										 const bool bPermanentlyActive)
: m_xParentSystem(xParentSystem)
, m_szName(szName)
, m_fTweenDuration(0.0f)
, m_uLastUsedItemHash(ASURA_HASH_ID_UNSET)
, m_uCurrentItemHash(ASURA_HASH_ID_UNSET)
, m_uPendingItemHash(ASURA_HASH_ID_UNSET)
, m_uPickupItemBlueprintHash(ASURA_HASH_ID_UNSET)
, m_uPreviousItemBlueprintHash(ASURA_HASH_ID_UNSET)
, m_fPreviousItemQuantity(0.0f)
, m_fPreviousItemSecondaryQuantity(0.0f)
, m_uDefaultItemHash(ASURA_HASH_ID_UNSET)
, m_fQuantity(0.0f)
, m_fSecondaryQuantity(0.0f)
, m_uFlags(0)
, m_fTimeInState(0.0f)
{
	const Asura_Hash_ID	uHash	= Asura_GetHashID(szName);

	m_eGroupType = NONE;

	// Register with the parent-system...
	m_uHashInParentSystem	= (xParentSystem.AddGroup(uHash, this)) ? uHash : ASURA_HASH_ID_UNSET;

	if(bPermanentlyActive)
	{
		m_eCurrentState	= PERMANENTLY_ACTIVE;
	}
}

/// End of function 'AVP_Inventory_Group::AVP_Inventory_Group'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Inventory_Group::~AVP_Inventory_Group
// Author       : Andy Tate
// Date         : 25-Oct-2006
////////////////////////////////////////////////////////////////////////////////

AVP_Inventory_Group::~AVP_Inventory_Group()
{
	Asura_TemplateHashTree_Iterator<AVP_Inventory_Item*>	xIterator(m_xItemTree);

	while(!xIterator.Done())
	{
		AVP_Inventory_Item**	ppxCurrentItem	= xIterator.GetCurrent();
		AVP_Inventory_Item*		pxCurrentItem	= (ppxCurrentItem == NULL) ? NULL : *ppxCurrentItem;

		delete(pxCurrentItem);
		xIterator.Next();
	}
}

/// End of function 'AVP_Inventory_Group::~AVP_Inventory_Group'. ///


////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Inventory_Group::AddItem
// Author       : Andy Tate
// Date         : 25-Oct-2006
////////////////////////////////////////////////////////////////////////////////

bool AVP_Inventory_Group::AddItem(const Asura_Hash_ID uHash,
								  AVP_Inventory_Item* pxNewItem)
{
	AVP_Inventory_Item**	ppxItem	= m_xItemTree.AddNode(uHash);

	// Did we already have this item?
	if(ppxItem == NULL)
	{
		Asura_Assert(false, "Cannot add item to inventory group - its already there.");
		return(false);
	}
	else
	{
		*ppxItem	= pxNewItem;

		if(IsPermanentlyActive())
		{
			pxNewItem->RequestActivation(true);
		}

		return(true);
	}
}

/// End of function 'AVP_Inventory_Group::AddItem'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Inventory_Group::DestroyItem
// Author       : Andy Tate
// Date         : 25-Oct-2006
////////////////////////////////////////////////////////////////////////////////

bool AVP_Inventory_Group::DestroyItemIfExists(const Asura_Hash_ID uHash)
{
	AVP_Inventory_Item**	ppxItem	= m_xItemTree.GetDataByHash(uHash);
	AVP_Inventory_Item*		pxItem	= (ppxItem == NULL) ? NULL : *ppxItem;

	// Did we not have this item?
	if(pxItem == NULL)
	{
		return(false);
	}
	else
	{
		delete(pxItem);
		return(m_xItemTree.DeleteNodeByHash(uHash));
	}
}

/// End of function 'AVP_Inventory_Group::DestroyItem'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Inventory_Group::DestroyAllItems
// Author       : Andy Tate
// Date         : 7-Dec-2007
////////////////////////////////////////////////////////////////////////////////

void AVP_Inventory_Group::DestroyAllItems(void)
{
	Asura_TemplateHashTree_Iterator<AVP_Inventory_Item*>	xIterator(m_xItemTree);

	while(!xIterator.Done())
	{
		AVP_Inventory_Item**	ppxItem	= xIterator.GetCurrent();
		AVP_Inventory_Item*		pxItem	= (ppxItem == NULL) ? NULL : *ppxItem;

		delete(pxItem);
		xIterator.Next();
	}
	m_xItemTree.Reset();
}

/// End of function 'AVP_Inventory_Group::DestroyAllItems'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Inventory_Group::ReadFromChunkStream
// Author       : Andy Tate
// Date         : 4-Feb-2008
////////////////////////////////////////////////////////////////////////////////

void AVP_Inventory_Group::ReadFromChunkStream(Asura_Chunk_Stream& xStream)
{
	u_int uVersion;
	xStream >> uVersion;

	if ( uVersion == uCURRENT_CHUNK_VERSION )
	{
		// Number of items in group
		u_int uNumItems;
		xStream >> uNumItems;
		Asura_Hash_ID uCurrentItemHash;
		xStream >> uCurrentItemHash;

		xStream >> m_uPreviousItemBlueprintHash;
		xStream >> m_fPreviousItemQuantity;
		xStream >> m_fPreviousItemSecondaryQuantity;

		int			  iItemType = NUM_AVP_INVENTORYITEMTYPES;
		Asura_Hash_ID uHashInParentGroup = ASURA_HASH_ID_UNSET;
		Asura_Hash_ID uBlueprintHash = ASURA_HASH_ID_UNSET;
		for ( u_int u = 0; u < uNumItems; ++u )
		{
			xStream >> iItemType;
			xStream >> uHashInParentGroup;
			xStream >> uBlueprintHash;

			AVP_Inventory_Item* pxItem = GetItem( uHashInParentGroup );
			if ( !pxItem )
			{
				pxItem = CreateItemFromType( iItemType, uBlueprintHash, uHashInParentGroup );
			}
			if ( pxItem )
			{
				pxItem->ReadFromChunkStream( xStream );
			}
			else
			{
				// We have to assume no-one's going to be removing types and classes,
				// at least without checking to see if any items of that type are saved out.
				// If this is going to happen we could save the size of the item and skip the data here.
				Asura_Assert( false, "AVP_Inventory_Item::ReadFromChunkStream - unable to create item" );
			}
		}

		SetCurrentItem( uCurrentItemHash, true );
	}
	else if ( uVersion == 3 )
	{
		// Number of items in group
		u_int uNumItems;
		xStream >> uNumItems;
		Asura_Hash_ID uCurrentItemHash;
		xStream >> uCurrentItemHash;
		xStream >> m_uPreviousItemBlueprintHash;

		int			  iItemType = NUM_AVP_INVENTORYITEMTYPES;
		Asura_Hash_ID uHashInParentGroup = ASURA_HASH_ID_UNSET;
		Asura_Hash_ID uBlueprintHash = ASURA_HASH_ID_UNSET;
		for ( u_int u = 0; u < uNumItems; ++u )
		{
			xStream >> iItemType;
			xStream >> uHashInParentGroup;
			xStream >> uBlueprintHash;

			AVP_Inventory_Item* pxItem = GetItem( uHashInParentGroup );
			if ( !pxItem )
			{
				pxItem = CreateItemFromType( iItemType, uBlueprintHash, uHashInParentGroup );
			}
			if ( pxItem )
			{
				pxItem->ReadFromChunkStream( xStream );
			}
			else
			{
				// We have to assume no-one's going to be removing types and classes,
				// at least without checking to see if any items of that type are saved out.
				// If this is going to happen we could save the size of the item and skip the data here.
				Asura_Assert( false, "AVP_Inventory_Item::ReadFromChunkStream - unable to create item" );
			}
		}

		SetCurrentItem( uCurrentItemHash, true );
	}
	else if ( uVersion == 2 )
	{
		// Number of items in group
		u_int uNumItems;
		xStream >> uNumItems;
		Asura_Hash_ID uCurrentItemHash;
		xStream >> uCurrentItemHash;

		int			  iItemType = NUM_AVP_INVENTORYITEMTYPES;
		Asura_Hash_ID uHashInParentGroup = ASURA_HASH_ID_UNSET;
		Asura_Hash_ID uBlueprintHash = ASURA_HASH_ID_UNSET;
		for ( u_int u = 0; u < uNumItems; ++u )
		{
			xStream >> iItemType;
			xStream >> uHashInParentGroup;
			xStream >> uBlueprintHash;

			AVP_Inventory_Item* pxItem = GetItem( uHashInParentGroup );
			if ( !pxItem )
			{
				pxItem = CreateItemFromType( iItemType, uBlueprintHash, uHashInParentGroup );
			}
			if ( pxItem )
			{
				pxItem->ReadFromChunkStream( xStream );
			}
			else
			{
				// We have to assume no-one's going to be removing types and classes,
				// at least without checking to see if any items of that type are saved out.
				// If this is going to happen we could save the size of the item and skip the data here.
				Asura_Assert( false, "AVP_Inventory_Item::ReadFromChunkStream - unable to create item" );
			}
		}

		SetCurrentItem( uCurrentItemHash, true );
	}
	else if ( uVersion == 1 )
	{
		u_int uNumItems = 0;
		Asura_Hash_ID uHashInParentGroup = ASURA_HASH_ID_UNSET;
		Asura_Hash_ID uBlueprintHash = ASURA_HASH_ID_UNSET;
		Asura_Hash_ID uCurrentItemHash = ASURA_HASH_ID_UNSET;

		// Number of items in group
		xStream >> uNumItems;
		xStream >> uCurrentItemHash;

		for( u_int u=0; u<uNumItems; u++ )
		{
			xStream >> uHashInParentGroup;
			xStream >> uBlueprintHash;

			AVP_Inventory_Item* pxItem = GetItem( uHashInParentGroup );
			if ( !pxItem )
			{
				pxItem = CreateItemFromBlueprint( uBlueprintHash, uHashInParentGroup );
			}
			if ( pxItem )
			{
				pxItem->ReadFromChunkStream( xStream );
			}
			else
			{
				// We have to assume no-one's going to be removing types and classes,
				// at least without checking to see if any items of that type are saved out.
				// If this is going to happen we could save the size of the item and skip the data here.
				Asura_Assert( false, "AVP_Inventory_Item::ReadFromChunkStream - unable to create item" );
			}
		}

		SetCurrentItem( uCurrentItemHash, true );
	}
	else if ( uVersion == 0 )
	{
	}
	else
	{
		Asura_Assertf( false, ( "AVP_Inventory_Group::ReadFromChunkStream - unhandled chunk version (%u)", uVersion ) );
	}
}

/// End of function 'AVP_Inventory_Group::ReadFromChunkStream'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Inventory_Group::WriteToChunkStream
// Author       : Andy Tate
// Date         : 4-Feb-2008
////////////////////////////////////////////////////////////////////////////////

void AVP_Inventory_Group::WriteToChunkStream(Asura_Chunk_Stream& xStream) const
{
	xStream << uCURRENT_CHUNK_VERSION;

	xStream << GetNumItems();

	Asura_Hash_ID uItemHash = (m_uPendingItemHash)?m_uPendingItemHash:m_uCurrentItemHash;
	xStream << uItemHash;

	xStream << m_uPreviousItemBlueprintHash;
	xStream << m_fPreviousItemQuantity;
	xStream << m_fPreviousItemSecondaryQuantity;

	Asura_TemplateHashTree_ConstIterator<AVP_Inventory_Item*>	xIt(m_xItemTree);

	while(!xIt.Done())
	{
		AVP_Inventory_Item* const*	ppxItem	= xIt.GetCurrent();
		AVP_Inventory_Item*			pxItem	= (ppxItem == NULL) ? NULL : *ppxItem;

		if(pxItem != NULL)
		{
			xStream << pxItem->GetItemType();
			xStream << pxItem->GetHashInParentGroup();
			xStream << pxItem->GetBlueprintHash();
			pxItem->WriteToChunkStream(xStream);
		}
		xIt.Next();
	}
}

/// End of function 'AVP_Inventory_Group::WriteToChunkStream'. ///


void AVP_Inventory_Group::PostLoadInitialise()
{
	Asura_TemplateHashTree_Iterator<AVP_Inventory_Item*> xIterator( m_xItemTree );
	while ( !xIterator.Done() )
	{
		AVP_Inventory_Item** ppxItem = xIterator.GetCurrent();
		AVP_Inventory_Item*	 pxItem  = (ppxItem == NULL) ? NULL : *ppxItem;
		if ( pxItem )
		{
			pxItem->PostLoadInitialise();
		}
		xIterator.Next();
	}
}


////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Inventory_Group::Activate
// Author       : Andy Tate
// Date         : 18-Apr-2008
////////////////////////////////////////////////////////////////////////////////

void AVP_Inventory_Group::Activate(void)
{
	if(IsPermanentlyActive())
	{
		Asura_TemplateHashTree_Iterator<AVP_Inventory_Item*>	xIterator(m_xItemTree);

		while(!xIterator.Done())
		{
			AVP_Inventory_Item**	ppxItem	= xIterator.GetCurrent();
			AVP_Inventory_Item*		pxItem	= (ppxItem == NULL) ? NULL : *ppxItem;

			if(pxItem != NULL)
			{
				pxItem->RequestActivation(true);
			}
			xIterator.Next();
		}
	}
	else
	{
		AVP_Inventory_Item*	pxCurrentItem	= GetCurrentInventoryItem();

		if(pxCurrentItem != NULL)
		{
			pxCurrentItem->RequestActivation(true);
		}
	}
}

/// End of function 'AVP_Inventory_Group::Activate'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Inventory_Group::Deactivate
// Author       : Andy Tate
// Date         : 18-Apr-2008
////////////////////////////////////////////////////////////////////////////////

void AVP_Inventory_Group::Deactivate(void)
{
	if(IsPermanentlyActive())
	{
		Asura_TemplateHashTree_Iterator<AVP_Inventory_Item*>	xIterator(m_xItemTree);

		while(!xIterator.Done())
		{
			AVP_Inventory_Item**	ppxItem	= xIterator.GetCurrent();
			AVP_Inventory_Item*		pxItem	= (ppxItem == NULL) ? NULL : *ppxItem;

			if(pxItem != NULL)
			{
				pxItem->RequestDeactivation(true);
			}
			xIterator.Next();
		}
	}
	else
	{
		AVP_Inventory_Item*	pxCurrentItem	= GetCurrentInventoryItem();

		if(pxCurrentItem != NULL)
		{
			pxCurrentItem->RequestDeactivation(true);
		}
	}
}

/// End of function 'AVP_Inventory_Group::Deactivate'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Inventory_Group::HandleMessage
// Author       : Andy Tate
// Date         : 15-Nov-2007
////////////////////////////////////////////////////////////////////////////////

bool AVP_Inventory_Group::HandleMessage(const Asura_Message* pxMessage)
{
	Asura_TemplateHashTree_Iterator<AVP_Inventory_Item*>	xIterator(m_xItemTree);

	bool													bMessageHandled	= false;

	while(!xIterator.Done())
	{
		const Asura_Hash_ID		uCurrentHash	= xIterator.GetCurrentHash();
		AVP_Inventory_Item**	ppxCurrentItem	= xIterator.GetCurrent();
		AVP_Inventory_Item*		pxCurrentItem	= (ppxCurrentItem == NULL) ? NULL : *ppxCurrentItem;

		if(pxCurrentItem != NULL)
		{
			const u_int	uMessageID	= pxMessage->GetMsgID();

			switch(uMessageID)
			{
				case AVP_MSGID_ITEM_BEGIN_IDLING:
				case AVP_MSGID_ITEM_BEGIN_USING:
				case AVP_MSGID_ITEM_REQUEST_END_USAGE:
				{
					const AVP_Message_InventoryItem*	pxData	= static_cast<const AVP_Message_InventoryItem*>(pxMessage->GetData());

					if(pxData->GetItemHash() == uCurrentHash)
					{
						bMessageHandled	|= pxCurrentItem->HandleMessage(pxMessage);
					}

					break;
				}

				case AVP_MSGID_INVENTORY_ITEM_RELOAD:
				case AVP_MSGID_ATTACH_WEAPON:
				case AVP_MSGID_DETACH_WEAPON:
				{
					bMessageHandled |= pxCurrentItem->HandleMessage(pxMessage);
					break;
				}

				default:
				{
					bMessageHandled	|= pxCurrentItem->HandleMessage(pxMessage);
					break;
				}
			}		// End of switch.
		}

		xIterator.Next();
	}

	return(bMessageHandled);
}

/// End of function 'AVP_Inventory_Group::HandleMessage'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Inventory_Group::Update
// Author       : Andy Tate
// Date         : 25-Oct-2006
////////////////////////////////////////////////////////////////////////////////

void AVP_Inventory_Group::Update(void)
{
	// Do we need to change state?
	if(m_bHasNewState)
	{
		do
		{
			m_bHasNewState	= false;
			m_fTimeInState	= 0.0f;

			OnStateChange(m_eCurrentState, m_eNewState);

			if(!m_bHasNewState)
			{
				m_eCurrentState	= m_eNewState;
			}

		} while(m_bHasNewState);
	}
	else
	{
		m_fTimeInState	+= Asura_Timers::GetGameTimeSlice();
	}

	// Decide what to do based on our current state...
	switch(m_eCurrentState)
	{
		case NO_ITEM_ACTIVE:
		{
			AVP_Inventory_Item*	pxCurrentItem	= GetItem(m_uCurrentItemHash);

			if(pxCurrentItem != NULL)
			{
				RequestStateChange(ACTIVATING_CURRENT_ITEM);
			}
			break;
		}

		case DEACTIVATING_CURRENT_ITEM:
		{
			AVP_Inventory_Item*	pxCurrentItem	= GetItem(m_uCurrentItemHash);

			if(pxCurrentItem == NULL)
			{
				Asura_Assert(false, "Current-item is NULL, we should be able to deactivate it.");
				RequestStateChange(SWAP_PENDING_ITEM_TO_CURRENT_ITEM);
			}
			else if(pxCurrentItem->IsDeactivating())
			{
				// Still deactivating - keep waiting...
			}
			else if(pxCurrentItem->IsInactive())
			{
				AVP_Inventory_Item*	pxPendingItem	= GetItem(m_uPendingItemHash);
			
				// We're either activating the appropriate item OR we're becoming inactive...
				if(pxPendingItem != NULL)
				{
					RequestStateChange(SWAP_PENDING_ITEM_TO_CURRENT_ITEM);
				}
				else if(m_uPickupItemBlueprintHash != ASURA_HASH_ID_UNSET)
				{
					RequestStateChange(SWAP_PICKUP_ITEM_WITH_CURRENT_ITEM);
				}
				else
				{
					RequestStateChange(NO_ITEM_ACTIVE);
				}
			}
			else
			{
				// we probably weren't able to deactivate our weapon last frame, try again now...
				pxCurrentItem->RequestDeactivation(false);
			}
			break;
		}

		case SWAP_PENDING_ITEM_TO_CURRENT_ITEM: // fall through
		case SWAP_PICKUP_ITEM_WITH_CURRENT_ITEM:
		{
			// Have we spent long enough tweening?
			if(m_fTimeInState >= m_fTweenDuration)
			{
				RequestStateChange(ACTIVATING_CURRENT_ITEM);
			}
			break;
		}

		case ACTIVATING_CURRENT_ITEM:
		{
			AVP_Inventory_Item*	pxCurrentItem	= GetItem(m_uCurrentItemHash);

			if(pxCurrentItem == NULL)
			{
				Asura_Assert(false, "Current-item is NULL, we should not be able to deactivate it.");
				RequestStateChange(NO_ITEM_ACTIVE);
			}
			else if(pxCurrentItem->IsActivating())
			{
				// Still activating - keep waiting...
			}
			else
			{
				RequestStateChange(ITEM_ACTIVE);
			}
			break;
		}

		case ITEM_ACTIVE:
		{
			AVP_Inventory_Item*	pxCurrentItem	= GetItem(m_uCurrentItemHash);

			if(pxCurrentItem == NULL)
			{
				RequestStateChange(NO_ITEM_ACTIVE);
			}
			break;
		}

		case PERMANENTLY_ACTIVE:
		{
			// Nothing to handle in this case...
			break;
		}

		default:
		{
			Asura_Assert(false, "Unexpected state when updating inventory-group.");
			break;
		}
	}		// End of switch.

	
	// Update each item in-turn...
	Asura_TemplateHashTree_Iterator<AVP_Inventory_Item*>	xIterator(m_xItemTree);
	while(!xIterator.Done())
	{
		AVP_Inventory_Item**	ppxCurrentItem	= xIterator.GetCurrent();
		AVP_Inventory_Item*		pxCurrentItem	= (ppxCurrentItem == NULL) ? NULL : *ppxCurrentItem;

		if(pxCurrentItem != NULL)
		{
			pxCurrentItem->Update();
		}

		xIterator.Next();
	}

	// Time to check whether we've run out of the current item...
	if(!m_bAllowSelectionOfEmptyItems)
	{
		const AVP_Inventory_Item*	pxCurrentItem	= GetItem(m_uCurrentItemHash);

		// Have we run out of the current item?
		if( (pxCurrentItem != NULL) &&
			pxCurrentItem->GetChangeOnZeroQuantity() &&
			pxCurrentItem->AreAllSubTypesEmpty() )
		{
			const AVP_Inventory_Item*	pxLastItem	= GetItem(m_uLastUsedItemHash);

			// Yes - and do we have enough of the last item to change to that?
			if((pxLastItem != NULL) && !pxLastItem->AreAllSubTypesEmpty())
			{
				// Yes - do so...
				SetCurrentItem(m_uLastUsedItemHash);
			}
			else
			{
				// No - remove all items...
				SetCurrentItem(ASURA_HASH_ID_UNSET);
			}
		}
	}
}

/// End of function 'AVP_Inventory_Group::Update'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Inventory_Group::Render
// Author       : Andy Tate
// Date         : 25-Oct-2006
////////////////////////////////////////////////////////////////////////////////

void AVP_Inventory_Group::Render(void) const
{
	Asura_TemplateHashTree_ConstIterator<AVP_Inventory_Item*>	xIterator(m_xItemTree);

	while(!xIterator.Done())
	{
		AVP_Inventory_Item const* const*	ppxCurrentItem	= xIterator.GetCurrent();
		AVP_Inventory_Item const*			pxCurrentItem	= (ppxCurrentItem == NULL) ? NULL : *ppxCurrentItem;

		if(pxCurrentItem != NULL)
		{
			pxCurrentItem->Render();
		}

		xIterator.Next();
	}
}

/// End of function 'AVP_Inventory_Group::Render'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Inventory_Group::FindHighestPriorityItem
// Author       : Andy Tate
// Date         : 25-Oct-2006
////////////////////////////////////////////////////////////////////////////////

Asura_Hash_ID AVP_Inventory_Group::FindHighestPriorityItem(void) const
{
	Asura_TemplateHashTree_ConstIterator<AVP_Inventory_Item*>	xIterator(m_xItemTree);

	Asura_Hash_ID												uBestMatchHash		= ASURA_HASH_ID_UNSET;
	u_int														uBestMatchPriority	= 0;

	while(!xIterator.Done())
	{
		const Asura_Hash_ID					uCurrentHash	= xIterator.GetCurrentHash();
		AVP_Inventory_Item const* const*	ppxCurrentItem	= xIterator.GetCurrent();
		AVP_Inventory_Item const*			pxCurrentItem	= (ppxCurrentItem == NULL) ? NULL : *ppxCurrentItem;

		if(pxCurrentItem != NULL)
		{
			const u_int	uCurrentPriority	= pxCurrentItem->GetPriority();

			// Do we already have a match?
			if( (uBestMatchHash == ASURA_HASH_ID_UNSET) ||
				IsPriorityGreaterThan(uCurrentPriority, uCurrentHash, uBestMatchPriority, uBestMatchHash) )
			{
				// No - then use this one...
				uBestMatchHash		= uCurrentHash;
				uBestMatchPriority	= uCurrentPriority;
			}
		}

		xIterator.Next();
	}

	return(uBestMatchHash);
}

/// End of function 'AVP_Inventory_Group::FindHighestPriorityItem'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Inventory_Group::FindLowestPriorityItem
// Author       : Andy Tate
// Date         : 25-Oct-2006
////////////////////////////////////////////////////////////////////////////////

Asura_Hash_ID AVP_Inventory_Group::FindLowestPriorityItem(void) const
{
	Asura_TemplateHashTree_ConstIterator<AVP_Inventory_Item*>	xIterator(m_xItemTree);

	Asura_Hash_ID												uBestMatchHash		= ASURA_HASH_ID_UNSET;
	u_int														uBestMatchPriority	= 0;

	while(!xIterator.Done())
	{
		const Asura_Hash_ID					uCurrentHash	= xIterator.GetCurrentHash();
		AVP_Inventory_Item const* const*	ppxCurrentItem	= xIterator.GetCurrent();
		AVP_Inventory_Item const*			pxCurrentItem	= (ppxCurrentItem == NULL) ? NULL : *ppxCurrentItem;

		if(pxCurrentItem != NULL)
		{
			const u_int	uCurrentPriority	= pxCurrentItem->GetPriority();

			// Do we already have a match?
			if( (uBestMatchHash == ASURA_HASH_ID_UNSET) ||
				IsPriorityLessThan(uCurrentPriority, uCurrentHash, uBestMatchPriority, uBestMatchHash) )
			{
				// No - then use this one...
				uBestMatchHash		= uCurrentHash;
				uBestMatchPriority	= uCurrentPriority;
			}
		}

		xIterator.Next();
	}

	return(uBestMatchHash);
}

/// End of function 'AVP_Inventory_Group::FindLowestPriorityItem'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Inventory_Group::FindNextHighestPriorityItem
// Author       : Andy Tate
// Date         : 25-Oct-2006
////////////////////////////////////////////////////////////////////////////////

Asura_Hash_ID AVP_Inventory_Group::FindNextHighestPriorityItem(const Asura_Hash_ID uHash) const
{
	AVP_Inventory_Item const*	pxInitialItem	= GetItem(uHash);

	if(pxInitialItem == NULL)
	{
		Asura_Assert(false, "Cannot find specified item.");
		return(ASURA_HASH_ID_UNSET);
	}
	else
	{
		Asura_TemplateHashTree_ConstIterator<AVP_Inventory_Item*>	xIterator(m_xItemTree);

		const u_int													uPriority			= pxInitialItem->GetPriority();
		Asura_Hash_ID												uBestMatchHash		= ASURA_HASH_ID_UNSET;
		u_int														uBestMatchPriority	= 0;

		while(!xIterator.Done())
		{
			const Asura_Hash_ID					uCurrentHash	= xIterator.GetCurrentHash();
			AVP_Inventory_Item const* const*	ppxCurrentItem	= xIterator.GetCurrent();
			AVP_Inventory_Item const*			pxCurrentItem	= (ppxCurrentItem == NULL) ? NULL : *ppxCurrentItem;

			if(pxCurrentItem != NULL)
			{
				const u_int	uCurrentPriority	= pxCurrentItem->GetPriority();

				// Do we already have a match?
				if( IsPriorityGreaterThan(uCurrentPriority, uCurrentHash, uPriority, uHash) &&
					((uBestMatchHash == ASURA_HASH_ID_UNSET) || IsPriorityLessThan(uCurrentPriority, uCurrentHash, uBestMatchPriority, uBestMatchHash)) )
				{
					// No - then use this one...
					uBestMatchHash		= uCurrentHash;
					uBestMatchPriority	= uCurrentPriority;
				}
			}

			xIterator.Next();
		}

		return(uBestMatchHash);
	}
}

/// End of function 'AVP_Inventory_Group::FindNextHighestPriorityItem'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Inventory_Group::FindNextLowestPriorityItem
// Author       : Andy Tate
// Date         : 25-Oct-2006
////////////////////////////////////////////////////////////////////////////////

Asura_Hash_ID AVP_Inventory_Group::FindNextLowestPriorityItem(const Asura_Hash_ID uHash) const
{
	AVP_Inventory_Item const*	pxInitialItem	= GetItem(uHash);

	if(pxInitialItem == NULL)
	{
		Asura_Assert(false, "Cannot find specified item.");
		return(ASURA_HASH_ID_UNSET);
	}
	else
	{
		Asura_TemplateHashTree_ConstIterator<AVP_Inventory_Item*>	xIterator(m_xItemTree);

		const u_int													uPriority			= pxInitialItem->GetPriority();
		Asura_Hash_ID												uBestMatchHash		= ASURA_HASH_ID_UNSET;
		u_int														uBestMatchPriority	= 0;

		while(!xIterator.Done())
		{
			const Asura_Hash_ID					uCurrentHash	= xIterator.GetCurrentHash();
			AVP_Inventory_Item const* const*	ppxCurrentItem	= xIterator.GetCurrent();
			AVP_Inventory_Item const*			pxCurrentItem	= (ppxCurrentItem == NULL) ? NULL : *ppxCurrentItem;

			if(pxCurrentItem != NULL)
			{
				const u_int	uCurrentPriority	= pxCurrentItem->GetPriority();

				// Do we already have a match?
				if( IsPriorityLessThan(uCurrentPriority, uCurrentHash, uPriority, uHash) &&
					((uBestMatchHash == ASURA_HASH_ID_UNSET) || IsPriorityLessThan(uCurrentPriority, uCurrentHash, uBestMatchPriority, uBestMatchHash)) )
				{
					// No - then use this one...
					uBestMatchHash		= uCurrentHash;
					uBestMatchPriority	= uCurrentPriority;
				}
			}

			xIterator.Next();
		}

		return(uBestMatchHash);
	}
}

/// End of function 'AVP_Inventory_Group::FindNextLowestPriorityItem'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Inventory_Group::SwapCurrentItemForPickup
// Author       : Chris Wallace
// Date         : 15-Sept-2008
////////////////////////////////////////////////////////////////////////////////

bool AVP_Inventory_Group::SwapCurrentItemForPickup(const u_int uPickupHash, const float fQuantity, const float fSecondaryQuantity)
{
	if(uPickupHash == ASURA_HASH_ID_UNSET) return false;

	if(GetCanSetNewItem(false))
	{
		m_uPickupItemBlueprintHash = uPickupHash;
		m_fQuantity = fQuantity;
		m_fSecondaryQuantity = fSecondaryQuantity;

		AVP_Inventory_Item*	pxCurrentItem = GetItem(m_uCurrentItemHash);

		if(pxCurrentItem == NULL)
		{
			RequestStateChange(ACTIVATING_CURRENT_ITEM);
		}
		else
		{
			m_uPreviousItemBlueprintHash = pxCurrentItem->GetCommonItemHash();
			m_fPreviousItemQuantity = (pxCurrentItem->GetNumSubTypes()>0)?pxCurrentItem->GetSubType(0)->GetCurrentQuantity():0.0f;
			m_fPreviousItemSecondaryQuantity = (pxCurrentItem->GetNumSubTypes()>1)?pxCurrentItem->GetSubType(1)->GetCurrentQuantity():0.0f;
			RequestStateChange(DEACTIVATING_CURRENT_ITEM);
		}
		return true;
	}
	return false;
}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Inventory_Group::AddPickup (Won't swap if there is space ) 
// Author       : Ali Jakamy
// Date         : 10-March-2009
////////////////////////////////////////////////////////////////////////////////

bool AVP_Inventory_Group::AddPickup(const u_int uPickupHash, const float fQuantity, const float fSecondaryQuantity)
{
	if(uPickupHash != ASURA_HASH_ID_UNSET)
	{
		Asura_Hash_ID uSlotToUse            = AVP_Inventory_System::GetFirstAvailableWeaponSlotHash(&GetParentSystem());
		bool          bPickupFillsBothSlots = false;
		bool          bPickupIsEncumberment = false;

		Asura_Blueprint* pxWeaponBP = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_INVENTORY_ITEMS, uPickupHash );
		if ( pxWeaponBP )
		{
			int iWeaponSlot       = pxWeaponBP->GetParameterAsInt( 0x86b33054, AVP_InventoryGroup_Weapons::AVP_INVENTORYWEAPON_SLOT_MIDDLE ); //WeaponSlotType
			bPickupFillsBothSlots = ( iWeaponSlot == AVP_InventoryGroup_Weapons::AVP_INVENTORYWEAPON_SLOT_TOP );
			int iItemType         = pxWeaponBP->GetParameterAsInt( uAVP_BLUEPRINT_PARAM_WEAPON_TYPE, AVP_INVENTORYITEMTYPE_OTHER );
			bPickupIsEncumberment = ( iItemType == AVP_INVENTORYITEMTYPE_BATTERY || iItemType == AVP_INVENTORYITEMTYPE_TEQUILA );
		}

		if ( uSlotToUse == ASURA_HASH_ID_UNSET || bPickupFillsBothSlots || bPickupIsEncumberment )
		{
			// No slot free, forced to swap
			return SwapCurrentItemForPickup(uPickupHash, fQuantity, fSecondaryQuantity);
		}
		else
		{
			DestroyItemIfExists(uSlotToUse);
			AVP_Inventory_Item* pxNewItem = CreateItemFromBlueprint(uPickupHash, uSlotToUse);
			if ( pxNewItem )
			{
				if(pxNewItem)
				{
					if(pxNewItem->GetNumSubTypes() > 0)
					{
						if(!pxNewItem->GetSubType(0)->IsDynamic())
						{
							pxNewItem->GetSubType(0)->SetCurrentQuantity(fQuantity);
						}

						if(pxNewItem->GetNumSubTypes() > 1)
						{
							if(!pxNewItem->GetSubType(0)->IsDynamic())
							{
								pxNewItem->GetSubType(1)->SetCurrentQuantity(fSecondaryQuantity);
							}
						}
					}
				}
			}
			return true;
		}
	}
	return false;
}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Inventory_Group::SetCurrentItem
// Author       : Andy Tate
// Date         : 25-Oct-2006
////////////////////////////////////////////////////////////////////////////////

bool AVP_Inventory_Group::SetCurrentItem(const u_int uHash,
										 const bool bInstant)
{
	if(m_uPickupItemBlueprintHash != ASURA_HASH_ID_UNSET) return false;

	// Already on this weapon?
	if((uHash == m_uPendingItemHash) || (m_uPendingItemHash == ASURA_HASH_ID_UNSET && m_uCurrentItemHash == uHash) )
	{
		return(true);
	}
	else if ( m_uPendingItemHash != ASURA_HASH_ID_UNSET )
	{
		return(false);
	}
	else
	{
		if(GetCanSetNewItem(bInstant))
		{
			AVP_Inventory_Item*	pxCurrentItem	= GetItem(m_uCurrentItemHash);
			AVP_Inventory_Item*	pxNewItem		= GetItem(uHash);

			if(bInstant)
			{
				if((pxCurrentItem != NULL) && !pxCurrentItem->RequestDeactivation(true))
				{
					Asura_Assert(false, "Failed to deactivate current item");
					return(false);
				}
				else if(pxNewItem == NULL)
				{
					return(false);
				}
				else if(pxNewItem->RequestActivation(true))
				{
					m_uLastUsedItemHash	= m_uCurrentItemHash;
					m_uPendingItemHash	= ASURA_HASH_ID_UNSET;
					m_uPickupItemBlueprintHash	= ASURA_HASH_ID_UNSET;
					m_uCurrentItemHash	= uHash;

					RequestStateChange(ITEM_ACTIVE);
					return(true);
				}
				else
				{
					Asura_Assert(false, "Failed to activate new item.");
					return(false);
				}
			}
			else
			{
				m_uPendingItemHash	= uHash;
				ResetPickupItemData();

				if(pxCurrentItem == NULL)
				{
					if(pxNewItem == NULL)
					{
						Asura_Assert((uHash == ASURA_HASH_ID_UNSET), "Cannot find specified item to set as current.");
					}
					else
					{
						RequestStateChange(ACTIVATING_CURRENT_ITEM);
					}
				}
				else
				{
					RequestStateChange(DEACTIVATING_CURRENT_ITEM);
				}
				return(true);
			}
		}
		else
		{
			return(false);
		}
	}
}

/// End of function 'AVP_Inventory_Group::SetCurrentItem'. ///

void AVP_Inventory_Group::ResetPickupItemData(void)
{
	m_uPickupItemBlueprintHash = ASURA_HASH_ID_UNSET;
	m_fQuantity = 0.0f;
}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Inventory_Group::GetNumSelectableItems
// Author       : Andy Tate
// Date         : 3-Jan-2007
////////////////////////////////////////////////////////////////////////////////

u_int AVP_Inventory_Group::GetNumSelectableItems(void) const
{
	Asura_TemplateHashTree_ConstIterator<AVP_Inventory_Item*>	xIterator(m_xItemTree);

	u_int														uCount	= 0;

	while(!xIterator.Done())
	{
		AVP_Inventory_Item const* const*	ppxCurrentItem	= xIterator.GetCurrent();
		AVP_Inventory_Item const*			pxCurrentItem	= (ppxCurrentItem == NULL) ? NULL : *ppxCurrentItem;

		if((pxCurrentItem != NULL) && pxCurrentItem->IsSelectable())
		{
			uCount++;
		}

		xIterator.Next();
	}
	return(uCount);
}

/// End of function 'AVP_Inventory_Group::GetNumSelectableItems'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Inventory_Group::GetNumActiveItems
// Author       : Chris Wallace
// Date         : 2-Feb-2007
////////////////////////////////////////////////////////////////////////////////

u_int AVP_Inventory_Group::GetNumInactiveItems(void) const
{
	Asura_TemplateHashTree_ConstIterator<AVP_Inventory_Item*>	xIterator(m_xItemTree);

	u_int														uCount	= 0;

	while(!xIterator.Done())
	{
		AVP_Inventory_Item const* const*	ppxCurrentItem	= xIterator.GetCurrent();
		AVP_Inventory_Item const*			pxCurrentItem	= (ppxCurrentItem == NULL) ? NULL : *ppxCurrentItem;

		if((pxCurrentItem != NULL) && pxCurrentItem->IsInactive())
		{
			uCount++;
		}

		xIterator.Next();
	}
	return(uCount);
}

/// End of function 'AVP_Inventory_Group::GetNumActiveItems'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Inventory_Group::GetNextItem
// Author       : Andy Tate
// Date         : 26-Oct-2006
////////////////////////////////////////////////////////////////////////////////

Asura_Hash_ID AVP_Inventory_Group::GetNextItem(const Asura_Hash_ID uCurrentHash,
											   const bool bAllowWrap,
											   const bool bAllowUnselectable /*= false*/) const
{
	Asura_Hash_ID	uHash		= uCurrentHash;
	Asura_Hash_ID	uNextHash	= FindNextHighestPriorityItem(uHash);

	// Check up to the end of the list...
	while(uNextHash != ASURA_HASH_ID_UNSET)
	{
		// Did we find a selectable item?
		if(bAllowUnselectable || IsItemSelectable(uNextHash))
		{
			return(uNextHash);
		}
		else
		{
			uHash		= uNextHash;
			uNextHash	= FindNextHighestPriorityItem(uHash);
		}
	}

	// Are we allowed to wrap around to the start of the list?
	if(bAllowWrap)
	{
		// No - try from the start of the list...
		uNextHash	= FindLowestPriorityItem();

		while(uNextHash != m_uCurrentItemHash)
		{
			// Did we find a selectable item?
			if(bAllowUnselectable || IsItemSelectable(uNextHash))
			{
				return(uNextHash);
			}
			else
			{
				uHash		= uNextHash;
				uNextHash	= FindNextHighestPriorityItem(uHash);
			}
		}
	}

	// Failed to find a match...
	return(ASURA_HASH_ID_UNSET);
}

/// End of function 'AVP_Inventory_Group::GetNextItem'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Inventory_Group::GetPreviousItem
// Author       : Andy Tate
// Date         : 26-Oct-2006
////////////////////////////////////////////////////////////////////////////////

Asura_Hash_ID AVP_Inventory_Group::GetPreviousItem(const Asura_Hash_ID uCurrentHash,
												   const bool bAllowWrap) const
{
	Asura_Hash_ID	uHash		= uCurrentHash;
	Asura_Hash_ID	uNextHash	= FindNextLowestPriorityItem(uHash);

	// Check up to the start of the list...
	while(uNextHash != ASURA_HASH_ID_UNSET)
	{
		// Did we find a selectable item?
		if(IsItemSelectable(uNextHash))
		{
			return(uNextHash);
		}
		else
		{
			uHash		= uNextHash;
			uNextHash	= FindNextLowestPriorityItem(uHash);
		}
	}

	// Are we allowed to wrap around to the end of the list?
	if(bAllowWrap)
	{
		// No - try from the end of the list...
		uNextHash	= FindHighestPriorityItem();

		while(uNextHash != m_uCurrentItemHash)
		{
			// Did we find a selectable item?
			if(IsItemSelectable(uNextHash))
			{
				return(uNextHash);
			}
			else
			{
				uHash		= uNextHash;
				uNextHash	= FindNextLowestPriorityItem(uHash);
			}
		}
	}

	// Failed to find a match...
	return(ASURA_HASH_ID_UNSET);
}

/// End of function 'AVP_Inventory_Group::GetPreviousItem'. ///

AVP_Inventory_Item* AVP_Inventory_Group::CreateItemFromBlueprint( const Asura_Hash_ID uBlueprintHash, const Asura_Hash_ID uNameHash )
{
	Asura_Assertf( false, ( "AVP_Inventory_Group::CreateItemFromBlueprint - unable to create item from blueprint %u", uBlueprintHash ) );
	return NULL;
}

AVP_Inventory_Item* AVP_Inventory_Group::CreateItemFromType( const int iType, const Asura_Hash_ID uBlueprintHash, const Asura_Hash_ID uNameHash )
{
	switch ( iType )
	{
	case AVP_INVENTORYITEMTYPE_TORCH:				return new AVP_InventoryItem_Torch( *this, uBlueprintHash, "Torch" );
	case AVP_INVENTORYITEMTYPE_FLARE:				return new AVP_InventoryItem_HumanFlare( *this, uBlueprintHash, "Flare" );
	case AVP_INVENTORYITEMTYPE_STIM:				return new AVP_InventoryItem_Stim( *this, uBlueprintHash, "Stim");
	case AVP_INVENTORYITEMTYPE_CLOAK:				return new AVP_InventoryItem_Cloak( *this );
	default:										
		Asura_Assertf( false, ( "AVP_Inventory_Group::CreateItemFromType - unable to create item of type %d", iType ) );
		return NULL;
	}
}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Inventory_Group_Enumerator::BeginEnumeration
// Author       : Andy Tate
// Date         : 22-Jan-2008
////////////////////////////////////////////////////////////////////////////////

void AVP_Inventory_Group_Enumerator::BeginEnumeration(void)
{
	// Reset the data...
	m_pxFirstItem	= NULL;
}

/// End of function 'AVP_Inventory_Group_Enumerator::BeginEnumeration'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Inventory_Group_Enumerator::Enumerate
// Author       : Andy Tate
// Date         : 22-Jan-2008
////////////////////////////////////////////////////////////////////////////////

bool AVP_Inventory_Group_Enumerator::Enumerate(const u_int uIndex,
											   const char* szName,
											   const Asura_Value& xValue)
{
	if ( uIndex >= m_uMaxItems ) // don't add more than m_uMaxItems items
	{
		return false;
	}

	const Asura_Hash_ID	uItemBlueprintHash	= xValue.GetHashIDValue();

	if ( uItemBlueprintHash == ASURA_HASH_ID_UNSET )
	{
		return true; // allow selection of UNSET in the enumerated parameters.
	}

	AVP_Inventory_Item*	pxNewItem	= m_xGroup.CreateItemFromBlueprint(uItemBlueprintHash, Asura_GetHashID(szName) );

	if(pxNewItem == NULL)
	{
		Asura_Assertf(false, ("Failed to create an inventory-item for blueprint \"%s\".", szName));
		return(false);
	}
	else
	{
		if(m_pxFirstItem == NULL)
		{
			m_pxFirstItem	= pxNewItem;
		}
		pxNewItem->SetPriority( uIndex );
		return(true);
	}
}

/// End of function 'AVP_Inventory_Group_Enumerator::Enumerate'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Inventory_Group_Enumerator::EndEnumeration
// Author       : Andy Tate
// Date         : 22-Jan-2008
////////////////////////////////////////////////////////////////////////////////

void AVP_Inventory_Group_Enumerator::EndEnumeration(const bool bSuccess)
{
	// No - that's the end of our list...
	if(bSuccess && m_bSetFirstItemAsDefault && (m_pxFirstItem != NULL))
	{
		const Asura_Hash_ID	uFirstItemHash	= m_pxFirstItem->GetHashInParentGroup();

		m_xGroup.SetDefaultItem(uFirstItemHash);
	}
}

/// End of function 'AVP_Inventory_Group_Enumerator::EndEnumeration'. ///


/// End of file. ///
