
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_Inventory_System.cpp
// Author : Andy Tate
// Date   : 25-Oct-2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "Inventory/AVP_Inventory_System.h"

// Asura independent header-files...
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "Network/Asura_Message.h"

// Game independent header-files...
#include "Core/AVP_Game_Core.h"
#include "Entity/AVP_Messages.h"
#include "Entity/PhysicalObject/Actor/AVP_ClientEntity_Actor.h"
#include "Entity/PhysicalObject/Actor/AVP_ServerEntity_Actor.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/AVP_ClientActor_Vertebrate.h"
#include "Inventory/AVP_AmmoStore.h"
#include "Inventory/AVP_Inventory_Group.h"
#include "Inventory/AVP_Inventory_HashIDs.h"
#include "Inventory/AVP_InventoryItem_Messages.h"
#include "Inventory/Items/AVP_InventoryItem_Weapon.h"
#include "Inventory/Groups/AVP_InventoryGroup_Weapons.h"
#include "Inventory/Groups/AVP_InventoryGroup_VisionModes.h"


////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Private functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Protected functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Public functions...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Inventory_System::AVP_Inventory_System
// Author       : Andy Tate
// Date         : 27-Nov-2007
////////////////////////////////////////////////////////////////////////////////

AVP_Inventory_System::AVP_Inventory_System(AVP_ClientEntity_Actor& xParentClient)
: m_uOwnerGuid(xParentClient.GetGuid())
, m_pxParentClient(&xParentClient)
, m_pxParentServer(NULL)
, m_pxAmmoStore(NULL)
, m_uFlags(0)
{
	m_pxAmmoStore = new AVP_AmmoStore();
}

/// End of function 'AVP_Inventory_System::AVP_Inventory_System'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Inventory_System::AVP_Inventory_System
// Author       : Andy Tate
// Date         : 27-Nov-2007
////////////////////////////////////////////////////////////////////////////////

AVP_Inventory_System::AVP_Inventory_System(AVP_ServerEntity_Actor& xParentServer)
: m_uOwnerGuid(xParentServer.GetGuid())
, m_pxParentClient(NULL)
, m_pxParentServer(&xParentServer)
, m_pxAmmoStore(NULL)
, m_uFlags(0)
{
	m_pxAmmoStore = new AVP_AmmoStore();
}

/// End of function 'AVP_Inventory_System::AVP_Inventory_System'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Inventory_System::~AVP_Inventory_System
// Author       : Andy Tate
// Date         : 25-Oct-2006
////////////////////////////////////////////////////////////////////////////////

AVP_Inventory_System::~AVP_Inventory_System()
{
	if(m_pxAmmoStore)
	{
		delete(m_pxAmmoStore);
		m_pxAmmoStore = NULL;
	}

	Reset();
}

/// End of function 'AVP_Inventory_System::~AVP_Inventory_System'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Inventory_System::Activate
// Author       : Andy Tate
// Date         : 18-Apr-2008
////////////////////////////////////////////////////////////////////////////////

void AVP_Inventory_System::Activate(void)
{
	AVP_Inventory_Group*									pxPermanentlyActiveGroup	= GetPermanentlyActiveItemGroup();
	Asura_TemplateHashTree_Iterator<AVP_Inventory_Group*>	xIterator(m_xGroupTree);

	// Activate ALL of the groups in the tree...
	while(!xIterator.Done())
	{
		AVP_Inventory_Group**	ppxCurrentGroup	= xIterator.GetCurrent();
		AVP_Inventory_Group*	pxCurrentGroup	= (ppxCurrentGroup == NULL) ? NULL : *ppxCurrentGroup;

		if(pxCurrentGroup != pxPermanentlyActiveGroup)
		{
			pxCurrentGroup->Activate();
		}
		xIterator.Next();
	}
}

/// End of function 'AVP_Inventory_System::Activate'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Inventory_System::Deactivate
// Author       : Andy Tate
// Date         : 18-Apr-2008
////////////////////////////////////////////////////////////////////////////////

void AVP_Inventory_System::Deactivate(void)
{
	Asura_TemplateHashTree_Iterator<AVP_Inventory_Group*>	xIterator(m_xGroupTree);

	// Destroy ALL of the groups in the tree...
	while(!xIterator.Done())
	{
		AVP_Inventory_Group**	ppxCurrentGroup	= xIterator.GetCurrent();
		AVP_Inventory_Group*	pxCurrentGroup	= (ppxCurrentGroup == NULL) ? NULL : *ppxCurrentGroup;

		pxCurrentGroup->Deactivate();
		xIterator.Next();
	}
}

/// End of function 'AVP_Inventory_System::Deactivate'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Inventory_System::Reset
// Author       : Andy Tate
// Date         : 4-Feb-2008
////////////////////////////////////////////////////////////////////////////////

void AVP_Inventory_System::Reset(void)
{
	Asura_TemplateHashTree_Iterator<AVP_Inventory_Group*>	xIterator(m_xGroupTree);

	// Destroy ALL of the groups in the tree...
	while(!xIterator.Done())
	{
		AVP_Inventory_Group**	ppxCurrentGroup	= xIterator.GetCurrent();
		AVP_Inventory_Group*	pxCurrentGroup	= (ppxCurrentGroup == NULL) ? NULL : *ppxCurrentGroup;

		delete(pxCurrentGroup);
		xIterator.Next();
	}
	m_xGroupTree.Reset();
}

/// End of function 'AVP_Inventory_System::Reset'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Inventory_System::AddGroup
// Author       : Andy Tate
// Date         : 25-Oct-2006
////////////////////////////////////////////////////////////////////////////////

bool AVP_Inventory_System::AddGroup(const Asura_Hash_ID uHash,
									AVP_Inventory_Group* pxNewGroup)
{
	AVP_Inventory_Group**	ppxGroup	= m_xGroupTree.AddNode(uHash);

	// Did we already have this group?
	if(ppxGroup == NULL)
	{
		Asura_Assert(false, "Cannot add group to inventory-system - its already there.");
		return(false);
	}
	else
	{
		*ppxGroup	= pxNewGroup;
		return(true);
	}
}

/// End of function 'AVP_Inventory_System::AddGroup'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Inventory_System::DestroyGroup
// Author       : Andy Tate
// Date         : 25-Oct-2006
////////////////////////////////////////////////////////////////////////////////

bool AVP_Inventory_System::DestroyGroup(const Asura_Hash_ID uHash)
{
	AVP_Inventory_Group**	ppxGroup	= m_xGroupTree.GetDataByHash(uHash);
	AVP_Inventory_Group*	pxGroup		= (ppxGroup == NULL) ? NULL : *ppxGroup;

	// Did we already have this group?
	if(pxGroup)
	{
		delete(pxGroup);
		return(m_xGroupTree.DeleteNodeByHash(uHash));
	}
	return(false);
}

/// End of function 'AVP_Inventory_System::DestroyGroup'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Inventory_System::DestroyAllGroups
// Author       : Andy Tate
// Date         : 7-Dec-2007
////////////////////////////////////////////////////////////////////////////////

void AVP_Inventory_System::DestroyAllGroups(void)
{
	Asura_TemplateHashTree_Iterator<AVP_Inventory_Group*>	xIterator(m_xGroupTree);

	while(!xIterator.Done())
	{
		AVP_Inventory_Group**	ppxGroup	= xIterator.GetCurrent();
		AVP_Inventory_Group*	pxGroup		= (ppxGroup == NULL) ? NULL : *ppxGroup;

		delete(pxGroup);
		xIterator.Next();
	}
	m_xGroupTree.Reset();
}

/// End of function 'AVP_Inventory_System::DestroyAllGroups'. ///


////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Inventory_System::ReadFromChunkStream
// Author       : Andy Tate
// Date         : 4-Feb-2008
////////////////////////////////////////////////////////////////////////////////

void AVP_Inventory_System::ReadFromChunkStream(Asura_Chunk_Stream& xStream)
{
	// Ensure correct settings after actor chunk reading.
	if( (IsServerSide() && !GetParentServer()->IsPlayer()) || (IsClientSide() && IsLocalPlayer()))
	{
		SetAsMaster();
	}
	else
	{
		SetAsSlave();
	}

	u_int	uVersion;

	// Most importantly - handle the versioning first...
	xStream >> uVersion;

	if(uVersion == uCURRENT_CHUNK_VERSION)
	{
		GetAmmoStore()->ReadFromChunkStream(xStream);

		u_int uNumGroups = 0;
		xStream >> uNumGroups; // Number of groups

		for(u_int u=0;u<uNumGroups;u++)
		{
			Asura_Hash_ID uGroupHash = ASURA_HASH_ID_UNSET;
			u_int uGroupType = AVP_Inventory_Group::NONE;
			bool bChunkingIgnored = false;

			xStream >> bChunkingIgnored;

			if(!bChunkingIgnored)
			{
				xStream >> uGroupHash;
				xStream >> uGroupType;
				
				AVP_Inventory_Group* pxGroup = GetGroup(uGroupHash);
				if(pxGroup)
				{					
					if(pxGroup != GetPermanentlyActiveItemGroup())
					{
						pxGroup->DestroyAllItems(); // remove all items, we only want those in the chunk.
					}
					pxGroup->ReadFromChunkStream(xStream);
				}
				else
				{
					switch(uGroupType)
					{
						case AVP_Inventory_Group::NONE:
						{
							pxGroup = GetPermanentlyActiveItemGroup();
							AddPermenantlyActiveItems(pxGroup);
						}
						break;

						case AVP_Inventory_Group::WEAPON:
						{
							pxGroup = new AVP_InventoryGroup_Weapons(*this);
						}
						break;

						case AVP_Inventory_Group::VISION_MODE:
						{
							pxGroup = new AVP_InventoryGroup_VisionModes(*this);
						}
						break;

						default: break;
					}
					if(pxGroup)
					{
						pxGroup->ReadFromChunkStream(xStream);
					}
					else
					{
						Asura_Assert(false,"AVP_Inventory_System::ReadFromChunkStream(): Group was not found and unable to be created. Should it have been ignored for chunking?");
					}
				}
			}
		}
	}
	else if (uVersion == 1)
	{
		GetAmmoStore()->ReadFromChunkStream(xStream);
	}
	else if (uVersion == 0)
	{
		// Version 0 didn't write anything but the version number
	}
	else
	{
		Asura_Assertf( false, ( "AVP_Inventory_System::ReadFromChunkStream - unhandled chunk version (%u)", uVersion ) );
	}
}

/// End of function 'AVP_Inventory_System::ReadFromChunkStream'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Inventory_System::WriteToChunkStream
// Author       : Andy Tate
// Date         : 4-Feb-2008
////////////////////////////////////////////////////////////////////////////////

void AVP_Inventory_System::WriteToChunkStream(Asura_Chunk_Stream& xStream) const
{
// #ifdef ASURA_TOOLS
// 	xStream.SetSwapEndian(false);
// #endif

	// Firstly, write out the version...
	xStream << uCURRENT_CHUNK_VERSION;

	GetAmmoStore()->WriteToChunkStream(xStream);

	xStream << m_xGroupTree.GetNumberOfNodes(); // Number of groups

	Asura_TemplateHashTree_ConstIterator<AVP_Inventory_Group*>	xIt(m_xGroupTree); // Write out each group
	while(!xIt.Done())
	{
		AVP_Inventory_Group* const*	ppxGroup = xIt.GetCurrent();
		AVP_Inventory_Group*		pxGroup = (ppxGroup == NULL) ? NULL : *ppxGroup;

		if(pxGroup != NULL)
		{
			xStream << pxGroup->IsIgnoredByChunkStream();
			
			if(!pxGroup->IsIgnoredByChunkStream())
			{
				xStream << pxGroup->GetHashInParentSystem();
				xStream << pxGroup->GetGroupType();
				pxGroup->WriteToChunkStream(xStream);	
			}
		}
		xIt.Next();
	}
}

/// End of function 'AVP_Inventory_System::WriteToChunkStream'. ///


void AVP_Inventory_System::PostLoadInitialise()
{
	Asura_TemplateHashTree_Iterator<AVP_Inventory_Group*> xIterator(m_xGroupTree);
	while ( !xIterator.Done() )
	{
		AVP_Inventory_Group**	ppxCurrentGroup	= xIterator.GetCurrent();
		AVP_Inventory_Group*	pxCurrentGroup	= (ppxCurrentGroup == NULL) ? NULL : *ppxCurrentGroup;
		if ( pxCurrentGroup )
		{
			pxCurrentGroup->PostLoadInitialise();
		}
		xIterator.Next();
	}
}


////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Inventory_System::HandleMessage
// Author       : Andy Tate
// Date         : 30-Nov-2007
////////////////////////////////////////////////////////////////////////////////

bool AVP_Inventory_System::HandleMessage(const Asura_Message* pxMessage)
{
	Asura_TemplateHashTree_Iterator<AVP_Inventory_Group*>	xIterator(m_xGroupTree);
	bool													bMessageHandled	= false;

	// Try each group in-turn...
	while(!xIterator.Done())
	{
		const Asura_Hash_ID		uCurrentHash	= xIterator.GetCurrentHash();
		AVP_Inventory_Group**	ppxCurrentGroup	= xIterator.GetCurrent();
		AVP_Inventory_Group*	pxCurrentGroup	= (ppxCurrentGroup == NULL) ? NULL : *ppxCurrentGroup;

		if(pxCurrentGroup != NULL)
		{
			const u_int	uMessageID	= pxMessage->GetMsgID();

			switch(uMessageID)
			{
				case AVP_MSGID_ITEM_BEGIN_IDLING:
				case AVP_MSGID_ITEM_BEGIN_USING:
				{
					const AVP_Message_InventoryItem*	pxData	= static_cast<const AVP_Message_InventoryItem*>(pxMessage->GetData());

					if(pxData->GetGroupHash() == uCurrentHash)
					{
						bMessageHandled	|= pxCurrentGroup->HandleMessage(pxMessage);
					}

					break;
				}

				case AVP_MSGID_INVENTORY_ITEM_RELOAD:
				case AVP_MSGID_ATTACH_WEAPON:
				case AVP_MSGID_DETACH_WEAPON:
				{
					if(pxCurrentGroup->GetHashInParentSystem() == uAVP_INVENTORY_GROUPHASH_WEAPON) // only handled by wepaons
					{
						bMessageHandled |= pxCurrentGroup->HandleMessage(pxMessage);
					}
					break;
				}

				default:
				{
					bMessageHandled	|= pxCurrentGroup->HandleMessage(pxMessage);
					break;
				}
			}		// End of switch.
		}

		xIterator.Next();
	}

	// Did we manage to handle this message?
	return(bMessageHandled);
}

/// End of function 'AVP_Inventory_System::HandleMessage'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Inventory_System::Update
// Author       : Andy Tate
// Date         : 25-Oct-2006
////////////////////////////////////////////////////////////////////////////////

void AVP_Inventory_System::Update(void)
{
	Asura_TemplateHashTree_Iterator<AVP_Inventory_Group*>	xIterator(m_xGroupTree);

	// Try each group in-turn...
	while(!xIterator.Done())
	{
		AVP_Inventory_Group**	ppxCurrentGroup	= xIterator.GetCurrent();
		AVP_Inventory_Group*	pxCurrentGroup	= (ppxCurrentGroup == NULL) ? NULL : *ppxCurrentGroup;

		if(pxCurrentGroup != NULL)
		{
			pxCurrentGroup->Update();
		}

		xIterator.Next();
	}
}

/// End of function 'AVP_Inventory_System::Update'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Inventory_System::Render
// Author       : Andy Tate
// Date         : 25-Oct-2006
////////////////////////////////////////////////////////////////////////////////

void AVP_Inventory_System::Render (void) const
{
	Asura_TemplateHashTree_ConstIterator<AVP_Inventory_Group*>	xIterator(m_xGroupTree);

	// Try each group in-turn...
	while(!xIterator.Done())
	{
		AVP_Inventory_Group const* const*	ppxCurrentGroup	= xIterator.GetCurrent();
		AVP_Inventory_Group const*			pxCurrentGroup	= (ppxCurrentGroup == NULL) ? NULL : *ppxCurrentGroup;

		if(pxCurrentGroup != NULL)
		{
			pxCurrentGroup->Render();
		}

		xIterator.Next();
	}
}

/// End of function 'AVP_Inventory_System::Render'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Inventory_System::GetPermanentlyActiveItemGroup
// Author       : Andy Tate
// Date         : 24-Jan-2008
////////////////////////////////////////////////////////////////////////////////

AVP_Inventory_Group* AVP_Inventory_System::GetPermanentlyActiveItemGroup(void)
{
	Asura_Char const* const	szName		= "PermanentlyActive";
	AVP_Inventory_Group**	ppxGroup	= m_xGroupTree.GetDataByName(szName);
	AVP_Inventory_Group*	pxGroup		= (ppxGroup == NULL) ? NULL : *ppxGroup;

	// If the group doesn't exist we'll need to create it...
	if(pxGroup == NULL)
	{
		pxGroup	= new AVP_Inventory_Group(*this, szName, true);
	}

	return(pxGroup);
}

/////////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Inventory_System::GetPermanentlyActiveItemGroup
// Author       : Chris
// Date         : 16-Dec-2008
// Description	: Same as above, but as its const, will no create group when missing
/////////////////////////////////////////////////////////////////////////////////////

const AVP_Inventory_Group* AVP_Inventory_System::GetPermanentlyActiveItemGroup(void) const
{
	Asura_Char const* const	szName		= "PermanentlyActive";
	AVP_Inventory_Group**	ppxGroup	= m_xGroupTree.GetDataByName(szName);
	AVP_Inventory_Group*	pxGroup		= (ppxGroup == NULL) ? NULL : *ppxGroup;

	// If the group doesn't exist we'll need to create it...
	if(pxGroup == NULL)
	{
		Asura_Assert(false, "Permanently active group not yet created.");
	}

	return(pxGroup);
}

/// End of function 'AVP_Inventory_System::GetPermanentlyActiveItemGroup'. ///

void AVP_Inventory_System::AddPermenantlyActiveItems(AVP_Inventory_Group* pxGroup)
{
	if(pxGroup)
	{
		if(IsClientSide())
		{
			GetParentClient()->AddPermenantlyActiveItems(pxGroup);
		}
		else
		{
			GetParentServer()->AddPermenantlyActiveItems(pxGroup);
		}
	}
}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Inventory_System::GetItem
// Author       : Andy Tate
// Date         : 25-Jan-2008
////////////////////////////////////////////////////////////////////////////////

AVP_Inventory_Item* AVP_Inventory_System::GetItem(const Asura_Hash_ID uHash) const
{
	Asura_TemplateHashTree_ConstIterator<AVP_Inventory_Group*>	xIterator(m_xGroupTree);

	// Try each group in-turn...
	while(!xIterator.Done())
	{
		AVP_Inventory_Group* const*	ppxCurrentGroup	= xIterator.GetCurrent();
		AVP_Inventory_Group*		pxCurrentGroup	= (ppxCurrentGroup == NULL) ? NULL : *ppxCurrentGroup;

		if(pxCurrentGroup != NULL)
		{
			AVP_Inventory_Item*	pxItem	= pxCurrentGroup->GetItem(uHash);

			if(pxItem != NULL)
			{
				return(pxItem);
			}
		}

		xIterator.Next();
	}

	return(NULL);
}

/// End of function 'AVP_Inventory_System::GetItem'. ///

Asura_Hash_ID AVP_Inventory_System::GetFirstAvailableWeaponSlotHash(AVP_Inventory_System* pxSystem)
{
	if(pxSystem)
	{
		AVP_Inventory_Group* pxGroup = pxSystem->GetGroup(uAVP_INVENTORY_GROUPHASH_WEAPON);
		if(pxGroup)
		{
			if(pxGroup->HasDualSlotWeapon())
			{
				return uAVP_INVENTORY_WEAPONHASH_WEAPON02; // Always in slot 2 (and 3)
			}

			u_int uMaxWeapons = 0;
			AVP_ClientActor_Vertebrate* pxClientVert = AVP_ClientActor_Vertebrate::FindEntity(pxSystem->GetOwnerGuid());
			if(pxClientVert)
			{
				uMaxWeapons = pxClientVert->GetMaxWeapons();
			}
						
			if(pxGroup->GetNumItems() < uMaxWeapons)
			{
				Asura_Hash_ID uStartingID = uAVP_INVENTORY_WEAPONHASH_WEAPON01;
				for(u_int u=0;u<uMaxWeapons;u++)
				{
					Asura_Hash_ID uWeaponHash = uStartingID+u;
					if(!pxGroup->GetItem(uWeaponHash))
					{
						return uWeaponHash;
					}
				}
			}
		}
	}
	return ASURA_HASH_ID_UNSET;
}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Inventory_System::IsLocalPlayer
// Author       : Andy Tate
// Date         : 29-Nov-2007
////////////////////////////////////////////////////////////////////////////////

bool AVP_Inventory_System::IsLocalPlayer(void) const
{
	return(AVP_Game_Core::IsLocalPlayer(m_uOwnerGuid));
}

/// End of function 'AVP_Inventory_System::IsLocalPlayer'. ///


// Helper functions.

Asura_Hash_ID AVP_Inventory_System::GetCurrentItemID(const Asura_Hash_ID uGroupHash) const
{
	const AVP_Inventory_Group* pxGroup = GetGroup(uGroupHash);
	return pxGroup ? pxGroup->GetCurrentItem() : ASURA_HASH_ID_UNSET;
}

AVP_Inventory_Item* AVP_Inventory_System::GetCurrentItem(const Asura_Hash_ID uGroupHash)
{
	AVP_Inventory_Group* pxGroup = GetGroup(uGroupHash);
	return pxGroup ? pxGroup->GetCurrentInventoryItem() : NULL;
}

const AVP_Inventory_Item* AVP_Inventory_System::GetCurrentItem(const Asura_Hash_ID uGroupHash) const
{
	const AVP_Inventory_Group* pxGroup = GetGroup(uGroupHash);
	return pxGroup ? pxGroup->GetCurrentInventoryItem() : NULL;
}

u_int AVP_Inventory_System::GetNumSelectableItems(const Asura_Hash_ID uGroupHash) const
{
	const AVP_Inventory_Group* pxGroup = GetGroup(uGroupHash);
	return pxGroup ? pxGroup->GetNumSelectableItems() : 0;
}

/// End of file. ///
