
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_InventoryItem_DynamicSubType.cpp
// Author : Will Plano
// Date   : 09/05/2008
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "Inventory/Items/AVP_InventoryItem_DynamicSubType.h"

#include "Blueprint/Asura_Blueprint_System.h"
#include "ChunkSystem/Asura_Chunk_Stream.h"

#include "Actor/AVP_Actor_DynamicStat.h"
#include "Blueprint/AVP_Blueprint_HashIDs.h"
#include "Inventory/AVP_Inventory_ItemTypes.h"

AVP_InventoryItem_DynamicSubType::AVP_InventoryItem_DynamicSubType( const int iSubType, const Asura_Hash_ID uBlueprintHashID )
: PARENT( iSubType, uBlueprintHashID )
, m_pxDynamicAmmo( NULL )
{
}

void AVP_InventoryItem_DynamicSubType::InitialiseFromBlueprint()
{
	PARENT::InitialiseFromBlueprint();

	const Asura_Blueprint* pxBlueprint = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_INVENTORY_ITEMS, m_uBlueprintHash );
	if ( pxBlueprint )
	{
		Asura_Value xValue;
		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_USES_DYNAMIC_AMMO, xValue ) )
		{
			m_pxDynamicAmmo = xValue.GetBoolValue() ? m_pxDynamicAmmo : NULL;
		}
	}
	else
	{
		Asura_Assertf( false, ( "AVP_InventoryItem_DynamicSubType::InitialiseFromBlueprint - cannot find blueprint %u to initialise from.", m_uBlueprintHash ) );
	}
}

void AVP_InventoryItem_DynamicSubType::ReadFromChunkStream( Asura_Chunk_Stream& xStream )
{
	u_int uVersion;
	xStream >> uVersion;

	PARENT::ReadFromChunkStream( xStream );
}

void AVP_InventoryItem_DynamicSubType::WriteToChunkStream( Asura_Chunk_Stream& xStream ) const
{
	xStream << uCURRENT_CHUNK_VERSION;

	PARENT::WriteToChunkStream( xStream );
}

float AVP_InventoryItem_DynamicSubType::DecreaseQuantity( const float fQuantity )
{
	if ( m_pxDynamicAmmo != NULL )
	{
		m_pxDynamicAmmo->Remove( fQuantity, true );
		return m_pxDynamicAmmo->GetCurrentValue();
	}
	else
	{
		return PARENT::DecreaseQuantity( fQuantity );
	}
}

float AVP_InventoryItem_DynamicSubType::IncreaseQuantity( const float fQuantity )
{
	if ( m_pxDynamicAmmo != NULL )
	{
		m_pxDynamicAmmo->Add( fQuantity, true );
		return m_pxDynamicAmmo->GetCurrentValue();
	}
	else
	{
		return PARENT::IncreaseQuantity( fQuantity );
	}
}

void AVP_InventoryItem_DynamicSubType::InitialiseQuantity( const float fInitial, const float fMax )
{
	if ( m_pxDynamicAmmo != NULL )
	{
		// Not sure if we want to do this.
		m_pxDynamicAmmo->SetMax( fMax );
		m_pxDynamicAmmo->Set( fInitial, true );
	}
	else
	{
		PARENT::InitialiseQuantity( fInitial, fMax );
	}
}

void AVP_InventoryItem_DynamicSubType::SetCurrentQuantity( const float fQuantity )
{
	if ( m_pxDynamicAmmo != NULL )
	{
		return m_pxDynamicAmmo->Set( fQuantity, true );
	}
	else
	{
		PARENT::SetCurrentQuantity( fQuantity );
	}
}

float AVP_InventoryItem_DynamicSubType::GetCurrentQuantity( void ) const
{
	if ( m_pxDynamicAmmo != NULL )
	{
		return m_pxDynamicAmmo->GetCurrentValue();
	}
	else
	{
		return PARENT::GetCurrentQuantity();
	}
}

void AVP_InventoryItem_DynamicSubType::SetMaxQuantity( const float fQuantity )
{
	if ( m_pxDynamicAmmo != NULL )
	{
		// Not sure if we want to do this.
		m_pxDynamicAmmo->SetMax( fQuantity );
	}
	else
	{
		PARENT::SetMaxQuantity( fQuantity );
	}
}

float AVP_InventoryItem_DynamicSubType::GetMaxQuantity( void ) const
{
	if ( m_pxDynamicAmmo != NULL )
	{
		return m_pxDynamicAmmo->GetMax();
	}
	else
	{
		return PARENT::GetMaxQuantity();
	}
}

bool AVP_InventoryItem_DynamicSubType::IsEmpty( void ) const
{
	if ( m_pxDynamicAmmo != NULL )
	{
		return m_pxDynamicAmmo->IsAtMinimum();
	}
	else
	{
		return PARENT::IsEmpty();
	}
}

bool AVP_InventoryItem_DynamicSubType::IsFull( void ) const
{
	if ( m_pxDynamicAmmo != NULL )
	{
		return m_pxDynamicAmmo->IsAtMaximum();
	}
	else
	{
		return PARENT::IsFull();
	}
}

void AVP_InventoryItem_DynamicSubType::SetDynamicAmmo( AVP_Actor_DynamicStat* pxDynamicAmmo )
{
	Asura_Blueprint* pxBlueprint = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_INVENTORY_ITEMS, m_uBlueprintHash );
	if ( pxBlueprint )
	{
		Asura_Value xValue;
		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_USES_DYNAMIC_AMMO, xValue ) )
		{
			m_pxDynamicAmmo = xValue.GetBoolValue() ? pxDynamicAmmo : NULL;
		}
	}
}
