
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_InventoryItem_HumanFlare.cpp
// Author : Will Plano
// Date   : 25/01/2008
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "Inventory/Items/AVP_InventoryItem_HumanFlare.h"

// Asura headers
#include "Animation/Asura_AnimationController_Animation.h"
#include "Animation/Asura_AnimationBlend.h"
#include "Blueprint/Asura_Blueprint.h"
#include "Blueprint/Asura_Blueprint_System.h"

// Game independent header-files...
#include "Actor/Vertebrates/Human/AVP_Human_AnimationHandler.h"
#include "Animation/AVP_AnimFlags.h"
#include "Blueprint/AVP_Blueprint_HashIDs.h"
#include "Core/AVP_Game_Core.h"
#include "Entity/AVP_Entity_Classification.h"
#include "Entity/AVP_Messages.h"
#include "Entity/GamePlayVolume/AVP_ServerEntity_GamePlayVolume.h"
#include "Entity/PhysicalObject/Actor/AVP_Actor_Messages.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Human/AVP_ClientActor_Human.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Human/AVP_ServerActor_Human.h"
#include "Entity/PhysicalObject/Projectile/AVP_Projectile_Messages.h"
#include "Entity/PhysicalObject/Projectile/AVP_GenericProjectileData.h"
#include "Inventory/AVP_Inventory_HashIDs.h"
#include "Inventory/AVP_Inventory_System.h"
#include "Inventory/AVP_Inventory_Group.h"
#include "Inventory/AVP_InventoryItem_SubType.h"

////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////

AVP_InventoryItem_HumanFlare::AVP_InventoryItem_HumanFlare(AVP_Inventory_Group& xParentGroup, const Asura_Hash_ID uBlueprint, const char* szName)
: PARENT(AVP_INVENTORYITEMTYPE_FLARE, xParentGroup, uAVP_INVENTORY_ITEMHASH_FLARE, uBlueprint, szName )
, m_uProjectileHash( ASURA_HASH_ID_UNSET )
, m_uFlareAACAHash( ASURA_HASH_ID_UNSET )
, m_uCurrentFlareGuid(  ASURA_GUID_UNREGISTERED )
{
	// Initialise the valid states...
	SetStateAsValid(BEGIN_USING);
	SetStateAsValid(USING);
}

AVP_InventoryItem_HumanFlare::~AVP_InventoryItem_HumanFlare()
{
}

void AVP_InventoryItem_HumanFlare::InitialiseFromBlueprint()
{
	PARENT::InitialiseFromBlueprint();

	const Asura_Blueprint* pxBlueprint = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_INVENTORY_ITEMS, m_uBlueprintHash );
	if ( pxBlueprint )
	{
		Asura_Hash_ID uSubTypeHash = ASURA_HASH_ID_UNSET;

		Asura_Value xValue;
		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_SUBTYPE, xValue ) )
		{
			uSubTypeHash = xValue.GetHashIDValue();
			const Asura_Blueprint* pxSubTypeBlueprint = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_INVENTORY_ITEMS, uSubTypeHash );
			if ( pxSubTypeBlueprint )
			{
				if ( pxSubTypeBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_PROJECTILETYPE, xValue ) )
				{
					m_uProjectileHash = xValue.GetHashIDValue();
				}
			}
		}

		InitialiseSubTypes(1);
		m_ppxSubTypes[0] = new AVP_InventoryItem_SubType( uSubTypeHash );
		if ( m_ppxSubTypes[0] )
		{
			m_ppxSubTypes[0]->InitialiseFromBlueprint();
		}
	}
}

void AVP_InventoryItem_HumanFlare::OnStateChange( const u_int uOldState, const u_int uNewState )
{
	AVP_Inventory_System&		xParentSystem	= GetParentSystem();

	// Call the base-function...
	PARENT::OnStateChange(uOldState, uNewState);

	switch(uNewState)
	{
		case BEGIN_USING:
		{
			m_uCurrentFlareGuid = ASURA_GUID_UNREGISTERED;
			if ( xParentSystem.IsMaster() && xParentSystem.IsClientSide() && GetParentSystem().GetParentClient() )
			{
				AVP_ClientActor_Human* pxHuman = static_cast < AVP_ClientActor_Human* >( GetParentSystem().GetParentClient() );
				pxHuman->OnThrowFlare();
			}
			if ( xParentSystem.IsClientSide() )
			{
				AVP_ClientEntity_Actor* pxActor = xParentSystem.GetParentClient();
				Asura_Hash_ID uActorAnim = ASURA_HASH_ID_UNSET;
				if ( pxActor != NULL )
				{
					const AVP_Actor_AnimStateData* pxStateData = pxActor->GetCurrentAnimStateData();
					if ( pxStateData )
					{
						uActorAnim = pxStateData->GetFlareActorAnimHash();
					}	
				}

				AVP_ClientActor_Human* pxHuman = static_cast< AVP_ClientActor_Human* >( xParentSystem.GetParentClient() );
				AVP_Human_AnimationHandler* pxAnimHandler = pxHuman != NULL ? static_cast<AVP_Human_AnimationHandler*>(pxHuman->GetAnimationHandler()) : NULL;
				u_int uFlags( AAC_FLAG_ACTIVE | AAC_FLAG_DESTROY_WHEN_FINISHED | AAC_FLAG_ENSURE_UNIQUE_HASH );
				u_int uPriority( AVP_AAPC_GENERAL_ANIM );
				Asura_AnimationBlend* pxBlend( new Asura_AnimationBlend_AnimatedTween( 0.1f, 0.2f, false ) );
				m_uFlareAACAHash = pxAnimHandler->PlayAnimation( uFlags, uPriority, uActorAnim, pxBlend );
				if ( m_uFlareAACAHash == ASURA_HASH_ID_UNSET )
				{
					Asura_Assert( false, "Flare anim not created" );
				}
			}
		}
		break;

		case USING:
		{

		}
		break;

		case IDLING:
		{
			m_uFlareAACAHash = ASURA_HASH_ID_UNSET;
		}
		break;

		default:
		{
			// no nothing
		}
		break;
	} // End of switch.

	if ( USING == uOldState && xParentSystem.IsMaster() )
	{
		ThrowFlare();
	}
}

/// End of function 'AVP_InventoryItem_HumanFlare::OnStateChange'. ///


////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_InventoryItem_HumanFlare::UpdateClientState
// Author       : Andy Tate
// Date         : 28-Nov-2007
////////////////////////////////////////////////////////////////////////////////

void AVP_InventoryItem_HumanFlare::UpdateClientState(AVP_ClientEntity_Actor& xClient)
{
	const State	eCurrentState	= GetCurrentState();

	// Decide what to do based on our current state...
	switch(eCurrentState)
	{
		case INACTIVE:
		case IDLING:
		{
			//do nothing
		}
		break;

		case BEGIN_USING:
		{
			if ( GetParentSystem().IsMaster() )
			{	
				const AVP_Actor_AnimationHandler*			pxAnimationHandler	= xClient.GetAnimationHandler();
				const Asura_Animation*						pxAnimation			= pxAnimationHandler != NULL ? pxAnimationHandler->GetAnimation() : NULL;
				const Asura_AnimationController_Animation*	pxAACA				= pxAnimation != NULL ? pxAnimation->GetControllerAnimationFromHash( m_uFlareAACAHash ) : NULL;
				u_int										uAnimFlags			= 0xFFFFFFFF;

				if ( pxAACA )
				{
					uAnimFlags = pxAACA->GetFlagsForLastFrame();
				}

				if ( ( uAnimFlags & AVP_ANIMFLAG_TRIGGER_ATTACHMENT ) != 0 )
				{
					CreateFlare();
				}
			}
			else if ( GetTimeInState() > 3.0f )
			{
				Asura_Assert( false, "Flare inventory item timing out, messages probably got handled in the wrong order on the server" );
				CreateFlare();
			}

			if ( m_uCurrentFlareGuid != ASURA_GUID_UNREGISTERED )
			{
				RequestNextUsingState();
			}
			else if ( GetTimeInState() > 5.0f )
			{
				Asura_Assert( false, "Flare inventory item timing out, messages probably got handled in the wrong order on the server" );
				RequestNextUsingState();
			}
		}
		break;

		case USING:
		{
			if ( GetParentSystem().IsMaster() )
			{
				const AVP_Actor_AnimationHandler*			pxAnimationHandler	= xClient.GetAnimationHandler();
				const Asura_Animation*						pxAnimation			= pxAnimationHandler != NULL ? pxAnimationHandler->GetAnimation() : NULL;
				const Asura_AnimationController_Animation*	pxAACA				= ( pxAnimation != NULL ) ? pxAnimation->GetControllerAnimationFromHash( m_uFlareAACAHash ) : NULL;
				u_int										uAnimFlags			= 0xFFFFFFFF;

				if ( pxAACA )
				{
					uAnimFlags = pxAACA->GetFlagsForLastFrame();
				}

				if ( ( uAnimFlags & AVP_ANIMFLAG_TRIGGER_DETACHMENT ) != 0 )
				{
					RequestIdling( false );
				}
			}
			else if ( GetTimeInState() > 3.0f )
			{
				Asura_Assert( false, "Flare inventory item timing out" );
				RequestIdling( false );
			}
		}
		break;
		
		default:
		{
			Asura_Assert(false, "Unexpected state found when updating client-state of flare.");
		}
		break;
	}		// End of switch.
}

/// End of function 'AVP_InventoryItem_HumanFlare::UpdateClientState'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_InventoryItem_HumanFlare::UpdateServerState
// Author       : Andy Tate
// Date         : 28-Nov-2007
////////////////////////////////////////////////////////////////////////////////

void AVP_InventoryItem_HumanFlare::UpdateServerState(AVP_ServerEntity_Actor& xServer)
{
	const State	eCurrentState	= GetCurrentState();
	const AVP_ClientActor_Human* pxClient = AVP_ClientActor_Human::FindEntity( xServer.GetGuid() );

	// Decide what to do based on our current state...
	switch(eCurrentState)
	{
		case INACTIVE:
		case IDLING:
		{
			// do nothing
		}
		break;

		case BEGIN_USING:
		{
			if ( GetParentSystem().IsMaster() )
			{	
				const AVP_Inventory_System*					pxClientSystem		= pxClient ? pxClient->GetInventorySystem() : NULL;
				const AVP_InventoryItem_HumanFlare*			pxClientFlareItem	= pxClientSystem ? static_cast< const AVP_InventoryItem_HumanFlare* >( pxClientSystem->GetItem( uAVP_BLUEPRINT_PARAM_FLARE ) ) : NULL;

				const AVP_Actor_AnimationHandler*			pxAnimationHandler	= pxClient ? pxClient->GetAnimationHandler() : NULL;
				const Asura_Animation*						pxAnimation			= pxAnimationHandler != NULL ? pxAnimationHandler->GetAnimation() : NULL;
				const Asura_AnimationController_Animation*	pxAACA				= ( pxAnimation != NULL && pxClientFlareItem != NULL ) ? pxAnimation->GetControllerAnimationFromHash( pxClientFlareItem->m_uFlareAACAHash ) : NULL;
				u_int										uAnimFlags			= 0xFFFFFFFF;
	
				if ( pxAACA )
				{
					uAnimFlags = pxAACA->GetFlagsForLastFrame();
				}

				if ( ( uAnimFlags & AVP_ANIMFLAG_TRIGGER_ATTACHMENT ) != 0 )
				{
					CreateFlare();
				}
			}
			else if ( GetTimeInState() > 2.0f )
			{
				Asura_Assert( false, "Flare inventory item timing out, messages probably got handled in the wrong order" );
				CreateFlare();
			}

			if ( m_uCurrentFlareGuid != ASURA_GUID_UNREGISTERED )
			{
				RequestNextUsingState();
			}
			else if ( GetTimeInState() > 5.0f )
			{
				Asura_Assert( false, "Flare inventory item timing out, messages probably got handled in the wrong order on the server" );
				RequestNextUsingState();
			}
		}
		break;

		case USING:
		{
			if ( GetParentSystem().IsMaster() )
			{
				const AVP_Inventory_System*					pxClientSystem		= pxClient ? pxClient->GetInventorySystem() : NULL;
				const AVP_InventoryItem_HumanFlare*			pxClientFlareItem	= pxClientSystem ? static_cast< const AVP_InventoryItem_HumanFlare* >( pxClientSystem->GetItem( uAVP_BLUEPRINT_PARAM_FLARE ) ) : NULL;
				const AVP_Actor_AnimationHandler*			pxAnimationHandler	= pxClient ? pxClient->GetAnimationHandler() : NULL;
				const Asura_Animation*						pxAnimation			= pxAnimationHandler != NULL ? pxAnimationHandler->GetAnimation() : NULL;
				const Asura_AnimationController_Animation*	pxAACA				= ( pxAnimation != NULL && pxClientFlareItem != NULL ) ? pxAnimation->GetControllerAnimationFromHash( pxClientFlareItem->m_uFlareAACAHash ) : NULL;
				u_int										uAnimFlags			= 0xFFFFFFFF;

				if ( pxAACA )
				{
					uAnimFlags = pxAACA->GetFlagsForLastFrame();
				}

				if ( ( uAnimFlags & AVP_ANIMFLAG_TRIGGER_DETACHMENT ) != 0 )
				{
					RequestIdling( false );
				}
			}
			else if ( GetTimeInState() > 2.0f )
			{
				Asura_Assert( false, "Flare inventory item timing out, messages probably got handled in the wrong order" );
				RequestIdling( false );
			}
		}
		break;

		default:
		{
			Asura_Assert(false, "Unexpected state found when updating master-state of weapon.");
		}
		break;
	}		// End of switch.
}

void AVP_InventoryItem_HumanFlare::CreateFlare()
{
	// Create a message
	Asura_Message* pxMessage = Asura_Network::CreateMessage( AVP_MSGID_CREATE_PROJECTILE_FLARE, sizeof( AVP_Message_CreateGenericProjectile ) );
	if ( pxMessage )
	{
		AVP_Message_CreateGenericProjectile* pxData = static_cast< AVP_Message_CreateGenericProjectile* >( pxMessage->GetData() );

		// Fill-in the message...
		pxData->SetPosition( Asura_Maths::ZeroVector3 ); // these will be ignored as we will be linked to a marker point if launch is not set.
		pxData->SetOrientation( Asura_Maths::IdentityQuat );
		pxData->SetProjectileBlueprintHash( m_uProjectileHash );
		pxData->SetDamageScale( 1.0f );
		pxData->SetLaunch( false );

		pxMessage->SetToGuid( GetParentSystem().GetOwnerGuid() );
		pxMessage->SetFromGuid( GetParentSystem().GetOwnerGuid() );
		Asura_Network::Send( pxMessage );
	}
}

void AVP_InventoryItem_HumanFlare::ThrowFlare()
{
	Asura_Message*		pxMessage	= Asura_Network::CreateMessage( AVP_MSGID_LAUNCH_PROJECTILE, sizeof(AVP_Message_ActorLaunchProjectile));
	const Asura_Guid	uFlareGuid	= m_uCurrentFlareGuid;

	Asura_Assert( uFlareGuid != ASURA_GUID_UNREGISTERED, "No flare" );

	if(pxMessage != NULL)
	{
		AVP_Message_ActorLaunchProjectile*	pxData	= static_cast<AVP_Message_ActorLaunchProjectile*>(pxMessage->GetData());
		const AVP_GenericProjectileData* pxGPD = AVP_GenericProjectileData::GetDataByHash( m_uProjectileHash );

		Asura_Vector_3 xAimDir;
		Asura_Vector_3 xVelocity;
		Asura_Vector_3 xPosition;
		Asura_Matrix_3x3 xOrientation;
		if ( GetParentSystem().IsClientSide() )
		{
			AVP_ClientEntity_Actor* pxClient = GetParentSystem().GetParentClient();
			Asura_AVector3 xAVec;
			pxClient->GetAimDirection( xAVec );
			xAimDir = xAVec;
			pxClient->GetCurrentVelocity( xVelocity );
			pxClient->GetWorldSpaceMarkerPoint( pxGPD->GetLinkMarkerPoint(), &xPosition, &xOrientation );
		}
		else
		{
			AVP_ServerEntity_Actor* pxServer = GetParentSystem().GetParentServer();
			xAimDir = pxServer->GetAimDirection();
			xVelocity = pxServer->GetVelocity();
			pxServer->GetWorldSpaceMarkerPoint( pxGPD->GetLinkMarkerPoint(), &xPosition, &xOrientation );
		}

		Asura_Vector_3 xFlareVel( xAimDir );
		xFlareVel *= pxGPD->GetMuzzleVelocity();
		xFlareVel += xVelocity;

		// Fill-in the message...
		pxData->SetClassification(AVP_ENTITYCLASS_HUMANFLARE);
		pxData->SetGuid(uFlareGuid);
		pxData->SetPosition(xPosition);
		pxData->SetOrientation(Asura_Quat(xOrientation));
		pxData->SetVelocity(xFlareVel);

		pxData->SetSticky(false);
		pxData->SetVisibleToAliens(true);

		pxMessage->SetToGuid(GetParentSystem().GetOwnerGuid());
		pxMessage->SetFromGuid(GetParentSystem().GetOwnerGuid());

		Asura_Network::Send(pxMessage);
	}
}

/// End of file. ///
