
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_InventoryItem_HumanFlare.h
// Author : Will Plano
// Date   : 24/01/2008
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __AVP_INVENTORYITEM_HUMANFLARE_H__
#define __AVP_INVENTORYITEM_HUMANFLARE_H__

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura_Maths.h"

#include "Inventory/AVP_Inventory_Item.h"
#include "Inventory/AVP_InventoryItem_SubType.h"

////////////////////////////////////////////////////////////////////////////////
// Forward class declarations...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Class declaration...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_InventoryItem_HumanFlare
// Author       : Will Plano
// Date			: 24/01/2008
//
// Description  : Class used to handle the human flares via the
//				  inventory-system.
////////////////////////////////////////////////////////////////////////////////

class AVP_InventoryItem_HumanFlare : public AVP_Inventory_Item
{
public:
	enum FlareUsageTypes
	{
		UT_FLARE = NUM_ITEM_USAGE_TYPES,
		NUM_FLARE_USAGE_TYPES
	};

	// Public functions...
	AVP_InventoryItem_HumanFlare(AVP_Inventory_Group& xParentGroup, const Asura_Hash_ID uBlueprint, const char* szName );
	virtual ~AVP_InventoryItem_HumanFlare();

	virtual void InitialiseFromBlueprint();

	void IncreaseQuantity() { m_ppxSubTypes[0]->IncreaseQuantity( 1.0f ); }
	virtual int	GetSubTypeIndexFromUsageType(const u_int uUsageType) const { return 0; }

	u_int GetMaxActiveFlares() const { return Asura_Maths::FloatToInt_Near(m_ppxSubTypes[0]->GetMaxQuantity()); }
	bool IsEmpty() const { return m_ppxSubTypes[0]->IsEmpty(); }
	void SetFlareGuid( const Asura_Guid uGuid ) { m_uCurrentFlareGuid = uGuid; }

protected:
	// Overridden state functions...
	virtual void	OnStateChange(const u_int uOldState, const u_int uNewState);

	virtual void	UpdateClientState(AVP_ClientEntity_Actor& xClient);
	virtual void	UpdateServerState(AVP_ServerEntity_Actor& xServer);

private:
	// Private typedefs...
	typedef AVP_Inventory_Item PARENT;

	// Private variables...
	Asura_Hash_ID m_uProjectileHash;
	Asura_Hash_ID m_uFlareAACAHash;

	Asura_Guid m_uCurrentFlareGuid;

	// Private functions...
	void CreateFlare();
	void ThrowFlare();

	AVP_InventoryItem_HumanFlare(void);
	AVP_InventoryItem_HumanFlare(const AVP_InventoryItem_HumanFlare& xOther);

	AVP_InventoryItem_HumanFlare&	operator=(const AVP_InventoryItem_HumanFlare& xOther);
};   // End of class 'AVP_InventoryItem_HumanFlare'.


#endif   // ifndef __AVP_INVENTORYITEM_HUMANFLARE_H__


/// End of file. ///
