
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_InventoryItem_VisionMode.cpp
// Author : Andy Tate
// Date   : 28-Nov-2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "Inventory/Items/AVP_InventoryItem_VisionMode.h"

// Asura
#include "Blueprint/Asura_Blueprint_System.h"
#include "Blueprint/Asura_Blueprint.h"

// Game independent header-files...
#include "Blueprint/AVP_Blueprint_HashIDs.h"
#include "Actor/AVP_Actor_HUDHandler.h"
#include "Entity/PhysicalObject/Actor/AVP_ClientEntity_Actor.h"
#include "Inventory/AVP_Inventory_System.h"
#include "SpecialFX/AVP_Laser.h"

////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////

DEBUGVAR float dbg_fDisruptTime		= 1.0f;
DEBUGVAR float dbg_fMinFlickerTime	= 0.2f;
DEBUGVAR float dbg_fMaxFlickerTime	= 0.5f;

////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Private functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Protected functions...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_InventoryItem_VisionMode::OnStateChange
// Author       : Andy Tate
// Date         : 14-Apr-2008
////////////////////////////////////////////////////////////////////////////////

void AVP_InventoryItem_VisionMode::OnStateChange(const u_int uOldState,
												 const u_int uNewState)
{
	AVP_Inventory_System&	xParentSystem	= GetParentSystem();
	AVP_ClientEntity_Actor*	pxClient		= xParentSystem.GetParentClient();
	AVP_Actor_HUDHandler*	pxHUDHandler	= (pxClient == NULL) ? NULL : pxClient->GetHUDHandler();

	// Call the base function...
	PARENT::OnStateChange(uOldState, uNewState);

	if(pxHUDHandler != NULL)
	{
		switch(uNewState)
		{
			case ACTIVATING:
			{
				pxHUDHandler->SetVisionMode(*pxClient, m_uBlueprintHash, false);
				AVP_Laser::SetColour(GetLaserColour());
				break;
			}

			case IDLING:
			{
				switch(uOldState)
				{
					case INACTIVE:
					{
						pxHUDHandler->SetVisionMode(*pxClient, m_uBlueprintHash, true);
						AVP_Laser::SetColour(GetLaserColour());
						break;
					}

					default:
					{
						break;
					}
				}		// End of switch.
			}

			default:
			{
				break;
			}
		}		// End of switch.
	}
}

/// End of function 'AVP_InventoryItem_VisionMode::OnStateChange'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_InventoryItem_VisionMode::UpdateClientState
// Author       : Andy Tate
// Date         : 28-Nov-2007
////////////////////////////////////////////////////////////////////////////////

void AVP_InventoryItem_VisionMode::UpdateClientState(AVP_ClientEntity_Actor& xClient)
{
	const State	eCurrentState	= GetCurrentState();

	UpdateDisruption( xClient );

	// Decide what to do based on our current state...
	switch(eCurrentState)
	{
		case INACTIVE:
		{
			break;
		}

		case ACTIVATING:
		{
			RequestNextIdlingState();
			break;
		}

		case IDLING:
		{
			break;
		}

		case DEACTIVATING:
		{
			RequestNextInactiveState();
			break;
		}

		default:
		{
			Asura_Assert(false, "Unexpected state found when updating master-state of weapon.");
			break;
		}
	}		// End of switch.
}

/// End of function 'AVP_InventoryItem_VisionMode::UpdateClientState'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_InventoryItem_VisionMode::UpdateServerState
// Author       : Andy Tate
// Date         : 28-Nov-2007
////////////////////////////////////////////////////////////////////////////////

void AVP_InventoryItem_VisionMode::UpdateServerState(AVP_ServerEntity_Actor& xServer)
{
	Asura_Assert(false, "Vision-modes are not currently supported server-side.");
}

/// End of function 'AVP_InventoryItem_VisionMode::UpdateServerState'. ///



////////////////////////////////////////////////////////////////////////////////
// Public functions...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_InventoryItem_VisionMode::AVP_InventoryItem_VisionMode
// Author       : Andy Tate
// Date         : 28-Nov-2007
////////////////////////////////////////////////////////////////////////////////

AVP_InventoryItem_VisionMode::AVP_InventoryItem_VisionMode(AVP_Inventory_Group& xParentGroup,
														   const Asura_Hash_ID uBlueprintHash,
														   const Asura_Hash_ID uNameHash)

: PARENT(AVP_INVENTORYITEMTYPE_VISION_MODE,  xParentGroup, uNameHash, uBlueprintHash, "Vision Mode")
, m_fFocusLockMultAlien( 0.0f )
, m_fFocusLockMultAndroid( 0.0f )
, m_fFocusLockMultHuman( 0.0f )
, m_fFocusLockMultPredator( 0.0f )
, m_uDSPChainHash( ASURA_HASH_ID_UNSET )
, m_fDisruptTimer( 0.0f )
, m_fFlickerTimer( 0.0f )
, m_xGlowColour(0)
, m_xLaserColour(1.0f, 0.0f, 0.0f, 1.0f)
, m_uBustedBlueprintHash( ASURA_HASH_ID_UNSET )
, m_uFlags( 0 )
, m_iFixedSlot( -1 )
{
	AVP_Inventory_System&	xParentSystem	= GetParentSystem();

	// Initialise the remaining data...
	if(xParentSystem.IsClientSide())
	{
		SetStateAsValid(ACTIVATING);
		SetStateAsValid(DEACTIVATING);
	}

	GetStaticDataFromBlueprint();
}

/// End of function 'AVP_InventoryItem_VisionMode::AVP_InventoryItem_VisionMode'. ///

float AVP_InventoryItem_VisionMode::GetVisionModeMultiplier( const AVP_ENTITY_CLASSIFICATION uClass ) const
{
	switch ( uClass )
	{
		case AVP_ENTITYCLASS_ALIEN:
		case AVP_ENTITYCLASS_ALIENQUEEN:
		case AVP_ENTITYCLASS_ALIENFACEHUGGER:
		case AVP_ENTITYCLASS_PREDALIEN:
		{
			return m_fFocusLockMultAlien;
		}

		case AVP_ENTITYCLASS_ANDROID:
		case AVP_ENTITYCLASS_SENTRYGUN:
		case AVP_ENTITYCLASS_DROPSHIP:
		{
			return m_fFocusLockMultAndroid;
		}

		case AVP_ENTITYCLASS_CIVILIAN:
		case AVP_ENTITYCLASS_MARINE:
		case AVP_ENTITYCLASS_HUMAN:
		{
			return m_fFocusLockMultHuman;
		}

		case AVP_ENTITYCLASS_PREDATOR:
		{
			return m_fFocusLockMultPredator;
		}

		default:
		{
			Asura_Assertf( false, ( "Unhandled classification, %d, in AVP_InventoryItem_VisionMode::GetVisionModeMultiplier", uClass ) );
			return 1.0f;
		}
	}
}

void AVP_InventoryItem_VisionMode::Disrupt()
{
	m_fDisruptTimer = dbg_fDisruptTime;
}

void AVP_InventoryItem_VisionMode::UpdateDisruption( AVP_ClientEntity_Actor& xClient )
{
	AVP_Actor_HUDHandler*	pxHUDHandler	= xClient.GetHUDHandler();

	if ( m_fDisruptTimer > 0.0f )
	{
		const float fTimeSlice = Asura_Timers::GetGameTimeSlice();
		m_fDisruptTimer -= fTimeSlice;
		if ( m_fDisruptTimer <= 0.0f )
		{
			m_fDisruptTimer = 0.0f;
			m_bDisrupted = false;
			if ( pxHUDHandler && IsActive() ) 
			{
				pxHUDHandler->SetVisionMode( xClient, m_uBlueprintHash, true );
			}
		}
		else if ( m_fFlickerTimer > 0.0f )
		{
			m_fFlickerTimer -= fTimeSlice;
		}
		else if ( m_bDisrupted )
		{
			m_bDisrupted = false;
			if ( pxHUDHandler && IsActive() ) 
			{
				pxHUDHandler->SetVisionMode( xClient, m_uBlueprintHash, true );
			}
		}
		else
		{
			m_bDisrupted = true;
			if ( pxHUDHandler && IsActive() ) 
			{
				pxHUDHandler->SetVisionMode( xClient, m_uBustedBlueprintHash, true );
			}
			m_fFlickerTimer = Asura_Random::GetFloat( dbg_fMinFlickerTime, dbg_fMaxFlickerTime );
		}
	}
}

void AVP_InventoryItem_VisionMode::GetStaticDataFromBlueprint()
{
	const Asura_Blueprint* pxBlueprint = Asura_Blueprint_System::GetBlueprintByHashID(uAVP_BLUEPRINT_TYPE_INVENTORY_ITEMS, GetBlueprintHash());
	if(pxBlueprint)
	{
		Asura_Value xValue;
		if(pxBlueprint->GetParameterValueByHashID(uAVP_BLUEPRINT_PARAM_DISABLESMOTIONTRACKER, xValue))
		{
			m_bDisablesMotionTracer = xValue.GetBoolValue();
		}
		if(pxBlueprint->GetParameterValueByHashID(uAVP_BLUEPRINT_PARAM_DISABLESTORCH, xValue))
		{
			m_bDisablesTorch = xValue.GetBoolValue();
		}
		if (pxBlueprint->GetParameterValueByHashID(uAVP_BLUEPRINT_PARAM_FOCUSLOCKMULTALIEN, xValue))
		{
			m_fFocusLockMultAlien = xValue.GetFloatValue();
		}
		if (pxBlueprint->GetParameterValueByHashID(uAVP_BLUEPRINT_PARAM_FOCUSLOCKMULTANDROID, xValue))
		{
			m_fFocusLockMultAndroid = xValue.GetFloatValue();
		}
		if (pxBlueprint->GetParameterValueByHashID(uAVP_BLUEPRINT_PARAM_FOCUSLOCKMULTHUMAN, xValue))
		{
			m_fFocusLockMultHuman = xValue.GetFloatValue();
		}
		if (pxBlueprint->GetParameterValueByHashID(uAVP_BLUEPRINT_PARAM_FOCUSLOCKMULTPREDATOR, xValue))
		{
			m_fFocusLockMultPredator = xValue.GetFloatValue();
		}
		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_WEAPON_SLOTINDEX, xValue ) )
		{
			m_iFixedSlot = xValue.GetIntValue();
		}
		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_BUSTEDVISION, xValue ) )
		{
			m_uBustedBlueprintHash = xValue.GetHashIDValue();
		}
		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_DSP, xValue ) )
		{
			m_uDSPChainHash = xValue.GetHashIDValue();
		}
		float fR = pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_TARGETGLOW_R, xValue) ? xValue.GetFloatValue() : 0.0f;
		float fG = pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_TARGETGLOW_G, xValue) ? xValue.GetFloatValue() : 0.0f;
		float fB = pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_TARGETGLOW_B, xValue) ? xValue.GetFloatValue() : 0.0f;
		m_xGlowColour = Asura_RGBA(fR, fG, fB, 1.0f);

		fR = pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_LASERCOLOUR_R, xValue) ? xValue.GetFloatValue() : 1.0f;
		fG = pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_LASERCOLOUR_G, xValue) ? xValue.GetFloatValue() : 0.0f;
		fB = pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_LASERCOLOUR_B, xValue) ? xValue.GetFloatValue() : 0.0f;
		m_xLaserColour = Asura_RGBA(fR, fG, fB, 1.0f);

	}
	else
	{
		Asura_Assertf( false, ( "AVP_InventoryItem_VisionMode::GetStaticDataFromBlueprint - cannot find blueprint %u to initialise from.", GetBlueprintHash() ) );
	}
}

/// End of file. ///
