
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_Alien_MovesBranch.h
// Author : Anthony Gregson
// Date   : 16/11/07
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __AVP_ALIEN_MOVESBRANCH_H__
#define __AVP_ALIEN_MOVESBRANCH_H__

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Entity/PhysicalObject/Moves/Asura_MoveData_Animation.h"
#include "Entity/PhysicalObject/Moves/Asura_MoveData_OrientationInterpolation.h"
#include "Entity/PhysicalObject/Moves/Asura_MoveData_PositionInterpolation.h"

#include "Moves/AVP_Moves.h"


////////////////////////////////////////////////////////////////////////////////
// Forward class declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Class declaration...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_Move_Alien_MountBranch
// Author       : Anthony Gregson
// Date         : 06/12/07
//
// Description  : Connect ourselves to a branch attractor
////////////////////////////////////////////////////////////////////////////////

class AVP_Move_Alien_MountBranch : public AVP_Move
{
public:
	AVP_Move_Alien_MountBranch(Asura_Movement_System& xParentSystem);

	virtual const char*		GetName(void) const	{ return("AVP_Move_Alien_MountBranch"); }
	static Asura_Hash_ID	GetStaticHash(void)	{ return(0x68CABE79); }	// Asura_GetHashID("AVP_Move_Alien_MountBranch")); }

protected:
	virtual bool Start(const Asura_Msg_Move* pxStartMsg);

	Asura_MoveData_PositionInterpolation m_xPositionInterpolationData;
	Asura_MoveData_OrientationInterpolation m_xOrientationInterpolationData;

private:
	typedef AVP_Move PARENT;
};   // End of class 'AVP_Move_Alien_MountBranch'.


// More bespoke move that shoves the actor off in the direction of their input.
class AVP_Move_Alien_DismountBranch : public AVP_Move
{
public:
	AVP_Move_Alien_DismountBranch(Asura_Movement_System& xParentSystem);

	// Accessor functions...
	virtual const char*		GetName(void) const	{ return("AVP_Move_Alien_DismountBranch"); }
	static Asura_Hash_ID	GetStaticHash(void)	{ return(0x0AF3B56F); }	// Asura_GetHashID("AVP_Move_Alien_DismountBranch")); }

	const Asura_Vector_3&	GetMovementDirection() const { return m_xMovementDirection; }

protected:
	// Overridden functions...
	virtual bool Start(const Asura_Msg_Move* pxStartMsg);
	virtual void UpdateStates(const float fTimeStep);
	virtual bool HasFinished() const;

private:
	// Private typedefs...
	typedef AVP_Move PARENT;

	// Private variables...
	Asura_Vector_3 m_xMovementDirection;
	float m_fTimeInMove; // TODO - Replace this with detecting falling, and entering the falling move.
};

#endif   // ifndef __AVP_ALIEN_MOVESBRANCH_H__


/// End of file. ///
