///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_Alien_MovesJump.cpp
// Author : Andy Tate
// Date   : 4-Sep-2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "Moves/Vertebrates/Alien/AVP_Alien_MovesJump.h"

// Asura independent header-files...
#include "Animation/Asura_AnimationController_Animation.h"
#include "Blueprint/Asura_Blueprint.h"
#include "Blueprint/Asura_Blueprint_System.h"
#include "Physics/Asura_Physics_MovementQuery.h"

// Game independent header-files...
#include "Actor/AVP_Actor_PhysicsObjectHandler.h"
#include "Actor/Vertebrates/AVP_Vertebrate_AnimationHandler.h"
#include "Actor/Vertebrates/Alien/AVP_Alien_JumpHandler.h"
#include "Blueprint/AVP_Blueprint_HashIDs.h"
#include "Entity/PhysicalObject/Actor/AVP_ClientEntity_Actor.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Alien/AVP_ClientActor_Alien.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Alien/AVP_Messages_Alien.h"

static const Asura_Hash_ID g_uPlayerJumpBP	= 0x41a03e34;	// Alien Player Jump
static const Asura_Hash_ID g_uAIJumpBP		= 0x8349accd;	// Alien AI Jump
static const Asura_Hash_ID g_uAIMovesBP		= 0xd81414b8;   // Asura_GetHashID( "Moves3rdPerson" )

static const Asura_Hash_ID g_uConcave90FloorToWallAllInOneAnimParam	= 0x3272f295;	// Jump Concave 90 Floor To Wall
static const Asura_Hash_ID g_uConcave90FloorToWallBeginAnimParam	= 0xbf257b5e;	// Jump Concave 90 Floor To Wall Begin
static const Asura_Hash_ID g_uConcave90FloorToWallMiddleAnimParam	= 0x38862d80;	// Jump Concave 90 Floor To Wall Middle
static const Asura_Hash_ID g_uConcave90FloorToWallEndAnimParam		= 0x03edb4d0;	// Jump Concave 90 Floor To Wall End
static const Asura_Hash_ID g_u180WallToWallAnimParam				= 0x07bb4d82;	// Jump 180 Wall To Wall
static const Asura_Hash_ID g_uDropStartAnimParam					= 0xd6fb7323;	// Jump Drop Start
static const Asura_Hash_ID g_uHopLeftParam							= 0xe462417e;	// Asura_GetHashID( "Hop Left" )
static const Asura_Hash_ID g_uHopRightParam							= 0xa83c4f85;	// Asura_GetHashID( "Hop Right" )
static const Asura_Hash_ID g_uWallHopLeftParam						= 0x159ef314;   // Asura_GetHashID( "Wall Hop Left" )
static const Asura_Hash_ID g_uWallHopRightParam						= 0x9e95d0af;   // Asura_GetHashID( "Wall Hop Right" )
static const Asura_Hash_ID g_uJumpOntoWallLeftParam					= 0x79cbf2b2;   // Asura_GetHashID( "Jump Concave 90 Floor To Wall Left" )
static const Asura_Hash_ID g_uJumpOntoWallRightParam				= 0xc008c4d1;   // Asura_GetHashID( "Jump Concave 90 Floor To Wall Right" )
static const Asura_Hash_ID g_u180TurnToCrawlLeft					= 0xdf8d55ed;   // Asura_GetHashID( "180TurnToCrawlLeft" )
static const Asura_Hash_ID g_u180TurnToCrawlRight					= 0x1273c8f6;   // Asura_GetHashID( "180TurnToCrawlRight" )
static const Asura_Hash_ID g_u90TurnToCrawlLeft						= 0x904aebff;   // Asura_GetHashID( "90TurnToCrawlLeft" )
static const Asura_Hash_ID g_uForwardBeforeJump						= 0x904aebff;   // Asura_GetHashID( "CrawlForward" )
static const Asura_Hash_ID g_u90TurnToCrawlRight					= 0x7968f524;   // Asura_GetHashID( "90TurnToCrawlRight" )
static const Asura_Hash_ID g_u90TurnOnSpotLeft						= 0x088021fc;   // Asura_GetHashID( "90TurnOnSpotLeft" )
static const Asura_Hash_ID g_u90TurnOnSpotRight						= 0x07da7ec7;   // Asura_GetHashID( "90TurnOnSpotRight" )
static const Asura_Hash_ID g_uBeginFalling							= 0xd605276a;   // Asura_GetHashID( "Jump Drop Begin" )
static const Asura_Hash_ID g_uFalling								= 0xfd9c02f4;   // Asura_GetHashID( "Jump Drop Middle" )
static const Asura_Hash_ID g_uEndFalling							= 0xfaa0f3dc;   // Asura_GetHashID( "Jump Drop End" )
static const Asura_Hash_ID g_uDuctSurfaceSwitch						= 0x53143925;   // Asura_GetHashID( "Duct Surface Switch" )
static const Asura_Hash_ID g_u180TurnOnSpotLeft						= 0x9771c24e;   // Asura_GetHashID( "180TurnOnSpotLeft" )
static const Asura_Hash_ID g_u180TurnOnSpotRight					= 0x571ce8b5;   // Asura_GetHashID( "180TurnOnSpotRight" )
static const Asura_Hash_ID g_uConaveJumpToSurfaceLeft				= 0xafd858b2;   // Asura_GetHashID( "JumpToSurfaceConcaveLeft" )
static const Asura_Hash_ID g_uConaveJumpToSurfaceRight				= 0x4b891ed1;   // Asura_GetHashID( "JumpToSurfaceConcaveRight" )
static const Asura_Hash_ID g_uAIFallingBegin						= 0x22eaaa82;   // Asura_GetHashID( "FallingBegin" )
static const Asura_Hash_ID g_uAIFallingMiddle						= 0x4d66e2dc;   // Asura_GetHashID( "FallingMiddle" )
static const Asura_Hash_ID g_uAIFallingEnd							= 0x60bc5cf4;   // Asura_GetHashID( "FallingEnd" )
static const Asura_Hash_ID g_uAIHiss								= 0x0030df21;   // Asura_GetHashID( "Hiss" )

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Move_Alien_JumpSurface::AVP_Move_Alien_JumpSurface
// Author       : Andy Tate
// Date         : 4-Sep-2007
////////////////////////////////////////////////////////////////////////////////

AVP_Move_Alien_JumpSurface::AVP_Move_Alien_JumpSurface( Asura_Movement_System& xParentSystem, const Asura_Hash_ID uMoveHash )
: PARENT(xParentSystem, uMoveHash)
{
	// Initialise the move flags...
	SetDisableMovement( true );
	SetAimDirectionUsage( AVP_ClientEntity_Actor::AIM_DIRECTION_RESET );
	SetPreventsFlinch( true );
	SetDisableSurfaceReorientation( true );
}

/// End of function 'AVP_Move_Alien_JumpSurface::AVP_Move_Alien_JumpSurface'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Move_Alien_JumpSurface::Start
// Author       : Andy Tate
// Date         : 4-Sep-2007
////////////////////////////////////////////////////////////////////////////////

bool AVP_Move_Alien_JumpSurface::Start(const Asura_Msg_Move* pxStartMsg)
{
	// Have we been told to start via a message?
	if(pxStartMsg == NULL)
	{
		Asura_Assert(false, "Alien surface-interaction move can only be triggered by message.");
		return(false);
	}
	else
	{
		const AVP_Message_Alien_Move_JumpSurfaceStart*	pxMessage = static_cast<const AVP_Message_Alien_Move_JumpSurfaceStart*>(pxStartMsg);
		AVP_ClientActor_Alien& xAlienOwner = static_cast<AVP_ClientActor_Alien&>(GetAVPParentSystem().GetActorOwner());

		xAlienOwner.SetStanding( false, false );

		// We need to ensure the correct anim hash is set up, as it's about to get used.
		if (m_pxAVPAnimationData)
		{
			if (xAlienOwner.IsUsingFirstPersonSkin())
			{
				m_pxAVPAnimationData->SetAnimHash(m_uFirstPersonAnimHash);
			}
			else
			{
				m_pxAVPAnimationData->SetAnimHash(m_uThirdPersonAnimHash);
			}
		}

		const bool bHasAnimation = m_pxAVPAnimationData && m_pxAVPAnimationData->HasValidAnimation();
		const float	fDuration = (bHasAnimation) ? m_pxAVPAnimationData->GetAnimationDuration() : 0.500f;

		Asura_Vector_3 xTargetNormal;
		Asura_Vector_3 xTargetPosition;
		Asura_Matrix_3x3 xTargetOrientation;

		// Calculate our target position AND orientation...
		pxMessage->GetDestinationPosition(xTargetPosition);
		pxMessage->GetDestinationNormal(xTargetNormal);

		CalculateTargetOrientation(xTargetNormal, xTargetOrientation);


		// For the time being we'll just move and rotate to our desired position...
		if(bHasAnimation)
		{
			m_xOrientationInterpolationData.SetMovementOffsetOperation(Asura_MoveData_OrientationInterpolation::STRETCH_MOVEMENT_OFFSET);
			m_xPositionInterpolationData.SetMovementOffsetOperation(Asura_MoveData_PositionInterpolation::STRETCH_MOVEMENT_OFFSET);
		}

		m_xOrientationInterpolationData.SetDuration(fDuration);
		m_xOrientationInterpolationData.SetFinal(xTargetOrientation);
		m_xPositionInterpolationData.SetDuration(fDuration);
		m_xPositionInterpolationData.SetFinal(xTargetPosition);

		InitialiseOrientationInterpolation(m_xOrientationInterpolationData);
		InitialisePositionInterpolation(m_xPositionInterpolationData);

		// Finally, call the base-function...
		return(PARENT::Start(pxStartMsg));
	}
}

/// End of function 'AVP_Move_Alien_JumpSurface::Start'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Move_Alien_JumpSurface180::AVP_Move_Alien_JumpSurface180
// Author       : Andy Tate
// Date         : 4-Sep-2007
////////////////////////////////////////////////////////////////////////////////

AVP_Move_Alien_JumpSurface180::AVP_Move_Alien_JumpSurface180(Asura_Movement_System& xParentSystem)
: PARENT(xParentSystem, GetStaticHash())
{
	SetUseSingularityAxis( true );

	Asura_Hash_ID uFPAnim = 0x8C72BDDC;				// Alien_hud_jump_floor_to_ceiling_3m
	Asura_Hash_ID uTPAnim = ASURA_HASH_ID_UNSET;	// 
	Asura_Value xValue;

	Asura_Blueprint* pxBP = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_ACTOR_STATE, g_uPlayerJumpBP );
	if ( pxBP )
	{
		if ( pxBP->GetParameterValueByHashID( g_u180WallToWallAnimParam, xValue ) )
		{
			uFPAnim = xValue.GetHashIDValue();
		}
	}

	pxBP = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_ACTOR_STATE, g_uAIJumpBP );
	if ( pxBP )
	{
		if ( pxBP->GetParameterValueByHashID( g_u180WallToWallAnimParam, xValue ) )
		{
			uTPAnim = xValue.GetHashIDValue();
		}
	}

	SetAnimHashes( uFPAnim, uTPAnim );
}

/// End of function 'AVP_Move_Alien_JumpSurface180::AVP_Move_Alien_JumpSurface180'. ///


////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Move_Alien_JumpSurface180::CalculateTargetOrientation
// Author       : Andy Tate
// Date         : 13-Sep-2007
////////////////////////////////////////////////////////////////////////////////

void AVP_Move_Alien_JumpSurface180::CalculateTargetOrientation(const Asura_Vector_3& xTargetNormal,
															   Asura_Matrix_3x3& xTargetOrientation) const
{
	const AVP_ClientActor_Vertebrate& xOwner = static_cast<const AVP_ClientActor_Vertebrate&>(GetAVPParentSystem().GetActorOwner());
	Asura_AMatrix3x3 xAlignedOri;

	AVP_Alien_JumpHandler::CalculateTargetOrientation(AVP_Alien_JumpHandler::JT_JUMP_180_WALL_TO_WALL, xOwner, xTargetNormal, xAlignedOri);

	xAlignedOri.ToMat3(xTargetOrientation);
}

/// End of function 'AVP_Move_Alien_JumpSurface180::CalculateTargetOrientation'. ///


//////////////////////////////////////////////////////////////////////////////
//
// AVP_Move_Alien_JumpSurfaceConcave90Begin
//
//////////////////////////////////////////////////////////////////////////////

AVP_Move_Alien_JumpSurfaceConcave90Begin::AVP_Move_Alien_JumpSurfaceConcave90Begin( Asura_Movement_System& xParentSystem )
: PARENT( xParentSystem, GetStaticHash() )
{
	Asura_Hash_ID uFPAnim = ASURA_HASH_ID_UNSET;	// player skips this bit
	Asura_Hash_ID uTPAnim = 0xad3d0cb1;				// Alien_Warrior_med_into_floor_to_wall_med
	Asura_Value xValue;

	Asura_Blueprint* pxBP = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_ACTOR_STATE, g_uPlayerJumpBP );
	if ( pxBP )
	{
		if ( pxBP->GetParameterValueByHashID( g_uConcave90FloorToWallBeginAnimParam, xValue ) )
		{
			uFPAnim = xValue.GetHashIDValue();
		}
	}

	pxBP = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_ACTOR_STATE, g_uAIJumpBP );
	if ( pxBP )
	{
		if ( pxBP->GetParameterValueByHashID( g_uConcave90FloorToWallBeginAnimParam, xValue ) )
		{
			uTPAnim = xValue.GetHashIDValue();
		}
	}

	SetAnimHashes( uFPAnim, uTPAnim );

    // Only tween in to our anim.
	if (m_pxAVPAnimationData)
	{
		m_pxAVPAnimationData->SetTweenData( 0.05f, 0.0f, true );
	}

    AddNextMove( AVP_Move_Alien_JumpSurfaceConcave90Middle::GetStaticHash() );
	//## fixme - Sort this out once the move interface has been cleaned up.
	//## Uncommenting this leads to the physics object glitching into the environment after our AVP_Move_Alien_JumpSurfaceConcave90End move.
	//SetVelocityBased( true );

	SetPreventsFlinch( true );
}

bool AVP_Move_Alien_JumpSurfaceConcave90Begin::Start( const Asura_Msg_Move* pxStartMsg )
{
    if ( !pxStartMsg )
    {
        Asura_Assert(false, "AVP_Move_Alien_JumpSurfaceConcave90Begin::Start - Alien surface-interaction move can only be triggered by message.");
        return false;
    }
    else
    {
        const AVP_Message_Alien_Move_JumpSurfaceStart* pxMessage = static_cast< const AVP_Message_Alien_Move_JumpSurfaceStart* >( pxStartMsg );

        // Pass the details on down the chain.
        AVP_Move_Alien_JumpSurfaceConcave90Middle* pxNextMove =
            static_cast< AVP_Move_Alien_JumpSurfaceConcave90Middle* >( GetParentSystem().GetMove( AVP_Move_Alien_JumpSurfaceConcave90Middle::GetStaticHash() ) );
        if ( pxNextMove )
        {
            pxNextMove->SetJumpStartMessage( *pxMessage );
        }

        return PARENT::Start( pxStartMsg );
    }
}

void AVP_Move_Alien_JumpSurfaceConcave90Begin::Abort()
{
	ResetPhysicsObjectHandler();

    PARENT::Abort();
}

//////////////////////////////////////////////////////////////////////////////
//
// AVP_Move_Alien_JumpSurfaceConcave90Middle
//
//////////////////////////////////////////////////////////////////////////////

AVP_Move_Alien_JumpSurfaceConcave90Middle::AVP_Move_Alien_JumpSurfaceConcave90Middle(Asura_Movement_System& xParentSystem)
: PARENT(xParentSystem, GetStaticHash())
, m_xJumpStartMessage()
{
	Asura_Hash_ID uFPAnim = 0xf48ac6a8;	// Alien_hud_floor_to_wall_med
	Asura_Hash_ID uTPAnim = 0x3c58d233;	// Alien_Warrior_floor_to_wall_med
	Asura_Value xValue;

	Asura_Blueprint* pxBP = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_ACTOR_STATE, g_uPlayerJumpBP );
	if ( pxBP )
	{
		if ( pxBP->GetParameterValueByHashID( g_uConcave90FloorToWallMiddleAnimParam, xValue ) )
		{
			uFPAnim = xValue.GetHashIDValue();
		}
	}

	pxBP = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_ACTOR_STATE, g_uAIJumpBP );
	if ( pxBP )
	{
		if ( pxBP->GetParameterValueByHashID( g_uConcave90FloorToWallMiddleAnimParam, xValue ) )
		{
			uTPAnim = xValue.GetHashIDValue();
		}
	}

	SetAnimHashes( uFPAnim, uTPAnim );

	// Don't tween in or out of our anim.
	if (m_pxAVPAnimationData)
	{
		m_pxAVPAnimationData->SetTweenData( 0.0f, 0.0f, false );
	}

    AddNextMove( AVP_Move_Alien_JumpSurfaceConcave90End::GetStaticHash() );
}

void AVP_Move_Alien_JumpSurfaceConcave90Middle::SetJumpStartMessage( const AVP_Message_Alien_Move_JumpSurfaceStart& xJumpStartMessage )
{
    m_xJumpStartMessage = xJumpStartMessage;

    // Also, pass this message on to our next move.
    AVP_Move_Alien_JumpSurfaceConcave90End* pxNextMove =
        static_cast< AVP_Move_Alien_JumpSurfaceConcave90End* >( GetParentSystem().GetMove( AVP_Move_Alien_JumpSurfaceConcave90End::GetStaticHash() ) );
    if ( pxNextMove )
    {
        pxNextMove->SetJumpStartMessage( xJumpStartMessage );
    }
}

bool AVP_Move_Alien_JumpSurfaceConcave90Middle::Start( const Asura_Msg_Move* pxStartMsg /*= NULL*/ )
{
    // If we have a start message we've come straight into here without going via the start move.
    // This is probably because we're a player.
    if ( pxStartMsg )
    {
        const AVP_Message_Alien_Move_JumpSurfaceStart* pxJumpStartMessage =
            static_cast< const AVP_Message_Alien_Move_JumpSurfaceStart* >( pxStartMsg );
        AVP_Move_Alien_JumpSurfaceConcave90End* pxNextMove =
            static_cast< AVP_Move_Alien_JumpSurfaceConcave90End* >( GetParentSystem().GetMove( AVP_Move_Alien_JumpSurfaceConcave90End::GetStaticHash() ) );
        if ( pxNextMove )
        {
            pxNextMove->SetJumpStartMessage( *pxJumpStartMessage );
        }

        return PARENT::Start( pxStartMsg );
    }
    else
    {
        return PARENT::Start( &m_xJumpStartMessage );
    }
}

void AVP_Move_Alien_JumpSurfaceConcave90Middle::CalculateTargetOrientation( const Asura_Vector_3& xTargetNormal,
                                                                      Asura_Matrix_3x3& xTargetOrientation ) const
{
	const AVP_ClientActor_Vertebrate& xOwner = static_cast<const AVP_ClientActor_Vertebrate&>(GetAVPParentSystem().GetActorOwner());
	Asura_AMatrix3x3 xAlignedOri;

	AVP_Alien_JumpHandler::CalculateTargetOrientation(AVP_Alien_JumpHandler::JT_JUMP_CONCAVE_90_FLOOR_TO_WALL, xOwner, xTargetNormal, xAlignedOri);

	xAlignedOri.ToMat3(xTargetOrientation);
}

//////////////////////////////////////////////////////////////////////////////
//
// AVP_Move_Alien_JumpSurfaceConcave90End
//
//////////////////////////////////////////////////////////////////////////////

AVP_Move_Alien_JumpSurfaceConcave90End::AVP_Move_Alien_JumpSurfaceConcave90End( Asura_Movement_System& xParentSystem )
: PARENT( xParentSystem, GetStaticHash() )
{
	Asura_Hash_ID uFPAnim = 0xcc4f7324;	// Alien_hud_floor_to_wall_med_into_med
	Asura_Hash_ID uTPAnim = 0x37fe33f9;	// Alien_Warrior_floor_to_wall_med_into_med
	Asura_Value xValue;

	Asura_Blueprint* pxBP = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_ACTOR_STATE, g_uPlayerJumpBP );
	if ( pxBP )
	{
		if ( pxBP->GetParameterValueByHashID( g_uConcave90FloorToWallEndAnimParam, xValue ) )
		{
			uFPAnim = xValue.GetHashIDValue();
		}
	}

	pxBP = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_ACTOR_STATE, g_uAIJumpBP );
	if ( pxBP )
	{
		if ( pxBP->GetParameterValueByHashID( g_uConcave90FloorToWallEndAnimParam, xValue ) )
		{
			uTPAnim = xValue.GetHashIDValue();
		}
	}

	SetAnimHashes( uFPAnim, uTPAnim );

    // Don't tween in or out of our anim.
	if (m_pxAVPAnimationData)
	{
		m_pxAVPAnimationData->SetTweenData( 0.0f, 0.05f, false );
	}

    SetPositionUsage(POSITION_SET_FROM_VELOCITY);

	SetPreventsFlinch( true );
}

void AVP_Move_Alien_JumpSurfaceConcave90End::SetJumpStartMessage( const AVP_Message_Alien_Move_JumpSurfaceStart& xJumpStartMessage )
{
}

bool AVP_Move_Alien_JumpSurfaceConcave90End::Start( const Asura_Msg_Move* pxStartMsg /*= NULL*/ )
{
    return PARENT::Start( pxStartMsg );
}

void AVP_Move_Alien_JumpSurfaceConcave90End::Finish()
{
	ResetPhysicsObjectHandler();

    PARENT::Finish();
}

void AVP_Move_Alien_JumpSurfaceConcave90End::Abort()
{
	ResetPhysicsObjectHandler();

    PARENT::Abort();
}

//////////////////////////////////////////////////////////////////////////////
//
// AVP_Move_Alien_JumpSurfaceConcave90
//
//////////////////////////////////////////////////////////////////////////////

AVP_Move_Alien_JumpSurfaceConcave90::AVP_Move_Alien_JumpSurfaceConcave90(Asura_Movement_System& xParentSystem)
: PARENT(xParentSystem, GetStaticHash())
{
	Asura_Hash_ID uFPAnim = 0xf48ac6a8;	// Alien_hud_floor_to_wall_med
	Asura_Hash_ID uTPAnim = 0x3c58d233;	// Alien_Warrior_floor_to_wall_med
	Asura_Value xValue;

	Asura_Blueprint* pxBP = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_ACTOR_STATE, g_uPlayerJumpBP );
	if ( pxBP )
	{
		if ( pxBP->GetParameterValueByHashID( g_uConcave90FloorToWallAllInOneAnimParam, xValue ) )
		{
			uFPAnim = xValue.GetHashIDValue();
		}
	}

	pxBP = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_ACTOR_STATE, g_uAIJumpBP );
	if ( pxBP )
	{
		if ( pxBP->GetParameterValueByHashID( g_uConcave90FloorToWallAllInOneAnimParam, xValue ) )
		{
			uTPAnim = xValue.GetHashIDValue();
		}
	}

	SetAnimHashes( uFPAnim, uTPAnim );

	const AVP_ClientActor_Alien& xAlienOwner = static_cast<const AVP_ClientActor_Alien&>(GetAVPParentSystem().GetActorOwner());
	if (m_pxAVPAnimationData && xAlienOwner.IsPlayer())
	{
		// Make sure players get some tween data.
		m_pxAVPAnimationData->SetTweenData(0.1f, 0.1f, true);
	}
}

void AVP_Move_Alien_JumpSurfaceConcave90::CalculateTargetOrientation(const Asura_Vector_3& xTargetNormal,
                                                                     Asura_Matrix_3x3& xTargetOrientation) const
{
	const AVP_ClientActor_Vertebrate& xOwner = static_cast<const AVP_ClientActor_Vertebrate&>(GetAVPParentSystem().GetActorOwner());
	Asura_AMatrix3x3 xAlignedOri;

	AVP_Alien_JumpHandler::CalculateTargetOrientation(AVP_Alien_JumpHandler::JT_JUMP_CONCAVE_90_FLOOR_TO_WALL, xOwner, xTargetNormal, xAlignedOri);

	xAlignedOri.ToMat3(xTargetOrientation);
}


AVP_Move_Alien_JumpDropStart::AVP_Move_Alien_JumpDropStart( Asura_Movement_System& xParentSystem )
: PARENT( xParentSystem, GetStaticHash() )
, m_xAwayFromSurfaceStart(Asura_Maths::ZeroAVector3)
, m_xOrientationInterpolationData()
, m_fTimeSinceStart(0.0f)
, m_bInAir(false)
, m_bFinished(false)
{
	Asura_Hash_ID uFPAnim = ASURA_HASH_ID_UNSET;
	Asura_Hash_ID uTPAnim = ASURA_HASH_ID_UNSET;
	Asura_Value xValue;

	Asura_Blueprint* pxBP = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_ACTOR_STATE, g_uPlayerJumpBP );
	if ( pxBP )
	{
		if ( pxBP->GetParameterValueByHashID( g_uDropStartAnimParam, xValue ) )
		{
			uFPAnim = xValue.GetHashIDValue();
		}
	}

	pxBP = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_ACTOR_STATE, g_uAIJumpBP );
	if ( pxBP )
	{
		if ( pxBP->GetParameterValueByHashID( g_uDropStartAnimParam, xValue ) )
		{
			uTPAnim = xValue.GetHashIDValue();
		}
	}

	SetAnimHashes( uFPAnim, uTPAnim );

	SetAimDirectionUsage( AVP_ClientEntity_Actor::AIM_DIRECTION_RESET );
	SetPreventsFlinch( true );
	SetPositionUsage(Asura_Move::POSITION_SET_FROM_VELOCITY);
}

bool AVP_Move_Alien_JumpDropStart::Start(const Asura_Msg_Move* pxStartMsg)
{
	AVP_ClientActor_Alien& xAlienOwner = static_cast<AVP_ClientActor_Alien&>(GetAVPParentSystem().GetActorOwner());

	// Calculate our target orientation...
	Asura_AMatrix3x3 xTargetOrientation;
	CalculateTargetOrientation(xTargetOrientation);

	// For the time being we'll just rotate to our desired position...
	const float	fDuration = 0.500f;
	m_xOrientationInterpolationData.SetDuration( fDuration );
	Asura_Matrix_3x3 xTargetOri;
	xTargetOrientation.ToMat3( xTargetOri );
	m_xOrientationInterpolationData.SetInitial( xAlienOwner.GetCurrentOrientation() );
	m_xOrientationInterpolationData.SetFinal( xTargetOri );
	m_xOrientationInterpolationData.SetSingularityHintAxis( Asura_AVector3( 0.0f, 0.0f, 1.0f ) );
	InitialiseOrientationInterpolation( m_xOrientationInterpolationData );

	// set the gravity down if we on the server machine
	AVP_PhysicsObjectHandler* pxPhyHandler = NULL;
	if (Asura_Network::IsServer())
	{
		AVP_ServerActor_Vertebrate* pxOwner = AVP_ServerActor_Vertebrate::FindEntity(GetAVPParentSystem().GetOwnerGuid());
		pxPhyHandler = pxOwner ? pxOwner->GetPhysicsObjectHandler() : NULL;
	}

	if ( pxPhyHandler == NULL )
	{
		AVP_ClientActor_Vertebrate* pxOwner = AVP_ClientActor_Vertebrate::FindEntity(GetAVPParentSystem().GetOwnerGuid());
		pxPhyHandler = pxOwner ? pxOwner->GetPhysicsObjectHandler() : NULL;
	}

	Asura_Assertf( pxPhyHandler, ( "Unable to get physics object handler for entity %d" , GetAVPParentSystem().GetOwnerGuid() ) );
	if ( pxPhyHandler )
	{
		pxPhyHandler->SetForcedTargetSurfaceNormal(Asura_Vector_3(0.0f, -1.0f, 0.0f));
		pxPhyHandler->GetCurrentSurfaceNormal(m_xAwayFromSurfaceStart);
	}

	m_fTimeSinceStart = 0.0f;
	m_bInAir = false;
	m_bFinished = false;

	// Finally, call the base-function...
	return(PARENT::Start(pxStartMsg));
}

void AVP_Move_Alien_JumpDropStart::Finish()
{
	AVP_PhysicsObjectHandler* pxPhyHandler = NULL;
	if (Asura_Network::IsServer())
	{
		AVP_ServerActor_Vertebrate* pxOwner = AVP_ServerActor_Vertebrate::FindEntity(GetAVPParentSystem().GetOwnerGuid());
		pxPhyHandler = pxOwner ? pxOwner->GetPhysicsObjectHandler() : NULL;
	}

	if ( pxPhyHandler == NULL )
	{
		AVP_ClientActor_Vertebrate* pxOwner = AVP_ClientActor_Vertebrate::FindEntity(GetAVPParentSystem().GetOwnerGuid());
		pxPhyHandler = pxOwner ? pxOwner->GetPhysicsObjectHandler() : NULL;
	}

	Asura_Assertf( pxPhyHandler, ( "Unable to get physics object handler for entity %d" , GetAVPParentSystem().GetOwnerGuid() ) );
	if ( pxPhyHandler )
	{
		pxPhyHandler->UnsetForcedTargetSurfaceNormal();
	}
	PARENT::Finish();
}

void AVP_Move_Alien_JumpDropStart::Abort()
{
	AVP_PhysicsObjectHandler* pxPhyHandler = NULL;
	if (Asura_Network::IsServer())
	{
		AVP_ServerActor_Vertebrate* pxOwner = AVP_ServerActor_Vertebrate::FindEntity(GetAVPParentSystem().GetOwnerGuid());
		pxPhyHandler = pxOwner ? pxOwner->GetPhysicsObjectHandler() : NULL;
	}

	if ( pxPhyHandler == NULL )
	{
		AVP_ClientActor_Vertebrate* pxOwner = AVP_ClientActor_Vertebrate::FindEntity(GetAVPParentSystem().GetOwnerGuid());
		pxPhyHandler = pxOwner ? pxOwner->GetPhysicsObjectHandler() : NULL;
	}

	Asura_Assertf( pxPhyHandler, ( "Unable to get physics object handler for entity %d" , GetAVPParentSystem().GetOwnerGuid() ) );
	if ( pxPhyHandler )
	{
		pxPhyHandler->UnsetForcedTargetSurfaceNormal();
	}
	PARENT::Abort();
}

void AVP_Move_Alien_JumpDropStart::UpdateStates(const float fTimeStep)
{
	bool bInContactWithFloor = false;
	AVP_Actor_PhysicsObjectHandler* pxPhyHandler = NULL;

	m_fTimeSinceStart += fTimeStep;

	if (Asura_Network::IsServer())
	{
		AVP_ServerEntity_Actor* pxActorOwner = AVP_ServerEntity_Actor::FindEntity( GetAVPParentSystem().GetOwnerGuid() );
		if ( pxActorOwner )
		{
			pxPhyHandler = pxActorOwner->GetPhysicsObjectHandler();
		}
	}

	if ( pxPhyHandler == NULL )
	{
		AVP_ClientEntity_Actor* pxActorOwner = AVP_ClientEntity_Actor::FindEntity( GetAVPParentSystem().GetOwnerGuid() );
		if ( pxActorOwner )
		{
			pxPhyHandler = pxActorOwner->GetPhysicsObjectHandler();
		}
	}

	Asura_Assert(pxPhyHandler, "PhysicsObjectHandler not found");

	bInContactWithFloor = pxPhyHandler ? pxPhyHandler->GetPhysicsObject()->IsSupported() : true;

	if ( m_bInAir && m_fTimeSinceStart > 0.2f && bInContactWithFloor )
	{
		m_bFinished = true;
	}
	else if ( !bInContactWithFloor )
	{
		m_bInAir = true;
	}
	else if ( m_fTimeSinceStart > 5.0f )
	{
		m_bFinished = true;
	}

	// release the orientation
	if ( m_bFinished )
	{
		pxPhyHandler->UnsetForcedTargetSurfaceNormal();
	}

	PARENT::UpdateStates(fTimeStep);
}

bool AVP_Move_Alien_JumpDropStart::HasFinished() const
{ 
	if (!m_bFinished)
	{
		return false;
	}
	else
	{
		return PARENT::HasFinished();
	}
}

DEBUGVAR float dbg_fAwayFromWallVelocity = 5.0f;

bool AVP_Move_Alien_JumpDropStart::CalculateVelocity(const float fTimeStep, Asura_AVector3_OutParam xVelocity) const
{
	bool bOK = PARENT::CalculateVelocity(fTimeStep, xVelocity);
	if (m_fTimeSinceStart < 1.0f )
	{
		xVelocity += ( m_xAwayFromSurfaceStart * dbg_fAwayFromWallVelocity ) + Asura_AVector3( 0.0f, -1.0f, 0.0f );
	}
	return bOK;
}

void AVP_Move_Alien_JumpDropStart::CalculateTargetOrientation( Asura_AMatrix3x3& xTargetOrientation ) const
{
	const AVP_ClientActor_Vertebrate& xOwner = static_cast<const AVP_ClientActor_Vertebrate&>(GetAVPParentSystem().GetActorOwner());
	const Asura_AVector3 xDown(0.0f, 1.0f, 0.0f);
	Asura_AVector3 xFwd;
	xOwner.GetAimDirection(xFwd);
	Asura_AVector3 xRight = xDown ^ xFwd;
	xRight.Normalise();
	xFwd = xRight ^ xDown;
	xTargetOrientation.SetColumn0( xRight );
	xTargetOrientation.SetColumn1( xDown );
	xTargetOrientation.SetColumn2( xFwd );
}

//////////////////////////////////////////////////////////////////////////////
//
// Hop left/right.
//
//////////////////////////////////////////////////////////////////////////////

AVP_Move_Alien_HopLeft::AVP_Move_Alien_HopLeft( Asura_Movement_System& xParentSystem )
: PARENT( xParentSystem, GetStaticHash() )
{
	SetOrientationUsage( ORIENTATION_SET_DIRECTLY );
	SetPositionUsage( POSITION_SET_FROM_VELOCITY );

	Asura_Hash_ID uFPAnim = ASURA_HASH_ID_UNSET;
	Asura_Hash_ID uTPAnim = 0xf38d5712; // Asura_GetHashID( "alien_warrior_hop_left" )
	Asura_Value xValue;

	Asura_Blueprint* pxBP = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_ACTOR_STATE, g_uPlayerJumpBP );
	if ( pxBP )
	{
		if ( pxBP->GetParameterValueByHashID( g_uHopLeftParam, xValue ) )
		{
			uFPAnim = xValue.GetHashIDValue();
		}
	}

	pxBP = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_ACTOR_STATE, g_uAIJumpBP );
	if ( pxBP )
	{
		if ( pxBP->GetParameterValueByHashID( g_uHopLeftParam, xValue ) )
		{
			uTPAnim = xValue.GetHashIDValue();
		}
	}

	SetAnimHashes( uFPAnim, uTPAnim ); 

	SetPreventsFlinch( true );
	SetDisableSurfaceReorientation( true );
}

bool AVP_Move_Alien_HopLeft::Start( const Asura_Msg_Move* pxStartMsg )
{
	if ( m_pxAVPAnimationData )
	{
		// We're not tweening out as this goes straight into crawl.
		m_pxAVPAnimationData->SetTweenData( 0.05f, 0.0f, true );
	}

	return PARENT::Start( pxStartMsg );
}

AVP_Move_Alien_HopRight::AVP_Move_Alien_HopRight( Asura_Movement_System& xParentSystem )
: PARENT( xParentSystem, GetStaticHash() )
{
	SetOrientationUsage( ORIENTATION_SET_DIRECTLY );
	SetPositionUsage( POSITION_SET_FROM_VELOCITY );

	Asura_Hash_ID uFPAnim = ASURA_HASH_ID_UNSET;
	Asura_Hash_ID uTPAnim = 0x7e73ec71; // Asura_GetHashID( "alien_warrior_hop_right" )
	Asura_Value xValue;

	Asura_Blueprint* pxBP = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_ACTOR_STATE, g_uPlayerJumpBP );
	if ( pxBP )
	{
		if ( pxBP->GetParameterValueByHashID( g_uHopRightParam, xValue ) )
		{
			uFPAnim = xValue.GetHashIDValue();
		}
	}

	pxBP = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_ACTOR_STATE, g_uAIJumpBP );
	if ( pxBP )
	{
		if ( pxBP->GetParameterValueByHashID( g_uHopRightParam, xValue ) )
		{
			uTPAnim = xValue.GetHashIDValue();
		}
	}

	SetAnimHashes( uFPAnim, uTPAnim );

	SetPreventsFlinch( true );
	SetDisableSurfaceReorientation( true );
}

bool AVP_Move_Alien_HopRight::Start( const Asura_Msg_Move* pxStartMsg )
{
	if ( m_pxAVPAnimationData )
	{
		// We're not tweening out as this goes straight into crawl.
		m_pxAVPAnimationData->SetTweenData( 0.05f, 0.0f, true );
	}

	return PARENT::Start( pxStartMsg );
}

AVP_Move_Alien_WallHopLeft::AVP_Move_Alien_WallHopLeft( Asura_Movement_System& xParentSystem )
: PARENT( xParentSystem, GetStaticHash() )
{
	SetOrientationUsage( ORIENTATION_SET_DIRECTLY );
	SetPositionUsage( POSITION_SET_FROM_VELOCITY );

	Asura_Hash_ID uFPAnim = ASURA_HASH_ID_UNSET;
	Asura_Hash_ID uTPAnim = 0x30cdeb04; // Asura_GetHashID( "alien_warrior_dodge_left" )
	Asura_Value xValue;

	Asura_Blueprint* pxBP = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_ACTOR_STATE, g_uPlayerJumpBP );
	if ( pxBP )
	{
		if ( pxBP->GetParameterValueByHashID( g_uWallHopLeftParam, xValue ) )
		{
			uFPAnim = xValue.GetHashIDValue();
		}
	}

	pxBP = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_ACTOR_STATE, g_uAIJumpBP );
	if ( pxBP )
	{
		if ( pxBP->GetParameterValueByHashID( g_uWallHopLeftParam, xValue ) )
		{
			uTPAnim = xValue.GetHashIDValue();
		}
	}

	SetAnimHashes( uFPAnim, uTPAnim ); 

	SetPreventsFlinch( true );
	SetDisableSurfaceReorientation( true );
}

bool AVP_Move_Alien_WallHopLeft::Start( const Asura_Msg_Move* pxStartMsg )
{
	if ( m_pxAVPAnimationData )
	{
		// We're not tweening out as this goes straight into crawl.
		m_pxAVPAnimationData->SetTweenData( 0.05f, 0.0f, true );
	}

	return PARENT::Start( pxStartMsg );
}

AVP_Move_Alien_WallHopRight::AVP_Move_Alien_WallHopRight( Asura_Movement_System& xParentSystem )
: PARENT( xParentSystem, GetStaticHash() )
{
	SetOrientationUsage( ORIENTATION_SET_DIRECTLY );
	SetPositionUsage( POSITION_SET_FROM_VELOCITY );

	Asura_Hash_ID uFPAnim = ASURA_HASH_ID_UNSET;
	Asura_Hash_ID uTPAnim = 0xe945d6bf; // Asura_GetHashID( "alien_warrior_dodge_right" )
	Asura_Value xValue;

	Asura_Blueprint* pxBP = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_ACTOR_STATE, g_uPlayerJumpBP );
	if ( pxBP )
	{
		if ( pxBP->GetParameterValueByHashID( g_uWallHopRightParam, xValue ) )
		{
			uFPAnim = xValue.GetHashIDValue();
		}
	}

	pxBP = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_ACTOR_STATE, g_uAIJumpBP );
	if ( pxBP )
	{
		if ( pxBP->GetParameterValueByHashID( g_uWallHopRightParam, xValue ) )
		{
			uTPAnim = xValue.GetHashIDValue();
		}
	}

	SetAnimHashes( uFPAnim, uTPAnim );

	SetPreventsFlinch( true );
	SetDisableSurfaceReorientation( true );
}

bool AVP_Move_Alien_WallHopRight::Start( const Asura_Msg_Move* pxStartMsg )
{
	if ( m_pxAVPAnimationData )
	{
		// We're not tweening out as this goes straight into crawl.
		m_pxAVPAnimationData->SetTweenData( 0.05f, 0.0f, true );
	}

	return PARENT::Start( pxStartMsg );
}

//////////////////////////////////////////////////////////////////////////////
//
// Jump floor -> wall left/right.
//
//////////////////////////////////////////////////////////////////////////////

AVP_Move_Alien_JumpOnToWallLeft::AVP_Move_Alien_JumpOnToWallLeft( Asura_Movement_System& xParentSystem )
: PARENT( xParentSystem, GetStaticHash() )
{
	Asura_Hash_ID uFPAnim = ASURA_HASH_ID_UNSET;
	Asura_Hash_ID uTPAnim = 0x6ec40ed9; // Asura_GetHashID( "alien_warrior_crawl_hop_wall_left" )
	Asura_Value xValue;

	Asura_Blueprint* pxBP = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_ACTOR_STATE, g_uPlayerJumpBP );
	if ( pxBP )
	{
		if ( pxBP->GetParameterValueByHashID( g_uJumpOntoWallLeftParam, xValue ) )
		{
			uFPAnim = xValue.GetHashIDValue();
		}
	}

	pxBP = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_ACTOR_STATE, g_uAIJumpBP );
	if ( pxBP )
	{
		if ( pxBP->GetParameterValueByHashID( g_uJumpOntoWallLeftParam, xValue ) )
		{
			uTPAnim = xValue.GetHashIDValue();
		}
	}

	SetAnimHashes( uFPAnim, uTPAnim );
}

void AVP_Move_Alien_JumpOnToWallLeft::CalculateTargetOrientation( const Asura_Vector_3& xTargetNormal, Asura_Matrix_3x3& xTargetOrientation ) const
{
	const AVP_ClientActor_Vertebrate& xOwner = static_cast< const AVP_ClientActor_Vertebrate& >( GetAVPParentSystem().GetActorOwner() );
	Asura_AMatrix3x3 xAlignedOri;
	AVP_Alien_JumpHandler::CalculateTargetOrientation( AVP_Alien_JumpHandler::JT_JUMP_CONCAVE_90_FLOOR_TO_WALL, xOwner, xTargetNormal, xAlignedOri );
	xAlignedOri.ToMat3(xTargetOrientation);
}

AVP_Move_Alien_JumpOnToWallRight::AVP_Move_Alien_JumpOnToWallRight( Asura_Movement_System& xParentSystem )
: PARENT( xParentSystem, GetStaticHash() )
{
	Asura_Hash_ID uFPAnim = ASURA_HASH_ID_UNSET;
	Asura_Hash_ID uTPAnim = 0x6a142d8a; // Asura_GetHashID( "alien_warrior_crawl_hop_wall_right" )
	Asura_Value xValue;

	Asura_Blueprint* pxBP = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_ACTOR_STATE, g_uPlayerJumpBP );
	if ( pxBP )
	{
		if ( pxBP->GetParameterValueByHashID( g_uJumpOntoWallRightParam, xValue ) )
		{
			uFPAnim = xValue.GetHashIDValue();
		}
	}

	pxBP = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_ACTOR_STATE, g_uAIJumpBP );
	if ( pxBP )
	{
		if ( pxBP->GetParameterValueByHashID( g_uJumpOntoWallRightParam, xValue ) )
		{
			uTPAnim = xValue.GetHashIDValue();
		}
	}

	SetAnimHashes( uFPAnim, uTPAnim );
}

void AVP_Move_Alien_JumpOnToWallRight::CalculateTargetOrientation( const Asura_Vector_3& xTargetNormal, Asura_Matrix_3x3& xTargetOrientation ) const
{
	const AVP_ClientActor_Vertebrate& xOwner = static_cast< const AVP_ClientActor_Vertebrate& >( GetAVPParentSystem().GetActorOwner() );
	Asura_AMatrix3x3 xAlignedOri;
	AVP_Alien_JumpHandler::CalculateTargetOrientation( AVP_Alien_JumpHandler::JT_JUMP_CONCAVE_90_FLOOR_TO_WALL, xOwner, xTargetNormal, xAlignedOri );
	xAlignedOri.ToMat3(xTargetOrientation);
}

//////////////////////////////////////////////////////////////////////////////
//
// 180 turn-and-go.
//
//////////////////////////////////////////////////////////////////////////////

AVP_Move_Alien_180TurnToCrawl::AVP_Move_Alien_180TurnToCrawl( Asura_Movement_System& xParentSystem )
: PARENT( xParentSystem, GetStaticHash() )
{
	SetOrientationUsage( ORIENTATION_SET_DIRECTLY );
	SetPositionUsage( POSITION_SET_FROM_VELOCITY );
	SetAimDirectionUsage( AVP_ClientEntity_Actor::AIM_DIRECTION_RESET );
	SetLookAtPositionUsage( AVP_ClientEntity_Actor::LOOK_AT_POSITION_RESET );

	Asura_Hash_ID uTPAnim = ASURA_HASH_ID_UNSET;
	Asura_Blueprint* pxBP = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_MOVESALIEN, g_uAIMovesBP );
	if ( pxBP )
	{
		Asura_Value xValue;
		if ( pxBP->GetParameterValueByHashID( g_u180TurnToCrawlRight, xValue ) )
		{
			uTPAnim = xValue.GetHashIDValue();
		}
	}
	SetAnimHashes( ASURA_HASH_ID_UNSET, uTPAnim );
	if ( m_pxAVPAnimationData )
	{
		// We're not tweening out as this goes straight into crawl.
		m_pxAVPAnimationData->SetTweenData( 0.05f, 0.0f, true );
	}

	SetPreventsFlinch( true );
}

//////////////////////////////////////////////////////////////////////////////
//
// 90 turn-and-go.
//
//////////////////////////////////////////////////////////////////////////////

AVP_Move_Alien_90TurnToCrawlLeft::AVP_Move_Alien_90TurnToCrawlLeft( Asura_Movement_System& xParentSystem )
: PARENT( xParentSystem, GetStaticHash() )
{
	SetOrientationUsage( ORIENTATION_SET_DIRECTLY );
	SetPositionUsage( POSITION_SET_FROM_VELOCITY );
	SetAimDirectionUsage( AVP_ClientEntity_Actor::AIM_DIRECTION_RESET );
	SetLookAtPositionUsage( AVP_ClientEntity_Actor::LOOK_AT_POSITION_RESET );

	Asura_Hash_ID uTPAnim = ASURA_HASH_ID_UNSET;
	Asura_Blueprint* pxBP = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_MOVESALIEN, g_uAIMovesBP );
	if ( pxBP )
	{
		Asura_Value xValue;
		if ( pxBP->GetParameterValueByHashID( g_u90TurnToCrawlLeft, xValue ) )
		{
			uTPAnim = xValue.GetHashIDValue();
		}
	}
	SetAnimHashes( ASURA_HASH_ID_UNSET, uTPAnim );
	if ( m_pxAVPAnimationData )
	{
		// We're not tweening out as this goes straight into crawl.
		m_pxAVPAnimationData->SetTweenData( 0.05f, 0.0f, true );
	}

	SetPreventsFlinch( true );
}

AVP_Move_Alien_90TurnToCrawlRight::AVP_Move_Alien_90TurnToCrawlRight( Asura_Movement_System& xParentSystem )
: PARENT( xParentSystem, GetStaticHash() )
{
	SetOrientationUsage( ORIENTATION_SET_DIRECTLY );
	SetPositionUsage( POSITION_SET_FROM_VELOCITY );
	SetAimDirectionUsage( AVP_ClientEntity_Actor::AIM_DIRECTION_RESET );
	SetLookAtPositionUsage( AVP_ClientEntity_Actor::LOOK_AT_POSITION_RESET );

	Asura_Hash_ID uTPAnim = ASURA_HASH_ID_UNSET;
	Asura_Blueprint* pxBP = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_MOVESALIEN, g_uAIMovesBP );
	if ( pxBP )
	{
		Asura_Value xValue;
		if ( pxBP->GetParameterValueByHashID( g_u90TurnToCrawlRight, xValue ) )
		{
			uTPAnim = xValue.GetHashIDValue();
		}
	}
	SetAnimHashes( ASURA_HASH_ID_UNSET, uTPAnim );
	if ( m_pxAVPAnimationData )
	{
		// We're not tweening out as this goes straight into crawl.
		m_pxAVPAnimationData->SetTweenData( 0.05f, 0.0f, true );
	}

	SetPreventsFlinch( true );
}

//////////////////////////////////////////////////////////////////////////////
//
// Falling stuff.
//
//////////////////////////////////////////////////////////////////////////////

AVP_Move_Alien_FallingBegin::AVP_Move_Alien_FallingBegin( Asura_Movement_System& xParentSystem )
: PARENT( xParentSystem, GetStaticHash() )
{
	Asura_Hash_ID uTPAnim = 0x68bc54cf; // Asura_GetHashID( "Alien_Warrior_ceiling_to_floor_drop" )
	Asura_Blueprint* pxBP = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_ACTOR_STATE, g_uAIJumpBP );
	if ( pxBP )
	{
		Asura_Value xValue;
		if ( pxBP->GetParameterValueByHashID( g_uBeginFalling, xValue ) )
		{
			uTPAnim = xValue.GetHashIDValue();
		}
	}
	SetAnimHashes( ASURA_HASH_ID_UNSET, uTPAnim );

	if (m_pxAVPAnimationData)
	{
		m_pxAVPAnimationData->SetTweenData( 0.05f, 0.0f, true );
		m_pxAVPAnimationData->SetLeaveAnimAtEnd( true );
	}

	// Currently the falling rest anim doesn't seem to work.
	AddNextMove( AVP_Move_Alien_Falling::GetStaticHash() );
	SetDisableMovement( true );
	SetDisableSurfaceReorientation( true );
	SetPreventsFlinch( true );
	SetAimDirectionUsage( AVP_ClientEntity_Actor::AIM_DIRECTION_RESET );
	//SetLookAtPositionUsage( AVP_ClientEntity_Actor::LOOK_AT_POSITION_RESET );
}

bool AVP_Move_Alien_FallingBegin::Start( const Asura_Msg_Move* pxStartMsg )
{
	if ( !pxStartMsg )
	{
		Asura_Assert(false, "AVP_Move_Alien_FallingBegin::Start - can only be triggered by message.");
		return false;
	}

	Asura_Vector_3 xTargetNormal;
	const AVP_Message_Alien_Move_JumpSurfaceStart* pxMessage = static_cast< const AVP_Message_Alien_Move_JumpSurfaceStart* >( pxStartMsg );
	pxMessage->GetDestinationNormal(xTargetNormal);

	const AVP_ClientActor_Vertebrate& xOwner = static_cast<const AVP_ClientActor_Vertebrate&>(GetAVPParentSystem().GetActorOwner());
	Asura_Matrix_3x3 xTargetOrientation;
	CalculateTargetOrientation(xTargetNormal, xTargetOrientation);

	AVP_ClientActor_Alien& xAlienOwner = static_cast<AVP_ClientActor_Alien&>(GetAVPParentSystem().GetActorOwner());
	xAlienOwner.SetStanding( false, false );

	// We need to ensure the correct anim hash is set up, as it's about to get used.
	if (m_pxAVPAnimationData)
	{
		if (xAlienOwner.IsUsingFirstPersonSkin())
		{
			m_pxAVPAnimationData->SetAnimHash(m_uFirstPersonAnimHash);
		}
		else
		{
			m_pxAVPAnimationData->SetAnimHash(m_uThirdPersonAnimHash);
		}
	}

	const float fDuration = m_pxAVPAnimationData->GetAnimationDuration();
	m_xOrientationInterpolationData.SetMovementOffsetOperation(Asura_MoveData_OrientationInterpolation::STRETCH_MOVEMENT_OFFSET);
	m_xOrientationInterpolationData.SetDuration(fDuration);
	m_xOrientationInterpolationData.SetInitial(xOwner.GetCurrentOrientation());
	m_xOrientationInterpolationData.SetFinal(xTargetOrientation);
	m_xOrientationInterpolationData.SetSingularityHintAxis(Asura_AVector3(0.0f,0.0f,-1.0f));
	InitialiseOrientationInterpolation(m_xOrientationInterpolationData);

	return PARENT::Start( pxStartMsg );
}

void AVP_Move_Alien_FallingBegin::CalculateTargetOrientation(const Asura_Vector_3& xTargetNormal, Asura_Matrix_3x3& xTargetOrientation) const
{
	const AVP_ClientActor_Vertebrate& xOwner = static_cast<const AVP_ClientActor_Vertebrate&>(GetAVPParentSystem().GetActorOwner());
	Asura_AMatrix3x3 xAlignedOri;

	AVP_Alien_JumpHandler::CalculateTargetOrientation(AVP_Alien_JumpHandler::JT_JUMP_180_WALL_TO_WALL, xOwner, xTargetNormal, xAlignedOri);

	xAlignedOri.ToMat3(xTargetOrientation);
}

AVP_Move_Alien_Falling::AVP_Move_Alien_Falling( Asura_Movement_System& xParentSystem )
: PARENT( xParentSystem, GetStaticHash() )
, m_fMoveTimer( 0.0f )
, m_bInAir( false )
, m_bFinished( false )
{
	AddNextMove( AVP_Move_Alien_FallingEnd::GetStaticHash() );

	Asura_Hash_ID uTPAnim = 0x6a82a601; // Asura_GetHashID( "Alien_Warrior_ceiling_to_floor_drop_pose" )
	Asura_Blueprint* pxBP = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_ACTOR_STATE, g_uAIJumpBP );
	if ( pxBP )
	{
		Asura_Value xValue;
		if ( pxBP->GetParameterValueByHashID( g_uFalling, xValue ) )
		{
			uTPAnim = xValue.GetHashIDValue();
		}
	}
	SetAnimHashes( ASURA_HASH_ID_UNSET, uTPAnim );
	if (m_pxAVPAnimationData)
	{
		m_pxAVPAnimationData->SetTweenData( 0.05f, 0.0f, true );
		m_pxAVPAnimationData->SetLeaveAnimAtEnd( true );
	}

	SetDisableSurfaceReorientation( true );
	SetLoopAtEnd( true );
	SetPreventsFlinch( true );
}

void AVP_Move_Alien_Falling::UpdateStates(const float fTimeStep)
{
	m_fMoveTimer += fTimeStep;

	bool bInContactWithFloor = false;
	AVP_ServerEntity_Actor* pxActorOwner = AVP_ServerEntity_Actor::FindEntity( GetAVPParentSystem().GetOwnerGuid() );
	if ( pxActorOwner )
	{
//		bInContactWithFloor = pxActorOwner->GetPhysicsObjectHandler()->GetPhysicsObject()->IsInContactWithFloor();
		bInContactWithFloor = pxActorOwner->GetPhysicsObjectHandler()->GetPhysicsObject()->IsSupported();
	}

	if ( m_bInAir )
	{
		m_bFinished = bInContactWithFloor;
	}
	else if ( !bInContactWithFloor )
	{
		m_bInAir = true;
	}
	else if ( m_fMoveTimer > 1.0f )
	{
		m_bFinished = true;
	}

	PARENT::UpdateStates(fTimeStep);
}

bool AVP_Move_Alien_Falling::HasFinished() const
{
	return m_bFinished;
}

AVP_Move_Alien_FallingEnd::AVP_Move_Alien_FallingEnd( Asura_Movement_System& xParentSystem )
: PARENT( xParentSystem, GetStaticHash() )
{
	Asura_Hash_ID uTPAnim = 0x68bfb7cb; // Asura_GetHashID( "Alien_Warrior_ceiling_to_floor_land" )
	Asura_Blueprint* pxBP = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_ACTOR_STATE, g_uAIJumpBP );
	if ( pxBP )
	{
		Asura_Value xValue;
		if ( pxBP->GetParameterValueByHashID( g_uEndFalling, xValue ) )
		{
			uTPAnim = xValue.GetHashIDValue();
		}
	}
	SetAnimHashes( ASURA_HASH_ID_UNSET, uTPAnim );
	if (m_pxAVPAnimationData)
	{
		m_pxAVPAnimationData->SetTweenData( 0.05f, 0.1f, true );
		m_pxAVPAnimationData->SetLeaveAnimAtEnd( true );
	}

	SetPreventsFlinch( true );
	SetAimDirectionUsage( AVP_ClientEntity_Actor::AIM_DIRECTION_RESET );
	SetLookAtPositionUsage( AVP_ClientEntity_Actor::LOOK_AT_POSITION_RESET );
}


AVP_Move_Alien_DuctSurfaceSwitch::AVP_Move_Alien_DuctSurfaceSwitch(Asura_Movement_System& xParentSystem)
: PARENT( xParentSystem, GetStaticHash() )
{
	Asura_Hash_ID uFPAnim = ASURA_HASH_ID_UNSET;
	Asura_Hash_ID uTPAnim = ASURA_HASH_ID_UNSET;
	const Asura_Blueprint* pxFPBP = Asura_Blueprint_System::GetBlueprintByHashID(uAVP_BLUEPRINT_TYPE_ACTOR_STATE, g_uPlayerJumpBP );
	const Asura_Blueprint* pxTPBP = Asura_Blueprint_System::GetBlueprintByHashID(uAVP_BLUEPRINT_TYPE_ACTOR_STATE, g_uAIJumpBP );
	if ( pxFPBP )
	{
		Asura_Value xValue;
		if (pxFPBP->GetParameterValueByHashID(g_uDuctSurfaceSwitch, xValue))
		{
			uFPAnim = xValue.GetHashIDValue();
		}
	}
	if ( pxTPBP )
	{
		Asura_Value xValue;
		if (pxTPBP->GetParameterValueByHashID(g_uDuctSurfaceSwitch, xValue))
		{
			uTPAnim = xValue.GetHashIDValue();
		}
	}
	SetAnimHashes( uFPAnim, uTPAnim );
	if (m_pxAVPAnimationData)
	{
		m_pxAVPAnimationData->SetTweenData( 0.1f, 0.1f, true );
		m_pxAVPAnimationData->SetLeaveAnimAtEnd( true );
	}

	SetPreventsFlinch(true);
	SetAimDirectionUsage(AVP_ClientEntity_Actor::AIM_DIRECTION_RESET);
	SetLookAtPositionUsage( AVP_ClientEntity_Actor::LOOK_AT_POSITION_RESET );
}

bool AVP_Move_Alien_DuctSurfaceSwitch::Start(const Asura_Msg_Move* pxStartMsg)
{
	m_xOrientationInterpolationData.SetSingularityHintAxis(Asura_AVector3(0.0f, 0.0f, -1.0f));

	return PARENT::Start(pxStartMsg);
}

void AVP_Move_Alien_DuctSurfaceSwitch::CalculateTargetOrientation(const Asura_Vector_3& xTargetNormal, Asura_Matrix_3x3& xTargetOrientation) const
{
	const AVP_ClientActor_Vertebrate& xOwner = static_cast<const AVP_ClientActor_Vertebrate&>(GetAVPParentSystem().GetActorOwner());
	Asura_AMatrix3x3 xAlignedOri;

	AVP_Alien_JumpHandler::CalculateTargetOrientation(AVP_Alien_JumpHandler::JT_JUMP_180_WALL_TO_WALL, xOwner, xTargetNormal, xAlignedOri);

	xAlignedOri.ToMat3(xTargetOrientation);
}


//////////////////////////////////////////////////////////////////////////////
//
// AVP_Move_Alien_AI_JumpSurfaceFlat
//
//////////////////////////////////////////////////////////////////////////////

AVP_Move_Alien_AI_JumpSurfaceFlat::AVP_Move_Alien_AI_JumpSurfaceFlat(Asura_Movement_System& xParentSystem)
: PARENT(xParentSystem, GetStaticHash())
, m_xJumpStartMessage()
{
	SetMovementDirectionUsage(MOVEMENTDIRECTION_SET_FROM_ORIENTATION_FORWARDS_VECTOR);
}

void AVP_Move_Alien_AI_JumpSurfaceFlat::SetJumpStartMessage( const AVP_Message_Alien_AI_Move_JumpSurfaceStart& xJumpStartMessage )
{
	m_xJumpStartMessage = xJumpStartMessage;
}

bool AVP_Move_Alien_AI_JumpSurfaceFlat::Start( const Asura_Msg_Move* pxStartMsg /*= NULL*/ )
{

	if ( pxStartMsg )
	{
		const AVP_Message_Alien_AI_Move_JumpSurfaceStart* pxMessage = static_cast< const AVP_Message_Alien_AI_Move_JumpSurfaceStart* >( pxStartMsg );

		SetJumpStartMessage( *pxMessage );

	}

	SetAnimHashes( ASURA_HASH_ID_UNSET, m_xJumpStartMessage.GetJumpAnimHash() );

	if (m_pxAVPAnimationData)
	{
		m_pxAVPAnimationData->SetTweenData( 0.3f, 0.1f, true );
	}

	SetAimDirectionUsage(AVP_ClientEntity_Actor::AIM_DIRECTION_RESET);
	SetLookAtPositionUsage(AVP_ClientEntity_Actor::LOOK_AT_POSITION_RESET);


	AVP_ClientActor_Alien& xAlienOwner = static_cast<AVP_ClientActor_Alien&>(GetAVPParentSystem().GetActorOwner());

	xAlienOwner.SetStanding( false, false );

	// We need to ensure the correct anim hash is set up, as it's about to get used.
	if (m_pxAVPAnimationData)
	{
		if (xAlienOwner.IsUsingFirstPersonSkin())
		{
			Asura_Assert(false, "AVP_Move_Alien_AI_JumpSurfaceFlat::Start - first person not expected.");
			return false;
		}
		else
		{
			m_pxAVPAnimationData->SetAnimHash(m_uThirdPersonAnimHash);
		}

	}

	const bool bHasAnimation = m_pxAVPAnimationData && m_pxAVPAnimationData->HasValidAnimation();
	const float	fDuration = (bHasAnimation) ? m_pxAVPAnimationData->GetAnimationDuration() : 0.500f;

	Asura_Vector_3 xTargetPosition;
	Asura_Matrix_3x3 xTargetOrientation;
	m_xJumpStartMessage.GetDestinationOrientation(xTargetOrientation);
	m_xJumpStartMessage.GetDestinationPosition(xTargetPosition);

	// For the time being we'll just move and rotate to our desired position...
	if(bHasAnimation)
	{
		m_xOrientationInterpolationData.SetMovementOffsetOperation(Asura_MoveData_OrientationInterpolation::STRETCH_MOVEMENT_OFFSET);
		m_xPositionInterpolationData.SetMovementOffsetOperation(Asura_MoveData_PositionInterpolation::STRETCH_MOVEMENT_OFFSET);
	}

	m_xOrientationInterpolationData.SetDuration(fDuration);
	m_xOrientationInterpolationData.SetFinal(xTargetOrientation);
	m_xPositionInterpolationData.SetDuration(fDuration);
	m_xPositionInterpolationData.SetFinal(xTargetPosition);

	InitialiseOrientationInterpolation(m_xOrientationInterpolationData);
	InitialisePositionInterpolation(m_xPositionInterpolationData);
	SetPreventsFlinch( true );

	// Finally, call the base-function...
	return(PARENT::Start(pxStartMsg));
}

bool AVP_Move_Alien_AI_JumpSurfaceFlat::ShouldStart() const
{
	if (m_xJumpStartMessage.GetJumpHash() != GetHash())
	{
		return false;
	}

	return PARENT::ShouldStart();
}


void AVP_Move_Alien_AI_JumpSurfaceFlat::Finish()
{
	ResetPhysicsObjectHandler();

	PARENT::Finish();
}
void AVP_Move_Alien_AI_JumpSurfaceFlat::Abort()
{
	ResetPhysicsObjectHandler();

	PARENT::Abort();
}

//////////////////////////////////////////////////////////////////////////////
//
// AVP_Move_Alien_AI_JumpSurfaceConcave90Begin
//
//////////////////////////////////////////////////////////////////////////////

AVP_Move_Alien_AI_JumpSurfaceConcave90Begin::AVP_Move_Alien_AI_JumpSurfaceConcave90Begin( Asura_Movement_System& xParentSystem )
: PARENT( xParentSystem, GetStaticHash() )
{
	Asura_Hash_ID uFPAnim = ASURA_HASH_ID_UNSET;	 // non player anim
	Asura_Hash_ID uTPAnim = 0xad3d0cb1;				// Alien_Warrior_med_into_floor_to_wall_med
	Asura_Value xValue;

	Asura_Blueprint* pxBP = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_ACTOR_STATE, g_uAIJumpBP );
	if ( pxBP )
	{
		if ( pxBP->GetParameterValueByHashID( g_uConcave90FloorToWallBeginAnimParam, xValue ) )
		{
			uTPAnim = xValue.GetHashIDValue();
		}
	}

	SetAnimHashes( uFPAnim, uTPAnim );

	// Only tween in to our anim.
	if (m_pxAVPAnimationData)
	{
		m_pxAVPAnimationData->SetTweenData( 0.1f, 0.0f, true );
	}

	AddNextMove( AVP_Move_Alien_AI_JumpSurfaceConcave90Middle::GetStaticHash() );
	//## fixme - Sort this out once the move interface has been cleaned up.
	//## Uncommenting this leads to the physics object glitching into the environment after our AVP_Move_Alien_JumpSurfaceConcave90End move.
	//SetVelocityBased( true );

	SetPreventsFlinch( true );
	SetAimDirectionUsage(AVP_ClientEntity_Actor::AIM_DIRECTION_RESET);
	SetLookAtPositionUsage(AVP_ClientEntity_Actor::LOOK_AT_POSITION_RESET);

}

bool AVP_Move_Alien_AI_JumpSurfaceConcave90Begin::Start( const Asura_Msg_Move* pxStartMsg )
{
	if ( pxStartMsg )
	{
		const AVP_Message_Alien_AI_Move_JumpSurfaceStart* pxMessage = static_cast< const AVP_Message_Alien_AI_Move_JumpSurfaceStart* >( pxStartMsg );

		SetJumpStartMessage( *pxMessage );

	}

	return PARENT::Start( &m_xJumpStartMessage );
}

bool AVP_Move_Alien_AI_JumpSurfaceConcave90Begin::ShouldStart() const
{
	if (m_xJumpStartMessage.GetJumpHash() != AVP_Move_Alien_AI_JumpSurfaceConcave90Begin::GetStaticHash())
	{
		return false;
	}

	return PARENT::ShouldStart();
}

void AVP_Move_Alien_AI_JumpSurfaceConcave90Begin::Abort()
{
	ResetPhysicsObjectHandler();

	PARENT::Abort();
}

void AVP_Move_Alien_AI_JumpSurfaceConcave90Begin::SetJumpStartMessage( const AVP_Message_Alien_AI_Move_JumpSurfaceStart& xJumpStartMessage )
{

	m_xJumpStartMessage = xJumpStartMessage;

	// Pass the details on down the chain.
	AVP_Move_Alien_AI_JumpSurfaceConcave90Middle* pxNextMove =
		static_cast< AVP_Move_Alien_AI_JumpSurfaceConcave90Middle* >( GetParentSystem().GetMove( AVP_Move_Alien_AI_JumpSurfaceConcave90Middle::GetStaticHash() ) );
	if ( pxNextMove )
	{
		pxNextMove->SetJumpStartMessage( xJumpStartMessage );
	}
}

//////////////////////////////////////////////////////////////////////////////
//
// AVP_Move_Alien_AI_JumpSurfaceConcave90Middle
//
//////////////////////////////////////////////////////////////////////////////

AVP_Move_Alien_AI_JumpSurfaceConcave90Middle::AVP_Move_Alien_AI_JumpSurfaceConcave90Middle(Asura_Movement_System& xParentSystem)
: PARENT(xParentSystem, GetStaticHash())
, m_xJumpStartMessage()
{
	Asura_Hash_ID uFPAnim = ASURA_HASH_ID_UNSET;	// non player anim
	Asura_Hash_ID uTPAnim = 0x3c58d233;	// Alien_Warrior_floor_to_wall_med
	Asura_Value xValue;

	Asura_Blueprint* pxBP = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_ACTOR_STATE, g_uAIJumpBP );
	if ( pxBP )
	{
		if ( pxBP->GetParameterValueByHashID( g_uConcave90FloorToWallMiddleAnimParam, xValue ) )
		{
			uTPAnim = xValue.GetHashIDValue();
		}
	}

	SetAnimHashes( uFPAnim, uTPAnim );

	// Don't tween in or out of our anim.
	if (m_pxAVPAnimationData)
	{
		m_pxAVPAnimationData->SetTweenData( 0.0f, 0.0f, false );
	}

	AddNextMove( AVP_Move_Alien_AI_JumpSurfaceConcave90End::GetStaticHash() );
	SetAimDirectionUsage(AVP_ClientEntity_Actor::AIM_DIRECTION_RESET);
	SetLookAtPositionUsage(AVP_ClientEntity_Actor::LOOK_AT_POSITION_RESET);

}

void AVP_Move_Alien_AI_JumpSurfaceConcave90Middle::SetJumpStartMessage( const AVP_Message_Alien_AI_Move_JumpSurfaceStart& xJumpStartMessage )
{
	m_xJumpStartMessage = xJumpStartMessage;

	// Also, pass this message on to our next move.
	AVP_Move_Alien_AI_JumpSurfaceConcave90End* pxNextMove =
		static_cast< AVP_Move_Alien_AI_JumpSurfaceConcave90End* >( GetParentSystem().GetMove( AVP_Move_Alien_AI_JumpSurfaceConcave90End::GetStaticHash() ) );
	if ( pxNextMove )
	{
		pxNextMove->SetJumpStartMessage( xJumpStartMessage );
	}
}

bool AVP_Move_Alien_AI_JumpSurfaceConcave90Middle::Start( const Asura_Msg_Move* pxStartMsg /*= NULL*/ )
{
	if ( pxStartMsg ) // start message is not expected
	{
		Asura_Assert(false, "AVP_Move_Alien_AI_JumpSurfaceConcave90Middle::Start - Alien surface-interaction move in the middle can not be triggered by message.");
		return false;
	}

	AVP_ClientActor_Alien& xAlienOwner = static_cast<AVP_ClientActor_Alien&>(GetAVPParentSystem().GetActorOwner());

	xAlienOwner.SetStanding( false, false );

	// We need to ensure the correct anim hash is set up, as it's about to get used.
	if (m_pxAVPAnimationData)
	{
		if (xAlienOwner.IsUsingFirstPersonSkin())
		{
			Asura_Assert(false, "AVP_Move_Alien_AI_JumpSurfaceConcave90Middle::Start - first person not expected.");
			return false;
		}
		else
		{
			m_pxAVPAnimationData->SetAnimHash(m_uThirdPersonAnimHash);
		}

	}

	const bool bHasAnimation = m_pxAVPAnimationData && m_pxAVPAnimationData->HasValidAnimation();
	const float	fDuration = (bHasAnimation) ? m_pxAVPAnimationData->GetAnimationDuration() : 0.500f;

	Asura_Vector_3 xTargetPosition;
	Asura_Matrix_3x3 xTargetOrientation;
	m_xJumpStartMessage.GetDestinationOrientation(xTargetOrientation);
	m_xJumpStartMessage.GetDestinationPosition(xTargetPosition);

	// For the time being we'll just move and rotate to our desired position...
	if(bHasAnimation)
	{
		m_xOrientationInterpolationData.SetMovementOffsetOperation(Asura_MoveData_OrientationInterpolation::STRETCH_MOVEMENT_OFFSET);
		m_xPositionInterpolationData.SetMovementOffsetOperation(Asura_MoveData_PositionInterpolation::STRETCH_MOVEMENT_OFFSET);
	}

	m_xOrientationInterpolationData.SetDuration(fDuration);
	m_xOrientationInterpolationData.SetFinal(xTargetOrientation);
	m_xPositionInterpolationData.SetDuration(fDuration);
	m_xPositionInterpolationData.SetFinal(xTargetPosition);

	InitialiseOrientationInterpolation(m_xOrientationInterpolationData);
	InitialisePositionInterpolation(m_xPositionInterpolationData);

	// Finally, call the base-function...
	return(PARENT::Start(pxStartMsg));
}

//////////////////////////////////////////////////////////////////////////////
//
// AVP_Move_Alien_AI_JumpSurfaceConcave90End
//
//////////////////////////////////////////////////////////////////////////////

AVP_Move_Alien_AI_JumpSurfaceConcave90End::AVP_Move_Alien_AI_JumpSurfaceConcave90End( Asura_Movement_System& xParentSystem )
: PARENT( xParentSystem, GetStaticHash() )
{
	Asura_Hash_ID uFPAnim = ASURA_HASH_ID_UNSET;	// non player anim
	Asura_Hash_ID uTPAnim = 0x37fe33f9;	// Alien_Warrior_floor_to_wall_med_into_med
	Asura_Value xValue;

	Asura_Blueprint* pxBP = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_ACTOR_STATE, g_uAIJumpBP );
	if ( pxBP )
	{
		if ( pxBP->GetParameterValueByHashID( g_uConcave90FloorToWallEndAnimParam, xValue ) )
		{
			uTPAnim = xValue.GetHashIDValue();
		}
	}

	SetAnimHashes( uFPAnim, uTPAnim );

	// Don't tween in our anim.
	if (m_pxAVPAnimationData)
	{
		m_pxAVPAnimationData->SetTweenData( 0.0f, 0.2f, false );
	}

	SetPositionUsage(POSITION_SET_FROM_VELOCITY);

	SetPreventsFlinch( true );
	SetAimDirectionUsage(AVP_ClientEntity_Actor::AIM_DIRECTION_RESET);
	SetLookAtPositionUsage(AVP_ClientEntity_Actor::LOOK_AT_POSITION_RESET);
	SetMovementDirectionUsage(MOVEMENTDIRECTION_SET_FROM_ORIENTATION_FORWARDS_VECTOR);

}

void AVP_Move_Alien_AI_JumpSurfaceConcave90End::SetJumpStartMessage( const AVP_Message_Alien_AI_Move_JumpSurfaceStart& /*xJumpStartMessage*/ )
{
}

bool AVP_Move_Alien_AI_JumpSurfaceConcave90End::Start( const Asura_Msg_Move* pxStartMsg /*= NULL*/ )
{
	return PARENT::Start( pxStartMsg );
}

void AVP_Move_Alien_AI_JumpSurfaceConcave90End::Finish()
{
	ResetPhysicsObjectHandler();

	PARENT::Finish();
}

void AVP_Move_Alien_AI_JumpSurfaceConcave90End::Abort()
{
	ResetPhysicsObjectHandler();

	PARENT::Abort();
}


//////////////////////////////////////////////////////////////////////////////
//
// AVP_Move_Alien_AI_TurnAndJumpSurfaceBase
// Base class for turn and jump
//////////////////////////////////////////////////////////////////////////////

AVP_Move_Alien_AI_TurnAndJumpSurfaceBase::AVP_Move_Alien_AI_TurnAndJumpSurfaceBase( Asura_Movement_System& xParentSystem, const Asura_Hash_ID uHash )
: PARENT( xParentSystem, uHash )
{
	AddNextMove( AVP_Move_Alien_AI_JumpSurfaceConcave90Begin::GetStaticHash() );
	AddNextMove( AVP_Move_Alien_AI_JumpSurfaceFlat::GetStaticHash() );

	SetPositionUsage(POSITION_SET_FROM_VELOCITY);
	SetPreventsFlinch( true );
	SetAimDirectionUsage(AVP_ClientEntity_Actor::AIM_DIRECTION_RESET);
	SetLookAtPositionUsage(AVP_ClientEntity_Actor::LOOK_AT_POSITION_RESET);
	SetPreventsFlinch(true);
}

bool AVP_Move_Alien_AI_TurnAndJumpSurfaceBase::Start( const Asura_Msg_Move* pxStartMsg )
{

	if ( !pxStartMsg ) // start message is not expected
	{
		Asura_Assert(false, "AVP_Move_Alien_AI_TurnAndJumpSurfaceBase::Start - must start with a message.");
		return false;
	}

	const AVP_Message_Alien_AI_Move_TurnAndJumpSurfaceStart* pxMessage = static_cast< const AVP_Message_Alien_AI_Move_TurnAndJumpSurfaceStart* >( pxStartMsg );

	// Also, pass this message on to our next move.
	{
		AVP_Move_Alien_AI_JumpSurfaceConcave90Begin* pxJumpNextMove =
			static_cast< AVP_Move_Alien_AI_JumpSurfaceConcave90Begin* >( GetParentSystem().GetMove( AVP_Move_Alien_AI_JumpSurfaceConcave90Begin::GetStaticHash() ) );
		if ( pxJumpNextMove )
		{
			pxJumpNextMove->SetJumpStartMessage( *pxMessage );
		}
	}
	{
		AVP_Move_Alien_AI_JumpSurfaceFlat* pxJumpNextMove =
			static_cast< AVP_Move_Alien_AI_JumpSurfaceFlat* >( GetParentSystem().GetMove( AVP_Move_Alien_AI_JumpSurfaceFlat::GetStaticHash() ) );
		if ( pxJumpNextMove )
		{
			pxJumpNextMove->SetJumpStartMessage( *pxMessage );
		}
	}
	// We need to ensure the correct anim hash is set up, as it's about to get used.
	AVP_ClientActor_Alien& xAlienOwner = static_cast<AVP_ClientActor_Alien&>(GetAVPParentSystem().GetActorOwner());

	xAlienOwner.SetStanding( false, false );

	if (m_pxAVPAnimationData)
	{
		if (xAlienOwner.IsUsingFirstPersonSkin())
		{
			Asura_Assert(false, "AVP_Move_Alien_AI_TurnAndJumpSurfaceBase::Start - first person not expected.");
			return false;
		}
		else
		{
			m_pxAVPAnimationData->SetAnimHash(m_uThirdPersonAnimHash);
		}

	}

	const bool bHasAnimation = m_pxAVPAnimationData && m_pxAVPAnimationData->HasValidAnimation();
	const float	fDuration = (bHasAnimation) ? m_pxAVPAnimationData->GetAnimationDuration() : 0.500f;

	Asura_Matrix_3x3 xTurntOrientation;
	pxMessage->GetTurnOrientation(xTurntOrientation);
	Asura_Vector_3 xStartJumpPosition;
	pxMessage->GetStartJumpPosition(xStartJumpPosition);

	if(bHasAnimation)
	{
		m_xOrientationInterpolationData.SetMovementOffsetOperation(Asura_MoveData_OrientationInterpolation::STRETCH_MOVEMENT_OFFSET);
	}

	if (m_pxAVPAnimationData)
	{
		m_pxAVPAnimationData->SetTweenData( 0.2f, 0.0f, true );
		m_pxAVPAnimationData->SetLeaveAnimAtEnd(true);
	}

	m_xOrientationInterpolationData.SetDuration(fDuration);
	m_xOrientationInterpolationData.SetFinal(xTurntOrientation);
	m_xPositionInterpolationData.SetDuration(fDuration);
	m_xPositionInterpolationData.SetFinal(Asura_AVector3(xStartJumpPosition));

	InitialiseOrientationInterpolation(m_xOrientationInterpolationData);
	InitialisePositionInterpolation(m_xPositionInterpolationData);


	// Finally, call the base-function...
	return PARENT::Start(pxStartMsg);
}

void AVP_Move_Alien_AI_TurnAndJumpSurfaceBase::Abort()
{
	ResetPhysicsObjectHandler();

	PARENT::Abort();
}


//////////////////////////////////////////////////////////////////////////////
//
// AVP_Move_Alien_AI_MoveForwardAndJumpSurface
// Turn Before Jump
//
//////////////////////////////////////////////////////////////////////////////

AVP_Move_Alien_AI_MoveForwardAndJumpSurface::AVP_Move_Alien_AI_MoveForwardAndJumpSurface( Asura_Movement_System& xParentSystem)
: PARENT( xParentSystem, GetStaticHash() )
{
	Asura_Hash_ID uFPAnim = ASURA_HASH_ID_UNSET;	// non player anim
	Asura_Hash_ID uTPAnim = ASURA_HASH_ID_UNSET;
	Asura_Value xValue;

	Asura_Blueprint* pxBP = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_MOVESALIEN, g_uAIMovesBP );
	if ( pxBP )
	{
		if ( pxBP->GetParameterValueByHashID( g_uForwardBeforeJump, xValue ) )
		{
			uTPAnim = xValue.GetHashIDValue();
		}
	}

	SetAnimHashes( uFPAnim, uTPAnim );

	// Don't tween in our anim.
	if (m_pxAVPAnimationData)
	{
		m_pxAVPAnimationData->SetTweenData( 0.05f, 0.0f, false );
	}
}


//////////////////////////////////////////////////////////////////////////////
//
// AVP_Move_Alien_AI_Turn90LeftAndJumpSurface
// Turn Before Jump
//
//////////////////////////////////////////////////////////////////////////////

AVP_Move_Alien_AI_Turn90LeftAndJumpSurface::AVP_Move_Alien_AI_Turn90LeftAndJumpSurface( Asura_Movement_System& xParentSystem)
: PARENT( xParentSystem, GetStaticHash() )
{
	Asura_Hash_ID uFPAnim = ASURA_HASH_ID_UNSET;	// non player anim
	Asura_Hash_ID uTPAnim = ASURA_HASH_ID_UNSET;
	Asura_Value xValue;

	Asura_Blueprint* pxBP = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_MOVESALIEN, g_uAIMovesBP );
	if ( pxBP )
	{
		if ( pxBP->GetParameterValueByHashID( g_u90TurnToCrawlLeft, xValue ) )
		{
			uTPAnim = xValue.GetHashIDValue();
		}
	}

	SetAnimHashes( uFPAnim, uTPAnim );

	// Don't tween in our anim.
	if (m_pxAVPAnimationData)
	{
		m_pxAVPAnimationData->SetTweenData( 0.05f, 0.0f, false );
	}
}

//////////////////////////////////////////////////////////////////////////////
//
// AVP_Move_Alien_AI_Turn90RightAndJumpSurface
// Turn Before Jump
//
//////////////////////////////////////////////////////////////////////////////

AVP_Move_Alien_AI_Turn90RightAndJumpSurface::AVP_Move_Alien_AI_Turn90RightAndJumpSurface( Asura_Movement_System& xParentSystem)
: PARENT( xParentSystem, GetStaticHash() )
{
	Asura_Hash_ID uFPAnim = ASURA_HASH_ID_UNSET;	// non player anim
	Asura_Hash_ID uTPAnim = ASURA_HASH_ID_UNSET;	
	Asura_Value xValue;

	Asura_Blueprint* pxBP = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_MOVESALIEN, g_uAIMovesBP );
	if ( pxBP )
	{
		if ( pxBP->GetParameterValueByHashID( g_u90TurnToCrawlRight, xValue ) )
		{
			uTPAnim = xValue.GetHashIDValue();
		}
	}

	SetAnimHashes( uFPAnim, uTPAnim );

	// Don't tween in our anim.
	if (m_pxAVPAnimationData)
	{
		m_pxAVPAnimationData->SetTweenData( 0.05f, 0.0f, false );
	}
}

//////////////////////////////////////////////////////////////////////////////
//
// AVP_Move_Alien_AI_Turn90OnSpotLeftAndJumpSurface
// Turn Before Jump
//
//////////////////////////////////////////////////////////////////////////////

AVP_Move_Alien_AI_Turn90OnSpotLeftAndJumpSurface::AVP_Move_Alien_AI_Turn90OnSpotLeftAndJumpSurface( Asura_Movement_System& xParentSystem)
: PARENT( xParentSystem, GetStaticHash() )
{
	Asura_Hash_ID uFPAnim = ASURA_HASH_ID_UNSET;	// non player anim
	Asura_Hash_ID uTPAnim = ASURA_HASH_ID_UNSET;
	Asura_Value xValue;

	Asura_Blueprint* pxBP = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_MOVESALIEN, g_uAIMovesBP );
	if ( pxBP )
	{
		if ( pxBP->GetParameterValueByHashID( g_u90TurnOnSpotLeft, xValue ) )
		{
			uTPAnim = xValue.GetHashIDValue();
		}
	}

	SetAnimHashes( uFPAnim, uTPAnim );

	// Don't tween in our anim.
	if (m_pxAVPAnimationData)
	{
		m_pxAVPAnimationData->SetTweenData( 0.05f, 0.0f, false );
	}
}

//////////////////////////////////////////////////////////////////////////////
//
// AVP_Move_Alien_AI_Turn90OnSpotRightAndJumpSurface
// Turn Before Jump
//
//////////////////////////////////////////////////////////////////////////////

AVP_Move_Alien_AI_Turn90OnSpotRightAndJumpSurface::AVP_Move_Alien_AI_Turn90OnSpotRightAndJumpSurface( Asura_Movement_System& xParentSystem)
: PARENT( xParentSystem, GetStaticHash() )
{
	Asura_Hash_ID uFPAnim = ASURA_HASH_ID_UNSET;	// non player anim
	Asura_Hash_ID uTPAnim = ASURA_HASH_ID_UNSET;	
	Asura_Value xValue;

	Asura_Blueprint* pxBP = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_MOVESALIEN, g_uAIMovesBP );
	if ( pxBP )
	{
		if ( pxBP->GetParameterValueByHashID( g_u90TurnOnSpotRight, xValue ) )
		{
			uTPAnim = xValue.GetHashIDValue();
		}
	}

	SetAnimHashes( uFPAnim, uTPAnim );

	// Don't tween in our anim.
	if (m_pxAVPAnimationData)
	{
		m_pxAVPAnimationData->SetTweenData( 0.05f, 0.0f, false );
	}
}


//////////////////////////////////////////////////////////////////////////////
//
// AVP_Move_Alien_AI_Turn180LeftAndJumpSurface
// Turn Before Jump
//
//////////////////////////////////////////////////////////////////////////////

AVP_Move_Alien_AI_Turn180LeftAndJumpSurface::AVP_Move_Alien_AI_Turn180LeftAndJumpSurface( Asura_Movement_System& xParentSystem)
: PARENT( xParentSystem, GetStaticHash() )
{
	Asura_Hash_ID uFPAnim = ASURA_HASH_ID_UNSET;	// non player anim
	Asura_Hash_ID uTPAnim = ASURA_HASH_ID_UNSET;
	Asura_Value xValue;

	Asura_Blueprint* pxBP = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_MOVESALIEN, g_uAIMovesBP );
	if ( pxBP )
	{
		if ( pxBP->GetParameterValueByHashID( g_u180TurnToCrawlLeft, xValue ) )
		{
			uTPAnim = xValue.GetHashIDValue();
		}
	}

	SetAnimHashes( uFPAnim, uTPAnim );

	// Don't tween in our anim.
	if (m_pxAVPAnimationData)
	{
		m_pxAVPAnimationData->SetTweenData( 0.05f, 0.0f, false );
	}
}

//////////////////////////////////////////////////////////////////////////////
//
// AVP_Move_Alien_AI_Turn180RightAndJumpSurface
// Turn Before Jump
//
//////////////////////////////////////////////////////////////////////////////

AVP_Move_Alien_AI_Turn180RightAndJumpSurface::AVP_Move_Alien_AI_Turn180RightAndJumpSurface( Asura_Movement_System& xParentSystem)
: PARENT( xParentSystem, GetStaticHash() )
{
	Asura_Hash_ID uFPAnim = ASURA_HASH_ID_UNSET;	// non player anim
	Asura_Hash_ID uTPAnim = ASURA_HASH_ID_UNSET;
	Asura_Value xValue;

	Asura_Blueprint* pxBP = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_MOVESALIEN, g_uAIMovesBP );
	if ( pxBP )
	{
		if ( pxBP->GetParameterValueByHashID( g_u180TurnToCrawlRight, xValue ) )
		{
			uTPAnim = xValue.GetHashIDValue();
		}
	}

	SetAnimHashes( uFPAnim, uTPAnim );

	// Don't tween in our anim.
	if (m_pxAVPAnimationData)
	{
		m_pxAVPAnimationData->SetTweenData( 0.05f, 0.0f, false );
	}

}

//////////////////////////////////////////////////////////////////////////////
//
// AVP_Move_Alien_AI_Turn180OnSpotLeftAndJumpSurface
// Turn Before Jump
//
//////////////////////////////////////////////////////////////////////////////

AVP_Move_Alien_AI_Turn180OnSpotLeftAndJumpSurface::AVP_Move_Alien_AI_Turn180OnSpotLeftAndJumpSurface( Asura_Movement_System& xParentSystem)
: PARENT( xParentSystem, GetStaticHash() )
{
	Asura_Hash_ID uFPAnim = ASURA_HASH_ID_UNSET;	// non player anim
	Asura_Hash_ID uTPAnim = ASURA_HASH_ID_UNSET;
	Asura_Value xValue;

	Asura_Blueprint* pxBP = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_MOVESALIEN, g_uAIMovesBP );
	if ( pxBP )
	{
		if ( pxBP->GetParameterValueByHashID( g_u180TurnOnSpotLeft, xValue ) )
		{
			uTPAnim = xValue.GetHashIDValue();
		}
	}

	SetAnimHashes( uFPAnim, uTPAnim );

	// Don't tween in our anim.
	if (m_pxAVPAnimationData)
	{
		m_pxAVPAnimationData->SetTweenData( 0.05f, 0.0f, false );
	}

	SetMovementDirectionUsage(MOVEMENTDIRECTION_SET_FROM_ORIENTATION_FORWARDS_VECTOR);
}

//////////////////////////////////////////////////////////////////////////////
//
// AVP_Move_Alien_AI_Turn180OnSpotRightAndJumpSurface
// Turn Before Jump
//
//////////////////////////////////////////////////////////////////////////////

AVP_Move_Alien_AI_Turn180OnSpotRightAndJumpSurface::AVP_Move_Alien_AI_Turn180OnSpotRightAndJumpSurface( Asura_Movement_System& xParentSystem)
: PARENT( xParentSystem, GetStaticHash() )
{
	Asura_Hash_ID uFPAnim = ASURA_HASH_ID_UNSET;	// non player anim
	Asura_Hash_ID uTPAnim = ASURA_HASH_ID_UNSET;
	Asura_Value xValue;

	Asura_Blueprint* pxBP = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_MOVESALIEN, g_uAIMovesBP );
	if ( pxBP )
	{
		if ( pxBP->GetParameterValueByHashID( g_u180TurnOnSpotRight, xValue ) )
		{
			uTPAnim = xValue.GetHashIDValue();
		}
	}

	SetAnimHashes( uFPAnim, uTPAnim );

	// Don't tween in our anim.
	if (m_pxAVPAnimationData)
	{
		m_pxAVPAnimationData->SetTweenData( 0.05f, 0.0f, false );
	}

	SetMovementDirectionUsage(MOVEMENTDIRECTION_SET_FROM_ORIENTATION_FORWARDS_VECTOR);

}


//////////////////////////////////////////////////////////////////////////////
//
// AVP_Move_Alien_AI_JumpSurfaceConcaveLeft
// Jump when start and end have same forward vector
//
//////////////////////////////////////////////////////////////////////////////

AVP_Move_Alien_AI_JumpSurfaceConcaveLeft::AVP_Move_Alien_AI_JumpSurfaceConcaveLeft( Asura_Movement_System& xParentSystem )
: PARENT( xParentSystem, GetStaticHash() )
{
	Asura_Hash_ID uFPAnim = ASURA_HASH_ID_UNSET;	 // non player anim
	Asura_Hash_ID uTPAnim = ASURA_HASH_ID_UNSET; 	 // 
	Asura_Value xValue;

	Asura_Blueprint* pxBP = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_MOVESALIEN, g_uAIMovesBP );
	if ( pxBP )
	{
		if ( pxBP->GetParameterValueByHashID( g_uConaveJumpToSurfaceLeft, xValue ) )
		{
			uTPAnim = xValue.GetHashIDValue();
		}
	}

	SetAnimHashes( uFPAnim, uTPAnim );

	// Only tween in to our anim.
	if (m_pxAVPAnimationData)
	{
		m_pxAVPAnimationData->SetTweenData( 0.05f, 0.0f, true );
	}

	//## fixme - Sort this out once the move interface has been cleaned up.
	//## Uncommenting this leads to the physics object glitching into the environment after our AVP_Move_Alien_JumpSurfaceConcave90End move.
	//SetVelocityBased( true );

	SetPreventsFlinch( true );
	SetAimDirectionUsage(AVP_ClientEntity_Actor::AIM_DIRECTION_RESET);
	SetLookAtPositionUsage(AVP_ClientEntity_Actor::LOOK_AT_POSITION_RESET);
	SetMovementDirectionUsage(MOVEMENTDIRECTION_SET_FROM_ORIENTATION_FORWARDS_VECTOR);

}

bool AVP_Move_Alien_AI_JumpSurfaceConcaveLeft::Start( const Asura_Msg_Move* pxStartMsg )
{

	if ( !pxStartMsg ) // start message is not expected
	{
		Asura_Assert(false, "AVP_Move_Alien_AI_JumpSurfaceConcaveLeft::Start - Alien AI Jump start only from Message.");
		return false;
	}

	const AVP_Message_Alien_AI_Move_JumpSurfaceStart* pxMessage = static_cast< const AVP_Message_Alien_AI_Move_JumpSurfaceStart* >( pxStartMsg );

	AVP_ClientActor_Alien& xAlienOwner = static_cast<AVP_ClientActor_Alien&>(GetAVPParentSystem().GetActorOwner());

	xAlienOwner.SetStanding( false, false );

	// We need to ensure the correct anim hash is set up, as it's about to get used.
	if (m_pxAVPAnimationData)
	{
		if (xAlienOwner.IsUsingFirstPersonSkin())
		{
			Asura_Assert(false, "AVP_Move_Alien_AI_JumpSurfaceConcave90Middle::Start - first person not expected.");
			return false;
		}
		else
		{
			m_pxAVPAnimationData->SetAnimHash(m_uThirdPersonAnimHash);
		}

	}

	const bool bHasAnimation = m_pxAVPAnimationData && m_pxAVPAnimationData->HasValidAnimation();
	const float	fDuration = (bHasAnimation) ? m_pxAVPAnimationData->GetAnimationDuration() : 0.500f;

	Asura_Vector_3 xTargetPosition;
	Asura_Matrix_3x3 xTargetOrientation;
	pxMessage->GetDestinationOrientation(xTargetOrientation);
	pxMessage->GetDestinationPosition(xTargetPosition);

	// For the time being we'll just move and rotate to our desired position...
	if(bHasAnimation)
	{
		m_xOrientationInterpolationData.SetMovementOffsetOperation(Asura_MoveData_OrientationInterpolation::STRETCH_MOVEMENT_OFFSET);
		m_xPositionInterpolationData.SetMovementOffsetOperation(Asura_MoveData_PositionInterpolation::STRETCH_MOVEMENT_OFFSET);
	}

	m_xOrientationInterpolationData.SetDuration(fDuration);
	m_xOrientationInterpolationData.SetFinal(xTargetOrientation);
	m_xPositionInterpolationData.SetDuration(fDuration);
	m_xPositionInterpolationData.SetFinal(xTargetPosition);

	InitialiseOrientationInterpolation(m_xOrientationInterpolationData);
	InitialisePositionInterpolation(m_xPositionInterpolationData);

	// Finally, call the base-function...
	return(PARENT::Start(pxStartMsg));

}

void AVP_Move_Alien_AI_JumpSurfaceConcaveLeft::Abort()
{
	ResetPhysicsObjectHandler();

	PARENT::Abort();
}

//////////////////////////////////////////////////////////////////////////////
//
// AVP_Move_Alien_AI_JumpSurfaceConcaveRight
// Jump when start and end have same forward vector
//
//////////////////////////////////////////////////////////////////////////////

AVP_Move_Alien_AI_JumpSurfaceConcaveRight::AVP_Move_Alien_AI_JumpSurfaceConcaveRight( Asura_Movement_System& xParentSystem )
: PARENT( xParentSystem, GetStaticHash() )
{
	Asura_Hash_ID uFPAnim = ASURA_HASH_ID_UNSET;	 // non player anim
	Asura_Hash_ID uTPAnim = ASURA_HASH_ID_UNSET; 	 // 
	Asura_Value xValue;

	Asura_Blueprint* pxBP = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_MOVESALIEN, g_uAIMovesBP );
	if ( pxBP )
	{
		if ( pxBP->GetParameterValueByHashID( g_uConaveJumpToSurfaceRight, xValue ) )
		{
			uTPAnim = xValue.GetHashIDValue();
		}
	}

	SetAnimHashes( uFPAnim, uTPAnim );

	// Only tween in to our anim.
	if (m_pxAVPAnimationData)
	{
		m_pxAVPAnimationData->SetTweenData( 0.05f, 0.0f, true );
	}

	//## fixme - Sort this out once the move interface has been cleaned up.
	//## Uncommenting this leads to the physics object glitching into the environment after our AVP_Move_Alien_JumpSurfaceConcave90End move.
	//SetVelocityBased( true );

	SetPreventsFlinch( true );
	SetAimDirectionUsage(AVP_ClientEntity_Actor::AIM_DIRECTION_RESET);
	SetLookAtPositionUsage(AVP_ClientEntity_Actor::LOOK_AT_POSITION_RESET);
	SetMovementDirectionUsage(MOVEMENTDIRECTION_SET_FROM_ORIENTATION_FORWARDS_VECTOR);

}

bool AVP_Move_Alien_AI_JumpSurfaceConcaveRight::Start( const Asura_Msg_Move* pxStartMsg )
{

	if ( !pxStartMsg ) // start message is not expected
	{
		Asura_Assert(false, "AVP_Move_Alien_AI_JumpSurfaceConcaveLeft::Start - Alien AI Jump start only from Message.");
		return false;
	}

	const AVP_Message_Alien_AI_Move_JumpSurfaceStart* pxMessage = static_cast< const AVP_Message_Alien_AI_Move_JumpSurfaceStart* >( pxStartMsg );

	AVP_ClientActor_Alien& xAlienOwner = static_cast<AVP_ClientActor_Alien&>(GetAVPParentSystem().GetActorOwner());

	xAlienOwner.SetStanding( false, false );

	// We need to ensure the correct anim hash is set up, as it's about to get used.
	if (m_pxAVPAnimationData)
	{
		if (xAlienOwner.IsUsingFirstPersonSkin())
		{
			Asura_Assert(false, "AVP_Move_Alien_AI_JumpSurfaceConcave90Middle::Start - first person not expected.");
			return false;
		}
		else
		{
			m_pxAVPAnimationData->SetAnimHash(m_uThirdPersonAnimHash);
		}

	}

	const bool bHasAnimation = m_pxAVPAnimationData && m_pxAVPAnimationData->HasValidAnimation();
	const float	fDuration = (bHasAnimation) ? m_pxAVPAnimationData->GetAnimationDuration() : 0.500f;

	Asura_Vector_3 xTargetPosition;
	Asura_Matrix_3x3 xTargetOrientation;
	pxMessage->GetDestinationOrientation(xTargetOrientation);
	pxMessage->GetDestinationPosition(xTargetPosition);

	// For the time being we'll just move and rotate to our desired position...
	if(bHasAnimation)
	{
		m_xOrientationInterpolationData.SetMovementOffsetOperation(Asura_MoveData_OrientationInterpolation::STRETCH_MOVEMENT_OFFSET);
		m_xPositionInterpolationData.SetMovementOffsetOperation(Asura_MoveData_PositionInterpolation::STRETCH_MOVEMENT_OFFSET);
	}

	m_xOrientationInterpolationData.SetDuration(fDuration);
	m_xOrientationInterpolationData.SetFinal(xTargetOrientation);
	m_xPositionInterpolationData.SetDuration(fDuration);
	m_xPositionInterpolationData.SetFinal(xTargetPosition);

	InitialiseOrientationInterpolation(m_xOrientationInterpolationData);
	InitialisePositionInterpolation(m_xPositionInterpolationData);

	// Finally, call the base-function...
	return(PARENT::Start(pxStartMsg));

}

void AVP_Move_Alien_AI_JumpSurfaceConcaveRight::Abort()
{
	ResetPhysicsObjectHandler();

	PARENT::Abort();
}

//////////////////////////////////////////////////////////////////////////////
//
// AVP_Move_Alien_AI_Turn180Left
// Turn Before Jump
//
//////////////////////////////////////////////////////////////////////////////

AVP_Move_Alien_AI_Turn180Left::AVP_Move_Alien_AI_Turn180Left( Asura_Movement_System& xParentSystem)
: PARENT( xParentSystem, GetStaticHash() )
{

	Asura_Hash_ID uFPAnim = ASURA_HASH_ID_UNSET;	// non player anim
	Asura_Hash_ID uTPAnim = ASURA_HASH_ID_UNSET;	
	Asura_Value xValue;

	Asura_Blueprint* pxBP = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_MOVESALIEN, g_uAIMovesBP );
	if ( pxBP )
	{
		if ( pxBP->GetParameterValueByHashID( g_u180TurnToCrawlLeft, xValue ) )
		{
			uTPAnim = xValue.GetHashIDValue();
		}
	}

	SetAnimHashes( uFPAnim, uTPAnim );

	// Don't tween in our anim.
	if (m_pxAVPAnimationData)
	{
		m_pxAVPAnimationData->SetTweenData( 0.05f, 0.0f, false );
	}

	SetPositionUsage(POSITION_SET_FROM_VELOCITY);
	SetAimDirectionUsage(AVP_ClientEntity_Actor::AIM_DIRECTION_RESET);
	SetLookAtPositionUsage(AVP_ClientEntity_Actor::LOOK_AT_POSITION_RESET);
	SetPreventsFlinch( true );
}

bool AVP_Move_Alien_AI_Turn180Left::Start( const Asura_Msg_Move* pxStartMsg )
{
	if ( !pxStartMsg ) // start message is not expected
	{
		Asura_Assert(false, "AVP_Move_Alien_AI_TurnAndJumpSurfaceBase::Start - must start with a message.");
		return false;
	}

	const AVP_Message_Alien_AI_Move_TurnStart* pxMessage = static_cast< const AVP_Message_Alien_AI_Move_TurnStart* >( pxStartMsg );


	// We need to ensure the correct anim hash is set up, as it's about to get used.
	AVP_ClientActor_Alien& xAlienOwner = static_cast<AVP_ClientActor_Alien&>(GetAVPParentSystem().GetActorOwner());

	xAlienOwner.SetStanding( false, false );

	if (m_pxAVPAnimationData)
	{
		if (xAlienOwner.IsUsingFirstPersonSkin())
		{
			Asura_Assert(false, "AVP_Move_Alien_AI_Turn180Left::Start - first person not expected.");
			return false;
		}
		else
		{
			m_pxAVPAnimationData->SetAnimHash(m_uThirdPersonAnimHash);
		}

	}

	const bool bHasAnimation = m_pxAVPAnimationData && m_pxAVPAnimationData->HasValidAnimation();
	const float	fDuration = (bHasAnimation) ? m_pxAVPAnimationData->GetAnimationDuration() : 0.500f;

	Asura_Matrix_3x3 xTurntOrientation;
	pxMessage->GetTurnOrientation(xTurntOrientation);

	if(bHasAnimation)
	{
		m_xOrientationInterpolationData.SetMovementOffsetOperation(Asura_MoveData_OrientationInterpolation::STRETCH_MOVEMENT_OFFSET);
	}

	m_xOrientationInterpolationData.SetDuration(fDuration);
	m_xOrientationInterpolationData.SetFinal(xTurntOrientation);

	InitialiseOrientationInterpolation(m_xOrientationInterpolationData);

	// Finally, call the base-function...
	return PARENT::Start(pxStartMsg);
}

void AVP_Move_Alien_AI_Turn180Left::Abort()
{
	ResetPhysicsObjectHandler();

	PARENT::Abort();
}


//////////////////////////////////////////////////////////////////////////////
//
// AVP_Move_Alien_AI_Turn180Right
// Turn Before Jump
//
//////////////////////////////////////////////////////////////////////////////

AVP_Move_Alien_AI_Turn180Right::AVP_Move_Alien_AI_Turn180Right( Asura_Movement_System& xParentSystem)
: PARENT( xParentSystem, GetStaticHash() )
{

	Asura_Hash_ID uFPAnim = ASURA_HASH_ID_UNSET;	// non player anim
	Asura_Hash_ID uTPAnim = ASURA_HASH_ID_UNSET;	
	Asura_Value xValue;

	Asura_Blueprint* pxBP = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_MOVESALIEN, g_uAIMovesBP );
	if ( pxBP )
	{
		if ( pxBP->GetParameterValueByHashID( g_u180TurnToCrawlRight, xValue ) )
		{
			uTPAnim = xValue.GetHashIDValue();
		}
	}

	SetAnimHashes( uFPAnim, uTPAnim );

	// Don't tween in our anim.
	if (m_pxAVPAnimationData)
	{
		m_pxAVPAnimationData->SetTweenData( 0.05f, 0.0f, false );
	}

	SetPositionUsage(POSITION_SET_FROM_VELOCITY);
	SetAimDirectionUsage(AVP_ClientEntity_Actor::AIM_DIRECTION_RESET);
	SetLookAtPositionUsage(AVP_ClientEntity_Actor::LOOK_AT_POSITION_RESET);
	SetPreventsFlinch( true );

}

bool AVP_Move_Alien_AI_Turn180Right::Start( const Asura_Msg_Move* pxStartMsg )
{
	if ( !pxStartMsg ) // start message is not expected
	{
		Asura_Assert(false, "AVP_Move_Alien_AI_TurnAndJumpSurfaceBase::Start - must start with a message.");
		return false;
	}

	const AVP_Message_Alien_AI_Move_TurnStart* pxMessage = static_cast< const AVP_Message_Alien_AI_Move_TurnStart* >( pxStartMsg );


	// We need to ensure the correct anim hash is set up, as it's about to get used.
	AVP_ClientActor_Alien& xAlienOwner = static_cast<AVP_ClientActor_Alien&>(GetAVPParentSystem().GetActorOwner());

	xAlienOwner.SetStanding( false, false );

	if (m_pxAVPAnimationData)
	{
		if (xAlienOwner.IsUsingFirstPersonSkin())
		{
			Asura_Assert(false, "AVP_Move_Alien_AI_Turn180Left::Start - first person not expected.");
			return false;
		}
		else
		{
			m_pxAVPAnimationData->SetAnimHash(m_uThirdPersonAnimHash);
		}

	}

	const bool bHasAnimation = m_pxAVPAnimationData && m_pxAVPAnimationData->HasValidAnimation();
	const float	fDuration = (bHasAnimation) ? m_pxAVPAnimationData->GetAnimationDuration() : 0.500f;

	Asura_Matrix_3x3 xTurntOrientation;
	pxMessage->GetTurnOrientation(xTurntOrientation);

	if(bHasAnimation)
	{
		m_xOrientationInterpolationData.SetMovementOffsetOperation(Asura_MoveData_OrientationInterpolation::STRETCH_MOVEMENT_OFFSET);
	}

	m_xOrientationInterpolationData.SetDuration(fDuration);
	m_xOrientationInterpolationData.SetFinal(xTurntOrientation);

	InitialiseOrientationInterpolation(m_xOrientationInterpolationData);

	// Finally, call the base-function...
	return PARENT::Start(pxStartMsg);
}

void AVP_Move_Alien_AI_Turn180Right::Abort()
{
	ResetPhysicsObjectHandler();

	PARENT::Abort();
}


 
//////////////////////////////////////////////////////////////////////////////
//
// AVP_Move_Alien_AI_FallStart
// Falling start
//
//////////////////////////////////////////////////////////////////////////////

DEBUGVAR float dbg_fLandDistance = 1.5f;
DEBUGVAR float dbg_fLandDistanceSq = dbg_fLandDistance * dbg_fLandDistance;
DEBUGVAR float dng_fTouchDistanceSq = 0.4f * 0.4f;
DEBUGVAR float dbg_fAIAwayFromWallVelocity = 4.0f;
DEBUGVAR float dbg_fAIMaxRotateTime = 0.5f;

AVP_Move_Alien_AI_FallStart::AVP_Move_Alien_AI_FallStart( Asura_Movement_System& xParentSystem )
: PARENT( xParentSystem, GetStaticHash() )
, m_xOrientationInterpolationData()
, m_fTimeSinceStart(0.0f)
, m_bFinished(false)
, m_bStartMiddleMove(false)
, m_bStartEndMove(false)
, m_fDuration(0.0f)
, m_xAwayFromSurfaceStart(Asura_Maths::ZeroAVector3)
, m_xEndPosition(Asura_Maths::ZeroAVector3)
{
	Asura_Hash_ID uFPAnim = ASURA_HASH_ID_UNSET;
	Asura_Hash_ID uTPAnim = ASURA_HASH_ID_UNSET;
	Asura_Value xValue;

	Asura_Blueprint* pxBP = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_MOVESALIEN, g_uAIMovesBP );
	if ( pxBP )
	{
		if ( pxBP->GetParameterValueByHashID( g_uAIFallingBegin, xValue ) )
		{
			uTPAnim = xValue.GetHashIDValue();
		}
	}

	SetAnimHashes( uFPAnim, uTPAnim );

	SetAimDirectionUsage( AVP_ClientEntity_Actor::AIM_DIRECTION_RESET );
	SetPreventsFlinch( true );
	SetPositionUsage(Asura_Move::POSITION_SET_FROM_VELOCITY);
	AddNextMove(AVP_Move_Alien_AI_FallMiddle::GetStaticHash());
	AddNextMove(AVP_Move_Alien_AI_FallEnd::GetStaticHash());

	if (m_pxAVPAnimationData)
	{
		m_pxAVPAnimationData->SetLeaveAnimAtEnd(true);
		m_pxAVPAnimationData->SetTweenData( 0.15f, 0.0f, false );
	}

}

bool AVP_Move_Alien_AI_FallStart::Start(const Asura_Msg_Move* pxStartMsg)
{
	if ( !pxStartMsg ) // start message is not expected
	{
		Asura_Assert(false, "AVP_Move_Alien_AI_TurnAndJumpSurfaceBase::Start - must start with a message.");
		return false;
	}

	const AVP_Message_Alien_AI_Move_FallStart* pxMessage = static_cast< const AVP_Message_Alien_AI_Move_FallStart* >( pxStartMsg );

	AVP_ClientActor_Alien& xAlienOwner = static_cast<AVP_ClientActor_Alien&>(GetAVPParentSystem().GetActorOwner());

	// Calculate our target orientation...
	Asura_Matrix_3x3 xDestOrientation; 
	pxMessage->GetDestinationOrientation(xDestOrientation);
	const Asura_Vector_3 xDestUp(xDestOrientation.e12, xDestOrientation.e22, xDestOrientation.e32);

	// For the time being we'll just rotate to our desired position...
	const Asura_Matrix_3x3& xCurrentOrientation = xAlienOwner.GetCurrentOrientation();
	const Asura_Vector_3 xCurrentUp(xCurrentOrientation.e12, xCurrentOrientation.e22, xCurrentOrientation.e32);
	float fTurnAngle = Asura_Maths::ArcCosine(xDestUp * xCurrentUp);
	const bool bHasAnimation = m_pxAVPAnimationData && m_pxAVPAnimationData->HasValidAnimation();
	const float	fAnimDuration = (bHasAnimation) ? m_pxAVPAnimationData->GetAnimationDuration() : 0.0f;
	m_fDuration = Asura_Maths::Max(fAnimDuration, fTurnAngle * dbg_fAIMaxRotateTime / Asura_Maths::Pi);
	m_xOrientationInterpolationData.SetDuration( m_fDuration );
	m_xOrientationInterpolationData.SetInitial( xAlienOwner.GetCurrentOrientation() );
	m_xOrientationInterpolationData.SetFinal( xDestOrientation );
	m_xOrientationInterpolationData.SetSingularityHintAxis( Asura_AVector3( 0.0f, 0.0f, 1.0f ) );
	InitialiseOrientationInterpolation( m_xOrientationInterpolationData );

	// set the gravity down if we on the server machine
	AVP_PhysicsObjectHandler* pxPhyHandler = NULL;
	if (Asura_Network::IsServer())
	{
		AVP_ServerActor_Vertebrate* pxOwner = AVP_ServerActor_Vertebrate::FindEntity(GetAVPParentSystem().GetOwnerGuid());
		pxPhyHandler = pxOwner ? pxOwner->GetPhysicsObjectHandler() : NULL;
	}

	if ( pxPhyHandler == NULL )
	{
		AVP_ClientActor_Vertebrate* pxOwner = AVP_ClientActor_Vertebrate::FindEntity(GetAVPParentSystem().GetOwnerGuid());
		pxPhyHandler = pxOwner ? pxOwner->GetPhysicsObjectHandler() : NULL;
	}

	// Asura_Assertf( pxPhyHandler, ( "Unable to get physics object handler for entity %d" , GetAVPParentSystem().GetOwnerGuid() ) );
	if ( pxPhyHandler )
	{
		pxPhyHandler->SetForcedTargetSurfaceNormal(Asura_Vector_3(0.0f, -1.0f, 0.0f));
		pxPhyHandler->GetCurrentSurfaceNormal(m_xAwayFromSurfaceStart);
		if (m_xAwayFromSurfaceStart.GetY() > 0.0f)
		{
			m_xAwayFromSurfaceStart.SetY(0.0f);
		}
	}


	m_fTimeSinceStart = 0.0f;
	m_bFinished = false;
	m_bStartMiddleMove = false;
	m_bStartEndMove = false;

	// Finally, call the base-function...
	return(PARENT::Start(pxStartMsg));
}

void AVP_Move_Alien_AI_FallStart::Abort()
{
	AVP_PhysicsObjectHandler* pxPhyHandler = NULL;
	if (Asura_Network::IsServer())
	{
		AVP_ServerActor_Vertebrate* pxOwner = AVP_ServerActor_Vertebrate::FindEntity(GetAVPParentSystem().GetOwnerGuid());
		pxPhyHandler = pxOwner ? pxOwner->GetPhysicsObjectHandler() : NULL;
	}

	if ( pxPhyHandler == NULL )
	{
		AVP_ClientActor_Vertebrate* pxOwner = AVP_ClientActor_Vertebrate::FindEntity(GetAVPParentSystem().GetOwnerGuid());
		pxPhyHandler = pxOwner ? pxOwner->GetPhysicsObjectHandler() : NULL;
	}

	// Asura_Assertf( pxPhyHandler, ( "Unable to get physics object handler for entity %d" , GetAVPParentSystem().GetOwnerGuid() ) );
	if ( pxPhyHandler )
	{
		pxPhyHandler->UnsetForcedTargetSurfaceNormal();
	}
	PARENT::Abort();
}

void AVP_Move_Alien_AI_FallStart::UpdateStates(const float fTimeStep)
{

	AVP_Actor_PhysicsObjectHandler* pxPhyHandler = NULL;

	m_fTimeSinceStart += fTimeStep;

	if ( m_fTimeSinceStart < m_fDuration)
	{
		PARENT::UpdateStates(fTimeStep);
		return;
	}

	Asura_AVector3 xStartPosition;
	Asura_Matrix_3x3 xOrientation;

	if (Asura_Network::IsServer())
	{
		AVP_ServerEntity_Actor* pxActorOwner = AVP_ServerEntity_Actor::FindEntity( GetAVPParentSystem().GetOwnerGuid() );
		if ( pxActorOwner )
		{
			xStartPosition = Asura_AVector3(pxActorOwner->GetPosition());
			xOrientation = pxActorOwner->GetOrientation();
			pxPhyHandler = pxActorOwner->GetPhysicsObjectHandler();
		}
	}

	if ( pxPhyHandler == NULL )
	{
		AVP_ClientEntity_Actor* pxActorOwner = AVP_ClientEntity_Actor::FindEntity( GetAVPParentSystem().GetOwnerGuid() );
		if ( pxActorOwner )
		{
			xStartPosition = Asura_AVector3(pxActorOwner->GetCurrentPosition());
			xOrientation = pxActorOwner->GetCurrentOrientation();
			pxPhyHandler = pxActorOwner->GetPhysicsObjectHandler();

		}
	}

	// Asura_Assertf( pxPhyHandler, ( "Unable to get physics object handler for entity %d" , GetAVPParentSystem().GetOwnerGuid() ) );
	const Asura_Bounding_Box& xBB = pxPhyHandler->GetPhysicsObject()->GetCollisionObject()->GetBoundingBox();
	Asura_OBB xOBB(xBB, Asura_AMatrix3x3(xOrientation));

	Asura_Physics_MovementQuery xQuery( xStartPosition, Asura_AVector3(0.0f, dbg_fLandDistance + 0.5f ,0.0f),
		xOBB,
		Asura_Physics_MovementQuery::ASURA_PMQCP_OBB,
		AsuraPolygonFlag_NoCollisions, 
		true, // bStopAtFirstCollision 
		true, // bCollideWithStaticObjects	
		true, // bCollideWithExternalControlObjects 
		false, // bCollideWithActorObjects 
		true, // bCollideWithEnvironment
		true //  bCollideWithDynamicObjects
		);
	xQuery.SetHandleToIgnore( pxPhyHandler->GetPhysicsObject()->GetHandle() );

	Asura_Physics_MovementQuery_Results xResults(xQuery);

	float fDisplacementAllowedSq = xResults.GetAllowedDisplacement().MagnitudeSquared();
	if (fDisplacementAllowedSq < dng_fTouchDistanceSq )
	{
		m_bStartMiddleMove = false;
		m_bStartEndMove = false;
		if (m_pxAVPAnimationData)
		{
			m_pxAVPAnimationData->SetTweenData( 0.15f, 0.15f, false );
			m_pxAVPAnimationData->UpdateTweenData();
		}
	}
	else if (fDisplacementAllowedSq < dbg_fLandDistanceSq )
	{
		m_bStartMiddleMove = false;
		m_bStartEndMove = true;
		m_xEndPosition = Asura_Vector_3(xResults.GetEndPosition());
	}
	else
	{
		m_bStartMiddleMove = true;
		m_bStartEndMove = false;
	}

	m_bFinished = true;

	// release the orientation if this is really the end
	if ( m_bFinished  && !m_bStartMiddleMove && !m_bStartEndMove && pxPhyHandler)
	{
		pxPhyHandler->UnsetForcedTargetSurfaceNormal();
	}

	PARENT::UpdateStates(fTimeStep);
}


bool AVP_Move_Alien_AI_FallStart::HasFinished() const
{ 
	if (m_bFinished)
	{
		return true;
	}
	else
	{
		return PARENT::HasFinished();
	}
}


bool AVP_Move_Alien_AI_FallStart::CalculateVelocity(const float fTimeStep, Asura_AVector3_OutParam xVelocity) const
{
//	bool bOK = PARENT::CalculateVelocity(fTimeStep, xVelocity);
	if (m_fTimeSinceStart < 1.0f )
	{
		xVelocity = ( m_xAwayFromSurfaceStart * dbg_fAIAwayFromWallVelocity ) /* + Asura_AVector3( 0.0f, -1.0f, 0.0f ) */;
	}
	return true;
}


//////////////////////////////////////////////////////////////////////////////
//
// AVP_Move_Alien_AI_FallMiddle
// Falling Middle
//
//////////////////////////////////////////////////////////////////////////////

AVP_Move_Alien_AI_FallMiddle::AVP_Move_Alien_AI_FallMiddle( Asura_Movement_System& xParentSystem )
: PARENT( xParentSystem, GetStaticHash() )
, m_fTimeSinceStart(0.0f)
, m_bFinished(false)
, m_xEndPosition(Asura_Maths::ZeroAVector3)
{
	Asura_Hash_ID uFPAnim = ASURA_HASH_ID_UNSET;
	Asura_Hash_ID uTPAnim = ASURA_HASH_ID_UNSET;
	Asura_Value xValue;

	Asura_Blueprint* pxBP = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_MOVESALIEN, g_uAIMovesBP );
	if ( pxBP )
	{
		if ( pxBP->GetParameterValueByHashID( g_uAIFallingMiddle, xValue ) )
		{
			uTPAnim = xValue.GetHashIDValue();
		}
	}

	SetAnimHashes( uFPAnim, uTPAnim );

	SetAimDirectionUsage( AVP_ClientEntity_Actor::AIM_DIRECTION_RESET );
	SetPreventsFlinch( true );
	SetPositionUsage(Asura_Move::POSITION_SET_FROM_VELOCITY);
	AddNextMove(AVP_Move_Alien_AI_FallEnd::GetStaticHash());

	if (m_pxAVPAnimationData)
	{
		m_pxAVPAnimationData->SetTweenData( 0.15f, 0.15f, true );
		SetLoopAtEnd( true );
		m_pxAVPAnimationData->SetLeaveAnimAtEnd(true);
	}

}

bool AVP_Move_Alien_AI_FallMiddle::ShouldStart() const
{
	const AVP_Move* pxPrevMove =
		static_cast< const AVP_Move* >( GetParentSystem().GetMove( AVP_Move_Alien_AI_FallStart::GetStaticHash() ) );
	if ( pxPrevMove )
	{
		return pxPrevMove->HasFinished() && static_cast<const AVP_Move_Alien_AI_FallStart*>(pxPrevMove)->ShouldMiddleStart();
	}
	else
	{
		Asura_Assert(false, "can not find previous move");
		return false;
	}
}

bool AVP_Move_Alien_AI_FallMiddle::Start(const Asura_Msg_Move* pxStartMsg)
{
	if ( pxStartMsg ) // start message is not expected
	{
		Asura_Assert(false, "AVP_Move_Alien_AI_FallMiddle::Start - must not start with a message.");
		return false;
	}

	m_fTimeSinceStart = 0.0f;
	m_bFinished = false;

	// Finally, call the base-function...
	return(PARENT::Start(pxStartMsg));
}

void AVP_Move_Alien_AI_FallMiddle::Abort()
{
	AVP_PhysicsObjectHandler* pxPhyHandler = NULL;
	if (Asura_Network::IsServer())
	{
		AVP_ServerActor_Vertebrate* pxOwner = AVP_ServerActor_Vertebrate::FindEntity(GetAVPParentSystem().GetOwnerGuid());
		pxPhyHandler = pxOwner ? pxOwner->GetPhysicsObjectHandler() : NULL;
	}

	if ( pxPhyHandler == NULL )
	{
		AVP_ClientActor_Vertebrate* pxOwner = AVP_ClientActor_Vertebrate::FindEntity(GetAVPParentSystem().GetOwnerGuid());
		pxPhyHandler = pxOwner ? pxOwner->GetPhysicsObjectHandler() : NULL;
	}

	// Asura_Assertf( pxPhyHandler, ( "Unable to get physics object handler for entity %d" , GetAVPParentSystem().GetOwnerGuid() ) );
	if ( pxPhyHandler )
	{
		pxPhyHandler->UnsetForcedTargetSurfaceNormal();
	}
	PARENT::Abort();
}


void AVP_Move_Alien_AI_FallMiddle::UpdateStates(const float fTimeStep)
{
	AVP_Actor_PhysicsObjectHandler* pxPhyHandler = NULL;

	m_fTimeSinceStart += fTimeStep;

	Asura_AVector3 xStartPosition;
	Asura_Matrix_3x3 xOrientation;

	if (Asura_Network::IsServer())
	{
		AVP_ServerEntity_Actor* pxActorOwner = AVP_ServerEntity_Actor::FindEntity( GetAVPParentSystem().GetOwnerGuid() );
		if ( pxActorOwner )
		{
			xStartPosition = Asura_AVector3(pxActorOwner->GetPosition());
			xOrientation = pxActorOwner->GetOrientation();
			pxPhyHandler = pxActorOwner->GetPhysicsObjectHandler();
		}
	}

	if ( pxPhyHandler == NULL )
	{
		AVP_ClientEntity_Actor* pxActorOwner = AVP_ClientEntity_Actor::FindEntity( GetAVPParentSystem().GetOwnerGuid() );
		if ( pxActorOwner )
		{
			xStartPosition = Asura_AVector3(pxActorOwner->GetCurrentPosition());
			xOrientation = pxActorOwner->GetCurrentOrientation();
			pxPhyHandler = pxActorOwner->GetPhysicsObjectHandler();

		}
	}

	// Asura_Assertf( pxPhyHandler, ( "Unable to get physics object handler for entity %d" , GetAVPParentSystem().GetOwnerGuid() ) );
	const Asura_Bounding_Box& xBB = pxPhyHandler->GetPhysicsObject()->GetCollisionObject()->GetBoundingBox();
	Asura_OBB xOBB(xBB, Asura_AMatrix3x3(xOrientation));

	Asura_Physics_MovementQuery xQuery( xStartPosition, Asura_AVector3(0.0f, dbg_fLandDistance + 0.5f ,0.0f),
		xOBB,
		Asura_Physics_MovementQuery::ASURA_PMQCP_OBB,
		AsuraPolygonFlag_NoCollisions, 
		true, // bStopAtFirstCollision 
		true, // bCollideWithStaticObjects	
		true, // bCollideWithExternalControlObjects 
		false, // bCollideWithActorObjects 
		true, // bCollideWithEnvironment
		true //  bCollideWithDynamicObjects
		);

	xQuery.SetHandleToIgnore( pxPhyHandler->GetPhysicsObject()->GetHandle() );
	Asura_Physics_MovementQuery_Results xResults(xQuery);

	float fDisplacementAllowedSq = xResults.GetAllowedDisplacement().MagnitudeSquared();
	if (fDisplacementAllowedSq <  dbg_fLandDistanceSq)
	{
		m_xEndPosition = Asura_Vector_3(xResults.GetEndPosition());
		m_bFinished = true;
	}


	// release the orientation if this is really the end
	if ( m_bFinished  && pxPhyHandler)
	{
		pxPhyHandler->UnsetForcedTargetSurfaceNormal();
	}

	PARENT::UpdateStates(fTimeStep);
}

bool AVP_Move_Alien_AI_FallMiddle::HasFinished() const
{
	return m_bFinished;
}

//////////////////////////////////////////////////////////////////////////////
//
// AVP_Move_Alien_AI_FallEnd
// Falling Middle
//
//////////////////////////////////////////////////////////////////////////////

AVP_Move_Alien_AI_FallEnd::AVP_Move_Alien_AI_FallEnd( Asura_Movement_System& xParentSystem )
: PARENT( xParentSystem, GetStaticHash() )
, m_xPositionInterpolationData()
{
	Asura_Hash_ID uFPAnim = ASURA_HASH_ID_UNSET;
	Asura_Hash_ID uTPAnim = ASURA_HASH_ID_UNSET;
	Asura_Value xValue;

	Asura_Blueprint* pxBP = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_MOVESALIEN, g_uAIMovesBP );
	if ( pxBP )
	{
		if ( pxBP->GetParameterValueByHashID( g_uAIFallingEnd, xValue ) )
		{
			uTPAnim = xValue.GetHashIDValue();
		}
	}

	SetAnimHashes( uFPAnim, uTPAnim );

	SetAimDirectionUsage( AVP_ClientEntity_Actor::AIM_DIRECTION_RESET );
	SetPreventsFlinch( true );
	SetPositionUsage(Asura_Move::POSITION_SET_FROM_VELOCITY);

	if (m_pxAVPAnimationData)
	{
		m_pxAVPAnimationData->SetTweenData( 0.05f, 0.05f, true );
	}


}

bool AVP_Move_Alien_AI_FallEnd::ShouldStart() const
{
	const Asura_Move* pxPrevMove = GetParentSystem().GetCurrentMove();

	Asura_Assert(pxPrevMove, "can not find previous move");

	if (!pxPrevMove)
	{
		return false;
	}
	else if (pxPrevMove->GetHash() == AVP_Move_Alien_AI_FallStart::GetStaticHash())
	{
		return pxPrevMove->HasFinished() && static_cast<const AVP_Move_Alien_AI_FallStart*>(pxPrevMove)->ShouldEndStart();
	}
	else if (pxPrevMove->GetHash() == AVP_Move_Alien_AI_FallMiddle::GetStaticHash())
	{
		return pxPrevMove->HasFinished();
	}
	else
	{
		Asura_Assert(false, "previous move unknown");
		return false;
	}
}

bool AVP_Move_Alien_AI_FallEnd::Start(const Asura_Msg_Move* pxStartMsg)
{

	// Have we been told to start via a message?
	if(pxStartMsg != NULL)
	{
		Asura_Assert(false, "AVP_Move_Alien_AI_FallEndStart move can not be triggered by message.");
		return(false);
	}
	else
	{
		AVP_ClientActor_Alien& xAlienOwner = static_cast<AVP_ClientActor_Alien&>(GetAVPParentSystem().GetActorOwner());

		xAlienOwner.SetStanding( false, false );

		// We need to ensure the correct anim hash is set up, as it's about to get used.
		if (m_pxAVPAnimationData)
		{
			if (xAlienOwner.IsUsingFirstPersonSkin())
			{
				m_pxAVPAnimationData->SetAnimHash(m_uFirstPersonAnimHash);
			}
			else
			{
				m_pxAVPAnimationData->SetAnimHash(m_uThirdPersonAnimHash);
			}
		}

		const bool bHasAnimation = m_pxAVPAnimationData && m_pxAVPAnimationData->HasValidAnimation();
		const float	fDuration = (bHasAnimation) ? m_pxAVPAnimationData->GetAnimationDuration() : 0.500f;

		if(bHasAnimation)
		{
			m_xPositionInterpolationData.SetMovementOffsetOperation(Asura_MoveData_PositionInterpolation::STRETCH_MOVEMENT_OFFSET);
		}


		const Asura_Move* pxPrevMove = GetParentSystem().GetCurrentMove();
		Asura_Assert(pxPrevMove, "can not find previous move");
		if (!pxPrevMove)
		{
			return false;
		}
		else if (pxPrevMove->GetHash() == AVP_Move_Alien_AI_FallStart::GetStaticHash())
		{
			m_xPositionInterpolationData.SetFinal(static_cast<const AVP_Move_Alien_AI_FallStart*>(pxPrevMove)->GetEndPosition());
		}
		else if (pxPrevMove->GetHash() == AVP_Move_Alien_AI_FallMiddle::GetStaticHash())
		{
			m_xPositionInterpolationData.SetFinal(static_cast<const AVP_Move_Alien_AI_FallMiddle*>(pxPrevMove)->GetEndPosition());
		}
		else
		{	
			return false;
		}

		m_xPositionInterpolationData.SetDuration(fDuration);
		InitialisePositionInterpolation(m_xPositionInterpolationData);

		// Finally, call the base-function...
		return(PARENT::Start(pxStartMsg));
	}
}


void AVP_Move_Alien_AI_FallEnd::Abort()
{
	ResetPhysicsObjectHandler();

	PARENT::Abort();
}


//////////////////////////////////////////////////////////////////////////////
//
// AVP_Move_Alien_Hiss
// Hissing
//
//////////////////////////////////////////////////////////////////////////////

AVP_Move_Alien_Hiss::AVP_Move_Alien_Hiss( Asura_Movement_System& xParentSystem )
: PARENT( xParentSystem, GetStaticHash() )
{
	Asura_Hash_ID uFPAnim = ASURA_HASH_ID_UNSET;
	Asura_Hash_ID uTPAnim = ASURA_HASH_ID_UNSET;
	Asura_Value xValue;

	Asura_Blueprint* pxBP = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_MOVESALIEN, g_uAIMovesBP );
	if ( pxBP )
	{
		if ( pxBP->GetParameterValueByHashID( g_uAIHiss, xValue ) )
		{
			uTPAnim = xValue.GetHashIDValue();
		}
	}

	SetAnimHashes( uFPAnim, uTPAnim );
	SetPreventsFlinch( true );
	SetAimDirectionUsage( AVP_ClientEntity_Actor::AIM_DIRECTION_RESET );

	if (m_pxAVPAnimationData)
	{
		m_pxAVPAnimationData->SetTweenData( 0.01f, 0.25f, true );
	}
}

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////
