
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_Alien_MovesVent.cpp
// Author : Will Plano
// Date   : 14/03/2008
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "Moves/Vertebrates/Alien/AVP_Alien_MovesVent.h"

#include "Entity/PhysicalObject/Moves/Asura_Movement_System.h"

#include "Actor/Vertebrates/Alien/AVP_Alien_Focus.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Alien/AVP_ClientActor_Alien.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Alien/AVP_Messages_Alien.h"
#include "Entity/Vent/AVP_ClientEntity_Vent.h"
#include "Moves/Vertebrates/Alien/AVP_Alien_MovesJump.h"

AVP_Move_Alien_Vent::AVP_Move_Alien_Vent( Asura_Movement_System& xParentSystem, Asura_Hash_ID uHash )
: PARENT( xParentSystem, uHash )
{
	m_bDisableSurfaceReorientation = true;
}

bool AVP_Move_Alien_Vent::Start( const Asura_Msg_Move* pxStartMsg )
{
	AVP_ClientActor_Alien* pxAlien = AVP_ClientActor_Alien::FindEntity( m_xParentSystem.GetOwnerGuid() );

	// We need to ensure the correct anim hash is set up, as it's about to get used.
	if (m_pxAVPAnimationData)
	{
		if (pxAlien->IsUsingFirstPersonSkin())
		{
			m_pxAVPAnimationData->SetAnimHash(m_uFirstPersonAnimHash);
		}
		else
		{
			m_pxAVPAnimationData->SetAnimHash(m_uThirdPersonAnimHash);
		}
	}

	const bool bHasAnimation = m_pxAVPAnimationData && m_pxAVPAnimationData->HasValidAnimation();
	const float	fDuration = (bHasAnimation) ? m_pxAVPAnimationData->GetAnimationDuration() : 0.500f;

	// For the time being we'll just move and rotate to our desired position...
	if(bHasAnimation)
	{
		m_xPositionInterpolationData.SetMovementOffsetOperation(Asura_MoveData_PositionInterpolation::STRETCH_MOVEMENT_OFFSET);
		m_xOrientationInterpolationData.SetMovementOffsetOperation(Asura_MoveData_OrientationInterpolation::STRETCH_MOVEMENT_OFFSET);
	}

	m_xPositionInterpolationData.SetDuration(fDuration);
	m_xPositionInterpolationData.SetFinal(m_xTargetPosition);
	InitialisePositionInterpolation(m_xPositionInterpolationData);

	Asura_Matrix_3x3 xMat;
	m_xTargetOrientation.ToMat3( xMat );
	m_xOrientationInterpolationData.SetDuration(fDuration);
	m_xOrientationInterpolationData.SetFinal( xMat );
	InitialiseOrientationInterpolation(m_xOrientationInterpolationData);

	SetAimDirectionUsage( AVP_ClientEntity_Actor::AIM_DIRECTION_RESET );

	if ( pxAlien->GetFocus() )
	{
		static_cast< AVP_Alien_Focus* >( pxAlien->GetFocus() )->UnsetTarget();
	}

	// Finally, call the base-function...
	return(PARENT::Start( pxStartMsg ));
}


AVP_Move_Alien_AI_TraverseVent_Begin::AVP_Move_Alien_AI_TraverseVent_Begin( Asura_Movement_System& xParentSystem )
: PARENT( xParentSystem, GetStaticHash() )
{
}

bool AVP_Move_Alien_AI_TraverseVent_Begin::Start( const Asura_Msg_Move* pxStartMsg )
{
	Asura_Assert( pxStartMsg != NULL, "Should only start this move from a messasge" );
	if ( pxStartMsg == NULL )
	{
		return false;
	}
	const AVP_Message_Alien_AI_Move_VentStart* pxData = static_cast< const AVP_Message_Alien_AI_Move_VentStart* >( pxStartMsg );

	SetAnimHashes( ASURA_HASH_ID_UNSET, pxData->GetAnimHash() );

	// Only tween in to our anim.
	if (m_pxAVPAnimationData)
	{
		m_pxAVPAnimationData->SetTweenData( 0.05f, 0.5f, true );
	}

	AVP_ClientActor_Alien* pxAlien = AVP_ClientActor_Alien::FindEntity( m_xParentSystem.GetOwnerGuid() );

	// We need to ensure the correct anim hash is set up, as it's about to get used.
	if (m_pxAVPAnimationData)
	{
		if (pxAlien->IsUsingFirstPersonSkin())
		{
			m_pxAVPAnimationData->SetAnimHash(m_uFirstPersonAnimHash);
		}
		else
		{
			m_pxAVPAnimationData->SetAnimHash(m_uThirdPersonAnimHash);
		}
	}

	const bool bHasAnimation = m_pxAVPAnimationData && m_pxAVPAnimationData->HasValidAnimation();
	const float	fDuration = .25f;// (bHasAnimation) ? m_pxAVPAnimationData->GetAnimationDuration() : 0.500f;

	const Asura_Hierarchy_Anim* pxHA = Asura_Animation_System::GetAnimByHash( pxData->GetAnimHash() );
	const Asura_Hierarchy_Skin* pxSkin = pxAlien->GetAnimation()->GetSkin();
	Asura_Vector_3 xDummy;
	Asura_Quat xAnimRotation;

	Asura_Matrix_3x3 xRotateToStart = Asura_Maths::IdentityMatrix3x3;;

	if (pxHA && pxSkin)
	{
		pxHA->GetMovementOffset(pxSkin, 1.0f, &xDummy, &xAnimRotation, false);		
		xAnimRotation.Invert();
		xAnimRotation.ToMatrix(xRotateToStart);		
	}
	else
	{
		Asura_Assert(pxHA, "Asura_Hierarchy_Anim not found");
		Asura_Assert(pxSkin, "Asura_Hierarchy_Skin not found");
	}

	// For the time being we'll just move and rotate to our desired position...
	if(bHasAnimation)
	{
		m_xPositionInterpolationData.SetMovementOffsetOperation(Asura_MoveData_PositionInterpolation::SUBTRACT_MOVEMENT_OFFSET);
		m_xPositionInterpolationData.SetVelocityOperation(Asura_MoveData_PositionInterpolation::IGNORE_VELOCITY);
		m_xOrientationInterpolationData.SetMovementOffsetOperation(Asura_MoveData_OrientationInterpolation::ADD_MOVEMENT_OFFSET);
	}

	m_xPositionInterpolationData.SetDuration(fDuration);

	m_xPositionInterpolationData.SetFinal(pxData->GetDestPos());
	InitialisePositionInterpolation(m_xPositionInterpolationData);

	Asura_Matrix_3x3 xOrientation;
	pxData->GetDestOri().ToMatrix( xOrientation );
	m_xOrientationInterpolationData.SetDuration(fDuration);
	m_xOrientationInterpolationData.SetFinal( xOrientation * xRotateToStart );
	InitialiseOrientationInterpolation(m_xOrientationInterpolationData);

	SetAimDirectionUsage( AVP_ClientEntity_Actor::AIM_DIRECTION_RESET );

	// Finally, call the base-function...
	return(PARENT::Start( pxStartMsg ));
}

// AVP_Move_Alien_AI_TraverseVent_End::AVP_Move_Alien_AI_TraverseVent_End( Asura_Movement_System& xParentSystem )
// : PARENT( xParentSystem, GetStaticHash() )
// {
// }
// 
// bool AVP_Move_Alien_AI_TraverseVent_End::Start( const Asura_Msg_Move* pxStartMsg )
// {
// 	Asura_Assert( pxStartMsg == NULL, "Should never start this move from a messasge" );
// 	if ( pxStartMsg != NULL )
// 	{
// 		return false;
// 	}
// 
// 	const Asura_Move* pxCurrentMove = m_xParentSystem.GetCurrentMove();
// 	Asura_Assert( pxCurrentMove != NULL && pxCurrentMove->GetHash() == AVP_Move_Alien_AI_TraverseVent_Begin::GetStaticHash(), "starting from wrong move" );
// 	if ( pxCurrentMove != NULL && pxCurrentMove->GetHash() == AVP_Move_Alien_AI_TraverseVent_Begin::GetStaticHash() )
// 	{
// 		const AVP_Move_Alien_AI_TraverseVent_Begin* pxStartMove = static_cast< const AVP_Move_Alien_AI_TraverseVent_Begin* >( pxCurrentMove );
// 		SetAnimHashes(ASURA_HASH_ID_UNSET, pxStartMove->GetFinalAnimHash());
// 
// 		// Only tween out of our anim.
// 		if (m_pxAVPAnimationData)
// 		{
// 			m_pxAVPAnimationData->SetTweenData( 0.0f, 0.05f, true );
// 		}
// 
// 		m_xTargetPosition = pxStartMove->GetFinalPosition();
// 		m_xTargetOrientation = pxStartMove->GetFinalOrientation();
// 
// 		return PARENT::Start( NULL );
// 	}
// 	else
// 	{
// 		return false;
// 	}
// }

AVP_Move_Alien_Player_TraverseVent_Begin::AVP_Move_Alien_Player_TraverseVent_Begin( Asura_Movement_System& xParentSystem )
: PARENT( xParentSystem, GetStaticHash() )
, m_xFinalPosition( Asura_Maths::ZeroAVector3 )
, m_xFinalOrientation( Asura_Maths::IdentityAMatrix3x3 )
{
	AddNextMove( AVP_Move_Alien_Player_TraverseVent_End::GetStaticHash() );

	SetAimDirectionUsage(AVP_ClientEntity_Actor::AIM_DIRECTION_RESET);
}

bool AVP_Move_Alien_Player_TraverseVent_Begin::Start( const Asura_Msg_Move* pxStartMsg )
{
	Asura_Assert( pxStartMsg != NULL, "Should only start this move from a messasge" );
	if ( pxStartMsg == NULL )
	{
		return false;
	}
	const AVP_Message_Alien_Move_VentStart* pxData = static_cast< const AVP_Message_Alien_Move_VentStart* >( pxStartMsg );

	m_xTargetPosition = pxData->GetBeginPos();
	pxData->GetBeginOri().ToMatrix( m_xTargetOrientation );
	m_xFinalPosition = pxData->GetFinalPos();
	pxData->GetFinalOri().ToMatrix( m_xFinalOrientation );

	SetAnimHashes( 0x7098B0B8, 0x44647743 ); //Alien_hud_vent_jump, Alien_Warrior_vent_jump

	// Only tween in to our anim.
	if (m_pxAVPAnimationData)
	{
		m_pxAVPAnimationData->SetTweenData( 0.1f, 0.0f, true );
		m_pxAVPAnimationData->SetLeaveAnimAtEnd(true);
	}

	return(PARENT::Start( pxStartMsg ));
}

AVP_Move_Alien_Player_TraverseVent_End::AVP_Move_Alien_Player_TraverseVent_End( Asura_Movement_System& xParentSystem )
: PARENT( xParentSystem, GetStaticHash() )
{
	SetAimDirectionUsage(AVP_ClientEntity_Actor::AIM_DIRECTION_RESET);
}

bool AVP_Move_Alien_Player_TraverseVent_End::Start( const Asura_Msg_Move* pxStartMsg )
{
	Asura_Assert( pxStartMsg == NULL, "Should never start this move from a messasge" );
	if ( pxStartMsg != NULL )
	{
		return false;
	}

	const Asura_Move* pxCurrentMove = m_xParentSystem.GetCurrentMove();
	Asura_Assert( pxCurrentMove != NULL && pxCurrentMove->GetHash() == AVP_Move_Alien_Player_TraverseVent_Begin::GetStaticHash(), "starting from wrong move" );
	if ( pxCurrentMove != NULL && pxCurrentMove->GetHash() == AVP_Move_Alien_Player_TraverseVent_Begin::GetStaticHash() )
	{
		const AVP_Move_Alien_Player_TraverseVent_Begin* pxStartMove = static_cast< const AVP_Move_Alien_Player_TraverseVent_Begin* >( pxCurrentMove );
		SetAnimHashes(0xC2BE4D12, 0x628FE567); //Alien_hud_vent_jump_land, Alien_Warrior_vent_jump_land

		// Only tween out of our anim.
		if (m_pxAVPAnimationData)
		{
			m_pxAVPAnimationData->SetTweenData( 0.1f, 0.1f, true );
			m_pxAVPAnimationData->SetLeaveAnimAtEnd(true);
		}

		m_xTargetPosition = pxStartMove->GetFinalPosition();
		m_xTargetOrientation = pxStartMove->GetFinalOrientation();

		return PARENT::Start( NULL );
	}
	else
	{
		return false;
	}
}
