
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_Alien_MovesVent.h
// Author : Will Plano
// Date   : 14/03/2008
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __AVP_ALIEN_MOVESVENT_H__
#define __AVP_ALIEN_MOVESVENT_H__

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Entity/PhysicalObject/Moves/Asura_MoveData_Animation.h"

#include "Moves/AVP_Moves.h"

class AVP_Move_Alien_Vent : public AVP_Move
{
public:
	// Public functions...
	AVP_Move_Alien_Vent(Asura_Movement_System& xParentSystem, Asura_Hash_ID uHash);
	virtual ~AVP_Move_Alien_Vent() {}

protected:
	bool Start(const Asura_Msg_Move* pxStartMsg);
	virtual	bool ShouldStart() const { return false; }

	Asura_AVector3		m_xTargetPosition;
	Asura_AMatrix3x3	m_xTargetOrientation;

private:
	// Private typedefs...
	typedef AVP_Move PARENT;

	Asura_MoveData_PositionInterpolation	m_xPositionInterpolationData;
	Asura_MoveData_OrientationInterpolation	m_xOrientationInterpolationData;

	AVP_Move_Alien_Vent  (void);
	AVP_Move_Alien_Vent  (const AVP_Move_Alien_Vent& xOther);
	AVP_Move_Alien_Vent&  operator=	(const AVP_Move_Alien_Vent& xOther);
};   // End of class 'AVP_Move_Alien_Vent'.

class AVP_Move_Alien_AI_TraverseVent_Begin : public AVP_Move
{
public:
	AVP_Move_Alien_AI_TraverseVent_Begin(Asura_Movement_System& xParentSystem);
	virtual ~AVP_Move_Alien_AI_TraverseVent_Begin() {}
	virtual const char*		GetName(void) const	{ return("AVP_Move_Alien_AI_TraverseVent_Begin"); }
	static Asura_Hash_ID GetStaticHash() { return 0xB8F49306; } // AVP_Move_Alien_AI_TraverseVent_Begin
protected:
	virtual bool Start(const Asura_Msg_Move* pxStartMsg);

private:

	Asura_MoveData_PositionInterpolation	m_xPositionInterpolationData;
	Asura_MoveData_OrientationInterpolation	m_xOrientationInterpolationData;

	typedef AVP_Move PARENT;
};   // End of class 'AVP_Move_Alien_AI_TraverseVent_Begin'.

// class AVP_Move_Alien_AI_TraverseVent_End : public AVP_Move_Alien_Vent
// {
// public:
// 	AVP_Move_Alien_AI_TraverseVent_End(Asura_Movement_System& xParentSystem);
// 	virtual ~AVP_Move_Alien_AI_TraverseVent_End() {}
// 	virtual const char*		GetName(void) const	{ return("AVP_Move_Alien_AI_TraverseVent_End"); }
// 	static Asura_Hash_ID GetStaticHash() { return 0x9A6E9678; } // AVP_Move_Alien_AI_TraverseVent_End
// 
// protected:
// 	virtual bool Start(const Asura_Msg_Move* pxStartMsg);
// 	virtual	bool ShouldStart() const { return true; }
// 
// private:
// 	typedef AVP_Move_Alien_Vent PARENT;
// };   // End of class 'AVP_Move_Alien_AI_TraverseVent_End'.

class AVP_Move_Alien_Player_TraverseVent_Begin : public AVP_Move_Alien_Vent
{
public:
	AVP_Move_Alien_Player_TraverseVent_Begin(Asura_Movement_System& xParentSystem);
	virtual ~AVP_Move_Alien_Player_TraverseVent_Begin() {}
	virtual const char* GetName() const { return("AVP_Move_Alien_Player_TraverseVent_Begin"); }
	static Asura_Hash_ID GetStaticHash() { return 0x80A000AD; } // AVP_Move_Alien_Player_TraverseVent_Begin

	const Asura_AVector3& GetFinalPosition() const { return m_xFinalPosition; }
	const Asura_AMatrix3x3& GetFinalOrientation() const { return m_xFinalOrientation; }

protected:
	virtual bool Start(const Asura_Msg_Move* pxStartMsg);

private:
	typedef AVP_Move_Alien_Vent PARENT;

	Asura_AVector3 m_xFinalPosition;
	Asura_AMatrix3x3 m_xFinalOrientation;
};   // End of class 'AVP_Move_Alien_Player_TraverseVent_Begin'.

class AVP_Move_Alien_Player_TraverseVent_End : public AVP_Move_Alien_Vent
{
public:
	AVP_Move_Alien_Player_TraverseVent_End(Asura_Movement_System& xParentSystem);
	virtual ~AVP_Move_Alien_Player_TraverseVent_End() {}
	virtual const char* GetName() const { return("AVP_Move_Alien_Player_TraverseVent_End"); }
	static Asura_Hash_ID GetStaticHash() { return 0x899741DF; } // AVP_Move_Alien_Player_TraverseVent_End

protected:
	virtual bool Start(const Asura_Msg_Move* pxStartMsg);
	virtual	bool ShouldStart() const { return true; }

private:
	typedef AVP_Move_Alien_Vent PARENT;
};   // End of class 'AVP_Move_Alien_Player_TraverseVent_End'.

#endif   // ifndef __AVP_ALIEN_MOVESVENT_H__

/// End of file. ///
