///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_AlienQueen_MovesMelee.h
// Author : Andy McCabe
// Date   : 19-Nov-2008
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __AVP_ALIENQUEEN_MOVES_MELEE_H__
#define __AVP_ALIENQUEEN_MOVES_MELEE_H__

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Entity/PhysicalObject/Moves/Asura_MoveData_Animation.h"

#include "Camera/AVP_CameraShake_System.h"
#include "Moves/Vertebrates/AVP_MovesMelee.h"

////////////////////////////////////////////////////////////////////////////////
// Forward class declarations...
////////////////////////////////////////////////////////////////////////////////

class Asura_AnimationAttachment;

////////////////////////////////////////////////////////////////////////////////
// Class declaration...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_AlienQueen_Move_Melee_Charge_BuildUp
// Author       : Andy McCabe
// Date         : 19/11/08
////////////////////////////////////////////////////////////////////////////////

class AVP_AlienQueen_Move_Melee_Charge_BuildUp : public AVP_Move_Melee
{
public:
	// Public functions...
	AVP_AlienQueen_Move_Melee_Charge_BuildUp(Asura_Movement_System& xParentSystem, Asura_Hash_ID uFirstPersonBlueprintHash, Asura_Hash_ID uThirdPersonBlueprintHash);
	//	virtual ~AVP_AlienQueen_Move_Melee_Charge_BuildUp();

	static Asura_Hash_ID GetStaticHash() { return 0x91EF2B3F; } //Asura_GetHashID("AVP_AlienQueen_Move_Melee_Charge_BuildUp");

protected:
	// Overridden functions...
	virtual void Abort();

private:
	// Private typedef...
	typedef AVP_Move_Melee PARENT;
};   // End of class 'AVP_AlienQueen_Move_Melee_Charge_BuildUp'.


////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_AlienQueen_Move_Melee_Charge_Loop
// Author       : Andy McCabe
// Date         : 19/11/08
////////////////////////////////////////////////////////////////////////////////

class AVP_AlienQueen_Move_Melee_Charge_Loop : public AVP_Move_Melee
{
public:
	// Public functions...
	AVP_AlienQueen_Move_Melee_Charge_Loop(Asura_Movement_System& xParentSystem, Asura_Hash_ID uFirstPersonBlueprintHash, Asura_Hash_ID uThirdPersonBlueprintHash);
	//	virtual ~AVP_AlienQueen_Move_Melee_Charge_Loop();

	static Asura_Hash_ID GetStaticHash() { return 0xC73AD8AE; } //Asura_GetHashID("AVP_AlienQueen_Move_Melee_Charge_Loop");

	bool ChargeCollide() const { return m_bChargeCollide; }
	bool ChargeCollideInjured() const { return m_bChargeCollideInjured; }
	bool ChargeIntoIdle() const { return m_bChargeIntoIdle; }
	bool HitTarget() const { return m_bHitTarget; }

protected:
	// Overridden functions...
	virtual bool Start(const Asura_Msg_Move* pxStartMsg);
	virtual	bool ShouldStart() const { return true; }
	virtual void UpdateStates(const float fTimeStep);
	virtual bool HasFinished() const;
	virtual void Abort();

	void UpdateOrientationToTarget(const float fTimeStep);

	bool ChargeMissedTarget() const;
	bool ChargeHitEnvironment() const;
	bool ChargeWillFallOffLedge() const;

private:
	void SetUpCamShakeParams();
	void SendImpulseMessage();

	float m_fTimeSinceStart;
	float m_fChargeTurnRate;
	bool m_bFinished;
	bool m_bChargeCollide;
	bool m_bChargeCollideInjured;
	bool m_bChargeIntoIdle;
	bool m_bHitTarget;

	AVP_CameraShake_Parameters m_xCameraShakeParams;

	// Private typedef...
	typedef AVP_Move_Melee PARENT;
};   // End of class 'AVP_AlienQueen_Move_Melee_Charge_Loop'.


////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_AlienQueen_Move_Melee_Charge_Collide
// Author       : Andy McCabe
// Date         : 19/11/08
////////////////////////////////////////////////////////////////////////////////

class AVP_AlienQueen_Move_Melee_Charge_Collide : public AVP_Move_Melee
{
public:
	// Public functions...
	AVP_AlienQueen_Move_Melee_Charge_Collide(Asura_Movement_System& xParentSystem, Asura_Hash_ID uFirstPersonBlueprintHash, Asura_Hash_ID uThirdPersonBlueprintHash);
	//	virtual ~AVP_AlienQueen_Move_Melee_Charge_Collide();

	static Asura_Hash_ID GetStaticHash() { return 0xBCC2A9F4; } //Asura_GetHashID("AVP_AlienQueen_Move_Melee_Charge_Collide");

	bool RoarAfter() const { return m_bRoarAfter; }

protected:
	// Overridden functions...
	virtual bool ShouldStart() const;
	virtual bool Start( const Asura_Msg_Move* pxStartMsg );

private:

	bool m_bRoarAfter;
	
	// Private typedef...
	typedef AVP_Move_Melee PARENT;
};   // End of class 'AVP_AlienQueen_Move_Melee_Charge_Collide'.


////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_AlienQueen_Move_Melee_Charge_Collide_Injured
// Author       : Andy McCabe
// Date         : 19/11/08
////////////////////////////////////////////////////////////////////////////////

class AVP_AlienQueen_Move_Melee_Charge_Collide_Injured : public AVP_Move_Melee
{
public:
	// Public functions...
	AVP_AlienQueen_Move_Melee_Charge_Collide_Injured(Asura_Movement_System& xParentSystem, Asura_Hash_ID uFirstPersonBlueprintHash, Asura_Hash_ID uThirdPersonBlueprintHash);
	//	virtual ~AVP_AlienQueen_Move_Melee_Charge_Collide_Injured();

	static Asura_Hash_ID GetStaticHash() { return 0x5A032216; } //Asura_GetHashID("AVP_AlienQueen_Move_Melee_Charge_Collide_Injured");

protected:
	// Overridden functions...
	virtual bool ShouldStart() const;

private:
	// Private typedef...
	typedef AVP_Move_Melee PARENT;
};   // End of class 'AVP_AlienQueen_Move_Melee_Charge_Collide_Injured'.


////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_AlienQueen_Move_Melee_Charge_Into_Idle
// Author       : Andy McCabe
// Date         : 19/11/08
////////////////////////////////////////////////////////////////////////////////

class AVP_AlienQueen_Move_Melee_Charge_Into_Idle : public AVP_Move_Melee
{
public:
	// Public functions...
	AVP_AlienQueen_Move_Melee_Charge_Into_Idle(Asura_Movement_System& xParentSystem, Asura_Hash_ID uFirstPersonBlueprintHash, Asura_Hash_ID uThirdPersonBlueprintHash);
	//	virtual ~AVP_AlienQueen_Move_Melee_Charge_Into_Idle();

	static Asura_Hash_ID GetStaticHash() { return 0x543F3289; } //Asura_GetHashID("AVP_AlienQueen_Move_Melee_Charge_Into_Idle");

protected:
	// Overridden functions...
	virtual bool ShouldStart() const;

private:
	// Private typedef...
	typedef AVP_Move_Melee PARENT;
};   // End of class 'AVP_AlienQueen_Move_Melee_Charge_Into_Idle'.


#endif   // ifndef __AVP_ALIENQUEEN_MOVES_MELEE_H__


////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_AlienQueen_Move_Melee_ClawSwipe_Left_Start
// Author       : Andy McCabe
// Date         : 28/11/08
////////////////////////////////////////////////////////////////////////////////

class AVP_AlienQueen_Move_Melee_ClawSwipe_Left_Start : public AVP_Move_Melee
{
public:
	// Public functions...
	AVP_AlienQueen_Move_Melee_ClawSwipe_Left_Start(Asura_Movement_System& xParentSystem, Asura_Hash_ID uFirstPersonBlueprintHash, Asura_Hash_ID uThirdPersonBlueprintHash);
	//	virtual ~AVP_AlienQueen_Move_Melee_ClawSwipe_Left_Start();

	static Asura_Hash_ID GetStaticHash() { return 0x0DA8082F; } //Asura_GetHashID("AVP_AlienQueen_Move_Melee_ClawSwipe_Left_Start");

protected:
	// Overridden functions...
	virtual void Abort();

private:
	// Private typedef...
	typedef AVP_Move_Melee PARENT;
};   // End of class 'AVP_AlienQueen_Move_Melee_ClawSwipe_Left_Start'.


////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_AlienQueen_Move_Melee_ClawSwipe_Left_Loop_Start
// Author       : Andy McCabe
// Date         : 28/11/08
////////////////////////////////////////////////////////////////////////////////

class AVP_AlienQueen_Move_Melee_ClawSwipe_Left_Loop_Start : public AVP_Move_Melee
{
public:
	// Public functions...
	AVP_AlienQueen_Move_Melee_ClawSwipe_Left_Loop_Start(Asura_Movement_System& xParentSystem, Asura_Hash_ID uFirstPersonBlueprintHash, Asura_Hash_ID uThirdPersonBlueprintHash);
	//	virtual ~AVP_AlienQueen_Move_Melee_ClawSwipe_Left_Loop_Start();

	static Asura_Hash_ID GetStaticHash() { return 0xC631775A; } //Asura_GetHashID("AVP_AlienQueen_Move_Melee_ClawSwipe_Left_Loop_Start");

protected:
	// Overridden functions...

private:
	// Private typedef...
	typedef AVP_Move_Melee PARENT;
};   // End of class 'AVP_AlienQueen_Move_Melee_ClawSwipe_Left_Loop_Start'.


////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_AlienQueen_Move_Melee_ClawSwipe_Left_Return
// Author       : Andy McCabe
// Date         : 28/11/08
////////////////////////////////////////////////////////////////////////////////

class AVP_AlienQueen_Move_Melee_ClawSwipe_Left_Return : public AVP_Move_Melee
{
public:
	// Public functions...
	AVP_AlienQueen_Move_Melee_ClawSwipe_Left_Return(Asura_Movement_System& xParentSystem, Asura_Hash_ID uFirstPersonBlueprintHash, Asura_Hash_ID uThirdPersonBlueprintHash);
	//	virtual ~AVP_AlienQueen_Move_Melee_ClawSwipe_Left_Return();

	static Asura_Hash_ID GetStaticHash() { return 0xA4D97303; } //Asura_GetHashID("AVP_AlienQueen_Move_Melee_ClawSwipe_Left_Return");

protected:
	// Overridden functions...
	virtual	bool ShouldStart() const { return true; }
	virtual bool Start( const Asura_Msg_Move* pxStartMsg );

private:
	// Private typedef...
	typedef AVP_Move_Melee PARENT;
};   // End of class 'AVP_AlienQueen_Move_Melee_ClawSwipe_Left_Return'.


////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_AlienQueen_Move_Melee_ClawSwipe_Right_Start
// Author       : Andy McCabe
// Date         : 28/11/08
////////////////////////////////////////////////////////////////////////////////

class AVP_AlienQueen_Move_Melee_ClawSwipe_Right_Start : public AVP_Move_Melee
{
public:
	// Public functions...
	AVP_AlienQueen_Move_Melee_ClawSwipe_Right_Start(Asura_Movement_System& xParentSystem, Asura_Hash_ID uFirstPersonBlueprintHash, Asura_Hash_ID uThirdPersonBlueprintHash);
	//	virtual ~AVP_AlienQueen_Move_Melee_ClawSwipe_Right_Start();

	static Asura_Hash_ID GetStaticHash() { return 0xE04D831A; } //Asura_GetHashID("AVP_AlienQueen_Move_Melee_ClawSwipe_Right_Start");

protected:
	// Overridden functions...
	virtual void Abort();

private:
	// Private typedef...
	typedef AVP_Move_Melee PARENT;
};   // End of class 'AVP_AlienQueen_Move_Melee_ClawSwipe_Right_Start'.


////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_AlienQueen_Move_Melee_ClawSwipe_Right_Loop_Start
// Author       : Andy McCabe
// Date         : 28/11/08
////////////////////////////////////////////////////////////////////////////////

class AVP_AlienQueen_Move_Melee_ClawSwipe_Right_Loop_Start : public AVP_Move_Melee
{
public:
	// Public functions...
	AVP_AlienQueen_Move_Melee_ClawSwipe_Right_Loop_Start(Asura_Movement_System& xParentSystem, Asura_Hash_ID uFirstPersonBlueprintHash, Asura_Hash_ID uThirdPersonBlueprintHash);
	//	virtual ~AVP_AlienQueen_Move_Melee_ClawSwipe_Right_Loop_Start();

	static Asura_Hash_ID GetStaticHash() { return 0x92EB174F; } //Asura_GetHashID("AVP_AlienQueen_Move_Melee_ClawSwipe_Right_Loop_Start");

protected:
	// Overridden functions...

private:
	// Private typedef...
	typedef AVP_Move_Melee PARENT;
};   // End of class 'AVP_AlienQueen_Move_Melee_ClawSwipe_Right_Loop_Start'.


////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_AlienQueen_Move_Melee_ClawSwipe_Right_Return
// Author       : Andy McCabe
// Date         : 28/11/08
////////////////////////////////////////////////////////////////////////////////

class AVP_AlienQueen_Move_Melee_ClawSwipe_Right_Return : public AVP_Move_Melee
{
public:
	// Public functions...
	AVP_AlienQueen_Move_Melee_ClawSwipe_Right_Return(Asura_Movement_System& xParentSystem, Asura_Hash_ID uFirstPersonBlueprintHash, Asura_Hash_ID uThirdPersonBlueprintHash);
	//	virtual ~AVP_AlienQueen_Move_Melee_ClawSwipe_Right_Return();

	static Asura_Hash_ID GetStaticHash() { return 0x26E35578; } //Asura_GetHashID("AVP_AlienQueen_Move_Melee_ClawSwipe_Right_Return");

protected:
	// Overridden functions...
	virtual	bool ShouldStart() const { return true; }
	virtual bool Start( const Asura_Msg_Move* pxStartMsg );

private:
	// Private typedef...
	typedef AVP_Move_Melee PARENT;
};   // End of class 'AVP_AlienQueen_Move_Melee_ClawSwipe_Right_Return'.


////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_AlienQueen_Move_Melee_ClawSwipeKill_Start
// Author       : Andy McCabe
// Date         : 02/12/08
////////////////////////////////////////////////////////////////////////////////

class AVP_AlienQueen_Move_Melee_ClawSwipeKill_Start : public AVP_Move_Melee_GenericAttack_Start
{
public:
	AVP_AlienQueen_Move_Melee_ClawSwipeKill_Start(Asura_Movement_System& xParentSystem, Asura_Hash_ID uFirstPersonBlueprintHash, Asura_Hash_ID uThirdPersonBlueprintHash);

	static Asura_Hash_ID GetStaticHash() { return 0x9020731B; } // Asura_GetHashID("AVP_AlienQueen_Move_Melee_ClawSwipeKill_Start")); }

private:
	typedef AVP_Move_Melee_GenericAttack_Start PARENT;
};   // End of class 'AVP_AlienQueen_Move_Melee_ClawSwipeKill_Start'.


////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_AlienQueen_Move_Melee_ClawSwipeKill_Return_Hit
// Author       : Andy McCabe
// Date         : 02/12/08
////////////////////////////////////////////////////////////////////////////////

class AVP_AlienQueen_Move_Melee_ClawSwipeKill_Return_Hit : public AVP_Move_Melee_GenericAttack_Return_Hit
{
public:
	AVP_AlienQueen_Move_Melee_ClawSwipeKill_Return_Hit(Asura_Movement_System& xParentSystem, Asura_Hash_ID uFirstPersonBlueprintHash, Asura_Hash_ID uThirdPersonBlueprintHash);

	virtual bool ShouldStart() const;

	static Asura_Hash_ID GetStaticHash() { return 0xA23084EB; } // Asura_GetHashID("AVP_AlienQueen_Move_Melee_ClawSwipeKill_Return_Hit")); }

private:
	typedef AVP_Move_Melee_GenericAttack_Return_Hit PARENT;
};   // End of class 'AVP_AlienQueen_Move_Melee_ClawSwipeKill_Return_Hit'.


////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_AlienQueen_Move_Melee_ClawSwipeKill_Return_Countered
// Author       : Andy McCabe
// Date         : 02/12/08
////////////////////////////////////////////////////////////////////////////////

// Again, not sure this is needed, we may just miss.
class AVP_AlienQueen_Move_Melee_ClawSwipeKill_Return_Countered : public AVP_Move_Melee_GenericAttack_Return_Countered
{
public:
	AVP_AlienQueen_Move_Melee_ClawSwipeKill_Return_Countered(Asura_Movement_System& xParentSystem, Asura_Hash_ID uFirstPersonBlueprintHash, Asura_Hash_ID uThirdPersonBlueprintHash);

	static Asura_Hash_ID GetStaticHash() { return 0x053462F3; } // Asura_GetHashID("AVP_AlienQueen_Move_Melee_ClawSwipeKill_Return_Countered")); }

private:
	typedef AVP_Move_Melee_GenericAttack_Return_Countered PARENT;
};   // End of class 'AVP_AlienQueen_Move_Melee_ClawSwipeKill_Return_Countered'.


////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_AlienQueen_Move_Melee_ClawSwipeKill_Return_Miss
// Author       : Andy McCabe
// Date         : 02/12/08
////////////////////////////////////////////////////////////////////////////////

class AVP_AlienQueen_Move_Melee_ClawSwipeKill_Return_Miss : public AVP_Move_Melee_GenericAttack_Return_Miss
{
public:
	AVP_AlienQueen_Move_Melee_ClawSwipeKill_Return_Miss(Asura_Movement_System& xParentSystem, Asura_Hash_ID uFirstPersonBlueprintHash, Asura_Hash_ID uThirdPersonBlueprintHash);

	static Asura_Hash_ID GetStaticHash() { return 0xA3E25EA4; } // Asura_GetHashID("AVP_AlienQueen_Move_Melee_ClawSwipeKill_Return_Miss")); }

private:
	typedef AVP_Move_Melee_GenericAttack_Return_Miss PARENT;
};   // End of class 'AVP_AlienQueen_Move_Melee_ClawSwipeKill_Return_Miss'.


////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_AlienQueen_Move_Melee_ClawSwipeKill_Grapple
// Author       : Andy McCabe
// Date         : 02/12/08
////////////////////////////////////////////////////////////////////////////////

class AVP_AlienQueen_Move_Melee_ClawSwipeKill_Grapple : public AVP_Move_Melee_GenericPuppeteer_Start
{
public:
	AVP_AlienQueen_Move_Melee_ClawSwipeKill_Grapple(Asura_Movement_System& xParentSystem);

	static Asura_Hash_ID GetStaticHash() { return 0xE5057934; } //"AVP_AlienQueen_FP_Move_Melee_MarineGrapple_Start"

	virtual bool ShouldStart() const; 

private:
	typedef AVP_Move_Melee_GenericPuppeteer_Start PARENT;
};

////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_AlienQueen_Move_Spit_Acid
// Author       : Andy McCabe
// Date         : 07/01/09
////////////////////////////////////////////////////////////////////////////////

class AVP_AlienQueen_Move_Spit_Acid : public AVP_Move
{
public:
	AVP_AlienQueen_Move_Spit_Acid(Asura_Movement_System& xParentSystem);

	static Asura_Hash_ID GetStaticHash() { return 0xB2271DFE; } // Asura_GetHashID("AVP_AlienQueen_Move_Spit_Acid")); }

protected:
	virtual bool Start(const Asura_Msg_Move* pxStartMsg);

private:
	typedef AVP_Move PARENT;
};   

////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_AlienQueen_Move_Melee_Stationary_Roar
// Author       : Callum Hope
// Date         : 26/3/09
////////////////////////////////////////////////////////////////////////////////

class AVP_AlienQueen_Move_Melee_Stationary_Roar : public AVP_Move_Melee
{
	public:
		AVP_AlienQueen_Move_Melee_Stationary_Roar( Asura_Movement_System& xParentSystem, Asura_Hash_ID uFirstPersonBlueprintHash, Asura_Hash_ID uThirdPersonBlueprintHash );

		static Asura_Hash_ID GetStaticHash() { return 0x6828AFB8; } //Asura_GetHashID("AVP_AlienQueen_Move_Melee_Stationary_Roar");

		virtual bool ShouldStart() const;

	protected:
		virtual void Abort();
		virtual bool Start(const Asura_Msg_Move* pxStartMsg);

	private:
		void SetUpCamShakeParams();

		AVP_CameraShake_Parameters m_xCameraShakeParams;
		
		typedef AVP_Move_Melee PARENT;
};   

////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_AlienQueen_Move_Stunned_Stumble
// Author       : Callum Hope
// Date         : 26/3/09
////////////////////////////////////////////////////////////////////////////////

class AVP_AlienQueen_Move_Stunned_Stumble : public AVP_Move
{
public:
	// Public functions...
	AVP_AlienQueen_Move_Stunned_Stumble( Asura_Movement_System& xParentSystem );

	static Asura_Hash_ID GetStaticHash() { return 0x47961080; } //Asura_GetHashID("AVP_AlienQueen_Move_Stunned_Stumble");

protected:

private:
	// Private typedef...
	typedef AVP_Move PARENT;
};  

/// End of file. ///
