
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_FaceHugger_FP_MovesMelee.cpp
// Author : Anthony Gregson
// Date   : 22/02/08
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"


// This file's header...
#include "Moves/Vertebrates/FaceHugger/AVP_FaceHugger_MovesMelee.h"

// Asura independent header-files...
#include "Animation/Asura_Hierarchy.h"
#include "Blueprint/Asura_Blueprint.h"
#include "Blueprint/Asura_Blueprint_System.h"
#include "Network/Asura_Message.h"

// Asura platform header-files...

// Game independent header-files...
#include "Actor/AVP_Actor_PhysicsObjectHandler.h"
#include "Actor/Vertebrates/AVP_Vertebrate_Description.h"
#include "Blueprint/AVP_Blueprint_HashIDs.h"
#include "Core/AVP_Settings.h"
#include "Entity/AVP_Messages.h"
#include "Entity/PhysicalObject/Actor/AVP_ServerEntity_Actor.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/AVP_Vertebrate_Messages.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/FaceHugger/AVP_ClientActor_FaceHugger.h"
#include "Moves/AVP_Moves.h"

// Game platform header-files...

// Tools header-files...

// constants 
DEBUGVAR float dbg_fPounceOrientationInterpolationTime = 0.3f;
DEBUGVAR Asura_Hash_ID dbg_uDeathHug		= 0x1ee47e06; //"DeathHug"


////////////////////////////////////////////////////////////////////////////////
// Class		: AVP_FaceHugger_FP_Move_Melee_JumpAttack_Start
////////////////////////////////////////////////////////////////////////////////

AVP_FaceHugger_Move_Melee_JumpAttack_Start::AVP_FaceHugger_Move_Melee_JumpAttack_Start(Asura_Movement_System& xParentSystem, Asura_Hash_ID uFirstPersonBlueprintHash, Asura_Hash_ID uThirdPersonBlueprintHash)
: PARENT(xParentSystem, GetStaticHash(),
		 uAVP_BLUEPRINT_TYPE_MELEE, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash, "DefaultAttackStart",
		 AVP_FaceHugger_FP_Move_Melee_JumpAttack_Return_Hit::GetStaticHash(), AVP_FaceHugger_FP_Move_Melee_JumpAttack_Return_Countered::GetStaticHash(), AVP_FaceHugger_FP_Move_Melee_JumpAttack_Return_Miss::GetStaticHash())
{
	AddNextMove(AVP_FaceHugger_FP_Move_Melee_PredatorGrapple_Start::GetStaticHash());
	AddNextMove(AVP_FaceHugger_FP_Move_Melee_MarineGrapple_Start::GetStaticHash());
	AddNextMove(AVP_FaceHugger_TP_Move_Melee_PredatorGrapple::GetStaticHash());
	AddNextMove(AVP_FaceHugger_TP_Move_Melee_MarineGrapple::GetStaticHash());

	m_bClearQueuedMeleeRequests = true;
	// Remove the default generic start aiming behaviour
	SetAimDirectionUsage(AVP_ClientEntity_Actor::AIM_DIRECTION_NOT_USED);
	SetDisableSurfaceReorientation(true);
}

bool AVP_FaceHugger_Move_Melee_JumpAttack_Start::Start(const Asura_Msg_Move* pxStartMsg)
{
	// There's not really much we can do without the message in this case, as we need a target position.
	if(!pxStartMsg) { return false; }

	const AVP_Msg_Melee_Move* pxMeleeStart = static_cast<const AVP_Msg_Melee_Move*>(pxStartMsg);
	const AVP_ClientEntity_PhysicalObject* pxTarget = AVP_ClientEntity_PhysicalObject::FindEntity(pxMeleeStart->GetTargetGuid());
	if (!pxTarget) { return false; }

	const AVP_ClientActor_FaceHugger& xFaceHuggerOwner = static_cast<AVP_ClientActor_FaceHugger&>(GetAVPParentSystem().GetActorOwner());

	SetVariation(pxMeleeStart->GetVariation());

	Asura_Vector_3 xTargetPosition = pxTarget->GetCurrentPosition();
	Asura_Vector_3 xDirection = xTargetPosition - xFaceHuggerOwner.GetCurrentPosition();
	xDirection.y = 0.0f;
	Asura_Matrix_3x3 xTargetOri;

	if (xDirection.ApproximatelyEquals(Asura_Maths::ZeroVector3))
	{
		Asura_Euler_3 xEuler;
		xEuler.FromMatrix(xFaceHuggerOwner.GetCurrentOrientation());
		xEuler.x = xEuler.z = 0.0f;
		xEuler.ToMatrix(xTargetOri);
	}
	else
	{
		xDirection.Normalise();
		Asura_Maths::OrientationFromDirection(xTargetOri, xDirection);
	}

	m_xPositionInterpolationData.SetFinal(xTargetPosition - xDirection * 1.0f);
	m_xOrientationInterpolationData.SetFinal(xTargetOri);

	if (m_pxAVPAnimationData)
	{
		// We need to ensure the correct anim hash is set up, as it's about to get used.
		if (xFaceHuggerOwner.IsUsingFirstPersonSkin())
		{
			m_pxAVPAnimationData->SetAnimHash(m_uFirstPersonAnimHash);
		}
		else
		{
			m_pxAVPAnimationData->SetAnimHash(m_uThirdPersonAnimHash);
		}
	}
	bool bHasAnimation = m_pxAVPAnimationData && m_pxAVPAnimationData->HasValidAnimation();
	float fDuration = bHasAnimation ? m_pxAVPAnimationData->GetAnimationDuration() : 1.000f;

	if(bHasAnimation)
	{
		m_xPositionInterpolationData.SetMovementOffsetOperation(Asura_MoveData_PositionInterpolation::STRETCH_MOVEMENT_OFFSET);
	}

	m_xPositionInterpolationData.SetDuration(fDuration);
	m_xOrientationInterpolationData.SetDuration(dbg_fPounceOrientationInterpolationTime);

	InitialisePositionInterpolation(m_xPositionInterpolationData);
	InitialiseOrientationInterpolation(m_xOrientationInterpolationData);


	return PARENT::Start(pxStartMsg);
}

////////////////////////////////////////////////////////////////////////////////
// Class	    : AVP_FaceHugger_FP_Move_Melee_JumpAttack_Return_Hit
////////////////////////////////////////////////////////////////////////////////

AVP_FaceHugger_FP_Move_Melee_JumpAttack_Return_Hit::AVP_FaceHugger_FP_Move_Melee_JumpAttack_Return_Hit(Asura_Movement_System& xParentSystem, Asura_Hash_ID uFirstPersonBlueprintHash, Asura_Hash_ID uThirdPersonBlueprintHash)
: PARENT(xParentSystem, GetStaticHash(),
		 uAVP_BLUEPRINT_TYPE_MELEE, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash, "DefaultAttackReturnHit")
{
}


bool AVP_FaceHugger_FP_Move_Melee_JumpAttack_Return_Hit::ShouldStart() const
{
	// If the parent says we can start, check that the entity is of a type that we don't attach on to.
	if (!PARENT::ShouldStart()) { return false; }

	// There's only one move we can have come from.
	const AVP_FaceHugger_Move_Melee_JumpAttack_Start* pxCurrentMove = static_cast<const AVP_FaceHugger_Move_Melee_JumpAttack_Start*>(GetParentSystem().GetCurrentMove());
	if (!pxCurrentMove) { return false; }

	const AVP_ClientEntity_Actor* pxVictim = AVP_ClientEntity_Actor::FindEntity(pxCurrentMove->GetHitEntity());
	// TODO - These next two checks take the safe option of using this move if our victim has gone missing or died, but we should possibly be going into miss in that case.
	if (!pxVictim) { return true; }
	if (pxVictim->IsDead()) { return true; }

	switch (pxVictim->GetClassification())
	{
	default:
		return true;

		// These entity classifications are attached to, so don't start this move.
	case AVP_ENTITYCLASS_MARINE:
	case AVP_ENTITYCLASS_CIVILIAN:
	case AVP_ENTITYCLASS_PREDATOR:
		return false;
	}

}


////////////////////////////////////////////////////////////////////////////////
// Class		: AVP_FaceHugger_FP_Move_Melee_JumpAttack_Return_Miss
////////////////////////////////////////////////////////////////////////////////

AVP_FaceHugger_FP_Move_Melee_JumpAttack_Return_Miss::AVP_FaceHugger_FP_Move_Melee_JumpAttack_Return_Miss(Asura_Movement_System& xParentSystem, Asura_Hash_ID uFirstPersonBlueprintHash, Asura_Hash_ID uThirdPersonBlueprintHash)
: PARENT(xParentSystem, GetStaticHash(),
		 uAVP_BLUEPRINT_TYPE_MELEE, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash, "DefaultAttackReturnMiss")
{
	SetAimDirectionUsage(AVP_ClientEntity_Actor::AIM_DIRECTION_RESET);
}


////////////////////////////////////////////////////////////////////////////////
// Class		: AVP_FaceHugger_FP_Move_Melee_JumpAttack_Return_Countered
////////////////////////////////////////////////////////////////////////////////

AVP_FaceHugger_FP_Move_Melee_JumpAttack_Return_Countered::AVP_FaceHugger_FP_Move_Melee_JumpAttack_Return_Countered(Asura_Movement_System& xParentSystem, Asura_Hash_ID uFirstPersonBlueprintHash, Asura_Hash_ID uThirdPersonBlueprintHash)
: PARENT(xParentSystem, GetStaticHash(),
		 uAVP_BLUEPRINT_TYPE_MELEE, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash, "DefaultAttackReturnCountered")
{
}


////////////////////////////////////////////////////////////////////////////////
// Class		: AVP_FaceHugger_FP_Move_Melee_PredatorGrapple_Start
////////////////////////////////////////////////////////////////////////////////

AVP_FaceHugger_FP_Move_Melee_PredatorGrapple_Start::AVP_FaceHugger_FP_Move_Melee_PredatorGrapple_Start(Asura_Movement_System& xParentSystem)
: PARENT(xParentSystem, GetStaticHash(),
		 AVP_Settings::AVP_SPECIES_ALIEN, AVP_Settings::AVP_SPECIES_PREDATOR, uAVP_BLUEPRINT_NAME_FACEHUGGER_PREDATORVICTIM,
		 AVP_FaceHugger_FP_Move_Melee_PredatorGrapple_Kill::GetStaticHash(), AVP_FaceHugger_FP_Move_Melee_PredatorGrapple_Countered::GetStaticHash())
{
}

bool AVP_FaceHugger_FP_Move_Melee_PredatorGrapple_Start::ShouldStart() const
{
	// There's only one move we can have come from.
	const AVP_FaceHugger_Move_Melee_JumpAttack_Start* pxCurrentMove = static_cast<const AVP_FaceHugger_Move_Melee_JumpAttack_Start*>(GetParentSystem().GetCurrentMove());
	if (!pxCurrentMove) { return false; }

	const AVP_ClientEntity_Actor* pxVictim = AVP_ClientEntity_Actor::FindEntity(pxCurrentMove->GetHitEntity());
	// TODO - These next two checks take the safe option of using this move if our victim has gone missing or died, but we should possibly be going into miss in that case.
	if (!pxVictim) { return false; }
	if (pxVictim->IsDead()) { return false; }

	// not a player should use different move
	if (!pxVictim->IsPlayer())
	{
		return false;
	}

	return PARENT::ShouldStart();
}

////////////////////////////////////////////////////////////////////////////////
// Class		: AVP_FaceHugger_FP_Move_Melee_PredatorGrapple_Kill
////////////////////////////////////////////////////////////////////////////////

AVP_FaceHugger_FP_Move_Melee_PredatorGrapple_Kill::AVP_FaceHugger_FP_Move_Melee_PredatorGrapple_Kill(Asura_Movement_System& xParentSystem)
: PARENT(xParentSystem, GetStaticHash(),
		 AVP_Settings::AVP_SPECIES_ALIEN, AVP_Settings::AVP_SPECIES_PREDATOR, uAVP_BLUEPRINT_NAME_FACEHUGGER_PREDATORVICTIM)
{
	m_bAlwaysFatal = true;
}


////////////////////////////////////////////////////////////////////////////////
// Class		: AVP_FaceHugger_FP_Move_Melee_PredatorGrapple_Countered
////////////////////////////////////////////////////////////////////////////////

AVP_FaceHugger_FP_Move_Melee_PredatorGrapple_Countered::AVP_FaceHugger_FP_Move_Melee_PredatorGrapple_Countered(Asura_Movement_System& xParentSystem)
: PARENT(xParentSystem, GetStaticHash(),
		 AVP_Settings::AVP_SPECIES_ALIEN, AVP_Settings::AVP_SPECIES_PREDATOR, uAVP_BLUEPRINT_NAME_FACEHUGGER_PREDATORVICTIM)
{
}

////////////////////////////////////////////////////////////////////////////////
// Class		: AVP_FaceHugger_FP_Move_Melee_MarineGrapple_Start
////////////////////////////////////////////////////////////////////////////////

AVP_FaceHugger_FP_Move_Melee_MarineGrapple_Start::AVP_FaceHugger_FP_Move_Melee_MarineGrapple_Start(Asura_Movement_System& xParentSystem)
: PARENT(xParentSystem, GetStaticHash(),
		 AVP_Settings::AVP_SPECIES_ALIEN, AVP_Settings::AVP_SPECIES_HUMAN, uAVP_BLUEPRINT_NAME_FACEHUGGER_MARINEVICTIM,
		 AVP_FaceHugger_FP_Move_Melee_MarineGrapple_Kill::GetStaticHash(), AVP_FaceHugger_FP_Move_Melee_MarineGrapple_Countered::GetStaticHash())
{
}

bool AVP_FaceHugger_FP_Move_Melee_MarineGrapple_Start::ShouldStart() const
{
	// There's only one move we can have come from.
	const AVP_FaceHugger_Move_Melee_JumpAttack_Start* pxCurrentMove = static_cast<const AVP_FaceHugger_Move_Melee_JumpAttack_Start*>(GetParentSystem().GetCurrentMove());
	if (!pxCurrentMove) { return false; }

	const AVP_ClientEntity_Actor* pxVictim = AVP_ClientEntity_Actor::FindEntity(pxCurrentMove->GetHitEntity());
	// TODO - These next two checks take the safe option of using this move if our victim has gone missing or died, but we should possibly be going into miss in that case.
	if (!pxVictim) { return false; }
	if (pxVictim->IsDead()) { return false; }

	// not a player should use different move
	if (!pxVictim->IsPlayer())
	{
		return false;
	}

	return PARENT::ShouldStart();
}


////////////////////////////////////////////////////////////////////////////////
// Class		: AVP_FaceHugger_FP_Move_Melee_MarineGrapple_Kill
////////////////////////////////////////////////////////////////////////////////

AVP_FaceHugger_FP_Move_Melee_MarineGrapple_Kill::AVP_FaceHugger_FP_Move_Melee_MarineGrapple_Kill(Asura_Movement_System& xParentSystem)
: PARENT(xParentSystem, GetStaticHash(),
		 AVP_Settings::AVP_SPECIES_ALIEN, AVP_Settings::AVP_SPECIES_HUMAN, uAVP_BLUEPRINT_NAME_FACEHUGGER_MARINEVICTIM)
{
	m_bAlwaysFatal = true;
}


////////////////////////////////////////////////////////////////////////////////
// Class		: AVP_FaceHugger_FP_Move_Melee_MarineGrapple_Countered
////////////////////////////////////////////////////////////////////////////////

AVP_FaceHugger_FP_Move_Melee_MarineGrapple_Countered::AVP_FaceHugger_FP_Move_Melee_MarineGrapple_Countered(Asura_Movement_System& xParentSystem)
: PARENT(xParentSystem, GetStaticHash(),
		 AVP_Settings::AVP_SPECIES_ALIEN, AVP_Settings::AVP_SPECIES_HUMAN, uAVP_BLUEPRINT_NAME_FACEHUGGER_MARINEVICTIM)
{
}


////////////////////////////////////////////////////////////////////////////////
// Class		: AVP_FaceHugger_TP_Move_Melee_MarineGrapple
////////////////////////////////////////////////////////////////////////////////
AVP_FaceHugger_TP_Move_Melee_MarineGrapple::AVP_FaceHugger_TP_Move_Melee_MarineGrapple(Asura_Movement_System& xParentSystem)
: PARENT(xParentSystem, GetStaticHash(),
		 AVP_Settings::AVP_SPECIES_ALIEN, AVP_Settings::AVP_SPECIES_HUMAN, uAVP_BLUEPRINT_NAME_FACEHUGGER_MARINEVICTIM_3RD)
{
	m_bAlwaysFatal = true;

	Asura_MoveData_Animation* pxAnimData = GetAVPAnimationData();
	if (pxAnimData)
	{
		pxAnimData->SetTweenData(pxAnimData->GetTweenInDuration(), 0.0f, true);
	}

	AddNextMove(AVP_FaceHugger_TP_Move_Melee_MarineDeath::GetStaticHash());


}

bool AVP_FaceHugger_TP_Move_Melee_MarineGrapple::ShouldStart() const
{
	// There's only one move we can have come from.
	const AVP_FaceHugger_Move_Melee_JumpAttack_Start* pxCurrentMove = static_cast<const AVP_FaceHugger_Move_Melee_JumpAttack_Start*>(GetParentSystem().GetCurrentMove());
	if (!pxCurrentMove) { return false; }

	const AVP_ClientEntity_Actor* pxVictim = AVP_ClientEntity_Actor::FindEntity(pxCurrentMove->GetHitEntity());
	// TODO - These next two checks take the safe option of using this move if our victim has gone missing or died, but we should possibly be going into miss in that case.
	if (!pxVictim) { return false; }
	if (pxVictim->IsDead()) { return false; }

	// a player should use different move
	if (pxVictim->IsPlayer())
	{
		return false;
	}

	return PARENT::ShouldStart();
}


////////////////////////////////////////////////////////////////////////////////
// Class		: AVP_FaceHugger_TP_Move_Melee_MarineDeath
////////////////////////////////////////////////////////////////////////////////

AVP_FaceHugger_TP_Move_Melee_MarineDeath::AVP_FaceHugger_TP_Move_Melee_MarineDeath(Asura_Movement_System& xParentSystem)
: PARENT(xParentSystem, GetStaticHash())
{
	SetPreventsFlinch( true );
	SetAimDirectionUsage( AVP_ClientEntity_Actor::AIM_DIRECTION_RESET );
	SetPositionUsage(Asura_Move::POSITION_NOT_USED);
	SetOrientationUsage(Asura_Move::ORIENTATION_NOT_USED);
	SetLoopAtEnd(true);

	Asura_Hash_ID uTPAnim = ASURA_HASH_ID_UNSET;
	Asura_Value xValue;

	Asura_Blueprint* pxBP = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_GRAPPLEALIEN, uAVP_BLUEPRINT_NAME_FACEHUGGER_MARINEVICTIM_3RD );
	if ( pxBP )
	{
		if ( pxBP->GetParameterValueByHashID( dbg_uDeathHug, xValue ) )
		{
			uTPAnim = xValue.GetHashIDValue();
		}
	}

	SetAnimHashes( ASURA_HASH_ID_UNSET, uTPAnim );

	Asura_MoveData_Animation* pxAnimData = GetAVPAnimationData();
	if (pxAnimData)
	{
		pxAnimData->SetTweenData(0.0f, 0.0f, true);
	}

}
bool AVP_FaceHugger_TP_Move_Melee_MarineDeath::Start(const Asura_Msg_Move* pxStartMsg)
{
	return PARENT::Start(pxStartMsg);
}


////////////////////////////////////////////////////////////////////////////////
// Class		: AVP_FaceHugger_TP_Move_Melee_PredatorGrapple
////////////////////////////////////////////////////////////////////////////////
AVP_FaceHugger_TP_Move_Melee_PredatorGrapple::AVP_FaceHugger_TP_Move_Melee_PredatorGrapple(Asura_Movement_System& xParentSystem)
: PARENT(xParentSystem, GetStaticHash(),
		 AVP_Settings::AVP_SPECIES_ALIEN, AVP_Settings::AVP_SPECIES_PREDATOR, uAVP_BLUEPRINT_NAME_FACEHUGGER_PREDATORVICTIM_3RD)
{
	m_bAlwaysFatal = true;
}


bool AVP_FaceHugger_TP_Move_Melee_PredatorGrapple::ShouldStart() const
{
	// There's only one move we can have come from.
	const AVP_Move_Melee* pxCurrentMove = static_cast<const AVP_Move_Melee*>(GetParentSystem().GetCurrentMove());
	if (!pxCurrentMove) { return false; }

	const AVP_ClientEntity_Actor* pxVictim = AVP_ClientEntity_Actor::FindEntity(pxCurrentMove->GetHitEntity());
	// TODO - These next two checks take the safe option of using this move if our victim has gone missing or died, but we should possibly be going into miss in that case.
	if (!pxVictim) { return false; }
	if (pxVictim->IsDead()) { return false; }

	// a player should use different move
	if (pxVictim->IsPlayer())
	{
		return false;
	}

	return PARENT::ShouldStart();
}


////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_FaceHugger_TP_Move_Melee_MarineDeathHug
////////////////////////////////////////////////////////////////////////////////

AVP_FaceHugger_TP_Move_Melee_MarineDeathHug::AVP_FaceHugger_TP_Move_Melee_MarineDeathHug(Asura_Movement_System& xParentSystem, Asura_Hash_ID uFirstPersonBlueprintHash, Asura_Hash_ID uThirdPersonBlueprintHash)
: PARENT(xParentSystem, GetStaticHash())
{
	Asura_Hash_ID uTPAnim = ASURA_HASH_ID_UNSET;
	Asura_Value xValue;

	Asura_Blueprint* pxBP = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_GRAPPLEALIEN, uAVP_BLUEPRINT_NAME_FACEHUGGER_MARINEVICTIM_3RD );
	if ( pxBP )
	{
		if ( pxBP->GetParameterValueByHashID( dbg_uDeathHug, xValue ) )
		{
			uTPAnim = xValue.GetHashIDValue();
		}
	}

	SetAnimHashes( ASURA_HASH_ID_UNSET, uTPAnim );

	Asura_MoveData_Animation* pxAnimData = GetAVPAnimationData();
	if (pxAnimData)
	{
		pxAnimData->SetTweenData(0.0f, 0.0f, true);
		pxAnimData->SetLeaveAnimAtEnd(true);
	}
}


//////////////////////////////////////////////////////////////////////////////
//
//////////////////////////////////////////////////////////////////////////////
