//----------------------------------//----------------------------------//----------------------------------//----------------------------------
//
// File   : AVP_Human_MovesCover.cpp
// Author : Gareth Thomas
// Date   : 26 / 06 / 2008
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
//----------------------------------//----------------------------------//----------------------------------//----------------------------------

//----------------------------------//----------------------------------//----------------------------------//----------------------------------
// Include files...
//----------------------------------//----------------------------------//----------------------------------//----------------------------------
// Core
#include "Core/Asura.h"

// This Files Header
#include "Moves/Vertebrates/Human/AVP_Human_MovesCover.h"

// Asura
#include "Blueprint/Asura_Blueprint_System.h"
#include "Blueprint/AVP_Blueprint_HashIDs.h"
#include "Moves/AVP_MoveData_Animation.h"

// AVP
#include "Actor/Vertebrates/AVP_Vertebrate_AnimationHandler.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Human/AVP_ClientActor_Human.h"
#include "Moves/AVP_Moves.h"

//----------------------------------//----------------------------------//----------------------------------//----------------------------------
bool AVP_Move_Human_EnterCover::Start(const Asura_Msg_Move* pxStartMsg)
{
	if ( pxStartMsg == NULL )
	{
		Asura_Assert( false, "Human enter cover move can only be triggered by message." );
		return(false);
	}

	const AVP_ClientEntity_Actor& xOwner =	GetAVPParentSystem().GetActorOwner();

	const AVP_Message_Human_EnterCover* pxMessage = static_cast< const AVP_Message_Human_EnterCover* >( pxStartMsg );

	Asura_Vector_3 xTargetPosition;
	pxMessage->GetDestinationPosition( xTargetPosition );

	Asura_Matrix_3x3 xTargetOrientation;
	pxMessage->GetDestinationOrientation( xTargetOrientation );

	const Asura_Blueprint* pxBP = Asura_Blueprint_System::GetBlueprintByHashID( xOwner.GetAnimStateBlueprintType(), pxMessage->GetAnimationHash() );
	Asura_Assertf( pxBP, ( "Unable to get blend template with hash ID 0x%08X specified in move start message.", pxMessage->GetAnimationHash() ) );
	if ( pxBP )
	{
		SetAnimBlendTemplateBlueprint( *pxBP );
		if ( m_pxAVPAnimationData && m_bUsingBlendTemplate )
		{
			Asura_AVector3 xMoveDir;
			xOwner.GetMovementDirection( xMoveDir );
			Asura_Vector_3 xMoveDir_3( xMoveDir );
			xMoveDir_3 = xOwner.GetCurrentOrientation().ApplyTranspose( xMoveDir_3 );
			static_cast< AVP_MoveData_BlendedAnimation* >( m_pxAVPAnimationData )->SetBlendAngle( Asura_Maths::ArcTangent( xMoveDir_3.GetX(), xMoveDir_3.GetZ() ) );
		}
	}

	const bool bHasAnimation = m_pxAVPAnimationData && m_pxAVPAnimationData->HasValidAnimation();
	const float fDuration = (bHasAnimation) ? m_pxAVPAnimationData->GetAnimationDuration() : 0.500f;

	// For the time being we'll just move and rotate to our desired position...
	if ( bHasAnimation )
	{
		m_xPositionInterpolationData.SetMovementOffsetOperation( Asura_MoveData_PositionInterpolation::STRETCH_MOVEMENT_OFFSET );
		m_xOrientationInterpolationData.SetMovementOffsetOperation( Asura_MoveData_OrientationInterpolation::IGNORE_MOVEMENT_OFFSET );
		m_pxAVPAnimationData->SetAnimPriority( 1000 );
	}

	m_xPositionInterpolationData.SetDuration( fDuration );
	m_xPositionInterpolationData.SetFinal( xTargetPosition );

	m_xOrientationInterpolationData.SetDuration( fDuration );
	m_xOrientationInterpolationData.SetFinal( xTargetOrientation );

	InitialisePositionInterpolation( m_xPositionInterpolationData );
	InitialiseOrientationInterpolation( m_xOrientationInterpolationData );

	// Finally, call the base-function...
	return(PARENT::Start(pxStartMsg));
}

//----------------------------------//----------------------------------//----------------------------------//----------------------------------
AVP_Move_Human_IntoPeekEmerge::AVP_Move_Human_IntoPeekEmerge( Asura_Movement_System& xParentSystem )
: PARENT( xParentSystem, GetStaticHash() ) 
{
	AddNextMove( AVP_Move_Human_PeekEmerge::GetStaticHash() );
}

//----------------------------------//----------------------------------//----------------------------------//----------------------------------
bool AVP_Move_Human_IntoPeekEmerge::Start( const Asura_Msg_Move* pxStartMsg )
{
	if ( pxStartMsg == NULL )
	{
		Asura_Assert( false, "Human Into Peek Emerge move can only be triggered by message." );
		return(false);
	}

	const AVP_Message_Human_PeekEmerge* pxMessage = static_cast< const AVP_Message_Human_PeekEmerge* >( pxStartMsg );
	Asura_Hash_ID uThirdPersonAnim = pxMessage->GetIntoPeekAnimationHash();
	SetAnimHashes( ASURA_HASH_ID_UNSET, uThirdPersonAnim );

	// Set the next moves anim hash
	SetNextMoveAnimHash( pxMessage->GetPeekAnimationHash() );

	if ( m_pxAVPAnimationData )
	{
		m_pxAVPAnimationData->SetAnimHash( m_uThirdPersonAnimHash );
		m_pxAVPAnimationData->SetLeaveAnimAtEnd( true );
	}

	// Finally, call the base-function...
	return( PARENT::Start( pxStartMsg ) );
}

//----------------------------------//----------------------------------//----------------------------------//----------------------------------
AVP_Move_Human_PeekEmerge::AVP_Move_Human_PeekEmerge( Asura_Movement_System& xParentSystem ) 
: PARENT( xParentSystem, GetStaticHash() ) 
{
}

//----------------------------------//----------------------------------//----------------------------------//----------------------------------
bool AVP_Move_Human_PeekEmerge::Start( const Asura_Msg_Move* pxStartMsg )
{
	AVP_ClientActor_Human&			xHumanOwner		= static_cast< AVP_ClientActor_Human& >( GetAVPParentSystem().GetActorOwner() );
	AVP_Move_Human_IntoPeekEmerge*	pxCurrentMove	= ( AVP_Move_Human_IntoPeekEmerge* )xHumanOwner.GetMovementSystem()->GetCurrentMove();

	if ( pxCurrentMove )
	{
		Asura_Hash_ID uThirdPersonAnim = pxCurrentMove->GetNextMoveAnimHash();

		SetAnimHashes( ASURA_HASH_ID_UNSET, uThirdPersonAnim );

		if (m_pxAVPAnimationData)
		{

			m_pxAVPAnimationData->SetTweenData( 0.0f, 0.0f, false );
			m_pxAVPAnimationData->SetAnimHash( m_uThirdPersonAnimHash );
//			m_pxAVPAnimationData->SetAnimPriority( AVP_AAPC_STATES_GROUP );
			m_pxAVPAnimationData->SetLeaveAnimAtEnd( true );
		}
	}

	// Finally, call the base-function...
	return( PARENT::Start( pxStartMsg ) );
}

//----------------------------------//----------------------------------//----------------------------------//----------------------------------
AVP_Move_Human_StepOutOfCover::AVP_Move_Human_StepOutOfCover( Asura_Movement_System& xParentSystem ) 
: PARENT( xParentSystem, GetStaticHash() ) 
{
}

//----------------------------------//----------------------------------//----------------------------------//----------------------------------
bool AVP_Move_Human_StepOutOfCover::Start( const Asura_Msg_Move* pxStartMsg )
{
 	AVP_ClientActor_Human& xHumanOwner = static_cast< AVP_ClientActor_Human& >( GetAVPParentSystem().GetActorOwner() );

	if ( pxStartMsg == NULL )
	{
		Asura_Assert( false, "Human Into Peek Emerge move can only be triggered by message." );
		return(false);
	}

	SetPositionUsage( POSITION_SET_DIRECTLY );

	const AVP_Message_Human_StepOutOfCover* pxMessage = static_cast< const AVP_Message_Human_StepOutOfCover* >( pxStartMsg );
	Asura_Hash_ID uThirdPersonAnim = pxMessage->GetStepOutAnimHash();
	SetAnimHashes( ASURA_HASH_ID_UNSET, uThirdPersonAnim );

	if (m_pxAVPAnimationData)
	{
		m_pxAVPAnimationData->SetTweenData( 0.0f, 0.0f, false );
		m_pxAVPAnimationData->SetAnimHash(m_uThirdPersonAnimHash);
//		m_pxAVPAnimationData->SetAnimPriority( AVP_AAPC_STATES_GROUP );

		m_xPositionInterpolationData.SetMovementOffsetOperation( Asura_MoveData_PositionInterpolation::ADD_MOVEMENT_OFFSET );
		m_xPositionInterpolationData.SetFinal( xHumanOwner.GetCurrentPosition() );
		m_xPositionInterpolationData.SetDuration( 0.0f );

		InitialisePositionInterpolation( m_xPositionInterpolationData );
//		m_pxAVPAnimationData->SetLeaveAnimAtEnd( true );
	}

	// Finally, call the base-function...
	return( PARENT::Start( pxStartMsg ) );
}
