////////////////////////////////////////////////////////////////////////////////
// Class		: AVP_Human_Move_Interaction_PlugBattery
// Author       : Ali Jakamy
// Date         : 23/10/08
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "Moves/Vertebrates/Human/AVP_Human_MovesInteraction.h"

// AVP
#include "Entity/PhysicalObject/Actor/AVP_Actor_Messages.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Human/AVP_ServerActor_Human.h"
#include "Entity/PhysicalObject/Interact/ToolReceptacle/BatteryReceptacle/AVP_ClientEntity_InteractBatteryReceptacle.h"
#include "Entity/PhysicalObject/Interact/ToolReceptacle/BatteryReceptacle/AVP_ServerEntity_InteractBatteryReceptacle.h"
#include "Entity/PhysicalObject/Interact/ToolReceptacle/HackingPoint/AVP_ClientEntity_InteractHackingPoint.h"
#include "Entity/PhysicalObject/Pickup/AVP_ServerEntity_Pickup.h"
#include "Inventory/AVP_Inventory_Group.h"
#include "Inventory/AVP_Inventory_HashIDs.h"
#include "Inventory/AVP_Inventory_Item.h"
#include "Inventory/AVP_Inventory_ItemTypes.h"
#include "Inventory/AVP_Inventory_System.h"
#include "Moves/AVP_Move_Interaction.h"

AVP_Human_Move_Interaction_PlugBattery::AVP_Human_Move_Interaction_PlugBattery( Asura_Movement_System& xParentSystem )
: PARENT( xParentSystem, GetStaticHash() )
{
	// Initialise the move flags...
	m_bClearQueuedCounterRequest	= true;
	m_bClearQueuedMeleeRequests		= true;
	m_bPreventsFlinch				= true;
	m_bDisableMovement				= true;
	m_eAimDirectionUsage			= AVP_ClientEntity_Actor::AIM_DIRECTION_RESET;
	m_bHideReticle					= true;

	AddNextMove(AVP_Human_Move_Interaction_LetGoBattery::GetStaticHash());
}

bool AVP_Human_Move_Interaction_PlugBattery::Start( const Asura_Msg_Move* pxStartMsg )
{
	// Have we been told to start via a message?
	if ( pxStartMsg == NULL )
	{
		Asura_Assert( false, "Interaction move can only be triggered by message." );
		return false;
	}
	else
	{
		const AVP_Message_InteractionMoveStart*	pxMessage = static_cast< const AVP_Message_InteractionMoveStart* >( pxStartMsg );
		// This is the automatic next move, so we initialise it to use this interaction object as it will not be conveyed to it by message
		static_cast< AVP_Human_Move_Interaction_LetGoBattery* >(GetParentSystem().GetMove(AVP_Human_Move_Interaction_LetGoBattery::GetStaticHash()))->SetInteractionObjectGuid( pxMessage->GetInteractionObject() );
		AVP_ClientEntity_InteractBatteryReceptacle* pxInteractObject = AVP_ClientEntity_InteractBatteryReceptacle::FindEntity( pxMessage->GetInteractionObject() );
		Asura_Assertf( pxInteractObject, ( "Unable to find client side interaction object %d to interact with", pxMessage->GetInteractionObject() ) );
		if ( pxInteractObject == NULL )
		{
			return false;
		}
		else
		{
			AVP_ClientEntity_Actor* pxActor = AVP_ClientEntity_Actor::FindEntity( GetParentSystem().GetOwnerGuid() );
			Asura_Assert( pxActor, "Only actors should be interacting" );
			if ( !pxActor || !pxActor->GetAnimationHandler() )
			{
				return false;
			}
			else
			{
				pxActor->DeactivateCurrentWeapon();

				Asura_Hash_ID uFPAnimHash = ASURA_HASH_ID_UNSET;
				Asura_Hash_ID uTPAnimHash = ASURA_HASH_ID_UNSET;
				pxInteractObject->GetMarineInsertBatteryAnimHashes( uFPAnimHash, uTPAnimHash );
				SetAnimHashes( uFPAnimHash, uTPAnimHash );
				if ( m_pxAVPAnimationData )
				{
					m_pxAVPAnimationData->SetLeaveAnimAtEnd(true);
					m_pxAVPAnimationData->SetTweenData( DEFAULT_TWEEN_TIME, 0.0f, true );
					m_pxAVPAnimationData->SetControllerGroupHash( pxActor->GetAnimationHandler()->GetWeaponGroupHash() );
					m_pxAVPAnimationData->SetAnimPriority( AVP_AAPC_WEAPON_ANIM );
				}

				const float	fDuration = 0.400f;
				Asura_Vector_3 xTargetPosition;
				Asura_Matrix_3x3 xTargetOrientation;

				if ( uFPAnimHash != ASURA_HASH_ID_UNSET || uTPAnimHash != ASURA_HASH_ID_UNSET )
				{
					const AVP_ClientEntity_InteractObject::AnimData xAnimData = pxInteractObject->GetInteractionData( *pxActor, pxMessage->GetOn() );
					if ( !pxInteractObject->GetMarkerLocationFromHash( xAnimData.m_uActorPositionMarkerPoint, &xTargetPosition, &xTargetOrientation ) )
					{
						xTargetPosition = pxActor->GetCurrentPosition();
						xTargetOrientation = pxActor->GetCurrentOrientation();
					}

					// For the time being we'll just move and rotate to our desired position...
					m_xOrientationInterpolationData.SetMovementOffsetOperation( Asura_MoveData_OrientationInterpolation::STRETCH_MOVEMENT_OFFSET );
					m_xPositionInterpolationData.SetMovementOffsetOperation( Asura_MoveData_PositionInterpolation::STRETCH_MOVEMENT_OFFSET );

					m_xOrientationInterpolationData.SetDuration( fDuration );
					m_xOrientationInterpolationData.SetFinal( xTargetOrientation );
					m_xPositionInterpolationData.SetDuration( fDuration );
					m_xPositionInterpolationData.SetFinal( xTargetPosition );

					InitialiseOrientationInterpolation( m_xOrientationInterpolationData );
					InitialisePositionInterpolation( m_xPositionInterpolationData );
				}

				// Finally, call the base-function...
				return PARENT::Start( pxStartMsg );
			}
		}
	}
}

////////////////////////////////////////////////////////////////////////////////
// Class		: AVP_Human_Move_Interaction_LetGoBattery
// Author       : Ali Jakamy
// Date         : 11/12/08
////////////////////////////////////////////////////////////////////////////////

AVP_Human_Move_Interaction_LetGoBattery::AVP_Human_Move_Interaction_LetGoBattery( Asura_Movement_System& xParentSystem )
: PARENT( xParentSystem, GetStaticHash() )
, m_uInteractionObject(ASURA_GUID_UNREGISTERED)
{
	// Initialise the move flags...
	m_bClearQueuedCounterRequest	= true;
	m_bClearQueuedMeleeRequests		= true;
	m_bPreventsFlinch				= true;
	m_bDisableMovement				= true;
	m_eAimDirectionUsage			= AVP_ClientEntity_Actor::AIM_DIRECTION_RESET;
	m_bHideReticle					= true;
}

bool AVP_Human_Move_Interaction_LetGoBattery::Start( const Asura_Msg_Move* pxStartMsg )
{
	AVP_ClientEntity_InteractBatteryReceptacle* pxInteractObject = AVP_ClientEntity_InteractBatteryReceptacle::FindEntity( m_uInteractionObject);

	if ( !pxInteractObject )
	{
		Asura_Assert( false, "Unable to find client side interaction object ");
		return false;
	}
	else
	{
		const AVP_ClientEntity_Actor* pxActor = AVP_ClientEntity_Actor::FindEntity( GetParentSystem().GetOwnerGuid() );
		Asura_Assert( pxActor, "Only actors should be interacting" );
		if ( !pxActor || !pxActor->GetAnimationHandler() )
		{
			return false;
		}
		else
		{
			Asura_Hash_ID uFPAnimHash = ASURA_HASH_ID_UNSET;
			Asura_Hash_ID uTPAnimHash = ASURA_HASH_ID_UNSET;
			pxInteractObject->GetMarineInsertBatteryLetGoAnimHashes(uFPAnimHash, uTPAnimHash );
			SetAnimHashes( uFPAnimHash, uTPAnimHash );
			if (m_pxAVPAnimationData)
			{
				m_pxAVPAnimationData->SetLeaveAnimAtEnd( true );
				m_pxAVPAnimationData->SetTweenData( 0.05f, 0.0f, true );
				m_pxAVPAnimationData->SetControllerGroupHash( pxActor->GetAnimationHandler()->GetWeaponGroupHash() );
				m_pxAVPAnimationData->SetAnimPriority( AVP_AAPC_WEAPON_ANIM );
			}

			AVP_ServerEntity_InteractBatteryReceptacle* pxServerBatteryReceptacle = AVP_ServerEntity_InteractBatteryReceptacle::FindEntity(pxInteractObject->GetGuid());

			// Play Pickup anim
			if(pxServerBatteryReceptacle)
			{
				pxInteractObject->PlayToolAnim(pxServerBatteryReceptacle->GetToolGuid(), pxInteractObject->GetToolAnimRelatedToMove(AVP_Human_Move_Interaction_LetGoBattery::GetStaticHash()));
			}

			// Finally, call the base-function...
			return PARENT::Start( pxStartMsg );
		}
	}
}

////////////////////////////////////////////////////////////////////////////////
// Class		: AVP_Human_Move_Interaction_UnPlugBattery
// Author       : Ali Jakamy
// Date         : 23/10/08
////////////////////////////////////////////////////////////////////////////////

AVP_Human_Move_Interaction_UnPlugBattery::AVP_Human_Move_Interaction_UnPlugBattery( Asura_Movement_System& xParentSystem )
: PARENT( xParentSystem, GetStaticHash() )
{
	// Initialise the move flags...
	m_bClearQueuedCounterRequest	= true;
	m_bClearQueuedMeleeRequests		= true;
	m_bPreventsFlinch				= true;
	m_bDisableMovement				= true;
	m_eAimDirectionUsage			= AVP_ClientEntity_Actor::AIM_DIRECTION_RESET;
	m_bHideReticle					= true;

	AddNextMove(AVP_Human_Move_Interaction_GrabBattery::GetStaticHash());
}

bool AVP_Human_Move_Interaction_UnPlugBattery::Start( const Asura_Msg_Move* pxStartMsg )
{
	// Have we been told to start via a message?
	if ( pxStartMsg != NULL )
	{
		Asura_Assert( false, "Interaction move can not be triggered by message." );
		return false;
	}
	else
	{
		const Asura_Move* pxMove = GetParentSystem().GetCurrentMove();
		if ( pxMove->GetHash() != AVP_Move_Interaction_BeginWeaponAway::GetStaticHash() )
		{
			Asura_Assert( false, "Can only be started from AVP_Move_Interaction_Begin" );
			return false;
		}
		const AVP_Move_Interaction_BeginWeaponAway* pxIntBeginMove = static_cast< const AVP_Move_Interaction_BeginWeaponAway* >( pxMove );

		Asura_Guid uInteractionGuid = pxIntBeginMove->GetInteractionObject();
		bool bOn = pxIntBeginMove->GetOn();

		// This is the automatic next move, so we initialise it to use this interaction object as it will not be conveyed to it by message
		static_cast< AVP_Human_Move_Interaction_GrabBattery* >(GetParentSystem().GetMove(AVP_Human_Move_Interaction_GrabBattery::GetStaticHash()))->SetInteractionObjectGuid( uInteractionGuid );
		AVP_ClientEntity_InteractBatteryReceptacle* pxInteractObject = AVP_ClientEntity_InteractBatteryReceptacle::FindEntity( uInteractionGuid );
		Asura_Assertf( pxInteractObject, ( "Unable to find client side interaction object %d to interact with", uInteractionGuid ) );
		if ( pxInteractObject == NULL )
		{
			return false;
		}
		else
		{
			AVP_ClientEntity_Actor* pxActor = AVP_ClientEntity_Actor::FindEntity( GetParentSystem().GetOwnerGuid() );
			Asura_Assert( pxActor, "Only actors should be interacting" );
			if ( !pxActor || !pxActor->GetAnimationHandler() )
			{
				return false;
			}
			else
			{
				// hide the weapon before we go to grab the battery
				pxActor->DeactivateCurrentWeapon();

				Asura_Hash_ID uFPAnimHash = ASURA_HASH_ID_UNSET;
				Asura_Hash_ID uTPAnimHash = ASURA_HASH_ID_UNSET;
				pxInteractObject->GetMarineRemoveBatteryAnimHashes(uFPAnimHash, uTPAnimHash );
				SetAnimHashes( uFPAnimHash, uTPAnimHash );
				if (m_pxAVPAnimationData)
				{
					m_pxAVPAnimationData->SetLeaveAnimAtEnd(true);
					m_pxAVPAnimationData->SetTweenData( DEFAULT_TWEEN_TIME, 0.0f, true );
					m_pxAVPAnimationData->SetControllerGroupHash( pxActor->GetAnimationHandler()->GetWeaponGroupHash() );
					m_pxAVPAnimationData->SetAnimPriority( AVP_AAPC_WEAPON_ANIM );
				}

				const float	fDuration = 0.400f;
				Asura_Vector_3 xTargetPosition;
				Asura_Matrix_3x3 xTargetOrientation;

				if ( uFPAnimHash != ASURA_HASH_ID_UNSET || uTPAnimHash != ASURA_HASH_ID_UNSET )
				{
					const AVP_ClientEntity_InteractObject::AnimData xAnimData = pxInteractObject->GetInteractionData( *pxActor, bOn );
					if ( !pxInteractObject->GetMarkerLocationFromHash( xAnimData.m_uActorPositionMarkerPoint, &xTargetPosition, &xTargetOrientation ) )
					{
						xTargetPosition = pxActor->GetCurrentPosition();
						xTargetOrientation = pxActor->GetCurrentOrientation();
					}

					// For the time being we'll just move and rotate to our desired position...
					m_xOrientationInterpolationData.SetMovementOffsetOperation( Asura_MoveData_OrientationInterpolation::STRETCH_MOVEMENT_OFFSET );
					m_xPositionInterpolationData.SetMovementOffsetOperation( Asura_MoveData_PositionInterpolation::STRETCH_MOVEMENT_OFFSET );

					m_xOrientationInterpolationData.SetDuration( fDuration );
					m_xOrientationInterpolationData.SetFinal( xTargetOrientation );
					m_xPositionInterpolationData.SetDuration( fDuration );
					m_xPositionInterpolationData.SetFinal( xTargetPosition );

					InitialiseOrientationInterpolation( m_xOrientationInterpolationData );
					InitialisePositionInterpolation( m_xPositionInterpolationData );
				}
				// Finally, call the base-function...
				return PARENT::Start( pxStartMsg );
			}
		}
	}
}

bool AVP_Human_Move_Interaction_UnPlugBattery::ShouldStart() const
{
	const Asura_Move* pxMove = GetParentSystem().GetCurrentMove();
	if ( pxMove->GetHash() != AVP_Move_Interaction_BeginWeaponAway::GetStaticHash() )
	{
		Asura_Assert( false, "Can only be started from AVP_Move_Interaction_Begin" );
		return false;
	}
	const AVP_Move_Interaction_BeginWeaponAway* pxIntBeginMove = static_cast< const AVP_Move_Interaction_BeginWeaponAway* >( pxMove );
	return pxIntBeginMove->GetNextMoveHash() == GetStaticHash();
}

////////////////////////////////////////////////////////////////////////////////
// Class		: AVP_Human_Move_Interaction_GrabBattery
// Author       : Ali Jakamy
// Date         : 11/12/08
////////////////////////////////////////////////////////////////////////////////

AVP_Human_Move_Interaction_GrabBattery::AVP_Human_Move_Interaction_GrabBattery( Asura_Movement_System& xParentSystem )
: PARENT( xParentSystem, GetStaticHash() )
, m_uInteractionObject(ASURA_GUID_UNREGISTERED)
{
	// Initialise the move flags...
	m_bClearQueuedCounterRequest	= true;
	m_bClearQueuedMeleeRequests		= true;
	m_bPreventsFlinch				= true;
	m_bDisableMovement				= true;
	m_eAimDirectionUsage			= AVP_ClientEntity_Actor::AIM_DIRECTION_RESET;
	m_bHideReticle					= true;
}

bool AVP_Human_Move_Interaction_GrabBattery::Start( const Asura_Msg_Move* pxStartMsg )
{
	AVP_ClientEntity_InteractBatteryReceptacle* pxInteractObject = AVP_ClientEntity_InteractBatteryReceptacle::FindEntity( m_uInteractionObject);

	if ( !pxInteractObject )
	{
		Asura_Assert( false, "Unable to find client side interaction object" );
		return false;
	}
	else
	{
		const AVP_ClientEntity_Actor* pxActor = AVP_ClientEntity_Actor::FindEntity( GetParentSystem().GetOwnerGuid() );
		Asura_Assert( pxActor, "Only actors should be interacting" );
		if ( !pxActor || !pxActor->GetAnimationHandler() )
		{
			return false;
		}
		else
		{
			Asura_Hash_ID uFPAnimHash = ASURA_HASH_ID_UNSET;
			Asura_Hash_ID uTPAnimHash = ASURA_HASH_ID_UNSET;
			pxInteractObject->GetMarineRemoveGrabBatteryAnimHashes(uFPAnimHash, uTPAnimHash );
			SetAnimHashes( uFPAnimHash, uTPAnimHash );
			if (m_pxAVPAnimationData)
			{
				m_pxAVPAnimationData->SetTweenData( 0.05f, 0.1f, true );
				m_pxAVPAnimationData->SetControllerGroupHash( pxActor->GetAnimationHandler()->GetWeaponGroupHash() );
				m_pxAVPAnimationData->SetAnimPriority( AVP_AAPC_WEAPON_ANIM );
			}

			AVP_ServerEntity_InteractBatteryReceptacle* pxServerInteractObject = AVP_ServerEntity_InteractBatteryReceptacle::FindEntity( m_uInteractionObject);

			if(pxServerInteractObject)
			{
				// Play Pickup anim
 				pxInteractObject->PlayToolAnim(pxServerInteractObject->GetLastUsedToolGuid(), pxInteractObject->GetToolAnimRelatedToMove(AVP_Human_Move_Interaction_GrabBattery::GetStaticHash()));
			}

			const AVP_ClientEntity_InteractObject::AnimData xAnimData = pxInteractObject->GetInteractionData( *pxActor, pxInteractObject->IsOn() );

			// play the object anim
			if ( xAnimData.m_uObjectAnim != ASURA_HASH_ID_UNSET )
			{
				pxInteractObject->PlayAnimation( xAnimData.m_uObjectAnim );
			}
			// Finally, call the base-function...
			return PARENT::Start( pxStartMsg );
		}
	}
}

Asura_Move* AVP_Human_Move_Interaction_GrabBattery::Update()
{
	const AVP_ClientEntity_Actor* pxActor = AVP_ClientEntity_Actor::FindEntity( GetParentSystem().GetOwnerGuid() );
	Asura_Assert( pxActor, "Only actors should be interacting" );
	if ( pxActor && pxActor->GetInventorySystem() )
	{
		AVP_Inventory_Group* pxGroup = pxActor->GetInventorySystem()->GetGroup( uAVP_INVENTORY_GROUPHASH_WEAPON );
		if ( pxGroup )
		{
			AVP_Inventory_Item* pxWeapon = pxGroup->GetCurrentInventoryItem();
			if ( pxWeapon && pxWeapon->GetItemType() == AVP_INVENTORYITEMTYPE_BATTERY )
			{
				pxWeapon->RequestDeactivation( true );
			}
		}
	}

	return PARENT::Update();
}

////////////////////////////////////////////////////////////////////////////////
// Class		: AVP_Human_Move_Interaction_PlugHackingTool
// Author       : Ali Jakamy
// Date         : 05/01/08
////////////////////////////////////////////////////////////////////////////////

AVP_Human_Move_Interaction_PlugHackingTool::AVP_Human_Move_Interaction_PlugHackingTool( Asura_Movement_System& xParentSystem )
: PARENT( xParentSystem, GetStaticHash() )
{
	// Initialise the move flags...
	m_bClearQueuedCounterRequest	= true;
	m_bClearQueuedMeleeRequests		= true;
	m_bPreventsFlinch				= true;
	m_bDisableMovement				= true;
	m_eAimDirectionUsage			= AVP_ClientEntity_Actor::AIM_DIRECTION_RESET;
	m_bHideReticle					= true;

	AddNextMove(AVP_Human_Move_Interaction_LetGoHackingTool::GetStaticHash());
}

bool AVP_Human_Move_Interaction_PlugHackingTool::Start( const Asura_Msg_Move* pxStartMsg )
{
	// Have we been told to start via a message?
	if ( pxStartMsg != NULL )
	{
		Asura_Assert( false, "Interaction move can not be triggered by message." );
		return false;
	}
	else
	{
		const Asura_Move* pxMove = GetParentSystem().GetCurrentMove();
		if ( pxMove->GetHash() != AVP_Move_Interaction_BeginWeaponAway::GetStaticHash() )
		{
			Asura_Assert( false, "Can only be started from AVP_Move_Interaction_Begin" );
			return false;
		}
		const AVP_Move_Interaction_BeginWeaponAway* pxIntBeginMove = static_cast< const AVP_Move_Interaction_BeginWeaponAway* >( pxMove );

		Asura_Guid uInteractionGuid = pxIntBeginMove->GetInteractionObject();
		bool bOn = pxIntBeginMove->GetOn();

		// This is the automatic next move, so we initialise it to use this interaction object as it will not be conveyed to it by message
		static_cast< AVP_Human_Move_Interaction_LetGoHackingTool* >(GetParentSystem().GetMove(AVP_Human_Move_Interaction_LetGoHackingTool::GetStaticHash()))->SetInteractionObjectGuid( uInteractionGuid );
		AVP_ClientEntity_InteractHackingPoint* pxInteractObject = AVP_ClientEntity_InteractHackingPoint::FindEntity( uInteractionGuid );
		Asura_Assertf( pxInteractObject, ( "Unable to find client side interaction object %d to interact with", uInteractionGuid ) );
		if ( pxInteractObject == NULL )
		{
			return false;
		}
		else
		{
			AVP_ClientEntity_Actor* pxActor = AVP_ClientEntity_Actor::FindEntity( GetParentSystem().GetOwnerGuid() );
			Asura_Assert( pxActor, "Only actors should be interacting" );
			if ( !pxActor || !pxActor->GetAnimationHandler() )
			{
				return false;
			}
			else
			{
				pxActor->DeactivateCurrentWeapon();
			
				// Unhide the hacking tool
				// Must be a human, check for it
				const AVP_ServerActor_Human* pxHuman = AVP_ServerActor_Human::FindEntity( GetParentSystem().GetOwnerGuid() );
				if(pxHuman)
				{
					AVP_ServerEntity_Pickup* pxPickup = AVP_ServerEntity_Pickup::FindEntity(pxHuman->GetHackingToolGuid());
					if(pxPickup)
					{	
					   pxPickup->SetHidden(false);
					}
				}

				Asura_Hash_ID uFPAnimHash = ASURA_HASH_ID_UNSET;
				Asura_Hash_ID uTPAnimHash = ASURA_HASH_ID_UNSET;
				pxInteractObject->GetMarineInsertHackingToolAnimHashes( uFPAnimHash, uTPAnimHash );
				SetAnimHashes( uFPAnimHash, uTPAnimHash );
				if (m_pxAVPAnimationData)
				{
					m_pxAVPAnimationData->SetLeaveAnimAtEnd(true);
					m_pxAVPAnimationData->SetTweenData( DEFAULT_TWEEN_TIME, 0.0f, true );
					m_pxAVPAnimationData->SetControllerGroupHash( pxActor->GetAnimationHandler()->GetWeaponGroupHash() );
					m_pxAVPAnimationData->SetAnimPriority( AVP_AAPC_WEAPON_ANIM );
				}

				const float	fDuration = 0.400f;
				Asura_Vector_3 xTargetPosition;
				Asura_Matrix_3x3 xTargetOrientation;

				if ( uFPAnimHash != ASURA_HASH_ID_UNSET || uTPAnimHash != ASURA_HASH_ID_UNSET )
				{
					const AVP_ClientEntity_InteractObject::AnimData xAnimData = pxInteractObject->GetInteractionData( *pxActor, bOn );
					if ( !pxInteractObject->GetMarkerLocationFromHash( xAnimData.m_uActorPositionMarkerPoint, &xTargetPosition, &xTargetOrientation ) )
					{
						xTargetPosition = pxActor->GetCurrentPosition();
						xTargetOrientation = pxActor->GetCurrentOrientation();
					}

					// play the object anim
					if ( xAnimData.m_uObjectAnim != ASURA_HASH_ID_UNSET )
					{
						pxInteractObject->PlayAnimation( xAnimData.m_uObjectAnim );
					}

					// For the time being we'll just move and rotate to our desired position...
					m_xOrientationInterpolationData.SetMovementOffsetOperation( Asura_MoveData_OrientationInterpolation::STRETCH_MOVEMENT_OFFSET );
					m_xPositionInterpolationData.SetMovementOffsetOperation( Asura_MoveData_PositionInterpolation::STRETCH_MOVEMENT_OFFSET );

					m_xOrientationInterpolationData.SetDuration( fDuration );
					m_xOrientationInterpolationData.SetFinal( xTargetOrientation );
					m_xPositionInterpolationData.SetDuration( fDuration );
					m_xPositionInterpolationData.SetFinal( xTargetPosition );

					InitialiseOrientationInterpolation( m_xOrientationInterpolationData );
					InitialisePositionInterpolation( m_xPositionInterpolationData );
				}

				// Finally, call the base-function...
				return PARENT::Start( pxStartMsg );
			}
		}
	}
}

bool AVP_Human_Move_Interaction_PlugHackingTool::ShouldStart() const
{
	const Asura_Move* pxMove = GetParentSystem().GetCurrentMove();
	if ( pxMove->GetHash() != AVP_Move_Interaction_BeginWeaponAway::GetStaticHash() )
	{
		Asura_Assert( false, "Can only be started from AVP_Move_Interaction_Begin" );
		return false;
	}
	const AVP_Move_Interaction_BeginWeaponAway* pxIntBeginMove = static_cast< const AVP_Move_Interaction_BeginWeaponAway* >( pxMove );
	return pxIntBeginMove->GetNextMoveHash() == GetStaticHash();
}

////////////////////////////////////////////////////////////////////////////////
// Class		: AVP_Human_Move_Interaction_UnPlugHackingTool
// Author       : Ali Jakamy
// Date         : 23/10/08
////////////////////////////////////////////////////////////////////////////////

AVP_Human_Move_Interaction_UnPlugHackingTool::AVP_Human_Move_Interaction_UnPlugHackingTool( Asura_Movement_System& xParentSystem )
: PARENT( xParentSystem, GetStaticHash() )
{
	// Initialise the move flags...
	m_bClearQueuedCounterRequest	= true;
	m_bClearQueuedMeleeRequests		= true;
	m_bPreventsFlinch				= true;
	m_bDisableMovement				= true;
	m_eAimDirectionUsage			= AVP_ClientEntity_Actor::AIM_DIRECTION_RESET;
	m_bHideReticle					= true;

	AddNextMove(AVP_Human_Move_Interaction_GrabHackingTool::GetStaticHash());
}

bool AVP_Human_Move_Interaction_UnPlugHackingTool::Start( const Asura_Msg_Move* pxStartMsg )
{
	// Have we been told to start via a message?
	if ( pxStartMsg != NULL )
	{
		Asura_Assert( false, "Interaction move can not be triggered by message." );
		return false;
	}
	else
	{
		const Asura_Move* pxMove = GetParentSystem().GetCurrentMove();
		if ( pxMove->GetHash() != AVP_Move_Interaction_BeginWeaponAway::GetStaticHash() )
		{
			Asura_Assert( false, "Can only be started from AVP_Move_Interaction_Begin" );
			return false;
		}
		const AVP_Move_Interaction_BeginWeaponAway* pxIntBeginMove = static_cast< const AVP_Move_Interaction_BeginWeaponAway* >( pxMove );

		Asura_Guid uInteractionGuid = pxIntBeginMove->GetInteractionObject();
		bool bOn = pxIntBeginMove->GetOn();

		// This is the automatic next move, so we initialise it to use this interaction object as it will not be conveyed to it by message
		static_cast< AVP_Human_Move_Interaction_GrabHackingTool* >(GetParentSystem().GetMove(AVP_Human_Move_Interaction_GrabHackingTool::GetStaticHash()))->SetInteractionObjectGuid( uInteractionGuid );
		AVP_ClientEntity_InteractHackingPoint* pxInteractObject = AVP_ClientEntity_InteractHackingPoint::FindEntity( uInteractionGuid );
		Asura_Assertf( pxInteractObject, ( "Unable to find client side interaction object %d to interact with", uInteractionGuid ) );
		if ( pxInteractObject == NULL )
		{
			return false;
		}
		else
		{
			AVP_ClientEntity_Actor* pxActor = AVP_ClientEntity_Actor::FindEntity( GetParentSystem().GetOwnerGuid() );
			Asura_Assert( pxActor, "Only actors should be interacting" );
			if ( !pxActor || !pxActor->GetAnimationHandler() )
			{
				return false;
			}
			else
			{
				pxActor->DeactivateCurrentWeapon();

				Asura_Hash_ID uFPAnimHash = ASURA_HASH_ID_UNSET;
				Asura_Hash_ID uTPAnimHash = ASURA_HASH_ID_UNSET;
				pxInteractObject->GetMarineRemoveHackingToolAnimHashes(uFPAnimHash, uTPAnimHash );
				SetAnimHashes( uFPAnimHash, uTPAnimHash );
				if (m_pxAVPAnimationData)
				{
					m_pxAVPAnimationData->SetLeaveAnimAtEnd(true);
					m_pxAVPAnimationData->SetTweenData( DEFAULT_TWEEN_TIME, 0.0f, true );
					m_pxAVPAnimationData->SetControllerGroupHash( pxActor->GetAnimationHandler()->GetWeaponGroupHash() );
					m_pxAVPAnimationData->SetAnimPriority( AVP_AAPC_WEAPON_ANIM );
				}

				const float	fDuration = 0.400f;
				Asura_Vector_3 xTargetPosition;
				Asura_Matrix_3x3 xTargetOrientation;

				if ( uFPAnimHash != ASURA_HASH_ID_UNSET || uTPAnimHash != ASURA_HASH_ID_UNSET )
				{
					const AVP_ClientEntity_InteractObject::AnimData xAnimData = pxInteractObject->GetInteractionData( *pxActor, bOn );
					if ( !pxInteractObject->GetMarkerLocationFromHash( xAnimData.m_uActorPositionMarkerPoint, &xTargetPosition, &xTargetOrientation ) )
					{
						xTargetPosition = pxActor->GetCurrentPosition();
						xTargetOrientation = pxActor->GetCurrentOrientation();
					}

					// For the time being we'll just move and rotate to our desired position...
					m_xOrientationInterpolationData.SetMovementOffsetOperation( Asura_MoveData_OrientationInterpolation::STRETCH_MOVEMENT_OFFSET );
					m_xPositionInterpolationData.SetMovementOffsetOperation( Asura_MoveData_PositionInterpolation::STRETCH_MOVEMENT_OFFSET );

					m_xOrientationInterpolationData.SetDuration( fDuration );
					m_xOrientationInterpolationData.SetFinal( xTargetOrientation );
					m_xPositionInterpolationData.SetDuration( fDuration );
					m_xPositionInterpolationData.SetFinal( xTargetPosition );

					InitialiseOrientationInterpolation( m_xOrientationInterpolationData );
					InitialisePositionInterpolation( m_xPositionInterpolationData );
				}
				// Finally, call the base-function...
				return PARENT::Start( pxStartMsg );
			}
		}
	}
}

bool AVP_Human_Move_Interaction_UnPlugHackingTool::ShouldStart() const
{
	const Asura_Move* pxMove = GetParentSystem().GetCurrentMove();
	if ( pxMove->GetHash() != AVP_Move_Interaction_BeginWeaponAway::GetStaticHash() )
	{
		Asura_Assert( false, "Can only be started from AVP_Move_Interaction_Begin" );
		return false;
	}
	const AVP_Move_Interaction_BeginWeaponAway* pxIntBeginMove = static_cast< const AVP_Move_Interaction_BeginWeaponAway* >( pxMove );
	return pxIntBeginMove->GetNextMoveHash() == GetStaticHash();
}

////////////////////////////////////////////////////////////////////////////////
// Class		: AVP_Human_Move_Interaction_LetGoHackingTool
// Author       : Ali Jakamy
// Date         : 23/10/08
////////////////////////////////////////////////////////////////////////////////

AVP_Human_Move_Interaction_LetGoHackingTool::AVP_Human_Move_Interaction_LetGoHackingTool( Asura_Movement_System& xParentSystem )
: PARENT( xParentSystem, GetStaticHash() )
, m_uInteractionObject(ASURA_GUID_UNREGISTERED)
{
	// Initialise the move flags...
	m_bClearQueuedCounterRequest	= true;
	m_bClearQueuedMeleeRequests		= true;
	m_bPreventsFlinch				= true;
	m_bDisableMovement				= true;
	m_eAimDirectionUsage			= AVP_ClientEntity_Actor::AIM_DIRECTION_RESET;
	m_bHideReticle					= true;

	AddNextMove( AVP_Move_Interaction_EndWeaponBack::GetStaticHash() );
}

bool AVP_Human_Move_Interaction_LetGoHackingTool::Start( const Asura_Msg_Move* pxStartMsg )
{
	AVP_ClientEntity_InteractHackingPoint* pxInteractObject = AVP_ClientEntity_InteractHackingPoint::FindEntity( m_uInteractionObject);

	if ( !pxInteractObject )
	{
		Asura_Assert( false, "Unable to find client side interaction object ");
		return false;
	}
	else
	{
		const AVP_ClientEntity_Actor* pxActor = AVP_ClientEntity_Actor::FindEntity( GetParentSystem().GetOwnerGuid() );
		Asura_Assert( pxActor, "Only actors should be interacting" );
		if ( !pxActor || !pxActor->GetAnimationHandler() )
		{
			return false;
		}
		else
		{
			Asura_Hash_ID uFPAnimHash = ASURA_HASH_ID_UNSET;
			Asura_Hash_ID uTPAnimHash = ASURA_HASH_ID_UNSET;
			pxInteractObject->GetMarineLetGoHackingToolAnimHashes(uFPAnimHash, uTPAnimHash );
			SetAnimHashes( uFPAnimHash, uTPAnimHash );
			if (m_pxAVPAnimationData)
			{
				m_pxAVPAnimationData->SetLeaveAnimAtEnd(true);
				m_pxAVPAnimationData->SetTweenData( 0.05f, 0.0f, true );
				m_pxAVPAnimationData->SetControllerGroupHash( pxActor->GetAnimationHandler()->GetWeaponGroupHash() );
				m_pxAVPAnimationData->SetAnimPriority( AVP_AAPC_WEAPON_ANIM );
			}

			// Finally, call the base-function...
			return PARENT::Start( pxStartMsg );
		}
	}
}

////////////////////////////////////////////////////////////////////////////////
// Class		: AVP_Human_Move_Interaction_GrabGoHackingTool
// Author       : Ali Jakamy
// Date         : 23/10/08
////////////////////////////////////////////////////////////////////////////////

AVP_Human_Move_Interaction_GrabHackingTool::AVP_Human_Move_Interaction_GrabHackingTool( Asura_Movement_System& xParentSystem )
: PARENT( xParentSystem, GetStaticHash() )
, m_uInteractionObject(ASURA_GUID_UNREGISTERED)
{
	// Initialise the move flags...
	m_bClearQueuedCounterRequest	= true;
	m_bClearQueuedMeleeRequests		= true;
	m_bPreventsFlinch				= true;
	m_bDisableMovement				= true;
	m_eAimDirectionUsage			= AVP_ClientEntity_Actor::AIM_DIRECTION_RESET;
	m_bHideReticle					= true;

	AddNextMove( AVP_Move_Interaction_EndWeaponBack::GetStaticHash() );
}

bool AVP_Human_Move_Interaction_GrabHackingTool::Start( const Asura_Msg_Move* pxStartMsg )
{
	AVP_ClientEntity_InteractHackingPoint* pxInteractObject = AVP_ClientEntity_InteractHackingPoint::FindEntity( m_uInteractionObject);

	if ( !pxInteractObject )
	{
		Asura_Assert( false, "Unable to find client side interaction object ");
		return false;
	}
	else
	{
		const AVP_ClientEntity_Actor* pxActor = AVP_ClientEntity_Actor::FindEntity( GetParentSystem().GetOwnerGuid() );
		Asura_Assert( pxActor, "Only actors should be interacting" );
		if ( !pxActor || !pxActor->GetAnimationHandler() )
		{
			return false;
		}
		else
		{
			Asura_Hash_ID uFPAnimHash = ASURA_HASH_ID_UNSET;
			Asura_Hash_ID uTPAnimHash = ASURA_HASH_ID_UNSET;
			pxInteractObject->GetMarineGrabHackingToolAnimHashes(uFPAnimHash, uTPAnimHash );
			SetAnimHashes( uFPAnimHash, uTPAnimHash );
			if (m_pxAVPAnimationData)
			{
				m_pxAVPAnimationData->SetLeaveAnimAtEnd(true);
				m_pxAVPAnimationData->SetTweenData( 0.05f, 0.0f, true );
				m_pxAVPAnimationData->SetControllerGroupHash( pxActor->GetAnimationHandler()->GetWeaponGroupHash() );
				m_pxAVPAnimationData->SetAnimPriority( AVP_AAPC_WEAPON_ANIM );
			}

			const AVP_ClientEntity_InteractObject::AnimData xAnimData = pxInteractObject->GetInteractionData( *pxActor, pxInteractObject->IsOn() );

			// play the object anim
			if ( xAnimData.m_uObjectAnim != ASURA_HASH_ID_UNSET )
			{
				pxInteractObject->PlayAnimation( xAnimData.m_uObjectAnim );
			}
			// Finally, call the base-function...
			return PARENT::Start( pxStartMsg );
		}
	}
}
