///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_Human_MovesMantle.cpp
// Author : Anthony Gregson
// Date   : 31/07/08
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

// Include files...
// Core
#include "Core/Asura.h"

// This Files Header
#include "Moves/Vertebrates/Human/AVP_Human_MovesMantle.h"

// Asura
#include "Animation/Asura_Animation.h"

// AVP
#include "Entity/PhysicalObject/Actor/AVP_ClientEntity_Actor.h"


DEBUGVAR float dbg_fMantleTweenDuration = DEFAULT_TWEEN_TIME;
DEBUGVAR float dbg_fMantleOrientationInterpolationDuration = 0.25f;
DEBUGVAR float dbg_fMantlePositionInterpolationDuration = 0.25f;

AVP_Move_Human_Mantle::AVP_Move_Human_Mantle(Asura_Movement_System& xParentSystem)
: PARENT(xParentSystem, GetStaticHash())
{
	m_xOrientationInterpolationData.SetDuration(dbg_fMantleOrientationInterpolationDuration);
	m_xOrientationInterpolationData.SetMovementOffsetOperation(Asura_MoveData_OrientationInterpolation::SUBTRACT_MOVEMENT_OFFSET);
	m_xPositionInterpolationData.SetDuration(dbg_fMantlePositionInterpolationDuration);
	m_xPositionInterpolationData.SetMovementOffsetOperation(Asura_MoveData_PositionInterpolation::SUBTRACT_MOVEMENT_OFFSET);
}

bool AVP_Move_Human_Mantle::Start(const Asura_Msg_Move* pxStartMsg)
{
	// This has to start by message, and we need to use our own special message.
	if (!pxStartMsg) { return false; }

	const AVP_Msg_Move_StartMantle* pxMantleMsg = static_cast<const AVP_Msg_Move_StartMantle*>(pxStartMsg);

	const bool bRequiresJump = pxMantleMsg->GetRequiresJump();

	SetAnimHashes(pxMantleMsg->GetFirstPersonAnim(), pxMantleMsg->GetThirdPersonAnim());
	if (!m_pxAVPAnimationData) { return false; }

	m_pxAVPAnimationData->SetTweenData(dbg_fMantleTweenDuration, dbg_fMantleTweenDuration, true);

	Asura_Quat xTargetOri;
	if (bRequiresJump)
	{
		xTargetOri = pxMantleMsg->GetTargetOrientation();
	}
	else
	{
		xTargetOri.FromMatrix(GetAVPParentSystem().GetActorOwner().GetCurrentOrientation());
	}
	m_xOrientationInterpolationData.SetFinal(xTargetOri);
	m_xPositionInterpolationData.SetFinal(pxMantleMsg->GetTargetPosition());

	InitialiseOrientationInterpolation(m_xOrientationInterpolationData);
	InitialisePositionInterpolation(m_xPositionInterpolationData);

	SetAimDirectionUsage(bRequiresJump ? AVP_ClientEntity_Actor::AIM_DIRECTION_RESET : AVP_ClientEntity_Actor::AIM_DIRECTION_NOT_USED);

	return PARENT::Start(pxStartMsg);
}

void AVP_Msg_Move_StartMantle::SetRequiresJump(bool bRequiresJump)
{
	if (bRequiresJump)
	{
		m_usProjectFlags |= AVP_MSG_MOVE_STARTMANTLE_FLAG_REQUIRESJUMP;
	}
	else
	{
		m_usProjectFlags &= ~AVP_MSG_MOVE_STARTMANTLE_FLAG_REQUIRESJUMP;
	}
}
