
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_Human_MovesMelee.cpp
// Author : Anthony Gregson
// Date   : 22/02/08
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"


// This file's header...
#include "Moves/Vertebrates/Human/AVP_Human_MovesMelee.h"

// Asura independent header-files...
#include "Blueprint/Asura_Blueprint_System.h"

// Asura platform header-files...

// Game independent header-files...
#include "Blueprint/AVP_Blueprint_HashIDs.h"
#include "Entity/PhysicalObject/Actor/AVP_ClientEntity_Actor.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Human/AVP_ClientActor_Human.h"
#include "Moves/AVP_Movement_System.h"



// Game platform header-files...

// Tools header-files...


////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////

static const float g_fTweenInDuration = 0.05f;
static const float g_fTweenOutDuration = 0.1f;

////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_Human_Move_Melee_CounterFaceHugger
////////////////////////////////////////////////////////////////////////////////

AVP_Human_Move_Melee_CounterFaceHugger::AVP_Human_Move_Melee_CounterFaceHugger(Asura_Movement_System& xParentSystem)
: PARENT(xParentSystem, GetStaticHash())
, m_uCurrentFirstPersonMeleeBlueprint(ASURA_HASH_ID_UNSET)
, m_uCurrentThirdPersonMeleeBlueprint(ASURA_HASH_ID_UNSET)
{
	m_bClearQueuedMeleeRequests = true;
}


bool AVP_Human_Move_Melee_CounterFaceHugger::Start(const Asura_Msg_Move* pxStartMsg)
{
	// The default counter gets a standard set of parameters from the current melee blueprint, and since that can change we may have to regenerate our animation list here.
	const AVP_ClientEntity_Actor& xOwner = GetAVPParentSystem().GetActorOwner();
	Asura_Hash_ID uFirstPersonBlueprintHash, uThirdPersonBlueprintHash;
	xOwner.GetCurrentMeleeBlueprintIDs(uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);

	if ((uFirstPersonBlueprintHash != m_uCurrentFirstPersonMeleeBlueprint) || (uThirdPersonBlueprintHash != m_uCurrentThirdPersonMeleeBlueprint))
	{
		// Yep, we do need to regenerate, get the appropriate blueprints
		const Asura_Blueprint* pxFirstPersonBlueprint = Asura_Blueprint_System::GetBlueprintByHashID(uAVP_BLUEPRINT_TYPE_MELEE, uFirstPersonBlueprintHash);
		const Asura_Blueprint* pxThirdPersonBlueprint = Asura_Blueprint_System::GetBlueprintByHashID(uAVP_BLUEPRINT_TYPE_MELEE, uThirdPersonBlueprintHash);
		if ( pxFirstPersonBlueprint || pxThirdPersonBlueprint )
		{
			SetupAnimHashesFromBlueprint(pxFirstPersonBlueprint, pxThirdPersonBlueprint, "FacehuggerCounter");
		}

		m_uCurrentFirstPersonMeleeBlueprint = uFirstPersonBlueprintHash;
		m_uCurrentThirdPersonMeleeBlueprint = uThirdPersonBlueprintHash;

		if (m_pxAVPAnimationData)
		{
			m_pxAVPAnimationData->SetTweenData(g_fTweenInDuration, g_fTweenOutDuration, true);
			m_pxAVPAnimationData->SetLeaveAnimAtEnd(true);
		}
	}

	// Having regenerated the list, we can run through the generic start function.
	return PARENT::Start(pxStartMsg);
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
AVP_Human_Move_Melee_Retaliation_Start::AVP_Human_Move_Melee_Retaliation_Start(Asura_Movement_System& xParentSystem)
: PARENT(xParentSystem, GetStaticHash())
{
	m_bClearQueuedMeleeRequests = true;
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
bool AVP_Human_Move_Melee_Retaliation_Start::Start(const Asura_Msg_Move* pxStartMsg)
{
	// The default counter gets a standard set of parameters from the current melee blueprint, and since that can change we may have to regenerate our animation list here.

	const AVP_ClientActor_Human& xOwner = (const AVP_ClientActor_Human&)GetAVPParentSystem().GetActorOwner();

	Asura_Hash_ID uFirstPersonBlueprintHash, uThirdPersonBlueprintHash;

	const AVP_Msg_MoveWithTargetEntity* pxMeleeStart = static_cast<const AVP_Msg_MoveWithTargetEntity*>(pxStartMsg);

	if ( pxMeleeStart )
	{
		xOwner.GetCurrentMeleeBlueprintIDs(uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);

		if ( uThirdPersonBlueprintHash != ASURA_HASH_ID_UNSET )
		{
			// Yep, we do need to regenerate, get the appropriate blueprints
			const Asura_Blueprint* pxThirdPersonBlueprint = Asura_Blueprint_System::GetBlueprintByHashID(uAVP_BLUEPRINT_TYPE_MELEE, uThirdPersonBlueprintHash);

			m_uNumThirdPersonVariations = 1;

			if ( pxThirdPersonBlueprint )
			{
				SetupAnimHashesFromBlueprint( NULL, pxThirdPersonBlueprint, "DefaultAttackStart");
			}

			if ( m_pxAVPAnimationData )
			{
				m_pxAVPAnimationData->SetTweenData(g_fTweenInDuration, g_fTweenOutDuration, true);
				
				m_pxAVPAnimationData->SetLeaveAnimAtEnd(true);
			}

			AVP_ClientActor_Human* pxTargetActor = AVP_ClientActor_Human::FindEntity( pxMeleeStart->GetTargetGuid() );
			Asura_Assert( pxTargetActor, " AVP_Human_Move_Melee_Retaliation_Start::Start - Target actor ( the player ) is not valid ");

			Asura_Vector_3		xDir( pxTargetActor->GetCurrentPosition() - xOwner.GetCurrentPosition() );
			Asura_Matrix_3x3	xTargetOrientation;
			Asura_Maths::OrientationFromDirection( xTargetOrientation, xDir );

			m_xOrientationInterpolationData.SetMovementOffsetOperation( Asura_MoveData_OrientationInterpolation::IGNORE_MOVEMENT_OFFSET );
			m_xOrientationInterpolationData.SetDuration( 0.45f ); // time from anim until key-frame
			m_xOrientationInterpolationData.SetFinal( xTargetOrientation );
		}
	}

	// Having regenerated the list, we can run through the generic start function.
	return PARENT::Start(pxStartMsg);
}


//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////
