
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_Predator_MovesBranch.h
// Author : Anthony Gregson
// Date   : 16/11/07
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __AVP_PREDATOR_MOVESBRANCH_H__
#define __AVP_PREDATOR_MOVESBRANCH_H__

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Entity/PhysicalObject/Moves/Asura_MoveData_Animation.h"
#include "Entity/PhysicalObject/Moves/Asura_MoveData_OrientationInterpolation.h"
#include "Entity/PhysicalObject/Moves/Asura_MoveData_PositionInterpolation.h"

#include "Moves/AVP_Moves.h"


////////////////////////////////////////////////////////////////////////////////
// Forward class declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Class declaration...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_Move_Predator_MountBranch
// Author       : Anthony Gregson
// Date         : 16/11/07
//
// Description  : Connect ourselves to a branch attractor
////////////////////////////////////////////////////////////////////////////////

class AVP_Move_Predator_MountBranch : public AVP_Move
{
public:
	// Public functions...
	AVP_Move_Predator_MountBranch(Asura_Movement_System& xParentSystem);
//	virtual ~AVP_Move_Predator_MountBranch();

	// Accessor functions...
	virtual const char*		GetName(void) const	{ return("AVP_Move_Predator_MountBranch"); }
	static Asura_Hash_ID	GetStaticHash(void)	{ return(0x745594FD); }	// Asura_GetHashID("AVP_Move_Predator_MountBranch")); }

protected:
	// Protected variables...
	Asura_MoveData_PositionInterpolation	m_xPositionInterpolationData;
	Asura_MoveData_OrientationInterpolation	m_xOrientationInterpolationData;

	// Overridden functions...
	virtual bool Start(const Asura_Msg_Move* pxStartMsg);

private:
	// Private typedefs...
	typedef AVP_Move PARENT;

	// Private variables...

	// Private functions...
	AVP_Move_Predator_MountBranch(void);
	AVP_Move_Predator_MountBranch(const AVP_Move_Predator_MountBranch& xOther);

	AVP_Move_Predator_MountBranch& operator=(const AVP_Move_Predator_MountBranch& xOther);

};   // End of class 'AVP_Move_Predator_MountBranch'.


// More bespoke move that shoves the actor off in the direction of their input.
class AVP_Move_Predator_DismountBranch : public AVP_Move
{
public:
	AVP_Move_Predator_DismountBranch(Asura_Movement_System& xParentSystem);

	// Accessor functions...
	virtual const char*		GetName(void) const	{ return("AVP_Move_Predator_DismountBranch"); }
	static Asura_Hash_ID	GetStaticHash(void)	{ return(0x3CAD206B); }	// Asura_GetHashID("AVP_Move_Predator_DismountBranch")); }

	const Asura_Vector_3&	GetMovementDirection() const { return m_xMovementDirection; }

protected:
	// Overridden functions...
	virtual bool Start(const Asura_Msg_Move* pxStartMsg);
	virtual void UpdateStates(const float fTimeStep);
	virtual bool HasFinished() const;

private:
	// Private typedefs...
	typedef AVP_Move PARENT;

	// Private variables...
	Asura_Vector_3 m_xMovementDirection; // Copy of input movement vector when move is started
	float m_fTimeInMove; // TODO - Replace this with detecting falling, and entering the falling move.
};

#endif   // ifndef __AVP_PREDATOR_MOVESBRANCH_H__


/// End of file. ///
