////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_Predator_Move_Interaction_PlugKey
// Author       : Ali Jakamy
// Date         : 23/10/08
////////////////////////////////////////////////////////////////////////////////

#include "Moves/AVP_Moves.h"

class AVP_ClientEntity_InteractRetinalScanner;

class AVP_Predator_Move_Interaction_PlugKey : public AVP_Move
{
public:
	AVP_Predator_Move_Interaction_PlugKey(Asura_Movement_System& xParentSystem);

	static Asura_Hash_ID GetStaticHash() { return 0xA6D1C75A; } // Asura_GetHashID("AVP_Predator_Move_Interaction_PlugKey")); }

protected:
	virtual bool Start(const Asura_Msg_Move* pxStartMsg);
	Asura_MoveData_OrientationInterpolation m_xOrientationInterpolationData;
	Asura_MoveData_PositionInterpolation    m_xPositionInterpolationData;

private:
	typedef AVP_Move PARENT;
};   

class AVP_Predator_Move_Interaction_UnPlugKey : public AVP_Move
{
public:
	AVP_Predator_Move_Interaction_UnPlugKey(Asura_Movement_System& xParentSystem);

	static Asura_Hash_ID GetStaticHash() { return 0xEF2C9C61; } // Asura_GetHashID("AVP_Predator_Move_Interaction_UnPlugKey")); }

protected:
	virtual bool Start(const Asura_Msg_Move* pxStartMsg);

private:
	typedef AVP_Move PARENT;
};  

class AVP_Predator_Move_Interaction_StandStill : public AVP_Move
{
public:
	AVP_Predator_Move_Interaction_StandStill(Asura_Movement_System& xParentSystem);

	static Asura_Hash_ID GetStaticHash() { return 0xBEB459A9; } // Asura_GetHashID("AVP_Predator_Move_Interaction_StandStill")); }
	void       SetInteractionObjectGuid( Asura_Guid uGuid ) { m_uInteractionObject = uGuid; }
	Asura_Guid GetInteractionObjectGuid() const { return m_uInteractionObject; }
	float      GetTweenHack() const { return m_fTweenHack; }
	void       AdvanceTweenHack();

protected:
	virtual bool Start(const Asura_Msg_Move* pxStartMsg);

private:
	
	float m_fTweenHack;  // Added because going fro a turn anim  to the same before the actual one is finished make the predator go to normal stand move position

	Asura_Guid m_uInteractionObject;    // Save the interaction GUID, in case it the moves starts without message
	typedef AVP_Move PARENT;
};   

class AVP_Predator_Move_Interaction_TurnLeft : public AVP_Move
{
public:
	AVP_Predator_Move_Interaction_TurnLeft(Asura_Movement_System& xParentSystem);

	static Asura_Hash_ID GetStaticHash() { return 0xAA4879DB; } // Asura_GetHashID("AVP_Predator_Move_Interaction_TurnLeft")); }

protected:
	virtual bool Start(const Asura_Msg_Move* pxStartMsg);

private:
	typedef AVP_Move PARENT;
};   

class AVP_Predator_Move_Interaction_TurnRight : public AVP_Move
{
public:
	AVP_Predator_Move_Interaction_TurnRight(Asura_Movement_System& xParentSystem);

	static Asura_Hash_ID GetStaticHash() { return 0x9F1D22C8; } // Asura_GetHashID("AVP_Predator_Move_Interaction_TurnRight")); }

protected:
	virtual bool Start(const Asura_Msg_Move* pxStartMsg);

private:
	typedef AVP_Move PARENT;
};   

class AVP_Predator_Move_Interaction_UseBattery : public AVP_Move
{
public:
	AVP_Predator_Move_Interaction_UseBattery(Asura_Movement_System& xParentSystem);

	static Asura_Hash_ID GetStaticHash() { return 0x32AC38BD; } // Asura_GetHashID("AVP_Predator_Move_Interaction_UseBattery")); }

protected:
	virtual bool Start(const Asura_Msg_Move* pxStartMsg);
	virtual bool ShouldStart() const;
	virtual void Abort();
	virtual void Finish();

	Asura_MoveData_OrientationInterpolation m_xOrientationInterpolationData;
	Asura_MoveData_PositionInterpolation    m_xPositionInterpolationData;

private:
	typedef AVP_Move PARENT;
}; 

class AVP_Predator_Move_Interaction_UseHead : public AVP_Move
{
public:
	// Public functions...
	AVP_Predator_Move_Interaction_UseHead(Asura_Movement_System& xParentSystem);
	virtual ~AVP_Predator_Move_Interaction_UseHead();

	// Accessor functions...
	static Asura_Hash_ID GetStaticHash() { return(0xBA61C130); } // Asura_GetHashID("AVP_Predator_Move_Interaction_UseHead")); }

protected:
	virtual bool Start(const Asura_Msg_Move* pxStartMsg);
	virtual bool ShouldStart() const;
	virtual void UpdateStates(const float fTimeStep);
	virtual void Abort();
	virtual void Finish();

	void AttachTrophy(const AVP_ClientEntity_InteractRetinalScanner& xRetinalScanner);
	void DetachTrophy();

	Asura_AnimationAttachment* m_pxTrophyAttachment;
	Asura_Hash_ID			   m_uTrophyMarkerPoint;
	Asura_Hash_ID			   m_uTrophyAnim;

	Asura_MoveData_OrientationInterpolation m_xOrientationInterpolationData;
	Asura_MoveData_PositionInterpolation    m_xPositionInterpolationData;

private:
	// Private typedef...
	typedef AVP_Move PARENT;
}; 