//////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_Multiplayer_GameRules.cpp
// Author : Anthony Gregson
// Date   : 04/12/07
//
// Copyright (c) 2007 Rebellion. All rights reserved.
//
// A static storage point for the constant (i.e. unchanging in a single game)
// settings for an AVP game. Set from local settings or across the network if
// joining a game (AVP_NetworkSession should include all the data we need).
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header.
#include "Multiplayer/AVP_Multiplayer_GameRules.h"

// Asura
#include "Blueprint/Asura_Blueprint_System.h"
#include "Interface/Asura_CommandConsole.h"

// Project
#include "Blueprint/AVP_Blueprint_HashIDs.h"
#include "Multiplayer/AVP_Multiplayer_ScoreRules.h"

// S T A T I C   C L A S S   D A T A /////////////////////////////////////////

int AVP_Multiplayer_GameRules::s_iPlayerScoreLimit = 0;
int AVP_Multiplayer_GameRules::s_iTeamScoreLimit = 0;
bool AVP_Multiplayer_GameRules::s_bAISpawnPointsOn = false;
bool AVP_Multiplayer_GameRules::s_bAllowMixedTeams = false;
bool AVP_Multiplayer_GameRules::s_bCapturePointsOn = false;
bool AVP_Multiplayer_GameRules::s_bCTFOn = false;
bool AVP_Multiplayer_GameRules::s_bFriendlyFireOn = false;
bool AVP_Multiplayer_GameRules::s_bHPForceTeamChange = false;
bool AVP_Multiplayer_GameRules::s_bHPHunterKillPreyJoin = false;
bool AVP_Multiplayer_GameRules::s_bHPPreyKillHunterSwitch = false;
int AVP_Multiplayer_GameRules::s_iLivesAvailablePerTeam = 50;
int AVP_Multiplayer_GameRules::s_iMaxNumberOfPlayers = 18; 
bool AVP_Multiplayer_GameRules::s_bMutuallyExclusiveTeams = true;
int AVP_Multiplayer_GameRules::s_iNumberOfTeams = 2; 

int AVP_Multiplayer_GameRules::s_iPlayerRatioAlien = 1;
int AVP_Multiplayer_GameRules::s_iPlayerRatioHuman = 1;
int AVP_Multiplayer_GameRules::s_iPlayerRatioPredator = 1;

int AVP_Multiplayer_GameRules::s_iPlayerScoreCaptureControlPoint = 1;
int AVP_Multiplayer_GameRules::s_iPlayerScoreCaptureFlag = 1;
int AVP_Multiplayer_GameRules::s_iPlayerScoreDefendControlPointBonus = 1;
int AVP_Multiplayer_GameRules::s_iPlayerScoreHunterKillPrimaryPreyBonus = 1;
int AVP_Multiplayer_GameRules::s_iPlayerScoreHunterKillSecondaryPreyBonus = 1;
int AVP_Multiplayer_GameRules::s_iPlayerScoreKillFlagCarrierBonus = 1;
int AVP_Multiplayer_GameRules::s_iPlayerScorePreyKillHunterBonus = 1;
int AVP_Multiplayer_GameRules::s_iPlayerScoreReturnFlagBonus = 1;

float AVP_Multiplayer_GameRules::s_fPreySurvivorMultiplierFirst = 1.0f;
float AVP_Multiplayer_GameRules::s_fPreySurvivorMultiplierSecond = 0.5f;
float AVP_Multiplayer_GameRules::s_fPreySurvivorMultiplierThird = 0.25f;

bool AVP_Multiplayer_GameRules::s_bUseWaveSpawning = false;
float AVP_Multiplayer_GameRules::s_fRespawnDelay = 10.0f;
bool AVP_Multiplayer_GameRules::s_bRespawnUponDeath = true;
AVP_Settings::RespawnType AVP_Multiplayer_GameRules::s_eRespawnType = AVP_Settings::AVP_RESPAWNTYPE_RANDOM;

AVP_Settings::Species AVP_Multiplayer_GameRules::s_eTeamOneSpecies = AVP_Settings::AVP_SPECIES_PREDATOR; 
AVP_Settings::Species AVP_Multiplayer_GameRules::s_eTeamTwoSpecies = AVP_Settings::AVP_SPECIES_HUMAN;
AVP_Settings::Species AVP_Multiplayer_GameRules::s_eTeamThreeSpecies = AVP_Settings::AVP_SPECIES_ALIEN;
bool AVP_Multiplayer_GameRules::s_bTeamLivesOn = false;
int AVP_Multiplayer_GameRules::s_iTeamScoreAllEnemiesDead = 1;
int AVP_Multiplayer_GameRules::s_iTeamScoreCaptureAllControlPoints = 1;
int AVP_Multiplayer_GameRules::s_iTeamScoreHoldControlPoint = 1;
bool AVP_Multiplayer_GameRules::s_bTeamScoreKillsAddToScore = false;
bool AVP_Multiplayer_GameRules::s_bTeamVictoryOn = false;

int AVP_Multiplayer_GameRules::s_iTimeLimit = 0;

bool AVP_Multiplayer_GameRules::s_bPersecutorModeOn = true; 
int AVP_Multiplayer_GameRules::s_iKillsNeededToPersecute = 5;

float AVP_Multiplayer_GameRules::s_fDDOMCaptureRadius = 5.0f;

// C L A S S   I M P L E M E N T A T I O N ///////////////////////////////////

////////////////////////////////////////////////////////////
// Rules (non trivial)
////////////////////////////////////////////////////////////
bool AVP_Multiplayer_GameRules::IsTeamBasedGame()
{
	// Check game type 
	if (AVP_Settings::GetGameType() == AVP_Settings::AVP_GAMETYPE_DEATH_MATCH)
	{
		// Sanity check
		if (GetNumberOfTeams() > 1)
		{
			Asura_Assert( false, "Unexpected number of teams, please fix the game rules for deathmatch, setting number of teams to 1" ); 
			SetNumberOfTeams(1);
		}
		return false; 
	}
	else if (AVP_Settings::GetGameType() == AVP_Settings::AVP_GAMETYPE_SPECIES_TEAM_DEATH_MATCH ||
			 AVP_Settings::GetGameType() == AVP_Settings::AVP_GAMETYPE_MIXED_SPECIES_TEAM_DEATH_MATCH ||
			 AVP_Settings::GetGameType() == AVP_Settings::AVP_GAMETYPE_DOMINATION)
	{
		// Sanity check 
		if (GetNumberOfTeams() != 2 && GetNumberOfTeams() != 3)
		{
			Asura_Assert( false, "Need two or three teams for a team based game, setting number of teams to 2" ); 
			SetNumberOfTeams(2); 
		}
		return true; 
	}
	else if (AVP_Settings::GetGameType() == AVP_Settings::AVP_GAMETYPE_DISRUPTION_DOMINATION)
	{
		// Sanity check 
		if (GetNumberOfTeams() != 3)
		{
			Asura_Assert( false, "Need three teams for a disruption domination game, setting number of teams to 3" ); 
			SetNumberOfTeams(3); 
		}
		return true; 
	}
	else
	{
		return (!IsRoleBasedGame() && GetNumberOfTeams() > 1);
	}
}

bool AVP_Multiplayer_GameRules::IsRoleBasedGame()
{
	return (AVP_Settings::GetGameType() == AVP_Settings::AVP_GAMETYPE_HUNT || 
			AVP_Settings::GetGameType() == AVP_Settings::AVP_GAMETYPE_SURVIVOR); 
}

bool AVP_Multiplayer_GameRules::IsDominationBasedGame()
{
	return (AVP_Settings::GetGameType() == AVP_Settings::AVP_GAMETYPE_DISRUPTION_DOMINATION || 
			AVP_Settings::GetGameType() == AVP_Settings::AVP_GAMETYPE_DOMINATION); 
}

////////////////////////////////////////////////////////////
//  Game rules system
////////////////////////////////////////////////////////////
void AVP_Multiplayer_GameRules::Initialise( void )
{
	// Read game rules from the blueprint (default to deathmatch)
	// This is an initialisation step that will be overriden by the server settings later on. It is useful to test stuff
	// and doing an early sanity check on game mode blueprints
	Asura_Hash_ID uBlueprintHashID = uAVP_BLUEPRINT_NAME_DEATH_MATCH; 
	Asura_Hash_ID uBlueprintTypeHashID = uAVP_BLUEPRINT_TYPE_MULTIPLAYER;
	Asura_Blueprint* pxBlueprint = Asura_Blueprint_System::GetBlueprintByHashID( uBlueprintTypeHashID, uBlueprintHashID );
    Asura_Assertf( pxBlueprint, ( "Asura_Entity::GetBlueprint - couldn't find blueprint %0X in type %0X", uBlueprintHashID, uBlueprintTypeHashID ) );
    
	if (pxBlueprint)
	{
		InitialiseFromBlueprint( *pxBlueprint ); 
	}
}

void AVP_Multiplayer_GameRules::AddCommandConsoleVars( void )
{
#ifdef ASURA_SUPPORT_COMMAND_CONSOLE
	if (!Asura_CommandConsole::VarExists(L"AVP.GameRules.NumberOfTeams"))
	{
		Asura_CommandConsole::AddVar(L"AVP.GameRules.NumberOfTeams", &s_iNumberOfTeams, 1, 3, Asura_CommandConsole::REMOVE_AT_SHUTDOWN);
	}
	if (!Asura_CommandConsole::VarExists(L"AVP.GameRules.MaxNumberOfPlayers"))
	{
		Asura_CommandConsole::AddVar(L"AVP.GameRules.MaxNumberOfPlayers", &s_iMaxNumberOfPlayers, 2, 18, Asura_CommandConsole::REMOVE_AT_SHUTDOWN);
	}
	if (!Asura_CommandConsole::VarExists(L"AVP.GameRules.TeamOneSpecies"))
	{
		void* pTeamOneSpecies = reinterpret_cast<void*>(&s_eTeamOneSpecies); 
		Asura_CommandConsole::AddVar(L"AVP.GameRules.TeamOneSpecies", (int*)pTeamOneSpecies, 0, AVP_Settings::AVP_NUM_SPECIES-1, Asura_CommandConsole::REMOVE_AT_SHUTDOWN);
	}
	if (!Asura_CommandConsole::VarExists(L"AVP.GameRules.TeamTwoSpecies"))
	{
		void* pTeamTwoSpecies = reinterpret_cast<void*>(&s_eTeamTwoSpecies);
		Asura_CommandConsole::AddVar(L"AVP.GameRules.TeamTwoSpecies", (int*)pTeamTwoSpecies, 0, AVP_Settings::AVP_NUM_SPECIES-1, Asura_CommandConsole::REMOVE_AT_SHUTDOWN);
	}
	if (!Asura_CommandConsole::VarExists(L"AVP.GameRules.TeamThreeSpecies"))
	{
		void* pTeamThreeSpecies = reinterpret_cast<void*>(&s_eTeamThreeSpecies);
		Asura_CommandConsole::AddVar(L"AVP.GameRules.TeamThreeSpecies", (int*)pTeamThreeSpecies, 0, AVP_Settings::AVP_NUM_SPECIES-1, Asura_CommandConsole::REMOVE_AT_SHUTDOWN);
	}
	if (!Asura_CommandConsole::VarExists(L"AVP.GameRules.MutuallyExclusiveTeams"))
	{
		Asura_CommandConsole::AddVar(L"AVP.GameRules.MutuallyExclusiveTeams", &s_bMutuallyExclusiveTeams, Asura_CommandConsole::REMOVE_AT_SHUTDOWN);
	}
	if (!Asura_CommandConsole::VarExists(L"AVP.GameRules.RespawnUponDeath"))
	{
		Asura_CommandConsole::AddVar(L"AVP.GameRules.RespawnUponDeath", &s_bRespawnUponDeath, Asura_CommandConsole::REMOVE_AT_SHUTDOWN);
	}
	if (!Asura_CommandConsole::VarExists(L"AVP.GameRules.RespawnType"))
	{
		void* pRespawnType = reinterpret_cast<void*>(&s_eRespawnType); 
		Asura_CommandConsole::AddVar(L"AVP.GameRules.RespawnType", (int*)pRespawnType, 0, AVP_Settings::AVP_NUM_RESPAWNTYPES-1, Asura_CommandConsole::REMOVE_AT_SHUTDOWN);
	}
	if (!Asura_CommandConsole::VarExists(L"AVP.GameRules.LivesAvailablePerTeam"))
	{
		Asura_CommandConsole::AddVar(L"AVP.GameRules.LivesAvailablePerTeam", &s_iLivesAvailablePerTeam, -1, 5, Asura_CommandConsole::REMOVE_AT_SHUTDOWN);
	}
	if (!Asura_CommandConsole::VarExists(L"AVP.GameRules.PlayerRatioAlien"))
	{
		Asura_CommandConsole::AddVar(L"AVP.GameRules.PlayerRatioAlien", &s_iPlayerRatioAlien, 1, 5, Asura_CommandConsole::REMOVE_AT_SHUTDOWN);
	}
	if (!Asura_CommandConsole::VarExists(L"AVP.GameRules.PlayerRatioHuman"))
	{
		Asura_CommandConsole::AddVar(L"AVP.GameRules.PlayerRatioHuman", &s_iPlayerRatioHuman, 1, 5, Asura_CommandConsole::REMOVE_AT_SHUTDOWN);
	}
	if (!Asura_CommandConsole::VarExists(L"AVP.GameRules.PlayerRatioPredator"))
	{
		Asura_CommandConsole::AddVar(L"AVP.GameRules.PlayerRatioPredator", &s_iPlayerRatioPredator, 1, 5, Asura_CommandConsole::REMOVE_AT_SHUTDOWN);
	}
	if (!Asura_CommandConsole::VarExists(L"AVP.GameRules.FriendlyFireOn"))
	{
		Asura_CommandConsole::AddVar(L"AVP.GameRules.FriendlyFireOn", &s_bFriendlyFireOn, Asura_CommandConsole::REMOVE_AT_SHUTDOWN);
	}
	if (!Asura_CommandConsole::VarExists(L"AVP.GameRules.PlayerScoreLimit"))
	{
		Asura_CommandConsole::AddVar(L"AVP.GameRules.PlayerScoreLimit", &s_iPlayerScoreLimit, -1, 0x7FFFFFFF, Asura_CommandConsole::REMOVE_AT_SHUTDOWN);
	}
	if (!Asura_CommandConsole::VarExists(L"AVP.GameRules.TeamScoreLimit"))
	{
		Asura_CommandConsole::AddVar(L"AVP.GameRules.TeamScoreLimit", &s_iTeamScoreLimit, -1, 0x7FFFFFFF, Asura_CommandConsole::REMOVE_AT_SHUTDOWN);
	}
	if (!Asura_CommandConsole::VarExists(L"AVP.GameRules.TimeLimit"))
	{
		Asura_CommandConsole::AddVar(L"AVP.GameRules.TimeLimit", &s_iTimeLimit, 0, 60, Asura_CommandConsole::REMOVE_AT_SHUTDOWN);
	}
	if (!Asura_CommandConsole::VarExists(L"AVP.GameRules.AllowMixedTeams"))
	{
		Asura_CommandConsole::AddVar(L"AVP.GameRules.AllowMixedTeams", &s_bAllowMixedTeams, Asura_CommandConsole::REMOVE_AT_SHUTDOWN);
	}
#endif //#ifdef ASURA_SUPPORT_COMMAND_CONSOLE
}

void AVP_Multiplayer_GameRules::Shutdown( void )
{
}

void AVP_Multiplayer_GameRules::Refresh( void )
{
	// Get the latest game mode
	AVP_Settings::GameType eGameType = AVP_Settings::GetGameType();
	
	// Find the blueprint for this game mode rules
	Asura_Hash_ID uBlueprintHashID = uAVP_BLUEPRINT_NAME_DEATH_MATCH; 
	switch( eGameType )
	{
		case AVP_Settings::AVP_GAMETYPE_DEATH_MATCH:
		{
			uBlueprintHashID = uAVP_BLUEPRINT_NAME_DEATH_MATCH; 
			break;
		}
		case AVP_Settings::AVP_GAMETYPE_DISRUPTION_DOMINATION:
		{
			uBlueprintHashID = uAVP_BLUEPRINT_NAME_DISRUPTION_DOMINATION; 
			break;
		}
		case AVP_Settings::AVP_GAMETYPE_HUNT:
		{
			uBlueprintHashID = uAVP_BLUEPRINT_NAME_HUNT;
			break;
		}
		case AVP_Settings::AVP_GAMETYPE_SPECIES_TEAM_DEATH_MATCH:
		{
			uBlueprintHashID = uAVP_BLUEPRINT_NAME_SPECIES_TEAM_DEATH_MATCH; 
			break;
		}
		case AVP_Settings::AVP_GAMETYPE_SURVIVOR:
		{
			uBlueprintHashID = uAVP_BLUEPRINT_NAME_SURVIVOR; 
			break;
		}
		case AVP_Settings::AVP_GAMETYPE_MIXED_SPECIES_TEAM_DEATH_MATCH:
		{
			uBlueprintHashID = uAVP_BLUEPRINT_NAME_MIXED_SPECIES_TEAM_DEATH_MATCH; 
			break;
		}
		case AVP_Settings::AVP_GAMETYPE_DOMINATION:
		{
			uBlueprintHashID = uAVP_BLUEPRINT_NAME_DOMINATION; 
			break;
		}
		default:
		{
			Asura_Assert( false, "Invalid multiplayer game type found when refreshing game rules, will default now to death match rules" );
			uBlueprintHashID = uAVP_BLUEPRINT_NAME_DEATH_MATCH;
			break;
		}
	}

	Asura_Hash_ID uBlueprintTypeHashID = uAVP_BLUEPRINT_TYPE_MULTIPLAYER;
	Asura_Blueprint* pxBlueprint = Asura_Blueprint_System::GetBlueprintByHashID( uBlueprintTypeHashID, uBlueprintHashID );
    Asura_Assertf( pxBlueprint, ( "Asura_Entity::GetBlueprint - couldn't find blueprint %0X in type %0X", uBlueprintHashID, uBlueprintTypeHashID ) );
    
	if (pxBlueprint)
	{
		InitialiseFromBlueprint( *pxBlueprint ); 
	}
}

void AVP_Multiplayer_GameRules::InitialiseFromBlueprint(const Asura_Blueprint &xBlueprint)
{
	// Spawn points
	{	
		Asura_Value xAISpawnPointsOn; 
		if ( xBlueprint.GetParameterValueByHashID( uAVP_BLUEPRINT_NAME_AI_SPAWN_POINTS_ON, xAISpawnPointsOn ))
		{
			SetAISpawnPointsOn( xAISpawnPointsOn.GetBoolValue() ); 
		}
		else
		{
			Asura_Assertf( false, ( "Couldn't find blueprint parameter %0X", uAVP_BLUEPRINT_NAME_AI_SPAWN_POINTS_ON )); 
		}
	}
	// Mixed teams
	{
		Asura_Value xAllowMixedTeams; 
		if ( xBlueprint.GetParameterValueByHashID( uAVP_BLUEPRINT_NAME_ALLOW_MIXED_TEAMS, xAllowMixedTeams ))
		{
			SetAllowMixedTeams( xAllowMixedTeams.GetBoolValue() ); 
		}
		else
		{
			Asura_Assertf( false, ( "Couldn't find blueprint parameter %0X", uAVP_BLUEPRINT_NAME_ALLOW_MIXED_TEAMS )); 
		}
	}
	// Capture points on
	{
		Asura_Value xCapturePointsOn; 
		if ( xBlueprint.GetParameterValueByHashID( uAVP_BLUEPRINT_NAME_CAPTURE_POINTS_ON, xCapturePointsOn ))
		{
			SetCapturePointsOn( xCapturePointsOn.GetBoolValue() ); 
		}
		else
		{
			Asura_Assertf( false, ( "Couldn't find blueprint parameter %0X", uAVP_BLUEPRINT_NAME_CAPTURE_POINTS_ON )); 
		}
	}
	// Capture the flag on
	{
		Asura_Value xCTFOn; 
		if ( xBlueprint.GetParameterValueByHashID( uAVP_BLUEPRINT_NAME_CTF_ON, xCTFOn ))
		{
			SetCTFOn( xCTFOn.GetBoolValue() ); 
		}
		else
		{
			Asura_Assertf( false, ( "Couldn't find blueprint parameter %0X", uAVP_BLUEPRINT_NAME_CTF_ON )); 
		}
	}
	// Friendly fire on
	{
		Asura_Value xFriendlyFireOn; 
		if ( xBlueprint.GetParameterValueByHashID( uAVP_BLUEPRINT_NAME_FRIENDLY_FIRE_ON, xFriendlyFireOn ))
		{
			SetFriendlyFireOn( xFriendlyFireOn.GetBoolValue() ); 
		}
		else
		{
			Asura_Assertf( false, ( "Couldn't find blueprint parameter %0X", uAVP_BLUEPRINT_NAME_FRIENDLY_FIRE_ON )); 
		}
	}
	// Hunter-Prey Force team change
	{
		Asura_Value xHPForceTeamChange; 
		if ( xBlueprint.GetParameterValueByHashID( uAVP_BLUEPRINT_NAME_HP_FORCE_TEAM_CHANGE, xHPForceTeamChange ))
		{
			SetHPForceTeamChange( xHPForceTeamChange.GetBoolValue() ); 
		}
		else
		{
			Asura_Assertf( false, ( "Couldn't find blueprint parameter %0X", uAVP_BLUEPRINT_NAME_HP_FORCE_TEAM_CHANGE )); 
		}
	}
	// Hunter-Prey Hunter kill prey join
	{
		Asura_Value xHPHunterKillPreyJoin; 
		if ( xBlueprint.GetParameterValueByHashID( uAVP_BLUEPRINT_NAME_HP_HUNTER_KILL_PREY_JOIN, xHPHunterKillPreyJoin ))
		{
			SetHPHunterKillPreyJoin( xHPHunterKillPreyJoin.GetBoolValue() ); 
		}
		else
		{
			Asura_Assertf( false, ( "Couldn't find blueprint parameter %0X", uAVP_BLUEPRINT_NAME_HP_HUNTER_KILL_PREY_JOIN )); 
		}
	}
	// Hunter-Prey Prey kill hunter switch
	{
		Asura_Value xHPPreyKillHunterSwitch; 
		if ( xBlueprint.GetParameterValueByHashID( uAVP_BLUEPRINT_NAME_HP_PREY_KILL_HUNTER_SWITCH, xHPPreyKillHunterSwitch ))
		{
			SetHPPreyKillHunterSwitch( xHPPreyKillHunterSwitch.GetBoolValue() ); 
		}
		else
		{
			Asura_Assertf( false, ( "Couldn't find blueprint parameter %0X", uAVP_BLUEPRINT_NAME_HP_PREY_KILL_HUNTER_SWITCH )); 
		}
	}
	// Lives available per team
	{
		Asura_Value xLives; 
		if ( xBlueprint.GetParameterValueByHashID( uAVP_BLUEPRINT_NAME_LIVES_AVAILABLE_PER_TEAM, xLives ))
		{
			SetLivesAvailablePerTeam( xLives.GetIntValue() ); 
		}
		else
		{
			Asura_Assertf( false, ( "Couldn't find blueprint parameter %0X", uAVP_BLUEPRINT_NAME_LIVES_AVAILABLE_PER_TEAM )); 
		}
	}
	// Max number of players
	{
		Asura_Value xMaxNumberOfPlayers; 
		if ( xBlueprint.GetParameterValueByHashID( uAVP_BLUEPRINT_NAME_MAX_NUMBER_OF_PLAYERS, xMaxNumberOfPlayers ))
		{
			SetMaxNumberOfPlayers( xMaxNumberOfPlayers.GetIntValue() ); 
		}
		else
		{
			Asura_Assertf( false, ( "Couldn't find blueprint parameter %0X", uAVP_BLUEPRINT_NAME_MAX_NUMBER_OF_PLAYERS )); 
		}
	}
	// Mutually exclusive teams
	{
		Asura_Value xMutuallyExclusiveTeams; 
		if ( xBlueprint.GetParameterValueByHashID( uAVP_BLUEPRINT_NAME_MUTUALLY_EXCLUSIVE_TEAMS, xMutuallyExclusiveTeams ))
		{
			SetMutuallyExclusiveTeams( xMutuallyExclusiveTeams.GetBoolValue() ); 
		}
		else
		{
			Asura_Assertf( false, ( "Couldn't find blueprint parameter %0X", uAVP_BLUEPRINT_NAME_MUTUALLY_EXCLUSIVE_TEAMS )); 
		}
	}
	// Number of teams
	{
		Asura_Value xNumberOfTeams; 
		if ( xBlueprint.GetParameterValueByHashID( uAVP_BLUEPRINT_NAME_NUMBER_OF_TEAMS, xNumberOfTeams ))
		{
			SetNumberOfTeams( xNumberOfTeams.GetIntValue() ); 
		}
		else
		{
			Asura_Assertf( false, ( "Couldn't find blueprint parameter %0X", uAVP_BLUEPRINT_NAME_NUMBER_OF_TEAMS )); 
		}
	}
	// Player ratio alien
	{
		Asura_Value xPlayerRatioAlien; 
		if ( xBlueprint.GetParameterValueByHashID( uAVP_BLUEPRINT_NAME_PLAYER_RATIO_ALIEN, xPlayerRatioAlien ))
		{
			SetPlayerRatioAlien( xPlayerRatioAlien.GetIntValue() ); 
		}
		else
		{
			Asura_Assertf( false, ( "Couldn't find blueprint parameter %0X", uAVP_BLUEPRINT_NAME_PLAYER_RATIO_ALIEN )); 
		}
	}
	// Player ratio human
	{
		Asura_Value xPlayerRatioHuman; 
		if ( xBlueprint.GetParameterValueByHashID( uAVP_BLUEPRINT_NAME_PLAYER_RATIO_HUMAN, xPlayerRatioHuman ))
		{
			SetPlayerRatioHuman( xPlayerRatioHuman.GetIntValue() ); 
		}
		else
		{
			Asura_Assertf( false, ( "Couldn't find blueprint parameter %0X", uAVP_BLUEPRINT_NAME_PLAYER_RATIO_HUMAN )); 
		}
	}
	// Player ratio human
	{
		Asura_Value xPlayerRatioPredator; 
		if ( xBlueprint.GetParameterValueByHashID( uAVP_BLUEPRINT_NAME_PLAYER_RATIO_PREDATOR, xPlayerRatioPredator ))
		{
			SetPlayerRatioPredator( xPlayerRatioPredator.GetIntValue() ); 
		}
		else
		{
			Asura_Assertf( false, ( "Couldn't find blueprint parameter %0X", uAVP_BLUEPRINT_NAME_PLAYER_RATIO_PREDATOR )); 
		}
	}
	// Player score capture control point
	{
		Asura_Value xPlayerScoreCaptureControlPoint; 
		if ( xBlueprint.GetParameterValueByHashID( uAVP_BLUEPRINT_NAME_PLAYER_SCORE_CAPTURE_CONTROL_POINT, xPlayerScoreCaptureControlPoint ))
		{
			SetPlayerScoreCaptureControlPoint( xPlayerScoreCaptureControlPoint.GetIntValue() ); 
		}
		else
		{
			Asura_Assertf( false, ( "Couldn't find blueprint parameter %0X", uAVP_BLUEPRINT_NAME_PLAYER_SCORE_CAPTURE_CONTROL_POINT )); 
		}
	}
	// Player score capture flag
	{
		Asura_Value xPlayerScoreCaptureFlag; 
		if ( xBlueprint.GetParameterValueByHashID( uAVP_BLUEPRINT_NAME_PLAYER_SCORE_CAPTURE_FLAG, xPlayerScoreCaptureFlag ))
		{
			SetPlayerScoreCaptureFlag( xPlayerScoreCaptureFlag.GetIntValue() ); 
		}
		else
		{
			Asura_Assertf( false, ( "Couldn't find blueprint parameter %0X", uAVP_BLUEPRINT_NAME_PLAYER_SCORE_CAPTURE_FLAG )); 
		}
	}
	// Player score defend control point bonus
	{
		Asura_Value xPlayerScoreDefendControlPointBonus; 
		if ( xBlueprint.GetParameterValueByHashID( uAVP_BLUEPRINT_NAME_PLAYER_SCORE_DEFEND_CONTROL_POINT_BONUS, xPlayerScoreDefendControlPointBonus ))
		{
			SetPlayerScoreDefendControlPointBonus( xPlayerScoreDefendControlPointBonus.GetIntValue() ); 
		}
		else
		{
			Asura_Assertf( false, ( "Couldn't find blueprint parameter %0X", uAVP_BLUEPRINT_NAME_PLAYER_SCORE_DEFEND_CONTROL_POINT_BONUS )); 
		}
	}
	// Player score hunter kill primary prey bonus
	{
		Asura_Value xPlayerScoreHunterKillPrimaryPreyBonus; 
		if ( xBlueprint.GetParameterValueByHashID( uAVP_BLUEPRINT_NAME_PLAYER_SCORE_HUNTER_KILL_PRIMARY_PREY_BONUS, xPlayerScoreHunterKillPrimaryPreyBonus ))
		{
			SetPlayerScoreHunterKillPrimaryPreyBonus( xPlayerScoreHunterKillPrimaryPreyBonus.GetIntValue() ); 
		}
		else
		{
			Asura_Assertf( false, ( "Couldn't find blueprint parameter %0X", uAVP_BLUEPRINT_NAME_PLAYER_SCORE_HUNTER_KILL_PRIMARY_PREY_BONUS )); 
		}
	}
	// Player score hunter kill secondary prey bonus
	{
		Asura_Value xPlayerScoreHunterKillSecondaryPreyBonus; 
		if ( xBlueprint.GetParameterValueByHashID( uAVP_BLUEPRINT_NAME_PLAYER_SCORE_HUNTER_KILL_SECONDARY_PREY_BONUS, xPlayerScoreHunterKillSecondaryPreyBonus ))
		{
			SetPlayerScoreHunterKillSecondaryPreyBonus( xPlayerScoreHunterKillSecondaryPreyBonus.GetIntValue() ); 
		}
		else
		{
			Asura_Assertf( false, ( "Couldn't find blueprint parameter %0X", uAVP_BLUEPRINT_NAME_PLAYER_SCORE_HUNTER_KILL_SECONDARY_PREY_BONUS )); 
		}
	}
	// Player score kill flag carrier bonus
	{
		Asura_Value xPlayerScoreKillFlagCarrierBonus; 
		if ( xBlueprint.GetParameterValueByHashID( uAVP_BLUEPRINT_NAME_PLAYER_SCORE_KILL_FLAG_CARRIER_BONUS, xPlayerScoreKillFlagCarrierBonus ))
		{
			SetPlayerScoreKillFlagCarrierBonus( xPlayerScoreKillFlagCarrierBonus.GetIntValue() ); 
		}
		else
		{
			Asura_Assertf( false, ( "Couldn't find blueprint parameter %0X", uAVP_BLUEPRINT_NAME_PLAYER_SCORE_KILL_FLAG_CARRIER_BONUS )); 
		}
	}
	// Player score prey kill hunter bonus
	{
		Asura_Value xPlayerScorePreyKillHunterBonus; 
		if ( xBlueprint.GetParameterValueByHashID( uAVP_BLUEPRINT_NAME_PLAYER_SCORE_PREY_KILL_HUNTER_BONUS, xPlayerScorePreyKillHunterBonus ))
		{
			SetPlayerScorePreyKillHunterBonus( xPlayerScorePreyKillHunterBonus.GetIntValue() ); 
		}
		else
		{
			Asura_Assertf( false, ( "Couldn't find blueprint parameter %0X", uAVP_BLUEPRINT_NAME_PLAYER_SCORE_PREY_KILL_HUNTER_BONUS )); 
		}
	}
	// Player score return flag bonus 
	{
		Asura_Value xPlayerScoreReturnFlagBonus; 
		if ( xBlueprint.GetParameterValueByHashID( uAVP_BLUEPRINT_NAME_PLAYER_SCORE_RETURN_FLAG_BONUS, xPlayerScoreReturnFlagBonus ))
		{
			SetPlayerScoreReturnFlagBonus( xPlayerScoreReturnFlagBonus.GetIntValue() ); 
		}
		else
		{
			Asura_Assertf( false, ( "Couldn't find blueprint parameter %0X", uAVP_BLUEPRINT_NAME_PLAYER_SCORE_RETURN_FLAG_BONUS )); 
		}
	}
	// Prey survivor multiplier first
	{
		Asura_Value xPreySurvivorMultiplierFirst; 
		if ( xBlueprint.GetParameterValueByHashID( uAVP_BLUEPRINT_NAME_PREY_SURVIVOR_MULTIPLIER_FIRST, xPreySurvivorMultiplierFirst ))
		{
			SetPreySurvivorMultiplierFirst( xPreySurvivorMultiplierFirst.GetFloatValue() ); 
		}
		else
		{
			Asura_Assertf( false, ( "Couldn't find blueprint parameter %0X", uAVP_BLUEPRINT_NAME_PREY_SURVIVOR_MULTIPLIER_FIRST )); 
		}
	}
	// Prey survivor multiplier second
	{
		Asura_Value xPreySurvivorMultiplierSecond; 
		if ( xBlueprint.GetParameterValueByHashID( uAVP_BLUEPRINT_NAME_PREY_SURVIVOR_MULTIPLIER_FIRST, xPreySurvivorMultiplierSecond ))
		{
			SetPreySurvivorMultiplierSecond( xPreySurvivorMultiplierSecond.GetFloatValue() ); 
		}
		else
		{
			Asura_Assertf( false, ( "Couldn't find blueprint parameter %0X", uAVP_BLUEPRINT_NAME_PREY_SURVIVOR_MULTIPLIER_FIRST )); 
		}
	}
	// Prey survivor multiplier third
	{
		Asura_Value xPreySurvivorMultiplierThird; 
		if ( xBlueprint.GetParameterValueByHashID( uAVP_BLUEPRINT_NAME_PREY_SURVIVOR_MULTIPLIER_FIRST, xPreySurvivorMultiplierThird ))
		{
			SetPreySurvivorMultiplierThird( xPreySurvivorMultiplierThird.GetFloatValue() ); 
		}
		else
		{
			Asura_Assertf( false, ( "Couldn't find blueprint parameter %0X", uAVP_BLUEPRINT_NAME_PREY_SURVIVOR_MULTIPLIER_FIRST )); 
		}
	}
	// Respawn - Use wave spawning
	{
		Asura_Value xRespawnUseWaveSpawning; 
		if ( xBlueprint.GetParameterValueByHashID( uAVP_BLUEPRINT_NAME_RESPAWN_USE_WAVE_SPAWNING, xRespawnUseWaveSpawning ))
		{
			SetUseWaveSpawning( xRespawnUseWaveSpawning.GetBoolValue() ); 
		}
		else
		{
			Asura_Assertf( false, ( "Couldn't find blueprint parameter %0X", uAVP_BLUEPRINT_NAME_RESPAWN_USE_WAVE_SPAWNING )); 
		}
	}
	// Respawn - Delay
	{
		Asura_Value xRespawnDelay; 
		if ( xBlueprint.GetParameterValueByHashID( uAVP_BLUEPRINT_NAME_RESPAWN_DELAY, xRespawnDelay ))
		{
			SetRespawnDelay( xRespawnDelay.GetFloatValue() ); 
		}
		else
		{
			Asura_Assertf( false, ( "Couldn't find blueprint parameter %0X", uAVP_BLUEPRINT_NAME_RESPAWN_DELAY )); 
		}
	}
	// Respawn - Type
	{
		Asura_Value xRespawnType; 
		if ( xBlueprint.GetParameterValueByHashID( uAVP_BLUEPRINT_NAME_RESPAWN_TYPE, xRespawnType ))
		{
			SetRespawnType( static_cast<AVP_Settings::RespawnType>(xRespawnType.GetIntValue()) ); 
		}
		else
		{
			Asura_Assertf( false, ( "Couldn't find blueprint parameter %0X", uAVP_BLUEPRINT_NAME_RESPAWN_TYPE )); 
		}
	}
	// Score limit
	{
		Asura_Value xScoreLimit; 
		if ( xBlueprint.GetParameterValueByHashID( uAVP_BLUEPRINT_NAME_SCORE_LIMIT, xScoreLimit ))
		{
			// TODO: offer this couple of values in blueprints or remove the redundant one altogether from game rules
			SetPlayerScoreLimit( xScoreLimit.GetIntValue() ); 
			SetTeamScoreLimit( xScoreLimit.GetIntValue() ); 
		}
		else
		{
			Asura_Assertf( false, ( "Couldn't find blueprint parameter %0X", uAVP_BLUEPRINT_NAME_SCORE_LIMIT )); 
		}
	}
	// Team 1 
	{
		Asura_Value xTeamOne; 
		if ( xBlueprint.GetParameterValueByHashID( uAVP_BLUEPRINT_NAME_TEAM_ONE, xTeamOne ))
		{
			Asura_Hash_ID uBlueprintTypeHashID = uAVP_BLUEPRINT_TYPE_MULTIPLAYER;
			Asura_Hash_ID uBlueprintHashID = xTeamOne.GetHashIDValue(); 
			Asura_Blueprint* pxTeamBlueprint = Asura_Blueprint_System::GetBlueprintByHashID( uBlueprintTypeHashID, uBlueprintHashID );

			if (pxTeamBlueprint)
			{
				Asura_Value xSpeciesTeamOne; 
				pxTeamBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_NAME_SPECIES, xSpeciesTeamOne ); 
				SetTeamOneSpecies( static_cast<AVP_Settings::Species>( xSpeciesTeamOne.GetIntValue() )); 
			}
			else
			{
				SetTeamOneSpecies( AVP_Settings::AVP_SPECIES_PREDATOR ); 	
			}
		}
		else
		{
			Asura_Assertf( false, ( "Couldn't find blueprint parameter %0X", uAVP_BLUEPRINT_NAME_TEAM_ONE )); 
		}
	}
	// Team 2
	{
		Asura_Value xTeamTwo; 
		if ( xBlueprint.GetParameterValueByHashID( uAVP_BLUEPRINT_NAME_TEAM_TWO, xTeamTwo ))
		{
			Asura_Hash_ID uBlueprintTypeHashID = uAVP_BLUEPRINT_TYPE_MULTIPLAYER;
			Asura_Hash_ID uBlueprintHashID = xTeamTwo.GetHashIDValue(); 
			Asura_Blueprint* pxTeamBlueprint = Asura_Blueprint_System::GetBlueprintByHashID( uBlueprintTypeHashID, uBlueprintHashID );

			if (pxTeamBlueprint)
			{
				Asura_Value xSpeciesTeamTwo; 
				pxTeamBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_NAME_SPECIES, xSpeciesTeamTwo ); 
				SetTeamTwoSpecies( static_cast<AVP_Settings::Species>( xSpeciesTeamTwo.GetIntValue() )); 
			}
			else
			{
				SetTeamTwoSpecies( AVP_Settings::AVP_SPECIES_ALIEN );
			}
		}
		else
		{
			Asura_Assertf( false, ( "Couldn't find blueprint parameter %0X", uAVP_BLUEPRINT_NAME_TEAM_TWO )); 
		}
	}
	// Team 3
	{
		Asura_Value xTeamThree; 
		if ( xBlueprint.GetParameterValueByHashID( uAVP_BLUEPRINT_NAME_TEAM_THREE, xTeamThree ))
		{
			Asura_Hash_ID uBlueprintTypeHashID = uAVP_BLUEPRINT_TYPE_MULTIPLAYER;
			Asura_Hash_ID uBlueprintHashID = xTeamThree.GetHashIDValue(); 
			Asura_Blueprint* pxTeamBlueprint = Asura_Blueprint_System::GetBlueprintByHashID( uBlueprintTypeHashID, uBlueprintHashID );

			if (pxTeamBlueprint)
			{
				Asura_Value xSpeciesTeamThree; 
				pxTeamBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_NAME_SPECIES, xSpeciesTeamThree ); 
				SetTeamThreeSpecies( static_cast<AVP_Settings::Species>( xSpeciesTeamThree.GetIntValue() )); 
			}
			else
			{
				SetTeamThreeSpecies( AVP_Settings::AVP_SPECIES_HUMAN );
			}
		}
		else
		{
			Asura_Assertf( false, ( "Couldn't find blueprint parameter %0X", uAVP_BLUEPRINT_NAME_TEAM_THREE )); 
		}
	}
	// Team lives on
	{
		Asura_Value xTeamLivesOn; 
		if ( xBlueprint.GetParameterValueByHashID( uAVP_BLUEPRINT_NAME_TEAM_LIVES_ON, xTeamLivesOn ))
		{
			SetTeamLivesOn( xTeamLivesOn.GetBoolValue() ); 
		}
		else
		{
			Asura_Assertf( false, ( "Couldn't find blueprint parameter %0X", uAVP_BLUEPRINT_NAME_TEAM_LIVES_ON )); 
		}
	}
	// Team score capture all enemies dead
	{
		Asura_Value xTeamScoreAllEnemiesDead; 
		if ( xBlueprint.GetParameterValueByHashID( uAVP_BLUEPRINT_NAME_TEAM_SCORE_ALL_ENEMIES_DEAD, xTeamScoreAllEnemiesDead ))
		{
			SetTeamScoreAllEnemiesDead( xTeamScoreAllEnemiesDead.GetIntValue() ); 
		}
		else
		{
			Asura_Assertf( false, ( "Couldn't find blueprint parameter %0X", uAVP_BLUEPRINT_NAME_TEAM_SCORE_ALL_ENEMIES_DEAD )); 
		}
	}
	// Team score capture all control points
	{
		Asura_Value xTeamScoreCaptureAllControlPoints; 
		if ( xBlueprint.GetParameterValueByHashID( uAVP_BLUEPRINT_NAME_TEAM_SCORE_CAPTURE_ALL_CONTROL_POINTS, xTeamScoreCaptureAllControlPoints ))
		{
			SetTeamScoreCaptureAllControlPoints( xTeamScoreCaptureAllControlPoints.GetIntValue() ); 
		}
		else
		{
			Asura_Assertf( false, ( "Couldn't find blueprint parameter %0X", uAVP_BLUEPRINT_NAME_TEAM_SCORE_CAPTURE_ALL_CONTROL_POINTS )); 
		}
	}
	// Team score hold control point
	{
		Asura_Value xTeamScoreHoldControlPoint; 
		if ( xBlueprint.GetParameterValueByHashID( uAVP_BLUEPRINT_NAME_TEAM_SCORE_HOLD_CONTROL_POINT, xTeamScoreHoldControlPoint ))
		{
			SetTeamScoreHoldControlPoint( xTeamScoreHoldControlPoint.GetIntValue() ); 
		}
		else
		{
			Asura_Assertf( false, ( "Couldn't find blueprint parameter %0X", uAVP_BLUEPRINT_NAME_TEAM_SCORE_HOLD_CONTROL_POINT )); 
		}
	}
	// Team score kills add to score
	{
		Asura_Value xTeamScoreKillsAddToScore; 
		if ( xBlueprint.GetParameterValueByHashID( uAVP_BLUEPRINT_NAME_TEAM_SCORE_KILLS_ADD_TO_SCORE, xTeamScoreKillsAddToScore ))
		{
			SetTeamScoreKillsAddToScore( xTeamScoreKillsAddToScore.GetBoolValue() ); 
		}
		else
		{
			Asura_Assertf( false, ( "Couldn't find blueprint parameter %0X", uAVP_BLUEPRINT_NAME_TEAM_SCORE_KILLS_ADD_TO_SCORE )); 
		}
	}
	// Team score team victory on
	{
		Asura_Value xTeamVictoryOn; 
		if ( xBlueprint.GetParameterValueByHashID( uAVP_BLUEPRINT_NAME_TEAM_VICTORY_ON, xTeamVictoryOn ))
		{
			SetTeamVictoryOn( xTeamVictoryOn.GetBoolValue() ); 
		}
		else
		{
			Asura_Assertf( false, ( "Couldn't find blueprint parameter %0X", uAVP_BLUEPRINT_NAME_TEAM_VICTORY_ON )); 
		}
	}
	// Time limit
	{
		Asura_Value xTimeLimit; 
		if ( xBlueprint.GetParameterValueByHashID( uAVP_BLUEPRINT_NAME_TIME_LIMIT, xTimeLimit ))
		{
			SetTimeLimit( xTimeLimit.GetIntValue() ); 
		}
		else
		{
			Asura_Assertf( false, ( "Couldn't find blueprint parameter %0X", uAVP_BLUEPRINT_NAME_TIME_LIMIT )); 
		}
	}
	// Persecutor mode on
	{
		Asura_Value xPersecutorModeOn; 
		if ( xBlueprint.GetParameterValueByHashID( uAVP_BLUEPRINT_NAME_PERSECUTOR_MODE_ON, xPersecutorModeOn ))
		{
			SetPersecutorModeOn( xPersecutorModeOn.GetBoolValue() ); 
		}
		else
		{
			Asura_Assertf( false, ( "Couldn't find blueprint parameter %0X", uAVP_BLUEPRINT_NAME_PERSECUTOR_MODE_ON )); 
		}
	}
	// Kills needed to persecute
	{
		Asura_Value xKillsNeededToPersecute; 
		if ( xBlueprint.GetParameterValueByHashID( uAVP_BLUEPRINT_NAME_PERSECUTOR_KILLS_NEEDED_TO_PERSECUTE, xKillsNeededToPersecute ))
		{
			SetKillsNeededToPersecute( xKillsNeededToPersecute.GetIntValue() ); 
		}
		else
		{
			Asura_Assertf( false, ( "Couldn't find blueprint parameter %0X", uAVP_BLUEPRINT_NAME_PERSECUTOR_KILLS_NEEDED_TO_PERSECUTE )); 
		}
	}
}


////////////////////////////////////////////////////////////////////////////////
//  CLASS    :  AVP_Multiplayer_GameRules
//  NAME     :  GetTeamSpeciesAreMutuallyExclusive()
//  CREATED  :  JG (25.11.2008)
//  DESC     :  Compare team species and returns if all of the relevant ones are 
//				distinct to each other
////////////////////////////////////////////////////////////////////////////////
bool AVP_Multiplayer_GameRules::GetTeamSpeciesAreMutuallyExclusive()
{ 
	bool bMutuallyExclusive = s_eTeamOneSpecies != s_eTeamTwoSpecies;
	if (bMutuallyExclusive && GetNumberOfTeams() > 2)
	{
		bMutuallyExclusive = (s_eTeamOneSpecies != s_eTeamThreeSpecies) &&
							 (s_eTeamTwoSpecies != s_eTeamThreeSpecies); 
	}
	return bMutuallyExclusive; 
}

//------------------------------------------------------------------------------
// Function     : AVP_Multiplayer_GameRules::GetValidPlayerSpecies
// Author       : Anthony Gregson
// Date         : 06/11/07
//------------------------------------------------------------------------------
AVP_Settings::Species AVP_Multiplayer_GameRules::GetValidPlayerSpecies(AVP_Settings::Species eRequestedSpecies, AVP_Settings::Team eMultiplayerTeam,
																	   AVP_Settings::Role eRole, bool bIsSinglePlayer)
{
	if (bIsSinglePlayer)
	{
		// Just return the single player species.
		return AVP_Settings::GetPlayerSpecies();
	}
	else
	{
		// Check for role based games
		if (AVP_Multiplayer_GameRules::IsRoleBasedGame())
		{
			if (eRole != AVP_Settings::AVP_ROLE_NORMAL)
			{
				AVP_Multiplayer_ScoreRules* pxScoreRules = AVP_Multiplayer_ScoreRules::GetScoreTable(eRequestedSpecies, AVP_Settings::GetGameType());
				if (pxScoreRules)
				{
					if (pxScoreRules->GetRole() == eRole)
					{
						return eRequestedSpecies; 
					}
					else
					{
						// Find the right species for this role
						AVP_Settings::Species eValidSpecies = AVP_Settings::AVP_SPECIES_UNKNOWN; 
						if ( AVP_Settings::GetGameType() == AVP_Settings::AVP_GAMETYPE_HUNT )
						{
							// By default, Hunter is Predator and Prey is Human
							eValidSpecies = ( eRole == AVP_Settings::AVP_ROLE_HUNTER ) ? AVP_Settings::AVP_SPECIES_PREDATOR : AVP_Settings::AVP_SPECIES_HUMAN;
						}
						else if ( AVP_Settings::GetGameType() == AVP_Settings::AVP_GAMETYPE_SURVIVOR )
						{
							// In Survivor, Hunter is Alien and Prey is Human
							eValidSpecies = ( eRole == AVP_Settings::AVP_ROLE_HUNTER ) ? AVP_Settings::AVP_SPECIES_ALIEN : AVP_Settings::AVP_SPECIES_HUMAN;
						}

						// Find the right species for this role
						for (int i = 0; i < AVP_Settings::AVP_NUM_SPECIES; ++i)
						{
							AVP_Settings::Species eCurrentSpecies = static_cast<AVP_Settings::Species>( i ); 
							AVP_Multiplayer_ScoreRules* pxScoreRules = AVP_Multiplayer_ScoreRules::GetScoreTable(eCurrentSpecies, AVP_Settings::GetGameType());
							if (pxScoreRules)
							{
								if (pxScoreRules->GetRole() == eRole)
								{
									// Found it, this is a valid Species to play this role
									eValidSpecies = eCurrentSpecies; 
									break; 
								}
							}
							else
							{
								Asura_Assert( false, "Couldn't find game score table" ); 
							}
						}
						
						// Either default or according to the game rules, there should be a valid Species here now
						Asura_Assert( eValidSpecies != AVP_Settings::AVP_SPECIES_UNKNOWN, "Unexpected species returned from GetValidPlayerSpecies"); 
						return eValidSpecies; 
					}
				}
				else
				{
					Asura_Assert( false, "Couldn't find game score table" ); 
				}
			}
			else
			{
				Asura_Assert(false, "Unexpected multiplayer role"); 
				return eRequestedSpecies; 
			}
		}

		// Check for species team based games
		if (AVP_Multiplayer_GameRules::IsTeamBasedGame())
		{
			// Can the player choose its own species?
			if (!AVP_Multiplayer_GameRules::GetAllowMixedTeams())
			{
				// No, the species are determined by the team
				switch (eMultiplayerTeam)
				{
					case AVP_Settings::AVP_TEAM_ONE:
					{
						return AVP_Multiplayer_GameRules::GetTeamOneSpecies(); 
					}
					case AVP_Settings::AVP_TEAM_TWO:
					{
						return AVP_Multiplayer_GameRules::GetTeamTwoSpecies(); 
					}
					case AVP_Settings::AVP_TEAM_THREE:
					{
						return AVP_Multiplayer_GameRules::GetTeamThreeSpecies(); 
					}
					case AVP_Settings::AVP_TEAM_NONE:
					default:
					{
						Asura_Assert( false, "Invalid team requested while spawning a new actor" ); 
						break; 
					}
				}
			}
		}
		return eRequestedSpecies;
	}
}
/// End of function 'AVP_Multiplayer_GameRules::GetValidPlayerSpecies'. ///

////////////////////////////////////////////////////////////
//  Player species ratio
////////////////////////////////////////////////////////////

// Simple calculations for three mutually exclusive species or a mixed team
float AVP_Multiplayer_GameRules::CalculateSpeciesRatio( int iPlayerSpeciesRatio )
{
	int iTotalPlayersInRatio =	GetPlayerRatioAlien() + GetPlayerRatioHuman() + GetPlayerRatioPredator();
	float fSpeciesPlayersInRatio = 1.0f;
	if (iTotalPlayersInRatio > 0)
	{
		fSpeciesPlayersInRatio = iPlayerSpeciesRatio / static_cast<float>( iTotalPlayersInRatio ); 
	}
	else
	{
		Asura_Assert(false, "All ratios for species are zero, at least one should be positive");
	}
	Asura_Assert(fSpeciesPlayersInRatio >= 0.0f, "Negative player species ratio!"); 
	Asura_Assert(fSpeciesPlayersInRatio <= 1.0f, "Denormalised player species ratio"); 
	return fSpeciesPlayersInRatio; 
}

int AVP_Multiplayer_GameRules::CalculateMaxNumberOfAlienPlayers(int iNumberOfPlayers)
{
	int iMaxNumberOfAlienPlayers = Asura_Maths::FloatToInt_Ceil( CalculateSpeciesRatio( GetPlayerRatioAlien() ) * iNumberOfPlayers );
	return iMaxNumberOfAlienPlayers; 
}

int AVP_Multiplayer_GameRules::CalculateMaxNumberOfPredatorPlayers(int iNumberOfPlayers)
{
	int iMaxNumberOfPredatorPlayers = Asura_Maths::FloatToInt_Ceil( CalculateSpeciesRatio( GetPlayerRatioPredator() ) * iNumberOfPlayers );
	return iMaxNumberOfPredatorPlayers; 
}

int AVP_Multiplayer_GameRules::CalculateMaxNumberOfHumanPlayers(int iNumberOfPlayers)
{
	int iMaxNumberOfHumanPlayers = Asura_Maths::FloatToInt_Ceil( CalculateSpeciesRatio( GetPlayerRatioHuman() ) * iNumberOfPlayers );
	return iMaxNumberOfHumanPlayers; 
}

// Species per team when team species are not mutually exclusive
int AVP_Multiplayer_GameRules::GetPlayerRatioBySpecies( AVP_Settings::Species eSpecies )
{
	switch (eSpecies)
	{
		case AVP_Settings::AVP_SPECIES_ALIEN:
		{
			return GetPlayerRatioAlien(); 
		}
		case AVP_Settings::AVP_SPECIES_HUMAN:
		{
			return GetPlayerRatioHuman(); 
		}
		case AVP_Settings::AVP_SPECIES_PREDATOR:
		{
			return GetPlayerRatioPredator(); 
		}
		default:
		{
			Asura_Assert( false, "Invalid species ratio requested" ); 
			return 0; 
		}
	}

}

int AVP_Multiplayer_GameRules::GetPlayerRatioByTeam( AVP_Settings::Team eTeam )
{
	switch (eTeam)
	{
		case AVP_Settings::AVP_TEAM_ONE:
		{
			return GetPlayerRatioBySpecies( GetTeamOneSpecies() ); 
		}
		case AVP_Settings::AVP_TEAM_TWO:
		{
			return GetPlayerRatioBySpecies( GetTeamTwoSpecies() ); 
		}
		case AVP_Settings::AVP_TEAM_THREE:
		{
			return GetPlayerRatioBySpecies( GetTeamThreeSpecies() ); 
		}
		default:
		{
			Asura_Assert( false, "Invalid team requested" ); 
			return 0; 
		}
	}
}

float AVP_Multiplayer_GameRules::CalculateSpeciesRatioInTeam( AVP_Settings::Species eSpecies, AVP_Settings::Team eTeam )
{
	int iNumberOfTeams = AVP_Multiplayer_GameRules::GetNumberOfTeams(); 
	Asura_Assert( iNumberOfTeams > 1, "Unexpected number of teams calculating species ratio" );
	
	int iTotalPlayersInRatio =	GetPlayerRatioByTeam( AVP_Settings::AVP_TEAM_ONE ) +
								GetPlayerRatioByTeam( AVP_Settings::AVP_TEAM_TWO );

	if (iNumberOfTeams == 3)
	{
		iTotalPlayersInRatio += GetPlayerRatioByTeam( AVP_Settings::AVP_TEAM_THREE );
	}
	float fSpeciesPlayersInRatio = 1.0f;
	if (iTotalPlayersInRatio > 0)
	{
		fSpeciesPlayersInRatio = GetPlayerRatioBySpecies( eSpecies ) / static_cast<float>( iTotalPlayersInRatio ); 
	}
	else
	{
		Asura_Assert(false, "All ratios for species are zero, at least one should be positive");
	}
	Asura_Assert(fSpeciesPlayersInRatio >= 0.0f, "Negative player species ratio!"); 
	Asura_Assert(fSpeciesPlayersInRatio <= 1.0f, "Denormalised player species ratio"); 
	return fSpeciesPlayersInRatio; 
}

int AVP_Multiplayer_GameRules::CalculateMaxNumberOfPlayerSpeciesInTeam(int iNumberOfPlayersInTeam, AVP_Settings::Species eSpecies, AVP_Settings::Team eTeam)
{
	int iMaxNumberOfPlayers = Asura_Maths::FloatToInt_Ceil( CalculateSpeciesRatioInTeam( eSpecies, eTeam  ) * iNumberOfPlayersInTeam );
	return iMaxNumberOfPlayers; 
}

int AVP_Multiplayer_GameRules::CalculateMaxNumberOfPlayersInTeam(int iMaxPlayers, int iNumberOfTeams, AVP_Settings::Team eTeam)
{
	if (iNumberOfTeams <= 0)
	{
		return iMaxPlayers; 
	}

	int iNumberOfPlayersPerTeam = iMaxPlayers / iNumberOfTeams; 
	int iRemainingPlayers = iMaxPlayers % iNumberOfTeams; 
	if (eTeam == AVP_Settings::AVP_TEAM_NONE)
	{
		iNumberOfPlayersPerTeam += iRemainingPlayers ? 1 : 0; 
	}
	else
	{
		// TODO: Calculate a different number of players per team
		if (iRemainingPlayers)
		{
			iNumberOfPlayersPerTeam += 1; 
		}
	}
	return iNumberOfPlayersPerTeam; 
}

// end of file
