//////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_Multiplayer_GameRules.h
// Author : Anthony Gregson
// Date   : 04/12/07
//
// Copyright (c) 2007 Rebellion. All rights reserved.
//
// A static storage point for the constant (i.e. unchanging in a single game)
// settings for an AVP game. Set from local settings or across the network if
// joining a game (AVP_NetworkSession should include all the data we need).
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef ___AVP_MULTIPLAYER_GAMERULES_H___
#define ___AVP_MULTIPLAYER_GAMERULES_H___ 1

// I N C L U D E S ///////////////////////////////////////////////////////////
#include "Core/Asura.h"

#include "Blueprint/Asura_Blueprint.h"
#include "Core/AVP_Settings.h"

// C O N S T A N T S /////////////////////////////////////////////////////////

// C L A S S E S /////////////////////////////////////////////////////////////

// Class implementation 
class AVP_Multiplayer_GameRules
{ 
public:
	static void Initialise(void); 
	static void Shutdown(void); 
	static void Refresh(void); 
	static void AddCommandConsoleVars(void);

	// TODO - these functions probably need sensible bounds checking, but since we don't have values for them yet...
	static void SetPlayerScoreLimit(int iLimit)							{ s_iPlayerScoreLimit = iLimit; }
	static void SetTeamScoreLimit(int iLimit)							{ s_iTeamScoreLimit = iLimit; }
	static void SetAISpawnPointsOn(bool bAISpawnPointsOn)				{ s_bAISpawnPointsOn = bAISpawnPointsOn; }
	static void SetAllowMixedTeams(bool bAllowMixedTeams)				{ s_bAllowMixedTeams = bAllowMixedTeams; }
	static void SetCapturePointsOn(bool bCapturePointsOn)				{ s_bCapturePointsOn = bCapturePointsOn; }
	static void SetCTFOn(bool bCTFOn)									{ s_bCTFOn = bCTFOn; }
	static void SetFriendlyFireOn(bool bFriendlyFireOn)					{ s_bFriendlyFireOn = bFriendlyFireOn; }
	static void SetHPForceTeamChange(bool bHPForceTeamChange)			{ s_bHPForceTeamChange = bHPForceTeamChange; }
	static void SetHPHunterKillPreyJoin(bool bHPHunterKillPreyJoin)		{ s_bHPHunterKillPreyJoin = bHPHunterKillPreyJoin; }
	static void SetHPPreyKillHunterSwitch(bool bHPPreyKillHunterSwitch) { s_bHPPreyKillHunterSwitch = bHPPreyKillHunterSwitch; }

	static void SetLivesAvailablePerTeam(int iLives)					{ s_iLivesAvailablePerTeam = iLives; }
	static void SetMaxNumberOfPlayers(int iMaxNumberOfPlayers)			{ s_iMaxNumberOfPlayers = iMaxNumberOfPlayers; }
	static void SetMutuallyExclusiveTeams(bool bMutuallyExclusive)		{ s_bMutuallyExclusiveTeams = bMutuallyExclusive; }
	static void SetNumberOfTeams(int iNumberOfTeams)					{ s_iNumberOfTeams = iNumberOfTeams; }
	
	static void SetPlayerRatioAlien(int iPlayerRatioAlien)				{ s_iPlayerRatioAlien = iPlayerRatioAlien; }
	static void SetPlayerRatioHuman(int iPlayerRatioHuman)				{ s_iPlayerRatioHuman = iPlayerRatioHuman; }
	static void SetPlayerRatioPredator(int iPlayerRatioPredator)		{ s_iPlayerRatioPredator = iPlayerRatioPredator; }
	
	static void SetPlayerScoreCaptureControlPoint(int iPlayerScoreCaptureControlPoint)						{ s_iPlayerScoreCaptureControlPoint = iPlayerScoreCaptureControlPoint; }
	static void SetPlayerScoreCaptureFlag(int iPlayerScoreCaptureFlag)										{ s_iPlayerScoreCaptureFlag = iPlayerScoreCaptureFlag; }
	static void SetPlayerScoreDefendControlPointBonus(int iPlayerScoreDefendControlPointBonus)				{ s_iPlayerScoreDefendControlPointBonus = iPlayerScoreDefendControlPointBonus; }
	static void SetPlayerScoreHunterKillPrimaryPreyBonus(int iPlayerScoreHunterKillPrimaryPreyBonus)		{ s_iPlayerScoreHunterKillPrimaryPreyBonus = iPlayerScoreHunterKillPrimaryPreyBonus; }
	static void SetPlayerScoreHunterKillSecondaryPreyBonus(int iPlayerScoreHunterKillSecondaryPreyBonus)	{ s_iPlayerScoreHunterKillSecondaryPreyBonus = iPlayerScoreHunterKillSecondaryPreyBonus; }
	static void SetPlayerScoreKillFlagCarrierBonus(int iPlayerScoreKillFlagCarrierBonus)					{ s_iPlayerScoreKillFlagCarrierBonus = iPlayerScoreKillFlagCarrierBonus; }
	static void SetPlayerScorePreyKillHunterBonus(int iPlayerScorePreyKillHunterBonus)						{ s_iPlayerScorePreyKillHunterBonus = iPlayerScorePreyKillHunterBonus; }
	static void SetPlayerScoreReturnFlagBonus(int iPlayerScoreReturnFlagBonus)								{ s_iPlayerScoreReturnFlagBonus = iPlayerScoreReturnFlagBonus; }
	
	static void SetPreySurvivorMultiplierFirst(float fPreySurvivorMultiplierFirst)			{ s_fPreySurvivorMultiplierFirst = fPreySurvivorMultiplierFirst; }
	static void SetPreySurvivorMultiplierSecond(float fPreySurvivorMultiplierSecond)		{ s_fPreySurvivorMultiplierSecond = fPreySurvivorMultiplierSecond; }
	static void SetPreySurvivorMultiplierThird(float fPreySurvivorMultiplierThird)			{ s_fPreySurvivorMultiplierThird = fPreySurvivorMultiplierThird; }
	
	static void SetUseWaveSpawning(bool bUseWaveSpawning)				{ s_bUseWaveSpawning = bUseWaveSpawning; }
	static void SetRespawnDelay(float fRespawnDelay)					{ s_fRespawnDelay = fRespawnDelay; }
	static void SetRespawnUponDeath(bool bRespawnUponDeath)				{ s_bRespawnUponDeath = bRespawnUponDeath; }
	static void SetRespawnType(AVP_Settings::RespawnType eRespawnType)	{ s_eRespawnType = eRespawnType; }

	static void SetTeamOneSpecies(AVP_Settings::Species eTeamOneSpecies)		{ s_eTeamOneSpecies = eTeamOneSpecies; } 
	static void SetTeamTwoSpecies(AVP_Settings::Species eTeamTwoSpecies)		{ s_eTeamTwoSpecies = eTeamTwoSpecies; }
	static void SetTeamThreeSpecies(AVP_Settings::Species eTeamThreeSpecies)	{ s_eTeamThreeSpecies = eTeamThreeSpecies; }
	static void SetTeamLivesOn(bool bTeamLivesOn)								{ s_bTeamLivesOn = bTeamLivesOn; }
	static void SetTeamScoreAllEnemiesDead(int iTeamScoreAllEnemiesDead)									{ s_iTeamScoreAllEnemiesDead = iTeamScoreAllEnemiesDead; }
	static void SetTeamScoreCaptureAllControlPoints(int iTeamScoreCaptureAllControlPoints)					{ s_iTeamScoreCaptureAllControlPoints = iTeamScoreCaptureAllControlPoints; }
	static void SetTeamScoreHoldControlPoint(int iTeamScoreHoldControlPoint)								{ s_iTeamScoreHoldControlPoint = iTeamScoreHoldControlPoint; }
	static void SetTeamScoreKillsAddToScore(bool bTeamScoreKillsAddToScore)									{ s_bTeamScoreKillsAddToScore = bTeamScoreKillsAddToScore; }
	static void SetTeamVictoryOn(bool bTeamVictoryOn)														{ s_bTeamVictoryOn = bTeamVictoryOn; }
	
	static void SetTimeLimit(int iTimeLimit)								{ s_iTimeLimit = iTimeLimit; }

	static void SetPersecutorModeOn(bool bPersecutorModeOn)					{ s_bPersecutorModeOn = bPersecutorModeOn; }
	static void SetKillsNeededToPersecute(int iKills)						{ s_iKillsNeededToPersecute = iKills; }

	static void SetDDOMCaptureRadius(float fTime)								{ s_fDDOMCaptureRadius = fTime; }

	static int GetPlayerScoreLimit()	{ return s_iPlayerScoreLimit; }
	static int GetTeamScoreLimit()		{ return s_iTeamScoreLimit; }

	static bool GetAISpawnPointsOn()				{ return s_bAISpawnPointsOn; }
	static bool GetAllowMixedTeams()				{ return s_bAllowMixedTeams; }
	static bool GetCapturePointsOn()				{ return s_bCapturePointsOn; }
	static bool GetCTFOn()							{ return s_bCTFOn; }
	static bool GetFriendlyFireOn()					{ return s_bFriendlyFireOn; }
	static bool GetHPForceTeamChange()				{ return s_bHPForceTeamChange; }
	static bool GetHPHunterKillPreyJoin()			{ return s_bHPHunterKillPreyJoin; }
	static bool GetHPPreyKillHunterSwitch()			{ return s_bHPPreyKillHunterSwitch; }

	static int GetLivesAvailablePerTeam()			{ return s_iLivesAvailablePerTeam; }
	static int GetMaxNumberOfPlayers()				{ return s_iMaxNumberOfPlayers; }
	static bool GetMutuallyExclusiveTeams()			{ return s_bMutuallyExclusiveTeams; }
	static int GetNumberOfTeams()					{ return s_iNumberOfTeams; }
	
	static int GetPlayerRatioAlien()				{ return s_iPlayerRatioAlien; }
	static int GetPlayerRatioHuman()				{ return s_iPlayerRatioHuman; }
	static int GetPlayerRatioPredator()				{ return s_iPlayerRatioPredator; }
	
	static int GetPlayerScoreCaptureControlPoint()					{ return s_iPlayerScoreCaptureControlPoint; }
	static int GetPlayerScoreCaptureFlag()							{ return s_iPlayerScoreCaptureFlag; }
	static int GetPlayerScoreDefendControlPointBonus()				{ return s_iPlayerScoreDefendControlPointBonus; }
	static int GetPlayerScoreHunterKillPrimaryPreyBonus()			{ return s_iPlayerScoreHunterKillPrimaryPreyBonus; }
	static int GetPlayerScoreHunterKillSecondaryPreyBonus()			{ return s_iPlayerScoreHunterKillSecondaryPreyBonus; }
	static int GetPlayerScoreKillFlagCarrierBonus()					{ return s_iPlayerScoreKillFlagCarrierBonus; }
	static int GetPlayerScorePreyKillHunterBonus()					{ return s_iPlayerScorePreyKillHunterBonus; }
	static int GetPlayerScoreReturnFlagBonus()						{ return s_iPlayerScoreReturnFlagBonus; }
	
	static float GetPreySurvivorMultiplierFirst()			{ return s_fPreySurvivorMultiplierFirst; }
	static float GetPreySurvivorMultiplierSecond()			{ return s_fPreySurvivorMultiplierSecond; }
	static float GetPreySurvivorMultiplierThird()			{ return s_fPreySurvivorMultiplierThird; }
	
	static bool GetUseWaveSpawning()				{ return s_bUseWaveSpawning; }
	static float GetRespawnDelay()					{ return s_fRespawnDelay; }
	static bool GetRespawnUponDeath()				{ return s_bRespawnUponDeath; }
	static AVP_Settings::RespawnType GetRespawnType()		{ return s_eRespawnType; }

	static AVP_Settings::Species GetTeamOneSpecies()		{ return s_eTeamOneSpecies; } 
	static AVP_Settings::Species GetTeamTwoSpecies()		{ return s_eTeamTwoSpecies; }
	static AVP_Settings::Species GetTeamThreeSpecies()		{ return s_eTeamThreeSpecies; }
	static bool GetTeamLivesOn()							{ return s_bTeamLivesOn; }
	static int GetTeamScoreAllEnemiesDead()					{ return s_iTeamScoreAllEnemiesDead; }
	static int GetTeamScoreCaptureAllControlPoints()		{ return s_iTeamScoreCaptureAllControlPoints; }
	static int GetTeamScoreHoldControlPoint()				{ return s_iTeamScoreHoldControlPoint; }
	static bool GetTeamScoreKillsAddToScore()				{ return s_bTeamScoreKillsAddToScore; }
	static bool GetTeamVictoryOn()							{ return s_bTeamVictoryOn; }
	
	static int GetTimeLimit()								{ return s_iTimeLimit; }

	static bool GetPersecutorModeOn()						{ return s_bPersecutorModeOn; }
	static int GetKillsNeededToPersecute()					{ return s_iKillsNeededToPersecute; }

	static float GetDDOMCaptureRadius()						{ return s_fDDOMCaptureRadius; }

	// Complex rules: based in one or more of the above settings
	static bool IsTeamBasedGame();
	static bool IsRoleBasedGame(); 
	static bool IsDominationBasedGame(); 
	static bool GetAllowTeamSelection()						{ return true; }
	static bool GetTeamSpeciesAreMutuallyExclusive();		

	static AVP_Settings::Species GetValidPlayerSpecies(AVP_Settings::Species eRequestedSpecies, AVP_Settings::Team eMultiplayerTeam, AVP_Settings::Role eRole, bool bIsSingleGame);

	static float CalculateSpeciesRatio( int iPlayerSpeciesRatio );
	static int CalculateMaxNumberOfPredatorPlayers( int iNumberOfPlayers ); 
	static int CalculateMaxNumberOfAlienPlayers( int iNumberOfPlayers ); 
	static int CalculateMaxNumberOfHumanPlayers( int iNumberOfPlayers ); 
	
	static int GetPlayerRatioBySpecies( AVP_Settings::Species eSpecies );
	static int GetPlayerRatioByTeam( AVP_Settings::Team eTeam );
	static float CalculateSpeciesRatioInTeam( AVP_Settings::Species eSpecies, AVP_Settings::Team eTeam );
	static int CalculateMaxNumberOfPlayerSpeciesInTeam( int iNumberOfPlayers, AVP_Settings::Species eSpecies, AVP_Settings::Team eTeam );

	static int CalculateMaxNumberOfPlayersInTeam( int iMaxPlayers, int iNumberOfTeams, AVP_Settings::Team eTeam ); 

private:
	static void InitialiseFromBlueprint(const Asura_Blueprint& xBlueprint); 

	// Includes both single- and multi-player games, and not every piece of following data is relevant to all game modes.
	static int s_iPlayerScoreLimit;
	static int s_iTeamScoreLimit;

	static bool 	s_bAISpawnPointsOn;
	static bool 	s_bAllowMixedTeams;
	static bool 	s_bCapturePointsOn;
	static bool 	s_bCTFOn;
	static bool 	s_bFriendlyFireOn;
	static bool 	s_bHPForceTeamChange;
	static bool 	s_bHPHunterKillPreyJoin;
	static bool 	s_bHPPreyKillHunterSwitch;
	static int		s_iLivesAvailablePerTeam;
	static int		s_iMaxNumberOfPlayers; 
	static bool		s_bMutuallyExclusiveTeams;
	static int		s_iNumberOfTeams; 
	static int		s_iPlayerRatioAlien;
	static int		s_iPlayerRatioHuman;
	static int		s_iPlayerRatioPredator;
	
	static int 		s_iPlayerScoreCaptureControlPoint;
	static int 		s_iPlayerScoreCaptureFlag;
	static int 		s_iPlayerScoreDefendControlPointBonus;
	static int 		s_iPlayerScoreHunterKillPrimaryPreyBonus;
	static int 		s_iPlayerScoreHunterKillSecondaryPreyBonus;
	static int 		s_iPlayerScoreKillFlagCarrierBonus;
	static int 		s_iPlayerScorePreyKillHunterBonus;
	static int 		s_iPlayerScoreReturnFlagBonus;
	
	static float 	s_fPreySurvivorMultiplierFirst;
	static float 	s_fPreySurvivorMultiplierSecond;
	static float 	s_fPreySurvivorMultiplierThird;
	
	static bool		s_bUseWaveSpawning;
	static float	s_fRespawnDelay;
	static bool		s_bRespawnUponDeath; 
	static AVP_Settings::RespawnType	s_eRespawnType; 

	static AVP_Settings::Species		s_eTeamOneSpecies; 
	static AVP_Settings::Species		s_eTeamTwoSpecies;
	static AVP_Settings::Species		s_eTeamThreeSpecies;

	static bool		s_bTeamLivesOn;
	static int		s_iTeamScoreAllEnemiesDead;
	static int		s_iTeamScoreCaptureAllControlPoints;
	static int		s_iTeamScoreHoldControlPoint;
	static bool		s_bTeamScoreKillsAddToScore;
	static bool		s_bTeamVictoryOn;
	
	static int		s_iTimeLimit;

	static bool		s_bPersecutorModeOn; 
	static int		s_iKillsNeededToPersecute; 

	static float	s_fDDOMCaptureRadius; 

};

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

#endif // #ifndef ___AVP_MULTIPLAYER_GAMERULES_H___
