
//----------------------------------------------------------------------------------------
//////////////////////////////////////////////////////////////////////////////////////////
//  CLASS    :  AVP_Multiplayer_ScoreBoard
//  CREATED  :  JG (29.5.2008)
//  DESC     :  Shows statistics about multiplayer games
//////////////////////////////////////////////////////////////////////////////////////////
//----------------------------------------------------------------------------------------

////////////////////////////////////////////////////////////
//  Include files
////////////////////////////////////////////////////////////

// main asura header
#include "Core/Asura.h"

// local header
#include "Multiplayer/AVP_Multiplayer_ScoreBoard.h"

// independent header files
#if !defined(ASURA_NO_DEBUG_OUTPUT)
#include "Debugging/Asura_DebugText.h"
#endif // !defined(ASURA_NO_DEBUG_OUTPUT)
#include "GUIMenu/Asura_GUIMenu_Widget_Group.h"
#include "GUIMenu/Asura_GUIMenu_Widget_ListBox.h"
#include "GUIMenu/Asura_GUIMenu_Element_Image.h"
#include "GUIMenu/Asura_GUIMenu_Element_Text.h"
#include "Interface/Asura_CommandConsole.h"
#include "Network/Asura_Network.h"
#if defined ASURA_SUPPORT_LOBBY
#include "Network/Online/Asura_Lobby.h"
#endif //#if defined ASURA_SUPPORT_LOBBY
#include "Rendering/Asura_Font_System.h"
#include "Rendering/Asura_Viewport.h"

// game header files
#include "Blueprint/AVP_Blueprint_HashIDs.h"
#include "Core/AVP_Game_Core.h"
#include "Entity/PhysicalObject/Actor/AVP_ClientEntity_Actor.h"
#include "Entity/PlayerInfo/AVP_ServerEntity_PlayerInfo.h"
#include "Multiplayer/AVP_Multiplayer_GameMessageSystem.h"
#include "Multiplayer/AVP_Multiplayer_GameRules.h"
#include "Multiplayer/AVP_Multiplayer_HashIDs.h"
#include "Multiplayer/AVP_Multiplayer_TeamScore.h"
#include "Network/AVP_Player.h"

// data
bool AVP_Multiplayer_ScoreBoard::s_bRenderDebugInfo = false;
bool AVP_Multiplayer_ScoreBoard::s_bRenderScoreBoard = false;
bool AVP_Multiplayer_ScoreBoard::s_bUseLargeTable = false;
bool AVP_Multiplayer_ScoreBoard::s_b3TeamsPlaying = false;
bool AVP_Multiplayer_ScoreBoard::s_bTeamBased = false;
bool AVP_Multiplayer_ScoreBoard::s_bSpeciesToggle = false;
Asura_List<AVP_ClientEntity_PlayerInfo> AVP_Multiplayer_ScoreBoard::m_axPlayerInfoSortedLists[AVP_Settings::AVP_NUM_TEAMS]; 

////////////////////////////////////////////////////////////////////////////////
//  NAME     :  AVP_Multiplayer_ScoreBoard()
//  CREATED  :  JG (29.5.2008)
//  DESC     :  Constructor
////////////////////////////////////////////////////////////////////////////////
AVP_Multiplayer_ScoreBoard::AVP_Multiplayer_ScoreBoard()
{
}

////////////////////////////////////////////////////////////////////////////////
//  NAME     :  ~AVP_Multiplayer_ScoreBoard()
//  CREATED  :  JG (29.5.2008)
//  DESC     :  Destructor
////////////////////////////////////////////////////////////////////////////////
AVP_Multiplayer_ScoreBoard::~AVP_Multiplayer_ScoreBoard()
{
}


////////////////////////////////////////////////////////////////////////////////
//  CLASS    :  AVP_Multiplayer_ScoreBoard
//  NAME     :  Initialise()
//  CREATED  :  JG (29.5.2008)
//  DESC     :  Initialise the score board system
////////////////////////////////////////////////////////////////////////////////
void AVP_Multiplayer_ScoreBoard::Initialise()
{

#ifdef ASURA_SUPPORT_COMMAND_CONSOLE
	// Add toggle to the console for debug info
	if ( !Asura_CommandConsole::VarExists( L"Multiplayer.RenderDebugInfo" ) )
	{
		Asura_CommandConsole::AddVar( L"Multiplayer.RenderDebugInfo",  &s_bRenderDebugInfo, Asura_CommandConsole::REMOVE_AT_SHUTDOWN  );
	}
	// Same for score board control on debug builds
	if ( !Asura_CommandConsole::VarExists( L"Multiplayer.RenderScoreBoard" ) )
	{
		Asura_CommandConsole::AddVar( L"Multiplayer.RenderScoreBoard",  &s_bRenderScoreBoard, Asura_CommandConsole::REMOVE_AT_SHUTDOWN  );
	}
#endif //#ifdef ASURA_SUPPORT_COMMAND_CONSOLE	
}

DEBUGVAR float dbg_uConnectionWarningTimeMS = 10000;

////////////////////////////////////////////////////////////////////////////////
//  CLASS    :  AVP_Multiplayer_ScoreBoard
//  NAME     :  InitialiseScoreBoardHudCommands()
//  CREATED  :  CAH (17.8.2008)
//	SCOPE	 :	Puplic
//  DESC     :  Adds all of the scoreboard HUD access commands, 
//				needs a seperate initialise because the Asura menu tool does not initialise the multiplayer functionality, but these must be accessed from it to make the HUD
////////////////////////////////////////////////////////////////////////////////

void AVP_Multiplayer_ScoreBoard::InitialiseScoreBoardHudCommands()
{
	Asura_CommandConsole::AddCmd( L"Multiplayer.CMD.PopulateScoreTeam0",	ConsoleCmd_PopulateScoreTeam0,	Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::AddCmd( L"Multiplayer.CMD.PopulateScoreTeam1",	ConsoleCmd_PopulateScoreTeam1,	Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::AddCmd( L"Multiplayer.CMD.PopulateScoreTeam2",	ConsoleCmd_PopulateScoreTeam2,	Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::AddCmd( L"Multiplayer.CMD.PopulateScoreTeam3",	ConsoleCmd_PopulateScoreTeam3,	Asura_CommandConsole::REMOVE_AT_SHUTDOWN );

	Asura_CommandConsole::AddCmd( L"Multiplayer.CMD.PopulateNamesTeam0",	ConsoleCmd_PopulateNameTeam0,	Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::AddCmd( L"Multiplayer.CMD.PopulateNamesTeam1",	ConsoleCmd_PopulateNameTeam1,	Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::AddCmd( L"Multiplayer.CMD.PopulateNamesTeam2",	ConsoleCmd_PopulateNameTeam2,	Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::AddCmd( L"Multiplayer.CMD.PopulateNamesTeam3",	ConsoleCmd_PopulateNameTeam3,	Asura_CommandConsole::REMOVE_AT_SHUTDOWN );

	Asura_CommandConsole::AddCmd( L"Multiplayer.CMD.PopulateKillsTeam0",	ConsoleCmd_PopulateKillsTeam0,	Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::AddCmd( L"Multiplayer.CMD.PopulateKillsTeam1",	ConsoleCmd_PopulateKillsTeam1,	Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::AddCmd( L"Multiplayer.CMD.PopulateKillsTeam2",	ConsoleCmd_PopulateKillsTeam2,	Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::AddCmd( L"Multiplayer.CMD.PopulateKillsTeam3",	ConsoleCmd_PopulateKillsTeam3,	Asura_CommandConsole::REMOVE_AT_SHUTDOWN );

	Asura_CommandConsole::AddCmd( L"Multiplayer.CMD.PopulateDeathsTeam0",	ConsoleCmd_PopulateDeathsTeam0,	Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::AddCmd( L"Multiplayer.CMD.PopulateDeathsTeam1",	ConsoleCmd_PopulateDeathsTeam1,	Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::AddCmd( L"Multiplayer.CMD.PopulateDeathsTeam2",	ConsoleCmd_PopulateDeathsTeam2,	Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::AddCmd( L"Multiplayer.CMD.PopulateDeathsTeam3",	ConsoleCmd_PopulateDeathsTeam3,	Asura_CommandConsole::REMOVE_AT_SHUTDOWN );

	Asura_CommandConsole::AddCmd( L"Multiplayer.CMD.PopulateStreakTeam0",	ConsoleCmd_PopulateStreakTeam0,	Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::AddCmd( L"Multiplayer.CMD.PopulateStreakTeam1",	ConsoleCmd_PopulateStreakTeam1,	Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::AddCmd( L"Multiplayer.CMD.PopulateStreakTeam2",	ConsoleCmd_PopulateStreakTeam2,	Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::AddCmd( L"Multiplayer.CMD.PopulateStreakTeam3",	ConsoleCmd_PopulateStreakTeam3,	Asura_CommandConsole::REMOVE_AT_SHUTDOWN );

	Asura_CommandConsole::AddCmd( L"Multiplayer.CMD.PopulateRatioTeam0",	ConsoleCmd_PopulateRatioTeam0,	Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::AddCmd( L"Multiplayer.CMD.PopulateRatioTeam1",	ConsoleCmd_PopulateRatioTeam1,	Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::AddCmd( L"Multiplayer.CMD.PopulateRatioTeam2",	ConsoleCmd_PopulateRatioTeam2,	Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::AddCmd( L"Multiplayer.CMD.PopulateRatioTeam3",	ConsoleCmd_PopulateRatioTeam3,	Asura_CommandConsole::REMOVE_AT_SHUTDOWN );

	Asura_CommandConsole::AddCmd( L"Multiplayer.CMD.PopulateTeamScore",		ConsoleCmd_PopulateTeamScore,	Asura_CommandConsole::REMOVE_AT_SHUTDOWN );

	Asura_CommandConsole::AddCmd( L"Multiplayer.CMD.CloseResultScreen",		DeactivateResultScreens,		Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::AddCmd( L"Multiplayer.CMD.OpenResultScreen",		ActivateResultScreens,			Asura_CommandConsole::REMOVE_AT_SHUTDOWN );

	Asura_CommandConsole::AddVar( L"Multiplayer.VAR.3TeamsPlaying",			&s_b3TeamsPlaying,				Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::AddVar( L"Multiplayer.VAR.HalfOrMorePlaying",		&s_bUseLargeTable,				Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::AddVar( L"Multiplayer.VAR.TeamBased",				&s_bTeamBased,					Asura_CommandConsole::REMOVE_AT_SHUTDOWN );

	Asura_CommandConsole::AddCmd( L"Multiplayer.CMD.VisibilityAlienTeam0",		ConsoleCmd_VisibleAlienTeam0,		Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::AddCmd( L"Multiplayer.CMD.VisibilityAlienTeam1",		ConsoleCmd_VisibleAlienTeam1,		Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::AddCmd( L"Multiplayer.CMD.VisibilityAlienTeam2",		ConsoleCmd_VisibleAlienTeam2,		Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::AddCmd( L"Multiplayer.CMD.VisibilityAlienTeam3",		ConsoleCmd_VisibleAlienTeam3,		Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	
	Asura_CommandConsole::AddCmd( L"Multiplayer.CMD.VisibilityMarineTeam0",		ConsoleCmd_VisibleMarineTeam0,		Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::AddCmd( L"Multiplayer.CMD.VisibilityMarineTeam1",		ConsoleCmd_VisibleMarineTeam1,		Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::AddCmd( L"Multiplayer.CMD.VisibilityMarineTeam2",		ConsoleCmd_VisibleMarineTeam2,		Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::AddCmd( L"Multiplayer.CMD.VisibilityMarineTeam3",		ConsoleCmd_VisibleMarineTeam3,		Asura_CommandConsole::REMOVE_AT_SHUTDOWN );

	Asura_CommandConsole::AddCmd( L"Multiplayer.CMD.VisibilityPredatorTeam0",	ConsoleCmd_VisiblePredatorTeam0,	Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::AddCmd( L"Multiplayer.CMD.VisibilityPredatorTeam1",	ConsoleCmd_VisiblePredatorTeam1,	Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::AddCmd( L"Multiplayer.CMD.VisibilityPredatorTeam2",	ConsoleCmd_VisiblePredatorTeam2,	Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::AddCmd( L"Multiplayer.CMD.VisibilityPredatorTeam3",	ConsoleCmd_VisiblePredatorTeam3,	Asura_CommandConsole::REMOVE_AT_SHUTDOWN );

	Asura_CommandConsole::AddCmd( L"Multiplayer.CMD.VisibilityTalkingTeam0",	ConsoleCmd_VisibleTalkingTeam0,		Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::AddCmd( L"Multiplayer.CMD.VisibilityTalkingTeam1",	ConsoleCmd_VisibleTalkingTeam1,		Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::AddCmd( L"Multiplayer.CMD.VisibilityTalkingTeam2",	ConsoleCmd_VisibleTalkingTeam2,		Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::AddCmd( L"Multiplayer.CMD.VisibilityTalkingTeam3",	ConsoleCmd_VisibleTalkingTeam3,		Asura_CommandConsole::REMOVE_AT_SHUTDOWN );

	Asura_CommandConsole::AddCmd( L"Multiplayer.CMD.VisibilityHighlightTeam0",	ConsoleCmd_VisibleHighlightTeam0,	Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::AddCmd( L"Multiplayer.CMD.VisibilityHighlightTeam1",	ConsoleCmd_VisibleHighlightTeam1,	Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::AddCmd( L"Multiplayer.CMD.VisibilityHighlightTeam2",	ConsoleCmd_VisibleHighlightTeam2,	Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::AddCmd( L"Multiplayer.CMD.VisibilityHighlightTeam3",	ConsoleCmd_VisibleHighlightTeam3,	Asura_CommandConsole::REMOVE_AT_SHUTDOWN );

	////////////////////////////////////////////////////
	// CAH Temp population function for the pseudo leaderboards. 
	Asura_CommandConsole::AddCmd( L"Multiplayer.CMD.PopulateLEADERBOARD",	ConsoleCmd_PopulateLeaderBoard,	Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::AddVar( L"Multiplayer.VAR.LEADERBOARDTempVar",	&s_bSpeciesToggle,				Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	
}

////////////////////////////////////////////////////////////////////////////////
//  CLASS    :  AVP_Multiplayer_ScoreBoard
//  NAME     :  Update()
//  CREATED  :  CAH (17.8.2008)
//	SCOPE	 :	Public
//  DESC     :  Updates all of the HUD elements that need it frame by frame. 
//				In particular the listboxs that keep the scoreboard dynamic information.
////////////////////////////////////////////////////////////////////////////////

void AVP_Multiplayer_ScoreBoard::Update()
{
	// update all of the data used by the hud elements. 
	UpdatePlayerInfo();

	// uListBoxHashID must be u_int ( as opposed to "AVP_GUIMenu_System::STORED_WIDGETS" ) so it can be incremented without overloads
	u_int uListBoxHashID;
	Asura_GUIMenu_Root* pxListBoxItem = NULL;

	// team mased games and free for all have different score boards so determine correct update path first
	if( AVP_Multiplayer_GameRules::IsTeamBasedGame() )
	{
		int uCounter;
		
		// update the number of teams playing. 
		if( m_axPlayerInfoSortedLists[AVP_Settings::AVP_TEAM_THREE].IsEmpty() )
		{
			s_b3TeamsPlaying = false;
		}
		else
		{
			s_b3TeamsPlaying = true;
		}

		// update the third team background panel. (We update instead of render because it has a display condition set in the tool to render or not automatically.)
		uListBoxHashID = AVP_GUIMenu_System::MULTIPLAYER_SCOREBOARD_TEAM3_BG;
		pxListBoxItem = AVP_GUIMenu_System::GetHUDElement( static_cast<AVP_GUIMenu_System::STORED_WIDGETS>( uListBoxHashID ), s_uAVP_MULTIPLAYER_SCOREBOARD_TEAM, false );

		if( pxListBoxItem )
		{
			// if we found the item cast it to the correct menu widget (Image) to unlock its specific functionality from here onwards. 
			Asura_GUIMenu_Element_Image* pxImage = static_cast<Asura_GUIMenu_Element_Image*>( pxListBoxItem );
			if ( pxImage )
			{
				// CAH removed functionality until design has been finalised more. 
				//pxImage->Update( Asura_Timers::GetServerAbsoluteTime() );
			}
			else
			{
				Asura_Assert( true, "The team 3 background image can not be found so its cant be drawn, check the Hash IDs for it are correct" );
			}
		}
		

		// update the team score listBox
		uListBoxHashID = AVP_GUIMenu_System::MULTIPLAYER_LISTBOX_TEAMSCORES;
		pxListBoxItem = AVP_GUIMenu_System::GetHUDElement( static_cast<AVP_GUIMenu_System::STORED_WIDGETS>( uListBoxHashID ), s_uAVP_MULTIPLAYER_SCOREBOARD_TEAM, false );

		if( pxListBoxItem )
		{
			// if we found the item cast it to the correct menu widget (ListBox) to unlock its specific functionality from here onwards. 
			Asura_GUIMenu_Widget_ListBox* pxListBox = static_cast<Asura_GUIMenu_Widget_ListBox*>( pxListBoxItem );

			if ( pxListBox )
			{
				pxListBox->Update( Asura_Timers::GetServerAbsoluteTime() );
			}
			else
			{
				Asura_Assert( true, "The team score listbox can not be found, scores can not be drawn, check The hash IDS are correct" );
			}
		}

		// now update the players lists for each team 
		uListBoxHashID = AVP_GUIMenu_System::MULTIPLAYER_LISTBOX_TEAM1;

		for( uCounter = 1; uCounter < AVP_Settings::AVP_NUM_TEAMS; uCounter ++ )
		{
			// get a pointer to the correct team list box, relies on the team ID enums being sequencial
			pxListBoxItem = AVP_GUIMenu_System::GetHUDElement( static_cast<AVP_GUIMenu_System::STORED_WIDGETS>( uListBoxHashID ), s_uAVP_MULTIPLAYER_SCOREBOARD_TEAM, false );

			if( pxListBoxItem )
			{
				// if we found the item cast it to the correct menu widget (ListBox) to unlock its specific functionality from here onwards. 
				Asura_GUIMenu_Widget_ListBox* pxListBox = static_cast<Asura_GUIMenu_Widget_ListBox*>( pxListBoxItem );

				if ( pxListBox )
				{
					pxListBox->Update( Asura_Timers::GetServerAbsoluteTime() );
				}
				else
				{
					Asura_Assertf( true, ( "The List Box for Team %d Can not be found, information for members of this team can not be drawn, check that its hash IDs are correct.", uCounter ) );
				}

			}

			// increment the hashID enum ID so that the next iteration the next team list will be updated. 
			uListBoxHashID ++;
		}
	}
	else // else this is a NON team based game
	{
		// if there are more than half of the allowed players raise flag to draw full board.
		if( static_cast<u_int>(m_axPlayerInfoSortedLists[AVP_Settings::AVP_TEAM_NONE].GetNumEntries()) >= ( uAVP_MAX_NET_PLAYERS_MAX / 2 ) )
		{
			s_bUseLargeTable = true;
		}
		else
		{
			s_bUseLargeTable = false;
		}

		// update the lower half background panel. (We update instead of render because it has a display condition set in the tool to render or not automatically.)
		uListBoxHashID = AVP_GUIMenu_System::MULTIPLAYER_SCOREBOARD_NOTEAM_BG;
		pxListBoxItem = AVP_GUIMenu_System::GetHUDElement( static_cast<AVP_GUIMenu_System::STORED_WIDGETS>( uListBoxHashID ), s_uAVP_MULTIPLAYER_SCOREBOARD_NON_TEAM, false );

		if( pxListBoxItem )
		{
			// if we found the item cast it to the correct menu widget (Image) to unlock its specific functionality from here onwards. 
			Asura_GUIMenu_Element_Image* pxImage = static_cast<Asura_GUIMenu_Element_Image*>( pxListBoxItem );
			if ( pxImage )
			{
				pxImage->Update( Asura_Timers::GetServerAbsoluteTime() );
			}
			else
			{
				Asura_Assert( true, "The lower background image can not be found so its cant be drawn, check the Hash IDs for it are correct" );
			}
		}

		// now update the player list
		uListBoxHashID = AVP_GUIMenu_System::MULTIPLAYER_LISTBOX_TEAM0;
		pxListBoxItem = AVP_GUIMenu_System::GetHUDElement( static_cast<AVP_GUIMenu_System::STORED_WIDGETS>( uListBoxHashID ), s_uAVP_MULTIPLAYER_SCOREBOARD_NON_TEAM, false );

		if( pxListBoxItem )
		{
			// if we found the item cast it to the correct menu widget (ListBox) to unlock its specific functionality from here onwards. 
			Asura_GUIMenu_Widget_ListBox* pxListBox = static_cast<Asura_GUIMenu_Widget_ListBox*>( pxListBoxItem );

			if ( pxListBox )
			{
				pxListBox->Update( Asura_Timers::GetServerAbsoluteTime() );
			}
			else
			{
				Asura_Assert( true, "The List Box Can not be found, information for members of this game can not be drawn to scoreboard, check that its hash IDs are correct." );
			}
		}
	}
}

////////////////////////////////////////////////////////////////////////////////
//  CLASS    :  AVP_Multiplayer_ScoreBoard
//  NAME     :  UpdatePlayerInfo()
//  CREATED  :  CAH (17.8.2008)
//	SCOPE	 :	Private
//  DESC     :  Updates player team lists and individual's data. 
//				All the information that is needed to populate and display the elements of the Scoreboard List Boxes.
////////////////////////////////////////////////////////////////////////////////

void AVP_Multiplayer_ScoreBoard::UpdatePlayerInfo()
{
	int iTeamIndex;
	u_int uCounter;
	Asura_Guid uLocalPlayerGuid = ASURA_GUID_UNREGISTERED; 
	const Asura_Hashed_List<AVP_ClientEntity_PlayerInfo>* pxPlayerInfoList = AVP_ClientEntity_PlayerInfo::GetList();
	Asura_Hashed_List_It<AVP_ClientEntity_PlayerInfo> xPlayerInfoListIt( pxPlayerInfoList ); 

	// clear the information from the last frame from the lists or the new info will just be added below it. 
	for( uCounter = 0; uCounter < AVP_Settings::AVP_NUM_TEAMS; uCounter ++ )
	{
		m_axPlayerInfoSortedLists[uCounter].ClearList();
	}

	// now we iterate through all of the current players ordering them. and setting HUD image display conditions for them 
	for ( ; !xPlayerInfoListIt.Done(); xPlayerInfoListIt.Next() )
	{
		AVP_ClientEntity_PlayerInfo* pxPlayerInfo = xPlayerInfoListIt.GetCurrent();

		iTeamIndex = pxPlayerInfo->GetMultiplayerTeam();

		if( ( iTeamIndex >= 0 ) && ( iTeamIndex < AVP_Settings::AVP_NUM_TEAMS ) )
		{
			// insert the player into lists defined by thier team, ordered by thier score within thier that team. 
			m_axPlayerInfoSortedLists[iTeamIndex].InsertInList( pxPlayerInfo, AVP_Multiplayer_ScoreBoard::SortByScore ); 

			// Also keep a global sorted list for rankings
			if (iTeamIndex > 0)
			{
				m_axPlayerInfoSortedLists[AVP_Settings::AVP_TEAM_NONE].InsertInList( pxPlayerInfo, AVP_Multiplayer_ScoreBoard::SortByScore ); 
		}
		}
		else
		{
			Asura_Assert( false, "Player has a team index outside of the score array index" ) ;
		}

		if ( AVP_Game_Core::IsLocalPlayer( pxPlayerInfo->GetGuid() ) )
		{
			// if this is the local player record thier Hash id for highlights later on. 
			uLocalPlayerGuid = pxPlayerInfo->GetGuid(); 
		}
	}

	// Update the rank 
	// Find the local player in the global sorted list
	Asura_List_It<AVP_ClientEntity_PlayerInfo> xPlayerInfoSortedListIt( m_axPlayerInfoSortedLists[AVP_Settings::AVP_TEAM_NONE] ); 
	int iRank = 1; 
	AVP_ClientEntity_PlayerInfo* pxLocalPlayerInfo = NULL; 
	for ( ; !xPlayerInfoSortedListIt.Done(); xPlayerInfoSortedListIt.Next() )
	{
		if ( xPlayerInfoSortedListIt.GetCurrent()->GetGuid() == uLocalPlayerGuid )
		{
			// Found it, this is our rank compared to other people score
			// We will need the local player info later on
			pxLocalPlayerInfo = xPlayerInfoSortedListIt.GetCurrent(); 
			break; 
		}
		else
		{
			++iRank; 
		}
	}


	AVP_Multiplayer_GameMessageSystem::UpdateRankPlayers( pxPlayerInfoList->GetNumEntries() ); 
	AVP_Multiplayer_GameMessageSystem::UpdateRankPlace( iRank ); 

	if (pxLocalPlayerInfo)
	{
		AVP_Multiplayer_GameMessageSystem::UpdateTeamScore( AVP_Settings::AVP_TEAM_ONE, pxLocalPlayerInfo->GetPlayerScore().GetTeamOneScore() ); 
		AVP_Multiplayer_GameMessageSystem::UpdateTeamScore( AVP_Settings::AVP_TEAM_TWO, pxLocalPlayerInfo->GetPlayerScore().GetTeamTwoScore() ); 
		AVP_Multiplayer_GameMessageSystem::UpdateTeamScore( AVP_Settings::AVP_TEAM_THREE, pxLocalPlayerInfo->GetPlayerScore().GetTeamThreeScore() ); 
	}
	else
	{
		Asura_Assert( AVP_Settings::GetNumberOfLocalPlayers() == 0, "Couldn't find the local player info in the sorted list, all player scores will probably be wrong" ); 
	}

}

////////////////////////////////////////////////////////////////////////////////
//  CLASS    :  AVP_Multiplayer_ScoreBoard
//  NAME     :  RenderDebugInfo()
//  CREATED  :  JG (29.5.2008)
//  DESC     :  Shows on screen multiplayer debug info
////////////////////////////////////////////////////////////////////////////////
void AVP_Multiplayer_ScoreBoard::RenderDebugInfo()
{
#if !defined(ASURA_NO_DEBUG_OUTPUT)
	// Only render debug info if is relevant
	if ( AVP_Settings::IsMultiPlayer() )
	{
		if (s_bRenderDebugInfo)
		{
			float fOffsetY = 100.0f;
			const float fSPACEY = 20.0f; 
			u_int uOffsetY = 45; 
			
			const Asura_Hashed_List<AVP_ClientEntity_PlayerInfo>* pxPlayerInfoList = AVP_ClientEntity_PlayerInfo::GetList();

			// Sort by player score. Also we find the local player and nemesis (if any)
			Asura_Guid uLocalPlayerGuid = ASURA_GUID_UNREGISTERED; 
			Asura_Guid uPersecutorGuid = ASURA_GUID_UNREGISTERED; 

			Asura_List<AVP_ClientEntity_PlayerInfo> xPlayerInfoSortedList; 
			Asura_Hashed_List_It<AVP_ClientEntity_PlayerInfo> xPlayerInfoListIt( pxPlayerInfoList ); 
			while ( !xPlayerInfoListIt.Done() )
			{
				AVP_ClientEntity_PlayerInfo* pxPlayerInfo = xPlayerInfoListIt.GetCurrent();
				if ( AVP_Game_Core::IsLocalPlayer( pxPlayerInfo->GetGuid() ))
				{
					uLocalPlayerGuid = pxPlayerInfo->GetGuid(); 
					uPersecutorGuid = pxPlayerInfo->GetPersecutorGuid(); 
				}
				xPlayerInfoSortedList.InsertInList( pxPlayerInfo, AVP_Multiplayer_ScoreBoard::SortByScore ); 
				xPlayerInfoListIt.Next();
			}

			// Some statistics
			Asura_List_ConstIt<AVP_ClientEntity_PlayerInfo> xPlayerInfoSortedListConstIt( xPlayerInfoSortedList ); 
			while ( !xPlayerInfoSortedListConstIt.Done() )
			{
				const AVP_ClientEntity_PlayerInfo* pxPlayerInfo = xPlayerInfoSortedListConstIt.GetCurrent();

				if (pxPlayerInfo)
				{
					Asura_Guid uPlayerGuid = pxPlayerInfo->GetGuid(); 
					Asura_Guid uPuppetGuid = pxPlayerInfo->GetPuppetGuid(); 
					int iPlayerTeam = pxPlayerInfo->GetMultiplayerTeam(); 
					const AVP_Multiplayer_PlayerScore& xPlayerScore = pxPlayerInfo->GetPlayerScore();
					u_int uKills = xPlayerScore.GetNumberOfKills(); 
					u_int uKillStreak = xPlayerScore.GetKillStreak(); 
					u_int uAssists = xPlayerScore.GetNumberOfKillAssists(); 
					u_int uDeaths = xPlayerScore.GetNumberOfTimesKilled(); 
					int	iPlayerScore = xPlayerScore.GetIndividualScore(); 				
					int iTeamScore = 0; 
					int iPlayerRole = pxPlayerInfo->GetMultiplayerRole(); 
					iTeamScore = GetBasicTeamScore( pxPlayerInfo->GetMultiplayerTeam() ); 
					// Different colour for the local player and persecutor
					u_int uPrintColour = 0xFFC0C0C0; 
					if (uPlayerGuid == uLocalPlayerGuid)
					{
						uPrintColour = 0xFFFFFFFF; 
					}
					if (uPlayerGuid == uPersecutorGuid)
					{
						uPrintColour = 0xFFC00000;
					}
					
					// Voice channels setup
					u_char ucVoiceChannels = 0;
#if defined ASURA_SUPPORT_LOBBY
					const Asura_Lobby* pxLobby = Asura_Lobby_System::GetLobby();
					if (pxLobby)
					{
						Asura_Lobby_Player* pxPlayer = pxLobby->FindPlayer( pxPlayerInfo->GetGuid() );
						Asura_Assertf(pxPlayer,("Could not find player %d in lobby", pxPlayerInfo->GetGuid()));
						if (pxPlayer)
						{
							ucVoiceChannels = pxPlayer->GetVoiceChannels();
						}
					}
#endif // #if defined ASURA_SUPPORT_LOBBY

					Asura_Font_System::Print( 0.5f, fOffsetY, uPrintColour, ASURA_TEXTFORMAT_LEFTJUSTIFIED | ASURA_TEXTFORMAT_FORMATSTRING, 
						"%10S / Puppet: %7d / Team: %1d / Role: %1d / Kills(Streak): %2d(%2d) / Assists: %2d / Deaths: %2d / Score(Team): %3d(%3d) Channels: %02X", 
						pxPlayerInfo->GetPlayerName(), uPuppetGuid, iPlayerTeam, iPlayerRole, uKills, uKillStreak, uAssists, uDeaths, iPlayerScore, iTeamScore, ucVoiceChannels ); 
					fOffsetY += fSPACEY; 
					++uOffsetY;
				}
				else
				{
					Asura_Assert(pxPlayerInfo, "Invalid player information pointer"); 
				}

				xPlayerInfoSortedListConstIt.Next(); 
			}
		}

#if defined(ASURA_SUPPORT_EXTERNAL_NETWORK)
		if(Asura_Network::IsNetworkInitialised() && (Asura_NetTransport_System::GetIdleTime() > dbg_uConnectionWarningTimeMS))
		{
			Asura_Font_System::Print( 0.5f, 0.6f, 0xffff5555, ASURA_TEXTFORMAT_CENTREJUSTIFIED | ASURA_TEXTFORMAT_VERTICALCENTREJUSTIFIED | ASURA_TEXTFORMAT_RELATIVEPOSITION, "Warning: connection issues." ); 
		}
#endif	//defined(ASURA_SUPPORT_EXTERNAL_NETWORK)
	}
#endif // !defined(ASURA_NO_DEBUG_OUTPUT)
}

////////////////////////////////////////////////////////////////////////////////
//  CLASS    :  AVP_Multiplayer_ScoreBoard
//  NAME     :  Render()
//  CREATED  :  CAH (15.9.2008)
//	SCOPE	 :	Puplic
//  DESC     :  Renders the data table for Multiplayer scores, started as a duplicate of Render Debug Info (above) and evolved from there. 
//				To be called on a platform specific button press. 
////////////////////////////////////////////////////////////////////////////////

void AVP_Multiplayer_ScoreBoard::Render()
{
	if ( AVP_Settings::IsMultiPlayer() )
	{
		// this function is called per frame so only enter here if the render flag is raised. 
		if ( s_bRenderScoreBoard && ( ! AVP_GUIMenu_System::IsInPauseMenu() ) )
		{
			Asura_GUIMenu_Root* pxBackDropRootItem = NULL;

			// get the hash id for the board back drop and get a pointer to the root item of it (depending on team or nonteam games).  
			AVP_GUIMenu_System::STORED_WIDGETS uBackdropHashID = AVP_GUIMenu_System::MULTIPLAYER_SCOREBOARD;

			if( AVP_Multiplayer_GameRules::IsTeamBasedGame() )
			{
				SetScoreBoardTitle( s_uAVP_MULTIPLAYER_SCOREBOARD_TEAM, true );
				pxBackDropRootItem = AVP_GUIMenu_System::GetHUDElement( uBackdropHashID, s_uAVP_MULTIPLAYER_SCOREBOARD_TEAM, false );
			}
			else
			{
				SetScoreBoardTitle( s_uAVP_MULTIPLAYER_SCOREBOARD_NON_TEAM, true );
				pxBackDropRootItem = AVP_GUIMenu_System::GetHUDElement( uBackdropHashID, s_uAVP_MULTIPLAYER_SCOREBOARD_NON_TEAM, false );
			}

			if( pxBackDropRootItem )
			{
				// if we found the item cast it to the correct menu widget (group) to unlock its specific functionality from here onwards. 
				Asura_GUIMenu_Widget_Group* pxBackDropGroup = static_cast<Asura_GUIMenu_Widget_Group*>( pxBackDropRootItem );

				if ( pxBackDropGroup )
				{
					Asura_Vector_2 xScreenCenter( Asura_Viewport::CentreX, Asura_Viewport::CentreY );
					Asura_Bounding_Rectangle xBackDropBounds( pxBackDropGroup->GetScaledWorldBounds() );
					
					// if the center of the screen resolution is not the point of the center of the backdrop ...
					if( xScreenCenter != xBackDropBounds.CalculateCentre() )
					{						
						// ... then displace the back drop by the difference of the two and "resize" to move it ( resize with same dimensions but different max/min co-ords. )
						xBackDropBounds.Displace( xScreenCenter - xBackDropBounds.CalculateCentre() );
						pxBackDropGroup->SetScaledWorldBounds( xBackDropBounds );
					}

					// set action flag to none because we are rendering a group of objects, not just one, so they all have thier own flags set in the tool and changing them here triggers an assert. 
					pxBackDropGroup->Render( ASURA_GUIMENU_ACTION_NONE );					
				}
				else
				{	
					Asura_Assert( true, "The scorboard item can not be found!! check coded hash ids are correct. " );
				}
			}
		}

#if defined(ASURA_SUPPORT_EXTERNAL_NETWORK)
		if (Asura_NetTransport_System::GetIdleTime() > dbg_uConnectionWarningTimeMS)
		{
			Asura_Font_System::Print( 0.5f, 0.6f, 0xffff5555, ASURA_TEXTFORMAT_CENTREJUSTIFIED | ASURA_TEXTFORMAT_VERTICALCENTREJUSTIFIED | ASURA_TEXTFORMAT_RELATIVEPOSITION, "Warning: connection issues." ); 
		}
#endif	//defined(ASURA_SUPPORT_EXTERNAL_NETWORK)
	}
}

////////////////////////////////////////////////////////////////////////////////
//  CLASS    :  AVP_Multiplayer_ScoreBoard
//  NAME     :  SortByScore()
//  CREATED  :  JG (27.8.2008)
//  DESC     :  Sort two player info entities based in individual score
////////////////////////////////////////////////////////////////////////////////
int AVP_Multiplayer_ScoreBoard::SortByScore(const AVP_ClientEntity_PlayerInfo& xPlayerInfoA, const AVP_ClientEntity_PlayerInfo& xPlayerInfoB )
{
	const int iPlayerScoreA = xPlayerInfoA.GetPlayerScore().GetIndividualScore(); 
	const int iPlayerScoreB = xPlayerInfoB.GetPlayerScore().GetIndividualScore(); 
	if ( iPlayerScoreA <= iPlayerScoreB )
	{
		return 1; 
	}
	else
	{
		return -1; 
	}
}

////////////////////////////////////////////////////////////////////////////////
//  CLASS    :  AVP_Multiplayer_ScoreBoard
//  NAME     :  SortByKills()
//  CREATED  :  CAH (9.9.2008)
//  DESC     :  Sort two player info entities based in individual Kills
////////////////////////////////////////////////////////////////////////////////
int AVP_Multiplayer_ScoreBoard::SortByKills(const AVP_ClientEntity_PlayerInfo& xPlayerInfoA, const AVP_ClientEntity_PlayerInfo& xPlayerInfoB )
{
	const int iPlayerScoreA = xPlayerInfoA.GetPlayerScore().GetNumberOfKills(); 
	const int iPlayerScoreB = xPlayerInfoB.GetPlayerScore().GetNumberOfKills(); 
	if ( iPlayerScoreA <= iPlayerScoreB )
	{
		return 1; 
	}
	else
	{
		return -1; 
	}
}

////////////////////////////////////////////////////////////////////////////////
//  CLASS    :  AVP_Multiplayer_ScoreBoard
//  NAME     :  SortByDeaths()
//  CREATED  :  CAH (9.9.2008)
//  DESC     :  Sort two player info entities based in individual Deaths (leaste deaths first). 
////////////////////////////////////////////////////////////////////////////////
int AVP_Multiplayer_ScoreBoard::SortByDeaths(const AVP_ClientEntity_PlayerInfo& xPlayerInfoA, const AVP_ClientEntity_PlayerInfo& xPlayerInfoB )
{
	const int iPlayerScoreA = xPlayerInfoA.GetPlayerScore().GetNumberOfTimesKilled(); 
	const int iPlayerScoreB = xPlayerInfoB.GetPlayerScore().GetNumberOfTimesKilled(); 
	if ( iPlayerScoreB <= iPlayerScoreA )
	{
		return 1; 
	}
	else
	{
		return -1; 
	}
}

////////////////////////////////////////////////////////////////////////////////
//  CLASS    :  AVP_Multiplayer_ScoreBoard
//  NAME     :  Shutdown()
//  CREATED  :  JG (1.8.2008)
//  DESC     :  Deallocate all resources used by this system
////////////////////////////////////////////////////////////////////////////////
void AVP_Multiplayer_ScoreBoard::Shutdown()
{
}


////////////////////////////////////////////////////////////////////////////////
//  CLASS    :  AVP_Multiplayer_ScoreBoard
//  NAME     :  GetBasicTeamScore()
//  CREATED  :  JG (27.8.2008)
//  DESC     :  Proper team score now sent through the PlayerScore class, updated 
//				this accordingly
////////////////////////////////////////////////////////////////////////////////
int AVP_Multiplayer_ScoreBoard::GetBasicTeamScore( AVP_Settings::Team eTeam )
{
	//Asura_Assert( eTeam != AVP_Settings::AVP_TEAM_NONE, "Requesting basic team score for an unset team" ); 
	Asura_Assert( eTeam < AVP_Settings::AVP_NUM_TEAMS, "Requesting basic team score for an invalid team" ); 
	int iTeamScore = 0; 

	if ( AVP_Settings::IsMultiPlayer() && AVP_Multiplayer_GameRules::IsTeamBasedGame() )
	{
		// Get the local player info through the player
		AVP_Player* pxLocalPlayer = AVP_Game_Core::GetPlayer(); 
		if (pxLocalPlayer)
		{
			Asura_Guid uLocalPlayerInfoGuid = pxLocalPlayer->GetGuid(); 
			AVP_ClientEntity_PlayerInfo* pxLocalPlayerInfo = AVP_ClientEntity_PlayerInfo::FindEntity( uLocalPlayerInfoGuid );
			if (pxLocalPlayerInfo)
			{
				// Get the team score for the required team
				switch ( eTeam )
				{
					case AVP_Settings::AVP_TEAM_ONE:
					default:
					{
						iTeamScore = pxLocalPlayerInfo->GetPlayerScore().GetTeamOneScore(); 
						break; 
					}
					case AVP_Settings::AVP_TEAM_TWO:
					{
						iTeamScore = pxLocalPlayerInfo->GetPlayerScore().GetTeamTwoScore(); 
						break; 
					}
					case AVP_Settings::AVP_TEAM_THREE:
					{
						iTeamScore = pxLocalPlayerInfo->GetPlayerScore().GetTeamThreeScore(); 
						break; 
					}
				}
			}
			else
			{
				Asura_Assert( false, "Couldn't find local player info" ); 
			}
		}
		else
		{
			Asura_Assert( false, "Couldn't find player" ); 
		}
	}

	return iTeamScore; 
}

////////////////////////////////////////////////////////////////////////////////
//  CLASS    :  AVP_Multiplayer_ScoreBoard
//  NAME     :  GetScoreListValue()
//  CREATED  :  CAH (17.9.2008)
//	SCOPE	 :	Private
//  DESC     :  Function to print out the score values into the scoreboard
//				This is called on update for each list teams list box
//				returning false will stop this function from being called again this update. 
////////////////////////////////////////////////////////////////////////////////

bool AVP_Multiplayer_ScoreBoard::GetScoreListValue( u_int uElementIndex, u_int uTeamIndex )
{
	u_int uEntry = 0; 
	Asura_List_ConstIt<AVP_ClientEntity_PlayerInfo> xPlayerInfoSortedListConstIt( m_axPlayerInfoSortedLists[uTeamIndex] ); 
	
	for( ; !xPlayerInfoSortedListConstIt.Done(); xPlayerInfoSortedListConstIt.Next() )
	{
		if ( uEntry == uElementIndex )
		{
			const AVP_Multiplayer_PlayerScore& xPlayerScore = xPlayerInfoSortedListConstIt.GetCurrent()->GetPlayerScore();

			Asura_CommandConsole::Printf( L"%i", xPlayerScore.GetIndividualScore() );

			return true;
		}
		++uEntry;
	}

	return false;
}

////////////////////////////////////////////////////////////////////////////////
//  CLASS    :  AVP_Multiplayer_ScoreBoard
//  NAME     :  GetNameListValue()
//  CREATED  :  CAH (18.9.2008)
//	SCOPE	 :	Private
//  DESC     :  Function to print out name values into the scoreboard
//				This is called on update for each team's listbox
//				returning false will stop this function from being called again that update. 
////////////////////////////////////////////////////////////////////////////////

bool AVP_Multiplayer_ScoreBoard::GetNameListValue( u_int uElementIndex, u_int uTeamIndex )
{
	AVP_ClientEntity_PlayerInfo* pxPlayerInfo = static_cast<AVP_ClientEntity_PlayerInfo*>( m_axPlayerInfoSortedLists[uTeamIndex].GetEntryByIndex( uElementIndex ) );

	if( pxPlayerInfo )
	{
		Asura_CommandConsole::Printf( L"%s", pxPlayerInfo->GetPlayerName() );
		return true;
	}
	else
	{
		return false;
	}
}

////////////////////////////////////////////////////////////////////////////////
//  CLASS    :  AVP_Multiplayer_ScoreBoard
//  NAME     :  GetKillsListValue()
//  CREATED  :  CAH (18.9.2008)
//	SCOPE	 :	Private
//  DESC     :  Function to print out kill values into the scoreboard
//				This is called on update for each team's listbox
//				returning false will stop this function from being called again that update. 
////////////////////////////////////////////////////////////////////////////////

bool AVP_Multiplayer_ScoreBoard::GetKillsListValue( u_int uElementIndex, u_int uTeamIndex )
{
	AVP_ClientEntity_PlayerInfo* pxPlayerInfo = static_cast<AVP_ClientEntity_PlayerInfo*>( m_axPlayerInfoSortedLists[uTeamIndex].GetEntryByIndex( uElementIndex ) );

	if( pxPlayerInfo )
	{
		const AVP_Multiplayer_PlayerScore& xPlayerScore = pxPlayerInfo->GetPlayerScore();

		Asura_CommandConsole::Printf( L"%u", xPlayerScore.GetNumberOfKills() );

		return true;
	}
	else
	{
		return false;
	}
}

////////////////////////////////////////////////////////////////////////////////
//  CLASS    :  AVP_Multiplayer_ScoreBoard
//  NAME     :  GetDeathsListValue()
//  CREATED  :  CAH (18.9.2008)
//	SCOPE	 :	Private
//  DESC     :  Function to print out kill values into the scoreboard
//				This is called on update for each team's listbox
//				returning false will stop this function from being called again that update. 
////////////////////////////////////////////////////////////////////////////////

bool AVP_Multiplayer_ScoreBoard::GetDeathsListValue( u_int uElementIndex, u_int uTeamIndex )
{
	AVP_ClientEntity_PlayerInfo* pxPlayerInfo = static_cast<AVP_ClientEntity_PlayerInfo*>( m_axPlayerInfoSortedLists[uTeamIndex].GetEntryByIndex( uElementIndex ) );

	if( pxPlayerInfo )
	{
		const AVP_Multiplayer_PlayerScore& xPlayerScore = pxPlayerInfo->GetPlayerScore();

		Asura_CommandConsole::Printf( L"%u", xPlayerScore.GetNumberOfTimesKilled() );

		return true;
	}
	else
	{
		return false;
	}
}

////////////////////////////////////////////////////////////////////////////////
//  CLASS    :  AVP_Multiplayer_ScoreBoard
//  NAME     :  GetStreakListValue()
//  CREATED  :  CAH (18.9.2008)
//	SCOPE	 :	Private
//  DESC     :  Function to print out kill values into the scoreboard
//				This is called on update for each team's listbox
//				returning false will stop this function from being called again that update. 
////////////////////////////////////////////////////////////////////////////////

bool AVP_Multiplayer_ScoreBoard::GetStreakListValue( u_int uElementIndex, u_int uTeamIndex )
{
	AVP_ClientEntity_PlayerInfo* pxPlayerInfo = static_cast<AVP_ClientEntity_PlayerInfo*>( m_axPlayerInfoSortedLists[uTeamIndex].GetEntryByIndex( uElementIndex ) );

	if( pxPlayerInfo )
	{
		const AVP_Multiplayer_PlayerScore& xPlayerScore = pxPlayerInfo->GetPlayerScore();

		Asura_CommandConsole::Printf( L"%u", xPlayerScore.GetKillStreak() );

		return true;
	}
	else
	{
		return false;
	}
}

////////////////////////////////////////////////////////////////////////////////
//  CLASS    :  AVP_Multiplayer_ScoreBoard
//  NAME     :  GetStreakListValue()
//  CREATED  :  CAH (18.9.2008)
//	SCOPE	 :	Private
//  DESC     :  Function to print out kill:death ratio values into the scoreboard
//				This is called on update for each team's listbox
//				returning false will stop this function from being called again that update. 
////////////////////////////////////////////////////////////////////////////////

bool AVP_Multiplayer_ScoreBoard::GetRatioListValue( u_int uElementIndex, u_int uTeamIndex )
{
	AVP_ClientEntity_PlayerInfo* pxPlayerInfo = static_cast<AVP_ClientEntity_PlayerInfo*>( m_axPlayerInfoSortedLists[uTeamIndex].GetEntryByIndex( uElementIndex ) );

	if( pxPlayerInfo )
	{
		const AVP_Multiplayer_PlayerScore& xPlayerScore = pxPlayerInfo->GetPlayerScore();

		Asura_UnicodeChar awcUnicodeString[16];
		Asura_UnicodeChar* pawcBufferToPass = awcUnicodeString;
				
		MakeRatioString( static_cast<float>( xPlayerScore.GetNumberOfKills() ), static_cast<float>( xPlayerScore.GetNumberOfTimesKilled() ), pawcBufferToPass );
		Asura_CommandConsole::Printf( awcUnicodeString );

		return true;

	}
	else
	{
		return false;
	}
}

////////////////////////////////////////////////////////////////////////////////
//  CLASS    :  AVP_Multiplayer_ScoreBoard
//  NAME     :  ConsoleCmd_PopulateTeamScore()
//  CREATED  :  CAH (18.9.2008)
//	SCOPE	 :	Private
//  DESC     :  Function to print out overall team scores. 
//				This is called on update once for each team
//				returning false will stop this function from being called again that update. 
////////////////////////////////////////////////////////////////////////////////

bool AVP_Multiplayer_ScoreBoard::ConsoleCmd_PopulateTeamScore( u_int uElementIndex, bool bInterrogate, bool bHelp )
{
	if( bInterrogate || bHelp )
	{
		return true;
	}

	u_int uEntry = 1; 
	
	while( uEntry < AVP_Settings::AVP_NUM_TEAMS )
	{
		if ( ( uEntry - 1 ) == uElementIndex )
		{
			// if this is the correct element in range and there are players on this team:
			// CAH comented out because just because there are not currently players in the team doesnt mean there never were. 
			//if( ! m_axPlayerInfoSortedLists[uEntry].IsEmpty() )
			{				
				Asura_CommandConsole::Printf( L"%u", GetBasicTeamScore( static_cast<AVP_Settings::Team>( uEntry ) ) );
			}

			return true;
		}

		++uEntry;
	}

	return true;
}

////////////////////////////////////////////////////////////////////////////////
//  CLASS    :  AVP_Multiplayer_ScoreBoard
//  NAME     :  GetSpeciesVisible()
//  CREATED  :  CAH (18.9.2008)
//	SCOPE	 :	Private
//  DESC     :  Function to determine if a particular species image should be visible in the ListBox.
//				This is called for every element in each list box three times per update (once for each species.) 
////////////////////////////////////////////////////////////////////////////////

bool AVP_Multiplayer_ScoreBoard::GetSpeciesVisible( u_int uElementIndex, u_int uTeamIndex, u_int uSpecies )
{
	AVP_Settings::Species ePlayerSpecies = AVP_Settings::AVP_SPECIES_UNKNOWN; 

	AVP_ClientEntity_PlayerInfo* pxPlayerInfo = static_cast<AVP_ClientEntity_PlayerInfo*>( m_axPlayerInfoSortedLists[uTeamIndex].GetEntryByIndex( uElementIndex ) );
	if ( pxPlayerInfo )
	{
		// Use the Puppet GUID to find the ClientEntityActor because the Puppet GUID is always the same as it's
		AVP_ClientEntity_Actor* pxClientEntityActor = AVP_ClientEntity_Actor::FindEntity( pxPlayerInfo->GetPuppetGuid() ); 
		if ( pxClientEntityActor )
		{
			// obtain the species from the entity classification of the ClientEntityActor. 
			ePlayerSpecies = pxClientEntityActor->GetSpeciesByClassification(); 
		}
	}

	if( ePlayerSpecies == static_cast<int>( uSpecies ) )
	{
		Asura_CommandConsole::Printf( L"1" );
	}
	else
	{
		Asura_CommandConsole::Printf( L"0" );
	}

	return true;
}

////////////////////////////////////////////////////////////////////////////////
//  CLASS    :  AVP_Multiplayer_ScoreBoard
//  NAME     :  GetTalkingVisible()
//  CREATED  :  CAH (18.9.2008)
//	SCOPE	 :	Private
//  DESC     :  Function to determine if a particular tallking image should be visible in the ListBox.
//				This is called for every element in each list box per update
////////////////////////////////////////////////////////////////////////////////

bool AVP_Multiplayer_ScoreBoard::GetTalkingVisible( u_int uElementIndex, u_int uTeamIndex )
{
	// CAH HELD LOW INDEPENDANT OF PLATFORM ATM. until xbox functionality for talking is inplace. 
	Asura_CommandConsole::Printf( L"0" );
	return true;
}

////////////////////////////////////////////////////////////////////////////////
//  CLASS    :  AVP_Multiplayer_ScoreBoard
//  NAME     :  GetHighlightVisible()
//  CREATED  :  CAH (18.9.2008)
//	SCOPE	 :	Private
//  DESC     :  Function to determine if a particular "local player highlight" image should be visible in the ListBox.
//				This is called for every element in each list box per update
////////////////////////////////////////////////////////////////////////////////

bool AVP_Multiplayer_ScoreBoard::GetHighlightVisible( u_int uElementIndex, u_int uTeamIndex )
{
	bool bLocalPlayer = false; 

	// get a pointer to the client side playerinfo. use uElementIndex (which is the index of the listbox element to fill) as the index of the linked lists in m_axPlayerInfoSortedLists
	AVP_ClientEntity_PlayerInfo* pxCurrentPlayerInfo = static_cast<AVP_ClientEntity_PlayerInfo*>( m_axPlayerInfoSortedLists[uTeamIndex].GetEntryByIndex( uElementIndex ) );
	if ( pxCurrentPlayerInfo )
	{
		// if we found the player info and it is the local play to this computer raise the flag. 
		bLocalPlayer = AVP_Game_Core::IsLocalPlayer( pxCurrentPlayerInfo->GetGuid() );	
	}
	
	if( bLocalPlayer )
	{
		Asura_CommandConsole::Printf( L"1" );
	}
	else
	{
		Asura_CommandConsole::Printf( L"0" );
	}

	return true;
}

////////////////////////////////////////////////////////////////////////////////
//  CLASS    :  AVP_Multiplayer_ScoreBoard
//  NAME     :  DeactivateResultScreens( u_int uPageID )
//  CREATED  :  CAH (18.9.2008)
//	SCOPE	 :	Static Public
//  DESC     :  Function to deactivate any of the scoreboard screens from the gui system, needed because the screens are code driven. 
//				the page id is for the page to be closed. 
////////////////////////////////////////////////////////////////////////////////

bool AVP_Multiplayer_ScoreBoard::DeactivateResultScreens( u_int uPageID, bool bInterrogate, bool bHelp )
{
	if( bInterrogate || bHelp )
	{
		Asura_CommandConsole::Printf("Closes the page with the passed hash ID. ");
		return true;
	}

	if( Asura_GUIMenu_System::IsMenuPageHashActive( uPageID ) )
	{
		// if the page is indeed active then deactivate it as requested. 
		AVP_GUIMenu_System::Deactivate( uPageID );
	}

	return true;
}

////////////////////////////////////////////////////////////////////////////////
//  CLASS    :  AVP_Multiplayer_ScoreBoard
//  NAME     :  ActivateResultScreens( u_int uPageID )
//  CREATED  :  CAH (18.9.2008)
//	SCOPE	 :	Static Public
//  DESC     :  Function to activate any of the scoreboard screens from the gui system, needed because the screens are code driven. 
//				the page id is for the page to be opened
////////////////////////////////////////////////////////////////////////////////

bool AVP_Multiplayer_ScoreBoard::ActivateResultScreens( u_int uPageID, bool bInterrogate, bool bHelp )
{
	if( bInterrogate || bHelp )
	{
		Asura_CommandConsole::Printf("Opens the page with the passed hash ID. ");
		return true;
	}

	if( ! Asura_GUIMenu_System::IsMenuPageHashActive( uPageID ) )
	{
		// if the page is not already active then activate it as requested. keep the environment living but lock controls. 
		AVP_GUIMenu_System::Activate( uPageID, false, true );
	}

	return true;
}

////////////////////////////////////////////////////////////////////////////////
//  CLASS    :  AVP_Multiplayer_ScoreBoard
//  NAME     :  SetScoreBoardTitle( u_int uPageIndex, bool bRender )
//  CREATED  :  CAH (18.9.2008)
//	SCOPE	 :	Private
//  DESC     :  Set the scoreboard title text according to the page hash id argument.  
//				In its own function to avoid clutter in the render function. 
////////////////////////////////////////////////////////////////////////////////

void AVP_Multiplayer_ScoreBoard::SetScoreBoardTitle( u_int uPageIndex, bool bRender )
{
	u_int uTextStingID;
	AVP_GUIMenu_System::STORED_WIDGETS uTitleEnumID;
	Asura_GUIMenu_Root* pxTitleRootItem = NULL;

	uTitleEnumID = AVP_GUIMenu_System::MULTIPLAYER_SCOREBOARD_TITLE;

	// get a pinter to the text widget that is the page title, possible because in every score page the widget has the same name. 
	pxTitleRootItem = AVP_GUIMenu_System::GetHUDElement( uTitleEnumID, uPageIndex, false, true );
	
	if( pxTitleRootItem )
	{
		// if we found the item cast it to the correct menu widget (group) to unlock its specific functionality from here onwards. 
		Asura_GUIMenu_Element_Text* pxTitleTextElement = static_cast<Asura_GUIMenu_Element_Text*>( pxTitleRootItem );

		if( pxTitleTextElement )
		{
			// get the multilangiage string ID for the title of the Game Type. IDs are for strings stored in Menu_XX.asr files.
			switch( AVP_Settings::GetGameType() )
			{
				case AVP_Settings::AVP_GAMETYPE_DISRUPTION_DOMINATION:
				{
					uTextStingID = s_uAVP_MULTIPLAYER_GAME_TITLE_DISDOMINATION;
					break;
				}
				case AVP_Settings::AVP_GAMETYPE_DEATH_MATCH:
				{
					uTextStingID = s_uAVP_MULTIPLAYER_GAME_TITLE_DEATHMATCH;
					break;
				}
				case AVP_Settings::AVP_GAMETYPE_HUNT:
				{
					uTextStingID = s_uAVP_MULTIPLAYER_GAME_TITLE_HUNT;
					break;
				}
				case AVP_Settings::AVP_GAMETYPE_SPECIES_TEAM_DEATH_MATCH:
				{
					uTextStingID = s_uAVP_MULTIPLAYER_GAME_TITLE_TEAMDEATHMATCH;
					break;
				}
				case AVP_Settings::AVP_GAMETYPE_SURVIVOR:
				{
					uTextStingID = s_uAVP_MULTIPLAYER_GAME_TITLE_SURVIVOR;
					break;
				}
				default:
				{
					// the default if a place holder string to indacate the problem but keep stable. 
					uTextStingID = s_uAVP_MULTIPLAYER_GAME_TITLE_PLACEHOLDER;
					break;
				}
			}

			// if the string is not the one already set in the text element set it and re-render it. (the title is not rendered each frame normally)
			if( pxTitleTextElement->GetTextID() != uTextStingID )
			{
				pxTitleTextElement->SetTextID( uTextStingID );
				//pxTitleTextElement->Render( ASURA_GUIMENU_ACTION_NONE );
			}
		}
		else
		{
			Asura_Assert( true, "The Multiplayer Scoreboard title item can not be found, so text can not be filled in." );
		}
	}
}

////////////////////////////////////////////////////////////////////////////////
//  CLASS    :  AVP_Multiplayer_ScoreBoard
//  NAME     :  ActivateResultsTables()
//  CREATED  :  CAH (8.10.2008)
//	SCOPE	 :	Static Public
//  DESC     :  Function to populate the dynamic data of the three possible match result screens. 
//				Work Through all of the elements one at a time filling them in. 
//				This seems long winded, but well, the GUI system is... and this is only done once each MP match
//				A list box could be used but because each entry in the list is based on a different variable 
//				the population function would be about the same as this anyway, and a lot harder to modify/tweak
////////////////////////////////////////////////////////////////////////////////

void AVP_Multiplayer_ScoreBoard::ActivateResultsTables()
{
	Asura_UnicodeChar awcUnicodeString[16];

	// update the display condition for the winners box, shown on both results screens. 
	s_bTeamBased = AVP_Multiplayer_GameRules::IsTeamBasedGame();

	const AVP_ClientEntity_PlayerInfo* pxPlayerInfo = AVP_ClientEntity_PlayerInfo::FindEntity( AVP_Game_Core::GetPlayerGuid() );
	if (pxPlayerInfo)
	{
		const AVP_Multiplayer_PlayerScore& xPlayerScore = pxPlayerInfo->GetPlayerScore();

		// KILLS AS MARINE
		Asura_PrintfToBfr( awcUnicodeString, L"%d", 16, true, xPlayerScore.GetNumberOfKillsAsMarine() );
		FillTextWidgetWithData( AVP_GUIMenu_System::MULTIPLAYER_STATS_MATCH_KILLSMARINE, s_uAVP_MULTIPLAYER_RESULTS_INDIVIDUAL, awcUnicodeString );
		//Asura_PrintfToBfr( awcUnicodeString, L"%d", 16, true, /*XXX*/ );
		//FillTextWidgetWithData( AVP_GUIMenu_System::MULTIPLAYER_STATS_CAREER_KILLSMARINE, s_uAVP_MULTIPLAYER_RESULTS_INDIVIDUAL, awcUnicodeString );

		// HEADSHOTS AS MARINE
		Asura_PrintfToBfr( awcUnicodeString, L"%d", 16, true, xPlayerScore.GetNumberOfHeadshots() );
		FillTextWidgetWithData( AVP_GUIMenu_System::MULTIPLAYER_STATS_MATCH_KILLSMARINE_SP, s_uAVP_MULTIPLAYER_RESULTS_INDIVIDUAL, awcUnicodeString );
		//Asura_PrintfToBfr( awcUnicodeString, L"%d", 16, true, /*XXX*/ );
		//FillTextWidgetWithData( AVP_GUIMenu_System::MULTIPLAYER_STATS_CAREER_KILLSMARINE_SP, s_uAVP_MULTIPLAYER_RESULTS_INDIVIDUAL, awcUnicodeString );

		// KILLS AS PREDATOR
		Asura_PrintfToBfr( awcUnicodeString, L"%d", 16, true, xPlayerScore.GetNumberOfKillsAsPredator() );
		FillTextWidgetWithData( AVP_GUIMenu_System::MULTIPLAYER_STATS_MATCH_KILLSPRED, s_uAVP_MULTIPLAYER_RESULTS_INDIVIDUAL, awcUnicodeString );
		//Asura_PrintfToBfr( awcUnicodeString, L"%d", 16, true, /*XXX*/ );
		//FillTextWidgetWithData( AVP_GUIMenu_System::MULTIPLAYER_STATS_CAREER_KILLSPRED, s_uAVP_MULTIPLAYER_RESULTS_INDIVIDUAL, awcUnicodeString );

		// TROPHIES AS PREDATOR
		Asura_PrintfToBfr( awcUnicodeString, L"%d", 16, true, xPlayerScore.GetNumberOfTrophies() );
		FillTextWidgetWithData( AVP_GUIMenu_System::MULTIPLAYER_STATS_MATCH_KILLSPRED_SP, s_uAVP_MULTIPLAYER_RESULTS_INDIVIDUAL, awcUnicodeString );
		//Asura_PrintfToBfr( awcUnicodeString, L"%d", 16, true, /*XXX*/ );
		//FillTextWidgetWithData( AVP_GUIMenu_System::MULTIPLAYER_STATS_CAREER_KILLSPRED_SP, s_uAVP_MULTIPLAYER_RESULTS_INDIVIDUAL, awcUnicodeString );

		// KILLS AS ALIEN
		Asura_PrintfToBfr( awcUnicodeString, L"%d", 16, true, xPlayerScore.GetNumberOfKillsAsAlien() );
		FillTextWidgetWithData( AVP_GUIMenu_System::MULTIPLAYER_STATS_MATCH_KILLSALIEN, s_uAVP_MULTIPLAYER_RESULTS_INDIVIDUAL, awcUnicodeString );
		//Asura_PrintfToBfr( awcUnicodeString, L"%d", 16, true, /*XXX*/ );
		//FillTextWidgetWithData( AVP_GUIMenu_System::MULTIPLAYER_STATS_CAREER_KILLSALIEN, s_uAVP_MULTIPLAYER_RESULTS_INDIVIDUAL, awcUnicodeString );

		// HEADBITES AS ALIEN
		Asura_PrintfToBfr( awcUnicodeString, L"%d", 16, true, xPlayerScore.GetNumberOfHeadbites() );
		FillTextWidgetWithData( AVP_GUIMenu_System::MULTIPLAYER_STATS_MATCH_KILLSALIEN_SP, s_uAVP_MULTIPLAYER_RESULTS_INDIVIDUAL, awcUnicodeString );
		//Asura_PrintfToBfr( awcUnicodeString, L"%d", 16, true, /*XXX*/ );
		//FillTextWidgetWithData( AVP_GUIMenu_System::MULTIPLAYER_STATS_CAREER_KILLSALIEN_SP, s_uAVP_MULTIPLAYER_RESULTS_INDIVIDUAL, awcUnicodeString );

		// TOTAL KILLS
		Asura_PrintfToBfr( awcUnicodeString, L"%d", 16, true, xPlayerScore.GetNumberOfKills() );
		FillTextWidgetWithData( AVP_GUIMenu_System::MULTIPLAYER_STATS_MATCH_KILLSTOTAL, s_uAVP_MULTIPLAYER_RESULTS_INDIVIDUAL, awcUnicodeString );
		//Asura_PrintfToBfr( awcUnicodeString, L"%d", 16, true, /*XXX*/ );
		//FillTextWidgetWithData( AVP_GUIMenu_System::MULTIPLAYER_STATS_CAREER_KILLSTOTAL, s_uAVP_MULTIPLAYER_RESULTS_INDIVIDUAL, awcUnicodeString );

		// FINAL SCORE
		Asura_PrintfToBfr( awcUnicodeString, L"%d", 16, true, xPlayerScore.GetIndividualScore() );
		FillTextWidgetWithData( AVP_GUIMenu_System::MULTIPLAYER_STATS_MATCH_SCORETOTAL, s_uAVP_MULTIPLAYER_RESULTS_INDIVIDUAL, awcUnicodeString );
		//Asura_PrintfToBfr( awcUnicodeString, L"%d", 16, true, /*XXX*/ );
		//FillTextWidgetWithData( AVP_GUIMenu_System::MULTIPLAYER_STATS_CAREER_SCORETOTAL, s_uAVP_MULTIPLAYER_RESULTS_INDIVIDUAL, awcUnicodeString );

		// KILL ASSISTS
		Asura_PrintfToBfr( awcUnicodeString, L"%d", 16, true, xPlayerScore.GetNumberOfKillAssists() );
		FillTextWidgetWithData( AVP_GUIMenu_System::MULTIPLAYER_STATS_MATCH_KILLASSISTS, s_uAVP_MULTIPLAYER_RESULTS_INDIVIDUAL, awcUnicodeString );
		//Asura_PrintfToBfr( awcUnicodeString, L"%d", 16, true, /*XXX*/ );
		//FillTextWidgetWithData( AVP_GUIMenu_System::MULTIPLAYER_STATS_CAREER_KILLASSISTS, s_uAVP_MULTIPLAYER_RESULTS_INDIVIDUAL, awcUnicodeString );

		// KILLS STREAK
		Asura_PrintfToBfr( awcUnicodeString, L"%d", 16, true, xPlayerScore.GetKillStreak() );
		FillTextWidgetWithData( AVP_GUIMenu_System::MULTIPLAYER_STATS_MATCH_KILLSTREAK, s_uAVP_MULTIPLAYER_RESULTS_INDIVIDUAL, awcUnicodeString );
		//Asura_PrintfToBfr( awcUnicodeString, L"%d", 16, true, /*XXX*/ );
		//FillTextWidgetWithData( AVP_GUIMenu_System::MULTIPLAYER_STATS_CAREER_KILLSTREAK, s_uAVP_MULTIPLAYER_RESULTS_INDIVIDUAL, awcUnicodeString );

		// NUMBER OF DEATHS
		Asura_PrintfToBfr( awcUnicodeString, L"%d", 16, true, xPlayerScore.GetNumberOfTimesKilled() );
		FillTextWidgetWithData( AVP_GUIMenu_System::MULTIPLAYER_STATS_MATCH_DEATHS, s_uAVP_MULTIPLAYER_RESULTS_INDIVIDUAL, awcUnicodeString );
		//Asura_PrintfToBfr( awcUnicodeString, L"%d", 16, true, /*XXX*/ );
		//FillTextWidgetWithData( AVP_GUIMenu_System::MULTIPLAYER_STATS_CAREER_DEATHS, s_uAVP_MULTIPLAYER_RESULTS_INDIVIDUAL, awcUnicodeString );

		// KILL DEATH RATIO
		{
			Asura_UnicodeChar* pawcBufferToPass = awcUnicodeString;
			MakeRatioString( static_cast<float>( xPlayerScore.GetNumberOfKills() ), static_cast<float>( xPlayerScore.GetNumberOfTimesKilled() ), pawcBufferToPass );
			FillTextWidgetWithData( AVP_GUIMenu_System::MULTIPLAYER_STATS_MATCH_RATIO, s_uAVP_MULTIPLAYER_RESULTS_INDIVIDUAL, awcUnicodeString );
			//MakeRatioString( static_cast<float>( /*XXX*/ ), static_cast<float>( /*XXX*/ ), pawcBufferToPass );
			//FillTextWidgetWithData( AVP_GUIMenu_System::MULTIPLAYER_STATS_CAREER_RATIO, s_uAVP_MULTIPLAYER_RESULTS_INDIVIDUAL, awcUnicodeString );
		}
		
		// XP POINTS GAINED
		//Asura_PrintfToBfr( awcUnicodeString, L"%d", 16, true, /*XXX*/ );
		//FillTextWidgetWithData( AVP_GUIMenu_System::MULTIPLAYER_STATS_MATCH_XPPOINTS, s_uAVP_MULTIPLAYER_RESULTS_INDIVIDUAL, awcUnicodeString );
		//Asura_PrintfToBfr( awcUnicodeString, L"%d", 16, true, /*XXX*/ );
		//FillTextWidgetWithData( AVP_GUIMenu_System::MULTIPLAYER_STATS_CAREER_XPPOINTS, s_uAVP_MULTIPLAYER_RESULTS_INDIVIDUAL, awcUnicodeString );

		// lastly set the score board sub title. No need to activate the page, this is done via the group results screen below. 
		SetScoreBoardTitle( s_uAVP_MULTIPLAYER_RESULTS_INDIVIDUAL, false );

		// Now populate the group results screen, which is dependant on team or nonteam based games. 
		if( AVP_Multiplayer_GameRules::IsTeamBasedGame() )
		{
			int iTempScore;
			u_int uTeam;
			AVP_Settings::Team eWinner = AVP_Settings::AVP_TEAM_ONE;
			int iHighestScore = GetBasicTeamScore( AVP_Settings::AVP_TEAM_ONE );

			for( uTeam = AVP_Settings::AVP_TEAM_TWO; uTeam < AVP_Settings::AVP_NUM_TEAMS; uTeam++ )
			{
				iTempScore = GetBasicTeamScore( static_cast<AVP_Settings::Team>( uTeam ) );
				 
				if( iTempScore > iHighestScore )
				{
					// if the score is bigger than the previous team's record the score and which team it was from.
					iHighestScore = iTempScore;
					eWinner = static_cast<AVP_Settings::Team>( uTeam );
				}
				else if( iTempScore == iHighestScore )
				{
					if( FindTotalKillsForTeam( uTeam ) == FindTotalKillsForTeam( eWinner ) )
					{
						// if the teams are drawn on both score and kills, go to number of times killed. 
						if( FindTotalDeathsForTeam( uTeam ) < FindTotalDeathsForTeam( eWinner ) )
						{
							// if the new team has less deaths then it becomes the winner. 
							eWinner = static_cast<AVP_Settings::Team>( uTeam );
						}
					}
					else if( FindTotalKillsForTeam( uTeam ) > FindTotalKillsForTeam( eWinner ) )
					{
						// if the new team has more kille then it becomes the winner. 
						eWinner = static_cast<AVP_Settings::Team>( uTeam );
					}
				}
			}

			// now we know the winning team and its score, so fill in the two widgets that display them for both pages, depending on which team it was. 
			switch( eWinner )
			{
				case AVP_Settings::AVP_TEAM_ONE:
				default:
				{
					uTeam = s_uAVP_MULTIPLAYER_GAME_TEAMNAME_GREEN;
					break;
				}
				case AVP_Settings::AVP_TEAM_TWO:
				{
					uTeam = s_uAVP_MULTIPLAYER_GAME_TEAMNAME_BLUE;
					break;
				}
				case AVP_Settings::AVP_TEAM_THREE:
				{
					uTeam = s_uAVP_MULTIPLAYER_GAME_TEAMNAME_RED;
					break;
				}
			}
			FillTextWidgetWithText( AVP_GUIMenu_System::MULTIPLAYER_STATS_WINNINGTEAM, s_uAVP_MULTIPLAYER_RESULTS_TEAM, uTeam );
			FillTextWidgetWithText( AVP_GUIMenu_System::MULTIPLAYER_STATS_WINNINGTEAM, s_uAVP_MULTIPLAYER_RESULTS_INDIVIDUAL, uTeam );
			
			Asura_PrintfToBfr( awcUnicodeString, L"%d", 16, true, iHighestScore );
			FillTextWidgetWithData( AVP_GUIMenu_System::MULTIPLAYER_STATS_WINNINGSCORE, s_uAVP_MULTIPLAYER_RESULTS_TEAM, awcUnicodeString );
			FillTextWidgetWithData( AVP_GUIMenu_System::MULTIPLAYER_STATS_WINNINGSCORE, s_uAVP_MULTIPLAYER_RESULTS_INDIVIDUAL, awcUnicodeString );

			// lastly set the screen subtitles and activate the correct first screen. 
			SetScoreBoardTitle( s_uAVP_MULTIPLAYER_RESULTS_TEAM, false );
			ActivateResultScreens( s_uAVP_MULTIPLAYER_RESULTS_TEAM, false, false );
		}
		else
		{	
			AVP_ClientEntity_PlayerInfo* pxWinningPlayerInfo = AVP_ClientEntity_PlayerInfo::FindEntity( FindHighestScoringIndividualInTeam( AVP_Settings::AVP_TEAM_NONE ) );
			if (pxWinningPlayerInfo)
			{
				FillTextWidgetWithData( AVP_GUIMenu_System::MULTIPLAYER_STATS_WINNINGPLAYER, s_uAVP_MULTIPLAYER_RESULTS_NON_TEAM, pxWinningPlayerInfo->GetPlayerName() );
				FillTextWidgetWithData( AVP_GUIMenu_System::MULTIPLAYER_STATS_WINNINGPLAYER, s_uAVP_MULTIPLAYER_RESULTS_INDIVIDUAL, pxWinningPlayerInfo->GetPlayerName() );

				Asura_PrintfToBfr( awcUnicodeString, L"%d", 16, true, pxWinningPlayerInfo->GetPlayerScore().GetIndividualScore() );
				FillTextWidgetWithData( AVP_GUIMenu_System::MULTIPLAYER_STATS_WINNINGSCORE, s_uAVP_MULTIPLAYER_RESULTS_NON_TEAM, awcUnicodeString );
				FillTextWidgetWithData( AVP_GUIMenu_System::MULTIPLAYER_STATS_WINNINGSCORE, s_uAVP_MULTIPLAYER_RESULTS_INDIVIDUAL, awcUnicodeString );
			}
			else
			{
				Asura_Assert(false, "We've managed to lost the winning player info and score. Please report in Mantis"); 
			}
			
			// lastly set the screen subtitles and activate the correct first screen. 
			SetScoreBoardTitle( s_uAVP_MULTIPLAYER_RESULTS_NON_TEAM, false );
			ActivateResultScreens( s_uAVP_MULTIPLAYER_RESULTS_NON_TEAM, false, false );
		}
	}
	else
	{
		Asura_Assert(false, "We've managed to lost the local player score. This is quite bad. Please report in Mantis"); 
	}
}

////////////////////////////////////////////////////////////////////////////////
//  CLASS    :  AVP_Multiplayer_ScoreBoard
//  NAME     :  FillTextWidgetWithData()
//  CREATED  :  CAH (8.10.2008)
//	SCOPE	 :	Private
//  DESC     :  Function to reduce code repetition, takes a page hash, a widget id and a string to enter, then fills it. 
////////////////////////////////////////////////////////////////////////////////

void AVP_Multiplayer_ScoreBoard::FillTextWidgetWithData( AVP_GUIMenu_System::STORED_WIDGETS eWidgetToFill, u_int uIdOfPageContainingWidget, const Asura_UnicodeChar* awcTextToEnter )
{
	Asura_GUIMenu_Root* pxItemRoot = AVP_GUIMenu_System::GetHUDElement( eWidgetToFill, uIdOfPageContainingWidget, false );

	if( pxItemRoot )
	{
		Asura_GUIMenu_Element_Text* pxTextElement = static_cast<Asura_GUIMenu_Element_Text*>( pxItemRoot );

		if( pxTextElement )
		{
			pxTextElement->SetText( awcTextToEnter );
		}
	}
	else
	{
		Asura_Assert( true, "One of the Marine Kills GUI text items can not be found so can not be update, check given hash id against the text widgets name." );
	}
}

////////////////////////////////////////////////////////////////////////////////
//  CLASS    :  AVP_Multiplayer_ScoreBoard
//  NAME     :  FillTextWidgetWithText()
//  CREATED  :  CAH (8.10.2008)
//	SCOPE	 :	Private
//  DESC     :  Function to reduce code repetition, takes a page hash, a widget id and a string to enter, then fills it. 
////////////////////////////////////////////////////////////////////////////////

void AVP_Multiplayer_ScoreBoard::FillTextWidgetWithText( AVP_GUIMenu_System::STORED_WIDGETS eWidgetToFill, u_int uIdOfPageContainingWidget, u_int uTextHashId )
{
	Asura_GUIMenu_Root* pxItemRoot = AVP_GUIMenu_System::GetHUDElement( eWidgetToFill, uIdOfPageContainingWidget, false );

	if( pxItemRoot )
	{
		Asura_GUIMenu_Element_Text* pxTextElement = static_cast<Asura_GUIMenu_Element_Text*>( pxItemRoot );

		if( pxTextElement )
		{
			pxTextElement->SetTextID( uTextHashId );
		}
	}
	else
	{
		Asura_Assert( true, "One of the Marine Kills GUI text items can not be found so can not be update, check given hash id against the text widgets name." );
	}
}

////////////////////////////////////////////////////////////////////////////////
//  CLASS    :  AVP_Multiplayer_ScoreBoard
//  NAME     :  MakeRatioString()
//  CREATED  :  CAH (9.10.2008)
//	SCOPE	 :	Private
//  DESC     :  Funtion takes two floats and a array of Asura_UnicodeChar, work out the ratio and fills the buffer. 
////////////////////////////////////////////////////////////////////////////////

void AVP_Multiplayer_ScoreBoard::MakeRatioString( float fValueOne, float fValueTwo, Asura_UnicodeChar* awcBufferToFill )
{
	float fRatio;

	if( ( fValueOne <= 0.0f ) && ( fValueTwo <= 0.0f ) )
	{
		// if BOTH values are 0 or below
		Asura_PrintfToBfr( awcBufferToFill, L"1.0 : 1.0", 16, true );
	}
	else if( fValueOne <= 0.0f )
	{
		Asura_PrintfToBfr( awcBufferToFill, L"0.0 : %.1f", 16, true, fValueTwo );
	}
	else if( fValueTwo <= 0.0f )
	{
		Asura_PrintfToBfr( awcBufferToFill, L"%.1f : 0.0", 16, true, fValueOne );
	}
	else if( fValueOne < fValueTwo )
	{
		fRatio = fValueTwo / fValueOne;
		Asura_PrintfToBfr( awcBufferToFill, L"1.0 : %.1f", 16, true, fRatio );
	}
	else
	{
		fRatio = fValueOne / fValueTwo;
		Asura_PrintfToBfr( awcBufferToFill, L"%.1f : 1.0", 16, true, fRatio );
	}
}

////////////////////////////////////////////////////////////////////////////////
//  CLASS    :  AVP_Multiplayer_ScoreBoard
//  NAME     :  FindTotalKillsForTeam( AVP_Settings::Team eTeam )
//  CREATED  :  CAH (9.10.2008)
//	SCOPE	 :	Private
//  DESC     :  Function takes a team index, goes through each member of that team accumulating the kills of each member and returns the total. 
////////////////////////////////////////////////////////////////////////////////

int AVP_Multiplayer_ScoreBoard::FindTotalKillsForTeam( u_int uTeam )
{
	if( uTeam > AVP_Settings::AVP_NUM_TEAMS )
	{
		Asura_Assert( true, "Request for team kills has been passed an invalid team index. score returned is not accurate. " );
		return 0;
	}

	int iKillTotal = 0;

	Asura_List_ConstIt<AVP_ClientEntity_PlayerInfo> xPlayerInfoSortedListConstIt( m_axPlayerInfoSortedLists[uTeam] ); 
	
	for( ; !xPlayerInfoSortedListConstIt.Done(); xPlayerInfoSortedListConstIt.Next() )
	{
		// for each menber of the team get individuals kills through the PlayerScore structure in the PlayerInfoStructure. 
		iKillTotal += xPlayerInfoSortedListConstIt.GetCurrent()->GetPlayerScore().GetNumberOfKills();
	}

	return iKillTotal;
}

////////////////////////////////////////////////////////////////////////////////
//  CLASS    :  AVP_Multiplayer_ScoreBoard
//  NAME     :  FindTotalDeathsForTeam( AVP_Settings::Team eTeam )
//  CREATED  :  CAH (9.10.2008)
//	SCOPE	 :	Private
//  DESC     :  Function takes a team index, goes through each member of that team accumulating the Deaths of each member and returns the total. 
////////////////////////////////////////////////////////////////////////////////

int AVP_Multiplayer_ScoreBoard::FindTotalDeathsForTeam( u_int uTeam )
{
	if( uTeam > AVP_Settings::AVP_NUM_TEAMS )
	{
		Asura_Assert( true, "Request for team kills has been passed an invalid team index. score returned is not accurate. " );
		return 0;
	}

	int iKillTotal = 0;

	Asura_List_ConstIt<AVP_ClientEntity_PlayerInfo> xPlayerInfoSortedListConstIt( m_axPlayerInfoSortedLists[uTeam] ); 
	
	for( ; !xPlayerInfoSortedListConstIt.Done(); xPlayerInfoSortedListConstIt.Next() )
	{
		// for each menber of the team get individuals kills through the PlayerScore structure in the PlayerInfoStructure. 
		iKillTotal += xPlayerInfoSortedListConstIt.GetCurrent()->GetPlayerScore().GetNumberOfTimesKilled();
	}

	return iKillTotal;
}

////////////////////////////////////////////////////////////////////////////////
//  CLASS    :  AVP_Multiplayer_ScoreBoard
//  NAME     :  FindHighestScoringIndividualInTeam( AVP_Settings::Team eTeam )
//  CREATED  :  CAH (9.10.2008)
//	SCOPE	 :	Private
//  DESC     :  Function take a team id and finds the winning player, based on score then kills, returning the GUID of the players info.
////////////////////////////////////////////////////////////////////////////////

int AVP_Multiplayer_ScoreBoard::FindHighestScoringIndividualInTeam( u_int uTeam )
{
	if( uTeam > AVP_Settings::AVP_NUM_TEAMS )
	{
		Asura_Assert( true, "Request for winning individual in team has been passed an invalid team index. score returned is not accurate. " );
		return 0;
	}

	Asura_List_It<AVP_ClientEntity_PlayerInfo> xPlayerInfoSortedListIt( m_axPlayerInfoSortedLists[uTeam] ); 
	u_int uNumberDrawn = 0;

	// first find the score of the currently first member in the list and iterate it, the lists are ordered so normally this will be the highest. 
	int iTempScore = xPlayerInfoSortedListIt.GetCurrent()->GetPlayerScore().GetIndividualScore();
	u_int uMemberIndex = xPlayerInfoSortedListIt.GetCurrent()->GetGuid();
	xPlayerInfoSortedListIt.Next();

	if( m_axPlayerInfoSortedLists[uTeam].GetNumEntries() == 1 )
	{
		// if there is only one player in the team return his score
		return uMemberIndex;
	}

	for( ; !xPlayerInfoSortedListIt.Done(); xPlayerInfoSortedListIt.Next() )
	{
		if( iTempScore == xPlayerInfoSortedListIt.GetCurrent()->GetPlayerScore().GetIndividualScore() )
		{
			// if the score of this player is as high as the previous there is a draw, record this and continue to next iteration. 
			uNumberDrawn ++;
			uMemberIndex = xPlayerInfoSortedListIt.GetCurrent()->GetGuid();
		}
		else if( uNumberDrawn == 0 )
		{
			// if this individual has a lower score than the last and there have been no draws so far the first is the highest, so return thier index (which should be 0)
			return uMemberIndex;
		}
		else
		{
			// else we have previously had a score draw but this player is below those drawn, so we have no need for the list anymore, just break. 
			break;
		}
	}

	// if we are here then we have players drawn for score, declare a new list to add these people to in order of total kills. 
	Asura_List<AVP_ClientEntity_PlayerInfo> m_axPlayersDrawnOnScore;
	
	// restart the original list iteration of all of those on this team. 
	xPlayerInfoSortedListIt.Restart();

	u_int uCount;

	for ( uCount = 0; uCount < uNumberDrawn; uCount ++ )
	{
		AVP_ClientEntity_PlayerInfo* pxPlayerInfo = xPlayerInfoSortedListIt.GetCurrent();

		// go through the number of people draw and order them by kills. 
		m_axPlayersDrawnOnScore.InsertInList( pxPlayerInfo, AVP_Multiplayer_ScoreBoard::SortByKills ); 

		xPlayerInfoSortedListIt.Next();
	}

	// now make an iterator of the new list. 
	Asura_List_ConstIt<AVP_ClientEntity_PlayerInfo> xPlayerInfoSortedListSECONDIt( m_axPlayersDrawnOnScore ); 

	// once again restart the first list. 
	xPlayerInfoSortedListIt.Restart();
	uCount = 0;

	for( ; !xPlayerInfoSortedListIt.Done(); xPlayerInfoSortedListIt.Next() )
	{
		if( xPlayerInfoSortedListSECONDIt.GetCurrent()->GetGuid() == xPlayerInfoSortedListIt.GetCurrent()->GetGuid() )
		{
			// if the first element of the list sorted by kills is the same as this one from the original list then this is the winner. 
			return xPlayerInfoSortedListIt.GetCurrent()->GetGuid();
		}
		uCount ++;
	}
	
	// else return the first element as they are draw on both scoreand kills
	xPlayerInfoSortedListIt.Restart();
	return xPlayerInfoSortedListIt.GetCurrent()->GetGuid();
}

////////////////////////////////////////////////////////////////////////////////
//  CLASS    :  AVP_Multiplayer_ScoreBoard
//  NAME     :  ConsoleCmd_PopulateLeaderBoard()
//  CREATED  :  CAH (18.12.2008)
//	SCOPE	 :	public static
//  DESC     :  PLACEHOLDER function for the 19/12/08 milestone, to fill the leaderboards with data, just to look nice. 
//				WILL BE REMOVED OR UPDATED when we have ranked multiplayer, online profiles and data saving to them. 
////////////////////////////////////////////////////////////////////////////////

bool AVP_Multiplayer_ScoreBoard::ConsoleCmd_PopulateLeaderBoard( u_int uElementIndex, bool bInterrogate, bool bHelp )
{
	if ( bInterrogate || bHelp )
	{ 
		return true; 
	} 

	// hard code the number of elements for now, as described in the description the idea is that this function doesnt actually do anything right now. 
	if( uElementIndex < 15 )
	{
		Asura_CommandConsole::Printf( L"N/A" );
		return true;
	}
	else
	{
		return false;
	}
}
