
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_Client.h
// Author : Andy Tate
// Date   : 5-Jul-2005
//
// Copyright (C) 2005 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __AVP_CLIENT_H__
#define __AVP_CLIENT_H__

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////


#include "SpecialFX/AVP_Decal_System.h"
#include "Core/Asura_Hash.h"
#include "Core/Asura_Maths.h"


#if defined(ASURA_TOOLS)
	#include "Misc/list_tem.hpp"
#endif	// defined(ASURA_TOOLS)


////////////////////////////////////////////////////////////////////////////////
// Forward class declarations...
////////////////////////////////////////////////////////////////////////////////

class Asura_Animation;
class Asura_Bounding_Box;
class Asura_Message;

class AVP_EntityDamage_ApplyDirectional;
class AVP_EntityDamage_ApplyToEntity;
class AVP_EntityDamage_ApplyToEnvironment;
class AVP_EntityDamage_ApplyRadialToBotanicals;
class AVP_EntityDamage_ApplyLinearToBotanicals;

////////////////////////////////////////////////////////////////////////////////
// Class declaration...
////////////////////////////////////////////////////////////////////////////////

class AVP_Client
{
public:
	
	// Message handing...
	static void	MessageHandler	(const Asura_Message* pxMessage);

protected:
	// Protected variables...


	// Protected functions...
	static bool	ApplyDamage(const AVP_EntityDamage_ApplyToEnvironment& xDamage);
	static bool	ApplyDamage(const AVP_EntityDamage_ApplyRadialToBotanicals& xDamage);
	static bool	ApplyDamage(const AVP_EntityDamage_ApplyLinearToBotanicals& xDamage);

	// Client request functions...
	static bool	SendJoinRequestMessage	(void);

	// Client message handling functions...
	static bool	ProcessJoinResponseMessage			(const Asura_Message* pxMessage);
	static bool	ProcessTerminationMessage			(const Asura_Message* pxMessage);
	static bool	ProcessRestartMessage				(const Asura_Message* pxMessage);
	static bool ProcessRestartAtCheckPointMessage	(const Asura_Message* pxMessage);
	static bool	ProcessRotateMessage				(const Asura_Message* pxMessage);
	static bool	ProcessGameTimeSyncMessage			(const Asura_Message* pxMessage);
	static bool	ProcessGameOverMessage				(const Asura_Message* pxMessage);

private:
	// Private variables...

	// Private functions...
	AVP_Client  (void);
	AVP_Client  (AVP_Client const& rOther);
	~AVP_Client ();

	AVP_Client&	operator= (AVP_Client const& rOther);

};   // End of class 'AVP_Client'.

#endif   // ifndef __AVP_CLIENT_H__


/// End of file. ///
