
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_Server.cpp
// Author : Andy Tate
// Date   : 16-Mar-2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "Network/AVP_Server.h"

// Game independent header-files...
#include "Core/AVP_Settings.h"
#include "Core/AVP_Core.h"
#include "Core/AVP_Game_Core.h"
#include "Core/AVP_Waypoint_Manager.h"
#include "Entity/AVP_Messages.h"
#include "Entity/AVP_Server_Messages.h"
#include "Entity/Damage/AVP_EntityDamage_Messages.h"
#include "Entity/Damage/AVP_ServerEntity_AcidDamage.h"
#include "Entity/PhysicalObject/Actor/AVP_ServerEntity_Actor.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/AVP_ClientActor_Vertebrate.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Predator/AVP_ServerActor_Predator.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Alien/AVP_ServerActor_Alien.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Human/Marine/AVP_ServerActor_Marine.h"
#include "Entity/PlayerInfo/AVP_ServerEntity_PlayerInfo.h"
#include "Input/AVP_Rumble_System.h"
#include "Multiplayer/AVP_Multiplayer_GameRules.h"
#include "Multiplayer/AVP_Multiplayer_TeamScore.h"
#include "Network/AVP_Player.h"

#if defined(ASURA_USE_GUI_MENU_SYSTEM)
#include "GUIMenu/AVP_GUIMenu_System.h"
#endif


////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////

DEBUGVAR float dbg_fClientInitialisationTimeoutLimit = 30.0f;
DEBUGVAR float dbg_fAllClientsInitialisedDelayTime = 1.0f;

////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////

Asura_LinkList<AVP_Server::AwaitingInitialisationData*> AVP_Server::s_xClientsInitialising;
float AVP_Server::s_fClientInitialisationTimeout = dbg_fClientInitialisationTimeoutLimit;
float AVP_Server::s_fAllClientsInitialisedDelay = dbg_fAllClientsInitialisedDelayTime;
bool AVP_Server::s_bCanInitialiseClients = false;

////////////////////////////////////////////////////////////////////////////////
// Private functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Protected functions...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Server::ProcessJoinRequestMessage
// Author       : Andy Tate
// Date         : 16-Mar-2007
////////////////////////////////////////////////////////////////////////////////

bool AVP_Server::ProcessJoinRequestMessage(const Asura_Message* pxMessage)
{
	if(Asura_Network::IsServer())
	{
#if 0
		Asura_Guid uReplyGuid = pxMessage->GetFromGuid();

		// TODO: check the client data.
		int iTerminationReason = AVP_CONNECTION_PROBLEM_NONE;

		const u_char* pucData = static_cast<const u_char*>(pxMessage->GetData());
		if(pucData)
		{
			// first char is number of players
			u_char ucNumPlayers = *pucData;
			// KOR - only one local player
			Asura_Assert(ucNumPlayers <= 1 /* AVP_MENU_PLATFORM_MAX_LOCAL_PLAYERS */, "Too many players?");

			Asura_Assert(pxMessage->GetDataSize() == sizeof(u_char) + (ucNumPlayers * sizeof(Gun_Msg_Player_JoinRequest_Data)), "Data size incorrect");

			// move on to first request struct...
			pucData++;

			for(u_int uPlayer = 0; uPlayer < static_cast<u_int>(ucNumPlayers); ++uPlayer)
			{
				Gun_Msg_Player_JoinRequest_Data xRequest;

				memcpy(&xRequest, pucData, sizeof(Gun_Msg_Player_JoinRequest_Data));

				Asura_Guid uClientGUID = xRequest.m_uGuid;

				Log(AVP_GAME_LOG_FLAGS, L"Join request from %ls (%d)", xRequest.m_wszPlayerName, xRequest.m_uGuid);

				// Check the client data.
				if (Gun_Settings::IsMultiplayer())
				{
					const AVP_LEVEL eClientLevel = static_cast<AVP_LEVEL>(xRequest.m_ucLevel);
					if (eClientLevel != Gun_Settings::GetLevel())
					{
						// Client has the wrong level loaded!
						// Client probably connected during a map rotation.
						iTerminationReason = AVP_CONNECTION_PROBLEM_WRONG_LEVEL;
						Asura_Assert(0, "Client has wrong level loaded!");
					}
				}

				// Check we have managed to load a zone before creating any players
				if( Asura_Streaming_System::GetNumZones( Asura_Streaming_System::ZONE_DATA ) == 0 )
				{
					iTerminationReason = AVP_CONNECTION_PROBLEM_SERVERBUSY;
					Asura_Assert( Asura_Streaming_System::GetNumZones( Asura_Streaming_System::ZONE_DATA ) > 0, "Server has no zones loaded yet. Cannot process join request." );
				}

				// create the server player
				if (iTerminationReason == AVP_CONNECTION_PROBLEM_NONE)
				{
					Gun_ServerHuman_Player* pxServerPlayer = Gun_ServerHuman_Player::FindEntity(uClientGUID);

					if (!pxServerPlayer)
					{
						switch( Gun_Settings::GetGameType() )
						{

						case AVP_GT_SP_POKER: //fallthrough
						case AVP_GT_QP_POKER:
							{
								//TI - i don't think we should do anything here if we're trying to start a single player poker game
							}

						case AVP_GT_MP_POKER:
							{
								if (Gun_Poker_Core::AddHumanPlayer( uClientGUID ))
								{
									Gun_ServerHuman_Player::Create( uClientGUID, Asura_Maths::ZeroVector3, Asura_Maths::IdentityMatrix3x3, 0 );

									if( Gun_Settings::IsMultiplayer() )
									{
										if(!Gun_MP_Scoring::GetPlayerInfo(uClientGUID))
										{
											// add player to scoring system
											Gun_MP_Scoring::Server_AddPlayer(uClientGUID, xRequest.m_wszPlayerName);
										}
									}
								}
								else
								{
									//poker game has already started, give server is full error.
									iTerminationReason = ASURA_NDR_SERVERFULL;
								}
							}

							break;

						default:

							// We're pretty sure they'll get in by now, so check if we need to disconnect a bot to fit this person in
							Gun_MultiplayerBot_System::HandlePlayerJoin();

							const int iStartPos = Server_FindStartPosition(uClientGUID);

							//					const AVP_CHARACTER_SKIN eSkin = static_cast<AVP_CHARACTER_SKIN>(xRequest.m_ucSkin);

#ifdef ASURA_TOOLS
							if(s_bUseStartPosOverride)
							{
								Asura_Matrix_3x3 xOri;
								xOri.Identity();
								Gun_ServerHuman_Player::Create( uClientGUID, s_xStartPosOverride, xOri);
								s_bUseStartPosOverride = false;
							}
							else
#endif
							{
								if (Gun_Settings::IsSinglePlayer() && s_bUseStoredPlayerStartPoint)
								{
									Gun_ServerHuman_Player::Create( uClientGUID, s_xLastPlayerPosition, s_xLastPlayerOrientation, xRequest.m_ucSkin );
								}
								else
								{
									Gun_ServerHuman_Player::Create( uClientGUID, Gun_Game_Core::GetStartPosition(iStartPos), Gun_Game_Core::GetStartOrientation(iStartPos), xRequest.m_ucSkin );
								}
							}

							if( Gun_Settings::IsMultiplayer() )
							{
								if(!Gun_MP_Scoring::GetPlayerInfo(uClientGUID))
								{
									// add player to scoring system
									Gun_MP_Scoring::Server_AddPlayer(uClientGUID, xRequest.m_wszPlayerName);
								}
							}
						}; // switch( Gun_Settings::GetGameType() )
					}
				}

				pucData += sizeof(Gun_Msg_Player_JoinRequest_Data);
			}

			if(iTerminationReason == AVP_CONNECTION_PROBLEM_NONE)
			{
				// send the acceptance message
				return SendJoinResponseMessage(uReplyGuid);
			}
			else if (iTerminationReason == AVP_CONNECTION_PROBLEM_WRONG_LEVEL)
			{
				// Tell the client that he has got the wrong level.
				return SendRotateMessage(ASURA_GUID_PLAYERS_CLIENTMACHINE, uReplyGuid);
			}
			else
			{
				// reject this player
				return SendTerminationMessage(iTerminationReason, ASURA_GUID_PLAYERS_CLIENTMACHINE, uReplyGuid, true);
			}
		}

		// only get here if there's a problem with a message... terminate.
		return SendTerminationMessage(AVP_CONNECTION_PROBLEM_ERROR_IN_MESSAGE, ASURA_GUID_PLAYERS_CLIENTMACHINE, uReplyGuid, true);
#else
		Asura_Assert(false, "AVP_Server::ProcessJoinRequestMessage(): Not yet implemented.");
		return(false);
#endif
	}
	else
	{
		Asura_Assert(false, "Should only be called on servers");
		return(false);
	}
}

/// End of function 'AVP_Server::ProcessJoinRequestMessage'. ///


////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Server::ProcessJoinReadyMessage
// Author       : Juan Garcia
// Date         : 2-Jun-2008
////////////////////////////////////////////////////////////////////////////////

bool AVP_Server::ProcessJoinReadyMessage(const Asura_Message* pxMessage)
{
	if(AVP_Settings::IsMultiPlayer())
	{
		if(Asura_Network::IsServer())
		{
			// Activate the player puppet
			const AVP_Message_JoinReady* pxData = static_cast<const AVP_Message_JoinReady*>( pxMessage->GetData() ); 
			AVP_ServerEntity_PlayerInfo* pxPlayerInfo = AVP_ServerEntity_PlayerInfo::FindEntity( pxData->GetPlayerGuid() ); 
			if (pxPlayerInfo)
			{
				AVP_ServerEntity_Actor* pxPlayerPuppet = AVP_ServerEntity_Actor::FindEntity( pxPlayerInfo->GetPuppetGuid() ); 
				if ( pxPlayerPuppet )
				{
					pxPlayerPuppet->SetInitiallyInactive( false );
					u_int uRequestedFlags = ASURA_PHYSICALOBJECT_FLAG_INVISIBLE | ASURA_PHYSICALOBJECT_FLAG_NOCOLLISIONS; 
					pxPlayerPuppet->SetPhysicalObjectFlags( pxPlayerPuppet->GetPhysicalObjectFlags() & ~uRequestedFlags ); 
					Asura_Message* pxMessage = Asura_Network::CreateMessage( ASURA_MSG_ACTIVATE );
					if( pxMessage )
					{
						pxMessage->SetEntityClassification(pxPlayerPuppet->GetClassification());
						pxMessage->SetToGuid(ASURA_GUID_MY_CLIENT_ENTITIES);
						pxMessage->SetFromGuid(pxPlayerInfo->GetPuppetGuid());
						pxMessage->UseGuaranteedDelivery();
						Asura_Network::Send(pxMessage);
					}
				}
				else
				{
					// No player actor in the server, it shouldn't get here
					Asura_Assert( pxPlayerPuppet, "Couldn't find player actor in the server" ); 
					return(false);
				}
				return(true); 
			}
			else
			{
				// No player actor in the server, it shouldn't get here
				Asura_Assert( pxPlayerInfo, "Couldn't find player info in the server" ); 
				return(false);
			}
		}
		else
		{
			Asura_Assert(false, "Should only be called on servers");
			return(false);
		}
	}
	return false;
}

/// End of function 'AVP_Server::ProcessJoinReadyMessage'. ///


////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Server::ProcessPlayerConnectedMessage
// Author       : Andy Tate
// Date         : 16-Mar-2007
////////////////////////////////////////////////////////////////////////////////

bool AVP_Server::ProcessPlayerConnectedMessage(const Asura_Message* pxMessage)
{
	if(Asura_Network::IsServer())
	{
		const Asura_MsgData_PlayerConnected*	pxData		= static_cast<const Asura_MsgData_PlayerConnected*>(pxMessage->GetData());
		const Asura_Guid						uPlayerGuid	= (pxData == NULL) ? ASURA_GUID_FIRSTPLAYER : pxData->PlayerGuid;

		AwaitingInitialisationData* pxInitialisationData = new AwaitingInitialisationData;
		if (!pxInitialisationData)
		{
			// TODO - Some nicer handling, such as disconnecting the player or asking them to request again.
			Asura_Assertf(false, ("Failed to create player with GUID %d.", uPlayerGuid));
			Asura_Network::RemovePlayer(uPlayerGuid, ASURA_NDR_SERVERFULL); // Probably want some other project-specific response, but in a way the server is full (it's created as many things as it can).
			return false;
		}

		pxInitialisationData->m_uClientGuid = uPlayerGuid;
		pxInitialisationData->m_eInitialisationState = AVP_CLIENTAWAITINGINITIALISATION_STATE_CREATEPLAYERINFO;
		pxInitialisationData->m_iSpeciesToCreate = -1; // Invalid, so we know if the message has been sent.
		pxInitialisationData->m_uBlueprintHashToCreate = ASURA_HASH_ID_UNSET;
		pxInitialisationData->m_iRequestedMultiplayerTeam = AVP_Settings::AVP_TEAM_NONE; 
		pxInitialisationData->m_iAssignedMultiplayerRole = AVP_Settings::AVP_ROLE_NORMAL;

		// Set up the timeout if there's no-one currently initialising.
		if (s_xClientsInitialising.Size() == 0)
		{
			s_fClientInitialisationTimeout = dbg_fClientInitialisationTimeoutLimit;
		}

		s_xClientsInitialising.Push_Front(pxInitialisationData);

		if (s_bCanInitialiseClients)
		{
			AVP_ServerEntity_PlayerInfo* pxPlayerInfo = CreatePlayerInfo(pxInitialisationData);
			if (!pxPlayerInfo)
			{
				// TODO - Some nicer handling, such as disconnecting the player or asking them to request again.
				Asura_Assertf(false, ("Failed to create player with GUID %d.", uPlayerGuid));
				Asura_Network::RemovePlayer(uPlayerGuid, ASURA_NDR_SERVERFULL); // Probably want some other project-specific response, but in a way the server is full (it's created as many entities as it can).
				return false;
			}
		}
		// No else case, as the player info will be created later if the bool is false.

		// Set this player as ignoring net messages, and add them to our list preparing to receive create messages.
#ifdef ASURA_SUPPORT_NETMSG_SYSTEM
		Asura_Network::AddClientToIgnoreNetMsgUpdatesList(uPlayerGuid);
#endif //#ifdef ASURA_SUPPORT_NETMSG_SYSTEM

		return(true);
	}
	else
	{
		Asura_Assert(false, "Should only be called on servers.");
		return(false);
	}
}

/// End of function 'AVP_Server::ProcessPlayerConnectedMessage'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Server::ProcessPlayerDisconnectedMessage
// Author       : Andy Tate
// Date         : 16-Mar-2007
////////////////////////////////////////////////////////////////////////////////

bool AVP_Server::ProcessPlayerDisconnectedMessage(const Asura_Message* pxMessage)
{
	if(Asura_Network::IsServer())
	{
		const Asura_MsgData_PlayerDisconnected*	pxData	= static_cast<const Asura_MsgData_PlayerDisconnected*>(pxMessage->GetData());

		if(pxData)
		{
#if 0
			// remove player from scoring system
			Gun_MP_Scoring::Server_RemovePlayer(pxData->PlayerGuid);
#endif

			Asura_Message* pxNewMsg = Asura_Network::CreateMessage(ASURA_MSG_DESTRUCT,0);
			if(pxNewMsg)
			{
				pxNewMsg->To = pxData->PlayerGuid;
				pxNewMsg->From = ASURA_GUID_SERVER;
				pxNewMsg->UseGuaranteedDelivery();

				Asura_Network::Send(pxNewMsg);
			}

			// Make sure they're no longer marked as awaiting create messages.
			// Asura_Network should already have removed them from the lists there, so just remove them from the local list.
			Asura_ListElt<AwaitingInitialisationData*>* pxElement = s_xClientsInitialising.Front();
			while (pxElement)
			{
				Asura_ListElt<AwaitingInitialisationData*>* pxCurrentElement = pxElement;
				pxElement = pxElement->Next();
				AwaitingInitialisationData* pxInitialisationData = pxCurrentElement->Data();

				if (pxInitialisationData->m_uClientGuid == pxData->PlayerGuid)
				{
					s_xClientsInitialising.Remove(pxCurrentElement);
					delete pxInitialisationData;
					break;
				}
			}
		}

		return(true);
	}
	else
	{
		Asura_Assert(false, "Should only be called on servers");
		return(false);
	}
}

/// End of function 'AVP_Server::ProcessPlayerDisconnectedMessage'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Server::ProcessEndOfLevelMessage
// Author       : Andy Tate
// Date         : 16-Mar-2007
////////////////////////////////////////////////////////////////////////////////

bool AVP_Server::ProcessEndOfLevelMessage(const Asura_Message* pxMessage)
{
	AVP_Core::RequestLevelComplete();
	return(true);
}

/// End of function 'AVP_Server::ProcessEndOfLevelMessage'. ///


////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Server::ProcessPlayerActorCreateMessage
// Author       : Anthony Gregson
// Date         : 06/11/07
////////////////////////////////////////////////////////////////////////////////

bool AVP_Server::ProcessPlayerActorCreateMessage(const Asura_Message* pxMessage)
{
	if(Asura_Network::IsServer())
	{
		const AVP_Message_CreatePlayerActor* pxData	 = static_cast<const AVP_Message_CreatePlayerActor*>(pxMessage->GetData());
		const Asura_Guid uPlayerGuid	= pxMessage->From;

		if (pxData->m_bRespawn)
		{
			// On respawn, just create a new actor.
			AVP_ServerEntity_PlayerInfo* pxPlayerInfo = AVP_ServerEntity_PlayerInfo::FindEntity(uPlayerGuid);
			if (!pxPlayerInfo)
			{
				return false;
			}

			// Additional checks for multiplayer games
			if ( AVP_Settings::IsMultiPlayer() )
			{
				// Is a team based game where the number of lives (respawns) is shared?
				if ( AVP_Multiplayer_GameRules::IsTeamBasedGame() && AVP_Multiplayer_GameRules::GetTeamLivesOn() )
				{
					// Yes, check if our team is allowed another respawn
					AVP_Settings::Team eTeam = pxPlayerInfo->GetMultiplayerTeam(); 
					AVP_Multiplayer_TeamScore* pxTeamScore = AVP_Multiplayer_TeamScore::FindTeamScore( eTeam ); 
					if ( pxTeamScore )
					{
						if ( pxTeamScore->GetNumberOfTeamLives() > 0 )
						{
							// OK, we can respawn, but we will be taking out a live from the shared pool
							pxTeamScore->SetNumberOfTeamLives( pxTeamScore->GetNumberOfTeamLives() - 1 ); 
						}
						else
						{
							// No lives left for this team, sorry but we can respawn
							return false; 
						}
					}
					else
					{
						Asura_Assertf( false, ( "Player %d belongs to team %d but there is no team score entry, respawning now but this bug needs fixing!", uPlayerGuid, eTeam ));  
					}
				}
			}

			AwaitingInitialisationData xTempData;
			xTempData.m_uClientGuid = uPlayerGuid;
			xTempData.m_eInitialisationState = AVP_CLIENTAWAITINGINITIALISATION_STATE_CREATEACTOR;
			xTempData.m_iSpeciesToCreate = static_cast<int>(pxData->m_uRequestedSpecies);
			xTempData.m_uBlueprintHashToCreate = AVP_Game_Core::GetPlayerBlueprintHashForSpecies( static_cast<AVP_Settings::Species>(xTempData.m_iSpeciesToCreate) ); 
			xTempData.m_iRequestedMultiplayerTeam = static_cast<AVP_Settings::Team>(pxData->m_uRequestedMultiplayerTeam);
			xTempData.m_iAssignedMultiplayerRole = pxPlayerInfo->GetMultiplayerRole(); 

			CreatePlayerActor(&xTempData, pxPlayerInfo);

			return true;
		}

		AVP_ServerEntity_PlayerInfo* pxPlayerInfo = AVP_ServerEntity_PlayerInfo::FindEntity(uPlayerGuid);
		if (pxPlayerInfo)
		{
			// If our player info already exists, our actor might already exist too (happens if restoring a saved game).
			Asura_Guid uPuppetGuid = pxPlayerInfo->GetPuppetGuid();
			if (uPuppetGuid != ASURA_GUID_UNREGISTERED)
			{
				if (AVP_Settings::IsMultiPlayer() && AVP_ServerEntity_Actor::FindEntity(uPuppetGuid))
				{
					// This player already has a valid puppet. They must have managed to request it again through lag or cheating. Ignore them.
					// No need to assert here, I can see this happening without any kind of error beyond some lag.
					// Note this is multiplayer-only. In single player, the actor already existing here is relied upon in order to make save games work properly.
					return true;
				}

				// Both actor and player info exist, we expect not to be already in the initialisation data at this point in that case
				Asura_ListElt<AwaitingInitialisationData*>* pxElement = s_xClientsInitialising.Front();

				while (pxElement)
				{
					if (pxElement->Data()->m_uClientGuid == uPlayerGuid)
					{
						break;
					}
					pxElement = pxElement->Next();
				}
				if (!pxElement)
				{
					// As expected, not here, create a new entry starting part-way through the process.
					AwaitingInitialisationData* pxInitialisationData = new AwaitingInitialisationData;
					if (!pxInitialisationData)
					{
						// TODO - Some nicer handling, such as disconnecting the player or asking them to request again.
						Asura_Assertf(false, ("Failed to create player with GUID %d.", uPlayerGuid));
						Asura_Network::RemovePlayer(uPlayerGuid, ASURA_NDR_SERVERFULL); // Probably want some other project-specific response, but in a way the server is full (it's created as many things as it can).
						return false;
					}

					pxInitialisationData->m_uClientGuid = uPlayerGuid;
					pxInitialisationData->m_eInitialisationState = AVP_CLIENTAWAITINGINITIALISATION_STATE_ADDTOINITIALISATIONLIST;
					pxInitialisationData->m_iSpeciesToCreate = static_cast<int>(pxData->m_uRequestedSpecies);
					pxInitialisationData->m_uBlueprintHashToCreate = AVP_Game_Core::GetPlayerBlueprintHashForSpecies( static_cast<AVP_Settings::Species>(pxInitialisationData->m_iSpeciesToCreate) ); 
					pxInitialisationData->m_iRequestedMultiplayerTeam = static_cast<AVP_Settings::Team>(pxData->m_uRequestedMultiplayerTeam); 
					pxInitialisationData->m_iAssignedMultiplayerRole = AVP_Settings::AVP_ROLE_NORMAL;

					if (s_xClientsInitialising.Size() == 0)
					{
						s_fClientInitialisationTimeout = dbg_fClientInitialisationTimeoutLimit;
					}

					s_xClientsInitialising.Push_Front(pxInitialisationData);

#ifdef ASURA_SUPPORT_NETMSG_SYSTEM
					Asura_Network::AddClientToIgnoreNetMsgUpdatesList(uPlayerGuid);
#endif //#ifdef ASURA_SUPPORT_NETMSG_SYSTEM
				}
				else
				{
					// Hopefully this means we've just had a delayed, resent message. Do no more.
					Asura_Assert(pxElement->Data()->m_eInitialisationState > AVP_CLIENTAWAITINGINITIALISATION_STATE_CREATEACTOR, "Unexpected initialisation state when both player info and actor exist.");
				}

				return true;
			}
		}

		// Make sure we're at least in the list of those waiting for a player info entity
		Asura_ListElt<AwaitingInitialisationData*>* pxElement = s_xClientsInitialising.Front();
		while (pxElement)
		{
			if (pxElement->Data()->m_uClientGuid == uPlayerGuid)
			{
				break;
			}
			pxElement = pxElement->Next();
		}

		AwaitingInitialisationData* pxInitialisationData = NULL;

		if (!pxElement)
		{
			// Hopefully, this only happens on multiplayer game restarts.
			Asura_Assert(pxPlayerInfo, "Expected a player info to already exist in this situation.");

			pxInitialisationData = new AwaitingInitialisationData;
			if (!pxInitialisationData)
			{
				// TODO - Some nicer handling, such as disconnecting the player or asking them to request again.
				Asura_Assertf(false, ("Failed to create player with GUID %d.", uPlayerGuid));
				Asura_Network::RemovePlayer(uPlayerGuid, ASURA_NDR_SERVERFULL); // Probably want some other project-specific response, but in a way the server is full (it's created as many things as it can).
				return false;
			}

			pxInitialisationData->m_uClientGuid = uPlayerGuid;
			pxInitialisationData->m_eInitialisationState = AVP_CLIENTAWAITINGINITIALISATION_STATE_CREATEACTOR;
			pxInitialisationData->m_iSpeciesToCreate = static_cast<int>(pxData->m_uRequestedSpecies);
			pxInitialisationData->m_uBlueprintHashToCreate = AVP_Game_Core::GetPlayerBlueprintHashForSpecies( static_cast<AVP_Settings::Species>(pxInitialisationData->m_iSpeciesToCreate) ); 
			pxInitialisationData->m_iRequestedMultiplayerTeam = static_cast<AVP_Settings::Team>(pxData->m_uRequestedMultiplayerTeam); 
			pxInitialisationData->m_iAssignedMultiplayerRole = AVP_Settings::AVP_ROLE_NORMAL; 

			if (s_xClientsInitialising.Size() == 0)
			{
				s_fClientInitialisationTimeout = dbg_fClientInitialisationTimeoutLimit;
			}

			s_xClientsInitialising.Push_Front(pxInitialisationData);

#ifdef ASURA_SUPPORT_NETMSG_SYSTEM
			Asura_Network::AddClientToIgnoreNetMsgUpdatesList(uPlayerGuid);
#endif //#ifdef ASURA_SUPPORT_NETMSG_SYSTEM
		}
		else
		{
			pxInitialisationData = pxElement->Data();
		}

		// Add the actor create data immediately.
		pxInitialisationData->m_iSpeciesToCreate = static_cast<int>(pxData->m_uRequestedSpecies);
		pxInitialisationData->m_uBlueprintHashToCreate = AVP_Game_Core::GetPlayerBlueprintHashForSpecies( static_cast<AVP_Settings::Species>(pxInitialisationData->m_iSpeciesToCreate) ); 
		pxInitialisationData->m_iRequestedMultiplayerTeam = static_cast<AVP_Settings::Team>(pxData->m_uRequestedMultiplayerTeam);
		pxInitialisationData->m_iAssignedMultiplayerRole = AVP_Settings::AVP_ROLE_NORMAL; 

		if (pxInitialisationData->m_eInitialisationState == AVP_CLIENTAWAITINGINITIALISATION_STATE_CREATEPLAYERINFO)
		{
			// Still waiting to create the player info, this is as far as we go for now.
			return true;
		}

		if (!pxPlayerInfo)
		{
			// TODO - Some nicer handling, such as disconnecting the player or asking them to request again.
			Asura_Assertf(false, ("Couldn't find player info %d. Perhaps they've already left.", uPlayerGuid));
			return false;
		}

		CreatePlayerActor(pxInitialisationData, pxPlayerInfo, AVP_Settings::IsMultiPlayer());

		return(true);
	}
	else
	{
		Asura_Assert(false, "Should only be called on servers.");
		return(false);
	}
}

/// End of function 'AVP_Server::ProcessPlayerActorCreateMessage'. ///




////////////////////////////////////////////////////////////////////////////////
// Public functions...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Server::MessageHandler
// Author       : Andy Tate
// Date         : 16-Mar-2007
////////////////////////////////////////////////////////////////////////////////

void AVP_Server::MessageHandler(const Asura_Message* pxMessage)
{
	switch (pxMessage->GetMsgID())
	{
		case AVP_MSGID_CONVERT_PLAYER_TO_AI:
		{
			// AJG - 09/10/07 - Disabled handling of this for now, as it isn't working at the moment and I don't know exactly how to tie everything in to fix it.
//			ProcessConvertPlayerToAIMessage(pxMessage);
			break;
		}

		case AVP_MSGID_JOINGAME_REQUEST:
		{
			ProcessJoinRequestMessage(pxMessage);
			break;
		}

		case AVP_MSGID_JOINGAME_READY:
		{
			ProcessJoinReadyMessage(pxMessage);
			break;
		}

		case ASURA_MSG_PLAYERCONNECTED:
		{
			ProcessPlayerConnectedMessage(pxMessage);
			break;
		}

		case ASURA_MSG_PLAYERDISCONNECTED:
		{
			ProcessPlayerDisconnectedMessage(pxMessage);
			break;
		}

		case AVP_MSGID_END_OF_LEVEL:
		{
			ProcessEndOfLevelMessage(pxMessage);
			break;
		}

		case AVP_MSGID_CREATEPLAYERACTOR:
		{
			ProcessPlayerActorCreateMessage(pxMessage);
			break;
		}

		case AVP_MSGID_CREATE_ACID_DAMAGE:
		{
			AVP_ServerEntity_AcidDamage::Create(pxMessage);
			break;
		}

		case AVP_MSGID_LIGHT_CHANGE_EVENT:
		case AVP_MSGID_LIGHT_CHANGE_EVENT_CODE:
		{
			AVP_Waypoint_Manager::HandleMessage( pxMessage );
			break;
		}

		default:
		{
			Asura_Assert(false, "Message not handled by server.");
			break;
		}
	}		// End of switch.
}

/// End of function 'AVP_Server::MessageHandler'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Server::SendJoinResponseMessage
// Author       : Andy Tate
// Date         : 16-Mar-2007
////////////////////////////////////////////////////////////////////////////////

bool AVP_Server::SendJoinResponseMessage(const Asura_Guid uPlayer)
{
	if(Asura_Network::IsServer())
	{
#if 0
		Asura_Message*	pxMessage	= Asura_Network::CreateMessage(AVP_MSGID_JOINGAME_RESPONSE, sizeof(AVP_Message_JoinResponse));

		if(pxMessage == NULL)
		{
			return(false);
		}
		else
		{
			AVP_Message_JoinResponse*	pxData	= static_cast<AVP_Message_JoinResponse*>(pxMessage->GetData());

			// Whilst the player was loading the level, the server may have changed the settings.
			// Send another copy of the session data to make sure that he is up to date.
			if(pxData != NULL)
			{
				pxData->Set();
			}

			Log(AVP_GAME_LOG_FLAGS, L"Sending join response");

			pxMessage->To	= ASURA_GUID_PLAYERS_CLIENTMACHINE;
			pxMessage->From	= uPlayer;
			pxMessage->UseGuaranteedDelivery();
			Asura_Network::Send(pxMessage);
			return(true);
		}
#else
		Asura_Assert(false, "AVP_Server::SendJoinResponseMessage(): Not yet implemented.");
		return(false);
#endif
	}
	else
	{
		Asura_Assert(false, "Should only be called on servers");
		return(false);
	}
}

/// End of function 'AVP_Server::SendJoinResponseMessage'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Server::SendTerminationMessage
// Author       : Andy Tate
// Date         : 16-Mar-2007
////////////////////////////////////////////////////////////////////////////////

bool AVP_Server::SendTerminationMessage(const int iReason,
										const Asura_Guid uTo,
										const Asura_Guid uFrom,
										const bool bGuaranteed)
{
	if(Asura_Network::IsServer())
	{
		Asura_Message*	pxMessage	= Asura_Network::CreateMessage(AVP_MSGID_GAME_TERMINATED, sizeof(u_char));

		if(pxMessage == NULL)
		{
			Asura_Assert(false, "Cannot create message.");
			return(false);
		}
		else
		{
			u_char*	pxData	= reinterpret_cast<u_char*>(pxMessage->GetData());

			*pxData			= static_cast<u_char>(iReason);

			pxMessage->From = uFrom;
			pxMessage->To   = uTo;

			if (bGuaranteed)
			{
				pxMessage->UseGuaranteedDelivery();
			}

			Asura_Network::Send(pxMessage);
			return(true);
		}
	}
	else
	{
		Asura_Assert(false, "Should only be called on servers");
		return(false);
	}
}

/// End of function 'AVP_Server::SendTerminationMessage'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Server::SendRestartMessage
// Author       : Andy Tate
// Date         : 16-Mar-2007
////////////////////////////////////////////////////////////////////////////////

bool AVP_Server::SendRestartMessage(void)
{
	if(Asura_Network::IsServer())
	{
#if 0
		if(Gun_Settings::IsSinglePlayer())
		{
			Asura_Assert(0, "Server_SendRestart is only for MP games");
			return false;
		}

		Asura_Environment* pxEnv = Asura_Environment_System::GetEnvironment();
		if (!pxEnv)
		{
			Asura_Assert(0, "Null environment");
			return false;
		}
		const int iNumStart = Gun_Game_Core::GetNumberOfStartPositions();

		// Reset MP scores and timers, before sending the messages.
		Gun_MP_Scoring::Restart();
		Gun_Game_Core::ResetLevelStatus(true);

		// Send each player a restart level message, but send each one to a different spawn point.
		const unsigned int uNumPlayers = Gun_MP_Scoring::GetNumberOfPlayers();
		Asura_Assert(uNumPlayers<=uAVP_MAX_NET_PLAYERS, "Error: Too many players!");

		// Keep track of the used spawn points.
		int aiSpawnPoints[uAVP_MAX_NET_PLAYERS]; // = { -1, -1, -1, -1, -1, -1 };
		memset(aiSpawnPoints, -1, uAVP_MAX_NET_PLAYERS);

		for (u_int u=0; u<uNumPlayers && u<static_cast<u_int>(iNumStart); ++u)
		{
			bool bOccupied = false;
			do
			{
				bOccupied = false;
				(aiSpawnPoints)[u] = Asura_Random::GetSeededInt(0, iNumStart-1);
				for (u_int v=0; v<u; ++v)
				{
					if ((aiSpawnPoints)[u] == (aiSpawnPoints)[v])
					{
						bOccupied = true;
						break;
					}
				}
			}
			while (bOccupied);
		}


		// Send the messages.
		for (u_int u=0; u<uNumPlayers; ++u)
		{
			const Gun_PlayerInfo* pxInfo = Gun_MP_Scoring::GetPlayerInfoByIndex(u);
			Asura_Assert(pxInfo, "Null player info");
			if (!pxInfo) continue;

			if (Gun_MultiplayerBot_System::GetIsBot(pxInfo->GetPlayerGuid()))
			{
				// Bots are on the server, and will have this message sent to the player there, so no need to do anything with them.
				continue;
			}

			Asura_Message* pxMessage = Asura_Network::CreateMessage(AVP_MSGID_GAME_RESTART, sizeof(Gun_Message_Game_Restart));
			Asura_Assert(pxMessage, "Null message");
			if (pxMessage)
			{
				Gun_Message_Game_Restart* pxData = static_cast<Gun_Message_Game_Restart*>(pxMessage->GetData());
				if (pxData)
				{
					pxData->Set();
					pxData->m_sSpawnPoint = static_cast<short>((aiSpawnPoints)[u]);
				}

				const Asura_Guid uFrom = pxInfo->GetPlayerGuid();
				const Asura_Guid uTo = ASURA_GUID_PLAYERS_CLIENTMACHINE;

				pxMessage->From = uFrom;
				pxMessage->To   = uTo;
				pxMessage->UseGuaranteedDelivery();

				Asura_Network::Send(pxMessage);

				Log(AVP_GAME_LOG_FLAGS, L"Sending restart message to %d from %d", uTo, uFrom);
			}
		}

		return true;
#else
		Asura_Message*	pxMessage	= Asura_Network::CreateMessage(AVP_MSGID_GAME_RESTART, 0);

		if(pxMessage == NULL)
		{
			Asura_Assert(false, "Cannot create message.");
			return(false);
		}
		else
		{
			pxMessage->From = ASURA_GUID_SERVER;
			pxMessage->To   = ASURA_GUID_ALL_OTHER_CLIENTMACHINES;

			pxMessage->UseGuaranteedDelivery();

			Asura_Network::Send(pxMessage);
			return(true);
		}
#endif
	}
	else
	{
		Asura_Assert(false, "Should only be called on servers.");
		return(false);
	}
}

/// End of function 'AVP_Server::SendRestartMessage'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Server::SendRotateMessage
// Author       : Andy Tate
// Date         : 16-Mar-2007
////////////////////////////////////////////////////////////////////////////////

bool AVP_Server::SendRotateMessage(const Asura_Guid uTo,
								   const Asura_Guid uFrom)
{
	if(Asura_Network::IsServer())
	{
#if 0
		if (Gun_Settings::IsSinglePlayer())
		{
			Asura_Assert(0, "Server_SendRotate is only for MP games");
			return false;
		}

		// TDP 24/03/06
		// Increment the level. TI - 10.7.06 - Check if multiplayer is unlocked before rotating
		if( Gun_Settings::GetLevel() == AVP_LAST_MULTIPLAYER_LEVEL )
		{
			Gun_Settings::SetLevel( AVP_FIRST_MULTIPLAYER_LEVEL );
		}
		else
		{
			const Gun_PlayerProfile* pxPlayerProfile = Gun_Settings::GetPlayerProfile(0);
			if (!pxPlayerProfile)
			{
				Asura_Assert(false, "Server_SendRotate: Could not get player profile");
				return false;
			}
			AVP_LEVEL eNextLevel = Gun_Settings::GetLevel();

			do
			{
				if( eNextLevel == AVP_LAST_MULTIPLAYER_LEVEL )
				{
					eNextLevel = AVP_FIRST_MULTIPLAYER_LEVEL;
				}
				else
				{
					eNextLevel = static_cast<AVP_LEVEL>( eNextLevel + 1 );
				}

			}while (!pxPlayerProfile->IsMultiplayerLevelAvailable(eNextLevel));

			Gun_Settings::SetLevel( static_cast<const AVP_LEVEL>( eNextLevel ) );
		}

		// Reset the scoreboard
		Gun_MP_Scoring::Restart();

		Asura_Message* pxMessage = Asura_Network::CreateMessage(AVP_MSGID_GAME_ROTATE, sizeof(Gun_Message_Game_Rotate));
		Asura_Assert(pxMessage, "Null message");

		if (pxMessage)
		{	
			Gun_Message_Game_Rotate* pxData = static_cast<Gun_Message_Game_Rotate*>(pxMessage->GetData());
			if (pxData)
			{
				// Copy data about game type and map.
				pxData->Set();
			}

			pxMessage->From = uFrom;
			pxMessage->To   = uTo;
			pxMessage->UseGuaranteedDelivery();

			Asura_Network::Send(pxMessage);

			Log(AVP_GAME_LOG_FLAGS, L"Sending rotate message to %d from %d", uTo, uFrom);
			return true;
		}

		return false;
#else
		Asura_Assert(false, "AVP_Server::SendRotateMessage(): Not yet implemented.");
		return(false);
#endif
	}
	else
	{
		Asura_Assert(false, "Should only be called on servers.");
		return(false);
	}
}

/// End of function 'AVP_Server::SendRotateMessage'. ///

//------------------------------------------------------------------------------
// Function     : AVP_Server::ProcessConvertPlayerToAIMessage
// Author       : Gareth Thomas
// Date         : 31 / 05 07
//------------------------------------------------------------------------------
bool AVP_Server::ProcessConvertPlayerToAIMessage(const Asura_Message* pxMessage)
{
	AVP_ServerEntity_Actor* pxPlayerActor = AVP_ServerEntity_Actor::FindEntity(1000);

	if (pxPlayerActor)
	{
		Asura_Message* pxForwardMessage = Asura_Network::CreateMessage(AVP_MSGID_CONVERT_PLAYER_TO_AI, sizeof(AVP_Message_Convert_Player_To_AI));

		if (pxForwardMessage)
		{
			pxForwardMessage->SetToGuid(1000); // Always Set the Players GUID
			pxForwardMessage->SetFromGuid(ASURA_GUID_UNREGISTERED);
			pxForwardMessage->UseGuaranteedDelivery();
			pxPlayerActor->HandleMessage(pxForwardMessage);
			Asura_Network::DestroyMessage(pxForwardMessage);
			return true;
		}
	}

	return false;
}
/// End of function 'AVP_Server::ProcessConvertPlayerToAIMessage'. ///


////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Server::SendGameOverMessage
// Author       : Andy Tate
// Date         : 16-Mar-2007
////////////////////////////////////////////////////////////////////////////////

bool AVP_Server::SendGameOverMessage(void)
{
	if(Asura_Network::IsServer())
	{
		Asura_Message*	pxMessage	= Asura_Network::CreateMessage(AVP_MSGID_GAME_OVER, 0);

		if(pxMessage == NULL)
		{
			Asura_Assert(false, "Cannot create message.");
			return(false);
		}
		else
		{
			pxMessage->From = ASURA_GUID_SERVER;
			pxMessage->To   = ASURA_GUID_ALL_OTHER_CLIENTMACHINES;

			pxMessage->UseGuaranteedDelivery();

			Asura_Network::Send(pxMessage);
			return(true);
		}
	}
	else
	{
		Asura_Assert(false, "Should only be called on servers.");
		return(false);
	}
}

/// End of function 'AVP_Server::SendGameOverMessage'. ///

void AVP_Server::UpdateClientInitialisation()
{
	if (!s_bCanInitialiseClients) { return; }

	if (s_xClientsInitialising.Size() == 0) { return; }

	bool bAllWaitingToContinue = true;

	Asura_ListElt<AwaitingInitialisationData*>* pxElement = s_xClientsInitialising.Front();

	while (pxElement)
	{
		AwaitingInitialisationData* pxData = pxElement->Data();
		pxElement = pxElement->Next();

		bAllWaitingToContinue &= pxData->m_eInitialisationState == AVP_CLIENTAWAITINGINITIALISATION_STATE_WAITINGTOCONTINUE;

		switch (pxData->m_eInitialisationState)
		{
		case AVP_CLIENTAWAITINGINITIALISATION_STATE_CREATEPLAYERINFO:
			{
				AVP_ServerEntity_PlayerInfo* pxPlayerInfo = CreatePlayerInfo(pxData);
				if (!pxPlayerInfo)
				{
					// TODO - Some nicer handling, such as disconnecting the player or asking them to request again.
					Asura_Assertf(false, ("Failed to create player with GUID %d.", pxData->m_uClientGuid));
					Asura_Network::RemovePlayer(pxData->m_uClientGuid, ASURA_NDR_SERVERFULL); // Probably want some other project-specific response, but in a way the server is full (it's created as many entities as it can).
				}
			}
			break;

		case AVP_CLIENTAWAITINGINITIALISATION_STATE_CREATEACTOR:
			{
				if (pxData->m_iSpeciesToCreate != -1)
				{
					AVP_ServerEntity_PlayerInfo* pxPlayerInfo = AVP_ServerEntity_PlayerInfo::FindEntity(pxData->m_uClientGuid);
					if (!pxPlayerInfo)
					{
						// TODO - Some nicer handling, such as disconnecting the player or asking them to request again.
						Asura_Assertf(false, ("Couldn't find player info %d. Perhaps they've already left.", pxData->m_uClientGuid));
					}
					else
					{
						CreatePlayerActor(pxData, pxPlayerInfo);
					}
				}
			}
			break;

		case AVP_CLIENTAWAITINGINITIALISATION_STATE_ADDTOINITIALISATIONLIST:
			{
				pxData->m_eInitialisationState = AVP_CLIENTAWAITINGINITIALISATION_STATE_CLIENTINITIALISING;
				Asura_Network::RegisterClientForCreateMessages(pxData->m_uClientGuid);
			}
			break;

		case AVP_CLIENTAWAITINGINITIALISATION_STATE_CLIENTINITIALISING:
		case AVP_CLIENTAWAITINGINITIALISATION_STATE_WAITINGTOCONTINUE:
			// Nothing to do in either of these states, we're just waiting for things to happen.
			break;

		default:
			Asura_Assert(false, "Invalid initialising state.");
			break;
		}
	}

	s_fClientInitialisationTimeout -= Asura_Timers::GetFrameTime();

	if (bAllWaitingToContinue)
	{
		s_fAllClientsInitialisedDelay -= Asura_Timers::GetFrameTime();
	}

	if ((s_fClientInitialisationTimeout <= 0.0f) || (bAllWaitingToContinue && (s_fAllClientsInitialisedDelay <= 0.0f)))
	{
		// Tell each waiting machine to go into the game.
		pxElement = s_xClientsInitialising.Front();
		while (pxElement)
		{
			Asura_ListElt<AwaitingInitialisationData*>* pxCurrentElement = pxElement;
			pxElement = pxElement->Next();

			AwaitingInitialisationData* pxData = pxCurrentElement->Data();

			if (pxData->m_eInitialisationState == AVP_CLIENTAWAITINGINITIALISATION_STATE_WAITINGTOCONTINUE)
			{
				// Message to enter the game proper.
				Asura_Message* pxMessage = Asura_Network::CreateMessage(AVP_MSGID_INITIALISECOMPLETE);
				if (pxMessage)
				{
					pxMessage->To = ASURA_GUID_ALL_CLIENTMACHINES;
					pxMessage->From = ASURA_GUID_SERVER;
					pxMessage->UseGuaranteedDelivery();

					Asura_Network::Send(pxMessage);
				}
				
#ifdef ASURA_SUPPORT_NETMSG_SYSTEM
				// Allow them to receive update messages.
				Asura_Network::RemoveClientFromIgnoreNetMsgUpdatesList(pxData->m_uClientGuid);
#endif //#ifdef ASURA_SUPPORT_NETMSG_SYSTEM
				// Tidy up.
				s_xClientsInitialising.Remove(pxCurrentElement);
				delete pxData;
			}
		}
	}
}


void AVP_Server::ClearClientInitialisationLists()
{
	Asura_ListElt<AwaitingInitialisationData*>* pxElement = s_xClientsInitialising.Front();
	while (pxElement)
	{
		Asura_ListElt<AwaitingInitialisationData*>* pxCurrentElement = pxElement;
		pxElement = pxElement->Next();

		AwaitingInitialisationData* pxData = pxCurrentElement->Data();
		Asura_Network::RemoveClientFromInitialisationLists(pxData->m_uClientGuid);
		s_xClientsInitialising.Remove(pxCurrentElement);
		delete pxData;
	}
}


AVP_ServerEntity_PlayerInfo* AVP_Server::CreatePlayerInfo(AwaitingInitialisationData* pxInitialisationData)
{
	Asura_Assert(pxInitialisationData->m_eInitialisationState == AVP_CLIENTAWAITINGINITIALISATION_STATE_CREATEPLAYERINFO, "Incorrect state to create the player info.");

	pxInitialisationData->m_eInitialisationState = AVP_CLIENTAWAITINGINITIALISATION_STATE_CREATEACTOR;
	return AVP_ServerEntity_PlayerInfo::Create(pxInitialisationData->m_uClientGuid);
}


void AVP_Server::CreatePlayerActor(AwaitingInitialisationData* pxInitialisationData, AVP_ServerEntity_PlayerInfo* pxPlayerInfo, bool bIsInactive)
{
	Asura_Assert(pxInitialisationData->m_eInitialisationState == AVP_CLIENTAWAITINGINITIALISATION_STATE_CREATEACTOR, "Incorrect state to create the actor.");

	// Check the player info
	if ( !pxPlayerInfo )
	{
		// No player info in the server, shouldn't get here
		Asura_Assert( pxPlayerInfo, "Couldn't find player info in the server" ); 
		return; 
	}

	Asura_Guid uCurrentPuppet = pxPlayerInfo->GetPuppetGuid();

	if (uCurrentPuppet != ASURA_GUID_UNREGISTERED)
	{
		if (AVP_ServerEntity_Actor::FindEntity(uCurrentPuppet))
		{
			// This player already has a valid puppet. They must have managed to request it again through lag or cheating. Ignore them.
			// No need to assert here, I can see this happening without any kind of error beyond some lag.
			return;
		}
	}

	// Assign a team to the player in multiplayer team based games
	if (AVP_Settings::IsMultiPlayer() && AVP_Multiplayer_GameRules::IsTeamBasedGame())
	{
		// If we get a request from the player to join a game, check if the game rules allow that
		if (pxInitialisationData->m_iRequestedMultiplayerTeam != AVP_Settings::AVP_TEAM_NONE &&
			AVP_Multiplayer_GameRules::GetAllowTeamSelection())
		{
			pxPlayerInfo->SetMultiplayerTeam( static_cast<AVP_Settings::Team>(pxInitialisationData->m_iRequestedMultiplayerTeam)); 
		}
		else
		{
			Asura_Assert(false, "Player didn't request a team, the server will randomly allocate one. Starting from the lobby you shouldn't see this assert"); 

			// For now, just assign each new player to a different team in rotation
			static AVP_Settings::Team eCurrentTeamToJoin = AVP_Settings::AVP_TEAM_ONE; 
			
			// Assign this player to the current team
			pxPlayerInfo->SetMultiplayerTeam(eCurrentTeamToJoin); 

			// Update the current team for the next player to join
			int iTeamCount = static_cast<int>(eCurrentTeamToJoin); 
			iTeamCount++; 
			eCurrentTeamToJoin = static_cast<AVP_Settings::Team>(iTeamCount); 

			// Wrap around the maximum number of teams
			if (eCurrentTeamToJoin >= AVP_Settings::AVP_NUM_TEAMS || 
				eCurrentTeamToJoin > AVP_Multiplayer_GameRules::GetNumberOfTeams())
			{
				eCurrentTeamToJoin = AVP_Settings::AVP_TEAM_ONE; 
			}
		}

		// Create the team score entry (only will be created if needed)
		AVP_Multiplayer_TeamScore::Create( pxPlayerInfo->GetMultiplayerTeam() ); 
	}

	// Assign a role to this player if the game is role based
	if (AVP_Settings::IsMultiPlayer() && AVP_Multiplayer_GameRules::IsRoleBasedGame())
	{
		// Do we already have a role
		if (pxPlayerInfo->GetMultiplayerRole() == AVP_Settings::AVP_ROLE_NORMAL )
		{
			// No, find if any player has already been selected to be a hunter
			const Asura_Hashed_List<AVP_ServerEntity_PlayerInfo>* pxPlayerInfoList = AVP_ServerEntity_PlayerInfo::GetList(); 
			Asura_Hashed_List_ConstIt<AVP_ServerEntity_PlayerInfo> xPlayerInfoListConstIt(pxPlayerInfoList);
			u_int uNumberOfHunters = 0; 
			u_int uNumberOfPlayersWithoutRole = 0; 
			while (!xPlayerInfoListConstIt.Done())
			{
				const AVP_ServerEntity_PlayerInfo* pxCurrentPlayerInfo = xPlayerInfoListConstIt.GetCurrent(); 
				if (pxCurrentPlayerInfo->GetMultiplayerRole() ==  AVP_Settings::AVP_ROLE_HUNTER)
				{
					++uNumberOfHunters; 
				}
				else if (pxCurrentPlayerInfo->GetMultiplayerRole() ==  AVP_Settings::AVP_ROLE_NORMAL)
				{
					++uNumberOfPlayersWithoutRole; 
				}

				xPlayerInfoListConstIt.Next();
			}

			if (uNumberOfHunters == 0)
			{
				// Need to decide the species and there is no hunter, so make us a hunter randomly based on the number of players left
				// without role
				u_int uRandomHunter = Asura_Random::GetIntU(0, uNumberOfPlayersWithoutRole-1); 

				// Arbitrarily, if the last randomly generated number was zero, we are the new hunter
				// Otherwise, the next player will have a higher chance of being the new hunter
				if (uRandomHunter == 0)
				{
					pxInitialisationData->m_iAssignedMultiplayerRole = AVP_Settings::AVP_ROLE_HUNTER; 
				}
				else
				{
					pxInitialisationData->m_iAssignedMultiplayerRole = AVP_Settings::AVP_ROLE_PREY;
				}
			}
			else if (uNumberOfHunters == 1)
			{
				// There is already a hunter, we are prey
				pxInitialisationData->m_iAssignedMultiplayerRole = AVP_Settings::AVP_ROLE_PREY;
			}
			else
			{
				Asura_Assert(uNumberOfHunters > 1, "Too many hunters in this game");
			}

			pxPlayerInfo->SetMultiplayerRole( static_cast<AVP_Settings::Role>(pxInitialisationData->m_iAssignedMultiplayerRole) ); 
		}
		else
		{
			// Update the initialisation list
			pxInitialisationData->m_iAssignedMultiplayerRole = pxPlayerInfo->GetMultiplayerRole(); 
		}
	}


	const Asura_Guid uActorGuid	= Asura_Entity_System::GetCreatedByGameGuid();
	const AVP_Settings::Species eRequestedSpecies = pxInitialisationData ? static_cast<AVP_Settings::Species>(pxInitialisationData->m_iSpeciesToCreate) : AVP_Settings::GetPlayerSpecies();
	const AVP_Settings::Species eSpecies = AVP_Multiplayer_GameRules::GetValidPlayerSpecies(
											eRequestedSpecies, 
											pxPlayerInfo->GetMultiplayerTeam(),
											pxPlayerInfo->GetMultiplayerRole(),
											AVP_Settings::IsSinglePlayer());

	AVP_ServerEntity_Actor* pxActor;

	Asura_Hash_ID uStartPointBlueprintParameterHash = AVP_Game_Core::GetStartPointBlueprintHashForSpecies( eSpecies ); 

	switch (eSpecies)
	{
		case AVP_Settings::AVP_SPECIES_ALIEN:
		{
			AVP_ServerActor_Alien::Create(uActorGuid);
			break;
		}

		case AVP_Settings::AVP_SPECIES_HUMAN:
		{
			AVP_ServerActor_Marine::Create(uActorGuid);
			break;
		}

		case AVP_Settings::AVP_SPECIES_PREDATOR:
		default:
		{
			AVP_ServerActor_Predator::Create(uActorGuid);
			break;
		}
	}

	// Multiplayer team based start points override the species ones
	if (AVP_Settings::IsMultiPlayer() && AVP_Multiplayer_GameRules::IsTeamBasedGame())
	{
		uStartPointBlueprintParameterHash = AVP_Game_Core::GetStartPointBlueprintHashForTeam( pxPlayerInfo->GetMultiplayerTeam() ); 
	}

	pxActor = AVP_ServerEntity_Actor::FindEntity(uActorGuid);

	if(pxActor == NULL)
	{
		Asura_Assertf(false, ("Failed to create player's actor with GUID %d.", uActorGuid));
	}
	else
	{
		Asura_Hash_ID uBlueprintHashToCreate = pxInitialisationData->m_uBlueprintHashToCreate; 

		// The server can override the client request in multiplayer games
		if (AVP_Settings::IsMultiPlayer())
		{
			uBlueprintHashToCreate = AVP_Game_Core::GetPlayerBlueprintHashForSpecies( eSpecies ); 
		}
		
		pxPlayerInfo->InitialiseMissionTime( eSpecies );
		pxPlayerInfo->SetPuppetGuid(uActorGuid);

		pxActor->SetBlueprintHashID(uBlueprintHashToCreate);
		pxActor->SetToStartPosition(uStartPointBlueprintParameterHash);
        pxActor->InitFromBlueprint();
        pxActor->PostLoadInitialise();
		pxActor->SetInitiallyInactive( bIsInactive );

		if (bIsInactive)
		{
			// Make this actor intangible until the player is in control
			u_int uRequestedFlags = ASURA_PHYSICALOBJECT_FLAG_INVISIBLE | ASURA_PHYSICALOBJECT_FLAG_NOCOLLISIONS; 
			pxActor->SetPhysicalObjectFlags( pxActor->GetPhysicalObjectFlags() | uRequestedFlags ); 
		}

		pxInitialisationData->m_eInitialisationState = AVP_CLIENTAWAITINGINITIALISATION_STATE_ADDTOINITIALISATIONLIST;

		AVP_Rumble_System::AddPlayer(0, uActorGuid);
	}
}


void AVP_Server::ClientCreateMessagesSent(Asura_Guid uClientGuid)
{
	Asura_ListElt<AwaitingInitialisationData*>* pxElement = s_xClientsInitialising.Front();
	while (pxElement)
	{
		AwaitingInitialisationData* pxData = pxElement->Data();
		pxElement = pxElement->Next();

		if (pxData->m_uClientGuid != uClientGuid) { continue; }

		// Just move them into the final state.
		Asura_Assert(pxData->m_eInitialisationState == AVP_CLIENTAWAITINGINITIALISATION_STATE_CLIENTINITIALISING, "Unexpected state to be finishing sending create messages.");
		pxData->m_eInitialisationState = AVP_CLIENTAWAITINGINITIALISATION_STATE_WAITINGTOCONTINUE;

		s_fAllClientsInitialisedDelay = AVP_Settings::GetExternalNetworkActive() ? dbg_fAllClientsInitialisedDelayTime : 0.0f;

		return;
	}

	Asura_Assertf(false, ("Failed to find initialisation data for client %d, have they left already?", uClientGuid));
}

/// End of file. ///
