//////////////////////////////////////////////////////////////////////////////
//
// Asura_Project_Matchmaking.cpp
//
// Copyright (c) 2009 Rebellion. All rights reserved.
//
// Description:
//
//     AVP implementations of Asura_Matchmaking_System functions
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

#ifdef ASURA_SUPPORT_LOBBY

// Asura includes
#include "Debugging/Asura_Assert.h"
#include "Network/Online/Asura_Lobby.h"
#include "Network/Online/Asura_Matchmaking_System.h"

// AVP includes
#include "FrontEnd/AVP_DevLobby.h"
#include "FrontEnd/AVP_GameLobby.h"


// functions that must be supplied by project-specific code.
//
// Project_SetSearchFilters():
// The skill threshold is used to set min skill and max skill for returned results, 
// where min/max skill = playerskill +/- uSkillThreshold. the number of filters set
// should be returned in uNumFilters, and must be <= ASURA_LOBBY_MAX_SEARCH_CRITERIA.
void Asura_Matchmaking_System::Project_SetSearchFilters( u_int uSkillThreshold, Asura_Lobby_SearchCriterion* axFilters, u_int& uNumFilters )
{
#ifdef AVP_USE_DEV_LOBBY
	// TODO this ought to have its own GUIMenu interface etc. - hacked to use DevLobby settings.
	Asura_Assert(AVP_DevLobby::IsInitialised(), "only works with dev lobby currently");
	AVP_DevLobby::SetSearchFilters(uSkillThreshold, axFilters, uNumFilters);
#endif
}

// Project_GetLobbySkill():
// Just extract the lobby attribute specified by the project which represents skill.
int Asura_Matchmaking_System::Project_GetLobbySkill( const Asura_Lobby_Attributes& xAttributes )
{
#if defined( AVP_USE_GAME_LOBBY ) || defined( AVP_USE_DEV_LOBBY )
	return xAttributes.GetIntAttribute(AVP_LOBBY_ATTRIBUTE_SKILL);
#else
	return 0;
#endif
}

// Project_CreateLobby():
// must call Asura_Lobby_System::CreateLobby() with suitable parameters.
void Asura_Matchmaking_System::Project_CreateLobby()
{
#ifdef AVP_USE_DEV_LOBBY
	// TODO this ought to have its own GUIMenu interface etc. - hacked to use DevLobby settings.
	Asura_Assert(AVP_DevLobby::IsInitialised(), "only works with dev lobby currently");
	AVP_DevLobby::CreateGameForAutomatch();
#endif
}

#endif	//def ASURA_SUPPORT_LOBBY
