
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_Collision_Descriptor.cpp
// Author : Will Plano
// Date   : 21/02/2008
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "Physics/AVP_Collision_Descriptor.h"

// Asura independent header-files...
#include "Blueprint/Asura_Blueprint_System.h"
#include "Core/Asura_Maths.h"

// Game independent header-files...
#include "Blueprint/AVP_Blueprint_HashIDs.h"

Asura_TemplateHashTree< AVP_Collision_Descriptor > AVP_Collision_Descriptor::s_xHashTree;

const AVP_Collision_Descriptor* AVP_Collision_Descriptor::GetDataByHash( const Asura_Hash_ID uHash )
{
	AVP_Collision_Descriptor* pxData = s_xHashTree.GetDataByHash( uHash, false );
	if ( pxData == NULL )
	{
		pxData = s_xHashTree.AddNode( uHash );
		Asura_Assert( pxData != NULL, "Unable to add new collision descriptor to tree" );
		if ( pxData != NULL )
		{
			pxData->InitialiseFromBlueprint( uHash );
		}
	}
	return pxData;
}

AVP_Collision_Descriptor::AVP_Collision_Descriptor()
: m_pxCollisionBox( NULL )
, m_eCollisionProxy( Asura_Physics_Object::COLLISION_PROXY_OBB )
{
	
}

void AVP_Collision_Descriptor::InitialiseFromBlueprint( const Asura_Hash_ID uBlueprintHash )
{
	Asura_Value xValue;
	Asura_Bounding_Box xBB;
	xBB.Zero();
	xBB.Grow( 0.1f ); // set up a sensible default
	Asura_Blueprint* pxBlueprint = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_ACTOR_STATE, uBlueprintHash );
	if ( pxBlueprint )
	{
		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_COLLISION_OBJ_X_MIN, xValue ) )
		{
			xBB.MinX = xValue.GetFloatValue();
		}

		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_COLLISION_OBJ_X_MAX, xValue ) )
		{
			xBB.MaxX = xValue.GetFloatValue();
		}

		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_COLLISION_OBJ_Y_MIN, xValue ) )
		{
			xBB.MinY = xValue.GetFloatValue();
		}

		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_COLLISION_OBJ_Y_MAX, xValue ) )
		{
			xBB.MaxY = xValue.GetFloatValue();
		}

		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_COLLISION_OBJ_Z_MIN, xValue ) )
		{
			xBB.MinZ = xValue.GetFloatValue();
		}

		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_COLLISION_OBJ_Z_MAX, xValue ) )
		{
			xBB.MaxZ = xValue.GetFloatValue();
		}

		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_COLLISIONPROXY, xValue ) )
		{
			u_int uNum = xValue.GetIntValue();
			Asura_Assert( uNum < Asura_Physics_Object::NUM_COLLISION_PROXIES, "Not a CollisionProxy" );
			if ( uNum < Asura_Physics_Object::NUM_COLLISION_PROXIES )
			{
				m_eCollisionProxy = static_cast< Asura_Physics_Object::COLLISION_PROXY > ( uNum );
			}
		}
	}
#ifdef ASURA_ASSERT
	else
	{
		Asura_Assertf( false, ( "Unable to get collision proxy BP with hash ID 0x%08x", uBlueprintHash ) );
	}
#endif
	m_pxCollisionBox = new Asura_Collision_Box( xBB );
}

/// End of file. ///
