
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_SurfaceReorientationNormalInterpolator_AngularVelocityInterceptor.h
// Author : Andy Tate
// Date   : 11-Mar-2008
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __AVP_SURFACEREORIENTATIONNORMALINTERPOLATOR_ANGULARVELOCITYINTERCEPTOR_H__
#define __AVP_SURFACEREORIENTATIONNORMALINTERPOLATOR_ANGULARVELOCITYINTERCEPTOR_H__

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Actor/AVP_Actor_VelocityInterceptor.h"
#include "Physics/AVP_SurfaceReorientationNormalInterpolator.h"


////////////////////////////////////////////////////////////////////////////////
// Forward class declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Class declaration...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_SurfaceReorientationNormalInterpolator_AngularVelocityInterceptor
// Author       : Andy Tate
// Date         : 7-Feb-2008
//
// Description  : Class used to interpolate to a desired normal (using a angular
//				  velocity-interceptor) during surface-reorientation.
////////////////////////////////////////////////////////////////////////////////

class AVP_SurfaceReorientationNormalInterpolator_AngularVelocityInterceptor : public AVP_SurfaceReorientationNormalInterpolator
{
public:
	// Public functions...
	AVP_SurfaceReorientationNormalInterpolator_AngularVelocityInterceptor(void);
	//virtual ~AVP_SurfaceReorientationNormalInterpolator_AngularVelocityInterceptor();

	// Overridden functions...
	virtual void	Reset(Asura_AVector3_InParam xNormal);

	virtual	void	Update(const float fTimeStep, Asura_AVector3_InParam xDesiredNormal, Asura_AVector3_InOutParam xCurrentNormal);
	virtual	void	Update(const float fTimeStep, const AVP_CalculateSurfaceReorientationNormal& xNormalCalculation, Asura_AVector3_InOutParam xCurrentNormal);

	void			SetDesiredRotationAxis( Asura_AVector3_InParam xDesiredRotationAxis ) { m_xDesiredRotationAxis = xDesiredRotationAxis; }

protected:
	// Protected typedefs...
	typedef AVP_Actor_VelocityInterceptor<Asura_Angle> AngleInterceptor;

	// Protected variables...
	AngleInterceptor	m_xAngularVelocityInterceptor;	// The velocity-interceptor to use for the angle between normals.
	Asura_AVector3		m_xInitialNormal;				// The normal we're interpolating from.
	Asura_AVector3		m_xTargetNormal;				// The normal we're interpolating to.
	Asura_AVector3		m_xAxis;						// The axis between these 2 normals.

	// Protected functions...


private:
	// Private typedefs...
	typedef AVP_SurfaceReorientationNormalInterpolator PARENT;

	// Private variables...

	// Private functions...
	AVP_SurfaceReorientationNormalInterpolator_AngularVelocityInterceptor(const AVP_SurfaceReorientationNormalInterpolator_AngularVelocityInterceptor& xOther);

	AVP_SurfaceReorientationNormalInterpolator_AngularVelocityInterceptor&	operator=(const AVP_SurfaceReorientationNormalInterpolator_AngularVelocityInterceptor& xOther);

};   // End of class 'AVP_SurfaceReorientationNormalInterpolator_AngularVelocityInterceptor'.

#endif   // ifndef __AVP_SURFACEREORIENTATIONNORMALINTERPOLATOR_ANGULARVELOCITYINTERCEPTOR_H__


/// End of file. ///
