
//////////////////////////////////////////////////////////////////////////////
//
// Asura_Project_Skin_Elements.h
//
// Copyright (c) 2006 Rebellion. All rights reserved.
//
// Defines the list of supported sub-elements (chunks) for projects using
// ASURA_SUPPORT_SKIN_ELEMENTS
//
//////////////////////////////////////////////////////////////////////////////


#ifndef _ASURA_PROJECT_SKIN_ELEMENTS_H_
#define _ASURA_PROJECT_SKIN_ELEMENTS_H_

enum AVPSkinElementType
{
	AVP_SET_RIGHT_LEG,
	AVP_SET_RIGHT_KNEE,
	AVP_SET_LEFT_LEG,
	AVP_SET_LEFT_KNEE,
	AVP_SET_RIGHT_ARM,
	AVP_SET_RIGHT_ELBOW,
	AVP_SET_LEFT_ARM,
	AVP_SET_LEFT_ELBOW,
	AVP_SET_ALIEN_PIPES_LEFT,
	AVP_SET_ALIEN_PIPES_RIGHT,
	AVP_SET_ALIEN_TAIL_TIP,
	AVP_SET_ALIEN_TAIL,
	AVP_SET_ALIEN_FACE,
	AVP_SET_HEAD,
	AVP_SET_BODY,
	AVP_SET_HEAD_AFTER_HEADBITE,
	AVP_SET_ALIEN_INNER_MOUTH,
	
	NUM_AVP_SKIN_ELEMENT_TYPES
};

enum
{
	TOTAL_AVP_CHAR_SKIN_ELEMENTS	= (NUM_AVP_SKIN_ELEMENT_TYPES * 2),
	TOTAL_AVP_SKIN_ELEMENTS			= 1 + TOTAL_AVP_CHAR_SKIN_ELEMENTS
};

#ifdef ASURA_SUPPORT_SKIN_ELEMENTS

class Asura_Project_Skin_Elements
{
public:
	static int			GetNumElementTypes();
	static int			GetNumElementNames();
	static const char **GetElementTypeNames();
	static const char **GetElementNames();

	static void  PackElementIDIntoPolyFlags(u_int *puFlags, int iElementIndex);
	static int   UnpackElementIDFromPolyFlags(u_int uFlags);

	static bool  IsElementVisible(int iIndex, int iUserData);
	static void  ToggleElementVisibility(int iIndex, int iUserData);
	static bool  GetDefaultElementVisibility(int iIndex);

	static int   AllocateDefaultVisibilityFlags(class Asura_Animation *pxAnim, bool **ppbFlags);	

	static u_int	GetDamagedElementIndex(const u_int uElementType)	{ return(1 + (uElementType * 2)); }
	static u_int	GetUndamagedElementIndex(const u_int uElementType)	{ return(2 + (uElementType * 2)); }

	static u_int	GetParentElementType(u_int uElementType);
	static bool		IsChildOfElement(u_int uTestElement, u_int uParentElement, bool bRecursive); // If recursive, we go up the hierarchy until we reach the top (or the parent), otherwise we just compare the immediate parent.

	static Asura_Hash_ID GetElementBlueprintParameter(u_int uElementType) { return s_auElementBlueprintParameters[uElementType]; }

private:
	static const char*	s_aszElementNames[NUM_AVP_SKIN_ELEMENT_TYPES];
	static const char*	s_aszElementTypes[TOTAL_AVP_SKIN_ELEMENTS];
	static bool			s_abCurrentToolsElementVisibility[TOTAL_AVP_SKIN_ELEMENTS];
	static bool			s_abDefaultElementVisibility[TOTAL_AVP_SKIN_ELEMENTS];	

	static const Asura_Hash_ID s_auElementBlueprintParameters[NUM_AVP_SKIN_ELEMENT_TYPES];
};


#endif // ASURA_SUPPORT_SKIN_ELEMENTS

#endif //_ASURA_PROJECT_SKIN_ELEMENTS_H_
