///////////////////////////////////////////////////////////////////////////////
//
// File: AVP_CloakEffect.h
//
// Created by Andrew Southgate 04/12/2007
// 
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
// Handles rendering of the predators cloak
//
///////////////////////////////////////////////////////////////////////////////

#if defined ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#if !defined __AVP_CLOAKEFFECT_H__
#define __AVP_CLOAKEFFECT_H__

#include "Rendering/Asura_Colour.h"
#include "Rendering/Asura_SortedRender.h"

class Asura_Animation;
class Asura_Hierarchy_Skin;
class Asura_Shape_Instance;

class AVP_ClientEntity_PhysicalObject;


class AVP_CloakEffect
{
	friend class AVP_Xbox360_CloakEffect;
	friend class AVP_Platform_CloakEffect;

	void CalcMaxMinBoneOffsets();

public:
	static const int NUM_CLOAK_EFFECT_PERMUTATIONS=8;
	// Public enumerated type...
	enum State
	{
		UNCLOAKED,
		CLOAKING,
		CLOAKED,
		UNCLOAKING,

		MAX_STATES
	};

	static void Initialise();
	static void Shutdown();

	// Public functions...
	AVP_CloakEffect();
	virtual ~AVP_CloakEffect();

	bool	BeginCloaking(const bool bInstant);
	bool	BeginUncloaking(const bool bInstant);

	bool	Update(AVP_ClientEntity_PhysicalObject& xClient);
	void	Render(const Asura_Animation& xAnimation, const u_int uCamVisFlags, const u_int uRenderFlags, const Asura_Vector_3& xBSCentre, const float fBSRadius) const;

	// Accessor functions...
	void	SetMaxBones(u_int uNumBones);

	void	SetBoneOffset(u_int uBoneIdx, float fValue);
	float	GetBoneOffset(u_int uBoneIdx) const	{ return uBoneIdx >= m_uNumBones ? 0.0f : m_pfBoneOffsets[uBoneIdx]; }
	float	GetBoneValue(u_int uBoneIdx) const	{ return GetBoneOffset(uBoneIdx) + m_fTransitionValue; }
	float	GetMaxBoneValue() const	{ return m_fMaxBoneValue + m_fTransitionValue; }
	float	GetMinBoneValue() const	{ return m_fMinBoneValue + m_fTransitionValue; }

	State	GetState(void) const	{ return(m_eState); }

	float 	GetEffectScale(void) const			{ return m_fEffectScale;}
	void	SetEffectScale(float fEffectScale)	{ m_fEffectScale = fEffectScale;}
	
	float 	GetLightningScale(void) const				{ return m_fLightningScale;}
	void	SetLightningScale(float fLightningScale)	{ m_fLightningScale = fLightningScale;}

	float 	GetVariationSpeed(void) const				{ return m_fVariationSpeed;}
	void	SetVariationSpeed(float fVariationSpeed)	{ m_fVariationSpeed = fVariationSpeed;}
	
	float 	GetVariationScale(void) const				{ return m_fVariationScale;}
	void	SetVariationScale(float fVariationScale)	{ m_fVariationScale = fVariationScale;}

	float 	GetTransitionValue(void) const	{ return m_fTransitionValue;}
	void	SetTransitionValue(float fTransitionValue);


	float 	GetMaxVisibilityDistance() const			{ return m_fMaxVisibilityDistance; }
	void	SetMaxVisibilityDistance(float fDistance)	{ m_fMaxVisibilityDistance = fDistance; }

	float 	GetMaxVisibilityAngle() const			{ return m_fMaxVisibilityAngle; }
	void	SetMaxVisibilityAngle(float fAngle)	{ m_fMaxVisibilityAngle = fAngle; }


	void    SetTransitionTimeAndState(float fTransTime, State eState, Asura_Animation& xAsuraAnim);
    void    UpdateTransitionValueAndApplyAlpha(Asura_Animation& xAsuraAnim);
	void	UpdateTransitionValueAndApplyAlpha(Asura_Shape_Instance& xShapeInstance);


	static void RenderTransparentPasses(void);
protected:

	float m_fEffectScale;
	float m_fLightningScale;
	float m_fVariationSpeed;
	float m_fVariationScale;
	float m_fTransitionTime;
	float m_fTransitionValue;
	float m_fMaxVisibilityDistance;
	float m_fMaxVisibilityAngle;

	Asura_RGBA m_xLightningColour;

	float m_fMaxBoneValue;
	float m_fMinBoneValue;
	u_int m_uNumBones;
	float* m_pfBoneOffsets;

	u_int m_uSeed;
	State	m_eState;

	void	Platform_BeginRender(void) const;
	void	Platform_RenderSkin(const Asura_Animation& xAnimation, const Asura_Hierarchy_Skin& xSkin) const;
	void	Platform_EndRender(void) const;

	void	UpdateTransitionValueAndAlpha(float& fAlpha);

	static void Platform_Initialise();
	static void Platform_Shutdown();
private:
	


};

#endif // #if !defined __AVP_CLOAKEFFECT_H__
