//////////////////////////////////////////////////////////////////////////////
//
// AVP_Decal_System.cpp
//
// Copyright (c) 2007 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

/******************************************************************************/
/* Included Files                                                             */
/******************************************************************************/

#include "Core/Asura.h"

// Asura Platform-independent
#include "Core/Asura_TextureManagement.h"
#include "SpecialFX/Asura_Decal_System.h"
#include "SpecialFX/Asura_Decal_Template.h"

// AVP Platform-independent
#ifdef ASURA_USE_DECAL_MANAGER
#include "SpecialFX/Asura_Decal_Manager.h"
#else
#include "SpecialFX/AVP_Decal_System.h"
#endif

/******************************************************************************/
/* Global Variables                                                           */
/******************************************************************************/




/******************************************************************************/
/* Asura_Decal_System Static Class Functions                                  */
/******************************************************************************/


#ifdef ASURA_USE_DECAL_MANAGER
void Asura_Decal_System::Project_SetupTemplates(void)
{
}

const Asura_Decal_Template* Asura_Decal_System::Project_GetDecalTemplate(u_int uDecalTemplateID)
{
	return Asura_Decal_Manager::GetDecalTemplate(uDecalTemplateID);
}

#if defined(ASURA_TARGET_SM3)
 const Asura_Decal_Material* Asura_Decal_System::Project_GetDecalMaterial(u_int uDecalMaterialID)
 {
 	return Asura_Decal_Manager::GetDecalMaterial(uDecalMaterialID);
 }
#endif

#else


static Asura_Decal_Template g_axDecalTemplates[ AVP_NUM_DECAL_TEMPLATES ] =
{
	// AVP_DECAL_ACID
	{
		0.0f,														// m_fU
			0.0f,														// m_fV
			1.0f,														// m_fDeltaU
			1.0f,														// m_fDeltaV
			1.5f,														// m_fHeight
			1.5f,														// m_fWidth
			0.05f,														// m_fDepth
#ifdef ASURA_TARGET_SM3
			AVP_DECAL_MATERIAL_ACID,									// m_iDecalMaterialIndex
#else
			ASURA_TEXTURE_NONE,											// m_iTextureIndex
#endif //ASURA_TARGET_SM3
			ASURA_DECAL_LIGHTONCREATION | ASURA_DECAL_RANDOMROTATION,	// m_uFlags
	},
	// AVP_DECAL_ACID_SMALL
	{
		0.0f,														// m_fU
			0.0f,														// m_fV
			1.0f,														// m_fDeltaU
			1.0f,														// m_fDeltaV
			0.8f,														// m_fHeight
			0.8f,														// m_fWidth
			0.05f,														// m_fDepth
#ifdef ASURA_TARGET_SM3
			AVP_DECAL_MATERIAL_ACID_SMALL,								// m_iDecalMaterialIndex
#else
			ASURA_TEXTURE_NONE,											// m_iTextureIndex
#endif //ASURA_TARGET_SM3
			ASURA_DECAL_LIGHTONCREATION | ASURA_DECAL_RANDOMROTATION,	// m_uFlags
		},
		// AVP_DECAL_CONCRETE
		{
			0.0f,														// m_fU
				0.0f,														// m_fV
				1.0f,														// m_fDeltaU
				1.0f,														// m_fDeltaV
				0.15f,														// m_fHeight
				0.15f,														// m_fWidth
				0.05f,														// m_fDepth
#ifdef ASURA_TARGET_SM3
				AVP_DECAL_MATERIAL_CONCRETE,								// m_iDecalMaterialIndex
#else
				ASURA_TEXTURE_NONE,											// m_iTextureIndex
#endif //ASURA_TARGET_SM3
				ASURA_DECAL_LIGHTONCREATION | ASURA_DECAL_RANDOMROTATION,	// m_uFlags
		},

		// AVP_DECAL_CONCRETE_PLASMA
		{
			0.0f,														// m_fU
				0.0f,														// m_fV
				1.0f,														// m_fDeltaU
				1.0f,														// m_fDeltaV
				1.0f,														// m_fHeight
				1.0f,														// m_fWidth
				0.05f,														// m_fDepth
#ifdef ASURA_TARGET_SM3
				AVP_DECAL_MATERIAL_CONCRETE,								// m_iDecalMaterialIndex
#else
				ASURA_TEXTURE_NONE,											// m_iTextureIndex
#endif //ASURA_TARGET_SM3
				ASURA_DECAL_LIGHTONCREATION | ASURA_DECAL_RANDOMROTATION,	// m_uFlags
			},

			// AVP_DECAL_DIRT
			{
				0.0f,														// m_fU
					0.0f,														// m_fV
					1.0f,														// m_fDeltaU
					1.0f,														// m_fDeltaV
					0.15f,														// m_fHeight
					0.15f,														// m_fWidth
					0.05f,														// m_fDepth
#ifdef ASURA_TARGET_SM3
					AVP_DECAL_MATERIAL_DIRT,									// m_iDecalMaterialIndex
#else
					ASURA_TEXTURE_NONE,											// m_iTextureIndex
#endif
					ASURA_DECAL_LIGHTONCREATION | ASURA_DECAL_RANDOMROTATION,	// m_uFlags
			},

			// AVP_DECAL_DIRT_PLASMA
			{
				0.0f,														// m_fU
					0.0f,														// m_fV
					1.0f,														// m_fDeltaU
					1.0f,														// m_fDeltaV
					1.0f,														// m_fHeight
					1.0f,														// m_fWidth
					0.05f,														// m_fDepth
#ifdef ASURA_TARGET_SM3
					AVP_DECAL_MATERIAL_DIRT,									// m_iDecalMaterialIndex
#else
					ASURA_TEXTURE_NONE,											// m_iTextureIndex
#endif //ASURA_TARGET_SM3
					ASURA_DECAL_LIGHTONCREATION | ASURA_DECAL_RANDOMROTATION,	// m_uFlags
				},

				// AVP_DECAL_METAL
				{
					0.0f,														// m_fU
						0.0f,														// m_fV
						1.0f,														// m_fDeltaU
						1.0f,														// m_fDeltaV
						0.15f,														// m_fHeight
						0.15f,														// m_fWidth
						0.05f,														// m_fDepth
#ifdef ASURA_TARGET_SM3
						AVP_DECAL_MATERIAL_METAL,									// m_iDecalMaterialIndex
#else
						ASURA_TEXTURE_NONE,											// m_iTextureIndex
#endif //ASURA_TARGET_SM3
						ASURA_DECAL_LIGHTONCREATION | ASURA_DECAL_RANDOMROTATION,	// m_uFlags
				},

				// AVP_DECAL_METAL_PLASMA
				{
					0.0f,														// m_fU
						0.0f,														// m_fV
						1.0f,														// m_fDeltaU
						1.0f,														// m_fDeltaV
						1.0f,														// m_fHeight
						1.0f,														// m_fWidth
						0.05f,														// m_fDepth
#ifdef ASURA_TARGET_SM3
						AVP_DECAL_MATERIAL_METAL,									// m_iDecalMaterialIndex
#else
						ASURA_TEXTURE_NONE,											// m_iTextureIndex
#endif //ASURA_TARGET_SM3
						ASURA_DECAL_LIGHTONCREATION | ASURA_DECAL_RANDOMROTATION,	// m_uFlags
					},

					// AVP_DECAL_SLASH
					{
						0.0f,														// m_fU
							0.0f,														// m_fV
							1.0f,														// m_fDeltaU
							1.0f,														// m_fDeltaV
							0.4f,														// m_fHeight
							0.4f,														// m_fWidth
							0.05f,														// m_fDepth
#ifdef ASURA_TARGET_SM3
							AVP_DECAL_MATERIAL_SLASH,									// m_iDecalMaterialIndex
#else
							ASURA_TEXTURE_NONE,
#endif //ASURA_TARGET_SM3
							ASURA_DECAL_LIGHTONCREATION | ASURA_DECAL_RANDOMROTATION,	// m_uFlags
					},

					// AVP_DECAL_WOOD
					{
						0.0f,														// m_fU
							0.0f,														// m_fV
							1.0f,														// m_fDeltaU
							1.0f,														// m_fDeltaV
							0.15f,														// m_fHeight
							0.15f,														// m_fWidth
							0.05f,														// m_fDepth
#ifdef ASURA_TARGET_SM3
							AVP_DECAL_MATERIAL_WOOD,									// m_iDecalMaterialIndex
#else
							ASURA_TEXTURE_NONE,											// m_iTextureIndex
#endif //ASURA_TARGET_SM3
							ASURA_DECAL_LIGHTONCREATION | ASURA_DECAL_RANDOMROTATION,	// m_uFlags
						},

						// AVP_DECAL_WOOD_PLASMA
						{
							0.0f,														// m_fU
								0.0f,														// m_fV
								1.0f,														// m_fDeltaU
								1.0f,														// m_fDeltaV
								1.0f,														// m_fHeight
								1.0f,														// m_fWidth
								0.05f,														// m_fDepth
#ifdef ASURA_TARGET_SM3
								AVP_DECAL_MATERIAL_WOOD,									// m_iDecalMaterialIndex
#else
								ASURA_TEXTURE_NONE,											// m_iTextureIndex
#endif //ASURA_TARGET_SM3
								ASURA_DECAL_LIGHTONCREATION | ASURA_DECAL_RANDOMROTATION,	// m_uFlags
						},

						// AVP_DECAL_DISC_SLASH
						{
								0.0f,														// m_fU
								0.0f,														// m_fV
								1.0f,														// m_fDeltaU
								1.0f,														// m_fDeltaV
								0.4f,														// m_fHeight
								0.4f,														// m_fWidth
								0.05f,														// m_fDepth
#ifdef ASURA_TARGET_SM3
								AVP_DECAL_MATERIAL_DISC_SLASH,								// m_iDecalMaterialIndex
#else
								ASURA_TEXTURE_NONE,
#endif //ASURA_TARGET_SM3
								ASURA_DECAL_LIGHTONCREATION | ASURA_DECAL_RANDOMROTATION,	// m_uFlags
							},

};

#if defined(ASURA_TARGET_SM3)
// Can't initialise these because there's a constructor
static Asura_Decal_Material g_axDecalMaterials[ AVP_NUM_DECAL_MATERIALS ];
#endif	//defined(ASURA_TARGET_SM3)


void Asura_Decal_System::Project_SetupTemplates()
{
	// Load the textures

	const int iAcidTexture		= Asura_Texture_Management::AddTextureWithResetGroup( "\\SpecialFX\\Decals\\Acid_Decal_512.tga",		ASURA_TEXTURE_LOADED_PERMANENT );
	const int iConcreteTexture	= Asura_Texture_Management::AddTextureWithResetGroup( "\\SpecialFX\\Decals\\Concrete_Decal_256.tga",	ASURA_TEXTURE_LOADED_PERMANENT );
	const int iDirtTexture		= Asura_Texture_Management::AddTextureWithResetGroup( "\\SpecialFX\\Decals\\Dirt_Decal_256.tga",		ASURA_TEXTURE_LOADED_PERMANENT );
	const int iMetalTexture		= Asura_Texture_Management::AddTextureWithResetGroup( "\\SpecialFX\\Decals\\Metal_Decal_256.tga",		ASURA_TEXTURE_LOADED_PERMANENT );
	const int iSlashTexture		= Asura_Texture_Management::AddTextureWithResetGroup( "\\SpecialFX\\Decals\\Slash_Decal_256.tga",		ASURA_TEXTURE_LOADED_PERMANENT );
	const int iWoodTexture		= Asura_Texture_Management::AddTextureWithResetGroup( "\\SpecialFX\\Decals\\Wood_Decal_256.tga",		ASURA_TEXTURE_LOADED_PERMANENT );
	const int iDiscSlashTexture	= Asura_Texture_Management::AddTextureWithResetGroup( "\\SpecialFX\\Decals\\Disc_Slash_Decal_256.tga",		ASURA_TEXTURE_LOADED_PERMANENT );

#if defined(ASURA_TARGET_SM3)

	const int iAcidNormals		= Asura_Texture_Management::AddTextureWithResetGroup( "\\SpecialFX\\Decals\\Acid_Nmap_512.tga",			ASURA_TEXTURE_LOADED_PERMANENT );
	const int iConcreteNormals	= Asura_Texture_Management::AddTextureWithResetGroup( "\\SpecialFX\\Decals\\Concrete_Nmap_256.tga",		ASURA_TEXTURE_LOADED_PERMANENT );
	const int iDirtNormals		= Asura_Texture_Management::AddTextureWithResetGroup( "\\SpecialFX\\Decals\\Dirt_Nmap_256.tga",			ASURA_TEXTURE_LOADED_PERMANENT );
	const int iMetalNormals		= Asura_Texture_Management::AddTextureWithResetGroup( "\\SpecialFX\\Decals\\Metal_Nmap_256.tga",		ASURA_TEXTURE_LOADED_PERMANENT );
	const int iSlashNormals		= Asura_Texture_Management::AddTextureWithResetGroup( "\\SpecialFX\\Decals\\Slash_Nmap_256.tga",		ASURA_TEXTURE_LOADED_PERMANENT );
	const int iWoodNormals		= Asura_Texture_Management::AddTextureWithResetGroup( "\\SpecialFX\\Decals\\Wood_Nmap_256.tga",			ASURA_TEXTURE_LOADED_PERMANENT );
	const int iDiscSlashNormals	= Asura_Texture_Management::AddTextureWithResetGroup( "\\SpecialFX\\Decals\\Disc_Slash_Nmap_256.tga",	ASURA_TEXTURE_LOADED_PERMANENT );

	const int iAcidSpecular		= Asura_Texture_Management::AddTextureWithResetGroup( "\\SpecialFX\\Decals\\Acid_Spec_512.bmp",			ASURA_TEXTURE_LOADED_PERMANENT );
	const int iDirtSpecular		= Asura_Texture_Management::AddTextureWithResetGroup( "\\SpecialFX\\Decals\\Dirt_Spec_256.bmp",			ASURA_TEXTURE_LOADED_PERMANENT );
	const int iConcreteSpecular	= Asura_Texture_Management::AddTextureWithResetGroup( "\\SpecialFX\\Decals\\Concrete_Spec_256.bmp",		ASURA_TEXTURE_LOADED_PERMANENT );
	const int iMetalSpecular	= Asura_Texture_Management::AddTextureWithResetGroup( "\\SpecialFX\\Decals\\Metal_Spec_256.bmp",		ASURA_TEXTURE_LOADED_PERMANENT );
	const int iSlashSpecular	= Asura_Texture_Management::AddTextureWithResetGroup( "\\SpecialFX\\Decals\\Slash_Spec_256.bmp",		ASURA_TEXTURE_LOADED_PERMANENT );
	const int iWoodSpecular		= Asura_Texture_Management::AddTextureWithResetGroup( "\\SpecialFX\\Decals\\Wood_Spec_256.bmp",			ASURA_TEXTURE_LOADED_PERMANENT );
	const int iDiscSlashSpecular= Asura_Texture_Management::AddTextureWithResetGroup( "\\SpecialFX\\Decals\\Disc_Slash_Spec_256.bmp",		ASURA_TEXTURE_LOADED_PERMANENT );

	// Set up material templates

	g_axDecalMaterials[ AVP_DECAL_MATERIAL_ACID ].m_iTextureIndex			= iAcidTexture;
	g_axDecalMaterials[ AVP_DECAL_MATERIAL_ACID ].m_iNormalMapIndex			= iAcidNormals;
	g_axDecalMaterials[ AVP_DECAL_MATERIAL_ACID ].m_iSpecularMapIndex		= iAcidSpecular;
	g_axDecalMaterials[ AVP_DECAL_MATERIAL_ACID ].m_bUseParallaxMapping		= true;
	g_axDecalMaterials[ AVP_DECAL_MATERIAL_ACID ].m_iMaxDecals				= 200;
	g_axDecalMaterials[ AVP_DECAL_MATERIAL_ACID ].m_fFadeInTime				= 1.0f;
	g_axDecalMaterials[ AVP_DECAL_MATERIAL_ACID ].m_fFadeOutTime			= 2.00f;
	g_axDecalMaterials[ AVP_DECAL_MATERIAL_ACID ].m_fDecalStayTime			= 60.0f;

	g_axDecalMaterials[ AVP_DECAL_MATERIAL_ACID_SMALL ].m_iTextureIndex			= iAcidTexture;
	g_axDecalMaterials[ AVP_DECAL_MATERIAL_ACID_SMALL ].m_iNormalMapIndex		= iAcidNormals;
	g_axDecalMaterials[ AVP_DECAL_MATERIAL_ACID_SMALL ].m_iSpecularMapIndex		= iAcidSpecular;
	g_axDecalMaterials[ AVP_DECAL_MATERIAL_ACID_SMALL ].m_bUseParallaxMapping	= true;
	g_axDecalMaterials[ AVP_DECAL_MATERIAL_ACID_SMALL ].m_iMaxDecals			= 200;
	g_axDecalMaterials[ AVP_DECAL_MATERIAL_ACID_SMALL ].m_fFadeInTime			= 1.0f;
	g_axDecalMaterials[ AVP_DECAL_MATERIAL_ACID_SMALL ].m_fFadeOutTime			= 2.00f;
	g_axDecalMaterials[ AVP_DECAL_MATERIAL_ACID_SMALL ].m_fDecalStayTime		= -1.0f;

	g_axDecalMaterials[ AVP_DECAL_MATERIAL_CONCRETE ].m_iTextureIndex		= iConcreteTexture;
	g_axDecalMaterials[ AVP_DECAL_MATERIAL_CONCRETE ].m_iNormalMapIndex		= iConcreteNormals;
	g_axDecalMaterials[ AVP_DECAL_MATERIAL_CONCRETE ].m_iSpecularMapIndex	= iConcreteSpecular;
	g_axDecalMaterials[ AVP_DECAL_MATERIAL_CONCRETE ].m_bUseParallaxMapping	= true;
	g_axDecalMaterials[ AVP_DECAL_MATERIAL_CONCRETE ].m_iMaxDecals			= 100;
	g_axDecalMaterials[ AVP_DECAL_MATERIAL_CONCRETE ].m_fFadeInTime			= 0.01f;
	g_axDecalMaterials[ AVP_DECAL_MATERIAL_CONCRETE ].m_fFadeOutTime		= 0.01f;
	g_axDecalMaterials[ AVP_DECAL_MATERIAL_CONCRETE ].m_fDecalStayTime		= -1.0f;

	g_axDecalMaterials[ AVP_DECAL_MATERIAL_DIRT ].m_iTextureIndex			= iDirtTexture;
	g_axDecalMaterials[ AVP_DECAL_MATERIAL_DIRT ].m_iNormalMapIndex			= iDirtNormals;
	g_axDecalMaterials[ AVP_DECAL_MATERIAL_DIRT ].m_iSpecularMapIndex		= iDirtSpecular;
	g_axDecalMaterials[ AVP_DECAL_MATERIAL_DIRT ].m_bUseParallaxMapping		= true;
	g_axDecalMaterials[ AVP_DECAL_MATERIAL_DIRT ].m_iMaxDecals				= 100;
	g_axDecalMaterials[ AVP_DECAL_MATERIAL_DIRT ].m_fFadeInTime				= 0.01f;
	g_axDecalMaterials[ AVP_DECAL_MATERIAL_DIRT ].m_fFadeOutTime			= 0.01f;
	g_axDecalMaterials[ AVP_DECAL_MATERIAL_DIRT ].m_fDecalStayTime			= -1.0f;

	g_axDecalMaterials[ AVP_DECAL_MATERIAL_METAL ].m_iTextureIndex			= iMetalTexture;
	g_axDecalMaterials[ AVP_DECAL_MATERIAL_METAL ].m_iNormalMapIndex		= iMetalNormals;
	g_axDecalMaterials[ AVP_DECAL_MATERIAL_METAL ].m_iSpecularMapIndex		= iMetalSpecular;
	g_axDecalMaterials[ AVP_DECAL_MATERIAL_METAL ].m_bUseParallaxMapping	= true;
	g_axDecalMaterials[ AVP_DECAL_MATERIAL_METAL ].m_iMaxDecals				= 100;
	g_axDecalMaterials[ AVP_DECAL_MATERIAL_METAL ].m_fFadeInTime			= 0.01f;
	g_axDecalMaterials[ AVP_DECAL_MATERIAL_METAL ].m_fFadeOutTime			= 0.01f;
	g_axDecalMaterials[ AVP_DECAL_MATERIAL_METAL ].m_fDecalStayTime			= -1.0f;

	g_axDecalMaterials[ AVP_DECAL_MATERIAL_SLASH ].m_iTextureIndex			= iSlashTexture;
	g_axDecalMaterials[ AVP_DECAL_MATERIAL_SLASH ].m_iNormalMapIndex		= iSlashNormals;
	g_axDecalMaterials[ AVP_DECAL_MATERIAL_SLASH ].m_iSpecularMapIndex		= iSlashSpecular;
	g_axDecalMaterials[ AVP_DECAL_MATERIAL_SLASH ].m_bUseParallaxMapping	= true;
	g_axDecalMaterials[ AVP_DECAL_MATERIAL_SLASH ].m_iMaxDecals				= 100;
	g_axDecalMaterials[ AVP_DECAL_MATERIAL_SLASH ].m_fFadeInTime			= 0.01f;
	g_axDecalMaterials[ AVP_DECAL_MATERIAL_SLASH ].m_fFadeOutTime			= 0.01f;
	g_axDecalMaterials[ AVP_DECAL_MATERIAL_SLASH ].m_fDecalStayTime			= -1.0f;

	g_axDecalMaterials[ AVP_DECAL_MATERIAL_WOOD ].m_iTextureIndex			= iWoodTexture;
	g_axDecalMaterials[ AVP_DECAL_MATERIAL_WOOD ].m_iNormalMapIndex			= iWoodNormals;
	g_axDecalMaterials[ AVP_DECAL_MATERIAL_WOOD ].m_iSpecularMapIndex		= iWoodSpecular;
	g_axDecalMaterials[ AVP_DECAL_MATERIAL_WOOD ].m_bUseParallaxMapping		= true;
	g_axDecalMaterials[ AVP_DECAL_MATERIAL_WOOD ].m_iMaxDecals				= 100;
	g_axDecalMaterials[ AVP_DECAL_MATERIAL_WOOD ].m_fFadeInTime				= 0.01f;
	g_axDecalMaterials[ AVP_DECAL_MATERIAL_WOOD ].m_fFadeOutTime			= 0.01f;
	g_axDecalMaterials[ AVP_DECAL_MATERIAL_WOOD ].m_fDecalStayTime			= -1.0f;

	g_axDecalMaterials[ AVP_DECAL_MATERIAL_DISC_SLASH ].m_iTextureIndex		= iDiscSlashTexture;
	g_axDecalMaterials[ AVP_DECAL_MATERIAL_DISC_SLASH ].m_iNormalMapIndex	= iDiscSlashNormals;
	g_axDecalMaterials[ AVP_DECAL_MATERIAL_DISC_SLASH ].m_iSpecularMapIndex	= iDiscSlashSpecular;
	g_axDecalMaterials[ AVP_DECAL_MATERIAL_DISC_SLASH ].m_bUseParallaxMapping	= true;
	g_axDecalMaterials[ AVP_DECAL_MATERIAL_DISC_SLASH ].m_iMaxDecals		= 100;
	g_axDecalMaterials[ AVP_DECAL_MATERIAL_DISC_SLASH ].m_fFadeInTime		= 0.01f;
	g_axDecalMaterials[ AVP_DECAL_MATERIAL_DISC_SLASH ].m_fFadeOutTime		= 0.01f;
	g_axDecalMaterials[ AVP_DECAL_MATERIAL_DISC_SLASH ].m_fDecalStayTime	= -1.0f;

#else

	g_axDecalTemplates[ AVP_DECAL_ACID ].m_iTextureIndex = iAcidTexture;
	g_axDecalTemplates[ AVP_DECAL_ACID_SMALL ].m_iTextureIndex = iAcidTexture;
	g_axDecalTemplates[ AVP_DECAL_CONCRETE ].m_iTextureIndex = iConcreteTexture;
	g_axDecalTemplates[ AVP_DECAL_CONCRETE_PLASMA ].m_iTextureIndex = iConcreteTexture;
	g_axDecalTemplates[ AVP_DECAL_DIRT ].m_iTextureIndex = iDirtTexture;
	g_axDecalTemplates[ AVP_DECAL_DIRT_PLASMA ].m_iTextureIndex = iDirtTexture;
	g_axDecalTemplates[ AVP_DECAL_METAL ].m_iTextureIndex = iMetalTexture;
	g_axDecalTemplates[ AVP_DECAL_METAL_PLASMA ].m_iTextureIndex = iMetalTexture;
	g_axDecalTemplates[ AVP_DECAL_SLASH ].m_iTextureIndex = iSlashTexture;
	g_axDecalTemplates[ AVP_DECAL_WOOD ].m_iTextureIndex = iWoodTexture;
	g_axDecalTemplates[ AVP_DECAL_WOOD_PLASMA ].m_iTextureIndex = iWoodTexture;
	g_axDecalTemplates[ AVP_DECAL_DISC_SLASH ].m_iTextureIndex = iDiscSlashTexture;

#endif	// defined(ASURA_TARGET_SM3)

	Asura_Texture_Management::LoadTextures();
}

const Asura_Decal_Template* Asura_Decal_System::Project_GetDecalTemplate( u_int uDecalTemplateID )
{
	if( uDecalTemplateID < AVP_NUM_DECAL_TEMPLATES )
	{
		return &g_axDecalTemplates[ uDecalTemplateID ];
	}
	else
	{
		Asura_Assertf( false , ("Invalid parameter (%u) passed to Asura_Decal_System::Project_GetDecalTemplate().", uDecalTemplateID) );
		return NULL;
	}
}

#if defined(ASURA_TARGET_SM3)

const Asura_Decal_Material* Asura_Decal_System::Project_GetDecalMaterial( u_int uDecalMaterialID )
{
	if( uDecalMaterialID < AVP_NUM_DECAL_MATERIALS )
	{
		return &g_axDecalMaterials[ uDecalMaterialID ];
	}
	else
	{
		Asura_Assertf( false , ("Invalid parameter (%u) passed to Asura_Decal_System::Project_GetDecalMaterial().", uDecalMaterialID) );
		return NULL;
	}
}

#endif	// defined(ASURA_TARGET_SM3)

#endif
// End Of File
