///////////////////////////////////////////////////////////////////////////////
//
// File: AVP_Laser.cpp
//
// Created by Andrew Southgate 12/11/2007
// 
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
// Handles rendering of the predators tri-laser thingy
//
///////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"
#include "Core/Asura_LOS.h"


#ifdef ASURA_TARGET_SM3
#include "Projectors/Asura_Projector.h"
#include "Projectors/Asura_Projector_System.h"
#endif

#include "AVP_Laser.h"

#include "Interface/Asura_CommandConsole.h"
#include "Light/Asura_Photon.h"

#include "Entity/PhysicalObject/Actor/AVP_ClientEntity_Actor.h"
#include "SpecialFX/AVP_Platform_Laser.h"


DVSVAR bool dvs_bEnableLaser = true;


AVP_Laser *AVP_Laser::s_pxFirst = NULL;
AVP_Laser *AVP_Laser::s_pxLast = NULL;
Asura_RGBA AVP_Laser::m_rgbaColour;

AVP_Laser::AVP_Laser()
{
	m_fBeamRadius = 0.001f;
	m_fBeamSpread = 0.005f;
	m_fRadius = 0.05;
	m_bVisiblity = true;
	m_bActive = false;
	m_fBeamSharpness = 0.005f;
	m_fMaxLength = 50.0f;
	m_fFallOffLength = 5.0f;
	m_uGuidToIgnore = ASURA_GUID_UNREGISTERED;
	m_uNumBeams = 0;

	m_fFallOffStartAngle = 15.f * Asura_Maths::DegToRadConstant;
	m_fFallOffEndAngle = 30.f * Asura_Maths::DegToRadConstant;
	m_fFallOffStartDistance = 0.5f;
	m_fFallOffEndDistance = 1.0f;

	m_bOriginPhotonEnabled = false;
	m_fOriginPhotonRadius = 1.0f;
	m_fOriginPhotonFalloffScale = 1.0f;
	m_fOriginPhotonFalloffPower = 1.0f;
	m_bOriginPhotonEnableDistanceFade = false;
	m_fOriginPhotonDistanceFadeStart = 0.0f;
	m_fOriginPhotonDistanceFadeDist = 0.0f;

	m_bContactPhotonEnabled = false;
	m_fContactPhotonRadius = 1.0f;
	m_fContactPhotonFalloffScale = 1.0f;
	m_fContactPhotonFalloffPower = 1.0f;
	m_bContactPhotonEnableDistanceFade = false;
	m_fContactPhotonDistanceFadeStart = 0.0f;
	m_fContactPhotonDistanceFadeDist = 0.0f;
	m_fContactPhotonBackOffDistance = 0.5f;

	m_xOriginPhotonColour = Asura_RGBA(1.0f, 0.f, 0.1f, 1.0f);
	m_xContactPhotonColour =  Asura_RGBA(1.0f, 0.f, 0.1f, 1.0f);


	m_pxOriginPhoton = NULL;

	m_fBeamVisibilityDistance = 1.0f / 40.0f;

	m_fFallOffAngleEndIntensity = 0.1f;
#ifdef ASURA_ASSERT
	m_bPosValid = false;
	m_bOrientValid = false;
#endif

	m_pxNext = NULL;
	m_pxPrev = s_pxLast;

	if (s_pxLast)
		s_pxLast->m_pxNext = this;
	else
		s_pxFirst = this;

	s_pxLast = this;
}
AVP_Laser::~AVP_Laser()
{
	SetNumBeams(0);

	if (m_pxNext)
		m_pxNext->m_pxPrev = m_pxPrev;
	else
		s_pxLast = m_pxPrev;

	if (m_pxPrev)
		m_pxPrev->m_pxNext = m_pxNext;
	else
		s_pxFirst = m_pxNext;
}

void AVP_Laser::SetNumBeams(u_int uNumBeams) 
{
	if (uNumBeams > MAX_NUM_BEAMS)
	{
		uNumBeams = MAX_NUM_BEAMS;
	}
	if (m_bContactPhotonEnabled)
	{
		// clear any we no longer need
		for (u_int i = uNumBeams; i < m_uNumBeams; ++i)
		{
			m_xBeamLengths[i] = 0.0f;
			if (m_pxContactPhotons)
			{
				Asura_Photon_System::DestroyPhoton(m_pxContactPhotons[i]);
				m_pxContactPhotons[i] = NULL;
			}
		}
		// create those we now need
		for (u_int i = m_uNumBeams; i < uNumBeams; ++i)
		{
			m_xBeamLengths[i] = 0.0f;
			m_pxContactPhotons[i] = Asura_Photon_System::CreatePhoton();
			m_pxContactPhotons[i]->SetAsActive(false);
			m_pxContactPhotons[i]->SetColours(m_xContactPhotonColour.m_fR,m_xContactPhotonColour.m_fG,m_xContactPhotonColour.m_fB);
			m_pxContactPhotons[i]->SetRange(m_fContactPhotonRadius);
			m_pxContactPhotons[i]->SetFalloffPower(m_fContactPhotonFalloffPower);
			m_pxContactPhotons[i]->SetFalloffScale(m_fContactPhotonFalloffScale);
			m_pxContactPhotons[i]->SetDistanceFade(m_bContactPhotonEnableDistanceFade, m_fContactPhotonDistanceFadeStart, m_fContactPhotonDistanceFadeDist );
		}
	}
	m_uNumBeams = uNumBeams;
}

void AVP_Laser::SetActive(bool bActive) 
{
	if (!bActive && m_bActive)
	{
		if (m_pxOriginPhoton && m_bOriginPhotonEnabled)
		{
			m_pxOriginPhoton->SetAsActive(false);
		}

		if (m_bContactPhotonEnabled)
		{
			for (u_int i = 0; i < m_uNumBeams; ++i)
			{
				if (m_pxContactPhotons[i])
				{
					m_pxContactPhotons[i]->SetAsActive(false);
				}
			}
		}
	}
	m_bActive = bActive;
}


void AVP_Laser::Initialise()
{
	s_pxFirst = NULL;
	s_pxLast = NULL;
	AVP_Platform_Laser_System::Initialise();

#ifdef ASURA_DEBUG_VARIABLE_SYSTEM
	if(!Asura_CommandConsole::VarExists(L"AVP.Laser.Enable"))
	{
		Asura_CommandConsole::AddVar(L"AVP.Laser.Enable", &dvs_bEnableLaser);
	}
#endif //ASURA_DEBUG_VARIABLE_SYSTEM
}

void AVP_Laser::SetOriginPhotonEnabled(bool bEnabled)
{
	if (bEnabled && !m_bOriginPhotonEnabled)
	{
		m_pxOriginPhoton = Asura_Photon_System::CreatePhoton();
		m_pxOriginPhoton->SetAsActive(false);
		m_pxOriginPhoton->SetColours(m_xOriginPhotonColour.m_fR, m_xOriginPhotonColour.m_fG, m_xOriginPhotonColour.m_fB);
		m_pxOriginPhoton->SetRange(m_fOriginPhotonRadius);
		m_pxOriginPhoton->SetFalloffPower(m_fOriginPhotonFalloffPower);
		m_pxOriginPhoton->SetFalloffScale(m_fOriginPhotonFalloffScale);
		m_pxOriginPhoton->SetDistanceFade(m_bOriginPhotonEnableDistanceFade, m_fOriginPhotonDistanceFadeStart, m_fOriginPhotonDistanceFadeDist );
	}
	else if (!bEnabled && m_bOriginPhotonEnabled)
	{
		if (m_pxOriginPhoton)
		{
			Asura_Photon_System::DestroyPhoton(m_pxOriginPhoton);;
			m_pxOriginPhoton = NULL;
		}
	}
	m_bContactPhotonEnabled = bEnabled;

}

void AVP_Laser::SetOriginPhotonRadius(float fRadius)
{
	if (m_pxOriginPhoton && m_bOriginPhotonEnabled)
	{
		m_pxOriginPhoton->SetRange(m_fOriginPhotonRadius = fRadius);
	}
}
void AVP_Laser::SetOriginPhotonColour(const Asura_RGBA &xColour)
{
	if (m_pxOriginPhoton && m_bOriginPhotonEnabled)
	{
		m_xOriginPhotonColour = xColour;
		m_pxOriginPhoton->SetColours(m_xOriginPhotonColour.m_fR,m_xOriginPhotonColour.m_fG,m_xOriginPhotonColour.m_fB);
	}
}
void AVP_Laser::SetOriginPhotonFalloffScale(float fScale)
{
	if (m_pxOriginPhoton && m_bOriginPhotonEnabled)
	{
		m_pxOriginPhoton->SetFalloffScale(m_fOriginPhotonFalloffScale = fScale);
	}
}
void AVP_Laser::SetOriginPhotonFalloffPower(float fPower)
{
	if (m_pxOriginPhoton && m_bOriginPhotonEnabled)
	{
		m_pxOriginPhoton->SetFalloffPower(m_fOriginPhotonFalloffPower);
	}
}
void AVP_Laser::SetOriginPhotonDistanceFade(bool bEnable, float fStart/* = 0.0f*/, float fDist/* = 0.0f*/)
{
	if (m_pxOriginPhoton && m_bOriginPhotonEnabled)
	{
		m_pxOriginPhoton->SetDistanceFade(m_bOriginPhotonEnableDistanceFade = bEnable, m_fOriginPhotonDistanceFadeStart = fStart, m_fOriginPhotonDistanceFadeDist = fDist );
	}
}

void AVP_Laser::SetContactPhotonEnabled(bool bEnabled)
{
	if (bEnabled && !m_bContactPhotonEnabled)
	{
		for (u_int i = 0; i < GetNumBeams(); ++i)
		{
			m_pxContactPhotons[i] = Asura_Photon_System::CreatePhoton();
			m_pxContactPhotons[i]->SetAsActive(false);
			m_pxContactPhotons[i]->SetColours(m_xContactPhotonColour.m_fR,m_xContactPhotonColour.m_fG,m_xContactPhotonColour.m_fB);
			m_pxContactPhotons[i]->SetRange(m_fContactPhotonRadius);
			m_pxContactPhotons[i]->SetFalloffPower(m_fContactPhotonFalloffPower);
			m_pxContactPhotons[i]->SetFalloffScale(m_fContactPhotonFalloffScale);
			m_pxContactPhotons[i]->SetDistanceFade(m_bContactPhotonEnableDistanceFade, m_fContactPhotonDistanceFadeStart, m_fContactPhotonDistanceFadeDist );
		}
	}
	else if (!bEnabled && m_bContactPhotonEnabled)
	{
		for (u_int i = 0; i < GetNumBeams(); ++i)
		{
			 Asura_Photon_System::DestroyPhoton(m_pxContactPhotons[i]);
			 m_pxContactPhotons[i] = NULL;
		}
	}
	m_bContactPhotonEnabled = bEnabled;
}


void AVP_Laser::SetContactPhotonRadius(float fRadius)
{
	m_fContactPhotonRadius = fRadius;
	if (m_bContactPhotonEnabled)
	{
		for (u_int i = 0; i < GetNumBeams() && m_bContactPhotonEnabled; ++i)
		{
			if (m_pxContactPhotons[i])
			{
				m_pxContactPhotons[i]->SetRange(fRadius);
			}
		}
	}
}
void AVP_Laser::SetContactPhotonColour(const Asura_RGBA &xColour)
{
	m_xContactPhotonColour = xColour;

	if (m_bContactPhotonEnabled)
	{
		for (u_int i = 0; i < GetNumBeams() && m_bContactPhotonEnabled; ++i)
		{
			if (m_pxContactPhotons[i])
			{
				m_pxContactPhotons[i]->SetColours(m_xContactPhotonColour.m_fR,m_xContactPhotonColour.m_fG,m_xContactPhotonColour.m_fB);
			}
		}
	}
}
void AVP_Laser::SetContactPhotonFalloffScale(float fScale)
{
	m_fContactPhotonFalloffScale = fScale;

	if (m_bContactPhotonEnabled)
	{
		for (u_int i = 0; i < GetNumBeams(); ++i)
		{
			if (m_pxContactPhotons[i])
			{
				m_pxContactPhotons[i]->SetFalloffScale(m_fContactPhotonFalloffScale);
			}
		}
	}
}
void AVP_Laser::SetContactPhotonFalloffPower(float fPower)
{
	m_fContactPhotonFalloffPower = fPower;

	if (m_bContactPhotonEnabled)
	{
		for (u_int i = 0; i < GetNumBeams(); ++i)
		{
			if (m_pxContactPhotons[i])
			{
				m_pxContactPhotons[i]->SetFalloffPower(m_fContactPhotonFalloffPower);
			}
		}
	}

}
void AVP_Laser::SetContactPhotonDistanceFade(bool bEnable, float fStart/* = 0.0f*/, float fDist/* = 0.0f*/)
{
	m_bContactPhotonEnableDistanceFade = bEnable;
	m_fContactPhotonDistanceFadeStart = fStart;
	m_fContactPhotonDistanceFadeDist = fDist;
	if (m_bContactPhotonEnabled)
	{
		for (u_int i = 0; i < GetNumBeams(); ++i)
		{
			if (m_pxContactPhotons[i])
			{
				m_pxContactPhotons[i]->SetDistanceFade(m_bContactPhotonEnableDistanceFade, m_fContactPhotonDistanceFadeStart, m_fContactPhotonDistanceFadeDist);
			}
		}
	}
}
void AVP_Laser::SetContactPhotonBackOffDistance(float fDistance)
{
	m_fContactPhotonBackOffDistance = fDistance;
}


void AVP_Laser::Update()
{
	if (GetVisiblity() && GetActive() && GetNumBeams() > 0)
	{
		Asura_Vector_3 xRight(m_xOrient.e11, m_xOrient.e21, m_xOrient.e31);
		Asura_Vector_3 xDown(m_xOrient.e12, m_xOrient.e22, m_xOrient.e32);
		Asura_Vector_3 xForward(m_xOrient.e13, m_xOrient.e23, m_xOrient.e33);

		Asura_Assert(m_bPosValid, "SetPosition not been called on AVP Laser");
		Asura_Assert(m_bOrientValid, "SetOrientation not been called on AVP Laser");

		if (m_bOriginPhotonEnabled && m_pxOriginPhoton)
		{
			m_pxOriginPhoton->SetPosition(GetPosition());
			m_pxOriginPhoton->SetAsActive(true);
		}

		if (GetNumBeams() == 1)
		{
			Asura_LOS_InputData xInputData( GetPosition(), xForward, GetGuidToIgnore(), GetMaximumLength());
			m_xBeamLengths[0] = Asura_LOS::TestAgainstEverything( &xInputData )->m_fDistanceToIntersection;
			if (m_pxContactPhotons[0] && m_bContactPhotonEnabled)
			{
				m_pxContactPhotons[0]->SetPosition(GetPosition() + xForward * (m_xBeamLengths[0] - m_fContactPhotonBackOffDistance));
			}
		}
		else 
		{
			float fAng = Asura_Maths::Pi * 2.0f / static_cast<float>(GetNumBeams());

			for (u_int i = 0; i < GetNumBeams(); ++i)
			{
				float fThisAng = fAng * float(i);
				float fS, fC;
				Asura_Maths::SinCos(fThisAng, fS, fC);

				Asura_Vector_3 xPos = xDown * -fC + xRight * fS;
				xPos *= GetRadius() * 0.57735026918962576450914878050225f;
				xPos += GetPosition();

				Asura_LOS_InputData xInputData( xPos, xForward, GetGuidToIgnore(), GetMaximumLength());
				m_xBeamLengths[i] = Asura_LOS::TestAgainstEverything( &xInputData )->m_fDistanceToIntersection;
				if (m_pxContactPhotons[i] && m_bContactPhotonEnabled)
				{
					m_pxContactPhotons[i]->SetPosition(xPos + xForward * (m_xBeamLengths[i] - m_fContactPhotonBackOffDistance));
				}
			}
		}
		for (u_int i = 0; i < GetNumBeams() && m_bContactPhotonEnabled; ++i)
		{
			if (m_pxContactPhotons[i])
			{
				m_pxContactPhotons[i]->SetAsActive(m_xBeamLengths[i] < GetMaximumLength());
			}
		}
	}
}


static float PRED_BEAM_PENTRATION = 0.1f;
void AVP_Laser::Render()
{
	if (s_pxFirst && dvs_bEnableLaser)
	{
		AVP_Platform_Laser_System::RenderStart();

		for (AVP_Laser *pxLaser = s_pxFirst; pxLaser; pxLaser = pxLaser->m_pxNext)
		{
			if (pxLaser->m_bVisiblity && pxLaser->m_bActive && pxLaser->m_rgbaColour.m_fA > 0.0f && pxLaser->m_uNumBeams > 0)
			{
				Asura_Vector_3 xRight(pxLaser->m_xOrient.e11, pxLaser->m_xOrient.e21, pxLaser->m_xOrient.e31);
				Asura_Vector_3 xDown(pxLaser->m_xOrient.e12, pxLaser->m_xOrient.e22, pxLaser->m_xOrient.e32);
				Asura_Vector_3 xForward(pxLaser->m_xOrient.e13, pxLaser->m_xOrient.e23, pxLaser->m_xOrient.e33);

				Asura_Assert(pxLaser->m_bPosValid, "SetPosition not been called on AVP Laser");
				Asura_Assert(pxLaser->m_bOrientValid, "SetOrientation not been called on AVP Laser");

				AVP_Platform_Laser_System::SetupLaserParams(pxLaser);
				if (pxLaser->m_uNumBeams == 1)
				{
					AVP_Platform_Laser_System::RenderLaser(pxLaser, pxLaser->m_xOrigin, xRight, xDown, xForward, pxLaser->m_xBeamLengths[0] + PRED_BEAM_PENTRATION);
				} 
				else 
				{
					float fAng = Asura_Maths::Pi * 2.0f / static_cast<float>(pxLaser->m_uNumBeams);

					for (u_int i = 0; i < pxLaser->m_uNumBeams; ++i)
					{
						float fThisAng = fAng * float(i);
						float fS, fC;
						Asura_Maths::SinCos(fThisAng, fS, fC);

						Asura_Vector_3 xPos = xDown * -fC + xRight * fS;
						xPos *= pxLaser->m_fRadius * 0.57735026918962576450914878050225f;
						xPos += pxLaser->m_xOrigin;

						AVP_Platform_Laser_System::RenderLaser(pxLaser, xPos, xRight, xDown, xForward, pxLaser->m_xBeamLengths[i] + PRED_BEAM_PENTRATION);
					}
				}
			}
		}

		AVP_Platform_Laser_System::RenderEnd();
	}

}

void AVP_Laser::Shutdown()
{
	if(Asura_CommandConsole::VarExists(L"AVP.Laser.Enable"))
	{
		Asura_CommandConsole::RemoveVar(L"AVP.Laser.Enable");
	}

	Asura_Assert(!s_pxFirst && !s_pxLast, "Shutting down AVP_Laser system with AVP_Laser's still around");
	AVP_Platform_Laser_System::Shutdown();
}

////////////////////////////////////////////////////////////////////////////////////////////////////
/*------------------------------------------------------**
** AVP_AnimationAttachment_Laser **
**------------------------------------------------------*/

AVP_AnimationAttachment_Laser::AVP_AnimationAttachment_Laser( AVP_Laser *pxLaser, bool bDeleteEffectWithParent )
: m_bEnableTarget(false), m_fInverseTargetLerpTime(2.0f), m_fTargetLerpValue(0.0f),m_xPrevTarget(0.0f, 0.0f, 0.0f),m_xTarget(0.0f, 0.0f, 0.0f),m_xCurTarget(0.0f, 0.0f, 0.0f),m_xOffset(0.0f, 0.0f, 0.0f) , m_pxLaser( pxLaser )
{
	m_iProjectorHandle = ASURA_INVALID_HANDLE;
#if defined(ASURA_ASSERT)
	Asura_Assert((pxLaser != NULL), "Cannot find Laser for attachment.");
#endif

	m_uDeleteAttachedObjectWithParent = bDeleteEffectWithParent;
}

// Called from Clone() function
AVP_AnimationAttachment_Laser::AVP_AnimationAttachment_Laser( const AVP_AnimationAttachment_Laser* pxOther )
: Asura_AnimationAttachment( pxOther )
{
	m_pxLaser = pxOther->m_pxLaser;
	m_uDeleteAttachedObjectWithParent = pxOther->m_uDeleteAttachedObjectWithParent;
}

AVP_AnimationAttachment_Laser::~AVP_AnimationAttachment_Laser()
{
	if( m_uDeleteAttachedObjectWithParent )
	{
 		Asura_Assert( m_pxLaser != NULL, "Unable to find projector for destruction" );
		if( m_pxLaser )
		{
			delete m_pxLaser;
		}
	}
	SetProjector(ASURA_INVALID_HANDLE, false);
}

Asura_AnimationAttachment* AVP_AnimationAttachment_Laser::Clone( Asura_Animation* pxParent )
{
	AVP_AnimationAttachment_Laser* pxAttachment = new AVP_AnimationAttachment_Laser( this );

	pxAttachment->m_pxParentAnimation = pxParent;

	// Recursively clone the list.
	if( m_pxNextAttachment )
	{
		pxAttachment->m_pxNextAttachment = m_pxNextAttachment->Clone( pxParent );
	}

	return pxAttachment;
}


void AVP_AnimationAttachment_Laser::Update( float fTime, bool bIgnoreTimerFrameStamp )
{

	//get world pos and orient
	Asura_Vector_3 xPos( Asura_Maths::ZeroVector3 );
	Asura_Matrix_3x3 xAnimOrient;
	if ( GetWorldRelativePosition( &xPos, &xAnimOrient) )
	{
		xPos += m_xOffset * xAnimOrient;
	
	
		if (m_fTargetLerpValue < 1.0f)
		{
			m_fTargetLerpValue += fTime * m_fInverseTargetLerpTime;
			if (m_fTargetLerpValue > 1.0f)
			{
				m_fTargetLerpValue = 1.0f;
			}
		} 
		else
		{
			// reached target set the prev to be current
			m_xPrevTarget = m_xTarget;
		}
	
		if (m_fTargetLerpValue > 0.0f)
		{
			if (m_iProjectorHandle != ASURA_INVALID_HANDLE)
			{
				Asura_Projector* const pxProjector = Asura_Projector_System::GetProjectorPointer( m_iProjectorHandle );
				if( pxProjector )
				{
					Asura_Matrix_3x3 xOrient( xAnimOrient );

					if ( m_bUsingTargetGuid )
					{
						const AVP_ClientEntity_Actor* pxTarget = AVP_ClientEntity_Actor::FindEntity( m_uTargetGuid );
						if ( pxTarget )
						{
							pxTarget->GetMarkerLocationFromHash( m_uTargetMPHash, &m_xTarget, NULL, true );
						}
					}

					m_xCurTarget = (m_xPrevTarget - m_xTarget);			// Move to origin
					float fLength = m_xCurTarget.Normalise();			// Make unit length
					m_xCurTarget *= (fLength*(1.0f-m_fTargetLerpValue));// Scale to parametric length
					m_xCurTarget += m_xTarget;							// Move back to position

					Asura_Vector_3 xTargetDir = m_xCurTarget - xPos;	// Convert to direction

					Asura_Maths::OrientationFromDirection(xAnimOrient, xTargetDir);

					pxProjector->SetPosition( xPos );
					pxProjector->SetOrientation( xAnimOrient );
					pxProjector->UpdateVertices();

					if( m_pxLaser )
					{
						m_pxLaser->SetPosition( xPos);
						m_pxLaser->SetOrientation(xAnimOrient);
						m_pxLaser->Update();
					}
				}
			} 
		}
	}
}

void AVP_AnimationAttachment_Laser::SetTarget(const Asura_Vector_3 &xTarget)
{
	if(m_bUsingTargetGuid)
	{
 		if(m_fTargetLerpValue > 0.0f && m_fTargetLerpValue < 1.0f)
 		{
			m_xPrevTarget = m_xCurTarget;
 		}

		SetTargetLerpValue(0.0f);
		m_bUsingTargetGuid = false;
	}
		
	m_uTargetGuid	= ASURA_GUID_UNREGISTERED;
	m_uTargetMPHash = ASURA_HASH_ID_UNSET;

	// Set new data
	m_xTarget = xTarget;
}
void AVP_AnimationAttachment_Laser::SetTarget(const Asura_Guid uTargetGuid, const Asura_Hash_ID uTargetMPHash)
{
	// Set new data
	if(m_uTargetGuid != uTargetGuid || m_uTargetMPHash != uTargetMPHash)
	{
		m_xPrevTarget = m_xCurTarget;

		SetTargetLerpValue(0.0f);
	}

	m_uTargetGuid = uTargetGuid;
	m_uTargetMPHash = uTargetMPHash;
	m_bUsingTargetGuid = true;
}
void AVP_AnimationAttachment_Laser::SetTargetEnabled(bool bEnable)
{
	m_bEnableTarget = bEnable;
}
void AVP_AnimationAttachment_Laser::SetTargetLerpTime(float fTargetLerpTime)
{
	m_fInverseTargetLerpTime = fTargetLerpTime > 0.0f ? 1.0f / fTargetLerpTime : 0.0f;
}
void AVP_AnimationAttachment_Laser::SetTargetLerpValue(float fTargetLerpValue)
{
	m_fTargetLerpValue = fTargetLerpValue;
}

const Asura_Vector_3 &AVP_AnimationAttachment_Laser::GetTarget(void) const
{
	return m_xTarget;
}
bool AVP_AnimationAttachment_Laser::GetTargetEnabled() const
{
	return m_bEnableTarget;
}
float AVP_AnimationAttachment_Laser::GetTargetLerpTime() const
{
	return m_fInverseTargetLerpTime > 0.0f ? 1.0f / m_fInverseTargetLerpTime : 0.0f;
}
float AVP_AnimationAttachment_Laser::GetTargetLerpValue() const
{
	return m_fTargetLerpValue;
}

void AVP_AnimationAttachment_Laser::OnParentVisibilityChanged( bool bInvisibility)
{
	if (GetLaser())
	{
		GetLaser()->SetActive((!bInvisibility) && m_pxParentAnimation);
	}
}


void AVP_AnimationAttachment_Laser::SetProjector(int iProjectorHandle, bool bDeleteWhenRemoved)
{
	if (iProjectorHandle != m_iProjectorHandle)
	{
		if (m_iProjectorHandle != ASURA_INVALID_HANDLE && m_bDeleteProjector)
		{
			Asura_Projector* const pxProjector = Asura_Projector_System::GetProjectorPointer( m_iProjectorHandle );
			Asura_Assert( pxProjector != NULL, "Unable to find projector for destruction" );
			if( pxProjector )
			{
				Asura_Projector_System::DestroyProjector( pxProjector );
			}
		}
		m_iProjectorHandle = iProjectorHandle;
		m_bDeleteProjector = bDeleteWhenRemoved;
	}
}

bool AVP_AnimationAttachment_Laser::LinkAttachment(Asura_Animation* pxParentAnimation,Asura_Hash_ID uMarkerHash,bool bDeleteWhenUnlinked)
{
	bool bRet = PARENT::LinkAttachment(pxParentAnimation, uMarkerHash, bDeleteWhenUnlinked);
	// cant do anything is LinkAttachment returns false as we might have just been deleted!
	if (bRet && GetLaser())
	{
		GetLaser()->SetActive(true);
	}
	return bRet;
}
void AVP_AnimationAttachment_Laser::UnlinkAttachment()
{
	if (GetLaser())
	{
		GetLaser()->SetActive(false);
	}
	PARENT::UnlinkAttachment();
}
