///////////////////////////////////////////////////////////////////////////////
//
// File: AVP_PerlinNoise.h
//
// Created by Andrew Southgate 05/12/2007
// 
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
// Handles AVP procedural perlin noise (to be used platform side to create textures)
//
///////////////////////////////////////////////////////////////////////////////

#if defined ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#if !defined __AVP_PERLINNOISE_H__
#define __AVP_PERLINNOISE_H__


#define AVP_NUM_PERLIN_NOISE_PERMUTATIONS		8

class AVP_PerlinNoise
{
public:
	struct _Grad
	{
		signed char x,y,z,w;
	};
private:
	static _Grad m_axGradients[16];
	static _Grad m_ax4dGradients[32];

	unsigned char m_aucPermutation[256];

public:
	AVP_PerlinNoise();
	const _Grad &grad(unsigned int index)
	{
		return m_axGradients[index % 16];
	}
	const _Grad &grad4d(unsigned int index)
	{
		return m_axGradients[index % 32];
	}
	unsigned int perm(unsigned int index)
	{
		return m_aucPermutation[index % 256];
	}
	const _Grad &permgrad(unsigned int index)
	{
		return grad(perm(index));
	}
	const _Grad &permgrad4d(unsigned int index)
	{
		return grad4d(perm(index));
	}
};

#endif // #if !defined __AVP_PERLINNOISE_H__
