/**
*
* AVP_PFX_Entity_Fire.cpp
*
* Created by RDM - 26 September 2006
*
* Copyright (C) 2006 Rebellion, All Rights Reserved.
*
**/

/************/
/* Includes */
/************/

// main asura header
#include "Core/Asura.h"

// local header
#include "SpecialFX/PFX/AVP_PFX_Entity_Fire.h"

// asura
#include "Animation/Asura_Animation.h"
#include "Animation/Asura_Hierarchy.h"
#include "Entity/PhysicalObject/Asura_ClientEntity_PhysicalObject.h"
#include "Rendering/Asura_Render.h"
#include "Shapes/Asura_Shapes.h"
#include "SpecialFX/PFX/Asura_PFX_Classes.h"
#include "SpecialFX/PFX/Asura_PFX_Source.h"
#include "SpecialFX/PFX/Asura_PFX_System.h"

/********/
/* Data */
/********/

// debug vars
DEBUGVAR bool  dbg_bDebugRender   = false;

/*************/
/* Functions */
/*************/

// constructor
AVP_PFX_Entity_Fire::AVP_PFX_Entity_Fire( void )
: m_pxSourceTemplates( NULL )
, m_uNumberOfSourceTemplates( 0 )
, m_pxSources( NULL )
, m_uNumberOfSources( 0 )
, m_bDieOut( false )
{
};

// destructor
AVP_PFX_Entity_Fire::~AVP_PFX_Entity_Fire()
{	
	DestroyAllSources();

	delete [] m_pxSourceTemplates;
}


void AVP_PFX_Entity_Fire::DestroyAllSources()
{
	// delete any existing sources
	if( m_pxSources )
	{
		for( u_int u=0; u< m_uNumberOfSources; ++u )
		{
			delete m_pxSources[u].m_pxSource;
		}

		delete [] m_pxSources;
	}

	m_pxSources = NULL;
	m_uNumberOfSources = 0;
}


//*************************************************************
// Static create method

u_int AVP_PFX_Entity_Fire::Create()
{
	if(Asura_PFX_System::CanCreateEffect())
	{
		AVP_PFX_Entity_Fire* pxFire = new AVP_PFX_Entity_Fire();
		if(pxFire && pxFire->IsDestroyed()==false )		// Check for (ASURA_PFX_EFFECT_DESTROYED)
		{
			return pxFire->GetID();
		}
	}
	return ASURA_PFX_INVALID_EFFECT_ID;
}


/*************************************************************
 *
 * Name          : AVP_PFX_Entity_Fire::UpdatePosition
 *
 * Author        : Richard May
 *
 * Description   : Updates the position of all the sources. This should 
 *				   be called from the appropriate render-related place
 *				   to ensure bone positions are up-to-date, etc.
 *
 *************************************************************/
void AVP_PFX_Entity_Fire::UpdatePosition ( const class Asura_Shape_Instance* pxShapeInstance )
{
	if( !pxShapeInstance ) return;

	// Do we have the right number of sources? There's a chance, if the skin/shape has 
	// changed since the effect was created that the sources will be out-of-date. 
	if( m_uNumberOfSources != GetNumberOfSourcesRequired( pxShapeInstance ) )
	{
		// Recreate the sources. Will delete any existing ones.
		CreateSources( pxShapeInstance );
	}

	Asura_Bounding_Box xCullingBB;
	xCullingBB.Reset();

	// update each source in turn
	for( u_int uSource=0; uSource<m_uNumberOfSources; ++uSource )
	{
		Asura_Bounding_Box xSourceBB;
		Asura_Vector_3     xSourcePos;
//		Asura_Matrix_3x3   xSourceOri;	 // EP. Blocked for testing purposes. Instead we are using matrix that points up

		Asura_PFX_Source* pxSource = m_pxSources[uSource].m_pxSource;

		if( pxShapeInstance->GetAnimation()
		 && pxShapeInstance->GetAnimation()->GetSkin() )
		{
			const Asura_Animation* pxAnim = pxShapeInstance->GetAnimation();

			// need the centre point of the bone
			if( (m_pxSources[uSource].m_iBoneID >= 0)
			 &&	(m_pxSources[uSource].m_iBoneID < pxAnim->GetSkin()->NumberOfBones) )
			{
				xSourceBB.Zero();
				if( pxAnim->GetBoneBB( m_pxSources[uSource].m_iBoneID ) )
				{
					xSourceBB = *pxAnim->GetBoneBB( m_pxSources[uSource].m_iBoneID );
				}
				
//				pxAnim->GetBoneLocation( m_pxSources[uSource].m_iBoneID, &xSourcePos, &xSourceOri, true );	 // EP. Blocked for testing purposes. Instead we are using matrix that points up
				pxAnim->GetBoneLocation( m_pxSources[uSource].m_iBoneID, &xSourcePos, NULL, true ); 
			}
		}
		else
		{
			Asura_Assert( (uSource == 0), "Shape fire effects shouldn't have more than one source!" );

			// just use shape's collision bb for now (it's displaced and rotated, so make object-relative)
			xSourceBB  = pxShapeInstance->GetShape()->GetLOSCollisionData()->GetBoundingBox();
			xSourcePos = pxShapeInstance->GetPosition();
//			xSourceOri = pxShapeInstance->GetOrientation();	// EP. Blocked for testing purposes. Instead we are using matrix that points up
		}
		
		// set position and orientation
		static const Asura_Vector_3 xUpVector( 0.0, -1.0f, 0.0f );
		Asura_Maths::OrientationFromDirection( pxSource->m_xDirection, xUpVector );	// Point fire all the time up.
//		pxSource->m_xDirection = xSourceOri;	// EP. Blocked for testing purposes. Instead we are using matrix that points up
		pxSource->m_xVolume = Asura_PFX_Volume( xSourceBB );
		pxSource->m_xVolume.m_bUseOrientation = true;

		// if the volume of the source volume is less than a metre cubed, spawn particles on the surface as it looks better
		if( pxSource->m_xVolume.m_xCuboidDesc.m_xBB.GetVolume() < 1.0f )
		{
			pxSource->m_xVolume.m_bSpawnOnSurface = true;
		}

		// update particle positions
		{
			Asura_Vector_3 xDisplacement = xSourcePos - pxSource->m_xPosition;
			for (u_int i=0;i<pxSource->m_uNumParticles;i++)
			{
				pxSource->m_pxParticles[i].m_xPosition += xDisplacement;
			}
		}

		pxSource->m_xPosition = xSourcePos;

		// update culling BB
		Asura_Bounding_Box xSourceCullingBB;
		xSourceCullingBB.FromOBB( xSourceBB, pxSource->m_xDirection );
		xSourceCullingBB.Displace( xSourcePos );	

		xCullingBB.Set( xSourceCullingBB.GetMax() );
		xCullingBB.Set( xSourceCullingBB.GetMin() );
	}

	// increase culling BB size slightly for paranoia's sake
	xCullingBB.MaxX += 0.1f;
	xCullingBB.MaxY += 0.1f;
	xCullingBB.MaxZ += 0.1f;

	xCullingBB.MinX -= 0.1f;
	xCullingBB.MinY -= 0.1f;
	xCullingBB.MinZ -= 0.1f;

	SetBoundingBox( xCullingBB );
}
/* end of function UpdatePosition */



/*************************************************************
*
* Name          : AVP_PFX_Entity_Fire::AddSourceTemplate
*
* Author        : Richard May
*
* Description   : Adds a source template to the list of templates and volumes
*
*************************************************************/
void AVP_PFX_Entity_Fire::AddSourceTemplate ( Asura_Hash_ID uSrcTmp, float fMinVolume )
{
	SrcTmpAndVolume* pxOldTemplates = m_pxSourceTemplates;
	
	m_pxSourceTemplates = new SrcTmpAndVolume[m_uNumberOfSourceTemplates+1];
	
	for( u_int u=0; u<m_uNumberOfSourceTemplates; ++u )
	{
		m_pxSourceTemplates[u] = pxOldTemplates[u];
	}

	delete [] pxOldTemplates;

	m_pxSourceTemplates[m_uNumberOfSourceTemplates].m_uSourceTemplate = uSrcTmp;
	m_pxSourceTemplates[m_uNumberOfSourceTemplates].m_fMinVolume      = fMinVolume;

	m_uNumberOfSourceTemplates++;
}
/* end of function AddSourceTemplate */



/*************************************************************
 *
 * Name          : AVP_PFX_Entity_Fire::Update
 *
 * Author        : Richard May
 *
 * Description   : Main update. Updates all the sources.
 *
 *************************************************************/
void AVP_PFX_Entity_Fire::Update ( void )
{
	Asura_Assertf( (m_uNumberOfSources > 0), ( "Entity Fire PFX (id %d) has no sources.", GetID() ) );

	// check visibility
	const bool bVisible = (GetCameraFlags(false) != 0);

	// update all the sources
	for( u_int u=0; u<m_uNumberOfSources; ++u )
	{
		Asura_PFX_Source* pxSource = m_pxSources[u].m_pxSource;
		
		pxSource->Update( bVisible, bVisible );
		
		if( (pxSource->m_fAge > 1.0f) && !m_bDieOut )
		{
			pxSource->m_fAge = 0.0f;		
		}
	}	
}
/* end of function Update */



/*************************************************************
 *
 * Name          : AVP_PFX_Entity_Fire::Render
 *
 * Author        : Richard May
 *
 * Description   : Renders all the sources! Woo!
 *
 *************************************************************/
void AVP_PFX_Entity_Fire::Render ( void )
{
	// render each source in turn
	for( u_int u=0; u<m_uNumberOfSources; ++u )
	{
		Asura_PFX_Source* pxSource = m_pxSources[u].m_pxSource;
		if( pxSource )
		{
			pxSource->Render();

			// debug rendering
			if( dbg_bDebugRender )
			{
				// source volume
				Asura_Render::RenderBB( pxSource->m_xVolume.m_xCuboidDesc.m_xBB, pxSource->m_xPosition, pxSource->m_xDirection, 0xffff0000 );

				// culling BB
				Asura_Render::RenderBB( GetBoundingBox(), Asura_Maths::ZeroVector3, Asura_Maths::IdentityMatrix3x3, 0xff00ff00 );

				// RDM - 26/09/06 - Okay, I'll admit it: I'm not sure why this is here. This was inherited from Rogue's beam rifle effect
				// so I'm assuming I put it here for a reason. Since this is debug rendering only, I'm going to leave it here for now.
				Asura_PFX_System::Platform_BeginRender();
			}
		}
	}
}
/* end of function Render */



/*************************************************************
 *
 * Name          : AVP_PFX_Entity_Fire::GetNumberOfSourcesRequired
 *
 * Author        : Richard May
 *
 * Description   : Returns the number of sources required for this effect
 *
 *************************************************************/
u_int AVP_PFX_Entity_Fire::GetNumberOfSourcesRequired ( const Asura_Shape_Instance* pxShapeInstance ) const
{
	// How many sources do we need? By default, just one for shapes.
	u_int uNumberOfSources = 1;

	const Asura_Animation*      pxAnim = pxShapeInstance->GetAnimation();
	const Asura_Hierarchy_Skin* pxSkin = pxAnim ? pxAnim->GetSkin() : NULL;

	if( pxSkin )
	{
#ifdef ASURA_TOOLS
		// make sure the section bounding boxes are valid
		if( !pxSkin->m_pxSectionBoundingBoxData )
		{
			Asura_Hierarchy_Skin* pxNonConstSkin = const_cast<Asura_Hierarchy_Skin*> (pxSkin);
			pxNonConstSkin->InitSectionBoundingBoxes();
		}
#endif
		
		uNumberOfSources = 0;

		// animations have a source for every bone that is greater than a minimum volume
		for( int iBone=0; iBone<pxSkin->NumberOfBones; ++iBone )
		{
			const Asura_Bounding_Box* pxBB = pxAnim->GetBoneBB(iBone);
			if( pxBB )
			{
				const float fVolume = pxBB->GetVolume();

//				Asura_OutputToDebugger(( "Bone: %i, Volume: %f, \n", iBone, fVolume));

				uNumberOfSources += GetNumOfSourcesForThisVolume( fVolume );

//				if( GetSourceTemplateForVolume( fVolume ) != ASURA_HASH_ID_UNSET )
//				{
//					uNumberOfSources++;
//				}
			}
		}
	}

	return uNumberOfSources;
}
/* end of function GetNumberOfSourcesRequired */





float AVP_PFX_Entity_Fire::GetBiggestBoneVolume ( const Asura_Shape_Instance* pxShapeInstance ) const
{
	// How many sources do we need? By default, just one for shapes.
	float fBiggestBoneVolume = 0.0f;

	const Asura_Animation*      pxAnim = pxShapeInstance->GetAnimation();
	const Asura_Hierarchy_Skin* pxSkin = pxAnim ? pxAnim->GetSkin() : NULL;

	if( pxSkin )
	{
#ifdef ASURA_TOOLS
		// make sure the section bounding boxes are valid
		if( !pxSkin->m_pxSectionBoundingBoxData )
		{
			Asura_Hierarchy_Skin* pxNonConstSkin = const_cast<Asura_Hierarchy_Skin*> (pxSkin);
			pxNonConstSkin->InitSectionBoundingBoxes();
		}
#endif

		// animations have a source for every bone that is greater than a minimum volume
		for( int iBone=0; iBone<pxSkin->NumberOfBones; ++iBone )
		{
			const Asura_Bounding_Box* pxBB = pxAnim->GetBoneBB(iBone);
			if( pxBB )
			{
				const float fVolume = pxBB->GetVolume();
				if( fVolume>fBiggestBoneVolume )
				{
					fBiggestBoneVolume = fVolume;
				}
			}
		}
	}

	return fBiggestBoneVolume;
}
/* end of function GetNumberOfSourcesRequired */



/*************************************************************
 *
 * Name          : AVP_PFX_Entity_Fire::GetSourceTemplateForVolume
 *
 * Author        : Richard May
 *
 * Description   : Returns the source template for a specific volume
 *
 *************************************************************/
/*Asura_Hash_ID AVP_PFX_Entity_Fire::GetSourceTemplateForVolume ( float fVolume ) const
{
	Asura_Hash_ID uTemplateID = ASURA_HASH_ID_UNSET;
	float fLargestMinVolumeSoFar = 0.0f;

	for( u_int u=0; u<m_uNumberOfSourceTemplates; ++u )
	{
		if( (fVolume > m_pxSourceTemplates[u].m_fMinVolume)
			&& (m_pxSourceTemplates[u].m_fMinVolume > fLargestMinVolumeSoFar) )
		{
			fLargestMinVolumeSoFar = m_pxSourceTemplates[u].m_fMinVolume;
			uTemplateID = m_pxSourceTemplates[u].m_uSourceTemplate;
		}				   
	}

	return uTemplateID;
}
*/
/* end of function GetSourceTemplateForVolume */



u_int AVP_PFX_Entity_Fire::GetNumOfSourcesForThisVolume( float fVolume ) const
{
	u_int uNumOfSources = 0;
	for( u_int u=0; u<m_uNumberOfSourceTemplates; ++u )
	{
		if( fVolume > m_pxSourceTemplates[u].m_fMinVolume )	// Potential error. We check only for one threshold value.
		{
			uNumOfSources++;
		}				   
	}

	return uNumOfSources;
}



/*************************************************************
 *
 * Name          : AVP_PFX_Entity_Fire::CreateSources
 *
 * Author        : Richard May
 *
 * Description   : Creates all the sources for the given shape/animation.
 *
 *************************************************************/
void AVP_PFX_Entity_Fire::CreateSources ( const class Asura_Shape_Instance* pxShapeInstance )
{
	Asura_Assert( m_uNumberOfSourceTemplates, "Fire effect has no source templates set!" );
	if( m_uNumberOfSourceTemplates == 0 ) return;

	// delete any existing sources
	DestroyAllSources();

	m_uNumberOfSources = GetNumberOfSourcesRequired( pxShapeInstance );

	const Asura_PFX_Volume xTempVolume( GetBoundingBox() );

	// allocate the source storage
	m_pxSources = new SourceAndBoneID[m_uNumberOfSources];

	// Clear sources.
	for( u_int uSource=0; uSource<m_uNumberOfSources; ++uSource )
	{				
		m_pxSources[uSource].m_iBoneID  = -1;
	}



	const Asura_Animation*	    pxAnim = pxShapeInstance->GetAnimation();
	const Asura_Hierarchy_Skin* pxSkin = pxAnim ? pxAnim->GetSkin() : NULL;

	u_int uSource = 0;

	if( pxSkin )
	{
		// Go trough all bones.
		// look on from the last valid bone we found - this should match the all sources to a valid bone
		for( int iBone=0; iBone < pxSkin->NumberOfBones; ++iBone )
		{
			const Asura_Bounding_Box* pxBB = pxAnim->GetBoneBB(iBone);
			if( !pxBB ) continue; 

			const float fVolume = pxBB->GetVolume();

			// check all sources and see what sources we need to create and attach.
			for( u_int u=0; u<m_uNumberOfSourceTemplates; ++u )
			{
				if( fVolume > m_pxSourceTemplates[u].m_fMinVolume )	// Potential error. We check only for one threshold value.
				{
					const Asura_Hash_ID uSourceTemplateID = m_pxSourceTemplates[u].m_uSourceTemplate;

					Asura_Assert( uSource<m_uNumberOfSources, "Fatal error. Trying to create source for fire, but there is something corrupted. Source code needs to be fixed." );
					// Create fire and attach it.
					if( uSourceTemplateID != ASURA_HASH_ID_UNSET && uSource<m_uNumberOfSources )
					{
						m_pxSources[uSource].m_pxSource = new Asura_PFX_Source( this, pxShapeInstance->GetPosition(), pxShapeInstance->GetOrientation(), uSourceTemplateID, xTempVolume );
						m_pxSources[uSource].m_iBoneID = iBone;
						++uSource;
					}
				}
			}
		}
	}
	//else	// Object has no skin. Attach all fire effect to original bounding box.
	//{
		//const Asura_Bounding_Box xSourceBB = pxShapeInstance->GetShape()->GetLOSCollisionData()->GetBoundingBox();
		//const float fVolume = xSourceBB.GetVolume();

		// Will deal with this later.
	//}
/*
	for( u_int uSource=0; uSource<m_uNumberOfSources; ++uSource )
	{				
		Asura_Hash_ID uSourceTemplateID = ASURA_HASH_ID_UNSET;
		m_pxSources[uSource].m_iBoneID  = -1;




	int iNextBoneID = 0;
	for( u_int uSource=0; uSource<m_uNumberOfSources; ++uSource )
	{				
		Asura_Hash_ID uSourceTemplateID = ASURA_HASH_ID_UNSET;
		m_pxSources[uSource].m_iBoneID  = -1;


		const Asura_Animation*	    pxAnim = pxShapeInstance->GetAnimation();
		const Asura_Hierarchy_Skin* pxSkin = pxAnim ? pxAnim->GetSkin() : NULL;

		// find the correct bone ID for this source (if it's a skin).		
		if( pxSkin )
		{
			// look on from the last valid bone we found - this should match the all sources to a valid bone
			for( int iBone=iNextBoneID; iBone<pxSkin->NumberOfBones; ++iBone )
			{
				const Asura_Bounding_Box* pxBB = pxAnim->GetBoneBB(iBone);

				if( !pxBB ) continue; 

				Asura_Hash_ID uSrcTemplate = GetSourceTemplateForVolume( pxBB->GetVolume() );                
				if( uSrcTemplate != ASURA_HASH_ID_UNSET )
				{
					uSourceTemplateID = uSrcTemplate;
					m_pxSources[uSource].m_iBoneID = iBone;
					iNextBoneID = iBone + 1;
					
					break;
				}
			}

			// make sure we found a bone for this source
			Asura_Assertf( (m_pxSources[uSource].m_iBoneID != -1), ("Failed to find a bone for source %d.",uSource) );			
		}		
		else
		{
			// use shape BB for scale calculation (never less than 1)
			const Asura_Bounding_Box xSourceBB = pxShapeInstance->GetShape()->GetLOSCollisionData()->GetBoundingBox();
			const float fVolume = xSourceBB.GetVolume();
			uSourceTemplateID = GetSourceTemplateForVolume( fVolume );
		}

		// finally, create the source
		Asura_Assert( uSourceTemplateID != ASURA_HASH_ID_UNSET, "Failed to get template for source." );
		if( uSourceTemplateID != ASURA_HASH_ID_UNSET )
		{
			m_pxSources[uSource].m_pxSource = new Asura_PFX_Source( this, pxShapeInstance->GetPosition(), pxShapeInstance->GetOrientation(), uSourceTemplateID, xTempVolume );
		}
	}
	*/
}
/* end of function CreateSources */



void AVP_PFX_Entity_Fire::Stop()
{
	m_eState = ASURA_PFX_EFFECT_STOPPING;

	// update each source in turn
	for( u_int uSource=0; uSource<m_uNumberOfSources; ++uSource )
	{
		Asura_PFX_Source* pxSource = m_pxSources[uSource].m_pxSource;
		pxSource->Stop( false );
	}
}


/***************/
/* End of File */
/***************/
