/******************************************************************************/
/* Included Files                                                             */
/******************************************************************************/

// Includes must be this way round here
#include "..\..\..\..\..\Asura\PC\Shaders\Shared.fxh"
#include "..\..\..\..\..\Asura\PC\Shaders\MaterialResponse_Shared.fxh"


/******************************************************************************/
/* Parameters                                                                 */
/******************************************************************************/

const float4x4 g_xWorldViewProjectionMatrix;
const float4x3 g_axBoneMatrices[80];

const float2 g_xOneOverScreenSize;


const float3 g_xBlurColour;
const float2 g_xBlurParams = {2.0f / 512.0f, 25.0f}; // x = blur kernel size, y = max distance


const texture g_xDepthTexture;
const texture g_xSceneTexture;

const sampler g_xSceneTextureSampler  = sampler_state
{
	Texture   = <g_xSceneTexture>;
	AddressU  = CLAMP;
	AddressV  = CLAMP;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
	MIPFILTER = LINEAR;
};

const sampler g_xDepthTextureSampler  = sampler_state
{
	Texture   = <g_xDepthTexture>;
	AddressU  = CLAMP;
	AddressV  = CLAMP;
	MinFilter = POINT;
	MagFilter = POINT;
	MIPFILTER = POINT;
};



/******************************************************************************/
/* Types                                                                      */
/******************************************************************************/

struct InputVertex
{
	float4 m_xPosition     : POSITION;

	int4   m_aiBoneIndices : BLENDINDICES;
	float4 m_afBoneWeights : BLENDWEIGHT;

	float3 m_xNormal       : NORMAL;

};

struct OutputVertex_Deferred
{
	float4 m_xPosition              : POSITION;
	float4  m_fColour               : TEXCOORD0;
	float4  m_fColour2               : TEXCOORD1;
};


struct VertexFS
{
	float4 m_xPosition : POSITION;
	float2 m_xUV       : TEXCOORD;
};

/******************************************************************************/
/* Functions                                                                  */
/******************************************************************************/

float EncodeDepth(float f)
{
	return 1.0f - saturate(f / g_xBlurParams.y);
}
float DecodeDepth(float f)
{
	return (1.0f - f) * g_xBlurParams.y;
}

OutputVertex_Deferred VShader( const InputVertex xIn )
{
	// Position and UVs
	float4   xObjectSpacePosition = float4(0,0,0,1); 
	float4x3 axBoneMatrix;
	
	OutputVertex_Deferred xOut;
				
	// Decrementing loop counter doesn't seem to work, and
	// so when we finish we want the most heavily weighted bone - BoneWeights[0] - in 
	// axBoneMatrix, hence the '3-i' indexing
	for(int i=0; i<4; i++)
	{
		axBoneMatrix = g_axBoneMatrices[ xIn.m_aiBoneIndices[3-i] ];		

		// Recompute the matrix elements that we've stored per-bone-effects values in
		axBoneMatrix[2] = cross (axBoneMatrix[0], axBoneMatrix[1]);
		xObjectSpacePosition.xyz += mul( xIn.m_xPosition, axBoneMatrix) * xIn.m_afBoneWeights[3-i];
	}
	float3 xNormal	= mul( xIn.m_xNormal, axBoneMatrix );
	xNormal	= mul(float4( xNormal, 0.0f), g_xWorldViewProjectionMatrix );


	xOut.m_xPosition = mul( xObjectSpacePosition, g_xWorldViewProjectionMatrix );;
	float fDepth = EncodeDepth(xOut.m_xPosition.w);
	xOut.m_fColour = float4(g_xBlurColour.rgb * pow(smoothstep(0.05f, 0.3f, fDepth ), 2.0f), fDepth);
	xOut.m_fColour2 = float4(0.0f, 0.0f, 0.0f, smoothstep(0.0f, 0.05f, fDepth));
	return xOut;
}
struct OutputPixel
{
	half4 m_xBufferColour    : COLOR0;
	half4 m_xBufferAlpha	: COLOR1;
};

OutputPixel PShader( const OutputVertex_Deferred xIn)
{
	OutputPixel xOut;
	xOut.m_xBufferColour = xIn.m_fColour;
	xOut.m_xBufferAlpha = xIn.m_fColour2;
	return xOut;
}

float4 VShader_FS( const VertexFS xIn ) : POSITION
{
	return xIn.m_xPosition;
}

float4 Avg4(sampler xSampler, const float2 xUV)
{
	float fSampleSize = 1.0f;
	float4 xInitialColour;
	xInitialColour  = tex2D(xSampler, xUV + float2(-fSampleSize*g_xOneOverScreenSize.x, -fSampleSize*g_xOneOverScreenSize.y));
	xInitialColour += tex2D(xSampler, xUV + float2(-fSampleSize*g_xOneOverScreenSize.x,  fSampleSize*g_xOneOverScreenSize.y));
	xInitialColour += tex2D(xSampler, xUV + float2( fSampleSize*g_xOneOverScreenSize.x, -fSampleSize*g_xOneOverScreenSize.y));
	xInitialColour += tex2D(xSampler, xUV + float2( fSampleSize*g_xOneOverScreenSize.x,  fSampleSize*g_xOneOverScreenSize.y));

	return xInitialColour / 4;
}


float4 BlurLine(sampler xSampler, const float2 xUV, const float2 xOffset)
{
	float4 xInitialColour = 
			tex2D(xSampler, xUV - xOffset * 5.0f)
		+	tex2D(xSampler, xUV - xOffset * 3.0f)
		+	tex2D(xSampler, xUV - xOffset)
		+	tex2D(xSampler, xUV + xOffset)
		+	tex2D(xSampler, xUV + xOffset * 3.0f)
		+	tex2D(xSampler, xUV + xOffset * 5.0f)
		;

	return xInitialColour * 1.0f / 6.0f;
}

float BlurDepth(sampler xSampler, const float2 xUV, const float2 xOffset)
{
	float fDepth;
	fDepth = tex2D(xSampler, xUV - xOffset * 3.0f).a;
	fDepth += tex2D(xSampler, xUV - xOffset).a;
	fDepth += tex2D(xSampler, xUV + xOffset).a;
	fDepth += tex2D(xSampler, xUV + xOffset * 3.0f).a;

	return fDepth / 4.0f;
}


float2 VPOS2UV(const float2 xScreenPos)
{
	return (xScreenPos + 0.5f)  * g_xOneOverScreenSize;
}


float4 PShader_BlurHoriz( const float2 xScreenPos : VPOS ) : COLOR
{
 	float fDepth = BlurDepth(g_xSceneTextureSampler, VPOS2UV(xScreenPos), float2(g_xBlurParams.x, 0.0f));
	float2 f2BlurSize = float2(g_xBlurParams.x, 0.0f);
 	f2BlurSize /= (0.25f + DecodeDepth(fDepth));
 	return float4(BlurLine(g_xSceneTextureSampler, VPOS2UV(xScreenPos),  f2BlurSize).rgb, fDepth);
}

float4 PShader_BlurVert( const float2 xScreenPos : VPOS ) : COLOR
{
	float fDepth = BlurDepth(g_xSceneTextureSampler, VPOS2UV(xScreenPos), float2(0.0f, g_xBlurParams.x));
	float2 f2BlurSize = float2(0.0f, g_xBlurParams.x);
 	f2BlurSize /= (0.25f + DecodeDepth(fDepth));
	float4 xInitialColour = BlurLine(g_xSceneTextureSampler, VPOS2UV(xScreenPos), f2BlurSize);


 	return float4(xInitialColour.rgb * 2.0f, 1.0f);
}

float4 PShader_ApplyToScreen( const float2 xScreenPos : VPOS ) : COLOR
{
 	float4 xRet = Avg4(g_xSceneTextureSampler, VPOS2UV(xScreenPos));

	return float4(pow(smoothstep(0.01f, 1.0f, xRet), 1.0f).xyz, 1.0f);
}




/******************************************************************************/
/* Techniques                                                                 */
/******************************************************************************/

// Perform material response using the SH
technique
{
	pass
	{
		VertexShader = compile vs_3_0 VShader( );
		PixelShader = compile ps_3_0 PShader( );
	}
}

technique FS
{
	pass
	{
		VertexShader = compile vs_3_0 VShader_FS();
		PixelShader = compile ps_3_0 PShader_BlurHoriz();
	}
	pass
	{
		ALPHABLENDENABLE = TRUE;
		SRCBLEND =	INVDESTALPHA;
		DESTBLEND = ZERO;
		SRCBLENDALPHA = ZERO;
		DESTBLENDALPHA = ZERO;
		BLENDOP = ADD;

		VertexShader = compile vs_3_0 VShader_FS();
		PixelShader = compile ps_3_0 PShader_BlurVert();
	}

	pass
	{
		ALPHABLENDENABLE = TRUE;
		SRCBLEND = ONE;
		DESTBLEND = ONE;
		SRCBLENDALPHA = ZERO;
		DESTBLENDALPHA = ZERO;
		BLENDOP = ADD;

		VertexShader = compile vs_3_0 VShader_FS();
		PixelShader = compile ps_3_0 PShader_ApplyToScreen();
	}

}
