/******************************************************************************/
/* Included Files                                                             */
/******************************************************************************/

// Includes must be this way round here
#include "..\..\..\..\..\Asura\PC\Shaders\Shared.fxh"
#include "..\..\..\..\..\Asura\PC\Shaders\MaterialResponse_Shared.fxh"


/******************************************************************************/
/* Parameters                                                                 */
/******************************************************************************/

const float4x4 g_xWorldViewProjectionMatrix;
const float4x4 g_xWorldViewMatrix;
const float4x4 g_xWorldMatrix;
const float4x3 g_axBoneMatrices[80];

const float3 g_xObjectSpaceCameraPosition;

const bool g_bUseDamageMaps;

const float2 g_xDistortionScales;
const float2 g_xOneOverScreenDimensions;

const float3 g_xCloakFadeOut = float3(1.0f / 30.0f, 0.707f, 1.0f / (1.0f - 0.707f));
// x = Maximum Distance
// y = cos(Maximum angle)
// z = 1.0f / (1.0f - cos(Maximum angle))

const float4 g_fOffset;

const float2 g_fParams = float2(3.61f, 0.06f);
// x = Scale
// y = LightningScale
const float4 g_fLightningColour;
// w = Threshold


const texture2D g_xSceneTexture;
const sampler2D g_xSceneSampler = sampler_state
{
	Texture   = <g_xSceneTexture>;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
	MipFilter = NONE;
	AddressU  = CLAMP;
	AddressV  = CLAMP;

	// We sample the post-tone-map back buffer capture, which is stored in sRGB
	sRGBTexture = <g_bUseGammaCorrection>;
};

const texture2D g_xNoiseTexture;
const sampler2D g_xNoiseTextureSampler = sampler_state
{
	Texture   = <g_xNoiseTexture>;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
	MipFilter = LINEAR;
	AddressU  = WRAP;
	AddressV  = WRAP;
};


/******************************************************************************/
/* Types                                                                      */
/******************************************************************************/

struct InputVertex
{
	float4 m_xPosition     : POSITION;
	float2 m_xUV				: TEXCOORD0;
	float2 m_xUV2				: TEXCOORD1;
	float  m_fDualTextureLerp   : TEXCOORD2;

	int4   m_aiBoneIndices : BLENDINDICES;
	float4 m_afBoneWeights : BLENDWEIGHT;

	float3 m_xNormal       : NORMAL;
	float3 m_xTangent      : TANGENT;
	float3 m_xBinormal     : BINORMAL;
};

struct OutputVertex_Deferred
{
	float4 m_xPosition              : POSITION;

	float3 m_xNormal                : NORMAL;
	float3 m_xTangent               : TANGENT;
	float3 m_xBinormal              : BINORMAL;

	float4 m_xUVs					: TEXCOORD0;
	float2 m_xBlends				: TEXCOORD1;

	float4 m_xCameraDirDistance     : TEXCOORD2; // Direction to camera, and distance from view plane in w
	float3 m_xTangentSpaceCameraDir : TEXCOORD3;

	float3 m_xObjectSpacePosition   : TEXCOORD4;

};

struct OutputVertex_Forward
{
	float4 m_xPosition              : POSITION;

	float3 m_xNormal                : NORMAL;

	float4 m_xObjectSpacePosition   : TEXCOORD0;
	float m_fDistortionIntensity    : TEXCOORD1; 

};


/******************************************************************************/
/* Functions                                                                  */
/******************************************************************************/

OutputVertex_Deferred VShader_Deferred( const InputVertex xIn )
{
	// Position and UVs
	float4   xObjectSpacePosition = float4(0,0,0,1); 
	float4x3 axBoneMatrix;
	float    fAlbedoBlend = 0;
	float	 fVertexOffset = 0.0f;
	
	OutputVertex_Deferred xOut;
	
	if ( g_bUseDamageMaps )
	{
		// the code section below and comment from Dave Reed

		// Decrementing loop counter doesn't seem to work, and
		// so when we finish we want the most heavily weighted bone - BoneWeights[0] - in 
		// axBoneMatrix, hence the '3-i' indexing
		for(int i=0; i<4; i++)
		{
			axBoneMatrix = g_axBoneMatrices[ xIn.m_aiBoneIndices[3-i] ];		
			
			// Get the damagemap blend value
			fAlbedoBlend += (axBoneMatrix[2][2]*xIn.m_afBoneWeights[3-i]);
			// Get the offset value
			fVertexOffset += (axBoneMatrix[2][1]*xIn.m_afBoneWeights[3-i]);

			// Recompute the matrix elements that we've stored per-bone-effects values in
			axBoneMatrix[2] = cross (axBoneMatrix[0], axBoneMatrix[1]);
			xObjectSpacePosition.xyz += mul( xIn.m_xPosition, axBoneMatrix) * xIn.m_afBoneWeights[3-i];
		}

		// Tangent, binormal & normal (just transformed to object space to match SH)
		xOut.m_xNormal	= mul( xIn.m_xNormal,	axBoneMatrix );
		xOut.m_xTangent = mul( xIn.m_xTangent,	axBoneMatrix );
		xOut.m_xBinormal= mul( xIn.m_xBinormal,	axBoneMatrix );
		
		// Damage map values are reversed with respect to normal mat resp lookups
		fAlbedoBlend = 1.0f - fAlbedoBlend;
	}
	else
	{
		fAlbedoBlend = xIn.m_fDualTextureLerp;
			
		// Decrementing loop counter doesn't seem to work, and
		// so when we finish we want the most heavily weighted bone - BoneWeights[0] - in 
		// axBoneMatrix, hence the '3-i' indexing
		for(int i=0; i<4; i++)
		{
			axBoneMatrix = g_axBoneMatrices[ xIn.m_aiBoneIndices[3-i] ];		
			
			// Get the offset value
			fVertexOffset += (axBoneMatrix[2][1]*xIn.m_afBoneWeights[3-i]);

			// Recompute the matrix elements that we've stored per-bone-effects values in
			axBoneMatrix[2] = cross (axBoneMatrix[0], axBoneMatrix[1]);
			xObjectSpacePosition.xyz += mul( xIn.m_xPosition, axBoneMatrix) * xIn.m_afBoneWeights[3-i];
		}

		// Tangent, binormal & normal (just transformed to object space to match SH)
		xOut.m_xNormal	= mul( xIn.m_xNormal,	axBoneMatrix );
		xOut.m_xTangent = mul( xIn.m_xTangent,	axBoneMatrix );
		xOut.m_xBinormal= mul( xIn.m_xBinormal,	axBoneMatrix );
	}
 
	xOut.m_xPosition = mul( xObjectSpacePosition, g_xWorldViewProjectionMatrix );;

	// damage maps use the first set of UV's twice	
	if (g_bUseDamageMaps)
	{
		xOut.m_xUVs = float4(xIn.m_xUV, xIn.m_xUV);
	} else {
		xOut.m_xUVs = float4(xIn.m_xUV, xIn.m_xUV2);
	}
	xOut.m_xBlends = float2(fAlbedoBlend, fVertexOffset);
	xOut.m_xObjectSpacePosition.xyz = xIn.m_xPosition;

	// Direction to camera, and world-space distance to vertex from viewer

	const float4 xWorldSpacePosition = mul( xObjectSpacePosition, g_xWorldMatrix );

	xOut.m_xCameraDirDistance.xyz = g_xObjectSpaceCameraPosition.xyz - xObjectSpacePosition.xyz;
	xOut.m_xCameraDirDistance.w = xOut.m_xPosition.w;

	const float3x3 xTangentToObjectMatrix = float3x3( xOut.m_xTangent,
													  xOut.m_xBinormal,
													  xOut.m_xNormal );

	// Note implicit transpose in the mul, by swapping operand order
	xOut.m_xTangentSpaceCameraDir = mul( xTangentToObjectMatrix, xOut.m_xCameraDirDistance.xyz );

	// We're doing lighting in world-space now
	xOut.m_xNormal	 = mul( xOut.m_xNormal,	  g_xWorldMatrix );
	xOut.m_xTangent	 = mul( xOut.m_xTangent,  g_xWorldMatrix );
	xOut.m_xBinormal = mul( xOut.m_xBinormal, g_xWorldMatrix );

	return xOut;
}

OutputVertex_Forward VShader_Forward( const InputVertex xIn )
{
	// Position and UVs
	float4   xObjectSpacePosition = float4(0,0,0,1); 
	float4x3 axBoneMatrix;
	float    fAlbedoBlend = 0;
	float	 fVertexOffset = 0.0f;
	
	OutputVertex_Forward xOut;
	
	// Decrementing loop counter doesn't seem to work, and
	// so when we finish we want the most heavily weighted bone - BoneWeights[0] - in 
	// axBoneMatrix, hence the '3-i' indexing
	for(int i=0; i<4; i++)
	{
		axBoneMatrix = g_axBoneMatrices[ xIn.m_aiBoneIndices[3-i] ];		
		
		// Get the offset value
		fVertexOffset += (axBoneMatrix[2][1]*xIn.m_afBoneWeights[3-i]);

		// Recompute the matrix elements that we've stored per-bone-effects values in
		axBoneMatrix[2] = cross (axBoneMatrix[0], axBoneMatrix[1]);
		xObjectSpacePosition.xyz += mul( xIn.m_xPosition, axBoneMatrix) * xIn.m_afBoneWeights[3-i];
	}

	// Tangent, binormal & normal (just transformed to object space to match SH)
	xOut.m_xNormal	= mul( xIn.m_xNormal,	axBoneMatrix );
 
 
	xOut.m_xPosition = mul( xObjectSpacePosition, g_xWorldViewProjectionMatrix );;
	xOut.m_xObjectSpacePosition = float4(xIn.m_xPosition.xyz, fVertexOffset);

	float3 xCameraDirDistance = normalize(mul(xObjectSpacePosition.xyz - g_xObjectSpaceCameraPosition.xyz,  g_xWorldViewMatrix ));
	xOut.m_fDistortionIntensity = saturate((xCameraDirDistance.z - g_xCloakFadeOut.y) * g_xCloakFadeOut.z);
	xOut.m_fDistortionIntensity *= (1.0f - saturate(xOut.m_xPosition.w * g_xCloakFadeOut.x)) / xOut.m_xPosition.w;


	// We're doing lighting in world-space now
	xOut.m_xNormal	 = mul( xOut.m_xNormal,	  g_xWorldViewMatrix );

	return xOut;
}


half3 GetDistortionValue(float3 xNormal, half2 xScreenPos : VPOS, float fDistortionIntensity )
{
	const half3 xViewSpaceNormal = xNormal;

	half hDistortionAmount = length( xViewSpaceNormal.xy) ;
	hDistortionAmount = (fmod( hDistortionAmount, 0.2h ) * 5.0h);

	const half2 xDistortion = xViewSpaceNormal.xy * hDistortionAmount;
	xScreenPos += xDistortion * g_xDistortionScales * fDistortionIntensity;

	xScreenPos = (xScreenPos + half2( 0.5h, 0.5h )) * g_xOneOverScreenDimensions;
	return tex2D( g_xSceneSampler, xScreenPos );
}


float NoiseLookup(float3 xPos)
{
	float fNoise = tex2D(g_xNoiseTextureSampler, xPos.xy).r
		+ tex2D(g_xNoiseTextureSampler, xPos.yz).g
		+ tex2D(g_xNoiseTextureSampler, xPos.zx).b;

	return fNoise * 0.3333f * 2.0f - 1.0f;
}

float CalculateNoise(float3 p, float fOffset)
{
	return NoiseLookup(p * g_fParams.x + g_fOffset) + fOffset;
}

DeferredOutputPixel PShader_Deferred( const OutputVertex_Deferred xIn, uniform bool bUseResponse)
{
	const half3 xInterpolatedTangent  = normalize( xIn.m_xTangent );
	const half3 xInterpolatedBinormal = normalize( xIn.m_xBinormal );
	const half3 xInterpolatedNormal   = normalize( xIn.m_xNormal );

	const half3x3 xTangentToWorldMatrix
		= { xInterpolatedTangent, xInterpolatedBinormal, xInterpolatedNormal };

	const half3 xViewDir = normalize( xIn.m_xCameraDirDistance.xyz );
	const half3 xTangentSpaceViewDir = normalize( xIn.m_xTangentSpaceCameraDir );

	const half hDualTextureLerp = xIn.m_xBlends.x;

	half4 xUVPair = xIn.m_xUVs;
	xUVPair = MaterialResponse_AnimateUVs( xUVPair );
	ApplyParallaxMapping( xTangentSpaceViewDir, hDualTextureLerp, xUVPair );

	const half4 xWorldSpaceNormal = NormalLookup( xUVPair, hDualTextureLerp, xTangentToWorldMatrix, xTangentToWorldMatrix );

	half3 xAlbedo;
	half hSpecular;
	float fMaterialsTextureY;
	half4 xColour;

	if( bUseResponse )
	{
		const half4 xSHLookupDir = half4( xInterpolatedNormal, 1.0h );
		const float3 xLightColour
			= SHLookup( xSHLookupDir, g_xRedSH, g_xGreenSH, g_xBlueSH ) * 2.0f;

		const float fLightStrength = GetLuminance( xLightColour );

		xColour = MaterialResponse( xUVPair, hDualTextureLerp, xWorldSpaceNormal, xViewDir,
			xInterpolatedNormal, xLightColour, xLightColour, fLightStrength, true, xAlbedo,
			hSpecular, fMaterialsTextureY );

		xColour.rgb += SelfIlluminationLookup( xUVPair, hDualTextureLerp, xAlbedo );
	}
	else
	{
		fMaterialsTextureY = g_xMaterialProperties.x;

		// For this base geometry pass, we'll just use the SH lighting + self illumination

		const float3 xAlbedoTimes2 = AlbedoLookup( xUVPair,
												   hDualTextureLerp,
												   xWorldSpaceNormal,
												   xViewDir,
												   xAlbedo );

		xColour.rgb =
#if defined( ASURA_TOOLS)
			g_bDisableLighting
			? xAlbedo :
#endif // #if defined( ASURA_TOOLS)
			SHLookup( xWorldSpaceNormal, g_xRedSH, g_xGreenSH, g_xBlueSH ) * xAlbedoTimes2
			+ SelfIlluminationLookup( xUVPair, hDualTextureLerp, xAlbedo );

		// Transparent env strips aren't sorted - but we need to use the transparency maps if present
		// to allow alpha decals
		xColour.a = TransparencyLookup( xUVPair, hDualTextureLerp );

		hSpecular = SpecularLookup( xUVPair, hDualTextureLerp );
	}

	// Do 1-bit alpha testing like this, since it seems to be done for deferred objects this way
	clip( xColour.a - 0.05h );

	DeferredOutputPixel xOut = Deferred_FillOutputPixel(
										xColour,
										xWorldSpaceNormal.xyz,
										xIn.m_xCameraDirDistance.w,
										xAlbedo,
										hSpecular,
										fMaterialsTextureY
										);

	float fNoise = CalculateNoise(xIn.m_xObjectSpacePosition, xIn.m_xBlends.y);
	clip(-fNoise);
																		
	return( xOut);
}

half4 PShader_Forward( const OutputVertex_Forward xIn, half2 xScreenPos : VPOS, uniform bool bUseNoise ) : COLOR
{
	half4 hScene;
	float fDistortionIntensity = saturate(xIn.m_fDistortionIntensity);
	half3 hDistortion = GetDistortionValue(normalize( xIn.m_xNormal ), xScreenPos, fDistortionIntensity );
	
	if (bUseNoise)
	{
		float fNoise = CalculateNoise(xIn.m_xObjectSpacePosition.xyz, xIn.m_xObjectSpacePosition.w);
		half4 h4Distortion = float4(hDistortion, sign(fNoise) * 0.5f + 0.5f);
		half4 h4Lightning = float4(g_fLightningColour.rgb, 1.0f);
		float fLightningAmount = pow(1.0f - min(abs(fNoise) * g_fParams.y, 1.0f), 5);

		hScene = lerp(h4Distortion, h4Lightning, fLightningAmount * (1.0f - pow(1.0f - fDistortionIntensity, 3.0f)));
		hScene.a *= g_fLightningColour.w;
	} else {
		hScene = float4(hDistortion, g_fLightningColour.w);
	}

	return hScene;
}

/******************************************************************************/
/* Techniques                                                                 */
/******************************************************************************/

// Perform material response using the SH
technique Deferred_MR
{
	pass
	{
		VertexShader = compile vs_3_0 VShader_Deferred();
		PixelShader = compile ps_3_0 PShader_Deferred( true );
	}
}

// Forward renderer, for transparent strips
technique Forward
{
	pass
	{
		VertexShader = compile vs_3_0 VShader_Forward();
		PixelShader = compile ps_3_0 PShader_Forward(true);
	}
}

// Forward renderer, Assume fully transparent (no transition lightning)
technique Forward_Cloaked
{
	pass
	{
		VertexShader = compile vs_3_0 VShader_Forward();
		PixelShader = compile ps_3_0 PShader_Forward(false);
	}
}
