
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_PC_Alien_ControlMapper.cpp
// Author : Andy Tate
// Date   : 27-Sep-2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "Actor/Vertebrates/Alien/AVP_PC_Alien_ControlMapper.h"

// Asura platform header-files...
#include "Input/Asura_PlatformInput.h"
#include "Input/Asura_PC_Input_Joystick.h"

// Game independent header-files...
#include "Actor/Vertebrates/Alien/AVP_Alien_InputHandler.h"


////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Private functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Protected functions...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_PC_Alien_ControlMapper::InitialiseControlsPC
// Author       : Andy Tate
// Date         : 27-Sep-2007
////////////////////////////////////////////////////////////////////////////////

void AVP_PC_Alien_ControlMapper::InitialiseControlsPC(AVP_Actor_InputHandler& xInputHandler)
{
	AVP_Alien_InputHandler&	xAlienInputHandler	= *(static_cast<AVP_Alien_InputHandler*>(&xInputHandler));

	Asura_ControlMap&		xFirst				= xAlienInputHandler.GetMapFirstPerson();

	PARENT::InitialiseControlsPC(xInputHandler);

	xAlienInputHandler.MapMovementKeys(xFirst, 'w', 's', 'a', 'd', ASURA_KEYID_LEFTCTRL, ASURA_KEYID_LEFTSHIFT, 'c');
	xAlienInputHandler.MapTailAttack(xFirst, 'q');
	xAlienInputHandler.MapPetFacehuggerControl(xFirst, 'f');
	xAlienInputHandler.MapCreateDistraction(xFirst, 'v');
	xAlienInputHandler.MapReload(xFirst, ASURA_KEYID_VOID);
	xAlienInputHandler.MapHealth(xFirst, 'h');
	xAlienInputHandler.MapTransition(xFirst, ASURA_KEYID_LEFTCTRL);
}

/// End of function 'AVP_PC_Alien_ControlMapper::InitialiseControlsPC'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_PC_Alien_ControlMapper::InitialiseControlsPS2
// Author       : Andy Tate
// Date         : 27-Sep-2007
////////////////////////////////////////////////////////////////////////////////

void AVP_PC_Alien_ControlMapper::InitialiseControlsPS2(AVP_Actor_InputHandler& xInputHandler)
{
	AVP_Alien_InputHandler&	xAlienInputHandler	= *(static_cast<AVP_Alien_InputHandler*>(&xInputHandler));

	Asura_ControlMap&		xFirst				= xAlienInputHandler.GetMapFirstPerson();

	PARENT::InitialiseControlsPS2(xInputHandler);

	xAlienInputHandler.MapMovementAxes(xFirst, ASURA_PC_AXISID_PS2PAD1_LSTICK_V, ASURA_PC_AXISID_PS2PAD1_LSTICK_H, ASURA_KEYID_VOID, ASURA_PC_KEYID_PS2PAD1_L3, ASURA_PC_KEYID_PS2PAD1_TRIANGLE);
	xAlienInputHandler.MapTailAttack(xFirst, ASURA_PC_KEYID_PS2PAD1_CIRCLE);
	xAlienInputHandler.MapInteractAndPetFacehuggerControlToSameKey(xFirst, ASURA_PC_KEYID_PS2PAD1_SQUARE );
	xAlienInputHandler.MapCreateDistraction(xFirst, ASURA_PC_KEYID_PS2PAD1_R3);
	xAlienInputHandler.MapReload(xFirst, ASURA_KEYID_VOID);
	xAlienInputHandler.MapHealth(xFirst, ASURA_KEYID_VOID);
	xAlienInputHandler.MapTransition(xFirst, ASURA_PC_KEYID_PS2PAD1_L1);

}

/// End of function 'AVP_PC_Alien_ControlMapper::InitialiseControlsPS2'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_PC_Alien_ControlMapper::InitialiseControlsPS3
// Author       : Andy Tate
// Date         : 27-Sep-2007
////////////////////////////////////////////////////////////////////////////////

void AVP_PC_Alien_ControlMapper::InitialiseControlsPS3(AVP_Actor_InputHandler& xInputHandler)
{
	AVP_Alien_InputHandler&	xAlienInputHandler	= *(static_cast<AVP_Alien_InputHandler*>(&xInputHandler));

	Asura_ControlMap&		xFirst				= xAlienInputHandler.GetMapFirstPerson();

	PARENT::InitialiseControlsPS3(xInputHandler);

	xAlienInputHandler.MapMovementAxes(xFirst, ASURA_PC_AXISID_PS3PAD1_LSTICK_V, ASURA_PC_AXISID_PS3PAD1_LSTICK_H, ASURA_KEYID_VOID, ASURA_PC_KEYID_PS3PAD1_L3, ASURA_PC_KEYID_PS3PAD1_TRIANGLE);
	xAlienInputHandler.MapTailAttack(xFirst, ASURA_PC_KEYID_PS3PAD1_CIRCLE);
	xAlienInputHandler.MapInteractAndPetFacehuggerControlToSameKey(xFirst, ASURA_PC_KEYID_PS3PAD1_SQUARE);
	xAlienInputHandler.MapCreateDistraction(xFirst, ASURA_PC_KEYID_PS3PAD1_R3);
	xAlienInputHandler.MapReload(xFirst, ASURA_KEYID_VOID);
	xAlienInputHandler.MapHealth(xFirst, ASURA_KEYID_VOID);
	xAlienInputHandler.MapTransition(xFirst, ASURA_PC_KEYID_PS3PAD1_L1);

}

/// End of function 'AVP_PC_Alien_ControlMapper::InitialiseControlsPS3'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_PC_Alien_ControlMapper::InitialiseControlsXBox360
// Author       : Andy Tate
// Date         : 27-Sep-2007
////////////////////////////////////////////////////////////////////////////////

void AVP_PC_Alien_ControlMapper::InitialiseControlsXBox360(AVP_Actor_InputHandler& xInputHandler)
{
	AVP_Alien_InputHandler&	xAlienInputHandler	= *(static_cast<AVP_Alien_InputHandler*>(&xInputHandler));

	Asura_ControlMap&		xFirst				= xAlienInputHandler.GetMapFirstPerson();

	PARENT::InitialiseControlsXBox360(xInputHandler);

	xAlienInputHandler.MapMovementAxes(xFirst, ASURA_PC_AXISID_XBOX360PAD1_LSTICK_V, ASURA_PC_AXISID_XBOX360PAD1_LSTICK_H, ASURA_KEYID_VOID, ASURA_PC_KEYID_XBOX360PAD1_L_THUMB, ASURA_PC_KEYID_XBOX360PAD1_Y);
	xAlienInputHandler.MapTailAttack(xFirst, ASURA_PC_KEYID_XBOX360PAD1_B);
	xAlienInputHandler.MapInteractAndPetFacehuggerControlToSameKey(xFirst, ASURA_PC_KEYID_XBOX360PAD1_X);
	xAlienInputHandler.MapCreateDistraction(xFirst, ASURA_PC_KEYID_XBOX360PAD1_R_THUMB);
	xAlienInputHandler.MapReload(xFirst, ASURA_KEYID_VOID);
	xAlienInputHandler.MapHealth(xFirst, ASURA_KEYID_VOID);
	xAlienInputHandler.MapTransition(xFirst, ASURA_PC_KEYID_XBOX360PAD1_L_BUMPER);

}

/// End of function 'AVP_PC_Alien_ControlMapper::InitialiseControlsXBox360'. ///



////////////////////////////////////////////////////////////////////////////////
// Public functions...
////////////////////////////////////////////////////////////////////////////////


/// End of file. ///
