
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_PC_Predator_ControlMapper.cpp
// Author : Andy Tate
// Date   : 27-Sep-2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "AVP_PC_Predator_ControlMapper.h"

// Asura platform header-files...
#include "Input/Asura_PlatformInput.h"
#include "Input/Asura_PC_Input_Joystick.h"

// Game independent header-files...
#include "Actor/Vertebrates/Predator/AVP_Predator_InputHandler.h"


////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Private functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Protected functions...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_PC_Predator_ControlMapper::InitialiseControlsPC
// Author       : Andy Tate
// Date         : 27-Sep-2007
////////////////////////////////////////////////////////////////////////////////

void AVP_PC_Predator_ControlMapper::InitialiseControlsPC(AVP_Actor_InputHandler& xInputHandler)
{
	AVP_Predator_InputHandler&	xPredatorInputHandler	= *(static_cast<AVP_Predator_InputHandler*>(&xInputHandler));

	Asura_ControlMap& 			xFirst 					= xPredatorInputHandler.GetMapFirstPerson();
	Asura_ControlMap&			xVisionSelector			= xPredatorInputHandler.GetMapVisionSelector();

	PARENT::InitialiseControlsPC(xInputHandler);

	// first person mappings
	xPredatorInputHandler.MapMovementKeys(xFirst, 'w', 's', 'a', 'd', ASURA_KEYID_LEFTCTRL, ASURA_KEYID_LEFTSHIFT, 'c');
	xPredatorInputHandler.MapVision(xFirst, ASURA_KEYID_VOID, 'q', ASURA_KEYID_VOID);
	xPredatorInputHandler.MapCloak(xFirst, 'f');
	xPredatorInputHandler.MapRecon( xFirst, 'e' );
	xPredatorInputHandler.MapZoom(xFirst, ASURA_KEYID_MOUSEWHEELUP, ASURA_KEYID_MOUSEWHEELDOWN, ASURA_KEYID_VOID);

	xPredatorInputHandler.MapLookMouse(xVisionSelector);
	xPredatorInputHandler.MapMenuToggle(xVisionSelector, ASURA_KEYID_ESCAPE);
	xPredatorInputHandler.MapObjectivesAndSelectToggle(xVisionSelector, ASURA_KEYID_BACKSPACE, ASURA_KEYID_TAB);
	xPredatorInputHandler.MapFreezeTime(xVisionSelector, 'p');
	xPredatorInputHandler.MapMovementKeys(xVisionSelector, 'w', 's', 'a', 'd', ASURA_KEYID_LEFTCTRL, ASURA_KEYID_LEFTSHIFT, 'c');

	// weapon and vision mode select mappings
	xPredatorInputHandler.MapVisionSelect( xVisionSelector, ASURA_KEYID_UP, ASURA_KEYID_DOWN, ASURA_KEYID_LEFT, ASURA_KEYID_RIGHT );

	xPredatorInputHandler.MapWeaponSelect( xFirst, '1', '3', '4', '2');
	xPredatorInputHandler.MapReload( xFirst, 'r' );
	xPredatorInputHandler.MapHealth(xFirst, 'h');

}

/// End of function 'AVP_PC_Predator_ControlMapper::InitialiseControlsPC'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_PC_Predator_ControlMapper::InitialiseControlsPS2
// Author       : Andy Tate
// Date         : 27-Sep-2007
////////////////////////////////////////////////////////////////////////////////

void AVP_PC_Predator_ControlMapper::InitialiseControlsPS2(AVP_Actor_InputHandler& xInputHandler)
{
	AVP_Predator_InputHandler&	xPredatorInputHandler	= *(static_cast<AVP_Predator_InputHandler*>(&xInputHandler));

	Asura_ControlMap& 			xFirst 					= xPredatorInputHandler.GetMapFirstPerson();
	Asura_ControlMap&			xVisionSelector			= xPredatorInputHandler.GetMapVisionSelector();

	PARENT::InitialiseControlsPS2(xInputHandler);

	// first person mappings
	xPredatorInputHandler.MapMovementAxes(xFirst, ASURA_PC_AXISID_PS2PAD1_LSTICK_V, ASURA_PC_AXISID_PS2PAD1_LSTICK_H, ASURA_KEYID_VOID, ASURA_PC_KEYID_PS2PAD1_L3, ASURA_KEYID_VOID);
	xPredatorInputHandler.MapVision(xFirst, ASURA_PC_KEYID_PS2PAD1_L2, ASURA_PC_KEYID_PS2PAD1_L2, ASURA_KEYID_VOID);
	xPredatorInputHandler.MapCloak(xFirst, ASURA_PC_KEYID_PS2PAD1_CIRCLE);
	xPredatorInputHandler.MapRecon( xFirst, ASURA_PC_KEYID_PS2PAD1_SQUARE );
	xPredatorInputHandler.MapZoom(xFirst, ASURA_KEYID_VOID, ASURA_KEYID_VOID, ASURA_PC_KEYID_PS2PAD1_R3);

	xPredatorInputHandler.MapLookAxes(xVisionSelector, ASURA_PC_AXISID_PS2PAD1_RSTICK_H, ASURA_PC_AXISID_PS2PAD1_RSTICK_V);
	xPredatorInputHandler.MapMenuToggle(xVisionSelector, ASURA_PC_KEYID_PS2PAD1_START);
	xPredatorInputHandler.MapObjectivesAndSelectToggle(xVisionSelector, ASURA_PC_KEYID_PS2PAD1_SELECT, ASURA_PC_KEYID_PS2PAD1_SELECT);
	xPredatorInputHandler.MapMovementAxes(xVisionSelector, ASURA_PC_AXISID_PS2PAD1_LSTICK_V, ASURA_PC_AXISID_PS2PAD1_LSTICK_H, ASURA_KEYID_VOID, ASURA_PC_KEYID_PS2PAD1_L3, ASURA_KEYID_VOID);

	// weapon and vision mode select mappings
	xPredatorInputHandler.MapVisionSelect(xVisionSelector, ASURA_PC_KEYID_PS2PAD1_TRIANGLE, ASURA_PC_KEYID_PS2PAD1_CROSS, ASURA_PC_KEYID_PS2PAD1_SQUARE, ASURA_PC_KEYID_PS2PAD1_CIRCLE );

	xPredatorInputHandler.MapWeaponSelect(xFirst, ASURA_PC_KEYID_PS2PAD1_UP, ASURA_PC_KEYID_PS2PAD1_DOWN, ASURA_PC_KEYID_PS2PAD1_LEFT, ASURA_PC_KEYID_PS2PAD1_RIGHT);
	xPredatorInputHandler.MapInteractAndReloadToSameKey( xFirst, ASURA_PC_KEYID_PS2PAD1_SQUARE);
	xPredatorInputHandler.MapHealth(xFirst, ASURA_PC_KEYID_PS2PAD1_TRIANGLE);
}

/// End of function 'AVP_PC_Predator_ControlMapper::InitialiseControlsPS2'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_PC_Predator_ControlMapper::InitialiseControlsPS3
// Author       : Andy Tate
// Date         : 27-Sep-2007
////////////////////////////////////////////////////////////////////////////////

void AVP_PC_Predator_ControlMapper::InitialiseControlsPS3(AVP_Actor_InputHandler& xInputHandler)
{
	AVP_Predator_InputHandler&	xPredatorInputHandler	= *(static_cast<AVP_Predator_InputHandler*>(&xInputHandler));

	Asura_ControlMap& 			xFirst 					= xPredatorInputHandler.GetMapFirstPerson();
	Asura_ControlMap&			xVisionSelector			= xPredatorInputHandler.GetMapVisionSelector();

	PARENT::InitialiseControlsPS3(xInputHandler);

	// first person mappings
	xPredatorInputHandler.MapMovementAxes(xFirst, ASURA_PC_AXISID_PS3PAD1_LSTICK_V, ASURA_PC_AXISID_PS3PAD1_LSTICK_H, ASURA_KEYID_VOID, ASURA_PC_KEYID_PS3PAD1_L3, ASURA_KEYID_VOID);
	xPredatorInputHandler.MapVision(xFirst, ASURA_PC_KEYID_PS3PAD1_L2, ASURA_PC_KEYID_PS3PAD1_L2, ASURA_KEYID_VOID);
	xPredatorInputHandler.MapCloak(xFirst, ASURA_PC_KEYID_PS3PAD1_CIRCLE);
	xPredatorInputHandler.MapRecon( xFirst, ASURA_PC_KEYID_PS3PAD1_SQUARE );
	xPredatorInputHandler.MapZoom(xFirst, ASURA_KEYID_VOID, ASURA_KEYID_VOID, ASURA_PC_KEYID_PS3PAD1_R3);

	xPredatorInputHandler.MapLookAxes(xVisionSelector, ASURA_PC_AXISID_PS3PAD1_RSTICK_H, ASURA_PC_AXISID_PS3PAD1_RSTICK_V);
	xPredatorInputHandler.MapMenuToggle(xVisionSelector, ASURA_PC_KEYID_PS3PAD1_START);
	xPredatorInputHandler.MapObjectivesAndSelectToggle(xVisionSelector, ASURA_PC_KEYID_PS3PAD1_SELECT, ASURA_PC_KEYID_PS3PAD1_SELECT);
	xPredatorInputHandler.MapMovementAxes(xVisionSelector, ASURA_PC_AXISID_PS3PAD1_LSTICK_V, ASURA_PC_AXISID_PS3PAD1_LSTICK_H, ASURA_KEYID_VOID, ASURA_PC_KEYID_PS3PAD1_L3, ASURA_KEYID_VOID);

	// weapon and vision mode select mappings
	xPredatorInputHandler.MapVisionSelect(xVisionSelector, ASURA_PC_KEYID_PS3PAD1_TRIANGLE, ASURA_PC_KEYID_PS3PAD1_CROSS, ASURA_PC_KEYID_PS3PAD1_SQUARE, ASURA_PC_KEYID_PS3PAD1_CIRCLE );

	xPredatorInputHandler.MapWeaponSelect(xFirst, ASURA_PC_KEYID_PS3PAD1_UP, ASURA_PC_KEYID_PS3PAD1_DOWN, ASURA_PC_KEYID_PS3PAD1_LEFT, ASURA_PC_KEYID_PS3PAD1_RIGHT);
	xPredatorInputHandler.MapInteractAndReloadToSameKey( xFirst, ASURA_PC_KEYID_PS3PAD1_SQUARE);
	xPredatorInputHandler.MapHealth(xFirst, ASURA_PC_KEYID_PS3PAD1_TRIANGLE);
}

/// End of function 'AVP_PC_Predator_ControlMapper::InitialiseControlsPS3'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_PC_Predator_ControlMapper::InitialiseControlsXBox360
// Author       : Andy Tate
// Date         : 27-Sep-2007
////////////////////////////////////////////////////////////////////////////////

void AVP_PC_Predator_ControlMapper::InitialiseControlsXBox360(AVP_Actor_InputHandler& xInputHandler)
{
	AVP_Predator_InputHandler&	xPredatorInputHandler	= *(static_cast<AVP_Predator_InputHandler*>(&xInputHandler));

	Asura_ControlMap& 			xFirst 					= xPredatorInputHandler.GetMapFirstPerson();
	Asura_ControlMap&			xVisionSelector			= xPredatorInputHandler.GetMapVisionSelector();

	PARENT::InitialiseControlsXBox360(xInputHandler);

	// first person mappings
	xPredatorInputHandler.MapMovementAxes(xFirst, ASURA_PC_AXISID_XBOX360PAD1_LSTICK_V, ASURA_PC_AXISID_XBOX360PAD1_LSTICK_H, ASURA_KEYID_VOID, ASURA_PC_KEYID_XBOX360PAD1_L_THUMB, ASURA_KEYID_VOID);
	xPredatorInputHandler.MapVision(xFirst, ASURA_PC_KEYID_XBOX360PAD1_L_BUMPER, ASURA_PC_KEYID_XBOX360PAD1_L_BUMPER, ASURA_KEYID_VOID);
	xPredatorInputHandler.MapCloak(xFirst, ASURA_PC_KEYID_XBOX360PAD1_B);
	xPredatorInputHandler.MapRecon(xFirst, ASURA_PC_KEYID_XBOX360PAD1_X);
	xPredatorInputHandler.MapZoom(xFirst, ASURA_KEYID_VOID, ASURA_KEYID_VOID, ASURA_PC_KEYID_XBOX360PAD1_R_THUMB);

	xPredatorInputHandler.MapLookAxes(xVisionSelector, ASURA_PC_AXISID_XBOX360PAD1_RSTICK_H, ASURA_PC_AXISID_XBOX360PAD1_RSTICK_V);
	xPredatorInputHandler.MapMenuToggle(xVisionSelector, ASURA_PC_KEYID_XBOX360PAD1_START);
	xPredatorInputHandler.MapObjectivesAndSelectToggle(xVisionSelector, ASURA_PC_KEYID_XBOX360PAD1_BACK, ASURA_PC_KEYID_XBOX360PAD1_BACK);
	xPredatorInputHandler.MapMovementAxes(xVisionSelector, ASURA_PC_AXISID_XBOX360PAD1_LSTICK_V, ASURA_PC_AXISID_XBOX360PAD1_LSTICK_H, ASURA_KEYID_VOID, ASURA_PC_KEYID_XBOX360PAD1_L_THUMB, ASURA_KEYID_VOID);

	// weapon and vision mode select mappings
	xPredatorInputHandler.MapVisionSelect(xVisionSelector, ASURA_PC_KEYID_XBOX360PAD1_Y, ASURA_PC_KEYID_XBOX360PAD1_A, ASURA_PC_KEYID_XBOX360PAD1_X, ASURA_PC_KEYID_XBOX360PAD1_B );

	xPredatorInputHandler.MapWeaponSelect(xFirst, ASURA_PC_KEYID_XBOX360PAD1_UP, ASURA_PC_KEYID_XBOX360PAD1_DOWN, ASURA_PC_KEYID_XBOX360PAD1_LEFT, ASURA_PC_KEYID_XBOX360PAD1_RIGHT);
	xPredatorInputHandler.MapInteractAndReloadToSameKey( xFirst, ASURA_PC_KEYID_XBOX360PAD1_X);
	xPredatorInputHandler.MapHealth(xFirst, ASURA_PC_KEYID_XBOX360PAD1_Y);
}

/// End of function 'AVP_PC_Predator_ControlMapper::InitialiseControlsXBox360'. ///



////////////////////////////////////////////////////////////////////////////////
// Public functions...
////////////////////////////////////////////////////////////////////////////////


/// End of file. ///
