
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_Platform_Core.cpp
// Author : Andy Tate
// Date   : 4-Oct-2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "Core/AVP_Core.h"

// Asura independent header-files...
#if !defined(ASURA_TOOLS)
#include "Camera/Asura_Camera.h"
#include "Core/Asura_Windows.h"
#include "Rendering/Asura_Direct3D.h"
#include "Rendering/Asura_Render.h"
#endif	// !defined(ASURA_TOOLS)

// Asura platform header-files...
#include "Input/Asura_PC_Input_Joystick.h"
#include "Input/Asura_PlatformInput.h"
#ifdef ASURA_PC_SUPPORT_STEAMWORKS
#include "Network/Steamworks/Asura_PC_Steamworks.h"
#endif


////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Private functions...
////////////////////////////////////////////////////////////////////////////////

#if !defined(ASURA_TOOLS)

////////////////////////////////////////////////////////////////////////////////
// Function     : AttemptToChangeResolution
// Author       : Andy Tate
// Date         : 25-Jul-2007
////////////////////////////////////////////////////////////////////////////////

// static bool AttemptToChangeResolution(const u_int uWidth, const u_int uHeight)
// {
// 	if(Asura_Direct3D::IsDisplayModeValid(uWidth, uHeight))
// 	{
// 		return(Asura_Render::SetVideoMode(uWidth, uHeight));
// 	}
// 	else
// 	{
// 		return(false);
// 	}
// }

/// End of function 'AttemptToChangeResolution'. ///

#endif	// !defined(ASURA_TOOLS)



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Core::Platform_Initialise
// Author       : Andy Tate
// Date         : 4-Oct-2006
////////////////////////////////////////////////////////////////////////////////

void AVP_Core::Platform_Initialise (void)
{
	if( Asura_PC_Input_Joystick::GetNumJoysticks() > 0 )
	{
		GUID xGUID;

		Asura_PC_Input_Joystick::GetJoystickGUID( 0, xGUID );
		Asura_PlatformInput::InitJoystickFromGUID( xGUID );
	}

#if !defined(ASURA_TOOLS)
 	if(Asura_Windows::RunningFullScreen())
 	{
// 		if(!AttemptToChangeResolution(1280, 720))
// 		{
// 			if(!AttemptToChangeResolution(1280, 1024))
// 			{
// 				Asura_Render::SetVideoMode(1280, 720);
// 			}
// 		}
 	}
 	else
 	{
		int iWidth = 1280;
		int iHeight = 720;

		// If window position or size has been specified on the command line, these functions will change the parameters
		if(Asura_Windows::GetCommandLineWindowSize(iWidth, iHeight))
		{
			RECT xRect = {0, 0, iWidth, iHeight};
			AdjustWindowRect(&xRect, WS_OVERLAPPEDWINDOW | WS_CAPTION | WS_THICKFRAME, FALSE);
			iWidth = (xRect.right - xRect.left) + 1;
			iHeight = (xRect.bottom - xRect.top) + 1;
		}

		float fAspectRatio = (float)iWidth / (float)iHeight;

	 	Asura_Camera::SetPhysicalScreenAspectRatio(fAspectRatio);
 		Asura_Render::SetVideoMode(iWidth, iHeight);
 	}
#endif	// !defined(ASURA_TOOLS)
}

/// End of function 'AVP_Core::Platform_Initialise'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Core::Platform_Shutdown
// Author       : Andy Tate
// Date         : 4-Oct-2006
////////////////////////////////////////////////////////////////////////////////

void AVP_Core::Platform_Shutdown (void)
{

}

/// End of function 'AVP_Core::Platform_Shutdown'. ///


////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Core::Platform_Update
// Author       : Neil Young
// Date         : 7-May-2009
////////////////////////////////////////////////////////////////////////////////

void AVP_Core::Platform_Update()
{
#ifdef ASURA_PC_SUPPORT_STEAMWORKS
	Asura_PC_Steamworks::Update();
#endif // #ifdef ASURA_PC_SUPPORT_STEAMWORKS
}

/// End of function 'AVP_Core::Platform_Update'. ///

#ifdef AVP_LOG_FILE_LOAD_MEMORY_USE
u_int AVP_Core::Platform_GetAvailableMemory()
{
	return 0; // 0 means we're not bothering to do the logging on this platform.
}
#endif //#ifdef AVP_LOG_FILE_LOAD_MEMORY_USE

void AVP_Core::Platform_BeginHUDRender(void)
{

}


////////////////////////////////////////////////////////////////////////////////
// Protected functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Public functions...
////////////////////////////////////////////////////////////////////////////////


/// End of file. ///
