//////////////////////////////////////////////////////////////////////////////
//
// AVP_Platform_DisplacementMap.cpp
//
// Copyright (c) 2009 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

/******************************************************************************/
/* Included Files                                                             */
/******************************************************************************/

#include "Core/Asura.h"

#if defined ASURA_USE_FSFX

// Asura Platform-independent
#include "FSFX/Asura_FSFX_System.h"
#include "Rendering/Asura_Render.h"
#include "Rendering/Asura_ViewPort.h"
#include "Rendering/Asura_Overlay_Quad.h"


// Asura PC
#include "Rendering/Asura_Direct3D.h"
#include "Rendering/Asura_Direct3D_BackBufferCapture.h"
#include "Rendering/Asura_Direct3D_Effects_System.h"

// AVP Platform-independent
#include "FSFX/AVP_FSFX.h"

// Misc
#include <d3d9.h>

#include "AVP_FSFX_DisplacementMap.h"
ASURA_DECLARE_DIRECT3D_EFFECT(AVP_FSFX_DisplacementMap);

static Asura_Direct3D_Effect*		g_pxDisplacementMapEffect					= NULL;
static Direct3D_VertexDeclaration*	g_pxDisplacementMapVertexDecl				= NULL;
static D3DXHANDLE					g_pxDisplacementMapInitialImageHandle		= 0;
static D3DXHANDLE					g_pxDisplacementMapIntensityHandle			= 0;
static D3DXHANDLE					g_pxDisplacementMapDisplacementImageHandle	= 0;


static void OnEffectUnloaded(Asura_Direct3D_Effect* pxEffect)
{
	g_pxDisplacementMapEffect = NULL;
}

static void OnEffectLoaded(Asura_Direct3D_Effect* pxEffect)
{
#define LOADPARAMETER(codeName, shaderName) { codeName = g_pxDisplacementMapEffect->GetD3DEffect()->GetParameterByName(NULL, shaderName); Asura_Assert(codeName, "Could not get shader parameter '" shaderName "'."); }

	g_pxDisplacementMapEffect = Asura_Direct3D_Effects_System::GetEffect("AVP_FSFX_DisplacementMap");

	if (!g_pxDisplacementMapEffect)
	{
		return;
	}

#if ASURA_LOAD_EFFECT_FILES == 1
	g_pxDisplacementMapEffect->SetOnLoadCallback(OnEffectLoaded);
	g_pxDisplacementMapEffect->SetOnUnloadCallback(OnEffectUnloaded);
#endif

	// Get the params
	LOADPARAMETER(g_pxDisplacementMapInitialImageHandle, "g_xInitialImageTexture" );
	LOADPARAMETER(g_pxDisplacementMapDisplacementImageHandle, "g_xDisplacementMapTexture" );
	LOADPARAMETER(g_pxDisplacementMapIntensityHandle, "g_fIntensity" );	

#undef LOADPARAMETER
}


void AVP_FSFX_Component_DisplacementMap::Platform_Initialise(  )
{
	const Direct3D_VertexElement axVertDecl[] =
	{
		{ 0, 0, D3DDECLTYPE_FLOAT2, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_POSITION, 0 },	// Position
		{ 0, 8, D3DDECLTYPE_FLOAT2, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_TEXCOORD, 0 },	// UVs
		D3DDECL_END()
	};

	Asura_Direct3D::D3DDevice->CreateVertexDeclaration( axVertDecl, &g_pxDisplacementMapVertexDecl );

	OnEffectLoaded(NULL);


}
void AVP_FSFX_Component_DisplacementMap::Platform_Shutdown(  )
{
	if (g_pxDisplacementMapVertexDecl)
	{
		g_pxDisplacementMapVertexDecl->Release();
		g_pxDisplacementMapVertexDecl = NULL;
	}
}



void AVP_FSFX_Component_DisplacementMap::Platform_Render( const float fIntensity )
{
	Asura_Direct3D_Effect* const pxEffect = g_pxDisplacementMapEffect;
	if( !pxEffect || !pxEffect->IsD3DEffectValid() || m_iTextureID == ASURA_TEXTURE_INVALID  || m_iTextureID == ASURA_TEXTURE_NONE) return;

	SCOPEDPIXEVENT(( 0xFFE0D0C0, L"FSFX: AVP - Displacement Map" ));

	// Setup
	Asura_Direct3D_BackBufferCapture::CaptureCurrentBackBuffer();

	pxEffect->SetParameterTexture( g_pxDisplacementMapInitialImageHandle, Asura_Direct3D_BackBufferCapture::GetCaptureTexture() );

	Direct3D_BaseTexture* const pxDisplacementMap	= Asura_Direct3D::GetDirect3DTextureByIndex( m_iTextureID );

	pxEffect->SetParameterTexture( g_pxDisplacementMapDisplacementImageHandle, pxDisplacementMap );
	pxEffect->SetParameter( g_pxDisplacementMapIntensityHandle, fIntensity * m_fAmount * 0.001f);

	Asura_Render::AllowFullScreenRendering();

	Asura_Render::CurrentStates.RequestAlphaTest( false );
	Asura_Render::CurrentStates.RequestCullMode( ASURA_CULLMODE_NONE );
	Asura_Render::CurrentStates.RequestTranslucencyMode( ASURA_TRANSLUCENCY_OFF );
	Asura_Render::CurrentStates.RequestZBufferEnabled( false );

	Direct3D_VertexDeclaration* const pxVD = g_pxDisplacementMapVertexDecl;
	Asura_Direct3D::D3DDevice->SetVertexDeclaration( pxVD );

	const float fWidth = static_cast< float >( Asura_Viewport::GetWidth() );
	const float fHeight = static_cast< float >( Asura_Viewport::GetHeight() );

	const struct Vertex { float fX, fY, fU, fV; } axVerts[] =
	{
		{ -1.0f,  1.0f, 0.5f / fWidth,            0.5f / fHeight },
		{  1.0f,  1.0f, (fWidth + 0.5f) / fWidth, 0.5f / fHeight },
		{  1.0f, -1.0f, (fWidth + 0.5f) / fWidth, (fHeight + 0.5f) / fHeight },
		{ -1.0f, -1.0f, 0.5f / fWidth,            (fHeight + 0.5f) / fHeight },
	};

	Asura_Direct3D_Effects_System::BeginEffect( *pxEffect );
	pxEffect->BeginPass(0);

	Asura_Direct3D::D3DDevice->DrawPrimitiveUP( D3DPT_TRIANGLEFAN, 2, axVerts, sizeof( Vertex ) );

	pxEffect->EndPass();
	Asura_Direct3D_Effects_System::FlushEffects();

	// Reset this parameter to NULL since the texture is in the default pool (for device resets)
	pxEffect->SetParameterTexture( g_pxDisplacementMapInitialImageHandle, NULL );
	Asura_Direct3D_BackBufferCapture::AdviseNextFrame();

	for( u_int u = 0; u < 16; ++u )
	{
		Asura_Direct3D::D3DDevice->SetSamplerState( 0, D3DSAMP_SRGBTEXTURE, FALSE );
	}
}
#endif //defined ASURA_USE_FSFX
