//////////////////////////////////////////////////////////////////////////////
//
// AVP_Platform_FSFX.cpp
//
// Copyright (c) 2007 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

/******************************************************************************/
/* Included Files                                                             */
/******************************************************************************/

#include "Core/Asura.h"

#if defined ASURA_USE_FSFX

// Asura Platform-independent
#include "FSFX/Asura_FSFX_System.h"

// Asura PS3
#include "Rendering/Asura_PS3_Graphics.h"
#include "Rendering/Asura_PS3_RenderingStates.h"
#include "Rendering/Asura_PS3_RSXProfiling.h"
#include "Rendering/Asura_PS3_FrameBuffer.h"
#include "Rendering/AVP_Platform_Stencil_Setup.h"

// AVP Platform-independent
#include "FSFX/AVP_FSFX.h"

/******************************************************************************/
/* Asura_FSFX_System Static Class Functions                                   */
/******************************************************************************/

void Asura_FSFX_System::PlatformProject_Initialise()
{
}

void Asura_FSFX_System::PlatformProject_Reset()
{
}

void Asura_FSFX_System::PlatformProject_Shutdown()
{
}

/******************************************************************************/
/* AVP_FSFX_Component_UseStencil Class Functions                              */
/******************************************************************************/

void AVP_FSFX_Component_UseStencil::Platform_Render( const float fIntensity )
{
	if (m_uBitsToCheck != 0)
	{
		ASURA_PS3_RSX_PROFILE__BEGIN( ASURA_PS3_RSX_PROFILE__MISC, "FSFX stencil" );


		// shift them up incase there are some used bits
		u_int uBitsToCheck = m_uBitsToCheck << AVP_STENCIL_BIT_VISION_FIRST;
		u_int uBitValues = m_uBitValues << AVP_STENCIL_BIT_VISION_FIRST;
		
		Asura_Assert( (uBitsToCheck & ~AVP_STENCIL_MASK_VISION) == 0, "AVP_FSFX_Component_UseStencil::Platform_Render - trying to test bits out of range" );
		Asura_Assert( (uBitValues & ~AVP_STENCIL_MASK_VISION) == 0, "AVP_FSFX_Component_UseStencil::Platform_Render - trying to test bits out of range" );

		// optimise stencil culling for this value.
		Asura_PS3_Graphics::SwitchStencilCullMode( ASURA_PS3_STENCIL_EQUAL, uBitValues, uBitsToCheck );
		// we can't do this^ before we write the value (which was written early on in the scene, and stencil culling has been used by asura for lighting & skybox since then)
		// so we have to load the value we're interested in into the SCULL buffer.
		Asura_PS3_Graphics::ReloadStencilCull();

		Asura_PS3_RenderingStates::SetStencilTestEnabled( true );
		Asura_PS3_RenderingStates::SetStencilMask( AVP_STENCIL_MASK_VISION ); //read/write mask
		Asura_PS3_RenderingStates::SetStencilFunction( ASURA_PS3_STENCIL_EQUAL, uBitValues, uBitsToCheck );
		Asura_PS3_RenderingStates::SetStencilOperation( ASURA_PS3_STENCIL_OPERATION_KEEP, ASURA_PS3_STENCIL_OPERATION_KEEP, ASURA_PS3_STENCIL_OPERATION_KEEP );

		ASURA_PS3_RSX_PROFILE__END( ASURA_PS3_RSX_PROFILE__MISC );

	}
	else
	{
		Asura_PS3_RenderingStates::SetStencilTestEnabled( false );
	}
}

#endif // defined ASURA_USE_FSFX
