//////////////////////////////////////////////////////////////////////////////
//
// AVP_Platform_DevelopmentFrontEnd.cpp
//
// Copyright (c) 2006 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

/******************************************************************************/
/* Included Files                                                             */
/******************************************************************************/

#include "Core/Asura.h"

#include "Input/Asura_ControlMap.h"
#include "FrontEnd/AVP_DevelopmentFrontEnd.h"

// Asura PS3
#include "Input/Asura_PlatformInput.h"
#include "FileAccess/Asura_PS3_FileSystem.h"

#if defined AVP_USE_DEVELOPMENT_FRONT_END

/******************************************************************************/
/* AVP_DevelopmentFrontEnd Static Class Data                              */
/******************************************************************************/

void AVP_DevelopmentFrontEnd::Platform_Initialise()
{
	s_pxControlMap->AddKeyMapping( AVP_DFE_BUTTON_UP,     ASURA_PS3_KEYID_PAD1_UP,     Asura_ControlMapping::ON_DEBOUNCED_PRESS );
	s_pxControlMap->AddKeyMapping( AVP_DFE_BUTTON_DOWN,   ASURA_PS3_KEYID_PAD1_DOWN,   Asura_ControlMapping::ON_DEBOUNCED_PRESS );
	s_pxControlMap->AddKeyMapping( AVP_DFE_BUTTON_LEFT,   ASURA_PS3_KEYID_PAD1_LEFT,   Asura_ControlMapping::ON_DEBOUNCED_PRESS );
	s_pxControlMap->AddKeyMapping( AVP_DFE_BUTTON_RIGHT,  ASURA_PS3_KEYID_PAD1_RIGHT,  Asura_ControlMapping::ON_DEBOUNCED_PRESS );
	s_pxControlMap->AddKeyMapping( AVP_DFE_BUTTON_SELECT, ASURA_PS3_KEYID_PAD1_CROSS,  Asura_ControlMapping::ON_DEBOUNCED_PRESS );
	s_pxControlMap->AddKeyMapping( AVP_DFE_BUTTON_CANCEL, ASURA_PS3_KEYID_PAD1_CIRCLE, Asura_ControlMapping::ON_DEBOUNCED_PRESS );
};

void AVP_DevelopmentFrontEnd::Platform_Shutdown(void)
{
}


bool AVP_DevelopmentFrontEnd::Platform_NeedToBuildFMVList()
{
	return false;
}

void AVP_DevelopmentFrontEnd::Platform_RenderBuildFMVList()
{
}

void AVP_DevelopmentFrontEnd::Platform_StartBuildFMVList()
{
}

bool AVP_DevelopmentFrontEnd::Platform_UpdateBuildFMVList()
{
	return true;
}

void AVP_DevelopmentFrontEnd::Platform_ClearFMVList()
{
}

void AVP_DevelopmentFrontEnd::Platform_PreviewFMV()
{
}

const Asura_UnicodeChar* AVP_DevelopmentFrontEnd::Platform_GetFMVDescription()
{
	return L"Only works on PSP.";
}

static void FindFiles(Asura_List< Asura_String >& xFiles, Asura_Char* szPath)
{
	Asura_Char szCurrentPath[256];
	Asura_Char szFullPath[256];
	int iRet;
	bool bWhileOk = true;
	PS3_DirectoryEntry xContents;	
	int xFileDescriptor;
	//open directory
	Asura_PS3_FileSystem::GenerateFullFilename( szPath, szFullPath, 256 );
	xFileDescriptor = Asura_PS3_FileSystem::OpenDir( szFullPath );
	
	while( bWhileOk )
	{
		iRet = Asura_PS3_FileSystem::ReadDir( xFileDescriptor, &xContents );
		if (iRet < 0)
		{
			Asura_Assert(false , "Error during reading on PSP (ReadDir)");
			
			Asura_PS3_FileSystem::CloseDir(xFileDescriptor);
			bWhileOk = false;
		}		
		else if( iRet == 0 ) //finished reading context of directory
		{
			Asura_PS3_FileSystem::CloseDir(xFileDescriptor);
			bWhileOk = false;
		}
		else
		{
			if( xContents.szName && strcmp( xContents.szName, "." ) && strcmp( xContents.szName, ".." )) // ignore . and ..
			{
				Asura_PrintfToBfr( szCurrentPath, "%s\\%s", 256, true, szPath, xContents.szName );		
				Asura_PS3_FileSystem::GenerateFullFilename( szCurrentPath, szFullPath, 256 );
				
				switch( xContents.ucType )
				{
				case PS3_TOF_File:
					{
						xFiles.AddToList( new Asura_String( szCurrentPath ) );
						break;
					}
				case PS3_TOF_Directory:
					{
						FindFiles( xFiles, szCurrentPath );
						break;
					}
				default:
					{
						break;
					}
				}
			}
		}
	}//while
}

static void GetLevelFilesWExtension(Asura_Char* szDirName, Asura_Char* szExt, Asura_List< Asura_String >& xFiles)
{
	//we create a list of all files in a given directory (including subdirectories)
	Asura_String* pxCurrentString;
	
	FindFiles( xFiles, szDirName);
	//ensure that all string ends with given extension and if it so we cut off the extension 		
	Asura_List_It<Asura_String> xIt(&xFiles);
	while (!xIt.Done())
	{
		pxCurrentString  = xIt.GetCurrent();
		if(!pxCurrentString->EndsWith(szExt))
		{
			delete xIt.GetCurrent();
			xIt.DeleteCurrent();		
		}
		else
		{
			xIt.Next();
		}
	}
}

void AVP_DevelopmentFrontEnd::Platform_GetLevelFiles( Asura_List< Asura_String >& xFiles )
{
	GetLevelFilesWExtension("Envs", ".ps3", xFiles );

#ifdef ASURA_ASSERT
	u_int uEntries = 0;		
	for( Asura_List_It<Asura_String> xIt(&xFiles); !xIt.Done(); xIt.Next() )
	{
		uEntries++;
	}
	Asura_Assert( uEntries, "No levels files found" );
#endif
}

#endif // defined AVP_USE_DEVELOPMENT_FRONT_END
