//
// Asura_PhysicsConfiguration.h
//
// Created by KJFL - Tuesday, December 14, 2004
//
// Copyright (C) 2004 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __ASURA_PHYSICSCONFIGURATION_H__
#define __ASURA_PHYSICSCONFIGURATION_H__


//#define	ASURA_USE_PHYSICS_VARIABLE_TIME_STEP (1)

// Gravity strength, with units of metres per second squared.
// A 'real' value would be in the region of 9.81, but most computer
// games need a value higher than that to 'look right'.
//
static const float fASURA_PHYSICS_GRAVITY = 10.0f;


// This is the total number of active (i.e. moving under Physics) 
// ragdolls allowed simultaneously. If this limit is reached then
// another active ragdoll will be selected and immediately
// destroyed, with the selection process aiming to minimise
// visible impact.
// 
static const u_int uASURA_PHYSICS_ACTIVE_RAGDOLL_LIMIT = 5;


// This upper limit is the absolute maximum number of ragdolls allowed. 
// Calls to create a new client ragdoll entity when there are already
// this number of ragdolls in the world will result in another ragdoll 
// being immediately destroyed to make room. (The system will try to
// choose a ragdoll to destroy which will have the least visible impact;
// see the description for the lower limit below.)
// 
static const u_int uASURA_PHYSICS_UPPER_RAGDOLL_LIMIT = 15;


// This lower ragdoll limit is the number of ragdolls at which old ones
// (or ones off-screen etc) should start to be removed or faded out.
// Having this lower limit allows for ragdolls in front of the player
// to be faded out discreetly in time for new ragdolls to be added,
// rather than suddenly running into the upper limit and having to
// remove a ragdoll in the same frame which might look ugly.
//
// (The lower limit can be the same as the upper limit, in which case
// the lower limit functionality is disabled.)
//
static const u_int uASURA_PHYSICS_LOWER_RAGDOLL_LIMIT = 10;


// Speed at which a ragdoll fades out. Time taken to fade out is 
// 1 over this value. i.e. a value of 2.0f would mean it takes 0.5 secs
// for the ragdoll to fade out.
static const float fASURA_PHYSICS_RAGDOLL_FADEOUT_SPEED = 0.1f;

#endif//__ASURA_PHYSICSCONFIGURATION_H__
