//////////////////////////////////////////////////////////////////////////////
//
// AVP_Platform_Font_System.cpp
//
// Copyright (c) 2007 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

#include "Rendering/AVP_Font_System.h"

#include "Core/Asura_TiledTextureManagement.h"
#include "Core/Asura_TextureManagement.h"
#include "Debugging/Asura_Assert.h"
#include "ChunkSystem/Asura_Chunk_Loaders.h"

#include "Core/AVP_Core.h"

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

Asura_Hash_ID AVP_Buttons::s_auSpecialCharTiles[AVP_FONTSYSTEM_NUM_SPECIAL_CHARS] =
{
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_NA,			0xE800	Asura_GetHashID("ps3_NA.tga")	N/A

	0xBAC39769,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_1,		0xE801	Asura_GetHashID("ps3_x.tga")		CROSS	
	0x7CDEE73F,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_2,		0xE802	Asura_GetHashID("ps3_circle.tga")	CIRCLE
	0x3BAD950C,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_3,		0xE803	Asura_GetHashID("ps3_square.tga")	SQUARE
	0x607E1C1A,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_4,		0xE804	Asura_GetHashID("ps3_triangle.tga")	TRIANGLE
	0x893FD4B4,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_5,		0xE805	Asura_GetHashID("ps3_l1.tga")		L1
	0x937CE86E,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_6,		0xE806	Asura_GetHashID("ps3_r1.tga")		R1
	0x894DEC35,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_7,		0xE807	Asura_GetHashID("ps3_l2.tga")		L2
	0x938AFFEF,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_8,		0xE808	Asura_GetHashID("ps3_r2.tga")		R2
	0x895C03B6,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_9,		0xE809	Asura_GetHashID("ps3_l3.tga")		L3
	0x93991770,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_10,		0xE80A	Asura_GetHashID("ps3_r3.tga")		R3
	0x9FF525D3,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_11,		0xE80B	Asura_GetHashID("ps3_start.tga")		START
	0x5C61696B,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_12,		0xE80C	Asura_GetHashID("ps3_select.tga")		SELECT
	0x60598FDC,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_13,		0xE80D	Asura_GetHashID("ps3_dpad_up.tga")		UP
	0x117E7623,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_14,		0xE80E	Asura_GetHashID("ps3_dpad_down.tga")	DOWN
	0x22DBB708,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_15,		0xE80F	Asura_GetHashID("ps3_dpad_left.tga")	LEFT
	0x75E1587B,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_16,		0xE810	Asura_GetHashID("ps3_dpad_right.tga")	RIGHT

	// CAH	All of these N/A entries are buttons that the PC has but the 360 doesnt,
	// the enum to define all of the buttons if platform independant (and should stay that way for general ease) so they just have to entered and not used. 
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_17,		0xE811	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_18,		0xE812	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_19,		0xE813	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_20,		0xE814	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_21,		0xE815	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_22,		0xE816	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_23,		0xE817	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_24,		0xE818	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_25,		0xE819	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_26,		0xE81A	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_27,		0xE81B	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_28,		0xE81C	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_29,		0xE81D	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_30,		0xE81E	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_31,		0xE81F	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_32,		0xE820	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_33,		0xE821	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_34,		0xE822	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_35,		0xE823	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_36,		0xE824	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_37,		0xE825	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_38,		0xE826	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_39,		0xE827	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_40,		0xE828	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_41,		0xE829	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_42,		0xE82A	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_43,		0xE82B	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_44,		0xE82C	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_45,		0xE82D	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_46,		0xE82E	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_47,		0xE82F	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_48,		0xE830	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_49,		0xE831	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_50,		0xE832	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_51,		0xE833	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_52,		0xE834	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_53,		0xE835	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_54,		0xE836	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_55,		0xE837	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_56,		0xE838	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_57,		0xE839	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_58,		0xE83A	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_59,		0xE83B	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_60,		0xE83C	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_61,		0xE83D	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_62,		0xE83E	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_63,		0xE83F	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_64,		0xE840	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_65,		0xE841	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_66,		0xE842	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_67,		0xE843	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_68,		0xE844	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_69,		0xE845	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_70,		0xE846	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_71,		0xE847	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_72,		0xE848	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_73,		0xE849	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_74,		0xE84A	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_75,		0xE84B	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_76,		0xE84C	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_77,		0xE84D	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_78,		0xE84E	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_79,		0xE84F	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_80,		0xE850	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_81,		0xE851	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_82,		0xE852	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_83,		0xE853	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_84,		0xE854	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_85,		0xE855	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_86,		0xE856	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_87,		0xE857	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_88,		0xE858	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_89,		0xE859	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_90,		0xE85A	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_91,		0xE85B	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_92,		0xE85C	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_93,		0xE85D	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_94,		0xE85E	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_95,		0xE85F	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_96,		0xE860	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_97,		0xE861	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_98,		0xE862	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_99,		0xE863	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_100,	0xE864	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_101,	0xE865	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_102,	0xE866	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_103,	0xE867	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_104,	0xE868	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_105,	0xE869	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_106,	0xE86A	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_107,	0xE86B	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_108,	0xE86C	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_109,	0xE86D	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_110,	0xE86E	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_111,	0xE86F	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_112,	0xE870	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_113,	0xE871	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_114,	0xE872	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_115,	0xE873	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_116,	0xE874	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_117,	0xE875	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_118,	0xE876	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_119,	0xE877	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_120,	0xE878	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_121,	0xE879	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_122,	0xE87A	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_123,	0xE87B	Asura_GetHashID("ps3_NA.tga")	N/A

	0x26135F40,	//AVP_FONTSYSTEM_SPECIALCHAR_AXIS_LSTICK_H,	0xE87C	Asura_GetHashID("ps3_analoge_left.tga")		AXIS LS H
	0x26135F40,	//AVP_FONTSYSTEM_SPECIALCHAR_AXIS_LSTICK_V,	0xE87D	Asura_GetHashID("ps3_analoge_left.tga")		AXIS LS V
	0xD99EB743,	//AVP_FONTSYSTEM_SPECIALCHAR_AXIS_RSTICK_H,	0xE87E	Asura_GetHashID("ps3_analoge_right.tga")	AXIS RS H
	0xD99EB743,	//AVP_FONTSYSTEM_SPECIALCHAR_AXIS_RSTICK_V,	0xE87F	Asura_GetHashID("ps3_analoge_right.tga")	AXIS RS V
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_AXIS_Z,		0xE880	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_AXIS_ROTATE_X,	0xE881	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_AXIS_ROTATE_Y,	0xE882	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_AXIS_ROTATE_Z,	0xE883	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_AXIS_MOUSE_H,	0xE884	Asura_GetHashID("ps3_NA.tga")	N/A
	0x8F4DEE22,	//AVP_FONTSYSTEM_SPECIALCHAR_AXIS_MOUSE_V,	0xE885	Asura_GetHashID("ps3_NA.tga")	N/A
};

void AVP_Buttons::Platform_Initialise()
{
	// if the special char tiles haven't been loaded, attempt to load them here
	if( !Asura_TiledTextureManagement::GetTileInfoByHash( AVP_Buttons::s_auSpecialCharTiles[0] ) )
	{
#if defined(ASURA_ASSERT)
		bool bOK = 
#endif
		AVP_Core::LoadFile( "graphics\\ps3_buttons." ASURA_STANDARD_FILE_EXTENSION, ASURA_TEXTURE_LOADED_PERMANENT );
		Asura_Assert(bOK, "failed to load button icons file");
	}
}

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////
