///////////////////////////////////////////////////////////////////////////////
//
// File: AVP_PS3_ProjectShaders.h
//
// Created by Ben Weston 19/05/08
// 
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//
// Unique IDs for project shaders (within the ASURA_SHADER_HASH_PROJECT shader group)
//
///////////////////////////////////////////////////////////////////////////////

#if defined ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#if !defined __AVP_PROJECTSHADERS_H__
#define __AVP_PROJECTSHADERS_H__

// each shader needs a unique ID, but it doesn't matter what that ID is.
enum {
	ASURA_PS3_CLOAKHOLES_SHADER,				
	ASURA_PS3_CLOAKEFFECT_SHADER,			
	ASURA_PS3_CLOAKEFFECT_SIMPLE_SHADER,		
	ASURA_PS3_LASER_SHADER,					
	ASURA_PS3_LASER_FSFX_SHADER,				
	ASURA_PS3_PHEROMONETRAIL_SHADER,					
	ASURA_PS3_GLOWOUTLINESSKIN_SHADER,					
	ASURA_PS3_GLOWOUTLINESFS_HOR_SHADER,					
	ASURA_PS3_GLOWOUTLINESFS_VER_SHADER,					
	ASURA_PS3_GLOWOUTLINESFS_FIN_SHADER,					
	ASURA_PS3_FSFX_DISPLACEMENTMAP_SHADER,					
	ASURA_PS3_FSFX_EDGES_PASS1_SHADER,					
	ASURA_PS3_FSFX_EDGES_PASS2_SHADER,					
};


class AVP_PS3_PixelShader
{
	Asura_Hash_ID m_uHashFP;
	Asura_PS3_FragmentShaderCopy*	m_pxFragmentShader;
protected:
	void SetFloats( int iElementID, const unsigned char ucElements, const float* pfData)
	{
		Asura_Assert(m_pxFragmentShader != NULL, "Need to Begin a shader before you can set params on it");
		m_pxFragmentShader->SetFloats(iElementID, ucElements, pfData);
	}

	void SetTexture( int iTextureID, int iTextureIndex)
	{
		Asura_Assert(m_pxFragmentShader != NULL, "Need to Begin a shader before you can set params on it");
		m_pxFragmentShader->SetTexture(iTextureID, iTextureIndex);
	}
	void SetTexture( int iTextureID, Asura_PS3_Texture* pxTexture, bool bNormalizedTextureCoordinates = true )
	{
		Asura_Assert(m_pxFragmentShader != NULL, "Need to Begin a shader before you can set params on it");
		m_pxFragmentShader->SetTexture(iTextureID, pxTexture, bNormalizedTextureCoordinates);
	}
#if defined(ASURA_PS3_ALLOW_SHADER_RELOAD)
	void ReloadFromDisc(char *pFileName, Asura_PS3_FP_Handles *pxHandles)
	{
		const char *aszIncludePath[2]= { "~/Project/AVP/Game/PS3/Shaders/", NULL };
		Asura_PS3_GCM_ShadersLibrary::LoadAndCompileShader( pFileName,
			ASURA_SHADER_HASH_PROJECT, m_uHashFP, NULL, aszIncludePath, pxHandles);

	}
#endif

public:
	AVP_PS3_PixelShader(Asura_Hash_ID uHashFP, Asura_PS3_FP_Handles *pxHandles, uint32_t *puBinStart, uint32_t *puBinEnd)
	{
		Asura_PS3_GCM_ShadersLibrary::AddFP( ASURA_SHADER_HASH_PROJECT, m_uHashFP = uHashFP, (char*)puBinStart, (char*)puBinEnd- (char*)puBinStart, pxHandles, 1, true );
		m_pxFragmentShader = NULL;
	}
	void Begin()
	{
		Asura_Assert(m_pxFragmentShader == NULL, "Begin without End in AVP_PS3_PixelShader");
		m_pxFragmentShader = Asura_PS3_GCM_ShadersLibrary::GetPermanentFP( ASURA_SHADER_HASH_PROJECT, m_uHashFP);
	}
	void Bind()
	{
		m_pxFragmentShader->Bind();
		m_pxFragmentShader = NULL;
	}
	void End()
	{
	}


};

#endif //#if !defined __AVP_PROJECTSHADERS_H__
