// Cloak effect - shared functions

/******************************************************************************/
/* Parameters                                                                 */
/******************************************************************************/

static const float2 g_xDistortionScales		= { 64.0f, -64.0f };

const float4 g_xOffset;

const float2 g_xParams = float2(3.61f, 0.06f);
// x = Scale
// y = LightningScale

const float4 g_xLightningColour;
// w = Threshold

const float2 g_xEffectCenter = {640.0f,360.0f}; //screen position about which to distort, e.g. center of character
const half g_iSteps = 5; //in the film this varies according to how close the predator is to the camera (more steps in close-up)


const sampler g_xSceneSampler				: register( s0 );
const sampler g_xNoiseTextureSampler		: register( s1 );

/******************************************************************************/
/* Types                                                                      */
/******************************************************************************/

struct OutputVertex
{
	float4 m_xPosition              : POSITION;
	float4 m_xNormal                : TEXCOORD0;
	float4 m_xObjectSpacePosition	: TEXCOORD1;
};

/******************************************************************************/
/* Functions                                                                  */
/******************************************************************************/

float NoiseLookup(float3 xPos)
{
	float fNoise;

	fNoise  = tex2D(g_xNoiseTextureSampler, xPos.xy).r
			+ tex2D(g_xNoiseTextureSampler, xPos.yz).g
			+ tex2D(g_xNoiseTextureSampler, xPos.zx).b;

	return (fNoise/3.0f) * 2.0f - 1.0f;

}


float CalculateNoise(float3 p, float fOffset)
{
	return NoiseLookup(p * g_xParams.x + g_xOffset.xyz) + fOffset;
}

half3 GetDistortionValue(float3 xNormal, half2 xScreenPos : WPOS, float fDistortionIntensity  )
{
	half3 xViewSpaceNormal = xNormal;
  
 	half hDistortionAmount = length( xViewSpaceNormal.xy );
	hDistortionAmount = fmod( hDistortionAmount, 0.2h ) * 5.0h;

	const half2 xDistortion = xViewSpaceNormal.xy * hDistortionAmount;
	xScreenPos += xDistortion * g_xDistortionScales * fDistortionIntensity;
	half4 xSceneSample = h4tex2D( g_xSceneSampler, xScreenPos );
	return xSceneSample.rgb;

//BW alternative effect, more like the film IMO - though this version isn't fully working yet
// 	xViewSpaceNormal.z = max(-xViewSpaceNormal.z,0); //only want the negative part (facing towards screen)
//  	half hDistortionIndex = 1-sqrt(1-xViewSpaceNormal.z);
//  	hDistortionIndex = floor( hDistortionIndex*g_iSteps )/g_iSteps;
//  	half hDistortionAmount = ( 1.6f - 1.0f*pow(hDistortionIndex,0.5f) ); //>1 = scale bigger, <1 = scale smaller
// 	xScreenPos = g_xEffectCenter + ((xScreenPos - g_xEffectCenter) / hDistortionAmount); //by scaling the position up we shrink the image
// 
// 	half4 xSceneSample = h4tex2D( g_xSceneSampler, xScreenPos );
// 	return xSceneSample.rgb+half3(.04*(.3-hDistortionIndex));
}
