///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_Platform_Actor_InputHandler.cpp
// Author : Callum Hope
// Date   : 13-Oct-2008
// Desc	  : AVP/Xbox360/Actor/... Platform dependant functionality and look up functions for data used by the input handler. 
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

// core header
#include "Core/Asura.h"

// This file's header
#include "Actor/AVP_Actor_InputHandler.h"


// Platform dependant functions

void AVP_Actor_InputHandler::Platform_RefreshTurnAndTiltParams( const Asura_Blueprint*	pxBlueprint )
{
	Asura_Value	xValue;

	if( pxBlueprint )
	{
		// look up all of the 360 analoge stick values, as the PC does not have a controller and if it does it is likely to be a USB 360 controller. 
		if(pxBlueprint->GetParameterValueByHashID(0xE3C02A2D, xValue))	//Asura_GetHashID("AnalogeStick360_TiltMaxSpeed")
		{
			s_fTiltMaxSpeedInDegree = xValue.GetFloatValue();
			s_fTiltMaxSpeedInRadian = Asura_Maths::DegToRad(s_fTiltMaxSpeedInDegree);
		}
		if(pxBlueprint->GetParameterValueByHashID(0xAD9818C4, xValue))	//Asura_GetHashID("AnalogeStick360_TiltRampUpTime")
		{
			s_fTiltRampUpTime = xValue.GetFloatValue();
		}
		if(pxBlueprint->GetParameterValueByHashID(0x92830C4B, xValue))	//Asura_GetHashID("AnalogeStick360_TiltRampDownTime")
		{
			s_fTiltRampDownTime = xValue.GetFloatValue();
		}
		if(pxBlueprint->GetParameterValueByHashID(0x57860B79, xValue))	//Asura_GetHashID("AnalogeStick360_TiltExponent")
		{
			s_fTiltExponent = xValue.GetFloatValue();
		}

		if(pxBlueprint->GetParameterValueByHashID(0x449610DA, xValue))	//Asura_GetHashID("AnalogeStick360_TurnDefaultMaxSpeed")
		{
			s_fTurnDefaultMaxSpeedInDegree = xValue.GetFloatValue();
			s_fTurnDefaultMaxSpeedInRadian = Asura_Maths::DegToRad(s_fTurnDefaultMaxSpeedInDegree);
		}
		if(pxBlueprint->GetParameterValueByHashID(0x386BB137, xValue))	//Asura_GetHashID("AnalogeStick360_TurnTurboMaxSpeed")
		{
			s_fTurnTurboMaxSpeedInDegree = xValue.GetFloatValue();
			s_fTurnTurboMaxSpeedInRadian = Asura_Maths::DegToRad(s_fTurnTurboMaxSpeedInDegree);
		}
		if(pxBlueprint->GetParameterValueByHashID(0x2257522F, xValue))	//Asura_GetHashID("AnalogeStick360_TurnTurboDelay")
		{
			s_fTurnTurboDelay = xValue.GetFloatValue();
		}
		if(pxBlueprint->GetParameterValueByHashID(0x857E054E, xValue))	//Asura_GetHashID("AnalogeStick360_TurnTurboRampUpTime")
		{
			s_fTurnTurboRampUpTime = xValue.GetFloatValue();
		}
		if(pxBlueprint->GetParameterValueByHashID(0x8C8A1684, xValue))	//Asura_GetHashID("AnalogeStick360_TurnRampUpTime")
		{
			s_fTurnRampUpTime = xValue.GetFloatValue();
		}
		if(pxBlueprint->GetParameterValueByHashID(0x7CEC9A0B, xValue))	//Asura_GetHashID("AnalogeStick360_TurnRampDownTime")
		{
			s_fTurnRampDownTime = xValue.GetFloatValue();
		}
		if(pxBlueprint->GetParameterValueByHashID(0x925C7939, xValue))	//Asura_GetHashID("AnalogeStick360_TurnExponent")
		{
			s_fTurnExponent = xValue.GetFloatValue();
		}

	}
	else
	{
		s_fTiltMaxSpeedInDegree = 90.0f;
		s_fTiltMaxSpeedInRadian = Asura_Maths::DegToRad(s_fTiltMaxSpeedInDegree);
		s_fTiltRampUpTime = 0.25f;
		s_fTiltRampDownTime = 0.05f;
		s_fTiltExponent = 2.0f;
		
		s_fTurnDefaultMaxSpeedInDegree = 180.0f;
		s_fTurnDefaultMaxSpeedInRadian = Asura_Maths::DegToRad(s_fTurnDefaultMaxSpeedInDegree);
		s_fTurnTurboMaxSpeedInDegree = 180.0f;
		s_fTurnTurboMaxSpeedInRadian = Asura_Maths::DegToRad(s_fTurnTurboMaxSpeedInDegree);
		s_fTurnTurboDelay = 1.0f;
		s_fTurnTurboRampUpTime = 0.5f;
		s_fTurnRampUpTime = 0.5f;
		s_fTurnRampDownTime = 0.10f;
		s_fTurnExponent = 2.0f;
	}
}

/// End of file. ///
