//////////////////////////////////////////////////////////////////////////////
//
// AVP_Platform_DisplacementMap.cpp
//
// Copyright (c) 2009 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

/******************************************************************************/
/* Included Files                                                             */
/******************************************************************************/

#include "Core/Asura.h"

#if defined ASURA_USE_FSFX

// Asura Platform-independent
#include "FSFX/Asura_FSFX_System.h"
#include "Rendering/Asura_Render.h"
#include "Rendering/Asura_ViewPort.h"
#include "Rendering/Asura_Overlay_Quad.h"

// Asura Xbox360
// 
// AVP Platform-independent
#include "FSFX/AVP_FSFX.h"

// Asura Xbox360
#include "Rendering/Asura_Xbox360_Graphics.h"

// Shader Include Files
#include "AVP_FSFX_DisplacementMap.h"


 
static Asura_Xbox360_Effect*					g_pxDisplacementMapEffect					= NULL;
static Direct3D_VertexDeclaration*				g_pxDisplacementMapVertexDecl				= NULL;
static Asura_Xbox360_Effect_System::Handle		g_pxDisplacementMapInitialImageHandle		= 0;
static Asura_Xbox360_Effect_System::Handle		g_pxDisplacementMapIntensityHandle			= 0;
static Asura_Xbox360_Effect_System::Handle		g_pxDisplacementMapDisplacementImageHandle	= 0;


static void OnEffectUnloaded(Asura_Xbox360_Effect* pxEffect)
{
	g_pxDisplacementMapEffect = NULL;
}
 
static void OnEffectLoaded(Asura_Xbox360_Effect* pxEffect)
{
#define LOADPARAMETER(codeName, shaderName) { codeName = g_pxDisplacementMapEffect->GetParameterHandle(shaderName); Asura_Assert(codeName, "Could not get shader parameter '" shaderName "'."); }
 
	g_pxDisplacementMapEffect = Asura_Xbox360_Effect_System::CreateEffectFromMemory(g_auAVP_FSFX_DisplacementMap);
 
 	if (!g_pxDisplacementMapEffect)
 	{
 		return;
 	}
 
 	// Get the params
 	LOADPARAMETER(g_pxDisplacementMapInitialImageHandle, "g_xInitialImage" );
 	LOADPARAMETER(g_pxDisplacementMapDisplacementImageHandle, "g_xDisplacementMap" );
 	LOADPARAMETER(g_pxDisplacementMapIntensityHandle, "g_fIntensity" );	
 
#undef LOADPARAMETER
}


void AVP_FSFX_Component_DisplacementMap::Platform_Initialise(  )
{
	const Direct3D_VertexElement axVertDecl[] =
	{
		{ 0, 0, D3DDECLTYPE_FLOAT2, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_POSITION, 0 },	// Position
		{ 0, 8, D3DDECLTYPE_FLOAT2, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_TEXCOORD, 0 },	// UVs
		D3DDECL_END()
	};

	Asura_Xbox360_Graphics::GetDevice()->CreateVertexDeclaration( axVertDecl, &g_pxDisplacementMapVertexDecl );

	OnEffectLoaded(NULL);

	if (g_pxDisplacementMapEffect)
	{
		g_pxDisplacementMapEffect->SetTexture(g_pxDisplacementMapInitialImageHandle, Asura_Xbox360_Graphics::GetPostToneMapTexture());
	}

}
void AVP_FSFX_Component_DisplacementMap::Platform_Shutdown(  )
{
	OnEffectUnloaded(NULL);
	if (g_pxDisplacementMapVertexDecl)
	{
		g_pxDisplacementMapVertexDecl->Release();
		g_pxDisplacementMapVertexDecl = NULL;
	}
}



void AVP_FSFX_Component_DisplacementMap::Platform_Render( const float fIntensity )
{
	Asura_Xbox360_Effect* const pxEffect = g_pxDisplacementMapEffect;
	if( !pxEffect ) return;

	SCOPEDPIXEVENT(( 0xFFE0D0C0, "FSFX: AVP - Displacement Map" ));

	// Setup

	// Resolve the current RT back out so we can read the initial colours
	Asura_Xbox360_Graphics::Resolve( Asura_Xbox360_Graphics::GetPostToneMapTexture() );

	g_pxDisplacementMapEffect->SetTexture( g_pxDisplacementMapDisplacementImageHandle, m_iTextureID );
	pxEffect->SetFloat( g_pxDisplacementMapIntensityHandle, fIntensity * m_fAmount * 0.001f);

	Asura_Render::AllowFullScreenRendering();

	Asura_Render::CurrentStates.RequestAlphaTest( false );
	Asura_Render::CurrentStates.RequestCullMode( ASURA_CULLMODE_NONE );
	Asura_Render::CurrentStates.RequestTranslucencyMode( ASURA_TRANSLUCENCY_OFF );
	Asura_Render::CurrentStates.RequestZBufferEnabled( false );

	Direct3D_VertexDeclaration* const pxVD = g_pxDisplacementMapVertexDecl;
	Asura_Xbox360_Graphics::GetDevice()->SetVertexDeclaration( pxVD );

	const float fWidth = static_cast< float >( Asura_Viewport::GetWidth() );
	const float fHeight = static_cast< float >( Asura_Viewport::GetHeight() );

	const struct Vertex { float fX, fY, fU, fV; } axVerts[] =
	{
		{ -1.0f,  1.0f, 0.5f / fWidth,            0.5f / fHeight },
		{  1.0f,  1.0f, (fWidth + 0.5f) / fWidth, 0.5f / fHeight },
		{  1.0f, -1.0f, (fWidth + 0.5f) / fWidth, (fHeight + 0.5f) / fHeight },
		{ -1.0f, -1.0f, 0.5f / fWidth,            (fHeight + 0.5f) / fHeight },
	};

	pxEffect->Begin(0,0);
	pxEffect->Commit();

	Asura_Xbox360_Graphics::GetDevice()->DrawPrimitiveUP( D3DPT_TRIANGLEFAN, 2, axVerts, sizeof( Vertex ) );

	pxEffect->End();

}
#endif //defined ASURA_USE_FSFX
