
/******************************************************************************/
/* Included Files                                                             */
/******************************************************************************/

#include "..\..\..\..\..\Asura\Xbox360\Shaders\Shared.fxh"

/******************************************************************************/
/* Parameters                                                                 */
/******************************************************************************/

const sampler2D g_xInitialImage = sampler_state
{
	AddressU = CLAMP;
	AddressV = CLAMP;
};


const float2 g_fInvScreenDims = float2(1.0f / 1000, 1.0f / 500.0f);
const float4 g_fTint = float4(135.0f/255.0f, 111.0f/255.0f, 0.0, 7.5f); // a = blur amount
const float3 g_fEdgeDetectionParams = float3(2.0f, 4.0f, 1.0f);

/******************************************************************************/
/* Constants                                                                  */
/******************************************************************************/



/******************************************************************************/
/* Types                                                                      */
/******************************************************************************/

struct Vertex
{
	float4 m_xPosition : POSITION;
	float2 m_xUV       : TEXCOORD;
};

/******************************************************************************/
/* Functions                                                                  */
/******************************************************************************/

Vertex VShader( const Vertex xIn )
{
	return xIn;
}

half4 tap(float2 xCentre, float2 xOffset, float fMult)
{
	return TexFetch2D_LinearLinearNone( g_xInitialImage, xCentre + xOffset * g_fInvScreenDims ) * fMult;
}

float4 PShader_LoG( const Vertex xIn ) : COLOR
{
	const static int NumEDTaps = 9;
	const static float fEDSamples[NumEDTaps][NumEDTaps] =
	{ 
		{  0,  1,  1,  2,  2,  2,  1,  1,  0}, // 10
		{  1,  2,  4,  5,  5,  5,  4,  2,  1}, // 29
		{  1,  4,  5,  3,  0,  3,  5,  4,  1}, // 26
		{  2,  5,  3,-12,-24,-12,  3,  5,  2}, // -28
		{  2,  5,  0,-24,-40,-24,  0,  5,  2}, // -74
		{  2,  5,  3,-12,-24,-12,  3,  5,  2}, // -28
		{  1,  4,  5,  3,  0,  3,  5,  4,  1}, // 26
		{  1,  2,  4,  5,  5,  5,  4,  2,  1}, // 29
		{  0,  1,  1,  2,  2,  2,  1,  1,  0}, // 10
	};

	half4 xSample = 0.0f;
	
	for (int i = 0; i < NumEDTaps; ++i)
	{
		for (int j = 0; j < NumEDTaps; ++j)
		{
			float2 fOffset = ((float2(i, j) / float(NumEDTaps - 1)) - (0.5f).xx) * NumEDTaps * g_fEdgeDetectionParams.z;
			xSample += tap(xIn.m_xUV, fOffset, fEDSamples[i][j]);
		}
	}

	return smoothstep(g_fEdgeDetectionParams.x, g_fEdgeDetectionParams.y, dot(xSample, 0.333f));
}

float4 PShader_L1( const Vertex xIn ) : COLOR
{
	half4 xSample = 0.0f;
	float fR = 0.5f * g_fEdgeDetectionParams.z;

	xSample += tap(xIn.m_xUV, float2(0.0f,  -fR), 1.0f);
	xSample += tap(xIn.m_xUV, float2( -fR, 0.0f), 1.0f) + tap(xIn.m_xUV, float2(0.0f, 0.0f), -4.0f) + tap(xIn.m_xUV, float2(fR, 0.0f), 1.0f);
	xSample += tap(xIn.m_xUV, float2(0.0f,   fR), 1.0f);

	return smoothstep(g_fEdgeDetectionParams.x * 0.1f, g_fEdgeDetectionParams.y * 0.1f, dot(xSample.rgb, 0.333f));
}

float4 PShader_L2( const Vertex xIn ) : COLOR
{
	const static int NumEDTaps = 3;
	const static float fEDSamples[NumEDTaps][NumEDTaps] =
	{ 
		{  1,  1,  1}, // 3
		{  1, -8,  1}, // -6
		{  1,  1,  1}, // 3
	};

	half4 xSample = 0.0f;
	
	for (int i = 0; i < NumEDTaps; ++i)
	{
		for (int j = 0; j < NumEDTaps; ++j)
		{
			float2 fOffset = ((float2(i, j) / float(NumEDTaps - 1)) - (0.5f).xx) * NumEDTaps * g_fEdgeDetectionParams.z;
			xSample += tap(xIn.m_xUV, fOffset, fEDSamples[i][j]);
		}
	}

	return smoothstep(g_fEdgeDetectionParams.x * 0.1f, g_fEdgeDetectionParams.y * 0.1f, dot(xSample.rgb, 0.333f));
}

float4 PShader_L3( const Vertex xIn ) : COLOR
{
	const static int NumEDTaps = 3;
	const static float fEDSamples[NumEDTaps][NumEDTaps] =
	{ 
		{ -1,  2, -1}, // 3
		{  2, -4,  2}, // -6
		{ -1,  2, -1}, // 3
	};

	half4 xSample = 0.0f;
	
	for (int i = 0; i < NumEDTaps; ++i)
	{
		for (int j = 0; j < NumEDTaps; ++j)
		{
			float2 fOffset = ((float2(i, j) / float(NumEDTaps - 1)) - (0.5f).xx) * NumEDTaps * g_fEdgeDetectionParams.z;
			xSample += tap(xIn.m_xUV, fOffset, fEDSamples[i][j]);
		}
	}

	return smoothstep(g_fEdgeDetectionParams.x * 0.1f, g_fEdgeDetectionParams.y * 0.1f, dot(xSample.rgb, 0.333f));
}

float4 PShader_Robert( const Vertex xIn ) : COLOR
{
	half4 xSample1 = 0.0f;
	half4 xSample2 = 0.0f;
	float fR = 0.25f * g_fEdgeDetectionParams.z;

	xSample1 += tap(xIn.m_xUV, float2(-fR, -fR), 1.0f);
	xSample1 += tap(xIn.m_xUV, float2( fR,  fR), -1.0f);

	xSample2 += tap(xIn.m_xUV, float2( fR, -fR), 1.0f);
	xSample2 += tap(xIn.m_xUV, float2( -fR,  fR), -1.0f);

	xSample1.r = dot(xSample1.rgb * xSample1.rgb, 0.333f * 0.333f);
	xSample2.r = dot(xSample2.rgb * xSample2.rgb, 0.333f * 0.333f);

	return smoothstep(g_fEdgeDetectionParams.x * 0.01f, g_fEdgeDetectionParams.y * 0.01f, sqrt(xSample1.r+ xSample2.r) );
}

float4 PShader_Sobel( const Vertex xIn ) : COLOR
{
	half4 xSample1 = 0.0f;
	half4 xSample2 = 0.0f;
	float fR = 0.5f * g_fEdgeDetectionParams.z;

	xSample1 += tap(xIn.m_xUV, float2( -fR, -fR), -1.0f);
	xSample1 += tap(xIn.m_xUV, float2( -fR,0.0f), -2.0f);
	xSample1 += tap(xIn.m_xUV, float2( -fR,  fR), -1.0f);
	
	xSample1 += tap(xIn.m_xUV, float2(  fR, -fR),  1.0f);
	xSample1 += tap(xIn.m_xUV, float2(  fR,0.0f),  2.0f);
	xSample1 += tap(xIn.m_xUV, float2(  fR,  fR),  1.0f);

	xSample2 += tap(xIn.m_xUV, float2( -fR,  fR), -1.0f);
	xSample2 += tap(xIn.m_xUV, float2(0.0f,  fR), -2.0f);
	xSample2 += tap(xIn.m_xUV, float2( -fR,  fR), -1.0f);

	xSample2 += tap(xIn.m_xUV, float2( -fR, -fR),  1.0f);
	xSample2 += tap(xIn.m_xUV, float2(0.0f, -fR),  2.0f);
	xSample2 += tap(xIn.m_xUV, float2( -fR, -fR),  1.0f);


	xSample1.r = dot(xSample1.rgb * xSample1.rgb, 0.333f * 0.333f);
	xSample2.r = dot(xSample2.rgb * xSample2.rgb, 0.333f * 0.333f);

	return smoothstep(g_fEdgeDetectionParams.x * 0.01f, g_fEdgeDetectionParams.y * 0.01f, sqrt(xSample1.r+ xSample2.r) );
}

float4 PShader_Prewitt( const Vertex xIn ) : COLOR
{
	half4 xSample1 = 0.0f;
	half4 xSample2 = 0.0f;
	float fR = 0.5f * g_fEdgeDetectionParams.z;

	xSample1 += tap(xIn.m_xUV, float2( -fR, -fR), -1.0f);
	xSample1 += tap(xIn.m_xUV, float2( -fR,0.0f), -1.0f);
	xSample1 += tap(xIn.m_xUV, float2( -fR,  fR), -1.0f);
	
	xSample1 += tap(xIn.m_xUV, float2(  fR, -fR),  1.0f);
	xSample1 += tap(xIn.m_xUV, float2(  fR,0.0f),  1.0f);
	xSample1 += tap(xIn.m_xUV, float2(  fR,  fR),  1.0f);

	xSample2 += tap(xIn.m_xUV, float2( -fR,  fR), -1.0f);
	xSample2 += tap(xIn.m_xUV, float2(0.0f,  fR), -1.0f);
	xSample2 += tap(xIn.m_xUV, float2( -fR,  fR), -1.0f);

	xSample2 += tap(xIn.m_xUV, float2( -fR, -fR),  1.0f);
	xSample2 += tap(xIn.m_xUV, float2(0.0f, -fR),  1.0f);
	xSample2 += tap(xIn.m_xUV, float2( -fR, -fR),  1.0f);


	xSample1.r = dot(xSample1.rgb * xSample1.rgb, 0.333f * 0.333f);
	xSample2.r = dot(xSample2.rgb * xSample2.rgb, 0.333f * 0.333f);

	return smoothstep(g_fEdgeDetectionParams.x * 0.01f, g_fEdgeDetectionParams.y * 0.01f, sqrt(xSample1.r+ xSample2.r) );
}




float4 PShader_Blur( const Vertex xIn ) : COLOR
{
	const static int NumTaps = 5;
	float fSamples[NumTaps][NumTaps] =
	{ 
// 		{0.0f, 1.0f, 1.0f, 1.0f, 0.0f}, // 3
// 		{1.0f, 1.0f, 4.0f, 1.0f, 1.0f}, // 8
// 		{1.0f, 4.0f, 6.0f, 4.0f, 1.0f}, // 16
// 		{1.0f, 1.0f, 4.0f, 1.0f, 1.0f}, // 8
// 		{0.0f, 1.0f, 1.0f, 1.0f, 0.0f}, // 3

		{0.0f, 1.0f, 1.0f, 1.0f, 0.0f}, // 3
		{1.0f, 1.0f, 3.0f, 1.0f, 1.0f}, // 7
		{1.0f, 3.0f, 3.0f, 3.0f, 1.0f}, // 11
		{1.0f, 1.0f, 3.0f, 1.0f, 1.0f}, // 7
		{0.0f, 1.0f, 1.0f, 1.0f, 0.0f}, // 3
	};
	float fSampleMultiplier = 1.0 / 31.0f;

	half4 xSample = 0.0f;

	for (int i = 0; i < NumTaps; ++i)
	{
		for (int j = 0; j < NumTaps; ++j)
		{
			float2 fOffset = ((float2(i, j) / float(NumTaps - 1)) - (0.5f).xx) * g_fTint.w;
			xSample += tap(xIn.m_xUV, fOffset, fSamples[i][j]);
		}
	}
	
	return float4(xSample.a * g_fTint.rgb * fSampleMultiplier, 1.0f);
}


/******************************************************************************/
/* Techniques                                                                 */
/******************************************************************************/

technique LoG // Laplacian of Gaussian - sweet but expensive
{
	pass
	{
		VertexShader = compile vs_3_0 VShader();
		PixelShader = compile ps_3_0 PShader_LoG();
	}
	pass
	{
		VertexShader = compile vs_3_0 VShader();
		PixelShader = compile ps_3_0 PShader_Blur();
	}
}

technique Prewitt 
{
	pass
	{
		VertexShader = compile vs_3_0 VShader();
		PixelShader = compile ps_3_0 PShader_Prewitt();
	}
	pass
	{
		VertexShader = compile vs_3_0 VShader();
		PixelShader = compile ps_3_0 PShader_Blur();
	}
}

technique Sobel
{
	pass
	{
		VertexShader = compile vs_3_0 VShader();
		PixelShader = compile ps_3_0 PShader_Sobel();
	}
	pass
	{
		VertexShader = compile vs_3_0 VShader();
		PixelShader = compile ps_3_0 PShader_Blur();
	}
}

technique Robert
{
	pass
	{
		VertexShader = compile vs_3_0 VShader();
		PixelShader = compile ps_3_0 PShader_Robert();
	}
	pass
	{
		VertexShader = compile vs_3_0 VShader();
		PixelShader = compile ps_3_0 PShader_Blur();
	}
}


technique L3 // discreet approximation of Laplacian 
{
	pass
	{
		VertexShader = compile vs_3_0 VShader();
		PixelShader = compile ps_3_0 PShader_L3();
	}
	pass
	{
		VertexShader = compile vs_3_0 VShader();
		PixelShader = compile ps_3_0 PShader_Blur();
	}
}


technique L2 // discreet approximation of Laplacian 
{
	pass
	{
		VertexShader = compile vs_3_0 VShader();
		PixelShader = compile ps_3_0 PShader_L2();
	}
	pass
	{
		VertexShader = compile vs_3_0 VShader();
		PixelShader = compile ps_3_0 PShader_Blur();
	}
}

technique L1 // discreet approximation of Laplacian 
{
	pass
	{
		VertexShader = compile vs_3_0 VShader();
		PixelShader = compile ps_3_0 PShader_L1();
	}
	pass
	{
		VertexShader = compile vs_3_0 VShader();
		PixelShader = compile ps_3_0 PShader_Blur();
	}
}


