///////////////////////////////////////////////////////////////////////////////
//
// File: ToolsAVP_Entity_CameraVolume.cpp
//
// Created by Martin G.Harris (7/9/2006)
// 
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"
#include "Entity/CameraVolume/ToolsAVP_Entity_CameraVolume.h"


#ifdef ASURA_USE_CAMERA_VOLUME

// Includes
#include "Camera/Asura_Camera.h"
#include "Core/Asura_OrientedVolume.h"

#include "Interface/Asura_Dialog.h"
#include "Rendering/Asura_Render.h"
#include "Rendering/Asura_Text_Output.h"

#include "Entity/Tools_Entity_Data.h"
#include "Load&Save/Tools_SaveAsura.h"
#include "Objects/Tools_Object_System.h"
#include "Objects/Tools_LineEdit_Object.h"
#include "Objects/Tools_PositionEdit_Object.h"

#include "Entity/CameraVolume/AVP_ClientEntity_CameraVolume.h"
#include "Entity/CameraVolume/AVP_ServerEntity_CameraVolume.h"

///////////////
// Constants //
///////////////
const u_int uAVP_CAMERA_RAIL_COLOUR = 0xffffffff;
const u_int uAVP_PLAYER_RAIL_COLOUR = 0xffaaaaff;

const u_int uAVP_FIXED_ANGLE_LINE_COLOUR = 0xffaaaaaa;

/////////////////
// Static data //
/////////////////
ToolsAVP_Entity_CameraVolume* ToolsAVP_Entity_CameraVolume::s_pxThisCameraVolume = 0;

///////////////////
// Constant data //
///////////////////
// Names of the different camera types.
const char* const g_aszCameraPropertyTypeNames[AVP_CameraVolumeData::NUM_CAMERA_TYPES] =
{
	"Fixed Angle",
	"Static Cam",
	"Rail Cam"
};

///////////////////////////////
// Tools_Entity_CameraVolume //
///////////////////////////////
// Create a project version of the entity.
void Tools_Entity_CameraVolume::Project_Create()
{
	ToolsAVP_Entity_CameraVolume::Create();
}

//////////////////////////////////////
// ToolsAVP_Entity_CameraVolume //
//////////////////////////////////////
ToolsAVP_Entity_CameraVolume::ToolsAVP_Entity_CameraVolume(Asura_Entity* pxEntity)
: Tools_Entity_CameraVolume(pxEntity)
, m_pxDialog_TimeToReset(0)
, m_pxDialog_FOV(0)
, m_pxDialog_Target(0)
, m_pxDialog_Rotation(0)
, m_pxDialog_Elevation(0)
, m_pxDialog_Distance(0)
, m_pxDialog_UsePlayerRail(0)
, m_pxDialog_EditStaticCam(0)
, m_pxDialog_EditCameraRail(0)
, m_pxDialog_EditPlayerRail(0)
, m_uStaticCameraPositionID(0)
, m_iCameraRailID(0)
, m_iPlayerRailID(0)
{
	// Copy data from client entity.
	CopyPropertiesToToolsEntity();
}

ToolsAVP_Entity_CameraVolume::~ToolsAVP_Entity_CameraVolume()
{
	if (s_pxThisCameraVolume == this)
	{
		s_pxThisCameraVolume = 0;
	}

	// Delete any position/line edit objects.
	Tools_PositionEdit_Object::Delete(m_uStaticCameraPositionID);

	Tools_LineEdit_Object::Delete(m_iCameraRailID);
	Tools_LineEdit_Object::Delete(m_iPlayerRailID);
}

/////////////////
// OnSelection //
/////////////////
void ToolsAVP_Entity_CameraVolume::OnSelection()
{
	PARENT::OnSelection();
	s_pxThisCameraVolume = this;
}

///////////////////
// OnDeselection //
///////////////////
void ToolsAVP_Entity_CameraVolume::OnDeselection()
{
	PARENT::OnDeselection();
	if (s_pxThisCameraVolume == this)
	{
		s_pxThisCameraVolume = 0;
	}
}

////////////
// Create //
////////////
void ToolsAVP_Entity_CameraVolume::Create(Asura_Entity* pxEntity)
{
	new ToolsAVP_Entity_CameraVolume(pxEntity);
}

void ToolsAVP_Entity_CameraVolume::Create()
{
	// Get a new GUID
	const Asura_Guid uGuid = Asura_Entity_System::GetCreatedByToolsGuid();

	Tools_Entity_Data* pxToolsData = Tools_Entity_Object::CreateToolsDataWithDefaultName( "Camera Volume", "CamVol", uGuid );
	if ( !pxToolsData ) return;

	// Create the game specfic server object and set its data.
	AVP_ServerEntity_CameraVolume* pxServerObject = new AVP_ServerEntity_CameraVolume(uGuid);

	Asura_Assert( pxServerObject, "ToolsAVP_Entity_CameraVolume::Create - no server object" );
	if ( pxServerObject )
	{
		pxServerObject->ToolsData = pxToolsData;
	}


	// Create the game specific client object.
	/*AVP_ClientEntity_CameraVolume* pxClientObject =*/ new AVP_ClientEntity_CameraVolume(uGuid);

	// Create the game specific tools object and select it.
	ToolsAVP_Entity_CameraVolume* pxToolsObject = new ToolsAVP_Entity_CameraVolume(pxServerObject);

	Asura_Assert( pxToolsObject, "ToolsAVP_Entity_CameraVolume::Create - no tools object" );
	Tools_Object_System::SelectObject( pxToolsObject );

	pxToolsObject->Position    = *Asura_Camera::GetPosition();
	//pxToolsObject->Orientation = *Asura_Camera::GetOrientation();
	pxToolsObject->Orientation.Identity();

	//Tools_Timed_Message_System::AddMessage( 0xffffff, "Created new Camera Volume");
	Tools_SaveAsura::SetEntityModified();
}

////////////
// Render //
////////////
void ToolsAVP_Entity_CameraVolume::Render()
{
	PARENT::Render();

	// Render extra project specific info.
	const AVP_ClientEntity_CameraVolume* pxClient = AVP_ClientEntity_CameraVolume::FindEntity(GetGuid());
	if (!pxClient) return;
	
	const AVP_CameraVolumeData& xProperty = pxClient->m_xProperty;
	const int iType = xProperty.m_xShared.m_iType;
	const bool bFixedAngle = (iType == AVP_CameraVolumeData::TYPE_FIXED_ANGLE);
	
	if (bFixedAngle)
	{
		// Render a line showing where the fixed angle camera will be.
		const AVP_CameraVolumeData::ChunkData_FixedAngle& xFixedAngle = xProperty.m_xFixedAngle;

		Asura_Euler_3 xEuler(xFixedAngle.m_fElevation, xFixedAngle.m_fRotation, 0.0f);
		Asura_Matrix_3x3 xOri;
		xEuler.ToMatrix(xOri);

		const Asura_Vector_3& xTargetPos = Position;
		const Asura_Vector_3 xCameraPosition = xTargetPos + Asura_Vector_3(0.0f, 0.0f, xFixedAngle.m_fDistance) * xOri;

		Asura_Render::RenderLine(xTargetPos, xCameraPosition, uAVP_FIXED_ANGLE_LINE_COLOUR);
	}
}

////////////////////////
// DisplaceProperties //
////////////////////////
void ToolsAVP_Entity_CameraVolume::DisplaceProperties(const Asura_Vector_3& xMove)
{
	// Update the client entity properties.
	AVP_ClientEntity_CameraVolume* pxClient = AVP_ClientEntity_CameraVolume::FindEntity(GetGuid());
	if (pxClient)
	{
		pxClient->m_xProperty.Displace(xMove);
	}

	// Copy data from client entity to tools.
	CopyPropertiesToToolsEntity();

	// Update static camera position object.
	if (m_uStaticCameraPositionID && pxClient)
	{
		Tools_PositionEdit_Object* pxObject = Tools_PositionEdit_Object::Find(m_uStaticCameraPositionID);
		if (pxObject)
		{
			pxObject->Position = pxClient->m_xProperty.m_xStaticCam.m_xPosition;
		}
	}

	// Update the line edit objects.
	if (m_iCameraRailID)
	{
		Tools_LineEdit_Object* pxRailObject = Tools_LineEdit_Object::Find(m_iCameraRailID);
		if (pxRailObject)
		{
			pxRailObject->RefreshPosition();
		}
	}

	if (m_iPlayerRailID)
	{
		Tools_LineEdit_Object* pxRailObject = Tools_LineEdit_Object::Find(m_iPlayerRailID);
		if (pxRailObject)
		{
			pxRailObject->RefreshPosition();
		}
	}
}

/////////////////////////
// UpdateWhileSelected //
/////////////////////////
void ToolsAVP_Entity_CameraVolume::UpdateWhileSelected()
{
	PARENT::UpdateWhileSelected();
	UpdateDialogues();
}

////////////
// Update //
////////////
void ToolsAVP_Entity_CameraVolume::Update()
{
	PARENT::Update();

	/*
	// Copy data back to client.
	AVP_ClientEntity_CameraVolume* pxClient = AVP_ClientEntity_CameraVolume::FindEntity(GetGuid());
	Asura_Assert(pxClient, "Couldn't find camera volume client");
	if (pxClient)
	{
		AVP_CameraVolumeData& xProperty = pxClient->m_xProperty;
		// TODO add anything here
	}
	*/

	// May not be currently selected, make sure that the entity file is flagged as being modified.
	if (Changed)
	{	
		Entity->ToolsData->m_bEntityModified = true;
		SetEntityFileModified();

		Changed = false;	
	}
}

/////////////////////////////////
// CopyPropertiesToToolsEntity //
/////////////////////////////////
// Copy properties from the client entity back to the tools entity.
void ToolsAVP_Entity_CameraVolume::CopyPropertiesToToolsEntity()
{
	/*
	const AVP_ClientEntity_CameraVolume* pxClient = AVP_ClientEntity_CameraVolume::FindEntity(GetGuid());
	Asura_Assert(pxClient, "Couldn't find camera volume client");
	if (pxClient)
	{
		const AVP_CameraVolumeData& xProperty = pxClient->m_xProperty;
		// TODO add anything here
	}
	*/
}

///////////////
// PrintInfo //
///////////////
void ToolsAVP_Entity_CameraVolume::PrintInfo(float& fX, float& fY)
{
	PARENT::PrintInfo(fX, fY);

	const AVP_ClientEntity_CameraVolume* pxClient = AVP_ClientEntity_CameraVolume::FindEntity(GetGuid());
	if (!pxClient) return;
	
	const AVP_CameraVolumeData& xProperty = pxClient->m_xProperty;
	const int iType = xProperty.m_xShared.m_iType;
	const bool bStaticCam = (iType == AVP_CameraVolumeData::TYPE_STATIC_CAM);

	if (bStaticCam)
	{
		const Asura_Vector_3& xStaticPos = xProperty.m_xStaticCam.m_xPosition;
		Asura_Text_Output::Print(fX, fY++, 0xffffffff, 0, "Static position: %.2f, %.2f, %.2f",
								 xStaticPos.x, xStaticPos.y, xStaticPos.z);
	}
}

////////////////////////
// AddOverlayControls //
////////////////////////
void ToolsAVP_Entity_CameraVolume::AddOverlayControls(Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset)
{
	PARENT::AddOverlayControls(pxOverlay, xOffset);
	
	AVP_ClientEntity_CameraVolume* pxClient = AVP_ClientEntity_CameraVolume::FindEntity(GetGuid());
	if (pxClient)
	{
		AVP_CameraVolumeData& xProperty = pxClient->m_xProperty;

		// Camera property type
		Asura_Dialog* pxDialog = new Asura_DropDown_List_Dialog(xOffset, g_aszCameraPropertyTypeNames, &xProperty.m_xShared.m_iType, 10, AVP_CameraVolumeData::NUM_CAMERA_TYPES, 0, &Changed, "Camera Type");
		pxDialog->SetHelpText("Select camera type");
		pxOverlay->AddDialog( pxDialog );
		xOffset.y += Asura_Dialog::LargeSpacing();

		// Override time to reset
		pxDialog = new Asura_FlagTickbox_Dialog("Override time to reset", xOffset, &xProperty.m_xShared.m_uFlags, AVP_CameraVolumeData::FLAG_SHARED_OVERRIDE_TIME_TO_RESET, &Changed);
		pxDialog->SetHelpText("Override the default time to reset");
		pxOverlay->AddDialog( pxDialog );
		xOffset.y += Asura_Dialog::DefaultSpacing();

		// Time to reset
		pxDialog = new Asura_Numerical_Dialog("Time to reset", xOffset, &xProperty.m_xShared.m_fTimeToReset, 0.0f, 100.0f, 1.0f, "%.2f", &Changed);
		pxDialog->SetHelpText("Interpolation time when switching to this camera volume");
		pxOverlay->AddDialog( pxDialog );
		xOffset.y += Asura_Dialog::DefaultSpacing();
		m_pxDialog_TimeToReset = pxDialog;

		// Override FOV
		pxDialog = new Asura_FlagTickbox_Dialog("Override FOV", xOffset, &xProperty.m_xShared.m_uFlags, AVP_CameraVolumeData::FLAG_SHARED_OVERRIDE_FOV, &Changed);
		pxDialog->SetHelpText("Override the default FOV");
		pxOverlay->AddDialog( pxDialog );
		xOffset.y += Asura_Dialog::DefaultSpacing();

		// FOV
		pxDialog = new Asura_AngleNumerical_Dialog("FOV", xOffset, &xProperty.m_xShared.m_fFOV, 1.0f, 89.0f, 1.0f, "%.1f", &Changed);
		pxDialog->SetHelpText("Field of view (in degrees)");
		pxOverlay->AddDialog( pxDialog );
		xOffset.y += Asura_Dialog::DefaultSpacing();
		m_pxDialog_FOV = pxDialog;

		// Override target
		pxDialog = new Asura_FlagTickbox_Dialog("Override target", xOffset, &xProperty.m_xShared.m_uFlags, AVP_CameraVolumeData::FLAG_SHARED_OVERRIDE_TARGET, &Changed);
		pxDialog->SetHelpText("Override the look-at target");
		pxOverlay->AddDialog( pxDialog );
		xOffset.y += Asura_Dialog::DefaultSpacing();

		// Target guid
		// TODO improve the selection interface
		pxDialog = new Asura_Numerical_Dialog("Target guid", xOffset, &xProperty.m_xShared.m_uTarget, 0, UINT_MAX, 1, "%10d", &Changed);
		pxDialog->SetHelpText("Guid of look-at target");
		pxOverlay->AddDialog( pxDialog );
		xOffset.y += Asura_Dialog::LargeSpacing();
		m_pxDialog_Target = pxDialog;

		// Properties for "fixed angle" camera type.
		// Rotation
		pxDialog = new Asura_AngleNumerical_Dialog("Rotation", xOffset, &xProperty.m_xFixedAngle.m_fRotation, 0.0f, 360.0f, 10.0f, "%.1f", &Changed);
		pxDialog->SetHelpText("Rotation angle for fixed angle camera (in degrees)");
		pxOverlay->AddDialog( pxDialog );
		xOffset.y += Asura_Dialog::DefaultSpacing();
		m_pxDialog_Rotation = pxDialog;

		// Elevation
		pxDialog = new Asura_AngleNumerical_Dialog("Elevation", xOffset, &xProperty.m_xFixedAngle.m_fElevation, -89.0f, 89.0f, 10.0f, "%.1f", &Changed);
		pxDialog->SetHelpText("Elevation angle for fixed angle camera (in degrees)");
		pxOverlay->AddDialog( pxDialog );
		xOffset.y += Asura_Dialog::DefaultSpacing();
		m_pxDialog_Elevation = pxDialog;

		// Distance
		pxDialog = new Asura_Numerical_Dialog("Distance", xOffset, &xProperty.m_xFixedAngle.m_fDistance, 0.0f, 100.0f, 1.0f, "%.2f", &Changed);
		pxDialog->SetHelpText("Distance for fixed angle camera");
		pxOverlay->AddDialog( pxDialog );
		xOffset.y += Asura_Dialog::LargeSpacing();
		m_pxDialog_Distance = pxDialog;
	
		// Properties for "static cam"
		// Edit position
		pxDialog = new Asura_Function_Dialog("Edit Static Position", xOffset, _EditStaticCameraPosition);
		pxDialog->SetHelpText("Set the position of the static camera.");
		pxOverlay->AddDialog(pxDialog);
		xOffset.y+=Asura_Dialog::DefaultSpacing();
		m_pxDialog_EditStaticCam = pxDialog;

		// Properties for "rail cam"
		pxDialog = new Asura_FlagTickbox_Dialog("Use player rail", xOffset, &xProperty.m_xShared.m_uFlags, AVP_CameraVolumeData::FLAG_RAIL_CAM_USE_PLAYER_RAIL, &Changed);
		pxDialog->SetHelpText("Use the player/target rail");
		pxOverlay->AddDialog( pxDialog );
		xOffset.y += Asura_Dialog::DefaultSpacing();
		m_pxDialog_UsePlayerRail = pxDialog;
		
		// Edit camera rail
		pxDialog = new Asura_Function_Dialog("Edit Camera Rail", xOffset, _EditCameraRail);
		pxDialog->SetHelpText("Set up the camera rail.");
		pxOverlay->AddDialog(pxDialog);
		xOffset.y+=Asura_Dialog::DefaultSpacing();
		m_pxDialog_EditCameraRail = pxDialog;

		// Edit player rail
		pxDialog = new Asura_Function_Dialog("Edit Player Rail", xOffset, _EditPlayerRail);
		pxDialog->SetHelpText("Set up the player/target rail.");
		pxOverlay->AddDialog(pxDialog);
		xOffset.y+=Asura_Dialog::DefaultSpacing();
		m_pxDialog_EditPlayerRail = pxDialog;
	}

	UpdateDialogues();
}


/////////////////////
// UpdateDialogues //
/////////////////////
void ToolsAVP_Entity_CameraVolume::UpdateDialogues()
{
	const AVP_ClientEntity_CameraVolume* pxClient = AVP_ClientEntity_CameraVolume::FindEntity(GetGuid());
	if (!pxClient) return;
	
	const AVP_CameraVolumeData& xProperty = pxClient->m_xProperty;
	const int iType = xProperty.m_xShared.m_iType;

	// Shared
	if (m_pxDialog_TimeToReset)
	{
		m_pxDialog_TimeToReset->SetEnabled(xProperty.IsEnabledOverrideTimeToReset());
	}
	if (m_pxDialog_FOV)
	{
		m_pxDialog_FOV->SetEnabled(xProperty.IsEnabledOverrideFOV());
	}
	if (m_pxDialog_Target)
	{
		m_pxDialog_Target->SetEnabled(xProperty.IsEnabledOverrideTarget());
	}

	// Fixed angle
	const bool bFixedAngle = (iType == AVP_CameraVolumeData::TYPE_FIXED_ANGLE);
	if (m_pxDialog_Rotation)
	{
		m_pxDialog_Rotation->SetEnabled(bFixedAngle);
	}
	if (m_pxDialog_Elevation)
	{
		m_pxDialog_Elevation->SetEnabled(bFixedAngle);
	}
	if (m_pxDialog_Distance)
	{
		m_pxDialog_Distance->SetEnabled(bFixedAngle);
	}

	// Static cam
	const bool bStaticCam = (iType == AVP_CameraVolumeData::TYPE_STATIC_CAM);
	if (m_pxDialog_EditStaticCam)
	{
		m_pxDialog_EditStaticCam->SetEnabled(bStaticCam);
	}

	// Rail cam
	const bool bRailCam = (iType == AVP_CameraVolumeData::TYPE_RAIL_CAM);
	if (m_pxDialog_UsePlayerRail)
	{
		m_pxDialog_UsePlayerRail->SetEnabled(bRailCam);
	}
	if (m_pxDialog_EditCameraRail)
	{
		m_pxDialog_EditCameraRail->SetEnabled(bRailCam);
	}
	if (m_pxDialog_EditPlayerRail)
	{
		const bool bUsePlayerRail = xProperty.IsEnabledUsePlayerRail();
		m_pxDialog_EditPlayerRail->SetEnabled(bRailCam && bUsePlayerRail);
	}
}

//////////////////
// EditPosition //
//////////////////
void ToolsAVP_Entity_CameraVolume::EditPosition(u_int& uPositionEditObjectID,
													Asura_Vector_3* pxPosition,
													const char* pszName,
													const char* pszMessage)
{
	// See if the position edit object already exists, if so then select it.
	Tools_PositionEdit_Object* pxObject = Tools_PositionEdit_Object::Find(uPositionEditObjectID);
	if (pxObject)
	{
		Tools_Object_System::SelectObject(pxObject);
		return;
	}

	if (!pxPosition) return;

	// Set a default value for the position if needed.
	if (*pxPosition == Asura_Maths::ZeroVector3)
	{
		*pxPosition = Position;
	}

	pxObject = new Tools_PositionEdit_Object(pxPosition, NULL, pszName, pszMessage, &Changed);
	
	uPositionEditObjectID = pxObject->GetID();
	
	// Select it.
	Tools_Object_System::SelectObject(pxObject);
}

//////////////////////////////
// EditStaticCameraPosition //
//////////////////////////////
void ToolsAVP_Entity_CameraVolume::EditStaticCameraPosition()
{
	AVP_CameraVolumeData* pxProperty = GetClientProperty();
	if (pxProperty)
	{
		EditPosition(m_uStaticCameraPositionID, &pxProperty->m_xStaticCam.m_xPosition, "Static position", "Edit static camera position");
	}
}

//////////////
// EditRail //
//////////////
void ToolsAVP_Entity_CameraVolume::EditRail(int& iLineEditObjectID,
												Asura_Vector_3* pxStartPos,
												Asura_Vector_3* pxEndPos,
												const char* pszName,
												const char* pszStartMessage,
												const char* pszEndMessage,
												u_int uColour)
{
	// See if the line edit object already exists, if so then select it.
	Tools_LineEdit_Object* pxObject = Tools_LineEdit_Object::Find(iLineEditObjectID);
	if (pxObject)
	{
		Tools_Object_System::SelectObject(pxObject);
		return;
	}

	if (!pxStartPos || !pxEndPos) return;

	// Set a default value for the start position, if needed.
	if (*pxStartPos == Asura_Maths::ZeroVector3)
	{
		*pxStartPos = Position;
		pxStartPos->x += 0.5f;
		pxStartPos->y -= 0.5f;
	}
	// Set a default value for the end position, if needed.
	if (*pxEndPos == Asura_Maths::ZeroVector3)
	{
		*pxEndPos = Position;
		pxEndPos->x -= 0.5f;
		pxEndPos->y -= 0.5f;
	}

	pxObject = new Tools_LineEdit_Object(pxStartPos, pxEndPos,
										 pszName, pszStartMessage, pszEndMessage,
										 &Changed);
	if (pxObject)
	{
		pxObject->SetLineColour(uColour);
	}
	
	iLineEditObjectID = pxObject->GetToolsID();
	
	// Select it.
	Tools_Object_System::SelectObject(pxObject);
}


////////////////////
// EditCameraRail //
////////////////////
void ToolsAVP_Entity_CameraVolume::EditCameraRail()
{
	AVP_CameraVolumeData* pxProperty = GetClientProperty();
	if (pxProperty)
	{
		EditRail(m_iCameraRailID,
				 &pxProperty->m_xRailCam.m_xCameraRailStart,
				 &pxProperty->m_xRailCam.m_xCameraRailEnd,
				 "Camera Rail", "Start point", "End point",
				 uAVP_CAMERA_RAIL_COLOUR);
	}
}

////////////////////
// EditPlayerRail //
////////////////////
void ToolsAVP_Entity_CameraVolume::EditPlayerRail()
{
	AVP_CameraVolumeData* pxProperty = GetClientProperty();
	if (pxProperty)
	{
		EditRail(m_iPlayerRailID,
				 &pxProperty->m_xRailCam.m_xPlayerRailStart,
				 &pxProperty->m_xRailCam.m_xPlayerRailEnd,
				 "Player/Target Rail", "Start point", "End point",
				 uAVP_PLAYER_RAIL_COLOUR);
	}
}

///////////////////////
// GetClientProperty //
///////////////////////
// Return pointer to the client properties (or null).
AVP_CameraVolumeData* ToolsAVP_Entity_CameraVolume::GetClientProperty()
{
	AVP_ClientEntity_CameraVolume* pxClient = AVP_ClientEntity_CameraVolume::FindEntity(GetGuid());
	Asura_Assert(pxClient, "Couldn't find client camera volume entity");
	return pxClient ? &pxClient->m_xProperty : 0;
}


#endif //ASURA_USE_CAMERA_VOLUME

