//////////////////////////////////////////////////////////////////////////////
//
// ToolsAVP_Entity_AlienTeam.cpp
//
// Copyright (C) 2007 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"

// header
#include "Entity/Controller/ToolsAVP_Entity_AlienTeam.h"

// asura
#include "Blueprint/Asura_Blueprint.h"
#include "Blueprint/Asura_Blueprint_System.h"
#include "Camera/Asura_Camera.h"
#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Overlay.h"

// avp
#include "Actor/Vertebrates/Alien/AVP_Alien_Description.h"
#include "Blueprint/AVP_Blueprint_HashIDs.h"
#include "Entity/Controller/AVP_Controller_Messages.h"
#include "Entity/Controller/AVP_ServerEntity_Team_Alien.h"
#include "Entity/Spawners/Alien/AVP_ServerEntity_AlienSpawnPoint.h"
#include "Entity/AVP_Entity_EventIDs.h"
#include "Entity/AVP_Messages.h"

// tools
#include "Entity/Spawners/Alien/ToolsAVP_Entity_AlienSpawnPoint.h"
#include "Entity/GamePlayVolume/ToolsAVP_Entity_GamePlayVolume.h"
#include "Load&Save/Tools_SaveAsura.h"
#include "Misc/Tools_Timed_Messages.h"
#include "Movement/Tools_Movement.h"
#include "Objects/Tools_Object_System.h"
#include "Objects/ToolsAVP_Object_Type.h"

//////////////////////////////////////////////////////////////////////////////
//
// ToolsAVP_Entity_AlienTeam
//
//////////////////////////////////////////////////////////////////////////////

#define BLUEPRINT_LABEL_X_OFFSET 8.0f

ToolsAVP_Entity_AlienTeam*		ToolsAVP_Entity_AlienTeam::s_pxToolsAlienTeam			= NULL;
const char**					ToolsAVP_Entity_AlienTeam::s_ppszSpawnerNames			= NULL;
const char**					ToolsAVP_Entity_AlienTeam::s_ppszCategoryNames			= NULL;
Asura_Guid*						ToolsAVP_Entity_AlienTeam::s_ppuSpawnerGuids			= NULL;
int								ToolsAVP_Entity_AlienTeam::s_iNumSpawners				= 0;
int								ToolsAVP_Entity_AlienTeam::s_iSelectedSpawner			= -1;
int								ToolsAVP_Entity_AlienTeam::s_iNumCategories				= 0;
Asura_List_Dialog*				ToolsAVP_Entity_AlienTeam::s_pxSpawnerListDialog		= NULL;
int								ToolsAVP_Entity_AlienTeam::s_iSelectedCategory			= 0;
bool							ToolsAVP_Entity_AlienTeam::s_bSelectedCategoryChanged	= false;
const char**					ToolsAVP_Entity_AlienTeam::s_ppszAgilityNames			= NULL;
int								ToolsAVP_Entity_AlienTeam::s_iSelectedAgility			= AVP_Navigation::NUM_AGILITY_RATINGS;
bool							ToolsAVP_Entity_AlienTeam::s_bSelectedAgilityExclusive	= false;
bool							ToolsAVP_Entity_AlienTeam::s_bSelectedAgilityChanged	= false;

/////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////
ToolsAVP_Entity_AlienTeam::ToolsAVP_Entity_AlienTeam( Asura_Entity* pxEntity )
: PARENT( pxEntity )
, m_pxAlienTeam( static_cast< AVP_ServerEntity_Team_Alien* >( pxEntity ) )
{
    Type = TOT_Entity_AlienTeam;

    SetMesh("ToolsShape_AlienTeam");

	// Initialise members
	m_pxSelectedSpawnPoint			= NULL;

	// Static
	s_pxToolsAlienTeam				= this;
}

/////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////
ToolsAVP_Entity_AlienTeam::~ToolsAVP_Entity_AlienTeam()
{
	ToolsAVP_Entity_SpawnPoint* pxSpawn = static_cast<ToolsAVP_Entity_SpawnPoint*>(m_pxSelectedSpawnPoint);

	delete [] s_ppszSpawnerNames;
	s_ppszSpawnerNames = 0;
	delete [] s_ppszCategoryNames;
	s_ppszCategoryNames = 0;
	delete [] s_ppszAgilityNames;
	s_ppszAgilityNames = 0;
	delete [] s_ppuSpawnerGuids;
	s_ppuSpawnerGuids = 0;

	if (pxSpawn)
	{
		ToolsAVP_Entity_SpawnPoint::ClearSelectedSpawner();
	}

	ToolsAVP_Entity_AlienSpawnPoint::ClearSelectedSpawner();
    s_pxToolsAlienTeam = NULL;
}

/////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////
bool ToolsAVP_Entity_AlienTeam::HandlesThisMessage( int iMessageID )
{
	switch (iMessageID)
	{
		case AVP_MSGID_SET_AI_TARGET_PRIORITIES:
		case AVP_MSGID_RESET_AI_TARGET_PRIORITIES:
		case AVP_MSGID_ALIEN_TEAM_THREAT_BONUS:
		{
			return true;
		}
	}

    return PARENT::HandlesThisMessage( iMessageID );
}


/////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////
bool ToolsAVP_Entity_AlienTeam::HandlesThisEvent(int iEventID) const
{
	switch ( iEventID )
	{
		case AVP_ENTITY_EVENT_ONALIENKILLED:
		case AVP_ENTITY_EVENT_ONALIENSGONE:
			return true;

		default:
			return PARENT::HandlesThisEvent(iEventID);
	}
}

/////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////
void ToolsAVP_Entity_AlienTeam::Create()
{
    // We only allow one of these per level.
    if ( AVP_ServerEntity_Team_Alien::GetInstance() )
    {
        Tools_Timed_Message_System::AddMessage( Tools_Timed_Message_System::COLOUR_ERROR, "Only one of these allowed per-level." );
        return;
    }

    const Asura_Guid uGuid = Asura_Entity_System::GetCreatedByToolsGuid();

    // Set up tools data and auto-create a unique name (prompt for new name if there is a clash).
    Tools_Entity_Data* pxToolsData = CreateToolsDataWithDefaultName( "Alien Team", "Alien Team", uGuid );
    if ( !pxToolsData ) { return; }

    AVP_ServerEntity_Team_Alien* pxGameAlienTeam = new AVP_ServerEntity_Team_Alien( uGuid );
    if ( !pxGameAlienTeam ) { return; }
    pxGameAlienTeam->ToolsData = pxToolsData;

    ToolsAVP_Entity_AlienTeam* pxToolsAlienTeam = new ToolsAVP_Entity_AlienTeam( pxGameAlienTeam );
    if ( !pxToolsAlienTeam ) { return; }

    Tools_Object_System::SelectObject( pxToolsAlienTeam );

    Asura_Vector_3 xCameraForward;
    Asura_Camera::GetForwardVector( xCameraForward );
    pxToolsAlienTeam->Position = *Asura_Camera::GetPosition() + xCameraForward * 2.0f;
    pxToolsAlienTeam->Orientation.Identity();

    Tools_Timed_Message_System::AddMessage( Tools_Timed_Message_System::COLOUR_DEFAULT, "Created new Alien Team" );
    Tools_SaveAsura::SetEntityModified();
}

/////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////
Tools_Object* ToolsAVP_Entity_AlienTeam::Create(Asura_Entity* pxEntity)
{
    return new ToolsAVP_Entity_AlienTeam(pxEntity);
}

/////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////
void ToolsAVP_Entity_AlienTeam::UpdateWhileSelected(void)
{
	PARENT::UpdateWhileSelected();

	UpdateOverlay();
	UpdateSelectedSpawnPoint();
}

/////////////////////////////////////////////////////////////////////////

void ToolsAVP_Entity_AlienTeam::UpdatePositionInAsuraData(bool /*updateLinkedObjects*/)
{
    // Moving the tools object around doesn't change any asura data, but we still want to save
    // the new tools object position out, so we set the entity data as modified.
    PARENT::UpdatePositionInAsuraData( true );
    Tools_SaveAsura::SetEntityModified();
}

/////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////
void ToolsAVP_Entity_AlienTeam::UpdateSelectedSpawnPoint()
{
	if (m_xAlienSpawnList.size() > 0)
	{
		if (s_iSelectedSpawner > -1)
		{
			s_iSelectedSpawner = Asura_Maths::Max( 0, Asura_Maths::Min( s_iSelectedSpawner, m_xAlienSpawnList.size() - 1 ) );
			m_pxSelectedSpawnPoint = m_xAlienSpawnList.get_entry(s_iSelectedSpawner);
		}
	}
	else
	{
		s_iSelectedSpawner = -1;
	}
}

/////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////
void ToolsAVP_Entity_AlienTeam::UpdateOverlay()
{
	if (s_bSelectedCategoryChanged || s_bSelectedAgilityChanged)
	{
		DeleteOverlay();
		CreateOverlay();
		s_bSelectedCategoryChanged = false;
		s_bSelectedAgilityChanged = false;
	}
}

/////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////
void ToolsAVP_Entity_AlienTeam::AddOverlayControls(Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset)
{
	PARENT::AddOverlayControls(pxOverlay, xOffset);
	xOffset.y += Asura_Dialog::LargeSpacing() - Asura_Dialog::DefaultSpacing();

	//Asura_Dialog* pxDialog;

	////-----------------------------
	//// Blueprints
	////-----------------------------
	//// Select Blueprint label
	//pxDialog = new Asura_StringDisplay_Dialog("Choose Initial Settings Blueprint", xOffset);
	//pxOverlay->AddDialog(pxDialog);
	//xOffset.y += Asura_Dialog::DefaultSpacing();

	//float OldXOffset = xOffset.x;

	//// Select blueprint functionality
	//pxDialog = new Asura_Function_Dialog("Blueprint: ", xOffset, ChooseBlueprint);
	//pxOverlay->AddDialog(pxDialog);
	//xOffset.x += BLUEPRINT_LABEL_X_OFFSET;

	//// Selected Blueprint label
	//char szBluePrintString[256] = "";
	//Asura_PrintfToBfr(szBluePrintString, "%s", 256, true, GetBlueprintString());
	//pxDialog = new Asura_StringDisplay_Dialog(szBluePrintString, xOffset);
	//pxOverlay->AddDialog(pxDialog);
	//xOffset.y += Asura_Dialog::LargeSpacing();
	////-----------------------------

	////---------------------------------------------------------
	//// Alien Spawn Point List
	////---------------------------------------------------------
	//xOffset.x = OldXOffset;

	////---------------------------------------------------------
	//// Filter list by gameplay zone
	////---------------------------------------------------------
	//UpdateCategoryList();
	//pxDialog = new Asura_StringDisplay_Dialog("Filter List by GamePlay Zone", xOffset);
	//pxOverlay->AddDialog(pxDialog);
	//xOffset.y += Asura_Dialog::DefaultSpacing();
	////add drop down list for categories
	//pxDialog = new Asura_DropDown_List_Dialog(xOffset, s_ppszCategoryNames, &s_iSelectedCategory, 10, s_iNumCategories, NULL, &s_bSelectedCategoryChanged);
	//pxOverlay->AddDialog(pxDialog);
	//xOffset.y += Asura_Dialog::LargeSpacing();

	////---------------------------------------------------------
	//// Filter list by Spawnpoint agility zone
	////---------------------------------------------------------
	//InitAgilityFilter();
	//pxDialog = new Asura_StringDisplay_Dialog("Filter List by Spawn Point Agility", xOffset);
	//pxOverlay->AddDialog(pxDialog);
	//xOffset.y += Asura_Dialog::DefaultSpacing();

	////add drop down list for categories
	//pxDialog = new Asura_DropDown_List_Dialog(xOffset, s_ppszAgilityNames, &s_iSelectedAgility, 4, 4, NULL, &s_bSelectedAgilityChanged);
	//pxOverlay->AddDialog(pxDialog);
	//const float fOldXOffset = xOffset.x;
	//xOffset.x += pxDialog->GetMaxCoords().x;

	//Asura_Tickbox_Dialog* pxTBDialog = new Asura_Tickbox_Dialog("exclusive", xOffset, &s_bSelectedAgilityExclusive);
	//pxTBDialog->SetClickedFunction(SelectedAgilityChanged);
	//pxTBDialog->SetEnabled( ( s_iSelectedAgility != AVP_Navigation::AR_LOW ) && ( s_iSelectedAgility <= AVP_Navigation::AR_HIGH ) );
	//pxOverlay->AddDialog(pxTBDialog);
	//xOffset.x = fOldXOffset;
	//xOffset.y += Asura_Dialog::LargeSpacing();

	////---------------------------------------------------------
	//// Add the spawn point list
	////---------------------------------------------------------
	//UpdateSpawnPointsList();
	//pxDialog = new Asura_StringDisplay_Dialog("Alien Spawn Point List :", xOffset);
	//pxOverlay->AddDialog(pxDialog);
	//xOffset.y += Asura_Dialog::DefaultSpacing();

	//Asura_List_Dialog* pxListDialog = s_pxSpawnerListDialog = new Asura_List_Dialog(xOffset, s_ppszSpawnerNames, &s_iSelectedSpawner, GetNumRowsToDisplay(), GetNumEntriesToDisplay(), NULL);
	//pxListDialog->SetHelpText("Choose spawn point (double-click to edit)");
	//pxListDialog->SetDoubleClickFunction( EditSelectedSpawnPoint, 0 );
	//pxOverlay->AddDialog(pxListDialog);
	//xOffset.y = pxListDialog->MaxCoord.y + Asura_Dialog::DefaultSpacing();

	////---------------------------------------------------------
	//// edit spawn point
	////---------------------------------------------------------
	//pxDialog = new Asura_Function_Dialog("Edit Selected Spawn Point", xOffset, EditSelectedSpawnPoint);
	//pxOverlay->AddDialog(pxDialog);
	//xOffset.y += Asura_Dialog::DefaultSpacing();

	////---------------------------------------------------------
	//// Focus camera on spawn point
	////---------------------------------------------------------
	//pxDialog = new Asura_Function_Dialog("Focus Selected Spawn Point", xOffset, GoToSpawnPoint);
	//pxOverlay->AddDialog(pxDialog);
	//xOffset.y += Asura_Dialog::DefaultSpacing();
}

/////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////
int ToolsAVP_Entity_AlienTeam::GetNumRowsToDisplay()
{
	if (s_iNumSpawners <= 0)
	{
		return 1;
	}
	else if (s_iNumSpawners >= 11)
	{
		return 10;
	}
	else
	{
		return s_iNumSpawners;
	}
}

/////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////
int ToolsAVP_Entity_AlienTeam::GetNumEntriesToDisplay()
{
	if (s_iNumSpawners <= 0)
	{
		return 1;
	}
	else
	{
		return s_iNumSpawners;
	}
}

/////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////
void ToolsAVP_Entity_AlienTeam::InitAgilityFilter()
{
	List <ToolsAVP_Entity_GamePlayVolume*> xAgilityList;

	delete [] s_ppszAgilityNames;

	s_ppszAgilityNames = new const char*[AVP_Navigation::NUM_AGILITY_RATINGS + 1];
	Asura_Assert( AVP_Navigation::NUM_AGILITY_RATINGS == 3, "Agility rating enum changed, please fix up this side!" );

	s_ppszAgilityNames[0] = "Low Agility";
	s_ppszAgilityNames[1] = "Medium Agility";
	s_ppszAgilityNames[2] = "High Agility";
	s_ppszAgilityNames[3] = "All Agilities";
}

/////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////
void ToolsAVP_Entity_AlienTeam::UpdateCategoryList()
{
	List <ToolsAVP_Entity_GamePlayVolume*> xCategoryList;

	xCategoryList.empty();
	s_iNumCategories = 0;

	// build a list of
	for (Asura_Hashed_List_It<ToolsAVP_Entity_GamePlayVolume> it(ToolsAVP_Entity_GamePlayVolume::GetList()); !it.Done(); it.Next())
	{
		ToolsAVP_Entity_GamePlayVolume* pxGPVolume = it.GetCurrent();

		if (pxGPVolume)
		{
			xCategoryList.add_entry(pxGPVolume);
		}
	}

	// reset our static array information
	s_iNumCategories = xCategoryList.size() + 1;

	if (s_iNumCategories > 0)
	{

		delete [] s_ppszCategoryNames;

		s_ppszCategoryNames = new const char*[s_iNumCategories];

		s_ppszCategoryNames[0] = "All";

		// Fill our static arrays with the spawn point information
		for (int i = 1; i < s_iNumCategories; i += 1)
		{
			s_ppszCategoryNames[i] = xCategoryList.get_entry(i - 1)->GetName();
		}
	}
}

/////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////
void ToolsAVP_Entity_AlienTeam::UpdateSpawnPointsList()
{
	m_xAlienSpawnList.empty();
	s_iNumSpawners = 0;

	// build a list of
	for (Asura_Hashed_List_It<ToolsAVP_Entity_AlienSpawnPoint> it(ToolsAVP_Entity_AlienSpawnPoint::GetList()); !it.Done(); it.Next())
	{
		ToolsAVP_Entity_AlienSpawnPoint* pxSpawnPoint = it.GetCurrent();

		if (pxSpawnPoint)
		{
			if (s_iSelectedAgility != AVP_Navigation::NUM_AGILITY_RATINGS)
			{
				if( (!s_bSelectedAgilityExclusive ) && ( s_iSelectedAgility < pxSpawnPoint->GetAgilityValue() ) )
				{
					continue;
				}
				else
				if( s_bSelectedAgilityExclusive && ( s_iSelectedAgility != pxSpawnPoint->GetAgilityValue() ) )
				{
					continue;
				}
			}

			if (s_iSelectedCategory > 0)
			{
				const Asura_Char* szCatName = s_ppszCategoryNames[s_iSelectedCategory]; 
				Asura_String szSpawnName = pxSpawnPoint->GetName();

				if (szSpawnName.Contains(szCatName))
				{
					m_xAlienSpawnList.add_entry(pxSpawnPoint);
				}
			}
			else
			{
				m_xAlienSpawnList.add_entry(pxSpawnPoint);
			}
		}
	}

	// reset our static array information
	s_iNumSpawners = m_xAlienSpawnList.size();

	if (s_iNumSpawners <= 0)
	{
		delete [] s_ppszSpawnerNames;
		delete [] s_ppuSpawnerGuids;

		s_ppszSpawnerNames = new const char*[1];
		s_ppuSpawnerGuids = new Asura_Guid[1];

		s_ppszSpawnerNames[0] = "No Alien Spawn Points Available";
		s_ppuSpawnerGuids[0] = ASURA_GUID_UNREGISTERED;

		m_pxSelectedSpawnPoint = NULL;
	}
	else
	{
		delete [] s_ppszSpawnerNames;
		delete [] s_ppuSpawnerGuids;

		s_ppszSpawnerNames = new const char*[s_iNumSpawners];
		s_ppuSpawnerGuids = new Asura_Guid[s_iNumSpawners];

		// Fill our static arrays with the spawn point information
		for (int i = 0; i < s_iNumSpawners; i += 1)
		{
			s_ppszSpawnerNames[i] = m_xAlienSpawnList.get_entry(i)->GetName();
			s_ppuSpawnerGuids[i] = m_xAlienSpawnList.get_entry(i)->GetGuid();
		}
	}
}

/////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////
Asura_String ToolsAVP_Entity_AlienTeam::GetBlueprintString()
{
	if (m_pxAlienTeam->GetBlueprintTypeHashID() != ASURA_HASH_ID_UNSET)
	{
		Asura_Blueprint*	pxBlueprint			= Asura_Blueprint_System::GetBlueprintByHashID(m_pxAlienTeam->GetBlueprintTypeHashID(), m_pxAlienTeam->GetBlueprintHashID());
		if (pxBlueprint)
		{
			return pxBlueprint->GetName();
		}
		else
		{
			return Asura_String("ERROR");
		}
	}
	else
	{
		return Asura_String("UNSET");
	}
}

/////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////
void ToolsAVP_Entity_AlienTeam::ChooseBlueprint(void)
{
	Asura_Hash_ID uBlueprintTypeHashID = s_pxToolsAlienTeam->m_pxAlienTeam->GetBlueprintTypeHashID();
	Asura_Hash_ID uBlueprintHashID     = s_pxToolsAlienTeam->m_pxAlienTeam->GetBlueprintHashID();

	BlueprintSelect_Overlay xBPOverlay( "Choose A Blueprint", &uBlueprintHashID, &uBlueprintTypeHashID, 1, &uAVP_BLUEPRINT_TYPE_AI_SETTINGS, 0, NULL );

	if (!xBPOverlay.DoModal())
	{
		return;
	}

	ToolsAVP_Entity_AlienTeam* pxAlienTeam = s_pxToolsAlienTeam;

	if (!pxAlienTeam)
	{
		return;
	}

	pxAlienTeam->m_pxAlienTeam->SetBlueprintTypeHashID(uBlueprintTypeHashID);
	pxAlienTeam->m_pxAlienTeam->SetBlueprintHashID(uBlueprintHashID);
	// Also update the properties of the object at this point.
	pxAlienTeam->m_pxAlienTeam->InitFromBlueprint();

	pxAlienTeam->DeleteOverlay();
	pxAlienTeam->CreateOverlay();
}

/////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////
void ToolsAVP_Entity_AlienTeam::GoToSpawnPoint(void)
{
	ToolsAVP_Entity_AlienTeam* pxAlienTeam = s_pxToolsAlienTeam;

	if (pxAlienTeam)
	{
		if (pxAlienTeam->m_pxSelectedSpawnPoint)
		{
			Asura_Vector_3		xNewCameraPos	= pxAlienTeam->m_pxSelectedSpawnPoint->Position;
			Asura_Matrix_3x3	xNewCameraOri	= *Asura_Camera::GetOrientation();
			Asura_Vector_3		xFwdVec;
			
			Asura_Camera::GetForwardVector(xFwdVec);
			xNewCameraPos -= xFwdVec * 2.0f;

			Tools_Movement::TeleportCameraToPosition(&xNewCameraPos, &xNewCameraOri);
		}
		else
		{
			Tools_Timed_Message_System::AddMessage(0xff0000,"No Spawn points available");
		}
	}
}

/////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////
void ToolsAVP_Entity_AlienTeam::EditSelectedSpawnPoint()
{
	ToolsAVP_Entity_AlienTeam* pxAlienTeam = s_pxToolsAlienTeam;

	if (pxAlienTeam)
	{
		if (pxAlienTeam->m_pxSelectedSpawnPoint)
		{
	        Tools_Object_System::SelectObject( pxAlienTeam->m_pxSelectedSpawnPoint );
		}
		else
		{
			Tools_Timed_Message_System::AddMessage(0xff0000,"No Spawn points available");
		}
	}
}

/////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////
void ToolsAVP_Entity_AlienTeam::AddFilesToExportListFromMessage(u_short usMessageID, void* pData, List<Asura_String>& nameList) const
{
	// If message is adding a spawn token, add the necessary files to the export list
	if( usMessageID == AVP_MSGID_ALIEN_TEAM_ADD_SPAWNTOKEN )
	{
		const AVP_Message_AddAlienTeamTokens* pxMsgData = static_cast<AVP_Message_AddAlienTeamTokens*>( pData );

		if( !pxMsgData )
		{
			Asura_Assert( 0, "Can't access message data" );
			return;
		}

		// Add default blueprint files if UNSET
		if( ( pxMsgData->m_uBlueprintHashID == ASURA_HASH_ID_UNSET ) || ( pxMsgData->m_uBlueprintTypeHashID == ASURA_HASH_ID_UNSET ) )
		{
			Asura_Assert(false, "ToolsAVP_Entity_AlienTeam :: AddFilesToExportListFromMessage - Add spawn token message does not have any blueprint set, please set a blueprint for the message.");
		}
		else
		{
			Asura_Assert( pxMsgData->m_uBlueprintTypeHashID == uAVP_BLUEPRINT_TYPE_ACTORS, "Invalid spawn token blueprint type, files not exported" );
			AVP_Alien_Description::AddFilesToExport( false, pxMsgData->m_uBlueprintHashID, nameList );
		}
	}
}

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////
