//////////////////////////////////////////////////////////////////////////////
//
// ToolsAVP_Entity_Squad_Alien.cpp
//
// Copyright (C) 2008 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"

// header
#include "Entity/Controller/ToolsAVP_Entity_Squad_Alien.h"

// asura
#include "Blueprint/Asura_Blueprint_System.h"
#include "Entity/Asura_Entity_Event.h"

// avp
#include "Blueprint/AVP_Blueprint_HashIDs.h"
#include "Entity/AVP_Messages.h"
#include "Entity/Controller/AVP_ServerEntity_Squad_Alien.h"

// tools
#include "Load&Save/Tools_SaveAsura.h"
#include "Misc/Tools_Timed_Messages.h"
#include "Objects/ToolsAVP_Object_Type.h"

//////////////////////////////////////////////////////////////////////////////
//
// ToolsAVP_Entity_Squad_Alien
//
//////////////////////////////////////////////////////////////////////////////

TOOLS_DECLARE_SHARED_PARAM( Asura_String,   0xd83fea79, AlienBehaviour_Blueprint_Func )					// Asura_GetHaBeshID("AlienBehaviour_Blueprint_Func")

const Asura_Hash_ID g_uAVP_BLUEPRINT_HASH_AGGRESSIVE = 0x445fdc04; //"Aggressive"
const Asura_Hash_ID g_uAVP_BLUEPRINT_HASH_HIDEINSHADOWS = 0x84a4678c; //"HideInShadows"

ToolsAVP_Entity_Squad_Alien::ToolsAVP_Entity_Squad_Alien( Asura_Entity* pxEntity )
: PARENT( pxEntity )
, m_bRefreshOverlay( false )
{
	Type = TOT_Entity_AlienSquad;
	SetMesh( "ToolsShape_AlienSquad" );
}

bool ToolsAVP_Entity_Squad_Alien::CanBeDestroyed()
{
	return ( !GetAlienSquad()->IsDefaultSquad() );
}


void ToolsAVP_Entity_Squad_Alien::AddOverlayControls( Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset )
{
	PARENT::AddOverlayControls( pxOverlay, xOffset );
	xOffset.y += Asura_Dialog::LargeSpacing() - Asura_Dialog::DefaultSpacing();

	AVP_ServerEntity_Squad_Alien* pxSquad = GetAlienSquad();




// 	Asura_Dialog* pxDialogue = new Asura_Dialog( "Jump", xOffset );
// 	pxOverlay->AddDialog( pxDialogue );
// 	xOffset.x += 5.0f;
// 
// 	static float s_fMin = 0.0f;
// 	static float s_fMax = 1.0f;
// 	Asura_NumberSlider_Dialog* pxNSDialogue = new Asura_NumberSlider_Dialog( xOffset, &pxSquad->m_fJumpFraction, &s_fMin, &s_fMax, 13.0f, "%0.2f", &Changed );
// 	pxOverlay->AddDialog( pxNSDialogue );
// 	xOffset.x += 15.0f;
// 
// 	pxDialogue = new Asura_Dialog( "Stand", xOffset );
// 	pxOverlay->AddDialog( pxDialogue );
// 	xOffset.x -= 20.0f;
// 	xOffset.y += Asura_Dialog::LargeSpacing();

	static float s_fMin = 0.0f;
	static float s_fMax = 1.0f;

	Asura_Dialog* pxDialogue = NULL;

	pxDialogue = new Asura_Dialog( "Behaviour:", xOffset );
	pxOverlay->AddDialog( pxDialogue );
	xOffset.x += 7.0f;

	Asura_DropDown_List_Dialog* pxDDLDialogue = new Asura_DropDown_List_Dialog(
		xOffset,
		AVP_AlienBehaviourData::s_apszBehaviourStrings,
		&pxSquad->m_xBehaviourData.m_iBehaviour,
		AVP_AlienBehaviourData::NUM_BEHAVIOURS,
		AVP_AlienBehaviourData::NUM_BEHAVIOURS,
		NULL,
		&Changed,
		"" );
	pxDDLDialogue->SetOnSelectCallback( BehaviourChanged, reinterpret_cast< int >( this ) );
	pxOverlay->AddDialog( pxDDLDialogue );
	xOffset.x -= 7.0f;
	xOffset.y += Asura_Dialog::LargeSpacing();

	const bool bDialoguesEnabled = ( pxSquad->m_xBehaviourData.m_iBehaviour == AVP_AlienBehaviourData::CUSTOM );
	const bool bOldBehaviours = pxSquad->m_xBehaviourData.m_iBehaviour != AVP_AlienBehaviourData::AGGRESIVE && 
								pxSquad->m_xBehaviourData.m_iBehaviour != AVP_AlienBehaviourData::HIDE_IN_SHADOWS &&
								pxSquad->m_xBehaviourData.m_iBehaviour != AVP_AlienBehaviourData::AMBUSH;

	if (bOldBehaviours)
	{
		pxDialogue = new Asura_Dialog( "Jump: ", xOffset);
		pxOverlay->AddDialog( pxDialogue );
		xOffset.x += 7.0;
		pxDialogue = new Asura_Numerical_Dialog( "", xOffset, &pxSquad->m_xBehaviourData.m_fJumpFraction, 0.0f, 1.0f, .05f, "%0.2f", &m_bRefreshOverlay);
		pxOverlay->AddDialog( pxDialogue );
		xOffset.x -= 7.0;
		xOffset.y += Asura_Dialog::SmallSpacing();

		if (pxSquad->m_xBehaviourData.m_fStandFraction > s_fMax - pxSquad->m_xBehaviourData.m_fJumpFraction)
		{
			pxSquad->m_xBehaviourData.m_fStandFraction = s_fMax - pxSquad->m_xBehaviourData.m_fJumpFraction;
		}

		pxDialogue = new Asura_Dialog( "Stand: ", xOffset);
		pxOverlay->AddDialog( pxDialogue );
		xOffset.x += 7.0;
		pxDialogue = new Asura_Numerical_Dialog( "", xOffset, &pxSquad->m_xBehaviourData.m_fStandFraction, 0.0f, 1.0f - pxSquad->m_xBehaviourData.m_fJumpFraction, .05f, "%0.2f", &m_bRefreshOverlay);
		pxOverlay->AddDialog( pxDialogue );
		xOffset.x -= 7.0;
		xOffset.y += Asura_Dialog::SmallSpacing();

		float fGrapple = s_fMax - pxSquad->m_xBehaviourData.m_fJumpFraction - pxSquad->m_xBehaviourData.m_fStandFraction;
		static float s_fDummy;
		s_fDummy = fGrapple;		

		pxDialogue = new Asura_Dialog( "Move In Mode:", xOffset );
		pxDialogue->SetEnabled( bDialoguesEnabled );
		pxOverlay->AddDialog( pxDialogue );
		xOffset.x += 9.0f;

		pxDDLDialogue = new Asura_DropDown_List_Dialog(
			xOffset,
			AVP_AlienBehaviourData::s_apszMoveInModeStrings,
			&pxSquad->m_xBehaviourData.m_iMoveInMode,
			AVP_AlienBehaviourData::NUM_MOVE_STYLES,
			AVP_AlienBehaviourData::NUM_MOVE_STYLES,
			NULL,
			&Changed,
			"" );
		pxDDLDialogue->SetOnSelectCallback( DropDownListChanged, reinterpret_cast< int >( this ) );
		pxDDLDialogue->SetEnabled( bDialoguesEnabled );
		pxOverlay->AddDialog( pxDDLDialogue );
		xOffset.x -= 9.0f;
		xOffset.y += Asura_Dialog::DefaultSpacing();

		pxDialogue = new Asura_Numerical_Dialog( "Ideal Target Distance", xOffset, &pxSquad->m_xBehaviourData.m_fIdealDistanceMin, 0.0f, 999.0f, 1.0f, "%.1f", &Changed );
		pxDialogue->SetEnabled( bDialoguesEnabled );
		pxOverlay->AddDialog( pxDialogue );
		xOffset.x += 22.0f;

		pxDialogue = new Asura_Numerical_Dialog( "", xOffset, &pxSquad->m_xBehaviourData.m_fIdealDistanceMax, 0.0f, 999.0f, 1.0f, "%.1f", &Changed );
		pxDialogue->SetEnabled( bDialoguesEnabled );
		pxOverlay->AddDialog( pxDialogue );
		xOffset.x -= 22.0f;
		xOffset.y += Asura_Dialog::DefaultSpacing();

		pxDialogue = new Asura_Numerical_Dialog( "Attack Delay", xOffset, &pxSquad->m_xBehaviourData.m_fAttackDelay, 0.0f, 99.0f, 0.5f, "%.2f", &Changed );
		pxDialogue->SetHelpText( "The time delay between attacks." );
		pxDialogue->SetEnabled( bDialoguesEnabled );
		pxOverlay->AddDialog( pxDialogue );
		xOffset.y += Asura_Dialog::DefaultSpacing();

		pxDialogue = new Asura_Numerical_Dialog( "Max Attackers Per Target", xOffset, &pxSquad->m_xBehaviourData.m_iMaxAttackersPerTarget, 1, 99, 1, "%d", &Changed );
		pxDialogue->SetEnabled( bDialoguesEnabled );
		pxOverlay->AddDialog( pxDialogue );
		xOffset.y += Asura_Dialog::DefaultSpacing();

		pxDialogue = new Asura_Numerical_Dialog( "Max Attackers Total", xOffset, &pxSquad->m_xBehaviourData.m_iMaxAttackersTotal, 0, 99, 1, "%d", &Changed );
		pxDialogue->SetEnabled( bDialoguesEnabled );
		pxOverlay->AddDialog( pxDialogue );
		xOffset.y += Asura_Dialog::DefaultSpacing();

		pxDialogue = new Asura_Numerical_Dialog( "Max Burst Per Target", xOffset, &pxSquad->m_xBehaviourData.m_iMaxBurstPerTarget, 0, 99, 1, "%d", &Changed );
		pxDialogue->SetEnabled( bDialoguesEnabled );
		pxOverlay->AddDialog( pxDialogue );
		xOffset.y += Asura_Dialog::DefaultSpacing();


		pxDialogue = new Asura_Dialog( "Post Attack Action:", xOffset );
		pxDialogue->SetEnabled( bDialoguesEnabled );
		pxOverlay->AddDialog( pxDialogue );
		xOffset.x += 11.0f;

		pxDDLDialogue = new Asura_DropDown_List_Dialog(
			xOffset,
			AVP_AlienBehaviourData::s_apszPostAttackActionStrings,
			&pxSquad->m_xBehaviourData.m_iPostAttackAction,
			AVP_AlienBehaviourData::NUM_POST_ATTACK_ACTIONS,
			AVP_AlienBehaviourData::NUM_POST_ATTACK_ACTIONS,
			NULL,
			&Changed,
			"" );
		pxDDLDialogue->SetOnSelectCallback( DropDownListChanged, reinterpret_cast< int >( this ) );
		pxDDLDialogue->SetEnabled( bDialoguesEnabled );
		pxOverlay->AddDialog( pxDDLDialogue );
		xOffset.x -= 11.0f;
		xOffset.y += Asura_Dialog::DefaultSpacing();

		if ( pxSquad->m_xBehaviourData.m_iPostAttackAction == AVP_AlienBehaviourData::CYCLE_ROUND_DELAYED )
		{
			pxDialogue = new Asura_Numerical_Dialog( "Respawn Delay", xOffset, &pxSquad->m_xBehaviourData.m_fRespawnDelay, 2.0f, 99.0f, 0.5f, "%.2f", &Changed );
			pxDialogue->SetHelpText( "The time delay between Aliens unspawning and respawning again." );
			pxDialogue->SetEnabled( bDialoguesEnabled );
			pxOverlay->AddDialog( pxDialogue );
			xOffset.y += Asura_Dialog::DefaultSpacing();
		}
	}

	bool bUseCustom = true;

	if (pxSquad->m_xBehaviourData.m_iBehaviour == AVP_AlienBehaviourData::AGGRESIVE || pxSquad->m_xBehaviourData.m_iBehaviour == AVP_AlienBehaviourData::HIDE_IN_SHADOWS ||
		pxSquad->m_xBehaviourData.m_iBehaviour == AVP_AlienBehaviourData::AMBUSH )
	{
		Asura_Dialog* pxDialog;
		pxDialog = new Asura_Function_Dialog( "Prototype: ", &g_xParam_AlienBehaviour_Blueprint_Func, xOffset, ChooseBehaviourBP, reinterpret_cast< int >( this ) );
		pxOverlay->AddDialog( pxDialog );
		xOffset.y += Asura_Dialog::DefaultSpacing();

		bUseCustom = pxSquad->m_xBehaviourData.m_uProtoBlueprintHashID == ASURA_HASH_ID_UNSET;

		pxDialogue = new Asura_Dialog( "Jump: ", xOffset);
		pxDialogue->SetEnabled( bUseCustom );
		pxOverlay->AddDialog( pxDialogue );
		xOffset.x += 7.0;
		pxDialogue = new Asura_Numerical_Dialog( "", xOffset, &pxSquad->m_xBehaviourData.m_fJumpFraction, 0.0f, 1.0f, .05f, "%0.2f", &m_bRefreshOverlay);
		pxDialogue->SetEnabled( bUseCustom );
		pxOverlay->AddDialog( pxDialogue );
		xOffset.x -= 7.0;
		xOffset.y += Asura_Dialog::SmallSpacing();

		if (pxSquad->m_xBehaviourData.m_fStandFraction > 1.0f - pxSquad->m_xBehaviourData.m_fJumpFraction)
		{
			pxSquad->m_xBehaviourData.m_fStandFraction = 1.0f - pxSquad->m_xBehaviourData.m_fJumpFraction;
		}

		pxDialogue = new Asura_Dialog( "Stand: ", xOffset);
		pxDialogue->SetEnabled( bUseCustom );
		pxOverlay->AddDialog( pxDialogue );
		xOffset.x += 7.0;
		pxDialogue = new Asura_Numerical_Dialog( "", xOffset, &pxSquad->m_xBehaviourData.m_fStandFraction, 0.0f, 1.0f - pxSquad->m_xBehaviourData.m_fJumpFraction, .05f, "%0.2f", &m_bRefreshOverlay);
		pxDialogue->SetEnabled( bUseCustom );
		pxOverlay->AddDialog( pxDialogue );
		xOffset.x -= 7.0;
		xOffset.y += Asura_Dialog::SmallSpacing();

		float fGrapple = 1.0f - pxSquad->m_xBehaviourData.m_fJumpFraction - pxSquad->m_xBehaviourData.m_fStandFraction;
		static float s_fDummy;
		s_fDummy = fGrapple;

		pxDialogue = new Asura_Dialog( "Grapple: ", xOffset);
		pxDialogue->SetEnabled( bUseCustom );
		pxOverlay->AddDialog( pxDialogue );
		xOffset.x += 7.0;
		pxDialogue = new Asura_Numerical_Dialog( "", xOffset, &s_fDummy, fGrapple, fGrapple, .05f, "%0.2f", &m_bRefreshOverlay);
		pxDialogue->SetEnabled( bUseCustom );
		pxOverlay->AddDialog( pxDialogue );
		xOffset.x -= 7.0;
		xOffset.y += Asura_Dialog::DefaultSpacing();

		pxDialogue = new Asura_Numerical_Dialog( "Time between Grapples", xOffset, &pxSquad->m_xBehaviourData.m_fMinTimeBetweenGrapples, 10.0f, 100.0f, 1.0f, "%0.1f", &Changed);
		pxDialogue->SetEnabled( bUseCustom );
		pxOverlay->AddDialog( pxDialogue );
		xOffset.y += Asura_Dialog::DefaultSpacing();

		pxDialogue = new Asura_Numerical_Dialog( "Damage from behind multiplier", xOffset, &pxSquad->m_xBehaviourData.m_fDamageFromBehindMultiplier, 0.0f, 1.0f, 0.05f, "%0.1f", &Changed);
		pxDialogue->SetEnabled( bUseCustom );
		pxOverlay->AddDialog( pxDialogue );
		xOffset.y += Asura_Dialog::DefaultSpacing();


		pxDialogue = new Asura_Numerical_Dialog( "Time Between Attacks", xOffset, &pxSquad->m_xBehaviourData.m_fTimeBetweenAttacks , 0.0f, 5.0f, 0.05f, "%.2f", &Changed );
		pxDialogue->SetEnabled( bUseCustom );
		pxOverlay->AddDialog( pxDialogue );				
		xOffset.y += Asura_Dialog::DefaultSpacing();

		pxDialogue = new Asura_Numerical_Dialog( "Max Number Of Chargers (0 unlimited)", xOffset, &pxSquad->m_xBehaviourData.m_uMaxNumberOfChargers , 0, 25, 1, "%d", &Changed );
		pxDialogue->SetEnabled( bUseCustom );
		pxOverlay->AddDialog( pxDialogue );				
		xOffset.y += Asura_Dialog::DefaultSpacing();

	}

	if (pxSquad->m_xBehaviourData.m_iBehaviour == AVP_AlienBehaviourData::HIDE_IN_SHADOWS)
	{

		pxDialogue = new Asura_Tickbox_Dialog("Attack if in marine light", xOffset, &pxSquad->m_xBehaviourData.m_bAttackIfInMarineLight, &Changed);
		pxDialogue->SetEnabled( bUseCustom );
		static_cast<Asura_Tickbox_Dialog *>(pxDialogue)->SetClickedFunction( TickBoxChanged, reinterpret_cast< int >( this ) );
		pxOverlay->AddDialog( pxDialogue );				
		xOffset.y += Asura_Dialog::DefaultSpacing();

		pxDialogue = new Asura_Numerical_Dialog( "Time in light before attack", xOffset, &pxSquad->m_xBehaviourData.m_fTimeInLightBeforeAttack , 0.5f, 10.0f, 0.1f, "%.2f", &Changed );
		pxDialogue->SetEnabled( bUseCustom && pxSquad->m_xBehaviourData.m_bAttackIfInMarineLight );
		pxOverlay->AddDialog( pxDialogue );				
		xOffset.y += Asura_Dialog::DefaultSpacing();

		pxDialogue = new Asura_Numerical_Dialog( "Min distance from target to hide at", xOffset, &pxSquad->m_xBehaviourData.m_fMinHideDistance , 5.0f, 10.0f, 0.5f, "%.2f", &Changed );
		pxDialogue->SetEnabled( bUseCustom );
		pxOverlay->AddDialog( pxDialogue );				
		xOffset.y += Asura_Dialog::DefaultSpacing();

		pxDialogue = new Asura_Dialog( "Too Close :", xOffset );
		pxDialogue->SetEnabled( bUseCustom );
		pxOverlay->AddDialog( pxDialogue );
		xOffset.x += 7.0f;
		pxDialogue = new Asura_Dialog( "Attack", xOffset );
		pxDialogue->SetEnabled( bUseCustom );
		pxOverlay->AddDialog( pxDialogue );
		xOffset.x += 5.0f;

		pxDialogue = new Asura_NumberSlider_Dialog( xOffset, &pxSquad->m_xBehaviourData.m_fChaneToRehideIfCloseFraction, &s_fMin, &s_fMax, 13.0f, "%0.2f", &Changed );
		pxDialogue->SetEnabled( bUseCustom );
		pxOverlay->AddDialog( pxDialogue );
		xOffset.x += 15.0f;

		pxDialogue = new Asura_Dialog( "Hide", xOffset );
		pxDialogue->SetEnabled( bUseCustom );
		pxOverlay->AddDialog( pxDialogue );
		xOffset.x -= 27.0f;
		xOffset.y += Asura_Dialog::DefaultSpacing();

		pxDialogue = new Asura_Dialog( "In Light :", xOffset );
		pxDialogue->SetEnabled( bUseCustom );
		pxOverlay->AddDialog( pxDialogue );
		xOffset.x += 7.0f;
		pxDialogue = new Asura_Dialog( "Attack", xOffset );
		pxDialogue->SetEnabled( bUseCustom );
		pxOverlay->AddDialog( pxDialogue );
		xOffset.x += 5.0f;

		pxDialogue = new Asura_NumberSlider_Dialog( xOffset, &pxSquad->m_xBehaviourData.m_fChaneToRehideIfInLightFraction, &s_fMin, &s_fMax, 13.0f, "%0.2f", &Changed );
		pxDialogue->SetEnabled( bUseCustom );
		pxOverlay->AddDialog( pxDialogue );
		xOffset.x += 15.0f;

		pxDialogue = new Asura_Dialog( "Hide", xOffset );
		pxDialogue->SetEnabled( bUseCustom );
		pxOverlay->AddDialog( pxDialogue );
		xOffset.x -= 27.0f;
		xOffset.y += Asura_Dialog::DefaultSpacing();

		pxDialogue = new Asura_Dialog( "After Attack :", xOffset );
		pxDialogue->SetEnabled( bUseCustom );
		pxOverlay->AddDialog( pxDialogue );
		xOffset.x += 7.0f;
		pxDialogue = new Asura_Dialog( "Attack", xOffset );
		pxDialogue->SetEnabled( bUseCustom );
		pxOverlay->AddDialog( pxDialogue );
		xOffset.x += 5.0f;

		pxDialogue = new Asura_NumberSlider_Dialog( xOffset, &pxSquad->m_xBehaviourData.m_fChaneToRehideAfterAttackFraction, &s_fMin, &s_fMax, 13.0f, "%0.2f", &Changed );
		pxDialogue->SetEnabled( bUseCustom );
		pxOverlay->AddDialog( pxDialogue );
		xOffset.x += 15.0f;

		pxDialogue = new Asura_Dialog( "Hide", xOffset );
		pxDialogue->SetEnabled( bUseCustom );
		pxOverlay->AddDialog( pxDialogue );
		xOffset.x -= 27.0f;
		xOffset.y += Asura_Dialog::DefaultSpacing();
	}
}

void ToolsAVP_Entity_Squad_Alien::OnSelection()
{
	m_bRefreshOverlay = false;
	PARENT::OnSelection();
}

void ToolsAVP_Entity_Squad_Alien::UpdateWhileSelected()
{
	PARENT::UpdateWhileSelected();

	if ( m_bRefreshOverlay )
	{
		DeleteOverlay();
		CreateOverlay();
		m_bRefreshOverlay = false;
		Changed = true;
	}
}

bool ToolsAVP_Entity_Squad_Alien::HandlesThisEvent( int iEventID ) const
{
	
	if( (ASURA_ENTITY_EVENT_ONHEALTHPERCENT_0 <= iEventID) && (iEventID <= ASURA_ENTITY_EVENT_ONHEALTHPERCENT_99) ) 
	{
		return true;
	}
	else
	{
		return PARENT::HandlesThisEvent( iEventID );
	}
}

bool ToolsAVP_Entity_Squad_Alien::HandlesThisMessage( int iMessage )
{
	if ( iMessage == AVP_MSGID_SET_ALIEN_SQUAD_BEHAVIOUR )
	{
		return true;
	}
	else
	{
		return PARENT::HandlesThisMessage( iMessage );
	}
}

void ToolsAVP_Entity_Squad_Alien::Update()
{
	PARENT::Update();
}

bool ToolsAVP_Entity_Squad_Alien::GetParameter( Asura_Hash_ID uParamID, Tools_Parameter* pxParam ) const
{
	switch(uParamID)
	{
	case g_uHash_AlienBehaviour_Blueprint_Func:
		if ( GetAlienSquad()->GetBehaviourData().m_uProtoBlueprintHashID == ASURA_HASH_ID_UNSET )
		{
			pxParam->SetString( "UNSET" );
		}
		else
		{
			Asura_Blueprint* pxBlueprint = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_AI_SETTINGS, GetAlienSquad()->GetBehaviourData().m_uProtoBlueprintHashID);
			if ( pxBlueprint )
			{
				pxParam->SetString( pxBlueprint->GetName() );
			}
			else
			{
				pxParam->SetString( "ERROR" );
			}
		}
		break;
	default:
		return PARENT::GetParameter(uParamID,pxParam);
	}

	return true;
}


void ToolsAVP_Entity_Squad_Alien::Create()
{
	const Asura_Guid uGuid = Asura_Entity_System::GetCreatedByToolsGuid();

	Tools_Entity_Data* pxToolsData = CreateToolsDataWithDefaultName( "Alien Squad", "Alien Squad", uGuid );
	if ( !pxToolsData ) { return; }

	AVP_ServerEntity_Squad_Alien* pxSquad = new AVP_ServerEntity_Squad_Alien( uGuid );
	if ( !pxSquad ) { return; }
	pxSquad->ToolsData = pxToolsData;

	ToolsAVP_Entity_Squad_Alien* pxToolsObject = new ToolsAVP_Entity_Squad_Alien( pxSquad );
	if ( !pxToolsObject ) { return; }
	Tools_Object_System::SelectObject( pxToolsObject );

	pxToolsObject->Position		= *Asura_Camera::GetPosition();
	pxToolsObject->Orientation  = Asura_Maths::IdentityMatrix3x3;
	pxToolsObject->m_uGuid		= uGuid;

	Tools_Timed_Message_System::AddMessage( Tools_Timed_Message_System::COLOUR_DEFAULT, "Created new Alien squad." );
	Tools_SaveAsura::SetEntityModified();
}

void ToolsAVP_Entity_Squad_Alien::CreateDefault()
{
	const Asura_Guid uGuid = Asura_Entity_System::GetCreatedByToolsGuid();

	Tools_Entity_Data* pxToolsData = CreateToolsDataWithDefaultName( "Default Alien Squad", "Default Alien Squad", uGuid );
	if ( !pxToolsData ) { return; }

	AVP_ServerEntity_Squad_Alien* pxSquad = new AVP_ServerEntity_Squad_Alien( uGuid );
	if ( !pxSquad ) { return; }
	pxSquad->ToolsData = pxToolsData;
	pxSquad->SetIsDefaultSquad( true );
	AVP_ServerEntity_Squad_Alien::s_pxDefaultSquad = pxSquad;

	ToolsAVP_Entity_Squad_Alien* pxToolsObject = new ToolsAVP_Entity_Squad_Alien( pxSquad );
	if ( !pxToolsObject ) { return; }
	Tools_Object_System::SelectObject( pxToolsObject );

	pxToolsObject->Position		= *Asura_Camera::GetPosition();
	pxToolsObject->Orientation  = Asura_Maths::IdentityMatrix3x3;
	pxToolsObject->m_uGuid		= uGuid;

	Tools_Timed_Message_System::AddMessage( Tools_Timed_Message_System::COLOUR_DEFAULT, "Created new default Alien squad." );
	Tools_SaveAsura::SetEntityModified();
}

Tools_Object* ToolsAVP_Entity_Squad_Alien::Create( Asura_Entity* pxEntity )
{
	return new ToolsAVP_Entity_Squad_Alien( pxEntity );
}

AVP_ServerEntity_Squad_Alien* ToolsAVP_Entity_Squad_Alien::GetAlienSquad() const
{
	return static_cast< AVP_ServerEntity_Squad_Alien* >( Entity );
}

void ToolsAVP_Entity_Squad_Alien::BehaviourChanged( int iThis )
{
	ToolsAVP_Entity_Squad_Alien* pxThis = reinterpret_cast< ToolsAVP_Entity_Squad_Alien* >( iThis );
	pxThis->GetAlienSquad()->m_xBehaviourData.SetParamsFromBehaviour();
	pxThis->GetAlienSquad()->m_xBehaviourData.m_uProtoBlueprintHashID = ASURA_HASH_ID_UNSET; // reset the prototype
	pxThis->SetEntityModified();
	pxThis->RequestRefreshOverlay();
}

void ToolsAVP_Entity_Squad_Alien::DropDownListChanged( int iThis )
{
	ToolsAVP_Entity_Squad_Alien* pxThis = reinterpret_cast< ToolsAVP_Entity_Squad_Alien* >( iThis );
	pxThis->SetEntityModified();
	pxThis->RequestRefreshOverlay();
}

void ToolsAVP_Entity_Squad_Alien::TickBoxChanged( int iThis )
{
	ToolsAVP_Entity_Squad_Alien* pxThis = reinterpret_cast< ToolsAVP_Entity_Squad_Alien* >( iThis );
	pxThis->SetEntityModified();
	pxThis->RequestRefreshOverlay();
}


void ToolsAVP_Entity_Squad_Alien::ChooseBehaviourBP( int iThis )
{
	ToolsAVP_Entity_Squad_Alien* pxThis = reinterpret_cast< ToolsAVP_Entity_Squad_Alien* >( iThis );
	if ( !pxThis ) { return; }

	AVP_ServerEntity_Squad_Alien* pxSquad = pxThis->GetAlienSquad();

	Asura_Hash_ID uBlueprintTypeHashID = uAVP_BLUEPRINT_TYPE_AI_SETTINGS;
	Asura_Hash_ID uParentBlueprintHashID = ASURA_HASH_ID_UNSET;

	switch (pxSquad->GetBehaviourData().m_iBehaviour)
	{
	case AVP_AlienBehaviourData::AGGRESIVE:
		uParentBlueprintHashID = g_uAVP_BLUEPRINT_HASH_AGGRESSIVE;
		break;
	case AVP_AlienBehaviourData::HIDE_IN_SHADOWS:
		uParentBlueprintHashID = g_uAVP_BLUEPRINT_HASH_HIDEINSHADOWS;
	case AVP_AlienBehaviourData::AMBUSH:
		uParentBlueprintHashID = g_uAVP_BLUEPRINT_HASH_AGGRESSIVE;
		break;
	}

	Asura_Hash_ID uBlueprintResultHashID = pxSquad->GetBehaviourData().m_uProtoBlueprintHashID;
	Asura_Hash_ID uBlueprintTypeResultHashID = uAVP_BLUEPRINT_TYPE_AI_SETTINGS;

	BlueprintSelect_Overlay xBPOverlay( "Choose A Blueprint", &uBlueprintResultHashID, &uBlueprintTypeResultHashID, 1, &uBlueprintTypeHashID, 1, &uParentBlueprintHashID );
	if ( !xBPOverlay.DoModal() ) { return; }

	// It's our job now to iterate through all of the selected tools objects (of the correct type) and set this as the blueprint hash ID.
	for ( LIF< Tools_Object* > xObjectIter( &Tools_Object_System::SelectedObjectList ); !xObjectIter.done(); xObjectIter.next() )
	{
		ToolsAVP_Entity_Squad_Alien* pxOtherTools = static_cast<ToolsAVP_Entity_Squad_Alien*>(GetList()->FindEntity( xObjectIter()->GetGuid() ));
		if ( !pxOtherTools || pxOtherTools->Type != TOT_Entity_AlienSquad ) { continue; }

		pxOtherTools->GetAlienSquad()->m_xBehaviourData.m_uProtoBlueprintHashID = uBlueprintResultHashID;
		// Also update the properties of the object at this point.
		if (uBlueprintResultHashID != ASURA_HASH_ID_UNSET)
		{
			pxOtherTools->GetAlienSquad()->m_xBehaviourData.SetParamsFromBlueprint();
		}
	}

	pxThis->SetEntityModified();
	pxThis->RequestRefreshOverlay();
}

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////
