
///////////////////////////////////////////////////////////////////////////////
//
// File   : ToolsAVP_Entity_FeedingZone.cpp
// Author : Stuart Perry
// Date   : 20-Mar-2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "Entity/FeedingZone/ToolsAVP_Entity_FeedingZone.h"

// Asura independent header-files...
#include "Camera/Asura_Camera.h"
#include "Mesh/Asura_Mesh_System.h"
#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Overlay.h"
#include "Rendering/Asura_Render.h"
#include "Network/Asura_Message.h"

// Tools header-files...
#include "Load&Save/Tools_SaveAsura.h"
#include "Misc/Tools_Timed_Messages.h"
#include "Objects/Tools_Object_System.h"
#include "Objects/ToolsAVP_Object_Type.h"

////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Private functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Protected functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Public functions...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Function     : ToolsAVP_Entity_FeedingZone::ToolsAVP_Entity_FeedingZone
// Author       : Stuart Perry
// Date         : 20-Mar-2007
////////////////////////////////////////////////////////////////////////////////

ToolsAVP_Entity_FeedingZone::ToolsAVP_Entity_FeedingZone(Asura_Entity* pxEntity)
: PARENT(pxEntity)
{
	Type	= TOT_Entity_FeedingZone;

	AVP_ServerEntity_FeedingZone *pxAVPEntity = (AVP_ServerEntity_FeedingZone*) pxEntity;

	// Initialise local variables with those from AVP_ServerEntity_FeedingZone.
	m_bActive = pxAVPEntity->GetActive();

	Asura_String mesh_name = "ToolsShape_AlienFeedingZone";
	Mesh = Asura_Mesh_System::GetMesh(mesh_name);
}

/// End of function 'ToolsAVP_Entity_FeedingZone::ToolsAVP_Entity_FeedingZone'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : ToolsAVP_Entity_FeedingZone::Project_Create
// Author       : Stuart Perry
// Date         : 20-Mar-2007
////////////////////////////////////////////////////////////////////////////////

void ToolsAVP_Entity_FeedingZone::Project_Create()
{
	ToolsAVP_Entity_FeedingZone::Create();
}

/// End of function 'ToolsAVP_Entity_FeedingZone::Project_Create'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : ToolsAVP_Entity_FeedingZone::Create
// Author       : Stuart Perry
// Date         : 20-Mar-2007
////////////////////////////////////////////////////////////////////////////////

Tools_Object* ToolsAVP_Entity_FeedingZone::Create( Asura_Entity* pxEntity )
{
	return new ToolsAVP_Entity_FeedingZone( pxEntity );
}

/// End of function 'ToolsAVP_Entity_FeedingZone::Create'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : ToolsAVP_Entity_FeedingZone::Create
// Author       : Stuart Perry
// Date         : 20-Mar-2007
////////////////////////////////////////////////////////////////////////////////

void ToolsAVP_Entity_FeedingZone::Create()
{

	const Asura_Guid	uGuid = Asura_Entity_System::GetCreatedByToolsGuid();

	// Set up tools data and auto-create a unique name (prompt for new name if there is a clash).
	Tools_Entity_Data *pxToolsData = CreateToolsDataWithDefaultName("FeedingZone", "FeedingZone", uGuid);

	if (pxToolsData == NULL)
	{
		Asura_Assert(false, "ToolsAVP_Entity_FeedingZone::Create() - no tools data");
	}
	else
	{
		AVP_ServerEntity_FeedingZone *pxServerObject = new AVP_ServerEntity_FeedingZone(uGuid);

		if (pxServerObject == NULL)
		{
			Asura_Assert(false, "AVP_ServerEntity_FeedingZone::Create() - no server object" );
		}
		else
		{
			ToolsAVP_Entity_FeedingZone *pxToolsObject = new ToolsAVP_Entity_FeedingZone(pxServerObject);

			pxServerObject->ToolsData = pxToolsData;
		
			// Create the game specific tools object and select it.
			if (pxToolsObject == NULL)
			{
				Asura_Assert(false, "ToolsAVP_Entity_FeedingZone::Create() - no tools object");
			}
			else
			{
				Tools_Object_System::SelectObject(pxToolsObject);

				pxToolsObject->Position		= *Asura_Camera::GetPosition();
				pxToolsObject->Orientation.Identity();//	= *Asura_Camera::GetOrientation();
				
				Tools_Timed_Message_System::AddMessage(0xffffff, "Created new FeedingZone");
				Tools_SaveAsura::SetEntityModified();
			}
		}
	}
}

/// End of function 'ToolsAVP_Entity_FeedingZone::Create'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : ToolsAVP_Entity_FeedingZone::UpdatePositionInAsuraData
// Author       : Stuart Perry
// Date         : 20-Mar-2007
////////////////////////////////////////////////////////////////////////////////

void ToolsAVP_Entity_FeedingZone::UpdatePositionInAsuraData(bool bUpdateLinkedObjects)
{

	AVP_ServerEntity_FeedingZone*	pxAsuraEntity	= static_cast<AVP_ServerEntity_FeedingZone*>(Entity);
	Asura_Quat								xOrientation(Orientation);

	if((pxAsuraEntity->m_xPosition != Position) || (pxAsuraEntity->m_xOrientation != xOrientation))
	{
		pxAsuraEntity->SetOrientation(xOrientation);
		pxAsuraEntity->SetPosition(Position);

		Changed	= true;
	}

	PARENT::UpdatePositionInAsuraData(bUpdateLinkedObjects);
}

/// End of function 'ToolsAVP_Entity_FeedingZone::UpdatePositionInAsuraData'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : ToolsAVP_Entity_FeedingZone::UpdateWhileSelected
// Author       : Stuart Perry
// Date         : 20-Mar-2007
////////////////////////////////////////////////////////////////////////////////

void ToolsAVP_Entity_FeedingZone::UpdateWhileSelected()
{
	AVP_ServerEntity_FeedingZone* pxAVPEntity = (AVP_ServerEntity_FeedingZone*)Entity;

	if(Changed)
	{
		//update the project data
		pxAVPEntity->SetActive(m_bActive);
	}

	Tools_Entity_Object::UpdateWhileSelected();
}

/// End of function 'ToolsAVP_Entity_FeedingZone::UpdateWhileSelected'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : ToolsAVP_Entity_FeedingZone::AddOverlayControls
// Author       : Stuart Perry
// Date         : 20-Mar-2007
////////////////////////////////////////////////////////////////////////////////

void ToolsAVP_Entity_FeedingZone::AddOverlayControls(class Asura_Overlay* pxOverlay,class Asura_Vector_2& xOffset)
{
	Tools_Entity_Object::AddOverlayControls(pxOverlay,xOffset);
	Asura_Dialog* pxDialog;

	// Add our own controls...
	xOffset.y += Asura_Dialog::SmallSpacing();

	pxDialog = new Asura_Tickbox_Dialog("Active",xOffset,&m_bActive,&Changed);
	pxDialog->SetHelpText("This feeding zone starts off active.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();
}

/// End of function 'ToolsAVP_Entity_FeedingZone::AddOverlayControls'. ///

/// End of file. ///