//
// ToolsAVP_Entity_GameMode.cpp
//
// Created by JG - 04 July 2008
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

// This file's header.
#include "Entity/GameMode/ToolsAVP_Entity_GameMode.h"

// Entities
#include "Entity/GameMode/AVP_ServerEntity_GameMode.h"

// Engine
#include "Entity/Asura_Entity_Event.h"
#include "Mesh/Asura_Mesh_System.h"

// Project
#include "Core/AVP_Settings.h"

// Tools
#include "Objects/ToolsAVP_Object_Type.h"
#include "Load&Save/Tools_SaveAsura.h"
#include "Misc/Tools_Timed_Messages.h"

void ToolsAVP_Entity_GameMode::Project_Create()
{
	ToolsAVP_Entity_GameMode::CreateAndReturn(); 
}

ToolsAVP_Entity_GameMode::ToolsAVP_Entity_GameMode(Asura_Entity* pxEntity) 
: PARENT(pxEntity)
, m_iGameMode( 0 )
, m_bGameModeChanged( false )
{
	Type = TOT_Entity_GameMode;
	Mesh = Asura_Mesh_System::GetMesh("ToolsShape_MPMode");

	AVP_ServerEntity_GameMode* pxServerEntity = static_cast< AVP_ServerEntity_GameMode* >( pxEntity );
	Position = pxServerEntity->GetPosition();
	m_iGameMode = pxServerEntity->GetGameMode(); 
}

ToolsAVP_Entity_GameMode::~ToolsAVP_Entity_GameMode(void)
{
}

Tools_Object* ToolsAVP_Entity_GameMode::CreateAndReturn(void)
{
	const Asura_Guid	uGuid	= Asura_Entity_System::GetCreatedByToolsGuid();

	// Set up tools data and auto-create a unique name (prompt for new name if there is a clash).
	Tools_Entity_Data* pxToolsData = CreateToolsDataWithDefaultName("Game Mode", "Game Mode", uGuid);

	if (pxToolsData == NULL)
	{
		Asura_Assert(false, "ToolsAVP_Entity_GameMode::Create() - no tools data");
	}
	else
	{
		AVP_ServerEntity_GameMode* pxServerObject = new AVP_ServerEntity_GameMode(uGuid);

		if (pxServerObject == NULL)
		{
			Asura_Assert(false, "ToolsAVP_Entity_GameMode::Create() - no server object" );
		}
		else
		{
			pxServerObject->ToolsData = pxToolsData;

			ToolsAVP_Entity_GameMode*	pxToolsObject	= new ToolsAVP_Entity_GameMode(pxServerObject);

			// Create the game specific tools object and select it.
			if (pxToolsObject == NULL)
			{
				Asura_Assert(false, "ToolsAVP_Entity_GameMode::Create() - no tools object");
			}
			else
			{
				Tools_Object_System::SelectObject(pxToolsObject);

				Asura_Matrix_3x3 xMat;

				xMat.Identity();

				pxToolsObject->Position		= *Asura_Camera::GetPosition();
				pxToolsObject->Orientation  = xMat;
				pxToolsObject->m_uGuid		= uGuid;

				pxServerObject->SetPosition(pxToolsObject->Position);

				Tools_Timed_Message_System::AddMessage(0xffffff, "Created new Game Mode node");
				Tools_SaveAsura::SetEntityModified();

				return pxToolsObject; 
			}
		}
	}

	return NULL; 
}

Tools_Object* ToolsAVP_Entity_GameMode::Create(Asura_Entity* pxEntity)
{
	return new ToolsAVP_Entity_GameMode(pxEntity);
}
Tools_Object* ToolsAVP_Entity_GameMode::CreateObjectOfThisType() const
{
	return CreateAndReturn();
}

void ToolsAVP_Entity_GameMode::AddOverlayControls( Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset )
{
	Asura_Dialog* pxNewDialog = new Asura_DropDown_List_Dialog(xOffset, AVP_Settings::GetGameTypeNames(), &m_iGameMode, AVP_Settings::AVP_NUM_GAMETYPES, AVP_Settings::AVP_NUM_GAMETYPES, NULL, &m_bGameModeChanged ); 
	pxNewDialog->SetHelpText("Game mode controlled by this node");
	pxOverlay->AddDialog(pxNewDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

 	PARENT::AddOverlayControls(pxOverlay, xOffset); 
}

void ToolsAVP_Entity_GameMode::UpdateWhileSelected()
{
	PARENT::UpdateWhileSelected(); 

	if (m_pxServerEntity)
	{
		AVP_ServerEntity_GameMode* pxServerEntity = static_cast<AVP_ServerEntity_GameMode*>( m_pxServerEntity ); 
		pxServerEntity->SetPosition( Position );

		if (m_bGameModeChanged)
		{
			pxServerEntity->SetGameMode( static_cast<AVP_Settings::GameType>(m_iGameMode) );
			m_bGameModeChanged = false; 
			
			Tools_SaveAsura::SetEntityModified(); 
		}
	}
}

bool ToolsAVP_Entity_GameMode::HandlesThisEvent(int iEventID) const
{
	switch(iEventID)
	{
		case ASURA_ENTITY_EVENT_ONACTIVATION:
		case ASURA_ENTITY_EVENT_ONDEACTIVATION:
		{
			// We want to handle activation / deactivation messages
			return true;
		}
		default:
		{
			return false;
		}
	}
}
