//----------------------------------------------------------------------------
//	File		:	ToolsAVP_Entity_CoverpointNode.cpp
//	Created		:	27 / 02 / 08
//	Author		:	Gareth Thomas
//	Copyright (C) 2006 Rebellion, All Rights Reserved.
//----------------------------------------------------------------------------


//----------------------------------------------------------------------------
// INCLUDES
//----------------------------------------------------------------------------
// Core
#include "core/Asura.h"

// This Files Header
#include "Entity/Locations/OrientedNode/CoverpointNode/ToolsAVP_Entity_CoverpointNode.h"

// asura
#include "Animation/Asura_Animation.h"
#include "Animation/Asura_AnimationController_Animation.h"

// AVP

// Tools
#include "Hierarchy/Tools_Animation_Files.h"
#include "Load&Save/Tools_SaveAsura.h"
#include "Misc/Tools_Render.h"
#include "Misc/Tools_Timed_Messages.h"
#include "ToolsAVP_ChooseObjects.h"

//----------------------------------------------------------------------------
// Statics
//----------------------------------------------------------------------------
ToolsAVP_Entity_CoverpointNode* ToolsAVP_Entity_CoverpointNode::s_pxSelectedCoverpointNode = NULL;

const char* ToolsAVP_Entity_CoverpointNode::s_aszLowCoverTypeNames[ToolsAVP_Entity_CoverpointNode::MAX_LOW_COVER_TYPES] =
{
	"LOW COVER",
	"MID COVER"
};

//----------------------------------------------------------------------------
// Function		ToolsAVP_Entity_CoverpointNode :: Constructor
//----------------------------------------------------------------------------
ToolsAVP_Entity_CoverpointNode::ToolsAVP_Entity_CoverpointNode(Asura_Entity* pxEntity)
: PARENT(pxEntity)
, m_uNumLinkedCoverPoints(0)
, m_bLinkChanged(false)
, m_iLowCoverType(0)
, m_bCoverTypeChanged(false)
, m_bPreviewRunning(false)
, m_bPreviewStarted(false)
, m_fTimer(0.0f)
, m_uAnim(0)
, m_pxPreview(NULL)
, m_xPreviewPosition(Asura_Maths::ZeroVector3)
{
	Type = TOT_Entity_AVPCoverpointNode;

	for (u_int u = 0; u < MAX_LINKED_COVERPOINTS; ++u)
	{
		m_auLinkedCoverPointGuids[u] = ASURA_GUID_UNREGISTERED;
	}

	AVP_ServerNode_CoverpointNode* pxAVPEntity = static_cast <AVP_ServerNode_CoverpointNode*> (pxEntity);

	if (pxAVPEntity)
	{
		m_uNumLinkedCoverPoints = pxAVPEntity->GetNumLinkedCoverPoints();

		for (u_int u = 0; u < MAX_LINKED_COVERPOINTS; ++u)
		{
			m_auLinkedCoverPointGuids[u] = pxAVPEntity->GetLinkedCoverPointGuid(u);
		}

		m_bLowCoverLastFrame = pxAVPEntity->GetIsLowCover();
		m_iLowCoverType = pxAVPEntity->GetLowCoverType();
	}
}

//----------------------------------------------------------------------------
// Function		ToolsAVP_Entity_CoverpointNode :: Deconstructor
//----------------------------------------------------------------------------
ToolsAVP_Entity_CoverpointNode::~ToolsAVP_Entity_CoverpointNode()
{
	delete m_pxPreview;
}

//----------------------------------------------------------------------------
// Function		ToolsAVP_Entity_CoverpointNode :: AddOverlayControls
//----------------------------------------------------------------------------
void ToolsAVP_Entity_CoverpointNode::AddOverlayControls(Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset)
{
	PARENT::AddOverlayControls(pxOverlay, xOffset);

	Asura_Dialog* pxDialog;

	xOffset.y += Asura_Dialog::DefaultSpacing();

	AVP_ServerNode_CoverpointNode* pxCoverPoint = (AVP_ServerNode_CoverpointNode*)Entity;

	if (pxCoverPoint)
	{
		if (pxCoverPoint->GetIsLowCover())
		{
			pxDialog = new Asura_StringDisplay_Dialog("Low Cover Type:", xOffset);
			pxOverlay->AddDialog(pxDialog);
			xOffset.y += Asura_Dialog::DefaultSpacing();

			pxDialog = new Asura_DropDown_List_Dialog(xOffset, GetLowCoverTypeNames(), &m_iLowCoverType, ToolsAVP_Entity_CoverpointNode::MAX_LOW_COVER_TYPES, ToolsAVP_Entity_CoverpointNode::MAX_LOW_COVER_TYPES, NULL, &m_bCoverTypeChanged, "");
			pxDialog->SetHelpText("The Type of Low Cover");
			pxOverlay->AddDialog(pxDialog);
			xOffset.y += Asura_Dialog::LargeSpacing();
		}
	}

	pxDialog = new Asura_StringDisplay_Dialog("Linked Cover Points:", xOffset);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	if (m_uNumLinkedCoverPoints <= MAX_LINKED_COVERPOINTS)
	{
		pxDialog = new Asura_Function_Dialog("Add Linked Cover Point", xOffset, AddCoverPointLink, NULL, NULL);
		pxDialog->SetHelpText("Add A Cover Point Link.");
		pxOverlay->AddDialog(pxDialog);
		xOffset.y += Asura_Dialog::DefaultSpacing();
	}

	if (m_uNumLinkedCoverPoints > 0)
	{
		pxDialog = new Asura_Function_Dialog("Remove Linked CoverPoint", xOffset, RemoveCoverPointLink, NULL, NULL);
		pxDialog->SetHelpText("Remove A Cover Point Link.");
		pxOverlay->AddDialog(pxDialog);
		xOffset.y += Asura_Dialog::DefaultSpacing();
	}

	char szText[256] = "";
	Asura_PrintfToBfr(szText, "Linked Cover Points: ( %u / %u )", 256, true, m_uNumLinkedCoverPoints, MAX_LINKED_COVERPOINTS);
	pxDialog = new Asura_StringDisplay_Dialog(szText, xOffset);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();

	// Preview
	pxDialog = new Asura_Function_Dialog( "Start Preview", xOffset, StartPreview );
	pxDialog->SetHelpText( "Starts Previewing The Cover" );
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog( "Stop Preview", xOffset, StopPreview );
	pxDialog->SetHelpText( "Stops Previewing The Cover" );
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();
}

//----------------------------------------------------------------------------
// Function		ToolsAVP_Entity_CoverpointNode :: IsBaseOrDerivativeOf
//----------------------------------------------------------------------------
void ToolsAVP_Entity_CoverpointNode::StartPreview(void)
{
	s_pxSelectedCoverpointNode->m_bPreviewRunning = true;
	s_pxSelectedCoverpointNode->m_fTimer = 0.0f;
}

//----------------------------------------------------------------------------
// Function		ToolsAVP_Entity_CoverpointNode :: IsBaseOrDerivativeOf
//----------------------------------------------------------------------------
void ToolsAVP_Entity_CoverpointNode::StopPreview(void)
{
	s_pxSelectedCoverpointNode->m_bPreviewRunning = false;
	s_pxSelectedCoverpointNode->m_fTimer = 0.0f;
	s_pxSelectedCoverpointNode->m_bPreviewStarted = false;

	delete s_pxSelectedCoverpointNode->m_pxPreview;
	s_pxSelectedCoverpointNode->m_pxPreview = NULL;
}

//----------------------------------------------------------------------------
// Function		ToolsAVP_Entity_CoverpointNode :: IsBaseOrDerivativeOf
//----------------------------------------------------------------------------
bool ToolsAVP_Entity_CoverpointNode::IsBaseOrDerivativeOf(int iToolsObjectType) const
{
	if (iToolsObjectType == TOT_Entity_AVPCoverpointNode)
	{
		return true;
	}

	return PARENT::IsBaseOrDerivativeOf(iToolsObjectType);
}

//----------------------------------------------------------------------------
// Function		ToolsAVP_Entity_CoverpointNode :: OnSelection
//----------------------------------------------------------------------------
void ToolsAVP_Entity_CoverpointNode::OnSelection(void)
{
	PARENT::OnSelection();

	s_pxSelectedCoverpointNode = this;
}

//----------------------------------------------------------------------------
// Function		ToolsAVP_Entity_CoverpointNode :: OnDeselection
//----------------------------------------------------------------------------
void ToolsAVP_Entity_CoverpointNode::OnDeselection(void)
{
	PARENT::OnDeselection();

	s_pxSelectedCoverpointNode = NULL;

	m_bPreviewRunning = false;
	m_bPreviewStarted = false;
	SetMesh("ToolsShape_Coverpoint");
	delete m_pxPreview;
	m_pxPreview = NULL;
	m_fTimer = 0.0f;
}

//----------------------------------------------------------------------------
// Function		ToolsAVP_Entity_CoverpointNode :: Render
//----------------------------------------------------------------------------
void ToolsAVP_Entity_CoverpointNode::Render(void)
{
	PARENT::Render();

	if (m_uNumLinkedCoverPoints > 0 && IsRootObjectGroupSelected())
	{
		for (u_int u = 0; u < MAX_LINKED_COVERPOINTS; ++u)
		{
			if (m_auLinkedCoverPointGuids[u] != ASURA_GUID_UNREGISTERED)
			{
				ToolsAVP_Entity_CoverpointNode* pxCoverPoint = (ToolsAVP_Entity_CoverpointNode*)ToolsAVP_Entity_CoverpointNode::FindEntity(m_auLinkedCoverPointGuids[u]);

				if (pxCoverPoint)
				{
					Tools_Render::RenderCylinder(Position, pxCoverPoint->Position, 0.05f, 0xff00ff00, false);
				}
			}
		}
	}

	if (m_pxPreview)
	{
		m_pxPreview->Update(m_xPreviewPosition, Orientation);
	}
}

//----------------------------------------------------------------------------
// Function		ToolsAVP_Entity_CoverpointNode :: Update
//----------------------------------------------------------------------------
void ToolsAVP_Entity_CoverpointNode::Update(void)
{
	PARENT::Update();

	bool bRemoved = false;

	// Clean up any links that have been destroyed!
	for (u_int u = 0; u < MAX_LINKED_COVERPOINTS; ++u)
	{
		if (m_auLinkedCoverPointGuids[u] != ASURA_GUID_UNREGISTERED)
		{
			Tools_Entity_Object* pxCPNode = ToolsAVP_Entity_CoverpointNode::FindEntity(m_auLinkedCoverPointGuids[u]);

			if (!pxCPNode || bRemoved)
			{
				bRemoved = true;
				m_bLinkChanged = true;

				if (u >= (MAX_LINKED_COVERPOINTS - 1))
				{
					m_auLinkedCoverPointGuids[u] = ASURA_GUID_UNREGISTERED;
				}
				else
				{
					m_auLinkedCoverPointGuids[u] = m_auLinkedCoverPointGuids[u + 1];
				}
			}
		}
	}

	u_int uCount = 0;

	for (u_int i = 0; i < MAX_LINKED_COVERPOINTS; i += 1)
	{
		if (m_auLinkedCoverPointGuids[i] != ASURA_GUID_UNREGISTERED)
		{
			uCount += 1;
		}
	}

	m_uNumLinkedCoverPoints = uCount;

	if (m_bLinkChanged)
	{
		AVP_ServerNode_CoverpointNode* pxAVPEntity = (AVP_ServerNode_CoverpointNode*)Entity;

		if (pxAVPEntity)
		{
			pxAVPEntity->SetNumLinkedCoverPoints(m_uNumLinkedCoverPoints);

			for (u_int u = 0; u < MAX_LINKED_COVERPOINTS; u += 1)
			{
				pxAVPEntity->SetLinkedCoverPointGuid(u, m_auLinkedCoverPointGuids[u]);
			}
		}

		m_bLinkChanged = false;

		if (this == s_pxSelectedCoverpointNode)
		{
			DeleteOverlay();
			CreateOverlay();
		}

		Tools_SaveAsura::SetEntityModified();
	}
}

//----------------------------------------------------------------------------
// Function		ToolsAVP_Entity_CoverpointNode :: UpdateWhileSelected
//----------------------------------------------------------------------------
void ToolsAVP_Entity_CoverpointNode::UpdateWhileSelected(void)
{
	PARENT::UpdateWhileSelected();

	if (m_bCoverTypeChanged)
	{
		AVP_ServerNode_CoverpointNode* pxCover = (AVP_ServerNode_CoverpointNode*)Entity;

		pxCover->SetLowCoverType(m_iLowCoverType);

		m_bCoverTypeChanged = false;
		DeleteOverlay();
		CreateOverlay();
	}

	AVP_ServerNode_CoverpointNode* pxCover = (AVP_ServerNode_CoverpointNode*)Entity;

	if (pxCover)
	{
		if ( ( m_bLowCoverLastFrame && !pxCover->GetIsLowCover() ) || ( !m_bLowCoverLastFrame && pxCover->GetIsLowCover() ) )
		{
			m_bLowCoverLastFrame = pxCover->GetIsLowCover();
			DeleteOverlay();
			CreateOverlay();
		}
	}


	if ( m_bPreviewRunning )
	{
		DoPreview();
	}
	else
	{
		m_bPreviewStarted = false;
	}
}

bool ToolsAVP_Entity_CoverpointNode::IsToTheLeft()
{
	AVP_ServerNode_CoverpointNode* pxCoverPoint = (AVP_ServerNode_CoverpointNode*)Entity;
	const Asura_Vector_3&	xEmergePos			= pxCoverPoint->GetEmergePointPosition( 0 );
	const Asura_Vector_3&	xCoverPos			= pxCoverPoint->GetPosition();
	Asura_Matrix_3x3		xCoverOri;			pxCoverPoint->GetOrientation(xCoverOri);
	Asura_Vector_3			xLeft(xCoverOri.e11, xCoverOri.e21, xCoverOri.e31);
	Asura_Vector_3			xDirToEmergePoint	= xEmergePos - xCoverPos;
	float					fDistOrigToPlane	= 0.0f;
	float					fDistToPlane		= 0.0f;

	xDirToEmergePoint.Normalise();
	fDistOrigToPlane = xLeft * xCoverPos;
	fDistToPlane = fDistOrigToPlane - (xLeft * xEmergePos);

	if (fDistToPlane <= 0.0f)
	{
		return true; // is to the left
	}
	else
	{
		return false; // is to the right
	}
}

//----------------------------------------------------------------------------
void ToolsAVP_Entity_CoverpointNode::DoPreview()
{
	Asura_Hash_ID uSkinHash = 0xbf8c8dc2; //"Marine"
	Asura_Hash_ID uAnimHash = ASURA_HASH_ID_UNSET;

	m_fTimer += Asura_Timers::GetFrameTime();

	if (m_fTimer > 2.0f)
	{
		m_bPreviewStarted = false;
		m_fTimer = 0.0f;
		++m_uAnim;

		if (m_uAnim >= 4)
		{
			m_uAnim = 0;
		}
	}

	AVP_ServerNode_CoverpointNode* pxCoverPoint = (AVP_ServerNode_CoverpointNode*)Entity;

	if (pxCoverPoint)
	{
		if (!pxCoverPoint->GetIsLowCover())
		{
			bool bIsToTheLeft = IsToTheLeft();

			if (bIsToTheLeft)
			{
				switch (m_uAnim)
				{
					case 0: uAnimHash = 0x017c0f8a; break; //"marine_standingcoverA_idle_thwn"
					case 1: uAnimHash = 0xa78ab7b6; break; //"marine_standingcoverA_peek_left_idle_thwn"
					case 2: uAnimHash = 0x017c0f8a; break; //"marine_standingcoverA_idle_thwn"
					case 3: uAnimHash = 0xf6447533; break; //"marine_standingcoverA_into_stepout_left_thwn"
					default:break;
				}
			}
			else
			{
				switch (m_uAnim)
				{
					case 0: uAnimHash = 0x017c0f8a; break; //"marine_standingcoverA_idle_thwn"
					case 1: uAnimHash = 0x4c2e2e5b; break; //"marine_standingcoverA_peek_right_idle_thwn"
					case 2: uAnimHash = 0x017c0f8a; break; //"marine_standingcoverA_idle_thwn"
					case 3: uAnimHash = 0xecde0ede; break; //"marine_standingcoverA_into_stepout_right_thwn"
					default:break;
				}
			}
		}
		else
		{
			if ( m_iLowCoverType == MID_COVER )
			{
				if (pxCoverPoint->GetEmergePointCount() == 0)
				{
					switch (m_uAnim)
					{
						case 0: uAnimHash = 0x64d2fc3e; break; //"marine_midcoverA_idle"
						case 1: uAnimHash = 0x30ce5871; break; //"marine_midcoverA_popup_idle"
						case 2: uAnimHash = 0x64d2fc3e; break; //"marine_midcoverA_idle"
						case 3: uAnimHash = 0x30ce5871; break; //"marine_midcoverA_popup_idle"
						default:break;
					}
				}
				else
				{
					bool bIsToTheLeft = IsToTheLeft();

					if (bIsToTheLeft)
					{
						switch (m_uAnim)
						{
							case 0: uAnimHash = 0x64d2fc3e; break; //"marine_midcoverA_idle"
							case 1: uAnimHash = 0xa7cfbb12; break; //"marine_midcoverA_peek_left_idle"
							case 2: uAnimHash = 0x64d2fc3e; break; //"marine_midcoverA_idle"
							case 3: uAnimHash = 0xa9082479; break; //"marine_midcoverA_into_stepout_left"
							default:break;
						}
					}
					else
					{
						switch (m_uAnim)
						{
							case 0: uAnimHash = 0x64d2fc3e; break; //"marine_midcoverA_idle"
							case 1: uAnimHash = 0x6d5372d1; break; //"marine_midcoverA_peek_right_idle"
							case 2: uAnimHash = 0x64d2fc3e; break; //"marine_midcoverA_idle"
							case 3: uAnimHash = 0x7852cbea; break; //"marine_midcoverA_into_stepout_right"
							default:break;
						}
					}
				}
			}
			else // low cover
			{
				switch (m_uAnim)
				{
					case 0: uAnimHash = 0xac369fb2; break; //"marine_lowcoverA_idle"
					case 1: uAnimHash = 0x3d55cbb0; break; //"marine_lowcoverA_low_popup_idle"
					case 2: uAnimHash = 0xac369fb2; break; //"marine_lowcoverA_idle"
					case 3: uAnimHash = 0x3d55cbb0; break; //"marine_lowcoverA_low_popup_idle"
					default:break;
				}
			}
		}
	}

	if ( !m_bPreviewStarted )
	{
		m_bPreviewStarted = true;
		delete m_pxPreview;
		m_pxPreview = NULL;
	}

	if ( !m_pxPreview )
	{
		Asura_Hierarchy_Skin*	pxSkin = Tools_Animation_Files::GetSkinFromID( uSkinHash );
		
		if ( pxSkin )
		{
			m_pxPreview = new Asura_Animation( pxSkin->Name.GetString() );
			Asura_AnimationController_Animation*	pxAnimation = NULL;
			const Asura_Hierarchy_Anim*				pxHierarchyAnim	= Tools_Animation_Files::GetAnimFromID( uAnimHash );

			if ( pxHierarchyAnim )
			{
				u_int uAnimFlags = AAC_FLAG_ACTIVE | AAC_FLAG_DESTROY_WHEN_FINISHED | AACA_FLAG_CALCULATE_VELOCITY | AACA_FLAG_CALCULATE_ANGULAR_VELOCITY | AACA_FLAG_CALCULATE_MOVEMENT_OFFSET | AAC_FLAG_ENSURE_UNIQUE_HASH | AACA_FLAG_LOOPED;
				const char* szAnimName = pxHierarchyAnim->GetName();
				pxAnimation = m_pxPreview->CreateControllerAnimation(szAnimName, uAnimFlags, 0x01, uAnimHash, NULL );
				if (pxAnimation)
				{
					pxAnimation->Play();
				}
				m_xPreviewPosition = Position;
			}
		}
	}
	else
	{
		m_xPreviewPosition = Position;
		Asura_Vector_3 xOffset = Asura_Maths::ZeroVector3;
		Asura_AnimationController_Animation* pxTemp = m_pxPreview->GetControllerAnimationFromHash( uAnimHash );
	
		if (pxTemp)
		{
			pxTemp->GetAsuraAnimation()->GetMovementOffset(&xOffset);
			const Asura_Quat& xQuat = pxTemp->GetAsuraAnimation()->GetOrientQuat();
			xOffset = xQuat.RotateVector(xOffset);
		}

		m_xPreviewPosition += xOffset;
	}
}

//----------------------------------------------------------------------------
// Function		ToolsAVP_Entity_CoverpointNode :: Copy
//----------------------------------------------------------------------------
void ToolsAVP_Entity_CoverpointNode::Copy(const Tools_Object* pxObject)
{
	PARENT::Copy(pxObject);
}

//----------------------------------------------------------------------------
// Function		ToolsAVP_Entity_CoverpointNode :: CreateObjectOfThisType
//----------------------------------------------------------------------------
Tools_Object* ToolsAVP_Entity_CoverpointNode::CreateObjectOfThisType(void) const
{
	return CreateAndReturn();
}

//----------------------------------------------------------------------------
// Function		ToolsAVP_Entity_CoverpointNode :: Create
//----------------------------------------------------------------------------
void ToolsAVP_Entity_CoverpointNode::Create()
{
	const Asura_Guid uGuid = Asura_Entity_System::GetCreatedByToolsGuid();

	Tools_Entity_Data *pxToolsData = CreateToolsDataWithDefaultName("Cover Point", "Cover Point", uGuid);

	if (pxToolsData == NULL)
	{
		Asura_Assert(false, "ToolsAVP_Entity_CoverpointNode::Create() - no tools data");
	}
	else
	{
		AVP_ServerNode_CoverpointNode* pxServerObject = new AVP_ServerNode_CoverpointNode(uGuid);

		if (pxServerObject == NULL)
		{
			Asura_Assert(false, "ToolsAVP_Entity_CoverpointNode::Create() - no server object" );
		}
		else
		{
			pxServerObject->ToolsData = pxToolsData;

			ToolsAVP_Entity_CoverpointNode* pxToolsObject = new ToolsAVP_Entity_CoverpointNode(pxServerObject);

			// Create the game specific tools object and select it.
			if (pxToolsObject == NULL)
			{
				Asura_Assert(false, "ToolsAVP_Entity_CoverpointNode::Create() - no tools object");
			}
			else
			{
				Tools_Object_System::SelectObject(pxToolsObject);

				pxToolsObject->Position		= *Asura_Camera::GetPosition();
				pxToolsObject->m_uGuid		= uGuid;

				pxServerObject->SetPosition(pxToolsObject->Position);

				Tools_Timed_Message_System::AddMessage(0xffffff, "Created New Cover Point Node");
				Tools_SaveAsura::SetEntityModified();
			}
		}
	}
}

//----------------------------------------------------------------------------
// Function		ToolsAVP_Entity_CoverpointNode :: Create1
//----------------------------------------------------------------------------
Tools_Object* ToolsAVP_Entity_CoverpointNode::Create(Asura_Entity* pxEntity)
{
	return new ToolsAVP_Entity_CoverpointNode(pxEntity);
}

//----------------------------------------------------------------------------
// Function		ToolsAVP_Entity_CoverpointNode :: CreateAndReturn
//----------------------------------------------------------------------------
ToolsAVP_Entity_CoverpointNode* ToolsAVP_Entity_CoverpointNode::CreateAndReturn()
{
	const Asura_Guid uGuid = Asura_Entity_System::GetCreatedByToolsGuid();

	Tools_Entity_Data *pxToolsData = CreateToolsDataWithDefaultName("Cover Point", "Cover Point", uGuid);

	if (pxToolsData == NULL)
	{
		Asura_Assert(false, "ToolsAVP_Entity_CoverpointNode::Create() - no tools data");
	}
	else
	{
		AVP_ServerNode_CoverpointNode* pxServerObject = new AVP_ServerNode_CoverpointNode(uGuid);

		if (pxServerObject == NULL)
		{
			Asura_Assert(false, "ToolsAVP_Entity_CoverpointNode::Create() - no server object" );
		}
		else
		{
			pxServerObject->ToolsData = pxToolsData;

			ToolsAVP_Entity_CoverpointNode* pxToolsObject = new ToolsAVP_Entity_CoverpointNode(pxServerObject);

			// Create the game specific tools object and select it.
			if (pxToolsObject == NULL)
			{
				Asura_Assert(false, "ToolsAVP_Entity_CoverpointNode::Create() - no tools object");
			}
			else
			{
				Tools_Object_System::SelectObject(pxToolsObject);

				pxToolsObject->Position		= *Asura_Camera::GetPosition();
				pxToolsObject->m_uGuid		= uGuid;

				pxServerObject->SetPosition(pxToolsObject->Position);

				Tools_Timed_Message_System::AddMessage(0xffffff, "Created New Cover Point Node");
				Tools_SaveAsura::SetEntityModified();
				return pxToolsObject;
			}
		}
	}

	return NULL;
}
//----------------------------------------------------------------------------
// Function		ToolsAVP_Entity_CoverpointNode :: AddCoverPointLink
//----------------------------------------------------------------------------
void ToolsAVP_Entity_CoverpointNode::AddCoverPointLink(int iIndex)
{
	ToolsAVP_ChooseObject_ByType* pxChooseObject = new ToolsAVP_ChooseObject_ByType(TOT_Entity_AVPCoverpointNode, ActuallyAddCoverPointLink);
	Tools_Object_System::ChooseObject("Select The Cover Point To Add Link", pxChooseObject);
}

//----------------------------------------------------------------------------
// Function		ToolsAVP_Entity_CoverpointNode :: RemoveCoverPointLink
//----------------------------------------------------------------------------
void ToolsAVP_Entity_CoverpointNode::RemoveCoverPointLink(int iIndex)
{
	ToolsAVP_ChooseObject_ByType* pxChooseObject = new ToolsAVP_ChooseObject_ByType(TOT_Entity_AVPCoverpointNode, ActuallyRemoveCoverPointLink);
	Tools_Object_System::ChooseObject("Select The Cover Point To Remove Link", pxChooseObject);
}

//----------------------------------------------------------------------------
// Function		ToolsAVP_Entity_CoverpointNode :: ActuallyAddCoverPointLink
//----------------------------------------------------------------------------
void ToolsAVP_Entity_CoverpointNode::ActuallyAddCoverPointLink(Asura_Guid uGuid)
{
	ToolsAVP_Entity_CoverpointNode* pxThisCoverPoint = s_pxSelectedCoverpointNode;

	u_int uMaxLinks = ToolsAVP_Entity_CoverpointNode::MAX_LINKED_COVERPOINTS;

	if (pxThisCoverPoint->m_uNumLinkedCoverPoints >= uMaxLinks)
	{
		Asura_Assert(false, "ToolsAVP_Entity_CoverpointNode::ActuallyAddCoverPointLink - Something has gone wrong trying to add a link when we are at the maximum");
		return;
	}

	if (pxThisCoverPoint->GetGuid() == uGuid)
	{
		return;
	}

	pxThisCoverPoint->m_auLinkedCoverPointGuids[pxThisCoverPoint->m_uNumLinkedCoverPoints] = uGuid;
	++pxThisCoverPoint->m_uNumLinkedCoverPoints;
	pxThisCoverPoint->m_bLinkChanged = true;

	ToolsAVP_Entity_CoverpointNode* pxCPToAddLinkTo = (ToolsAVP_Entity_CoverpointNode*)ToolsAVP_Entity_CoverpointNode::FindEntity(uGuid);
	AddLinkToCPNode(pxCPToAddLinkTo, pxThisCoverPoint->GetGuid());
}

//----------------------------------------------------------------------------
// Function		ToolsAVP_Entity_CoverpointNode :: ActuallyRemoveCoverPointLink
//----------------------------------------------------------------------------
void ToolsAVP_Entity_CoverpointNode::ActuallyRemoveCoverPointLink(Asura_Guid uGuid)
{
	ToolsAVP_Entity_CoverpointNode* pxThisCoverPoint = s_pxSelectedCoverpointNode;

	if (pxThisCoverPoint->m_uNumLinkedCoverPoints <= 0)
	{
		Asura_Assert(false, "ToolsAVP_Entity_CoverpointNode::ActuallyRemoveCoverPointLink - Something has gone wrong trying to Remove a link when we are at the minimum");
		return;
	}

	if (pxThisCoverPoint->GetGuid() == uGuid)
	{
		return;
	}

	pxThisCoverPoint->m_auLinkedCoverPointGuids[pxThisCoverPoint->m_uNumLinkedCoverPoints - 1];
	pxThisCoverPoint->m_auLinkedCoverPointGuids[pxThisCoverPoint->m_uNumLinkedCoverPoints - 1] = ASURA_GUID_UNREGISTERED;
	--pxThisCoverPoint->m_uNumLinkedCoverPoints;
	pxThisCoverPoint->m_bLinkChanged = true;

	ToolsAVP_Entity_CoverpointNode* pxCPToRemoveLinkFrom = (ToolsAVP_Entity_CoverpointNode*)ToolsAVP_Entity_CoverpointNode::FindEntity(uGuid);
	RemoveLinkFromCPNode(pxCPToRemoveLinkFrom, pxThisCoverPoint->GetGuid());
}

//----------------------------------------------------------------------
// ToolsAVP_Entity_CoverpointNode :: AddLinkToGPVolume
//----------------------------------------------------------------------
void ToolsAVP_Entity_CoverpointNode::AddLinkToCPNode(ToolsAVP_Entity_CoverpointNode* pxCPToAddTo, Asura_Guid uGuidToAdd)
{
	// if either aren't valid return
	if ((!pxCPToAddTo) || (uGuidToAdd == ASURA_GUID_UNREGISTERED))
	{
		return;
	}

	u_int uMaxLinks = ToolsAVP_Entity_CoverpointNode::MAX_LINKED_COVERPOINTS;

	if (pxCPToAddTo->m_uNumLinkedCoverPoints > uMaxLinks)
	{
		return;
	}

	pxCPToAddTo->m_auLinkedCoverPointGuids[pxCPToAddTo->m_uNumLinkedCoverPoints] = uGuidToAdd;
	++pxCPToAddTo->m_uNumLinkedCoverPoints;
	pxCPToAddTo->m_bLinkChanged = true;
}

//----------------------------------------------------------------------
// ToolsAVP_Entity_CoverpointNode :: AddLinkToGPVolume
//----------------------------------------------------------------------
void ToolsAVP_Entity_CoverpointNode::RemoveLinkFromCPNode(ToolsAVP_Entity_CoverpointNode* pxCPToRemoveFrom, Asura_Guid uGuidToRemove)
{
	// if either aren't valid return
	if ((!pxCPToRemoveFrom) || (uGuidToRemove == ASURA_GUID_UNREGISTERED))
	{
		return;
	}

	if (pxCPToRemoveFrom->m_uNumLinkedCoverPoints == 0)
	{
		return;
	}

	bool bFound = false;

	for (u_int u = 0; u < MAX_LINKED_COVERPOINTS; u += 1)
	{
		if ((pxCPToRemoveFrom->m_auLinkedCoverPointGuids[u] == uGuidToRemove) || (bFound))
		{
			bFound = true;

			if (u >= (MAX_LINKED_COVERPOINTS - 1))
			{
				pxCPToRemoveFrom->m_auLinkedCoverPointGuids[u] = ASURA_GUID_UNREGISTERED;
			}
			else
			{
				pxCPToRemoveFrom->m_auLinkedCoverPointGuids[u] = pxCPToRemoveFrom->m_auLinkedCoverPointGuids[u + 1];
			}
		}
	}

	if (bFound)
	{
		--pxCPToRemoveFrom->m_uNumLinkedCoverPoints;
		pxCPToRemoveFrom->m_bLinkChanged = true;
	}
}