//----------------------------------------------------------------------------
//	File		:	ToolsAVP_Entity_CoverpointNode.h
//	Created		:	27 / 02 / 08
//	Author		:	Gareth Thomas
//	Copyright (C) 2006 Rebellion, All Rights Reserved.
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// BEGIN FILE WRAPPER
#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif
#ifndef __TOOLSAVP_ENTITYNODE_COVERPOINTNODE_H__
#define __TOOLSAVP_ENTITYNODE_COVERPOINTNODE_H__ 1
// END FILE WRAPPER
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// INCLUDES
//----------------------------------------------------------------------------
// asura
#include "Entity/Locations/OrientedNode/CoverpointNode/AVP_ServerNode_CoverpointNode.h"

// tools
#include "Entity/Locations/OrientedNode/CoverpointNode/Tools_EntityNode_CoverpointNode.h"

#include "Objects/ToolsAVP_Object_Type.h"


class Asura_Animation;
//----------------------------------------------------------------------------
//	Class		:	ToolsAVP_Entity_CoverpointNode
//	Created		:	27 / 02 / 08
//	Author		:	Gareth Thomas
//----------------------------------------------------------------------------
class ToolsAVP_Entity_CoverpointNode : public Tools_EntityNode_CoverpointNode
{
	typedef Tools_EntityNode_CoverpointNode PARENT;

public:

	enum LowCoverTypes
	{
		LOW_COVER,
		MID_COVER,
		MAX_LOW_COVER_TYPES
	};

	// Construction / Deconstruction...
	ToolsAVP_Entity_CoverpointNode(Asura_Entity* pxEntity);
	virtual ~ToolsAVP_Entity_CoverpointNode();

	// Virtual Overridden Public Member Functions...
	virtual void			AddOverlayControls			(Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset);
	virtual bool			IsBaseOrDerivativeOf		(int iToolsObjectType) const;
	virtual void			OnSelection					(void);
	virtual void			OnDeselection				(void);
	virtual void			Render						(void);
	virtual void			Update						(void);
	virtual void			UpdateWhileSelected			(void);
	virtual void			Copy						(const Tools_Object* pxObject);
	virtual Tools_Object*	CreateObjectOfThisType		(void) const;

	virtual const char*		GetTypeDescription			(void)					{return "AVP Cover Point Node";}
	virtual bool			CanBeCopied					(void) const			{return Type == TOT_Entity_AVPCoverpointNode;}

	// Static Public Member Functions...
	static void				Create						(void);         //Create a new object in tools
	static Tools_Object*	Create						(Asura_Entity* pxEntity);

protected:

	static void AddCoverPointLink				(int iIndex);
	static void RemoveCoverPointLink			(int iIndex);
	static void ActuallyAddCoverPointLink		(Asura_Guid uGuid);
	static void ActuallyRemoveCoverPointLink	(Asura_Guid uGuid);
	static void AddLinkToCPNode					(ToolsAVP_Entity_CoverpointNode* pxCPToAddTo, Asura_Guid uGuidToAdd);
	static void RemoveLinkFromCPNode			(ToolsAVP_Entity_CoverpointNode* pxCPToRemoveFrom, Asura_Guid uGuidToRemove);

	static const char**	GetLowCoverTypeNames	( void )		{ return s_aszLowCoverTypeNames; }

	static void StartPreview(void);
	static void StopPreview(void);
	void DoPreview(void);
	bool	IsToTheLeft();

private:

	enum {MAX_LINKED_COVERPOINTS = 4};

	Asura_Guid	m_auLinkedCoverPointGuids[MAX_LINKED_COVERPOINTS];
	u_int		m_uNumLinkedCoverPoints;

	bool		m_bLinkChanged;
	int			m_iLowCoverType;
	bool		m_bCoverTypeChanged;

	bool		m_bLowCoverLastFrame;
	bool		m_bPreviewRunning;
	bool		m_bPreviewStarted;
	float		m_fTimer;
	u_int		m_uAnim;

	static const char* s_aszLowCoverTypeNames[MAX_LOW_COVER_TYPES];

	Asura_Animation* m_pxPreview;
	Asura_Vector_3 m_xPreviewPosition;

	// Private Static Member Functions...
	static ToolsAVP_Entity_CoverpointNode* CreateAndReturn();

	// Private Static Member Variables...
	static ToolsAVP_Entity_CoverpointNode* s_pxSelectedCoverpointNode;
};
/// End Of Class 'ToolsAVP_Entity_CoverpointNode'
//----------------------------------------------------------------------------
//----------------------------------------------------------------------------
#endif //__TOOLSAVP_ENTITYNODE_COVERPOINTNODE_H__
