//////////////////////////////////////////////////////////////////////////////
//
// ToolsAVP_Node_GS2Attractor.cpp
//
// Copyright (C) 2008 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"

// header
#include "Entity/Locations/OrientedNode/GamesceneNode/AttractorNode/ToolsAVP_Node_GS2Attractor.h"

// asura

// axon
#include "Behaviour/Axon_BehaviourParams.h"
#include "Behaviour/AVP_BehaviourParams.h"
#include "Behaviour/BehaviourTypes/Axon_Behaviour_Types.h"

// avp
#include "Entity/Locations/OrientedNode/GamesceneNode/AttractorNode/AVP_ServerNode_GS2Attractor.h"

// tools
#include "Entity/Tools_SecondaryEntityFile_System.h"
#include "Entity/Locations/OrientedNode/GamesceneNode/ToolsAVP_Node_GS2Node.h"
#include "Entity/Locations/Spline/GamesceneSpline/ToolsAVP_Node_GS2Spline.h"
#include "Load&Save/Tools_SaveAsura.h"
#include "Misc/Tools_Timed_Messages.h"
#include "Objects/ToolsAVP_Object_Type.h"
#include "Objects/Axon/Tools_Object_Axon_Interface.h"

//////////////////////////////////////////////////////////////////////////////
//
// ToolsAVP_Node_GS2Attractor
//
//////////////////////////////////////////////////////////////////////////////

ToolsAVP_Node_GS2Attractor::ToolsAVP_Node_GS2Attractor( Asura_Entity* pxEntity )
: Tools_EntityNode_GS2Node( pxEntity )
, ToolsAVP_Node_GS2Node( pxEntity )
, Tools_EntityNode_GS2Attractor( pxEntity )
{
	Type = TOT_Entity_AVPGS2Attractor;
}

bool ToolsAVP_Node_GS2Attractor::IsBaseOrDerivativeOf( int iToolsObjectType ) const
{
	if ( iToolsObjectType == TOT_Entity_AVPGS2Attractor )
	{
		return true;
	}
	else
	{
		return PARENT_AVPTOOLS::IsBaseOrDerivativeOf( iToolsObjectType );
	}
}

void ToolsAVP_Node_GS2Attractor::Render()
{
	PARENT_TOOLS::Render();
	PARENT_AVPTOOLS::Render();
}

bool ToolsAVP_Node_GS2Attractor::HandlesThisMessage( int iMessageID )
{
	bool bRet = true;
	bRet = PARENT_AVPTOOLS::HandlesThisMessage( iMessageID );
	bRet = PARENT_TOOLS::HandlesThisMessage( iMessageID );
	return bRet;
}

void ToolsAVP_Node_GS2Attractor::Copy( const Tools_Object* pxObject )
{
	// Copy parent data.
	PARENT_TOOLS::Copy( pxObject );
	PARENT_AVPTOOLS::Copy( pxObject );

	// Copy our data.
	// No extra data.

	// Copy the engine entity's data across.
	// No extra data.
}

Tools_Object* ToolsAVP_Node_GS2Attractor::CreateObjectOfThisType() const
{
	return CreateAndReturn();
}

void ToolsAVP_Node_GS2Attractor::Create()
{
	CreateAndReturn();
}

ToolsAVP_Node_GS2Attractor* ToolsAVP_Node_GS2Attractor::Create( Asura_Entity* pxEntity )
{
	return new ToolsAVP_Node_GS2Attractor( pxEntity );
}

ToolsAVP_Node_GS2Attractor* ToolsAVP_Node_GS2Attractor::CreateAndReturn()
{
	// Get a new GUID
	const Asura_Guid uGuid = Asura_Entity_System::GetCreatedByToolsGuid();

	// Set up tools data and auto-create a unique name (prompt for new name if there is a clash).
	Tools_Entity_Data* pxToolsData = CreateToolsDataWithDefaultName( "AVP GS2 Attractor", "AVP GS2 Attractor", uGuid );
	if ( !pxToolsData ) { return NULL; }

	// Create the game specfic server object and set its data.
	AVP_ServerNode_GS2Attractor* pxGameAttractor = new AVP_ServerNode_GS2Attractor( uGuid );
	if ( !pxGameAttractor ) { return NULL; }

	pxGameAttractor->ToolsData = pxToolsData;

	// Create the game specific tools object and select it.
	ToolsAVP_Node_GS2Attractor* pxToolsAttractor = new ToolsAVP_Node_GS2Attractor( pxGameAttractor );
	if ( !pxToolsAttractor ) { return NULL; }

	// Set everything to the default values.
	pxToolsAttractor->SetThisToDefaults();

	Tools_Object_System::SelectObject( pxToolsAttractor );

	pxToolsAttractor->Position = *Asura_Camera::GetPosition();
	// Set the orient to the camera heading but vertical.
	Asura_Euler_3 xHeading = *Asura_Camera::GetOriEuler();
	xHeading.x = xHeading.z = 0.0f;
	xHeading.ToMatrix( pxToolsAttractor->Orientation );

	pxGameAttractor->SetPosition( pxToolsAttractor->Position );
	pxGameAttractor->SetOrientation( pxToolsAttractor->Orientation );

	Tools_Timed_Message_System::AddMessage( Tools_Timed_Message_System::COLOUR_DEFAULT, "Created new GS2 attractor." );
	Tools_SaveAsura::SetEntityModified();

	return pxToolsAttractor;
}

void ToolsAVP_Node_GS2Attractor::AddPropertiesTabControls( Asura_Tabs_Dialog* pxTabsDialogue, Asura_Vector_2& xOffset )
{
	AddStructureTypeDialogue( pxTabsDialogue, xOffset );
	AddSmoothOverNodeOverlayControls( pxTabsDialogue, xOffset );
	AddFollowSurfaceDialogue( pxTabsDialogue, xOffset );
	AddCollisionsDialogues( pxTabsDialogue, xOffset );

	Asura_Dialog* pxNewDialog = new Asura_Tickbox_Dialog( "Integrate With Nav Data", xOffset, &GetAVPGameNode()->m_bIntegrateWithNavData, &Changed );
	pxNewDialog->SetHelpText( "Determines whether this node should integrate into the navigation data." );
	pxTabsDialogue->AddDialog( pxNewDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxNewDialog = new Asura_Tickbox_Dialog( "Leave Gamescene on Threat Information", xOffset, &GetAVPGameNode()->m_bLeaveGamesceneOnKnowledge, &Changed );
	pxNewDialog->SetHelpText( "If the actor gains threat or alert knowledge and this is selected the actor will leave gamescene." );
	pxTabsDialogue->AddDialog( pxNewDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	AddAttractParamsDialogs( pxTabsDialogue, xOffset );
	xOffset.y += Asura_Dialog::LargeSpacing() - Asura_Dialog::DefaultSpacing();

	AddLinkWeightsDialogue( pxTabsDialogue, xOffset );
	xOffset.y += Asura_Dialog::LargeSpacing() - Asura_Dialog::DefaultSpacing();
}

void ToolsAVP_Node_GS2Attractor::AddUnderTabsControls( Asura_Overlay* pxNewOverlay, Asura_Vector_2& xOffset )
{
	Asura_Dialog* pxNewDialog = new Asura_Function_Dialog( "Create Linked Node", xOffset, CreateLinkedNode, NULL, CanCreateLinkedNode );
	pxNewDialog->SetHelpText( "Create a new gamescene link and target node from this node." );
	pxNewOverlay->AddDialog( pxNewDialog );
	xOffset.x += 15.0f;

	pxNewDialog = new Asura_Function_Dialog( "Create Link", xOffset, CreateLink, NULL, CanCreateLink );
	pxNewDialog->SetHelpText( "Create a new gamescene link between two nodes." );
	pxNewOverlay->AddDialog( pxNewDialog );
	xOffset.x -= 15.0f;
	xOffset.y += Asura_Dialog::LargeSpacing();

	AddMainDefaultsOverlayControls( pxNewOverlay, xOffset );
	Tools_Object_Axon_Interface::AddPreviewControls( pxNewOverlay, xOffset );
}

void ToolsAVP_Node_GS2Attractor::CreateLinkedNode()
{
	// Get the source node.
	const Asura_Guid uSourceGuid = Tools_Object_System::SelectedObject->GetGuid();
	Tools_EntityNode_GS2Node* pxSourceNode = Tools_EntityNode_GS2Node::FindEntity( uSourceGuid );
	if ( !pxSourceNode )
	{
		return;
	}

	// Create the target node.
	Tools_SecondaryEntityFile_System::PreCreateObject();
	ToolsAVP_Node_GS2Node::Create();
	Tools_SecondaryEntityFile_System::PostCreateObject();
	const Asura_Guid uTargetGuid = Tools_Object_System::SelectedObject->GetGuid();
	Tools_EntityNode_GS2Node* pxTargetNode = Tools_EntityNode_GS2Node::FindEntity( uTargetGuid );
	if ( !pxTargetNode )
	{
		return;
	}

	// Create the link between the nodes.
	Tools_Object_System::SelectObject( pxSourceNode );
	Tools_Object_System::SelectObject( pxTargetNode, true );
	Tools_SecondaryEntityFile_System::PreCreateObject();
	ToolsAVP_Node_GS2Spline::Create();
	Tools_SecondaryEntityFile_System::PostCreateObject();
	Tools_Object_System::SelectObject( pxTargetNode );
}

bool ToolsAVP_Node_GS2Attractor::CanCreateLinkedNode()
{
	return ( Tools_Object_System::SelectedObjectList.size() == 1 );
}

void ToolsAVP_Node_GS2Attractor::CreateLink()
{
	Tools_EntityNode_GS2Spline::Create();
}

bool ToolsAVP_Node_GS2Attractor::CanCreateLink()
{
	if ( Tools_Object_System::SelectedObjectList.size() != 2 ) { return false; }

	if ( Tools_EntityNode_GS2Node::FindEntity( Tools_Object_System::SelectedObjectList.first_entry()->GetGuid() ) == NULL )
	{
		return false;
	}

	if ( Tools_EntityNode_GS2Node::FindEntity( Tools_Object_System::SelectedObjectList.last_entry()->GetGuid() ) == NULL )
	{
		return false;
	}

	return true;
}

AVP_ServerNode_GS2Node* ToolsAVP_Node_GS2Attractor::GetAVPGameNode() const
{
	return AVP_ServerNode_GS2Node::FindEntity( GetGuid() );
}

bool ToolsAVP_Node_GS2Attractor::GetParameter( Asura_Hash_ID uParamID, Tools_Parameter* pxParam ) const
{
	bool bRet = true;
	bRet = PARENT_AVPTOOLS::GetParameter( uParamID, pxParam );
	bRet = PARENT_TOOLS::GetParameter( uParamID, pxParam );
	return bRet;
}

bool ToolsAVP_Node_GS2Attractor::SetParameter( Asura_Hash_ID uParamID, const Tools_Parameter* pxParam )
{
	bool bRet = true;
	bRet = PARENT_AVPTOOLS::SetParameter( uParamID, pxParam );
	bRet = PARENT_TOOLS::SetParameter( uParamID, pxParam );
	return bRet;
}

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////
