//////////////////////////////////////////////////////////////////////////////
//
// ToolsAVP_Node_OrientedNode.cpp
//
// created by Stuart Perry ( 18.04.07 )
//
// Copyright (c) 2007 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header.
#include "Entity/Locations/OrientedNode/ToolsAVP_Node_OrientedNode.h"

//Asura
#include "Camera/Asura_Camera.h"
#include "Interface/Asura_Dialog.h"
#include "Rendering/Asura_Render.h"

//Project
#include "Entity/Locations/OrientedNode/AVP_ServerNode_OrientedNode.h"
#include "Entity/Locations/GuardZone/AVP_ServerNode_GuardZone.h"

//Tools
#include "Load&Save/Tools_SaveAsura.h"
#include "Misc/Tools_Timed_Messages.h"
#include "Objects/Tools_Object_System.h"
#include "Objects/ToolsAVP_Object_Type.h"
#include "Entity/Locations/GuardZone/ToolsAVP_Node_GuardZone.h"

////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////

ToolsAVP_Node_OrientedNode* ToolsAVP_Node_OrientedNode::s_pxAVPToolsOrientedNode = NULL;

Asura_Hashed_List<ToolsAVP_Node_OrientedNode> ToolsAVP_Node_OrientedNode::s_xList(2);

//////////////////////////////////////////////////////////////////////////////
//
// ToolsAVP_Node_OrientedNode
//
//////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Function     : ToolsAVP_Node_OrientedNode::ToolsAVP_Node_OrientedNode
// Author       : Stuart Perry
// Date         : 18-APRIL-2007
////////////////////////////////////////////////////////////////////////////////

ToolsAVP_Node_OrientedNode::ToolsAVP_Node_OrientedNode( Asura_Entity* pxEntity )
: PARENT( pxEntity )
, m_pxRandomOffsetDialog(NULL)
, m_bChanged(true)
, m_bChangedPatrolOrder(false)
{

	s_xList.AddToList( this );

	AVP_ServerNode_OrientedNode *pxAVPEntity = (AVP_ServerNode_OrientedNode*) pxEntity;

	m_fRandomWeighting		= static_cast<float>( pxAVPEntity->m_uRandomWeighting );
	m_uPatrolOrder			= pxAVPEntity->m_uPatrolOrder;
	m_fWaitTime				= pxAVPEntity->m_fWaitTime;
	m_fWaitTimeRandomOffset = pxAVPEntity->m_fWaitTimeRandomOffset;
	m_bWaitTimeRandomOffset = pxAVPEntity->m_bWaitTimeRandomOffset;
}

/// End of function 'ToolsAVP_Node_OrientedNode::ToolsAVP_Node_OrientedNode'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : ToolsAVP_Node_OrientedNode::~ToolsAVP_Node_OrientedNode
// Author       : Stuart Perry
// Date         : 18-APRIL-2007
////////////////////////////////////////////////////////////////////////////////

ToolsAVP_Node_OrientedNode::~ToolsAVP_Node_OrientedNode() 
{
	s_xList.RemoveFromList( this );
}

/// End of function 'ToolsAVP_Node_OrientedNode::~ToolsAVP_Node_OrientedNode'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : ToolsAVP_Node_OrientedNode::Create
// Author       : Stuart Perry
// Date         : 18-APRIL-2007
////////////////////////////////////////////////////////////////////////////////

Tools_Object* ToolsAVP_Node_OrientedNode::Create( Asura_Entity* pxEntity )
{
	return new ToolsAVP_Node_OrientedNode( pxEntity );
}

/// End of function 'ToolsAVP_Node_OrientedNode::Create'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : ToolsAVP_Node_OrientedNode::Create
// Author       : Gareth Thomas
// Date         : 08 / 11 / 07
////////////////////////////////////////////////////////////////////////////////
void ToolsAVP_Node_OrientedNode::Project_Create(void)
{
	PARENT::Create();
}

////////////////////////////////////////////////////////////////////////////////
// Function     : ToolsAVP_Node_OrientedNode::AddOverlayControls
// Author       : Stuart Perry
// Date         : 18-APRIL-2007
////////////////////////////////////////////////////////////////////////////////

void ToolsAVP_Node_OrientedNode::AddOverlayControls( class Asura_Overlay* pxNewOverlay, class Asura_Vector_2& xOffset )
{
	PARENT::AddOverlayControls( pxNewOverlay, xOffset );

	Asura_Dialog* pxNewDialog = NULL;

	ToolsAVP_Node_GuardZone* pxToolsNode = static_cast<ToolsAVP_Node_GuardZone*>( Tools_EntityNode_GuardZone::FindEntity( m_uMyToolsGZGuid ) );
	Asura_Assert( pxToolsNode, "ToolsAVP_Node_OrientedNode::AddOverlayControls - invalid pointer to ToolsAVP_Node_GuardZone" );
	if ( !pxToolsNode )
	{
		return;
	}

	// Order of the patrol points - number dialog (max is the current number of guard points)
	pxNewDialog = new Asura_Numerical_Dialog("Patrol Order", xOffset, &m_uPatrolOrder, 0, pxToolsNode->GetNumPatrolPoints() - 1 , 1, "%d", &m_bChangedPatrolOrder);
	pxNewDialog->SetHelpText("This sets the order of patrolling.");
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

// 	// Random weighting of guard point - number dialog
// 	pxNewDialog = new Asura_Numerical_Dialog("Random weighting", xOffset, &m_fRandomWeighting, 1, 99, 1, "%.0f", &m_bChanged);
// 	pxNewDialog->SetHelpText("This sets the random weighting for the guard point.");
// 	pxNewOverlay->AddDialog(pxNewDialog);
// 	xOffset.y += Asura_Dialog::DefaultSpacing();

	// Wait time - number dialog
	pxNewDialog = new Asura_Numerical_Dialog("Wait Time", xOffset, &m_fWaitTime, 0.0f, 99.0f, 0.1f, "%.1f", &m_bChanged);
	pxNewDialog->SetHelpText("This sets the time to wait at this guard point.");
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();

	pxNewDialog = new Asura_StringDisplay_Dialog("Random Wait Offset:", xOffset);
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	// Random offset - tick box
	pxNewDialog = new Asura_Tickbox_Dialog("",xOffset,&m_bWaitTimeRandomOffset,&m_bChanged);
	pxNewDialog->SetHelpText("This set active the Random offset.");
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.x += Asura_Dialog::LargeSpacing(); xOffset.x += Asura_Dialog::LargeSpacing(); xOffset.x += Asura_Dialog::LargeSpacing();

	//	Random offset time - number dialog
	pxNewDialog = new Asura_Numerical_Dialog("", xOffset, &m_fWaitTimeRandomOffset, 1.0f, 99.0f, 0.1f, "%.1f", &m_bChanged);
	pxNewDialog->SetHelpText("This sets the random offset to the time.");
	pxNewOverlay->AddDialog(pxNewDialog);
	pxNewDialog->SetEnabled(m_bWaitTimeRandomOffset);
	m_pxRandomOffsetDialog = pxNewDialog;
	xOffset.y += Asura_Dialog::DefaultSpacing();
}

/// End of function 'ToolsAVP_Node_OrientedNode::AddOverlayControls'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : ToolsAVP_Node_OrientedNode::OnSelection
// Author       : Stuart Perry
// Date         : 18-APRIL-2007
////////////////////////////////////////////////////////////////////////////////

void ToolsAVP_Node_OrientedNode::OnSelection()
{
	s_pxAVPToolsOrientedNode = this;

	PARENT::OnSelection();
}

/// End of function 'ToolsAVP_Node_OrientedNode::OnSelection'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : ToolsAVP_Node_OrientedNode::OnDeselection
// Author       : Stuart Perry
// Date         : 18-APRIL-2007
////////////////////////////////////////////////////////////////////////////////

void ToolsAVP_Node_OrientedNode::OnDeselection()
{
	PARENT::OnDeselection();

	s_pxAVPToolsOrientedNode = NULL;
}

/// End of function 'ToolsAVP_Node_OrientedNode::OnDeselection'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : ToolsAVP_Node_OrientedNode::UpdateWhileSelected
// Author       : Stuart Perry
// Date         : 18-APRIL-2007
////////////////////////////////////////////////////////////////////////////////

void ToolsAVP_Node_OrientedNode::UpdateWhileSelected()
{
	PARENT::UpdateWhileSelected();

	AVP_ServerNode_OrientedNode* pxAVPEntity	= (AVP_ServerNode_OrientedNode*)Entity;

	if ( m_bChanged )
	{
		if (m_pxRandomOffsetDialog)
		{
			m_pxRandomOffsetDialog->SetEnabled(m_bWaitTimeRandomOffset);
		}

		////update the project data
		pxAVPEntity->m_uRandomWeighting			= static_cast<u_int>( m_fRandomWeighting );
		pxAVPEntity->m_fWaitTime				= m_fWaitTime;
		pxAVPEntity->m_fWaitTimeRandomOffset	= m_fWaitTimeRandomOffset;
		pxAVPEntity->m_bWaitTimeRandomOffset	= m_bWaitTimeRandomOffset;
	}

	if ( m_bChangedPatrolOrder )
	{
		pxAVPEntity->m_uPatrolOrder				= static_cast<u_int>( m_uPatrolOrder );
		m_bChangedPatrolOrder					= false;
	}

	// Update tools side patrol points from orderPatrol()
	m_uPatrolOrder = pxAVPEntity->m_uPatrolOrder;
}

/// End of function 'ToolsAVP_Node_OrientedNode::UpdateWhileSelected'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : ToolsAVP_Node_OrientedNode::CreateGuardPoint
// Author       : Stuart Perry
// Date         : 18-APRIL-2007
////////////////////////////////////////////////////////////////////////////////

void ToolsAVP_Node_OrientedNode::CreateGuardPoint( ToolsAVP_Node_OrientedNode** ppxNewNode, Asura_Guid uGZGuid, int iNumGuardPoints  )
{
	*ppxNewNode = ToolsAVP_Node_OrientedNode::CreateAndReturn( iNumGuardPoints, uGZGuid );
}

/// End of function 'ToolsAVP_Node_OrientedNode::CreateGuardPoint'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : ToolsAVP_Node_OrientedNode::CreateAndReturn
// Author       : Stuart Perry
// Date         : 18-APRIL-2007
////////////////////////////////////////////////////////////////////////////////

ToolsAVP_Node_OrientedNode* ToolsAVP_Node_OrientedNode::CreateAndReturn( int iNumGuardPoints, Asura_Guid uGZGuid )
{
	// Get a new GUID
	const Asura_Guid uGuid = Asura_Entity_System::GetCreatedByToolsGuid();

	// Set up tools data and auto-create a unique name (prompt for new name if there is a clash).
	Tools_Entity_Data* pxToolsData = CreateToolsDataWithDefaultName( "Orientated Node", "Orientated Node", uGuid );
	if ( !pxToolsData ) { return NULL; }

	// Create the game specfic server object and set its data.
	AVP_ServerNode_OrientedNode* pxGameNode = new AVP_ServerNode_OrientedNode( uGuid );
	if ( !pxGameNode ) { return NULL; }

	pxGameNode->ToolsData = pxToolsData;

	// Create the game specific tools object and select it.
	ToolsAVP_Node_OrientedNode* pxToolsNode = new ToolsAVP_Node_OrientedNode( pxGameNode );
	if ( !pxToolsNode ) { return NULL; }

	pxToolsNode->Position = *Asura_Camera::GetPosition();
	pxToolsNode->Orientation.Identity();// = *Asura_Camera::GetOrientation();
	pxToolsNode->m_uPatrolOrder = iNumGuardPoints;
	pxToolsNode->m_uMyToolsGZGuid = uGZGuid;

	pxGameNode->SetPosition( pxToolsNode->Position );
	pxGameNode->SetOrientation( pxToolsNode->Orientation );
	pxGameNode->m_uPatrolOrder = pxToolsNode->m_uPatrolOrder;

	Tools_Object_System::SelectObject( pxToolsNode );

	Tools_Timed_Message_System::AddMessage( Tools_Timed_Message_System::COLOUR_DEFAULT, "Created new Orientated Node" );
	Tools_SaveAsura::SetEntityModified();

	return pxToolsNode;
}

/// End of function 'ToolsAVP_Node_OrientedNode::CreateAndReturn'. ///

//////////////////////////////////////////////////////////////////////////////
//
//
//////////////////////////////////////////////////////////////////////////////
