//-----------------------------------------------------------------------------
// File			: ToolsAVP_Entity_ShootAt.cpp
// Author		: Gareth Thomas
// Created		: 18 / 12 / 2007
// Copyright (c) 2007 Rebellion. All rights reserved.
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Includes
//-----------------------------------------------------------------------------

// Core
#include "Core/Asura.h"

// Header
#include "Entity/Locations/ShootAt/ToolsAVP_Entity_ShootAt.h"

// Asura
#include "Camera/Asura_Camera.h"
#include "Mesh/Asura_Mesh_System.h"

// AVP
#include "Entity/Locations/ShootAt/AVP_ServerEntity_ShootAt.h"

// Tools
#include "Entity/PhysicalObject/ToolsAVP_Entity_PhysicalObject.h"
#include "Load&Save/Tools_SaveAsura.h"
#include "Misc/Tools_Render.h"
#include "Misc/Tools_Timed_Messages.h"
#include "Objects/ToolsAVP_Object_Type.h"

//------------------------------------------------------------------------------
// Constants, Defines, Statics and Globals
//------------------------------------------------------------------------------
Asura_Hashed_List<ToolsAVP_Entity_ShootAt>			ToolsAVP_Entity_ShootAt::s_xList;
ToolsAVP_Entity_ShootAt*							ToolsAVP_Entity_ShootAt::s_pxSelectedShootAtNode = NULL;

//-----------------------------------------------------------------------------
// Function Implementations
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// ToolsAVP_Entity_ShootAt :: Constructor
//-----------------------------------------------------------------------------
ToolsAVP_Entity_ShootAt::ToolsAVP_Entity_ShootAt(Asura_Entity* pxEntity)
: PARENT(pxEntity)
, m_bShootAtPlayer( false )
, m_bShootAtLinkedEntity( false )
, m_uLinkedShootAtEntity( ASURA_GUID_UNREGISTERED )
, m_bShootAtPlayerChanged( false )
, m_bShootAtLinkedEntityChanged( false )
{
	AVP_ServerEntity_ShootAt* pxShootAt = (AVP_ServerEntity_ShootAt*)pxEntity;

	if (pxShootAt)
	{
		m_bShootAtPlayer		= pxShootAt->GetShootAtPlayer();
		m_bShootAtLinkedEntity	= pxShootAt->GetShouldShootAtLinkedEntity();
		m_uLinkedShootAtEntity	= pxShootAt->GetShootAtEntityGuid();
	}

	Type = TOT_Entity_ShootAt;

	SetMesh("ToolsShape_ShootAtNode");

	s_xList.AddToList(this);
}

//-----------------------------------------------------------------------------
// ToolsAVP_Entity_ShootAt :: Deconstructor
//-----------------------------------------------------------------------------
ToolsAVP_Entity_ShootAt::~ToolsAVP_Entity_ShootAt(void)
{
	s_xList.RemoveFromList(this);
}

//-----------------------------------------------------------------------------
// ToolsAVP_Entity_ShootAt :: Create
//-----------------------------------------------------------------------------
void ToolsAVP_Entity_ShootAt::Create(void)
{
	const Asura_Guid uGuid = Asura_Entity_System::GetCreatedByToolsGuid();

	Tools_Entity_Data *pxToolsData = CreateToolsDataWithDefaultName("ShootAtNode", "ShootAtNode", uGuid);

	if (pxToolsData == NULL)
	{
		Asura_Assert(false, "ToolsAVP_Entity_ShootAt::Create() - no tools data");
	}
	else
	{
		AVP_ServerEntity_ShootAt* pxServerObject = new AVP_ServerEntity_ShootAt(uGuid);

		if (pxServerObject == NULL)
		{
			Asura_Assert(false, "ToolsAVP_Entity_ShootAt::Create() - no server object" );
		}
		else
		{
			pxServerObject->ToolsData = pxToolsData;

			ToolsAVP_Entity_ShootAt* pxToolsObject = new ToolsAVP_Entity_ShootAt(pxServerObject);

			// Create the game specific tools object and select it.
			if (pxToolsObject == NULL)
			{
				Asura_Assert(false, "ToolsAVP_Entity_ShootAt::Create() - no tools object");
			}
			else
			{
				Tools_Object_System::SelectObject(pxToolsObject);

				pxToolsObject->Position		= *Asura_Camera::GetPosition();
				pxToolsObject->m_uGuid		= uGuid;

				pxServerObject->SetPosition(pxToolsObject->Position);

				Tools_Timed_Message_System::AddMessage(0xffffff, "Created New Shoot At Node");
				Tools_SaveAsura::SetEntityModified();
			}
		}
	}
}

//-----------------------------------------------------------------------------
// ToolsAVP_Entity_ShootAt :: Create
//-----------------------------------------------------------------------------
Tools_Object* ToolsAVP_Entity_ShootAt::Create(Asura_Entity* pxEntity)
{
	return new ToolsAVP_Entity_ShootAt(pxEntity);
}

//-----------------------------------------------------------------------------
// ToolsAVP_Entity_ShootAt :: CreateAndReturn
//-----------------------------------------------------------------------------
Tools_Object* ToolsAVP_Entity_ShootAt::CreateAndReturn(void)
{
	const Asura_Guid uGuid = Asura_Entity_System::GetCreatedByToolsGuid();

	Tools_Entity_Data *pxToolsData = CreateToolsDataWithDefaultName("ShootAtNode", "ShootAtNode", uGuid);

	if (pxToolsData == NULL)
	{
		Asura_Assert(false, "ToolsAVP_Entity_ShootAt::Create() - no tools data");
	}
	else
	{
		AVP_ServerEntity_ShootAt* pxServerObject = new AVP_ServerEntity_ShootAt(uGuid);

		if (pxServerObject == NULL)
		{
			Asura_Assert(false, "ToolsAVP_Entity_ShootAt::Create() - no server object" );
		}
		else
		{
			pxServerObject->ToolsData = pxToolsData;

			ToolsAVP_Entity_ShootAt* pxToolsObject = new ToolsAVP_Entity_ShootAt(pxServerObject);

			// Create the game specific tools object and select it.
			if (pxToolsObject == NULL)
			{
				Asura_Assert(false, "ToolsAVP_Entity_ShootAt::Create() - no tools object");
			}
			else
			{
				Tools_Object_System::SelectObject(pxToolsObject);

				pxToolsObject->Position		= *Asura_Camera::GetPosition();
				pxToolsObject->m_uGuid		= uGuid;

				pxServerObject->SetPosition(pxToolsObject->Position);

				Tools_Timed_Message_System::AddMessage(Tools_Timed_Message_System::COLOUR_INFO, "Created New Shoot At Node");
				Tools_SaveAsura::SetEntityModified();
				return pxToolsObject;
			}
		}
	}

	return NULL;
}

//-----------------------------------------------------------------------------
// ToolsAVP_Entity_ShootAt :: Project_Create
//-----------------------------------------------------------------------------
void ToolsAVP_Entity_ShootAt::Project_Create(void)
{
	ToolsAVP_Entity_ShootAt::Create();
}

//-----------------------------------------------------------------------------
// ToolsAVP_Entity_ShootAt :: AddOverlayControls
//-----------------------------------------------------------------------------
void ToolsAVP_Entity_ShootAt::AddOverlayControls(class Asura_Overlay* pxOverlay, class Asura_Vector_2& xOffset)
{
	PARENT::AddOverlayControls(pxOverlay, xOffset);

	Asura_Dialog* pxDialog;

	// Shoot at linked entity rather than this target
	pxDialog = new Asura_Tickbox_Dialog("Shoot at player", xOffset, &m_bShootAtPlayer, &m_bShootAtPlayerChanged);
	pxDialog->SetHelpText("Have this selected to shoot the player");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	// Shoot at linked entity rather than this target
	pxDialog = new Asura_Tickbox_Dialog("Shoot at linked Actor rather than this target", xOffset, &m_bShootAtLinkedEntity, &m_bShootAtLinkedEntityChanged);
	pxDialog->SetHelpText("Have this selected to shoot at linked entity rather than this target");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Select shoot at Actor", xOffset, SelectShootAtEntity);
	pxDialog->SetHelpText("Allows selection of the entity that will be shot at");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();
	if ( m_uLinkedShootAtEntity != ASURA_GUID_UNREGISTERED )
	{
		ToolsAVP_Entity_PhysicalObject* pxObject = ToolsAVP_Entity_PhysicalObject::FindEntity( m_uLinkedShootAtEntity );
		if ( pxObject ) pxDialog->UpdateLabel( pxObject->GetName() );
	}
	if (!m_bShootAtLinkedEntity) { pxDialog->SetEnabled( false ); }

	pxDialog = new Asura_Function_Dialog("Clear shoot at entity", xOffset, ClearShootAtEntity);
	pxDialog->SetHelpText("Clears the selection of the entity that will be shot at");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();
	if (!m_bShootAtLinkedEntity) { pxDialog->SetEnabled( false ); }
}

//-----------------------------------------------------------------------------
// ToolsAVP_Entity_ShootAt :: Copy
//-----------------------------------------------------------------------------
void ToolsAVP_Entity_ShootAt::SelectShootAtEntity(void)
{
	if ( s_pxSelectedShootAtNode != NULL )
	{
		ToolsAVP_ChooseObject_AnyActor* pxChooseObject = new ToolsAVP_ChooseObject_AnyActor( &s_pxSelectedShootAtNode->m_uLinkedShootAtEntity, &s_pxSelectedShootAtNode->m_bShootAtLinkedEntityChanged );
		Tools_Object_System::ChooseObject("Select The Target Actor", pxChooseObject);
	}
}

//-----------------------------------------------------------------------------
// ToolsAVP_Entity_ShootAt :: Copy
//-----------------------------------------------------------------------------
void ToolsAVP_Entity_ShootAt::ClearShootAtEntity(void)
{
	if ( s_pxSelectedShootAtNode != NULL )
	{
		s_pxSelectedShootAtNode->m_uLinkedShootAtEntity			= ASURA_GUID_UNREGISTERED;
		s_pxSelectedShootAtNode->m_bShootAtLinkedEntityChanged	= true;
	}
}

//-----------------------------------------------------------------------------
// ToolsAVP_Entity_ShootAt :: Copy
//-----------------------------------------------------------------------------
void ToolsAVP_Entity_ShootAt::Copy(const Tools_Object* pxObject)
{
	PARENT::Copy(pxObject);
}

//-----------------------------------------------------------------------------
// ToolsAVP_Entity_ShootAt :: CreateObjectOfThisType
//-----------------------------------------------------------------------------
Tools_Object* ToolsAVP_Entity_ShootAt::CreateObjectOfThisType(void) const
{
	return CreateAndReturn();
}

//-----------------------------------------------------------------------------
// ToolsAVP_Entity_ShootAt :: OnSelection
//-----------------------------------------------------------------------------
void ToolsAVP_Entity_ShootAt::OnSelection(void)
{
	s_pxSelectedShootAtNode = this;
	PARENT::OnSelection();
}

//-----------------------------------------------------------------------------
// ToolsAVP_Entity_ShootAt :: OnDeselection
//-----------------------------------------------------------------------------
void ToolsAVP_Entity_ShootAt::OnDeselection(void)
{
	PARENT::OnDeselection();
	s_pxSelectedShootAtNode = NULL;
}

//-----------------------------------------------------------------------------
// ToolsAVP_Entity_ShootAt :: Render
//-----------------------------------------------------------------------------
void ToolsAVP_Entity_ShootAt::Render(void)
{
	PARENT::Render();

	if (!m_bShootAtPlayer)
	{
		if ( m_bShootAtLinkedEntity && m_uLinkedShootAtEntity != ASURA_GUID_UNREGISTERED )
		{
			ToolsAVP_Entity_PhysicalObject* pxObject = ToolsAVP_Entity_PhysicalObject::FindEntity( m_uLinkedShootAtEntity );

			if ( pxObject )
			{
				Asura_Vector_3 xObjPos = pxObject->Position; 
				
				if (pxObject->Type != TOT_Entity_FaceHugger)
				{
					xObjPos.y -= 1.0f;
				}

				Asura_Vector_3 xOurPos = Position; xOurPos.y -= 0.2f;
				Tools_Render::RenderCylinder(xOurPos, xObjPos, 0.025f, 0xffff0000, false);
				Asura_Vector_3 xDir = xObjPos - xOurPos; xDir.Normalise();
				Asura_Matrix_3x3 xOri; Asura_Maths::OrientationFromDirection(xOri, -xDir);
				Tools_Render::RenderCone(xObjPos - xDir * 0.1f, xOri, 1.0f, Asura_Maths::Deg90 / 4.0f, 0xffff0000, false);
			}
		}
	}
}

//-----------------------------------------------------------------------------
// ToolsAVP_Entity_ShootAt :: UpdateWhileSelected
//-----------------------------------------------------------------------------
void ToolsAVP_Entity_ShootAt::UpdateWhileSelected(void)
{
	PARENT::UpdateWhileSelected();

	if ( m_bShootAtLinkedEntityChanged )
	{
		GetGameShootAtNode()->SetShouldShootAtLinkedEntity( m_bShootAtLinkedEntity );
		GetGameShootAtNode()->SetShootAtEntityGuid( m_uLinkedShootAtEntity );

		if ( m_bShootAtLinkedEntity )
		{
			m_bShootAtPlayer = false;
			GetGameShootAtNode()->SetShootAtPlayer( m_bShootAtPlayer );
		}

		DeleteOverlay();
		CreateOverlay();
		m_bShootAtLinkedEntityChanged = false;
		SetEntityModified();
	}

	if ( m_bShootAtPlayerChanged )
	{
		GetGameShootAtNode()->SetShootAtPlayer( m_bShootAtPlayer );

		if ( m_bShootAtPlayer )
		{
			m_bShootAtLinkedEntity = false;
			GetGameShootAtNode()->SetShouldShootAtLinkedEntity( m_bShootAtLinkedEntity );
		}

		DeleteOverlay();
		CreateOverlay();
		m_bShootAtPlayerChanged = false;
		SetEntityModified();
	}
}

//-----------------------------------------------------------------------------
// ToolsAVP_Entity_ShootAt :: IsBaseOrDerivativeOf
//-----------------------------------------------------------------------------
bool ToolsAVP_Entity_ShootAt::IsBaseOrDerivativeOf( int iToolsObjectType ) const
{
	if ( iToolsObjectType == TOT_Entity_ShootAt )
	{
		return true;
	}

	return PARENT::IsBaseOrDerivativeOf( iToolsObjectType );
}

//-----------------------------------------------------------------------------
// ToolsAVP_Entity_ShootAt :: GetGameShootAtNode
//-----------------------------------------------------------------------------
AVP_ServerEntity_ShootAt* ToolsAVP_Entity_ShootAt::GetGameShootAtNode() const
{
	return static_cast< AVP_ServerEntity_ShootAt* >( Entity );
}

//-----------------------------------------------------------------------------
// ToolsAVP_ChooseObject_AnyActor :: CanBeSelected
//-----------------------------------------------------------------------------
bool ToolsAVP_ChooseObject_AnyActor::CanBeSelected( Tools_Object* pxObject )
{
	if ( pxObject->Type == TOT_Entity_Marine || pxObject->Type == TOT_Entity_Android || pxObject->Type == TOT_Entity_Civilian ||
		pxObject->Type == TOT_Entity_Alien || pxObject->Type == TOT_Entity_FaceHugger || pxObject->Type == TOT_Entity_Predator ||
		pxObject->Type == TOT_Entity_Predalien)
	{
		return true;
	}
	else
	{
		return false;
	}
}
 
//-----------------------------------------------------------------------------
// ToolsAVP_ChooseObject_AnyActor :: ObjectSelected
//-----------------------------------------------------------------------------
void ToolsAVP_ChooseObject_AnyActor::ObjectSelected( Tools_Object* pxObject )
{
	if ( pxObject && pxObject->GetGuid() != ASURA_GUID_UNREGISTERED )
	{
		*m_puGuidToSet	= pxObject->GetGuid();
		*m_pbChanged	= true;
	}
}