//-----------------------------------------------------------------------------
// File			: ToolsAVP_Entity_MotionTrackerBlip.cpp
// Author		: Gareth Thomas
// Created		: 18 / 12 / 2007
// Copyright (c) 2007 Rebellion. All rights reserved.
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Includes
//-----------------------------------------------------------------------------

// Core
#include "Core/Asura.h"

// Header
#include "Entity/MotionTrackerBlip/ToolsAVP_Entity_MotionTrackerBlip.h"

// Asura
#include "Camera/Asura_Camera.h"
#include "Mesh/Asura_Mesh_System.h"

// AVP
#include "Entity/Locations/OrientedNode/GamesceneNode/AVP_ServerNode_GS2Node.h"
#include "Entity/MotionTrackerBlip/AVP_ServerEntity_MotionTrackerBlip.h"

// Tools
#include "Entity/Locations/OrientedNode/GamesceneNode/ToolsAVP_Node_GS2Node.h"
#include "Load&Save/Tools_SaveAsura.h"
#include "Misc/Tools_Render.h"
#include "Misc/Tools_Timed_Messages.h"
#include "Objects/ToolsAVP_Object_Type.h"

//------------------------------------------------------------------------------
// Constants, Defines, Statics and Globals
//------------------------------------------------------------------------------
Asura_Hashed_List<ToolsAVP_Entity_MotionTrackerBlip>				ToolsAVP_Entity_MotionTrackerBlip::s_xList;
ToolsAVP_Entity_MotionTrackerBlip*									ToolsAVP_Entity_MotionTrackerBlip::s_pxSelectedMotionTrackerBlip = NULL;

//------------------------------------------------------------------------------
// Motion tracker Blip Shared Params
//------------------------------------------------------------------------------
TOOLS_DECLARE_SHARED_PARAM( Asura_Guid, 0xccba97e1, AVP_MTBlip_StartNode );	// Asura_GetHashID( "AVP_MTBlip_StartNode" )

//-----------------------------------------------------------------------------
// Function Implementations
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// ToolsAVP_Entity_MotionTrackerBlip :: Constructor
//-----------------------------------------------------------------------------
ToolsAVP_Entity_MotionTrackerBlip::ToolsAVP_Entity_MotionTrackerBlip(Asura_Entity* pxEntity)
: PARENT(pxEntity)
{
	Type = TOT_Entity_MotionTrackerBlip;
	SetMesh("ToolsShape_MotionTrackerBlip");

	AVP_ServerEntity_MotionTrackerBlip* pxAVPEntity = AVP_ServerEntity_MotionTrackerBlip::FindEntity( pxEntity->GetGuid() );

	if ( pxAVPEntity )
	{
		pxGameEntity = pxAVPEntity;
	}

	s_xList.AddToList(this);
}

//-----------------------------------------------------------------------------
// ToolsAVP_Entity_MotionTrackerBlip :: Deconstructor
//-----------------------------------------------------------------------------
ToolsAVP_Entity_MotionTrackerBlip::~ToolsAVP_Entity_MotionTrackerBlip(void)
{
	s_xList.RemoveFromList(this);
}

//-----------------------------------------------------------------------------
// ToolsAVP_Entity_MotionTrackerBlip :: Create
//-----------------------------------------------------------------------------
void ToolsAVP_Entity_MotionTrackerBlip::Create(void)
{
	const Asura_Guid uGuid = Asura_Entity_System::GetCreatedByToolsGuid();

	Tools_Entity_Data *pxToolsData = CreateToolsDataWithDefaultName( "Motion Tracker Blip", "Motion Tracker Blip", uGuid );

	if ( pxToolsData == NULL )
	{
		Asura_Assert( false, "ToolsAVP_Entity_MotionTrackerBlip::Create() - no tools data" );
	}
	else
	{
		AVP_ServerEntity_MotionTrackerBlip* pxServerObject = new AVP_ServerEntity_MotionTrackerBlip( uGuid );

		if ( pxServerObject == NULL )
		{
			Asura_Assert(false, "ToolsAVP_Entity_MotionTrackerBlip::Create() - no server object" );
		}
		else
		{
			pxServerObject->ToolsData = pxToolsData;

			ToolsAVP_Entity_MotionTrackerBlip* pxToolsObject = new ToolsAVP_Entity_MotionTrackerBlip( pxServerObject );

			// Create the game specific tools object and select it.
			if ( pxToolsObject == NULL )
			{
				Asura_Assert(false, "ToolsAVP_Entity_MotionTrackerBlip::Create() - no tools object");
			}
			else
			{
				Tools_Object_System::SelectObject(pxToolsObject);

				pxToolsObject->Position		= *Asura_Camera::GetPosition();
				pxToolsObject->m_uGuid		= uGuid;

				pxServerObject->SetPosition( pxToolsObject->Position );

				Tools_Timed_Message_System::AddMessage( Tools_Timed_Message_System::COLOUR_INFO, "Created New Formation Point" );
				Tools_SaveAsura::SetEntityModified();
			}
		}
	}
}

//-----------------------------------------------------------------------------
// ToolsAVP_Entity_MotionTrackerBlip :: Create
//-----------------------------------------------------------------------------
Tools_Object* ToolsAVP_Entity_MotionTrackerBlip::Create(Asura_Entity* pxEntity)
{
	return new ToolsAVP_Entity_MotionTrackerBlip( pxEntity );
}

//-----------------------------------------------------------------------------
// ToolsAVP_Entity_MotionTrackerBlip :: CreateAndReturn
//-----------------------------------------------------------------------------
Tools_Object* ToolsAVP_Entity_MotionTrackerBlip::CreateAndReturn( void )
{
	const Asura_Guid uGuid = Asura_Entity_System::GetCreatedByToolsGuid();

	Tools_Entity_Data *pxToolsData = CreateToolsDataWithDefaultName( "Motion Tracker Blip", "Motion Tracker Blip", uGuid );

	if (pxToolsData == NULL)
	{
		Asura_Assert(false, "ToolsAVP_Entity_MotionTrackerBlip::Create() - no tools data");
	}
	else
	{
		AVP_ServerEntity_MotionTrackerBlip* pxServerObject = new AVP_ServerEntity_MotionTrackerBlip(uGuid);

		if (pxServerObject == NULL)
		{
			Asura_Assert(false, "ToolsAVP_Entity_MotionTrackerBlip::Create() - no server object" );
		}
		else
		{
			pxServerObject->ToolsData = pxToolsData;

			ToolsAVP_Entity_MotionTrackerBlip* pxToolsObject = new ToolsAVP_Entity_MotionTrackerBlip(pxServerObject);

			// Create the game specific tools object and select it.
			if (pxToolsObject == NULL)
			{
				Asura_Assert(false, "ToolsAVP_Entity_MotionTrackerBlip::Create() - no tools object");
			}
			else
			{
				Tools_Object_System::SelectObject(pxToolsObject);

				pxToolsObject->Position		= *Asura_Camera::GetPosition();
				pxToolsObject->m_uGuid		= uGuid;

				pxServerObject->SetPosition(pxToolsObject->Position);

				Tools_Timed_Message_System::AddMessage(Tools_Timed_Message_System::COLOUR_INFO, "Created New Formation Point");
				Tools_SaveAsura::SetEntityModified();
				return pxToolsObject;
			}
		}
	}

	return NULL;
}

//-----------------------------------------------------------------------------
// ToolsAVP_Entity_MotionTrackerBlip :: Project_Create
//-----------------------------------------------------------------------------
void ToolsAVP_Entity_MotionTrackerBlip::Project_Create(void)
{
	ToolsAVP_Entity_MotionTrackerBlip::Create();
}

//-----------------------------------------------------------------------------
// ToolsAVP_Entity_MotionTrackerBlip :: CreateObjectOfThisType
//-----------------------------------------------------------------------------
Tools_Object* ToolsAVP_Entity_MotionTrackerBlip::CreateObjectOfThisType(void) const
{
	return CreateAndReturn();
}

//-----------------------------------------------------------------------------
// ToolsAVP_Entity_MotionTrackerBlip :: OnSelection
//-----------------------------------------------------------------------------
void ToolsAVP_Entity_MotionTrackerBlip::OnSelection(void)
{
	s_pxSelectedMotionTrackerBlip = this;
	PARENT::OnSelection();
}

//-----------------------------------------------------------------------------
// ToolsAVP_Entity_MotionTrackerBlip :: OnDeselection
//-----------------------------------------------------------------------------
void ToolsAVP_Entity_MotionTrackerBlip::OnDeselection(void)
{
	PARENT::OnDeselection();
	s_pxSelectedMotionTrackerBlip = NULL;
}

//-----------------------------------------------------------------------------
// ToolsAVP_Entity_MotionTrackerBlip :: IsBaseOrDerivativeOf
//-----------------------------------------------------------------------------
bool ToolsAVP_Entity_MotionTrackerBlip::IsBaseOrDerivativeOf( int iToolsObjectType ) const
{
	if ( iToolsObjectType == TOT_Entity_FormationPoint )
	{
		return true;
	}
	return PARENT::IsBaseOrDerivativeOf( iToolsObjectType );
}

//-----------------------------------------------------------------------------
// ToolsAVP_Entity_MotionTrackerBlip :: Render
//-----------------------------------------------------------------------------
void ToolsAVP_Entity_MotionTrackerBlip::Render( void )
{
	PARENT::Render();

	ToolsAVP_Node_GS2Node* pxGS2Node = ToolsAVP_Node_GS2Node::FindEntity( pxGameEntity->GetStartGS2NodeGuid() );

	if ( pxGS2Node )
	{
		Tools_Render::RenderCylinder( Position, pxGS2Node->Position, 0.0125f, 0xff662266, false );
	}
}

//-----------------------------------------------------------------------------
// ToolsAVP_Entity_MotionTrackerBlip :: UpdateWhileSelected
//-----------------------------------------------------------------------------
void ToolsAVP_Entity_MotionTrackerBlip::UpdateWhileSelected( void )
{
	PARENT::UpdateWhileSelected();

	if ( m_bChanged )
	{
		m_bChanged = false;
		DeleteOverlay();
		CreateOverlay();
	}
}

//-----------------------------------------------------------------------------
// ToolsAVP_Entity_MotionTrackerBlip :: AddOverlayControls
//-----------------------------------------------------------------------------
void ToolsAVP_Entity_MotionTrackerBlip::AddOverlayControls( class Asura_Overlay* pxOverlay, class Asura_Vector_2& xOffset )
{
	PARENT::AddOverlayControls( pxOverlay, xOffset );

	Asura_Dialog* pxDialog = NULL;

	pxDialog = new Asura_Function_Dialog( "Choose Start Node", xOffset, SelectStartNode );
	((Asura_Function_Dialog*)pxDialog)->SetUseSharedDialogColour( true );
	pxDialog->SetHelpText("Allows Selection Of The GS2 Node The Motion Tracker Blip Will Start At.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_StringDisplay_Dialog(" **NONE SELECTED**", xOffset );
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	if ( pxGameEntity->GetStartGS2NodeGuid() != ASURA_GUID_UNREGISTERED )
	{
		ToolsAVP_Node_GS2Node* pxGS2Node = ToolsAVP_Node_GS2Node::FindEntity( pxGameEntity->GetStartGS2NodeGuid() );

		if ( pxGS2Node )
		{
			pxDialog->UpdateLabel( pxGS2Node->GetName() );
		}
		else
		{
			pxDialog->UpdateLabel( " **ERROR** " );
		}
	}

	
	pxDialog = new Asura_Function_Dialog( "Clear Start Node", xOffset, ClearStartNode );
	((Asura_Function_Dialog*)pxDialog)->SetUseSharedDialogColour( true );
	pxDialog->SetHelpText("Clears the Selected GS2 Node The Motion Tracker Blip Will Start At.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	if ( pxGameEntity->GetStartGS2NodeGuid() == ASURA_GUID_UNREGISTERED )
	{
		pxDialog->SetEnabled( false );
	}
	else
	{
		pxDialog->SetEnabled( true );
	}
}

//-----------------------------------------------------------------------------
// ToolsAVP_Entity_MotionTrackerBlip :: AddOverlayControls
//-----------------------------------------------------------------------------
void ToolsAVP_Entity_MotionTrackerBlip::SelectStartNode( void )
{
	ToolsAVP_MTBChooseObject* pxChooseObject = new ToolsAVP_MTBChooseObject( SetSelectedStartNode );
	Tools_Object_System::ChooseObject( "Pick The Start GS2 Node Entity.", pxChooseObject );
}

//-----------------------------------------------------------------------------
// ToolsAVP_Entity_MotionTrackerBlip :: SetSelectedStartNode
//-----------------------------------------------------------------------------
void ToolsAVP_Entity_MotionTrackerBlip::SetSelectedStartNode( Asura_Guid uGuid )
{
	g_xParam_AVP_MTBlip_StartNode.SetValue( uGuid );
	s_pxSelectedMotionTrackerBlip->m_bChanged = true;
}

//-----------------------------------------------------------------------------
// ToolsAVP_Entity_MotionTrackerBlip :: ClearStartNode
//-----------------------------------------------------------------------------
void ToolsAVP_Entity_MotionTrackerBlip::ClearStartNode( void )
{
	g_xParam_AVP_MTBlip_StartNode.SetValue( ASURA_GUID_UNREGISTERED );
	s_pxSelectedMotionTrackerBlip->m_bChanged = true;
}

//-----------------------------------------------------------------------------
// ToolsAVP_Entity_MotionTrackerBlip :: GetParameter
//-----------------------------------------------------------------------------
bool ToolsAVP_Entity_MotionTrackerBlip::GetParameter( const Asura_Hash_ID uParamID, Tools_Parameter* pxParam ) const
{
	switch ( uParamID )
	{
		case g_uHash_AVP_MTBlip_StartNode:
		{
			pxParam->SetUInt( ( u_int )pxGameEntity->GetStartGS2NodeGuid() );
			return true;
		}break;

		default:
		{
			return PARENT::GetParameter( uParamID, pxParam );
		}break;
	}
}

//-----------------------------------------------------------------------------
// ToolsAVP_Entity_MotionTrackerBlip :: SetParameter
//-----------------------------------------------------------------------------
bool ToolsAVP_Entity_MotionTrackerBlip::SetParameter( const Asura_Hash_ID uParamID, const Tools_Parameter* pxParam )
{
	switch ( uParamID )
	{
		case g_uHash_AVP_MTBlip_StartNode:
		{
			pxGameEntity->SetStartGS2Node( ( Asura_Guid )pxParam->GetUInt() );
			m_bChanged = true;
			return true;
		}break;

		default:
		{
			return PARENT::SetParameter( uParamID, pxParam );
		}break;
	}
}

//-----------------------------------------------------------------------------
// ToolsAVP_Entity_MotionTrackerBlip :: CanBeSelected
//-----------------------------------------------------------------------------
bool ToolsAVP_MTBChooseObject::CanBeSelected( Tools_Object* pxObject )
{
	if ( pxObject->Type != TOT_Entity_AVPGS2Node )
	{
		return false;
	}

	AVP_ServerNode_GS2Node* pxNode = AVP_ServerNode_GS2Node::FindEntity( pxObject->GetGuid() );

	if ( ( !pxNode ) || ( pxNode->GetStructureType() != AVP_GS_ST_MOTION_TRACKER_BLIP ) )
	{
		return false;
	}

	return true;
}