//////////////////////////////////////////////////////////////////////////////
//
// ToolsAVP_Entity_RoomNode.h
//
// Created by AJK 13/01/2009
//
// Copyright (c) 2009 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////


// Includes
#include "Core/Asura.h"

// This file's header.
#include "Entity/ObjectiveSystem/RoomNode/ToolsAVP_Entity_RoomNode.h"

// Asura
#include "Core/Asura_UserInput.h"
#include "Mesh/Asura_Mesh.h"
#include "Mesh/Asura_Mesh_System.h"
#include "Rendering/Asura_Render.h"
#include "Volume/Asura_Volume_Cuboid.h"
#include "Volume/Asura_Volume_Cylinder.h"
#include "Volume/Asura_Volume_Sphere.h"

// PC rendering include for mesh rendering.
#include "Rendering/Asura_PlatformRenderObject.h"

// AVP
#include "Entity/ObjectiveSystem/RoomNode/AVP_RoomNode_ExitLink.h"

// Tools Asura
#include "Entity/Tools_Entity_Data.h"
#include "Load&Save/Tools_SaveAsura.h"
#include "Misc/Tools_LOS.h"
#include "Misc/Tools_Render.h"
#include "Misc/Tools_Timed_Messages.h"

// Class statics
// Shared params
TOOLS_DECLARE_SHARED_PARAM(bool,	0xCD150B60,	ExitLink_UseByMarine)    // "UseByMarine"
TOOLS_DECLARE_SHARED_PARAM(bool,	0x80721115,	ExitLink_UseByPredator)  // "UseByPredator"
TOOLS_DECLARE_SHARED_PARAM(bool,	0x504BED69,	ExitLink_UseByAlien)     // "UseByAlien"

ToolsAVP_Entity_RoomNode*					ToolsAVP_Entity_RoomNode::s_pxThis;
Asura_Hashed_List<ToolsAVP_Entity_RoomNode>	ToolsAVP_Entity_RoomNode::s_xList;
Asura_List<ToolsAVP_Object_ExitLink>        ToolsAVP_Object_ExitLink::s_xList;

bool  ToolsAVP_Entity_RoomNode::s_bRefreshOverlay          = false;
bool ToolsAVP_Entity_RoomNode::s_bMoveVolumeWithObject     = true;
bool ToolsAVP_Entity_RoomNode::s_bShowAllVolumes           = false;
bool ToolsAVP_Entity_RoomNode::s_bShowSelectedVolume       = false;
bool ToolsAVP_Entity_RoomNode::s_bMoveExitPointsWithEntity = true;
bool ToolsAVP_Object_ExitLink::s_bShowMergeMessage         = false;

ToolsAVP_Entity_RoomNode* ToolsAVP_Entity_RoomNode::s_pxLastSelected = NULL;

ToolsAVP_Entity_RoomNode::ToolsAVP_Entity_RoomNode( AVP_ServerEntity_RoomNode* pxRoomNode )
: PARENT( pxRoomNode )
, m_pxRoomNode( pxRoomNode )
, m_pxVolumeController( NULL )
, m_pszRoomNode(NULL)
, m_iSelectedRoomNode(0)
, m_numRoomStrings(0)
, m_uSelectedExitPoint(0)
{
	m_xPreviousPosition = Position;
	m_xPreviousOrientation = Orientation;

	
	m_pxExitPointMesh = Asura_Mesh_System::GetMesh("ToolsShape_Particle");

	Type = TOT_Entity_RoomNode;
	s_xList.AddToList(this);

	// Create Tools Exit points
	Asura_LinkList_It< AVP_RoomNode_ExitLink* > xLinkListIter( pxRoomNode->GetLinkList() );
	while ( !xLinkListIter.Done() )
	{
		// Create tool object
		ToolsAVP_Object_ExitLink* pxNewToolsExitLink = new ToolsAVP_Object_ExitLink();
		if ( pxNewToolsExitLink )
		{
			// Sync positions
			pxNewToolsExitLink->Position = (*xLinkListIter.GetCurrent())->GetPosition();
			pxNewToolsExitLink->SetExitLink( (*xLinkListIter.GetCurrent()) );
			pxNewToolsExitLink->SetSourceRoomGuid( GetGuid() );
		}

		xLinkListIter.Next();
	}

	// Set as modified if old version
	if ( pxRoomNode->IsOldVersion())
	{
		SetEntityModified();
	}
}

ToolsAVP_Entity_RoomNode::~ToolsAVP_Entity_RoomNode()
{
	delete m_pxVolumeController;

	for ( u_int uItr = 0; uItr < m_numRoomStrings; ++uItr )
	{
		delete [] m_pszRoomNode[ uItr ];
	}
	delete [] m_pszRoomNode;

	s_xList.RemoveFromList(this);
}

void ToolsAVP_Entity_RoomNode::Destroy()
{
	// Remove me from any any room that links to me
	// Destroy and delete tools and AVP objects
	if ( m_pxRoomNode )
	{
		Asura_LinkList_It< AVP_RoomNode_ExitLink* > xLinkListIter( m_pxRoomNode->m_xLinkList );
		while ( !xLinkListIter.Done() )
		{
			ToolsAVP_Object_ExitLink* pxToDestroy = ToolsAVP_Object_ExitLink::GetToolsObjectExitLink( *xLinkListIter.GetCurrent() );
			xLinkListIter.Next();
			pxToDestroy->Destroy();
		}
	}

	PARENT::Destroy();
}

void ToolsAVP_Entity_RoomNode::Create()
{
	const Asura_Guid uGuid = Asura_Entity_System::GetCreatedByToolsGuid();

	Tools_Entity_Data *pxToolsData = CreateToolsDataWithDefaultName("Room Node", "RoomNode", uGuid);

	if (pxToolsData == NULL)
	{
		Asura_Assert(false, "ToolsAVP_Entity_RoomNode::Create() - no tools data");
	}
	else
	{
		AVP_ServerEntity_RoomNode* pxServerObject = new AVP_ServerEntity_RoomNode(uGuid);

		if (pxServerObject == NULL)
		{
			Asura_Assert(false, "ToolsAVP_Entity_RoomNode::Create() - no server object" );
		}
		else
		{
			pxServerObject->ToolsData = pxToolsData;
			pxServerObject->SetPosition(pxToolsData->Position);

			ToolsAVP_Entity_RoomNode* pxToolsObject = new ToolsAVP_Entity_RoomNode(pxServerObject);

			if (pxToolsObject == NULL)
			{
				Asura_Assert(false, "ToolsAVP_Entity_RoomNode::Create() - no tools object");
			}
			else
			{
				Tools_Object_System::SelectObject(pxToolsObject);

				pxToolsObject->Position		= *Asura_Camera::GetPosition();
				pxToolsObject->m_uGuid		= uGuid;

				Tools_Timed_Message_System::AddMessage(Tools_Timed_Message_System::COLOUR_INFO, "Created New Room Node");
				Tools_SaveAsura::SetEntityModified();
			}
		}
	}
}

Tools_Object* ToolsAVP_Entity_RoomNode::Create( Asura_Entity* pxEntity )
{
	ToolsAVP_Entity_RoomNode* pxObject = new ToolsAVP_Entity_RoomNode( AVP_ServerEntity_RoomNode::FindEntity( pxEntity->GetGuid() ) );
	return pxObject;
}

void ToolsAVP_Entity_RoomNode::Copy( const Tools_Object* pxObject )
{
	PARENT::Copy( pxObject );

	ToolsAVP_Entity_RoomNode* pxFrom = ToolsAVP_Entity_RoomNode::FindEntity( pxObject->GetGuid() );
	if ( pxFrom )
	{
		m_xPreviousPosition    = pxFrom->Position;
		m_xPreviousOrientation = pxFrom->Orientation;

		m_pxRoomNode->m_xPosition = pxFrom->m_pxRoomNode->m_xPosition;
	}
}

Tools_Object* ToolsAVP_Entity_RoomNode::CreateObjectOfThisType() const
{
	const Asura_Guid uGuid = Asura_Entity_System::GetCreatedByToolsGuid();

	// Set up tools data and auto-create a unique name (prompt for new name if there is a clash).
	Tools_Entity_Data *pxToolsData = CreateToolsDataWithDefaultName("Room Node", "RoomNode", uGuid);

	if(pxToolsData == NULL)
	{
		Asura_Assert(false, "ToolsAVP_Entity_RoomNode::Create() - no tools data");
	}
	else
	{
		AVP_ServerEntity_RoomNode* pxServerObject = new AVP_ServerEntity_RoomNode(uGuid);
		pxServerObject->ToolsData = pxToolsData;
		ToolsAVP_Entity_RoomNode *pxToolsObject = new ToolsAVP_Entity_RoomNode(pxServerObject);
		Tools_Object_System::SelectObject(pxToolsObject);

		pxToolsObject->Position		= *Asura_Camera::GetPosition();
		pxToolsObject->Orientation.Identity();

		Tools_SaveAsura::SetEntityModified();
		return pxToolsObject;
	}
	return NULL;
}

void ToolsAVP_Entity_RoomNode::Render()
{
	PARENT::Render();

	Asura_LinkList_ConstIt< AVP_RoomNode_ExitLink* > xLinkListIter( m_pxRoomNode->m_xLinkList );
	while ( !xLinkListIter.Done() )
	{
		Asura_Vector_3   xPosition = (*xLinkListIter.GetCurrent())->GetPosition();
		Asura_Matrix_3x3 xOrientation;
		xOrientation.Identity();

		const AVP_ServerEntity_RoomNode* pxLinkedRoom = AVP_ServerEntity_RoomNode::FindEntity((*xLinkListIter.GetCurrent())->GetLinkedRoomGuid());
		if(pxLinkedRoom)
		{
			Asura_Render::RenderLine( Position, xPosition, 0xff00ff00 );
		}
		else
		{
			Asura_Render::RenderLine( Position, xPosition, 0xffffffff );
		}

		xLinkListIter.Next();
	}

	ShowVolume(s_bShowAllVolumes);
}

void ToolsAVP_Entity_RoomNode::UpdateWhileSelected()
{
	// Execute keyboard shortcuts
	if( !Asura_Overlay_System::OverlaysHaveKeyboardFocus() )
	{
		if(!Asura_UserInput::KeyPressed(ASURA_KEYID_ALT) && !Asura_UserInput::KeyPressed(ASURA_KEYID_CTRL))
		{
			if ( !Asura_UserInput::KeyPressed( ASURA_KEYID_SHIFT ) )
			{
				if(Asura_UserInput::DebouncedKeyPressed('g'))
				{
					EditVolume();
				}
				else if(Asura_UserInput::DebouncedKeyPressed('e'))
				{
					AddExitLink(1);
				}
			}
		}
	}

	if ( s_bRefreshOverlay )
	{
		DeleteOverlay();
		CreateOverlay();
		s_bRefreshOverlay = false;
	}

	PARENT::UpdateWhileSelected();
}

void ToolsAVP_Entity_RoomNode::UpdatePositionInAsuraData( bool bUpdateLinkedObjects )
{
	// Move the volume
	if(s_bMoveVolumeWithObject)
	{
		Asura_Vector_3 xOffset = Position-m_xPreviousPosition;
		m_pxRoomNode->GetVolumeGroup().DisplaceVolumes(xOffset);
		if(m_pxVolumeController)
		{
			m_pxVolumeController->MoveVolumeObjects(xOffset);
		}

		Asura_Euler_3 xLast;
		xLast.FromMatrix(m_xPreviousOrientation);
		Asura_Euler_3 xThis;
		xThis.FromMatrix(Orientation);
		Asura_Euler_3 xRot = xThis;
		xThis.Element[0]-=xLast.Element[0];
		xThis.Element[1]-=xLast.Element[1];
		xThis.Element[2]-=xLast.Element[2];
		Asura_Matrix_3x3 xAsMat;
		xThis.ToMatrix(xAsMat);

		if (xThis.MagnitudeSquared() > 0.0f)
		{
			m_pxRoomNode->GetVolumeGroup().RotateVolumesAbout(Position, xAsMat);
			if(m_pxVolumeController)
			{
				m_pxVolumeController->RotateVolumeObjectsAbout(Position, xAsMat);
			}
		}
	}

	// Move the exit links
	if(Position != Entity->ToolsData->Position || Orientation != Entity->ToolsData->m_xOrientation)
	{
		if (s_bMoveExitPointsWithEntity)
		{
			Asura_Quat xInverseOldOri(Entity->ToolsData->m_xOrientation);
			xInverseOldOri.Invert();
			Asura_Quat xNewOri(Orientation);
			const Asura_Vector_3& xOldPos = Entity->ToolsData->Position;
			const Asura_Vector_3& xNewPos = Position;

			Asura_LinkList_It< AVP_RoomNode_ExitLink* > xLinkListIter( m_pxRoomNode->m_xLinkList );
			while ( !xLinkListIter.Done() )
			{
				Asura_Vector_3& xPos = (*xLinkListIter.GetCurrent())->GetPosition();
				xPos -= xOldPos;
				xPos = xInverseOldOri.RotateVector(xPos);
				xPos = xNewOri.RotateVector(xPos);
				xPos += xNewPos;
				(*xLinkListIter.GetCurrent())->SetPosition( xPos );
				xLinkListIter.Next();
			}
		} 
	}

	m_pxRoomNode->SetPosition( Position );

	m_xPreviousOrientation = Orientation;
	m_xPreviousPosition    = Position;

	PARENT::UpdatePositionInAsuraData( bUpdateLinkedObjects );
}

void ToolsAVP_Entity_RoomNode::AddOverlayControls( Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset )
{
	s_pxLastSelected = this;

	UpdateRoomNodeListStrings();

	PARENT::AddOverlayControls( pxOverlay, xOffset );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	Asura_Dialog* pxDialog = new Asura_Tickbox_Dialog( "Show all volumes", xOffset, &s_bShowAllVolumes);
	pxDialog->SetHelpText("Shows volumes for all room nodes.");
	pxOverlay->AddDialog( pxDialog );
	xOffset.x =  1.0f;
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Tickbox_Dialog( "Show selected", xOffset, &s_bShowSelectedVolume);
	pxDialog->SetHelpText("Shows volume for last selected room node.");
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Tickbox_Dialog( "Move Volume With Object", xOffset, &s_bMoveVolumeWithObject);
	pxDialog->SetHelpText("The volume attached to this room node should be moved when the rooms tools object is moved.");
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::LargeSpacing();

	pxDialog = new Asura_Function_Dialog( "Edit volume ( g )", xOffset, EditVolume, &Changed );
	pxDialog->SetHelpText( "Edits the volume of the room the player will be in." );
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_StringDisplay_Dialog("Room Nodes Links", xOffset);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_List_Dialog(xOffset, m_pszRoomNode, &m_iSelectedRoomNode, (m_pxRoomNode->GetLinkedExitLinksCount() >= 8 ? 8 : m_pxRoomNode->GetLinkedExitLinksCount()), m_pxRoomNode->GetLinkedExitLinksCount(), NULL, &Changed);
	pxDialog->SetHelpText("rooms linked to this one");
	pxOverlay->AddDialog(pxDialog);
	pxDialog->SetEnabled( ( m_pxRoomNode->GetLinkedExitLinksCount() > 0 ) );
	xOffset.y = pxDialog->m_xMaxCoord.y + Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Add exit link ( e )", xOffset, AddExitLink, 0);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Remove Selected exit link to room node", xOffset, RemoveRoomNode);
	pxOverlay->AddDialog(pxDialog);
	pxDialog->SetEnabled( ( m_pxRoomNode->GetLinkedExitLinksCount() > 0 ) );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Tickbox_Dialog("Move Player Exit Points With Object", xOffset, &s_bMoveExitPointsWithEntity);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();
}

bool ToolsAVP_Entity_RoomNode::Verify( const bool bAutomatic )
{
	const bool bParentOk = PARENT::Verify( bAutomatic );
	bool bUsOk = true;
	Asura_LinkList_It< AVP_RoomNode_ExitLink* > xLinkListIter( m_pxRoomNode->m_xLinkList );
	while ( !xLinkListIter.Done() )
	{
		if ( (*xLinkListIter.GetCurrent())->GetLinkedRoomGuid() != ASURA_GUID_UNREGISTERED )
		{
			const AVP_ServerEntity_RoomNode* pxRoom = AVP_ServerEntity_RoomNode::FindEntity( (*xLinkListIter.GetCurrent())->GetLinkedRoomGuid() );
			if ( !pxRoom )
			{
				Tools_MessageLog::AddMessage( Tools_MessageLog::VERIFICATION, "Room nodes", new Tools_MessageLogAction_GotoEntity( GetGuid() ),
											  "Could not find room node #%u referenced in #%u.", (*xLinkListIter.GetCurrent())->GetLinkedRoomGuid(), GetGuid() );
				bUsOk = false;
			}
		}
		xLinkListIter.Next();
	}

	return bParentOk && bUsOk;
}

void ToolsAVP_Entity_RoomNode::UpdateRoomNodeListStrings()
{
	// reset our static array information
	if ( m_numRoomStrings > 0 )
	{
		for ( u_int uItr = 0; uItr < m_numRoomStrings; ++uItr )
		{
			delete [] m_pszRoomNode[ uItr ];
		}
	}

	delete [] m_pszRoomNode;
	m_pszRoomNode = NULL;

	m_numRoomStrings = m_pxRoomNode->GetLinkedExitLinksCount();
	if ( m_numRoomStrings > 0 )
	{
 		m_pszRoomNode = new char*[ m_numRoomStrings ];

		Asura_LinkList_ConstIt< AVP_RoomNode_ExitLink* > xLinkListIter( m_pxRoomNode->m_xLinkList );

		u_int u = 0;
		while ( !xLinkListIter.Done() )
		{
			if ( (*xLinkListIter.GetCurrent())->GetLinkedRoomGuid() != ASURA_GUID_UNREGISTERED )
			{
				Asura_String xString;
				ToolsAVP_Entity_RoomNode* pxToolsLinkedRoom = ToolsAVP_Entity_RoomNode::FindEntity( (*xLinkListIter.GetCurrent())->GetLinkedRoomGuid());
				if(pxToolsLinkedRoom)
				{
					xString = pxToolsLinkedRoom->GetName();
				}
				else
				{
					Asura_String xGuidString;
					Asura_StringUtil::ToString((*xLinkListIter.GetCurrent())->GetLinkedRoomGuid(), xGuidString );
					xString = "Error, GUID:" +  xGuidString;
				}
				m_pszRoomNode[ u ]   = new char[ xString.GetLength() + 1 ];
				Asura_PrintfToBfr( m_pszRoomNode[ u ], "%s", xString.GetLength() + 1, true, xString.GetString() );
				++u;
			}
			xLinkListIter.Next();
		}
	}
	else
	{
		m_pszRoomNode = new char*[1];
		m_pszRoomNode[0] = "No Nodes";
	}
}

void ToolsAVP_Entity_RoomNode::RemoveRoomNode( const Asura_Guid uNodeGuid )
{
	Asura_LinkList_ConstIt< AVP_RoomNode_ExitLink* > xLinkListIter( m_pxRoomNode->m_xLinkList );

	while ( !xLinkListIter.Done() )
	{
		if( (*xLinkListIter.GetCurrent())->GetLinkedRoomGuid() == uNodeGuid )
		{
			m_pxRoomNode->m_xLinkList.Remove(*xLinkListIter.GetCurrent());
			s_bRefreshOverlay = true;
			return;
		}
		xLinkListIter.Next();
	}
}

void ToolsAVP_Entity_RoomNode::AddExitLink(int iUserData)
{
	Asura_Assert(s_pxThis, "No RoomNode selected, should be impossible.");
	if (!s_pxThis) { return; }
	
    // Create a new Exit link
	AVP_RoomNode_ExitLink* pxNewExitLink = new AVP_RoomNode_ExitLink();
	if ( pxNewExitLink )
	{			
		Asura_Vector_3 xPosition;
		// Add an emerge point under the cursor.
		int iPolyIndex, iModuleIndex;
		if ( iUserData == 0 || !Tools_LOS::FindPolyAndPositionUnderCursor( iPolyIndex, iModuleIndex, xPosition ) )
		{
			xPosition = *Asura_Camera::GetPosition();				
		}
		
		pxNewExitLink->SetMyID( s_pxThis->m_pxRoomNode->GetMaxExitLinkID() + 1 );
		pxNewExitLink->SetPosition( xPosition );
		s_pxThis->m_pxRoomNode->m_xLinkList.Push_Back( pxNewExitLink );

		// Add the tools side exit link
		// Create the tools exit link, with position linked to exit link in AVP server entity
		ToolsAVP_Object_ExitLink* pxNewToolsExitLink = new ToolsAVP_Object_ExitLink();
		if ( pxNewToolsExitLink )
		{
			// Sync positions
			pxNewToolsExitLink->Position = xPosition;
			pxNewToolsExitLink->SetExitLink( pxNewExitLink );
			pxNewToolsExitLink->SetSourceRoomGuid( s_pxThis->GetGuid() );
		}
	}
	s_pxThis->SetEntityModified();
}

void ToolsAVP_Entity_RoomNode::RemoveRoomNode()
{
	Asura_LinkList_ConstIt< AVP_RoomNode_ExitLink* > xLinkListIter( s_pxThis->m_pxRoomNode->m_xLinkList );

	int i = 0;
	while ( !xLinkListIter.Done() )
	{
		// Must be a two way link
		if ( (*xLinkListIter.GetCurrent())->GetLinkedRoomGuid() != ASURA_GUID_UNREGISTERED  )
		{
			if ( i == s_pxThis->m_iSelectedRoomNode )
			{
				Asura_Guid uRoomGuid = (*xLinkListIter.GetCurrent())->GetLinkedRoomGuid();
				(*xLinkListIter.GetCurrent())->SetLinkedRoomGuid( ASURA_GUID_UNREGISTERED );

				// Set as modified
				s_pxThis->SetEntityModified();

				// Set other side as modified also
				ToolsAVP_Entity_RoomNode* pxOtherRoom = ToolsAVP_Entity_RoomNode::FindEntity( uRoomGuid );
				if ( pxOtherRoom )
				{
					AVP_RoomNode_ExitLink* pxExitLink = pxOtherRoom->m_pxRoomNode->GetExitLinkTo( s_pxThis->GetGuid(), (*xLinkListIter.GetCurrent())->GetMyID() );
					if( pxExitLink ) pxExitLink->SetLinkedRoomGuid( ASURA_GUID_UNREGISTERED );
					pxOtherRoom->SetEntityModified();
				}
				s_bRefreshOverlay = true;
				return;
			}
			++i;
		}
		xLinkListIter.Next();
	}
}

void ToolsAVP_Entity_RoomNode::OnSelection()
{
	s_pxThis = this;
	PARENT::OnSelection();
}

void ToolsAVP_Entity_RoomNode::OnDeselection()
{
	PARENT::OnDeselection();
	s_pxThis = NULL;
}

void ToolsAVP_Entity_RoomNode::ShowVolume(bool bShow)
{

	if(!m_pxVolumeController && !bShow && !s_bShowSelectedVolume) return;

	//create controller for editing volumes if it hasn't been done yet
	if(!m_pxVolumeController)
	{
		m_pxVolumeController = new ToolsAVP_RoomNode_VolumeEdit_Controller(this);
	}

	if (s_bShowSelectedVolume)
	{
		if (this == s_pxLastSelected) bShow =true;
	}

	//hide/show volume objects
	m_pxVolumeController->ShowVolumes(bShow || m_pxVolumeController->IsSelected());
}

void ToolsAVP_Entity_RoomNode::EditVolume()
{
	Asura_Assert(ThisEntity,"");

	ToolsAVP_Entity_RoomNode* pxToolsEntity = static_cast< ToolsAVP_Entity_RoomNode* > ( ThisEntity );

	if(!pxToolsEntity->m_pxVolumeController)
	{
		pxToolsEntity->m_pxVolumeController = new ToolsAVP_RoomNode_VolumeEdit_Controller(pxToolsEntity);
	}

	//and get it to create the volume objects
	pxToolsEntity->m_pxVolumeController->StartVolumeEditing();
}

ToolsAVP_RoomNode_VolumeEdit_Controller::ToolsAVP_RoomNode_VolumeEdit_Controller(ToolsAVP_Entity_RoomNode* pxToolsEntity)
:Tools_VolumeGroupEdit_Controller(&pxToolsEntity->Changed)
{
	m_pxToolsEntity = pxToolsEntity;
}

void ToolsAVP_RoomNode_VolumeEdit_Controller::StartVolumeEditing()
{
	Tools_VolumeGroupEdit_Controller::StartVolumeEditing();
	if(GetNumberOfVolumes() == 0)
	{
		//create a volume if none exists yet
		CreateVolumeCuboid();
	}
}

void ToolsAVP_RoomNode_VolumeEdit_Controller::EndVolumeEditing()
{
	Tools_VolumeGroupEdit_Controller::EndVolumeEditing();
	//select the owner
	Tools_Object_System::SelectObject(m_pxToolsEntity);
}

void ToolsAVP_RoomNode_VolumeEdit_Controller::CreateVolumeCuboidMain()
{
	Asura_Volume_Cuboid* pxVolume = new Asura_Volume_Cuboid();
	Asura_Bounding_Box xBox;
	xBox.MinX = m_pxToolsEntity->Position.x-1.0f;
	xBox.MaxX = m_pxToolsEntity->Position.x+1.0f;
	xBox.MinY = m_pxToolsEntity->Position.y-4.0f;
	xBox.MaxY = m_pxToolsEntity->Position.y+1.0f;
	xBox.MinZ = m_pxToolsEntity->Position.z-4.0f;
	xBox.MaxZ = m_pxToolsEntity->Position.z+4.0f;
	pxVolume->SetBoundingBox(xBox);
	m_pxToolsEntity->m_pxRoomNode->GetVolumeGroup().AddVolume(pxVolume);
}

void ToolsAVP_RoomNode_VolumeEdit_Controller::CreateVolumeSphereMain()
{
	Asura_Volume_Sphere* pxVolume = new Asura_Volume_Sphere();
	pxVolume->SetPosition(m_pxToolsEntity->Position);
	m_pxToolsEntity->m_pxRoomNode->GetVolumeGroup().AddVolume(pxVolume);
}

void ToolsAVP_RoomNode_VolumeEdit_Controller::CreateVolumeCylinderMain()
{
	Asura_Volume_Cylinder* pxVolume = new Asura_Volume_Cylinder();
	pxVolume->SetPosition(m_pxToolsEntity->Position);
	m_pxToolsEntity->m_pxRoomNode->GetVolumeGroup().AddVolume(pxVolume);
}

void ToolsAVP_RoomNode_VolumeEdit_Controller::DeleteVolumeMain(int iIndex)
{
	m_pxToolsEntity->m_pxRoomNode->GetVolumeGroup().RemoveAndDestroyVolume(static_cast<u_int>(iIndex));
}

ToolsAVP_Object_ExitLink::ToolsAVP_Object_ExitLink()
{
    SetMesh( "ToolsShape_CutsceneTrackPoint" );
	s_xList.AddToList(this);
}

void ToolsAVP_Object_ExitLink::AddOverlayControls( Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset )
{
	PARENT::AddOverlayControls(pxOverlay, xOffset);

	if ( m_pxExitLink )
	{
		bool bEnable =  ( m_pxExitLink->GetLinkedRoomGuid() != ASURA_GUID_UNREGISTERED );

		PARENT::AddOverlayControls( pxOverlay, xOffset );
		xOffset.y += Asura_Dialog::DefaultSpacing();

		Asura_Dialog* pxDialog = new Asura_SharedTickbox_Dialog("Use by Aliens", xOffset, &g_xParam_ExitLink_UseByAlien);
		pxDialog->SetHelpText("Objective route by this exit point used by aliens only");
		pxOverlay->AddDialog(pxDialog);
		pxDialog->SetEnabled( bEnable );
		xOffset.y += Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_SharedTickbox_Dialog("Use by Marines", xOffset, &g_xParam_ExitLink_UseByMarine);
		pxDialog->SetHelpText("Objective route by this exit point used by marines only");
		pxOverlay->AddDialog(pxDialog);
		pxDialog->SetEnabled( bEnable );
		xOffset.y += Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_SharedTickbox_Dialog("Use by Predators", xOffset, &g_xParam_ExitLink_UseByPredator);
		pxDialog->SetHelpText("Objective route by this exit point used by predators only");
		pxOverlay->AddDialog(pxDialog);
		pxDialog->SetEnabled( bEnable );
		xOffset.y += Asura_Dialog::DefaultSpacing();
	}
}

void ToolsAVP_Object_ExitLink::Render()
{
	extern void Render_Sphere( const Asura_Vector_3& xCentre, float fRadius, int iColour );

	PARENT::Render();

	// WHITE  0xffffffff
	// YELLOW 0xffffff00
	// BLUE   0xff0000ff
	// GREEN  0xff00ff00
	// RED    0xffff0000
	// ORANGE 0xffff8c0e
	// PURPLE 0xffff00ff
	// BLACK  0xff000000;

	int iSpeciesColour = 0xffffffff;
	// Render a sphere around the exit link based on the species allowed to cross it
	if ( m_pxExitLink && m_pxExitLink->GetLinkedRoomGuid() != ASURA_GUID_UNREGISTERED )
	{
		if( ( m_pxExitLink->IsUsedByAlien() && m_pxExitLink->IsUsedByPredator() && m_pxExitLink->IsUsedByMarine() ) )
		{
			// White
			iSpeciesColour = 0xffffffff;
			Render_Sphere( m_pxExitLink->GetPosition(), 0.22f, iSpeciesColour );
		}
		else if (  m_pxExitLink->IsUsedByAlien() && !m_pxExitLink->IsUsedByPredator() && !m_pxExitLink->IsUsedByMarine() )
		{
			// Yellow
			iSpeciesColour = 0xffffff00;
			Render_Sphere( m_pxExitLink->GetPosition(), 0.22f, iSpeciesColour );
		}
		else if ( m_pxExitLink->IsUsedByMarine() && !m_pxExitLink->IsUsedByAlien() && !m_pxExitLink->IsUsedByPredator() )
		{
			// Blue
			iSpeciesColour = 0xff0000ff;
			Render_Sphere( m_pxExitLink->GetPosition(), 0.22f, iSpeciesColour );
		}
		else if ( m_pxExitLink->IsUsedByPredator() && !m_pxExitLink->IsUsedByMarine() && !m_pxExitLink->IsUsedByAlien() )
		{
			// Red
			iSpeciesColour = 0xffff0000;
			Render_Sphere( m_pxExitLink->GetPosition(), 0.22f, iSpeciesColour );
		}
		else if ( m_pxExitLink->IsUsedByPredator() && m_pxExitLink->IsUsedByAlien() )
		{
			// Orange
			iSpeciesColour = 0xffff8c0e;
			Render_Sphere( m_pxExitLink->GetPosition(), 0.22f, iSpeciesColour );
		}
		else if ( m_pxExitLink->IsUsedByMarine() && m_pxExitLink->IsUsedByAlien() )
		{
			// Green
			iSpeciesColour = 0xff00ff00;
			Render_Sphere( m_pxExitLink->GetPosition(), 0.22f, iSpeciesColour );
		}
		else if ( m_pxExitLink->IsUsedByMarine() && m_pxExitLink->IsUsedByPredator() )
		{
			// Purple
			iSpeciesColour = 0xffff00ff;
			Render_Sphere( m_pxExitLink->GetPosition(), 0.22f, iSpeciesColour );
		}

		// Render a linking line between the two exit links
		AVP_ServerEntity_RoomNode* pxRoomNode = AVP_ServerEntity_RoomNode::FindEntity( m_pxExitLink->GetLinkedRoomGuid());
		if ( pxRoomNode )
		{
			const AVP_RoomNode_ExitLink* pxOtherExitLink = pxRoomNode->GetExitLinkTo( GetSourceRoomGuid(), m_pxExitLink->GetMyID() );
			if ( pxOtherExitLink )
			{
				// Render line with same colour 
				Asura_Render::RenderLine( m_pxExitLink->GetPosition(), pxOtherExitLink->GetPosition(), iSpeciesColour );
			}

		}
	}
}

void ToolsAVP_Object_ExitLink::Update()
{
	if ( m_pxExitLink )
	{
		Position = m_pxExitLink->GetPosition();
	}
}

void ToolsAVP_Object_ExitLink::UpdatePositionInAsuraData( bool bUpdateLinkedObjects )
{
	if ( m_pxExitLink )
	{
		m_pxExitLink->SetPosition( Position );
	}
}

bool ToolsAVP_Object_ExitLink::GetParameter(Asura_Hash_ID uParamID,class Tools_Parameter* pxParam) const
{
	if ( m_pxExitLink )
	{
		switch(uParamID)
		{
			case g_uHash_ExitLink_UseByAlien :
				pxParam->SetBool(m_pxExitLink->IsUsedByAlien());
				break;

			case g_uHash_ExitLink_UseByMarine :
				pxParam->SetBool(m_pxExitLink->IsUsedByMarine());
				break;

			case g_uHash_ExitLink_UseByPredator :
				pxParam->SetBool(m_pxExitLink->IsUsedByPredator());
				break;

			default :
				return false;
		}
		return true;
	}
	return false;
}

bool ToolsAVP_Object_ExitLink::SetParameter(Asura_Hash_ID uParamID,const class Tools_Parameter* pxParam)
{
	switch(uParamID)
	{
		case g_uHash_ExitLink_UseByAlien :
			( pxParam->GetBool() ) ? m_pxExitLink->SetUseByAlien(true) : m_pxExitLink->SetUseByAlien(false);
			break;

		case g_uHash_ExitLink_UseByMarine :
			( pxParam->GetBool() ) ? m_pxExitLink->SetUseByMarine(true) : m_pxExitLink->SetUseByMarine(false);
			break;

		case g_uHash_ExitLink_UseByPredator :
			( pxParam->GetBool() ) ? m_pxExitLink->SetUseByPredator(true) : m_pxExitLink->SetUseByPredator(false);
			break;

		default :
			return false;
	}
	return true;
}

void ToolsAVP_Object_ExitLink::UpdateWhileSelected()
{
	PARENT::UpdateWhileSelected();

	// If two exit points are selected
	// Execute keyboard shortcuts
	if( !Asura_Overlay_System::OverlaysHaveKeyboardFocus() )
	{
		if(!Asura_UserInput::KeyPressed(ASURA_KEYID_ALT) && !Asura_UserInput::KeyPressed(ASURA_KEYID_CTRL))
		{
			if ( !Asura_UserInput::KeyPressed( ASURA_KEYID_SHIFT ) )
			{
				if(Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_SPACE))
				{
					MergeExitLinks(true);
					return;
				}
			}
		}
	}
	MergeExitLinks(false);

	// Set Tools objects related to this exit link as modified
	if ( m_pxExitLink )
	{
		ToolsAVP_Entity_RoomNode* pxRoomNode = ToolsAVP_Entity_RoomNode::FindEntity(m_pxExitLink->GetLinkedRoomGuid());
		if ( pxRoomNode ) pxRoomNode->SetEntityModified();
	}

	return;
}

void ToolsAVP_Object_ExitLink::MergeExitLinks(bool bTryMerge)
{
	// Test if only two exit links are selected at any time
	u_int u = 0;
	Asura_List_It<ToolsAVP_Object_ExitLink> xExitLinkIter(*ToolsAVP_Object_ExitLink::GetList());
	for ( ; !xExitLinkIter.Done(); xExitLinkIter.Next() )
	{
		if ( xExitLinkIter.GetCurrent()->IsGroupSelected() )
		{
			++u;
		}
	}

	if ( u != 2 )
	{
		return; // only 2 nodes must be selected
	}

	// check if this not an already linked exit link
	if ( !m_pxExitLink || ( m_pxExitLink->GetLinkedRoomGuid() != ASURA_GUID_UNREGISTERED && bTryMerge ) )
	{
		Tools_Timed_Message_System::AddMessage( 0xffff00,"Cannot merge an already connected exit link!");
		return;
	}

	// Merge exit points (only the ones that have room2 unset if two are selected and select the new merged one
	ToolsAVP_Object_ExitLink* pxOtherExitLink = NULL;
	xExitLinkIter.Restart();
	u = 0;
	// Loop through all, and get other exit link if exists;
	for ( ; !xExitLinkIter.Done() && u <= 1; xExitLinkIter.Next() )
	{
		if ( xExitLinkIter.GetCurrent()->IsGroupSelected() )
		{
			if ( xExitLinkIter.GetCurrent()->GetExitLink() && ( xExitLinkIter.GetCurrent()->GetSourceRoomGuid() != GetSourceRoomGuid() ) )
			{
				if ( xExitLinkIter.GetCurrent()->GetExitLink()->GetLinkedRoomGuid() == ASURA_GUID_UNREGISTERED )
				{
					++u;
					pxOtherExitLink = xExitLinkIter.GetCurrent();
				}
			}
		}
	}
	
	if ( u == 1 )
	{
		if ( s_bShowMergeMessage )
		{
			Tools_Timed_Message_System::AddMessage( 0xffff00,"Pressing space will merge the two selected exit links");
			s_bShowMergeMessage = false;
		}
		if ( bTryMerge )
		{
			// Link
			pxOtherExitLink->m_pxExitLink->SetLinkedRoomGuid( GetSourceRoomGuid());
			pxOtherExitLink->m_pxExitLink->SetLinkedExitLinkID( m_pxExitLink->GetMyID());
			m_pxExitLink->SetLinkedRoomGuid( pxOtherExitLink->GetSourceRoomGuid());
			m_pxExitLink->SetLinkedExitLinkID( pxOtherExitLink->m_pxExitLink->GetMyID());
		}
	}
}

void ToolsAVP_Object_ExitLink::OnSelection()
{
	s_bShowMergeMessage = true;
	DeleteObjectOverlay();
	CreateObjectOverlay();
}

void ToolsAVP_Object_ExitLink::OnDeselection()
{
	s_bShowMergeMessage = true;
	DeleteObjectOverlay();
}

ToolsAVP_Object_ExitLink::~ToolsAVP_Object_ExitLink()
{
	s_xList.RemoveFromList(this);
}

void ToolsAVP_Object_ExitLink::Destroy()
{
	// Remove me from any any room that links to me
	if ( m_pxExitLink )
	{		
		// Set other room as modified, and modify it's link to unregistred
		if ( m_pxExitLink->GetLinkedRoomGuid() != ASURA_GUID_UNREGISTERED )
		{
			// Two way exit link (Destroy the exit link and its tool counterpart, and get the other room node and change it's exit link room to unregistred )
			ToolsAVP_Entity_RoomNode* pxToolEntity = ToolsAVP_Entity_RoomNode::FindEntity( m_pxExitLink->GetLinkedRoomGuid() );
			if ( pxToolEntity )
			{
				Asura_LinkList_It< AVP_RoomNode_ExitLink* > xOtherLinkListIter( pxToolEntity->GetRoomNode()->GetLinkList() );
				while ( !xOtherLinkListIter.Done() )
				{
					if ( ( (*xOtherLinkListIter.GetCurrent())->GetLinkedRoomGuid() == GetSourceRoomGuid() ) && ( (*xOtherLinkListIter.GetCurrent())->GetMyID() == m_pxExitLink->GetLinkedExitLinkID() ) )
					{
						(*xOtherLinkListIter.GetCurrent())->SetLinkedRoomGuid( ASURA_GUID_UNREGISTERED );
						break;
					}
					xOtherLinkListIter.Next();
				}
				pxToolEntity->SetEntityModified();		
			}
		}

		RemoveMeFromRoomNodeList( GetSourceRoomGuid() );
	}

	// Delete overlay
	DeleteObjectOverlay();

	// Delete AVP object
	delete m_pxExitLink;

	PARENT::Destroy();
}

void ToolsAVP_Object_ExitLink::RemoveMeFromRoomNodeList(Asura_Guid uRoomGuid) const
{
	AVP_ServerEntity_RoomNode* pxRoomNode = AVP_ServerEntity_RoomNode::FindEntity(  uRoomGuid );
	if ( pxRoomNode )
	{
		Asura_LinkList_It< AVP_RoomNode_ExitLink* > xLinkListIter( pxRoomNode->GetLinkList() );
		while ( !xLinkListIter.Done() )
		{
			if( *xLinkListIter.GetCurrent() == m_pxExitLink )
			{
				ToolsAVP_Entity_RoomNode* pxToolEntity = ToolsAVP_Entity_RoomNode::FindEntity( pxRoomNode->GetGuid() );
				if (pxToolEntity)
				{
					pxToolEntity->SetEntityModified();
				}
				xLinkListIter.DeleteCurrent();
				break;
			}
			xLinkListIter.Next();
		}
	}
}

ToolsAVP_Object_ExitLink* ToolsAVP_Object_ExitLink::GetToolsObjectExitLink(const AVP_RoomNode_ExitLink* pxExitLink )
{
	Asura_List_It<ToolsAVP_Object_ExitLink> xExitLinkIter(*ToolsAVP_Object_ExitLink::GetList());

	// Loop through all, and get other exit link if exists;
	for ( ; !xExitLinkIter.Done(); xExitLinkIter.Next() )
	{
		if ( xExitLinkIter.GetCurrent()->GetExitLink() == pxExitLink)
		{
			return xExitLinkIter.GetCurrent();
		}
	}
	return NULL;
}