//////////////////////////////////////////////////////////////////////////////
//
// ToolsAVP_Entity_Actor.cpp
//
// Created by NJD - 22 November 2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"

// header
#include "Entity/PhysicalObject/Actor/ToolsAVP_Entity_Actor.h"

// asura
#include "Entity/Locations/OrientedNode/GamesceneNode/Asura_ServerNode_GS2Node.h"
#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Overlay.h"
#include "Rendering/Asura_Render.h"
#include "Rendering/Asura_Text_Output.h"

// avp
#include "Blueprint/AVP_Blueprint_HashIDs.h"
#include "Entity/PhysicalObject/Actor/AVP_ServerEntity_Actor.h"
#include "Entity/AVP_Messages.h"
#include "Mesh/Asura_Mesh_System.h"

// axon
#include "Core/BrainTypes/AVP_Brain_Actor.h"

// tools
#include "Tools_System.h"
#include "Entity/Tools_Entity_Data.h"
#include "Hierarchy/Tools_Animation_Files.h"
#include "Load&Save/Tools_SaveAsura.h"
#include "Misc/Tools_Timed_Messages.h"
#include "Objects/Axon/Tools_Object_Axon_ChooseObjects.h"
#include "Objects/Axon/Tools_Object_Axon_Interface.h"
#include "Objects/Tools_ObjectSystem_Param.h"

// tools avp
#include "ToolsAVP_ChooseObjects.h"
#include "Objects/ToolsAVP_Object_Type.h"

TOOLS_DECLARE_SHARED_PARAM( bool, uTOOLS_SHAREDPARAMHASH_ACTOR_DO_PANIC_MECHANIC,  Actor_DoPanicMechanic )

//////////////////////////////////////////////////////////////////////////////
//
// ToolsAVP_Entity_Actor
//
//////////////////////////////////////////////////////////////////////////////

ToolsAVP_Entity_Actor* ToolsAVP_Entity_Actor::s_pxCurrentObject = NULL;

ToolsAVP_Entity_Actor::ToolsAVP_Entity_Actor( AVP_ServerEntity_Actor* pxActor )
: Tools_Entity_PhysicalObject( pxActor )
, PARENT(pxActor)
, m_pxActor( pxActor )
, m_bIsDeadBody( m_pxActor->GetIsDeadBody() )
, m_bInitialGSNodeChanged( false )
, m_bSupportGS2( true )
{
	Type = TOT_Actor;

	if ( m_bIsDeadBody )
	{
		SetDeadSkinAndAnim();
	}
	SetPhysicalObjectFlags( GetPhysicalObjectFlags() | ASURA_PHYSICALOBJECT_FLAG_NOSHADOW );
}

void ToolsAVP_Entity_Actor::SetNotSupportGS2()
{
	m_bSupportGS2 = false;
}

void ToolsAVP_Entity_Actor::SetDeadSkinAndAnim()
{
	if (m_pxActor)
	{
		Asura_Hash_ID uSkinID;
		Asura_Blueprint* pxActorBP = m_pxPhysicalObject->GetBlueprint();
		if ( pxActorBP )
		{
			Asura_Value xValue;
			if ( pxActorBP->GetParameterValueByHashID( 0xbe7b9f98, xValue ) ) // Asura_GetHashID( "ThirdPerson.Skin.Main" )
			{
				uSkinID = xValue.GetHashIDValue();
			}
			SetSkinAndAnim(uSkinID, m_pxActor->GetDeadAnimHash());
		}
	}
}

void ToolsAVP_Entity_Actor::Destroy()
{
	m_pxActor->SetInitialGamesceneGuid( ASURA_GUID_UNREGISTERED );
	if ( Tools_Object_Axon_Interface::GetInstance() )
	{
		Tools_Object_Axon_Interface::GetInstance()->EstablishGS2ObjectOwnerData();
	}

	PARENT::Destroy();
}

bool ToolsAVP_Entity_Actor::IsSelectable()
{
	//Prevent actor objects from being selected outside of object mode , to prevent crashes
	//from editing them while the axon tools aren't set up.
	//RAR 01/04/2009
	if(!(Tools_System::IsInObjectMode() || Tools_Object_System::IsChoosingObject())) return false;
	return PARENT::IsSelectable();
}

void ToolsAVP_Entity_Actor::OnSelection()
{
    PARENT::OnSelection();
    s_pxCurrentObject = this;
}

void ToolsAVP_Entity_Actor::OnDeselection()
{
    s_pxCurrentObject = NULL;
    PARENT::OnDeselection();
}

void ToolsAVP_Entity_Actor::UpdateWhileSelected()
{
    PARENT::UpdateWhileSelected();
    if ( m_bInitialGSNodeChanged )
    {
        m_bInitialGSNodeChanged = false;
        m_pxActor->InitFromBlueprint();

		Tools_Object_Axon_Interface::GetInstance()->EstablishGS2ObjectOwnerData();
        Tools_SaveAsura::SetEntityModified();
    }
}

void ToolsAVP_Entity_Actor::AddOverlayControls( Asura_Overlay* pxNewOverlay, Asura_Vector_2& xOffset )
{
	Asura_Dialog* pxDialog;

	PARENT::AddOverlayControls( pxNewOverlay, xOffset );
	xOffset.y += Asura_Dialog::LargeSpacing() - Asura_Dialog::DefaultSpacing();

	if(m_pxActor->GetBlueprint())
	{
		Asura_Tickbox_Dialog* pxTDialog = new Asura_Tickbox_Dialog( "Start actor as dead?", xOffset, &m_bIsDeadBody );
		pxTDialog->SetHelpText( "If this is ticked the actor starts in dead position, is dead, and can be ragdolled" );
		pxTDialog->SetClickedFunction( ActorDeadChanged, reinterpret_cast< int >( this ) );
		pxNewOverlay->AddDialog(pxTDialog);
		xOffset.y += Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_Function_Dialog("Select Dead Anim", xOffset, SelectAnim_Dialog);
		pxDialog->SetIsEnabledFunction(IsActorDead);
		pxDialog->m_iControlID = Asura_GetHashID("SelectDeadAnimation");
		pxDialog->SetHelpText("Select the dead pose anim, but before a blueprint must be selected.");
		pxNewOverlay->AddDialog(pxDialog);
		xOffset.y += Asura_Dialog::DefaultSpacing();
	}

	if ( m_pxActor->GetBrain() && m_bSupportGS2 )
	{
		Asura_String xGS2NodeString( "Initial GS2 Node: " );
		if ( m_pxActor->GetInitialGamesceneGuid() == ASURA_GUID_UNREGISTERED )
		{
			xGS2NodeString += "not set";
		}
		else
		{
			Asura_ServerNode_GS2Node* pxGS2StartNode = Asura_ServerNode_GS2Node::FindEntity( m_pxActor->GetInitialGamesceneGuid() );
			if ( pxGS2StartNode )
			{
				xGS2NodeString += pxGS2StartNode->ToolsData->Name.GetString();
			}
			else
			{
				xGS2NodeString += "NOT FOUND";
			}
		}

		pxDialog = new Asura_Function_Dialog( xGS2NodeString.GetString(), xOffset, SetInitialGamesceneNode );
		pxNewOverlay->AddDialog( pxDialog );
		xOffset.y += Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_Function_Dialog( "Clear Initial GS2 Node", xOffset, ClearInitialGamesceneNode );
		pxNewOverlay->AddDialog( pxDialog );
		xOffset.y += Asura_Dialog::DefaultSpacing();
	}

	UpdateDeadAnimation(pxNewOverlay);
}

bool ToolsAVP_Entity_Actor::IsActorDead()
{
	return s_pxCurrentObject->m_bIsDeadBody;
}

void ToolsAVP_Entity_Actor::SelectAnim_Dialog()
{
	Asura_Assert(s_pxCurrentObject, "ToolsAVP_Entity_Actor::SelectAnim_Dialog() called with no object selected");
	if (s_pxCurrentObject)
	{
		s_pxCurrentObject->SelectAnim();
	}
}

void ToolsAVP_Entity_Actor::SelectAnim()
{
	if (m_pxPhysicalObject)
	{
		m_pxPhysicalObject->InitFromBlueprint();
		Asura_Blueprint* pxActorBP = m_pxPhysicalObject->GetBlueprint();

		if ( pxActorBP )
		{
			Asura_Value xValue;
			if ( pxActorBP->GetParameterValueByHashID( 0xbe7b9f98, xValue ) ) // Asura_GetHashID( "ThirdPerson.Skin.Main" )
			{
				Asura_Hash_ID uSkinID;
				uSkinID = xValue.GetHashIDValue();

				const Asura_Hierarchy_Skin* pxSkin =  Tools_Animation_Files::GetSkinFromID( uSkinID );
				if(pxSkin) 
				{
					Asura_String xSkinName;
					xSkinName= pxSkin->GetName();			

					Asura_Assert(IsSelected(), "SelectAnim() called on non-selected object");
					if (!IsSelected()) return;

					Asura_Assert(m_pxPhysicalObject, "Tools_Entity_Lift has no m_pxPhysicalObject");
					if (!m_pxPhysicalObject) return;

					Asura_String xAnimName;
					Asura_String xAnimFilename;
					Asura_Hierarchy_Anim* pxInitialAnim = 0;
					if(m_pxPhysicalObject->GetAnimID())
					{
						pxInitialAnim = Asura_Animation_System::GetAnimByHash(m_pxPhysicalObject->GetAnimID());
					}

					Asura_Hash_ID uAnimHashID = ASURA_HASH_ID_UNSET;  

					Asura_Hierarchy_Anim* pxAnim = Tools_Animation_Files::ChooseAnimation(xSkinName.GetString(), NULL, &xAnimName, &xAnimFilename);
					if (!pxAnim)
					{
						return;
					}

					uAnimHashID = Asura_GetHashID(xAnimName.GetString());
					SetSkinAndAnim(uSkinID, uAnimHashID);
					Changed = true;
					(static_cast<AVP_ServerEntity_Actor*> (m_pxPhysicalObject))->m_uDeadAnimHashID = uAnimHashID;
				}
				else
				{
					Asura_Assert(false, "Blueprint has incorrect skin!");
				}
			}
		}
	}
}

void ToolsAVP_Entity_Actor::UpdateDeadAnimation(Asura_Overlay* pxNewOverlay)
{
	if(m_pxActor)
	{
		char szText[100];

		if ( m_pxActor && m_pxActor->GetDeadAnimHash() != ASURA_HASH_ID_UNSET)
		{
			Asura_String xFilename(Asura_HashedFilename_System::GetFilename(m_pxActor->GetDeadAnimHash()));
			Asura_PrintfToBfr(szText,"Dead Anim : %s",sizeof(szText),true,xFilename.GetString());	
		}
		else
		{
			strcpy(szText,"Select Dead Anim");
		}
		pxNewOverlay->UpdateDialogLabel(Asura_GetHashID("SelectDeadAnimation"),szText);
	}
}

void ToolsAVP_Entity_Actor::AddFilesToExportList(List<Asura_String>& xNameList) const
{
	PARENT::AddFilesToExportList(xNameList);

	if ( m_pxActor && m_pxActor->GetDeadAnimHash() != ASURA_HASH_ID_UNSET)
	{
		Asura_String xFilename(Asura_HashedFilename_System::GetFullFilename(m_pxActor->GetDeadAnimHash()));
		if (xFilename.GetLength() > 0)
		{
			Tools_Level_Export::AddFileToExportList(xNameList, xFilename, false);
		}
	}
}

void ToolsAVP_Entity_Actor::UpdatePositionInAsuraData( bool bUpdateLinkedObjects )
{
	PARENT::UpdatePositionInAsuraData( bUpdateLinkedObjects );
	if ( m_pxActor )
	{
		m_pxActor->m_xDeadPos = Position;
		m_pxActor->m_xDeadOri = Orientation;
	}
}

bool ToolsAVP_Entity_Actor::HandlesThisMessage( int iMessageID )
{
    if ( iMessageID == AVP_MSGID_CHUNK ||
		 iMessageID == AVP_MSGID_SET_PREFERRED_TARGET_TYPE ||
		 iMessageID == AVP_MSGID_SET_AI_TARGET_PRIORITIES ||
		 iMessageID == AVP_MSGID_RESET_AI_TARGET_PRIORITIES || 
		 iMessageID == AVP_MSGID_ATTACH_WEAPON ||
		 iMessageID == AVP_MSGID_DETACH_WEAPON ||
		 iMessageID == AVP_MSGID_CREATE_ATTACHMENT ||
		 iMessageID == AVP_MSGID_DESTROY_ATTACHMENT ||
		 iMessageID == AVP_MSGID_DETACH_ATTACHMENT ||
		 iMessageID == AVP_MSGID_DISMEMBER )
    {
        return true;
    }
    else
    {
        return PARENT::HandlesThisMessage( iMessageID );
    }
}

bool ToolsAVP_Entity_Actor::HandlesThisEvent( int iEventID ) const
{
	if ( iEventID == AVP_ENTITY_EVENT_GS2_ACTION_ANIM_FLAG_HIT )
	{
		return true;
	}
	else
	{
		return PARENT::HandlesThisEvent( iEventID );
	}
}

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

void ToolsAVP_Entity_Actor::SetInitialGamesceneNode()
{
	ToolsAVP_ChooseObject_ByType* pxChooseObject = new ToolsAVP_ChooseObject_ByType( TOT_Entity_AVPGS2Node, SetInitialGamesceneNodeToThis );
    if ( pxChooseObject )
    {
		pxChooseObject->AddType( TOT_Entity_AVPGS2Attractor );
        Tools_Object_System::ChooseObject( "Choose The GS2 Node To Start At", pxChooseObject );
    }
}

void ToolsAVP_Entity_Actor::SetInitialGamesceneNodeToThis( Asura_Guid uGuid )
{
	s_pxCurrentObject->m_pxActor->m_uInitialGamesceneGuid = uGuid;
	s_pxCurrentObject->m_bInitialGSNodeChanged = true;
	s_pxCurrentObject->RefreshOverlay();
}

void ToolsAVP_Entity_Actor::ClearInitialGamesceneNode()
{
    s_pxCurrentObject->m_pxActor->m_uInitialGamesceneGuid = ASURA_GUID_UNREGISTERED;
    s_pxCurrentObject->m_bInitialGSNodeChanged = true;
	s_pxCurrentObject->RefreshOverlay();
}

void ToolsAVP_Entity_Actor::Copy(const Tools_Object* pxObject)
{
	PARENT::Copy( pxObject );

	if (pxObject)
	{
		// copy the tools shape
		Mesh = pxObject->Mesh;

		ToolsAVP_Entity_Actor* pxFromObject = static_cast< ToolsAVP_Entity_Actor* >( ToolsAVP_Entity_PhysicalObject::FindEntity( pxObject->GetGuid() ) );
		m_bIsDeadBody = pxFromObject->m_bIsDeadBody;

		AVP_ServerEntity_Actor* pxFromEntity = AVP_ServerEntity_Actor::FindEntity(pxObject->GetGuid());
		AVP_ServerEntity_Actor* pxToEntity = m_pxActor;

		if (pxFromEntity && pxToEntity)
		{
			pxToEntity->SetBlueprintHashID( pxFromEntity->GetBlueprintHashID() );
			pxToEntity->SetBlueprintTypeHashID( pxFromEntity->GetBlueprintTypeHashID() );
			pxToEntity->SetInitialGamesceneGuid( pxFromEntity->GetInitialGamesceneGuid() );
			pxToEntity->SetInitialSquadGuid( pxFromEntity->GetInitialSquadGuid() );
			pxToEntity->m_bDeadBody		  = pxFromEntity->m_bDeadBody;
			pxToEntity->m_uDeadAnimHashID = pxFromEntity->m_uDeadAnimHashID;
			if( pxToEntity->m_bDeadBody)
			{
				SetDeadSkinAndAnim();
			}
		}
	}
}

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

bool ToolsAVP_Entity_Actor::GetParameter(Asura_Hash_ID uParamID, class Tools_Parameter* pxParam) const
{
	switch(uParamID)
	{
        case g_uHash_Actor_DoPanicMechanic:
		    pxParam->SetBool( m_pxActor->GetActorBrain()->GetDoPanicMechanicOnGamesceneExit() );
		    break;

	    default:
		    return PARENT::GetParameter(uParamID, pxParam);
	}
	return true;
}

bool ToolsAVP_Entity_Actor::SetParameter(Asura_Hash_ID uParamID, const class Tools_Parameter* pxParam)
{
	switch(uParamID)
	{
	    case g_uHash_Actor_DoPanicMechanic:
        {
            m_pxActor->GetActorBrain()->SetDoPanicMechanicOnGamesceneExit( pxParam->GetBool() );
            m_pxActor->InitFromBlueprint();
            SetEntityModified();
		    return true;
        }

	    default:
        {
		    return PARENT::SetParameter(uParamID, pxParam);
        }
	}
}

Asura_Hash_ID ToolsAVP_Entity_Actor::GetDefaultBlueprintTypeHashID() const
{
    return uAVP_BLUEPRINT_TYPE_ACTORS;
}

Asura_Hash_ID ToolsAVP_Entity_Actor::GetDefaultBlueprintHashID() const
{
    return 0x0585a9f5; // Asura_GetHashID( "Actor" )
}

void ToolsAVP_Entity_Actor::Render()
{
	PARENT::Render();

	if ( m_pxActor->GetInitialGamesceneGuid() != ASURA_GUID_UNREGISTERED )
	{
		// Draw a line to our initial gamescene node.
		const Asura_ServerNode_GS2Node* pxGS2Node = Asura_ServerNode_GS2Node::FindEntity( m_pxActor->GetInitialGamesceneGuid() );
		if ( pxGS2Node )
		{
			Asura_Render::RenderLine( m_pxActor->GetPosition(), pxGS2Node->GetPosition(), 0xffffffff );
		}
	}
}

bool ToolsAVP_Entity_Actor::Verify( const bool bAutomatic )
{
	bool bOk = PARENT::Verify( bAutomatic );

	if ( !m_pxActor->GetBlueprint() )
	{
		Tools_MessageLog::AddMessage( Tools_MessageLog::VERIFICATION, "Actors", new Tools_MessageLogAction_GotoEntity( GetGuid() ),
			"Cannot find the blueprint for actor #%u. Is it set?", m_pxActor->GetGuid() );
		bOk = false;
	}

	if ( m_bIsDeadBody && m_pxActor->GetDeadAnimHash() == ASURA_HASH_ID_UNSET )
	{
		Tools_MessageLog::AddMessage( Tools_MessageLog::VERIFICATION, "Actors", new Tools_MessageLogAction_GotoEntity( GetGuid() ),
			"Actor #%u is a dead body but does not have a dead anim.", m_pxActor->GetGuid() );
		bOk = false;
	}

	if ( m_pxActor->GetInitialGamesceneGuid() != ASURA_GUID_UNREGISTERED )
	{
		const Asura_ServerNode_GS2Node* pxGS2Node = Asura_ServerNode_GS2Node::FindEntity( m_pxActor->GetInitialGamesceneGuid() );
		if ( !pxGS2Node )
		{
			Tools_MessageLog::AddMessage( Tools_MessageLog::VERIFICATION, "Actors", new Tools_MessageLogAction_GotoEntity( GetGuid() ),
				"Cannot find start gamescene node #%u for actor #%u.", m_pxActor->GetInitialGamesceneGuid(), m_pxActor->GetGuid() );
			bOk = false;
		}
	}

	Asura_Blueprint* pxBlueprint = Asura_Blueprint_System::GetBlueprintByHashID( m_pxActor->GetBlueprintTypeHashID(), m_pxActor->GetBlueprintHashID() );
	if ( !pxBlueprint )
	{
		Tools_MessageLog::AddMessage( Tools_MessageLog::VERIFICATION, "Actors", new Tools_MessageLogAction_GotoEntity( GetGuid() ),
			"Could not find blueprint set on Actor with guid #%u.", m_pxActor->GetGuid() );
		bOk = false;
	}

	return bOk;
}

void ToolsAVP_Entity_Actor::ActorDeadChanged( int iThis )
{
	ToolsAVP_Entity_Actor* pxThis = reinterpret_cast< ToolsAVP_Entity_Actor* >( iThis );
	if ( pxThis->m_bIsDeadBody )
	{
		pxThis->m_pxActor->SetIsDeadBody( true );
		pxThis->m_pxActor->ShutdownAndDestroyBrain();
		pxThis->SetDeadSkinAndAnim();
	} 
	else
	{
		pxThis->m_pxActor->SetIsDeadBody( false );
		pxThis->m_pxActor->CreateAndInitialiseBrain();
		// We probably only need to initialise the brain again, but there's no harm in making sure everything's up to date.
		pxThis->m_pxActor->InitFromBlueprint();
		pxThis->RestoreMesh();
	}
	pxThis->SetEntityModified();
}

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////
