//////////////////////////////////////////////////////////////////////////////
//
// FILE : ToolsAVP_Entity_Alien.cpp
//
// Inherits from Tools_Entity_PhysicalObject
//
// AUTHORS : Chris Wallace
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"

// header
#include "Entity/PhysicalObject/Actor/Vertebrates/Alien/ToolsAVP_Entity_Alien.h"

// asura
#include "Camera/Asura_Camera.h"
#include "Mesh/Asura_Mesh_System.h"

// avp
#include "Actor/Vertebrates/Alien/AVP_Alien_Description.h"
#include "Blueprint/AVP_Blueprint_HashIDs.h"
#include "Entity/Controller/AVP_ServerEntity_Squad.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Alien/AVP_ServerActor_Alien.h"

// tools
#include "Entity/Tools_Entity_Data.h"
#include "Load&Save/Tools_SaveAsura.h"
#include "Misc/Tools_Timed_Messages.h"
#include "Objects/Tools_Object_System.h"
#include "Particles/Tools_Mode_PFX.h"

// tools avp
#include "ToolsAVP_ChooseObjects.h"
#include "Objects/ToolsAVP_Object_Type.h"

//////////////////////////////////////////////////////////////////////////////
//
// ToolsAVP_Entity_Alien
//
//////////////////////////////////////////////////////////////////////////////

ToolsAVP_Entity_Alien* ToolsAVP_Entity_Alien::s_pxThisToolsAlien = NULL;

ToolsAVP_Entity_Alien::ToolsAVP_Entity_Alien(AVP_ServerActor_Alien* pxActor)
: Tools_Entity_PhysicalObject(pxActor)
, PARENT( pxActor )
{
	Asura_String mesh_name = "ToolsShape_Alien";
	Mesh = Asura_Mesh_System::GetMesh(mesh_name);

	Type = TOT_Entity_Alien;
}

void ToolsAVP_Entity_Alien::RestoreMesh()
{
	// Restore original mesh
	SetMesh(Asura_Mesh_System::GetMesh("ToolsShape_Alien"));
}

void ToolsAVP_Entity_Alien::Project_Create()
{
    ToolsAVP_Entity_Alien::Create( true );
}

Tools_Object* ToolsAVP_Entity_Alien::Create( Asura_Entity* pxEntity )
{
    return new ToolsAVP_Entity_Alien( AVP_ServerActor_Alien::FindEntity(pxEntity->GetGuid()) );
}

Tools_Object* ToolsAVP_Entity_Alien::Create( const bool bFullInit )
{
	// Get the user to pick a blueprint to initialise from.
	Asura_Hash_ID uBlueprintTypeHashID = uAVP_BLUEPRINT_TYPE_ACTORS;
	Asura_Hash_ID uBlueprintHashID     = 0xc97f0d2f;
	if ( bFullInit )
	{
		BlueprintSelect_Overlay xBPOverlay( "Choose A Blueprint", &uBlueprintHashID, &uBlueprintTypeHashID );
		if ( !xBPOverlay.DoModal() ) { return NULL; }
	}

	const Asura_Guid uGuid = Asura_Entity_System::GetCreatedByToolsGuid();

	// Set up tools data and auto-create a unique name (prompt for new name if there is a clash).
	Tools_Entity_Data* pxToolsData = CreateToolsDataWithDefaultName("Alien", "Alien", uGuid);
	if ( !pxToolsData ) { return NULL; }

	AVP_ServerActor_Alien* pxServerObject = new AVP_ServerActor_Alien(uGuid);
	if ( !pxServerObject )
	{
		delete pxToolsData;
		return NULL;
	}

	// Set the tools data before we create the tools object - otherwise it will have already been created!
	pxServerObject->ToolsData = pxToolsData;
	if ( bFullInit )
	{
		pxServerObject->SetPosition( pxToolsData->Position );
		pxServerObject->SetBlueprintTypeHashID( uBlueprintTypeHashID );
		pxServerObject->SetBlueprintHashID( uBlueprintHashID );
		pxServerObject->MakeAI();
		pxServerObject->CreateAndInitialiseBrain();
		pxServerObject->InitFromBlueprint();
	}

	ToolsAVP_Entity_Alien* pxToolsObject = new ToolsAVP_Entity_Alien( pxServerObject );
	if ( !pxToolsObject )
	{
		delete pxToolsObject;
		delete pxServerObject;
		return NULL;
	}
	Tools_Object_System::SelectObject(pxToolsObject);

	Tools_Timed_Message_System::AddMessage( Tools_Timed_Message_System::COLOUR_DEFAULT, "Created new Alien." );
	Tools_SaveAsura::SetEntityModified();

	return pxToolsObject;
}

void ToolsAVP_Entity_Alien::AddFilesToExportList(List<Asura_String>& nameList) const
{
	// Call the base-function...
	PARENT::AddFilesToExportList(nameList);

	// Export the necessary alien data
	AVP_Alien_Description::AddFilesToExport(false, m_uBlueprintHashID, nameList);
}

void ToolsAVP_Entity_Alien::OnSelection()
{
	s_pxThisToolsAlien = this;
	PARENT::OnSelection();
}

void ToolsAVP_Entity_Alien::OnDeselection()
{
	PARENT::OnDeselection();
	s_pxThisToolsAlien = NULL;
}

void ToolsAVP_Entity_Alien::AddOverlayControls( Asura_Overlay* pxNewOverlay, Asura_Vector_2& xOffset )
{
	PARENT::AddOverlayControls( pxNewOverlay, xOffset );

	Asura_String xSquadString( "Initial Squad: " );
	if ( GetAlien()->GetInitialSquadGuid() == ASURA_GUID_UNREGISTERED )
	{
		xSquadString += "not set";
	}
	else
	{
		AVP_ServerEntity_Squad* pxSquad = AVP_ServerEntity_Squad::FindEntity( GetAlien()->GetInitialSquadGuid() );
		if ( pxSquad )
		{
			xSquadString += pxSquad->ToolsData->Name.GetString();
		}
		else
		{
			xSquadString += "NOT FOUND";
		}
	}

	Asura_Dialog* pxDialog = new Asura_Function_Dialog( xSquadString.GetString(), xOffset, SetSquad, &Changed);
	pxNewOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Clear Initial Squad", xOffset, ClearSquad, &Changed);
	pxNewOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();
}

AVP_ServerActor_Alien* ToolsAVP_Entity_Alien::GetAlien() const
{
	return static_cast< AVP_ServerActor_Alien* >( AVP_ServerActor_Alien::FindEntity( GetGuid() ) );
}

void ToolsAVP_Entity_Alien::SetSquad()
{
	ToolsAVP_ChooseObject_ByType* pxChooseObject = new ToolsAVP_ChooseObject_ByType(TOT_Entity_AlienSquad, SetThisSquad);
	Tools_Object_System::ChooseObject("Select The Squad To Join", pxChooseObject);
}

void ToolsAVP_Entity_Alien::ClearSquad()
{
	s_pxThisToolsAlien->GetAlien()->SetInitialSquadGuid( ASURA_GUID_UNREGISTERED );
	s_pxThisToolsAlien->RefreshOverlay();
}

void ToolsAVP_Entity_Alien::SetThisSquad( Asura_Guid uSquadGuid )
{
	s_pxThisToolsAlien->GetAlien()->SetInitialSquadGuid( uSquadGuid );
	s_pxThisToolsAlien->RefreshOverlay();
}
