//////////////////////////////////////////////////////////////////////////////
//
// FILE : ToolsAVP_Entity_FaceHugger.cpp
//
// Inherits from Tools_Entity_PhysicalObject
//
// AUTHORS : Chris Wallace
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"

// header
#include "Entity/PhysicalObject/Actor/Vertebrates/Alien/ToolsAVP_Entity_FaceHugger.h"

// asura
#include "Mesh/Asura_Mesh_System.h"

// avp
#include "Actor/Vertebrates/FaceHugger/AVP_FaceHugger_Description.h"
#include "Entity/AVP_Messages.h"
#include "Blueprint/AVP_Blueprint_HashIDs.h"
#include "Entity/Controller/AVP_ServerEntity_Squad_FaceHugger.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/FaceHugger/AVP_ServerActor_FaceHugger.h"
#include "Camera/Asura_Camera.h"

// tools avp
#include "ToolsAVP_ChooseObjects.h"
#include "Entity/Tools_Entity_Data.h"
#include "Load&Save/Tools_SaveAsura.h"
#include "Misc/Tools_Timed_Messages.h"
#include "Objects/ToolsAVP_Object_Type.h"
#include "Objects/Tools_Object_System.h"

//////////////////////////////////////////////////////////////////////////////
//
// ToolsAVP_Entity_FaceHugger
//
//////////////////////////////////////////////////////////////////////////////

ToolsAVP_Entity_FaceHugger* ToolsAVP_Entity_FaceHugger::s_pxThisTool = NULL;

ToolsAVP_Entity_FaceHugger::ToolsAVP_Entity_FaceHugger(AVP_ServerActor_FaceHugger* pxActor)
: Tools_Entity_PhysicalObject(pxActor)
, PARENT( pxActor )
, m_pxFaceHugger(pxActor)
{
	Asura_String mesh_name = "ToolsShape_FaceHugger";
	Mesh = Asura_Mesh_System::GetMesh(mesh_name);

	Type = TOT_Entity_FaceHugger;
}

bool ToolsAVP_Entity_FaceHugger::HandlesThisMessage( int iMessageID )
{
	switch ( iMessageID )
	{
		case AVP_MSGID_FACEHUGGER_HUG_ENTITY:
			return true;
		default:
			return PARENT::HandlesThisMessage( iMessageID );
	}
}


void ToolsAVP_Entity_FaceHugger::RestoreMesh()
{
	// Restore original mesh
	SetMesh(Asura_Mesh_System::GetMesh("ToolsShape_FaceHugger"));
}

void ToolsAVP_Entity_FaceHugger::Project_Create()
{
    ToolsAVP_Entity_FaceHugger::Create( true );
}

Tools_Object* ToolsAVP_Entity_FaceHugger::Create( Asura_Entity* pxEntity )
{
    return new ToolsAVP_Entity_FaceHugger( AVP_ServerActor_FaceHugger::FindEntity(pxEntity->GetGuid()) );
}

Tools_Object* ToolsAVP_Entity_FaceHugger::Create( const bool bFullInit )
{
	// Get the user to pick a blueprint to initialise from.
	Asura_Hash_ID uBlueprintTypeHashID = uAVP_BLUEPRINT_TYPE_ACTORS;
	Asura_Hash_ID uBlueprintHashID     = 0x4633fcb7;
	if ( bFullInit )
	{
		BlueprintSelect_Overlay xBPOverlay( "Choose A Blueprint", &uBlueprintHashID, &uBlueprintTypeHashID );
		if ( !xBPOverlay.DoModal() ) { return NULL; }
	}

	const Asura_Guid uGuid = Asura_Entity_System::GetCreatedByToolsGuid();

	// Set up tools data and auto-create a unique name (prompt for new name if there is a clash).
	Tools_Entity_Data* pxToolsData = CreateToolsDataWithDefaultName("FaceHugger", "FaceHugger", uGuid);
	if ( !pxToolsData ) { return NULL; }

	AVP_ServerActor_FaceHugger* pxServerObject = new AVP_ServerActor_FaceHugger(uGuid);
	if ( !pxServerObject )
	{
		delete pxToolsData;
		return NULL;
	}

	// Set the tools data before we create the tools object - otherwise it will have already been created!
	pxServerObject->ToolsData = pxToolsData;
	if ( bFullInit )
	{
		pxServerObject->SetPosition( pxToolsData->Position );
		pxServerObject->SetBlueprintTypeHashID( uBlueprintTypeHashID );
		pxServerObject->SetBlueprintHashID( uBlueprintHashID );
		pxServerObject->MakeAI();
		pxServerObject->CreateAndInitialiseBrain();
		pxServerObject->InitFromBlueprint();
	}

	ToolsAVP_Entity_FaceHugger* pxToolsObject = new ToolsAVP_Entity_FaceHugger( pxServerObject );
	if ( !pxToolsObject )
	{
		delete pxToolsObject;
		delete pxServerObject;
		return NULL;
	}
	Tools_Object_System::SelectObject(pxToolsObject);

	Tools_Timed_Message_System::AddMessage( Tools_Timed_Message_System::COLOUR_DEFAULT, "Created new FaceHugger." );
	Tools_SaveAsura::SetEntityModified();

	return pxToolsObject;
}

void ToolsAVP_Entity_FaceHugger::AddOverlayControls( Asura_Overlay* pxNewOverlay, Asura_Vector_2& xOffset )
{
	PARENT::AddOverlayControls( pxNewOverlay, xOffset );

	Asura_String xSquadString( "Initial Squad: " );
	if ( m_pxFaceHugger->GetInitialSquadGuid() == ASURA_GUID_UNREGISTERED )
	{
		xSquadString += "not set";
	}
	else
	{
		AVP_ServerEntity_Squad_FaceHugger* pxSquad = AVP_ServerEntity_Squad_FaceHugger::FindEntity( m_pxFaceHugger->GetInitialSquadGuid() );
		if ( pxSquad )
		{
			xSquadString += pxSquad->ToolsData->Name.GetString();
		}
		else
		{
			xSquadString += "NOT FOUND";
		}
	}

	Asura_Dialog* pxDialog = new Asura_Function_Dialog( xSquadString.GetString(), xOffset, SetSquad, &Changed);
	pxNewOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Clear Initial Squad", xOffset, ClearSquad, &Changed);
	pxNewOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

}

void ToolsAVP_Entity_FaceHugger::OnSelection()
{
	Validate();

	s_pxThisTool = this;
	PARENT::OnSelection();
}

void ToolsAVP_Entity_FaceHugger::OnDeselection()
{
	PARENT::OnDeselection();
	s_pxThisTool = NULL;
}


void ToolsAVP_Entity_FaceHugger::AddFilesToExportList(List<Asura_String>& nameList) const
{
	// Call the base-function...
	PARENT::AddFilesToExportList(nameList);

	// Export the FaceHugger data...
	AVP_FaceHugger_Description::AddFilesToExport(false, m_uBlueprintHashID, nameList);
}

void ToolsAVP_Entity_FaceHugger::SetSquad()
{
	ToolsAVP_ChooseObject_ByType* pxChooseObject = new ToolsAVP_ChooseObject_ByType(TOT_Entity_FaceHuggerSquad, SetThisSquad);
	Tools_Object_System::ChooseObject("Select The Squad To Join", pxChooseObject);
}

void ToolsAVP_Entity_FaceHugger::ClearSquad()
{
	s_pxThisTool->m_pxFaceHugger->SetInitialSquadGuid( ASURA_GUID_UNREGISTERED );
	s_pxThisTool->RefreshOverlay();
}

void ToolsAVP_Entity_FaceHugger::SetThisSquad( Asura_Guid uSquadGuid )
{
	s_pxThisTool->m_pxFaceHugger->SetInitialSquadGuid( uSquadGuid );
	s_pxThisTool->RefreshOverlay();
}

void ToolsAVP_Entity_FaceHugger::Validate()
{
	Asura_Guid uSquadGuid = m_pxFaceHugger->GetInitialSquadGuid();

	if (uSquadGuid != ASURA_GUID_UNREGISTERED && !AVP_ServerEntity_Squad_FaceHugger::FindEntity(uSquadGuid))
	{
		m_pxFaceHugger->SetInitialSquadGuid( ASURA_GUID_UNREGISTERED );
		Tools_MessageLog::AddMessage( "Face-Hugger", "Missing Dependent Squad", new Tools_MessageLogAction_GotoEntity( GetGuid() ), "Could not retrieve Face-Hugger Squad %u, reseting to default squad", uSquadGuid );
	}
}


/// End of file. ///
