//-----------------------------------------------------------------------------
// File		: ToolsAVP_Entity_Civilian.cpp
// Author	: Gareth Thomas
// Created	: 18 / 12 / 07
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// INCLUDES
//-----------------------------------------------------------------------------
// Core
#include "Core/Asura.h"

// Header
#include "Entity/PhysicalObject/Actor/Vertebrates/Human/ToolsAVP_Entity_Civilian.h"

// Asura
#include "Blueprint/Asura_Blueprint_System.h"
#include "Camera/Asura_Camera.h"
#include "Entity/Tools_Entity_Data.h"
#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Overlay.h"
#include "Load&Save/Tools_SaveAsura.h"
#include "Mesh/Asura_Mesh_System.h"
#include "Objects/Tools_Object_System.h"
#include "Rendering/Asura_Render.h"

// AvP
#include "ToolsAVP_ChooseObjects.h"
#include "Blueprint/AVP_Blueprint_HashIDs.h"
#include "Entity/AVP_Entity_EventIDs.h"
#include "Entity/Locations/GuardZone/AVP_ServerNode_GuardZone.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Human/Civilian/AVP_ServerActor_Civilian.h"
#include "Misc/Tools_Render.h"
#include "Misc/Tools_Timed_Messages.h"
#include "Objects/ToolsAVP_Object_Type.h"

ToolsAVP_Entity_Civilian*						ToolsAVP_Entity_Civilian::s_pxSelectedCivilian		= NULL;	
Asura_Hashed_List<ToolsAVP_Entity_Civilian>		ToolsAVP_Entity_Civilian::s_xList;

//-----------------------------------------------------------------------------
// ToolsAVP_Entity_Civilian :: Constructor
//-----------------------------------------------------------------------------
ToolsAVP_Entity_Civilian::ToolsAVP_Entity_Civilian(AVP_ServerActor_Civilian* pxActor)
: Tools_Entity_PhysicalObject(pxActor)
, PARENT(pxActor)
, m_pxCivilian(pxActor)
{
	Type = TOT_Entity_Civilian;

	s_pxSelectedCivilian = this;

	Mesh	= Asura_Mesh_System::GetMesh("ToolsShape_Civilian");

	s_xList.AddToList(this);
}

void ToolsAVP_Entity_Civilian::RestoreMesh()
{
	// Restore original mesh
	SetMesh(Asura_Mesh_System::GetMesh("ToolsShape_Civilian"));
}


//-----------------------------------------------------------------------------
// ToolsAVP_Entity_Civilian :: Destructor
//-----------------------------------------------------------------------------
ToolsAVP_Entity_Civilian::~ToolsAVP_Entity_Civilian()
{
	s_xList.RemoveFromList(this);
}

//-----------------------------------------------------------------------------
// ToolsAVP_Entity_Civilian :: Project_Create
//-----------------------------------------------------------------------------
void ToolsAVP_Entity_Civilian::Project_Create()
{
	ToolsAVP_Entity_Civilian::Create( true );
}

//-----------------------------------------------------------------------------
// ToolsAVP_Entity_Civilian :: Create
//-----------------------------------------------------------------------------
Tools_Object* ToolsAVP_Entity_Civilian::Create(Asura_Entity* pxEntity)
{
	return new ToolsAVP_Entity_Civilian(AVP_ServerActor_Civilian::FindEntity(pxEntity->GetGuid()));
}

//-----------------------------------------------------------------------------
// ToolsAVP_Entity_Civilian :: Create
//-----------------------------------------------------------------------------
Tools_Object* ToolsAVP_Entity_Civilian::Create( const bool bFullInit )
{
	// Get the user to pick a blueprint to initialise from.
	Asura_Hash_ID uBlueprintTypeHashID = uAVP_BLUEPRINT_TYPE_ACTORS;
	Asura_Hash_ID uBlueprintHashID     = 0x05f0612d;
	if ( bFullInit )
	{
		BlueprintSelect_Overlay xBPOverlay( "Choose A Blueprint", &uBlueprintHashID, &uBlueprintTypeHashID );
		if ( !xBPOverlay.DoModal() ) { return NULL; }
	}

	const Asura_Guid uGuid = Asura_Entity_System::GetCreatedByToolsGuid();

	// Set up tools data and auto-create a unique name (prompt for new name if there is a clash).
	Tools_Entity_Data* pxToolsData = CreateToolsDataWithDefaultName("Civilian", "Civilian", uGuid);
	if ( !pxToolsData ) { return NULL; }

	AVP_ServerActor_Civilian* pxServerObject = new AVP_ServerActor_Civilian(uGuid);
	if ( !pxServerObject )
	{
		delete pxToolsData;
		return NULL;
	}

	// Set the tools data before we create the tools object - otherwise it will have already been created!
	pxServerObject->ToolsData = pxToolsData;
	if ( bFullInit )
	{
		pxServerObject->SetPosition( pxToolsData->Position );
		pxServerObject->SetBlueprintTypeHashID( uBlueprintTypeHashID );
		pxServerObject->SetBlueprintHashID( uBlueprintHashID );
		pxServerObject->MakeAI();
		pxServerObject->CreateAndInitialiseBrain();
		pxServerObject->InitFromBlueprint();
	}

	ToolsAVP_Entity_Civilian* pxToolsObject = new ToolsAVP_Entity_Civilian( pxServerObject );
	if ( !pxToolsObject )
	{
		delete pxToolsObject;
		delete pxServerObject;
		return NULL;
	}
	Tools_Object_System::SelectObject(pxToolsObject);

	Tools_Timed_Message_System::AddMessage( Tools_Timed_Message_System::COLOUR_DEFAULT, "Created new Civilian." );
	Tools_SaveAsura::SetEntityModified();

	return pxToolsObject;
}

void ToolsAVP_Entity_Civilian::Copy(const Tools_Object* pxObject)
{
	PARENT::Copy(pxObject);

	ToolsAVP_Entity_Civilian* pxFromObject = ToolsAVP_Entity_Civilian::FindEntity( pxObject->GetGuid() );

	if (pxFromObject)
	{
		AVP_ServerActor_Civilian* pxFromEntity = pxFromObject->m_pxCivilian;
		AVP_ServerActor_Civilian* pxToEntity = m_pxCivilian;

		if (pxFromEntity && pxToEntity)
		{
			pxToEntity->SetGuardzoneGuid( pxFromEntity->GetGuardZoneGuid() );
		}
	}
}

//-----------------------------------------------------------------------------
// ToolsAVP_Entity_Civilian :: OnSelection
//-----------------------------------------------------------------------------
void ToolsAVP_Entity_Civilian::OnSelection()
{
	PARENT::OnSelection();

	s_pxSelectedCivilian = this;
}

//-----------------------------------------------------------------------------
// ToolsAVP_Entity_Civilian :: OnDeselection
//-----------------------------------------------------------------------------
void ToolsAVP_Entity_Civilian::OnDeselection()
{
	s_pxSelectedCivilian = NULL;

	PARENT::OnDeselection();
}

//-----------------------------------------------------------------------------
// ToolsAVP_Entity_Civilian :: AddOverlayControls
//-----------------------------------------------------------------------------
void ToolsAVP_Entity_Civilian::Render(void)
{
	PARENT::Render();

	AVP_ServerActor_Civilian* pxActor = AVP_ServerActor_Civilian::FindEntity(GetGuid());

	if (pxActor)
	{
		if (pxActor->GetGuardZoneGuid() != ASURA_GUID_UNREGISTERED)
		{
			AVP_ServerNode_GuardZone* pxGuardZone = AVP_ServerNode_GuardZone::FindEntity(pxActor->GetGuardZoneGuid());

			if (pxGuardZone)
			{
				Tools_Render::RenderCylinder(Position, pxGuardZone->GetPosition(), 0.05f, 0xffff8800, false);
			}
		}
	}
}

//-----------------------------------------------------------------------------
// ToolsAVP_Entity_Civilian :: SelectGuardZoneAddLink
//-----------------------------------------------------------------------------
void ToolsAVP_Entity_Civilian::SelectGuardZoneAddLink(void)
{
	ToolsAVP_ChooseObject_ByType* pxChooseObject = new ToolsAVP_ChooseObject_ByType(TOT_Entity_GuardZone, SetGuardZoneGuid);
	Tools_Object_System::ChooseObject("Select The Guard Zone Entity", pxChooseObject);
}

//-----------------------------------------------------------------------------
// ToolsAVP_Entity_Civilian :: SelectGuardZoneAddLink
//-----------------------------------------------------------------------------
void ToolsAVP_Entity_Civilian::SelectGuardZoneRemoveLink(void)
{
	if (!s_pxSelectedCivilian)
		return;

	AVP_ServerActor_Civilian* pxCivilianActor = AVP_ServerActor_Civilian::FindEntity(s_pxSelectedCivilian->GetGuid());

	if (pxCivilianActor)
	{
		pxCivilianActor->SetGuardzoneGuid(ASURA_GUID_UNREGISTERED);
	}
}

//-----------------------------------------------------------------------------
// ToolsAVP_Entity_Civilian :: SetGuardZoneGuid
//-----------------------------------------------------------------------------
void ToolsAVP_Entity_Civilian::SetGuardZoneGuid(Asura_Guid uGuid)
{
	if (!s_pxSelectedCivilian)
		return;

	AVP_ServerActor_Civilian* pxCivilianActor = AVP_ServerActor_Civilian::FindEntity(s_pxSelectedCivilian->GetGuid());

	if (pxCivilianActor)
	{
		pxCivilianActor->SetGuardzoneGuid(uGuid);
	}
}

//-----------------------------------------------------------------------------
// ToolsAVP_Entity_Civilian :: SetGuardZoneGuid
//-----------------------------------------------------------------------------
bool ToolsAVP_Entity_Civilian::HandlesThisEvent(int iEventID) const
{
	switch ( iEventID )
	{
		case AVP_ENTITY_EVENT_CIVILIAN_ALERTED:
		{
			return true;

		}break;

		default:
		{
			return PARENT::HandlesThisEvent(iEventID);
		}break;
	}
}

/// End of file. ///

