//-----------------------------------------------------------------------------
// File		: ToolsAVP_Entity_Human.cpp
// Author	: Gareth Thomas
// Created	: 18 / 12 / 07
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// INCLUDES
//-----------------------------------------------------------------------------
// Core
#include "Core/Asura.h"

// Header
#include "Entity/PhysicalObject/Actor/Vertebrates/Human/ToolsAVP_Entity_Marine.h"

// Asura
#include "Blueprint/Asura_Blueprint_System.h"
#include "Camera/Asura_Camera.h"
#include "Entity/Tools_Entity_Data.h"
#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Overlay.h"
#include "Load&Save/Tools_SaveAsura.h"
#include "Mesh/Asura_Mesh_System.h"
#include "Objects/Tools_Object_System.h"
#include "Rendering/Asura_Render.h"

// AvP
#include "Blueprint/AVP_Blueprint_HashIDs.h"
#include "Entity/AVP_Entity_EventIDs.h"
#include "Entity/AVP_Messages.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Human/Marine/AVP_ServerActor_Marine.h"
#include "Entity/Controller/ToolsAVP_Entity_MarineSquad.h"
#include "Objects/ToolsAVP_Object_Type.h"

// tools
#include "Movement/Tools_Movement.h"
#include "Misc/Tools_Timed_Messages.h"

TOOLS_DECLARE_SHARED_PARAM( bool, 0x60661668, HasMotionTracker );

ToolsAVP_Entity_Marine*							ToolsAVP_Entity_Marine::s_pxSelectedMarine		= NULL;	
Asura_Hashed_List<ToolsAVP_Entity_Marine>		ToolsAVP_Entity_Marine::s_xList;

//-----------------------------------------------------------------------------
// ToolsAVP_Entity_Human :: Constructor
//-----------------------------------------------------------------------------
ToolsAVP_Entity_Marine::ToolsAVP_Entity_Marine(AVP_ServerActor_Marine* pxActor)
: Tools_Entity_PhysicalObject(pxActor)
, PARENT(pxActor)
{
	Type = TOT_Entity_Marine;

	s_pxSelectedMarine = this;
	m_pxMarine = pxActor;

	Mesh	= Asura_Mesh_System::GetMesh("ToolsShape_Marine");

	s_xList.AddToList(this);
}

void ToolsAVP_Entity_Marine::RestoreMesh()
{
	// Restore original mesh
	SetMesh(Asura_Mesh_System::GetMesh("ToolsShape_Marine"));
}

//-----------------------------------------------------------------------------
// ToolsAVP_Entity_Human :: Destructor
//-----------------------------------------------------------------------------
ToolsAVP_Entity_Marine::~ToolsAVP_Entity_Marine()
{
	s_xList.RemoveFromList(this);
}

//-----------------------------------------------------------------------------
// ToolsAVP_Entity_Human :: Project_Create
//-----------------------------------------------------------------------------
void ToolsAVP_Entity_Marine::Project_Create()
{
	ToolsAVP_Entity_Marine::Create( true );
}

//-----------------------------------------------------------------------------
// ToolsAVP_Entity_Human :: Create
//-----------------------------------------------------------------------------
Tools_Object* ToolsAVP_Entity_Marine::Create(Asura_Entity* pxEntity)
{
	return new ToolsAVP_Entity_Marine(AVP_ServerActor_Marine::FindEntity(pxEntity->GetGuid()));
}

//-----------------------------------------------------------------------------
// ToolsAVP_Entity_Human :: Create
//-----------------------------------------------------------------------------
Tools_Object* ToolsAVP_Entity_Marine::Create( const bool bFullInit )
{
	Asura_Hash_ID uBlueprintTypeHashID = uAVP_BLUEPRINT_TYPE_ACTORS;
	Asura_Hash_ID uBlueprintHashID     = 0x0ea0316a; // Asura_GetHashID( "MarineAI" )
	if ( bFullInit )
	{
		// Get the user to pick a blueprint to initialise from.
		BlueprintSelect_Overlay xBPOverlay( "Choose A Blueprint", &uBlueprintHashID, &uBlueprintTypeHashID );
		if ( !xBPOverlay.DoModal() ) { return NULL; }
	}

	const Asura_Guid uGuid = Asura_Entity_System::GetCreatedByToolsGuid();

	// Set up tools data and auto-create a unique name (prompt for new name if there is a clash).
	Tools_Entity_Data* pxToolsData = CreateToolsDataWithDefaultName("Marine", "Marine", uGuid);
	if ( !pxToolsData ) { return NULL; }

	AVP_ServerActor_Marine* pxServerObject = new AVP_ServerActor_Marine(uGuid);
	if ( !pxServerObject )
	{
		delete pxToolsData;
		return NULL;
	}

	// Set the tools data before we create the tools object - otherwise it will have already been created!
	pxServerObject->ToolsData = pxToolsData;
	if ( bFullInit )
	{
		pxServerObject->SetPosition( pxToolsData->Position );
		pxServerObject->SetBlueprintTypeHashID( uBlueprintTypeHashID );
		pxServerObject->SetBlueprintHashID( uBlueprintHashID );
		pxServerObject->MakeAI();
		pxServerObject->CreateAndInitialiseBrain();
		pxServerObject->InitFromBlueprint();
	}

	ToolsAVP_Entity_Marine* pxToolsObject = new ToolsAVP_Entity_Marine( pxServerObject );
	if ( !pxToolsObject )
	{
		delete pxToolsObject;
		delete pxServerObject;
		return NULL;
	}
	Tools_Object_System::SelectObject(pxToolsObject);

	Tools_Timed_Message_System::AddMessage( Tools_Timed_Message_System::COLOUR_DEFAULT, "Created new Marine." );
	Tools_SaveAsura::SetEntityModified();

	return pxToolsObject;
}

//-----------------------------------------------------------------------------
// ToolsAVP_Entity_Human :: OnSelection
//-----------------------------------------------------------------------------
void ToolsAVP_Entity_Marine::OnSelection()
{
	PARENT::OnSelection();

	s_pxSelectedMarine = this;
}

//-----------------------------------------------------------------------------
// ToolsAVP_Entity_Human :: OnDeselection
//-----------------------------------------------------------------------------
void ToolsAVP_Entity_Marine::OnDeselection()
{
	s_pxSelectedMarine = NULL;

	PARENT::OnDeselection();
}

//-----------------------------------------------------------------------------
// ToolsAVP_Entity_Marine :: OnDeselection
//-----------------------------------------------------------------------------
void ToolsAVP_Entity_Marine::AddOverlayControls(Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset)
{
	PARENT::AddOverlayControls(pxOverlay, xOffset);

	Asura_Dialog* pxDialog = new Asura_SharedTickbox_Dialog( "Has motion tracker", xOffset, &g_xParam_HasMotionTracker );
	pxDialog->SetHelpText("Set this marine as having/not having a motion tracker");
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog( "Edit And Focus Marines Squad", xOffset, SelectSquad);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();
}

bool ToolsAVP_Entity_Marine::GetParameter( Asura_Hash_ID uParamID, Tools_Parameter* pxParam ) const
{
	switch(uParamID)
	{
		case g_uHash_HasMotionTracker:
		{
			pxParam->SetBool( m_pxMarine->m_bHasMotionTracker != 0 );
		}
		break;

		default :
		{
			return PARENT::GetParameter(uParamID, pxParam);
		}
	}
	return true;
}

bool ToolsAVP_Entity_Marine::SetParameter( Asura_Hash_ID uParamID, const Tools_Parameter* pxParam )
{
	switch(uParamID)
	{
		case g_uHash_HasMotionTracker:
		{
			m_pxMarine->m_bHasMotionTracker = pxParam->GetBool();
		}
		break;

		default :
		{
			return PARENT::SetParameter(uParamID, pxParam);
		}
	}

	SetEntityModified();
	return true;
}

//-----------------------------------------------------------------------------
// ToolsAVP_Entity_Marine :: SelectSquad
//-----------------------------------------------------------------------------
void ToolsAVP_Entity_Marine::SelectSquad(void)
{
	if (s_pxSelectedMarine)
	{
		ToolsAVP_Entity_Marine* pxMarine		= s_pxSelectedMarine;
		AVP_ServerActor_Marine* pxGameMarine	= pxMarine->m_pxMarine;

		if (pxGameMarine)
		{
			if (pxGameMarine->GetInitialSquadGuid() != ASURA_GUID_UNREGISTERED)
			{
				ToolsAVP_Entity_MarineSquad* pxSquad = (ToolsAVP_Entity_MarineSquad*)ToolsAVP_Entity_MarineSquad::FindEntity(pxGameMarine->GetInitialSquadGuid());

				if (pxSquad)
				{
					Asura_Vector_3		xNewCameraPos	= pxSquad->Position;
					Asura_Matrix_3x3	xNewCameraOri	= *Asura_Camera::GetOrientation();
					Asura_Vector_3		xFwdVec;

					Asura_Camera::GetForwardVector(xFwdVec);
					xNewCameraPos -= xFwdVec * 8.0f;

					Tools_Movement::TeleportCameraToPosition(&xNewCameraPos, &xNewCameraOri);

					Tools_Object_System::SelectObject(pxSquad);

					return;
				}
			}
		}
	}

	Tools_Timed_Message_System::AddMessage(0xff0000, "Marine Has No Squad");
}

//-----------------------------------------------------------------------------
// ToolsAVP_Entity_Marine :: HandlesThisMessage
//-----------------------------------------------------------------------------
bool ToolsAVP_Entity_Marine::HandlesThisMessage( int iMessageID )
{
	switch(iMessageID)
	{
		case AVP_MSGID_ADD_AI_OBJECTIVE:
		case AVP_MSGID_PROMOTE_TO_SQUAD_LEADER:
		case AVP_MSGID_CHANGE_HUMANOID_SQUAD_MOVEMENT_MODE:
		case AVP_MSGID_SET_PICKUPABLE:
		case AVP_MSGID_SET_NOT_PICKUPABLE:
		case AVP_MSGID_SET_DROPPABLE:
		case AVP_MSGID_SET_NOT_DROPPABLE:
		case AVP_MSGID_FORCE_AIM:
		case AVP_MSGID_STOP_FORCE_AIM:
		{
			return true;
		}

		default:
		{
			return ( PARENT::HandlesThisMessage( iMessageID ) );
		}
	}
}

//-----------------------------------------------------------------------------
// ToolsAVP_Entity_Marine :: HandlesThisMessage
//-----------------------------------------------------------------------------
bool ToolsAVP_Entity_Marine::HandlesThisEvent( int iEventID ) const
{
	switch(iEventID)
	{
		case AVP_ENTITY_EVENT_MARINE_ALERTED:
		case AVP_ENTITY_EVENT_MARINE_THREAT_SPOTTED:
		{
			return true;
		}

		default:
		{
			return ( PARENT::HandlesThisEvent( iEventID ) );
		}
	}
}

/// End of file. ///

