//////////////////////////////////////////////////////////////////////////////////////////
//
// FILE : ToolsAVP_Entity_Predalien.cpp
//
// Inherits from Tools_Entity_PhysicalObject
//
// AUTHORS : Chris Wallace
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////////////////

//////////////
// INCLUDES //
//////////////

#include "Core/Asura.h"

#include "Entity/PhysicalObject/Actor/Vertebrates/Predator/ToolsAVP_Entity_Predalien.h"
//#include "Load&Save/ToolsAVP_Level_Export.h"
//#include "Load&Save/Tools_Level_Export.h"
#include "Particles/Tools_Mode_PFX.h"

// Project independent.
//#include "Entity/PhysicalObject/ToolsAVP_Entity_PhysicalObject.h"
#include "Actor/Vertebrates/Predalien/AVP_Predalien_Description.h"
#include "Blueprint/AVP_Blueprint_HashIDs.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Predalien/AVP_ServerActor_Predalien.h"
#include "Mesh/Asura_Mesh_System.h"
#include "Objects/Tools_Object_System.h"
#include "Camera/Asura_Camera.h"
#include "Load&Save/Tools_SaveAsura.h"
#include "Entity/Tools_Entity_Data.h"
#include "Objects/ToolsAVP_Object_Type.h"

#include "Entity/AVP_Messages.h"

TOOLS_DECLARE_SHARED_PARAM( bool, 0x29FC0644, UseTestBehaviour );

///////////////////////////////////
// ToolsAVP_Entity_Predalien //
///////////////////////////////////

ToolsAVP_Entity_Predalien::ToolsAVP_Entity_Predalien( AVP_ServerActor_Predalien* pxActor )
: Tools_Entity_PhysicalObject( pxActor )
, PARENT( pxActor )
, m_pxPredalien( pxActor )
, m_bRefreshOverlay(false)
{
	Mesh = Asura_Mesh_System::GetMesh("ToolsShape_Predalien");

	Type = TOT_Entity_Predalien;
}

void ToolsAVP_Entity_Predalien::RestoreMesh()
{
	// Restore original mesh
	SetMesh(Asura_Mesh_System::GetMesh("ToolsShape_Predalien"));
}

void ToolsAVP_Entity_Predalien::Project_Create()
{
    ToolsAVP_Entity_Predalien::Create( true );
}

Tools_Object* ToolsAVP_Entity_Predalien::Create( Asura_Entity* pxEntity )
{
    return new ToolsAVP_Entity_Predalien( AVP_ServerActor_Predalien::FindEntity(pxEntity->GetGuid()) );
}

Tools_Object* ToolsAVP_Entity_Predalien::Create( const bool bFullInit )
{
	// Get the user to pick a blueprint to initialise from.
	Asura_Hash_ID uBlueprintTypeHashID = uAVP_BLUEPRINT_TYPE_ACTORS;
	Asura_Hash_ID uBlueprintHashID     = 0xaf3a2146;
	if ( bFullInit )
	{
		BlueprintSelect_Overlay xBPOverlay( "Choose A Blueprint", &uBlueprintHashID, &uBlueprintTypeHashID );
		if ( !xBPOverlay.DoModal() ) { return NULL; }
	}

	const Asura_Guid uGuid = Asura_Entity_System::GetCreatedByToolsGuid();

	// Set up tools data and auto-create a unique name (prompt for new name if there is a clash).
	Tools_Entity_Data* pxToolsData = CreateToolsDataWithDefaultName("Predalien", "Predalien", uGuid);
	if ( !pxToolsData ) { return NULL; }

	AVP_ServerActor_Predalien* pxServerObject = new AVP_ServerActor_Predalien(uGuid);
	if ( !pxServerObject )
	{
		delete pxToolsData;
		return NULL;
	}

	// Set the tools data before we create the tools object - otherwise it will have already been created!
	pxServerObject->ToolsData = pxToolsData;
	if ( bFullInit )
	{
		pxServerObject->SetPosition( pxToolsData->Position );
		pxServerObject->SetBlueprintTypeHashID( uBlueprintTypeHashID );
		pxServerObject->SetBlueprintHashID( uBlueprintHashID );
		pxServerObject->MakeAI();
		pxServerObject->CreateAndInitialiseBrain();
		pxServerObject->InitFromBlueprint();
	}

	ToolsAVP_Entity_Predalien* pxToolsObject = new ToolsAVP_Entity_Predalien( pxServerObject );
	if ( !pxToolsObject )
	{
		delete pxToolsObject;
		delete pxServerObject;
		return NULL;
	}
	Tools_Object_System::SelectObject(pxToolsObject);

	Tools_Timed_Message_System::AddMessage( Tools_Timed_Message_System::COLOUR_DEFAULT, "Created new Predalien." );
	Tools_SaveAsura::SetEntityModified();

	return pxToolsObject;
}

bool ToolsAVP_Entity_Predalien::GetParameter( Asura_Hash_ID uParamID, Tools_Parameter* pxParam ) const
{
	switch(uParamID)
	{	
		case g_uHash_UseTestBehaviour:
		{
			pxParam->SetBool( m_pxPredalien->m_bUseTestBehaviour );
		}
		break;

		default :
		{
			return PARENT::GetParameter(uParamID, pxParam);
		}
	}
	return true;
}

bool ToolsAVP_Entity_Predalien::SetParameter( Asura_Hash_ID uParamID, const Tools_Parameter* pxParam )
{
	switch(uParamID)
	{
		case g_uHash_UseTestBehaviour:
		{
			m_pxPredalien->m_bUseTestBehaviour = pxParam->GetBool();
		}
		break;

		default :
		{
			return PARENT::SetParameter(uParamID, pxParam);
		}
	}

	SetEntityModified();
	return true;
}

void ToolsAVP_Entity_Predalien::UpdateWhileSelected()
{
	PARENT::UpdateWhileSelected();

	if ( m_bRefreshOverlay )
	{
		DeleteOverlay();
		CreateOverlay();
		m_bRefreshOverlay = false;
	}
}


void ToolsAVP_Entity_Predalien::AddFilesToExportList(List<Asura_String>& nameList) const
{
	// Call the base-function...
	PARENT::AddFilesToExportList(nameList);

	// Export the necessary predator data...
	AVP_Predalien_Description::AddFilesToExport(false, m_uBlueprintHashID, nameList);
}

void ToolsAVP_Entity_Predalien::AddOverlayControls( Asura_Overlay* pxNewOverlay, Asura_Vector_2& xOffset )
{
	PARENT::AddOverlayControls( pxNewOverlay, xOffset );

	Asura_Dialog* pxDialog;

	pxDialog = new Asura_SharedTickbox_Dialog("Use Test Behaviour", xOffset, &g_xParam_UseTestBehaviour);
	pxDialog->SetHelpText("Having this ticked will make the AI use the test behaviour instead of the normal Predalien AI behaviour.");
	pxNewOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();
}

/// End of file. ///
