//////////////////////////////////////////////////////////////////////////////////////////
//
// FILE : ToolsAVP_Entity_Predator.cpp
//
// Inherits from Tools_Entity_PhysicalObject
//
// AUTHORS : Chris Wallace
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////////////////

//////////////
// INCLUDES //
//////////////

#include "Core/Asura.h"

#include "Entity/PhysicalObject/Actor/Vertebrates/Predator/ToolsAVP_Entity_Predator.h"
//#include "Load&Save/ToolsAVP_Level_Export.h"
//#include "Load&Save/Tools_Level_Export.h"
#include "Particles/Tools_Mode_PFX.h"

// Project independent.
//#include "Entity/PhysicalObject/ToolsAVP_Entity_PhysicalObject.h"
#include "Actor/Vertebrates/Predator/AVP_Predator_Description.h"
#include "Blueprint/AVP_Blueprint_HashIDs.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Predator/AVP_ServerActor_Predator.h"
#include "Mesh/Asura_Mesh_System.h"
#include "Objects/Tools_Object_System.h"
#include "Camera/Asura_Camera.h"
#include "Load&Save/Tools_SaveAsura.h"
#include "Entity/Tools_Entity_Data.h"
#include "Objects/ToolsAVP_Object_Type.h"

#include "Entity/AVP_Messages.h"

TOOLS_DECLARE_SHARED_PARAM( bool, 0x29FC0644, UseTestBehaviour );
TOOLS_DECLARE_SHARED_PARAM( bool, 0xe17e9efe, StartsCloaked );

///////////////////////////////////
// ToolsAVP_Entity_Predator //
///////////////////////////////////

ToolsAVP_Entity_Predator::ToolsAVP_Entity_Predator( AVP_ServerActor_Predator* pxActor )
: Tools_Entity_PhysicalObject( pxActor )
, PARENT( pxActor )
, m_pxPredator( pxActor )
, m_bRefreshOverlay(false)
{
	Mesh = Asura_Mesh_System::GetMesh("ToolsShape_Predator");

	Type = TOT_Entity_Predator;
}

void ToolsAVP_Entity_Predator::RestoreMesh()
{
	// Restore original mesh
	SetMesh(Asura_Mesh_System::GetMesh("ToolsShape_Predator"));
}

void ToolsAVP_Entity_Predator::Project_Create()
{
    ToolsAVP_Entity_Predator::Create( true );
}

Tools_Object* ToolsAVP_Entity_Predator::Create( Asura_Entity* pxEntity )
{
    return new ToolsAVP_Entity_Predator( AVP_ServerActor_Predator::FindEntity(pxEntity->GetGuid()) );
}

Tools_Object* ToolsAVP_Entity_Predator::Create( const bool bFullInit )
{
	// Get the user to pick a blueprint to initialise from.
	Asura_Hash_ID uBlueprintTypeHashID = uAVP_BLUEPRINT_TYPE_ACTORS;
	Asura_Hash_ID uBlueprintHashID     = 0xb31282f7;
	if ( bFullInit )
	{
		BlueprintSelect_Overlay xBPOverlay( "Choose A Blueprint", &uBlueprintHashID, &uBlueprintTypeHashID );
		if ( !xBPOverlay.DoModal() ) { return NULL; }
	}

	const Asura_Guid uGuid = Asura_Entity_System::GetCreatedByToolsGuid();

	// Set up tools data and auto-create a unique name (prompt for new name if there is a clash).
	Tools_Entity_Data* pxToolsData = CreateToolsDataWithDefaultName("Predator", "Predator", uGuid);
	if ( !pxToolsData ) { return NULL; }

	AVP_ServerActor_Predator* pxServerObject = new AVP_ServerActor_Predator(uGuid);
	if ( !pxServerObject )
	{
		delete pxToolsData;
		return NULL;
	}

	// Set the tools data before we create the tools object - otherwise it will have already been created!
	pxServerObject->ToolsData = pxToolsData;
	if ( bFullInit )
	{
		pxServerObject->SetPosition( pxToolsData->Position );
		pxServerObject->SetBlueprintTypeHashID( uBlueprintTypeHashID );
		pxServerObject->SetBlueprintHashID( uBlueprintHashID );
		pxServerObject->MakeAI();
		pxServerObject->CreateAndInitialiseBrain();
		pxServerObject->InitFromBlueprint();
	}

	ToolsAVP_Entity_Predator* pxToolsObject = new ToolsAVP_Entity_Predator( pxServerObject );
	if ( !pxToolsObject )
	{
		delete pxToolsObject;
		delete pxServerObject;
		return NULL;
	}
	Tools_Object_System::SelectObject(pxToolsObject);

	Tools_Timed_Message_System::AddMessage( Tools_Timed_Message_System::COLOUR_DEFAULT, "Created new Predator." );
	Tools_SaveAsura::SetEntityModified();

	return pxToolsObject;
}

bool ToolsAVP_Entity_Predator::GetParameter( Asura_Hash_ID uParamID, Tools_Parameter* pxParam ) const
{
	switch(uParamID)
	{
		case g_uHash_StartsCloaked:
		{
			pxParam->SetBool( m_pxPredator->m_bStartsCloaked );
		}
		break;	
		
		case g_uHash_UseTestBehaviour:
		{
			pxParam->SetBool( m_pxPredator->m_bUseTestBehaviour );
		}
		break;

		default :
		{
			return PARENT::GetParameter(uParamID, pxParam);
		}
	}
	return true;
}

bool ToolsAVP_Entity_Predator::SetParameter( Asura_Hash_ID uParamID, const Tools_Parameter* pxParam )
{
	switch(uParamID)
	{
		case g_uHash_StartsCloaked:
		{
			m_pxPredator->m_bStartsCloaked = pxParam->GetBool();
		}
		break;	

		case g_uHash_UseTestBehaviour:
		{
			m_pxPredator->m_bUseTestBehaviour = pxParam->GetBool();
		}
		break;

		default :
		{
			return PARENT::SetParameter(uParamID, pxParam);
		}
	}

	SetEntityModified();
	return true;
}

void ToolsAVP_Entity_Predator::UpdateWhileSelected()
{
	PARENT::UpdateWhileSelected();

	if ( m_bRefreshOverlay )
	{
		DeleteOverlay();
		CreateOverlay();
		m_bRefreshOverlay = false;
	}
}


bool ToolsAVP_Entity_Predator::HandlesThisMessage( int iMessageID )
{
	switch (iMessageID)
	{
	case AVP_MSGID_REQUEST_CLOAK_ON:
	case AVP_MSGID_REQUEST_CLOAK_OFF:
	case AVP_MSGID_SET_PREDATOR_BEHAVIOUR:
	case AVP_MSGID_ATTACK_SCRIPTED_TARGET:
	case AVP_MSGID_STOP_ATTACKING_SCRIPTED_TARGET:
		return true;
	default:
		return PARENT::HandlesThisMessage( iMessageID );

	}
}

void ToolsAVP_Entity_Predator::AddFilesToExportList(List<Asura_String>& nameList) const
{
	// Call the base-function...
	PARENT::AddFilesToExportList(nameList);

	// Export the necessary predator data...
	AVP_Predator_Description::AddFilesToExport(false, m_uBlueprintHashID, nameList);
}

void ToolsAVP_Entity_Predator::AddOverlayControls( Asura_Overlay* pxNewOverlay, Asura_Vector_2& xOffset )
{
	PARENT::AddOverlayControls( pxNewOverlay, xOffset );

	Asura_Dialog* pxDialog;

	pxDialog = new Asura_SharedTickbox_Dialog("Start Cloaked", xOffset, &g_xParam_StartsCloaked);
	pxNewOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::LargeSpacing();

	pxDialog = new Asura_SharedTickbox_Dialog("Use Test Behaviour", xOffset, &g_xParam_UseTestBehaviour);
	pxDialog->SetHelpText("Having this ticked will make the AI use the test behaviour instead of the normal Predator AI behaviour.");
	pxNewOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();

	Asura_DropDown_List_Dialog* pxDDLDialogue = new Asura_DropDown_List_Dialog(
		xOffset,
		AVP_PredatorBehaviourData::s_apszBehaviourStrings,
		&m_pxPredator->m_xBehaviourData.m_iBehaviour,
		AVP_PredatorBehaviourData::NUM_BEHAVIOURS,
		AVP_PredatorBehaviourData::NUM_BEHAVIOURS,
		NULL,
		&Changed,
		"" );
	pxDDLDialogue->SetOnSelectCallback( BehaviourChanged, reinterpret_cast< int >( this ) );
	pxNewOverlay->AddDialog( pxDDLDialogue );
	xOffset.x -= 7.0f;
	xOffset.y += Asura_Dialog::LargeSpacing();

}

void ToolsAVP_Entity_Predator::BehaviourChanged( int iThis )
{
	ToolsAVP_Entity_Predator* pxThis = reinterpret_cast< ToolsAVP_Entity_Predator* >( iThis );
	pxThis->m_pxPredator->m_xBehaviourData.SetParamsFromBehaviour();
	pxThis->RequestRefreshOverlay();
	Tools_SaveAsura::SetEntityModified();
}


/// End of file. ///
