//////////////////////////////////////////////////////////////////////////////
//
// ToolsAVP_Entity_InteractHackingPoint.cpp
//
// Created by AJK 24/11/2008
//
// Copyright (c) 2008 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// Includes
#include "Core/Asura.h"

// This file's header.
#include "Entity/PhysicalObject/Interact/ToolReceptacle/HackingPoint/ToolsAVP_Entity_InteractHackingPoint.h"

// Asura
#include "Entity/Asura_StaticMessage_System.h"

// AVP
#include "Entity/PhysicalObject/Interact/ToolReceptacle/HackingPoint/AVP_ServerEntity_InteractHackingPoint.h"
#include "Entity/PhysicalObject/Pickup/AVP_ServerEntity_Pickup.h"

// Tools Asura
#include "Entity/Tools_MessageEditor.h"
#include "Load&Save/Tools_SaveAsura.h"

// Class statics
Asura_Hashed_List<ToolsAVP_Entity_InteractHackingPoint>	ToolsAVP_Entity_InteractHackingPoint::s_xList;

ToolsAVP_Entity_InteractHackingPoint::ToolsAVP_Entity_InteractHackingPoint( AVP_ServerEntity_InteractHackingPoint* pxInteractObject )
: PARENT( pxInteractObject )
, Tools_Entity_PhysicalObject( pxInteractObject )
{
	Type = TOT_Entity_InteractHackingPoint;
	s_xList.AddToList(this);
}

ToolsAVP_Entity_InteractHackingPoint::~ToolsAVP_Entity_InteractHackingPoint()
{
	s_xList.RemoveFromList(this);
}

void ToolsAVP_Entity_InteractHackingPoint::Create()
{
	CreationHelper( "Hacking point", "HackingPoint", 
		AVP_ServerEntity_InteractHackingPoint::CreateFromTools, Create);
}

Tools_Object* ToolsAVP_Entity_InteractHackingPoint::Create( Asura_Entity* pxEntity )
{
	ToolsAVP_Entity_InteractHackingPoint* pxObject = new ToolsAVP_Entity_InteractHackingPoint( AVP_ServerEntity_InteractHackingPoint::FindEntity( pxEntity->GetGuid() ) );
	pxObject->UpdateToolsData();
	return pxObject;
}

Tools_Object* ToolsAVP_Entity_InteractHackingPoint::CreateObjectOfThisType() const
{
	const Asura_Guid uGuid = Asura_Entity_System::GetCreatedByToolsGuid();

	// Set up tools data and auto-create a unique name (prompt for new name if there is a clash).
	Tools_Entity_Data *pxToolsData = CreateToolsDataWithDefaultName("Hacking Point", "HackingPoint", uGuid);

	if(pxToolsData == NULL)
	{
		Asura_Assert(false, "ToolsAVP_Entity_InteractHackingPoint::Create() - no tools data");
	}
	else
	{
		AVP_ServerEntity_InteractHackingPoint* pxServerObject = new AVP_ServerEntity_InteractHackingPoint(uGuid);
		pxServerObject->ToolsData = pxToolsData;
		ToolsAVP_Entity_InteractHackingPoint *pxToolsObject = new ToolsAVP_Entity_InteractHackingPoint(pxServerObject);
		Tools_Object_System::SelectObject(pxToolsObject);

		pxToolsObject->Position		= *Asura_Camera::GetPosition();
		pxToolsObject->Orientation.Identity();

		Tools_SaveAsura::SetEntityModified();
		return pxToolsObject;
	}
	return NULL;
}

void ToolsAVP_Entity_InteractHackingPoint::AddActionsControls( Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset )
{
	PARENT::AddActionsControls(pxOverlay, xOffset);

	Asura_Char szBuffer[32];

	AVP_ServerEntity_InteractHackingPoint* pxHackingPoint = static_cast<AVP_ServerEntity_InteractHackingPoint*> (m_pxInteractObject);

	Asura_PrintfToBfr( szBuffer,"On marine insert HackingTool(%d)", 32, true, Asura_StaticMessage_System::GetNumberOfMessagesInBlock( pxHackingPoint->m_usMarineInsertHackingToolMessageBlock ) );
	Asura_Dialog* pxDialog = new Asura_Function_Dialog( szBuffer, xOffset, EditOnMarineInsertHackingToolMessages );
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	Asura_PrintfToBfr( szBuffer,"On marine remove HackingTool (%d)", 32, true, Asura_StaticMessage_System::GetNumberOfMessagesInBlock( pxHackingPoint->m_usMarineRemoveHackingToolMessageBlock ) );
	pxDialog = new Asura_Function_Dialog( szBuffer, xOffset, EditOnMarineRemoveHackingToolMessages );
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();
}

bool ToolsAVP_Entity_InteractHackingPoint::GetEventDescForMessageBlock( unsigned short usMessageBlockID, Asura_String& xOutputDesc )
{
	AVP_ServerEntity_InteractHackingPoint* pxHackingPoint = static_cast<AVP_ServerEntity_InteractHackingPoint*> (m_pxInteractObject);

	if ( m_pxInteractObject ) 
	{
		if ( pxHackingPoint->m_usMarineInsertHackingToolMessageBlock == usMessageBlockID )
		{
			xOutputDesc = "On Marine Insert HackingTool";
			return true;
		}
		else if ( pxHackingPoint->m_usMarineRemoveHackingToolMessageBlock == usMessageBlockID )
		{
			xOutputDesc = "On Marine Remove HackingTool";
			return true;
		}
		return PARENT::GetEventDescForMessageBlock(usMessageBlockID, xOutputDesc);
	}
	else
	{
		Asura_Assert(false, "No Interaction object!");
		return false;
	}
}

void ToolsAVP_Entity_InteractHackingPoint::EditOnMarineInsertHackingToolMessages()
{
	Asura_Assert( s_pxThis, "No current tools HackingTool receptacle object!, will crash!" );
	AVP_ServerEntity_InteractHackingPoint* pxHackingPoint = static_cast<AVP_ServerEntity_InteractHackingPoint*> (s_pxThis->GetInteractObject());
	Asura_Assert( pxHackingPoint, "No HackingTool receptacle; will crash!" );

	#ifdef ASURA_TOOLS_USE_NEW_MESSAGE_EDITOR
	Tools_MessageEditor::StartMessageEditingMode(&pxHackingPoint->m_usMarineInsertHackingToolMessageBlock);
	#else
	Tools_Object_System::EditMessageBlock( "On Do Action", &pxHackingPoint->m_usMarineInsertHackingToolMessageBlock );
	#endif
}

void ToolsAVP_Entity_InteractHackingPoint::EditOnMarineRemoveHackingToolMessages()
{
	Asura_Assert( s_pxThis, "No current tools HackingTool receptacle object!, will crash!" );
	AVP_ServerEntity_InteractHackingPoint* pxHackingPoint = static_cast<AVP_ServerEntity_InteractHackingPoint*> (s_pxThis->GetInteractObject());
	Asura_Assert( pxHackingPoint, "No HackingTool receptacle; will crash!" );

	#ifdef ASURA_TOOLS_USE_NEW_MESSAGE_EDITOR
	Tools_MessageEditor::StartMessageEditingMode(&pxHackingPoint->m_usMarineRemoveHackingToolMessageBlock);
	#else
	Tools_Object_System::EditMessageBlock( "On Undo Action", &pxHackingPoint->m_usMarineRemoveHackingToolMessageBlock );
	#endif
}

bool ToolsAVP_Entity_InteractHackingPoint::OwnsThisMessageBlock( unsigned short usMessageBlockID )
{
	AVP_ServerEntity_InteractHackingPoint* pxHackingPoint = static_cast<AVP_ServerEntity_InteractHackingPoint*> (m_pxInteractObject);
	if (    pxHackingPoint 
		&& (   pxHackingPoint->m_usMarineInsertHackingToolMessageBlock == usMessageBlockID 
		|| pxHackingPoint->m_usMarineRemoveHackingToolMessageBlock == usMessageBlockID
		   ) 
	   )
	{
		return true;
	}

	return PARENT::OwnsThisMessageBlock( usMessageBlockID );
}

void ToolsAVP_Entity_InteractHackingPoint::Destroy()
{
	AVP_ServerEntity_InteractHackingPoint* pxHackingPoint = static_cast<AVP_ServerEntity_InteractHackingPoint*> (m_pxInteractObject);

	//delete all the static messages
	Asura_StaticMessage_System::DeleteMessageBlock(pxHackingPoint->m_usMarineInsertHackingToolMessageBlock);
	pxHackingPoint->m_usMarineInsertHackingToolMessageBlock = ASURA_STATICMESSAGE_EMPTYBLOCK;

	Asura_StaticMessage_System::DeleteMessageBlock(pxHackingPoint->m_usMarineRemoveHackingToolMessageBlock);
	pxHackingPoint->m_usMarineRemoveHackingToolMessageBlock = ASURA_STATICMESSAGE_EMPTYBLOCK;

	PARENT::Destroy();
}