//////////////////////////////////////////////////////////////////////////////
//
// ToolsAVP_Entity_PhysicalObject.cpp
//
// Copyright (c) 2006 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header.
#include "Entity/PhysicalObject/ToolsAVP_Entity_PhysicalObject.h"

// Engine independent.
#include "Blueprint/Asura_Blueprint.h"
#include "Blueprint/Asura_Blueprint_System.h"
#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Overlay.h"
#include "Shapes/Asura_Shape_System.h"

// Project independent.
#include "Blueprint/AVP_Blueprint_HashIDs.h"
#include "Entity/AVP_Messages.h"
#include "Entity/AVP_Entity_EventIDs.h"
#include "Entity/PhysicalObject/AVP_ServerEntity_PhysicalObject.h"

// Tools
#include "Blueprint/ToolsAVP_Mode_Blueprint.h"
#include "Load&Save/Tools_SaveAsura.h"
#include "Misc/Tools_Timed_Messages.h"
#include "Objects/Tools_ObjectSystem_Param.h"
#include "Shapes/Tools_Shapes.h"


TOOLS_DECLARE_SHARED_PARAM( Asura_String,	0x1e420133,	PhysicalObject_Material_Func )					// Asura_GetHashID("PhysicalObject_Material_Func")
TOOLS_DECLARE_SHARED_PARAM( Asura_String,   0x56e0b579, PhysicalObject_Blueprint_Func )					// Asura_GetHashID("PhysicalObject_Blueprint_Func")
TOOLS_DECLARE_SHARED_PARAM( bool,			0xdbd0aebe, PhysicalObject_IgnoreForSurfaceReorientation )	// Asura_GetHashID("PhysicalObject_IgnoreForSurfaceReorientation")
TOOLS_DECLARE_SHARED_PARAM( bool,			0x45B3A542, PhysicalObject_IsFlammable )					// Asura_GetHashID("PhysicalObject_IsFlammable")

// C L A S S   I M P L E M E N T A T I O N S /////////////////////////////////

//
// ToolsAVP_Entity_PhysicalObject
//

ToolsAVP_Entity_PhysicalObject* ToolsAVP_Entity_PhysicalObject::s_pxThisToolsPhyObj = NULL;
Asura_Hashed_List< ToolsAVP_Entity_PhysicalObject > ToolsAVP_Entity_PhysicalObject::s_xList;

ToolsAVP_Entity_PhysicalObject::ToolsAVP_Entity_PhysicalObject(AVP_ServerEntity_PhysicalObject* pxPhysicalObject)
: Tools_Entity_PhysicalObject( pxPhysicalObject ) // ignored
, m_pxPhysicalObject(pxPhysicalObject)
, m_uBlueprintHashID(pxPhysicalObject->GetBlueprintHashID())
, m_bChanged( false )
{
    // Add the shape to the shape table if necessary.
    if ( m_pxPhysicalObject )
    {
        Asura_Hash_ID uShapeFileHash = m_pxPhysicalObject->GetShapeFileID();
        if( ASURA_HASH_ID_UNSET != uShapeFileHash && !Asura_Shape_System::GetShape(uShapeFileHash) )
        {
            const char* szFilename = Asura_HashedFilename_System::GetFilename( uShapeFileHash, false );
            if( szFilename )
            {
                Tools_ShapeTable::AddShapeByFilename( szFilename, false );
                SetToolsShape( uShapeFileHash );
            }
        }
    }

	s_pxCurrentlySelectedObject = this;

    s_xList.AddToList( this );
}

ToolsAVP_Entity_PhysicalObject::~ToolsAVP_Entity_PhysicalObject()
{
    s_xList.RemoveFromList( this );
}

void ToolsAVP_Entity_PhysicalObject::OnSelection()
{
	s_pxThisToolsPhyObj = this;
	PARENT::OnSelection();
}

void ToolsAVP_Entity_PhysicalObject::OnDeselection()
{
	PARENT::OnDeselection();
	s_pxThisToolsPhyObj = NULL;
}

void ToolsAVP_Entity_PhysicalObject::UpdateWhileSelected()
{
	PARENT::UpdateWhileSelected();

	if ( m_bChanged )
	{
        m_bChanged = false;
        m_pxPhysicalObject->InitFromBlueprint();
        Tools_SaveAsura::SetEntityModified();
	}
}

void ToolsAVP_Entity_PhysicalObject::AddOverlayControls( Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset )
{
	PARENT::AddOverlayControls( pxOverlay, xOffset );

	xOffset.y += Asura_Dialog::LargeSpacing() - Asura_Dialog::DefaultSpacing();

	Asura_Dialog* pxDialog;
    pxDialog = new Asura_Function_Dialog( "Blueprint: ", &g_xParam_PhysicalObject_Blueprint_Func, xOffset, ChooseBlueprint, reinterpret_cast< int >( this ) );
    pxOverlay->AddDialog( pxDialog );
    xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog( "Edit Current Blueprint", xOffset, EditBlueprint, reinterpret_cast< int >( this ) );
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	Asura_Function_Dialog* pxFunctionDialog = new Asura_Function_Dialog("Refresh Blueprint Properties", xOffset, RefreshBlueprintProperties, &Changed);
	// This works as a shared dialog, since it doesn't change any properties other than by reusing the blueprint initialisation.
	pxFunctionDialog->SetUseSharedDialogColour(true);
	pxOverlay->AddDialog( pxFunctionDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

    pxFunctionDialog = new Asura_Function_Dialog( "Select All With This Blueprint", xOffset, SelectAllWithBlueprint, reinterpret_cast< int >( this ) );
    pxOverlay->AddDialog( pxFunctionDialog );
    xOffset.y += Asura_Dialog::DefaultSpacing();

	const bool bGotBlueprint = ( m_pxPhysicalObject->GetBlueprintHashID() != ASURA_HASH_ID_UNSET &&	m_pxPhysicalObject->GetBlueprintTypeHashID() != ASURA_HASH_ID_UNSET );
	
	Asura_SharedTickbox_Dialog* pxTickDialog = new Asura_SharedTickbox_Dialog("Ignore For Surface Reorientation", xOffset, &g_xParam_PhysicalObject_IgnoreForSurfaceReorientation );
	pxTickDialog->SetHelpText( "This object should be ignored for reorientation (alien movement)" );
	pxTickDialog->SetEnabled( !bGotBlueprint );
	pxOverlay->AddDialog( pxTickDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxTickDialog = new Asura_SharedTickbox_Dialog("Is Flammable", xOffset, &g_xParam_PhysicalObject_IsFlammable );
	pxTickDialog->SetHelpText( "If true, this object can be set on fire" );
	pxTickDialog->SetEnabled( !bGotBlueprint );
	pxOverlay->AddDialog( pxTickDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	AddOverlayControls_CollisionFXMaterialBP( pxOverlay, xOffset, bGotBlueprint );
	xOffset.y += Asura_Dialog::DefaultSpacing();
}

// This function is called also from ToolsAVP_Entity_Lift !!! (it is because lift has 2 parents. See more comments in Lift class...)
void ToolsAVP_Entity_PhysicalObject::AddOverlayControls_CollisionFXMaterialBP( Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset, const bool bGotBlueprint )
{
	Asura_Dialog* pxDialog;
	pxDialog = new Asura_Function_Dialog( "Collision FX Material Blueprint: ", &g_xParam_PhysicalObject_Material_Func, xOffset, ChooseMaterialBlueprint, reinterpret_cast< int >( this ) );
	pxDialog->SetHelpText( "Choose a material type for this object's collision FX (NOT used for surface effects, this uses the surface type from the material response)" );
	pxDialog->SetEnabled( !bGotBlueprint );
	pxOverlay->AddDialog( pxDialog );
}

bool ToolsAVP_Entity_PhysicalObject::HandlesThisEvent( int iEventID ) const
{
	switch ( iEventID )
	{
// 		case AVP_ENTITY_EVENT_ONHEALTHPERCENT0: //- those events never worked.
// 		case AVP_ENTITY_EVENT_ONHEALTHPERCENT1:
// 		case AVP_ENTITY_EVENT_ONHEALTHPERCENT2:
// 		case AVP_ENTITY_EVENT_ONHEALTHPERCENT3:
// 		case AVP_ENTITY_EVENT_ONHEALTHPERCENT4:
// 		case AVP_ENTITY_EVENT_ONHEALTHPERCENT5:
// 		case AVP_ENTITY_EVENT_ONHEALTHPERCENT6:
// 		case AVP_ENTITY_EVENT_ONHEALTHPERCENT7:
// 		case AVP_ENTITY_EVENT_ONHEALTHPERCENT8:
// 		case AVP_ENTITY_EVENT_ONHEALTHPERCENT9:
// 			return true;

		default:
			return PARENT::HandlesThisEvent( iEventID );
	}
}

bool ToolsAVP_Entity_PhysicalObject::HandlesThisMessage( int iMessageID )
{
	if ( iMessageID == AVP_MSGID_SET_HEALTH )
	{
		return true;
	}
	else
	{

		return PARENT::HandlesThisMessage( iMessageID );
	}
}

void ToolsAVP_Entity_PhysicalObject::AddEventMessageBlock( int iEventID )
{
	PARENT::AddEventMessageBlock( iEventID );

	if ( AVP_ENTITY_EVENT_ONHEALTHPERCENT0 <= iEventID && iEventID <= AVP_ENTITY_EVENT_ONHEALTHPERCENT9 )
	{
		AVP_ServerEntity_PhysicalObject* pxPhyObj = s_pxThisToolsPhyObj ? s_pxThisToolsPhyObj->m_pxPhysicalObject : NULL;
		if ( pxPhyObj )
		{
			pxPhyObj->AddOnHealthEventData( iEventID );
		}
		else
		{
			Asura_Assert( false, "ToolsAVP_Entity_PhysicalObject::AddEventMessageBlock - could not find associated physical object" );
		}
	}	
}

int ToolsAVP_Entity_PhysicalObject::RemoveEventMessageBlock( int iEventID )
{
	int iResult = PARENT::RemoveEventMessageBlock( iEventID );

	if ( AVP_ENTITY_EVENT_ONHEALTHPERCENT0 <= iEventID && iEventID <= AVP_ENTITY_EVENT_ONHEALTHPERCENT9 )
	{
		AVP_ServerEntity_PhysicalObject* pxPhyObj = s_pxThisToolsPhyObj ? s_pxThisToolsPhyObj->m_pxPhysicalObject : NULL;
		if ( pxPhyObj )
		{
			pxPhyObj->RemoveOnHealthEventData( iEventID );
		}
		else
		{
			Asura_Assert( false, "ToolsAVP_Entity_PhysicalObject::AddEventMessageBlock - could not find associated physical object" );
		}
	}	

	return iResult;
}


bool ToolsAVP_Entity_PhysicalObject::GetParameter(Asura_Hash_ID uParamID, Tools_Parameter* pxParam) const
{
	switch(uParamID)
	{
		case g_uHash_PhysicalObject_Material_Func:
		{
			if ( m_pxPhysicalObject->GetMaterialType() == ASURA_HASH_ID_UNSET )
			{
				pxParam->SetString( "UNSET" );
			}
			else
			{
				Asura_Blueprint* pxBlueprint = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_SURFACEEFFECTS, m_pxPhysicalObject->GetMaterialType() );
				if ( pxBlueprint )
				{
					pxParam->SetString( pxBlueprint->GetName() );
				}
				else
				{
					pxParam->SetString( "ERROR" );
				}
			}

			break;
		}

        case g_uHash_PhysicalObject_Blueprint_Func:
        {
            if ( m_pxPhysicalObject->GetBlueprintHashID() == ASURA_HASH_ID_UNSET )
            {
                pxParam->SetString( "UNSET" );
            }
            else
            {
                Asura_Blueprint* pxBlueprint = Asura_Blueprint_System::GetBlueprintByHashID( m_pxPhysicalObject->GetBlueprintTypeHashID(), m_pxPhysicalObject->GetBlueprintHashID() );
                if ( pxBlueprint )
                {
                    pxParam->SetString( pxBlueprint->GetName() );
                }
                else
                {
                    pxParam->SetString( "ERROR" );
                }
            }

            break;
        }

		case g_uHash_PhysicalObject_IgnoreForSurfaceReorientation:
		{
			pxParam->SetBool( m_pxPhysicalObject->IsIgnoredForSurfaceReorientation() );
		}
		break;

		case g_uHash_PhysicalObject_IsFlammable:
		{
			pxParam->SetBool( m_pxPhysicalObject->IsFlammable() );
		}
		break;

	    default:
		    return PARENT::GetParameter(uParamID,pxParam);
	}

	return true;
}

bool ToolsAVP_Entity_PhysicalObject::SetParameter(Asura_Hash_ID uParamID, const Tools_Parameter* pxParam)
{
	switch(uParamID)
	{
		case g_uHash_PhysicalObject_IgnoreForSurfaceReorientation:
		{
			m_pxPhysicalObject->SetIgnoreForSurfaceReorientation( pxParam->GetBool() );
		}
		break;

		case g_uHash_PhysicalObject_IsFlammable:
		{
			m_pxPhysicalObject->SetAsFlammable( pxParam->GetBool() );
		}
		break;

		default:
		    return PARENT::SetParameter(uParamID,pxParam);
	}

	SetEntityModified();

	return true;
}

void ToolsAVP_Entity_PhysicalObject::SetBlueprintHashID( const Asura_Hash_ID uBlueprintHashID )
{
	m_uBlueprintHashID = uBlueprintHashID;
	m_pxPhysicalObject->SetBlueprintHashID( m_uBlueprintHashID );
	m_pxPhysicalObject->InitFromBlueprint();
	UpdateToolsData();
}

void ToolsAVP_Entity_PhysicalObject::RefreshOverlay()
{
	DeleteOverlay();
	CreateOverlay();
}

void ToolsAVP_Entity_PhysicalObject::AddFilesToExportList(List<Asura_String>& xNameList) const
{
	const Asura_Hash_ID	uBlueprintHash	= m_pxPhysicalObject->GetBlueprintHashID();
	if(uBlueprintHash != ASURA_HASH_ID_UNSET)
	{
		const Asura_Hash_ID	uTypeHash	= m_pxPhysicalObject->GetBlueprintTypeHashID();
		const Asura_Hash_ID	uParentHash	= uBlueprintHash;

		Tools_Mode_Blueprint::AddBlueprintFilesToExportList(xNameList, uTypeHash, uParentHash, false);
	}

	// Call the base-function...
	PARENT::AddFilesToExportList(xNameList);
}

void ToolsAVP_Entity_PhysicalObject::ChooseBlueprint( int iThis )
{
    ToolsAVP_Entity_PhysicalObject* pxThis = reinterpret_cast< ToolsAVP_Entity_PhysicalObject* >( iThis );
    if ( !pxThis ) { return; }

    Asura_Hash_ID uBlueprintTypeHashID = pxThis->m_pxPhysicalObject->GetBlueprintTypeHashID();
    Asura_Hash_ID uBlueprintHashID     = pxThis->m_pxPhysicalObject->GetBlueprintHashID();

    if ( uBlueprintTypeHashID == ASURA_HASH_ID_UNSET || uBlueprintHashID == ASURA_HASH_ID_UNSET )
    {
        uBlueprintTypeHashID = pxThis->GetDefaultBlueprintTypeHashID();
        uBlueprintHashID     = pxThis->GetDefaultBlueprintHashID();
    }

    BlueprintSelect_Overlay xBPOverlay( "Choose A Blueprint", &uBlueprintHashID, &uBlueprintTypeHashID );
    if ( !xBPOverlay.DoModal() ) { return; }

    // It's our job now to iterate through all of the selected tools objects (of the correct type) and set this as the blueprint hash ID.
    for ( LIF< Tools_Object* > xObjectIter( &Tools_Object_System::SelectedObjectList ); !xObjectIter.done(); xObjectIter.next() )
    {
        ToolsAVP_Entity_PhysicalObject* pxPhyObjObject = s_xList.FindEntity( xObjectIter()->GetGuid() );
        if ( !pxPhyObjObject ) { continue; }

        pxPhyObjObject->m_pxPhysicalObject->SetBlueprintHashID( uBlueprintHashID );
		// Also update the properties of the object at this point.
		pxPhyObjObject->m_pxPhysicalObject->InitFromBlueprint();
		pxPhyObjObject->UpdateToolsData();
    }

	pxThis->SetEntityModified();
	pxThis->RefreshOverlay();
}

void ToolsAVP_Entity_PhysicalObject::ChooseMaterialBlueprint( int iThis )
{
	static const Asura_Hash_ID s_auValidTypesArray[1]	= { uAVP_BLUEPRINT_TYPE_SURFACEEFFECTS };
	static const Asura_Hash_ID s_auValidParentsArray[1]	= { uAVP_BLUEPRINT_NAME_SURFACETYPE_MATERIAL };
	ToolsAVP_Entity_PhysicalObject* pxThis = reinterpret_cast< ToolsAVP_Entity_PhysicalObject* >( iThis );
	if ( !pxThis ) { return; }

	Asura_Hash_ID uBlueprintTypeHashID = uAVP_BLUEPRINT_TYPE_SURFACEEFFECTS;
	Asura_Hash_ID uBlueprintHashID     = pxThis->m_pxPhysicalObject->GetMaterialType();

	if ( uBlueprintHashID == ASURA_HASH_ID_UNSET )
	{
		uBlueprintHashID     = uAVP_BLUEPRINT_NAME_SURFACETYPE_MATERIAL_METAL;
	}

	BlueprintSelect_Overlay xBPOverlay( "Choose A Blueprint For The Collision FX Material", &uBlueprintHashID, &uBlueprintTypeHashID, 1, s_auValidTypesArray, 1, s_auValidParentsArray );
	if ( !xBPOverlay.DoModal() ) { return; }

	// It's our job now to iterate through all of the selected tools objects (of the correct type) and set this as the blueprint hash ID.
	for ( LIF< Tools_Object* > xObjectIter( &Tools_Object_System::SelectedObjectList ); !xObjectIter.done(); xObjectIter.next() )
	{
		ToolsAVP_Entity_PhysicalObject* pxPhyObjObject = s_xList.FindEntity( xObjectIter()->GetGuid() );
		if ( !pxPhyObjObject ) { continue; }

		pxPhyObjObject->m_pxPhysicalObject->SetMaterialType( uBlueprintHashID );
	}

	pxThis->SetEntityModified();
	pxThis->RefreshOverlay();
}

Asura_Hash_ID ToolsAVP_Entity_PhysicalObject::GetDefaultBlueprintTypeHashID() const
{
    return uAVP_BLUEPRINT_TYPE_PHYSICAL_OBJECTS;    
}

Asura_Hash_ID ToolsAVP_Entity_PhysicalObject::GetDefaultBlueprintHashID() const
{
    return 0x19277846; // Asura_GetHashID( "PhysicsObject" )
}

void ToolsAVP_Entity_PhysicalObject::RefreshBlueprintProperties()
{
    // Iterate through all of the selected tools objects (of the correct type) and reset the blueprint data.
	for ( LIF< Tools_Object* > xObjectIter( &Tools_Object_System::SelectedObjectList ); !xObjectIter.done(); xObjectIter.next() )
	{
		ToolsAVP_Entity_PhysicalObject* pxPhyObjObject = s_xList.FindEntity( xObjectIter()->GetGuid() );
		if ( !pxPhyObjObject ) { continue; }

		pxPhyObjObject->m_pxPhysicalObject->InitFromBlueprint();
		pxPhyObjObject->UpdateToolsData();
	}
}

void ToolsAVP_Entity_PhysicalObject::SelectAllWithBlueprint( int iThis )
{
    ToolsAVP_Entity_PhysicalObject* pxThis = reinterpret_cast< ToolsAVP_Entity_PhysicalObject* >( iThis );
    if ( !pxThis ) { return; }

    Tools_Object* pxCurrentlySelectedObject = Tools_Object_System::SelectedObject;

    const Asura_Hash_ID uBlueprintTypeHashID = pxThis->m_pxPhysicalObject->GetBlueprintTypeHashID();
    const Asura_Hash_ID uBlueprintHashID     = pxThis->m_pxPhysicalObject->GetBlueprintHashID();
    for ( Asura_Hashed_List_It< Tools_Entity_Object > xEOIter( Tools_Entity_Object::GetList() ); !xEOIter.Done(); xEOIter.Next() )
    {
        Tools_Entity_Object* pxEO = xEOIter.GetCurrent();
        if ( !pxEO || !pxEO->Entity ) { continue; }

        if ( pxEO->Entity->GetBlueprintTypeHashID() != uBlueprintTypeHashID ) { continue; }
        if ( pxEO->Entity->GetBlueprintHashID()     != uBlueprintHashID )     { continue; }

        Tools_Object_System::SelectObject( pxEO, true );
    }

    if ( pxCurrentlySelectedObject )
    {
        Tools_Object_System::SelectObject( pxCurrentlySelectedObject, true );
    }
}

void ToolsAVP_Entity_PhysicalObject::EditBlueprint( int iThis )
{
	ToolsAVP_Entity_PhysicalObject* pxThis = reinterpret_cast< ToolsAVP_Entity_PhysicalObject* >( iThis );
	if ( !pxThis ) { return; }

	const Asura_Hash_ID uBlueprintTypeHashID = pxThis->m_pxPhysicalObject->GetBlueprintTypeHashID();
	const Asura_Hash_ID uBlueprintHashID     = pxThis->m_pxPhysicalObject->GetBlueprintHashID();
	if ( uBlueprintTypeHashID == ASURA_HASH_ID_UNSET || uBlueprintHashID == ASURA_HASH_ID_UNSET )
	{
		Tools_Timed_Message_System::AddMessage( Tools_Timed_Message_System::COLOUR_WARNING, "Blueprint details have not been set." );
		return;
	}

	if ( !Tools_Mode_Blueprint::CreateEditOverlay( uBlueprintTypeHashID, uBlueprintHashID ) )
	{
		Tools_Timed_Message_System::AddMessage( Tools_Timed_Message_System::COLOUR_ERROR, "Could not open the blueprint edit overlay." );
	}
}

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////
