//------------------------------------------------------------------------------
// File   : ToolsAVP_Entity_AlienSpawnPoint.cpp
// Author : Gareth Thomas
// Date   : 22 / 10 / 07
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// Include files...
//------------------------------------------------------------------------------
// Always include Asura Core first
#include "Core/Asura.h"

// This Files Header
#include "Entity/Spawners/Alien/ToolsAVP_Entity_AlienSpawnPoint.h"

// The Game Entities Include Files
#include "Entity/Spawners/Alien/AVP_ServerEntity_AlienSpawnPoint.h"

// Asura include files
#include "Camera/Asura_Camera.h"
#include "Core/Asura_Timers.h"
#include "Interface/Asura_Gadget.h"
#include "Interface/Asura_Overlay_System.h"
#include "Mesh/Asura_Mesh_System.h"
#include "Network/Asura_Message.h"
#include "Entity/Locations/Asura_ServerEntity_Node.h"
#include "Rendering/Asura_Render.h"

// Axon Include Files
// AVP Include files
#include "Blueprint/AVP_Blueprint_HashIDs.h"
#include "Entity/GamePlayVolume/AVP_ServerEntity_GamePlayVolume.h"
#include "Actor/Vertebrates/Alien/AVP_Alien_Description.h"

// AVP Axon Include Files
// Tools Include Files
#include "ToolsAVP_ChooseObjects.h"
#include "Entity/Controller/ToolsAVP_Entity_AlienTeam.h"
#include "Entity/GamePlayVolume/ToolsAVP_Entity_GamePlayVolume.h"
#include "Entity/Tools_Entity_Data.h"
#include "Load&Save/Tools_SaveAsura.h"
#include "Misc/Tools_Timed_Messages.h"
#include "Objects/Axon/Tools_Object_Axon_ChooseObjects.h"

//------------------------------------------------------------------------------
// Constants, Defines, Statics and Globals
//------------------------------------------------------------------------------
bool												ToolsAVP_Entity_AlienSpawnPoint::s_bShowAllAlienSpawnPointLinks = false;
ToolsAVP_Entity_AlienSpawnPoint*					ToolsAVP_Entity_AlienSpawnPoint::s_pxToolsAlienSpawnPoint       = NULL;
Asura_Hashed_List<ToolsAVP_Entity_AlienSpawnPoint>	ToolsAVP_Entity_AlienSpawnPoint::s_xList;

const char* ToolsAVP_Entity_AlienSpawnPoint::s_ppszAgilityCategoryNames[AVP_Navigation::NUM_AGILITY_RATINGS]=
{
	"LOW AGILITY",
	"MEDIUM AGILITY",
	"HIGH AGILITY",
};

static const float	s_fLinkCheckPeriod   = 5.0f;

TOOLS_DECLARE_SHARED_PARAM( int,			0x17AB28FB, SpawnPoint_Agility )			// Asura_GetHashID("SpawnPoint_Agility")
TOOLS_DECLARE_SHARED_PARAM( Asura_String,	0xFB7FA229, SpawnPoint_CalculateAgility )	// Asura_GetHashID("SpawnPoint_CalculateAgility")
TOOLS_DECLARE_SHARED_PARAM( u_int,			0x33A41010, SpawnPoint_ClosestWaypoint )	// Asura_GetHashID("SpawnPoint_ClosestWaypoint")
TOOLS_DECLARE_SHARED_PARAM( Asura_String,	0x55EF6EB2, SpawnPoint_RenderRadius )		// Asura_GetHashID("SpawnPoint_RenderRadius")
TOOLS_DECLARE_SHARED_PARAM( float,			0xC298451C, SpawnPoint_Radius )				// Asura_GetHashID("SpawnPoint_Radius")

//------------------------------------------------------------------------------
// Function Implementations
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
ToolsAVP_Entity_AlienSpawnPoint::ToolsAVP_Entity_AlienSpawnPoint(Asura_Entity* pxEntity)
: PARENT(pxEntity)
, m_fRadius(AVP_ServerEntity_AlienSpawnPoint::fALIEN_SPAWNPOINT_DEFAULT_RADIUS)
, m_fRadiusRenderTimer(0.0f)
, m_iSelectedAgility(0)
, m_fLinkCheckTimer( Asura_Random::GetFloat( 0.0f, s_fLinkCheckPeriod ) )
, m_xLastPos( Asura_Maths::ZeroVector3 )
{
	Type	= TOT_Entity_AlienSpawnPoint;
	Mesh	= Asura_Mesh_System::GetMesh("ToolsShape_AlienSpawnPoint");

	AVP_ServerEntity_AlienSpawnPoint* pxGameAlienSpawnPoint = GetGameAlienSpawnPoint();
	m_iSelectedAgility	= pxGameAlienSpawnPoint->m_iAgilityValue;
	m_fRadius			= pxGameAlienSpawnPoint->m_fRadius;

	s_xList.AddToList(this);
}

//------------------------------------------------------------------------------
ToolsAVP_Entity_AlienSpawnPoint::~ToolsAVP_Entity_AlienSpawnPoint(void)
{
	s_xList.RemoveFromList(this);
}

//------------------------------------------------------------------------------
void ToolsAVP_Entity_AlienSpawnPoint::Project_Create(void)
{
	ToolsAVP_Entity_AlienSpawnPoint::Create();
}

//------------------------------------------------------------------------------
void ToolsAVP_Entity_AlienSpawnPoint::Create(void)
{
	const Asura_Guid	uGuid	= Asura_Entity_System::GetCreatedByToolsGuid();

	// Set up tools data and auto-create a unique name (prompt for new name if there is a clash).
	Tools_Entity_Data* pxToolsData = CreateToolsDataWithDefaultName("Alien Spawn Point", "Alien Spawn Point", uGuid);

	if (pxToolsData == NULL)
	{
		Asura_Assert(false, "ToolsAVP_Entity_AlienSpawnPoint::Create() - no tools data");
	}
	else
	{
		AVP_ServerEntity_AlienSpawnPoint* pxServerObject = new AVP_ServerEntity_AlienSpawnPoint(uGuid);

		if (pxServerObject == NULL)
		{
			Asura_Assert(false, "ToolsAVP_Entity_AlienSpawnPoint::Create() - no server object" );
		}
		else
		{
			pxServerObject->ToolsData = pxToolsData;

			ToolsAVP_Entity_AlienSpawnPoint*	pxToolsObject	= new ToolsAVP_Entity_AlienSpawnPoint(pxServerObject);

			// Create the game specific tools object and select it.
			if (pxToolsObject == NULL)
			{
				Asura_Assert(false, "ToolsAVP_Entity_AlienSpawnPoint::Create() - no tools object");
			}
			else
			{
				Tools_Object_System::SelectObject(pxToolsObject);

				pxToolsObject->Position		= *Asura_Camera::GetPosition();
				pxToolsObject->m_uGuid		= uGuid;

				pxServerObject->SetPosition(pxToolsObject->Position);

				Tools_Timed_Message_System::AddMessage(0xffffff, "Created New Alien Spawn Point");
				Tools_SaveAsura::SetEntityModified();
			}
		}
	}
}

//------------------------------------------------------------------------------
ToolsAVP_Entity_AlienSpawnPoint* ToolsAVP_Entity_AlienSpawnPoint::CreateAndReturn()
{
	const Asura_Guid	uGuid	= Asura_Entity_System::GetCreatedByToolsGuid();

	// Set up tools data and auto-create a unique name (prompt for new name if there is a clash).
	Tools_Entity_Data* pxToolsData = CreateToolsDataWithDefaultName("Alien Spawn Point", "Alien Spawn Point", uGuid);

	if (pxToolsData == NULL)
	{
		Asura_Assert(false, "ToolsAVP_Entity_AlienSpawnPoint::CreateAndReturn() - No Tools Data");
		return NULL;
	}
	else
	{
		AVP_ServerEntity_AlienSpawnPoint* pxServerObject = new AVP_ServerEntity_AlienSpawnPoint(uGuid);

		if (pxServerObject == NULL)
		{
			Asura_Assert(false, "ToolsAVP_Entity_AlienSpawnPoint::CreateAndReturn() - No Game Object");
			return NULL;
		}
		else
		{
			pxServerObject->ToolsData = pxToolsData;

			ToolsAVP_Entity_AlienSpawnPoint*	pxToolsObject	= new ToolsAVP_Entity_AlienSpawnPoint(pxServerObject);

			// Create the game specific tools object and select it.
			if (pxToolsObject == NULL)
			{
				Asura_Assert(false, "ToolsAVP_Entity_AlienSpawnPoint::CreateAndReturn() - No Tools Object");
				return NULL;
			}
			else
			{
				Tools_Object_System::SelectObject(pxToolsObject);

				pxToolsObject->Position		= *Asura_Camera::GetPosition();
				pxToolsObject->m_uGuid		= uGuid;

				pxServerObject->SetPosition(pxToolsObject->Position);

				Tools_Timed_Message_System::AddMessage(0xffffff, "Created New Alien Spawn Point");
				Tools_SaveAsura::SetEntityModified();

				return pxToolsObject;
			}
		}
	}
}

//------------------------------------------------------------------------------
Tools_Object* ToolsAVP_Entity_AlienSpawnPoint::Create(Asura_Entity* pxEntity)
{
	return new ToolsAVP_Entity_AlienSpawnPoint(pxEntity);
}

//------------------------------------------------------------------------------
void ToolsAVP_Entity_AlienSpawnPoint::Render()
{
	PARENT::Render();

	if( m_fRadiusRenderTimer > 0.0f )
	{
		static const float fMAX_ALPHA_TIME = 2.0f;
		static const u_int uMAX_ALPHA = 5;
		u_int uAlpha = ( m_fRadiusRenderTimer > fMAX_ALPHA_TIME ) ? uMAX_ALPHA : static_cast<u_int>( uMAX_ALPHA * ( m_fRadiusRenderTimer / fMAX_ALPHA_TIME ) );

		extern void Render_Sphere( const Asura_Vector_3& xCentre, float fRadius, int iColour );
		Render_Sphere( Position, m_fRadius, 0x00FFFFFF + ( uAlpha << 28 ) );
	}

	// Render this spawn point's links, if necessary.
	if ( IsSelected() || s_bShowAllAlienSpawnPointLinks )
	{
		AVP_ServerEntity_AlienSpawnPoint* pxGameAlienSpawnPoint = GetGameAlienSpawnPoint();
        const Asura_Vector_3&			  xOurPosition          = GetGameAlienSpawnPoint()->GetPosition();

		for ( u_int uLink = 0; uLink < pxGameAlienSpawnPoint->m_uNumSpawnPointLinks; ++uLink )
		{
			const AVP_ServerEntity_AlienSpawnPoint* pxTarget =
				AVP_ServerEntity_AlienSpawnPoint::FindEntity( pxGameAlienSpawnPoint->m_puSpawnPointLinks[ uLink ] );
			if ( pxTarget )
			{
				// Is there a link coming the other way?
				bool bReturnLink = false;
				for ( u_int uTargetLink = 0; uTargetLink < pxTarget->m_uNumSpawnPointLinks; ++uTargetLink )
				{
					if ( pxTarget->m_puSpawnPointLinks[ uTargetLink ] == GetGuid() )
					{
						bReturnLink = true;
						break;
					}
				}

				if ( bReturnLink )
				{
					Asura_Render::RenderLine( xOurPosition, pxTarget->GetPosition(), 0xff5050ff );
				}
				else
				{
					RenderLine( xOurPosition, pxTarget->GetPosition(), Asura_Vector_3( 0.3125f, 0.3125f, 1.0f ), Asura_Vector_3( 1.0f, 1.0f, 1.0f ) );
				}
			}
		}
	}
}

void ToolsAVP_Entity_AlienSpawnPoint::RenderLine( const Asura_Vector_3& xStart, const Asura_Vector_3& xEnd,
												  const Asura_Vector_3& xStartColour, const Asura_Vector_3& xEndColour )
{
	const Asura_Vector_3 xDiff      = ( xEnd - xStart ) / static_cast< float >( 5 );
	const float          fInvLength = 1.0f / ( xEnd - xStart ).Magnitude();
	Asura_Vector_3       xP1        = xStart;
	for ( int i = 0; i < 5; ++i )
	{
		const float    fColourBlend = ( ( ( xP1 + ( xDiff * 0.5f ) ) - xStart ) ).Magnitude() * fInvLength;
		Asura_Vector_3 xColourBlend = ( xEndColour * fColourBlend ) + ( xStartColour * ( 1.0f - fColourBlend ) );

		unsigned int   uColour = 0xff000000;
		uColour |= Asura_Maths::FloatToInt( xColourBlend.x * 255.0f ) << 16;
		uColour |= Asura_Maths::FloatToInt( xColourBlend.y * 255.0f ) << 8;
		uColour |= Asura_Maths::FloatToInt( xColourBlend.z * 255.0f );
		Asura_Render::RenderLine( xP1, xP1 + xDiff, uColour );

		xP1 += xDiff;
	}
}

//------------------------------------------------------------------------------
void ToolsAVP_Entity_AlienSpawnPoint::Copy(const Tools_Object* pxObject)
{
	// Copy our parent's data.
	PARENT::Copy(pxObject);

	// Copy our data.
	const ToolsAVP_Entity_AlienSpawnPoint* pxFrom = static_cast< const ToolsAVP_Entity_AlienSpawnPoint* >( pxObject );
	m_iSelectedAgility = pxFrom->m_iSelectedAgility;
	m_fRadius		   = pxFrom->m_fRadius;
	// It's probably not a good idea to copy the gamescene node over.

	// Copy our game entity's data.
	const AVP_ServerEntity_AlienSpawnPoint* pxFromEntity = pxFrom->GetGameAlienSpawnPoint();
	AVP_ServerEntity_AlienSpawnPoint*       pxToEntity   = GetGameAlienSpawnPoint();
	pxToEntity->m_iAgilityValue = pxFromEntity->m_iAgilityValue;
	pxToEntity->m_fRadius		= pxFromEntity->m_fRadius;
	// It's probably not a good idea to copy the gamescene node over.
}

//------------------------------------------------------------------------------
Tools_Object* ToolsAVP_Entity_AlienSpawnPoint::CreateObjectOfThisType()	const
{
	return CreateAndReturn();
}

//------------------------------------------------------------------------------
void ToolsAVP_Entity_AlienSpawnPoint::UpdateWhileSelected(void)
{
	PARENT::UpdateWhileSelected();

	UpdateToolsObjectName();
}

//------------------------------------------------------------------------------
void ToolsAVP_Entity_AlienSpawnPoint::Update()
{
	PARENT::Update();

	AVP_ServerEntity_AlienSpawnPoint* pxGameAlienSpawnPoint = GetGameAlienSpawnPoint();

	if (pxGameAlienSpawnPoint->m_iAgilityValue != m_iSelectedAgility)
	{
		pxGameAlienSpawnPoint->m_iAgilityValue = m_iSelectedAgility;
	}

	if (pxGameAlienSpawnPoint->m_fRadius != m_fRadius)
	{
		pxGameAlienSpawnPoint->m_fRadius = m_fRadius;
		RadiusRender();
	}

	if ( m_fRadiusRenderTimer > 0.0f )
	{
		m_fRadiusRenderTimer -= Asura_Timers::GetGameTimeSlice();
	}

	// Periodically check the links and remove any that are not valid.
	m_fLinkCheckTimer -= Asura_Timers::GetGameTimeSlice();
	if ( m_fLinkCheckTimer <= 0.0f )
	{
		m_fLinkCheckTimer += s_fLinkCheckPeriod;

		for ( u_int uLink = 0; uLink < pxGameAlienSpawnPoint->m_uNumSpawnPointLinks;   )
		{
			const Asura_Guid uTargetGuid = pxGameAlienSpawnPoint->m_puSpawnPointLinks[ uLink ];
			const AVP_ServerEntity_AlienSpawnPoint* pxTarget = AVP_ServerEntity_AlienSpawnPoint::FindEntity( uTargetGuid );
			if ( pxTarget )
			{
				++uLink;
			}
			else
			{
				DeleteSpawnerLinkToThis( uTargetGuid );
				Tools_Timed_Message_System::AddMessage(
					Tools_Timed_Message_System::COLOUR_WARNING,
					"Removed the link between Alien Spawn Point %u and %u.", GetGuid(), uTargetGuid );
			}
		}
	}
}

//------------------------------------------------------------------------------
void ToolsAVP_Entity_AlienSpawnPoint::UpdateToolsObjectName()
{
	AVP_ServerEntity_AlienSpawnPoint* pxSpawnPoint = GetGameAlienSpawnPoint();
	if (pxSpawnPoint)
	{
		bool bFound = false;

		// set the name depending on what volume the spawn point is in.
		for (Asura_Hashed_List_It<ToolsAVP_Entity_GamePlayVolume> it(ToolsAVP_Entity_GamePlayVolume::GetList()); !it.Done(); it.Next())
		{
			ToolsAVP_Entity_GamePlayVolume* pxToolsGamePlayVol = it.GetCurrent();

			if (pxToolsGamePlayVol)
			{
				if (pxToolsGamePlayVol->Entity)
				{
					AVP_ServerEntity_GamePlayVolume* pxGamePlayVol = (AVP_ServerEntity_GamePlayVolume*)pxToolsGamePlayVol->Entity;

					if (pxGamePlayVol)
					{
						if (pxGamePlayVol->GetNumVolumes() > 0)
						{
							if (pxGamePlayVol->Contains(pxSpawnPoint->GetPosition()))
							{
								bFound = true;

								char szNameBuffer[128];
								sprintf(szNameBuffer, "%s-%s-%u", pxToolsGamePlayVol->GetName(), "ASP", GetGuid() - 100000);
								Asura_String xName = szNameBuffer;
								Entity->ToolsData->Name = xName;
								break;
							}
						}
					}
				}
			}
		}

		if (!bFound)
		{
			char szNameBuffer[128];
			sprintf(szNameBuffer, "%s-%s-%u", "NO ZONE", "ASP", GetGuid() - 100000);
			Asura_String xName = szNameBuffer;
			Entity->ToolsData->Name = xName;
		}

		// Update closest waypoint
		const Asura_Vector_3 xPos = pxSpawnPoint->GetPosition();

		if( xPos != m_xLastPos )
		{
			pxSpawnPoint->FindNearestWaypoint();
			m_xLastPos = xPos;
		}
	}
}

//------------------------------------------------------------------------------
void ToolsAVP_Entity_AlienSpawnPoint::OnSelection(void)
{
	s_pxToolsAlienSpawnPoint = this;

	// Invalidate prev pos
	m_xLastPos = GetGameAlienSpawnPoint()->GetPosition();
	m_xLastPos.x += 1.0f;
	//

	PARENT::OnSelection();
}

//------------------------------------------------------------------------------
void ToolsAVP_Entity_AlienSpawnPoint::OnDeselection(void)
{
	PARENT::OnDeselection();

	s_pxToolsAlienSpawnPoint = NULL;
}

//------------------------------------------------------------------------------
void ToolsAVP_Entity_AlienSpawnPoint::AddOverlayControls(Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset)
{
	Asura_Dialog* pxDialog;

	PARENT::AddOverlayControls(pxOverlay, xOffset);

	m_iSelectedAgility = Asura_Maths::Max( Asura_Maths::Min( m_iSelectedAgility, static_cast<int>( AVP_Navigation::AR_HIGH ) ), 0 );

	//add drop down list for categories
 	pxDialog = new Asura_DropDown_List_Dialog(xOffset, s_ppszAgilityCategoryNames, &g_xParam_SpawnPoint_Agility, AVP_Navigation::NUM_AGILITY_RATINGS, AVP_Navigation::NUM_AGILITY_RATINGS, NULL, &Changed);
 	pxOverlay->AddDialog(pxDialog);
	const float fOldXOffset = xOffset.x;
	xOffset.x += pxDialog->GetMaxCoords().x;

	pxDialog = new Asura_Function_Dialog( "", &g_xParam_SpawnPoint_CalculateAgility, xOffset, UpdateAgilityFromSpawnPoint);
	pxDialog->SetHelpText( "Calculate minimum agility value from current waypoint" );
	pxOverlay->AddDialog(pxDialog);
	xOffset.x = fOldXOffset;
	xOffset.y += Asura_Dialog::DefaultSpacing();

	// Display current waypoint num
	Asura_Numerical_Dialog* pxNumericDialog = new Asura_Numerical_Dialog("Closest waypoint", xOffset, &g_xParam_SpawnPoint_ClosestWaypoint, 0, ASURA_INVALID_WAYPOINT_ID, 1, "%u" );
	pxNumericDialog->SetReadOnly(true);
	pxOverlay->AddDialog(pxNumericDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();

	pxDialog = new Asura_Function_Dialog( "", &g_xParam_SpawnPoint_RenderRadius, xOffset, RadiusRender);
	pxDialog->SetHelpText( "Display radius sphere." );
	pxOverlay->AddDialog(pxDialog);
	xOffset.x += ( pxDialog->m_pxGadgetList ? pxDialog->m_pxGadgetList->m_fWidth : Asura_Dialog::LargeSpacing() );

	pxDialog = new Asura_Numerical_Dialog( "Radius", xOffset, &g_xParam_SpawnPoint_Radius, 0.1f, 100.0f, 0.1f, "%.1f");	
	pxDialog->SetHelpText( "The effective radius of the spawn point. This value has multiple uses. For example a new alien can only be spawned at this spawn point if there are no aliens currently inside the radius. Furthermore, un-spawning aliens is only possible if they are inside the radius." );
	pxOverlay->AddDialog(pxDialog);
	xOffset.x = fOldXOffset;
	xOffset.y += Asura_Dialog::LargeSpacing();
/*
	pxDialog = new Asura_Function_Dialog( "Add Nav Link To Another Spawner", xOffset, AddSpawnerLink );
	pxDialog->SetHelpText( "Links this Alien spawner to another Alien spawner. Aliens may then traverse this link in game." );
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog( "Delete Nav Link To Another Spawner", xOffset, DeleteSpawnerLink, &Changed, CanDeleteSpawnerLink );
	pxDialog->SetHelpText( "Deletes a link to another Alien spawner." );
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	Asura_Tickbox_Dialog* pxTickboxDialog = new Asura_Tickbox_Dialog( "Show All Alien Spawn Point Links", xOffset, &s_bShowAllAlienSpawnPointLinks );
	pxTickboxDialog->SetUseSharedDialogColour( true );
	pxOverlay->AddDialog( pxTickboxDialog );
	xOffset.y += Asura_Dialog::LargeSpacing();
*/
	pxDialog = new Asura_Function_Dialog( "Edit Alien Team Object", xOffset, SelectAlienTeam);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();
}

//------------------------------------------------------------------------------
void ToolsAVP_Entity_AlienSpawnPoint::SetSelectedSpawner(ToolsAVP_Entity_AlienSpawnPoint* pxSpawnPoint)
{
	s_pxToolsAlienSpawnPoint = pxSpawnPoint;
}

//------------------------------------------------------------------------------
void ToolsAVP_Entity_AlienSpawnPoint::ClearSelectedSpawner()
{
	s_pxToolsAlienSpawnPoint = NULL;
}

//------------------------------------------------------------------------------
void ToolsAVP_Entity_AlienSpawnPoint::Rename()
{
	Asura_Assert(s_pxToolsAlienSpawnPoint,"");

	Tools_Object_System::AskForName(s_pxToolsAlienSpawnPoint->Entity->ToolsData->Name,"Rename entity :", s_pxToolsAlienSpawnPoint);
	s_pxToolsAlienSpawnPoint->SetEntityFileModified();
}

//------------------------------------------------------------------------------
void ToolsAVP_Entity_AlienSpawnPoint::EditComment()
{
	Asura_Assert(s_pxToolsAlienSpawnPoint,"");

	GetString_Overlay xOverlay("Comment Text :",s_pxToolsAlienSpawnPoint->Entity->ToolsData->m_xComment.GetString(),1000);
	if(xOverlay.DoModal())
	{
		s_pxToolsAlienSpawnPoint->Entity->ToolsData->m_xComment = xOverlay.GetString();
		s_pxToolsAlienSpawnPoint->SetEntityFileModified();

		Tools_Object_System::SetCommentText(s_pxToolsAlienSpawnPoint->Entity->ToolsData->m_xComment.GetString(),s_pxToolsAlienSpawnPoint);
	}
}

//------------------------------------------------------------------------------
void ToolsAVP_Entity_AlienSpawnPoint::RadiusRender()
{
    // Iterate through all of the selected tools objects (of the correct type) and update render timer.
    for ( LIF< Tools_Object* > xObjectIter( &Tools_Object_System::SelectedObjectList ); !xObjectIter.done(); xObjectIter.next() )
    {
        ToolsAVP_Entity_AlienSpawnPoint* pxSpawnPoint = s_xList.FindEntity( xObjectIter()->GetGuid() );
        if ( !pxSpawnPoint )
		{ 
			continue;
		}

		static const float fRENDER_RADIUS_TIME = 3.0f;
        pxSpawnPoint->m_fRadiusRenderTimer = fRENDER_RADIUS_TIME;
    }
}

//------------------------------------------------------------------------------
void ToolsAVP_Entity_AlienSpawnPoint::SelectAlienTeam()
{
	if (ToolsAVP_Entity_AlienTeam::GetInstance())
	{
        Tools_Object_System::SelectObject( ToolsAVP_Entity_AlienTeam::GetInstance() );
	}
	else
	{
		Tools_Timed_Message_System::AddMessage(0xff0000,"Alien team not created");
	}
}

void ToolsAVP_Entity_AlienSpawnPoint::AddSpawnerLink()
{
	Tools_ChooseObject_ByType* pxChooseObject =
		new Tools_ChooseObject_ByType( TOT_Entity_AlienSpawnPoint, StaticAddSpawnerLinkToThis, &s_pxToolsAlienSpawnPoint->Changed );
	if ( pxChooseObject )
	{
		Tools_Object_System::ChooseObject( "Pick The Alien Spawn Point To Link.", pxChooseObject );
	}
}

void ToolsAVP_Entity_AlienSpawnPoint::StaticAddSpawnerLinkToThis( Asura_Guid uGuid )
{
	if ( !s_pxToolsAlienSpawnPoint || uGuid == ASURA_GUID_UNREGISTERED )
	{
		return;
	}
	if ( s_pxToolsAlienSpawnPoint->GetGuid() == uGuid )
	{
		Tools_Timed_Message_System::AddMessage( Tools_Timed_Message_System::COLOUR_ERROR, "Cannot link to self." );
		return;
	}

	// Check we're not already linked to this spawner.
	AVP_ServerEntity_AlienSpawnPoint* pxGameAlienSpawnPoint = s_pxToolsAlienSpawnPoint->GetGameAlienSpawnPoint();
	for ( u_int uLink = 0; uLink < pxGameAlienSpawnPoint->m_uNumSpawnPointLinks; ++uLink )
	{
		if ( pxGameAlienSpawnPoint->m_puSpawnPointLinks[ uLink ] == uGuid )
		{
			Tools_Timed_Message_System::AddMessage( Tools_Timed_Message_System::COLOUR_ERROR, "Already linked to this Alien spawn point." );
			return;
		}
	}

	// Link this guid in.
	Asura_Guid* puNewLinks = new Asura_Guid[ pxGameAlienSpawnPoint->m_uNumSpawnPointLinks + 1 ];
	for ( u_int uLink = 0; uLink < pxGameAlienSpawnPoint->m_uNumSpawnPointLinks; ++uLink )
	{
		puNewLinks[ uLink ] = pxGameAlienSpawnPoint->m_puSpawnPointLinks[ uLink ];
	}
	puNewLinks[ pxGameAlienSpawnPoint->m_uNumSpawnPointLinks ] = uGuid;

	// Swap the arrays over.
	delete [] pxGameAlienSpawnPoint->m_puSpawnPointLinks;
	pxGameAlienSpawnPoint->m_puSpawnPointLinks = puNewLinks;
	++pxGameAlienSpawnPoint->m_uNumSpawnPointLinks;
}

void ToolsAVP_Entity_AlienSpawnPoint::DeleteSpawnerLink()
{
	Tools_ChooseObject_ByType* pxChooseObject =
		new Tools_ChooseObject_ByType( TOT_Entity_AlienSpawnPoint, StaticDeleteSpawnerLinkToThis, &s_pxToolsAlienSpawnPoint->Changed );
	if ( pxChooseObject )
	{
		Tools_Object_System::ChooseObject( "Pick The Alien Spawn Point To Unlink.", pxChooseObject );
	}
}

void ToolsAVP_Entity_AlienSpawnPoint::StaticDeleteSpawnerLinkToThis( Asura_Guid uGuid )
{
	if ( s_pxToolsAlienSpawnPoint )
	{
		s_pxToolsAlienSpawnPoint->DeleteSpawnerLinkToThis( uGuid );
	}
}

void ToolsAVP_Entity_AlienSpawnPoint::DeleteSpawnerLinkToThis( Asura_Guid uGuid )
{
	if ( uGuid == ASURA_GUID_UNREGISTERED || GetGuid() == uGuid )
	{
		return;
	}

	// Check we're already linked to this spawner.
	AVP_ServerEntity_AlienSpawnPoint* pxGameAlienSpawnPoint = GetGameAlienSpawnPoint();
	u_int uIndexToDelete = 0xffffffff;
	for ( u_int uLink = 0; uLink < pxGameAlienSpawnPoint->m_uNumSpawnPointLinks; ++uLink )
	{
		if ( pxGameAlienSpawnPoint->m_puSpawnPointLinks[ uLink ] == uGuid )
		{
			uIndexToDelete = uLink;
			break;
		}
	}
	if ( uIndexToDelete == 0xffffffff )
	{
		Tools_Timed_Message_System::AddMessage( Tools_Timed_Message_System::COLOUR_ERROR, "Not linked to this Alien spawn point." );
		return;
	}

	// Delete this guid from the links.
	if ( pxGameAlienSpawnPoint->m_uNumSpawnPointLinks == 1 )
	{
		delete [] pxGameAlienSpawnPoint->m_puSpawnPointLinks;
		pxGameAlienSpawnPoint->m_puSpawnPointLinks   = NULL;
		pxGameAlienSpawnPoint->m_uNumSpawnPointLinks = 0;
	}
	else
	{
		Asura_Guid* puNewLinks = new Asura_Guid[ pxGameAlienSpawnPoint->m_uNumSpawnPointLinks - 1 ];
		for ( u_int uReadIndex = 0, uWriteIndex = 0; uReadIndex < pxGameAlienSpawnPoint->m_uNumSpawnPointLinks; ++uReadIndex )
		{
			if ( pxGameAlienSpawnPoint->m_puSpawnPointLinks[ uReadIndex ] != uGuid )
			{
				puNewLinks[ uWriteIndex++ ] = pxGameAlienSpawnPoint->m_puSpawnPointLinks[ uReadIndex ];
			}
		}

		// Swap the arrays over.
		delete [] pxGameAlienSpawnPoint->m_puSpawnPointLinks;
		pxGameAlienSpawnPoint->m_puSpawnPointLinks = puNewLinks;
		--pxGameAlienSpawnPoint->m_uNumSpawnPointLinks;
	}
}

bool ToolsAVP_Entity_AlienSpawnPoint::CanDeleteSpawnerLink()
{
	if ( s_pxToolsAlienSpawnPoint )
	{
		return ( s_pxToolsAlienSpawnPoint->GetGameAlienSpawnPoint()->m_uNumSpawnPointLinks > 0 );
	}
	else
	{
		return false;
	}
}

AVP_ServerEntity_AlienSpawnPoint* ToolsAVP_Entity_AlienSpawnPoint::GetGameAlienSpawnPoint() const
{
	return static_cast< AVP_ServerEntity_AlienSpawnPoint* >( Entity );
}

void ToolsAVP_Entity_AlienSpawnPoint::UpdateAgilityFromSpawnPoint()
{
    // Iterate through all of the selected tools spawn point entities, and update the agility values.
    for ( LIF< Tools_Object* > xObjectIter( &Tools_Object_System::SelectedObjectList ); !xObjectIter.done(); xObjectIter.next() )
    {
        ToolsAVP_Entity_AlienSpawnPoint* pxToolsSpawnPoint = s_xList.FindEntity( xObjectIter()->GetGuid() );
        if ( !pxToolsSpawnPoint )
		{ 
			continue;
		}

		AVP_ServerEntity_AlienSpawnPoint* pxSpawnPoint = pxToolsSpawnPoint->GetGameAlienSpawnPoint();

		pxSpawnPoint->FindNearestWaypoint();
		pxSpawnPoint->CalculateRealAgility();
		pxToolsSpawnPoint->m_iSelectedAgility = pxSpawnPoint->m_iRealAgilityValue;
	}
}

bool ToolsAVP_Entity_AlienSpawnPoint::GetParameter(Asura_Hash_ID uParamID, Tools_Parameter* pxParam) const
{
	switch(uParamID)
	{
		case g_uHash_SpawnPoint_Agility:
			pxParam->SetInt(m_iSelectedAgility);
		    break;

		case g_uHash_SpawnPoint_CalculateAgility:
			pxParam->SetString("Calc. from WP");
		    break;

		case g_uHash_SpawnPoint_ClosestWaypoint:
			pxParam->SetUInt(GetGameAlienSpawnPoint()->m_usWaypointID);
			break;

		case g_uHash_SpawnPoint_RenderRadius:
			pxParam->SetString(" ");
		    break;

		case g_uHash_SpawnPoint_Radius:
		    pxParam->SetFloat(m_fRadius);
		    break;

	    default:
		    return PARENT::GetParameter(uParamID,pxParam);
	}

	return true;
}

bool ToolsAVP_Entity_AlienSpawnPoint::SetParameter(Asura_Hash_ID uParamID, const Tools_Parameter* pxParam)
{
	switch(uParamID)
	{
		case g_uHash_SpawnPoint_Agility:
			m_iSelectedAgility = pxParam->GetInt();
		    break;

		case g_uHash_SpawnPoint_CalculateAgility:
		    break;

		case g_uHash_SpawnPoint_ClosestWaypoint:
			// Variable is read-only
			break;

		case g_uHash_SpawnPoint_RenderRadius:
		    break;

		case g_uHash_SpawnPoint_Radius:
		    m_fRadius = pxParam->GetFloat();
		    break;

	    default:
		    return PARENT::SetParameter(uParamID,pxParam);
	}

	SetEntityModified();
	return true;
}
