//------------------------------------------------------------------------------
// File   : ToolsAVP_Entity_HumanSpawnPoint.cpp
// Author : Gareth Thomas
// Date   : 25 / 02 / 08
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// Include files...
//------------------------------------------------------------------------------
// Always include Asura Core first
#include "Core/Asura.h"

// This Files Header
#include "Entity/Spawners/Human/ToolsAVP_Entity_HumanSpawnPoint.h"

// The Game Entities Include Files
#include "Entity/Spawners/Human/AVP_ServerEntity_HumanSpawnPoint.h"

// Asura include files
#include "Camera/Asura_Camera.h"
#include "Core/Asura_Timers.h"
#include "Interface/Asura_Gadget.h"
#include "Interface/Asura_Overlay_System.h"
#include "Mesh/Asura_Mesh_System.h"
#include "Network/Asura_Message.h"
#include "Entity/Locations/Asura_ServerEntity_Node.h"
#include "Rendering/Asura_Render.h"

// Axon Include Files
// AVP Include files
#include "Blueprint/AVP_Blueprint_HashIDs.h"
#include "Entity/GamePlayVolume/AVP_ServerEntity_GamePlayVolume.h"
#include "Actor/Vertebrates/Human/AVP_Human_Description.h"
#include "Entity/Locations/GuardZone/AVP_ServerNode_GuardZone.h"

// AVP Axon Include Files
// Tools Include Files
#include "ToolsAVP_ChooseObjects.h"
#include "Entity/Controller/ToolsAVP_Entity_MarineTeam.h"
#include "Entity/GamePlayVolume/ToolsAVP_Entity_GamePlayVolume.h"
#include "Entity/Tools_Entity_Data.h"
#include "Load&Save/Tools_SaveAsura.h"
#include "Misc/Tools_Timed_Messages.h"
#include "Movement/Tools_Movement.h"
#include "Misc/Tools_Render.h"

//------------------------------------------------------------------------------
// Constants, Defines, Statics and Globals
//------------------------------------------------------------------------------
ToolsAVP_Entity_HumanSpawnPoint*					ToolsAVP_Entity_HumanSpawnPoint::s_pxToolsHumanSpawnPoint       = NULL;
Asura_Hashed_List<ToolsAVP_Entity_HumanSpawnPoint>	ToolsAVP_Entity_HumanSpawnPoint::s_xList;

//------------------------------------------------------------------------------
// Function Implementations
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
ToolsAVP_Entity_HumanSpawnPoint::ToolsAVP_Entity_HumanSpawnPoint(Asura_Entity* pxEntity)
: PARENT(pxEntity)
{
	Type	= TOT_Entity_HumanSpawnPoint;
	Mesh	= Asura_Mesh_System::GetMesh("ToolsShape_HumanSpawnPoint");

	SetUseGameSceneIn(false);

	s_xList.AddToList(this);
}

//------------------------------------------------------------------------------
ToolsAVP_Entity_HumanSpawnPoint::~ToolsAVP_Entity_HumanSpawnPoint(void)
{
	s_xList.RemoveFromList(this);
}

//------------------------------------------------------------------------------
void ToolsAVP_Entity_HumanSpawnPoint::Project_Create(void)
{
	ToolsAVP_Entity_HumanSpawnPoint::Create();
}

//------------------------------------------------------------------------------
void ToolsAVP_Entity_HumanSpawnPoint::Create(void)
{
	const Asura_Guid	uGuid	= Asura_Entity_System::GetCreatedByToolsGuid();

	// Set up tools data and auto-create a unique name (prompt for new name if there is a clash).
	Tools_Entity_Data* pxToolsData = CreateToolsDataWithDefaultName("Human Spawn Point", "Human Spawn Point", uGuid);

	if (pxToolsData == NULL)
	{
		Asura_Assert(false, "ToolsAVP_Entity_HumanSpawnPoint::Create() - no tools data");
	}
	else
	{
		AVP_ServerEntity_HumanSpawnPoint* pxServerObject = new AVP_ServerEntity_HumanSpawnPoint(uGuid);

		if (pxServerObject == NULL)
		{
			Asura_Assert(false, "ToolsAVP_Entity_HumanSpawnPoint::Create() - no server object" );
		}
		else
		{
			pxServerObject->ToolsData = pxToolsData;

			ToolsAVP_Entity_HumanSpawnPoint*	pxToolsObject	= new ToolsAVP_Entity_HumanSpawnPoint(pxServerObject);

			// Create the game specific tools object and select it.
			if (pxToolsObject == NULL)
			{
				Asura_Assert(false, "ToolsAVP_Entity_HumanSpawnPoint::Create() - no tools object");
			}
			else
			{
				Tools_Object_System::SelectObject(pxToolsObject);

				pxToolsObject->Position		= *Asura_Camera::GetPosition();
				pxToolsObject->m_uGuid		= uGuid;

				pxServerObject->SetPosition(pxToolsObject->Position);

				Tools_Timed_Message_System::AddMessage(0xffffff, "Created New Human Spawn Point");
				Tools_SaveAsura::SetEntityModified();
			}
		}
	}
}

//------------------------------------------------------------------------------
ToolsAVP_Entity_HumanSpawnPoint* ToolsAVP_Entity_HumanSpawnPoint::CreateAndReturn()
{
	const Asura_Guid	uGuid	= Asura_Entity_System::GetCreatedByToolsGuid();

	// Set up tools data and auto-create a unique name (prompt for new name if there is a clash).
	Tools_Entity_Data* pxToolsData = CreateToolsDataWithDefaultName("Human Spawn Point", "Human Spawn Point", uGuid);

	if (pxToolsData == NULL)
	{
		Asura_Assert(false, "ToolsAVP_Entity_HumanSpawnPoint::CreateAndReturn() - No Tools Data");
		return NULL;
	}
	else
	{
		AVP_ServerEntity_HumanSpawnPoint* pxServerObject = new AVP_ServerEntity_HumanSpawnPoint(uGuid);

		if (pxServerObject == NULL)
		{
			Asura_Assert(false, "ToolsAVP_Entity_HumanSpawnPoint::CreateAndReturn() - No Game Object");
			return NULL;
		}
		else
		{
			pxServerObject->ToolsData = pxToolsData;

			ToolsAVP_Entity_HumanSpawnPoint*	pxToolsObject	= new ToolsAVP_Entity_HumanSpawnPoint(pxServerObject);

			// Create the game specific tools object and select it.
			if (pxToolsObject == NULL)
			{
				Asura_Assert(false, "ToolsAVP_Entity_HumanSpawnPoint::CreateAndReturn() - No Tools Object");
				return NULL;
			}
			else
			{
				Tools_Object_System::SelectObject(pxToolsObject);

				pxToolsObject->Position		= *Asura_Camera::GetPosition();
				pxToolsObject->m_uGuid		= uGuid;

				pxServerObject->SetPosition(pxToolsObject->Position);

				Tools_Timed_Message_System::AddMessage(0xffffff, "Created New Human Spawn Point");
				Tools_SaveAsura::SetEntityModified();

				return pxToolsObject;
			}
		}
	}
}

//------------------------------------------------------------------------------
Tools_Object* ToolsAVP_Entity_HumanSpawnPoint::Create(Asura_Entity* pxEntity)
{
	return new ToolsAVP_Entity_HumanSpawnPoint(pxEntity);
}

//------------------------------------------------------------------------------
void ToolsAVP_Entity_HumanSpawnPoint::Render()
{
	PARENT::Render();

	AVP_ServerEntity_HumanSpawnPoint* pxSpawnPoint = AVP_ServerEntity_HumanSpawnPoint::FindEntity(GetGuid());

	if (pxSpawnPoint)
	{
		if (pxSpawnPoint->GetCivilianGuardZoneGuid() != ASURA_GUID_UNREGISTERED)
		{
			AVP_ServerNode_GuardZone* pxGuardZone = AVP_ServerNode_GuardZone::FindEntity(pxSpawnPoint->GetCivilianGuardZoneGuid());

			if (pxGuardZone)
			{
				Tools_Render::RenderCylinder(Position, pxGuardZone->GetPosition(), 0.05f, 0xffff8800, false);
			}
		}
	}
}

//------------------------------------------------------------------------------
Tools_Object* ToolsAVP_Entity_HumanSpawnPoint::CreateObjectOfThisType()	const
{
	return CreateAndReturn();
}

//------------------------------------------------------------------------------
void ToolsAVP_Entity_HumanSpawnPoint::UpdateWhileSelected(void)
{
	PARENT::UpdateWhileSelected();
}

//------------------------------------------------------------------------------
void ToolsAVP_Entity_HumanSpawnPoint::Update()
{
	PARENT::Update();

	UpdateToolsObjectName();
}

//------------------------------------------------------------------------------
void ToolsAVP_Entity_HumanSpawnPoint::UpdateToolsObjectName()
{
	AVP_ServerEntity_HumanSpawnPoint* pxSpawnPoint = GetGameHumanSpawnPoint();
	if (pxSpawnPoint)
	{
		bool bFound = false;

		// set the name depending on what volume the spawn point is in.
		for (Asura_Hashed_List_It<ToolsAVP_Entity_GamePlayVolume> it(ToolsAVP_Entity_GamePlayVolume::GetList()); !it.Done(); it.Next())
		{
			ToolsAVP_Entity_GamePlayVolume* pxToolsGamePlayVol = it.GetCurrent();

			if (pxToolsGamePlayVol)
			{
				if (pxToolsGamePlayVol->Entity)
				{
					AVP_ServerEntity_GamePlayVolume* pxGamePlayVol = (AVP_ServerEntity_GamePlayVolume*)pxToolsGamePlayVol->Entity;

					if (pxGamePlayVol)
					{
						if (pxGamePlayVol->GetNumVolumes() > 0)
						{
							if (pxGamePlayVol->Contains(pxSpawnPoint->GetPosition()))
							{
								
								bFound = true;

								char szNameBuffer[128];
								sprintf(szNameBuffer, "%s-%s-%u", pxToolsGamePlayVol->GetName(), "HSP", GetGuid() - 100000);
								Asura_String xName = szNameBuffer;

								if (Entity->ToolsData->Name.Compare(xName) != 0)
								{
									Entity->ToolsData->Name = xName;
								}

								// We might not have changed the name but we have still found the first volume with the spawn point inside - 
								// break out so we don't change the name again, if the spawn point is inside two volumes.
								break;
							}
						}
					}
				}
			}
		}

		if (!bFound)
		{
			char szNameBuffer[128];
			sprintf(szNameBuffer, "%s-%s-%u", "NO ZONE", "HSP", GetGuid() - 100000);
			Asura_String xName = szNameBuffer;
			
			if (Entity->ToolsData->Name.Compare(xName) != 0)
			{
				Entity->ToolsData->Name = xName;
			}
		}
	}
}

//------------------------------------------------------------------------------
void ToolsAVP_Entity_HumanSpawnPoint::OnSelection(void)
{
	s_pxToolsHumanSpawnPoint = this;

	PARENT::OnSelection();
}

//------------------------------------------------------------------------------
void ToolsAVP_Entity_HumanSpawnPoint::OnDeselection(void)
{
	PARENT::OnDeselection();

	s_pxToolsHumanSpawnPoint = NULL;
}

//------------------------------------------------------------------------------
void ToolsAVP_Entity_HumanSpawnPoint::AddOverlayControls(Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset)
{
	PARENT::AddOverlayControls(pxOverlay, xOffset);

	Asura_Dialog* pxDialog;

	// Members to remove!
	pxDialog = new Asura_StringDisplay_Dialog("Guard Zone for Civilians To Use", xOffset);
	pxDialog->SetHelpText("Set this and only civilians will take notice all others types of humanoid will Ignore");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	//Button to add a link to an entity
	pxDialog = new Asura_Function_Dialog("Add GuardZone Link", xOffset, SelectGuardZoneAddLink, NULL);
	pxDialog->SetHelpText("Select a GuardZne to link to.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	// Button to remove a link to an entity
	pxDialog = new Asura_Function_Dialog("Remove GuardZone Link", xOffset, SelectGuardZoneRemoveLink, NULL);
	pxDialog->SetHelpText("Remove a Guard Zone link.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();

	// Focus on marine team
	pxDialog = new Asura_Function_Dialog( "Edit And Focus On Marine Team", xOffset, SelectMarineTeam);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();
}

//------------------------------------------------------------------------------
void ToolsAVP_Entity_HumanSpawnPoint::SelectMarineTeam(void)
{
	ToolsAVP_Entity_MarineTeam* pxMarineTeam = ToolsAVP_Entity_MarineTeam::GetInstance();

	if (pxMarineTeam)
	{
		Asura_Vector_3		xNewCameraPos	= pxMarineTeam->Position;
		Asura_Matrix_3x3	xNewCameraOri	= *Asura_Camera::GetOrientation();
		Asura_Vector_3		xFwdVec;

		Asura_Camera::GetForwardVector(xFwdVec);
		xNewCameraPos -= xFwdVec * 8.0f;

		Tools_Movement::TeleportCameraToPosition(&xNewCameraPos, &xNewCameraOri);

		Tools_Object_System::SelectObject(ToolsAVP_Entity_MarineTeam::GetInstance());
	}
	else
	{
		Tools_Timed_Message_System::AddMessage(0xff0000, "Marine Team Not Created");
	}
}

//------------------------------------------------------------------------------
AVP_ServerEntity_HumanSpawnPoint* ToolsAVP_Entity_HumanSpawnPoint::GetGameHumanSpawnPoint() const
{
	return static_cast< AVP_ServerEntity_HumanSpawnPoint* >( Entity );
}


//-----------------------------------------------------------------------------
// ToolsAVP_Entity_Civilian :: SelectGuardZoneAddLink
//-----------------------------------------------------------------------------
void ToolsAVP_Entity_HumanSpawnPoint::SelectGuardZoneAddLink(void)
{
	ToolsAVP_ChooseObject_ByType* pxChooseObject = new ToolsAVP_ChooseObject_ByType(TOT_Entity_GuardZone, SetGuardZoneGuid);
	Tools_Object_System::ChooseObject("Select The Guard Zone", pxChooseObject);
}

//-----------------------------------------------------------------------------
// ToolsAVP_Entity_Civilian :: SelectGuardZoneAddLink
//-----------------------------------------------------------------------------
void ToolsAVP_Entity_HumanSpawnPoint::SelectGuardZoneRemoveLink(void)
{
	if (!s_pxToolsHumanSpawnPoint)
		return;

	AVP_ServerEntity_HumanSpawnPoint* pxSpawnPoint = AVP_ServerEntity_HumanSpawnPoint::FindEntity(s_pxToolsHumanSpawnPoint->GetGuid());

	if (pxSpawnPoint)
	{
		pxSpawnPoint->SetCivilianGuardzoneGuid(ASURA_GUID_UNREGISTERED);
	}
}

//-----------------------------------------------------------------------------
// ToolsAVP_Entity_Civilian :: SetGuardZoneGuid
//-----------------------------------------------------------------------------
void ToolsAVP_Entity_HumanSpawnPoint::SetGuardZoneGuid(Asura_Guid uGuid)
{
	if (!s_pxToolsHumanSpawnPoint)
		return;

	AVP_ServerEntity_HumanSpawnPoint* pxSpawnPoint = AVP_ServerEntity_HumanSpawnPoint::FindEntity(s_pxToolsHumanSpawnPoint->GetGuid());

	if (pxSpawnPoint)
	{
		pxSpawnPoint->SetCivilianGuardzoneGuid(uGuid);
	}
}