//------------------------------------------------------------------------------
// File   : ToolsAVP_Entity_HumanSpawnPoint.h
// Author : Gareth Thomas
// Date   : 25 / 02 / 08
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//------------------------------------------------------------------------------

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE
#ifndef __TOOLSAVP_ENTITY_HUMANSPAWNPOINT_H__
#define __TOOLSAVP_ENTITY_HUMANSPAWNPOINT_H__

//------------------------------------------------------------------------------
// Include files...
//------------------------------------------------------------------------------
#include "Entity/Spawners/ToolsAVP_Entity_SpawnPoint.h"
#include "Entity/Tools_Entity_Object.h"
#include "Entity/Tools_Entity_StartPoint.h"
#include "Interface/Asura_Dialog.h"
#include "Objects/Tools_Object_System.h"

// axon
#include "Core/AVP_Navigation.h"

//------------------------------------------------------------------------------
// Forward class declarations...
//------------------------------------------------------------------------------
class AVP_ServerEntity_HumanSpawnPoint;
class ToolsAVP_Entity_SpawnPoint;

//------------------------------------------------------------------------------
// Class declarations
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// Class        : ToolsAVP_Entity_HumanSpawnPoint
// Author       : Gareth Thomas
// Date         : 22 / 10 / 07
// Description  : Class to handle Tools editing of Human Spawn Points.
//------------------------------------------------------------------------------
class ToolsAVP_Entity_HumanSpawnPoint : public ToolsAVP_Entity_SpawnPoint
{
	typedef ToolsAVP_Entity_SpawnPoint PARENT;

public:

	// Public Construction / Deconstruction functions...
	ToolsAVP_Entity_HumanSpawnPoint						(Asura_Entity* pxEntity);
	virtual ~ToolsAVP_Entity_HumanSpawnPoint			(void);

	// Public Static functions
	static void				Project_Create				(void);
	static void				Create						(void);
	static Tools_Object*	Create						(Asura_Entity* pxEntity);
	static void				SetSelectedSpawner			(ToolsAVP_Entity_HumanSpawnPoint* pxSpawnPoint);
	static void				ClearSelectedSpawner		(void);
	static void				SelectHumanTeam				(void);

	static ToolsAVP_Entity_HumanSpawnPoint* CreateAndReturn(void);

	// Public overridden Virtual functions...
	virtual void			OnSelection					(void);
	virtual void			OnDeselection				(void);
	virtual void			UpdateWhileSelected			(void);
	virtual void			Update						(void);
	virtual void			Render						(void);
	virtual const char*		GetTypeDescription			(void)													{return("Human Spawn Point");}
	virtual void			AddOverlayControls			(Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset);
	virtual bool			CanBeCopied					(void) const											{ return Type == TOT_Entity_HumanSpawnPoint; }

	virtual Tools_Object*	CreateObjectOfThisType()	const;

	static ToolsAVP_Entity_HumanSpawnPoint*								FindEntity		(Asura_Guid uGuid)		{return s_xList.FindEntity(uGuid);}
	static const Asura_Hashed_List<ToolsAVP_Entity_HumanSpawnPoint>*	GetList			(void)					{return &s_xList;}

protected:

	void UpdateToolsObjectName(void);

	static void SelectMarineTeam();
	static void SelectMarineTeam(int iDummyVar)	{SelectMarineTeam();}

	static void SelectGuardZoneAddLink(void);
	static void SelectGuardZoneRemoveLink(void);
	static void SetGuardZoneGuid(Asura_Guid uGuid);

	AVP_ServerEntity_HumanSpawnPoint* GetGameHumanSpawnPoint() const;

private:

	// Private functions and operators...
	ToolsAVP_Entity_HumanSpawnPoint						(void);
	ToolsAVP_Entity_HumanSpawnPoint						(const ToolsAVP_Entity_HumanSpawnPoint& xOther);
	ToolsAVP_Entity_HumanSpawnPoint&	operator=		(const ToolsAVP_Entity_HumanSpawnPoint& xOther);

	static ToolsAVP_Entity_HumanSpawnPoint*				s_pxToolsHumanSpawnPoint;
	static const char*									s_ppszAgilityCategoryNames[AVP_Navigation::NUM_AGILITY_RATINGS];

	static Asura_Hashed_List<ToolsAVP_Entity_HumanSpawnPoint> s_xList;
};   /// End of class 'ToolsAVP_Entity_SpawnPoint'.

#endif   // ifndef __TOOLSAVP_ENTITY_SPAWNPOINT_H__
/// End of file 'ToolsAVP_Entity_SpawnPoint'.