//////////////////////////////////////////////////////////////////////////////
//
// ToolsAVP_Entity_SpawnController.h
//
// Copyright (C) 2008 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __TOOLSAVP_ENTITY_SPAWNCONTROLLER_H__
#define __TOOLSAVP_ENTITY_SPAWNCONTROLLER_H__ 1

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// tools
#include "Entity/Tools_Entity_Object.h"

// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////////

class Asura_Table_Dialog;
class Asura_TableCell_Dialog;
class AVP_ServerEntity_SpawnController;

//////////////////////////////////////////////////////////////////////////////
//
// ToolsAVP_Entity_SpawnController
//
//////////////////////////////////////////////////////////////////////////////

class ToolsAVP_Entity_SpawnController : public Tools_Entity_Object
{
	typedef Tools_Entity_Object PARENT;

public:
	ToolsAVP_Entity_SpawnController( Asura_Entity* pxEntity );
	virtual ~ToolsAVP_Entity_SpawnController();

	virtual const char*	GetTypeDescription() { return "Spawn Controller"; }
	virtual void		AddOverlayControls( Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset );
	virtual void		Update();
	virtual void		OnSelection();
	virtual void		OnDeselection();
	virtual void		UpdateWhileSelected();
	virtual bool		Verify( const bool bAutomatic );

	virtual bool GetParameter( Asura_Hash_ID uParamID, Tools_Parameter* pxParam ) const;
	virtual bool SetParameter( Asura_Hash_ID uParamID, const Tools_Parameter* pxParam );

	virtual bool CanBeCopied() const { return true; }
	virtual void Copy( const Tools_Object* pxObject );
	virtual Tools_Object* CreateObjectOfThisType() const;

	virtual bool HandlesThisEvent( int iEventID ) const;
	virtual bool HandlesThisMessage( int iMessage );
	virtual void AddFilesToExportList( List< Asura_String >& xNameList ) const;

	static void Create();
	static ToolsAVP_Entity_SpawnController* CreateAndReturn();
	static Tools_Object* Create( Asura_Entity* pxEntity );

private:
	AVP_ServerEntity_SpawnController* GetSpawnController() const;
	void VerifyGameObjectData();
	void RequestRefreshOverlay() { m_bRefreshOverlay = true; }
	void RefreshOverlay();
	void AddNewSpawneeType( const Asura_Hash_ID uBlueprintTypeHashID, const Asura_Hash_ID uBlueprintHashID );
	void RemoveSpawneeType( const Asura_Hash_ID uBlueprintTypeHashID, const Asura_Hash_ID uBlueprintHashID );
	void AddNewSpawnPoint( const Asura_Guid uSpawnPointGuid );
	void RemoveSpawnPoint( const Asura_Guid uSpawnPointGuid );
	void AddNewGPVolume( const Asura_Guid uGPVolumeGuid );
	void RemoveGPVolume( const Asura_Guid uGPVolumeGuid );

	void BuildSquadFunctionDialogLabel();
	static void SetSpawneeSquad( int iThis );
	static void SetSpawneeSquad( int iThis, const Asura_Guid uSpawneeSquadGuid );
	static void ResetSpawneeSquad( int iThis );

	Asura_Table_Dialog* CreateSpawneeTypeTable( const Asura_Vector_2& xOffset );
	void CreateSpawneeTypeList();
	void DestroySpawneeTypeList();
	static Asura_TableCell_Dialog* CreateSpawneeTypeCell( Asura_Table_Dialog& xTableDialog,
														  const Asura_Vector_2& xOffset, const Asura_Vector_2& xMinCellSize,
														  const u_int uRow, const u_int uColumn,
														  void* const pxUserData );
	static void AddSpawneeType( int iThis );
	static void RemoveSelectedSpawneeType( int iThis );

	static void OnSpawnSchemeChanged( int iThis );

	Asura_Table_Dialog* CreateSpawnPointTable( const Asura_Vector_2& xOffset );
	void CreateSpawnPointList();
	void DestroySpawnPointList();
	static Asura_TableCell_Dialog* CreateSpawnPointCell( Asura_Table_Dialog& xTableDialog,
														 const Asura_Vector_2& xOffset, const Asura_Vector_2& xMinCellSize,
														 const u_int uRow, const u_int uColumn,
														 void* const pxUserData );
	static void AddSpawnPoints( int iThis );
	static void AddNewSpawnPoint( int iThis, Asura_Guid uSpawnPointGuid );
	static void RemoveSelectedSpawnPoint( int iThis );

	Asura_Table_Dialog* CreateGPVolumeTable( const Asura_Vector_2& xOffset );
	void CreateGPVolumeList();
	void DestroyGPVolumeList();
	static Asura_TableCell_Dialog* CreateGPVolumeCell( Asura_Table_Dialog& xTableDialog,
													   const Asura_Vector_2& xOffset, const Asura_Vector_2& xMinCellSize,
													   const u_int uRow, const u_int uColumn,
													   void* const pxUserData );
	static void AddGPVolume( int iThis );
	static void AddNewGPVolume( int iThis, Asura_Guid uGPVolumeGuid );
	static void RemoveSelectedGPVolume( int iThis );


	ToolsAVP_Entity_SpawnController();
	ToolsAVP_Entity_SpawnController( const ToolsAVP_Entity_SpawnController& xOther );
	ToolsAVP_Entity_SpawnController& operator= ( const ToolsAVP_Entity_SpawnController& xOther );

private:
	bool m_bRefreshOverlay;
	bool m_bMaxInPlayChanged;

	Asura_String		m_xSquadFunctionDialogLabel;

	u_int				m_uSpawneeTypeListSize;
	char**				m_ppszSpawneeTypeList;
	Asura_Table_Dialog* m_pxSpawneeTypeTableDialog;

	u_int				m_uSpawnPointListSize;
	char**				m_ppszSpawnPointList;
	Asura_Table_Dialog* m_pxSpawnPointTableDialog;

	u_int				m_uGPVolumeListSize;
	char**				m_ppszGPVolumeList;
	Asura_Table_Dialog* m_pxGPVolumeTableDialog;

	static Asura_Hash_ID s_uLastSpawneeTypeBPHashID;
};

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

#endif //__TOOLSAVP_ENTITY_SPAWNCONTROLLER_H__
