
///////////////////////////////////////////////////////////////////////////////
//
// File   : Tools_Project_Entity_CheckPointNode.cpp
// Author : Chris Wallace
// Date   : 28-June-2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "Entity/Tools_Project_Entity_CheckPointNode.h"

// Asura independent header-files...
#include "Camera/Asura_Camera.h"
#include "Mesh/Asura_Mesh_System.h"
#include "Network/Asura_Message.h"

// AVP
#include "Entity/CheckPointNode/AVP_ServerEntity_CheckPointNode.h"

// Tools header-files...
#include "Load&Save/Tools_SaveAsura.h"
#include "Misc/Tools_Timed_Messages.h"
#include "Objects/Tools_Object_System.h"
#include "Objects/ToolsAVP_Object_Type.h"


////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Private functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Protected functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Public functions...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Project_Entity_CheckPointNode::Tools_Project_Entity_CheckPointNode
// Author       : Chris Wallace
// Date         : 28-June-2007
////////////////////////////////////////////////////////////////////////////////

Tools_Project_Entity_CheckPointNode::Tools_Project_Entity_CheckPointNode(Asura_Entity* pxEntity)
: PARENT(pxEntity)
{
	Type	= TOT_Entity_CheckPointNode;
	Mesh	= Asura_Mesh_System::GetMesh("ToolsShape_TeleportExit");
}

/// End of function 'Tools_Entity_TeleportDestination::Tools_Entity_TeleportDestination'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Project_Entity_CheckPointNode::Create
// Author       : Chris Wallace
// Date         : 28-June-2007
////////////////////////////////////////////////////////////////////////////////

void Tools_Project_Entity_CheckPointNode::Create(void)
{
	const Asura_Guid	uGuid	= Asura_Entity_System::GetCreatedByToolsGuid();

	// Set up tools data and auto-create a unique name (prompt for new name if there is a clash).
	Tools_Entity_Data* pxToolsData = CreateToolsDataWithDefaultName("Checkpoint Node", "Checkpoint Node", uGuid);

	if(pxToolsData == NULL)
	{
		Asura_Assert(false, "Tools_Project_Entity_CheckPointNode::Create() - no tools data");
	}
	else
	{
		AVP_ServerEntity_CheckPointNode*	pxServerObject	= new AVP_ServerEntity_CheckPointNode(uGuid);

		if(pxServerObject == NULL)
		{
			Asura_Assert(false, "AVP_ServerEntity_CheckPointNode::Create() - no server object" );
		}
		else
		{
			pxServerObject->ToolsData = pxToolsData;

			Tools_Project_Entity_CheckPointNode*	pxToolsObject	= new Tools_Project_Entity_CheckPointNode(pxServerObject);

			// Create the game specific tools object and select it.
			if(pxToolsObject == NULL)
			{
				Asura_Assert(false, "Tools_Project_Entity_CheckPointNode::Create() - no tools object");
			}
			else
			{
				Tools_Object_System::SelectObject(pxToolsObject);

				pxToolsObject->Position		= *Asura_Camera::GetPosition();
				pxToolsObject->Orientation	= *Asura_Camera::GetOrientation();

				pxServerObject->TeleportTo(pxToolsObject->Position, Asura_Quat(pxToolsObject->Orientation));

				Tools_Timed_Message_System::AddMessage(0xffffff, "Created new Checkpoint Node");
				Tools_SaveAsura::SetEntityModified();
			}
		}
	}
}

/// End of function 'Tools_Project_Entity_CheckPointNode::Create'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Project_Entity_CheckPointNode::UpdatePositionInAsuraData
// Author       : Chris Wallace
// Date         : 28-June-2007
////////////////////////////////////////////////////////////////////////////////

void Tools_Project_Entity_CheckPointNode::UpdatePositionInAsuraData(bool bUpdateLinkedObjects)
{
	AVP_ServerEntity_CheckPointNode*		pxAsuraEntity	= static_cast<AVP_ServerEntity_CheckPointNode*>(Entity);
	Asura_Quat								xOrientation(Orientation);

	if((pxAsuraEntity->m_xPosition != Position) || (pxAsuraEntity->m_xOrientation != xOrientation))
	{
		pxAsuraEntity->m_xOrientation	= xOrientation;
		pxAsuraEntity->m_xPosition		= Position;

		Changed							= true;
	}

	PARENT::UpdatePositionInAsuraData(bUpdateLinkedObjects);
}

/// End of function 'Tools_Project_Entity_CheckPointNode::UpdatePositionInAsuraData'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Project_Entity_CheckPointNode::HandlesThisMessage
// Author       : Chris Wallace
// Date         : 29-June-2007
////////////////////////////////////////////////////////////////////////////////

bool Tools_Project_Entity_CheckPointNode::HandlesThisMessage (int iMessageID)
{
	switch(iMessageID)
	{
	case ASURA_MSG_TRIGGER:
		{
			return(true);
		}

	default:
		{
			return(PARENT::HandlesThisMessage(iMessageID));
		}
	}		// End of switch.
}

/// End of function 'Tools_Project_Entity_CheckPointNode::HandlesThisMessage'. ///



/// End of file. ///
