//
// ToolsAVP_Entity_MultiDamageVolume.cpp
//
// Created by Andrew Weinkove - 14/04/2004
// 
// Copyright (C) 2004 Rebellion, All Rights Reserved.
//


#include "Core/Asura.h"
#include "Entity/Triggers/ToolsAVP_Entity_MultiDamageVolume.h"

#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Overlay.h"

#include "Entity/Triggers/AVP_ServerEntity_MultiDamageVolume.h"

#include "Objects/Tools_Object_System.h"
#include "Load&Save/Tools_SaveAsura.h"
#include "Misc/Tools_Timed_Messages.h"

//////////////////////////////////////////////////////////////////////////////////////////////////
/*---------------------------------------**
** ToolsAVP_Entity_MultiDamageVolume **
**---------------------------------------*/

ToolsAVP_Entity_MultiDamageVolume* ToolsAVP_Entity_MultiDamageVolume::s_pxThis = 0;

//////////////////////////////////////////////////////////////////////////////

ToolsAVP_Entity_MultiDamageVolume::ToolsAVP_Entity_MultiDamageVolume(Asura_Entity* pxEntity)
: Tools_Entity_MultiDamageVolume(pxEntity)
{
}

ToolsAVP_Entity_MultiDamageVolume::~ToolsAVP_Entity_MultiDamageVolume()
{
}

void ToolsAVP_Entity_MultiDamageVolume::Create()
{
	//get a name , and allocate the tools data
	Asura_Guid uGuid = Asura_Entity_System::GetCreatedByToolsGuid();
	Tools_Entity_Data* pxToolsData = 0;
	if(Tools_Object_System::IsCopying())
	{
		//This has been created as part of a copy/paste operation , so pick a default name. RAR 23/06/2006
		pxToolsData = Tools_Entity_Object::CreateToolsDataWithDefaultName("Multi Damage Volume","Multi Damage Volume",uGuid);
	}
	else
	{
		pxToolsData = Tools_Entity_Object::CreateToolsData("Multi Damage Volume");
	}
	if(!pxToolsData) return;

	//create the asura data
	AVP_ServerEntity_MultiDamageVolume* pxAsuraEntity = new AVP_ServerEntity_MultiDamageVolume(uGuid);
	pxAsuraEntity->ToolsData = pxToolsData;

	//create the tools object
	Tools_Object* pxObject = new ToolsAVP_Entity_MultiDamageVolume(pxAsuraEntity);
	Tools_Object_System::SelectObject(pxObject);

	Tools_SaveAsura::SetEntityModified();
}

void ToolsAVP_Entity_MultiDamageVolume::OnSelection()
{
    Tools_Entity_MultiDamageVolume::OnSelection();
    s_pxThis = this;
}

void ToolsAVP_Entity_MultiDamageVolume::OnDeselection()
{
    Tools_Entity_MultiDamageVolume::OnDeselection();
    s_pxThis = 0;
}

void ToolsAVP_Entity_MultiDamageVolume::UpdateWhileSelected()
{
	Tools_Entity_MultiDamageVolume::UpdateWhileSelected();
}

Tools_Object* ToolsAVP_Entity_MultiDamageVolume::Create( Asura_Entity* pxEntity )
{
	return new ToolsAVP_Entity_MultiDamageVolume( pxEntity );
}

void ToolsAVP_Entity_MultiDamageVolume::AddOverlayControls( class Asura_Overlay* pxNewOverlay, class Asura_Vector_2& xOffset )
{
	Tools_Entity_MultiDamageVolume::AddOverlayControls( pxNewOverlay, xOffset );

	Asura_Assert( ThisEntity, "" );
	Tools_Entity_MultiDamageVolume* pxToolsEntity = static_cast<Tools_Entity_MultiDamageVolume*>( ThisEntity );
	AVP_ServerEntity_MultiDamageVolume* pxAsuraEntity = static_cast<AVP_ServerEntity_MultiDamageVolume*>( pxToolsEntity->Entity );
	Asura_Assert( NUM_AVP_MULTIDAMAGEVOLUME_TYPES<=32, "Error, maximum number of volume trigger lists is 32" );

    Asura_Dialog* pxNewDialog;

	// first add all of the AI species that this can effect, all have the same help text so can go into a for loop
	for( int iCounter = 0; iCounter < NUM_AVP_AIDAMAGABLE_TYPES; iCounter ++ )
	{
		pxNewDialog = new Asura_FlagTickbox_Dialog(AVP_ServerEntity_MultiDamageVolume::s_aszAVPVolumeTypes[iCounter], xOffset, &pxAsuraEntity->m_iListFlags, ( 1 << iCounter ), &Changed);
		char szTemp[255];
		sprintf( szTemp, "Make this damage volume active for AI Entities of this type: %s", AVP_ServerEntity_MultiDamageVolume::s_aszAVPVolumeTypes[iCounter] );
		pxNewDialog->SetHelpText( szTemp );
		pxNewOverlay->AddDialog( pxNewDialog );
		xOffset.y += Asura_Dialog::DefaultSpacing();
	}

	// now add any other tick boxes with correct help text. 
	pxNewDialog = new Asura_FlagTickbox_Dialog(AVP_ServerEntity_MultiDamageVolume::s_aszAVPVolumeTypes[AVP_MDVT_PLAYER], xOffset, &pxAsuraEntity->m_iListFlags, ( 1 << AVP_MDVT_PLAYER ), &Changed);
	pxNewDialog->SetHelpText( "Make the Damage Volume effect the Player entity, \nNO MATTER WHAT SPECIES IT IS.\n On By Default" );
	pxNewOverlay->AddDialog( pxNewDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();
}