//////////////////////////////////////////////////////////////////////////////
//
// FILE : ToolsAVP_Entity_Vent.cpp
//
// AUTHORS : Will Plano
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"

// header
#include "Entity/Vent/ToolsAVP_Entity_Vent.h"

// asura
#include "Core/Asura_String.h"
#include "Core/Asura_TextureManagement.h"
#include "Mesh/Asura_Mesh_System.h"
#include "Rendering/Asura_Render.h"

// PC rendering include for mesh rendering.
#include "Rendering/Asura_PlatformRenderObject.h"

// avp
#include "Entity/Locations/OrientedNode/GamesceneNode/Asura_ServerNode_GS2Node.h"
#include "Entity/Locations/Spline/GamesceneSpline/Asura_ServerNode_GS2Spline.h"
#include "Entity/PhysicalObject/PhysicsObject/DamageableObject/ToolsAVP_Entity_DamageableObject.h"
#include "Entity/Vent/AVP_ServerEntity_Vent.h"

// tools
#include "Entity/Locations/OrientedNode/GamesceneNode/Tools_EntityNode_GS2Node.h"
#include "Entity/Tools_Entity_Data.h"
#include "Hierarchy/Tools_Animation_Files.h"
#include "Load&Save/Tools_AsuraFileContents.h"
#include "Load&Save/Tools_SaveAsura.h"
#include "Misc/Tools_Render.h"
#include "Misc/Tools_Timed_Messages.h"
#include "Movement/Tools_Movement.h"
#include "Objects/Tools_PositionEdit_Object.h"
#include "Objects/Axon/Tools_Object_Axon_ChooseObjects.h"
#include "PC/FileSystem/Tools_FileSystem.h"

// avp tools
#include "Objects/ToolsAVP_Object_Type.h"

//////////////////////////////////////////////////////////////////////////////
//
// ToolsAVP_Entity_Vent
//
//////////////////////////////////////////////////////////////////////////////

TOOLS_DECLARE_SHARED_PARAM( Asura_String,	0x2d9ab460,	Vent_UndamagedSkinString ); // Asura_GetHashID( "Vent_UndamagedSkinString" )
TOOLS_DECLARE_SHARED_PARAM( u_int,			0x4992e369, Vent_UndamagedSkinUInt );   // Asura_GetHashID( "Vent_UndamagedSkinUInt" )
TOOLS_DECLARE_SHARED_PARAM( Asura_String,	0x76cdc359,	Vent_DamagedSkinString );   // Asura_GetHashID( "Vent_DamagedSkinString" )
TOOLS_DECLARE_SHARED_PARAM( u_int,			0x7bff5ca2,	Vent_DamagedSkinUInt );     // Asura_GetHashID( "Vent_DamagedSkinUInt" )
TOOLS_DECLARE_SHARED_PARAM( bool,			0xe72d6b5a,	Vent_StartsDamaged );

u_int					ToolsAVP_Entity_Vent::s_uObjectCount = 0;
char**					ToolsAVP_Entity_Vent::s_pszVentNode = NULL;
int						ToolsAVP_Entity_Vent::s_iSelectedVentNode = 0;
u_int					ToolsAVP_Entity_Vent::s_uNumVentNodes = 0;
bool					ToolsAVP_Entity_Vent::s_bRefreshOverlay = false;
bool					ToolsAVP_Entity_Vent::s_bNewVentNode = false;
Asura_Guid				ToolsAVP_Entity_Vent::s_uNewVentNode = ASURA_GUID_UNREGISTERED;
ToolsAVP_Entity_Vent*	ToolsAVP_Entity_Vent::s_pxSelectedVent = NULL;

bool ToolsAVP_Entity_Vent::s_bMoveExitPointsWithEntity = true;

Asura_Hashed_List<ToolsAVP_Entity_Vent> ToolsAVP_Entity_Vent::s_xList;

ToolsAVP_Entity_Vent::ToolsAVP_Entity_Vent( AVP_ServerEntity_Vent* pxVentEntity )
: PARENT( pxVentEntity )
, m_pxVentEntity( pxVentEntity )
, m_uSelectedExitPoint(0)
, m_uPositionEditObjectID(0)
, m_iLOSHitExitPoint(-1)
{
	if ( !pxVentEntity->m_bIsDamaged && pxVentEntity->m_uUndamagedSkinHash != ASURA_HASH_ID_UNSET )
	{
		Asura_Hierarchy_Skin* pxSkin = Tools_Animation_Files::GetSkinFromID( m_pxVentEntity->m_uUndamagedSkinHash );
		if ( pxSkin != NULL )
		{
			SetSkinAndAnim( pxSkin, NULL );
		}
	}
	else if ( pxVentEntity->m_bIsDamaged && pxVentEntity->m_uDamagedSkinHash != ASURA_HASH_ID_UNSET )
	{
		Asura_Hierarchy_Skin* pxSkin = Tools_Animation_Files::GetSkinFromID( m_pxVentEntity->m_uDamagedSkinHash );
		if ( pxSkin != NULL )
		{
			SetSkinAndAnim( pxSkin, NULL );
		}
	}
	else
	{
		SetMesh("ToolsShape_Vent");
	}

	m_pxExitPointMesh = Asura_Mesh_System::GetMesh("ToolsShape_Particle");

	Type = TOT_Entity_Vent;
	s_pxSelectedVent = this;
	++s_uObjectCount;
	s_xList.AddToList( this );
}

ToolsAVP_Entity_Vent::~ToolsAVP_Entity_Vent()
{
	s_xList.RemoveFromList( this );
	if ( --s_uObjectCount == 0 )
	{
		if ( s_uNumVentNodes > 0 )
		{
			for ( u_int uItr = 0; uItr < s_uNumVentNodes; ++uItr )
			{
				delete [] s_pszVentNode[ uItr ];
			}
		}
		delete [] s_pszVentNode;
		s_pszVentNode = NULL;
		s_uNumVentNodes = 0;
	}
}

Tools_Object* ToolsAVP_Entity_Vent::Create( Asura_Entity* pxEntity )
{
	return new ToolsAVP_Entity_Vent( AVP_ServerEntity_Vent::FindEntity( pxEntity->GetGuid() ) );
}

void ToolsAVP_Entity_Vent::Create()
{
	// Get a new GUID
	Asura_Guid uGuid = Asura_Entity_System::GetCreatedByToolsGuid();

	// Set up tools data and auto-create a unique name (prompt for new name if there is a clash).
	Tools_Entity_Data* pxToolsData = CreateToolsDataWithDefaultName("Vent", "Vent", uGuid);
	if (!pxToolsData) return;

	pxToolsData->Position = *Asura_Camera::GetPosition();
	pxToolsData->m_xOrientation.Identity(); // At Phil's request, always create at the default orientation.

	// Create the game specfic server object and set its data.
	AVP_ServerEntity_Vent* pxServerObject = new AVP_ServerEntity_Vent(uGuid);
	Asura_Assert(pxServerObject,"");
	if (pxServerObject)
	{
		pxServerObject->ToolsData = pxToolsData;
		pxServerObject->SetPosition(pxToolsData->Position);
		pxServerObject->SetOrientation(pxToolsData->m_xOrientation);
		// Add a couple of exit points immediately.
		pxServerObject->CreateDefaultExitPoints();
	}

	// Create the game specific tools object and select it.
	Tools_Object* pxToolsObject = Create(pxServerObject);
	Tools_Object_System::SelectObject(pxToolsObject);
}


bool ToolsAVP_Entity_Vent::ObjectIsInLOS(const Asura_Vector_3& xLOSOrigin, const Asura_Vector_3& xLOSDirection, float& fNearest)
{
	m_iLOSHitExitPoint = -1;

	bool bInLOS = PARENT::ObjectIsInLOS(xLOSOrigin, xLOSDirection, fNearest);

	if (!bInLOS && m_pxExitPointMesh)
	{
		for (u_int u=0; !bInLOS && (u < m_pxVentEntity->m_uNumExitPoints); ++u)
		{
			const Asura_Vector_3& xPos = m_pxVentEntity->m_pxExitPoints[u].m_xPosition;
			Asura_Matrix_3x3 xOri;
			m_pxVentEntity->m_pxExitPoints[u].m_xOrientation.ToMatrix(xOri);

			Asura_Mesh* pxMeshToUse = m_pxExitPointMesh;
			if (m_pxExitPointMesh->m_pxNextLowerMesh)
			{
				pxMeshToUse = m_pxExitPointMesh->GetDetailMeshFromDistance((*Asura_Camera::GetPosition() - m_pxVentEntity->m_pxExitPoints[u].m_xPosition).Magnitude());
			}

			int iPolygonIndex;
			bInLOS |= pxMeshToUse->LOSTest(xLOSOrigin, xLOSDirection, xPos, xOri, fNearest, iPolygonIndex, IsScaled() ? &GetScale() : NULL);
			if (bInLOS)
			{
				m_iLOSHitExitPoint = static_cast<int>(u);
			}
		}
	}

	return bInLOS;
}

void ToolsAVP_Entity_Vent::UpdatePositionInAsuraData(bool bUpdateLinkedObjects)
{
	if(Position != Entity->ToolsData->Position || Orientation != Entity->ToolsData->m_xOrientation)
	{
		if (s_bMoveExitPointsWithEntity)
		{
			Asura_Quat xInverseOldOri(Entity->ToolsData->m_xOrientation);
			xInverseOldOri.Invert();
			Asura_Quat xNewOri(Orientation);
			const Asura_Vector_3& xOldPos = Entity->ToolsData->Position;
			const Asura_Vector_3& xNewPos = Position;
			for (u_int u=0; u<m_pxVentEntity->m_uNumExitPoints; ++u)
			{
				Asura_Vector_3& xPos = m_pxVentEntity->m_pxExitPoints[u].m_xPosition;
				Asura_Quat& xOri = m_pxVentEntity->m_pxExitPoints[u].m_xOrientation;
				xPos -= xOldPos;
				xPos = xInverseOldOri.RotateVector(xPos);
				xPos = xNewOri.RotateVector(xPos);
				xPos += xNewPos;

				xOri = xInverseOldOri * xOri;
				xOri = xNewOri * xOri;
			}
		}

		m_pxVentEntity->SetPosition( Position );
		m_pxVentEntity->SetOrientation( Orientation );
	}

	PARENT::UpdatePositionInAsuraData(bUpdateLinkedObjects);
}


void ToolsAVP_Entity_Vent::Copy( const Tools_Object* pxObject )
{
	PARENT::Copy( pxObject );

	ToolsAVP_Entity_Vent* pxFrom = ToolsAVP_Entity_Vent::FindEntity( pxObject->GetGuid() );
	if ( pxFrom )
	{
		m_pxVentEntity->m_bIsDamaged = pxFrom->m_pxVentEntity->m_bIsDamaged;

		m_pxVentEntity->m_uUndamagedSkinHash = pxFrom->m_pxVentEntity->m_uUndamagedSkinHash;
		m_pxVentEntity->m_uDamagedSkinHash = pxFrom->m_pxVentEntity->m_uDamagedSkinHash;

		Asura_Hash_ID uSkinHash = m_pxVentEntity->m_bIsDamaged ? m_pxVentEntity->m_uDamagedSkinHash : m_pxVentEntity->m_uUndamagedSkinHash;
		Asura_Hierarchy_Skin* pxSkin = uSkinHash != ASURA_HASH_ID_UNSET ? Tools_Animation_Files::GetSkinFromID( uSkinHash ) : NULL;
		if( pxSkin )
		{
			SetSkinAndAnim( pxSkin, NULL );
		}

		m_pxVentEntity->CopyExitPoints(pxFrom->m_pxVentEntity->m_uNumExitPoints, pxFrom->m_pxVentEntity->m_pxExitPoints);

		if (s_bMoveExitPointsWithEntity)
		{
			Asura_Quat xInverseOldOri(pxFrom->Orientation);
			xInverseOldOri.Invert();
			Asura_Quat xNewOri(Orientation);
			const Asura_Vector_3& xOldPos = pxFrom->Position;
			const Asura_Vector_3& xNewPos = Position;
			for (u_int u=0; u<m_pxVentEntity->m_uNumExitPoints; ++u)
			{
				Asura_Vector_3& xPos = m_pxVentEntity->m_pxExitPoints[u].m_xPosition;
				Asura_Quat& xOri = m_pxVentEntity->m_pxExitPoints[u].m_xOrientation;
				xPos -= xOldPos;
				xPos = xInverseOldOri.RotateVector(xPos);
				xPos = xNewOri.RotateVector(xPos);
				xPos += xNewPos;

				xOri = xInverseOldOri * xOri;
				xOri = xNewOri * xOri;
			}
		}

		s_bRefreshOverlay = true;
	}
}

Tools_Object* ToolsAVP_Entity_Vent::CreateObjectOfThisType() const
{
	const Asura_Guid uGuid = Asura_Entity_System::GetCreatedByToolsGuid();

	// Set up tools data and auto-create a unique name (prompt for new name if there is a clash).
	Tools_Entity_Data *pxToolsData = CreateToolsDataWithDefaultName("Vent", "Vent", uGuid);

	if(pxToolsData == NULL)
	{
		Asura_Assert(false, "ToolsAVP_Entity_Vent::Create() - no tools data");
	}
	else
	{
		AVP_ServerEntity_Vent* pxServerObject = new AVP_ServerEntity_Vent(uGuid);
		pxServerObject->ToolsData = pxToolsData;
		ToolsAVP_Entity_Vent *pxToolsObject = new ToolsAVP_Entity_Vent(pxServerObject);
		Tools_Object_System::SelectObject(pxToolsObject);

		pxToolsObject->Position		= *Asura_Camera::GetPosition();
		pxToolsObject->Orientation.Identity();

		Tools_SaveAsura::SetEntityModified();
		return pxToolsObject;
	}
	return NULL;
}

void ToolsAVP_Entity_Vent::OnSelection()
{
	s_pxSelectedVent = this;

	// If we were pointing at one of the exit points, make that the current one.
	if ((m_iLOSHitExitPoint >= 0) && (m_iLOSHitExitPoint < static_cast<int>(m_pxVentEntity->m_uNumExitPoints)))
	{
		m_uSelectedExitPoint = static_cast<u_int>(m_iLOSHitExitPoint);
	}

	PARENT::OnSelection();
}

void ToolsAVP_Entity_Vent::OnReselection()
{
	PARENT::OnReselection();

	if ((m_iLOSHitExitPoint >= 0) && (m_iLOSHitExitPoint < static_cast<int>(m_pxVentEntity->m_uNumExitPoints)))
	{
		// If we were pointing at one of the exit points...
		u_int uHitPoint = static_cast<u_int>(m_iLOSHitExitPoint);
		if (uHitPoint != m_uSelectedExitPoint)
		{
			// If it's not our current one, make it the current one.
			m_uSelectedExitPoint = uHitPoint;
		}
		else
		{
			// If it is our current one, start editing it.
			EditExitPoint();
		}
	}
}

void ToolsAVP_Entity_Vent::OnDeselection()
{
	Verify( true );

	PARENT::OnDeselection();

	s_pxSelectedVent = NULL;
}

bool ToolsAVP_Entity_Vent::GetParameter(Asura_Hash_ID uParamID, Tools_Parameter* pxParam) const
{
	switch(uParamID)
	{
		case g_uHash_Vent_StartsDamaged:
		{
			pxParam->SetBool( m_pxVentEntity->m_bIsDamaged );
		}
		break;

		case g_uHash_Vent_UndamagedSkinString:
		{
			if ( m_pxVentEntity->m_uUndamagedSkinHash == ASURA_HASH_ID_UNSET )
			{
				pxParam->SetString( "UNSET" );
			}
			else
			{
				Asura_Hierarchy_Skin* pxSkin = Tools_Animation_Files::GetSkinFromID( m_pxVentEntity->m_uUndamagedSkinHash );
				if ( pxSkin != NULL )
				{
					pxParam->SetString( pxSkin->GetName() );
				}
				else
				{
					pxParam->SetString( "ERROR" );
				}
			}
		}
		break;
	
		case g_uHash_Vent_DamagedSkinString:
		{
			if ( m_pxVentEntity->m_uDamagedSkinHash == ASURA_HASH_ID_UNSET )
			{
				pxParam->SetString( "UNSET" );
			}
			else
			{
				Asura_Hierarchy_Skin* pxSkin = Tools_Animation_Files::GetSkinFromID( m_pxVentEntity->m_uDamagedSkinHash );
				if ( pxSkin != NULL )
				{
					pxParam->SetString( pxSkin->GetName() );
				}
				else
				{
					pxParam->SetString( "ERROR" );
				}
			}
		}
		break;

		case g_uHash_Vent_UndamagedSkinUInt:
		case g_uHash_Vent_DamagedSkinUInt:
			// This is only ever set, not got, because of the way shared function dialogues work.
			break;

		default:
			return PARENT::GetParameter(uParamID,pxParam);
	}

	return true;
}

bool ToolsAVP_Entity_Vent::SetParameter(Asura_Hash_ID uParamID, const Tools_Parameter* pxParam)
{
	switch(uParamID)
	{
		case g_uHash_Vent_StartsDamaged:
		{
			m_pxVentEntity->m_bIsDamaged = pxParam->GetBool();
			if ( m_pxVentEntity->m_bIsDamaged )
			{
				Asura_Hierarchy_Skin* pxSkin = m_pxVentEntity->m_uDamagedSkinHash != ASURA_HASH_ID_UNSET ? Tools_Animation_Files::GetSkinFromID( m_pxVentEntity->m_uDamagedSkinHash ) : NULL;
				if( pxSkin )
				{
					SetSkinAndAnim( pxSkin, NULL );
				}
				else
				{
					SetMesh( "ToolsShape_Vent" );
				}
			}
			else
			{
				Asura_Hierarchy_Skin* pxSkin = m_pxVentEntity->m_uUndamagedSkinHash != ASURA_HASH_ID_UNSET ? Tools_Animation_Files::GetSkinFromID( m_pxVentEntity->m_uUndamagedSkinHash ) : NULL;
				if( pxSkin )
				{
					SetSkinAndAnim( pxSkin, NULL );
				}
				else
				{
					SetMesh( "ToolsShape_Vent" );
				}
			}
		}
		break;

		case g_uHash_Vent_UndamagedSkinString:
		case g_uHash_Vent_DamagedSkinString:
			// This is only ever got, not set, because of the way shared function dialogues work.
			break;

		case g_uHash_Vent_UndamagedSkinUInt:
		{
			m_pxVentEntity->m_uUndamagedSkinHash = pxParam->GetUInt();
			if ( !m_pxVentEntity->m_bIsDamaged )
			{
				Asura_Hierarchy_Skin* pxSkin = m_pxVentEntity->m_uUndamagedSkinHash != ASURA_HASH_ID_UNSET ? Tools_Animation_Files::GetSkinFromID( m_pxVentEntity->m_uUndamagedSkinHash ) : NULL;
				if( pxSkin )
				{
					SetSkinAndAnim( pxSkin, NULL );
				}
				else
				{
					SetMesh( "ToolsShape_Vent" );
				}
			}
			break;
		}

		case g_uHash_Vent_DamagedSkinUInt:
		{
			m_pxVentEntity->m_uDamagedSkinHash = pxParam->GetUInt();
			if ( m_pxVentEntity->m_bIsDamaged )
			{
				Asura_Hierarchy_Skin* pxSkin = m_pxVentEntity->m_uDamagedSkinHash != ASURA_HASH_ID_UNSET ? Tools_Animation_Files::GetSkinFromID( m_pxVentEntity->m_uDamagedSkinHash ) : NULL;
				if( pxSkin )
				{
					SetSkinAndAnim( pxSkin, NULL );
				}
				else
				{
					SetMesh( "ToolsShape_Vent" );
				}
			}
			break;
		}

		default:
			return PARENT::SetParameter(uParamID,pxParam);
	}

	SetEntityModified();

	return true;
}

void ToolsAVP_Entity_Vent::Update()
{
	if (m_uPositionEditObjectID != 0)
	{
		Tools_PositionEdit_Object* pxObject = Tools_PositionEdit_Object::Find(m_uPositionEditObjectID);
		if (!pxObject || (Tools_Object_System::SelectedObject != pxObject))
		{
			if (pxObject)
			{
				// Object selection change, make sure the edit object gets destroyed.
				pxObject->SetAsDestroyed();
			} // Otherwise, the object must have been deleted, just go to the reselection.
			if (Tools_Object_System::SelectedObject == NULL)
			{
				Tools_Object_System::SelectObject(this);
			}
			m_uPositionEditObjectID = 0;
		}
	}

	UpdateVentNodeList();
	PARENT::Update();
}

void ToolsAVP_Entity_Vent::UpdateWhileSelected()
{
	if ( s_bNewVentNode )
	{
		AddVentNode( s_uNewVentNode );
		s_pxSelectedVent->SetEntityModified();
		s_bNewVentNode = false;
		s_bRefreshOverlay = true;
	}

	if ( s_bRefreshOverlay )
	{
		DeleteOverlay();
		CreateOverlay();
		s_bRefreshOverlay = false;
	}

	PARENT::UpdateWhileSelected();
}

void ToolsAVP_Entity_Vent::AddOverlayControls( Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset )
{
	PARENT::AddOverlayControls( pxOverlay, xOffset );

	xOffset.y += Asura_Dialog::DefaultSpacing();

	// Vent Node List
	UpdateVentNodeListStrings();

	Asura_Dialog* pxDialog;

	pxDialog = new Asura_Function_Dialog("Undamaged Skin: ", &g_xParam_Vent_UndamagedSkinString, xOffset, SetUndamagedSkin );
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Damaged Skin: ", &g_xParam_Vent_DamagedSkinString, xOffset, SetDamagedSkin );
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_SharedTickbox_Dialog("Starts Damaged", xOffset, &g_xParam_Vent_StartsDamaged );
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Current Player Exit Point", xOffset, &m_uSelectedExitPoint, 0, m_pxVentEntity->m_uNumExitPoints-1, 1, "%d");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Add Player Exit Point", xOffset, AddExitPoint);
	pxOverlay->AddDialog(pxDialog);
	pxDialog->SetEnabled(m_pxVentEntity->m_uNumExitPoints < AVP_Message_Vent_ClientCreate::MAX_EXIT_POINTS);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Edit Current Player Exit Point", xOffset, EditExitPoint);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Remove Current Player Exit Point", xOffset, RemoveExitPoint);
	pxOverlay->AddDialog(pxDialog);
	pxDialog->SetEnabled(m_pxVentEntity->m_uNumExitPoints > 1);
	xOffset.y += Asura_Dialog::LargeSpacing();

	pxDialog = new Asura_StringDisplay_Dialog("Vent Nodes", xOffset);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_List_Dialog(xOffset, s_pszVentNode, &s_iSelectedVentNode, Asura_Maths::ClampToRange< int >( m_pxVentEntity->m_uNumVentNodes, 1, 4 ), Asura_Maths::Max< int >( m_pxVentEntity->m_uNumVentNodes, 1 ), NULL);
	pxDialog->SetHelpText("Choose Vent (Gamescene) Node");
	pxOverlay->AddDialog(pxDialog);
	pxDialog->SetEnabled( ( s_pxSelectedVent->m_pxVentEntity->m_uNumVentNodes > 0 ) );
	xOffset.y = pxDialog->m_xMaxCoord.y + Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Add Vent Node", xOffset, AddVentNode);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Remove Selected Vent Node", xOffset, RemoveVentNode);
	pxOverlay->AddDialog(pxDialog);
	pxDialog->SetEnabled( ( s_pxSelectedVent->m_pxVentEntity->m_uNumVentNodes > 0 ) );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Edit Selected Vent Node", xOffset, EditVentNode);
	pxOverlay->AddDialog(pxDialog);
	pxDialog->SetEnabled( ( s_pxSelectedVent->m_pxVentEntity->m_uNumVentNodes > 0 ) );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Focus On Selected Vent Node", xOffset, FocusVentNode);
	pxOverlay->AddDialog(pxDialog);
	pxDialog->SetEnabled( ( s_pxSelectedVent->m_pxVentEntity->m_uNumVentNodes > 0 ) );
	xOffset.y += Asura_Dialog::LargeSpacing();

	pxDialog = new Asura_Function_Dialog("Verify Vent", xOffset, VerifyVent);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();

	pxDialog = new Asura_Tickbox_Dialog("Move Player Exit Points With Object", xOffset, &s_bMoveExitPointsWithEntity);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();
}

void ToolsAVP_Entity_Vent::AddFilesToExportList( List<Asura_String>& xNameList ) const
{
	PARENT::AddFilesToExportList(xNameList);

	if ( m_pxVentEntity->m_uUndamagedSkinHash != ASURA_HASH_ID_UNSET )
	{
		const char* szFileName = Asura_HashedFilename_System::GetFullFilename(m_pxVentEntity->m_uUndamagedSkinHash, false);
		if (szFileName)
		{
			Tools_Level_Export::AddFileToExportList(xNameList, Asura_String(szFileName), true);
		}
		else
		{
			Asura_Assertf(false, ("Failed to find undamaged skin with hash ID 0x%X, used by vent %u ('%s').",
				m_pxVentEntity->m_uUndamagedSkinHash, m_uGuid, Entity->ToolsData->Name.GetString()));
		}
	}

	if ( m_pxVentEntity->m_uDamagedSkinHash != ASURA_HASH_ID_UNSET )
	{
		const char* szFileName = Asura_HashedFilename_System::GetFullFilename(m_pxVentEntity->m_uDamagedSkinHash, false);
		if (szFileName)
		{
			Tools_Level_Export::AddFileToExportList(xNameList, Asura_String(szFileName), true);
		}
		else
		{
			Asura_Assertf(false, ("Failed to find undamaged skin with hash ID 0x%X, used by vent %u ('%s').",
				m_pxVentEntity->m_uDamagedSkinHash, m_uGuid, Entity->ToolsData->Name.GetString()));
		}
	}
}

void ToolsAVP_Entity_Vent::Render()
{
	PARENT::Render();

	const bool bRenderSelectedExitPoint = RenderAsSelected();
	const bool bEditingExitPoint = (m_uPositionEditObjectID != 0);

	for (u_int u=0; u<m_pxVentEntity->m_uNumExitPoints; ++u)
	{
		const bool bIsSelectedPoint = (m_uSelectedExitPoint == u);

		Asura_Vector_3 xPosition = m_pxVentEntity->m_pxExitPoints[u].m_xPosition; // Have to copy the vector as the render function later wants a non-const pointer.
		Asura_Matrix_3x3 xOrientation;
		m_pxVentEntity->m_pxExitPoints[u].m_xOrientation.ToMatrix(xOrientation);

		const float fLineRadius = (Position - xPosition).Magnitude() * 0.5f;
		Asura_Vector_3 xLineCentre((Position + xPosition) * 0.5f);

		if (Asura_Camera::IsSphereInFrustum(xLineCentre, fLineRadius))
		{
			Asura_Render::RenderLine( Position, xPosition, (bRenderSelectedExitPoint && bIsSelectedPoint) ? 0xff66ff66 : 0xff6666ff );
		}

		if (m_pxExitPointMesh && (!bEditingExitPoint || !bIsSelectedPoint))
		{
			if(!m_pxExitPointMesh->BoundingBoxDataValid) { m_pxExitPointMesh->CalculateBoundingBox(); }

			Asura_Mesh* pxMeshToUse = m_pxExitPointMesh;
			if (m_pxExitPointMesh->m_pxNextLowerMesh)
			{
				pxMeshToUse = m_pxExitPointMesh->GetDetailMeshFromDistance((*Asura_Camera::GetPosition() - m_pxVentEntity->m_pxExitPoints[u].m_xPosition).Magnitude());
			}

			//check that the shape is on screen.
			const Asura_Vector_3& xScale = GetScale();
			Asura_Bounding_Box xScaledBB = m_pxExitPointMesh->BoundingBox;
			if(IsScaled())
			{
				xScaledBB.MaxX *= xScale.x;
				xScaledBB.MinX *= xScale.x;
				xScaledBB.MaxY *= xScale.y;
				xScaledBB.MinY *= xScale.y;
				xScaledBB.MaxZ *= xScale.z;
				xScaledBB.MinZ *= xScale.z;
			}

			float fRadius = xScaledBB.CalculateRadius();

			Asura_Vector_3 xCentre(xPosition + (xScaledBB.CalculateCentre() * xOrientation));
			if (!Asura_Camera::IsSphereInFrustum(xCentre, fRadius)) { continue; }

			Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_NORMAL);

			Asura_Direct3D_ObjectRenderer::ToolsRender(pxMeshToUse, xPosition, xOrientation, false, 1.0f, xScale, -1, 0xffffff);

			if (bRenderSelectedExitPoint && bIsSelectedPoint)
			{
				Tools_Render::RenderMeshOutline(pxMeshToUse, &xPosition, &xOrientation, 0xff00ff00, true, xScale);
			}
		}
	}
}

void ToolsAVP_Entity_Vent::UpdateVentNodeList()
{
	Asura_Guid auKillList[ AVP_MAX_VENT_NODES ];
	u_int uNumKill = 0;
	for ( u_int uVentItr = 0; uVentItr < m_pxVentEntity->m_uNumVentNodes; ++uVentItr )
	{
		const Asura_ServerNode_GS2Node* pxVentNode = Asura_ServerNode_GS2Node::FindEntity( m_pxVentEntity->m_auVentNode[ uVentItr ] );
		if ( pxVentNode == NULL )
		{
			auKillList[ uNumKill++ ] = m_pxVentEntity->m_auVentNode[ uVentItr ];
		}
	}

	for ( u_int uKillItr = 0; uKillItr < uNumKill; ++uKillItr )
	{
		RemoveVentNode( auKillList[ uKillItr ] );
		s_bRefreshOverlay = true;
	}
}

void ToolsAVP_Entity_Vent::UpdateVentNodeListStrings()
{
	// reset our static array information
	if ( s_uNumVentNodes > 0 )
	{
		for ( u_int uItr = 0; uItr < s_uNumVentNodes; ++uItr )
		{
			delete [] s_pszVentNode[ uItr ];
		}
	}
	delete [] s_pszVentNode;
	s_pszVentNode = NULL;
	s_uNumVentNodes = m_pxVentEntity->m_uNumVentNodes;
	if ( s_uNumVentNodes > 0 )
	{
		s_pszVentNode = new char*[ s_uNumVentNodes ];

		for ( u_int uItr = 0; uItr < s_uNumVentNodes; ++uItr )
		{
			Asura_String xString;
			Asura_StringUtil::ToString( m_pxVentEntity->m_auVentNode[ uItr ], xString );
			s_pszVentNode[ uItr ] = new char[ xString.GetLength() + 1 ];
			Asura_PrintfToBfr( s_pszVentNode[ uItr ], "%s", xString.GetLength() + 1, true, xString.GetString() );
		}
	}
	else
	{
		s_pszVentNode = new char*[1];
		s_pszVentNode[0] = "No Nodes";
	}
}

void ToolsAVP_Entity_Vent::AddVentNode( const Asura_Guid uNode )
{
	if ( uNode != ASURA_GUID_UNREGISTERED )
	{
		Asura_ServerNode_GS2Node* pxNewNode = Asura_ServerNode_GS2Node::FindEntity( uNode );
		Asura_Assert( pxNewNode != NULL, "Not a valid node to add" );

		if ( m_pxVentEntity->m_uNumVentNodes >= AVP_MAX_VENT_NODES )
		{
			Tools_Timed_Message_System::AddMessage( 0xffff00,"Warning: Can only have a maximum of %d vent nodes", AVP_MAX_VENT_NODES );
			return;
		}

		bool bInList = false;

		for ( u_int uItr = 0; uItr < m_pxVentEntity->m_uNumVentNodes; ++uItr )
		{
			bInList |= ( m_pxVentEntity->m_auVentNode[ uItr ] == uNode );
		}

		if ( bInList )
		{
			Tools_Timed_Message_System::AddMessage( 0xffff00,"Warning: Node %d is already in the list", uNode );
			return;
		}

		m_pxVentEntity->m_auVentNode[ m_pxVentEntity->m_uNumVentNodes ] = uNode;
		++(m_pxVentEntity->m_uNumVentNodes);
	}
}

void ToolsAVP_Entity_Vent::RemoveVentNode( const Asura_Guid uNodeGuid )
{
	bool bFound = false;
	for ( u_int uItr = 0; uItr < m_pxVentEntity->m_uNumVentNodes; ++uItr )
	{
		if ( bFound )
		{
			m_pxVentEntity->m_auVentNode[ uItr - 1 ] = m_pxVentEntity->m_auVentNode[ uItr ];
		}
		// this is in this order so that we only do stuff next time around the loop
		if ( m_pxVentEntity->m_auVentNode[ uItr ] == uNodeGuid )
		{
			bFound = true;
		}
	}
	if ( bFound )
	{
		--(m_pxVentEntity->m_uNumVentNodes);
	}
}

void ToolsAVP_Entity_Vent::AddExitPoint()
{
	Asura_Assert(s_pxSelectedVent, "No vent selected, should be impossible.");
	if (!s_pxSelectedVent) { return; }

	Asura_Vector_3 xPos = *Asura_Camera::GetPosition();
	Asura_Quat xOri(*Asura_Camera::GetOrientation());
	s_pxSelectedVent->m_pxVentEntity->CreateExitPoint(xPos, xOri);

	s_pxSelectedVent->SetEntityModified();

	// Select the new point automatically.
	s_pxSelectedVent->m_uSelectedExitPoint = s_pxSelectedVent->m_pxVentEntity->m_uNumExitPoints - 1;

	// And immediately start editing it.
	EditExitPoint();
}

void ToolsAVP_Entity_Vent::RemoveExitPoint()
{
	Asura_Assert(s_pxSelectedVent, "No vent selected, should be impossible.");
	if (!s_pxSelectedVent) { return; }

	s_pxSelectedVent->m_pxVentEntity->DeleteExitPoint(s_pxSelectedVent->m_uSelectedExitPoint);
	// Make sure our selected point is within the new range.
	s_pxSelectedVent->m_uSelectedExitPoint = Asura_Maths::Min(s_pxSelectedVent->m_uSelectedExitPoint, s_pxSelectedVent->m_pxVentEntity->m_uNumExitPoints - 1);

	s_pxSelectedVent->SetEntityModified();
	s_bRefreshOverlay = true;
}

void ToolsAVP_Entity_Vent::EditExitPoint()
{
	Asura_Assert(s_pxSelectedVent, "No vent selected, should be impossible.");
	if (!s_pxSelectedVent) { return; }

	const u_int uSelectedPoint = s_pxSelectedVent->m_uSelectedExitPoint;
	AVP_ServerEntity_Vent::ExitPoint& xExitPoint = s_pxSelectedVent->m_pxVentEntity->m_pxExitPoints[uSelectedPoint];
	Asura_Vector_3& xPos = xExitPoint.m_xPosition;
	Tools_PositionEdit_Object* pxObject = new Tools_PositionEdit_Object(&xPos, NULL, "Exit point", "Edit position and orientation of player exit point.", &s_pxSelectedVent->Changed);
	if (pxObject)
	{
		s_pxSelectedVent->m_uPositionEditObjectID = pxObject->GetID();
		pxObject->SetQuatToEdit(&xExitPoint.m_xOrientation);
		Tools_Object_System::SelectObject(pxObject);
	}
}

bool ToolsAVP_Entity_Vent::Verify( const bool bAutomatic )
{
	const bool bParentGood = PARENT::Verify( bAutomatic );

	bool bGood = true;
	if ( m_pxVentEntity->m_uNumVentNodes > AVP_MAX_VENT_NODES )
	{
		Tools_MessageLog::AddMessage( Tools_MessageLog::VERIFICATION, "Vents", new Tools_MessageLogAction_GotoEntity( GetGuid() ),
			"Vent %d has too many nodes, should be %d max", GetGuid(), AVP_MAX_VENT_NODES );
		bGood = false;
	}

	for ( u_int uItr = 0; uItr < m_pxVentEntity->m_uNumVentNodes; ++uItr )
	{
		const Asura_ServerNode_GS2Node* pxNode = Asura_ServerNode_GS2Node::FindEntity( m_pxVentEntity->m_auVentNode[ uItr ] );
		if ( pxNode )
		{
			if ( pxNode->GetNumberLinks() != 1 )
			{
				Tools_MessageLog::AddMessage( Tools_MessageLog::VERIFICATION, "Vents", new Tools_MessageLogAction_GotoEntity( GetGuid() ),
					"Vent node %d should have one exit spline", m_pxVentEntity->m_auVentNode[ uItr ] );
				bGood = false;
			}

			if ( pxNode->GetNumberLinks() == 1 )
			{
				const Asura_ServerNode_GS2Spline* pxLink = Asura_ServerNode_GS2Spline::FindEntity( pxNode->GetLink( 0 ) );
				if ( pxLink )
				{
					const Asura_ServerNode_GS2Node* pxExitNode = Asura_ServerNode_GS2Node::FindEntity( pxLink->GetTargetGuid() );
					if ( !pxExitNode )
					{
						Tools_MessageLog::AddMessage( Tools_MessageLog::VERIFICATION, "Vents", new Tools_MessageLogAction_GotoEntity( GetGuid() ),
							"Can't find vent %d node %d exit node with guid %d", GetGuid(), pxNode->GetGuid(), pxLink->GetTargetGuid() );
						bGood  = false;
					}
				}
				else
				{
					Tools_MessageLog::AddMessage( Tools_MessageLog::VERIFICATION, "Vents", new Tools_MessageLogAction_GotoEntity( GetGuid() ),
						"Can't find vent %d node %d exit spline with guid %d", GetGuid(), pxNode->GetGuid(), pxNode->GetLink( 0 ) );
					bGood  = false;
				}
			}

			// check for entry nodes
			u_int uNumStartNodes = 0;
			for ( Asura_Hashed_List_ConstIt< Asura_ServerNode_GS2Node > xIt( Asura_ServerNode_GS2Node::GetList() ); !xIt.Done(); xIt.Next() )
			{
				if ( AVP_ServerEntity_Vent::AreNodesConnected( xIt.GetCurrent()->GetGuid(), m_pxVentEntity->m_auVentNode[ uItr ] ) )
				{
					++uNumStartNodes;
				}
			}
			if ( uNumStartNodes == 0 )
			{
				Tools_MessageLog::AddMessage( Tools_MessageLog::VERIFICATION, "Vents", new Tools_MessageLogAction_GotoEntity( GetGuid() ),
					"Vent node %d has no entry nodes", m_pxVentEntity->m_auVentNode[ uItr ] );
				bGood  = false;
			}
		}
		else
		{
			Tools_MessageLog::AddMessage( Tools_MessageLog::VERIFICATION, "Vents", new Tools_MessageLogAction_GotoEntity( GetGuid() ),
				"Can't find vent %d gamescene node with guid %d", GetGuid(), m_pxVentEntity->m_auVentNode[ uItr ] );
			bGood  = false;
		}
	}

	return ( bParentGood && bGood );
}

void ToolsAVP_Entity_Vent::SetUndamagedSkin()
{
	Asura_String xName;
	Asura_String xFilename;
	Asura_Hash_ID uHash = ASURA_HASH_ID_UNSET;
	if ( Tools_Animation_Files::ChooseSkin( &xName, &xFilename ) )
	{
		uHash = xName.GetHash();
	}

	// Setting this will cause SetParameter to be called with this parameter hash on all selected objects.
	g_xParam_Vent_UndamagedSkinUInt.SetValue( uHash );
	Asura_Texture_Management::LoadTextures();
}

void ToolsAVP_Entity_Vent::SetDamagedSkin()
{
	Asura_String xName;
	Asura_String xFilename;
	Asura_Hash_ID uHash = ASURA_HASH_ID_UNSET;
	if ( Tools_Animation_Files::ChooseSkin( &xName, &xFilename ) )
	{
		uHash = xName.GetHash();
	}

	// Setting this will cause SetParameter to be called with this parameter hash on all selected objects.
	g_xParam_Vent_DamagedSkinUInt.SetValue( uHash );
	Asura_Texture_Management::LoadTextures();
}

void ToolsAVP_Entity_Vent::AddVentNode()
{
	Tools_ChooseObject_GS2Node* pxChooseObject = new Tools_ChooseObject_GS2Node( &s_uNewVentNode, &s_bNewVentNode );
	if ( pxChooseObject )
	{
		Tools_Object_System::ChooseObject( "Choose The Gamescene Node To Associate With This Vent", pxChooseObject );
	}
}

void ToolsAVP_Entity_Vent::RemoveVentNode()
{
	u_int uNodeIndex = static_cast< u_int >( s_iSelectedVentNode );
	Asura_Assert( uNodeIndex < s_pxSelectedVent->m_pxVentEntity->m_uNumVentNodes, "" );
	if ( uNodeIndex < s_pxSelectedVent->m_pxVentEntity->m_uNumVentNodes )
	{
		s_pxSelectedVent->RemoveVentNode( s_pxSelectedVent->m_pxVentEntity->m_auVentNode[ uNodeIndex ] );
		s_pxSelectedVent->SetEntityModified();
		s_bRefreshOverlay = true;
	}
}

void ToolsAVP_Entity_Vent::EditVentNode()
{
	Tools_EntityNode_GS2Node* pxNode = Tools_EntityNode_GS2Node::FindEntity( s_pxSelectedVent->m_pxVentEntity->m_auVentNode[ s_iSelectedVentNode ] );
	Tools_Object_System::SelectObject( pxNode );
}

void ToolsAVP_Entity_Vent::FocusVentNode()
{
	const Tools_EntityNode_GS2Node* pxNode = Tools_EntityNode_GS2Node::FindEntity( s_pxSelectedVent->m_pxVentEntity->m_auVentNode[ s_iSelectedVentNode ] );

	Asura_Vector_3		xNewCameraPos	= pxNode->Position;
	Asura_Matrix_3x3	xNewCameraOri	= *Asura_Camera::GetOrientation();
	Asura_Vector_3		xFwdVec;

	Asura_Camera::GetForwardVector(xFwdVec);
	xNewCameraPos -= xFwdVec * 2.0f;

	Tools_Movement::TeleportCameraToPosition(&xNewCameraPos, &xNewCameraOri);
}

void ToolsAVP_Entity_Vent::VerifyVent()
{
	if ( s_pxSelectedVent->Verify( false ) )
	{
		Message_Overlay xMessage( "No errors found." );
		xMessage.DoModal();
	}
	else
	{
		Tools_MessageLog::ShowOverlay( Tools_MessageLog::VERIFICATION, NULL );
	}
}
