//
// ToolsAVP_EntityTemplate_System.cpp
//
// Created by Richard Rice - 08/05/2009
// 
// Copyright (C) 2009 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"
#include "EntityTemplate/ToolsAVP_EntityTemplate_System.h"

//asura
#include "Entity/Asura_StaticMessage_System.h"

//project
#include "Camera/AVP_CameraShake_System.h"
#include "Entity/AVP_Messages.h"
#include "Entity/Controller/AVP_Controller_Messages.h"
#include "Entity/ObjectiveSystem/PlayerObjective/AVP_PlayerObjective_Messages.h"
#include "Entity/PhysicalObject/Actor/AVP_Actor_Messages.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/AlienQueen/AVP_AlienQueen_Messages.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/FaceHugger/AVP_Messages_FaceHugger.h"
#include "Entity/PhysicalObject/AVP_PhysicalObject_Messages.h"
#include "Entity/PlayerInfo/AVP_PlayerInfo_Messages.h"
#include "Entity/Spawners/AVP_SpawnerInterface.h"


void ToolsAVP_EntityTemplate_System::ConvertStaticMessage(Asura_StaticMessage* pxMessage, const Tools_EntityTemplateInstance* pxInstance) const
{
	switch(pxMessage->m_usMsgID)
	{

		case AVP_MSGID_CHUNK:
		case AVP_MSGID_CHANGE_ALIEN_SPAWN_SETTINGS:
		case AVP_MSGID_ADD_ALIENS_TO_SPAWN_QUEUE:
			break;

		case AVP_MSGID_ADD_AI_OBJECTIVE:
		{
			AVP_Message_AddObjective* pxMsgData = static_cast<AVP_Message_AddObjective*>(pxMessage->m_pData);
			pxMsgData->m_uObjectiveGuid = pxInstance->ConvertGuid(pxMsgData->m_uObjectiveGuid);
			break;
		}
		
		case AVP_MSGID_CHANGE_HUMANOID_SPAWN_SETTINGS:
			break;


		case AVP_MSGID_SPAWN_HUMANOIDS:
		{
			AVP_Message_SpawnHumanoids* pxMsgData = static_cast<AVP_Message_SpawnHumanoids*>(pxMessage->m_pData);
			pxMsgData->m_uSquadControllerGuid = pxInstance->ConvertGuid(pxMsgData->m_uSquadControllerGuid);
			pxMsgData->m_uGamePlayVolumeGuid = pxInstance->ConvertGuid(pxMsgData->m_uGamePlayVolumeGuid);
			pxMsgData->m_uSpawnPointGuid = pxInstance->ConvertGuid(pxMsgData->m_uSpawnPointGuid);
			break;
		}

		case AVP_MSGID_MERGE_HUMANOID_SQUADS:
		{
			AVP_Message_MergeSquads* pxMsgData = static_cast<AVP_Message_MergeSquads*>(pxMessage->m_pData);
			pxMsgData->m_uSquadToMergeInto = pxInstance->ConvertGuid(pxMsgData->m_uSquadToMergeInto);
			break;
		}

		case AVP_MSGID_SPLIT_HUMANOID_SQUAD:
		{
			AVP_Message_SplitSquad* pxMsgData = static_cast<AVP_Message_SplitSquad*>(pxMessage->m_pData);
			pxMsgData->m_uSquadToAddTo = pxInstance->ConvertGuid(pxMsgData->m_uSquadToAddTo);
			for (u_int u = 0; u < AVP_Message_SplitSquad::MAX_SQUAD_MEMBERS; ++u)
			{
				pxMsgData->m_auSplitMembers[u] = pxInstance->ConvertGuid(pxMsgData->m_auSplitMembers[u]);
			}
			break;
		}

		case AVP_MSGID_ADD_SQUAD_MEMBER :
		{
			AVP_Message_AddSquadMember* pxMsgData = static_cast<AVP_Message_AddSquadMember*>(pxMessage->m_pData);
			pxMsgData->m_uGuidToAdd = pxInstance->ConvertGuid(pxMsgData->m_uGuidToAdd);
			break;
		}

		case AVP_MSGID_REMOVE_SQUAD_MEMBER :
		{
			AVP_Message_RemoveSquadMember* pxMsgData = static_cast<AVP_Message_RemoveSquadMember*>(pxMessage->m_pData);
			pxMsgData->m_uGuidToRemove = pxInstance->ConvertGuid(pxMsgData->m_uGuidToRemove);
			break;
		}

		case AVP_MSGID_REQUEST_HUMANOID_PATROL :
		{
			AVP_Message_RequestPatrol* pxMsgData = static_cast<AVP_Message_RequestPatrol*>(pxMessage->m_pData);
			pxMsgData->m_uGuardZoneGuid = pxInstance->ConvertGuid(pxMsgData->m_uGuardZoneGuid);
			break;
			
		}
		
		case AVP_MSGID_ALIEN_TEAM_SET_BEHAVIOUR:
		case AVP_MSGID_ALIEN_TEAM_ADD_SPAWNTOKEN:
		case AVP_MSGID_ALIEN_TEAM_REMOVE_SPAWNTOKEN:
		case AVP_MSGID_ALIEN_TEAM_THREAT_BONUS:
			break;
	
		case AVP_MSGID_PERFORM_AI_OBJECTIVE_ACTION:
		{
			AVP_Message_PerformObjectiveAction* pxMsgData = static_cast<AVP_Message_PerformObjectiveAction*>(pxMessage->m_pData);
			pxMsgData->m_uActionGuid = pxInstance->ConvertGuid(pxMsgData->m_uActionGuid);
			break;
		}

		case AVP_MSGID_ALIEN_TEAM_PREPARE_ROOM:
        case AVP_MSGID_REQUEST_CLOAK_ON:
        case AVP_MSGID_REQUEST_CLOAK_OFF:
		case AVP_MSGID_REQUEST_TORCH_ON:
		case AVP_MSGID_REQUEST_TORCH_OFF:
		case AVP_MSGID_SET_PREFERRED_TARGET_TYPE:
			break;


		case AVP_MSGID_SPAWN:
		{
			AVP_Msg_Spawn* pxMsgData = static_cast<AVP_Msg_Spawn*>(pxMessage->m_pData);
			pxMsgData->m_uSquadGuid = pxInstance->ConvertGuid(pxMsgData->m_uSquadGuid);
			break;
		}
		
		case AVP_MSGID_SPAWN_SET_MAX_SPAWN_COUNT:
		case AVP_MSGID_SPAWN_RESET_SPAWN_COUNTER:
		case AVP_MSGID_SPAWN_SET_MAX_IN_PLAY:
		case AVP_MSGID_SET_ALIEN_SQUAD_BEHAVIOUR:
			break;

				 
		case AVP_MSGID_SET_AI_TARGET_PRIORITIES:
		{
			AVP_Msg_SetAITargetPriorities* pxMsgData = static_cast<AVP_Msg_SetAITargetPriorities*>(pxMessage->m_pData);
			if(pxMsgData->m_uNumEntries)
			{
				AVP_Msg_SetAITargetPriorities::TPData* pxTPData = 	pxMsgData->GetFirstTPDataPtr();
				for(u_int i=0; i<pxMsgData->m_uNumEntries; i++, pxTPData++)
				{
					pxTPData->m_uSpecificGuid = pxInstance->ConvertGuid(pxTPData->m_uSpecificGuid);
				}
			}
			break;
			
		}

		case AVP_MSGID_CHANGE_MARINE_SQUAD_SETTINGS :
		{
			AVP_Message_ChangeMarineSquadSettings* pxMsgData = static_cast<AVP_Message_ChangeMarineSquadSettings*>(pxMessage->m_pData);
			pxMsgData->m_uCombatZoneGPVGuid = pxInstance->ConvertGuid(pxMsgData->m_uCombatZoneGPVGuid);
			break;
		}

		case AVP_MSGID_ATTACK_SCRIPTED_TARGET:
		{
			AVP_Message_AttackScriptedTarget* pxMsgData = static_cast<AVP_Message_AttackScriptedTarget*>(pxMessage->m_pData);
			pxMsgData->m_uShootAtGuid = pxInstance->ConvertGuid(pxMsgData->m_uShootAtGuid);
			break;
		}

		case AVP_MSGID_PICKUP_SET_INTELLIGENT_AMOUNT:
			break;


		case AVP_MSGID_ADD_PLAYER_OBJECTIVE:
		{
			AVP_Message_AddPlayerObjective* pxMsgData = static_cast<AVP_Message_AddPlayerObjective*>(pxMessage->m_pData);
			pxMsgData->m_uObjectiveGuid = pxInstance->ConvertGuid(pxMsgData->m_uObjectiveGuid);
			break;
		}

		case AVP_MSGID_SET_PLAYER_OBJECTIVE_COMPLETE:
		{
			AVP_Message_PlayerObjectiveComplete* pxMsgData = static_cast<AVP_Message_PlayerObjectiveComplete*>(pxMessage->m_pData);
			pxMsgData->m_uObjectiveGuid = pxInstance->ConvertGuid(pxMsgData->m_uObjectiveGuid);
			break;
		}

		case AVP_MSGID_PLAYER_OBJECTIVE_SET_MARKER_LOCATION :
		{
			AVP_Message_PlayerObjectiveSetMarker* pxMsgData = static_cast<AVP_Message_PlayerObjectiveSetMarker*>(pxMessage->m_pData);
			pxMsgData->m_uMarkerGuid = pxInstance->ConvertGuid(pxMsgData->m_uMarkerGuid);
			break;
		}

		case AVP_MSGID_PLAYER_OBJECTIVE_SET_TARGET_ENTITY:
		{
			AVP_Message_PlayerObjectiveSetEntity* pxMsgData = static_cast<AVP_Message_PlayerObjectiveSetEntity*>(pxMessage->m_pData);
			pxMsgData->m_uEntityGuid = pxInstance->ConvertGuid(pxMsgData->m_uEntityGuid);
			break;
		
		}
		
		case AVP_MSGID_ADD_PLAYER_PICKUP:
		{
			AVP_Message_AddPlayerPickup* pxMsgData = static_cast<AVP_Message_AddPlayerPickup*>(pxMessage->m_pData);
			pxMsgData->m_uPickupGuid = pxInstance->ConvertGuid(pxMsgData->m_uPickupGuid);
			break;
		}

		case AVP_MSGID_SET_DROPSHIP_ATTACK_MODE:
		case AVP_MSGID_CHANGE_HUMANOID_SQUAD_MOVEMENT_MODE:
		case AVP_MSGID_SET_PREDATOR_BEHAVIOUR:
		case AVP_MSGID_ALIENQUEEN_SET_BEHAVIOUR:
			break;


		case AVP_MSGID_LOOK_AT_ENTITY :
		{
			AVP_Message_LookAtEntity* pxMsgData = static_cast<AVP_Message_LookAtEntity*>(pxMessage->m_pData);
			pxMsgData->m_uLookAtGuid = pxInstance->ConvertGuid(pxMsgData->m_uLookAtGuid);
			break;
		}

		case AVP_MSGID_SET_GS2_MOVE_SPEED:
			break;

		case AVP_MSGID_SET_HEALTH:
		{
			AVP_Message_SetHealth* pxMsgData = static_cast<AVP_Message_SetHealth*>(pxMessage->m_pData);
			pxMsgData->m_uDamageFromGuid = pxInstance->ConvertGuid(pxMsgData->m_uDamageFromGuid);
			break;
		}

		case AVP_MSGID_SET_FACEHUGGER_SQUAD_BEHAVIOUR:
		case AVP_MSGID_CHANGE_HUMANOID_KNOWLEDGE_SETTINGS:
		case AVP_MSGID_SET_SENTRYGUN_SWEEP_SPEED:
		case AVP_MSGID_SET_SENTRYGUN_VISION_RANGE:
		case AVP_MSGID_SET_SENTRYGUN_HOR_FOV:
		case AVP_MSGID_SET_SENTRYGUN_VER_FOV:
		case AVP_MSGID_SET_SENTRYGUN_FIRING_ARC:
		case AVP_MSGID_SET_SENTRYGUN_LETHAL_RANGE:
		case AVP_MSGID_SENTRYGUN_DRYFIRE_TOGGLE:
			break;

		case AVP_MSGID_DETACH_WEAPON:
		case AVP_MSGID_CREATE_ATTACHMENT:
		case AVP_MSGID_DESTROY_ATTACHMENT:
		case AVP_MSGID_DETACH_ATTACHMENT:
			break;

		case AVP_MSGID_ATTACH_WEAPON:
			break;

		case AVP_MSGID_ALIENQUEEN_ADD_TO_TARGET_LIST:
		{
			AVP_Msg_Add_Entity_To_Queen_Target_List* pxMsgData = static_cast<AVP_Msg_Add_Entity_To_Queen_Target_List*>(pxMessage->m_pData);
			pxMsgData->m_uQueenGuid = pxInstance->ConvertGuid(pxMsgData->m_uQueenGuid);
			break;
		}
		
		case AVP_MSGID_ALIENQUEEN_REMOVE_FROM_TARGET_LIST:
		{
			AVP_Msg_Remove_Entity_From_Queen_Target_List* pxMsgData = static_cast<AVP_Msg_Remove_Entity_From_Queen_Target_List*>(pxMessage->m_pData);
			pxMsgData->m_uQueenGuid = pxInstance->ConvertGuid(pxMsgData->m_uQueenGuid);
			break;
		}

		case AVP_MSGID_ALIENQUEEN_FORCE_ROAR:
		{
			AVP_Msg_Alien_Queen_Start_Roar_Behaviour* pxMsgData = static_cast<AVP_Msg_Alien_Queen_Start_Roar_Behaviour*>(pxMessage->m_pData);
			pxMsgData->m_uTargetGuid = pxInstance->ConvertGuid(pxMsgData->m_uTargetGuid);
			break;
			
		}

		case AVP_MSGID_START_CAMERA_SHAKE:
		case AVP_MSGID_STOP_CAMERA_SHAKE:
		{
			AVP_Msg_CameraShake* pxMsgData = static_cast<AVP_Msg_CameraShake*>(pxMessage->m_pData);
			pxMsgData->m_uSourceGuid = pxInstance->ConvertGuid(pxMsgData->m_uSourceGuid);
			break;
		}

		case AVP_MSGID_FACEHUGGER_HUG_ENTITY:
		{
			AVP_FaceHugger_Hug_Entity* pxMsgData = static_cast<AVP_FaceHugger_Hug_Entity*>(pxMessage->m_pData);
			pxMsgData->m_uTarget = pxInstance->ConvertGuid(pxMsgData->m_uTarget);
			break;
		}

		case AVP_MSGID_DISMEMBER:
		{
			AVP_Msg_Dismember* pxMsgData = static_cast<AVP_Msg_Dismember*>(pxMessage->m_pData);
			pxMsgData->SetFromGuid( pxInstance->ConvertGuid( pxMsgData->GetFromGuid()));
			break;
		}
		



		default :
			if(pxMessage->m_usDataSize && pxMessage->m_usMsgID > AVP_MSGID_START_OF_STATIC_MESSAGES)
			{
				//assert about unhandled project messages that have data
				Asura_Assertf(0, ("Static message type %d isn't handled by ToolsAVP_EntityTemplate_System::ConvertStaticMessage()\nIf the message data contains any guids , then they may get messed up.",pxMessage->m_usMsgID));
			}

			PARENT::ConvertStaticMessage(pxMessage, pxInstance);
			break;
	}
}



