
///////////////////////////////////////////////////////////////////
// ToolsAVP_SurfaceOverlay.h
// Created by Chris Wallace - 31/07/2007
// Copyright (C) 2007 Rebellion, All Rights Reserved.
///////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef __TOOLS_SURFACEOVERLAY_H__
#define __TOOLS_SURFACEOVERLAY_H__

#include "Blueprint/Asura_Blueprint.h"
#include "Blueprint/Asura_Blueprint_System.h"
#include "Interface/Asura_Table_Dialog.h"
#include "Interface/Asura_Overlay.h"
#include "SpecialFX/Asura_Decal_Manager.h"

#include "Core/AVP_SurfaceEffects.h"
#include "Blueprint/AVP_Blueprint_HashIDs.h"

#include "Misc/Tools_Modal_Overlay.h"


//////////////////////////////////////////////////////////////////////////
// ToolsAVP_SurfaceOverlay
//////////////////////////////////////////////////////////////////////////

class ToolsAVP_SurfaceOverlay_FilterData
{
public:
	// Public functions...
	ToolsAVP_SurfaceOverlay_FilterData(const Asura_Hash_ID uTypeHash,
									   const Asura_Hash_ID uFilterHash,
									   const u_int uNumSelectableBlueprintTypes,
									   const Asura_Hash_ID* puSelectableBlueprintTypes,
									   const u_int uNumSelectableBlueprintParents,
									   const Asura_Hash_ID* puSelectableBlueprintParents)
	: m_uTypeHash(uTypeHash)
	, m_uFilterHash(uFilterHash)
	, m_uMaxNames(0)
	, m_uNumNamesAdded(0)
	, m_pszNames(NULL)
	, m_uNumSelectableBlueprintTypes(uNumSelectableBlueprintTypes)
	, m_puSelectableBlueprintTypes(puSelectableBlueprintTypes)
	, m_uNumSelectableBlueprintParents(uNumSelectableBlueprintParents)
	, m_puSelectableBlueprintParents(puSelectableBlueprintParents)
	{
	}

	~ToolsAVP_SurfaceOverlay_FilterData()
	{
		delete[](m_pszNames);
	}

	void Allocate(const u_int uNumNames)
	{
		delete[](m_pszNames);
		m_pszNames			= new const char* [uNumNames];
		m_uMaxNames			= uNumNames;
		m_uNumNamesAdded	= 0;
	}

	void Deallocate(void)
	{
		delete[](m_pszNames);
		m_pszNames			= NULL;
		m_uMaxNames			= 0;
		m_uNumNamesAdded	= 0;
	}

	const Asura_Blueprint* RequestBlueprint(const char* szRequest);

	// Accessor functions...
	Asura_Hash_ID	GetFilterHash(void) const				{ return(m_uFilterHash); }
	Asura_Hash_ID	GetTypeHash(void) const					{ return(m_uTypeHash); }

	void			AddName(const char* szName)				{ m_pszNames[m_uNumNamesAdded++] = szName; }
	void			AddName(const Asura_String& xName)		{ m_pszNames[m_uNumNamesAdded++] = xName.GetString(); }
	const char*		GetName(const u_int uIndex) const		{ return(m_pszNames[uIndex]); }
	Asura_Hash_ID	GetNameHash(const u_int uIndex) const	{ return(Asura_GetHashID(m_pszNames[uIndex])); }
	u_int			GetMaxNames(void) const					{ return(m_uMaxNames); }
	u_int			GetNumNamesAdded(void) const			{ return(m_uNumNamesAdded); }
	const char**	GetNamesArray(void) const				{ return(m_pszNames); }

	const Asura_Blueprint*	GetFilterBlueprint(void) const	{ return(Asura_Blueprint_System::GetBlueprintByHashID(m_uTypeHash, m_uFilterHash)); }


protected:
	// Protected variables...

	// Protected functions...


private:
	// Private variables...
	Asura_Hash_ID			m_uTypeHash;
	Asura_Hash_ID			m_uFilterHash;

	u_int					m_uMaxNames;
	u_int					m_uNumNamesAdded;
	const char**			m_pszNames;

	const u_int				m_uNumSelectableBlueprintTypes;
	const Asura_Hash_ID*	m_puSelectableBlueprintTypes;
	const u_int				m_uNumSelectableBlueprintParents;
	const Asura_Hash_ID*	m_puSelectableBlueprintParents;

	// Private functions...
	ToolsAVP_SurfaceOverlay_FilterData(void);
	ToolsAVP_SurfaceOverlay_FilterData(const ToolsAVP_SurfaceOverlay_FilterData& xOther);

	ToolsAVP_SurfaceOverlay_FilterData&	operator=(const ToolsAVP_SurfaceOverlay_FilterData& xOther);

};   // End of class 'ToolsAVP_SurfaceOverlay_FilterData'.


class ToolsAVP_SurfaceOverlay : public Asura_Overlay
{
public :
	virtual void PerFrameUpdate();
	
	static void Initialise();
	static void Shutdown();

	static void ToggleSurfaceOverlay();
	static bool SurfaceOverlayIsOn();

	static void SetAsChanged(void);

	//mainly load/saves position of overlay
	static void LoadSettingsFromIniFile();
	static void SaveSettingsToIniFile();

	static u_int				GetNumEffectTypes()									{ return s_xEffectTypes.GetNumTypes(); }
	static const char* const*	GetEffectTypeNameArray()							{ return s_xEffectTypes.GetTypeNameArray(); }
	static Asura_Hash_ID		GetEffectTypeHashFromIndex( u_int uIndex )			{ return s_xEffectTypes.GetTypeHashFromIndex(uIndex);}

	static u_int				GetNumDamageTypes()									{ return s_xDamageTypes.GetNumTypes(); }
	static const char* const*	GetDamageTypeNameArray()							{ return s_xDamageTypes.GetTypeNameArray(); }
	static Asura_Hash_ID		GetDamageTypeHashFromIndex( u_int uIndex )			{ return s_xDamageTypes.GetTypeHashFromIndex(uIndex);}

	static u_int				GetNumSurfaceTypes()								{ return s_xSurfaceTypes.GetNumTypes(); }
	static const char* const*	GetSurfaceTypeNameArray()							{ return s_xSurfaceTypes.GetTypeNameArray(); }
	static Asura_Hash_ID		GetSurfaceTypeHashFromIndex( u_int uIndex )			{ return s_xSurfaceTypes.GetTypeHashFromIndex(uIndex);}

	static u_int				GetNumMaterialSurfaceTypes()						{ return s_xMaterialSurfaceTypes.GetNumTypes(); }
	static const char* const*	GetMaterialSurfaceTypeNameArray()					{ return s_xMaterialSurfaceTypes.GetTypeNameArray(); }
	static Asura_Hash_ID		GetMaterialSurfaceTypeHashFromIndex( u_int uIndex )	{ return s_xMaterialSurfaceTypes.GetTypeHashFromIndex(uIndex);}


private :
	ToolsAVP_SurfaceOverlay( const char* szName, bool bAlwaysOnTop, Asura_Vector_2* xPosition = 0 ) : Asura_Overlay( szName, bAlwaysOnTop, xPosition ) {}

	static bool Save();
	static void SaveNoReturn() { Save(); }
	static void TransferModeData( int iLoadOrSave );

	static void CreateSurfaceOverlay();

	static void InitialiseArrays();

	static void	CreateDamageTable(Asura_Table_Dialog* pxTableDialog);
	static void	CreateCollisionTable(Asura_Table_Dialog* pxTableDialog);
	static void	CreateEffectTypeTable(Asura_Table_Dialog* pxTableDialog);

	static Asura_TableCell_Dialog*	CreateDamageCellDialog( Asura_Table_Dialog& xTableDialog, const Asura_Vector_2& xOffset, const Asura_Vector_2& xMinCellSize, const u_int uRow, const u_int uColumn, void* const pxUserData );
	static Asura_TableCell_Dialog*	CreateCollisionCellDialog( Asura_Table_Dialog& xTableDialog, const Asura_Vector_2& xOffset, const Asura_Vector_2& xMinCellSize, const u_int uRow, const u_int uColumn, void* const pxUserData );
	static Asura_TableCell_Dialog*	CreateEffectTypeCellDialog( Asura_Table_Dialog& xTableDialog, const Asura_Vector_2& xOffset, const Asura_Vector_2& xMinCellSize, const u_int uRow, const u_int uColumn, void* const pxUserData );

	static void ApplyFilterBlueprint(const bool bRefreshOverlay, ToolsAVP_SurfaceOverlay_FilterData& xFilterData);

	static void RequestFilter(const char* szRequest, Asura_Function_Dialog* pxFunctionDialogue, ToolsAVP_SurfaceOverlay_FilterData& xFilterData);

	static void OnSetDamageTableRowFilter( int iFunctionDialog );
	static void OnSetDamageTableColumnFilter( int iFunctionDialog );
	static void OnSetCollisionTableRowFilter( int iFunctionDialog );
	static void OnSetCollisionTableColumnFilter( int iFunctionDialog );
	static void OnSetEffectTypeTableRowFilter( int iFunctionDialog );
	static void OnSetEffectTypeTableColumnFilter( int iFunctionDialog );

	static Asura_Table_Dialog*	s_pxDamageTableDialogue;
	static Asura_Table_Dialog*	s_pxCollisionTableDialogue;
	static Asura_Table_Dialog*	s_pxEffectTypeTableDialogue;
	
	static Asura_Vector_2	s_xSurfaceOverlayPosition;
	static Asura_Vector_2	s_xSurfaceOverlaySize;

	static bool	s_bModified;


	class TypeNameArray
	{
	public:
		u_int				GetNumTypes()		{ return s_uNumTypesAdded; }
		const char* const*	GetTypeNameArray()	{ return s_pszTypeNameArray; }
		Asura_Hash_ID		GetTypeHashFromIndex( u_int uIndex );
		void				Initialise( const Asura_Hash_ID uTypeHashID, const Asura_Hash_ID uParentBlueprintHashID );
		void				Deallocate();

		TypeNameArray() : s_uMaxTypes(0), s_uNumTypesAdded(0), s_pszTypeNameArray(NULL)	{}
		~TypeNameArray()	{Deallocate();}

	private:
		u_int		s_uMaxTypes;
		u_int		s_uNumTypesAdded;
		const char**s_pszTypeNameArray;
	};

	static TypeNameArray	s_xEffectTypes;
	static TypeNameArray	s_xDamageTypes;
	static TypeNameArray	s_xSurfaceTypes;
	static TypeNameArray	s_xMaterialSurfaceTypes;


	static ToolsAVP_SurfaceOverlay_FilterData	s_xDamageTableRowFilterData;
	static ToolsAVP_SurfaceOverlay_FilterData	s_xDamageTableColumnFilterData;
	static ToolsAVP_SurfaceOverlay_FilterData	s_xCollisionTableRowFilterData;
	static ToolsAVP_SurfaceOverlay_FilterData	s_xCollisionTableColumnFilterData;
	static ToolsAVP_SurfaceOverlay_FilterData	s_xEffectTypeTableRowFilterData;
	static ToolsAVP_SurfaceOverlay_FilterData	s_xEffectTypeTableColumnFilterData;

	friend class ToolsAVP_DamageCell_Dialog;
	friend class ToolsAVP_CollisionCell_Dialog;
	friend class ToolsAVP_EffectTypeCell_Dialog;
	friend class ToolsAVP_MaterialSurfaceType_DropDownListDialog;
};

//////////////////////////////////////////////////////////////////////////
// ToolsAVP_DamageCell_Dialog
//////////////////////////////////////////////////////////////////////////

class ToolsAVP_DamageCell_Dialog : public Asura_TableCell_Dialog
{
public:
	ToolsAVP_DamageCell_Dialog( Asura_Table_Dialog& xTableDialogue, const Asura_Vector_2& xOffset, const Asura_Vector_2& xMinCellSize, const u_int uRow, const u_int uColumn );
	virtual int Compare (const Asura_TableCell_Dialog& xOther) const {return 1;}

private:
	typedef Asura_TableCell_Dialog PARENT;

	void		OnSelect(void);
	static void	OnSelect(int iCellDialog);

	Asura_Function_Dialog*	m_pxSelectEffectDialogue;
	Asura_Hash_ID			m_uEffectHash;

	ToolsAVP_DamageCell_Dialog  (void);
	ToolsAVP_DamageCell_Dialog  (const ToolsAVP_DamageCell_Dialog& xOther);

	ToolsAVP_DamageCell_Dialog&	operator=	(const ToolsAVP_DamageCell_Dialog& xOther);
};

//////////////////////////////////////////////////////////////////////////
// ToolsAVP_CollisionCell_Dialog
//////////////////////////////////////////////////////////////////////////

class ToolsAVP_CollisionCell_Dialog : public Asura_TableCell_Dialog
{
public:
	ToolsAVP_CollisionCell_Dialog( Asura_Table_Dialog& xTableDialogue, const Asura_Vector_2& xOffset, const Asura_Vector_2& xMinCellSize, const u_int uRow, const u_int uColumn );
	virtual int Compare (const Asura_TableCell_Dialog& xOther) const {return 1;}

private:
	typedef Asura_TableCell_Dialog PARENT;

	void		OnSelect(void);
	static void	OnSelect(int iCellDialog);

	Asura_Function_Dialog*	m_pxSelectEffectDialogue;
	Asura_Hash_ID			m_uEffectHash;

	ToolsAVP_CollisionCell_Dialog  (void);
	ToolsAVP_CollisionCell_Dialog  (const ToolsAVP_CollisionCell_Dialog& xOther);

	ToolsAVP_CollisionCell_Dialog&	operator=	(const ToolsAVP_CollisionCell_Dialog& xOther);
};

//////////////////////////////////////////////////////////////////////////
// ToolsAVP_EffectTypeCell_Dialog
//////////////////////////////////////////////////////////////////////////

class ToolsAVP_EffectTypeCell_Dialog : public Asura_TableCell_Dialog
{
public:
	ToolsAVP_EffectTypeCell_Dialog( Asura_Table_Dialog& xTableDialogue, const Asura_Vector_2& xOffset, const Asura_Vector_2& xMinCellSize, const u_int uRow, const u_int uColumn );
	virtual int Compare (const Asura_TableCell_Dialog& xOther) const {return 1;}

private:
	typedef Asura_TableCell_Dialog PARENT;

	void		OnSelect(void);
	static void	OnSelect(int iCellDialog);

	Asura_Function_Dialog*	m_pxSelectEffectDialogue;
	Asura_Hash_ID			m_uEffectHash;

	ToolsAVP_EffectTypeCell_Dialog  (void);
	ToolsAVP_EffectTypeCell_Dialog  (const ToolsAVP_EffectTypeCell_Dialog& xOther);

	ToolsAVP_EffectTypeCell_Dialog&	operator=	(const ToolsAVP_EffectTypeCell_Dialog& xOther);
};

//////////////////////////////////////////////////////////////////////////
// ToolsAVP_MaterialSurfaceType_DropDownListDialog
//////////////////////////////////////////////////////////////////////////

class ToolsAVP_MaterialSurfaceType_DropDownListDialog : public Asura_Hash_DropDownList_Dialog
{
public:
	ToolsAVP_MaterialSurfaceType_DropDownListDialog( const Asura_Vector_2& xOffset, const int iNumberOfLines, Asura_Hash_ID* puSelected, bool* pbChangedFlag = NULL, const char* szName = NULL )
		: PARENT(xOffset)
	{
		Initialise(xOffset, iNumberOfLines, puSelected, pbChangedFlag, szName);
	}

protected:
	virtual u_int GetNumberOfHashValues(void) const { return ToolsAVP_SurfaceOverlay::s_xMaterialSurfaceTypes.GetNumTypes(); }
	virtual const char* GetNameByIndex(int iIndex) const 
	{ 
		Asura_Assertf( iIndex >= 0 && iIndex < static_cast< int >( ToolsAVP_SurfaceOverlay::s_xMaterialSurfaceTypes.GetNumTypes() ), ( "%d", iIndex ) ); 
		return (ToolsAVP_SurfaceOverlay::s_xMaterialSurfaceTypes.GetTypeNameArray()[iIndex] ); 
	}

private:

	typedef Asura_Hash_DropDownList_Dialog PARENT;

	ToolsAVP_MaterialSurfaceType_DropDownListDialog  (void);
	ToolsAVP_MaterialSurfaceType_DropDownListDialog  (const ToolsAVP_MaterialSurfaceType_DropDownListDialog& xOther);

	ToolsAVP_MaterialSurfaceType_DropDownListDialog&	operator=	(const ToolsAVP_MaterialSurfaceType_DropDownListDialog& xOther);

};

#endif //__TOOLS_SURFACEOVERLAY_H__
