//////////////////////////////////////////////////////////////////////////////
//
// ToolsAVP_ChooseObjects.h
//
// Copyright (C) 2008 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __TOOLSAVP_CHOOSEOBJECTS_H__
#define __TOOLSAVP_CHOOSEOBJECTS_H__ 1

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// asura
#include "Core/Asura_LinkList.h"

// tools
#include "Objects/Tools_Object.h"
#include "Objects/Tools_Object_System.h"

// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
//
// Choose Objects
//
//////////////////////////////////////////////////////////////////////////////

class ToolsAVP_ChooseObject_ByType : public Tools_ChooseObject
{
public:
	ToolsAVP_ChooseObject_ByType( const int iType, Asura_Guid* puGuidToSet, bool* pbChanged = NULL )
	: m_xTypeList()
	, m_puGuidToSet( puGuidToSet )
	, m_pfnOnChosenGuid( NULL )
	, m_pfnOnChosenIntGuid( NULL )
	, m_iUserData( 0 )
	, m_pbChanged( pbChanged )
	, m_bAllowMultipleSelection( false )
	{
		AddType( iType );
	}
	ToolsAVP_ChooseObject_ByType( const int iType, void( *pfnOnChosen )( Asura_Guid ), bool* pbChanged = NULL )
	: m_xTypeList()
	, m_puGuidToSet( NULL )
	, m_pfnOnChosenGuid( pfnOnChosen )
	, m_pfnOnChosenIntGuid( NULL )
	, m_iUserData( 0 )
	, m_pbChanged( pbChanged )
	, m_bAllowMultipleSelection( false )
	{
		AddType( iType );
	}
	ToolsAVP_ChooseObject_ByType( const int iType, void( *pfnOnChosen )( int, Asura_Guid ), int iUserData, bool* pbChanged = NULL )
	: m_xTypeList()
	, m_puGuidToSet( NULL )
	, m_pfnOnChosenGuid( NULL )
	, m_pfnOnChosenIntGuid( pfnOnChosen )
	, m_iUserData( iUserData )
	, m_pbChanged( pbChanged )
	, m_bAllowMultipleSelection( false )
	{
		AddType( iType );
	}

	virtual bool SupportMultipleSelection() { return m_bAllowMultipleSelection; }
	void SetAllowMultipleSelection( const bool bAllow ) { m_bAllowMultipleSelection = bAllow; }

	void AddType( const int iType ) { m_xTypeList.Push_Front( iType ); }

	virtual bool CanBeSelected( Tools_Object* pxObject )
	{
		if ( pxObject )
		{
			return ( m_xTypeList.Find( pxObject->Type ) != NULL );
		}
		else
		{
			return false;
		}
	}
	virtual void ObjectSelected( Tools_Object* pxObject )
	{
		if ( m_bAllowMultipleSelection )
		{
			for ( CLIF< Tools_Object* > xIter( GetSelectionList() ); !xIter.done(); xIter.next() )
			{
				if ( m_puGuidToSet )
				{
					*m_puGuidToSet = xIter()->GetGuid();
				}
				if ( m_pfnOnChosenGuid )
				{
					m_pfnOnChosenGuid( xIter()->GetGuid() );
				}
				if ( m_pfnOnChosenIntGuid )
				{
					m_pfnOnChosenIntGuid( m_iUserData, xIter()->GetGuid() );
				}
				if ( m_pbChanged )
				{
					*m_pbChanged = true;
				}
			}
		}
		else
		{
			const Asura_Guid uObjectGuid = pxObject ? pxObject->GetGuid() : ASURA_GUID_UNREGISTERED;
			if ( m_puGuidToSet )
			{
				*m_puGuidToSet = uObjectGuid;
			}
			if ( m_pfnOnChosenGuid )
			{
				m_pfnOnChosenGuid( uObjectGuid );
			}
			if ( m_pfnOnChosenIntGuid )
			{
				m_pfnOnChosenIntGuid( m_iUserData, uObjectGuid );
			}
			if ( m_pbChanged )
			{
				*m_pbChanged = true;
			}
		}
	}
	virtual void OnFinish() { delete this; }

private:
	Asura_LinkList< int > m_xTypeList;
	Asura_Guid*			  m_puGuidToSet;
	void				  ( *m_pfnOnChosenGuid )( Asura_Guid );
	void				  ( *m_pfnOnChosenIntGuid )( int, Asura_Guid );
	int					  m_iUserData;
	bool*				  m_pbChanged;
	bool				  m_bAllowMultipleSelection;
};

class ToolsAVP_ChooseActorOrSquad : public Tools_ChooseObject
{
public:
	ToolsAVP_ChooseActorOrSquad( Asura_Guid* puGuidToSet )
	: m_puGuidToSet( puGuidToSet ), m_pfnOnChosen( NULL ), m_iUserData( 0 )
	{
	}

	void SetCallbackFn( void( *pfnOnChosen )( int iData ), int iUserData ) { m_pfnOnChosen = pfnOnChosen; m_iUserData = iUserData; }

	virtual bool CanBeSelected( Tools_Object* pxObject );
	virtual void ObjectSelected( Tools_Object* pxObject )
	{
		const Asura_Guid uObjectGuid = pxObject ? pxObject->GetGuid() : ASURA_GUID_UNREGISTERED;
		if ( m_puGuidToSet )
		{
			*m_puGuidToSet = uObjectGuid;
		}
		if ( m_pfnOnChosen )
		{
			m_pfnOnChosen( m_iUserData );
		}
	}
	virtual void OnFinish() { delete this; }

private:
	Asura_Guid*	m_puGuidToSet;
	void ( *m_pfnOnChosen )( int iData );
	int m_iUserData;
};

class ToolsAVP_LookAtChooseObject : public Tools_ChooseObject
{
public:
	ToolsAVP_LookAtChooseObject( void( *pfnOnChosen )( Asura_Guid ), Asura_Guid uGuidToIgnore = ASURA_GUID_UNREGISTERED )
	: m_pfnOnChosen( pfnOnChosen ), m_uGuidToIgnore ( uGuidToIgnore )
	{
	}

	void SetCallbackFn( void( *pfnOnChosen )( Asura_Guid uGuid ), int iUserData ) { m_pfnOnChosen = pfnOnChosen; }

	virtual bool CanBeSelected( Tools_Object* pxObject );

	virtual void ObjectSelected( Tools_Object* pxObject )
	{
		const Asura_Guid uObjectGuid = pxObject ? pxObject->GetGuid() : ASURA_GUID_UNREGISTERED;

		if ( m_pfnOnChosen )
		{
			m_pfnOnChosen( uObjectGuid );
		}
	}
	virtual void OnFinish() { delete this; }

private:
	void ( *m_pfnOnChosen )( Asura_Guid uGuid );
	Asura_Guid m_uGuidToIgnore;
};

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

#endif //__TOOLSAVP_CHOOSEOBJECTS_H__
