//////////////////////////////////////////////////////////////////////////////
//
// ToolsAVP_System.cpp
//
// Copyright (c) 2006 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header.
#include "ToolsAVP_System.h"

// Asura
#include "Blueprint/Asura_Blueprint.h"
#include "Blueprint/Asura_Blueprint_System.h"
#include "Camera/Asura_Camera.h"
#include "Core/Asura_Maths.h"
#include "Core/Asura_UserInput.h"
#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Overlay.h"
#include "Interface/Asura_Overlay_System.h"
#include "Interface/Asura_Key_Combination.h"
#include "Mesh/Asura_Env_System.h"

// project
#include "Actor/AVP_Actor_HUDHandler.h"
#include "Attractors/AVP_Attractors_System.h"
#include "Blueprint/AVP_Blueprint_HashIDs.h"
#include "Core/AVP_Core.h"
#include "Core/AVP_Game_Core.h"
#include "Mesh/AVP_PolygonFlags.h"
#include "Physics/AVP_Collision_Descriptor.h"
#include "SpecialFX/AVP_CloakEffect.h"

// axon
#include "Core/Axon_System.h"

// Tools.
#include "Tools_System.h"
#include "Entity/Tools_Entity_Teleporter.h"
#if defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)
#include "FacialAnimation/Tools_FacialAnimation_System.h"
#include "FacialAnimation/LipsyncConversion/Tools_FacialAnimation_LipsyncConversion_Annosoft.h"
#endif	//defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)
#include "Flags/Tools_Mode_Flags.h"
#include "Hierarchy/Tools_Mode_Hierarchy.h"
#include "Interface/Asura_Menubar_System.h"
#include "Interface/Asura_Menubar.h"
#include "Mesh/Asura_Mesh_System.h"
#include "MeshProcessing/Tools_SmoothSkin_Processor.h"
#include "Misc/Tools_Render_Options.h"
#include "Movement/Tools_Movement.h"
#include "Texturing/Tools_Texturing_MaterialResponse.h"
#include "Waypointing/Tools_Mode_Waypointing.h"

// Project tools.
#include "Blueprint/ToolsAVP_Mode_Blueprint.h"
#include "Blueprint/ToolsAVP_Blueprint_AnimSpreadsheetExporter.h"
#include "Cutscene/ToolsAVP_Mode_Cutscene.h"
#include "Entity/ToolsAVP_Message_Editing.h"
#include "EntityTemplate/ToolsAVP_EntityTemplate_System.h"
#include "Load&Save/ToolsAVP_Level_Export.h"
#include "Load&Save/ToolsAVP_OptionsFile.h"
#include "Load&Save/ToolsAVP_SaveAsura.h"
#include "Material/ToolsAVP_SurfaceOverlay.h"
#include "MeshEdit/ToolsAVP_Mode_MeshEdit.h"
#include "Objects/ToolsAVP_Mode_Object.h"
#include "Sound/ToolsAVP_Mode_Sound.h"
#include "Waypointing/ToolsAVP_Mode_Waypointing.h"

//////////////////////////////////////////////////////////////////////////
// Static variables...
//////////////////////////////////////////////////////////////////////////
bool ToolsAVP_System::s_bRequestCancelQuit = false;

bool ToolsAVP_QuitToGameOverlay::s_bStartAtCamera = false;


//////////////////////////////////////////////////////////////////////////////
//
// Tools_System
//
//////////////////////////////////////////////////////////////////////////////

static Asura_Hash_ID*	g_puVisionModeHashes	= NULL;

#if defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)
static Tools_FacialAnimation_LipsyncConversion_Annosoft*	g_pxFacialAnimationLipsyncConversion	= NULL;
#endif	//defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)

void ToolsAVP_MaterialResponse_AddProjectUI(Tools_MaterialResponse* const pxMaterialResponse, Asura_Tabs_Dialog *pxParentDialog, Asura_Vector_2 xOffset);


void Tools_System::Project_Initialise( void )
{
	//CW - Load in AvP specific Tools Models
	if(!Asura_Mesh_System::GetMesh("toolsShape_Alien"))
	{
		LoadAsuraFile("Objects\\Tools\\AvPToolsShapes.asr");
	}

	Tools_Texturing_MaterialResponse::SetProjectUICallback(ToolsAVP_MaterialResponse_AddProjectUI);

    Tools_SmoothSkin_Processor::SetAllowMultipleTextures( true );
	
	if(Tools_SaveAsura::GetFileType()==AsuraFile_Environment || Tools_SaveAsura::GetFileType()==AsuraFile_Template)
	{
		// restore entities from quickload in case we came back from quit to game
		AVP_Core::ToolsQuickLoad();
	}

	// Code from RT, sets camera position to first start point
	static bool s_bCameraPositionInitialised = false;
	if(!s_bCameraPositionInitialised)
	{
		s_bCameraPositionInitialised = true;
		//initialise the camera position/orientation from the player start position
		Asura_Vector_3 xStartPos = Asura_Environment_System::GetEnvironment()->GetStartPosition();
		xStartPos.y -= 1.8f;
		Asura_Camera::SetPosition(xStartPos);
		Asura_Camera::SetOrientation(Asura_Environment_System::GetEnvironment()->GetStartOrientation());
		Tools_Movement::TeleportCameraToPosition(Asura_Camera::GetPosition(),Asura_Camera::GetOrientation());
	}

	ToolsAVP_SurfaceOverlay::Initialise();
	
	AVP_Attractors_System::Tools_Initialise();

	Asura_Menubar_Title* pxTitle;
	pxTitle = Asura_Menubar_System::GetTitle(BaseMenuHandle,"Utility");
	pxTitle->AddItem("Surface Types",ToolsAVP_SurfaceOverlay::ToggleSurfaceOverlay,ToolsAVP_SurfaceOverlay::SurfaceOverlayIsOn);

	pxTitle = Asura_Menubar_System::AddTitle( BaseMenuHandle, "AVP3" );
	if( pxTitle )
	{
		const u_int	uNumVisionModes	= Asura_Blueprint_System::GetNumChildBlueprints(uAVP_BLUEPRINT_TYPE_INVENTORY_ITEMS, uAVP_BLUEPRINT_NAME_INVENTORYITEM_VISIONMODE, false);

		char		szTemp[256];

		g_puVisionModeHashes	= new Asura_Hash_ID [uNumVisionModes];

		for(u_int u = 0; u < uNumVisionModes; u++)
		{
			const Asura_Blueprint*	pxBlueprint	= Asura_Blueprint_System::GetChildBlueprint(uAVP_BLUEPRINT_TYPE_INVENTORY_ITEMS, uAVP_BLUEPRINT_NAME_INVENTORYITEM_VISIONMODE, false, u);

			g_puVisionModeHashes[u]	= pxBlueprint->GetHashID();

			if(pxBlueprint != NULL)
			{
				const Asura_String&	xVisionModeName	= pxBlueprint->GetName();
				const void*			pUserData		= static_cast<void*>(&g_puVisionModeHashes[u]);

				Asura_PrintfToBfr(szTemp, "Use \"%s\" vision-mode", 256, true, xVisionModeName.GetString());
				pxTitle->AddItemWithUserData(szTemp, pUserData, ToolsAVP_System::SetVisionModeCamera, ToolsAVP_System::IsUsingVisionModeCamera);
			}
		}

		pxTitle->AddSeparator();

		pxTitle->AddItem( "Use Custom Camera", ToolsAVP_System::SetCustomCamera,
			ToolsAVP_System::IsUsingCustomCamera );

		pxTitle->AddSeparator();

		pxTitle->AddItem( "Run Anim Spreadsheet Exporter", ToolsAVP_Blueprint_AnimSpreadsheetExporter::Run );
	}

	Tools_Object_System::DeleteAllModeSpecific();

    // We don't want the physics walkable flag to be selected whenever the floor (use for waypointing) flag is.
    Tools_Mode_Flags::SetSelectPhysicsWalkableWithFloorFlag( false );

	// Use the setting loaded from AsuraTools.ini
	ToolsAVP_System::SetCamera( ToolsAVP_System::GetCurrVisionMode() );

	Tools_ChooseObject_EntityToTeleport::SetProjectCanBeSelectedFunction(ToolsAVP_System::Project_Teleporter_EntityCanBeSelectedFunction);

#if defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)
	// Initialise the lip-sync conversion code...
	g_pxFacialAnimationLipsyncConversion	= new Tools_FacialAnimation_LipsyncConversion_Annosoft ("ALIEN_V_PRED_3", "REBELLION", "6D54-5B49-4E40-4E49-5B3C-5159-7049-5153-5F5A-405A-5F53-515C-4A4D-533F-5872-5E23-4127-5925");

	Tools_FacialAnimation_System::SetLipsyncConversion(g_pxFacialAnimationLipsyncConversion);
#endif	//defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)
}

void ToolsAVP_System::Project_Update()
{
    PARENT::Project_Update();

    AVP_Attractors_System::Tools_Update();

	// When cutscene is playing don't update vision mode as the cutscene event may need to change vision mode
	if(!Tools_Mode_Cutscene::IsPlayingCutscene())
	{
		SetCamera(s_uCurrVisionMode);
	}
}

void ToolsAVP_System::Project_LateRender()
{
	AVP_Attractors_System::Tools_Render();

	AVP_CloakEffect::RenderTransparentPasses();
}


void Tools_System::Project_Shutdown( void )
{
#if defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)
	delete(g_pxFacialAnimationLipsyncConversion);
	g_pxFacialAnimationLipsyncConversion	= NULL;
#endif	//defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)

	delete[](g_puVisionModeHashes);
	g_puVisionModeHashes	= NULL;

	if(Tools_SaveAsura::GetFileType()==AsuraFile_Environment || Tools_SaveAsura::GetFileType()==AsuraFile_Template)
	{
		if(!QuitToDesktop)
		{
			// Time to do our fake-export...
			Tools_Level_Export::DoFakeLevelExport();

			// save entities from tools in case we are quitting to game
			AVP_Core::ToolsQuickSave();
			AVP_Core::ToolsQuickLoad();
		}
	}

	ToolsAVP_SurfaceOverlay::Shutdown();

}

void Tools_System::Project_Allocate_Tools_System( void )
{
    new ToolsAVP_System();
}

void ToolsAVP_System::Allocate_Tools_OptionsFile()
{
	new ToolsAVP_OptionsFile;
}

void ToolsAVP_System::Allocate_Tools_SaveAsura(void)
{
	new ToolsAVP_SaveAsura;
}

void ToolsAVP_System::Allocate_Tools_EntityTemplate_System()
{
	new ToolsAVP_EntityTemplate_System;
}

const char* Tools_System::Project_GetIniFileDirectory( void )
{
    return "ToolsIniFiles";
}

void Tools_System::Project_ChangeToNextMode( void )
{
    switch( NextMode )
    {
	   case TOOLS_MODE_SOUND:
		    CurrentModeClass = new ToolsAVP_Mode_Sound();
		    break;
       case TOOLS_MODE_CUTSCENE:
            CurrentModeClass = new ToolsAVP_Mode_Cutscene();
            break;

        case TOOLS_MODE_OBJECT:
            CurrentModeClass = new ToolsAVP_Mode_Object();
            break;

		case TOOLS_MODE_WAYPOINTING :
			CurrentModeClass = new ToolsAVP_Mode_Waypointing();
			break;

		case TOOLS_MODE_MESHEDIT:
			CurrentModeClass = new ToolsAVP_Mode_MeshEdit();
			break;

        default:
            break;
    }
}

//////////////////////////////////////////////////////////////////////////////
//
// ToolsAVP_System
//
//////////////////////////////////////////////////////////////////////////////

const int ToolsAVP_System::s_aiPolyFlagValues[ NUM_AVP_PROJECT_POLYGON_FLAGS ] =
{
    AVP_PolygonFlag_IgnoreForPlayerCollision,
    AVP_PolygonFlag_IgnoreForReorientation,
	AVP_PolygonFlag_IgnoreForAlienCollision,
	AVP_PolygonFlag_IgnoreForFaceHuggerCollision,
	AVP_PolygonFlag_IgnoreForHumanCollision,
	AVP_PolygonFlag_IgnoreForPredatorCollision,
	AVP_PolygonFlag_IgnoreForPhysicsObjectCollision,
};

const char* ToolsAVP_System::s_aszPolyFlagNames[ NUM_AVP_PROJECT_POLYGON_FLAGS ] =
{
    "Ignore for Player Collision",
	"Ignore for Reorientation",
	"Ignore for Alien Collision",
	"Ignore for Face-Hugger Collision",
	"Ignore for human Collision",
	"Ignore for Predator Collision",
	"Ignore for PhysicsObject Collision",
};

const char* ToolsAVP_System::s_aszPolyFlagHelpTexts[ NUM_AVP_PROJECT_POLYGON_FLAGS ] =
{
    "Alien will creep when moving over these polys, if reasonable.",
    "Reorientation code will ignore this poly.",
	"Alien collision will ignore this poly.",
	"Face-Hugger collision will ignore this poly.",
	"Human collision will ignore this poly.",
	"Predator collision will ignore this poly.",
	"PhysicsObjects collision will ignore this poly.",
};

Asura_Hash_ID ToolsAVP_System::s_uCurrVisionMode = ASURA_HASH_ID_UNSET;

void ToolsAVP_System::Allocate_Tools_Level_Export()
{
	new ToolsAVP_Level_Export();
}

int ToolsAVP_System::GetNumberOfProjectPolygonFlags() const
{
    return NUM_AVP_PROJECT_POLYGON_FLAGS;
}

int ToolsAVP_System::GetProjectPolygonFlagValue(int iIndex) const
{
    return s_aiPolyFlagValues[ iIndex ];
}

const char* ToolsAVP_System::GetProjectPolygonFlagName(int iIndex) const
{
    return s_aszPolyFlagNames[ iIndex ];
}

const char* ToolsAVP_System::GetProjectPolygonFlagHelpText(int iIndex) const
{
    return s_aszPolyFlagHelpTexts[ iIndex ];
}

void ToolsAVP_System::AddExportOptions( Asura_Menubar_Title* pxTitle )
{
	PARENT::AddExportOptions( pxTitle );

	pxTitle->AddItem("Export Weapons",ToolsAVP_Level_Export::CreateExportWeaponOverlay);
}

void ToolsAVP_System::Allocate_Tools_Message_Editing(void)
{
	new ToolsAVP_Message_Editing();
}

void ToolsAVP_System::SetCamera(const Asura_Hash_ID uHash)
{
	AVP_Game_Core::SetVisionMode(uHash, s_uCurrVisionMode, true);
	s_uCurrVisionMode = uHash;

#if 0
	// Set FOV

	Asura_Hash_ID uBlueprintHash = ASURA_HASH_ID_UNSET;

	switch(uHash)
	{
		case uAVP_BLUEPRINT_NAME_ALIENVISIONMODE:
		{
			uBlueprintHash = uAVP_BLUEPRINT_NAME_ALIENPLAYER;
			break;
		}

		case uAVP_BLUEPRINT_NAME_MARINEVISIONMODE:
		case uAVP_BLUEPRINT_NAME_VMNIGHTVISION:
		{
			uBlueprintHash = uAVP_BLUEPRINT_NAME_HUMANPLAYER;
			break;
		}

		case uAVP_BLUEPRINT_NAME_PREDATORVISIONMODE:
		case uAVP_BLUEPRINT_NAME_VMSEEALIENS:
		case uAVP_BLUEPRINT_NAME_VMHEATVISION:
		case uAVP_BLUEPRINT_NAME_VMPREDTECH:
		{
			uBlueprintHash = uAVP_BLUEPRINT_NAME_PREDATORPLAYER;
			break;
		}

		case ASURA_HASH_ID_UNSET:
		{
			uBlueprintHash = ASURA_HASH_ID_UNSET;
			break;
		}

		default:
		{
			Asura_Assertf(false, ("Unexpected hash ID passed to ToolsAVP_System::SetCamera() (%u)",
								  uHash));

			uBlueprintHash = ASURA_HASH_ID_UNSET;
			break;
		}
	}

	if(uBlueprintHash != ASURA_HASH_ID_UNSET)
	{
		Asura_Blueprint* pxBlueprint = NULL;
		Asura_Blueprint_System::GetBlueprint(uAVP_BLUEPRINT_TYPE_PLAYERS, uBlueprintHash, pxBlueprint);

		Asura_Value xValue;

		if( pxBlueprint
		 && pxBlueprint->GetParameterValueByHashID(uAVP_BLUEPRINT_PARAM_DEFAULT_FOV, xValue))
		{
			Tools_Render_Options::DefaultFieldOfView = xValue.GetFloatValue();

			Tools_Render_Options::DefaultFieldOfViewIsVertical = false;
			Tools_Render_Options::UseDefaultFieldOfView();
		}
		else
		{
			Asura_Assertf(false, ("Failed to get player blueprint with hash ID %u.", uBlueprintHash));
		}
	}
#endif
}


////////////////////////////////////////////////////////////////////////////////
// Function     : ToolsAVP_System::SetVisionModeCamera
// Author       : Andy Tate
// Date         : 23-Jan-2008
////////////////////////////////////////////////////////////////////////////////

void ToolsAVP_System::SetVisionModeCamera(const void* pUserData)
{
	const Asura_Hash_ID*	puHash	= static_cast<const Asura_Hash_ID*>(pUserData);

	if(puHash != NULL)
	{
		SetCamera(*puHash);
	}
}

/// End of function 'ToolsAVP_System::SetVisionModeCamera'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : ToolsAVP_System::IsUsingVisionModeCamera
// Author       : Andy Tate
// Date         : 23-Jan-2008
////////////////////////////////////////////////////////////////////////////////

bool ToolsAVP_System::IsUsingVisionModeCamera(const void* pUserData)
{
	const Asura_Hash_ID*	puHash	= static_cast<const Asura_Hash_ID*>(pUserData);

	return((puHash != NULL) && (s_uCurrVisionMode == *puHash));
}

/// End of function 'ToolsAVP_System::IsUsingVisionModeCamera'. ///


bool ToolsAVP_System::Project_Teleporter_EntityCanBeSelectedFunction(const Tools_Object* pxObject)
{
	Asura_Guid uGuid = pxObject->GetGuid();

	if ((uGuid >= ASURA_GUID_CLIENT_START) && (uGuid <= ASURA_GUID_CLIENT_END)) { return true; }

	return false;
}

////////////////////////////////////////////////////////////////////////////////
// Function     : ToolsAVP_QuitToGameOverlay::ToolsAVP_QuitToGameOverlay
// Author       : Andy Tate
// Date         : 16-Mar-2007
////////////////////////////////////////////////////////////////////////////////

ToolsAVP_QuitToGameOverlay::ToolsAVP_QuitToGameOverlay(void)
: PARENT("Quit To Game")
{
}

/// End of function 'ToolsAVP_QuitToGameOverlay::ToolsAVP_QuitToGameOverlay'. ///

void ToolsAVP_QuitToGameOverlay::Update()
{
	if (Asura_UserInput::DebouncedKeyPressed('c'))
	{
		s_bStartAtCamera = !s_bStartAtCamera;
	}
	else if (Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_SPACE))
	{
		// Same as enter
		ApplySettings(AVP_Settings::GetPlayerSpecies());
		if (GetOverlay())
		{
			GetOverlay()->m_bShouldClose = true;
		}
	}

	PARENT::Update();
}

////////////////////////////////////////////////////////////////////////////////
// Function     : ToolsAVP_QuitToGameOverlay::ApplySettings
// Author       : Anthony Gregson
// Date         : 12/10/07
////////////////////////////////////////////////////////////////////////////////

void ToolsAVP_QuitToGameOverlay::ApplySettings(int iPlayerSpecies)
{
	const AVP_Settings::Species	eSpecies	= static_cast<AVP_Settings::Species>(iPlayerSpecies);

	AVP_Settings::SetPlayerSpecies(eSpecies);
	AVP_Game_Core::SetRespawnSpecies(eSpecies);

	if (s_bStartAtCamera)
	{
		Asura_Environment_System::SetUseCameraAsStartPoint();
	}
	else
	{
		Asura_Environment_System::ResetUseCameraAsStartPoint();
	}
}

/// End of function 'ToolsAVP_QuitToGameOverlay::ApplySettings'. ///


////////////////////////////////////////////////////////////////////////////////
// Function     : ToolsAVP_QuitToGameOverlay::CancelQuit
// Author       : Anthony Gregson
// Date         : 12/10/07
////////////////////////////////////////////////////////////////////////////////

void ToolsAVP_QuitToGameOverlay::CancelQuit()
{
	ToolsAVP_System::SetRequestCancelQuit(true);
}

/// End of function 'ToolsAVP_QuitToGameOverlay::CancelQuit'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : ToolsAVP_QuitToGameOverlay::InitialiseOverlay
// Author       : Andy Tate
// Date         : 16-Mar-2007
////////////////////////////////////////////////////////////////////////////////

void ToolsAVP_QuitToGameOverlay::InitialiseOverlay(Asura_Overlay* pxOverlay)
{
	Asura_Vector_2	xOffset(1.0f, 3.0f);
	Asura_Dialog*	pxDialog;

	pxDialog	= new Asura_StringDisplay_Dialog("Player species:", xOffset);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y	+= Asura_Dialog::DefaultSpacing();

	pxDialog	= new Asura_FunctionWithKeybinding_Dialog("1) Marine", xOffset, ApplySettings, AVP_Settings::AVP_SPECIES_HUMAN, '1', &pxOverlay->m_bShouldClose);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y	+= Asura_Dialog::DefaultSpacing();

	pxDialog	= new Asura_FunctionWithKeybinding_Dialog("2) Alien", xOffset, ApplySettings, AVP_Settings::AVP_SPECIES_ALIEN, '2', &pxOverlay->m_bShouldClose);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y	+= Asura_Dialog::DefaultSpacing();

	pxDialog	= new Asura_FunctionWithKeybinding_Dialog("3) Predator", xOffset, ApplySettings, AVP_Settings::AVP_SPECIES_PREDATOR, '3', &pxOverlay->m_bShouldClose);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y	+= Asura_Dialog::LargeSpacing();

	char szLastSetting[128];
	// We now default to unknown.
	if (AVP_Settings::GetPlayerSpecies() == AVP_Settings::AVP_SPECIES_UNKNOWN)
	{
		AVP_Settings::SetPlayerSpecies(AVP_Settings::AVP_SPECIES_HUMAN);
	}
	Asura_PrintfToBfr(szLastSetting, "Last species (%s)", 128, true, AVP_Settings::GetSpeciesName(AVP_Settings::GetPlayerSpecies()));
	pxDialog	= new Asura_FunctionWithKeybinding_Dialog(szLastSetting, xOffset, ApplySettings, AVP_Settings::GetPlayerSpecies(), ASURA_KEYID_ENTER, &pxOverlay->m_bShouldClose);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y	+= Asura_Dialog::LargeSpacing();

	pxDialog	= new Asura_Tickbox_Dialog("Start at (C)amera", xOffset, &s_bStartAtCamera);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y	+= Asura_Dialog::LargeSpacing();

	pxDialog	= new Asura_FunctionWithKeybinding_Dialog("Cancel", xOffset, CancelQuit, ASURA_KEYID_ESCAPE, &pxOverlay->m_bShouldClose);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y	+= Asura_Dialog::LargeSpacing();
}

/// End of function 'ToolsAVP_QuitToGameOverlay::InitialiseOverlay'. ///


//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////
