//////////////////////////////////////////////////////////////////////////////
//
// ToolsAVP_System.h
//
// Copyright (c) 2006 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef TOOLSAVP_SYSTEM_H___
#define TOOLSAVP_SYSTEM_H___ 1

// I N C L U D E S ///////////////////////////////////////////////////////////

// Tools.
#include "Tools_System.h"

// Asura Platform-independent.
#include "Camera/Asura_Camera.h"

// AVP3 Platform-independent.
#include "Blueprint/AVP_Blueprint_HashIDs.h"

// C L A S S E S /////////////////////////////////////////////////////////////
class Tools_Object;

class ToolsAVP_System : public Tools_System
{
	friend class ToolsAVP_OptionsFile;

public:
	ToolsAVP_System() {}

    virtual void Project_Update();
	//project specific per-frame Render for tools
	virtual void Project_LateRender();

    virtual void Allocate_Tools_Level_Export();
	virtual void Allocate_Tools_Message_Editing();
	virtual void Allocate_Tools_OptionsFile();
	virtual void Allocate_Tools_SaveAsura();
	virtual void Allocate_Tools_EntityTemplate_System();

    virtual int         GetNumberOfProjectPolygonFlags()          const;
    virtual int         GetProjectPolygonFlagValue(int iIndex)    const;
    virtual const char* GetProjectPolygonFlagName(int iIndex)     const;
    virtual const char* GetProjectPolygonFlagHelpText(int iIndex) const;

	virtual bool CanExportGlobalSoundsInCommon(void) const	{ return(true); }

	virtual void AddExportOptions( Asura_Menubar_Title* pxTitle );

	static void SetRequestCancelQuit(bool bRequest) { s_bRequestCancelQuit = bRequest; }
	static bool IsCancelQuitRequested() { return s_bRequestCancelQuit; }

	static Asura_Hash_ID GetCurrVisionMode() { return s_uCurrVisionMode; }
	static void SetCamera(const Asura_Hash_ID uHash);

	static void SetVisionModeCamera(const void* pUserData);
	static bool IsUsingVisionModeCamera(const void* pUserData);

	static void SetCustomCamera()		{ SetCamera(ASURA_HASH_ID_UNSET); }
	static bool IsUsingCustomCamera()	{ return s_uCurrVisionMode == ASURA_HASH_ID_UNSET; }

	static bool Project_Teleporter_EntityCanBeSelectedFunction(const Tools_Object* pxObject);

protected:
    static const int   s_aiPolyFlagValues[];
    static const char* s_aszPolyFlagNames[];
    static const char* s_aszPolyFlagHelpTexts[];

	static Asura_Hash_ID s_uCurrVisionMode;

	static bool s_bRequestCancelQuit;

private:
	typedef Tools_System PARENT;
};


class ToolsAVP_QuitToGameOverlay : public Tools_Modal_Overlay
{
public :
	ToolsAVP_QuitToGameOverlay(void);

protected :
	virtual void InitialiseOverlay(class Asura_Overlay* pxOverlay);
	virtual void Update();

	static void ApplySettings(int iPlayerSpecies);
	static void CancelQuit();

private:
	// Private typedefs...
	typedef Tools_Modal_Overlay PARENT;

	// Private variables...
	static bool s_bStartAtCamera;	// Whether to start at the camera or the level start.
};

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

#endif // #ifndef TOOLSAVP_SYSTEM_H___
