//////////////////////////////////////////////////////////////////////////////
//
// ToolsAVP_Waypoint_Analysis.h
//
// Copyright (C) 2007 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __TOOLSAVP_WAYPOINT_ANALYSIS_H__
#define __TOOLSAVP_WAYPOINT_ANALYSIS_H__ 1

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// axon
#include "Core/AVP_Navigation.h"

// tools
#include "Waypointing/Tools_Waypoint_Analysis.h"

// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
//
// ToolsAVP_Waypoint_Analysis
//
//////////////////////////////////////////////////////////////////////////////

class ToolsAVP_Waypoint_Analysis : public Tools_Waypoint_Analysis
{
    typedef Tools_Waypoint_Analysis PARENT;

public:
    ToolsAVP_Waypoint_Analysis();

	virtual void Update();

    virtual void AddOverlayItemsToTabsDialogue( Asura_Tabs_Dialog* pxTabsDialogue, Asura_Vector_2& xOffset );
    virtual void RenderWorld();

	static bool IsInterestedInAlienRouteFailure()    { return s_bIsInterestedInAlienRouteFailure; }
	static bool IsInterestedInNonAlienRouteFailure() { return s_bIsInterestedInNonAlienRouteFailure; }

protected:
	virtual bool AllowedToAnalyseRouteFind();

	virtual void AnalyseLink();
    virtual void StopLinkAnalysis();
	virtual void AnalyseRouteFind();
	virtual void StopRouteFindAnalysis();
    virtual void NextRouteFindStep();
    virtual void NextRouteFindSteps();

private:
	void OnRouteFindFinished();

	static void PathfinderSelectionChanged( int iThis );
	static bool AllowRangeChange( int iThis );
	static bool AllowDirectionChange( int iThis );
	static void RotateDirectionLeft( int iThis );
	static void RotateDirectionRight( int iThis );
	static bool AllowLoFAvoidWPSelection( int iThis );
	static void SelectLoFAvoidWP( int iThis );
	static void ClearLoFAvoidWP( int iThis );

private:
    Asura_Bounding_Box m_xOutputBoundingBox;
    Asura_Quat*        m_pxOrientationOutputArray;
    u_int              m_uPathfinderFlags;
	float			   m_fRange;
	Asura_Vector_3	   m_xDirection;
	AVP_Navigation::AStarFunctorBase* m_pxRouteFindFunctor;
	int				   m_iPathfinderSelection;
	int				   m_iFunctorSelection;
	bool			   m_bRotatingDirectionLeft;
	bool			   m_bRotatingDirectionRight;
	u_int			   m_uSelectedAvoidLoFWPIndex;

	static bool s_bIsInterestedInAlienRouteFailure;
	static bool s_bIsInterestedInNonAlienRouteFailure;

	enum { PATHFINDER_HUMANOID, PATHFINDER_ALIEN, PATHFINDER_PREDATOR, NUM_PATHFINDER_TYPES };
	static const char* s_pszPathfinderStrings[ NUM_PATHFINDER_TYPES ];

	enum SupportedFunctors
	{
		HUMANOID_STANDARD,
		ALIEN_STANDARD,
		PREDATOR_STANDARD,
		ALIEN_AWAY_FROM,
		ALIEN_AWAY_FROM_IN_DIRECTION,
		ALIEN_FIND_COVER_IN_DIRECTION,
		ALIEN_FIND_NEAREST_COVER,
		ALIEN_FIND_WALL_IN_DIRECTION,
		ALIEN_FIND_FLOOR_IN_DIRECTION,
		HUMANOID_COMBAT,
		NUM_FUNCTOR_TYPES
	};
	struct FunctorData
	{
		bool m_bRequiresTwoWPs;
		bool m_bRangeValid;
		bool m_bDirectionValid;
	};
	static const char* s_pszFunctorStrings[ NUM_FUNCTOR_TYPES ];
	static const FunctorData s_axFunctorData[ NUM_FUNCTOR_TYPES ];
};

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

#endif //__TOOLSAVP_WAYPOINT_ANALYSIS_H__
