//////////////////////////////////////////////////////////////////////////////
//
// Axon_Gamescene_AnimSequenceData.cpp
//
// Copyright (c) 2006 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

#ifdef ASURA_USE_AXON

// header
#include "AnimTemplates/Axon_Gamescene_AnimSequenceData.h"

// Asura
#include "Animation/Asura_Animation.h"
#include "Animation/Asura_AnimationController_Animation.h"
#include "Network/Asura_Message.h"
#include "Network/Asura_NetMsg.h"

// axon
#include "AnimTemplates/Axon_Gamescene_AnimTemplate_System.h"

//////////////////////////////////////////////////////////////////////////////
//
// Axon_Gamescene_AnimSequenceData
//
//////////////////////////////////////////////////////////////////////////////

Axon_Gamescene_AnimSequenceData::Axon_Gamescene_AnimSequenceData()
: m_fStartTimer( -Asura_Maths::LargeFloat )
, m_uAnimTemplateSequenceID( uAXON_ANIMATION_SEQUENCE_ID_NONE )
, m_xStartPosition( Asura_Maths::ZeroVector3 )
, m_xStartOrientQ( Asura_Maths::IdentityQuat )
{
}

#ifdef ASURA_SUPPORT_NETMSG_SYSTEM
void Axon_Gamescene_AnimSequenceData::RegisterFields(class Asura_NetMsg* pxNetMsg)
{
	Axon_Gamescene_AnimSequenceData xDummy; 
	
    Asura_NetMsg_RegisterField( pxNetMsg, xDummy, Asura_FieldType_float,   m_fStartTimer,             1, Asura_Threshold_AnyChange, 0.0f ); 
	Asura_NetMsg_RegisterField( pxNetMsg, xDummy, Asura_FieldType_uint,    m_uAnimTemplateSequenceID, 1, Asura_Threshold_AnyChange, 0.0f );
	Asura_NetMsg_RegisterField( pxNetMsg, xDummy, Asura_FieldType_vector3, m_xStartPosition,          1, Asura_Threshold_AnyChange, 0.0f ); 
	Asura_NetMsg_RegisterField( pxNetMsg, xDummy, Asura_FieldType_quat,    m_xStartOrientQ,           1, Asura_Threshold_AnyChange, 0.0f ); 
}
#endif // #ifdef ASURA_SUPPORT_NETMSG_SYSTEM

//////////////////////////////////////////////////////////////////////////////
//
// Axon_GS2_AnimSequenceData_Handler
//
//////////////////////////////////////////////////////////////////////////////

Axon_GS2_AnimSequenceData_Handler::Axon_GS2_AnimSequenceData_Handler()
: m_ucReceivedNewData( 0 )
, m_ucAnimChangedThisUpdate( 0 )
, m_ucFidgetAnimChangedThisUpdate( 0 )
, m_xNewAnimSequenceData()
, m_xAnimSequenceData()
, m_pxAnimTemplate( NULL )
, m_pxAnimTemplateEntry( NULL )
, m_uAnimHashID( ASURA_HASH_ID_UNSET )
, m_fAnimPST( 0.0f )
, m_uAnimFlags( 0 )
, m_pxFidgetAnimTemplate( NULL )
, m_pxPrevFidgetAnimTemplateEntry( NULL )
, m_uFidgetAnimHashID( ASURA_HASH_ID_UNSET )
, m_uFidgetAnimFlags( 0 )
, m_fFidgetDelayTimer( 0.0f )
, m_fFidgetAnimTimer( 0.0f )
{
}

bool Axon_GS2_AnimSequenceData_Handler::HandleMessage( const Asura_Message* pxMessage )
{
	if ( pxMessage->GetMsgID() == ASURA_MSG_AXON_GS2_ANIM_SEQUENCE_DATA )
	{
		const Axon_Gamescene_AnimSequenceData* pxData = static_cast< const Axon_Gamescene_AnimSequenceData* >( pxMessage->GetData() );
		HandleData( *pxData );
		return true;
	}
	else
	{
		return false;
	}
}

void Axon_GS2_AnimSequenceData_Handler::HandleData( const Axon_Gamescene_AnimSequenceData& xAnimSequenceData )
{
	if ( Axon_Gamescene_AnimTemplate_System::GetTimeForAnimSequence() < xAnimSequenceData.m_fStartTimer ||
		m_xNewAnimSequenceData.m_uAnimTemplateSequenceID != xAnimSequenceData.m_uAnimTemplateSequenceID ||
		m_xNewAnimSequenceData.m_fStartTimer != xAnimSequenceData.m_fStartTimer )
	{
		m_xNewAnimSequenceData = xAnimSequenceData;
        m_ucReceivedNewData    = 1;
	}
}

void Axon_GS2_AnimSequenceData_Handler::UpdateAnimSequence( void* pMappingData )
{
	m_ucAnimChangedThisUpdate = 0;
	m_ucFidgetAnimChangedThisUpdate = 0;

	if ( m_ucReceivedNewData )
	{
		if ( Axon_Gamescene_AnimTemplate_System::GetTimeForAnimSequence() >= m_xNewAnimSequenceData.m_fStartTimer )
		{
			// If we have a pending sequence and it is time for it to start, the pending sequence becomes our current sequence.
            m_ucReceivedNewData             = 0;
			m_ucAnimChangedThisUpdate       = 1;
			// Fidget anims change every time a template or the main template entry changes.
			m_ucFidgetAnimChangedThisUpdate = ( m_uFidgetAnimHashID != ASURA_HASH_ID_UNSET );
			ResetCurrentData();
            m_xAnimSequenceData = m_xNewAnimSequenceData;
            m_pxAnimTemplate	= Axon_Gamescene_AnimTemplate_System::GetAnimTemplateByID( m_xAnimSequenceData.m_uAnimTemplateSequenceID );
		}
	}

	// If we have no template we've nothing more to do.
	if ( !m_pxAnimTemplate )
	{
		return;
	}

	// Calculate current template entry.
	const float fTimeInTemplate = Axon_Gamescene_AnimTemplate_System::GetTimeForAnimSequence() - m_xAnimSequenceData.m_fStartTimer;
	// fAnimTimeRemainder is the time we are into the anim we should be playing.
	float fAnimTimeRemainder = 0.0f;
	Axon_Gamescene_AnimTemplate_Entry* pxCurrentEntry = m_pxAnimTemplate->GetCurrentEntry( fTimeInTemplate, pMappingData, fAnimTimeRemainder );

	// If we play this template past the end then pxCurrentEntry will be NULL.
	// In this case we keep the same template entry until we receive new data (which should arrive in the next couple of frames).
	if ( pxCurrentEntry && pxCurrentEntry != m_pxAnimTemplateEntry )
	{
		// Current anim stuff.
        m_ucAnimChangedThisUpdate       = 1;
		m_ucFidgetAnimChangedThisUpdate |= ( m_uFidgetAnimHashID != ASURA_HASH_ID_UNSET );
        m_pxAnimTemplateEntry = pxCurrentEntry;
		m_uAnimHashID = m_pxAnimTemplateEntry->CalcAnimHashIDToPlay( pMappingData );
        m_fAnimPST    = 0.0f;
        m_uAnimFlags  = AACA_FLAG_CALCULATE_VELOCITY | AACA_FLAG_CALCULATE_ANGULAR_VELOCITY | AACA_FLAG_CALCULATE_MOVEMENT_OFFSET;
		if ( m_pxAnimTemplateEntry->m_bLoop )      { m_uAnimFlags |= AACA_FLAG_LOOPED;   }
		if ( m_pxAnimTemplateEntry->m_bBackwards ) { m_uAnimFlags |= AACA_FLAG_REVERSED; }
		if ( m_pxAnimTemplateEntry->m_bMirror )    { m_uAnimFlags |= AACA_FLAG_MIRRORED; }

		// Fidget stuff.
		m_pxFidgetAnimTemplate = NULL;
		m_uFidgetAnimHashID = ASURA_HASH_ID_UNSET;
        m_uFidgetAnimFlags  = 0;
        m_fFidgetDelayTimer = 0.0f;
        m_fFidgetAnimTimer  = 0.0f;
		if ( m_pxAnimTemplate->GetType() == AXON_ATT_STATIC )
		{
			const u_int uFidgetBehaviour = static_cast< Axon_Gamescene_StaticAnimTemplate_Entry* >( m_pxAnimTemplateEntry )->GetFidgetBehaviourID();
			m_pxFidgetAnimTemplate = 
				static_cast< Axon_Gamescene_FidgetAnimTemplate* >( Axon_Gamescene_AnimTemplate_System::GetAnimTemplateByID( uFidgetBehaviour ) );
			if ( m_pxFidgetAnimTemplate )
			{
				m_fFidgetDelayTimer = Asura_Random::GetFloat( m_pxFidgetAnimTemplate->m_fMinFidgetDelay, m_pxFidgetAnimTemplate->m_fMaxFidgetDelay );
				m_fFidgetAnimTimer = 0.0f;
			}
		}
	}
	{
	const Asura_Hierarchy_Anim* pxHierAnim = Asura_Animation_System::GetAnimByHash( m_uAnimHashID );
	if ( pxHierAnim )
	{
		m_fAnimPST = fAnimTimeRemainder / pxHierAnim->Duration;
	}
	}

	// Update the fidget parameters if necessary.
	if ( m_pxFidgetAnimTemplate )
	{
		if ( m_fFidgetAnimTimer > 0.0f )
		{
			// Playing a fidget.
			m_fFidgetAnimTimer -= Asura_Timers::GetGameTimeSlice();
			if ( m_fFidgetAnimTimer <= 0.0f )
			{
				// Finished playing our fidget.
				m_ucFidgetAnimChangedThisUpdate = 1;
				m_uFidgetAnimHashID = ASURA_HASH_ID_UNSET;
                m_uFidgetAnimFlags  = 0;
			}
		}

		if ( m_fFidgetAnimTimer <= 0.0f )
		{
			// Not playing a fidget.
			m_fFidgetDelayTimer -= Asura_Timers::GetGameTimeSlice();
			if ( m_fFidgetDelayTimer < 0.0f )
			{
				// Should start playing a fidget.
				const Axon_Gamescene_FidgetAnimTemplate_Entry* pxFidget = m_pxFidgetAnimTemplate->ChooseFidget();
				// If there is more than one valid entry try and pick a fidget different to the last one we played.
				if ( m_pxFidgetAnimTemplate->GetNumValidEntries() > 1 )
				{
					u_int uMaxLoops = 10;
					while ( pxFidget && pxFidget == m_pxPrevFidgetAnimTemplateEntry && uMaxLoops-- > 0 )
					{
						pxFidget = m_pxFidgetAnimTemplate->ChooseFidget();
					}
				}
				if ( pxFidget )
				{
					m_pxPrevFidgetAnimTemplateEntry = pxFidget;
					m_uFidgetAnimHashID = pxFidget->CalcAnimHashIDToPlay( pMappingData );
                    m_uFidgetAnimFlags  = 0;
					if ( pxFidget->m_bLoop )      { m_uFidgetAnimFlags |= AACA_FLAG_LOOPED;   }
					if ( pxFidget->m_bBackwards ) { m_uFidgetAnimFlags |= AACA_FLAG_REVERSED; }
					if ( pxFidget->m_bMirror )    { m_uFidgetAnimFlags |= AACA_FLAG_MIRRORED; }

					const Asura_Hierarchy_Anim* pxHierAnim = Asura_Animation_System::GetAnimByHash( m_uFidgetAnimHashID );
					if ( pxHierAnim )
					{
						m_ucFidgetAnimChangedThisUpdate = 1;
						m_fFidgetAnimTimer = pxHierAnim->GetDuration();
					}
					else
					{
                        m_fFidgetAnimTimer  = 0.0f;
						m_uFidgetAnimHashID = ASURA_HASH_ID_UNSET;
                        m_uFidgetAnimFlags  = 0;
					}
				}

				m_fFidgetDelayTimer = Asura_Random::GetFloat( m_pxFidgetAnimTemplate->m_fMinFidgetDelay, m_pxFidgetAnimTemplate->m_fMaxFidgetDelay );
			}
		}
	}
}

bool Axon_GS2_AnimSequenceData_Handler::PlayingAnimSequence() const
{
	return ( m_xAnimSequenceData.m_uAnimTemplateSequenceID != uAXON_ANIMATION_SEQUENCE_ID_NONE );
}

bool Axon_GS2_AnimSequenceData_Handler::IsRigidAnimTemplate() const
{
	return ( m_pxAnimTemplate ? m_pxAnimTemplate->GetType() == AXON_ATT_RIGIDMOVEMENT : false );
}

bool Axon_GS2_AnimSequenceData_Handler::AnimShouldTween() const
{
	return ( m_pxAnimTemplateEntry && m_pxAnimTemplateEntry->ShouldTween() );
}

bool Axon_GS2_AnimSequenceData_Handler::AnimShouldLoop() const
{
	return ( ( m_uAnimFlags & AACA_FLAG_LOOPED ) == 0 );
}

bool Axon_GS2_AnimSequenceData_Handler::AnimShouldBeReversed() const
{
	return ( ( m_uAnimFlags & AACA_FLAG_REVERSED ) == 0 );
}

bool Axon_GS2_AnimSequenceData_Handler::AnimShouldBeMirrored() const
{
	return ( ( m_uAnimFlags & AACA_FLAG_MIRRORED ) == 0 );
}

float Axon_GS2_AnimSequenceData_Handler::GetAnimTweenInTime() const
{
	if ( AnimShouldTween() )
	{
		// Already NULL checking in AnimShouldTween().
		return m_pxAnimTemplateEntry->GetTweenTime();
	}
	else
	{
		return 0.0f;
	}
}

bool Axon_GS2_AnimSequenceData_Handler::IsLastAnim() const
{
	return ( !m_pxAnimTemplateEntry || m_pxAnimTemplateEntry->GetNextEntry() == NULL );
}

bool Axon_GS2_AnimSequenceData_Handler::FidgetShouldTween() const
{
	return ( m_pxPrevFidgetAnimTemplateEntry && m_pxPrevFidgetAnimTemplateEntry->ShouldTween() );
}

float Axon_GS2_AnimSequenceData_Handler::GetFidgetTweenInTime() const
{
	return ( m_pxPrevFidgetAnimTemplateEntry ? m_pxPrevFidgetAnimTemplateEntry->GetTweenTime() : 0.0f );
}

void Axon_GS2_AnimSequenceData_Handler::ResetCurrentData()
{
	m_xAnimSequenceData.SetInvalid();
    m_pxAnimTemplate	  = NULL;
	m_pxAnimTemplateEntry = NULL;
	m_uAnimHashID		  = ASURA_HASH_ID_UNSET;
	m_fAnimPST			  = 0.0f;
    m_uAnimFlags	      = 0;
	m_pxFidgetAnimTemplate = NULL;
    m_uFidgetAnimHashID	   = ASURA_HASH_ID_UNSET;
    m_uFidgetAnimFlags	   = 0;
    m_fFidgetDelayTimer	   = 0.0f;
    m_fFidgetAnimTimer	   = 0.0f;
}

//////////////////////////////////////////////////////////////////////////////
//
//
//
//////////////////////////////////////////////////////////////////////////////

#endif // ASURA_USE_AXON
