#ifdef ASURA_USE_AXON

//////////////////////////////////////////////////////////////////////////////
//
// Axon_Gamescene_AnimSequenceData.h
//
// Was the old <project>_Anim_Sequence_Data made by Andrew Weinkove - 22/07/2005
// Animation sequence data. Used by actors to run animation templates.
//
// (AW - 10.08.2004) - When the gamescene system reaches a node with an animation
// template associated with it, it tells the server actor. The server actor will then
// send regular messages to the client actor to inform about the animation template.
// To beat problems of lag, when the game is online there is actually a delay of 0.5 seconds
// before the sequence starts. NJD - This isn't true any more, but can be replaced if necessary.
// The client keeps 2 copies of the anim sequence data, one
// for the currently playing sequence and one for the pending sequence.
// A flag in the actor client update message will inform the client actor if there is no current
// animation sequence.
//
// Axon_GS2_AnimSequenceData_Handler is provided to handle most of the template management
// stuff for each project.
//
// Copyright (c) 2006 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef AXON_GAMESCENE_ANIMSEQUENCEDATA_H___
#define AXON_GAMESCENE_ANIMSEQUENCEDATA_H___ 1

// I N C L U D E S ///////////////////////////////////////////////////////////

//Asura
#include "Core/Asura_Maths.h"

// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////////

class Asura_Message;
class Asura_NetMsg;
class Axon_Gamescene_AnimTemplate;
class Axon_Gamescene_AnimTemplate_Entry;
class Axon_Gamescene_FidgetAnimTemplate;
class Axon_Gamescene_FidgetAnimTemplate_Entry;

//////////////////////////////////////////////////////////////////////////////
//
// Axon_Gamescene_AnimSequenceData
// Note that this message needs compression when co-op is implemented.
//
//////////////////////////////////////////////////////////////////////////////

class Axon_Gamescene_AnimSequenceData
{
public:
    Axon_Gamescene_AnimSequenceData();

    bool IsValid() const { return ( m_fStartTimer != -Asura_Maths::LargeFloat ); }
    void SetInvalid() { m_fStartTimer = -Asura_Maths::LargeFloat; }

	float          m_fStartTimer;
	u_int          m_uAnimTemplateSequenceID;
	Asura_Vector_3 m_xStartPosition;
	Asura_Quat     m_xStartOrientQ;

	#ifdef ASURA_SUPPORT_NETMSG_SYSTEM
	static void RegisterFields( Asura_NetMsg* pxNetMsg );
	static u_char GetNumFields()
	{
		return static_cast<u_char> (4);
	}
	#endif // ASURA_SUPPORT_NETMSG_SYSTEM
};

//////////////////////////////////////////////////////////////////////////////
//
// Axon_GS2_AnimSequenceData_Handler
// Class that can be embedded in a client actor or similar and will handle
// the Axon_Gamescene_AnimSequenceData messages arriving, providing the current
// animation to play and other necessary data.
//
// Call UpdateAnimSequence() every frame.
// PlayingAnimSequence() returns whether there is a valid anim sequence running.
// If so you can get the anim to play and associated data.
// If there is a valid fidget anim hash ID you can play the fidget anim if desired.
// Once finished you should resume the normal anim.
//
//////////////////////////////////////////////////////////////////////////////

class Axon_GS2_AnimSequenceData_Handler
{
public:
	Axon_GS2_AnimSequenceData_Handler();

	// Call HandleMessage or HandleData with the incoming data from the server.
	bool HandleMessage( const Asura_Message* pxMessage );
	void HandleData( const Axon_Gamescene_AnimSequenceData& xAnimSequenceData );
	// Call UpdateAnimSequence once-a-frame before querying.
	void UpdateAnimSequence( void* pMappingData );

	bool		  PlayingAnimSequence() const;
	bool		  IsRigidAnimTemplate() const;
	bool		  AnimChangedThisUpdate() const		  { return ( m_ucAnimChangedThisUpdate > 0 ); }
	Asura_Hash_ID GetAnimHashID() const				  { return m_uAnimHashID; }
	float		  GetAnimParametricStartTime() const  { return m_fAnimPST; }
	u_int		  GetAnimFlags() const				  { return m_uAnimFlags; }
	bool		  AnimShouldTween() const;
	bool		  AnimShouldLoop() const;
	bool		  AnimShouldBeReversed() const;
	bool		  AnimShouldBeMirrored() const;
	float		  GetAnimTweenInTime() const;
	// The server position/orientation the anim started from.
	// Used for rigid animations that should ideally set the pos/ori directly on the client using the animation movement.
	const Asura_Vector_3& GetAnimStartPosition() const { return m_xAnimSequenceData.m_xStartPosition; }
	const Asura_Quat&     GetAnimStartOrientQ() const  { return m_xAnimSequenceData.m_xStartOrientQ; }
	bool		  IsLastAnim() const;

	bool		  FidgetAnimChangedThisUpdate() const { return ( m_ucFidgetAnimChangedThisUpdate > 0 ); }
	Asura_Hash_ID GetFidgetAnimHashID() const		  { return m_uFidgetAnimHashID; }
	u_int		  GetFidgetAnimFlags() const		  { return m_uFidgetAnimFlags; }
	bool		  FidgetShouldTween() const;
	float		  GetFidgetTweenInTime() const;

private:
	void ResetCurrentData();

private:
    u_char m_ucReceivedNewData;
    u_char m_ucAnimChangedThisUpdate;
	u_char m_ucFidgetAnimChangedThisUpdate;

    Axon_Gamescene_AnimSequenceData    m_xNewAnimSequenceData;
    Axon_Gamescene_AnimSequenceData    m_xAnimSequenceData;
    Axon_Gamescene_AnimTemplate*	   m_pxAnimTemplate;
	Axon_Gamescene_AnimTemplate_Entry* m_pxAnimTemplateEntry;
    Asura_Hash_ID	                   m_uAnimHashID;
    float	                           m_fAnimPST;
    u_int	                           m_uAnimFlags;
	Axon_Gamescene_FidgetAnimTemplate*			   m_pxFidgetAnimTemplate;
	const Axon_Gamescene_FidgetAnimTemplate_Entry* m_pxPrevFidgetAnimTemplateEntry;
    Asura_Hash_ID	                               m_uFidgetAnimHashID;
    u_int	                                       m_uFidgetAnimFlags;
    float	                                       m_fFidgetDelayTimer;
    float	                                       m_fFidgetAnimTimer;
};

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

#endif // AXON_GAMESCENE_ANIMSEQUENCEDATA_H___
#endif // ASURA_USE_AXON
