//////////////////////////////////////////////////////////////////////////////
//
// Axon_BehaviourParams_GS2Node.cpp
//
// Copyright (C) 2008 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"

#ifdef ASURA_USE_AXON

// header
#include "Behaviour/BehaviourParamsTypes/Axon_BehaviourParams_GS2Node.h"

// asura
#include "Animation/Asura_Animation.h"
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "Entity/Locations/Asura_ServerEntity_Node.h"
#include "Entity/Locations/Spline/GamesceneSpline/Asura_ServerNode_GS2Spline.h"
#ifdef ASURA_TOOLS
#include "Interface/Asura_Dialog.h"
#endif
#include "Network/Asura_Message.h"

// axon
#include "AnimTemplates/Axon_Gamescene_AnimTemplate_System.h"
#include "Core/Axon_Messages.h"

// tools
#ifdef ASURA_TOOLS
#include "AnimTemplates/Tools_AnimTemplates_Play.h"
#include "AnimTemplates/Tools_Mode_AnimTemplates.h"
#include "Entity/Locations/Tools_Entity_Node.h"
#include "Entity/Locations/OrientedNode/GamesceneNode/Tools_EntityNode_GS2Node.h"
#include "Entity/Locations/Spline/GamesceneSpline/Tools_EntityNode_GS2Spline.h"
#include "Objects/Axon/Tools_Object_Axon_Interface.h"
#endif

//////////////////////////////////////////////////////////////////////////////
//
// Axon_BehaviourParams_GS2Node
//
//////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_TOOLS
TOOLS_DECLARE_SHARED_PARAM( bool, 0xf14567bb, GS2Node_Param_StopHere ); // Asura_GetHashID( "GS2Node_Param_StopHere" )
TOOLS_DECLARE_SHARED_PARAM( Asura_String, 0xadbd80d9, GS2Node_Param_AnimSetString ); // Asura_GetHashID( "GS2Node_Param_AnimSetString" )
TOOLS_DECLARE_SHARED_PARAM( int,          0x13926c27, GS2Node_Param_AnimSetInt );    // Asura_GetHashID( "GS2Node_Param_AnimSetInt" )
TOOLS_DECLARE_SHARED_PARAM( Asura_String, 0x0258eba5, GS2Node_Param_ExitConditionString ); // Asura_GetHashID( "GS2Node_Param_ExitConditionString" )
TOOLS_DECLARE_SHARED_PARAM( int,          0xc56fd4db, GS2Node_Param_ExitConditionInt );    // Asura_GetHashID( "GS2Node_Param_ExitConditionInt" )
TOOLS_DECLARE_SHARED_PARAM( float, 0xd6ce54ab, GS2Node_Param_MinWaitTime ); // Asura_GetHashID( "GS2Node_Param_MinWaitTime" )
TOOLS_DECLARE_SHARED_PARAM( float, 0xff2f21bd, GS2Node_Param_MaxWaitTime ); // Asura_GetHashID( "GS2Node_Param_MaxWaitTime" )
#endif

Axon_BehaviourParams_GS2Node::Axon_BehaviourParams_GS2Node( const Axon_BehaviourParams_GS2Node& xNodeParams )
: PARENT( xNodeParams )
, m_bStopHere( xNodeParams.m_bStopHere )
, m_uAnimSet( xNodeParams.m_uAnimSet )
, m_iExitCondition( xNodeParams.m_iExitCondition )
, m_fMinExitWaitTime( xNodeParams.m_fMinExitWaitTime )
, m_fMaxExitWaitTime( xNodeParams.m_fMaxExitWaitTime )
#ifdef ASURA_TOOLS
, m_pxPreviewAnimation( NULL )
, m_fPreviewTimeoutTime( 0.0f )
#endif
{
}

Axon_BehaviourParams_GS2Node::Axon_BehaviourParams_GS2Node( const u_int uBehaviourType )
: PARENT( AXON_BP_TYPE_GS2_NODE, uBehaviourType )
, m_bStopHere( false )
, m_uAnimSet( uAXON_ANIMATION_SEQUENCE_ID_NONE )
, m_iExitCondition( AXON_GS2_NEC_NONE )
, m_fMinExitWaitTime( 0.0f )
, m_fMaxExitWaitTime( 0.0f )
#ifdef ASURA_TOOLS
, m_pxPreviewAnimation( NULL )
, m_fPreviewTimeoutTime( 0.0f )
#endif
{
}

Axon_BehaviourParams_GS2Node::Axon_BehaviourParams_GS2Node( const u_int uParamsType, const u_int uBehaviourType )
: PARENT( uParamsType, uBehaviourType )
, m_bStopHere( false )
, m_uAnimSet( uAXON_ANIMATION_SEQUENCE_ID_NONE )
, m_iExitCondition( AXON_GS2_NEC_NONE )
, m_fMinExitWaitTime( 0.0f )
, m_fMaxExitWaitTime( 0.0f )
#ifdef ASURA_TOOLS
, m_pxPreviewAnimation( NULL )
, m_fPreviewTimeoutTime( 0.0f )
#endif
{
}

bool Axon_BehaviourParams_GS2Node::IsBaseOrDerivativeOf( const u_int uParamsType ) const
{
	return ( uParamsType == AXON_BP_TYPE_GS2_NODE ) ? true : PARENT::IsBaseOrDerivativeOf( uParamsType );
}

void Axon_BehaviourParams_GS2Node::ReadFromChunkStream( Asura_Chunk_Stream& xStream )
{
	int iVersion;
	xStream >> iVersion;

	PARENT::ReadFromChunkStream( xStream );

	if ( iVersion == CURRENT_CHUNK_VERSION )
	{
		xStream >> m_bStopHere;
		xStream >> m_uAnimSet;
		xStream >> m_iExitCondition;
		xStream >> m_fMinExitWaitTime;
		xStream >> m_fMaxExitWaitTime;
	}
	else
	{
		Asura_Assertf( false, ( "Axon_BehaviourParams_GS2Node::ReadFromChunkStream - unhandled chunk version (%d)", iVersion ) );
	}
}

void Axon_BehaviourParams_GS2Node::WriteToChunkStream( Asura_Chunk_Stream& xStream ) const
{
	xStream << CURRENT_CHUNK_VERSION;

	PARENT::WriteToChunkStream( xStream );

	xStream << m_bStopHere;
	xStream << m_uAnimSet;
	xStream << m_iExitCondition;
	xStream << m_fMinExitWaitTime;
	xStream << m_fMaxExitWaitTime;
}

bool Axon_BehaviourParams_GS2Node::HandleMessage( const Asura_Message* pxMessage )
{
	if ( pxMessage->GetMsgID() == ASURA_MSG_AXON_GS2_SET_NODE_PARAMS )
	{
		const Axon_Msg_GS2_Set_Node_Params* pxSNPMsg = static_cast< const Axon_Msg_GS2_Set_Node_Params* >( pxMessage->GetData() );

		if ( pxSNPMsg->m_iDataValidFlags & Axon_Msg_GS2_Set_Node_Params::STOP_AT_NODE )
		{
			m_bStopHere = pxSNPMsg->m_bStopAtNode;
		}
		if ( pxSNPMsg->m_iDataValidFlags & Axon_Msg_GS2_Set_Node_Params::ANIM_SET )
		{
			m_uAnimSet = pxSNPMsg->m_uAnimSet;
		}
		if ( pxSNPMsg->m_iDataValidFlags & Axon_Msg_GS2_Set_Node_Params::EXIT_CONDITION )
		{
			m_iExitCondition = pxSNPMsg->m_iExitCondition;
		}
		if ( pxSNPMsg->m_iDataValidFlags & Axon_Msg_GS2_Set_Node_Params::MIN_EXIT_WAIT_TIME )
		{
			m_fMinExitWaitTime = pxSNPMsg->m_fMinExitWaitTime;
		}
		if ( pxSNPMsg->m_iDataValidFlags & Axon_Msg_GS2_Set_Node_Params::MAX_EXIT_WAIT_TIME )
		{
			m_fMaxExitWaitTime = pxSNPMsg->m_fMaxExitWaitTime;
		}

		return true;
	}
	else
	{
		return false;
	}
}

#ifdef ASURA_TOOLS
const char* Axon_BehaviourParams_GS2Node::s_pszExitConditionStrings[ AXON_GS2_NEC_NUM_CORE_CONDITIONS ] =
{
	"*none*",
	"timeout",
	"anims finished",
	"next node free"
};

void Axon_BehaviourParams_GS2Node::Render()
{
	PARENT::Render();

	if ( m_pxPreviewAnimation )
	{
		m_pxPreviewAnimation->Render( GetToolsNode()->Position, GetToolsNode()->Orientation, ANIMTIME_USEGAMETIME );
	}
}

void Axon_BehaviourParams_GS2Node::StartPreview()
{
	PARENT::StartPreview();

	if ( !m_bStopHere ) { return; }

	Tools_EntityNode_GS2Node* pxToolsGS2Node = Tools_EntityNode_GS2Node::FindEntity( GetToolsNode()->GetGuid() );
	if ( !pxToolsGS2Node ) { return; }
	pxToolsGS2Node->PreviewSetIsOccupied( true );

	const Asura_String& xSkinName = pxToolsGS2Node->GetOwnerSkinName();
	if ( xSkinName.IsBlank() ) { return; }

	Axon_Gamescene_AnimTemplate* pxAnimationTemplate = Axon_Gamescene_AnimTemplate_System::GetAnimTemplateByID( m_uAnimSet );
	if ( pxAnimationTemplate )
	{
		m_pxPreviewAnimation = new Tools_AnimTemplates_Preview();
		m_pxPreviewAnimation->Initialise();
		m_pxPreviewAnimation->Play( xSkinName, ASURA_HASH_ID_UNSET, ASURA_HASH_ID_UNSET,
			Tools_Object_Axon_Interface::GetInstance()->GetPreviewScoringData(),
			pxAnimationTemplate, pxToolsGS2Node->Position, pxToolsGS2Node->Orientation );
	}

	if ( m_iExitCondition == AXON_GS2_NEC_TIMEOUT )
	{
		m_fPreviewTimeoutTime = Asura_Random::GetFloat( m_fMinExitWaitTime, m_fMaxExitWaitTime );
	}
}

bool Axon_BehaviourParams_GS2Node::UpdatePreview()
{
	if ( !PARENT::UpdatePreview() )
	{
		return false;
	}

	if ( !m_pxPreviewAnimation )
	{
		return false;
	}

	if ( m_iExitCondition == AXON_GS2_NEC_TIMEOUT )
	{
		m_fPreviewTimeoutTime -= Asura_Timers::GetGameTimeSlice();
		return ( m_fPreviewTimeoutTime > 0.0f );
	}
	else if ( m_iExitCondition == AXON_GS2_NEC_ANIMS_FINISHED )
	{
		return m_pxPreviewAnimation->IsPlaying();
	}
	else if ( m_iExitCondition == AXON_GS2_NEC_NEXT_NODE_FREE )
	{
		for ( u_int u = 0; u < GetToolsNode()->GetGameNode()->GetNumberLinks(); ++u )
		{
			const Tools_EntityNode_GS2Spline* pxToolsGS2Spline = Tools_EntityNode_GS2Spline::FindEntity( GetToolsNode()->GetGameNode()->GetLink( u ) );
			if ( !pxToolsGS2Spline ) { return false; }

			if ( pxToolsGS2Spline->PreviewIsOccupied() )
			{
				continue;
			}

			const Tools_EntityNode_GS2Node* pxNextToolsGS2Node = Tools_EntityNode_GS2Node::FindEntity( pxToolsGS2Spline->GetGameGS2Spline()->GetTargetGuid() );
			if ( !pxNextToolsGS2Node ) { return false; }

			if ( pxNextToolsGS2Node->PreviewIsOccupied() || pxNextToolsGS2Node->PreviewHasPendingActors() )
			{
				continue;
			}

			return false;
		}
	}

	return true;
}

void Axon_BehaviourParams_GS2Node::StopPreview()
{
	if ( m_pxPreviewAnimation )
	{
		m_pxPreviewAnimation->Stop();
		delete m_pxPreviewAnimation;
		m_pxPreviewAnimation = NULL;
	}

	Tools_EntityNode_GS2Node* pxToolsGS2Node = Tools_EntityNode_GS2Node::FindEntity( GetToolsNode()->GetGuid() );
	if ( pxToolsGS2Node )
	{
		pxToolsGS2Node->PreviewSetIsOccupied( false );
	}

	PARENT::StopPreview();
}

void Axon_BehaviourParams_GS2Node::AddOverlayControls( Asura_Tabs_Dialog* pxTabsDialog, Asura_Vector_2& xOffset )
{
	PARENT::AddOverlayControls( pxTabsDialog, xOffset );

	Asura_Dialog* pxNewDialog = new Asura_SharedTickbox_Dialog( "Stop Here", xOffset, &g_xParam_GS2Node_Param_StopHere );
	pxNewDialog->SetHelpText( "Actors should stop here when traversing gamescene until their exit conditions are satisfied." );
	pxTabsDialog->AddDialog( pxNewDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	// Add the anim and exit conditions stuff if we should stop.
	if ( m_bStopHere )
	{
		pxNewDialog = new Asura_Function_Dialog( "Anim Set: ", &g_xParam_GS2Node_Param_AnimSetString, xOffset, PickAnimSet, reinterpret_cast< int >( this ) );
		pxTabsDialog->AddDialog( pxNewDialog );
		xOffset.y += Asura_Dialog::DefaultSpacing();

		pxNewDialog = new Asura_Function_Dialog( "Exit Condition: ", &g_xParam_GS2Node_Param_ExitConditionString, xOffset, PickExitCondition, reinterpret_cast< int >( this ) );
		pxNewDialog->SetHelpText( "Pick the condition which will trigger the actor to leave this node." );
		pxTabsDialog->AddDialog( pxNewDialog );
		xOffset.y += Asura_Dialog::DefaultSpacing();

		switch ( m_iExitCondition )
		{
			case AXON_GS2_NEC_TIMEOUT:
			{
				pxNewDialog = new Asura_Numerical_Dialog( "Min Wait Time", xOffset, &g_xParam_GS2Node_Param_MinWaitTime, 0.0f, 999.9f, 0.1f, "%.2f" );
				pxNewDialog->SetHelpText( "The minimum amount of time to wait as part of the exit conditions." );
				pxTabsDialog->AddDialog( pxNewDialog );
				xOffset.y += Asura_Dialog::DefaultSpacing();

				pxNewDialog = new Asura_Numerical_Dialog( "Max Wait Time", xOffset, &g_xParam_GS2Node_Param_MaxWaitTime, 0.0f, 999.9f, 0.1f, "%.2f" );
				pxNewDialog->SetHelpText( "The maximum amount of time to wait as part of the exit conditions." );
				pxTabsDialog->AddDialog( pxNewDialog );
				xOffset.y += Asura_Dialog::DefaultSpacing();
				break;
			}

			case AXON_GS2_NEC_ANIMS_FINISHED:
			case AXON_GS2_NEC_NEXT_NODE_FREE:
			default:
				break;
		}
	}
}

void Axon_BehaviourParams_GS2Node::AddFilesToExportList( List< Asura_String >& xNameList ) const
{
	PARENT::AddFilesToExportList( xNameList );

	// Add the animation files from the anim template to the export list.
	Axon_Gamescene_AnimTemplate* pxTemplate = Axon_Gamescene_AnimTemplate_System::GetAnimTemplateByID( m_uAnimSet );
	if ( pxTemplate )
	{
		pxTemplate->AddFilesToExportList( xNameList );
	}
}

bool Axon_BehaviourParams_GS2Node::HandlesThisMessage( const int iMessageID )
{
	if ( iMessageID == ASURA_MSG_AXON_GS2_SET_NODE_PARAMS )
	{
		return true;
	}
	else
	{
		return PARENT::HandlesThisMessage( iMessageID );
	}
}

bool Axon_BehaviourParams_GS2Node::GetParameter( const Asura_Hash_ID uParamID, Tools_Parameter* pxParam ) const
{
	switch ( uParamID )
	{
		case g_uHash_GS2Node_Param_StopHere:
			pxParam->SetBool( m_bStopHere );
			return true;

		case g_uHash_GS2Node_Param_AnimSetString:
		{
			if ( m_uAnimSet == uAXON_ANIMATION_SEQUENCE_ID_NONE )
			{
				pxParam->SetString( "*none*" );
			}
			else
			{
				const Axon_Gamescene_AnimTemplate* pxTemplate = Axon_Gamescene_AnimTemplate_System::GetAnimTemplateByID( m_uAnimSet );
				if ( pxTemplate )
				{
					pxParam->SetString( pxTemplate->m_xTemplateName );
				}
				else
				{
					pxParam->SetString( "NOT FOUND" );
				}
			}
			return true;
		}

		case g_uHash_GS2Node_Param_AnimSetInt:
		case g_uHash_GS2Node_Param_ExitConditionInt:
			// This is only ever set, not got, because of the way shared function dialogues work.
			return true;

		case g_uHash_GS2Node_Param_ExitConditionString:
			pxParam->SetString( s_pszExitConditionStrings[ m_iExitCondition ] );
			return true;

		case g_uHash_GS2Node_Param_MinWaitTime:
			pxParam->SetFloat( m_fMinExitWaitTime );
			return true;

		case g_uHash_GS2Node_Param_MaxWaitTime:
			pxParam->SetFloat( m_fMaxExitWaitTime );
			return true;

		default:
			return PARENT::GetParameter( uParamID, pxParam );
	}
}

bool Axon_BehaviourParams_GS2Node::SetParameter( const Asura_Hash_ID uParamID, const Tools_Parameter* pxParam )
{
	switch ( uParamID )
	{
		case g_uHash_GS2Node_Param_StopHere:
			m_bStopHere = pxParam->GetBool();
			GetToolsNode()->SetEntityModified();
			if ( GetToolsNode()->IsSelected() )
			{
				GetToolsNode()->RequestRefreshOverlay();
			}
			return true;

		case g_uHash_GS2Node_Param_AnimSetString:
		case g_uHash_GS2Node_Param_ExitConditionString:
			// This is only ever got, not set, because of the way shared function dialogues work.
			return true;

		case g_uHash_GS2Node_Param_AnimSetInt:
			m_uAnimSet = pxParam->GetInt();
			GetToolsNode()->SetEntityModified();
			if ( GetToolsNode()->IsSelected() )
			{
				GetToolsNode()->RequestRefreshOverlay();
			}
			return true;

		case g_uHash_GS2Node_Param_ExitConditionInt:
			m_iExitCondition = pxParam->GetInt();
			GetToolsNode()->SetEntityModified();
			if ( GetToolsNode()->IsSelected() )
			{
				GetToolsNode()->RequestRefreshOverlay();
			}
			return true;

		case g_uHash_GS2Node_Param_MinWaitTime:
			m_fMinExitWaitTime = pxParam->GetFloat();
			GetToolsNode()->SetEntityModified();
			return true;

		case g_uHash_GS2Node_Param_MaxWaitTime:
			m_fMaxExitWaitTime = pxParam->GetFloat();
			GetToolsNode()->SetEntityModified();
			return true;

		default:
			return PARENT::SetParameter( uParamID, pxParam );
	}
}

bool Axon_BehaviourParams_GS2Node::Verify( const bool bAutomatic )
{
	bool bOk = PARENT::Verify( bAutomatic );

	// Check if the anim set still exists.
	if ( m_uAnimSet != uAXON_ANIMATION_SEQUENCE_ID_NONE )
	{
		if ( !Axon_Gamescene_AnimTemplate_System::GetAnimTemplateByID( m_uAnimSet ) )
		{
			bOk = false;
			Tools_MessageLog::AddMessage( Tools_MessageLog::VERIFICATION, "Gamescene", new Tools_MessageLogAction_GotoEntity( GetToolsNode()->GetGuid() ),
				"GS2 node %u cannot find its anim set.", GetToolsNode()->GetGuid() );
		}
	}

	if ( m_iExitCondition == AXON_GS2_NEC_ANIMS_FINISHED )
	{
		// Check we have an anim set.
		if ( m_uAnimSet == uAXON_ANIMATION_SEQUENCE_ID_NONE )
		{
			bOk = false;
			Tools_MessageLog::AddMessage( Tools_MessageLog::VERIFICATION, "Gamescene", new Tools_MessageLogAction_GotoEntity( GetToolsNode()->GetGuid() ),
				"GS2 node %u set to exit when anims finished but has no anim set.", GetToolsNode()->GetGuid() );
		}

		// Check that the anim template doesn't contain a looping anim.
		const Axon_Gamescene_AnimTemplate* pxAnimationTemplate = Axon_Gamescene_AnimTemplate_System::GetAnimTemplateByID( m_uAnimSet );
		if ( pxAnimationTemplate && !pxAnimationTemplate->WillFinish() )
		{
			Tools_MessageLog::AddMessage( Tools_MessageLog::VERIFICATION, "Gamescene", new Tools_MessageLogAction_GotoEntity( GetToolsNode()->GetGuid() ),
				"GS2 node %u set to exit when anims finished but anim set will never finish.", GetToolsNode()->GetGuid() );
		}
	}
	else if ( m_iExitCondition == AXON_GS2_NEC_TIMEOUT )
	{
		// Check the timeout times are reasonable.
		if ( m_fMinExitWaitTime > m_fMaxExitWaitTime )
		{
			bOk = false;
			Tools_MessageLog::AddMessage( Tools_MessageLog::VERIFICATION, "Gamescene", new Tools_MessageLogAction_GotoEntity( GetToolsNode()->GetGuid() ),
				"GS2 node %u has corrupt exit wait times.", GetToolsNode()->GetGuid() );
		}
	}
	else if ( m_iExitCondition == AXON_GS2_NEC_NEXT_NODE_FREE )
	{
		// Check we have a next node.
		if ( GetToolsNode()->GetGameNode()->GetNumberLinks() == 0 )
		{
			bOk = false;
			Tools_MessageLog::AddMessage( Tools_MessageLog::VERIFICATION, "Gamescene", new Tools_MessageLogAction_GotoEntity( GetToolsNode()->GetGuid() ),
				"GS2 node %u set to exit when next node free but does not have a next node.", GetToolsNode()->GetGuid() );
		}
	}

	// Check 'smooth over node' is used correctly.
	if ( GetToolsNode()->GetGameNode()->IsSmoothOverNode() )
	{
		// Check we're not set to stop here.
		if ( m_bStopHere )
		{
			bOk = false;
			Tools_MessageLog::AddMessage( Tools_MessageLog::VERIFICATION, "Gamescene", new Tools_MessageLogAction_GotoEntity( GetToolsNode()->GetGuid() ),
				"GS2 node %u has 'smooth over node' ticked and 'stop here' ticked. These are mutually exclusive.", GetToolsNode()->GetGuid() );
		}
	}

	return bOk;
}

Axon_BehaviourParams* Axon_BehaviourParams_GS2Node::Clone() const
{
	return new Axon_BehaviourParams_GS2Node( *this );
}

void Axon_BehaviourParams_GS2Node::PickAnimSet( int iThis )
{
	Axon_BehaviourParams_GS2Node* pxThis = reinterpret_cast< Axon_BehaviourParams_GS2Node* >( iThis );

	const u_int uTypeBitfield = (1 << AXON_ATT_STATIC);
	u_int uSequenceID;
	if ( Tools_AnimTemplates::ChooseAnimTemplate( uSequenceID, uTypeBitfield, uAXON_ANIMATION_SEQUENCE_ID_NONE, pxThis->m_uAnimSet, true ) )
	{
		// Setting this will cause SetParameter to be called with this parameter hash on all selected objects.
		g_xParam_GS2Node_Param_AnimSetInt.SetValue( uSequenceID );
	}
}

void Axon_BehaviourParams_GS2Node::PickExitCondition( int iThis )
{
	Axon_BehaviourParams_GS2Node* pxThis = reinterpret_cast< Axon_BehaviourParams_GS2Node* >( iThis );

	ListSelection_Overlay xBehaviourList( "Pick The Exit Condition:" );
	for ( int i = 0; i < AXON_GS2_NEC_NUM_CORE_CONDITIONS; ++i )
	{
		xBehaviourList.AddItem( s_pszExitConditionStrings[ i ], i );
	}
	xBehaviourList.SetInitialSelection( pxThis->m_iExitCondition );
	if ( xBehaviourList.DoModal() )
	{
		// Setting this will cause SetParameter to be called with this parameter hash on all selected objects.
		g_xParam_GS2Node_Param_ExitConditionInt.SetValue( xBehaviourList.GetSelectionData() );
	}
}
#endif

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

#endif // ASURA_USE_AXON
