#ifdef ASURA_USE_AXON
//////////////////////////////////////////////////////////////////////////////
//
// Axon_Behaviour_GS2Actor.h
//
// Copyright (C) 2008 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __AXON_BEHAVIOUR_GS2ACTOR_H__
#define __AXON_BEHAVIOUR_GS2ACTOR_H__ 1

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// asura
#include "Core/Asura_LinkList.h"
#include "Core/Asura_Maths.h"
#include "Network/Asura_Guid.h"

// axon
#include "Behaviour/Axon_Behaviour.h"
#include "Behaviour/Axon_BehaviourParams.h"

// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////////

class Asura_ServerEntity_PhysicalObject;
class Asura_ServerNode_GS2Attractor;
class Asura_ServerNode_GS2Node;
class Asura_ServerNode_GS2Spline;
class Axon_Behaviour_GS2Interface;
class Axon_Gamescene_AnimTemplate;
class Axon_Gamescene_AnimSequenceData;

//////////////////////////////////////////////////////////////////////////////
//
// Axon_Behaviour_GS2Actor
//
//////////////////////////////////////////////////////////////////////////////

class Axon_Behaviour_GS2Actor : public Axon_Behaviour
{
	typedef Axon_Behaviour PARENT;

public:
	Axon_Behaviour_GS2Actor( Axon_Brain* pxBrain );

	// If our owner is set to start on a GS2 node we supply that node guid here.
	// When this GS2 behaviour is created in tools it just remembers which node it should start at.
	// In game, at PostLoadInitialse time it teleports itself to its initial node.
	// This avoids node arrival/departure messages and anim template messages being sent at tools time.
	void SetInitialNodeGuid( const Asura_Guid uGuid ) { m_uInitialNodeGuid = uGuid; }

	Asura_Guid GetCurrentEntityGuid() const { return m_uCurrentEntityGuid; }

	virtual void SetOwnerPhysicsToExtControl();
	virtual void ResetOwnerPhysicsFromExtControl();
	virtual void EnableOwnerPhysicsCollisions( const bool bEnable );

	virtual void GetOwnerPosition( Asura_Vector_3& xPosition ) const;
	// This should test for intersection with other physics objects if necessary and only set the position is the location is free.
	// Returns true if the position was successfully set.
	virtual bool SetOwnerPosition( const Asura_Vector_3& xPosition, bool bShouldIgnoreCollisions = false );
	virtual void GetOwnerOrientation( Asura_Matrix_3x3& xOrientation ) const;
	virtual void SetOwnerOrientation( const Asura_Matrix_3x3& xOrientation );
	virtual void TeleportOwnerToPosOri( const Asura_Vector_3& xPosition, const Asura_Matrix_3x3& xOrientation );
	virtual bool GetOwnerAnimationVelocity( Asura_Vector_3& xVelocity ) const;
	virtual bool GetOwnerAnimationAngularVelocity( Asura_Vector_3& xVelocity, Asura_Angle& fAngle ) const;
	virtual bool GetOwnerAnimationMovementOffset( Asura_Vector_3& xPosition, Asura_Quat& xQOrient ) const;

	u_int GetCurrentAnimSet() const { return m_uCurrentAnimSet; }
	float GetAnimSetDuration( const u_int uAnimSet );
	void PlayAnimSet( const u_int uAnimSet );
	bool AnimSetIsFinished( const u_int uAnimSet );
	void StopAnimSet();

	struct GetToBehaviourParams
	{
		enum Contents
		{
			POSITION    = 0x00000001,
			ORIENTATION = 0x00000002,
			NODE		= 0x00000004,
			LINK		= 0x00000008,
			ATTRACTOR	= 0x00000010
		};
		u_int						   m_uContentsFlag;
		Asura_Vector_3				   m_xPosition;
		Asura_Matrix_3x3			   m_xOrientation;
		Asura_ServerNode_GS2Node*      m_pxNode;
        Asura_ServerNode_GS2Spline*	   m_pxLink;
		Asura_ServerNode_GS2Attractor* m_pxAttractor;

		GetToBehaviourParams()
		: m_uContentsFlag( 0 )
		, m_xPosition( Asura_Maths::ZeroVector3 )
		, m_xOrientation( Asura_Maths::IdentityMatrix3x3 )
		, m_pxNode( NULL )
		, m_pxLink( NULL )
		, m_pxAttractor( NULL )
		{}

		void SetPosition( const Asura_Vector_3& xPosition )			{ m_xPosition	 = xPosition;	 SetContains( POSITION ); }
		void SetOrientation( const Asura_Matrix_3x3& xOrientation ) { m_xOrientation = xOrientation; SetContains( ORIENTATION ); }
		void SetNode( Asura_ServerNode_GS2Node* pxNode )			{ m_pxNode		 = pxNode;		 SetContains( NODE ); }
		void SetLink( Asura_ServerNode_GS2Spline* pxLink )			{ m_pxLink		 = pxLink;		 SetContains( LINK ); }
		void SetAttractor( Asura_ServerNode_GS2Attractor* pxAtt )	{ m_pxAttractor  = pxAtt;		 SetContains( ATTRACTOR ); }

		void SetContains( Contents eContents ) { m_uContentsFlag |= eContents; }
		bool Contains( Contents eContents ) const { return ( ( m_uContentsFlag & eContents ) != 0 ); }
	};
	virtual Axon_Behaviour* CreateGetToBehaviour( const GetToBehaviourParams* pxParams = NULL ) = 0;
	virtual void			DestroyGetToBehaviour( Axon_Behaviour* pxBehaviour ) = 0;

	// Data to help with link choice in GS2 object behaviours.
	const Asura_LinkList< Asura_Guid >& GetLinksToTake() const { return m_xLinksToTake; }
	const Asura_LinkList< Asura_Guid >& GetRoute() const { return m_xRoute; }
	void IncrementAlongRoute() { m_xRoute.PopFront(); }
	void ClearRoute() { m_xRoute.Clear(); }
	void AppendRoute( const Asura_LinkList< Asura_Guid >& xRoute );

	static bool FollowSurfaceTest( const Asura_Vector_3& xOrigin, const Asura_Vector_3& xDirection, float& fDistanceToIntersection );

protected:
	virtual bool HandleMessage( const Asura_Message* pxMessage );
	virtual void ReadFromChunkStream( Asura_Chunk_Stream& xStream );
	virtual void WriteToChunkStream( Asura_Chunk_Stream& xStream ) const;
	virtual bool Activate();
	virtual void Deactivate();
	virtual void PostLoadInitialise();
	virtual void PerFrameUpdate( const float fTimeslice );

	Asura_ServerEntity_PhysicalObject* GetOwnerPhyObj() const { return m_pxOwnerPhyObj; }

	enum LinkChoiceResult
	{
		LCR_NO_LINKS_EXIST,
		LCR_NO_LINKS_ACTIVE,
		LCR_LINK_CHOSEN,
		LCR_ERROR
	};
	virtual LinkChoiceResult ChooseLinkToTake( const Asura_Guid uSourceNodeGuid, Asura_Guid& uLinkGuid );

	// Provides data that will be passed to the anim package mapping functions.
	virtual void* GetAnimMappingData() { return NULL; }

	int GetCurrentState() const { return m_iState; }
	bool ShouldSkipNextNode() const { return m_bSkipNextNode; }
	bool ShouldSkipAllNodes() const { return m_bSkipAllNodes; }
	const Asura_LinkList< Asura_Guid >& GetNodesToSkip() const { return m_xNodesToSkip; }

	void SetSkipNextNode( bool bSkip ) { m_bSkipNextNode = bSkip; }
	void SetSkipAllNodes( bool bSkip ) { m_bSkipAllNodes = bSkip; }

#ifndef ASURA_NO_DEBUG_OUTPUT
	virtual const char* GetStateString() const;
#endif

	enum State
	{
		UNSET,
		MOVING_TO_ATTRACTOR,
		AT_NODE,
		ON_LINK,
		FINISHED,
		NUM_STATES
	};

	void SwitchToState( const int iNextState, const Asura_Guid uNextEntityGuid = ASURA_GUID_UNREGISTERED );
	virtual void EnterState( const int iState, const Asura_Guid uNextEntityGuid );
	virtual void ExitState( const int iState );

private:
	void UpdateStateMovingToAttractor();
	void UpdateStateAtNode();
	void UpdateStateOnLink();

	// Create the behaviours necessary using the supplied parameters.
	Axon_Behaviour* CreateBehaviours( const Axon_BehaviourParams* pxBehaviourParams );

	Axon_Behaviour_GS2Interface* GetInterface( Axon_Behaviour* pxBehaviour );

	void ActionPendingAnimStateCommand();
	void StopAnimTemplate();
	void StartAnimTemplate( Axon_Gamescene_AnimTemplate* pxAnimationTemplate );
	void RestartAnimTemplate( Axon_Gamescene_AnimTemplate* pxAnimationTemplate );
	void SendAnimationSequenceData( const Axon_Gamescene_AnimSequenceData& xSequenceData ) const;

private:
	Asura_ServerEntity_PhysicalObject* m_pxOwnerPhyObj;
	int		   m_iState;
	bool       m_bLeaveNode;
	float      m_fRemainingTimeslice;
	// The guid of our initial node, if the owner is set to start on GS2.
	Asura_Guid m_uInitialNodeGuid;
	Asura_Guid m_uCurrentEntityGuid;

	u_int						 m_uCurrentAnimSet;
	float						 m_fAnimSequenceStartTime;
	Axon_Gamescene_AnimTemplate* m_pxCurrentAnimationTemplate;
	// We remember the last anim set command and execute it at the end of the update.
	// This is to stop unnecessary anim commands being sent and smooth out transitions over non-stopping nodes.
	enum PendingAnimSetCmd { NO_CMD_PENDING, PLAY_ANIM_SET, STOP_ANIM_SET, NUM_ANIM_SET_CMDS };
	int							 m_iPendingAnimSetCmd;
	u_int						 m_uPendingAnimSet;

	Asura_LinkList< Asura_Guid > m_xLinksToTake;
	bool						 m_bSkipNextNode;
	bool						 m_bSkipAllNodes;
	Asura_LinkList< Asura_Guid > m_xNodesToSkip;

	// A route is a list of link guids.
	Asura_LinkList< Asura_Guid > m_xRoute;

#ifndef ASURA_NO_DEBUG_OUTPUT
	static const char* s_pszStateStrings[ NUM_STATES ];
#endif

	static const u_int uCURRENT_CHUNK_VERSION = 5;
};

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

#endif //__AXON_BEHAVIOUR_GS2ACTOR_H__
#endif // ASURA_USE_AXON
