#ifdef ASURA_USE_AXON
//////////////////////////////////////////////////////////////////////////////
//
// Axon_Behaviour_GS2Node.h
//
// Copyright (C) 2008 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __AXON_BEHAVIOUR_GS2NODE_H__
#define __AXON_BEHAVIOUR_GS2NODE_H__ 1

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// axon
#include "Behaviour/BehaviourTypes/Axon_Behaviour_GS2Interface.h"

// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////////

class Axon_Brain;

//////////////////////////////////////////////////////////////////////////////
//
// Axon_GS2_NodeExitConditions
//
//////////////////////////////////////////////////////////////////////////////

// NOTE:
// These values are saved out to chunk buffers, so do not reorder or change them.
enum Axon_GS2_NodeExitConditions
{
	AXON_GS2_NEC_NONE					= 0,

	AXON_GS2_NEC_TIMEOUT				= 1,
	AXON_GS2_NEC_ANIMS_FINISHED			= 2,
	AXON_GS2_NEC_NEXT_NODE_FREE			= 3,

	AXON_GS2_NEC_NUM_CORE_CONDITIONS,

	AXON_GS2_NEC_FIRST_PROJECT_SPECIFIC = 1000
};

//////////////////////////////////////////////////////////////////////////////
//
// Axon_Behaviour_GS2Node
//
//////////////////////////////////////////////////////////////////////////////

class Axon_Behaviour_GS2Node : public Axon_Behaviour_GS2Interface
{
	typedef Axon_Behaviour_GS2Interface PARENT;

public:
	Axon_Behaviour_GS2Node( Axon_Brain* pxBrain );
	Axon_Behaviour_GS2Node( const u_int uBehaviourType, Axon_Brain* pxBrain );

	virtual bool       HasNextGuidControl()			{ return ( m_iExitConditions == AXON_GS2_NEC_NEXT_NODE_FREE ); }
	virtual Asura_Guid GetNextGamesceneEntityGuid() { return m_uLinkToTake; }
	virtual float      GetRemainingTimeslice()		{ return m_fRemainingTimeslice; }

protected:
	virtual void ReadFromChunkStream( Asura_Chunk_Stream& xStream );
	virtual void WriteToChunkStream( Asura_Chunk_Stream& xStream ) const;
	virtual void HandleParams( const Axon_BehaviourParams* pxParams );
	virtual bool HandleMessage( const Asura_Message* pxMessage );
	virtual bool Activate();
	virtual void Deactivate();
	virtual void PerFrameUpdate( const float fTimeslice );

	// Returns success or failure.
	virtual bool InitialiseExitConditions();
	// Returns whether the exit conditions have been satisfied or not.
	virtual bool UpdateExitConditions( const float fTimeslice );
	virtual void ShutdownExitConditions();

	int GetExitConditions() const { return m_iExitConditions; }

	enum State
	{
		UNSET,
		WAITING,
		FINISHED,
		NUM_STATES
	};

	void SwitchToState( const State iState );
	bool GetLinkToNextFreeNode( Asura_Guid& uNextFreeNodeGuid ) const;

private:
	int			m_iState;
	bool        m_bStopHere;
	u_int		m_uAnimSet;
	int			m_iExitConditions;
	float		m_fExitWaitTime;
	Asura_Guid	m_uLinkToTake;
	float		m_fRemainingTimeslice;

	enum { CURRENT_CHUNK_VERSION = 0 };
};

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

#endif //__AXON_BEHAVIOUR_GS2NODE_H__
#endif // ASURA_USE_AXON
