#ifdef ASURA_USE_NEW_NAV_STUFF

//////////////////////////////////////////////////////////////////////////////
//
// Axon_Behaviour_LookAt.h
//
// Copyright (C) 2009 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __AXON_BEHAVIOUR_LOOKAT_H__
#define __AXON_BEHAVIOUR_LOOKAT_H__ 1

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// asura
#include "Entity/PhysicalObject/Asura_ServerEntity_PhysicalObject.h"
#include "Network/Asura_Guid.h"

// axon
#include "Behaviour/Axon_Behaviour.h"

// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
//
// Axon_Behaviour_LookAt
//
//////////////////////////////////////////////////////////////////////////////

class Axon_Behaviour_LookAt : public Axon_Behaviour
{
	typedef Axon_Behaviour PARENT;

public:
	Axon_Behaviour_LookAt( Axon_Brain* pxBrain );

	virtual void ReadFromChunkStream( Asura_Chunk_Stream& xStream );
	virtual void WriteToChunkStream( Asura_Chunk_Stream& xStream ) const;

	void SetLookAtGuid( const Asura_Guid uLookAtGuid );
	void SetLookAtMarkerHash( const Asura_Hash_ID uMarkerHash );
	void SetMaxLookAtRange( const float fRange ) { m_fMaxLookAtRange = fRange; }
	void SetUseLOS( const bool bUseLOS ) { m_bUseLOS = bUseLOS; }

	Asura_Guid GetLookAtGuid() const { return m_uLookAtGuid; }
	Asura_Hash_ID GetLookAtMarkerHash() const { return m_uLookAtMarkerHash; }
	float GetMaxLookAtRange() const { return m_fMaxLookAtRange; }
	bool GetUseLOS() const { return m_bUseLOS; }
	bool GetInRange() const { return m_bInRange; }
	const Asura_Vector_3& GetLookAtPosition() const { return m_xLookAtPosition; }

protected:
	enum State
	{
		WAITING_FOR_LOOK_AT,
		LOOKING_AT,
		SEARCHING_FOR_LOOK_AT_TARGET
	};

	virtual void Deactivate();
	virtual void PostLoadInitialise();
	virtual void Update( const float fTimeslice );
	
	virtual bool IsLookAtPositionInLOS();
	virtual void OnNewLookAtGuid();
	virtual void SwitchToState( int iNextState );
	virtual void OwnerLookAtPosition();

	const Asura_ServerEntity_PhysicalObject* GetLookAtPhyObj() const { return m_pxLookAtPhyObj; }
	const Asura_ServerEntity_PhysicalObject* GetOwnerPhyObj() const { return m_pxOwnerPhyObj; }

#ifndef ASURA_NO_DEBUG_OUTPUT
	virtual void RenderDebugInfo();
#endif

private:
	bool UpdateLookAtPhyObjPointer();
	bool UpdateOwnerPhyObjPtr();
	void UpdateLookAtPositionAndRange();

private:

	union 
	{
		u_int m_uLookAtFlags;
		struct
		{
			u_int	m_bUseLOS			: 1,
					m_bInRange			: 1,
					LOOKAT_FLAGS_UNUSED	: 30;
		};
	};

	Asura_Guid		m_uLookAtGuid;
	Asura_Hash_ID	m_uLookAtMarkerHash;
	int				m_iState;
	float			m_fWaitTimer;
	float			m_fMaxLookAtRange;
	Asura_Vector_3	m_xLookAtPosition;
	Asura_ServerEntity_PhysicalObject* m_pxLookAtPhyObj;
	Asura_ServerEntity_PhysicalObject* m_pxOwnerPhyObj;

	static const u_int uCURRENT_CHUNK_VERSION = 0;
};

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

#endif //__AXON_BEHAVIOUR_LOOKAT_H__
#endif // #ifdef ASURA_USE_NEW_NAV_STUFF
