#ifdef ASURA_USE_NEW_NAV_STUFF

//////////////////////////////////////////////////////////////////////////////
//
// Axon_Behaviour_Movement.h
//
// Copyright (C) 2009 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __AXON_BEHAVIOUR_MOVEMENT_H__
#define __AXON_BEHAVIOUR_MOVEMENT_H__ 1

// I N C L U D E S ///////////////////////////////////////////////////////////

// asura
#include "AI/Asura_Nav_Types.h"
#include "Core/Asura_Maths.h"

// axon
#include "Behaviour/Axon_Behaviour.h"

// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////////

class Asura_ServerEntity_PhysicalObject;
class Axon_Brain;

//////////////////////////////////////////////////////////////////////////////
//
// Axon_Behaviour_Movement
// Inherit from this behaviour and implement any versions you need project-side
// to supply movement for your different project actor types.
// Instances of this behaviour will be created by the various core Axon behaviours
// to provide them with movement.
// Movement should be as simple as calculating the movement vector and then moving
// along it. Project-side collision avoidance will most likely be needed.
// The path can be queried directly for more detailed information.
//
//////////////////////////////////////////////////////////////////////////////

class Axon_Behaviour_Movement : public Axon_Behaviour
{
	typedef Axon_Behaviour PARENT;

public:
	Axon_Behaviour_Movement( Axon_Brain* pxBrain );

	// Path find and traversal interface.
	bool FindPathTo( const Asura_Vector_3& xDestPos );
	bool FindPathAwayFrom( const Asura_Vector_3& xAvoidPos, const float fAvoidDistance );
	bool FindPath( Asura_Navigation::AStarFunctor& xFunctor );
	Asura_Navigation::Path& GetPath() { return m_xPath; }

	virtual bool StartPathTraversal() { return true; }
	// Speed varies between 0 and 1. Projects should map this to whatever movement speed or anims they require.
	virtual void SetSpeed( const float fSpeed ) {}
	enum PathTraversalState
	{
		PTS_MOVING,
		PTS_IN_ERROR,
		PTS_AT_DESTINATION
	};
	virtual PathTraversalState GetPathTraversalState() { return PTS_AT_DESTINATION; }
	virtual bool StopPathTraversal() { return true; }

	// Look interface.
	void LookInDirection( const Asura_Vector_3& xDirection );
	void LookAtPosition( const Asura_Vector_3& xPosition );

	// Get/Set interface.
	virtual const Asura_Navigation::TestParams& GetTestParams() const { return Asura_Navigation::s_xStandardTParams; }
	virtual const Asura_Navigation::APParams&   GetAPParams()   const { return Asura_Navigation::s_xStandardAPParams; }

	virtual void TeleportOwnerTo( const Asura_Vector_3& xPosition, const Asura_Matrix_3x3& xOrientation );
	virtual void SetOwnerPosition( const Asura_Vector_3& xPosition );
	virtual const Asura_Vector_3& GetOwnerPosition() const;
	virtual void SetOwnerOrientation( const Asura_Matrix_3x3& xOrientation );
	virtual const Asura_Matrix_3x3& GetOwnerOrientation() const;

protected:
	virtual void Deactivate();

	virtual void OnPathChanged();

	bool CalculateMovementVector( Asura_Vector_3& xVector, bool& bAtDestination );

private:
	Asura_ServerEntity_PhysicalObject* m_pxOwnerPhyObj;
	Asura_Navigation::Path m_xPath;
};

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

#endif //__AXON_BEHAVIOUR_MOVEMENT_H__

#endif // #ifdef ASURA_USE_NEW_NAV_STUFF
