//////////////////////////////////////////////////////////////////////////////
//
// Axon_Sound_Registry.cpp
//
// Created by NJD - 31 August 2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"

#ifdef ASURA_USE_AXON

// header
#include "Core/Axon_Sound_Registry.h"

// asura
#include "Core/Asura_Maths.h"
#include "Core/Asura_Mempool.h"
#include "Core/Asura_Timers.h"
#include "Debugging/Asura_Assert.h"
#include "Network/Asura_Guid.h"

//////////////////////////////////////////////////////////////////////////////
//
// Axon_Sound_Registry
//
//////////////////////////////////////////////////////////////////////////////

int   Axon_Sound_Registry::s_iMempoolPageSize = 18;
float Axon_Sound_Registry::s_fSoundDuration   = 2.0f;

Axon_Sound_Registry::Axon_Sound_Registry()
: m_pxEntryMempool( NULL )
, m_pxListEntryMempool( NULL )
, m_xEntryList()
{
}

void Axon_Sound_Registry::Initialise()
{
    m_pxEntryMempool     = new Asura_ResizingMempool( s_iMempoolPageSize, sizeof( SoundEntry ), __FILE__, __LINE__ );
    m_pxListEntryMempool = new Asura_ResizingMempool( s_iMempoolPageSize, sizeof( m_xEntryList.GetMemoryElementSize() ), __FILE__, __LINE__ );
    Asura_Assert( m_pxEntryMempool && m_pxListEntryMempool, "Axon_Sound_Registry::Initialise - could not allocate both mempools" );
}

void Axon_Sound_Registry::Update()
{
    // Age the currently registered sounds. Cull any that are too old.
    const float fTimeslice = Asura_Timers::GetGameTimeSlice();

    Asura_List_It< SoundEntry > xListIt( &m_xEntryList );
    while ( !xListIt.Done() )
    {
        SoundEntry* pxEntry = xListIt.GetCurrent();

        pxEntry->m_fLifeTime -= fTimeslice;
        if ( pxEntry->m_fLifeTime < 0.0f )
        {
            xListIt.DeleteCurrent();
            m_pxEntryMempool->Deallocate( pxEntry );
        }
        else
        {
            xListIt.Next();
        }
    }
}

void Axon_Sound_Registry::Shutdown()
{
    m_xEntryList.ClearList();
    delete m_pxListEntryMempool;
    m_pxListEntryMempool = NULL;
    delete m_pxEntryMempool;
    m_pxListEntryMempool = NULL;
}

void Axon_Sound_Registry::Clear()
{
	Asura_List_It< SoundEntry > xListIt( &m_xEntryList );

	while ( !xListIt.Done() )
	{
		SoundEntry* pxEntry = xListIt.GetCurrent();

		xListIt.DeleteCurrent();
		m_pxEntryMempool->Deallocate( pxEntry );
	}
}

void Axon_Sound_Registry::RegisterSound( const Asura_Guid uSourceGuid,
                                         const u_int uClassification,
                                         const Asura_Vector_3& xPosition,
                                         const float fRange,
                                         const u_int uSoundType,
										 const u_int uProjectData /*= 0*/ )
{
    SoundEntry* pxEntry = static_cast< SoundEntry* >( m_pxEntryMempool->Allocate() );
    if ( pxEntry )
    {
        pxEntry->m_uSourceGuid     = uSourceGuid;
        pxEntry->m_uClassification = uClassification;
        pxEntry->m_xPosition       = xPosition;
        pxEntry->m_fRange          = fRange;
        pxEntry->m_uSoundType      = uSoundType;
        pxEntry->m_fLifeTime       = s_fSoundDuration;
		pxEntry->m_uProjectData	   = uProjectData;

        m_xEntryList.AddToList( pxEntry );
    }
    else
    {
        Asura_Assert( false, "Axon_Sound_Registry::RegisterSound - could not allocate memory" );
    }
}

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

#endif
