#ifdef ASURA_USE_AXON
//////////////////////////////////////////////////////////////////////////////
//
// Axon_Sound_Registry.h
//
// Created by NJD - 31 August 2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __AXON_SOUND_REGISTRY_H__
#define __AXON_SOUND_REGISTRY_H__ 1

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// asura
#include "Core/Asura_List.h"
#include "Core/Asura_Maths.h"
#include "Network/Asura_Guid.h"

//////////////////////////////////////////////////////////////////////////////
//
// Axon_Sound_Type
//
//////////////////////////////////////////////////////////////////////////////

enum Axon_Sound_Type
{
	AXON_SOUND_TYPE_NO_TYPE,
    AXON_SOUND_TYPE_FOOTSTEP,
    AXON_SOUND_TYPE_GUNFIRE,
    AXON_SOUND_TYPE_RICOCHET,
    AXON_SOUND_TYPE_EXPLOSION,
    AXON_SOUND_TYPE_FIRST_PROJECT_SPECIFIC = 0x00010000
};

//////////////////////////////////////////////////////////////////////////////
//
// Axon_Sound_Registry
//
//////////////////////////////////////////////////////////////////////////////

class Axon_Sound_Registry
{
public:
    struct SoundEntry
    {
        Asura_Guid     m_uSourceGuid;
        u_int          m_uClassification;
        Asura_Vector_3 m_xPosition;
        float          m_fRange;
        u_int          m_uSoundType;
        float          m_fLifeTime;
		u_int		   m_uProjectData;
    };

    Axon_Sound_Registry();
    virtual ~Axon_Sound_Registry() {}

    virtual void Initialise();
    virtual void Update();
    virtual void Shutdown();

    virtual void RegisterSound( const Asura_Guid uSourceGuid,
                                const u_int uClassification,
                                const Asura_Vector_3& xPosition,
                                const float fRange,
                                const u_int uSoundType,
								const u_int uProjectData = 0 );

			void Clear();

    static void SetMempoolPageSize( const int i )         { s_iMempoolPageSize = i; }
    static void SetSoundDuration( const float fDuration ) { s_fSoundDuration   = fDuration; }

    const Asura_List< SoundEntry >& GetEntryList() const { return m_xEntryList; }

protected:
    class Asura_ResizingMempool* m_pxEntryMempool;
    class Asura_ResizingMempool* m_pxListEntryMempool;
    Asura_List< SoundEntry >     m_xEntryList;

    static int   s_iMempoolPageSize;
    static float s_fSoundDuration;
};

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

#endif //__AXON_SOUND_REGISTRY_H__
#endif
