// DSP_SimpleFilter_SPU.cpp
// Thomas Young - 03.04.09
// Copyright (C) 2009 Rebellion, All Rights Reserved.

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Asura_DSP_SPU.h"
#include "Sound/Asura_PS3_DSP_Params.h"

#include <stdlib.h>
#include <math.h>
#include <spu_mfcio_gcc.h>
#include <string.h>

// E N U M E R A T I O N S ///////////////////////////////////////////////////

enum SPU_GET_PUT
{
	SPU_GET = 0,
	SPU_PUT = 1
};

// F U N C T I O N   P R O T O T Y P E S  ///////////////////////////////////

void DMATransfer			( void *pSPUAddr, void* pRAMAddr, unsigned int uSize, SPU_GET_PUT eGetOrPut, MS_DSP_INFO *pxDSP_Info );
void WaitforDMACompletion	( );

// F U N C T I O N   D E F I N I T I O N S ///////////////////////////////////

void DSPInit ( char *pcParams, void *pWorkArea, MS_DSP_INFO *pxDSP_Info )
{
}


int DSPApply ( void* pInput, void* pOutput, char *pcParams, void *pWorkArea, MS_DSP_INFO *pxDSP_Info )
{
	Asura_PS3_DSP_Params::SimpleFilter* pxParams = reinterpret_cast< Asura_PS3_DSP_Params::SimpleFilter* >( pcParams ) ;

	if( pxParams->m_uScratchMemSize < Asura_PS3_DSP_Params::SimpleFilter::uSCRATCH_MEMORY_REQ )
	{
		return 0;
	}

	if( !pxParams->m_bValid )
	{
		return 0;
	}

	char* pcWorkArea = reinterpret_cast< char* >( pWorkArea );

	DMATransfer( pcWorkArea, pxParams->m_pScratchMem, Asura_PS3_DSP_Params::SimpleFilter::uSCRATCH_MEMORY_REQ, SPU_GET, pxDSP_Info );

	float* pfInput	= reinterpret_cast< float* >( pInput );
	float* pfOutput = reinterpret_cast< float* >( pOutput );

	// The work memory now contains the last samples of the old buffer, copy the new buffer after it
	// So that we have || end of last buffer | new buffer ||
	memcpy( &pcWorkArea[ Asura_PS3_DSP_Params::SimpleFilter::uNUM_COEFFICIENTS * sizeof( float ) ], pfInput, 512 * sizeof( float ) );
	memset( pfOutput, 0, 512 * sizeof( float ) );

	float* pfOldSignal = reinterpret_cast< float* >( pcWorkArea );

	// Convolve

	//		|1|2|3|			- Original Signal	( pfOldSignal )
	//	|4|5|6|	->			- Coefficients		( pxParams->m_afCoefficients )
	//		|x|y|z|			- Result			( pfOutput )

	for( int iResultPos = 0; iResultPos < 512; ++iResultPos )
	{
		// loop over coefficients and multiply-add to the result
		for( int iCoeff = 0; iCoeff < static_cast<int>( Asura_PS3_DSP_Params::SimpleFilter::uNUM_COEFFICIENTS ); ++iCoeff )
		{
			const int iSourcePos = ( iResultPos + iCoeff );
			
			pfOutput[ iResultPos ] += pfOldSignal[ iSourcePos ] * pxParams->m_afCoefficients[ iCoeff ];
		}

		// Apply the gain adjustment afterwards
		pfOutput[ iResultPos ] *= pxParams->m_fGain;
		
		// Clip if necessary
		if( pfOutput[ iResultPos ] > 1.0f )
		{
			pfOutput[ iResultPos ] = 1.0f;
		}
		else if( pfOutput[ iResultPos ] < -1.0f )
		{
			pfOutput[ iResultPos ] = -1.0f;
		}
	}

	// Now save the last bit of the buffer for use next time
	char* pcScratch = reinterpret_cast< char* >( pxParams->m_pScratchMem );

	DMATransfer( &pcWorkArea[ 512 * sizeof( float ) ], pcScratch, Asura_PS3_DSP_Params::SimpleFilter::uSCRATCH_MEMORY_REQ, SPU_PUT, pxDSP_Info );
	
	return 0;
}


void DSPPlugInSetup( DSPPlugin* pxPluginDetails )
{
	pxPluginDetails->DSPInit	= (int(*)(void*))( DSPInit );
	pxPluginDetails->DSPApply	= (int(*)(void*))( DSPApply);
	pxPluginDetails->Domain	= MS_DSP_TIMEBASE;
}


void DMATransfer( void *pSPUAddr, void* pRAMAddr, unsigned int uSize, SPU_GET_PUT eGetOrPut, MS_DSP_INFO *pxDSP_Info )
{
	if( eGetOrPut == SPU_GET )
	{
		spu_mfcdma32 ( pSPUAddr, reinterpret_cast<unsigned int>(pRAMAddr), uSize, 0, MFC_GET_CMD );
		pxDSP_Info->bandwidthIn += uSize;
		WaitforDMACompletion();
	}
	else
	{
		spu_mfcdma32( pSPUAddr, reinterpret_cast<unsigned int>(pRAMAddr), uSize, 0, MFC_PUT_CMD );
		pxDSP_Info->bandwidthOut += uSize;
	}
}

void WaitforDMACompletion ()
{
	spu_writech( MFC_WrTagUpdate, 0x0 );
	while ( spu_readchcnt( MFC_WrTagUpdate ) !=1 ) {};
	spu_readch(MFC_RdTagStat);
	spu_writech(MFC_WrTagMask, ( 0xffff));
	spu_writech(MFC_WrTagUpdate, 0x2);
	spu_readch(MFC_RdTagStat);
}
