// Asura_DSP_SPU.h
// Thomas Young - 23.03.09
// Copyright (C) 2009 Rebellion, All Rights Reserved.

#ifndef _ASURA_DSP_SPU_
#define _ASURA_DSP_SPU_

// Allow TTY output
#include <sys/spu_event.h>
typedef int(*pfnSpuPrintfFunc)(uint32_t _spup, const char *fmt, ...);
int (*_spu_call_event_va_arg)(uint32_t _spup, const char *fmt, ...);
#define spu_printf(fmt, args...) _spu_call_event_va_arg(EVENT_PRINTF_PORT<<EVENT_PORT_SHIFT, fmt, ## args)

// D E F I N E S /////////////////////////////////////////////////////////////

#define MS_DSP_FREQUENCYBASE  (0)	// Used to define a DSP effect as being in Frequency Domain
#define MS_DSP_TIMEBASE       (1)	// Used to define a DSP effect as being in Time Domain

/*#ifndef NULL
#define NULL 0
#endif*/

// S T R U C T U R E S ///////////////////////////////////////////////////////

// The following is the structure passed to both the DSPInit and DSPApply functions by MultiStream

typedef struct MS_DSP_INFO
{
	int				callNumber;		// DSP slot (0-7) being processed by DSP effect, or -1 for a mono stream
	char			inMask; 		// If not a mono stream, what the incoming channel mask is
	char			outMask;		// If not a mono stream, what the outgoing channel mask is
	int				state;			// 0 = Stream is currently playing. !0 = Number of calls to DSP since stream is complete
	void*			Reserved1;		// Reserved for system use
	void*			Reserved2;		// Reserved for system use
	float*			Reserved3;		// Reserved for system use
	unsigned char	Reserved4;		// Reserved for system use
	void*			dbgOut;			// Used for TTY output
	int				bandwidthIn;	// For analysis. For each DMA from main RAM to SPU, user should increase this by the size transferred
	int				bandwidthOut;	// For analysis. For each DMA from SPU to main RAM, user should increase this by the size transferred
} MS_DSP_INFO;

// Structure required to be completed within a MultiStream DSP effects SPU entry point
// See "DSP Effect Programming" within the MultiStream Overview documentation for more information

typedef struct DSPPlugin
{
	int ( *DSPInit  ) ( void* );	// DSP initialisation function (or NULL if no init function required)
	int ( *DSPApply ) ( void* );	// DSP Apply function (actually processes the DSP effect)
	int Domain;						// 0 = Freq, 1 = Time
} DSPPlugin;

// F U N C T I O N   P R O T O T Y P E S ///////////////////////////////////////

// Notes copied from the DSP effect creation samples

/**********************************************************************************

DSPInit
This routine is called by MultiStream when the DSP effect is first called.

Requires:
paramList		Address of static parameter data (248 bytes)
				0-127 = Read Only (example: users input parameters)
				128-247 = Read / Write (example: DSP effects internal parameters)
workArea		Work area available for DSP effects (40 K)
				Any DMA from main RAM must fit into this area
dspInfo			Contains information as to the slot number to be processed, input / output bitmasks

Notes:
Each DSP effect has 248 bytes of RAM available.
If more RAM is required, this area can be used to store a pointer to the actual data.
The data can then be transferred from main RAM as required.
Splitting the paramList into a read and a read/write section allows us to make sure that if
the user is modifying parameters via PPU commands (example: distortion level), there is no conflicts
between the SPU updating parameter memory at the same time.

**********************************************************************************/

extern "C" void DSPInit ( char *pcParams, void *pWorkArea, MS_DSP_INFO *pxDSP_Info );

/**********************************************************************************

DSPApply
Apply DSP effect routine

Requires:
input			Address of input data (Time Domain float32 data OR Frequency Domain FFT data)
				For PCM, 1024 samples are passed in.
				For FFT, 512 frequency bands are passed in.

output			Address to write output data to

paramList		Address of static parameter data (248 bytes)
				0-127 = Read Only
				128-247 = Read / Write

workArea		Work area available for DSP effects (40 K)
				Any DMA from main RAM must fit into this area

dspInfo			Contains information as to the slot number to be processed, input / output bitmasks..

Returns:
0				DSP will complete when stream finishes
!0				DSP will stay active after stream finishes (for delays / reverbs)

Notes:
For time domain input data, the user must process all 1024 samples.
But note that any parameters used for modifying data must be preserved after 512 samples.
This is due to windowing within MultiStream.

**********************************************************************************/

extern "C" int DSPApply ( void* pInput, void* pOutput, char *pcParams, void *pWorkArea, MS_DSP_INFO *pxDSP_Info );

/**********************************************************************************

DSPPlugInSetup
Setup as the entry point function within the DSP makefile

Requires:
plugInStruct		Address of a DSPPlugin structure

Notes:
This function sets the address of both the Init and the Apply functions.
It also specifies if the input / output data is required as time domain or FFT data

**********************************************************************************/

extern "C" void DSPPlugInSetup ( DSPPlugin* pxPluginDetails );

#endif

// eof
