#ifdef ASURA_USE_AXON
//////////////////////////////////////////////////////////////////////////////
//
// Tools_Mode_AnimTemplates.h
//
// Created by NJD - 07 November 2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __TOOLS_MODE_ANIMTEMPLATES_H__
#define __TOOLS_MODE_ANIMTEMPLATES_H__ 1

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// asura
#include "Core/Asura_LinkList.h"
#include "Core/Asura_String.h"
#include "Interface/Asura_Tree_Dialog.h"

// axon
#include "AnimTemplates/Axon_Gamescene_AnimTemplates.h"
#include "AnimTemplates/Axon_GS2_AnimPackage.h"

// tools
#include "Misc/Tools_MessageLog.h"

// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////////

class Asura_Overlay;
class Asura_Panel_Dialog;
class Asura_Table_Dialog;
class Asura_TableCell_Dialog;
class Asura_Tabs_Dialog;
class Axon_Gamescene_AnimTemplate;
class Axon_Gamescene_AnimTemplate_Entry;
class Tools_ContextMenu;
class Tools_ContextMenu_Context;
class Tools_ModeDataFile;

//////////////////////////////////////////////////////////////////////////////
//
// Tools_AnimTemplates
//
//////////////////////////////////////////////////////////////////////////////

class Tools_AnimTemplates
{
public:
	static void LoadAnimTemplatesAndPackages();

	static void Initialise();
	static void Update();
	static void Render();
	static void Shutdown();

	static void SetFilesToCheckForExternalModification();
	static void SaveAll();

	static bool MainOverlayIsOpen();
	static void ToggleOverlays();

	static void OpenAnimTemplate( const u_int uUniqueID );

	// Can be used by any other piece of code to choose an anim template, even if the mode has not been initialised.
	static bool ChooseAnimTemplate( u_int& uSelectedUniqueID,
									u_int uTypeBitfield = 0xffffffff, u_int uUniqueIDToIgnore = uAXON_ANIMATION_SEQUENCE_ID_NONE,
									u_int uInitiallySelectedID = uAXON_ANIMATION_SEQUENCE_ID_NONE, bool bAllowNone = true );
	// Can be used by any other piece of code to choose an anim package, even if the mode has not been initialised.
	static bool ChooseAnimPackage( u_int& uSelectedUniqueID,
								   u_int uUniqueIDToIgnore = Axon_GS2_AnimPackage::uUNSET_ID,
								   u_int uInitiallySelectedID = Axon_GS2_AnimPackage::uUNSET_ID, bool bAllowNone = true );

	// Returns whether this is a new modification or not.
	static bool SetAnimTemplateModified( const u_int uUniqueID );
	static bool SetAnimPackageModified( const u_int uUniqueID );

	static void RequestRefreshMainOverlay() { s_bRefreshMainOverlay = true; }
	static void RequestRefreshAnimTemplateOverlay() { s_bRefreshAnimTemplateOverlay = true; }
	static void RequestRefreshAnimPackageOverlay() { s_bRefreshAnimPackageOverlay = true; }

protected:
	Tools_AnimTemplates() {}
	virtual ~Tools_AnimTemplates() {}

	virtual void InitialiseMode();
	virtual void UpdateMode();
	virtual void RenderMode();
	virtual void ShutdownMode();

	void TransferModeData( const int iMode );
	virtual void TransferModeData( Tools_ModeDataFile& xFile );

	struct AnimTemplateTabData
	{
		u_int m_uTemplateUniqueID;
		Asura_LinkList< bool > m_xEntryMinimisedList;
		Asura_Guid GetGuid() const { return m_uTemplateUniqueID; }
		bool operator==( const AnimTemplateTabData& xRhs ) { return ( m_uTemplateUniqueID == xRhs.m_uTemplateUniqueID ); }
	};

	// Overlay stuff.
	void CreateMainOverlay();
	virtual void AddMainOverlayItems( Asura_Overlay& xNewOverlay, Asura_Vector_2& xOffset );
	virtual void AddPreviewScoringDataControls( Asura_Overlay& xNewOverlay, Asura_Vector_2& xOffset ) {}
	void DestroyMainOverlay();
	void CreateAnimTemplateOverlay();
	void AddAnimTemplateOverlayItems( Axon_Gamescene_AnimTemplate& xTemplate,
									  AnimTemplateTabData& xTabData,
									  Asura_Tabs_Dialog& xTabsDialogue,
									  Asura_Vector_2& xOffset );
	void AddAnimTemplateEntryOverlayItems( Axon_Gamescene_AnimTemplate& xTemplate,
										   Axon_Gamescene_AnimTemplate_Entry& xEntry,
										   int iEntryIndex,
										   Asura_Panel_Dialog& xPanelDialogue,
										   Asura_Vector_2& xOffset,
										   bool& bAllowFurtherEntries );
	static void CloseCurrentAnimTemplateTab();
	void DestroyAnimTemplateOverlay();
	void CreateAnimPackageOverlay();
	virtual void AddAnimPackageOverlayItems( Axon_GS2_AnimPackage& xPackage, Asura_Tabs_Dialog& xTabsDialogue, Asura_Vector_2& xOffset );
	static void CloseCurrentAnimPackageTab();
	void DestroyAnimPackageOverlay();
	static void ContextMenuCallback( Tools_ContextMenu& xMenu, const Tools_ContextMenu_Context& xContext );
	virtual void AddContextMenuItems( Tools_ContextMenu& xMenu, const Tools_ContextMenu_Context& xContext );

	static void MainOverlayClosedByUser();
	static void AnimTemplateOverlayClosedByUser();
	static void RequestRefreshAnimTemplateOverlayWithTemplate( const u_int uUniqueID ) { s_bRefreshAnimTemplateOverlay = true; s_uRefreshTemplate = uUniqueID; }
	static void RequestRefreshAnimTemplateTreeSelection() { s_bRefreshTemplateTreeSelection = true; }
	static void AnimPackageOverlayClosedByUser();
	static void RequestRefreshAnimPackageOverlayWithPackage( const u_int uUniqueID ) { s_bRefreshAnimPackageOverlay = true; s_uRefreshPackage = uUniqueID; }
	static void RequestRefreshAnimPackageTreeSelection() { s_bRefreshPackageTreeSelection = true; }

	static void SetPreviewDefaultSkin();
	static void SetPreviewObject();
	static void SetPreviewObjectToThis( const Asura_Guid uGuid );
	static void StartPreview();
	static void StopPreview();
	static bool CanStartPreview();
	static bool CanStopPreview();
	virtual void* GetPreviewScoringData() { return NULL; }

	static void CreateAnimTemplateTree();
	static void SetAnimTemplateTreeModifiedColours();
	static void ResetAnimTemplateTreeColours();
	static void ResetTreeItemAndChildrensColour( Asura_TreeDialog_Item* pxItem );
	static void AnimTemplateTreeDoubleClicked();
	static bool AnyTemplateTreeItemIsSelected();
	static bool AnimTemplateTreeTemplateItemIsSelected();
	static bool FidgetTemplateTreeItemIsSelected();
	static bool LoopingTemplateTreeItemIsSelected();
	static bool RigidTemplateTreeItemIsSelected();
	static bool StaticTemplateTreeItemIsSelected();
	static bool LocalWritableTemplateTreeItemIsSelected();
	static bool EmptyFolderTemplateTreeItemIsSelected();
	static bool CanPasteIntoSelectedTemplate();

	static void CreateAnimPackageTree();
	static void SetAnimPackageTreeModifiedColours();
	static void ResetAnimPackageTreeColours();
	static void AnimPackageTreeDoubleClicked();
	static bool AnyPackageTreeItemIsSelected();
	static bool AnimPackageTreeItemIsSelected();
	static bool LocalWritablePackageTreeItemIsSelected();
	static bool EmptyFolderPackageTreeItemIsSelected();
	virtual Asura_TableCell_Dialog* CreateAnimPackageTableCell( Asura_Table_Dialog& xTableDialog,
																const Asura_Vector_2& xOffset, const Asura_Vector_2& xMinCellSize,
																const u_int uRow, const u_int uColumn, void* const pxUserData );

	// Data editing stuff.
	static void NewAnimTemplate();
	static void EditAnimTemplate();
	static void EditAnimTemplate( const u_int uUniqueID );
	static void DeleteAnimTemplate();
	static void AddAnimTemplateEntry( int iIndex );
	static void PasteAnimTemplateEntry( Axon_Gamescene_AnimTemplate* pxTemplate, Axon_Gamescene_AnimTemplate_Entry* pxNewEntry, const int iIndex );
	static void MoveAnimTemplateEntryUp( int iIndex );
	static void MoveAnimTemplateEntryDown( int iIndex );
	static void DeleteAnimTemplateEntry( int iIndex );
	static void CutAnimTemplateEntry( Axon_Gamescene_AnimTemplate* pxTemplate, Axon_Gamescene_AnimTemplate_Entry*& pxCopyBuffer, const int iIndex );
	static void ChooseFidgetAnimTemplate( int iIndex );
	static void SetAnimTemplateEntryAnim( int iIndex );
	static void SetAnimTemplateEntryPackage( int iIndex );
	static void CopySelectedTemplateEntries();
	static void PasteEntriesIntoSelectedTemplate();
	static void ClearTemplateCopyBuffer();

	static void NewAnimTemplateFolder();
	static void DeleteAnimTemplateFolder();

	static bool GetUniqueTemplateName( Asura_String& xName );
	static bool GetValidFolderName( Asura_String& xName );

	static void NewAnimPackageFolder();
	static void DeleteAnimPackageFolder();
	static void NewAnimPackage();
	static void EditAnimPackage();
	static void DeleteAnimPackage();

	static bool GetUniquePackageName( Asura_String& xName );

	static u_int GetMainOverlayID() { return reinterpret_cast< u_int >( s_pxThis ); }
	static u_int GetAnimTemplateOverlayID() { return reinterpret_cast< u_int >( s_pxThis + 1 ); }
	static u_int GetAnimPackageOverlayID() { return reinterpret_cast< u_int >( s_pxThis + 2 ); }
	static int GetMainOverlayTabDialogueID() { return reinterpret_cast< int >( s_pxThis + 3 ); }
	static int GetMainOverlayTemplateTreeDialogueID() { return reinterpret_cast< int >( s_pxThis + 4 ); }
	static int GetAnimTemplateOverlayTabDialogueID() { return reinterpret_cast< int >( s_pxThis + 5 ); }
	static int GetAnimPackageOverlayTabDialogueID() { return reinterpret_cast< int >( s_pxThis + 6 ); }

	// Returns the anim template associated with the selected tab on the anim template overlay.
	static Axon_Gamescene_AnimTemplate* GetSelectedAnimTemplate();
	// Returns the anim package associated with the selected tab on the anim package overlay.
	static Axon_GS2_AnimPackage* GetSelectedAnimPackage();

	virtual u_int GetNumAnimPackageMappingFactors() { return 0; }
	virtual const char* const* GetAnimPackageMappingNames() { return NULL; }
	static Asura_TableCell_Dialog* CreateTableCell( Asura_Table_Dialog& xTableDialog,
													const Asura_Vector_2& xOffset, const Asura_Vector_2& xMinCellSize,
													const u_int uRow, const u_int uColumn, void* const pxUserData )
	{
		return s_pxThis->CreateAnimPackageTableCell( xTableDialog, xOffset, xMinCellSize, uRow, uColumn, pxUserData );
	}

private:
	static Tools_AnimTemplates* Project_CreateAnimTemplateTools();
	static void EstablishLocalWritableAttributes();
	//## to be removed
	static void ConvertToIndividualFiles();

private:
	enum TabNames
	{
		TN_ANIM_TEMPLATES,
		TN_ANIM_PACKAGES,
		NUM_TAB_NAMES
	};
	static const char* s_aszTabNames[ NUM_TAB_NAMES ];

	static Tools_AnimTemplates*		s_pxThis;
	static bool						s_bFirstUpdate;

	static bool						s_bRefreshMainOverlay;
	static bool						s_bRefreshAnimTemplateOverlay;
	static bool						s_bRefreshAnimPackageOverlay;
	static u_int					s_uRefreshTemplate;
	static Asura_Hash_ID			s_uRefreshPackage;

	static Asura_String				s_xPreviewDefaultSkin;
	static Asura_Guid				s_uPreviewObjectGuid;
	static bool						s_bPreviewActive;

	static bool						s_bMainOverlayOpenOnShutdown;
	static bool						s_bMainOverlayWasOnTop;
	static Asura_Vector_2			s_xMainOverlayPosition;
	static u_int					s_uMainOverlayActiveTab;
	static const Asura_Vector_2		s_xTreeDialogueDimensions;
	static Asura_TreeDialog_Tree	s_xAnimTemplateTree;
	static Asura_TreeDialog_Item*   s_pxSelectedTemplateTreeItemPrev;
	static Asura_TreeDialog_Item*   s_pxSelectedTemplateTreeItem;
	static Asura_String				s_xNewTemplateTypeString;
	static Asura_TreeDialog_Tree	s_xAnimPackageTree;
	static Asura_TreeDialog_Item*   s_pxSelectedPackageTreeItemPrev;
	static Asura_TreeDialog_Item*   s_pxSelectedPackageTreeItem;

	static Asura_Vector_2			s_xAnimTemplateOverlayPosition;
	static u_int					s_uAnimTemplateActiveTabPrev;
	static u_int					s_uAnimTemplateActiveTab;
	static bool						s_bRefreshTemplateTreeSelection;
	static Asura_LinkList< AnimTemplateTabData > s_xOpenAnimTemplateList;
	static bool						s_bSelectedAnimTemplateChanged;
	static bool						s_bSelectedAnimTemplateChangedRefresh;
	static Asura_LinkList< u_int >  s_xAnimTemplateModifiedList;

	static Asura_Vector_2			s_xAnimPackageOverlayPosition;
	static u_int					s_uAnimPackageActiveTabPrev;
	static u_int					s_uAnimPackageActiveTab;
	static bool						s_bRefreshPackageTreeSelection;
	static Asura_LinkList< u_int >  s_xOpenAnimPackageList;
	static bool						s_bSelectedAnimPackageChanged;
	static bool						s_bSelectedAnimPackageChangedRefresh;
	static Asura_LinkList< u_int >  s_xAnimPackageModifiedList;
	static Asura_String*			s_pxAnimPackageTableHeadings;

	// More permanent buffer for copying/pasting.
	static u_int							  s_uAnimTemplateCopyBufferType;
	static Axon_Gamescene_AnimTemplate_Entry* s_pxAnimTemplateCopyBufferEntry;

	static const u_int uCURRENT_MODE_FILE_VERSION = 0;
};

//////////////////////////////////////////////////////////////////////////////
//
// Tools_MessageLogAction_OpenAnimTemplate
//
//////////////////////////////////////////////////////////////////////////////

class Tools_MessageLogAction_OpenAnimTemplate : public Tools_MessageLogAction
{
public:
	Tools_MessageLogAction_OpenAnimTemplate( const u_int uUniqueID ) : m_uUniqueID( uUniqueID ) {}
	virtual void Execute() { Tools_AnimTemplates::OpenAnimTemplate( m_uUniqueID ); }

private:
	u_int m_uUniqueID;
};

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

#endif //__TOOLS_MODE_ANIMTEMPLATES_H__
#endif // ASURA_USE_AXON
