////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

//
// Tools_Mode_AssetManager.cpp
//
// Created by Dan Clarke - 16/03/2008
// 
// Copyright (C) 2002 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

#ifdef ASURA_USE_ASSET_MANAGER

#include "Interface/Asura_Overlay_System.h"
#include "Interface/Asura_Overlay.h"
#include "Core/Asura_TextureManagement.h"
#include "Rendering/Asura_Text_Output.h"
#include "Tools_Overlays.h"
#include "Core/Asura_NameList.h"
#include "AssetManager/Tools_Mode_AssetManager.h"
#include "Interface/Asura_Menubar_System.h"
#include "Interface/Asura_Menubar.h"
#include "Load&Save/Tools_ModeDataFile.h"
#include "Objects/Tools_Object_System.h"
#include "Misc/Tools_Render_Options.h"
#include "Load&Save/Tools_SaveAsura.h"
#include "Interface/Asura_Dialog.h"
#include "Animation/Asura_Animation.h"
#include "Rendering/Asura_Render.h"
#include "PC/FileSystem/Tools_FileSystem.h"
#include "Rendering/Asura_Direct3D_BackBufferCapture.h"
#include "Misc/Tools_Timed_Messages.h"
#include "Debugging/Asura_Logfile.h"
#include "Mesh/Asura_Mesh.h"
#include "Core/Asura_MaterialResponse_System.h"
#include "Lighting/Tools_Photon_Object.h"
#include "Light/Asura_GI_Solution.h"
#include "Interface/Tools_ColourPicker.h"
#include "SpecialFX/Asura_Skybox.h"
#include "Load&Save/Tools_AsuraFileContents.h"
#include "Rendering/Asura_Font_System.h"
#include "Rendering/Asura_Viewport.h"
#include "Interface/Asura_MultiLineString_Dialog.h"
#include "Lighting/Tools_PhotonMapping_System.h"
#include "Lighting/Tools_PhotonMapping_LightSource.h"
#include "Lighting/Tools_PhotonMapping_LightSource_Object.h"
#include "Lighting/Tools_PhotonMapping_System_Internal.h"
#include "Debugging/Asura_DebugText.h"
#include "Lighting/Tools_PhotonMapping_PolygonLightSource_Object.h"
#include "Light/Asura_Light.h"
#include "Light/Asura_Photon.h"

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

//#define ASSET_DATA_FOLDER	"\\\\oxnsvr-prodtool\\assetman\\AssetData"
#define ASSET_DATA_FOLDER	"\\\\Oxnsrv-art\\ArtResources\\assetmanager_data"

#define SCREENSHOT_TOP		(100)
#define SCREENSHOT_LEFT		(500)
#define SCREENSHOT_RIGHT	(1200)
#define SCREENSHOT_BOTTOM	(800)

//#define DEFAULT_BACKGROUND_COLOUR	(0xffa1a1a1)
#define DEFAULT_BACKGROUND_COLOUR	(0xffffffff)

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

struct AMLightInfoStruct
{
	Asura_Vector_3 xColour;
	Tools_PhotonMapping_LightSource* pxLightSource;
};

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

static const char* szCMD_LINE_LOGFILE = "logfiles\\assetman.txt";

static int s_iRequestScreenGrab = -1;

Asura_Vector_2 Tools_Mode_AssetManager::s_xAssetInfoOverlayPosition(10.0f, 10.0f);

Tools_Mode_AssetManager* Tools_Mode_AssetManager::s_pxThis = NULL;

const char* Tools_Mode_AssetManager::s_paszCategoryNames[] = {"Characters", "Pickups", "Plants", "Props", "Vehicles", "Weapons"};

u_int Tools_Mode_AssetManager::g_auStockBGColours[4] = { 0xffffffff, 0xff505050, 0xffd9fbff, 0xffe8ffd9 };

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

Tools_Mode_AssetManager::Tools_Mode_AssetManager()
{
	s_pxThis = this;

	m_pxAnimation = NULL;

	xAssetManagerModePosition = Asura_Vector_2(3.0f,3.0f);
	bAssetManagerOpen = true;

	Asura_Mem::Set( m_aiTempLightToolsIDs, 0, sizeof( m_aiTempLightToolsIDs ) );
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void Tools_Mode_AssetManager::InitialiseMode(void)
{
	Asura_SimpleLogfileClear(szCMD_LINE_LOGFILE);

	// Set the background colour

	m_uBackgroundColour = DEFAULT_BACKGROUND_COLOUR;

	m_pxAssetInfoOverlay = NULL;

	s_iRequestScreenGrab = -1;

	Asura_Render::SetEnableScreenClear(true);

	// Disable skybox rendering

	m_bWasSkyboxVisible = Asura_Render::IsSystemEnabled(ASURA_RENDER_SYSTEM_SKYBOX);
	Asura_Render::SetSystemEnabled(ASURA_RENDER_SYSTEM_SKYBOX, false);

	// Create the menubar
	xMenuHandle = Asura_Menubar_System::CreateMenuSection();

	Asura_Menubar_Title* pxTitle = Asura_Menubar_System::AddTitle(xMenuHandle,"Overlays");
	pxTitle->AddItem("Asset Manager Mode", Tools_Mode_AssetManager::_ToggleMainOverlay, Tools_Mode_AssetManager::_AssetManagerOverlayIsOn);

	// Load values from .dat file
	TransferModeData(TOOLSMODEDATAFILE_LOAD);

	// Create overlays
	CreateMainOverlay();

	Tools_Render_Options::HideEnvironment = true;

	// Enable object system
	Tools_Object_System::EnableAllControls(true);
	// But not undo , since the keys clash
	Tools_Object_System::DisableUndo();

	Asura_Vector_3 xCameraPosition(-3.4345291f, -0.99240857f, 4.1064320f);
	Asura_Matrix_3x3 xCameraOrientation(	-0.87142992f, 0.0019160890f, 0.49051628f,
											0.00000000f, 0.99999237f, -0.0039062400f,
											-0.49052003f, -0.0034040145f, -0.87142324f);

	Asura_Camera::SetPosition(xCameraPosition);
	Asura_Camera::SetOrientation(xCameraOrientation);

	// Prevent the environment lighting from affecting th models in this mode
	Asura_GI_Solution::DisableStaticLighting(true);

	{
		// DLH - 15 March 2007 12:52:30 - Find all the Active Photon, turen them off and note them so they can be turned back on later.
		Asura_TrackingHandlePool<Asura_Photon>* pxPhotonList = Asura_Photon_System::GetDynamicPhotonList();
		Asura_TrackingHandlePool<Asura_Photon>::Iterator xIterator = pxPhotonList->GetIterator();
		
		this->m_xPhotonsActiveAtStart.empty();

		for (xIterator.Begin(); !xIterator.End(); xIterator.Next())
		{
			Asura_Photon* pxPhoton = xIterator.GetCurrentPointer();
			
			if( false == pxPhoton->IsActive())
			{
				continue;
			}
			
			pxPhoton->SetAsActive( false);
			this->m_xPhotonsActiveAtStart.add_entry( pxPhoton);
		}
	}

	// Create the light sources.  [DC] Sorry about the magic numbers!  These are values requested by JamesM.

	Asura_Vector_3 axLightPos[NUM_TEMP_LIGHTS]=
	{
		Asura_Vector_3(10, -25, 12.5),
		Asura_Vector_3(-5, -12.5, 12.5),
		Asura_Vector_3(-5, -4, -17.5),
	};

	float afLightColours[NUM_TEMP_LIGHTS][3]=
	{
		{ 255, 255, 240 },
		{ 80, 80, 111 },
		{ 228, 225, 255 },
	};

	float afLightNormals[NUM_TEMP_LIGHTS][3]=
	{
		{-1.0f, 0.0f, 0.5f},
		{-1.0f, 0.0f, -0.5f},
		{1.0f, 0.0f, 0.0f},
	};

	int iShowLights = 0;

	/*
	for(u_int u = 0; u < NUM_TEMP_LIGHTS; u++)
	{
		Asura_Matrix_3x3 xOrientation;
		Asura_Maths::OrientationFromNormal(xOrientation, Asura_Vector_3(afLightNormals[u][0], afLightNormals[u][1], afLightNormals[u][2]));

		Asura_Handle xLightSource = Tools_PhotonMapping_System::CreateLightSource();
		Tools_PhotonMapping_LightSource* pxLightSource = Tools_PhotonMapping_System::GetLightSource(xLightSource);
		pxLightSource->SetPosition(axLightPos[u]);
		pxLightSource->SetColour(Asura_Vector_3(afLightColours[u][0], afLightColours[u][1], afLightColours[u][2]));
		pxLightSource->SetOrientation(xOrientation);
		pxLightSource->SetAngle(afAngleDegrees[u]);

		//Tools_PhotonMapping_LightSource_Object* pxObject = new Tools_PhotonMapping_LightSource_Object(xLightSource);
		//pxObject->SetLightingModified();

		Tools_PhotonMapping_System::RecalculatePhotonMapForLightSource(xLightSource);
	}
	*/

	for(u_int u = 0; u < NUM_TEMP_LIGHTS; u++)
	{
		Asura_Matrix_3x3 xOrientation;
		Asura_Vector_3 xLightNormal (afLightNormals[u][0], afLightNormals[u][1], afLightNormals[u][2]);
		xLightNormal.Normalise();
		Asura_Maths::OrientationFromNormal(xOrientation, xLightNormal);

		Tools_Photon_Object* const pxNewLight = new Tools_Photon_Object(&axLightPos[u], &xOrientation);

		if(pxNewLight)
		{
			pxNewLight->SetLightColour(Asura_Vector_3(afLightColours[u][0] / 255.0f, afLightColours[u][1] / 255.0f, afLightColours[u][2] / 255.0f));
			pxNewLight->SetLightBrightness(5.0f);
			pxNewLight->SetLightRange(200.0f);

			pxNewLight->UpdatePhoton();

			m_aiTempLightToolsIDs[u] = pxNewLight->GetToolsID();
		}
	}

	if(iShowLights == 0)
	{
		Tools_Object_System::SetTypeHidden(TOT_ToolsPhoton, ObjectHideFlag_Mode, true);
		Tools_Object_System::SetTypeHidden(TOT_PhotonMapping_LightSource, ObjectHideFlag_Mode, true);
		Tools_Object_System::SetTypeHidden(TOT_PhotonMapping_PolygonLightSource, ObjectHideFlag_Mode, true);
	}

	/*
	// Change the tonemap attributes

	Asura_CameraData::HDRSettings xHDRSettings;

	xHDRSettings.SetToDefaults();
	xHDRSettings.m_fBlackPoint = 0.0f;
	xHDRSettings.m_fWhitePoint = 1.0f;
	xHDRSettings.m_bDynamicExposure = false;
	xHDRSettings.m_fMinLuminance = 0.0f;
	xHDRSettings.m_fMaxLuminance = 1.0f;

	Tools_Render_Options::GetHDRSettings(m_xOldHDRSettings);
	Tools_Render_Options::SetHDRSettings(xHDRSettings);
	*/
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void Tools_Mode_AssetManager::DisableOldLights()
{
	// Empty the list

	List_Iterator_Forward<AMLightInfoStruct*> xIter(&m_xOldLightColourList);

	for(; !xIter.done(); xIter.next())
	{
		delete xIter();
	}

	m_xOldLightColourList.empty();

	// Iterate through the existing lights setting the colour to black (note that we also store the old colours to restore when we leave this mode)

	Asura_TrackingHandlePool<Tools_PhotonMapping_LightData>::Iterator xIterator = Tools_PhotonMapping_System::GetLightDataTrackingHandlePool()->GetIterator();

	for (xIterator.Begin(); !xIterator.End(); xIterator.Next())
	{
		Tools_PhotonMapping_LightData* pxLightData = xIterator.GetCurrentPointer();
		Tools_PhotonMapping_LightSource* pxLightSource = pxLightData->GetLightSource();

		AMLightInfoStruct* pxLightInfo = new AMLightInfoStruct;

		pxLightInfo->xColour = pxLightSource->GetColour();
		pxLightInfo->pxLightSource = pxLightSource;

		m_xOldLightColourList.add_entry(pxLightInfo);

		pxLightSource->SetColour(Asura_Vector_3(0.0f, 0.0f, 0.0f));
	}
}

void Tools_Mode_AssetManager::RestoreOldLights()
{
	List_Iterator_Forward<AMLightInfoStruct*> xIter(&m_xOldLightColourList);

	for(; !xIter.done(); xIter.next())
	{
		AMLightInfoStruct* pxLightInfo = xIter();

		pxLightInfo->pxLightSource->SetColour(pxLightInfo->xColour);

		delete pxLightInfo;
	}

	m_xOldLightColourList.empty();
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void Tools_Mode_AssetManager::ShutdownMode(void)
{
	// Save values to .dat file
	TransferModeData(TOOLSMODEDATAFILE_SAVE);

	// Shutdown the menu
	if(xMenuHandle != ASURA_INVALID_HANDLE)
	{
		Asura_Menubar_System::DestroyMenuSection(xMenuHandle);
		xMenuHandle = ASURA_INVALID_HANDLE;
	}

	if(m_pxAnimation)
	{
		delete m_pxAnimation;
	}

	Tools_Render_Options::HideEnvironment = false;

	{
		// DLH - 15 March 2007 12:52:05 - Reactivate Dynamic Photons
		while( 0 != this->m_xPhotonsActiveAtStart.size())
		{
			Asura_Photon* pxPhoton = this->m_xPhotonsActiveAtStart.first_entry();
			pxPhoton->SetAsActive( true);
			
			this->m_xPhotonsActiveAtStart.delete_first_entry();
		}
	}

	// Close overlays
	Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_MODE);

	Tools_Object_System::DeleteAllModeSpecific();

	Tools_Object_System::EnableAllControls(false);
	Tools_Object_System::EnableUndo();

	for( u_int u = 0; u < NUM_TEMP_LIGHTS; ++u )
	{
		Tools_Object* const pxCurrLight = Tools_Object_System::GetObjectFromToolsID( m_aiTempLightToolsIDs[u] );
		if( pxCurrLight )
		{
			pxCurrLight->Destroy();
		}
	}

	// Restore skydome to previous state

	Asura_Render::SetSystemEnabled(ASURA_RENDER_SYSTEM_SKYBOX, m_bWasSkyboxVisible);

	// Restore the tonemap attributes
	//Tools_Render_Options::SetHDRSettings(m_xOldHDRSettings);

	// Re-enable lighting
	Asura_GI_Solution::DisableStaticLighting(false);
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void Tools_Mode_AssetManager::TransferModeData(int mode)
{
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void Tools_Mode_AssetManager::Update(void)
{
	Tools_Mode_AssetManager::HandleKeyboard();

	/*
	if(m_pxAssetInfoOverlay)
	{
		Tools_Modal_Overlay::DoModal(m_pxAssetInfoOverlay);
		m_pxAssetInfoOverlay = NULL;
	}
	*/
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void Tools_Mode_AssetManager::EarlyRender()
{
	//Asura_Direct3D::D3DDevice->Clear(0, NULL, D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER, 0xffa1a1a1, 0.0f, 0);

	int iViewportWidth = Asura_Viewport::GetWidth();
	int iViewportHeight = Asura_Viewport::GetHeight();

	Asura_Render::Overlay.Colour			= m_uBackgroundColour;
	Asura_Render::Overlay.TextureIndex		= ASURA_TEXTURE_NONE;
	Asura_Render::Overlay.TranslucencyMode	= ASURA_TRANSLUCENCY_NORMAL;
	Asura_Render::Overlay.SetRectXY(0.0f, 0.0f, (float)iViewportWidth, (float)iViewportHeight);
	Asura_Render::Overlay.Render();
	Asura_Render::FlushDisplay();

	if(m_pxAnimation)
	{
		Asura_Matrix_3x3 xMatrix;

		xMatrix.Identity();

		m_pxAnimation->Update(Asura_Vector_3(0.0f, 0.0f, 0.0f), xMatrix);
	}
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void Tools_Mode_AssetManager::RenderUI(void)
{
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void Tools_Mode_AssetManager::QuickSave(void)
{
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void Tools_Mode_AssetManager::PreSave()
{
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool Tools_Mode_AssetManager::IsModified()
{
	return false;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void Tools_Mode_AssetManager::HandleKeyboard()
{
	if(Asura_Overlay_System::OverlaysHaveKeyboardFocus()) return;
 	if(Asura_UserInput::KeyPressed(ASURA_KEYID_ALT)) return;

	if(Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_SPACE))
	{
		RequestScreenGrab();
	}
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void Tools_Mode_AssetManager::RequestScreenGrab()
{
	s_iRequestScreenGrab = 5;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void Tools_Mode_AssetManager::ToggleMainOverlay(void)
{
	bAssetManagerOpen = !bAssetManagerOpen;

	if (bAssetManagerOpen)
	{
		CreateMainOverlay();
	}
	else
	{
		Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_MODE);
	}
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool Tools_Mode_AssetManager::AssetManagerOverlayIsOn(void)
{
	return bAssetManagerOpen;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void Tools_Mode_AssetManager::CreateMainOverlay(void)
{
	if(!bAssetManagerOpen) return;

	Asura_Overlay* pxOverlay;
	pxOverlay = new Asura_Overlay("Asset Manager Mode", false, &xAssetManagerModePosition);
	
	Asura_Dialog* pxDialog;
	Asura_Vector_2 xOffset = Asura_Vector_2(1.0f,3.0f);

	pxDialog = new Asura_Function_Dialog("Load asset", xOffset, _LoadAsset);
	pxDialog->SetHelpText("");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();

	Asura_StringDisplay_Dialog* pxStringDisplay = new Asura_StringDisplay_Dialog("Choose background colour:", xOffset);
	pxOverlay->AddDialog(pxStringDisplay);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	//Asura_Vector_2 xButtonSize(32.0f,32.0f);

	static int iStockBGColoursTextureIndex = -1;

	if(iStockBGColoursTextureIndex == -1)
	{
		u_int uOldResetGroup = Asura_Texture_Management::GetTextureResetGroup();
		Asura_Texture_Management::SetTextureResetGroup(ASURA_TEXTURE_LOADED_PERMANENT);
		iStockBGColoursTextureIndex = Asura_Texture_Management::AddTexture("\\Tools\\AMStockBGColours.tga");
		Asura_Texture_Management::SetTextureFlags(iStockBGColoursTextureIndex, AsuraTextureFlag_HighQualityCompression);
		Asura_Texture_Management::SetTextureResetGroup(uOldResetGroup);
	}

	Asura_Texture_Management::LoadTextures();

	pxDialog = new Asura_Button_Dialog(xOffset, iStockBGColoursTextureIndex, 128, Asura_Vector_2(0.0f, 0.0f), Asura_Vector_2(32.0f, 32.0f), _ChangeToStockBGColour1, NULL);
	pxDialog->SetHelpText("");
	pxOverlay->AddDialog(pxDialog);
	xOffset.x += 3.0f;

	pxDialog = new Asura_Button_Dialog(xOffset, iStockBGColoursTextureIndex, 128, Asura_Vector_2(32.0f, 0.0f), Asura_Vector_2(32.0f, 32.0f), _ChangeToStockBGColour2, NULL);
	pxDialog->SetHelpText("");
	pxOverlay->AddDialog(pxDialog);
	xOffset.x += 3.0f;

	pxDialog = new Asura_Button_Dialog(xOffset, iStockBGColoursTextureIndex, 128, Asura_Vector_2(64.0f, 0.0f), Asura_Vector_2(32.0f, 32.0f), _ChangeToStockBGColour3, NULL);
	pxDialog->SetHelpText("");
	pxOverlay->AddDialog(pxDialog);
	xOffset.x += 3.0f;

	pxDialog = new Asura_Button_Dialog(xOffset, iStockBGColoursTextureIndex, 128, Asura_Vector_2(96.0f, 0.0f), Asura_Vector_2(32.0f, 32.0f), _ChangeToStockBGColour4, NULL);
	pxDialog->SetHelpText("");
	pxOverlay->AddDialog(pxDialog);
	xOffset.x += 3.0f;

	xOffset.y += Asura_Dialog::LargeSpacing();
	xOffset.y += Asura_Dialog::DefaultSpacing();
	xOffset.x = 1.0f;

	pxDialog = new Asura_Function_Dialog("Custom colour", xOffset, _ChangeToCustomBGColour);
	pxDialog->SetHelpText("");
	pxOverlay->AddDialog(pxDialog);

	xOffset.y += Asura_Dialog::LargeSpacing();

	pxDialog = new Asura_Function_Dialog("Screengrab asset and add to database (spacebar)", xOffset, _RequestScreenGrab);
	pxDialog->SetHelpText("");
	pxOverlay->AddDialog(pxDialog);

	pxOverlay->m_uID = TOOLS_OVERLAY_MODE;

	Asura_Overlay_System::SubmitNewOverlay(pxOverlay,false);
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void Tools_Mode_AssetManager::RenderWorld_PreToneMap()
{
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void Tools_Mode_AssetManager::RenderWorld_PostToneMap()
{
	int iViewportWidth = Asura_Viewport::GetWidth();
	int iViewportHeight = Asura_Viewport::GetHeight();

	// Render a box around the screenshot area

	if(s_iRequestScreenGrab == -1)
	{
		const float fTop = SCREENSHOT_TOP + 20.0f;
		const float fLeft = SCREENSHOT_LEFT + 20.0f;
		const float fBottom = SCREENSHOT_BOTTOM - 20.0f;
		const float fRight = SCREENSHOT_RIGHT - 20.0f;

		const float fCornerSize = 100.0f;
		const u_int uCornerColour = 0xff000000;

		Asura_Render::Render2DLine(fLeft, fTop, fLeft + fCornerSize, fTop, uCornerColour);
		Asura_Render::Render2DLine(fLeft, fTop, fLeft, fTop + fCornerSize, uCornerColour);

		Asura_Render::Render2DLine(fRight, fTop, fRight - fCornerSize, fTop, uCornerColour);
		Asura_Render::Render2DLine(fRight, fTop, fRight, fTop + fCornerSize, uCornerColour);

		Asura_Render::Render2DLine(fRight, fBottom, fRight - fCornerSize, fBottom, uCornerColour);
		Asura_Render::Render2DLine(fRight, fBottom, fRight, fBottom - fCornerSize, uCornerColour);

		Asura_Render::Render2DLine(fLeft, fBottom, fLeft + fCornerSize, fBottom, uCornerColour);
		Asura_Render::Render2DLine(fLeft, fBottom, fLeft, fBottom - fCornerSize, uCornerColour);
	}

	// Render the translucent mask over the non screenshot area

	Asura_Render::Overlay.Colour			= /*(m_uBackgroundColour & ~0xff000000) | */ 0xBF000000;
	Asura_Render::Overlay.TextureIndex		= ASURA_TEXTURE_NONE;
	Asura_Render::Overlay.TranslucencyMode	= ASURA_TRANSLUCENCY_NORMAL;
	Asura_Render::Overlay.SetRectXY(0.0f, 0.0f, (float)SCREENSHOT_LEFT, (float)iViewportHeight);
	Asura_Render::Overlay.Render();
	Asura_Render::Overlay.SetRectXY((float)SCREENSHOT_RIGHT, 0.0f, (float)(iViewportWidth - SCREENSHOT_RIGHT), (float)iViewportHeight);
	Asura_Render::Overlay.Render();
	Asura_Render::Overlay.SetRectXY((float)SCREENSHOT_LEFT, 0.0f, (float)(SCREENSHOT_RIGHT - SCREENSHOT_LEFT), (float)SCREENSHOT_TOP);
	Asura_Render::Overlay.Render();
	Asura_Render::Overlay.SetRectXY((float)SCREENSHOT_LEFT, (float)SCREENSHOT_BOTTOM, (float)(SCREENSHOT_RIGHT - SCREENSHOT_LEFT), (float)(iViewportHeight - SCREENSHOT_TOP));
	Asura_Render::Overlay.Render();
	Asura_Render::FlushDisplay();

	if(s_iRequestScreenGrab != -1)
	{
		Asura_DebugText::AddTextBlock((u_int)((SCREENSHOT_LEFT / ASURA_TEXT_OUTPUT_FONT_WIDTH) + 2), (u_int)((SCREENSHOT_BOTTOM / ASURA_TEXT_OUTPUT_FONT_HEIGHT) + 2), 0xffffff00, 0, "Screengrab in progress ...");
		s_iRequestScreenGrab--;
	}

	if(s_iRequestScreenGrab == 0)
	{
		s_iRequestScreenGrab = -1;

		if(m_pxAnimation == NULL)
		{
			Tools_Timed_Message_System::AddMessage (0xffff00, "An object must be loaded first!");
			return;
		}

		// Grab the backbuffer

		Asura_Direct3D_BackBufferCapture::CaptureCurrentBackBuffer();

		Direct3D_Texture* pxTexture = Asura_Direct3D_BackBufferCapture::GetCaptureTexture();

		// Crop the backbuffer-grab to our frame and save to file

		D3DSURFACE_DESC xSurfaceDesc;
		HRESULT ret;
		Direct3D_Texture* pxCroppedTexture;
		Direct3D_Surface* pxSourceSurface;
		Direct3D_Surface* pxTargetSurface;

		ret = pxTexture->GetLevelDesc(0, &xSurfaceDesc);
		Asura_Assert(SUCCEEDED(ret), "Failed to GetSurfaceDesc");

		ret = Asura_Direct3D::D3DDevice->CreateTexture(SCREENSHOT_RIGHT - SCREENSHOT_LEFT, SCREENSHOT_BOTTOM - SCREENSHOT_TOP, 1, xSurfaceDesc.Usage, xSurfaceDesc.Format, xSurfaceDesc.Pool, &pxCroppedTexture, NULL);
		Asura_Assert(SUCCEEDED(ret), "Failed to create cropped texture");

		ret = pxTexture->GetSurfaceLevel(0, &pxSourceSurface);
		Asura_Assert(SUCCEEDED(ret), "Failed to GetSurfaceLevel");

		ret = pxCroppedTexture->GetSurfaceLevel(0, &pxTargetSurface);
		Asura_Assert(SUCCEEDED(ret), "Failed to GetSurfaceLevel");

		RECT xRect = {SCREENSHOT_LEFT + 1, SCREENSHOT_TOP + 1, SCREENSHOT_RIGHT, SCREENSHOT_BOTTOM};

		ret = Asura_Direct3D::D3DDevice->StretchRect(pxSourceSurface, &xRect, pxTargetSurface, NULL, D3DTEXF_POINT);
		Asura_Assert(SUCCEEDED(ret), "Failed to blit texture");

		pxSourceSurface->Release();
		pxTargetSurface->Release();

		D3DXSaveTextureToFileA("Graphics\\Am_Preview.jpg", D3DXIFF_JPG, pxCroppedTexture, NULL);

		// Create the smaller thumbnail

		Direct3D_Texture* pxThumbnailTexture;

		ret = Asura_Direct3D::D3DDevice->CreateTexture(128, 128, 1, xSurfaceDesc.Usage, xSurfaceDesc.Format, xSurfaceDesc.Pool, &pxThumbnailTexture, NULL);
		Asura_Assert(SUCCEEDED(ret), "Failed to create thumbnail texture");

		ret = pxCroppedTexture->GetSurfaceLevel(0, &pxSourceSurface);
		Asura_Assert(SUCCEEDED(ret), "Failed to GetSurfaceLevel");

		ret = pxThumbnailTexture->GetSurfaceLevel(0, &pxTargetSurface);
		Asura_Assert(SUCCEEDED(ret), "Failed to GetSurfaceLevel");

		ret = Asura_Direct3D::D3DDevice->StretchRect(pxSourceSurface, NULL, pxTargetSurface, NULL, D3DTEXF_LINEAR);
		Asura_Assert(SUCCEEDED(ret), "Failed to blit texture");

		D3DXSaveTextureToFileA("Graphics\\Am_Thumbnail.jpg", D3DXIFF_JPG, pxThumbnailTexture, NULL);

		pxSourceSurface->Release();
		pxTargetSurface->Release();

		pxThumbnailTexture->Release();
		pxCroppedTexture->Release();

		// Fill in asset information

		m_xAssetInfo.xTextureFileName = "\\Am_Preview.jpg";
		m_xAssetInfo.xThumbnailFileName = "\\Am_Thumbnail.jpg";

		m_xAssetInfo.iPolyCount = 0;

		m_xAssetInfo.xNameList.Clear();

		Asura_Hierarchy_Skin* pxSkin = m_pxAnimation->GetSkin();

		if(pxSkin)
		{
			if(pxSkin->IsSkinnedModel())
			{
				Asura_Mesh* pxMesh = pxSkin->GetMesh();

				m_xAssetInfo.iPolyCount = pxMesh->NumberOfPolygons;

				for(int p = 0; p < m_xAssetInfo.iPolyCount; p++)
				{
					Asura_Mesh_TextureInfo* pxTexInfo = &pxMesh->TextureDataPtr[p];
					AddTextureToList(pxTexInfo);
				}

				m_xAssetInfo.xBoundingBoxSize = pxMesh->BoundingBox.GetMax() - pxMesh->BoundingBox.GetMin();
			}
			else
			{
				Asura_Bounding_Box xBoundingBox;

				bool bFirst = true;

				for(int b = 0; b < pxSkin->NumberOfBones; b++)
				{
					Asura_Mesh* pxMesh = pxSkin->SectionData[b].Mesh;

					if(pxMesh)
					{
						m_xAssetInfo.iPolyCount += pxMesh->NumberOfPolygons;

						for(int p = 0; p < pxMesh->NumberOfPolygons; p++)
						{
							Asura_Mesh_TextureInfo* pxTexInfo = &pxMesh->TextureDataPtr[p];
							AddTextureToList(pxTexInfo);
						}

						if(bFirst)
						{
							bFirst = false;
							xBoundingBox = pxMesh->BoundingBox;
						}
						else
						{
							xBoundingBox.Intersection(pxMesh->BoundingBox);
						}
					}
				}

				m_xAssetInfo.xBoundingBoxSize = xBoundingBox.GetMax() - xBoundingBox.GetMin();
			}
		}

		CreateAssetInfoOverlay();
	}
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void Tools_Mode_AssetManager::LoadAsset()
{
	// Open the 'load file' dialog box

	Tools_FileSystem_DialogInfo xDialogInfo;

	static char szDefaultPath[256] = "Characters";

	xDialogInfo.DefaultExtension = ".asr";
	xDialogInfo.DefaultFilename = NULL;
	xDialogInfo.DefaultPath = szDefaultPath;
	xDialogInfo.AllowMultipleSelect = false;
	xDialogInfo.FileFilter = "Asura Files\0*.asr;\0";
	xDialogInfo.WindowTitle = "Choose file to achieve";

	m_xAssetInfo.xASRFileName = Tools_FileSystem::GetLoadFilename(xDialogInfo);

	if(m_xAssetInfo.xASRFileName.IsNull())
	{
		return;
	}

	Asura_String xPathOnly;
	m_xAssetInfo.xASRFileName.SplitFilename(&xPathOnly, NULL, NULL);
	strncpy(szDefaultPath, xPathOnly.GetString(), 256);

	// Check the type of asura file

	switch(Tools_AsuraFileContents::GetAsuraFileType(m_xAssetInfo.xASRFileName.GetString()))
	{
		case AsuraFile_Shapes:
		case AsuraFile_Skin:
		case AsuraFile_Anim:
		{
			break;
		}
		default:
		{
			Tools_Timed_Message_System::AddMessage (0xff0000, "Sorry, this type of ASR file is not supported.");
			return;
		}
	}

	// Work out the skin name from the filename

	Asura_String xSkinName;

	m_xAssetInfo.xASRFileName.SplitFilename(NULL, &xSkinName, NULL);

	Asura_Hierarchy_Skin* pxSkin = Asura_Animation_System::GetSkin(xSkinName.GetString());
	if(!pxSkin)
	{
		LoadAsuraFile(m_xAssetInfo.xASRFileName.GetString());
		pxSkin = Asura_Animation_System::GetSkin(xSkinName.GetString());
	}

	Asura_Assert(pxSkin, "");

	if(m_pxAnimation)
	{
		delete m_pxAnimation;
	}

	m_pxAnimation = new Asura_Animation(xSkinName.GetString());

	Asura_Assert(m_pxAnimation, "");
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void Tools_Mode_AssetManager::CreateAssetInfoOverlay()
{
	// Create the overlay

	m_pxAssetInfoOverlay = new Asura_Overlay("Asset Manager", false, &s_xAssetInfoOverlayPosition);
	m_pxAssetInfoOverlay->m_uID = TOOLS_OVERLAY_ASSET_MANAGER;

	// Load the texture

	int iTextureIndex = Asura_Texture_Management::GetTextureIndex(m_xAssetInfo.xTextureFileName);

	if(iTextureIndex != ASURA_TEXTURE_NONE)
	{
		Asura_Texture_Management::DestroyTexture(iTextureIndex);
	}

	u_int uOldResetGroup = Asura_Texture_Management::GetTextureResetGroup();
	Asura_Texture_Management::SetTextureResetGroup(ASURA_TEXTURE_LOADED_PERMANENT);
	iTextureIndex = Asura_Texture_Management::AddTexture(m_xAssetInfo.xTextureFileName);
	Asura_Texture_Management::SetTextureFlags(iTextureIndex, AsuraTextureFlag_HighQualityCompression);
	Asura_Texture_Management::SetTextureResetGroup(uOldResetGroup);

	Asura_Texture_Management::LoadTextures();

	// Add the dialogs

	Asura_Vector_2 xPosition(1.0f, 2.5f);

	Asura_Image_Dialog* pxImage = new Asura_Image_Dialog(xPosition, iTextureIndex, 431.0f, 428.0f);
	m_pxAssetInfoOverlay->AddDialog(pxImage);

	Asura_Function_Dialog* pxButton = new Asura_Function_Dialog("Add Asset to Database", xPosition + Asura_Vector_2(0.0f, 22.0f), _AddToDatabase);
	pxButton->SetHelpText("");
	m_pxAssetInfoOverlay->AddDialog(pxButton);

	pxButton = new Asura_Function_Dialog("Cancel", xPosition + Asura_Vector_2(20.0f, 22.0f), _CloseAssetInfoOverlay);
	pxButton->SetHelpText("");
	m_pxAssetInfoOverlay->AddDialog(pxButton);

	xPosition.x += 35.0f;

	Asura_StringDisplay_Dialog* pxStringDisplay = new Asura_StringDisplay_Dialog("Filename:", xPosition);
	m_pxAssetInfoOverlay->AddDialog(pxStringDisplay);

	Asura_String xFileNameOnly, xExtensionOnly;
	m_xAssetInfo.xASRFileName.SplitFilename(NULL, &xFileNameOnly, &xExtensionOnly);
	xFileNameOnly += "." + xExtensionOnly;

	pxStringDisplay = new Asura_StringDisplay_Dialog(xFileNameOnly.GetString(), xPosition + Asura_Vector_2(10.0f, 0.0f));
	m_pxAssetInfoOverlay->AddDialog(pxStringDisplay);

	xPosition.y += Asura_Dialog::DefaultSpacing();

	pxStringDisplay = new Asura_StringDisplay_Dialog("Category:", xPosition);
	pxStringDisplay->SetColour(0xffffff00);
	m_pxAssetInfoOverlay->AddDialog(pxStringDisplay);

	m_uCategorySelectionIndex = 0;
	m_pxCategoryDropdown = new Asura_DropDown_List_Dialog(xPosition + Asura_Vector_2(10.0f, 0.0f), s_paszCategoryNames, &m_uCategorySelectionIndex, 6, 6, NULL);
	m_pxAssetInfoOverlay->AddDialog(m_pxCategoryDropdown);

	xPosition.y += Asura_Dialog::DefaultSpacing();

	pxStringDisplay = new Asura_StringDisplay_Dialog("Project:", xPosition);
	m_pxAssetInfoOverlay->AddDialog(pxStringDisplay);

	pxStringDisplay = new Asura_StringDisplay_Dialog(PROJECT_NAME, xPosition + Asura_Vector_2(10.0f, 0.0f));
	m_pxAssetInfoOverlay->AddDialog(pxStringDisplay);
	xPosition.y += Asura_Dialog::DefaultSpacing();

	pxStringDisplay = new Asura_StringDisplay_Dialog("Artist:", xPosition);
	pxStringDisplay->SetColour(0xffffff00);
	m_pxAssetInfoOverlay->AddDialog(pxStringDisplay);

	Asura_String_Dialog* pxTextbox = new Asura_String_Dialog("", xPosition + Asura_Vector_2(10.0f, 0.0f), &m_xArtistString, 30);
	m_pxAssetInfoOverlay->AddDialog(pxTextbox);
	xPosition.y += Asura_Dialog::LargeSpacing();

	pxStringDisplay = new Asura_StringDisplay_Dialog("Polycount:", xPosition);
	m_pxAssetInfoOverlay->AddDialog(pxStringDisplay);

	Asura_Char szBuffer[256];
	Asura_PrintfToBfr(szBuffer, "%d", 256, true, m_xAssetInfo.iPolyCount);
	pxStringDisplay = new Asura_StringDisplay_Dialog(szBuffer, xPosition + Asura_Vector_2(10.0f, 0.0f));
	m_pxAssetInfoOverlay->AddDialog(pxStringDisplay);
	xPosition.y += Asura_Dialog::LargeSpacing();

	pxStringDisplay = new Asura_StringDisplay_Dialog("Textures:", xPosition);
	m_pxAssetInfoOverlay->AddDialog(pxStringDisplay);
	xPosition.x += 10.0f;

	float fMaxTexturePathWidth = 0.0f;

	for(int n = 0; n < m_xAssetInfo.xNameList.GetNumUsedEntries(); n++)
	{
		Asura_String xPathOnly;
		m_xAssetInfo.xNameList.GetName(n).SplitFilename(&xPathOnly, NULL, NULL);

		float fThisWidth = Asura_Font_System::GetWidthOfString(xPathOnly.GetString());

		fMaxTexturePathWidth = Asura_Maths::Max(fMaxTexturePathWidth, fThisWidth);
	}

	fMaxTexturePathWidth /= ASURA_TEXT_OUTPUT_FONT_WIDTH;

	for(int n = 0; n < m_xAssetInfo.xNameList.GetNumUsedEntries(); n++)
	{
		Asura_String xPathOnly, xFileNameOnly, xExtensionOnly;
		m_xAssetInfo.xNameList.GetName(n).SplitFilename(&xPathOnly, &xFileNameOnly, &xExtensionOnly);
		xFileNameOnly += "." + xExtensionOnly;
		if(!xPathOnly.EndsWith("\\")) { xPathOnly += "\\"; }

		pxStringDisplay = new Asura_StringDisplay_Dialog(xPathOnly.GetString(), xPosition);
		m_pxAssetInfoOverlay->AddDialog(pxStringDisplay);

		pxStringDisplay = new Asura_StringDisplay_Dialog(xFileNameOnly.GetString(), xPosition + Asura_Vector_2(fMaxTexturePathWidth + 3.0f, 0.0f));
		m_pxAssetInfoOverlay->AddDialog(pxStringDisplay);

		xPosition.y += Asura_Dialog::DefaultSpacing();
	}

	xPosition.y += Asura_Dialog::LargeSpacing();
	xPosition.x -= 10.0f;

	pxStringDisplay = new Asura_StringDisplay_Dialog("Size:", xPosition);
	m_pxAssetInfoOverlay->AddDialog(pxStringDisplay);

	Asura_PrintfToBfr(szBuffer, "%.2f x %.2f x %.2f", 256, true, m_xAssetInfo.xBoundingBoxSize.x, m_xAssetInfo.xBoundingBoxSize.y, m_xAssetInfo.xBoundingBoxSize.z);
	pxStringDisplay = new Asura_StringDisplay_Dialog(szBuffer, xPosition + Asura_Vector_2(10.0f, 0.0f));
	m_pxAssetInfoOverlay->AddDialog(pxStringDisplay);

	xPosition.y += Asura_Dialog::DefaultSpacing();

	pxStringDisplay = new Asura_StringDisplay_Dialog("Flags:", xPosition);
	m_pxAssetInfoOverlay->AddDialog(pxStringDisplay);
	xPosition.y += Asura_Dialog::LargeSpacing();

	pxStringDisplay = new Asura_StringDisplay_Dialog("Tags:", xPosition);
	pxStringDisplay->SetColour(0xffffff00);
	m_pxAssetInfoOverlay->AddDialog(pxStringDisplay);

	m_xTagsString = "";
	pxTextbox = new Asura_String_Dialog("", xPosition + Asura_Vector_2(10.0f, 0.0f), &m_xTagsString, 30);
	pxTextbox->SetMaxStringLength(512);
	m_pxAssetInfoOverlay->AddDialog(pxTextbox);
	xPosition.y += Asura_Dialog::LargeSpacing();

	m_xNotesString = "";
	pxStringDisplay = new Asura_StringDisplay_Dialog("Notes:", xPosition);
	pxStringDisplay->SetColour(0xffffff00);
	m_pxAssetInfoOverlay->AddDialog(pxStringDisplay);

	Asura_MultiLineString_Dialog* pxMultiLineTextbox = new Asura_MultiLineString_Dialog(xPosition + Asura_Vector_2(10.0f, 0.0f), &m_xNotesString, 30.0f, 10.0f, NULL);
	m_pxAssetInfoOverlay->AddDialog(pxMultiLineTextbox);
	xPosition.y += Asura_Dialog::DefaultSpacing();

	// Submit the overlay
	Asura_Overlay_System::SubmitNewOverlay(m_pxAssetInfoOverlay);
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void Tools_Mode_AssetManager::CloseAssetInfoOverlay()
{
	Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_ASSET_MANAGER);
	m_pxAssetInfoOverlay = NULL;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void Tools_Mode_AssetManager::AddToDatabase()
{
	char szAssetPath[256];
	Asura_String xAssetName;
	char szBuffer[256];
	Asura_String xExtension;
	Asura_String xNameOnly;
	Direct3D_Texture* pxTexture;
	Direct3D_CubeTexture* pxCubeTexture;
	D3DSURFACE_DESC xSurfaceDesc;
	BOOL ret;

	// Check that the R drive is mapped

	if(GetDriveType("r:\\") == 1)
	{
		// R drive doesn't exist, so show an error to the user

		Tools_Timed_Message_System::AddMessage (0xffff00, "There is no R drive mapped to the art server!");
		return;
	}

	int iTextureIndex = Asura_Texture_Management::GetTextureIndex(m_xAssetInfo.xTextureFileName);

	if(iTextureIndex != ASURA_TEXTURE_NONE)
	{
		Asura_Texture_Management::DestroyTexture(iTextureIndex);
	}

	m_xAssetInfo.xASRFileName.SplitFilename(NULL, &xAssetName, &xExtension);

	Asura_PrintfToBfr(szAssetPath, "%s\\%s.%s", 256, true, ASSET_DATA_FOLDER, PROJECT_NAME, xAssetName.GetString());

	// Create a new folder for this asset

	CreateDirectoryA(szAssetPath, NULL);

	// Copy the ASR file to the folder

	Asura_PrintfToBfr(szBuffer, "%s\\%s.%s", 256, true, szAssetPath, xAssetName.GetString(), xExtension.GetString());

	DWORD dwAttributes = GetFileAttributes(szBuffer);
	SetFileAttributes(szBuffer, dwAttributes & ~FILE_ATTRIBUTE_READONLY);

	ret = CopyFileA(m_xAssetInfo.xASRFileName.GetString(), szBuffer, false);
	if(!ret) { Asura_SimpleLogfileOutput((szCMD_LINE_LOGFILE, "Error = %ld\n", GetLastError())); }
	Asura_Assert(ret, "Failed to copy file");

	// Copy the render to the folder

	m_xAssetInfo.xTextureFileName.SplitFilename(NULL, &xNameOnly, &xExtension);

	Asura_PrintfToBfr(szBuffer, "%s\\%s.%s", 256, true, szAssetPath, xNameOnly.GetString(), xExtension.GetString());

	Asura_String xSourceFileName = "Graphics\\" + m_xAssetInfo.xTextureFileName;

	DeleteFileA(szBuffer);
	ret = CopyFileA(xSourceFileName.GetString(), szBuffer, false);
	if(!ret) { Asura_SimpleLogfileOutput((szCMD_LINE_LOGFILE, "Error = %ld\n", GetLastError())); }
	Asura_Assert(ret, "Failed to copy file");

	// Copy the thumbnail to the folder

	m_xAssetInfo.xThumbnailFileName.SplitFilename(NULL, &xNameOnly, &xExtension);

	Asura_PrintfToBfr(szBuffer, "%s\\%s.%s", 256, true, szAssetPath, xNameOnly.GetString(), xExtension.GetString());

	xSourceFileName = "Graphics\\" + m_xAssetInfo.xThumbnailFileName;

	ret = CopyFileA(xSourceFileName.GetString(), szBuffer, false);
	if(!ret) { Asura_SimpleLogfileOutput((szCMD_LINE_LOGFILE, "Error = %ld\n", GetLastError())); }
	Asura_Assert(ret, "Failed to copy file");

	// Generate the metadata xml file

	Asura_PrintfToBfr(szBuffer, "%s\\MetaData.xml", 256, true, szAssetPath);

	FILE* xFile = fopen(szBuffer, "w");

	Asura_Assert(xFile, "Failed to create metadata xml file");

	if(xFile)
	{
		fprintf(xFile, "<Root>\n");
		fprintf(xFile, "\t<Category>%s</Category>\n", s_paszCategoryNames[m_uCategorySelectionIndex]);
		fprintf(xFile, "\t<ProjectName>%s</ProjectName>\n", PROJECT_NAME);
		fprintf(xFile, "\t<ArtistName>%s</ArtistName>\n", m_xArtistString.GetString());
		fprintf(xFile, "\t<PolyCount>%d</PolyCount>\n", m_xAssetInfo.iPolyCount);

		fprintf(xFile, "\t<Textures>\n");
		for(int n = 0; n < m_xAssetInfo.xNameList.GetNumUsedEntries(); n++)
		{
			fprintf(xFile, "\t\t<FileName>%s</FileName>\n", m_xAssetInfo.xNameList.GetName(n));
		}
		fprintf(xFile, "\t</Textures>\n");

		fprintf(xFile, "\t<TextureInfo>\n");
		for(int n = 0; n < m_xAssetInfo.xNameList.GetNumUsedEntries(); n++)
		{
			bool bDone = false;

			ret = D3DXCreateTextureFromFile(Asura_Direct3D::D3DDevice, ("Graphics" + m_xAssetInfo.xNameList.GetName(n)).GetString(), &pxTexture);

			if(FAILED(ret))
			{
				// Creation failed, so try loading it as a cubemap texture

				ret = D3DXCreateCubeTextureFromFile(Asura_Direct3D::D3DDevice, ("Graphics" + m_xAssetInfo.xNameList.GetName(n)).GetString(), &pxCubeTexture);

				if(SUCCEEDED(ret))
				{
					ret = pxCubeTexture->GetLevelDesc(0, &xSurfaceDesc);
					Asura_Assert(SUCCEEDED(ret), "Failed to get dimension and format information from cube texture.");

					if(SUCCEEDED(ret))
					{
						fprintf(xFile, "\t\t<Info>%u:%u:%u</Info>\n", xSurfaceDesc.Format, xSurfaceDesc.Width, xSurfaceDesc.Height);
						bDone = true;
					}
				}
			}

			Asura_Assert(SUCCEEDED(ret), "Failed to create texture to get dimension and format information.");

			if(!bDone)
			{
				if(SUCCEEDED(ret))
				{
					ret = pxTexture->GetLevelDesc(0, &xSurfaceDesc);
					Asura_Assert(SUCCEEDED(ret), "Failed to get dimension and format information from texture.");

					if(SUCCEEDED(ret))
					{
						fprintf(xFile, "\t\t<Info>%u:%u:%u</Info>\n", xSurfaceDesc.Format, xSurfaceDesc.Width, xSurfaceDesc.Height);
						bDone = true;
					}
				}
			}

			if(!bDone)
			{
				fprintf(xFile, "\t\t<Info>-1:-1:-1</Info>\n");
			}
		}
		fprintf(xFile, "\t</TextureInfo>\n");

		fprintf(xFile, "\t<Dimensions>\n");
		fprintf(xFile, "\t\t<x>%f</x>\n", m_xAssetInfo.xBoundingBoxSize.x);
		fprintf(xFile, "\t\t<y>%f</y>\n", m_xAssetInfo.xBoundingBoxSize.y);
		fprintf(xFile, "\t\t<z>%f</z>\n", m_xAssetInfo.xBoundingBoxSize.z);
		fprintf(xFile, "\t</Dimensions>\n");
		fprintf(xFile, "\t<Tags>%s</Tags>\n", m_xTagsString.GetString());
		fprintf(xFile, "\t<Notes>%s</Notes>\n", m_xNotesString.GetString());
		fprintf(xFile, "</Root>\n");

		fclose(xFile);
	}

	// Copy all the textures

	for(int n = 0; n < m_xAssetInfo.xNameList.GetNumUsedEntries(); n++)
	{
		Asura_String xTargetFile = szAssetPath + m_xAssetInfo.xNameList.GetName(n);
		Asura_String xPath;

		xTargetFile.SplitFilename(&xPath, NULL, NULL);

		int iCharIndex;

		xPath.Contains("\\", &iCharIndex);

		while(iCharIndex != -1)
		{
			int iLastCharIndex = iCharIndex + 1;

			Asura_String xTemp = xPath.SubString(iLastCharIndex, xPath.GetLength() - iCharIndex);

			if(xTemp.Contains("\\", &iCharIndex))
			{
				iCharIndex += iLastCharIndex;
			}
			else
			{
				iCharIndex = -1;
			}

			Asura_String xBuffer = xPath.SubString(0, iCharIndex);

			Tools_FileSystem::EnsureDirectoryExists(xBuffer.GetString(), true);
		}

		xSourceFileName = "Graphics" + m_xAssetInfo.xNameList.GetName(n);

		BOOL ret = CopyFileA(xSourceFileName.GetString(), xTargetFile.GetString(), false);
		__Asura_OutputToDebugger("Error = %ld\n", GetLastError());

		if(!ret)
		{
			Tools_Timed_Message_System::AddMessage (0xffff00, ("Texture not found: " + xSourceFileName).GetString());
		}
	}

	delete m_pxAnimation;
	m_pxAnimation = NULL;

	// Completion message to the user

	Tools_Timed_Message_System::AddMessage (0xff00ff00, "Asset added to database.");

	Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_ASSET_MANAGER);
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void Tools_Mode_AssetManager::ChangeToStockBGColour1(void*)
{
	m_uBackgroundColour = g_auStockBGColours[0];
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void Tools_Mode_AssetManager::ChangeToStockBGColour2(void*)
{
	m_uBackgroundColour = g_auStockBGColours[1];
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void Tools_Mode_AssetManager::ChangeToStockBGColour3(void*)
{
	m_uBackgroundColour = g_auStockBGColours[2];
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void Tools_Mode_AssetManager::ChangeToStockBGColour4(void*)
{
	m_uBackgroundColour = g_auStockBGColours[3];
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void Tools_Mode_AssetManager::ChangeToCustomBGColour()
{
	bool bChanged;

	float fRed = (float)((u_char)(m_uBackgroundColour >> 16)) / 255.0f;
	float fGreen = (float)((u_char)(m_uBackgroundColour >> 8)) / 255.0f;
	float fBlue = (float)((u_char)m_uBackgroundColour) / 255.0f;

	Tools_ColourPicker* pxColourPicker = new Tools_ColourPicker("Pick Background Colour", &fRed, &fGreen, &fBlue, 1.0f, &bChanged);

	pxColourPicker->DoModal();

	m_uBackgroundColour = 0xff000000 | ((static_cast<u_int>(fRed * 255.0f)) << 16) | ((static_cast<u_int>(fGreen * 255.0f)) << 8) | (static_cast<u_int>(fRed * 255.0f));
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void Tools_Mode_AssetManager::SetToDefaultBackgroundColour()
{
	m_uBackgroundColour = DEFAULT_BACKGROUND_COLOUR;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void Tools_Mode_AssetManager::AddTextureToList(Asura_Mesh_TextureInfo* pxTexInfo)
{
	for(int i = 0; i < TOOLS_NUM_MATERIALRESPONSE_PER_POLY; i++)
	{
		const Asura_MaterialResponse* pxMatResponse = Asura_MaterialResponse_System::GetMaterialByHashID(pxTexInfo->m_auMaterialResponseHashID[i]);

		for(int j = 0; j < Asura_MaterialResponse::NUMBER_OF_TEXTURE_SLOTS; j++)
		{
			int iTextureIndex = pxMatResponse->GetTexture((Asura_MaterialResponse::TEXTURE_SLOT_ID) j);

			const Asura_String* pxTextureName = Asura_Texture_Management::GetTextureName(iTextureIndex);

			if(pxTextureName)
			{
				if(m_xAssetInfo.xNameList.GetIndexOfName(*pxTextureName) == -1)
				{
					m_xAssetInfo.xNameList.AddName(*pxTextureName);
				}
			}
		}
	}
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#endif // ASURA_USE_ASSET_MANAGER

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
