//
// Tools_Attractor.h
//
// Created by JGP 10/05/06
//
// Copyright (C) 2006 Rebellion (Derby), All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __TOOLS_ATTRACTOR_H__
#define __TOOLS_ATTRACTOR_H__


// base class
#include "Attractors/Asura_Attractor_Data.h"


/*
	Tools_Attractor

	Storage for a single attractor. 
	Contains a pointer to the first
	attractor node in the attractor
	and settings for the attractor.
*/
class Tools_Attractor : public Asura_Attractor_Data
{

public:

	// friend for editing class
	friend class Asura_Chunk_Tools_Attractors;
	friend class Tools_Object_Attractor_Node;

	Tools_Attractor(u_int uID);
	Tools_Attractor(bool bMissionFile);
	~Tools_Attractor();

	void SetFirstNode( class Tools_Attractor_Node* pxFirstNode ) { m_pxFirstNode = pxFirstNode; }
	class Tools_Attractor_Node* GetFirstNode() { return m_pxFirstNode; }
	const class Tools_Attractor_Node* GetFirstNode() const { return m_pxFirstNode; }

	void SetClosed( const bool bClosed ) { m_bClosed = bClosed; }
	const bool IsClosed() const { return m_bClosed; }

	//determines how many attractors this will become in the asura system	
	u_int CountCompiledAttractors() const;
	bool HasAtLeastTwoNodes() const; 

	//Add/remove links to other attractors
	void AddLink(u_int uID);
	void RemoveLink(u_int uID);

	Asura_Guid GetTemplateGuid() const {return m_uTemplateGuid;}
	void SetTemplateGuid(Asura_Guid uGuid) {m_uTemplateGuid = uGuid;}

	Asura_Guid GetControllerGuid() const;
	void SetControllerGuid(Asura_Guid uGuid) {m_uControllerGuid = uGuid;}

	int GetTemplateVersion() const {return m_iTemplateVersion;}
	void SetTemplateVersion(int iVersion) {m_iTemplateVersion = iVersion;}

	Asura_Guid GetParentGuid() const {return m_uParentEntityGuid;}
	void SetParentGuid(Asura_Guid uGuid) {m_uParentEntityGuid = uGuid;}

	bool IsParentRelative() const {return m_uParentEntityGuid != ASURA_GUID_UNREGISTERED;}

	void ApplyChangesForTemplateInstance(class Tools_EntityTemplateInstance* pxInstance);

	void GetNormalFromDirection(Asura_Vector_3& xNormal, const Asura_Vector_3& xDirection) const;

	// Sets up the rotation from a normal. Tries to take the fact the normal may not actually be perpendicular into account
	// pxSelectedNode is used to determine which part of the attractor to base the calculations on
	void SetNormal(const Asura_Vector_3& xNormal,const Tools_Attractor_Node* pxSelectedNode = 0);

	//Sets attractor type and uses any default properties for the type
	//pxSelectedNode is used with horizontal/vertical types for determining which position to use.
	void SetAttractorType(Asura_Hash_ID uHash,const Tools_Attractor_Node* pxSelectedNode = 0);

	//updates non-editable properties from the attractor type
	void UpdateFromType(const Tools_Attractor_Node* pxSelectedNode = 0);

	//Deals with upgrading old env hooks to deal with the new way in which near vertical attractors are handled
	void HandleUpdateForChangeToNormalCalculationForNearVerticalEnvHooks();

	bool BelongsToMissionFile() const;

	//Flags the file that contains this attractor as modified	
	void SetModified() const;

	//returns true if any of the tools objects for the nodes are selected
	bool AreAnyNodesSelected() const;

	//As above but will also return true for nodes within templates that are selected
	bool AreAnyNodesOrTheirRootObjectsSelected() const;


	float GetAngleForNormal(void) const		{	return m_fAngleForNormal; }

private:
	void Initialise();

	//For attractors that are part of an entity template
	Asura_Guid m_uTemplateGuid;
	int m_iTemplateVersion;

	//Guid of controller entity. Used for attractors that need to recieve activate/deactivate messages
	mutable Asura_Guid m_uControllerGuid;

	//if set then the attractor stores its position relative to the entity
	Asura_Guid m_uParentEntityGuid;

	class Tools_Attractor_Node* m_pxFirstNode;
	bool m_bClosed;

	// An angle that defines the normal for this attractor - a rotation around the direction from one node to the next.
	// This means the angle must work for all node pairs; if it doesn't the tools attractor needs splitting up.
	// Defining in this way makes things harder to envisage from the tools values, but allows easier editing (also ensuring this is a tangent from the line,
	// which is more difficult with a vector), can be applied to all node pairs, works with relative attractors and supports deleting nodes.
	float m_fAngleForNormal;
};


#endif // __TOOLS_ATTRACTOR_H__
