//
// Tools_Mode_Attractors.h
//
// Created by JGP 02/05/06
//
// Copyright (C) 2006 Rebellion (Derby), All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __TOOLS_MODE_ATTRACTORS_H__
#define __TOOLS_MODE_ATTRACTORS_H__

// base class
#include "Tools_Mode.h"

// required
#include "Core/Asura_Maths.h"
#include "Core/Asura_List.h"
#include "Misc/list_tem.hpp"


/*
	Tools_Mode_Attractors

	Tools mode for creating and
	editing attractors.
*/
class Tools_Mode_Attractors : public Tools_Mode
{

public:

	Tools_Mode_Attractors();
	virtual ~Tools_Mode_Attractors();

	// Tools_Mode functions

	virtual void InitialiseMode();
	virtual void ShutdownMode();

	// set modified

	virtual void QuickSave();
	virtual bool IsModified();
	virtual void Update();
	virtual void RenderWorld_PostToneMap();

	static void ShowDisplayFilterOverlay();
	static void CloseDisplayFilterOverlay();

	static bool IsAttractorTypeVisible(Asura_Hash_ID uHashID);
	static void ShowAllAttractorTypes();
	static void HideAllAttractorTypes();
	static void InvertAttractorTypesVisible();
	static bool *GetAttractorTypeVisibilityFlag(Asura_Hash_ID uHashID);

	static class Tools_Attractor_Node* const CreateAttractorNode( class Tools_Attractor_Node* const pxNode, const Asura_Vector_3* const pxPos = NULL )
	{
		if (s_pxThis)
		{
			return s_pxThis->AddNewAttractorNode(pxNode, pxPos);
		}
		return NULL;
	}

protected:
	STATIC_VOID_FN( OnMakeEdgeAttractorNode );
	static Asura_Hash_ID s_xDefaultBlueprintHash;

private:
	void TransferModeData(int iMode);

	// update

	void HandleKeyboard();

	// Initialise / Close Overlay

	void InitialiseOverlay();
	void CloseOverlay();

	// callbacks

	STATIC_VOID_FN( OnAddAttractorNode );
	STATIC_BOOL_FN( IsAddAttractorNodeEnabled );
	STATIC_BOOL_FN( IsMakeEdgeAttractorNodeEnabled );

	STATIC_VOID_FN( FindByID )
	STATIC_VOID_FN( FindByID_MissionFile )

	void FindByIDMain(bool bMissionFile);

	// functions

	virtual class Tools_Attractor_Node* const AddNewAttractorNode( class Tools_Attractor_Node* const pxNode, const Asura_Vector_3* const pxPos = NULL );

	// static this pointer

	static Tools_Mode_Attractors* s_pxThis;

	// static data

	static Asura_Vector_2 s_xOverlayPosition;
	static Asura_Vector_2 s_xDisplayFilterPosition;

	static bool s_bUseDefaultRadius;
	static float s_fDefaultRadius;

	static Asura_TemplateHashTree<bool> s_xBluePrintTypesVisible;

	// data

	bool m_bEdgeSelectionMode;
	Asura_Handle m_xMenuHandle;

};


#endif // __TOOLS_MODE_ATTRACTORS_H__
