//
// Tools_Object_Attractor_Node.cpp
//
// Created by JGP 02/05/06
//
// Copyright (C) 2006 Rebellion (Derby), All Rights Reserved.
//
#include "Core/Asura.h"

#ifdef ASURA_USE_ATTRACTORS

// This file's header
#include "Tools_Object_Attractor_Node.h"

// Asura headers
#include "Interface/Asura_Overlay.h"
#include "Interface/Asura_Overlay_System.h"
#include "Interface/Asura_Dialog.h"
#include "Entity/PhysicalObject/Asura_ServerEntity_PhysicalObject.h"
#include "Entity/Attractors/Asura_ServerEntity_AttractorController.h"
#include "Mesh/Asura_Mesh_System.h"
#include "Network/Asura_Message.h"
#include "Rendering/Asura_Module.h"
#include "Rendering/Asura_Render.h"
#include "Rendering/Asura_Text_Output.h"

//Tools
#include "Tools_Overlays.h"
#include "Tools_Attractor.h"
#include "Tools_Attractor_Node.h"
#include "Tools_Attractor_System.h"
#include "Tools_Mode_Attractors.h"
#include "Entity/Tools_Entity_Data.h"
#include "Interface/Tools_ContextMenu_System.h"
#include "Load&Save/Tools_SaveAsura.h"
#include "Misc/Tools_Adjacency.h"
#include "Misc/Tools_LOS.h"
#include "Misc/Tools_Prompted_Selection.h"
#include "Misc/Tools_Render.h"
#include "Objects/Tools_Object.h"
#include "Objects/Tools_Object_System.h"
#include "Objects/Tools_ObjectSystem_Param.h"

TOOLS_DECLARE_SHARED_PARAM(Asura_Hash_ID,	0x8F804289,	EnvHook_Type)
TOOLS_DECLARE_SHARED_PARAM(float,	 		0xABB88821,	EnvHook_Radius)
TOOLS_DECLARE_SHARED_PARAM(float,	 		0x5F772F64,	EnvHook_Angle)
TOOLS_DECLARE_SHARED_PARAM(Asura_String,	0x2ECB395C,	EnvHook_ParentEntity)
TOOLS_DECLARE_SHARED_PARAM(bool,	 		0x92BFFCFB,	EnvHook_Closed)
TOOLS_DECLARE_SHARED_PARAM(bool,	 		0x8ED99A95,	EnvHook_Active)




////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*-------------------------------------**
** Tools_ChooseObject_Attactor_AddLink **
**-------------------------------------*/

//choose object class for picking a linked attactor

class Tools_ChooseObject_Attractor_AddLink : public Tools_ChooseObject
{
public :
	Tools_ChooseObject_Attractor_AddLink(Tools_Object_Attractor_Node* pxAttractorObject)
	{
		m_pxAttractorObject = pxAttractorObject;
	}

	virtual bool CanBeSelected(Tools_Object* pxObject);
	virtual void ObjectSelected(Tools_Object* pxObject);
	virtual void OnFinish(){delete this;}

	Tools_Object_Attractor_Node* m_pxAttractorObject;
};

bool Tools_ChooseObject_Attractor_AddLink::CanBeSelected(Tools_Object* pxObject)
{
	//Only allow attractor objects
	if(pxObject->Type != TOT_AttractorNode) return false;

	Tools_Object_Attractor_Node* pxAttractorObject = (Tools_Object_Attractor_Node*) pxObject;

	//only allow links between attractors in the same file
	if(m_pxAttractorObject->BelongsToMissionFile() != pxAttractorObject->BelongsToMissionFile()) return false;

	//only allow attractors consisting of at least 2 nodes
	return pxAttractorObject->m_pxNode->GetNext() || pxAttractorObject->m_pxNode->GetPrev();

}

void Tools_ChooseObject_Attractor_AddLink::ObjectSelected(Tools_Object* pxObject)
{
	if(!pxObject) return;

	Asura_Assert(pxObject->Type == TOT_AttractorNode,"");

	//get the ID of the selected attractor
	Tools_Object_Attractor_Node* pxLinkObject = (Tools_Object_Attractor_Node*) pxObject;
	u_int uID = pxLinkObject->m_pxNode->GetAttractor()->GetID();

	m_pxAttractorObject->m_pxNode->GetAttractor()->AddLink(uID);
	m_pxAttractorObject->SetModified();

}

////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*----------------------------------------**
** Tools_ChooseObject_Attactor_RemoveLink **
**----------------------------------------*/

//choose object class for picking a linked attactor

class Tools_ChooseObject_Attractor_RemoveLink : public Tools_ChooseObject
{
public :
	Tools_ChooseObject_Attractor_RemoveLink(Tools_Object_Attractor_Node* pxAttractorObject)
	{
		m_pxAttractorObject = pxAttractorObject;
	}

	virtual bool CanBeSelected(Tools_Object* pxObject);
	virtual void ObjectSelected(Tools_Object* pxObject);
	virtual void OnFinish(){delete this;}

	Tools_Object_Attractor_Node* m_pxAttractorObject;
};

bool Tools_ChooseObject_Attractor_RemoveLink::CanBeSelected(Tools_Object* pxObject)
{
	//Only allow attractor objects
	if(pxObject->Type != TOT_AttractorNode) return false;

	//allow nodes in this attactor and any linked attractors to be selected
	//(allowing this attractor is just to make sure it is visible)
	Tools_Object_Attractor_Node* pxAttractorObject = (Tools_Object_Attractor_Node*) pxObject;

	const Tools_Attractor* pxThisAttractor = m_pxAttractorObject->m_pxNode->GetAttractor();
	const Tools_Attractor* pxOtherAttractor = pxAttractorObject->m_pxNode->GetAttractor();
	if( pxThisAttractor == pxOtherAttractor) return true;

	for(u_int i=0;i<pxThisAttractor->GetNumLinks();i++)
	{
		if(pxThisAttractor->GetLink(i) == pxOtherAttractor->GetID()) return true;
	}

	return false;

}

void Tools_ChooseObject_Attractor_RemoveLink::ObjectSelected(Tools_Object* pxObject)
{
	if(!pxObject) return;

	Asura_Assert(pxObject->Type == TOT_AttractorNode,"");

	//get the ID of the selected attractor
	Tools_Object_Attractor_Node* pxLinkObject = (Tools_Object_Attractor_Node*) pxObject;
	u_int uID = pxLinkObject->m_pxNode->GetAttractor()->GetID();

	m_pxAttractorObject->m_pxNode->GetAttractor()->RemoveLink(uID);
	m_pxAttractorObject->SetModified();
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*---------------------------------------**
** Tools_ChooseObject_Attactor_SetParent **
**---------------------------------------*/

//choose object class for picking attractor's parent entity

class Tools_ChooseObject_Attractor_SetParent : public Tools_ChooseObject
{
public :
	Tools_ChooseObject_Attractor_SetParent(){}

	virtual bool CanBeSelected(Tools_Object* pxObject);
	virtual void ObjectSelected(Tools_Object* pxObject);
	virtual void OnFinish(){delete this;}

};

bool Tools_ChooseObject_Attractor_SetParent::CanBeSelected(Tools_Object* pxObject)
{
	//only allow selection of entities derived from physical object
	Asura_Guid uGuid = pxObject->GetGuid();
	if(uGuid == ASURA_GUID_UNREGISTERED) return false;

	return Asura_ServerEntity_PhysicalObject::FindEntity(uGuid)!=0;
}

void Tools_ChooseObject_Attractor_SetParent::ObjectSelected(Tools_Object* pxObject)
{
	Asura_Guid uGuid = pxObject ? pxObject->GetGuid() : ASURA_GUID_UNREGISTERED;

	//Update all selected attractor objects
	//This may result in each attractor being done multiple times if more than one node in the attractor is selected ,
	//but it shouldn't really matter.
	List<Tools_Object*> xSelectedList;
	Tools_Object_System::ListSelectedObjectsOfType(TOT_AttractorNode,xSelectedList);
	
	for(LIF<Tools_Object*> xSelectedLIF(&xSelectedList);!xSelectedLIF.done();xSelectedLIF.next())
	{
		Tools_Object_Attractor_Node* pxObject = (Tools_Object_Attractor_Node*) xSelectedLIF();

		//set the new parent
		Tools_Attractor* pxAttractor = pxObject->GetNode()->GetAttractor();
		pxAttractor->SetParentGuid(uGuid);
		pxAttractor->SetModified();

		//update the relative positions in the nodes
		Tools_Attractor_Node* pxNode = pxAttractor->GetFirstNode();
		while(pxNode)
		{
			Tools_Object_Attractor_Node* pxNodeObject = pxNode->GetToolsNode();
			if(pxNodeObject) pxNodeObject->UpdateRelativePositionFromPosition();

			pxNode = pxNode->GetNext();
		}

	}
	
}


////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*-----------------------------**
** Tools_Attractor_Type_Dialog **
**-----------------------------*/

//drop down list for selecting an attractor type
class Tools_Attractor_Type_Dialog : public Asura_Hash_DropDownList_Dialog
{
public :
	Tools_Attractor_Type_Dialog(Asura_Vector_2& xOffset,int iNumberOfLines,Asura_Dialog_SharedValue<Asura_Hash_ID>* pxSharedHash,const char* szName = NULL)
	:Asura_Hash_DropDownList_Dialog(xOffset)
	{
		SetOnlyUpdateSelectionIndexOnClick(true);
		Initialise(xOffset,iNumberOfLines,pxSharedHash,0,szName);
	}
	
protected :
	virtual u_int GetNumberOfHashValues() const
	{
		return Tools_Attractor_System::GetInstance()->GetNumAttractorTypes();
	}
	
	virtual const char* GetNameByIndex(int iIndex) const
	{
		const Tools_Attractor_Type* pxType = Tools_Attractor_System::GetInstance()->GetAttractorTypeByIndex(iIndex);
		return pxType ? pxType->m_xName.GetString() : "*Error*";
	}
};


////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*----------------------------**
** Tools_Object_Attactor_Node **
**----------------------------*/

// static variables
Asura_Vector_2 Tools_Object_Attractor_Node::s_xOverlayPosition( 70.0f, 14.0f );

// list of tools
Tools_Object_Attractor_Node::NODE_LIST Tools_Object_Attractor_Node::s_xNodeList;

bool Tools_Object_Attractor_Node::s_bToolsObjectsShouldExist = false;
float Tools_Object_Attractor_Node::s_fScaleWhenCloseDist = 5.0f;


///////////////////////////////////////////////////////////////////////////////////////
// Construct / Destruct
///////////////////////////////////////////////////////////////////////////////////////


Tools_Object_Attractor_Node::Tools_Object_Attractor_Node( Tools_Attractor_Node* const pxNode )
{
	Asura_Assert(s_bToolsObjectsShouldExist,"Not expecting env hook tools objects to be created at this point");
	
	m_pxNode = pxNode;
	m_pxNode->SetToolsNode( this );


	Mesh = Asura_Mesh_System::GetMesh( "ToolsShape_Attractor" );
	Orientation.Identity();
	Position = m_pxNode->GetPosition();
	Type = TOT_AttractorNode;
	
	s_xNodeList.add_entry( this );

	//set colour based on type
	UpdateColour();

	m_bTypeChanged = false;
	m_bChanged = false;

}


Tools_Object_Attractor_Node::~Tools_Object_Attractor_Node()
{
	if(m_pxNode) m_pxNode->SetToolsNode(0);
	s_xNodeList.delete_entry( this );
}


void Tools_Object_Attractor_Node::CreateAttractorObjects()
{
	s_bToolsObjectsShouldExist = true;
	for ( Tools_Attractor_Node_List* pxItNodes = Tools_Attractor_System::GetInstance()->GetNodeList(); pxItNodes; pxItNodes = pxItNodes->Next() )
	{
		new Tools_Object_Attractor_Node( pxItNodes->Data() );
	}
}

void Tools_Object_Attractor_Node::DeleteAttractorObjects()
{
	while(s_xNodeList.size())
	{
		Tools_Object_System::DeleteObject(s_xNodeList.first_entry());
	}
	s_bToolsObjectsShouldExist = false;
}


///////////////////////////////////////////////////////////////////////////////////////
// Tools_Object over-rides
///////////////////////////////////////////////////////////////////////////////////////


const char* Tools_Object_Attractor_Node::GetName()
{
	const Tools_Attractor* const pxAttractor = m_pxNode->GetAttractor();
	Asura_Assert( pxAttractor, "Cant get env hook from  node. Error" );

	const Tools_Attractor_Node* pxNode = pxAttractor->GetFirstNode();
	int iCount = 0;
	int iIndex = -1;
	while ( pxNode )
	{
		iCount++;
		if ( pxNode == m_pxNode )
		{
			iIndex = iCount;
		}
		pxNode = pxNode->GetNext();
	}

	char szText[ 256 ];
	Asura_PrintfToBfr( szText, "Env Hook Node %d of %d", 256, true, iIndex, iCount );
	m_xName = szText;

	return m_xName.GetString();

}


void Tools_Object_Attractor_Node::UpdatePositionInAsuraData( bool bUpdateLinkedObjects )
{
	UpdateRelativePositionFromPosition();

	if(bUpdateLinkedObjects)
	{
		const Tools_Attractor_Type* pxType = GetAttractorType();
		if(pxType && (pxType->m_bMustBeHorizontal || pxType->m_bMustBeVertical))
		{
			//enforce horizontal/vertical as needed
			Tools_Attractor* pxAttractor = m_pxNode->GetAttractor();
			for(NODE_LIST_IT xNodeLIF(&s_xNodeList); !xNodeLIF.done(); xNodeLIF.next())
			{
				Tools_Object_Attractor_Node* pxObject = xNodeLIF();
				if((pxObject->m_pxNode->GetAttractor() == pxAttractor) && (pxObject != this))
				{
					if(pxType->m_bMustBeHorizontal)
					{
						pxObject->Position.y = Position.y;
					}
					if(pxType->m_bMustBeVertical)
					{
						pxObject->Position.x = Position.x;
						pxObject->Position.z = Position.z;
					}
					pxObject->UpdatePositionInAsuraData(false);
				}		
			}
		}
	}

}


void Tools_Object_Attractor_Node::UpdateWhileSelected()
{
}



void Tools_Object_Attractor_Node::Update()
{
	if (IsGroupSelected() && IsHidden())
	{
		//Don't deselect hidden objects in this way if currently choosing an object
		//RAR 22/08/2008
		if(!Tools_Object_System::IsChoosingObject())
		{
			Tools_Object_System::DeselectObject(this);
		}
	}



	// check for change of type
	if ( m_bTypeChanged )
	{
		//need to update the colour of all the tools objects that are associated with this attractor
		const Tools_Attractor* pxAttractor = m_pxNode->GetAttractor();

		for(NODE_LIST_IT xNodeLIF(&s_xNodeList); !xNodeLIF.done(); xNodeLIF.next())
		{
			if(xNodeLIF()->m_pxNode->GetAttractor() == pxAttractor)
			{
				xNodeLIF()->UpdateColour();
			}		
		}

				
		m_bTypeChanged = false;
		m_bChanged = true;
	}

	//Make sure any attractors attached to entities stay in the correct relative positions
	UpdatePositionFromRelativePosition();

	if(Position != m_pxNode->GetPosition())
	{
		m_pxNode->SetPosition( Position );
		m_bChanged = true;
	}


	if(m_bChanged)
	{
		SetModified();
		m_bChanged = false;
	}

	SetScaleWhenCloseDist(s_fScaleWhenCloseDist);
}


void Tools_Object_Attractor_Node::Render()
{
	Tools_Object::Render();
	
	m_pxNode->Render();

	const Tools_Attractor* const pxAttractor = m_pxNode->GetAttractor();
	Asura_Assert( pxAttractor, "Cant get env hook from node. Error" );

	if(IsSelected())
	{
		
		//draw lines for links to and from this attractor
		for(Asura_Hashed_List_It<Tools_Attractor> xIt(Tools_Attractor_System::GetInstance()->GetAttractorList());!xIt.Done();xIt.Next())
		{
			const Tools_Attractor* const pxAttractor2 = xIt.GetCurrent();
			if(pxAttractor == pxAttractor2) continue;

			u_int uColour = 0xff000000;
			//is this attractor linked to the other?
			for(u_int i=0;i<pxAttractor->GetNumLinks();i++)
			{
				if(pxAttractor->GetLink(i) == pxAttractor2->GetID())
				{
					uColour |= 0xff0000;
					break;
				}
			}
			//is the other attractor linked to this
			for(u_int i=0;i<pxAttractor2->GetNumLinks();i++)
			{
				if(pxAttractor2->GetLink(i) == pxAttractor->GetID())
				{
					uColour |= 0x0000ff;
					break;
				}
			}

			if(!(uColour & 0xffffff)) continue;

			//Draw a line to the nearest point in the other attractor
			float fNearDist = Asura_Maths::LargeFloat;
			Asura_Vector_3 xNearPos;

			for(const Tools_Attractor_Node* pxNode = pxAttractor2->GetFirstNode();pxNode;pxNode = pxNode->GetNext())
			{
				Asura_Vector_3 xPos = pxNode->GetPosition();
				float fDist = (xPos-Position).Magnitude();	
				if(fDist < fNearDist)
				{
					fNearDist = fDist;
					xNearPos = xPos;
				}
			}

			if(fNearDist != Asura_Maths::LargeFloat)
			{
				Asura_Render::RenderLine(Position,xNearPos,uColour);
		
			}
	

		}
	}

}


void Tools_Object_Attractor_Node::OnSelection()
{
	// create overlay
	Tools_Attractor* const pxAttractor = m_pxNode->GetAttractor();
	InitialiseOverlay( pxAttractor );
}


void Tools_Object_Attractor_Node::OnDeselection()
{
	CloseOverlay();
}

bool Tools_Object_Attractor_Node::IsSelectable()
{
	//disallow selection of attractors that belong to templates
	if(m_pxNode->GetAttractor()->GetTemplateGuid() != ASURA_GUID_UNREGISTERED) return false;
	return Tools_Object::IsSelectable();

}

bool Tools_Object_Attractor_Node::IsHidden()
{
	return Tools_Object::IsHidden() || (m_pxNode->GetAttractor() && !Tools_Mode_Attractors::IsAttractorTypeVisible(m_pxNode->GetAttractor()->GetType()));
}



Asura_Guid Tools_Object_Attractor_Node::GetGuid() const
{
	return m_pxNode->GetAttractor()->GetControllerGuid();
}


bool Tools_Object_Attractor_Node::AcquireGuid()
{
	Tools_Attractor* pxAttractor = m_pxNode->GetAttractor();
	if(pxAttractor->GetControllerGuid() == ASURA_GUID_UNREGISTERED)
	{
		//Don't allow a guid to be acquired if the main entity file is readonly.
		//This is needed in case other files are wanting to store this guid
		//which won't remain fixed unless the entity file is saved
		
		//(Copied from phonon object.. TBD ,check if the logic applies here...)
		if(!Tools_SaveAsura::CanSaveEntity() && !Tools_SaveAsura::IsTemplateEditing())
		{
			ShowMessageOverlay("Can't assign guid to env hook because the main entity file isn't checked out.");
			return false;
		}
		
		
		//create a controller entity ,and associate it with this attractor
		Asura_Guid uGuid = Asura_Entity_System::GetCreatedByToolsGuid();

		Asura_ServerEntity_AttractorController* pxEntity = new Asura_ServerEntity_AttractorController(uGuid);
		pxEntity->SetToolsAttractorID(pxAttractor->GetID());

		pxEntity->ToolsData = new Tools_Entity_Data;
		pxEntity->ToolsData->Name = "EnvHook";

		pxAttractor->SetControllerGuid(uGuid);

		Tools_SaveAsura::SetEntityModified(pxEntity);
	}
	else
	{
		Asura_Assert(0,"Shouldn't be using AcquireGuid() on an object that already has one.");
	}
	return true;
}

bool Tools_Object_Attractor_Node::HandlesThisMessage(int iMessageID)
{
	return (iMessageID == ASURA_MSG_ACTIVATE) || (iMessageID == ASURA_MSG_DEACTIVATE);
}

bool Tools_Object_Attractor_Node::ObjectIsInLOS(const Asura_Vector_3& alpha, const Asura_Vector_3& beta, float& lowest)
{
	if (Tools_Object::ObjectIsInLOS(alpha, beta, lowest))
		return true;

	const Tools_Attractor* const pxAttractor = m_pxNode->GetAttractor();
	if (pxAttractor && m_pxNode && m_pxNode->GetNext())
	{
		Asura_Vector_3 xPos1 = m_pxNode->GetPosition();
		Asura_Vector_3 xPos2 = m_pxNode->GetNext()->GetPosition();
		Asura_Vector_3 xCentre = (xPos2 + xPos1) * 0.5f;

		float fIntersectionDistance = lowest;
		if(Asura_Maths::RayIntersectsSphere(alpha,beta,xCentre,0.5f, &fIntersectionDistance))
		{
			if(fIntersectionDistance < lowest)
			{
				lowest = fIntersectionDistance;
				return true;
			}
		}
	}
	return false;
}

void Tools_Object_Attractor_Node::Copy(const Tools_Object* pxObject)
{
	if(pxObject->Type != TOT_AttractorNode) return;

	Tools_Object_Attractor_Node *pxFromObject = (Tools_Object_Attractor_Node *)pxObject;

	if (GetNode() && pxFromObject->GetNode())
	{
		if (GetNode()->GetAttractor() && pxFromObject->GetNode()->GetAttractor())
		{
			GetNode()->GetAttractor()->SetAttractorType(pxFromObject->GetNode()->GetAttractor()->GetType());
			GetNode()->GetAttractor()->SetFlags(pxFromObject->GetNode()->GetAttractor()->GetFlags());
			GetNode()->GetAttractor()->SetRadius(pxFromObject->GetNode()->GetAttractor()->GetRadius());
			GetNode()->GetAttractor()->m_fAngleForNormal = pxFromObject->GetNode()->GetAttractor()->m_fAngleForNormal;
			GetNode()->GetAttractor()->SetParentGuid(pxFromObject->GetNode()->GetAttractor()->GetParentGuid());
		}
	}
 }

//Creates a new object of the same type as this.
Tools_Object* Tools_Object_Attractor_Node::CreateObjectOfThisType() const
{
	Tools_Attractor_Node *pxNode = Tools_Mode_Attractors::CreateAttractorNode(NULL);
	return pxNode ? pxNode->GetToolsNode() : NULL;
}

void Tools_Object_Attractor_Node::Destroy()
{
	SetModified();
	Tools_Attractor_System::GetInstance()->DeleteAttractorNode( m_pxNode );
	m_pxNode = 0;
	Tools_Object::Destroy();
}


void Tools_Object_Attractor_Node::AddContextMenuItems(class Tools_ContextMenu& xMenu, const class Tools_ContextMenu_Context& xContext)
{
	Tools_Object::AddContextMenuItems(xMenu, xContext);

	Tools_ContextMenu_Section* pxSection = xMenu.GetSection(TOOLS_CONTEXTMENU_SECTION_SELECTION);
	pxSection->AddSeparator();

	//Add option for selecting/deselecting this env hook
	u_int uMode;
	const char* szText;
	if(xContext.m_bShiftHeld)
	{
		szText = "Deselect Env Hook's Nodes";
		uMode = 0;
	}
	else if(xContext.m_bControlHeld)
	{
		szText = "Select Env Hook's Nodes";
		uMode = 1;
	}
	else
	{
		szText = "Select Env Hook's Nodes";
		uMode = 2;
	}

	Tools_ContextMenu_Item* pxItem = new Tools_ContextMenu_Item_Function(szText, ContextMenu_SelectAllInAttractor, this, uMode);
	pxItem->SetID("Select env hook");
	pxSection->AddItem(pxItem);
	
}


///////////////////////////////////////////////////////////////////////////////////////
// Overlay
///////////////////////////////////////////////////////////////////////////////////////
int Tools_Object_Attractor_Node::s_iRenderVolumeMode = TOOLS_ATTRACTOR_VOLUME_WIREFRAME;

void Tools_Object_Attractor_Node::InitialiseOverlay( Tools_Attractor* const pxAttractor )
{

	CloseOverlay();

	// create an overlay
	Asura_Overlay* pxOverlay = new Asura_Overlay( "Env Hook", false, &s_xOverlayPosition );
	pxOverlay->m_uID = TOOLS_OVERLAY_SELECTEDOBJECT;
	Asura_Assert( pxOverlay != NULL, "Cant create env hook edit overlay" );
	
	Asura_Vector_2 xOffset( 1.0f, 3.0f );
	Asura_Dialog* pxDialog;

	m_bTypeChanged = false;
	pxDialog = new Tools_Attractor_Type_Dialog(xOffset,12,&g_xParam_EnvHook_Type,"Type");
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog( "Radius", xOffset, &g_xParam_EnvHook_Radius, 0.0f, 100.0f, 0.5f, "%0.2f");
	pxDialog->SetIsEnabledFunction(CanEditRadius,(int)this);
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_SharedTickbox_Dialog( "Active", xOffset, &g_xParam_EnvHook_Active);
	pxDialog->SetHelpText("An env hook must be active to be useable.");
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_SharedTickbox_Dialog( "Closed", xOffset, &g_xParam_EnvHook_Closed);
	pxDialog->SetHelpText("Causes a connection to be added from the last node in the env hook to the first.");
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::LargeSpacing();

	pxDialog = new Asura_Function_Dialog( "Reverse Env Hook", xOffset, OnReverseAttractor, NULL, IsReverseAttractorEnabled );
	pxDialog->SetHelpText("Reverses the node order , which will reverse the normals.");
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog( "Add Link", xOffset, AddLink, (int) this);
	pxDialog->SetHelpText("Adds a link to another env hook. The exact effect of linking a hook is dependent on the project");
	pxDialog->SetIsEnabledFunction(CanAddLink,(int) this);
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog( "Remove Link", xOffset, RemoveLink, (int) this);
	pxDialog->SetHelpText("Removes a link to another env hook.");
	pxDialog->SetIsEnabledFunction(CanRemoveLink,(int) this);
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Parent : ", &g_xParam_EnvHook_ParentEntity,xOffset, SetParent, (int) this);
	pxDialog->SetHelpText("Set the parent entity that this env hook will be moved relative to.");
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog( "Angle for normal", xOffset, &g_xParam_EnvHook_Angle, -360.0f, 360.0f, 0.1f, "%0.2f");
	pxDialog->SetHelpText("Set the angle around the env hook that the normal points out at.");
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog( "Set normal from poly", xOffset, ChooseNormalFromPolygon, reinterpret_cast<int>(this), &m_bChanged );
	pxDialog->SetHelpText("Set up the normal for this env hook using the normal of a polygon.");
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog( "Set normal from edge", xOffset, ChooseNormalFromEdge, reinterpret_cast<int>(this), &m_bChanged );
	pxDialog->SetHelpText("Set up the normal for this env hook using the normal of an edge.");
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog( "Align to normal", xOffset, AlignToNormal, reinterpret_cast<int>(this));
	pxDialog->SetHelpText("Aligns the tools object to the normal.\nThe z direction will point along the normal , and the x direction along the env hook.");
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::LargeSpacing();


	static const char* s_aszRenderVolumeStrings[] =
	{
		"Disabled",
		"Wireframe",
		"Solid",
	};

	pxDialog = new Asura_DropDown_List_Dialog(xOffset, s_aszRenderVolumeStrings, &s_iRenderVolumeMode, 3, 3, NULL,	0, "Render Volume"); 
	pxDialog->SetHelpText("Determines how the volume around the env hooks should be rendered");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();
	

	AddOverlayControls( pxOverlay, xOffset );

	//set things up so that right clicking on the overlay will be equivalent to right clicking on the object
	//RAR 06/11/2007
	pxOverlay->SetContextMenuCallback(Tools_Object_System::AddContextMenuItemsFromOverlay, this);

	// submit the overlay
	Asura_Overlay_System::SubmitNewOverlay( pxOverlay, false );

}


void Tools_Object_Attractor_Node::CloseOverlay()
{
	// if it exists already, shut it down
	if ( Asura_Overlay_System::DoesOverlayExist( TOOLS_OVERLAY_SELECTEDOBJECT ) )
	{
		Asura_Overlay_System::CloseOverlays( TOOLS_OVERLAY_SELECTEDOBJECT );
	}
}



///////////////////////////////////////////////////////////////////////////////////////
// Callbacks
///////////////////////////////////////////////////////////////////////////////////////


void Tools_Object_Attractor_Node::ContextMenu_SelectAllInAttractor(void* pObject, u_int uMode)
{
	Tools_Object* pxObject = (Tools_Object*) pObject;

	Asura_Assert( pxObject && pxObject->Type == TOT_AttractorNode, "Tools_Object_Attractor_Node::SelectAllInAttractor called with non attractor node" );
	Tools_Object_Attractor_Node* pxAttractorNode = static_cast< Tools_Object_Attractor_Node* >( pxObject );

	List<Tools_Object*> xSelectionList;
	// get first node of our attractor

	Tools_Attractor_Node* pxNode = pxAttractorNode->GetNode()->GetAttractor()->GetFirstNode();

	while ( pxNode )
	{
		xSelectionList.add_entry( pxNode->GetToolsNode() );
		pxNode = pxNode->GetNext();
	}

	//Move the object clicked on to the end of the list , so it becomes the primary selection. RAR 14/08/2006
	if(xSelectionList.contains(pxAttractorNode))
	{
		xSelectionList.delete_entry(pxAttractorNode);
		xSelectionList.add_entry(pxAttractorNode);
	}
	//(de)select these objects
	if(uMode == 0)
	{
		Tools_Object_System::DeselectObjects(xSelectionList);
	}
	else
	{
		Tools_Object_System::SelectObjects(xSelectionList, uMode == 1);
	}
	
}


void Tools_Object_Attractor_Node::OnReverseAttractor()
{
	Tools_Object* pxSelected = Tools_Object_System::SelectedObject;
	if(!pxSelected || pxSelected->Type != TOT_AttractorNode) return;


	Tools_Attractor* const pxAttractor = ( static_cast<Tools_Object_Attractor_Node*>( pxSelected ) )->GetNode()->GetAttractor();

	Tools_Attractor_Node* pxNode = pxAttractor->GetFirstNode();

	while ( pxNode )
	{

		// set this as start point (means we'll have the last one selected)
		pxAttractor->SetFirstNode( pxNode );

		// swap previous and next

		pxNode->SetPrevAndNext( pxNode->GetNext(), pxNode->GetPrev() );

		// move to next (which is now previous)

		pxNode = pxNode->GetPrev();
	}

	pxAttractor->SetModified();
}


bool Tools_Object_Attractor_Node::IsReverseAttractorEnabled()
{
	Tools_Object* pxSelected = Tools_Object_System::SelectedObject;

	return pxSelected && pxSelected->Type == TOT_AttractorNode;

}

void Tools_Object_Attractor_Node::PrintInfo(float& x,float& y)
{
	Tools_Object::PrintInfo(x,y);
	
	Tools_Attractor* pxAttractor = m_pxNode->GetAttractor();

	if(Tools_SaveAsura::IsEditingMission())
	{
		Asura_Text_Output::Print(x,y++,0xffffffff,0,"Mission : %s",pxAttractor->BelongsToMissionFile() ? "Yes" : "No");
	}

	Asura_Text_Output::Print(x,y++,0xffffffff,0,"EnHk ID : %d",pxAttractor->GetID()&~TOOLS_ATTRACTORID_MISSIONFILE);	
	Asura_Text_Output::Print(x,y++,0xffffffff,0,"Node ID : %d",m_pxNode->GetID()&~TOOLS_ATTRACTORID_MISSIONFILE);	

	if(pxAttractor->m_uNumLinks)
	{
		Asura_Text_Output::Print(x,y++, 0xffffffff,0,"Link IDs : ");
		for(u_int i=0;i<pxAttractor->m_uNumLinks;i++)
		{
			Asura_Text_Output::Print(x,y++,0xffffffff,0,"  %s%d",pxAttractor->m_puLinks[i]&TOOLS_ATTRACTORID_MISSIONFILE ? "(M) " : "", pxAttractor->m_puLinks[i]&~TOOLS_ATTRACTORID_MISSIONFILE);	
		}
	}
}

bool Tools_Object_Attractor_Node::CanAddLink()
{
	//attractor must have at least one other node
	return m_pxNode->GetNext() || m_pxNode->GetPrev();
}


void Tools_Object_Attractor_Node::AddLink()
{
	Tools_ChooseObject* pxChooseObject = new Tools_ChooseObject_Attractor_AddLink(this);
	Tools_Object_System::ChooseObject("Select a node in the env hook to link to",pxChooseObject);
}


bool Tools_Object_Attractor_Node::CanRemoveLink()
{
	//are there any to remove?
	Tools_Attractor* pxAttractor = m_pxNode->GetAttractor();

	return pxAttractor->m_uNumLinks != 0;
}

void Tools_Object_Attractor_Node::RemoveLink()
{
	Tools_ChooseObject* pxChooseObject = new Tools_ChooseObject_Attractor_RemoveLink(this);
	Tools_Object_System::ChooseObject("Select a node from the env hook to unlink",pxChooseObject);
}

bool Tools_Object_Attractor_Node::CanEditRadius()
{
	const Tools_Attractor_Type* pxType = GetAttractorType();
	return pxType ? !pxType->m_bMustBeDefaultRadius : true;
}


void Tools_Object_Attractor_Node::UpdateColour()
{
	Tools_Attractor* const pxAttractor = m_pxNode->GetAttractor();

	const Tools_Attractor_Type* pxType = Tools_Attractor_System::GetInstance()->GetAttractorTypeByHash(pxAttractor->GetType());
	SetColour(pxType ? pxType->m_uColour : 0xffffffff);
}

void Tools_Object_Attractor_Node::SetParent()
{
	Tools_Object* pxInitialSelection = Tools_Object_System::GetObjectFromGuid(m_pxNode->GetAttractor()->GetParentGuid());

	Tools_ChooseObject* pxChooseObject = new Tools_ChooseObject_Attractor_SetParent();
	Tools_Object_System::ChooseObject("Env hook should be attached to which entity?",pxChooseObject,pxInitialSelection);
}

void Tools_Object_Attractor_Node::AlignToNormal()
{
	const Tools_Attractor* pxAttractor = m_pxNode->GetAttractor();
	//Determine the two nodes to use for the direction
	const Tools_Attractor_Node* pxNode1 = m_pxNode;
	const Tools_Attractor_Node* pxNode2 = pxNode1->GetNext();

	if(!pxNode2)
	{
		//The last node was selected ,so use the direction back to the first node if closed , or
		//use the last two nodes otherwise.
		if(pxAttractor->IsClosed())
		{
			pxNode2 = pxAttractor->m_pxFirstNode;
		}
		else
		{
			pxNode2 = pxNode1;
			pxNode1 = pxNode2->GetPrev();
		}
	}

	Asura_Vector_3 xDir = pxNode2->GetPosition() - pxNode1->GetPosition();
	if(!xDir.Magnitude()) return;
	xDir.Normalise();

	//get the normal to use
	Asura_Vector_3 xNormal;
	pxAttractor->GetNormalFromDirection(xNormal,xDir);
	
	//Use the normal as the z direction ,and xDir as the x direction for the orientation
	Asura_Vector_3 xYDir = xNormal ^ xDir;

	Orientation.e11 = xDir.x;
	Orientation.e21 = xDir.y;
	Orientation.e31 = xDir.z;
	
	Orientation.e12 = xYDir.x;
	Orientation.e22 = xYDir.y;
	Orientation.e32 = xYDir.z;
	
	Orientation.e13 = xNormal.x;
	Orientation.e23 = xNormal.y;
	Orientation.e33 = xNormal.z;


}

void Tools_Object_Attractor_Node::UpdateRelativePositionFromPosition()
{
	Asura_Vector_3 xRelative;
	xRelative.Zero();

	Tools_Object* pxObject = Tools_Object_System::GetObjectFromGuid(m_pxNode->GetAttractor()->GetParentGuid());
	if(pxObject)
	{
		xRelative = Position - pxObject->Position;

		Asura_Matrix_3x3 xInverseOrient = pxObject->Orientation;
		xInverseOrient.Transpose();

		xRelative = xRelative * xInverseOrient;
	}

	m_pxNode->SetRelativePosition(xRelative);
}

void Tools_Object_Attractor_Node::UpdatePositionFromRelativePosition()
{
	Asura_Guid uGuid = m_pxNode->GetAttractor()->GetParentGuid();
	if(uGuid == ASURA_GUID_UNREGISTERED) return;//nothing to do if there isn't a parent

	Tools_Object* pxObject = Tools_Object_System::GetObjectFromGuid(uGuid);
	if(!pxObject) return;

	Position = (m_pxNode->GetRelativePosition() * pxObject->Orientation) + pxObject->Position;
	m_pxNode->SetPosition(Position);
}

//static Tools_Object_Attractor_Node* g_pxToolsAttractorObjectForNormalEditing = NULL;

void Tools_Object_Attractor_Node::ChooseNormalFromPolygon(int iCurrentNodePointer)
{
	const Tools_Object* pxToolsObject  = reinterpret_cast<const Tools_Object*>(iCurrentNodePointer);
	if (!pxToolsObject || pxToolsObject->Type != TOT_AttractorNode)  { return; }

	//Tools_Object_Attractor_Node* pxCurrentNode = reinterpret_cast<Tools_Object_Attractor_Node*>(iCurrentNodePointer);
	//g_pxToolsAttractorObjectForNormalEditing = pxCurrentNode;

	Tools_Prompted_Selection::ChoosePolygon("Choose poly for normal", PolyForNormal_Callback);
}

void Tools_Object_Attractor_Node::ChooseNormalFromEdge(int iCurrentNodePointer)
{
	const Tools_Object* pxToolsObject  = reinterpret_cast<const Tools_Object*>(iCurrentNodePointer);
	if (!pxToolsObject || pxToolsObject->Type != TOT_AttractorNode)  { return; }

	//Tools_Object_Attractor_Node* pxCurrentNode = reinterpret_cast<Tools_Object_Attractor_Node*>(iCurrentNodePointer);
	//g_pxToolsAttractorObjectForNormalEditing = pxCurrentNode;

	Tools_Prompted_Selection::ChooseEdge("Choose edge for normal", EdgeForNormal_Callback);
}

void Tools_Object_Attractor_Node::PolyForNormal_Callback(int iModuleIndex, int iPolygonIndex)
{
	//if (!g_pxToolsAttractorObjectForNormalEditing) { return; }

	const Asura_Mesh* const pxMesh = Asura_Module_System::GetMeshForModule( iModuleIndex );
	if (pxMesh)
	{
		Asura_Vector_3 xPolyNormal;
		Asura_Vector_3 xDir1(pxMesh->VertexDataPtr[ pxMesh->PolygonDataPtr[iPolygonIndex].VertexID[1] ].Position - pxMesh->VertexDataPtr[ pxMesh->PolygonDataPtr[iPolygonIndex].VertexID[0] ].Position);
		Asura_Vector_3 xDir2(pxMesh->VertexDataPtr[ pxMesh->PolygonDataPtr[iPolygonIndex].VertexID[2] ].Position - pxMesh->VertexDataPtr[ pxMesh->PolygonDataPtr[iPolygonIndex].VertexID[1] ].Position);
		xPolyNormal = xDir1 ^ xDir2;
		xPolyNormal.Normalise();

		//g_pxToolsAttractorObjectForNormalEditing->m_pxNode->GetAttractor()->SetNormal(xPolyNormal,g_pxToolsAttractorObjectForNormalEditing->m_pxNode);

		int iNumSelected = Tools_Object_System::SelectedObjectList.size();
		for ( int i=0; i< iNumSelected; i++ )
		{
			Tools_Object* pxObject = Tools_Object_System::SelectedObjectList.get_entry(i);
			if ( pxObject && pxObject->Type == TOT_AttractorNode)
			{
				Tools_Object_Attractor_Node* pxCurrentNode = reinterpret_cast<Tools_Object_Attractor_Node*>(pxObject);
				if ( pxCurrentNode )
				{
					pxCurrentNode->m_pxNode->GetAttractor()->SetNormal(xPolyNormal,pxCurrentNode->m_pxNode);
				}
			}
		}
	}
}

void Tools_Object_Attractor_Node::EdgeForNormal_Callback(Tools_LOS_EdgeOutput xEdge)
{
	//if (!g_pxToolsAttractorObjectForNormalEditing) { return; }

	Asura_Vector_3 xNormal(0.0f,0.0f,0.0f);

	//add the normals from the polygons on either side of the edge
	for(int i=0;i<2;i++)
	{
		if(xEdge.m_axEdge[i].m_iModuleIndex < 0) continue;

		Asura_Mesh* pxMesh = Asura_Module_System::GetMeshForModule( xEdge.m_axEdge[i].m_iModuleIndex );

		if(!pxMesh->AreFaceNormalsValid()) pxMesh->CreateFaceNormals();
		int iPolyIndex = xEdge.m_axEdge[i].m_iPolygonIndex;
		xNormal += pxMesh->NormalsDataPtr[iPolyIndex].FaceNormal;

	}
		
	if(xNormal == Asura_Maths::ZeroVector3) return;
	xNormal.Normalise();

	// Finally, convert this to our rotation.
	//g_pxToolsAttractorObjectForNormalEditing->m_pxNode->GetAttractor()->SetNormal(xNormal,g_pxToolsAttractorObjectForNormalEditing->m_pxNode);
	int iNumSelected = Tools_Object_System::SelectedObjectList.size();
	for ( int i=0; i< iNumSelected; i++ )
	{
		Tools_Object* pxObject = Tools_Object_System::SelectedObjectList.get_entry(i);
		if ( pxObject && pxObject->Type == TOT_AttractorNode)
		{
			Tools_Object_Attractor_Node* pxCurrentNode = reinterpret_cast<Tools_Object_Attractor_Node*>(pxObject);
			if ( pxCurrentNode )
			{
				pxCurrentNode->m_pxNode->GetAttractor()->SetNormal(xNormal,pxCurrentNode->m_pxNode);
			}
		}
	}
}

const Tools_Attractor_Type* Tools_Object_Attractor_Node::GetAttractorType() const
{
	Tools_Attractor* pxAttractor = m_pxNode->GetAttractor();
	return Tools_Attractor_System::GetInstance()->GetAttractorTypeByHash(pxAttractor->m_xType);
}




bool Tools_Object_Attractor_Node::GetParameter(Asura_Hash_ID uParamID,class Tools_Parameter* pxParam) const
{
	Tools_Attractor* pxAttractor = m_pxNode->GetAttractor();
	
	switch(uParamID)
	{
		case g_uHash_EnvHook_Type :
			pxParam->SetHash(pxAttractor->m_xType);
			break;
		
		case g_uHash_EnvHook_Radius :
			pxParam->SetFloat(pxAttractor->m_fRadius);
			break;

		case g_uHash_EnvHook_Angle :
			pxParam->SetFloat(Asura_Maths::RadToDeg(pxAttractor->m_fAngleForNormal));
			break;

		case g_uHash_EnvHook_Closed :
			pxParam->SetBool(pxAttractor->m_bClosed);
			break;

		case g_uHash_EnvHook_Active :
			pxParam->SetBool(pxAttractor->IsActive());
			break;

		case g_uHash_EnvHook_ParentEntity :
		{
			Asura_Guid uGuid = pxAttractor->GetParentGuid();

			//Get the text for the parent selection control
			if(uGuid == ASURA_GUID_UNREGISTERED)
			{
				pxParam->SetString("*None*");
			}
			else
			{
				Tools_Object* pxObject = Tools_Object_System::GetObjectFromGuid(uGuid);
				if(pxObject)
				{
					pxParam->SetString(pxObject->GetName());
				}
				else
				{
					pxParam->SetString("*Not Found*");
				}
			}

			break;
		}

		default :
			return false;

	}

	return true;
}

bool Tools_Object_Attractor_Node::SetParameter(Asura_Hash_ID uParamID,const class Tools_Parameter* pxParam)
{
	Tools_Attractor* pxAttractor = m_pxNode->GetAttractor();

	switch(uParamID)
	{
		case g_uHash_EnvHook_Type :
			pxAttractor->SetAttractorType(pxParam->GetHash(), m_pxNode);
			m_bTypeChanged = true;
			break;
		
		case g_uHash_EnvHook_Radius :
			if(CanEditRadius())
			{
				pxAttractor->m_fRadius = pxParam->GetFloat();
			}
			break;

		case g_uHash_EnvHook_Angle :
			pxAttractor->m_fAngleForNormal = Asura_Maths::DegToRad(pxParam->GetFloat());
			break;

		case g_uHash_EnvHook_Closed :
			pxAttractor->m_bClosed = pxParam->GetBool();
			break;

		case g_uHash_EnvHook_Active :
		{
			pxAttractor->SetActive(pxParam->GetBool());
			break;
		}

		default :
			return false;

	}

	m_bChanged = true;

	return true;

}

void Tools_Object_Attractor_Node::SetModified() const
{
	m_pxNode->GetAttractor()->SetModified();
}

bool Tools_Object_Attractor_Node::BelongsToMissionFile() const
{
	return m_pxNode->GetAttractor()->BelongsToMissionFile();
}


#endif // ASURA_USE_ATTRACTORS
