//
// Tools_Object_Attractor_Node.h
//
// Created by JGP 03/05/06
//
// Copyright (C) 2006 Rebellion (Derby), All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __TOOLS_OBJECT_ATTRACTOR_NODE_H__
#define __TOOLS_OBJECT_ATTRACTOR_NODE_H__


// base class
#include "Objects/Tools_Object.h"

struct Tools_LOS_EdgeOutput;
class Tools_Attractor_Type;

/*
	Tools_Object_Attractor_Node

	Tools version of Asura_Attractor_Node.
	Defines editing for the node. Which
	is mostly actually editing for the 
	attractor the node belongs to.
*/

enum TOOLS_ATTRACTOR_RENDERVOLUMEMODE
{
	TOOLS_ATTRACTOR_VOLUME_DISABLED,
	TOOLS_ATTRACTOR_VOLUME_WIREFRAME,
	TOOLS_ATTRACTOR_VOLUME_SOLID,
};

class Tools_Object_Attractor_Node : public Tools_Object
{

public:
	typedef List<Tools_Object_Attractor_Node*> NODE_LIST;
	typedef LIF<Tools_Object_Attractor_Node*> NODE_LIST_IT;

	Tools_Object_Attractor_Node( class Tools_Attractor_Node* const pxNode );
	~Tools_Object_Attractor_Node();
	
	virtual const char* GetName();
	virtual const char* GetTypeDescription() { return "Env Hook Node"; }

	virtual void AddOverlayControls( class Asura_Overlay* pxOverlay,Asura_Vector_2& xOffset ) {}
	virtual void UpdateWhileSelected();
	virtual void Update();
	virtual void Render();
	virtual void OnSelection();
	virtual void OnDeselection();
	virtual void UpdatePositionInAsuraData( bool bUpdateLinkedObjects );
	virtual void Destroy();
	virtual void AddContextMenuItems(class Tools_ContextMenu& xMenu, const class Tools_ContextMenu_Context& xContext);
 	virtual void PrintInfo(float& x,float& y);
	virtual bool IsSelectable();
	virtual bool IsHidden();
	virtual bool UsePivotForSnapToVertex() const {return true;}

	virtual Asura_Guid GetGuid() const;
    virtual bool AcquireGuid();
	virtual bool HandlesThisMessage(int iMessageID);

	virtual bool ObjectIsInLOS(const Asura_Vector_3& alpha, const Asura_Vector_3& beta, float& lowest);

	virtual void Copy(const Tools_Object* pxObject);
	//Does this object support copying?
	virtual bool CanBeCopied() const {return true;}
	//Creates a new object of the same type as this.
	virtual Tools_Object* CreateObjectOfThisType() const;


	class Tools_Attractor_Node* const GetNode() { return m_pxNode; }
	const class Tools_Attractor_Node* const GetNode() const { return m_pxNode; }

	static NODE_LIST* GetList() { return &s_xNodeList; }

	static void SetToolsObjectsShouldExist(bool bSet) {s_bToolsObjectsShouldExist = bSet;}
	static bool GetToolsObjectsShouldExist() {return s_bToolsObjectsShouldExist;}

	static void CreateAttractorObjects();
	static void DeleteAttractorObjects();

	virtual bool GetParameter(Asura_Hash_ID uParamID,class Tools_Parameter* pxParam) const;
	virtual bool SetParameter(Asura_Hash_ID uParamID,const class Tools_Parameter* pxParam);

	bool IsDestroyed() const {return false;}

	//flags the file containing this attractor as modified
	void SetModified() const;

	bool BelongsToMissionFile() const;

	static int GetRenderVolumeMode(){return (TOOLS_ATTRACTOR_RENDERVOLUMEMODE) s_iRenderVolumeMode;}

protected:
	bool HasTypeChanged() const { return m_bTypeChanged; }

	// Initialise / Close Overlay
	void InitialiseOverlay( class Tools_Attractor* const pxAttractor );
	void CloseOverlay();

	// vars
	Tools_Attractor_Node* m_pxNode;

private:
	void UpdateColour();

	void UpdatePositionFromRelativePosition();
	void UpdateRelativePositionFromPosition();

	// callbacks

	static void OnReverseAttractor();
	static bool IsReverseAttractorEnabled();
	static void ContextMenu_SelectAllInAttractor(void* pObject, u_int uMode);

	#define OVERLAY_VOID_CALLBACK(__FnName)\
	static void __FnName(int iThis){((Tools_Object_Attractor_Node*)iThis)->__FnName();}\
	void __FnName();

	#define OVERLAY_BOOL_CALLBACK(__FnName)\
	static bool __FnName(int iThis){return ((Tools_Object_Attractor_Node*)iThis)->__FnName();}\
	bool __FnName();

	OVERLAY_VOID_CALLBACK(AddLink);
	OVERLAY_VOID_CALLBACK(RemoveLink);
	OVERLAY_VOID_CALLBACK(SetParent);
	OVERLAY_VOID_CALLBACK(AlignToNormal);

	OVERLAY_BOOL_CALLBACK(CanAddLink);
	OVERLAY_BOOL_CALLBACK(CanRemoveLink);
	OVERLAY_BOOL_CALLBACK(CanEditRadius);

	
	#undef OVERLAY_VOID_CALLBACK
	#undef OVERLAY_BOOL_CALLBACK

	static void ChooseNormalFromPolygon(int iCurrentNodePointer);
	static void ChooseNormalFromEdge(int iCurrentNodePointer);

	static void PolyForNormal_Callback(int iModuleIndex, int iPolygonIndex);
	static void EdgeForNormal_Callback(Tools_LOS_EdgeOutput xEdge);

	const Tools_Attractor_Type* GetAttractorType() const;

	// static data

	static Asura_Vector_2 s_xOverlayPosition;

	static NODE_LIST s_xNodeList;

	static bool s_bToolsObjectsShouldExist;

	// vars

	Asura_String m_xName;
	bool m_bTypeChanged;
	bool m_bChanged;
	static int s_iRenderVolumeMode;

	friend class Tools_ChooseObject_Attractor_AddLink;
	friend class Tools_ChooseObject_Attractor_RemoveLink;
	friend class Tools_ChooseObject_Attractor_SetParent;

public :
	static float s_fScaleWhenCloseDist;//distance at which the tools objects should start being scaled down


};


#endif // __TOOLS_OBJECT_ATTRACTOR_NODE_H__
