//
// Tools_BillboardPlant_Object.cpp
//
// Created by Andrew Weinkove 06/12/02
// 
// Copyright (C) 2002 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"
#include "BillboardPlants/Tools_BillboardPlant_Object.h"
#include "Debugging/Asura_Assert.h"
#include "SpecialFX/Asura_BillboardPlant_TemplateStore.h"
#include "Camera/Asura_Camera.h"
#include "Interface/Asura_Dialog.h"
#include "Rendering/Asura_Direct3d.h"
#include "Rendering/Asura_Direct3D_Shaders.h"
#include "Mesh/Asura_Mesh.h"
#include "Mesh/Asura_Mesh_System.h"
#include "Rendering/Asura_Module.h"
#include "Interface/Asura_Overlay.h"
#include "Interface/Asura_Overlay_System.h"
#include "Rendering/Asura_Render.h"
#include "Tools_Overlays.h"
#include "Misc/Tools_Modal_Overlay.h"

#ifdef ASURA_USE_BILLBOARD_PLANTS

Asura_Vector_2					Tools_BillboardPlant_Object::s_xOverlayPosition = Asura_Vector_2(10.0f, 10.0f);
Tools_BillboardPlant_Object*		Tools_BillboardPlant_Object::s_pxThisPlant = NULL;

Tools_BillboardPlant_Object::Tools_BillboardPlant_Object(const Asura_Vector_3 &xPosition, const unsigned uPlantID)
: Tools_Object(TOT_Billboard_Plant)
{
	Position = xPosition;
	Orientation.Identity();
	m_uPlantType = uPlantID;
	Asura_String xMeshName = "ToolsShape_CutsceneEvent"; // Use this mesh for the time being.
	Mesh = Asura_Mesh_System::GetMesh(xMeshName);
}

Tools_BillboardPlant_Object::~Tools_BillboardPlant_Object()
{
}

void Tools_BillboardPlant_Object::OnDeselection()
{
	s_pxThisPlant = NULL;
	Tools_Object::OnDeselection();
}

void Tools_BillboardPlant_Object::OnSelection()
{
	s_pxThisPlant = this;
	Tools_Object::OnSelection();
}

void Tools_BillboardPlant_Object::UpdateWhileSelected()
{
	Orientation.Identity();
	Tools_Object::UpdateWhileSelected();
}

void Tools_BillboardPlant_Object::Render()
{
	Asura_Bounding_Box xBB;
	xBB.Reset();
	xBB.Set(Position);
	if (!Asura_Module_System::DoesBBIntersectVisibleModule(xBB))
		return;
	if (!Asura_Camera::IsSphereInFrustum(Position, 1.0f))
		return;
	int iPlantIndex = Asura_BillboardPlant_TemplateStore::GetIndexFromUniqueID(m_uPlantType);
	if (iPlantIndex == -1)
	{
		Tools_Object::Render();
		return;
	}
	if (IsGroupSelected())
		Tools_Object::Render();
	Asura_BillboardPlant_Template *pxTemplate = Asura_BillboardPlant_TemplateStore::GetTemplate(iPlantIndex);
	Asura_Vector_3 xPositions[4];
	Asura_Matrix_3x3 xMat = *Asura_Camera::GetOrientation();
	Asura_Euler_3 xOri;
	xOri.FromMatrix(xMat);
	xOri.z = xOri.x = 0.0f;
	xOri.ToMatrix(xMat);
	for (int i=0;i<4;i++)
	{
		xPositions[i] = Asura_Vector_3(0.0f, 0.0f, 0.0f);
		if (i < 2)
			xPositions[i].y -= pxTemplate->m_fHeight; 
		if (!(i % 2))
			xPositions[i].x -= pxTemplate->m_fWidth * 0.5f;
		else
			xPositions[i].x += pxTemplate->m_fWidth * 0.5f;
		xPositions[i] = xPositions[i] * xMat;
		xPositions[i] += Position;
	}
	{
		D3DGenericVertex v[4];
		for (int i=0;i<4;i++)
		{
			v[i].U = (i % 2) ? pxTemplate->m_fBottomRightU : pxTemplate->m_fTopLeftU;
			v[i].V = (i < 2) ? pxTemplate->m_fTopLeftV : pxTemplate->m_fBottomRightV;
			v[i].X = xPositions[i].x;
			v[i].Y = xPositions[i].y;
			v[i].Z = xPositions[i].z;
			v[i].Diffuse = 0xffffffff;
		}

		Asura_Direct3D_Shaders::RequestFixedFunctionVertexShader(D3DFVF_GENERICVERTEX);
		Asura_Direct3D::D3DDevice->SetRenderState(D3DRS_CULLMODE, D3DCULL_NONE);
		Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_NORMAL);
		Asura_Render::CurrentStates.RequestTextureOpMode(ASURA_TEXTUREOP_SINGLE_PASS);
		Asura_Render::CurrentStates.RequestTextureIndex(0,pxTemplate->m_iTextureIndex);
		// Request an alpha test - this prevents alpha pixels from obscuring non-alpha in
		// the z-test.
		Asura_Render::CurrentStates.RequestAlphaTest( true );
		Asura_Render::CurrentStates.RequestZBufferWriteEnabled(true);
		
		D3DXMATRIX matWorld;

		matWorld._12 = matWorld._13 = matWorld._14 = matWorld._21 = matWorld._23 = matWorld._24 = 0.0f;
		matWorld._31 = matWorld._32 = matWorld._34 = matWorld._41 = matWorld._42 = matWorld._43 = 0.0f;
		matWorld._11 = matWorld._22 = matWorld._33 = 1.0f;
		matWorld._44 = 1.0f;
		matWorld._41 = 0.0f;
		matWorld._42 = 0.0f;
		matWorld._43 = 0.0f;

		Asura_Direct3D::D3DDevice->SetTransform( D3DTS_WORLD,      &matWorld );


		// Only render the front facing triangles because we can guarantee that the
		// backs will never be visible (the billboards turn to face the camera).
		short indices[6] = 
		{
			0,1,2,
			1,2,3,
		};
		Asura_Direct3D::D3DDevice->DrawIndexedPrimitiveUP
		(
		D3DPT_TRIANGLELIST,
			0,
			4,
			2,
			(void*)indices,
			D3DFMT_INDEX16,
			(void*)v,
			sizeof(D3DGenericVertex)
		);
		if (IsSelected())
		{
			Asura_Render::CurrentStates.RequestZBufferCompareMode(ASURA_ZBUFFERCMP_ALWAYS);
			// Render the wireframe.
			for (int i=0;i<4;i++)
			{
				v[i].Diffuse = 0xffff7070;
			}
			short indices[12] = 
			{
				0,2,2,3,3,1,1,0,0,3,2,1
			};
			Asura_Direct3D::D3DDevice->DrawIndexedPrimitiveUP
			(
				D3DPT_LINELIST,
				0,
				4,
				6,
				(void*)indices,
				D3DFMT_INDEX16,
				(void*)v,
				sizeof(D3DGenericVertex)
			);
			Asura_Render::CurrentStates.RequestZBufferCompareMode(ASURA_ZBUFFERCMP_LESSEQUAL);
		}
	}
}


void Tools_BillboardPlant_Object::CreateOverlay()
{
	Asura_Overlay* newOverlay;
	newOverlay = new Asura_Overlay(GetTypeDescription(),false,&s_xOverlayPosition);
	
	Asura_Dialog* newDialog;
	Asura_Vector_2 offset = Asura_Vector_2(1.0f,3.0f);

	newDialog = new Asura_Function_Dialog("Change type",offset,ChangePlantType);
	newOverlay->AddDialog(newDialog);
	offset.y+=Asura_Dialog::DefaultSpacing();

	newOverlay->ID = TOOLS_OVERLAY_MODE;

	newOverlay->Origin = s_xOverlayPosition;
 	Asura_Overlay_System::SubmitNewOverlay(newOverlay,false);
}

void Tools_BillboardPlant_Object::ChangePlantType()
{
	ListSelection_Overlay xOverlay("Select plant : ");
	bool bAddedOne = false;
	for (int i=0;i<Asura_BillboardPlant_TemplateStore::GetNumberOfTemplates();i++)
	{
		xOverlay.AddItem(Asura_BillboardPlant_TemplateStore::GetTemplate(i)->m_szTemplateName, i);
		bAddedOne = true;
	}
	if (!bAddedOne)
		return;
	if (!xOverlay.DoModal())
		return;
	s_pxThisPlant->m_uPlantType = Asura_BillboardPlant_TemplateStore::GetTemplate(xOverlay.GetSelectionData())->m_uUniqueID;
}

#endif //ASURA_USE_BILLBOARD_PLANTS
