//////////////////////////////////////////////////////////////////////////////
//
// Tools_Blueprint_Edit_Object.cpp
//
// Created by NJD - 05 October 2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"

// header
#include "Blueprint/Tools_Blueprint_Edit_Object.h"

// asura
#include "Animation/Asura_AnimTable_System.h"
#include "Animation/Asura_Hierarchy.h"
#include "Blueprint/Asura_Blueprint.h"
#include "Blueprint/Asura_Blueprint_System.h"
#include "ChunkSystem/Asura_Chunk_FileInfo.h"
#include "Core/Asura_HashedLocalisedText.h"
#include "Core/Asura_LinkList.h"
#include "Core/Asura_LocalisedText.h"
#include "Core/Asura_NameList.h"
#include "Core/Asura_Maths.h"
#include "Core/Asura_SortAndSearch.h"
#include "Debugging/Asura_Assert.h"
#include "Debugging/Asura_Logfile.h"
#if defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)
#include "FacialAnimation/Tools_FacialAnimation_System.h"
#endif	//defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)
#include "FSFX/Asura_FSFX_Template.h"
#ifdef ASURA_USE_GUI_MENU_SYSTEM
	#include "GUIMenu/Asura_GUIMenu_System.h"
	#include "GUIMenu/Asura_GUIMenu_Menu_Page.h"
#endif 
#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Overlay_System.h"
#include "Interface/Asura_Overlay.h"
#include "Interface/Asura_Table_Dialog.h"
#include "Physics/Asura_Physics_Ragdoll.h"
#include "Physics/Asura_Physics_Ragdoll_Description.h"
#ifdef ASURA_SUPPORT_SKIN_ELEMENTS
#include "Rendering/Asura_Project_Skin_Elements.h"
#endif
#include "Rendering/Asura_Text_Output.h"
#include "Sound/Asura_Dialogue_System.h"
#include "Sound/Asura_Dialogue.h"
#include "Sound/Asura_Sound_Event.h"
#include "Sound/Asura_Sound_Event_Storage.h"
#include "Sound/Asura_Sound_Event_System.h"
#ifdef ASURA_USE_DECAL_MANAGER
	#include "SpecialFX/Asura_Decal_Manager.h"
#endif
#include "Rendering/Asura_Colour.h"

// tools
#include "Blueprint/Tools_Mode_Blueprint.h"
#if defined(ASURA_USE_CONTAINER_SYSTEM)
#include "Entity/Container/Tools_Container_System.h"
#endif	//defined(ASURA_USE_CONTAINER_SYSTEM)
#include "FSFX/Tools_FSFX_System.h"
#include "FSFX/Tools_Mode_FSFX.h"
#include "Hierarchy/Tools_Animation_Files.h"
#include "Interface/Tools_ColourPicker.h"
#include "Load&Save/Tools_AsuraFileContents.h"
#include "Misc/Tools_Flag_IniFile.h"
#include "Misc/Tools_Modal_Overlay.h"
#include "Misc/Tools_Timed_Messages.h"
#include "Particles/Tools_Mode_PFX.h"
#include "Particles/Tools_PFX_Overlay.h"
#include "PC/FileSystem/Tools_FileSystem.h"
#include "Shapes/Tools_Shapes.h"
#include "Sound/Tools_PickDialogueEvent_Overlay.h"
#include "Sound/Tools_PickSoundEvent_Overlay.h"
#include "Sound/Tools_SoundEvent_System.h"
#include "Tools_System.h"

//////////////////////////////////////////////////////////////////////////////
//
// Tools_Blueprint_ParameterGroupColumnHeading
//
//////////////////////////////////////////////////////////////////////////////

class Tools_Blueprint_ParameterGroupColumnHeading : public Asura_TableColumnHeading_Dialog
{
    typedef Asura_TableColumnHeading_Dialog PARENT;

public:
    Tools_Blueprint_ParameterGroupColumnHeading( Asura_Table_Dialog& xTableDialog, const char* szName,
        const Asura_Vector_2& xOffset, const Asura_Vector_2& xMinCellSize, const u_int uColumn )
        : PARENT( xTableDialog, szName, xOffset, xMinCellSize, uColumn )
    {
        Asura_Vector_2 xLocalOffset( 0.0f, 0.0f );
        Asura_Dialog* pxDialog = new Asura_Function_Dialog( szName, xLocalOffset, Tools_Mode_Blueprint::AddParameterGroupName );
        AddDialogToPanel( pxDialog );
        xLocalOffset.x += pxDialog->m_xMaxCoord.x;
    }

    virtual int	Compare( const Asura_TableCell_Dialog& xOther ) const
    {
        return ( GetName().Compare( xOther.GetName() ) );
    }
};

//////////////////////////////////////////////////////////////////////////////
//
// Tools_Blueprint_TableCell
// Base table cell class for the blueprint cells.
// Holds information about the associated blueprint edit object, blueprint, and blueprint parameter.
//
//////////////////////////////////////////////////////////////////////////////

class Tools_Blueprint_TableCell : public Asura_TableCell_Dialog
{
    typedef Asura_TableCell_Dialog PARENT;

public:
    Tools_Blueprint_TableCell( Tools_Blueprint_Edit_Object* pxBlueprintEditObject,
                               const Asura_Vector_2& xOffset, const Asura_Vector_2& xMinCellSize, const u_int uRow, const u_int uColumn )
    : PARENT( *(pxBlueprintEditObject->m_pxTableDialogue), xOffset, xMinCellSize, uRow, uColumn, 0 )
    , m_pxBlueprintEditObject( pxBlueprintEditObject )
    , m_uBlueprintTypeHashID( pxBlueprintEditObject->GetBlueprintTypeHashID() )
    , m_uBlueprintHashID( pxBlueprintEditObject->GetBlueprintHashID() )
    , m_uBlueprintParamHashID( Asura_GetHashID( pxBlueprintEditObject->m_ppszParameterStrings[ uRow ] ) )
    {
    }

    Tools_Blueprint_Edit_Object* GetBlueprintEditObject()  const { return m_pxBlueprintEditObject; }
    Asura_Hash_ID                GetBlueprintTypeHashID()  const { return m_uBlueprintTypeHashID; }
    Asura_Hash_ID                GetBlueprintHashID()      const { return m_uBlueprintHashID; }
    Asura_Hash_ID                GetBlueprintParamHashID() const { return m_uBlueprintParamHashID; }
    Asura_Blueprint*             GetBlueprint()            const { return Asura_Blueprint_System::GetBlueprintByHashID( GetBlueprintTypeHashID(), GetBlueprintHashID() ); }

private:
    Tools_Blueprint_Edit_Object* m_pxBlueprintEditObject;
    Asura_Hash_ID                m_uBlueprintTypeHashID;
    Asura_Hash_ID                m_uBlueprintHashID;
    Asura_Hash_ID                m_uBlueprintParamHashID;
};

//////////////////////////////////////////////////////////////////////////////
//
// Tools_Blueprint_TableCell_ParameterGroup
// Table cell class for displaying the blueprint parameter group.
//
//////////////////////////////////////////////////////////////////////////////

class Tools_Blueprint_TableCell_ParameterGroup : public Tools_Blueprint_TableCell
{
    typedef Tools_Blueprint_TableCell PARENT;

public:
    Tools_Blueprint_TableCell_ParameterGroup( Tools_Blueprint_Edit_Object* pxBlueprintEditObject,
                                              const Asura_Vector_2& xOffset, const Asura_Vector_2& xMinCellSize, const u_int uRow, const u_int uColumn )
    : PARENT( pxBlueprintEditObject, xOffset, xMinCellSize, uRow, uColumn )
    , m_iParameterGroup( 0 )
    {
        Asura_Blueprint* pxBlueprint = GetBlueprint();
        if ( pxBlueprint )
        {
            Asura_Parameter* pxParameter = pxBlueprint->GetParameterByHashID( GetBlueprintParamHashID(), true );
            if ( pxParameter )
            {
                m_iParameterGroup = Asura_Parameter::GetGroupListIndex( pxParameter->GetGroupName() );

                Asura_Vector_2 xLocalOffset( 0.0f, 0.0f );
                Asura_DropDown_List_Dialog* pxDDLDialog = new Asura_DropDown_List_Dialog( xLocalOffset, Asura_Parameter::GetGroupNameList(), &m_iParameterGroup,
                                                                                          Asura_Parameter::GetNumParameterGroups(),
                                                                                          Asura_Parameter::GetNumParameterGroups(), NULL );
                pxDDLDialog->SetOnSelectCallback( ParameterGroupChangedStatic, reinterpret_cast< int >( this ) );
                AddDialogToPanel( pxDDLDialog );
                xLocalOffset.x += pxDDLDialog->m_xMaxCoord.x;
            }
        }
    }

    virtual int Compare( const Asura_TableCell_Dialog& xOther ) const
    {
        // We're only ever going to be sorting by column, so we can guarantee that the other cell is of the same type as us.
        const Tools_Blueprint_TableCell_ParameterGroup* pxOther = static_cast< const Tools_Blueprint_TableCell_ParameterGroup* >( &xOther );

        Asura_String xOurName, xOtherName;
        Asura_Parameter::GetGroupName( m_iParameterGroup, xOurName );
        Asura_Parameter::GetGroupName( pxOther->m_iParameterGroup, xOtherName );

		const int iCompare = xOurName.Compare( xOtherName );
		if ( iCompare == 0 )
		{
			// Group names are equal, so we must also compare the parameter names.
			Asura_Blueprint* pxOurBP = GetBlueprint();
			Asura_Blueprint* pxOtherBP = pxOther->GetBlueprint();
			if ( !pxOurBP || !pxOtherBP )
			{
				return iCompare;
			}

			const Asura_Parameter* pxOurParam = pxOurBP->GetParameterByHashID( GetBlueprintParamHashID(), true );
			const Asura_Parameter* pxOtherParam = pxOtherBP->GetParameterByHashID( pxOther->GetBlueprintParamHashID(), true );
			if ( !pxOurParam || !pxOtherParam )
			{
				return iCompare;
			}

			return pxOurParam->GetName().Compare( pxOtherParam->GetName() );
		}
		else
		{
			return iCompare;
		}
    }

private:
    static void ParameterGroupChangedStatic( int iUserData )
    {
        Tools_Blueprint_TableCell_ParameterGroup* pxThis = reinterpret_cast< Tools_Blueprint_TableCell_ParameterGroup* >( iUserData );
        pxThis->ParameterGroupChanged();
    }

    void ParameterGroupChanged()
    {
        Asura_Blueprint* pxBlueprint = GetBlueprint();
        if ( !pxBlueprint ) { return; }
        Asura_Parameter* pxParameter = pxBlueprint->GetParameterByHashID( GetBlueprintParamHashID(), true );
        if ( !pxParameter ) { return; }

        // Update the group name if necessary.
        Asura_String xNewGroupName;
        Asura_Parameter::GetGroupName( m_iParameterGroup, xNewGroupName );
        if ( pxParameter->GetGroupName().Compare( xNewGroupName ) == 0 )
        {
            // Nothing to do.
            return;
        }
        else
        {
			// We must update the group name for all associated parameters in this type.
			for ( Asura_Blueprint_ConstIterator xBPIter( GetBlueprintTypeHashID() ); !xBPIter.Done(); xBPIter.Next() )
			{
				const Asura_Blueprint* pxTargetBlueprint = xBPIter.GetCurrent();
				Asura_Parameter* pxTargetParameter = pxTargetBlueprint->GetParameterByHashID( GetBlueprintParamHashID(), false );
				if ( pxTargetParameter )
				{
					pxTargetParameter->SetGroupName( xNewGroupName );
				}
			}
        }

        Tools_Mode_Blueprint::SetTypeAsModified( GetBlueprintTypeHashID() );
        Tools_Mode_Blueprint::RequestRefreshBlueprintEditObjectOverlays();
    }

private:
    // Also the selection index into the drop-down list dialogue.
    int m_iParameterGroup;
};

//////////////////////////////////////////////////////////////////////////////
//
// Tools_Blueprint_TableCell_ParameterType
// Table cell class for displaying the blueprint parameter type.
//
//////////////////////////////////////////////////////////////////////////////

class Tools_Blueprint_TableCell_ParameterType : public Tools_Blueprint_TableCell
{
    typedef Tools_Blueprint_TableCell PARENT;

public:
    Tools_Blueprint_TableCell_ParameterType( Tools_Blueprint_Edit_Object* pxBlueprintEditObject,
                                             const Asura_Vector_2& xOffset, const Asura_Vector_2& xMinCellSize, const u_int uRow, const u_int uColumn )
    : PARENT( pxBlueprintEditObject, xOffset, xMinCellSize, uRow, uColumn )
    {
        Asura_Blueprint* pxBlueprint = GetBlueprint();
        if ( pxBlueprint )
        {
            Asura_Parameter* pxParameter = pxBlueprint->GetParameterByHashID( GetBlueprintParamHashID(), true );
            if ( pxParameter )
            {
                Asura_Vector_2 xLocalOffset( 0.0f, 0.0f );
                const char* pszValueTypeString = NULL;
                const int iTemplatedType = GetBlueprintEditObject()->GetTemplatedTypeFromTemplateID( pxParameter->GetTemplate() );
                if ( iTemplatedType != -1 )
                {
                    pszValueTypeString = GetBlueprintEditObject()->GetTemplatedTypeName( iTemplatedType );
                }
                else
                {
                    pszValueTypeString = Asura_Value::s_apszValueTypeStrings[ pxParameter->GetValue().GetValueType() ];
                }
                Asura_Dialog* pxDialog = new Asura_Dialog( pszValueTypeString, xLocalOffset );
                pxDialog->SetControlID( reinterpret_cast< int >( this ) );
                AddDialogToPanel( pxDialog );
                xLocalOffset.x += pxDialog->m_xMaxCoord.x;
            }
        }
    }

    virtual int Compare( const Asura_TableCell_Dialog& xOther ) const
    {
		// We're only ever going to be sorting by column, so we can guarantee that the other cell is of the same type as us.
		const Tools_Blueprint_TableCell_ParameterType* pxOther = static_cast< const Tools_Blueprint_TableCell_ParameterType* >( &xOther );

        const Asura_Dialog* pxOurDialogue   = FindDialogOnPanel( reinterpret_cast< int >( this ) );
        const Asura_Dialog* pxOtherDialogue = xOther.FindDialogOnPanel( reinterpret_cast< int >( &xOther ) );
		if ( !pxOurDialogue || !pxOtherDialogue )
		{
			return 0;
		}

		const int iCompare = pxOurDialogue->GetName().Compare( pxOtherDialogue->GetName() );
		if ( iCompare == 0 )
		{
			// Type names are equal, so we must also compare the parameter names.
			Asura_Blueprint* pxOurBP = GetBlueprint();
			Asura_Blueprint* pxOtherBP = pxOther->GetBlueprint();
			if ( !pxOurBP || !pxOtherBP )
			{
				return iCompare;
			}

			const Asura_Parameter* pxOurParam = pxOurBP->GetParameterByHashID( GetBlueprintParamHashID(), true );
			const Asura_Parameter* pxOtherParam = pxOtherBP->GetParameterByHashID( pxOther->GetBlueprintParamHashID(), true );
			if ( !pxOurParam || !pxOtherParam )
			{
				return iCompare;
			}

			return pxOurParam->GetName().Compare( pxOtherParam->GetName() );
		}
		else
		{
			return iCompare;
		}
    }
};

//////////////////////////////////////////////////////////////////////////////
//
// Tools_Blueprint_TableCell_InheritedValue
// Table cell class for displaying the value of the blueprint parameter as inherited by this blueprint.
//
//////////////////////////////////////////////////////////////////////////////

class Tools_Blueprint_TableCell_InheritedValue : public Tools_Blueprint_TableCell
{
    typedef Tools_Blueprint_TableCell PARENT;

public:
    Tools_Blueprint_TableCell_InheritedValue( Tools_Blueprint_Edit_Object* pxBlueprintEditObject,
                                              const Asura_Vector_2& xOffset, const Asura_Vector_2& xMinCellSize, const u_int uRow, const u_int uColumn )
    : PARENT( pxBlueprintEditObject, xOffset, xMinCellSize, uRow, uColumn )
    {
        Asura_Blueprint* pxBlueprint = GetBlueprint();
        if ( pxBlueprint )
        {
            Asura_Blueprint* pxParentBlueprint = pxBlueprint->GetParent();
            if ( pxParentBlueprint )
            {
                // We need to find the raw parameter so that we know what template it uses (if any).
                Asura_Parameter* pxParameter = pxParentBlueprint->GetParameterByHashID( GetBlueprintParamHashID(), true );
                Asura_Value xValue;
                if ( pxParameter && pxParentBlueprint->GetParameterValueByHashID( GetBlueprintParamHashID(), xValue ) )
                {
                    Asura_Vector_2 xLocalOffset( 0.0f, 0.0f );
					Asura_String xValueString;
                    GetBlueprintEditObject()->GetValueString( xValue,
                        GetBlueprintEditObject()->GetTemplatedTypeFromTemplateID( pxParameter->GetTemplate() ),
                        xValueString );

                    Asura_ClickableString_Dialog* pxDialog = NULL;
                    if ( GetBlueprintEditObject()->ShouldShowFullStrings() )
                    {
						pxDialog = new Asura_ClickableString_Dialog( xValueString.GetString(), xLocalOffset );
						pxDialog->SetClickFunction( OpenDefiningBlueprint, reinterpret_cast< int >( this ) );
                    }
                    else
                    {
						Asura_String xCollapsedString = xValueString;
                        const bool bCollapsedString = Tools_Blueprint_Edit_Object::CollapseString( xCollapsedString );
						pxDialog = new Asura_ClickableString_Dialog( xCollapsedString.GetString(), xLocalOffset );
						pxDialog->SetClickFunction( OpenDefiningBlueprint, reinterpret_cast< int >( this ) );
                        if ( bCollapsedString )
                        {
                            pxDialog->SetHelpText( xValueString.GetString() );
                        }
                    }

                    AddDialogToPanel( pxDialog );
                    xLocalOffset.x += pxDialog->m_xMaxCoord.x;
                }
            }
        }
    }

    virtual int Compare( const Asura_TableCell_Dialog& xOther ) const { return 0; }

protected:
	static void OpenDefiningBlueprint( int iThis )
	{
		Tools_Blueprint_TableCell_InheritedValue* pxThis = reinterpret_cast< Tools_Blueprint_TableCell_InheritedValue* >( iThis );

		// Find the next highest blueprint where this parameter is defined and open it.
		Asura_Blueprint* pxParent = pxThis->GetBlueprint()->GetParent();
		while ( pxParent )
		{
			if ( pxParent->GetParameterByHashID( pxThis->GetBlueprintParamHashID() ) )
			{
				Tools_Mode_Blueprint::CreateEditOverlay( pxThis->GetBlueprintTypeHashID(), pxParent->GetHashID(), pxThis->GetBlueprintParamHashID() );
				break;
			}
			pxParent = pxParent->GetParent();
		}
	}
};

//////////////////////////////////////////////////////////////////////////////
//
// Tools_Blueprint_TableCell_OverrideType
// Table cell class for displaying and editing the override type.
//
//////////////////////////////////////////////////////////////////////////////

class Tools_Blueprint_TableCell_OverrideType : public Tools_Blueprint_TableCell
{
    typedef Tools_Blueprint_TableCell PARENT;

public:
    Tools_Blueprint_TableCell_OverrideType( Tools_Blueprint_Edit_Object* pxBlueprintEditObject,
                                            const Asura_Vector_2& xOffset, const Asura_Vector_2& xMinCellSize, const u_int uRow, const u_int uColumn )
    : PARENT( pxBlueprintEditObject, xOffset, xMinCellSize, uRow, uColumn )
    , m_iOverrideType( 0 )
    {
        // If this is an inherited parameter we can specify an override type, otherwise we leave this blank.
        Asura_Blueprint* pxBlueprint = GetBlueprint();
        if ( pxBlueprint )
        {
            // Is it inherited?
            bool bIsInheritedParameter = false;
            Asura_Blueprint* pxParentBlueprint = pxBlueprint->GetParent();
            if ( pxParentBlueprint )
            {
                // If we can get the parameter value from the parent blueprint it must be an inherited parameter.
                Asura_Value xValue;
                bIsInheritedParameter = pxParentBlueprint->GetParameterValueByHashID( GetBlueprintParamHashID(), xValue );
            }

            if ( bIsInheritedParameter )
            {
                // If we can get the parameter from our own blueprint there must already be an override, otherwise we haven't overridden this parameter yet.
                Asura_Parameter* pxLocalParameter = pxBlueprint->GetParameterByHashID( GetBlueprintParamHashID(), false );
                if ( pxLocalParameter )
                {
                    m_iOverrideType = static_cast< int >( pxLocalParameter->GetOverrideType() );
                }

                Asura_Vector_2 xLocalOffset( 0.0f, 0.0f );
                Asura_DropDown_List_Dialog* pxDDLDialog = new Asura_DropDown_List_Dialog( xLocalOffset, s_apszOverrideTypeStrings, &m_iOverrideType,
                                                                                          ABOT_END_OVERRIDE_TYPES, ABOT_END_OVERRIDE_TYPES, NULL );
                pxDDLDialog->SetOnSelectCallback( OverrideTypeChangedStatic, reinterpret_cast< int >( this ) );
                AddDialogToPanel( pxDDLDialog );
                xLocalOffset.x += pxDDLDialog->m_xMaxCoord.x;
            }
            else
            {
                // Can't override an non-inherited parameter. Set the override type to -1 so that these sort separately from non-overridden inherited parameters.
                m_iOverrideType = -1;
            }
        }
    }

    virtual int Compare( const Asura_TableCell_Dialog& xOther ) const
    {
        // We're only ever going to be sorting by column, so we can guarantee that the other cell is of the same type as us.
        const Tools_Blueprint_TableCell_OverrideType* pxOther = static_cast< const Tools_Blueprint_TableCell_OverrideType* >( &xOther );
        const int iCompare = m_iOverrideType - pxOther->m_iOverrideType;

		if ( iCompare == 0 )
		{
			// Override type names are equal, so we must also compare the parameter names.
			Asura_Blueprint* pxOurBP = GetBlueprint();
			Asura_Blueprint* pxOtherBP = pxOther->GetBlueprint();
			if ( !pxOurBP || !pxOtherBP )
			{
				return iCompare;
			}

			const Asura_Parameter* pxOurParam = pxOurBP->GetParameterByHashID( GetBlueprintParamHashID(), true );
			const Asura_Parameter* pxOtherParam = pxOtherBP->GetParameterByHashID( pxOther->GetBlueprintParamHashID(), true );
			if ( !pxOurParam || !pxOtherParam )
			{
				return iCompare;
			}

			return pxOurParam->GetName().Compare( pxOtherParam->GetName() );
		} 
		else
		{
			return iCompare;
		}
    }

private:
    static void OverrideTypeChangedStatic( int iUserData )
    {
        Tools_Blueprint_TableCell_OverrideType* pxThis = reinterpret_cast< Tools_Blueprint_TableCell_OverrideType* >( iUserData );
        pxThis->OverrideTypeChanged();
    }

    void OverrideTypeChanged()
    {
        Asura_Blueprint* pxBlueprint = GetBlueprint();
        if ( !pxBlueprint ) { return; }
        Asura_Parameter* pxParameter = pxBlueprint->GetParameterByHashID( GetBlueprintParamHashID(), true );
        if ( !pxParameter ) { return; }

        // If the override type has changed we need to:
        //  - make sure the override type is allowed (most parameters can only have a fixed override)
        if ( !GetBlueprintEditObject()->OverrideTypeIsAllowed( *pxParameter, static_cast< Asura_Blueprint_Override_Type >( m_iOverrideType ) ) )
        {
            // If it is not allowed then make it ABOT_FIXED.
            m_iOverrideType = static_cast< int >( ABOT_FIXED );
        }

        Asura_Parameter*                    pxOverrideParameter = pxBlueprint->GetParameterByHashID( GetBlueprintParamHashID(), false );
        const Asura_Blueprint_Override_Type eOldOverrideType    = pxOverrideParameter ? pxOverrideParameter->GetOverrideType() : ABOT_NONE;
        const Asura_Blueprint_Override_Type eNewOverrideType    = static_cast< Asura_Blueprint_Override_Type >( m_iOverrideType );
        if ( eOldOverrideType == eNewOverrideType )
        {
            // Nothing to do.
            return;
        }

        //  - create or destroy the override as necessary
        // Do we already have an override in this blueprint?
        // No need to check that the parameter in our blueprint is an override, as we're only presented with this dialogue if this parameter is inherited,
        // so if we have a parameter in our blueprint it must be an override.
        const bool bAlreadyHaveOverride = ( pxOverrideParameter != NULL );
        if ( bAlreadyHaveOverride )
        {
            if ( eNewOverrideType == ABOT_NONE )
            {
                // Remove the override parameter in our blueprint.
                pxBlueprint->DeleteParameter( pxOverrideParameter->GetName() );
            }
            else
            {
                pxOverrideParameter->SetOverrideType( eNewOverrideType );
            }
        }
        else
        {
            // Create an override parameter in our blueprint.
            pxBlueprint->AddParameter( pxParameter->GetName(), pxParameter->GetGroupName(),
									   eNewOverrideType, pxParameter->GetValue(),
									   pxParameter->GetTemplate() );
        }

        Tools_Mode_Blueprint::SetTypeAsModified( GetBlueprintTypeHashID() );
        Tools_Mode_Blueprint::RequestRefreshBlueprintEditObjectOverlays();
    }

private:
    // Also the selection index into the drop-down list dialogue.
    int m_iOverrideType;
};

//////////////////////////////////////////////////////////////////////////////
//
// Tools_Blueprint_TableCell_EditableValue
// Base table cell class for the cells (override value and final value) that either simply display a value or allow a value to be edited.
//
//////////////////////////////////////////////////////////////////////////////

class Tools_Blueprint_TableCell_EditableValue : public Tools_Blueprint_TableCell
{
    typedef Tools_Blueprint_TableCell PARENT;

public:
    Tools_Blueprint_TableCell_EditableValue( Tools_Blueprint_Edit_Object* pxBlueprintEditObject,
                                             const Asura_Vector_2& xOffset, const Asura_Vector_2& xMinCellSize, const u_int uRow, const u_int uColumn )
    : PARENT( pxBlueprintEditObject, xOffset, xMinCellSize, uRow, uColumn )
    , m_xFloatIntBool()
    , m_bChanged( false )
    {
        m_xFloatIntBool.m_iInt = 0;
    }

    virtual void PerFrameUpdate()
    {
        PARENT::PerFrameUpdate();

        if ( m_bChanged )
        {
            m_bChanged = false;

            // We can't keep a pointer to the value directly, as blueprints/parameters being added/removed may cause it to move in memory.
            Asura_Blueprint* pxBlueprint = GetBlueprint();
            if ( pxBlueprint )
            {
                Asura_Parameter* pxLocalParameter = pxBlueprint->GetParameterByHashID( GetBlueprintParamHashID(), false );
                if ( pxLocalParameter )
                {
                    Asura_Value xValue = pxLocalParameter->GetValue();
                    if ( xValue.GetValueType() == Asura_Value::VT_INT )
                    {
                        xValue.SetValue( m_xFloatIntBool.m_iInt );
                    }
                    else if ( xValue.GetValueType() == Asura_Value::VT_FLOAT )
                    {
                        xValue.SetValue( m_xFloatIntBool.m_fFloat );
                    }
                    else if ( xValue.GetValueType() == Asura_Value::VT_BOOL )
                    {
                        xValue.SetValue( m_xFloatIntBool.m_bBool );
                    }
                    pxLocalParameter->SetValue( xValue );

                    Tools_Mode_Blueprint::SetTypeAsModified( GetBlueprintTypeHashID() );
                    Tools_Mode_Blueprint::RequestRefreshBlueprintEditObjectOverlays();
                }
            }
        }
    }

protected:
    void CreateDisplayAndEditDialogue( const Asura_Value& xValue, const int iTemplatedType, const Asura_Hash_ID uTemplateHashID,
                                       const Asura_String& xValueString, const Asura_String& xCollapsedString )
    {
        Asura_Vector_2 xLocalOffset( 0.0f, 0.0f );
        Asura_Dialog* pxDialog = NULL;
        if ( iTemplatedType != -1 || xValue.GetValueType() == Asura_Value::VT_HASH )
        {
            if ( GetBlueprintEditObject()->ShouldShowFullStrings() )
            {
                pxDialog = new Asura_Function_Dialog( xValueString.GetString(), xLocalOffset, EditParameterValueStatic, reinterpret_cast< int >( this ) );
            }
            else
            {
                pxDialog = new Asura_Function_Dialog( xCollapsedString.GetString(), xLocalOffset, EditParameterValueStatic, reinterpret_cast< int >( this ) );
                if ( xValueString != xCollapsedString )
                {
                    pxDialog->SetHelpText( xValueString.GetString() );
                }
            }
        }
        else if ( xValue.GetValueType() == Asura_Value::VT_INT )
        {
            m_xFloatIntBool.m_iInt = xValue.GetIntValue();

            const Asura_Blueprint_Template* pxTemplate = GetBlueprintEditObject()->GetTemplate( uTemplateHashID );
            if ( pxTemplate )
            {
                pxDialog = new Asura_Numerical_Dialog( "", xLocalOffset, &m_xFloatIntBool.m_iInt,
                                                       pxTemplate->GetMinInt(), pxTemplate->GetMaxInt(), pxTemplate->GetStepInt(), "%d", &m_bChanged );
            }
            else
            {
                pxDialog = new Asura_Numerical_Dialog( "", xLocalOffset, &m_xFloatIntBool.m_iInt,
                                                       INT_MIN, INT_MAX, 1, "%d", &m_bChanged );
            }
        }
        else if ( xValue.GetValueType() == Asura_Value::VT_FLOAT )
        {
            m_xFloatIntBool.m_fFloat = xValue.GetFloatValue();

            const Asura_Blueprint_Template* pxTemplate = GetBlueprintEditObject()->GetTemplate( uTemplateHashID );
            if ( pxTemplate )
            {
                pxDialog = new Asura_Numerical_Dialog( "", xLocalOffset, &m_xFloatIntBool.m_fFloat,
                                                       pxTemplate->GetMinFloat(), pxTemplate->GetMaxFloat(), pxTemplate->GetStepFloat(), "%.3f", &m_bChanged );
            }
            else
            {
                // This is an arbitrary limit on the value the float can take, but it stops the dialogue box being huge.
                pxDialog = new Asura_Numerical_Dialog( "", xLocalOffset, &m_xFloatIntBool.m_fFloat,
                                                       -99999999.0f, 99999999.0f, 0.1f, "%.3f", &m_bChanged );
            }
        }
        else if ( xValue.GetValueType() == Asura_Value::VT_BOOL )
        {
            m_xFloatIntBool.m_bBool = xValue.GetBoolValue();

            pxDialog = new Asura_DropDown_List_Dialog( xLocalOffset, s_apszBooleanStrings, &m_xFloatIntBool.m_iInt, 2, 2, NULL, &m_bChanged );
        }
        AddDialogToPanel( pxDialog );
        xLocalOffset.x += pxDialog->m_xMaxCoord.x;
    }

    virtual int Compare( const Asura_TableCell_Dialog& xOther ) const { return 0; }

private:
    static void EditParameterValueStatic( int iUserData )
    {
        Tools_Blueprint_TableCell_EditableValue* pxThis = reinterpret_cast< Tools_Blueprint_TableCell_EditableValue* >( iUserData );
        pxThis->EditParameterValue();
    }

    void EditParameterValue()
    {
        Asura_Blueprint* pxBlueprint = GetBlueprint();
        if ( !pxBlueprint ) { return; }
        Asura_Parameter* pxLocalParameter = pxBlueprint->GetParameterByHashID( GetBlueprintParamHashID(), false );
        if ( !pxLocalParameter ) { return; }

        const int iTemplatedType = GetBlueprintEditObject()->GetTemplatedTypeFromTemplateID( pxLocalParameter->GetTemplate() );
        if ( iTemplatedType != -1 )
        {
            Asura_Value xValue = pxLocalParameter->GetValue();
            if ( GetBlueprintEditObject()->GetValueForTemplatedType( iTemplatedType,
																	 pxLocalParameter->GetTemplate(),
																	 pxLocalParameter->GetName(),
																	 xValue ) )
            {
                pxLocalParameter->SetValue( xValue );
                Tools_Mode_Blueprint::SetTypeAsModified( GetBlueprintTypeHashID() );
                Tools_Mode_Blueprint::RequestRefreshBlueprintEditObjectOverlays();
            }
        }
        else if ( pxLocalParameter->GetValue().GetValueType() == Asura_Value::VT_HASH )
        {
            Asura_Value xValue = pxLocalParameter->GetValue();
            if ( pxLocalParameter->GetTemplate() != ASURA_HASH_ID_UNSET )
            {
                Asura_Blueprint_Template* pxTemplate = Asura_Blueprint_System::GetTemplate( pxLocalParameter->GetTemplate() );
                if ( !pxTemplate ) { return; }
				Asura_Blueprint_System::Project_RefreshTemplate( pxTemplate );

                ListSelection_Overlay xListOverlay( "Choose value:" );
                pxTemplate->GetStrings( xListOverlay );
                if ( !xListOverlay.DoModal() ) { return; }

                xValue.SetValue( Asura_GetHashID( xListOverlay.GetSelectionString() ), xListOverlay.GetSelectionString() );
            }
            else
            {
                GetString_Overlay xGetStringOverlay( "Enter the string value:",
                    pxLocalParameter->GetValue().GetHashDisplayTextHash() != ASURA_HASH_ID_UNSET ? pxLocalParameter->GetValue().GetHashStringValue() : NULL,
                    64 );
                if ( !xGetStringOverlay.DoModal() ) { return; }

                xValue.SetValue( Asura_GetHashID( xGetStringOverlay.GetString() ), xGetStringOverlay.GetString() );
            }
            pxLocalParameter->SetValue( xValue );
            Tools_Mode_Blueprint::SetTypeAsModified( GetBlueprintTypeHashID() );
            Tools_Mode_Blueprint::RequestRefreshBlueprintEditObjectOverlays();
        }
    }

private:
    union
    {
        int   m_iInt;
        float m_fFloat;
        bool  m_bBool;
    }    m_xFloatIntBool;
    bool m_bChanged;
};

//////////////////////////////////////////////////////////////////////////////
//
// Tools_Blueprint_TableCell_OverrideValue
// Table cell class for displaying and editing the blueprint parameter override value.
// If this parameter is not overridden in this blueprint it displays nothing.
//
//////////////////////////////////////////////////////////////////////////////

class Tools_Blueprint_TableCell_OverrideValue : public Tools_Blueprint_TableCell_EditableValue
{
    typedef Tools_Blueprint_TableCell_EditableValue PARENT;

public:
    Tools_Blueprint_TableCell_OverrideValue( Tools_Blueprint_Edit_Object* pxBlueprintEditObject,
                                             const Asura_Vector_2& xOffset, const Asura_Vector_2& xMinCellSize, const u_int uRow, const u_int uColumn )
    : PARENT( pxBlueprintEditObject, xOffset, xMinCellSize, uRow, uColumn )
    {
        Asura_Blueprint* pxBlueprint = GetBlueprint();
        if ( pxBlueprint )
        {
            // Only display something if this parameter is overridden in this blueprint.
            Asura_Parameter* pxLocalParameter = pxBlueprint->GetParameterByHashID( GetBlueprintParamHashID(), false );
            if ( pxLocalParameter && pxLocalParameter->GetOverrideType() != ABOT_NONE )
            {
                // Get the display strings.
                const int          iTemplatedType = GetBlueprintEditObject()->GetTemplatedTypeFromTemplateID( pxLocalParameter->GetTemplate() );
                const Asura_Value& xValue         = pxLocalParameter->GetValue();
                Asura_String xValueString;
                GetBlueprintEditObject()->GetValueString( xValue, iTemplatedType, xValueString );
                Asura_String xCollapsedString = xValueString;
                Tools_Blueprint_Edit_Object::CollapseString( xCollapsedString );

                CreateDisplayAndEditDialogue( pxLocalParameter->GetValue(), iTemplatedType, pxLocalParameter->GetTemplate(), xValueString, xCollapsedString );
            }
        }
    }
};

//////////////////////////////////////////////////////////////////////////////
//
// Tools_Blueprint_TableCell_FinalValue
// Table cell class for displaying and editing the final value of the blueprint parameter after any overrides in this blueprint have been applied.
// Editing of the final value is only allowed if the parameter is new in this blueprint (not inherited).
//
//////////////////////////////////////////////////////////////////////////////

class Tools_Blueprint_TableCell_FinalValue : public Tools_Blueprint_TableCell_EditableValue
{
    typedef Tools_Blueprint_TableCell_EditableValue PARENT;

public:
    Tools_Blueprint_TableCell_FinalValue( Tools_Blueprint_Edit_Object* pxBlueprintEditObject,
                                          const Asura_Vector_2& xOffset, const Asura_Vector_2& xMinCellSize, const u_int uRow, const u_int uColumn )
    : PARENT( pxBlueprintEditObject, xOffset, xMinCellSize, uRow, uColumn )
    {
        Asura_Blueprint* pxBlueprint = GetBlueprint();
        if ( pxBlueprint )
        {
            // We need to find the raw parameter so that we know what template it uses (if any).
            Asura_Parameter* pxParameter = pxBlueprint->GetParameterByHashID( GetBlueprintParamHashID(), true );
            Asura_Value xValue;
            if ( pxParameter && pxBlueprint->GetParameterValueByHashID( GetBlueprintParamHashID(), xValue ) )
            {
                // Get the display strings.
                const int iTemplatedType = GetBlueprintEditObject()->GetTemplatedTypeFromTemplateID( pxParameter->GetTemplate() );
                Asura_String xValueString;
                GetBlueprintEditObject()->GetValueString( xValue, iTemplatedType, xValueString );
                Asura_String xCollapsedString = xValueString;
                Tools_Blueprint_Edit_Object::CollapseString( xCollapsedString );

                // Create the correct dialogue.
                Asura_Parameter* pxLocalParameter = pxBlueprint->GetParameterByHashID( GetBlueprintParamHashID(), false );
				if ( !pxLocalParameter )
				{
					// If this is inherited we don't want to be able to edit the final value.
					// If we click on it though then we create a fixed override local parameter.
					CreateDisplayAndClickDialogue( xValueString, xCollapsedString );
				}
                else if ( pxLocalParameter->GetOverrideType() != ABOT_NONE )
                {
                    // If this is an override we don't want to be able to edit the final value.
					CreateDisplayOnlyDialogue( xValueString, xCollapsedString );
                }
                else
                {
                    // If this is not inherited or an override we should be able to edit the final value.
                    CreateDisplayAndEditDialogue( pxLocalParameter->GetValue(), iTemplatedType, pxLocalParameter->GetTemplate(), xValueString, xCollapsedString );
                }
            }
        }
    }

protected:
	void CreateDisplayOnlyDialogue( const Asura_String& xValueString, const Asura_String& xCollapsedString )
	{
		Asura_Vector_2 xLocalOffset( 0.0f, 0.0f );
		Asura_Dialog* pxDialog = NULL;
		if ( GetBlueprintEditObject()->ShouldShowFullStrings() )
		{
			pxDialog = new Asura_Dialog( xValueString.GetString(), xLocalOffset );
		}
		else
		{
			pxDialog = new Asura_Dialog( xCollapsedString.GetString(), xLocalOffset );
			if ( xValueString != xCollapsedString )
			{
				pxDialog->SetHelpText( xValueString.GetString() );
			}
		}
		AddDialogToPanel( pxDialog );
		xLocalOffset.x += pxDialog->m_xMaxCoord.x;
	}

	void CreateDisplayAndClickDialogue( const Asura_String& xValueString, const Asura_String& xCollapsedString )
	{
	    Asura_Vector_2 xLocalOffset( 0.0f, 0.0f );
	    Asura_ClickableString_Dialog* pxDialog = NULL;
	    if ( GetBlueprintEditObject()->ShouldShowFullStrings() )
	    {
	        pxDialog = new Asura_ClickableString_Dialog( xValueString.GetString(), xLocalOffset );
			pxDialog->SetClickFunction( CreateFixedOverride, reinterpret_cast< int >( this ) );
	    }
	    else
	    {
	        pxDialog = new Asura_ClickableString_Dialog( xCollapsedString.GetString(), xLocalOffset );
			pxDialog->SetClickFunction( CreateFixedOverride, reinterpret_cast< int >( this ) );
	        if ( xValueString != xCollapsedString )
	        {
	            pxDialog->SetHelpText( xValueString.GetString() );
	        }
	    }
	    AddDialogToPanel( pxDialog );
	    xLocalOffset.x += pxDialog->m_xMaxCoord.x;
	}

	static void CreateFixedOverride( int iThis )
	{
		// We can guarantee we won't already have an override of this parameter in our blueprint as we'd not have created this dialogue if we had.
		Tools_Blueprint_TableCell_FinalValue* pxThis = reinterpret_cast< Tools_Blueprint_TableCell_FinalValue* >( iThis );

		Asura_Blueprint* pxBlueprint = pxThis->GetBlueprint();
		if ( !pxBlueprint ) { return; }
		Asura_Parameter* pxParameter = pxBlueprint->GetParameterByHashID( pxThis->GetBlueprintParamHashID(), true );
		if ( !pxParameter ) { return; }

		// Create an override parameter in our blueprint.
		pxBlueprint->AddParameter( pxParameter->GetName(), pxParameter->GetGroupName(),
								   ABOT_FIXED, pxParameter->GetValue(),
								   pxParameter->GetTemplate() );

		Tools_Mode_Blueprint::SetTypeAsModified( pxThis->GetBlueprintTypeHashID() );
		Tools_Mode_Blueprint::RequestRefreshBlueprintEditObjectOverlays();
	}
};

//////////////////////////////////////////////////////////////////////////////
//
// Tools_Modal_Overlay_BPParameterSelect
// Provides a modal overlay for selecting the parameters to cut/copy from a blueprint.
//
//////////////////////////////////////////////////////////////////////////////

class Tools_Modal_Overlay_BPParameterSelect : public Tools_Modal_Overlay
{
public:
    Tools_Modal_Overlay_BPParameterSelect( const char* pszTitleString, const Asura_Hash_ID uBlueprintTypeHashID, const Asura_Hash_ID uBlueprintHashID,
                                           const bool bOnlyLocalParameters )
    : Tools_Modal_Overlay( pszTitleString )
    , m_bReturnValue( false )
    , m_iNumParameterStrings( 0 )
    , m_ppszParameterStrings( NULL )
    , m_pbParameterTickboxIsChecked( NULL )
    , m_iSelectionIndex( 0 )
    {
        CreateStringAndBoolArrays( uBlueprintTypeHashID, uBlueprintHashID, bOnlyLocalParameters );
    }
    virtual ~Tools_Modal_Overlay_BPParameterSelect()
    {
        DestroyStringAndBoolArrays();
    }

    virtual bool DoModal()
    {
        m_bReturnValue = false;
		if ( m_iNumParameterStrings > 0 )
		{
			ShowOverlay();
		}
        return m_bReturnValue;
    }

	int GetNumItems() const
	{
		return m_iNumParameterStrings;
	}

    int GetNumSelectedParameters() const
    {
        int iCount = 0;
        for ( int i = 0; i < m_iNumParameterStrings; ++i )
        {
            if ( m_pbParameterTickboxIsChecked[ i ] )
            {
                ++iCount;
            }
        }
        return iCount;
    }

    Asura_Hash_ID GetSelectedParameterHashID( const int iIndex ) const
    {
        int iCount = 0;
        for ( int i = 0; i < m_iNumParameterStrings; ++i )
        {
            if ( m_pbParameterTickboxIsChecked[ i ] )
            {
                if ( iCount == iIndex )
                {
                    return Asura_GetHashID( m_ppszParameterStrings[ i ] );
                }
                ++iCount;
            }
        }
        return ASURA_HASH_ID_UNSET;
    }

    Asura_String GetSelectedParameterName( const int iIndex ) const
    {
        int iCount = 0;
        for ( int i = 0; i < m_iNumParameterStrings; ++i )
        {
            if ( m_pbParameterTickboxIsChecked[ i ] )
            {
                if ( iCount == iIndex )
                {
                    return Asura_String( m_ppszParameterStrings[ i ] );
                }
                ++iCount;
            }
        }
        return Asura_String( "" );
    }

    bool ParameterIsSelected( const Asura_String& xParameterName ) const
    {
        for ( int i = 0; i < m_iNumParameterStrings; ++i )
        {
            if ( xParameterName.Compare( m_ppszParameterStrings[ i ] ) == 0 )
            {
                return m_pbParameterTickboxIsChecked[ i ];
            }
        }

        return false;
    }

protected:
    virtual void InitialiseOverlay( Asura_Overlay* pxOverlay );

private:
    void CreateStringAndBoolArrays( const Asura_Hash_ID uBlueprintTypeHashID, const Asura_Hash_ID uBlueprintHashID, const bool bOnlyLocalParameters );
    void DestroyStringAndBoolArrays()
    {
        for ( int i = 0; i < m_iNumParameterStrings; ++i )
        {
            delete [] m_ppszParameterStrings[ i ];
        }
        delete [] m_ppszParameterStrings;
        m_ppszParameterStrings = NULL;
        m_iNumParameterStrings = 0;

        delete [] m_pbParameterTickboxIsChecked;
        m_pbParameterTickboxIsChecked = NULL;
    }
    static bool TickboxIsChecked( int iIndex, int iUserData )
    {
        Tools_Modal_Overlay_BPParameterSelect* pxThis = reinterpret_cast< Tools_Modal_Overlay_BPParameterSelect* >( iUserData );
        return pxThis->m_pbParameterTickboxIsChecked[ iIndex ];
    }
    static void ToggleTickboxChecked( int iIndex, int iUserData )
    {
        Tools_Modal_Overlay_BPParameterSelect* pxThis = reinterpret_cast< Tools_Modal_Overlay_BPParameterSelect* >( iUserData );
        pxThis->m_pbParameterTickboxIsChecked[ iIndex ] = !pxThis->m_pbParameterTickboxIsChecked[ iIndex ];
    }
    static void SelectAll( int iUserData )
    {
        Tools_Modal_Overlay_BPParameterSelect* pxThis = reinterpret_cast< Tools_Modal_Overlay_BPParameterSelect* >( iUserData );
        for ( int i = 0; i < pxThis->m_iNumParameterStrings; ++i )
        {
            pxThis->m_pbParameterTickboxIsChecked[ i ] = true;
        }
    }
    static void InvertSelection( int iUserData )
    {
        Tools_Modal_Overlay_BPParameterSelect* pxThis = reinterpret_cast< Tools_Modal_Overlay_BPParameterSelect* >( iUserData );
        for ( int i = 0; i < pxThis->m_iNumParameterStrings; ++i )
        {
            pxThis->m_pbParameterTickboxIsChecked[ i ] = !pxThis->m_pbParameterTickboxIsChecked[ i ];
        }
    }
    static void ClearSelection( int iUserData )
    {
        Tools_Modal_Overlay_BPParameterSelect* pxThis = reinterpret_cast< Tools_Modal_Overlay_BPParameterSelect* >( iUserData );
        for ( int i = 0; i < pxThis->m_iNumParameterStrings; ++i )
        {
            pxThis->m_pbParameterTickboxIsChecked[ i ] = false;
        }
    }
    static void OkayPressed( int iUserData )
    {
        Tools_Modal_Overlay_BPParameterSelect* pxThis = reinterpret_cast< Tools_Modal_Overlay_BPParameterSelect* >( iUserData );
        pxThis->m_bReturnValue = true;
    }
    static void CancelPressed( int iUserData )
    {
        Tools_Modal_Overlay_BPParameterSelect* pxThis = reinterpret_cast< Tools_Modal_Overlay_BPParameterSelect* >( iUserData );
        pxThis->m_bReturnValue = false;
    }

private:
    bool   m_bReturnValue;
    int    m_iNumParameterStrings;
    char** m_ppszParameterStrings;
    bool*  m_pbParameterTickboxIsChecked;
    int    m_iSelectionIndex;
};

void Tools_Modal_Overlay_BPParameterSelect::InitialiseOverlay( Asura_Overlay* pxOverlay )
{
    // First initialise any overlay data.
    m_iSelectionIndex = 0;
    m_bReturnValue    = false;

    // Submit the dialogues.
    Asura_Vector_2 xOffset( 1.0f, 3.0f );
    const int iNumberOfLines = Asura_Maths::Min( m_iNumParameterStrings, 20 );
    Asura_Dialog* pxDialogue = new Asura_List_Dialog( xOffset, m_ppszParameterStrings, &m_iSelectionIndex, iNumberOfLines, m_iNumParameterStrings,
                                                      TickboxIsChecked, ToggleTickboxChecked, reinterpret_cast< int >( this ) );
    pxOverlay->AddDialog( pxDialogue );
    xOffset.y = pxDialogue->m_xMaxCoord.y + 1.0f;

    pxDialogue = new Asura_Function_Dialog( "Select All", xOffset, SelectAll, reinterpret_cast< int >( this ) );
    pxOverlay->AddDialog( pxDialogue );
    xOffset.y += Asura_Dialog::DefaultSpacing();

    pxDialogue = new Asura_Function_Dialog( "Invert Selection", xOffset, InvertSelection, reinterpret_cast< int >( this ) );
    pxOverlay->AddDialog( pxDialogue );
    xOffset.y += Asura_Dialog::DefaultSpacing();

    pxDialogue = new Asura_Function_Dialog( "Clear Selection", xOffset, ClearSelection, reinterpret_cast< int >( this ) );
    pxOverlay->AddDialog( pxDialogue );
    xOffset.y += Asura_Dialog::LargeSpacing();

    pxDialogue = new Asura_Function_Dialog( "Okay", xOffset, OkayPressed, reinterpret_cast< int >( this ), &pxOverlay->m_bShouldClose );
    pxOverlay->AddDialog( pxDialogue );
    xOffset.x += 7.0f;

    pxDialogue = new Asura_Function_Dialog( "Cancel", xOffset, CancelPressed, reinterpret_cast< int >( this ), &pxOverlay->m_bShouldClose );
    pxOverlay->AddDialog( pxDialogue );
}


void Tools_Modal_Overlay_BPParameterSelect::CreateStringAndBoolArrays( const Asura_Hash_ID uBlueprintTypeHashID, const Asura_Hash_ID uBlueprintHashID,
                                                                       const bool bOnlyLocalParameters )
{
    Asura_Blueprint* pxBlueprint = Asura_Blueprint_System::GetBlueprintByHashID( uBlueprintTypeHashID, uBlueprintHashID );
    if ( !pxBlueprint ) { return; }
    Asura_Blueprint* const pxOriginalBlueprint = pxBlueprint;

    // Traverse from the current blueprint upwards and count each individual parameter.
    while ( pxBlueprint )
    {
        const int iNumParameters = pxBlueprint->GetNumParameters();
        for ( int i = 0; i < iNumParameters; ++i )
        {
            Asura_Parameter* pxParameter = pxBlueprint->GetParameterByIndex( i );
            if ( pxParameter && pxParameter->GetOverrideType() == ABOT_NONE )
            {
                ++m_iNumParameterStrings;
            }
        }

        pxBlueprint = bOnlyLocalParameters ? NULL : pxBlueprint->GetParent();
    }

	if ( m_iNumParameterStrings == 0 )
	{
		return;
	}

    m_ppszParameterStrings = new char* [ m_iNumParameterStrings ];
    Asura_Mem::Set( m_ppszParameterStrings, 0, m_iNumParameterStrings * sizeof( char* ) );

    // Now do the same again, filling in the strings.
    pxBlueprint = pxOriginalBlueprint;

    int iStringIndex = 0;
    while ( pxBlueprint )
    {
        const int iNumParameters = pxBlueprint->GetNumParameters();
        for ( int i = 0; i < iNumParameters; ++i )
        {
            Asura_Parameter* pxParameter = pxBlueprint->GetParameterByIndex( i );
            if ( pxParameter && pxParameter->GetOverrideType() == ABOT_NONE )
            {
                const int iStringLength = pxParameter->GetName().GetLength() + 1;
                m_ppszParameterStrings[ iStringIndex ] = new char [ iStringLength ];
                Asura_PrintfToBfr( m_ppszParameterStrings[ iStringIndex ], "%s", iStringLength, true, pxParameter->GetName().GetString() );
                ++iStringIndex;
            }
        }

        pxBlueprint = bOnlyLocalParameters ? NULL : pxBlueprint->GetParent();
    }

    // Sort the strings in the array.
	Asura_SortAndSearch::Quicksort( m_ppszParameterStrings, m_iNumParameterStrings, Asura_SortAndSearch::SortStringsCaseInsensitive );

    // Finally, create the bool array.
    m_pbParameterTickboxIsChecked = new bool [ m_iNumParameterStrings ];
    Asura_Mem::Set( m_pbParameterTickboxIsChecked, 0, m_iNumParameterStrings * sizeof( bool ) );
}

//////////////////////////////////////////////////////////////////////////////
//
// Tools_Blueprint_Edit_Object
//
//////////////////////////////////////////////////////////////////////////////

Asura_Parameter* Tools_Blueprint_Edit_Object::s_pxParameterBuffer    = NULL;
u_int            Tools_Blueprint_Edit_Object::s_uParameterBufferSize = 0;
u_int            Tools_Blueprint_Edit_Object::s_uNumInBuffer         = 0;
Asura_Hash_ID    Tools_Blueprint_Edit_Object::s_uParameterSourceType = ASURA_HASH_ID_UNSET;
Asura_Hash_ID    Tools_Blueprint_Edit_Object::s_uParameterSourceBP   = ASURA_HASH_ID_UNSET;
Asura_NameList   Tools_Blueprint_Edit_Object::s_xParameterPasteConflictList(false);

const char* Tools_Blueprint_Edit_Object::s_aszAllColumnHeadingStrings[ CV_NUM_COLUMN_VALUES ] =
{
	"Parameter",
    "Group",
    "Type",
    "Inhtd Value",
    "Ovrd Type",
    "Ovrd Value",
    "Final Value"
};

Asura_Vector_2 Tools_Blueprint_Edit_Object::s_xBlueprintOverlayPosition( 5.0f, 5.0f );
Asura_List< Tools_Blueprint_Edit_Object > Tools_Blueprint_Edit_Object::s_xEditObjectList;
bool (*Tools_Blueprint_Edit_Object::s_pfnProjectHandleTemplatedTypeLoad)(const Asura_Parameter* pxParameter) = NULL;
bool (*Tools_Blueprint_Edit_Object::s_pfnProjectExportTemplatedType)(List<Asura_String>& xNameList,const Asura_Parameter* pxParameter,const Asura_Blueprint* pxBlueprint) = NULL;

Asura_TemplateHashTree< Asura_Blueprint_Template > Tools_Blueprint_Edit_Object::s_xTemplatedTypeTree;

struct NameAndType
{
	const char* szName;
	u_int uType;
};

// please make sure the newest name for the templated type comes before any old ones in the list
static const NameAndType g_axTemplatedTypeNames[] =
{
	{ "Blueprint Type",			Tools_Blueprint_Edit_Object::TVT_BLUEPRINT_TYPE },
	{ "Shape",					Tools_Blueprint_Edit_Object::TVT_SHAPE },
	{ "Colour",					Tools_Blueprint_Edit_Object::TVT_COLOUR },
	{ "Skin/Anim",				Tools_Blueprint_Edit_Object::TVT_SKIN_OR_ANIM },
	{ "Skin",					Tools_Blueprint_Edit_Object::TVT_SKIN_OR_ANIM }, // old name for 'Skin/Anim' type
	{ "PFX",					Tools_Blueprint_Edit_Object::TVT_PFX },
	{ "Sound Event",			Tools_Blueprint_Edit_Object::TVT_SOUNDEVENT },
#ifdef ASURA_USE_DIALOGUE_SYSTEM
	{ "Dialogue Event",			Tools_Blueprint_Edit_Object::TVT_DIALOGUEEVENT },
#endif
	{ "Texture",				Tools_Blueprint_Edit_Object::TVT_TEXTURE },
	{ "Texture Tile",			Tools_Blueprint_Edit_Object::TVT_TEXTURETILE },
	{ "Localised Text",			Tools_Blueprint_Edit_Object::TVT_LOCALISED_TEXT },
	{ "Blueprint",				Tools_Blueprint_Edit_Object::TVT_BLUEPRINT },
	{ "Blueprint and children",	Tools_Blueprint_Edit_Object::TVT_BLUEPRINTANDCHILDREN },
#ifdef ASURA_USE_FSFX
	{ "FSFX",					Tools_Blueprint_Edit_Object::TVT_FSFX },
#endif
	{ "Hashed Localised Text",	Tools_Blueprint_Edit_Object::TVT_HASHED_TEXT },
#ifdef ASURA_USE_DECAL_MANAGER
	{ "Decal",					Tools_Blueprint_Edit_Object::TVT_DECAL },
#endif
	{ "Ragdoll Description",	Tools_Blueprint_Edit_Object::TVT_RAGDOLL },
	{ "Export Platforms",		Tools_Blueprint_Edit_Object::TVT_EXPORT_PLATFORMS },
#ifndef ASURA_DISABLE_ANIM_TABLE_CODE
	{ "Animtable",				Tools_Blueprint_Edit_Object::TVT_ANIMTABLE },
#endif
#ifdef ASURA_USE_GUI_MENU_SYSTEM
	{ "Menu Page",				Tools_Blueprint_Edit_Object::TVT_MENUPAGE },
#endif
	{ "Marker Point",			Tools_Blueprint_Edit_Object::TVT_MARKERPOINT },
#if defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)
	{ "Face",					Tools_Blueprint_Edit_Object::TVT_FACE },
	{ "Face Animation",			Tools_Blueprint_Edit_Object::TVT_FACEANIMATION },
#endif	//defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)
#if defined(ASURA_USE_CONTAINER_SYSTEM)
	{ "Container",				Tools_Blueprint_Edit_Object::TVT_CONTAINER },
#endif	//defined(ASURA_USE_CONTAINER_SYSTEM)
	{ "Animation Flag",			Tools_Blueprint_Edit_Object::TVT_ANIMATIONFLAG },
#ifdef ASURA_SUPPORT_SKIN_ELEMENTS
	{ "Skin Element",			Tools_Blueprint_Edit_Object::TVT_SKINELEMENT },
#endif

	{ NULL,						Tools_Blueprint_Edit_Object::NUM_TEMPLATED_TYPES } // terminator
};

Tools_Blueprint_Edit_Object::Tools_Blueprint_Edit_Object()
: m_pxTableDialogue(NULL)
, m_uBlueprintTypeHashID( ASURA_HASH_ID_UNSET )
, m_uBlueprintHashID( ASURA_HASH_ID_UNSET )
, m_xBlueprintOverlayPosition( s_xBlueprintOverlayPosition )
, m_bChanged( false )
, m_bShowFullStrings( Tools_Mode_Blueprint::ShouldShowFullStrings() )
, m_bShowEditButtons( Tools_Mode_Blueprint::ShouldShowEditButtons() )
, m_bShowFilterButtons( Tools_Mode_Blueprint::ShouldShowFilterButtons() )
, m_uNumParameters( 0 )
, m_ppszParameterStrings( NULL )
, m_iFilterChoice( 0 )
, m_xFilterGroupName( Tools_Mode_Blueprint::GetFilterGroupName() )
, m_iCurrentNumColumns( 0 )
{
    for ( int i = 0; i < CV_NUM_COLUMN_VALUES; ++i )
    {
        m_abColumnVisible[ i ] = Tools_Mode_Blueprint::ColumnIsVisible( i );
    }
    CalculateCurrentColumnData( m_abColumnVisible, m_iCurrentNumColumns, m_aeCurrentColumnType, m_aszCurrentColumnHeadingStrings );

    m_iFilterChoice = Asura_Parameter::GetGroupListIndex( m_xFilterGroupName );

    // Put this overlay in a handy position.
    Asura_Vector_2 xOverlayPositionMax( -Asura_Maths::LargeFloat, -Asura_Maths::LargeFloat );
    for ( Asura_List_It< Tools_Blueprint_Edit_Object > xIt( &s_xEditObjectList ); !xIt.Done(); xIt.Next() )
    {
        Asura_Overlay* pxOverlay = Asura_Overlay_System::GetOverlay( reinterpret_cast< int >( xIt.GetCurrent() ) );
        if ( !pxOverlay )
        {
            continue;
        }
        if ( pxOverlay->m_xPosition.x > xOverlayPositionMax.x )
        {
            xOverlayPositionMax.x = pxOverlay->m_xPosition.x;
        }
        if ( pxOverlay->m_xPosition.y > xOverlayPositionMax.y )
        {
            xOverlayPositionMax.y = pxOverlay->m_xPosition.y;
        }
    }
    // If we've updated the max position then work out where this overlay should go.
    if ( xOverlayPositionMax.x != -Asura_Maths::LargeFloat )
    {
        m_xBlueprintOverlayPosition.x = xOverlayPositionMax.x + 5.0f;
        m_xBlueprintOverlayPosition.y = xOverlayPositionMax.y + 5.0f;
    }

    s_xEditObjectList.AddToList( this );
	s_xTemplatedTypeTree.Reset();
}

Tools_Blueprint_Edit_Object::~Tools_Blueprint_Edit_Object()
{
    DestroyOverlay();
    DestroyParameterLists();
    s_xEditObjectList.RemoveFromList( this );
}

void Tools_Blueprint_Edit_Object::Update()
{
    if ( m_bChanged )
    {
        m_bChanged = false;
		Tools_BEO_Display_Data xDisplayData;
        RecordOverlayState( xDisplayData );
        DestroyOverlay();
        RestoreOverlayState( xDisplayData );
    }
}

void Tools_Blueprint_Edit_Object::RecordOverlayState( Tools_BEO_Display_Data& xData ) const
{
	Asura_Overlay* pxOverlay = Asura_Overlay_System::GetOverlay( reinterpret_cast< unsigned int >( this ) );
	if ( pxOverlay )
	{
		// We gave the table dialogue a control ID of 'this' also.
		Asura_Table_Dialog* pxTableDialogue = static_cast< Asura_Table_Dialog* >( pxOverlay->GetDialog( reinterpret_cast< int >( this ) ) );
		if ( pxTableDialogue )
		{
			xData.m_iOverlayIndex = Asura_Overlay_System::GetOverlayIndex( pxOverlay->m_uID );
			xData.m_uBlueprintTypeHashID = GetBlueprintTypeHashID();
			xData.m_uBlueprintHashID = GetBlueprintHashID();
			xData.m_xOverlayPosition = m_xBlueprintOverlayPosition;
			xData.m_xOverlayScrollPosition = pxTableDialogue->GetScrollBarValue();
			pxTableDialogue->GetSortParameters( xData.m_uSortIndex, xData.m_bSortBackwards );
		}
	}
}

void Tools_Blueprint_Edit_Object::RestoreOverlayState( const Tools_BEO_Display_Data& xData )
{
	DestroyOverlay();

	SetBlueprintTypeHashID( xData.m_uBlueprintTypeHashID );
	SetBlueprintHashID( xData.m_uBlueprintHashID );
	m_xBlueprintOverlayPosition = xData.m_xOverlayPosition;

	CreateOverlay();

	Asura_Overlay* pxOverlay = Asura_Overlay_System::GetOverlay( reinterpret_cast< unsigned int >( this ) );
	if ( pxOverlay )
	{
		// We gave the table dialogue a control ID of 'this' also.
		Asura_Table_Dialog* pxTableDialogue = static_cast< Asura_Table_Dialog* >( pxOverlay->GetDialog( reinterpret_cast< int >( this ) ) );
		if ( pxTableDialogue )
		{
			pxTableDialogue->SetScrollBarValue( xData.m_xOverlayScrollPosition );
			pxTableDialogue->SortRowsUsingColumn( xData.m_uSortIndex, xData.m_bSortBackwards );
			pxTableDialogue->UpdateAll();
		}
	}

	// Update the filter group choice, in case the group names have changed.
	m_iFilterChoice = Asura_Parameter::GetGroupListIndex( m_xFilterGroupName );
	if ( m_iFilterChoice == -1 )
	{
		m_iFilterChoice = 0;
		Asura_Parameter::GetGroupName( m_iFilterChoice, m_xFilterGroupName );
	}
}

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

void Tools_Blueprint_Edit_Object::CreateParameterLists( Asura_Blueprint* pxBlueprint )
{
    const bool bFilterByGroup = !m_xFilterGroupName.IsBlank();

    m_uNumParameters = 0;
    Asura_Blueprint* pxLocalBlueprint = pxBlueprint;
    while ( pxLocalBlueprint )
    {
        for ( int i = 0; i < pxLocalBlueprint->GetNumParameters(); ++i )
        {
            Asura_Parameter* pxParameter = pxLocalBlueprint->GetParameterByIndex( i );
            if ( pxParameter && pxParameter->GetOverrideType() == ABOT_NONE &&
                ( !bFilterByGroup || pxParameter->GetGroupName().Compare( m_xFilterGroupName ) == 0 ) )
            {
                ++m_uNumParameters;
            }
        }

        pxLocalBlueprint = pxLocalBlueprint->GetParent();
    }
    if ( m_uNumParameters )
    {
        m_ppszParameterStrings = new char* [ m_uNumParameters ];
        for ( u_int u = 0; u < m_uNumParameters; ++u )
        {
            m_ppszParameterStrings[ u ] = NULL;
        }

        int iWriteIndex = 0;
        Asura_Blueprint* pxLocalBlueprint = pxBlueprint;
        while ( pxLocalBlueprint )
        {
            for ( int i = 0; i < pxLocalBlueprint->GetNumParameters(); ++i )
            {
                Asura_Parameter* pxParameter = pxLocalBlueprint->GetParameterByIndex( i );
                Asura_Value xValue;
                if ( pxParameter && pxParameter->GetOverrideType() == ABOT_NONE && 
                    ( !bFilterByGroup || pxParameter->GetGroupName().Compare( m_xFilterGroupName ) == 0 ) &&
                    pxBlueprint->GetParameterValueByHashID( pxParameter->GetHashID(), xValue ) )
                {
                    if ( pxParameter->GetName().GetString() )
                    {
                        m_ppszParameterStrings[ iWriteIndex ] = new char[ pxParameter->GetName().GetLength() + 1 ];
                        strcpy( m_ppszParameterStrings[ iWriteIndex ], pxParameter->GetName().GetString() );
                        ++iWriteIndex;
                    }
                }
            }

            pxLocalBlueprint = pxLocalBlueprint->GetParent();
        }
    }
}

void Tools_Blueprint_Edit_Object::GetValueString( const Asura_Value& xValue, const int iTemplatedType, Asura_String& xString )
{
	if ( iTemplatedType != -1 )
	{
		switch ( iTemplatedType )
		{
			case TVT_COLOUR:
			{
				char szHexString[32];
				Asura_PrintfToBfr( szHexString, "%X", 32, true, static_cast<u_int> (xValue.GetIntValue()) );
				xString = szHexString;
                break;
            } 

			case TVT_SKIN_OR_ANIM:
			{			
				if ( xValue.GetHashIDValue() == ASURA_HASH_ID_UNSET )
				{
					xString = "*Unset*";
				}
				else
				{
					Asura_String xSearchString;	
					Asura_String xExt;

					xString = xValue.GetHashStringValue();
					xString.SplitFilename(NULL, &xSearchString, &xExt);
					xSearchString += ".asr";		
					
					// Previously, the hashstring value was the same as the hash ID (it didn't contain an extension)
					// The only thing to do in this case is search for the file using the basic filename
					if ( xExt.IsBlank() )
					{					
						List<Asura_String> xOutputList;
						Tools_FileSystem::FindFiles( xSearchString.GetString(), xOutputList, (TOOLS_FILEFIND_RECURSIVE | TOOLS_FILEFIND_IGNOREBACKUPDIR) );
						
						// The file search may return multiple filenames if the file in question has platform-specific versions 
						// in Asura_Files. The code below creates a new list of filenames making sure there aren't any spurious
						// duplicates caused by the platform filenames.
						Asura_NameList xNameList;
						for( int i=0; i<xOutputList.size(); ++i )
						{
							const Asura_String& xEntry = xOutputList.get_entry(i);
							char* szTempBuffer = new char[xEntry.GetLength() + 1]; 
							Tools_Platform::RemovePlatformFromPath( xEntry.GetString(), szTempBuffer );

							Asura_String xTempString = szTempBuffer;
							if( xNameList.GetIndexOfName( xTempString ) == -1 )
							{
								xNameList.AddName( xTempString );
							}

							delete [] szTempBuffer;
						}						

						// Armed with the stripped list, find the relevant filename
						if( xNameList.GetNumUsedEntries() == 0 )
						{
							Asura_Assertf( false,
								( "Failed to find file that matches skin/anim name '%s'.", xSearchString.GetString() ) );
						}
						else if( xNameList.GetNumUsedEntries() > 1 )
						{
							Asura_Assertf( false,
								( "Found multiple files that match skin/anim name '%s'. Using first file found.", xSearchString.GetString() ) );
							xString = xNameList.GetName(0);
						}
						else
						{
							xString = xNameList.GetName(0);
						}		

						if ( !xString.IsBlank() )
						{
                            RemoveLeadingJunk( xString );
							
							// RDM - 29/02/08 - Currently the file search runs rather slowly and happens whenever this is
							// called. This would continue to happen until the value is re-set by an artist or designer but
							// it's very hard for them to know which ones to do. Try re-setting the string automatically here, 
							// using a nasty const cast.
							Asura_Value& xNonConstValue = const_cast<Asura_Value&> (xValue);
							xNonConstValue.SetValue( xNonConstValue.GetHashIDValue(), xString.GetString() );
                        }
                        else
                        {
                            xString = "*Null String*";
                        }						
					}
				}

				break;
			}

            case TVT_PFX:
            {
                if ( xValue.GetHashIDValue() == ASURA_HASH_ID_UNSET )
                {
                    xString = "*Unset*";
                }
                else
                {
                    xString = xValue.GetHashStringValue();
                    if ( xString.IsBlank() )
                    {
                        xString = "*Null String*";
                    }
                }

                break;
            }

#ifdef ASURA_USE_DECAL_MANAGER
			case TVT_DECAL:
			{
				if ( xValue.GetHashIDValue() == ASURA_HASH_ID_UNSET )
				{
					xString = "*Unset*";
				}
				else
				{
					xString = xValue.GetHashStringValue();
					if ( xString.IsBlank() )
					{
						xString = "*Null String*";
					}
				}

				break;
			}
#endif

			case TVT_RAGDOLL:
			{
#ifdef ASURA_PHYSICS_DISABLE_RAGDOLL
				xString = "*Unset*";
#else
				const Asura_Hash_ID xRagdollID = xValue.GetHashIDValue();
				if ( xRagdollID == ASURA_HASH_ID_UNSET )
				{
					xString = "*Unset*";
				}
				else
				{
					const Asura_Physics_Ragdoll_Description* const pxRagdoll = Asura_Physics_Ragdoll_System::GetRagdollDescription( xRagdollID );
					if ( pxRagdoll )
					{
						xString = pxRagdoll->GetName();
					}
					else
					{
						xString = "*Error - cannot find ragdoll*";
					}
				}
#endif

				break;
			}

			case TVT_SOUNDEVENT:
			{
				const Asura_Hash_ID uEventID = xValue.GetHashIDValue();
				if ( uEventID == ASURA_HASH_ID_UNSET )
				{
					xString = "*Unset*";
				}
				else
				{
					const Asura_Sound_Event* pxEvent = Asura_Sound_Event_Storage::FindEvent( uEventID );
					if ( pxEvent )
					{
                        xString = pxEvent->GetName();
					}
					else
					{
						xString = "*Error - cannot find event*";
					}
				}

				break;
			}

#ifdef ASURA_USE_DIALOGUE_SYSTEM
			case TVT_DIALOGUEEVENT:
			{
				const Asura_Hash_ID uEventID = xValue.GetHashIDValue();
				if ( uEventID == ASURA_HASH_ID_UNSET )
				{
					xString = "*Unset*";
				}
				else
				{
					const Asura_Dialogue_Event* pxEvent = Asura_Dialogue_System::FindEvent( uEventID );
					if ( pxEvent )
					{
						xString = pxEvent->GetName();
					}
					else
					{
						xString = "*Error - cannot find event*";
					}
				}

				break;
			}
#endif

			case TVT_TEXTURE:
			{			
				if ( xValue.GetHashIDValue() == ASURA_HASH_ID_UNSET )
				{
					xString = "*Unset*";
				}
				else
				{
					// Texture hashes use the string Path\Name (i.e. full filename without extension)
					xString = xValue.GetHashStringValue();	
					Asura_String xSearchString;
					Asura_String xExt;
					xString.SplitFilename(NULL, &xSearchString, &xExt);

					// Previously, the hashstring value was the same as the hash ID (it didn't contain an extension)
					// The only thing to do in this case is search for the file using the basic filename
					if ( xExt.IsBlank() )
					{					
						// Multiple extensions to try
						xSearchString += ".tga";

						List<Asura_String> xOutputList;
						Tools_FileSystem::FindFiles( xSearchString.GetString(), xOutputList, (TOOLS_FILEFIND_RECURSIVE | TOOLS_FILEFIND_IGNOREBACKUPDIR) );

						if (xOutputList.size() == 0)
						{
							xSearchString.SplitFilename(NULL, &xSearchString, NULL);
							xSearchString += ".bmp";
							xOutputList.empty();
							Tools_FileSystem::FindFiles( xSearchString.GetString(), xOutputList, (TOOLS_FILEFIND_RECURSIVE | TOOLS_FILEFIND_IGNOREBACKUPDIR) );

							if (xOutputList.size() == 0)
							{
								xSearchString.SplitFilename(NULL, &xSearchString, NULL);
								xSearchString += ".dds";
								xOutputList.empty();
								Tools_FileSystem::FindFiles( xSearchString.GetString(), xOutputList, (TOOLS_FILEFIND_RECURSIVE | TOOLS_FILEFIND_IGNOREBACKUPDIR) );
							}
						}

						if( xOutputList.size() == 0 )
						{
							Asura_Assert( false, "Failed to find file that matches this texture name." );										
						}
						else if( xOutputList.size() > 1 )
						{
							Asura_Assert( false, "Found multiple files that match this texture name. Using first file found." );					
							xString = xOutputList.get_entry(0);
						}
						else
						{
							xString = xOutputList.get_entry(0);
						}		

                        if ( !xString.IsBlank() )
                        {
                            RemoveLeadingJunk( xString );
                        }
                        else
                        {
                            xString = "*Null String*";
                        }
					}
				}

				break;
			}

			case TVT_TEXTURETILE:
			{			
				if ( xValue.GetHashIDValue() == ASURA_HASH_ID_UNSET )
				{
					xString = "*Unset*";
				}
				else
				{
					// Texture tile hashes use the string Name.Ext (i.e. no path)
					xString = xValue.GetHashStringValue();	

					List<Asura_String> xOutputList;
					Tools_FileSystem::FindFiles( xString.GetString(), xOutputList, (TOOLS_FILEFIND_RECURSIVE | TOOLS_FILEFIND_IGNOREBACKUPDIR) );

					if( xOutputList.size() == 0 )
					{
						Asura_Assertf( false, ("Failed to find file that matches \"%s\" tiled-texture name.", xString.GetString()) );
					}
					else if( xOutputList.size() > 1 )
					{
						Asura_Assertf( false, ("Found multiple files that match \"%s\" tiled-texture name. Using first file found.", xString.GetString()) );					
						xString = xOutputList.get_entry(0);
					}
					else
					{
						xString = xOutputList.get_entry(0);
					}		

                    if ( !xString.IsBlank() )
                    {
                        RemoveLeadingJunk( xString );
                    }
                    else
                    {
                        xString = "*Null String*";
                    }
				}

				break;
			}

			case TVT_LOCALISED_TEXT:
			{
                const Asura_UnicodeChar* pwszTextString = Asura_LocalisedText::GetString( xValue.GetIntValue() );
                if ( pwszTextString )
                {
					char* pszTextString = NULL;
                    const size_t uStringLength = wcslen( pwszTextString );
                    pszTextString = new char[ uStringLength + 1 ];
                    if ( pszTextString )
                    {
                        Asura_UnicodeToUTF8( pwszTextString, pszTextString );
                    }
					xString = pszTextString;
					delete [] pszTextString;
                }
				else
				{
					xString = "*Error - cannot find text*";
				}
                break;
			}

			case TVT_SHAPE:
			{
				if ( xValue.GetHashIDValue() == ASURA_HASH_ID_UNSET )
				{
					xString = "*Unset*";
				}
				else
				{
					xString = xValue.GetHashStringValue();	
					if ( xString.GetString() != NULL )
					{
                        RemoveLeadingJunk( xString );
					}
					else
					{
						xString = "*Null String*";
					}
				}

                break;
			}

			case TVT_BLUEPRINT:
			case TVT_BLUEPRINTANDCHILDREN:
			case TVT_BLUEPRINT_TYPE:
			{
				if ( xValue.GetHashIDValue() == ASURA_HASH_ID_UNSET )
				{
					xString = "*Unset*";
				}
				else
				{
					xString = xValue.GetHashStringValue();
					if ( xString.IsBlank() )
					{
						xString = "*Null String*";
					}
				}

				break;			
			}

#ifdef ASURA_USE_FSFX
			case TVT_FSFX:
			{
				if ( xValue.GetHashIDValue() == ASURA_HASH_ID_UNSET )
				{
					xString = "*Unset*";
				}
				else
				{
					xString = xValue.GetHashStringValue();
					if ( xString.IsBlank() )
					{
						xString = "*Null String*";
					}
				}

				break;
			}
#endif
			case TVT_HASHED_TEXT:
			{
				if (xValue.GetHashIDValue() == ASURA_HASH_ID_UNSET)
				{
					xString = "*Unset*";
				} 
				else 
				{
					const char* pszTextString = Asura_HashedLocalisedText::GetTextIDStringFromAnyPage( xValue.GetHashIDValue() );
					if (!pszTextString)
					{
						xString = "*Not Found*";
					} 
					else
					{
						xString = pszTextString;
					}
				}
				break;
			}

#ifndef ASURA_DISABLE_ANIM_TABLE_CODE
			case TVT_ANIMTABLE:
			{
				xString = xValue.GetHashStringValue();
				if ( !xString.IsBlank() )
				{
					RemoveLeadingJunk( xString );
				}
				else
				{
					xString = "*Unset*";
				}
				break;
			}
#endif

			case TVT_EXPORT_PLATFORMS:
			{
				xString.MakeBlank();
				
				// compiles a list of the platforms selected, e.g. "PC, PSP, Xbox", etc.
				// if all possible platforms are selected 'All' is displayed.
				bool bAllPlatforms = true;
				for( int iPlatform=0; iPlatform<Tools_Level_Export::ExportType_NumPlatforms; ++iPlatform )
				{
					if( Tools_Level_Export::IsPlatformSupported( (Tools_Level_Export::EXPORT_TYPE)iPlatform ) )
					{
						if( xValue.GetIntValue() & (1 << iPlatform) )
						{
							if( !xString.IsBlank() )
							{
								xString += ", ";
							}

							xString += Tools_Level_Export::GetPlatformName( iPlatform );
						}
						else 
						{
							bAllPlatforms = false;
						}
					}
				}

				if( bAllPlatforms )
				{
					xString = "All";
				}

				break;
			}

#ifdef ASURA_USE_GUI_MENU_SYSTEM
			case TVT_MENUPAGE:
			{
				if (xValue.GetHashIDValue() == ASURA_HASH_ID_UNSET)
				{
					xString = "*Unset*";
				} 
				else 
				{
					xString = xValue.GetHashStringValue();
					if (xString.IsBlank())
					{
						xString = "*Null String*";
					}
				}

				break;
			}
#endif

			case TVT_MARKERPOINT:
			{
				if (xValue.GetHashIDValue() == ASURA_HASH_ID_UNSET)
				{
					xString = "*Unset*";
				}
				else
				{
					xString = xValue.GetHashStringValue();
					if (xString.IsBlank())
					{
						xString = "*Null String*";
					}
				}

				break;
			}

#ifdef ASURA_USE_FACIAL_ANIMATION_SYSTEM
			case TVT_FACE:
			{
				if(xValue.GetHashIDValue() == ASURA_HASH_ID_UNSET)
				{
					xString	= "*Unset*";
				}
				else
				{
					const char*							szFaceName	= xValue.GetHashStringValue();
					const Tools_FacialAnimation_Face*	pxFace		= Tools_FacialAnimation_System::FindOrLoadFace(szFaceName);
					xString	= (pxFace == NULL) ? "*Not Found*" : pxFace->GetName();
				}
				break;
			}

			case TVT_FACEANIMATION:
			{
				if(xValue.GetHashIDValue() == ASURA_HASH_ID_UNSET)
				{
					xString	= "*Unset*";
				}
				else
				{
					const char*									szFaceAnimationName	= xValue.GetHashStringValue();
					const Tools_FacialAnimation_FaceAnimation*	pxFaceAnimation		= Tools_FacialAnimation_System::FindOrLoadAnimation(szFaceAnimationName);

					xString	= (pxFaceAnimation == NULL) ? "*Not Found*" : pxFaceAnimation->GetName();
				}
				break;
			}
#endif

#if defined(ASURA_USE_CONTAINER_SYSTEM)
			case TVT_CONTAINER:
			{
				if(xValue.GetHashIDValue() == ASURA_HASH_ID_UNSET)
				{
					xString	= "*Unset*";
				}
				else
				{
					xString	= xValue.GetHashStringValue();
				}
				break;
			}
#endif	//defined(ASURA_USE_CONTAINER_SYSTEM)

			case TVT_ANIMATIONFLAG:
			{
				int iValue = xValue.GetIntValue();
				if (iValue == 0)
				{
					xString = "*None*";
				}
				else
				{
					Tools_Flag_List* pxFlagList = Tools_System::CommonFlagIni ? Tools_System::CommonFlagIni->GetFlagList("Keyframe Flags") : NULL;
					if (pxFlagList)
					{
						xString = pxFlagList->GetFlagName(iValue);
					}
					else
					{
						xString = "<Unknown Flag>";
					}
				}
				break;
			}

#ifdef ASURA_SUPPORT_SKIN_ELEMENTS
			case TVT_SKINELEMENT:
			{
				int iValue = xValue.GetIntValue();
				if (iValue < 0)
				{
					xString = "*None*";
				}
				else
				{
					int iNumElementNames = Asura_Project_Skin_Elements::GetNumElementNames();
					if (iValue >= iNumElementNames)
					{
						xString = "<Unknown>";
					}
					else
					{
						const char** szElementNames = Asura_Project_Skin_Elements::GetElementNames();
						xString = szElementNames[iValue];
					}
				}
				break;
			}
#endif //#ifdef ASURA_SUPPORT_SKIN_ELEMENTS

			default: break;
		}
	}
    else
    {
	    switch ( xValue.GetValueType() )
	    {
		    default:
		    case Asura_Value::VT_INT:
		    {
                char szString[ 32 ];
                Asura_PrintfToBfr( szString, "%d", 32, true, xValue.GetIntValue() );
                xString = szString;
                break;
		    }

		    case Asura_Value::VT_FLOAT:
		    {
                char szString[ 32 ];
                Asura_PrintfToBfr( szString, "%.3f", 32, true, xValue.GetFloatValue() );
                xString = szString;
                break;
		    }

		    case Asura_Value::VT_BOOL:
		    {
                xString = xValue.GetBoolValue() ? "true" : "false";
                break;
		    }

            case Asura_Value::VT_HASH:
		    {
                xString = xValue.GetHashStringValue();
                if ( !xString.IsBlank() )
                {
                    RemoveLeadingJunk( xString );
                }
                else
                {
                    xString = "*Null String*";
                }
                break;
		    }
	    }
    }
}

bool Tools_Blueprint_Edit_Object::CollapseString( Asura_String& xString )
{
	static const int iStringStripLength = 17;

	// If the string is a path (contains slashes) then collapse it down to the final name.
	int iPositionInString = -1;
	if ( xString.Contains( "\\", &iPositionInString ) || xString.Contains( "/", &iPositionInString ) )
	{
		const Asura_String xOldString( xString );
		Asura_String xExtension;
		xOldString.SplitFilename( NULL, &xString, &xExtension );
		xString += "." + xExtension;
		return true;
	}
	else if ( xString.GetLength() > iStringStripLength )
	{
		for ( int iWriteIndex = 0, iReadIndex = xString.GetLength() - iStringStripLength; iReadIndex < xString.GetLength(); ++iWriteIndex, ++iReadIndex )
		{
			xString[ iWriteIndex ] = xString[ iReadIndex ];
		}

		xString.Truncate( iStringStripLength );
		xString[ 0 ] = xString[ 1 ] = xString[ 2 ] = '.';
		return true;
	}
    else
    {
        return false;
    }
}

void Tools_Blueprint_Edit_Object::RemoveLeadingJunk( Asura_String& xString )
{
    // Remove any junk from the front of the string.
    int iPosition = 0;
    while ( (xString[iPosition] == '.') || (xString[iPosition] == '/') || (xString[iPosition] == '\\') )
    {
        iPosition++;
    }
    if ( iPosition < xString.GetLength() )
    {
        Asura_String xOldFilename = xString;
        xString = &xOldFilename[ iPosition ];
    }
}

void Tools_Blueprint_Edit_Object::DestroyParameterLists()
{
    for ( u_int u = 0; u < m_uNumParameters; ++u )
    {
        delete [] m_ppszParameterStrings[ u ];
    }
    delete [] m_ppszParameterStrings;
    m_ppszParameterStrings = NULL;
    m_uNumParameters       = 0;
}

void Tools_Blueprint_Edit_Object::CreateOverlay()
{
	// add and intialise any templated types 
	if( s_xTemplatedTypeTree.GetNumberOfNodes() == 0 )
	{
		SetupTemplatedTypes();
	}

	Asura_Blueprint* pxBlueprint = Asura_Blueprint_System::GetBlueprintByHashID( m_uBlueprintTypeHashID, m_uBlueprintHashID );
	if ( !pxBlueprint ) { return; }

	// run through the parameters in the blueprint and make sure the hash IDs for any templated types are up to date
	for( int i=0; i<pxBlueprint->GetNumParameters(); ++i )
	{
		Asura_Parameter* pxParameter = pxBlueprint->GetParameterByIndex(i);
		if( pxParameter )
		{
			int iTemplatedType = GetTemplatedTypeFromTemplateID( pxParameter->GetTemplate() );	
			if( iTemplatedType != -1 )
			{
				pxParameter->SetTemplate( Asura_GetHashID(GetTemplatedTypeName(iTemplatedType)) );
			}
		}
	}

	Asura_Overlay* pxNewOverlay = new Asura_Overlay( "Blueprint Overlay", false, &m_xBlueprintOverlayPosition );
	if ( !pxNewOverlay ) { return; }

	Asura_Vector_2 xOffset = Asura_Vector_2( 1.0f, 3.0f );
	pxNewOverlay->m_uID = reinterpret_cast< unsigned int >( this );

	AddBlueprintOverlayItems( *pxNewOverlay, xOffset, pxBlueprint );

	Asura_Dialog* pxNewDialogue = new Asura_Function_Dialog( "Close", xOffset, CloseOverlay, pxNewOverlay->m_uID );
	pxNewDialogue->SetHelpText( "Close this dialogue." );
	pxNewOverlay->AddDialog( pxNewDialogue );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxNewOverlay->MakeDialogsRelativeToResizableDialog();

	pxNewOverlay->CalculateOverlaySize();


	// Add as much title as we can fit in.
	const Asura_Char szSpecialString[ 2 ] = { 13, 0 };
	const Asura_Char cSpecialChar = 13;

	Asura_String xOverlayTitle;
	xOverlayTitle = pxBlueprint->GetName();
	Asura_Blueprint* pxParentBlueprint = pxBlueprint->GetParent();
	while ( pxParentBlueprint )
	{
		xOverlayTitle.Prepend( szSpecialString, 1 );
		xOverlayTitle.Prepend( pxParentBlueprint->GetName().GetString(), pxParentBlueprint->GetName().GetLength() );
		pxParentBlueprint = pxParentBlueprint->GetParent();
	}
	const Asura_String* pxTypeName = Asura_Blueprint_System::GetTypeName( m_uBlueprintTypeHashID );
	if ( pxTypeName )
	{
		xOverlayTitle.Prepend( szSpecialString, 1 );
		xOverlayTitle.Prepend( pxTypeName->GetString(), pxTypeName->GetLength() );
	}
	// Make sure the string isn't too long.
	u_int uNumSections = Asura_StringUtil::CountInstancesOfCharacter( xOverlayTitle.GetString(), cSpecialChar ) + 1;
	float fStringWidth = Asura_Overlay_System::UseSkinnedUI() ?
		Asura_Text_Output::WidthOfString( xOverlayTitle.GetString(), ASURA_TEXT_FORMAT_SMALLFONT ) :
		( float )Asura_Text_Output::LengthOfString( xOverlayTitle.GetString() );
	while ( fStringWidth > pxNewOverlay->m_fWidth - 10.0f && uNumSections > 2 )
	{
		// Remove penultimate section.
		u_int uSectionStartIndex, uSectionLength;
		if ( Asura_StringUtil::FindStringPartByIndex( xOverlayTitle.GetString(), cSpecialChar, ( uNumSections - 2 ), uSectionStartIndex, uSectionLength ) )
		{
			xOverlayTitle.Replace( uSectionStartIndex, uSectionLength, " ... " );
			fStringWidth = Asura_Overlay_System::UseSkinnedUI() ?
				Asura_Text_Output::WidthOfString( xOverlayTitle.GetString(), ASURA_TEXT_FORMAT_SMALLFONT ) :
				( float )Asura_Text_Output::LengthOfString( xOverlayTitle.GetString() );
			--uNumSections;
		}
		else
		{
			break;
		}
	}
	// Replace our special section break characters with dots.
	xOverlayTitle.ReplaceAllChars( cSpecialChar, '.' );
	xOverlayTitle.Prepend( "Blueprint: ", strlen( "Blueprint: " ) );
	pxNewOverlay->SetTitle( xOverlayTitle.GetString() );

	Asura_Overlay_System::SubmitNewOverlay( pxNewOverlay, false );
}

void Tools_Blueprint_Edit_Object::DestroyOverlay()
{
    Asura_Overlay_System::CloseOverlays( reinterpret_cast< unsigned int >( this ) );
}

Asura_TableCell_Dialog* Tools_Blueprint_Edit_Object::CreateTableCell( Asura_Table_Dialog& xTableDialog,
																	  const Asura_Vector_2& xOffset,
                                                                      const Asura_Vector_2& xMinCellSize,
                                                                      const u_int uRow,
                                                                      const u_int uColumn,
                                                                      void* const pxUserData )
{
    Tools_Blueprint_Edit_Object*	pxThis			= reinterpret_cast< Tools_Blueprint_Edit_Object* >( pxUserData );
    Asura_TableCell_Dialog*			pxCellDialogue	= NULL;

    // Look up the cell type from the column index.
    switch ( pxThis->m_aeCurrentColumnType[ uColumn ] )
    {  
		case CV_PARAMETER_NAME:
		{
			const char*	szName = pxThis->m_ppszParameterStrings[uRow];

			pxCellDialogue	= Asura_TableRowHeading_StandardDialog::Create(xTableDialog, szName, xOffset, xMinCellSize, uRow);
			break;
		}

        case CV_PARAMETER_GROUP:
        {
            Tools_Blueprint_TableCell_ParameterGroup* pxTypeCell = new Tools_Blueprint_TableCell_ParameterGroup( pxThis, xOffset, xMinCellSize, uRow, uColumn );
            pxCellDialogue = pxTypeCell;
            break;
        }

        case CV_PARAMETER_TYPE:
        {
            Tools_Blueprint_TableCell_ParameterType* pxTypeCell = new Tools_Blueprint_TableCell_ParameterType( pxThis, xOffset, xMinCellSize, uRow, uColumn );
            pxCellDialogue = pxTypeCell;
            break;
        }

        case CV_INHERITED_VALUE:
        {
            Tools_Blueprint_TableCell_InheritedValue* pxValueCell = new Tools_Blueprint_TableCell_InheritedValue( pxThis, xOffset, xMinCellSize, uRow, uColumn );
            pxCellDialogue = pxValueCell;
            break;
        }

        case CV_OVERRIDE_TYPE:
        {
            Tools_Blueprint_TableCell_OverrideType* pxTypeCell = new Tools_Blueprint_TableCell_OverrideType( pxThis, xOffset, xMinCellSize, uRow, uColumn );
            pxCellDialogue = pxTypeCell;
            break;
        }

        case CV_OVERRIDE_VALUE:
        {
            Tools_Blueprint_TableCell_OverrideValue* pxValueCell = new Tools_Blueprint_TableCell_OverrideValue( pxThis, xOffset, xMinCellSize, uRow, uColumn );
            pxCellDialogue = pxValueCell;
            break;
        }

        case CV_FINAL_VALUE:
        {
            Tools_Blueprint_TableCell_FinalValue* pxValueCell = new Tools_Blueprint_TableCell_FinalValue( pxThis, xOffset, xMinCellSize, uRow, uColumn );
            pxCellDialogue = pxValueCell;
            break;
        }
    }

    return pxCellDialogue;
}

void Tools_Blueprint_Edit_Object::TableFilterChoiceChanged( int iUserData )
{
    Tools_Blueprint_Edit_Object* pxThis = reinterpret_cast< Tools_Blueprint_Edit_Object* >( iUserData );
    Asura_Parameter::GetGroupName( pxThis->m_iFilterChoice, pxThis->m_xFilterGroupName );
    Tools_Mode_Blueprint::RequestRefreshBlueprintEditObjectOverlays();
}

void Tools_Blueprint_Edit_Object::AddBlueprintOverlayItems( Asura_Overlay& xNewOverlay, Asura_Vector_2& xOffset, Asura_Blueprint* pxBlueprint )
{
	Asura_DropDown_List_Dialog*	pxDDLDialogue;
	Asura_Dialog*				pxNewDialogue;

    DestroyParameterLists();
    CreateParameterLists( pxBlueprint );
    CalculateCurrentColumnData( m_abColumnVisible, m_iCurrentNumColumns, m_aeCurrentColumnType, m_aszCurrentColumnHeadingStrings );

	// Other dialogues.
	pxNewDialogue = new Asura_Tickbox_Dialog( "Show Full Strings", xOffset, &m_bShowFullStrings, &m_bChanged );
	pxNewDialogue->SetHelpText( "Expands/collapses strings." );
	xNewOverlay.AddDialog( pxNewDialogue );

	xOffset.x += 15.0f;
	pxNewDialogue = new Asura_Tickbox_Dialog( "Show Edit Buttons", xOffset, &m_bShowEditButtons, &m_bChanged );
	pxNewDialogue->SetHelpText( "Shows/hides the overlay edit buttons." );
	xNewOverlay.AddDialog( pxNewDialogue );

	xOffset.x += 15.0f;
	pxNewDialogue = new Asura_Tickbox_Dialog( "Show Filter Buttons", xOffset, &m_bShowFilterButtons, &m_bChanged );
	pxNewDialogue->SetHelpText( "Shows/hides the overlay filter buttons." );
	xNewOverlay.AddDialog( pxNewDialogue );

	xOffset.x -= 15.0f * 2;
	xOffset.y += Asura_Dialog::LargeSpacing();

    // Table dialogue.
    const Asura_Vector_2 xTableOffset( xOffset.x, xOffset.y );
    static const Asura_Vector_2	s_xMinimumCellSize( 6.0f, 1.2f );
	m_pxTableDialogue =
		new Asura_Table_Dialog( xTableOffset, Tools_Mode_Blueprint::GetMaxTableSizeX(), Tools_Mode_Blueprint::GetMaxTableSizeY(), true );
    m_pxTableDialogue->SetControlID( reinterpret_cast< int >( this ) );
    Asura_Vector_2 xCellSpacing = m_pxTableDialogue->GetCellSpacing();
    xCellSpacing.y = 0.25f;
    m_pxTableDialogue->SetCellSpacing( xCellSpacing );

    xNewOverlay.AddDialog( m_pxTableDialogue );
    m_pxTableDialogue->CreateTable( s_xMinimumCellSize, m_uNumParameters, m_iCurrentNumColumns, CreateTableCell, this,
									NULL, m_aszCurrentColumnHeadingStrings, NULL, NULL, CreateTableColumnHeading, this);
	m_pxTableDialogue->SortRowsUsingColumn(0, false);
	m_pxTableDialogue->EnableRowSelection(false);
    xOffset.y = xTableOffset.y + m_pxTableDialogue->GetTableSize().y + Asura_Dialog::SmallSpacing();

	//add control for resizing the table
	pxNewDialogue = new Asura_Resize_Dialog(m_pxTableDialogue, Tools_Mode_Blueprint::GetMaxTableSizePtr());
	xNewOverlay.AddDialog( pxNewDialogue);


	if(m_bShowFilterButtons)
	{
		// Filter dialogues.
		pxNewDialogue = new Asura_Dialog( "Filter:", xOffset );
		xNewOverlay.AddDialog( pxNewDialogue );

		xOffset.x += 4.5f;
		pxDDLDialogue = new Asura_DropDown_List_Dialog( xOffset, Asura_Parameter::GetGroupNameList(), &m_iFilterChoice,
			6, Asura_Parameter::GetNumParameterGroups(), NULL, NULL, "" );
		pxDDLDialogue->SetOnSelectCallback( TableFilterChoiceChanged, reinterpret_cast< int >( this ) );
		xNewOverlay.AddDialog( pxDDLDialogue );

		xOffset.x  = pxDDLDialogue->m_xMaxCoord[0] + 3.0f;
		pxNewDialogue = new Asura_Function_Dialog( "Add Parameter Group Name", xOffset, Tools_Mode_Blueprint::AddParameterGroupName );
		xNewOverlay.AddDialog( pxNewDialogue );

		xOffset.x  = 1.0f;
		xOffset.y += Asura_Dialog::LargeSpacing();
		pxNewDialogue = new Asura_Dialog( "Columns:", xOffset );
		xNewOverlay.AddDialog( pxNewDialogue );

		xOffset.x += 6.0f;
		for ( u_int uColumn = 0; uColumn < Tools_Blueprint_Edit_Object::CV_NUM_COLUMN_VALUES; ++uColumn )
		{
			pxNewDialogue = new Asura_Tickbox_Dialog( "", xOffset, &m_abColumnVisible[ uColumn ], &m_bChanged );
			pxNewDialogue->SetHelpText(s_aszAllColumnHeadingStrings[uColumn]);
			pxNewDialogue->SetEnabled((uColumn != CV_PARAMETER_NAME) && (uColumn != CV_FINAL_VALUE));
			xNewOverlay.AddDialog( pxNewDialogue );
			xOffset.x += 3.0f;
		}
		xOffset.x = 1.0f;
		xOffset.y += Asura_Dialog::LargeSpacing();
	}

    if ( m_bShowEditButtons )
    {
        pxNewDialogue = new Asura_Function_Dialog( "Add Parameter", xOffset, AddParameterStatic, reinterpret_cast< int >( this ) );
        pxNewDialogue->SetHelpText( "Add a new parameter to this blueprint." );
        xNewOverlay.AddDialog( pxNewDialogue );
        xOffset.x += 15.0f;

        pxNewDialogue = new Asura_Function_Dialog( "Delete Parameter", xOffset, DeleteSelectedParameterStatic, reinterpret_cast< int >( this ) );
        pxNewDialogue->SetHelpText( "Delete a parameter from this blueprint." );
		pxNewDialogue->SetIsEnabledFunction(CanDeleteSelectedParameterStatic, reinterpret_cast<int>(this));
        xNewOverlay.AddDialog( pxNewDialogue );
        xOffset.x += 15.0f;

        pxNewDialogue = new Asura_Function_Dialog( "Rename Parameter", xOffset, RenameSelectedParameterStatic, reinterpret_cast< int >( this ) );
        pxNewDialogue->SetHelpText( "Renames a parameter in this blueprint hierarchy." );
		pxNewDialogue->SetIsEnabledFunction(CanRenameSelectedParameterStatic, reinterpret_cast<int>(this));
        xNewOverlay.AddDialog( pxNewDialogue );
        xOffset.x -= 30.0f;
        xOffset.y += Asura_Dialog::DefaultSpacing();

        pxNewDialogue = new Asura_Function_Dialog( "Copy Parameters", xOffset, CopyParametersStatic, reinterpret_cast< int >( this ) );
        pxNewDialogue->SetHelpText( "Copys a parameter/parameters from this blueprint." );
        xNewOverlay.AddDialog( pxNewDialogue );
        xOffset.x += 15.0f;

        pxNewDialogue = new Asura_Function_Dialog( "Cut Parameters", xOffset, CutParametersStatic, reinterpret_cast< int >( this ) );
        pxNewDialogue->SetHelpText( "Cuts a parameter/parameters from this blueprint." );
        xNewOverlay.AddDialog( pxNewDialogue );
        xOffset.x += 15.0f;

        pxNewDialogue = new Asura_Function_Dialog( "Paste Parameters", xOffset, PasteParametersStatic, reinterpret_cast< int >( this ), NULL, ParametersInPasteBuffer );
        pxNewDialogue->SetHelpText( "Pastes previously copied parameters into this blueprint, provided they do not already exist in the hierarchy." );
        xNewOverlay.AddDialog( pxNewDialogue );
        xOffset.x += 15.0f;

        pxNewDialogue = new Asura_Function_Dialog( "Paste Over Parameters", xOffset, PasteOverParametersStatic, reinterpret_cast< int >( this ), NULL, ParametersInPasteBuffer );
        pxNewDialogue->SetHelpText( "Pastes previously copied parameters into this blueprint, overwriting and merging where necessary." );
        xNewOverlay.AddDialog( pxNewDialogue );
        xOffset.x -= 45.0f;
        xOffset.y += Asura_Dialog::LargeSpacing();
    }
}

void Tools_Blueprint_Edit_Object::AddParameterStatic( int iUserData )
{
    Tools_Blueprint_Edit_Object* pxThis = reinterpret_cast< Tools_Blueprint_Edit_Object* >( iUserData );
    pxThis->AddParameter();
}

void Tools_Blueprint_Edit_Object::AddParameter()
{
    Asura_Blueprint* pxBlueprint = Asura_Blueprint_System::GetBlueprintByHashID( m_uBlueprintTypeHashID, m_uBlueprintHashID );
    if ( !pxBlueprint ) { return; }

    ListSelection_Overlay xTypeOverlay( "Pick the parameter type:" );
    AddTypesToTypeOverlay( xTypeOverlay );
    if ( !xTypeOverlay.GetNumItems() ) { return; }
	if ( !Tools_Mode_Blueprint::GetLastTypeChoice().IsBlank() )
	{
		xTypeOverlay.SetInitialSelection( Tools_Mode_Blueprint::GetLastTypeChoice() );
	}
    if ( !xTypeOverlay.DoModal() )     { return; }
	Tools_Mode_Blueprint::SetLastTypeChoice( xTypeOverlay.GetSelectionString() );

    // Check for a template with the same name as the type we've selected.
    Asura_Hash_ID uTemplateHashID = ASURA_HASH_ID_UNSET;
    if ( GetTemplate( Asura_GetHashID( xTypeOverlay.GetSelectionString() ) ) )
    {
        uTemplateHashID = Asura_GetHashID( xTypeOverlay.GetSelectionString() );
    }
    else
    {		   
        // Check for any templates to choose manually.
        ListSelection_Overlay xOverrideList( "Choose template:" );	
        const int iCount = Asura_Blueprint_System::GetTemplateList( xOverrideList, static_cast< Asura_Value::VALUE_TYPE >( xTypeOverlay.GetSelectionData() ) );
        if ( iCount )
        {			
            YesNo_Overlay xYesNo( "Apply template?" );
            if ( xYesNo.DoModal() )
            {				
                xOverrideList.DoModal();
                uTemplateHashID = static_cast< Asura_Hash_ID >( xOverrideList.GetSelectionData() );
            }
        }
    }

    // If there is a template, apply the initial value, otherwise just set it to a default.
	Asura_Value xValue;
	const Asura_Blueprint_Template* pxTemplate = GetTemplate( uTemplateHashID );
    const Asura_Value::VALUE_TYPE   eNewType   = pxTemplate ? pxTemplate->GetType() : static_cast< Asura_Value::VALUE_TYPE >( xTypeOverlay.GetSelectionData() );
	switch ( eNewType )
	{
		case Asura_Value::VT_FLOAT: xValue.SetValue( pxTemplate ? pxTemplate->GetInitialFloat() : 0.0f ); break;
		case Asura_Value::VT_INT:   xValue.SetValue( pxTemplate ? pxTemplate->GetInitialInt() : 0 ); break;
		case Asura_Value::VT_BOOL:  xValue.SetValue( false ); break;
		case Asura_Value::VT_HASH:  xValue.SetValue( ASURA_HASH_ID_UNSET ); break;
		default : Asura_Assert( false, "Unknown value type!" ); break;
	}

    // Now pick the parameter name. (In future we will take into account whether there is a set list of allowed names if this is templated.)
    Asura_String xNewParameterName( "" );
    bool         bConvertChildrenToOverrides = false;
    if ( !GetNewParameterName( eNewType, uTemplateHashID, true, xNewParameterName, bConvertChildrenToOverrides ) ) { return; }

    // Convert the first matching parameter in each child branch to a fixed override (before we add the new parameter, so we don't alter any current values).
    if ( bConvertChildrenToOverrides )
    {
        ConvertTopParametersToOverrides( pxBlueprint, m_uBlueprintTypeHashID, Asura_GetHashID( xNewParameterName.GetString() ) );
    }

    pxBlueprint->AddParameter( xNewParameterName, "", ABOT_NONE, xValue, uTemplateHashID );

    Tools_Mode_Blueprint::SetTypeAsModified( m_uBlueprintTypeHashID );
    Tools_Mode_Blueprint::RequestRefreshBlueprintEditObjectOverlays();
}


bool Tools_Blueprint_Edit_Object::CanDeleteSelectedParameterStatic( int iUserData )
{
	const Tools_Blueprint_Edit_Object*	pxThis	= reinterpret_cast< Tools_Blueprint_Edit_Object* >( iUserData );

	return(pxThis->CanDeleteSelectedParameter());
}

bool Tools_Blueprint_Edit_Object::CanDeleteSelectedParameter(void) const
{
	const int	iRow	= m_pxTableDialogue->GetCurrentlySelectedRow();

	if(iRow >= 0)
	{
		const char*				szName		= m_ppszParameterStrings[iRow];
		const Asura_Hash_ID		uHash		= Asura_GetHashID(szName);
		const Asura_Blueprint*	pxBlueprint	= GetBlueprint();
		const Asura_Parameter*	pxParameter	= pxBlueprint->GetParameterByHashID(uHash);

		// Ensure that we have data to delete...
		return((pxParameter != NULL) && (pxParameter->GetOverrideType() == ABOT_NONE));
	}
	else
	{
		return(false);
	}
}

void Tools_Blueprint_Edit_Object::DeleteSelectedParameterStatic( int iUserData )
{
    Tools_Blueprint_Edit_Object* pxThis = reinterpret_cast< Tools_Blueprint_Edit_Object* >( iUserData );
    pxThis->DeleteSelectedParameter();
}

void Tools_Blueprint_Edit_Object::DeleteSelectedParameter()
{
    Asura_Blueprint* pxBlueprint = GetBlueprint();
    if ( !pxBlueprint ) { return; }

	const int	iRow	= m_pxTableDialogue->GetCurrentlySelectedRow();

    // Check down the hierarchy to see if this parameter is overridden anywhere.
    // If it is, we have two options:
    // 1) The override becomes an actual parameter, meaning everything lower in the hierarchy stays as is.
    // 2) All overrides lower in the hierarchy must be deleted.

    const char*         pszName      = m_ppszParameterStrings[iRow];
    const Asura_Hash_ID uParamHashID = Asura_GetHashID( pszName );
    bool bOverrideFound = false;
    OverrideExists( pxBlueprint, GetBlueprintTypeHashID(), uParamHashID, bOverrideFound );

    if ( bOverrideFound )
    {
        Asura_String xTitle( "This parameter is overridden further down the hierarchy. Do you want to delete these overrides?\n"
                             "If the overrides are not deleted the next override down will become a normal parameter." );
        YesNoCancel_Overlay xYesNoCancelOverlay( xTitle );
        if ( !xYesNoCancelOverlay.DoModal() ) { return; }
        if ( xYesNoCancelOverlay.GetResult() )
        {
            // Delete this parameter and all overrides of it.
            pxBlueprint->DeleteParameter( pszName );
            DeleteAllOverrides( pxBlueprint, GetBlueprintTypeHashID(), uParamHashID );
        }
        else
        {
            // Convert the next overrides down into a proper parameters first, so that values do not change.
            ConvertTopOverridesToParameters( pxBlueprint, GetBlueprintTypeHashID(), uParamHashID );

            // Now delete the original parameter.
            pxBlueprint->DeleteParameter( pszName );
        }
    }
    else
    {
        Asura_String xTitle( "Are you sure you wish to delete this parameter?" );
        YesNo_Overlay xYesNoOverlay( xTitle );
        if ( !xYesNoOverlay.DoModal() ) { return; }

        pxBlueprint->DeleteParameter( pszName );
    }

    Tools_Mode_Blueprint::SetTypeAsModified( GetBlueprintTypeHashID() );
    Tools_Mode_Blueprint::RequestRefreshBlueprintEditObjectOverlays();
}

bool Tools_Blueprint_Edit_Object::CanRenameSelectedParameterStatic( int iUserData )
{
	const Tools_Blueprint_Edit_Object* pxThis = reinterpret_cast< Tools_Blueprint_Edit_Object* >( iUserData );
	return pxThis->CanRenameSelectedParameter();
}

bool Tools_Blueprint_Edit_Object::CanRenameSelectedParameter() const
{
	const int iRow = m_pxTableDialogue->GetCurrentlySelectedRow();
	if ( iRow >= 0 )
	{
		const char*				szName				= m_ppszParameterStrings[iRow];
		const Asura_Hash_ID		uHash				= Asura_GetHashID(szName);
		const Asura_Blueprint*	pxLocalBlueprint	= GetBlueprint();

		while ( pxLocalBlueprint )
		{
			const Asura_Parameter* pxParameter = pxLocalBlueprint->GetParameterByHashID(uHash);
			if ( pxParameter && pxParameter->GetOverrideType() == ABOT_NONE )
			{
				return true;
			}

			pxLocalBlueprint = pxLocalBlueprint->GetParent();
		}
	}

	return false;
}

void Tools_Blueprint_Edit_Object::RenameSelectedParameterStatic( int iUserData )
{
    Tools_Blueprint_Edit_Object* pxThis = reinterpret_cast< Tools_Blueprint_Edit_Object* >( iUserData );
    pxThis->RenameSelectedParameter();
}

void Tools_Blueprint_Edit_Object::RenameSelectedParameter()
{
    Asura_Blueprint* pxBlueprint = GetBlueprint();
    if ( !pxBlueprint ) { return; }

	int					iRow	= m_pxTableDialogue->GetCurrentlySelectedRow();
	Asura_String		xNewName( m_ppszParameterStrings[ iRow ] );
    Asura_Parameter*	pxParameter = pxBlueprint->GetParameterByHashID( xNewName.GetHash(), true );
    if ( !pxParameter ) { return; }

    // Get a new unique name for the parameter. (We may want to pick from a list at this point.)
    bool bDummy;
    if ( !GetNewParameterName( pxParameter->GetValue().GetValueType(), pxParameter->GetTemplate(), false, xNewName, bDummy ) ) { return; }

    YesNo_Overlay xConfirmation( "Are you sure you want to rename this parameter?" );
    if ( !xConfirmation.DoModal() ) { return; }

	// This must be a copy, not a reference, as pxParameter will become invalid during this operation.
	const Asura_String  xOldName			  = pxParameter->GetName();
    const Asura_Hash_ID uOldParameterNameHash = pxParameter->GetHashID();
    const Asura_Hash_ID uNewParameterNameHash = Asura_GetHashID( xNewName.GetString() );
	if ( uNewParameterNameHash == uOldParameterNameHash ) { return; }

	// Find the root blueprint for this parameter.
	// Work up the hierarchy and find the first blueprint which contains a non-overridden parameter with this hash.
	while ( pxBlueprint )
	{
		Asura_Parameter* pxRootParameter = pxBlueprint->GetParameterByHashID( uOldParameterNameHash );
		if ( pxRootParameter && pxRootParameter->GetOverrideType() == ABOT_NONE )
		{
			break;
		}
		pxBlueprint = pxBlueprint->GetParent();
	}
	if ( !pxBlueprint ) { return; }

	// Rename the parameter in the top blueprint.
	pxBlueprint->RenameParameter( xOldName, xNewName );
	// Do not use pxParameter from this point on.

	// Rename all parameters with this hash from this point in the hierarchy downwards.
	for ( Asura_Blueprint_Iterator xBPIter( GetBlueprintTypeHashID() ); !xBPIter.Done(); xBPIter.Next() )
	{
		Asura_Blueprint* pxCurrBlueprint = xBPIter.GetCurrent();
		if ( pxCurrBlueprint->IsChildOf( GetBlueprintTypeHashID(), pxBlueprint->GetHashID(), false ) )
		{
			Asura_Parameter* pxTargetParameter = pxCurrBlueprint->GetParameterByHashID( uOldParameterNameHash, false );
			if ( pxTargetParameter )
			{
				pxCurrBlueprint->RenameParameter( xOldName, xNewName );
			}
		}
	}

    Tools_Mode_Blueprint::SetTypeAsModified( GetBlueprintTypeHashID() );
    Tools_Mode_Blueprint::RequestRefreshBlueprintEditObjectOverlays();
}

void Tools_Blueprint_Edit_Object::CopyParametersStatic( int iUserData )
{
    Tools_Blueprint_Edit_Object* pxThis = reinterpret_cast< Tools_Blueprint_Edit_Object* >( iUserData );
    pxThis->CopyParameters();
}

void Tools_Blueprint_Edit_Object::CutParametersStatic( int iUserData )
{
    Tools_Blueprint_Edit_Object* pxThis = reinterpret_cast< Tools_Blueprint_Edit_Object* >( iUserData );
    pxThis->CutParameters();
}

void Tools_Blueprint_Edit_Object::PasteParametersStatic( int iUserData )
{
    Tools_Blueprint_Edit_Object* pxThis = reinterpret_cast< Tools_Blueprint_Edit_Object* >( iUserData );
    pxThis->PasteParameters();
}

void Tools_Blueprint_Edit_Object::PasteOverParametersStatic( int iUserData )
{
    Tools_Blueprint_Edit_Object* pxThis = reinterpret_cast< Tools_Blueprint_Edit_Object* >( iUserData );
    pxThis->PasteOverParameters();
}

void Tools_Blueprint_Edit_Object::CopyParameters()
{
    // Work out which parameters to copy.
    Tools_Modal_Overlay_BPParameterSelect xParameterSelectOverlay( "Select the parameters to copy:", GetBlueprintTypeHashID(), GetBlueprintHashID(), false );
	if ( xParameterSelectOverlay.GetNumItems() == 0 )
	{
		Message_Overlay xMessage( "There are no parameters to copy." );
		xMessage.DoModal();
		return;
	}
    if ( !xParameterSelectOverlay.DoModal() ) { return; }

    // Get the number of parameters that the user wants copied.
    const int iNumToCopy = xParameterSelectOverlay.GetNumSelectedParameters();
	CopyParameters( GetBlueprint(), iNumToCopy, GetBlueprintTypeHashID(), &xParameterSelectOverlay );
}

void Tools_Blueprint_Edit_Object::CopyParameters( Asura_Blueprint* pxBlueprint, const int iNumToCopy,
												  const Asura_Hash_ID uSourceTypeID,
												  const Tools_Modal_Overlay_BPParameterSelect* pxOverlay /*=NULL*/ )
{
	if ( !pxBlueprint ) { return; }

    // Reallocate the paste buffer.
    DestroyBufferStorage();
    if ( iNumToCopy == 0 ) { return; }
    s_pxParameterBuffer    = new Asura_Parameter[ iNumToCopy ];
    s_uParameterBufferSize = static_cast< u_int >( iNumToCopy );
    s_uNumInBuffer         = 0;   
	s_uParameterSourceType = uSourceTypeID;
	s_uParameterSourceBP   = pxBlueprint->GetHashID();    

	// Copy them into the buffer
	for ( int i = 0; i < iNumToCopy; ++i )
    {
        Asura_Parameter* pxParameter = (pxOverlay != NULL)
									 ? pxBlueprint->GetParameterByHashID( pxOverlay->GetSelectedParameterHashID( i ), true )
									 : pxBlueprint->GetParameterByIndex( i );

        if ( pxParameter )
        {
			s_pxParameterBuffer[ s_uNumInBuffer ] = *pxParameter;
            ++s_uNumInBuffer;
        }
    }
}

void Tools_Blueprint_Edit_Object::CutParameters()
{
    // Work out which parameters to cut. Note that we can only cut parameters that are new in this blueprint.
    Tools_Modal_Overlay_BPParameterSelect xParameterSelectOverlay( "Select the parameters to cut:", GetBlueprintTypeHashID(), GetBlueprintHashID(), true );
	if ( xParameterSelectOverlay.GetNumItems() == 0 )
	{
		Message_Overlay xMessage( "You can only cut parameters, not overrides. There are no parameters to cut in this blueprint." );
		xMessage.DoModal();
		return;
	}
    if ( !xParameterSelectOverlay.DoModal() ) { return; }

    // Get the number of parameters that the user wants cut.
    const int iNumToCut = xParameterSelectOverlay.GetNumSelectedParameters();

    // Reallocate the paste buffer.
    DestroyBufferStorage();
    if ( iNumToCut == 0 ) { return; }
    s_pxParameterBuffer    = new Asura_Parameter[ iNumToCut ];
    s_uParameterBufferSize = static_cast< u_int >( iNumToCut );
    s_uNumInBuffer         = 0;
	s_uParameterSourceType = GetBlueprintTypeHashID();
	s_uParameterSourceBP   = GetBlueprintHashID();    

    // Check whether any of the selected parameters have overrides further down the hierarchy.
    Asura_Blueprint* pxBlueprint     = GetBlueprint();
    bool             bOverridesExist = false;
    for ( int i = 0; i < iNumToCut && !bOverridesExist; ++i )
    {
        OverrideExists( pxBlueprint, GetBlueprintTypeHashID(), xParameterSelectOverlay.GetSelectedParameterHashID( i ), bOverridesExist );
    }

    bool bDeleteOverrides = false;
    if ( bOverridesExist )
    {
        // Ask the user what they want to do with the overridden parameters.
        YesNoCancel_Overlay xYNCOverlay( "Some parameters are overridden further down the hierarchy. Do you want to delete these overrides?\n"
                                         "If the overrides are not deleted the next override down will become a normal parameter." );
        if ( !xYNCOverlay.DoModal() ) { return; }
        bDeleteOverrides = xYNCOverlay.GetResult();
    }

    // Copy the selected parameters into the buffer.
    if ( !pxBlueprint ) { return; }
    for ( int i = 0; i < iNumToCut; ++i )
    {
        Asura_Parameter* pxParameter = pxBlueprint->GetParameterByHashID( xParameterSelectOverlay.GetSelectedParameterHashID( i ), false );
        if ( pxParameter )
        {
            s_pxParameterBuffer[ s_uNumInBuffer ] = *pxParameter;
            ++s_uNumInBuffer;
        }
    }

    // Resolve any overrides futher down the hierarchy and then remove the selected parameters from the source blueprint.
    if ( bOverridesExist )
    {
        if ( bDeleteOverrides )
        {
            for ( int i = 0; i < iNumToCut; ++i )
            {
                DeleteAllOverrides( pxBlueprint, GetBlueprintTypeHashID(), xParameterSelectOverlay.GetSelectedParameterHashID( i ) );
            }
        }
        else
        {
            for ( int i = 0; i < iNumToCut; ++i )
            {
                ConvertTopOverridesToParameters( pxBlueprint, GetBlueprintTypeHashID(), xParameterSelectOverlay.GetSelectedParameterHashID( i ) );
            }
        }
    }

    // Finally, delete the parameters in the blueprint.
    for ( int i = 0; i < iNumToCut; ++i )
    {
        pxBlueprint->DeleteParameter( xParameterSelectOverlay.GetSelectedParameterName( i ) );
    }

    Tools_Mode_Blueprint::SetTypeAsModified( GetBlueprintTypeHashID() );
    Tools_Mode_Blueprint::RequestRefreshBlueprintEditObjectOverlays();
}

void Tools_Blueprint_Edit_Object::PasteParameters()
{
	ClearParameterPasteConflictList();
	PasteParameters( GetBlueprint(), GetBlueprintTypeHashID() );
	ReportParameterPasteConflicts( "The following parameter(s) could not be pasted, as they already exist in this blueprint hierarchy:" );
}

void Tools_Blueprint_Edit_Object::PasteParameters( Asura_Blueprint* pxBlueprint, const Asura_Hash_ID uBPTypeHashID )
{
	if ( !pxBlueprint ) { return; }
    if ( s_uNumInBuffer == 0 ) { return; }

    // Work out whether any of these parameters exist in the hierarchy already, either above us or below.
    for ( u_int u = 0; u < s_uNumInBuffer; ++u )
    {
        const Asura_Hash_ID uParameterHashID = s_pxParameterBuffer[ u ].GetHashID();
		
		bool bExistsBelow = false;
		ParameterExists( pxBlueprint, uBPTypeHashID, s_pxParameterBuffer[ u ].GetHashID(), bExistsBelow );
        
        if ( (pxBlueprint->GetParent() && pxBlueprint->GetParent()->GetParameterByHashID( uParameterHashID, true )) // Exists above?
		 ||	pxBlueprint->GetParameterByHashID( uParameterHashID, false ) // Exists in here?
		 || bExistsBelow ) // Exists below? (If it's not above or in here then it must be a parameter, not an override, if it's below us.)
        {
			AddParameterPasteConflict( s_pxParameterBuffer[ u ].GetName() );
        }
		else
		{
			// Copy the parameters that do not conflict into our blueprint.
			pxBlueprint->AddParameter( s_pxParameterBuffer[ u ].GetName(), s_pxParameterBuffer[ u ].GetGroupName(),
									   ABOT_NONE, s_pxParameterBuffer[ u ].GetValue(),
									   s_pxParameterBuffer[ u ].GetTemplate() );

			Tools_Mode_Blueprint::SetTypeAsModified( uBPTypeHashID );
			Tools_Mode_Blueprint::RequestRefreshBlueprintEditObjectOverlays();
		}
    }
}

void Tools_Blueprint_Edit_Object::ClearParameterPasteConflictList()
{
	s_xParameterPasteConflictList.Clear();
}

void Tools_Blueprint_Edit_Object::AddParameterPasteConflict( const Asura_String& xName )
{
	// don't add multiples of the same parameter to the list
	if( s_xParameterPasteConflictList.GetIndexOfName( xName ) == -1 )
	{
		s_xParameterPasteConflictList.AddName( xName );
	}	
}

void Tools_Blueprint_Edit_Object::ReportParameterPasteConflicts( const char* szTitle )
{
	if( s_xParameterPasteConflictList.GetNumUsedEntries() > 0 )
	{
		Asura_String xMessage;
		xMessage += szTitle;
		xMessage += "\n\n";
		
		// add all conflicts in the list
		for( int i=0; i<s_xParameterPasteConflictList.GetNumUsedEntries(); ++i )
		{
			xMessage += "        ";
			xMessage += s_xParameterPasteConflictList.GetName(i);
			xMessage += "\n";
		}

		Message_Overlay xMessageOverlay( xMessage );
		xMessageOverlay.DoModal();
	}
}


void Tools_Blueprint_Edit_Object::PasteOverParameters()
{
	ClearParameterPasteConflictList();	
	PasteOverParameters( GetBlueprint(), GetBlueprintTypeHashID() );
	ReportParameterPasteConflicts( "The following parameter(s) could not be pasted, as they already exist as different types in this blueprint hierarchy:" );
}

void Tools_Blueprint_Edit_Object::PasteOverParameters( Asura_Blueprint* pxBlueprint, const Asura_Hash_ID uBPTypeHashID )
{
    if ( s_uNumInBuffer == 0 ) { return; }
    if ( !pxBlueprint ) { return; }

    // Paste the parameters in, resolving where necessary.
    for ( u_int u = 0; u < s_uNumInBuffer; ++u )
    {
        const Asura_Hash_ID uParameterHashID = s_pxParameterBuffer[ u ].GetHashID();

        // Exists above?
        if ( pxBlueprint->GetParent() && pxBlueprint->GetParent()->GetParameterByHashID( uParameterHashID, true ) )
        {
			// check for type conflicts between the original override and the one we're trying to paste in 
			Asura_Parameter* pxOverrideAbove = pxBlueprint->GetParent()->GetParameterByHashID( uParameterHashID, true );
			if ( pxOverrideAbove->GetValue().GetValueType() == s_pxParameterBuffer[ u ].GetValue().GetValueType() )
			{
				// Try and delete it first, in case an override exists already in this blueprint.
				pxBlueprint->DeleteParameter( s_pxParameterBuffer[ u ].GetName() );

				if ( s_pxParameterBuffer[ u ].GetOverrideType() == ABOT_NONE )
				{
					// We must add this parameter as a fixed override so it maintains its value.
					pxBlueprint->AddParameter( s_pxParameterBuffer[ u ].GetName(), s_pxParameterBuffer[ u ].GetGroupName(),
											   ABOT_FIXED, s_pxParameterBuffer[ u ].GetValue(),
											   s_pxParameterBuffer[ u ].GetTemplate() );
					continue;
				}
				else
				{			
					// Types match so we can just paste it in as is.
					pxBlueprint->AddParameter( s_pxParameterBuffer[ u ].GetName(), s_pxParameterBuffer[ u ].GetGroupName(),
											   s_pxParameterBuffer[ u ].GetOverrideType(), s_pxParameterBuffer[ u ].GetValue(),
											   s_pxParameterBuffer[ u ].GetTemplate() );
					continue;
				}
			}
			else
			{
				// differing types, can't paste parameter/override				
				AddParameterPasteConflict( s_pxParameterBuffer[ u ].GetName() );
				continue;							
			}
        }

		bool bExistsBelow = false;
		Asura_Parameter* pxParameterBelow = NULL;
		ParameterExists( pxBlueprint, uBPTypeHashID, s_pxParameterBuffer[ u ].GetHashID(), bExistsBelow, pxParameterBelow );

		// Exists in here?		
		Asura_Parameter* pxParameterInHere = pxBlueprint->GetParameterByHashID( uParameterHashID, false );
		if ( pxParameterInHere )
		{
			// If they are different types we can only blat over this is there aren't any overrides below.
			// Otherwise we can just blat over this parameter. It didn't exist above us, so it must be a parameter, not an override, in our blueprint.
			if( (pxParameterInHere->GetValue().GetValueType() == s_pxParameterBuffer[ u ].GetValue().GetValueType())
			 || !bExistsBelow )
			{			
				pxBlueprint->DeleteParameter( s_pxParameterBuffer[ u ].GetName() );
				pxBlueprint->AddParameter( s_pxParameterBuffer[ u ].GetName(), s_pxParameterBuffer[ u ].GetGroupName(),
										   ABOT_NONE, s_pxParameterBuffer[ u ].GetValue(),
										   s_pxParameterBuffer[ u ].GetTemplate() );
				continue;
			}
			else
			{
				// Different types and an override exists below. Can't add.
				AddParameterPasteConflict( s_pxParameterBuffer[ u ].GetName() );	
				continue;	
			}
		}

        // Exists below? (If it's not above or in here then it must be a parameter, not an override, if it's below us.)
        if ( bExistsBelow )
        {
			if( pxParameterBelow->GetValue().GetValueType() == s_pxParameterBuffer[ u ].GetValue().GetValueType() )
			{
				// We must convert any top child parameters into fixed overrides, so they keep their current value.
				ConvertTopParametersToOverrides( pxBlueprint, uBPTypeHashID, uParameterHashID );

				// Then we add this parameter as a fresh parameter.
				pxBlueprint->AddParameter( s_pxParameterBuffer[ u ].GetName(), s_pxParameterBuffer[ u ].GetGroupName(),
										   ABOT_NONE, s_pxParameterBuffer[ u ].GetValue(),
										   s_pxParameterBuffer[ u ].GetTemplate() );
				continue;
			}				
			else
			{
				// types different, can't add
				AddParameterPasteConflict( s_pxParameterBuffer[ u ].GetName() );
				continue;
			}
        }       
		        
		// If we've got here then this parameter doesn't exist in this hierarchy yet.
		if( s_pxParameterBuffer[ u ].GetOverrideType() != ABOT_NONE )
		{
			// if it was an override and there's no parameter to attach to above, convert into a full-blown parameter
			// need to look up the original value of the parameter from the source blueprint, assuming it still exists
			Asura_Value xFinalValue;
			xFinalValue = s_pxParameterBuffer[ u ].GetValue();

			Asura_Blueprint* pxSourceBP = Asura_Blueprint_System::GetBlueprintByHashID( s_uParameterSourceType, s_uParameterSourceBP );
			if( pxSourceBP )
			{			
				pxSourceBP->GetParameterValueByHashID( s_pxParameterBuffer[ u ].GetHashID(), xFinalValue );								
			}

			pxBlueprint->AddParameter( s_pxParameterBuffer[ u ].GetName(), s_pxParameterBuffer[ u ].GetGroupName(),
									   ABOT_NONE, xFinalValue,
									   s_pxParameterBuffer[ u ].GetTemplate() );
		}
		else
		{		
			// Parameter, just add it straight in.
			pxBlueprint->AddParameter( s_pxParameterBuffer[ u ].GetName(), s_pxParameterBuffer[ u ].GetGroupName(),
									   s_pxParameterBuffer[ u ].GetOverrideType(), s_pxParameterBuffer[ u ].GetValue(),
									   s_pxParameterBuffer[ u ].GetTemplate() );
		}
    }

    Tools_Mode_Blueprint::SetTypeAsModified( uBPTypeHashID );
    Tools_Mode_Blueprint::RequestRefreshBlueprintEditObjectOverlays();
}

void Tools_Blueprint_Edit_Object::DestroyBufferStorage()
{
    delete [] s_pxParameterBuffer;
    s_pxParameterBuffer    = NULL;
    s_uParameterBufferSize = 0;
    s_uNumInBuffer         = 0;
}

void Tools_Blueprint_Edit_Object::ParameterExists( Asura_Blueprint* pxBlueprint,
												   const Asura_Hash_ID uBPTypeHashID,
												   const Asura_Hash_ID uParameterHashID,
												   bool& bParameterExists,
												   Asura_Parameter* pxOutputParameter /*=NULL*/ )
{
    // Check this blueprint first.
    Asura_Parameter* pxParameter = pxBlueprint->GetParameterByHashID( uParameterHashID, false );
    if ( pxParameter && pxParameter->GetOverrideType() == ABOT_NONE )
    {
        bParameterExists = true;
		if( pxOutputParameter )
		{
			pxOutputParameter = pxParameter;
		}

        return;
    }

    // Then check all of its children.
    const int iNumChildren = Asura_Blueprint_System::GetNumChildBlueprints( uBPTypeHashID, pxBlueprint->GetHashID(), true );
    for ( int iChildIndex = 0; iChildIndex < iNumChildren; ++iChildIndex )
    {
        Asura_Blueprint* pxChild = Asura_Blueprint_System::GetChildBlueprint( uBPTypeHashID, pxBlueprint->GetHashID(), true, iChildIndex );
        if ( pxChild )
        {
            ParameterExists( pxChild, uBPTypeHashID, uParameterHashID, bParameterExists );
        }
    }
}

void Tools_Blueprint_Edit_Object::OverrideExists( Asura_Blueprint* pxBlueprint, const Asura_Hash_ID uBPTypeHashID, const Asura_Hash_ID uParameterHashID, bool& bOverrideExists )
{
    // Check this blueprint first.
    Asura_Parameter* pxParameter = pxBlueprint->GetParameterByHashID( uParameterHashID, false );
    if ( pxParameter && pxParameter->GetOverrideType() != ABOT_NONE )
    {
        bOverrideExists = true;
        return;
    }

    // Then check all of its children.
    const int iNumChildren = Asura_Blueprint_System::GetNumChildBlueprints( uBPTypeHashID, pxBlueprint->GetHashID(), true );
    for ( int iChildIndex = 0; iChildIndex < iNumChildren; ++iChildIndex )
    {
        Asura_Blueprint* pxChild = Asura_Blueprint_System::GetChildBlueprint( uBPTypeHashID, pxBlueprint->GetHashID(), true, iChildIndex );
        if ( pxChild )
        {
            OverrideExists( pxChild, uBPTypeHashID, uParameterHashID, bOverrideExists );
        }
    }
}

void Tools_Blueprint_Edit_Object::DeleteAllOverrides( Asura_Blueprint* pxBlueprint, const Asura_Hash_ID uBPTypeHashID, const Asura_Hash_ID uParameterHashID )
{
    // Check this blueprint first.
    Asura_Parameter* pxParameter = pxBlueprint->GetParameterByHashID( uParameterHashID, false );
    if ( pxParameter && pxParameter->GetOverrideType() != ABOT_NONE )
    {
        pxBlueprint->DeleteParameter( pxParameter->GetName() );
    }

    // Then check all of its children.
    const int iNumChildren = Asura_Blueprint_System::GetNumChildBlueprints( uBPTypeHashID, pxBlueprint->GetHashID(), true );
    for ( int iChildIndex = 0; iChildIndex < iNumChildren; ++iChildIndex )
    {
        Asura_Blueprint* pxChild = Asura_Blueprint_System::GetChildBlueprint( uBPTypeHashID, pxBlueprint->GetHashID(), true, iChildIndex );
        if ( pxChild )
        {
            DeleteAllOverrides( pxChild, uBPTypeHashID, uParameterHashID );
        }
    }
}

void Tools_Blueprint_Edit_Object::ConvertTopOverridesToParameters( Asura_Blueprint* pxBlueprint, const Asura_Hash_ID uBPTypeHashID, const Asura_Hash_ID uParameterHashID )
{
    // Check this blueprint first.
    Asura_Parameter* pxParameter = pxBlueprint->GetParameterByHashID( uParameterHashID, false );
    if ( pxParameter && pxParameter->GetOverrideType() != ABOT_NONE )
    {
        // Get the value of the parameter in this blueprint.
        Asura_Value xValue;
        if ( pxBlueprint->GetParameterValueByHashID( uParameterHashID, xValue ) )
        {
            pxParameter->SetValue( xValue );
        }
        pxParameter->SetOverrideType( ABOT_NONE );

        // We only want to convert the top override in each branch, so we're done.
        return;
    }

    // Then check all of its children.
    const int iNumChildren = Asura_Blueprint_System::GetNumChildBlueprints( uBPTypeHashID, pxBlueprint->GetHashID(), true );
    for ( int iChildIndex = 0; iChildIndex < iNumChildren; ++iChildIndex )
    {
        Asura_Blueprint* pxChild = Asura_Blueprint_System::GetChildBlueprint( uBPTypeHashID, pxBlueprint->GetHashID(), true, iChildIndex );
        if ( pxChild )
        {
            ConvertTopOverridesToParameters( pxChild, uBPTypeHashID, uParameterHashID );
        }
    }
}

void Tools_Blueprint_Edit_Object::ConvertTopParametersToOverrides( Asura_Blueprint* pxBlueprint, const Asura_Hash_ID uBPTypeHashID, const Asura_Hash_ID uParameterHashID )
{
    if ( !pxBlueprint ) { return; }

    // If we have a parameter with this hash ID convert it to a fixed override and return, otherwise check our children.
    Asura_Parameter* pxParameter = pxBlueprint->GetParameterByHashID( uParameterHashID, false );
    if ( pxParameter && pxParameter->GetOverrideType() == ABOT_NONE )
    {
        pxParameter->SetOverrideType( ABOT_FIXED );
        return;
    }

    // Then check all of its children.
    const int iNumChildren = Asura_Blueprint_System::GetNumChildBlueprints( uBPTypeHashID, pxBlueprint->GetHashID(), true );
    for ( int iChildIndex = 0; iChildIndex < iNumChildren; ++iChildIndex )
    {
        Asura_Blueprint* pxChild = Asura_Blueprint_System::GetChildBlueprint( uBPTypeHashID, pxBlueprint->GetHashID(), true, iChildIndex );
        if ( pxChild )
        {
            ConvertTopParametersToOverrides( pxChild, uBPTypeHashID, uParameterHashID );
        }
    }
}

void Tools_Blueprint_Edit_Object::SelectParameterRow( const Asura_Hash_ID uParamHashID )
{
	for ( u_int uRow = 0; uRow < m_pxTableDialogue->GetNumRows(); ++uRow )
	{
		const Asura_TableCell_Dialog* pxCell = m_pxTableDialogue->GetCell( uRow, 1 );
		if ( pxCell )
		{
			// All non-heading cells in the table inherit from Tools_Blueprint_TableCell.
			const Tools_Blueprint_TableCell* pxBPCell = static_cast< const Tools_Blueprint_TableCell* >( pxCell );
			if ( pxBPCell->GetBlueprintParamHashID() == uParamHashID )
			{
				m_pxTableDialogue->SetCurrentlySelectedRow( uRow );
				break;
			}
		}
	}
}

Asura_Blueprint* Tools_Blueprint_Edit_Object::GetBlueprint() const
{
    return Asura_Blueprint_System::GetBlueprintByHashID( GetBlueprintTypeHashID(), GetBlueprintHashID() );
}

bool Tools_Blueprint_Edit_Object::OverrideTypeIsAllowed( const Asura_Parameter& xParameter, const Asura_Blueprint_Override_Type& eOverrideType ) const
{
    if ( eOverrideType == ABOT_NONE )
    {
        return true;
    }

    const int iTemplatedType = GetTemplatedTypeFromTemplateID( xParameter.GetTemplate() );
    if ( iTemplatedType != -1 )
    {
        // Templated types must have a fixed override.
        return ( eOverrideType == ABOT_FIXED );
    }
    else if ( xParameter.GetValue().GetValueType() == Asura_Value::VT_INT || xParameter.GetValue().GetValueType() == Asura_Value::VT_FLOAT )
    {
        // Ints and floats can have any override type.
        return true;
    }
    else
    {
        // Anything else (bools, hashes) must be a fixed override.
        return ( eOverrideType == ABOT_FIXED );
    }
}

bool Traverse_AddTemplatedTypes( Asura_Hash_ID uNodeHash, Asura_Blueprint_Template* pxTemplate, void* pxData )
{
	ListSelection_Overlay* pxOverlay = static_cast<ListSelection_Overlay*> (pxData);
	pxOverlay->AddItem( pxTemplate->GetName(), pxTemplate->GetType() );

	return true;
}

void Tools_Blueprint_Edit_Object::AddTypesToTypeOverlay( ListSelection_Overlay& xTypeOverlay )
{
	// add basic types
    for ( int i = 0; i < Asura_Value::VT_END_VALUE_TYPES; ++i )
    {
        xTypeOverlay.AddItem( Asura_Value::s_apszValueTypeStrings[ i ], i );
    }

	// add any templated types
	s_xTemplatedTypeTree.Traverse( Traverse_AddTemplatedTypes, &xTypeOverlay );
}

bool Tools_Blueprint_Edit_Object::GetValueForTemplatedType( const int iTemplatedType, const Asura_Hash_ID uTemplateHashID, const Asura_String xParameterName, Asura_Value& xValue )
{
	switch ( iTemplatedType )
	{
		case TVT_SHAPE:
		{		
			Asura_String xShapeName;
			Asura_Hash_ID uHash = Tools_ShapeTable::SelectShape(xValue.GetHashIDValue());
			
			if (uHash !=ASURA_HASH_ID_UNSET)
			{			
				xShapeName=Asura_HashedFilename_System::GetFilename(uHash,false);		
			}
			else
			{
				xShapeName="";
			}

			xValue.SetValue(uHash,xShapeName.GetString());
			return true;
		}

		case TVT_COLOUR:
		{
			Asura_RGBA xColour;
			xColour.FromUIntARGB( static_cast<u_int> (xValue.GetIntValue()) );
	
			Asura_String xWindowTitle( "Choose Colour For '" );
			xWindowTitle += xParameterName;
			xWindowTitle += "'";

			bool bChanged = false;
			Tools_ColourPicker xPicker( xWindowTitle.GetString(), &xColour.m_fR, &xColour.m_fG, &xColour.m_fB, 1.0f, &bChanged );
			if ( !xPicker.DoModal() ) { return false; }

			xValue.SetValue( static_cast<int>(xColour.ToUIntARGB()) );

			return true;
		} 

		case TVT_SKIN_OR_ANIM:
		{
			Tools_FileSystem_DialogInfo	xFileSelector;
			xFileSelector.DefaultExtension		= ".ASR";
			xFileSelector.DefaultFilename		= "";
			Asura_String xDefaultPath;
			if ( Asura_UserInput::KeyPressed( ASURA_KEYID_CTRL ) )
			{
				// If ctrl is held down whilst we open this dialogue then we should default to using the current value's path.
				Asura_String xValueString;
				xValueString = xValue.GetHashStringValue();
				Asura_String xName;
				Asura_String xExt;
				xValueString.SplitFilename( &xDefaultPath, &xName, &xExt );
			}
			else
			{
				// If not then we default to using the last path.
				xDefaultPath = Tools_Mode_Blueprint::GetLastSkinAnimPath();
			}
			xFileSelector.DefaultPath			= xDefaultPath.IsBlank() ? "Characters" : xDefaultPath.GetString();
			xFileSelector.FileFilter			= "Asura Files\0*.ASR;\0";
			Asura_String xWindowTitle( "Choose Skin/Anim File For '" );
			xWindowTitle += xParameterName;
			xWindowTitle += "'";
			xFileSelector.WindowTitle			= xWindowTitle.GetString();
			xFileSelector.AllowMultipleSelect	= false;

			Asura_String xFilename = Tools_FileSystem::GetLoadFilename( xFileSelector );
			if( xFilename.IsNull() )
			{
				// pop up a window asking if they want it to be added as 'unset'
				YesNo_Overlay xYesNo( "No file selected. Do you wish to add the parameter anyway? This will have the value '*Unset*'." );
				if( !xYesNo.DoModal() ) return false;

				xValue.SetValue( ASURA_HASH_ID_UNSET );
			}
			else
			{
				// check this is actually a skin/anim asura file
				Tools_AsuraFileContents xFileContents;
				if( !xFileContents.LoadAsuraFile( xFilename.GetString() ) )	
				{
					Asura_Assertf( false, ("Failed to get file contents for file '%s'", xFilename.GetString()) );
					return false;
				}

				if( (xFileContents.GetFileType() != AsuraFile_Skin) && (xFileContents.GetFileType() != AsuraFile_Anim) )
				{
                    Asura_String xMessage( "File '" );
                    xMessage += xFilename.GetString();
                    xMessage += "' is not a skin or anim.";
                    Message_Overlay xMessageOverlay( xMessage );
                    xMessageOverlay.DoModal();
					return false;
				}
				
				// split the path to get the hash ID
				Asura_String xPath;
				Asura_String xName;
				Asura_String xExt;
				xFilename.SplitFilename( &xPath, &xName, &xExt );

				Tools_Mode_Blueprint::SetLastSkinAnimPath( xPath );

                Asura_String xOldPath = xPath;
                xOldPath.MakeLowercase();
                const char* szOutput = strstr( xOldPath.GetString(), "asura_files\\" );
                if ( !szOutput )
                {
                    Message_Overlay xMessageOverlay( "Skin/anim must be in the asura_files directory structure." );
                    xMessageOverlay.DoModal();
                    return false;
                }
                szOutput += strlen("asura_files\\");

                xFilename = szOutput;
                xFilename += "\\";
                xFilename += xName;
                xFilename += ".";
                xFilename += xExt;

                xValue.SetValue( Asura_GetHashID( xName.GetString() ), xFilename.GetString() );
			}
					
			return true;
		}

		case TVT_PFX:
		{
			Asura_String xWindowTitle( "Choose PFX Effect For '" );
			xWindowTitle += xParameterName;
			xWindowTitle += "'";

			Tools_PFXSelect_Overlay xOverlay(xWindowTitle.GetString(), Tools_PFXSelect_Overlay::MODE_EFFECTS);
			xOverlay.SetSelected( xValue.GetHashIDValue() );

			if ( xOverlay.DoModal() )
			{
				xValue.SetValue( xOverlay.GetTemplateHash(), xOverlay.GetTemplateName() );
				return true;
			}
			else
			{
				return false;
			}
		}

#ifdef ASURA_USE_DECAL_MANAGER
		case TVT_DECAL:
		{
			Asura_String xWindowTitle( "Choose Decal For '" );
			xWindowTitle += xParameterName;
			xWindowTitle += "'";

			ListSelection_Overlay xOverlay( xWindowTitle.GetString() );
			xOverlay.AddItem("[None]");
			
			if(Asura_Decal_Manager::GetNumDecalSets()<0)
			{
				return false;
			}

			for ( int i=0; i<Asura_Decal_Manager::GetNumDecalSets(); ++i )
			{
				Asura_DecalManager_DecalSet* pxSet = Asura_Decal_Manager::GetDecalSetByIndex(i);
				if(pxSet)
				{
					const Asura_Char* szDecalName = pxSet->m_xSetName.GetString();
					xOverlay.AddItem( szDecalName, Asura_GetHashID(szDecalName) );
				}
			}			

			if ( !xOverlay.DoModal() ) { return false; }

			if (strcmp(xOverlay.GetSelectionString(),"[None]")==0)
			{
				xValue.SetValue(ASURA_HASH_ID_UNSET);
			}
			else
			{			
				xValue.SetValue( static_cast<Asura_Hash_ID> (xOverlay.GetSelectionData()), xOverlay.GetSelectionString() );
			}

			return true;
		}
#endif

		case TVT_RAGDOLL:
		{
			Asura_String xWindowTitle( "Choose Ragdoll Description For '" );
			xWindowTitle += xParameterName;
			xWindowTitle += "'";

			ListSelection_Overlay xOverlay( xWindowTitle.GetString() );
			xOverlay.AddItem("[None]");
#ifdef ASURA_PHYSICS_DISABLE_RAGDOLL
			return false;
#else
			const u_int uNumDescriptions = Asura_Physics_Ragdoll_System::GetNumRagdollDescriptions();
			if ( uNumDescriptions == 0 )
			{
				return false;
			}

			const char** pszRagdollDescriptionNames = new const char*[uNumDescriptions];
			Asura_Physics_Ragdoll_System::GetRagdollDescriptionNames( pszRagdollDescriptionNames );
			for ( u_int u = 0; u < uNumDescriptions; u++ )
			{
				const char* szRagdollDescriptionName = pszRagdollDescriptionNames[u];
				xOverlay.AddItem( szRagdollDescriptionName, Asura_GetHashID( szRagdollDescriptionName ) );
			}			

			if ( !xOverlay.DoModal() ) 
			{ 
				delete[] pszRagdollDescriptionNames;
				return false; 
			}

			if (strcmp(xOverlay.GetSelectionString(),"[None]")==0)
			{
				xValue.SetValue(ASURA_HASH_ID_UNSET);
			}
			else
			{			
				xValue.SetValue( static_cast<Asura_Hash_ID> (xOverlay.GetSelectionData()), xOverlay.GetSelectionString() );
			}
			delete[] pszRagdollDescriptionNames;

			return true;
#endif
		}

		case TVT_SOUNDEVENT:
		{
			Asura_String xWindowTitle( "Choose Sound Event For '" );
			xWindowTitle += xParameterName;
			xWindowTitle += "'";

			Tools_PickSoundEvent_Overlay xOverlay( xWindowTitle.GetString() );
			const Asura_Hash_ID uEventID = xOverlay.DoModal( xValue.GetHashIDValue() );

			if ( xOverlay.WasCancelled() ) { return false; }

			const Asura_Sound_Event* pxEvent = Asura_Sound_Event_Storage::GetEventByHash(uEventID);
			if( !pxEvent )
			{
				xValue.SetValue( ASURA_HASH_ID_UNSET, "*None*" );
			}
			else
			{
				xValue.SetValue( pxEvent->GetHashID(), pxEvent->GetName() );
			}
			return true;
		}

#ifdef ASURA_USE_DIALOGUE_SYSTEM
		case TVT_DIALOGUEEVENT:
		{
			Asura_String xWindowTitle( "Choose Dialogue Event For '" );
			xWindowTitle += xParameterName;
			xWindowTitle += "'";

			const Asura_Hash_ID uEventID = Tools_PickDialogueEvent_Overlay::PickEvent( xWindowTitle.GetString(), xValue.GetHashIDValue() );			
			if ( Tools_PickDialogueEvent_Overlay::WasCancelled() ) { return false; }

			if( uEventID == ASURA_HASH_ID_UNSET )
			{
				xValue.SetValue( uEventID, "*None*" );
			}
			else
			{
				const Asura_Dialogue_Event* pxEvent = Asura_Dialogue_System::FindEvent( uEventID );
				if( !pxEvent ) return false;

				xValue.SetValue( pxEvent->GetHashID(), pxEvent->GetName() );	
			}
			return true;
		}
#endif

		case TVT_TEXTURE:
		{
			// Textures are a little unusual: they store the hash ID without the extension
			// but the hashstring value does contain the extension. Something to be aware of.
			Tools_FileSystem_DialogInfo	xFileSelector;
			xFileSelector.DefaultExtension		= ".TGA";
			xFileSelector.DefaultFilename		= "";
			Asura_String xDefaultPath;
			if ( Asura_UserInput::KeyPressed( ASURA_KEYID_CTRL ) )
			{
				// If ctrl is held down whilst we open this dialogue then we should default to using the current value's path.
				Asura_String xValueString;
				xValueString = xValue.GetHashStringValue();
				Asura_String xName;
				Asura_String xExt;
				xValueString.SplitFilename( &xDefaultPath, &xName, &xExt );
				// We strip the 'graphics' off the path below, so we have to add it here.
				xDefaultPath.Prepend( "graphics", strlen( "graphics" ) );
			}
			else
			{
				// If not then we default to using the last path.
				xDefaultPath = Tools_Mode_Blueprint::GetLastTexturePath();
			}
			xFileSelector.DefaultPath			= xDefaultPath.IsBlank() ? "Graphics" : xDefaultPath.GetString();
			xFileSelector.FileFilter			= "Texture files\0*.TGA;*.BMP;*.DDS\0";
			Asura_String xWindowTitle( "Choose Texture File For '" );
			xWindowTitle += xParameterName;
			xWindowTitle += "'";
			xFileSelector.WindowTitle			= xWindowTitle.GetString();
			xFileSelector.AllowMultipleSelect	= false;

			Asura_String xFilename = Tools_FileSystem::GetLoadFilename( xFileSelector );
			if( xFilename.IsNull() )
			{
				// pop up a window asking if they want it to be added as 'unset'
				YesNo_Overlay xYesNo( "No file selected. Do you wish to add the parameter anyway? This will have the value '*Unset*'." );
				if( !xYesNo.DoModal() ) return false;

				xValue.SetValue( ASURA_HASH_ID_UNSET );
			}
			else
			{
				Asura_String xPath;
				Asura_String xName;
				Asura_String xExt;
				xFilename.SplitFilename( &xPath, &xName, &xExt );

				Tools_Mode_Blueprint::SetLastTexturePath( xPath );

				// Remove the extension
				xFilename = xPath;
				xFilename += "\\";
				xFilename += xName;

				// Go up to the graphics directory
				Asura_StringUtil::MakeLowercase(xFilename.GetString());
				const char* szOutput = strstr(xFilename.GetString(), "graphics");
				if ( !szOutput )
                {
                    Message_Overlay xMessage( "The texture must be in the \\graphics directory." );
                    xMessage.DoModal();
                    return false;
                }

				// Skip the graphics directory.
				szOutput += strlen( "graphics" );

				Asura_String xFullFilename = szOutput;
				xFullFilename += ".";
				xFullFilename += xExt;

				xValue.SetValue( Asura_GetHashID_IgnoreExtension( xFullFilename.GetString() ), xFullFilename.GetString() );
			}
			return true;
		}

		case TVT_TEXTURETILE:
		{
			Tools_FileSystem_DialogInfo	xFileSelector;
			xFileSelector.DefaultExtension		= ".TGA";
			xFileSelector.DefaultFilename		= "";
			xFileSelector.DefaultPath			= "Graphics";
			xFileSelector.FileFilter			= "Texture files\0*.TGA;*.BMP;*.DDS\0";
			Asura_String xWindowTitle( "Choose Texture File For '" );
			xWindowTitle += xParameterName;
			xWindowTitle += "'";
			xFileSelector.WindowTitle			= xParameterName.GetString();
			xFileSelector.AllowMultipleSelect	= false;

			Asura_String xFilename = Tools_FileSystem::GetLoadFilename( xFileSelector );
			if( xFilename.IsNull() )
			{
				// pop up a window asking if they want it to be added as 'unset'
				YesNo_Overlay xYesNo( "No file selected. Do you wish to add the parameter anyway? This will have the value '*Unset*'." );
				if( !xYesNo.DoModal() ) return false;

				xValue.SetValue( ASURA_HASH_ID_UNSET );
			}
			else
			{
				Asura_String xPath;
				Asura_String xName;
				Asura_String xExt;
				xFilename.SplitFilename( &xPath, &xName, &xExt );

				// Tiled textures use name plus extension
				xName += ".";
				xName += xExt;

				xValue.SetValue( Asura_GetHashID( xName.GetString() ), xName.GetString() );
			}
					
			return true;
		}

		case TVT_LOCALISED_TEXT:
		{
			Asura_String xWindowTitle( "Choose Text For '" );
			xWindowTitle += xParameterName;
			xWindowTitle += "'";

			ListSelection_Overlay xTextEnumOverlay( xWindowTitle.GetString() );
            int   iStringLength = 0;
            char* pszTextString = NULL;
            for ( u_int uTextID = 0; uTextID < Asura_LocalisedText::GetNumLoadedStrings(); ++uTextID )
            {
                const Asura_UnicodeChar* pwszTextString = Asura_LocalisedText::GetString( uTextID );
                if ( !pwszTextString ) { continue; }

                const int iNewStringLength = wcslen( pwszTextString );
                if ( iNewStringLength > iStringLength )
                {
                    delete [] pszTextString;
                    pszTextString = new char[ iNewStringLength + 1 ];
                    iStringLength = pszTextString ? iNewStringLength : 0;
                }

                if ( pszTextString )
                {
                    Asura_UnicodeToUTF8( pwszTextString, pszTextString );
                    xTextEnumOverlay.AddItem( pszTextString, uTextID );	
                }
            }
            delete [] pszTextString;

			if ( !xTextEnumOverlay.DoModal() ) { return false; }

			xValue.SetValue( static_cast<int> (xTextEnumOverlay.GetSelectionData()));
					
			return true;
		}

		case TVT_HASHED_TEXT:
		{
			Asura_String xWindowTitle( "Choose Text For '" );
			xWindowTitle += xParameterName;
			xWindowTitle += "'";

			ListSelection_Overlay xTextEnumOverlay( xWindowTitle.GetString() );

			const Asura_HashedLocalisedText::PageContainer&	xPageContainer	= Asura_HashedLocalisedText::GetPageContainer();

			for(Asura_HashedLocalisedText::PageConstIterator xIterator(xPageContainer); !xIterator.Done(); xIterator.Next())
			{
				Asura_Hash_ID	uPageHash	= xIterator.GetCurrentHash();

				const u_int uNumStrings = Asura_HashedLocalisedText::GetNumberOfStrings( uPageHash );
				for ( u_int uTextIndex = 0; uTextIndex < uNumStrings; ++uTextIndex )
				{
					const Asura_Hash_ID uEntryHash = Asura_HashedLocalisedText::GetTextIDByIndex( uPageHash, uTextIndex );
                    const char* pszTextString      = Asura_HashedLocalisedText::GetTextIDStringByIndex( uPageHash, uTextIndex );
					if ( pszTextString )
					{
						xTextEnumOverlay.AddItem( pszTextString, uEntryHash );	
					}
				}
			}
			if ( !xTextEnumOverlay.DoModal() ) { return false; }
			xValue.SetValue( static_cast< Asura_Hash_ID >( xTextEnumOverlay.GetSelectionData() ) );
			return true;
		}

		case TVT_BLUEPRINT:
		case TVT_BLUEPRINTANDCHILDREN:
		{
			// Extract blueprint type from value string, formatted as 'type.blueprint...blueprint'.
			Asura_String xTypeName = xValue.GetHashStringValue();
			while ( xTypeName.StripExtension() ) {}

			Asura_Hash_ID uBPHashID = xValue.GetHashIDValue();
			Asura_Hash_ID uTypeHashID = xTypeName.GetHash();

			if ( uBPHashID == ASURA_HASH_ID_UNSET )
			{
				Tools_Mode_Blueprint::GetLastBlueprintChoice( uTypeHashID, uBPHashID );
			}

			Asura_String xWindowTitle( "Choose Blueprint For '" );
			xWindowTitle += xParameterName;
			xWindowTitle += "'";
 			BlueprintSelect_Overlay xSelectDialog( xWindowTitle.GetString(), &uBPHashID, &uTypeHashID ); 
			if ( !xSelectDialog.DoModal() )
			{
				return false;
			}

			Tools_Mode_Blueprint::SetLastBlueprintChoice( uTypeHashID, uBPHashID );

			if ( uBPHashID == ASURA_HASH_ID_UNSET || uTypeHashID == ASURA_HASH_ID_UNSET )
			{
				xValue.SetValue( ASURA_HASH_ID_UNSET );
				return true;
			}

			const Asura_Blueprint* const pxChosenBlueprint = Asura_Blueprint_System::GetBlueprintByHashID( uTypeHashID, uBPHashID );
			if ( !pxChosenBlueprint ) return false;

			// Construct the value string in the 'type.blueprint...blueprint' format.
			Asura_String xValueString;
			const Asura_Blueprint* pxBlueprint = pxChosenBlueprint;
			while ( pxBlueprint )
			{
				xValueString.Prepend( pxBlueprint->GetName() );
				xValueString.Prepend( ".", strlen( "." ) );
				pxBlueprint = pxBlueprint->GetParent();
			}
			xValueString.Prepend( *Asura_Blueprint_System::GetTypeName( uTypeHashID ) );

			xValue.SetValue( pxChosenBlueprint->GetHashID(), xValueString.GetString() );

			return true;
		}
		
		case TVT_BLUEPRINT_TYPE:
		{
			Asura_String xTypeName = xValue.GetHashStringValue();
			Asura_Hash_ID uTypeHashID = xValue.GetHashIDValue();

			Asura_String xWindowTitle( "Choose Blueprint Type For '" );
			xWindowTitle += xParameterName;
			xWindowTitle += "'";
 			BlueprintTypeSelect_Overlay xSelectDialog( xWindowTitle.GetString(), uTypeHashID ); 
			if ( !xSelectDialog.DoModal() )
			{
				return false;
			}

			if ( uTypeHashID == ASURA_HASH_ID_UNSET )
			{
				xValue.SetValue( ASURA_HASH_ID_UNSET );
				return true;
			}

			// Construct the value string from the type
			const Asura_String* pxTypeString = Asura_Blueprint_System::GetTypeName( uTypeHashID );
			Asura_Assertf( pxTypeString, ( "Unable to get name for blueprint type 0x%08x", uTypeHashID ) );
			if ( pxTypeString )
			{
				xValue.SetValue( uTypeHashID, pxTypeString->GetString() );
			}

			return true;
		}

#ifdef ASURA_USE_FSFX
		case TVT_FSFX:
		{
			Asura_String xWindowTitle( "Choose FSFX Template For '" );
			xWindowTitle += xParameterName;
			xWindowTitle += "'";

			ListSelection_Overlay xOverlay( xWindowTitle.GetString() );
			xOverlay.AddItem("[None]");

			for( u_int uIndex = 0; uIndex<Asura_FSFX_System::GetNumTemplates(); uIndex++ )
			{
				Asura_FSFX_Template* pxTemplate = Asura_FSFX_System::GetTemplateByIndex(uIndex);

				xOverlay.AddItem( pxTemplate->GetName(), pxTemplate->GetNameHash() );
			}

			if( !xOverlay.DoModal() ) return false;

			if( strcmp( xOverlay.GetSelectionString(), "[None]" ) == 0 )
			{
				xValue.SetValue( ASURA_HASH_ID_UNSET );
			}
			else
			{
				xValue.SetValue( xOverlay.GetSelectionData(), xOverlay.GetSelectionString() );
			}
			
			return true;
		}
#endif

		case TVT_EXPORT_PLATFORMS:
		{
			Asura_String xWindowTitle( "Choose Platform For '" );
			xWindowTitle += xParameterName;
			xWindowTitle += "'";

			Asura_Overlay* pxOverlay = new Asura_Overlay( xWindowTitle.GetString(), false );
			Asura_Dialog* pxDialog = NULL;
			Asura_Vector_2 xOffset( 1.0f, 3.0f );

			bool bChanged = false;
			int iPlatformFlags = xValue.GetIntValue();

			for( int iPlatform=0; iPlatform<Tools_Level_Export::ExportType_NumPlatforms; ++iPlatform )
			{
				if( Tools_Level_Export::IsPlatformSupported( (Tools_Level_Export::EXPORT_TYPE)iPlatform ) )
				{
					pxDialog = new Asura_FlagTickbox_Dialog( Tools_Level_Export::GetPlatformName(iPlatform), xOffset, &iPlatformFlags, (1 << iPlatform), &bChanged );
					pxOverlay->AddDialog( pxDialog );
					xOffset.y += Asura_Dialog::DefaultSpacing();
				}				
			}			

			Asura_Overlay_System::SubmitNewOverlay(pxOverlay);
			Tools_Modal_Overlay::DoModal(pxOverlay);

			if( bChanged )
			{
				xValue.SetValue( iPlatformFlags );
			}			

			return true;
		}

#ifndef ASURA_DISABLE_ANIM_TABLE_CODE
		case TVT_ANIMTABLE:
		{
			Asura_String xWindowTitle( "Choose Animtable For '" );
			xWindowTitle += xParameterName;
			xWindowTitle += "'";

			ListSelection_Overlay xTextEnumOverlay( xWindowTitle.GetString() );

			const u_int uNumTables = Asura_AnimTable_System::GetNumTables();
						
			for ( u_int uTable = 0; uTable < uNumTables; ++uTable )
			{
				const Asura_Hash_ID uEntryHash = Asura_AnimTable_System::GetTableHashByIndex(uTable);
				const char*			pszTextString      = Asura_AnimTable_System::GetTableNameByIndex(uTable);
				xTextEnumOverlay.AddItem( pszTextString, uEntryHash );	
			}
			if ( !xTextEnumOverlay.DoModal() ) { return false; }
			xValue.SetValue( static_cast< Asura_Hash_ID > (xTextEnumOverlay.GetSelectionData()), xTextEnumOverlay.GetSelectionString() );
			return true;
		}
#endif

#ifdef ASURA_USE_GUI_MENU_SYSTEM
		case TVT_MENUPAGE:
		{
			// populate a tree select overlay with the menu pages
			Asura_String xWindowTitle( "Choose Menu For '" );
			xWindowTitle += xParameterName;
			xWindowTitle += "'";

			TreeSelect_Overlay xTreeOverlay( xWindowTitle.GetString() );
			for( u_int u=0; u<Asura_GUIMenu_System::GetCurrentMenuListCount(); ++u )
			{
				Asura_GUIMenu_Menu_Page* pxPage = Asura_GUIMenu_System::GetMenuPageByIndex(u);
				if( pxPage )
				{
					// make sure the parent (i.e. the file the menu is in) is present already
					xTreeOverlay.AddItem( pxPage->GetFilename() );
					// then add the page
					xTreeOverlay.AddItem( pxPage->GetNameAsChar(), pxPage->GetFilename(), (int)pxPage->GetNameHash() );
				}
			}

			if( xTreeOverlay.DoModal() )
			{
				// make sure they haven't selected a file rather than a page!
				if( xTreeOverlay.GetSelectionData() == 0 ) return false;

				xValue.SetValue( static_cast<Asura_Hash_ID> (xTreeOverlay.GetSelectionData()), xTreeOverlay.GetSelection() );
			}

			return true;
		}
#endif 

		case TVT_MARKERPOINT:
		{
			const Asura_Hierarchy_Skin* pxSkin;
			if (!Tools_Animation_Files::ChooseSkin(&pxSkin) || !pxSkin)
			{
				// pop up a window asking if they want it to be added as 'unset'
				YesNo_Overlay xYesNo("No file selected. Do you wish to add the parameter anyway? This will have the value '<UNSET>'.");
				if (!xYesNo.DoModal()) { return false; }

				xValue.SetValue(ASURA_HASH_ID_UNSET);
				return true;
			}

			ListSelection_Overlay xOverlay("Select marker point");
			for(int i=0; i<pxSkin->NumberOfMarkerPoints; ++i)
			{
				xOverlay.AddItem(pxSkin->MarkerPoint[i].GetName().GetString());
			}

			if(!xOverlay.DoModal())
			{
				// pop up a window asking if they want it to be added as 'unset'
				YesNo_Overlay xYesNo("No marker point selected. Do you wish to add the parameter anyway? This will have the value '<UNSET>'.");
				if (!xYesNo.DoModal()) { return false; }

				xValue.SetValue(ASURA_HASH_ID_UNSET);
				return true;
			}

			xValue.SetValue(Asura_GetHashID(xOverlay.GetSelectionString()), xOverlay.GetSelectionString());
					
			return true;
		}

#ifdef ASURA_USE_FACIAL_ANIMATION_SYSTEM
		case TVT_FACE:
		{
			Asura_String xWindowTitle( "Choose Face For '" );
			xWindowTitle += xParameterName;
			xWindowTitle += "'";

			Asura_Hash_ID	uValueHashID	= xValue.GetHashIDValue();
			Asura_String	xValueName		= xValue.GetHashStringValue();
			const char*		szDirectory		= Tools_FacialAnimation_System::GetFaceDirectoryName();
			const char*		szFileExtension	= Tools_FacialAnimation_System::GetFaceFileExtension();


			FileSelect_Overlay xSelectDialog( xWindowTitle.GetString(), szDirectory, szFileExtension ); 

			xSelectDialog.SelectFile( uValueHashID, false );

			if ( xSelectDialog.DoModal() )
			{
				xSelectDialog.GetSelectedFile( uValueHashID, xValueName );
				
				xValue.SetValue( uValueHashID, xValueName.GetString() );

				return true;
			}
		}

		case TVT_FACEANIMATION:
		{
			Asura_String xWindowTitle( "Choose Facial Animation For '" );
			xWindowTitle += xParameterName;
			xWindowTitle += "'";

			Asura_Hash_ID	uValueHashID	= xValue.GetHashIDValue();
			Asura_String	xValueName		= xValue.GetHashStringValue();
			const char*		szDirectory		= Tools_FacialAnimation_System::GetAnimationDirectoryName();
			const char*		szFileExtension	= Tools_FacialAnimation_System::GetAnimationFileExtension();

			FileSelect_Overlay xSelectDialog( xWindowTitle.GetString(), szDirectory, szFileExtension ); 

			xSelectDialog.SelectFile( uValueHashID, true );
			
			if ( xSelectDialog.DoModal() )
			{
				xSelectDialog.GetSelectedFileWithPath( uValueHashID, xValueName );

				xValue.SetValue( uValueHashID, xValueName.GetString() );
				return true;
			}

			return false;
		}
#endif

#if defined(ASURA_USE_CONTAINER_SYSTEM)
		case TVT_CONTAINER:
		{
			Asura_Hash_ID	uContainerHash	= ASURA_HASH_ID_UNSET;

			if(Tools_Container_System::SelectContainerModalOverlay(uContainerHash, ASURA_HASH_ID_UNSET, true, true, ASURA_HASH_ID_UNSET))
			{
				if(uContainerHash == ASURA_HASH_ID_UNSET)
				{
					xValue.SetValue(ASURA_HASH_ID_UNSET);
					return(true);
				}
				else
				{
					const Tools_Container* const	pxContainer	= Tools_Container_System::FindContainerByHash(uContainerHash);

					if(pxContainer == NULL)
					{
						return(false);
					}
					else
					{
						const Asura_String&	xContainerName	= pxContainer->GetContainerName();
						const char*			szContainerName	= xContainerName.GetString();

						xValue.SetValue(uContainerHash, szContainerName);
						return(true);
					}
				}
			}
			else
			{
				return(false);
			}
		}
#endif	//defined(ASURA_USE_CONTAINER_SYSTEM)

		case TVT_ANIMATIONFLAG:
		{
			Tools_Flag_List* pxFlagList = Tools_System::CommonFlagIni ? Tools_System::CommonFlagIni->GetFlagList("Keyframe Flags") : NULL;
			if (!pxFlagList) { return false; }

			ListSelection_Overlay xOverlay("Select animation flag");
			for (List_Iterator_Forward<NameAndValue> xIt(&(pxFlagList->ValueList)); !xIt.done(); xIt.next())
			{
				xOverlay.AddItem(xIt().Name.GetString(), xIt().Value);
			}
			xOverlay.AddItem("*None*", 0);

			if(!xOverlay.DoModal())
			{
				// pop up a window asking if they want it to be added as 'unset'
				YesNo_Overlay xYesNo("No animation flag selected. Do you wish to add the parameter anyway? This will have the value '*None*'.");
				if (!xYesNo.DoModal()) { return false; }

				xValue.SetValue(0);
				return true;
			}

			xValue.SetValue(xOverlay.GetSelectionData());
					
			return true;
		}

#ifdef ASURA_SUPPORT_SKIN_ELEMENTS
		case TVT_SKINELEMENT:
		{
			const char** szSkinElementNames = Asura_Project_Skin_Elements::GetElementNames();
			ListSelection_Overlay xOverlay("Select skin element");
			for (int i=0; i<Asura_Project_Skin_Elements::GetNumElementNames(); ++i)
			{
				xOverlay.AddItem(szSkinElementNames[i], i);
			}
			xOverlay.AddItem("*None*", -1);

			if(!xOverlay.DoModal())
			{
				// pop up a window asking if they want it to be added as 'unset'
				YesNo_Overlay xYesNo("No skin element selected. Do you wish to add the parameter anyway? This will have the value '*None*'.");
				if (!xYesNo.DoModal()) { return false; }

				xValue.SetValue(-1);
				return true;
			}

			xValue.SetValue(xOverlay.GetSelectionData());
					
			return true;
		}
#endif //#ifdef ASURA_SUPPORT_SKIN_ELEMENTS

		default:
		{
			Asura_Assertf( false, ("Unrecognised templated type: %d",iTemplatedType) );
			break;
		} 
	}

	return false;
}

bool Tools_Blueprint_Edit_Object::GetNewParameterName( const Asura_Value::VALUE_TYPE eValueType, const Asura_Hash_ID uNewTemplateHashID, const bool bAllowConversions,
                                                       Asura_String& xNewName, bool& bConvertChildrenToOverrides )
{
	const Asura_String xOriginalName( xNewName );

    Asura_Blueprint* pxBlueprint = Asura_Blueprint_System::GetBlueprintByHashID( m_uBlueprintTypeHashID, m_uBlueprintHashID );
    if ( pxBlueprint )
    {
        // Get the name, making sure that it doesn't already exist in this hierarchy.
        bool bNameOK = true;
        do 
        {
            bNameOK = true;
            GetString_Overlay xGetStringOverlay( "Enter the name of the new parameter:", xNewName.GetString(), 256 );
            if ( !xGetStringOverlay.DoModal() ) return false;
            xNewName = xGetStringOverlay.GetString();

            const Asura_Hash_ID uNewNameHashID = Asura_GetHashID( xNewName.GetString() );
            if ( uNewNameHashID == ASURA_HASH_ID_UNSET )
            {
                Message_Overlay xMessageOverlay( "I think you might have forgotten something..." );
                xMessageOverlay.DoModal();
                bNameOK = false;
                continue;
            }

			if ( xOriginalName == xNewName )
			{
				// Handle this like a cancel - no further work to do.
				return false;
			}

            if ( pxBlueprint->GetParameterByHashID( uNewNameHashID, false ) )
            {
                Message_Overlay xMessageOverlay( "This blueprint already contains a parameter with this name." );
                xMessageOverlay.DoModal();
                bNameOK = false;
                continue;
            }

            if ( pxBlueprint->GetParameterByHashID( uNewNameHashID, true ) )
            {
                Message_Overlay xMessageOverlay( "A parameter with this name already exists further up the hierarchy." );
                xMessageOverlay.DoModal();
                bNameOK = false;
                continue;
            }

            bool bDuplicateChildParameterFound = false;
            bool bDuplicateChildTypeMismatch   = false;
            const int iNumChildren = Asura_Blueprint_System::GetNumChildBlueprints( m_uBlueprintTypeHashID, pxBlueprint->GetHashID(), false );
            for ( int iChildIndex = 0; iChildIndex < iNumChildren; ++iChildIndex )
            {
                Asura_Blueprint* pxChildBlueprint =
                    Asura_Blueprint_System::GetChildBlueprint( m_uBlueprintTypeHashID, pxBlueprint->GetHashID(), false, iChildIndex );
                if ( pxChildBlueprint )
                {
                    Asura_Parameter* pxParameter = pxChildBlueprint->GetParameterByHashID( uNewNameHashID );
                    if ( pxParameter && pxParameter->GetOverrideType() == ABOT_NONE )
                    {
                        bDuplicateChildParameterFound = true;
                        bDuplicateChildTypeMismatch   = ( pxParameter->GetValue().GetValueType() != eValueType || pxParameter->GetTemplate() != uNewTemplateHashID );
                        break;
                    }
                }
            }
            if ( bDuplicateChildParameterFound )
            {
                if ( !bAllowConversions )
                {
                    // If we don't allow conversions we must reject this name.
                    Message_Overlay xMessageOverlay( "A parameter with this name already exists further down the hierarchy." );
                    xMessageOverlay.DoModal();
                    bNameOK = false;
                }
                else if ( bDuplicateChildTypeMismatch )
                {
                    // If the types don't match we can't add this full stop.
                    Message_Overlay xMessageOverlay( "A parameter with this name and a different type already exists further down the hierarchy." );
                    xMessageOverlay.DoModal();
                    bNameOK = false;
                }
                else
                {
                    // If the types match we have the choice of converting the lower parameter(s) into override(s).
                    YesNoCancel_Overlay xMessageOverlay( "A parameter with this name already exists further down the hierarchy. Do you wish to make it an override?" );
                    if ( !xMessageOverlay.DoModal() ) { return false; }
                    if ( xMessageOverlay.GetResult() )
                    {
                        bConvertChildrenToOverrides = true;
                        bNameOK = true;
                    }
                    else
                    {
                        bNameOK = false;
                    }
                }
            }
        } while ( !bNameOK );

        return bNameOK;
    }
    else
    {
        return false;
    }
}

void Tools_Blueprint_Edit_Object::CloseOverlay( int iOverlayID )
{
    Tools_Blueprint_Edit_Object* pxEditObject = reinterpret_cast< Tools_Blueprint_Edit_Object* >( iOverlayID );
    delete pxEditObject;
}

int Tools_Blueprint_Edit_Object::GetNumObjects()
{
    int iCount = 0;
    for ( Asura_List_It< Tools_Blueprint_Edit_Object > xIt( &s_xEditObjectList ); !xIt.Done(); xIt.Next(), ++iCount ) {}
    return iCount;
}

void Tools_Blueprint_Edit_Object::CalculateCurrentColumnData( const bool abColumnVisible[ CV_NUM_COLUMN_VALUES ],
                                                              int& iCurrentNumColumns,
                                                              ColumnValue ( &aeCurrentColumnType )[ CV_NUM_COLUMN_VALUES ],
                                                              const char* ( &aszCurrentColumnHeadingStrings )[ CV_NUM_COLUMN_VALUES ] )
{
    // Calculate how many columns are visible and the correct column headings.
    iCurrentNumColumns = 0;
    for ( int iColumn = 0; iColumn < CV_NUM_COLUMN_VALUES; ++iColumn )
    {
        if ( abColumnVisible[ iColumn ] )
        {
            aeCurrentColumnType[ iCurrentNumColumns ]            = static_cast< ColumnValue >( iColumn );
            aszCurrentColumnHeadingStrings[ iCurrentNumColumns ] = s_aszAllColumnHeadingStrings[ iColumn ];
            ++iCurrentNumColumns;
        }
    }
    for ( int iColumn = iCurrentNumColumns; iColumn < CV_NUM_COLUMN_VALUES; ++iColumn )
    {
        aeCurrentColumnType[ iColumn ]            = CV_NUM_COLUMN_VALUES;
        aszCurrentColumnHeadingStrings[ iColumn ] = "";
    }
}

Asura_TableColumnHeading_Dialog* Tools_Blueprint_Edit_Object::CreateTableColumnHeading( Asura_Table_Dialog& xTableDialog, const char* szName,
                                                                                        const Asura_Vector_2& xOffset, const Asura_Vector_2& xMinCellSize,
                                                                                        const u_int uColumn, void* const pxUserData )
{
    Tools_Blueprint_Edit_Object*		pxThis					= reinterpret_cast< Tools_Blueprint_Edit_Object* >( pxUserData );
	const ColumnValue					eColumnValue			= pxThis->m_aeCurrentColumnType[ uColumn ];
    Asura_TableColumnHeading_Dialog*	pxColumnHeadingDialogue = Asura_TableColumnHeading_StandardDialog::Create( xTableDialog, szName, xOffset, xMinCellSize, uColumn );

	// Enable sorting on the columns which its appropriate to...
	switch(eColumnValue)
	{
		case CV_PARAMETER_NAME:
		case CV_PARAMETER_GROUP:
		case CV_PARAMETER_TYPE:
		case CV_OVERRIDE_TYPE:
		{
			pxColumnHeadingDialogue->SetClickToSort(true);
			break;
		}

		default:
		{
			break;
		}
	}	// End of switch.

    return pxColumnHeadingDialogue;
}

void Tools_Blueprint_Edit_Object::SetupTemplatedTypes()
{
	for( u_int u=0; u<GetNumTemplatedTypes(); ++u )
	{
		const char* szTemplateName = GetTemplatedTypeName(u);
		const Asura_Hash_ID uTemplateHashID = Asura_GetHashID( szTemplateName );

		Asura_Blueprint_Template* pxTemplate = s_xTemplatedTypeTree.AddNode( uTemplateHashID );
		if( pxTemplate )
		{
			pxTemplate->SetName( szTemplateName );				

			if( !InitialiseTemplatedTypeTemplate( pxTemplate, u ) )
			{
				s_xTemplatedTypeTree.DeleteNodeByHash( uTemplateHashID );
				Asura_Assertf( false, ("Couldn't add template '%s'.", szTemplateName) );
			}
		}
	}	
}

bool Tools_Blueprint_Edit_Object::InitialiseTemplatedTypeTemplate( Asura_Blueprint_Template* pxTemplate, u_int uType )
{
	switch( uType )
	{
		case TVT_COLOUR :
		{
			pxTemplate->SetType( Asura_Value::VT_INT );
			pxTemplate->SetInitial( static_cast<int> (0xFFFFFFFF) );
			return true;
		}

		case TVT_SKIN_OR_ANIM :
		case TVT_PFX :		
#ifdef ASURA_USE_DECAL_MANAGER
		case TVT_DECAL :		
#endif
		case TVT_RAGDOLL :		
		case TVT_SOUNDEVENT :
#ifdef ASURA_USE_DIALOGUE_SYSTEM
		case TVT_DIALOGUEEVENT :
#endif
		case TVT_TEXTURE:
		case TVT_TEXTURETILE:
		case TVT_SHAPE:
		case TVT_BLUEPRINT:
		case TVT_BLUEPRINTANDCHILDREN:
		case TVT_BLUEPRINT_TYPE:
#ifdef ASURA_USE_FSFX
		case TVT_FSFX:
#endif
#ifndef ASURA_DISABLE_ANIM_TABLE_CODE
		case TVT_ANIMTABLE:
#endif
#ifdef ASURA_USE_GUI_MENU_SYSTEM
		case TVT_MENUPAGE:
#endif 
        case TVT_HASHED_TEXT:
		case TVT_MARKERPOINT:
#if defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)
		case TVT_FACE:
		case TVT_FACEANIMATION:
#endif	//defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)
#if defined(ASURA_USE_CONTAINER_SYSTEM)
		case TVT_CONTAINER:
#endif	// defined(ASURA_USE_CONTAINER_SYSTEM)
		{
            pxTemplate->SetType( Asura_Value::VT_HASH );
			return true;
		}

		case TVT_LOCALISED_TEXT:
		{
            pxTemplate->SetType( Asura_Value::VT_INT );
			return true;
		}

		case TVT_EXPORT_PLATFORMS :
		{
			pxTemplate->SetType( Asura_Value::VT_INT );

			int iInitialFlags = 0;
			for( int iPlatform=0; iPlatform<Tools_Level_Export::ExportType_NumPlatforms; ++iPlatform )
			{
				if( Tools_Level_Export::IsPlatformSupported( (Tools_Level_Export::EXPORT_TYPE)iPlatform ) )
				{
					iInitialFlags |= (1 << iPlatform);
				}
			}

			pxTemplate->SetInitial( iInitialFlags );
			return true;
		}

		case TVT_ANIMATIONFLAG:
		{
			pxTemplate->SetType(Asura_Value::VT_INT);
			pxTemplate->SetInitial(0);
			return true;
		}

#ifdef ASURA_SUPPORT_SKIN_ELEMENTS
		case TVT_SKINELEMENT:
		{
			pxTemplate->SetType(Asura_Value::VT_INT);
			pxTemplate->SetInitial(-1);
			return true;
		}
#endif //#ifdef ASURA_SUPPORT_SKIN_ELEMENTS

		default : 
		{
			Asura_Assertf( false, ("Unrecognised templated type: (%d).", uType) );
			break;
		} 
	}

	return false;
}

int Tools_Blueprint_Edit_Object::GetTemplatedTypeFromTemplateID( Asura_Hash_ID uTemplateHashID ) const
{
	for( u_int u=0; g_axTemplatedTypeNames[u].szName; ++u )
	{
		if( uTemplateHashID == Asura_GetHashID(g_axTemplatedTypeNames[u].szName) ) 
		{
			return g_axTemplatedTypeNames[u].uType;
		}
	}

    // Not convinced this should be here.
	//if( uTemplateHashID != ASURA_HASH_ID_UNSET )
	//{
	//	Asura_Assertf( false, ("Unknown templated type (hash ID: %x)",uTemplateHashID) );
	//}
	
	return -1;
}

const char* Tools_Blueprint_Edit_Object::GetTemplatedTypeName( u_int uType ) const
{
	for( u_int u=0; g_axTemplatedTypeNames[u].szName; ++u )
	{
		if( g_axTemplatedTypeNames[u].uType == uType )
		{
			return g_axTemplatedTypeNames[u].szName;
		}
	}

	return "<UNKNOWN>";
}

Asura_Blueprint_Template* Tools_Blueprint_Edit_Object::GetTemplate( Asura_Hash_ID uTemplateID ) const
{
	// check our own list of templated type templates first
	Asura_Blueprint_Template* pxTemplate = s_xTemplatedTypeTree.GetDataByHash( uTemplateID );	
	if( !pxTemplate )
	{
		// didn't find a templated type template so check the blueprint system's own registered templates
		pxTemplate = Asura_Blueprint_System::GetTemplate( uTemplateID );
	}

	return pxTemplate;			
}

void Tools_Blueprint_Edit_Object::HandleTemplatedTypeLoad( const Asura_Parameter* pxParameter )
{
	// handle using project-specific function first, if available
	if( s_pfnProjectHandleTemplatedTypeLoad 
	 && s_pfnProjectHandleTemplatedTypeLoad(pxParameter) )
	{
		return;
	}

	// Okay, this is a bit ucky as we can't use the normal GetTemplatedTypeFromTemplateID function as the edit object
	// doesn't exist at this stage. We'll have to do things manually.
	for( u_int u=0; g_axTemplatedTypeNames[u].szName; ++u )
	{
		if( pxParameter->GetTemplate() == Asura_GetHashID(g_axTemplatedTypeNames[u].szName) ) 
		{
			switch( g_axTemplatedTypeNames[u].uType )
			{
				case TVT_SOUNDEVENT :
				{
					// Need to register the sound event with the system, to make sure it thinks it's being used			
					const Asura_Value& xValue = pxParameter->GetValue();
					if( (xValue.GetHashIDValue() != ASURA_HASH_ID_UNSET) && xValue.GetHashStringValue() )
					{
						Tools_SoundEvent_System::RegisterCodeEvent( xValue.GetHashStringValue() );
					}
					
					return;
				}
				
				default : break;
			}
		}
	}
}

static const u_int uBLUEPRINT_EXPORT_OUTPUT_BUFFER_SIZE = 1024;
static char  g_szOutputBuffer[uBLUEPRINT_EXPORT_OUTPUT_BUFFER_SIZE];
u_int g_uBlueprintLogfileIndent = 0;

static void OutputToLogfile( const char* szFormat, ... )
{
	va_list xArgs;
	va_start( xArgs, szFormat );
	Asura_PrintfToBfr( g_szOutputBuffer, szFormat, xArgs, uBLUEPRINT_EXPORT_OUTPUT_BUFFER_SIZE, true ); \
	va_end( xArgs );
	Asura_SimpleLogfileOutput(( Tools_Level_Export::s_szBlueprintLogPath, g_szOutputBuffer )); \
}

#define LOG_BLUEPRINT_EXPORT( __pxBlueprint, __pxParameter, __szArgs ) \
{ \
	for( u_int u=0; u<g_uBlueprintLogfileIndent; ++u ) Asura_SimpleLogfileOutput(( Tools_Level_Export::s_szBlueprintLogPath, "   " )); \
	Asura_SimpleLogfileOutput(( Tools_Level_Export::s_szBlueprintLogPath, "From %s->%s: ", __pxBlueprint->GetName().GetString(), __pxParameter->GetName().GetString() )); \
	OutputToLogfile __szArgs; \
	Asura_SimpleLogfileOutput(( Tools_Level_Export::s_szBlueprintLogPath, "\n" )); \
} \


bool Tools_Blueprint_Edit_Object::ExportTemplatedType( List<Asura_String>& xNameList, const Asura_Parameter* pxParameter, const Asura_Blueprint* pxBlueprint, bool bSpecifiedTypeOnly, uint32 uTemplatedValueType )
{
	Asura_Assert( pxParameter && pxBlueprint, "ExportTemplatedType called with NULL parameter/blueprint." );
	if( !pxParameter || !pxBlueprint ) return false;

	// handle using project-specific function first, if available
	if( s_pfnProjectExportTemplatedType 
	 && s_pfnProjectExportTemplatedType( xNameList, pxParameter, pxBlueprint ) )
	{
		return true;
	}

	// Okay, this is a bit ucky as we can't use the normal GetTemplatedTypeFromTemplateID function as the edit object
	// doesn't exist at this stage. We'll have to do things manually.
	for( u_int u=0; g_axTemplatedTypeNames[u].szName; ++u )
	{
		if( pxParameter->GetTemplate() == Asura_GetHashID(g_axTemplatedTypeNames[u].szName) ) 
		{
			if (bSpecifiedTypeOnly == false || g_axTemplatedTypeNames[u].uType == uTemplatedValueType)
			{			
				switch( g_axTemplatedTypeNames[u].uType )
				{
					// the following templated types aren't appropriate to export or can't easily be 
					// added to the export list
					case TVT_BLUEPRINT_TYPE:
					case TVT_EXPORT_PLATFORMS:
					case TVT_COLOUR:
					case TVT_TEXTURETILE:
					case TVT_LOCALISED_TEXT:
					case TVT_HASHED_TEXT:
#ifdef ASURA_USE_DECAL_MANAGER
					case TVT_DECAL:
#endif
					case TVT_RAGDOLL:
					case TVT_MARKERPOINT:
					case TVT_ANIMATIONFLAG:
#ifdef ASURA_SUPPORT_SKIN_ELEMENTS
					case TVT_SKINELEMENT:
#endif //#ifdef ASURA_SUPPORT_SKIN_ELEMENTS
					{
						return true;
					}
					
					
					case TVT_BLUEPRINT:
					{
						if(Tools_Mode_Blueprint::AllowRecursiveExportsUsingParameters())
						{
							Asura_Value xValue;
							if( !pxBlueprint->GetParameterValueByHashID( pxParameter->GetHashID(), xValue ) ) 
							{
								Asura_Assertf( false, ("Failed to get parameter value for parameter '%s' in blueprint '%s'.", pxParameter->GetName().GetString(), pxBlueprint->GetName().GetString() ) );
								return false;
							}

							// Extract blueprint type from value string, formatted as 'type.blueprint...blueprint'.
							Asura_String xTypeName = xValue.GetHashStringValue();
							while ( xTypeName.StripExtension() ) {}

							Asura_Hash_ID uBPHashID = xValue.GetHashIDValue();
							Asura_Hash_ID uTypeHashID = xTypeName.GetHash();

							// Get this new blueprint.
							Asura_Blueprint* pxNestedBlueprint = Asura_Blueprint_System::GetBlueprintByHashID(uTypeHashID, uBPHashID);
							if(pxNestedBlueprint)
							{
								LOG_BLUEPRINT_EXPORT( pxBlueprint, pxParameter, ("Exported blueprint '%s'", pxNestedBlueprint->GetName().GetString()) );
								g_uBlueprintLogfileIndent++;
								Tools_Mode_Blueprint::AddBlueprintFilesToExport(xNameList, pxNestedBlueprint, false);
								g_uBlueprintLogfileIndent--;									
							}
						}
						return true;
					}

					case TVT_BLUEPRINTANDCHILDREN:
					{
						if(Tools_Mode_Blueprint::AllowRecursiveExportsUsingParameters())
						{
							Asura_Value xValue;
							if( !pxBlueprint->GetParameterValueByHashID( pxParameter->GetHashID(), xValue ) ) 
							{
								Asura_Assertf( false, ("Failed to get parameter value for parameter '%s' in blueprint '%s'.", pxParameter->GetName().GetString(), pxBlueprint->GetName().GetString() ) );
								return false;
							}

							// Extract blueprint type from value string, formatted as 'type.blueprint...blueprint'.
							Asura_String xTypeName = xValue.GetHashStringValue();
							while ( xTypeName.StripExtension() ) {}

							Asura_Hash_ID uBPHashID = xValue.GetHashIDValue();
							Asura_Hash_ID uTypeHashID = xTypeName.GetHash();

							// The point of this type is to export all blueprints under and including this one, so use the following function with the final bool parameter set to true.
							Tools_Mode_Blueprint::AddBlueprintFilesToExportList(xNameList, uTypeHashID, uBPHashID, true);
						}
						return true;
					}
					

					case TVT_TEXTURE :
					{
						// Texture hashes use the string Path\Name (i.e. full filename without extension)
						Asura_Value xValue;
						if( !pxBlueprint->GetParameterValueByHashID( pxParameter->GetHashID(), xValue ) ) 
						{
							Asura_Assertf( false, ("Failed to get parameter value for parameter '%s' in blueprint '%s'.", pxParameter->GetName().GetString(), pxBlueprint->GetName().GetString() ) );
							return false;
						}

						if(xValue.GetHashIDValue() == ASURA_HASH_ID_UNSET)
						{
							return(false);
						}

						int iTextureIndex = Asura_Texture_Management::GetIndexFromHashID( xValue.GetHashIDValue() );
						if( iTextureIndex == ASURA_TEXTURE_NONE )
						{
							// texture not loaded yet, so generate a filename (graphics directory relative) and add/load the texture
							Asura_String xTextureFilename = xValue.GetHashStringValue();	
							Asura_String xSearchString;
							Asura_String xExt;
							xTextureFilename.SplitFilename(NULL, &xSearchString, &xExt);

							// Previously, the hashstring value was the same as the hash ID (it didn't contain an extension)
							// The only thing to do in this case is search for the file using the basic filename
							if( xExt.IsBlank() )
							{				
								// Multiple extensions to try
								xSearchString += ".tga";

								List<Asura_String> xOutputList;
								Tools_FileSystem::FindFiles( xSearchString.GetString(), xOutputList, (TOOLS_FILEFIND_RECURSIVE | TOOLS_FILEFIND_IGNOREBACKUPDIR) );

								if (xOutputList.size() == 0)
								{
									xSearchString.SplitFilename(NULL, &xSearchString, NULL);
									xSearchString += ".bmp";
									xOutputList.empty();
									Tools_FileSystem::FindFiles( xSearchString.GetString(), xOutputList, (TOOLS_FILEFIND_RECURSIVE | TOOLS_FILEFIND_IGNOREBACKUPDIR) );

									if (xOutputList.size() == 0)
									{
										xSearchString.SplitFilename(NULL, &xSearchString, NULL);
										xSearchString += ".dds";
										xOutputList.empty();
										Tools_FileSystem::FindFiles( xSearchString.GetString(), xOutputList, (TOOLS_FILEFIND_RECURSIVE | TOOLS_FILEFIND_IGNOREBACKUPDIR) );
									}
								}

								if( xOutputList.size() == 0 )
								{
									Asura_Assert( false, "Failed to find file that matches this texture name." );										
								}
								else if( xOutputList.size() > 1 )
								{
									Asura_Assert( false, "Found multiple files that match this texture name. Using first file found." );					
									xTextureFilename = xOutputList.get_entry(0);
								}
								else
								{
									xTextureFilename = xOutputList.get_entry(0);
								}		
							}

							if( !xTextureFilename.IsBlank() )
							{						
								// remove any junk from the front of the string
								xTextureFilename.ReplaceAllChars( '/', '\\' );							

								//int iPosition = 0;
								//Asura_String xOldFilename = xTextureFilename;
								//xTextureFilename = &xOldFilename[iPosition];

								iTextureIndex = Asura_Texture_Management::AddTexture( xTextureFilename );
								Asura_Texture_Management::LoadTextures();
							}
						}

						if( iTextureIndex != ASURA_TEXTURE_NONE )
						{
							Tools_Level_Export::AddTextureToExportList( iTextureIndex );
							LOG_BLUEPRINT_EXPORT( pxBlueprint, pxParameter, ("Exported texture '%s'", Asura_Texture_Management::GetTextureName(iTextureIndex)->GetString()) );
						}						

						return true;
					}

					case TVT_SHAPE:
					{
						Asura_Value xValue;
						if( !pxBlueprint->GetParameterValueByHashID( pxParameter->GetHashID(), xValue ) ) 
						{
							Asura_Assertf( false, ("Failed to get parameter value for parameter '%s' in blueprint '%s'.", pxParameter->GetName().GetString(), pxBlueprint->GetName().GetString() ) );
							return false;
						}

						const char *szString = xValue.GetHashStringValue();
						if (szString)
						{				
							if(Tools_FileSystem::FileExists(szString))
							{		
								Tools_ShapeTable::AddShapeByFilename( szString, false);
								Asura_String xFilename = szString;
								Tools_Level_Export::AddFileToExportList( xNameList, xFilename, false);
								LOG_BLUEPRINT_EXPORT( pxBlueprint, pxParameter, ("Added shape '%s'", xFilename.GetString()) );
							}						
						}					

						return true;
					}

					case TVT_SKIN_OR_ANIM :
					{
						Asura_Value xValue;
						if( !pxBlueprint->GetParameterValueByHashID( pxParameter->GetHashID(), xValue ) ) 
						{
							Asura_Assertf( false, ("Failed to get parameter value for parameter '%s' in blueprint '%s'.", pxParameter->GetName().GetString(), pxBlueprint->GetName().GetString() ) );
							return false;
						}

						if( xValue.GetHashIDValue() == ASURA_HASH_ID_UNSET )
						{
							return true;
						}

						Asura_String xFilename = xValue.GetHashStringValue();
						Asura_String xExt;
						xFilename.SplitFilename( NULL, NULL, &xExt );
						
						if( xExt.IsBlank() )
						{
							xFilename = Asura_HashedFilename_System::GetFullFilename( xValue.GetHashIDValue() );
							if( xFilename.IsNull() || xFilename.IsBlank() )
							{
								Asura_Assertf( false, ("Failed to get filename for parameter '%s' in blueprint '%s'.\nWas searching for '%s', does this file still exist?", pxParameter->GetName().GetString(), pxBlueprint->GetName().GetString(), xValue.GetHashStringValue() ) );
								return false;	
							}
						}

						/*Tools_AsuraFileContents xFileContents;
						if( !xFileContents.LoadAsuraFile( xFilename.GetString() ) )	
						{
							Asura_Assertf( false, ("Failed to get file contents for file '%s' referenced by parameter '%s' in blueprint '%s'.\nDoes this file still exist?", xFilename.GetString(), pxParameter->GetName().GetString(), pxBlueprint->GetName().GetString()) );
							return false;
						}

						const bool bIsSkinFile = (xFileContents.GetSkinList()->size() != 0);*/

						// DF 23/10/08
						// Changed how to determine whether an ASR file contains a SKIN or an ANIM.
						// Previously we read the file to determin, but this is slow.
						// Now check the file path to determine what the file is.
						// Ideally change this to check the file extension at a later date.

						//Move to a function in Tools_Mode_Blueprint to allow for project override of the behavior
						//RAR 20/01/2009

						xFilename.MakeLowercase();
						const bool bIsSkinFile = Tools_Mode_Blueprint::IsSkinFilename_Static(xFilename.GetString());

						Tools_Level_Export::AddFileToExportList( xNameList, xFilename, bIsSkinFile );
						LOG_BLUEPRINT_EXPORT( pxBlueprint, pxParameter, ("Exported %s '%s'", (bIsSkinFile ? "skin" : "anim"), xFilename.GetString()) );
						return true;
					}

					case TVT_PFX :
					{
						Asura_Value xValue;
						if( !pxBlueprint->GetParameterValueByHashID( pxParameter->GetHashID(), xValue ) ) 
						{
							Asura_Assertf( false, ("Failed to get parameter value for '%s' in blueprint '%s'.", pxParameter->GetName().GetString(), pxBlueprint->GetName().GetString()) );
							return false;
						}

						// add pfx effect to export list
						bool bRet = Tools_Mode_PFX::AddTemplateToExportListByHash( &xNameList, xValue.GetHashIDValue(), ASURA_GUID_UNREGISTERED );
						Asura_Assertf( (bRet || xValue.GetHashIDValue() == 0 ), ("Failed to add parameter value for '%s' in blueprint '%s'.", pxParameter->GetName().GetString(), pxBlueprint->GetName().GetString()) );
						if( bRet )
						{
							LOG_BLUEPRINT_EXPORT( pxBlueprint, pxParameter, ("Exported pfx '%s'", xValue.GetHashStringValue()) );
						}
						
						return bRet;
					}

					case TVT_SOUNDEVENT :
					{		
						Asura_Value xValue;
						if( !pxBlueprint->GetParameterValueByHashID( pxParameter->GetHashID(), xValue ) ) 
						{
							Asura_Assertf( false, ("Failed to get parameter value for '%s' in blueprint '%s'.", pxParameter->GetName().GetString(), pxBlueprint->GetName().GetString()) );
							return false;
						}

						if( xValue.GetHashIDValue() == ASURA_HASH_ID_UNSET ) return true; // Nothing to export

 						const Asura_Sound_Event* pxEvent = Asura_Sound_Event_Storage::GetEventByHash( xValue.GetHashIDValue() );
						Asura_Assertf( pxEvent, ("Couldn't find sound event referenced by blueprint parameter '%s->%s', was looking for event '%s'", pxBlueprint->GetName().GetString(), pxParameter->GetName().GetString(), xValue.GetHashStringValue() ) );
						if( !pxEvent ) return false;
						if( pxEvent->IsGlobalSound() ) return true; // in global export list so no need to export again

						// add sound event (doesn't need namelist parameter - handled internally by 'level' export stuff
						Tools_Level_Export::AddSoundEventToExportList( xValue.GetHashIDValue() );
						LOG_BLUEPRINT_EXPORT( pxBlueprint, pxParameter, ("Exported sound event '%s'", pxEvent->GetName()) );
						return true;
					}

#ifdef ASURA_USE_DIALOGUE_SYSTEM
					case TVT_DIALOGUEEVENT :
					{
						Asura_Value xValue;
						if( !pxBlueprint->GetParameterValueByHashID( pxParameter->GetHashID(), xValue ) ) 
						{
							Asura_Assertf( false, ("Failed to get parameter value for '%s' in blueprint '%s'.", pxParameter->GetName().GetString(), pxBlueprint->GetName().GetString()) );
							return false;
						}

						if (xValue.GetHashIDValue() == ASURA_HASH_ID_UNSET) return true; // Nothing to export

						const Asura_Dialogue_Event* pxEvent = Asura_Dialogue_System::FindEvent( xValue.GetHashIDValue() );
						Asura_Assertf( pxEvent, ("Couldn't find dialogue event referenced by blueprint parameter '%s->%s', was looking for event '%s'", pxBlueprint->GetName().GetString(), pxParameter->GetName().GetString(), xValue.GetHashStringValue() ) );
						if( !pxEvent ) return false;
						if( pxEvent->GetIsGlobal() ) return true; // in global export list so no need to export again

						Tools_Level_Export::AddDialogueEventToExportList( xValue.GetHashIDValue() );
						LOG_BLUEPRINT_EXPORT( pxBlueprint, pxParameter, ("Exported dialogue event '%s'", pxEvent->GetName()) );
						return true;
					}		
#endif // ASURA_USE_DIALOGUE_SYSTEM:

#ifdef ASURA_USE_FSFX
					case TVT_FSFX :
					{
						const Asura_Hash_ID uNameHash = pxParameter->GetHashID();

						Asura_Value xValue;
						if( !pxBlueprint->GetParameterValueByHashID( uNameHash, xValue ) )
						{
							Asura_Assertf( false, ("Failed to get parameter value for '%s' in blueprint '%s'.",
													pxParameter->GetName().GetString(),
													pxBlueprint->GetName().GetString()) );
							return false;
						}

						if (xValue.GetHashIDValue() == ASURA_HASH_ID_UNSET)
						{
							return false;
						}

						const Asura_FSFX_Template* const pxTemplate
							= Asura_FSFX_System::FindTemplate( xValue.GetHashIDValue() );

						if( !pxTemplate )
						{
							Asura_Assertf( false, ("Cannot find FSFX template '%s' referenced by blueprint '%s' parameter '%s'.",
													xValue.GetHashStringValue(),
													pxParameter->GetName().GetString(),
													pxBlueprint->GetName().GetString()) );
							return false;
						}

						List< const Asura_FSFX_Template* const > xTemplateList;
						xTemplateList.add_entry( pxTemplate );

						Tools_Mode_FSFX::ExportTemplateList( &xNameList, xTemplateList );
						LOG_BLUEPRINT_EXPORT( pxBlueprint, pxParameter, ("Exported FSFX template '%s'", xValue.GetHashStringValue()) );

						return true;
					}
#endif // ASURA_USE_FSFX
#ifndef ASURA_DISABLE_ANIM_TABLE_CODE
					case TVT_ANIMTABLE:
					{
						Asura_Value xValue;
						if( pxBlueprint->GetParameterValueByHashID( pxParameter->GetHashID(), xValue ) )
						{
							// make sure it's been loaded
							Asura_Assertf( Asura_AnimTable_System::GetAnimTable( xValue.GetHashIDValue() ), ("Trying to export animtable '%s' for blueprint '%s' but it's not been loaded.",xValue.GetHashStringValue(),pxBlueprint->GetName()) );
							Tools_Level_Export::AddAnimTableToExportList( xValue.GetHashIDValue() );
							LOG_BLUEPRINT_EXPORT( pxBlueprint, pxParameter, ("Exported anim table '%s'", xValue.GetHashStringValue()) );
						}
						return true;
					}
#endif
#ifdef ASURA_USE_GUI_MENU_SYSTEM 
					case TVT_MENUPAGE:
					{
						// Can't really export menu pages yet but, if the gui tools were changed this is how it would work
						//Asura_Value xValue;
						//if( pxBlueprint->GetParameterValueByHashID( pxParameter->GetHashID(), xValue ) )
						//{
						//	Asura_GUIMenu_Menu_Page* pxPage = Asura_GUIMenu_System::GetMenuPageByID( xValue.GetHashIDValue() );
						//	Asura_Assertf( pxPage, ("Couldn't find menu page '%s' to export. Make sure it exists and is loaded.",xValue.GetHashStringValue()));
						//	if( pxPage )
						//	{
						//		// add page to export list
						//	}
						//}
						return true;
					}
#endif

#if defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)
					case TVT_FACE:
					{
						const Asura_Hash_ID	uParameterHash	= pxParameter->GetHashID();

						Asura_Value			xValue;

						if(pxBlueprint->GetParameterValueByHashID(uParameterHash, xValue))
						{
							const Asura_Hash_ID	uFaceHash	= xValue.GetHashIDValue();

							if(uFaceHash == ASURA_HASH_ID_UNSET)
							{
								// Its perfectly reasonable to have no face set...
								return(true);
							}
							else
							{
								const char*							szFaceName	= xValue.GetHashStringValue();
								const Tools_FacialAnimation_Face*	pxToolsFace	= Tools_FacialAnimation_System::FindOrLoadFace(szFaceName);

								if(pxToolsFace == NULL)
								{
									Asura_Assertf(false, ("Failed to find face '%s' for parameter '%s' in blueprint '%s'.",
										szFaceName, pxParameter->GetName().GetString(), pxBlueprint->GetName().GetString()));
									return(false);
								}
								else
								{
									Tools_Level_Export::AddFaceToExportList(szFaceName, uFaceHash);
									return(true);
								}
							}
						}
						else
						{
							Asura_Assertf(false, ("Failed to find parameter '%s' for blueprint '%s'.",
								pxParameter->GetName().GetString(), pxBlueprint->GetName().GetString()));
							return(false);
						}
					}

					case TVT_FACEANIMATION:
					{
						const Asura_Hash_ID	uParameterHash	= pxParameter->GetHashID();

						Asura_Value			xValue;

						if(pxBlueprint->GetParameterValueByHashID(uParameterHash, xValue))
						{
							const Asura_Hash_ID	uFaceAnimationHash	= xValue.GetHashIDValue();

							if(uFaceAnimationHash == ASURA_HASH_ID_UNSET)
							{
								// Its perfectly reasonable to have no face-animation set...
								return(true);
							}
							else
							{
								const char*									szFaceAnimationName		= xValue.GetHashStringValue();
								const Tools_FacialAnimation_FaceAnimation*	pxToolsFaceAnimation	= Tools_FacialAnimation_System::FindOrLoadAnimation(szFaceAnimationName);

								if(pxToolsFaceAnimation == NULL)
								{
									Asura_Assertf(false, ("Failed to find face-animation '%s' for parameter '%s' in blueprint '%s'.",
										szFaceAnimationName, pxParameter->GetName().GetString(), pxBlueprint->GetName().GetString()));
									return(false);
								}
								else
								{
									Tools_Level_Export::AddFaceAnimationToExportList(szFaceAnimationName, uFaceAnimationHash);
									return(true);
								}
							}
						}
						else
						{
							Asura_Assertf(false, ("Failed to find parameter '%s' for blueprint '%s'.",
								pxParameter->GetName().GetString(), pxBlueprint->GetName().GetString()));
							return(false);
						}
					}
#endif	//defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)

#if defined(ASURA_USE_CONTAINER_SYSTEM)
					case TVT_CONTAINER:
					{
						const Asura_Hash_ID	uParameterHash	= pxParameter->GetHashID();

						Asura_Value			xValue;

						if(pxBlueprint->GetParameterValueByHashID(uParameterHash, xValue))
						{
							const Asura_Hash_ID	uContainerHash	= xValue.GetHashIDValue();

							if(uContainerHash == ASURA_HASH_ID_UNSET)
							{
								return(true);
							}
							else
							{
								const Tools_Container* const	pxContainer		= Tools_Container_System::FindContainerByHash(uContainerHash);

								if(pxContainer == NULL)
								{
									Asura_Assertf(false, ("Failed to find container '%s' for parameter '%s' in blueprint '%s'.", xValue.GetHashStringValue(), pxParameter->GetName().GetString(), pxBlueprint->GetName().GetString()));
									return(false);
								}
								else
								{
									pxContainer->LoadRequiredFiles();
									pxContainer->AddFilesToExportList(xNameList);
									return(true);
								}
							}
						}
						else
						{
							Asura_Assertf(false, ("Failed to find parameter '%s' for blueprint '%s'.", pxParameter->GetName().GetString(), pxBlueprint->GetName().GetString()));
							return(false);
						}
					}
#endif	//defined(ASURA_USE_CONTAINER_SYSTEM)

					default:
					{
						// recognised but unhandled templated type
						Asura_Assertf( false, ("Failed to export templated type '%s'", g_axTemplatedTypeNames[u].szName) );
						return false;
					}
				}		// End of switch.
			}
		}
	}

	// not a recognises templated type, just return true
	return true;	
}

#undef LOG_BLUEPRINT_EXPORT

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////
