//////////////////////////////////////////////////////////////////////////////
//
// Tools_Blueprint_Edit_Object.h
//
// Created by NJD - 05 October 2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __TOOLS_BLUEPRINT_EDIT_OBJECT_H__
#define __TOOLS_BLUEPRINT_EDIT_OBJECT_H__ 1

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// asura
#include "Blueprint/Asura_Blueprint.h"
#include "Core/Asura_Hash.h"
#include "Core/Asura_HashTree.h"
#include "Core/Asura_List.h"
#include "Core/Asura_String.h"
#include "Misc/list_tem.hpp"

//////////////////////////////////////////////////////////////////////////////
//
// Tools_Blueprint_Edit_Object
//
//////////////////////////////////////////////////////////////////////////////

class Asura_Parameter;
class Asura_Blueprint_Template;
class Asura_Overlay;
class Asura_Table_Dialog;
class Asura_TableCell_Dialog;
class Asura_TableColumnHeading_Dialog;

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

struct Tools_BEO_Display_Data
{
	Tools_BEO_Display_Data()
	: m_iOverlayIndex( -1 )
	, m_xOverlayPosition( 0.0f, 0.0f )
	, m_uBlueprintTypeHashID( ASURA_HASH_ID_UNSET )
	, m_uBlueprintHashID( ASURA_HASH_ID_UNSET )
	, m_xOverlayScrollPosition( 0.0f, 0.0f )
	, m_bSortBackwards( false )
	, m_uSortIndex( 0 )
	{
	}
	int			   m_iOverlayIndex;
	Asura_Vector_2 m_xOverlayPosition;
	Asura_Hash_ID  m_uBlueprintTypeHashID;
	Asura_Hash_ID  m_uBlueprintHashID;
	Asura_Vector_2 m_xOverlayScrollPosition;
	bool		   m_bSortBackwards;
	u_int		   m_uSortIndex;
};

class Tools_Blueprint_Edit_Object
{
    friend class Tools_Blueprint_TableCell;
    friend class Tools_Blueprint_ParameterGroupColumnHeading;
    friend class Tools_Mode_Blueprint;

public:
	enum TEMPLATED_VALUE_TYPE
	{
		TVT_COLOUR = 0,
		TVT_SKIN_OR_ANIM,
		TVT_PFX,
		TVT_SOUNDEVENT,
#ifdef ASURA_USE_DIALOGUE_SYSTEM
		TVT_DIALOGUEEVENT,
#endif
		TVT_TEXTURE,
		TVT_TEXTURETILE,
		TVT_LOCALISED_TEXT,
		TVT_SHAPE,
		TVT_BLUEPRINT,
		TVT_BLUEPRINTANDCHILDREN,
		TVT_BLUEPRINT_TYPE,

#ifdef ASURA_USE_FSFX
		TVT_FSFX,
#endif
		TVT_HASHED_TEXT,
#ifdef ASURA_USE_DECAL_MANAGER
		TVT_DECAL,
#endif
		TVT_RAGDOLL,
		TVT_EXPORT_PLATFORMS,
#ifndef ASURA_DISABLE_ANIM_TABLE_CODE
		TVT_ANIMTABLE,
#endif
#ifdef ASURA_USE_GUI_MENU_SYSTEM
		TVT_MENUPAGE,
#endif
		TVT_MARKERPOINT,
#if defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)
		TVT_FACE,
		TVT_FACEANIMATION,
#endif	//defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)
#if defined(ASURA_USE_CONTAINER_SYSTEM)
		TVT_CONTAINER,
#endif	//defined(ASURA_USE_CONTAINER_SYSTEM)

		TVT_ANIMATIONFLAG,

#ifdef ASURA_SUPPORT_SKIN_ELEMENTS
		TVT_SKINELEMENT,
#endif

		END_OF_TEMPLATED_TYPES, 
		NUM_TEMPLATED_TYPES = END_OF_TEMPLATED_TYPES,
	};


    Tools_Blueprint_Edit_Object();
    virtual ~Tools_Blueprint_Edit_Object();

    virtual void Update();
    virtual void Render() {}

    void SetBlueprintTypeHashID( const Asura_Hash_ID uHashID ) { m_uBlueprintTypeHashID = uHashID; }
    void SetBlueprintHashID( const Asura_Hash_ID uHashID )     { m_uBlueprintHashID     = uHashID; }
    Asura_Hash_ID GetBlueprintTypeHashID() const { return m_uBlueprintTypeHashID; }
    Asura_Hash_ID GetBlueprintHashID()     const { return m_uBlueprintHashID; }

	void RecordOverlayState( Tools_BEO_Display_Data& xData ) const;
	void RestoreOverlayState( const Tools_BEO_Display_Data& xData );

    bool ShouldShowFullStrings() const { return m_bShowFullStrings; }

    // Sets the display string for the given value.
    virtual void GetValueString( const Asura_Value& xValue, const int iTemplatedType, Asura_String& xString );
    virtual int  GetTemplatedTypeFromTemplateID( Asura_Hash_ID uTemplateHashID ) const;
    virtual const char* GetTemplatedTypeName( u_int uType ) const;
    // Whether this parameter can have an override of this type.
    virtual bool OverrideTypeIsAllowed( const Asura_Parameter& xParameter, const Asura_Blueprint_Override_Type& eOverrideType ) const;
    virtual Asura_Blueprint_Template* GetTemplate( Asura_Hash_ID uTemplateID ) const;
    virtual bool GetValueForTemplatedType( const int iTemplatedType, const Asura_Hash_ID uTemplateHashID, const Asura_String xParameterName, Asura_Value& xValue );

    static bool CollapseString( Asura_String& xString );
    static void RemoveLeadingJunk( Asura_String& xString );
    static void HandleTemplatedTypeLoad( const Asura_Parameter* pxParameter );
    static bool ExportTemplatedType( List<Asura_String>& xNameList, const Asura_Parameter* pxParameter, const Asura_Blueprint* pxBlueprint, bool bSpecifiedTypeOnly = false, uint32 uTemplatedValueType = 0 );
    static void SetProjectTemplateHandlerFunction( bool (*pfnProjectHandleTemplatedTypeLoad)(const Asura_Parameter* pxParameter) ) { s_pfnProjectHandleTemplatedTypeLoad = pfnProjectHandleTemplatedTypeLoad; };
    static void SetProjectTemplateExportFunction( bool (*pfnProjectExportTemplatedType)(List<Asura_String>& xNameList,const Asura_Parameter* pxParameter,const Asura_Blueprint* pxBlueprint) )	{ s_pfnProjectExportTemplatedType = pfnProjectExportTemplatedType; };

    static Asura_List< Tools_Blueprint_Edit_Object >* GetList() { return &s_xEditObjectList; }

protected:
    enum ColumnValue
    {
		CV_PARAMETER_NAME,
        CV_PARAMETER_GROUP,
        CV_PARAMETER_TYPE,
        CV_INHERITED_VALUE,
        CV_OVERRIDE_TYPE,
        CV_OVERRIDE_VALUE,
        CV_FINAL_VALUE,

        CV_NUM_COLUMN_VALUES
    };

    void CreateParameterLists( Asura_Blueprint* pxBlueprint );
    void DestroyParameterLists();
	void CreateOverlay();
    void DestroyOverlay();
    void AddParameter();
    bool CanDeleteSelectedParameter() const;
	void DeleteSelectedParameter();
	bool CanRenameSelectedParameter() const;
    void RenameSelectedParameter();
    bool GetNewParameterName( const Asura_Value::VALUE_TYPE eValueType, const Asura_Hash_ID uNewTemplateHashID, const bool bAllowConversions,
                              Asura_String& xNewParam, bool& bConvertChildrenToOverrides );
    void CopyParameters(); // calls function below with result of GetBlueprint() and user input data
	static void CopyParameters( Asura_Blueprint* pxBlueprint, const int iNumToCopy, const Asura_Hash_ID uSoureTypeID, const class Tools_Modal_Overlay_BPParameterSelect* pxOverlay = NULL );
	void CutParameters();
    void PasteParameters(); // calls function below with result of GetBlueprint()
	static void PasteParameters( Asura_Blueprint* pxBlueprint, const Asura_Hash_ID uBPTypeHashID );
    void PasteOverParameters(); // calls function below with result of GetBlueprint()
	static void PasteOverParameters( Asura_Blueprint* pxBlueprint, const Asura_Hash_ID uBPTypeHashID );

    // Determines whether a matching parameter (not override) exists in either this blueprint of any of its children.
    static void ParameterExists( Asura_Blueprint* pxBlueprint, const Asura_Hash_ID uBPTypeHashID, const Asura_Hash_ID uParameterHashID,
								 bool& bParameterExists, Asura_Parameter* pxOutputParameter = NULL );
    // Determines whether a matching override exists in either this blueprint of any of its children.
	static void OverrideExists( Asura_Blueprint* pxBlueprint, const Asura_Hash_ID uBPTypeHashID, const Asura_Hash_ID uParameterHashID, bool& bOverrideExists );
    // Deletes all overrides of the parameter in the blueprint and all of its children.
	static void DeleteAllOverrides( Asura_Blueprint* pxBlueprint, const Asura_Hash_ID uBPTypeHashID, const Asura_Hash_ID uParameterHashID );
    // Converts the first matching override found in either this blueprint or in each lower branch from this blueprint into a parameter.
	static void ConvertTopOverridesToParameters( Asura_Blueprint* pxBlueprint, const Asura_Hash_ID uBPTypeHashID, const Asura_Hash_ID uParameterHashID );
    // Converts the first matching parameter found in either this blueprint or in each lower branch from this blueprint into a fixed override.
	static void ConvertTopParametersToOverrides( Asura_Blueprint* pxBlueprint, const Asura_Hash_ID uBPTypeHashID, const Asura_Hash_ID uParameterHashID );

	void SelectParameterRow( const Asura_Hash_ID uParamHashID );

    Asura_Blueprint* GetBlueprint() const;

    virtual void  AddBlueprintOverlayItems( Asura_Overlay& xNewOverlay, Asura_Vector_2& xOffset, Asura_Blueprint* pxBlueprint );
    virtual void  AddTypesToTypeOverlay( class ListSelection_Overlay& xTypeOverlay );
	virtual void  SetupTemplatedTypes( void );
	virtual u_int GetNumTemplatedTypes( void ) const { return END_OF_TEMPLATED_TYPES; }
	virtual bool  InitialiseTemplatedTypeTemplate( Asura_Blueprint_Template* pxTemplate, u_int uType );

    static void CloseOverlay( int iOverlayID );
    static int  GetNumObjects();
    static void CalculateCurrentColumnData( const bool abColumnVisible[ CV_NUM_COLUMN_VALUES ],
                                            int& iCurrentNumColumns,
                                            ColumnValue ( &aeCurrentColumnType )[ CV_NUM_COLUMN_VALUES ],
                                            const char* ( &aszCurrentColumnHeadingStrings )[ CV_NUM_COLUMN_VALUES ] );
    static Asura_TableColumnHeading_Dialog* CreateTableColumnHeading( Asura_Table_Dialog& xTableDialog, const char* szName,
                                                                      const Asura_Vector_2& xOffset, const Asura_Vector_2& xMinCellSize,
																	  const u_int uColumn,
                                                                      void* const pxUserData );
	static Asura_TableCell_Dialog* CreateTableCell( Asura_Table_Dialog& xTableDialog,
													const Asura_Vector_2& xOffset,
                                                    const Asura_Vector_2& xMinCellSize,
                                                    const u_int uRow,
                                                    const u_int uColumn,
                                                    void* const pxUserData );
    static void TableFilterChoiceChanged( int iUserData );
    static void AddParameterStatic( int iUserData );
	static bool CanDeleteSelectedParameterStatic( int iUserData );
    static void DeleteSelectedParameterStatic( int iUserData );
	static bool CanRenameSelectedParameterStatic( int iUserData );
    static void RenameSelectedParameterStatic( int iUserData );
    static void CopyParametersStatic( int iUserData );
    static void CutParametersStatic( int iUserData );
    static void PasteParametersStatic( int iUserData );
    static void PasteOverParametersStatic( int iUserData );
    static bool ParametersInPasteBuffer() { return ( s_uNumInBuffer > 0 ); }
	
	static void ClearParameterPasteConflictList();
	static void AddParameterPasteConflict( const Asura_String& xName );
	static void ReportParameterPasteConflicts( const char* szTitle );
   
	static void DestroyBufferStorage();

protected:
	Asura_Table_Dialog*	m_pxTableDialogue;

    Asura_Hash_ID  m_uBlueprintTypeHashID;
    Asura_Hash_ID  m_uBlueprintHashID;
    Asura_Vector_2 m_xBlueprintOverlayPosition;
    bool           m_bChanged;
    bool           m_bShowFullStrings;
    bool           m_bShowEditButtons;
	bool           m_bShowFilterButtons;
    u_int          m_uNumParameters;
    char**         m_ppszParameterStrings;
    // The filter choice is only used for the UI. Filtering should be done based on the filter group name.
    int            m_iFilterChoice;
    Asura_String   m_xFilterGroupName;
    bool           m_abColumnVisible[ CV_NUM_COLUMN_VALUES ];
    int            m_iCurrentNumColumns;
    ColumnValue    m_aeCurrentColumnType[ CV_NUM_COLUMN_VALUES ];
    const char*    m_aszCurrentColumnHeadingStrings[ CV_NUM_COLUMN_VALUES ];

    // Cut, copy, paste buffer.
    static Asura_Parameter* s_pxParameterBuffer;
    static u_int            s_uParameterBufferSize;
    static u_int            s_uNumInBuffer;
	static Asura_Hash_ID	s_uParameterSourceType;
	static Asura_Hash_ID    s_uParameterSourceBP;
	static Asura_NameList   s_xParameterPasteConflictList;

    static const char* s_aszAllColumnHeadingStrings[ CV_NUM_COLUMN_VALUES ];

    static Asura_Vector_2                            s_xBlueprintOverlayPosition;
    static Asura_List< Tools_Blueprint_Edit_Object > s_xEditObjectList;
	static bool (*s_pfnProjectHandleTemplatedTypeLoad)(const Asura_Parameter* pxParameter);
	static bool (*s_pfnProjectExportTemplatedType)(List<Asura_String>& xNameList,const Asura_Parameter* pxParameter,const Asura_Blueprint* pxBlueprint);

	static Asura_TemplateHashTree< Asura_Blueprint_Template > s_xTemplatedTypeTree;

private:
    // Forbid copy construction and assignment.
    Tools_Blueprint_Edit_Object( const Tools_Blueprint_Edit_Object& xOther );
    Tools_Blueprint_Edit_Object& operator=( const Tools_Blueprint_Edit_Object& xOther );
};

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

#endif //__TOOLS_BLUEPRINT_EDIT_OBJECT_H__
