//////////////////////////////////////////////////////////////////////////////
//
// Tools_Mode_Blueprint.h
//
// Created by NJD - 07 September 2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __TOOLS_MODE_BLUEPRINT_H__
#define __TOOLS_MODE_BLUEPRINT_H__ 1

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// asura
#include "Core/Asura_Maths.h"
#include "Core/Asura_String.h"

// tools
#include "Tools_Mode.h"
#include "Blueprint/Tools_Blueprint_Edit_Object.h"
#include "Interface/Asura_Tree_Dialog.h"
#include "Misc/Tools_MessageLog.h"

// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////////

class Asura_Blueprint;
template< typename T > class Asura_LinkList;
class Asura_Parameter;
class Asura_Blueprint_Parameter_ConstIterator;

class Tools_ModeDataFile;
class Tools_MessageLogAction_Blueprint;
class Tools_ProgressIndicator;

class Tools_Blueprint_PasteBuffer;
class Tools_Blueprint_PasteBuffer_Entry;


//////////////////////////////////////////////////////////////////////////////
//
// Tools_Mode_Blueprint
//
// Not a true tools mode anymore, as people requested the ability to access and edit blueprints
// whenever they wanted. Kept the name and interface as this caused the least upset to current
// projects.
//
//////////////////////////////////////////////////////////////////////////////

class Tools_Mode_Blueprint
{
	friend Tools_MessageLogAction_Blueprint;

public:
	static void LoadBlueprints();
    static void Initialise();
    static void Update();
    static void Render();
    static void Shutdown();

    static void ToggleOverlay();
    static bool OverlayIsOpen() { return s_bOverlayIsOpen; }
	static void RefreshOverlay( void );
	
	// PAZ - only expected to be used for one callback - will assert otherwise
	static void SetBlueprintChangedCallback( void( *pfnBlueprintChanged )( void ) );
	static void BlueprintChanged( void )							{ if( s_pfnBlueprintChanged != NULL )	{ s_pfnBlueprintChanged(); } }

    // This function allows other tools code to open a blueprint edit overlay, provided the blueprint 'mode' has been initialised.
	// If the parameter hash ID is set then this parameter row will be selected initially.
    static bool CreateEditOverlay( const Asura_Hash_ID uBlueprintTypeHashID, const Asura_Hash_ID uBlueprintHashID, const Asura_Hash_ID uParamHashID = ASURA_HASH_ID_UNSET );
	// This function allows you to access the edit objects created by the above
	static Tools_Blueprint_Edit_Object* GetEditObject( const Asura_Hash_ID uBlueprintTypeHashID, const Asura_Hash_ID uBlueprintHashID );

    static void RequestRefreshBlueprintEditObjectOverlays() { s_bRefreshEditObjectOverlays = true; }
    static void SetTypeAsModified( const Asura_Hash_ID uTypeHashID );
    static void SaveAll( bool bOutputAllNotifications = true );
	static void PopulateBlueprintTree( Asura_TreeDialog_Tree* pxTree, int* piNumTypes=NULL, int iNumValidTypes = 0, const Asura_Hash_ID* puValidTypesArray = NULL, int iNumValidParents = 0, const Asura_Hash_ID* puValidParentsArray = NULL );
	static void SetSelectedBlueprint( const Asura_Blueprint* pxBlueprint );

    static bool                ShouldShowFullStrings()              { return s_bShowFullStrings; }
    static bool                ShouldShowEditButtons()              { return s_bShowEditButtons; }
	static bool                ShouldShowFilterButtons()            { return s_bShowFilterButtons; }
    static const Asura_String& GetFilterGroupName()                 { return s_xFilterGroupName; }
    static bool                ColumnIsVisible( const int iColumn ) { return s_abColumnVisible[ iColumn ]; }
	static float			   GetMaxTableSizeX()					{ return s_xMaxTableSize.x; }
	static float			   GetMaxTableSizeY()					{ return s_xMaxTableSize.y; }
	static Asura_Vector_2*     GetMaxTableSizePtr()                 { return &s_xMaxTableSize; }
    static const Asura_String& GetLastSkinAnimPath()				{ return s_xLastSkinAnimPath; }
	static void				   SetLastSkinAnimPath( const Asura_String& xPath ) { s_xLastSkinAnimPath = xPath; }
	static const Asura_String& GetLastTexturePath()					{ return s_xLastTexturePath; }
	static void				   SetLastTexturePath( const Asura_String& xPath ) { s_xLastTexturePath = xPath; }
	static const Asura_String& GetLastTypeChoice()					{ return s_xLastTypeChoice; }
	static void				   SetLastTypeChoice( const Asura_String& xChoice ) { s_xLastTypeChoice = xChoice; }
	static void				   GetLastBlueprintChoice( Asura_Hash_ID& uBPTypeHashID, Asura_Hash_ID& uBPHashID ) { uBPTypeHashID = s_uLastBPTypeChoice; uBPHashID = s_uLastBPChoice; }
	static void				   SetLastBlueprintChoice( const Asura_Hash_ID uBPTypeHashID, const Asura_Hash_ID uBPHashID ) { s_uLastBPTypeChoice = uBPTypeHashID; s_uLastBPChoice = uBPHashID; }
	static bool				   AllowRecursiveExportsUsingParameters() { return s_bAllowRecursiveExportsUsingParameters; }

    static void AddParameterGroupName();
	static void SetTemplateFilterList( const Asura_Hash_ID* puTemplateList ) { s_puTemplateFilterList = puTemplateList; }

	// Export functions...
	static void AddBlueprintFilesToExportList( List<Asura_String>& xNameList, const Asura_Hash_ID uTypeHashID, const Asura_Hash_ID uParentHashID = ASURA_HASH_ID_UNSET, const bool bExportChildBlueprints = true );
	static void AddBlueprintFilesToExport( List<Asura_String>& xNameList, Asura_Blueprint* pxBlueprint, const bool bExportChildBlueprints );
	static Tools_Blueprint_Edit_Object* CreateBlueprintEditObject_Static() { return s_pxThisBlueprintMode ? s_pxThisBlueprintMode->CreateBlueprintEditObject() : NULL; }
	
	//Makes a guess as to whether a skin/anim file is a skin file based on the path
	static bool IsSkinFilename_Static(const char* szLowerCaseFilename);

protected:
    Tools_Mode_Blueprint() {}
    virtual ~Tools_Mode_Blueprint() {}

    virtual void InitialiseMode();
    virtual void ShutdownMode();
    virtual void UpdateMode();
    virtual void RenderUI();

    virtual Tools_Blueprint_Edit_Object* CreateBlueprintEditObject();

	Asura_TreeDialog_Tree* GetTree() { return &s_xTree; }

    void CreateOverlay();
    void DestroyOverlay();
    void CreateDisplayOptionsOverlay();
    void DestroyDisplayOptionsOverlay();
	void CreatePerTypeData();
    void CreateBlueprintTreeAndPerTypeData();
    void DestroyBlueprintTree();
    void SetTreeTypeNodeColours();
    virtual void AddOverlayItems( class Asura_Overlay& xNewOverlay, class Asura_Vector_2& xOffset );

    static void ContextMenuCallback( class Tools_ContextMenu& xMenu, const class Tools_ContextMenu_Context& xContext );
    virtual void AddContextMenuItems( class Tools_ContextMenu& xMenu, const class Tools_ContextMenu_Context& xContext );
	virtual void AddBlueprintFilesToExportList( List<Asura_String>& xNameList, Asura_Blueprint* pxBlueprint, const bool bExportChildBlueprints );

	static void RememberEditObjects();
	void RecreateStoredEditObjects();
	static void DestroyEditObjects();

    void TransferModeData( const int iMode );
    virtual void TransferToolsModeData( Tools_ModeDataFile& xFile );

    static bool TreeSelectionValid();
    static bool TreeSelectionIsType();
    static bool TreeSelectionIsBlueprint();

	static void RenameTreeItem(Asura_TreeDialog_Item* pxTreeItem, const char* szName);//callback for renaming items on the tree
    static void EditSelectedBlueprint();
	static void RenameSelectedType();
	static void RenameBlueprintType(Asura_TreeDialog_Item* pxTreeItem, const char* szName);
	static void UpdateUserDataRecursive(Asura_TreeDialog_Item* pxItem, Asura_Hash_ID uOldTypeID, Asura_Hash_ID uNewType);
	static void SetSelectedTypeFileName();
	static void RenameSelectedBlueprint();
	static void RenameBlueprint(Asura_TreeDialog_Item* pxTreeItem, const char* szName);
	static void CreateNewType();
    static void CreateNewBlueprint();
    static void DeleteSelected( int iSilent );
    static void TreeDoubleClicked();
    static void VerifyBlueprintInformation();
	virtual void DoVerificationTests();
	static void ExportAllBlueprints();
	static void ExportAllBlueprintsRaw();
	static void ExportBlueprintNames();

	static void SearchBlueprints();
    static void RefreshBlueprintEditObjectOverlays();

	static bool CanPasteBlueprint();
	static void CopySelected();
	static void CopySelected(Tools_Blueprint_PasteBuffer& xPasteBuffer);
	static void CutSelectedBlueprint();
	static void PasteBlueprintOrType();
	static void PasteBlueprintOrType(const Tools_Blueprint_PasteBuffer& xPasteBuffer, const Asura_TreeDialog_Item* pxParentItem);
	static void ReleasePasteBuffer();
	static void DragAndDrop(Asura_TreeDialog_Item* pxDropTarget);

	// Searches for an item with this name starting from each child of the given item.
    static Asura_TreeDialog_Item* FindByNameFromChildren( Asura_TreeDialog_Tree* pxTree, const char* pszName, Asura_TreeDialog_Item* pxParentItem );
    static void InsertTreeNode( Asura_TreeDialog_Tree* pxTree, Asura_TreeDialog_Item* pxTypeItem, const class Asura_Blueprint* pxBlueprint );

    static bool GetUniqueBlueprintName( const Asura_String& xOverlayTitle, const Asura_Hash_ID uTypeHashID, Asura_String& xName );
    static bool GetUniqueBlueprintTypeName( const Asura_String& xOverlayTitle, Asura_String& xName );
	static bool GetUniqueBlueprintTypeFileName( const Asura_String& xOverlayTitle, Asura_String& xName );

	static bool IsValidBlueprintName(const char* szOriginalName, const Asura_Hash_ID uTypeHashID, const char* szNewName);
	static bool IsValidBlueprintTypeName(const char* szNewName);

	static void ReportOrphanedFromBlueprint( const char* szTemplatedTypeName, const char* szValueString,
		const Asura_Blueprint* pxBlueprint, const Asura_Hash_ID uBlueprintTypeHashID, const Asura_Hash_ID uParamHashID = ASURA_HASH_ID_UNSET );
	static void TestForOphanedFiles(const char* szType, const char* szFilenamePrefix, const char* szFilenameSuffix, Tools_ProgressIndicator& xProgressIndicator);
#ifdef ASURA_USE_DECAL_MANAGER
	static void TestForOrphanedDecals( Tools_ProgressIndicator& xProgressIndicator );
#endif
	static void TestForOrphanedRagdollDescriptions( Tools_ProgressIndicator& xProgressIndicator );
#ifdef ASURA_USE_FSFX
	static void TestForOrphanedFSFX( Tools_ProgressIndicator& xProgressIndicator );
#endif
	static void TestForOphanedPFX( Tools_ProgressIndicator& xProgressIndicator );
	static void TestForOphanedSkinsAndAnims( Tools_ProgressIndicator& xProgressIndicator );
	static void TestForOphanedSoundEvents( Tools_ProgressIndicator& xProgressIndicator );
	static void TestForOrphanedTextures( Tools_ProgressIndicator& xProgressIndicator );
	static void TestForOrphanedTextureTiles( Tools_ProgressIndicator& xProgressIndicator );
	static void TestForOrphanedBlueprints( Tools_ProgressIndicator& xProgressIndicator );
	static void TestForOrphanedBlueprintTypes( Tools_ProgressIndicator& xProgressIndicator );
	static void TestForOrphanedLocalisedText( Tools_ProgressIndicator& xProgressIndicator );
    static void TestForAndConvertOrphanedOverrides( Tools_ProgressIndicator& xProgressIndicator );
    static void TestForAndConvertOrphanedOverrides( const Asura_Hash_ID uTypeHashID, Asura_Blueprint* pxBlueprint, bool& bFoundOrphans );
	static void CorrectBadGroupNames( Tools_ProgressIndicator& xProgressIndicator );
	static void CorrectBadGroupNames( const Asura_Hash_ID uTypeHashID, const Asura_Hash_ID uParamHashID,
									  const Asura_String& xGroupName, Asura_Blueprint* pxBlueprint );
	static void OutputBlueprintDataStats();

    static void SaveAndCloseOverlay();
    static void ReleaseStaticStorage();
    static void CloseDisplayOverlay();

    static void SetDisplayOptions();
    static void SetFilterGroupNameFromFilterChoice();
    static void SetFilterGroupChoiceFromFilterName();

    static Tools_Mode_Blueprint* Project_CreateBlueprintMode();

	//Makes a guess as to whether a skin/anim file is a skin file based on the path
	virtual bool IsSkinFilename(const char* szLowerCaseFilename);


protected:
    enum TAB_NAMES
    {
        TAB_EDITING,
        TAB_TOOLS,
        NUM_TAB_NAMES
    };
    static const char*           s_aszTabNames[ NUM_TAB_NAMES ];

    static Asura_Vector_2        s_xOverlayPosition;
    static Asura_Vector_2        s_xTreeDialogueDimensions;
    static bool                  s_bOverlayWasOnTop;
    static Asura_TreeDialog_Tree s_xTree;
    static bool                  s_bFirstUpdate;
    static bool                  s_bRefreshMainOverlay;
    static bool                  s_bRefreshDisplayOverlay;
    static bool                  s_bRefreshEditObjectOverlays;
	static bool					 s_bRefreshDisplayAndEditOverlays;

    static Asura_Vector_2        s_xDisplayOverlayPosition;
    static bool                  s_bDisplayOverlayIsOpen;

    struct ModifiedTypeHashID
    {
        bool          m_bModified;
        Asura_Hash_ID m_uTypeHashID;
    };
    static int                   s_iNumTypes;
    static ModifiedTypeHashID*   s_pxPerTypeData;
    static Tools_Mode_Blueprint* s_pxThisBlueprintMode;
    static bool                  s_bOverlayIsOpen;
    static bool                  s_bOverlayWasOpenOnModeShutdown;
	static Asura_LinkList< Tools_BEO_Display_Data > s_xEditObjectDisplayData;

	static bool					 s_bShowFullStrings;
    static bool					 s_bShowEditButtons;
	static bool                  s_bShowFilterButtons;
    static int                   s_iFilterChoice;
    static Asura_String          s_xFilterGroupName;
    static bool                  s_abColumnVisible[];
    static int					 s_iCurrentNumColumns;
    static const char*           s_aszCurrentColumnHeadingStrings[];
	static Asura_Vector_2		 s_xMaxTableSize;

    enum { LOG_DEBUG, LOG_SEARCH, LOG_VERIFICATION, LOG_ERRORS, END_LOG_SUBCATEGORIES };
    static const char*           s_pszLogCategory;
    static const char*           s_pszLogSubCategory[ END_LOG_SUBCATEGORIES ];

	static Asura_String s_xLastSkinAnimPath;
	static Asura_String s_xLastTexturePath;
	static Asura_String s_xLastTypeChoice;
	static Asura_Hash_ID s_uLastBPTypeChoice;
	static Asura_Hash_ID s_uLastBPChoice;

	static const Asura_Hash_ID* s_puTemplateFilterList;
	
	static Tools_Blueprint_PasteBuffer s_xPasteBuffer;

	static bool					s_bAllowRecursiveExportsUsingParameters;

private:
    enum { CURRENT_SAVE_VERSION = 8 };

    // Forbid copy construction and assignment.
    Tools_Mode_Blueprint( const Tools_Mode_Blueprint& xOther );
    Tools_Mode_Blueprint& operator=( const Tools_Mode_Blueprint& xOther );

    static class StaticStorageReleaser
    {
    public:
        ~StaticStorageReleaser()
        {
            Tools_Mode_Blueprint::ReleaseStaticStorage();
        }
    } s_xStaticStorageReleaser;

	// PAZ - callback for when the blueprint is changed
	static void ( *s_pfnBlueprintChanged )( void );
};

class Tools_MessageLogAction_Blueprint : public Tools_MessageLogAction
{
	typedef Tools_MessageLogAction PARENT;

public :
	Tools_MessageLogAction_Blueprint( Asura_Hash_ID uTypeID, Asura_Hash_ID uBPID, Asura_Hash_ID uParamID=ASURA_HASH_ID_UNSET );
	virtual void Execute();

private:
	Asura_Hash_ID m_uTypeID;
	Asura_Hash_ID m_uBPID;
	Asura_Hash_ID m_uParamID;
};


//Deals with storing a copy of the blueprints for copy/paste
class Tools_Blueprint_PasteBuffer
{
public :
	Tools_Blueprint_PasteBuffer(){}	
	~Tools_Blueprint_PasteBuffer(){Reset();}
	void Reset();

	bool Add(const Asura_TreeDialog_Item* pxItem);

	bool IsValid() const {return m_xList.size() > 0;}

	//Gets the number of 'root' blueprints added
	int GetNumEntries() const{return m_xList.size();}
	const Tools_Blueprint_PasteBuffer_Entry* GetEntry(int iIndex) const{return m_xList[iIndex];}
		

	//Gets the total of all the root blueprints and all their children
	int GetTotalNumberOfBlueprints() const;

private :
	Tools_Blueprint_PasteBuffer(const Tools_Blueprint_PasteBuffer&);
	Tools_Blueprint_PasteBuffer& operator = (const Tools_Blueprint_PasteBuffer&);



	List<Tools_Blueprint_PasteBuffer_Entry*> m_xList;

};


class Tools_Blueprint_PasteBuffer_Entry
{
	friend class Tools_Blueprint_PasteBuffer;
public :
	Tools_Blueprint_PasteBuffer_Entry();
	~Tools_Blueprint_PasteBuffer_Entry();

	int GetNumBlueprints() const {return m_iNumBlueprints;}
	const Asura_Blueprint* GetBlueprint(int iIndex) const {return &m_pxBlueprints[iIndex];}
	Asura_Hash_ID GetType() const {return m_uType;}

private :
	Tools_Blueprint_PasteBuffer_Entry(const Tools_Blueprint_PasteBuffer_Entry&);
	Tools_Blueprint_PasteBuffer_Entry& operator = (const Tools_Blueprint_PasteBuffer_Entry&);

	int m_iNumBlueprints;
	Asura_Blueprint* m_pxBlueprints;
	Asura_Hash_ID m_uType;
};


//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

#endif //__TOOLS_MODE_BLUEPRINT_H__
