
//////////////////////////////////////////////////////////////////////////////////////////
//
//  FILE        :  Tools_BotanicalCreation_BotanicalShape.cpp
//
//  Created by Dave Reed - 05/01/06
//
//  Copyright (C) 2005 Rebellion (Derby), All Rights Reserved.
//
//  Functions for dealing with 'BotanicalShapes' in tools mode
//
//////////////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

#ifdef ASURA_USE_BOTANICALS

#include "BotanicalCreation/Tools_BotanicalCreation_BotanicalShape.h"
#include "BotanicalCreation/Tools_BotanicalCreation_Rendering.h"
#include "BotanicalCreation/Tools_Mode_BotanicalCreation.h"
#include "Camera/Asura_Camera.h"
#include "Mesh/Asura_Mesh.h"
#include "Mesh/Asura_Mesh_System.h"
#include "Misc/Tools_MessageLog.h"
#include "Misc/Tools_Render.h"
#include "Rendering/Asura_Render.h"
#include "Rendering/Asura_Viewport.h"


// -----------------------------------------------------------------------------
// BotanicalShape Members
// -----------------------------------------------------------------------------

int Tools_BotanicalShape::GetNumElements() const
{
	int iCount=0;
	const u_int uSectionCount = this->m_xHashTree_xSections.GetNumberOfNodes();
	for( u_int uSectionIndex = 0; uSectionIndex < uSectionCount; uSectionIndex += 1)
	{
		Tools_BotanicalSection* pxToolSection = this->m_xHashTree_xSections.GetDataByIndex( uSectionIndex);
		
		const u_int uVersionCount = pxToolSection->m_xHashTree_xVersions.GetNumberOfNodes();
		for( u_int uVersionIndex = 0; uVersionIndex < uVersionCount; uVersionIndex += 1)
		{
			Tools_BotanicalVersion* pxToolVersion = pxToolSection->m_xHashTree_xVersions.GetDataByIndex( uVersionIndex);
			for(Asura_ListElt<Tools_BotanicalElement*>* pxElt = pxToolVersion->m_xList_pxElements.Front(); pxElt;pxElt=pxElt->Next())
			{
				iCount++;
			}
		}
	}

	return iCount;
}


Tools_BotanicalElement *Tools_BotanicalShape::GetElementByIndex(int iIdx)
{
	const Tools_BotanicalShape* pxShape = const_cast<const Tools_BotanicalShape*>( this);
	const Tools_BotanicalElement* pxElement = pxShape->GetElementByIndex( iIdx);
	return( const_cast< Tools_BotanicalElement*>( pxElement));
}

const Tools_BotanicalElement *Tools_BotanicalShape::GetElementByIndex(int iIdx) const
{
	int iCount=0;
	const u_int uSectionCount = this->m_xHashTree_xSections.GetNumberOfNodes();
	for( u_int uSectionIndex = 0; uSectionIndex < uSectionCount; uSectionIndex += 1)
	{
		Tools_BotanicalSection* pxToolSection = this->m_xHashTree_xSections.GetDataByIndex( uSectionIndex);
		
		const u_int uVersionCount = pxToolSection->m_xHashTree_xVersions.GetNumberOfNodes();
		for( u_int uVersionIndex = 0; uVersionIndex < uVersionCount; uVersionIndex += 1)
		{
			Tools_BotanicalVersion* pxToolVersion = pxToolSection->m_xHashTree_xVersions.GetDataByIndex( uVersionIndex);
			for(Asura_ListElt<Tools_BotanicalElement*>* pxElt = pxToolVersion->m_xList_pxElements.Front(); pxElt;pxElt=pxElt->Next())
			{
				if (iIdx==iCount)
				{
					return pxElt->Data();
				}
				iCount++;
			}
		}
	}
	return NULL;
}

Tools_BotanicalSection *Tools_BotanicalShape::GetSectionByIndex(int iIdx)
{
	const Tools_BotanicalShape* pxShape = const_cast<const Tools_BotanicalShape*>( this);
	const Tools_BotanicalSection* pxElement = pxShape->GetSectionByIndex( iIdx);
	return( const_cast< Tools_BotanicalSection*>( pxElement));
}

const Tools_BotanicalSection *Tools_BotanicalShape::GetSectionByIndex(int iIdx) const
{
	int iCount=0;
	const u_int uSectionCount = this->m_xHashTree_xSections.GetNumberOfNodes();
	for( u_int uSectionIndex = 0; uSectionIndex < uSectionCount; uSectionIndex += 1)
	{
		Tools_BotanicalSection* pxToolSection = this->m_xHashTree_xSections.GetDataByIndex( uSectionIndex);
		
		const u_int uVersionCount = pxToolSection->m_xHashTree_xVersions.GetNumberOfNodes();
		for( u_int uVersionIndex = 0; uVersionIndex < uVersionCount; uVersionIndex += 1)
		{
			Tools_BotanicalVersion* pxToolVersion = pxToolSection->m_xHashTree_xVersions.GetDataByIndex( uVersionIndex);
			for(Asura_ListElt<Tools_BotanicalElement*>* pxElt = pxToolVersion->m_xList_pxElements.Front(); pxElt;pxElt=pxElt->Next())
			{
				if (iIdx==iCount)
				{
					return( pxToolSection);
				}
				iCount++;
			}
		}
	}
	return NULL;
}

Tools_BotanicalVersion *Tools_BotanicalShape::GetVersionByIndex(int iIdx)
{
	const Tools_BotanicalShape* pxShape = const_cast<const Tools_BotanicalShape*>( this);
	const Tools_BotanicalVersion* pxElement = pxShape->GetVersionByIndex( iIdx);
	return( const_cast< Tools_BotanicalVersion*>( pxElement));
}

const Tools_BotanicalVersion *Tools_BotanicalShape::GetVersionByIndex(int iIdx) const
{
	int iCount=0;
	const u_int uSectionCount = this->m_xHashTree_xSections.GetNumberOfNodes();
	for( u_int uSectionIndex = 0; uSectionIndex < uSectionCount; uSectionIndex += 1)
	{
		Tools_BotanicalSection* pxToolSection = this->m_xHashTree_xSections.GetDataByIndex( uSectionIndex);
		
		const u_int uVersionCount = pxToolSection->m_xHashTree_xVersions.GetNumberOfNodes();
		for( u_int uVersionIndex = 0; uVersionIndex < uVersionCount; uVersionIndex += 1)
		{
			Tools_BotanicalVersion* pxToolVersion = pxToolSection->m_xHashTree_xVersions.GetDataByIndex( uVersionIndex);
			for(Asura_ListElt<Tools_BotanicalElement*>* pxElt = pxToolVersion->m_xList_pxElements.Front(); pxElt;pxElt=pxElt->Next())
			{
				if (iIdx==iCount)
				{
					return( pxToolVersion);
				}
				iCount++;
			}
		}
	}
	return NULL;
}


Tools_BotanicalShape::~Tools_BotanicalShape()
{
	const u_int uSectionCount = this->m_xHashTree_xSections.GetNumberOfNodes();
	for( u_int uSectionIndex = 0; uSectionIndex < uSectionCount; uSectionIndex += 1)
	{
		Tools_BotanicalSection* pxToolSection = this->m_xHashTree_xSections.GetDataByIndex( uSectionIndex);
		
		const u_int uVersionCount = pxToolSection->m_xHashTree_xVersions.GetNumberOfNodes();
		for( u_int uVersionIndex = 0; uVersionIndex < uVersionCount; uVersionIndex += 1)
		{
			Tools_BotanicalVersion* pxToolVersion = pxToolSection->m_xHashTree_xVersions.GetDataByIndex( uVersionIndex);
			for(Asura_ListElt<Tools_BotanicalElement*>* pxElt = pxToolVersion->m_xList_pxElements.Front(); pxElt;pxElt=pxElt->Next())
			{
				delete pxElt->Data();
			}
		}
	}
	
	this->m_xHashTree_xSections.Reset();
}

// Set some evenly spaced LOD distances, with no links or fades, as the default for new shapes
void Tools_BotanicalShape::SetDefaultLODs()
{
	const u_int uSectionCount = this->m_xHashTree_xSections.GetNumberOfNodes();
	for( u_int uSectionIndex = 0; uSectionIndex < uSectionCount; uSectionIndex += 1)
	{
		Tools_BotanicalSection* pxToolSection = this->m_xHashTree_xSections.GetDataByIndex( uSectionIndex);
		
		const u_int uVersionCount = pxToolSection->m_xHashTree_xVersions.GetNumberOfNodes();
		for( u_int uVersionIndex = 0; uVersionIndex < uVersionCount; uVersionIndex += 1)
		{
			Tools_BotanicalVersion* pxToolVersion = pxToolSection->m_xHashTree_xVersions.GetDataByIndex( uVersionIndex);
			
			for( Asura_ListElt< Tools_BotanicalElement*>* pxElt = pxToolVersion->m_xList_pxElements.Front(); NULL != pxElt ; pxElt = pxElt->Next())
			{	
				Tools_BotanicalElement *pxElement=pxElt->Data();
				int iNumLODs=pxElement->GetNumLODs();

				//  We need the end value to be fractionally more than 1, to ensure
				//  that objects won't disappear at LOD=1.0, as it uses
				//  (LOD>=fStart && LOD<fEnd) to determine whether the LOD is in range
				float fLod0=0;
				float fStep=1.0001f/((float)(iNumLODs));

				for(Asura_ListElt<Tools_BotanicalElementLOD*>* pxLODElt=pxElement->m_xLODList.Front(); pxLODElt;pxLODElt=pxLODElt->Next())
				{
					Tools_BotanicalElementLOD *pxLOD=pxLODElt->Data();

					pxLOD->m_bLinkToNext=pxLOD->m_bLinkToPrev=false;
					pxLOD->m_fFadeLength0=pxLOD->m_fFadeLength1=0;			
					pxLOD->m_fLod0=fLod0;
					fLod0+=fStep;
					pxLOD->m_fLod1=fLod0;
				}
			}	
			
		}
	}

	this->m_fInvMaxDrawDistance = 1.0f / 100.0f;
	this->m_fMaxDrawDistanceFOV = Asura_Camera::GetHorizontalFieldOfView();
	this->m_uMaxDrawDistanceRes = Asura_Viewport::GetWidth();
}


bool Tools_BotanicalShape::ContainsMesh(Asura_Mesh *pxMesh)
{
	const u_int uSectionCount = this->m_xHashTree_xSections.GetNumberOfNodes();
	for( u_int uSectionIndex = 0; uSectionIndex < uSectionCount; uSectionIndex += 1)
	{
		Tools_BotanicalSection* pxToolSection = this->m_xHashTree_xSections.GetDataByIndex( uSectionIndex);
		
		const u_int uVersionCount = pxToolSection->m_xHashTree_xVersions.GetNumberOfNodes();
		for( u_int uVersionIndex = 0; uVersionIndex < uVersionCount; uVersionIndex += 1)
		{
			Tools_BotanicalVersion* pxToolVersion = pxToolSection->m_xHashTree_xVersions.GetDataByIndex( uVersionIndex);
			
			for( Asura_ListElt< Tools_BotanicalElement*>* pxElt = pxToolVersion->m_xList_pxElements.Front(); NULL != pxElt ; pxElt = pxElt->Next())
			{	
				Tools_BotanicalElement *pxElement=pxElt->Data();
				for(Asura_ListElt<Tools_BotanicalElementLOD*>* pxLODElt=pxElement->m_xLODList.Front(); pxLODElt;pxLODElt=pxLODElt->Next())
				{
					Tools_BotanicalElementLOD *pxLOD=pxLODElt->Data();

					if (pxLOD->m_pxMesh==pxMesh)
					{
						return true;
					}

					if (pxElement->m_eType==TOOLS_BOTANICALS_ELEMENTTYPE_IMPOSTER)
					{
						for(int i=0;i<pxLOD->GetNumImposters();i++)
						{
							if (pxLOD->GetImposterByIndexConst(i)->m_pxMesh==pxMesh)
							{
								return true;
							}
						}
					}				
				}
			}
		}
	}
	return false;
}

void Tools_BotanicalShape::DiscardMeshes(void)
{
	const u_int uSectionCount = this->m_xHashTree_xSections.GetNumberOfNodes();
	for( u_int uSectionIndex = 0; uSectionIndex < uSectionCount; uSectionIndex += 1)
	{
		Tools_BotanicalSection* pxToolSection = this->m_xHashTree_xSections.GetDataByIndex( uSectionIndex);
		
		const u_int uVersionCount = pxToolSection->m_xHashTree_xVersions.GetNumberOfNodes();
		for( u_int uVersionIndex = 0; uVersionIndex < uVersionCount; uVersionIndex += 1)
		{
			Tools_BotanicalVersion* pxToolVersion = pxToolSection->m_xHashTree_xVersions.GetDataByIndex( uVersionIndex);
			
			for( Asura_ListElt< Tools_BotanicalElement*>* pxElt = pxToolVersion->m_xList_pxElements.Front(); NULL != pxElt ; pxElt = pxElt->Next())
			{	
				Tools_BotanicalElement *pxElement=pxElt->Data();
				for(Asura_ListElt<Tools_BotanicalElementLOD*>* pxLODElt=pxElement->m_xLODList.Front(); pxLODElt;pxLODElt=pxLODElt->Next())
				{
					Tools_BotanicalElementLOD *pxLOD=pxLODElt->Data();

					if (pxLOD->m_pxMesh)
					{
						Asura_Mesh_System::DeleteMesh(pxLOD->m_pxMesh);
						pxLOD->m_pxMesh=NULL;
					}
					//Check for meshes used by imposters
					for(Asura_ListElt<Tools_Botanicals_Imposter*>* pxImposterElt=pxLOD->m_xImposterList.Front(); pxImposterElt; pxImposterElt=pxImposterElt->Next())
					{
						Tools_Botanicals_Imposter* pxImposter = pxImposterElt->Data();
						if(pxImposter->m_pxMesh)
						{
							Asura_Mesh_System::DeleteMesh(pxImposter->m_pxMesh);
							pxImposter->m_pxMesh=NULL;
						}
					}
				}
			}
		}
	}
}


// Create the renderables for this element
void Tools_BotanicalShape::UpdateRenderables(void)
{
	const u_int uSectionCount = this->m_xHashTree_xSections.GetNumberOfNodes();
	for( u_int uSectionIndex = 0; uSectionIndex < uSectionCount; uSectionIndex += 1)
	{
		Tools_BotanicalSection* pxToolSection = this->m_xHashTree_xSections.GetDataByIndex( uSectionIndex);
		
		const u_int uVersionCount = pxToolSection->m_xHashTree_xVersions.GetNumberOfNodes();
		for( u_int uVersionIndex = 0; uVersionIndex < uVersionCount; uVersionIndex += 1)
		{
			Tools_BotanicalVersion* pxToolVersion = pxToolSection->m_xHashTree_xVersions.GetDataByIndex( uVersionIndex);
			
			for( Asura_ListElt< Tools_BotanicalElement*>* pxElt = pxToolVersion->m_xList_pxElements.Front(); NULL != pxElt ; pxElt = pxElt->Next())
			{	
				Tools_BotanicalElement *pxElement=pxElt->Data();
				pxElement->UpdateRenderables();
			}
		}
	}
}


// Simpler but slower render function, for the tools
void Tools_BotanicalShape::Render(Asura_Vector_3 *pxPosition, Asura_Matrix_3x3 *pxOrientation, float fScale, uint32 uFlags)
{
	const u_int uSectionCount = this->m_xHashTree_xSections.GetNumberOfNodes();
	for( u_int uSectionIndex = 0; uSectionIndex < uSectionCount; uSectionIndex += 1)
	{
		Tools_BotanicalSection* pxToolSection = this->m_xHashTree_xSections.GetDataByIndex( uSectionIndex);
		
		const u_int uVersionCount = pxToolSection->m_xHashTree_xVersions.GetNumberOfNodes();
		for( u_int uVersionIndex = 0; uVersionIndex < uVersionCount; uVersionIndex += 1)
		{
			Tools_BotanicalVersion* pxToolVersion = pxToolSection->m_xHashTree_xVersions.GetDataByIndex( uVersionIndex);
			
			for( Asura_ListElt< Tools_BotanicalElement*>* pxElt = pxToolVersion->m_xList_pxElements.Front(); NULL != pxElt ; pxElt = pxElt->Next())
			{	
			}
		}
	}
}



// -----------------------------------------------------------------------------
// BotanicalElement Members
// -----------------------------------------------------------------------------

Tools_BotanicalElement::Tools_BotanicalElement()
{
	m_eType=TOOLS_BOTANICALS_ELEMENTTYPE_2SIDED_MESH;
}


int Tools_BotanicalElement::GetNumLODs() const
{
	int iCount=0;
	for(Asura_ListElt<Tools_BotanicalElementLOD*>* pxElt=m_xLODList.Front(); pxElt;pxElt=pxElt->Next())
	{
		iCount++;
	}
	return iCount;
}


Tools_BotanicalElementLOD *Tools_BotanicalElement::GetLODByIndex(int iIdx)
{
	int iCount=0;
	for(Asura_ListElt<Tools_BotanicalElementLOD*>* pxElt=m_xLODList.Front(); pxElt;pxElt=pxElt->Next())
	{
		if (iIdx==iCount)
		{
			return pxElt->Data();
		}
		iCount++;
	}
	return NULL;
}


const Tools_BotanicalElementLOD *Tools_BotanicalElement::GetLODByIndex(int iIdx) const
{
	int iCount=0;
	for(Asura_ListElt<Tools_BotanicalElementLOD*>* pxElt=m_xLODList.Front(); pxElt;pxElt=pxElt->Next())
	{
		if (iIdx==iCount)
		{
			return pxElt->Data();
		}
		iCount++;
	}
	return NULL;
}

Tools_BotanicalElementLOD *Tools_BotanicalElement::GetLODByName( const char* pszLODName)
{
	const Tools_BotanicalElement* pxElement = const_cast< const Tools_BotanicalElement*>( this);
	const Tools_BotanicalElementLOD* pxLOD = pxElement->GetLODByName( pszLODName);
	
	return( const_cast< Tools_BotanicalElementLOD*>( pxLOD));
}

const Tools_BotanicalElementLOD *Tools_BotanicalElement::GetLODByName( const char* pszLODName) const
{
	for(Asura_ListElt<Tools_BotanicalElementLOD*>* pxElt=m_xLODList.Front(); pxElt;pxElt=pxElt->Next())
	{
		Tools_BotanicalElementLOD *pxLOD=pxElt->Data();
		
		if( pxLOD->m_xName == pszLODName)
		{
			return pxElt->Data();
		}
	}
	return NULL;
}


Tools_BotanicalElement::~Tools_BotanicalElement()
{
	// Element destructor - need to delete any LODs
	for(Asura_ListElt<Tools_BotanicalElementLOD*>* pxElt=m_xLODList.Front(); pxElt;pxElt=pxElt->Next())
	{
		delete pxElt->Data();
	}
	m_xLODList.Clear();
}



void Tools_BotanicalElement::UpdateRenderables()
{	
	for(Asura_ListElt<Tools_BotanicalElementLOD*>* pxElt=m_xLODList.Front(); pxElt;pxElt=pxElt->Next())
	{
		Tools_BotanicalElementLOD *pxLOD=pxElt->Data();

		if (pxLOD->m_pxRenderable!=NULL)
		{
			delete pxLOD->m_pxRenderable;
			pxLOD->m_pxRenderable=NULL;
		}

		switch (m_eType)
		{
			case TOOLS_BOTANICALS_ELEMENTTYPE_1SIDED_MESH:
			case TOOLS_BOTANICALS_ELEMENTTYPE_2SIDED_MESH:
				pxLOD->m_pxRenderable=new Tools_BotanicalRenderable_Mesh(pxLOD);
				break;

			case TOOLS_BOTANICALS_ELEMENTTYPE_BILLBOARDS:
				pxLOD->m_pxRenderable=new Tools_BotanicalRenderable_Billboards(pxLOD);
				break;

			case TOOLS_BOTANICALS_ELEMENTTYPE_IMPOSTER:
				pxLOD->m_pxRenderable=new Tools_BotanicalRenderable_Imposters(pxLOD);
				break;
		}
	}	
}


// -----------------------------------------------------------------------------
// BotanicalElementLOD Members
// -----------------------------------------------------------------------------

Tools_BotanicalElementLOD::Tools_BotanicalElementLOD()
{
	m_pxMesh=NULL;
	m_pxRenderable=NULL;
	m_eLightingType=TOOLS_BOTANICALS_LIGHTINGTYPE_NORMALS;
}


Tools_BotanicalElementLOD::~Tools_BotanicalElementLOD()
{
	if (m_pxRenderable)
	{
		delete m_pxRenderable;
		m_pxRenderable=NULL;
	}

	// If there's anything in the imposter list, delete it
	for(Asura_ListElt<Tools_Botanicals_Imposter*> *pxElt=m_xImposterList.Front();pxElt;pxElt=pxElt->Next())
	{
		Tools_Botanicals_Imposter *pxImposter=pxElt->Data();
		delete pxImposter;		
	}

	m_xImposterList.Clear();
}


void Tools_BotanicalElementLOD::AddImposter(Asura_Mesh* pxMesh)
{
	Tools_Botanicals_Imposter* pxImposter = new Tools_Botanicals_Imposter();
	
	pxImposter->m_pxMesh = pxMesh;

	// Some fairly arbitrary defaults
	pxImposter->m_fAngleRange = 90;
	pxImposter->m_fFadeRange  = 20;
	pxImposter->m_bValid      = false;

	if (!pxMesh->AreNormalsValid())
	{
		pxMesh->CreateNormals();
	}

	for(int i=0;i<pxMesh->NumberOfPolygons;i++)
	{
		if (!pxImposter->m_bValid && pxMesh->PolygonDataPtr[i].IsQuad())
		{
			pxImposter->m_xBillboard.xNormal = pxMesh->NormalsDataPtr[i].FaceNormal;
#if defined( ASURA_USE_MATERIAL_RESPONSE_SYSTEM )
			pxImposter->m_uMaterialResponseHashID = pxMesh->TextureDataPtr[i].m_auMaterialResponseHashID[0];
#endif
			pxImposter->m_iTextureIndexID = Asura_Material_System::GetTextureID(pxMesh->TextureDataPtr[i].m_iMaterialID);

			Asura_Vector_3 xCentre=Asura_Vector_3(0,0,0);			
			
			Tools_Mode_BotanicalCreation::CreateBillboardFromQuad(&pxImposter->m_xBillboard,pxMesh,i);
			
			// Adjust the origin to the bottom
			pxImposter->m_xBillboard.xPos.y+=pxImposter->m_xBillboard.fHeight*0.5f;

			pxImposter->m_bValid=true;
		}		
	}

	// Add it to the list
	m_xImposterList.Push_Back(pxImposter);
}


int Tools_BotanicalElementLOD::GetNumImposters(void) const
{
	int iCount=0;

	for(Asura_ListElt<Tools_Botanicals_Imposter *>*pxElt=m_xImposterList.Front();pxElt;pxElt=pxElt->Next())
	{
		iCount++;
	}

	return iCount;
}


Tools_Botanicals_Imposter* Tools_BotanicalElementLOD::GetImposterByIndex(int iIndex)
{
	int iCount=0;
	for(Asura_ListElt<Tools_Botanicals_Imposter *>*pxElt=m_xImposterList.Front();pxElt;pxElt=pxElt->Next())
	{
		if (iCount==iIndex)
		{
			return pxElt->Data();
		}
		else
		{
			iCount++;
		}		
	}

	return NULL;
}


// Yes. This is completely identical to the above function, but declared const... 
const Tools_Botanicals_Imposter* Tools_BotanicalElementLOD::GetImposterByIndexConst(int iIndex) const
{
	int iCount=0;
	for(Asura_ListElt<Tools_Botanicals_Imposter *>*pxElt=m_xImposterList.Front();pxElt;pxElt=pxElt->Next())
	{
		if (iCount==iIndex)
		{
			return pxElt->Data();
		}
		else
		{
			iCount++;
		}		
	}

	return NULL;
}


void Tools_BotanicalElementLOD::Render(TOOLS_BOTANICALS_ELEMENTTYPE eType,float fLOD,Asura_Vector_3* pxPosition, Asura_Matrix_3x3 *pxOrientation, float fScale, uint32 uFlags)
{
	if (!m_pxRenderable)
	{
		// Nothing to render
		return;
	}

	// Do the LOD calculations here, then draw the renderable
	if (fLOD>=m_fLod0 && fLOD<m_fLod1)
	{	
		float fFade=1.0f;

		float fVal=fLOD-m_fLod0;
		if (fVal<m_fFadeLength0)
		{
			fFade=fVal/m_fFadeLength0;
		}

		fVal=m_fLod1-fLOD;
		if (fVal<m_fFadeLength1)
		{
			fFade=fVal/m_fFadeLength1;
		}
		
		m_pxRenderable->Render(fFade,pxPosition,pxOrientation,fScale,uFlags);
	}
}

#if defined( ASURA_BOTANICAL_SUPPORT_STATES) && 1 == ASURA_BOTANICAL_SUPPORT_STATES
bool Tools_BotanicalShape::ValidateStateTable()
{
	for( const Asura_ListElt< Tools_Botanical_StateTable1D>* pxItStateTable = this->m_xList_xStateTables1D.Front(); NULL != pxItStateTable; pxItStateTable = pxItStateTable->Next())
	{
		const Tools_Botanical_StateTable1D& xToolStateTable = pxItStateTable->Data();
		
		for( Asura_ListElt< Tools_Botanical_StateTable1D_Section>* pxItStateTableSection = xToolStateTable.m_xList_xSection.Front(); NULL != pxItStateTableSection; pxItStateTableSection = pxItStateTableSection->Next())
		{
			Tools_Botanical_StateTable1D_Section& xToolStateTableSection = pxItStateTableSection->Data();
			
			Tools_BotanicalSection* pxShapeSection = this->m_xHashTree_xSections.GetDataByName( xToolStateTableSection.m_xTargetSection.GetString());
			if( NULL == pxShapeSection)
			{
				Tools_MessageLog::AddMessage( "Botanicals",
					"Missing State Table Section",
					"Error: botanical shape '%s' does not contain section '%s'. This section is present in state '%s'",
					this->m_xName.GetString(), xToolStateTableSection.m_xTargetSection.GetString(), xToolStateTable.m_xSourceState.GetString());

				Tools_MessageLog::ShowOverlay( "Botanicals", "Bad State Table" );

				continue;
			}
			
			for( Asura_ListElt< Tools_Botanical_StateTable1D_Section_VersionRange>* pxItStateTableSectionVersion = xToolStateTableSection.m_xList_xVersionRanges.Front(); NULL != pxItStateTableSectionVersion; pxItStateTableSectionVersion = pxItStateTableSectionVersion->Next())
			{
				Tools_Botanical_StateTable1D_Section_VersionRange& xToolStateTableSectionVersion = pxItStateTableSectionVersion->Data();
				
				Tools_BotanicalVersion* pxShapeSectionVersion = pxShapeSection->m_xHashTree_xVersions.GetDataByName( xToolStateTableSectionVersion.m_xTargetVersion.GetString());
				if( NULL == pxShapeSectionVersion)
				{
					Tools_MessageLog::AddMessage( "Botanicals",
						"Missing State Table Version",
						"Error: botanical shape '%s' does not contain version '%s' in section '%s' . This section version is present in state '%s'",
						this->m_xName.GetString(), 
						xToolStateTableSectionVersion.m_xTargetVersion.GetString(), xToolStateTableSection.m_xTargetSection.GetString(),
						xToolStateTable.m_xSourceState.GetString()
						);

					Tools_MessageLog::ShowOverlay( "Botanicals", "Bad State Table" );

					continue;
				}
			}
		}
	}

	return( true);	
}
#endif


#endif // ASURA_USEBOTANICALS
