//////////////////////////////////////////////////////////////////////////////////////////
//
//  FILE        :  Tools_Mode_BotanicalCreation.cpp
//
//  Created by Dave Reed - 04/01/06
//
//  Copyright (C) 2005 Rebellion (Derby), All Rights Reserved.
//
//  Mode for creating 'BotanicalShapes' - setting up LODs, etc
//
//////////////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

#ifdef ASURA_USE_BOTANICALS

#include "Rendering/Asura_Viewport.h"
#include "Rendering/Asura_Render.h"

#include "Tools_Overlays.h"
#include "Misc/Tools_Modal_Overlay.h"

#include "Interface/Asura_Interface.h"
#include "Interface/Asura_CommandConsole.h"
#include "Interface/Asura_Overlay.h"
#include "Interface/Asura_Overlay_System.h"
#include "Interface/Asura_Dialog.h"

#include "Rendering/Asura_Render.h"
#include "Rendering/Asura_Direct3d.h"
#include "Rendering/Asura_Direct3d_Shaders.h"

#include "Camera/Asura_Camera.h"

#include "BotanicalCreation/Tools_BotanicalCreation_BotanicalShape.h"
#include "BotanicalCreation/Tools_Mode_BotanicalCreation.h"
#include "BotanicalCreation/Tools_BotanicalCreation_EditLODs.h"


Asura_Vector_2	Tools_BotanicalCreation_EditLODs::s_xOverlayPosition=Asura_Vector_2(4,24);
bool			Tools_BotanicalCreation_EditLODs::s_bOverlayOpen=false;
int				Tools_BotanicalCreation_EditLODs::s_iNewSelectedElement=0;	// The 'new' ones are those referenced by the UI
int				Tools_BotanicalCreation_EditLODs::s_iSelectedElement=0;
int				Tools_BotanicalCreation_EditLODs::s_iNewSelectedLOD=0;
int				Tools_BotanicalCreation_EditLODs::s_iSelectedLOD=0;
float			Tools_BotanicalCreation_EditLODs::s_fTemp=0;
bool			Tools_BotanicalCreation_EditLODs::s_bTemp=false;

float           Tools_BotanicalCreation_EditLODs::s_fNewLod0=0;
float           Tools_BotanicalCreation_EditLODs::s_fNewLod1=0;
float           Tools_BotanicalCreation_EditLODs::s_fNewFadeLength0=0;
float           Tools_BotanicalCreation_EditLODs::s_fNewFadeLength1=0;

Asura_Vector_2	Tools_BotanicalCreation_EditLODs::s_xImpostersOverlayPosition=Asura_Vector_2(4,24);
bool			Tools_BotanicalCreation_EditLODs::s_bImpostersOverlayOpen=false;
int				Tools_BotanicalCreation_EditLODs::s_iNewCurrentImposter=0;
int				Tools_BotanicalCreation_EditLODs::s_iCurrentImposter=0;

float			Tools_BotanicalCreation_EditLODs::s_fMaxDrawDistance = 100.0f;

Tools_BotanicalElementLOD* Tools_BotanicalCreation_EditLODs::s_pxImposterLOD=NULL;




// The dialog used for rendering our LOD graph

Tools_BotanicalCreation_LODGraph_Dialog::Tools_BotanicalCreation_LODGraph_Dialog( const Asura_Vector_2& xOffset, float fWidth, float fHeight)
: Asura_Dialog( "", xOffset )
, m_xSize( fWidth, fHeight )
{
	m_xMaxCoord.x = xOffset.x + (fWidth  / ASURA_TEXT_OUTPUT_FONT_WIDTH);
	m_xMaxCoord.y = xOffset.y + (fHeight / ASURA_TEXT_OUTPUT_FONT_HEIGHT);
}


void Tools_BotanicalCreation_LODGraph_Dialog::Render( const Asura_Vector_2& xOrigin )
{
	if( !IsHidden() )
	{
		Asura_Vector_2 xPosition = xOrigin + m_xOffset;

		xPosition.x *= ASURA_TEXT_OUTPUT_FONT_WIDTH;
		xPosition.y *= ASURA_TEXT_OUTPUT_FONT_HEIGHT;

		Tools_BotanicalCreation_EditLODs::RenderGraph( xPosition.x, xPosition.y, m_xSize.x, m_xSize.y );				
	}
}



// -------------------------------------------------------------------------------------
// Reset the LOD editing system
// -------------------------------------------------------------------------------------

void Tools_BotanicalCreation_EditLODs::Reset()
{
	s_iNewSelectedElement=s_iSelectedElement=0;
	s_iNewSelectedLOD=s_iSelectedLOD=0;
	s_fMaxDrawDistance = 100.0f;
	Asura_BotanicalShape* pxAsuraShape = Tools_Mode_BotanicalCreation::GetCurrentBotanicalShape();
	Asura_Assert( NULL != pxAsuraShape, "No Current Botanical Shape");
	if( NULL != pxAsuraShape)
	{
		Tools_BotanicalShape* pxToolsShape = pxAsuraShape->GetToolsShape();

		if((NULL != pxToolsShape) && (pxToolsShape->m_fInvMaxDrawDistance >= 0.001f) && (pxToolsShape->m_fInvMaxDrawDistance <= 0.1f))
		{
			s_fMaxDrawDistance = 1.0f / pxToolsShape->m_fInvMaxDrawDistance;
		}
	}

	if (IsActive())
	{
		CloseOverlay();		
		CloseImpostersOverlay();
	}	
}


// -------------------------------------------------------------------------------------
// Create the LOD-editing overlay
// -------------------------------------------------------------------------------------

void Tools_BotanicalCreation_EditLODs::CreateOverlay()
{
	CloseOverlay();
	s_bOverlayOpen=true;

	Asura_Overlay* pxNewOverlay;
	Asura_Dialog*  pxNewDialog;
	Asura_Vector_2 xOffset = Asura_Vector_2(1.0f,3.0f);

	pxNewOverlay = new Asura_Overlay("LOD Control",false,&s_xOverlayPosition);
	pxNewOverlay->SetAlwaysFullBright(true);

	// Layout should look like:
	//
	// Element <n>   LOD <n>
	//
	//              (graph space)
	//
	// start <n>      fadein<n>     [x] link to prev
	// end   <n>      fadeout<n>    [x] link to next
	//
	// Close

	Asura_BotanicalShape*       pxAsuraShape=Tools_Mode_BotanicalCreation::GetCurrentBotanicalShape();
	Tools_BotanicalShape*       pxShape=pxAsuraShape->GetToolsShape();
	Tools_BotanicalElement*     pxElement=pxShape->GetElementByIndex(s_iSelectedElement);
	Tools_BotanicalElementLOD*  pxLOD=pxElement->GetLODByIndex(s_iSelectedLOD);


	s_fNewLod0        = pxLOD->m_fLod0;
	s_fNewLod1        = pxLOD->m_fLod1;
	s_fNewFadeLength0 = pxLOD->m_fFadeLength0;
	s_fNewFadeLength1 = pxLOD->m_fFadeLength1;

	// Add the element/LOD selectors
	pxNewDialog = new Asura_Numerical_Dialog("Element",xOffset,&s_iNewSelectedElement,0,pxShape->GetNumElements()-1,1,"%d");	
	xOffset.x+=14*Asura_Dialog::DefaultSpacing();	
	pxNewOverlay->AddDialog(pxNewDialog);
	
	if (pxElement->m_eType==TOOLS_BOTANICALS_ELEMENTTYPE_IMPOSTER)
	{
		pxNewDialog = new Asura_Function_Dialog("Edit Imposters",xOffset,EditImposters);		
	}
	else
	{
		const char *aszTmp[3]={	"Element Type: Single-sided mesh",
								"Element Type: Double-sided mesh",
								"Element Type: Billboards"};
		
		pxNewDialog = new Asura_Function_Dialog(aszTmp[(uint32)pxElement->m_eType],xOffset,ChangeElementType);
	}

	pxNewOverlay->AddDialog(pxNewDialog);	

#if defined( ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS) && 1 == ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS	
	{
		Asura_Vector_2 xOffsetStartSections = xOffset;
		
		xOffset.y+=1.0f*Asura_Dialog::DefaultSpacing();
		
		Tools_BotanicalSection* pxSection = pxShape->GetSectionByIndex(s_iSelectedElement);
		if( 0 != pxSection->m_xName.Compare( ""))
		{
			Asura_Vector_2 xOffsetLine = xOffset;
			
			pxNewDialog = new Asura_Dialog( "Section: ", xOffsetLine);
			pxNewOverlay->AddDialog(pxNewDialog);	
			xOffsetLine.x += (float)(pxNewDialog->m_xName.GetString() ? Asura_Text_Output::WidthOfString( pxNewDialog->m_xName.GetString(),ASURA_TEXT_FORMAT_SMALLFONT) : 0);

			//xOffset.x+=5.0f*Asura_Dialog::DefaultSpacing();
			
			pxNewDialog = new Asura_Dialog( pxSection->m_xName.GetString(), xOffsetLine);
			pxNewOverlay->AddDialog(pxNewDialog);	
			
			xOffset.y+=1.0f*Asura_Dialog::DefaultSpacing();
		}
		
		Tools_BotanicalVersion* pxVersion = pxShape->GetVersionByIndex(s_iSelectedElement);
		if( 0 != pxVersion->m_xName.Compare( ""))
		{
			Asura_Vector_2 xOffsetLine = xOffset;
			
			pxNewDialog = new Asura_Dialog( "Version: ", xOffsetLine);
			pxNewOverlay->AddDialog( pxNewDialog);	
			xOffsetLine.x += (float)(pxNewDialog->m_xName.GetString() ? Asura_Text_Output::WidthOfString( pxNewDialog->m_xName.GetString(),ASURA_TEXT_FORMAT_SMALLFONT) : 0);
			
			pxNewDialog = new Asura_Dialog( pxVersion->m_xName.GetString(), xOffsetLine);
			pxNewOverlay->AddDialog(pxNewDialog);	
		}
		
		// DLH - 06 December 2006 11:51:34 - Reset the cursor to the correct location
		xOffset = xOffsetStartSections;
	}
#endif

	xOffset.y+=1.5f*Asura_Dialog::DefaultSpacing();
	xOffset.x-=14*Asura_Dialog::DefaultSpacing();

	// LOD selector
	pxNewDialog = new Asura_Numerical_Dialog("LOD Level",xOffset,&s_iNewSelectedLOD,0,pxElement->GetNumLODs()-1,1,"%d");		
	xOffset.y+=1.5f*Asura_Dialog::DefaultSpacing();
	pxNewOverlay->AddDialog(pxNewDialog);

/*
	// DaveR - Stopped messing with lighting. But we might want to resurrect this later, to
	//         give the artists some control

	// Add the lighting controls
	{
	
		const char *aszTmp[5]={	"Lighting: Original Normals",
								"Lighting: Spherical",
								"Lighting: Cylindrical",
								"Lighting: Ground",
								"Lighting: Canopy"};

		pxNewDialog = new Asura_Function_Dialog(aszTmp[(uint32)pxLOD->m_eLightingType],xOffset,NULL); //ChangeLightingType);	
		pxNewOverlay->AddDialog(pxNewDialog);	
		xOffset.x+=14*Asura_Dialog::DefaultSpacing();	


		pxNewDialog = new Asura_Numerical_Dialog("Contrast",xOffset,&pxLOD->m_fContrast,0.0f,1.0f,0.1f,"%0.01f");
		pxNewOverlay->AddDialog(pxNewDialog);
		xOffset.y+=1.5f*Asura_Dialog::DefaultSpacing();
		xOffset.x-=14*Asura_Dialog::DefaultSpacing();
	}	
*/

	// Add the start/end/link items
	pxNewDialog = new Asura_Numerical_Dialog("Start",xOffset,&s_fNewLod0,0,1,0.005f,"%0.03f");	
	xOffset.x+=15*Asura_Dialog::DefaultSpacing();	
	pxNewOverlay->AddDialog(pxNewDialog);

	pxNewDialog = new Asura_Numerical_Dialog("Fade In",xOffset,&s_fNewFadeLength0,0,1,0.002f,"%0.03f");	
	xOffset.x+=15*Asura_Dialog::DefaultSpacing();	
	pxNewOverlay->AddDialog(pxNewDialog);	

	if (s_iSelectedLOD!=0)
	{	
		pxNewDialog = new Asura_Tickbox_Dialog("Link to prev.",xOffset,&pxLOD->m_bLinkToPrev);
		xOffset.x+=15*Asura_Dialog::DefaultSpacing();	
		pxNewOverlay->AddDialog(pxNewDialog);	
		xOffset.x-=15*Asura_Dialog::DefaultSpacing();		
	}

	xOffset.y+=1.0f*Asura_Dialog::DefaultSpacing();
	xOffset.x-=30*Asura_Dialog::DefaultSpacing();		

	

	pxNewDialog = new Asura_Numerical_Dialog("End",xOffset,&s_fNewLod1,0,1,0.005f,"%0.03f");	
	xOffset.x+=15*Asura_Dialog::DefaultSpacing();	
	pxNewOverlay->AddDialog(pxNewDialog);

	pxNewDialog = new Asura_Numerical_Dialog("Fade Out",xOffset,&s_fNewFadeLength1,0,1,0.002f,"%0.03f");	
	xOffset.x+=15*Asura_Dialog::DefaultSpacing();	
	pxNewOverlay->AddDialog(pxNewDialog);

	if (s_iSelectedLOD<pxElement->GetNumLODs()-1)
	{	
		pxNewDialog = new Asura_Tickbox_Dialog("Link to next",xOffset,&pxLOD->m_bLinkToNext);
		xOffset.x+=15*Asura_Dialog::DefaultSpacing();	
		pxNewOverlay->AddDialog(pxNewDialog);	
		xOffset.x-=15*Asura_Dialog::DefaultSpacing();		
	}

	xOffset.y+=1.5f*Asura_Dialog::DefaultSpacing();
	xOffset.x-=30*Asura_Dialog::DefaultSpacing();		

	
	// Add the graph
	pxNewDialog = new Tools_BotanicalCreation_LODGraph_Dialog(xOffset,500,100);	
	xOffset.y+=6.0f*Asura_Dialog::DefaultSpacing();
	pxNewOverlay->AddDialog(pxNewDialog);

	// Add a scale to the graph...
	pxNewDialog = new Asura_Dialog("0% (Far)",xOffset);	
	pxNewOverlay->AddDialog(pxNewDialog);	
	xOffset.x=25.0f;
	pxNewDialog = new Asura_Dialog("50%",xOffset);	
	pxNewOverlay->AddDialog(pxNewDialog);	
	xOffset.x=44.0f;
	pxNewDialog = new Asura_Dialog("(Near) 100%",xOffset);	
	pxNewOverlay->AddDialog(pxNewDialog);	
	
	// Finally, the close button
	xOffset.x=1.0f;
	xOffset.y+=1.5f*Asura_Dialog::DefaultSpacing();
//	pxNewDialog = new Asura_Function_Dialog("Close",xOffset,&CloseOverlay);
//	pxNewOverlay->AddDialog(pxNewDialog);	

	// Draw Distance
	xOffset.x = 20.0f;
	pxNewDialog = new Asura_Numerical_Dialog("Draw Distance", xOffset, &s_fMaxDrawDistance, 10, 2000, 1.0f, "%0.01f");	
	pxNewOverlay->AddDialog(pxNewDialog);


	//submit the overlay
	pxNewOverlay->m_uID = TOOLS_OVERLAY_BOTANICALCREATION_LODS;
	Asura_Overlay_System::SubmitNewOverlay( pxNewOverlay, true, true, false);

}


void Tools_BotanicalCreation_EditLODs::CloseOverlay()
{
	if (s_bOverlayOpen)
	{
		Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_BOTANICALCREATION_LODS);
		s_bOverlayOpen=false;
	}
}


void Tools_BotanicalCreation_EditLODs::ChangeElementType(void)
{
	ListSelection_Overlay xOverlay("Select Element Type: ");
		
	xOverlay.AddItem("1) Single-sided mesh", 0);
	xOverlay.AddItem("2) Double-sided mesh", 1);
	xOverlay.AddItem("3) Billboards", 2);

	Asura_BotanicalShape   *pxShape=Tools_Mode_BotanicalCreation::GetCurrentBotanicalShape();
	Tools_BotanicalShape   *pxToolsShape=pxShape->GetToolsShape();
	Tools_BotanicalElement *pxElement=pxToolsShape->GetElementByIndex(s_iSelectedElement);

//  This doesn't work. Can't find a way to set the default selection :(	
//	xOverlay.SetItemTicked((uint32)pxElement->m_eType,true);
	
	if (!xOverlay.DoModal()) return;
	int iIdx=xOverlay.GetSelectionData();
					
	pxElement->m_eType=(TOOLS_BOTANICALS_ELEMENTTYPE)iIdx;

	// Re-create the renderables
	pxElement->UpdateRenderables();

	// Refresh the overlay
	CloseOverlay();
	CreateOverlay();
}



void Tools_BotanicalCreation_EditLODs::Update(void)
{
	if (s_bOverlayOpen) 
	{		
		// If the selected LOD or element has changed, update the overlay
		if (s_iNewSelectedLOD!=s_iSelectedLOD ||
			s_iNewSelectedElement!=s_iSelectedElement)
		{
			s_iSelectedLOD=s_iNewSelectedLOD;

			if (s_iNewSelectedElement!=s_iSelectedElement)
			{
				s_iSelectedElement=s_iNewSelectedElement;
				s_iNewSelectedLOD=0;
				s_iSelectedLOD=0;
			}				
			
			CloseOverlay();
			CreateOverlay();
		}
		else
		{
			// Normal editing - Update linked LODs and check LOD params against constraints		

			Asura_BotanicalShape*       pxAsuraShape=Tools_Mode_BotanicalCreation::GetCurrentBotanicalShape();
			Tools_BotanicalShape*		pxShape=pxAsuraShape->GetToolsShape();
			Tools_BotanicalElement*     pxElement=pxShape->GetElementByIndex(s_iSelectedElement);
			Tools_BotanicalElementLOD*  pxLOD=pxElement->GetLODByIndex(s_iSelectedLOD);
			Tools_BotanicalElementLOD*  pxLODPrev=NULL;
			Tools_BotanicalElementLOD*  pxLODNext=NULL;

			bool  bConstraintsOK=true;
			float fNewPrevEnd=0,fNewNextStart=0;

			if (s_iSelectedLOD>0 && pxLOD->m_bLinkToPrev)
			{
				pxLODPrev=pxElement->GetLODByIndex(s_iSelectedLOD-1);

				fNewPrevEnd=pxLOD->m_fLod0+pxLOD->m_fFadeLength0+pxLODPrev->m_fFadeLength1;
				
				if (fNewPrevEnd>1.0f || pxLODPrev->m_fFadeLength0 + pxLODPrev->m_fFadeLength1 > fNewPrevEnd - pxLODPrev->m_fLod0)
				{
					bConstraintsOK=false;	
					pxLOD->m_bLinkToPrev=false;				
				}
			}		


			if (s_iSelectedLOD<(pxElement->GetNumLODs()-1) && pxLOD->m_bLinkToNext)
			{
				pxLODNext=pxElement->GetLODByIndex(s_iSelectedLOD+1);

				fNewNextStart=pxLOD->m_fLod1-pxLOD->m_fFadeLength1-pxLODNext->m_fFadeLength0;

				if (fNewNextStart<0.0f || pxLODNext->m_fFadeLength0 + pxLODNext->m_fFadeLength1 > pxLODNext->m_fLod1 - fNewNextStart)
				{
					bConstraintsOK=false;	
					pxLOD->m_bLinkToNext=false;
					pxLODNext->m_bLinkToPrev=false;
				}
			}


			if (s_iSelectedLOD>0)
			{		
				Tools_BotanicalElementLOD *pxLODTemp=pxElement->GetLODByIndex(s_iSelectedLOD-1);
				if (pxLODTemp)		
				{
					pxLODTemp->m_bLinkToNext=pxLOD->m_bLinkToPrev;						
				}
			}

			if (s_iSelectedLOD<(pxElement->GetNumLODs()-1))
			{		
				Tools_BotanicalElementLOD *pxLODTemp=pxElement->GetLODByIndex(s_iSelectedLOD+1);
				if (pxLODTemp)
				{
					pxLODTemp->m_bLinkToPrev=pxLOD->m_bLinkToNext;					
				}
			}


			if (s_fNewFadeLength0+s_fNewFadeLength1 > s_fNewLod1-s_fNewLod0)
			{
				bConstraintsOK=false;	
			}



			if (bConstraintsOK)
			{
				pxLOD->m_fLod0	      = s_fNewLod0;
				pxLOD->m_fLod1        = s_fNewLod1;
				pxLOD->m_fFadeLength0 = s_fNewFadeLength0;
				pxLOD->m_fFadeLength1 = s_fNewFadeLength1;

				if (pxLOD==pxElement->GetLODByIndex(pxElement->GetNumLODs()-1))
				{
					if (pxLOD->m_fLod1>=1.0f)
					{
						pxLOD->m_fLod1=1.0001f;
					}
				}

				if (pxLODPrev)
				{
					pxLODPrev->m_fLod1 = fNewPrevEnd;
				}

				if (pxLODNext)
				{
					pxLODNext->m_fLod0 = fNewNextStart;
				}
			}

			s_fNewLod0		   = pxLOD->m_fLod0;
			s_fNewLod1		   = pxLOD->m_fLod1;
			s_fNewFadeLength0  = pxLOD->m_fFadeLength0;
			s_fNewFadeLength1  = pxLOD->m_fFadeLength1;
		
			if (s_iSelectedLOD==pxElement->GetNumLODs()-1)
			{
				s_fNewFadeLength1=pxLOD->m_fFadeLength1;
				s_fNewLod1=pxLOD->m_fLod1;
			}

			Tools_BotanicalShape* pxBotanicalShape = pxAsuraShape->GetToolsShape();
			
			pxBotanicalShape->m_fInvMaxDrawDistance = 1.0f / s_fMaxDrawDistance;
			pxBotanicalShape->m_fMaxDrawDistanceFOV = Asura_Camera::GetHorizontalFieldOfView();
			pxBotanicalShape->m_uMaxDrawDistanceRes = Asura_Viewport::GetWidth();
		}
	}

	if (s_bImpostersOverlayOpen)
	{
		// If the currently selected imposter has changed, we need
		// to recreate the overlay
		if (s_iCurrentImposter!=s_iNewCurrentImposter)
		{
			s_iCurrentImposter=s_iNewCurrentImposter;
			CloseImpostersOverlay();
			CreateImpostersOverlay();
		}
		
		Tools_Botanicals_Imposter *pxImposter=s_pxImposterLOD->GetImposterByIndex(s_iCurrentImposter);

		// Check the fade range is valid
		if (pxImposter->m_fFadeRange>(pxImposter->m_fAngleRange*0.5f))
		{
			pxImposter->m_fFadeRange=pxImposter->m_fAngleRange*0.5f;
		}		
	}	
}

// -------------------------------------------------------------------------------------
// Render the special bits (the graph) on the overlay
// -------------------------------------------------------------------------------------

void Tools_BotanicalCreation_EditLODs::RenderGraph(float fX,float fY,float fW,float fH)
{
	Asura_Vector_2 x1,x2,x3,x4;

	// Draw the grid lines
	for(int n=0;n<=10;n++)
	{	
		float fN=((float)(n))/10.0f;
		uint32 uCol=0x20ffffff;		
		if (n==0 || n==10) uCol=0x80ffffff;
		if (n==5) uCol=0x40ffffff;
		
		x1.x=fX;
		x1.y=fY+(fH*fN);
		x2=x1;
		x2.x+=fW;		
		Asura_Render::Render2DLine(x1,x2,uCol);

		x1.x=fX+(fW*fN);
		x1.y=fY;
		x2=x1;
		x2.y+=fH;		
		Asura_Render::Render2DLine(x1,x2,uCol);
	}

	// Render the lines for each LOD level
	Asura_BotanicalShape*       pxAsuraShape=Tools_Mode_BotanicalCreation::GetCurrentBotanicalShape();
	Tools_BotanicalShape*		pxShape=pxAsuraShape->GetToolsShape();
	Tools_BotanicalElement*     pxElement=pxShape->GetElementByIndex(s_iSelectedElement);

	for(int nPass=0; nPass<2; nPass++)
	{	
		for(int i=0;i<pxElement->GetNumLODs();i++)
		{
			Tools_BotanicalElementLOD*  pxLOD=pxElement->GetLODByIndex(i);			

			if ((nPass==0 && i!=s_iSelectedLOD) || (nPass==1 && i==s_iSelectedLOD))
			{			
				uint32		  uCol=0x10004000;

				// Selected LOD - highlight in red
				if (nPass==1) uCol=0x8fffA000;

				// Darken LODs further than the selection
				int diff=abs(i-s_iNewSelectedLOD);
				if (diff==1)  uCol=0x4000FF00;
				if (diff==2)  uCol=0x3000B000;
				if (diff==3)  uCol=0x20008000;

				float fA=pxLOD->m_fLod0;
				float fB=pxLOD->m_fLod0+pxLOD->m_fFadeLength0;
				float fC=pxLOD->m_fLod1-pxLOD->m_fFadeLength1;
				float fD=pxLOD->m_fLod1;

				x1.y=x4.y=fY+fH;
				x2.y=x3.y=fY;

				x1.x=fX+(fW*fA);
				x2.x=fX+(fW*fB);
				x3.x=fX+(fW*fC);
				x4.x=fX+(fW*fD);

				for(int iX=-1;iX<=1;iX++)
					for(int iY=-1;iY<=1;iY++)
					{	
						Asura_Vector_2 xo1,xo2,xo3,xo4,xOfs;
						xOfs.x=(float)iX;
						xOfs.y=(float)iY;
						xo1=x1+xOfs;
						xo2=x2+xOfs;
						xo3=x3+xOfs;
						xo4=x4+xOfs;
						Asura_Render::Render2DLine(xo1,xo2,uCol);
						Asura_Render::Render2DLine(xo2,xo3,uCol);
						Asura_Render::Render2DLine(xo3,xo4,uCol);
					}				
			}
		}
	}	
	

	// Draw the current LOD value as a vertical yellow line
	x1.x=fX+(fW*Tools_Mode_BotanicalCreation::GetCurrentLODValue());
	x1.y=fY;
	x2.x=x1.x;
	x2.y=fY+fH;
	Asura_Render::Render2DLine(x1,x2,0xa0ffff00);
}




void Tools_BotanicalCreation_EditLODs::EditImposters()
{
	Asura_BotanicalShape*       pxAsuraShape=Tools_Mode_BotanicalCreation::GetCurrentBotanicalShape();
	Tools_BotanicalShape*		pxShape=pxAsuraShape->GetToolsShape();
	Tools_BotanicalElement*     pxElement=pxShape->GetElementByIndex(s_iSelectedElement);

	// Store pointer to the impostor LOD. Imposter elements can only ever have one LOD
	s_pxImposterLOD=pxElement->GetLODByIndex(0);
	
	// Select imposter #0
	s_iCurrentImposter=0;
	s_iNewCurrentImposter=0;

	// Create the editing overlay
	CreateImpostersOverlay();
}


// -------------------------------------------------------------------------------------
// Create the imposter editing overlay
// -------------------------------------------------------------------------------------

void Tools_BotanicalCreation_EditLODs::CreateImpostersOverlay()
{
	// Hide both overlays, if either is already open
	CloseOverlay();		
	CloseImpostersOverlay();
	s_bImpostersOverlayOpen=true;

	Asura_Overlay* pxNewOverlay;
	Asura_Dialog*  pxNewDialog;
	Asura_Vector_2 xOffset = Asura_Vector_2(1.0f,3.0f);

	pxNewOverlay = new Asura_Overlay("Imposters",false,&s_xOverlayPosition);
		
	// Imposter selector	
	pxNewDialog = new Asura_Numerical_Dialog("Imposter",xOffset,&s_iNewCurrentImposter,0,s_pxImposterLOD->GetNumImposters()-1,1,"%d");	
	xOffset.y+=1.5f*Asura_Dialog::DefaultSpacing();
	pxNewOverlay->AddDialog(pxNewDialog);

	Tools_Botanicals_Imposter* pxImposter=s_pxImposterLOD->GetImposterByIndex(s_iCurrentImposter);

	// On range   (degrees)
	pxNewDialog = new Asura_Numerical_Dialog("Angle Range (degrees)",xOffset,&pxImposter->m_fAngleRange,0,360.0f,1,"%0.01f");	
	xOffset.y+=1.5f*Asura_Dialog::DefaultSpacing();
	pxNewOverlay->AddDialog(pxNewDialog);

	// Fade range (degrees)
	pxNewDialog = new Asura_Numerical_Dialog("Fade Range (degrees)",xOffset,&pxImposter->m_fFadeRange,0,360.0f,1,"%0.01f");	
	xOffset.y+=1.5f*Asura_Dialog::DefaultSpacing();
	pxNewOverlay->AddDialog(pxNewDialog);

	// Finally, the close button		
	pxNewDialog = new Asura_Function_Dialog("Close",xOffset,&CloseImpostersOverlay);
	pxNewOverlay->AddDialog(pxNewDialog);	

	//submit the overlay
	pxNewOverlay->m_uID = TOOLS_OVERLAY_BOTANICALCREATION_IMPOSTERS;
	Asura_Overlay_System::SubmitNewOverlay(pxNewOverlay,false, true, false);

}


void Tools_BotanicalCreation_EditLODs::CloseImpostersOverlay()
{
	if (s_bImpostersOverlayOpen)
	{
		Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_BOTANICALCREATION_IMPOSTERS);
		s_bImpostersOverlayOpen=false;
	}
}



void Tools_BotanicalCreation_EditLODs::Draw3DQuad(D3DGenericVertex *pxVerts)
{
	// Set renderstates
	Asura_Render::CurrentStates.RequestTextureOpMode(ASURA_TEXTUREOP_SINGLE_PASS_2X);
	Asura_Direct3D_Shaders::RequestFixedFunctionVertexShader(D3DFVF_GENERICVERTEX);
	Asura_Direct3D::D3DDevice->SetRenderState(D3DRS_CULLMODE, D3DCULL_NONE);
	Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_NORMAL);
	Asura_Render::CurrentStates.RequestZBufferWriteEnabled(false);	
	Asura_Render::CurrentStates.RequestTextureIndex(0,ASURA_TEXTURE_NONE);								

	// Draw it
	Asura_Direct3D::D3DDevice->DrawPrimitiveUP(D3DPT_TRIANGLEFAN, 2, (void*)pxVerts, sizeof(pxVerts[0]));			

	Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_OFF);
	Asura_Render::CurrentStates.RequestZBufferWriteEnabled(true);
}


void Tools_BotanicalCreation_EditLODs::RenderAngleRange(Asura_Vector_3 xCentre,Asura_Vector_3 xDir,float fPreRotate,float fAngle,uint32 uCol,float fAlphaStart,float fAlphaEnd)
{
	int   iSteps=1+abs((int)(fAngle/3.0f));	
	float fAngleStep=fAngle/iSteps;
	
	Asura_Euler_3 xEuler=Asura_Euler_3(0,fPreRotate*(Asura_Maths::Pi/180.0f),0);
	Asura_Matrix_3x3 xRotation;
	xEuler.ToMatrix(xRotation);
	xDir=xDir*xRotation;

	xEuler=Asura_Euler_3(0,fAngleStep*(Asura_Maths::Pi/180.0f),0);	
	xEuler.ToMatrix(xRotation);

	D3DGenericVertex xVtx[4];

	float fAlpha=fAlphaStart;	
	float fAlphaStep=(fAlphaEnd-fAlphaStart)/((float)(iSteps));
	
	int iAlpha=0;

	for(int i=0;i<iSteps;i++)
	{
		for(int j=0;j<4;j++)
		{	
			xVtx[j].X=xCentre.x;
			xVtx[j].Y=xCentre.y;
			xVtx[j].Z=xCentre.z;			
		}


		// Set up first edge
		iAlpha=(int)(fAlpha*255.0f);
		xVtx[1].X+=xDir.x;
		xVtx[1].Y+=xDir.y;
		xVtx[1].Z+=xDir.z;		
		xVtx[0].Diffuse=(uCol&0x00ffffff);
		xVtx[1].Diffuse=(uCol&0x00ffffff)|(iAlpha<<24);

		// Rotate around
		xDir=xDir*xRotation;
		fAlpha+=fAlphaStep;

		// Set up next edge
		iAlpha=(int)(fAlpha*255.0f);
		xVtx[2].X+=xDir.x;
		xVtx[2].Y+=xDir.y;
		xVtx[2].Z+=xDir.z;
		xVtx[2].Diffuse=(uCol&0x00ffffff)|(iAlpha<<24);
		xVtx[3].Diffuse=(uCol&0x00ffffff);
		

		// Draw it
		Draw3DQuad(xVtx);
	}
}


void Tools_BotanicalCreation_EditLODs::Render()
{
	if (s_bImpostersOverlayOpen)
	{		
		int iNumImposters = s_pxImposterLOD->GetNumImposters();

		for (int iPass=0;iPass<2;iPass++)
		{		
			for(int i=0;i<iNumImposters;i++)
			{
				Tools_Botanicals_Imposter *pxImposter = s_pxImposterLOD->GetImposterByIndex(i);

				if ((iPass==0 && i!=s_iCurrentImposter) || (iPass==1 && i==s_iCurrentImposter))
				{				
					// Render the normal
					Asura_Vector_3 xNormal=pxImposter->m_xBillboard.xNormal;
					xNormal.y=0;
					xNormal.Normalise();

					Asura_Vector_3 xFrom=Tools_Mode_BotanicalCreation::GetOrigin();
					Asura_Vector_3 xTo=xFrom+(xNormal*10.0f);
					Asura_Vector_3 xDir=xNormal*10.0f;
					
					float fRange=(pxImposter->m_fAngleRange)*0.5f-pxImposter->m_fFadeRange;
					if (iPass==0)
					{
						RenderAngleRange(xFrom,xDir,0,fRange,0x80000080,0.5f,0.5f);
						RenderAngleRange(xFrom,xDir,0,-fRange,0x80000080,0.5f,0.5f);
						RenderAngleRange(xFrom,xDir,fRange,pxImposter->m_fFadeRange,0x80000080,0.5f,0.0f);
						RenderAngleRange(xFrom,xDir,-fRange,-pxImposter->m_fFadeRange,0x80000080,0.5f,0.0f);
					}
					else
					{
						RenderAngleRange(xFrom,xDir,0,fRange,0x80008000,0.5f,0.5f);						
						RenderAngleRange(xFrom,xDir,0,-fRange,0x80008000,0.5f,0.5f);
						RenderAngleRange(xFrom,xDir,fRange,pxImposter->m_fFadeRange,0x80008000,0.5f,0.0f);
						RenderAngleRange(xFrom,xDir,-fRange,-pxImposter->m_fFadeRange,0x80008000,0.5f,0.0f);
						Asura_Render::RenderLine(xFrom,xTo,0xff88ff88);	

						Asura_Vector_3 *pxCam=(Asura_Vector_3 *)Asura_Camera::GetPosition();
						Asura_Vector_3 xCam=*pxCam-xFrom;
						xCam.y=0;
						xCam.Normalise();
						xTo=Tools_Mode_BotanicalCreation::GetOrigin()+(xCam*10);
						Asura_Render::RenderLine(xFrom,xTo,0xff00ff00);	
					}					
				}
			}
		}		
	}
}



#endif // ASURA_USE_BOTANICALS