//
// Tools_BotanicalCreation_Rendering.h
//
// Created by Dave Reed - 11/01/2006
// 
// Copyright (C) 2005 Rebellion (Derby), All Rights Reserved.
//
// Botanical rendering in tools mode
//

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifdef ASURA_USE_BOTANICALS

#include "BotanicalCreation/Tools_BotanicalCreation_BotanicalShape.h"
#include "BotanicalCreation/Tools_BotanicalCreation_Rendering.h"
#include "Mesh/Asura_Mesh_System.h"
#include "Misc/Tools_Render.h"
#include "Rendering/Asura_Text_Output.h"

#ifndef __TOOLS_BOTANICALCREATION_EDITLODS_H__
#define __TOOLS_BOTANICALCREATION_EDITLODS_H__



// Asura_UserRender_Dialog was exiled from Asura_Dialog.cpp. So we've got this.
// For now It's still essentially a user rendered dialog...
// Could do with some improvements (mouse dragging of graph points), and could
// do with being separated from the botanicals data, for other systems that may
// need to deal with LODs.

class Tools_BotanicalCreation_LODGraph_Dialog : public Asura_Dialog
{
public:
	/* Constructor & Destructor */
	Tools_BotanicalCreation_LODGraph_Dialog(const Asura_Vector_2& xOffset, float fWidth, float fHeight);
	virtual ~Tools_BotanicalCreation_LODGraph_Dialog() {}

public:
	/* Public Interface Functions */
	virtual void Render(const Asura_Vector_2& xOrigin);

private:
	/* Data */	
	void(*m_pfnUserRender)(float fX,float fY,float fW,float fH);
	Asura_Vector_2 m_xSize;
};




class Tools_BotanicalCreation_EditLODs
{
public:
	static void Reset();				// Call this at startup, and when the number of loaded
										// elements/lods changes
	static void CreateOverlay();
	static void CloseOverlay();
	static void RenderGraph(float fX,float fY,float fW,float fH);
	static void Update();
	static void Render();
	static void ChangeElementType();
	
	static void EditImposters();
	static void CreateImpostersOverlay();
	static void CloseImpostersOverlay();

	static bool IsActive()		     {return s_bOverlayOpen || s_bImpostersOverlayOpen;};
	static int  GetSelectedElement() {return s_iSelectedElement;};
	static int  GetSelectedLOD()     {return s_iSelectedLOD;};

private:

	static void					Draw3DQuad(D3DGenericVertex *pxVerts);
	static void					RenderAngleRange(Asura_Vector_3 xCentre,Asura_Vector_3 xDir,float fPreRotate,float fAngle,uint32 uCol, float fAlphaStart, float fAlphaEnd);

	static bool					s_bOverlayOpen;
	static Asura_Vector_2		s_xOverlayPosition;
	
	static bool					s_bImpostersOverlayOpen;
	static Asura_Vector_2		s_xImpostersOverlayPosition;
	static int					s_iCurrentImposter;
	static int					s_iNewCurrentImposter;
	static Tools_BotanicalElementLOD*  s_pxImposterLOD;

	static int					s_iNewSelectedElement;
	static int					s_iSelectedElement;
	static int					s_iNewSelectedLOD;
	static int					s_iSelectedLOD;

	static float				s_fNewLod0;
	static float				s_fNewLod1;
	static float				s_fNewFadeLength0;
	static float				s_fNewFadeLength1;	
	
	static float				s_fTemp;
	static bool					s_bTemp;

	static float				s_fMaxDrawDistance;
};


#endif // ASURA_USE_BOTANICALS
#endif // __TOOLS_BOTANICALCREATION_EDITLODS_H__
