
//
// Tools_Mode_BotanicalCreation_Rendering.h
//
// Created by Dave Reed - 12/01/2006
// 
// Copyright (C) 2006 Rebellion (Derby), All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifdef ASURA_USE_BOTANICALS

#ifndef __TOOLS_BOTANICALCREATION_RENDERING_H__
#define __TOOLS_BOTANICALCREATION_RENDERING_H__

#include "BotanicalCreation/Tools_BotanicalCreation_BotanicalShape.h"
#include "Mesh/Asura_Mesh_System.h"
#include "Misc/Tools_Render.h"
#include "Rendering/Asura_Direct3d.h"
#include "Light/Asura_GI_Solution.h"
#include "Light/Asura_Lighting_Solution.h"

// Base class for Botanical Renderables.
// We have a subclass for each component type (Mesh, Billboards, Imposters)

class Tools_BotanicalRenderable
{
public:
	virtual ~Tools_BotanicalRenderable() {}
	virtual void Render(float fFade, Asura_Vector_3 *pxPosition, Asura_Matrix_3x3* pxOrientation, float fScale, uint32 uFlags)=0;
};



class Tools_BotanicalRenderable_Mesh : public Tools_BotanicalRenderable
{
public:	
	Tools_BotanicalRenderable_Mesh(Tools_BotanicalElementLOD *pxLOD);	
	virtual ~Tools_BotanicalRenderable_Mesh();
	virtual void Render(float fFade, Asura_Vector_3 *pxPosition, Asura_Matrix_3x3* pxOrientation, float fScale, uint32 uFlags);	

private:
	Tools_BotanicalElementLOD *m_pxLOD;
	D3DGenericVertex*		   m_pxVertices;	
	int						   m_iNumTriangles;	
	int						   m_iTextureID;
	Asura_Lighting_Solution	   m_xLightingSolution; 
};





class Tools_BotanicalRenderable_Billboards : public Tools_BotanicalRenderable
{
public:	

	Tools_BotanicalRenderable_Billboards(const Tools_BotanicalElementLOD *pxLOD);	
	virtual ~Tools_BotanicalRenderable_Billboards();
	virtual void Render(float fFade, Asura_Vector_3 *pxPosition, Asura_Matrix_3x3 *pxOrientation, float fScale,uint32 uFlags);	

	const Asura_Vector_3& GetCentre() const { return m_xCentre; }

	int GetNumBillboards() const { return m_iNumBillboards; }

	const Tools_Botanicals_Billboard& GetBillboard( const int iBillboard ) const
	{
		if( 0 <= iBillboard && iBillboard < m_iNumBillboards )
		{
			return m_pxBillboards[ iBillboard ];
		}
		else
		{
			Asura_Assert( false, "Invalid parameter passed to Tools_BotanicalRenderable_Billboards::GetBillboard()." );
			return m_pxBillboards[0];
		}
	}

private:
	const Tools_BotanicalElementLOD*	 m_pxLOD;
	Tools_Botanicals_Billboard*			 m_pxBillboards;
	int									 m_iNumBillboards;
	int									 m_iTextureID;
	D3DGenericVertex*					 m_pxVertices;
	uint16*								 m_pIndices;
	Asura_Vector_3						 m_xCentre;
	Asura_Lighting_Solution				 m_xLightingSolution; 
};




class Tools_BotanicalRenderable_Imposters : public Tools_BotanicalRenderable
{
public:	

	Tools_BotanicalRenderable_Imposters(Tools_BotanicalElementLOD *pxLOD);	
	virtual ~Tools_BotanicalRenderable_Imposters();
	virtual void Render(float fFade, Asura_Vector_3 *pxPosition, Asura_Matrix_3x3 *pxOrientation, float fScale,uint32 uFlags);	

private:
	Tools_BotanicalElementLOD*			 m_pxLOD;
	Asura_Lighting_Solution				 m_xLightingSolution; 
};



#endif // __TOOLS_BOTANICALCREATION_RENDERING_H__
#endif // ASURA_USE_BOTANICALS
