//*******************************************************************
// File: Tools_BotanicalCreation_StateTables.cpp
//
// Created: 06 December 2006 DLH
//
// Description: 
//
//*******************************************************************

// system libraries
#include "Core/Asura.h"

#if defined( ASURA_USE_BOTANICALS) && defined( ASURA_TOOLS)

// main header
#include "BotanicalCreation/Tools_BotanicalCreation_StateTables.h"

#if defined( ASURA_BOTANICAL_SUPPORT_STATES) && 1 == ASURA_BOTANICAL_SUPPORT_STATES

// other libraries
#include "Tools_Overlays.h"
#include "Hierarchy/Tools_Animation_Files.h"
#include "Animation/Asura_Animation.h"

#include "BotanicalCreation/Tools_BotanicalCreation_BotanicalShape.h"
#include "BotanicalCreation/Tools_Mode_BotanicalCreation.h"
#include "Botanicals/Asura_Botanicals_ShapeManager.h"

#include "Botanicals/Asura_Botanical_Game_Render_Types.h"

#include "Interface/Asura_Interface.h"
#include "Interface/Asura_CommandConsole.h"
#include "Interface/Asura_Overlay.h"
#include "Interface/Asura_Overlay_System.h"
#include "Interface/Asura_Dialog.h"

#include "Misc/Tools_DynamicArray.h"
#include "Misc/Tools_Modal_Overlay.h"

#include "Rendering/Asura_Viewport.h"

// internal dependencies

//*******************************************************************
// Constants
//*******************************************************************

#define STATE_SECTION_VERSION_RANGE_PRINTF_FORMAT "%4.0f"
#define STATE_TABLE_DIALOG_LIST_SIZE 5

//*******************************************************************
// Local Data
//*******************************************************************

Asura_Vector_2	Tools_BotanicalCreation_StateTables::s_xOverlayPosition = Asura_Vector_2( 4, 24);
Asura_Overlay_Botanical_State_Tables* Tools_BotanicalCreation_StateTables::s_pxOverlay = NULL;

bool	Tools_BotanicalCreation_StateTables::s_bShowCurrentVersionOnly = false;
bool	Tools_BotanicalCreation_StateTables::s_bPreviewStateValue	   = false;
bool	Tools_BotanicalCreation_StateTables::s_bShowFoliageCollisionMeshes = true;
bool	Tools_BotanicalCreation_StateTables::s_bShowCurrentVersionOnlyChanged = false;
bool	Tools_BotanicalCreation_StateTables::s_bPreviewStateValueChanged	   = false;
bool	Tools_BotanicalCreation_StateTables::s_bShowFoliageCollisionMeshesChanged = false;
int		Tools_BotanicalCreation_StateTables::s_iStateValueToPreview	   = 0;

//*******************************************************************
// Function Definitions
//*******************************************************************

Asura_Overlay_Botanical_State_Tables::Asura_Overlay_Botanical_State_Tables( const char* pszOverlayName, const bool bAlwaysOnTop, Asura_Vector_2* pxPosition) :
	Asura_Overlay( pszOverlayName, bAlwaysOnTop, pxPosition)
{
	this->m_pxAsuraShape = Tools_Mode_BotanicalCreation::GetCurrentBotanicalShape();
	this->m_pxToolShape = this->m_pxAsuraShape->GetToolsShape();
	this->m_pxToolShapeSection = NULL;
	
	this->m_pxStateTable = NULL;
	this->m_pxStateTableSection = NULL;
	this->m_pxStateTableSectionVersion = NULL;
	
//	this->m_pxState_DropDown = NULL;
	this->m_pxState_List = NULL;
	this->m_iStateDropDownNameSelected = 0;
	this->m_iStateDropDownNameCount = 0;

#if 0	
	this->m_pxSection_DropDown = NULL;
	this->m_iSectionDropDownNameCount = 0;
#endif

	this->m_pxStateSection_List = NULL;
	this->m_iStateSectionListNameSelected = 0;
	this->m_iStateSectionListNameCount = 0;
	
	this->m_pxStateSectionVersion_List = NULL;
	this->m_pxStateSectionVersion_Add = NULL;
	this->m_pxStateSectionVersion_Delete = NULL;
	this->m_pxStateSectionVersion_Start = NULL;
	this->m_pxStateSectionVersion_Stop = NULL;
	this->m_pxStateSectionVersion_SetCollisionMesh = NULL;
	this->m_pxStateSectionVersion_CollisionMeshDensity = NULL;

	this->m_iStateSectionVersionListNameSelected = 0;
	this->m_iStateSectionVersionListNameCount = 0;
	
	this->m_fVersionStart = 0;
	this->m_fVersionStop = 0;
	
	this->m_fVersionMin = 0;
	this->m_fVersionMax = static_cast< Asura_BGR_State_Value>( ~0L);

	this->m_uProjectFlags = 0;
	
	this->m_pxSectionVersion_DropDown = NULL;
	this->m_iSectionVersionDropDownNameCount = 0;
	this->m_iSectionVersionDropDownNameSelected = 0;
	
	this->m_bDataChanged = false;
}

Asura_Overlay_Botanical_State_Tables::~Asura_Overlay_Botanical_State_Tables()
{
	Tools_BotanicalCreation_StateTables::CloseOverlay();
	DeleteStateDropDownNameArray();
	DeleteStateSectionListNameArray();
	DeleteStateSectionVersionListNameArray();
	DeleteSectionVersionDropDownNameArray();
}

void Tools_BotanicalCreation_StateTables::Reset()
{
	CloseOverlay();		
}

void Tools_BotanicalCreation_StateTables::CreateOverlay()
{
	CloseOverlay();

	Asura_Overlay* pxNewOverlay = s_pxOverlay = new Asura_Overlay_Botanical_State_Tables("State Tables",false,&s_xOverlayPosition);
	pxNewOverlay->SetAlwaysFullBright(true);
	
	s_pxOverlay->CreateControls();
	
	pxNewOverlay->m_uID = TOOLS_OVERLAY_BOTANICALCREATION_STATETABLE;
	Asura_Overlay_System::SubmitNewOverlay( pxNewOverlay, true, true, false);
}

void Tools_BotanicalCreation_StateTables::CloseOverlay()
{
	if ( NULL != s_pxOverlay)
	{
		s_pxOverlay = NULL;
		
		Asura_Overlay_System::CloseOverlays( TOOLS_OVERLAY_BOTANICALCREATION_STATETABLE);
	}
	
}

void Tools_BotanicalCreation_StateTables::Update()
{
	if ( NULL != s_pxOverlay)
	{
		s_pxOverlay->Update();
	}
}

void Tools_BotanicalCreation_StateTables::Render()
{
}

void Tools_BotanicalCreation_StateTables::ClearSectionVersionVisibleFilters( void)
{
	s_bPreviewStateValue = false;
	s_bShowCurrentVersionOnly = false;
}

bool Tools_BotanicalCreation_StateTables::IsSectionVersionVisibleFiltered( void)
{
	if( true == s_bPreviewStateValue)
	{
		return( true);
	}
	
	if( true == s_bShowCurrentVersionOnly)
	{
		return( true);
	}
	
	return( false);
}

#if 1
bool Tools_BotanicalCreation_StateTables::IsSectionVersionVisible( const Tools_BotanicalSection* pxToolSection, const Tools_BotanicalVersion* pxToolVersion)
{

	if (Tools_Mode_BotanicalCreation::GetCurrentBotanicalShape() == NULL || s_pxOverlay == NULL || (s_bPreviewStateValue == false && s_bShowCurrentVersionOnly == false))
	{
		// In these cases, draw all sections+versions
		return true;
	}

	Tools_Botanical_StateTable1D* pxStateTable = s_pxOverlay->GetCurrentStateTable();

	if (!pxStateTable)
	{
		return( false);
	}	

	const u_int uSelectedTableSectionIndex = s_pxOverlay->GetSectionListSelectionIdx();
	const u_int uSelectedTableVersionIndex = s_pxOverlay->GetVersionListSelectionIdx();
	
	const Tools_Botanical_StateTable1D_Section* pxToolStateTableSection = NULL;
	u_int uListSectionIndex = 0;
	for( Asura_ListElt< Tools_Botanical_StateTable1D_Section>* pxItStateTableSection = pxStateTable->m_xList_xSection.Front(); NULL != pxItStateTableSection; pxItStateTableSection = pxItStateTableSection->Next(), uListSectionIndex += 1)
	{
		const Tools_Botanical_StateTable1D_Section& xToolStateTableSection = (pxItStateTableSection->Data());
		
		if( xToolStateTableSection.m_xTargetSection != pxToolSection->m_xName)
		{
			continue;
		}
		if( true == s_bShowCurrentVersionOnly && uSelectedTableSectionIndex != uListSectionIndex)
		{
			continue;
		}
		pxToolStateTableSection = &xToolStateTableSection;
		break;
	}	
	
	if( NULL == pxToolStateTableSection)
	{
		return( false);
	}		
	
	const Tools_Botanical_StateTable1D_Section_VersionRange* pxToolStateTableSectionVersion = NULL;
	u_int uListSectionVersionIndex = 0;
	for( const Asura_ListElt< Tools_Botanical_StateTable1D_Section_VersionRange>* pxItStateTableSectionVersion = pxToolStateTableSection->m_xList_xVersionRanges.Front(); NULL != pxItStateTableSectionVersion; pxItStateTableSectionVersion = pxItStateTableSectionVersion->Next(), uListSectionVersionIndex += 1)
	{
		const Tools_Botanical_StateTable1D_Section_VersionRange& xToolStateSectionVersionRange = pxItStateTableSectionVersion->Data();
		
		if( xToolStateSectionVersionRange.m_xTargetVersion != pxToolVersion->m_xName)
		{
			continue;
		}
		if( true == s_bShowCurrentVersionOnly && uSelectedTableVersionIndex != uListSectionVersionIndex)
		{
			continue;
		}
		pxToolStateTableSectionVersion = &xToolStateSectionVersionRange;
		break;
	}
	
	if( NULL == pxToolStateTableSectionVersion)
	{
		return( false);
	}
	
	if( true == s_bPreviewStateValue)
	{
		if( s_iStateValueToPreview < pxToolStateTableSectionVersion->m_fRangeStart)
		{
			return( false);
		}
		if( pxToolStateTableSectionVersion->m_fRangeEnd < s_iStateValueToPreview)
		{
			return( false);
		}
	}
	return( true);
}


const Asura_CollisionMesh *Tools_BotanicalCreation_StateTables::GetVisibleSectionVersionCollisionMesh( const Tools_BotanicalSection* pxToolSection, const Tools_BotanicalVersion* pxToolVersion)
{
	if ((!s_bShowFoliageCollisionMeshes) || Tools_Mode_BotanicalCreation::GetCurrentBotanicalShape() == NULL || s_pxOverlay == NULL)
	{
		return NULL;
	}


	Tools_Botanical_StateTable1D* pxStateTable = s_pxOverlay->GetCurrentStateTable();

	if (!pxStateTable)
	{
		return NULL;
	}	

	const u_int uSelectedTableSectionIndex = s_pxOverlay->GetSectionListSelectionIdx();
	const u_int uSelectedTableVersionIndex = s_pxOverlay->GetVersionListSelectionIdx();

	const Tools_Botanical_StateTable1D_Section* pxToolStateTableSection = NULL;
	u_int uListSectionIndex = 0;
	for( Asura_ListElt< Tools_Botanical_StateTable1D_Section>* pxItStateTableSection = pxStateTable->m_xList_xSection.Front(); NULL != pxItStateTableSection; pxItStateTableSection = pxItStateTableSection->Next(), uListSectionIndex += 1)
	{
		const Tools_Botanical_StateTable1D_Section& xToolStateTableSection = (pxItStateTableSection->Data());

		if( xToolStateTableSection.m_xTargetSection != pxToolSection->m_xName)
		{
			continue;
		}
		if( true == s_bShowCurrentVersionOnly && uSelectedTableSectionIndex != uListSectionIndex)
		{
			continue;
		}
		pxToolStateTableSection = &xToolStateTableSection;
		break;
	}	

	if( NULL == pxToolStateTableSection)
	{
		return NULL;
	}		

	const Tools_Botanical_StateTable1D_Section_VersionRange* pxToolStateTableSectionVersion = NULL;
	u_int uListSectionVersionIndex = 0;
	for( const Asura_ListElt< Tools_Botanical_StateTable1D_Section_VersionRange>* pxItStateTableSectionVersion = pxToolStateTableSection->m_xList_xVersionRanges.Front(); NULL != pxItStateTableSectionVersion; pxItStateTableSectionVersion = pxItStateTableSectionVersion->Next(), uListSectionVersionIndex += 1)
	{
		const Tools_Botanical_StateTable1D_Section_VersionRange& xToolStateSectionVersionRange = pxItStateTableSectionVersion->Data();

		if( xToolStateSectionVersionRange.m_xTargetVersion != pxToolVersion->m_xName)
		{
			continue;
		}
		if( true == s_bShowCurrentVersionOnly && uSelectedTableVersionIndex != uListSectionVersionIndex)
		{
			continue;
		}

		pxToolStateTableSectionVersion = &xToolStateSectionVersionRange;
		break;
	}

	if( NULL == pxToolStateTableSectionVersion)
	{
		return NULL;
	}

	if( true == s_bPreviewStateValue)
	{
		if( s_iStateValueToPreview < pxToolStateTableSectionVersion->m_fRangeStart)
		{
			return NULL;
		}
		if( pxToolStateTableSectionVersion->m_fRangeEnd < s_iStateValueToPreview)
		{
			return NULL;
		}
	}


	if (pxToolStateTableSectionVersion)
	{
		Asura_Hierarchy_Skin *pxSkin = Asura_Animation_System::GetSkinByHash(pxToolStateTableSectionVersion->m_xCollisionMeshHash);

		if (pxSkin)
		{
			return pxSkin->GetCollisionMesh();			
		}
	}

	return NULL;
}

#endif

#if 0
bool Tools_BotanicalCreation_StateTables::IsSectionVersionVisible(uint32 uSectionVersionIdx)
{

	if (Tools_Mode_BotanicalCreation::GetCurrentBotanicalShape() == NULL || s_pxOverlay == NULL || (s_bPreviewStateValue == false && s_bShowCurrentVersionOnly == false))
	{
		// In these cases, draw all sections+versions
		return true;
	}

	Tools_BotanicalShape* pxToolsShape = Tools_Mode_BotanicalCreation::GetCurrentBotanicalShape()->GetToolsShape();
	Tools_Botanical_StateTable1D* pxStateTable = s_pxOverlay->GetCurrentStateTable();

	if (!pxStateTable)
	{
		return false;
	}	

		
	// This is a bit tricky... We have a section and version index, and have to find the matching entry in the state table. 
	// iCount is the index of the shape within the tools shape - the 'SectionVersionIndex'
	int iCount = 0;
	int iSectionListIndex = 0;
	for( Asura_ListElt< Tools_Botanical_StateTable1D_Section>* pxItStateTableSection = pxStateTable->m_xList_xSection.Front(); NULL != pxItStateTableSection; pxItStateTableSection = pxItStateTableSection->Next())
	{
		Tools_Botanical_StateTable1D_Section& xToolStateTableSection = pxItStateTableSection->Data();		 

		Tools_BotanicalSection *pxToolsShapeSection = pxToolsShape->m_xHashTree_xSections.GetDataByName(xToolStateTableSection.m_xTargetSection.GetString());
		
		if (pxToolsShapeSection == pxToolsShape->GetSectionByIndex((int)uSectionVersionIdx))
		{
			// We've found our current section - now is the version referenced by it's state table?
			int iVersionListIndex = 0;

			for( const Asura_ListElt< Tools_Botanical_StateTable1D_Section_VersionRange>* pxItStateTableSectionVersion = xToolStateTableSection.m_xList_xVersionRanges.Front(); NULL != pxItStateTableSectionVersion; pxItStateTableSectionVersion = pxItStateTableSectionVersion->Next())
			{
				int iCountInner = iCount;

				const Tools_Botanical_StateTable1D_Section_VersionRange& xVersionRange = pxItStateTableSectionVersion->Data();
								
				Tools_BotanicalVersion *pxVersion = pxToolsShapeSection->m_xHashTree_xVersions.GetDataByName( xVersionRange.m_xTargetVersion.GetString() );

				int iNumVersions = pxToolsShapeSection->m_xHashTree_xVersions.GetNumberOfNodes();

				for(int j=0;j<iNumVersions;j++)
				{
					if (pxVersion==pxToolsShape->GetVersionByIndex(iCountInner))
					{
						if ((uint32)iCountInner==uSectionVersionIdx)
						{			
							if (s_bShowCurrentVersionOnly)
							{
								// Showing only the currently selected version
								if (iSectionListIndex == s_pxOverlay->GetSectionListSelectionIdx() &&
									iVersionListIndex == s_pxOverlay->GetVersionListSelectionIdx())
								{
									return true;
								}
							}
							else
							{			
								// Previewing a specific state value
								if (s_iStateValueToPreview>=xVersionRange.m_fRangeStart && s_iStateValueToPreview<xVersionRange.m_fRangeEnd)
								{
									return true;
								}							
							}
						}
					}

					iCountInner++;
				}

				iVersionListIndex++;
			}						
		}						
		
		iCount+=pxToolsShapeSection->m_xHashTree_xVersions.GetNumberOfNodes();			

		iSectionListIndex++;
	}




	return false;
}
#endif

//*******************************************************************
// Overlay
//*******************************************************************

void Asura_Overlay_Botanical_State_Tables::Update()
{
	if( true == Tools_BotanicalCreation_StateTables::s_bShowCurrentVersionOnlyChanged)
	{
		Tools_BotanicalCreation_StateTables::s_bPreviewStateValue = false;
		
		Tools_BotanicalCreation_StateTables::s_bShowCurrentVersionOnlyChanged = false;
		
		Tools_Mode_BotanicalCreation::SetRenderStateToShowAllElements();
	}

	if( true == Tools_BotanicalCreation_StateTables::s_bPreviewStateValueChanged)
	{
		Tools_BotanicalCreation_StateTables::s_bShowCurrentVersionOnly = false;
		
		Tools_BotanicalCreation_StateTables::s_bPreviewStateValueChanged = false;
		
		Tools_Mode_BotanicalCreation::SetRenderStateToShowAllElements();
	}

	if( true == this->m_bDataChanged)
	{
		Tools_Botanical_StateTable1D_Section_VersionRange* pxStateSectionVersion = this->m_pxStateTableSectionVersion;
		
		if( NULL != pxStateSectionVersion)
		{
			pxStateSectionVersion->m_fRangeStart   = this->m_fVersionStart;
			pxStateSectionVersion->m_fRangeEnd	   = this->m_fVersionStop;
			pxStateSectionVersion->m_uProjectFlags = this->m_uProjectFlags;

			pxStateSectionVersion->m_fCollisionMeshDensity		= this->m_fVersionCollisionMeshDensity;
			pxStateSectionVersion->m_xCollisionMeshHash			= this->m_xVersionCollisionMeshHash;
			pxStateSectionVersion->m_xCollisionMeshName			= this->m_xVersionCollisionMeshName;
			pxStateSectionVersion->m_bCollisionUseBoundingBox   = this->m_bVersionCollisionUseBoundingBox;
		}
		
		this->m_bDataChanged = false;

		this->m_pxToolShape->ValidateStateTable();
	}
}

bool Asura_Overlay_Botanical_State_Tables::CreateControls()
{
	Asura_Dialog*  pxNewDialog;
	Asura_Vector_2 xOffset = Asura_Vector_2(1.0f,3.0f);
	const float fSpacingY = 1.0f * Asura_Dialog::DefaultSpacing();
	const float fSpacingX = 1.0f * Asura_Dialog::DefaultSpacing();
	const float fMinimumColumSpacingX = 15.0f * fSpacingX;

	const int ipxOverlay = this->GetThisAsInt();
	
	Asura_Vector_2 xOffsetColumTop = xOffset;
	Asura_Vector_2 xSizeDialog;
	
	{
		// DLH - 07 December 2006 10:55:01 - State Control
		Asura_Vector_2 xSizeColum( xOffset.x + fMinimumColumSpacingX, 0);

		pxNewDialog = new Asura_Dialog("States", xOffset);
		this->AddDialog( pxNewDialog);	
		xOffset.y += fSpacingY;
		xSizeColum.x = Asura_Maths::Max( xSizeColum.x, (pxNewDialog->GetMaxCoords()).x);
		
		pxNewDialog = this->CreateStateDropDown( xOffset);
		this->AddDialog( pxNewDialog);	
		xSizeDialog = pxNewDialog->GetMaxCoords();
		xOffset.y = xSizeDialog.y;
		xSizeColum.x = Asura_Maths::Max( xSizeColum.x, xSizeDialog.x);
		
		pxNewDialog = new Asura_Function_Dialog("Add", xOffset, Asura_Overlay_Botanical_State_Tables::AddStateButton, ipxOverlay);
		this->AddDialog(pxNewDialog);	
		xOffset.y += fSpacingY;
		xSizeColum.x = Asura_Maths::Max( xSizeColum.x, (pxNewDialog->GetMaxCoords()).x);
			
		pxNewDialog = new Asura_Function_Dialog("Delete", xOffset, Asura_Overlay_Botanical_State_Tables::DeleteStateButton, ipxOverlay);
		this->AddDialog(pxNewDialog);	
		xOffset.y += fSpacingY;
		xSizeColum.x = Asura_Maths::Max( xSizeColum.x, (pxNewDialog->GetMaxCoords()).x);
			
		pxNewDialog = new Asura_Function_Dialog("Rename", xOffset, Asura_Overlay_Botanical_State_Tables::RenameStateButton, ipxOverlay);
		this->AddDialog(pxNewDialog);	
		xOffset.y += Asura_Dialog::LargeSpacing();
		xSizeColum.x = Asura_Maths::Max( xSizeColum.x, (pxNewDialog->GetMaxCoords()).x);
		

		// Extra options to control what gets rendered
		pxNewDialog = new Asura_Tickbox_Dialog("Show selected version only", xOffset, &Tools_BotanicalCreation_StateTables::s_bShowCurrentVersionOnly, &Tools_BotanicalCreation_StateTables::s_bShowCurrentVersionOnlyChanged);
		this->AddDialog(pxNewDialog);	
		xOffset.y += fSpacingY;

		pxNewDialog = new Asura_Tickbox_Dialog("Preview at state value", xOffset, &Tools_BotanicalCreation_StateTables::s_bPreviewStateValue, &Tools_BotanicalCreation_StateTables::s_bPreviewStateValueChanged);
		this->AddDialog(pxNewDialog);			
		xOffset.x += 16.0f;		
		pxNewDialog = new Asura_Numerical_Dialog("", xOffset, &Tools_BotanicalCreation_StateTables::s_iStateValueToPreview,0,254,4,"%d");
		this->AddDialog(pxNewDialog);			
		xOffset.x -= 16.0f;
		xOffset.y += fSpacingY;

		pxNewDialog = new Asura_Tickbox_Dialog("Show foliage collision meshes", xOffset, &Tools_BotanicalCreation_StateTables::s_bShowFoliageCollisionMeshes, &Tools_BotanicalCreation_StateTables::s_bShowFoliageCollisionMeshesChanged);
		this->AddDialog(pxNewDialog);	
		xOffset.y += fSpacingY;

		xOffsetColumTop.x = xSizeColum.x + fSpacingX;
		//xOffsetColumTop.x += xSizeColum.x + fSpacingX;
	}
	
	xOffset = xOffsetColumTop;
	
	{
		// DLH - 07 December 2006 10:55:17 - State Section Control
		Asura_Vector_2 xSizeColum( xOffset.x + fMinimumColumSpacingX, 0);
			
		pxNewDialog = new Asura_Dialog("Sections", xOffset);
		this->AddDialog( pxNewDialog);	
		xOffset.y += fSpacingY;
		xSizeColum.x = Asura_Maths::Max( xSizeColum.x, (pxNewDialog->GetMaxCoords()).x);
		
		pxNewDialog = this->CreateStateSectionList( xOffset);
		this->AddDialog( pxNewDialog);	
		xSizeDialog = pxNewDialog->GetMaxCoords();
		xOffset.y = xSizeDialog.y;
		xSizeColum.x = Asura_Maths::Max( xSizeColum.x, xSizeDialog.x);
		
		pxNewDialog = new Asura_Function_Dialog("Add", xOffset, Asura_Overlay_Botanical_State_Tables::AddStateSectionButton, ipxOverlay);
		this->AddDialog(pxNewDialog);	
		xOffset.y += fSpacingY;
		xSizeColum.x = Asura_Maths::Max( xSizeColum.x, (pxNewDialog->GetMaxCoords()).x);
			
		pxNewDialog = new Asura_Function_Dialog("Delete", xOffset, Asura_Overlay_Botanical_State_Tables::DeleteStateSectionButton, ipxOverlay);
		this->AddDialog(pxNewDialog);	
		xOffset.y += Asura_Dialog::LargeSpacing();
		xSizeColum.x = Asura_Maths::Max( xSizeColum.x, (pxNewDialog->GetMaxCoords()).x);
			
		xOffsetColumTop.x = xSizeColum.x + fSpacingX;
		//xOffsetColumTop.x += xSizeColum.x + fSpacingX;
	}
	
	xOffset = xOffsetColumTop;
	
	{
		// DLH - 07 December 2006 10:55:17 - State Section Control
		Asura_Vector_2 xSizeColum( xOffset.x + fMinimumColumSpacingX, 0);
		
		pxNewDialog = new Asura_Dialog("Versions", xOffset);
		this->AddDialog( pxNewDialog);	
		xOffset.y += fSpacingY;
		xSizeColum.x = Asura_Maths::Max( xSizeColum.x, (pxNewDialog->GetMaxCoords()).x);
		
		pxNewDialog = this->CreateStateSectionVersionList( xOffset);
		this->AddDialog( pxNewDialog);	
		xSizeDialog = pxNewDialog->GetMaxCoords();
		xOffset.y = xSizeDialog.y;
		xSizeColum.x = Asura_Maths::Max( xSizeColum.x, xSizeDialog.x);
		
		pxNewDialog = this->m_pxStateSectionVersion_Add = new Asura_Function_Dialog("Add", xOffset, Asura_Overlay_Botanical_State_Tables::AddStateSectionVersionButton, ipxOverlay);
		this->AddDialog(pxNewDialog);	
		xOffset.y += fSpacingY;
		xSizeColum.x = Asura_Maths::Max( xSizeColum.x, (pxNewDialog->GetMaxCoords()).x);
			
		pxNewDialog = this->m_pxStateSectionVersion_Delete = new Asura_Function_Dialog("Delete", xOffset, Asura_Overlay_Botanical_State_Tables::DeleteStateSectionVersionButton, ipxOverlay);
		this->AddDialog(pxNewDialog);	
		xOffset.y += Asura_Dialog::LargeSpacing();
		xSizeColum.x = Asura_Maths::Max( xSizeColum.x, (pxNewDialog->GetMaxCoords()).x);

			
		pxNewDialog = this->CreateSectionVersionDropDown( xOffset);
		this->AddDialog( pxNewDialog);	
		xOffset.y += Asura_Dialog::LargeSpacing();
		xSizeColum.x = Asura_Maths::Max( xSizeColum.x, (pxNewDialog->GetMaxCoords()).x);
		

		pxNewDialog = this->CreateStateSectionVersionStart( xOffset);
		this->AddDialog( pxNewDialog);	
		xOffset.y += fSpacingY;
		xSizeColum.x = Asura_Maths::Max( xSizeColum.x, (pxNewDialog->GetMaxCoords()).x);
		
		pxNewDialog = this->CreateStateSectionVersionStop( xOffset);
		this->AddDialog( pxNewDialog);	
		xOffset.y += Asura_Dialog::LargeSpacing();
		xSizeColum.x = Asura_Maths::Max( xSizeColum.x, (pxNewDialog->GetMaxCoords()).x);

		
		// 'Use Bounding Box'
		pxNewDialog = this->m_pxStateSectionVersion_CollisionUseBoundingBox = new Asura_Tickbox_Dialog("Use instance BB as Foliage Collision Mesh", xOffset, &this->m_bVersionCollisionUseBoundingBox, &(this->m_bDataChanged));
		pxNewDialog->SetHelpText("Not recommended. Unless you've got cuboid-shaped botanicals...");
		this->AddDialog(pxNewDialog);
		xOffset.y += Asura_Dialog::LargeSpacing();
		xSizeColum.x = Asura_Maths::Max( xSizeColum.x, (pxNewDialog->GetMaxCoords()).x);

		// Create the collision mesh selector
		pxNewDialog = this->m_pxStateSectionVersion_SetCollisionMesh = new Asura_Function_Dialog("Set Foliage Collision Mesh (Skin)", xOffset, Asura_Overlay_Botanical_State_Tables::SetStateSectionVersionCollMesh, ipxOverlay);
		this->AddDialog(pxNewDialog);
		xOffset.y += Asura_Dialog::LargeSpacing();
		xSizeColum.x = Asura_Maths::Max( xSizeColum.x, (pxNewDialog->GetMaxCoords()).x);

		// Collision mesh density selector
		pxNewDialog = this->m_pxStateSectionVersion_CollisionMeshDensity = new Asura_Numerical_Dialog("Collision Mesh Density", xOffset, &this->m_fVersionCollisionMeshDensity, 0.0f, 1.0f, 0.01f, "%0.03f", &(this->m_bDataChanged));
		this->AddDialog(pxNewDialog);
		xOffset.y += Asura_Dialog::LargeSpacing();
		xSizeColum.x = Asura_Maths::Max( xSizeColum.x, (pxNewDialog->GetMaxCoords()).x);

		Project_CreateVersionOverlay(xOffset, xSizeColum.x);
		

		xOffsetColumTop.x = xSizeColum.x + fSpacingX;
		//xOffsetColumTop.x += xSizeColum.x + fSpacingX;
	}
	
	{
		// DLH - 07 December 2006 10:55:24 - Version Control
	}
	
	this->SetStateSectionVersionData();
	this->SetSectionVersionSelection();
	
	return( true);
}


//*******************************************************************
// States
//*******************************************************************

void Asura_Overlay_Botanical_State_Tables::CreateStateDropDownNameArray()
{
	this->DeleteStateDropDownNameArray();
	
	this->m_xArray_pszStateDropDownNames.SetSize( this->m_pxToolShape->m_xList_xStateTables1D.Size());
	
	int iStateDropDownNameIndex = 0;
	for( const Asura_ListElt< Tools_Botanical_StateTable1D>* pxItStateTable = this->m_pxToolShape->m_xList_xStateTables1D.Front(); NULL != pxItStateTable; pxItStateTable = pxItStateTable->Next(), iStateDropDownNameIndex += 1)
	{
		const Tools_Botanical_StateTable1D& xToolStateTable = pxItStateTable->Data();
		char*& pszStateDropDownName = this->m_xArray_pszStateDropDownNames[ iStateDropDownNameIndex];
		
		const Asura_String& xToolSectionName = xToolStateTable.m_xSourceState;
		
		if( 0 != xToolSectionName.Compare( ""))
		{
			pszStateDropDownName = Asura_StringUtil::Duplicate( xToolSectionName.GetString());
			continue;
		}
		
		pszStateDropDownName = Asura_StringUtil::Duplicate( "DEFAULT STATE");
	}
	
	this->m_iStateDropDownNameCount = iStateDropDownNameIndex;
	
	if( 0 == this->m_xArray_pszStateDropDownNames.GetSize())
	{
		// DLH - 08 December 2006 13:51:11 - The list is emtpy but the Asura_list_Dialog require that the name array has at least one item
		this->m_xArray_pszStateDropDownNames.SetSize( 1);
		this->m_xArray_pszStateDropDownNames[ 0 ] = Asura_StringUtil::Duplicate( "######");
	}
}

void Asura_Overlay_Botanical_State_Tables::DeleteStateDropDownNameArray()
{
	const int iStateDropDownNameCount = m_xArray_pszStateDropDownNames.GetSize();
	for( int iStateDropDownNameIndex = 0; iStateDropDownNameIndex < iStateDropDownNameCount; iStateDropDownNameIndex += 1)
	{
		char*& pszStateDropDownName = this->m_xArray_pszStateDropDownNames[ iStateDropDownNameIndex];
		
		delete[] pszStateDropDownName;
		pszStateDropDownName = NULL;
	}
	
	this->m_xArray_pszStateDropDownNames.Reset();
	this->m_iStateDropDownNameCount = 0;
	this->m_pxStateTable = NULL;
}

void Asura_Overlay_Botanical_State_Tables::CreateState()
{
	Asura_String xNewStateName = this->CreateStateUniqueName();
	
	this->m_iStateDropDownNameSelected = this->m_pxToolShape->m_xList_xStateTables1D.Size();
	Asura_ListElt< Tools_Botanical_StateTable1D>* pxItStateTable = this->m_pxToolShape->m_xList_xStateTables1D.Push_Back( Tools_Botanical_StateTable1D());
	Tools_Botanical_StateTable1D& xToolStateTable = pxItStateTable->Data();
	
	xToolStateTable.m_xSourceState = xNewStateName;
	this->m_pxStateTable = &xToolStateTable;
	
	this->SetupStatePointerFromSelection();
}

void Asura_Overlay_Botanical_State_Tables::DeleteState()
{
	int iStateDropDownNameIndex = 0;
	for( Asura_ListElt< Tools_Botanical_StateTable1D>* pxItStateTable = this->m_pxToolShape->m_xList_xStateTables1D.Front(); NULL != pxItStateTable; pxItStateTable = pxItStateTable->Next(), iStateDropDownNameIndex += 1)
	{
		if( this->m_iStateDropDownNameSelected != iStateDropDownNameIndex)
		{
			continue;
		}
		
		this->m_pxToolShape->m_xList_xStateTables1D.Remove( pxItStateTable);
		break;
	}
	
	const int iStateDropDownNameCount = this->m_pxToolShape->m_xList_xStateTables1D.Size();
	if( 0 == iStateDropDownNameCount)
	{
		this->m_iStateDropDownNameSelected = 0;
	}
	else
	{
		if( iStateDropDownNameCount <= this->m_iStateDropDownNameSelected )
		{
			this->m_iStateDropDownNameSelected = iStateDropDownNameCount - 1;
		}
	}

	this->SetupStatePointerFromSelection();
}

Asura_String Asura_Overlay_Botanical_State_Tables::CreateStateUniqueName()
{
	const u_int iTryMax = 1000;
	for( u_int iTry = 1; iTry < iTryMax; iTry += 1)
	{
		// DLH - 07 December 2006 10:40:09 - I was trying to use a Asura_String here but could not find out how to do the printf type functions
		const u_int uNewNameSize = 20;
		char szNewName[ uNewNameSize + 1];
		
		Asura_PrintfToBfr( szNewName, "State%d", uNewNameSize, true, iTry);
		
		bool bFoundName = false;
		
		for( const Asura_ListElt< Tools_Botanical_StateTable1D>* pxItStateTable = this->m_pxToolShape->m_xList_xStateTables1D.Front(); NULL != pxItStateTable; pxItStateTable = pxItStateTable->Next())
		{
			const Tools_Botanical_StateTable1D& xToolStateTable = pxItStateTable->Data();
			
			const Asura_String& xToolSectionName = xToolStateTable.m_xSourceState;
			
			if( 0 == xToolSectionName.Compare( szNewName))
			{
				bFoundName = true;
				break;
			}
		}
		
		if( false == bFoundName)
		{
			return( szNewName);
		}
	}
	
	Asura_Assert( false, "Cannot find a new unique name for a Section");
	return( Asura_String( "UnsafeSectionName"));
}

void Asura_Overlay_Botanical_State_Tables::UpdateStateDropDown()
{
	this->CreateStateDropDownNameArray();

	char** ppszList = &(this->m_xArray_pszStateDropDownNames[ 0]);
	this->m_pxState_List->ReplaceStrings( this->m_iStateDropDownNameCount, ppszList);

	this->SetupStatePointerFromSelection();
}

Asura_Dialog* Asura_Overlay_Botanical_State_Tables::CreateStateDropDown( Asura_Vector_2& xOffset)
{
	this->m_iStateDropDownNameSelected =  0;

	this->CreateStateDropDownNameArray();

	char** ppszList = &(this->m_xArray_pszStateDropDownNames[ 0]);

	this->m_pxState_List = new Asura_List_Dialog( xOffset, ppszList, &(this->m_iStateDropDownNameSelected), STATE_TABLE_DIALOG_LIST_SIZE, this->m_iStateDropDownNameCount, NULL, NULL, 0, &(this->m_bDataChanged));
	this->m_pxState_List->SetSingleClickFunction( this->SelectionChangedStateDropDown, this->GetThisAsInt());

	this->SetupStatePointerFromSelection();

	return( this->m_pxState_List);
}

void Asura_Overlay_Botanical_State_Tables::SetupStatePointerFromSelection()
{
	this->m_pxStateTable = NULL;

	int iStateDropDownNameIndex = 0;
	for( Asura_ListElt< Tools_Botanical_StateTable1D>* pxItStateTable = this->m_pxToolShape->m_xList_xStateTables1D.Front(); NULL != pxItStateTable; pxItStateTable = pxItStateTable->Next(), iStateDropDownNameIndex += 1)
	{
		Tools_Botanical_StateTable1D& xToolStateTable = pxItStateTable->Data();
		
		if( this->m_iStateDropDownNameSelected != iStateDropDownNameIndex)
		{
			continue;
		}
		
		this->m_pxStateTable = &xToolStateTable;
		return;
	}
}

void Asura_Overlay_Botanical_State_Tables::SelectionChangedStateDropDown( int ipxData)
{
	Asura_Overlay_Botanical_State_Tables* pxOverlay = GetIntAsClass( ipxData);

	pxOverlay->SetupStatePointerFromSelection();
	
	pxOverlay->ChangeState();
}

void Asura_Overlay_Botanical_State_Tables::AddStateButton( int ipxData)
{
	Asura_Overlay_Botanical_State_Tables* pxOverlay = GetIntAsClass( ipxData);
	
	pxOverlay->CreateState();
	
	pxOverlay->UpdateStateDropDown();
	pxOverlay->ChangeState();
	
	pxOverlay->m_bDataChanged = true;
}

void Asura_Overlay_Botanical_State_Tables::DeleteStateButton( int ipxData)
{
	Asura_Overlay_Botanical_State_Tables* pxOverlay = GetIntAsClass( ipxData);

	pxOverlay->DeleteState();

	pxOverlay->UpdateStateDropDown();
	pxOverlay->ChangeState();
	
	pxOverlay->m_bDataChanged = true;
}

void Asura_Overlay_Botanical_State_Tables::RenameStateButton( int ipxData)
{
	Asura_Overlay_Botanical_State_Tables* pxOverlay = GetIntAsClass( ipxData);

	GetString_Overlay xOverlay( "Rename State", pxOverlay->m_pxStateTable->m_xSourceState.GetString());

	if( false == xOverlay.DoModal())
	{
		return;
	}

	const char* pszRenamedName = xOverlay.GetString();
	
	for( Asura_ListElt< Tools_Botanical_StateTable1D>* pxItStateTable = pxOverlay->m_pxToolShape->m_xList_xStateTables1D.Front(); NULL != pxItStateTable; pxItStateTable = pxItStateTable->Next())
	{
		Tools_Botanical_StateTable1D& xToolStateTable = pxItStateTable->Data();

		if( 0 != xToolStateTable.m_xSourceState.Compare( pszRenamedName))
		{
			continue;
		}

		Message_Overlay xOverlayMessage( "State Already Exists");
		xOverlayMessage.DoModal();
		return;
	}
	
	pxOverlay->m_pxStateTable->m_xSourceState = pszRenamedName;
	
	pxOverlay->UpdateStateDropDown();

	pxOverlay->m_bDataChanged = true;
}

void Asura_Overlay_Botanical_State_Tables::ChangeState()
{
	this->m_iStateSectionVersionListNameSelected = 0;
	
	this->UpdateStateSectionList();	
	this->ChangeStateSection();
}

//*******************************************************************
// State Sections
//*******************************************************************

void Asura_Overlay_Botanical_State_Tables::CreateStateSectionListNameArray()
{
	this->DeleteStateSectionListNameArray();
	
	Tools_Botanical_StateTable1D* pxStateTable = this->m_pxStateTable;
	if( NULL != pxStateTable)
	{
		this->m_xArray_pszStateSectionListNames.SetSize( pxStateTable->m_xList_xSection.Size());
		
		int iStateSectionNameIndex = 0;
		for( const Asura_ListElt< Tools_Botanical_StateTable1D_Section>* pxItStateTableSection = pxStateTable->m_xList_xSection.Front(); NULL != pxItStateTableSection; pxItStateTableSection = pxItStateTableSection->Next(), iStateSectionNameIndex += 1)
		{
			const Tools_Botanical_StateTable1D_Section& xToolStateTableSection = pxItStateTableSection->Data();
			char*& pszStateSectionName = this->m_xArray_pszStateSectionListNames[ iStateSectionNameIndex];
			
			const Asura_String& xToolSectionSectionName = xToolStateTableSection.m_xTargetSection;
			
			if( 0 != xToolSectionSectionName.Compare( ""))
			{
				pszStateSectionName = Asura_StringUtil::Duplicate( xToolSectionSectionName.GetString());
				continue;
			}
			
			pszStateSectionName = Asura_StringUtil::Duplicate( "DEFAULT SECTION");
		}
		
		this->m_iStateSectionListNameCount = iStateSectionNameIndex;
	}
	
	if( 0 == this->m_xArray_pszStateSectionListNames.GetSize())
	{
		this->m_xArray_pszStateSectionListNames.SetSize( 1);
		this->m_xArray_pszStateSectionListNames[0] = Asura_StringUtil::Duplicate( "#######");
	}
}

void Asura_Overlay_Botanical_State_Tables::DeleteStateSectionListNameArray()
{
	const int iStateSectionNameCount = m_xArray_pszStateSectionListNames.GetSize();
	for( int iStateSectionNameIndex = 0; iStateSectionNameIndex < iStateSectionNameCount; iStateSectionNameIndex += 1)
	{
		char*& pszStateSectionName = this->m_xArray_pszStateSectionListNames[ iStateSectionNameIndex];
		
		delete[] pszStateSectionName;
		pszStateSectionName = NULL;
	}
	
	this->m_xArray_pszStateSectionListNames.Reset();
	this->m_iStateSectionListNameCount = 0;
	this->m_pxStateTableSection = NULL;
}

void Asura_Overlay_Botanical_State_Tables::UpdateStateSectionList()
{
	this->CreateStateSectionListNameArray();
	
	char** ppszList = &(this->m_xArray_pszStateSectionListNames[ 0]);
	this->m_pxStateSection_List->ReplaceStrings( this->m_iStateSectionListNameCount, ppszList);
	
	this->SetupStateSectionPointerFromSelection();
}

Asura_Dialog* Asura_Overlay_Botanical_State_Tables::CreateStateSectionList( Asura_Vector_2& xOffset)
{
	this->m_iStateSectionListNameSelected =  0;
	
	this->CreateStateSectionListNameArray();
	
	char** ppszList = &(this->m_xArray_pszStateSectionListNames[ 0]);
	this->m_pxStateSection_List = new Asura_List_Dialog( xOffset, ppszList, &(this->m_iStateSectionListNameSelected), STATE_TABLE_DIALOG_LIST_SIZE, this->m_iStateSectionListNameCount, NULL, NULL, 0, &(this->m_bDataChanged));
	this->m_pxStateSection_List->SetSingleClickFunction( this->SelectionChangedStateSectionList, this->GetThisAsInt());

	this->SetupStateSectionPointerFromSelection();
	
	return( this->m_pxStateSection_List);
}

void Asura_Overlay_Botanical_State_Tables::SetupStateSectionPointerFromSelection()
{
	this->m_pxStateTableSection = NULL;
	this->m_pxToolShapeSection = NULL;
	
	Tools_Botanical_StateTable1D* pxStateTable = this->m_pxStateTable;
	if( NULL == pxStateTable)
	{
		return;
	}
	
	int iStateSectionNameIndex = 0;
	for( Asura_ListElt< Tools_Botanical_StateTable1D_Section>* pxItStateTableSection = pxStateTable->m_xList_xSection.Front(); NULL != pxItStateTableSection; pxItStateTableSection = pxItStateTableSection->Next(), iStateSectionNameIndex += 1)
	{
		Tools_Botanical_StateTable1D_Section& xToolStateTableSection = pxItStateTableSection->Data();
		
		if( this->m_iStateSectionListNameSelected != iStateSectionNameIndex)
		{
			continue;
		}
		
		this->m_pxStateTableSection = &xToolStateTableSection;
		this->m_pxToolShapeSection = this->m_pxToolShape->m_xHashTree_xSections.GetDataByName( xToolStateTableSection.m_xTargetSection.GetString());
		return;
	}
}

bool Asura_Overlay_Botanical_State_Tables::CreateStateSection_GetName( Asura_String& xSectionName)
{
	Tools_Botanical_StateTable1D* pxStateTable = this->m_pxStateTable;
	Tools_BotanicalShape* pxToolsShape = this->m_pxToolShape;
	
	if( NULL == pxToolsShape || NULL == pxStateTable)
	{
		return( false);
	}
	
	ListSelection_Overlay xOverlay("Select Section");
	
	const int iSectionCountMax = pxToolsShape->m_xHashTree_xSections.GetNumberOfNodes();
	for( int iSectionIndex = 0; iSectionIndex < iSectionCountMax; iSectionIndex += 1)
	{
		Tools_BotanicalSection* pxToolsSection = pxToolsShape->m_xHashTree_xSections.GetDataByIndex( iSectionIndex);
		
		bool bFoundMatch = false;
		for( Asura_ListElt< Tools_Botanical_StateTable1D_Section>* pxItStateTableSection = pxStateTable->m_xList_xSection.Front(); NULL != pxItStateTableSection; pxItStateTableSection = pxItStateTableSection->Next())
		{
			Tools_Botanical_StateTable1D_Section& xStateTableSection = pxItStateTableSection->Data();
			
			if( 0 != xStateTableSection.m_xTargetSection.Compare( pxToolsSection->m_xName))
			{
				continue;
			}
			
			bFoundMatch = true;
			break;
		}
		
		if( true == bFoundMatch)
		{
			continue;
		}
		
		if( 0 != pxToolsSection->m_xName.Compare( ""))
		{
			xOverlay.AddItem( pxToolsSection->m_xName.GetString(), iSectionIndex);
		}
		else
		{
			xOverlay.AddItem( "DEFAULT SECTION", iSectionIndex);
		}
	}
		
	if( false == xOverlay.DoModal())
	{
		return( false);
	}
	
	int iSectionIndex = xOverlay.GetSelectionData();
	Tools_BotanicalSection* pxToolsSection = pxToolsShape->m_xHashTree_xSections.GetDataByIndex( iSectionIndex);
					
	xSectionName = pxToolsSection->m_xName;
	
	return( true);
}

bool Asura_Overlay_Botanical_State_Tables::CreateStateSection()
{
	Asura_String xStateSectionName;
	if( false == this->CreateStateSection_GetName( xStateSectionName))
	{
		return( false);
	}
	
	this->m_iStateSectionListNameSelected = this->m_pxStateTable->m_xList_xSection.Size();
	Asura_ListElt< Tools_Botanical_StateTable1D_Section>* pxItSection = this->m_pxStateTable->m_xList_xSection.Push_Back( Tools_Botanical_StateTable1D_Section());
	Tools_Botanical_StateTable1D_Section& xItSection = pxItSection->Data();
	
	xItSection.m_xTargetSection = xStateSectionName;
	
	this->SetupStateSectionPointerFromSelection();
	return( true);
}

void Asura_Overlay_Botanical_State_Tables::DeleteStateSection()
{
	Tools_Botanical_StateTable1D* pxStateTable = this->m_pxStateTable;
	if( NULL == pxStateTable)
	{
		return;
	}
	
	int iStateSectionNameIndex = 0;
	for( Asura_ListElt< Tools_Botanical_StateTable1D_Section>* pxItStateTableSection = pxStateTable->m_xList_xSection.Front(); NULL != pxItStateTableSection; pxItStateTableSection = pxItStateTableSection->Next(), iStateSectionNameIndex += 1)
	{
		if( this->m_iStateSectionListNameSelected != iStateSectionNameIndex)
		{
			continue;
		}
		
		pxStateTable->m_xList_xSection.Remove( pxItStateTableSection);
		break;
	}
	
	const int iStateSectionNameCount = pxStateTable->m_xList_xSection.Size();
	if( 0 == iStateSectionNameCount)
	{
		this->m_iStateSectionListNameSelected = 0;
	}
	else
	{
		if( iStateSectionNameCount <= this->m_iStateSectionListNameSelected )
		{
			this->m_iStateSectionListNameSelected = iStateSectionNameCount - 1;
		}
	}

	this->SetupStateSectionPointerFromSelection();
}

void Asura_Overlay_Botanical_State_Tables::SelectionChangedStateSectionList( int ipxData)
{
	Asura_Overlay_Botanical_State_Tables* pxOverlay = GetIntAsClass( ipxData);
	
	pxOverlay->SetupStateSectionPointerFromSelection();

	pxOverlay->ChangeStateSection();
}

void Asura_Overlay_Botanical_State_Tables::AddStateSectionButton( int ipxData)
{
	Asura_Overlay_Botanical_State_Tables* pxOverlay = GetIntAsClass( ipxData);

	if( false == pxOverlay->CreateStateSection())
	{
		return;
	}
	
	pxOverlay->UpdateStateSectionList();	
	pxOverlay->ChangeStateSection();
	pxOverlay->m_bDataChanged = true;
}

void Asura_Overlay_Botanical_State_Tables::DeleteStateSectionButton( int ipxData)
{
	Asura_Overlay_Botanical_State_Tables* pxOverlay = GetIntAsClass( ipxData);

	pxOverlay->DeleteStateSection();
	
	pxOverlay->UpdateStateSectionList();	
	pxOverlay->ChangeStateSection();
	pxOverlay->m_bDataChanged = true;
}

void Asura_Overlay_Botanical_State_Tables::ChangeStateSection()
{
	this->UpdateStateSectionVersionList();
	this->ChangeStateSectionVersion();
	this->ChangeSectionVersion();
}

//*******************************************************************
// State Sections Version
//*******************************************************************

void Asura_Overlay_Botanical_State_Tables::CreateStateSectionVersionListNameArray()
{
	this->DeleteStateSectionVersionListNameArray();
	
	Tools_Botanical_StateTable1D_Section* pxStateTableSection = this->m_pxStateTableSection;
	if( NULL != pxStateTableSection)
	{
		this->m_xArray_pszStateSectionVersionListNames.SetSize( pxStateTableSection->m_xList_xVersionRanges.Size());
		
		int iStateSectionVersionNameIndex = 0;
		for( const Asura_ListElt< Tools_Botanical_StateTable1D_Section_VersionRange>* pxItStateTableSectionVersion = pxStateTableSection->m_xList_xVersionRanges.Front(); NULL != pxItStateTableSectionVersion; pxItStateTableSectionVersion = pxItStateTableSectionVersion->Next(), iStateSectionVersionNameIndex += 1)
		{
			const Tools_Botanical_StateTable1D_Section_VersionRange& xToolStateTableSectionVersion = pxItStateTableSectionVersion->Data();
			char*& pszStateSectionVersionName = this->m_xArray_pszStateSectionVersionListNames[ iStateSectionVersionNameIndex];
			
			const Asura_String& xToolSectionSectionVersionName = xToolStateTableSectionVersion.m_xTargetVersion;
			
			if( 0 != xToolSectionSectionVersionName.Compare( ""))
			{
				pszStateSectionVersionName = Asura_StringUtil::Duplicate( xToolSectionSectionVersionName.GetString());
				continue;
			}
			
			pszStateSectionVersionName = Asura_StringUtil::Duplicate( "DEFAULT VERSION");
		}
		
		this->m_iStateSectionVersionListNameCount = iStateSectionVersionNameIndex;
	}
	
	if( 0 == this->m_xArray_pszStateSectionVersionListNames.GetSize())
	{
		this->m_xArray_pszStateSectionVersionListNames.SetSize( 1);
		this->m_xArray_pszStateSectionVersionListNames[0] = Asura_StringUtil::Duplicate( "#######");
	}
}

void Asura_Overlay_Botanical_State_Tables::DeleteStateSectionVersionListNameArray()
{
	const int iStateSectionVersionNameCount = m_xArray_pszStateSectionVersionListNames.GetSize();
	for( int iStateSectionVersionNameIndex = 0; iStateSectionVersionNameIndex < iStateSectionVersionNameCount; iStateSectionVersionNameIndex += 1)
	{
		char*& pszStateSectionVersionName = this->m_xArray_pszStateSectionVersionListNames[ iStateSectionVersionNameIndex];
		
		delete[] pszStateSectionVersionName;
		pszStateSectionVersionName = NULL;
	}
	
	this->m_xArray_pszStateSectionVersionListNames.Reset();
	this->m_iStateSectionVersionListNameCount = 0;
	this->m_pxStateTableSectionVersion = NULL;
}

void Asura_Overlay_Botanical_State_Tables::CreateStateSectionVersion()
{
	Asura_String xStateSectionVersionName = "";
	
	this->m_iStateSectionVersionListNameSelected = this->m_pxStateTableSection->m_xList_xVersionRanges.Size();
	Asura_ListElt< Tools_Botanical_StateTable1D_Section_VersionRange>* pxItVersion = this->m_pxStateTableSection->m_xList_xVersionRanges.Push_Back( Tools_Botanical_StateTable1D_Section_VersionRange());
	Tools_Botanical_StateTable1D_Section_VersionRange& xItVersion = pxItVersion->Data();
	
	xItVersion.m_fRangeStart = 0;
	xItVersion.m_fRangeEnd = 255;
	xItVersion.m_uProjectFlags = 0;
	xItVersion.m_xTargetVersion = xStateSectionVersionName;
	
	this->SetupStateSectionVersionPointerFromSelection();
}

void Asura_Overlay_Botanical_State_Tables::DeleteStateSectionVersion()
{
	Tools_Botanical_StateTable1D_Section* pxStateTableSection = this->m_pxStateTableSection;
	if( NULL == pxStateTableSection)
	{
		return;
	}
	
	int iStateSectionVersionNameIndex = 0;
	for( Asura_ListElt< Tools_Botanical_StateTable1D_Section_VersionRange>* pxItStateTableSectionVersion = pxStateTableSection->m_xList_xVersionRanges.Front(); NULL != pxItStateTableSectionVersion; pxItStateTableSectionVersion = pxItStateTableSectionVersion->Next(), iStateSectionVersionNameIndex += 1)
	{
		if( this->m_iStateSectionVersionListNameSelected != iStateSectionVersionNameIndex)
		{
			continue;
		}
		
		pxStateTableSection->m_xList_xVersionRanges.Remove( pxItStateTableSectionVersion);
		break;
	}
	
	const int iStateSectionVersionNameCount = pxStateTableSection->m_xList_xVersionRanges.Size();
	if( 0 == iStateSectionVersionNameCount)
	{
		this->m_iStateSectionVersionListNameSelected = 0;
	}
	else
	{
		if( iStateSectionVersionNameCount <= this->m_iStateSectionVersionListNameSelected )
		{
			this->m_iStateSectionVersionListNameSelected = iStateSectionVersionNameCount - 1;
		}
	}

	this->SetupStateSectionVersionPointerFromSelection();
}

void Asura_Overlay_Botanical_State_Tables::UpdateStateSectionVersionList()
{
	this->CreateStateSectionVersionListNameArray();
	
	char** ppszList = &(this->m_xArray_pszStateSectionVersionListNames[ 0]);
	this->m_pxStateSectionVersion_List->ReplaceStrings( this->m_iStateSectionVersionListNameCount, ppszList);
	
	this->SetupStateSectionVersionPointerFromSelection();
}

Asura_Dialog* Asura_Overlay_Botanical_State_Tables::CreateStateSectionVersionList( Asura_Vector_2& xOffset)
{
	this->m_iStateSectionVersionListNameSelected =  0;
	
	this->CreateStateSectionVersionListNameArray();
	
	char** ppszList = &(this->m_xArray_pszStateSectionVersionListNames[ 0]);
	this->m_pxStateSectionVersion_List = new Asura_List_Dialog( xOffset, ppszList, &(this->m_iStateSectionVersionListNameSelected), STATE_TABLE_DIALOG_LIST_SIZE, this->m_iStateSectionVersionListNameCount, NULL, NULL, 0, &(this->m_bDataChanged));
	this->m_pxStateSectionVersion_List->SetSingleClickFunction( this->SelectionChangedStateSectionVersionList, this->GetThisAsInt());

	this->SetupStateSectionVersionPointerFromSelection();
	
	return( this->m_pxStateSectionVersion_List);
}

Asura_Dialog* Asura_Overlay_Botanical_State_Tables::CreateStateSectionVersionStart( Asura_Vector_2& xOffset)
{
	this->m_pxStateSectionVersion_Start = new Asura_Numerical_Dialog( "Start", xOffset, &(this->m_fVersionStart), this->m_fVersionMin, this->m_fVersionMax, 1, STATE_SECTION_VERSION_RANGE_PRINTF_FORMAT, &(this->m_bDataChanged));
	return( this->m_pxStateSectionVersion_Start);
}

Asura_Dialog* Asura_Overlay_Botanical_State_Tables::CreateStateSectionVersionStop( Asura_Vector_2& xOffset)
{
	this->m_pxStateSectionVersion_Stop = new Asura_Numerical_Dialog( "Stop", xOffset, &this->m_fVersionStop, this->m_fVersionMin, this->m_fVersionMax, 1, STATE_SECTION_VERSION_RANGE_PRINTF_FORMAT, &(this->m_bDataChanged));
	return( this->m_pxStateSectionVersion_Stop);
}

void Asura_Overlay_Botanical_State_Tables::SetupStateSectionVersionPointerFromSelection()
{
	this->m_pxStateTableSectionVersion = NULL;
	
	Tools_Botanical_StateTable1D_Section* pxStateTableSection = this->m_pxStateTableSection;
	if( NULL == pxStateTableSection)
	{
		return;
	}
	
	int iStateSectionVersionNameIndex = 0;
	for( Asura_ListElt< Tools_Botanical_StateTable1D_Section_VersionRange>* pxItStateTableSectionVersion = pxStateTableSection->m_xList_xVersionRanges.Front(); NULL != pxItStateTableSectionVersion; pxItStateTableSectionVersion = pxItStateTableSectionVersion->Next(), iStateSectionVersionNameIndex += 1)
	{
		Tools_Botanical_StateTable1D_Section_VersionRange& xToolStateTableSectionVersion = pxItStateTableSectionVersion->Data();
		
		if( this->m_iStateSectionVersionListNameSelected != iStateSectionVersionNameIndex)
		{
			continue;
		}
		
		this->m_pxStateTableSectionVersion = &xToolStateTableSectionVersion;
		return;
	}
}

void Asura_Overlay_Botanical_State_Tables::SelectionChangedStateSectionVersionList( int ipxData)
{
	Asura_Overlay_Botanical_State_Tables* pxOverlay = GetIntAsClass( ipxData);
	
	pxOverlay->SetupStateSectionVersionPointerFromSelection();

	pxOverlay->ChangeStateSectionVersion();
}

void Asura_Overlay_Botanical_State_Tables::AddStateSectionVersionButton( int ipxData)
{
	Asura_Overlay_Botanical_State_Tables* pxOverlay = GetIntAsClass( ipxData);

	pxOverlay->CreateStateSectionVersion();
	
	pxOverlay->UpdateStateSectionVersionList();	
	pxOverlay->ChangeStateSectionVersion();
	pxOverlay->m_bDataChanged = true;
}

void Asura_Overlay_Botanical_State_Tables::DeleteStateSectionVersionButton( int ipxData)
{
	Asura_Overlay_Botanical_State_Tables* pxOverlay = GetIntAsClass( ipxData);

	pxOverlay->DeleteStateSectionVersion();
	
	pxOverlay->UpdateStateSectionVersionList();	
	pxOverlay->ChangeStateSectionVersion();
	pxOverlay->m_bDataChanged = true;
}


void Asura_Overlay_Botanical_State_Tables::SetStateSectionVersionCollMesh( int ipxData)
{
	Asura_Overlay_Botanical_State_Tables* pxOverlay = GetIntAsClass( ipxData);

	Asura_String xSkinName, xSkinFileName;
	Tools_Animation_Files::ChooseSkin(&xSkinName, &xSkinFileName);

	pxOverlay->m_xVersionCollisionMeshHash = Asura_GetHashID(xSkinName.GetString());
	pxOverlay->m_xVersionCollisionMeshName = xSkinFileName;

	pxOverlay->m_bDataChanged = true;
}


void Asura_Overlay_Botanical_State_Tables::ChangeStateSectionVersion()
{
	this->SetStateSectionVersionData();
}

void Asura_Overlay_Botanical_State_Tables::SetStateSectionVersionData()
{
	Tools_Botanical_StateTable1D_Section_VersionRange* pxStateTableSectionVersion = this->m_pxStateTableSectionVersion;
	if( NULL == pxStateTableSectionVersion)
	{
		this->m_fVersionStart = 0.0f;
		this->m_fVersionStop  = 1.0f;
		this->m_uProjectFlags = 0;

		this->m_fVersionCollisionMeshDensity   = 1.0f;
		this->m_xVersionCollisionMeshHash = ASURA_HASH_ID_UNSET;
		this->m_xVersionCollisionMeshName = "";
		this->m_bVersionCollisionUseBoundingBox = false;
	}
	else
	{
		this->m_fVersionStart = pxStateTableSectionVersion->m_fRangeStart;
		this->m_fVersionStop  = pxStateTableSectionVersion->m_fRangeEnd;
		this->m_uProjectFlags = pxStateTableSectionVersion->m_uProjectFlags;

		this->m_fVersionCollisionMeshDensity    = pxStateTableSectionVersion->m_fCollisionMeshDensity;
		this->m_xVersionCollisionMeshHash		= pxStateTableSectionVersion->m_xCollisionMeshHash;
		this->m_xVersionCollisionMeshName		= pxStateTableSectionVersion->m_xCollisionMeshName;
		this->m_bVersionCollisionUseBoundingBox = pxStateTableSectionVersion->m_bCollisionUseBoundingBox;
	}
	
	Tools_BotanicalSection* pxToolShapeSection = this->m_pxToolShapeSection;
	if( NULL != pxToolShapeSection)
	{
		//this->m_pxStateSectionVersion_Add->SetEnabled( this->m_iStateSectionVersionListNameCount < pxToolShapeSection->m_xHashTree_xVersions.GetNumberOfNodes());
		this->m_pxStateSectionVersion_Add->SetEnabled( true);
		this->m_pxStateSectionVersion_Delete->SetEnabled( 0 < this->m_iStateSectionVersionListNameCount);
		this->m_pxStateSectionVersion_SetCollisionMesh->SetEnabled(m_iStateSectionVersionListNameCount>0);
		this->m_pxStateSectionVersion_CollisionMeshDensity->SetEnabled(m_iStateSectionVersionListNameCount>0);
		this->m_pxStateSectionVersion_CollisionUseBoundingBox->SetEnabled(m_iStateSectionVersionListNameCount>0);
	}
	else
	{		
		this->m_pxStateSectionVersion_Add->SetEnabled(false);
		this->m_pxStateSectionVersion_Delete->SetEnabled(false);
		this->m_pxStateSectionVersion_SetCollisionMesh->SetEnabled(false);
		this->m_pxStateSectionVersion_CollisionMeshDensity->SetEnabled(false);
		this->m_pxStateSectionVersion_CollisionUseBoundingBox->SetEnabled(false);
	}
	
	this->SetSectionVersionSelection();
}

//*******************************************************************
// State Sections Version Data
//*******************************************************************


//*******************************************************************
// Section Version 
//*******************************************************************
void Asura_Overlay_Botanical_State_Tables::DeleteSectionVersionDropDownNameArray()
{
	const int iSectionVersionNameCount = m_xArray_pszSectionVersionDropDownNames.GetSize();
	for( int iSectionVersionNameIndex = 0; iSectionVersionNameIndex < iSectionVersionNameCount; iSectionVersionNameIndex += 1)
	{
		char*& pszSectionVersionName = this->m_xArray_pszSectionVersionDropDownNames[ iSectionVersionNameIndex];
		
		delete[] pszSectionVersionName;
		pszSectionVersionName = NULL;
	}
	
	this->m_xArray_pszSectionVersionDropDownNames.Reset();
	this->m_iSectionVersionDropDownNameCount = 0;
}

void Asura_Overlay_Botanical_State_Tables::CreateSectionVersionDropDownNameArray()
{
	this->DeleteSectionVersionDropDownNameArray();
	
	Tools_BotanicalSection* pxToolShapeSection = this->m_pxToolShapeSection;
	if( NULL != pxToolShapeSection)
	{
		const int iSectionVersionDropDownNameCount = pxToolShapeSection->m_xHashTree_xVersions.GetNumberOfNodes();
		this->m_xArray_pszSectionVersionDropDownNames.SetSize( iSectionVersionDropDownNameCount);
		
		for( int iSectionVersionDropDownNameIndex = 0; iSectionVersionDropDownNameIndex < iSectionVersionDropDownNameCount; iSectionVersionDropDownNameIndex += 1)
		{
			Tools_BotanicalVersion* pxToolsSectionVersion = pxToolShapeSection->m_xHashTree_xVersions.GetDataByIndex( iSectionVersionDropDownNameIndex);
			char*& pszSectionVersionDropDownName = this->m_xArray_pszSectionVersionDropDownNames[ iSectionVersionDropDownNameIndex];
			
			const Asura_String& xToolSectionVersionName = pxToolsSectionVersion->m_xName;
			
			if( 0 != xToolSectionVersionName.Compare( ""))
			{
				pszSectionVersionDropDownName = Asura_StringUtil::Duplicate( xToolSectionVersionName.GetString());
				continue;
			}
			
			pszSectionVersionDropDownName = Asura_StringUtil::Duplicate( "DEFAULT VERSION");
		}
		
		this->m_iSectionVersionDropDownNameCount = iSectionVersionDropDownNameCount;
	}
	
	if( 0 == this->m_xArray_pszSectionVersionDropDownNames.GetSize())
	{
		this->m_xArray_pszSectionVersionDropDownNames.SetSize( 1);
		this->m_xArray_pszSectionVersionDropDownNames[0] = Asura_StringUtil::Duplicate( "######");
		// DLH - 08 December 2006 15:40:23 - The drop down box cannot copy with 0 entries
		this->m_iSectionVersionDropDownNameCount = 1;
	}
}

void Asura_Overlay_Botanical_State_Tables::UpdateSectionVersionList()
{
	this->CreateSectionVersionDropDownNameArray();
	
	char** ppszList = &(this->m_xArray_pszSectionVersionDropDownNames[ 0]);
	this->m_pxSectionVersion_DropDown->ReplaceStrings( this->m_iSectionVersionDropDownNameCount, ppszList);
}

Asura_Dialog* Asura_Overlay_Botanical_State_Tables::CreateSectionVersionDropDown( Asura_Vector_2& xOffset)
{
	this->m_iStateDropDownNameSelected = 0;
	
	this->CreateSectionVersionDropDownNameArray();
	
	char** ppszList = &(this->m_xArray_pszSectionVersionDropDownNames[ 0]);
	this->m_pxSectionVersion_DropDown = new Asura_DropDown_List_Dialog( xOffset, ppszList, &(this->m_iSectionVersionDropDownNameSelected), STATE_TABLE_DIALOG_LIST_SIZE, this->m_iSectionVersionDropDownNameCount, NULL, &(this->m_bDataChanged), NULL);
	this->m_pxSectionVersion_DropDown->SetOnSelectCallback( this->SelectionChangedSectionVersionDropDown, this->GetThisAsInt());
	
	return( this->m_pxSectionVersion_DropDown);
}

void Asura_Overlay_Botanical_State_Tables::SetSectionVersionSelection()
{
	this->m_iSectionVersionDropDownNameSelected = 0;
	
	Tools_BotanicalSection* pxToolShapeSection = this->m_pxToolShapeSection;
	Tools_Botanical_StateTable1D_Section_VersionRange* pxStateTableSectionVersion = this->m_pxStateTableSectionVersion;
	if( NULL == pxStateTableSectionVersion || NULL == this->m_pxToolShapeSection)
	{
		this->m_pxSectionVersion_DropDown->SetEnabled( false);
		this->m_pxStateSectionVersion_Start->SetEnabled( false);
		this->m_pxStateSectionVersion_Stop->SetEnabled( false);
		this->m_pxStateSectionVersion_SetCollisionMesh->SetEnabled( false);
		this->m_pxStateSectionVersion_CollisionMeshDensity->SetEnabled( false);
		this->m_pxStateSectionVersion_CollisionUseBoundingBox->SetEnabled(false);

		return;
	}
	
	this->m_pxSectionVersion_DropDown->SetEnabled( true);
	this->m_pxStateSectionVersion_Start->SetEnabled( true);
	this->m_pxStateSectionVersion_Stop->SetEnabled( true);
	this->m_pxStateSectionVersion_SetCollisionMesh->SetEnabled( true);
	this->m_pxStateSectionVersion_CollisionMeshDensity->SetEnabled( true);
	this->m_pxStateSectionVersion_CollisionUseBoundingBox->SetEnabled(true);
	
	const Asura_String& xTaregtVersionName = pxStateTableSectionVersion->m_xTargetVersion;
	
	const int iSectionVersionDropDownNameCount = pxToolShapeSection->m_xHashTree_xVersions.GetNumberOfNodes();
	
	for( int iSectionVersionDropDownNameIndex = 0; iSectionVersionDropDownNameIndex < iSectionVersionDropDownNameCount; iSectionVersionDropDownNameIndex += 1)
	{
		Tools_BotanicalVersion* pxToolsSectionVersion = pxToolShapeSection->m_xHashTree_xVersions.GetDataByIndex( iSectionVersionDropDownNameIndex);
		const Asura_String& xToolSectionVersionName = pxToolsSectionVersion->m_xName;
		
		if( 0 != xToolSectionVersionName.Compare( xTaregtVersionName))
		{
			continue;
		}

		this->m_iSectionVersionDropDownNameSelected = iSectionVersionDropDownNameIndex;
		break;
	}
	
}

void Asura_Overlay_Botanical_State_Tables::SelectionChangedSectionVersionDropDown( int ipxData)
{
	Asura_Overlay_Botanical_State_Tables* pxOverlay = GetIntAsClass( ipxData);

	Tools_BotanicalSection* pxToolShapeSection = pxOverlay->m_pxToolShapeSection;
	Tools_Botanical_StateTable1D_Section_VersionRange* pxStateTableSectionVersion = pxOverlay->m_pxStateTableSectionVersion;
	if( NULL == pxStateTableSectionVersion || NULL == pxOverlay->m_pxToolShapeSection)
	{
		return;
	}
	
	const int iSectionVersionDropDownNameCount = pxToolShapeSection->m_xHashTree_xVersions.GetNumberOfNodes();
	
	for( int iSectionVersionDropDownNameIndex = 0; iSectionVersionDropDownNameIndex < iSectionVersionDropDownNameCount; iSectionVersionDropDownNameIndex += 1)
	{
		if( pxOverlay->m_iSectionVersionDropDownNameSelected != iSectionVersionDropDownNameIndex)
		{
			continue;
		}
		
		Tools_BotanicalVersion* pxToolsSectionVersion = pxToolShapeSection->m_xHashTree_xVersions.GetDataByIndex( iSectionVersionDropDownNameIndex);
		const Asura_String& xToolSectionVersionName = pxToolsSectionVersion->m_xName;
		
		pxStateTableSectionVersion->m_xTargetVersion = xToolSectionVersionName;
	
		pxOverlay->UpdateStateSectionVersionList();
		break;
	}
}

void Asura_Overlay_Botanical_State_Tables::ChangeSectionVersion()
{
	this->SetSectionVersionSelection();
	this->UpdateSectionVersionList();
}

//*******************************************************************
// Sections
//*******************************************************************

void Asura_Overlay_Botanical_State_Tables::CreateSectionDropDownNameArray()
{
#if 0
	this->DeleteSectionDropDownNameArray();

	const int iSectionDropDownNameCount = this->m_pxToolShape->m_xHashTree_xSections.GetNumberOfNodes();
	this->m_xArray_pszSectionDropDownNames.SetSize( iSectionDropDownNameCount);
	
	for( int iSectionDropDownNameIndex = 0; iSectionDropDownNameIndex < iSectionDropDownNameCount; iSectionDropDownNameIndex += 1)
	{
		const Tools_BotanicalSection* pxToolSection = this->m_pxToolShape->m_xHashTree_xSections.GetDataByIndex( iSectionDropDownNameIndex);
		char*& pszSectionDropDownName = this->m_xArray_pszSectionDropDownNames[ iSectionDropDownNameIndex];
		
		const Asura_String& xToolSectionName = pxToolSection->m_xName;
		
		pszSectionDropDownName = Asura_StringUtil::Duplicate( xToolSectionName.GetString());
	}
	
	this->m_iSectionDropDownNameCount = iSectionDropDownNameCount;
#endif
}

void Asura_Overlay_Botanical_State_Tables::DeleteSectionDropDownNameArray()
{
#if 0
	const int iSectionDropDownNameCount = this->m_xArray_pszSectionDropDownNames.GetSize();
	for( int iSectionDropDownNameIndex = 0; iSectionDropDownNameIndex < iSectionDropDownNameCount; iSectionDropDownNameIndex += 1)
	{
		char*& pszSectionDropDownName = this->m_xArray_pszSectionDropDownNames[ iSectionDropDownNameIndex];
		
		delete[] pszSectionDropDownName;
		pszSectionDropDownName = NULL;
	}
	
	this->m_xArray_pszSectionDropDownNames.Reset();
	this->m_iSectionDropDownNameCount = 0;
	this->m_pxToolShapeSection = NULL;
#endif
}

Asura_Dialog* Asura_Overlay_Botanical_State_Tables::CreateSectionDropDown( Asura_Vector_2& xOffset)
{
#if 0
	this->m_iSectionDropDownNameSelected =  0;
	
	this->CreateSectionDropDownNameArray();
	
	char** ppszList = &(this->m_xArray_pszSectionDropDownNames[ 0]);
	this->m_pxSection_DropDown = new Asura_DropDown_List_Dialog( xOffset, ppszList, &(this->m_iSectionDropDownNameSelected), STATE_TABLE_DIALOG_LIST_SIZE, this->m_iSectionDropDownNameCount, NULL, &(this->m_bDataChanged), "Sections");
	this->m_pxSection_DropDown->SetOnSelectCallback( this->SelectionChangedSectionDropDown, this->GetThisAsInt());
	
	this->SetupSectionPointerFromSelection();
	
	return( this->m_pxSection_DropDown);
#endif
	return( NULL);
}

void Asura_Overlay_Botanical_State_Tables::SetupSectionPointerFromSelection()
{
#if 0
	this->m_pxToolShapeSection = this->m_pxToolShape->m_xHashTree_xSections.GetDataByIndex( this->m_iSectionDropDownNameSelected);
#endif
}

void Asura_Overlay_Botanical_State_Tables::SelectionChangedSectionDropDown( int ipxData)
{
#if 0
	Asura_Overlay_Botanical_State_Tables* pxOverlay = GetIntAsClass( ipxData);

	pxOverlay->SetupSectionPointerFromSelection();
	
	pxOverlay->ChangeSection();
#endif
}

void Asura_Overlay_Botanical_State_Tables::ChangeSection()
{
	
}

//*******************************************************************
//
// (c) 2006 Rebellion
//
// Warning: This computer program is protected by copyright law and 
// international treaties. Unauthorized reproduction or distribution
// of this program, or any portion of it, may result in severe civil and
// criminal penalties, and will be prosecuted to the maximum extent 
// possible under the law.
//
//*******************************************************************

#endif //defined( ASURA_BOTANICAL_SUPPORT_STATES) && 1 == ASURA_BOTANICAL_SUPPORT_STATES
#endif //defined( ASURA_USE_BOTANICALS)
