//*******************************************************************
// File: Tools_Botanicals_Game_Render.cpp
//
// Created: 15 December 2006 DLH
//
// Description: 
//
//*******************************************************************

// system libraries
#include "Core/Asura.h"

#if defined( ASURA_TOOLS) && defined( ASURA_USE_BOTANICALS) 

// main header
#include "Botanicals/Asura_Botanical_Game_Render_Type.h"
#include "Botanicals/Asura_Botanical_Game_Render_Type_BasicInteractive.h"
#include "Botanicals/Tools_Botanicals_Object.h"

#include "ChunkSystem/Asura_Chunk_Stream.h"


// other libraries

// internal dependencies

//*******************************************************************
// Constants
//*******************************************************************


//*******************************************************************
// Local Classes
//*******************************************************************


//*******************************************************************
// Local Datas
//*******************************************************************


//*******************************************************************
// Function Definitions
//*******************************************************************

#if defined( ASURA_TOOLS) && defined( ASURA_BOTANICAL_SUPPORT_INTERACTIVE) && 1 == ASURA_BOTANICAL_SUPPORT_INTERACTIVE
Tools_BGR_Type_BoneWeightCalc::Tools_BGR_Type_BoneWeightCalc()
{
	this->m_xBoneWeightPlane.ZeroW0();
	this->m_fBoneWeightGamma = 1.0f;
}
#endif

#if defined( ASURA_TOOLS) && defined( ASURA_BOTANICAL_SUPPORT_INTERACTIVE) && 1 == ASURA_BOTANICAL_SUPPORT_INTERACTIVE
void Tools_BGR_Type_BoneWeightCalc::SetupBoneWeight( const Asura_BGR_Type_BasicInteractive& xTypeBasicInteractive)
{
	Asura_AVector3 xBonesDirection = xTypeBasicInteractive.m_xBonePosition1 - xTypeBasicInteractive.m_xBonePosition0;

	float fBoneDirectionLength = xBonesDirection.NormaliseReturningMagnitude();
	xBonesDirection /= fBoneDirectionLength;

	float fBonePlaneDistance = xBonesDirection * xTypeBasicInteractive.m_xBonePosition0;

	this->m_xBoneWeightPlane = Asura_AVector4( xBonesDirection, -fBonePlaneDistance);
	this->m_fBoneWeightGamma = xTypeBasicInteractive.m_fBoneWeightGamma;
}
#endif

#if defined( ASURA_TOOLS) && defined( ASURA_BOTANICAL_SUPPORT_INTERACTIVE) && 1 == ASURA_BOTANICAL_SUPPORT_INTERACTIVE
float Tools_BGR_Type_BoneWeightCalc::CalculateBoneWeight( const Asura_AVector3& xPosition) const 
{
	Asura_AVector4 xVertex4( xPosition, 1.0f);
	float fBoneWeight = this->m_xBoneWeightPlane.DotProduct4( xVertex4);
	fBoneWeight = Asura_Maths::Max( 0.0f, fBoneWeight);
	fBoneWeight = Asura_Maths::Min( 1.0f, fBoneWeight);
	fBoneWeight = powf( fBoneWeight, this->m_fBoneWeightGamma);

	return( fBoneWeight);
}
#endif

#if defined( ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS) && 1 == ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS
u_int Tools_BGR_Type::GetSectionCount() const
{
	return( this->m_xHashTree_xSectionID.GetNumberOfNodes());
}
#endif
#if defined( ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS) && 1 == ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS
Asura_BGR_Section_ID Tools_BGR_Type::GetSectionID( const Asura_Hash_ID& xHashID) const
{
	const Asura_BGR_Section_ID* pxSectionID = this->m_xHashTree_xSectionID.GetDataByHash( xHashID);
	if( NULL == pxSectionID)
	{
		return( ASURA_BGR_SECTION_ID_INVALID);
	}
	return( *pxSectionID);
}
#endif
#if defined( ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS) && 1 == ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS
Asura_BGR_Version_ID Tools_BGR_Type::GetVersionID( const Asura_Hash_ID& xHashID) const
{
	const Asura_BGR_Version_ID* pxVersionID = this->m_xHashTree_xVersionID.GetDataByHash( xHashID);
	if( NULL == pxVersionID)
	{
		return( ASURA_BGR_VERSION_ID_OFF);
	}
	return( *pxVersionID);
}
#endif

#if defined( ASURA_TOOLS) && defined( ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA) && 1 == ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA
static inline u_int Asura_BGR_Instances_Header_AlignValue( const u_int uOffsetOriginal, const u_int uAlignTo)
{
	u_int uOffset = uOffsetOriginal;

	u_int uAligned = uOffset % uAlignTo;
	if( 0 != uAligned)
	{
		uOffset += uAlignTo - uAligned;
	}
	Asura_Assert( 0 == (uOffset % uAlignTo), "Alignment is out");

	return( uOffset);
}
#endif

#if defined( ASURA_TOOLS) && defined( ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA) && 1 == ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA
template< class Tdata>
static inline void Asura_BGR_Instances_Header_AddDynamicData( u_int& uInstanceSize, u_int& uInstanceOffset, const u_int uDataCount, u_int& uAlignMax)
{
	uInstanceSize = Asura_BGR_Instances_Header_AlignValue( uInstanceSize, __alignof( Tdata));

	uInstanceOffset = uInstanceSize;

	uInstanceSize += uDataCount * sizeof( Tdata);

	uAlignMax = Asura_Maths::Max( uAlignMax, __alignof( Tdata));
}
#endif

#if defined( ASURA_TOOLS) && defined( ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA) && 1 == ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA
bool Asura_BGR_DynamicMemoryLayout::SetupDynamicData( const Asura_BGR_Type& xType)
{
	this->Clear();

	this->m_uDynamicSizeInstance = 0;

	u_int uAlignMax = 1;

#if defined( ASURA_BOTANICAL_SUPPORT_INTERACTIVE)
	if( NULL != xType.GetBasicInteractive())
	{
		// DLH - 31 October 2006 17:00:42 - Setup Interactive Data
		Asura_BGR_Instances_Header_AddDynamicData< Asura_BGR_InteractiveBasic>( this->m_uDynamicSizeInstance, this->m_uDynamicOffsetInteractiveData, 1, uAlignMax);
	}
#endif

#if defined( ASURA_BOTANICAL_INSTANCE_USER_DATA_TYPE)	
	// DLH - 31 October 2006 17:00:28 - Setup User Data
	{
		u_int uUserDataSize = 1;
		Asura_BGR_Instances_Header_AddDynamicData< ASURA_BOTANICAL_INSTANCE_USER_DATA_TYPE>( this->m_uDynamicSizeInstance, this->m_uDynamicOffsetUserData, uUserDataSize, uAlignMax);
	}
#endif

#if defined( ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS) && 1 == ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS
	// DLH - 31 October 2006 17:00:00 - Setup Section Version Data
	{
		u_int uSectionCount = xType.GetSectionCount();
		Asura_BGR_Instances_Header_AddDynamicData< Asura_BGR_Version_ID>( this->m_uDynamicSizeInstance, this->m_uDynamicOffsetSectionData, uSectionCount, uAlignMax);
	}
#endif

#if defined( ASURA_BOTANICAL_SUPPORT_STATES) && 1 == ASURA_BOTANICAL_SUPPORT_STATES
	// DLH - 31 October 2006 17:00:17 - Setup State Data
	{
		u_int uStateCount = xType.GetStateCount();
		Asura_BGR_Instances_Header_AddDynamicData< Asura_BGR_State_Value>( this->m_uDynamicSizeInstance, this->m_uDynamicOffsetStateData, uStateCount, uAlignMax);
	}
#endif

	this->m_uDynamicSizeInstance = Asura_BGR_Instances_Header_AlignValue( this->m_uDynamicSizeInstance, uAlignMax);
	Asura_Assertf( 0 == (this->m_uDynamicSizeInstance % uAlignMax), ("The alignment of this Dynamic data needs to be at least %d bytes. The strtucture size calculated is %d which will not create an array that is aligned to %d", uAlignMax, this->m_uDynamicSizeInstance, uAlignMax));

	return( true);
}
#endif

#if defined( ASURA_BOTANICAL_SUPPORT_INTERACTIVE)
void Asura_BGR_Type_BasicInteractive::Setup( const Tools_BotanicalShape& xToolShape)
{
	this->m_xBonePosition0 = xToolShape.m_xInteractiveBone0;
	this->m_xBonePosition1 = xToolShape.m_xInteractiveBone1;
	this->m_fMass = xToolShape.m_fMass;
	this->m_fDrag = xToolShape.m_fDrag;
	this->m_fSpringStrength = xToolShape.m_fSpringStrength;
	this->m_fSpringLength = xToolShape.m_fSpringLength;
	this->m_fBoneLength = this->m_xBonePosition1.Magnitude();
#if defined( ASURA_TOOLS)
	this->m_fBoneWeightGamma = xToolShape.m_fBoneWeightGamma;
	this->m_bUsePaintedColoursAsWeights = xToolShape.m_bUsePaintedColoursAsWeights;
#endif
}
#endif


#if defined( ASURA_BOTANICAL_SUPPORT_STATES) && 1 == ASURA_BOTANICAL_SUPPORT_STATES
bool Asura_BGR_Type::CreateStateTables( const Tools_BotanicalShape& xBotanicalToolShape)
{
	this->m_xArray_xStateTables.Clear();
	this->m_xArray_xStateTables.SetSize( this->m_xHashTree_xStateID.GetNumberOfNodes());

	for( const Asura_ListElt< Tools_Botanical_StateTable1D>* pxItStateTable = xBotanicalToolShape.m_xList_xStateTables1D.Front(); NULL != pxItStateTable; pxItStateTable = pxItStateTable->Next())
	{
		const Tools_Botanical_StateTable1D& xToolStateTable = pxItStateTable->Data();
		const Asura_String& xStateName = xToolStateTable.m_xSourceState;
		const Asura_Hash_ID xStateHash = xStateName.GetHash();

		Asura_BGR_State_ID* pxStateID = this->m_xHashTree_xStateID.GetDataByHash( xStateHash);
		if( NULL == pxStateID)
		{
			continue;
		}

		Asura_BGR_State_ID xStateID = *pxStateID;
		Asura_BGR_StateTable& xBGRStateTable = this->m_xArray_xStateTables[ xStateID];

		Asura_Assertf( 0 == xBGRStateTable.m_xArray_xSections.Size(), ("State table for state '%s' already seems to be setup", xStateName.GetString()));
		const u_int uSectionCount = xToolStateTable.m_xList_xSection.Size();
		xBGRStateTable.m_xArray_xSections.SetSize( uSectionCount);

		u_int uSectionIndex = 0;
		for( const Asura_ListElt< Tools_Botanical_StateTable1D_Section>* pxItSectionTable = xToolStateTable.m_xList_xSection.Front(); NULL != pxItSectionTable; pxItSectionTable = pxItSectionTable->Next())
		{
			const Tools_Botanical_StateTable1D_Section& xToolSectionTable = pxItSectionTable->Data();
			const Asura_String& xSectionName = xToolSectionTable.m_xTargetSection;
			const Asura_Hash_ID xSectionHash = xSectionName.GetHash();

			const Asura_BGR_Section_ID xSectionID = this->GetSectionID( xSectionHash);
			if( ASURA_BGR_SECTION_ID_INVALID == xSectionID)
			{
				continue;
			}

			Asura_BGR_StateTable_Section& xBGRSectionTable = xBGRStateTable.m_xArray_xSections[ uSectionIndex];
			uSectionIndex += 1;

			xBGRSectionTable.m_xSectionID = xSectionID;

			const u_int uVersionCount = xToolSectionTable.m_xList_xVersionRanges.Size();
			xBGRSectionTable.m_xArray_xVersions.SetSize( uVersionCount);

			u_int uVersionIndex = 0;
			for( const Asura_ListElt< Tools_Botanical_StateTable1D_Section_VersionRange>* pxItVersionTable = xToolSectionTable.m_xList_xVersionRanges.Front(); NULL != pxItVersionTable; pxItVersionTable = pxItVersionTable->Next())
			{
				const Tools_Botanical_StateTable1D_Section_VersionRange& xToolVersionTable = pxItVersionTable->Data();
				const Asura_String& xVersionName = xToolVersionTable.m_xTargetVersion;
				const Asura_Hash_ID xVersionHash = xVersionName.GetHash();

				Asura_BGR_Version_ID xVersionID = this->GetVersionID( xVersionHash);
				if( ASURA_BGR_VERSION_ID_OFF == xVersionID)
				{
					continue;
				}

				Asura_BGR_StateTable_Version& xBGRVersionTable = xBGRSectionTable.m_xArray_xVersions[ uVersionIndex];
				uVersionIndex += 1;

				xBGRVersionTable.m_xVersionID    = xVersionID;
				xBGRVersionTable.m_xRangeStart   = static_cast< Asura_BGR_State_Value>( xToolVersionTable.m_fRangeStart);
				xBGRVersionTable.m_xRangeStop    = static_cast< Asura_BGR_State_Value>( xToolVersionTable.m_fRangeEnd);
				xBGRVersionTable.m_uProjectFlags = xToolVersionTable.m_uProjectFlags;

				xBGRVersionTable.m_xCollisionMeshHash		= xToolVersionTable.m_xCollisionMeshHash;
				xBGRVersionTable.m_xCollisionMeshName		= xToolVersionTable.m_xCollisionMeshName;
				xBGRVersionTable.m_fCollisionMeshDensity	= xToolVersionTable.m_fCollisionMeshDensity;
				xBGRVersionTable.m_bCollisionUseBoundingBox = xToolVersionTable.m_bCollisionUseBoundingBox;
			}
		}
	}

	return( true);
}
#endif

#if defined( ASURA_BOTANICAL_SUPPORT_STATES) && 1 == ASURA_BOTANICAL_SUPPORT_STATES
bool Asura_BGR_Type::CreateStates( const Tools_BotanicalShape& xBotanicalToolShape)
{
	this->m_xHashTree_xStateID.Reset();

	for( const Asura_ListElt< Tools_Botanical_StateTable1D>* pxItStateTable = xBotanicalToolShape.m_xList_xStateTables1D.Front(); NULL != pxItStateTable; pxItStateTable = pxItStateTable->Next())
	{
		const Tools_Botanical_StateTable1D& xToolStateTable = pxItStateTable->Data();

		const Asura_String& xStateName = xToolStateTable.m_xSourceState;
		const Asura_Hash_ID xStateHash = xStateName.GetHash();

		Asura_BGR_State_ID* pxStateID = this->m_xHashTree_xStateID.GetDataByHash( xStateHash);
		if( NULL == pxStateID)
		{
			this->m_xHashTree_xStateID.AddNode( xStateHash, this->m_xHashTree_xStateID.GetNumberOfNodes());
			pxStateID = this->m_xHashTree_xStateID.GetDataByHash( xStateHash);
			if( NULL == pxStateID)
			{
				return( false);
			}
		}
	}

	return( true);
}
#endif

#if defined( ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS) && 1 == ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS
bool Tools_BGR_Type::CreateSectionAndVersionData( const Tools_BotanicalShape& xBotanicalToolShape)
{
	this->m_xHashTree_xSectionID.Reset();
	this->m_xHashTree_xVersionID.Reset();

	Asura_String xStringDefault( "");
	Asura_Hash_ID xDefaultHashID = xStringDefault.GetHash();
	this->m_xHashTree_xSectionID.AddNode( xDefaultHashID, 0);
	this->m_xHashTree_xVersionID.AddNode( xDefaultHashID, 0);
	
	const u_int uSectionCount = xBotanicalToolShape.m_xHashTree_xSections.GetNumberOfNodes();
	for( u_int uSectionIndex = 0; uSectionIndex < uSectionCount; uSectionIndex += 1)
	{
		Tools_BotanicalSection* pxSection = xBotanicalToolShape.m_xHashTree_xSections.GetDataByIndex( uSectionIndex);
		if( NULL == pxSection)
		{
			continue;
		}
		
		Asura_Hash_ID xSectionHashID = pxSection->m_xName.GetHash();
		if( NULL == this->m_xHashTree_xSectionID.GetDataByHash( xSectionHashID))
		{
			this->m_xHashTree_xSectionID.AddNode( xSectionHashID, static_cast< Asura_BGR_Section_ID>( this->m_xHashTree_xSectionID.GetNumberOfNodes()) );
		}
		
		const u_int uVersionCount = pxSection->m_xHashTree_xVersions.GetNumberOfNodes();
		for( u_int uVersionIndex = 0; uVersionIndex < uVersionCount; uVersionIndex += 1)
		{
			Tools_BotanicalVersion* pxVersion = pxSection->m_xHashTree_xVersions.GetDataByIndex( uVersionIndex);
			if( NULL == pxVersion)
			{
				continue;
			}
			
			Asura_Hash_ID xVersionHashID = pxVersion->m_xName.GetHash();
			if( NULL == this->m_xHashTree_xVersionID.GetDataByHash( xVersionHashID))
			{
				this->m_xHashTree_xVersionID.AddNode( xVersionHashID, static_cast< Asura_BGR_Version_ID>( this->m_xHashTree_xVersionID.GetNumberOfNodes()) );
			}
		}
	}

	return( true);
}
#endif

Tools_BGR_Type::Tools_BGR_Type( const Tools_BotanicalShape& xCurrShape)
{
	this->m_xTypeID = Asura_BGR_TypeID( Asura_BGR_TypeID::INVALID_ID);
	this->m_fBoundingRadius = -1.0f;
	this->m_xBoundingBox.Reset();
	
#if defined( ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS) && 1 == ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS
	this->CreateSectionAndVersionData( xCurrShape);
#endif

#if defined( ASURA_BOTANICAL_SUPPORT_INTERACTIVE)
	Asura_Assert( NULL == this->GetBasicInteractive(), "");
	if( true == xCurrShape.m_bIsInteractive)
	{
		Asura_BGR_Type_BasicInteractive* pxBasicInteractive = this->CreateBasicInteractive();

		pxBasicInteractive->Setup( xCurrShape);
	}
#endif

#if defined( ASURA_BOTANICAL_SUPPORT_STATES) && 1 == ASURA_BOTANICAL_SUPPORT_STATES
	this->CreateStates( xCurrShape);
	this->CreateStateTables( xCurrShape);
#endif
}

//*******************************************************************
//
// (c) 2006 Rebellion
//
// Warning: This computer program is protected by copyright law and 
// international treaties. Unauthorized reproduction or distribution
// of this program, or any portion of it, may result in severe civil and
// criminal penalties, and will be prosecuted to the maximum extent 
// possible under the law.
//
//*******************************************************************

#endif //defined( ASURA_USE_BOTANICALS) 
