//
// Tools_Botanicals_Object.h
//
// Created by Dave Reed - 04/11/05
// 
// Copyright (C) 2005 Rebellion (Derby), All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifdef ASURA_USE_BOTANICALS

#ifndef _TOOLS_BOTANICALS_OBJECT_
#define _TOOLS_BOTANICALS_OBJECT_

#include "Botanicals/Asura_Botanicals_TemplateManager.h"
#include "Botanicals/Asura_Botanicals_ShapeManager.h"
#include "Objects/Tools_Object.h"
#include "Shapes/Asura_Shapes.h"

#if defined( ASURA_BOTANICALS_BGR_IN_EDIT)
	class Asura_Direct3D_BGR_Instances_Header;
	class Asura_Direct3D_BGR_Manager;
#endif

class Tools_Botanicals_Object : public Tools_Object
{
	friend class Asura_Botanicals_InstanceManager;

public :
	Tools_Botanicals_Object(const Asura_Vector_3& xPosition, int iTemplateIdx, u_int uInstanceID = 0);
	~Tools_Botanicals_Object();

	void Destroy();

	const char*				   GetTypeDescription(){return "Botanical";}
	const char*				   GetName();

	void					   CreateOverlay();
	void					   OnSelection();
	void					   OnDeselection();
	bool					   IsSelectable();
	void					   UpdateWhileSelected();
	void					   Update();
	void					   Render();	

	void                       UpdatePositionInAsuraData(bool bUpdateLinkedObjects);


	void					   SetTemplateIdx(int iTemplateIdx);	
	Asura_Botanicals_Template* GetTemplate();	
	Asura_Botanicals_Template* GetTemplate() const;	
	int						   GetTemplateIdx() const  { return m_iTemplateIdx; };
	
	float					   GetScale()	const	  {return m_fInstanceScale;};	
	void					   SetScale(float fScale) {m_fInstanceScale=fScale;};	
	float*					   GetScalePtr()		  {return &m_fInstanceScale;};
	bool*                      GetScaleModifiedPtr()  {return &m_bScaleModified;}

	float					   GetBrightness() const			{return m_fInstanceBrightness;};	
	void					   SetBrightness(float fBrightness) {m_fInstanceBrightness=fBrightness;};	
	float*					   GetBrightnessPtr()				{return &m_fInstanceBrightness;};
	bool*                      GetBrightnessModifiedPtr()		{return &m_bBrightnessModified;}


	u_int                      GetBotanicalFlags() const {return m_uBotanicalFlags;}
	void                       SetBotanicalFlags(u_int uFlags) {m_uBotanicalFlags = uFlags;}

	u_int                      GetInstanceID() const {return m_uInstanceID;}

	void                       UnmergePolygonsFromEnvironment(bool bForceUnmerge = false);

	//merges all botanicals into environment as needed
	static void                MergeObjectsIntoEnvironmentIfNeeded(bool bForceUpdateAll=false);
								
	//Permanently merges the selected  botanicals into the environment
	static void                PermanentlyMergeSelectedObjects();

	//Permanently merges botanicals using templates that have been flagged to do so
	static void                PermanentlyMergeFlaggedObjects();


	//does a full check to make sure the asura file and botanicals file aren't out of sync
	static void                CheckAllMerging();

#if defined( ASURA_BOTANICALS_BGR_IN_EDIT)
	void UpdateBGRInstance();
	void RegenerateBGRInstance();//destroys and re-creates the BGR instance
#endif

	virtual void			   Copy(const Tools_Object* pxObject);
	virtual bool			   CanBeCopied() const {return true;}
	virtual Tools_Object*	   CreateObjectOfThisType() const;


private:
	//If bPermanentMerge then the object will be deleted after doing the merge
	void                       MergePolygonsIntoEnvironmentIfNeeded(bool bPermanentMerge);
	u_int                      GetChecksumForMerge() const;

	//Gets list of meshes to merge in from a botanical shape
	void                       ListMeshesToMerge(const Asura_Botanicals_Template* pxTemplate,Tools_BotanicalShape* pxToolsShape,List<Asura_Mesh*>& xOutputList,bool bCollMeshes) const;

	

	//Gets a fake guid for the merging code... main thing is that it shouldn't be in the tools guid range. RAR 17/03/2006
	Asura_Guid                 GetMergeGuid() const {return m_uInstanceID + ASURA_GUID_TOOLSCREATEDENTITY_END;}

	float					   m_fInstanceScale;
	float					   m_fInstanceBrightness;

	bool                       m_bScaleModified;
	bool					   m_bBrightnessModified;


	int						   m_iTemplateIdx;			// Current index of the veg template
	int						   m_iTemplateID;			// Template ID, for remapping indices when the template list changes	
	Asura_Shape_Instance*	   m_pxShapeInstance;		// ShapeInstance to represent non-LOD model instances
	Asura_BotanicalShape*      m_pxBotanicalShape;		// And this one is for models with LOD

	int						   m_iModuleIdx;
	bool                       m_bRecalculateModuleIndex;

	u_int                      m_uBotanicalFlags;
	u_int                      m_uInstanceID;

	static                     List<Tools_Botanicals_Object*> s_xObjectList;
	
#if defined( ASURA_BOTANICALS_BGR_IN_EDIT)
	// DLH - 12 October 2006 17:16:20 - BGR Instance Storage data so we know which BGR instance to edit
	Asura_Direct3D_BGR_Instances_Header* m_pxBGRInstanceHeader;
	u_int m_uBGRInstanceHeaderInstanceIndex;
	
	friend class Asura_Direct3D_BGR_Manager;
#endif
	
};

#endif //_TOOLS_BOTANICALS_OBJECT_
#endif //ASURA_USE_BOTANICALS
