//////////////////////////////////////////////////////////////////////////////////////////
//
//  FILE        :  Tools_Botanicals_Painting.cpp
//
//  Created by Dave Reed - 04/12/05
//
//  Copyright (C) 2004 Rebellion (Derby), All Rights Reserved.
//
//  Vegetation painting
//
//////////////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

#ifdef ASURA_USE_BOTANICALS

#include "Core/Asura_UserInput.h"
#include "Core/Asura_LOS.h"
#include "Core/Asura_Timers.h"
#include "../Asura/Independent/Rendering/Asura_Module.h"

#include "Mesh/Asura_Mesh.h"
#include "Rendering/Asura_Render.h"

#include "Tools_Mode_Botanicals.h"
#include "Tools_Botanicals_Object.h"

#include "Tools_Overlays.h"


#include "Botanicals/Asura_Botanicals_TemplateManager.h"
#include "Botanicals/Tools_Botanicals_Styles.h"
#include "BotanicalCreation/Tools_Mode_BotanicalCreation.h"

#include "Misc/Tools_Polygon_Selection.h"
#include "Misc/Tools_Timed_Messages.h"
#include "Misc/Tools_LOS.h"

#include "Objects/Tools_Object_System.h"

#include "Debugging/Asura_Logfile.h"
#include "Interface/Asura_Interface.h"
#include "Interface/Asura_CommandConsole.h"
#include "Interface/Asura_Menubar_System.h"
#include "Interface/Asura_Menubar.h"
#include "Interface/Asura_Overlay.h"
#include "Interface/Asura_Overlay_System.h"
#include "Interface/Asura_Dialog.h"

#if defined( ASURA_BOTANICALS_BGR_IN_EDIT)
	#include "Botanicals/Asura_Botanicals_InstanceManager.h"
#endif



// Data for the overlays
float		    Tools_Mode_Botanicals::s_fSprayRate=2.0f;
float           Tools_Mode_Botanicals::s_fSprayDensity=2.0f;
float		    Tools_Mode_Botanicals::s_fSprayRadius=10.0f;
float		    Tools_Mode_Botanicals::s_fNumToSpray=0.0f;
Asura_Vector_2  Tools_Mode_Botanicals::s_xPaintOverlayPosition=Asura_Vector_2(1.0f, 15.0f);
bool			Tools_Mode_Botanicals::s_bPaintingActive=false;
bool            Tools_Mode_Botanicals::s_bRightMouseDelete=false;
bool			Tools_Mode_Botanicals::s_bAvoidSprayOverlap=true;
Tools_Botanicals_AlignToTerrain Tools_Mode_Botanicals::s_eAlignToTerrain=TOOLS_BOTANICALS_ALIGN_FACEUP;
bool            Tools_Mode_Botanicals::s_bUseDensityTarget=false;
bool			Tools_Mode_Botanicals::s_bOneClickPlacement=false;
int				Tools_Mode_Botanicals::s_iSelectedTemplate=-1;
int				Tools_Mode_Botanicals::s_iSelectedStyle=-1;
bool			Tools_Mode_Botanicals::s_bRestrictSelection=false;
bool			Tools_Mode_Botanicals::s_bJustChangedTemplate=false;
bool			Tools_Mode_Botanicals::s_bButton1PressedForPaint=false;
bool			Tools_Mode_Botanicals::s_bButton2PressedForPaint=false;
bool			Tools_Mode_Botanicals::s_bPaintingBrightness=false;
float			Tools_Mode_Botanicals::s_fBrightnessToPaint=1.0f;

static			Asura_Overlay*	g_pxPaintOverlay=NULL;


extern void Render_Sphere( const Asura_Vector_3& xCentre, float fRadius, int iColour );


void ForceCollisionReMerge(void)
{
	Tools_Botanicals_Object::MergeObjectsIntoEnvironmentIfNeeded(true);
}


void Tools_Mode_Botanicals::DisablePainting(void)
{
	s_bPaintingActive=false;
}

void Tools_Mode_Botanicals::CreatePaintOverlay(void)
{
	// If it's already open, don't create another
	if(s_bPaintOverlayOpen) return;
	s_bPaintOverlayOpen=true;

	Asura_Overlay* pxNewOverlay;
	pxNewOverlay = new Asura_Overlay("Botanical Placement",false,&s_xPaintOverlayPosition);
	g_pxPaintOverlay=pxNewOverlay;

	Asura_Dialog*  pxNewDialog;
	Asura_Vector_2 xOffset = Asura_Vector_2(1.0f,3.0f);

	pxNewDialog = new Asura_Dialog("Active template: <None>",xOffset);
	pxNewDialog->SetHelpText("Template or Style to be placed");
	pxNewDialog->m_iControlID = DialogID_SelectedStyle;
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.y+=1.5f*Asura_Dialog::DefaultSpacing();
	UpdateSelectedPaintStyle(s_iSelectedTemplate,s_iSelectedStyle);

	pxNewDialog = new Asura_Function_Dialog("Select Template [T]",xOffset,ChangeActiveTemplate);
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.y+=1.0f*Asura_Dialog::DefaultSpacing();

	pxNewDialog = new Asura_Function_Dialog("Select Style [Y]",xOffset,ChangeActiveStyle);
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.y+=1.5f*Asura_Dialog::DefaultSpacing();

	pxNewDialog = new Asura_Function_Dialog("Edit Templates/Styles",xOffset,EditTemplates);
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.y+=1.5f*Asura_Dialog::DefaultSpacing();

	pxNewDialog = new Asura_Tickbox_Dialog("Placement Active [P]",xOffset,&s_bPaintingActive);
	pxNewDialog->SetHelpText("If ticked then the mouse buttons will be used for spraying/placing vegetation, instead of polygon selection.\nShortcut key : P");
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.y+=1.0f*Asura_Dialog::DefaultSpacing();

	pxNewDialog = new Asura_Tickbox_Dialog("One-per-click [O]",xOffset,&s_bOneClickPlacement);
	pxNewDialog->SetHelpText("Create/delete one veg instance per left-click, instead of spraying");
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();	

	pxNewDialog = new Asura_Tickbox_Dialog("Restrict Selection",xOffset,&s_bRestrictSelection);	
	pxNewDialog->SetHelpText("Only allow instances of the currently selected template to be selected");
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxNewDialog = new Asura_Tickbox_Dialog("Right-mouse Delete",xOffset,&s_bRightMouseDelete);
	pxNewDialog->SetHelpText("If ticked then the right mouse button will erase vegetation instances of the current style within the sphere cursor. Otherwise, it will select them");
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.y+=1.5f*Asura_Dialog::DefaultSpacing();

	pxNewDialog = new Asura_Numerical_Dialog("Spray radius", xOffset, &s_fSprayRadius, 0.0f, 100.0f, 0.1f, "%.1f");
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxNewDialog = new Asura_Numerical_Dialog("Spray rate", xOffset, &s_fSprayRate, 0.0f, 100.0f, 1.0f, "%.1f");
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.y+=1.0f*Asura_Dialog::DefaultSpacing();

	pxNewDialog = new Asura_Numerical_Dialog("Spray density", xOffset, &s_fSprayDensity, 0.0f, 20.0f, 0.1f, "%.1f");
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.y+=1.0f*Asura_Dialog::DefaultSpacing();

	pxNewDialog = new Asura_Dialog("",xOffset);	
	pxNewDialog->m_iControlID = DialogID_DensityInfo;
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.y+=1.5f*Asura_Dialog::DefaultSpacing();
	

	pxNewDialog = new Asura_Tickbox_Dialog("Use target density",xOffset,&s_bUseDensityTarget);
	pxNewDialog->SetHelpText("When spraying, add/remove instances to reach the target density");
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxNewDialog = new Asura_FunctionTickbox_Dialog("Align to terrain - Up [I]",xOffset, IsTicked_AlignToTerrainUp, SetTicked_AlignToTerrainUp);
	pxNewDialog->SetHelpText("Rotate instances to fit the terrain\nFavors orienting the botanicals so that they face up.");
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxNewDialog = new Asura_FunctionTickbox_Dialog("Align to terrain - Forwards [I]",xOffset, IsTicked_AlignToTerrainForwards, SetTicked_AlignToTerrainForwards);
	pxNewDialog->SetHelpText("Rotate instances to fit the terrain\nFavors orienting the botanicals so that they face forwards.");
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();


	pxNewDialog = new Asura_Tickbox_Dialog("Avoid Overlap",xOffset,&s_bAvoidSprayOverlap);
	pxNewDialog->SetHelpText("Enable collision detection to reduce vegetation overlaps when spraying");
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxNewDialog = new Asura_Tickbox_Dialog("Show static objects",xOffset,&s_bShowStaticObjects);
	pxNewDialog->SetHelpText("Show static objects - so they don't end up with a tree stuck through them...");
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();	

	pxNewDialog = new Asura_Tickbox_Dialog("Hide botanicals [H]",xOffset,&s_bHideBotanicals,&s_bHideBotanicalsClicked);
	pxNewDialog->SetHelpText("Hide/unhide all botanical objects");
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();	

	pxNewDialog = new Asura_Tickbox_Dialog("Enable lighting",xOffset,&Tools_Mode_BotanicalCreation::s_bLightingEnabled);
	pxNewDialog->SetHelpText("Turning this off makes it run faster...");
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();	

	pxNewDialog = new Asura_Tickbox_Dialog( "Show bounding volumes", xOffset, &s_bShowBoundingVolumes );
	pxNewDialog->SetHelpText( "Show bounding spheres for botanical instances and bounding boxes for botanical, erm, group things. Volumes rendered in red are outside the environment and will never be culled (except against the frustum)." );
	pxNewOverlay->AddDialog( pxNewDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxNewDialog = new Asura_Tickbox_Dialog( "Check botanical instance visibility", xOffset, &s_bCheckInstanceVisibility );
	pxNewDialog->SetHelpText( "Check the visibility system before rendering botanical instances. This applies to rendering in this tools mode only. Outside of this mode or in non-tools builds visibility will always be checked." );
	pxNewOverlay->AddDialog( pxNewDialog );
	xOffset.y += Asura_Dialog::LargeSpacing();

	pxNewDialog = new Asura_Function_Dialog("Force collision mesh re-merge",xOffset,&ForceCollisionReMerge);
	pxNewDialog->SetHelpText("Fixes up meshes not flagged as 'don't render' (which are receiving unwanted decals)");
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();	

	pxNewDialog = new Asura_Function_Dialog("Permanently merge selected",xOffset,Tools_Botanicals_Object::PermanentlyMergeSelectedObjects);
	pxNewDialog->SetHelpText("This permanently merges the meshes of the selected botanicals objects in the environment.\nOnce done the botanicals instances will be deleted.");
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();	

	pxNewDialog = new Asura_Function_Dialog("Commit permanently merged templates",xOffset,Tools_Botanicals_Object::PermanentlyMergeFlaggedObjects);
	pxNewDialog->SetHelpText("This causes any instances of templates that have been flagged for permanent merge to be merged into the environment now.\nOnce done this instances will be deleted.");
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.y+=Asura_Dialog::LargeSpacing();	



	pxNewDialog = new Asura_Tickbox_Dialog("Brightness Painting",xOffset,&s_bPaintingBrightness);
	pxNewDialog->SetHelpText("Paint instance brightness onto all instances within sphere");
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxNewDialog = new Asura_Numerical_Dialog("Brightness to paint", xOffset, &s_fBrightnessToPaint, 0.0f, 2.0f, 0.05f, "%.1f");
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.y+=1.0f*Asura_Dialog::DefaultSpacing();

	

	pxNewOverlay->m_uID = TOOLS_OVERLAY_BOTANICALS_PAINT;
	Asura_Overlay_System::SubmitNewOverlay(pxNewOverlay,false);	
}




bool Tools_Mode_Botanicals::IsInActiveStyle(int iTemplateIdx)
{
	// Single-template mode
	if (s_iSelectedTemplate>=0)
	{
		if (s_iSelectedTemplate==iTemplateIdx)
		{
			return true;
		}
		else
		{
			return false;
		}
	}

	// Style mode
	if (s_iSelectedStyle>=0)
	{
		// Todo when styles are added...
	}

	// Nothing selected - Anything is valid?
	return true;
}


void Tools_Mode_Botanicals::ChangeActiveTemplateTo(int iTemplateIdx)
{
	Asura_Botanicals_Template *pxTemplate = Asura_Botanicals_TemplateManager::GetTemplate(iTemplateIdx);

	if (!s_bJustChangedTemplate)
	{	
		Tools_Timed_Message_System::AddMessage(0xff00ff00, "Selected template: '%s'", pxTemplate->xTemplateName.GetString());		
		UpdateSelectedPaintStyle(iTemplateIdx,-1);
		s_bJustChangedTemplate = true;
	}
}



void Tools_Mode_Botanicals::ChangeActiveTemplate()
{
	ListSelection_Overlay xOverlay("Select Botanical Template:");
	DisablePainting();
	
	xOverlay.AddItem("<None>",0);
	for (int i=0;i<Asura_Botanicals_TemplateManager::GetNumTemplates();i++)	
	{
		xOverlay.AddItem(Asura_Botanicals_TemplateManager::GetTemplateName(i), 1+i);	
	}
	
	if (!xOverlay.DoModal())
	{
		return;
	}

	UpdateSelectedPaintStyle(xOverlay.GetSelectionData()-1,-1);	
}


void Tools_Mode_Botanicals::ChangeActiveStyle()
{
	ListSelection_Overlay xOverlay("Select Botanical Style:");

	DisablePainting();

	xOverlay.AddItem("<None>",0);
	for (int i=0;i<Tools_Botanicals_StyleManager::GetNumStyles();i++)	
	{		
		xOverlay.AddItem(Tools_Botanicals_StyleManager::GetStyleName(i), 1+i);	
	}

	if (!xOverlay.DoModal())
	{
		return;
	}

	UpdateSelectedPaintStyle(-1,xOverlay.GetSelectionData()-1);	
}



void Tools_Mode_Botanicals::UpdatePainting(void)
{	
	if(Asura_UserInput::KeyPressed(ASURA_KEYID_MOUSEWHEELUP))
	{
		if (s_fSprayRadius>0.5f)
		{
			s_fSprayRadius-=0.5f;
		}
	}

	if(Asura_UserInput::KeyPressed(ASURA_KEYID_MOUSEWHEELDOWN))
	{
		if (s_fSprayRadius<100.0f)
		{
			s_fSprayRadius+=0.5f;
		}
	}		
}


void Tools_Mode_Botanicals::UpdatePaintingUI(void)
{
	// Update the density info label
	static char szText[128];
	Asura_PrintfToBfr(szText,"Target num within current radius: %d", 128,true,GetTargetCountWithinSphere());
	g_pxPaintOverlay->UpdateDialogLabel(DialogID_DensityInfo,szText);
}



// Draw the sphere and up-vector line
void Tools_Mode_Botanicals::RenderSphereCursor(Asura_Vector_3& xPosition,Asura_Vector_3& xNormal)		
{		
	float fRadius=s_fSprayRadius;

	// In one-click mode we show a fixed-radius sphere cursor,
	// and change it's colour to indicate the mode
	if (s_bOneClickPlacement)
	{
		fRadius=1.5f;
	}

	Asura_Vector_3 xPos2=xPosition;

	if (s_eAlignToTerrain)
	{
		xPos2+=(xNormal*fRadius);
	}
	else
	{				
		xPos2[1]-= fRadius;
	}

	if (s_bOneClickPlacement)
	{
		Asura_Render::RenderLine(xPosition,xPos2,0xff80ffff);
		Render_Sphere(xPosition,fRadius,0x7f00ffff);
	}
	else
	{
		if (s_bPaintingBrightness)
		{
			Asura_Render::RenderLine(xPosition,xPos2,0xff808080);
			Render_Sphere(xPosition,fRadius,0x7f909090);
		}
		else
		{		
			Asura_Render::RenderLine(xPosition,xPos2,0xff80ff80);
			Render_Sphere(xPosition,fRadius,0x7f00ff00);
		}
	}
	
}


int  Tools_Mode_Botanicals::GetTargetCountWithinSphere()
{
	// Units for the density value are count per 100m^2
	float fCount=(s_fSprayRadius*s_fSprayRadius)*s_fSprayDensity* 0.03142f;

	return (int)fCount;
}



void Tools_Mode_Botanicals::OrientationFromNormal(Tools_Botanicals_AlignToTerrain eMode, Asura_Matrix_3x3& xOrientation, const Asura_Vector_3& xNormal)
{
    Asura_Vector_3 xDown(-xNormal);
	Asura_Vector_3 xForwards, xLeft;

	Asura_Assertf( Asura_Maths::Abs( 1.0f - xNormal.MagnitudeSquared() ) < 1.0e-3, ( "Normal is not normalised, mag sq. is %f", xNormal.MagnitudeSquared() ) );


    if(eMode == TOOLS_BOTANICALS_ALIGN_FACEUP)
	{
    	//favor facing up
    	if ( Asura_Maths::Abs(xDown.y) < 0.999f )
    	{
			xForwards.x = 0.0f;
			xForwards.y = -1.0f;
			xForwards.z = 0.0f;
			xLeft	  = xDown ^ xForwards;
			xLeft.Normalise();
    	    xForwards = xLeft ^ xDown;
    	}
    	else
    	{
			xForwards.x = 0.0f;
			xForwards.y = 0.0f;
			xForwards.z = 1.0f;
			xLeft	  = xDown ^ xForwards;
			xLeft.Normalise();
    	    xForwards = xLeft ^ xDown;
    	}
	}
	else
	{
    	//favor facing forward
    	if ( Asura_Maths::Abs(xDown.z) < 0.999f )
    	{
			xForwards.x = 0.0f;
			xForwards.y = 0.0f;
			xForwards.z = 1.0f;
			xLeft	  = xDown ^ xForwards;
			xLeft.Normalise();
    	    xForwards = xLeft ^ xDown;
    	}
    	else
    	{
			xForwards.x = 0.0f;
			xForwards.y = -1.0f;
			xForwards.z = 0.0f;
			xLeft	  = xDown ^ xForwards;
			xLeft.Normalise();
    	    xForwards = xLeft ^ xDown;
    	}
	}

    // Re-create the orientation matrix...
    xOrientation.e11 = xLeft[ 0 ];
    xOrientation.e21 = xLeft[ 1 ];
    xOrientation.e31 = xLeft[ 2 ];
    xOrientation.e12 = xDown[ 0 ];
    xOrientation.e22 = xDown[ 1 ];
    xOrientation.e32 = xDown[ 2 ];
    xOrientation.e13 = xForwards[ 0 ];
    xOrientation.e23 = xForwards[ 1 ];
    xOrientation.e33 = xForwards[ 2 ];

}


// This function actually does the painting (and one-click placement)
void Tools_Mode_Botanicals::Paint(Asura_Vector_3& xPosition, Asura_Vector_3& xNormal)
{
	Asura_LinkList<Tools_Botanicals_Object*> xObjectList;

	// Keep the spray rate independent of the framerate
	float fNumToAdd=Asura_Timers::GetFrameTime();
	fNumToAdd *= (s_fSprayRate/10.0f);

	
	// New feature to paint brightnesses onto existing botanicals
	if (s_bPaintingBrightness)
	{
		GetObjsWithinSphere(xPosition, s_fSprayRadius, &xObjectList, true);

		for (Asura_ListElt<Tools_Botanicals_Object*>* pxInst = xObjectList.Front(); pxInst; pxInst = pxInst->Next())
		{	
			Tools_Botanicals_Object* pObj=pxInst->Data();						

			pObj->SetBrightness(s_fBrightnessToPaint);
#if defined( ASURA_BOTANICALS_BGR_IN_EDIT)
			pObj->UpdateBGRInstance();
#endif
		}

		return;
	}


	if (s_bOneClickPlacement)
	{
		// One-click mode
		// ---------------------------------------------------
		if (Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_MOUSEBUTTON1))
		{
			s_fNumToSpray=1.0f;
		}
	}
	else
	{
		// Spray mode
		// ---------------------------------------------------
		// Keep the spray density independent of the spray area
		fNumToAdd *= (s_fSprayRadius*s_fSprayRadius);
		s_fNumToSpray+=fNumToAdd;
	}



	while(s_fNumToSpray>=1.0f)
	{
		s_fNumToSpray-=1.0f;

		// Get list of nearby objects, for collision avoidance - to avoid traversing
		// the whole list several times later on...
		xObjectList.Clear();
		
		int iCount=GetObjsWithinSphere(xPosition, s_fSprayRadius+10.0f, &xObjectList, true);

		// Calculate how many objects we need to create,
		// Create objects within the sphere
		bool bReject=true;
		int  iAttempts=10;

		Asura_Vector_3   xSpawnPos;
		Asura_Matrix_3x3 xOrientation;

		float fFrameTime=Asura_Timers::GetFrameTime();

		// Clamp the frame time.... as placement during the last frame will
		// have an adverse effect on the real frame time. Causing more to
		// be placed the frame after... unpleasant feedback effect at high
		// spray rates...
		if (fFrameTime>(1.0f/20.0f)) 
		{
			fFrameTime=(1.0f/20.0f);
		}

		// Use the selected template.
		// If we've got a style selected, randomise the template
		// selection here
		int iTemplateType = s_iSelectedTemplate;

		if (s_iSelectedStyle>=0)
		{
			iTemplateType=Tools_Botanicals_StyleManager::GetRandomTemplate(s_iSelectedStyle);
		}
		

		float fRadius=s_fSprayRadius;

		if (s_bOneClickPlacement)
		{
			fRadius=1.5f;
		}

		
		// We randomly try out several positions.
		//
		// Use a scoring system to determine which position is best. Lowest score 'wins',
		// Points are added on for being close to another instance.
		// If no position scores under SCORE_REJECT, nothing is placed
		const float    SCORE_REJECT=1000000.0f;		
		float		   fBestScore=SCORE_REJECT;
		Asura_Vector_3 xBestPos;
		Asura_Vector_3 xBestNormal;

		while (iTemplateType>=0 && (iAttempts-- >0))
		{	
			float fScore=0.0f;
			xSpawnPos = xPosition;
			bReject=false;					

			float fRandX= Asura_Random::GetFloat(-fRadius,fRadius);
			float fRandZ= Asura_Random::GetFloat(-fRadius,fRadius);

			// Random offsets, if we're spraying
			if (!s_bOneClickPlacement)
			{		
				Asura_Vector_3 xOfs=Asura_Vector_3(fRandX,0,fRandZ);

				// Orient the offset using the normal of the selected polygon
				Asura_Matrix_3x3 xOrientation;
				Asura_Maths::OrientationFromNormal(xOrientation, xNormal);				

				xSpawnPos=xSpawnPos+(xOfs*xOrientation);
			}

			// Do a height lookup. Reject it if we can't find the ground					
			Asura_Vector_3 xStartPos  = xSpawnPos;
			Asura_Vector_3 xDirection = Asura_Vector_3(0,0,0)-xNormal;  
			xStartPos=xStartPos+(xNormal*fRadius);

			Asura_LOS_InputData xLOSInput = Asura_LOS_InputData(xStartPos, xDirection, 0, fRadius*2);
			xLOSInput.m_uPolyFlagsToIgnore = AsuraPolygonFlag_DontRender;//Do test based on the render mesh instead of based on polygons that can be tested for LOS. RAR 24/10/2008
			const Asura_LOS_OutputData* pxLOSData = Asura_LOS::TestAgainstEverything( &xLOSInput);										

			if (!pxLOSData->m_bHitEnvironment)
			{
				// Nowhere to put it
				fScore+=SCORE_REJECT;
			}
			else
			{	
				xSpawnPos[1]= (xSpawnPos[1]-fRadius)+pxLOSData->m_fDistanceToIntersection;

				// Reject it if it's outside the sphere
				float fDX= xSpawnPos[0]-xPosition[0];
				float fDY= xSpawnPos[1]-xPosition[1];
				float fDZ= xSpawnPos[2]-xPosition[2];

				float fDist= (fDX*fDX)+(fDY*fDY)+(fDZ*fDZ);

				if (fDist>(fRadius*fRadius))
				{
					fScore+=SCORE_REJECT;
				}
				else
				{
					// We've got a candidate position. Do the collision test/scoring
					Asura_Botanicals_Template* pxNewTemplate=Asura_Botanicals_TemplateManager::GetTemplate(iTemplateType);

					Asura_Vector_3 xNewPos=xSpawnPos;
					xNewPos[1]+=pxNewTemplate->fCollisionHeight;

					for (Asura_ListElt<Tools_Botanicals_Object*>* pxInst = xObjectList.Front(); pxInst; pxInst = pxInst->Next())
					{	
						Tools_Botanicals_Object* pObj=pxInst->Data();						
						Asura_Botanicals_Template* pxTemplate=pObj->GetTemplate();

						Asura_Vector_3 xPos=pObj->Position;
						xPos[1]+=pxTemplate->fCollisionHeight;

						Asura_Vector_3 xSep=xNewPos-xPos;
						float fDist=xSep.Magnitude();

												
						if (s_bAvoidSprayOverlap && fDist<(pxTemplate->fCollisionRadius+pxNewTemplate->fCollisionRadius))
						{
							// Collision
							fScore+=SCORE_REJECT;
						}
						else
						{
							// Ignore height now
							xSep=xSpawnPos-pObj->Position;
							fDist=xSep.Magnitude();

							if (fDist<15)
							{							
								fScore+= ((15-fDist)*(15-fDist)*(15-fDist));
							}							
						}
					}

					// Score penalty for being close to the edge of the sphere, to avoid
					// the clumping so much near the edge
					Asura_Vector_3 xSep=xSpawnPos-xPosition;
					float fDist=fRadius-xSep.Magnitude();

					if (fDist<10.0f)
					{	
						// Scale the effect depending on the number of instances within the sphere
						float fCount=((float)iCount)/5.0f;
						fScore+= (fCount*((10.0f-fDist)*(10.0f-fDist)*(10.0f-fDist)));
					}							



					// Best candidate position yet?
					if (fScore<fBestScore)
					{
						fBestScore=fScore;
						xBestPos=xSpawnPos;
						xBestNormal=pxLOSData->m_xNormal;							
					}					
				}
			}
		}

		if (fBestScore<SCORE_REJECT)
		{				
			Tools_Botanicals_Object* pxObject=new Tools_Botanicals_Object(xBestPos,iTemplateType);					

			Tools_Mode_Botanicals::SetModifiedFlag(true);

			// Do we want to align it to the terrain?			
			if (s_eAlignToTerrain)
			{
				OrientationFromNormal(s_eAlignToTerrain,xOrientation, xBestNormal);		
				pxObject->Orientation=xOrientation;
			}

			// Random rotation around Y
			Asura_Botanicals_Template* pxTemplate=Asura_Botanicals_TemplateManager::GetTemplate(iTemplateType);
			
			Asura_Assertf( pxTemplate->fMinYRotation <= pxTemplate->fMaxYRotation, ("Rotation Values are in the wrong order. Min %f Max %f", pxTemplate->fMinYRotation, pxTemplate->fMaxYRotation));
			float fMinYRotation = Asura_Maths::Min( pxTemplate->fMinYRotation, pxTemplate->fMaxYRotation);
			float fMaxYRotation = Asura_Maths::Max( pxTemplate->fMinYRotation, pxTemplate->fMaxYRotation);
			Asura_Assertf( fMinYRotation <= fMaxYRotation, ("Rotation Values are in the wrong order. Min %f Max %f", fMinYRotation, fMaxYRotation));
			float fMinRot=((fMinYRotation/180.0f)*3.142f);
			float fMaxRot=((fMaxYRotation/180.0f)*3.142f);

			Asura_Matrix_3x3 xTmp;
			Asura_Euler_3 xEuler=Asura_Euler_3(0.0f,Asura_Random::GetFloat(fMinRot,fMaxRot),0.0f);
			Asura_Quat xQuat=Asura_Quat(xEuler);
			xQuat.ToMatrix(xTmp);
			pxObject->Orientation=pxObject->Orientation*xTmp;

			// Random scale - based on the template
			pxObject->SetScale(Asura_Random::GetFloat(pxTemplate->fMinScale,pxTemplate->fMaxScale));
			
#if defined( ASURA_BOTANICALS_BGR_IN_EDIT)
			Asura_Botanicals_InstanceManager::CreateToolObjectInBGR( pxObject);
#endif
		}
	}		
 

	xObjectList.Clear();
	int iNumObjectsInSphere=GetObjsWithinSphere(xPosition, s_fSprayRadius, &xObjectList, true);

	if (s_bUseDensityTarget && !s_bOneClickPlacement)
	{	
		if (iNumObjectsInSphere>GetTargetCountWithinSphere())
		{
			// We need to delete some...
			for (Asura_ListElt<Tools_Botanicals_Object*>* pxInst = xObjectList.Front(); pxInst; pxInst = pxInst->Next())
			{					
				if (iNumObjectsInSphere>GetTargetCountWithinSphere())
				{
					iNumObjectsInSphere--;
					delete pxInst->Data();
				}
			}
		}
	}

	xObjectList.Clear();
}



int Tools_Mode_Botanicals::GetObjsWithinSphere(Asura_Vector_3& xPosition, float fRadius,
												  Asura_LinkList<Tools_Botanicals_Object*>* pxObjectList, bool bCurrentTypeOnly)
{
	int iCount=0;
	float fRadiusSquared=s_fSprayRadius*s_fSprayRadius;

	pxObjectList->Clear();

	// Find all 
	for(LIF<Tools_Object*> xOlif(&Tools_Object_System::ObjectList);!xOlif.done();xOlif.next())
	{
		if (xOlif()->Type == TOT_Botanicals)
		{	
			Tools_Botanicals_Object* pxToolsObject = (Tools_Botanicals_Object*)xOlif();

			Asura_Vector_3 vDist=pxToolsObject->Position-xPosition;

			if (vDist.MagnitudeSquared()<=fRadiusSquared)
			{
				if ((!bCurrentTypeOnly) || IsInActiveStyle(pxToolsObject->GetTemplateIdx()))
				{
					// Check it belongs to the selected style or template
					pxObjectList->Push_Back(pxToolsObject);
					iCount++;
				}				
			}
		}
	}

	return iCount;
}




void Tools_Mode_Botanicals::Select(Asura_Vector_3& xPosition)
{	
	Asura_LinkList<Tools_Botanicals_Object*> xSelectedToolsObjs;


	float fRadiusSquared=s_fSprayRadius*s_fSprayRadius;

	if (s_bOneClickPlacement)
	{
		fRadiusSquared=1.5f*1.5f;
	}

	// Find all 
	for(LIF<Tools_Object*> xOlif(&Tools_Object_System::ObjectList);!xOlif.done();xOlif.next())
	{
		if (xOlif()->Type == TOT_Botanicals)
		{	
			Tools_Botanicals_Object* pxToolsObject = (Tools_Botanicals_Object *)xOlif();

			Asura_Vector_3 xDist=pxToolsObject->Position-xPosition;
			
			if (xDist.MagnitudeSquared()<=fRadiusSquared)
			{
				if (IsInActiveStyle(pxToolsObject->GetTemplateIdx()))
				{
					// Check it belongs to the selected style or template
					xSelectedToolsObjs.Push_Back(pxToolsObject);
				}				
			}
		}
	}


	// Select or delete the objects (depending on mode)
	for (Asura_ListElt<Tools_Botanicals_Object*>* pxInst = xSelectedToolsObjs.Front();
		pxInst; pxInst = pxInst->Next())
	{
		if (s_bRightMouseDelete)
		{
			Tools_Object_System::DeselectObject(pxInst->Data());
			pxInst->Data()->Destroy();
		}
		else
		{
			Tools_Object_System::SelectObject(pxInst->Data(),true);			
		}
	}	

	xSelectedToolsObjs.Clear();
}



void Tools_Mode_Botanicals::RenderPainting(void)
{
	//Only use the button presses if they are initially pressed while the mouse is off an overlay.
	//(The check for whether the cursor is currently on an overlay is done before this)
	//RAR 24/01/2007

	if(Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_MOUSEBUTTON1))
	{
		s_bButton1PressedForPaint = true;
	}
	else if(!Asura_UserInput::KeyPressed(ASURA_KEYID_MOUSEBUTTON1))
	{
		s_bButton1PressedForPaint = false;
	}

	if(Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_MOUSEBUTTON2))
	{
		s_bButton2PressedForPaint = true;
	}
	else if(!Asura_UserInput::KeyPressed(ASURA_KEYID_MOUSEBUTTON2))
	{
		s_bButton2PressedForPaint = false;
	}
	
	//draw a sphere where the cursor is
	int iPoly,iModule;
	Asura_Vector_3 xPosition;
	if(Tools_LOS::FindPolyAndPositionUnderCursor(iPoly,iModule,xPosition))
	{		
		Asura_Vector_3 xNormal=Asura_Vector_3(0,-1,0);

		Asura_Mesh* pxMesh = Asura_Module_System::GetMeshForModule(iModule);
		
		//get the normal for the polygon
		if(pxMesh && pxMesh->AreNormalsValid())
		{		
			xNormal = pxMesh->NormalsDataPtr[iPoly].FaceNormal;
		}


		RenderSphereCursor(xPosition, xNormal);

		// Paint - Add/remove instances to reach a target density
		if (s_bButton1PressedForPaint)
		{
			Paint(xPosition, xNormal);
		}

		// Select - Select all instances of the current style under
		// the sphere cursor
		if (s_bButton2PressedForPaint)
		{
			Select(xPosition);
		}
	}
}



void Tools_Mode_Botanicals::UpdateSelectedPaintStyle(int iTemplate, int iStyle)
{	
	s_iSelectedTemplate=iTemplate;
	s_iSelectedStyle=iStyle;

	static char s_szText[64];

	if (s_bPaintOverlayOpen)
	{
		if (s_iSelectedTemplate>=0)
		{
			Asura_PrintfToBfr(s_szText,"Template: %s",64,true,Asura_Botanicals_TemplateManager::GetTemplateName(s_iSelectedTemplate));
		}
		else
		{
			if (s_iSelectedStyle>=0)			
			{				
				Asura_PrintfToBfr(s_szText,"Style: %s",64,true,Tools_Botanicals_StyleManager::GetStyleName(s_iSelectedStyle));
			}
			else
			{
				Asura_PrintfToBfr(s_szText,"Style/Template: <none>",64,true);
			}
		}

		g_pxPaintOverlay->UpdateDialogLabel(DialogID_SelectedStyle,s_szText);
	}
}


bool Tools_Mode_Botanicals::AllowSelectionOfTemplate(int iTemplateIdx)
{
	if (!s_bRestrictSelection || s_iSelectedStyle>=0)
	{
		return true;
	}
	else
	{
		if (s_iSelectedTemplate<0 || s_iSelectedTemplate==iTemplateIdx)
		{
			return true;
		}
	}
	
	return false;
}


#endif //ASURA_USE_BOTANICALS
