#include "Core/Asura.h"

#if defined( ASURA_USE_BOTANICALS) && defined( ASURA_TOOLS) && defined( ASURA_BOTANICALS_BGR_IN_EDIT)

// main header
#include "Botanical/Asura_Direct3D_Botanicals_Game_Render.h"

// other libraries
#include "Botanical/Asura_Direct3D_Botanicals_Game_Render_Manager.h"
#include "Botanicals/Asura_Botanical_Game_Render_Instance_Lighting.h"
#include "Botanicals/Asura_Botanical_Game_Render_Instance_Transform.h"
#include "Botanicals/Asura_Botanicals_InstanceManager.h"
#include "Botanicals/Tools_Botanicals_Object.h"
#include "Rendering/Asura_Module.h"

// internal dependencies

//*******************************************************************
// Local Classes
//*******************************************************************

//*******************************************************************
// Function Definitions
//*******************************************************************

u_int Asura_Direct3D_BGR_Instances_Header::FindInActiveInstance(void)
{
	const u_int uInstanceHeaderOldInstanceCount = this->GetInstanceCount();

	{	
		for( u_int uInstanceHeaderInstanceIndex = 0; uInstanceHeaderInstanceIndex < uInstanceHeaderOldInstanceCount; uInstanceHeaderInstanceIndex += 1)
		{
			const Asura_Direct3D_BGR_Instance* pxInstance = this->GetDirect3DInstance( uInstanceHeaderInstanceIndex);
			const bool bIsActive = pxInstance->IsActive();
			if( false == bIsActive)
			{
				return( uInstanceHeaderInstanceIndex);
			}
		}
	}
	
	{
		const u_int uInstanceHeaderNewInstanceCount = Asura_Maths::Max( 10u, 2u * uInstanceHeaderOldInstanceCount);
		this->m_xArrayInstances.Exspand( uInstanceHeaderNewInstanceCount);
        m_xInstanceVisQueries.Exspand( uInstanceHeaderNewInstanceCount );
	}
	
	{
		const u_int uInstanceHeaderNewInstanceCount = this->GetInstanceCount();
		
		for( u_int uInstanceHeaderInstanceIndex = uInstanceHeaderOldInstanceCount; uInstanceHeaderInstanceIndex < uInstanceHeaderNewInstanceCount; uInstanceHeaderInstanceIndex += 1)
		{
			Asura_Direct3D_BGR_Instance* pxInstance = this->GetDirect3DInstance( uInstanceHeaderInstanceIndex);
			pxInstance->SetActive( false);
		}
		
#if defined( ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA) && 1 == ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA
		this->AllocateDynamicData( uInstanceHeaderNewInstanceCount);
#endif
	}

	return( uInstanceHeaderOldInstanceCount);
}

Asura_Direct3D_BGR_Instances_Header* Asura_Direct3D_BGR_Manager::GetModuleTypeInstanceHeader( const u_int uModuleIndex, const Asura_BGR_TypeID& xTypeID)
{
	const u_int uModuleCount = Asura_Module_System::GetNumberOfModules();
	Asura_Assertf( uModuleIndex <= uModuleCount, ("Module Index %D is out of range. Environemnt Module Count %d.", uModuleIndex, uModuleCount));
	Asura_Assertf( uModuleIndex < this->m_xArray_xModule.Size(), ("Module Index %D is out of range. BGR Module Count %d.", uModuleIndex, this->m_xArray_xModule.Size()));
	
	Asura_Direct3D_BGR_Module& xModule = m_xArray_xModule[ uModuleIndex];
	
	
	Asura_Direct3D_BGR_Instances_Header** ppxInstanceHeader = &(this->m_xArray_pxInstanceHeaders[ xModule.m_uInstanceHeaderStartIndex]);
	Asura_Direct3D_BGR_Instances_Header** ppxInstanceHeaderStop = ppxInstanceHeader + xModule.m_uInstanceHeadeCount;
	for( ; ppxInstanceHeader < ppxInstanceHeaderStop; ppxInstanceHeader += 1)
	{
		Asura_Direct3D_BGR_Instances_Header* pxInstanceHeader = *ppxInstanceHeader;
		if( NULL == pxInstanceHeader)
		{
			continue;
		}
		
		const Asura_BGR_TypeID xInstanceHeaderTypeID = pxInstanceHeader->GetType();
		if( xInstanceHeaderTypeID != xTypeID)
		{
			continue;
		}
		
		return( pxInstanceHeader);
	}
	Asura_Assert( false, "Was not able to find the Type in Module");
	return( NULL);
}

void Asura_Direct3D_BGR_Manager::MoveInstanceTo( Tools_Botanicals_Object* pxToolsObject, Asura_Direct3D_BGR_Instances_Header*& pxBGRInstanceHeader, u_int& uBGRInstanceHeaderInstanceIndex)
{
	Asura_Assert( NULL != pxToolsObject, "A valid pointer is needed");
	Asura_Assert( NULL != pxBGRInstanceHeader, "A valid pointer is needed");
		
	const u_int uModuleCount = Asura_Module_System::GetNumberOfModules();
	
	// DLH - 13 October 2006 17:05:03 - Find out if the Instance still lives in the same Module
	int iModuleEnvironmentIndex = Asura_Botanicals_InstanceManager::GetModuleIndexFromPosition( pxToolsObject->Position, pxToolsObject->Orientation);
	int iModuleBGRIndex = iModuleEnvironmentIndex;
	if( iModuleBGRIndex < 0)
	{
		iModuleBGRIndex = uModuleCount;
	}
			
	Asura_Direct3D_BGR_Instances_Header* const pxBGRInstanceHeaderFound = this->GetModuleTypeInstanceHeader( iModuleBGRIndex, pxBGRInstanceHeader->GetType());
	if( NULL == pxBGRInstanceHeaderFound)
	{
		Asura_Assert( NULL != pxBGRInstanceHeaderFound, "No Instance Header with TypeID found in module");
		return;
	}
	
	if( pxBGRInstanceHeaderFound != pxBGRInstanceHeader)
	{
		Asura_Direct3D_BGR_Instance* pxBGRInstance = pxBGRInstanceHeader->GetDirect3DInstance( uBGRInstanceHeaderInstanceIndex);
		if( NULL != pxBGRInstance)
		{
			pxBGRInstance->SetActive( false);
		}

		pxBGRInstanceHeader = pxBGRInstanceHeaderFound;
		
		const u_int uBGRInstanceHeaderInstanceIndexFound = pxBGRInstanceHeader->FindInActiveInstance();
		Asura_Assert( uBGRInstanceHeaderInstanceIndexFound < pxBGRInstanceHeader->GetInstanceCount(), "Spare Space does not seem to be forth comming.");
		uBGRInstanceHeaderInstanceIndex = uBGRInstanceHeaderInstanceIndexFound;
		
		pxToolsObject->m_iModuleIdx = iModuleEnvironmentIndex;
	}
	
	Asura_Direct3D_BGR_Instance* pxBGRInstance = pxBGRInstanceHeader->GetDirect3DInstance( uBGRInstanceHeaderInstanceIndex);
	if( NULL == pxBGRInstance)
	{
		return;
	}
	
	Asura_BGR_InstanceTransform xBGRInstanceTransform;
	
	xBGRInstanceTransform.Set( pxToolsObject->Position, pxToolsObject->Orientation, pxToolsObject->m_fInstanceScale);
		
	pxBGRInstance->SetInstanceData( xBGRInstanceTransform);

	pxBGRInstance->SetInstanceBrightness(pxToolsObject->m_fInstanceBrightness);
	pxBGRInstance->Relight();
	pxBGRInstance->SetActive( true);
	
	{
		Asura_Direct3D_BGR_Type* pxBGRType = this->GetDirect3DType( pxBGRInstanceHeader->GetType());
		if( NULL != pxBGRType)
		{
			pxBGRInstanceHeader->ExtendBoundingBox( pxBGRType, pxToolsObject->Position, pxToolsObject->Orientation, pxToolsObject->m_fInstanceScale);
		}
	}
}

bool Asura_Direct3D_BGR_Manager::FindExistingInstance( Tools_Botanicals_Object* pxToolsObject)
{
	Asura_Assert( NULL != pxToolsObject, "A Valid pointer would be nice");
	
	pxToolsObject->m_pxBGRInstanceHeader = NULL;
	pxToolsObject->m_uBGRInstanceHeaderInstanceIndex = 0;
	
	const uint32 uInstanceHeaderCount = this->m_xArray_pxInstanceHeaders.Size();
	if( 0 == uInstanceHeaderCount)
	{
		return( false);
	}
	
	Asura_Botanicals_Template* pxToolsObjectTemplate = pxToolsObject->GetTemplate();
	if( NULL == pxToolsObjectTemplate)
	{
		return( false);
	}
	
	const int iTemplateID = pxToolsObject->m_iTemplateID;
	const Asura_BGR_TypeID xTypeID( iTemplateID);
	
	{
		Asura_Direct3D_BGR_Type* pxBGRType = this->GetDirect3DType( xTypeID);
		
		if( NULL != pxBGRType)
		{
			pxBGRType->m_bHidden = (false == pxToolsObjectTemplate->bVisible);
		}
	}
	
	// DLH - 13 October 2006 11:55:39 - Set the Initial distance to the MAX value for a float. I did check this define could be used by looking for a presedent in the code.
	float fDistanceSquaredFound = FLT_MAX;
	
	Asura_AVector3 xObjectPosition( pxToolsObject->Position);
	
	const u_int uModuleCount = Asura_Module_System::GetNumberOfModules();
	int iModuleIndex = pxToolsObject->m_iModuleIdx;
	if( iModuleIndex < 0)
	{
		iModuleIndex = Asura_Botanicals_InstanceManager::GetModuleIndexFromPosition( pxToolsObject->Position, pxToolsObject->Orientation);
		pxToolsObject->m_iModuleIdx = iModuleIndex;
		if( iModuleIndex < 0)
		{
			iModuleIndex = uModuleCount;
		}
	}
	
	const u_int uModuleIndex = iModuleIndex;
	Asura_Assertf( uModuleIndex <= uModuleCount, ("Module Index %D is out of range. Environemnt Module Count %d.", uModuleIndex, uModuleCount));
	Asura_Assertf( uModuleIndex < this->m_xArray_xModule.Size(), ("Module Index %D is out of range. BGR Module Count %d.", uModuleIndex, this->m_xArray_xModule.Size()));
	
	Asura_Direct3D_BGR_Module& xModule = m_xArray_xModule[ uModuleIndex];
	
	Asura_Direct3D_BGR_Instances_Header** ppxInstanceHeader = &(this->m_xArray_pxInstanceHeaders[ xModule.m_uInstanceHeaderStartIndex]);
	Asura_Direct3D_BGR_Instances_Header** ppxInstanceHeaderStop = ppxInstanceHeader + xModule.m_uInstanceHeadeCount;
	for( ; ppxInstanceHeader < ppxInstanceHeaderStop; ppxInstanceHeader += 1)
	{
		Asura_Direct3D_BGR_Instances_Header* pxInstanceHeader = *ppxInstanceHeader;
		if( NULL == pxInstanceHeader)
		{
			continue;
		}
		
		if( xTypeID != pxInstanceHeader->GetType())
		{
			continue;
		}

		const uint32 uInstanceCount = pxInstanceHeader->GetInstanceCount();
		for( uint32 uInstanceIndex = 0; uInstanceIndex < uInstanceCount; uInstanceIndex += 1)			
		{
			Asura_Direct3D_BGR_Instance* pxInstance = pxInstanceHeader->GetDirect3DInstance( uInstanceIndex);
			
			Asura_AVector3 xInstancePosition;
			pxInstance->GetInstancePosition( xInstancePosition);

			const Asura_AVector3 xToolObjectToInstanceDelta = xInstancePosition - xObjectPosition;
			const float fToolObjectToInstanceDistanceSquared = xToolObjectToInstanceDelta.MagnitudeSquared();
			
			if( fDistanceSquaredFound <= fToolObjectToInstanceDistanceSquared)
			{
				continue;
			}
			
			fDistanceSquaredFound = fToolObjectToInstanceDistanceSquared;
			pxToolsObject->m_pxBGRInstanceHeader = pxInstanceHeader;
			pxToolsObject->m_uBGRInstanceHeaderInstanceIndex = uInstanceIndex;
		}					
	}		
	
	if( NULL == pxToolsObject->m_pxBGRInstanceHeader)
	{
		this->CreateToolObjectInBGR( pxToolsObject);
	}
	
	Asura_Assert( NULL != pxToolsObject->m_pxBGRInstanceHeader, "Instance not found");
	return( (NULL != pxToolsObject->m_pxBGRInstanceHeader) ? true : false);
}

void Asura_Direct3D_BGR_Manager::CreateToolObjectInBGR( Tools_Botanicals_Object *pxToolsObject)
{
	Asura_Direct3D_BGR_Manager& xBGRManager = *this;

	const int iTemplateIdx = pxToolsObject->m_iTemplateIdx;
	Asura_Botanicals_Template* pxBotanicalTemplate = Asura_Botanicals_TemplateManager::GetTemplate( iTemplateIdx);
	if( NULL == pxBotanicalTemplate)
	{
		return;
	}
	
	const int iTemplateID = pxToolsObject->m_iTemplateID;
	Asura_BGR_TypeID xTypeID( iTemplateID);
	
	u_int uModuleIndex = xBGRManager.GetModuleIndexFromPosition( pxToolsObject->Position, pxToolsObject->Orientation);

	Asura_Direct3D_BGR_Instances_Header* pxInstanceHeader = xBGRManager.GetModuleTypeInstanceHeader( uModuleIndex, xTypeID);
	if( NULL == pxInstanceHeader)
	{
		return;
	}
	
	pxToolsObject->m_pxBGRInstanceHeader = pxInstanceHeader;
	pxToolsObject->m_uBGRInstanceHeaderInstanceIndex = pxInstanceHeader->FindInActiveInstance();
	pxToolsObject->m_iModuleIdx = uModuleIndex;
	
	pxToolsObject->UpdateBGRInstance();
}


bool Asura_Direct3D_BGR_Manager::AddMissingTypeToModuleInstanceHeaderArrays()
{
	const u_int uModuleArraySize = this->m_xArray_xModule.Size();
	const u_int uTypeCount = this->m_xArray_xTypes.Size();
	const u_int uTypeCeiling = this->GetTypeIDCeiling();
	const u_int uModuleTypeNewCount = (Asura_BGR_TypeID::INVALID_ID != uTypeCeiling) ? (1 + uTypeCeiling) : 0;
	
	Asura_Assert( uTypeCount <= uModuleTypeNewCount, "There should really be less TypeIDs than needed!");
	
	const u_int uModuleInstanceHeaderCount = uModuleArraySize * uModuleTypeNewCount;
	
	Asura_Botanical_Array< Asura_Direct3D_BGR_Instances_Header*> xArray_xInstanceHeaders_Old = this->m_xArray_pxInstanceHeaders;
	
	if( this->m_xArray_pxInstanceHeaders.Size() == uModuleInstanceHeaderCount)
	{
		return( true);
	}
	
	this->m_xArray_pxInstanceHeaders.Exspand( uModuleInstanceHeaderCount);

	u_int uInstanceHeaderNewIndex = 0;
	for( u_int uModuleIndex = 0; uModuleIndex < uModuleArraySize; uModuleIndex += 1)
	{
		Asura_Direct3D_BGR_Module& xModule = this->m_xArray_xModule[ uModuleIndex];
		const u_int uModuleTypeCount = xModule.m_uInstanceHeadeCount;
		Asura_Assert( uModuleTypeCount < uModuleTypeNewCount, "Why are we copying an array if the size is less?");
		
		const u_int uModuleInstanceHeaderStartNewIndex = uInstanceHeaderNewIndex;
		
		for( u_int uTypeIndex = 0; uTypeIndex < uModuleTypeCount; uTypeIndex += 1, uInstanceHeaderNewIndex += 1)
		{
			Asura_Direct3D_BGR_Instances_Header* pxInstanceHeaderOld = xArray_xInstanceHeaders_Old[ xModule.m_uInstanceHeaderStartIndex + uTypeIndex];
			Asura_Direct3D_BGR_Instances_Header*& pxInstanceHeaderNew = this->m_xArray_pxInstanceHeaders[ uInstanceHeaderNewIndex];
			
			Asura_Assert( NULL == pxInstanceHeaderOld || pxInstanceHeaderOld->GetType() == uTypeIndex, "TypeID missmatch");
			pxInstanceHeaderNew = pxInstanceHeaderOld;
		}
		
		for( u_int uTypeIndex = xModule.m_uInstanceHeadeCount; uTypeIndex < uModuleTypeNewCount; uTypeIndex += 1, uInstanceHeaderNewIndex += 1)
		{
			Asura_Direct3D_BGR_Instances_Header*& pxInstanceHeader = this->m_xArray_pxInstanceHeaders[ uInstanceHeaderNewIndex];
			
			pxInstanceHeader = this->CreateInstanceHeader( 0);
			
			pxInstanceHeader->SetType( uTypeIndex);
			pxInstanceHeader->m_uImposterVBOffset = 0;
			pxInstanceHeader->m_uImposterVBStart = 0;
			pxInstanceHeader->m_uImposterVBSize = 0;
			pxInstanceHeader->m_uImposterIBStart = 0;
			pxInstanceHeader->m_uImposterIBSize = 0;

			Asura_Bounding_Box xInstanceHeaderBoundingBox;
			xInstanceHeaderBoundingBox.Reset();
			pxInstanceHeader->SetBoundingBox( xInstanceHeaderBoundingBox);
			
#if defined( ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA) && 1 == ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA
			const Asura_Direct3D_BGR_Type* pxType = this->GetDirect3DType( uTypeIndex);
			if( NULL != pxType)
			{
				pxInstanceHeader->SetupDynamicData( *pxType);
				pxInstanceHeader->AllocateDynamicData( 0);
			}
#endif			

		}
		xModule.m_uInstanceHeaderStartIndex = uModuleInstanceHeaderStartNewIndex;
		xModule.m_uInstanceHeadeCount = uModuleTypeNewCount;
	}
	
	Asura_Assert( uModuleInstanceHeaderCount == uInstanceHeaderNewIndex, "Mis match in calculations");
	
	return( true);
}

bool Asura_Direct3D_BGR_Manager::DeleteTypeID( const Asura_BGR_TypeID& xTypeID)
{
#if defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS)
	this->ClearDirect3DData_Imposters();
	this->m_bDirect3DImpostersOutofDate = true;
#endif
	
	
	u_int uTypeCount = this->m_xArray_xTypes.Size();
	for( u_int uTypeIndex = 0; uTypeIndex < uTypeCount; uTypeIndex += 1)
	{
		Asura_Direct3D_BGR_Type& xType = this->m_xArray_xTypes[ uTypeIndex];
		
		if( xTypeID != xType.GetType())
		{
			continue;
		}

		this->ClearDirect3DData_Type( xType);
		
		xType.Clear();
	}
	
	u_int uInstanceHeaderCount = this->m_xArray_pxInstanceHeaders.Size();
	for( u_int uInstanceHeaderIndex = 0; uInstanceHeaderIndex < uInstanceHeaderCount; uInstanceHeaderIndex += 1)
	{
		Asura_Direct3D_BGR_Instances_Header*& pxInstanceHeader = this->m_xArray_pxInstanceHeaders[ uInstanceHeaderIndex];
		
		if( NULL == pxInstanceHeader)
		{
			continue;
		}
		
		if( xTypeID != pxInstanceHeader->GetType())
		{
			continue;
		}
		
		delete pxInstanceHeader;
		pxInstanceHeader = NULL;
	}
	
	return( true);
}

bool Asura_Direct3D_BGR_Instances_Header::RecalculateBoundingBox( const Asura_Direct3D_BGR_Type& xType)
{
	this->m_xBoundingBox.Reset();
	
	const u_int uInstanceCount = this->GetInstanceCount();
	for( u_int uInstanceIndex = 0; uInstanceIndex < uInstanceCount; uInstanceIndex += 1)
	{
		Asura_Direct3D_BGR_Instance* pxInstance = this->GetDirect3DInstance( uInstanceIndex);
		
		if( NULL == pxInstance)
		{
			continue;
		}
		
		if( false == pxInstance->IsActive())
		{
			continue;
		}
		
		Asura_BGR_InstanceTransform xInstanceTransform;
		
		pxInstance->GetInstanceData( xInstanceTransform);
		
		const Asura_Vector_3 xPosition( xInstanceTransform.m_xPosition);
		const Asura_Matrix_3x3 xOrientation = xInstanceTransform.GetOrientation();	
		
		this->ExtendBoundingBox( &xType, xPosition, xOrientation, 1.0f);
	}
	return( true);
}

bool Asura_Direct3D_BGR_Manager::InstanceHeaderTypeChanged( const Asura_Direct3D_BGR_Type& xType)
{
	const Asura_BGR_TypeID xTypeID = xType.GetType();
	
	u_int uInstanceHeaderCount = this->m_xArray_pxInstanceHeaders.Size();
	for( u_int uInstanceHeaderIndex = 0; uInstanceHeaderIndex < uInstanceHeaderCount; uInstanceHeaderIndex += 1)
	{
		Asura_Direct3D_BGR_Instances_Header*& pxInstanceHeader = this->m_xArray_pxInstanceHeaders[ uInstanceHeaderIndex];
		
		if( NULL == pxInstanceHeader)
		{
			continue;
		}
		
		if( xTypeID != pxInstanceHeader->GetType())
		{
			continue;
		}
		
		pxInstanceHeader->RecalculateBoundingBox( xType);
		
#if defined( ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA) && 1 == ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA
		pxInstanceHeader->SetDynamicDataLayout( xType.GetDynamicDataLayout());
		pxInstanceHeader->AllocateDynamicData( pxInstanceHeader->GetInstanceCount());
#endif
	}
	
	return( true);
}

#endif