//
// Tools_Mode_Botanicals.h
//
// Created by Dave Reed - 03/11/2005
// 
// Copyright (C) 2005 Rebellion (Derby), All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifdef ASURA_USE_BOTANICALS

#ifndef __TOOLS_MODE_BOTANICALS_H__
#define __TOOLS_MODE_BOTANICALS_H__

#include "Core/Asura_Handle.h"
#include "Core/Asura_Maths.h"
#include "Core/Asura_LinkList.h"
#include "Misc/list_tem.hpp"
#include "Network/Asura_Guid.h"
#include "Tools_Mode.h"

class Tools_Botanicals_Object;

enum Tools_Botanicals_AlignToTerrain
{
	TOOLS_BOTANICALS_ALIGN_OFF,
	TOOLS_BOTANICALS_ALIGN_FACEUP,
	TOOLS_BOTANICALS_ALIGN_FACEFORWARD,

	TOOLS_BOTANICALS_ALIGN_NUMENTRY,
};


class Tools_Mode_Botanicals : public Tools_Mode
{
public:
	Tools_Mode_Botanicals()
	{
		Initialise();
	}

	virtual ~Tools_Mode_Botanicals()
	{
		Shutdown();
	}

	virtual void PreSave();
	virtual void PostSave();
	virtual void QuickSave(void);

	virtual void InitialiseMode( void);
	virtual void ShutdownMode( void);

	virtual bool IsModified();
	
	static void SetModifiedFlag(bool bValue);

	void Initialise(void);
	void Update(void);
	void RenderWorld_PostToneMap(void);
	void Shutdown(void);
	void PreShutdown( void );
	
	static bool	AllowSelectionOfTemplate(int iTemplateIdx);
	static void UpdateStaticObjects(bool *pbStaticObjectsVisible, bool *pbStaticObjectsCreated, bool *pbShowStaticObjects);

	static void	AddCollisionMeshesToExportList( List<Asura_String>* pxList );

    static bool ShouldShowBoundingVolumes( void ) { return s_bShowBoundingVolumes; }
    static bool ShouldCheckInstanceVisibility( void ) { return s_bCheckInstanceVisibility; }

private:

	enum DialogIDs
	{
		DialogID_SelectedStyle,
		DialogID_DensityInfo
	};

	// General mode-specific data
	// -----------------------------------------------------------
	static Asura_Handle		s_iMenuHandle;
	static bool				s_bPaintOverlayOpen;			// Painting window
	static bool				s_bEditTemplatesOverlayOpen;	// Template list/add window
	static bool				s_bTemplateOverlayOpen;			// Single template edit overlay

	static void				TogglePaintOverlay(void);
	static void				ToggleEditTemplatesOverlay(void);
	static void				ToggleTemplateOverlay(void);

	static bool				PaintOverlayIsOn(void)			{ return s_bPaintOverlayOpen;}
	static bool				EditTemplatesOverlayIsOn(void)  { return s_bEditTemplatesOverlayOpen;}
	static bool				TemplateOverlayIsOn(void)		{ return s_bTemplateOverlayOpen;}

	static void				EditTemplates(void);
	static void				UpdateTemplateInstanceCount(void);

	// Template editing methods/members
	// -----------------------------------------------------------
	static void				CreateEditTemplatesOverlay(void);
	static void				CreateTemplateOverlay(void);
	
	static void				CreateTemplate(void);
	static void				EditTemplate(void);
	static void				DeleteTemplate(void);
	static int				RebuildTemplateList(void);
	static void				FreeTemplateNameStrings(void);
	static void				ToggleTemplateHidden(void);
	static void             InvertTemplatesHidden(void);
	static void             UnhideAllTemplates(void);

	static void				CreateStyle(void);
	static void				EditStyle(void);
	static void				DeleteStyle(void);
	static int				RebuildStyleList(void);
	static void				FreeStyleNameStrings(void);

	static void				UpdateTemplateEditing();
	static void				ChangeShape(void);
	static void				ChangeSelectionShape(void);
	static void				ChangeBotanicalShape(void);
	static void				RemoveSelectionShape(void);
	static void				RemoveBotanicalShape(void);


	static char**			s_pszTemplateNames;			// List of template names in an array, for the UI
	static int				s_iNumTemplatesInList;
	static int 				s_iTemplateListOffset;
	static int				s_iSelectedTemplateInList;
	static int				s_iTemplateBeingEdited;

	static char**			s_pszStyleNames;				// List of style names in an array, for the UI
	static int				s_iNumStylesInList;
	static int 				s_iStyleListOffset;
	static int				s_iSelectedStyleInList;
	static class Asura_Dialog* s_pxTemplateCountDialog;
	static bool				s_bSelectedTemplateChanged;

	// Instance properties related methods/members
	// -----------------------------------------------------------
public:
	static void				CreateInstanceOverlay(Tools_Botanicals_Object* pxObj);
	static void				ChangeInstanceTemplate(void);
	static void				CloseInstanceOverlay(void);
	

	// Painting-related methods/members
	// -----------------------------------------------------------
public:
	static void             ChangeActiveTemplateTo(int iTemplateIdx);
private:
		

	static void				CreatePaintOverlay(void);
	static void				UpdatePainting(void);
	static void				UpdatePaintingUI();
	static void				RenderPainting(void);
	static void				ChangeActiveTemplate(void);	
	static void				ChangeActiveStyle(void);	
	static void				UpdateSelectedPaintStyle(int iTemplate, int iStyle);
	static void				RenderSphereCursor(Asura_Vector_3& xPosition, Asura_Vector_3& xNormal);
	static void				Paint(Asura_Vector_3& xPosition, Asura_Vector_3& xNormal);
	static void				Select(Asura_Vector_3& xPosition);
	static bool				IsInActiveStyle(int iTemplateIdx);	
	static int				GetObjsWithinSphere(Asura_Vector_3& xPosition, float fRadius, Asura_LinkList<Tools_Botanicals_Object*>* pxObjectList, bool bCurrentTypeOnly);
	static void				DisablePainting(void);
	static int				GetTargetCountWithinSphere();	
	static void				OrientationFromNormal(Tools_Botanicals_AlignToTerrain eMode, Asura_Matrix_3x3& xOrientation, const Asura_Vector_3& xNormal);

	static bool				IsTicked_AlignToTerrainUp() {return s_eAlignToTerrain==TOOLS_BOTANICALS_ALIGN_FACEUP;}
	static bool				IsTicked_AlignToTerrainForwards() {return s_eAlignToTerrain==TOOLS_BOTANICALS_ALIGN_FACEFORWARD;}
	static void				SetTicked_AlignToTerrainUp(bool bSet) {s_eAlignToTerrain = bSet ? TOOLS_BOTANICALS_ALIGN_FACEUP : TOOLS_BOTANICALS_ALIGN_OFF;}
	static void				SetTicked_AlignToTerrainForwards(bool bSet) {s_eAlignToTerrain = bSet ? TOOLS_BOTANICALS_ALIGN_FACEFORWARD : TOOLS_BOTANICALS_ALIGN_OFF;}

	static bool				s_bPaintingActive;
	static float			s_fSprayRate;
	static float			s_fSprayRadius;
	static float            s_fSprayDensity;
	static bool				s_bAvoidSprayOverlap;
	static Tools_Botanicals_AlignToTerrain s_eAlignToTerrain;
	static bool				s_bOneClickPlacement;
	static bool             s_bUseDensityTarget;
	static bool				s_bRightMouseDelete;
	static float			s_fNumToSpray;
	static int				s_iSelectedTemplate;
	static int				s_iSelectedStyle;
	static bool				s_bRestrictSelection;
	static bool				s_bJustChangedTemplate;

	static bool				s_bStaticObjectsVisible;	
	static bool				s_bStaticObjectsCreated;
	static bool				s_bShowStaticObjects;

    static bool             s_bShowBoundingVolumes;
    static bool             s_bCheckInstanceVisibility;

	static Asura_Vector_2	s_xPaintOverlayPosition;
	static Asura_Vector_2   s_xEditTemplatesOverlayPosition;	
	static Asura_Vector_2   s_xTemplateOverlayPosition;
	static Asura_Vector_2   s_xInstanceOverlayPosition;

	static bool s_bHideBotanicals;
	static bool s_bHideBotanicalsClicked;

	static bool s_bButton1PressedForPaint;
	static bool s_bButton2PressedForPaint;

	static bool	 s_bPaintingBrightness;
	static float s_fBrightnessToPaint;

};

#endif //__TOOLS_MODE_BOTANICALS_H__
#endif //ASURA_USE_BOTANICALS


