

//
// Tools_Carpet_Styles.h
//
// Created by Dave Reed - 23/11/2005
// 
// Copyright (C) 2005 Rebellion (Derby), All Rights Reserved.
//


#include "Core/Asura.h"

#ifdef ASURA_USE_CARPET

#include "Carpet/Asura_Carpet_StyleManager.h"
#include "Carpet/Asura_Carpet_MeshManager.h"
#include "Core/Asura_Maths.h"
#include "Core/Asura_NameTree.h"
#include "Core/Asura_String.h"
#include "Core/Asura_UserInput.h"
#include "Interface/Asura_Interface.h"
#include "Interface/Asura_CommandConsole.h"
#include "Interface/Asura_Menubar_System.h"
#include "Interface/Asura_Menubar.h"
#include "Interface/Asura_Overlay.h"
#include "Interface/Asura_Overlay_System.h"
#include "Interface/Asura_Dialog.h"
#include "Rendering/Asura_Overlay_Quad.h"
#include "Rendering/Asura_Render.h"
#include "Rendering/Asura_Viewport.h"

#include "Tools_Overlays.h"
#include "Carpet/Tools_Carpet_Styles.h"
#include "Carpet/Tools_Mode_Carpet.h"
#include "Misc/Tools_Modal_Overlay.h"
#include "Objects/Tools_Object_System.h"
#include "Texturing/Tools_PickTexture.h"
#include "Texturing/Tools_Texturing_Texture.h"

bool			Tools_Carpet_Styles::s_bStyleListOverlayOpen   = false;
bool			Tools_Carpet_Styles::s_bStyleEditOverlayOpen   = false;
Asura_Vector_2  Tools_Carpet_Styles::s_vListOverlayPosition    = Asura_Vector_2(25.0f, 15.0f);
Asura_Vector_2  Tools_Carpet_Styles::s_vEditOverlayPosition    = Asura_Vector_2(45.0f, 15.0f);

char		  **Tools_Carpet_Styles::s_apszStyleNames	      = NULL;
int			    Tools_Carpet_Styles::s_iNumStylesInList	      = 0;
int			    Tools_Carpet_Styles::s_iStyleListOffset		  = 0;
int             Tools_Carpet_Styles::s_iSelectedStyleInList	  = 0;
int			    Tools_Carpet_Styles::s_iStyleBeingEdited		  =	0;
int			    Tools_Carpet_Styles::s_iBillboardBeingEdited	  =	0;
int				Tools_Carpet_Styles::s_iBillboardSelectedForEditing = 0;

Tools_UV_Editor*Tools_Carpet_Styles::s_pxUVEditor = NULL;

Asura_String	Tools_Carpet_Styles::s_xStyleNameString		  = "";

static Asura_List_Dialog*    g_pxStyleList = NULL;

void Tools_Carpet_Styles::Initialise(void)
{
}


void Tools_Carpet_Styles::Shutdown(void)
{
	// Close the overlays
	CloseStyleList();
	CloseStyleEditor();	

	FreeStyleListStrings();
}


void Tools_Carpet_Styles::Update(void)
{
	if (s_bStyleEditOverlayOpen)
	{
		// Update style name string
		Asura_Carpet_Style *pxStyle=Asura_Carpet_StyleManager::GetStyle(s_iStyleBeingEdited);
		pxStyle->xName=s_xStyleNameString;

		// Check if selected billboard has changed, and recreate the overlay
		// if it has
		if (s_iBillboardSelectedForEditing!=s_iBillboardBeingEdited)
		{
			s_iBillboardBeingEdited=s_iBillboardSelectedForEditing;
			FinishUVEdit();
			OpenStyleEditor(s_iStyleBeingEdited);
		}

		if(s_pxUVEditor)
		{	
			Tools_Object_System::EnableAllControls(false);
		}

		// Update the UV editor
		UpdateUVEdit();

		// 'tab' - Toggle UV edit
		if(Asura_UserInput::DebouncedKeyPressed('\t'))
		{
			StartUVEdit();
		}
	}

	if (s_bStyleListOverlayOpen)
	{
		// Update the style list
		int nCount=RebuildStyleList();
		g_pxStyleList->ReplaceStrings(nCount,s_apszStyleNames);
	}	
}


void Tools_Carpet_Styles::Render(void)
{
	if (s_bStyleEditOverlayOpen)
	{
		// Draw the selected billboard in the bottom-right corner
		float fW=256;
		float fH=256;

		if(s_pxUVEditor)
		{
			s_pxUVEditor->Render();
		}
		else
		{		
			Asura_Carpet_Style *pxStyle=Asura_Carpet_StyleManager::GetStyle(s_iStyleBeingEdited);

			float fRatio=pxStyle->axBillboards[s_iBillboardBeingEdited].fWidth/
						pxStyle->axBillboards[s_iBillboardBeingEdited].fHeight;

			fW=fRatio*fH;

			Asura_Overlay_Quad xQuad;
			xQuad.Colour = 0xBB102050;
			xQuad.m_bNonRotated = true;
			xQuad.TextureIndex = ASURA_TEXTURE_NONE;			        
			xQuad.TranslucencyMode = ASURA_TRANSLUCENCY_NORMAL;
			xQuad.SetRectXY(Asura_Viewport::GetWidth()-(fW+16.0f), Asura_Viewport::GetHeight()-(fH+16.0f), fW, fH);
			xQuad.SetRectUV(0, 0, 1, 1);
			xQuad.Render();
			Asura_Render::FlushDisplay();

			xQuad.Colour = 0xFFFFFFFF;
			xQuad.TextureIndex=pxStyle->iTextureID;
			xQuad.SetRectUV(pxStyle->axBillboards[s_iBillboardBeingEdited].fU[0],
							pxStyle->axBillboards[s_iBillboardBeingEdited].fV[0],
							pxStyle->axBillboards[s_iBillboardBeingEdited].fU[1]-pxStyle->axBillboards[s_iBillboardBeingEdited].fU[0],
							pxStyle->axBillboards[s_iBillboardBeingEdited].fV[1]-pxStyle->axBillboards[s_iBillboardBeingEdited].fV[0]);
	
			xQuad.Render();
			Asura_Render::FlushDisplay();
		}
	}
}


void Tools_Carpet_Styles::CloseStyleList(void)
{
	if (s_bStyleListOverlayOpen) 
	{
		Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_CARPET_STYLE_LIST);
		s_bStyleListOverlayOpen=false;
	}
}


void Tools_Carpet_Styles::OpenStyleList(void)
{
	// Already open?  Close it and reopen (we might be trying to force an update)
	CloseStyleList();
	
	Asura_Overlay* pxOverlay;
	pxOverlay = new Asura_Overlay("Carpet Styles",false,&s_vListOverlayPosition);	
	Asura_Dialog*  pxDialog;
	Asura_Vector_2 offset = Asura_Vector_2(1.0f,3.0f);


	// 'Styles' list header
	pxDialog=new Asura_Dialog("Styles",offset);
	pxOverlay->AddDialog(pxDialog);
	offset.y+=1.0f*Asura_Dialog::DefaultSpacing();

	// Style List
	int nNumStyleItems=RebuildStyleList();

	Asura_List_Dialog* pxList = new Asura_List_Dialog(offset,s_apszStyleNames,&s_iStyleListOffset,&s_iSelectedStyleInList,8,nNumStyleItems,NULL);
	pxDialog = pxList;	
	g_pxStyleList=pxList;

	pxOverlay->AddDialog(pxDialog);
	offset.y += 8.0f+(2.0f*Asura_Dialog::DefaultSpacing());

	// Create, Edit, Delete buttons
	pxDialog = new Asura_Function_Dialog("Create Style",offset,CreateStyle);
	pxOverlay->AddDialog(pxDialog);
	offset.y+=1.0f*Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Edit Style",offset,EditStyle);
	pxOverlay->AddDialog(pxDialog);
	offset.y+=1.0f*Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Delete Style",offset,DeleteStyle);
	pxOverlay->AddDialog(pxDialog);
	offset.y+=1.5f*Asura_Dialog::DefaultSpacing();

	// Close button
	pxDialog = new Asura_Function_Dialog("Close",offset,CloseStyleList);
	pxOverlay->AddDialog(pxDialog);
	offset.y+=1.5f*Asura_Dialog::DefaultSpacing();

	pxOverlay->m_uID = TOOLS_OVERLAY_CARPET_STYLE_LIST;
	Asura_Overlay_System::SubmitNewOverlay(pxOverlay,false);	

	s_bStyleListOverlayOpen=true;
}



void Tools_Carpet_Styles::OpenStyleEditor(int iStyle)
{	
	Asura_Carpet_Style *pxStyle=Asura_Carpet_StyleManager::GetStyle(iStyle);

	// Close it if it's already open
	CloseStyleEditor();

	Asura_Overlay* pxOverlay;
	pxOverlay = new Asura_Overlay("Edit Carpet Style",false,&s_vEditOverlayPosition);	
	Asura_Dialog*  pxDialog;
	Asura_Vector_2 offset = Asura_Vector_2(1.0f,3.0f);

	// Style Name	
	s_xStyleNameString=pxStyle->xName;
	pxDialog= new Asura_String_Dialog("Name",offset,&s_xStyleNameString,16);
	pxOverlay->AddDialog(pxDialog);
	offset.y+=1.5f*Asura_Dialog::DefaultSpacing();
	
	// Texture Selector
	pxDialog = new Asura_Function_Dialog("Choose Texture",offset,ChooseTexture);
	pxOverlay->AddDialog(pxDialog);
	offset.y+=1.5f*Asura_Dialog::DefaultSpacing();
	
	// Density
	pxDialog = new Asura_Numerical_Dialog("Density",offset,&pxStyle->fDensity,0.1f,50.0f,0.1f,"%0.01f");
	pxOverlay->AddDialog(pxDialog);
	offset.y+=1.5f*Asura_Dialog::DefaultSpacing();

	// LOD fade start/end distance
	pxDialog = new Asura_Numerical_Dialog("LOD Fade - Near Distance",offset,&pxStyle->fLodFadeNear,1.0f,1000.0f,1.0f,"%0.01f");
	pxOverlay->AddDialog(pxDialog);
	offset.y+=1.0f*Asura_Dialog::DefaultSpacing();
	
	pxDialog = new Asura_Numerical_Dialog("LOD Fade - Far Distance",offset,&pxStyle->fLodFadeFar,1.0f,1000.0f,1.0f,"%0.01f");
	pxOverlay->AddDialog(pxDialog);
	offset.y+=1.5f*Asura_Dialog::DefaultSpacing();

#ifdef ASURA_TARGET_SM3
	// DMR 04/12/07 - Add some control over carpet movement
	// Not currently planning to go back and support this on last-gen platforms. It should be easy enough to do, though
	// if wanted, for someone that's set up to compile PS2

	// Movement scale
	pxDialog = new Asura_Numerical_Dialog("Movement Scale",offset,&pxStyle->fMovementScale,0.0f,5.0f,0.01f,"%0.01f");
	pxOverlay->AddDialog(pxDialog);
	offset.y+=1.5f*Asura_Dialog::DefaultSpacing();
#endif

	// Sequence	
	pxDialog = new Asura_Dialog("Billboard Sequence",offset);	
	pxOverlay->AddDialog(pxDialog);	
	offset.y+=1.5f*Asura_Dialog::DefaultSpacing();

	for (int n=0;n<ASURA_CARPET_BILLBOARD_SEQUENCE_LENGTH;n++)
	{
		pxDialog = new Asura_Numerical_Dialog("",offset,&pxStyle->aiSequence[n],0,ASURA_CARPET_BILLBOARDS_PER_STYLE-1,1,"%d");
		pxOverlay->AddDialog(pxDialog);

		offset.x+=5.5f;
		if ((n&3)==3)
		{
			offset.x=1.0f;
			offset.y+=Asura_Dialog::DefaultSpacing();
		}
	}
	
	offset.y+=(Asura_Dialog::DefaultSpacing()/2);

	// Billboard defs
	pxDialog = new Asura_Dialog("Billboard Definitions",offset);	
	pxOverlay->AddDialog(pxDialog);	
	offset.y+=1.5f*Asura_Dialog::DefaultSpacing();

	// Billboard #
	Asura_Numerical_Dialog *pxNumDialog = new Asura_Numerical_Dialog("Select Billboard",offset,&s_iBillboardSelectedForEditing,0,ASURA_CARPET_BILLBOARDS_PER_STYLE-1,1,"%d");	
	pxOverlay->AddDialog(pxNumDialog);	
	offset.y+=1.5f*Asura_Dialog::DefaultSpacing();

	// UVs
	pxDialog = new Asura_Function_Dialog("Edit UVs (Tab)",offset,StartUVEdit);
	offset.y+=1.0f*Asura_Dialog::DefaultSpacing();

	pxOverlay->AddDialog(pxDialog);	
	pxDialog = new Asura_Numerical_Dialog("U",offset,&pxStyle->axBillboards[s_iBillboardBeingEdited].fU[0],0.0f,1.0f,0.01f,"%0.02f");
	pxOverlay->AddDialog(pxDialog);	
	offset.x=12;
	pxDialog = new Asura_Numerical_Dialog("V (Top Left)",offset,&pxStyle->axBillboards[s_iBillboardBeingEdited].fV[0],0.0f,1.0f,0.01f,"%0.02f");	
	pxOverlay->AddDialog(pxDialog);
	offset.x=1;
	offset.y+=1.0f*Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("U",offset,&pxStyle->axBillboards[s_iBillboardBeingEdited].fU[1],0.0f,1.0f,0.01f,"%0.02f");
	pxOverlay->AddDialog(pxDialog);	
	offset.x=12;
	pxDialog = new Asura_Numerical_Dialog("V (Bottom Right)",offset,&pxStyle->axBillboards[s_iBillboardBeingEdited].fV[1],0.0f,1.0f,0.01f,"%0.02f");	
	pxOverlay->AddDialog(pxDialog);
	offset.x=1;
	offset.y+=1.5f*Asura_Dialog::DefaultSpacing();

	// MaxWidth/Height
	pxDialog = new Asura_Numerical_Dialog("Width (metres)",offset,&pxStyle->axBillboards[s_iBillboardBeingEdited].fWidth,0.01f,10.0f,0.01f,"%0.02f");
	pxOverlay->AddDialog(pxDialog);
	offset.y+=1.0f*Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Height (metres)",offset,&pxStyle->axBillboards[s_iBillboardBeingEdited].fHeight,0.01f,10.0f,0.01f,"%0.02f");
	pxOverlay->AddDialog(pxDialog);
	offset.y+=1.5f*Asura_Dialog::DefaultSpacing();

	// Min Scale
	pxDialog = new Asura_Numerical_Dialog("Minimum Scale",offset,&pxStyle->axBillboards[s_iBillboardBeingEdited].fMinScale,0.01f,1.0f,0.01f,"%0.02f");
	pxOverlay->AddDialog(pxDialog);
	offset.y+=2.0f*Asura_Dialog::DefaultSpacing();


	// Close button
	pxDialog = new Asura_Function_Dialog("Close",offset,CloseStyleEditor);
	pxOverlay->AddDialog(pxDialog);
	offset.y+=1.5f*Asura_Dialog::DefaultSpacing();

	pxOverlay->m_uID = TOOLS_OVERLAY_CARPET_STYLE_EDIT;
	Asura_Overlay_System::SubmitNewOverlay(pxOverlay,false);	
	

	s_bStyleEditOverlayOpen=true;
}



void Tools_Carpet_Styles::CloseStyleEditor(void)
{
	if (s_bStyleEditOverlayOpen) 
	{
		Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_CARPET_STYLE_EDIT);
		s_bStyleEditOverlayOpen=false;

		FinishUVEdit();
	}
}



void Tools_Carpet_Styles::FreeStyleListStrings(void)
{
	if (s_apszStyleNames!=NULL)
	{
		for(int n=0;n<s_iNumStylesInList;n++)
		{
			delete[] s_apszStyleNames[n];
			s_apszStyleNames[n] = 0;
		}

		delete [] s_apszStyleNames;
		s_apszStyleNames = 0;
	}
}

int Tools_Carpet_Styles::RebuildStyleList(void)
{
	// Same again, this time for the style list
	int nNumItems=Asura_Carpet_StyleManager::GetNumStyles();

	FreeStyleListStrings();
	
	s_apszStyleNames=new char*[nNumItems];
	s_iNumStylesInList=nNumItems;

	for(int n=0;n<nNumItems;n++)
	{
		s_apszStyleNames[n]=new char[32];		
		strncpy(s_apszStyleNames[n], Asura_Carpet_StyleManager::GetStyleName(n),32);
	}

	return nNumItems;
}




void Tools_Carpet_Styles::CreateStyle(void)
{
	int iStyle=Asura_Carpet_StyleManager::CreateStyle();
	
	s_iSelectedStyleInList=iStyle;
	s_iStyleBeingEdited=iStyle;
	s_iBillboardSelectedForEditing=s_iBillboardBeingEdited=0;
	OpenStyleEditor(s_iStyleBeingEdited);
}


void Tools_Carpet_Styles::EditStyle(void)
{
	if (Asura_Carpet_StyleManager::GetNumStyles()>0)
	{	
		s_iStyleBeingEdited=s_iSelectedStyleInList;
		s_iBillboardSelectedForEditing=s_iBillboardBeingEdited=0;
		OpenStyleEditor(s_iStyleBeingEdited);
	}
}


void Tools_Carpet_Styles::DeleteStyle(void)
{
	if (Asura_Carpet_StyleManager::GetNumStyles()>0)
	{	

	// Todo:
	//		 Close style editor	
	//       Delete it
	//		 Remap tris from UIDs


		// Ask for confirmation first...
		YesNo_Overlay xDlg("Delete this style? Are you sure?");
		if (!xDlg.DoModal())
		{
			return;
		}


		CloseStyleEditor();

		Asura_Carpet_StyleManager::RemoveStyle(s_iSelectedStyleInList);
			
		Asura_Carpet_MeshManager::ConvertFromToolData();
		Asura_Carpet_MeshManager::RemapStyles();
		Asura_Carpet_MeshManager::ConvertToToolData();		
	}
}




void Tools_Carpet_Styles::ChooseTexture(void)
{
	
	Asura_Carpet_Style *pxStyle=Asura_Carpet_StyleManager::GetStyle(s_iStyleBeingEdited);
	if (!pxStyle)
	{
		return;
	}

	int iNewTexture = Tools_PickTexture::PickTexture();

	pxStyle->iTextureID = iNewTexture;

	if (iNewTexture != ASURA_TEXTURE_NONE)
	{
		const Asura_String *pxTextureName = Asura_Texture_Management::GetTextureName(iNewTexture);
		pxStyle->xTextureName=*pxTextureName;
	}
	else
	{
		pxStyle->xTextureName="";
	}
}




void Tools_Carpet_Styles::StartUVEdit(void)
{
	Asura_Carpet_Style *pxStyle=Asura_Carpet_StyleManager::GetStyle(s_iStyleBeingEdited);

	if(s_pxUVEditor)
	{
		// Make this function work as a toggle
		FinishUVEdit();
		return;
	}

	// Stop object system mouse controls interfering
	Tools_Object_System::EnableAllControls(false);
	
	// Create the UV editor
	s_pxUVEditor = new Tools_UV_Editor;
	s_pxUVEditor->SetTextureID(pxStyle->iTextureID);
	
	// Init the uv coordinates
	s_pxUVEditor->Nodes[0].x = pxStyle->axBillboards[s_iBillboardBeingEdited].fU[0];
	s_pxUVEditor->Nodes[0].y = pxStyle->axBillboards[s_iBillboardBeingEdited].fV[0];
	s_pxUVEditor->Nodes[1].x = pxStyle->axBillboards[s_iBillboardBeingEdited].fU[1];
	s_pxUVEditor->Nodes[1].y = pxStyle->axBillboards[s_iBillboardBeingEdited].fV[0];
	s_pxUVEditor->Nodes[2].x = pxStyle->axBillboards[s_iBillboardBeingEdited].fU[1];
	s_pxUVEditor->Nodes[2].y = pxStyle->axBillboards[s_iBillboardBeingEdited].fV[1];
	s_pxUVEditor->Nodes[3].x = pxStyle->axBillboards[s_iBillboardBeingEdited].fU[0];
	s_pxUVEditor->Nodes[3].y = pxStyle->axBillboards[s_iBillboardBeingEdited].fV[1];

	s_pxUVEditor->CreateMouseModeOverlay();

}							  


void Tools_Carpet_Styles::FinishUVEdit(void)
{
	if(s_pxUVEditor)
	{
		s_pxUVEditor->CloseMouseModeOverlay();
		delete s_pxUVEditor;
		s_pxUVEditor = 0;
		Tools_Object_System::EnableAllControls(true);
	}
}

void Tools_Carpet_Styles::UpdateUVEdit(void)
{
	if(!s_pxUVEditor) return;

	Asura_Carpet_Style *pxStyle=Asura_Carpet_StyleManager::GetStyle(s_iStyleBeingEdited);

	s_pxUVEditor->HandleMouse();

	if(s_pxUVEditor->NodesMoved)
	{

		//make sure the nodes stay in rectangular formation
		s_pxUVEditor->EnsureNodesAreStillOkayForTextureWrap();

		s_pxUVEditor->Nodes[1].x = s_pxUVEditor->Nodes[2].x;
		s_pxUVEditor->Nodes[1].y = s_pxUVEditor->Nodes[0].y;

		s_pxUVEditor->Nodes[3].x = s_pxUVEditor->Nodes[0].x;
		s_pxUVEditor->Nodes[3].y = s_pxUVEditor->Nodes[2].y;

		s_pxUVEditor->NodesMoved = false;

		pxStyle->axBillboards[s_iBillboardBeingEdited].fU[0] = s_pxUVEditor->Nodes[0].x; 
		pxStyle->axBillboards[s_iBillboardBeingEdited].fV[0] = s_pxUVEditor->Nodes[0].y; 
		pxStyle->axBillboards[s_iBillboardBeingEdited].fU[1] = s_pxUVEditor->Nodes[2].x; 
		pxStyle->axBillboards[s_iBillboardBeingEdited].fV[1] = s_pxUVEditor->Nodes[2].y; 
	}
	else
	{
		//make sure the uv coordinates being edited match those on the overlay
		s_pxUVEditor->Nodes[0].x = pxStyle->axBillboards[s_iBillboardBeingEdited].fU[0];
		s_pxUVEditor->Nodes[0].y = pxStyle->axBillboards[s_iBillboardBeingEdited].fV[0];
		s_pxUVEditor->Nodes[1].x = pxStyle->axBillboards[s_iBillboardBeingEdited].fU[1];
		s_pxUVEditor->Nodes[1].y = pxStyle->axBillboards[s_iBillboardBeingEdited].fV[0];
		s_pxUVEditor->Nodes[2].x = pxStyle->axBillboards[s_iBillboardBeingEdited].fU[1];
		s_pxUVEditor->Nodes[2].y = pxStyle->axBillboards[s_iBillboardBeingEdited].fV[1];
		s_pxUVEditor->Nodes[3].x = pxStyle->axBillboards[s_iBillboardBeingEdited].fU[0];
		s_pxUVEditor->Nodes[3].y = pxStyle->axBillboards[s_iBillboardBeingEdited].fV[1];
	}
}


#endif //ASURA_USE_CARPET
