
///////////////////////////////////////////////////////////////////////////////
//
// File   : Tools_Chunk_FacialAnimation.cpp
// Author : Andy Tate
// Date   : 9-May-2008
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

#if defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)

// This file's header...
#include "ChunkSystem/Tools_Chunk_FacialAnimation.h"

// Asura independent header-files...
#include "ChunkSystem/Asura_Chunk_Stream.h"

// Tools header-files...
#include "FacialAnimation/Tools_FacialAnimation_System.h"


////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////

const u_int	Tools_Chunk_FacialAnimation_Face::s_uChunkVersion			= 1;
const u_int	Tools_Chunk_FacialAnimation_FaceAnimation::s_uChunkVersion	= 1;
const u_int	Tools_Chunk_FacialAnimation_FacePose::s_uChunkVersion		= 1;
const u_int	Tools_Chunk_FacialAnimation_FaceType::s_uChunkVersion		= 1;


////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Private functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Protected functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Public functions...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Chunk_FacialAnimation_FaceType::Process
// Author       : Andy Tate
// Date         : 9-May-2008
////////////////////////////////////////////////////////////////////////////////

bool Tools_Chunk_FacialAnimation_FaceType::Process(const Asura_Handle iFileHandle)
{
	switch(Version)
	{
		case 1:
		{
			Asura_Chunk_File_Stream		xStream(iFileHandle);
			Asura_Hash_ID				uHash;

			xStream >> uHash;
			Tools_FacialAnimation_System::CreateType(uHash, xStream);
			return(true);
		}

		default:
		{
			Asura_Assert(false, "Unexpected chunk-version found for this Tools facial-animation face.");
			return(false);
		}
	}		// End of switch.
}

/// End of function 'Tools_Chunk_FacialAnimation_FaceType::Process'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Chunk_FacialAnimation_FaceType::Create
// Author       : Andy Tate
// Date         : 9-May-2008
////////////////////////////////////////////////////////////////////////////////

void Tools_Chunk_FacialAnimation_FaceType::Create(const Asura_Hash_ID uHash,
												  const Tools_FacialAnimation_FaceType& xType,
												  Asura_Chunk_Stream& xStream)
{
	xStream.WriteChunk_Start(TOOLS_CHUNK_FACIALANIMATION_FACETYPE, s_uChunkVersion, 0);
	{
		xStream << uHash;
		xType.WriteToChunkStream(xStream);
	}
	xStream.WriteChunk_End();
}

/// End of function 'Tools_Chunk_FacialAnimation_FaceType::Create'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Chunk_FacialAnimation_Face::Process
// Author       : Andy Tate
// Date         : 9-May-2008
////////////////////////////////////////////////////////////////////////////////

bool Tools_Chunk_FacialAnimation_Face::Process(const Asura_Handle iFileHandle)
{
	switch(Version)
	{
		case 1:
		{
			Asura_Chunk_File_Stream		xStream(iFileHandle);
			Asura_Hash_ID				uHash;

			xStream >> uHash;
			Tools_FacialAnimation_System::CreateFace(uHash, xStream);
			return(true);
		}

		default:
		{
			Asura_Assert(false, "Unexpected chunk-version found for this Tools facial-animation face.");
			return(false);
		}
	}		// End of switch.
}

/// End of function 'Tools_Chunk_FacialAnimation_Face::Process'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Chunk_FacialAnimation_Face::Create
// Author       : Andy Tate
// Date         : 9-May-2008
////////////////////////////////////////////////////////////////////////////////

void Tools_Chunk_FacialAnimation_Face::Create(const Asura_Hash_ID uHash,
											  const Tools_FacialAnimation_Face& xFace,
											  Asura_Chunk_Stream& xStream)
{
	xStream.WriteChunk_Start(TOOLS_CHUNK_FACIALANIMATION_FACE, s_uChunkVersion, 0);
	{
		xStream << uHash;
		xFace.WriteToChunkStream(xStream);
	}
	xStream.WriteChunk_End();
}

/// End of function 'Tools_Chunk_FacialAnimation_Face::Create'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Chunk_FacialAnimation_FaceAnimation::Process
// Author       : Andy Tate
// Date         : 9-May-2008
////////////////////////////////////////////////////////////////////////////////

bool Tools_Chunk_FacialAnimation_FaceAnimation::Process(const Asura_Handle iFileHandle)
{
	switch(Version)
	{
		case 1:
		{
			Asura_Chunk_File_Stream		xStream(iFileHandle);
			Asura_Hash_ID				uHash;

			xStream >> uHash;
			Tools_FacialAnimation_System::CreateAnimation(uHash, xStream);
			return(true);
		}

		default:
		{
			Asura_Assert(false, "Unexpected chunk-version found for this Tools facial-animation face-animation.");
			return(false);
		}
	}		// End of switch.
}

/// End of function 'Tools_Chunk_FacialAnimation_FaceAnimation::Process'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Chunk_FacialAnimation_FaceAnimation::Create
// Author       : Andy Tate
// Date         : 9-May-2008
////////////////////////////////////////////////////////////////////////////////

void Tools_Chunk_FacialAnimation_FaceAnimation::Create(const Asura_Hash_ID uHash,
													   const Tools_FacialAnimation_FaceAnimation& xFaceAnimation,
													   Asura_Chunk_Stream& xStream)
{
	xStream.WriteChunk_Start(TOOLS_CHUNK_FACIALANIMATION_FACEANIMATION, s_uChunkVersion, 0);
	{
		xStream << uHash;
		xFaceAnimation.WriteToChunkStream(xStream);
	}
	xStream.WriteChunk_End();
}

/// End of function 'Tools_Chunk_FacialAnimation_FaceAnimation::Create'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Chunk_FacialAnimation_FacePose::Process
// Author       : Andy Tate
// Date         : 9-May-2008
////////////////////////////////////////////////////////////////////////////////

bool Tools_Chunk_FacialAnimation_FacePose::Process(const Asura_Handle iFileHandle)
{
	switch(Version)
	{
		case 1:
		{
			Asura_Chunk_File_Stream		xStream(iFileHandle);
			Asura_Hash_ID				uHash;

			xStream >> uHash;
			Tools_FacialAnimation_System::CreatePose(uHash, xStream);
			return(true);
		}
		
		default:
		{
			Asura_Assert(false, "Unexpected chunk-version found for this Tools facial-animation face-pose.");
			return(false);
		}
	}		// End of switch.
}

/// End of function 'Tools_Chunk_FacialAnimation_FacePose::Process'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Chunk_FacialAnimation_FacePose::Create
// Author       : Andy Tate
// Date         : 9-May-2008
////////////////////////////////////////////////////////////////////////////////

void Tools_Chunk_FacialAnimation_FacePose::Create(const Asura_Hash_ID uHash,
												  const Tools_FacialAnimation_FacePose& xFacePose,
												  Asura_Chunk_Stream& xStream)
{
	xStream.WriteChunk_Start(TOOLS_CHUNK_FACIALANIMATION_FACEPOSE, s_uChunkVersion, 0);
	{
		xStream << uHash;
		xFacePose.WriteToChunkStream(xStream);
	}
	xStream.WriteChunk_End();
}

/// End of function 'Tools_Chunk_FacialAnimation_FacePose::Create'. ///

#endif	//defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)


/// End of file. ///
