//////////////////////////////////////////////////////////////////////////////
//
// Tools_Chunk_StreamingSounds.cpp
//
// Copyright (c) 2008 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

/******************************************************************************/
/* Included Files                                                             */
/******************************************************************************/

#include "Core/Asura.h"

#if defined ASURA_USE_AUDIO_SYSTEM

#include "ChunkSystem/Tools_Chunk_StreamingSounds.h"

#include "ChunkSystem/Asura_Chunk_Loaders.h"
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "Core/Asura_HashTree.h"
#include "Core/Asura_String.h"
#include "Debugging/Asura_Assert.h"
#include "Sound/Asura_Audio_System.h"

/******************************************************************************/
/* Tools_Chunk_StreamingSounds Static Class Data                              */
/******************************************************************************/

const int Tools_Chunk_StreamingSounds::iCURRENT_VERSION = 0;

/******************************************************************************/
/* Tools_Chunk_StreamingSounds Class Functions                                */
/******************************************************************************/

////////////////////////////////////////////////////////////////////////////////
//
// Tools_Chunk_StreamingSounds::Process()
//
// Processes a Tools streaming sounds chunk from a file handle
//
////////////////////////////////////////////////////////////////////////////////

bool Tools_Chunk_StreamingSounds::Process( const Asura_Handle iFileHandle )
{
	Asura_Chunk_File_Stream xStream( iFileHandle );
	return Process( xStream );
}

////////////////////////////////////////////////////////////////////////////////
//
// Tools_Chunk_StreamingSounds::Process()
//
// Processes a Tools streaming sounds chunk from a stream
//
////////////////////////////////////////////////////////////////////////////////

bool Tools_Chunk_StreamingSounds::Process( Asura_Chunk_Stream& xStream )
{
	if( Version > iCURRENT_VERSION )
	{
		Asura_Assertf( false,
			("Streaming sounds chunk version (%d) is too new. Must be <= %d. You may need new code.",
			Version, iCURRENT_VERSION) );

		return false;
	}

	u_int uNumStreamingSounds;
	xStream >> uNumStreamingSounds;

	for( u_int u = 0; u < uNumStreamingSounds; ++u )
	{
		Asura_String xFilename;
		xStream >> xFilename;

		bool bLooping;
		xStream >> bLooping;

		// In Tools, we just play the source waves, so no location within the file

		const char* const szFilename = xFilename.GetString();
		Asura_Audio_System::AddStreamInfo( szFilename, szFilename, 0, bLooping );
	}

	return true;
}

/******************************************************************************/
/* Tools_Chunk_StreamingSounds Static Class Functions                         */
/******************************************************************************/

////////////////////////////////////////////////////////////////////////////////
//
// Tools_Chunk_StreamingSounds::Create()
//
// Creates a Tools streaming sounds chunk and outputs it to a stream
//
////////////////////////////////////////////////////////////////////////////////

void Tools_Chunk_StreamingSounds::Create( Asura_Chunk_Stream& xStream )
{
	xStream.WriteChunk_Start( TOOLS_CHUNK_STREAMING_SOUNDS, iCURRENT_VERSION, 0 );

	const Asura_TemplateHashTree< Asura_Audio_StreamInfo >& xTree =
		Asura_Audio_System::GetStreamInfoTree();

	// Count number of non-dialogue streams in the tree
	u_int uNumStreamingSounds = 0;
	for( Asura_TemplateHashTree_ConstIterator< Asura_Audio_StreamInfo > xIter( xTree );
		!xIter.Done(); xIter.Next() )
	{
		if( xIter.GetCurrent()->m_bDialogue ) continue;
		++uNumStreamingSounds;
	}

	xStream << uNumStreamingSounds;

	for( Asura_TemplateHashTree_ConstIterator< Asura_Audio_StreamInfo > xIter( xTree );
		!xIter.Done(); xIter.Next() )
	{
		const Asura_Audio_StreamInfo* const pxCurrInfo = xIter.GetCurrent();

		// Skip streaming dialogue which has been added through playback
		if( pxCurrInfo->m_bDialogue ) continue;

		const Asura_String& xFilename = pxCurrInfo->m_szFilename;
		xStream << xFilename;

		const bool bLooping = pxCurrInfo->m_bLooping;
		xStream << bLooping;
	}

	xStream.WriteChunk_End();
}

#endif // defined ASURA_USE_AUDIO_SYSTEM
