//////////////////////////////////////////////////////////////////////////////
//
// Tools_Chunk_ToolsLights.cpp
//
// Copyright (c) 2007 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

/******************************************************************************/
/* Included Files                                                             */
/******************************************************************************/

#include "Core/Asura.h"

#include "ChunkSystem/Tools_Chunk_ToolsLights.h"

// Asura Platform-independent
#include "ChunkSystem/Asura_Chunk_Loaders.h"
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "Core/Asura_Handle.h"

// Tools
#include "Lighting/Tools_Photon_Object.h"
#include "Objects/Tools_Object.h"
#include "Objects/Tools_Object_System.h"

/******************************************************************************/
/* Constants                                                                  */
/******************************************************************************/

const int iCURRENT_VERSION = 0;

/******************************************************************************/
/* Tools_Chunk_ToolsLights Static Class Data                                  */
/******************************************************************************/

bool Tools_Chunk_ToolsLights::s_bLoadedAnyLights = false;

/******************************************************************************/
/* Tools_Chunk_ToolsLights Class Functions                                    */
/******************************************************************************/

bool Tools_Chunk_ToolsLights::Process( Asura_Chunk_Stream& xStream ) const
{
	if( Version > iCURRENT_VERSION )
	{
		Asura_Assertf( false, ("Unknown Tools light chunk version (%d). You may need a new Tools build.", Version) );
		return false;
	}

	int iNumLights;
	xStream >> iNumLights;

	for( int i = 0; i < iNumLights; ++i )
	{
		Asura_Vector_3 xPosition;
		xStream >> xPosition;

		Asura_Matrix_3x3 xOrientation;
		xStream >> xOrientation;

		Tools_Photon_Object* const pxNewLight = Tools_Photon_Object::CreateLight( xPosition, xOrientation );
		if( !pxNewLight ) return false;

		xStream >> pxNewLight->m_fRed;
		xStream >> pxNewLight->m_fGreen;
		xStream >> pxNewLight->m_fBlue;
		xStream >> pxNewLight->m_fBrightness;
		xStream >> pxNewLight->m_fRange;
		xStream >> pxNewLight->m_bActive;

		pxNewLight->UpdatePhoton();
	}

	s_bLoadedAnyLights = (iNumLights > 0);

	return true;
}

bool Tools_Chunk_ToolsLights::Process( const int iFileHandle ) const
{
	Asura_Chunk_File_Stream xStream( iFileHandle );
	return Process( xStream );
}

/******************************************************************************/
/* Tools_Chunk_ToolsLights Static Class Functions                             */
/******************************************************************************/

void Tools_Chunk_ToolsLights::Create( Asura_Chunk_Stream& xStream )
{
	List< Tools_Object* > xLightList;
	Tools_Object_System::ListObjectsOfType( TOT_ToolsPhoton, xLightList );

	xStream.WriteChunk_Start( TOOLS_CHUNK_TOOLSLIGHTS, iCURRENT_VERSION, 0 );

	xStream << xLightList.size();

	for( CLIF< Tools_Object* > xCLIF( &xLightList ); !xCLIF.done(); xCLIF.next() )
	{
		Tools_Photon_Object* const pxCurrLight = static_cast< Tools_Photon_Object* >( xCLIF() );

		xStream << pxCurrLight->Position;
		xStream << pxCurrLight->Orientation;
		xStream << pxCurrLight->m_fRed;
		xStream << pxCurrLight->m_fGreen;
		xStream << pxCurrLight->m_fBlue;
		xStream << pxCurrLight->m_fBrightness;
		xStream << pxCurrLight->m_fRange;
		xStream << pxCurrLight->m_bActive;
	}

	xStream.WriteChunk_End();
}

void Tools_Chunk_ToolsLights::Create( const Asura_Handle iFileHandle )
{
	Asura_Chunk_File_Stream xStream( iFileHandle );
	Create( xStream );
}