//
// Tools_CutsceneModeAVI.cpp
//
// Created by Richard Rice - 16/05/2006
// 
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"
#include "Cutscene/Tools_CutsceneModeAVI.h"

#include "Core/Asura_UserInput.h"
#include "Core/Asura_Timers.h"
#include "Cutscene/Asura_Cutscene.h"
#include "Cutscene/Asura_Cutscene_Actor.h"
#include "Cutscene/Asura_Cutscene_System.h"
#include "Entity/ParticleGenerator/Asura_ClientEntity_PFX_Effect.h"
#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Overlay_System.h"
#include "Interface/Asura_Overlay.h"
#include "Cutscene/Tools_Mode_Cutscene.h"
#include "Rendering/Asura_Fog.h"
#include "Rendering/Asura_Viewport.h"
#include "Rendering/Asura_Render.h"
#include "SpecialFX/Asura_Corona_System.h"

#include "Rendering/Asura_Direct3D.h"


#include "Tools_Overlays.h"
#include "Cutscene/Tools_Cutscene_Object.h"
#include "Cutscene/Tools_Mode_Cutscene.h"
#include "Interface/Tools_ColourPicker.h"
#include "Misc/Tools_Timed_Messages.h"
#include "Misc/Tools_AVI_Writer.h"
#include "Misc/Tools_Modal_Overlay.h"
#include "Misc/Tools_IniFile.h"
#include "Misc/Tools_Render.h"
#include "PC/FileSystem/Tools_FileSystem.h"
#include "SpecialFX/PFX/Asura_PFX_System.h"


Asura_Vector_2 Tools_CutsceneModeAVI::s_xOverlayPosition(20.0f,10.0f);

bool Tools_CutsceneModeAVI::s_bConfigNeedsReset;
Tools_CutsceneModeAVI_Config Tools_CutsceneModeAVI::s_xOverlayAVIConfig;

bool Tools_CutsceneModeAVI::s_bAskForCompressionSettings;
List<Tools_CutsceneModeAVI_Config*> Tools_CutsceneModeAVI::s_xQueue;

bool Tools_CutsceneModeAVI::s_bInitialVideoModeValid;
int Tools_CutsceneModeAVI::s_iInitialWidth;
int Tools_CutsceneModeAVI::s_iInitialHeight;

Tools_CutsceneModeAVI_Config* Tools_CutsceneModeAVI::s_pxCurrentAVI = 0;

Tools_FogBackup Tools_CutsceneModeAVI::s_xFogBackup;

int Tools_CutsceneModeAVI::s_iSelectedActor = 0;
int Tools_CutsceneModeAVI::s_iActorNameListSize = 0;
char** Tools_CutsceneModeAVI::s_pszActorNames = 0;

Asura_RandomGenerator Tools_CutsceneModeAVI::s_xHeatHazeRNG;

bool Tools_CutsceneModeAVI::s_bLockAspectRatio = false;
float Tools_CutsceneModeAVI::s_fVideoAspectRatio = 0.0f;
float Tools_CutsceneModeAVI::s_fAVIAspectRatio = 0.0f;
bool Tools_CutsceneModeAVI::s_bVideoWidthChanged = false;
bool Tools_CutsceneModeAVI::s_bVideoHeightChanged = false;
bool Tools_CutsceneModeAVI::s_bAVIWidthChanged = false;
bool Tools_CutsceneModeAVI::s_bAVIHeightChanged = false;


void Tools_CutsceneModeAVI::Initialise()
{
	s_bConfigNeedsReset = true;
	s_bInitialVideoModeValid = false;

}

void Tools_CutsceneModeAVI::Shutdown()
{
	CancelAVIQueue();
	CloseOverlay();

	Asura_PFX_System::SetHeatHazeRNG(0);

}

void Tools_CutsceneModeAVI::ShowOverlay()
{
	if(Asura_Overlay_System::DoesOverlayExist(TOOLS_OVERLAY_CUTSCENE_AVI)) return;
	
	//init config to be edited if required
	if(s_bConfigNeedsReset)
	{
		s_xOverlayAVIConfig.Reset();
		s_bConfigNeedsReset = false;
	}
	//and get the current cutscene name
	if(s_xOverlayAVIConfig.m_xCutsceneName.IsBlank())
	{
		Asura_Cutscene* pxCutscene = Tools_Mode_Cutscene::GetCurrentCutscene();
		if(pxCutscene) s_xOverlayAVIConfig.SetCutscene( pxCutscene->GetName().GetString());
	}

	Asura_Overlay* pxOverlay = new Asura_Overlay("Cutscene AVI",false,&s_xOverlayPosition);

	Asura_Vector_2 xOffset(1.0f,3.0f);
	Asura_Dialog* pxDialog;

	//initialise tabs
	static char* s_aszTabs[3] = {"Load&Save","Settings","Actors"};
	Asura_Tabs_Dialog* pxTabsDialog = new Asura_Tabs_Dialog(xOffset,s_aszTabs,3);
	pxOverlay->AddDialog(pxTabsDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();

	//redirect all subsequent dialogs to the tab dialog
	pxOverlay->SetCurrentParentDialog(pxTabsDialog);	

	float fTabsStartY = xOffset.y;
	float fTabsEndY = xOffset.y;

	
	//////////////Default tab
	pxTabsDialog->SelectTab(0);

	pxDialog = new Asura_Function_Dialog("Record Default",xOffset,RecordDefault);
	pxDialog->SetHelpText("Records the currently edited cutscene using default settings");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Record NTSC",xOffset,RecordNTSC);
	pxDialog->SetHelpText("Records the currently edited cutscene using default NTSC settings");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();
	
	pxDialog = new Asura_Function_Dialog("Record PAL",xOffset,RecordPAL);
	pxDialog->SetHelpText("Records the currently edited cutscene using default PAL settings");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Record PAL & NTSC",xOffset,RecordPALAndNTSC);
	pxDialog->SetHelpText("Records the currently edited cutscene using PAL settings ,and then again using NTSC settings");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::LargeSpacing();

	pxDialog = new Asura_Function_Dialog("Load Config",xOffset,LoadConfig);
	pxDialog->SetHelpText("Load settings from a text file.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Save Config",xOffset,SaveConfig);
	pxDialog->SetHelpText("Save current settings to a text file.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Do Batch",xOffset,BatchRecord);
	pxDialog->SetHelpText("Loads settings from multiple text files , executing each one in turn.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::LargeSpacing();

	pxDialog = new Asura_Function_Dialog("Reset to Defaults",xOffset,SetToDefault);	
	pxDialog->SetHelpText("Reset everything to the default values.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	if(xOffset.y > fTabsEndY) fTabsEndY = xOffset.y;

	/////////////Main tab
	pxTabsDialog->SelectTab(1);
	xOffset.y = fTabsStartY;
	
	//cutscene selection
	char szText[200];
	if(s_xOverlayAVIConfig.m_xCutsceneName.IsBlank())
	{
		strcpy(szText,"Cutscene : *Not Set*");
	}
	else
	{
		Asura_PrintfToBfr(szText,"Cutscene : %s",sizeof(szText),true,s_xOverlayAVIConfig.m_xCutsceneName.GetString());
	}
	pxDialog = new Asura_Function_Dialog(szText,xOffset,ChooseCutscene);
	pxDialog->SetHelpText("The cutscene to be recorded");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	//filename selection
	if(s_xOverlayAVIConfig.m_xFilename.IsBlank())
	{
		strcpy(szText,"Filename : *Default*");
	}
	else
	{
		Asura_PrintfToBfr(szText,"Filename : %s",sizeof(szText),true,Tools_FileSystem::StripPath(s_xOverlayAVIConfig.m_xFilename.GetString()));
	}
	pxDialog = new Asura_Function_Dialog(szText,xOffset,ChooseFilename);
	pxDialog->SetHelpText("The filename of the AVI to write. If none is chosen , then a default one is used based on the cutscene name");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();


	pxDialog = new Asura_Tickbox_Dialog("Write Bmps",xOffset,&s_xOverlayAVIConfig.m_bWriteBmps);
	pxDialog->SetHelpText("A sequence of bitmaps will be created instead of an avi");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Frame Rate",xOffset,&s_xOverlayAVIConfig.m_fFrameRate,1.0f,999.0f,1.0f,"%.3f");
	pxDialog->SetHelpText("The frame rate of the AVI");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::LargeSpacing();

	UpdateStoredAspectRatio();

	pxDialog = new Asura_Tickbox_Dialog("'Lock Ratio'",xOffset,&s_bLockAspectRatio);
	pxDialog->SetHelpText("If enabled then editing the width or height will cause the other setting to change so as to maintain the same ratio.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();
	
	s_bVideoWidthChanged = false;
	pxDialog = new Asura_Numerical_Dialog("Video Mode Width",xOffset,&s_xOverlayAVIConfig.m_iVideoModeWidth,0,9999,1,"%d",&s_bVideoWidthChanged);
	pxDialog->SetHelpText("The video mode to use when rendering the cutscene\nIf set to zero then the current mode will be used.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	s_bVideoHeightChanged = false;
	pxDialog = new Asura_Numerical_Dialog("Video Mode Height",xOffset,&s_xOverlayAVIConfig.m_iVideoModeHeight,0,9999,1,"%d",&s_bVideoHeightChanged);
	pxDialog->SetHelpText("The video mode to use when rendering the cutscene\nIf set to zero then the current mode will be used.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	s_bAVIWidthChanged = false;
	pxDialog = new Asura_Numerical_Dialog("AVI Width",xOffset,&s_xOverlayAVIConfig.m_iAVIWidth,0,9999,1,"%d",&s_bAVIWidthChanged);
	pxDialog->SetHelpText("The width of AVI to record. This should be no larger than the video mode width.\nIf it is smaller then it the AVI will be taken from the centre of the screen.\nIf set to zero then the maximum value allowed by the video mode will be used.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	s_bAVIHeightChanged = false;
	pxDialog = new Asura_Numerical_Dialog("AVI Height",xOffset,&s_xOverlayAVIConfig.m_iAVIHeight,0,9999,1,"%d",&s_bAVIHeightChanged);
	pxDialog->SetHelpText("The height of AVI to record. This should be no larger than the video mode height.\nIf it is smaller then it the AVI will be taken from the centre of the screen.\nIf set to zero then the maximum value allowed by the video mode will be used.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Tickbox_Dialog("Half Size",xOffset,&s_xOverlayAVIConfig.m_bHalfSize);
	pxDialog->SetHelpText("If this is enabled , then each pixel in AVI is taken as the average of 4 pixels from the screen.\nThe AVI width and height must be at most half the video mode width and height if using this.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::LargeSpacing();

	pxDialog = new Asura_Tickbox_Dialog("'Env Fog'",xOffset,&s_xOverlayAVIConfig.m_bUseEnvFog);
	pxDialog->SetHelpText("If enabled then the environment will be drawn entirely using the 'fog' colour");
	pxOverlay->AddDialog(pxDialog);
	
	xOffset.x = pxDialog->m_xMaxCoord.x + 1.0f;
	pxDialog = new Tools_ColourPicker_Dialog("Env Fog Colour",xOffset,Asura_Vector_2(1.5f,1.0f),&s_xOverlayAVIConfig.m_xEnvFog.m_fR,&s_xOverlayAVIConfig.m_xEnvFog.m_fG,&s_xOverlayAVIConfig.m_xEnvFog.m_fB,1.0f,NULL);
	pxOverlay->AddDialog(pxDialog);
	xOffset.x = 1.0f;
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Tickbox_Dialog("'Actor Fog'",xOffset,&s_xOverlayAVIConfig.m_bUseActorFog);
	pxDialog->SetHelpText("If enabled then the actors will be drawn entirely using the 'fog' colour");
	pxOverlay->AddDialog(pxDialog);
	
	xOffset.x = pxDialog->m_xMaxCoord.x + 1.0f;
	pxDialog = new Tools_ColourPicker_Dialog("Actor Fog Colour",xOffset,Asura_Vector_2(1.5f,1.0f),&s_xOverlayAVIConfig.m_xActorFog.m_fR,&s_xOverlayAVIConfig.m_xActorFog.m_fG,&s_xOverlayAVIConfig.m_xActorFog.m_fB,1.0f,NULL);
	pxOverlay->AddDialog(pxDialog);
	xOffset.x = 1.0f;
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Tickbox_Dialog("Disable PFX",xOffset,&s_xOverlayAVIConfig.m_bDisablePFX);
	pxDialog->SetHelpText("Disables PFX rendering while recording the cutscene");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	


	if(xOffset.y > fTabsEndY) fTabsEndY = xOffset.y;

	/////////////Actor tab
	pxTabsDialog->SelectTab(2);
	xOffset.y = fTabsStartY;
	
	
	//Add controls for hiding actors
	BuildActorNameList();
	if(s_iActorNameListSize)
	{
		pxDialog = new Asura_Dialog("Render Actors :",xOffset);
		pxOverlay->AddDialog(pxDialog);
		xOffset.y+=Asura_Dialog::SmallSpacing();

		s_iSelectedActor = Asura_Maths::ClampToRange(s_iSelectedActor,0,s_iActorNameListSize-1);
		int iNumLines = Asura_Maths::Min(8,s_iActorNameListSize);

		pxDialog = new Asura_List_Dialog(xOffset,s_pszActorNames,&s_iSelectedActor,iNumLines,s_iActorNameListSize,IsActorVisible,ToggleActorHidden,0);
		pxDialog->SetHelpText("Select the actors to be included in the AVI");
		pxOverlay->AddDialog(pxDialog);
		xOffset.y = pxDialog->m_xMaxCoord.y + 1.0f;

		pxDialog = new Asura_Function_Dialog("Hide All",xOffset,HideAllActors);	
		pxDialog->SetHelpText("Sets all the actors to be hidden");
		pxOverlay->AddDialog(pxDialog);
		xOffset.y+=Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_Function_Dialog("Show All",xOffset,ShowAllActors);	
		pxDialog->SetHelpText("Sets all the actors to be rendered");
		pxOverlay->AddDialog(pxDialog);
		xOffset.y+=Asura_Dialog::DefaultSpacing();

	}
	else
	{
		pxDialog = new Asura_Dialog("(No Actors)",xOffset);
		pxOverlay->AddDialog(pxDialog);
		xOffset.y+=Asura_Dialog::SmallSpacing();
	}

	if(xOffset.y > fTabsEndY) fTabsEndY = xOffset.y;

	////////////End of tabs
	pxOverlay->SetCurrentParentDialog(0);
	static u_int s_uSelectedTab = 0;
	pxTabsDialog->SetSelectedTabPointer(&s_uSelectedTab);
	xOffset.y = fTabsEndY + 1.0f;



	pxDialog = new Asura_Function_Dialog("Record AVI",xOffset,RecordAVI);	
	pxDialog->SetHelpText("Start recording using the current settings.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();


	pxOverlay->m_uID = TOOLS_OVERLAY_CUTSCENE_AVI;
	Asura_Overlay_System::SubmitNewOverlay(pxOverlay);
	
}

void Tools_CutsceneModeAVI::CloseOverlay()
{
	Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_CUTSCENE_AVI);
	DeleteActorNameList();
}

void Tools_CutsceneModeAVI::UpdateStoredAspectRatio()
{
	if(s_xOverlayAVIConfig.m_iVideoModeWidth && s_xOverlayAVIConfig.m_iVideoModeHeight)
	{
		s_fVideoAspectRatio = s_xOverlayAVIConfig.m_iVideoModeWidth/(float)s_xOverlayAVIConfig.m_iVideoModeHeight;
	}
	else
	{
		s_fVideoAspectRatio = 0.0f;
	}
	if(s_xOverlayAVIConfig.m_iAVIWidth && s_xOverlayAVIConfig.m_iAVIHeight)
	{
		s_fAVIAspectRatio = s_xOverlayAVIConfig.m_iAVIWidth/(float)s_xOverlayAVIConfig.m_iAVIHeight;
	}
	else
	{
		s_fAVIAspectRatio = 0.0f;
	}
}


void Tools_CutsceneModeAVI::Update()
{
	if (Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_ESCAPE))
	{
		//pressing escape will cancel AVI recording (among other things) , so
		//also empty queue of subsequent AVIs to record
		CancelAVIQueue();
	}

	UpdateOverlay();
	


	if(!TheAviWriter.IsWritingFile())
	{
		if(s_pxCurrentAVI)
		{
			EndAVI();
		}
		
		while (s_xQueue.size() && !TheAviWriter.IsWritingFile())
		{
			s_pxCurrentAVI = s_xQueue.first_entry();
			s_xQueue.delete_first_entry();

			if(!WriteAVI(s_pxCurrentAVI))
			{
				delete s_pxCurrentAVI;
				s_pxCurrentAVI = 0;
			}
		}
		
		if(!TheAviWriter.IsWritingFile())
		{
			//queue has been emptied , so ask again for compressions settings next time an avi is done
			s_bAskForCompressionSettings = true;

			if(s_bInitialVideoModeValid)
			{
				//restore original video mode
				Tools_Render::SetVideoMode(s_iInitialWidth,s_iInitialHeight);
				s_bInitialVideoModeValid = false;
			}
		}
	}
}

void Tools_CutsceneModeAVI::UpdateOverlay()
{
	if(!Asura_Overlay_System::DoesOverlayExist(TOOLS_OVERLAY_CUTSCENE_AVI)) return;

	if(s_bLockAspectRatio)
	{
		//check for any changes to width/height
		if(s_fVideoAspectRatio)
		{
			if(s_bVideoWidthChanged)
			{
				s_xOverlayAVIConfig.m_iVideoModeHeight = Asura_Maths::FloatToInt_Near(s_xOverlayAVIConfig.m_iVideoModeWidth/s_fVideoAspectRatio);
			}
			else if(s_bVideoHeightChanged)
			{
				s_xOverlayAVIConfig.m_iVideoModeWidth = Asura_Maths::FloatToInt_Near(s_xOverlayAVIConfig.m_iVideoModeHeight*s_fVideoAspectRatio);
			}
		}
		if(s_fAVIAspectRatio)
		{
			if(s_bAVIWidthChanged)
			{
				s_xOverlayAVIConfig.m_iAVIHeight = Asura_Maths::FloatToInt_Near(s_xOverlayAVIConfig.m_iAVIWidth/s_fAVIAspectRatio);
			}
			else if(s_bAVIHeightChanged)
			{
				s_xOverlayAVIConfig.m_iAVIWidth = Asura_Maths::FloatToInt_Near(s_xOverlayAVIConfig.m_iAVIHeight*s_fAVIAspectRatio);
			}
		}
	}
	else
	{
		UpdateStoredAspectRatio();
	}
	s_bVideoWidthChanged = false;
	s_bVideoHeightChanged = false;
	s_bAVIWidthChanged = false;
	s_bAVIHeightChanged = false;
}


void Tools_CutsceneModeAVI::RecordDefault()
{
	Asura_Cutscene* pxCutscene = Tools_Mode_Cutscene::GetCurrentCutscene();

	if(pxCutscene)
	{
		//add default cutscene avi request to queue
		Tools_CutsceneModeAVI_Config* pxConfig = new Tools_CutsceneModeAVI_Config();
		pxConfig->m_xCutsceneName = pxCutscene->GetName();
		pxConfig->m_fFrameRate = 0;
		
		s_xQueue.add_entry(pxConfig);
	}
	else
	{
		Tools_Timed_Message_System::AddMessage(0xff0000,"Must be editing a cutscene first");
	}

	CloseOverlay();
}

void Tools_CutsceneModeAVI::RecordNTSC()
{
	Asura_Cutscene* pxCutscene = Tools_Mode_Cutscene::GetCurrentCutscene();

	if(pxCutscene)
	{
		//add default ntsc avi request to queue
		Tools_CutsceneModeAVI_Config* pxConfig = new Tools_CutsceneModeAVI_Config();
		pxConfig->m_xCutsceneName = pxCutscene->GetName();

		char szFilename[MAX_PATH];
		Asura_PrintfToBfr(szFilename,"Screenshots\\%s_ntsc_30fps_640_256.avi",MAX_PATH,true,pxCutscene->GetName().GetString());
		pxConfig->m_xFilename = szFilename;

		pxConfig->m_iVideoModeWidth = 1280;
		pxConfig->m_iVideoModeHeight = 960;
		pxConfig->m_iAVIWidth = 640;
		pxConfig->m_iAVIHeight = 256;
		pxConfig->m_bHalfSize = true;
		pxConfig->m_fFrameRate = 30.0f;
		
		s_xQueue.add_entry(pxConfig);
	}
	else
	{
		Tools_Timed_Message_System::AddMessage(0xff0000,"Must be editing a cutscene first");
	}

	CloseOverlay();
}

void Tools_CutsceneModeAVI::RecordPAL()
{
	Asura_Cutscene* pxCutscene = Tools_Mode_Cutscene::GetCurrentCutscene();

	if(pxCutscene)
	{
		//add default pal avi request to queue
		Tools_CutsceneModeAVI_Config* pxConfig = new Tools_CutsceneModeAVI_Config();
		pxConfig->m_xCutsceneName = pxCutscene->GetName();

		char szFilename[MAX_PATH];
		Asura_PrintfToBfr(szFilename,"Screenshots\\%s_pal_25fps_640_288.avi",MAX_PATH,true,pxCutscene->GetName().GetString());
		pxConfig->m_xFilename = szFilename;

		pxConfig->m_iVideoModeWidth = 1280;
		pxConfig->m_iVideoModeHeight = 960;
		pxConfig->m_iAVIWidth = 640;
		pxConfig->m_iAVIHeight = 288;
		pxConfig->m_bHalfSize = true;
		pxConfig->m_fFrameRate = 25.0f;
		
		s_xQueue.add_entry(pxConfig);
	}
	else
	{
		Tools_Timed_Message_System::AddMessage(0xff0000,"Must be editing a cutscene first");
	}

	CloseOverlay();
}

void Tools_CutsceneModeAVI::RecordPALAndNTSC()
{
	RecordPAL();
	RecordNTSC();
}

void Tools_CutsceneModeAVI::RecordAVI()
{
	//record using the current settings
	if(!s_xOverlayAVIConfig.m_xCutsceneName.IsBlank())
	{
		Tools_CutsceneModeAVI_Config* pxConfig = new Tools_CutsceneModeAVI_Config();
		*pxConfig = s_xOverlayAVIConfig;

		s_xQueue.add_entry(pxConfig);

		CloseOverlay();
	}
	else
	{
		Tools_Timed_Message_System::AddMessage(0xff0000,"No cutscene has been selected");
	}
}


void Tools_CutsceneModeAVI::ChooseCutscene()
{
	ListSelection_Overlay xOverlay("Select cutscene to record");
	
	for(int i=0;i<Asura_Cutscene_System::GetNumberOfCutscenes();i++)
	{
		Asura_Cutscene* pxCutscene = Asura_Cutscene_System::GetCutsceneByIndex(i);
		xOverlay.AddItem(pxCutscene->GetName().GetString());
	}

	if(!xOverlay.DoModal()) return;

	//get the selected cutscene name
	s_xOverlayAVIConfig.SetCutscene( xOverlay.GetSelectionString() );

	//rebuild the overlay , since the actor list will need to be redone
	CloseOverlay();
	ShowOverlay();

}

void Tools_CutsceneModeAVI::ChooseFilename()
{
	//ask for a filename
	Tools_FileSystem_DialogInfo xInfo;

	xInfo.DefaultExtension = "avi";

	xInfo.DefaultFilename = s_xOverlayAVIConfig.m_xFilename.GetString();
	if(!xInfo.DefaultFilename) xInfo.DefaultFilename = "";
	xInfo.DefaultPath = ".";
//	xInfo.FileFilter = "AVI Files\0*.avi\0BMP Files\0*.bmp\0";
	xInfo.FileFilter = "BMP/AVI Files\0*.avi;*.bmp\0";
	xInfo.WindowTitle = "Choose  Filename";

	Asura_Char* pszName = Tools_FileSystem::GetSaveFilename(xInfo);
	if(!pszName) return;
	
	s_xOverlayAVIConfig.m_xFilename = pszName;


	CloseOverlay();
	ShowOverlay();
}

bool Tools_CutsceneModeAVI::WriteAVI(const Tools_CutsceneModeAVI_Config* pxConfig)
{
	//find the cutscene to record
	Tools_Cutscene_Object* pxObject = Tools_Cutscene_Object::FindCutsceneObject(pxConfig->m_xCutsceneName.GetString());
	if(!pxObject)
	{
		Tools_Timed_Message_System::AddMessage(0xff0000,"Failed to find cutscene to record");
		return false;
	}

	//make sure this is the current cutscene
	if(pxObject != Tools_Mode_Cutscene::GetCurrentCutsceneObject())
	{
		Tools_Mode_Cutscene::s_pxThis->EditCutscene(pxObject);	
	}

	//get frame rate
	float fFrameRate = pxConfig->m_fFrameRate;
	if(!fFrameRate)
	{
		GetNumber_Overlay xOverlay("AVI Frame Rate",25.0f,1,1000);
		if(!xOverlay.DoModal()) return false;
		fFrameRate = xOverlay.GetNumber();
	}

	//set video mode
	if(pxConfig->m_iVideoModeWidth && pxConfig->m_iVideoModeHeight)
	{
		if(pxConfig->m_iVideoModeWidth != Asura_Viewport::GetWidth() || pxConfig->m_iVideoModeHeight != Asura_Viewport::GetHeight())
		{
			if(!s_bInitialVideoModeValid)
			{
				s_iInitialWidth = Asura_Viewport::GetWidth();
				s_iInitialHeight = Asura_Viewport::GetHeight();
				s_bInitialVideoModeValid = true;
			}
			
			if(!Tools_Render::SetVideoMode(pxConfig->m_iVideoModeWidth,pxConfig->m_iVideoModeHeight))
			{
				Tools_Timed_Message_System::AddMessage(0xff0000,"Failed to set video mode to %dx%d",pxConfig->m_iVideoModeWidth,pxConfig->m_iVideoModeHeight);
				return false;
			}
		}
	}

	//determine avi width/height to use
	int iAVIWidth,iAVIHeight;
	if(pxConfig->m_iAVIWidth && pxConfig->m_iAVIHeight)
	{
		iAVIWidth = pxConfig->m_iAVIWidth;
		iAVIHeight = pxConfig->m_iAVIHeight;

		if(pxConfig->m_bHalfSize)
		{
			iAVIWidth = Asura_Maths::Min(iAVIWidth,Asura_Viewport::GetWidth()/2);
			iAVIHeight = Asura_Maths::Min(iAVIWidth,Asura_Viewport::GetHeight()/2);
		}
		else
		{
			iAVIWidth = Asura_Maths::Min(iAVIWidth,Asura_Viewport::GetWidth());
			iAVIHeight = Asura_Maths::Min(iAVIWidth,Asura_Viewport::GetHeight());
		}
	}
	else
	{
		iAVIWidth = Asura_Viewport::GetWidth();
		iAVIHeight = Asura_Viewport::GetHeight();
		if(pxConfig->m_bHalfSize)
		{
			iAVIWidth /= 2;
			iAVIHeight /= 2;
		}
	}

	//generate filename
	char szFilenameBuffer[MAX_PATH];
	const char* szFilename = pxConfig->m_xFilename.GetString();

	if(!szFilename || !*szFilename)
	{
		int iIndex = 0;
		do
		{
			char szIndexText[20];
			if(!iIndex)
			{
				//first time try without an index
				szIndexText[0] = 0;
			}
			else
			{
				Asura_PrintfToBfr(szIndexText,"_%d",20,true,iIndex);
			}
			
			if(pxConfig->m_bWriteBmps)
			{
				//for bmps , check for the first frame of the sequence
				sprintf(szFilenameBuffer,"Screenshots\\%s%s_0000.bmp",pxConfig->m_xCutsceneName.GetString(),szIndexText);
			}
			else
			{
				//skip if multipart avi exists at this index
				sprintf(szFilenameBuffer,"Screenshots\\%s%s_Part00.avi",pxConfig->m_xCutsceneName.GetString(),szIndexText);
				if(GetFileAttributes(szFilenameBuffer)!=0xffffffff)	continue;
								
			
				sprintf(szFilenameBuffer,"Screenshots\\%s%s.avi",pxConfig->m_xCutsceneName.GetString(),szIndexText);
			}
			iIndex++;


		}while(GetFileAttributes(szFilenameBuffer)!=0xffffffff);

		szFilename = szFilenameBuffer;
			
	}
	
	//determine recording flags
	u_int uFlags = 0;
	if(!pxConfig->m_bWriteBmps)
	{
		if(!s_bAskForCompressionSettings)
		{
			uFlags |= TOOLS_AVIFLAG_USEPREVIOUSCOMPRESSIONSETTINGS;
		}
		s_bAskForCompressionSettings = false;
	}

	if(pxConfig->m_bHalfSize)
	{
		uFlags |= TOOLS_AVIFLAG_HALFSIZE;
	}

	if(pxConfig->m_bWriteBmps)
	{
		uFlags |= TOOLS_AVIFLAG_WRITEBMPS;
	}

	///start cutscene
	Tools_Mode_Cutscene::s_pxThis->AdvanceCutsceneToTime(pxObject->Cutscene->m_fCutsceneStartTime);
	Asura_Cutscene::DisableSubtitles(true);//don't want subtitles in AVI version

	//setup some RNGs
	s_xHeatHazeRNG.SetSeed(24523823);//initialise to some fixed value
	Asura_PFX_System::SetHeatHazeRNG(&s_xHeatHazeRNG);


	//and start the avi
	return TheAviWriter.StartAVI(szFilename,fFrameRate,iAVIWidth,iAVIHeight,uFlags);

}

void Tools_CutsceneModeAVI::CancelAVIQueue()
{
	//remove any queued AVIs
	while(s_xQueue.size())
	{
		delete s_xQueue.first_entry();
		s_xQueue.delete_first_entry();
	}

	//stop current one
	EndAVI();

	if(s_bInitialVideoModeValid)
	{
		//restore original video mode
		Tools_Render::SetVideoMode(s_iInitialWidth,s_iInitialHeight);
		s_bInitialVideoModeValid = false;
	}
}

void Tools_CutsceneModeAVI::EndAVI()
{
	if(s_pxCurrentAVI)
	{
		//re-enable PFX
		Asura_PFX_System::DisablePFXSystem(Tools_Mode_Cutscene::GetDisablePFX());

		delete s_pxCurrentAVI;
		s_pxCurrentAVI = NULL;

		TheAviWriter.EndAVI();
	}
}

void Tools_CutsceneModeAVI::BeginRenderWorld()
{
	if(!s_pxCurrentAVI) return;

	//fiddle with fog values if needed for AVI
	if(s_pxCurrentAVI->m_bUseEnvFog)
	{
		StartFogOverride(s_pxCurrentAVI->m_xEnvFog);

	}

	if(s_pxCurrentAVI->m_bUseEnvFog || s_pxCurrentAVI->m_bUseActorFog)
	{
		//Disable the coronas if fogging ,since they will show through the fog otherwise.
		Asura_Corona_System::DisableCoronas(true);
	}


	if(s_pxCurrentAVI->m_bDisablePFX)
	{
		//make sure PFX rendering stays disabled
		Asura_PFX_System::DisablePFXSystem(true);
	}
}

void Tools_CutsceneModeAVI::EndRenderWorld()
{
	//make sure everything is back to normal
	EndFogOverride();
	Asura_Corona_System::DisableCoronas(false);

}

void Tools_CutsceneModeAVI::PreCutsceneRender()
{
	if(!s_pxCurrentAVI) return;

	//fiddle with fog for the cutscene actors
	EndFogOverride();
	if(s_pxCurrentAVI->m_bUseActorFog)
	{
		StartFogOverride(s_pxCurrentAVI->m_xActorFog);		
	}

}

void Tools_CutsceneModeAVI::PostCutsceneRender()
{
	if(!s_pxCurrentAVI) return;

	//restore env fog value override
	EndFogOverride();
	if(s_pxCurrentAVI->m_bUseEnvFog)
	{
		StartFogOverride(s_pxCurrentAVI->m_xEnvFog);		
	}
}


void Tools_CutsceneModeAVI::StartFogOverride(const Asura_RGBA& xColour)
{
	EndFogOverride();

	//store a copy of the current valuse
	s_xFogBackup.bValid = true;
	s_xFogBackup.bEnabled = Asura_Fog::IsFogEnabled();
	s_xFogBackup.bGraphEnabled = Asura_Fog::IsGraphFogEnabled();
	s_xFogBackup.fNearPlane = Asura_Fog::GetFogNearPlane();
	s_xFogBackup.fFarPlane = Asura_Fog::GetFogFarPlane();
	s_xFogBackup.fFarPlaneValue = Asura_Fog::GetFogValueAtFarPlane();
	s_xFogBackup.fSkybox = Asura_Fog::GetSkyboxFogBlendFactor();
	s_xFogBackup.xColour = Asura_Fog::GetFogColour();

	//Insert override values in , so everything is fully fogged
	Asura_Fog::EnableGraphFog(true);
	Asura_Fog::EnableFog(true);
	Asura_Fog::SetFogNearPlane(-1000.0f);
	Asura_Fog::SetFogFarPlane(-1000.0f);
	Asura_Fog::SetFogValueAtFarPlane(1.0f);
	Asura_Fog::SetSkyboxFogBlendFactor(1.0f);
	Asura_Fog::SetFogColour(xColour);

	//Unfortunately some of the fog values only get used at the start of the frame to set up
	//direct 3d values... so we have to go behind the interface's back here.

	float fStart = Asura_Fog::GetFogNearPlane();
	float fEnd = Asura_Fog::GetFogFarPlane();
	Asura_Direct3D::D3DDevice->SetRenderState(D3DRS_FOGCOLOR, Asura_Fog::GetFogColourAsUint());
	// set the linear range info
	Asura_Direct3D::D3DDevice->SetRenderState(D3DRS_FOGSTART, *(DWORD *)(&fStart));
	Asura_Direct3D::D3DDevice->SetRenderState(D3DRS_FOGEND,   *(DWORD *)(&fEnd));
	


}

void Tools_CutsceneModeAVI::EndFogOverride()
{
	if(!s_xFogBackup.bValid) return;
	s_xFogBackup.bValid = false;

	//restore from stored fog settings
	Asura_Fog::EnableGraphFog(s_xFogBackup.bGraphEnabled);
	Asura_Fog::EnableFog(s_xFogBackup.bEnabled);
	Asura_Fog::SetFogNearPlane(s_xFogBackup.fNearPlane);
	Asura_Fog::SetFogFarPlane(s_xFogBackup.fFarPlane);
	Asura_Fog::SetFogValueAtFarPlane(s_xFogBackup.fFarPlaneValue);
	Asura_Fog::SetSkyboxFogBlendFactor(s_xFogBackup.fSkybox);
	Asura_Fog::SetFogColour(s_xFogBackup.xColour);

	float fStart = Asura_Fog::GetFogNearPlane();
	float fEnd = Asura_Fog::GetFogFarPlane();
	Asura_Direct3D::D3DDevice->SetRenderState(D3DRS_FOGCOLOR, Asura_Fog::GetFogColourAsUint());
	// set the linear range info
	Asura_Direct3D::D3DDevice->SetRenderState(D3DRS_FOGSTART, *(DWORD *)(&fStart));
	Asura_Direct3D::D3DDevice->SetRenderState(D3DRS_FOGEND,   *(DWORD *)(&fEnd));

}

void Tools_CutsceneModeAVI::BuildActorNameList()
{
	DeleteActorNameList();

	//find the selected cutscene
	if(s_xOverlayAVIConfig.m_xCutsceneName.IsBlank()) return;
	const Asura_Cutscene* pxCutscene = Asura_Cutscene_System::GetCutscene(s_xOverlayAVIConfig.m_xCutsceneName.GetString());
	if(!pxCutscene) return;


	s_iActorNameListSize = pxCutscene->NumberOfActors;
	s_pszActorNames = new char*[s_iActorNameListSize];

	int iIndex = 0;
	for(int i=0;i<pxCutscene->NumberOfActors;i++)
	{
		//skip camera and camera target actors
		const Asura_Cutscene_Actor* pxActor = pxCutscene->Actors[i];
		if(pxActor->IsCamera() || pxActor->IsCameraTarget())
		{
			s_iActorNameListSize--;
		}
		else
		{
			//get this actor's name
			s_pszActorNames[iIndex] = new char[pxActor->ActorName.GetLength()+1];
			strcpy(s_pszActorNames[iIndex],pxActor->ActorName.GetString());
			iIndex++;
		}
	}

}

void Tools_CutsceneModeAVI::DeleteActorNameList()
{
	for(int i=0;i<s_iActorNameListSize;i++)
	{
		delete [] s_pszActorNames[i];
	}
	delete [] s_pszActorNames;

	s_iActorNameListSize = 0;
	s_pszActorNames = 0;
}

bool Tools_CutsceneModeAVI::IsActorVisible(int iIndex,int iUserData)
{
	Asura_Assert(iIndex>=0 && iIndex<s_iActorNameListSize,"");

	const char* szName = s_pszActorNames[iIndex];
	//search the list for this name
	for(LIF<Asura_String> xNameLIF(&s_xOverlayAVIConfig.m_xHideActorList);!xNameLIF.done();xNameLIF.next())
	{
		if(!xNameLIF().Compare(szName)) return false;
	}
	return true;
}

void Tools_CutsceneModeAVI::ToggleActorHidden(int iIndex,int iUserData)
{
	Asura_Assert(iIndex>=0 && iIndex<s_iActorNameListSize,"");

	const char* szName = s_pszActorNames[iIndex];
	//search the list for this name
	for(LIF<Asura_String> xNameLIF(&s_xOverlayAVIConfig.m_xHideActorList);!xNameLIF.done();xNameLIF.next())
	{
		if(!xNameLIF().Compare(szName))
		{
			//remove from list then
			xNameLIF.delete_current();
			return;
		}
	}
	//wasn't in list , so add it
	s_xOverlayAVIConfig.m_xHideActorList.add_entry(szName);
}

bool Tools_CutsceneModeAVI::ShouldHideActor(const class Asura_Cutscene_Actor* pxActor)
{
	//This system will only need to hide actors if an AVI is currently being recorded.
	if(!s_pxCurrentAVI) return false;

	//see if the actor is in the hidden actor list
	Asura_Assert(pxActor,"");
	const char* szName = pxActor->ActorName.GetString();

	for(LIF<Asura_String> xNameLIF(&s_pxCurrentAVI->m_xHideActorList);!xNameLIF.done();xNameLIF.next())
	{
		if(!xNameLIF().Compare(szName)) return true;
	}

	return false;
}

void Tools_CutsceneModeAVI::HideAllActors()
{
	//empty the list
	ShowAllActors();

	//and add all actor names in again
	for(int i=0;i<s_iActorNameListSize;i++)
	{
		s_xOverlayAVIConfig.m_xHideActorList.add_entry(s_pszActorNames[i]);
	}
}

void Tools_CutsceneModeAVI::ShowAllActors()
{
	while(s_xOverlayAVIConfig.m_xHideActorList.size())	
	{
		s_xOverlayAVIConfig.m_xHideActorList.delete_first_entry();
	}
}


void Tools_CutsceneModeAVI::SetToDefault()
{
	s_xOverlayAVIConfig.Reset();
	CloseOverlay();
	ShowOverlay();
}

void Tools_CutsceneModeAVI::SaveConfig()
{
	const Tools_CutsceneModeAVI_Config* pxConfig = &s_xOverlayAVIConfig;
	
	//get filename for text file ,use a default based on the cutscene name
	Tools_FileSystem_DialogInfo xInfo;

	char szDefaultName[MAX_PATH] = "";
	if(!pxConfig->m_xCutsceneName.IsBlank())
	{
		strcpy(szDefaultName,  pxConfig->m_xCutsceneName.GetString());
	}

	xInfo.DefaultExtension = "txt";
    xInfo.DefaultFilename = szDefaultName;
	xInfo.DefaultPath = "AviConfig";
	xInfo.FileFilter = "Text files\0*.txt\0";
	xInfo.WindowTitle = "Select filename for config";

	Tools_FileSystem::EnsureDirectoryExists(xInfo.DefaultPath);

	Asura_Char* pszName = Tools_FileSystem::GetSaveFilename(xInfo);
	if(!pszName) return;

	FILE* pFile = fopen(pszName,"wt");
	if(!pFile)
	{
		ShowMessageOverlay("Failed to write to %s",pszName);
		return;
	}
			
	fprintf(pFile,"#Cutscene mode AVI configuration file\n\n");

	fprintf(pFile,"[Settings]\n");
	if(!pxConfig->m_xCutsceneName.IsBlank())
	{
		fprintf(pFile,"Cutscene = %s\n",pxConfig->m_xCutsceneName.GetString());
	}
	if(!pxConfig->m_xFilename.IsBlank())
	{
		fprintf(pFile,"Filename = %s\n",pxConfig->m_xFilename.GetString());
	}
	fprintf(pFile,"Write Bmps = %s\n",pxConfig->m_bWriteBmps ? "True" : "False");
	
	fprintf(pFile,"\nVideo Mode Width = %d\n",pxConfig->m_iVideoModeWidth);
	fprintf(pFile,"Video Mode Height = %d\n",pxConfig->m_iVideoModeHeight);
	fprintf(pFile,"AVI Width = %d\n",pxConfig->m_iAVIWidth);
	fprintf(pFile,"AVI Height = %d\n",pxConfig->m_iAVIHeight);
	fprintf(pFile,"Half Size = %s\n",pxConfig->m_bHalfSize ? "True" : "False");

	fprintf(pFile,"\nDisable PFX = %s\n",pxConfig->m_bDisablePFX ? "True" : "False");
	fprintf(pFile,"Frame Rate = %.3f\n",pxConfig->m_fFrameRate);

	fprintf(pFile,"\nUse Env Fog = %s\n",pxConfig->m_bUseEnvFog ? "True" : "False");
	fprintf(pFile,"Env Fog Red = %d\n",Asura_Maths::FloatToInt_Near(pxConfig->m_xEnvFog.m_fR*255.0f));
	fprintf(pFile,"Env Fog Green = %d\n",Asura_Maths::FloatToInt_Near(pxConfig->m_xEnvFog.m_fG*255.0f));
	fprintf(pFile,"Env Fog Blue = %d\n",Asura_Maths::FloatToInt_Near(pxConfig->m_xEnvFog.m_fB*255.0f));

	fprintf(pFile,"\nUse Actor Fog = %s\n",pxConfig->m_bUseActorFog ? "True" : "False");
	fprintf(pFile,"Actor Fog Red = %d\n",Asura_Maths::FloatToInt_Near(pxConfig->m_xActorFog.m_fR*255.0f));
	fprintf(pFile,"Actor Fog Green = %d\n",Asura_Maths::FloatToInt_Near(pxConfig->m_xActorFog.m_fG*255.0f));
	fprintf(pFile,"Actor Fog Blue = %d\n",Asura_Maths::FloatToInt_Near(pxConfig->m_xActorFog.m_fB*255.0f));

	fprintf(pFile,"\n");

	fprintf(pFile,"#List of actors to hide\n");
	fprintf(pFile,"[Actors]\n");
	for(CLIF<Asura_String> xActorLIF(&pxConfig->m_xHideActorList);!xActorLIF.done();xActorLIF.next())
	{
		fprintf(pFile,"%s\n",xActorLIF().GetString());
	}

	fclose(pFile);

	Tools_Timed_Message_System::AddMessage(0xffffff,"%s written",pszName);

}

void Tools_CutsceneModeAVI::LoadConfig()
{
	Tools_FileSystem_DialogInfo	xFileSelector;

	xFileSelector.DefaultExtension		= ".txt";
	xFileSelector.DefaultFilename		= "";
	xFileSelector.DefaultPath			= "AviConfig";
	xFileSelector.FileFilter			= "Text Files\0*.txt;\0";
	xFileSelector.WindowTitle			= "Load config from which file?";
	xFileSelector.AllowMultipleSelect	= false;

	Asura_String xFilename = Tools_FileSystem::GetLoadFilename( xFileSelector );
	if(xFilename.IsBlank()) return;

	if(LoadConfig(&s_xOverlayAVIConfig,xFilename.GetString()))
	{
		CloseOverlay();
		ShowOverlay();
	}
}

void Tools_CutsceneModeAVI::BatchRecord()
{
	//get a list of config files to use
	Tools_FileSystem_DialogInfo	xFileSelector;

	xFileSelector.DefaultExtension		= ".txt";
	xFileSelector.DefaultFilename		= "";
	xFileSelector.DefaultPath			= "AviConfig";
	xFileSelector.FileFilter			= "Text Files\0*.txt;\0";
	xFileSelector.WindowTitle			= "Load config from which file?";
	xFileSelector.AllowMultipleSelect	= true;

	Asura_String xFilename = Tools_FileSystem::GetLoadFilename( xFileSelector );

	while(!xFilename.IsBlank())
	{
		//load each config and add it to the queue
		Tools_CutsceneModeAVI_Config* pxConfig = new Tools_CutsceneModeAVI_Config;
		if(LoadConfig(pxConfig,xFilename.GetString()))
		{
			s_xQueue.add_entry(pxConfig);
		}
		else
		{
			delete pxConfig;
		}

		xFilename = Tools_FileSystem::GetNextLoadFilename();
	}

	if(s_xQueue.size())
	{
		CloseOverlay();
	}
}

bool Tools_CutsceneModeAVI::LoadConfig(Tools_CutsceneModeAVI_Config* pxConfig,const char* szFilename)
{
	//make sure the ini file can be selected
	Tools_IniFile xIniFile(szFilename);
	if(!xIniFile.LoadIniFile())
	{
		Tools_Timed_Message_System::AddMessage(0xffffff,"Failed to load %s",szFilename);
		return false;
	}

	//and that it has a settings section
	if(!xIniFile.GetIniTextSection("Settings"))
	{
		Tools_Timed_Message_System::AddMessage(0xffffff,"Failed to find [Settings] in %s",szFilename);
		return false;
	}

	//reset the config ,and load what we can
	pxConfig->Reset();

	xIniFile.GetStringInCurrentSection("Cutscene",pxConfig->m_xCutsceneName);
	xIniFile.GetStringInCurrentSection("Filename",pxConfig->m_xFilename);
	xIniFile.GetBoolInCurrentSection("Write Bmps",pxConfig->m_bWriteBmps);

	xIniFile.GetIntInCurrentSection("Video Mode Width",pxConfig->m_iVideoModeWidth);
	xIniFile.GetIntInCurrentSection("Video Mode Height",pxConfig->m_iVideoModeHeight);
	xIniFile.GetIntInCurrentSection("AVI Width",pxConfig->m_iAVIWidth);
	xIniFile.GetIntInCurrentSection("AVI Height",pxConfig->m_iAVIHeight);
	xIniFile.GetBoolInCurrentSection("Half Size",pxConfig->m_bHalfSize);
	xIniFile.GetFloatInCurrentSection("Frame Rate",pxConfig->m_fFrameRate);

	xIniFile.GetBoolInCurrentSection("Disable PFX",pxConfig->m_bDisablePFX);

	int iTemp;

	xIniFile.GetBoolInCurrentSection("Use Env Fog",pxConfig->m_bUseEnvFog);
	if(xIniFile.GetIntInCurrentSection("Env Fog Red",iTemp))
	{
		pxConfig->m_xEnvFog.m_fR = iTemp/255.0f;
	}
	if(xIniFile.GetIntInCurrentSection("Env Fog Green",iTemp))
	{
		pxConfig->m_xEnvFog.m_fG = iTemp/255.0f;
	}
	if(xIniFile.GetIntInCurrentSection("Env Fog Blue",iTemp))
	{
		pxConfig->m_xEnvFog.m_fB = iTemp/255.0f;
	}

	xIniFile.GetBoolInCurrentSection("Use Actor Fog",pxConfig->m_bUseActorFog);
	if(xIniFile.GetIntInCurrentSection("Actor Fog Red",iTemp))
	{
		pxConfig->m_xActorFog.m_fR = iTemp/255.0f;
	}
	if(xIniFile.GetIntInCurrentSection("Actor Fog Green",iTemp))
	{
		pxConfig->m_xActorFog.m_fG = iTemp/255.0f;
	}
	if(xIniFile.GetIntInCurrentSection("Actor Fog Blue",iTemp))
	{
		pxConfig->m_xActorFog.m_fB = iTemp/255.0f;
	}

	//see if there is a list of actors to hide
	if(xIniFile.GetIniTextSection("Actors"))
	{
		while(xIniFile.GetNextIniTextLine())
		{
			Asura_String xActorName;
			if(xIniFile.GetString(xActorName))
			{
				pxConfig->m_xHideActorList.add_entry(xActorName);
			}
		}
	}

	return true;
}

bool Tools_CutsceneModeAVI::IsRecording()
{
	return s_pxCurrentAVI && TheAviWriter.IsWritingFile();
}

bool Tools_CutsceneModeAVI::IsDisablingPFX()
{
	if(!s_pxCurrentAVI) return false;
	return s_pxCurrentAVI->m_bDisablePFX;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////
void Tools_CutsceneModeAVI_Config::Reset()
{
	m_xFilename.MakeNull();

	m_bWriteBmps = false;

	m_iVideoModeWidth = Asura_Viewport::GetWidth(); 
	m_iVideoModeHeight = Asura_Viewport::GetHeight(); 

	m_iAVIWidth = m_iVideoModeWidth;
	m_iAVIHeight = m_iVideoModeHeight;

	m_bHalfSize = false;

	m_fFrameRate = 25.0f;

	m_bUseEnvFog = false;
	m_xEnvFog.m_fR = 0.0f;
	m_xEnvFog.m_fG = 0.0f;
	m_xEnvFog.m_fB = 0.0f;
	m_xEnvFog.m_fA = 1.0f;

	m_bUseActorFog = false;
	m_xActorFog.m_fR = 1.0f;
	m_xActorFog.m_fG = 1.0f;
	m_xActorFog.m_fB = 1.0f;
	m_xActorFog.m_fA = 1.0f;

	m_bDisablePFX = false;

	while(m_xHideActorList.size())
	{
		m_xHideActorList.delete_first_entry();
	}
}

void Tools_CutsceneModeAVI_Config::SetCutscene(const char* szCutsceneName)
{
	if(szCutsceneName && !m_xCutsceneName.IsBlank())
	{
		if(!m_xCutsceneName.Compare(szCutsceneName)) return;//name hasn't changed
	}
	
	m_xCutsceneName = szCutsceneName;

	//clear out list of actors if cutscene is changing
	while(m_xHideActorList.size())
	{
		m_xHideActorList.delete_first_entry();
	}
}