//
// Tools_Cutscene_Attachment.h
//
// Created by Richard Rice - 21/10/02
// 
// Copyright (C) 2002 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef __TOOLS_CUTSCENE_ATTACHMENT_H__
#define __TOOLS_CUTSCENE_ATTACHMENT_H__

#include "Core/Asura_Maths.h"
#include "Core/Asura_String.h"
#include "Cutscene/Asura_Cutscene_Event.h"
#include "Cutscene/Asura_Cutscene_Attachment.h"

#include "Objects/Tools_Object_System.h"

class Asura_Cutscene_Attachment;
class Tools_CutsceneActor_Object;

class Tools_Cutscene_Attachment
{

public:

	static Tools_Cutscene_Attachment* AllocateToolsAttachment(Asura_Cutscene_Attachment* pxAsuraAttachment,Tools_CutsceneActor_Object* pxToolsActor);

	Tools_Cutscene_Attachment(Asura_Cutscene_Attachment* pxAsuraAttachment,Tools_CutsceneActor_Object* pxToolsActor);
	virtual ~Tools_Cutscene_Attachment();

	virtual void Destroy();
	void OnSelection();
	void OnDeselection();
	
	virtual void SetSkin(const Asura_Hash_ID uSkinHash) {};

	void CreateOverlay();
	void DeleteOverlay();
	virtual void AddOverlayControls(class Asura_Overlay*,Asura_Vector_2& xOffset){};	
	virtual void UpdateOverlay(){};

	virtual void UpdateCutsceneEdit();

	static void DestroyAttachment();

	static void _SelectMarkerPoint(){s_pxThis->SelectMarkerPoint();}
	void SelectMarkerPoint();

	static void _EditName(){s_pxThis->EditName();}
	void EditName();

	void UpdateToolsName();

	const char* GetName(){return m_xToolsName.GetString();}
	u_int GetID(){return m_pxAsuraAttachment->GetID();}

	const char* GetAttachmentOrMarkerName() const {return m_pxAsuraAttachment->m_xName.IsBlank() ? m_pxAsuraAttachment->m_xMarkerName.GetString() : m_pxAsuraAttachment->m_xName.GetString();}

	enum DialogID
	{
		DialogID_SelectTexture,
	};


	Asura_Cutscene_Attachment* m_pxAsuraAttachment;
	Tools_CutsceneActor_Object* m_pxToolsActor;
	bool m_bChanged;
	Asura_String m_xToolsName;

	//static data for overlay
	static Asura_Vector_2 s_xOverlayPosition;
	static Tools_Cutscene_Attachment* s_pxThis;

private:

	static Tools_Cutscene_Attachment* Project_AllocateToolsAttachment( Asura_Cutscene_Attachment* pxAsuraAttachment,
                                                                       Tools_CutsceneActor_Object* pxToolsActorObj );

};


class Tools_Cutscene_Attachment_Hierarchy : public Tools_Cutscene_Attachment
{
public :	
	Tools_Cutscene_Attachment_Hierarchy(Asura_Cutscene_Attachment* pxAsuraAttachment,Tools_CutsceneActor_Object* pxToolsActor);

	virtual void AddOverlayControls(class Asura_Overlay*,Asura_Vector_2& xOffset);	

	void SetSkin( const Asura_Hash_ID uSkinHash, int iSkinIndex=0 );

protected :
	Asura_String m_xCurrentDir;
	
	const char* GetSkinName(int iSkinIndex);
	const char* GetAnimName();

	void ChooseSkin(int iSkinIndex);
	void ChooseAnim();
	bool IsEnabled_ChooseSkin(int iSkinIndex);

	static void _ChooseSkin(int iSkinIndex){((Tools_Cutscene_Attachment_Hierarchy*)s_pxThis)->ChooseSkin(iSkinIndex);}
	static void _ChooseAnim(){((Tools_Cutscene_Attachment_Hierarchy*)s_pxThis)->ChooseAnim();}

	static bool _IsEnabled_ChooseSkin(int iSkinIndex) {return ((Tools_Cutscene_Attachment_Hierarchy*)s_pxThis)->IsEnabled_ChooseSkin(iSkinIndex);}
};


class Tools_Cutscene_Attachment_Shape : public Tools_Cutscene_Attachment
{
public :	
	Tools_Cutscene_Attachment_Shape(Asura_Cutscene_Attachment* pxAsuraAttachment,Tools_CutsceneActor_Object* pxToolsActor);

	virtual void Destroy();

	virtual void AddOverlayControls(class Asura_Overlay*,Asura_Vector_2& xOffset);	

protected :
	void ChooseShape();

	static void _ChooseShape(){((Tools_Cutscene_Attachment_Shape*)s_pxThis)->ChooseShape();}

};

class Tools_Cutscene_Attachment_Actor : public Tools_Cutscene_Attachment
{
public :
	Tools_Cutscene_Attachment_Actor(Asura_Cutscene_Attachment* pxAsuraAttachment,Tools_CutsceneActor_Object* pxToolsActor);

	virtual void AddOverlayControls(class Asura_Overlay*,Asura_Vector_2& xOffset);

protected :
	static void _SelectActor(){((Tools_Cutscene_Attachment_Actor*)s_pxThis)->SelectActor();}
	void SelectActor();

	
};

class Tools_Cutscene_Attachment_Entity : public Tools_Cutscene_Attachment
{
public :
	Tools_Cutscene_Attachment_Entity(Asura_Cutscene_Attachment* pxAsuraAttachment,Tools_CutsceneActor_Object* pxToolsActor);
	~Tools_Cutscene_Attachment_Entity();

	virtual void AddOverlayControls(class Asura_Overlay*,Asura_Vector_2& xOffset);


protected :
	static void _SelectEntity(){((Tools_Cutscene_Attachment_Entity*)s_pxThis)->SelectEntity();}
	void SelectEntity();
	void SetEntity(Tools_Object* pxObject);

	class Tools_ChooseObject* m_pxChooseObject;

	friend class Tools_ChooseEntityForAttachment;
};


typedef	bool (*PROJECT_CAN_SELECT_ENTITY_FOR_ATTACHMENT_FN)(Tools_Object* pxObject);
class Tools_ChooseEntityForAttachment : public Tools_ChooseObject
{
public :
	Tools_ChooseEntityForAttachment(Tools_Cutscene_Attachment_Entity* pxAttachment){m_pxAttachment = pxAttachment;}

	virtual bool CanBeSelected(Tools_Object* pxObject);
	virtual void ObjectSelected(Tools_Object* pxObject);
	virtual void OnFinish(){m_pxAttachment->m_pxChooseObject = 0;delete this;}

	Tools_Cutscene_Attachment_Entity* m_pxAttachment;
	static void SetProjectSetEntityPositionFunction(PROJECT_CAN_SELECT_ENTITY_FOR_ATTACHMENT_FN pfnProjectFunction)
	{
		s_pfnProjectCanBeSelected = pfnProjectFunction;	
	}
private:
	static PROJECT_CAN_SELECT_ENTITY_FOR_ATTACHMENT_FN s_pfnProjectCanBeSelected;
};



class Tools_Cutscene_Attachment_Corona : public Tools_Cutscene_Attachment
{
public :
	Tools_Cutscene_Attachment_Corona(Asura_Cutscene_Attachment* pxAsuraAttachment,Tools_CutsceneActor_Object* pxToolsActor);
	~Tools_Cutscene_Attachment_Corona();

	virtual void AddOverlayControls(class Asura_Overlay*,Asura_Vector_2& xOffset);
	virtual void UpdateOverlay();


protected :

	static void _SelectTexture(){((Tools_Cutscene_Attachment_Corona*)s_pxThis)->SelectTexture();}
	void SelectTexture();
	
};

class Tools_Cutscene_Attachment_PFX : public Tools_Cutscene_Attachment
{
public :
	Tools_Cutscene_Attachment_PFX(Asura_Cutscene_Attachment* pxAsuraAttachment,Tools_CutsceneActor_Object* pxToolsActor);
	~Tools_Cutscene_Attachment_PFX();

	virtual void AddOverlayControls(class Asura_Overlay*,Asura_Vector_2& xOffset);

	
};

class Tools_Cutscene_Attachment_SoundEvent : public Tools_Cutscene_Attachment
{
public :
	Tools_Cutscene_Attachment_SoundEvent(Asura_Cutscene_Attachment* pxAsuraAttachment,Tools_CutsceneActor_Object* pxToolsActor);
	~Tools_Cutscene_Attachment_SoundEvent();

	virtual void AddOverlayControls(class Asura_Overlay*,Asura_Vector_2& xOffset);

	
};






#endif //__TOOLS_CUTSCENE_ATTACHMENT_H__
