//
// Tools_Cutscene_Dialog.h
//
// Created by Richard Rice - 12/04/2002
// 
// Copyright (C) 2001 Rebellion, All Rights Reserved.
//

#ifndef __TOOLS_CUTSCENE_DIALOG_H__
#define __TOOLS_CUTSCENE_DIALOG_H__

#include "Interface/Asura_Gadget.h"
#include "Interface/Asura_Dialog.h"

/////////////////////////////////////////////////////////////////////////////////////////////////
/*-------------------------------**
** Tools_CutsceneTimeline_Dialog **
**-------------------------------*/
//Special dialog control for getting the timeline display on an overlay.
//Only likely to be useable in the one place in cutscene mdoe

class Tools_CutsceneTimeline_Dialog : public Asura_Dialog
{
	public :
		Tools_CutsceneTimeline_Dialog( const Asura_Vector_2& xOffset );

		void UpdateFromCutsceneMode(float fBottomY);

		virtual void StartResize(u_int uResizeFlags);
		virtual void UpdateResize(u_int uResizeFlags, const Asura_Vector_2& xDelta) ;
		virtual u_int GetAllowedResizeFlags() const {return ASURA_OVERLAY_RESIZE_MINY;}

	private :

		float m_fPreResizeHeight;
		class Tools_CutsceneTimeline_Gadget* m_pxTimelineGadget;
		class Asura_Scrollbar_Gadget* m_pxVerticalScroll;

};

/////////////////////////////////////////////////////////////////////////////////////////////////
/*-------------------------------**
** Tools_CutsceneTimeline_Gadget **
**-------------------------------*/

class Tools_CutsceneTimeline_Gadget : public Asura_Gadget
{
	public:
		Tools_CutsceneTimeline_Gadget( const Asura_Vector_2& xOffset, float fWidth );

		virtual void Render( const Asura_Vector_2& xOrigin );
		virtual void Update( const Asura_Vector_2& xOrigin );
		virtual void FocusLost(int focusType);

		void UpdateFromCutsceneMode(float fBottomY);

};

/////////////////////////////////////////////////////////////////////////////////////////////////
/*---------------------------**
** Tools_CutsceneTime_Dialog **
**---------------------------*/

//Numerical gadget that takes time in seconds , but edits/displays the value in seconds or frames
//according to the current mode.

class Tools_CutsceneTime_Dialog : public Asura_Dialog
{
	public :
		Tools_CutsceneTime_Dialog(const char* pszName,const Asura_Vector_2& xOffset,float* pfTime,float fMin,float fMax,float fDelta,bool* pbChangedFlag);
		virtual void PerFrameUpdate();

	private :
		void CreateGadgets();

		float ConvertToDisplayedTime(float fValue);
		float ConvertToSeconds(float fValue);

		bool* m_pbChangedFlag;
		float* m_pfTime;
		float m_fTimeBackup;//backup copy of value stored in m_pfTime for detecting changes
		float m_fMin;
		float m_fMax;
		float m_fDelta;
		Asura_Vector_2 m_xGadgetOffset;

		//values to be used by the gadget
		float m_fDisplayedTime;
		bool m_bGadgetChanged;
		int m_iTimerMode;


};

/////////////////////////////////////////////////////////////////////////////////////////////////
/*---------------------------------**
** Tools_CutsceneTimeSlider_Dialog **
**---------------------------------*/

//Similarly for the time slider
class Tools_CutsceneTimeSlider_Dialog : public Asura_Dialog
{
	public :
		Tools_CutsceneTimeSlider_Dialog(const Asura_Vector_2& xOffset,float* pfTime,float* pfMin,float* pfMax,float fSlideWidth,bool* pbChangedFlag);
		virtual void PerFrameUpdate();

	private :
		void CreateGadgets();

		//float ConvertToDisplayedTime(float fValue);
		//float ConvertToSeconds(float fValue);

		bool* m_pbChangedFlag;
		float* m_pfTime;
		float* m_pfMin;
		float* m_pfMax;

		float m_fTimeBackup;//backup copy of value stored in m_pfTime for detecting changes
		float m_fMinBackup; // and for Min
		float m_fMaxBackup; // and for Max

		Asura_Vector_2 m_xOffset;
		float m_fSlideWidth;

		//values to be used by the gadget
		float m_fDisplayedTime;
		float m_fDisplayedMin;
		float m_fDisplayedMax;
		bool m_bGadgetChanged;
		int m_iTimerMode;


};





#endif __TOOLS_CUTSCENE_DIALOG_H__
