#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef _TOOLS_CUTSCENE_EVENT_H_
#define _TOOLS_CUTSCENE_EVENT_H_

#include "Tools_Undo.h"
#include "Cutscene/Asura_Cutscene_Event.h"
#include "Objects/Tools_Object.h"
#include "Misc/list_tem.hpp"
#include "Interface/Asura_Dialog.h"
#include "GUIMenu/Asura_GUIMenu_CommandConsole.h"

class Asura_Cutscene_Actor;
class Asura_Cutscene_Event;
class Tools_Cutscene_Event;
class Tools_Cutscene_Object;
class Tools_Timeline_Data_Event;

//////////////////////////////////////////////////////////////////////////////////////////////////
/*----------------------------**
** Tools_CutsceneEvent_Object **
**----------------------------*/

class Tools_CutsceneEvent_Object : public Tools_Object
{
	public :
		Tools_CutsceneEvent_Object(Tools_Cutscene_Event* event);
		~Tools_CutsceneEvent_Object();

		//virtual function overides
		void OnSelection();
		void OnDeselection();
		const char* GetName();
		const char* GetTypeDescription(){return "Event Object";}
		void Destroy();
		bool CanBeDestroyed();

		virtual void UpdatePositionInAsuraData(bool bUpdateLinkedObjects);

		Tools_Cutscene_Event* Event;
		Asura_String Name;

};

////////////////////////////////////////////////////////////////////////////////////////////
/*----------------------**
** Tools_Cutscene_Event **
**----------------------*/


class Tools_Cutscene_Event
{
public :
	Tools_Cutscene_Event(Asura_Cutscene_Event* event,Tools_Cutscene_Object* cutsceneObject,int eventID);
	virtual ~Tools_Cutscene_Event();

	virtual void Destroy();
	virtual void OnSelection();
	virtual void OnDeselection();
	void CreateOverlay();
    static void ChooseActor();
	static void ChooseAnimFlagActor();
	static void RemoveAnimFlagActor();
	static void SetAnimFlag();
	void DeleteOverlay();
	virtual const char* GetTypeDescription(){return "Unknown";}
	//gets shortened description for timeline headings
	virtual const char* GetAbbreviatedTypeDescription(){return GetTypeDescription();}
	virtual void Update();

	void UpdatePosition();

	virtual void AddOverlayControls(class Asura_Overlay* newOverlay,class Asura_Vector_2& offset){}

	virtual int GetTimelineColour(){return 0xffffffff;}
	virtual void FillInTimelineData(Tools_Timeline_Data_Event* data);
	virtual void SetNewTriggerTime(float fTriggerTime) { Event->m_fTriggerTime = fTriggerTime; }
	virtual Asura_String GetDescription(){return GetTypeDescription();}
	void PrintDescription(float /*x*/,float /*y*/);

	//For use by the timeline for changing the duration of events that allow it
	virtual bool CanEditDuration() const{return false;}
	virtual void SetTotalDuration(float fDuration){}

	Asura_String GetNameOrDescription();

	static void DestroyEvent();

	virtual void UpdateBackupVariables();
	//gets the name of one of the variables that differ from the backups
	virtual const char* GetNameOfChangedVariable();

	//updates the tools event ID members using the index numbers in the asura data
	virtual void UpdateEventIDsFromIndeces();
	//updates the event index numbers in the asura data from the ids stored in the tools data
	virtual void UpdateEventIndecesFromIDs();

    virtual bool ActorIsValidTarget( Asura_Cutscene_Actor* pxActor );
	virtual bool AllowNoTarget() const {return false;}
	virtual const char* GetNoTargetString() const {return "*None*";}
    virtual void SetTargetActor( Asura_Cutscene_Actor* pxActor );

	bool IsEditable() const;

	//for overidding the default event name
	static void _Rename(int iThis){((Tools_Cutscene_Event*)iThis)->Rename();}
	void Rename();

	//for setting a tools comment for the event
	static void _EditComment(int iThis){((Tools_Cutscene_Event*)iThis)->EditComment();}
	void EditComment();

	static void _CopyEvent(int iThis){((Tools_Cutscene_Event*)iThis)->CopyEvent();}
	void CopyEvent();

	//for drop down list of events
	static void UpdateEventListStrings();
	static void DeleteEventListStrings();
	static void EventListSelectionChanged();

	//Used on a newly created event to ask user for values that must be set
	//for event to be valid.
	virtual void AskForRequiredData(){}


	// member data
	Asura_Cutscene_Event* Event;
	Tools_Cutscene_Object* CutsceneObject;
	Tools_CutsceneEvent_Object* EventObject;//event object that this is currently attached to

	int Index;

	bool Changed;
	u_int m_uTimelineSelectionFlags;

	int EventID;//ID that remains valid while a given cutscene is being edited.. used by undo
	Asura_Vector_3 Position; // current location for this event to be drawn

	int ParentEventID;
	float RelativeTime;


	//backup variables for undo
	float TriggerTimeBackup;
	Asura_String TargetActorNameBackup;
	int ParentEventIDBackup;
	u_int m_uEventFlagsBackup;
	int m_iEnableIDBackup;

	Asura_String m_xNameBackup;//For overiding the name displayed in tools
	Asura_String m_xCommentBackup;//Optional comment for use in tools
	Asura_Vector_3 m_xPositionBackup;//For overiding the event object's position
	Asura_String m_xNameOfSourceTemplateBackup;//if this event belongs to a template, template event's name is here.
	Asura_String m_xNameOfEventWrapperBackup;//if this event belongs to a template, wrapper event name is here.


	//static data for overlay
	static Asura_Vector_2 OverlayPosition;
	static Tools_Cutscene_Event* ThisEvent;
	static int NextEventID;

	static int s_iEventListSize;
	static char** s_pszEventListStrings;
	static int* s_piEventListConversion;
	static int s_iEventListIndex;
	static bool s_bEventListForCurrentActorOnly;

	enum DialogID
	{
		DialogID_EventSelection,
		DialogID_EventList,
		DialogID_Actor,
		DialogID_Anim,
		DialogID_AnimFlagActor,
		DialogID_NearZ,
		DialogID_FarZ,
	};

};


////////////////////////////////////////////////////////////////////////////////////////////
/*--------------------------------------**
** Tools_Cutscene_Event_AnimationChange **
**--------------------------------------*/

class Tools_Cutscene_Event_AnimationChange : public Tools_Cutscene_Event
{
public :
	Tools_Cutscene_Event_AnimationChange(Asura_Cutscene_Event* event,Tools_Cutscene_Object* cutsceneObject,int eventID);

	const char* GetTypeDescription(){return "Animation Change";}

	void AddOverlayControls(class Asura_Overlay* newOverlay,class Asura_Vector_2& offset);
	int GetTimelineColour(){return 0xff707000;}
	void FillInTimelineData(Tools_Timeline_Data_Event* data);
	virtual Asura_String GetDescription();
	void Update();
    virtual bool ActorIsValidTarget(class Asura_Cutscene_Actor* pxActor);

	virtual bool CanEditDuration() const;
	virtual void SetTotalDuration(float fDuration);


	static void ChooseAnimation();

	static bool IsTargetHierarchy();
	static bool IsTargetCamera();

	virtual void UpdateBackupVariables();
	virtual const char* GetNameOfChangedVariable();

	virtual void AskForRequiredData();

	virtual bool UsesAnimatedTween() const {return false;}


	//backup variables for undo
	Asura_String AnimationNameBackup;
	Asura_String AnimationFileNameBackup;
	int AnimationFlagsBackup;
	float TweenDurationBackup;
	float m_fStartTimerBackup;
	float m_fEndTimerBackup;
	float m_fAnimDelayBackup;

};

////////////////////////////////////////////////////////////////////////////////////////////
/*-----------------------------------------**
** Tools_Cutscene_Event_SecondaryAnimation **
**-----------------------------------------*/

class Tools_Cutscene_Event_SecondaryAnimation : public Tools_Cutscene_Event_AnimationChange
{
public :
	Tools_Cutscene_Event_SecondaryAnimation(Asura_Cutscene_Event* event,Tools_Cutscene_Object* cutsceneObject,int eventID);

	const char* GetTypeDescription(){return "Secondary Animation";}

	void AddOverlayControls(class Asura_Overlay* newOverlay,class Asura_Vector_2& offset);
	void FillInTimelineData(Tools_Timeline_Data_Event* data);
	virtual Asura_String GetDescription();

	virtual bool CanEditDuration() const {return true;}
	virtual void SetTotalDuration(float fDuration);


    virtual bool ActorIsValidTarget(class Asura_Cutscene_Actor* pxActor)
	{return Tools_Cutscene_Event::ActorIsValidTarget(pxActor);}

	virtual void UpdateBackupVariables();
	virtual const char* GetNameOfChangedVariable();

	virtual bool UsesAnimatedTween() const {return true;}

	//backup variables for undo
	float WeightBackup;
	float DurationBackup;

};

////////////////////////////////////////////////////////////////////////////////////////////
/*-----------------------------------------**
** Tools_Cutscene_Event_TextureAnimation **
**-----------------------------------------*/

class Tools_Cutscene_Event_TextureAnimation : public Tools_Cutscene_Event
{
public :
	Tools_Cutscene_Event_TextureAnimation(Asura_Cutscene_Event* event,Tools_Cutscene_Object* cutsceneObject,int eventID);

	const char* GetTypeDescription(){return "Texture Animation";}
	virtual const char* GetAbbreviatedTypeDescription(){return "TxAnim";}

	void AddOverlayControls(class Asura_Overlay* newOverlay,class Asura_Vector_2& offset);
	int GetTimelineColour(){return 0xff60a060;}
    virtual bool ActorIsValidTarget(class Asura_Cutscene_Actor* pxActor);

	static bool IsTargetHierarchy();
	static bool IsEnabled_Sequence();
	static bool IsEnabled_Speed();


	virtual void UpdateBackupVariables();
	virtual const char* GetNameOfChangedVariable();

	virtual void AskForRequiredData();


	//backup variables for undo
	int m_iSequenceBackup;
	float m_fSpeedBackup;
	u_int m_uTextureAnimFlagsBackup;


};



////////////////////////////////////////////////////////////////////////////////////////////
/*--------------------------------**
** Tools_Cutscene_Event_TimeScale **
**--------------------------------*/

class Tools_Cutscene_Event_TimeScale : public Tools_Cutscene_Event
{
public :
	Tools_Cutscene_Event_TimeScale(Asura_Cutscene_Event* event,Tools_Cutscene_Object* cutsceneObject,int eventID);

	const char* GetTypeDescription(){return "Time Scale";}
	virtual const char* GetAbbreviatedTypeDescription(){return "TmScle";}

	void AddOverlayControls(class Asura_Overlay* newOverlay,class Asura_Vector_2& offset);
	virtual Asura_String GetDescription();

	virtual void UpdateBackupVariables();
	virtual const char* GetNameOfChangedVariable();

	int GetTimelineColour(){return 0xff608060;}

	//backup variables for undo
	float TimeScaleBackup;

};

////////////////////////////////////////////////////////////////////////////////////////////
/*----------------------------------**
** Tools_Cutscene_Event_FieldOfView **
**----------------------------------*/

class Tools_Cutscene_Event_FieldOfView : public Tools_Cutscene_Event
{
public :
	Tools_Cutscene_Event_FieldOfView(Asura_Cutscene_Event* event,Tools_Cutscene_Object* cutsceneObject,int eventID);

	const char* GetTypeDescription(){return "Change Field Of View";}
	virtual const char* GetAbbreviatedTypeDescription(){return "FOV";}

	void AddOverlayControls(class Asura_Overlay* newOverlay,class Asura_Vector_2& offset);
	virtual Asura_String GetDescription();
	void FillInTimelineData(Tools_Timeline_Data_Event* data);
	int GetTimelineColour(){return 0xff6060a0;}

	virtual bool CanEditDuration() const{return true;}
	virtual void SetTotalDuration(float fDuration);

	virtual void UpdateBackupVariables();
	virtual const char* GetNameOfChangedVariable();

	virtual void AskForRequiredData();
    virtual bool ActorIsValidTarget(class Asura_Cutscene_Actor* pxActor);
	virtual bool AllowNoTarget() const {return true;}
	virtual const char* GetNoTargetString() const {return "*All*";}

	//backup variabls for undo
	float DurationBackup;//time over which the change happens
	float FinalFOVBackup;//final field of view (in degrees)

};

////////////////////////////////////////////////////////////////////////////////////////////
/*------------------------------**
** Tools_Cutscene_Event_Trigger **
**------------------------------*/

class Tools_Cutscene_Event_Trigger : public Tools_Cutscene_Event
{
public :
	Tools_Cutscene_Event_Trigger(Asura_Cutscene_Event* event,Tools_Cutscene_Object* cutsceneObject,int eventID);

	const char* GetTypeDescription(){return "Trigger";}
	virtual const char* GetAbbreviatedTypeDescription(){return "Trig";}

	void AddOverlayControls(class Asura_Overlay* newOverlay,class Asura_Vector_2& offset);
	virtual Asura_String GetDescription();
	int GetTimelineColour(){return 0xffa060a0;}

	static void EditMessageBlock();


};

////////////////////////////////////////////////////////////////////////////////////////////
/*------------------------------**
** Tools_Cutscene_Event_LoopBack**
**------------------------------*/

class Tools_Cutscene_Event_LoopBack : public Tools_Cutscene_Event
{
public :
	Tools_Cutscene_Event_LoopBack(Asura_Cutscene_Event* event,Tools_Cutscene_Object* cutsceneObject,int eventID);

	const char* GetTypeDescription(){return "Loop Back";}
	virtual const char* GetAbbreviatedTypeDescription(){return "LoopBk";}

	virtual void AddOverlayControls(class Asura_Overlay* newOverlay,class Asura_Vector_2& offset);
	virtual void FillInTimelineData(Tools_Timeline_Data_Event* data);
	virtual Asura_String GetDescription();
	virtual int GetTimelineColour(){return 0xffa00000;}

	virtual void UpdateBackupVariables();
	virtual const char* GetNameOfChangedVariable();
	virtual void UpdateEventIDsFromIndeces();
	virtual void UpdateEventIndecesFromIDs();

	static void ChooseEvent();
	static bool ChooseEventFilterFunction(Tools_Cutscene_Event* pxEvent,int iData);


	int m_iLoopBackEventID;

	//backup variables for undo
    float m_fSecondsToLoopBack;
	int m_iLoopBackEventIDBackup;

};

////////////////////////////////////////////////////////////////////////////////////////////
/*---------------------------------**
** Tools_Cutscene_Event_Attachment **
**---------------------------------*/

class Tools_Cutscene_Event_Attachment : public Tools_Cutscene_Event
{
public :
	Tools_Cutscene_Event_Attachment(Asura_Cutscene_Event* pxEvent,Tools_Cutscene_Object* pxCutsceneObject,int iEventID);
	const char* GetTypeDescription();
	virtual const char* GetAbbreviatedTypeDescription();

	virtual void AddOverlayControls(class Asura_Overlay* newOverlay,class Asura_Vector_2& offset);
	virtual void FillInTimelineData(Tools_Timeline_Data_Event* data);
	virtual Asura_String GetDescription();
	virtual int GetTimelineColour(){return 0xff007f7f;}

	virtual void UpdateBackupVariables();
	virtual const char* GetNameOfChangedVariable();

	virtual void AskForRequiredData();


	static void _SelectAttachment(){((Tools_Cutscene_Event_Attachment*)ThisEvent)->SelectAttachment();}
	void SelectAttachment();
	class Tools_Cutscene_Attachment* GetToolsAttachment();


	//backup variables for undo
	u_int m_uAttachmentIDBackup;

};

////////////////////////////////////////////////////////////////////////////////////////////
/*----------------------------------------**
** Tools_Cutscene_Event_AttachmentControl **
**----------------------------------------*/

class Tools_Cutscene_Event_AttachmentControl : public Tools_Cutscene_Event
{
public :
	Tools_Cutscene_Event_AttachmentControl(Asura_Cutscene_Event* pxEvent,Tools_Cutscene_Object* pxCutsceneObject,int iEventID);
	~Tools_Cutscene_Event_AttachmentControl();
	const char* GetTypeDescription() {return "Attachment Control";}
	virtual const char* GetAbbreviatedTypeDescription(){return "AttCon";}

	virtual void AddOverlayControls(class Asura_Overlay* newOverlay,class Asura_Vector_2& offset);
	virtual void FillInTimelineData(Tools_Timeline_Data_Event* data);
	virtual Asura_String GetDescription();
	virtual int GetTimelineColour(){return 0xff007f7f;}

	virtual void UpdateBackupVariables();
	virtual const char* GetNameOfChangedVariable();

	virtual void AskForRequiredData();


	static void _AddAttachment(){((Tools_Cutscene_Event_AttachmentControl*)ThisEvent)->AddAttachment();}
	void AddAttachment();
	const char* GetAttachmentNameByIndex(u_int uIndex) const;


	//backup variables for undo
	u_int m_uNumAttachmentsBackup;
	Asura_Cutscene_Event_AttachmentControl_Attachment* m_pxAttachmentsBackup;

};



////////////////////////////////////////////////////////////////////////////////////////////
/*---------------------------**
** Tools_Cutscene_Event_Beam **
**---------------------------*/

class Tools_Cutscene_Event_Beam : public Tools_Cutscene_Event
{
public :
	Tools_Cutscene_Event_Beam(Asura_Cutscene_Event* pxEvent,Tools_Cutscene_Object* pxCutsceneObject,int iEventID);
	~Tools_Cutscene_Event_Beam();

	const char* GetTypeDescription(){return "Beam";}
	virtual Asura_String GetDescription(){return "Beam";}
	virtual void AddOverlayControls(class Asura_Overlay* pxOverlay,class Asura_Vector_2& xOffset);

	virtual int GetTimelineColour(){return 0xffaf3f3f;}
	virtual void FillInTimelineData(Tools_Timeline_Data_Event* pxData);

	virtual bool CanEditDuration() const{return true;}
	virtual void SetTotalDuration(float fDuration);

	virtual void UpdateBackupVariables();

	static void _EditTailPosition(){((Tools_Cutscene_Event_Beam*)ThisEvent)->EditTailPosition();}
	void EditTailPosition();

	static void _EditHeadPosition(){((Tools_Cutscene_Event_Beam*)ThisEvent)->EditHeadPosition();}
	void EditHeadPosition();

	static void _EditTexture(){((Tools_Cutscene_Event_Beam*)ThisEvent)->EditTexture();}
	void EditTexture();

	unsigned m_uTailPositionEditID;
	unsigned m_uHeadPositionEditID;
	
	//backup variables for undo
	Asura_Cutscene_BeamParameters m_xParamBackup;
};

////////////////////////////////////////////////////////////////////////////////////////////
/*---------------------------**
** Tools_Cutscene_Event_Fade **
**---------------------------*/

class Tools_Cutscene_Event_Fade : public Tools_Cutscene_Event
{
public :
	Tools_Cutscene_Event_Fade(Asura_Cutscene_Event* pxEvent,Tools_Cutscene_Object* pxCutsceneObject,int iEventID);

	const char* GetTypeDescription(){return "Fade In/Out";}
	virtual const char* GetAbbreviatedTypeDescription(){return "Fade";}

	void AddOverlayControls(class Asura_Overlay* pxOverlay,class Asura_Vector_2& xOffset);
	virtual Asura_String GetDescription();
	void FillInTimelineData(Tools_Timeline_Data_Event* pxData);
	int GetTimelineColour(){return 0xff4f4f4f;}

	virtual bool CanEditDuration() const{return true;}
	virtual void SetTotalDuration(float fDuration);

	virtual void UpdateBackupVariables();
	virtual const char* GetNameOfChangedVariable();

	//backup variables for undo
	float m_fRedBackup;
	float m_fGreenBackup;
	float m_fBlueBackup;
	float m_fDurationBackup;
	u_int m_uFlagsBackup;

};

////////////////////////////////////////////////////////////////////////////////////////////
/*--------------------------------------**
** Tools_Cutscene_Event_TemplateWrapper **
**--------------------------------------*/

class Tools_Cutscene_Event_TemplateWrapper : public Tools_Cutscene_Event
{
	friend class CutsceneUndoData_Event_Changed_TemplateWrapper;

public :
	Tools_Cutscene_Event_TemplateWrapper(Asura_Cutscene_Event* pxEvent,Tools_Cutscene_Object* pxCutsceneObject,int iEventID);

	// virtual function overrides
	const char* GetTypeDescription(){return "Event Template Wrapper";}
	void AddOverlayControls(class Asura_Overlay* pxOverlay,class Asura_Vector_2& xOffset);
	int GetTimelineColour(){return 0xff4f4faf;}
	void FillInTimelineData(Tools_Timeline_Data_Event* data);
	void SetNewTriggerTime(float fTriggerTime);
	void Destroy();
	void Update();

	// non-virtual member functions
	static void UnwrapEvents_Static(int iThis) { reinterpret_cast<Tools_Cutscene_Event_TemplateWrapper*>(iThis)->UnwrapEvents(); }
	void UnwrapEvents();

private:

	void UpdateWrappedEventTriggerTimes();
	float m_fTimeOffset;
};


////////////////////////////////////////////////////////////////////////////////////////////
/*-------------------------------**
** Tools_Cutscene_Event_Subtitle **
**------------------------------*/

class Tools_Cutscene_Event_Subtitle : public Tools_Cutscene_Event
{
public :
	Tools_Cutscene_Event_Subtitle(Asura_Cutscene_Event* pxEvent,Tools_Cutscene_Object* pxCutsceneObject,int iEventID);

	const char* GetTypeDescription(){return "Subtitle";}
	virtual const char* GetAbbreviatedTypeDescription(){return "Subtl";}

	void AddOverlayControls(class Asura_Overlay* pxOverlay,class Asura_Vector_2& xOffset);
	virtual Asura_String GetDescription();
	void FillInTimelineData(Tools_Timeline_Data_Event* pxData);
	virtual int GetTimelineColour(){return 0xff3f7f00;}

	virtual bool CanEditDuration() const{return true;}
	virtual void SetTotalDuration(float fDuration);

	virtual void AskForRequiredData();

	static void _SelectSubtitle(){((Tools_Cutscene_Event_Subtitle*)ThisEvent)->SelectSubtitle();}
	void SelectSubtitle();

	static void _SetSubtitleEnd(){((Tools_Cutscene_Event_Subtitle*)ThisEvent)->SetSubtitleEnd();}
	void SetSubtitleEnd();


	virtual void UpdateBackupVariables();
	virtual const char* GetNameOfChangedVariable();

	//backup variables for undo
	float m_fDurationBackup;
	Asura_Hash_ID m_uTextHashBackup;

};

////////////////////////////////////////////////////////////////////////////////////////////
/*----------------------------**
** Tools_Cutscene_Event_Sound **
**----------------------------*/

class Tools_Cutscene_Event_Sound : public Tools_Cutscene_Event
{
public :
	Tools_Cutscene_Event_Sound(Asura_Cutscene_Event* pxEvent,Tools_Cutscene_Object* pxCutsceneObject,int iEventID);

	const char* GetTypeDescription(){return "Tools Test Sound";}
	virtual const char* GetAbbreviatedTypeDescription(){return "Sound";}

	void AddOverlayControls(class Asura_Overlay* pxOverlay,class Asura_Vector_2& xOffset);
	virtual Asura_String GetDescription();
	virtual int GetTimelineColour(){return 0xffaf4faf;}

	virtual void AskForRequiredData();

	static void _SelectSound(){((Tools_Cutscene_Event_Sound*)ThisEvent)->SelectSound();}
	void SelectSound();


	virtual void UpdateBackupVariables();
	virtual const char* GetNameOfChangedVariable();

	//backup variables for undo
	Asura_String m_xSoundNameBackup;

};

////////////////////////////////////////////////////////////////////////////////////////////
/*----------------------------------**
** Tools_Cutscene_Event_ChangeCamera **
**-----------------------------------*/

class Tools_Cutscene_Event_ChangeCamera : public Tools_Cutscene_Event
{
public :
	Tools_Cutscene_Event_ChangeCamera(Asura_Cutscene_Event* pxEvent,Tools_Cutscene_Object* pxCutsceneObject,int iEventID);

	const char* GetTypeDescription(){return "Change Camera";}
	virtual const char* GetAbbreviatedTypeDescription(){return "Cmra";}

	void AddOverlayControls(class Asura_Overlay* pxOverlay,class Asura_Vector_2& xOffset);
	virtual Asura_String GetDescription();
	virtual int GetTimelineColour(){return 0xffafafaf;}

	virtual void AskForRequiredData();

    virtual bool ActorIsValidTarget(class Asura_Cutscene_Actor* pxActor);


};

////////////////////////////////////////////////////////////////////////////////////////////
/*-----------------------------------------**
** Tools_Cutscene_Event_ChangeCameraTarget **
**-----------------------------------------*/

class Tools_Cutscene_Event_ChangeCameraTarget : public Tools_Cutscene_Event
{
public :
	Tools_Cutscene_Event_ChangeCameraTarget(Asura_Cutscene_Event* pxEvent,Tools_Cutscene_Object* pxCutsceneObject,int iEventID);

	const char* GetTypeDescription(){return "Change Camera Target";}
	virtual const char* GetAbbreviatedTypeDescription(){return "CmTa";}

	void AddOverlayControls(class Asura_Overlay* pxOverlay,class Asura_Vector_2& xOffset);
	virtual Asura_String GetDescription();
	virtual int GetTimelineColour(){return 0xffaf7f3f;}

	virtual void AskForRequiredData();

    virtual bool ActorIsValidTarget(class Asura_Cutscene_Actor* pxActor);

	static void _SelectCameraTarget(){((Tools_Cutscene_Event_ChangeCameraTarget*)ThisEvent)->SelectCameraTarget();}
	void SelectCameraTarget();


	virtual void UpdateBackupVariables();
	virtual const char* GetNameOfChangedVariable();

	//backup variables for undo
	Asura_String m_xCameraTargetNameBackup;
};

////////////////////////////////////////////////////////////////////////////////////////////
/*----------------------------------**
** Tools_Cutscene_Event_Hide **
**-----------------------------------*/

class Tools_Cutscene_Event_Hide : public Tools_Cutscene_Event
{
public :
	Tools_Cutscene_Event_Hide(Asura_Cutscene_Event* pxEvent,Tools_Cutscene_Object* pxCutsceneObject,int iEventID);

	const char* GetTypeDescription(){return "Hide";}
	virtual const char* GetAbbreviatedTypeDescription(){return "Hide";}

	void AddOverlayControls(class Asura_Overlay* pxOverlay,class Asura_Vector_2& xOffset);
	virtual Asura_String GetDescription();
	virtual int GetTimelineColour(){return 0xff303030;}

	virtual void AskForRequiredData();

	virtual bool ActorIsValidTarget(class Asura_Cutscene_Actor* pxActor);
};

////////////////////////////////////////////////////////////////////////////////////////////
/*----------------------------------**
** Tools_Cutscene_Event_Unhide **
**-----------------------------------*/

class Tools_Cutscene_Event_Unhide : public Tools_Cutscene_Event
{
public :
	Tools_Cutscene_Event_Unhide(Asura_Cutscene_Event* pxEvent,Tools_Cutscene_Object* pxCutsceneObject,int iEventID);

	const char* GetTypeDescription(){return "Unhide";}
	virtual const char* GetAbbreviatedTypeDescription(){return "UnHd";}

	void AddOverlayControls(class Asura_Overlay* pxOverlay,class Asura_Vector_2& xOffset);
	virtual Asura_String GetDescription();
	virtual int GetTimelineColour(){return 0xffC0C0C0;}

	virtual void AskForRequiredData();

	virtual bool ActorIsValidTarget(class Asura_Cutscene_Actor* pxActor);
};

////////////////////////////////////////////////////////////////////////////////////////////
/*-----------------------------------------**
** Tools_Cutscene_Event_ChangeCameraShake **
**-----------------------------------------*/

class Tools_Cutscene_Event_ChangeCameraShake : public Tools_Cutscene_Event
{
public :
	Tools_Cutscene_Event_ChangeCameraShake(Asura_Cutscene_Event* pxEvent,Tools_Cutscene_Object* pxCutsceneObject,int iEventID);

	const char* GetTypeDescription(){return "Change Camera Shake";}
	virtual const char* GetAbbreviatedTypeDescription(){return "CmSh";}

	void AddOverlayControls(class Asura_Overlay* pxOverlay,class Asura_Vector_2& xOffset);
	virtual Asura_String GetDescription();
	virtual void FillInTimelineData(Tools_Timeline_Data_Event* pxData);
	virtual int GetTimelineColour(){return 0xffc03030;}

	virtual bool CanEditDuration() const{return true;}
	virtual void SetTotalDuration(float fDuration);

	virtual void AskForRequiredData();

	virtual bool ActorIsValidTarget(class Asura_Cutscene_Actor* pxActor);

	virtual void UpdateBackupVariables();
	virtual const char* GetNameOfChangedVariable();

	bool IsShakeEnabled();
	void SetShakeEnabled(bool bShakeEnabled);

	static bool _IsShakeEnabled();
	static void _SetShakeEnabled(bool bShakeEnabled);

	//backup variables for undo
	Asura_Cutscene_Camera_Shake m_xShakeBackup;
	float m_fDurationBackup;
};


////////////////////////////////////////////////////////////////////////////////////////////
/*-----------------------------------------**
** Tools_Cutscene_Event_SetAlpha **
**-----------------------------------------*/

class Tools_Cutscene_Event_SetAlpha : public Tools_Cutscene_Event
{
public :
	Tools_Cutscene_Event_SetAlpha(Asura_Cutscene_Event* pxEvent,Tools_Cutscene_Object* pxCutsceneObject,int iEventID);

	const char* GetTypeDescription(){return "SetAlpha";}
	virtual const char* GetAbbreviatedTypeDescription(){return "Alph";}

	void AddOverlayControls(class Asura_Overlay* pxOverlay,class Asura_Vector_2& xOffset);
	virtual Asura_String GetDescription();
	virtual void FillInTimelineData(Tools_Timeline_Data_Event* pxData);
	virtual int GetTimelineColour(){return 0xffC0C030;}

	virtual bool CanEditDuration() const{return true;}
	virtual void SetTotalDuration(float fDuration);

	virtual void AskForRequiredData();
	virtual bool ActorIsValidTarget(class Asura_Cutscene_Actor* pxActor);

	virtual void UpdateBackupVariables();
	virtual const char* GetNameOfChangedVariable();

	//backup variables for undo
	Asura_Graph<float> m_xAlphaBackup;
	float m_fDurationBackup;
};

////////////////////////////////////////////////////////////////////////////////////////////
/*------------------------------------------------**
** Tools_Cutscene_Event_ChangePlayerCameraControl **
**------------------------------------------------*/

class Tools_Cutscene_Event_ChangePlayerCameraControl : public Tools_Cutscene_Event
{
public :
	Tools_Cutscene_Event_ChangePlayerCameraControl(Asura_Cutscene_Event* pxEvent,Tools_Cutscene_Object* pxCutsceneObject,int iEventID);

	const char* GetTypeDescription(){return "Change Player Camera Control";}
	virtual const char* GetAbbreviatedTypeDescription(){return "CPCC";}

	void AddOverlayControls(class Asura_Overlay* pxOverlay,class Asura_Vector_2& xOffset);
	virtual Asura_String GetDescription();
	virtual void FillInTimelineData(Tools_Timeline_Data_Event* pxData);
	virtual int GetTimelineColour(){return 0xff00C000;}

	virtual bool CanEditDuration() const{return true;}
	virtual void SetTotalDuration(float fDuration);

	virtual void AskForRequiredData();

	virtual bool ActorIsValidTarget(class Asura_Cutscene_Actor* pxActor);

	virtual void UpdateBackupVariables();
	virtual const char* GetNameOfChangedVariable();

	//backup variables for undo
	float m_fPitchBackup;
	float m_fYawBackup;
	float m_fDurationBackup;
};


////////////////////////////////////////////////////////////////////////////////////////////
/*--------------------------------------**
** Tools_Cutscene_Event_FacialAnimation **
**--------------------------------------*/

#ifdef ASURA_USE_FACIAL_ANIMATION_SYSTEM
class Tools_Cutscene_Event_FacialAnimation : public Tools_Cutscene_Event
{
public :
	Tools_Cutscene_Event_FacialAnimation(Asura_Cutscene_Event* pxEvent,Tools_Cutscene_Object* pxCutsceneObject,int iEventID);

	const char* GetTypeDescription(){return "Facial Animation";}
	virtual const char* GetAbbreviatedTypeDescription(){return "FaAn";}

	void AddOverlayControls(class Asura_Overlay* pxOverlay,class Asura_Vector_2& xOffset);
	virtual Asura_String GetDescription();
	void FillInTimelineData(Tools_Timeline_Data_Event* pxData);
	int GetTimelineColour(){return 0xff407040;}

	virtual void AskForRequiredData();


	virtual void UpdateBackupVariables();
	virtual const char* GetNameOfChangedVariable();

	//backup variables for undo
	Asura_Hash_ID m_uAnimIDBackup;
	u_int m_uAnimationFlagsBackup;
};
#endif

////////////////////////////////////////////////////////////////////////////////////////////
/*----------------------------------**
** Tools_Cutscene_Event_MixerPreset **
**----------------------------------*/

class Tools_Cutscene_Event_MixerPreset : public Tools_Cutscene_Event
{
public :
	Tools_Cutscene_Event_MixerPreset(Asura_Cutscene_Event* pxEvent,Tools_Cutscene_Object* pxCutsceneObject,int iEventID);

	const char* GetTypeDescription(){return "Mixer Preset";}
	virtual const char* GetAbbreviatedTypeDescription(){return "MiPr";}

	void AddOverlayControls(class Asura_Overlay* pxOverlay,class Asura_Vector_2& xOffset);
	virtual Asura_String GetDescription();
	int GetTimelineColour(){return 0xff404070;}


	virtual void UpdateBackupVariables();
	virtual const char* GetNameOfChangedVariable();

	//backup variables for undo
	Asura_Hash_ID m_uMixerPresetBackup;
};



//////////////////////////////////////////////////////////////////////////////////////////////////
/*---------------**
** Cutscene Undo **
**---------------*/

class CutsceneUndoData_Event_Created : public RootUndoData
{
public :
	CutsceneUndoData_Event_Created(Tools_Cutscene_Event* event);
	virtual RootUndoData* ProcessUndoData();
	virtual int GetPriority() const;


	int EventID;

};

class CutsceneUndoData_Event_Destroyed : public RootUndoData
{
public :
	CutsceneUndoData_Event_Destroyed(Tools_Cutscene_Event* event);
	virtual RootUndoData* ProcessUndoData();
	virtual int GetPriority() const;

	int EventID;
	int Type;
};


class CutsceneUndoData_Event_Changed : public RootUndoData
{
public :
	CutsceneUndoData_Event_Changed(Tools_Cutscene_Event* event);

	virtual RootUndoData* ProcessUndoData();
	virtual int GetPriority() const;

	virtual void SwapWithEvent(Tools_Cutscene_Event* event);

	int EventID;
	float TriggerTime;
	Asura_String TargetActorName;
	int ParentEventID;
	u_int m_uEventFlags;
	int m_iEnableID;

	Asura_String m_xName;//For overiding the name displayed in tools
	Asura_String m_xComment;//Optional comment for use in tools
	Asura_Vector_3 m_xPosition;//For overiding the event object's position
	Asura_String m_xNameOfSourceTemplate;//if this event belongs to a template, template event's name is here.
	Asura_String m_xNameOfEventWrapper;//if this event belongs to a template, wrapper event name is here.

};

class CutsceneUndoData_Event_Changed_AnimationChange : public CutsceneUndoData_Event_Changed
{
public :
	CutsceneUndoData_Event_Changed_AnimationChange(Tools_Cutscene_Event* event);
	void SwapWithEvent(Tools_Cutscene_Event* event);

	Asura_String AnimationName;
	Asura_String AnimationFileName;
	int AnimationFlags;
	float TweenDuration;
	float m_fStartTimer;
	float m_fEndTimer;
	float m_fAnimDelay;

};

class CutsceneUndoData_Event_Changed_SecondaryAnimation : public CutsceneUndoData_Event_Changed_AnimationChange
{
public :
	CutsceneUndoData_Event_Changed_SecondaryAnimation(Tools_Cutscene_Event* event);
	void SwapWithEvent(Tools_Cutscene_Event* event);

	float Weight;
	float Duration;

};

class CutsceneUndoData_Event_Changed_TextureAnimation : public CutsceneUndoData_Event_Changed
{
public :
	CutsceneUndoData_Event_Changed_TextureAnimation(Tools_Cutscene_Event* event);
	void SwapWithEvent(Tools_Cutscene_Event* event);

	int m_iSequence;
	float m_fSpeed;
	u_int m_uTextureAnimFlags;

};

class CutsceneUndoData_Event_Changed_TimeScale : public CutsceneUndoData_Event_Changed
{
public :
	CutsceneUndoData_Event_Changed_TimeScale(Tools_Cutscene_Event* event);
	void SwapWithEvent(Tools_Cutscene_Event* event);

	float TimeScale;


};

class CutsceneUndoData_Event_Changed_FieldOfView : public CutsceneUndoData_Event_Changed
{
public :
	CutsceneUndoData_Event_Changed_FieldOfView(Tools_Cutscene_Event* event);
	void SwapWithEvent(Tools_Cutscene_Event* event);

	float Duration;
	float FinalFOV;

};

class CutsceneUndoData_Event_Changed_LoopBack : public CutsceneUndoData_Event_Changed
{
public:

    CutsceneUndoData_Event_Changed_LoopBack(Tools_Cutscene_Event* event);
	void SwapWithEvent(Tools_Cutscene_Event* event);

    float m_fSecondsToLoopBack;
	int m_iLoopBackEventID;
};

class CutsceneUndoData_Event_Changed_Attachment : public CutsceneUndoData_Event_Changed
{
public:

    CutsceneUndoData_Event_Changed_Attachment(Tools_Cutscene_Event* pxEvent);
	void SwapWithEvent(Tools_Cutscene_Event* pxEvent);

	int m_uAttachmentID;
};


class CutsceneUndoData_Event_Changed_AttachmentControl : public CutsceneUndoData_Event_Changed
{
public:

    CutsceneUndoData_Event_Changed_AttachmentControl(Tools_Cutscene_Event* pxEvent);
	~CutsceneUndoData_Event_Changed_AttachmentControl();
	void SwapWithEvent(Tools_Cutscene_Event* pxEvent);

	u_int m_uNumAttachments;
	Asura_Cutscene_Event_AttachmentControl_Attachment* m_pxAttachments;


};



class CutsceneUndoData_Event_Changed_Beam : public CutsceneUndoData_Event_Changed
{
public:

    CutsceneUndoData_Event_Changed_Beam(Tools_Cutscene_Event* pxEvent);
	void SwapWithEvent(Tools_Cutscene_Event* pxEvent);

	Asura_Cutscene_BeamParameters m_xParam;
};

class CutsceneUndoData_Event_Changed_Fade : public CutsceneUndoData_Event_Changed
{
public :
	CutsceneUndoData_Event_Changed_Fade(Tools_Cutscene_Event* pxEvent);
	void SwapWithEvent(Tools_Cutscene_Event* pxEvent);

	float m_fRed;
	float m_fGreen;
	float m_fBlue;
	float m_fDuration;
	u_int m_uFlags;

};

class CutsceneUndoData_Event_Changed_TemplateWrapper : public CutsceneUndoData_Event_Changed
{
public :
	CutsceneUndoData_Event_Changed_TemplateWrapper(Tools_Cutscene_Event* pxEvent);
	void SwapWithEvent(Tools_Cutscene_Event* pxEvent);
};


class CutsceneUndoData_Event_Changed_Subtitle : public CutsceneUndoData_Event_Changed
{
public :
	CutsceneUndoData_Event_Changed_Subtitle(Tools_Cutscene_Event* pxEvent);
	void SwapWithEvent(Tools_Cutscene_Event* pxEvent);

	float m_fDuration;
	Asura_Hash_ID m_uTextHash;

};



class CutsceneUndoData_Event_Changed_Sound : public CutsceneUndoData_Event_Changed
{
public :
	CutsceneUndoData_Event_Changed_Sound(Tools_Cutscene_Event* pxEvent);
	void SwapWithEvent(Tools_Cutscene_Event* pxEvent);

	Asura_String m_xSoundName;

};


class CutsceneUndoData_Event_Changed_ChangeCameraTarget : public CutsceneUndoData_Event_Changed
{
public :
	CutsceneUndoData_Event_Changed_ChangeCameraTarget(Tools_Cutscene_Event* pxEvent);
	void SwapWithEvent(Tools_Cutscene_Event* pxEvent);

	Asura_String m_xCameraTargetName;
};

class CutsceneUndoData_Event_Changed_ChangeCameraShake : public CutsceneUndoData_Event_Changed
{
public :
	CutsceneUndoData_Event_Changed_ChangeCameraShake(Tools_Cutscene_Event* pxEvent);
	void SwapWithEvent(Tools_Cutscene_Event* pxEvent);

	Asura_Cutscene_Camera_Shake m_xShake;
	float m_fDuration;
};

class CutsceneUndoData_Event_Changed_SetAlpha : public CutsceneUndoData_Event_Changed
{
public :
	CutsceneUndoData_Event_Changed_SetAlpha(Tools_Cutscene_Event* pxEvent);
	void SwapWithEvent(Tools_Cutscene_Event* pxEvent);

	Asura_Graph<float> m_xAlpha;
	float m_fDuration;
};

class CutsceneUndoData_Event_Changed_ChangePlayerCameraControl : public CutsceneUndoData_Event_Changed
{
public :
	CutsceneUndoData_Event_Changed_ChangePlayerCameraControl(Tools_Cutscene_Event* pxEvent);
	void SwapWithEvent(Tools_Cutscene_Event* pxEvent);

	float m_fPitch;
	float m_fYaw;
	float m_fDuration;
};

#ifdef ASURA_USE_FACIAL_ANIMATION_SYSTEM
class CutsceneUndoData_Event_Changed_FacialAnimation : public CutsceneUndoData_Event_Changed
{
public :
	CutsceneUndoData_Event_Changed_FacialAnimation(Tools_Cutscene_Event* pxEvent);
	void SwapWithEvent(Tools_Cutscene_Event* pxEvent);

	Asura_Hash_ID m_uAnimID;
	u_int m_uAnimationFlags;

};
#endif

class CutsceneUndoData_Event_Changed_MixerPreset : public CutsceneUndoData_Event_Changed
{
public :
	CutsceneUndoData_Event_Changed_MixerPreset(Tools_Cutscene_Event* pxEvent);
	void SwapWithEvent(Tools_Cutscene_Event* pxEvent);

	Asura_Hash_ID m_uMixerPreset;

};



#if defined(ASURA_USE_GUI_MENU_SYSTEM)
//
// Tools_Cutscene_Event_GUIMenu
//
class Tools_Cutscene_Event_GUIMenu : public Tools_Cutscene_Event
{
public :

	Tools_Cutscene_Event_GUIMenu(Asura_Cutscene_Event* pxEvent,Tools_Cutscene_Object* pxCutsceneObject, int iEventID);

	virtual const char*  GetTypeDescription(){ return "Render a GUI menu in a cutscene event"; }
	virtual const char*  GetAbbreviatedTypeDescription(){ return "GUI Menu"; }
	virtual Asura_String GetDescription(){ return "GUI Menu event"; }


	virtual void AddOverlayControls(Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset);
	virtual void UpdateBackupVariables();

	// Accessors
	Asura_Hash_ID GetGUIMenuHashBackup() const { return m_uGUIMenuHashBackup; }
	float         GetDurationBackup() const { return m_fDurationBackup; }

private :

	enum MENUS_DISPLAYED
	{
		NUM_MENU_PAGES_DISPLAYED = 8
	};

	Asura_Hash_ID m_uGUIMenuHashBackup;
	float         m_fDurationBackup;

	typedef Tools_Cutscene_Event PARENT;
};

//
// CLASS Tools_GUIMenu_Event_DropDownList_Dialog
//
class Tools_GUIMenu_Event_DropDownList_Dialog : public Asura_Hash_DropDownList_Dialog
{
public:

	Tools_GUIMenu_Event_DropDownList_Dialog( const Asura_Vector_2& xOffset, const int iNumberOfLines, Asura_Hash_ID* puSelected, bool* pbChangedFlag, const char* szName);
	
	virtual u_int GetNumberOfHashValues() const;
	virtual const char* GetNameByIndex(int iIndex) const;

private :

    typedef Asura_Hash_DropDownList_Dialog PARENT;
};

//
// Tools_Cutscene_Event_Changed_GUIMenu
//
class CutsceneUndoData_Event_Changed_GUIMenu : public CutsceneUndoData_Event_Changed
{
public :

	CutsceneUndoData_Event_Changed_GUIMenu(Tools_Cutscene_Event* pxEvent);
	virtual void SwapWithEvent(Tools_Cutscene_Event* pxEvent);

private :

	//backup variables for undo
	Asura_Hash_ID m_uGUIMenuHash;
	float         m_fDuration;

};

//
// Tools_Cutscene_Event_ConsoleVar
//
class Tools_Cutscene_Event_ConsoleVar : public Tools_Cutscene_Event
{
public :

	Tools_Cutscene_Event_ConsoleVar(Asura_Cutscene_Event* pxEvent,Tools_Cutscene_Object* pxCutsceneObject, int iEventID);

	virtual const char*  GetTypeDescription(){ return "Change console variables in cutscenes"; }
	virtual const char*  GetAbbreviatedTypeDescription(){ return "Console Vars"; }
	virtual Asura_String GetDescription(){ return "Display Console Vars"; }


	virtual void AddOverlayControls(Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset);
	virtual void UpdateBackupVariables();

	// Accessors
	const Asura_String& GetValueBackup() const { return m_xValueBackup; }
	const Asura_GUIMenu_ConsoleVar& GetConsoleVarBackup() const { return m_xConsoleVarBackup; }

private :

	Asura_String m_xValueBackup;
	Asura_GUIMenu_ConsoleVar m_xConsoleVarBackup;

	typedef Tools_Cutscene_Event PARENT;
};

//
// CutsceneUndoData_Event_Changed_ConsoleVar
class CutsceneUndoData_Event_Changed_ConsoleVar : public CutsceneUndoData_Event_Changed
{
public :

	CutsceneUndoData_Event_Changed_ConsoleVar(Tools_Cutscene_Event* pxEvent);
	virtual void SwapWithEvent(Tools_Cutscene_Event* pxEvent);

private :

	//backup variables for undo
	Asura_String m_xValue;
	Asura_GUIMenu_ConsoleVar m_xConsoleVar;
};
#endif	//defined(ASURA_USE_GUI_MENU_SYSTEM)

#ifdef ASURA_USE_HDR
//
// Tools_Cutscene_Event_HDRSettings
//
class Tools_Cutscene_Event_HDRSettings : public Tools_Cutscene_Event
{
public :

	Tools_Cutscene_Event_HDRSettings(Asura_Cutscene_Event* pxEvent,Tools_Cutscene_Object* pxCutsceneObject, int iEventID);

	virtual const char*  GetTypeDescription(){ return "Set HDR Settings for this event"; }
	virtual const char*  GetAbbreviatedTypeDescription(){ return "HDR Settings"; }
	virtual Asura_String GetDescription(){ return "HDR Settings event"; }


	virtual void AddOverlayControls(Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset);
	virtual void UpdateBackupVariables();

	// Accessors
	const Asura_CameraData::HDRSettings& GetHDRSettingsBackup() const { return s_xHDRSettingsBackup; }
	float GetDurationBackup() const { return m_fDurationBackup; }

private :

    static void CaptureFixedLuminance();

	static Asura_CameraData::HDRSettings s_xHDRSettingsBackup;
	float  m_fDurationBackup;

	typedef Tools_Cutscene_Event PARENT;
};

//
// CutsceneUndoData_Event_Changed_HDRSettings
//
class CutsceneUndoData_Event_Changed_HDRSettings : public CutsceneUndoData_Event_Changed
{
public :

	CutsceneUndoData_Event_Changed_HDRSettings(Tools_Cutscene_Event* pxEvent);
	virtual void SwapWithEvent(Tools_Cutscene_Event* pxEvent);

private :

	//backup variables for undo
	static Asura_CameraData::HDRSettings s_xHDRSettings;
	float  m_fDuration;

};

#endif ASURA_USE_HDR

#endif //_TOOLS_CUTSCENE_EVENT_H_