#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef _TOOLS_CUTSCENE_OBJECT_H_
#define _TOOLS_CUTSCENE_OBJECT_H_


#include "Sound/Asura_Sound.h"


#include "Misc/list_tem.hpp"
#include "Objects/Tools_Object.h"

class Asura_Cutscene;
class Asura_Cutscene_Actor;
class Asura_Cutscene_Event;
class Asura_Cutscene_Track;
class Asura_ServerEntity_CutsceneController;
class Asura_String;
class Asura_Tabs_Dialog;
class Tools_Cutscene_Event;
class Tools_Cutscene_Sound;
class Tools_Cutscene_Track;
class Tools_CutsceneActor_Object;

//Debugging tool for dumping cutscenes to a file in a form which should hopefully be safe to reload elsewhere
#define ALLOW_CUTSCENE_EXPORT 1

typedef bool (*ToolsChooseEventFilterFn)(Tools_Cutscene_Event* pxEvent,int iData);

class Tools_Cutscene_Object : public Tools_Object
{
	public :
		Tools_Cutscene_Object(Asura_Cutscene* _Cutscene);
		~Tools_Cutscene_Object();


		//virtual function overides
		void UpdatePositionFromAsuraData();
		void UpdatePositionInAsuraData(bool /*updateLinkedObjects*/);
		virtual void CalculateRelativePosition(Tools_Object* pxObject);
		virtual void UpdateRelative(Tools_Object* pxObject);
		void Edit();
		const char* GetName();
		const char* GetTypeDescription(){return "Cutscene";}
		void PrintInfo(float& x, float& y);
		void Destroy();
		void AddContextMenuItems(class Tools_ContextMenu& xMenu, const class Tools_ContextMenu_Context& xContext);

		virtual bool SupportsMessageEditor() { return m_pxCutsceneController!=0; }
		virtual Asura_Entity_Server* GetServerEntity();
		
		//Warn if a cutscene gets selected for deletion as part of a multiple object
		//deletion
		virtual bool ExtraConfirmationForMultiDestruction(){return true;}

		virtual void UpdateCutsceneEdit();

		virtual void StartCutsceneEdit();
		virtual void FinishCutsceneEdit();

		virtual void AddOverlayControls(class Asura_Overlay* newOverlay,class Asura_Vector_2& offset);
		void UpdateOverlayLabels();


		
		static void CreateAllCutsceneObjects();//creates objects for all existing cutscenes

		virtual void CreateChildObjects();
		void DeleteChildObjects();
		
		virtual bool CheckNameOK(const Asura_String& xName);
		Tools_CutsceneActor_Object* CreateActor(int actorType,const char* name = NULL,Asura_Guid uGuid = ASURA_GUID_UNREGISTERED,bool bCreateTrack = true);
		void CreateActor();
		virtual int SelectActorType();
		virtual Asura_Cutscene_Actor* AllocateActor(int actorType,const Asura_String& actorName);
		virtual Tools_CutsceneActor_Object* AllocateToolsActor(Asura_Cutscene_Actor* actor);
		
        virtual Tools_Cutscene_Track* AllocateToolsTrack( Asura_Cutscene_Track* pxTrack, Tools_CutsceneActor_Object* pxActorToolsObj, int iTrackID );
		
		void RemoveActorFromCutscene(class Asura_Cutscene_Actor* actor);
		void HandleActorNameChange(class Asura_Cutscene_Actor* actor);
		void UpdateTracks();
		void PerframeUpdateWhilePlaying();
		void UpdateAfterCutscene();
		
		void RecordWavFile();
		void PickWavFile();

		void RenderSubtitle(float&x, float& y);


		//adds event options to a seleciton overlay for creating new events
		virtual void AddEventsToSelectionOverlay(class ListSelection_Overlay& overlay);
		//allocate an Asura_Cutscene_Event based on the event type
		Asura_Cutscene_Event* AllocateAsuraEvent(int eventType);
		//allocate a Tools_Cutscene_Event appropriate to an Asura_Cutscene_Event
		virtual Tools_Cutscene_Event* AllocateToolsEvent(Asura_Cutscene_Event* event,int eventID);
		//craete a new event , or re-create using passed paramters
		Tools_Cutscene_Event* CreateCutsceneEvent(bool bShowUI,int eventType=-1,int eventID=-1,Asura_Cutscene_Event* pxEventToCopy = 0, Asura_Cutscene_Actor* pxTarget = 0, float fTime = -1.0f);

		void RemoveEventFromCutscene(class Asura_Cutscene_Event* event);
		void UpdateEventIndeces();
		void SortEvents();
		void UpdateEventsAfterUndo();
		void UpdateParentEvents();
		void UpdateEventsAfterTimeChange(List<Tools_Cutscene_Event*> alteredEventList);

		void SortActors();

		void EditCutsceneDirectory();
		const char* GetCutsceneDirectory();
        bool Rename();  // returns whether it was renamed.

	
		class Tools_CutsceneActor_Object* GetActor(const char* actorName);
		void CopyActor();
		void DuplicateSelectedActors();
		void CopyActorEvents(const Asura_Cutscene* pxCutscene, 
							 const Asura_Cutscene_Actor* pxActorToCopy, 
							 Asura_Cutscene_Actor* pxTargetActor,
							 List<Tools_Cutscene_Event*>& xListOfCopiedToolsEvents);

		Asura_Cutscene_Actor* CopyActor(Asura_Cutscene* pxCutsceneToCopy,Asura_Cutscene_Actor* pxActorToCopy,Tools_CutsceneActor_Object* pxActorToOverwrite,const char* szActorName);

		bool EditFirstEventForActor(Tools_CutsceneActor_Object* pxActor, int iEventType = -1);
		
		void SelectEventByIndex(int iIndex);
		void SelectEvent(Tools_Cutscene_Event* event);

		bool ChooseEvent(const char* message,Tools_Cutscene_Event*& outputEvent,ToolsChooseEventFilterFn fnFilter=0,int iData=0);

		static void ContextMenu_SelectCutscene(void* pObject, u_int uMode);

		static void EditMessageBlock();

		void ExportCutscene();

		bool IsServerCutscene() const;

		//Updates the cutscene length to match the sound track , if flagged to do so.
		void UpdateLengthFromSoundTrack();

		//find cutscene by name
		static Tools_Cutscene_Object* FindCutsceneObject(const char* pszName);

		//converts initial animations into animation change events
		static void ConvertInitialAnimations(Asura_Cutscene* pxCutscene);

		static void ReorderActorsForTimeline();

		void UpdateBackupVariables();

		//data
		Asura_Cutscene* Cutscene;
		List<class Tools_CutsceneActor_Object*> ActorObjects;
		List<Tools_Cutscene_Event*> EventList;

		Tools_Cutscene_Event* SelectedEvent;
		float SelectedEventIndex;
		bool SelectedEventIndexChanged;

		bool m_bChanged;
		bool m_bUseSoundTrackChanged;
		bool m_bTakeLengthFromSoundTrackChanged;

		//for keeping track of speech during tools cutscene playback
		int m_iPreviousScriptLine;
		List<Tools_Cutscene_Sound*> m_xSoundList;

		bool m_bEventSortRequired;

		static bool s_bMoveCutsceneMembersWithCutsceneObject;

		Asura_ServerEntity_CutsceneController* m_pxCutsceneController;

		//For determining whether the 'Edit End Cutscene Message' button needs updating
		int m_iPreviousNumMessages;

		enum DialogID
		{
			DialogID_CutsceneDirectory,
			DialogID_SoundTrack,
			DialogID_EditEndCutsceneMessage,

            DialogID_FirstAvailableForSubclass,
		};

		//backup for undo
		Asura_Matrix_3x3 m_xOrientationBackup;
		Asura_Vector_3 m_xPositionBackup;


protected:
	bool IsEnabled_LoopToStartTime();


	// Members for the overlay controls.
	enum TAB_NAMES
	{
		TAB_MAIN,
		TAB_FLAGS,
		TAB_NUM_TAB_NAMES
	};

	static Asura_Tabs_Dialog* s_pxCutsceneTabs;
};

class CutsceneUndoData_Cutscene_Changed : public RootUndoData
{
public :
	CutsceneUndoData_Cutscene_Changed(Tools_Cutscene_Object* pxCutsceneObject);
	virtual RootUndoData* ProcessUndoData();

	virtual int GetPriority() const;

	Asura_String m_xCutsceneName;

	Asura_Vector_3 Position;
	Asura_Matrix_3x3 Orientation;

};


#endif //_TOOLS_CUTSCENE_OBJECT_H_
