#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef _TOOLS_CUTSCENE_TIMELINE_H_
#define _TOOLS_CUTSCENE_TIMELINE_H_

#include "Core/Asura_Maths.h"
#include "Misc/list_tem.hpp"

#include "Misc/Tools_Modal_Overlay.h"

class Asura_Cutscene_Actor;

class Tools_Cutscene_Object;
class Tools_CutsceneActor_Object;
class Tools_CutsceneTrackPoint_Object;
class Tools_Cutscene_Event;

class Tools_Timeline_Actor;
class Tools_Timeline_Category;
class Tools_Timeline_Data;


///////////////////////////////////////////////////////////////////////////////
enum Tools_Timeline_CategoryType
{
	TTC_Position = -3,
	TTC_Animation = -2,
	TTC_AnimationSecondary = -1,
	//0 upwards match event IDs
	TTC_Misc = 0x7fffffff,//should be higher than highest event ID
};

///////////////////////////////////////////////////////////////////////////////
/*-------------------------**
** Tools_Cutscene_Timeline **
**-------------------------*/

class Tools_Cutscene_Timeline
{
public :
	static void ShowTimeline(Tools_Cutscene_Object* cutsceneObject);

	static void UpdateForOverlay(float fBottomY,float fWidth,int focusFlags);
	static void RenderForOverlay(float fBottomY,float fWidth);

	static void Update(int focusFlags = 0xffffffff);
	static void Render();

	static void Invalidate(){Valid = false;}
	static void SetCutscene(Tools_Cutscene_Object* cutsceneObject);
	static void SetActor(Tools_CutsceneActor_Object* actorObject);

	static void FocusLost();

	static void SetTimelineBarHeight(float fHeight);
	static float GetTimelineBarHeight();
	static float GetTimelineBarMinX();


//private :
	static void GenerateTimelineData();
	static void DeleteTimelineData();

	static void AddTimelineData(Tools_CutsceneActor_Object* actor,Tools_Timeline_Data * data,int iCategory);
	static void AddTimelineData(Asura_Cutscene_Actor* actor,Tools_Timeline_Data * data,int iCategory);

	static void HandleScrollAndZoom();

	static void CalculateScreenCoords();
	static Tools_Timeline_Data* GetItemAtCoord(int mx,int my, u_int* puSelectionFlags = 0,bool ignoreSelected = false);
	static Tools_Timeline_Actor* GetActorAtCoord(int mx,int my);

	static Tools_Timeline_Data* GetItemByID(int iID);

	static void SelectItem(Tools_Timeline_Data* item, u_int uSelectionFlags, bool bSelect);
	static Tools_Timeline_Data* GetPrimarySelection(){return SelectedList.size() ? SelectedList.first_entry() : 0;}

	static void MoveSelected(float fDeltaTime);
	static void UpdateDragMovement();

	static void SetParentEvent(Tools_Cutscene_Event* parentEvent);

	static void EditHiddenActors();

	static void EditHiddenEvents();
	static bool IsEventTypeHidden(int iType);
	static void SetEventTypeHidden(int iType,bool bHide);

	static void DeselectHiddenItems();

	//Calculates the area available for rendering each of the parts of the timeline
	static void CalculateRenderRectangles();

	//Finds the nearest snap point ,if any are close enough.
	//returns true if a snap point was found.
	static bool GetSnappedTime(float fTime, float& fOutputTime);
	
	//Get the Y coordinate of the end of the timeline data
	static float GetMaxTimelineDataY();

	static Tools_Cutscene_Object* CutsceneObject;
	static Tools_CutsceneActor_Object* ActorObject;
	static bool s_bCurrentActorChanged;
	static List<Tools_Timeline_Actor*> ActorList;
	static float CutsceneLength;
	static List<Tools_Timeline_Data*> SelectedList;
	static int s_iPrimarySelectedID;//ID of time line data that is the primary selection

	//events to hide
	static List<int> s_xFullScreenHiddenEventTypes;
	//timeline bar works the other way around ,and assumes everything is hidden by default
	static List<int> s_xTimelineBarVisibleEventTypes;

	static bool Valid;

	static float MinVisibleTime;//minimum visible time (for overlay control)
	static float MaxVisibleTime;//maximum visible time (for overlay control)

	//How much the timeline screen should be scrolled down
	static float s_fFullScreenActorYOffset;
	static float s_fTimelineBarActorYOffset;


	static bool s_bUseSnap;
};

///////////////////////////////////////////////////////////////////////////////
/*----------------------**
** Tools_Timeline_Actor **
**----------------------*/
class Tools_Timeline_Actor
{
public :
	Tools_Timeline_Actor(Tools_CutsceneActor_Object* actorObject);
	~Tools_Timeline_Actor();

	void AddTimelineData(Tools_Timeline_Data * data,int iCategory);
	Tools_Timeline_Category* GetCategory(int iCategory,bool bCreate = false);

	void DetermineScreenCoords(float& y);
//	void AdjustScreenCoords(const Asura_Vector_2& xOffset);
	void RenderHeadings();
	void Render();
	Tools_Timeline_Data* GetItemAtCoord(int mx,int my, u_int* puSelectionFlags,bool ignoreSelected);
	bool PointIsInName(int mx,int my);
	void SelectInRect(Asura_Vector_2 p1,Asura_Vector_2 p2,bool select);

	float GetNearestSnapTime(float fTime);

	Tools_Timeline_Data* GetItemByID(int iID);

	void Edit();

	bool IsHidden();


	Tools_CutsceneActor_Object* ActorObject;
	List<Tools_Timeline_Category*> m_xCategoryList;
	float Y;

};

///////////////////////////////////////////////////////////////////////////////
/*-------------------------**
** Tools_Timeline_Category **
**-------------------------*/

class Tools_Timeline_Category
{
public :
	Tools_Timeline_Category(Tools_Timeline_Actor* pxTimelineActor,int iCategory);
	~Tools_Timeline_Category();

	void AddTimelineData(Tools_Timeline_Data * pxData);
	void RenderHeading(float x,float y);
	bool IsHidden();


	int m_iCategory;
	Tools_Timeline_Actor* m_pxTimelineActor;

	List<Tools_Timeline_Data*> m_xTimelineData;

};

///////////////////////////////////////////////////////////////////////////////
/*---------------------**
** Tools_Timeline_Data **
**---------------------*/

#define MAX_LINES_FOR_TIMELINE_CAT 4

#define TOOLS_TIMELINE_SELECTED_START 0x00000001
#define TOOLS_TIMELINE_SELECTED_END 0x00000002
#define TOOLS_TIMELINE_SELECTED_ALL (TOOLS_TIMELINE_SELECTED_START|TOOLS_TIMELINE_SELECTED_END)

class Tools_Timeline_Data
{
public :
	Tools_Timeline_Data();

	virtual ~Tools_Timeline_Data();

	virtual int GetColour(){return 0;}
	virtual void PrintDescription(float /*x*/,float /*y*/) const {}
	virtual bool Selectable(){return true;}
	
	virtual void SetTime(float time){Time = time;}
	float GetStartTime() const {return Time;}
	float GetEndTime() const {return Time+Duration;}

	virtual bool CanEditDuration() const{return false;}
	virtual void SetDuration(float fDuration){};

	void SelectStart(bool bSelect);
	void SelectEnd(bool bSelect);

	virtual void SetSelectionFlags(u_int uSelectionFlags){}
	virtual u_int GetSelectionFlags() const{return 0;}

	bool IsSelected() const {return GetSelectionFlags() != 0;}
	bool IsStartSelected() const {return (GetSelectionFlags() & TOOLS_TIMELINE_SELECTED_START) ? true : false;}
	bool IsEndSelected() const {return (GetSelectionFlags() & TOOLS_TIMELINE_SELECTED_END) ? true : false;}

	bool CanSnapEnd() const {return m_bCanSnapEnd;}


	virtual bool Edit(){return false;}

	virtual Tools_Cutscene_Event* GetEvent(){return 0;}

	//Gets an ID for the data that will persist between calls to GenerateTimelineData()
	//essentially just uses the pointer of the underlying event or position object
	virtual int GetID(){return 0;}

	bool IsHidden(){return m_pxTimelineCategory ? m_pxTimelineCategory->IsHidden() : false;}

	
	void DetermineScreenCoords(float y,int& numLinesUsed,float highestXOnLine[MAX_LINES_FOR_TIMELINE_CAT]);
//	void AdjustScreenCoords(const Asura_Vector_2& xOffset);
	void Render();
	
	float Time;
	float Duration;
	float Left,Top,Right,Bottom;
	
	bool m_bCanSnapEnd;

	bool IsOkay;
	Tools_Timeline_Category* m_pxTimelineCategory;

};

///////////////////////////////////////////////////////////////////////////////
/*------------------------------**
** Tools_Timeline_Data_Position **
**------------------------------*/
class Tools_Timeline_Data_Position : public Tools_Timeline_Data
{
public :
	Tools_Timeline_Data_Position(Tools_CutsceneTrackPoint_Object* pointObject);
	int GetColour();
	void PrintDescription(float x,float y) const;
	void SetSelectionFlags(u_int uSelectionFlags);
	u_int GetSelectionFlags() const;
	bool Edit();

	void SetTime(float time);

	bool CanEditDuration() const{return true;}
	void SetDuration(float fDuration);


	virtual int GetID(){return (int)PointObject;}


	Tools_CutsceneTrackPoint_Object* PointObject;
};

///////////////////////////////////////////////////////////////////////////////
/*--------------------------**
** Tools_Timeline_Data_Event**
**--------------------------*/
class Tools_Timeline_Data_Event : public Tools_Timeline_Data
{
public :
	Tools_Timeline_Data_Event(Tools_Cutscene_Event* event);
	~Tools_Timeline_Data_Event();
	int GetColour();
	void PrintDescription(float x,float y) const;
	void SetSelectionFlags(u_int uSelectionFlags);
   	u_int GetSelectionFlags() const;
	bool Selectable();
	bool Edit();
	Tools_Cutscene_Event* GetEvent(){return Event;}

	virtual int GetID(){return (int)Event;}



	void SetTime(float time);
	
	bool CanEditDuration() const;
	void SetDuration(float fDuration);

	Tools_Cutscene_Event* Event;

};

///////////////////////////////////////////////////////////////////////////////
/*----------------------------------**
** Tools_Timeline_Data_Initial_Anim **
**----------------------------------*/
class Tools_Timeline_Data_Initial_Anim : public Tools_Timeline_Data
{
public :
	Tools_Timeline_Data_Initial_Anim(Tools_CutsceneActor_Object* actorObject);
	int GetColour();
	void PrintDescription(float x,float y) const;
	bool Selectable(){return false;}
	bool Edit();

	void SetTime(float){/*Can't change the time for this one*/}
	
	Tools_CutsceneActor_Object* ActorObject;

};

///////////////////////////////////////////////////////////////////////////////
/*---------------------------**
** TimelineHideActor_Overlay **
**---------------------------*/

///////////////////////////////////////////////////////////////////////////////

class TimelineHideActor_Overlay : public ListSelection_Overlay
{
	public :
		TimelineHideActor_Overlay(Tools_Cutscene_Object* pxCutsceneObject);
		virtual bool DoModal();

	protected :

		virtual bool IsTickboxList() const {return true;}
		virtual const char* GetListHeadingText() const {return "Visible Actors";}
		virtual const char* GetTickAllButtonText() const {return "Show All";}
		virtual const char* GetTickNoneButtonText() const {return "Hide All";}

		Tools_Cutscene_Object* m_pxCutsceneObject;
		
};


///////////////////////////////////////////////////////////////////////////////
/*----------------------------**
** TimelineHideByType_Overlay **
**----------------------------*/

class TimelineHideByType_Overlay : public ListSelection_Overlay
{
	public :
		TimelineHideByType_Overlay(Tools_Cutscene_Object* pxCutsceneObject);
		virtual bool DoModal();

	protected :

		virtual bool IsTickboxList() const {return true;}
		virtual const char* GetListHeadingText() const {return "Visible Types";}
		virtual const char* GetTickAllButtonText() const {return "Show All";}
		virtual const char* GetTickNoneButtonText() const {return "Hide All";}

		Tools_Cutscene_Object* m_pxCutsceneObject;
		
};


#endif //_TOOLS_CUTSCENE_TIMELINE_H_
