//
// Tools_WavWriter.h
//
// Created by Richard Rice - 19/11/2002
// 
// Copyright (C) 2002 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef _TOOLS_WAVWRITER_H_
#define _TOOLS_WAVWRITER_H_

// Stdlib platform-specific.
#include <windows.h>

// These definitions trim mmsystem.h down to only what we need
#define MMNODRV
#define MMNOSOUND
#define MMNOMIDI
#define MMNOAUX
#define MMNOMIXER
#define MMNOTIMER
#define MMNOJOY
#define MMNOMCI
#define MMNOMMIO
#define MMNOMMSYSTEM

// Need to include this manually: it'll have been stripped out of windows.h if WIN32_LEAN_AND_MEAN is defined
#include <mmsystem.h>

// Engine independent.
#include "Core/Asura_Maths.h"

// Tools.
#include "Misc/list_tem.hpp"


struct InputWavFile;

struct ToolsWavSample
{
	short m_sLeft;
	short m_sRight;
};

struct InputWavFileProp
{
	InputWavFileProp();
	
	Asura_Vector_3* m_pxLoc;
	float m_fInnerRange;
	float m_fOuterRange;
	float m_fVolume;
	float m_fPitch;

	bool m_bLooped;
	float m_fDuration;
};

class Tools_WavWriter
{
public :
	Tools_WavWriter();
	~Tools_WavWriter();

	bool StartOutput(const char* pszFilename);
	bool UpdateOutput(float fTime,const Asura_Vector_3* pxLocation,const Asura_Matrix_3x3* pxOrient);
	bool FinishOutput();

	void AddInputWavFile(const char* pszFilename,float fStartTime,const InputWavFileProp* pxProp);

private :
	HANDLE m_hOutputFile;	
	PCMWAVEFORMAT m_xWavFormat;
	int m_iDataOffset; //offset to 'data' chunk
	float m_fCurrentTime;
	int m_iSamplesWritten;
	int m_iLastNonZeroSample;
	
	int m_iBufferSize;
	ToolsWavSample* m_pxSampleBuffer;

	Asura_Vector_3 m_xLocation;
	Asura_Matrix_3x3 m_xInverseOrient;

	List<InputWavFile*> m_xWavList;

	friend struct InputWavFile;

};

#endif //_TOOLS_WAVWRITER_H_
