

//////////////////////////////////////////////////////////////////////////////////////////
//
//  FILE : Tools_Mode_Decals.h
//
//  Created by Dave Reed - 07/08/07
//
//  Copyright (C) 2007 Rebellion (Derby), All Rights Reserved.
//
//  Decal mode - for creating and editing static decals
//
//////////////////////////////////////////////////////////////////////////////////////////


#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __TOOLS_MODE_DECALS_H__
#define __TOOLS_MODE_DECALS_H__

#include "Tools_Mode.h"
#include "Objects/Tools_Object.h"
#include "Core/Asura_Handle.h"



class Tools_StaticDecal_Object : public Tools_Object
{
	friend class Tools_Mode_Decals;

public:

	Tools_StaticDecal_Object(const Asura_Vector_3& xPosition, const Asura_Matrix_3x3 &xOrient, bool bUpdateDecal = true);	

	const char*					GetTypeDescription()	{return "StaticDecal";}
	const char*					GetName();

	void						Destroy();
	void						UpdateWhileSelected();
	void						Update();
	void						Render();	
	void						OnSelection();
	void						OnDeselection();

	virtual void				Copy(const Tools_Object* pxObject);
	virtual bool				CanBeCopied() const {return true;}
	virtual Tools_Object*		CreateObjectOfThisType() const;


	class Tools_StaticDecal    *m_pxDecal;	
	Asura_Hash_ID				m_uDecalSetHash;
	int							m_iDecalIndex;
	float						m_fScale;
	float						m_fAlpha;

	bool						m_bModified;	

};




class Tools_Mode_Decals : public Tools_Mode
{
	friend class Tools_StaticDecal_Object;

public:

	enum DialogIDs
	{
		DialogID_NumDecals,
		DialogID_NumBuffers,
		DialogID_NumPolys,
		DialogID_NumKBytes
	};

	Tools_Mode_Decals()
	{
		Initialise();
	}

	virtual ~Tools_Mode_Decals()
	{
		Shutdown();
	}

	void PreSave();
	void PostSave();
	void QuickSave(void);	
	bool IsModified(void);

	void SetModified(const bool bHasChanged = true);	
	void Initialise(void);
	void Update(void);
	void RenderWorld_PostToneMap(void);
	void Shutdown(void);	

private:

	static Asura_Handle		s_iMenuHandle;

	static bool				s_bOverlayOpen;	
	static bool				s_bExitingMode;
	static void				CreateOverlay(void);
	static void				ToggleOverlay(void);
	static bool				OverlayIsOn(void) {return s_bOverlayOpen;}	
	static void				CreateDecalOverlay(Tools_StaticDecal_Object* pxObj);
	static void				CloseInstanceOverlay(void);
	static void				SelectDecalSet(void);
	static void				CreateDecalObject(Asura_Vector_3 xPos, Asura_Matrix_3x3 xOrient, bool bSelect, bool bHandleRandomOrient);

	static Asura_Vector_2	s_xOverlayPosition;		
	static Asura_Vector_2	s_xDecalOverlayPosition;
	static bool				s_bQuickPlacementEnabled;
	static int				s_iSelectedDecalIndex;
	static uint32			s_uSelectedDecalSetHash;
	static int				s_iNextDecalIndex;
	static float			s_fNextDecalAngle;
    static bool             s_bShowModuleBBForSelectedDecal;

	STATIC_VOID_FN(			CreateDecal  );
	STATIC_VOID_FN(			EditDecals   );
	STATIC_VOID_FN(			UpdateDecals );
	STATIC_VOID_FN(			SelectCurrentDecalSet );


	// static this pointer
	static Tools_Mode_Decals* s_pxThis;
};



#endif // __TOOLS_MODE_DECALS_H__
