///////////////////////////////////////////////////////////////////////////////
//
// File: Tools_Entity_CameraVolume.h
//
// Created by Martin G.Harris (7/9/2006)
// 
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef __TOOLS_ENTITY_CAMERA_VOLUME_H__
#define __TOOLS_ENTITY_CAMERA_VOLUME_H__

#ifdef ASURA_USE_CAMERA_VOLUME

// Includes
#include "Entity/Tools_Entity_Activatable_Object.h"
#include "Objects/Tools_VolumeEdit_Object.h"

// Forward declarations
class Tools_CameraVolume_VolumeEdit_Controller;
class Asura_CameraMultiVolume;
class Asura_Overlay;

///////////////////////////////
// Tools_Entity_CameraVolume //
///////////////////////////////
class Tools_Entity_CameraVolume : public Tools_Entity_Activatable_Object
{
	typedef Tools_Entity_Activatable_Object PARENT;

public:
	Tools_Entity_CameraVolume(Asura_Entity* pxEntity);
	virtual ~Tools_Entity_CameraVolume();
	
	virtual const char* GetTypeDescription() { return "Camera Volume"; }

	virtual const char* GetMyNamePrefix() const { return Tools_Entity_CameraVolume::GetNamePrefix(); }

	virtual void UpdatePositionInAsuraData(bool bUpdateLinkedObjects);
	virtual void Update();

	virtual void AddOverlayControls(Asura_Overlay* pxNewOverlay, Asura_Vector_2& xOffset);
	virtual bool HandlesThisMessage(int iMessageID);
	virtual void PrintInfo(float& fX, float& fY);

	virtual void Copy(const Tools_Object* pxObject);
	virtual bool CanBeCopied() const { return Type == TOT_Entity_CameraVolume; }
	virtual Tools_Object* CreateObjectOfThisType() const;

	virtual void UpdateClient();

	// Create a project version of the entity.
	static void Project_Create();

	static bool ShouldShowSelectedVolume()	{ return s_bShowSelectedVolume; }
	static bool ShouldShowAllVolumes()		{ return s_bShowAllVolumes; }

	static const char* GetNamePrefix() { return "CV_"; }

protected:
	// The object has been moved by the specified distance, displace the camera properties.
	virtual void DisplaceProperties(const Asura_Vector_3& xMove) = 0;

	static void EditVolume();
	void ShowVolume(bool bShow);

	static bool s_bMoveVolumeWithObject;
	static bool s_bMovePropertiesWithObject;
	static bool s_bShowAllVolumes;
	static bool s_bShowSelectedVolume;

private:
	Asura_Vector_3 m_xPreviousPosition;
	int m_iPriority;

	Tools_CameraVolume_VolumeEdit_Controller* m_pxVolumeController;
};

//////////////////////////////////////////////
// Tools_CameraVolume_VolumeEdit_Controller //
//////////////////////////////////////////////
// For editing the volumes attached to the camera volume entity.
class Tools_CameraVolume_VolumeEdit_Controller : public Tools_VolumeEdit_Controller
{
public:
	Tools_CameraVolume_VolumeEdit_Controller(Tools_Entity_CameraVolume* pxToolsEntity);

	virtual void StartVolumeEditing();
	virtual void EndVolumeEditing();

	// Should highlight the volumes when rendering them?
	virtual bool IsHighlighted() const;
	
protected:
	virtual bool CanCreateVolume() { return true; }		
	virtual bool CanDeleteVolume() { return GetNumberOfVolumes()>0; }		

	// Gets the current number of volumes.
	virtual int GetNumberOfVolumes();
	
	// Gets a pointer to the bounding box for a given volume.
	virtual Asura_Bounding_Box* GetVolumeBox(int iIndex);

	// Gets a pointer to the orientation for a given volume. Returns 0 if there is none.
	virtual Asura_Quat* GetVolumeOrient(int iIndex);

	// Gets a default position for volume objects that have yet to gain a proper volume.
	virtual Asura_Vector_3 GetDefaultPosition() { return m_pxToolsEntity->Position; }

	// Gets the name of the owner, to use for part of the volume names
	virtual const char* GetName() { return m_pxToolsEntity->GetName(); }

	// Creates the new volume in the asura data
	virtual void CreateVolumeMain();

	// Deletes the volume from the asura data
	virtual void DeleteVolumeMain(int iIndex);

private:
	Tools_Entity_CameraVolume* m_pxToolsEntity;
	Asura_CameraMultiVolume* m_pxMultiVolume;
};

#endif //ASURA_USE_CAMERA_VOLUME

#endif //__TOOLS_ENTITY_CAMERA_VOLUME_H__

