#ifdef ASURA_USE_AXON
//////////////////////////////////////////////////////////////////////////////
//
// Tools_EntityNode_GuardZone.h
//
// (TI - 09.11.2006) 
//
// Copyright (c) 2006 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef TOOLS_ENTITYNODE_GUARDZONE_H___
#define TOOLS_ENTITYNODE_GUARDZONE_H___ 1

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// asura
#include "Core/Asura_OrientedVolume.h"
#include "Entity/Locations/GuardZone/Asura_ServerNode_GuardZone.h"

// tools
#include "Entity/Locations/Tools_Entity_Node.h"
#include "Objects/Tools_Object_System.h"
#include "Objects/Tools_VolumeEdit_Object.h"

//////////////////////////////////////////////////////////////////////////////
//
// Tools_EntityNode_GuardZone
//
//////////////////////////////////////////////////////////////////////////////

class Tools_EntityNode_GuardZone : public Tools_Entity_Node
{
private:
	typedef Tools_Entity_Node PARENT;
	friend  class Tools_GuardZone_VolumeEdit_Controller;

public:
	Tools_EntityNode_GuardZone( Asura_Entity* pxEntity );
	virtual ~Tools_EntityNode_GuardZone();

	virtual void AddOverlayControls( class Asura_Overlay* pxNewOverlay, class Asura_Vector_2& xOffset );
	const char*  GetTypeDescription() { return "Guard Zone"; }
	virtual bool IsBaseOrDerivativeOf( int iToolsObjectType ) const;
	virtual void OnSelection();
	virtual void OnDeselection();
	virtual void Render();
	virtual void Update();
	virtual void UpdatePositionInAsuraData(bool /*updateLinkedObjects*/);
	virtual void UpdateWhileSelected();
	virtual void Destroy();

    virtual bool          CanBeCopied() const { return false; }
    //virtual void          Copy( const Tools_Object* pxObject );
    //virtual Tools_Object* CreateObjectOfThisType() const;

	virtual const char* GetMyNamePrefix() const { return Tools_EntityNode_GuardZone::GetNamePrefix(); }

	void RemoveGuardPoint( Asura_Guid uGPGuid );

	static void Create();

	static Tools_EntityNode_GuardZone* FindEntity( Asura_Guid uGuid ) { return s_xList.FindEntity( uGuid ); }
	static const Asura_Hashed_List< Tools_EntityNode_GuardZone >* GetList() { return &s_xList; }

	static const char* GetNamePrefix() { return "GZ_"; }

protected:
	virtual bool AddGuardPoint( const Asura_Vector_3& xPosition );
	bool IsPositionInsideGuardZone( const Asura_Vector_3& xPosition ) const;

	Asura_ServerNode_GuardZone* m_pxGameNode;

private:
    static Tools_EntityNode_GuardZone* CreateAndReturn();

	void ShowVolume( bool bShow );
	static void AddGuardPoint();
	static void EditVolume();
    bool IsSelected() { return ( this == s_pxThisToolsGuardZone ); }

	bool m_bMoveVolumeWithObject;
	bool m_bOverlayChanged;
	bool m_bFirstUpdate;

	class Tools_GuardZone_VolumeEdit_Controller* m_pxVolumeController;

	class Asura_Vector_3 m_xPreviousPosition;

	static bool s_bShowAllVolumes;
	static Tools_EntityNode_GuardZone* s_pxThisToolsGuardZone;
	static Asura_Hashed_List< Tools_EntityNode_GuardZone > s_xList;
};

//////////////////////////////////////////////////////////////////////////////
//
// Tools_GuardZone_VolumeEdit_Controller
// For dealing with editing the volumes attached to the trigger
//
//////////////////////////////////////////////////////////////////////////////

class Tools_GuardZone_VolumeEdit_Controller : public Tools_VolumeEdit_Controller
{
    typedef Tools_VolumeEdit_Controller PARENT;

public :
	Tools_GuardZone_VolumeEdit_Controller( Tools_EntityNode_GuardZone* pxToolsEntity );

	virtual void StartVolumeEditing();
	virtual void EndVolumeEditing();

protected :
	virtual bool CanCreateVolume() { return true; }
	virtual bool CanDeleteVolume() { return GetNumberOfVolumes() > 1; }

	//Gets the current number of volumes
	virtual int GetNumberOfVolumes() { return m_pxAsuraEntity->m_iNumVolumes; }

	//Gets a pointer to the bounding box for a given volume
	virtual Asura_Bounding_Box* GetVolumeBox( int iIndex ) { return &( m_pxAsuraEntity->m_pxVolumes[ iIndex ].m_xBox ); }

	//Gets a pointer to the orientation for a given volume. Returns 0 if there is none
	virtual Asura_Quat* GetVolumeOrient( int iIndex ) { return &( m_pxAsuraEntity->m_pxVolumes[ iIndex ].m_xOrient ); }

	//Gets a default position for volume objects that have yet to gain a proper volume
	virtual Asura_Vector_3 GetDefaultPosition() { return m_pxToolsEntity->Position; }

	//Gets the name of the owner , to use for part of the volume names
	virtual const char* GetName() { return m_pxToolsEntity->GetName(); }

	//Creates the new volume in the asura data
	virtual void CreateVolumeMain();

	//Deletes the volume from the asura data
	virtual void DeleteVolumeMain( int iIndex );

private :
	Tools_EntityNode_GuardZone*	m_pxToolsEntity;
	Asura_ServerNode_GuardZone* m_pxAsuraEntity;
};


//////////////////////////////////////////////////////////////////////////////
//
// Tools_Object_GuardZone_ChooseGuardZoneObject
// Accepts any guardzone object.
//
//////////////////////////////////////////////////////////////////////////////

class Tools_Object_GuardZone_ChooseGuardZoneObject : public Tools_ChooseObject
{
public :
	Tools_Object_GuardZone_ChooseGuardZoneObject( Asura_Guid* puGuidToSet, bool* bChanged = NULL )
		: m_puGuidToSet( puGuidToSet ), m_bChanged( bChanged )
	{}
	virtual bool CanBeSelected( Tools_Object* pxObject );
	virtual void ObjectSelected( Tools_Object* pxObject );
	virtual void OnFinish() { delete this; }

private:
	Asura_Guid* m_puGuidToSet;
	bool*       m_bChanged;
};

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

#endif // #ifndef TOOLS_ENTITYNODE_GUARDZONE_H___
#endif //ASURA_USE_AXON
