//////////////////////////////////////////////////////////////////////////////
//
// Tools_EntityNode_GS2Attractor.cpp
//
// Copyright (C) 2008 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"

#ifdef ASURA_USE_AXON

// header
#include "Entity/Locations/OrientedNode/GamesceneNode/AtttactorNode/Tools_EntityNode_GS2Attractor.h"

// asura
#include "Camera/Asura_Camera.h"
#include "Entity/PhysicalObject/Asura_ServerEntity_PhysicalObject.h"
#include "Entity/Locations/OrientedNode/GamesceneNode/AttractorNode/Asura_ServerNode_GS2Attractor.h"
#include "Network/Asura_Message.h"
#include "Rendering/Asura_Render.h"

// axon
#include "AnimTemplates/Axon_Gamescene_AnimTemplate_System.h"
#include "Behaviour/Axon_BehaviourParams.h"
#include "Behaviour/BehaviourTypes/Axon_Behaviour_Types.h"
#include "Core/Axon_Messages.h"

// tools
#include "Entity/Tools_Entity_Data.h"
#include "Load&Save/Tools_ModeDataFile.h"
#include "Load&Save/Tools_SaveAsura.h"
#include "Misc/Tools_Timed_Messages.h"
#include "Objects/Axon/Tools_Object_Axon_ChooseObjects.h"
#include "Objects/Axon/Tools_Object_Axon_Interface.h"

//////////////////////////////////////////////////////////////////////////////
//
// Tools_EntityNode_GS2Attractor
//
//////////////////////////////////////////////////////////////////////////////

TOOLS_DECLARE_SHARED_PARAM( bool,         0xbf5bf75b, GS2Attr_UseAttractRange );     // Asura_GetHashID( "GS2Attr_UseAttractRange" )
TOOLS_DECLARE_SHARED_PARAM( float,        0x46dcf864, GS2Attr_AttractRange );        // Asura_GetHashID( "GS2Attr_AttractRange" )
TOOLS_DECLARE_SHARED_PARAM( Asura_String, 0xd291bd04, GS2Attr_AttractTypeString );         // Asura_GetHashID( "GS2Attr_AttractTypeString" )
TOOLS_DECLARE_SHARED_PARAM( int,		  0x965c97dc, GS2Attr_AttractTypeInt );            // Asura_GetHashID( "GS2Attr_AttractTypeInt" )
TOOLS_DECLARE_SHARED_PARAM( Asura_String, 0x1382378e, GS2Attr_EnRouteBehaviourSetString ); // Asura_GetHashID( "GS2Attr_EnRouteBehaviourSetString" )
TOOLS_DECLARE_SHARED_PARAM( int,          0x85af3992, GS2Attr_EnRouteBehaviourSetInt );    // Asura_GetHashID( "GS2Attr_EnRouteBehaviourSetInt" )
TOOLS_DECLARE_SHARED_PARAM( Asura_String, uTOOLS_SHAREDPARAMHASH_ENTITYNODE_SETTHISTODEFAULTS, EntityNode_SetThisToDefaults );

Tools_EntityNode_GS2Attractor* Tools_EntityNode_GS2Attractor::s_pxThisToolsAttractor = NULL;
const char*					   Tools_EntityNode_GS2Attractor::s_aszBehaviourStringList[ MAX_NUM_BEHAVIOUR_STRINGS ];
int							   Tools_EntityNode_GS2Attractor::s_iNumListEntries = 0;

Asura_Hashed_List< Tools_EntityNode_GS2Attractor > Tools_EntityNode_GS2Attractor::s_xList;
const char* Tools_EntityNode_GS2Attractor::s_aszTabNames[ TAB_END_ENUM ] =
{
	"Properties",
	"En-Route Behaviour",
	"At Node Behaviour"
};

u_int Tools_EntityNode_GS2Attractor::s_uCurrentTab = Tools_EntityNode_GS2Attractor::TAB_PROPERTIES;

Asura_Vector_2 Tools_EntityNode_GS2Attractor::s_xDefaultsOverlayPosition = Asura_Vector_2( 10.0f, 10.0f );
bool           Tools_EntityNode_GS2Attractor::s_bRefreshDefaultsOverlay = false;
bool           Tools_EntityNode_GS2Attractor::s_bDefaultUseAttractRange = true;
float          Tools_EntityNode_GS2Attractor::s_fDefaultAttractRange = 10.0f;
int	           Tools_EntityNode_GS2Attractor::s_iDefaultAttractType = AXON_GS2_AAT_ANYTHING;
Asura_Guid     Tools_EntityNode_GS2Attractor::s_uDefaultSpecificEntityGuid = ASURA_GUID_UNREGISTERED;
int	           Tools_EntityNode_GS2Attractor::s_iDefaultEnRouteBehaviourSet = AXON_BEHAVIOUR_TYPE_GS2_ATTRACTOR_EN_ROUTE;

Tools_EntityNode_GS2Attractor::Tools_EntityNode_GS2Attractor( Asura_Entity* pxEntity )
: PARENT( pxEntity )
, m_bUseAttractRange( false )
, m_bChanged( false )
{
	Type = TOT_Entity_GS2Attractor;

	Asura_ServerNode_GS2Attractor* pxGameAttractor = GetGameGS2Attr();
	if ( GetGameGS2Attr()->GetEnRouteBehaviourParams() )
	{
		GetGameGS2Attr()->GetEnRouteBehaviourParams()->DoSetToolsNode( this );
	}
	m_bUseAttractRange = ( pxGameAttractor->GetAttractRange() > 0.0f );

	SetMesh( "ToolsShape_GSAttractor" );

	s_xList.AddToList( this );
}

Tools_EntityNode_GS2Attractor::~Tools_EntityNode_GS2Attractor()
{
	if ( GetGameGS2Attr() && GetGameGS2Attr()->GetEnRouteBehaviourParams() )
	{
		GetGameGS2Attr()->GetEnRouteBehaviourParams()->DoSetToolsNode( NULL );
	}

	s_xList.RemoveFromList( this );
}

bool Tools_EntityNode_GS2Attractor::IsBaseOrDerivativeOf( int iToolsObjectType ) const
{
	if ( iToolsObjectType == TOT_Entity_GS2Attractor )
	{
		return true;
	}
	else
	{
		return PARENT::IsBaseOrDerivativeOf( iToolsObjectType );
	}
}

void Tools_EntityNode_GS2Attractor::AddOverlayControls( Asura_Overlay* pxNewOverlay, Asura_Vector_2& xOffset )
{
	Tools_EntityNode_OrientedNode::AddOverlayControls( pxNewOverlay, xOffset );

	xOffset.y += Asura_Dialog::LargeSpacing() - Asura_Dialog::DefaultSpacing();

	m_pxTabsDialogue = new Asura_Tabs_Dialog( xOffset, s_aszTabNames, sizeof( s_aszTabNames ) / sizeof( s_aszTabNames[ 0 ] ) );
	pxNewOverlay->AddDialog( m_pxTabsDialogue );

	m_pxTabsDialogue->SelectTab( TAB_PROPERTIES );
	xOffset.y = m_pxTabsDialogue->m_xMaxCoord.y + 0.5f;
	float fTabsEndY = xOffset.y;

	AddPropertiesTabControls( m_pxTabsDialogue, xOffset );
	fTabsEndY = Asura_Maths::Max( fTabsEndY, xOffset.y );

	m_pxTabsDialogue->SelectTab( TAB_EN_ROUTE_BEHAVIOUR );
	xOffset.y = m_pxTabsDialogue->m_xMaxCoord.y + 0.5f;
	AddEnRouteBehaviourTabControls( m_pxTabsDialogue, xOffset );
	fTabsEndY = Asura_Maths::Max( fTabsEndY, xOffset.y );

	m_pxTabsDialogue->SelectTab( TAB_AT_NODE_BEHAVIOUR );
	xOffset.y = m_pxTabsDialogue->m_xMaxCoord.y + 0.5f;
	AddBehaviourTabControls( m_pxTabsDialogue, xOffset );
	fTabsEndY = Asura_Maths::Max( fTabsEndY, xOffset.y );

	xOffset.y = fTabsEndY + Asura_Dialog::DefaultSpacing();
	AddUnderTabsControls( pxNewOverlay, xOffset );

	m_pxTabsDialogue->SetSelectedTabPointer( &s_uCurrentTab );
}

void Tools_EntityNode_GS2Attractor::OnSelection()
{
	s_pxThisToolsAttractor = this;
	PARENT::OnSelection();
}

void Tools_EntityNode_GS2Attractor::OnDeselection()
{
	PARENT::OnDeselection();
	s_pxThisToolsAttractor = NULL;
}

bool Tools_EntityNode_GS2Attractor::HandlesThisMessage( int iMessageID )
{
	if ( GetGameGS2Attr()->GetEnRouteBehaviourParams() && GetGameGS2Attr()->GetEnRouteBehaviourParams()->DoHandlesThisMessage( iMessageID ) )
	{
		return true;
	}
	else
	{
		return PARENT::HandlesThisMessage( iMessageID );
	}
}

bool Tools_EntityNode_GS2Attractor::OwnsThisMessageBlock( unsigned short usMessageBlockID )
{
	if ( GetGameGS2Attr()->GetEnRouteBehaviourParams() && GetGameGS2Attr()->GetEnRouteBehaviourParams()->DoOwnsThisMessageBlock( usMessageBlockID ) )
	{
		return true;
	}

	return PARENT::OwnsThisMessageBlock( usMessageBlockID );
}

void Tools_EntityNode_GS2Attractor::Render()
{
	extern void Render_Sphere( const Asura_Vector_3& xCentre, float fRadius, int iColour );

	PARENT::Render();

	if ( GetGameGS2Attr()->GetEnRouteBehaviourParams() )
	{
		GetGameGS2Attr()->GetEnRouteBehaviourParams()->DoRender();
	}

	if ( IsGroupSelected() && m_bUseAttractRange )
	{
		Render_Sphere( Position, GetGameGS2Attr()->GetAttractRange(), 0xff00ff00 );
	}
}

void Tools_EntityNode_GS2Attractor::Update()
{
	PARENT::Update();

	if ( GetGameGS2Attr()->GetEnRouteBehaviourParams() )
	{
		GetGameGS2Attr()->GetEnRouteBehaviourParams()->DoUpdate();
	}

	if ( s_bRefreshDefaultsOverlay )
	{
		s_bRefreshDefaultsOverlay = false;
		RefreshDefaultsOverlay();
	}
}

void Tools_EntityNode_GS2Attractor::UpdateWhileSelected()
{
	PARENT::UpdateWhileSelected();

	if ( GetGameGS2Attr()->GetEnRouteBehaviourParams() )
	{
		GetGameGS2Attr()->GetEnRouteBehaviourParams()->DoUpdateWhileSelected();
	}
}

void Tools_EntityNode_GS2Attractor::AddFilesToExportList( List< Asura_String >& xNameList ) const
{
	PARENT::AddFilesToExportList( xNameList );

	if ( GetGameGS2Attr()->GetEnRouteBehaviourParams() )
	{
		GetGameGS2Attr()->GetEnRouteBehaviourParams()->DoAddFilesToExportList( xNameList );
	}
}

bool Tools_EntityNode_GS2Attractor::GetParameter( Asura_Hash_ID uParamID, Tools_Parameter* pxParam ) const
{
	if ( GetGameGS2Attr()->GetEnRouteBehaviourParams() && GetGameGS2Attr()->GetEnRouteBehaviourParams()->DoGetParameter( uParamID, pxParam ) )
	{
		return true;
	}

	switch ( uParamID )
	{
		case g_uHash_GS2Attr_UseAttractRange:
			pxParam->SetBool( m_bUseAttractRange );
			return true;

		case g_uHash_GS2Attr_AttractRange:
			pxParam->SetFloat( GetGameGS2Attr()->GetAttractRange() );
			return true;

		case g_uHash_GS2Attr_AttractTypeString:
		{
			ListSelection_Overlay xAttractTypeList( "" );
			Tools_Object_Axon_Interface::GetInstance()->BuildGS2AttractorAttractTypesList( xAttractTypeList, s_pxThisToolsAttractor );
			pxParam->SetString( xAttractTypeList.GetItemNameByData( GetGameGS2Attr()->GetAttractType() ) );
			return true;
		}

		case g_uHash_GS2Attr_AttractTypeInt:
			// This is only ever set, not got, because of the way shared function dialogues work.
			return true;

		case g_uHash_GS2Attr_EnRouteBehaviourSetString:
		{
			ListSelection_Overlay xBehaviourList( "" );
			Tools_Object_Axon_Interface::GetInstance()->BuildGS2AttractorEnRouteBehaviourSetList( xBehaviourList, s_pxThisToolsAttractor );
			pxParam->SetString( xBehaviourList.GetItemNameByData( GetGameGS2Attr()->GetEnRouteBehaviourSet() ) );
			return true;
		}

		case g_uHash_GS2Attr_EnRouteBehaviourSetInt:
			// This is only ever set, not got, because of the way shared function dialogues work.
			return true;

		default:
			return PARENT::GetParameter( uParamID, pxParam );
	}
}

bool Tools_EntityNode_GS2Attractor::SetParameter( Asura_Hash_ID uParamID, const Tools_Parameter* pxParam )
{
	if ( GetGameGS2Attr()->GetEnRouteBehaviourParams() && GetGameGS2Attr()->GetEnRouteBehaviourParams()->DoSetParameter( uParamID, pxParam ) )
	{
		return true;
	}

	switch ( uParamID )
	{
		case g_uHash_GS2Attr_UseAttractRange:
		{
			m_bUseAttractRange = pxParam->GetBool();
			if ( !m_bUseAttractRange )
			{
				GetGameGS2Attr()->SetAttractRange( 0.0f );
			}
			SetEntityModified();
			return true;
		}

		case g_uHash_GS2Attr_AttractRange:
		{
			GetGameGS2Attr()->SetAttractRange( pxParam->GetFloat() );
			SetEntityModified();
			return true;
		}

		case g_uHash_GS2Attr_AttractTypeString:
			// This is only ever got, not set, because of the way shared function dialogues work.
			return true;

		case g_uHash_GS2Attr_AttractTypeInt:
		{
			GetGameGS2Attr()->SetAttractType( pxParam->GetInt() );
			if ( IsSelected() )
			{
				RequestRefreshOverlay();
			}
			SetEntityModified();
			return true;
		}

		case g_uHash_GS2Attr_EnRouteBehaviourSetString:
			// This is only ever got, not set, because of the way shared function dialogues work.
			return true;

		case g_uHash_GS2Attr_EnRouteBehaviourSetInt:
		{
			GetGameGS2Attr()->SetEnRouteBehaviourSet( pxParam->GetInt() );
			if ( GetGameGS2Attr()->GetEnRouteBehaviourParams() )
			{
				GetGameGS2Attr()->GetEnRouteBehaviourParams()->DoSetToolsNode( this );
			}
			SetEntityModified();
			if ( IsSelected() )
			{
				RequestRefreshOverlay();
			}
			return true;
		}

		case g_uHash_EntityNode_SetThisToDefaults:
		{
			// It's pretty certain that the node defaults will be ok for us also. If this isn't the case we can split later.
			PARENT::SetParameter( uParamID, pxParam );

			m_bUseAttractRange = s_bDefaultUseAttractRange;
			if ( m_bUseAttractRange )
			{
				GetGameGS2Attr()->SetAttractRange( s_fDefaultAttractRange );
			}
			else
			{
				GetGameGS2Attr()->SetAttractRange( 0.0f );
			}
			GetGameGS2Attr()->SetAttractType( s_iDefaultAttractType );
			if ( GetGameGS2Attr()->GetEnRouteBehaviourSet() != s_iDefaultEnRouteBehaviourSet )
			{
				GetGameGS2Attr()->SetEnRouteBehaviourSet( s_iDefaultEnRouteBehaviourSet );
				if ( GetGameGS2Attr()->GetEnRouteBehaviourParams() )
				{
					GetGameGS2Attr()->GetEnRouteBehaviourParams()->DoSetToolsNode( this );
				}
			}
			SetEntityModified();
			if ( IsSelected() )
			{
				RequestRefreshOverlay();
			}
			return true;
		}

		default:
			return PARENT::SetParameter( uParamID, pxParam );
	}
}

bool Tools_EntityNode_GS2Attractor::Verify( const bool bAutomatic )
{
	bool bOk = PARENT::Verify( bAutomatic );

	if ( GetGameGS2Attr()->GetEnRouteBehaviourParams() )
	{
		bOk &= GetGameGS2Attr()->GetEnRouteBehaviourParams()->DoVerify( bAutomatic );
	}

	// Check our attract range is reasonable.
	if ( m_bUseAttractRange )
	{
		if ( GetGameGS2Attr()->GetAttractRange() < 1.0f )
		{
			bOk = false;
			Tools_MessageLog::AddMessage( Tools_MessageLog::VERIFICATION, "Gamescene", new Tools_MessageLogAction_GotoEntity( GetGuid() ),
				"GS2 attractor %u has a target range that is unexpectedly small.", GetGuid() );
		}
	}

	// Check our specific entity exists if necessary.
	if ( GetGameGS2Attr()->GetAttractType() == AXON_GS2_AAT_SPECIFIC_GUID )
	{
		if ( GetGameGS2Attr()->GetSpecificGuid() == ASURA_GUID_UNREGISTERED )
		{
			bOk = false;
			Tools_MessageLog::AddMessage( Tools_MessageLog::VERIFICATION, "Gamescene", new Tools_MessageLogAction_GotoEntity( GetGuid() ),
				"GS2 attractor %u is missing a specific entity to attract.", GetGuid() );
		}
		else if ( !Asura_ServerEntity_PhysicalObject::FindEntity( GetGameGS2Attr()->GetSpecificGuid() ) )
		{
			bOk = false;
			Tools_MessageLog::AddMessage( Tools_MessageLog::VERIFICATION, "Gamescene", new Tools_MessageLogAction_GotoEntity( GetGuid() ),
				"GS2 attractor %u cannot find specific entity %u to attract.", GetGuid(), GetGameGS2Attr()->GetSpecificGuid() );
		}
	}

	return bOk;
}

void Tools_EntityNode_GS2Attractor::Copy( const Tools_Object* pxObject )
{
	// Copy parent data.
	PARENT::Copy( pxObject );

	// Copy our data.
	const Tools_EntityNode_GS2Attractor* pxFromObject = Tools_EntityNode_GS2Attractor::FindEntity( pxObject->GetGuid() );
	m_bUseAttractRange = pxFromObject->m_bUseAttractRange;

	// Copy the engine entity's data across.
	Asura_ServerNode_GS2Attractor* pxToEntity   = Asura_ServerNode_GS2Attractor::FindEntity( Entity->GetGuid() );
	Asura_ServerNode_GS2Attractor* pxFromEntity = Asura_ServerNode_GS2Attractor::FindEntity( pxFromObject->Entity->GetGuid() );
	pxToEntity->m_fPulseTimer				= pxFromEntity->m_fPulseTimer;
	pxToEntity->m_fAttractRange				= pxFromEntity->m_fAttractRange;
	pxToEntity->m_iAttractType				= pxFromEntity->m_iAttractType;
	pxToEntity->m_uSpecificGuid				= pxFromEntity->m_uSpecificGuid;
	pxToEntity->m_uAttractedGuid			= pxFromEntity->m_uAttractedGuid;
	pxToEntity->m_uGuidMask					= pxFromEntity->m_uGuidMask;
	pxToEntity->m_iEnRouteBehaviourSet		= pxFromEntity->m_iEnRouteBehaviourSet;
	delete pxToEntity->m_pxEnRouteBehaviourParams;
	pxToEntity->m_pxEnRouteBehaviourParams	= pxFromEntity->m_pxEnRouteBehaviourParams ? pxFromEntity->m_pxEnRouteBehaviourParams->DoClone() : NULL;
	if ( pxToEntity->m_pxEnRouteBehaviourParams )
	{
		pxToEntity->m_pxEnRouteBehaviourParams->DoSetToolsNode( this );
	}
}

Tools_Object* Tools_EntityNode_GS2Attractor::CreateObjectOfThisType() const
{
	return CreateAndReturn();
}

Asura_Guid Tools_EntityNode_GS2Attractor::GetOwnerGuidFromSettings() const
{
	return GetGameGS2Attr()->GetSpecificGuid();
}

void Tools_EntityNode_GS2Attractor::TransferToolsModeData( Tools_ModeDataFile& xFile )
{
	u_int uVersion = uCURRENT_TOOLS_DATA_VERSION;
	xFile.Transfer( uVersion );

	// No need to do the parent here, it's called separately.

	if ( uVersion == uCURRENT_TOOLS_DATA_VERSION )
	{
		xFile.Transfer( s_xDefaultsOverlayPosition );
		xFile.Transfer( s_bDefaultUseAttractRange );
		xFile.Transfer( s_fDefaultAttractRange );
		xFile.Transfer( s_iDefaultAttractType );
		xFile.Transfer( s_uDefaultSpecificEntityGuid );
		xFile.Transfer( s_iDefaultEnRouteBehaviourSet );
	}
}

void Tools_EntityNode_GS2Attractor::Create()
{
	CreateAndReturn();
}

Tools_EntityNode_GS2Attractor* Tools_EntityNode_GS2Attractor::CreateAndReturn()
{
	// Get a new GUID
	const Asura_Guid uGuid = Asura_Entity_System::GetCreatedByToolsGuid();

	// Set up tools data and auto-create a unique name (prompt for new name if there is a clash).
	Tools_Entity_Data* pxToolsData = CreateToolsDataWithDefaultName( "GS2 Attractor", "GS2 Attractor", uGuid, GetNamePrefix() );
	if ( !pxToolsData ) { return NULL; }

	// Create the game specfic server object and set its data.
	Asura_ServerNode_GS2Attractor* pxGameAttractor = new Asura_ServerNode_GS2Attractor( uGuid );
	if ( !pxGameAttractor ) { return NULL; }

	pxGameAttractor->ToolsData = pxToolsData;

	// Create the game specific tools object and select it.
	Tools_EntityNode_GS2Attractor* pxToolsAttractor = new Tools_EntityNode_GS2Attractor( pxGameAttractor );
	if ( !pxToolsAttractor ) { return NULL; }

	// Set everything to the default values.
	pxToolsAttractor->SetThisToDefaults();

	Tools_Object_System::SelectObject( pxToolsAttractor );

	pxToolsAttractor->Position = *Asura_Camera::GetPosition();
	// Set the orient to the camera heading but vertical.
	Asura_Euler_3 xHeading = *Asura_Camera::GetOriEuler();
	xHeading.x = xHeading.z = 0.0f;
	xHeading.ToMatrix( pxToolsAttractor->Orientation );

	pxGameAttractor->SetPosition( pxToolsAttractor->Position );
	pxGameAttractor->SetOrientation( pxToolsAttractor->Orientation );

	Tools_Timed_Message_System::AddMessage( Tools_Timed_Message_System::COLOUR_DEFAULT, "Created new GS2 attractor." );
	Tools_SaveAsura::SetEntityModified();

	return pxToolsAttractor;
}

void Tools_EntityNode_GS2Attractor::AddPropertiesTabControls( Asura_Tabs_Dialog* pxTabsDialogue, Asura_Vector_2& xOffset )
{
	if ( Type == TOT_Entity_GS2Attractor )
	{
		AddStructureTypeDialogue( pxTabsDialogue, xOffset );
		AddSmoothOverNodeOverlayControls( pxTabsDialogue, xOffset );
		AddFollowSurfaceDialogue( pxTabsDialogue, xOffset );
		AddCollisionsDialogues( pxTabsDialogue, xOffset );
		AddAttractParamsDialogs( pxTabsDialogue, xOffset );
		xOffset.y += Asura_Dialog::LargeSpacing() - Asura_Dialog::DefaultSpacing();

		AddLinkWeightsDialogue( pxTabsDialogue, xOffset );
		xOffset.y += Asura_Dialog::LargeSpacing() - Asura_Dialog::DefaultSpacing();
	}
}

void Tools_EntityNode_GS2Attractor::AddEnRouteBehaviourTabControls( Asura_Tabs_Dialog* pxTabsDialogue, Asura_Vector_2& xOffset )
{
	Asura_Function_Dialog* pxFDialog = new Asura_Function_Dialog( "Behaviour: ", &g_xParam_GS2Attr_EnRouteBehaviourSetString, xOffset, PickEnRouteBehaviourSet );
	pxFDialog->SetHelpText( "Selects the behaviour for the actor to use whilst getting to this attractor." );
	pxTabsDialogue->AddDialog( pxFDialog );
	xOffset.y += Asura_Dialog::LargeSpacing();

	if ( GetGameGS2Attr()->GetEnRouteBehaviourParams() )
	{
		GetGameGS2Attr()->GetEnRouteBehaviourParams()->DoAddOverlayControls( pxTabsDialogue, xOffset );
	}
}

void Tools_EntityNode_GS2Attractor::AddUnderTabsControls( Asura_Overlay* pxNewOverlay, Asura_Vector_2& xOffset )
{
	if ( Type == TOT_Entity_GS2Attractor )
	{
		AddLinkControls( pxNewOverlay, xOffset );
		xOffset.y += Asura_Dialog::LargeSpacing() - Asura_Dialog::DefaultSpacing();

		AddMainDefaultsOverlayControls( pxNewOverlay, xOffset );
		Tools_Object_Axon_Interface::AddPreviewControls( pxNewOverlay, xOffset );
	}
}

void Tools_EntityNode_GS2Attractor::AddAttractParamsDialogs( Asura_Tabs_Dialog* pxTabsDialogue, Asura_Vector_2& xOffset )
{
	AddAttractRangeDialogs( pxTabsDialogue, xOffset );
	AddAttractTypeDialogs( pxTabsDialogue, xOffset );
}

void Tools_EntityNode_GS2Attractor::AddAttractRangeDialogs( Asura_Tabs_Dialog* pxTabsDialogue, Asura_Vector_2& xOffset )
{
	Asura_Dialog* pxDialogue = new Asura_SharedTickbox_Dialog( "Use Attract Range:", xOffset, &g_xParam_GS2Attr_UseAttractRange );
	pxDialogue->SetHelpText( "Whether to use a range within which to attract entities." );
	pxTabsDialogue->AddDialog( pxDialogue );
	xOffset.x += 15.0f;

	pxDialogue = new Asura_Numerical_Dialog( "", xOffset, &g_xParam_GS2Attr_AttractRange, 0.0f, 10000.0f, 0.25f, "%0.2f" );
	pxDialogue->SetHelpText( "The range within which to attract entities." );
	pxDialogue->SetIsEnabledFunction( IsUsingAttractRange, reinterpret_cast< int >( this ) );
	pxTabsDialogue->AddDialog( pxDialogue );
	xOffset.x -= 15.0f;
	xOffset.y += Asura_Dialog::DefaultSpacing();
}

void Tools_EntityNode_GS2Attractor::AddAttractTypeDialogs( Asura_Tabs_Dialog* pxTabsDialogue, Asura_Vector_2& xOffset )
{
	Asura_Dialog* pxDialogue = new Asura_Function_Dialog( "Attract Type: ", &g_xParam_GS2Attr_AttractTypeString, xOffset, PickAttractType );
	pxDialogue->SetHelpText( "The type of thing to attract." );
	pxTabsDialogue->AddDialog( pxDialogue );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	Asura_ServerNode_GS2Attractor* pxGameAttractor = GetGameGS2Attr();

	if ( pxGameAttractor->GetAttractType() == AXON_GS2_AAT_SPECIFIC_GUID )
	{
		Asura_String xEntityString( "Specific Entity: " );
		if ( pxGameAttractor->GetSpecificGuid() == ASURA_GUID_UNREGISTERED )
		{
			xEntityString += "*none*";
		}
		else
		{
			const Asura_ServerEntity_PhysicalObject* pxPhyObj = Asura_ServerEntity_PhysicalObject::FindEntity( pxGameAttractor->GetSpecificGuid() );
			if ( pxPhyObj )
			{
				xEntityString += pxPhyObj->ToolsData->Name;
			}
			else
			{
				xEntityString += "NOT FOUND";
			}
		}

		pxDialogue = new Asura_Function_Dialog( xEntityString.GetString(), xOffset, SetSpecificEntity, &Changed );
		pxTabsDialogue->AddDialog( pxDialogue );
		xOffset.y += Asura_Dialog::DefaultSpacing();
	}
}

void Tools_EntityNode_GS2Attractor::SetDefaultsToThis()
{
	// It's pretty certain that the node defaults will be ok for us. If this isn't the case we can split later.
	PARENT::SetDefaultsToThis();

    s_bDefaultUseAttractRange     = m_bUseAttractRange;
    s_fDefaultAttractRange        = GetGameGS2Attr()->GetAttractRange();
    s_iDefaultAttractType         = GetGameGS2Attr()->GetAttractType();
    s_uDefaultSpecificEntityGuid  = GetGameGS2Attr()->GetSpecificGuid();
	s_iDefaultEnRouteBehaviourSet = GetGameGS2Attr()->GetEnRouteBehaviourSet();
}

void Tools_EntityNode_GS2Attractor::AddPropertiesDefaultControls( Asura_Overlay* pxNewOverlay, Asura_Vector_2& xOffset )
{
	PARENT::AddPropertiesDefaultControls( pxNewOverlay, xOffset );
	xOffset.y += Asura_Dialog::LargeSpacing() - Asura_Dialog::DefaultSpacing();

	Asura_Dialog* pxDialogue = new Asura_Tickbox_Dialog( "Use Attract Range:", xOffset, &s_bDefaultUseAttractRange );
	pxDialogue->SetHelpText( "Whether to use a range within which to attract entities." );
	pxNewOverlay->AddDialog( pxDialogue );
	xOffset.x += 15.0f;

	pxDialogue = new Asura_Numerical_Dialog( "", xOffset, &s_fDefaultAttractRange, 0.0f, 10000.0f, 0.25f, "%0.2f" );
	pxDialogue->SetHelpText( "The range within which to attract entities." );
	pxDialogue->SetIsEnabledFunction( IsUsingDefaultAttractRange );
	pxNewOverlay->AddDialog( pxDialogue );
	xOffset.x -= 15.0f;
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialogue = new Asura_Function_Dialog( "Attract Type:", xOffset, PickDefaultAttractType );
	pxDialogue->SetHelpText( "The type of thing to attract." );
	pxNewOverlay->AddDialog( pxDialogue );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	if ( s_iDefaultAttractType == AXON_GS2_AAT_SPECIFIC_GUID )
	{
		Asura_String xEntityString( "Specific Entity: " );
		if ( s_uDefaultSpecificEntityGuid == ASURA_GUID_UNREGISTERED )
		{
			xEntityString += "*none*";
		}
		else
		{
			const Asura_ServerEntity_PhysicalObject* pxPhyObj = Asura_ServerEntity_PhysicalObject::FindEntity( s_uDefaultSpecificEntityGuid );
			if ( pxPhyObj )
			{
				xEntityString += pxPhyObj->ToolsData->Name;
			}
			else
			{
				xEntityString += "NOT FOUND";
			}
		}

		pxDialogue = new Asura_Function_Dialog( xEntityString.GetString(), xOffset, SetDefaultSpecificEntity, &Changed );
		pxNewOverlay->AddDialog( pxDialogue );
		xOffset.y += Asura_Dialog::DefaultSpacing();
	}
}

void Tools_EntityNode_GS2Attractor::AddBehaviourDefaultControls( Asura_Overlay* pxNewOverlay, Asura_Vector_2& xOffset )
{
	ListSelection_Overlay xBehaviourList( "" );
	Tools_Object_Axon_Interface::GetInstance()->BuildGS2AttractorEnRouteBehaviourSetList( xBehaviourList );
	Asura_String xBehaviourSetString = "En-Route Behaviour: ";
	xBehaviourSetString += xBehaviourList.GetItemNameByData( s_iDefaultEnRouteBehaviourSet );

	Asura_Dialog* pxDialog = new Asura_Function_Dialog( xBehaviourSetString.GetString(), xOffset, PickDefaultEnRouteBehaviourSet, reinterpret_cast< int >( this ) );
	pxDialog->SetHelpText( "Selects the behaviour for the actor to use whilst getting to this attractor." );
	pxNewOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::LargeSpacing();

	PARENT::AddBehaviourDefaultControls( pxNewOverlay, xOffset );
}

void Tools_EntityNode_GS2Attractor::PickAttractType()
{
	ListSelection_Overlay xAttractTypeList( "Pick Attract Type:" );
	Tools_Object_Axon_Interface::GetInstance()->BuildGS2AttractorAttractTypesList( xAttractTypeList, s_pxThisToolsAttractor );
	if ( !xAttractTypeList.DoModal() )
	{
		return;
	}

	// Setting this will cause SetParameter to be called with this parameter hash on all selected objects.
	g_xParam_GS2Attr_AttractTypeInt.SetValue( xAttractTypeList.GetSelectionData() );
}

void Tools_EntityNode_GS2Attractor::PickEnRouteBehaviourSet()
{
	ListSelection_Overlay xBehaviourList( "Pick Behaviour To Use En-Route To This Attractor:" );
	Tools_Object_Axon_Interface::GetInstance()->BuildGS2AttractorEnRouteBehaviourSetList( xBehaviourList, s_pxThisToolsAttractor );
	if ( !xBehaviourList.DoModal() )
	{
		return;
	}

	// Warn if the behaviour is about to be reset.
	const int iNewBehaviourSet = xBehaviourList.GetSelectionData();
	if ( s_pxThisToolsAttractor->IsSelected() && Tools_Object_System::SelectedObjectList.size() == 1 &&
		iNewBehaviourSet == s_pxThisToolsAttractor->GetGameGS2Attr()->GetEnRouteBehaviourSet() && iNewBehaviourSet != AXON_BEHAVIOUR_TYPE_UNSET )
	{
		YesNo_Overlay xYNOverlay( "This will reset the current behaviour. Are you sure?" );
		if ( !xYNOverlay.DoModal() )
		{
			return;
		}
	}

	// Setting this will cause SetParameter to be called with this parameter hash on all selected objects.
	g_xParam_GS2Attr_EnRouteBehaviourSetInt.SetValue( iNewBehaviourSet );
}

Asura_ServerNode_GS2Attractor* Tools_EntityNode_GS2Attractor::GetGameGS2Attr() const
{
	return Asura_ServerNode_GS2Attractor::FindEntity( Entity->GetGuid() );
}

bool Tools_EntityNode_GS2Attractor::IsUsingAttractRange( int iThis )
{
	Tools_EntityNode_GS2Attractor* pxToolsAttractor = reinterpret_cast< Tools_EntityNode_GS2Attractor* >( iThis );
	return pxToolsAttractor->m_bUseAttractRange;
}

void Tools_EntityNode_GS2Attractor::SetSpecificEntity()
{
	Tools_ChooseObject_GS2AttractorTarget* pxChooseObject = new Tools_ChooseObject_GS2AttractorTarget( SetSpecificEntity );
	Tools_Object_System::ChooseObject( "Choose the entity you wish to attract.", pxChooseObject );
}

void Tools_EntityNode_GS2Attractor::SetSpecificEntity( Asura_Guid uGuid )
{
	s_pxThisToolsAttractor->GetGameGS2Attr()->SetSpecificGuid( uGuid );
	s_pxThisToolsAttractor->Changed = true;
	s_pxThisToolsAttractor->RefreshOverlay();
}

void Tools_EntityNode_GS2Attractor::DefaultAttractTypeChanged()
{
	s_bRefreshDefaultsOverlay = true;
}

void Tools_EntityNode_GS2Attractor::SetDefaultSpecificEntity()
{
	Tools_ChooseObject_GS2AttractorTarget* pxChooseObject = new Tools_ChooseObject_GS2AttractorTarget( SetDefaultSpecificEntity );
	Tools_Object_System::ChooseObject( "Choose the entity you wish to attract.", pxChooseObject );
}

void Tools_EntityNode_GS2Attractor::SetDefaultSpecificEntity( Asura_Guid uGuid )
{
	s_uDefaultSpecificEntityGuid = uGuid;
	s_bRefreshDefaultsOverlay = true;
}

void Tools_EntityNode_GS2Attractor::PickDefaultAttractType()
{
	ListSelection_Overlay xAttractTypeList( "Pick Attract Type:" );
	Tools_Object_Axon_Interface::GetInstance()->BuildGS2AttractorAttractTypesList( xAttractTypeList, s_pxThisToolsAttractor );
	if ( !xAttractTypeList.DoModal() )
	{
		s_iDefaultAttractType = xAttractTypeList.GetSelectionData();
		s_bRefreshDefaultsOverlay = true;
	}
}

void Tools_EntityNode_GS2Attractor::PickDefaultEnRouteBehaviourSet( int iThis )
{
	ListSelection_Overlay xBehaviourList( "Pick Behaviour To Use En-Route To This Attractor:" );
	Tools_Object_Axon_Interface::GetInstance()->BuildGS2AttractorEnRouteBehaviourSetList( xBehaviourList );
	xBehaviourList.SetInitialSelection( s_iDefaultEnRouteBehaviourSet );
	if ( xBehaviourList.DoModal() )
	{
		s_iDefaultEnRouteBehaviourSet = xBehaviourList.GetSelectionData();
		s_bRefreshDefaultsOverlay = true;
	}
}

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

#endif // ASURA_USE_AXON
