//////////////////////////////////////////////////////////////////////////////
//
// Tools_EntityNode_LiftNode.cpp
//
// Copyright (c) 2006 Rebellion. All rights reserved.
//
// Description:
//
//     Node class for lift splines
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header.
#include "Entity/Locations/OrientedNode/LiftNode/Tools_EntityNode_LiftNode.h"

// asura
#include "Camera/Asura_Camera.h"
#include "Entity/Asura_Entity_Classification.h"
#include "Entity/Asura_Entity_Event.h"
#include "Entity/Asura_StaticMessage_System.h"
#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Overlay.h"
#include "Rendering/Asura_Render.h"

//Tools
#include "Entity/Locations/Spline/LiftSpline/Tools_EntityNode_LiftSpline.h"
#include "Entity/Tools_SecondaryEntityFile_System.h"
#include "Load&Save/Tools_SaveAsura.h"
#include "Misc/Tools_Timed_Messages.h"

TOOLS_DECLARE_SHARED_PARAM( bool, 0x7AB08764, LiftNode_StopHere );
TOOLS_DECLARE_SHARED_PARAM( bool, 0x3C4D6CB3, LiftNode_BlockWhenOccupied );
TOOLS_DECLARE_SHARED_PARAM( bool, 0x882BA10C, LiftNode_BlockWhenUnoccupied );

//////////////////////////////////////////////////////////////////////////////
//
// Tools_Lift_SelectLiftNode
//
//////////////////////////////////////////////////////////////////////////////

bool Tools_Lift_SelectLiftNode::CanBeSelected( Tools_Object* pxObject )
{
	if ( !pxObject )
	{
		return false;
	}

	if ( m_uIgnoreGuid != ASURA_GUID_UNREGISTERED && pxObject->GetGuid() == m_uIgnoreGuid )
	{
		return false;
	}

	return ( Asura_ServerNode_LiftNode::FindEntity( pxObject->GetGuid() ) != NULL );
}

//////////////////////////////////////////////////////////////////////////////
//
// Tools_Lift_SelectSplineEnd
//
//////////////////////////////////////////////////////////////////////////////

bool Tools_Lift_SelectSplineEnd::CanBeSelected( Tools_Object* pxObject )
{
	return ( Asura_ServerNode_LiftNode::FindEntity( pxObject->GetGuid() ) != NULL );
}

//////////////////////////////////////////////////////////////////////////////
//
// Tools_Lift_SelectGotoNode
//
//////////////////////////////////////////////////////////////////////////////

bool Tools_Lift_SelectGotoNode::CanBeSelected( Tools_Object* pxObject )
{
	return ( Asura_ServerNode_LiftNode::FindEntity( pxObject->GetGuid() ) != NULL );
}


//////////////////////////////////////////////////////////////////////////////
//
// Tools_EntityNode_LiftNode
//
//////////////////////////////////////////////////////////////////////////////

Tools_EntityNode_LiftNode* Tools_EntityNode_LiftNode::s_pxThisToolsLiftNode = NULL;

Tools_EntityNode_LiftNode::Tools_EntityNode_LiftNode(Asura_Entity* pxEntity)
: PARENT( pxEntity )
, m_bChanged( false )
, m_bStopHere( false )
, m_bBlockWhenOccupied( false )
, m_bBlockWhenUnoccupied( false )
, m_uSelectedSplineEndGuid( ASURA_GUID_UNREGISTERED )
{
	// set tools type and mesh
	Type = TOT_Entity_LiftNode;

    SetMesh( "ToolsShape_LiftNode" );

	Asura_ServerNode_LiftNode* pxGameNode = Asura_ServerNode_LiftNode::FindEntity( pxEntity->GetGuid() );
    if ( pxGameNode )
    {
        m_bStopHere            = pxGameNode->GetStopHere();
        m_bBlockWhenOccupied   = pxGameNode->GetBlockWhenOccupied();
		//m_bBlockWhenUnoccupied = pxGameNode->GetBlockWhenUnoccupied();
    }
}

void Tools_EntityNode_LiftNode::AddMessageOverlayControls(Asura_StaticMessage* pxMessage,class Asura_Overlay* pxOverlay,class Asura_Vector_2& xOffset)
{
	Asura_Assert(pxMessage->m_pData,"");
	Asura_Message_Lift_Node_SetFlags* pxLiftMessage = static_cast< Asura_Message_Lift_Node_SetFlags* >( pxMessage->m_pData );

	Asura_Dialog* pxDialog = 0;

	pxDialog = new Asura_FlagTickbox_Dialog("Set Stop Here On", xOffset, pxLiftMessage->GetFlagsPtr(), 1<<0);
	pxDialog->SetHelpText("turn Stop Here on for this lift");
	pxDialog->SetRadioButtonGroup(20, false);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_FlagTickbox_Dialog("Set Stop Here Off", xOffset, pxLiftMessage->GetFlagsPtr(), 1<<1);
	pxDialog->SetHelpText("turn Stop Here off for this lift");
	pxDialog->SetRadioButtonGroup(20, false);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing() * 1.5f;
}
bool Tools_EntityNode_LiftNode::HandlesThisEvent(int iEventID) const
{
	if (iEventID == ASURA_ENTITY_EVENT_ONARRIVEDAT
		|| iEventID == ASURA_ENTITY_EVENT_ONDEPARTEDFROM)
	{
		return true;
	}
	return PARENT::HandlesThisEvent(iEventID);
}

bool Tools_EntityNode_LiftNode::HandlesThisMessage(int iMessageID)
{
	if(iMessageID == ASURA_MSG_LIFT_NODE_SETFLAGS) return true;

	//Calling parent to avoid forcing projects to define this function for derived classes
	//(since it doesn't really matter if the parent version gets called twice)
	return PARENT::HandlesThisMessage(iMessageID);
}

void Tools_EntityNode_LiftNode::AddOverlayControls(Asura_Overlay* pxNewOverlay, Asura_Vector_2& xOffset)
{
	PARENT::AddOverlayControls(pxNewOverlay, xOffset);

	AddSmoothOverNodeOverlayControls( pxNewOverlay, xOffset );

	Asura_Dialog* pxNewDialog;

	// Stop here
	pxNewDialog = new Asura_SharedTickbox_Dialog( "Stop Here", xOffset, &g_xParam_LiftNode_StopHere );
    pxNewDialog->SetHelpText( "Whether or not lifts will deactive when they reach this node." );
    pxNewOverlay->AddDialog(pxNewDialog);
    xOffset.y += Asura_Dialog::DefaultSpacing();

	pxNewDialog = new Asura_SharedTickbox_Dialog( "Block Links When Occupied", xOffset, &g_xParam_LiftNode_BlockWhenOccupied );
	pxNewDialog->SetHelpText( "Whether or not lifts will block waypoint links when at this node." );
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	//pxNewDialog = new Asura_SharedTickbox_Dialog( "Block Links When Unoccupied", xOffset, &g_xParam_LiftNode_BlockWhenUnoccupied );
	//pxNewDialog->SetHelpText( "Whether or not lifts will block waypoint links when not at this node." );
	//pxNewOverlay->AddDialog(pxNewDialog);
	//xOffset.y += Asura_Dialog::DefaultSpacing();

	pxNewDialog = new Asura_Function_Dialog( "Create lift spline from this node", xOffset, SelectSplineEnd );
	pxNewDialog->SetHelpText( "Prompts for a target lift node to create a spline to." );
	pxNewOverlay->AddDialog( pxNewDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();
}

void Tools_EntityNode_LiftNode::SelectSplineEnd()
{
	if (!s_pxThisToolsLiftNode)
    {
        return;
    }
	Tools_Lift_SelectSplineEnd* pxChooseObject = new Tools_Lift_SelectSplineEnd( &s_pxThisToolsLiftNode->m_uSelectedSplineEndGuid );
    Tools_Object_System::ChooseObject( "Pick the node the spline will end on", pxChooseObject );
}

bool Tools_EntityNode_LiftNode::IsBaseOrDerivativeOf( int iToolsObjectType ) const
{
    if ( iToolsObjectType == TOT_Entity_LiftNode )
    {
        return true;
    }
    else
    {
        return PARENT::IsBaseOrDerivativeOf( iToolsObjectType );
    }
}

void Tools_EntityNode_LiftNode::OnSelection()
{
	PARENT::OnSelection();
	s_pxThisToolsLiftNode = this;
}

void Tools_EntityNode_LiftNode::OnDeselection()
{
	PARENT::OnDeselection();
	s_pxThisToolsLiftNode = NULL;
}

bool Tools_EntityNode_LiftNode::Verify( const bool bAutomatic )
{
	bool bOk = PARENT::Verify( bAutomatic );

	// Check 'smooth over node' is used correctly.
	if ( GetGameNode()->IsSmoothOverNode() )
	{
		// Check we only have one outgoing link.
		if ( GetGameNode()->GetNumberLinks() != 1 )
		{
			bOk = false;
			Tools_MessageLog::AddMessage( Tools_MessageLog::VERIFICATION, "Gamescene", new Tools_MessageLogAction_GotoEntity( GetGuid() ),
				"Lift node %u has 'smooth over node' ticked. This option can only be used with exactly one outgoing link.", GetGuid() );
		}
		// Check we only have one incoming link.
		u_int uIncomingLinkCount = 0;
		for ( Asura_Hashed_List_ConstIt< Tools_Entity_Node > xNodeIter( Tools_Entity_Node::GetList() ); !xNodeIter.Done(); xNodeIter.Next() )
		{
			const Tools_Entity_Node* pxNode = xNodeIter.GetCurrent();
			if ( !pxNode->IsBaseOrDerivativeOf( TOT_Entity_LiftSpline ) )
			{
				continue;
			}

			const Tools_EntityNode_LiftSpline* pxLiftSpline = static_cast< const Tools_EntityNode_LiftSpline* >( pxNode );
			if ( pxLiftSpline->GetGameLiftSpline()->GetTargetGuid() == GetGuid() )
			{
				++uIncomingLinkCount;
			}
			if ( uIncomingLinkCount > 1 )
			{
				break;
			}
		}
		if ( uIncomingLinkCount != 1 )
		{
			bOk = false;
			Tools_MessageLog::AddMessage( Tools_MessageLog::VERIFICATION, "Gamescene", new Tools_MessageLogAction_GotoEntity( GetGuid() ),
				"Lift node %u has 'smooth over node' ticked. This option can only be used with exactly one incoming link.", GetGuid() );
		}
	}

	return bOk;
}

void Tools_EntityNode_LiftNode::UpdateWhileSelected()
{
	if (m_uSelectedSplineEndGuid != ASURA_GUID_UNREGISTERED)
	{
		Tools_SecondaryEntityFile_System::PreCreateObject();
		Tools_EntityNode_LiftSpline::Create(GetGuid(), m_uSelectedSplineEndGuid);
		Tools_SecondaryEntityFile_System::PostCreateObject();
		UpdateAdjacentSplines();
		Tools_Entity_Node* pxEndNode = Tools_Entity_Node::FindEntity( m_uSelectedSplineEndGuid );
		if ( pxEndNode )
		{
			pxEndNode->UpdateAdjacentSplines();
		}
		m_uSelectedSplineEndGuid = ASURA_GUID_UNREGISTERED;
	}

	PARENT::UpdateWhileSelected();

	if ( m_bChanged )
	{
		m_bChanged = false;

        Asura_ServerNode_LiftNode* pxGameNode =
            static_cast< Asura_ServerNode_LiftNode* >( Asura_ServerEntity_Node::GetList()->FindEntity( GetGuid() ) );
		if ( pxGameNode )
		{
			pxGameNode->SetStopHere( m_bStopHere );
			pxGameNode->SetBlockWhenOccupied( m_bBlockWhenOccupied );
			//pxGameNode->SetBlockWhenUnoccupied( m_bBlockWhenUnoccupied );
			SetEntityModified();
		}
	}
}

void Tools_EntityNode_LiftNode::Copy( const Tools_Object* pxObject )
{
    // Copy parent data.
    PARENT::Copy( pxObject );

    // Copy our data.
    const Tools_EntityNode_LiftNode* pxFromNodeObject = static_cast< const Tools_EntityNode_LiftNode* >( pxObject );
    m_bChanged               = pxFromNodeObject->m_bChanged;
    m_bStopHere              = pxFromNodeObject->m_bStopHere;
	m_bBlockWhenOccupied	 = pxFromNodeObject->m_bBlockWhenOccupied;
	m_bBlockWhenUnoccupied   = pxFromNodeObject->m_bBlockWhenUnoccupied;
    m_uSelectedSplineEndGuid = pxFromNodeObject->m_uSelectedSplineEndGuid;

    // Copy the engine entity's data across.
    const Asura_ServerNode_LiftNode* pxFromEntity = static_cast< const Asura_ServerNode_LiftNode* >( pxFromNodeObject->Entity );
    Asura_ServerNode_LiftNode*       pxToEntity   = static_cast< Asura_ServerNode_LiftNode* >( Entity );
    pxToEntity->m_uLiftNodeFlags = pxFromEntity->m_uLiftNodeFlags;
}

Tools_Object* Tools_EntityNode_LiftNode::CreateObjectOfThisType() const
{
    return CreateAndReturn();
}

void Tools_EntityNode_LiftNode::Create()
{
    CreateAndReturn();
}

Tools_EntityNode_LiftNode* Tools_EntityNode_LiftNode::CreateAndReturn()
{
    // Get a new GUID
    const Asura_Guid uGuid = Asura_Entity_System::GetCreatedByToolsGuid();

    // Set up tools data and auto-create a unique name (prompt for new name if there is a clash).
    Tools_Entity_Data* pxToolsData = CreateToolsDataWithDefaultName("Lift Node","Lift Node",uGuid,GetNamePrefix());
    if ( !pxToolsData ) { return NULL; }

    // Create the game specfic server object and set its data.
    Asura_ServerNode_LiftNode* pxGameNode = new Asura_ServerNode_LiftNode( uGuid );
    if ( !pxGameNode ) { return NULL; }

    pxGameNode->ToolsData = pxToolsData;

    // Create the game specific tools object and select it.
    Tools_EntityNode_LiftNode* pxToolsNode = new Tools_EntityNode_LiftNode( pxGameNode );
    if ( !pxToolsNode ) { return NULL; }

    Tools_Object_System::SelectObject( pxToolsNode );

    pxToolsNode->Position = *Asura_Camera::GetPosition();
    pxToolsNode->Orientation = *Asura_Camera::GetOrientation();

    pxGameNode->SetPosition( pxToolsNode->Position );
    pxGameNode->SetOrientation( pxToolsNode->Orientation );

    Tools_Timed_Message_System::AddMessage( Tools_Timed_Message_System::COLOUR_DEFAULT, "Created new lift Node" );
    Tools_SaveAsura::SetEntityModified();

    return pxToolsNode;
}

bool Tools_EntityNode_LiftNode::GetParameter( Asura_Hash_ID uParamID, Tools_Parameter* pxParam ) const
{
	switch ( uParamID )
	{
		case g_uHash_LiftNode_StopHere:
		{
			pxParam->SetBool( m_bStopHere );
			return true;
		}

		case g_uHash_LiftNode_BlockWhenOccupied:
		{
			pxParam->SetBool( m_bBlockWhenOccupied );
			return true;
		}

		case g_uHash_LiftNode_BlockWhenUnoccupied:
		{
			pxParam->SetBool( m_bBlockWhenUnoccupied );
			return true;
		}

		default:
		{
			return PARENT::GetParameter( uParamID, pxParam );
		}
	}
}

bool Tools_EntityNode_LiftNode::SetParameter( Asura_Hash_ID uParamID, const Tools_Parameter* pxParam )
{
	switch ( uParamID )
	{
		case g_uHash_LiftNode_StopHere:
		{
			m_bStopHere = pxParam->GetBool();
			m_bChanged  = true;
			return true;
		}

		case g_uHash_LiftNode_BlockWhenOccupied:
		{
			m_bBlockWhenOccupied = pxParam->GetBool();
            m_bChanged           = true;
			return true;
		}

		case g_uHash_LiftNode_BlockWhenUnoccupied:
		{
			m_bBlockWhenUnoccupied = pxParam->GetBool();
            m_bChanged             = true;
			return true;
		}

		default:
		{
			return PARENT::SetParameter( uParamID, pxParam );
		}
	}
}
