//////////////////////////////////////////////////////////////////////////////
//
// Tools_EntityNode_OrientedNode.cpp
//
// Created by NJD - 12 October 2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"

// header
#include "Entity/Locations/OrientedNode/Tools_EntityNode_OrientedNode.h"

// asura
#include "Camera/Asura_Camera.h"
#include "Entity/Asura_Entity_Classification.h"
#include "Entity/Locations/OrientedNode/Asura_ServerNode_OrientedNode.h"
#include "Rendering/Asura_Render.h"

//tools
#include "Entity/Locations/GuardZone/Tools_EntityNode_GuardZone.h"
#include "Load&Save/Tools_SaveAsura.h"
#include "Misc/Tools_Timed_Messages.h"
#include "Objects/Tools_Object_System.h"

Tools_EntityNode_OrientedNode* Tools_EntityNode_OrientedNode::s_pxThisToolsOrientedNode = NULL;

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

Tools_EntityNode_OrientedNode::Tools_EntityNode_OrientedNode( Asura_Entity* pxEntity )
: PARENT( pxEntity )
#if defined ASURA_USE_AXON
, m_uMyToolsGZGuid( ASURA_GUID_UNREGISTERED )
#endif
{
    Type = TOT_Entity_OrientedNode;

	SetMesh( "ToolsShape_Guardpoint" );

    if ( Asura_ServerEntity_Node::GetList()->FindEntity( pxEntity->GetGuid() ) )
    {
        const Asura_ServerNode_OrientedNode* const pxGameNode = GetGameOrientedNode();
        Asura_Assert( pxGameNode->IsBaseOrDerivativeOf( AsuraEntityClass_OrientedNode ),
            "Tools_EntityNode_OrientedNode::Tools_EntityNode_OrientedNode - wrong entity type" );
        Position = pxGameNode->GetPosition();
        pxGameNode->GetOrientation( Orientation );
    }
}

void Tools_EntityNode_OrientedNode::Destroy()
{
#ifdef ASURA_USE_AXON
	//if we're part of a GuardZone inform the guardzone
	if ( m_uMyToolsGZGuid != ASURA_GUID_UNREGISTERED )
	{
		Tools_EntityNode_GuardZone* pxGZ = Tools_EntityNode_GuardZone::FindEntity( m_uMyToolsGZGuid );
		if ( pxGZ )
		{
			pxGZ->RemoveGuardPoint( GetGuid() );
		}

		m_uMyToolsGZGuid = ASURA_GUID_UNREGISTERED;
	}
#endif

	PARENT::Destroy();
}

void Tools_EntityNode_OrientedNode::UpdatePositionInAsuraData(bool bUpdateLinkedObjects)
{
    PARENT::UpdatePositionInAsuraData( bUpdateLinkedObjects );

	Asura_ServerNode_OrientedNode* pxGameOrientedNode = GetGameOrientedNode();
    if ( pxGameOrientedNode )
    {
        pxGameOrientedNode->SetOrientation( Orientation );
    }
}

bool Tools_EntityNode_OrientedNode::IsBaseOrDerivativeOf( int iToolsObjectType ) const
{
    if ( iToolsObjectType == TOT_Entity_OrientedNode )
    {
        return true;
    }
    else
    {
        return PARENT::IsBaseOrDerivativeOf( iToolsObjectType );
    }
}

void Tools_EntityNode_OrientedNode::OnSelection()
{
	PARENT::OnSelection();

	s_pxThisToolsOrientedNode = this;
}

void Tools_EntityNode_OrientedNode::OnDeselection()
{
	PARENT::OnDeselection();

	s_pxThisToolsOrientedNode = NULL;
}

void Tools_EntityNode_OrientedNode::Copy( const Tools_Object* pxObject )
{
    // Copy parent data.
    PARENT::Copy( pxObject );

    // Copy our data.

    // Copy the engine entity's data across.
    // We don't copy the orientation, as the copy is either to a new node, or over an existing node.
}

Tools_Object* Tools_EntityNode_OrientedNode::CreateObjectOfThisType() const
{
    return CreateAndReturn();
}

void Tools_EntityNode_OrientedNode::Create()
{
    CreateAndReturn();
}

#if defined ASURA_USE_AXON
void Tools_EntityNode_OrientedNode::CreateGuardPoint( Tools_EntityNode_OrientedNode** ppxNewNode, Asura_Guid uGZGuid )
{
	*ppxNewNode = Tools_EntityNode_OrientedNode::CreateAndReturn();
	s_pxThisToolsOrientedNode->m_uMyToolsGZGuid = uGZGuid;
}
#endif

Tools_EntityNode_OrientedNode* Tools_EntityNode_OrientedNode::CreateAndReturn()
{
    // Get a new GUID
    const Asura_Guid uGuid = Asura_Entity_System::GetCreatedByToolsGuid();

    // Set up tools data and auto-create a unique name (prompt for new name if there is a clash).
    Tools_Entity_Data* pxToolsData = CreateToolsDataWithDefaultName( "Orientated Node", "Orientated Node", uGuid, GetNamePrefix() );
    if ( !pxToolsData ) { return NULL; }

    // Create the game specfic server object and set its data.
    Asura_ServerNode_OrientedNode* pxGameNode = new Asura_ServerNode_OrientedNode( uGuid );
    if ( !pxGameNode ) { return NULL; }

    pxGameNode->ToolsData = pxToolsData;

    // Create the game specific tools object and select it.
    Tools_EntityNode_OrientedNode* pxToolsNode = new Tools_EntityNode_OrientedNode( pxGameNode );
    if ( !pxToolsNode ) { return NULL; }

    Tools_Object_System::SelectObject( pxToolsNode );

    pxToolsNode->Position = *Asura_Camera::GetPosition();
    pxGameNode->SetPosition( pxToolsNode->Position );
    Asura_Euler_3 xEulerOrient;
    xEulerOrient.FromMatrix( *Asura_Camera::GetOrientation() );
	xEulerOrient.x = xEulerOrient.z = 0.0f;
    pxGameNode->SetOrientation( xEulerOrient );
	xEulerOrient.ToMatrix( pxToolsNode->Orientation );

    Tools_Timed_Message_System::AddMessage( Tools_Timed_Message_System::COLOUR_DEFAULT, "Created new Orientated Node" );
    Tools_SaveAsura::SetEntityModified();

    return pxToolsNode;
}

Asura_ServerNode_OrientedNode* Tools_EntityNode_OrientedNode::GetGameOrientedNode() const
{
	return static_cast< Asura_ServerNode_OrientedNode* >( GetGameNode() );
}