//////////////////////////////////////////////////////////////////////////////
//
// Tools_EntityNode_GS2Spline.cpp
//
// Copyright (C) 2008 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"

#ifdef ASURA_USE_AXON

// header
#include "Entity/Locations/Spline/GamesceneSpline/Tools_EntityNode_GS2Spline.h"

// asura
#include "Entity/Asura_Entity_Classification.h"
#include "Entity/Locations/OrientedNode/GamesceneNode/Asura_ServerNode_GS2Node.h"
#include "Entity/Locations/Spline/GamesceneSpline/Asura_ServerNode_GS2Spline.h"
#include "Interface/Asura_Graph_Dialog.h"
#include "Network/Asura_Message.h"
#include "Rendering/Asura_Render.h"
#include "Rendering/Asura_Text_Output.h"

// axon
#include "AnimTemplates/Axon_Gamescene_AnimTemplate_System.h"
#include "Behaviour/Axon_BehaviourParams.h"
#include "Behaviour/BehaviourTypes/Axon_Behaviour_Types.h"
#include "Core/Axon_Messages.h"

// tools
#include "Entity/Locations/Spline/GamesceneSpline/Tools_Object_GamesceneSplinePoint.h"
#include "Entity/Locations/OrientedNode/GamesceneNode/Tools_EntityNode_GS2Node.h"
#include "Hierarchy/Tools_Animation_Files.h"
#include "Load&Save/Tools_ModeDataFile.h"
#include "Load&Save/Tools_SaveAsura.h"
#include "Misc/Tools_Modal_Overlay.h"
#include "Misc/Tools_Timed_Messages.h"
#include "Objects/Tools_Object_System.h"
#include "Objects/Axon/Tools_Object_Axon_ChooseObjects.h"
#include "Objects/Axon/Tools_Object_Axon_Interface.h"

//////////////////////////////////////////////////////////////////////////////
//
// Tools_EntityNode_GS2Spline
//
//////////////////////////////////////////////////////////////////////////////

TOOLS_DECLARE_SHARED_PARAM( bool, 0x76ae1b90, GS2Spline_TestForCollisions );    // Asura_GetHashID( "GS2Spline_TestForCollisions" )
TOOLS_DECLARE_SHARED_PARAM( bool, 0xa46485fb, GS2Spline_IgnoreCollisions );     // Asura_GetHashID( "GS2Spline_IgnoreCollisions" )
TOOLS_DECLARE_SHARED_PARAM( bool, 0x2ed6a59f, GS2Spline_UsePointOrientation );  // Asura_GetHashID( "GS2Spline_UsePointOrientation" )
TOOLS_DECLARE_SHARED_PARAM( bool, 0x5d7d61b4, GS2Spline_FollowSurface );        // Asura_GetHashID( "GS2Spline_FollowSurface" )
TOOLS_DECLARE_SHARED_PARAM( Asura_String, 0x722413fe, GS2Spline_BehaviourSetString ); // Asura_GetHashID( "GS2Spline_BehaviourSetString" )
TOOLS_DECLARE_SHARED_PARAM( int,          0x26803322, GS2Spline_BehaviourSetInt );    // Asura_GetHashID( "GS2Spline_BehaviourSetInt" )
TOOLS_DECLARE_SHARED_PARAM( Asura_String, uTOOLS_SHAREDPARAMHASH_ENTITYNODE_SETTHISTODEFAULTS, EntityNode_SetThisToDefaults );

Tools_EntityNode_GS2Spline* Tools_EntityNode_GS2Spline::s_pxThisToolsSpline = NULL;

Asura_Hashed_List< Tools_EntityNode_GS2Spline > Tools_EntityNode_GS2Spline::s_xList;
const char* Tools_EntityNode_GS2Spline::s_aszTabNames[ TAB_END_ENUM ] =
{
	"Properties",
	"   Path   ",
	"Behaviour"
};
u_int Tools_EntityNode_GS2Spline::s_uCurrentTab = Tools_EntityNode_GS2Spline::TAB_PROPERTIES;

Asura_Vector_2 Tools_EntityNode_GS2Spline::s_xDefaultsOverlayPosition = Asura_Vector_2( 10.0f, 10.0f );
bool Tools_EntityNode_GS2Spline::s_bRefreshDefaultsOverlay = false;
bool Tools_EntityNode_GS2Spline::s_bDefaultFollowSurface = false;
bool Tools_EntityNode_GS2Spline::s_bDefaultTestForCollisions = true;
bool Tools_EntityNode_GS2Spline::s_bDefaultIgnoreCollisions = false;
int Tools_EntityNode_GS2Spline::s_iDefaultBehaviourSet = AXON_BEHAVIOUR_TYPE_UNSET;

Tools_EntityNode_GS2Spline::Tools_EntityNode_GS2Spline( Asura_Entity* pxEntity )
: PARENT( pxEntity )
, m_bChanged( false )
, m_bChangedSpeedGraph( false )
, m_bPreviewing( false )
, m_bPreviewIsOccupied( false )
, m_uOwnerGuid( ASURA_GUID_UNREGISTERED )
, m_pxTabsDialogue( NULL )
, m_pxGraphDialogue( NULL )
, m_bOverrideDuration( false )
, m_bDistanceVsTimeGraph( false )
, m_fDurationOverride( false )
, m_fLength( 0.0f )
, m_bAllowPathEditing( true )
, m_bAllowGraphEditing( true )
{
	Type = TOT_Entity_GS2Spline;

	CreateAllToolsPathObjects( -1, false );

	// Fill in the data from Asura.
	Asura_ServerNode_GS2Spline* pxGSSplineNode = GetGameGS2Spline();
	if ( pxGSSplineNode->GetBehaviourParams() )
	{
		pxGSSplineNode->GetBehaviourParams()->DoSetToolsNode( this );
	}

	m_bOverrideDuration	   = pxGSSplineNode->GetOverrideDurationFlag();
	m_bDistanceVsTimeGraph = pxGSSplineNode->GetDistanceVsTimeGraph();

	m_fDurationOverride = pxGSSplineNode->GetTotalLinkTime();
	m_fLength           = pxGSSplineNode->GetTotalLength();

	s_xList.AddToList( this );
}

Tools_EntityNode_GS2Spline::~Tools_EntityNode_GS2Spline()
{
	if ( m_bPreviewing )
	{
		StopPreview();
	}

	if ( GetGameGS2Spline() && GetGameGS2Spline()->GetBehaviourParams() )
	{
		GetGameGS2Spline()->GetBehaviourParams()->DoSetToolsNode( NULL );
	}

	s_xList.RemoveFromList( this );
}

bool Tools_EntityNode_GS2Spline::IsBaseOrDerivativeOf( int iToolsObjectType ) const
{
	if ( iToolsObjectType == TOT_Entity_GS2Spline )
	{
		return true;
	}
	else
	{
		return PARENT::IsBaseOrDerivativeOf( iToolsObjectType );
	}
}

bool Tools_EntityNode_GS2Spline::IsSelectable()
{
	return ( Tools_System::GetCurrentMode() == TOOLS_MODE_OBJECT ) && PARENT::IsSelectable();
}

void Tools_EntityNode_GS2Spline::OnSelection()
{
	s_pxThisToolsSpline = this;

	const float fOverrideDurationValuePrev = GetGameGS2Spline()->GetOverrideDurationValue();
	const float fLengthPrev = GetGameGS2Spline()->GetTotalLength();
	// We'll need to update our length/duration stuff now in case other splines have affected our points at all.
	UpdateSpeedGraph();
	const float fOverrideDurationValue = GetGameGS2Spline()->GetOverrideDurationValue();
	const float fLength = GetGameGS2Spline()->GetTotalLength();
	if ( fOverrideDurationValuePrev != fOverrideDurationValue || fLengthPrev != fLength )
	{
		SetEntityModified();
	}

	PARENT::OnSelection();
}

void Tools_EntityNode_GS2Spline::OnDeselection()
{
	PARENT::OnDeselection();
	m_pxTabsDialogue = NULL;
	m_pxGraphDialogue = NULL;
	s_pxThisToolsSpline = NULL;
}

void Tools_EntityNode_GS2Spline::Update()
{
	PARENT::Update();

	UpdatePreview();

	Asura_ServerNode_GS2Spline* pxGameGS2Spline = GetGameGS2Spline();

	if ( pxGameGS2Spline->GetBehaviourParams() )
	{
		pxGameGS2Spline->GetBehaviourParams()->DoUpdate();
	}

	if ( m_bChanged )
	{
		// Update the asura data.
		pxGameGS2Spline->SetOverrideDurationFlag( m_bOverrideDuration );
		pxGameGS2Spline->SetOverrideDurationValue( m_fDurationOverride );
		pxGameGS2Spline->SetDistanceVsTimeGraph( m_bDistanceVsTimeGraph );

		pxGameGS2Spline->ChangedDistanceTimeGraph();
		if ( m_bOverrideDuration || m_bDistanceVsTimeGraph )
		{
			pxGameGS2Spline->SetOverrideDurationValue( m_fDurationOverride );
		}
		else
		{
			m_fDurationOverride = pxGameGS2Spline->GetTotalLinkTime();
		}
		m_fLength = pxGameGS2Spline->GetTotalLength();

		SetEntityModified();
		m_bChanged = false;

		if ( IsSelected() )
		{
			// If this is the primary selection then we need to re-create the overlay.
			RefreshOverlay();
		}
	}

	if ( s_bRefreshDefaultsOverlay )
	{
		RefreshDefaultsOverlay();
		s_bRefreshDefaultsOverlay = false;
	}

	//## 2009/05/05 This can be removed once all levels have been re-saved.
	if ( pxGameGS2Spline->m_xOccupierGuids.Size() )
	{
		Tools_Timed_Message_System::AddMessage( Tools_Timed_Message_System::COLOUR_INFO, "Cleared spline %u occupier list.", GetGuid() );
		pxGameGS2Spline->m_xOccupierGuids.Clear();
		SetEntityModified();
	}
}

void Tools_EntityNode_GS2Spline::UpdateWhileSelected()
{
	PARENT::UpdateWhileSelected();

	if ( GetGameGS2Spline() && GetGameGS2Spline()->GetBehaviourParams() )
	{
		GetGameGS2Spline()->GetBehaviourParams()->DoUpdateWhileSelected();
	}

	if ( m_bChangedSpeedGraph )
	{
		UpdateSpeedGraph();
		SetEntityModified();
		m_bChangedSpeedGraph = false;
	}
}

void Tools_EntityNode_GS2Spline::Render()
{
	Tools_Entity_Node::Render();

	if ( GetGameGS2Spline() && GetGameGS2Spline()->GetBehaviourParams() )
	{
		GetGameGS2Spline()->GetBehaviourParams()->DoRender();
	}

	const Asura_ServerNode_Spline* pxSpline = GetGameGS2Spline();
	UpdateToolsSpline( pxSpline->GetSourceGuid(), pxSpline->GetTargetGuid() );

	// Render the path that links the nodes.
	RenderPath( GetColour() );

	// Render the line that links the nodes.
	Asura_Render::RenderLine( m_xSourceNodePosition, m_xTargetNodePosition, GetColour() );
}

void Tools_EntityNode_GS2Spline::AddOverlayControls( Asura_Overlay* pxNewOverlay, Asura_Vector_2& xOffset )
{
	PARENT::AddOverlayControls( pxNewOverlay, xOffset );
	xOffset.y += Asura_Dialog::LargeSpacing() - Asura_Dialog::DefaultSpacing();

	m_pxTabsDialogue = new Asura_Tabs_Dialog( xOffset, s_aszTabNames, sizeof( s_aszTabNames ) / sizeof( s_aszTabNames[ 0 ] ) );
	pxNewOverlay->AddDialog( m_pxTabsDialogue );

	m_pxTabsDialogue->SelectTab( TAB_PROPERTIES );
	xOffset.y = m_pxTabsDialogue->m_xMaxCoord.y + 0.5f;
	float fTabsEndY = xOffset.y;

	AddPropertiesTabControls( m_pxTabsDialogue, xOffset );
	fTabsEndY = Asura_Maths::Max( fTabsEndY, xOffset.y );

	m_pxTabsDialogue->SelectTab( TAB_PATH );
	xOffset.y = m_pxTabsDialogue->m_xMaxCoord.y + 0.5f;
	AddPathTabControls( m_pxTabsDialogue, xOffset );
	fTabsEndY = Asura_Maths::Max( fTabsEndY, xOffset.y );

	m_pxTabsDialogue->SelectTab( TAB_BEHAVIOUR );
	xOffset.y = m_pxTabsDialogue->m_xMaxCoord.y + 0.5f;
	AddBehaviourTabControls( m_pxTabsDialogue, xOffset );
	fTabsEndY = Asura_Maths::Max( fTabsEndY, xOffset.y );

	xOffset.y = fTabsEndY + Asura_Dialog::DefaultSpacing();
	AddUnderTabsControls( pxNewOverlay, xOffset );

	m_pxTabsDialogue->SetSelectedTabPointer( &s_uCurrentTab );
}

bool Tools_EntityNode_GS2Spline::GetParameter( Asura_Hash_ID uParamID, Tools_Parameter* pxParam ) const
{
	if ( GetGameGS2Spline()->GetBehaviourParams() && GetGameGS2Spline()->GetBehaviourParams()->DoGetParameter( uParamID, pxParam ) )
	{
		return true;
	}

	switch ( uParamID )
	{
		case g_uHash_GS2Spline_TestForCollisions:
			pxParam->SetBool( GetGameGS2Spline()->ShouldTestForCollisions() );
			return true;

		case g_uHash_GS2Spline_IgnoreCollisions:
			pxParam->SetBool( GetGameGS2Spline()->ShouldIgnoreCollisions() );
			return true;

		case g_uHash_GS2Spline_UsePointOrientation:
			pxParam->SetBool( GetGameGS2Spline()->ShouldUsePointOrientation() );
			return true;

		case g_uHash_GS2Spline_FollowSurface:
			pxParam->SetBool( GetGameGS2Spline()->ShouldFollowSurface( true ) );
			return true;

		case g_uHash_GS2Spline_BehaviourSetString:
		{
			ListSelection_Overlay xBehaviourList( "" );
			Tools_Object_Axon_Interface::GetInstance()->BuildGS2SplineBehaviourSetList( xBehaviourList, this );
			pxParam->SetString( xBehaviourList.GetItemNameByData( GetGameGS2Spline()->GetBehaviourSet() ) );
			return true;
		}

		case g_uHash_GS2Spline_BehaviourSetInt:
			// This is only ever set, not got, because of the way shared function dialogues work.
			return true;

		default:
			return PARENT::GetParameter( uParamID, pxParam );
	}
}

bool Tools_EntityNode_GS2Spline::SetParameter( Asura_Hash_ID uParamID, const Tools_Parameter* pxParam )
{
	if ( GetGameGS2Spline()->GetBehaviourParams() && GetGameGS2Spline()->GetBehaviourParams()->DoSetParameter( uParamID, pxParam ) )
	{
		return true;
	}

	switch ( uParamID )
	{
		case g_uHash_GS2Spline_TestForCollisions:
		{
			GetGameGS2Spline()->SetShouldTestForCollisions( pxParam->GetBool() );
			SetEntityModified();
			return true;
		}

		case g_uHash_GS2Spline_IgnoreCollisions:
		{
			GetGameGS2Spline()->SetShouldIgnoreCollisions( pxParam->GetBool() );
			SetEntityModified();
			return true;
		}

		case g_uHash_GS2Spline_UsePointOrientation:
		{
			GetGameGS2Spline()->SetUsePointOrientation( pxParam->GetBool() );
			if ( IsSelected() )
			{
				RequestRefreshOverlay();
			}
			SetEntityModified();
			return true;
		}

		case g_uHash_GS2Spline_FollowSurface:
		{
			GetGameGS2Spline()->SetShouldFollowSurface( pxParam->GetBool() );
			GetGameGS2Spline()->UpdateFollowSurface();
			SetEntityModified();
			return true;
		}

		case g_uHash_GS2Spline_BehaviourSetInt:
		{
			GetGameGS2Spline()->SetBehaviourSet( pxParam->GetInt() );
			if ( GetGameGS2Spline()->GetBehaviourParams() )
			{
				GetGameGS2Spline()->GetBehaviourParams()->DoSetToolsNode( this );
			}
			if ( IsSelected() )
			{
				RequestRefreshOverlay();
			}
			SetEntityModified();
			return true;
		}

		case g_uHash_GS2Spline_BehaviourSetString:
			// This is only ever got, not set, because of the way shared function dialogues work.
			return true;

		case g_uHash_EntityNode_SetThisToDefaults:
		{
			GetGameGS2Spline()->SetShouldFollowSurface( s_bDefaultFollowSurface );
			GetGameGS2Spline()->SetShouldTestForCollisions( s_bDefaultTestForCollisions );
			GetGameGS2Spline()->SetShouldIgnoreCollisions( s_bDefaultIgnoreCollisions );
			if ( GetGameGS2Spline()->GetBehaviourSet() != s_iDefaultBehaviourSet )
			{
				GetGameGS2Spline()->SetBehaviourSet( s_iDefaultBehaviourSet );
				if ( GetGameGS2Spline()->GetBehaviourParams() )
				{
					GetGameGS2Spline()->GetBehaviourParams()->DoSetToolsNode( this );
				}
			}
			if ( IsSelected() )
			{
				RequestRefreshOverlay();
			}
			SetEntityModified();
			return true;
		}

		default:
			return PARENT::SetParameter( uParamID, pxParam );
	}
}

void Tools_EntityNode_GS2Spline::AddFilesToExportList( List< Asura_String >& xNameList ) const
{
	PARENT::AddFilesToExportList( xNameList );

	if ( GetGameGS2Spline()->GetBehaviourParams() )
	{
		GetGameGS2Spline()->GetBehaviourParams()->DoAddFilesToExportList( xNameList );
	}
}

void Tools_EntityNode_GS2Spline::AddFilesToExportListFromMessage( u_short usMessageID, void* pData, List< Asura_String >& xNameList ) const
{
	PARENT::AddFilesToExportListFromMessage( usMessageID, pData, xNameList );

	if ( GetGameGS2Spline()->GetBehaviourParams() )
	{
		GetGameGS2Spline()->GetBehaviourParams()->DoAddFilesToExportListFromMessage( usMessageID, pData, xNameList );
	}
}

void Tools_EntityNode_GS2Spline::OnSplinePointMoved( const int iIndex )
{
	PARENT::OnSplinePointMoved( iIndex );
	// We'll need to update our length/duration stuff now.
	m_bChangedSpeedGraph = true;
}

void Tools_EntityNode_GS2Spline::CreateAllToolsPathObjects( const int iSelectIndexAfter /*= -1*/, const bool bShowEndPoints /*= true*/ )
{
	// Remove all of the old tools objects.
	RemoveAllToolsPathObjects();

	// Create the tools objects for the link points.
	Asura_ServerNode_GS2Spline* pxGameSpline = GetGameGS2Spline();
	for (u_int u = 1; u < pxGameSpline->GetNumberOfPoints() - 1; ++u)
	{
		const bool bIsEndPoint = (u == 1 || u == pxGameSpline->GetNumberOfPoints() - 2);
		Tools_Object_GS2Point* pxPoint = new Tools_Object_GS2Point(pxGameSpline, this, u, bIsEndPoint);
		m_xListAttachedPoints.add_entry(pxPoint);
		if ( iSelectIndexAfter + 1 == static_cast< int >( u ) )
		{
			Tools_Object_System::SelectObject(pxPoint, false);
		}
		else if ( !bShowEndPoints && bIsEndPoint )
		{
			pxPoint->SetHiddenFlag( ObjectHideFlag_User, true );
		}
	}
}

void Tools_EntityNode_GS2Spline::UpdateBehaviourTree()
{
	if ( GetGameGS2Spline()->UpdateBehaviourTree() )
	{
		SetEntityModified();
	}
}

bool Tools_EntityNode_GS2Spline::PromptForOwnerSkinName()
{
	Asura_String xSkinFilename;
	Tools_Animation_Files::ChooseSkin( &m_xOwnerSkinName, &xSkinFilename );
	return ( !m_xOwnerSkinName.IsBlank() );
}

void Tools_EntityNode_GS2Spline::SetSourcePosOriFromTarget( const Asura_Vector_3& xPositionOffset, const Asura_Quat& xOrientOffsetQ )
{
	// Pos and Ori are in 'anim start' space.
	Asura_ServerNode_GS2Node* pxSourceNode = Asura_ServerNode_GS2Node::FindEntity( GetGameGS2Spline()->GetSourceGuid() );
	Asura_ServerNode_GS2Node* pxTargetNode = Asura_ServerNode_GS2Node::FindEntity( GetGameGS2Spline()->GetTargetGuid() );
	Tools_EntityNode_GS2Node* pxToolsSourceNode = Tools_EntityNode_GS2Node::FindEntity( GetGameGS2Spline()->GetSourceGuid() );
	if ( pxSourceNode && pxTargetNode && pxToolsSourceNode )
	{
		// Apply inverse orientation to calculate the start node orientation.
		Asura_Quat xTargetOrientQ;
		pxTargetNode->GetOrientation( xTargetOrientQ );
		Asura_Quat xInverseOrientOffsetQ = xOrientOffsetQ;
		xInverseOrientOffsetQ.Invert();
		const Asura_Quat xResultantOrientQ = xTargetOrientQ * xInverseOrientOffsetQ;
		pxSourceNode->SetOrientation( xResultantOrientQ );
		xResultantOrientQ.ToMatrix( pxToolsSourceNode->Orientation );

		Asura_Vector_3 xResultantOffset = xResultantOrientQ.RotateVector( -xPositionOffset );
		const Asura_Vector_3 xResultantPosition = pxTargetNode->GetPosition() + xResultantOffset;
		pxSourceNode->SetPosition( xResultantPosition );
		pxToolsSourceNode->Position = xResultantPosition;
	}
}

void Tools_EntityNode_GS2Spline::SetTargetPosOriFromSource( const Asura_Vector_3& xPositionOffset, const Asura_Quat& xOrientOffsetQ )
{
	// Pos and Ori are in 'anim start' space.
	Asura_ServerNode_GS2Node* pxSourceNode = Asura_ServerNode_GS2Node::FindEntity( GetGameGS2Spline()->GetSourceGuid() );
	Asura_ServerNode_GS2Node* pxTargetNode = Asura_ServerNode_GS2Node::FindEntity( GetGameGS2Spline()->GetTargetGuid() );
	Tools_EntityNode_GS2Node* pxToolsTargetNode = Tools_EntityNode_GS2Node::FindEntity( GetGameGS2Spline()->GetTargetGuid() );
	if ( pxSourceNode && pxTargetNode && pxToolsTargetNode )
	{
		Asura_Matrix_3x3 xSourceOrientM;
		pxSourceNode->GetOrientation( xSourceOrientM );
		const Asura_Vector_3 xResultantPosition = pxSourceNode->GetPosition() + ( xPositionOffset * xSourceOrientM );
		pxTargetNode->SetPosition( xResultantPosition );
		pxToolsTargetNode->Position = xResultantPosition;

		Asura_Quat xSourceOrientQ;
		pxSourceNode->GetOrientation( xSourceOrientQ );
		const Asura_Quat xResultantOrientQ = xSourceOrientQ * xOrientOffsetQ;
		pxTargetNode->SetOrientation( xResultantOrientQ );
		xResultantOrientQ.ToMatrix( pxToolsTargetNode->Orientation );
	}
}

Tools_EntityNode_GS2Spline* Tools_EntityNode_GS2Spline::Create()
{
	if ( Tools_Object_System::SelectedObjectList.size() != 2 ) { return NULL; }

	const Asura_Guid uSourceGuid = Tools_Object_System::SelectedObjectList.first_entry()->GetGuid();
	const Asura_Guid uTargetGuid = Tools_Object_System::SelectedObjectList.last_entry()->GetGuid();

	Asura_ServerNode_GS2Node* pxSourceNode = Asura_ServerNode_GS2Node::FindEntity( uSourceGuid );
	const Asura_ServerNode_GS2Node* pxTargetNode = Asura_ServerNode_GS2Node::FindEntity( uTargetGuid );
	if ( !pxSourceNode || !pxTargetNode ) { return NULL; }

	// Create the game object.
	// Get a new GUID
	const Asura_Guid uGuid = Asura_Entity_System::GetCreatedByToolsGuid();

	// Set up tools data and auto-create a unique name (prompt for new name if there is a clash).
	Tools_Entity_Data* pxToolsData = CreateToolsDataWithDefaultName( "GS2 Spline", "GS2 Spline", uGuid, GetNamePrefix() );
	if ( !pxToolsData ) { return NULL; }

	// Create the game specfic server object and set its data.
	Asura_ServerNode_GS2Spline* pxGameSpline = new Asura_ServerNode_GS2Spline( uGuid );
	if ( !pxGameSpline ) { return NULL; }
	pxGameSpline->ToolsData = pxToolsData;

	Asura_Vector_3 xPosition;
	GetSplineRenderPosition( xPosition, uSourceGuid, uTargetGuid );
	pxGameSpline->SetPosition( xPosition );
	pxGameSpline->SetSourceGuid( uSourceGuid );
	pxGameSpline->SetTargetGuid( uTargetGuid );
	pxGameSpline->CreateBasePoints();
	pxSourceNode->AddNewSpline( pxGameSpline->GetGuid() );

	// Create the tools object.
	Tools_EntityNode_GS2Spline* pxToolsSpline = new Tools_EntityNode_GS2Spline( pxGameSpline );
	if ( !pxToolsSpline ) { return NULL; }

	pxToolsSpline->Position = pxGameSpline->GetPosition();
	pxToolsSpline->SetThisToDefaults();

	Tools_Object_System::SelectObject( pxToolsSpline );

	Tools_Timed_Message_System::AddMessage( Tools_Timed_Message_System::COLOUR_DEFAULT, "Created new GS2 spline." );
	Tools_SaveAsura::SetEntityModified();

	return pxToolsSpline;
}

void Tools_EntityNode_GS2Spline::AddPropertiesTabControls( Asura_Tabs_Dialog* pxTabsDialogue, Asura_Vector_2& xOffset )
{
	AddSplineSourceTargetControls( pxTabsDialogue, xOffset );
	xOffset.y += Asura_Dialog::LargeSpacing() - Asura_Dialog::DefaultSpacing();

	AddCollisionsDialogues( pxTabsDialogue, xOffset );
	xOffset.y += Asura_Dialog::LargeSpacing() - Asura_Dialog::DefaultSpacing();

	AddSplineEventOverlayControls( pxTabsDialogue, xOffset );
}

void Tools_EntityNode_GS2Spline::AddPathTabControls( Asura_Tabs_Dialog* pxTabsDialogue, Asura_Vector_2& xOffset )
{
	if ( m_bAllowPathEditing )
	{
		AddSplinePathEditControls( pxTabsDialogue, xOffset );
	}
	if ( m_bAllowGraphEditing )
	{
		AddGraphControls( pxTabsDialogue, xOffset );
	}
	else
	{
		// The graph sometimes disappears whilst this object is selected - some behaviours turn it off.
		// Reset the pointer here so that it is not left dangling if this happens.
		m_pxGraphDialogue = NULL;
	}
}

void Tools_EntityNode_GS2Spline::AddBehaviourTabControls( Asura_Tabs_Dialog* pxTabsDialogue, Asura_Vector_2& xOffset )
{
	ListSelection_Overlay xBehaviourList( "" );
	Tools_Object_Axon_Interface::GetInstance()->BuildGS2SplineBehaviourSetList( xBehaviourList, this );
	Asura_Function_Dialog* pxFDialog = new Asura_Function_Dialog( "Behaviour: ", &g_xParam_GS2Spline_BehaviourSetString, xOffset, PickBehaviourSet );
	pxFDialog->SetHelpText( "Selects the behaviour for the actor to use on this link." );
	pxTabsDialogue->AddDialog( pxFDialog );
	xOffset.y += Asura_Dialog::LargeSpacing();

	if ( GetGameGS2Spline()->GetBehaviourParams() )
	{
		GetGameGS2Spline()->GetBehaviourParams()->DoAddOverlayControls( pxTabsDialogue, xOffset );
	}
}

void Tools_EntityNode_GS2Spline::AddUnderTabsControls( Asura_Overlay* pxNewOverlay, Asura_Vector_2& xOffset )
{
	AddMainDefaultsOverlayControls( pxNewOverlay, xOffset );
	Tools_Object_Axon_Interface::AddPreviewControls( pxNewOverlay, xOffset );
}

void Tools_EntityNode_GS2Spline::SetDefaultsToThis()
{
	s_bDefaultFollowSurface = GetGameGS2Spline()->ShouldFollowSurface();
	s_bDefaultTestForCollisions = GetGameGS2Spline()->ShouldTestForCollisions();
	s_bDefaultIgnoreCollisions = GetGameGS2Spline()->ShouldIgnoreCollisions();
	s_iDefaultBehaviourSet = GetGameGS2Spline()->GetBehaviourSet();
	s_bRefreshDefaultsOverlay = true;
}

void Tools_EntityNode_GS2Spline::AddDefaultsOverlayControls( Asura_Overlay* pxNewOverlay, Asura_Vector_2& xOffset )
{
	Asura_Dialog* pxDialog = new Asura_Tickbox_Dialog( "Follow Surface", xOffset, &s_bDefaultFollowSurface );
	pxDialog->SetHelpText( "Whether the actor should use LOS tests when moving to stick closely to the surface." );
	pxNewOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Tickbox_Dialog( "Collide With Movable Objects", xOffset, &s_bDefaultTestForCollisions );
	pxDialog->SetHelpText( "If ticked the entity will be solid and will push movable objects out of the way. (It will always go through immovable barriers.)\n"
		"If unticked the entity will be ghostlike and pass through everything." );
	pxNewOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Tickbox_Dialog( "Ignore Blocked Path", xOffset, &s_bDefaultIgnoreCollisions );
	pxDialog->SetHelpText( "Whether the actor should ignore movable objects in its way or stop when blocked." );
	pxNewOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::LargeSpacing();

	ListSelection_Overlay xBehaviourList( "" );
	Tools_Object_Axon_Interface::GetInstance()->BuildGS2SplineBehaviourSetList( xBehaviourList );
	Asura_String xBehaviourSetString = "Behaviour: ";
	xBehaviourSetString += xBehaviourList.GetItemNameByData( s_iDefaultBehaviourSet );

	pxDialog = new Asura_Function_Dialog( xBehaviourSetString.GetString(), xOffset, PickDefaultBehaviourSet, reinterpret_cast< int >( this ) );
	pxDialog->SetHelpText( "Selects the behaviour for the actor to use on this link." );
	pxNewOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();
}

Tools_ChooseObject* Tools_EntityNode_GS2Spline::CreateSourceNodeChooseObject( Asura_Guid* pxGuid ) const
{
	Tools_ChooseObject_GS2Node* pxChooseObject = new Tools_ChooseObject_GS2Node( pxGuid );
	pxChooseObject->AddGuidToIgnore( GetGameGS2Spline()->GetSourceGuid() );
	pxChooseObject->AddGuidToIgnore( GetGameGS2Spline()->GetTargetGuid() );
	return pxChooseObject;
}

Tools_ChooseObject* Tools_EntityNode_GS2Spline::CreateTargetNodeChooseObject( Asura_Guid* pxGuid ) const
{
	Tools_ChooseObject_GS2Node* pxChooseObject = new Tools_ChooseObject_GS2Node( pxGuid );
	pxChooseObject->AddGuidToIgnore( GetGameGS2Spline()->GetSourceGuid() );
	pxChooseObject->AddGuidToIgnore( GetGameGS2Spline()->GetTargetGuid() );
	return pxChooseObject;
}

void Tools_EntityNode_GS2Spline::AddCollisionsDialogues( Asura_Tabs_Dialog* pxTabsDialogue, Asura_Vector_2& xOffset )
{
	Asura_Dialog* pxTDialogue = new Asura_SharedTickbox_Dialog( "Collide With Movable Objects", xOffset, &g_xParam_GS2Spline_TestForCollisions );
	pxTDialogue->SetHelpText( "If ticked the entity will be solid and will push movable objects out of the way. (It will always go through immovable barriers.)\n"
		"If unticked the entity will be ghostlike and pass through everything." );
	pxTabsDialogue->AddDialog( pxTDialogue );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxTDialogue = new Asura_SharedTickbox_Dialog( "Ignore Blocked Path", xOffset, &g_xParam_GS2Spline_IgnoreCollisions );
	pxTDialogue->SetHelpText( "Whether the actor should ignore movable objects in its way or stop when blocked." );
	pxTabsDialogue->AddDialog( pxTDialogue );
	xOffset.y += Asura_Dialog::DefaultSpacing();
}

void Tools_EntityNode_GS2Spline::AddSplinePathEditControls( Asura_Tabs_Dialog* pxTabsDialogue, Asura_Vector_2& xOffset )
{
	Tools_EntityNode_Spline::AddSplinePathEditControls( pxTabsDialogue, xOffset );

	Asura_Dialog* pxNewDialog = new Asura_SharedTickbox_Dialog( "Use Point Orientation", xOffset, &g_xParam_GS2Spline_UsePointOrientation );
	pxNewDialog->SetHelpText( "Take orientation from the spline points when moving rather than orientating to the spline itself.");
	m_pxTabsDialogue->AddDialog(pxNewDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	if ( GetGameGS2Spline()->ShouldUsePointOrientation() )
	{
		pxNewDialog = new Asura_Function_Dialog( "Set To Default Orientations", xOffset, SetToDefaultOrientations, &m_bChanged );
		pxNewDialog->SetHelpText( "Set the orientation of the track points to the direction of the spline." );
		m_pxTabsDialogue->AddDialog(pxNewDialog);
		xOffset.y += Asura_Dialog::DefaultSpacing();
	}

	AddFollowSurfaceDialogue( pxTabsDialogue, xOffset );
}

void Tools_EntityNode_GS2Spline::AddFollowSurfaceDialogue( Asura_Tabs_Dialog* pxTabsDialogue, Asura_Vector_2& xOffset )
{
	Asura_Dialog* pxNewDialog = new Asura_SharedTickbox_Dialog( "Follow Surface", xOffset, &g_xParam_GS2Spline_FollowSurface );
	pxNewDialog->SetHelpText( "Whether the actor should use LOS tests when moving to stick closely to the surface." );
	pxTabsDialogue->AddDialog( pxNewDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();
}

void Tools_EntityNode_GS2Spline::AddGraphControls( Asura_Tabs_Dialog* pxTabsDialog, Asura_Vector_2& xOffset )
{
	Asura_Dialog* pxNewDialog = new Asura_Tickbox_Dialog("Distance Vs Time", xOffset, &m_bDistanceVsTimeGraph, &m_bChanged);
	pxNewDialog->SetHelpText("Change whether the below graph is distance vs time. Otherwise it is speed vs time.");
	pxTabsDialog->AddDialog(pxNewDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	if (!m_bDistanceVsTimeGraph)
	{
		pxNewDialog = new Asura_Tickbox_Dialog("Override Duration", xOffset, &m_bOverrideDuration, &m_bChanged);
		pxNewDialog->SetHelpText("Override how long the object takes to traverse this spline with a fixed time. Note that speed values on the graph will then become relative.");
		pxTabsDialog->AddDialog(pxNewDialog);
		xOffset.y += Asura_Dialog::DefaultSpacing();
	} 

	pxNewDialog = new Asura_Numerical_Dialog("Duration", xOffset, &m_fDurationOverride, 0.1f, 500.0f, 0.1f, "%.1f", &m_bChangedSpeedGraph);
	pxNewDialog->SetHelpText("The object will take this amount of time to traverse this spline.");
	static_cast<Asura_Numerical_Dialog*>(pxNewDialog)->SetReadOnly( !m_bOverrideDuration && !m_bDistanceVsTimeGraph );
	pxTabsDialog->AddDialog(pxNewDialog);
	xOffset.x += 20;

	pxNewDialog = new Asura_Numerical_Dialog("Length", xOffset, &m_fLength, 0.1f, 500.0f, 0.1f, "%.1f");
	pxNewDialog->SetHelpText("This is the total length of this spline.");
	static_cast<Asura_Numerical_Dialog*>(pxNewDialog)->SetReadOnly(true);
	pxTabsDialog->AddDialog(pxNewDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();
	xOffset.x -= 20;

	if (!m_bDistanceVsTimeGraph)
	{
		// Use speed from end of last link.
		pxNewDialog = new Asura_Function_Dialog("Auto-Set Start Speed", xOffset, AutoSetStartSpeed, &m_bChangedSpeedGraph, CanAutoSetStartSpeed);
		pxNewDialog->SetHelpText("Automatically sets the start speed of this link to the end speed of the previous link");
		pxTabsDialog->AddDialog(pxNewDialog);
		xOffset.x += 20.0f;

		// Use speed from start of next link.
		pxNewDialog = new Asura_Function_Dialog("Auto-Set End Speed", xOffset, AutoSetEndSpeed, &m_bChangedSpeedGraph, CanAutoSetEndSpeed);
		pxNewDialog->SetHelpText("Automatically sets the end speed of this link to the start speed of the next link");
		pxTabsDialog->AddDialog(pxNewDialog);
		xOffset.x -= 20.0f;
		xOffset.y += Asura_Dialog::DefaultSpacing();

		// Copy speeds start->end->start.
		pxNewDialog = new Asura_Function_Dialog("Copy Start Speed To End", xOffset, CopyStartSpeedToEnd, &m_bChangedSpeedGraph);
		pxNewDialog->SetHelpText("Sets the start speed of this link to the end speed.");
		pxTabsDialog->AddDialog(pxNewDialog);
		xOffset.x += 20.0f;

		pxNewDialog = new Asura_Function_Dialog("Copy End Speed To Start", xOffset, CopyEndSpeedToStart, &m_bChangedSpeedGraph);
		pxNewDialog->SetHelpText("Sets the end speed of this link to the start speed.");
		pxTabsDialog->AddDialog(pxNewDialog);
		xOffset.x -= 20.0f;
		xOffset.y += Asura_Dialog::DefaultSpacing();
	}

	Asura_ServerNode_GS2Spline* pxGameSpline = GetGameGS2Spline();
	if ( m_bDistanceVsTimeGraph )
	{
		// Position over time graph.
		m_pxGraphDialogue = new Asura_Graph_Dialog<float>("Position", xOffset, pxGameSpline->GetSpeedOverTimeGraph(), 0.0f, 1.0f, &m_bChangedSpeedGraph);
		m_pxGraphDialogue->SetHelpText( "Spline Position/Time Graph" );
		m_pxGraphDialogue->m_pxGraphGadget->AdjustToFit(false, true);
	}
	else
	{
		// Speed over time graph.
		m_pxGraphDialogue = new Asura_Graph_Dialog<float>("Speed", xOffset, pxGameSpline->GetSpeedOverTimeGraph(), 0.0f, 10.0f, &m_bChangedSpeedGraph);
		m_pxGraphDialogue->SetHelpText( "Spline Speed/Time Graph" );
		m_pxGraphDialogue->m_pxGraphGadget->AdjustToFit(false, true);
	}

	UpdateGraphGridLines();

	pxTabsDialog->AddDialog( m_pxGraphDialogue );
	xOffset.y = m_pxGraphDialogue->m_xMaxCoord.y;
}

void Tools_EntityNode_GS2Spline::UpdateSpeedGraph()
{
	Asura_ServerNode_GS2Spline* pxGamesceneSpline = GetGameGS2Spline();
	if ( m_bOverrideDuration || m_bDistanceVsTimeGraph )
	{
		pxGamesceneSpline->SetOverrideDurationValue( m_fDurationOverride );
	}
	else
	{
		m_fDurationOverride = pxGamesceneSpline->GetTotalLinkTime();
	}
	m_fLength = pxGamesceneSpline->GetTotalLength();

	UpdateGraphGridLines();
}

void Tools_EntityNode_GS2Spline::UpdateGraphGridLines()
{
	if ( m_pxGraphDialogue && m_pxGraphDialogue->m_pxGraphGadget )
	{
		Asura_ServerNode_GS2Spline* pxGameSpline = GetGameGS2Spline();

		const int   iNumPoints		 = pxGameSpline->GetNumberOfPoints() - 2;
		const u_int uNumSplineEvents = pxGameSpline->GetNumSplineEvents();

		m_pxGraphDialogue->m_pxGraphGadget->SetUseCustomXGridLines( static_cast< int >( iNumPoints + uNumSplineEvents ) );
		Asura_Graph_Gadget< float >::CustomGridline* pxGridLines = m_pxGraphDialogue->m_pxGraphGadget->GetCustomXGridlines();

		int iLineIndex = 0;
		for ( int i = 1; i < ( iNumPoints - 1 ); ++i )
		{
			float fDistance = pxGameSpline->GetCumulativeLength(i);
			float fProp = pxGameSpline->GetProportionAlongSpline(fDistance);
			float fTime = pxGameSpline->GetTimeFromProportion(fProp);
			pxGridLines[ iLineIndex ].m_fValue = fTime / pxGameSpline->GetTotalLinkTime();
			++iLineIndex;
		}
		for ( u_int u = 0; u < uNumSplineEvents; ++u )
		{
			pxGridLines[ iLineIndex ].m_fValue  = pxGameSpline->GetSplineEventDistance( u ) / pxGameSpline->GetTotalLength();
			pxGridLines[ iLineIndex ].m_uColour = 0xffff8040;
			++iLineIndex;
		}
	}
}

void Tools_EntityNode_GS2Spline::UpdatePreview()
{
	if ( m_bPreviewing )
	{
		bool bPreviewRunning = true;

		Axon_BehaviourParams* pxBehaviourParams = GetGameGS2Spline()->GetBehaviourParams();
		if ( pxBehaviourParams )
		{
			bPreviewRunning = pxBehaviourParams->DoUpdatePreview();
		}
		else
		{
			bPreviewRunning = false;
		}

		// Propagate the preview if necessary.
		// Note that we should propagate before stopping ourselves so that the count of items previewing in Tools_Object_Axon_Interface
		// doesn't reach zero before the whole structure has previewed.
		if ( !bPreviewRunning )
		{
			if ( Tools_Object_Axon_Interface::ShouldPropagatePreview() )
			{
				Tools_EntityNode_GS2Node* pxToolsNode = Tools_EntityNode_GS2Node::FindEntity( GetGameGS2Spline()->GetTargetGuid() );
				if ( pxToolsNode )
				{
					pxToolsNode->StartPreview();
				}
			}

			// Stop ourselves.
			StopPreview();
		}
	}
}

void Tools_EntityNode_GS2Spline::TransferToolsModeData( Tools_ModeDataFile& xFile )
{
	u_int uVersion = uCURRENT_TOOLS_DATA_VERSION;
	xFile.Transfer( uVersion );

	if ( uVersion == uCURRENT_TOOLS_DATA_VERSION )
	{
		xFile.Transfer( s_xDefaultsOverlayPosition );
		xFile.Transfer( s_bDefaultFollowSurface );
		xFile.Transfer( s_bDefaultTestForCollisions );
		xFile.Transfer( s_bDefaultIgnoreCollisions );
		xFile.Transfer( s_iDefaultBehaviourSet );
	}
	else if ( uVersion == 0 )
	{
		xFile.Transfer( s_xDefaultsOverlayPosition );
		xFile.Transfer( s_bDefaultFollowSurface );
		xFile.Transfer( s_bDefaultTestForCollisions );
		xFile.Transfer( s_iDefaultBehaviourSet );
	}
}

void Tools_EntityNode_GS2Spline::StartPreview()
{
	if ( !m_bPreviewing )
	{
		m_bPreviewing = true;
		if ( GetGameGS2Spline()->GetBehaviourParams() )
		{
			GetGameGS2Spline()->GetBehaviourParams()->DoStartPreview();
		}
		Tools_Object_Axon_Interface::IncrementNumPreviewing();
	}
}

void Tools_EntityNode_GS2Spline::StopPreview()
{
	if ( m_bPreviewing )
	{
		m_bPreviewing = false;
		if ( GetGameGS2Spline()->GetBehaviourParams() )
		{
			GetGameGS2Spline()->GetBehaviourParams()->DoStopPreview();
		}

		// This must come last otherwise the looping doesn't work correctly.
		Tools_Object_Axon_Interface::DecrementNumPreviewing();
	}
}

void Tools_EntityNode_GS2Spline::PickBehaviourSet()
{
	ListSelection_Overlay xBehaviourList( "Pick Behaviour To Use:" );
	Tools_Object_Axon_Interface::GetInstance()->BuildGS2SplineBehaviourSetList( xBehaviourList, s_pxThisToolsSpline );
	if ( !xBehaviourList.DoModal() )
	{
		return;
	}

	// Warn if the behaviour is about to be reset.
	const int iNewBehaviourSet = xBehaviourList.GetSelectionData();
	if ( s_pxThisToolsSpline->IsSelected() && Tools_Object_System::SelectedObjectList.size() == 1 &&
		iNewBehaviourSet == s_pxThisToolsSpline->GetGameGS2Spline()->GetBehaviourSet() && iNewBehaviourSet != AXON_BEHAVIOUR_TYPE_UNSET )
	{
		YesNo_Overlay xYNOverlay( "This will reset the current behaviour. Are you sure?" );
		if ( !xYNOverlay.DoModal() )
		{
			return;
		}
	}

	// Setting this will cause SetParameter to be called with this parameter hash on all selected objects.
	g_xParam_GS2Spline_BehaviourSetInt.SetValue( iNewBehaviourSet );
}

void Tools_EntityNode_GS2Spline::AutoSetStartSpeed()
{
	Asura_ServerNode_GS2Spline* pxGameSpline = s_pxThisToolsSpline->GetGameGS2Spline();

	Asura_ServerNode_GS2Node* pxSourceNode = Asura_ServerNode_GS2Node::FindEntity( pxGameSpline->GetSourceGuid() );
	if ( !pxSourceNode ) { return; }

	for ( Asura_Hashed_List_It< Asura_ServerNode_GS2Spline > xSplineIter( Asura_ServerNode_GS2Spline::GetList() );
		!xSplineIter.Done();
		xSplineIter.Next() )
	{
		Asura_ServerNode_GS2Spline* pxSpline = xSplineIter.GetCurrent();
		if ( pxSpline->GetTargetGuid() == pxSourceNode->GetGuid() )
		{
			float fStartSpeed = pxSpline->GetSpeedFromTime(1.0f);
			Asura_Graph_Point<float>* pxGraphPoints = pxGameSpline->GetSpeedOverTimeGraph()->GetControlPoints();
			pxGraphPoints->m_pxNext->m_xValue = fStartSpeed * pxGameSpline->GetVelocityMultiplier();
			pxGameSpline->GetSpeedOverTimeGraph()->Update();
			s_pxThisToolsSpline->m_fDurationOverride = pxGameSpline->GetTotalLinkTime();
			s_pxThisToolsSpline->m_fLength = pxGameSpline->GetTotalLength();
			return;
		}
	}
}

bool Tools_EntityNode_GS2Spline::CanAutoSetStartSpeed()
{
	Asura_ServerNode_GS2Spline* pxGameSpline = s_pxThisToolsSpline->GetGameGS2Spline();

	Asura_ServerNode_GS2Node* pxSourceNode = Asura_ServerNode_GS2Node::FindEntity( pxGameSpline->GetSourceGuid() );
	if ( !pxSourceNode ) { return false; }

	for ( Asura_Hashed_List_It< Asura_ServerNode_GS2Spline > xSplineIter( Asura_ServerNode_GS2Spline::GetList() );
		!xSplineIter.Done();
		xSplineIter.Next() )
	{
		Asura_ServerNode_GS2Spline* pxSpline = xSplineIter.GetCurrent();
		if ( pxSpline->GetTargetGuid() == pxSourceNode->GetGuid() && !pxSpline->GetDistanceVsTimeGraph() )
		{
			return true;
		}
	}

	return false;
}

void Tools_EntityNode_GS2Spline::AutoSetEndSpeed()
{
	Asura_ServerNode_GS2Spline* pxGameSpline = s_pxThisToolsSpline->GetGameGS2Spline();

	Asura_ServerNode_GS2Node* pxTargetNode = Asura_ServerNode_GS2Node::FindEntity( pxGameSpline->GetTargetGuid() );
	if ( !pxTargetNode ) { return; }

	if ( pxTargetNode->GetNumberLinks() > 0 )
	{
		Asura_ServerNode_GS2Spline* pxNextSpline = Asura_ServerNode_GS2Spline::FindEntity( pxTargetNode->GetLink( 0 ) );
		if ( pxNextSpline )
		{
			float fSpeed = pxNextSpline->GetSpeedFromTime(0.0f);
			Asura_Graph_Point<float>* pxGraphPoints = pxGameSpline->GetSpeedOverTimeGraph()->GetControlPoints();
			while (pxGraphPoints->m_pxNext->m_pxNext)
			{
				pxGraphPoints = pxGraphPoints->m_pxNext;
			}
			pxGraphPoints->m_xValue = fSpeed * pxGameSpline->GetVelocityMultiplier();
			pxGameSpline->GetSpeedOverTimeGraph()->Update();
			s_pxThisToolsSpline->m_fDurationOverride = pxGameSpline->GetTotalLinkTime();
			s_pxThisToolsSpline->m_fLength = pxGameSpline->GetTotalLength();
		}
	}
}

bool Tools_EntityNode_GS2Spline::CanAutoSetEndSpeed()
{
	Asura_ServerNode_GS2Spline* pxGameSpline = s_pxThisToolsSpline->GetGameGS2Spline();

	Asura_ServerNode_GS2Node* pxTargetNode = Asura_ServerNode_GS2Node::FindEntity( pxGameSpline->GetTargetGuid() );
	return ( pxTargetNode && pxTargetNode->GetNumberLinks() > 0 );
}

void Tools_EntityNode_GS2Spline::CopyStartSpeedToEnd()
{
	Asura_ServerNode_GS2Spline* pxGameSpline = s_pxThisToolsSpline->GetGameGS2Spline();

	const float fStartSpeed = pxGameSpline->GetSpeedOverTimeGraph()->GetValueAtTime(0.0f);

	Asura_Graph_Point<float>* pxGraphPoints = pxGameSpline->GetSpeedOverTimeGraph()->GetControlPoints();
	while ( pxGraphPoints->m_pxNext->m_pxNext )
	{
		pxGraphPoints = pxGraphPoints->m_pxNext;
	}
	pxGraphPoints->m_xValue = fStartSpeed;
	pxGameSpline->GetSpeedOverTimeGraph()->Update();
}

void Tools_EntityNode_GS2Spline::CopyEndSpeedToStart()
{
	Asura_ServerNode_GS2Spline* pxGameSpline = s_pxThisToolsSpline->GetGameGS2Spline();

	const float fEndSpeed = pxGameSpline->GetSpeedOverTimeGraph()->GetValueAtTime(1.0f);

	Asura_Graph_Point<float>* pxGraphPoints = pxGameSpline->GetSpeedOverTimeGraph()->GetControlPoints();
	pxGraphPoints->m_pxNext->m_xValue = fEndSpeed;
	pxGameSpline->GetSpeedOverTimeGraph()->Update();
}

Asura_ServerNode_GS2Spline* Tools_EntityNode_GS2Spline::GetGameGS2Spline() const
{
	return static_cast< Asura_ServerNode_GS2Spline* >( Entity );
}

void Tools_EntityNode_GS2Spline::RefreshOverlay()
{
	const u_int uCurrentTab = m_pxTabsDialogue ? m_pxTabsDialogue->GetSelected() : 0xffffffff;
	DeleteOverlay();
	CreateOverlay();
	if ( uCurrentTab < 0xffffffff )
	{
		m_pxTabsDialogue->SelectTab( static_cast< int >( uCurrentTab ) );
	}
}

bool Tools_EntityNode_GS2Spline::HandlesThisMessage( int iMessageID )
{
	if ( GetGameGS2Spline()->GetBehaviourParams() && GetGameGS2Spline()->GetBehaviourParams()->DoHandlesThisMessage( iMessageID ) )
	{
		return true;
	}

	if ( iMessageID == ASURA_MSG_AXON_GS2_DONT_STOP_AT_NODE ||
		iMessageID == ASURA_MSG_AXON_GS2_TAKE_LINK ||
		iMessageID == ASURA_MSG_AXON_GS2_ANIM_CHANGE ||
		iMessageID == ASURA_MSG_AXON_GS2_EXIT_GAMESCENE ||
		iMessageID == ASURA_MSG_AXON_GS2_PATHFIND_TO_NODE )
	{
		return true;
	}

	return PARENT::HandlesThisMessage( iMessageID );
}

bool Tools_EntityNode_GS2Spline::OwnsThisMessageBlock( unsigned short uMessageBlockID )
{
	if ( GetGameGS2Spline()->GetBehaviourParams() && GetGameGS2Spline()->GetBehaviourParams()->DoOwnsThisMessageBlock( uMessageBlockID ) )
	{
		return true;
	}

	return PARENT::OwnsThisMessageBlock( uMessageBlockID );
}

void Tools_EntityNode_GS2Spline::UpdatePositionInAsuraData( bool bUpdateLinkedObjects )
{
	PARENT::UpdatePositionInAsuraData( bUpdateLinkedObjects );

	GetGameGS2Spline()->UpdateFollowSurface();
}

bool Tools_EntityNode_GS2Spline::Verify( const bool bAutomatic )
{
	bool bOk = PARENT::Verify( bAutomatic );

	if ( GetGameGS2Spline()->GetBehaviourParams() )
	{
		bOk &= GetGameGS2Spline()->GetBehaviourParams()->DoVerify( bAutomatic );
	}

	return bOk;
}

void Tools_EntityNode_GS2Spline::PrintInfo( float& fX, float& fY )
{
	PARENT::PrintInfo( fX, fY );

	if ( m_uOwnerGuid == ASURA_GUID_UNREGISTERED )
	{
		Asura_Text_Output::Print( fX, fY++, 0xffffffff, 0, "Owner Guid : *unset*" );
	}
	else
	{
		Asura_Text_Output::Print( fX, fY++, 0xffffffff, 0, "Owner Guid : %u", m_uOwnerGuid );
	}
	Asura_Text_Output::Print( fX, fY++, 0xffffffff, 0, "Owner Skin : %s", m_xOwnerSkinName.IsBlank() ? "*unset*" : m_xOwnerSkinName.GetString() );
}

void Tools_EntityNode_GS2Spline::Copy( const Tools_Object* pxObject )
{
	// Copy parent data.
	PARENT::Copy( pxObject );

	// Copy our data.
	const Tools_EntityNode_GS2Spline* pxFromObject = static_cast< const Tools_EntityNode_GS2Spline* >( pxObject );

	// Copy the engine entity's data across.
	Asura_ServerNode_GS2Spline* pxToEntity   = static_cast< Asura_ServerNode_GS2Spline* >( Entity );
	Asura_ServerNode_GS2Spline* pxFromEntity = static_cast< Asura_ServerNode_GS2Spline* >( pxFromObject->Entity );

	// Don't copy the speed time graph, or associated data, as we're not copying the spline positions, etc.
    pxToEntity->m_bIgnoreCollisions   = pxFromEntity->m_bIgnoreCollisions;
    pxToEntity->m_bTestForCollisions  = pxFromEntity->m_bTestForCollisions;
    pxToEntity->m_bFollowSurface      = pxFromEntity->m_bFollowSurface;
	pxToEntity->m_bFollowSurfaceTools = pxFromEntity->m_bFollowSurfaceTools;
	pxToEntity->m_iBehaviourSet = pxFromEntity->m_iBehaviourSet;
	delete pxToEntity->m_pxBehaviourParams;
	pxToEntity->m_pxBehaviourParams = pxFromEntity->m_pxBehaviourParams ? pxFromEntity->m_pxBehaviourParams->DoClone() : NULL;
	if ( pxToEntity->m_pxBehaviourParams )
	{
		pxToEntity->m_pxBehaviourParams->DoSetToolsNode( this );
	}
}

void Tools_EntityNode_GS2Spline::SetToDefaultOrientations()
{
	s_pxThisToolsSpline->SetPointsToDefaultOrientations();
}

void Tools_EntityNode_GS2Spline::PickDefaultBehaviourSet( int iThis )
{
	ListSelection_Overlay xBehaviourList( "Pick Behaviour To Use:" );
	Tools_Object_Axon_Interface::GetInstance()->BuildGS2SplineBehaviourSetList( xBehaviourList );
	xBehaviourList.SetInitialSelection( s_iDefaultBehaviourSet );
	if ( xBehaviourList.DoModal() )
	{
		s_iDefaultBehaviourSet = xBehaviourList.GetSelectionData();
		s_bRefreshDefaultsOverlay = true;
	}
}

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

#endif // ASURA_USE_AXON
