//////////////////////////////////////////////////////////////////////////////
//
// Tools_Object_NodeSplinePoint.cpp
//
// Created by KOR - 23/01/2008
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"

// header
#include "Entity/Locations/Spline/Tools_Object_NodeSplinePoint.h"

// asura
#include "Camera/Asura_Camera.h"
#include "Core/Asura_Spline.h"
#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Overlay.h"
#include "Interface/Asura_Overlay_System.h"

// tools
#include "Entity/Locations/Spline/Tools_EntityNode_Spline.h"
#include "Objects/Tools_Object_System.h"

//////////////////////////////////////////////////////////////////////////////
//
// Tools_Object_NodeSplinePoint
//
//////////////////////////////////////////////////////////////////////////////


Tools_Object_NodeSplinePoint* Tools_Object_NodeSplinePoint::s_pxThisSplinePoint = NULL;

Tools_Object_NodeSplinePoint::Tools_Object_NodeSplinePoint(Asura_ServerNode_Spline* pxParentSplineNode,
                                                   Tools_EntityNode_Spline* pxParentSplineTools,
                                                   int iIndex,
                                                   bool bIsEndPoint)
: PARENT(pxParentSplineNode, pxParentSplineTools, iIndex, bIsEndPoint)
, m_pxParentSplineNode(pxParentSplineNode)
, m_pxParentTools(pxParentSplineTools)
{
    Position = *m_pxPositionInAsura;
    Orientation.Identity();

    SetMesh( "ToolsShape_CutsceneTrackPoint" );
}

bool Tools_Object_NodeSplinePoint::CanBeMoved()
{
    if (m_bIsEndPoint)
    {
        const Asura_Guid uGuidWeMustRemainWith = (m_iIndex == 1) ? m_pxParentSplineNode->GetSourceGuid() : m_pxParentSplineNode->GetTargetGuid();

        int iNumberOfSelectedObjects = Tools_Object_System::SelectedObjectList.size();
        for (int i=0;i<iNumberOfSelectedObjects;i++)
        {
            if (Tools_Object_System::SelectedObjectList[i]->GetGuid() == uGuidWeMustRemainWith)
            {
                return true;
            }
        }
        return false;
    }
    return true;
}

void Tools_Object_NodeSplinePoint::UpdatePositionInAsuraData(bool /*updateLinkedObjects*/)
{
    PARENT::UpdatePositionInAsuraData(true);
	m_pxParentTools->SetEntityModified();
	m_pxParentTools->OnSplinePointMoved( m_iIndex );
}

void Tools_Object_NodeSplinePoint::Destroy()
{
	m_pxParentTools->SetEntityModified();
	PARENT::Destroy();
    m_pxParentTools->CreateAllToolsPathObjects();
	// object will have been deleted by CreateAllToolsPathObjects, don't try to do anything else with it.
}

void Tools_Object_NodeSplinePoint::CreateNew(const Asura_Vector_3& xPosition, const int iIndexAfter)
{
	m_pxParentTools->SetEntityModified();
	PARENT::CreateNew(xPosition, iIndexAfter);
    m_pxParentTools->CreateAllToolsPathObjects(iIndexAfter);
	// object will have been deleted by CreateAllToolsPathObjects, don't try to do anything else with it.
}

void Tools_Object_NodeSplinePoint::AddOverlayControls(class Asura_Overlay* pxOverlay, class Asura_Vector_2& xOffset)
{
	PARENT::AddOverlayControls(pxOverlay, xOffset);

	Asura_Dialog* pxNewDialog = NULL;
	pxNewDialog = new Asura_Function_Dialog("Edit Link Properties", xOffset, EditLinkProperties);
	pxNewDialog->SetHelpText("Select and edit the properties of the spline that owns this control point");
	pxOverlay->AddDialog(pxNewDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();
}

void Tools_Object_NodeSplinePoint::OnDeselection()
{
	s_pxThisSplinePoint = NULL;
	SetScale( 1.0f );
	PARENT::OnDeselection();
}

void Tools_Object_NodeSplinePoint::OnSelection()
{
	PARENT::OnSelection();
	s_pxThisSplinePoint = this;
}

void Tools_Object_NodeSplinePoint::UpdateWhileSelected()
{
	PARENT::UpdateWhileSelected();

	UpdateRenderMeshScaling();

	if ( m_bChanged )
	{
		SetEntityModified();
		m_pxParentTools->SetEntityModified();
	}
}

void Tools_Object_NodeSplinePoint::Update()
{
	PARENT::Update();

	// If we're an end point we must make sure we move with the corresponding node.
	if ( m_bIsEndPoint )
	{
		const Asura_Guid uGuidWeMustRemainWith = (m_iIndex == 1) ? m_pxParentSplineNode->GetSourceGuid() : m_pxParentSplineNode->GetTargetGuid();
		const Asura_ServerEntity_Node* pxNode = Asura_ServerEntity_Node::FindEntity( uGuidWeMustRemainWith );
		if ( pxNode )
		{
			if ( !Position.ApproximatelyEquals( pxNode->GetPosition() ) )
			{
				Position = pxNode->GetPosition();
				UpdatePositionInAsuraData( true );
			}
		}
	}
}

void Tools_Object_NodeSplinePoint::EditLinkProperties()
{
	Asura_Assert(s_pxThisSplinePoint && s_pxThisSplinePoint->m_pxParentTools, "Unexpected NULL pointer");
	Tools_Object_System::SelectObject(s_pxThisSplinePoint->m_pxParentTools);
}

void Tools_Object_NodeSplinePoint::SetOrientationToTangent()
{
	const float fDistanceAtThisIndex = m_pxParentSplineNode->GetCumulativeLength( m_iIndex - 1 );
	Asura_Vector_3 xTangent;
	m_pxParentSplineNode->GetTangentFromDistance( fDistanceAtThisIndex, xTangent );

	Asura_Maths::OrientationFromDirection( Orientation, xTangent );
	UpdatePositionInAsuraData( true );
}

void Tools_Object_NodeSplinePoint::UpdateRenderMeshScaling()
{
	SetScale( Tools_EntityNode_Spline::GetMeshScaleFactor( Position ) );
}

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////
