//////////////////////////////////////////////////////////////////////////////
//
// Tools_Object_PFXSplinePoint.cpp
//
// Created by KOR - 23/01/2008
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"

// header
#include "Entity/Locations/Spline/Tools_Object_PFXSplinePoint.h"

// asura
#include "Camera/Asura_Camera.h"
#include "Core/Asura_Spline.h"
#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Overlay.h"
#include "Interface/Asura_Overlay_System.h"

// tools
#include "Entity/Tools_Entity_PFX_Effect.h"
#include "Objects/Tools_Object_System.h"

//////////////////////////////////////////////////////////////////////////////
//
// Tools_Object_PFXSplinePoint
//
//////////////////////////////////////////////////////////////////////////////


Tools_Object_PFXSplinePoint* Tools_Object_PFXSplinePoint::s_pxThisSplinePoint = NULL;

Tools_Object_PFXSplinePoint::Tools_Object_PFXSplinePoint(Asura_Spline<Asura_Vector_3>* pxParentSpline,
                                                   Tools_Entity_PFX_Effect* pxParentSplineTools,
                                                   int iIndex,
                                                   bool bCanBeDestroyed)
: PARENT(pxParentSpline, pxParentSplineTools, iIndex, bCanBeDestroyed)
, m_pxParentTools(pxParentSplineTools)
{
}

Tools_Object_PFXSplinePoint::~Tools_Object_PFXSplinePoint()
{
}

void Tools_Object_PFXSplinePoint::UpdatePositionInAsuraData(bool /*updateLinkedObjects*/)
{
    PARENT::UpdatePositionInAsuraData(true);
	m_pxParentTools->ChangedSpline();
}

void Tools_Object_PFXSplinePoint::Destroy()
{
	m_pxParentTools->SetEntityModified();
	PARENT::Destroy();
	m_pxParentTools->ChangedSpline();
    m_pxParentTools->CreateSplinePoints();
	// object will have been deleted by CreateSplinePoints, don't try to do anything else with it.
}

bool Tools_Object_PFXSplinePoint::CanBeMoved()
{
	if (m_iIndex == 1)
	{
		const Asura_Guid uGuidWeMustRemainWith = m_pxParentTools->GetGuid();

		int iNumberOfSelectedObjects = Tools_Object_System::SelectedObjectList.size();
		for (int i=0;i<iNumberOfSelectedObjects;i++)
		{
			if (Tools_Object_System::SelectedObjectList[i]->GetGuid() == uGuidWeMustRemainWith)
			{
				return true;
			}
		}
		return false;
	}
	return true;
}

void Tools_Object_PFXSplinePoint::CreateNew(const Asura_Vector_3& xPosition, const int iIndexAfter)
{
	m_pxParentTools->SetEntityModified();
	PARENT::CreateNew(xPosition, iIndexAfter);
	m_pxParentTools->ChangedSpline();
    m_pxParentTools->CreateSplinePoints(iIndexAfter);
	// object will have been deleted by CreateSplinePoints, don't try to do anything else with it.
}

void Tools_Object_PFXSplinePoint::AddOverlayControls(class Asura_Overlay* pxOverlay, class Asura_Vector_2& xOffset)
{
	PARENT::AddOverlayControls(pxOverlay, xOffset);

	Asura_Dialog* pxNewDialog = NULL;
	pxNewDialog = new Asura_Function_Dialog("Select PFX Effect", xOffset, SelectEffect);
	pxNewDialog->SetHelpText("Select and edit the properties of the PFX effect that owns this control point");
	pxOverlay->AddDialog(pxNewDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();
}

void Tools_Object_PFXSplinePoint::OnDeselection()
{
	s_pxThisSplinePoint = NULL;
	PARENT::OnDeselection();
}

void Tools_Object_PFXSplinePoint::OnSelection()
{
	PARENT::OnSelection();
	s_pxThisSplinePoint = this;
}

void Tools_Object_PFXSplinePoint::UpdateWhileSelected()
{
	const bool bChanged = m_bChanged;
	PARENT::UpdateWhileSelected();
	if(bChanged)
	{
		m_pxParentTools->ChangedSpline();
	}
}

void Tools_Object_PFXSplinePoint::SelectEffect()
{
	Asura_Assert(s_pxThisSplinePoint && s_pxThisSplinePoint->m_pxParentTools, "Unexpected NULL pointer");
	Tools_Object_System::SelectObject(s_pxThisSplinePoint->m_pxParentTools);
}
