//
// Tools_Entity_AdvancedVolumeTrigger.h
//
// Created by Andrew Weinkove - 30/01/2003
// 
// Copyright (C) 2003 Rebellion, All Rights Reserved.
//


#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef _TOOLS_ENTITY_ADVANCEDVOLUMETRIGGER_H_
#define _TOOLS_ENTITY_ADVANCEDVOLUMETRIGGER_H_

#include "Entity/Triggers/Asura_ServerEntity_AdvancedVolumeTrigger.h"

#include "Tools_Entity_Trigger_Object.h"
#include "Objects/Tools_VolumeEdit_Object.h"


class Tools_AdvancedVolumeTrigger_VolumeEdit_Controller;

//////////////////////////////////////////////////////////////////////////////////////////////////
/*-----------------------------------**
** Tools_Entity_AdvancedVolumeTrigger **
**-----------------------------------*/

class Tools_Entity_AdvancedVolumeTrigger : public Tools_Entity_Trigger_Object
{
	public :
		Tools_Entity_AdvancedVolumeTrigger(Asura_Entity* pxEntity);
		~Tools_Entity_AdvancedVolumeTrigger();
		virtual bool UsesUnTriggerMessage() { return true; }
		static void Project_Create();

		#ifndef TOOLS_USE_OLD_ENTITY_MANAGER
		virtual const char* GetShortName(){return "VT";}
		#endif
	
		//virtual function overides
		const char* GetTypeDescription(){return "Advanced Volume Trigger";}
		void UpdatePositionInAsuraData(bool /*updateLinkedObjects*/);
		void Update();

		static const char* GetNamePrefix() { return "AVT_"; }
		virtual const char* GetMyNamePrefix() const { return Tools_Entity_AdvancedVolumeTrigger::GetNamePrefix(); }

		void AddOverlayControls(class Asura_Overlay* pxNewOverlay,class Asura_Vector_2& xOffset);

		virtual void Copy(const Tools_Object*);
		virtual bool CanBeCopied() const {return Type == TOT_Entity_AdvancedVolumeTrigger;}
		virtual Tools_Object* CreateObjectOfThisType() const;

		virtual void UpdateClient();

	protected:
		static void EditVolume();
		void ShowVolume(bool bShow);

		static bool s_bMoveVolumeWithObject;
		static bool s_bShowAllVolumes;
	private :
		Asura_Vector_3 m_xPreviousPosition;

		Tools_AdvancedVolumeTrigger_VolumeEdit_Controller* m_pxVolumeController;

};

//////////////////////////////////////////////////////////////////////////////////////////////////
/*---------------------------------------------------**
** Tools_AdvancedVolumeTrigger_VolumeEdit_Controller **
**---------------------------------------------------*/

//For dealing with editing the volumes attached to the trigger
class Tools_AdvancedVolumeTrigger_VolumeEdit_Controller : public Tools_VolumeEdit_Controller
{
	public :
		Tools_AdvancedVolumeTrigger_VolumeEdit_Controller(Tools_Entity_AdvancedVolumeTrigger* pxToolsEntity);

		virtual void StartVolumeEditing();
		virtual void EndVolumeEditing();
	
	protected :
		virtual bool CanCreateVolume()
		{return true;}
		
		virtual bool CanDeleteVolume()
		{return GetNumberOfVolumes()>1;}
		
		

		//Gets the current number of volumes
		virtual int GetNumberOfVolumes()
		{return m_pxAsuraEntity->m_iNumberOfVolumes;}
		
		//Gets a pointer to the bounding box for a given volume
		virtual Asura_Bounding_Box* GetVolumeBox(int iIndex)
		{return &(m_pxAsuraEntity->m_pxVolumes[iIndex].m_xBox);}

		//Gets a pointer to the orientation for a given volume. Returns 0 if there is none
		virtual Asura_Quat* GetVolumeOrient(int iIndex)
		{return &(m_pxAsuraEntity->m_pxVolumes[iIndex].m_xOrient);}

		//Gets a default position for volume objects that have yet to gain a proper volume
		virtual Asura_Vector_3 GetDefaultPosition()
		{return m_pxToolsEntity->Position;}

		//Gets the name of the owner , to use for part of the volume names
		virtual const char* GetName()
		{return m_pxToolsEntity->GetName();}

		//Creates the new volume in the asura data
		virtual void CreateVolumeMain();

		//Deletes the volume from the asura data
		virtual void DeleteVolumeMain(int iIndex);

	private :
		Tools_Entity_AdvancedVolumeTrigger*	m_pxToolsEntity;
		Asura_ServerEntity_AdvancedVolumeTrigger* m_pxAsuraEntity;

};




#endif //_TOOLS_ENTITY_ADVANCEDVOLUMETRIGGER_H_
