
///////////////////////////////////////////////////////////////////////////////
//
// File   : Tools_Entity_Affector.h
// Author : Andy Tate
// Date   : 6-Dec-2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __TOOLS_ENTITY_AFFECTOR_H__
#define __TOOLS_ENTITY_AFFECTOR_H__

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Entity/Tools_Entity_Object.h"


#if defined(ASURA_USE_AFFECTORS)

////////////////////////////////////////////////////////////////////////////////
// Forward class declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Class declaration...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Class        : Tools_Entity_Affector
// Author       : Andy Tate
// Date         : 6-Dec-2006
//
// Description  : Class to handle the Tools editing of affectors.
////////////////////////////////////////////////////////////////////////////////

class Tools_Entity_Affector : public Tools_Entity_Object
{
public:
	// Public functions...
	Tools_Entity_Affector          (Asura_Entity* pxEntity);
//	virtual ~Tools_Entity_Affector ();

	// Overridden functions...
	virtual	const char*	GetTypeDescription	(void)	{ return("Affector"); }

	virtual	void		AddOverlayControls	(Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset);
	virtual	void		UpdateWhileSelected	(void);
	virtual	bool		HandlesThisMessage	(int iMessageID);
	virtual	void		PrintInfo			(float& fX, float& fY);

	virtual const char*	GetMyNamePrefix		() const				{ return Tools_Entity_Affector::GetNamePrefix(); }

	// Overridden copy functions...
	virtual	bool			CanBeCopied				(void) const	{ return(true); }
	virtual	void			Copy					(const Tools_Object* pxObject);
	virtual	Tools_Object*	CreateObjectOfThisType	(void) const;

	// Overridden functions to allow us to use a sphere...
	virtual	bool	ObjectIsInLOS	(const Asura_Vector_3& xOrigin, const Asura_Vector_3& xDirection, float& fLowest);
	virtual	void	Render			(void);

	// Static functions...
	static void		Create	(void);

	static const char* GetNamePrefix()								{ return "Aff_"; }

	virtual void SetEntityModified();

protected:
	// Protected variables...
	float	m_fRange;		// The range of this affector.
	float	m_fStrength;	// The strength of this affector.

	// Protected functions...


private:
	// Private typedefs...
	typedef Tools_Entity_Object PARENT;

	// Private variables...

	// Private functions...
	Tools_Entity_Affector  (void);
	Tools_Entity_Affector  (const Tools_Entity_Affector& xOther);

	Tools_Entity_Affector&	operator=	(const Tools_Entity_Affector& xOther);

};   // End of class 'Tools_Entity_Affector'.

#endif	// defined(ASURA_USE_AFFECTORS)


#endif   // ifndef __TOOLS_ENTITY_AFFECTOR_H__


/// End of file. ///
