//
// Tools_Entity_ClientVolumeTrigger_Object.cpp
//
// Created by Richard Rice - 24/11/2004
// 
// Copyright (C) 2004 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"
#include "Entity/Tools_Entity_ClientVolumeTrigger_Object.h"

#include "Entity/Triggers/Asura_ClientEntity_ClientVolumeTrigger.h"
#include "Entity/Triggers/Asura_ServerEntity_ClientVolumeTrigger.h"
#include "Mesh/Asura_Mesh_System.h"

#include "Load&Save/Tools_SaveAsura.h"
#include "Objects/Tools_Object_System.h"

Tools_Entity_ClientVolumeTrigger_Object::Tools_Entity_ClientVolumeTrigger_Object(Asura_Entity* pxEntity)
:Tools_Entity_AdvancedVolumeTrigger(pxEntity)
{
	Type = TOT_Entity_ClientVolumeTrigger;
	Mesh = Asura_Mesh_System::GetMesh("ToolsShape_VBlue");
	if(!Mesh) Mesh = Asura_Mesh_System::GetMesh("ToolsShape_V");
}


void Tools_Entity_ClientVolumeTrigger_Object::Create()
{
	Tools_Entity_ClientVolumeTrigger_Object* pxObject = Create(false);
	
	if(pxObject) Tools_Object_System::SelectObject( pxObject );
}

Tools_Entity_ClientVolumeTrigger_Object* Tools_Entity_ClientVolumeTrigger_Object::Create(bool bGenerateName)
{
	//get a name , and the position data
	Asura_Guid uGuid = Asura_Entity_System::GetNextToolsCreatedGuid();
	Tools_Entity_Data* pxToolsData = 0;
	if(bGenerateName)
	{
		pxToolsData = Tools_Entity_Object::CreateToolsDataWithDefaultName("Client Volume Trigger","Client Volume Trigger",uGuid,GetNamePrefix());
	}
	else
	{
		pxToolsData = Tools_Entity_Object::CreateToolsData("Client Volume Trigger",0,GetNamePrefix());
	}
	if(!pxToolsData) return 0;
	
	

	Asura_Bounding_Box xDefaultBB;
	xDefaultBB.Zero();
	
	// Create the game specfic server object and set its data.
    Asura_ServerEntity_ClientVolumeTrigger* pxServerObject = new Asura_ServerEntity_ClientVolumeTrigger(xDefaultBB);
    new Asura_ClientEntity_ClientVolumeTrigger(pxServerObject->GetGuid());

	pxServerObject->ToolsData = pxToolsData;

	// Create the tools object.
	Tools_Entity_ClientVolumeTrigger_Object* pxObject = new Tools_Entity_ClientVolumeTrigger_Object(pxServerObject);

	//update the client from server
	pxObject->UpdateClient();

    Tools_SaveAsura::SetEntityModified();

	return pxObject;

}




void Tools_Entity_ClientVolumeTrigger_Object::Copy(const Tools_Object* pxObject)
{
	Tools_Entity_AdvancedVolumeTrigger::Copy(pxObject);

	//nothing to copy , but the client entity will need updating
	UpdateClient();

}

Tools_Object* Tools_Entity_ClientVolumeTrigger_Object::CreateObjectOfThisType() const
{
	return Create(true);	
}
