//
// Tools_Entity_CountedTrigger_Object.cpp
//
// Created by Richard Rice - 20/09/2006
// 
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"
#include "Tools_Entity_CountedTrigger_Object.h"

#include "Debugging/Asura_Assert.h"
#include "Entity/Asura_Entity_Classification.h"
#include "Entity/Asura_StaticMessage_System.h"
#include "Entity/Triggers/Asura_ServerEntity_CountedTrigger.h"
#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Overlay.h"
#include "Mesh/Asura_Mesh_System.h"
#include "Rendering/Asura_Text_Output.h"

#include "Entity/Tools_Entity_Data.h"
#include "Entity/Tools_Message_Editing.h"
#include "Entity/Tools_Entity_Simulation.h"
#include "Objects/Tools_Object_System.h"
#include "Load&Save/Tools_SaveAsura.h"

#ifdef ASURA_TOOLS_USE_NEW_MESSAGE_EDITOR
#include "Entity/Tools_MessageEditor.h"
#endif



Tools_Entity_CountedTrigger_Object::Tools_Entity_CountedTrigger_Object(Asura_Entity* pxEntity)
:Tools_Entity_Trigger_Object(pxEntity)
{

//  AMC (19/12/2006) - Removed the assert below, as any project tools class inherited from this class will cause it to fire
//	Asura_Assert(pxEntity->GetClassification()==AsuraEntityClass_CountedTrigger,"");
	Type = TOT_Entity_CountedTrigger;
	Mesh = Asura_Mesh_System::GetMesh("ToolsShape_CountedTrigger");

	m_pbActive = 0;
	m_pbRetriggerable = 0;
	m_piTriggerCount = 0;
	AllocateArrays();

}

Tools_Entity_CountedTrigger_Object::~Tools_Entity_CountedTrigger_Object()
{
	delete [] m_pbActive;
	delete [] m_pbRetriggerable;
	delete [] m_piTriggerCount;
}

void Tools_Entity_CountedTrigger_Object::AddOverlayControls(class Asura_Overlay* pxOverlay,class Asura_Vector_2& xOffset)
{
	Tools_Entity_Trigger_Object::AddOverlayControls(pxOverlay,xOffset);
	xOffset.y += Asura_Dialog::LargeSpacing()-Asura_Dialog::DefaultSpacing();

	Asura_Dialog* pxDialog;

	Asura_ServerEntity_CountedTrigger* pxAsuraEntity = (Asura_ServerEntity_CountedTrigger*) Entity;


	char szText[100];

	//add/delete messages
	pxDialog = new Asura_Dialog("Messages :",xOffset);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::SmallSpacing();

	pxDialog = new Asura_Function_Dialog("Add",xOffset,AddMessage,(int)this);
 	pxDialog->SetHelpText("Add a new message block to this trigger.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.x = pxDialog->m_xMaxCoord.x + 1.0f;

	pxDialog = new Asura_Function_Dialog("Delete",xOffset,DeleteMessage,(int)this);
 	pxDialog->SetHelpText("Deletes a message block from this trigger.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.x = pxDialog->m_xMaxCoord.x + 1.0f;

	pxDialog = new Asura_Function_Dialog("Rename",xOffset,RenameMessage,(int)this);
 	pxDialog->SetHelpText("Renames a message block.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();
	xOffset.x = 1.0f;




	if(pxAsuraEntity->m_uNumberOfMessageBlocks)
	{
		//do some headings
		xOffset.x += 2.0f;
		pxDialog = new Asura_Dialog("Counter",xOffset);
		pxOverlay->AddDialog(pxDialog);
		xOffset.x += 7.0f;

		pxDialog = new Asura_Dialog("Enbl.",xOffset);
		pxDialog->SetHelpText("Enabled");
		pxOverlay->AddDialog(pxDialog);
		xOffset.x += 4.0f;

		pxDialog = new Asura_Dialog("Retr.",xOffset);
		pxDialog->SetHelpText("Retriggerable");
		pxOverlay->AddDialog(pxDialog);
		xOffset.x += 7.5f;


		xOffset.x = 1.0f;
		xOffset.y += Asura_Dialog::DefaultSpacing();
	}


	Asura_CountedTrigger_MessageBlock* pxMessageBlock = pxAsuraEntity->m_pxMessageBlocks;
	for(unsigned i=0;i<pxAsuraEntity->m_uNumberOfMessageBlocks;i++,pxMessageBlock++)
	{
		pxDialog = new Asura_Numerical_Dialog("",xOffset,&m_piTriggerCount[i],1,9999,1,"%d",&Changed);
		pxDialog->SetHelpText("The count at which this message block will be triggered.");
		pxOverlay->AddDialog(pxDialog);
		xOffset.x = pxDialog->m_xMaxCoord.x;

		pxDialog = new Asura_Tickbox_Dialog("",xOffset,&m_pbActive[i],&Changed);
	 	pxDialog->SetHelpText("Indicates whether this message block is initially enabled.\nIf disabled at the point the count is reached , then the messages won't be sent.");
		pxOverlay->AddDialog(pxDialog);
		xOffset.x = pxDialog->m_xMaxCoord.x;

		pxDialog = new Asura_Tickbox_Dialog("",xOffset,&m_pbRetriggerable[i],&Changed);
	 	pxDialog->SetHelpText("Indicates whether this message block is retriggerable.\nIf not retriggerable then the message block will become disabled when it triggers.");
		pxOverlay->AddDialog(pxDialog);
		xOffset.x = pxDialog->m_xMaxCoord.x;


 		Asura_PrintfToBfr(szText,"%s (%d)",100,true,pxMessageBlock->m_xName.GetString(),Asura_StaticMessage_System::GetNumberOfMessagesInBlock(pxMessageBlock->m_usMessageBlock));
		pxDialog = new Asura_Function_Dialog(szText,xOffset,EditMessages,(int)i);
		pxDialog->SetHelpText("Edit the messages to be sent for this time.");
		pxOverlay->AddDialog(pxDialog);

		xOffset.x = 1.0f;
		xOffset.y += Asura_Dialog::LargeSpacing();

	}
}


void Tools_Entity_CountedTrigger_Object::AddOverlayControls_EnableMessage(struct Asura_StaticMessage* pxMessage,class Asura_Overlay* pxOverlay,class Asura_Vector_2& xOffset)
{
	Asura_Assert(pxMessage->m_pData,"");
	Asura_Message_CountedTrigger_MessageHash* pxData = (Asura_Message_CountedTrigger_MessageHash*) pxMessage->m_pData;

	Asura_Dialog* pxDialog = 0;

	//find the entity
	Asura_ServerEntity_CountedTrigger* pxEntity = Asura_ServerEntity_CountedTrigger::FindEntity(Tools_Message_Editing::GetFinalDestinationGuid(pxMessage));
	if(!pxEntity)
	{
		pxDialog = new Asura_Dialog("*Unknown Target*",xOffset);	
		pxOverlay->AddDialog(pxDialog);
		xOffset.y += Asura_Dialog::DefaultSpacing();
		return;
	}


	Asura_Assert(pxEntity->ToolsData && pxEntity->ToolsData->ToolsObject,"");
	Tools_Entity_CountedTrigger_Object* pxObject = (Tools_Entity_CountedTrigger_Object*) pxEntity->ToolsData->ToolsObject;

	//get the name
	const char* pszName;
	char szText[100];
	if(pxData->m_uMessageHash)
	{
		pszName = pxObject->GetMessageBlockNameFromHash(pxData->m_uMessageHash);
	}
	else
	{
		pszName = "*Not Set*";
	}

	Asura_PrintfToBfr(szText,"Message Block : %s",100,true,pszName);
	pxDialog = new Asura_Function_Dialog(szText,xOffset,EditMessageBlockHash,(int)pxMessage);
	pxDialog->SetHelpText("Select message block to enable/disable");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();
}



void Tools_Entity_CountedTrigger_Object::AddOverlayControls_SetTriggerCount(struct Asura_StaticMessage* pxMessage,class Asura_Overlay* pxOverlay,class Asura_Vector_2& xOffset)
{
	Asura_Assert(pxMessage->m_pData,"");
	Asura_Message_CountedTrigger_SetTriggerCount* pxData = (Asura_Message_CountedTrigger_SetTriggerCount*) pxMessage->m_pData;

	Asura_Dialog* pxDialog = 0;

	//find the entity
	Asura_ServerEntity_CountedTrigger* pxEntity = Asura_ServerEntity_CountedTrigger::FindEntity(Tools_Message_Editing::GetFinalDestinationGuid(pxMessage));
	if(!pxEntity)
	{
		pxDialog = new Asura_Dialog("*Unknown Target*",xOffset);	
		pxOverlay->AddDialog(pxDialog);
		xOffset.y += Asura_Dialog::DefaultSpacing();
		return;
	}


	Asura_Assert(pxEntity->ToolsData && pxEntity->ToolsData->ToolsObject,"");
	Tools_Entity_CountedTrigger_Object* pxObject = (Tools_Entity_CountedTrigger_Object*) pxEntity->ToolsData->ToolsObject;

	//get the name
	const char* pszName;
	char szText[100];
	if(pxData->m_uMessageHash)
	{
		pszName = pxObject->GetMessageBlockNameFromHash(pxData->m_uMessageHash);
	}
	else
	{
		pszName = "*Not Set*";
	}

	Asura_PrintfToBfr(szText,"Message Block : %s",100,true,pszName);
	pxDialog = new Asura_Function_Dialog(szText,xOffset,EditMessageBlockHash,(int)pxMessage);
	pxDialog->SetHelpText("Select message block to change");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Trigger Count",xOffset,&pxData->m_iTriggerCount,1,9999,1,"%d");
	pxDialog->SetHelpText("The new value at which the message should trigger");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();
}

void Tools_Entity_CountedTrigger_Object::AddOverlayControls_SetCounter(struct Asura_StaticMessage* pxMessage,class Asura_Overlay* pxOverlay,class Asura_Vector_2& xOffset)
{
	Asura_Assert(pxMessage->m_pData,"");
	Asura_Message_CountedTrigger_SetCounter* pxData = (Asura_Message_CountedTrigger_SetCounter*) pxMessage->m_pData;

	Asura_Dialog* pxDialog = 0;

	pxDialog = new Asura_Numerical_Dialog("Counter",xOffset,&pxData->m_iCounter,0,9999,1,"%d");
	pxDialog->SetHelpText("Set the current value of the counter. If increased then it can cause the messages to be triggered (if the trigger is active).");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();
}

void Tools_Entity_CountedTrigger_Object::AddOverlayControls_AddCounter(struct Asura_StaticMessage* pxMessage,class Asura_Overlay* pxOverlay,class Asura_Vector_2& xOffset)
{
	Asura_Assert(pxMessage->m_pData,"");
	Asura_Message_CountedTrigger_AddCounter* pxData = (Asura_Message_CountedTrigger_AddCounter*) pxMessage->m_pData;

	Asura_Dialog* pxDialog = 0;

	pxDialog = new Asura_Numerical_Dialog("Counter Increment",xOffset,&pxData->m_iChange,-9999,9999,1,"%d");
	pxDialog->SetHelpText("Increases/decreases the current value of the counter. If increased then it can cause the messages to be triggered (if the trigger is active).");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();
}




void Tools_Entity_CountedTrigger_Object::Create()
{
	Tools_Object* pxObject = Create(false);
	Tools_Object_System::SelectObject(pxObject);
}

Tools_Entity_CountedTrigger_Object* Tools_Entity_CountedTrigger_Object::Create(bool bGenerateName)
{
	//get a name , and the position data
	Asura_Guid uGuid = Asura_Entity_System::GetNextToolsCreatedGuid();
	Tools_Entity_Data* pxToolsData = 0;
	if(bGenerateName)
	{
		pxToolsData = Tools_Entity_Object::CreateToolsDataWithDefaultName("Counted Trigger","Counted Trigger",uGuid,GetNamePrefix());
	}
	else
	{
		pxToolsData = Tools_Entity_Object::CreateToolsData("Counted Trigger",0,GetNamePrefix());
	}
	if(!pxToolsData) return 0 ;

	//create the asura data
	Asura_ServerEntity_CountedTrigger* pxAsuraEntity = new Asura_ServerEntity_CountedTrigger();
	pxAsuraEntity->ToolsData = pxToolsData;

	//create the tools pxObject
	Tools_Entity_CountedTrigger_Object* pxObject = new Tools_Entity_CountedTrigger_Object(pxAsuraEntity);

	Tools_SaveAsura::SetEntityModified();

	return pxObject;
}



void Tools_Entity_CountedTrigger_Object::Destroy()
{
	Asura_ServerEntity_CountedTrigger* pxAsuraEntity = (Asura_ServerEntity_CountedTrigger*) Entity;
	//clear out any message blocks associated with this

	for(u_int i=0;i<pxAsuraEntity->m_uNumberOfMessageBlocks;i++)
	{
		Asura_StaticMessage_System::DeleteMessageBlock(pxAsuraEntity->m_pxMessageBlocks[i].m_usMessageBlock);
		pxAsuraEntity->m_pxMessageBlocks[i].m_usMessageBlock = ASURA_STATICMESSAGE_EMPTYBLOCK;
	}

	Tools_Entity_Trigger_Object::Destroy();
}

void Tools_Entity_CountedTrigger_Object::UpdateWhileSelected()
{
	Asura_ServerEntity_CountedTrigger* pxAsuraEntity = (Asura_ServerEntity_CountedTrigger*) Entity;
		
	if(Changed)
	{
		//update the asura data
		for(unsigned i=0;i<pxAsuraEntity->m_uNumberOfMessageBlocks;i++)
		{
	   		pxAsuraEntity->m_pxMessageBlocks[i].m_bActive = m_pbActive[i];
	   		pxAsuraEntity->m_pxMessageBlocks[i].m_bRetriggerable = m_pbRetriggerable[i];
	   		pxAsuraEntity->m_pxMessageBlocks[i].m_iTriggerCount = m_piTriggerCount[i];
	   	}

		#ifdef ASURA_TOOLS_USE_NEW_MESSAGE_EDITOR
		Tools_MessageEditor::UpdateMessageEditorTable();
		#endif
	}

	Tools_Entity_Trigger_Object::UpdateWhileSelected();
}

bool Tools_Entity_CountedTrigger_Object::HandlesThisMessage(int iMessageID)
{
	if(iMessageID == ASURA_MSG_COUNTEDTRIGGER_ENABLEMESSAGE ||
	   iMessageID == ASURA_MSG_COUNTEDTRIGGER_DISABLEMESSAGE ||
	   iMessageID == ASURA_MSG_COUNTEDTRIGGER_SETCOUNTER ||
	   iMessageID == ASURA_MSG_COUNTEDTRIGGER_ADDCOUNTER ||
	   iMessageID == ASURA_MSG_COUNTEDTRIGGER_SETTRIGGERCOUNT)
	{
		return true;
	}
	return Tools_Entity_Trigger_Object::HandlesThisMessage(iMessageID); 
}


void Tools_Entity_CountedTrigger_Object::AddMessage()
{
	Asura_Assert(ThisEntity,"");
	Asura_ServerEntity_CountedTrigger* pxAsuraEntity = (Asura_ServerEntity_CountedTrigger*) Entity;

	//get a name for the new message block
	GetString_Overlay xOverlay("Enter name for the new message block","");
	if(!xOverlay.DoModal()) return;

	const char* pszName = xOverlay.GetString();
	if(!strlen(pszName)) return;

	//make sure this name is okay
	Asura_Hash_ID uHash = Asura_GetHashID(pszName);
	for(u_int i=0;i<pxAsuraEntity->m_uNumberOfMessageBlocks;i++)
	{
		if(pxAsuraEntity->m_pxMessageBlocks[i].m_uHashID == uHash)
		{
			ShowMessageOverlay("A message block with that name already exists.");
			return;
		}
	}
	
	
	//allocate an enlarged array of message blocks , and copy over the old values
	Asura_CountedTrigger_MessageBlock* pxMessageBlocks = new Asura_CountedTrigger_MessageBlock[pxAsuraEntity->m_uNumberOfMessageBlocks+1];
	for(unsigned i=0;i<pxAsuraEntity->m_uNumberOfMessageBlocks;i++)
	{
		pxMessageBlocks[i] = pxAsuraEntity->m_pxMessageBlocks[i];
	}

	//add the new input
	int iIndex = pxAsuraEntity->m_uNumberOfMessageBlocks;
	pxMessageBlocks[iIndex].SetDefaultValues();
	pxMessageBlocks[iIndex].m_xName = pszName;
	pxMessageBlocks[iIndex].m_uHashID = uHash;

	delete [] pxAsuraEntity->m_pxMessageBlocks;
	pxAsuraEntity->m_pxMessageBlocks = pxMessageBlocks;
	pxAsuraEntity->m_uNumberOfMessageBlocks++;


	//update tools copy of the data
	AllocateArrays();

	//update the overlay
	ThisEntity->DeleteOverlay();
	ThisEntity->CreateOverlay();

	#ifdef ASURA_TOOLS_USE_NEW_MESSAGE_EDITOR
	Tools_MessageEditor::UpdateMessageEditorTable();
	#endif
}

void Tools_Entity_CountedTrigger_Object::DeleteMessage()
{
	DeleteMessage((int) this, -1);
}

void Tools_Entity_CountedTrigger_Object::DeleteMessage(int iThis, int iIndex)
{
	Asura_ServerEntity_CountedTrigger* pxAsuraEntity = (Asura_ServerEntity_CountedTrigger*) Entity;

	if(iIndex == -1)
	{
		//ask which input to delete
		iIndex = SelectMessageIndex("Delete which message block?");
		if(iIndex<0) return;
	}

	//delete the message block from static messages system
	Asura_StaticMessage_System::DeleteMessageBlock(pxAsuraEntity->m_pxMessageBlocks[iIndex].m_usMessageBlock);

	
	//shuffle down the remaining inputs
	for(u_int i=iIndex;i<pxAsuraEntity->m_uNumberOfMessageBlocks-1;i++)
	{
		pxAsuraEntity->m_pxMessageBlocks[i] = pxAsuraEntity->m_pxMessageBlocks[i+1];
	}

	pxAsuraEntity->m_uNumberOfMessageBlocks--;

	AllocateArrays();

	//overlay needs updating
	DeleteOverlay();
	CreateOverlay();

	SetEntityModified();

	#ifdef ASURA_TOOLS_USE_NEW_MESSAGE_EDITOR
	Tools_MessageEditor::UpdateMessageEditorTable();
	#endif
}

void Tools_Entity_CountedTrigger_Object::RenameMessage()
{
	RenameMessage((int) this, -1);
}

void Tools_Entity_CountedTrigger_Object::RenameMessage(int iThis, int iIndex)
{
	Asura_ServerEntity_CountedTrigger* pxAsuraEntity = (Asura_ServerEntity_CountedTrigger*) Entity;

	//ask which message to rename
	if(iIndex == -1)
	{
		iIndex = SelectMessageIndex("Rename which message?");
		if(iIndex<0) return;
	}
	
	Asura_CountedTrigger_MessageBlock* pxMessage = &pxAsuraEntity->m_pxMessageBlocks[iIndex];
	Asura_Hash_ID uOldHash = pxMessage->m_uHashID;

	//ask for the new input name
	GetString_Overlay xOverlay("Enter new name for message",pxMessage->m_xName.GetString());
	if(!xOverlay.DoModal()) return;

	const char* szNewName = xOverlay.GetString();
	if(!szNewName) return;

	//make sure the name doesn't clash with any other messages
	Asura_Hash_ID uNewHash = Asura_GetHashID(szNewName);

	for(u_int i=0;i<pxAsuraEntity->m_uNumberOfMessageBlocks;i++)
	{
		if(i==(u_int)iIndex) continue;
		
		if(pxAsuraEntity->m_pxMessageBlocks[i].m_uHashID == uNewHash)
		{
			ShowMessageOverlay("A message with that name already exists.");
			return;
		}
	}
	
	//update the message with the new name
	pxMessage->m_xName = szNewName;
	pxMessage->m_uHashID = uNewHash;



	//update any static messages that were targeting the input
	for(Tools_StaticMessageIterator xMessageIt;!xMessageIt.Done();xMessageIt.Next())
	{
		Asura_StaticMessage* pxMessage = xMessageIt.GetCurrent();
		//message targets this?
		if(pxMessage->m_uSendTo != pxAsuraEntity->GetGuid()) continue;
		//is it a counted trigger message affecting message?
		if(pxMessage->m_usMsgID == ASURA_MSG_COUNTEDTRIGGER_ENABLEMESSAGE ||
			pxMessage->m_usMsgID == ASURA_MSG_COUNTEDTRIGGER_DISABLEMESSAGE ||
			pxMessage->m_usMsgID == ASURA_MSG_COUNTEDTRIGGER_SETTRIGGERCOUNT)
		{
			Asura_Message_CountedTrigger_MessageHash* pxData = (Asura_Message_CountedTrigger_MessageHash*) pxMessage->m_pData;

			//update the targeted message hash , and set as modified
			if(pxData->m_uMessageHash == uOldHash)
			{
				pxData->m_uMessageHash = uNewHash;
				xMessageIt.SetModified();
			}
		}
	}
	

	//overlay needs updating
	DeleteOverlay();
	CreateOverlay();

	SetEntityModified();

	#ifdef ASURA_TOOLS_USE_NEW_MESSAGE_EDITOR
	Tools_MessageEditor::UpdateMessageEditorTable();
	#endif
}


bool Tools_Entity_CountedTrigger_Object::OwnsThisMessageBlock(unsigned short usBlockID)
{
	Asura_ServerEntity_CountedTrigger* pxAsuraEntity = (Asura_ServerEntity_CountedTrigger*) Entity;

	for(u_int i=0;i<pxAsuraEntity->m_uNumberOfMessageBlocks;i++)
	{
		if(pxAsuraEntity->m_pxMessageBlocks[i].m_usMessageBlock == usBlockID) return true;
	}
	
	return Tools_Entity_Trigger_Object::OwnsThisMessageBlock(usBlockID);
}

bool Tools_Entity_CountedTrigger_Object::GetEventDescForMessageBlock(unsigned short usBlockID,Asura_String& xOutputDesc)
{
	Asura_ServerEntity_CountedTrigger* pxAsuraEntity = (Asura_ServerEntity_CountedTrigger*) Entity;

	for(u_int i=0;i<pxAsuraEntity->m_uNumberOfMessageBlocks;i++)
	{
		if(pxAsuraEntity->m_pxMessageBlocks[i].m_usMessageBlock == usBlockID)
		{
			char szTemp[100];
			Asura_PrintfToBfr(szTemp,"Trigger '%s'",sizeof(szTemp),true,pxAsuraEntity->m_pxMessageBlocks[i].m_xName.GetString());
			xOutputDesc = szTemp;
			return true;
		}
	}
	
	return Tools_Entity_Trigger_Object::GetEventDescForMessageBlock(usBlockID,xOutputDesc);
}


Tools_Object* Tools_Entity_CountedTrigger_Object::CreateObjectOfThisType() const
{
	return Create(true);	
}

void Tools_Entity_CountedTrigger_Object::Copy(const Tools_Object* pxObject)
{
	Tools_Entity_Activatable_Object::Copy(pxObject);

	const Tools_Entity_CountedTrigger_Object* pxFromObject = (const Tools_Entity_CountedTrigger_Object*) pxObject;

	Asura_ServerEntity_CountedTrigger* pxFromEntity = (Asura_ServerEntity_CountedTrigger*) pxFromObject->Entity;
	Asura_ServerEntity_CountedTrigger* pxToEntity = (Asura_ServerEntity_CountedTrigger*) Entity;

	//destroy the old messages
	for(u_int i=0;i<pxToEntity->m_uNumberOfMessageBlocks;i++)
	{
		Asura_StaticMessage_System::DeleteMessageBlock(pxToEntity->m_pxMessageBlocks[i].m_usMessageBlock);
	}
	
	delete [] pxToEntity->m_pxMessageBlocks;
	pxToEntity->m_pxMessageBlocks = 0;
	
	
	//copy between the entities
	pxToEntity->m_uNumberOfMessageBlocks = pxFromEntity->m_uNumberOfMessageBlocks;


	if(pxToEntity->m_uNumberOfMessageBlocks)
	{
		pxToEntity->m_pxMessageBlocks = new Asura_CountedTrigger_MessageBlock[pxToEntity->m_uNumberOfMessageBlocks];
		for(u_int i=0;i<pxToEntity->m_uNumberOfMessageBlocks;i++)
		{
			pxToEntity->m_pxMessageBlocks[i] = pxFromEntity->m_pxMessageBlocks[i];
			pxToEntity->m_pxMessageBlocks[i].m_usMessageBlock = Asura_StaticMessage_System::CopyMessageBlock(pxToEntity->m_pxMessageBlocks[i].m_usMessageBlock,pxFromObject->GetGuid(),GetGuid());
		}
	}

	//update tools copy of data
	AllocateArrays();
}


int Tools_Entity_CountedTrigger_Object::SelectMessageIndex(const char* pszMessage)
{
	Asura_ServerEntity_CountedTrigger* pxAsuraEntity = (Asura_ServerEntity_CountedTrigger*) Entity;

	if(!pxAsuraEntity->m_uNumberOfMessageBlocks)
	{
		ShowMessageOverlay("This trigger doesn't have any message blocks");
		return -1;
	}

	ListSelection_Overlay xOverlay(pszMessage);
	//add the names of all the messages
	for(u_int i=0;i<pxAsuraEntity->m_uNumberOfMessageBlocks;i++)
	{
		xOverlay.AddItem(pxAsuraEntity->m_pxMessageBlocks[i].m_xName.GetString(),i);
	}

	if(!xOverlay.DoModal()) return -1;

	return xOverlay.GetSelectionData();
	
}

Asura_Hash_ID Tools_Entity_CountedTrigger_Object::SelectMessageHash(const char* pszMessage)
{
	Asura_ServerEntity_CountedTrigger* pxAsuraEntity = (Asura_ServerEntity_CountedTrigger*) Entity;

	int iIndex = SelectMessageIndex(pszMessage);
	if(iIndex<0) return ASURA_HASH_ID_UNSET;

	return pxAsuraEntity->m_pxMessageBlocks[iIndex].m_uHashID;
}



const char* Tools_Entity_CountedTrigger_Object::GetMessageBlockNameFromHash(Asura_Hash_ID uNameHash) const
{
	const Asura_ServerEntity_CountedTrigger* pxAsuraEntity = (const Asura_ServerEntity_CountedTrigger*) Entity;

	for(u_int i=0;i<pxAsuraEntity->m_uNumberOfMessageBlocks;i++)
	{
		if(uNameHash == pxAsuraEntity->m_pxMessageBlocks[i].m_uHashID)
		{
			return pxAsuraEntity->m_pxMessageBlocks[i].m_xName.GetString();
		}
	}

	return "*Unknown*";
}



void Tools_Entity_CountedTrigger_Object::PrintInfo(float& x,float& y)
{
	Tools_Entity_Activatable_Object::PrintInfo(x,y);
	
	if(Tools_Entity_Simulation::IsActive())
	{
		Asura_ServerEntity_CountedTrigger* pxAsuraEntity = (Asura_ServerEntity_CountedTrigger*) Entity;
	
		for(unsigned i=0;i<pxAsuraEntity->m_uNumberOfMessageBlocks;i++)
		{
			Asura_Text_Output::Print(x,y++,0xffffff00,0,"%s : %s",pxAsuraEntity->m_pxMessageBlocks[i].m_xName.GetString(),pxAsuraEntity->m_pxMessageBlocks[i].m_bActive ? "Enabled" : "Disabled");
		}
		Asura_Text_Output::Print(x,y++,0xffffff00,0,"Counter : %d",pxAsuraEntity->m_iCounter);
	}

}


void Tools_Entity_CountedTrigger_Object::EditMessages(int iIndex)
{
	#ifdef ASURA_TOOLS_USE_NEW_MESSAGE_EDITOR
	Tools_MessageEditor::StartMessageEditingMode(ThisEntity->m_pxServerEntity->GetCustomEvent(iIndex));
	#else

	Asura_Assert(ThisEntity,"");
	Asura_ServerEntity_CountedTrigger* pxAsuraEntity = (Asura_ServerEntity_CountedTrigger*) ThisEntity->Entity;
	Asura_Assert(iIndex>=0 && iIndex<(int)pxAsuraEntity->m_uNumberOfMessageBlocks,"");

	Tools_Object_System::EditMessageBlock(pxAsuraEntity->m_pxMessageBlocks[iIndex].m_xName.GetString(),&pxAsuraEntity->m_pxMessageBlocks[iIndex].m_usMessageBlock);

	#endif
}


void Tools_Entity_CountedTrigger_Object::EditMessageBlockHash(int iMessage)
{
	EditMessageBlockHash((Asura_StaticMessage*)iMessage);
	Tools_Message_Editing::UpdateMessageOverlay();
}



void Tools_Entity_CountedTrigger_Object::EditMessageBlockHash(Asura_StaticMessage* pxMessage)
{
	Asura_Assert(pxMessage->m_pData,"");
	Asura_Message_CountedTrigger_MessageHash* pxData = (Asura_Message_CountedTrigger_MessageHash*) pxMessage->m_pData;

	//find the entity and tools object for this
	Asura_ServerEntity_CountedTrigger* pxEntity = Asura_ServerEntity_CountedTrigger::FindEntity(Tools_Message_Editing::GetFinalDestinationGuid(pxMessage));
	if(!pxEntity) return;
	if(!pxEntity->ToolsData) return;
	if(!pxEntity->ToolsData->ToolsObject) return;

	Tools_Entity_CountedTrigger_Object* pxObject = (Tools_Entity_CountedTrigger_Object*) pxEntity->ToolsData->ToolsObject;

	const char* szTitle = 0;
	switch(pxMessage->m_usMsgID)
	{
		case ASURA_MSG_COUNTEDTRIGGER_ENABLEMESSAGE :
			szTitle = "Select message block to enable.";
			break;
		case ASURA_MSG_COUNTEDTRIGGER_DISABLEMESSAGE :
			szTitle = "Select message block to disable.";
			break;
		case ASURA_MSG_COUNTEDTRIGGER_SETTRIGGERCOUNT :
			szTitle = "Select message block to alter.";
			break;
	}

	//ask for the message to enable
	Asura_Hash_ID uHash = pxObject->SelectMessageHash(szTitle);
	if(uHash)
	{
		pxData->m_uMessageHash = uHash;
	}
}



void Tools_Entity_CountedTrigger_Object::AllocateArrays()
{
	delete [] m_pbActive;
	delete [] m_pbRetriggerable;
	delete [] m_piTriggerCount;

	Asura_ServerEntity_CountedTrigger* pxAsuraEntity = (Asura_ServerEntity_CountedTrigger*) Entity;
	if(pxAsuraEntity->m_uNumberOfMessageBlocks)
	{
		m_pbActive = new bool[pxAsuraEntity->m_uNumberOfMessageBlocks];
		m_pbRetriggerable = new bool[pxAsuraEntity->m_uNumberOfMessageBlocks];
		m_piTriggerCount = new int[pxAsuraEntity->m_uNumberOfMessageBlocks];
	
		for(unsigned i=0;i<pxAsuraEntity->m_uNumberOfMessageBlocks;i++)
		{
			m_pbActive[i] = pxAsuraEntity->m_pxMessageBlocks[i].m_bActive ? true : false;
			m_pbRetriggerable[i] = pxAsuraEntity->m_pxMessageBlocks[i].m_bRetriggerable;
			m_piTriggerCount[i] = pxAsuraEntity->m_pxMessageBlocks[i].m_iTriggerCount;
		}
	}
	else
	{
		m_pbActive = 0;
		m_pbRetriggerable = 0;
		m_piTriggerCount = 0;
	}
}