//////////////////////////////////////////////////////////////////////////////
//
// Tools_Entity_CountedVolumeTrigger.cpp
//
// Copyright (c) 2005 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
//
// includes
//
//////////////////////////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"

// header
#include "Entity/Tools_Entity_CountedVolumeTrigger.h"

// asura
#include "Core/Asura_Maths.h"
#include "Debugging/Asura_Assert.h"
#include "Entity/Asura_Entity.h"
#include "Entity/Asura_Entity_Classification.h"
#include "Entity/Asura_StaticMessage_System.h"
#include "Entity/Triggers/Asura_ServerEntity_CountedVolumeTrigger.h"
#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Overlay.h"
#include "Mesh/Asura_Mesh_System.h"

// tools
#include "EntityTemplate/Tools_EntityTemplate_System.h"
#include "Load&Save/Tools_SaveAsura.h"
#include "Objects/Tools_VolumeEdit_Object.h"
#include "Objects/Tools_Object_System.h"

//////////////////////////////////////////////////////////////////////////////
//
// Tools_CountedVolumeTrigger_VolumeEdit_Controller
//
//////////////////////////////////////////////////////////////////////////////

Tools_CountedVolumeTrigger_VolumeEdit_Controller::Tools_CountedVolumeTrigger_VolumeEdit_Controller( Tools_Entity_CountedVolumeTrigger* pxToolsEntity )
: Tools_VolumeEdit_Controller( &pxToolsEntity->Changed )
{
    m_pxToolsEntity = pxToolsEntity;
    m_pxAsuraEntity = (Asura_ServerEntity_CountedVolumeTrigger*) pxToolsEntity->Entity;
}

void Tools_CountedVolumeTrigger_VolumeEdit_Controller::StartVolumeEditing()
{
    Tools_VolumeEdit_Controller::StartVolumeEditing();

    //create a volume if none exists yet
    if(!GetNumberOfVolumes()) CreateVolume();
}

void Tools_CountedVolumeTrigger_VolumeEdit_Controller::EndVolumeEditing()
{
    Tools_VolumeEdit_Controller::EndVolumeEditing();
    //select the owner
    Tools_Object_System::SelectObject(m_pxToolsEntity);
}

void Tools_CountedVolumeTrigger_VolumeEdit_Controller::CreateVolumeMain()
{
    //allocate a new array of volumes , and copy the old ones across
    Asura_OrientedVolume* pxVolumes = new Asura_OrientedVolume[m_pxAsuraEntity->m_iNumVolumes+1];

    for(int i=0;i<m_pxAsuraEntity->m_iNumVolumes;i++)
    {
        pxVolumes[i] = m_pxAsuraEntity->m_pxVolumes[i];
    }

    //initialise the new volume
    pxVolumes[m_pxAsuraEntity->m_iNumVolumes].m_xBox.Reset();
    pxVolumes[m_pxAsuraEntity->m_iNumVolumes].m_xBox.Set(Asura_Maths::ZeroVector3);
    pxVolumes[m_pxAsuraEntity->m_iNumVolumes].m_xOrient = Asura_Maths::IdentityQuat;

    //replace the old array
    delete [] m_pxAsuraEntity->m_pxVolumes;
    m_pxAsuraEntity->m_pxVolumes = pxVolumes;
    m_pxAsuraEntity->m_iNumVolumes++;

}

void Tools_CountedVolumeTrigger_VolumeEdit_Controller::DeleteVolumeMain(int iIndex)
{
    Asura_Assert(iIndex>=0 && iIndex<m_pxAsuraEntity->m_iNumVolumes,"");

    //shuffle down any volumes after the one being deleted
    for(int i=iIndex;i<m_pxAsuraEntity->m_iNumVolumes-1;i++)
    {
        m_pxAsuraEntity->m_pxVolumes[i] = m_pxAsuraEntity->m_pxVolumes[i+1];
    }
    m_pxAsuraEntity->m_iNumVolumes--;
}

//////////////////////////////////////////////////////////////////////////////
//
// Tools_Entity_CountedVolumeTrigger
//
//////////////////////////////////////////////////////////////////////////////

bool Tools_Entity_CountedVolumeTrigger::s_bMoveVolumeWithObject = true;

Tools_Entity_CountedVolumeTrigger::Tools_Entity_CountedVolumeTrigger( Asura_Entity* pxEntity )
: Tools_Entity_Trigger_Object( pxEntity )
{
    Type = TOT_Entity_AdvancedVolumeTrigger;
    Mesh = Asura_Mesh_System::GetMesh( "ToolsShape_V" );

    m_xPreviousPosition  = Position;
    m_pxVolumeController = NULL;
}

Tools_Entity_CountedVolumeTrigger::~Tools_Entity_CountedVolumeTrigger()
{
    delete m_pxVolumeController;
}

bool Tools_Entity_CountedVolumeTrigger::OwnsThisMessageBlock( unsigned short uMessageBlockID )
{
    if ( Tools_Entity_Trigger_Object::OwnsThisMessageBlock( uMessageBlockID ) )
    {
        return true;
    }

    Asura_ServerEntity_CountedVolumeTrigger* pxAsuraEntity = ( Asura_ServerEntity_CountedVolumeTrigger* )Entity;
	for ( int i = 0; i < Asura_ServerEntity_CountedVolumeTrigger::NUM_TRIGGER_TYPES; ++i )
	{
		if ( pxAsuraEntity->m_ausMultipleTriggerMessageBlocks[ i ] == uMessageBlockID )
        {
            return true;
        }
	}

	return false;
}

void Tools_Entity_CountedVolumeTrigger::Destroy()
{
	Asura_ServerEntity_CountedVolumeTrigger* pxAsuraEntity = static_cast<Asura_ServerEntity_CountedVolumeTrigger*>(Entity);
	if (pxAsuraEntity)
	{
		for ( int i = 0; i < Asura_ServerEntity_CountedVolumeTrigger::NUM_TRIGGER_TYPES; ++i )
		{
			Asura_StaticMessage_System::DeleteMessageBlock(pxAsuraEntity->m_ausMultipleTriggerMessageBlocks[i]);
			pxAsuraEntity->m_ausMultipleTriggerMessageBlocks[ i ] = ASURA_STATICMESSAGE_EMPTYBLOCK;
		}
	}

	Tools_Entity_Trigger_Object::Destroy();
}


void Tools_Entity_CountedVolumeTrigger::UpdatePositionInAsuraData( bool bUpdateLinkedObjects )
{
    // shift the bounding box by the amount that the object has moved
    Asura_ServerEntity_CountedVolumeTrigger* pxAsuraEntity = ( Asura_ServerEntity_CountedVolumeTrigger* )Entity;
    if ( s_bMoveVolumeWithObject )
    {
        for ( int i = 0; i < pxAsuraEntity->m_iNumVolumes; ++i )
        {
            pxAsuraEntity->m_pxVolumes[ i ].m_xBox.Displace( Position - m_xPreviousPosition );
        }
        if ( m_pxVolumeController )
        {
            m_pxVolumeController->MoveVolumeObjects( Position - m_xPreviousPosition );
        }
    }

    m_xPreviousPosition = Position;

    Tools_Entity_Trigger_Object::UpdatePositionInAsuraData( bUpdateLinkedObjects );
}

void Tools_Entity_CountedVolumeTrigger::AddOverlayControls( class Asura_Overlay* pxNewOverlay, class Asura_Vector_2& xOffset )
{
    Tools_Entity_Trigger_Object::AddOverlayControls( pxNewOverlay, xOffset );

    Asura_ServerEntity_CountedVolumeTrigger* pxAsuraEntity = ( Asura_ServerEntity_CountedVolumeTrigger* )Entity;

    Asura_Dialog* pxNewDialog;
    char acText[ 100 ];

    sprintf( acText, "On Increment Multiple Trigger (%d)",
        Asura_StaticMessage_System::GetNumberOfMessagesInBlock( pxAsuraEntity->m_ausMultipleTriggerMessageBlocks[ Asura_ServerEntity_CountedVolumeTrigger::INCREMENT_TRIGGER_MULTIPLE_INDEX ] ) );
    pxNewDialog = new Asura_Function_Dialog( acText, xOffset, EditMessages_OnTriggerIncrementMultiple );
    pxNewOverlay->AddDialog( pxNewDialog );
    xOffset.y += Asura_Dialog::DefaultSpacing();

    sprintf( acText, "On Decrement Multiple Trigger (%d)",
        Asura_StaticMessage_System::GetNumberOfMessagesInBlock( pxAsuraEntity->m_ausMultipleTriggerMessageBlocks[ Asura_ServerEntity_CountedVolumeTrigger::DECREMENT_TRIGGER_MULTIPLE_INDEX ] ) );
    pxNewDialog = new Asura_Function_Dialog( acText, xOffset, EditMessages_OnTriggerDecrementMultiple );
    pxNewOverlay->AddDialog( pxNewDialog );
    xOffset.y += Asura_Dialog::DefaultSpacing();

    sprintf( acText, "On Count Limit Trigger (%d)",
        Asura_StaticMessage_System::GetNumberOfMessagesInBlock( pxAsuraEntity->m_ausMultipleTriggerMessageBlocks[ Asura_ServerEntity_CountedVolumeTrigger::LIMIT_TRIGGER_INDEX ] ) );
    pxNewDialog = new Asura_Function_Dialog( acText, xOffset, EditMessages_OnTriggerCountLimit );
    pxNewOverlay->AddDialog( pxNewDialog );
    xOffset.y += Asura_Dialog::DefaultSpacing();

    pxNewDialog = new Asura_Function_Dialog( "Edit Volume", xOffset, EditVolume );
    pxNewDialog->SetHelpText( "Edits the volume for this trigger." );
    pxNewOverlay->AddDialog( pxNewDialog );
    xOffset.y += Asura_Dialog::DefaultSpacing();

    pxNewDialog = new Asura_Tickbox_Dialog( "Move Volume With Object", xOffset, &s_bMoveVolumeWithObject );
    pxNewDialog->SetHelpText( "The volume attached to this trigger should be moved when the trigger's tools object is moved.\n"
                              "This is a global setting for all trigger volumes." );
    pxNewOverlay->AddDialog( pxNewDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

    pxNewDialog = new Asura_FlagTickbox_Dialog( "Trigger Once", xOffset, &pxAsuraEntity->m_uFlags, ASURACOUNTEDVOLUMETRIGGERFLAG_TRIGGERONCE, &Changed );
	pxNewDialog->SetHelpText( "This trigger will deactivate itself once it becomes untriggered.\n"
                              "Therefore it will only be triggerable once, unless it is re-activated." );
	pxNewOverlay->AddDialog( pxNewDialog );
    xOffset.y += Asura_Dialog::LargeSpacing();

    pxNewDialog = new Asura_Numerical_Dialog( "Trigger Every Multiple Of This Arriving", xOffset,
                                              &pxAsuraEntity->m_uIncrementTriggerMultiple, 0, UINT_MAX, 1, "%u", &Changed );
    pxNewDialog->SetHelpText( "If this is set to a value other than zero then the volume will trigger every time the count "
                              "rises to a multiple of this value." );
    pxNewOverlay->AddDialog( pxNewDialog );
    xOffset.y += Asura_Dialog::DefaultSpacing();

    pxNewDialog = new Asura_Numerical_Dialog( "Trigger Every Multiple Of This Leaving", xOffset,
                                              &pxAsuraEntity->m_uDecrementTriggerMultiple, 0, UINT_MAX, 1, "%u", &Changed );
    pxNewDialog->SetHelpText( "If this is set to a value other than zero then the volume will trigger every time the count "
                              "falls to a multiple of this value." );
    pxNewOverlay->AddDialog( pxNewDialog );
    xOffset.y += Asura_Dialog::DefaultSpacing();

    pxNewDialog = new Asura_Numerical_Dialog( "Trigger At This Count", xOffset,
                                              &pxAsuraEntity->m_uTriggerCount, 0, UINT_MAX, 1, "%u", &Changed );
    pxNewDialog->SetHelpText( "The trigger will go off when the count reaches this number." );
    pxNewOverlay->AddDialog( pxNewDialog );
    xOffset.y += Asura_Dialog::LargeSpacing();
}

void Tools_Entity_CountedVolumeTrigger::Copy( const Tools_Object* pxObject )
{
    Tools_Entity_Trigger_Object::Copy( pxObject );

    const Tools_Entity_CountedVolumeTrigger* pxFromObject = ( const Tools_Entity_CountedVolumeTrigger* ) pxObject;

    Asura_ServerEntity_CountedVolumeTrigger* pxFromEntity = ( Asura_ServerEntity_CountedVolumeTrigger* ) pxFromObject->Entity;
    Asura_ServerEntity_CountedVolumeTrigger* pxToEntity = ( Asura_ServerEntity_CountedVolumeTrigger* ) Entity;

    // delete the volume controller if it exists, since we will be replacing the volumes
    delete m_pxVolumeController;
    m_pxVolumeController = 0;

    // copy everything between the entities
    pxToEntity->m_iListFlags = pxFromEntity->m_iListFlags;
    pxToEntity->m_iNumVolumes = pxFromEntity->m_iNumVolumes;
    for ( int i = 0; i < Asura_ServerEntity_CountedVolumeTrigger::NUM_TRIGGER_TYPES; ++i )
    {
        pxToEntity->m_ausMultipleTriggerMessageBlocks[ i ] = pxFromEntity->m_ausMultipleTriggerMessageBlocks[ i ];
    }

    pxToEntity->m_uIncrementTriggerMultiple = pxFromEntity->m_uIncrementTriggerMultiple;
    pxToEntity->m_uDecrementTriggerMultiple = pxFromEntity->m_uDecrementTriggerMultiple;
    pxToEntity->m_uTriggerCount             = pxFromEntity->m_uTriggerCount;

    delete [] pxToEntity->m_pxVolumes;
    if ( pxToEntity->m_iNumVolumes )
    {
        pxToEntity->m_pxVolumes = new Asura_OrientedVolume[ pxToEntity->m_iNumVolumes ];
        for ( int i = 0; i < pxToEntity->m_iNumVolumes; ++i )
        {
            pxToEntity->m_pxVolumes[ i ] = pxFromEntity->m_pxVolumes[ i ];
        }
    }
    else
    {
        pxToEntity->m_pxVolumes = NULL;
    }
}

Tools_Object* Tools_Entity_CountedVolumeTrigger::CreateObjectOfThisType() const
{
    Tools_Object* pxOldSelection = Tools_Object_System::SelectedObject;
    Project_Create();
    Tools_Object* pxNewSelection = Tools_Object_System::SelectedObject;

	return ( pxOldSelection != pxNewSelection ) ? pxNewSelection : NULL;
}

void Tools_Entity_CountedVolumeTrigger::EditVolume()
{
	Tools_Entity_CountedVolumeTrigger* pxToolsEntity = ( Tools_Entity_CountedVolumeTrigger* )ThisEntity;
	// create controller for editing volumes if it hasn't been done yet
	if ( !pxToolsEntity->m_pxVolumeController )
	{
		pxToolsEntity->m_pxVolumeController = new Tools_CountedVolumeTrigger_VolumeEdit_Controller( pxToolsEntity );
	}

	// and get it to create the volume objects
	pxToolsEntity->m_pxVolumeController->StartVolumeEditing();
}

void Tools_Entity_CountedVolumeTrigger::EditMessages_OnTriggerIncrementMultiple()
{
	Tools_Entity_CountedVolumeTrigger*       pxToolsEntity = ( Tools_Entity_CountedVolumeTrigger* )ThisEntity;
	Asura_ServerEntity_CountedVolumeTrigger* pxAsuraEntity = ( Asura_ServerEntity_CountedVolumeTrigger* )pxToolsEntity->Entity;

	Tools_Object_System::EditMessageBlock( "On Increment Multiple Trigger",
        &pxAsuraEntity->m_ausMultipleTriggerMessageBlocks[ Asura_ServerEntity_CountedVolumeTrigger::INCREMENT_TRIGGER_MULTIPLE_INDEX ] );
}

void Tools_Entity_CountedVolumeTrigger::EditMessages_OnTriggerDecrementMultiple()
{
	Tools_Entity_CountedVolumeTrigger*       pxToolsEntity = ( Tools_Entity_CountedVolumeTrigger* )ThisEntity;
	Asura_ServerEntity_CountedVolumeTrigger* pxAsuraEntity = ( Asura_ServerEntity_CountedVolumeTrigger* )pxToolsEntity->Entity;

	Tools_Object_System::EditMessageBlock( "On Decrement Multiple Trigger",
        &pxAsuraEntity->m_ausMultipleTriggerMessageBlocks[ Asura_ServerEntity_CountedVolumeTrigger::DECREMENT_TRIGGER_MULTIPLE_INDEX ] );
}

void Tools_Entity_CountedVolumeTrigger::EditMessages_OnTriggerCountLimit()
{
	Tools_Entity_CountedVolumeTrigger*       pxToolsEntity = ( Tools_Entity_CountedVolumeTrigger* )ThisEntity;
	Asura_ServerEntity_CountedVolumeTrigger* pxAsuraEntity = ( Asura_ServerEntity_CountedVolumeTrigger* )pxToolsEntity->Entity;

	Tools_Object_System::EditMessageBlock( "On Count Limit Trigger",
        &pxAsuraEntity->m_ausMultipleTriggerMessageBlocks[ Asura_ServerEntity_CountedVolumeTrigger::LIMIT_TRIGGER_INDEX ] );
}

//////////////////////////////////////////////////////////////////////////////
//
//
//
//////////////////////////////////////////////////////////////////////////////
