//////////////////////////////////////////////////////////////////////////////
//
// Tools_Entity_CountedVolumeTrigger.h
//
// Copyright (c) 2005 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef ___TOOLS_ENTITY_COUNTEDVOLUMETRIGGER_H___
#define ___TOOLS_ENTITY_COUNTEDVOLUMETRIGGER_H___ 1

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
//
// includes
//
//////////////////////////////////////////////////////////////////////////////

// asura
#include "Entity/Triggers/Asura_ServerEntity_CountedVolumeTrigger.h"

// tools
#include "Entity/Tools_Entity_Trigger_Object.h"
#include "Objects/Tools_VolumeEdit_Object.h"

//////////////////////////////////////////////////////////////////////////////
//
// forward reference
//
//////////////////////////////////////////////////////////////////////////////

class Tools_CountedVolumeTrigger_VolumeEdit_Controller;

//////////////////////////////////////////////////////////////////////////////
//
// Tools_Entity_CountedVolumeTrigger
//
//////////////////////////////////////////////////////////////////////////////

class Tools_Entity_CountedVolumeTrigger : public Tools_Entity_Trigger_Object
{
public:
    Tools_Entity_CountedVolumeTrigger( Asura_Entity* pxEntity );
    virtual ~Tools_Entity_CountedVolumeTrigger();

    virtual bool UsesUnTriggerMessage()    { return true; }
    virtual bool UsesTriggerMessage()      { return false; }
    virtual bool UsesActivationMessage()   { return true; }
    virtual bool UsesDeactivationMessage() { return true; }
    virtual bool UsesDestructionMessage()  { return true; }

    virtual bool OwnsThisMessageBlock( unsigned short uMessageBlockID );
	virtual void Destroy();

    virtual const char* GetTypeDescription() { return "Counted Volume Trigger"; }
    virtual void UpdatePositionInAsuraData( bool /*updateLinkedObjects*/ );
    virtual void AddOverlayControls( class Asura_Overlay* pxNewOverlay, class Asura_Vector_2& xOffset );

	virtual const char* GetMyNamePrefix() const { return Tools_Entity_CountedVolumeTrigger::GetNamePrefix(); }

    virtual void Copy( const Tools_Object* pxObject );
    virtual bool CanBeCopied() const { return Type == TOT_Entity_CountedVolumeTrigger; }
    virtual Tools_Object* CreateObjectOfThisType() const;

    static void Project_Create();
    static void EditVolume();
    static void EditMessages_OnTriggerIncrementMultiple();
    static void EditMessages_OnTriggerDecrementMultiple();
    static void EditMessages_OnTriggerCountLimit();

	static const char* GetNamePrefix() { return "CVT_"; }

protected:
    static bool s_bMoveVolumeWithObject;

private :
    Asura_Vector_3                                    m_xPreviousPosition;
    Tools_CountedVolumeTrigger_VolumeEdit_Controller* m_pxVolumeController;
};

//////////////////////////////////////////////////////////////////////////////
//
// Tools_CountedVolumeTrigger_VolumeEdit_Controller
//
//////////////////////////////////////////////////////////////////////////////

class Tools_CountedVolumeTrigger_VolumeEdit_Controller : public Tools_VolumeEdit_Controller
{
public :
    Tools_CountedVolumeTrigger_VolumeEdit_Controller( Tools_Entity_CountedVolumeTrigger* pxToolsEntity );

    virtual void StartVolumeEditing();
    virtual void EndVolumeEditing();

protected :
    virtual bool CanCreateVolume()    { return true; }
    virtual bool CanDeleteVolume()    { return GetNumberOfVolumes() > 1; }
    virtual int  GetNumberOfVolumes() { return m_pxAsuraEntity->m_iNumVolumes; }
    virtual Asura_Bounding_Box* GetVolumeBox( int iIndex )    { return &( m_pxAsuraEntity->m_pxVolumes[ iIndex ].m_xBox ); }
    virtual Asura_Quat*         GetVolumeOrient( int iIndex ) { return &( m_pxAsuraEntity->m_pxVolumes[ iIndex ].m_xOrient ); }

    //Gets a default position for volume objects that have yet to gain a proper volume
    virtual Asura_Vector_3 GetDefaultPosition() { return m_pxToolsEntity->Position; }
    //Gets the name of the owner , to use for part of the volume names
    virtual const char* GetName() { return m_pxToolsEntity->GetName(); }
    //Creates the new volume in the asura data
    virtual void CreateVolumeMain();
    //Deletes the volume from the asura data
    virtual void DeleteVolumeMain( int iIndex );

private :
    Tools_Entity_CountedVolumeTrigger*       m_pxToolsEntity;
    Asura_ServerEntity_CountedVolumeTrigger* m_pxAsuraEntity;
};

//////////////////////////////////////////////////////////////////////////////
//
//
//
//////////////////////////////////////////////////////////////////////////////

#endif
