//
// Tools_Entity_Data.h
//
// Created by Richard Rice - 01/05/2002
// 
// Copyright (C) 2002 Rebellion, All Rights Reserved.
//


#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef _TOOLS_ENTITY_DATA_H_
#define _TOOLS_ENTITY_DATA_H_


#include "Core/Asura_String.h"
#include "Core/Asura_Maths.h"
#include "Network/Asura_Guid.h"

struct Tools_Entity_Data
{
	Tools_Entity_Data()
	{
		Position = Asura_Maths::ZeroVector3;
		m_xOrientation = Asura_Maths::IdentityMatrix3x3;
		m_bEntityModified = false;
		ToolsObject = 0;
		m_xComment = "";
		m_xCreationData.MakeBlank();
		m_xLastModified.MakeBlank();
		m_xScale.x = 1.0f;
		m_xScale.y = 1.0f;
		m_xScale.z = 1.0f;
		m_xSecondaryEntityFilename = "";
		m_iSecondaryEntityID = 0;
		m_bSecondaryEntityGuidConstant = false;
		m_uPlatforms = 0xffffffff;
		m_uLightSets = 0xffffffff;
		m_uProjectFlags = 0;
		m_uParentTemplate = ASURA_GUID_UNREGISTERED;
	}


	bool ShouldExportToPlatform(int iPlatform) const
	{
		return (m_uPlatforms & (1<<iPlatform)) ? true : false;
	}
	void SetExportToPlatform(int iPlatform,bool bSet)
	{
		if(bSet)
			m_uPlatforms |= (1<<iPlatform);
		else
			m_uPlatforms &=~ (1<<iPlatform);
	}

	bool IsMemberOfLightSets(u_int uSets) const
	{
		return (m_uLightSets & uSets)==uSets;
	}
	void SetExportForLightSet(int iLightSet,bool bSet)
	{
		if(bSet)
			m_uLightSets |= (1<<iLightSet);
		else
			m_uLightSets &=~ (1<<iLightSet);
	}

	bool IsMemberOfTemplate() const {return m_uParentTemplate != ASURA_GUID_UNREGISTERED;}

	Asura_String Name;
	Asura_String m_xComment;
	Asura_String m_xCreationData; // Date/time of creation and name of person who created it
	Asura_String m_xLastModified; // Date/time of last modification and name of person who did it
	Asura_String m_xEntityBrowserPath;
	Asura_Vector_3 Position;
	Asura_Matrix_3x3 m_xOrientation;
	class Tools_Entity_Object* ToolsObject;
	Asura_Vector_3 m_xScale;//scaling factor to use for objects merged into environment
	u_int m_uPlatforms;//bitfield of platforms that the entity is for
	u_int m_uLightSets;//bitfield of light sets that the entity is for
	u_int m_uProjectFlags;//bitfield for project use

	//secondary entity file stuff
	Asura_String m_xSecondaryEntityFilename;
	int m_iSecondaryEntityID;
	bool m_bSecondaryEntityGuidConstant;//indicates that a secondary file entity has been saved out to main entity file , so guid won't be changing

	bool m_bEntityModified;	//OBSOLETE indicates whther the entity has been modified at all while in tools(for returning to game)
	Asura_Guid m_uParentTemplate;//If part of a tempalte instance then this will contain the guid of the parent instance entity
};


#endif //_TOOLS_ENTITY_DATA_H_
