
//////////////////////////////////////////////////////////////////////////////
//
// Tools_Entity_DynamicLight.h
//
// Copyright (c) 2007 Rebellion (Derby). All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef ___TOOLS_ENTITY_DYNAMICLIGHT_H___
#define ___TOOLS_ENTITY_DYNAMICLIGHT_H___ 1

#ifdef ASURA_TARGET_SM3

// I N C L U D E S ///////////////////////////////////////////////////////////

// Tools.
#include "Core/Asura_String.h"
#include "Objects/Tools_Object.h"
#include "Entity/Tools_Entity_Object.h"
#include "Objects/Tools_VolumeEdit_Object.h"
#include "Objects/Tools_Object_System.h"
#include "Interface/Asura_Dialog.h"
#include "Rendering/Asura_Text_Output.h"

// Project independent.
#include "Entity/DynamicLight/Asura_ServerEntity_DynamicLight.h"
#include "Entity/DynamicLight/Asura_ClientEntity_DynamicLight.h"

// C L A S S E S /////////////////////////////////////////////////////////////

struct Asura_StaticMessage;
class Tools_DynamicLight_VolumeEdit_Controller;
class Tools_DynamicLight_LimitingPlane_Object;

class Tools_Entity_DynamicLight : public Tools_Entity_Object
{
	friend class Tools_DynamicLight_VolumeEdit_Controller;

    public:

        // C L A S S   M E T H O D S /////////////////////////////////////////

        static void Create( void );
        static void Create( Asura_Entity* pxEntity );
        static Tools_Entity_DynamicLight* CreateMain( void );

        // M E T H O D S /////////////////////////////////////////////////////

        // Query methods.
        virtual const char*		GetTypeDescription( void );
		virtual bool			HandlesThisMessage(int iMessageID);

		virtual const char*		GetMyNamePrefix() const { return Tools_Entity_DynamicLight::GetNamePrefix(); }

        // Behaviour / appearance methods.
		virtual void			AddOverlayControls( Asura_Overlay* pxOverlay, Asura_Vector_2& xOffsetWithinOverlay );
		static void				AddMessageOverlayControls_Fade(Asura_StaticMessage* pxMessage, Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset);

        virtual void			OnDeselection( void );
        virtual void			OnSelection( void );
	    virtual void			Render( void );
		virtual void		    Update();
        virtual void			UpdateWhileSelected( void );

		virtual void			Copy(const Tools_Object* pxObject);
        virtual bool			CanBeCopied( void ) const;
		virtual Tools_Object*	CreateObjectOfThisType() const;

		void					ReloadTextures(void) const;
		
		static void				RefreshOverlay(void);

		static void				EditCustomLightVolume(void);
		static void				ResetCustomLightVolume(void);
		

		virtual bool GetParameter(Asura_Hash_ID uParamID,class Tools_Parameter* pxParam) const;
		virtual bool SetParameter(Asura_Hash_ID uParamID,const class Tools_Parameter* pxParam);

		virtual void SetEntityModified();

		//Runs the Update for each client entity
		static void UpdateEntities();

		const Asura_DynamicLight_Parameters& GetDynamicLightParam() const { return m_xParams; }

		static const char* GetNamePrefix() { return "DL_"; }

    protected:

        // C O N S T R U C T O R S   /   D E S T R U C T O R S ///////////////
        
        virtual ~Tools_Entity_DynamicLight( void );

    private:

		//get/set a bit in the flags field
		//doesn't set the modified flag or do anything else
		bool GetFlag(u_int uFlag) const;
		void SetFlag(u_int uFlag,bool bSet);
		bool GetShadowFlag(u_int uFlag) const;
		void SetShadowFlag(u_int uFlag,bool bSet);

		static void EditLimitingPlane(void);

		// Export/Import
		static void ImportParams(void);
		static void ExportParams(void); 

		// Attachmess
		static void CreateAttachDlg(void);
		static void UpdateAttachDlg();
		static void CreateSelectMasterDlg(void);
		static void SetAttachOffset(void);
		static void SetAttachMarker();
		static void SetMasterObject(void);
		static void SetLightShaft(void);
		static void GetAttachMarkerName(Asura_String& xName);
		static void UnattachLight(void);

		static bool CanCastDarkeningShadow();
		static bool CanCastShadow();

		enum DialogID
		{
			DialogID_SelectMarker
		};

        // T Y P E S /////////////////////////////////////////////////////////

        typedef Tools_Entity_Object PARENT;

        // C O N S T R U C T O R S   /   D E S T R U C T O R S ///////////////
        
        Tools_Entity_DynamicLight( Asura_ServerEntity_DynamicLight* pxEntity );

        // M E T H O D S /////////////////////////////////////////////////////


        // S T A T E /////////////////////////////////////////////////////////        
		Asura_DynamicLight_Parameters			m_xParams;        
		Asura_ServerEntity_DynamicLight		   *m_pxServerEntity;
        bool									m_bParamsModified;			
		
        // Display.
        static bool	m_bShowVolume;		
		static bool								s_bModified;
		static class Asura_Tabs_Dialog		   *s_pxTabs;
		static u_int							s_uTab;
		static int								s_iShowLightVolumesMode;

		// Needed for the attachment stuff - so reselect the current shape after
		// selecting the attach target
		static bool s_bResetCurrentEntityPtrOnDeselect;

		Tools_DynamicLight_VolumeEdit_Controller* m_pxVolumeController;
		Tools_DynamicLight_LimitingPlane_Object*  m_pxLimitingPlane;
		
public:
		static Tools_Entity_DynamicLight *s_pxSelected;

		
};





/*---------------------------------------------------*
 * Tools_DynamicLight_VolumeEdit_Controller			 *
 * For editing custom light volumes					 *
 *---------------------------------------------------*/

class Tools_DynamicLight_VolumeEdit_Controller : public Tools_VolumeEdit_Controller
{
public :
	Tools_DynamicLight_VolumeEdit_Controller(Tools_Entity_DynamicLight* pxToolsEntity) :Tools_VolumeEdit_Controller(&pxToolsEntity->Changed)
	{
		m_pxToolsEntity = pxToolsEntity;

		Asura_Bounding_Box *pxBox = GetVolumeBox(0);
		//if there is only one volume , and it has zero size , then give it a default position/size
		if(pxBox->GetVolume() == 0.0f)
		{
			pxBox->Reset();
			pxBox->Set(m_pxToolsEntity->Position);
			pxBox->Grow(pxToolsEntity->m_xParams.m_fPhotonRange);
		}		
	}


	virtual void StartVolumeEditing()
	{
		Tools_VolumeEdit_Controller::StartVolumeEditing();
		if(GetNumberOfVolumes() == 0)
		{	
			CreateVolume();
		}
	}

	virtual void EndVolumeEditing();
	
protected :
	// Don't allow >1 volume
	virtual bool CanCreateVolume() {return false;}
	virtual bool CanDeleteVolume() {return false;}

	//Gets the current number of volumes
	virtual int GetNumberOfVolumes() {return 1;}

	//Gets a pointer to the bounding box for a given volume
	virtual Asura_Bounding_Box* GetVolumeBox(int iIndex) {return &m_pxToolsEntity->m_xParams.m_xLightVolume;}

	//Gets a pointer to the orientation for a given volume. Returns 0 if there is none
	virtual Asura_Quat* GetVolumeOrient(int iIndex) {return &m_pxToolsEntity->m_xParams.m_xLightVolumeOrient;}

	//Gets a default position for volume objects that have yet to gain a proper volume
	virtual Asura_Vector_3 GetDefaultPosition() {return m_pxToolsEntity->Position;}

	//Gets the name of the owner , to use for part of the volume names
	virtual const char* GetName() {return m_pxToolsEntity->GetName();}

	//Creates the new volume in the asura data
	virtual void CreateVolumeMain() {};

	//Deletes the volume from the asura data
	virtual void DeleteVolumeMain(int iIndex) {};

private :
	Tools_Entity_DynamicLight*	m_pxToolsEntity;	

};

//---------------------------------------------------
// Asura_Dialog_FalloffGraph			 
// For displaying the falloff as a simple graph	     
//---------------------------------------------------
class Asura_Dialog_FalloffGraph : public Asura_Dialog
{
public:
	Asura_Dialog_FalloffGraph(const Asura_Vector_2& xOffset, float fWidth, float fHeight);

public:
	virtual void Render(const Asura_Vector_2& xOrigin);

protected:
	// Return true if the parameters were written to.
	virtual bool GetParameters(float& fInnerRatio, float& fPower) const = 0;

	// Draw the grid lines
	void RenderGrid(const Asura_Vector_2& xPosition) const;

	// Draw the curve
	void RenderFunction(const Asura_Vector_2& xPosition, float fInnerRatio, float fPower) const;
	
protected:
	Asura_Vector_2 m_xSize;
};

//---------------------------------------------------
// Asura_Dialog_DynamicLight_FalloffGraph			 
// For displaying the falloff as a simple graph	     
//---------------------------------------------------
class Asura_Dialog_DynamicLight_FalloffGraph : public Asura_Dialog_FalloffGraph
{
public:
	Asura_Dialog_DynamicLight_FalloffGraph( const Asura_Vector_2& xOffset, float fWidth, float fHeight)
		: Asura_Dialog_FalloffGraph(xOffset, fWidth, fHeight)
	{
	}

protected:
	// Return true if the parameters were written to.
	virtual bool GetParameters(float& fInnerRatio, float& fPower) const
	{
		const Tools_Entity_DynamicLight* pxSelected = Tools_Entity_DynamicLight::s_pxSelected;
		if (!pxSelected) return false;
		
		const Asura_DynamicLight_Parameters& xParam = pxSelected->GetDynamicLightParam();
		fInnerRatio = xParam.m_fFalloffInnerRatio;
		fPower = xParam.m_fFalloffPower;
		return true;
	}
};



//------------------------------------------------------------------------------------------------------
// Asura_DynamicLight_ImportOverlay					 
// For selecting which parts of a template to import 
//------------------------------------------------------------------------------------------------------

class Asura_DynamicLight_ImportOverlay : public Tools_Modal_Overlay
{
public :
	Asura_DynamicLight_ImportOverlay();
	bool DoModal();

	bool m_bLightType;
	bool m_bFalloff;
	bool m_bMovement;
	bool m_bColourAndFlicker;
	bool m_bShadows;
	bool m_bSpecialFX;

private:

	void InitialiseOverlay(class Asura_Overlay* pxOverlay);

	static void OkaySelected(int iData);
	static void CancelSelected(int iData);

	bool m_bReturnValue;
};




//------------------------------------------------------------------------------------------------------
// The plane editing object - more-or-less a copy of the PFX collision plane
// editing code from Tools_PFX_Vector_Object.cpp
//------------------------------------------------------------------------------------------------------

class Tools_DynamicLight_LimitingPlane_Object : public Tools_Object
{
public:
	Tools_DynamicLight_LimitingPlane_Object();
	~Tools_DynamicLight_LimitingPlane_Object();

	// Virtual Tools_Object methods	
	void		 UpdateWhileSelected();
	void		 UpdatePositionInAsuraData(bool);
	void		 OnDeselection();
	void		 OnSelection();
	void		 Render();
	const char*  GetTypeDescription() { return m_xTypeDescription.GetString();}
	const char*  GetName() { return m_xName.GetString();}
	virtual bool IsTemporaryEditObject() const {return true;}

	// Members
	bool		 m_bGameEntitiesNeedUpdating;
	bool		 m_bReadyToDelete;
	Asura_String m_xTypeDescription;
	Asura_String m_xName;
};



// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

#endif //ASURA_TARGET_SM3

#endif // #ifndef ___TOOLS_ENTITY_DYNAMICLIGHT_H___
