// Tools_Entity_DynamicMusicTrigger_Object.cpp
// Thomas Young - 14.08.08
// Copyright (C) 2008 Rebellion, All Rights Reserved.

#ifdef ASURA_SUPPORT_DYNAMIC_MUSIC_SYSTEM

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"													//Main asura header
#include "Tools_Entity_DynamicMusicTrigger_Object.h"					//My header

#include "Entity/Triggers/Asura_ServerEntity_DynamicMusicTrigger.h"		//Other asura headers
#include "Sound/Asura_Dynamic_Music_System_Triggers.h"
#include "Sound/Asura_Dynamic_Music_System.h"

#include "Entity/Tools_Entity_Data.h"									//Tools headers
#include "Entity/Tools_SecondaryEntityFile_System.h"
#include "Interface/Asura_Dialog.h"	
#include "Interface/Asura_Overlay.h"
#include "Interface/Asura_Channel_Graph_Dialog.h"
#include "Load&Save/Tools_SaveAsura.h"
#include "Mesh/Asura_Mesh_System.h"
#include "Sound/Tools_Overlay_DynamicMusic.h"

// C O N S T A N T S /////////////////////////////////////////////////////////

// G L O B A L   V A R I A B L E S ///////////////////////////////////////////

// S T A T I C   M E M B E R   V A R I A B L E S /////////////////////////////

Tools_Entity_DynamicMusicTrigger_Object* Tools_Entity_DynamicMusicTrigger_Object::s_pxThis = 0;

// F U N C T I O N   D E F I N I T I O N S ///////////////////////////////////

Tools_Entity_DynamicMusicTrigger_Object::Tools_Entity_DynamicMusicTrigger_Object (Asura_Entity* pxEntity) : Tools_Entity_Activatable_Object ( pxEntity )
{
	Type = TOT_Entity_DynamicMusicTrigger;
	Mesh = Asura_Mesh_System::GetMesh("ToolsShape_Note");
}

void Tools_Entity_DynamicMusicTrigger_Object::Create ()
{
	Tools_Object* pxObject = Create(false);
	Tools_Object_System::SelectObject(pxObject);
}

Tools_Object* Tools_Entity_DynamicMusicTrigger_Object::CreateObjectOfThisType() const
{
	return Create(true);
}

Tools_Entity_DynamicMusicTrigger_Object* Tools_Entity_DynamicMusicTrigger_Object::Create(bool bGenerateName)
{
	//get a name , and the position data
	Asura_Guid uGuid = Asura_Entity_System::GetNextToolsCreatedGuid();
	Tools_Entity_Data* pxToolsData = 0;
	if(bGenerateName)
	{
		pxToolsData = Tools_Entity_Object::CreateToolsDataWithDefaultName("Dynamic Music Trigger", "Dynamic Music Trigger",uGuid,GetNamePrefix());
	}
	else
	{
		pxToolsData = Tools_Entity_Object::CreateToolsData("Dynamic Music Trigger",0,GetNamePrefix());
	}
	if(!pxToolsData) return 0 ;

	//create the asura data
	Asura_ServerEntity_DynamicMusicTrigger* pxAsuraEntity = new Asura_ServerEntity_DynamicMusicTrigger();
	if( pxAsuraEntity )
	{
		pxAsuraEntity->ToolsData = pxToolsData;

		//Create a new trigger in the dynamic music system
		Asura_Dynamic_Music_Trigger_Data* pxData = Asura_Dynamic_Music_System::CreateNewTriggerData();
		if( pxData )
		{
			//
			// Give it some sensible defaults
			//

			// now using tools entity name hash ID
			pxData->m_uHash = pxToolsData->Name.GetHash();
			pxData->SetTriggerType( DYNAMIC_MUSIC_TRIGGER_PLAY );
			
			pxAsuraEntity->m_uTriggerHash = pxData->m_uHash;

			//create the tools Object
			Tools_Entity_DynamicMusicTrigger_Object* pxObject = new Tools_Entity_DynamicMusicTrigger_Object(pxAsuraEntity);

			Tools_SaveAsura::SetEntityModified();

			Tools_Overlay_DynamicMusic::TriggersHaveChanged();

			return pxObject;
		}
	}
	
	return 0;
}

void Tools_Entity_DynamicMusicTrigger_Object::Destroy ()
{
	//First we need to get the trigger data from the music system, looked up from the hash ID stored by the server entity
	Asura_ServerEntity_DynamicMusicTrigger* pxAsuraEntity = (Asura_ServerEntity_DynamicMusicTrigger*) Entity;
	Asura_Assert( pxAsuraEntity, "Failed to find corresponding Asura entity");

	//Don't clear out the data from the music system if updating from secondary files ,since the entity is quite likely to be
	//created again and have need of that data. This may mean that data for entities that have been deleted from the secondary
	//files may hang around , but I'm not sure there is a safe way to get rid of that at the moment.
	//RAR 13/05/2009
	if(!Tools_SecondaryEntityFileUpdater::IsUpdatingFromSecondaryFile())
	{
		if( pxAsuraEntity )
		{
			Asura_Dynamic_Music_System::DeleteTriggerData( pxAsuraEntity->m_uTriggerHash );
		}
	}

	PARENT::Destroy ();

	Tools_Overlay_DynamicMusic::TriggersHaveChanged();
}

void Tools_Entity_DynamicMusicTrigger_Object::Copy (const Tools_Object*)
{
	//TODO: Write
	Asura_Assert(false, "Haven't written this yet");
}

const char* Tools_Entity_DynamicMusicTrigger_Object::GetTypeDescription()
{
	return "Dynamic Music Trigger";
}

void Tools_Entity_DynamicMusicTrigger_Object::RecreateCurrentOverlay ()
{
	if (s_pxThis)
	{
		s_pxThis->RecreateOverlay();
	}
}

void Tools_Entity_DynamicMusicTrigger_Object::RecreateOverlay ()
{
	DeleteOverlay();
	CreateOverlay();
}

void Tools_Entity_DynamicMusicTrigger_Object::AddOverlayControls(class Asura_Overlay* pxOverlay, class Asura_Vector_2& xOffset)
{
	Tools_Entity_Activatable_Object::AddOverlayControls(pxOverlay, xOffset);

	//First we need to get the trigger data from the music system, looked up from the guid stored by the server entity
	Asura_ServerEntity_DynamicMusicTrigger* pxAsuraEntity = (Asura_ServerEntity_DynamicMusicTrigger*) Entity;
	Asura_Assert( pxAsuraEntity, "Failed to find corresponding Asura entity");

	Asura_Dialog* pxDialog = new Asura_Divider_Dialog( xOffset );
	pxOverlay->AddDialog(pxDialog);

	xOffset.y += Asura_Dialog::DefaultSpacing();

	Tools_Overlay_DynamicMusic::AddTriggerOverlayControls(pxOverlay, xOffset, pxAsuraEntity->m_uTriggerHash);
}


void Tools_Entity_DynamicMusicTrigger_Object::UpdateWhileSelected()
{
	bool bRecreateOverlay = Tools_Overlay_DynamicMusic::UpdateTriggerEditOverlay();

	if (bRecreateOverlay)
	{
		RecreateOverlay();
	}

	Tools_Entity_Activatable_Object::UpdateWhileSelected();
}



void Tools_Entity_DynamicMusicTrigger_Object::OnSelection	()
{
	// We can have either the Entity Trigger overlay open (this one) OR the Code Trigger overlay - not both.
	// This is because they share the same static vars now (inside Tools_Overlay_DynamicMusic).
	Tools_Overlay_DynamicMusic::CloseTriggerEditOverlay();

	//Remember which instance is selected so that button presses have an instance to use
	s_pxThis = this;
	PARENT::OnSelection();
}


void Tools_Entity_DynamicMusicTrigger_Object::OnDeselection	()
{
	s_pxThis = 0;
	PARENT::OnDeselection();
}


void Tools_Entity_DynamicMusicTrigger_Object::HandleTrackRename (Asura_Hash_ID uOldHash, Asura_Hash_ID uNewHash)
{

}

#endif //#ifdef ASURA_SUPPORT_DYNAMIC_MUSIC_SYSTEM

//eof
