//////////////////////////////////////////////////////////////////////////////////////////
//
//  FILE        :  Tools_Entity_Explosion.cpp
//
//  CREATED     :  Adam A. Brown  (1.4.2003)
//
//  Copyright (C) 2003 Rebellion, All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////////////////


#include "Core/Asura.h"

// This file's header.
#include "Entity/Tools_Entity_Explosion.h"

#include "Entity/Tools_Entity_Data.h"
#include "Objects/Tools_Object_System.h"

#include "Entity/Asura_Entity_Classification.h"
#include "Entity/ParticleGenerator/Asura_ServerEntity_Explosion.h"
#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Overlay.h"
#include "Mesh/Asura_Mesh_System.h"
#include "SpecialFX/Asura_Explosion.h"
#include "SpecialFX/Asura_Particle.h"


Tools_Entity_Explosion* Tools_Entity_Explosion::s_pxThis = NULL;
char** Tools_Entity_Explosion::s_pszNameList = NULL;

////////////////////////////////////////////////////////////////////////////////
//
// Constructor / Destructor
//
////////////////////////////////////////////////////////////////////////////////
Tools_Entity_Explosion::Tools_Entity_Explosion(Asura_Entity* pxEntity)
: Tools_Entity_Activatable_Object(pxEntity)
, m_eType(ASURA_EXPLOSION_NONE)
, m_iListSelection(0)
, m_iListOffset(0)
, m_bActive(false)
{
	Type = TOT_Entity_ExtendedParticleGenerator;
	m_uExplosionGuid = ASURA_PARTICLE_EFFECT_INVALID_GUID;

	//Get data from server.
	if(pxEntity != NULL)
	{
		Asura_ServerEntity_Explosion* pxServerEntity = static_cast<Asura_ServerEntity_Explosion*>(pxEntity);

		Position = pxServerEntity->GetPosition();
		Orientation = pxServerEntity->GetOrientation();
		m_eType = pxServerEntity->GetType();

		//Set the position in the list according to the generator template being used.
		int iRef = Asura_Explosion_Template_System::GetReferenceNumber(m_eType);

		m_iListSelection = 7;
		m_iListOffset = iRef-7;

		if(m_iListOffset < 0)
		{
			m_iListSelection += m_iListOffset;
			m_iListOffset = 0;
		}

		Asura_Entity_Client* pxClientEntity = Asura_Entity_Client::FindEntity(pxEntity->GetGuid());
		if(!pxClientEntity)
		{
			//Update the server entity once , so as to create the client entity. RAR 03/06/04
			pxServerEntity->Update();
		}
	}
	else
	{
		Orientation.Identity();
	}

	s_pxThis = this;

	Asura_String xMeshName("ToolsShape_CutsceneTrackPoint");
	Mesh = Asura_Mesh_System::GetMesh(xMeshName);
}


Tools_Entity_Explosion::~Tools_Entity_Explosion()
{
	FreeNames();
	ReleaseExplosion();
}

void Tools_Entity_Explosion::ReleaseExplosion()
{
	if (m_uExplosionGuid != ASURA_PARTICLE_EFFECT_INVALID_GUID)
	{
		Asura_Particle_Effect *pxEffect = Asura_Particle_System::FindEffect(m_uExplosionGuid);
		if (pxEffect)
		{
			pxEffect->SetAsDestroyed();
			m_uExplosionGuid = ASURA_PARTICLE_EFFECT_INVALID_GUID;
		}
	}
}
////////////////////////////////////////////////////////////////////////////////
//
// UpdateWhileSelected
//
////////////////////////////////////////////////////////////////////////////////
void Tools_Entity_Explosion::UpdateWhileSelected()
{
	m_eType = Asura_Explosion_Template_System::GetTemplatePointer(m_iListSelection+m_iListOffset)->m_eType;

	Asura_ServerEntity_Explosion* pxServerEntity = static_cast<Asura_ServerEntity_Explosion*>(Entity);

	if(!(pxServerEntity->GetPosition() == Position) || pxServerEntity->GetType() != m_eType)
	{
		Changed = true;
	}

	pxServerEntity->SetType(m_eType);
	pxServerEntity->SetPosition(Position);
	pxServerEntity->SetOrientation(Orientation);
}



////////////////////////////////////////////////////////////////////////////////
//
// On selection etc...
//
////////////////////////////////////////////////////////////////////////////////
void Tools_Entity_Explosion::OnSelection()
{
	s_pxThis = this;

	Tools_Entity_Activatable_Object::OnSelection();
}


void Tools_Entity_Explosion::OnDeselection()
{
	Tools_Entity_Activatable_Object::OnDeselection();
}



////////////////////////////////////////////////////////////////////////////////
//
// Add overlay controls
//
////////////////////////////////////////////////////////////////////////////////
void Tools_Entity_Explosion::AddOverlayControls(class Asura_Overlay* pxNewOverlay,class Asura_Vector_2& xOffset)
{
	//Tools_Entity_Activatable_Object::AddOverlayControls(pxNewOverlay, xOffset);

	MakeNames();

	Asura_Dialog *pxNewDialog = new Asura_List_Dialog(xOffset, s_pszNameList, &s_pxThis->m_iListOffset, &s_pxThis->m_iListSelection, 9, ASURA_NUM_EXPLOSIONS_IN_ENUM, NULL);
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.y += 12.f;

	pxNewDialog = new Asura_Function_Dialog("Start Preview", xOffset, StartPreview);
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.y += 1.5f;

	pxNewDialog = new Asura_Function_Dialog("Stop Preview", xOffset, StopPreview);
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.y += 1.5f;
}



////////////////////////////////////////////////////////////////////////////////
//
// static functions
//
////////////////////////////////////////////////////////////////////////////////
void Tools_Entity_Explosion::Create()
{
	const char* szTypeName = "Particle Explosion";
	const char* szShortName = "Explosion";

	// Create the server object and set its data.
	Asura_ServerEntity_Explosion* pxServerObject = Asura_ServerEntity_Explosion::CreateFromTools();
	if(!pxServerObject) return;

	// Set up tools data and auto-create a unique name (prompt for new name if there is a clash).
	Tools_Entity_Data* pxToolsData = CreateToolsDataWithDefaultName(szTypeName, szShortName, pxServerObject->GetGuid(), GetNamePrefix());
	if (!pxToolsData) return;
	
	if (pxServerObject)
	{
		pxServerObject->ToolsData = pxToolsData;
		pxServerObject->SetPosition(pxToolsData->Position);
	}

	// Create the game specific tools object and select it.
	Tools_Object* pxToolsObject = static_cast<Tools_Object*>(new Tools_Entity_Explosion(pxServerObject));
	Tools_Object_System::SelectObject(pxToolsObject);
}



Tools_Object* Tools_Entity_Explosion::Create(Asura_Entity* pxEntity)
{
	Asura_Assert(pxEntity,"");
	if (!pxEntity) return NULL;
	
	if( pxEntity->GetClassification() != AsuraEntityClass_Explosion )
	{
		Asura_Assert(false, "ERROR - Not a particle explosion" );		
		return NULL;
	}

	Asura_ServerEntity_Explosion* pxGen = static_cast<Asura_ServerEntity_Explosion*>( pxEntity );

	if(pxGen == NULL)
	{
		Asura_Assert(false, "NULL server object");
		return NULL;
	}

	return new Tools_Entity_Explosion(pxGen);
}



void Tools_Entity_Explosion::MakeNames()
{
	if(s_pszNameList == NULL)
	{
		s_pszNameList = new char * [ASURA_NUM_EXPLOSIONS_IN_ENUM];

		for(int i = 0; i < ASURA_NUM_EXPLOSIONS_IN_ENUM; i++)
		{
			const char* szName = Asura_Explosion_Template_System::GetName(i);
			int j;

			s_pszNameList[i] = new char [40];

			for(j = 0; j < 39; j++)
			{
				s_pszNameList[i][j] = szName[j];
				if(szName[j] == 0) break;
			}

			if(j == 39)
			{
				s_pszNameList[i][j] = 0;
			}
		}
	}
}


void Tools_Entity_Explosion::FreeNames()
{
	if(s_pszNameList != NULL)
	{
		for(int i = 0; i < ASURA_NUM_EXPLOSIONS_IN_ENUM; i++)
		{
			if(s_pszNameList[i] != NULL)
			{
				delete [] s_pszNameList[i];
				s_pszNameList[i] = NULL;
			}
		}

		delete [] s_pszNameList;
		s_pszNameList = NULL;		
	}
}


////////////////////////////////////////////////////////////////////////////////
//
// Preview functions
//
////////////////////////////////////////////////////////////////////////////////
void Tools_Entity_Explosion::StartPreview()
{
	if (s_pxThis)
	{
		Asura_Vector_3 xOriVec;

		xOriVec.x = s_pxThis->Orientation.e13;
		xOriVec.y = s_pxThis->Orientation.e23;
		xOriVec.z = s_pxThis->Orientation.e33;

		s_pxThis->ReleaseExplosion();
		Asura_Particle_Effect *pxExplosion = Asura_Explosion::Create(Asura_Explosion_Template_System::GetTemplatePointer(s_pxThis->m_eType), s_pxThis->Position, xOriVec);
		s_pxThis->m_uExplosionGuid = pxExplosion->GetEffectGuid();
	}
}

void Tools_Entity_Explosion::StopPreview()
{
	if (s_pxThis)
	{
		s_pxThis->ReleaseExplosion();
	}
}




Tools_Object* Tools_Entity_Explosion::CreateObjectOfThisType() const
{
	Tools_Object* pxOldSelection = Tools_Object_System::SelectedObject;
	Create();
	Tools_Object* pxNewSelection = Tools_Object_System::SelectedObject;

	return (pxOldSelection!=pxNewSelection) ? pxNewSelection : 0;
}


void Tools_Entity_Explosion::Copy(const Tools_Object* pxObject)
{
	ReleaseExplosion();
	
	Tools_Entity_Activatable_Object::Copy(pxObject);

	const Tools_Entity_Explosion* pxFromObject = (const Tools_Entity_Explosion*) pxObject;

	Asura_ServerEntity_Explosion* pxFromEntity = (Asura_ServerEntity_Explosion*) pxFromObject->Entity;
	Asura_ServerEntity_Explosion* pxToEntity = (Asura_ServerEntity_Explosion*) Entity;
	
	//copy everything between the entities
	pxToEntity->SetType (pxFromEntity->GetType());
	
	//and between the tools objects
	m_eType = pxFromObject->m_eType;
	m_iListOffset = pxFromObject->m_iListOffset;
	m_iListSelection = pxFromObject->m_iListSelection;
	m_bActive = pxFromObject->m_bActive;
}
