
////////////////////////////////////////////////////////////////////////////////
//
// Tools_Entity_FMVController.h
// Author    : Jason Turner
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//
////////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef _TOOLS_ENTITY_FMVCONTROLLER_H_
#define _TOOLS_ENTITY_FMVCONTROLLER_H_

#include "Entity/Tools_Entity_Activatable_Object.h"

////////////////////////////////////////////////////////////////////////////////
//
////////////////////////////////////////////////////////////////////////////////

class Tools_Entity_FMVController;

class Tools_ChooseObject_FMVSpeaker : public Tools_ChooseObject
{
public:
	Tools_ChooseObject_FMVSpeaker( Tools_Entity_FMVController* pxSourceObj, bool bAdd);

	virtual bool SupportMultipleSelection() {return true;}	
	virtual bool CanBeSelected( Tools_Object* pxObject );

	virtual void ObjectSelected( Tools_Object* pxObject );
	virtual void OnFinish() { delete this; }

private:
	Tools_ChooseObject_FMVSpeaker();

	// function to call per selected Guid
	Tools_Entity_FMVController*	m_pxSourceObj;
	bool						m_bAddRemoveFlag;	// 0=remove, 1=add
};

////////////////////////////////////////////////////////////////////////////////
//
////////////////////////////////////////////////////////////////////////////////

class Tools_Entity_FMVController : public Tools_Entity_Activatable_Object
{
	typedef Tools_Entity_Activatable_Object PARENT;

	public:
		Tools_Entity_FMVController(Asura_Entity* pxEntity);
		static void Create();
		static Tools_Entity_FMVController* Create(bool bGenerateName);

		virtual bool HandlesThisEvent(int iEventID) const;
		virtual bool HandlesThisMessage(int iMessageID);

		//virtual function overides
		virtual const char* GetTypeDescription(){return "FMV Controller";}
		virtual void AddOverlayControls(class Asura_Overlay* pxOverlay,class Asura_Vector_2& xOffset);

		static const char* GetNamePrefix() { return "FMVC_"; }
		virtual const char* GetMyNamePrefix() const { return Tools_Entity_FMVController::GetNamePrefix(); }

		virtual void Copy(const Tools_Object*);
		virtual bool CanBeCopied() const {return Type == TOT_Entity_FMVController;}
		virtual Tools_Object* CreateObjectOfThisType() const;
		virtual void Update(void);
		virtual void Render(void);

	protected:

		static void SelectFMV(int iThis){((Tools_Entity_FMVController*)iThis)->SelectFMV();}
		void SelectFMV();
		static bool GetLoopChecked(void *pData);
		static void SetLoopChecked(bool bValue, void *pData);

		static void OnClickAddSpeakerLink (int iIndex);
		static void OnClickRemoveSpeakerLink (int iIndex);

	private:
		friend class Tools_Entity_FMVSpeaker;
		friend class Tools_ChooseObject_FMVSpeaker;

		void AddLinkedSpeaker(Asura_Guid uSpeakerGuid);
		void RemoveLinkedSpeaker(Asura_Guid uSpeakerGuid);

		bool		m_bNeedsUpdate;
};

#endif // _TOOLS_ENTITY_FMVCONTROLLER_H_
