//
// Tools_Entity_FMVTrigger_Object.cpp
//
// Created by Richard Rice - 12/06/2003
// 
// Copyright (C) 2003 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"
#include "Entity/Tools_Entity_FMVTrigger_Object.h"

#include "Entity/Triggers/Asura_ServerEntity_FMVTrigger.h"
#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Overlay.h"
#include "Mesh/Asura_Mesh_System.h"

#include "Load&Save/LevelExport/Tools_Level_Export.h"
#include "Load&Save/Tools_SaveAsura.h"
#include "Misc/Tools_Modal_Overlay.h"
#include "Objects/Tools_Object_System.h"

#include "PC/FileSystem/Tools_FileSystem.h"


// class implementation
Tools_Entity_FMVTrigger_Object::Tools_Entity_FMVTrigger_Object(Asura_Entity* pxEntity)
:Tools_Entity_Trigger_Object(pxEntity)
{
	Type = TOT_Entity_FMVTrigger;
	Mesh = Asura_Mesh_System::GetMesh("ToolsShape_Cutscene");
}


void Tools_Entity_FMVTrigger_Object::AddOverlayControls(class Asura_Overlay* pxOverlay,class Asura_Vector_2& xOffset)
{
	Tools_Entity_Trigger_Object::AddOverlayControls(pxOverlay,xOffset);
	Asura_Dialog* pxDialog;

	Asura_ServerEntity_FMVTrigger* pxAsuraEntity = (Asura_ServerEntity_FMVTrigger*) Entity;

	//get text for fmv selection control
	char szText[100];
	if(pxAsuraEntity->m_xFMVName.IsBlank())
	{
		strcpy(szText,"FMV : <Not Set>");
	}
	else
	{
		Asura_PrintfToBfr(szText,"FMV : %s",100,true,pxAsuraEntity->m_xFMVName.GetString());
	}
	

	pxDialog = new Asura_Function_Dialog(szText,xOffset,SelectFMV,(int)this);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += 1.5f;

	if (Tools_Level_Export::IsPlatformSupported(Tools_Level_Export::Export_PC))
	{
		pxDialog = new Asura_FlagTickbox_Dialog("PC", xOffset, &pxAsuraEntity->m_uFlags, ASURA_FMVTRIGGER_FLAG_PC, &Changed);
		pxOverlay->AddDialog(pxDialog);
		xOffset.y += 1.5f;
	}

	if (Tools_Level_Export::IsPlatformSupported(Tools_Level_Export::Export_PS2))
	{
		pxDialog = new Asura_FlagTickbox_Dialog("PS2", xOffset, &pxAsuraEntity->m_uFlags, ASURA_FMVTRIGGER_FLAG_PS2, &Changed);
		pxOverlay->AddDialog(pxDialog);
		xOffset.y += 1.5f;
	}

	if (Tools_Level_Export::IsPlatformSupported(Tools_Level_Export::Export_PSP))
	{
		pxDialog = new Asura_FlagTickbox_Dialog("PSP", xOffset, &pxAsuraEntity->m_uFlags, ASURA_FMVTRIGGER_FLAG_PSP, &Changed);
		pxOverlay->AddDialog(pxDialog);
		xOffset.y += 1.5f;
	}

	if (Tools_Level_Export::IsPlatformSupported(Tools_Level_Export::Export_PS3))
	{
		pxDialog = new Asura_FlagTickbox_Dialog("PS3", xOffset, &pxAsuraEntity->m_uFlags, ASURA_FMVTRIGGER_FLAG_PS3, &Changed);
		pxOverlay->AddDialog(pxDialog);
		xOffset.y += 1.5f;
	}

	if (Tools_Level_Export::IsPlatformSupported(Tools_Level_Export::Export_Xbox360))
	{
		pxDialog = new Asura_FlagTickbox_Dialog("XBox360", xOffset, &pxAsuraEntity->m_uFlags, ASURA_FMVTRIGGER_FLAG_XBOX360, &Changed);
		pxOverlay->AddDialog(pxDialog);
		xOffset.y += 1.5f;
	}

	if (Tools_Level_Export::IsPlatformSupported(Tools_Level_Export::Export_Wii))
	{
		pxDialog = new Asura_FlagTickbox_Dialog("Wii", xOffset, &pxAsuraEntity->m_uFlags, ASURA_FMVTRIGGER_FLAG_WII, &Changed);
		pxOverlay->AddDialog(pxDialog);
		xOffset.y += 1.5f;
	}

	// Include a new dialog for if the FMV is to be played at the start of the level
	{
		pxDialog = new Asura_FlagTickbox_Dialog("Play at level start", xOffset, &pxAsuraEntity->m_uFlags, ASURA_FMVTRIGGER_FLAG_PLAY_AT_START, &Changed);
		pxDialog->SetHelpText("If checked this FMV will play when the level starts up");
		pxOverlay->AddDialog(pxDialog);
		xOffset.y += 1.5f;
	}
}

void Tools_Entity_FMVTrigger_Object::Create()
{
	Tools_Object* pxObject = Create(false);
	Tools_Object_System::SelectObject(pxObject);
}

Tools_Entity_FMVTrigger_Object* Tools_Entity_FMVTrigger_Object::Create(bool bGenerateName)
{
	//get a name , and the position data
	Asura_Guid uGuid = Asura_Entity_System::GetNextToolsCreatedGuid();
	Tools_Entity_Data* pxToolsData = 0;
	if(bGenerateName)
	{
		pxToolsData = Tools_Entity_Object::CreateToolsDataWithDefaultName("FMV Trigger","FMV Trigger",uGuid,GetNamePrefix());
	}
	else
	{
		pxToolsData = Tools_Entity_Object::CreateToolsData("FMV Trigger",0,GetNamePrefix());
	}
	if(!pxToolsData) return 0 ;

	//create the asura data
	Asura_ServerEntity_FMVTrigger* pxAsuraEntity = new Asura_ServerEntity_FMVTrigger();
	pxAsuraEntity->ToolsData = pxToolsData;

	//create the tools pxObject
	Tools_Entity_FMVTrigger_Object* pxObject = new Tools_Entity_FMVTrigger_Object(pxAsuraEntity);
	Tools_SaveAsura::SetEntityModified();

	return pxObject;
}

void Tools_Entity_FMVTrigger_Object::SelectFMV()
{
	Asura_ServerEntity_FMVTrigger* pxAsuraEntity = (Asura_ServerEntity_FMVTrigger*) Entity;

	Tools_FileSystem_DialogInfo	xFileSelector;

	xFileSelector.DefaultExtension		= "*.bik";
	xFileSelector.DefaultFilename		= "";
	xFileSelector.DefaultPath			= NULL;
	xFileSelector.FileFilter			= "Movie files (*.bik; *.wmv; *.avi)\0*.bik; *.wmv; *.avi;\0";
	xFileSelector.WindowTitle			= "Choose source FMV...";

	xFileSelector.AllowMultipleSelect	= false;

	const Asura_String	xSelectedFilename(Tools_FileSystem::GetLoadFilename(xFileSelector));

	if(!xSelectedFilename.IsBlank())
	{
		Asura_String	xWorkingDirectory;
		
		xWorkingDirectory	 = Asura_FileAccess::PlatformGetAsuraFilesDir();
		xWorkingDirectory	+= "\\FMV\\";

		if(xSelectedFilename.StartsWith(xWorkingDirectory.GetString()))
		{
			const u_int		uWorkingDirectoryLength	= xWorkingDirectory.GetLength();
			const u_int		uSelectedFilenameLength	= xSelectedFilename.GetLength();
			const u_int		uRelativeFilenameLength	= uSelectedFilenameLength - uWorkingDirectoryLength;

			Asura_String	xFilename(xSelectedFilename.SubString(uWorkingDirectoryLength, uRelativeFilenameLength));

			xFilename.StripExtension();
			pxAsuraEntity->m_xFMVName	= xFilename;
			SetEntityModified();	

			//update the FMV trigger's overlay
			DeleteOverlay();
			CreateOverlay();
		}
		else
		{
			Asura_String	xMessage;

			xMessage	 = "Chosen file is not under \"";
			xMessage	+= xWorkingDirectory;
			xMessage	+= "\").";

			ShowMessageOverlay(xMessage.GetString());
		}
	}
}

Tools_Object* Tools_Entity_FMVTrigger_Object::CreateObjectOfThisType() const
{
	return Create(true);	
}

void Tools_Entity_FMVTrigger_Object::Copy(const Tools_Object* pxObject)
{
	Tools_Entity_Trigger_Object::Copy(pxObject);

	const Tools_Entity_FMVTrigger_Object* pxFromObject = (const Tools_Entity_FMVTrigger_Object*) pxObject;

	Asura_ServerEntity_FMVTrigger* pxFromEntity = (Asura_ServerEntity_FMVTrigger*) pxFromObject->Entity;
	Asura_ServerEntity_FMVTrigger* pxToEntity = (Asura_ServerEntity_FMVTrigger*) Entity;

	pxToEntity->m_xFMVName = pxFromEntity->m_xFMVName;
	pxToEntity->m_uFlags = pxFromEntity->m_uFlags;
}

