
///////////////////////////////////////////////////////////////////////////////
//
// File   : Tools_Entity_ForceField_Conveyor.cpp
// Date   : 28-Apr-2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////



////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"


// This file's header...
#include "Entity/Tools_Entity_ForceField_Conveyor.h"

// Asura independent header-files...
#include "Camera/Asura_Camera.h"
#include "Entity/Asura_StaticMessage_System.h"
#include "Entity/Intangible/Asura_ServerEntity_ForceField_Conveyor.h"
#include "Entity/PhysicalObject/Asura_ServerEntity_PhysicalObject.h"
#include "Entity/Spawners/Asura_ServerEntity_TeleportDestination.h"
#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Overlay.h"
#include "Network/Asura_Message.h"
#include "Rendering/Asura_Colour.h"
#include "Rendering/Asura_Render.h"
#include "Rendering/Asura_Text_Output.h"
#include "Teleport/Asura_Teleport_Messages.h"

// Tools header-files...
#include "Interface/Tools_ColourPicker.h"
#include "Load&Save/Tools_SaveAsura.h"
#include "Misc/Tools_Timed_Messages.h"
#include "Objects/Tools_Object_System.h"


////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////

Tools_Entity_ForceField_Conveyor*	Tools_Entity_ForceField_Conveyor::s_pxSelectedForceField		= NULL;

bool								Tools_Entity_ForceField_Conveyor::s_bFieldModified			= false;


////////////////////////////////////////////////////////////////////////////////
// Private functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Protected functions...
////////////////////////////////////////////////////////////////////////////////



////////////////////////////////////////////////////////////////////////////////
// Public functions...
////////////////////////////////////////////////////////////////////////////////



Tools_Entity_ForceField_Conveyor::Tools_Entity_ForceField_Conveyor(Asura_Entity* pxEntity)
: PARENT(pxEntity)
{
	const Asura_ServerEntity_ForceField_Conveyor*	pxAsuraEntity	= static_cast<Asura_ServerEntity_ForceField_Conveyor*>(Entity);
	Asura_RGBA								xRGBA;

	// Set the type...
	Type	= TOT_Entity_ForceField_Conveyor;

	SetMesh("ToolsShape_CutsceneTrackPoint");
	
	m_pxVolumeController = NULL;
	m_xPreviousPosition = Position;

	

	Asura_Vector_3 xForceDirection = pxAsuraEntity->GetForce();
	m_fForceMagnitude = xForceDirection.Magnitude();

	m_xForceEuler = pxAsuraEntity->GetForceEuler();
}

Tools_Entity_ForceField_Conveyor::~Tools_Entity_ForceField_Conveyor()
{
	if(m_pxVolumeController)
	{
		delete m_pxVolumeController;
	}
	m_pxVolumeController = 0;
}

void Tools_Entity_ForceField_Conveyor::AddOverlayControls (Asura_Overlay* pxOverlay,
												  Asura_Vector_2& xOffset)
{
	Asura_Dialog*					pxDialog;

	// Add any parent controls...
	PARENT::AddOverlayControls(pxOverlay, xOffset);

	// Add our own controls...
	xOffset.y += Asura_Dialog::SmallSpacing();

	pxDialog = new Asura_Function_Dialog("Edit Volume",xOffset,EditVolume);
	pxDialog->SetHelpText("Edits the volume for this field.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();


	pxDialog = new Asura_Numerical_Dialog("Force Magnitude",xOffset,&m_fForceMagnitude,0,100.0f,0.1f,"%0.2f",&Changed);	
	pxDialog->SetHelpText("Sets the magnitude of the force, in metres/second.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();


	pxDialog = new Asura_AngleNumerical_Dialog("Force Horizontal Angle",xOffset,&m_xForceEuler.y,-180.0f,180.0f,1.0f,"%0.2f",&Changed);	
	pxDialog->SetHelpText("Sets the Horizontal component of the force direction.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();
	
	pxDialog = new Asura_AngleNumerical_Dialog("Force Vertical Angle",xOffset,&m_xForceEuler.x,-180.0f,180.0f,1.0f,"%0.2f",&Changed);	
	pxDialog->SetHelpText("Sets the vertical component of the force direction.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Get force orientation from object orientation",xOffset,GetForceOrientFromObjectOrient,(int)this);
	pxDialog->SetHelpText("Generates the force orientation from the orientation of the forcefield object.");	
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

}

void Tools_Entity_ForceField_Conveyor::UpdateWhileSelected (void)
{
	Asura_ServerEntity_ForceField_Conveyor*	pxAsuraEntity	= static_cast<Asura_ServerEntity_ForceField_Conveyor*>(Entity);

	if(Changed)
	{
		// Initialise our local variables...
		pxAsuraEntity->SetActive(m_bActive);

		// Set the vector from the stored Euler
		Asura_Matrix_3x3 xForceOri;
		m_xForceEuler.ToMatrix(xForceOri);
		Asura_Vector_3 xForceDir(0.0f, 0.0f, 1.0f);
		xForceDir = xForceDir * xForceOri;
		xForceDir = xForceDir * m_fForceMagnitude;
		pxAsuraEntity->SetForce(xForceDir);

		pxAsuraEntity->SetForceEuler(m_xForceEuler);

		pxAsuraEntity->UpdatePhysicsFields();

	}

	// Call the parent...
	PARENT::UpdateWhileSelected();
}






bool Tools_Entity_ForceField_Conveyor::HandlesThisMessage (int iMessageID)
{
	switch(iMessageID)
	{
		case ASURA_MSG_DESTRUCT:
		case ASURA_MSG_ACTIVATE:
		case ASURA_MSG_DEACTIVATE:
		case ASURA_MSG_TRIGGER:
		case ASURA_MSGID_TELEPORT_ENTITY:
		{
			return(true);
		}

		default:
		{
			return(PARENT::HandlesThisMessage(iMessageID));
		}
	}		// End of switch.
}









void Tools_Entity_ForceField_Conveyor::Copy (const Tools_Object* pxObject)
{
	const Tools_Entity_ForceField_Conveyor*			pxForceFieldObject	= static_cast<const Tools_Entity_ForceField_Conveyor*>(pxObject);
	const Asura_ServerEntity_ForceField_Conveyor*	pxEntityToCopy		= static_cast<Asura_ServerEntity_ForceField_Conveyor*>(pxForceFieldObject->Entity);
	Asura_ServerEntity_ForceField_Conveyor*			pxEntity			= static_cast<Asura_ServerEntity_ForceField_Conveyor*>(Entity);

	PARENT::Copy(pxObject);

	// Copy our data...
	pxEntity->SetActive(pxEntityToCopy->IsActive());

	// Initialise our local variables...
	m_bActive		= pxEntity->IsActive();

	//delete the volume controller if it exists ,since we will be replacing the volumes
	delete m_pxVolumeController;
	m_pxVolumeController = 0;
	
	//copy everything between the entities
	pxEntity->m_uNumVolumes = pxEntityToCopy->m_uNumVolumes;

	delete [] pxEntity->m_pxVolumes;
	if(pxEntity->m_uNumVolumes)
	{
		pxEntity->m_pxVolumes = new Asura_OrientedVolume[pxEntity->m_uNumVolumes];
		for(u_int u=0;u<pxEntity->m_uNumVolumes;u++)
		{
			pxEntity->m_pxVolumes[u] = pxEntityToCopy->m_pxVolumes[u];
		}
	}
	else
	{
		pxEntity->m_pxVolumes = 0;
	}

}




Tools_Object* Tools_Entity_ForceField_Conveyor::CreateObjectOfThisType (void) const
{
	Create();
	return(Tools_Object_System::SelectedObject);
}




void Tools_Entity_ForceField_Conveyor::OnSelection(void)
{
	PARENT::OnSelection();
	s_pxSelectedForceField	= this;
}




void Tools_Entity_ForceField_Conveyor::OnDeselection(void)
{
	PARENT::OnDeselection();
	s_pxSelectedForceField	= NULL;
}




void Tools_Entity_ForceField_Conveyor::Create (void)
{
	const Asura_Guid	uGuid		= Asura_Entity_System::GetCreatedByToolsGuid();
	Tools_Entity_Data*	pxToolsData	= CreateToolsDataWithDefaultName("ForceField", "ForceField", uGuid, GetNamePrefix());

	if(pxToolsData == NULL)
	{
		Asura_Assert(false, "Tools_Entity_ForceField_Conveyor::Create() - no tools data" );
	}
	else
	{
		Asura_ServerEntity_ForceField_Conveyor::Create(uGuid);

		Asura_ServerEntity_ForceField_Conveyor*	pxServer	= Asura_ServerEntity_ForceField_Conveyor::FindEntity(uGuid);

		if(pxServer == NULL)
		{
			Asura_Assert(false, "Tools_Entity_ForceField_Conveyor::Create() - no server object");
		}
		else
		{
			pxServer->ToolsData	= pxToolsData;

			Tools_Entity_ForceField_Conveyor* pxToolsObject = new Tools_Entity_ForceField_Conveyor(pxServer);

			if(pxToolsObject == NULL)
			{
				Asura_Assert(false, "Tools_Entity_ForceField_Conveyor::Create() - no Tools object" );
			}
			else
			{
				Tools_Object_System::SelectObject(pxToolsObject);

				pxToolsObject->Position		= *Asura_Camera::GetPosition();
				pxToolsObject->Orientation	= *Asura_Camera::GetOrientation();

				Tools_Timed_Message_System::AddMessage(0xffffff, "Created new ForceField");
				Tools_SaveAsura::SetEntityModified();
			}
		}
	}
}



void Tools_Entity_ForceField_Conveyor::UpdateTeleportEntityText(const Asura_StaticMessage& xMessage,
													   Asura_String& xString)
{
#ifdef ASURA_DONT_USE_TELEPORTERS
	const Asura_Guid					uGuid	= ASURA_GUID_UNREGISTERED;
#else // #ifdef ASURA_DONT_USE_TELEPORTERS
	const Asura_Message_TeleportEntity*	pxData	= static_cast<Asura_Message_TeleportEntity*>(xMessage.m_pData);
	const Asura_Guid					uGuid	= pxData->GetGuidToTeleport();
#endif // #else // #ifdef ASURA_DONT_USE_TELEPORTERS

	if(uGuid == ASURA_GUID_UNREGISTERED)
	{
		xString	= "GUID: <None>";
	}
	else
	{
		Tools_Object*	pxObject	= Tools_Object_System::GetObjectFromGuid(uGuid);
		char			szText[256];

		if(pxObject == NULL)
		{
			Asura_PrintfToBfr(szText, "GUID: %d - <Not found>", 256, true, uGuid);
		}
		else
		{
			const char*	szName	= pxObject->GetName();

			Asura_PrintfToBfr(szText, "GUID: %d - \"%s\"", 256, true, uGuid, szName);
		}

		xString	= szText;
	}
}


void Tools_Entity_ForceField_Conveyor::AddMessageOverlayControls_TeleportEntity(Asura_StaticMessage* pxMessage,
																	   Asura_Overlay* pxOverlay,
																	   Asura_Vector_2& xOffset)
{
	Asura_String	xString;

	UpdateTeleportEntityText(*pxMessage, xString);
}





void Tools_Entity_ForceField_Conveyor::Render(void)
{
	// Only render lines for selected ForceFields...
	if (IsSelected())
	{
		const Asura_ServerEntity_ForceField_Conveyor*	pxAsuraEntity	= static_cast<Asura_ServerEntity_ForceField_Conveyor*>(Entity);

		//if(pxServer)
		//{
		//	Asura_Render::RenderLine(Position,pxServer->GetPosition(),0xFFFF8000);
		//}

		Asura_Render::RenderLine(pxAsuraEntity->GetPosition(), pxAsuraEntity->GetPosition() + pxAsuraEntity->GetForce(), 0xFFFF0000);
	}


	// Call the base-function...
	PARENT::Render();
}

/// End of function 'Tools_Entity_ForceField_Conveyor::Render'. ///


void Tools_Entity_ForceField_Conveyor::EditVolume()
{
	Asura_Assert(ThisEntity,"");

	Tools_Entity_ForceField_Conveyor* pxToolsEntity = (Tools_Entity_ForceField_Conveyor*) ThisEntity;
	//create controller for editing volumes if it hasn't been done yet
	if(!pxToolsEntity->m_pxVolumeController)
	{
		pxToolsEntity->m_pxVolumeController = new Tools_ForceField_VolumeEdit_Controller(pxToolsEntity);
	}

	//and get it to create the volume objects
	pxToolsEntity->m_pxVolumeController->StartVolumeEditing();
	pxToolsEntity->Changed = true;
}

void Tools_Entity_ForceField_Conveyor::UpdatePositionInAsuraData(bool bUpdateLinkedObjects)
{
	m_xPreviousPosition = Position;

	PARENT::UpdatePositionInAsuraData(bUpdateLinkedObjects);
}


void Tools_Entity_ForceField_Conveyor::GetForceOrientFromObjectOrient()
{
	const Asura_ServerEntity_ForceField_Conveyor*	pxAsuraEntity	= static_cast<Asura_ServerEntity_ForceField_Conveyor*>(Entity);

	if(pxAsuraEntity)
	{
		const Asura_Matrix_3x3 xOri = pxAsuraEntity->GetOrientation();
		Asura_Euler_3 xEuler;

		xEuler.FromMatrix(xOri);

		m_xForceEuler.x = xEuler.x;
		m_xForceEuler.y = xEuler.y;
		m_xForceEuler.z = 0.0f;

		Changed = true;
	}	
}




Tools_ForceField_VolumeEdit_Controller::Tools_ForceField_VolumeEdit_Controller(Tools_Entity_ForceField_Conveyor* pxToolsEntity)
:Tools_VolumeEdit_Controller(&pxToolsEntity->Changed)
, m_bInUse(false)
{
	m_pxToolsEntity = pxToolsEntity;
	m_pxAsuraEntity = static_cast<Asura_ServerEntity_ForceField_Conveyor*>(pxToolsEntity->Entity);
}

void Tools_ForceField_VolumeEdit_Controller::StartVolumeEditing()
{
	m_bInUse = true;
	for (u_int u = 0; u < m_pxAsuraEntity->GetNumberOfVolumes(); ++u)
	{
		Asura_Vector_3 xDisplacement(m_pxToolsEntity->Position);
		m_pxAsuraEntity->GetVolumeBox(u)->Displace(xDisplacement);
	}

	Tools_VolumeEdit_Controller::StartVolumeEditing();

	if(GetNumberOfVolumes() == 0)
	{
		//create a volume if none exists yet
		CreateVolume();
	}
}

void Tools_ForceField_VolumeEdit_Controller::EndVolumeEditing()
{
	Tools_VolumeEdit_Controller::EndVolumeEditing();

	for (u_int u = 0; u < m_pxAsuraEntity->GetNumberOfVolumes(); ++u)
	{
		Asura_Vector_3 xDisplacement(-m_pxToolsEntity->Position);
		m_pxAsuraEntity->GetVolumeBox(u)->Displace(xDisplacement);
	}

	m_bInUse = false;

	//select the owner
	Tools_Object_System::SelectObject(m_pxToolsEntity);
}

bool Tools_ForceField_VolumeEdit_Controller::CanDeleteVolume()
{
	return GetNumberOfVolumes()>1;
}

//Gets the current number of volumes
int Tools_ForceField_VolumeEdit_Controller::GetNumberOfVolumes()
{
	return m_pxAsuraEntity->GetNumberOfVolumes();
}

//Gets a pointer to the bounding box for a given volume
Asura_Bounding_Box* Tools_ForceField_VolumeEdit_Controller::GetVolumeBox(int iIndex)
{
	return m_pxAsuraEntity->GetVolumeBox(iIndex);
}

//Gets a pointer to the orientation for a given volume. Returns 0 if there is none
Asura_Quat* Tools_ForceField_VolumeEdit_Controller::GetVolumeOrient(int iIndex)
{
	return m_pxAsuraEntity->GetVolumeOrient(iIndex);
}

//Gets a default position for volume objects that have yet to gain a proper volume
Asura_Vector_3 Tools_ForceField_VolumeEdit_Controller::GetDefaultPosition()
{
	return m_pxToolsEntity->Position;
}

//Gets the name of the owner , to use for part of the volume names
const char* Tools_ForceField_VolumeEdit_Controller::GetName()
{
	return m_pxToolsEntity->GetName();
}

void Tools_ForceField_VolumeEdit_Controller::CreateVolumeMain()
{
	m_pxAsuraEntity->AddVolume();

}

void Tools_ForceField_VolumeEdit_Controller::DeleteVolumeMain(int iIndex)
{
	m_pxAsuraEntity->RemoveVolume(iIndex);
}



/// End of file. ///
