
///////////////////////////////////////////////////////////////////////////////
//
// File   : Tools_Entity_ForceField_Conveyor.h
// Date   : 28-Apr-2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////



#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __TOOLS_ENTITY_FORCEFIELD_CONVEYOR_H__
#define __TOOLS_ENTITY_FORCEFIELD_CONVEYOR_H__


////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Teleport/Asura_Teleport_System.h"

#include "Entity/Intangible/Asura_ServerEntity_ForceField_Conveyor.h"
#include "Entity/Tools_Entity_Activatable_Object.h"
#include "Entity/Tools_Entity_Object.h"
#include "Objects/Tools_Object_System.h"
#include "Objects/Tools_VolumeEdit_Object.h"

////////////////////////////////////////////////////////////////////////////////
// Forward class declarations...
////////////////////////////////////////////////////////////////////////////////



////////////////////////////////////////////////////////////////////////////////
// Typedefs...
////////////////////////////////////////////////////////////////////////////////
typedef bool ProjectCanBeSelectedFunction(const Tools_Object* pxObject);


////////////////////////////////////////////////////////////////////////////////
// Class declaration...
////////////////////////////////////////////////////////////////////////////////

struct Asura_StaticMessage;
class Tools_ForceField_VolumeEdit_Controller;


////////////////////////////////////////////////////////////////////////////////
// Class        : Tools_Entity_ForceField_Conveyor
// Author       : Andy Tate
// Date         : 28-Feb-2007
//
// Description  : Tools class used to allow manipulation of ForceField entities.
////////////////////////////////////////////////////////////////////////////////

class Tools_Entity_ForceField_Conveyor : public Tools_Entity_ActivatableWithPosition_Object
{
public:
	// Public functions...
	Tools_Entity_ForceField_Conveyor          (Asura_Entity* pxEntity);
	~Tools_Entity_ForceField_Conveyor ();

	static void	AddMessageOverlayControls_TeleportEntity	(Asura_StaticMessage* pxMessage, Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset);
	static void	UpdateTeleportEntityText					(const Asura_StaticMessage& xMessage, Asura_String& xString);

	virtual void UpdatePositionInAsuraData(bool /*updateLinkedObjects*/);

	// Overridden functions...
	virtual	const char*	GetTypeDescription	(void)	{ return("ForceField"); }

	virtual const char*	GetMyNamePrefix		(void) const	{ return Tools_Entity_ForceField_Conveyor::GetNamePrefix(); }

	virtual	void		AddOverlayControls	(Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset);
	virtual	void		UpdateWhileSelected	(void);
	virtual	bool		HandlesThisMessage	(int iMessageID);
	virtual void		Render				(void);

	virtual void		OnSelection			(void);
	virtual void		OnDeselection		(void);

	virtual bool SupportsMessageEditor() { return false; }

	// Overridden copy functions...
	virtual	bool			CanBeCopied				(void) const	{ return(true); }
	virtual	void			Copy					(const Tools_Object* pxObject);
	virtual	Tools_Object*	CreateObjectOfThisType	(void) const;

	// Static functions...
	static void		Create	(void);

	static const char* GetNamePrefix() { return "FFC_"; }

protected:
	// Protected variables...

	// Protected functions...
	void		UpdateDialogueText			(const int iIndex);

	static void EditVolume();


private:
	// Private typedefs...
	typedef Tools_Entity_ActivatableWithPosition_Object PARENT;

	static Tools_Entity_ForceField_Conveyor*	s_pxSelectedForceField;

	static bool s_bFieldModified;

	// Private functions...
	Tools_Entity_ForceField_Conveyor  (void);
	Tools_Entity_ForceField_Conveyor  (const Tools_Entity_ForceField_Conveyor& xOther);

	Tools_Entity_ForceField_Conveyor&	operator=	(const Tools_Entity_ForceField_Conveyor& xOther);

	Tools_ForceField_VolumeEdit_Controller* m_pxVolumeController;
	
	Asura_Vector_3 m_xPreviousPosition;

	float			m_fForceMagnitude;
	Asura_Euler_3	m_xForceEuler;

	//overlay callback functions
	//defines for routing static callback functions to non-static versions
	#define OVERLAY_VOID_CALLBACK(__FnName)\
	static void __FnName(int iThis){((Tools_Entity_ForceField_Conveyor*)iThis)->__FnName();}\
	void __FnName();

	OVERLAY_VOID_CALLBACK(GetForceOrientFromObjectOrient)

	#undef OVERLAY_VOID_CALLBACK
};   // End of class 'Tools_Entity_ForceField_Conveyor'.





class Tools_ForceField_VolumeEdit_Controller : public Tools_VolumeEdit_Controller
{
private:
	typedef Tools_VolumeEdit_Controller PARENT;
	public :
		Tools_ForceField_VolumeEdit_Controller(Tools_Entity_ForceField_Conveyor* pxToolsEntity);

		virtual void StartVolumeEditing();
		virtual void EndVolumeEditing();

		bool IsInUse() const {return m_bInUse;}
	
	protected :
		virtual bool CanCreateVolume() { return true;}
		
		virtual bool CanDeleteVolume();

		//Gets the current number of volumes
		virtual int GetNumberOfVolumes();
		
		//Gets a pointer to the bounding box for a given volume
		virtual Asura_Bounding_Box* GetVolumeBox(int iIndex);

		//Gets a pointer to the orientation for a given volume. Returns 0 if there is none
		virtual Asura_Quat* GetVolumeOrient(int iIndex);

		//Gets a default position for volume objects that have yet to gain a proper volume
		virtual Asura_Vector_3 GetDefaultPosition();

		//Gets the name of the owner , to use for part of the volume names
		virtual const char* GetName();

		//Creates the new volume in the asura data
		virtual void CreateVolumeMain();

		//Deletes the volume from the asura data
		virtual void DeleteVolumeMain(int iIndex);

	private :
		Tools_Entity_ForceField_Conveyor*	m_pxToolsEntity;
		Asura_ServerEntity_ForceField_Conveyor * m_pxAsuraEntity;
		bool m_bInUse;

};




#endif   // ifndef __TOOLS_ENTITY_FORCEFIELD_CONVEYOR_H__

/// End of file. ///
