//////////////////////////////////////////////////////////////////////////////
//
// Tools_Entity_Lift.h
//
// Copyright (c) 2006 Rebellion. All rights reserved.
//
// Description:
//
//     Lift Entity - External control physics object moving along a spline
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef ___TOOLS_ENTITY_LIFT_H___
#define ___TOOLS_ENTITY_LIFT_H___ 1

// Engine independent.
#include "Entity/Tools_Entity_PhysicalObject.h"
#include "Network/Asura_Guid.h"

// tools
#include "Objects/Tools_Object.h"
#include "Objects/Tools_Object_System.h"

// C O N S T A N T S /////////////////////////////////////////////////////////

const Asura_Hash_ID uTOOLS_SHAREDPARAMHASH_LIFT_ANIM_LOOPING      = 0x610DA8B7; // Asura_GetHashID( "Asura_Lift_Looping" );
const Asura_Hash_ID uTOOLS_SHAREDPARAMHASH_LIFT_ANIM_REVERSED     = 0x2A4B3EE9; // Asura_GetHashID( "Asura_Lift_Reversed" );
const Asura_Hash_ID uTOOLS_SHAREDPARAMHASH_LIFT_USE_ANIM_MOVEMENT = 0x6BECD5C0; // Asura_GetHashID( "Asura_Lift_UseAnimMovement" );
const Asura_Hash_ID uTOOLS_SHAREDPARAMHASH_LIFT_NOSHADOW          = 0x96C8A748; // Asura_GetHashID( "Asura_Lift_NoShadow" );
const Asura_Hash_ID uTOOLS_SHAREDPARAMHASH_LIFT_NOCOLLISIONS      = 0x04F679A9; // Asura_GetHashID( "Asura_Lift_NoCollisions" );
const Asura_Hash_ID uTOOLS_SHAREDPARAMHASH_LIFT_IGNOREMODLINKING  = 0x78DF19A3; // Asura_GetHashID( "Asura_Lift_IgnoreModuleLinking" );
const Asura_Hash_ID uTOOLS_SHAREDPARAMHASH_LIFT_HIDDEN            = 0x4646F091; // Asura_GetHashID( "Asura_Lift_Hidden" );
const Asura_Hash_ID uTOOLS_SHAREDPARAMHASH_LIFT_IGNOREFORLOS      = 0xAE2917E0; // Asura_GetHashID( "Asura_Lift_IgnoreForLOS" );
const Asura_Hash_ID uTOOLS_SHAREDPARAMHASH_LIFT_FULLBRIGHT        = 0xDD72F1B0; // Asura_GetHashID( "Asura_Lift_FullBright" );
const Asura_Hash_ID uTOOLS_SHAREDPARAMHASH_LIFT_POLYGONAL         = 0x15E56A9E; // Asura_GetHashID( "Asura_Lift_Polygonal" );
const Asura_Hash_ID uTOOLS_SHAREDPARAMHASH_LIFT_RUNNING           = 0xA8A11FB8; // Asura_GetHashID( "Asura_Lift_Running" );
const Asura_Hash_ID uTOOLS_SHAREDPARAMHASH_LIFT_REVERSE           = 0x8DC05CDB; // Asura_GetHashID( "Asura_Lift_Reverse" );
const Asura_Hash_ID uTOOLS_SHAREDPARAMHASH_LIFT_AUTOREVERSE       = 0xFB25E7CC; // Asura_GetHashID( "Asura_Lift_AutoReverse" );
const Asura_Hash_ID uTOOLS_SHAREDPARAMHASH_LIFT_COLLIDEENV        = 0x9226CF56; // Asura_GetHashID( "Asura_Lift_CollideEnv" );
const Asura_Hash_ID uTOOLS_SHAREDPARAMHASH_LIFT_COLLIDESTATIC     = 0xEA8A8365; // Asura_GetHashID( "Asura_Lift_CollideStatic" );
const Asura_Hash_ID uTOOLS_SHAREDPARAMHASH_LIFT_COLLIDEDYNAMIC    = 0x567F4A28; // Asura_GetHashID( "Asura_Lift_CollideDynamic" );
const Asura_Hash_ID uTOOLS_SHAREDPARAMHASH_LIFT_PAUSEONCOLLIDE    = 0x09BF8430; // Asura_GetHashID( "Asura_Lift_PauseOnCollide" );

// C L A S S E S /////////////////////////////////////////////////////////////

class Asura_ServerEntity_Lift;
struct Asura_StaticMessage;

class Tools_Lift_SelectSpline : public Tools_ChooseObject
{
public:
    Tools_Lift_SelectSpline( Asura_Guid* puGuidToSet ) : m_puGuidToSet( puGuidToSet ) {}
    virtual bool CanBeSelected( Tools_Object* pxObject );
	virtual void ObjectSelected( Tools_Object* pxObject ) { pxObject?(*m_puGuidToSet = pxObject->GetGuid()):ASURA_GUID_UNREGISTERED; }
    virtual void OnFinish() { delete this; }

private:
    Asura_Guid* m_puGuidToSet;
};

////////////////////////////////////////////////////////////////////////////////
// Tools_Entity_Lift														  //
//																			  //
// *NOTE*: only tools objects that have Asura_ServerEntitiy_Lift			  //
// entities associated with them should inherit from this class.              //
//                                                                            //
////////////////////////////////////////////////////////////////////////////////

class Tools_Entity_Lift : public virtual Tools_Entity_PhysicalObject
{
public:
	static void Project_Create( void );
    static Tools_Entity_Lift* FindEntity( Asura_Guid uGUID ) { return s_xList.FindEntity( uGUID ); }
	static const Asura_Hashed_List<Tools_Entity_Lift>* GetList( void ) { return &s_xList; }

	static void SelectSpline();
	static void AddLiftNodeAtCurrentLiftPosition();
	static void ResetSpline();
	static void Preview();
	static void PreviewEnd();

	static void AddFlagsMessageOverlayControls(Asura_StaticMessage* pxMessage,class Asura_Overlay* pxOverlay,class Asura_Vector_2& xOffset);

    virtual bool AddSharedParamDialogToOverlay( Asura_Overlay* pxOverlay, const Asura_Vector_2& xOffset, Asura_Hash_ID uParamID );
	virtual bool GetParameter( Asura_Hash_ID uParamID, Tools_Parameter* pxParam ) const;
    virtual void Render( void );
	virtual bool SetParameter( Asura_Hash_ID uParamID, const Tools_Parameter* pxParam );

	virtual void AddFilesToExportList(List<Asura_String>& xNameList) const;

	virtual void AddContextMenuItems(class Tools_ContextMenu& xMenu, const class Tools_ContextMenu_Context& xContext);

protected:
	void AddLiftNodeAtCurrentPosition();

	static void SelectShape_Dialog();
    static void SelectSkinAndAnim_Dialog();
    static void SelectAnim_Dialog();
	static void SelectAllOfType();
	static void ControlPointsUseThisShape();
	static void ControlPointsUseDefaultShape();
	static void CreateLiftNodeAtCurrentPosition();

	static bool IsNoneAnimatingObject();
	static bool IsLiftSplinePresent();
	static bool IsLiftSplineMissing();
	static bool IsPreviewActiveAndLiftSplinePresent();
	static bool IsPreviewInactive();
	static bool IsPreviewInactiveAndLiftSplinePresent();
	static bool AlreadyCreatedNode();
	static bool CanAddLiftNode();
	static bool CanSetSnapSplineNode();
	static bool CanSetStartPosition();

	static void ContextMenu_SelectLiftAndSpline(void* pObject, u_int uMode);

	Tools_Entity_Lift(Asura_ServerEntity_Lift* pxLift);
	virtual ~Tools_Entity_Lift();

	virtual const char* GetTypeDescription(){return "Lift";}
	virtual const char* GetTypeDescriptionForObjectType(){return "Lift";}
	virtual void AddOverlayControls(Asura_Overlay* pxNewOverlay, Asura_Vector_2& xOffset);
	
	virtual void PrintInfo(float& x,float& y) {}

	virtual void OnSelection();
	virtual void OnDeselection();
	virtual void UpdateWhileSelected();
	virtual void Update();
	virtual bool ShouldHideDuringCutscene();

    virtual void SelectShape( void );
    virtual void SelectSkinAndAnim( void );
    virtual void SelectAnim( void );

	virtual void Copy(const Tools_Object*);
	virtual bool CanBeCopied() const {return Type == TOT_Entity_Lift;}
	virtual Tools_Object* CreateObjectOfThisType() const;

	virtual bool UseModuleLinking();

	virtual bool HandlesThisMessage(int iMessageID);
	virtual bool HandlesThisEvent(int iEventID) const;

protected:
	void PlayLiftSound(Asura_Hash_ID uSoundEventHash);

    int m_iAnimFlags;

	Asura_ServerEntity_Lift* m_pxLift;

	bool		m_bPreviewing;
	Asura_Guid	m_uPreviewSpline;
	bool		m_bPreviewReverse;
	float		m_fPreviewProgress;
	bool		m_bChangedSoundEvent;
	Asura_Handle m_iPreviewSoundHandle;

	Asura_LinkList< Asura_Guid > m_xCreationList;

	bool		m_bSnapLiftToSplineNode;		// Flag to snap the lift to the specified spline-node.
	bool		m_bSnapLiftToSplineNodeChanged;
	int			m_iCurrentSplineNode;			// The spline-node we currently have selected.
	int			m_iPreviousSplineNode;			// The spline-node we previosuly selected.

	static const char* const s_szLongName;
	static const char* const s_szShortName;
	static Tools_Entity_Lift* s_pxCurrentlySelectedObject;

private:
	static void StartChoiceChanged();

	bool m_bShowStartPosition;
	bool m_bShowStartPositionChanged;
	int m_iStartLiftNode;
	bool m_bStartLiftNodeChanged;

    typedef Tools_Entity_PhysicalObject PARENT;

	static Asura_Hashed_List<Tools_Entity_Lift> s_xList;
};

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

#endif // #ifndef ___TOOLS_ENTITY_LIFT_H___
