//
// Tools_Entity_MusicTrigger_Object.cpp
//
// Created by Richard Rice - 23/01/2004
// 
// Copyright (C) 2004 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

#ifndef ASURA_SUPPORT_DYNAMIC_MUSIC_SYSTEM

#include "Entity/Tools_Entity_MetaMusicTrigger_Object.h"

#include "Entity/Triggers/Asura_ServerEntity_MetaMusicTrigger.h"
#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Overlay.h"
#include "Mesh/Asura_Mesh_System.h"
#include "Sound/Asura_MetaMusic_System.h"

#include "Load&Save/Tools_SaveAsura.h"
#include "Misc/Tools_Modal_Overlay.h"
#include "Objects/Tools_Object_System.h"
#include "Sound/Tools_Mode_MetaMusic.h"




Tools_Entity_MetaMusicTrigger_Object::Tools_Entity_MetaMusicTrigger_Object(Asura_Entity* pxEntity)
:Tools_Entity_Activatable_Object(pxEntity)
{
	Type = TOT_Entity_MetaMusicTrigger;
	Mesh = Asura_Mesh_System::GetMesh("ToolsShape_Note");


	m_bAmbientChanged = false;
	m_bTensionChanged = false;
	m_bActionChanged = false;
	m_pxDialog_SelectTrack = 0;
	m_pxDialog_HighPriority = 0;
	m_pxDialog_ChangeTrack = 0;
	m_pxDialog_RestartLastTrack = 0;

}


void Tools_Entity_MetaMusicTrigger_Object::AddOverlayControls(class Asura_Overlay* pxOverlay,class Asura_Vector_2& xOffset)
{
	Tools_Entity_Activatable_Object::AddOverlayControls(pxOverlay,xOffset);
	Asura_Dialog* pxDialog;

	Asura_ServerEntity_MetaMusicTrigger* pxAsuraEntity = (Asura_ServerEntity_MetaMusicTrigger*) Entity;

	pxDialog = new Asura_FlagTickbox_Dialog("Restart Last Track", xOffset, &pxAsuraEntity->m_uFlags, ASURA_METAMUSICTRIGGER_FLAG_RESTARTLASTTRACK, &Changed);
	m_pxDialog_RestartLastTrack = pxDialog;
	pxDialog->SetHelpText("Restarts the most recently played track , if it has stopped.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_FlagTickbox_Dialog("Change Track", xOffset, &pxAsuraEntity->m_uFlags, ASURA_METAMUSICTRIGGER_FLAG_CHANGETRACK, &Changed);
	m_pxDialog_ChangeTrack = pxDialog;
	pxDialog->SetHelpText("If set then the meta music track will be changed to the one selected , or just stopped if none is selected.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_FlagTickbox_Dialog("High Priority", xOffset, &pxAsuraEntity->m_uFlags, ASURA_METAMUSICTRIGGER_FLAG_HIGHPRIORITY, &Changed);
	m_pxDialog_HighPriority = pxDialog;
#ifdef ASURA_USE_METAMUSICZONES
	pxDialog->SetHelpText( "If 'High Priority' is set then the current track will fade out immediately. "
                           "If not set then the outro of the current track will be played first. "
                           "If a Meta Music Zone is currently controlling the music and 'High Priority' "
                           "is set then this trigger will override the meta music zone." );
#else
	pxDialog->SetHelpText( "If 'High Priority' is set then the current track will fade out immediately. "
                           "If not set then the outro of the current track will be played first." );
#endif
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	//get text for music selection control
	char szText[100];
	if(pxAsuraEntity->m_uTrackID == ASURA_HASH_ID_UNSET)
	{
		strcpy(szText,"Track : <Stop Music>");
	}
	else
	{
		const char* pszTrackName = GetTrackName();
		Asura_PrintfToBfr(szText,"Track : %s",100,true,pszTrackName ? pszTrackName : "<Unknown>");
	}
	

	pxDialog = new Asura_Function_Dialog(szText,xOffset,SelectMusicTrack,(int)this);
	m_pxDialog_SelectTrack = pxDialog;
	pxDialog->SetHelpText("Choose the track to play.\nAlso used for stopping music.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_FlagTickbox_Dialog("Change To Ambient", xOffset, &pxAsuraEntity->m_uFlags, ASURA_METAMUSICTRIGGER_FLAG_AMBIENT, &m_bAmbientChanged);
	pxDialog->SetHelpText("Switch the currently playing meta music track to the ambient state.\nIf no track is playing , this does nothing.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_FlagTickbox_Dialog("Change To Tension", xOffset, &pxAsuraEntity->m_uFlags, ASURA_METAMUSICTRIGGER_FLAG_TENSION, &m_bTensionChanged);
	pxDialog->SetHelpText("Switch the currently playing meta music track to the tension state.\nIf no track is playing , this does nothing.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_FlagTickbox_Dialog("Change To Action", xOffset, &pxAsuraEntity->m_uFlags, ASURA_METAMUSICTRIGGER_FLAG_ACTION, &m_bActionChanged);
	pxDialog->SetHelpText("Switch the currently playing meta music track to the action state.\nIf no track is playing , this does nothing.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();



}

void Tools_Entity_MetaMusicTrigger_Object::UpdateWhileSelected()
{
	Asura_ServerEntity_MetaMusicTrigger* pxAsuraEntity = (Asura_ServerEntity_MetaMusicTrigger*) Entity;

	//make sure onlt one of ambient/tension/action gets set
	if(m_bAmbientChanged)
	{
		pxAsuraEntity->m_uFlags &=~ (ASURA_METAMUSICTRIGGER_FLAG_TENSION|ASURA_METAMUSICTRIGGER_FLAG_ACTION);
		m_bAmbientChanged = false;
		Changed = true;		
	}
	if(m_bTensionChanged)
	{
		pxAsuraEntity->m_uFlags &=~ (ASURA_METAMUSICTRIGGER_FLAG_AMBIENT|ASURA_METAMUSICTRIGGER_FLAG_ACTION);
		m_bTensionChanged = false;
		Changed = true;		
	}
	if(m_bActionChanged)
	{
		pxAsuraEntity->m_uFlags &=~ (ASURA_METAMUSICTRIGGER_FLAG_TENSION|ASURA_METAMUSICTRIGGER_FLAG_AMBIENT);
		m_bActionChanged = false;
		Changed = true;		
	}

	//update the enabled status of some controls
	m_pxDialog_RestartLastTrack->SetEnabled(true);	
	m_pxDialog_ChangeTrack->SetEnabled(true);
	if(pxAsuraEntity->m_uFlags & ASURA_METAMUSICTRIGGER_FLAG_RESTARTLASTTRACK)
	{
		m_pxDialog_ChangeTrack->SetEnabled(false);
		pxAsuraEntity->m_uFlags &=~ ASURA_METAMUSICTRIGGER_FLAG_CHANGETRACK;
	}
	else if(pxAsuraEntity->m_uFlags & ASURA_METAMUSICTRIGGER_FLAG_CHANGETRACK)
	{
		m_pxDialog_RestartLastTrack->SetEnabled(false);	
	}

	bool bEnable = (pxAsuraEntity->m_uFlags & ASURA_METAMUSICTRIGGER_FLAG_CHANGETRACK) ? true : false;
	if(m_pxDialog_HighPriority) m_pxDialog_HighPriority->SetEnabled(bEnable);
	if(m_pxDialog_SelectTrack) m_pxDialog_SelectTrack->SetEnabled(bEnable);

	Tools_Entity_Activatable_Object::UpdateWhileSelected();
}

void Tools_Entity_MetaMusicTrigger_Object::Create()
{
	Tools_Object* pxObject = Create(false);
	Tools_Object_System::SelectObject(pxObject);
}

Tools_Entity_MetaMusicTrigger_Object* Tools_Entity_MetaMusicTrigger_Object::Create(bool bGenerateName)
{
	//get a name , and the position data
	Asura_Guid uGuid = Asura_Entity_System::GetNextToolsCreatedGuid();
	Tools_Entity_Data* pxToolsData = 0;
	if(bGenerateName)
	{
		pxToolsData = Tools_Entity_Object::CreateToolsDataWithDefaultName("Meta Music Trigger","Meta Music Trigger",uGuid,GetNamePrefix());
	}
	else
	{
		pxToolsData = Tools_Entity_Object::CreateToolsData("Meta Music Trigger",0,GetNamePrefix());
	}
	if(!pxToolsData) return 0 ;

	//create the asura data
	Asura_ServerEntity_MetaMusicTrigger* pxAsuraEntity = new Asura_ServerEntity_MetaMusicTrigger();
	pxAsuraEntity->ToolsData = pxToolsData;

	//create the tools pxObject
	Tools_Entity_MetaMusicTrigger_Object* pxObject = new Tools_Entity_MetaMusicTrigger_Object(pxAsuraEntity);

	Tools_SaveAsura::SetEntityModified();

	return pxObject;
}

const char* Tools_Entity_MetaMusicTrigger_Object::GetTrackName() const
{
	const Asura_ServerEntity_MetaMusicTrigger* pxAsuraEntity = (const Asura_ServerEntity_MetaMusicTrigger*) Entity;

	const Asura_MetaMusic_TrackIndex xIndex = Asura_MetaMusic_System::GetTrackIndexFromHash(pxAsuraEntity->m_uTrackID);
	if (xIndex.IsNull()) return 0;

	return Asura_MetaMusic_System::GetTrackName(xIndex);
}

void Tools_Entity_MetaMusicTrigger_Object::SelectMusicTrack()
{
	Asura_ServerEntity_MetaMusicTrigger* pxAsuraEntity = (Asura_ServerEntity_MetaMusicTrigger*) Entity;

									
	ListSelection_Overlay xOverlay("Select Meta Music Track");
	#ifdef ASURA_USE_STREAMING
	//add global track names
	Asura_MetaMusic_Handler* pxGlobalHandler = Tools_Mode_MetaMusic::GetGlobalHandler();
	for(int i=0;i<pxGlobalHandler->GetNumberOfTracks();i++)
	{
		xOverlay.AddItem(pxGlobalHandler->GetTrackName(i),(int)pxGlobalHandler->GetTrackHash(i));
	}
 	#endif

	//add track names for this level
	for(int i=0;i<Asura_MetaMusic_System::GetNumberOfTracks();i++)
	{
		xOverlay.AddItem(Asura_MetaMusic_System::GetTrackName(i),(int)Asura_MetaMusic_System::GetTrackHash(i));
	}

	//add option for stopping music
	xOverlay.AddItem("<Stop Music>",0);


	//show selection overlay
	if(!xOverlay.DoModal()) return;

	pxAsuraEntity->m_uTrackID = (Asura_Hash_ID)xOverlay.GetSelectionData();
	SetEntityModified();

	//update the music trigger's overlay
	DeleteOverlay();
	CreateOverlay();

}

Tools_Object* Tools_Entity_MetaMusicTrigger_Object::CreateObjectOfThisType() const
{
	return Create(true);	
}

void Tools_Entity_MetaMusicTrigger_Object::Copy(const Tools_Object* pxObject)
{
	Tools_Entity_Activatable_Object::Copy(pxObject);

	const Tools_Entity_MetaMusicTrigger_Object* pxFromObject = (const Tools_Entity_MetaMusicTrigger_Object*) pxObject;


	Asura_ServerEntity_MetaMusicTrigger* pxFromEntity = (Asura_ServerEntity_MetaMusicTrigger*) pxFromObject->Entity;
	Asura_ServerEntity_MetaMusicTrigger* pxToEntity = (Asura_ServerEntity_MetaMusicTrigger*) Entity;

	pxToEntity->m_uTrackID = pxFromEntity->m_uTrackID;
	pxToEntity->m_uFlags = pxFromEntity->m_uFlags;
}


void Tools_Entity_MetaMusicTrigger_Object::HandleTrackRename(Asura_Hash_ID uOldID,Asura_Hash_ID uNewID)
{
	//get a list of all meta music triggers
	List<Tools_Object*> xObjectList;
	Tools_Object_System::ListObjectsOfType(TOT_Entity_MetaMusicTrigger,xObjectList);

	for(LIF<Tools_Object*> xObjectLIF(&xObjectList);!xObjectLIF.done();xObjectLIF.next())
	{
		//check each trigger to see if it refers to the old track ID
		Tools_Entity_MetaMusicTrigger_Object* pxToolsEntity = (Tools_Entity_MetaMusicTrigger_Object*) xObjectLIF();
		Asura_ServerEntity_MetaMusicTrigger* pxAsuraEntity = (Asura_ServerEntity_MetaMusicTrigger*) pxToolsEntity->Entity;

		if(pxAsuraEntity->m_uTrackID == uOldID)
		{
			pxAsuraEntity->m_uTrackID = uNewID;
			pxToolsEntity->SetEntityModified();

			if(pxToolsEntity->IsSelected())
			{
				//update overlay for the selected trigger
				pxToolsEntity->DeleteOverlay();	
				pxToolsEntity->CreateOverlay();	
			}
		}
	}
}

#endif // #ifndef ASURA_SUPPORT_DYNAMIC_MUSIC_SYSTEM

//eof
