//
// Tools_Entity_Object.h
//
// Created by Richard Rice - 01/05/2002
// 
// Copyright (C) 2002 Rebellion, All Rights Reserved.
//


#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef _TOOLS_ENTITY_OBJECT_H_
#define _TOOLS_ENTITY_OBJECT_H_

// Includes.

// Asura
#include "Core/Asura_Hash.h"
#include "Core/Asura_HashedList.h"
#include "Core/Asura_String.h"
#include "Entity/Asura_Entity.h"

// Tools
#include "Tools_Overlays.h"
#include "Misc/list_tem.hpp"
#include "Misc/Tools_MessageLog.h"
#include "Objects/Tools_Object.h"
#include "Objects/Tools_ObjectSystem_Param.h"

// Forward Declarations.

class Asura_Overlay;
struct Tools_Entity_Data;
class Tools_BaseEntity_Object;

// 'Starts active' shared parameter.
// We need the 'starts active' shared parameter to be accessible from other classes.
//TOOLS_DECLARE_SHARED_PARAM(bool, 0xabf43573, Entity_StartsActive);
static const Asura_Hash_ID g_uHash_Entity_StartsActive = 0xabf43573;
extern Tools_ObjectSystem_SharedValue< bool > g_xParam_Entity_StartsActive;

static const Asura_Hash_ID uTOOLS_DIALOGID_ENTITY_PLATFORM_TAB = 0x6fbd6873; //"Platform"

//////////////////////////////////////////////////////////////////////////////
//
// Tools_BaseEntity_Object
// Base class for tools objects representing entities.
//
//////////////////////////////////////////////////////////////////////////////

class Tools_BaseEntity_Object : public Tools_Object
{
	typedef Tools_Object PARENT;

public:
	Tools_BaseEntity_Object(Asura_Entity* entity);
	virtual ~Tools_BaseEntity_Object();

	static void Rename();
	//Attempts to rename to the given name
	bool Rename(const char* szNewName);

	// Rename all selected entities
	static void RenameAll();

	static void EditComment();

    static Tools_BaseEntity_Object* FindEntity(Asura_Guid uGuid)
    {
	    return s_xList.FindEntity(uGuid);
    }
    static const Asura_Hashed_List<Tools_BaseEntity_Object>* GetList()
    {
	    return &s_xList;
    }

	//Allocates and fills in tools data for a new entity.
	static Tools_Entity_Data* CreateToolsData(const char* szTypeName, const char* szInitialName = 0, const char* szNamePrefix = 0);
	static Tools_Entity_Data* CreateToolsDataWithDefaultName(const char* szTypeName, const char* szShortName, Asura_Guid uGuid, const char* szNamePrefix = 0);
	static Tools_Entity_Data* CreateToolsDataWithDuplicateName(const char* szTypeName, const char* szShortName, Asura_Guid uGuid);
    static Tools_Entity_Data* CreateToolsDataWithThisName(const char* szName);

	// Virtual function overrides
	//void UpdatePositionFromAsuraData();
	virtual void UpdatePositionInAsuraData(bool /*updateLinkedObjects*/);
	virtual const char* GetName();
	#ifndef TOOLS_USE_OLD_ENTITY_MANAGER
	virtual const char* GetShortName(){return "ENT";}
	#endif
	virtual const char* GetTypeDescription(){return "Entity";}
	virtual void Destroy();
	virtual void OnSelection();
	virtual void OnDeselection();
	virtual void UpdateWhileSelected();
	virtual Asura_Guid GetGuid() const { return m_uGuid; }
	virtual bool IsEntityObject(){return true;}
	virtual void SetEntityModified();
	virtual bool IsSelectable();
	virtual void PrintInfo(float& x,float& y);

	virtual bool CanBeDestroyed(){return !IsMemberOfTemplateInstance();}
	virtual bool CanBeMoved(){return !IsMemberOfTemplateInstance();}

	virtual bool OwnsThisMessageBlock(unsigned short uMessageBlockID);
	virtual bool GetEventDescForMessageBlock(unsigned short usMessageBlockID,Asura_String& xOutputDesc);

    virtual Asura_String GetBlueprintTypeName() const;
    virtual Asura_String GetBlueprintName()     const;


	//returns true if this is a member of a template instance
	bool IsMemberOfTemplateInstance() const;

	virtual void Copy(const Tools_Object* pxObject);

	virtual bool IsHidden();

    // Other functions
	void ClearEntity()	{ Entity = NULL; }
    bool IsDestroyed()	{ return Entity ? Entity->IsDestroyed() : false; }
	virtual void CreateOverlay();
	void DeleteOverlay();
	virtual void AddOverlayControls(class Asura_Overlay* newOverlay,class Asura_Vector_2& offset);
	virtual void AddSimulationOverlayControls(class Asura_Overlay* newOverlay,class Asura_Vector_2& offset){}


	void UpdateOverlay();
	virtual void UpdateOverlayControls(class Asura_Overlay* pxOverlay){};
	virtual void UpdateSimulationOverlayControls(class Asura_Overlay* pxOverlay){};

	static void EditPlatformsClicked();
	static void CreatePlatformOverlay();
	static void DeletePlatformOverlay();
	virtual void AddPlatformOverlayControls( class Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset );
	
	//renders links for one message block
	void RenderMessageBlockLinks(int messageBlock);

	virtual bool CanBeRenamed(){return true;}

	// for compatability with existing code, some entities still deal with activate/deactivate themselves.
	virtual bool DealsInternallyWithActivate() const { return false; }
	virtual bool HandlesThisMessage( int iMessage );
	//Should return true if this entity can have a message block attached to the specified event
	virtual bool HandlesThisEvent(int iEventID) const;

    // (MH - 18.10.2002) virtual functions to allow entities to add their required files to tools exports. 
    virtual void AddFilesToExportList(List<Asura_String>& nameList) const {}
    virtual void AddFragmentHashesToExportList( List<Asura_Hash_ID>& xExportList ) const { }
    virtual void AddFilesToExportListFromMessage(u_short usMessageID, void* pData, List<Asura_String>& nameList) const {}
	// AW - 09/04/03 - Virtual function to add textures to export list.
	// This is only for those entities that have textures but no mesh (e.g. Glass).
	virtual Asura_String GetTextureNameForExport() 
	{
		Asura_String xRet;
		xRet.MakeBlank();
		return xRet;
	}

#if defined(ASURA_TOOLS)
	//Determines whether AddFilesToExportList will be called for this entity during level export.
	//Normally entities that aren't being exported won't want to include their files.
	virtual bool ShouldAddFilesToExportList() const {return Entity->ShouldIncludeInLevelExport();}
#endif	// defined(ASURA_TOOLS)

	//for updating the client entity (if any) from the server entity
	virtual void UpdateClient(){};

	//sets the appropriate file(s) as modified
	void SetEntityFileModified();

	//returns true if the owning entity file can be saved
	bool CanSaveFile() const;


	virtual bool GetParameter(Asura_Hash_ID uParamID,class Tools_Parameter* pxParam) const;
	virtual bool SetParameter(Asura_Hash_ID uParamID,const class Tools_Parameter* pxParam);

	int GetEntityClass() const {return Entity->GetClassification();}

	//This function is no longer used. RAR 04/10/2006
	void AddEventOverlayControl(Asura_Overlay* pxNewOverlay, Asura_Vector_2& xOffset){};

	//Gets name of secondary entity file used ,or Main if in the main file.
	const char* GetEntityFileName() const;

	virtual void AddEventMessageBlock( int iEventID );
	virtual int RemoveEventMessageBlock(int iEventID);
	static void EditMessages(int iIndex);

	void SetEntityActive(bool bActive);

	//Sets the light sets that this entity is associated with
	virtual void SetLightSets(u_int uLightSets);

	//Returns the light sets that this entity is associated with
	virtual u_int GetLightSets() const;


	virtual Asura_Entity_Server* GetServerEntity() { return m_pxServerEntity; }

protected:
	virtual bool CanAddAnyMoreMessageBlocks( void ) const;

private:
	void CreateSimulationOverlay();

	void GenerateCopiedName(const Tools_Object* pxFromObject);

	//overlay callbacks for events
	static void AddMessageBlock();

	//gets rid of any events that have empty message blocks assigned to them
	void RemoveEmptyMessageBlocks();

	// Updates the 'last modified' timestamp with the current windows username and current date/time
	// If bCreated == true, it also updates the 'created' timestamp
	static void UpdateToolsEntityTimeStamp( Tools_Entity_Data* const pxToolsData, bool bCreated = false );

public:
    // Data
	Asura_Entity* Entity;
	Asura_Guid m_uGuid;

	Asura_Entity_Server* m_pxServerEntity;
	Asura_Entity_Client* m_pxClientEntity;

	bool Changed;

public:

	//static data for overlay
	static Asura_Vector_2 OverlayPosition;
	static Asura_Vector_2 s_xPlatformOverlayPosition;
	static Tools_BaseEntity_Object* ThisEntity;

private:
	

	static bool s_bShowPlatformOverlay;        
    static Asura_Hashed_List<Tools_BaseEntity_Object> s_xList;

};


#ifdef TOOLS_ENTITY_OBJECT_PARENT
// Project needs to supply this file with definition of project specific tools class included in it...
#include "Entity/Tools_Project_Entity_Object.h"
#else
#define TOOLS_ENTITY_OBJECT_PARENT Tools_BaseEntity_Object
#endif

class Tools_Entity_Object: public TOOLS_ENTITY_OBJECT_PARENT
{
	typedef TOOLS_ENTITY_OBJECT_PARENT PARENT;

public:
	Tools_Entity_Object(Asura_Entity* entity);
	virtual ~Tools_Entity_Object();

	//Creates all the tools objects from the asura data
	static void CreateAllObjects();
	//creates just the tools objects for physical entities
	static void CreatePhysicalObjects();

	//create the tools object for one entity
	static void CreateObject(Asura_Entity* pxEntity);
	//For handling project specific entities.Return false if the entity is unknown
	static bool Project_CreateToolsEntityObject(Asura_Entity* entity);
    // Static functions
	static void SetProjectCreateAllObjectsFunction( void (*pfnProjectCreateAllObjects)() )		{ s_pfnProjectCreateAllObjects = pfnProjectCreateAllObjects; }

	//Deletes any entities that don't belong to the current light set when quitting to game.
	//Obviously this must be done after the quicksave , otherwise the entities will be gone for good.
	static void DeleteUnwantedEntitiesForQuitToGame();

	static bool HasCreatedPhysicalObjects(){return s_eCreatedEntityObjects == TOOLS_CREATEDENTITYOBJECTS_PHYSICAL || s_eCreatedEntityObjects == TOOLS_CREATEDENTITYOBJECTS_ALL;}
	static bool HasCreatedAllObjects(){return s_eCreatedEntityObjects == TOOLS_CREATEDENTITYOBJECTS_ALL;}

	//called by Tools_Object_System after deleting all tools objects so that Tools_Entity_Object knows that the objects don't exist any more
	static void PostDeleteAllObjects() {s_eCreatedEntityObjects = TOOLS_CREATEDENTITYOBJECTS_NONE;}

	static Tools_Entity_Object* FindEntity(Asura_Guid uGuid)
    {
	    return s_xList.FindEntity(uGuid);
    }
    static const Asura_Hashed_List<Tools_Entity_Object>* GetList()
    {
	    return &s_xList;
    }

	virtual u_short* AddCustomEvent() { return NULL; }
	virtual bool DeleteCustomEvent(int iIndex) { return false; }
	virtual bool RenameCustomEvent(int iIndex) { return false; }

private:
	static Asura_Hashed_List<Tools_Entity_Object> s_xList;

	static void (*s_pfnProjectCreateAllObjects)();

	enum CreatedEntityObjects
	{
		TOOLS_CREATEDENTITYOBJECTS_NONE,
		TOOLS_CREATEDENTITYOBJECTS_PHYSICAL,
		TOOLS_CREATEDENTITYOBJECTS_ALL,
	};
 
	static CreatedEntityObjects s_eCreatedEntityObjects;

};


//For associating message log entries with entities
class Tools_MessageLogAction_GotoEntity : public Tools_MessageLogAction
{
public :
	Tools_MessageLogAction_GotoEntity(Asura_Guid uGuid){m_uGuid = uGuid;}
	void Execute();

private :
	Asura_Guid m_uGuid;
};

#endif //_TOOLS_ENTITY_OBJECT_H_
