//////////////////////////////////////////////////////////////////////////////
//
// Tools_Entity_Objective.h
//
// Copyright (c) 2007 Rebellion. All rights reserved.
//
// Description:
//
//     Base Asura Objective Tools Object
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef TOOLS_ENTITY_OBJECTIVE_H___
#define TOOLS_ENTITY_OBJECTIVE_H___ 1

#include "Entity/Objective/Asura_ServerEntity_Objective.h"
#include "Entity/Tools_Entity_Object.h"

const Asura_Hash_ID uTOOLS_SHAREDPARAMHASH_OBJECTIVE_PENDING = 0x3f142584; //"Asura_Objective_Pending"
const Asura_Hash_ID uTOOLS_SHAREDPARAMHASH_OBJECTIVE_COMPLETE = 0x6cf512ec; //"Asura_Objective_Complete"
const Asura_Hash_ID uTOOLS_SHAREDPARAMHASH_OBJECTIVE_FAILED = 0x9e2946b0; //"Asura_Objective_Failed"
const Asura_Hash_ID uTOOLS_SHAREDPARAMHASH_OBJECTIVE_SKIPPED = 0xe7c3031d; //"Asura_Objective_Skipped"

// state is expressed as a set of shared param booleans
TOOLS_DECLARE_SHARED_PARAM( bool, uTOOLS_SHAREDPARAMHASH_OBJECTIVE_PENDING,			Objective_Pending )
TOOLS_DECLARE_SHARED_PARAM( bool, uTOOLS_SHAREDPARAMHASH_OBJECTIVE_COMPLETE,		Objective_Complete )
TOOLS_DECLARE_SHARED_PARAM( bool, uTOOLS_SHAREDPARAMHASH_OBJECTIVE_FAILED,			Objective_Failed )
TOOLS_DECLARE_SHARED_PARAM( bool, uTOOLS_SHAREDPARAMHASH_OBJECTIVE_SKIPPED,			Objective_Skipped )

class Tools_Entity_Objective : public Tools_Entity_Object
{
public :
	Tools_Entity_Objective(Asura_Entity* pxEntity);
	~Tools_Entity_Objective();
	static Tools_Object* Create(Asura_Entity* pxEntity);
	static void Create();
	
	const char* GetTypeDescription(){return "Objective";}
	virtual bool HandlesThisMessage(int iMessageID);
	virtual bool HandlesThisEvent(int iEventID) const;

	static const char* GetNamePrefix() { return "Obj_"; }
	virtual const char* GetMyNamePrefix() const { return Tools_Entity_Objective::GetNamePrefix(); }

	void AddOverlayControls(class Asura_Overlay* pxOverlay,class Asura_Vector_2& xOffset);

	virtual void UpdateWhileSelected(void);	
	virtual void PrintInfo(float& fX,float& fY);

	virtual bool GetParameter( Asura_Hash_ID uParamID, Tools_Parameter* pxParam ) const;
	virtual bool SetParameter( Asura_Hash_ID uParamID, const Tools_Parameter* pxParam );
    virtual bool AddSharedParamDialogToOverlay( Asura_Overlay* pxOverlay, const Asura_Vector_2& xOffset, Asura_Hash_ID uParamID );

	virtual void Copy(const Tools_Object* pxObject);
	virtual bool CanBeCopied() const {return Type == TOT_Entity_Objective;}
	virtual Tools_Object* CreateObjectOfThisType() const;
protected:

	void GenerateTextList();

	Asura_ServerEntity_Objective* m_pxObjective;//keep a pointer to entity so we don't have cast or search the whole time

	int m_iTextIndex;			// current text expressed as array index
	const char** m_pszTextList;	// array of possible text entries
	int m_iTextListLength;		// size of preceding array
private:
	typedef Tools_Entity_Object PARENT;
};

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

#endif // #ifndef TOOLS_ENTITY_OBJECTIVE_H___
