//
// Tools_Entity_PFX_Effect.cpp
//
// Created by KOR - 05/03/2004 11:15:01
//
// Copyright (C) 2004 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"


// This file's header.
#include "Entity/Tools_Entity_PFX_Effect.h"

// Asura includes
#include "Camera/Asura_Camera.h"
#include "Core/Asura_Material_System.h"
#include "Core/Asura_Spline.h"
#include "Core/Asura_TextureManagement.h"
#include "Core/Asura_UserInput.h"
#include "Entity/Asura_Entity_Event.h"
#include "Entity/Asura_StaticMessage_System.h"
#include "Entity/ParticleGenerator/Asura_ServerEntity_PFX_Effect.h"
#include "Entity/ParticleGenerator/Asura_ClientEntity_PFX_Effect.h"
#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Overlay.h"
#include "Interface/Asura_Overlay_System.h"
#include "Mesh/Asura_Mesh_System.h"
#include "Rendering/Asura_Render.h"
#include "SpecialFX/PFX/Asura_PFX_Templates.h"

// Tools includes
#include "Cutscene/Tools_Mode_Cutscene.h"
#include "Entity/Tools_Entity_Data.h"
#include "Interface/Tools_ContextMenu_System.h"
#include "Load&Save/Tools_SaveAsura.h"
#include "Misc/Tools_Timed_Messages.h"
#include "Objects/Tools_Object_System.h"
#include "Objects/Tools_PFX_Vector_Object.h"
#include "Entity/Locations/Spline/Tools_Object_PFXSplinePoint.h"
#include "Particles/Tools_Mode_PFX.h"
#include "Particles/Tools_PFX_SourceVolume_Object.h"
#include "Particles/Tools_PFX_Dialog.h"
#include "Texturing/Tools_Texturing_Texture.h"
#include "Tools_Overlays.h"

//STATIC MEMBER VARIABLES///////////////////////////

Tools_Entity_PFX_Effect*		Tools_Entity_PFX_Effect::s_pxCurrentObject = NULL;
List<Tools_Entity_PFX_Effect*>	Tools_Entity_PFX_Effect::s_xPFXEffectList;

// Overlay members
Tools_PFXSelect_Dialog*		Tools_Entity_PFX_Effect::s_pxEffectTempDialog = NULL;
bool							Tools_Entity_PFX_Effect::s_bEffectTempChanged = false;
Asura_Hash_ID					Tools_Entity_PFX_Effect::s_uEffectTempHash = ASURA_HASH_ID_UNSET;
bool							Tools_Entity_PFX_Effect::s_bCollisionPlaneRelative = true;
bool							Tools_Entity_PFX_Effect::s_bRenderEffectBBs = false;
Asura_Hash_ID					Tools_Entity_PFX_Effect::s_uSourceTempHash = ASURA_HASH_ID_UNSET;
// KOR - 18/05/2006 - to ensure that editing special entities e.g. gravity don't break template editing
bool							Tools_Entity_PFX_Effect::s_bTempEntityEdit = false;
Asura_Tabs_Dialog*				Tools_Entity_PFX_Effect::s_pxTabs = NULL;
bool							Tools_Entity_PFX_Effect::s_bNeedToRefreshOverlay = false;

Asura_Vector_2					Tools_Entity_PFX_Effect::s_xEffectTabOverlayOffset;
Asura_Vector_2					Tools_Entity_PFX_Effect::s_xParticleTabOverlayOffset;
Asura_Vector_2					Tools_Entity_PFX_Effect::s_xSourceTabOverlayOffset;

//GLOBALS///////////////////////////////////////////
static int g_iNumPFXEffects;
// cached tools shape meshes
static Asura_Mesh* g_pxToolsShapeMesh;
static Asura_Mesh* g_pxToolsShapeMeshSmall;

//MEMBER FUNCTIONS//////////////////////////////////

Tools_Entity_PFX_Effect::Tools_Entity_PFX_Effect(Asura_Entity* xEntity)
: Tools_Entity_Object(xEntity)
, m_pxServerEntity(NULL)
, m_pxClientEntity(NULL)
, m_pxCollisionPlane(NULL)
, m_pxSourceVolumeEdit(NULL)
, m_bRestartEffect(false)
{
	// first one created stores the ptrs to cached tools shapes
	if (!s_xPFXEffectList.size())
	{
		g_pxToolsShapeMesh = Asura_Mesh_System::GetMesh("ToolsShape_Particle");
		if (!g_pxToolsShapeMesh)
		{
			g_pxToolsShapeMesh = Asura_Mesh_System::GetMesh("ToolsShape_CutsceneTrackPoint");
		}
		g_pxToolsShapeMeshSmall = Asura_Mesh_System::GetMesh("ToolsShape_ParticleSmall");
		if (!g_pxToolsShapeMeshSmall)
		{
			g_pxToolsShapeMeshSmall = Asura_Mesh_System::GetMesh("ToolsShape_CutsceneTrackPoint");
		}
	}

	s_xPFXEffectList.add_entry(this);

	m_pxServerEntity = (Asura_ServerEntity_PFX_Effect*) xEntity;
	Asura_Assert(m_pxServerEntity, "Unable to retrieve server entity");
	if(m_pxServerEntity)
	{
		//Set Tools entity properties (i.e mesh and flags)
		Type = TOT_Entity_PFX_Effect;
		Position = m_pxServerEntity->m_xPosition;
		Orientation = m_pxServerEntity->m_xOrientation;
		m_bIgnoreModuleLinking = m_pxServerEntity->m_xPFXEffectData.m_bIgnoreModuleLinking;
		m_bSkyboxEffect = m_pxServerEntity->m_xPFXEffectData.m_bSkyboxEffect;
		if(m_bSkyboxEffect)
		{
			m_xSkyboxOffset = m_pxServerEntity->m_xPosition;
		}
		else
		{
			m_xSkyboxOffset = Asura_Vector_3(0.f, 0.f, 0.f);
		}
		m_bWaterDistorterEffect = m_pxServerEntity->m_xPFXEffectData.m_bWaterDistorterEffect;
		m_xSourceVolume = m_pxServerEntity->m_xPFXEffectData.m_xSourceVolume;
		m_fMinRetriggerTime = m_pxServerEntity->m_xPFXEffectData.m_fMinRetriggerTime;
		m_fMaxRetriggerTime = m_pxServerEntity->m_xPFXEffectData.m_fMaxRetriggerTime;

		UpdateShape();

		//Create a client entity based on the server entity
		
		//Check to see if the client entity already exists
		m_pxClientEntity = static_cast<Asura_ClientEntity_PFX_Effect*> (Asura_ClientEntity_PFX_Effect::FindEntity(m_pxServerEntity->GetGuid()));
		if( !m_pxClientEntity )
		{
			//If there isn't already a client, make a new one.
			Asura_Entity_System::SetNextClientGuid(m_pxServerEntity->GetGuid());
			m_pxClientEntity =  new Asura_ClientEntity_PFX_Effect(Position, Orientation, m_pxServerEntity->m_xPFXEffectData.m_uEffectTemplateHash);
			m_pxClientEntity->m_xPFXEffectData.m_xCollisionDirection = m_pxServerEntity->m_xPFXEffectData.m_xCollisionDirection;
			m_pxClientEntity->m_xPFXEffectData.m_xCollisionPosition = m_pxServerEntity->m_xPFXEffectData.m_xCollisionPosition;
		}
	}
	m_bGameEntitiesNeedUpdating = false;
	m_bMinModified = false;
	m_bMaxModified = false;

	CreateSplinePoints();
}

Tools_Entity_PFX_Effect::~Tools_Entity_PFX_Effect()
{
	//May well require other shutdown code in here
	if (s_pxCurrentObject == this)
	{
		s_pxCurrentObject = NULL;
	}
	if(m_pxClientEntity && m_pxServerEntity)
	{
		//reset the starts active state back to the initial setting
		//RAR 29/06/2006 Used to do both active and starts active , but I now deal with the currently active
		//state when exiting tools. Not 100% sure that the starts active state needs to be done here anyway.
		m_pxClientEntity->m_bStartsActive = m_pxServerEntity->IsActive();
	}
	s_xPFXEffectList.delete_entry(this);

	// last one clears the cached pointers.
	if (!s_xPFXEffectList.size())
	{
		g_pxToolsShapeMesh = NULL;
		g_pxToolsShapeMeshSmall = NULL;
	}
}

//////////////////////////////////////
// Static Create function			//
//									//
//////////////////////////////////////


void Tools_Entity_PFX_Effect::Create()
{
	Tools_Entity_PFX_Effect* pxObject = Create(false);
	if(pxObject) Tools_Object_System::SelectObject(pxObject);
}

Tools_Entity_PFX_Effect* Tools_Entity_PFX_Effect::Create(bool bGenerateName)
{
	if(s_bTempEntityEdit)
	{
		//Prevent creation of new PFX while in TempEntityEdit to prevent issues with not being allowed to select the new entity.
		Tools_Timed_Message_System::AddMessage(0xffff00,"Creation of new PFX entities disabled while editing PFX directions or source volumes");
		return 0;
	}

	//get a name , and the position data
	Asura_Guid uGuid = Asura_Entity_System::GetNextToolsCreatedGuid();
	Tools_Entity_Data* pxToolsData = 0;
	if(bGenerateName)
	{
		pxToolsData = Tools_Entity_Object::CreateToolsDataWithDefaultName("PFX Effect","PFX Effect",uGuid,GetNamePrefix());
	}
	else
	{
		pxToolsData = Tools_Entity_Object::CreateToolsData("PFX Effect",0,GetNamePrefix());
	}
	if(!pxToolsData) return 0;

	//Get orientation and position of new object from the camera
	const Asura_Vector_3* pxPosition = Asura_Camera::GetPosition();
	const Asura_Matrix_3x3* pxOrientation = Asura_Camera::GetOrientation();

	//Create a default particle generator server entity
	Asura_ServerEntity_PFX_Effect* pxServerEffect = new Asura_ServerEntity_PFX_Effect(uGuid, *pxPosition, *pxOrientation);
	Asura_Entity_System::RegisterToolsGuid(uGuid);
	Asura_Assert(pxServerEffect, "Error: Unable to create effect object.\nPlease abort");

	//Attach tools data to the server entity
	pxServerEffect->ToolsData = pxToolsData;

	//Create the corresponding tools entity
	Tools_Entity_PFX_Effect* pxEffect = new Tools_Entity_PFX_Effect(pxServerEffect);

	Tools_SaveAsura::SetEntityModified();

	return pxEffect;
}


Tools_Object* Tools_Entity_PFX_Effect::CreateObjectOfThisType() const
{
	return Create(true);
}

void Tools_Entity_PFX_Effect::Copy(const Tools_Object* pxObject)
{
	Tools_Entity_Object::Copy(pxObject);

	const Tools_Entity_PFX_Effect* pxFrom = (const Tools_Entity_PFX_Effect*) pxObject;

	//copy from the other tools object
	m_bIgnoreModuleLinking	= pxFrom->m_bIgnoreModuleLinking;
	m_bSkyboxEffect			= pxFrom->m_bSkyboxEffect;
	m_bWaterDistorterEffect = pxFrom->m_bWaterDistorterEffect;
	m_xSourceVolume			= pxFrom->m_xSourceVolume;
	m_fMinRetriggerTime		= pxFrom->m_fMinRetriggerTime;
	m_fMaxRetriggerTime		= pxFrom->m_fMaxRetriggerTime;

	//copy between server entities
	m_pxServerEntity->m_xPFXEffectData = pxFrom->m_pxServerEntity->m_xPFXEffectData;

	//and client entities
	m_pxClientEntity->StopEffect(true);
	m_pxClientEntity->m_xPFXEffectData = pxFrom->m_pxClientEntity->m_xPFXEffectData;
	m_pxClientEntity->SetWaterDistorterEffect(m_bWaterDistorterEffect);
}


void Tools_Entity_PFX_Effect::UpdateWhileSelected()
{
	//Get a pointer to the currently selected object	
	s_pxCurrentObject = this;

	if(s_uEffectTempHash != m_pxServerEntity->m_xPFXEffectData.m_uEffectTemplateHash)
	{
		Changed = true;
		SetEntityModified();
		m_pxServerEntity->m_xPFXEffectData.m_uEffectTemplateHash = s_uEffectTempHash;
		m_pxClientEntity->m_xPFXEffectData.m_uEffectTemplateHash = s_uEffectTempHash;
		if(m_pxClientEntity->IsEntityActive())
		{
			m_pxClientEntity->RestartEffect();
		}
		// Don't refresh immediately otherwise it breaks scrolling through templates.
		s_bNeedToRefreshOverlay = true;
	}

	// PAZ - 24/10/2007 - select all (objects mode) caused a crash, s_pxTabs was NULL
	if( s_pxTabs )
	{
		// KOR - 22/05/2006 - check whether the current source template has changed.
		const u_int uCurrentTab = s_pxTabs->GetSelected();
		u_int uCurrentSource = Tools_Mode_PFX::s_iEffectCurrentSourceNum;


		if(uCurrentTab != 0 && s_bNeedToRefreshOverlay)
		{
			// RAR 22/02/2008 - Reset to editing the first source when the effect template changes
			uCurrentSource = 0;
			Tools_Mode_PFX::s_iEffectCurrentSourceNum = 0;
			Tools_Mode_PFX::s_iEffectSourceNum = 0;
			

			// KOR - 17/05/2006 - refresh the overlay with our new effect template.
			CloseOverlays();
			CreateOverlay();
			s_pxTabs->SelectTab(uCurrentTab);
			s_bNeedToRefreshOverlay = false;
		}

		if(uCurrentTab != 0)
			//	if(uCurrentTab == 2 || uCurrentTab == 3)
		{
			// We're on the particle or source tab. Make sure we're editing the correct template
			Asura_Hash_ID uSourceTempHash = 0;
			if(Tools_Mode_PFX::s_iEffectCurrentSourceNum != -1 && Tools_Mode_PFX::s_pxCurrentEffectTemp)
			{
				uSourceTempHash = Tools_Mode_PFX::s_pxCurrentEffectTemp->m_pxSources[Tools_Mode_PFX::s_iEffectCurrentSourceNum].m_uTemplate;
			}
			// Template has changed.
			if(s_uSourceTempHash != uSourceTempHash)
			{
				CreateSourceTab(uCurrentSource);
				CreateParticleTab();

				// Slightly hacky - set the recreated overlay up to how we had it before.
				s_pxTabs->SelectTab(uCurrentTab);
				Tools_Mode_PFX::s_iEffectSourceNum = uCurrentSource;
			}
		}
	}

	if(!Asura_Overlay_System::OverlaysHaveKeyboardFocus())
	{
		if(!Asura_UserInput::KeyPressed(ASURA_KEYID_CTRL) && 
		   !Asura_UserInput::KeyPressed(ASURA_KEYID_ALT))
		{
			if(Asura_UserInput::KeyPressed(ASURA_KEYID_SHIFT))
			{
				if(Asura_UserInput::DebouncedKeyPressed('1'))
				{
					TestAllEffects();
				}
				if(Asura_UserInput::DebouncedKeyPressed('2'))
				{
					StopAllEffectsFast();
				}
				if(Asura_UserInput::DebouncedKeyPressed('3'))
				{
					StopAllEffectsSlow();
				}
			}
			else
			{
				if(Asura_UserInput::DebouncedKeyPressed('1'))
				{
					TestSelectedEffects();
				}
				if(Asura_UserInput::DebouncedKeyPressed('2'))
				{
					StopSelectedFast();
				}
				if(Asura_UserInput::DebouncedKeyPressed('3'))
				{
					StopSelectedSlow();
				}
			}
		
		}
	}
	Tools_Entity_Object::UpdateWhileSelected();
}


void Tools_Entity_PFX_Effect::Update()
{
	// Has the template changed?
	if(s_bEffectTempChanged)
	{
		if(s_uEffectTempHash != ASURA_HASH_ID_UNSET)
		{
			// Check the template is complete
			const Asura_PFX_Effect_Template* pxTemplate = Asura_PFX_System::GetEffectTempByHash(s_uEffectTempHash);
			if(pxTemplate)
			{
				for(u_int uSource = 0; uSource < pxTemplate->m_uNumSources; ++uSource)
				{
					const Asura_PFX_Source_Template* pxSourceTemp = Asura_PFX_System::GetSourceTempByHash(pxTemplate->m_pxSources[uSource].m_uTemplate);
					if(pxSourceTemp)
					{
						if(!Asura_PFX_System::GetParticleTempByHash(pxSourceTemp->m_uParticleTemplateHash))
						{
							const Asura_String* pxSourceName = Asura_PFX_System::GetSourceTempNameByHash(pxTemplate->m_pxSources[uSource].m_uTemplate);
							Tools_Timed_Message_System::AddMessage(0xffff00,"Warning: source %u of chosen effect template is using source template %s, which has no particle template", uSource, pxSourceName->GetString());
						}
					}
					else
					{
						Tools_Timed_Message_System::AddMessage(0xffff00,"Warning: source %u of chosen effect template has no source template", uSource);
					}
				}
			}
			else
			{
				Tools_Timed_Message_System::AddMessage(0xff0000,"Error: no template selected");
			}
		}

		s_bEffectTempChanged = false;
		Tools_Mode_PFX::s_uChooseTemplateHash = s_uEffectTempHash;
	}

	//Update Tools
	Tools_Entity_Object::Update();

	m_pxClientEntity->m_bStartsActive = m_pxServerEntity->IsActive();

	//make sure mins are less than maxs
	if(m_bMinModified)
	{
		m_fMaxRetriggerTime = Asura_Maths::Max(m_fMinRetriggerTime,m_fMaxRetriggerTime);
		m_bMinModified = false;
		m_bGameEntitiesNeedUpdating = true;
	}
	if(m_bMaxModified)
	{
		m_fMinRetriggerTime = Asura_Maths::Min(m_fMinRetriggerTime,m_fMaxRetriggerTime);
		m_bMaxModified = false;
		m_bGameEntitiesNeedUpdating = true;
	}

	Asura_Entity_PFX_Effect_Data& xServerData = m_pxServerEntity->m_xPFXEffectData;
	Asura_Entity_PFX_Effect_Data& xClientData = m_pxClientEntity->m_xPFXEffectData;
	if(m_bGameEntitiesNeedUpdating)
	{
		Changed = true;
		SetEntityModified();

		m_bGameEntitiesNeedUpdating = false;
		m_xSkyboxOffset = Position - *Asura_Camera::GetPosition();

		Asura_Vector_3 xDelta = -m_pxServerEntity->m_xPosition;
		UpdateEffectPosition();
		xDelta += m_pxServerEntity->m_xPosition;
		//Update server entity params
		m_pxServerEntity->m_xOrientation			= Orientation;
		xServerData.m_bIgnoreModuleLinking	= m_bIgnoreModuleLinking;
		xServerData.m_bSkyboxEffect			= m_bSkyboxEffect;
		xServerData.m_bWaterDistorterEffect   = m_bWaterDistorterEffect;
		xServerData.m_fMinRetriggerTime		= m_fMinRetriggerTime;
		xServerData.m_fMaxRetriggerTime		= m_fMaxRetriggerTime;

		//Update client entity params
		m_pxClientEntity->m_xCurrentOrientation		= Orientation;
		m_pxClientEntity->SetIgnoreModuleLinking(m_bIgnoreModuleLinking);
		m_pxClientEntity->SetSkyboxEffect(m_bSkyboxEffect);
		m_pxClientEntity->SetWaterDistorterEffect(m_bWaterDistorterEffect);

		xClientData.m_fMinRetriggerTime		= m_fMinRetriggerTime;
		xClientData.m_fMaxRetriggerTime		= m_fMaxRetriggerTime;

		if(s_bCollisionPlaneRelative)
		{
			xServerData.m_xCollisionPosition += xDelta;
			xClientData.m_xCollisionPosition += xDelta;
		}

		if(xServerData.m_bUsesSpline)
		{
			// Lock the first spline point to the emitter
			for (CLIF <Tools_Object_PFXSplinePoint*> xIt(&m_xSplinePointList); !xIt.done(); xIt.next())
			{
				Tools_Object_PFXSplinePoint* pxPoint = xIt();
				if(pxPoint->GetIndex() == 1)
				{
					pxPoint->Position = Position;
					pxPoint->UpdatePositionInAsuraData(false);
				}
			}
		}
	}
	// Update collision plane
	if(m_pxCollisionPlane)
	{
		if(m_pxCollisionPlane->m_bGameEntitiesNeedUpdating)
		{
			// Vector has moved,  so update collision plane
			// Server (calculate from object)
			xServerData.m_xCollisionDirection.x = m_pxCollisionPlane->Orientation.e13;
			xServerData.m_xCollisionDirection.y = m_pxCollisionPlane->Orientation.e23;
			xServerData.m_xCollisionDirection.z = m_pxCollisionPlane->Orientation.e33;
			xServerData.m_xCollisionDirection.Normalise();
			xServerData.m_xCollisionPosition = m_pxCollisionPlane->Position;
			// Client (copy from server)
			xClientData.m_xCollisionDirection = xServerData.m_xCollisionDirection;
			xClientData.m_xCollisionPosition = xServerData.m_xCollisionPosition;
			// Reset flag
			m_pxCollisionPlane->m_bGameEntitiesNeedUpdating = false;
			// Set flag so game entities get updated
			m_bGameEntitiesNeedUpdating = true;
		}

		//Remove object if neccessary...
		if(m_pxCollisionPlane->m_bReadyToDelete)
		{
			delete m_pxCollisionPlane;
			m_pxCollisionPlane = NULL;
			SelectThisObject();
		}
	}

	//update source volume
	if(m_pxSourceVolumeEdit)
	{
		if(m_pxSourceVolumeEdit->m_bGameEntitiesNeedUpdating)
		{
			m_xSourceVolume = m_pxSourceVolumeEdit->m_xVolume;
			Position = m_pxSourceVolumeEdit->Position;
			Orientation = m_pxSourceVolumeEdit->Orientation;

			UpdateEffectPosition();
			//update server
			xServerData.m_xSourceVolume = m_xSourceVolume;
			m_pxServerEntity->m_xOrientation		= Orientation;

			//and client
			xClientData.m_xSourceVolume = m_xSourceVolume;
			m_pxClientEntity->m_xCurrentOrientation	= Orientation;
			
			// Reset flag
			m_pxSourceVolumeEdit->m_bGameEntitiesNeedUpdating = false;

			//need to restart test generator if it was playing
			if(m_pxClientEntity->IsEntityActive())
			{
				m_pxClientEntity->RestartEffect();
			}
		}
		//Remove object if neccessary...
		if(m_pxSourceVolumeEdit->m_bReadyToDelete)
		{
			delete m_pxSourceVolumeEdit;
			m_pxSourceVolumeEdit = NULL;
			SelectThisObject();
		}
	}

	//Don't update client if in cutscene mode and playing a cutscene , to prevent the position
	//from being updated twice per frame (here and in the cutscene attachment code). RAR 14/02/2005
	if(!Tools_Mode_Cutscene::IsPlayingCutscene())
	{
		m_pxClientEntity->UpdateEffectProperties();
	}

	//Update selected status
	m_pxClientEntity->SetSelectedInTools(IsGroupSelected());

	// Update edit overlays
	Tools_Mode_PFX::UpdateEditParticleTemplate();
	Tools_Mode_PFX::UpdateEditSourceTemplate();
	Tools_Mode_PFX::UpdateEditEffectTemplate();

	//update shape used by tools object
	UpdateShape();

	if(m_bRestartEffect)
	{
		if(m_pxClientEntity) m_pxClientEntity->RestartEffect();
		m_bRestartEffect = false;	
	}

	//stop preview if the entity isn't flagged for the current platform
	if(!Entity->ToolsData->ShouldExportToPlatform(Tools_Mode_PFX::GetPreviewPlatform()))
	{
		m_pxClientEntity->StopEffect(true);
	}

}


void Tools_Entity_PFX_Effect::UpdatePositionInAsuraData(bool bUpdateLinkedObjects)
{
	Tools_Entity_Object::UpdatePositionInAsuraData(bUpdateLinkedObjects);
	
	//Update position and orientation of tools object. Set flag so we know
	//to update the server and client entities
	m_bGameEntitiesNeedUpdating = true;
}

void Tools_Entity_PFX_Effect::OnSelection(void)
{
	Tools_Entity_Object::OnSelection();
}

void Tools_Entity_PFX_Effect::OnDeselection(void)
{
	// Over-ride default behaviour - don't delete overlay
	Tools_Object_System::ClearCommentText(this);

	if(!s_bTempEntityEdit)
	{
		DeleteOverlay();
		CloseOverlays();
	}
	else
	{
		Asura_Overlay* pxOverlay = Asura_Overlay_System::GetOverlay(GetOverlayID());
		if(pxOverlay)
		{
			pxOverlay->SetHideFlag(ASURA_OVERLAY_HIDEFLAG_MODE, true);
		}
	}


	DeletePlatformOverlay();
	ThisEntity = 0;
}

void Tools_Entity_PFX_Effect::CreateOverlay()
{
	// Only create if it doesn't exist or we're selecting a different entity
	// This is so that when switching to, say, gravity entity and back, we keep the same overlay.
	if(!Asura_Overlay_System::DoesOverlayExist(GetOverlayID()) || this != s_pxCurrentObject)
	{
		CloseOverlays();
		Tools_Entity_Object::CreateOverlay();
	}
	else
	{
		Asura_Overlay* pxOverlay = Asura_Overlay_System::GetOverlay(GetOverlayID());
		if(pxOverlay)
		{
			pxOverlay->SetHideFlag(ASURA_OVERLAY_HIDEFLAG_MODE, false);
		}
	}
}

void Tools_Entity_PFX_Effect::CloseOverlays()
{
	//Close all overlays
	for(LIF<Tools_Entity_PFX_Effect*> xEffectLIF(&s_xPFXEffectList); !xEffectLIF.done(); xEffectLIF.next())
	{
		Asura_Overlay_System::CloseOverlays(xEffectLIF()->GetOverlayID());
	}

	//	Tools_Mode_PFX::CloseChooseTemplate();
	Tools_Mode_PFX::CloseEditParticleTemplate();
	Tools_Mode_PFX::CloseEditSourceTemplate();
	Tools_Mode_PFX::CloseEditEffectTemplate();

	s_pxEffectTempDialog = 0;
}

void Tools_Entity_PFX_Effect::AddOverlayControls(class Asura_Overlay* pxOverlay, class Asura_Vector_2& xOverlayOffset)
{
	s_pxCurrentObject = this;
	
	pxOverlay->m_uID = TOOLS_OVERLAY_PFX_EDIT_EFFECT;
	Asura_Vector_2 xOffset = xOverlayOffset;
	Asura_Dialog* pxDialog;

	Tools_Entity_Object::AddOverlayControls(pxOverlay, xOffset);

	const char* aszTabNames[] = { "Emitter", "Effect", "Source", "Particle" };
	pxDialog = s_pxTabs = new Asura_Tabs_Dialog(xOffset, aszTabNames, 4, 30);
	pxOverlay->AddDialog(pxDialog);
	s_pxTabs->SelectTab(0);
	xOffset.y = pxDialog->m_xMaxCoord.y + 0.5f;

	// Save y position
	const float fTabsPosition = xOffset.y;

	pxDialog = new Asura_Function_Dialog	("Rename using template name", xOffset, &SetCurrentNameFromTemplate);
	s_pxTabs->AddDialog(pxDialog);

	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_Dialog("-------------------------------", xOffset);
	s_pxTabs->AddDialog(pxDialog);

	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_Dialog("Effect attributes", xOffset);
	s_pxTabs->AddDialog(pxDialog);

	xOffset.y += Asura_Dialog::LargeSpacing();
	pxDialog = new Asura_Dialog("Template", xOffset);
	s_pxTabs->AddDialog(pxDialog);

	xOffset.x = pxDialog->m_xMaxCoord.x + 1.f;
	pxDialog = new Asura_Function_Dialog("New", xOffset, &CreateNewTemplate);
	pxDialog->SetHelpText("Create a new template and use it for this emitter");
	s_pxTabs->AddDialog(pxDialog);

	xOffset.x = pxDialog->m_xMaxCoord.x + 1.f;
	pxDialog = new Asura_Function_Dialog("Copy To", xOffset, &CopyTemplate);
	pxDialog->SetHelpText("Copy the currently selected template and use it for this emitter");
	s_pxTabs->AddDialog(pxDialog);

	xOffset.x = pxDialog->m_xMaxCoord.x + 1.f;
	pxDialog = new Asura_Function_Dialog("Rename To", xOffset, &RenameTemplate);
	pxDialog->SetHelpText("Rename the currently selected template");
	s_pxTabs->AddDialog(pxDialog);
	xOffset.x = 1.f;

	// Choose current template
	s_bEffectTempChanged = false;
	s_uEffectTempHash = ASURA_HASH_ID_UNSET;
	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxDialog = s_pxEffectTempDialog = new Tools_PFXSelect_Dialog(Tools_PFXSelect_Dialog::MODE_EFFECTS, xOffset, &s_uEffectTempHash, &s_bEffectTempChanged);
	if(m_pxServerEntity)
	{
		s_pxEffectTempDialog->SetTemplateHash(m_pxServerEntity->m_xPFXEffectData.m_uEffectTemplateHash);
		s_pxTabs->AddDialog(pxDialog);
	}
	//make sure PFX's mode of the current template stays in sync. RAR  25/06/2007
	Tools_Mode_PFX::s_uChooseTemplateHash = s_uEffectTempHash;
	

	xOffset.y += Asura_Dialog::LargeSpacing();
	pxDialog = new Asura_Function_Dialog("Edit particle collision plane", xOffset, &EditCollisionPlane);
	s_pxTabs->AddDialog(pxDialog);

	xOffset.x += pxDialog->m_xMaxCoord.x;
	s_bCollisionPlaneRelative = true;
	pxDialog = new Asura_Tickbox_Dialog("Local position", xOffset, &s_bCollisionPlaneRelative, NULL);
	pxDialog->SetHelpText("Moves collision plane with effect");
	s_pxTabs->AddDialog(pxDialog);
	xOffset.x = 1.f;

	xOffset.y += Asura_Dialog::DefaultSpacing();		
	pxDialog = new Asura_Function_Dialog("Edit source volume", xOffset, EditSourceVolume);
	pxDialog->SetHelpText("This allows the source volume in which particles get generated to be overriden for this entity.\nIf the volume is set as anything other than point , then it will be used for all particle sources.");
	s_pxTabs->AddDialog(pxDialog);

	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_Dialog("Spline: ", xOffset);
	s_pxTabs->AddDialog(pxDialog);

	xOffset.x = pxDialog->m_xMaxCoord.x + 1.f;
	pxDialog = new Asura_Function_Dialog("Add", xOffset, AddSpline, NULL, CanAddSpline);
	pxDialog->SetHelpText("Create a spline for the particles to move along");
	s_pxTabs->AddDialog(pxDialog);

	xOffset.x = pxDialog->m_xMaxCoord.x + 1.f;
	pxDialog = new Asura_Function_Dialog("Remove", xOffset, RemoveSpline, NULL, CanRemoveSpline);
	pxDialog->SetHelpText("Removes the spline");
	s_pxTabs->AddDialog(pxDialog);

	xOffset.x = 1.f;



	xOffset.y += Asura_Dialog::DefaultSpacing();
#if defined (ASURA_USE_NEW_VISIBILITY_SYSTEM )
	pxDialog = new Asura_Tickbox_Dialog("Ignore Visibility System", xOffset, &m_bIgnoreModuleLinking, &m_bGameEntitiesNeedUpdating);
#else
	pxDialog = new Asura_Tickbox_Dialog("Ignore Module Linking", xOffset, &m_bIgnoreModuleLinking, &m_bGameEntitiesNeedUpdating);
#endif
	s_pxTabs->AddDialog(pxDialog);

	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_Tickbox_Dialog("Skybox effect", xOffset, &m_bSkyboxEffect, &m_bGameEntitiesNeedUpdating);
	s_pxTabs->AddDialog(pxDialog);

#ifdef ASURA_TARGET_SM3
	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_Tickbox_Dialog("Water Distorter effect", xOffset, &m_bWaterDistorterEffect, &m_bGameEntitiesNeedUpdating);
	s_pxTabs->AddDialog(pxDialog);
#endif

	//retrigger time
	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_Numerical_Dialog("Retrigger Min", xOffset, &m_fMinRetriggerTime, 0.f, 999.0f, .1f, "%.1f", &m_bMinModified);
	pxDialog->SetHelpText("If set to a non-zero value then the effect will randomly retrigger based on these times whenever it finishes (while active).");
	s_pxTabs->AddDialog(pxDialog);

	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_Numerical_Dialog("Retrigger Max", xOffset, &m_fMaxRetriggerTime, 0.f, 999.0f, .1f, "%.1f", &m_bMaxModified);
	pxDialog->SetHelpText("If set to a non-zero value then the effect will randomly retrigger based on these times whenever it finishes (while active).");
	s_pxTabs->AddDialog(pxDialog);


	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_Dialog("-------------------------------", xOffset);
	s_pxTabs->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	xOffset.y += Asura_Dialog::LargeSpacing();
	pxDialog = new Asura_Tickbox_Dialog("Render visibility BBs for selected", xOffset, &s_bRenderEffectBBs, NULL);
	s_pxTabs->AddDialog(pxDialog);

	// Next tab: effect
	s_xEffectTabOverlayOffset[0] = xOffset[0];
	s_xEffectTabOverlayOffset[1] = fTabsPosition;

	CreateEffectTab();

	// Next tab: source
	s_xSourceTabOverlayOffset[0] = xOffset[0];
	s_xSourceTabOverlayOffset[1] = fTabsPosition;

	CreateSourceTab(0);

	// Next tab: particle
	s_xParticleTabOverlayOffset[0] = xOffset[0];
	s_xParticleTabOverlayOffset[1] = fTabsPosition;

	CreateParticleTab();

	// End of tabs
	s_pxTabs->SelectTab(0);
	xOffset.y = s_pxTabs->GetMaxCoords().y;

	//	pxDialog = new Asura_Dialog("Effect testing", xOffset);
	//	pxOverlay->AddDialog(pxDialog);

	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_Dialog("Selected: ", xOffset);
	pxOverlay->AddDialog(pxDialog);

	xOffset.x = 10.f;
	pxDialog = new Asura_Function_Dialog("Start", xOffset, &TestSelectedEffects);
	pxDialog->SetHelpText("Shortcut key : 1");
	pxOverlay->AddDialog(pxDialog);

	xOffset.x = pxDialog->m_xMaxCoord.x + 1.f;
	pxDialog = new Asura_Function_Dialog("Stop instantly ", xOffset, &StopSelectedFast);
	pxDialog->SetHelpText("Shortcut key : 2");
	pxOverlay->AddDialog(pxDialog);

	xOffset.x = pxDialog->m_xMaxCoord.x + 1.f;
	pxDialog = new Asura_Function_Dialog("Stop", xOffset, &StopSelectedSlow);
	pxDialog->SetHelpText("Shortcut key : 3");
	pxOverlay->AddDialog(pxDialog);

	xOffset.x = 1.f;

	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_Dialog("All: ", xOffset);
	pxOverlay->AddDialog(pxDialog);

	xOffset.x = 10.f;
	pxDialog = new Asura_Function_Dialog("Start", xOffset, &TestAllEffects);
	pxDialog->SetHelpText("Shortcut key : Shift + 1");
	pxOverlay->AddDialog(pxDialog);

	xOffset.x = pxDialog->m_xMaxCoord.x + 1.f;
	pxDialog = new Asura_Function_Dialog("Stop instantly ", xOffset, &StopAllEffectsFast);
	pxDialog->SetHelpText("Shortcut key : Shift + 2");
	pxOverlay->AddDialog(pxDialog);

	xOffset.x = pxDialog->m_xMaxCoord.x + 1.f;
	pxDialog = new Asura_Function_Dialog("Stop", xOffset, &StopAllEffectsSlow);
	pxDialog->SetHelpText("Shortcut key : Shift + 3");
	pxOverlay->AddDialog(pxDialog);
	xOffset.x = 1.f;

}

bool Tools_Entity_PFX_Effect::HandlesThisMessage(int messageID)
{
	if( messageID == ASURA_MSG_ACTIVATE 
	 || messageID == ASURA_MSG_DEACTIVATE
	 || messageID == ASURA_MSG_DESTRUCT )
	{
		return true;
	}

	return false;
}

bool Tools_Entity_PFX_Effect::HandlesThisEvent(int iEventID) const
{
	switch(iEventID)
	{
		case ASURA_ENTITY_EVENT_ONACTIVATION:
		case ASURA_ENTITY_EVENT_ONDEACTIVATION:
		{
			return true;
		}
		default:
		{
			return PARENT::HandlesThisEvent(iEventID);
		}
	}
}

void Tools_Entity_PFX_Effect::Destroy()
{
	if(m_pxClientEntity)
	{
		m_pxClientEntity->SetEntityActive(false);
		m_pxClientEntity->StopEffect(false);
		m_pxClientEntity->SetAsDestroyed();
		m_pxClientEntity = NULL;
	}

	RemoveSplinePoints();

	Tools_Entity_Object::Destroy();

	if(s_pxCurrentObject == this)
	{
		Tools_Object_System::DeleteObject(s_pxCurrentObject);
		s_pxCurrentObject = 0;
	}
}

static const u_int uSECTIONS_PER_RING = 16;
void Tools_Entity_PFX_Effect::Render_Ring(const Asura_Vector_3& xCentre, float fRadius, u_int uColour, int iAxis, Asura_Matrix_3x3& xOrient)
{
	Asura_Vector_3 xEdge1, xEdge2;
	xEdge1 = xCentre;
	xEdge2 = xCentre;
	const float fRadiansPerSection = 2 * Asura_Maths::Pi / uSECTIONS_PER_RING;
	Asura_Vector_3 xAxis1, xAxis2;
	switch(iAxis)
	{
		case 0:
			xAxis1 = Asura_Vector_3(0.f, 1.f, 0.f);
			xAxis2 = Asura_Vector_3(0.f, 0.f, 1.f);
			break;
		case 1:
			xAxis1 = Asura_Vector_3(0.f, 0.f, 1.f);
			xAxis2 = Asura_Vector_3(1.f, 0.f, 0.f);
			break;
		case 2:
			xAxis1 = Asura_Vector_3(1.f, 0.f, 0.f);
			xAxis2 = Asura_Vector_3(0.f, 1.f, 0.f);
			break;
	}
	xAxis1 = xAxis1 * xOrient;
	xAxis2 = xAxis2 * xOrient;
	xEdge1 += xAxis1 * fRadius;
	for(u_int u = 0; u < uSECTIONS_PER_RING; ++u)
	{
		float fRadians = (u + 1) * fRadiansPerSection;
		xEdge2 = xCentre + (xAxis1 * Asura_Maths::Cosine(fRadians) + xAxis2 * Asura_Maths::Sine(fRadians)) * fRadius;
		Asura_Render::RenderLine(xEdge1, xEdge2, uColour);
		xEdge1 = xEdge2;
	}
}

void Tools_Entity_PFX_Effect::Render()
{
	// Update on render to avoid lag
	if(m_bSkyboxEffect && !m_bGameEntitiesNeedUpdating)
	{
		Position = *Asura_Camera::GetPosition() + m_xSkyboxOffset;
        
	}
	Tools_Entity_Object::Render();
	if(s_bRenderEffectBBs && IsGroupSelected())
	{
		m_pxClientEntity->RenderVisibility();
	}
	const Asura_Entity_PFX_Effect_Data& xData = m_pxServerEntity->m_xPFXEffectData;
	if(xData.m_bUsesSpline && xData.m_pxSpline)
	{
		xData.m_pxSpline->RenderLines(Asura_RGBA(0xff808080), Asura_RGBA(0xff0000ff));
	}
}

void Tools_Entity_PFX_Effect::RenderAngle(const Asura_Vector_3* pxAngle)
{
	if(!s_pxCurrentObject || !s_pxCurrentObject->IsSelected()) return;
	Render_Ring(s_pxCurrentObject->Position, 1.f, 0xffff0000, 0, s_pxCurrentObject->Orientation);
	Render_Ring(s_pxCurrentObject->Position, 1.f, 0xff00ff00, 1, s_pxCurrentObject->Orientation);
	Render_Ring(s_pxCurrentObject->Position, 1.f, 0xff0000ff, 2, s_pxCurrentObject->Orientation);
	if(pxAngle)
	{
		Asura_Vector_3 xVelocity( 0.f, 0.f,	1.f);
		const Asura_Euler_3 xEuler( (pxAngle->x * Asura_Maths::Pi),
									(pxAngle->y * Asura_Maths::Pi), 
									0.f);
		const Asura_Quat xPitchYawQuat(xEuler);
		// Roll must be applied after pitch and yaw, otherwise it does nothing to the z-axis
		const Asura_Euler_3 xRollEuler( 0.f, 0.f, (pxAngle->z * Asura_Maths::Pi));
		const Asura_Quat xRollQuat(xRollEuler);
		const Asura_Quat xQuat = xRollQuat * xPitchYawQuat;
		xVelocity = xQuat.RotateVector(xVelocity);
		xVelocity = xVelocity * s_pxCurrentObject->Orientation;
		Asura_Render::RenderLine(s_pxCurrentObject->Position, s_pxCurrentObject->Position + xVelocity, 0xffffffff);
	}
}

/// Start/stop effect static methods //////////////////////////////////////////////////////////////

void Tools_Entity_PFX_Effect::TestSelectedEffects()
{
	// Start all selected effects
	for(LIF<Tools_Entity_PFX_Effect*> xEffectLIF(&s_xPFXEffectList); !xEffectLIF.done(); xEffectLIF.next())
	{
		if(!xEffectLIF()->IsGroupSelected()) continue;

		Asura_ClientEntity_PFX_Effect* pxEffect = xEffectLIF()->m_pxClientEntity;
		Asura_Assert(pxEffect, "No client entity for this effect");
		if(!pxEffect) continue;

		//make sure position is up to date (in case this has been used by a cutscene)
		pxEffect->m_xCurrentPosition	= xEffectLIF()->m_bSkyboxEffect ? xEffectLIF()->m_xSkyboxOffset : xEffectLIF()->Position;
		pxEffect->m_xCurrentOrientation	= xEffectLIF()->Orientation;

		// Restart
		pxEffect->RestartEffect();
	}
}

void Tools_Entity_PFX_Effect::StopSelectedEffects(bool bInstant)
{
	// Stop all selected effects
	for(LIF<Tools_Entity_PFX_Effect*> xEffectLIF(&s_xPFXEffectList); !xEffectLIF.done(); xEffectLIF.next())
	{
		if(!xEffectLIF()->IsGroupSelected()) continue;
		Asura_Assert(xEffectLIF()->m_pxClientEntity, "No client entity for this effect");
		// Stop
		xEffectLIF()->m_pxClientEntity->StopEffect(bInstant);
	}
}

void Tools_Entity_PFX_Effect::TestAllEffects()
{
	// Start all effects
	for(LIF<Tools_Entity_PFX_Effect*> xEffectLIF(&s_xPFXEffectList); !xEffectLIF.done(); xEffectLIF.next())
	{
		Asura_ClientEntity_PFX_Effect* pxEffect = xEffectLIF()->m_pxClientEntity;
		Asura_Assert(pxEffect, "No client entity for this effect");
		if(!pxEffect) continue;

		//make sure position is up to date (in case this has been used by a cutscene)
		pxEffect->m_xCurrentPosition	= xEffectLIF()->m_bSkyboxEffect ? xEffectLIF()->m_xSkyboxOffset : xEffectLIF()->Position;
		pxEffect->m_xCurrentOrientation	= xEffectLIF()->Orientation;

		// Restart
		pxEffect->RestartEffect();
	}
}

void Tools_Entity_PFX_Effect::StopAllEffects(bool bInstant)
{
	// Stop all selected effects
	for(LIF<Tools_Entity_PFX_Effect*> xEffectLIF(&s_xPFXEffectList); !xEffectLIF.done(); xEffectLIF.next())
	{
		Asura_Assert(xEffectLIF()->m_pxClientEntity, "No client entity for this effect");
		// Stop
		xEffectLIF()->m_pxClientEntity->StopEffect(bInstant);
	}
}

void Tools_Entity_PFX_Effect::ActivateStartsActivePFX(bool bActive)
{
	for(Asura_Hashed_List_It<Asura_ClientEntity_PFX_Effect> xIt(Asura_ClientEntity_PFX_Effect::GetList()) ;!xIt.Done(); xIt.Next() )
	{
		Asura_ClientEntity_PFX_Effect* pxGenerator = xIt.GetCurrent();
		if(pxGenerator->IsDestroyed()) continue;

		if(bActive && pxGenerator->StartsActive())
		{
			pxGenerator->Activate(true);
		}
		else
		{
			pxGenerator->Activate(false);
		}
	}
	if(!bActive) Asura_PFX_System::ClearAllEffects();
}

void Tools_Entity_PFX_Effect::UpdatePFXEntities()
{
	for(Asura_Hashed_List_It<Asura_ClientEntity_PFX_Effect> xIt(Asura_ClientEntity_PFX_Effect::GetList()) ;!xIt.Done(); xIt.Next() )
	{
		Asura_ClientEntity_PFX_Effect* pxGenerator = xIt.GetCurrent();
		if(pxGenerator->IsDestroyed()) continue;

		if(pxGenerator->ShouldUpdate())
		{
			pxGenerator->Update();
		}
	}
}

void Tools_Entity_PFX_Effect::RestartAllEffects()
{
	for(LIF<Tools_Entity_PFX_Effect*> xEffectLIF(&s_xPFXEffectList); !xEffectLIF.done(); xEffectLIF.next())
	{
		Asura_Assert(xEffectLIF()->m_pxClientEntity, "No client entity for this effect");
		if(xEffectLIF()->m_pxClientEntity->IsEntityActive())
		{
			xEffectLIF()->m_pxClientEntity->RestartEffect();
		}
	}
}

void Tools_Entity_PFX_Effect::ClearAllEffectsAndFlagForRestart()
{
	for(LIF<Tools_Entity_PFX_Effect*> xEffectLIF(&s_xPFXEffectList); !xEffectLIF.done(); xEffectLIF.next())
	{
		Asura_Assert(xEffectLIF()->m_pxClientEntity, "No client entity for this effect");
		if(xEffectLIF()->m_pxClientEntity->IsEffectRunning())
		{
			xEffectLIF()->m_bRestartEffect = true;
		}
	}

	Asura_PFX_System::ClearAllEffects();
}

void Tools_Entity_PFX_Effect::ClearAndFlagForRestart_EffectTemplate(Asura_Hash_ID uEffectHash)
{
	if(!uEffectHash) return;

	for(LIF<Tools_Entity_PFX_Effect*> xEffectLIF(&s_xPFXEffectList); !xEffectLIF.done(); xEffectLIF.next())
	{
		Asura_Assert(xEffectLIF()->m_pxClientEntity, "No client entity for this effect");
		if(xEffectLIF()->m_pxClientEntity->IsEffectRunning() && 
			xEffectLIF()->m_pxClientEntity->GetCurrentActiveTemplateHash() == uEffectHash)
		{
			xEffectLIF()->m_bRestartEffect = true;
		}
	}

	Asura_PFX_System::ClearAllEffectsUsingEffectTemplate(uEffectHash);
}

void Tools_Entity_PFX_Effect::ClearAndFlagForRestart_SourceTemplate(Asura_Hash_ID uSourceHash)
{
	if(!uSourceHash) return;

	//Find the effect templates using the source , and restart anything using those
	Asura_PFX_EffectTemplateResourceSet_ResourceIterator xIterator(Asura_PFX_System::GetEffectTempResourceSet());
	for(;!xIterator.Done(); xIterator.Next())
	{
		const Asura_Hash_ID	uEffectHash	= xIterator.GetCurrentHash();
		if(!Asura_PFX_System::IsEffectTempLoadedByHash(uEffectHash)) continue;

		const Asura_PFX_Effect_Template* pxEffectTemplate = xIterator.GetCurrent();
		Asura_Assert(pxEffectTemplate, "");

		bool bUsedByThisEffect = false;

		for(u_int i=0; i<pxEffectTemplate->m_uNumSources; i++)
		{
			if(pxEffectTemplate->m_pxSources[i].m_uTemplate == 	uSourceHash)
			{
				bUsedByThisEffect = true;
				break;
			}
		}

		if(bUsedByThisEffect)
		{
			ClearAndFlagForRestart_EffectTemplate(uEffectHash);
		}
	}
}

void Tools_Entity_PFX_Effect::ClearAndFlagForRestart_ParticleTemplate(Asura_Hash_ID uParticleHash)
{
	if(!uParticleHash) return;

	//Find the effect templates using the particle , and restart anything using those
	Asura_PFX_EffectTemplateResourceSet_ResourceIterator xIterator(Asura_PFX_System::GetEffectTempResourceSet());
	for(;!xIterator.Done(); xIterator.Next())
	{
		const Asura_Hash_ID	uEffectHash	= xIterator.GetCurrentHash();
		if(!Asura_PFX_System::IsEffectTempLoadedByHash(uEffectHash)) continue;

		const Asura_PFX_Effect_Template* pxEffectTemplate = xIterator.GetCurrent();
		Asura_Assert(pxEffectTemplate, "");

		bool bUsedByThisEffect = false;

		for(u_int i=0; i<pxEffectTemplate->m_uNumSources; i++)
		{
			const Asura_PFX_Source_Template* pxSource = Asura_PFX_System::GetSourceTempByHash(pxEffectTemplate->m_pxSources[i].m_uTemplate);
			if(!pxSource) continue;

			if(pxSource->m_uParticleTemplateHash == uParticleHash)
			{
				bUsedByThisEffect = true;
				break;
			}
		}

		if(bUsedByThisEffect)
		{
			ClearAndFlagForRestart_EffectTemplate(uEffectHash);
		}
	}
}



void Tools_Entity_PFX_Effect::EditCollisionPlane()
{
	Asura_Assert(s_pxCurrentObject, "No current effect to edit collision plane for.");

	if(!s_pxCurrentObject->m_pxCollisionPlane)
	{
		s_pxCurrentObject->m_pxCollisionPlane = new Tools_PFX_Effect_Collision_Plane();
	}
		
	Asura_Assert(s_pxCurrentObject->m_pxCollisionPlane, "Null wind vector");
	if(s_pxCurrentObject->m_pxCollisionPlane)
	{
		//Set orientation and position based on current collision plane
		//s_pxCurrentObject->m_pxCollisionPlane->Orientation.Identity();
		Asura_Maths::OrientationFromDirection(s_pxCurrentObject->m_pxCollisionPlane->Orientation, s_pxCurrentObject->m_pxServerEntity->m_xPFXEffectData.m_xCollisionDirection);
		s_pxCurrentObject->m_pxCollisionPlane->Position = s_pxCurrentObject->m_pxServerEntity->m_xPFXEffectData.m_xCollisionPosition;
		//Select new object.
		Tools_Object_System::SelectObject(s_pxCurrentObject->m_pxCollisionPlane);
	}
}

void Tools_Entity_PFX_Effect::EditSourceVolume()
{
	Asura_Assert(s_pxCurrentObject, "No current effect to edit source volume for.");

	if(!s_pxCurrentObject->m_pxSourceVolumeEdit)
	{
		s_pxCurrentObject->m_pxSourceVolumeEdit = new Tools_PFX_SourceVolume_Object(s_pxCurrentObject->m_xSourceVolume);
	}
		
	if(s_pxCurrentObject->m_pxSourceVolumeEdit)
	{
		s_pxCurrentObject->m_pxSourceVolumeEdit->SetPositionAndOrient(s_pxCurrentObject->Position,s_pxCurrentObject->Orientation);
	}

}

void Tools_Entity_PFX_Effect::SelectThisObject()
{
	Tools_Object_System::DeselectAllObjects();
	Tools_Object_System::SelectObject(s_pxCurrentObject);
}

void Tools_Entity_PFX_Effect::AddFilesToExportList(List<Asura_String>& xNameList) const
{
    if (m_pxServerEntity)
    {
		// We need to add the effect, source, and particles templates for this effect
		if(!Tools_Mode_PFX::AddTemplateToExportListByHash(&xNameList, m_pxServerEntity->m_xPFXEffectData.m_uEffectTemplateHash, m_pxServerEntity->Guid))
		{
			Tools_Timed_Message_System::AddMessage(0xffff00,"Warning: PFX effect with guid: %u has no effect template.", m_pxServerEntity->Guid);
		}
    }
}

void Tools_Entity_PFX_Effect::ReplaceEffectTemplate(Asura_Hash_ID uOldHash, Asura_Hash_ID uNewHash)
{
	if(s_uEffectTempHash == uOldHash)
	{
		s_uEffectTempHash = uNewHash;
	}
	for(LIF<Tools_Entity_PFX_Effect*> xEffectLIF(&s_xPFXEffectList); !xEffectLIF.done(); xEffectLIF.next())
	{
		Asura_Assert(xEffectLIF()->m_pxServerEntity && xEffectLIF()->m_pxClientEntity, "Missing entities for this PFX effect");
		if(xEffectLIF()->m_pxServerEntity && xEffectLIF()->m_pxClientEntity)
		{
			if(xEffectLIF()->m_pxServerEntity->m_xPFXEffectData.m_uEffectTemplateHash == uOldHash)
			{
				xEffectLIF()->m_pxServerEntity->m_xPFXEffectData.m_uEffectTemplateHash = uNewHash;
				xEffectLIF()->m_pxClientEntity->m_xPFXEffectData.m_uEffectTemplateHash = uNewHash;
			}
		}
	}
}

void Tools_Entity_PFX_Effect::RemoveEffectTemplate(Asura_Hash_ID uHash)
{
	if(s_uEffectTempHash == uHash)
	{
		s_uEffectTempHash = ASURA_HASH_ID_UNSET;
	}
	for(LIF<Tools_Entity_PFX_Effect*> xEffectLIF(&s_xPFXEffectList); !xEffectLIF.done(); xEffectLIF.next())
	{
		//Asura_Assert(xEffectLIF()->m_pxServerEntity && xEffectLIF()->m_pxClientEntity, "Missing entities for this PFX effect");
		if(xEffectLIF()->m_pxServerEntity && xEffectLIF()->m_pxClientEntity)
		{
			if(xEffectLIF()->m_pxServerEntity->m_xPFXEffectData.m_uEffectTemplateHash == uHash)
			{
				xEffectLIF()->m_pxServerEntity->m_xPFXEffectData.m_uEffectTemplateHash = 0;
				xEffectLIF()->m_pxClientEntity->m_xPFXEffectData.m_uEffectTemplateHash = 0;
				xEffectLIF()->m_pxClientEntity->StopEffect(true);
			}

			if(xEffectLIF()->m_pxClientEntity->GetCurrentActiveTemplateHash() == uHash)
			{
				xEffectLIF()->m_pxClientEntity->StopEffect(true);
			}
		}
	}
}

void Tools_Entity_PFX_Effect::UpdateShape()
{
	//pick shape based on how close the object is
	float fDist = (*Asura_Camera::GetPosition() - Position).Magnitude();
	if(fDist < 3.0f)
	{
		Mesh = g_pxToolsShapeMeshSmall;
	}
	else
	{
		Mesh = g_pxToolsShapeMesh;
	}
}

void Tools_Entity_PFX_Effect::CreateNewTemplate()
{
	const Asura_PFX_Effect_Template*	pxTemplate	= Tools_Mode_PFX::CreateNewEffectTemplate();

	if(pxTemplate != NULL)
	{
		s_uEffectTempHash = pxTemplate->GetTemplateHash();
		UpdateEffectList();
	}
}

void Tools_Entity_PFX_Effect::CopyTemplate()
{
	const Asura_PFX_Effect_Template*	pxTemplate	= Tools_Mode_PFX::CopyEffectTemplate();

	if(pxTemplate != NULL)
	{
		s_uEffectTempHash = pxTemplate->GetTemplateHash();
		UpdateEffectList();
	}
}

void Tools_Entity_PFX_Effect::RenameTemplate()
{
	Asura_PFX_Effect_Template*	pxTemplate	= Tools_Mode_PFX::RenameEffectTemplate();

	if(pxTemplate != NULL)
	{
		Tools_Mode_PFX::s_uChooseTemplateHash = s_uEffectTempHash = pxTemplate->GetTemplateHash();
		UpdateEffectList();
	}
}

void Tools_Entity_PFX_Effect::UpdateEffectList()
{
	// No point updating the list if the dialog doesn't exist
	if(!s_pxEffectTempDialog) return;

	// Select current hash
	s_pxEffectTempDialog->SetTemplateHash(s_uEffectTempHash);
	s_pxEffectTempDialog->UpdateMax();
}

void Tools_Entity_PFX_Effect::SetNameFromTemplate(Tools_Entity_PFX_Effect* pxPFXEntity /* = s_pxCurrentObject */)
{
	Asura_Assert(pxPFXEntity->Entity && pxPFXEntity->Entity->ToolsData,"No entity/name");
	const Asura_String* pxTemplateName = Asura_PFX_System::GetEffectTempNameByHash(pxPFXEntity->m_pxServerEntity->m_xPFXEffectData.m_uEffectTemplateHash);
	if(pxTemplateName)
	{
		bool bNameInUse;
		u_int uNum = 1;
		Asura_String xNewName;
		do
		{
			char szNum[8];
			Asura_PrintfToBfr(szNum, " %03u", 8, true, uNum);
			uNum++;
			xNewName = *pxTemplateName;
			xNewName += Asura_String(szNum);
			bNameInUse = false;
			for(LIF<Tools_Object*> olif(&Tools_Object_System::ObjectList);!olif.done();olif.next())
			{
				if(!stricmp(xNewName.GetString(),olif()->GetName()))
				{
					bNameInUse = true;//name already in use
					break;
				}
			}

		}
		while(bNameInUse);
		pxPFXEntity->Entity->ToolsData->Name = xNewName;
	}
}

void Tools_Entity_PFX_Effect::UpdateEffectPosition()
{
	if(m_bSkyboxEffect)
	{
		m_pxServerEntity->m_xPosition			= m_xSkyboxOffset;
		m_pxClientEntity->m_xCurrentPosition	= m_xSkyboxOffset;
	}
	else
	{
		m_pxServerEntity->m_xPosition			= Position;
		m_pxClientEntity->m_xCurrentPosition	= Position;
	}
}

void Tools_Entity_PFX_Effect::AddContextMenuItems(class Tools_ContextMenu& xMenu, const class Tools_ContextMenu_Context& xContext)
{
	Tools_Entity_Object::AddContextMenuItems(xMenu, xContext);

	Tools_ContextMenu_Item* pxItem;
	Tools_ContextMenu_Section* pxSection = xMenu.GetSection(TOOLS_CONTEXTMENU_SECTION_SELECTION);
	pxSection->AddSeparator();

	//Add option for selecting/deselecting all PFX entities using this template
	u_int uMode;
	const char* szText;
	if(xContext.m_bShiftHeld)
	{
		szText = "Deselect by template";
		uMode = 0;
	}
	else if(xContext.m_bControlHeld)
	{
		szText = "Select by template";
		uMode = 1;
	}
	else
	{
		szText = "Select by template";
		uMode = 2;
	}

	pxItem = new Tools_ContextMenu_Item_Function(szText, ContextMenu_SelectByTemplate, this, uMode);
	pxItem->SetID("Select by template");
	pxItem->SetHelpText("Selects/deselects all PFX effects using this template");
	pxSection->AddItem(pxItem);


}

void Tools_Entity_PFX_Effect::ContextMenu_SelectByTemplate(void* pObject, u_int uMode)
{
	Tools_Object* pxObject = (Tools_Object*) pObject;

	Asura_Assert(pxObject && pxObject->Type==TOT_Entity_PFX_Effect,"");
	Tools_Entity_PFX_Effect* pxPFXObject = (Tools_Entity_PFX_Effect*) pxObject;

	//list all PFX effects with the same template
	List<Tools_Object*> xSelectionList;
	for(LIF<Tools_Entity_PFX_Effect*> xEffectLIF(&s_xPFXEffectList);!xEffectLIF.done();xEffectLIF.next())
	{
		if(pxPFXObject->m_pxServerEntity->m_xPFXEffectData.m_uEffectTemplateHash == xEffectLIF()->m_pxServerEntity->m_xPFXEffectData.m_uEffectTemplateHash)
		{
			xSelectionList.add_entry(xEffectLIF());
		}
	}
	if(xSelectionList.contains(pxObject))
	{
		xSelectionList.delete_entry(pxObject);
		xSelectionList.add_entry(pxObject);
	}

	//select/deselect them
	if(uMode == 0)
	{
		Tools_Object_System::DeselectObjects(xSelectionList);
	}
	else
	{
		Tools_Object_System::SelectObjects(xSelectionList, uMode == 1);
	}
}

void Tools_Entity_PFX_Effect::StartTempEntityEdit()
{
	Asura_Assert(!s_bTempEntityEdit, "Already in temporary entity edit");
	s_bTempEntityEdit = true;
}

void Tools_Entity_PFX_Effect::FinishTempEntityEdit()
{
	Asura_Assert(s_bTempEntityEdit, "Not in temporary entity edit");
	s_bTempEntityEdit = false;
	if(Tools_Object_System::SelectedObject == NULL)
	{
		SelectThisObject();
	}
	else
	{
		CloseOverlays();
	}
}

void Tools_Entity_PFX_Effect::CreateEffectTab (void)
{
	Asura_Vector_2	xOffset(s_xEffectTabOverlayOffset);

	u_int uStoreSelectedTab_Effects = Tools_Mode_PFX::s_uSelectedTab_Effects;	// keep previous tab. Because ClearTab can zero the value Tools_Mode_PFX::s_uSelectedTab_Effects, we need to store it and restore it again.
	s_pxTabs->ClearTab(1);
	s_pxTabs->SelectTab(1);
	Tools_Mode_PFX::s_uSelectedTab_Effects = uStoreSelectedTab_Effects;
	Tools_Mode_PFX::AddEffectTemplateOverlayControls(s_pxTabs, xOffset, s_uEffectTempHash);
}

void Tools_Entity_PFX_Effect::CreateParticleTab (void)
{
	Asura_Vector_2				xOffset(s_xSourceTabOverlayOffset);
	Asura_Hash_ID				uParticleTempHash	= ASURA_HASH_ID_UNSET;
	Asura_PFX_Source_Template*	pxSourceTemp		= Asura_PFX_System::GetSourceTempByHash(s_uSourceTempHash);

	if(pxSourceTemp)
	{
		uParticleTempHash = pxSourceTemp->m_uParticleTemplateHash;
	}

	u_int uStoreSelectedTab_Particles = Tools_Mode_PFX::s_uSelectedTab_Particles;		// keep previous tab. Because ClearTab can zero the value Tools_Mode_PFX::s_uSelectedTab_Effects, we need to store it and restore it again.
	s_pxTabs->ClearTab(3);
	s_pxTabs->SelectTab(3);
	Tools_Mode_PFX::s_uSelectedTab_Particles = uStoreSelectedTab_Particles;
	Tools_Mode_PFX::AddParticleTemplateOverlayControls(s_pxTabs, xOffset, uParticleTempHash);
}

void Tools_Entity_PFX_Effect::CreateSourceTab (const u_int uCurrentSource)
{
	Asura_Vector_2	xOffset(s_xSourceTabOverlayOffset);

	s_uSourceTempHash = ASURA_HASH_ID_UNSET;
	Asura_PFX_Effect_Template* pxEffectTemp = Asura_PFX_System::GetEffectTempByHash(s_uEffectTempHash);
	if(pxEffectTemp && pxEffectTemp->m_pxSources)
	{
		s_uSourceTempHash = pxEffectTemp->m_pxSources[uCurrentSource].m_uTemplate;
	}

	u_int uStoreSelectedTab_Source = Tools_Mode_PFX::s_uSelectedTab_Source;		// keep previous tab. Because ClearTab can zero the value Tools_Mode_PFX::s_uSelectedTab_Effects, we need to store it and restore it again.
	s_pxTabs->ClearTab(2);
	s_pxTabs->SelectTab(2);
	Tools_Mode_PFX::s_uSelectedTab_Source = uStoreSelectedTab_Source;
	Tools_Mode_PFX::AddSourceTemplateOverlayControls(s_pxTabs, xOffset, s_uSourceTempHash);
}

void Tools_Entity_PFX_Effect::CreateSplinePoints(int iIndexAfter /* = -1 */)
{
	Asura_Entity_PFX_Effect_Data& xData = m_pxServerEntity->m_xPFXEffectData;
	if(xData.m_bUsesSpline && xData.m_pxSpline)
	{
		// First clean out any pre-existing objects
		RemoveSplinePoints();
		// Now create new ones.
		// Create the tools objects for the link points.
		const u_int uNumPoints = xData.m_pxSpline->GetNumberOfPoints();
		if(!uNumPoints) { return; }

		for (u_int u = 1; u < uNumPoints - 1; ++u)
		{
			const bool bCanBeDestroyed = (u != 1 && u != uNumPoints - 2);
			Tools_Object_PFXSplinePoint* pxPoint = new Tools_Object_PFXSplinePoint(xData.m_pxSpline, this, u, bCanBeDestroyed);
			m_xSplinePointList.add_entry(pxPoint);
			if (iIndexAfter + 1 == static_cast<int>(u))
			{
				Tools_Object_System::SelectObject(pxPoint, false);
			}
		}
	}
}

void Tools_Entity_PFX_Effect::RemoveSplinePoints()
{
	while(m_xSplinePointList.size())
	{
		m_xSplinePointList.first_entry()->DestroyToolsObjectOnly();
		m_xSplinePointList.delete_first_entry();
	}
}

void Tools_Entity_PFX_Effect::ChangedSpline()
{
	m_pxClientEntity->m_xPFXEffectData = m_pxServerEntity->m_xPFXEffectData;
	m_pxClientEntity->RestartEffect();
	SetEntityModified();
}

void Tools_Entity_PFX_Effect::AddSpline()
{
	Asura_Entity_PFX_Effect_Data& xData = s_pxCurrentObject->m_pxServerEntity->m_xPFXEffectData;
	Asura_Assert(!xData.m_bUsesSpline && !xData.m_pxSpline, "");
	xData.m_bUsesSpline = true;
	xData.m_pxSpline = new Asura_Spline<Asura_Vector_3>();
	Asura_Vector_3 axPoints[2];
	axPoints[0] = s_pxCurrentObject->m_pxServerEntity->m_xPosition;
	axPoints[1] = s_pxCurrentObject->m_pxServerEntity->m_xPosition;
	axPoints[1] += Asura_Vector_3(0.f, 0.f, 1.f) * s_pxCurrentObject->m_pxServerEntity->m_xOrientation;
	xData.m_pxSpline->SetPointPositions(2, axPoints);
	// Now create the tools objects.
	s_pxCurrentObject->CreateSplinePoints();
	s_pxCurrentObject->ChangedSpline();
}

void Tools_Entity_PFX_Effect::RemoveSpline()
{
	YesNo_Overlay xAreYouSure("Spline data will be lost. Are you sure you want to remove the spline from this effect?");
	if (xAreYouSure.DoModal())
	{
		// Clean up the tools objects
		s_pxCurrentObject->RemoveSplinePoints();
		// Delete the spline
		Asura_Entity_PFX_Effect_Data& xData = s_pxCurrentObject->m_pxServerEntity->m_xPFXEffectData;
		Asura_Assert(xData.m_bUsesSpline && xData.m_pxSpline, "");
		xData.m_bUsesSpline = false;
		delete xData.m_pxSpline;
		xData.m_pxSpline = NULL;
	}
}

bool Tools_Entity_PFX_Effect::CanAddSpline()
{
	return !s_pxCurrentObject->m_pxServerEntity->m_xPFXEffectData.m_bUsesSpline;
}

bool Tools_Entity_PFX_Effect::CanRemoveSpline()
{
	return s_pxCurrentObject->m_pxServerEntity->m_xPFXEffectData.m_bUsesSpline;
}

void Tools_Entity_PFX_Effect::GetBoundingBoxForModLinkTest(Asura_Bounding_Box& xOutputBB) const
{
	GetBoundingBox(xOutputBB);
	xOutputBB.Displace(Position);
	for (CLIF <Tools_Object_PFXSplinePoint*> xIt(&m_xSplinePointList); !xIt.done(); xIt.next())
	{
		// Bounding box will be offset by this object's position, so preemptively subtract it.
		xOutputBB.Set(xIt()->Position - Position);
	}
}

void Tools_Entity_PFX_Effect::DisableExportForAllPlatforms()
{
	Entity->ToolsData->m_uPlatforms = 0;
}


/// End of file. ///
