//
// Tools_Entity_PFX_Effect.h
//
// Created by KOR - 05/03/2004 10:13:45
//
// Copyright (C) 2004 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif

#ifndef __TOOLS_ENTITY_PFX_EFFECT_H__
#define __TOOLS_ENTITY_PFX_EFFECT_H__

#include "Entity/Tools_Entity_Object.h"
#include "SpecialFX/PFX/Asura_PFX_Volume.h"

class Asura_ClientEntity_PFX_Effect;
class Asura_ServerEntity_PFX_Effect;
class Tools_Object_PFXSplinePoint;
class Tools_PFX_Effect_Collision_Plane;
class Tools_PFX_SourceVolume_Object;

class Tools_Entity_PFX_Effect : public Tools_Entity_Object
{
	typedef Tools_Entity_Object PARENT;

	friend class Tools_Mode_PFX;
	friend class Tools_MessageLogAction_PFXEffect;

public:
	// Constructor and destructor
	Tools_Entity_PFX_Effect(Asura_Entity* xEntity);
	~Tools_Entity_PFX_Effect();

	#ifndef TOOLS_USE_OLD_ENTITY_MANAGER
	virtual const char* GetShortName(){return "PFX";}
	#endif

	// Static create functions	
	static void Create();
	static Tools_Entity_PFX_Effect* Create(bool bGenerateName);

	// Virtual Tools_Object methods
	const char* GetTypeDescription(){return "PFX Effect";}		
	void UpdateWhileSelected();
	void Update();
	void Destroy();
	void Render();
	bool HandlesThisMessage(int iMessageID);
	bool HandlesThisEvent(int iEventID) const;
	virtual void UpdatePositionInAsuraData(bool);

	static const char* GetNamePrefix() { return "PFX_"; }
	virtual const char* GetMyNamePrefix() const { return Tools_Entity_PFX_Effect::GetNamePrefix(); }

	// Virtual Tools_Entity_Object methods
	void AddOverlayControls(class Asura_Overlay* xNewOverlay,class Asura_Vector_2& xOffset);		

	// Re-selects current effect - use this after editing collision plane.
	static void SelectThisObject(void);

	// Renders axes and direction for angle editing
	static void RenderAngle(const Asura_Vector_3* pxAngle);

	//Disable movement undo for skybox effects , since their position changes whenever the camera moves
	virtual bool AllowMovementUndo(){return !m_bSkyboxEffect;}

	// Don't allow selection during temp entity edit.
	virtual bool IsSelectable() { return !s_bTempEntityEdit; }
	static void StartTempEntityEdit();
	static void FinishTempEntityEdit();

	//Activates/deactivates all starts active PFX effects
	static void ActivateStartsActivePFX(bool bActive);
	//Does ther per-frame update for each PFX client entity
	static void UpdatePFXEntities();

	void CreateSplinePoints(int iIndexAfter = -1);
	void RemoveSplinePoints();

	void ChangedSpline();

	void GetBoundingBoxForModLinkTest(Asura_Bounding_Box& xOutputBB) const;

	void DisableExportForAllPlatforms();

	bool m_bGameEntitiesNeedUpdating;
private:
	// Virtual Tools_Object methods
	void OnSelection(void);
	void OnDeselection(void);
	void AddFilesToExportList(List<Asura_String>& xNameList) const;

	void AddContextMenuItems(class Tools_ContextMenu& xMenu, const class Tools_ContextMenu_Context& xContext);
	static void ContextMenu_SelectByTemplate(void* pObject, u_int uMode);


	// Testing functions
	static void TestSelectedEffects();
	static void TestAllEffects();
	static void StopSelectedSlow() { StopSelectedEffects(false); }
	static void StopSelectedFast() { StopSelectedEffects(true); }
	static void StopSelectedEffects(bool bInstant);
	static void StopAllEffectsSlow() { StopAllEffects(false); }
	static void StopAllEffectsFast() { StopAllEffects(true); }
	static void StopAllEffects(bool bInstant);
	// Edit the particle collision plane entity
	static void EditCollisionPlane();
	// Edit the optional source volume
	static void EditSourceVolume();
	// Replace effect template hashes after a rename
	static void ReplaceEffectTemplate(Asura_Hash_ID uOldHash, Asura_Hash_ID uNewHash);
	// Zero effect template hashes after a delete
	static void RemoveEffectTemplate(Asura_Hash_ID uHash);
	static void CreateNewTemplate();
	static void CopyTemplate();
	static void RenameTemplate();
	// Update effect list
	static void UpdateEffectList();
	static void SetCurrentNameFromTemplate() { SetNameFromTemplate(s_pxCurrentObject); }
	static void SetNameFromTemplate(Tools_Entity_PFX_Effect* pxPFXEntity);

	//Force any entities to be restarted (if they were playing)
	static void RestartAllEffects();

	//Ensures that all running effects are cleared out , and flags them for restart next frame
	static void ClearAllEffectsAndFlagForRestart();
	//similar but just for a particular effect/source/particle template
	static void ClearAndFlagForRestart_EffectTemplate(Asura_Hash_ID uEffectHash);
	static void ClearAndFlagForRestart_SourceTemplate(Asura_Hash_ID uSourceHash);
	static void ClearAndFlagForRestart_ParticleTemplate(Asura_Hash_ID uParticleHash);

	// Copy/paste support
	virtual void Copy(const Tools_Object* pxObject);
	virtual bool CanBeCopied() const {return true;}
	virtual Tools_Object* CreateObjectOfThisType() const;

	void UpdateShape();
	void UpdateEffectPosition();

	static void Render_Ring(const Asura_Vector_3& xCentre, float fRadius, u_int uColour, int iAxis, Asura_Matrix_3x3& xOrient);

	virtual void CreateOverlay();
	static void CloseOverlays();

	static void CreateEffectTab(void);
	static void CreateParticleTab(void);
	static void CreateSourceTab(const u_int uCurrentSource);

	static void AddSpline();
	static void RemoveSpline();
	static bool CanAddSpline();
	static bool CanRemoveSpline();

	// Private members
	Asura_ServerEntity_PFX_Effect*			m_pxServerEntity;
	Asura_ClientEntity_PFX_Effect*			m_pxClientEntity;
	Tools_PFX_Effect_Collision_Plane*		m_pxCollisionPlane;
	Tools_PFX_SourceVolume_Object*			m_pxSourceVolumeEdit;

	bool									m_bIgnoreModuleLinking;
	bool									m_bSkyboxEffect;
	Asura_Vector_3							m_xSkyboxOffset;

	bool									m_bWaterDistorterEffect;

	Asura_PFX_Volume						m_xSourceVolume;

	float									m_fMinRetriggerTime;
	float									m_fMaxRetriggerTime;

	bool									m_bMinModified;
	bool									m_bMaxModified;
	List<Tools_Object_PFXSplinePoint*>		m_xSplinePointList;

	bool									m_bRestartEffect;

	// Static members

	// Object being currently edited
	static Tools_Entity_PFX_Effect*			s_pxCurrentObject;
	// List of PFX effects
	static List<Tools_Entity_PFX_Effect*>	s_xPFXEffectList;

	// Overlay members
	static class Tools_PFXSelect_Dialog*		s_pxEffectTempDialog;
	static bool									s_bEffectTempChanged;
	static Asura_Hash_ID						s_uEffectTempHash;
	static bool									s_bCollisionPlaneRelative;
	static bool									s_bRenderEffectBBs;
	static Asura_Hash_ID						s_uSourceTempHash;
	// KOR - 18/05/2006 - to ensure that editing special entities e.g. gravity don't break template editing
	static bool									s_bTempEntityEdit;
	static class Asura_Tabs_Dialog*				s_pxTabs;
	static bool									s_bNeedToRefreshOverlay;

	static Asura_Vector_2						s_xEffectTabOverlayOffset;
	static Asura_Vector_2						s_xParticleTabOverlayOffset;
	static Asura_Vector_2						s_xSourceTabOverlayOffset;
};

#endif//__TOOLS_ENTITY_PFX_EFFECT_H__
