//////////////////////////////////////////////////////////////////////////////
//
// Tools_Entity_Projector.cpp
//
// Copyright (c) 2006 Rebellion (Derby). All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

#ifdef ASURA_TARGET_SM3


// This file's header.
#include "Entity/Tools_Entity_Projector.h"

// Engine independent.
#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Overlay.h"
#include "Interface/Asura_Overlay_System.h"

// Tools.
#include "Entity/Tools_Entity_Data.h"
#include "Load&Save/Tools_SaveAsura.h"
#include "Objects/Tools_Object_System.h"
#include "Mesh/Asura_Mesh_System.h"
#include "Misc/Tools_Timed_Messages.h"
#include "Texturing/Tools_PickTexture.h"
#include "Rendering/Asura_Render.h"
#include "Animation/Asura_Animation.h"
#include "Entity/Asura_Entity_Classification.h"
#include "Interface/Tools_ColourPicker.h"

// Project independent.
#include "Entity/Projector/Asura_ClientEntity_Projector.h"
#include "Entity/Projector/Asura_ServerEntity_Projector.h"
#include "Entity/PhysicalObject/Asura_ServerEntity_PhysicalObject.h"

// C L A S S   I M P L E M E N T A T I O N S /////////////////////////////////


Tools_Entity_Projector*     Tools_Entity_Projector::s_pxSelected = NULL;
bool						Tools_Entity_Projector::m_bShowVolume = false;
bool						Tools_Entity_Projector::s_bDynamicLightFlagChanged = false;
bool						Tools_Entity_Projector::s_bFakeLightFlagChanged = false;
bool						Tools_Entity_Projector::s_bModified = false;
bool					    Tools_Entity_Projector::s_bResetCurrentProjectorPtrOnDeselect = true;

static Asura_Dialog	*g_apxTextureNames[3] = {NULL,NULL,NULL};

Tools_Entity_Projector::Tools_Entity_Projector( Asura_ServerEntity_Projector* pxEntity )
    : PARENT( pxEntity )
{
	Type = TOT_Entity_Projector;
	m_pxServerEntity = pxEntity;
    m_xParams = pxEntity->GetParams();
    m_bParamsModified = true;	
	s_bModified	= false;


	Position = m_xParams.m_xPosition;
	Orientation = m_xParams.m_xOrientation;

	Asura_String mesh_name = "ToolsShape_Light";
	Mesh = Asura_Mesh_System::GetMesh(mesh_name);

	// Add the textures now. Can't rely on AddFilesToExportList to load the textures, as it only
	// gets called on quit to game. Which my SS2 build was always doing when entering tools.
	Asura_Texture_Management::AddTexture(m_xParams.m_xTextureName);
//	Asura_Texture_Management::AddTexture(m_xParams.m_xFalloffTextureName);	
	Asura_Texture_Management::AddTexture(m_xParams.m_xDetailTextureName);
	Asura_Texture_Management::LoadTextures();
}

Tools_Entity_Projector::~Tools_Entity_Projector( void )
{

}

const char * const g_aszBlendModes[] = {"Additive", "Multiply (0-100%)", "Multiply X2 (0-200%)", "Standard Alpha Blend"};

void Tools_Entity_Projector::AddOverlayControls( Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset )
{
    Asura_Dialog* pxDialog;

    // Sort out spacing between our gadgets and default object stuff.
    if( pxOverlay->m_pxDialogList )
    {
        xOffset.y += Asura_Dialog::LargeSpacing() - Asura_Dialog::DefaultSpacing();
    }

	// Projector Params	
	pxDialog = new Asura_FlagTickbox_Dialog( "Show Frustum", xOffset, &s_pxSelected->m_xParams.m_uProjectorParams_Flags, 1 << 7 );
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_FlagTickbox_Dialog( "Fake Light", xOffset, &s_pxSelected->m_xParams.m_uProjectorParams_Flags, 1 << 0, &s_bFakeLightFlagChanged);
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_FlagTickbox_Dialog( "Dynamic Light", xOffset, &s_pxSelected->m_xParams.m_uProjectorParams_Flags, 1 << 1, &s_bDynamicLightFlagChanged);
	pxOverlay->AddDialog( pxDialog );		
	xOffset.y += Asura_Dialog::DefaultSpacing();
	xOffset.x += 15;
	xOffset.y -= (3 * Asura_Dialog::DefaultSpacing());

	Asura_Vector_2 xOfs2 = xOffset + Asura_Vector_2(23,0);
	pxDialog = new Tools_ColourPicker_Dialog("Colour",	xOfs2,Asura_Vector_2(2.0f,1.0f),
		&s_pxSelected->m_xParams.m_afRGB[0],
		&s_pxSelected->m_xParams.m_afRGB[1],
		&s_pxSelected->m_xParams.m_afRGB[2],1.0f,&s_bModified);
	pxOverlay->AddDialog( pxDialog );
	

	pxDialog = new Asura_Numerical_Dialog("Light Intensity  RGB:", xOffset, &s_pxSelected->m_xParams.m_fIntensity , 0.0f, 100.0f, 0.01f, "%0.02f");
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Distance Fadeout: Start Dist", xOffset, &s_pxSelected->m_xParams.m_fFadeoutStart , 0.0f, 2500.0f, 1.0f, "%0.1f");
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();
	
	pxDialog = new Asura_Numerical_Dialog("Distance Fadeout: Range", xOffset, &s_pxSelected->m_xParams.m_fFadeoutDist , 0.0f, 2500.0f, 1.0f, "%0.1f");
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();
	xOffset.x-=15;

	xOffset.y += Asura_Dialog::LargeSpacing();
	pxDialog = new Asura_DropDown_List_Dialog(xOffset, g_aszBlendModes, &s_pxSelected->m_xParams.m_iBlendMode,4,4,NULL,NULL,"Blend Mode");
	pxOverlay->AddDialog( pxDialog );


	xOffset.y += Asura_Dialog::LargeSpacing();
	pxDialog = new Asura_Function_Dialog("Select Texture", xOffset, PickTexture_Projected);
	pxOverlay->AddDialog( pxDialog );
	xOffset.x+=15.0f;
	pxDialog = new Asura_Function_Dialog("Clear", xOffset, ClearTexture_Projected);
	pxOverlay->AddDialog( pxDialog );
	xOffset.x+=7.0f;
	g_apxTextureNames[0] = pxDialog = new Asura_Dialog("(None)",xOffset);
	pxOverlay->AddDialog( pxDialog);
	xOffset.x-=22.0f;

	
/*
	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_Function_Dialog("Select Falloff Texture", xOffset, PickTexture_Falloff);
	pxOverlay->AddDialog( pxDialog );
	xOffset.x+=15.0f;
	pxDialog = new Asura_Function_Dialog("Clear", xOffset, ClearTexture_Falloff);
	pxOverlay->AddDialog( pxDialog );
	xOffset.x+=7.0f;
	g_apxTextureNames[1] = pxDialog = new Asura_Dialog("(None)",xOffset);
	pxOverlay->AddDialog( pxDialog);
	xOffset.x-=22.0f;
*/
	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_Function_Dialog("Select Detail Texture", xOffset, PickTexture_Detail);
	pxDialog->SetHelpText("Currently experimental - for SS2 jungle 'dapples' over large areas");
	pxOverlay->AddDialog( pxDialog );
	
	xOffset.x+=15.0f;
	pxDialog = new Asura_Function_Dialog("Clear", xOffset, ClearTexture_Detail);
	pxOverlay->AddDialog( pxDialog );
	xOffset.x+=7.0f;
	g_apxTextureNames[2] = pxDialog = new Asura_Dialog("(None)",xOffset);
	pxOverlay->AddDialog( pxDialog);
	xOffset.x-=22.0f;

	UpdateTextureNames();

	xOffset.y += Asura_Dialog::LargeSpacing();

	pxDialog = new Asura_Numerical_Dialog("Detail Tex Repeats", xOffset, &s_pxSelected->m_xParams.m_fDetailTextureScale , 0.0f, 150.0f, 0.1f, "%0.01f");
	pxOverlay->AddDialog( pxDialog );	
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_AngleNumerical_Dialog("FoV", xOffset, &s_pxSelected->m_xParams.m_fFoV , 0.0f, 150.0f, 0.1f, "%0.01f");
	pxOverlay->AddDialog( pxDialog );	
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("U:V Aspect Ratio", xOffset, &s_pxSelected->m_xParams.m_fAspect , 0.1f, 10.0f, 0.01f, "%0.02f");
	pxOverlay->AddDialog( pxDialog ); 
	xOffset.y += Asura_Dialog::DefaultSpacing();

	xOffset.y -= (3 *Asura_Dialog::DefaultSpacing());
	xOffset.x +=21;

	pxDialog = new Asura_Numerical_Dialog("Near Plane Distance", xOffset, &s_pxSelected->m_xParams.m_fMinRange , 0.0f, 2500.0f, 0.01f, "%0.02f");
	pxOverlay->AddDialog( pxDialog );	
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Far Plane Distance", xOffset, &s_pxSelected->m_xParams.m_fMaxRange , 0.0f, 2500.0f, 0.01f, "%0.02f");
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Parallel Proj Scale", xOffset, &s_pxSelected->m_xParams.m_fScale , 0.1f, 2500.0f, 0.1f, "%0.02f");
	pxOverlay->AddDialog( pxDialog ); 
	xOffset.y += Asura_Dialog::LargeSpacing();

	// Parallel? - Scale (Parallel proj)
	pxDialog = new Asura_FlagTickbox_Dialog( "Parallel Projection", xOffset, &s_pxSelected->m_xParams.m_uProjectorParams_Flags, 1 << 2 );
	pxOverlay->AddDialog( pxDialog );
	xOffset.x -=21;

	// Movement Params
	pxDialog = new Asura_FlagTickbox_Dialog("Rotation", xOffset, &s_pxSelected->m_xParams.m_uProjectorParams_Flags, 1 << 3);
	pxOverlay->AddDialog( pxDialog );		

	xOffset.y += Asura_Dialog::LargeSpacing();

	for(u_int u = 0; u < 3; ++u)
	{
		char *aszLabels[3] = {"X","Y","Z"};
		
		float x = xOffset.x;

		pxDialog = new Asura_Dialog(aszLabels[u], xOffset);
		pxOverlay->AddDialog( pxDialog );
		xOffset.x+=1.0f;

		pxDialog = new Asura_AngleNumerical_Dialog("Speed", xOffset, &s_pxSelected->m_xParams.m_xRotSpeed[u] , -720.0f, 720.0f, 0.001f, "%0.03f");
		pxOverlay->AddDialog( pxDialog );
		xOffset.x+=20.0f;

		pxDialog = new Asura_FlagTickbox_Dialog("Swing", xOffset, &s_pxSelected->m_xParams.m_uProjectorParams_Flags, 1 << (u + 4));
		pxOverlay->AddDialog( pxDialog );
		xOffset.x+=9.0f;

		pxDialog = new Asura_AngleNumerical_Dialog("Range", xOffset, &s_pxSelected->m_xParams.m_xSwingDist[u] , -360.0f, 360.0f, 0.1f, "%0.01f");
		pxOverlay->AddDialog( pxDialog );
		xOffset.x = x;
		xOffset.y += Asura_Dialog::DefaultSpacing();
	}

	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_Numerical_Dialog("Rotation Offset (time in seconds)", xOffset, &s_pxSelected->m_xParams.m_fRotationOffset , -360.0f, 360.0f, 0.1f, "%0.01f");
	pxOverlay->AddDialog( pxDialog );


	xOffset.y += Asura_Dialog::LargeSpacing();
	pxDialog = new Asura_Function_Dialog("Attach To Object", xOffset, CreateAttachDlg );
	pxDialog->SetHelpText("Select object that this light should be attached to (If attached to an advanced light, it will inherit the colour from that light)");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Unattach", xOffset, UnattachLight );
	pxDialog->SetHelpText("Detach this light from whatever object it was attached to.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();



}

bool Tools_Entity_Projector::CanBeCopied( void ) const
{
    return (Type == TOT_Entity_Projector);
}

void Tools_Entity_Projector::Copy(const Tools_Object* pxObject)
{
	Tools_Entity_Object::Copy(pxObject);

	const Tools_Entity_Projector* pxFromObject = (const Tools_Entity_Projector*) pxObject;

	//copy between the tools objects
	m_xParams = pxFromObject->m_xParams;

	// and update the server entity
	Asura_ServerEntity_Projector* pxEntity = static_cast<Asura_ServerEntity_Projector*>( Asura_Entity_Server::FindEntity(GetGuid()) );
	if( !pxEntity ) { return; }
	pxEntity->SetParams( m_xParams );

	Orientation = m_xParams.m_xOrientation;	

}

void Tools_Entity_Projector::Create( void )
{
    Tools_Entity_Projector* pxToolsObj = CreateMain();
	if(pxToolsObj)
	{    
    	Tools_Object_System::SelectObject( pxToolsObj );
	}
}

Tools_Entity_Projector* Tools_Entity_Projector::CreateMain()
{
    const Asura_Guid uGUID = Asura_Entity_System::GetCreatedByToolsGuid();

    Tools_Entity_Data*                 pxToolsData = NULL;
    Asura_ServerEntity_Projector*      pxEntity    = NULL;
    Tools_Entity_Projector*			   pxToolsObj  = NULL;

    pxToolsData = CreateToolsDataWithDefaultName( "TextureProjector", "Projector", uGUID, GetNamePrefix() );
    if( !pxToolsData ) { goto CreateFailed; }

	pxEntity = Asura_ServerEntity_Projector::CreateFromTools( uGUID,pxToolsData->Position);
    pxEntity->ToolsData = pxToolsData;
    if( !pxEntity ) { goto CreateFailed; }

    pxToolsObj = new Tools_Entity_Projector( pxEntity );
    if( !pxToolsObj ) { goto CreateFailed; }

    Tools_SaveAsura::SetEntityModified();

    return pxToolsObj;

CreateFailed:

    if( !pxEntity ) { delete pxToolsObj; }
	if ( pxEntity )
	{
		pxEntity->PreDestructionShutdown();
	}
    delete pxEntity;
    delete pxToolsData;
	return 0;
}

void Tools_Entity_Projector::Create( Asura_Entity* pxEntity )
{
    Asura_Assert( AsuraEntityClass_Projector == pxEntity->GetClassification(), "" );
    new Tools_Entity_Projector( static_cast<Asura_ServerEntity_Projector*>(pxEntity) );
}

void Tools_Entity_Projector::OnDeselection( void )
{	
	PARENT::OnDeselection();

	if(s_bResetCurrentProjectorPtrOnDeselect)
	{
		s_pxSelected = NULL;
	}		

}

void Tools_Entity_Projector::OnSelection( void )
{
	s_pxSelected = this;

	PARENT::OnSelection();

	if(m_pxServerEntity)
	{				
		// Fill in the params
	}	
}

void Tools_Entity_Projector::Render( void )
{

	// Has the tools object moved?
	// Had to add this extra case to make it work with templates, as the updatewhileselected isn't called.
	Asura_ServerEntity_Projector* pxEntity = static_cast<Asura_ServerEntity_Projector*>( Asura_Entity_Server::FindEntity(GetGuid()) );
	if (pxEntity)
	{	
		if( m_xParams.m_xPosition != Position  ||
			m_xParams.m_xOrientation != Orientation)
		{
			m_xParams.m_xPosition = Position;
			m_xParams.m_xOrientation = Orientation;        
			pxEntity->SetParams( m_xParams );  		
		}
	}


    PARENT::Render();

	if( IsSelected() )
	{
		// PAZ - fix for obscure bug when creating an Advanced Volume Trigger, then selecting a projector (found by Matt Wilmot @ Derby)
		if( s_pxSelected != NULL )
		{
			// If attached to something, draw a line to the parent entity
			Tools_Object* pxMaster = Tools_Object_System::GetObjectFromGuid( s_pxSelected->m_xParams.m_uMasterEntity);
			if(pxMaster) 
			{
				Asura_Render::RenderLine( Position, pxMaster->Position, 0xff00ff00 );
			}
		}
	}

	if (m_pxServerEntity)
	{
		Asura_ClientEntity_Projector *pxClient = (Asura_ClientEntity_Projector *)Asura_ClientEntity_Projector::FindEntity(m_pxServerEntity->GetGuid());

		if (!pxClient)
		{
			// There's no client entity yet - create one	
			pxClient = Asura_ClientEntity_Projector::CreateFromTools(m_pxServerEntity->GetGuid(), m_pxServerEntity->GetParams());
		}

		if (pxClient)		
		{			
			pxClient->Update();
			
			pxClient->RenderFrustum();
		}
	}

    if( m_bShowVolume )
    {
		// Get the projector object and render it's frustum
    }
}

const char* Tools_Entity_Projector::GetTypeDescription( void )
{
	return "Texture Projector";
}



void Tools_Entity_Projector::UpdateWhileSelected( void )
{
    PARENT::UpdateWhileSelected();

	// Hacktastic mutually exclusive checkboxes...
	if (s_bFakeLightFlagChanged)
	{
		s_bFakeLightFlagChanged = false;
		
		if (m_xParams.m_bFakeLight)
			m_xParams.m_bDynamicLight = false;
	}

	if (s_bDynamicLightFlagChanged)
	{
		s_bDynamicLightFlagChanged = false;

		if (m_xParams.m_bDynamicLight)
			m_xParams.m_bFakeLight = false;
	}


    // Find corresponding entity.
	Asura_ServerEntity_Projector* pxEntity = static_cast<Asura_ServerEntity_Projector*>( Asura_Entity_Server::FindEntity(GetGuid()) );
	
    if( !pxEntity ) { return; }

    // Has the tools object moved?
    if( m_xParams.m_xPosition != Tools_Object_System::SelectedObject->Position  ||
		m_xParams.m_xOrientation != Tools_Object_System::SelectedObject->Orientation)
    {
        m_xParams.m_xPosition = Tools_Object_System::SelectedObject->Position;
		m_xParams.m_xOrientation = Tools_Object_System::SelectedObject->Orientation;        
    }
    
    // Update entity with new parameters
    pxEntity->SetParams( m_xParams );  		

   
}


void Tools_Entity_Projector::UpdateTextureNames()
{
	if (!s_pxSelected)
	{
		return;
	}

	if (s_pxSelected->m_xParams.m_xTextureName=="")
	{
		g_apxTextureNames[0]->UpdateLabel("(None)");
	}
	else
	{
		g_apxTextureNames[0]->UpdateLabel(s_pxSelected->m_xParams.m_xTextureName.GetString());
	}
/*
	if (s_pxSelected->m_xParams.m_xFalloffTextureName=="")
	{
		g_apxTextureNames[1]->UpdateLabel("(None)");
	}
	else
	{
		g_apxTextureNames[1]->UpdateLabel(s_pxSelected->m_xParams.m_xFalloffTextureName.GetString());
	}
*/
	if (s_pxSelected->m_xParams.m_xDetailTextureName=="")
	{
		g_apxTextureNames[2]->UpdateLabel("(None)");
	}
	else
	{
		g_apxTextureNames[2]->UpdateLabel(s_pxSelected->m_xParams.m_xDetailTextureName.GetString());
	}
}


void Tools_Entity_Projector::PickTexture_Projected(void)
{	
	int iCurrent = Asura_Texture_Management::GetIndexFromHashID(s_pxSelected->m_xParams.m_uTextureHash);
	int iSelected = Tools_PickTexture::PickTexture("Select Texture to be Projected",iCurrent,PICKTEXTURE_ALLTEXTURES);

	if(iSelected != ASURA_TEXTURE_NONE && iSelected != iCurrent)
	{				
		s_pxSelected->m_xParams.m_xTextureName = *Asura_Texture_Management::GetTextureName(iSelected);
		s_pxSelected->m_xParams.m_uTextureHash = Asura_Texture_Management::GetHashIDFromIndex(iSelected);		
	}
	UpdateTextureNames();
}

/*
void Tools_Entity_Projector::PickTexture_Falloff(void)
{	
	int iCurrent = Asura_Texture_Management::GetIndexFromHashID(s_pxSelected->m_xParams.m_uFalloffTextureHash);
	int iSelected = Tools_PickTexture::PickTexture("Select Falloff Texture (1D)",iCurrent,PICKTEXTURE_ALLTEXTURES);

	if(iSelected != ASURA_TEXTURE_NONE && iSelected != iCurrent)
	{		
		s_pxSelected->m_xParams.m_xFalloffTextureName = *Asura_Texture_Management::GetTextureName(iSelected);
		s_pxSelected->m_xParams.m_uFalloffTextureHash = Asura_Texture_Management::GetHashIDFromIndex(iSelected);		
	}
	UpdateTextureNames();
}
*/


void Tools_Entity_Projector::PickTexture_Detail(void)
{
	int iCurrent = Asura_Texture_Management::GetIndexFromHashID(s_pxSelected->m_xParams.m_uFalloffTextureHash);
	int iSelected = Tools_PickTexture::PickTexture("Select Detail Texture",iCurrent,PICKTEXTURE_ALLTEXTURES);

	if(iSelected != ASURA_TEXTURE_NONE && iSelected != iCurrent)
	{
		s_pxSelected->m_xParams.m_xDetailTextureName = *Asura_Texture_Management::GetTextureName(iSelected);
		s_pxSelected->m_xParams.m_uDetailTextureHash = Asura_Texture_Management::GetHashIDFromIndex(iSelected);		
	}
	UpdateTextureNames();
}

void Tools_Entity_Projector::ClearTexture_Detail(void)
{	
	s_pxSelected->m_xParams.m_xDetailTextureName = "";
	s_pxSelected->m_xParams.m_uDetailTextureHash = 0;
	UpdateTextureNames();
}

void Tools_Entity_Projector::ClearTexture_Projected(void)
{	
	s_pxSelected->m_xParams.m_xTextureName = "";
	s_pxSelected->m_xParams.m_uTextureHash = 0;		
	UpdateTextureNames();
}

/*
void Tools_Entity_Projector::ClearTexture_Falloff(void)
{	
	s_pxSelected->m_xParams.m_xFalloffTextureName = "";
	s_pxSelected->m_xParams.m_uFalloffTextureHash = 0;		
	UpdateTextureNames();
}
*/

Tools_Object* Tools_Entity_Projector::CreateObjectOfThisType() const
{
	return CreateMain();	
}


void Tools_Entity_Projector::AddFilesToExportList(List<Asura_String>& xNameList) const
{
	
	Asura_Texture_Management::AddTexture(m_xParams.m_xTextureName);
//	Asura_Texture_Management::AddTexture(m_xParams.m_xFalloffTextureName);	
	Asura_Texture_Management::AddTexture(m_xParams.m_xDetailTextureName);

	int iTex = Asura_Texture_Management::GetIndexFromHashID(m_xParams.m_uTextureHash);
	if (iTex!=ASURA_TEXTURE_NONE)
	{	
		Tools_Level_Export::AddTextureToExportList(iTex);
	}
/*
	iTex = Asura_Texture_Management::GetIndexFromHashID(m_xParams.m_uFalloffTextureHash);
	if (iTex!=ASURA_TEXTURE_NONE)
	{	
		Tools_Level_Export::AddTextureToExportList(iTex);
	}
*/
	iTex = Asura_Texture_Management::GetIndexFromHashID(m_xParams.m_uDetailTextureHash);
	if (iTex!=ASURA_TEXTURE_NONE)
	{	
		Tools_Level_Export::AddTextureToExportList(iTex);
	}
}






bool Tools_Entity_Projector::HandlesThisMessage(int iMessageID)
{
/*
	switch (iMessageID)
	{
	case SS2_MSG_CHANGE_WEATHER:
	case SS2_MSG_TRIGGER_LIGHTNING:
		return true;
	}
*/
	if (PARENT::HandlesThisMessage(iMessageID))
	{
		return true;
	}

	return false;
}






void Tools_Entity_Projector::CreateAttachDlg(void)
{
	Asura_Overlay* pxNewOverlay = new Asura_Overlay( "Attach To Object", false );
	Asura_Dialog* pxNewDialog;
	Asura_Vector_2 xOffset(1.0f,3.0f);

	s_bResetCurrentProjectorPtrOnDeselect = false;

	if(!s_pxSelected) return;
	Tools_Entity_Projector* pxObject = s_pxSelected;

	pxNewDialog = new Asura_Function_Dialog("Attach", xOffset, SetMasterObject );
	pxNewDialog->SetHelpText("Attaches this projector to the selected object.");
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxNewDialog = new Asura_Function_Dialog("Set Offset", xOffset, SetAttachOffset );
	pxNewDialog->SetHelpText("Projector should maintain the current offset from the object that it is attached to.");
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxNewDialog = new Asura_Function_Dialog("Set Marker", xOffset, SetAttachMarker );
	pxNewDialog->SetHelpText("Attaches the projector to a marker point on an animating object.");
	pxNewDialog->m_iControlID = DialogID_SelectMarker;
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.y+=Asura_Dialog::LargeSpacing();

	pxNewDialog = new Asura_FlagTickbox_Dialog( "Use Master Orient", xOffset, &pxObject->m_xParams.m_uProjectorParams_Flags, 1 << 9, &s_bModified );
	pxNewDialog->SetHelpText("Projector should use the orientation of the object/marker point that it is attached to.");
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxNewOverlay->m_uID = TOOLS_OVERLAY_PROJECTORATTACHMAIN;
	Asura_Overlay_System::SubmitNewOverlay(pxNewOverlay);	

	UpdateAttachDlg();
}

void Tools_Entity_Projector::UpdateAttachDlg()
{
	Asura_Overlay* pxOverlay = Asura_Overlay_System::GetOverlay(TOOLS_OVERLAY_PROJECTORATTACHMAIN);
	if(!pxOverlay) return;

	//get marker name
	Asura_String xName;
	GetAttachMarkerName(xName);

	char szText[200];
	sprintf(szText,"Marker : %s",xName.GetString());
	//update the marker point control
	pxOverlay->UpdateDialogLabel(DialogID_SelectMarker,szText);
}

void Tools_Entity_Projector::CreateSelectMasterDlg(void)
{
	Asura_Overlay* pxNewOverlay = new Asura_Overlay( "Select Master", false );
	Asura_Dialog* pxNewDialog;
	Asura_Vector_2 xOffset(1.0f,3.0f);

	pxNewDialog = new Asura_Function_Dialog("Attach", xOffset, SetMasterObject );
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxNewOverlay->m_uID = TOOLS_OVERLAY_PROJECTORATTACH;
	Asura_Overlay_System::SubmitNewOverlay(pxNewOverlay,false,false);	
}


void Tools_Entity_Projector::SetMasterObject(void)
{
	if(!s_pxSelected) 
	{
		Tools_Timed_Message_System::AddMessage( 0xff0000, "ERROR - lost light pointer!" );	
	}
	else
	{
		if( Tools_Object_System::SelectedObject->GetGuid() != s_pxSelected->GetGuid())		 
		{
			s_pxSelected->m_xParams.m_uMasterEntity = Tools_Object_System::SelectedObject->GetGuid();
		}
		else
		{
			Tools_Timed_Message_System::AddMessage( 0xff0000, "Can't attach projector to itself! Select a different object!" );
		}
	}	

	s_bResetCurrentProjectorPtrOnDeselect = true;
	Asura_Overlay* pxOverlay = Asura_Overlay_System::GetOverlay(TOOLS_OVERLAY_PROJECTORATTACH);
	if(pxOverlay)
	{
		s_bModified = true;
		pxOverlay->m_bShouldClose = true;
	}	

	Tools_Object_System::SelectObject(s_pxSelected);
}

void Tools_Entity_Projector::SetAttachOffset(void)
{
	if( NULL == s_pxSelected)
	{
		Tools_Timed_Message_System::AddMessage( 0xff0000, "ERROR - No Selected projector" );
		return;
	}
	
	Tools_Object* pxMaster = Tools_Object_System::GetObjectFromGuid( s_pxSelected->m_xParams.m_uMasterEntity);
	if(!pxMaster) 
	{
		Tools_Timed_Message_System::AddMessage( 0xff0000, "ERROR - Couldn't find master object" );
		return;
	}

	s_bModified = true;

	s_pxSelected->m_xParams.m_xOffsetFromMaster = s_pxSelected->Position - pxMaster->Position;

	// Todo - orientation!
}


void Tools_Entity_Projector::SetAttachMarker()
{
	if( NULL == s_pxSelected)
	{
		Tools_Timed_Message_System::AddMessage( 0xff0000, "ERROR - No Selected projector" );
		return;
	}

	Asura_ServerEntity_PhysicalObject* pxMaster = Asura_ServerEntity_PhysicalObject::FindEntity(s_pxSelected->m_xParams.m_uMasterEntity);
	if(!pxMaster) 
	{
		Tools_Timed_Message_System::AddMessage( 0xff0000, "ERROR - Couldn't find master object" );
		return;
	}

	//does the object use a skin?
	Asura_Hash_ID uSkin = pxMaster->GetSkinID();
	if(uSkin == ASURA_HASH_ID_UNSET)
	{
		Tools_Timed_Message_System::AddMessage( 0xff0000, "ERROR - Master object doesn't have a skin/hierarchy" );
		return;
	}

	//get the skin
	Asura_Hierarchy_Skin* pxSkin = Asura_Animation_System::GetSkinByHash(uSkin);
	if(!pxSkin)
	{
		Tools_Timed_Message_System::AddMessage(0xff0000,"Failed to find skin for master object. Unable to give list of marker points.");
		return;
	}

	if(!pxSkin->NumberOfMarkerPoints)
	{
		Tools_Timed_Message_System::AddMessage(0xff0000,"The master object's skin doesn't have any marker points");
		return;
	}

	//add all the marker points to a selection overlay
	ListSelection_Overlay xOverlay("Select marker point to attach projector to");
	for(int i=0;i<pxSkin->NumberOfMarkerPoints;i++)
	{
		xOverlay.AddItem(pxSkin->MarkerPoint[i].GetName().GetString(),Asura_GetHashID(pxSkin->MarkerPoint[i].GetName().GetString()));
	}
	//add option for clearing marker point
	xOverlay.AddItem("*None*",ASURA_HASH_ID_UNSET);
	if(!xOverlay.DoModal()) return;

	s_bModified = true;
	s_pxSelected->m_xParams.m_uMasterMarker = (Asura_Hash_ID)xOverlay.GetSelectionData();

	UpdateAttachDlg();

}

void Tools_Entity_Projector::GetAttachMarkerName(Asura_String& xName)
{
	if( NULL == s_pxSelected)
	{
		Tools_Timed_Message_System::AddMessage( 0xff0000, "ERROR - No Selected projector" );
		return;
	}

	Asura_Hash_ID uMarkerHash = s_pxSelected->m_xParams.m_uMasterMarker;
	if(!uMarkerHash)
	{
		xName = "*None*";
		return;
	}
	//default to "unknown"
	xName = "*Unknown*";


	Asura_ServerEntity_PhysicalObject* pxMaster = Asura_ServerEntity_PhysicalObject::FindEntity(s_pxSelected->m_xParams.m_uMasterEntity);
	if(!pxMaster) 
	{
		return;
	}

	//does the object use a skin?
	Asura_Hash_ID uSkin = pxMaster->GetSkinID();
	if(uSkin == ASURA_HASH_ID_UNSET)
	{
		return;
	}

	//get the skin
	Asura_Hierarchy_Skin* pxSkin = Asura_Animation_System::GetSkinByHash(uSkin);
	if(!pxSkin)
	{
		Tools_Timed_Message_System::AddMessage(0xff0000,"Failed to find skin for master object. Unable to give list of marker points.");
		return;
	}

	//check the marker points for one with the correct hash
	for(int i=0;i<pxSkin->NumberOfMarkerPoints;i++)
	{
		if(uMarkerHash == Asura_GetHashID(pxSkin->MarkerPoint[i].GetName().GetString()))
		{
			xName = pxSkin->MarkerPoint[i].GetName().GetString();
			return;
		}
	}
}

void Tools_Entity_Projector::UnattachLight(void)
{
	
	s_pxSelected->m_xParams.m_uMasterEntity     = ASURA_GUID_UNREGISTERED;
	s_pxSelected->m_xParams.m_xOffsetFromMaster = Asura_Maths::ZeroVector3;
	s_pxSelected->m_xParams.m_bUseMasterOrient  = true;
	s_pxSelected->m_xParams.m_uMasterMarker     = ASURA_HASH_ID_UNSET;

	s_bModified = true;
}


#endif //#ifdef ASURA_TARGET_SM3
