//
// Tools_Entity_ProxyTrigger_Object.cpp
//
// Created by Richard Rice - 21/09/2006
// 
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"
#include "Entity/Tools_Entity_ProxyTrigger_Object.h"

//asura
#include "Core/Asura_Timers.h"
#include "Entity/Asura_Entity_Classification.h"
#include "Entity/Asura_StaticMessage_System.h"
#include "Entity/Triggers/Asura_ServerEntity_ProxyTrigger.h"
#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Overlay.h"
#include "Network/Asura_Message.h"

//tools
#include "Entity/Tools_Message_Editing.h"
#include "Entity/Tools_SecondaryEntityFile_System.h"
#include "Interface/Tools_ContextMenu_System.h"
#include "Load&Save/Tools_SaveAsura.h"
#include "Misc/Tools_Modal_Overlay.h"
#include "Objects/Tools_Object_System.h"


///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*----------------------------------------------**
** Tools_ChooseObject_ProxyTriggerMessageTarget **
**----------------------------------------------*/

class Tools_ChooseObject_ProxyTriggerMessageTarget : public Tools_ChooseObject
{
public :
	Tools_ChooseObject_ProxyTriggerMessageTarget(Asura_Guid uProxyGuid,int iTargetClass,Asura_Message_ProxyTrigger_Target* pxMessageData)
	{
		m_iTargetClass = iTargetClass;
		m_pxMessageData = pxMessageData;
		m_uProxyGuid = uProxyGuid;
	}

	virtual bool CanBeSelected(Tools_Object* pxObject);
	virtual void ObjectSelected(Tools_Object* pxObject);
	virtual void OnFinish(){delete this;}

	int m_iTargetClass;
	Asura_Message_ProxyTrigger_Target* m_pxMessageData;
	Asura_Guid m_uProxyGuid;
};

bool Tools_ChooseObject_ProxyTriggerMessageTarget::CanBeSelected(Tools_Object* pxObject)
{
	//Only allow entity objects of the correct class
	if(!pxObject->IsEntityObject()) return false;
	Tools_Entity_Object* pxEntityObject = (Tools_Entity_Object*) pxObject;

	if(m_uProxyGuid == pxEntityObject->GetGuid()) return false;
	if(m_iTargetClass == AsuraEntityClass_NotSet) return true;

	return (pxEntityObject->GetEntityClass() == m_iTargetClass);
}

void Tools_ChooseObject_ProxyTriggerMessageTarget::ObjectSelected(Tools_Object* pxObject)
{
	if(pxObject)
	{
		if(!pxObject->IsEntityObject()) return;
		Tools_Entity_Object* pxEntityObject = (Tools_Entity_Object*) pxObject;

		m_pxMessageData->m_uTargetGuid = pxEntityObject->GetGuid();

	}
	else
	{
		//If no object is selected , treat it as clearing the guid
		m_pxMessageData->m_uTargetGuid = ASURA_GUID_UNREGISTERED;
	}

	Tools_Message_Editing::UpdateMessageOverlay();
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*------------------------------------------**
** Tools_ChooseObject_ProxyTriggerAddTarget **
**------------------------------------------*/

class Tools_ChooseObject_ProxyTriggerAddTarget : public Tools_ChooseObject
{
public :
	Tools_ChooseObject_ProxyTriggerAddTarget(int iTargetClass,Tools_Entity_ProxyTrigger_Object* pxProxyObject,bool bAlterExistingTargets)
	{
		m_iTargetClass = iTargetClass;
		m_pxProxyObject = pxProxyObject;
		m_bAlterExistingTargets = bAlterExistingTargets;
		m_bHideExistingTargets = false;
	}

	virtual bool SupportMultipleSelection(){return true;}
	virtual bool CanBeSelected(Tools_Object* pxObject);
	virtual void ObjectSelected(Tools_Object* pxObject);
	virtual void OnFinish(){delete this;}
	virtual void OnStart();

	void SetHideExistingTargets(bool bSet) {m_bHideExistingTargets = bSet;}

	int m_iTargetClass;
	Tools_Entity_ProxyTrigger_Object* m_pxProxyObject;
	bool m_bAlterExistingTargets;
	bool m_bHideExistingTargets;
};

bool Tools_ChooseObject_ProxyTriggerAddTarget::CanBeSelected(Tools_Object* pxObject)
{
	//Only allow entity objects of the correct class
	if(!pxObject->IsEntityObject()) return false;
	Tools_Entity_Object* pxEntityObject = (Tools_Entity_Object*) pxObject;

	if(pxEntityObject->GetGuid() == m_pxProxyObject->GetGuid()) return false;

	if(m_iTargetClass != AsuraEntityClass_NotSet)
	{
		if(pxEntityObject->GetEntityClass() != m_iTargetClass) return false;
	}

	if(m_bHideExistingTargets && !m_bAlterExistingTargets)
	{
		//Hide existing objects by preventing them from being selectable
		if(m_pxProxyObject->IsInTargetList(pxEntityObject->GetGuid()))
		{
			return false;
		}
	}

	return true;
}

void Tools_ChooseObject_ProxyTriggerAddTarget::ObjectSelected(Tools_Object* pxObject)
{
	m_pxProxyObject->AddTarget_TargetChosen(GetSelectionList(), m_bAlterExistingTargets);
}

void Tools_ChooseObject_ProxyTriggerAddTarget::OnStart()
{
	if(!m_bAlterExistingTargets) return;

	//select all the current targets
	List<Tools_Object*> xTargetList;
	m_pxProxyObject->GetTargetList(xTargetList);
	
	while(xTargetList.size())
	{
		SelectObject(xTargetList.first_entry(),true);
		xTargetList.delete_first_entry();
	}
}


///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*----------------------------------**
** Tools_Entity_ProxyTrigger_Object **
**----------------------------------*/

Tools_Entity_ProxyTrigger_Object::Tools_Entity_ProxyTrigger_Object(Asura_Entity* pxEntity)
:Tools_Entity_Object(pxEntity)
{
	Type = TOT_Entity_ProxyTrigger;
	m_pxTargetObject = 0;
	m_uFindTargetFrameStamp = 0xffffffff;

	m_iListSelection = 0;
	m_iListSize = 0;
	m_pszListStrings = 0;
	m_pxListDialog = 0;

	m_xTypeDescription = "Proxy Trigger - ";

	SetMesh("ToolsShape_Proxy");
}

Tools_Entity_ProxyTrigger_Object::~Tools_Entity_ProxyTrigger_Object()
{
	if(m_pxTargetObject)
	{
		m_pxTargetObject->RemoveReference(this);
		m_pxTargetObject = 0;
	}

	DeleteListStrings();
}

void Tools_Entity_ProxyTrigger_Object::Create()
{
	//Ask which entity type this should be a proxy for.
	ListSelection_Overlay xOverlay("Create what type of proxy?");
	List<int> xClassesAdded;

	//use 'not set' for proxies that can be used on any type of entity
	xOverlay.AddItem("*All Entity Types*", AsuraEntityClass_NotSet);
	
	for(Asura_Hashed_List_It<Tools_Entity_Object> xIt(Tools_Entity_Object::GetList());!xIt.Done();xIt.Next())
	{
		Tools_Entity_Object* pxEntityObject = xIt.GetCurrent();

		int iClass = pxEntityObject->GetEntityClass();
		if(!xClassesAdded.contains(iClass))
		{
			if(iClass == AsuraEntityClass_ProxyTrigger) continue;//avoid proxies for proxies for now I think
			if(pxEntityObject->Type == TOT_Entity_TemplateMaster) continue;//Doesn't make sense to choose the template master
			xClassesAdded.add_entry(iClass);

			xOverlay.AddItem(pxEntityObject->GetTypeDescriptionForObjectType(),iClass);
		}
	}

	if(!xOverlay.GetNumItems())
	{
		ShowMessageOverlay("Can't create a proxy trigger until some suitable entities have been created that it can be a proxy for.");
		return;
	}

	if(!xOverlay.DoModal()) return;

	//go ahead and create the object
	Tools_Entity_ProxyTrigger_Object* pxObject = Create(false,xOverlay.GetSelectionData());

	if (pxObject)
	{
		pxObject->m_xTypeDescription += xOverlay.GetSelectionString();
	}

	Tools_Object_System::SelectObject(pxObject);
}


Tools_Entity_ProxyTrigger_Object* Tools_Entity_ProxyTrigger_Object::Create(bool bGenerateName,int iTargetClass)
{
	//get a name , and the position data
	Asura_Guid uGuid = Asura_Entity_System::GetNextToolsCreatedGuid();
	Tools_Entity_Data* pxToolsData = 0;
	if(bGenerateName)
	{
		pxToolsData = Tools_Entity_Object::CreateToolsDataWithDefaultName("Proxy Trigger","Proxy Trigger",uGuid,GetNamePrefix());
	}
	else
	{
		pxToolsData = Tools_Entity_Object::CreateToolsData("Proxy Trigger",0,GetNamePrefix());
	}
	if(!pxToolsData) return 0 ;

	//create the asura data
	Asura_ServerEntity_ProxyTrigger* pxAsuraEntity = new Asura_ServerEntity_ProxyTrigger();
	pxAsuraEntity->ToolsData = pxToolsData;
	pxAsuraEntity->m_iTargetClass = iTargetClass;

	//create the tools object
	Tools_Entity_ProxyTrigger_Object* pxObject = new Tools_Entity_ProxyTrigger_Object(pxAsuraEntity);

	Tools_SaveAsura::SetEntityModified();

	return pxObject;

}


void Tools_Entity_ProxyTrigger_Object::Copy(const Tools_Object* pxObject)
{
	Tools_Entity_Object::Copy(pxObject);

	const Tools_Entity_ProxyTrigger_Object* pxFromObject = (const Tools_Entity_ProxyTrigger_Object*) pxObject;

	Asura_ServerEntity_ProxyTrigger* pxFromEntity = (Asura_ServerEntity_ProxyTrigger*) pxFromObject->Entity;
	Asura_ServerEntity_ProxyTrigger* pxToEntity = (Asura_ServerEntity_ProxyTrigger*) Entity;

	
	//copy between the entities
	pxToEntity->m_iTargetClass = pxFromEntity->m_iTargetClass;
	pxToEntity->m_uProxyFlags = pxFromEntity->m_uProxyFlags;
	pxToEntity->m_xTargetList = pxFromEntity->m_xTargetList;

}

Tools_Object* Tools_Entity_ProxyTrigger_Object::CreateObjectOfThisType() const
{
	return Create(true,AsuraEntityClass_NotSet);
}


bool Tools_Entity_ProxyTrigger_Object::HandlesThisMessage(int iMessageID)
{
	if(iMessageID == ASURA_MSG_PROXY_ACTIVATE ||
		iMessageID == ASURA_MSG_PROXY_DEACTIVATE ||
		iMessageID == ASURA_MSG_PROXY_ADDTARGET ||
		iMessageID == ASURA_MSG_PROXY_REMOVETARGET ||
		iMessageID == ASURA_MSG_PROXY_SETTARGET)
	{
		return true;
	}

	Asura_ServerEntity_ProxyTrigger* pxProxy = (Asura_ServerEntity_ProxyTrigger*) Entity;

	if(pxProxy->m_iTargetClass == AsuraEntityClass_NotSet)
	{
		//Proxy can target any entity , so just allow a limited set of messages for now
		//Could in theory allow everything ,but certain message types can't be edited without a valid target ,
		//and it would increase the chance of entities being sent inappropriate messages

		if(iMessageID == ASURA_MSG_DESTRUCT ||
		   iMessageID == ASURA_MSG_ACTIVATE ||
		   iMessageID == ASURA_MSG_DEACTIVATE ||
		   iMessageID == ASURA_MSG_TRIGGER ||
		   iMessageID == ASURA_MSG_UNTRIGGER)
		{
			return true;
		}
		else
		{
			return false;
		}
		
	}
	
	
	if(!m_pxTargetObject) FindTargetObject();
	if(m_pxTargetObject) return m_pxTargetObject->HandlesThisMessage(iMessageID);
	return false;
}



void Tools_Entity_ProxyTrigger_Object::AddOverlayControls(class Asura_Overlay* pxOverlay,class Asura_Vector_2& xOffset)
{
	Tools_Entity_Object::AddOverlayControls(pxOverlay, xOffset);

	BuildListStrings();
	
	Asura_Dialog* pxDialog;

	xOffset.y += Asura_Dialog::LargeSpacing() - Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Dialog("Initial targets :",xOffset);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::SmallSpacing();

	pxDialog = m_pxListDialog = new Asura_List_Dialog(xOffset,m_pszListStrings,&m_iListSelection,8,m_iListSize,0);
	m_pxListDialog->EnableMultipleSelection(true);
	pxDialog->SetHelpText("Initial targets");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += 1.2f* 8.0f;

	pxDialog = new Asura_Function_Dialog("Add",xOffset,AddTarget,(int)this);
	pxDialog->SetHelpText("Add entities to the intitial target list");
	pxOverlay->AddDialog(pxDialog);

	xOffset.x = pxDialog->m_xMaxCoord.x + 1.0f;
	pxDialog = new Asura_Function_Dialog("Remove",xOffset,RemoveTarget,(int)this);
	pxDialog->SetIsEnabledFunction(CanRemoveTarget,(int) this);
	pxDialog->SetHelpText("Remove the selected entity from the target list");
	pxOverlay->AddDialog(pxDialog);
	xOffset.x = 1.0f;
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Add (Hide Existing)",xOffset,AddTargetHideExisting,(int)this);
	pxDialog->SetHelpText("Add entities to the intitial target list.\nEntities that are already in the target list will be hidden.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();


	pxDialog = new Asura_Function_Dialog("Edit Target List",xOffset,EditTargets,(int)this);
	pxDialog->SetHelpText("Similar to 'add' except that the current target list is replaced by the new selection.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();

	pxDialog = new Asura_Function_Dialog("Select Targets",xOffset,SelectTargets,(int)this);
	pxDialog->SetHelpText("Replaces the current selection with the objects in the target list.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	

}

void Tools_Entity_ProxyTrigger_Object::BuildListStrings()
{
	DeleteListStrings();
	Asura_ServerEntity_ProxyTrigger* pxProxy = (Asura_ServerEntity_ProxyTrigger*) Entity;

	m_iListSize = pxProxy->m_xTargetList.Size();

	if(m_iListSize)
	{
		m_pszListStrings = new char*[m_iListSize];

		//Get a list of all the target objects
		int iIndex = 0;
		for(const Asura_ListElt<Asura_Guid>* pxIt = pxProxy->m_xTargetList.Front(); pxIt; pxIt = pxIt->Next(),iIndex++)
		{
			Tools_Entity_Object* pxObject = Tools_Entity_Object::FindEntity(pxIt->Data());		
			if(pxObject)
			{
				const char* szName = pxObject->GetName();
				m_pszListStrings[iIndex] = new char[strlen(szName)+1];
				strcpy(m_pszListStrings[iIndex],szName);
			}
			else
			{
				m_pszListStrings[iIndex] = new char[30];
				Asura_PrintfToBfr(m_pszListStrings[iIndex],"Guid %d",30,true,pxIt->Data());
			}
		}
		
	}
	else
	{
		m_iListSize = 1;
		m_pszListStrings = new char*[1];
		m_pszListStrings[0] = new char[20];
		strcpy(m_pszListStrings[0],"*No Targets*");
	}

	m_iListSelection = Asura_Maths::ClampToRange(m_iListSelection,0,m_iListSize-1);
}

void Tools_Entity_ProxyTrigger_Object::DeleteListStrings()
{
	for(int i=0;i<m_iListSize;i++)
	{
		delete [] m_pszListStrings[i];
	}
	delete [] m_pszListStrings;

	m_pszListStrings = 0;
	m_iListSize = 0;
}

void Tools_Entity_ProxyTrigger_Object::UpdateList()
{
	if(IsSelected() && m_pxListDialog)
	{
		BuildListStrings();
		m_pxListDialog->ReplaceStrings(m_iListSize,m_pszListStrings);
	}
	
}



void Tools_Entity_ProxyTrigger_Object::AddTarget()
{
	Asura_ServerEntity_ProxyTrigger* pxProxy = (Asura_ServerEntity_ProxyTrigger*) Entity;

	//Start the object choosing
	Tools_ChooseObject_ProxyTriggerAddTarget* pxChooseObject = new Tools_ChooseObject_ProxyTriggerAddTarget(pxProxy->m_iTargetClass,this,false);
	Tools_Object_System::ChooseObject("Select entities to add as initial targets",pxChooseObject);
}

void Tools_Entity_ProxyTrigger_Object::AddTargetHideExisting()
{
	Asura_ServerEntity_ProxyTrigger* pxProxy = (Asura_ServerEntity_ProxyTrigger*) Entity;

	//Start the object choosing
	Tools_ChooseObject_ProxyTriggerAddTarget* pxChooseObject = new Tools_ChooseObject_ProxyTriggerAddTarget(pxProxy->m_iTargetClass,this,false);
	pxChooseObject->SetHideExistingTargets(true);
	Tools_Object_System::ChooseObject("Select entities to add as initial targets",pxChooseObject);
}


void Tools_Entity_ProxyTrigger_Object::EditTargets()
{
	Asura_ServerEntity_ProxyTrigger* pxProxy = (Asura_ServerEntity_ProxyTrigger*) Entity;

	//Start the object choosing
	Tools_ChooseObject_ProxyTriggerAddTarget* pxChooseObject = new Tools_ChooseObject_ProxyTriggerAddTarget(pxProxy->m_iTargetClass,this,true);
	Tools_Object_System::ChooseObject("Edit list of entities to use as initial targets",pxChooseObject);
}


void Tools_Entity_ProxyTrigger_Object::AddTarget_TargetChosen(const List<Tools_Object*>* pxObjectList, bool bAlterExistingTargets)
{
	Asura_ServerEntity_ProxyTrigger* pxProxy = (Asura_ServerEntity_ProxyTrigger*) Entity;

	bool bAnyChange = false;

	if(bAlterExistingTargets)
	{
		//clear out the previous selection.
		//(this will result in things being flagged as changed if the same list has been selected again , but tough)
		if(pxProxy->m_xTargetList.Size())
		{
			pxProxy->m_xTargetList.Clear();
			bAnyChange = true;
		}

	}

	int iDisallowedTargets = 0;
	for(CLIF<Tools_Object*> xObjectLIF(pxObjectList); !xObjectLIF.done(); xObjectLIF.next())
	{
		Asura_Guid uGuid = xObjectLIF()->GetGuid();
		if(uGuid == ASURA_GUID_UNREGISTERED) continue;

		if(!Tools_SecondaryEntityFile_System::IsAllowableTarget(this, xObjectLIF(), false))
		{
			iDisallowedTargets++;
			continue;
		}


		if(!pxProxy->m_xTargetList.Find(uGuid))
		{
			pxProxy->m_xTargetList.Push_Back(uGuid);
			bAnyChange = true;
		}
	}

	if(bAnyChange)
	{
		SetEntityModified();

		//need to update the list on the overlay
		UpdateList();

		//re-find the target object used for message descriptions etc , in case a new one is more suitable now
		FindTargetObject(true);
	}

	if(iDisallowedTargets)
	{
		ShowMessageOverlay("%d object(s) were excluded because they belong to secondary entity files and have not been saved to the main entity file at least once.",iDisallowedTargets);
	}
}

static int SortSelection(const void* p1, const void* p2)
{
	return *(const int*)p1 - *(const int*)p2;
}

void Tools_Entity_ProxyTrigger_Object::RemoveTarget()
{
	Asura_ServerEntity_ProxyTrigger* pxProxy = (Asura_ServerEntity_ProxyTrigger*) Entity;

	if(!pxProxy->m_xTargetList.Size()) return;

	//get a sorted list of the selection
	int iNumSelected = m_pxListDialog->GetNumSelected();
	if(!m_pxListDialog->GetNumSelected()) return;
	int* piSelection = new int[iNumSelected];

	for(int i=0;i<iNumSelected;i++)
	{
		piSelection[i] = m_pxListDialog->GetSelection(i);
	}

	qsort(piSelection, iNumSelected, sizeof(int), SortSelection);



	int iListIndex = 0;
	int iSelIndex = 0;
	for(Asura_ListElt<Asura_Guid>* pxIt = pxProxy->m_xTargetList.Front(); pxIt;)
	{
		
		if(iListIndex == piSelection[iSelIndex])
		{
			Asura_ListElt<Asura_Guid>* pxElementToDelete = pxIt;
			pxIt = pxIt->Next();

			//reached the entry being deleted
			pxProxy->m_xTargetList.Remove(pxElementToDelete);
			iSelIndex++;
		}
		else
		{
			pxIt = pxIt->Next();
		}
		iListIndex++;
	}
	Asura_Assert(iNumSelected == iSelIndex, "");
	delete [] piSelection;

	SetEntityModified();

	//need to update the list on the overlay
	UpdateList();

	//re-find the target object used for message descriptions etc , in case a new one is more suitable now
	FindTargetObject(true);
	
}

bool Tools_Entity_ProxyTrigger_Object::CanRemoveTarget()
{
	Asura_ServerEntity_ProxyTrigger* pxProxy = (Asura_ServerEntity_ProxyTrigger*) Entity;
	return m_pxListDialog->GetNumSelected() && pxProxy->m_xTargetList.Size();
}


void Tools_Entity_ProxyTrigger_Object::AddMessageOverlayControls(struct Asura_StaticMessage* pxMessage,class Asura_Overlay* pxOverlay,class Asura_Vector_2& xOffset)
{
	Asura_Assert(pxMessage->m_pData,"");
	Asura_Message_ProxyTrigger_Target* pxData = (Asura_Message_ProxyTrigger_Target*) pxMessage->m_pData;

	Asura_Dialog* pxDialog = 0;


	//Find the name of the current target in the message
	const char* szName;
	char szText[100];
	if(pxData->m_uTargetGuid != ASURA_GUID_UNREGISTERED)
	{
		Tools_Entity_Object* pxObject = Tools_Entity_Object::FindEntity(pxData->m_uTargetGuid);
		if(pxObject)
		{
			szName = pxObject->GetName();
		}
		else
		{
			szName = "*Not Found*";
		}
	}
	else
	{
		szName = "*None*";
	}

	Asura_PrintfToBfr(szText,"Entity : %s",100,true,szName);
	pxDialog = new Asura_Function_Dialog(szText,xOffset,EditProxyMessageTarget,(int)pxMessage);
	pxDialog->SetHelpText("Select the entity that should be added/removed from the target list of the proxy trigger");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();
}

void Tools_Entity_ProxyTrigger_Object::EditProxyMessageTarget(int iMessage)
{
	Asura_StaticMessage* pxMessage = (Asura_StaticMessage*) iMessage;
	Asura_Message_ProxyTrigger_Target* pxData = (Asura_Message_ProxyTrigger_Target*) pxMessage->m_pData;

	//find the target proxy to determine which classes the message can target
	Tools_Entity_Object* pxEntityObject = Tools_Entity_Object::FindEntity(pxMessage->m_uSendTo);
	if(!pxEntityObject || pxEntityObject->Type != TOT_Entity_ProxyTrigger)
	{
		Asura_Assert(0,"Failed to find proxy trigger");
		return;
	}

	Asura_ServerEntity_ProxyTrigger* pxProxy = (Asura_ServerEntity_ProxyTrigger*) pxEntityObject->Entity;

	//start the object choosing 'sub mode'
	Tools_Object* pxInitialSelection = Tools_Object_System::GetObjectFromGuid(pxData->m_uTargetGuid);

	Tools_ChooseObject_ProxyTriggerMessageTarget* pxChooseObject = new Tools_ChooseObject_ProxyTriggerMessageTarget(pxProxy->GetGuid(),pxProxy->m_iTargetClass,pxData);
	Tools_Object_System::ChooseObject("Select entity to be added/removed from the proxy trigger",pxChooseObject,pxInitialSelection);
}

void Tools_Entity_ProxyTrigger_Object::FindTargetObject(bool bIgnoreFrameStamp /*=false*/)
{
	//Don't search if we've already done so this frame
	//This is mainly to prevent multiple searches for each call to HandlesThisMessage() should there
	//not be any suitable target
	u_int uFrame = Asura_Timers::GetFrameNumber();
	if(uFrame == m_uFindTargetFrameStamp && !bIgnoreFrameStamp) return;
	m_uFindTargetFrameStamp = uFrame;

	if(m_pxTargetObject)
	{
		m_pxTargetObject->RemoveReference(this);
		m_pxTargetObject = 0;
	}

	Asura_ServerEntity_ProxyTrigger* pxProxy = (Asura_ServerEntity_ProxyTrigger*) Entity;

	if(pxProxy->m_iTargetClass == AsuraEntityClass_NotSet)
	{
		//don't bother with a target object for proxies that can target all entities
		return;
	}

	//Try objects in the intial target list first

	for(const Asura_ListElt<Asura_Guid>* pxIt = pxProxy->m_xTargetList.Front(); pxIt; pxIt = pxIt->Next())
	{
		Tools_Entity_Object* pxObject = Tools_Entity_Object::FindEntity(pxIt->Data());		
		if(!pxObject) continue;

		//make sure the found object is actually of the right type
		if(pxObject->GetEntityClass() != pxProxy->m_iTargetClass) continue;

		m_pxTargetObject = pxObject;
		m_pxTargetObject->AddReference(this);
		return;
	}

	//Just search for any entity of the correct type then
	for(Asura_Hashed_List_It<Tools_Entity_Object> xIt(Tools_Entity_Object::GetList());!xIt.Done();xIt.Next())
	{
		Tools_Entity_Object* pxObject = xIt.GetCurrent();
		if(pxObject->GetEntityClass() == pxProxy->m_iTargetClass)
		{
			m_pxTargetObject = pxObject;
			m_pxTargetObject->AddReference(this);
			return;
		}		
	}
}

void Tools_Entity_ProxyTrigger_Object::ReferencedObjectIsBeingDeleted(Tools_Object* pxObject)
{
	Tools_Entity_Object::ReferencedObjectIsBeingDeleted(pxObject);
	
	if(pxObject == m_pxTargetObject)
	{
		//Target object is no more. We will need to search for another one later.
		m_pxTargetObject = 0;
		m_uFindTargetFrameStamp = 0xffffffff;
	}
}

bool Tools_Entity_ProxyTrigger_Object::IsInTargetList(Asura_Guid uGuid) const
{
	Asura_ServerEntity_ProxyTrigger* pxProxy = (Asura_ServerEntity_ProxyTrigger*) Entity;

	return pxProxy->m_xTargetList.Find(uGuid) != 0;
	
}

void Tools_Entity_ProxyTrigger_Object::GetTargetList(List<Tools_Object*>& xOutputList) const
{
	Asura_ServerEntity_ProxyTrigger* pxProxy = (Asura_ServerEntity_ProxyTrigger*) Entity;

	for(const Asura_ListElt<Asura_Guid>* pxIt = pxProxy->m_xTargetList.Front(); pxIt; pxIt = pxIt->Next())
	{
		Tools_Entity_Object* pxObject = Tools_Entity_Object::FindEntity(pxIt->Data());		
		if(pxObject)
		{
			xOutputList.add_entry(pxObject);
		}
	}
}


void Tools_Entity_ProxyTrigger_Object::SelectTargets()
{
	SelectTargets(false);
}


void Tools_Entity_ProxyTrigger_Object::SelectTargets(bool bAddToSelection) const
{
	List<Tools_Object*> xTargets;
	GetTargetList(xTargets);

	Tools_Object_System::SelectObjects(xTargets, bAddToSelection);
}

void Tools_Entity_ProxyTrigger_Object::DeselectTargets() const
{
	List<Tools_Object*> xTargets;
	GetTargetList(xTargets);

	Tools_Object_System::DeselectObjects(xTargets);
}



void Tools_Entity_ProxyTrigger_Object::AddContextMenuItems(class Tools_ContextMenu& xMenu, const class Tools_ContextMenu_Context& xContext)
{

	Tools_ContextMenu_Item* pxItem;
	Tools_ContextMenu_Section* pxSection = xMenu.GetSection(TOOLS_CONTEXTMENU_SECTION_SELECTION);
	pxSection->AddSeparator();


	//select/deselect proxy targets
	if(xContext.m_bShiftHeld)
	{
		pxItem = new Tools_ContextMenu_Item_Function("Deselect proxy targets", ContextMenu_DeselectProxyTargets, this);
		pxItem->SetID("Select proxy targets");
		pxSection->AddItem(pxItem);
	}
	else if(xContext.m_bControlHeld)
	{
		pxItem = new Tools_ContextMenu_Item_Function("Select proxy targets", ContextMenu_SelectProxyTargets_Add, this);
		pxItem->SetID("Select proxy targets");
		pxSection->AddItem(pxItem);
	}
	else
	{
		pxItem = new Tools_ContextMenu_Item_Function("Select proxy targets", ContextMenu_SelectProxyTargets_Replace, this);
		pxItem->SetID("Select proxy targets");
		pxSection->AddItem(pxItem);
	}
}

void Tools_Entity_ProxyTrigger_Object::ContextMenu_SelectProxyTargets_Add(void* pObject)
{
	Tools_Entity_ProxyTrigger_Object* pxObject = (Tools_Entity_ProxyTrigger_Object*) pObject;
	pxObject->SelectTargets(true);
}

void Tools_Entity_ProxyTrigger_Object::ContextMenu_SelectProxyTargets_Replace(void* pObject)
{
	Tools_Entity_ProxyTrigger_Object* pxObject = (Tools_Entity_ProxyTrigger_Object*) pObject;
	pxObject->SelectTargets(false);
}

void Tools_Entity_ProxyTrigger_Object::ContextMenu_DeselectProxyTargets(void* pObject)
{
	Tools_Entity_ProxyTrigger_Object* pxObject = (Tools_Entity_ProxyTrigger_Object*) pObject;
	pxObject->DeselectTargets();
}

