//
// Tools_Entity_Simulation.cpp
//
// Created by Richard Rice - 20/10/2004
// 
// Copyright (C) 2004 Rebellion, All Rights Reserved.
//


#include "Core/Asura.h"
#include "Entity/Tools_Entity_Simulation.h"

#include "Core/Asura_Timers.h"
#include "Cutscene/Asura_Cutscene_System.h"
#include "Cutscene/Asura_Cutscene.h"
#include "Cutscene/Asura_Cutscene_Actor.h"
#include "Debugging/Asura_Logfile.h"
#include "Entity/Asura_Entity.h"
#include "Entity/EntityTemplate/Asura_ServerEntity_Template.h"
#include "Entity/Asura_StaticMessage_System.h"
#ifdef ASURA_SUPPORT_ENTITYWATCH
#include "Entity/Asura_EntityWatch_System.h"
#endif
#ifdef ASURA_SUPPORT_COMMAND_CONSOLE
#include "Interface/Asura_CommandConsole.h"
#endif
#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Overlay.h"
#include "Interface/Asura_Overlay_System.h"
#include "Network/Asura_Message.h"
#include "Rendering/Asura_Viewport.h"
#include "Sound/Asura_Music.h"
#include "Sound/Asura_MetaMusic_System.h"
#ifdef ASURA_USE_DIALOGUE_SYSTEM
#include "Sound/Asura_Dialogue.h"
#endif

#include "Tools_Overlays.h"
#include "Entity/Tools_Entity_Data.h"
#include "Entity/Tools_Message_Editing.h"
#include "Entity/Tools_SecondaryEntityFile_System.h"
#include "Misc/Tools_Timed_Messages.h"
#include "Objects/Tools_Object_System.h"
#include "Objects/Tools_Object.h"
#include "PC/OS/Tools_OS.h"
#include "Tools_System.h"



//////////////////////////////////////////////////////////////////////////////////////////////
struct Tools_Logged_Message
{
	Asura_Guid m_uFrom;
	Asura_Guid m_uTo;
	u_int m_uMessage;
	Asura_String m_xDesc;
	Asura_String m_xHelpText;
	bool m_bVisible;
};

static List<Tools_Logged_Message*> g_xMessageList;

static const char* g_ppszDefaultMessageText[1] =
{
	"*None*"
};

//////////////////////////////////////////////////////////////////////////////////////////////

bool Tools_Entity_Simulation::s_bActive = false;
bool Tools_Entity_Simulation::s_bEnabled = true;
bool Tools_Entity_Simulation::s_bPaused = false;
float Tools_Entity_Simulation::s_fTimer = 0.0f;
bool Tools_Entity_Simulation::s_bOverlayOpen = false;

bool Tools_Entity_Simulation::s_bShowMessages = false;

bool Tools_Entity_Simulation::s_bMessageListBoxValid = false;

int Tools_Entity_Simulation::s_iMessageListBoxSize = 0;
char** Tools_Entity_Simulation::s_ppszMessageStrings = 0;
char** Tools_Entity_Simulation::s_ppszMessageHelp = 0;
int* Tools_Entity_Simulation::s_piMessageIndeces = 0;
int Tools_Entity_Simulation::s_iSelectedMessage = 0;

bool Tools_Entity_Simulation::s_bMessageDatFileNeedsReset = true;

bool Tools_Entity_Simulation::s_bAllowTemplateMemberSelection = false;

Asura_List_Dialog* Tools_Entity_Simulation::s_pxListDialog = 0;

Tools_Entity_Simulation::FilterType Tools_Entity_Simulation::s_eFilter = Tools_Entity_Simulation::MESSAGEFILTER_NONE;
int Tools_Entity_Simulation::s_iFilter_MessageIndex;
List<Asura_Guid> Tools_Entity_Simulation::s_xFilter_GuidList;

int Tools_Entity_Simulation::s_iMessageLogIndent = -1;
List<Asura_Hash_ID> Tools_Entity_Simulation::s_xMessageLogFilter;
bool Tools_Entity_Simulation::s_bMessageLogFilter_TemplateMessages = false;

const char* g_pszMessageDatFilename = "Logfiles/MessageLog.dat";

void Tools_Entity_Simulation::Start()
{
	#if 0
	if(s_bActive) return;
	s_bActive = true;
	s_bEnabled = true;
	s_fTimer = 0;
	s_eFilter = MESSAGEFILTER_NONE;

	Asura_ClientCutscene_Actor_Camera::Disable = true;

	//initialise the entities
	for(Asura_Classified_List_It<Asura_Entity_Server> it(Asura_Entity_Server::GetList());!it.Done();it.Next())
	{
		Asura_Entity_Server* pxEntity = it.GetCurrent();
		//ignore entities marked for destruction
		if(pxEntity->IsDestroyed()) continue;
		//and those with non-tools guids
		if(pxEntity->GetGuid()<ASURA_GUID_TOOLSCREATEDENTITY_START || pxEntity->GetGuid()>ASURA_GUID_TOOLSCREATEDENTITY_END) continue;

		pxEntity->Tools_StartSim();
	}

	for(Asura_Classified_List_It<Asura_Entity_Client> it(Asura_Entity_Client::GetList());!it.Done();it.Next())
	{
		Asura_Entity_Client* pxEntity = it.GetCurrent();
		//ignore entities marked for destruction
		if(pxEntity->IsDestroyed()) continue;
		//and those with non-tools guids
		if(pxEntity->GetGuid()<ASURA_GUID_TOOLSCREATEDENTITY_START || pxEntity->GetGuid()>ASURA_GUID_TOOLSCREATEDENTITY_END) continue;

		pxEntity->Tools_StartSim();
	}
	#endif

}

void Tools_Entity_Simulation::End()
{

	#if 0
	if(s_bActive)
	{
		//Set asinactive before resetting everything to ensure that any cutscene server actors
		//get destroyed properly. There will still probably be issues if any get destroyed
		//during the simulation.
		s_bActive = false;

		//reset all the entities
		for(Asura_Classified_List_It<Asura_Entity_Server> it(Asura_Entity_Server::GetList());!it.Done();it.Next())
		{
			Asura_Entity_Server* pxEntity = it.GetCurrent();
			//ignore entities marked for destruction
			if(pxEntity->IsDestroyed()) continue;
			//and those with non-tools guids
			if(pxEntity->GetGuid()<ASURA_GUID_TOOLSCREATEDENTITY_START || pxEntity->GetGuid()>ASURA_GUID_TOOLSCREATEDENTITY_END) continue;

			pxEntity->Tools_Reset();
		}

		for(Asura_Classified_List_It<Asura_Entity_Client> it(Asura_Entity_Client::GetList());!it.Done();it.Next())
		{
			Asura_Entity_Client* pxEntity = it.GetCurrent();
			//ignore entities marked for destruction
			if(pxEntity->IsDestroyed()) continue;
			//and those with non-tools guids
			if(pxEntity->GetGuid()<ASURA_GUID_TOOLSCREATEDENTITY_START || pxEntity->GetGuid()>ASURA_GUID_TOOLSCREATEDENTITY_END) continue;

			pxEntity->Tools_Reset();
		}

		Asura_StaticMessage_System::ResetDelayedMessages();

		//Event flags should get reset by the cutscene controllers being reset, but
		//just in case there are any of those nasty wave attachments messing with stuff , 
		//reset them all here.
		Asura_Cutscene_System::ResetEnabledEventFlags();

		//stop any music that was started
		Asura_MetaMusic_System::StopTrack(true);
		Asura_Music_System::Stop();

		#ifdef ASURA_USE_DIALOGUE_SYSTEM
		Asura_Dialogue_Storage::ResetInGameState();
		#endif


		Asura_ClientCutscene_Actor_Camera::Disable = false;

	}
	#endif

	while(g_xMessageList.size())
	{
		delete g_xMessageList.first_entry();
		g_xMessageList.delete_first_entry();
	}

	while(s_xFilter_GuidList.size())
	{
		s_xFilter_GuidList.delete_first_entry();
	}
	s_eFilter = MESSAGEFILTER_NONE;


	UpdateMessageListBox();


}

void Tools_Entity_Simulation::Reset()
{
	End();
	Start();
}

void Tools_Entity_Simulation::Update()
{
	#if 0
	if(s_bActive && !s_bPaused)
	{
		s_fTimer += Asura_Timers::GetGameTimeSlice();

		Asura_StaticMessage_System::Update();

		for(Asura_Classified_List_It<Asura_Entity_Server> it(Asura_Entity_Server::GetList());!it.Done();it.Next())
		{
			Asura_Entity_Server* pxEntity = it.GetCurrent();
			//ignore entities marked for destruction
			if(pxEntity->IsDestroyed()) continue;
			//and those with non-tools guids
			if(pxEntity->GetGuid()<ASURA_GUID_TOOLSCREATEDENTITY_START || pxEntity->GetGuid()>ASURA_GUID_TOOLSCREATEDENTITY_END) continue;

			pxEntity->Tools_Update();
		}
		for(Asura_Classified_List_It<Asura_Entity_Client> it(Asura_Entity_Client::GetList());!it.Done();it.Next())
		{
			Asura_Entity_Client* pxEntity = it.GetCurrent();
			//ignore entities marked for destruction
			if(pxEntity->IsDestroyed()) continue;
			//and those with non-tools guids
			if(pxEntity->GetGuid()<ASURA_GUID_TOOLSCREATEDENTITY_START || pxEntity->GetGuid()>ASURA_GUID_TOOLSCREATEDENTITY_END) continue;

			pxEntity->Tools_Update();
		}
	}
	#endif

	if(!s_bMessageListBoxValid)
	{
		UpdateMessageListBox();
	}

	if(s_bOverlayOpen)
	{
		Tools_Object::SetAllowChildObjectSelection(s_bAllowTemplateMemberSelection);
	}
}

void Tools_Entity_Simulation::Render()
{
	#if 0
	if(s_bActive && !s_bPaused)
	{
		for(int i=0;i<Asura_Cutscene_System::GetNumberOfCutscenes();i++)
		{
			Asura_Cutscene* pxCutscene = Asura_Cutscene_System::GetCutsceneByIndex(i);
			if(pxCutscene->IsPlaying()) pxCutscene->Render();
		}
	}
	#endif
}


void Tools_Entity_Simulation::ToggleOverlay(void)
{
	s_bOverlayOpen = !s_bOverlayOpen;

	if (s_bOverlayOpen)
	{
        CreateOverlay();
	}
	else
	{
		CloseOverlay();
	}
}
bool Tools_Entity_Simulation::OverlayIsOn(void)
{
	return s_bOverlayOpen;
}


void Tools_Entity_Simulation::CreateOverlay(void)
{
	if(!s_bOverlayOpen) return;
	CloseOverlay();

	static Asura_Vector_2 xPosition;
	xPosition.x = 0.0f;
	xPosition.y = (Asura_Viewport::GetHeight()/14.0f)-11.0f;
	Asura_Overlay* pxOverlay = new Asura_Overlay("Entity Simulation", false, &xPosition);
	
	Asura_Vector_2 xOffset = Asura_Vector_2(0.0f, 1.0f);

	Asura_Dialog* pxDialog;

//	pxDialog = new Asura_Function_Dialog("Start",xOffset,Reset);
//	pxDialog->SetHelpText("(Re)start simulation of entities");
//	pxOverlay->AddDialog(pxDialog);
//	xOffset.y += Asura_Dialog::DefaultSpacing();

//	pxDialog = new Asura_Function_Dialog("End",xOffset,End);
//	pxDialog->SetHelpText("stop simulation ,and reset all the entities to the initial states");
//	pxOverlay->AddDialog(pxDialog);
//	xOffset.y += Asura_Dialog::DefaultSpacing();
    	
//	pxDialog = new Asura_Function_Dialog("Pause/Resume",xOffset,PauseOrResume);
//	pxOverlay->AddDialog(pxDialog);
//	xOffset.y += Asura_Dialog::LargeSpacing();


//	Asura_Numerical_Dialog* pxNumDialog = new Asura_Numerical_Dialog("Timer",xOffset,&s_fTimer,0,999.0f,1.0f,"%.3f");
//	pxNumDialog->SetReadOnly(true);
//	pxOverlay->AddDialog(pxNumDialog);
//	xOffset.y += Asura_Dialog::LargeSpacing();

	pxDialog = new Asura_Function_Dialog("Load Messages",xOffset,LoadMessageDatFile);
	pxDialog->SetHelpText("Load message log from previous visit to game");
	pxOverlay->AddDialog(pxDialog);
	
	

	

	xOffset.x = 20.0f;
	xOffset.y = 0.0f;
	
	s_iSelectedMessage = 0;
	pxDialog = s_pxListDialog = new Asura_List_Dialog(xOffset,g_ppszDefaultMessageText,&s_iSelectedMessage,8,1,0);
	s_pxListDialog->SetExpandWidthToFillOverlay( false );
	pxOverlay->AddDialog(pxDialog);
	xOffset.y = pxDialog->m_xMaxCoord.y + 1.0f;

	xOffset.y = 10.0f;

	pxDialog = new Asura_Dialog("Filters:",xOffset);
	pxOverlay->AddDialog(pxDialog);
	xOffset.x = pxDialog->m_xMaxCoord.x + 1.0f;;
	
	pxDialog = new Asura_Function_Dialog("None",xOffset,SetFilter_None);
	pxDialog->SetHelpText("Force all messages to be shown in the above list.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.x = pxDialog->m_xMaxCoord.x + 1.0f;;

	pxDialog = new Asura_Function_Dialog("Selected Objects",xOffset,SetFilter_SelectedObjects);
	pxDialog->SetHelpText("Just show messages involving the selected objects.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.x = pxDialog->m_xMaxCoord.x + 1.0f;;
	
	pxDialog = new Asura_Function_Dialog("Caused By",xOffset,SetFilter_CausedByMessage);
	pxDialog->SetHelpText("Just show the currently selected message , and all message that may be caused by it.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.x = pxDialog->m_xMaxCoord.x + 1.0f;;

	pxDialog = new Asura_Function_Dialog("Causing",xOffset,SetFilter_CausingMessage);
	pxDialog->SetHelpText("Just show the currently selected message , and all messages that might have caused it.");
	pxOverlay->AddDialog(pxDialog);
	

	xOffset.x = Asura_Viewport::GetWidth()/10.0f - 20.0f;
	xOffset.y = 1.0f;
	
	pxDialog = new Asura_Function_Dialog("Find Sender",xOffset,FindMessageFrom);
	pxDialog->SetHelpText("Find and select the sender of the current message");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Find Recipient",xOffset,FindMessageTo);
	pxDialog->SetHelpText("Find and select the recipient of the current message");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();

//	pxDialog = new Asura_Function_Dialog("Send Message",xOffset,SendTestMessage);
	//pxDialog->SetIsEnabledFunction(IsActive);
//	pxDialog->SetHelpText("Send a test message to the selected object");
//	pxOverlay->AddDialog(pxDialog);
//	xOffset.y += Asura_Dialog::LargeSpacing();

//	pxDialog = new Asura_Tickbox_Dialog("Report Messages",xOffset,&s_bShowMessages);
//	pxDialog->SetHelpText("Show a description of any message sent as it occurs");
//	pxOverlay->AddDialog(pxDialog);
//	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Tickbox_Dialog("Template Member Selection",xOffset,&s_bAllowTemplateMemberSelection);
	pxDialog->SetHelpText("If ticked then members of template objects may be selected to check their state");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();



	pxOverlay->m_uID = TOOLS_OVERLAY_ENTITYSIMULATION;
	pxOverlay->m_bOnPanel = true;

	Asura_Overlay_System::SubmitNewOverlay(pxOverlay, false, true, false);

	UpdateMessageListBox();
}

void Tools_Entity_Simulation::CloseOverlay()
{
	Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_ENTITYSIMULATION);
	Tools_Object::SetAllowChildObjectSelection(false);
	s_pxListDialog = 0;
}



void Tools_Entity_Simulation::AddToMessageLog(const class Asura_Message* pxMessage,bool bQuiet,float fDelay,bool bDelayed)
{

	char szFrom[100];
	char szMessage[100];
	char szTo[100];

	Asura_Entity_Server* pxFromEntity = Asura_Entity_Server::FindEntity(pxMessage->From);
	Asura_Entity_Server* pxToEntity = Asura_Entity_Server::FindEntity(pxMessage->To);


	//get the name of the sender
	if(pxMessage->From == ASURA_GUID_TOOLSUSER)
	{
		Asura_PrintfToBfr(szFrom,"User",100,true);
	}
	else if(pxFromEntity && pxFromEntity->ToolsData && !pxFromEntity->ToolsData->Name.IsBlank())
	{
		Asura_PrintfToBfr(szFrom,"%s",100,true,pxFromEntity->ToolsData->Name.GetString());
	}
	else
	{
		Asura_PrintfToBfr(szFrom,"Entity<%u>",100,true,pxMessage->From);
	}

	//get the description of the message
	Asura_String xMessageDesc;

	if(Tools_Message_Editing::GetMessageDescription(pxMessage->GetMsgID(),0,xMessageDesc))
	{
		Asura_PrintfToBfr(szMessage,"%s",100,true,xMessageDesc.GetString());
	}
	else
	{
		Asura_PrintfToBfr(szMessage,"Message<%u>",100,true,pxMessage->GetMsgID());
	}

	//and the name of the recipient
	if(pxToEntity && pxToEntity->ToolsData && !pxToEntity->ToolsData->Name.IsBlank())
	{
		Asura_PrintfToBfr(szTo,"%s",100,true,pxToEntity->ToolsData->Name.GetString());
	}
	else
	{
		Asura_PrintfToBfr(szTo,"Entity<%u>",100,true,pxMessage->To);
	}


	
	


	if(s_bShowMessages && !bQuiet)
	{
		char szMessageBuffer[500];
		Asura_PrintfToBfr(szMessageBuffer,"%8.3f %s sent %s to %s",500,true,Asura_Maths::FtoD(pxMessage->SentTime),szFrom,szMessage,szTo);
		Tools_Timed_Message_System::AddMessage(0xffffffff, "%s",szMessageBuffer);
	}

	const char* szDelayListText = "";
	if(fDelay > 0.0f)
	{
		szDelayListText = bDelayed ? " (Delayed)" : " (Received)";
	}

	char szListText[90];
	Asura_PrintfToBfr(szListText,"%8.3f %s sent %s to %s%s",90,true,Asura_Maths::FtoD(pxMessage->SentTime),szFrom,szMessage,szTo,szDelayListText);
	
	char szHelpText[550];
	Asura_PrintfToBfr(szHelpText,"Time : %8.3f\nFrom : %s\nTo : %s\nMessage : %s\n",500,true,Asura_Maths::FtoD(pxMessage->SentTime),szFrom,szTo,szMessage);

	char* pcPos = szHelpText + strlen(szHelpText);
	if(fDelay > 0.0f)
	{
		if(bDelayed)
		{
			Asura_PrintfToBfr(pcPos,"Delayed message sent.\nDelay : %.3f\n",50,true,fDelay);
		}
		else
		{
			Asura_PrintfToBfr(pcPos,"Delayed message received.\nDelay : %.3f\n",50,true,fDelay);
		}
	}



	//add to message list
	Tools_Logged_Message* pxLog = new Tools_Logged_Message;
	pxLog->m_uFrom = pxMessage->From;
	pxLog->m_uTo = pxMessage->To;
	pxLog->m_uMessage = pxMessage->GetMsgID();
	pxLog->m_xDesc = szListText;
	pxLog->m_xHelpText = szHelpText;
	pxLog->m_bVisible = true;
	g_xMessageList.add_entry(pxLog);
	//and invalidate listbox
	s_bMessageListBoxValid = false;
	
}

void Tools_Entity_Simulation::LogInGameMessage(const Asura_Message* pxMessage,Asura_Guid uTo, Asura_Guid uFrom)
{
	//create a temporary static message so we can use the other function
	Asura_StaticMessage xStaticMessage;
	xStaticMessage.Initialise();

	xStaticMessage.m_usMsgID = (u_short) pxMessage->m_usMsgID;
	xStaticMessage.m_usDataSize = (u_short)pxMessage->m_uDataSize;
	xStaticMessage.m_uSendTo = uTo;
	xStaticMessage.m_pData = pxMessage->m_pData;

	LogInGameMessage(&xStaticMessage,uFrom,false,0.0f);
}

void Tools_Entity_Simulation::LogInGameMessage(const Asura_StaticMessage* pxMessage,Asura_Guid uFrom,bool bDelayed,float fDelay)
{
	if ( Tools_System::IsActive() == true )
	{
		return;
	}
	char szMessageBuffer[1024];
	char* pcPos = szMessageBuffer;

	if(s_iMessageLogIndent>0)
	{
		Asura_Assert( s_iMessageLogIndent < 64, "Far too many nested messages is there an infinite loop somewhere" );
		const int iIndent = ( s_iMessageLogIndent > 64 ) ? 64 : s_iMessageLogIndent;
		for(int i=0;i<10+iIndent*2;i++)
		{
			*pcPos++ = ' ';
		}
	}
	else
	{
		Asura_PrintfToBfr( pcPos,"%8.3f  ", 20, true, Asura_Maths::FtoD( Asura_Timers::GetLocalUnscaledTime() ) );
		pcPos += strlen(pcPos);
	}

	Asura_Entity_Server* pxFromEntity = Asura_Entity_Server::FindEntity(uFrom);
	Asura_Entity_Server* pxToEntity = Asura_Entity_Server::FindEntity(pxMessage->m_uSendTo);

	//check to see if this message should be filtered out of the logging
	if(!ShouldLogMessage(pxFromEntity, pxToEntity)) return;



	#ifdef ASURA_SUPPORT_ENTITYWATCH
	const char* szSentPos = pcPos;
	#endif
	
	//get the name of the sender
	if(pxFromEntity && pxFromEntity->ToolsData && !pxFromEntity->ToolsData->Name.IsBlank())
	{
		Asura_PrintfToBfr(pcPos,"%s sent ",100,true,pxFromEntity->ToolsData->Name.GetString());
	}
	else
	{
		Asura_PrintfToBfr(pcPos,"Entity<%u> sent ",100,true,uFrom);
	}
	pcPos += strlen(pcPos);

	//get the description of the message
	Asura_String xMessageDesc;
	if(Tools_Message_Editing::GetMessageDescription(pxMessage->m_usMsgID,pxMessage,xMessageDesc))
	{
		Asura_PrintfToBfr(pcPos,"%s to ",100,true,xMessageDesc.GetString());
	}
	else
	{
		Asura_PrintfToBfr(pcPos,"Message<%u> to ",100,true,pxMessage->m_usMsgID);
	}
	pcPos += strlen(pcPos);

	//and the name of the recipient
	if(pxToEntity && pxToEntity->ToolsData && !pxToEntity->ToolsData->Name.IsBlank())
	{
		Asura_PrintfToBfr(pcPos,"%s",100,true,pxToEntity->ToolsData->Name.GetString());
	}
	else
	{
		Asura_PrintfToBfr(pcPos,"Entity<%u>",100,true,pxMessage->m_uSendTo);
	}
	pcPos += strlen(pcPos);

	if(fDelay)
	{
		if(bDelayed)
			Asura_PrintfToBfr(pcPos," (Delay %.3f sec)",100,true,fDelay);
		else
			Asura_PrintfToBfr(pcPos," (Received)",100,true);

		pcPos += strlen(pcPos);
	}

	static bool s_bLogFileCleared = false;
	const char* pszLogFilename = "Logfiles\\StaticMessages.txt";

	if(!s_bLogFileCleared)
	{
		Asura_SimpleLogfileClear(pszLogFilename);
		s_bLogFileCleared = true;
	}

	Asura_SimpleLogfileOutput((pszLogFilename,"%s\n",szMessageBuffer));

	//test
#ifdef ASURA_SUPPORT_ENTITYWATCH
	if(Asura_EntityWatch_System::ShouldDisplayMessageFor(uFrom) || Asura_EntityWatch_System::ShouldDisplayMessageFor(pxMessage->m_uSendTo))
	{
		Asura_CommandConsole::Printf("%8.3f  %s",Asura_Maths::FtoD(Asura_Timers::GetLocalUnscaledTime()),szSentPos);
		Asura_CommandConsole::TimedShowOutputLine(5.0f);
	}
#endif

	AddMessageToDatFile(uFrom,pxMessage,bDelayed);
}



void Tools_Entity_Simulation::AddMessageToDatFile(Asura_Guid uFrom,const Asura_StaticMessage* pxMessage,bool bDelayed)
{
	FILE* pFile = 0;
	if(s_bMessageDatFileNeedsReset)
	{
		//open as new file
		pFile = fopen(g_pszMessageDatFilename,"wb");
		if(!pFile)
		{
			Asura_Assertf(0,("Error opening %s for writing.\n Error message : %s",g_pszMessageDatFilename,Tools_OS::GetLastSystemError()));		
			return;
		}
		//add header
		if(fwrite("MESSGLG2",8,1,pFile) != 1)
		{
			Asura_Assertf(0,("Error writing to %s.\n Error message : %s",g_pszMessageDatFilename,Tools_OS::GetLastSystemError()));
		}
		s_bMessageDatFileNeedsReset = false;

		int iVersion = 0;
		if(fwrite(&iVersion,sizeof(iVersion),1,pFile) != 1)
		{
			Asura_Assertf(0,("Error writing to %s.\n Error message : %s",g_pszMessageDatFilename,Tools_OS::GetLastSystemError()));
		}
	}
	else
	{
		//open for appending
		pFile = fopen(g_pszMessageDatFilename,"ab");
		if(!pFile)
		{
			Asura_Assertf(0,("Error opening %s for appending.\n Error message : %s",g_pszMessageDatFilename,Tools_OS::GetLastSystemError()));
			return;
		}
	}

	//fill in message to be saved
	Tools_SavedMessageData xData;
	xData.m_fTime = Asura_Timers::GetLocalUnscaledTime();
	xData.m_uFrom = uFrom;
	xData.m_uTo = pxMessage->m_uSendTo;
	xData.m_uMessage = pxMessage->m_usMsgID;
	xData.m_fDelay = pxMessage->m_fDelay;
	xData.m_bDelayed = bDelayed;

	if(fwrite(&xData,sizeof(xData),1,pFile) != 1)
	{
		Asura_Assertf(0,("Error writing to %s.\n Error message : %s",g_pszMessageDatFilename,Tools_OS::GetLastSystemError()));
	}
	fclose(pFile);
}

void Tools_Entity_Simulation::LoadMessageDatFile()
{
	//stop current simulation
	End();

	FILE* pFile = fopen(g_pszMessageDatFilename,"rb");
	if(!pFile)
	{
		Tools_Timed_Message_System::AddMessage(0xffffffff, "Failed to load %s",g_pszMessageDatFilename);
		return;
	}
	
	//make sure header is okay
	char szHeader[9];
	szHeader[8] = 0;
	fread(szHeader,8,1,pFile);

	if(strcmp(szHeader,"MESSGLG2"))
	{
		Tools_Timed_Message_System::AddMessage(0xffffffff, "%s isn't a valid log file",g_pszMessageDatFilename);
		fclose(pFile);
		return;
	}

	int iVersion =- 1;
	fread(&iVersion,sizeof(iVersion),1,pFile);
	if(iVersion != 0)
	{
		Tools_Timed_Message_System::AddMessage(0xffffffff, "%s uses an unrecognised version",g_pszMessageDatFilename);
		fclose(pFile);
		return;
	}


	Tools_SavedMessageData xData;
	while(fread(&xData,sizeof(xData),1,pFile))
	{
		//create and fill in the message
		Asura_Message xMessage;
		memset(&xMessage,0,sizeof(xMessage));
		xMessage.To = xData.m_uTo;
		xMessage.From = xData.m_uFrom;
		xMessage.SentTime = xData.m_fTime;
		xMessage.DeliveryTime = xMessage.SentTime;
		xMessage.m_usMsgID = (u_short)xData.m_uMessage;
		xMessage.m_usEntityClassification = 0;
		xMessage.m_uDataSize = 0;
		xMessage.m_pData = 0;

		AddToMessageLog(&xMessage,true,xData.m_fDelay,xData.m_bDelayed);
		
	}
	fclose(pFile);

}



void Tools_Entity_Simulation::UpdateMessageListBox()
{
	//determine the current selection
	int iSelectedIndex = 0;
	if(s_iMessageListBoxSize && s_piMessageIndeces)
	{
		iSelectedIndex = s_piMessageIndeces[s_iSelectedMessage];
	}

	//clear out old message strings
	s_iMessageListBoxSize = 0;
	delete [] s_ppszMessageStrings;
	s_ppszMessageStrings = 0;
	delete [] s_ppszMessageHelp;
	s_ppszMessageHelp = 0;
	delete [] s_piMessageIndeces;
	s_piMessageIndeces = 0;
	s_iSelectedMessage = 0;


	//Initialise all messages to visible
	switch(s_eFilter)
	{
		case MESSAGEFILTER_NONE :
		{
			//mark all as visible
			for(LIF<Tools_Logged_Message*> xMessageLIF(&g_xMessageList);!xMessageLIF.done();xMessageLIF.next())
			{
				xMessageLIF()->m_bVisible = true;
			}
			break;
		}
		case MESSAGEFILTER_OBJECT :
		{
			//all messages that are from or to objects in the guid list should be shown
			for(LIF<Tools_Logged_Message*> xMessageLIF(&g_xMessageList);!xMessageLIF.done();xMessageLIF.next())
			{
				Tools_Logged_Message* pxLog = xMessageLIF();
				if(s_xFilter_GuidList.contains(pxLog->m_uFrom) || s_xFilter_GuidList.contains(pxLog->m_uTo))
				{
					pxLog->m_bVisible = true;
				}
				else
				{
					pxLog->m_bVisible = false;
				}
			}
			break;
		}
		case MESSAGEFILTER_CAUSEDBY :
		{
			int iIndex = 0;
			List<Asura_Guid> xGuidList;

			for(LIF<Tools_Logged_Message*> xMessageLIF(&g_xMessageList);!xMessageLIF.done();xMessageLIF.next(),iIndex++)
			{
				Tools_Logged_Message* pxLog = xMessageLIF();
				if(iIndex<s_iFilter_MessageIndex)
				{
					//messages before the causing message can be ignored
					pxLog->m_bVisible = false;
				}
				else if(iIndex == s_iFilter_MessageIndex)
				{
					//this is the message that we are interested in , and so should be visible
					pxLog->m_bVisible = true;
					//add its destination to the guid list to check
					if(pxLog->m_uTo != ASURA_GUID_UNREGISTERED && pxLog->m_uTo != ASURA_GUID_TOOLSUSER)
					{
						xGuidList.add_entry(pxLog->m_uTo);
					}
				}
				else
				{
					//If this message is from any of the objects in the list we have built so far , then
					//it may have been caused by the original message , and so should be visible
					if(xGuidList.contains(pxLog->m_uFrom))
					{
						pxLog->m_bVisible = true;
						//add the destination of this message
						if(pxLog->m_uTo != ASURA_GUID_UNREGISTERED && pxLog->m_uTo != ASURA_GUID_TOOLSUSER)
						{
							if(!xGuidList.contains(pxLog->m_uTo))
							{
								xGuidList.add_entry(pxLog->m_uTo);
							}
						}
					}
					else
					{
						pxLog->m_bVisible = false;
					}
				}
			}
			break;
		}
		case MESSAGEFILTER_CAUSING :
		{
			int iIndex = g_xMessageList.size()-1;
			List<Asura_Guid> xGuidList;

			for(LIB<Tools_Logged_Message*> xMessageLIB(&g_xMessageList);!xMessageLIB.done();xMessageLIB.next(),iIndex--)
			{
				Tools_Logged_Message* pxLog = xMessageLIB();
				if(iIndex>s_iFilter_MessageIndex)
				{
					//messages after the message can't have caused it
					pxLog->m_bVisible = false;
				}
				else if(iIndex == s_iFilter_MessageIndex)
				{
						//this is the message that we are interested in , and so should be visible
					pxLog->m_bVisible = true;
					//add its source to the guid list to check
					if(pxLog->m_uFrom != ASURA_GUID_UNREGISTERED && pxLog->m_uFrom != ASURA_GUID_TOOLSUSER)
					{
						xGuidList.add_entry(pxLog->m_uFrom);
					}
				}
				else
				{
					//If this message is to any of the objects in the list we have built so far , then
					//it may have caused the original message , and so should be visible
					if(xGuidList.contains(pxLog->m_uTo))
					{
						pxLog->m_bVisible = true;
						//add the destination of this message
						if(pxLog->m_uFrom != ASURA_GUID_UNREGISTERED && pxLog->m_uFrom != ASURA_GUID_TOOLSUSER)
						{
							if(!xGuidList.contains(pxLog->m_uFrom))
							{
								xGuidList.add_entry(pxLog->m_uFrom);
							}
						}
					}
					else
					{
						pxLog->m_bVisible = false;
					}
				}
			}
			break;		
		}

		default :
			Asura_Assert(0,"Unknown message filter");
			break;

	}
	

	//count the visible messages
	for(LIF<Tools_Logged_Message*> xMessageLIF(&g_xMessageList);!xMessageLIF.done();xMessageLIF.next())
	{
		if(xMessageLIF()->m_bVisible)
		{
			s_iMessageListBoxSize++;
		}
	}

	if(s_iMessageListBoxSize)
	{
		s_ppszMessageStrings = new char*[s_iMessageListBoxSize];
		s_ppszMessageHelp = new char*[s_iMessageListBoxSize];
		s_piMessageIndeces = new int[s_iMessageListBoxSize];

		int iIndexFrom = 0;
		int iIndexTo = 0;
		//get the strings and indeces of the visible messages
		for(LIF<Tools_Logged_Message*> xMessageLIF(&g_xMessageList);!xMessageLIF.done();xMessageLIF.next(),iIndexFrom++)
		{
			if(xMessageLIF()->m_bVisible)
			{
				s_ppszMessageStrings[iIndexTo] = xMessageLIF()->m_xDesc.GetString();
				s_ppszMessageHelp[iIndexTo] = xMessageLIF()->m_xHelpText.GetString();
				s_piMessageIndeces[iIndexTo] = iIndexFrom;
				if(iSelectedIndex == iIndexFrom)
				{
					s_iSelectedMessage = iIndexTo;	
				}
				iIndexTo++;
			}
		}
		if(s_pxListDialog)
		{
			s_pxListDialog->ReplaceStrings(s_iMessageListBoxSize,s_ppszMessageStrings);
			s_pxListDialog->SetListHelpStrings(s_ppszMessageHelp);
		}
	}
	else
	{
		if(s_pxListDialog)
		{
			s_pxListDialog->ReplaceStrings(1,g_ppszDefaultMessageText);
			s_pxListDialog->SetListHelpStrings(g_ppszDefaultMessageText);
		}
	}
	s_bMessageListBoxValid = true;
}


void Tools_Entity_Simulation::PauseOrResume()
{
	if(Tools_System::UserSetGameTimeScale == 0.0f)
	{
		Tools_System::UserSetGameTimeScale = 1.0f;
	}
	else
	{
		Tools_System::UserSetGameTimeScale = 0.0f;
	}
	Asura_Timers::SetGameTimeScale(Tools_System::UserSetGameTimeScale);

}

void Tools_Entity_Simulation::SetFilter_None()
{
	s_eFilter = MESSAGEFILTER_NONE;
	s_bMessageListBoxValid = false;

	//Tools_Timed_Message_System::AddMessage(0xffffffff, "Message filter removed");
}

void Tools_Entity_Simulation::SetFilter_SelectedObjects()
{
	if(!Tools_Object_System::SelectedObjectList.size())
	{
		Tools_Timed_Message_System::AddMessage(0xffffffff, "No objects are selected");
		return;
	}

	while(s_xFilter_GuidList.size()) s_xFilter_GuidList.delete_first_entry();

	//get the guids of all selected objects
	for(LIF<Tools_Object*> xObjectLIF(&Tools_Object_System::SelectedObjectList);!xObjectLIF.done();xObjectLIF.next())
	{
		Asura_Guid uGuid = xObjectLIF()->GetGuid();
		if(uGuid != ASURA_GUID_UNREGISTERED)
		{
			s_xFilter_GuidList.add_entry(uGuid);
		}

		//add guids of child objects as well
		List<Tools_Object*> xChildList;
		xObjectLIF()->ListChildObjects(xChildList,true);
		for(LIF<Tools_Object*> xChildLIF(&xChildList);!xChildLIF.done();xChildLIF.next())
		{
			Asura_Guid uGuid = xChildLIF()->GetGuid();
			if(uGuid != ASURA_GUID_UNREGISTERED)
			{
				s_xFilter_GuidList.add_entry(uGuid);
			}
		}
	}

	s_eFilter = MESSAGEFILTER_OBJECT;
	s_bMessageListBoxValid = false;

	//Tools_Timed_Message_System::AddMessage(0xffffffff, "Showing messages involving selected objects");
}

void Tools_Entity_Simulation::SetFilter_CausedByMessage()
{
	if(!s_iMessageListBoxSize || ! s_piMessageIndeces) return;

	s_iFilter_MessageIndex = s_piMessageIndeces[s_iSelectedMessage];

	s_eFilter = MESSAGEFILTER_CAUSEDBY;
	s_bMessageListBoxValid = false;

	//Tools_Timed_Message_System::AddMessage(0xffffffff, "Showing messages caused by the current message");
	
}

void Tools_Entity_Simulation::SetFilter_CausingMessage()
{
	if(!s_iMessageListBoxSize || ! s_piMessageIndeces) return;

	s_iFilter_MessageIndex = s_piMessageIndeces[s_iSelectedMessage];

	s_eFilter = MESSAGEFILTER_CAUSING;
	s_bMessageListBoxValid = false;

	//Tools_Timed_Message_System::AddMessage(0xffffffff, "Showing messages causing the current message");
	
}


void Tools_Entity_Simulation::FindMessageFrom()
{
	if(!s_iMessageListBoxSize || ! s_piMessageIndeces) return;

	int iMessageIndex = s_piMessageIndeces[s_iSelectedMessage];
	Asura_Guid uGuid = g_xMessageList[iMessageIndex]->m_uFrom;

	Tools_Object* pxObject = Tools_Object_System::GetObjectFromGuid(uGuid);
	if(pxObject)
	{
		Tools_Object_System::GotoAndSelect(pxObject);
	}
	else
	{
		Tools_Timed_Message_System::AddMessage(0xffffffff, "Failed to find the sender of the message");
	}
	
}

void Tools_Entity_Simulation::FindMessageTo()
{
	int iMessageIndex = s_piMessageIndeces[s_iSelectedMessage];
	Asura_Guid uGuid = g_xMessageList[iMessageIndex]->m_uTo;

	Tools_Object* pxObject = Tools_Object_System::GetObjectFromGuid(uGuid);
	if(pxObject)
	{
		Tools_Object_System::GotoAndSelect(pxObject);
	}
	else
	{
		Tools_Timed_Message_System::AddMessage(0xffffffff, "Failed to find the recipient of the message");
	}
}


void Tools_Entity_Simulation::AddMessageLogFilterOverlayControls(Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset)
{
	Asura_Dialog* pxDialog;

	pxDialog = new Asura_Dialog("Files to ignore:",xOffset);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	const char* szHelpText = "If ticked then messages between entities within this file won't be logged to MessageLog.txt";

	//add a tickbox for the main file
	pxDialog = new Asura_FunctionTickbox_Dialog(Tools_SecondaryEntityFile_System::GetMainFileLabel(), xOffset, (int)0, IsEntityFileFiltered, SetEntityFileFiltered);
	pxDialog->SetHelpText(szHelpText);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	//and for each of the secondary files
	for(int i=0;i<Tools_SecondaryEntityFile_System::GetNumSecondaryFiles();i++)
	{
		const char* szName = Tools_SecondaryEntityFile_System::GetSecondaryFileName(i);
		Asura_Assert(szName, "");
		Asura_Hash_ID uHash = Asura_GetHashID(szName);

		pxDialog = new Asura_FunctionTickbox_Dialog(szName, xOffset, (int)uHash, IsEntityFileFiltered, SetEntityFileFiltered);
		pxDialog->SetHelpText(szHelpText);
		pxOverlay->AddDialog(pxDialog);
		xOffset.y += Asura_Dialog::DefaultSpacing();
	}

	xOffset.y += Asura_Dialog::LargeSpacing()-Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_Tickbox_Dialog("Filter Template Msgs", xOffset, &s_bMessageLogFilter_TemplateMessages);
	pxDialog->SetHelpText("Filter out messages involving entities within template instances");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

}

bool Tools_Entity_Simulation::IsEntityFileFiltered(int iHash)
{
	return s_xMessageLogFilter.contains(iHash);
}

void Tools_Entity_Simulation::SetEntityFileFiltered(bool bSet, int iHash)
{
	bool bIsSet = s_xMessageLogFilter.contains(iHash);

	if(bIsSet != bSet)
	{
		if(bSet)
		{
			s_xMessageLogFilter.add_entry(iHash);
		}
		else
		{
			s_xMessageLogFilter.delete_entry(iHash);
		}
	}
}

bool Tools_Entity_Simulation::ShouldLogMessage(class Asura_Entity_Server* pxFrom, class Asura_Entity_Server* pxTo)
{
	if(s_bMessageLogFilter_TemplateMessages)
	{
		//Ignore if either entity belongs to a template
		if(pxFrom && pxFrom->ToolsData && pxFrom->ToolsData->IsMemberOfTemplate()) return false;
		if(pxTo && pxTo->ToolsData && pxTo->ToolsData->IsMemberOfTemplate()) return false;
	}
	//don't bother with further checks if no files are being filtered out
	if(!s_xMessageLogFilter.size()) return true;

	//Check for entities being in entity files that are filtered out
	bool bToFilteredOut = false;
	while(pxTo && pxTo->ToolsData && pxTo->ToolsData->IsMemberOfTemplate())
	{
		pxTo = Asura_ServerEntity_Template::FindEntity(pxTo->ToolsData->m_uParentTemplate);
	}

	if(pxTo && pxTo->ToolsData)
	{
		Asura_Hash_ID uHash = Asura_GetHashID(pxTo->ToolsData->m_xSecondaryEntityFilename.GetString());

		if(!s_xMessageLogFilter.contains(uHash))
		{
			//to entity isn't filtered out , so log the message
			return true;
		}
		else
		{
			bToFilteredOut = true;
		}
	}

	bool bFromFilteredOut = false;

	//for templates we need to get the root template instance to determine the file
	while(pxFrom && pxFrom->ToolsData && pxFrom->ToolsData->IsMemberOfTemplate())
	{
		pxFrom = Asura_ServerEntity_Template::FindEntity(pxFrom->ToolsData->m_uParentTemplate);
	}

	if(pxFrom && pxFrom->ToolsData)
	{
		Asura_Hash_ID uHash = Asura_GetHashID(pxFrom->ToolsData->m_xSecondaryEntityFilename.GetString());

		if(!s_xMessageLogFilter.contains(uHash))
		{
			//from entity isn't filtered out , so log the message
			return true;
		}
		else
		{
			bFromFilteredOut = true;
		}
	}


	//At this point , the entities are either filtered out , or aren't tools entities ,or don't exist.
	//Only log the message if neither were explicitly filtered out.
	return !(bFromFilteredOut || bToFilteredOut);
}