//
// Tools_Entity_Trigger_Object.cpp
//
// Created by Richard Rice - 01/05/2002
// 
// Copyright (C) 2002 Rebellion, All Rights Reserved.
//


#include "Core/Asura.h"
#include "Debugging/Asura_Assert.h"

#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Overlay.h"

#include "Camera/Asura_Camera.h"
#include "Core/Asura_LOS.h"
#include "Entity/Asura_Entity.h"
#include "Entity/Asura_Entity_Classification.h"
#include "Entity/Triggers/Asura_ClientEntity_LookAtTrigger.h"
#include "Entity/Triggers/Asura_ClientEntity_Trigger.h"
#include "Entity/Triggers/Asura_ServerEntity_Trigger.h"
#include "Entity/Triggers/Asura_ServerEntity_TimeTrigger.h"
#include "Entity/Asura_StaticMessage_System.h"
#include "Entity/Triggers/Asura_ServerEntity_CountedTrigger.h"
#ifdef ASURA_USE_DEBUG_MESSAGE_TRIGGER
#include "Entity/Triggers/Asura_ServerEntity_DebugMessageTrigger.h"
#endif
#include "Entity/Triggers/Asura_ServerEntity_LookAtTrigger.h"
#include "Mesh/Asura_Mesh_System.h"
#include "Rendering/Asura_Text_Output.h"

#include "Entity/Tools_Entity_Trigger_Object.h"
#include "Entity/Tools_Entity_Simulation.h"
#include "Interface/Tools_ColourPicker.h"
#include "Objects/Tools_VolumeEdit_Object.h"
#include "Objects/Tools_Object_System.h"
#include "Load&Save/Tools_SaveAsura.h"

//////////////////////////////////////////////////////////////////////////////////////////////////
/*-----------------------------**
** Tools_Entity_Trigger_Object **
**-----------------------------*/

Tools_Entity_Trigger_Object::Tools_Entity_Trigger_Object(Asura_Entity* entity)
:Tools_Entity_Activatable_Object(entity)
{
}


void Tools_Entity_Trigger_Object::Destroy()
{
	Asura_ServerEntity_Trigger* asuraEntity = (Asura_ServerEntity_Trigger*) Entity;
	//clear out any message blocks associated with this
	for(int i=0;i<ASURA_TRIGGER_ENTITY_NOOFSTATES;i++)
	{
		Asura_StaticMessage_System::DeleteMessageBlock(asuraEntity->MessageBlock[i]);
		asuraEntity->MessageBlock[i] = ASURA_STATICMESSAGE_EMPTYBLOCK;
	}

	Tools_Entity_Object::Destroy();
}


void Tools_Entity_Trigger_Object::AddOverlayControls(class Asura_Overlay* pxNewOverlay,class Asura_Vector_2& xOffset)
{
	Tools_Entity_Activatable_Object::AddOverlayControls(pxNewOverlay,xOffset);

	#ifndef ASURA_TOOLS_USE_NEW_MESSAGE_EDITOR

	Asura_Dialog* pxNewDialog;

	Asura_ServerEntity_Trigger* pxAsuraEntity = (Asura_ServerEntity_Trigger*) Entity;

	//overlay controls for message block editing
	char text[100];
	if(UsesActivationMessage())
	{
		sprintf(text,"On Activation (%d)",Asura_StaticMessage_System::GetNumberOfMessagesInBlock(pxAsuraEntity->MessageBlock[ASURA_TRIGGER_ENTITY_ONACTIVATION]));
		pxNewDialog = new Asura_Function_Dialog(text,xOffset,EditMessages_OnActivation);
		pxNewOverlay->AddDialog(pxNewDialog);
		xOffset.y += Asura_Dialog::DefaultSpacing();
	}

	if(UsesDeactivationMessage())
	{
		sprintf(text,"On Deactivation (%d)",Asura_StaticMessage_System::GetNumberOfMessagesInBlock(pxAsuraEntity->MessageBlock[ASURA_TRIGGER_ENTITY_ONDEACTIVATION]));
		pxNewDialog = new Asura_Function_Dialog(text,xOffset,EditMessages_OnDeactivation);
		pxNewOverlay->AddDialog(pxNewDialog);
		xOffset.y += Asura_Dialog::DefaultSpacing();
	}

	if(UsesTriggerMessage())
	{
		sprintf(text,"On Trigger (%d)",Asura_StaticMessage_System::GetNumberOfMessagesInBlock(pxAsuraEntity->MessageBlock[ASURA_TRIGGER_ENTITY_ONBEINGTRIGGERED]));
		pxNewDialog = new Asura_Function_Dialog(text,xOffset,EditMessages_OnBeingTriggered);
		pxNewOverlay->AddDialog(pxNewDialog);
		xOffset.y += Asura_Dialog::DefaultSpacing();
	}

	if(UsesDestructionMessage())
	{
		sprintf(text,"On Destruction (%d)",Asura_StaticMessage_System::GetNumberOfMessagesInBlock(pxAsuraEntity->MessageBlock[ASURA_TRIGGER_ENTITY_ONDESTRUCTION]));
		pxNewDialog = new Asura_Function_Dialog(text,xOffset,EditMessages_OnDestruction);
		pxNewOverlay->AddDialog(pxNewDialog);
		xOffset.y += Asura_Dialog::DefaultSpacing();
	}

	if( UsesUnTriggerMessage() )
	{
		sprintf(text, "On Untrigger (%d)", Asura_StaticMessage_System::GetNumberOfMessagesInBlock(pxAsuraEntity->MessageBlock[ASURA_TRIGGER_ENTITY_ONUNTRIGGER]));
		pxNewDialog = new Asura_Function_Dialog(text, xOffset, EditMessages_OnUnTrigger);
		pxNewOverlay->AddDialog(pxNewDialog);
		xOffset.y += Asura_Dialog::DefaultSpacing();
	}
	#endif
}

void Tools_Entity_Trigger_Object::EditMessages_OnActivation()
{
	Asura_Assert(ThisEntity,"");
	Tools_Entity_Trigger_Object* toolsEntity = (Tools_Entity_Trigger_Object*) ThisEntity;
	Asura_ServerEntity_Trigger* asuraEntity = (Asura_ServerEntity_Trigger*) toolsEntity->Entity;

	Tools_Object_System::EditMessageBlock("On Activation",&asuraEntity->MessageBlock[ASURA_TRIGGER_ENTITY_ONACTIVATION],toolsEntity->IsClientSideTrigger());
}

void Tools_Entity_Trigger_Object::EditMessages_OnDeactivation()
{
	Asura_Assert(ThisEntity,"");
	Tools_Entity_Trigger_Object* toolsEntity = (Tools_Entity_Trigger_Object*) ThisEntity;
	Asura_ServerEntity_Trigger* asuraEntity = (Asura_ServerEntity_Trigger*) toolsEntity->Entity;

	Tools_Object_System::EditMessageBlock("On Deactivation",&asuraEntity->MessageBlock[ASURA_TRIGGER_ENTITY_ONDEACTIVATION],toolsEntity->IsClientSideTrigger());
}

void Tools_Entity_Trigger_Object::EditMessages_OnBeingTriggered()
{
	Asura_Assert(ThisEntity,"");
	Tools_Entity_Trigger_Object* toolsEntity = (Tools_Entity_Trigger_Object*) ThisEntity;
	Asura_ServerEntity_Trigger* asuraEntity = (Asura_ServerEntity_Trigger*) toolsEntity->Entity;

	Tools_Object_System::EditMessageBlock("On Trigger",&asuraEntity->MessageBlock[ASURA_TRIGGER_ENTITY_ONBEINGTRIGGERED],toolsEntity->IsClientSideTrigger());
}

void Tools_Entity_Trigger_Object::EditMessages_OnDestruction()
{
	Asura_Assert(ThisEntity,"");
	Tools_Entity_Trigger_Object* toolsEntity = (Tools_Entity_Trigger_Object*) ThisEntity;
	Asura_ServerEntity_Trigger* asuraEntity = (Asura_ServerEntity_Trigger*) toolsEntity->Entity;

	Tools_Object_System::EditMessageBlock("On Destruction",&asuraEntity->MessageBlock[ASURA_TRIGGER_ENTITY_ONDESTRUCTION],toolsEntity->IsClientSideTrigger());
}

void Tools_Entity_Trigger_Object::EditMessages_OnUnTrigger()
{
	Asura_Assert(ThisEntity, "");
	Tools_Entity_Trigger_Object* toolsEntity = (Tools_Entity_Trigger_Object*) ThisEntity;
	Asura_ServerEntity_Trigger* asuraEntity = (Asura_ServerEntity_Trigger*) toolsEntity->Entity;

	Tools_Object_System::EditMessageBlock("On Untrigger", &asuraEntity->MessageBlock[ASURA_TRIGGER_ENTITY_ONUNTRIGGER],toolsEntity->IsClientSideTrigger());
}

bool Tools_Entity_Trigger_Object::OwnsThisMessageBlock(unsigned short usBlockID)
{
	Asura_ServerEntity_Trigger* pxAsuraEntity = (Asura_ServerEntity_Trigger*) Entity;
	for(int i=0;i<ASURA_TRIGGER_ENTITY_NOOFSTATES;i++)
	{
		if(pxAsuraEntity->MessageBlock[i] == usBlockID) return true;
	}

	return Tools_Entity_Activatable_Object::OwnsThisMessageBlock(usBlockID);
}

bool Tools_Entity_Trigger_Object::GetEventDescForMessageBlock(unsigned short usBlockID,Asura_String& xOutputDesc)
{
	Asura_ServerEntity_Trigger* pxAsuraEntity = (Asura_ServerEntity_Trigger*) Entity;

	const char* aszStates[] =
	{
		"Activation",
		"Deactivation",
		"Trigger",
		"Destruction",
		"Untrigger",
	};
	const int iNumStates = sizeof(aszStates)/sizeof(const char*);
	Asura_Assert(iNumStates == ASURA_TRIGGER_ENTITY_NOOFSTATES,"");
		
	for(int i=0;i<iNumStates;i++)
	{
		if(pxAsuraEntity->MessageBlock[i] == usBlockID)
		{
			xOutputDesc = aszStates[i];
			return true;
		}
	}

	return false;

}


void Tools_Entity_Trigger_Object::Copy(const Tools_Object* pxObject)
{
	Tools_Entity_Activatable_Object::Copy(pxObject);

	const Tools_Entity_Trigger_Object* pxFromObject = (const Tools_Entity_Trigger_Object*) pxObject;

	Asura_ServerEntity_Trigger* pxFromEntity = (Asura_ServerEntity_Trigger*) pxFromObject->Entity;
	Asura_ServerEntity_Trigger* pxToEntity = (Asura_ServerEntity_Trigger*) Entity;

	//copy message blocks (deleting old ones as we go)
	for(int i=0;i<ASURA_TRIGGER_ENTITY_NOOFSTATES;i++)
	{
		Asura_StaticMessage_System::DeleteMessageBlock(pxToEntity->MessageBlock[i]);
		pxToEntity->MessageBlock[i] = Asura_StaticMessage_System::CopyMessageBlock(pxFromEntity->MessageBlock[i],pxFromObject->GetGuid(),GetGuid());
	}
}

void Tools_Entity_Trigger_Object::UpdateClient()
{
	if(!m_pxClientEntity) return;

	Tools_Entity_Activatable_Object::UpdateClient();

	Asura_ServerEntity_Trigger* pxServerEntity = (Asura_ServerEntity_Trigger*) Entity;
	Asura_ClientEntity_Trigger* pxClientEntity = (Asura_ClientEntity_Trigger*) m_pxClientEntity;

	for(int i=0;i<ASURA_TRIGGER_ENTITY_NOOFSTATES;i++)
	{
		pxClientEntity->MessageBlock[i] = pxServerEntity->MessageBlock[i];
	}
	
}


//////////////////////////////////////////////////////////////////////////////////////////////////
/*---------------------------------**
** Tools_Entity_TimeTrigger_Object **
**---------------------------------*/

Tools_Entity_TimeTrigger_Object::Tools_Entity_TimeTrigger_Object(Asura_Entity* entity)
:Tools_Entity_Trigger_Object(entity)
{
	Asura_Assert(entity->GetClassification()==AsuraEntityClass_TimeTrigger,"");
	Type = TOT_Entity_TimeTrigger;
	SetMesh("ToolsShape_TimeTrigger");
}

Tools_Entity_TimeTrigger_Object* Tools_Entity_TimeTrigger_Object::Create(bool bGenerateName)
{
	//get a name , and the position data
	Asura_Guid uGuid = Asura_Entity_System::GetNextToolsCreatedGuid();
	Tools_Entity_Data* toolsData = 0;
	if(bGenerateName)
	{
		toolsData = Tools_Entity_Object::CreateToolsDataWithDefaultName("Time Trigger","Time Trigger",uGuid,GetNamePrefix());
	}
	else
	{
		toolsData = Tools_Entity_Object::CreateToolsData("Time Trigger",0,GetNamePrefix());
	}
	if(!toolsData) return 0;

	//create the asura data
	Asura_ServerEntity_TimeTrigger* asuraEntity = new Asura_ServerEntity_TimeTrigger(0,true);
	asuraEntity->ToolsData = toolsData;

	//create the tools object
	Tools_Entity_TimeTrigger_Object* object = new Tools_Entity_TimeTrigger_Object(asuraEntity);

	Tools_SaveAsura::SetEntityModified();

	return object;
}



void Tools_Entity_TimeTrigger_Object::AddOverlayControls(class Asura_Overlay* newOverlay,class Asura_Vector_2& offset)
{
	Tools_Entity_Trigger_Object::AddOverlayControls(newOverlay,offset);

	Asura_ServerEntity_TimeTrigger* asuraEntity = (Asura_ServerEntity_TimeTrigger*) Entity;

	Asura_Dialog* newDialog;
	newDialog = new Asura_Numerical_Dialog("Time Delay",offset,&asuraEntity->Time,0,9999,0.1f,"%.3f",&Changed);
	newOverlay->AddDialog(newDialog);
	offset.y += Asura_Dialog::DefaultSpacing();

	newDialog = new Asura_FlagTickbox_Dialog("Looped",offset,&asuraEntity->m_uTimeTriggerFlags,ASURA_TIMETRIGGERFLAG_LOOP,&Changed);
	newDialog->SetHelpText("If looped then the timer won't deactivate when it triggers ,it will instead reset the timer and continue.");
	newOverlay->AddDialog(newDialog);
	offset.y += Asura_Dialog::DefaultSpacing();

	newDialog = new Asura_FlagTickbox_Dialog( "Pause During Camera Cutscenes",
											  offset,
											  &asuraEntity->m_uTimeTriggerFlags,
											  ASURA_TIMETRIGGERFLAG_PAUSE_DURING_CUTSCENES,
											  &Changed );

	newDialog->SetHelpText( "If ticked, the timer won't count down during cutscenes which control the camera." );
	newOverlay->AddDialog( newDialog );
	offset.y += Asura_FlagTickbox_Dialog::DefaultSpacing();
}

bool Tools_Entity_TimeTrigger_Object::HandlesThisMessage(int iMessageID)
{
	if(iMessageID == ASURA_MSG_RESET ||
		iMessageID == ASURA_MSG_PAUSETIMER ||
		iMessageID == ASURA_MSG_UNPAUSETIMER ||
		iMessageID == ASURA_MSG_SETTIMER ||
		iMessageID == ASURA_MSG_TIMETRIGGER_SETTRIGGERTIME ||
		iMessageID == ASURA_MSG_TIMETRIGGER_ADDTOTRIGGERTIME ||
		iMessageID == ASURA_MSG_TIMETRIGGER_ADDTIME) return true;
	return Tools_Entity_Trigger_Object::HandlesThisMessage(iMessageID);
}

void Tools_Entity_TimeTrigger_Object::UpdateWhileSelected()
{
	if(Changed)
	{
		//update time remaining from time delay
		Asura_ServerEntity_TimeTrigger* pxAsuraEntity = (Asura_ServerEntity_TimeTrigger*) Entity;
		pxAsuraEntity->m_fTimeRemaining = pxAsuraEntity->Time;
	}
	Tools_Entity_Trigger_Object::UpdateWhileSelected();
}

void Tools_Entity_TimeTrigger_Object::PrintInfo(float& x,float& y)
{
	Tools_Entity_Trigger_Object::PrintInfo(x,y);
	
	if(Tools_Entity_Simulation::IsActive())
	{
		Asura_ServerEntity_TimeTrigger* pxAsuraEntity = (Asura_ServerEntity_TimeTrigger*) Entity;
	
		//show timer
		Asura_Text_Output::Print(x,y++,0xffffff00,0,"Timer : %.3f / %.3f",pxAsuraEntity->Time-pxAsuraEntity->m_fTimeRemaining,pxAsuraEntity->Time);
	}

}

Tools_Object* Tools_Entity_TimeTrigger_Object::CreateObjectOfThisType() const
{
	return Create(true);	
}

void Tools_Entity_TimeTrigger_Object::Copy(const Tools_Object* pxObject)
{
	Tools_Entity_Trigger_Object::Copy(pxObject);

	const Tools_Entity_TimeTrigger_Object* pxFromObject = (const Tools_Entity_TimeTrigger_Object*) pxObject;

	Asura_ServerEntity_TimeTrigger* pxFromEntity = (Asura_ServerEntity_TimeTrigger*) pxFromObject->Entity;
	Asura_ServerEntity_TimeTrigger* pxToEntity = (Asura_ServerEntity_TimeTrigger*) Entity;

	pxToEntity->Time = pxFromEntity->Time;
	pxToEntity->m_fTimeRemaining = pxFromEntity->m_fTimeRemaining;
}

void Tools_Entity_TimeTrigger_Object::AddOverlayControls_SetTimer(struct Asura_StaticMessage* pxMessage,class Asura_Overlay* pxOverlay,class Asura_Vector_2& xOffset)
{
	Asura_Assert(pxMessage->m_pData,"");
	Asura_Message_SetTimer* pxData = (Asura_Message_SetTimer*) pxMessage->m_pData;

	Asura_Dialog* pxDialog = 0;

	pxDialog = new Asura_Numerical_Dialog("Timer",xOffset,&pxData->m_fTimer,0.0f,9999.0f,1.0f,"%.3f");
	pxDialog->SetHelpText("Set the current value of the timer. If increased then it can cause the messages to be triggered");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();
}

void Tools_Entity_TimeTrigger_Object::AddOverlayControls_SetTriggerTime(struct Asura_StaticMessage* pxMessage,class Asura_Overlay* pxOverlay,class Asura_Vector_2& xOffset)
{
	Asura_Assert(pxMessage->m_pData,"");
	Asura_Message_TimeTrigger_SetTriggerTime* pxData = (Asura_Message_TimeTrigger_SetTriggerTime*) pxMessage->m_pData;

	Asura_Dialog* pxDialog = 0;

	pxDialog = new Asura_Numerical_Dialog("Trigger Time",xOffset,&pxData->m_fTriggerTime,0.0f,9999.0f,1.0f,"%.3f");
	pxDialog->SetHelpText("The new trigger time");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();
}

void Tools_Entity_TimeTrigger_Object::AddOverlayControls_AddToTriggerTime(struct Asura_StaticMessage* pxMessage,class Asura_Overlay* pxOverlay,class Asura_Vector_2& xOffset)
{
	Asura_Assert(pxMessage->m_pData,"");
	Asura_Message_TimeTrigger_AddToTriggerTime* pxData = (Asura_Message_TimeTrigger_AddToTriggerTime*) pxMessage->m_pData;

	Asura_Dialog* pxDialog = 0;

	pxDialog = new Asura_Numerical_Dialog("Trigger time delta",xOffset,&pxData->m_fTriggerTimeDelta,-9999.0f,9999.0f,1.0f,"%.3f");
	pxDialog->SetHelpText("Amount to add/subtract to the trigger time.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();
}


void Tools_Entity_TimeTrigger_Object::AddOverlayControls_AddTime(struct Asura_StaticMessage* pxMessage,class Asura_Overlay* pxOverlay,class Asura_Vector_2& xOffset)
{
	Asura_Assert(pxMessage->m_pData,"");
	Asura_Message_TimeTrigger_AddTime* pxData = (Asura_Message_TimeTrigger_AddTime*) pxMessage->m_pData;

	Asura_Dialog* pxDialog = 0;

	pxDialog = new Asura_Numerical_Dialog("Time delta", xOffset, &pxData->m_fTimeDelta, -9999.0f, 9999.0f, 1.0f, "%.3f");
	pxDialog->SetHelpText("This amount will be added to the time remaining , so a positive value will delay the trigger and a negative value will cause it to trigger earlier.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();
}



//////////////////////////////////////////////////////////////////////////////////////////////////
/*------------------------------------**
** Tools_Entity_LookAtTrigger_Object **
**------------------------------------*/

#define CALLBACK_BOOL(__Function) Asura_Callback_MemberFunction<bool,Tools_Entity_LookAtTrigger_Object>(this, &Tools_Entity_LookAtTrigger_Object::__Function) 


const char* Tools_Entity_LookAtTrigger_Object::s_aszNameList[4] = 
{
	"All players",
	"All non-players",
	"All actors",
	"Specified player",
};

bool Tools_Entity_LookAtTrigger_Object::s_bShowCone = false;
bool Tools_Entity_LookAtTrigger_Object::s_bWireframeCone = true;

Tools_Entity_LookAtTrigger_Object::Tools_Entity_LookAtTrigger_Object(Asura_Entity* pxEntity)
:Tools_Entity_Trigger_Object(pxEntity)
{
	Asura_Assert(pxEntity->GetClassification()==AsuraEntityClass_LookAtTrigger,"");
	Type = TOT_Entity_LookAtTrigger;
	Mesh = Asura_Mesh_System::GetMesh("ToolsShape_CutsceneCameraTarget");

	Asura_ServerEntity_LookAtTrigger* pxAsuraEntity = static_cast<Asura_ServerEntity_LookAtTrigger*>(pxEntity);
	m_fMinAngle = Asura_Maths::ArcCosine(pxAsuraEntity->m_fCosMinAngle);
	m_fMaxAngle = Asura_Maths::ArcCosine(pxAsuraEntity->m_fCosMaxAngle);
	// Since 0 degrees is also being used as ignore, err on the safe side of floating point errors by forcing values
	m_fInwardConeMaxAngle = (pxAsuraEntity->m_fInwardConeCosMaxAngle >= 1.0f) ? 0.0f : Asura_Maths::ArcCosine(pxAsuraEntity->m_fInwardConeCosMaxAngle);
	m_iListOffset = 0;
	switch (pxAsuraEntity->m_uTriggeringTypes)
	{
	default:
	case ASURA_LOOKATTRIGGER_TYPE_ALLPLAYERS:
		m_iListSelection = 0;
		break;
	case ASURA_LOOKATTRIGGER_TYPE_ALLOTHERACTORS:
		m_iListSelection = 1;
		break;
	case ASURA_LOOKATTRIGGER_TYPE_ALLACTORS:
		m_iListSelection = 2;
		break;
	case ASURA_LOOKATTRIGGER_TYPE_SPECIFICPLAYER:
		m_iListSelection = 3;
		break;
	}
	m_pxMinAngleDialog = NULL;
	m_pxMaxAngleDialog = NULL;
	m_pxEdgeDeadzoneDialog = NULL;
	m_pxCentreDeadzoneDialog = NULL;
	m_pxSpecifyPlayerDialog = NULL;
	m_bCameraPassesTests = false;
	Asura_Maths::OrientationFromDirection(Orientation, pxAsuraEntity->m_xDirection);
}

void Tools_Entity_LookAtTrigger_Object::Create()
{
	Tools_Object* pxObject = Create(false);
	Tools_Object_System::SelectObject(pxObject);
}

Tools_Entity_LookAtTrigger_Object* Tools_Entity_LookAtTrigger_Object::Create(bool bGenerateName)
{
	//get a name , and the position data
	Asura_Guid uGuid = Asura_Entity_System::GetNextToolsCreatedGuid();
	Tools_Entity_Data* pxToolsData = 0;
	if(bGenerateName)
	{
		pxToolsData = Tools_Entity_Object::CreateToolsDataWithDefaultName("Look-at Trigger","Look-at Trigger",uGuid,GetNamePrefix());
	}
	else
	{
		pxToolsData = Tools_Entity_Object::CreateToolsData("Look-at Trigger",0,GetNamePrefix());
	}
	if(!pxToolsData) return 0;

	//create the asura data
	Asura_ServerEntity_LookAtTrigger* pxAsuraEntity = new Asura_ServerEntity_LookAtTrigger(true);
	pxAsuraEntity->ToolsData = pxToolsData;

	//create the tools object
	Tools_Entity_LookAtTrigger_Object* pxObject = new Tools_Entity_LookAtTrigger_Object(pxAsuraEntity);

	Tools_SaveAsura::SetEntityModified();

	return pxObject;
}

void Tools_Entity_LookAtTrigger_Object::AddOverlayControls(class Asura_Overlay* pxNewOverlay, class Asura_Vector_2& xOffset)
{
	Tools_Entity_Trigger_Object::AddOverlayControls(pxNewOverlay, xOffset);

	Asura_ServerEntity_LookAtTrigger* pxAsuraEntity = static_cast<Asura_ServerEntity_LookAtTrigger*>(Entity);

	Asura_Dialog* pxNewDialog;

	pxNewDialog = new Asura_Numerical_Dialog("Max distance", xOffset, &pxAsuraEntity->m_fMaxDistance, 0.0f, 10000.0f, 0.1f, "%.1f", &Changed);
	pxNewDialog->SetHelpText("Maximum distance from the actor to the trigger entity.");
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxNewDialog = new Asura_Numerical_Dialog("Min distance", xOffset, &pxAsuraEntity->m_fMinDistance, 0.0f, 10000.0f, 0.1f, "%.1f", &Changed);
	pxNewDialog->SetHelpText("Minimum distance from the actor to the trigger entity.");
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

    pxNewDialog = new Asura_FlagTickbox_Dialog("Use screen-relative tests", xOffset, &pxAsuraEntity->m_uLookAtFlags, 0x1, &Changed);
	pxNewDialog->SetHelpText("Chooses whether the test is against a section of the screen, or against the angle between the actor the trigger entity.");
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxNewDialog = new Asura_AngleNumerical_Dialog("Max angle", xOffset, &m_fMaxAngle, 0.0f, 180.0f, 1.0f, "%.2f", &Changed);
	m_pxMaxAngleDialog = pxNewDialog;
	pxNewDialog->SetHelpText("Maximum angle (in degrees) between the direction faced by the current actor and the direction from the actor to the trigger entity.");
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxNewDialog = new Asura_AngleNumerical_Dialog("Min angle", xOffset, &m_fMinAngle, 0.0f, 180.0f, 1.0f, "%.2f", &Changed);
	m_pxMinAngleDialog = pxNewDialog;
	pxNewDialog->SetHelpText("Minimum angle (in degrees) between the direction faced by the current actor and the direction from the actor to the trigger entity.");
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxNewDialog = new Asura_Numerical_Dialog("Screen edge deadzone", xOffset, &pxAsuraEntity->m_fScreenEdgeDeadzone, 0.0f, 1.0f, 0.01f, "%.2f", &Changed);
	m_pxEdgeDeadzoneDialog = pxNewDialog;
	pxNewDialog->SetHelpText("Size of border from edge of screen to centre that does not count for looking at the trigger.");
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxNewDialog = new Asura_Numerical_Dialog("Screen centre deadzone", xOffset, &pxAsuraEntity->m_fScreenCentreDeadzone, 0.0f, 1.0f, 0.01f, "%.2f", &Changed);
	m_pxCentreDeadzoneDialog = pxNewDialog;
	pxNewDialog->SetHelpText("Size of border from centre of screen to edge that does not count for looking at the trigger.");
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	if (pxAsuraEntity->m_bUseScreenArea)
	{
		m_pxMaxAngleDialog->SetEnabled(false);
		m_pxMinAngleDialog->SetEnabled(false);
	}
	else
	{
		m_pxEdgeDeadzoneDialog->SetEnabled(false);
		m_pxCentreDeadzoneDialog->SetEnabled(false);
	}

    pxNewDialog = new Asura_FlagTickbox_Dialog("Use LOS test", xOffset, &pxAsuraEntity->m_uLookAtFlags, 0x2, &Changed);
	pxNewDialog->SetHelpText("Do a line of sight test to check for environment blocking view of the trigger.");
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

    pxNewDialog = new Asura_FlagTickbox_Dialog("Trigger on not looking", xOffset, &pxAsuraEntity->m_uLookAtFlags, 0x20, &Changed);
	pxNewDialog->SetHelpText("This becomes triggered based on the number of players that aren't looking at it.");
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();


	pxNewDialog = new Asura_Numerical_Dialog("Time to trigger", xOffset, &pxAsuraEntity->m_fTime, 0.0f, 10000.0f, 0.1f, "%.1f", &Changed);
	pxNewDialog->SetHelpText("Time this trigger must be looked at before a trigger event occurs.");
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

    pxNewDialog = new Asura_FlagTickbox_Dialog("Reset time on look away", xOffset, &pxAsuraEntity->m_uLookAtFlags, 0x4, &Changed);
	pxNewDialog->SetHelpText("If the triggering time should be reset once an actor stops looking at the trigger.");
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();

	pxNewDialog = new Asura_AngleNumerical_Dialog("Max angle from trigger to entity", xOffset, &m_fInwardConeMaxAngle, 0.0f, 180.0f, 1.0f, "%.2f",&Changed);
	pxNewDialog->SetHelpText("Maximum angle (in degrees) between the direction faced by the trigger entity and the direction from the trigger entity to the actor. Use to limit where the actor can successfully look at this trigger. Set to zero to ignore.");
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();
	
	// (AMM 7/11/08) allow user to view look-at angle as a cone (toggle)
	pxNewDialog = new Asura_Tickbox_Dialog("Show Cone",xOffset,&s_bShowCone);
	pxNewDialog->SetHelpText("Draws a cone to show look-at angle");
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	// (AMM 7/11/08) allow user to view look-at angle cone as wireframe (toggle)
	pxNewDialog = new Asura_Tickbox_Dialog("Wireframe",xOffset,&s_bWireframeCone);
	pxNewDialog->SetHelpText("Causes cone to be drawn wireframe , if being shown");
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.y+=Asura_Dialog::LargeSpacing();


    pxNewDialog = new Asura_FlagTickbox_Dialog("Adjust distance for FOV", xOffset, &pxAsuraEntity->m_uLookAtFlags, 0x8, &Changed);
	pxNewDialog->SetHelpText("Adjust testing distance for camera FOV settings (allows triggers to be set off by zooming into range, rather than just moving within range).");
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing() * 1.5f;

	pxNewDialog = new Asura_List_Dialog(xOffset, s_aszNameList, &m_iListOffset, &m_iListSelection, 4, 4, NULL);
	pxNewDialog->SetHelpText("Types of entity that can trigger this.");
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.y = pxNewDialog->m_xMaxCoord.y;
	xOffset.y += 0.5f;

	pxNewDialog = new Asura_Numerical_Dialog("Specified player", xOffset, &pxAsuraEntity->m_iSpecifiedPlayer, 0, 16, 1, "%d", &Changed);
	pxNewDialog->SetHelpText("Set the player which is the only one allowed to trigger this.");
	m_pxSpecifyPlayerDialog = pxNewDialog;
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing() * 1.5f;
	pxNewDialog->SetIsEnabledFunction(CALLBACK_BOOL(IsEnabled_SpecifiedPlayer));

	pxNewDialog = new Asura_Numerical_Dialog("Num inputs required", xOffset, &pxAsuraEntity->m_iNumberRequired, 0, 1000, 1, "%d", &Changed);
	pxNewDialog->SetHelpText("Number of entities currently correctly looking at this trigger entity needed to cause a trigger event.");
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();
}

void Tools_Entity_LookAtTrigger_Object::UpdateWhileSelected()
{
	Asura_ServerEntity_LookAtTrigger* pxAsuraEntity = static_cast<Asura_ServerEntity_LookAtTrigger*>(Entity);

	if (!pxAsuraEntity->m_xPosition.ApproximatelyEquals(Position))
	{
		pxAsuraEntity->m_xPosition = Position;
		Changed = true;
	}
	Asura_Vector_3 xDirection(Orientation.e13, Orientation.e23, Orientation.e33);
	if (!pxAsuraEntity->m_xDirection.ApproximatelyEquals(xDirection))
	{
		pxAsuraEntity->m_xDirection = xDirection;
		Changed = true;
	}

	if (Changed)
	{
		// Update our angles
		pxAsuraEntity->m_fCosMinAngle = Asura_Maths::Cosine(m_fMinAngle);
		pxAsuraEntity->m_fCosMaxAngle = Asura_Maths::Cosine(m_fMaxAngle);
		// Since 0 degrees is also being used as ignore, err on the safe side of floating point errors by forcing values
		pxAsuraEntity->m_fInwardConeCosMaxAngle = (m_fInwardConeMaxAngle <= 0.0f) ? 1.0f : Asura_Maths::Cosine(m_fInwardConeMaxAngle);
		// Update which dialogs should be active
		if (pxAsuraEntity->m_bUseScreenArea)
		{
			m_pxMaxAngleDialog->SetEnabled(false);
			m_pxMinAngleDialog->SetEnabled(false);
			m_pxEdgeDeadzoneDialog->SetEnabled(true);
			m_pxCentreDeadzoneDialog->SetEnabled(true);
		}
		else
		{
			m_pxMaxAngleDialog->SetEnabled(true);
			m_pxMinAngleDialog->SetEnabled(true);
			m_pxEdgeDeadzoneDialog->SetEnabled(false);
			m_pxCentreDeadzoneDialog->SetEnabled(false);
		}
		// Update the triggering types
		switch (m_iListSelection)
		{
		default:
		case 0:
			pxAsuraEntity->m_uTriggeringTypes = ASURA_LOOKATTRIGGER_TYPE_ALLPLAYERS;
			break;
		case 1:
			pxAsuraEntity->m_uTriggeringTypes = ASURA_LOOKATTRIGGER_TYPE_ALLOTHERACTORS;
			break;
		case 2:
			pxAsuraEntity->m_uTriggeringTypes = ASURA_LOOKATTRIGGER_TYPE_ALLACTORS;
			break;
		case 3:
			pxAsuraEntity->m_uTriggeringTypes = ASURA_LOOKATTRIGGER_TYPE_SPECIFICPLAYER;
			break;
		}
		m_pxSpecifyPlayerDialog->SetEnabled(pxAsuraEntity->m_uTriggeringTypes == ASURA_LOOKATTRIGGER_TYPE_SPECIFICPLAYER);
		//make sure the current state is reset to initial state
		pxAsuraEntity->m_fTimeRemaining = pxAsuraEntity->m_fTime;

		// If the client has been created, update that too as otherwise that won't happen until you return to tools from the game
		Asura_ClientEntity_LookAtTrigger* pxClient = static_cast<Asura_ClientEntity_LookAtTrigger*>(Asura_ClientEntity_LookAtTrigger::FindEntity(pxAsuraEntity->GetGuid()));
		if (pxClient)
		{
			pxClient->SetEntityActive(pxAsuraEntity->IsActive());
			pxClient->m_bAdjustForFOV = pxAsuraEntity->m_bAdjustForFOV;
			pxClient->m_bUseLOS = pxAsuraEntity->m_bUseLOS;
			pxClient->m_bTriggerOnNotLooking = pxAsuraEntity->m_bTriggerOnNotLooking;
			pxClient->m_bUseScreenArea = pxAsuraEntity->m_bUseScreenArea;
			pxClient->m_fMaxAngleOrEdgeDeadzone = pxAsuraEntity->m_bUseScreenArea ? pxAsuraEntity->m_fScreenEdgeDeadzone : pxAsuraEntity->m_fCosMaxAngle;
			pxClient->m_fMinAngleOrCentreDeadzone = pxAsuraEntity->m_bUseScreenArea ? pxAsuraEntity->m_fScreenCentreDeadzone : pxAsuraEntity->m_fCosMinAngle;
			pxClient->m_fInwardConeCosMaxAngle = pxAsuraEntity->m_fInwardConeCosMaxAngle;
			pxClient->m_fMaxDistance = pxAsuraEntity->m_fMaxDistance;
			pxClient->m_fMinDistance = pxAsuraEntity->m_fMinDistance;
			pxClient->m_iSpecifiedPlayer = (pxAsuraEntity->m_uTriggeringTypes == ASURA_LOOKATTRIGGER_TYPE_SPECIFICPLAYER) ? pxAsuraEntity->m_iSpecifiedPlayer : ((pxAsuraEntity->m_uTriggeringTypes & ASURA_LOOKATTRIGGER_TYPE_ALLPLAYERS) ? -1 : -2);
			pxClient->m_xDirection = pxAsuraEntity->m_xDirection;
			pxClient->m_xPosition = pxAsuraEntity->m_xPosition;
		}
	}

	// See if the camera currently passes the tests
	Asura_Vector_3 xCameraPosition = *Asura_Camera::GetPosition();
	Asura_Vector_3 xCameraDirection;
	Asura_Camera::GetForwardVector(xCameraDirection);

	Asura_Vector_3 xCameraToTriggerDirection = Position - xCameraPosition;
	float fCameraToTriggerDistance = xCameraToTriggerDirection.Normalise();

	float fAdjustedMaxDistance = pxAsuraEntity->m_fMaxDistance;
	float fAdjustedMinDistance = pxAsuraEntity->m_fMinDistance;
	if (pxAsuraEntity->m_bAdjustForFOV)
	{
		float fFOVAdjustment = Asura_Camera::GetHorizontalFieldOfView() * fLOOKAT_TRIGGER_ONE_OVER_STANDARD_FOV;
		fAdjustedMaxDistance /= fFOVAdjustment;
		fAdjustedMinDistance /= fFOVAdjustment;
	}

	if ((fCameraToTriggerDistance > fAdjustedMaxDistance) || (fCameraToTriggerDistance < fAdjustedMinDistance))
	{
		m_bCameraPassesTests = false;
	}
	else
	{
		bool bGoToInwardConeStage = false;
		bool bGoToLOSStage = false;

		if (pxAsuraEntity->m_bUseScreenArea)
		{
			Asura_Vector_3 xScreenSpace = Position;
			Asura_Camera::ProjectWorldCoordsToScreenSpace(xScreenSpace);
			if (xScreenSpace.z > 0.0f) // In front of camera
			{
				float fActualFromCentreLivezone = (1.0f - pxAsuraEntity->m_fScreenEdgeDeadzone) * 0.5f;
				float fActualCentreDeadzone = pxAsuraEntity->m_fScreenCentreDeadzone * 0.5f;
				float fXFromCentre = Asura_Maths::Abs(xScreenSpace.x - 0.5f);
				float fYFromCentre = Asura_Maths::Abs(xScreenSpace.y - 0.5f);
				bGoToInwardConeStage = ((fXFromCentre <= fActualFromCentreLivezone) && (fYFromCentre <= fActualFromCentreLivezone)
										&& ((fXFromCentre >= fActualCentreDeadzone) || (fYFromCentre >= fActualCentreDeadzone)));
			}
		}
		else
		{
			float fCosAngle = xCameraDirection * xCameraToTriggerDirection;
			bGoToInwardConeStage = ((fCosAngle >= pxAsuraEntity->m_fCosMaxAngle) && (fCosAngle <= pxAsuraEntity->m_fCosMinAngle));
		}

		if (bGoToInwardConeStage)
		{
			if (pxAsuraEntity->m_fInwardConeCosMaxAngle >= 1.0f)
			{
				bGoToLOSStage = true;
			}
			else
			{
				float fCosOtherAngle = pxAsuraEntity->m_xDirection * -xCameraToTriggerDirection;
				bGoToLOSStage = (fCosOtherAngle >= pxAsuraEntity->m_fInwardConeCosMaxAngle);
			}
		}

		if (bGoToLOSStage)
		{
			if (pxAsuraEntity->m_bUseLOS)
			{
				Asura_LOS_InputData xInput(xCameraPosition, xCameraToTriggerDirection, ASURA_GUID_UNREGISTERED, fCameraToTriggerDistance);
				const Asura_LOS_OutputData* pxOutput = Asura_LOS::TestAgainstEnvironment(&xInput);
				m_bCameraPassesTests = !pxOutput->m_bHitEnvironment;
			}
			else
			{
				m_bCameraPassesTests = true;
			}
		}
		else
		{
			m_bCameraPassesTests = false;
		}
	}
	
	Tools_Entity_Trigger_Object::UpdateWhileSelected();
}

Tools_Object* Tools_Entity_LookAtTrigger_Object::CreateObjectOfThisType() const
{
	return Create(true);	
}

void Tools_Entity_LookAtTrigger_Object::Copy(const Tools_Object* pxObject)
{
	Tools_Entity_Trigger_Object::Copy(pxObject);

	const Tools_Entity_LookAtTrigger_Object* pxFromObject = static_cast<const Tools_Entity_LookAtTrigger_Object*>(pxObject);

	Asura_ServerEntity_LookAtTrigger* pxFromEntity = static_cast<Asura_ServerEntity_LookAtTrigger*>(pxFromObject->Entity);
	Asura_ServerEntity_LookAtTrigger* pxToEntity = static_cast<Asura_ServerEntity_LookAtTrigger*>(Entity);

	pxToEntity->m_fMaxDistance = pxFromEntity->m_fMaxDistance;
	pxToEntity->m_fMinDistance = pxFromEntity->m_fMinDistance;
	pxToEntity->m_fCosMaxAngle = pxFromEntity->m_fCosMaxAngle;
	pxToEntity->m_fCosMinAngle = pxFromEntity->m_fCosMinAngle;
	pxToEntity->m_bUseLOS = pxFromEntity->m_bUseLOS;
	pxToEntity->m_bTriggerOnNotLooking = pxFromEntity->m_bTriggerOnNotLooking;
	pxToEntity->m_fTime = pxFromEntity->m_fTime;
	pxToEntity->m_bResetTimeOnEarlyLookAway = pxFromEntity->m_bResetTimeOnEarlyLookAway;
	pxToEntity->m_fInwardConeCosMaxAngle = pxFromEntity->m_fInwardConeCosMaxAngle;
	pxToEntity->m_bAdjustForFOV = pxFromEntity->m_bAdjustForFOV;
	pxToEntity->m_uTriggeringTypes = pxFromEntity->m_uTriggeringTypes;
	pxToEntity->m_iSpecifiedPlayer = pxFromEntity->m_iSpecifiedPlayer;
	pxToEntity->m_iNumberRequired = pxFromEntity->m_iNumberRequired;
	pxToEntity->m_fTimeRemaining = pxFromEntity->m_fTimeRemaining;

	// DF 02/11/06 - Fix up the local tools data thats setup to control overlay values.
	m_fMinAngle = Asura_Maths::ArcCosine(pxFromEntity->m_fCosMinAngle);
	m_fMaxAngle = Asura_Maths::ArcCosine(pxFromEntity->m_fCosMaxAngle);

	switch (pxFromEntity->m_uTriggeringTypes)
	{
		default:
		case ASURA_LOOKATTRIGGER_TYPE_ALLPLAYERS:
			m_iListSelection = 0;
			break;
		case ASURA_LOOKATTRIGGER_TYPE_ALLOTHERACTORS:
			m_iListSelection = 1;
			break;
		case ASURA_LOOKATTRIGGER_TYPE_ALLACTORS:
			m_iListSelection = 2;
			break;
		case ASURA_LOOKATTRIGGER_TYPE_SPECIFICPLAYER:
			m_iListSelection = 3;
			break;
	}
}

void Tools_Entity_LookAtTrigger_Object::PrintInfo(float& x,float& y)
{
	Tools_Entity_Trigger_Object::PrintInfo(x, y);

	// Display if the camera is passing the tests
	Asura_ServerEntity_LookAtTrigger* pxAsuraEntity = static_cast<Asura_ServerEntity_LookAtTrigger*>(Entity);

	if (pxAsuraEntity->m_bUseScreenArea)
	{
		if (pxAsuraEntity->m_bUseLOS)
		{
			Asura_Text_Output::Print(x, y++, 0xFFFFFFFF, 0, "Camera passes distance, screen position and LOS tests :");
		}
		else
		{
			Asura_Text_Output::Print(x, y++, 0xFFFFFFFF, 0, "Camera passes distance and screen position tests :");
		}
	}
	else
	{
		if (pxAsuraEntity->m_bUseLOS)
		{
			Asura_Text_Output::Print(x, y++, 0xFFFFFFFF, 0, "Camera passes distance, angle and LOS tests :");
		}
		else
		{
			Asura_Text_Output::Print(x, y++, 0xFFFFFFFF, 0, "Camera passes distance and angle tests :");
		}
	}

	if (m_bCameraPassesTests)
	{
		Asura_Text_Output::Print(x+3, y++, 0xFF66FF66, 0, "Yes");
	}
	else
	{
		Asura_Text_Output::Print(x+3, y++, 0xFFFF6666, 0, "No");
	}
}

// JAT - 12:03 27/07/06
// Need a look at trigger specific version otherwise client data gets trashed
void Tools_Entity_LookAtTrigger_Object::UpdateClient()
{
	if ( !m_pxClientEntity ) return;

	Tools_Entity_Activatable_Object::UpdateClient();

	Asura_ServerEntity_LookAtTrigger* pxServerEntity = static_cast<Asura_ServerEntity_LookAtTrigger*>( Entity );
	Asura_ClientEntity_LookAtTrigger* pxClient = static_cast<Asura_ClientEntity_LookAtTrigger*>(Asura_ClientEntity_LookAtTrigger::FindEntity(pxServerEntity->GetGuid()));
	if (pxClient)
	{
		pxClient->SetEntityActive(pxServerEntity->IsActive());
		pxClient->m_bAdjustForFOV = pxServerEntity->m_bAdjustForFOV;
		pxClient->m_bUseLOS = pxServerEntity->m_bUseLOS;
		pxClient->m_bTriggerOnNotLooking = pxServerEntity->m_bTriggerOnNotLooking;
		pxClient->m_bUseScreenArea = pxServerEntity->m_bUseScreenArea;
		pxClient->m_fMaxAngleOrEdgeDeadzone = pxServerEntity->m_bUseScreenArea ? pxServerEntity->m_fScreenEdgeDeadzone : pxServerEntity->m_fCosMaxAngle;
		pxClient->m_fMinAngleOrCentreDeadzone = pxServerEntity->m_bUseScreenArea ? pxServerEntity->m_fScreenCentreDeadzone : pxServerEntity->m_fCosMinAngle;
		pxClient->m_fInwardConeCosMaxAngle = pxServerEntity->m_fInwardConeCosMaxAngle;
		pxClient->m_fMaxDistance = pxServerEntity->m_fMaxDistance;
		pxClient->m_fMinDistance = pxServerEntity->m_fMinDistance;
		pxClient->m_iSpecifiedPlayer = (pxServerEntity->m_uTriggeringTypes == ASURA_LOOKATTRIGGER_TYPE_SPECIFICPLAYER) ? pxServerEntity->m_iSpecifiedPlayer : ((pxServerEntity->m_uTriggeringTypes & ASURA_LOOKATTRIGGER_TYPE_ALLPLAYERS) ? -1 : -2);
		pxClient->m_xDirection = pxServerEntity->m_xDirection;
		pxClient->m_xPosition = pxServerEntity->m_xPosition;
	}

}

bool Tools_Entity_LookAtTrigger_Object::IsEnabled_SpecifiedPlayer()
{
	return (m_iListSelection == 3);
}


#ifdef ASURA_USE_DEBUG_MESSAGE_TRIGGER

//////////////////////////////////////////////////////////////////////////////////////////////////
/*------------------------------------**
** Tools_Entity_DebugMessageTrigger_Object **
**------------------------------------*/

Tools_Entity_DebugMessageTrigger_Object::Tools_Entity_DebugMessageTrigger_Object(Asura_Entity* pxEntity)
:Tools_Entity_Trigger_Object(pxEntity)
{
	Asura_Assert(pxEntity->GetClassification()==AsuraEntityClass_DebugMessageTrigger,"");
	Type = TOT_Entity_DebugMessageTrigger;
	Mesh = Asura_Mesh_System::GetMesh("ToolsShape_DebugMessage");
}


void Tools_Entity_DebugMessageTrigger_Object::Create()
{
	Tools_Object* pxObject = Create(false);
	Tools_Object_System::SelectObject(pxObject);
}

Tools_Entity_DebugMessageTrigger_Object* Tools_Entity_DebugMessageTrigger_Object::Create(bool bGenerateName)
{
	//get a name , and the position data
	Asura_Guid uGuid = Asura_Entity_System::GetNextToolsCreatedGuid();
	Tools_Entity_Data* pxToolsData = 0;
	if(bGenerateName)
	{
		pxToolsData = Tools_Entity_Object::CreateToolsDataWithDefaultName("DebugMessage Trigger","DebugMessage Trigger",uGuid,GetNamePrefix());
	}
	else
	{
		pxToolsData = Tools_Entity_Object::CreateToolsData("DebugMessage Trigger",0,GetNamePrefix());
	}
	if(!pxToolsData) return 0;

	//create the asura data
	Asura_ServerEntity_DebugMessageTrigger* pxAsuraEntity = new Asura_ServerEntity_DebugMessageTrigger();
	pxAsuraEntity->ToolsData = pxToolsData;
	//JCS

	//create the tools object
	Tools_Entity_DebugMessageTrigger_Object* pxObject = new Tools_Entity_DebugMessageTrigger_Object(pxAsuraEntity);

	Tools_SaveAsura::SetEntityModified();

	return pxObject;
}

void Tools_Entity_DebugMessageTrigger_Object::AddOverlayControls(class Asura_Overlay* pxNewOverlay,class Asura_Vector_2& xOffset)
{
	Tools_Entity_Trigger_Object::AddOverlayControls(pxNewOverlay,xOffset);

	Asura_ServerEntity_DebugMessageTrigger* pxAsuraEntity = (Asura_ServerEntity_DebugMessageTrigger*) Entity;

	Asura_Dialog* pxNewDialog;
	Asura_String_Dialog* pxStringDialog;

	pxNewDialog = pxStringDialog = new Asura_String_Dialog("Text To Display",xOffset,&pxAsuraEntity->m_xStringToDisplay,40,&Changed);
	pxStringDialog->SetMaxStringLength( 255 );
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxNewDialog = new Asura_Numerical_Dialog("Time To Display Text",xOffset,&pxAsuraEntity->m_fTimeToDisplayString,-1.0f,100.0f,1.0f,"%.3f",&Changed);
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.x += 25.0f;

	pxNewDialog = new Asura_FlagTickbox_Dialog("Show text in world",xOffset,&pxAsuraEntity->m_iDrawInWorld,0xFFFFFFFF,&Changed);
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.x -= 25.0f;
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxNewDialog = new Asura_Numerical_Dialog("Pre-display time",xOffset,&pxAsuraEntity->m_fPreDisplayTime,0.0f,100.0f,1.0f,"%.3f",&Changed);
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxNewDialog = new Tools_ColourPicker_Dialog("Text Colour",xOffset, Asura_Vector_2(4.0f,3.0f),&pxAsuraEntity->m_fR,&pxAsuraEntity->m_fG,&pxAsuraEntity->m_fB,255.0f,&Changed);
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();
}


bool Tools_Entity_DebugMessageTrigger_Object::HandlesThisMessage(int iMessageID)
{
	if(iMessageID == ASURA_MSG_RESET) return true;
	return Tools_Entity_Trigger_Object::HandlesThisMessage(iMessageID);
}

void Tools_Entity_DebugMessageTrigger_Object::UpdateWhileSelected()
{
	if(Changed)
	{
		//Asura_ServerEntity_DebugMessageTrigger* pxAsuraEntity = (Asura_ServerEntity_DebugMessageTrigger*) Entity;
		//make sure the current state is reset to initial state
//		pxAsuraEntity->m_iRemainingInputsBeforeTrigger = pxAsuraEntity->m_iNumInputsBeforeTrigger;
	}

	Tools_Entity_Trigger_Object::UpdateWhileSelected();
}

void Tools_Entity_DebugMessageTrigger_Object::PrintInfo(float& fX,float& fY)
{
	Tools_Entity_Trigger_Object::PrintInfo(fX,fY);
}

Tools_Object* Tools_Entity_DebugMessageTrigger_Object::CreateObjectOfThisType() const
{
	return Create(true);	
}

void Tools_Entity_DebugMessageTrigger_Object::Copy(const Tools_Object* pxObject)
{
	Tools_Entity_Trigger_Object::Copy(pxObject);

	const Tools_Entity_DebugMessageTrigger_Object* pxFromObject = (const Tools_Entity_DebugMessageTrigger_Object*) pxObject;

	Asura_ServerEntity_DebugMessageTrigger* pxFromEntity = (Asura_ServerEntity_DebugMessageTrigger*) pxFromObject->Entity;
	Asura_ServerEntity_DebugMessageTrigger* pxToEntity = (Asura_ServerEntity_DebugMessageTrigger*) Entity;

	pxToEntity->m_xStringToDisplay = pxFromEntity->m_xStringToDisplay;
	pxToEntity->m_fTimeToDisplayString = pxFromEntity->m_fTimeToDisplayString;
	pxToEntity->m_fPreDisplayTime = pxFromEntity->m_fPreDisplayTime;
	pxToEntity->m_fR = pxFromEntity->m_fR;
	pxToEntity->m_fG = pxFromEntity->m_fG;
	pxToEntity->m_fB = pxFromEntity->m_fB;
	pxToEntity->m_iDrawInWorld = pxFromEntity->m_iDrawInWorld;
}

#endif //ASURA_USE_DEBUG_MESSAGE_TRIGGER
