//
// Tools_SecondaryEntityFile_System.cpp
//
// Created by Richard Rice - 21/04/2005
// 
// Copyright (C) 2005 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"
#include "Entity/Tools_SecondaryEntityFile_System.h"

#include "Camera/Asura_Camera.h"
#include "ChunkSystem/Asura_SaveBlock.h"
#include "ChunkSystem/Asura_Chunk_Entity.h"
#include "ChunkSystem/Asura_Chunk_Texture.h"
#include "ChunkSystem/Asura_Chunk_Phonons.h"
#include "Core/Asura_UserInput.h"
#include "Entity/Asura_Entity.h"
#include "Entity/Asura_StaticMessage_System.h"
#include "Interface/Asura_Overlay_System.h"
#include "Interface/Asura_Overlay.h"
#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Tree_Dialog.h"


#include "Entity/Tools_Entity_Data.h"
#include "Entity/Tools_Entity_Object.h"
#include "Entity/Tools_Entity_PhysicalObject.h"
#include "EntityTemplate/Tools_EntityTemplate_System.h"
#include "EntityTemplate/Tools_Entity_Template_Object.h"
#include "Interface/Tools_ContextMenu_System.h"
#include "Lighting/Tools_LightSets.h"
#include "Load&Save/Tools_SaveAsura.h"
#include "Load&Save/Tools_AsuraFileContents.h"
#include "Load&Save/Tools_LoadedFileList.h"
#include "Misc/Tools_Modal_Overlay.h"
#include "Misc/Tools_Timed_Messages.h"
#include "Movement/Tools_Movement.h"
#include "Objects/Tools_Mode_Object.h"
#include "Objects/Tools_Object.h"
#include "Objects/Tools_Object_System.h"
#include "PC/FileSystem/Tools_FileSystem.h"
#include "Shapes/Tools_Shapes.h"
#include "Sound/Tools_PickSound.h"
#include "Tools_Overlays.h"

#ifdef TOOLS_USE_OLD_ENTITY_MANAGER
#define TOOLS_SECONDARY_ENTITY_DISABLED_COLOUR (0xaaaaaa)
#else
#define TOOLS_SECONDARY_ENTITY_DISABLED_COLOUR (0xff0000)
#endif
#define TOOLS_SECONDARY_ENTITY_COMMON_COLOUR (0x00ffff)

#define TOOLS_SECONDARY_ENTITY_DISABLED_COMMON_COLOUR (0x00aaaa)
#define TOOLS_SECONDARY_ENTITY_DISABLED_COMMON_COLOUR (0x00aaaa)

#define TOOLS_SECONDARY_ENTITY_FOLDER_COLOUR (0xaaaaff)
#define TOOLS_SECONDARY_ENTITY_DISABLED_FOLDER_COLOUR (0x7777aa)

#define TOOLS_SECONDARY_ENTITY_HIDDEN_TREEITEM_COLOUR (0x808080)


static const int g_iFolderTreeItemID = -2;

//search types
enum
{
	#ifdef TOOLS_USE_OLD_ENTITY_MANAGER
	DIALOGUE_SEARCH_BEGINS,
	DIALOGUE_SEARCH_CONTAINS,
	#else
	DIALOGUE_SEARCH_CONTAINS,
	DIALOGUE_SEARCH_BEGINS,
	#endif
};

static const char* g_aszSearchOptions[] =
{
	#ifdef TOOLS_USE_OLD_ENTITY_MANAGER
	"Begins With",
	"Contains",
	#else
	"Contains",
	"Begins With",
	#endif
};


////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*-----------------------------------**
** Tools_Overlay_SecondaryEntityFile **
**-----------------------------------*/

class Tools_Overlay_SecondaryEntityFile : public Asura_Overlay
{
public :
	Tools_Overlay_SecondaryEntityFile(const char *szName, bool bAlwaysOnTop, Asura_Vector_2* pxPosition)
	:Asura_Overlay(szName, bAlwaysOnTop, pxPosition)
	{}

	~Tools_Overlay_SecondaryEntityFile()
	{
		Tools_SecondaryEntityFile_System::OnOverlayClosed(IsClosingByCloseGadget());
	}
};

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

class Tools_Overlay_EntityReadOnlyWarning : public Tools_BaseMessage_Overlay
{
public :
	Tools_Overlay_EntityReadOnlyWarning(const char* szText)
	:Tools_BaseMessage_Overlay(Asura_String(szText))
	{
	}

	void DoModal() {ShowOverlay();}
	
	static bool GetIgnore(){return s_bIgnore;}

protected :
	virtual void InitialiseOverlay(class Asura_Overlay* pxOverlay,Asura_Vector_2& xOffset)
	{
		Asura_Dialog* pxDialog;
	
		pxDialog = new Asura_Tickbox_Dialog("Suppress these warnings", xOffset, &s_bIgnore);
		pxDialog->SetHelpText("Gets rid of these warning messages for this session of tools. Use if you don't need to be be able to save any changes.");
		pxOverlay->AddDialog(pxDialog);
		xOffset.y += Asura_Dialog::LargeSpacing();

		pxDialog = new Asura_FunctionWithKeybinding_Dialog("Ok",xOffset,0,ASURA_KEYID_ENTER,&pxOverlay->m_bShouldClose);
		pxOverlay->AddDialog(pxDialog);
	}

private :
	static bool s_bIgnore;

};
bool Tools_Overlay_EntityReadOnlyWarning::s_bIgnore = false; 

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*----------------------------------**
** Tools_SecondaryEntityFile_System **
**----------------------------------*/

List<Tools_SecondaryEntityFile*> Tools_SecondaryEntityFile_System::s_xFileList;
Tools_SecondaryEntityFile* Tools_SecondaryEntityFile_System::s_pxCurrentFile = 0;
Asura_String Tools_SecondaryEntityFile_System::s_xCurrentFolder;
bool Tools_SecondaryEntityFile_System::s_bDoneUpdate;

bool Tools_SecondaryEntityFile_System::s_bOverlayIsOn = false;
Asura_Vector_2 Tools_SecondaryEntityFile_System::s_xOverlayPosition(20,5);
Asura_Vector_2 Tools_SecondaryEntityFile_System::s_xTreeDialogSize(0.0f,0.0f);
Asura_Vector_2 Tools_SecondaryEntityFile_System::s_xFilterListDialogSize(0.0f,0.0f);

bool Tools_SecondaryEntityFile_System::s_bHideObjectsInOtherFiles = false;
#ifndef TOOLS_USE_OLD_ENTITY_MANAGER
bool Tools_SecondaryEntityFile_System::s_bShowHiddenEntities = true;
#endif
bool Tools_SecondaryEntityFile_System::s_bHideObjectsInThisFile = false;
bool Tools_SecondaryEntityFile_System::s_bTreeValid = false;
u_int Tools_SecondaryEntityFile_System::s_uTreeItemColourValidity = 0;
Asura_Tree_Dialog* Tools_SecondaryEntityFile_System::s_pxTreeDialog = 0;

bool Tools_SecondaryEntityFile_System::s_bSelectionChanged = false;
u_int Tools_SecondaryEntityFile_System::s_uObjectSystemSelectionChangedStamp = 0;

static Asura_Dialog* g_pxSelectFileDialog = 0;

static int g_iNextToolsID = -1;
static int g_iCreateObjectRecursion = 0;

bool Tools_SecondaryEntityFile_System::s_bFiltersMinimised = true;
bool Tools_SecondaryEntityFile_System::s_bHistoryMinimised = true;

bool Tools_SecondaryEntityFile_System::s_bFilterChanged;
int Tools_SecondaryEntityFile_System::s_iSearchOption;
Asura_String Tools_SecondaryEntityFile_System::s_xSearchText;


List<ListSelectionItem*> Tools_SecondaryEntityFile_System::s_xFilterList;
char** Tools_SecondaryEntityFile_System::s_pszFilterStrings = 0;

int Tools_SecondaryEntityFile_System::s_iFilterType = 0;
bool Tools_SecondaryEntityFile_System::s_bFilterTypeChanged = false;

int Tools_SecondaryEntityFile_System::s_iFilterIndex = 0;
bool Tools_SecondaryEntityFile_System::s_bFilterIndexChanged = false;

Asura_List_Dialog* Tools_SecondaryEntityFile_System::s_pxFilterDialog;


List<int> Tools_SecondaryEntityFile_System::s_xFilterList_Type;
List<Asura_String> Tools_SecondaryEntityFile_System::s_xFilterList_Shape;
List<Asura_String> Tools_SecondaryEntityFile_System::s_xFilterList_Blueprint;
List<Asura_String> Tools_SecondaryEntityFile_System::s_xFilterList_BlueprintType;

u_int Tools_SecondaryEntityFile_System::s_uSelectedTab = 0;
#ifndef TOOLS_USE_OLD_ENTITY_MANAGER
Asura_String_Dialog* Tools_SecondaryEntityFile_System::s_pxFilterSearchStringDialog = NULL;
#endif


Asura_TreeDialog_Tree Tools_SecondaryEntityFile_System::s_xTree;
Asura_Panel_Dialog* Tools_SecondaryEntityFile_System::s_pxPanelDialog = NULL;
Asura_Panel_Dialog* Tools_SecondaryEntityFile_System::s_pxHistoryPanelDialog = NULL;
Asura_List_Dialog* Tools_SecondaryEntityFile_System::s_pxHistoryListDialog = NULL;
Asura_Overlay* Tools_SecondaryEntityFile_System::s_pxOverlay = NULL;
Tools_Object* Tools_SecondaryEntityFile_System::s_axHistory[] = {NULL};
const char* g_szEmptyHistorySlot = "*empty*";
const char* Tools_SecondaryEntityFile_System::s_axHistoryStrings[s_uNumHistorySlots] = {g_szEmptyHistorySlot};
u_int Tools_SecondaryEntityFile_System::s_uNumUsedHistorySlots = 1;
bool Tools_SecondaryEntityFile_System::s_bNeedToUpdateHistoryStrings = false;

bool Tools_SecondaryEntityFileUpdater::s_bUpdatingFromSecondaryFile = false;

static int s_iHistorySelectionIndex = 0;

void Tools_SecondaryEntityFile_System::Initialise()
{
	s_bDoneUpdate = false;
	Tools_SecondaryEntityFile* pxMain = new Tools_SecondaryEntityFile("");
	pxMain->m_bMainFile = true;
	pxMain->m_iFileID = 0;
	s_xFileList.add_entry(pxMain);

	s_xTree.EnableMultipleSelection(true);

}

void Tools_SecondaryEntityFile_System::Shutdown()
{
	while(s_xFileList.size())
	{
		delete s_xFileList.first_entry();
		s_xFileList.delete_first_entry();
	}
	s_pxCurrentFile = 0;

	DeleteFilterList();
}

void Tools_SecondaryEntityFile_System::SelectInitialEntityFile()
{
	static bool s_bFirstTime = true;
	if(!s_bFirstTime) return;
	s_bFirstTime = false;

	Asura_Assert(s_xFileList.size(), "Should be an entry for the main entity file.");

	//get a list of writeable entity files.
	List<Tools_SecondaryEntityFile*> xWriteableFileList;


	for(LIF<Tools_SecondaryEntityFile*> xFileLIF(&s_xFileList); !xFileLIF.done(); xFileLIF.next())
	{
		if(xFileLIF()->CanSaveFile(false))
		{
			xWriteableFileList.add_entry(xFileLIF());
		}
	}

	if(!xWriteableFileList.size())
	{
		//no files are writeable, so choose the main file
		Tools_Timed_Message_System::AddMessage(0xffff00,"No entity files are writeable , defaulting to the %s entity file.",Tools_SaveAsura::IsEditingMission() ? "mission" : "main");
		SetSelectedFile(s_xFileList.first_entry());
		return;
	}
	else if(xWriteableFileList.size() == 1)
	{
		//1 file is writeable ,so go with that
		Tools_SecondaryEntityFile* pxFile = xWriteableFileList.first_entry();

		const char* szName = pxFile->GetDisplayName();

		Tools_Timed_Message_System::AddMessage(0xffffff,"New entities will be created in '%s'",szName);

		SetSelectedFile(pxFile);
	}
	else
	{
		//Ask which file to create entities in
		ListSelection_Overlay xOverlay("Create entities in which file?");
		xOverlay.SetSortList(false);

		for(LIF<Tools_SecondaryEntityFile*> xFileLIF(&xWriteableFileList); !xFileLIF.done(); xFileLIF.next())
		{
			const char* szName = xFileLIF()->GetDisplayName();
			xOverlay.AddItem(szName, (int)xFileLIF());
		}

		if(xOverlay.DoModal())
		{
			SetSelectedFile((Tools_SecondaryEntityFile*) xOverlay.GetSelectionData());
		}
		else
		{
			SetSelectedFile(xWriteableFileList.first_entry());
		}
	}
}


void Tools_SecondaryEntityFile_System::Update()
{
	Asura_Assert(!g_iCreateObjectRecursion,"Looks like a call to PreCreateObject() wasn't matched with one to PostCreateObject()");
	g_iCreateObjectRecursion = 0;

	Asura_Assert(s_pxCurrentFile, "");

	//update hidden status of entity objects
	for(LIF<Tools_Object*> xObjectLIF(&Tools_Object_System::ObjectList);!xObjectLIF.done();xObjectLIF.next())
	{
		//Get the object that would be selected if this object was clicked on.
		//This should get the root template instance object in the case of objects that are part
		//of a template.
		//This is to get around the fact that the entities that make up a template instance
		//are always flagged as part of the main file at the moment.
		//RAR 17/09/2007
		Tools_Object* pxRootObject = xObjectLIF()->GetObjectToSelect();
		
		if(!pxRootObject->IsEntityObject()) continue;

		Tools_Entity_Object* pxEntityObject = (Tools_Entity_Object*) pxRootObject;

		bool bHide = false;
		
		if(s_bHideObjectsInOtherFiles)
		{
			Tools_Entity_Data* pxToolsData = pxEntityObject->Entity->ToolsData;
			if(s_pxCurrentFile->IsMainFile())
			{
				bHide = !pxToolsData->m_xSecondaryEntityFilename.IsBlank();
			}
			else
			{
				bHide = stricmp(pxToolsData->m_xSecondaryEntityFilename.GetString(),s_pxCurrentFile->m_xName.GetString())!=0;
			}
		}
		else if(s_bHideObjectsInThisFile)
		{
			Tools_Entity_Data* pxToolsData = pxEntityObject->Entity->ToolsData;
			if(s_pxCurrentFile->IsMainFile())
			{
				bHide = pxToolsData->m_xSecondaryEntityFilename.IsBlank();
			}
			else
			{
				bHide = stricmp(pxToolsData->m_xSecondaryEntityFilename.GetString(),s_pxCurrentFile->m_xName.GetString())==0;
			}
		}
		xObjectLIF()->SetHiddenFlag(ObjectHideFlag_EntityFile,bHide);
	}

	// Update tree data
	UpdateTree();

	#ifndef TOOLS_USE_OLD_ENTITY_MANAGER
	// Update dialog sizes

	if (s_pxOverlay && s_pxTreeDialog)
	{
		Asura_Vector_2 xMinSize = s_pxPanelDialog->GetMinSize ();
		xMinSize.x = s_pxTreeDialog->GetWidth () - 2;
		s_pxPanelDialog->SetMinSize (xMinSize);
		s_pxHistoryPanelDialog->SetMinSize (xMinSize);
		xMinSize.x -= 4;
		s_pxFilterDialog->SetMinWidth (xMinSize.x);
		s_pxHistoryListDialog->SetMinWidth (xMinSize.x);
	}
	#endif

	if(s_pxOverlay && s_pxOverlay->IsClosingByCloseGadget())
	{
		s_pxOverlay = NULL;
	}

	// Handle the history list
	ProcessHistoryList();
}

void Tools_SecondaryEntityFile_System::RemoveFromHistoryList(Tools_Object* pxObject)
{
	if(pxObject)
	{
		for(int n = 0; n < s_uNumHistorySlots; n++)
		{
			if(pxObject == s_axHistory[n])
			{
				s_axHistory[n] = NULL;
				delete [] s_axHistoryStrings[n];

				u_int uElementsToCopy = (s_uNumHistorySlots - 1) - n;

				if(uElementsToCopy > 0)
				{
					Asura_Mem::Move(&s_axHistory[n], &s_axHistory[n+1], sizeof(Tools_Object*) * uElementsToCopy);
					Asura_Mem::Move(&s_axHistoryStrings[n], &s_axHistoryStrings[n+1], sizeof(const char*) * uElementsToCopy);
				}

				s_uNumUsedHistorySlots--;
				s_axHistoryStrings[s_uNumUsedHistorySlots] = 0;
				s_axHistory[s_uNumUsedHistorySlots] = 0;

				if(s_pxOverlay && s_pxHistoryListDialog)
				{
					s_pxHistoryListDialog->ReplaceStrings(s_uNumUsedHistorySlots, s_axHistoryStrings);
				}

				break;
			}
		}
	}
}

void Tools_SecondaryEntityFile_System::ProcessHistoryList()
{
	if(s_bNeedToUpdateHistoryStrings)
	{
		s_bNeedToUpdateHistoryStrings = false;
		
		//An object may have been renamed ,so grab all the names again
		//RAR 09/03/2009
		for(u_int i=0; i<s_uNumUsedHistorySlots; i++)
		{
			if(s_axHistory[i])
			{
				delete [] s_axHistoryStrings[i];
				s_axHistoryStrings[i] = Asura_StringUtil::Duplicate(s_axHistory[i]->GetName());
			}
		}
		if(s_pxOverlay && s_pxHistoryListDialog)
		{
			s_pxHistoryListDialog->ReplaceStrings(s_uNumUsedHistorySlots, s_axHistoryStrings);
		}

	}

	if(Tools_Object_System::SelectedObject && (Tools_Object_System::SelectedObject != s_axHistory[0]))
	{
		// Look to see if the newly selected object is already in our history

		int iDupIndex = -1;

		for(int n = 0; n < s_uNumHistorySlots; n++)
		{
			if(s_axHistory[n] == Tools_Object_System::SelectedObject)
			{
				iDupIndex = n;
				break;
			}
		}

		const char* szMovedHistoryString = 0;
		if(iDupIndex == -1)
		{
			//New entry , so the last available entry in the history will be lost (if currently filled)
			if(g_szEmptyHistorySlot != s_axHistoryStrings[s_uNumHistorySlots - 1])
			{
				delete [] s_axHistoryStrings[s_uNumHistorySlots - 1];
				s_axHistoryStrings[s_uNumHistorySlots - 1] = 0;
			}
		}
		else
		{
			//Keep hold of the name of the object that is being moved to the front of the list
			szMovedHistoryString = s_axHistoryStrings[iDupIndex];
		}

		// Shift the elements of the array

		int n = s_uNumHistorySlots - 1;

		for(; n > 0; n--)
		{
			int i = n;

			if((iDupIndex == -1) || (iDupIndex > (n - 1)))
			{
				i--;
			}

			s_axHistory[n] = s_axHistory[i];
			s_axHistoryStrings[n] = s_axHistoryStrings[i];
		}

		s_axHistory[0] = Tools_Object_System::SelectedObject;
		if(szMovedHistoryString)
		{
			s_axHistoryStrings[0] = szMovedHistoryString;
		}
		else
		{
			s_axHistoryStrings[0] = Asura_StringUtil::Duplicate(s_axHistory[0]->GetName());
		}

		//Determine how many slots are being used now.
		s_uNumUsedHistorySlots = 0;
		for(u_int n = 0; n < s_uNumHistorySlots; n++)
		{
			if(s_axHistory[n])
			{
				s_uNumUsedHistorySlots++;
			}
		}

		// Update the strings in the overlay
		if(s_pxOverlay && s_pxHistoryListDialog)
		{
			s_pxHistoryListDialog->ReplaceStrings(s_uNumUsedHistorySlots, s_axHistoryStrings);
		}
	}
}

void Tools_SecondaryEntityFile_System::CreateEntityFile()
{
	CreateEntityFile(false);
}

void Tools_SecondaryEntityFile_System::CreateCommonEntityFile()
{
	CreateEntityFile(true);
}


void Tools_SecondaryEntityFile_System::CreateEntityFile(bool bCommon)
{
	//disallow creation of new files if the main entity file isn't checked out.
	if(!Tools_SaveAsura::CanSaveEntity())
	{
		ShowMessageOverlay("The main entity file must be checked out for a secondary file to be created.");
		return;
	}

	bool bMissionDir = !bCommon && Tools_SaveAsura::IsEditingMission();

	//ask for a name for the file
	GetString_Overlay xOverlay("Enter name for new entity file (without extension).","");
	if(!xOverlay.DoModal()) return;
	if(!strlen(xOverlay.GetString())) return;

	char acName[MAX_PATH];
	Asura_PrintfToBfr(acName,"%s",sizeof(acName), true, xOverlay.GetString());

	//make sure there isn't already an entry with this name
	for(LIF<Tools_SecondaryEntityFile*> xFileLIF(&s_xFileList);!xFileLIF.done();xFileLIF.next())
	{
		if(!stricmp(xFileLIF()->m_xName.GetString(),acName))
		{
			ShowMessageOverlay("An entity file with this name already exists.");
			return;
		}
	}

	//make sure a file doesn't already exist with that name
	Asura_String xFilename;
	GenerateFilename(acName,bMissionDir,xFilename);

	if(Tools_FileSystem::FileExists(xFilename.GetString()))
	{
		ShowMessageOverlay("%s already exists.",xFilename.GetString());
		return;
	}

	//add an entry for the new file then
	Tools_SecondaryEntityFile* pxFile = new Tools_SecondaryEntityFile(acName);
	pxFile->m_bModified = true;
	pxFile->m_bCommonFile = bCommon;
	pxFile->m_bMissionDir = bMissionDir;
	s_xFileList.add_entry(pxFile);

	//force the entity files to be saved now , to prevent the main entity file
	//from being checked in without the changes.
	Tools_SaveAsura::SaveEntities();

	//add new file to list that needs to be tracked for external modification
	Tools_LoadedFileList::TrackExternalModification(xFilename.GetString());

	if(s_bTreeValid)
	{
		Asura_TreeDialog_Item* pxNewItem = s_xTree.InsertItem(CompareTreeItem, s_xTree.GetRoot(), pxFile->GetDisplayName (), pxFile->m_iFileID);
		pxNewItem->SetColourOverride(GetItemColour(pxNewItem));

		pxNewItem->SetHash(GetItemHash(pxNewItem));
	}

}

Asura_Hash_ID Tools_SecondaryEntityFile_System::GetItemHash(Asura_TreeDialog_Item* pxItem)
{
	Asura_Char szHashString[512];

	Asura_PrintfToBfr(szHashString, "%u%d%s%s", 512, true,
		pxItem->GetDepth(),
		pxItem->GetUserData(),
		pxItem->GetName(),
		pxItem->GetHelpText());

	return Asura_GetHashID(szHashString);
}

void Tools_SecondaryEntityFile_System::AddEntityFile()
{
	//disallow creation of new files if the main entity file isn't checked out.
	if(!Tools_SaveAsura::CanSaveEntity())
	{
		ShowMessageOverlay("The main entity file must be checked out for a secondary file to be added.");
		return;
	}


	//search for entity files that are in the same directory as the main asr
	Asura_String xEnvFilename = Tools_SaveAsura::GetMainFilename();
	Asura_String xPath;
	xEnvFilename.SplitFilename(&xPath,0,0);

	char szSearch[MAX_PATH];
	Asura_PrintfToBfr(szSearch,"%s\\*.ent",sizeof(szSearch),true,xPath.IsBlank() ? "." : xPath.GetString());

	List<Asura_String> xFileNameList;
	Tools_FileSystem::FindFiles(szSearch, xFileNameList);

	//and ones in the mission dir
	if(Tools_SaveAsura::IsEditingMission())
	{
		Asura_String xMissionFilename = Tools_SaveAsura::GetMissionFilenameWithoutExtension();
		xMissionFilename.SplitFilename(&xPath,0,0);

		Asura_PrintfToBfr(szSearch,"%s\\*.ent",sizeof(szSearch),true,xPath.IsBlank() ? "." : xPath.GetString());

		Tools_FileSystem::FindFiles(szSearch, xFileNameList);
	}


	//Search the list of entity files for acceptable candidates
	ListSelection_Overlay xListOverlay("Add which entity file?");

	for(LIF<Asura_String> xFileNameLIF(&xFileNameList); !xFileNameLIF.done();xFileNameLIF.next())
	{
		bool bOkay = true;
		//get the name without path and extension
		char acBuffer[MAX_PATH];
		Asura_PrintfToBfr(acBuffer,"%s",sizeof(acBuffer),true,Tools_FileSystem::StripPath(xFileNameLIF().GetString()));
		char* szFileName = acBuffer;

		char* pcDot = strrchr(szFileName,'.');
		if(pcDot) *pcDot = 0;

		//make sure the file isn't already in use
		for(LIF<Tools_SecondaryEntityFile*> xFileLIF(&s_xFileList);!xFileLIF.done() && bOkay;xFileLIF.next())
		{
			if(!stricmp(xFileLIF()->m_xName.GetString(),szFileName))
			{
				bOkay = false;
				break;
			}
		}
		if(!bOkay) continue;

		//check that its a secondary entity file?
		Tools_AsuraFileChunkLister xChunkLister;
		if(!xChunkLister.LoadAsuraFile(xFileNameLIF().GetString())) continue;
		
		//Search for the secondary file chunks
		//It should be a secondary file if there is a chunk matching its own name
		List<const Asura_Chunk_Header*> xChunkList;
		xChunkLister.ListChunksByID(ASURA_CHUNK_ENTITY_SECONDARYFILE, xChunkList);
		int iChunkFlags = 0;

		bool bFoundChunk = false;
		for(LIF<const Asura_Chunk_Header*> xChunkLIF(&xChunkList); !xChunkLIF.done(); xChunkLIF.next())
		{
			const Asura_Chunk_Entity_SecondaryFile* pxChunk = (const Asura_Chunk_Entity_SecondaryFile*) xChunkLIF();
			const char* szSecondaryFileName = (const char*)(pxChunk+1);

			if(!stricmp(szSecondaryFileName, szFileName))
			{
				//found a matching chunk , so this should be a secondary entity file
				bFoundChunk = true;
				iChunkFlags = pxChunk->Flags;
			}
		}
		if(!bFoundChunk) continue;

		if(bOkay)
		{
			xListOverlay.AddItem(szFileName, iChunkFlags);
		}
	}

	if(!xListOverlay.GetNumItems())
	{
		ShowMessageOverlay("Failed to find any suitable secondary entity files to add");
		return;
	}

	if(!xListOverlay.DoModal()) return;

	const char* pszName = xListOverlay.GetSelectionString();
	int iChunkFlags = xListOverlay.GetSelectionData();

	//warn if adding a non-common entity file
	if(!(iChunkFlags & Asura_Chunk_Entity_SecondaryFile::ChunkFlag_CommonFile))
	{
		bool bAdd = ShowYesNoOverlay("Warning.\nThe entity file being added hasn't been flagged as a common entity file. This may cause problems if it is being used by another main entity file at the same time.\nAdd this anyway?");
		if(!bAdd) return;
		
	}



	//add an entry for the new file then
	Tools_SecondaryEntityFile* pxFile = new Tools_SecondaryEntityFile(pszName);
	pxFile->m_bCommonFile = (iChunkFlags & Asura_Chunk_Entity_SecondaryFile::ChunkFlag_CommonFile) ? true : false;
	pxFile->m_bMissionDir = (iChunkFlags & Asura_Chunk_Entity_SecondaryFile::ChunkFlag_MissionDir) ? true : false;
	s_xFileList.add_entry(pxFile);

	if(s_bTreeValid)
	{
		Asura_TreeDialog_Item* pxNewItem = s_xTree.InsertItem(CompareTreeItem, s_xTree.GetRoot(), pxFile->GetDisplayName (), pxFile->m_iFileID);
		pxNewItem->SetColourOverride(GetItemColour(pxNewItem));
	}

	//load the entities from this file
	Tools_SecondaryEntityFileUpdater xUpdater(pxFile);
	xUpdater.Update();


	//force the entity files to be saved now , to prevent the main entity file
	//from being checked in without the changes.
	Tools_SaveAsura::SaveEntities();

	//add new file to list that needs to be tracked for external modification
	Asura_String xFilename;
	GenerateFilename(pxFile,xFilename);
	Tools_LoadedFileList::TrackExternalModification(xFilename.GetString());

}


void Tools_SecondaryEntityFile_System::MergeEntityFile()
{
	if(!s_pxCurrentFile || s_pxCurrentFile->IsMainFile())
	{
		ShowMessageOverlay("Need to select a secondary entity file first.");
		return;
	}

	//the main entity file needs to be writeable
	if(!Tools_SaveAsura::CanSaveEntity())
	{
		ShowMessageOverlay("The main entity file must be checked out for a secondary file to be merged back in.");
		return;
	}

	//as does the file being removed
	Asura_String xFilename;
	GenerateFilename(s_pxCurrentFile,xFilename);

	if(!Tools_SaveAsura::CanSaveFile(xFilename.GetString(),false))
	{
		ShowMessageOverlay("The secondary entity file must be checked out to merge the entites back in.");
		return;
	}
	
	//confirm that this does want to be done.
	if(!ShowYesNoOverlay("Would you like to merge the entities in %s back into the main entity file?",s_pxCurrentFile->m_xName.GetString()))
	{
		return;
	}

	//find all the entities using this file , and adjust their tools data
	for(Asura_Classified_List_It<Asura_Entity_Server> it(Asura_Entity_Server::GetList());!it.Done();it.Next())
	{
		Asura_Entity_Server* pxEntity = it.GetCurrent();
		//ignore entities marked for destruction
		if(pxEntity->IsDestroyed()) continue;

		if(!pxEntity->ToolsData) continue;

		if(!stricmp(pxEntity->ToolsData->m_xSecondaryEntityFilename.GetString(),s_pxCurrentFile->m_xName.GetString()))
		{
			//This entity belongs to the file being merged
			pxEntity->ToolsData->m_xSecondaryEntityFilename = "";
			pxEntity->ToolsData->m_iSecondaryEntityID = 0;

			if(pxEntity->ToolsData->ToolsObject)
			{
				//move the entity in the tree
				UpdateEntityInTree((Tools_Entity_Object*) pxEntity->ToolsData->ToolsObject, false);
			}
		}
	}

	//remove the file from the tree
	Asura_TreeDialog_Item* pxFileItem = GetFileOrFolderFromTree(s_pxCurrentFile, 0 , false);
	if(pxFileItem)
	{
		s_xTree.DeleteItem(pxFileItem);
	}

	//delete the file , and remove the entry for it
	DeleteFile(xFilename.GetString());
	s_xFileList.delete_entry(s_pxCurrentFile);
	delete s_pxCurrentFile;
	s_pxCurrentFile = GetMainFile();

	Tools_SaveAsura::SaveEntities();

	UpdateOverlay();
}

void Tools_SecondaryEntityFile_System::DeleteEntityFile()
{
	if(!s_pxCurrentFile || s_pxCurrentFile->IsMainFile())
	{
		ShowMessageOverlay("Need to select a secondary entity file first.");
		return;
	}
	RemoveEntityFile(s_pxCurrentFile, true);
}

void Tools_SecondaryEntityFile_System::RemoveEntityFile()
{
	if(!s_pxCurrentFile || s_pxCurrentFile->IsMainFile())
	{
		ShowMessageOverlay("Need to select a secondary entity file first.");
		return;
	}
	RemoveEntityFile(s_pxCurrentFile, false);
}


void Tools_SecondaryEntityFile_System::RemoveEntityFile(Tools_SecondaryEntityFile* pxFile, bool bDelete)
{
	if(!pxFile || pxFile->IsMainFile())
	{
		ShowMessageOverlay("The main entity file can't be deleted in this way.");
		return;
	}

	//confirm that this does want to be done.
	if(!ShowYesNoOverlay("Would you like to %s %s and its entities?",bDelete ? "delete" : "remove" ,pxFile->m_xName.GetString()))
	{
		return;
	}


	//warn if the main entity file isn't writeable
	if(!Tools_SaveAsura::CanSaveEntity())
	{
		ShowMessageOverlay("The main entity isn't checked out , so it isn't possible to remove the reference to the secondary file.");
	}


	//find all the entities using this file , and delete them 
	List<Tools_Object*> xObjectsToDestroy;

	for(LIF<Tools_Object*> xObjectLIF(&Tools_Object_System::ObjectList);!xObjectLIF.done();xObjectLIF.next())
	{
		if(!xObjectLIF()->IsEntityObject()) continue;
		Tools_Entity_Object* pxEntityObject = (Tools_Entity_Object*) xObjectLIF();
		Asura_Entity* pxEntity = pxEntityObject->Entity;

		if(!pxEntity->ToolsData) continue;
		
		if(pxEntity->ToolsData->m_xSecondaryEntityFilename.IsBlank()) continue;

		//does this entity belong with this file?
		if(!stricmp(pxFile->m_xName.GetString(),pxEntity->ToolsData->m_xSecondaryEntityFilename.GetString()))
		{
			//add to the list of objects to be destroyed
			xObjectsToDestroy.add_entry(pxEntityObject);
		}
	}

	//destroy the entities
	for(LIF<Tools_Object*> xDestroyLIF(&xObjectsToDestroy);!xDestroyLIF.done();xDestroyLIF.next())
	{
		if(Tools_Object_System::ObjectList.contains(xDestroyLIF()))
		{
			xDestroyLIF()->Destroy();
		}
	}
	Asura_Entity_System::PruneEntityLists();

	//remove the file from the tree
	Asura_TreeDialog_Item* pxFileItem = GetFileOrFolderFromTree(pxFile, 0 , false);
	if(pxFileItem)
	{
		s_xTree.DeleteItem(pxFileItem);
	}

	//delete the file , and remove the entry for it
	if(bDelete)
	{
		Asura_String xFilename;
		GenerateFilename(pxFile,xFilename);
		DeleteFile(xFilename.GetString());
	}
	s_xFileList.delete_entry(pxFile);
	delete pxFile;
	if(s_pxCurrentFile == pxFile)
	{
		s_pxCurrentFile = GetMainFile();
	}
		
	Tools_SaveAsura::SetEntityModified();


	UpdateOverlay();
}

void Tools_SecondaryEntityFile_System::RenameEntityFile()
{
	if(!s_pxCurrentFile || s_pxCurrentFile->IsMainFile())
	{
		ShowMessageOverlay("Need to select a secondary entity file first.");
		return;
	}

	//the file being renamed needs to be writeable
	if(!CanSaveFile(s_pxCurrentFile))
	{
		if(!ShowYesNoOverlay("%s doesn't appear to be checked out.\nAttempt to rename it anyway?",s_pxCurrentFile->m_xName.GetString()))
		{
			return;
		}
	}

	//warn if the main entity file isn't writeable
	if(!Tools_SaveAsura::CanSaveEntity())
	{
		ShowMessageOverlay("The main entity isn't checked out , so it won't be possible to update it to refer to the renamed file.");
	}

	//ask for a new name
	GetString_Overlay xOverlay("Enter new name for entity file (without extension).",s_pxCurrentFile->m_xName.GetString());
	if(!xOverlay.DoModal()) return;
	if(!strlen(xOverlay.GetString())) return;

	Asura_String xNewName = xOverlay.GetString();

	RenameEntityFile(s_pxCurrentFile, xNewName.GetString(), false);

}

void Tools_SecondaryEntityFile_System::RenameEntityFile(Tools_SecondaryEntityFile* pxFile, const char* szNewName, bool bShowInitialMessages)
{
	Asura_Assert(pxFile, "");
	Asura_Assert(!pxFile->IsMainFile(), "");

	if(bShowInitialMessages)
	{
		//the file being renamed needs to be writeable
		if(!CanSaveFile(pxFile))
		{
			if(!ShowYesNoOverlay("%s doesn't appear to be checked out.\nAttempt to rename it anyway?",pxFile->m_xName.GetString()))
			{
				return;
			}
		}

		//warn if the main entity file isn't writeable
		if(!Tools_SaveAsura::CanSaveEntity())
		{
			ShowMessageOverlay("The main entity isn't checked out , so it won't be possible to update it to refer to the renamed file.");
		}
	}

	//make sure there isn't already an entry with this name
	for(LIF<Tools_SecondaryEntityFile*> xFileLIF(&s_xFileList);!xFileLIF.done();xFileLIF.next())
	{
		if(!stricmp(xFileLIF()->m_xName.GetString(),szNewName))
		{
			Tools_Timed_Message_System::AddMessage(0xff0000,"An entity file with this name already exists.");
			return;
		}
	}

	//make sure a file doesn't already exist with that name
	Asura_String xFilename;
	GenerateFilename(szNewName,pxFile->m_bMissionDir,xFilename);


	if(Tools_FileSystem::FileExists_CheckNetwork(xFilename.GetString()))
	{
		Tools_Timed_Message_System::AddMessage(0xff0000,"%s already exists.",xFilename.GetString());

		return;
	}

	Asura_TreeDialog_Item* pxFileItem = GetFileOrFolderFromTree(pxFile, NULL, false);
	Asura_Assert(pxFileItem, "Failed to find entry for file in tree");



	Asura_String xOldName = pxFile->m_xName;

	//update all the entities that refer to the old filename
	for(Asura_Classified_List_It<Asura_Entity_Server> xIt(Asura_Entity_Server::GetList());!xIt.Done();xIt.Next())
	{
		Asura_Entity_Server* pxEntity = xIt.GetCurrent();

		if(!pxEntity->ToolsData) continue;
		if(pxEntity->ToolsData->m_xSecondaryEntityFilename.IsBlank()) continue;

		if(!pxEntity->ToolsData->m_xSecondaryEntityFilename.Compare(xOldName))
		{
			pxEntity->ToolsData->m_xSecondaryEntityFilename = szNewName;
		}
	}

	//update the entry for this file
	pxFile->m_xName = szNewName;

	//add the new file to list that needs to be tracked for external modification
	Tools_LoadedFileList::TrackExternalModification(xFilename.GetString());

	Tools_SaveAsura::SetEntityModified();

	//force a save now , since the renamed file will be somewhat broken otherwise
	s_pxCurrentFile->m_bModified = true;
	Tools_SaveAsura::SaveEntities();

	//Make sure the new file saved okay
	if(Tools_FileSystem::FileExists(xFilename.GetString()))
	{													  
		//can delete the old file then
		Asura_String xOldFilename;
		GenerateFilename(xOldName.GetString(),s_pxCurrentFile->m_bMissionDir,xOldFilename);
		DeleteFile(xOldFilename.GetString());
	}
	else
	{
		//bit of a problem if this happens
		ShowMessageOverlay("Failed to save the entity file under the new name.");
	}

	//update the tree
	if(pxFileItem)
	{
		pxFileItem->SetName(szNewName);
		s_xTree.MoveItem(CompareTreeItem, pxFileItem, 0 );
	}
}


void Tools_SecondaryEntityFile_System::ReplaceEntityFile()
{
	if(!s_pxCurrentFile || s_pxCurrentFile->IsMainFile())
	{
		ShowMessageOverlay("Need to select a secondary entity file first.");
		return;
	}

	//the file being renamed needs to be writeable
	if(!CanSaveFile(s_pxCurrentFile))
	{
		if(!ShowYesNoOverlay("%s doesn't appear to be checked out.\nAttempt to replace it anyway?",s_pxCurrentFile->m_xName.GetString()))
		{
			return;
		}
	}

	//warn if the main entity file isn't writeable
	if(!Tools_SaveAsura::CanSaveEntity())
	{
		ShowMessageOverlay("The main entity isn't checked out , so it won't be possible to update it to refer to the replaced file.");
	}

	Asura_String xEnvFilename = Tools_SaveAsura::GetMainFilename();
	Asura_String xPath;
	xEnvFilename.SplitFilename(&xPath,0,0);

	char szSearch[MAX_PATH];
	Asura_PrintfToBfr(szSearch,"%s\\*.ent",sizeof(szSearch),true,xPath.IsBlank() ? "." : xPath.GetString());

	List<Asura_String> xFileNameList;
	Tools_FileSystem::FindFiles(szSearch, xFileNameList);

	//and ones in the mission dir
	if(Tools_SaveAsura::IsEditingMission())
	{
		Asura_String xMissionFilename = Tools_SaveAsura::GetMissionFilenameWithoutExtension();
		xMissionFilename.SplitFilename(&xPath,0,0);

		Asura_PrintfToBfr(szSearch,"%s\\*.ent",sizeof(szSearch),true,xPath.IsBlank() ? "." : xPath.GetString());

		Tools_FileSystem::FindFiles(szSearch, xFileNameList);
	}



	//Search the list of entity files for acceptable candidates
	char acText[200];
	Asura_PrintfToBfr(acText,"Replace '%s' with which entity file?",sizeof(acText),true,s_pxCurrentFile->m_xName.GetString());
	ListSelection_Overlay xListOverlay(acText);

	for(LIF<Asura_String> xFileNameLIF(&xFileNameList); !xFileNameLIF.done();xFileNameLIF.next())
	{
		bool bOkay = true;
		//get the name without path and extension
		char acBuffer[MAX_PATH];
		Asura_PrintfToBfr(acBuffer,"%s",sizeof(acBuffer),true,Tools_FileSystem::StripPath(xFileNameLIF().GetString()));
		char* szFileName = acBuffer;

		char* pcDot = strrchr(szFileName,'.');
		if(pcDot) *pcDot = 0;

		//make sure the file isn't already in use
		for(LIF<Tools_SecondaryEntityFile*> xFileLIF(&s_xFileList);!xFileLIF.done() && bOkay;xFileLIF.next())
		{
			if(!stricmp(xFileLIF()->m_xName.GetString(),szFileName))
			{
				bOkay = false;
				break;
			}
		}
		if(!bOkay) continue;

		//check that its a secondary entity file?
		Tools_AsuraFileChunkLister xChunkLister;
		if(!xChunkLister.LoadAsuraFile(xFileNameLIF().GetString())) continue;
		
		//Search for the secondary file chunks
		//It should be a secondary file if there is a chunk matching its own name
		List<const Asura_Chunk_Header*> xChunkList;
		xChunkLister.ListChunksByID(ASURA_CHUNK_ENTITY_SECONDARYFILE, xChunkList);
		int iChunkFlags = 0;

		bool bFoundChunk = false;
		for(LIF<const Asura_Chunk_Header*> xChunkLIF(&xChunkList); !xChunkLIF.done(); xChunkLIF.next())
		{
			const Asura_Chunk_Entity_SecondaryFile* pxChunk = (const Asura_Chunk_Entity_SecondaryFile*) xChunkLIF();
			const char* szSecondaryFileName = (const char*)(pxChunk+1);

			if(!stricmp(szSecondaryFileName, szFileName))
			{
				//found a matching chunk , so this should be a secondary entity file
				bFoundChunk = true;
				iChunkFlags = pxChunk->Flags;
			}
		}
		if(!bFoundChunk) continue;

		if(bOkay)
		{
			xListOverlay.AddItem(szFileName, iChunkFlags);
		}
	}

	if(!xListOverlay.GetNumItems())
	{
		ShowMessageOverlay("Failed to find any suitable secondary entity files to add");
		return;
	}

	if(!xListOverlay.DoModal()) return;

	Asura_String xNewName = xListOverlay.GetSelectionString();
	Asura_String xOldName = s_pxCurrentFile->m_xName;
	int iChunkFlags = xListOverlay.GetSelectionData();

	
	//update all the entities that refer to the old filename
	for(Asura_Classified_List_It<Asura_Entity_Server> xIt(Asura_Entity_Server::GetList());!xIt.Done();xIt.Next())
	{
		Asura_Entity_Server* pxEntity = xIt.GetCurrent();

		if(!pxEntity->ToolsData) continue;
		if(pxEntity->ToolsData->m_xSecondaryEntityFilename.IsBlank()) continue;

		if(!pxEntity->ToolsData->m_xSecondaryEntityFilename.Compare(xOldName))
		{
			pxEntity->ToolsData->m_xSecondaryEntityFilename = xNewName;
		}
	}

	//update the entry for this file
	s_pxCurrentFile->m_xName = xNewName;
	s_pxCurrentFile->m_bCommonFile = (iChunkFlags & Asura_Chunk_Entity_SecondaryFile::ChunkFlag_CommonFile) ? true : false;
	s_pxCurrentFile->m_bMissionDir = (iChunkFlags & Asura_Chunk_Entity_SecondaryFile::ChunkFlag_MissionDir) ? true : false;
	

	//add the new file to list that needs to be tracked for external modification
	Asura_String xFilename;
	GenerateFilename(s_pxCurrentFile,xFilename);
	Tools_LoadedFileList::TrackExternalModification(xFilename.GetString());

	Tools_SaveAsura::SetEntityModified();
	
	//force an update from the existing file
	Tools_SecondaryEntityFileUpdater xUpdater(s_pxCurrentFile);
	xUpdater.Update(true);


	InvalidateTree();
}



void Tools_SecondaryEntityFile_System::TransferSelectedObjects()
{
	if(!Tools_Object_System::SelectedObjectList.size()) return;

	Asura_Assert(s_pxCurrentFile, "");

	if(!CanSaveCurrentFile())
	{
		ShowMessageOverlay("Can't transfer entities , because the current file isn't checked out.");
		return;
	}

	if(!ShowYesNoOverlay("Would you like to transfer the %d selected object(s) to %s.",Tools_Object_System::SelectedObjectList.size(),s_pxCurrentFile ? s_pxCurrentFile->GetDisplayName() : "the main entity file"))
	{
		return;
	}


	for(LIF<Tools_Object*> xObjectLIF(&Tools_Object_System::SelectedObjectList);!xObjectLIF.done();xObjectLIF.next())
	{
		//only interested in entities.
		if(!xObjectLIF()->IsEntityObject()) continue;
		Tools_Entity_Object* pxEntityObject = (Tools_Entity_Object*) xObjectLIF();

		//get the tools data
		Tools_Entity_Data* pxToolsData = pxEntityObject->Entity->ToolsData;

		//is it in the correct file already?
		if(s_pxCurrentFile->IsMainFile())
		{
			if(pxToolsData->m_xSecondaryEntityFilename.IsBlank())
			{
				continue;
			}
		}
		else
		{
			if(!stricmp(s_pxCurrentFile->m_xName.GetString(),pxToolsData->m_xSecondaryEntityFilename.GetString()))
			{
				continue;
			}
		}

		//the entity file that the object is coming from must be writeable
		bool bCanSave = CanSaveFile(pxToolsData->m_xSecondaryEntityFilename.GetString());
		

		if(!bCanSave)
		{
			ShowMessageOverlay("Can't transfer %s because its entity file isn't checked out.",pxEntityObject->GetName());
			continue;
		}

		if(!pxEntityObject->Entity->CanBeIncludedInTemplate())
		{
			ShowMessageOverlay("%s can't be included in a secondary entity file.\nOnly entities that are supported by template files may be used.",pxEntityObject->GetName());
			continue;
		}

		//set the source as modified
		SetFileModified(pxToolsData->m_xSecondaryEntityFilename.GetString());

		//transfer the object
		if(s_pxCurrentFile->IsMainFile())
		{
			pxToolsData->m_xSecondaryEntityFilename = "";
			pxToolsData->m_iSecondaryEntityID = 0;
		}
		else
		{
			pxToolsData->m_xSecondaryEntityFilename = s_pxCurrentFile->m_xName;
			pxToolsData->m_iSecondaryEntityID = s_pxCurrentFile->m_iNextID++;
		}

		//set the destination as modified
		SetFileModified(pxToolsData->m_xSecondaryEntityFilename.GetString());

		UpdateEntityInTree(pxEntityObject, false);

	}
}

void Tools_SecondaryEntityFile_System::AddToCurrentFile(Tools_Object* pxObject)
{

	if(s_pxCurrentFile)
	{
		AddToFile(pxObject, s_pxCurrentFile->m_xName.GetString(), s_xCurrentFolder.GetString());
	}
	else
	{
		AddToFile(pxObject, "", 0);
	}
}

void Tools_SecondaryEntityFile_System::AddToFile(Tools_Object* pxObject, const char* szEntityFile, const char* szFolder)
{
	if(!pxObject->IsEntityObject()) return;


	Tools_SecondaryEntityFile* pxFile = GetFile(szEntityFile);
	if(!pxFile) return;

	Tools_Entity_Object* pxEntityObject = (Tools_Entity_Object*) pxObject;
	Asura_Entity* pxEntity = pxEntityObject->Entity;
	if(!pxEntity->ToolsData) return;

	if(!Tools_SaveAsura::IsTemplateEditing())
	{
		if(!CanSaveFile(pxFile))
		{
			//The chosen file can't be saved , so search for a safe one instead
			pxFile = 0;
			for(LIF<Tools_SecondaryEntityFile*> xFileLIF(&s_xFileList); !xFileLIF.done(); xFileLIF.next())
			{
				if(CanSaveFile(xFileLIF()))
				{
					pxFile = xFileLIF();
					break;
				}
			}
			if(!pxFile)
			{
				//no file is writeable , so resort to the main file
				pxFile = GetFile("");
				Asura_Assert(pxFile, "Failed to get entry for main entity file");
				if(!pxFile) return;
			}
		}


		if(!pxFile->IsMainFile())
		{
	
			//only allow objects that work in template
			if(!pxEntity->CanBeIncludedInTemplate())
			{
			  	ShowMessageOverlay("%s can't be included in a secondary entity file.\nOnly entities that are supported by template files may be used.",pxObject->GetName());
				//pick the main file instead
				pxFile = GetFile("");
				Asura_Assert(pxFile, "Failed to get entry for main entity file");
				if(!pxFile) return;
			}
			else
			{
				//set the file in the tools data
				pxEntity->ToolsData->m_xSecondaryEntityFilename = pxFile->m_xName;
				pxEntity->ToolsData->m_iSecondaryEntityID = pxFile->m_iNextID++;
			}
		}

		//complain if the chosen file is read-only
		if(!CanSaveFile(pxFile))
		{
			WarnAboutUnwriteableEntity(pxObject, pxFile->GetDisplayName());
		}
	}
	
	if(szFolder && *szFolder)
	{
		pxEntity->ToolsData->m_xEntityBrowserPath = szFolder;
	}
	else
	{
		pxEntity->ToolsData->m_xEntityBrowserPath.MakeBlank();
	}
		
	SetFileModified(pxFile->m_xName.GetString());

	UpdateEntityInTree(pxEntityObject, false);}

bool Tools_SecondaryEntityFile_System::TransferEntity(Asura_TreeDialog_Item* pxEntityItem, Tools_SecondaryEntityFile* pxDestFile, const char* szFolder)
{
	//find the entity being moved
	Asura_Guid uGuid = GetGuidFromTreeItem(pxEntityItem);
	if(uGuid == ASURA_GUID_UNREGISTERED) return false;

	Asura_Entity* pxEntity = Asura_Entity_Server::FindEntity(uGuid);
	if(!pxEntity) return false;
	if(!pxEntity->ToolsData) return false;

	//make sure the destination is writeable
	if(!pxDestFile) return false;
	if(!pxDestFile->CanSaveFile(false))
	{
		Tools_Timed_Message_System::AddMessage(0xff0000,"The target file (%s) isn't writeable",pxDestFile->GetDisplayName());
		return false;
	}

	//make sure the source id writeable
	Tools_SecondaryEntityFile* pxSourceFile = GetFile(pxEntity->ToolsData->m_xSecondaryEntityFilename.GetString());
	if(!pxSourceFile) return false;
	if(!pxSourceFile->CanSaveFile(false))
	{
		Tools_Timed_Message_System::AddMessage(0xff0000,"The source file (%s) isn't writeable",pxSourceFile->GetDisplayName());
		return false;
	}

	//If moving to a secondary file , then make sure it is supported for this entity type
	if(!pxDestFile->IsMainFile() && !pxEntity->CanBeIncludedInTemplate())
	{
		Tools_Timed_Message_System::AddMessage(0xff0000,"'%s' can't be used in secondary entity files",pxEntityItem->GetName());
		return false;
	}

	//update the current file
	if(pxDestFile != pxSourceFile)
	{
		if(pxDestFile->IsMainFile())
		{
			pxEntity->ToolsData->m_xSecondaryEntityFilename.MakeNull();
			pxEntity->ToolsData->m_iSecondaryEntityID = 0;
		}
		else
		{
			pxEntity->ToolsData->m_xSecondaryEntityFilename = pxDestFile->m_xName;
			pxEntity->ToolsData->m_iSecondaryEntityID = pxDestFile->m_iNextID++;
		}
	}
	//update the path
	if(szFolder && *szFolder)
	{
		pxEntity->ToolsData->m_xEntityBrowserPath = szFolder;
	}
	else
	{
		pxEntity->ToolsData->m_xEntityBrowserPath.MakeNull();
	}

	SetFileModified(pxSourceFile->m_xName.GetString());
	SetFileModified(pxDestFile->m_xName.GetString());

	return true;
}


void Tools_SecondaryEntityFile_System::AddFileFromLoad(const char* pszName,int iFileVersion,int iNextID, int iChunkFlags)
{
	//make sure there isn't already an entry with this name
	for(LIF<Tools_SecondaryEntityFile*> xFileLIF(&s_xFileList);!xFileLIF.done();xFileLIF.next())
	{
		if(!stricmp(xFileLIF()->m_xName.GetString(),pszName))
		{
			Asura_Assert(0,"Details for this secondary entity file already exist.");
			return;
		}
	}

	Tools_SecondaryEntityFile* pxFile = new Tools_SecondaryEntityFile(pszName);
	pxFile->m_iFileVersion = iFileVersion;
	pxFile->m_iFileVersionToSave = iFileVersion;
	pxFile->m_iNextID = iNextID;
	pxFile->m_bCommonFile = (iChunkFlags & Asura_Chunk_Entity_SecondaryFile::ChunkFlag_CommonFile) ? true : false;
	pxFile->m_bMissionDir = (iChunkFlags & Asura_Chunk_Entity_SecondaryFile::ChunkFlag_MissionDir) ? true : false;
	s_xFileList.add_entry(pxFile);
}


void Tools_SecondaryEntityFile_System::GenerateFilename(const Tools_SecondaryEntityFile* pxFile,Asura_String& xOutputFilename)
{
	GenerateFilename(pxFile->m_xName.GetString(), pxFile->m_bMissionDir, xOutputFilename);
}

void Tools_SecondaryEntityFile_System::GenerateFilename(const char* pszName,bool bMissionDir,Asura_String& xOutputFilename)
{
	//get the main level filename , so we can get the path
	Asura_String xFilename;
	if(bMissionDir && Tools_SaveAsura::IsEditingMission())
	{
		xFilename = Tools_SaveAsura::GetMissionFilenameWithoutExtension();
	}
	else
	{
		Asura_Assert(!bMissionDir,"Requesting filename for mission specific entity file when not editing a mission file.");
		xFilename = Tools_SaveAsura::GetMainFilename();
	}
	Asura_String xPath;
	xFilename.SplitFilename(&xPath,0,0);

	//build the new filename from this
	char szEntityFilename[MAX_PATH];
	Asura_PrintfToBfr(szEntityFilename,"%s\\%s.ent",MAX_PATH,true,xPath.GetString(),pszName);

	xOutputFilename = szEntityFilename;

}

void Tools_SecondaryEntityFile_System::SaveFiles()
{
	if(!s_xFileList.size()) return;
																			 
	//bump up the version numbers of any that can be saved
	//(All get saved regardless of modified flag , because the general entity file modified flag
	//may be set instead of for the individual files)
	bool bAnyToSave = false;
	for(LIF<Tools_SecondaryEntityFile*> xFileLIF(&s_xFileList);!xFileLIF.done();xFileLIF.next())
	{
		if(xFileLIF()->IsMainFile()) continue;
		
		Asura_String xFilename;
		GenerateFilename(xFileLIF(),xFilename);
		
		if(Tools_SaveAsura::CanSaveFile(xFilename.GetString(),false))
		{
			xFileLIF()->m_iFileVersion++;
			xFileLIF()->m_iFileVersionToSave = xFileLIF()->m_iFileVersion;
			bAnyToSave = true;
		}
		else if(xFileLIF()->m_bModified)
		{
			//Entities in secondary file have been modified without the file being checked out.
			//Therefore save out as version 0 in the main entity file to ensure that they get
			//updated again when next loaded.
			xFileLIF()->m_iFileVersionToSave = 0;
			Tools_Timed_Message_System::AddMessage(0xff0000,"Can't save %s.",Tools_FileSystem::StripPath(xFilename.GetString()));
		}
		else
		{
			xFileLIF()->m_iFileVersionToSave = xFileLIF()->m_iFileVersion;
		}
	}

	if(Tools_Overlay_EntityReadOnlyWarning::GetIgnore())
	{
		if(bAnyToSave || Tools_SaveAsura::CanSaveEntity())
		{
			ShowMessageOverlay("Warning.\nYou have previously ignored messages about entities being created in unwriteable files , and are now saving changes to entity files.\nThis may result in loss of data.");
		}
	}

	if(!bAnyToSave) return;

	//Going to save a complete copy of the entity file to each of the secondary files
	//(the appropriate bits will be extracted upon load).
	//Therefore we may as well generate the save block once.

	Asura_SaveBlock xSaveBlock;
	Tools_SaveAsura::Static_AddChunksToEntityFile(&xSaveBlock);
	xSaveBlock.AddAsuraTerminator();

	//save all the copies of the file
	for(LIF<Tools_SecondaryEntityFile*> xFileLIF(&s_xFileList);!xFileLIF.done();xFileLIF.next())
	{
		if(xFileLIF()->IsMainFile()) continue;
		
		Asura_String xFilename;
		GenerateFilename(xFileLIF(),xFilename);

		if(Tools_SaveAsura::CanSaveFile(xFilename.GetString(),false))
		{
			if(xSaveBlock.SaveFile(xFilename.GetString(),true))
			{
				Tools_Timed_Message_System::AddMessage(0x00ff00,"Saved %s successfully.",Tools_FileSystem::StripPath(xFilename.GetString()));
				Tools_LoadedFileList::UpdateModifiedTime(xFilename.GetString());
				xFileLIF()->m_bModified = false;
			}
			else if(xFileLIF()->m_bModified)
			{
				Tools_Timed_Message_System::AddMessage(0xff0000,"Failed to save %s.",Tools_FileSystem::StripPath(xFilename.GetString()));
			}
		}
	}
}

bool Tools_SecondaryEntityFile_System::CanSaveFile(const Tools_SecondaryEntityFile* pxFile)
{
	Asura_Assert(pxFile, "");
	return pxFile ? pxFile->CanSaveFile(false) : false;
}

bool Tools_SecondaryEntityFile_System::CanSaveFileCached(const Tools_SecondaryEntityFile* pxFile)
{
	Asura_Assert(pxFile, "");
	return pxFile ? pxFile->CanSaveFileCached() : false;
}


bool Tools_SecondaryEntityFile_System::CanSaveFile(const char* szFile)
{
	Tools_SecondaryEntityFile* pxFile = 0;

	pxFile = GetFile(szFile);
	Asura_Assert(pxFile, "Failed to find entry for secondary entity file");
	

	return CanSaveFile(pxFile);	
}

void Tools_SecondaryEntityFile_System::MakeBackups()
{
	for(LIF<Tools_SecondaryEntityFile*> xFileLIF(&s_xFileList);!xFileLIF.done();xFileLIF.next())
	{
		if(xFileLIF()->IsMainFile()) continue;

		Asura_String xFilename;
		GenerateFilename(xFileLIF(),xFilename);

		Tools_SaveAsura::BackupFile(xFilename.GetString());
	}
}

void Tools_SecondaryEntityFile_System::WarnIfCurrentFileIsReadOnly()
{
	//limit this to once per frame , in case multiple objects are done at once
	static u_int s_uFrameStamp = 0;
	if(s_uFrameStamp == Asura_Timers::GetFrameNumber()) return;
	s_uFrameStamp = Asura_Timers::GetFrameNumber();

	if(s_pxCurrentFile)
	{
		s_pxCurrentFile->WarnIfReadOnly();
	}

}

void Tools_SecondaryEntityFile_System::AddChunksToFile(class Asura_SaveBlock* pxSaveBlock)
{
	for(LIF<Tools_SecondaryEntityFile*> xFileLIF(&s_xFileList);!xFileLIF.done();xFileLIF.next())
	{
		if(xFileLIF()->IsMainFile()) continue;
		Asura_Chunk_Entity_SecondaryFile::Create(xFileLIF(),*pxSaveBlock);
	}
}


void Tools_SecondaryEntityFile_System::UpdateFromSecondaryFiles()
{
	if(s_bDoneUpdate) return;
	s_bDoneUpdate = true;
	if(s_xFileList.size() <= 1) return;

	//beore doing the update , mark all existing entities to indicate that they were loaded from
	//the main entity file , and so their guid is fixed.
	for(LIF<Tools_Object*> xObjectLIF(&Tools_Object_System::ObjectList);!xObjectLIF.done();xObjectLIF.next())
	{
		if(!xObjectLIF()->IsEntityObject()) continue;
		Tools_Entity_Object* pxEntityObject = (Tools_Entity_Object*) xObjectLIF();
		Asura_Entity* pxEntity = pxEntityObject->Entity;

		if(!pxEntity->ToolsData) continue;
		pxEntity->ToolsData->m_bSecondaryEntityGuidConstant = true;

	}


	for(LIF<Tools_SecondaryEntityFile*> xFileLIF(&s_xFileList);!xFileLIF.done();xFileLIF.next())
	{
		if(xFileLIF()->IsMainFile()) continue;
		Tools_SecondaryEntityFileUpdater xUpdater(xFileLIF());
		xUpdater.Update();
	}
}

void Tools_SecondaryEntityFile_System::SetFileModified(const char* pszName)
{
	if(pszName && *pszName)
	{
		//Don't flag the secondary files as modified while updating from them
		//(Prevents the destruction of the old objects causing the files from
		//being flagged as modified incorrectly)
		//RAR 29/08/2007
		if(!Tools_SecondaryEntityFileUpdater::IsUpdatingFromSecondaryFile())
		{
			Tools_SecondaryEntityFile* pxFile = GetFile(pszName);
			if(pxFile && !pxFile->m_bModified)
			{
				pxFile->WarnIfReadOnly();
				pxFile->m_bModified = true;
			}
			
		}
	}
	//set the main entity file as modified as well.
	//(required at the moment at any rate for any saving to be attempted)
	Tools_SaveAsura::SetEntityModified();
}

bool Tools_SecondaryEntityFile_System::IsAllowableTarget(const char* pszEntityFile,bool bSourceIsCutsceneFile,Tools_Object* pxTargetObject,bool bShowMessage /*= true*/)
{
	//having no target is fine
	if(!pxTargetObject) return true;

	//if not editing things in a secondary file  , then its okay
	if(!bSourceIsCutsceneFile && (!pszEntityFile || !*pszEntityFile)) return true;

	//find out where the target lies
	const char* pszTargetFile = 0;
	Tools_Entity_Object* pxEntityObject = 0;
	if(pxTargetObject->IsEntityObject())
	{
		pxEntityObject = (Tools_Entity_Object*) pxTargetObject;
		pszTargetFile = pxEntityObject->Entity->ToolsData->m_xSecondaryEntityFilename.GetString();
	}


	Tools_SecondaryEntityFile* pxSourceFile = GetFile(pszEntityFile);
	if(pszEntityFile && pxSourceFile && pxSourceFile->m_bCommonFile)
	{
		if(pxTargetObject->Type == TOT_Phonon || pxTargetObject->Type == TOT_StreamingBackgroundSound)
		{
			//phonons and streaming sounds are valid targets for triggers in the common file , because
			//their guids will remain fixed.
			//could do with a better way of dealing with these...
			Asura_Guid uGuid = pxTargetObject->GetGuid();
			if(uGuid == ASURA_GUID_UNREGISTERED || (uGuid >=ASURA_GUID_TOOLS_SOUNDFILE_START && uGuid<=ASURA_GUID_TOOLS_SOUNDFILE_END))
			{
				return true;
			}
		}
		//If the source file is a common file , then it is only allowable for it to target other entities in the same file
		if(!pszTargetFile || stricmp(pszTargetFile,pszEntityFile))
		{
			if(bShowMessage)
			{
				ShowMessageOverlay("Can't target %s because the source entity is in a common entity file and the target belongs to a different file.",pxTargetObject->GetName());
			}
			return false;
		}
	}

	//If the target isn't an entity object , then it is assumed not to be in a secondary file
	//and so is fair game
	if(!pxTargetObject->IsEntityObject()) return true;

	//targets in the main file are okay
	if(!pszTargetFile || !*pszTargetFile) return true;

	if(!bSourceIsCutsceneFile)
	{
		//If the target is in the same file , than that is okay
		if(!stricmp(pszEntityFile,pszTargetFile)) return true;
	}

	//If the main entity file is writeable , then thats fine , since the guid of the other entity
	//should remain fixed
	if(Tools_SaveAsura::CanSaveEntity()) return true;

	//Also okay if the target has been saved at least once to the main entity file , since its
	//guid will now be fixed.
	if(pxEntityObject->Entity->ToolsData->m_bSecondaryEntityGuidConstant) return true;
	
	//can't use this target then
	if(bShowMessage)
	{
		ShowMessageOverlay("Can't target %s because it belongs to a secondary entity file , and has not yet been saved at least once to the main entity file.",pxTargetObject->GetName());
	}
	return false;		
}

bool Tools_SecondaryEntityFile_System::IsAllowableTarget(Tools_Object* pxSourceObject,Tools_Object* pxTargetObject,bool bShowMessage /*= true*/)
{
	Asura_Assert(pxSourceObject, "");
	if(!pxSourceObject) return false;

	bool bCutsceneFile = (pxSourceObject->Type == TOT_Cutscene || pxSourceObject->Type == TOT_Entity_CutsceneController);

	if(bCutsceneFile)
	{
		return IsAllowableTarget(NULL, true, pxTargetObject, bShowMessage);
	}
	else if(pxSourceObject->IsEntityObject())
	{
		Tools_Entity_Object* pxSourceEntityObject = (Tools_Entity_Object*) pxSourceObject;

		const char* szEntityFile = pxSourceEntityObject->Entity->ToolsData->m_xSecondaryEntityFilename.GetString();
		return IsAllowableTarget(szEntityFile, false, pxTargetObject, bShowMessage);
	}
	else
	{
		return IsAllowableTarget(NULL, false, pxTargetObject, bShowMessage);
	}
}

void Tools_SecondaryEntityFile_System::ToggleOverlay()
{
	if(!OverlayIsOn())
	{
		s_bOverlayIsOn = true;
		CreateOverlay();
	}
	else
	{
		s_bOverlayIsOn = false;
		CloseOverlay();
	}
}

bool Tools_SecondaryEntityFile_System::OverlayIsOn()
{
	return Asura_Overlay_System::DoesOverlayExist(TOOLS_OVERLAY_SECONDARYENTFILE);
}

void HistoryDoubleClickCallback(int iUserData)
{
	Tools_Object* pxObject = Tools_SecondaryEntityFile_System::s_axHistory[s_iHistorySelectionIndex];
	Tools_Object_System::SelectObject(pxObject);
	s_iHistorySelectionIndex = 0;
}

void Tools_SecondaryEntityFile_System::CreateOverlay()
{

	if(!s_bOverlayIsOn) return;
	if(Asura_Overlay_System::DoesOverlayExist(TOOLS_OVERLAY_SECONDARYENTFILE)) return;


	Asura_Overlay* pxOverlay = new Tools_Overlay_SecondaryEntityFile("Entity Manager",false,&s_xOverlayPosition);

	s_pxOverlay = pxOverlay;

	Asura_Vector_2 xOffset(1.0f,3.0f);
	Asura_Dialog* pxDialog;

	#ifdef TOOLS_USE_OLD_ENTITY_MANAGER
	Asura_Tabs_Dialog* pxTabsDialog = new Asura_Tabs_Dialog(xOffset, 0, 0);
	pxOverlay->AddDialog(pxTabsDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();
	pxOverlay->SetCurrentParentDialog(pxTabsDialog);

	float fTabsStartY = xOffset.y;
	float fTabsEndY = xOffset.y;

	//Add the main tree and filters to the main tab
	pxTabsDialog->AddTab("Main");
	#endif

	///////////////////

	#ifdef TOOLS_USE_OLD_ENTITY_MANAGER
	g_pxSelectFileDialog = pxDialog = new Asura_Dialog("File : ",xOffset);
	#else
	g_pxSelectFileDialog = pxDialog = new Asura_Dialog("Working File : ",xOffset);
	#endif

	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	///////////////////
	//   Tree
	///////////////////

	CreateTree();

	pxDialog = s_pxTreeDialog = new Asura_Tree_Dialog( xOffset, &s_xTree, 30.0f, 20.0f, &s_bSelectionChanged);
	s_pxTreeDialog->SetDragDropCallback( Asura_Callback_Void_1Arg<void,Asura_TreeDialog_Item*>(&Tools_SecondaryEntityFile_System::DragAndDrop));
	s_pxTreeDialog->SetIconCallback( Asura_Callback_Void_1Arg<int,Asura_TreeDialog_Item*>(&Tools_SecondaryEntityFile_System::GetItemIconIndex));
	#ifndef TOOLS_USE_OLD_ENTITY_MANAGER
	s_pxTreeDialog->SetAdditionalIconCallback( Asura_Callback_Void_2Arg<int,Asura_TreeDialog_Item*,u_int>(&Tools_SecondaryEntityFile_System::GetAdditionalItemIconIndex));
	#endif
	s_pxTreeDialog->SetDoubleClickedFunction(GotoSelected);
	s_pxTreeDialog->SetRenameCallback( Asura_Callback_Void_2Arg<void, Asura_TreeDialog_Item*,const char*> (RenameItem));
	s_pxTreeDialog->SetIconClickCallback( Asura_Callback_Void_2Arg<void, Asura_TreeDialog_Item*, bool>(IconClicked));
	#ifndef TOOLS_USE_OLD_ENTITY_MANAGER
	s_pxTreeDialog->SetAdditionalIconClickCallback( Asura_Callback_Void_2Arg<void, Asura_TreeDialog_Item*, bool>(AdditionalIconClicked));
	#endif
	pxOverlay->AddDialog( pxDialog );
	xOffset.y+=20.0f+1.0f;

#if 0
	//add control for resizing the tree
	//Placing it on the right of the overlay , below the tree.
	//Being placed outside of the tab control to avoid issues caused by the tab dialog resizing to be slightly larget
	//than the extents of everything within it
	pxOverlay->SetCurrentParentDialog(0);
	pxDialog = new Asura_Resize_Dialog(s_pxTreeDialog, &s_xTreeDialogSize);
	pxDialog->SetRelativePositionFlags(s_pxTreeDialog, Asura_Vector_2(-pxDialog->GetWidth(), 0.0f), ASURA_DIALOG_RELATIVE_X_OVERLAY|ASURA_DIALOG_RELATIVE_X_RIGHT|ASURA_DIALOG_RELATIVE_Y_DIALOG|ASURA_DIALOG_RELATIVE_Y_BOTTOM);
	pxDialog->SetIsHiddenFunction(IsNotOnMainTab,0);
	pxOverlay->AddDialog( pxDialog );
	pxOverlay->SetCurrentParentDialog(pxTabsDialog);
#else
	pxDialog = new Asura_Resize_Dialog(s_pxTreeDialog, &s_xTreeDialogSize);
	#ifdef TOOLS_USE_OLD_ENTITY_MANAGER
	pxDialog->SetRelativePosition(s_pxTreeDialog, Asura_Vector_2(-pxDialog->GetWidth(), -pxDialog->GetHeight()), true, true);
	#endif
	pxOverlay->AddDialog( pxDialog );
#endif

	///////////////////

	#ifndef TOOLS_USE_OLD_ENTITY_MANAGER
	pxDialog = new Asura_Function_Dialog("Create File",xOffset,CreateEntityFile);
	pxDialog->SetHelpText("Create a new secondary entity file.\nThe main entity file must be checked out to do this.");
	pxDialog->SetRelativePosition (s_pxTreeDialog, true, false);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Create Folder",xOffset,CreateFolder);
	pxDialog->SetHelpText("Create a folder for organising the entities within tools.");
	pxDialog->SetRelativePosition (s_pxTreeDialog, true, false);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Collapse all",xOffset,CollapseAllTreeBranches);
	pxDialog->SetHelpText("Collapse all the tree branches.");
	pxDialog->SetRelativePosition (s_pxTreeDialog, true, false);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::LargeSpacing();
	#endif

	#ifndef TOOLS_USE_OLD_ENTITY_MANAGER
	pxDialog = new Asura_Tickbox_Dialog("Show Hidden Entities",xOffset,&s_bShowHiddenEntities);
	pxDialog->SetHelpText("Toggles whether hidden entities are displayed in the treeview.");
	pxDialog->SetRelativePosition(s_pxTreeDialog,true,false);
	(static_cast <Asura_Tickbox_Dialog *> (pxDialog))->SetClickedFunction (&OnShowHiddenEntitiesTickboxTicked);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing()*2.0f;
	#endif
	
	#ifdef TOOLS_USE_OLD_ENTITY_MANAGER
	pxDialog = new Asura_Tickbox_Dialog("Hide Other Files",xOffset,&s_bHideObjectsInOtherFiles);
	pxDialog->SetHelpText("If ticked then any entities that don't belong to the current entity file will be hidden.");
	pxDialog->SetRelativePosition(s_pxTreeDialog,true,false);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Tickbox_Dialog("Hide This File",xOffset,&s_bHideObjectsInThisFile);
	pxDialog->SetHelpText("If ticked then any entities that don't belong to the current entity file will be hidden.");
	pxDialog->SetRelativePosition(s_pxTreeDialog,true,false);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing()*2.0f;
	#endif

	// History panel

	Asura_Vector_2 xPanelOffset;
	xPanelOffset.x = 1.0f;
	xPanelOffset.y = 1.5f;

	s_pxHistoryPanelDialog = new Asura_Panel_Dialog(xOffset, Asura_Vector_2(0.0f,0.0f), 0);
	s_pxHistoryPanelDialog->SetRelativePosition(s_pxTreeDialog,true,false);
	pxOverlay->AddDialog(s_pxHistoryPanelDialog);
	s_pxHistoryPanelDialog->UpdateLabel("History (click to expand/contract)");
	s_pxHistoryPanelDialog->SetRenderBorder(true);
	s_pxHistoryPanelDialog->EnableMinimise(&s_bHistoryMinimised);

	s_pxHistoryListDialog = new Asura_List_Dialog(xPanelOffset, s_axHistoryStrings, &s_iHistorySelectionIndex, s_uNumHistorySlots, s_uNumUsedHistorySlots, NULL);
	s_pxHistoryPanelDialog->AddDialogToPanel(s_pxHistoryListDialog);

	s_pxHistoryListDialog->SetDoubleClickFunction(HistoryDoubleClickCallback, 0);

	xPanelOffset.y += Asura_Dialog::LargeSpacing();

	if(s_bHistoryMinimised)
	{
		xOffset.y += Asura_Dialog::LargeSpacing();
	}
	else
	{
		xOffset.y += s_pxHistoryPanelDialog->GetHeight() + Asura_Dialog::SmallSpacing();
	}

	// Filters panel
	s_pxPanelDialog = new Asura_Panel_Dialog(xOffset, Asura_Vector_2(0.0f,0.0f), 0);
	s_pxPanelDialog->SetRelativePosition(s_pxHistoryPanelDialog,true,false);
	pxOverlay->AddDialog(s_pxPanelDialog);
	#ifdef TOOLS_USE_OLD_ENTITY_MANAGER
	s_pxPanelDialog->UpdateLabel("Filters");
	#else
	s_pxPanelDialog->UpdateLabel("Filters (click to expand/contract)");
	#endif
	s_pxPanelDialog->SetRenderBorder(true);
	s_pxPanelDialog->EnableMinimise(&s_bFiltersMinimised);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	xPanelOffset.x = 1.0f;
	xPanelOffset.y = 1.5f;

	//Search/Search Type
	Asura_DropDown_List_Dialog* pxDropListDialog = new Asura_DropDown_List_Dialog(xPanelOffset,g_aszSearchOptions,&s_iSearchOption,2,2,NULL,&s_bFilterChanged,"");
	pxDropListDialog->SetHelpText("Determines how the search text should be used for filtering the tree.");
	s_pxPanelDialog->AddDialogToPanel(pxDropListDialog);
	xPanelOffset.y += Asura_Dialog::DefaultSpacing();


	//xPanelOffset.x = pxDropListDialog->MaxCoord.x + 1.0f;
	pxDialog = new Asura_String_Dialog("",xPanelOffset,&s_xSearchText,20,&s_bFilterChanged);
	pxDialog->SetHelpText("Filters the tree so that it only shows entities beginning with/containing this string");
	//xPanelOffset.x = 1.0f;
	#ifndef TOOLS_USE_OLD_ENTITY_MANAGER
	s_pxFilterSearchStringDialog = static_cast <Asura_String_Dialog *> (pxDialog);
	#endif
	s_pxPanelDialog->AddDialogToPanel(pxDialog);

	#ifndef TOOLS_USE_OLD_ENTITY_MANAGER
	Asura_Callback_Void <void> xMyCallback (FilterTextChangedCallback);
	s_pxFilterSearchStringDialog->SetTextChangeCallback (xMyCallback);
	#endif

	xPanelOffset.y += Asura_Dialog::LargeSpacing();

	//add filter types
	static const char* s_apszFilterNames [FILTER_NUMTYPES] =
	{
		"Type",
		"Shape/Skin",
        "Blueprint Type",
        "Blueprint"
	};
	

	pxDialog = new Asura_DropDown_List_Dialog(xPanelOffset,s_apszFilterNames,&s_iFilterType,FILTER_NUMTYPES,FILTER_NUMTYPES,NULL,&s_bFilterTypeChanged,"Filter By");
	s_pxPanelDialog->AddDialogToPanel(pxDialog);
	xPanelOffset.y += Asura_Dialog::LargeSpacing();


	//add controls for the list of object types
	pxDialog = new Asura_Dialog("Filter List :",xPanelOffset);
	s_pxPanelDialog->AddDialogToPanel(pxDialog);
	xPanelOffset.y += Asura_Dialog::SmallSpacing();

	static const char* s_apszDummyList[1] = {"*All*"};

	//filter strings will be filled in properly later
	pxDialog = s_pxFilterDialog = new Asura_List_Dialog(xPanelOffset,s_apszDummyList,&s_iFilterIndex,10,1,NULL,&s_bFilterIndexChanged);
	s_pxFilterDialog->SetExpandWidthToFillOverlay(false);
	s_pxFilterDialog->EnableMultipleSelection(true, FilterList_IsSelected, FilterList_SetSelected);
	pxDialog->SetHelpText("Select type to filter main object list by.");
	s_pxPanelDialog->AddDialogToPanel(pxDialog);
	xPanelOffset.y=pxDialog->m_xMaxCoord.y;
	xPanelOffset.y+=1.0f;

	#ifdef TOOLS_USE_OLD_ENTITY_MANAGER
	//add control for resizing the list
	//not putting it in the panel , but only making it visible when the panel is maximised
	pxDialog = new Asura_Resize_Dialog(s_pxFilterDialog, &s_xFilterListDialogSize);
	pxDialog->SetRelativePosition(s_pxPanelDialog, Asura_Vector_2(-pxDialog->GetWidth(),-pxDialog->GetHeight()), true, true);
	pxDialog->SetIsHiddenFunction(IsFilterPanelMinimised, 0);
	pxOverlay->AddDialog( pxDialog );
	#endif

	//mark the filters as changed to force them to be updated immediately
	s_bFilterTypeChanged = true;
	s_bFilterIndexChanged = true;
	DeleteFilterList();

	xOffset.y = s_pxPanelDialog->m_xMaxCoord.y + 1.0f;

	#ifdef TOOLS_USE_OLD_ENTITY_MANAGER

	//end of main tab
	if(xOffset.y > fTabsEndY) fTabsEndY = xOffset.y;

	//utility tab for some less used stuff
	pxTabsDialog->AddTab("Utility");
	xOffset.y = fTabsStartY;
	

	pxDialog = new Asura_Function_Dialog("Create File",xOffset,CreateEntityFile);
	pxDialog->SetHelpText("Create a new secondary entity file.\nThe main entity file must be checked out to do this.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	//test
	pxDialog = new Asura_Function_Dialog("Create Folder",xOffset,CreateFolder);
	pxDialog->SetHelpText("Create a folder for organising the entities within tools.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::LargeSpacing();


	pxDialog = new Asura_Function_Dialog("Create Common File",xOffset,CreateCommonEntityFile);
	pxDialog->SetHelpText("Create a common secondary entity file , which is intended for sharing between multiple main entity files , but has more restrictions on how it can be used.\nMainly intended for use with the mission file system.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Add File",xOffset,AddEntityFile);
	pxDialog->SetHelpText("Add an existing secondary entity file.\nThe main entity file must be checked out to do this.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::LargeSpacing();

	///////////////////

	pxDialog = new Asura_Function_Dialog("Merge File",xOffset,MergeEntityFile);
	pxDialog->SetHelpText("Merges the contents of the current entity file back into the main entity file.\nBoth files need to be checked out to do this.");
	pxDialog->SetIsEnabledFunction(IsGroupSelected);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Transfer Entities",xOffset,TransferSelectedObjects);
	pxDialog->SetHelpText("Transfers the entities that are selected in the world to the current entity file.\nBoth the source and destination files must be checked out to do this.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::LargeSpacing();

	pxDialog = new Asura_Function_Dialog("Delete File",xOffset,DeleteEntityFile);
	pxDialog->SetHelpText("Deletes the secondary file and all entities within it.\nBoth files need to be checked out to do this.");
	pxDialog->SetIsEnabledFunction(IsGroupSelected);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Remove File",xOffset,RemoveEntityFile);
	pxDialog->SetHelpText("Removes the secondary file and deletes the entities from the level.\nThe main asura file needs to be checked out to do this.");
	pxDialog->SetIsEnabledFunction(IsGroupSelected);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::LargeSpacing();

	///////////////////

	pxDialog = new Asura_Function_Dialog("Rename File",xOffset,RenameEntityFile);
	pxDialog->SetHelpText("Renames the secondary file and updates the existing entities so that they refer to it instead.");
	pxDialog->SetIsEnabledFunction(IsGroupSelected);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Replace File",xOffset,ReplaceEntityFile);
	pxDialog->SetHelpText("Replaces the secondary file with another existing file and updates from that file.\nMainly for dealing with shared files that have already been renamed by another mission.");
	pxDialog->SetIsEnabledFunction(IsGroupSelected);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	//end of utility tab with no good name
	if(xOffset.y > fTabsEndY) fTabsEndY = xOffset.y;

	//end of tabs
	xOffset.y = fTabsEndY + 1.0f;
	pxOverlay->SetCurrentParentDialog(0);

 	pxTabsDialog->SetSelectedTabPointer(&s_uSelectedTab);

	#endif

	pxOverlay->SetContextMenuCallback(AddContextMenuItems, NULL);

	pxOverlay->m_uID = TOOLS_OVERLAY_SECONDARYENTFILE;

	Asura_Overlay_System::SubmitNewOverlay(pxOverlay);

	UpdateOverlay();
	UpdateFilterList();

	RestoreTreeState();
}

#ifndef TOOLS_USE_OLD_ENTITY_MANAGER
void Tools_SecondaryEntityFile_System::OnShowHiddenEntitiesTickboxTicked ()
{
	UpdateTreeVisibility ();
}
#endif

#ifndef TOOLS_USE_OLD_ENTITY_MANAGER
void Tools_SecondaryEntityFile_System::FilterTextChangedCallback ()
{
	s_bFilterChanged = true;
}
#endif

struct SecondaryEntityFileSystem_TreeStateStruct
{
	Asura_Hash_ID uHashId;
	bool bIsExpanded;
};

void Tools_SecondaryEntityFile_System::StoreTreeStateRecurse(FILE* pFile, Asura_TreeDialog_Item* pxFirstSibling, u_int& uCounter)
{
	Asura_TreeDialog_Item* pxItem = pxFirstSibling;

	for(; pxItem; pxItem = pxItem->GetSibling())
	{
		if(pFile)
		{
			SecondaryEntityFileSystem_TreeStateStruct xStateInfo;

			xStateInfo.uHashId = pxItem->GetHash();
			xStateInfo.bIsExpanded = pxItem->IsExpanded();

			fwrite(&xStateInfo, sizeof(SecondaryEntityFileSystem_TreeStateStruct), 1, pFile);
		}

		uCounter++;

		StoreTreeStateRecurse(pFile, pxItem->GetChild(), uCounter);
	}
}

void Tools_SecondaryEntityFile_System::StoreTreeState()
{
	Tools_FileSystem::EnsureDirectoryExists("Temp");

	FILE* pFile = fopen("Temp\\EntManStates.dat", "wb");

	Asura_TreeDialog_Item* pxRoot = s_xTree.GetRoot();

	u_int uCounter = 0;

	// Count the number of lines and store as first entry in file
	StoreTreeStateRecurse(NULL, pxRoot, uCounter);
	fwrite(&uCounter, sizeof(u_int), 1, pFile);

	// Write the tree data to the file
	StoreTreeStateRecurse(pFile, pxRoot, uCounter);

	fclose(pFile);
}

void Tools_SecondaryEntityFile_System::RestoreTreeState()
{
	FILE* pFile = fopen("Temp\\EntManStates.dat", "rb");

	if(!pFile)
	{
		return;
	}

	// Read the state info from the file

	u_int uNumEntries;

	fread(&uNumEntries, sizeof(u_int), 1, pFile);

	SecondaryEntityFileSystem_TreeStateStruct* axStateInfo = new SecondaryEntityFileSystem_TreeStateStruct[uNumEntries];

	fread(axStateInfo, sizeof(SecondaryEntityFileSystem_TreeStateStruct) * uNumEntries, 1, pFile);

	// Set the state for each entry

	for(u_int n = 0; n < uNumEntries; n++)
	{
		Asura_TreeDialog_Item* pxItem = s_xTree.FindByHash(axStateInfo[n].uHashId);

		if(pxItem)
		{
			pxItem->Expand(axStateInfo[n].bIsExpanded);
		}
	}

	// Tidy up

	delete [] axStateInfo;

	fclose(pFile);
}

void Tools_SecondaryEntityFile_System::CloseOverlay()
{
	Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_SECONDARYENTFILE);

	s_pxOverlay = NULL;
}

void Tools_SecondaryEntityFile_System::OnOverlayClosed(bool bClosedByUser)
{
	StoreTreeState();

	g_pxSelectFileDialog = 0;
	s_pxFilterDialog = 0;
	s_pxTreeDialog = 0;
	if(bClosedByUser) s_bOverlayIsOn = false;
}

void Tools_SecondaryEntityFile_System::UpdateOverlay()
{
	Asura_Overlay* pxOverlay = Asura_Overlay_System::GetOverlay(TOOLS_OVERLAY_SECONDARYENTFILE);

	if(pxOverlay && g_pxSelectFileDialog)
	{
		char szText[100];
		if(s_pxCurrentFile)
		{
			#ifdef TOOLS_USE_OLD_ENTITY_MANAGER
			Asura_PrintfToBfr(szText,"File : %s",100,true,s_pxCurrentFile->GetDisplayName());
			#else
			Asura_PrintfToBfr(szText,"Working File : %s",100,true,s_pxCurrentFile->GetDisplayName());
			#endif
		}
		else
		{
			#ifdef TOOLS_USE_OLD_ENTITY_MANAGER
			Asura_PrintfToBfr(szText,"File : ????",100,true);
			#else
			Asura_PrintfToBfr(szText,"Working File : ????",100,true);
			#endif
		}

		pxOverlay->UpdateDialogLabel(g_pxSelectFileDialog,szText);
	}	
}

void Tools_SecondaryEntityFile_System::SetFilesToCheckForExternalModification()
{
	for(LIF<Tools_SecondaryEntityFile*> xFileLIF(&s_xFileList);!xFileLIF.done();xFileLIF.next())
	{
		if(xFileLIF()->IsMainFile()) continue;

		Asura_String xFilename;
		GenerateFilename(xFileLIF(),xFilename);
		
		Tools_LoadedFileList::TrackExternalModification(xFilename.GetString());
	}
}


bool Tools_SecondaryEntityFile_System::CanSaveCurrentFile()
{
	Asura_Assert(s_pxCurrentFile, "");

	if(s_pxCurrentFile)
	{
		return s_pxCurrentFile->CanSaveFile(false);
	}
	return false;
}

void Tools_SecondaryEntityFile_System::UpdateTree( void )
{
	//can skip this (I think) if the overlay doesn't exist.
	//RAR 30/07/2007
	if(!Asura_Overlay_System::GetOverlay(TOOLS_OVERLAY_SECONDARYENTFILE)) return;


	if(!s_bTreeValid) CreateTree();

	if(s_uTreeItemColourValidity != Tools_System::GetReactivationCount())
	{
		//some alt-tabbing has happened since the last update.
		//check the colours of everything in the tree , in case a file has become writeable
		UpdateColourRecursive(s_xTree.GetRoot());
		s_uTreeItemColourValidity = Tools_System::GetReactivationCount();
	}

	//check for changes to the filters
	if(s_bFilterTypeChanged)
	{
		UpdateFilterList();
		s_bFilterTypeChanged = false;
	}
	
	if(s_bFilterIndexChanged)
	{
		s_bFilterIndexChanged = false;
		s_bFilterChanged = true;
	}


	if(s_bFilterChanged)
	{
		UpdateTreeVisibility();
	}


	//Tools_Object* pxPrevSelectedObject = Tools_Object_System::SelectedObject;
	if(s_bSelectionChanged)
	{
		s_bSelectionChanged = false;
		UpdateObjectSelectionFromTree();

	}
	else if(Tools_Object_System::GetSelectionChangedStamp() != s_uObjectSystemSelectionChangedStamp)
	{
		UpdateTreeFromObjectSelection();
	}

	Asura_TreeDialog_Item* pxSelectedItem = s_xTree.GetSelectedItem();	


	if(pxSelectedItem)
	{
		Tools_SecondaryEntityFile* pxFile = GetFileFromTreeItem(pxSelectedItem);
		if(pxFile && pxFile->CanSaveFile(false))
		{
			s_pxCurrentFile = pxFile;
			GetFolderPathFromTreeItem(pxSelectedItem, s_xCurrentFolder);
		}
		if(!s_pxCurrentFile) s_pxCurrentFile = GetMainFile();
		Asura_Assert(s_pxCurrentFile, "");
	
		
		{
			//select the entity in main display
			//Tools_Object *pxToolsObject = Tools_Object_System::SelectedObject;

			//if (pxToolsObject && pxToolsObject != pxPrevSelectedObject)
			//{
				//Tools_Object_System::SelectObject(pxToolsObject);
 				//Asura_Vector_3 xPosition = pxToolsObject->Position;
 				//Asura_Vector_3 xFwdVec;
 				//Asura_Camera::GetForwardVector(xFwdVec);
 				//xPosition -= xFwdVec * 10.0f;
 				//Asura_Matrix_3x3 xMat = *Asura_Camera::GetOrientation();
 				//Tools_Movement::TeleportCameraToPosition(&xPosition, &xMat);
			//}
		}
	}

	UpdateOverlay();
}

void Tools_SecondaryEntityFile_System::UpdateObjectSelectionFromTree()
{
	//list of objects to select
	List<Tools_Object*> xObjectList;

	for(CLIF<Asura_TreeDialog_Item*> xSelectionLIF(&s_xTree.GetSelection()); !xSelectionLIF.done(); xSelectionLIF.next())
	{
		//just checking for selected entities in the tree at the moment.
		//(may or may not be desirable to select based on folder/file later)
		Tools_Object* pxObject = GetToolsEntityObjectFromTreeItem(xSelectionLIF());
		if(pxObject)
		{
			xObjectList.add_entry(pxObject);
		}
	}

	//Only going to update the selection if there is at least one object in the list
	//(to be roughly equivalent to the old behaviour)
	if(xObjectList.size())
	{
		Tools_Object_System::SelectObjects(xObjectList, false);

		Asura_Overlay_System::BringOverlayToTop(TOOLS_OVERLAY_SECONDARYENTFILE);
	}

	s_uObjectSystemSelectionChangedStamp = Tools_Object_System::GetSelectionChangedStamp();
}

void Tools_SecondaryEntityFile_System::UpdateTreeFromObjectSelection()
{
	s_xTree.ClearSelection();

	List<Tools_Object*> xSelectedObjects;
	Tools_Object_System::GetSelectionListForCurrentSelectionMode(xSelectedObjects);
	
	//Add to the tree selection for each selected entity object

	for(LIF<Tools_Object*> xObjectLIF(&xSelectedObjects); !xObjectLIF.done(); xObjectLIF.next())
	{
		if(!xObjectLIF()->IsEntityObject()) continue;

		Asura_Guid uGuid = ((Tools_Entity_Object*)xObjectLIF())->GetGuid();

		Asura_TreeDialog_Item* pxEntityItem = s_xTree.FindByData(uGuid);
		if(pxEntityItem)
		{
			s_xTree.AddToSelection(pxEntityItem);
		}
	}

	Tools_Object* pxMainSelection = Tools_Object_System::GetMainSelectionForCurrentSelectionMode();
	//Add the main selected object again to make it to the main selection in the tree
	if(pxMainSelection && pxMainSelection->IsEntityObject())
	{
		Asura_Guid uGuid = ((Tools_Entity_Object*)pxMainSelection)->GetGuid();

		Asura_TreeDialog_Item* pxEntityItem = s_xTree.FindByData(uGuid);
		if(pxEntityItem)
		{
			s_xTree.AddToSelection(pxEntityItem);
			if(s_pxTreeDialog)
			{
				s_pxTreeDialog->MakeSelectionVisible();
			}
		}
	}
	
	
	s_uObjectSystemSelectionChangedStamp = Tools_Object_System::GetSelectionChangedStamp();
}

bool Tools_SecondaryEntityFile_System::IsGroupSelected()
{
	if(s_pxCurrentFile && !s_pxCurrentFile->IsMainFile())
	{
		Asura_TreeDialog_Item* pxItem = s_xTree.GetSelectedItem();
		if(pxItem)
		{
			return(pxItem->GetDepth() == 1);
		}
	}
	
	return(false);
}



void Tools_SecondaryEntityFile_System::CreateTree()
{
	// Get the currently selected item:
	Asura_TreeDialog_Item* pxCurrentItem = s_xTree.GetSelectedItem();
	Asura_String xCurrentFileName;
	Asura_String xCurrentPath;
	Asura_Guid uCurrentGuid = ASURA_GUID_UNREGISTERED;
	if(pxCurrentItem)
	{
		xCurrentFileName = GetFileDisplayNameFromTreeItem(pxCurrentItem);
		GetFolderPathFromTreeItem(pxCurrentItem, xCurrentPath);
		uCurrentGuid = GetGuidFromTreeItem(pxCurrentItem);

	}

	//Strip the tree
	s_xTree.DeleteAll();
	Asura_TreeDialog_Item* pxRoot = s_xTree.GetRoot();	


	//Rebuild the tree
	// Add each of the entity files
	for(int i=0;i<s_xFileList.size();i++)
	{
		Asura_TreeDialog_Item* pxNewItem = s_xTree.InsertItem(CompareTreeItem, pxRoot, s_xFileList[i]->GetDisplayName(), s_xFileList[i]->m_iFileID);
		pxNewItem->SetColourOverride(GetItemColour(pxNewItem));

		//Add folders for those entity files
//		for(LIF<Asura_String> xFolderLIF(&s_xFileList[i]->m_xFolderList); !xFolderLIF.done(); xFolderLIF.next())
//		{
//			GetFileOrFolderFromTree(s_xFileList[i], xFolderLIF().GetString(), true);
//		}
	}

	Asura_TreeDialog_Item* pxItemToSelect = 0;

	//Add all the entities
	for(LIF<Tools_Object*> xObjectLIF(&Tools_Object_System::ObjectList);!xObjectLIF.done();xObjectLIF.next())
	{
		//only interested in entities.
		if(!xObjectLIF()->IsEntityObject()) continue;
		Tools_Entity_Object* pxEntityObject = (Tools_Entity_Object*) xObjectLIF();

		Asura_TreeDialog_Item* pxItem = AddEntityToTree(pxEntityObject);
		if(uCurrentGuid != ASURA_GUID_UNREGISTERED && pxEntityObject->GetGuid() == uCurrentGuid)
		{
			pxItemToSelect = pxItem;
		}
	}

	//If no item to reselect has been found yet , try to get as close to the file/folder path as we can
	if(!pxItemToSelect && !xCurrentFileName.IsBlank())
	{
		pxItemToSelect = s_xTree.GetRoot()->FindChildByName( xCurrentFileName.GetString(), true);

		if(pxItemToSelect &&  !xCurrentPath.IsBlank())
		{
			char szBuffer[500];
			Asura_Assert(sizeof(szBuffer)>xCurrentPath.GetLength(), "");
			Asura_PrintfToBfr(szBuffer, "%s", sizeof(szBuffer), true, xCurrentPath.GetString());

			const char* szToken = strtok(szBuffer, "\\");

			while (szToken)
			{
				Asura_TreeDialog_Item* pxChildItem = pxItemToSelect->FindChildByName(szToken, true);
				if(!pxChildItem)
				{
					break;
				}
				pxItemToSelect = pxChildItem;
				szToken = strtok(NULL, "\\");
			}
		}
	}

	//Open up the currently in use part of the table
	if(pxItemToSelect)
	{
		s_xTree.SetSelectedItem(pxItemToSelect);
		pxItemToSelect->MakeVisible();
		pxItemToSelect->Expand(true);
	}
	else
	{
		//re-select the file so that the appropriate top level item gets selected
		SetSelectedFile(s_pxCurrentFile ? s_pxCurrentFile : GetMainFile());
	}
	
	s_bTreeValid = true;
	//force item visibilities to be recalculated
	s_bFilterChanged = true;
	s_bFilterTypeChanged = true;
	s_uTreeItemColourValidity = Tools_System::GetReactivationCount();
}

Asura_TreeDialog_Item* Tools_SecondaryEntityFile_System::AddEntityToTree(Tools_BaseEntity_Object* pxEntityObject)
{
	//Skip template instance members, since only the template instance itself is selectable
	if(pxEntityObject->IsMemberOfTemplateInstance()) return 0;

	//get the tools data
	Tools_Entity_Data* pxToolsData = pxEntityObject->Entity->ToolsData;

	//get the name of the entity file
	const char* szEntityFilename = pxToolsData->m_xSecondaryEntityFilename.IsBlank() ? GetMainFileLabel() : pxToolsData->m_xSecondaryEntityFilename.GetString();

	//and find it in the tree
	Asura_TreeDialog_Item* pxParentItem = s_xTree.GetRoot()->FindChildByName(szEntityFilename, true);
	if(!pxParentItem)
	{
		Asura_Assert(0, "Failed to find entry for entity file in tree");
		return 0;
	}

	//find/create each of the folders needed
	if(!pxToolsData->m_xEntityBrowserPath.IsBlank())
	{
		char szBuffer[500];
		Asura_Assert(sizeof(szBuffer)>pxToolsData->m_xEntityBrowserPath.GetLength(), "");
		Asura_PrintfToBfr(szBuffer, "%s", sizeof(szBuffer), true, pxToolsData->m_xEntityBrowserPath.GetString());

		const char* szToken = strtok(szBuffer, "\\");
		while (szToken)
		{
			Asura_TreeDialog_Item* pxChildItem = pxParentItem->FindChildByName(szToken, true);
			if(!pxChildItem)
			{
				pxChildItem = s_xTree.InsertItem( CompareTreeItem, pxParentItem, szToken, g_iFolderTreeItemID);
				pxChildItem->SetColourOverride(GetItemColour(pxChildItem));
			}
			pxParentItem = pxChildItem;
			szToken = strtok(NULL, "\\");
		}
	}

	#ifdef TOOLS_USE_OLD_ENTITY_MANAGER
	Asura_TreeDialog_Item* pxItem = s_xTree.InsertItem(CompareTreeItem, pxParentItem, pxEntityObject->GetName (), pxEntityObject->GetGuid());
	#else
	int iStartIndex, iEndIndex;
	Asura_String xExpandedName = Tools_SecondaryEntityFile_System::GetExpandedName (pxEntityObject, iStartIndex, iEndIndex);
	Asura_TreeDialog_Item* pxItem = s_xTree.InsertItem(CompareTreeItem, pxParentItem, xExpandedName.GetString (), pxEntityObject->GetGuid());
	pxItem->SetRenameExtents (iStartIndex, iEndIndex);
	#endif

	pxItem->SetColourOverride(GetItemColour(pxItem)); 

	return pxItem;
}

#ifndef TOOLS_USE_OLD_ENTITY_MANAGER
Asura_String Tools_SecondaryEntityFile_System::GetExpandedName (Tools_BaseEntity_Object* pxEntityObject, int& iStartIndex, int& iEndIndex)
{
	char szDisplayName [256];
	Asura_PrintfToBfr (szDisplayName, "%s: %s <%u>", 256, true, pxEntityObject->GetShortName (), pxEntityObject->GetName (), pxEntityObject->GetGuid ());

	iStartIndex = -1;
	iEndIndex = -1;

	for (u_int n = 0; n < strlen (szDisplayName); n ++)
	{
		if ((szDisplayName [n] == ':') && (iStartIndex == -1))
		{
			iStartIndex = n + 2;
		}
		else if ((szDisplayName [n] == '<') && (iEndIndex == -1))
		{
			iEndIndex = n - 2;
		}
	}

	return Asura_String (szDisplayName);
}

Asura_String Tools_SecondaryEntityFile_System::GetNameSubString (Asura_TreeDialog_Item* pxTreeItem, const char* pszFullString)
{
	int iLeftIndex, iRightIndex;

	pxTreeItem->GetRenameExtents (iLeftIndex, iRightIndex);

	if ((iLeftIndex != -1) && (iRightIndex != -1))
	{
		return Asura_String (pszFullString).SubString (iLeftIndex, (iRightIndex - iLeftIndex) + 1);
	}
	else
	{
		return pszFullString;
	}
}
#endif

void Tools_SecondaryEntityFile_System::UpdateEntityInTree(Tools_BaseEntity_Object* pxEntityObject, bool bSelect)
{
	if(!s_bTreeValid) return;

	//Skip template instance members, since only the template instance itself is selectable
	if(pxEntityObject->IsMemberOfTemplateInstance()) return;
	
	//search for an existing item
	Asura_Guid uGuid = pxEntityObject->GetGuid();
	Asura_TreeDialog_Item* pxEntityItem = s_xTree.FindByData(uGuid);
	if(!pxEntityItem)
	{
		//add a new entry
		pxEntityItem = AddEntityToTree(pxEntityObject);
	}
	else
	{
		//get the tools data
		Tools_Entity_Data* pxToolsData = pxEntityObject->Entity->ToolsData;

		//find the destination file/folder
		Asura_TreeDialog_Item* pxFolderItem = GetFileOrFolderFromTree(pxToolsData->m_xSecondaryEntityFilename.GetString(), pxToolsData->m_xEntityBrowserPath.GetString(), true);
		Asura_Assert(pxFolderItem, "");
		if(!pxFolderItem) return;

		#ifdef TOOLS_USE_OLD_ENTITY_MANAGER
		pxEntityItem->SetName(pxToolsData->Name.GetString());
		#else
		int iStartIndex, iEndIndex;
		Asura_String xExpandedName = Tools_SecondaryEntityFile_System::GetExpandedName (pxEntityObject, iStartIndex, iEndIndex);
		pxEntityItem->SetName (xExpandedName.GetString ());
		pxEntityItem->SetRenameExtents (iStartIndex, iEndIndex);
		#endif

		//and move it
		s_xTree.MoveItem(CompareTreeItem, pxEntityItem, pxFolderItem);
		pxEntityItem->SetColourOverride(GetItemColour(pxEntityItem));
	}
	if(pxEntityItem && bSelect)
	{
		s_xTree.SetSelectedItem(pxEntityItem);
	}

	//If this is being called , then it is posible that the history strings will need regenerating , because of a renamed object
	s_bNeedToUpdateHistoryStrings = true;

	//force item visibilities to be recalculated
	s_bFilterChanged = true;
	s_bFilterTypeChanged = true;
}

void Tools_SecondaryEntityFile_System::RemoveEntityFromTree(Asura_Guid uGuid)
{
	if(!s_bTreeValid) return;
	if(uGuid == ASURA_GUID_UNREGISTERED) return;

	//Search for an entry with this guid in the tree
	Asura_TreeDialog_Item* pxEntityItem = s_xTree.FindByData(uGuid);
	if(pxEntityItem)
	{
		s_xTree.DeleteItem(pxEntityItem);	
	}
}

Asura_TreeDialog_Item* Tools_SecondaryEntityFile_System::GetFileOrFolderFromTree(const char* szEntityFile, const char* szFolder, bool bCreateIfNeeded)
{
	return GetFileOrFolderFromTree( GetFile(szEntityFile), szFolder, bCreateIfNeeded);
}

Asura_TreeDialog_Item* Tools_SecondaryEntityFile_System::GetFileOrFolderFromTree(Tools_SecondaryEntityFile* pxFile, const char* szFolder, bool bCreateIfNeeded)
{
	Asura_Assert(pxFile, "");
	if(!pxFile) return 0;

	Asura_TreeDialog_Item* pxRoot = s_xTree.GetRoot();
	
	//first get the item corresponding to the file
	Asura_TreeDialog_Item* pxItem = pxRoot->FindChildByName(pxFile->GetDisplayName(), false);
	if(!pxItem)
	{
		Asura_Assert(0, "Not expecting to fail to find a tree item for the file");
		if(!bCreateIfNeeded) return 0;

		pxItem = s_xTree.InsertItem(CompareTreeItem, pxRoot, pxFile->GetDisplayName(), pxFile->m_iFileID);
		pxItem->SetColourOverride(GetItemColour(pxItem));
	}

	if(szFolder && *szFolder)
	{
		//find/create each item in the path
		char szBuffer[500];
		Asura_Assert(sizeof(szBuffer)> strlen(szFolder), "");
		Asura_PrintfToBfr(szBuffer, "%s", sizeof(szBuffer), true, szFolder);

		const char* szToken = strtok(szBuffer, "\\");

		while (szToken)
		{
			Asura_TreeDialog_Item* pxChildItem = pxItem->FindChildByName(szToken, true);
			if(!pxChildItem)
			{
				if(!bCreateIfNeeded) return 0;
				pxChildItem = s_xTree.InsertItem( CompareTreeItem, pxItem, szToken, g_iFolderTreeItemID);
				pxChildItem->SetColourOverride(GetItemColour(pxChildItem));
			}
			pxItem = pxChildItem;
			szToken = strtok(NULL, "\\");
		}
	}

	return pxItem;
}

int Tools_SecondaryEntityFile_System::CompareTreeItem( const Asura_TreeDialog_Item& xOne, const Asura_TreeDialog_Item& xTwo)
{
	//Sort by name , but put folders first if there are a mix of folders an entities at a given level
	//also put the main entity file first at the top level of the tree

	//The main entity file should end up with an index of 0 , so test for that
	if(xOne.GetUserData() == 0) return -1;
	if(xTwo.GetUserData() == 0) return 1;

	bool bOneIsFolder = xOne.GetUserData() == g_iFolderTreeItemID;
	bool bTwoIsFolder = xTwo.GetUserData() == g_iFolderTreeItemID;

	if(bOneIsFolder != bTwoIsFolder)
	{
		return bOneIsFolder ? -1 : 1;
	}

	return stricmp(xOne.GetName(), xTwo.GetName());
}


//Returns a colour value based if the file can be written
u_int Tools_SecondaryEntityFile_System::GetItemColour(Asura_TreeDialog_Item* pxItem)
{
	Tools_SecondaryEntityFile* pxCurrentFile = GetFileFromTreeItem(pxItem);

	
	bool bCommonFile = pxCurrentFile ? pxCurrentFile->m_bCommonFile : false;
//	bool bFolder = pxItem->GetUserData() == g_iFolderTreeItemID;

	{
		if(!CanSaveFileCached(pxCurrentFile))
		{
			if(bCommonFile && pxItem->GetDepth() == 1)
				return TOOLS_SECONDARY_ENTITY_DISABLED_COMMON_COLOUR; 
//			else if(bFolder)
//				return TOOLS_SECONDARY_ENTITY_DISABLED_FOLDER_COLOUR;
			else
				return TOOLS_SECONDARY_ENTITY_DISABLED_COLOUR;
		}
		else if(bCommonFile && pxItem->GetDepth() == 1)
		{
			return TOOLS_SECONDARY_ENTITY_COMMON_COLOUR;
		}
//		else if (bFolder)
//		{
//			return TOOLS_SECONDARY_ENTITY_FOLDER_COLOUR;
//		}
	}
	return 0;
}

int Tools_SecondaryEntityFile_System::GetItemIconIndex(Asura_TreeDialog_Item* pxItem)
{
	if(IsEntityTreeItem(pxItem))
	{
		//entities show whether they are hidden/visible
		Tools_Entity_Object* pxObject = GetToolsEntityObjectFromTreeItem(pxItem);
		if(pxObject && !pxObject->IsHidden())
		{
			return ASURA_TREEDIALOG_ICON_VISIBLE;
		}
		else
		{
			return ASURA_TREEDIALOG_ICON_HIDDEN;
		}
	}
	else if(IsFolderTreeItem(pxItem))
	{
		//folders show open/closed state 
		if(pxItem->IsExpanded() && pxItem->GetNumChildren())
		{
			return ASURA_TREEDIALOG_ICON_OPENEDFOLDER;
		}
		else
		{
			return ASURA_TREEDIALOG_ICON_CLOSEDFOLDER;
		}
	}
	else if(IsFileTreeItem(pxItem))
	{
		//files show whether they are writeable
		if(CanSaveFileCached(GetFileFromTreeItem(pxItem)))
		{
			return ASURA_TREEDIALOG_ICON_WRITEABLEFILE;
		}
		else
		{
			return ASURA_TREEDIALOG_ICON_READONLYFILE;
		}
	}
	return -1;
}

#ifndef TOOLS_USE_OLD_ENTITY_MANAGER
int Tools_SecondaryEntityFile_System::GetAdditionalItemIconIndex(Asura_TreeDialog_Item* pxItem, u_int uIconIndex)
{
	if (IsFileTreeItem (pxItem))
	{
		switch (uIconIndex)
		{
			case 0:
			{
				bool bShouldHide = RecursiveCheckShouldHide(pxItem, true);
				return bShouldHide ? ASURA_TREEDIALOG_ICON_VISIBLE : ASURA_TREEDIALOG_ICON_HIDDEN;
			}
			case 1:
			{
				return (!CanSaveFileCached (GetFileFromTreeItem (pxItem))) ? ASURA_TREEDIALOG_ICON_PADLOCK : -1;
			}
		}
	}
	else if(IsFolderTreeItem(pxItem))
	{
		if (uIconIndex == 0)
		{
			bool bShouldHide = RecursiveCheckShouldHide(pxItem, true);
			return bShouldHide ? ASURA_TREEDIALOG_ICON_VISIBLE : ASURA_TREEDIALOG_ICON_HIDDEN;
		}
	}
	else
	{
		if (uIconIndex == 0)
		{
			if (!CanSaveFileCached (GetFileFromTreeItem (pxItem)))
			{
				return ASURA_TREEDIALOG_ICON_PADLOCK;
			}
		}
	}

	return -1;
}
#endif

void Tools_SecondaryEntityFile_System::PreCreateObject()
{
	if(g_iCreateObjectRecursion == 0)
	{
		g_iNextToolsID = Tools_Object::GetNextToolsID();
	}
	g_iCreateObjectRecursion++;
}

void Tools_SecondaryEntityFile_System::PostCreateObject()
{
	Asura_Assert(g_iCreateObjectRecursion>0,"Not called PreCreateObject()?");
	if(g_iCreateObjectRecursion<=0) return;

	g_iCreateObjectRecursion--;

	//Deal with transferring the entities once any nested calls to PreCreateObject() have been matched with 
	//calls to PostCreateObject()
	if(g_iCreateObjectRecursion == 0)
	{
		//If currently editing a secondary entity file then find the object(s)
		//that have just been created and assign them to the file

		//Also deal with assigning default light sets to entities while we are at it.
		//Doesn't belong here as such , but there isn't any convenient place to catch
		//it elsewhere.
		for(int iID=g_iNextToolsID;iID<Tools_Object::GetNextToolsID();iID++)
		{
			Tools_Object* pxObject = Tools_Object_System::GetObjectFromToolsID(iID);
			if(pxObject)
			{
				if(IsEditingSecondaryFile())
				{
					AddToCurrentFile(pxObject);
				}
				if(pxObject->IsEntityObject())
				{
					WarnIfCurrentFileIsReadOnly();
				
					Tools_Entity_Object* pxEntityObject = (Tools_Entity_Object*) pxObject;
					pxEntityObject->SetLightSets(Tools_LightSets::GetLightSetsForNewObject(pxEntityObject->Position));

					UpdateEntityInTree(pxEntityObject, true);

				}
			}
		}
	}
	
}

void Tools_SecondaryEntityFile_System::SetSelectedFile(Tools_SecondaryEntityFile* pxFile)
{
	Asura_Assert(pxFile, "");
	if(!pxFile) pxFile = GetMainFile();

	s_pxCurrentFile = pxFile;

	//search the top level items in the tree for a matching entry
	const char* szName = pxFile->GetDisplayName();

	Asura_TreeDialog_Item* pxRoot = s_xTree.GetRoot();
	Asura_Assert(pxRoot,"");

	for(Asura_TreeDialog_Item* pxItem = pxRoot->GetChild(); pxItem; pxItem = pxItem->GetSibling())
	{
		if(!stricmp(szName,pxItem->GetName()))
		{
			s_xTree.SetSelectedItem(pxItem);
			break;
		}
	}

}

Tools_SecondaryEntityFile* Tools_SecondaryEntityFile_System::GetFile(const char* szName)
{
	if(!szName) szName = "";

	for(LIF<Tools_SecondaryEntityFile*> xFileLIF(&s_xFileList); !xFileLIF.done(); xFileLIF.next())
	{
		if(!stricmp(xFileLIF()->m_xName.GetString(), szName))
		{
			return xFileLIF();
		}
	}
	return 0;
}

const char* Tools_SecondaryEntityFile_System::GetMainFileLabel()
{
	if(Tools_SaveAsura::IsEditingMission())
		return "Mission";
	else
		return "Main";
}

int Tools_SecondaryEntityFile_System::GetNumSecondaryFiles()
{
	return s_xFileList.size()-1;
}

const char* Tools_SecondaryEntityFile_System::GetSecondaryFileName(int iIndex)
{
	Asura_Assert(iIndex>=0 && iIndex<s_xFileList.size()-1,"");
	return s_xFileList[iIndex+1]->m_xName.GetString();
}

Tools_SecondaryEntityFile* Tools_SecondaryEntityFile_System::GetFileFromTreeItem(const Asura_TreeDialog_Item* pxItem)
{
	if(!pxItem || pxItem->GetDepth()<1) return 0;
		
	//go up the tree to the file item
	while(pxItem->GetDepth() > 1)
	{
		pxItem = pxItem->GetParent();
	}

	int iID = pxItem->GetUserData();


	for(LIF<Tools_SecondaryEntityFile*> xFileLIF(&s_xFileList); !xFileLIF.done(); xFileLIF.next())
	{
		if(xFileLIF()->m_iFileID == iID)
		{
			return xFileLIF();
		}
	}
	return 0;
}

const char* Tools_SecondaryEntityFile_System::GetFileDisplayNameFromTreeItem(const Asura_TreeDialog_Item* pxItem)
{
	if(!pxItem || pxItem->GetDepth()<1) return "";
		
	//go up the tree to the file item
	while(pxItem->GetDepth() > 1)
	{
		pxItem = pxItem->GetParent();
	}
	return pxItem->GetName();
}


void Tools_SecondaryEntityFile_System::GetFolderPathFromTreeItem(const Asura_TreeDialog_Item* pxItem , Asura_String& xOutput)
{
	if(!pxItem || pxItem->GetDepth() < 2)
	{
		xOutput.MakeNull();
		return;
	}

	//get an array of all the tree items in the path
	const Asura_TreeDialog_Item** ppxPath = new const Asura_TreeDialog_Item*[pxItem->GetDepth()];
	
	int iPathLength = 0;
	int iNameLength = 1;
	for(const Asura_TreeDialog_Item* pxItemIt = pxItem; pxItemIt->GetDepth() > 1; pxItemIt = pxItemIt->GetParent())
	{
		//just interested in entries that correspond to folders
		if(pxItemIt->GetUserData() == g_iFolderTreeItemID)
		{
			ppxPath[iPathLength++] = pxItemIt;
			iNameLength += 1 + strlen(pxItemIt->GetName());
		}
	}

	if(!iPathLength)
	{
		delete [] ppxPath;
		xOutput.MakeNull();
		return;
	}

	//allocate a buffer for the name , and stick it together
	char* acBuffer = new char[iNameLength];
	char* pcPos = acBuffer;

	for(int i=iPathLength-1; i>=0; i--)
	{
		strcpy(pcPos, ppxPath[i]->GetName());
		pcPos += strlen(pcPos);
			
		if(i)
		{
			*pcPos++ = '\\';
		}
	}

	xOutput = acBuffer;
	delete [] acBuffer;
	delete [] ppxPath;
}

Asura_Guid Tools_SecondaryEntityFile_System::GetGuidFromTreeItem(const Asura_TreeDialog_Item* pxItem)
{
	if(!pxItem || pxItem->GetDepth() < 2) return ASURA_GUID_UNREGISTERED;
	if(pxItem->GetUserData() == g_iFolderTreeItemID) return ASURA_GUID_UNREGISTERED;
	return pxItem->GetUserData();
}

Tools_Entity_Object*  Tools_SecondaryEntityFile_System::GetToolsEntityObjectFromTreeItem(const Asura_TreeDialog_Item* pxItem)
{
	Asura_Guid uGuid = GetGuidFromTreeItem(pxItem);
	if(uGuid == ASURA_GUID_UNREGISTERED) return 0;

	Tools_Object* pxObject = Tools_Object_System::GetObjectFromGuid(uGuid);
	if(!pxObject || !pxObject->IsEntityObject()) return 0;

	return (Tools_Entity_Object*) pxObject;
}

bool Tools_SecondaryEntityFile_System::IsFileTreeItem(const Asura_TreeDialog_Item* pxItem)
{
	if(!pxItem) return false;
	return pxItem->GetDepth() == 1;
}

bool Tools_SecondaryEntityFile_System::IsFolderTreeItem(const Asura_TreeDialog_Item* pxItem)
{
	if(!pxItem || pxItem->GetDepth() < 2) return false;
	return pxItem->GetUserData() == g_iFolderTreeItemID;
}

bool Tools_SecondaryEntityFile_System::IsEntityTreeItem(const Asura_TreeDialog_Item* pxItem)
{
	if(!pxItem || pxItem->GetDepth() < 2) return false;
	return pxItem->GetUserData() != g_iFolderTreeItemID;
}


void Tools_SecondaryEntityFile_System::CreateFolder()
{
	Asura_TreeDialog_Item* pxSelectedItem = s_xTree.GetSelectedItem();
	if(!pxSelectedItem) return;

	Asura_TreeDialog_Item* pxFolderParent = pxSelectedItem;
	//search up from the selected item to find something that can have a folder as a child	
	while(pxFolderParent->GetDepth() > 1 && pxFolderParent->GetUserData() != g_iFolderTreeItemID)
	{
		pxFolderParent = pxFolderParent->GetParent();		
	}

	//get a name , and make sure its okay
	GetString_Overlay xOverlay("Folder name:", "");
	if(!xOverlay.DoModal()) return;

	const char* szName = xOverlay.GetString();
	if(!szName || !*szName) return;

	//make sure this name isn't already used as a child
	if(pxFolderParent->FindChildByName(szName, true)) return;

	//add to the tree
	Asura_TreeDialog_Item* pxFolderItem = s_xTree.InsertItem( CompareTreeItem, pxFolderParent, szName, g_iFolderTreeItemID);
 	pxFolderItem->SetColourOverride(GetItemColour(pxFolderItem));
	pxFolderItem->Expand(true);
}

void Tools_SecondaryEntityFile_System::CollapseAllTreeBranches()
{
	CollapseAllTreeBranchesRecursive(s_xTree.GetRoot());

	s_pxTreeDialog->MakeSelectionVisible();
}

void Tools_SecondaryEntityFile_System::CollapseAllTreeBranchesRecursive(Asura_TreeDialog_Item* pxItem)
{
	if(!pxItem->IsRoot())
	{
		pxItem->Expand(false);
	}

	//recurse down the tree
	for(Asura_TreeDialog_Item* pxChild = pxItem->GetChild(); pxChild; pxChild = pxChild->GetSibling())
	{
		CollapseAllTreeBranchesRecursive(pxChild);
	}
}

void Tools_SecondaryEntityFile_System::DragAndDrop(Asura_TreeDialog_Item* pxDropTarget)
{

	//take a copy of the selection , since it may change as a result of fiddling with the tree
	List<Asura_TreeDialog_Item*> xSelection = s_xTree.GetSelection();

	//only only dragging to a file or folder
	if(!IsFileTreeItem(pxDropTarget) && !IsFolderTreeItem(pxDropTarget))
	{
		Tools_Timed_Message_System::AddMessage(0xff0000,"Can only drag onto a file or folder");
		return;
	}

	Tools_SecondaryEntityFile* pxTargetFile = GetFileFromTreeItem(pxDropTarget);
	Asura_Assert(pxTargetFile,"");

	Asura_String xTargetPath;
	GetFolderPathFromTreeItem(pxDropTarget, xTargetPath);

	if(!pxTargetFile->CanSaveFile(false))
	{
		Tools_Timed_Message_System::AddMessage(0xff0000,"Target file for drag isn't writeable");
		return;
	}

	//make sure the target isn't a child of any part of the selection
	for(LIF<Asura_TreeDialog_Item*> xSelectionLIF(&xSelection); !xSelectionLIF.done(); xSelectionLIF.next())
	{
		if(pxDropTarget->IsDescendentOf(xSelectionLIF()))
		{
			Tools_Timed_Message_System::AddMessage(0xff0000,"Target of drag operation is part of the selection (or a child of a selected folder)");
			return;
		}
	}

	//If both a folder and some of its children are selected , then discard the child selections , since they
	//will be moved with the parent.
	RemoveChildrenFromList(xSelection);

	List<Asura_TreeDialog_Item*> xNewSelection;

	//deal with moving each selected item to the target file/path
	for(LIF<Asura_TreeDialog_Item*> xSelectionLIF(&xSelection); !xSelectionLIF.done(); xSelectionLIF.next())
	{
		Asura_TreeDialog_Item* pxItem = xSelectionLIF();
		
		if(IsEntityTreeItem(pxItem))
		{
			if(TransferEntity(pxItem, pxTargetFile, xTargetPath.GetString()))
			{
				//update the position in the tree
				s_xTree.MoveItem(CompareTreeItem, pxItem, pxDropTarget);
				pxItem->SetColourOverride(GetItemColour(pxItem));
			}
			xNewSelection.add_entry(pxItem);
		}
		else 
		{
			Asura_TreeDialog_Item* pxTransferredItem = TransferFolder(pxItem, pxTargetFile, xTargetPath.GetString());
			xNewSelection.add_entry(pxTransferredItem ? pxTransferredItem : pxItem);
		}
	}

	s_xTree.SetSelection(xNewSelection);
}

Asura_TreeDialog_Item* Tools_SecondaryEntityFile_System::TransferFolder(Asura_TreeDialog_Item* pxFolderItem, Tools_SecondaryEntityFile* pxDestFile, const char* szFolder)
{
	//get a list of all the children , so it can be used for iteration without worrying about it changing half-way
	List<Asura_TreeDialog_Item*> xChildList;
	for(Asura_TreeDialog_Item* pxChild = pxFolderItem->GetChild(); pxChild; pxChild = pxChild->GetSibling())
	{
		xChildList.add_entry(pxChild);
	}

	//If this is a folder (rather than a file) then add the name to the path for the children
	Asura_String xChildPath = szFolder;
	if(IsFolderTreeItem(pxFolderItem))
	{
		xChildPath += "\\";
		xChildPath += pxFolderItem->GetName();
	}

	Asura_TreeDialog_Item* pxCopiedFolder = 0;
	
	//transfer each child
	for(LIF<Asura_TreeDialog_Item*> xChildLIF(&xChildList); !xChildLIF.done(); xChildLIF.next())
	{
		Asura_TreeDialog_Item* pxItem = xChildLIF();
		if(IsEntityTreeItem(pxItem))
		{
			if(TransferEntity(pxItem, pxDestFile, xChildPath.GetString()))
			{
				if(!pxCopiedFolder) pxCopiedFolder = GetFileOrFolderFromTree(pxDestFile, xChildPath.GetString(), true);
				//update the position in the tree
				s_xTree.MoveItem(CompareTreeItem, pxItem, pxCopiedFolder);
				pxItem->SetColourOverride(GetItemColour(pxItem));
			}
		}
		else
		{
			TransferFolder(pxItem, pxDestFile, xChildPath.GetString());
		}
	}

	//Everything transferred?
	if(!pxFolderItem->GetChild())
	{
		//Make sure the destination folder has been created
		if(!pxCopiedFolder) pxCopiedFolder = GetFileOrFolderFromTree(pxDestFile, xChildPath.GetString(), true);

		//and remove this unless its a file
		if(pxFolderItem != pxCopiedFolder)
		{
			if(IsFolderTreeItem(pxFolderItem))
			{
				s_xTree.DeleteItem(pxFolderItem);
			}
		}
	}
	return pxCopiedFolder;
}

void Tools_SecondaryEntityFile_System::IconClicked(Asura_TreeDialog_Item* pxItem, bool bDrag)
{
	static enum
	{
		ICONCLICK_STATE_NONE,
		ICONCLICK_STATE_HIDE,
		ICONCLICK_STATE_UNHIDE,
	} s_eState;

	if(bDrag)
	{
		//continue hiding/unhiding depending on what was done for the first item clicked on
		if(s_eState != ICONCLICK_STATE_NONE)
		{
			Tools_Entity_Object* pxObject =  GetToolsEntityObjectFromTreeItem(pxItem);
			if(pxObject)
			{
				pxObject->SetHiddenFlagIncludingChildren(ObjectHideFlag_User, s_eState == ICONCLICK_STATE_HIDE);

				#ifndef TOOLS_USE_OLD_ENTITY_MANAGER
				pxItem->SetColourOverride ((s_eState == ICONCLICK_STATE_HIDE) ? TOOLS_SECONDARY_ENTITY_HIDDEN_TREEITEM_COLOUR : GetItemColour (pxItem));
				#endif
			}
		}
	}
	else
	{
		s_eState = ICONCLICK_STATE_NONE;
		if(IsEntityTreeItem(pxItem))
		{
			//clicked on an entity so hide/unhide according to its current hiddenness
			Tools_Entity_Object* pxObject = GetToolsEntityObjectFromTreeItem(pxItem);
			if(pxObject)
			{
				if(pxObject->IsHiddenFlagSet(ObjectHideFlag_User))
				{
					pxObject->SetHiddenFlagIncludingChildren(ObjectHideFlag_User, false);
					s_eState = ICONCLICK_STATE_UNHIDE;

					#ifndef TOOLS_USE_OLD_ENTITY_MANAGER
					pxItem->SetColourOverride (GetItemColour (pxItem));
					#endif
				}
				else
				{
					pxObject->SetHiddenFlagIncludingChildren(ObjectHideFlag_User, true);
					s_eState = ICONCLICK_STATE_HIDE;

					#ifndef TOOLS_USE_OLD_ENTITY_MANAGER
					pxItem->SetColourOverride (TOOLS_SECONDARY_ENTITY_HIDDEN_TREEITEM_COLOUR);
					#endif
				}

			}

		}
		else
		{
			//clicked on something else , so just expand/unexpand it
			pxItem->Expand(!pxItem->IsExpanded());
		}
	}
}

#ifndef TOOLS_USE_OLD_ENTITY_MANAGER
void Tools_SecondaryEntityFile_System::AdditionalIconClicked(Asura_TreeDialog_Item* pxItem, bool bDrag)
{
	if(IsFileTreeItem(pxItem) || IsFolderTreeItem(pxItem))
	{
		if(!bDrag)
		{
			// Iterate through all the children and decide whether we need to hide or unhide the hierarchy.  If anything
			// is hidden, then we're unhiding everything.  If nothing is hidden, then we're hiding everything.

			bool bShouldHide = RecursiveCheckShouldHide(pxItem, true);

			HideRecursive (pxItem, bShouldHide);
		}
	}
}
#endif

#ifndef TOOLS_USE_OLD_ENTITY_MANAGER
bool Tools_SecondaryEntityFile_System::RecursiveCheckShouldHide(Asura_TreeDialog_Item* pxItem, bool bCurrentFlag)
{
	Asura_TreeDialog_Item* pxChild = pxItem->GetChild();

	while(pxChild)
	{
		if(!RecursiveCheckShouldHide(pxChild, bCurrentFlag))
		{
			return false;
		}

		pxChild = pxChild->GetSibling();
	}

	Tools_Entity_Object* pxObject = GetToolsEntityObjectFromTreeItem(pxItem);

	if(pxObject && pxObject->IsHiddenFlagSet(ObjectHideFlag_User))
	{
		return false;
	}

	return true;
}
#endif

void Tools_SecondaryEntityFile_System::RenameFolder(Asura_TreeDialog_Item* pxFolderItem, const char* szNewName)
{
	Asura_Assert(pxFolderItem && IsFolderTreeItem(pxFolderItem), "");

	//make sure there isn't a different sibling with this name
	Asura_TreeDialog_Item* pxParent = pxFolderItem->GetParent();

	Asura_TreeDialog_Item* pxSibling = pxParent->FindChildByName(szNewName, true);
	if(pxSibling && pxSibling != pxFolderItem)
	{
		Tools_Timed_Message_System::AddMessage(0xff0000,"A folder with that name already exists.");
		return;
	}
	
	//get the current path for the folder
	Asura_String xOldPath;
	GetFolderPathFromTreeItem(pxFolderItem, xOldPath);
	int iOldPathLength = xOldPath.GetLength();

	//determine the new path for the folder
	char acNewPath[512];
	strcpy(acNewPath, xOldPath.GetString());
	strcpy(&acNewPath[iOldPathLength-strlen(pxFolderItem->GetName())], szNewName);
	int iNewPathLength = strlen(acNewPath);


	//and the file
	Tools_SecondaryEntityFile* pxFile = GetFileFromTreeItem(pxFolderItem);
	Asura_Assert(pxFile, "");



	///update all the entities using the old path
	for(LIF<Tools_Object*> xObjectLIF(&Tools_Object_System::ObjectList);!xObjectLIF.done();xObjectLIF.next())
	{
		//only interested in entities.
		if(!xObjectLIF()->IsEntityObject()) continue;
		Tools_Entity_Object* pxEntityObject = (Tools_Entity_Object*) xObjectLIF();

		Tools_Entity_Data* pxToolsData = pxEntityObject->Entity->ToolsData;

		//ignore entities that are in the wrong file
		if(pxToolsData->m_xSecondaryEntityFilename.IsBlank())
		{
			if(!pxFile->IsMainFile()) continue;
		}
		else
		{
			if(pxFile->IsMainFile()) continue;
			if(stricmp(pxFile->m_xName.GetString(), pxToolsData->m_xSecondaryEntityFilename.GetString())) continue;
		}

		if(pxToolsData->m_xEntityBrowserPath.IsBlank()) continue;
		const char* szEntityPath = pxToolsData->m_xEntityBrowserPath.GetString();

		//is the entity in the folder being renamed?
		if(strnicmp(xOldPath.GetString(), szEntityPath, iOldPathLength)) continue;
		if(szEntityPath[iOldPathLength] != 0 && szEntityPath[iOldPathLength] != '\\') continue;

		//copy the end of the path onto the new path
		strcpy(&acNewPath[iNewPathLength], &szEntityPath[iOldPathLength]);

		//and update the entity
		pxToolsData->m_xEntityBrowserPath = acNewPath;
		pxEntityObject->SetEntityModified();
		
	}

	//update the folder item in the tree
	pxFolderItem->SetName(szNewName);
	s_xTree.MoveItem(CompareTreeItem, pxFolderItem, pxParent);

}




void Tools_SecondaryEntityFile_System::AddContextMenuItems(class Tools_ContextMenu& xMenu, const class Tools_ContextMenu_Context& xContext)
{
	Asura_TreeDialog_Item* pxTreeItem = s_xTree.GetSelectedItem();
	Tools_Entity_Object* pxEntityObject = GetToolsEntityObjectFromTreeItem(pxTreeItem);	

	Tools_ContextMenu_Section* pxSection = xMenu.GetSection(TOOLS_CONTEXTMENU_SECTION_SELECTION);
	Tools_ContextMenu_Item* pxItem;

	//Many of the editing options are disabled during choose object to be on the safe side. RAR 14/01/2009
	const bool bDoingChooseObject = Tools_Object_System::IsChoosingObject();


	pxItem = new Tools_ContextMenu_Item_Function("Select all", ContextMenu_SelectAll, pxTreeItem);
	pxItem->SetHelpText("Select all entities in the tree.");
	pxSection->AddItem(pxItem);

	pxItem = new Tools_ContextMenu_Item_Function("Select none", ContextMenu_SelectNone);
	pxItem->SetHelpText("Deselect everything in the tree.");
	pxSection->AddItem(pxItem);

	pxItem = new Tools_ContextMenu_Item_Function("Invert selection", ContextMenu_InvertSelection);
	pxItem->SetHelpText("Inverts the selection of all entities in the tree.");
	pxSection->AddItem(pxItem);

	pxSection->AddSeparator();

	if(IsEntityTreeItem(pxTreeItem))
	{
		pxItem = new Tools_ContextMenu_Item_Function("Select type", ContextMenu_SelectType, pxTreeItem);
		pxItem->SetHelpText("Select all entities of the same type as the one clicked on.");		
		pxSection->AddItem(pxItem);

		pxItem = new Tools_ContextMenu_Item_Function("Deselect type", ContextMenu_DeselectType, pxTreeItem);
		pxItem->SetHelpText("Deselect all entities of the same type as the one clicked on.");		
		pxSection->AddItem(pxItem);
	}

	pxItem = new Tools_ContextMenu_Item_Function("Select visible", ContextMenu_SelectVisible);
	pxItem->SetHelpText("Select all entities that are visible.");		
	pxSection->AddItem(pxItem);

	pxItem = new Tools_ContextMenu_Item_Function("Deselect visible", ContextMenu_DeselectVisible);
	pxItem->SetHelpText("Deselect all entities that are visible.");		
	pxSection->AddItem(pxItem);

	//use the 'display' menu for a tenuously named 'edit' for now
	pxSection = xMenu.GetSection(TOOLS_CONTEXTMENU_SECTION_DISPLAY);
	pxSection->SetHeading("Edit");

	//copy/paste
	if(!bDoingChooseObject)
	{
		pxItem = new Tools_ContextMenu_Item_Function("Copy", ContextMenu_Copy, pxTreeItem);
		pxItem->SetEnabled(pxEntityObject && pxEntityObject->CanBeCopied());
		pxSection->AddItem(pxItem);

		pxItem = new Tools_ContextMenu_Item_Function("Paste", ContextMenu_Paste, pxTreeItem);
		pxItem->SetEnabled(Tools_Object_System::CanPaste());
		pxSection->AddItem(pxItem);
	}

//	pxItem = new Tools_ContextMenu_Item_Function("Paste over", ContextMenu_PasteOver, pxTreeItem);
//	pxSection->AddItem(pxItem);

	pxSection->AddSeparator();

	pxItem = new Tools_ContextMenu_Item_Function("Goto", ContextMenu_Goto, pxTreeItem);
	pxItem->SetHelpText("Moves the camera to the entity clicked on");
	pxItem->SetEnabled(IsEntityTreeItem(pxTreeItem));
	pxSection->AddItem(pxItem);
	
	
	pxItem = new Tools_ContextMenu_Item_Function("Hide", ContextMenu_Hide);
	pxItem->SetHelpText("Hides the tools objects corresponding to the selected items in the tree.");
	pxSection->AddItem(pxItem);

	pxItem = new Tools_ContextMenu_Item_Function("Unhide", ContextMenu_Unhide);
	pxSection->AddItem(pxItem);
	pxItem->SetHelpText("Unhides the tools objects corresponding to the selected items in the tree. If they are hidden by other systems , then they may still remain hidden after doing this.");

	pxSection->AddSeparator();

	bool bCanRename = true;
	if(IsFileTreeItem(pxTreeItem))
	{
		//disable renaming if the main file was clicked on
		Tools_SecondaryEntityFile* pxFile = GetFileFromTreeItem(pxTreeItem);
		Asura_Assert(pxFile, "");
		bCanRename = !pxFile->IsMainFile();
	}
	
	if(!bDoingChooseObject)
	{
		pxItem = new Tools_ContextMenu_Item_Function("Create", ContextMenu_Create);
		pxItem->SetHelpText("Create an entity/object of the type currently selected in the object mode overlay");
		pxItem->SetEnabled(Tools_System::GetCurrentMode () == TOOLS_MODE_OBJECT && Tools_Mode_Object::ObjectModeOverlayIsOn());
		pxSection->AddItem(pxItem);

		pxItem = new Tools_ContextMenu_Item_Function("Rename", ContextMenu_Rename, pxTreeItem);
		pxItem->SetEnabled(bCanRename);
		pxSection->AddItem(pxItem);

		pxItem = new Tools_ContextMenu_Item_Function("Delete", ContextMenu_Delete);
		pxItem->SetHelpText("Delete the selected files,folders and entities");
		pxSection->AddItem(pxItem);
	}


	if(!bDoingChooseObject)
	{
		//use the 'object' menu for file/folder options
		pxSection = xMenu.GetSection(TOOLS_CONTEXTMENU_SECTION_OBJECT);
		pxSection->SetHeading("File");

		pxItem = new Tools_ContextMenu_Item_Function("Create Folder", CreateFolder);
		pxItem->SetHelpText("Create a folder for organising the entities within tools.");
		pxSection->AddItem(pxItem);

		pxSection->AddSeparator();

		pxItem = new Tools_ContextMenu_Item_Function("Create File", CreateEntityFile);
		pxItem->SetHelpText("Create a new secondary entity file.\nThe main entity file must be checked out to do this.");
		pxSection->AddItem(pxItem);

		pxSection->AddSeparator();

		pxItem = new Tools_ContextMenu_Item_Function("Create Common File", CreateCommonEntityFile);
		pxItem->SetHelpText("Create a common secondary entity file , which is intended for sharing between multiple main entity files , but has more restrictions on how it can be used.\nMainly intended for use with the mission file system.");
		pxSection->AddItem(pxItem);

		pxSection->AddSeparator();

		pxItem = new Tools_ContextMenu_Item_Function("Add File", AddEntityFile);
		pxItem->SetHelpText("Add an existing secondary entity file.\nThe main entity file must be checked out to do this.");
		pxSection->AddItem(pxItem);

		pxSection->AddSeparator();


		pxItem = new Tools_ContextMenu_Item_Function("Transfer Entities", TransferSelectedObjects);
		pxItem->SetHelpText("Transfers the entities that are selected in the world to the current entity file.\nBoth the source and destination files must be checked out to do this.");
		pxSection->AddItem(pxItem);

		if(IsGroupSelected())
		{
			pxItem = new Tools_ContextMenu_Item_Function("Merge File", MergeEntityFile);
			pxItem->SetHelpText("Merges the contents of the current entity file back into the main entity file.\nBoth files need to be checked out to do this.");
			pxSection->AddItem(pxItem);
		}

		pxSection->AddSeparator();

		if(IsGroupSelected())
		{
			pxItem = new Tools_ContextMenu_Item_Function("Rename File", RenameEntityFile);
			pxItem->SetHelpText("Renames the secondary file and updates the existing entities so that they refer to it instead.");
			pxItem->SetEnabled(IsGroupSelected());
			pxSection->AddItem(pxItem);

			pxItem = new Tools_ContextMenu_Item_Function("Replace File", ReplaceEntityFile);
			pxItem->SetHelpText("Replaces the secondary file with another existing file and updates from that file.\nMainly for dealing with shared files that have already been renamed by another mission.");
			pxItem->SetEnabled(IsGroupSelected());
			pxSection->AddItem(pxItem);

			pxSection->AddSeparator();

			pxItem = new Tools_ContextMenu_Item_Function("Delete File", DeleteEntityFile);
			pxItem->SetHelpText("Deletes the secondary file and all entities within it.\nBoth files need to be checked out to do this.");
			pxItem->SetEnabled(IsGroupSelected());
			pxSection->AddItem(pxItem);

			pxItem = new Tools_ContextMenu_Item_Function("Remove File", RemoveEntityFile);
			pxItem->SetHelpText("Removes the secondary file and deletes the entities from the level.\nThe main asura file needs to be checked out to do this.");
			pxItem->SetEnabled(IsGroupSelected());
			pxSection->AddItem(pxItem);

			pxSection->AddSeparator();
		}
	}
}


void Tools_SecondaryEntityFile_System::ContextMenu_SelectAll(void* pClickedItem)
{
	Asura_TreeDialog_Item* pxClickedItem = (Asura_TreeDialog_Item*) pClickedItem;

	//empty the selection , then recurse down the tree selecting all entity items
	s_xTree.ClearSelection();

	Asura_TreeDialog_Item* pxItem = s_xTree.GetRoot();
	SelectAllEntityRecursive(pxItem);

	//If an item was right clicked on , then set that as the main selection
	if(pxClickedItem)
	{
		s_xTree.AddToSelection(pxClickedItem);
	}

	UpdateObjectSelectionFromTree();
}


void Tools_SecondaryEntityFile_System::ContextMenu_SelectNone()
{
	s_xTree.ClearSelection();
	Tools_Object_System::DeselectAllObjects();
}

void Tools_SecondaryEntityFile_System::ContextMenu_InvertSelection()
{
	
	Asura_TreeDialog_Item* pxItem = s_xTree.GetRoot();
	InvertEntitySelectionRecursive(pxItem);

	UpdateObjectSelectionFromTree();
}

void Tools_SecondaryEntityFile_System::ContextMenu_SelectType(void* pClickedItem)
{
	//find out the type of entity being selected
	Asura_TreeDialog_Item* pxClickedItem = (Asura_TreeDialog_Item*) pClickedItem;
	Tools_Entity_Object* pxEntityObject = GetToolsEntityObjectFromTreeItem(pxClickedItem);
	if(!pxEntityObject) return;

	int iEntityClass = pxEntityObject->GetEntityClass();

	//loop over all the entities and select the tree items for entities with the correct class
	for(Asura_Hashed_List_It<Tools_Entity_Object> xIt(Tools_Entity_Object::GetList());!xIt.Done();xIt.Next())
	{
		Tools_Entity_Object* pxEntityObject2 = xIt.GetCurrent();
		if(pxEntityObject2->GetEntityClass() == iEntityClass)
		{
			Asura_TreeDialog_Item* pxItem = s_xTree.FindByData(pxEntityObject2->GetGuid());
			if(pxItem && !pxItem->IsHidden())
			{
				s_xTree.AddToSelection(pxItem);
			}
		}
	}
	//re-select the clicked item to make it the primary selection
	s_xTree.AddToSelection(pxClickedItem);

	UpdateObjectSelectionFromTree();
}

void Tools_SecondaryEntityFile_System::ContextMenu_DeselectType(void* pClickedItem)
{
	//find out the type of entity being deselected
	Asura_TreeDialog_Item* pxClickedItem = (Asura_TreeDialog_Item*) pClickedItem;
	Tools_Entity_Object* pxEntityObject = GetToolsEntityObjectFromTreeItem(pxClickedItem);
	if(!pxEntityObject) return;

	int iEntityClass = pxEntityObject->GetEntityClass();

	//loop over all the entities and deselect the tree items for entities with the correct class
	for(Asura_Hashed_List_It<Tools_Entity_Object> xIt(Tools_Entity_Object::GetList());!xIt.Done();xIt.Next())
	{
		Tools_Entity_Object* pxEntityObject2 = xIt.GetCurrent();
		if(pxEntityObject2->GetEntityClass() == iEntityClass)
		{
			Asura_TreeDialog_Item* pxItem = s_xTree.FindByData(pxEntityObject2->GetGuid());
			if(pxItem)
			{
				s_xTree.RemoveFromSelection(pxItem);
			}
		}
	}

	UpdateObjectSelectionFromTree();
}

void Tools_SecondaryEntityFile_System::ContextMenu_SelectVisible()
{

	//loop over all the entities and select the tree items for entities which aren't hidden
	for(Asura_Hashed_List_It<Tools_Entity_Object> xIt(Tools_Entity_Object::GetList());!xIt.Done();xIt.Next())
	{
		Tools_Entity_Object* pxEntityObject = xIt.GetCurrent();
		if(!pxEntityObject->IsHidden())
		{
			Asura_TreeDialog_Item* pxItem = s_xTree.FindByData(pxEntityObject->GetGuid());
			if(pxItem && !pxItem->IsHidden())
			{
				s_xTree.AddToSelection(pxItem);
			}
		}
	}

	UpdateObjectSelectionFromTree();
}

void Tools_SecondaryEntityFile_System::ContextMenu_DeselectVisible()
{

	//loop over all the entities and deselect the tree items for entities which aren't hidden
	for(Asura_Hashed_List_It<Tools_Entity_Object> xIt(Tools_Entity_Object::GetList());!xIt.Done();xIt.Next())
	{
		Tools_Entity_Object* pxEntityObject = xIt.GetCurrent();
		if(!pxEntityObject->IsHidden())
		{
			Asura_TreeDialog_Item* pxItem = s_xTree.FindByData(pxEntityObject->GetGuid());
			if(pxItem)
			{
				s_xTree.RemoveFromSelection(pxItem);
			}
		}
	}

	UpdateObjectSelectionFromTree();
}


void Tools_SecondaryEntityFile_System::SelectAllEntityRecursive(Asura_TreeDialog_Item* pxItem)
{

	//loop over the siblings
	for(; pxItem; pxItem = pxItem->GetSibling())
	{
		//select if an entity
		if(IsEntityTreeItem(pxItem) && !pxItem->IsHidden())
		{
			s_xTree.AddToSelection(pxItem);
		}

		//recurse over children
		if(pxItem->GetChild())
		{
			SelectAllEntityRecursive(pxItem->GetChild());
		}
		
	}
}

void Tools_SecondaryEntityFile_System::InvertEntitySelectionRecursive(Asura_TreeDialog_Item* pxItem)
{

	//loop over the siblings
	for(; pxItem; pxItem = pxItem->GetSibling())
	{
		//toggle selection if an entity
		if(IsEntityTreeItem(pxItem))
		{
			if(pxItem->IsHidden())
			{
				s_xTree.RemoveFromSelection(pxItem);
			}
			else
			{
				s_xTree.ToggleSelection(pxItem);
			}
		}

		//recurse over children
		if(pxItem->GetChild())
		{
			InvertEntitySelectionRecursive(pxItem->GetChild());
		}
		
	}
}

void Tools_SecondaryEntityFile_System::ContextMenu_Copy(void* pClickedItem)
{
	Asura_TreeDialog_Item* pxClickedItem = (Asura_TreeDialog_Item*) pClickedItem;
	Tools_Entity_Object* pxEntityObject = GetToolsEntityObjectFromTreeItem(pxClickedItem);
	if(!pxEntityObject) return;

	Tools_Object_System::CopyToClipboard(pxEntityObject);
}

void Tools_SecondaryEntityFile_System::ContextMenu_Paste(void* pClickedItem)
{
	Tools_Object_System::Paste();
}


void Tools_SecondaryEntityFile_System::ContextMenu_Rename(void* pClickedItem)
{
	Asura_TreeDialog_Item* pxClickedItem = (Asura_TreeDialog_Item*) pClickedItem;
	if(pxClickedItem)
	{
		Asura_Assert(s_pxTreeDialog, "");
		s_pxTreeDialog->StartRename(pxClickedItem);
	}
}

void Tools_SecondaryEntityFile_System::ContextMenu_Hide()
{
	const List<Asura_TreeDialog_Item*>& xSelection = s_xTree.GetSelection();

	for(CLIF<Asura_TreeDialog_Item*> xSelectionLIF(&xSelection); !xSelectionLIF.done(); xSelectionLIF.next())
	{
		HideRecursive(xSelectionLIF(), true);
	}
}

void Tools_SecondaryEntityFile_System::ContextMenu_Unhide()
{
	const List<Asura_TreeDialog_Item*>& xSelection = s_xTree.GetSelection();

	for(CLIF<Asura_TreeDialog_Item*> xSelectionLIF(&xSelection); !xSelectionLIF.done(); xSelectionLIF.next())
	{
		HideRecursive(xSelectionLIF(), false);
	}
}

void Tools_SecondaryEntityFile_System::HideRecursive(Asura_TreeDialog_Item* pxItem, bool bHide)
{
	//hide/unhide this item if it corresponds to an entity
	Tools_Entity_Object* pxObject = GetToolsEntityObjectFromTreeItem(pxItem);
	if(pxObject)
	{
		pxObject->SetHiddenFlagIncludingChildren(ObjectHideFlag_User,bHide);

		#ifndef TOOLS_USE_OLD_ENTITY_MANAGER
		pxItem->SetColourOverride ((bHide) ? TOOLS_SECONDARY_ENTITY_HIDDEN_TREEITEM_COLOUR : GetItemColour (pxItem));
		#endif
	}

	//recurse down the tree
	for(Asura_TreeDialog_Item* pxChild = pxItem->GetChild(); pxChild; pxChild = pxChild->GetSibling())
	{
		HideRecursive(pxChild, bHide);
	}
}

void Tools_SecondaryEntityFile_System::ContextMenu_Create()
{
	if(Tools_System::GetCurrentMode () == TOOLS_MODE_OBJECT && Tools_Mode_Object::ObjectModeOverlayIsOn())
	{
		Tools_Mode_Object::CreateObject_SnapToVertex();
	}
}

void Tools_SecondaryEntityFile_System::ContextMenu_Goto(void* pClickedItem)
{
	Asura_TreeDialog_Item* pxClickedItem = (Asura_TreeDialog_Item*) pClickedItem;
	Tools_Entity_Object* pxEntityObject = GetToolsEntityObjectFromTreeItem(pxClickedItem);
	if(!pxEntityObject) return;

	Tools_Object_System::GotoObject(pxEntityObject);	
	
}

void Tools_SecondaryEntityFile_System::ContextMenu_Delete()
{
	List<Tools_SecondaryEntityFile*> xFileList;
	List<Asura_TreeDialog_Item*> xFolderItemList;
	List<Tools_Object*> xEntityList;

	//Get the selection , removing any cases where both a parent and child are selected
	List<Asura_TreeDialog_Item*> xSelection = s_xTree.GetSelection();
	RemoveChildrenFromList(xSelection);


	//Get a list of files,folders and entities that have been selected.
	//In the case of folders , we want to recurse down and add any subfolders and entities.
	//For files , just take the file on its own , since all the children will be removed if the
	//file is succesfully deleted

	for(LIF<Asura_TreeDialog_Item*> xSelectionLIF(&xSelection); !xSelectionLIF.done(); xSelectionLIF.next())
	{
		AddToDeletionLists(xSelectionLIF(), xFileList, xFolderItemList, xEntityList);
	}

	//delete the selected entities (getting the normal confirmation for doing so)
	if(xEntityList.size())
	{
		Tools_Object_System::DestroyObjects(xEntityList);		
	}

	//Delete any of the selected folders that no longer contain any children
	//Going in reverse so that subfolders get checked first.
	for(LIB<Asura_TreeDialog_Item*> xFolderLIF(&xFolderItemList); !xFolderLIF.done(); xFolderLIF.next())
	{
		if(!xFolderLIF()->GetChild())
		{
			s_xTree.DeleteItem(xFolderLIF());
		}
	}

	//delete any selected files
	for(LIF<Tools_SecondaryEntityFile*> xFileLIF(&xFileList); !xFileLIF.done(); xFileLIF.next())
	{
		RemoveEntityFile(xFileLIF(), true);
	}
}

void Tools_SecondaryEntityFile_System::RemoveChildrenFromList(List<Asura_TreeDialog_Item*>& xSelectionList)
{
	//Check for any cases where a descendent of a selected item is also selected.
	for(LIF<Asura_TreeDialog_Item*> xSelectionLIF(&xSelectionList); !xSelectionLIF.done();)
	{
		Asura_TreeDialog_Item* pxItem1 = xSelectionLIF();
		
		bool bRemoveItem1 = false;
		LIF<Asura_TreeDialog_Item*> xSelectionLIF2(xSelectionLIF);

		for(xSelectionLIF2.next(); !xSelectionLIF2.done();)
		{
			Asura_TreeDialog_Item* pxItem2 = xSelectionLIF2();

			if(pxItem2->IsDescendentOf(pxItem1))
			{
				xSelectionLIF2.delete_current();
			}
			else if(pxItem1->IsDescendentOf(pxItem2))
			{
				bRemoveItem1 = true;
				break;
			}
			else
			{
				xSelectionLIF2.next();
			}
		}

		if(bRemoveItem1)
		{
			xSelectionLIF.delete_current();
		}
		else
		{
			xSelectionLIF.next();
		}
	}
}

void Tools_SecondaryEntityFile_System::AddToDeletionLists(Asura_TreeDialog_Item* pxItem, List<Tools_SecondaryEntityFile*>& xFileList, List<Asura_TreeDialog_Item*>& xFolderItemList, List<Tools_Object*>& xEntityList)
{
	if(IsFileTreeItem(pxItem))
	{
		Tools_SecondaryEntityFile* pxFile = GetFileFromTreeItem(pxItem);
		if(pxFile)
		{
			xFileList.add_entry(pxFile);
		}
	}
	else if(IsEntityTreeItem(pxItem))
	{
		Tools_Entity_Object* pxEntity = GetToolsEntityObjectFromTreeItem(pxItem);
		if(pxEntity)
		{
			xEntityList.add_entry(pxEntity);
		}
	}
	else if(IsFolderTreeItem(pxItem))
	{
		xFolderItemList.add_entry(pxItem);
		for(Asura_TreeDialog_Item* pxChild = pxItem->GetChild(); pxChild; pxChild = pxChild->GetSibling())
		{
			AddToDeletionLists(pxChild, xFileList, xFolderItemList, xEntityList);
		}
	}
}


void Tools_SecondaryEntityFile_System::UpdateTreeVisibility()
{
	s_bFilterChanged = false;
	
	#ifndef TOOLS_USE_OLD_ENTITY_MANAGER
	s_pxFilterSearchStringDialog->GetStringGadget ()->UpdateValue ();
	#endif

	//prepare search string for wildcard search
	char szWildcardSearchString[100] = "";
	if(!s_xSearchText.IsBlank())
	{
		if(s_iSearchOption == DIALOGUE_SEARCH_CONTAINS)
		{
			Asura_PrintfToBfr(szWildcardSearchString,"*%s*",sizeof(szWildcardSearchString),true,s_xSearchText.GetString());
		}
		else
		{
			Asura_PrintfToBfr(szWildcardSearchString,"%s*",sizeof(szWildcardSearchString),true,s_xSearchText.GetString());
		}
	}

	bool bUseFilterList = FilterList_AnySelected() && !FilterList_IsSelected(0);


	Asura_TreeDialog_Item* pxRoot = s_xTree.GetRoot();
	UpdateTreeVisibility(pxRoot, *szWildcardSearchString ? szWildcardSearchString : 0, bUseFilterList);
}

void Tools_SecondaryEntityFile_System::UpdateTreeVisibility(Asura_TreeDialog_Item* pxItem, const char* szWildcardSearchString, bool bUseFilterList)
{
	bool bHide = false;

	if(pxItem->GetChild())
	{
		//recurse over the children , and see if any of them are visible
		bool bAnyVisibleChildren = false;
		for(Asura_TreeDialog_Item* pxChild = pxItem->GetChild(); pxChild; pxChild = pxChild->GetSibling())
		{
			UpdateTreeVisibility(pxChild, szWildcardSearchString, bUseFilterList);
			bAnyVisibleChildren |= !pxChild->IsHidden();
		}

		if(!bAnyVisibleChildren && IsFolderTreeItem(pxItem))
		{
			//This is a non-empty folder , and all the children have become hidden.
			//So hide this as well
			bHide = true;
		}
	}
	else if(IsEntityTreeItem(pxItem))
	{
		//this item corresponds to an entity.
		//see if it matches the search criteria
		if(szWildcardSearchString)
		{
			#ifdef TOOLS_USE_OLD_ENTITY_MANAGER
			if(!Asura_StringUtil::CompareAgainstWildcardString(pxItem->GetName(),szWildcardSearchString,true))
			#else
			Asura_String xRenameSubString = GetNameSubString (pxItem, pxItem->GetName ());

			if(!Asura_StringUtil::CompareAgainstWildcardString(xRenameSubString.GetString (), szWildcardSearchString, true))
			#endif
			{
				bHide = true;
			}
		}

		//check filter by type etc
		if(bUseFilterList && !bHide)
		{
			Tools_Entity_Object* pxObject = GetToolsEntityObjectFromTreeItem(pxItem);
			if(pxObject)
			{
				switch(s_iFilterType)
				{
					case FILTER_OBJECTTYPE :
						bHide = !s_xFilterList_Type.contains(pxObject->Type);
						break;

					case FILTER_SHAPE :
						bHide = !s_xFilterList_Shape.contains(pxObject->GetShapeOrSkinName());
						break;

	    		    case FILTER_BLUEPRINTTYPE :
						bHide = !s_xFilterList_BlueprintType.contains(pxObject->GetBlueprintTypeName());
						break;

	    		    case FILTER_BLUEPRINT :
						bHide = !s_xFilterList_Blueprint.contains(pxObject->GetBlueprintName());
						break;

					default :
						Asura_Assert(0,"Unexpected filter type");
						break;
				}
			}
		}

		#ifndef TOOLS_USE_OLD_ENTITY_MANAGER
		if (!s_bShowHiddenEntities)
		{
			Tools_Entity_Object* pxObject = GetToolsEntityObjectFromTreeItem(pxItem);

			if (pxObject->IsHidden ())
			{
				bHide = true;
			}
		}
		#endif
	}

	pxItem->SetHidden(bHide);
}

void Tools_SecondaryEntityFile_System::DeleteFilterList()
{
	while(s_xFilterList.size())
	{
		delete s_xFilterList.first_entry();
		s_xFilterList.delete_first_entry();
	}

	delete [] s_pszFilterStrings;
	s_pszFilterStrings = 0;
}

void Tools_SecondaryEntityFile_System::UpdateFilterList()
{
	//clear out old strings
	DeleteFilterList();

	//add special entry to filter list
	AddToFilterList("*All*",0,false);

	//get list of object types/skins according to filter type
	for(CLIF<Tools_Object*> xObjectLIF(&Tools_Object_System::ObjectList);!xObjectLIF.done();xObjectLIF.next())
	{
		Tools_Object* pxObject = xObjectLIF();
		//only interested in entities
		if(!pxObject->IsEntityObject()) continue;

		switch(s_iFilterType)
		{
			case FILTER_OBJECTTYPE :
				AddToFilterList(pxObject->GetTypeDescriptionForObjectType(),pxObject->Type,false);
				break;

			case FILTER_SHAPE :
				AddToFilterList(pxObject->GetShapeOrSkinName().GetString(),0,true);
				break;

            case FILTER_BLUEPRINTTYPE :
                AddToFilterList(pxObject->GetBlueprintTypeName().GetString(),0,true);
                break;

            case FILTER_BLUEPRINT :
                AddToFilterList(pxObject->GetBlueprintName().GetString(),0,true);
                break;

			default :
				Asura_Assert(0,"Unexpected filter type");
				break;
		}
	}

	//build array of strings
	s_pszFilterStrings = new char*[s_xFilterList.size()];
	for(int i=0;i<s_xFilterList.size();i++)
	{
		s_pszFilterStrings[i] = s_xFilterList[i]->xString.GetString();
	}

	//Check the previous selection and remove any entries that no longer exist
	if(s_iFilterType == FILTER_OBJECTTYPE)
	{
		for(LIF<int> xSelectedFilterLIF(&s_xFilterList_Type); !xSelectedFilterLIF.done();)
		{
			bool bDelete =true;
			for(int i=0; i<s_xFilterList.size(); i++)
			{
				if(s_xFilterList[i]->iData == xSelectedFilterLIF())
				{
					bDelete = false;
					break;
				}
			}
			if(bDelete)
			{
				xSelectedFilterLIF.delete_current();
			}
			else
			{
				xSelectedFilterLIF.next();
			}
		}
		
	}
	else
	{
		List<Asura_String>* pxList;

		switch(s_iFilterType)
		{
			default :
				Asura_Assert(0,"Unexpected filter type");	
			case FILTER_SHAPE :
				pxList = &s_xFilterList_Shape;
				break;

            case FILTER_BLUEPRINTTYPE :
				pxList = &s_xFilterList_BlueprintType;
                break;

            case FILTER_BLUEPRINT :
				pxList = &s_xFilterList_Blueprint;
                break;
					
		}

		for(LIF<Asura_String> xSelectedFilterLIF(pxList); !xSelectedFilterLIF.done();)
		{
			bool bDelete =true;
			for(int i=0; i<s_xFilterList.size(); i++)
			{
				if(!stricmp(s_xFilterList[i]->xString.GetString(), xSelectedFilterLIF().GetString()))
				{
					bDelete = false;
					break;
				}
			}
			if(bDelete)
			{
				xSelectedFilterLIF.delete_current();
			}
			else
			{
				xSelectedFilterLIF.next();
			}
		}
	}
	
	//determine initial selection
	s_iFilterIndex = 0;
	s_bFilterIndexChanged = true;
	if(FilterList_AnySelected())
	{
		for(int i=0;i<s_xFilterList.size();i++)
		{
			bool bMatch = false;
			switch(s_iFilterType)
			{
				case FILTER_OBJECTTYPE :
					bMatch = (s_xFilterList[i]->iData == s_xFilterList_Type.last_entry());
					break;

				case FILTER_SHAPE :
					bMatch = !s_xFilterList_Shape.last_entry().Compare(s_xFilterList[i]->xString);
					break;

    	        case FILTER_BLUEPRINTTYPE :
    	            bMatch = !s_xFilterList_BlueprintType.last_entry().Compare(s_xFilterList[i]->xString);
    	            break;

    	        case FILTER_BLUEPRINT :
    	            bMatch = !s_xFilterList_Blueprint.last_entry().Compare(s_xFilterList[i]->xString);
    	            break;

				default :
					Asura_Assert(0,"Unexpected filter type");
					break;
			}
			if(bMatch)
			{
				s_iFilterIndex = i;
				break;
			}
		}
	}

	//update list dialog
	s_pxFilterDialog->ReplaceStrings(s_xFilterList.size(),s_pszFilterStrings);	
}

void Tools_SecondaryEntityFile_System::AddToFilterList(const char* pszName,int iData,bool bMatchByName)
{
	if(!pszName || !strlen(pszName)) return;

	//check for this type already being added
	LIF<ListSelectionItem*> xFilterLIF(&s_xFilterList);

	for(xFilterLIF.restart();!xFilterLIF.done();xFilterLIF.next())
	{
		if(bMatchByName)
		{
			if(!stricmp(pszName,xFilterLIF()->xString.GetString())) return;
		}
		else
		{
			if(iData == xFilterLIF()->iData) return;
		}
	}


	//create a ListSelectionItem
	ListSelectionItem* pxItem = new ListSelectionItem;
	pxItem->xString = pszName;
	pxItem->iData = iData;
	pxItem->bSelectedInitially = false;
	pxItem->bHidden = false;

	//add it into the list (sorted)

	for(xFilterLIF.restart();!xFilterLIF.done();xFilterLIF.next())
	{
		if(pxItem->xString.Compare(xFilterLIF()->xString,true)<=0)
		{
			s_xFilterList.add_entry_before(pxItem,xFilterLIF());
			break;
		}
	}
	if(xFilterLIF.done())
	{
		s_xFilterList.add_entry(pxItem);
	}
}

bool Tools_SecondaryEntityFile_System::FilterList_IsSelected(int iIndex)
{
	if(s_iFilterType == FILTER_OBJECTTYPE)
	{
		int iData = s_xFilterList[iIndex]->iData;
		return s_xFilterList_Type.contains(iData);
	}
	else
	{
		const Asura_String& xString = s_xFilterList[iIndex]->xString;
		const List<Asura_String>* pxList;

		switch(s_iFilterType)
		{
			case FILTER_SHAPE :
				pxList = &s_xFilterList_Shape;
				break;

            case FILTER_BLUEPRINTTYPE :
				pxList = &s_xFilterList_BlueprintType;
                break;

            case FILTER_BLUEPRINT :
				pxList = &s_xFilterList_Blueprint;
                break;

			default :
				Asura_Assert(0,"Unexpected filter type");
				return false;
		}
		return pxList->contains(xString);
	}
}

void Tools_SecondaryEntityFile_System::FilterList_SetSelected(int iIndex, bool bSet)
{
	if(s_iFilterType == FILTER_OBJECTTYPE)
	{
		int iData = s_xFilterList[iIndex]->iData;
		s_xFilterList_Type.delete_entry_if_exists(iData);
		if(bSet)
		{
			s_xFilterList_Type.add_entry(iData);
		}
		
	}
	else
	{
		const Asura_String& xString = s_xFilterList[iIndex]->xString;
		List<Asura_String>* pxList;

		switch(s_iFilterType)
		{
			case FILTER_SHAPE :
				pxList = &s_xFilterList_Shape;
				break;

            case FILTER_BLUEPRINTTYPE :
				pxList = &s_xFilterList_BlueprintType;
                break;

            case FILTER_BLUEPRINT :
				pxList = &s_xFilterList_Blueprint;
                break;

			default :
				Asura_Assert(0,"Unexpected filter type");
				return;
		}
		pxList->delete_entry_if_exists(xString);
		if(bSet)
		{
			pxList->add_entry(xString);
		}
	}
	s_bFilterChanged = true;
}

bool Tools_SecondaryEntityFile_System::FilterList_AnySelected()
{
	switch(s_iFilterType)
	{
		case FILTER_OBJECTTYPE :
			return s_xFilterList_Type.size()!=0;

		case FILTER_SHAPE :
			return s_xFilterList_Shape.size()!=0;

		case FILTER_BLUEPRINTTYPE :
			return s_xFilterList_BlueprintType.size()!=0;

		case FILTER_BLUEPRINT :
			return s_xFilterList_Blueprint.size()!=0;

		default :
			Asura_Assert(0,"Unexpected filter type");
			return false;
	}
}

void Tools_SecondaryEntityFile_System::GotoSelected()
{
	Asura_TreeDialog_Item* pxItem = s_xTree.GetSelectedItem();
	if(!pxItem) return;

	Tools_Object* pxObject = GetToolsEntityObjectFromTreeItem(pxItem);
	if(!pxObject) return;


	Tools_Object_System::GotoObject(pxObject);	
}

void Tools_SecondaryEntityFile_System::RenameItem(Asura_TreeDialog_Item* pxItem, const char* szName)
{
	Asura_Assert(pxItem, "");
	Asura_Assert(szName, "");
	if(!*szName) return;

	if(IsEntityTreeItem(pxItem))
	{
		Tools_Entity_Object* pxObject = GetToolsEntityObjectFromTreeItem(pxItem);
		if(pxObject)
		{
			#ifdef TOOLS_USE_OLD_ENTITY_MANAGER
			pxObject->Rename(szName);
			#else
			pxObject->Rename (GetNameSubString (pxItem, szName).GetString ());
			#endif
		}
	}
	else if(IsFileTreeItem(pxItem))
	{
		Tools_SecondaryEntityFile* pxFile = GetFileFromTreeItem(pxItem);
		if(!pxFile) return;

		if(pxFile->IsMainFile())
		{
			Tools_Timed_Message_System::AddMessage(0xff0000,"Can't rename the main entity file in this way.");
			return;			
		}

		RenameEntityFile(pxFile, szName, true);
	}
	else if(IsFolderTreeItem(pxItem))
	{
		RenameFolder(pxItem, szName);

	}
}

void Tools_SecondaryEntityFile_System::UpdateColourRecursive(Asura_TreeDialog_Item* pxItem)
{
	if(!pxItem->IsRoot())
	{
		pxItem->SetColourOverride(GetItemColour(pxItem));
	}

	for(Asura_TreeDialog_Item* pxChild = pxItem->GetChild(); pxChild; pxChild = pxChild->GetSibling())
	{
		UpdateColourRecursive(pxChild);
	}
}

void Tools_SecondaryEntityFile_System::WarnAboutUnwriteableEntity(Tools_Object* pxObject, const char* szFileDisplayName)
{
	//limit error message to once per frame , in case creating multiple objects
	static u_int s_uFrameStamp = 0;
 	if(s_uFrameStamp != Asura_Timers::GetFrameNumber())
	{
		if(!Tools_Overlay_EntityReadOnlyWarning::GetIgnore())
		{
			char acMessage[1024];
			Asura_PrintfToBfr(acMessage, "Warning.\n"
							"An entity has been created in an unwriteable file , and may be lost if not careful.\n"
							"Entity : %s\n"
							"File : %s",
							sizeof(acMessage), true,
							pxObject->GetName(), szFileDisplayName);

			Tools_Overlay_EntityReadOnlyWarning xOverlay(acMessage);
			xOverlay.DoModal();
		}
		
		Tools_Timed_Message_System::AddMessageRemovingDuplicate(0xff0000,"Entity created in unwriteable file");
		s_uFrameStamp = Asura_Timers::GetFrameNumber();
	}
}


//////////////////////////////////////////////////////////////////////////////////////////////////
int Tools_SecondaryEntityFile::s_iNextFileID = 1;

Tools_SecondaryEntityFile::Tools_SecondaryEntityFile(const char* pszName)
{
	m_xName = pszName;
	m_iNextID = 1;
	m_iFileVersion = 0;
	m_bModified = false;
	m_bCommonFile = false;
	m_bMissionDir = false;
	m_bMainFile = false;
	m_bCachedWriteableFlag = false;
	m_uCachedWriteableFlagValidity = 0xffffffff;
	m_iFileID = s_iNextFileID++;
}

bool Tools_SecondaryEntityFile::CanSaveFile(bool bShowMessages) const
{
	if(m_bMainFile)
	{
		m_bCachedWriteableFlag = Tools_SaveAsura::CanSaveEntity();
	}
	else
	{
		Asura_String xFilename;
		GenerateFilename(xFilename);

		m_bCachedWriteableFlag = Tools_SaveAsura::CanSaveFile(xFilename.GetString(), bShowMessages);
	}
	m_uCachedWriteableFlagValidity = Tools_System::GetReactivationCount();
	return m_bCachedWriteableFlag;
}

bool Tools_SecondaryEntityFile::CanSaveFileCached() const
{
	if(m_uCachedWriteableFlagValidity == Tools_System::GetReactivationCount())
	{
		//Use the previously determined writeable flag
		return m_bCachedWriteableFlag;
	}
	return CanSaveFile(false);
}


void Tools_SecondaryEntityFile::WarnIfReadOnly() const
{
	if(m_bMainFile)
	{
		Tools_SaveAsura::WarnIfReadOnly(Tools_SaveAsura::GetEntityFilename());
	}
	else
	{
		Asura_String xFilename;
		GenerateFilename(xFilename);

		Tools_SaveAsura::WarnIfReadOnly(xFilename.GetString());
	}
}

void Tools_SecondaryEntityFile::GenerateFilename(Asura_String& xOutputFilename) const
{
	Asura_Assert(!m_bMainFile, "Not expecting this function to be called for the main entity file");
	Tools_SecondaryEntityFile_System::GenerateFilename(m_xName.GetString(), m_bMissionDir, xOutputFilename);
}

const char* Tools_SecondaryEntityFile::GetDisplayName() const
{
	if(m_bMainFile)
	{
		return Tools_SecondaryEntityFile_System::GetMainFileLabel();
	}
	else
	{
		return m_xName.GetString();
	}
}


//////////////////////////////////////////////////////////////////////////////////////////////////
Tools_SecondaryEntityFileUpdater::Tools_SecondaryEntityFileUpdater(Tools_SecondaryEntityFile* pxFile)
{
	m_pxFile = pxFile;

	m_pxMessageChunk = 0;
	m_pxEntityTextureChunk = 0;
}

Tools_SecondaryEntityFileUpdater::~Tools_SecondaryEntityFileUpdater()
{
	while(m_xEntityList.size())
	{
		delete  m_xEntityList.first_entry();
		m_xEntityList.delete_first_entry();
	}

	while(m_xIDGuidList.size())
	{
		delete m_xIDGuidList.first_entry();
		m_xIDGuidList.delete_first_entry();
	}
}

void Tools_SecondaryEntityFileUpdater::Update(bool bIgnoreVersion /*= false*/)
{
	//get the filename and attempt to open the entity file
	Asura_String xFilename;
	Tools_SecondaryEntityFile_System::GenerateFilename(m_pxFile,xFilename);

	Tools_AsuraFileChunkLister xChunkLister;
	if(!xChunkLister.LoadAsuraFile(xFilename.GetString())) return;

	//search for the secondary entity file chunk to see if an update is needed
	List<const Asura_Chunk_Header*> xChunkList;
	xChunkLister.ListChunksByID(ASURA_CHUNK_ENTITY_SECONDARYFILE,xChunkList);

	int iLoadedNextID;
	int iLoadedVersion;
	int iChunkFlags;
	bool bFoundChunk = false;
	
	for(LIF<const Asura_Chunk_Header*> xChunkLIF(&xChunkList);!xChunkLIF.done();xChunkLIF.next())
	{
		const Asura_Chunk_Entity_SecondaryFile* pxFileChunk = (const Asura_Chunk_Entity_SecondaryFile*) xChunkLIF();

		//get the name from after the chunk header
		const char* pszName = (const char*)(pxFileChunk+1);

		if(!stricmp(pszName,m_pxFile->m_xName.GetString()))
		{
			bFoundChunk = true;
			iLoadedNextID = pxFileChunk->m_iNextID;
			iLoadedVersion = pxFileChunk->m_iFileVersion;
			iChunkFlags = pxFileChunk->Flags;
			break;
		}
	}
	if(!bFoundChunk)
	{
		//something dodgy with the file then
		Asura_Assertf(0,("Failed to update from secondary entity file %s",xFilename.GetString()));
		return;
	}

	if(!bIgnoreVersion)
	{
		if(iLoadedVersion == m_pxFile->m_iFileVersion)
		{
			//no update required
			return;
		}
	}
	m_pxFile->m_bCommonFile = (iChunkFlags & Asura_Chunk_Entity_SecondaryFile::ChunkFlag_CommonFile) ? true : false;

	//need to update in that case... extract all the chunks we will need
	const List<const Asura_Chunk_Header*>* pxChunkList = xChunkLister.GetChunkList();

	Tools_EntityChunkPair* m_pxLastEntity = 0;
	const Asura_Chunk_TextureNames* pxTextureNamesChunk = 0;

	for(CLIF<const Asura_Chunk_Header*> xChunkLIF(pxChunkList);!xChunkLIF.done();xChunkLIF.next())
	{
		switch(xChunkLIF()->ID)
		{
			case ASURA_CHUNK_ENTITY :
			{
				const Asura_Chunk_Entity* pxEntity = (const Asura_Chunk_Entity*) xChunkLIF();

				m_pxLastEntity = new Tools_EntityChunkPair;
				m_pxLastEntity->m_pxEntityChunk = pxEntity;
				m_pxLastEntity->m_pxToolsChunk = 0;
				m_xEntityList.add_entry(m_pxLastEntity);
				

				if(pxTextureNamesChunk)
				{
					//texture name chunk before entities should belong to them
					m_pxEntityTextureChunk = pxTextureNamesChunk;
					pxTextureNamesChunk = 0;
				}


				break;
			}

			case ASURA_CHUNK_ENTITY_TOOLSDATA :
			{
				const Asura_Chunk_Entity_ToolsData* pxToolsData = (const Asura_Chunk_Entity_ToolsData*) xChunkLIF();

				//expect the tools data to come immediately after the corresponding entity
				Asura_Assert(m_pxLastEntity,"");
				Asura_Assert(!m_pxLastEntity->m_pxToolsChunk,"");
				Asura_Assert(m_pxLastEntity->m_pxEntityChunk->Guid == pxToolsData->Guid,"");

				m_pxLastEntity->m_pxToolsChunk = pxToolsData;				
				break;
			}

			case ASURA_CHUNK_STATICMESSAGES :
			{
				m_pxMessageChunk = (const Asura_Chunk_StaticMessages*) xChunkLIF();
				break;
			}


			case ASURA_CHUNK_TEXTURENAMES :
			{
				//store pointer to chunk , until we find out what us using it
				pxTextureNamesChunk = (const Asura_Chunk_TextureNames*) xChunkLIF();
				break;
			}
			
			case ASURA_CHUNK_SOUNDNAMES :
			{
				m_xSoundChunkList.add_entry((const Asura_Chunk_SoundNames*) xChunkLIF());
				break;
			}


			default :
				break;
		}
	}
	
	//Delete all the old entities that belong to this file , but keep track
	//of the guids used so that updated entities keep the same guid.

	List<Tools_Object*> xObjectsToDestroy;

	for(LIF<Tools_Object*> xObjectLIF(&Tools_Object_System::ObjectList);!xObjectLIF.done();xObjectLIF.next())
	{
		if(!xObjectLIF()->IsEntityObject()) continue;
		Tools_Entity_Object* pxEntityObject = (Tools_Entity_Object*) xObjectLIF();
		Asura_Entity* pxEntity = pxEntityObject->Entity;

		if(!pxEntity->ToolsData) continue;
		
		if(pxEntity->ToolsData->m_xSecondaryEntityFilename.IsBlank()) continue;

		//does this entity belong with this file?
		if(!stricmp(m_pxFile->m_xName.GetString(),pxEntity->ToolsData->m_xSecondaryEntityFilename.GetString()))
		{
			//store the ID and Guid
			Tools_IDAndGuids* pxIDGuid = new Tools_IDAndGuids;
			pxIDGuid->m_iID = pxEntity->ToolsData->m_iSecondaryEntityID;
			pxIDGuid->m_uOldGuid = ASURA_GUID_UNREGISTERED;
			pxIDGuid->m_uNewGuid = pxEntity->GetGuid();
			pxIDGuid->m_bUsableInSecondaryFile = pxEntity->CanBeIncludedInTemplate();
			m_xIDGuidList.add_entry(pxIDGuid);

			if(pxIDGuid->m_bUsableInSecondaryFile)
			{
				//and add to the list of objects to be destroyed
				//only done for entities that are still allowed in secondary files , to limit the damage from
				//them becoming disallowed
				xObjectsToDestroy.add_entry(pxEntityObject);
				pxIDGuid->m_pxOldEntity = 0;
			}
			else
			{
				pxIDGuid->m_pxOldEntity = pxEntityObject;
			}
		}
	}

	SetUpdatingFromSecondaryFile(true);
	
	//destroy the entities
	for(LIF<Tools_Object*> xDestroyLIF(&xObjectsToDestroy);!xDestroyLIF.done();xDestroyLIF.next())
	{
		if(Tools_Object_System::ObjectList.contains(xDestroyLIF()))
		{
			xDestroyLIF()->Destroy();
		}
	}
	Asura_Entity_System::PruneEntityLists();

	//process the texture chunk , so that the conversion array is setup
	if(m_pxEntityTextureChunk)
	{
		m_pxEntityTextureChunk->Process((const char*)(m_pxEntityTextureChunk+1));
	}


	List<Asura_Entity*> xEntityList;
	List<Asura_Guid> xBadEntityList;
	

	//Set update from template flag  , since similar rules apply as far as creating the entities
	Tools_EntityTemplate::SetUpdatingFromTemplate(true);
	
	//Create entities from the chunks
	for(LIF<Tools_EntityChunkPair*> xEntityLIF(&m_xEntityList);!xEntityLIF.done();xEntityLIF.next())
	{
		Asura_Entity_Server* pxEntity;

		const Asura_Chunk_Entity* pxEntityChunk = xEntityLIF()->m_pxEntityChunk;
		const Asura_Chunk_Entity_ToolsData* pxToolsChunk = xEntityLIF()->m_pxToolsChunk;

		if(!pxToolsChunk) continue;

		//read in the tools data to determine whether this entity is wanted
		Tools_Entity_Data xToolsData;

		pxToolsChunk->Process((const char*)(pxToolsChunk+1),&xToolsData);

		if (xToolsData.m_xSecondaryEntityFilename.IsBlank()) continue;

		if(stricmp(xToolsData.m_xSecondaryEntityFilename.GetString(),m_pxFile->m_xName.GetString()))
		{
			continue;
		}

		//Need to get a guid for this entity then. 
		//See if the entity matches one in the list we just destroyed

		Asura_Guid uGuid = ASURA_GUID_UNREGISTERED;
		bool bReusingGuid = false;
		bool bSkipEntity = false;
		for(LIF<Tools_IDAndGuids*> xGuidLIF(&m_xIDGuidList);!xGuidLIF.done();xGuidLIF.next())
		{
			if(xGuidLIF()->m_iID == xToolsData.m_iSecondaryEntityID)
			{
				uGuid = xGuidLIF()->m_uNewGuid;
				xGuidLIF()->m_uOldGuid = pxToolsChunk->Guid;
				bReusingGuid = true;
				if(!xGuidLIF()->m_bUsableInSecondaryFile)
				{
					//Entity is no longer allowed in a secondary file , and so can't be updated.
					//To limit the damage , just leave it alone.
					Asura_Assertf(0,("Entity '%s' in secondary entity file '%s' is no longer allowed in secondary files.",xGuidLIF()->m_pxOldEntity->GetName(),m_pxFile->m_xName.GetString()));
					bSkipEntity = true;
				}
				break;
			}
		}
		if(bSkipEntity) continue;
		if(uGuid == ASURA_GUID_UNREGISTERED)
		{
			//failed to find a guid , so allocate a new one
			while(true)
			{
				//get the guid to use for the new entity
				uGuid = Tools_EntityTemplate_System::GetNextEntityGuid();
				
				//make sure this doesn't already exist
				pxEntity = Asura_Entity_Server::FindEntity(uGuid);
				Asura_Assert(!pxEntity,"Guid already used??");
				if(!pxEntity)
				{
					break;	
				}
			}

			//and add to the list of ids and guids
			Tools_IDAndGuids* pxIDGuid = new Tools_IDAndGuids;
			pxIDGuid->m_iID = xToolsData.m_iSecondaryEntityID;
			pxIDGuid->m_uOldGuid = pxToolsChunk->Guid;
			pxIDGuid->m_uNewGuid = uGuid;
			pxIDGuid->m_bUsableInSecondaryFile = true;//may not be correct , but not really neded , and better than being uninitialised
			pxIDGuid->m_pxOldEntity = 0;
			m_xIDGuidList.add_entry(pxIDGuid);

		}


		//make a copy of the chunk header , with the new guid
		Asura_Chunk_Entity xHeader = *pxEntityChunk;
		xHeader.Guid = uGuid;

		//create the entity
		xHeader.Process((const char*)(pxEntityChunk+1));

		//make sure the entity does exist now
		pxEntity = Asura_Entity_Server::FindEntity(uGuid);
		Asura_Assertf(pxEntity,("Failed to create entity in secondary entity file '%s'",m_pxFile->m_xName.GetString()));
		if(!pxEntity)
		{
			//return the guid that we were going to use
			if(!bReusingGuid)
			{
				Tools_EntityTemplate_System::RecycleEntityGuid(uGuid);
			}

			//also take note of the bad entry , so we can fix them in just a sec...
			xBadEntityList.add_entry(pxEntityChunk->Guid);

			continue;
		}
		
		xEntityList.add_entry(pxEntity);


		//now do the tools data
		if(pxToolsChunk)
		{
			Asura_Chunk_Entity_ToolsData xToolsHeader = *pxToolsChunk;
			xToolsHeader.Guid = uGuid;

			xToolsHeader.Process((const char*)(pxToolsChunk+1));

			//If the entity existed before the update , then mark the guid as being constant
			Asura_Assert(pxEntity->ToolsData,"");
			if(pxEntity->ToolsData)
			{
				pxEntity->ToolsData->m_bSecondaryEntityGuidConstant = bReusingGuid;
			}

		}

		//make sure the entity is allowed in a template (typically this should be caught before
		//the template file is saved in the first place)
		if(!pxEntity->CanBeIncludedInTemplate())
		{
			Asura_Assertf(0,("Entity %s in entity file %s isn't useable in entity templates.\n(And so shouldn't exist in secondary entity files either)",pxEntity->ToolsData->Name.GetString(),m_pxFile->m_xName.GetString()));

			//delete the dud entity ,and release the guid used
			pxEntity->SetAsDestroyed();
			Tools_EntityTemplate_System::RecycleEntityGuid(uGuid);

			xEntityList.delete_entry(pxEntity);

			//also take note of the bad entry , so we can fix them in just a sec...
			xBadEntityList.add_entry(pxEntityChunk->Guid);
		}
	}
	
	SetUpdatingFromSecondaryFile(false);
	Tools_EntityTemplate::SetUpdatingFromTemplate(false);

	//Deal with entities that are no longer allowed in secondary files.
	//If there are any that no longer exist in the secondary file , they can now be safely deleted
	//RAR 17/11/2008
	for(LIF<Tools_IDAndGuids*> xGuidLIF(&m_xIDGuidList);!xGuidLIF.done();xGuidLIF.next())
	{
		Tools_IDAndGuids* pxIDAndGuid = xGuidLIF();
		if(!pxIDAndGuid->m_bUsableInSecondaryFile && pxIDAndGuid->m_uOldGuid == ASURA_GUID_UNREGISTERED)
		{
			pxIDAndGuid->m_pxOldEntity->Destroy();
		}
	}
	Asura_Entity_System::PruneEntityLists();
	


	//if there were any dodgy entities , then adjust the new guid for use in any conversions
	for(LIF<Asura_Guid> xBadGuidLIF(&xBadEntityList);!xBadGuidLIF.done();xBadGuidLIF.next())
	{
		for(LIF<Tools_IDAndGuids*> xGuidLIF(&m_xIDGuidList);!xGuidLIF.done();xGuidLIF.next())
		{
			if(xGuidLIF()->m_uNewGuid == xBadGuidLIF())
			{
				xGuidLIF()->m_uNewGuid = ASURA_GUID_UNREGISTERED;
				break;
			}
		}
	}

	//adjust the message blocks etc for all the entities
	for(LIF<Asura_Entity*> xEntityLIF(&xEntityList);!xEntityLIF.done();xEntityLIF.next())
	{
		//update the entity
		xEntityLIF()->ApplyChangesForTemplateInstance(this);

        xEntityLIF()->InitFromBlueprint();

		//Check for a client entity. 
		//In some cases these need updating as well (particle generators)
		Asura_Entity_Client* pxClientEntity = Asura_Entity_Client::FindEntity(xEntityLIF()->GetGuid());
		if(pxClientEntity)
		{
			pxClientEntity->ApplyChangesForTemplateInstance(this);
		}
	}


	Tools_ShapeTable::BuildShapeTable();

	//create tools objects for all the entities
	for(LIF<Asura_Entity*> xEntityLIF(&xEntityList);!xEntityLIF.done();xEntityLIF.next())
	{
		Tools_Entity_Object::CreateObject(xEntityLIF());

		if(Tools_SecondaryEntityFile_System::IsTreeValid())
		{
			Tools_Entity_Object* pxToolsObject = Tools_Entity_Object::FindEntity( xEntityLIF()->GetGuid() );
			if(pxToolsObject)
			{
				Tools_SecondaryEntityFile_System::UpdateEntityInTree(pxToolsObject, false);
			}
		}
	}

	//call OnUpdateFromTemplate() for each entity since much the same stuff needs to be done
	//for secondary entity files
	for(LIF<Asura_Entity*> xEntityLIF(&xEntityList);!xEntityLIF.done();xEntityLIF.next())
	{
		Tools_Entity_Object* pxObject = xEntityLIF()->ToolsData->ToolsObject;
		if(pxObject) pxObject->OnUpdateFromTemplate();
	}


	m_pxFile->m_iNextID = iLoadedNextID;
	m_pxFile->m_iFileVersion = iLoadedVersion;
	
}

u_short Tools_SecondaryEntityFileUpdater::ConvertMessageBlock(u_short usBlockID) const
{
	if(usBlockID == ASURA_STATICMESSAGE_EMPTYBLOCK) return usBlockID;//empty blocks don't need converting

	if(!m_pxMessageChunk) return ASURA_STATICMESSAGE_EMPTYBLOCK;
	usBlockID = m_pxMessageChunk->ProcessSingleMessageBlock((const char*)(m_pxMessageChunk+1),usBlockID);

	if(usBlockID == ASURA_STATICMESSAGE_EMPTYBLOCK) return usBlockID;//failed to find in chunk , so leaving as empty is best we can do

	//now need to check the destinations for all the messages ,and remap the guids
	int iNumMessages = Asura_StaticMessage_System::GetNumberOfMessagesInBlock(usBlockID);
	for(int i=0;i<iNumMessages;i++)
	{
		Asura_StaticMessage* pxMessage = Asura_StaticMessage_System::GetMessagePointer(usBlockID,i);
		if(!pxMessage) continue;

		pxMessage->m_uSendTo = ConvertGuid(pxMessage->m_uSendTo);

		//deal with message specific guids
		Tools_EntityTemplate_System::GetThis()->ConvertStaticMessage(pxMessage,this);
	}
	return usBlockID;
}

Asura_Guid Tools_SecondaryEntityFileUpdater::ConvertGuid(Asura_Guid uGuid) const
{
	if(uGuid == ASURA_GUID_UNREGISTERED) return uGuid;

	//search for guid in the list
	for(CLIF<Tools_IDAndGuids*> xGuidLIF(&m_xIDGuidList);!xGuidLIF.done();xGuidLIF.next())
	{
		if(xGuidLIF()->m_uOldGuid == uGuid)
		{
			return xGuidLIF()->m_uNewGuid;
		}
	}

	//failed to find guid , so leave unaltered
	return uGuid;
}

int Tools_SecondaryEntityFileUpdater::ConvertSoundID(int iSoundID) const
{
#if defined ASURA_USE_AUDIO_SYSTEM

	// AUDIOTODO: this function (and its implementation in all Tools_EntityTemplateInstance-derived
	// types) simply doesn't need to exist with the new audio system
	return iSoundID;

#else

	if(!m_xSoundChunkList.size()) return ASURA_SID_NO_SOUND;

	//if not an entity sound ID , then no conversion is needed
	bool bEntity = iSoundID >= ASURA_SID_ENTITY_MIN && iSoundID <= ASURA_SID_ENTITY_MAX;
	bool bEnv = iSoundID >= ASURA_SID_ENVIRONMENT_00 && iSoundID <= ASURA_SID_ENVIRONMENT_MAX;
	if(!bEntity && !bEnv) return iSoundID;

	Asura_Sound_ID eMin = bEntity ? ASURA_SID_ENTITY_MIN : ASURA_SID_ENVIRONMENT_00;
	Asura_Sound_ID eMax = bEntity ? ASURA_SID_ENTITY_MAX : ASURA_SID_ENVIRONMENT_MAX;
	
	for(CLIF<const Asura_Chunk_SoundNames*> xChunkLIF(&m_xSoundChunkList);!xChunkLIF.done();xChunkLIF.next())
	{
		const char* pszFilename = xChunkLIF()->GetFilename((const char*)(xChunkLIF()+1),iSoundID);

		if(pszFilename)
		{
			return Tools_PickSound::GetSoundIDForFilename(pszFilename,eMin,eMax);
		}
		
	}
	return ASURA_SID_NO_SOUND;

#endif
}