//
// Tools_Entity_Template_Object.cpp
//
// Created by Richard Rice - 05/04/2004
// 
// Copyright (C) 2004 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"
#include "EntityTemplate/Tools_Entity_Template_Object.h"

#include "Core/Asura_Los.h"
#include "Core/Asura_Windows.h"
#include "Core/Asura_SortAndSearch.h"
#ifdef ASURA_USE_ATTRACTORS
#include "Entity/Attractors/Asura_ServerEntity_AttractorController.h"
#endif
#include "Entity/EntityTemplate/Asura_ServerEntity_Template.h"
#include "Entity/PhysicalObject/Asura_ServerEntity_PhysicalObject.h"
#include "Entity/Asura_Entity_Classification.h"
#include "Entity/Asura_StaticMessage_System.h"
#include "FileAccess/Asura_FileAccess.h"
#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Overlay.h"
#include "Interface/Asura_Overlay_System.h"
#include "Mesh/Asura_Mesh_System.h"
#include "Rendering/Asura_Text_Output.h"
#include "Sound/Asura_Phonon.h"
#include "Sound/Asura_StreamingBackgroundSound_System.h"
#include "SpecialFX/Asura_StaticCorona_System.h"
#include "SpecialFX/Asura_Skybox.h"

#include "Tools_Overlays.h"
#ifdef ASURA_USE_ATTRACTORS
#include "Attractors/Tools_Attractor.h"
#include "Attractors/Tools_Attractor_Node.h"
#include "Attractors/Tools_Attractor_System.h"
#include "Attractors/Tools_Object_Attractor_Node.h"
#endif
#include "Entity/Tools_Entity_Data.h"
#include "Entity/Tools_Entity_PhysicalObject.h"
#include "Entity/Tools_MessageEditor.h"
#include "Entity/Tools_Message_Editing.h"
#include "EntityTemplate/Tools_EntityTemplate_System.h"
#include "Entity/Tools_SecondaryEntityFile_System.h"
#include "Hierarchy/Tools_Animation_Files.h"
#include "Hierarchy/Tools_Dialog_PickSkin.h"
#include "Interface/Tools_ContextMenu_System.h"
#include "Lighting/Tools_Mode_PhotonMapping.h"
#include "Lighting/Tools_PhotonMapping_LightSource.h"
#include "Lighting/Tools_PhotonMapping_LightSource_Object.h"
#include "Lighting/Tools_PhotonMapping_System.h"
#include "Lighting/Tools_PhotonMapping_System_Internal.h"
#include "Lighting/Tools_Corona_Object.h"
#include "Lighting/Tools_LightSets.h"
#include "Load&Save/Tools_SaveAsura.h"
#include "Misc/Tools_Timed_Messages.h"
#include "Objects/Tools_Object_System.h"
#include "PC/FileSystem/Tools_FileSystem.h"
#include "PC/OS/Tools_OS.h"
#include "Sound/Tools_Phonon_Object.h"
#include "Sound/Tools_StreamingBackgroundSound_Object.h"

Asura_Hashed_List<Tools_Entity_Template_Object> Tools_Entity_Template_Object::s_xList;

//////////////////////////////////////////////////////////////////////////////////////////////////
/*------------------------------**
** Tools_Entity_Template_Object **
**------------------------------*/

Tools_Object* Tools_Entity_Template_Object::CreateToolsObject(Asura_Entity* pxEntity)
{
	if(pxEntity->GetClassification() != AsuraEntityClass_Template)
	{
		Asura_Assert(0,"This entity isn't a template entity");
		return 0;
	}

	Asura_ServerEntity_Template* pxTemplate = (Asura_ServerEntity_Template*) pxEntity;

	//create master or instance object as needed
	if(pxTemplate->IsMaster())
	{
		return new Tools_Entity_TemplateMaster_Object(pxEntity);
	}
	else
	{
		return new Tools_Entity_TemplateInstance_Object(pxEntity);
	}
}

void Tools_Entity_Template_Object::Destroy()
{
	Asura_ServerEntity_Template* pxAsuraEntity = (Asura_ServerEntity_Template*) Entity;

	//destroy any message blocks
	for(int i=0;i<pxAsuraEntity->m_iNumInputs;i++)
	{
		Asura_StaticMessage_System::DeleteMessageBlock(pxAsuraEntity->m_pxInputs[i].m_usMessageBlock);
	}
	for(int i=0;i<pxAsuraEntity->m_iNumOutputs;i++)
	{
		Asura_StaticMessage_System::DeleteMessageBlock(pxAsuraEntity->m_pxOutputs[i].m_usMessageBlock);
	}


	Tools_Entity_Object::Destroy();
}

bool Tools_Entity_Template_Object::OwnsThisMessageBlock(unsigned short usMessageBlockID)
{
	Asura_ServerEntity_Template* pxAsuraEntity = (Asura_ServerEntity_Template*) Entity;

	for(int i=0;i<pxAsuraEntity->m_iNumInputs;i++)
	{
		if(usMessageBlockID == pxAsuraEntity->m_pxInputs[i].m_usMessageBlock) return true;
	}
	for(int i=0;i<pxAsuraEntity->m_iNumOutputs;i++)
	{
		if(usMessageBlockID == pxAsuraEntity->m_pxOutputs[i].m_usMessageBlock) return true;
	}

	return Tools_Entity_Object::OwnsThisMessageBlock(usMessageBlockID);
}

bool Tools_Entity_Template_Object::GetEventDescForMessageBlock(unsigned short usMessageBlockID,Asura_String& xOutputDesc)
{
	Asura_ServerEntity_Template* pxAsuraEntity = (Asura_ServerEntity_Template*) Entity;

	if(pxAsuraEntity->m_pxInputNames)
	{
		for(int i=0;i<pxAsuraEntity->m_iNumInputs;i++)
		{
			if(usMessageBlockID == pxAsuraEntity->m_pxInputs[i].m_usMessageBlock)
			{
				char szTemp[100];
				Asura_PrintfToBfr(szTemp,"Input %s",sizeof(szTemp),true,pxAsuraEntity->m_pxInputNames[i].GetString());
				xOutputDesc = szTemp;

				return true;
			}
		}
	}
	if(pxAsuraEntity->m_pxOutputNames)
	{
		for(int i=0;i<pxAsuraEntity->m_iNumOutputs;i++)
		{
			if(usMessageBlockID == pxAsuraEntity->m_pxOutputs[i].m_usMessageBlock)
			{
				const char* szOutputName = GetOutputNameFromHash(pxAsuraEntity->m_pxOutputs[i].m_uNameHash);
				
				char szTemp[100];
				Asura_PrintfToBfr(szTemp,"Output %s",sizeof(szTemp),true,szOutputName);
				xOutputDesc = szTemp;

				return true;
			}
		}
	}

	return false;
}


bool Tools_Entity_Template_Object::HandlesThisMessage(int iMessageID)
{
	if(iMessageID == ASURA_MSG_TEMPLATE_TRIGGERINPUT ||
	   iMessageID == ASURA_MSG_TEMPLATE_TRIGGEROUTPUT)
	{
		return true;
	}
	return PARENT::HandlesThisMessage(iMessageID);
}



int Tools_Entity_Template_Object::SelectInputIndex(const char* pszMessage)
{
	Asura_ServerEntity_Template* pxAsuraEntity = (Asura_ServerEntity_Template*) Entity;

	if(!pxAsuraEntity->m_iNumInputNames)
	{
		ShowMessageOverlay("This template doesn't have any inputs");
		return -1;
	}

	ListSelection_Overlay xOverlay(pszMessage);
	//add the names of all the inputs
	for(int i=0;i<pxAsuraEntity->m_iNumInputNames;i++)
	{
		xOverlay.AddItem(pxAsuraEntity->m_pxInputNames[i].GetString(),i);
	}

	if(!xOverlay.DoModal()) return -1;

	return xOverlay.GetSelectionData();
	
}

Asura_Hash_ID Tools_Entity_Template_Object::SelectInputHash(const char* pszMessage)
{
	Asura_ServerEntity_Template* pxAsuraEntity = (Asura_ServerEntity_Template*) Entity;

	int iIndex = SelectInputIndex(pszMessage);
	if(iIndex<0) return ASURA_HASH_ID_UNSET;

	return Asura_GetHashID(pxAsuraEntity->m_pxInputNames[iIndex].GetString());
}

Asura_Hash_ID Tools_Entity_Template_Object::SelectOutputHash(const char* pszMessage)
{
	Asura_ServerEntity_Template* pxAsuraEntity = (Asura_ServerEntity_Template*) Entity;

	int iIndex = SelectOutputIndex(pszMessage);
	if(iIndex<0) return ASURA_HASH_ID_UNSET;

	return Asura_GetHashID(pxAsuraEntity->m_pxOutputNames[iIndex].GetString());
}


int Tools_Entity_Template_Object::SelectOutputIndex(const char* pszMessage)
{
	Asura_ServerEntity_Template* pxAsuraEntity = (Asura_ServerEntity_Template*) Entity;

	if(!pxAsuraEntity->m_iNumOutputNames)
	{
		ShowMessageOverlay("This template doesn't have any outputs");
		return -1;
	}

	ListSelection_Overlay xOverlay(pszMessage);
	//add the names of all the inputs
	for(int i=0;i<pxAsuraEntity->m_iNumOutputNames;i++)
	{
		xOverlay.AddItem(pxAsuraEntity->m_pxOutputNames[i].GetString(),i);
	}

	if(!xOverlay.DoModal()) return -1;

	return xOverlay.GetSelectionData();
	
}

const char* Tools_Entity_Template_Object::GetInputNameFromHash(Asura_Hash_ID uNameHash) const
{
	const Asura_ServerEntity_Template* pxAsuraEntity = (const Asura_ServerEntity_Template*) Entity;

	for(int i=0;i<pxAsuraEntity->m_iNumInputNames;i++)
	{
		if(uNameHash == Asura_GetHashID(pxAsuraEntity->m_pxInputNames[i].GetString()))
		{
			return pxAsuraEntity->m_pxInputNames[i].GetString();
		}
	}

	return "*Unknown*";
}


const char* Tools_Entity_Template_Object::GetOutputNameFromHash(Asura_Hash_ID uNameHash) const
{
	const Asura_ServerEntity_Template* pxAsuraEntity = (const Asura_ServerEntity_Template*) Entity;

	for(int i=0;i<pxAsuraEntity->m_iNumOutputNames;i++)
	{
		if(uNameHash == Asura_GetHashID(pxAsuraEntity->m_pxOutputNames[i].GetString()))
		{
			return pxAsuraEntity->m_pxOutputNames[i].GetString();
		}
	}

	return "*Unknown*";
}

void Tools_Entity_Template_Object::AddOverlayControls_TemplateTrigger(struct Asura_StaticMessage* pxMessage,class Asura_Overlay* pxOverlay,class Asura_Vector_2& xOffset)
{
	Asura_Assert(pxMessage->m_pData,"");
	Asura_Message_Template_Trigger* pxData = (Asura_Message_Template_Trigger*) pxMessage->m_pData;

	Asura_Dialog* pxDialog = 0;

	//find the entity
	Asura_ServerEntity_Template* pxEntity = Asura_ServerEntity_Template::FindEntity(Tools_Message_Editing::GetFinalDestinationGuid(pxMessage));
	if(!pxEntity)
	{
		pxDialog = new Asura_Dialog("*Unknown Target*",xOffset);	
		pxOverlay->AddDialog(pxDialog);
		return;
	}


	Asura_Assert(pxEntity->ToolsData && pxEntity->ToolsData->ToolsObject,"");
	Tools_Entity_Template_Object* pxObject = (Tools_Entity_Template_Object*) pxEntity->ToolsData->ToolsObject;

	//get the name
	const char* pszName;
	char szText[100];
	if(pxData->m_uNameHash)
	{
		if(pxMessage->m_usMsgID == ASURA_MSG_TEMPLATE_TRIGGERINPUT)
		{
			pszName = pxObject->GetInputNameFromHash(pxData->m_uNameHash);

		}
		else
		{
			pszName = pxObject->GetOutputNameFromHash(pxData->m_uNameHash);
		}
	}
	else
	{
		pszName = "*Not Set*";
	}

	if(pxMessage->m_usMsgID == ASURA_MSG_TEMPLATE_TRIGGERINPUT)
	{
		Asura_PrintfToBfr(szText,"Input : %s",100,true,pszName);
		pxDialog = new Asura_Function_Dialog(szText,xOffset,EditTriggerInputMessage,(int)pxMessage);
		pxDialog->SetHelpText("Select input to trigger");
		pxOverlay->AddDialog(pxDialog);
		xOffset.y += Asura_Dialog::DefaultSpacing();

	}
	else
	{
		Asura_PrintfToBfr(szText,"Output : %s",100,true,pszName);
		pxDialog = new Asura_Function_Dialog(szText,xOffset,EditTriggerOutputMessage,(int)pxMessage);
		pxDialog->SetHelpText("Select output to trigger");
		pxOverlay->AddDialog(pxDialog);
		xOffset.y += Asura_Dialog::DefaultSpacing();
	}
}

void Tools_Entity_Template_Object::EditTriggerInputMessage(int iMessage)
{
	EditTriggerInputMessage((Asura_StaticMessage*)iMessage);
	Tools_Message_Editing::UpdateMessageOverlay();
	Tools_MessageEditor::UpdateMessageEditorTable();
}

void Tools_Entity_Template_Object::EditTriggerInputMessage(Asura_StaticMessage* pxMessage)
{
	Asura_Assert(pxMessage->m_pData,"");
	Asura_Message_Template_Trigger* pxData = (Asura_Message_Template_Trigger*) pxMessage->m_pData;

	//find the entity and tools object for this
	Asura_ServerEntity_Template* pxEntity = Asura_ServerEntity_Template::FindEntity(Tools_Message_Editing::GetFinalDestinationGuid(pxMessage));
	if(!pxEntity) return;
	if(!pxEntity->ToolsData) return;
	if(!pxEntity->ToolsData->ToolsObject) return;

	Tools_Entity_Template_Object* pxObject = (Tools_Entity_Template_Object*) pxEntity->ToolsData->ToolsObject;

	//ask for the input to trigger
	Asura_Hash_ID uHash = pxObject->SelectInputHash("Select input to trigger.");
	if(uHash)
	{
		pxData->m_uNameHash = uHash;
	}
}

void Tools_Entity_Template_Object::EditTriggerOutputMessage(int iMessage)
{
	EditTriggerOutputMessage((Asura_StaticMessage*)iMessage);
	Tools_Message_Editing::UpdateMessageOverlay();
}

void Tools_Entity_Template_Object::EditTriggerOutputMessage(Asura_StaticMessage* pxMessage)
{
	Asura_Assert(pxMessage->m_pData,"");
	Asura_Message_Template_Trigger* pxData = (Asura_Message_Template_Trigger*) pxMessage->m_pData;

	//find the entity and tools object for this
	Asura_ServerEntity_Template* pxEntity = Asura_ServerEntity_Template::FindEntity(Tools_Message_Editing::GetFinalDestinationGuid(pxMessage));
	if(!pxEntity) return;
	if(!pxEntity->ToolsData) return;
	if(!pxEntity->ToolsData->ToolsObject) return;

	Tools_Entity_Template_Object* pxObject = (Tools_Entity_Template_Object*) pxEntity->ToolsData->ToolsObject;

	//ask for the input to trigger
	Asura_Hash_ID uHash = pxObject->SelectOutputHash("Select input to trigger.");
	if(uHash)
	{
		pxData->m_uNameHash = uHash;
	}
}

const char* Tools_Entity_Template_Object::GetTemplateName() const
{
	Asura_ServerEntity_Template* pxAsuraEntity = (Asura_ServerEntity_Template*) Entity;
	return pxAsuraEntity->m_xName.GetString();
}

//////////////////////////////////////////////////////////////////////////////////////////////////
/*------------------------------------**
** Tools_Entity_TemplateMaster_Object **
**------------------------------------*/

List<Tools_Entity_TemplateMaster_Object*> Tools_Entity_TemplateMaster_Object::s_xObjectList;

Tools_Entity_TemplateMaster_Object::Tools_Entity_TemplateMaster_Object(Asura_Entity* pxEntity)
:Tools_Entity_Template_Object(pxEntity)
{
	s_xObjectList.add_entry(this);

	Asura_ServerEntity_Template* pxTemplate = (Asura_ServerEntity_Template*) Entity;
	Asura_Assert(pxTemplate->m_bMaster,"Shouldn't be creating Tools_Entity_TemplateMaster_Object for template instances");

	Type = TOT_Entity_TemplateMaster;
	Mesh = Asura_Mesh_System::GetMesh("ToolsShape_Diamond");

	Position = pxTemplate->m_xPosition;
	Orientation = pxTemplate->m_xOrient;

}

Tools_Entity_TemplateMaster_Object::~Tools_Entity_TemplateMaster_Object()
{
	s_xObjectList.delete_entry(this);
}


void Tools_Entity_TemplateMaster_Object::Create()
{
	//get the name for the master from the filename
	//This is probably incorrect , since it should really include the path relative to the Templates directory.
	//However , I don't think the name stored in the master template ever gets used anyway. RAR 07/06/2006
	Asura_String xFilename = Tools_SaveAsura::GetTemplateFilename();
	Asura_String xName;
	xFilename.SplitFilename(0,&xName,0);

	//make sure the master template entity doesn't already exist
	for(Asura_Classified_List_It<Asura_Entity_Server> it(Asura_Entity_Server::GetList());!it.Done();it.Next())
	{
		Asura_Entity_Server* pxEntity = it.GetCurrent();
		//ignore entities marked for destruction
		if(pxEntity->IsDestroyed()) continue;
		//and those with non-tools guids
		if(pxEntity->GetGuid()<ASURA_GUID_TOOLSCREATEDENTITY_START || pxEntity->GetGuid()>ASURA_GUID_TOOLSCREATEDENTITY_END) continue;
		
		//is this a template entity?
		if(pxEntity->GetClassification() !=  AsuraEntityClass_Template) continue;

		Asura_ServerEntity_Template* pxTemplate = (Asura_ServerEntity_Template*) pxEntity;

		if(pxTemplate->m_bMaster)
		{
			//master template already exists
			//make sure it is named correctly (to handle files that get renamed in windows)
			pxTemplate->m_xName = xName;
			if(pxTemplate->ToolsData) pxTemplate->ToolsData->Name = xName;
			return;
		}
	}

	if(Tools_SaveAsura::GetFileType() != AsuraFile_Template)
	{
		Asura_Assert(0,"Should only be creating a master template entity when editing a template file");
		return;
	}

	//Create the tools data for the new template , taking the name from the filename
	Tools_Entity_Data* pxToolsData = new Tools_Entity_Data;
	pxToolsData->Name = xName;
	pxToolsData->Position.Zero();
	pxToolsData->m_bEntityModified = true;

	//create the asura entity
	Asura_ServerEntity_Template* pxTemplate = new Asura_ServerEntity_Template();
	pxTemplate->ToolsData = pxToolsData;
	pxTemplate->m_bMaster = true;
	pxTemplate->m_xName = pxToolsData->Name;
	
}


void Tools_Entity_TemplateMaster_Object::AddOverlayControls(class Asura_Overlay* pxOverlay,class Asura_Vector_2& xOffset)
{

	Asura_ServerEntity_Template* pxAsuraEntity = (Asura_ServerEntity_Template*) Entity;

	Asura_Dialog* pxDialog;

	//inputs
	pxDialog = new Asura_Dialog("Inputs :",xOffset);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Create",xOffset,CreateInput,(int)this);
	pxDialog->SetHelpText("Create a new input.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.x += 7.0f;

	pxDialog = new Asura_Function_Dialog("Delete",xOffset,DeleteInput,(int)this);
	pxDialog->SetIsEnabledFunction(CanDeleteInput);
	pxDialog->SetHelpText("Delete an input.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.x -= 7.0f;
	xOffset.y += Asura_Dialog::DefaultSpacing();

	//add controls for editing the input messages
	Asura_Assert(pxAsuraEntity->m_iNumInputs == pxAsuraEntity->m_iNumInputNames,"");
	for(int i=0;i<pxAsuraEntity->m_iNumInputNames;i++)
	{
		char szText[100];
		int iNumMessages = Asura_StaticMessage_System::GetNumberOfMessagesInBlock(pxAsuraEntity->m_pxInputs[i].m_usMessageBlock);
		Asura_PrintfToBfr(szText,"%s (%d)",100,true,pxAsuraEntity->m_pxInputNames[i].GetString(),iNumMessages);
		pxDialog = new Asura_Function_Dialog(szText,xOffset,EditInput,i);
		pxDialog->SetHelpText("Edit messages that should be sent when this input is triggered.");
		pxOverlay->AddDialog(pxDialog);
		xOffset.y+=Asura_Dialog::DefaultSpacing();
	
	}

	
	xOffset.y+=Asura_Dialog::SmallSpacing();

	//outputs
	pxDialog = new Asura_Dialog("Outputs :",xOffset);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Create",xOffset,CreateOutput,(int)this);
	pxDialog->SetHelpText("Create a new output.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.x += 7.0f;

	pxDialog = new Asura_Function_Dialog("Delete",xOffset,DeleteOutput,(int)this);
	pxDialog->SetIsEnabledFunction(CanDeleteOutput);
	pxDialog->SetHelpText("Delete an output.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.x -= 7.0f;
	xOffset.y += Asura_Dialog::DefaultSpacing();

	//add controls showing the output names
	for(int i=0;i<pxAsuraEntity->m_iNumOutputNames;i++)
	{
		pxDialog = new Asura_Dialog(pxAsuraEntity->m_pxOutputNames[i].GetString(),xOffset);
		pxDialog->SetHelpText("Output Names.\nMessages for these are edited in the instances of this template.");
		pxOverlay->AddDialog(pxDialog);
		xOffset.y+=Asura_Dialog::DefaultSpacing();
	
	}


}

void Tools_Entity_TemplateMaster_Object::UpdatePositionInAsuraData(bool bUpdateLinkedObjects)
{
	Asura_ServerEntity_Template* pxAsuraEntity = (Asura_ServerEntity_Template*) Entity;

	pxAsuraEntity->m_xPosition = Position;
	pxAsuraEntity->m_xOrient = Orientation;


	Tools_Entity_Object::UpdatePositionInAsuraData(bUpdateLinkedObjects);
}


void Tools_Entity_TemplateMaster_Object::UpdateChildListInEntity()
{
	//find the tools object (if it exists) and the server entity
	Asura_ServerEntity_Template* pxTemplateEntity = 0;
	Tools_Entity_TemplateMaster_Object* pxTemplateObject = 0;

	if(s_xObjectList.size())
	{
		pxTemplateObject = s_xObjectList.first_entry();
		pxTemplateEntity = (Asura_ServerEntity_Template*) pxTemplateObject->Entity;
	}
	else
	{
		for(Asura_Hashed_List_It<Asura_ServerEntity_Template> xIt(Asura_ServerEntity_Template::GetList());!xIt.Done();xIt.Next())
		{
			if(xIt.GetCurrent()->IsDestroyed()) continue;
			if(xIt.GetCurrent()->IsMaster())
			{
				pxTemplateEntity = xIt.GetCurrent();
				break;
			}
		}				
	}
	if(!pxTemplateEntity) return;

	UpdateChildListInEntity(pxTemplateEntity,pxTemplateObject,false);
}

void Tools_Entity_TemplateMaster_Object::UpdateChildListInEntity(Asura_ServerEntity_Template* pxTemplateEntity,Tools_Entity_TemplateMaster_Object* pxTemplateObject,bool bSelectedOnly)
{
	UpdateChildList_Entity(pxTemplateEntity,pxTemplateObject,bSelectedOnly);  
	UpdateChildList_Attractor(pxTemplateEntity,bSelectedOnly);  
	if(!bSelectedOnly)
	{
		UpdateChildList_Phonon(pxTemplateEntity);  
		UpdateChildList_Light(pxTemplateEntity);  
		UpdateChildList_Corona(pxTemplateEntity);  
		UpdateChildList_StreamingSound(pxTemplateEntity);  
	}
}

void Tools_Entity_TemplateMaster_Object::UpdateChildList_Entity(Asura_ServerEntity_Template* pxTemplateEntity,Tools_Entity_TemplateMaster_Object* pxTemplateObject,bool bSelectedOnly)
{
	//get a list of all server entites with tools guids , that aren't members of other template instances
	List<u_int> xIDList;
	List<Asura_Guid> xNewBadList;

	for(Asura_Classified_List_It<Asura_Entity_Server> it(Asura_Entity_Server::GetList());!it.Done();it.Next())
	{
		Asura_Entity_Server* pxEntity = it.GetCurrent();
		//ignore entities marked for destruction
		if(pxEntity->IsDestroyed()) continue;
		//and those with non-tools guids
		if(pxEntity->GetGuid()<ASURA_GUID_TOOLSCREATEDENTITY_START || pxEntity->GetGuid()>ASURA_GUID_TOOLSCREATEDENTITY_END) continue;
		
		if(pxEntity == pxTemplateEntity) continue;//don't add this template to its own list
		if(!pxEntity->ToolsData) continue;
		if(pxEntity->GetClassification() == AsuraEntityClass_CutsceneController) continue;//just say no to cutscenes
		if(pxEntity->ToolsData->IsMemberOfTemplate()) continue;

		if(bSelectedOnly)
		{
			#ifdef ASURA_USE_ATTRACTORS
			if(pxEntity->GetClassification() == AsuraEntityClass_AttractorController)
			{
				//only include attractor controller entities if at least one of the nodes is selected
				u_int uID = ((Asura_ServerEntity_AttractorController*) pxEntity)->GetToolsAttractorID();
				Tools_Attractor* pxAttractor = Tools_Attractor_System::GetInstance()->GetAttractorByID(uID);
				if(!pxAttractor) continue;
				if(!pxAttractor->AreAnyNodesSelected()) continue;
			}
			else
			#endif
			{
				//ignore entities that don't have a selected tools object
				Tools_Object* pxObject = pxEntity->ToolsData->ToolsObject;
				if(!pxObject) continue;
				if(!pxObject->IsGroupSelected()) continue;
			}
		}

		if(!pxEntity->CanBeIncludedInTemplate())
		{
			if(pxTemplateObject)
			{
				xNewBadList.add_entry(pxEntity->GetGuid());
				if(!pxTemplateObject->m_xBadGuidList.contains(pxEntity->GetGuid()))
				{
					//complain about this object , since this is the first time
					ShowMessageOverlay("Warning.\nThe entity type used by '%s' cannot be used in template files.",pxEntity->ToolsData->Name.GetString());

				}
			}
			else if(bSelectedOnly)
			{
				Tools_Timed_Message_System::AddMessage(0xffff0000, "%s cannot be included in template.",pxEntity->ToolsData->Name.GetString());
			}
			continue;
		}

		xIDList.add_entry(pxEntity->GetGuid());
	}

	if(pxTemplateObject)
	{
		pxTemplateObject->m_xBadGuidList = xNewBadList;
	}

	//update array in entity
	pxTemplateEntity->m_xChildIDHandler.SetIDList(Tools_EntityTemplate_ChildType_Entity,xIDList);
}

void Tools_Entity_TemplateMaster_Object::UpdateChildList_Phonon(Asura_ServerEntity_Template* pxTemplateEntity)
{
	//Search for guids of sounds that should belong to this
	List<u_int> xIDList;
	
	
	for(int i=0;i<Asura_Phonon_System::GetNumberOfPhonons();i++)
	{
		const Asura_Phonon* pxPhonon = Asura_Phonon_System::GetPhononByIndex(i);
		if(pxPhonon->IsMemberOfTemplateInstance()) continue;//ignore phonons that are members of other templaet instances
    	Asura_Assert(pxPhonon->GetGuid() != ASURA_GUID_UNREGISTERED,"All phonons should have valid guids by now?");
		xIDList.add_entry(pxPhonon->GetGuid());
	}
 
	//update array in entity
	pxTemplateEntity->m_xChildIDHandler.SetIDList(Tools_EntityTemplate_ChildType_Phonon,xIDList);
}

void Tools_Entity_TemplateMaster_Object::UpdateChildList_Light(Asura_ServerEntity_Template* pxTemplateEntity)
{
	//search for IDs of lights that should belong to this
	List<u_int> xIDList;

	Asura_TrackingHandlePool<Tools_PhotonMapping_LightData>::Iterator xIterator = Tools_PhotonMapping_System::GetLightDataTrackingHandlePool()->GetIterator();

	for (xIterator.Begin(); !xIterator.End(); xIterator.Next())
	{
		Tools_PhotonMapping_TemplateData* pxTemplateData = Tools_PhotonMapping_System::GetTemplateData(xIterator.GetCurrentHandle());
		if(pxTemplateData->m_uTemplateGuid != ASURA_GUID_UNREGISTERED) continue;//ignore lights that are members of other template instances
		if(!pxTemplateData->IsTemplateLight()) continue;

		xIDList.add_entry(pxTemplateData->m_uID);
	}
	//update array in entity
	pxTemplateEntity->m_xChildIDHandler.SetIDList(Tools_EntityTemplate_ChildType_Light,xIDList);

}

void Tools_Entity_TemplateMaster_Object::UpdateChildList_Corona(Asura_ServerEntity_Template* pxTemplateEntity)
{
	//search for IDs of corona that should belong to this
	List<u_int> xIDList;

	for(int i=0;i<Asura_StaticCorona_System::GetArraySize();i++)
	{
		if(!Asura_StaticCorona_System::IsValidCorona(i)) continue;
		Asura_StaticCorona* pxCorona = Asura_StaticCorona_System::GetCorona(i);
		if(pxCorona->m_uTemplateGuid != ASURA_GUID_UNREGISTERED) continue;//ignore coronas that are members of other template instances

		xIDList.add_entry(pxCorona->m_uID);
	}
	
	//update array in entity
	pxTemplateEntity->m_xChildIDHandler.SetIDList(Tools_EntityTemplate_ChildType_Corona,xIDList);
}

void Tools_Entity_TemplateMaster_Object::UpdateChildList_Attractor(Asura_ServerEntity_Template* pxTemplateEntity, bool bSelectedOnly)
{
	#ifdef ASURA_USE_ATTRACTORS
	//search for IDs of attractors that should belong to this
	List<u_int> xIDList;

	for(Asura_Hashed_List_It<Tools_Attractor> xIt(Tools_Attractor_System::GetInstance()->GetAttractorList());!xIt.Done();xIt.Next())
	{
		const Tools_Attractor* pxAttractor = xIt.GetCurrent();
		if(pxAttractor->GetTemplateGuid() != ASURA_GUID_UNREGISTERED) continue;//ignore attractors that are members of other template instances

		if(bSelectedOnly)
		{
			//only include attractors that have at least one selected node
			if(!pxAttractor->AreAnyNodesSelected()) continue;
		}

		xIDList.add_entry(pxAttractor->GetID());
	}
	
	//update array in entity
	pxTemplateEntity->m_xChildIDHandler.SetIDList(Tools_EntityTemplate_ChildType_Attractor,xIDList);
	#endif
}

void Tools_Entity_TemplateMaster_Object::UpdateChildList_StreamingSound(Asura_ServerEntity_Template* pxTemplateEntity)
{
	//search for IDs of sounds that should belong to this
	List<u_int> xIDList;

	for(int i=0; i<Asura_StreamingBackgroundSound_System::GetNumberOfSounds(); i++)
	{
		const Asura_StreamingBackgroundSound* pxSound = Asura_StreamingBackgroundSound_System::GetSound(i);
		if(pxSound->IsMemberOfTemplateInstance()) continue;//ignore sounds that are members of other template instances

		xIDList.add_entry(pxSound->m_uID);
	}
	
	//update array in entity
	pxTemplateEntity->m_xChildIDHandler.SetIDList(Tools_EntityTemplate_ChildType_StreamingSound,xIDList);
}



void Tools_Entity_TemplateMaster_Object::ExportSelectedAsTemplate()
{
	//check how many invalid objects are selected
	int iNonEntities = 0;
	int iInvalidEntities = 0;
	int iValidEntities = 0;

	for(CLIF<Tools_Object*> xObjectLIF(&Tools_Object_System::SelectedObjectList);!xObjectLIF.done();xObjectLIF.next())
	{
		if(xObjectLIF()->IsEntityObject())
		{
			Asura_Entity_Server* pxEntity = Asura_Entity_Server::FindEntity(xObjectLIF()->GetGuid());

			if(pxEntity && pxEntity->CanBeIncludedInTemplate())
			{
				iValidEntities++;
			}
			else
			{
				iInvalidEntities++;
			}
		
		}
		else if(xObjectLIF()->Type == TOT_AttractorNode)
		{
			iValidEntities++;
		}
		else if(!xObjectLIF()->SuppressCantExportToTemplateWarning())
		{
			iNonEntities++;
		}
	}

	if(!iValidEntities)
	{
		ShowMessageOverlay("No allowable entities have been selected for inclusion in the template.");	
		return;
	}
	if(iNonEntities || iInvalidEntities)
	{
		char szPart1[200] = "";
		if(iNonEntities)
		{
			Asura_PrintfToBfr(szPart1,"%d of the selected objects aren't entities.\n",sizeof(szPart1),true,iNonEntities);
		}
		char szPart2[200] = "";
		if(iInvalidEntities)
		{
			Asura_PrintfToBfr(szPart2,"%d of the selected entities aren't allowed in template files.\n",sizeof(szPart2),true,iInvalidEntities);
		}
		
		bool bAnswer = ShowYesNoOverlay("%s%sCreate the template from the remaining entities anyway?",szPart1,szPart2);
		if(!bAnswer) return;
	}

	enum
	{
		TEMPLATEOPTION_USE_SELECTED_POSITION_AND_ORIENT,
		TEMPLATEOPTION_USE_SELECTED_POSITION_ONLY,
		TEMPLATEOPTION_USE_BB_CENTRE,
		TEMPLATEOPTION_USE_WORLD_ORIGIN,
	};

	const Tools_Object* pxSelected = Tools_Object_System::SelectedObject;

	//ask how the template should be centred
	ListSelection_Overlay xOptionList("Select position for template centre :");
	if(pxSelected)
	{
		xOptionList.AddItem("Use primary selection's position AND orientation",TEMPLATEOPTION_USE_SELECTED_POSITION_AND_ORIENT,true);
		xOptionList.AddItem("Use primary selection's position",TEMPLATEOPTION_USE_SELECTED_POSITION_ONLY);
	}
	xOptionList.AddItem("Use centre of all selected objects",TEMPLATEOPTION_USE_BB_CENTRE);
	xOptionList.AddItem("Use world origin",TEMPLATEOPTION_USE_WORLD_ORIGIN);

	if(!xOptionList.DoModal()) return;
	int iOption = xOptionList.GetSelectionData();



	//ask for a template filename
	Tools_FileSystem_DialogInfo xInfo;

	xInfo.DefaultExtension = "AsuraTemplate";

	xInfo.DefaultFilename = "";
	xInfo.DefaultPath = "Templates";
	xInfo.FileFilter = "Template Files\0*.AsuraTemplate\0";
	xInfo.WindowTitle = "Save template file as...";

	const Asura_Char* szTemplateFilename = Tools_FileSystem::GetSaveFilename(xInfo);
	if(!szTemplateFilename) return;

	//make sure the file is in the templates directory
	szTemplateFilename = Asura_FileAccess::PlatformRemoveNetworkOrExecutableDir(szTemplateFilename);
	if(strnicmp(szTemplateFilename,"Templates\\",10))
	{
		ShowMessageOverlay("Template should be exported to the Templates directory (or a subdirectory of that)");
		return;
	}

	//get the template name , which is the filename without "Templates\\" and the extension
	char szTemplateName[200];
	Asura_PrintfToBfr(szTemplateName,"%s",200,true,szTemplateFilename+10);
	char* pcDot = strrchr(szTemplateName,'.');
	*pcDot = 0;

	//Make sure that saving the template using this name won't result in a template being included in itself
	for(CLIF<Tools_Object*> xObjectLIF(&Tools_Object_System::SelectedObjectList);!xObjectLIF.done();xObjectLIF.next())
	{
		//search for template instances
		if(xObjectLIF()->Type != TOT_Entity_TemplateInstance) continue;

		Tools_Entity_TemplateInstance_Object* pxTemplateInstance = (Tools_Entity_TemplateInstance_Object*) xObjectLIF();
		const char* szChildTemplateName = pxTemplateInstance->GetTemplateName();
		
		Tools_EntityTemplate* pxChildTemplate = Tools_EntityTemplate_System::GetTemplate(szChildTemplateName);
		List<const char*> xNameList;
		xNameList.add_entry(szChildTemplateName);
		if(!pxChildTemplate || !pxChildTemplate->GetDependencies(xNameList))
		{
			ShowMessageOverlay("Unable to determine the template dependencies for template %s , so cannot include instances of it in template file.",szChildTemplateName);
			return;
		}

		for(LIF<const char*> xNameLIF(&xNameList);!xNameLIF.done();xNameLIF.next())
		{
			if(!stricmp(xNameLIF(),szTemplateName))
			{
				ShowMessageOverlay("Cannot save the template using this name , because it would result in a template including itself.");
				return;
			}
		}
	}

	//get the position and orient for the template master
	Asura_Vector_3 xTemplatePosition = Asura_Maths::ZeroVector3;
	Asura_Matrix_3x3 xTemplateOrient = Asura_Maths::IdentityMatrix3x3;

	switch(iOption)
	{
		case TEMPLATEOPTION_USE_SELECTED_POSITION_AND_ORIENT:
			if(pxSelected) xTemplatePosition = pxSelected->Position;
			if(pxSelected) xTemplateOrient = pxSelected->Orientation; 
			break;
		case TEMPLATEOPTION_USE_SELECTED_POSITION_ONLY:
			if(pxSelected) xTemplatePosition = pxSelected->Position;
			break;

		case TEMPLATEOPTION_USE_BB_CENTRE :
		{
			Asura_Bounding_Box xBB;
			xBB.Reset();
			for(CLIF<Tools_Object*> xObjectLIF(&Tools_Object_System::SelectedObjectList);!xObjectLIF.done();xObjectLIF.next())
			{
				xBB.Set(xObjectLIF()->Position);
			}

			xTemplatePosition = xBB.CalculateCentre();

			break;
		}

		case TEMPLATEOPTION_USE_WORLD_ORIGIN :
		default :
			break;

	}



	//Create the tools data for the new template , taking the name from the filename
	Tools_Entity_Data* pxToolsData = new Tools_Entity_Data;
	pxToolsData->Name = szTemplateName;
	pxToolsData->Position = xTemplatePosition;
	pxToolsData->m_bEntityModified = true;

	//create the asura entity
	Asura_ServerEntity_Template* pxTemplateEntity = new Asura_ServerEntity_Template();
	pxTemplateEntity->ToolsData = pxToolsData;
	pxTemplateEntity->m_bMaster = true;
	pxTemplateEntity->m_xName = szTemplateName;
	pxTemplateEntity->m_iTemplateVersion = 1;


	pxTemplateEntity->m_xPosition = xTemplatePosition;
	pxTemplateEntity->m_xOrient = xTemplateOrient;


	//if the template file already exists , then find out the version number , to make sure it is different
	if(Tools_FileSystem::FileExists(szTemplateFilename))
	{
		Tools_EntityTemplate* pxTemplate = Tools_EntityTemplate_System::GetTemplate(szTemplateName);
		if(pxTemplate)
		{
			pxTemplateEntity->m_iTemplateVersion = pxTemplate->GetVersion()+1;
		}
	}

	//add all the selected objects to the template
	UpdateChildListInEntity(pxTemplateEntity,0,true);

	//save
	Tools_SaveAsura::SaveTemplateFile(pxTemplateEntity,szTemplateFilename);

	//and kill off the template master entity
	pxTemplateEntity->SetAsDestroyed();

	//Invalidate the loaded version of the template (if any)
	Tools_EntityTemplate_System::InvalidateTemplate(szTemplateName);


	//rebuild template overlay so that the new template appears in the list
	Tools_Entity_TemplateInstance_Object::CloseTemplateOverlay();
	Tools_Entity_TemplateInstance_Object::CreateTemplateOverlay();

}

bool Tools_Entity_TemplateMaster_Object::CreateInput()
{
	Asura_ServerEntity_Template* pxAsuraEntity = (Asura_ServerEntity_Template*) Entity;

	//get a name for the new input
	GetString_Overlay xOverlay("Enter name for the new input","");
	if(!xOverlay.DoModal()) return false;

	const char* pszName = xOverlay.GetString();
	if(!strlen(pszName)) return false;

	//make sure this name is okay
	Asura_Hash_ID uHash = Asura_GetHashID(pszName);
	for(int i=0;i<pxAsuraEntity->m_iNumInputs;i++)
	{
		if(pxAsuraEntity->m_pxInputs[i].m_uNameHash == uHash)
		{
			ShowMessageOverlay("An input with that name already exists.");
			return false;
		}
	}

	//create enlarged input arrays , and copy the old values
	Asura_TemplateEntityMessage* pxInputs = new Asura_TemplateEntityMessage[pxAsuraEntity->m_iNumInputs+1];
	Asura_String* pxInputNames = new Asura_String[pxAsuraEntity->m_iNumInputs+1];

	for(int i=0;i<pxAsuraEntity->m_iNumInputs;i++)
	{
		pxInputs[i] = pxAsuraEntity->m_pxInputs[i];
		pxInputNames[i] = pxAsuraEntity->m_pxInputNames[i];
	}

	//add the new input
	int iIndex = pxAsuraEntity->m_iNumInputs;
	pxInputs[iIndex].m_uNameHash = uHash;
	pxInputs[iIndex].m_usMessageBlock = ASURA_STATICMESSAGE_EMPTYBLOCK;
	pxInputNames[iIndex] = pszName;

	//and replace
	delete [] pxAsuraEntity->m_pxInputs;
	pxAsuraEntity->m_pxInputs = pxInputs;
	pxAsuraEntity->m_iNumInputs++;

	delete [] pxAsuraEntity->m_pxInputNames;
	pxAsuraEntity->m_pxInputNames = pxInputNames;
	pxAsuraEntity->m_iNumInputNames++;

	#ifdef ASURA_TOOLS_USE_NEW_MESSAGE_EDITOR
	Tools_MessageEditor::UpdateMessageEditorTable();
	#endif

	//overlay needs updating
	DeleteOverlay();
	CreateOverlay();

	SetEntityModified();

	return true;
}

bool Tools_Entity_TemplateMaster_Object::CanDeleteInput()
{
	Asura_Assert(ThisEntity->Type == TOT_Entity_TemplateMaster,"");
	Asura_ServerEntity_Template* pxAsuraEntity = (Asura_ServerEntity_Template*) ThisEntity->Entity;

	return pxAsuraEntity->m_iNumInputs!=0;	
}

void Tools_Entity_TemplateMaster_Object::DeleteInput()
{
	Asura_ServerEntity_Template* pxAsuraEntity = (Asura_ServerEntity_Template*) Entity;

	//ask which input to delete
	int iIndex = SelectInputIndex("Delete which input?");
	if(iIndex<0) return;

	//delete any messages associated with this input
	Asura_StaticMessage_System::DeleteMessageBlock(pxAsuraEntity->m_pxInputs[iIndex].m_usMessageBlock);

	//shuffle down the remaining inputs
	for(int i=iIndex;i<pxAsuraEntity->m_iNumInputs-1;i++)
	{
		pxAsuraEntity->m_pxInputs[i] = pxAsuraEntity->m_pxInputs[i+1];
		pxAsuraEntity->m_pxInputNames[i] = pxAsuraEntity->m_pxInputNames[i+1];
	}

	pxAsuraEntity->m_iNumInputs--;
	pxAsuraEntity->m_iNumInputNames--;

	#ifdef ASURA_TOOLS_USE_NEW_MESSAGE_EDITOR
	Tools_MessageEditor::UpdateMessageEditorTable();
	#endif

	//overlay needs updating
	DeleteOverlay();
	CreateOverlay();

	SetEntityModified();
  
}

void Tools_Entity_TemplateMaster_Object::CreateOutput()
{
	Asura_ServerEntity_Template* pxAsuraEntity = (Asura_ServerEntity_Template*) Entity;

	//get a name for the new output
	GetString_Overlay xOverlay("Enter name for the new output","");
	if(!xOverlay.DoModal()) return;

	const char* pszName = xOverlay.GetString();
	if(!strlen(pszName)) return;

	//make sure this name is okay
	Asura_Hash_ID uHash = Asura_GetHashID(pszName);
	for(int i=0;i<pxAsuraEntity->m_iNumOutputNames;i++)
	{
		if(Asura_GetHashID(pxAsuraEntity->m_pxOutputNames[i].GetString()) == uHash)
		{
			ShowMessageOverlay("An output with that name already exists.");
			return;
		}
	}

	//create enlarged output arrays , and copy the old values
	Asura_String* pxOutputNames = new Asura_String[pxAsuraEntity->m_iNumOutputNames+1];

	for(int i=0;i<pxAsuraEntity->m_iNumOutputNames;i++)
	{
		pxOutputNames[i] = pxAsuraEntity->m_pxOutputNames[i];
	}

	//add the new output
	int iIndex = pxAsuraEntity->m_iNumOutputNames;
	pxOutputNames[iIndex] = pszName;

	//and replace
	delete [] pxAsuraEntity->m_pxOutputNames;
	pxAsuraEntity->m_pxOutputNames = pxOutputNames;
	pxAsuraEntity->m_iNumOutputNames++;

	//overlay needs updating
	DeleteOverlay();
	CreateOverlay();

	SetEntityModified();
}

bool Tools_Entity_TemplateMaster_Object::CanDeleteOutput()
{
	Asura_Assert(ThisEntity->Type == TOT_Entity_TemplateMaster,"");
	Asura_ServerEntity_Template* pxAsuraEntity = (Asura_ServerEntity_Template*) ThisEntity->Entity;

	return pxAsuraEntity->m_iNumOutputNames!=0;	
}

void Tools_Entity_TemplateMaster_Object::DeleteOutput()
{
	Asura_ServerEntity_Template* pxAsuraEntity = (Asura_ServerEntity_Template*) Entity;

	//ask which output to delete
	int iIndex = SelectOutputIndex("Delete which output?");
	if(iIndex<0) return;

	//shuffle down the remaining outputs
	for(int i=iIndex;i<pxAsuraEntity->m_iNumOutputNames-1;i++)
	{
		pxAsuraEntity->m_pxOutputNames[i] = pxAsuraEntity->m_pxOutputNames[i+1];
	}

	pxAsuraEntity->m_iNumOutputNames--;

	//overlay needs updating
	DeleteOverlay();
	CreateOverlay();

	SetEntityModified();
}

void Tools_Entity_TemplateMaster_Object::EditInput(int iIndex)
{
	Asura_Assert(ThisEntity->Type == TOT_Entity_TemplateMaster,"");

	#ifdef ASURA_TOOLS_USE_NEW_MESSAGE_EDITOR
	Tools_MessageEditor::StartMessageEditingMode(ThisEntity->m_pxServerEntity->GetCustomEvent(iIndex));
	#else
	Asura_ServerEntity_Template* pxAsuraEntity = (Asura_ServerEntity_Template*) ThisEntity->Entity;
	Asura_Assert(iIndex>=0 && iIndex< pxAsuraEntity->m_iNumInputs,"");

	Tools_Object_System::EditMessageBlock(pxAsuraEntity->m_pxInputNames[iIndex].GetString(),&pxAsuraEntity->m_pxInputs[iIndex].m_usMessageBlock);
	#endif
}


u_short* Tools_Entity_TemplateMaster_Object::AddCustomEvent()
{
	if(CreateInput())
	{
		Asura_ServerEntity_Template* pxAsuraEntity = (Asura_ServerEntity_Template*) Entity;
		return &pxAsuraEntity->m_pxInputs[pxAsuraEntity->m_iNumInputs-1].m_usMessageBlock;
	}
	else
	{
		return NULL;
	}
}

/////////////////////////////////////////////////////////////////////////////////////////////////
/*--------------------------------------**
** Tools_Entity_TemplateInstance_Object **
**--------------------------------------*/

TOOLS_DECLARE_SHARED_PARAM(Asura_Hash_ID, 0xF5448E22,	Template_Skin)
TOOLS_DECLARE_SHARED_PARAM( bool,  uTOOLS_SHAREDPARAMHASH_PHYSICALOBJECT_MERGEWITHENV,      PhysicalObject_MergeWithEnv      );



List<Tools_Entity_TemplateInstance_Object*> Tools_Entity_TemplateInstance_Object::s_xObjectList;

Tools_Entity_TemplateInstance_Object::Tools_Entity_TemplateInstance_Object(Asura_Entity* pxEntity)
:Tools_Entity_Template_Object(pxEntity)
{
	s_xObjectList.add_entry(this);

	Asura_ServerEntity_Template* pxTemplate = (Asura_ServerEntity_Template*) Entity;
	Asura_Assert(!pxTemplate->m_bMaster,"Shouldn't be creating Tools_Entity_TemplateInstace_Object for template masters");


	Type = TOT_Entity_TemplateInstance;
	UpdateMesh();

	Position = pxTemplate->m_xPosition;
	Orientation = pxTemplate->m_xOrient;

	m_iInputListSize = 0;
	m_pszInputList = 0;
	m_iInputSelection = 0;

	m_iInputToTriggerListSize = 0;
	m_pszInputToTriggerList = 0;
	m_iInputToTriggerSelection = 0;
	m_pxInputToTriggerListDialog = 0;

	m_bHiddenForLOSAndRender = false;
	m_bForceUpdate = false;	
}

Tools_Entity_TemplateInstance_Object::~Tools_Entity_TemplateInstance_Object()
{
	s_xObjectList.delete_entry(this);
	DeleteInputToTriggerList();
	DeleteInputList();
	EmptyChildList();
}


void Tools_Entity_TemplateInstance_Object::UpdateMesh()
{
	Asura_ServerEntity_Template* pxTemplate = (Asura_ServerEntity_Template*) Entity;
	if(pxTemplate->ShouldNeverUpdate())
	{
		//use a different shape to show objects that have been split from their template
		Mesh = Asura_Mesh_System::GetMesh("ToolsShape_RedDiamond");
		if(Mesh) return;
	}
	
	Mesh = Asura_Mesh_System::GetMesh("ToolsShape_Diamond");
}

void Tools_Entity_TemplateInstance_Object::AddOverlayControls(class Asura_Overlay* pxOverlay,class Asura_Vector_2& xOffset)
{
	Tools_Entity_Object::AddOverlayControls(pxOverlay,xOffset);
	Asura_ServerEntity_Template* pxAsuraEntity = (Asura_ServerEntity_Template*) Entity;

	Asura_Dialog* pxDialog;

	//Merge with env tickbox
	pxDialog = new Asura_SharedTickbox_Dialog("Merge With Env", xOffset, &g_xParam_PhysicalObject_MergeWithEnv); 
	pxDialog->SetHelpText("Forces any physics objects within the template to have merge with environment set.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	//skin selection
	pxDialog = new Tools_Dialog_PickSkin("Skin : ", xOffset, &g_xParam_Template_Skin);
	pxDialog->SetHelpText("Causes any physical objects within the template to use this skin instead of the one defined in the template file.\nThis doesn't work for objects that have their skin set through a blueprint.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	//inputs
	pxDialog = new Asura_Dialog("Inputs :",xOffset);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	UpdateInputList();
	pxDialog = new Asura_List_Dialog(xOffset, m_pszInputList, &m_iInputSelection, Asura_Maths::Min(8,m_iInputListSize), m_iInputListSize, NULL);
	pxDialog->SetHelpText("Input Names.\nThese can't be edited  ,but you can send messages to this template to trigger them.\nDouble clicking on one will add it to the 'initial state' list ,  causing it to be triggered when the game starts.");
	((Asura_List_Dialog*)pxDialog)->SetDoubleClickFunction(AddTriggeredInput,(int)this);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y = pxDialog->m_xMaxCoord.y + 1.0f;
	
	
	//triggered inputs
	pxDialog = new Asura_Dialog("Initial State :",xOffset);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Add",xOffset,AddTriggeredInput,(int)this);
	pxDialog->SetIsEnabledFunction(CanAddTriggeredInput,(int)this);
	pxDialog->SetHelpText("Adds the input select above to the list to be triggered when the game starts.\nAny change to this list will result in the states being copied across to all selected instances that use the same template.");
	((Asura_Function_Dialog*)pxDialog)->SetUseSharedDialogColour(true);
	pxOverlay->AddDialog(pxDialog);
	xOffset.x += 7.0f;

	pxDialog = new Asura_Function_Dialog("Remove",xOffset,RemoveTriggeredInput,(int)this);
	pxDialog->SetIsEnabledFunction(CanRemoveTriggeredInput,(int)this);
	pxDialog->SetHelpText("Removes a triggered input from the list.\nAny change to this list will result in the states being copied across to all selected instances that use the same template.");
	((Asura_Function_Dialog*)pxDialog)->SetUseSharedDialogColour(true);
	pxOverlay->AddDialog(pxDialog);
	xOffset.x -= 7.0f;
	xOffset.y += Asura_Dialog::DefaultSpacing();

	m_pxInputToTriggerListDialog = 0;
	UpdateInputToTriggerList();
	pxDialog = m_pxInputToTriggerListDialog = new Asura_List_Dialog(xOffset, m_pszInputToTriggerList, &m_iInputToTriggerSelection, 6, m_iInputToTriggerListSize, NULL);
	pxDialog->SetHelpText("List of inputs that will be triggered when the game starts");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y = pxDialog->m_xMaxCoord.y + 1.0f;



	//outputs
	pxDialog = new Asura_Dialog("Outputs :",xOffset);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Add",xOffset,AddOutput,(int)this);
	pxDialog->SetIsEnabledFunction(CanAddOutput);
	pxDialog->SetHelpText("Add a message block for one of the templates outputs.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.x += 7.0f;

	pxDialog = new Asura_Function_Dialog("Remove",xOffset,RemoveOutput,(int)this);
	pxDialog->SetIsEnabledFunction(CanRemoveOutput);
	pxDialog->SetHelpText("Delete the message block for one of the outputs.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.x -= 7.0f;
	xOffset.y += Asura_Dialog::DefaultSpacing();

	//add controls showing the output names
	for(int i=0;i<pxAsuraEntity->m_iNumOutputs;i++)
	{
		char szText[100];
		int iNumMessages = Asura_StaticMessage_System::GetNumberOfMessagesInBlock(pxAsuraEntity->m_pxOutputs[i].m_usMessageBlock);
		Asura_PrintfToBfr(szText,"%s (%d)",100,true,GetOutputNameFromHash(pxAsuraEntity->m_pxOutputs[i].m_uNameHash),iNumMessages);
		pxDialog = new Asura_Function_Dialog(szText,xOffset,EditOutput,i);
		pxDialog->SetHelpText("Edit messages that should be sent when this output is triggered.");
		pxOverlay->AddDialog(pxDialog);
		xOffset.y+=Asura_Dialog::DefaultSpacing();
	}

	xOffset.y += Asura_Dialog::LargeSpacing() - Asura_Dialog::DefaultSpacing();



//	xOffset.y += Asura_Dialog::LargeSpacing()-Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_Function_Dialog("De-Templatify",xOffset,Detemplatify);
	pxDialog->SetIsEnabledFunction(CanDetemplatify);
	pxDialog->SetHelpText("Flags all selected template instances so that they will never get updated from their templates again.\nDoing this will allow the sub-entities to be edited as well.");
	pxOverlay->AddDialog(pxDialog);

}

void Tools_Entity_TemplateInstance_Object::UpdatePositionInAsuraData(bool bUpdateLinkedObjects)
{

	Asura_ServerEntity_Template* pxAsuraEntity = (Asura_ServerEntity_Template*) Entity;

	if(pxAsuraEntity->m_xPosition != Position ||
	   pxAsuraEntity->m_xOrient != Orientation)
	{
		//Set the version number in the instance back to zero , to force an update of all the
		//entities the next time we deal with such.
		//This is needed since a lot of the internal position related data (trigger volumes etc)
		//won't get updated correctly by simply moving the tools object.
		pxAsuraEntity->FlagForUpdate();
	}

	pxAsuraEntity->m_xPosition = Position;
	pxAsuraEntity->m_xOrient = Orientation;

	if(bUpdateLinkedObjects)
	{
		//update positions of all members of this template
		for(LIF<Tools_Object*> xChildLIF(&m_xChildList);!xChildLIF.done();xChildLIF.next())
		{
			xChildLIF()->UpdateRelative(this);
		}
	}

	Tools_Entity_Object::UpdatePositionInAsuraData(bUpdateLinkedObjects);
}

void Tools_Entity_TemplateInstance_Object::Destroy()
{
	DestroyChildren();

	Tools_Entity_Template_Object::Destroy();
}

void Tools_Entity_TemplateInstance_Object::UpdateFromTemplate()
{
	Asura_ServerEntity_Template* pxEntity = (Asura_ServerEntity_Template*) Entity;

	bool bUpdate = false;

	//make sure the template is available
	Tools_EntityTemplate* pxTemplate = Tools_EntityTemplate_System::GetTemplate(pxEntity->m_xName.GetString());
	Asura_Assertf(pxTemplate, ("Couldn't find template for object %s, Guid %i", pxEntity->m_xName.GetString(), pxEntity->GetGuid()));
	if(pxTemplate && !pxEntity->ShouldNeverUpdate())
	{
		//only update if version number doesn't match
		if(pxTemplate->GetVersion() != pxEntity->m_iTemplateVersion)
		{
			Tools_EntityTemplateInstanceMain xTempInstance(pxEntity,pxTemplate);

			//delete everything that belongs to this instance then
			DestroyChildren();

			//and update from the template
			pxTemplate->CreateTemplateMembers(&xTempInstance);
			bUpdate = true;
		}
	}

	FindChildren();

	if(bUpdate)
	{
		//also need to update any children template entities
		for(LIF<Tools_Object*> xChildLIF(&m_xChildList);!xChildLIF.done();xChildLIF.next())
		{
			xChildLIF()->OnUpdateFromTemplate();
			
			if(xChildLIF()->Type == TOT_Entity_TemplateInstance)
			{
				((Tools_Entity_TemplateInstance_Object*) xChildLIF())->UpdateFromTemplate();
			}
		}

		//invalidate the skybox objects , in case any were created
		Asura_Skybox::InvalidateSkyboxObjects();

		// finally ensure the non-tools child guids are up-to-date, 
		// and set the child template active flags correctly
		pxEntity->UpdateGuidListFromToolsData();
		pxEntity->UpdateChildrenTemplateActiveFlag();

		SetEntityModified();

		if(IsSelected())
		{
			DeleteOverlay();
			CreateOverlay();
		}
	}
}

void Tools_Entity_TemplateInstance_Object::OnUpdateFromTemplate()
{
	//parent template has been updated, so this needs updating as well
	UpdateFromTemplate();
}

void Tools_Entity_TemplateInstance_Object::DestroyChildren()
{
	Asura_ServerEntity_Template* pxTemplate = (Asura_ServerEntity_Template*) Entity;
	if(pxTemplate->ShouldNeverUpdate()) return;//if object doesn't update, then it has no control over its children anyway
	
	//delete all the children
	DestroyChildren_Entity();
	DestroyChildren_Phonon();
	DestroyChildren_Light();
	DestroyChildren_Corona();
	DestroyChildren_Attractor();
	DestroyChildren_StreamingSound();

	//clear out the child ID list
	pxTemplate->m_xChildIDHandler.Clear();


	//clear out the list of child tools objects
	EmptyChildList();
}

void Tools_Entity_TemplateInstance_Object::DestroyChildren_Entity()
{
	Asura_ServerEntity_Template* pxTemplate = (Asura_ServerEntity_Template*) Entity;
	
	//find the tools object for each child entity and destroy it
	const Tools_EntityTemplate_ChildIDArray* pxArray = pxTemplate->m_xChildIDHandler.GetArray(Tools_EntityTemplate_ChildType_Entity);
	if(!pxArray) return;

	for(int i=0;i<pxArray->m_iNumIDs;i++)
	{
		Tools_Object* pxObject = Tools_Object_System::GetObjectFromGuid(pxArray->m_puIDs[i]);
		if(pxObject)
		{
			pxObject->Destroy();
		}
		Asura_Entity_Server* pxServerEntity = Asura_Entity_Server::FindEntity(pxArray->m_puIDs[i]);
		if(pxServerEntity)
		{
			pxServerEntity->SetAsDestroyed();
		}
		Asura_Entity_Client* pxClientEntity = Asura_Entity_Client::FindEntity(pxArray->m_puIDs[i]);
		if(pxClientEntity)
		{
			pxClientEntity->SetAsDestroyed();
		}

		Tools_EntityTemplate_System::RecycleEntityGuid(pxArray->m_puIDs[i]);
	}
	Asura_Entity_System::PruneEntityLists();
}

void Tools_Entity_TemplateInstance_Object::DestroyChildren_Phonon()
{
	Asura_ServerEntity_Template* pxTemplate = (Asura_ServerEntity_Template*) Entity;
	
	//destroy child sounds
	const Tools_EntityTemplate_ChildIDArray* pxArray = pxTemplate->m_xChildIDHandler.GetArray(Tools_EntityTemplate_ChildType_Phonon);
	if(!pxArray) return;

	for(int i=0;i<pxArray->m_iNumIDs;i++)
	{
		Tools_Phonon_Object* pxObject = Tools_Phonon_Object::GetPhononObjectFromSoundGuid(pxArray->m_puIDs[i]);
		if(pxObject)
		{
			pxObject->Destroy();
		}
		else
		{
			Asura_Phonon* pxPhonon = Asura_Phonon_System::GetPhonon(pxArray->m_puIDs[i]);
			if(pxPhonon)
			{
				Asura_Phonon_System::DeallocatePhonon(pxPhonon);
			}
		}
		Tools_EntityTemplate_System::RecycleSoundGuid(pxArray->m_puIDs[i]);

	}
}

void Tools_Entity_TemplateInstance_Object::DestroyChildren_Light()
{
	Asura_ServerEntity_Template* pxTemplate = (Asura_ServerEntity_Template*) Entity;
	
	//destroy child lights
	const Tools_EntityTemplate_ChildIDArray* pxArray = pxTemplate->m_xChildIDHandler.GetArray(Tools_EntityTemplate_ChildType_Light);
	if(!pxArray) return;

	for(int i=0;i<pxArray->m_iNumIDs;i++)
	{
		Tools_Object* pxObject = Tools_PhotonMapping_LightSource_Object::GetLightObjectFromID(pxArray->m_puIDs[i]);
		if(pxObject)
		{
			pxObject->Destroy();
		}
		else
		{
			Asura_Handle iLightHandle = Tools_PhotonMapping_System::GetLightHandleFromID(pxArray->m_puIDs[i]);
			if(iLightHandle != ASURA_INVALID_HANDLE)
			{
				Tools_PhotonMapping_System::DestroyLightSource(iLightHandle);
				Tools_Mode_PhotonMapping::RelightEverything();
			}
		}
		Tools_EntityTemplate_System::RecycleLightID(pxArray->m_puIDs[i]);

	}
}

void Tools_Entity_TemplateInstance_Object::DestroyChildren_Corona()
{
	Asura_ServerEntity_Template* pxTemplate = (Asura_ServerEntity_Template*) Entity;

	//destroy child coronas
	const Tools_EntityTemplate_ChildIDArray* pxArray = pxTemplate->m_xChildIDHandler.GetArray(Tools_EntityTemplate_ChildType_Corona);
	if(!pxArray) return;

	for(int i=0;i<pxArray->m_iNumIDs;i++)
	{
		Tools_Object* pxObject = Tools_Corona_Object::GetCoronaObjectFromID(pxArray->m_puIDs[i]);
		if(pxObject)
		{
			pxObject->Destroy();
		}
		else
		{
			int iIndex = Asura_StaticCorona_System::GetCoronaIndexFromID(pxArray->m_puIDs[i]);
			if(iIndex >= 0)
			{
				Asura_StaticCorona_System::DeallocateCorona(iIndex);
			}
		}
		Tools_EntityTemplate_System::RecycleCoronaID(pxArray->m_puIDs[i]);

	}
}

void Tools_Entity_TemplateInstance_Object::DestroyChildren_Attractor()
{
	#ifdef ASURA_USE_ATTRACTORS
	Asura_ServerEntity_Template* pxTemplate = (Asura_ServerEntity_Template*) Entity;

	//destroy child attractors
	const Tools_EntityTemplate_ChildIDArray* pxArray = pxTemplate->m_xChildIDHandler.GetArray(Tools_EntityTemplate_ChildType_Attractor);
	if(!pxArray) return;

	for(int i=0;i<pxArray->m_iNumIDs;i++)
	{
		Tools_Attractor_System::GetInstance()->DeleteAttractorByID(pxArray->m_puIDs[i],true);
	}
	#endif
}

void Tools_Entity_TemplateInstance_Object::DestroyChildren_StreamingSound()
{
	Asura_ServerEntity_Template* pxTemplate = (Asura_ServerEntity_Template*) Entity;

	//destroy child sounds
	const Tools_EntityTemplate_ChildIDArray* pxArray = pxTemplate->m_xChildIDHandler.GetArray(Tools_EntityTemplate_ChildType_StreamingSound);
	if(!pxArray) return;

	for(int i=0;i<pxArray->m_iNumIDs;i++)
	{
		Tools_Object* pxObject = Tools_StreamingBackgroundSound_Object::GetSoundObjectFromID(pxArray->m_puIDs[i]);
		if(pxObject)
		{
			pxObject->Destroy();
		}
		else
		{
			int iIndex = Asura_StreamingBackgroundSound_System::GetSoundIndexFromID(pxArray->m_puIDs[i]);
			if(iIndex >= 0)
			{
				Asura_StreamingBackgroundSound_System::DeleteSound(iIndex);
			}
		}
	}
}





void Tools_Entity_TemplateInstance_Object::ReferencedObjectIsBeingDeleted(Tools_Object* pxObject)
{
	PARENT::ReferencedObjectIsBeingDeleted(pxObject);

	m_xChildList.delete_entry_if_exists(pxObject);
}


void Tools_Entity_TemplateInstance_Object::FindChildren()
{
	EmptyChildList();

	Asura_ServerEntity_Template* pxTemplate = (Asura_ServerEntity_Template*) Entity;
	if(pxTemplate->ShouldNeverUpdate()) return;
	
	FindChildren_Entity();
	FindChildren_Phonon();
	FindChildren_Light();
	FindChildren_Corona();
	FindChildren_Attractor();
	FindChildren_StreamingSound();

	UpdateTriggerVisibility();
}

void Tools_Entity_TemplateInstance_Object::FindChildren_Entity()
{
	Asura_ServerEntity_Template* pxTemplate = (Asura_ServerEntity_Template*) Entity;
	const Tools_EntityTemplate_ChildIDArray* pxArray = pxTemplate->m_xChildIDHandler.GetArray(Tools_EntityTemplate_ChildType_Entity);
	if(!pxArray) return;


	//find the tools object for each child
	for(int i=0;i<pxArray->m_iNumIDs;i++)
	{
		Tools_Object* pxObject = Tools_Object_System::GetObjectFromGuid(pxArray->m_puIDs[i]);
		if(pxObject)
		{
			AddChild(pxObject);
		}
	}
}

void Tools_Entity_TemplateInstance_Object::FindChildren_Phonon()
{
	Asura_ServerEntity_Template* pxTemplate = (Asura_ServerEntity_Template*) Entity;
	const Tools_EntityTemplate_ChildIDArray* pxArray = pxTemplate->m_xChildIDHandler.GetArray(Tools_EntityTemplate_ChildType_Phonon);
	if(!pxArray) return;


	//and the tools objects for any sounds
	for(int i=0;i<pxArray->m_iNumIDs;i++)
	{
		Tools_Object* pxObject = Tools_Phonon_Object::GetPhononObjectFromSoundGuid(pxArray->m_puIDs[i]);
		//Need to make sure objects don't get added twice , since any sounds with sound controllers
		//will already have been added when dealing with the child entities
		if(pxObject && !m_xChildList.contains(pxObject))
		{
			AddChild(pxObject);
		}
	}
}

void Tools_Entity_TemplateInstance_Object::FindChildren_Light()
{
	Asura_ServerEntity_Template* pxTemplate = (Asura_ServerEntity_Template*) Entity;
	const Tools_EntityTemplate_ChildIDArray* pxArray = pxTemplate->m_xChildIDHandler.GetArray(Tools_EntityTemplate_ChildType_Light);
	if(!pxArray) return;


	//find the tools objects for any lights
	for(int i=0;i<pxArray->m_iNumIDs;i++)
	{
		Tools_PhotonMapping_LightSource_Object* pxObject = Tools_PhotonMapping_LightSource_Object::GetLightObjectFromID(pxArray->m_puIDs[i]);
		if(pxObject)
		{
			AddChild(pxObject);
		}
	}
}

void Tools_Entity_TemplateInstance_Object::FindChildren_Corona()
{
	Asura_ServerEntity_Template* pxTemplate = (Asura_ServerEntity_Template*) Entity;
	const Tools_EntityTemplate_ChildIDArray* pxArray = pxTemplate->m_xChildIDHandler.GetArray(Tools_EntityTemplate_ChildType_Corona);
	if(!pxArray) return;


	//find the tools objects for any coronas
	for(int i=0;i<pxArray->m_iNumIDs;i++)
	{
		Tools_Object* pxObject = Tools_Corona_Object::GetCoronaObjectFromID(pxArray->m_puIDs[i]);
		if(pxObject)
		{
			AddChild(pxObject);
		}
	}
}

void Tools_Entity_TemplateInstance_Object::FindChildren_Attractor()
{
	#ifdef ASURA_USE_ATTRACTORS
	Asura_ServerEntity_Template* pxTemplate = (Asura_ServerEntity_Template*) Entity;
	const Tools_EntityTemplate_ChildIDArray* pxArray = pxTemplate->m_xChildIDHandler.GetArray(Tools_EntityTemplate_ChildType_Attractor);
	if(!pxArray) return;


	//find the tools objects for any attractors
	for(int i=0;i<pxArray->m_iNumIDs;i++)
	{
		Tools_Attractor* pxAttractor = Tools_Attractor_System::GetInstance()->GetAttractorByID(pxArray->m_puIDs[i]);
		if(pxAttractor)
		{
			Tools_Attractor_Node* pxNode = pxAttractor->GetFirstNode();
			while(pxNode)
			{
				if(pxNode->GetToolsNode()) AddChild(pxNode->GetToolsNode());
				pxNode = pxNode->GetNext();
			}
		}

	}
	#endif
}

void Tools_Entity_TemplateInstance_Object::FindChildren_StreamingSound()
{
	Asura_ServerEntity_Template* pxTemplate = (Asura_ServerEntity_Template*) Entity;
	const Tools_EntityTemplate_ChildIDArray* pxArray = pxTemplate->m_xChildIDHandler.GetArray(Tools_EntityTemplate_ChildType_StreamingSound);
	if(!pxArray) return;


	//find the tools objects for any sounds
	for(int i=0;i<pxArray->m_iNumIDs;i++)
	{
		Tools_StreamingBackgroundSound_Object* pxSound = Tools_StreamingBackgroundSound_Object::GetSoundObjectFromID(pxArray->m_puIDs[i]);
		if(pxSound)
		{
			AddChild(pxSound);
		}
	}
}

void Tools_Entity_TemplateInstance_Object::EmptyChildList()
{
	while(m_xChildList.size())
	{
		m_xChildList.first_entry()->RemoveReference(this);
		m_xChildList.delete_first_entry();
	}
}



void Tools_Entity_TemplateInstance_Object::AddChild(Tools_Object* pxObject)
{
	if(pxObject && !m_xChildList.contains(pxObject))
	{
   		pxObject->CalculateRelativePosition(this);
   		pxObject->SetObjectToSelectInstead(this);
		pxObject->AddReference(this);
   		m_xChildList.add_entry(pxObject);

		pxObject->AddChildObjectsToTemplate(this);
	}
}

void Tools_Entity_TemplateInstance_Object::UpdateAllTemplates()
{
	for(LIF<Tools_Entity_TemplateInstance_Object*> xObjectLIF(&s_xObjectList);!xObjectLIF.done();xObjectLIF.next())
	{
		//xObjectLIF()->FindChildren();
		xObjectLIF()->UpdateFromTemplate();
	}

	//Check for entities that think they are in a template but have no parent template instance
	for(Asura_Classified_List_It<Asura_Entity_Server> xIt(Asura_Entity_Server::GetList());!xIt.Done();xIt.Next())
	{
		Asura_Entity_Server* pxEntity = xIt.GetCurrent();
		//ignore entities marked for destruction
		if(pxEntity->IsDestroyed()) continue;
		//and those with non-tools guids
		if(pxEntity->GetGuid()<ASURA_GUID_TOOLSCREATEDENTITY_START || pxEntity->GetGuid()>ASURA_GUID_TOOLSCREATEDENTITY_END) continue;

		if(pxEntity->ToolsData && pxEntity->ToolsData->m_uParentTemplate != ASURA_GUID_UNREGISTERED)
		{
			//This entity thinks it is part of a template. Make sure that the template instance exists
			if(!Asura_ServerEntity_Template::FindEntity(pxEntity->ToolsData->m_uParentTemplate))
			{
				//Flag as no longer being part of a template
				pxEntity->ToolsData->m_uParentTemplate = ASURA_GUID_UNREGISTERED;
			}
		}
	}

	//Do a check for shared message blocks , since it can occur if there are errors
	//in the entity code for handling inclusion in templates.
	Tools_Object_System::CheckForMessageBlockOwnershipProblems();
}

bool Tools_Entity_TemplateInstance_Object::CanAddOutput()
{
	Asura_Assert(ThisEntity->Type == TOT_Entity_TemplateInstance,"");
	Asura_ServerEntity_Template* pxAsuraEntity = (Asura_ServerEntity_Template*) ThisEntity->Entity;

	//check for any unused outputs
	for(int i=0;i<pxAsuraEntity->m_iNumOutputNames;i++)
	{
		bool bAdd = true;
		//make sure this output isn't already used
		Asura_Hash_ID uHash = Asura_GetHashID(pxAsuraEntity->m_pxOutputNames[i].GetString());
		for(int j=0;j<pxAsuraEntity->m_iNumOutputs;j++)
		{
			if(pxAsuraEntity->m_pxOutputs[j].m_uNameHash == uHash)
			{
				bAdd = false;
				break;
			}
		}
		if(bAdd)
		{
			return true;
		}
	}
	return false;
}


bool Tools_Entity_TemplateInstance_Object::AddOutput()
{
	Asura_ServerEntity_Template* pxAsuraEntity = (Asura_ServerEntity_Template*) Entity;

	//Need to ask which output to add
	//Build a list of unused outputs
	ListSelection_Overlay xOverlay("Select output to add to this template instance");
	for(int i=0;i<pxAsuraEntity->m_iNumOutputNames;i++)
	{
		bool bAdd = true;
		//make sure this output isn't already used
		Asura_Hash_ID uHash = Asura_GetHashID(pxAsuraEntity->m_pxOutputNames[i].GetString());
		for(int j=0;j<pxAsuraEntity->m_iNumOutputs;j++)
		{
			if(pxAsuraEntity->m_pxOutputs[j].m_uNameHash == uHash)
			{
				bAdd = false;
				break;
			}
		}
		if(bAdd)
		{
			xOverlay.AddItem(pxAsuraEntity->m_pxOutputNames[i].GetString(),uHash);
		}
	}

	if(!xOverlay.GetNumItems())
	{
		ShowMessageOverlay("All available outputs have already been added.");
		return false;
	}

	if(!xOverlay.DoModal()) return false;

	//get the id of the output to add
	Asura_Hash_ID uHash = (Asura_Hash_ID)xOverlay.GetSelectionData();	


	//create enlarged output arrays , and copy the old values
	Asura_TemplateEntityMessage* pxOutputs = new Asura_TemplateEntityMessage[pxAsuraEntity->m_iNumOutputs+1];

	for(int i=0;i<pxAsuraEntity->m_iNumOutputs;i++)
	{
		pxOutputs[i] = pxAsuraEntity->m_pxOutputs[i];
	}

	//add the new output
	int iIndex = pxAsuraEntity->m_iNumOutputs;
	pxOutputs[iIndex].m_uNameHash = uHash;
	pxOutputs[iIndex].m_usMessageBlock = ASURA_STATICMESSAGE_EMPTYBLOCK;

	//and replace
	delete [] pxAsuraEntity->m_pxOutputs;
	pxAsuraEntity->m_pxOutputs = pxOutputs;
	pxAsuraEntity->m_iNumOutputs++;

	#ifdef ASURA_TOOLS_USE_NEW_MESSAGE_EDITOR
	Tools_MessageEditor::UpdateMessageEditorTable();
	#endif

	//overlay needs updating
	DeleteOverlay();
	CreateOverlay();

	SetEntityModified();

	return true;
}

u_short* Tools_Entity_TemplateInstance_Object::AddCustomEvent()
{
	if(AddOutput())
	{
		Asura_ServerEntity_Template* pxAsuraEntity = (Asura_ServerEntity_Template*) Entity;
		return &pxAsuraEntity->m_pxOutputs[pxAsuraEntity->m_iNumOutputs-1].m_usMessageBlock;
	}
	else
	{
		return NULL;
	}
}

bool Tools_Entity_TemplateInstance_Object::CanRemoveOutput()
{
	Asura_Assert(ThisEntity->Type == TOT_Entity_TemplateInstance,"");
	Asura_ServerEntity_Template* pxAsuraEntity = (Asura_ServerEntity_Template*) ThisEntity->Entity;

	return pxAsuraEntity->m_iNumOutputs!=0;	
}

void Tools_Entity_TemplateInstance_Object::RemoveOutput()
{
	Asura_ServerEntity_Template* pxAsuraEntity = (Asura_ServerEntity_Template*) Entity;

	if(!pxAsuraEntity->m_iNumOutputs) return;

	//ask which output to delete
	ListSelection_Overlay xOverlay("Remove which output?");
	for(int i=0;i<pxAsuraEntity->m_iNumOutputs;i++)
	{
		xOverlay.AddItem(GetOutputNameFromHash(pxAsuraEntity->m_pxOutputs[i].m_uNameHash),i);
	}

	if(!xOverlay.DoModal()) return;
	int iIndex = xOverlay.GetSelectionData();

	//delete any messages associated with this output
	Asura_StaticMessage_System::DeleteMessageBlock(pxAsuraEntity->m_pxOutputs[iIndex].m_usMessageBlock);

	//shuffle down the remaining outputs
	for(int i=iIndex;i<pxAsuraEntity->m_iNumOutputs-1;i++)
	{
		pxAsuraEntity->m_pxOutputs[i] = pxAsuraEntity->m_pxOutputs[i+1];
	}

	pxAsuraEntity->m_iNumOutputs--;

	#ifdef ASURA_TOOLS_USE_NEW_MESSAGE_EDITOR
	Tools_MessageEditor::UpdateMessageEditorTable();
	#endif
	
	//overlay needs updating
	DeleteOverlay();
	CreateOverlay();
		   
	SetEntityModified();
}

void Tools_Entity_TemplateInstance_Object::EditOutput(int iIndex)
{
	Asura_Assert(ThisEntity->Type == TOT_Entity_TemplateInstance,"");

	#ifdef ASURA_TOOLS_USE_NEW_MESSAGE_EDITOR
	Tools_MessageEditor::StartMessageEditingMode(ThisEntity->m_pxServerEntity->GetCustomEvent(iIndex));
	#else
	Asura_ServerEntity_Template* pxAsuraEntity = (Asura_ServerEntity_Template*) ThisEntity->Entity;
	Tools_Entity_TemplateInstance_Object* pxObject = (Tools_Entity_TemplateInstance_Object*) ThisEntity;

	Asura_Assert(iIndex>=0 && iIndex< pxAsuraEntity->m_iNumOutputs,"");

	const char* pszName = pxObject->GetOutputNameFromHash(pxAsuraEntity->m_pxOutputs[iIndex].m_uNameHash);
	Tools_Object_System::EditMessageBlock(pszName,&pxAsuraEntity->m_pxOutputs[iIndex].m_usMessageBlock);
	#endif
}

void Tools_Entity_TemplateInstance_Object::Detemplatify()
{
	//confirm
	if(!ShowYesNoOverlay("Would you like to split the selected template instances from their templates?.\nDoing so will allow the sub-entities to be edited , but prevent these objects from ever being updated from their templates again."))
	{
		return;
	}

	List<Tools_Object*> xSelectedList;
	Tools_Object_System::ListSelectedObjectsOfType(TOT_Entity_TemplateInstance,xSelectedList);

	for(LIF<Tools_Object*> xSelectedLIF(&xSelectedList);!xSelectedLIF.done();xSelectedLIF.next())
	{
		Tools_Entity_TemplateInstance_Object* pxObject = (Tools_Entity_TemplateInstance_Object*) xSelectedLIF();
		Asura_ServerEntity_Template* pxEntity = (Asura_ServerEntity_Template*) pxObject->Entity;

		if(pxEntity->ShouldNeverUpdate()) continue;//already detemplatified

		//Do an update from the template before the detemplatification , in case it is out of date from
		//the template instance being moved.
		pxObject->UpdateFromTemplate();

		//tell all the children that they can be selected directly again
		while(pxObject->m_xChildList.size())
		{
			pxObject->m_xChildList.first_entry()->SetHiddenFlag( ObjectHideFlag_TemplateTriggers, false );
			pxObject->m_xChildList.first_entry()->SetObjectToSelectInstead(0);
			pxObject->m_xChildList.first_entry()->RemoveReference(pxObject);

			// Make sure the child is marked as having an active template
			if(pxObject->m_xChildList.first_entry()->IsEntityObject())
			{
				Tools_Entity_Object* pxEntityObject = (Tools_Entity_Object*) pxObject->m_xChildList.first_entry();

				pxEntityObject->Entity->UpdateTemplateInstanceActive( true );
			}

			pxObject->m_xChildList.delete_first_entry();
		}

		const char* szEntityFile = pxEntity->ToolsData->m_xSecondaryEntityFilename.GetString();
		const char* szFolder = pxEntity->ToolsData->m_xEntityBrowserPath.GetString();
		if((szEntityFile && *szEntityFile) || (szFolder && *szFolder))
		{
			//attempt to transfer all the entities to the same file as the template
			const Tools_EntityTemplate_ChildIDArray* pxArray = pxEntity->m_xChildIDHandler.GetArray(Tools_EntityTemplate_ChildType_Entity);
			if(pxArray)
			{
				for(int i=0;i<pxArray->m_iNumIDs;i++)
				{
					Tools_Object* pxChildObject = Tools_Object_System::GetObjectFromGuid(pxArray->m_puIDs[i]);
					if(pxChildObject)
					{
						Tools_SecondaryEntityFile_System::AddToFile(pxChildObject, szEntityFile, szFolder);
					}
				}
			}
		}
		pxObject->UnflagTemplateInstanceMembers();

		pxEntity->FlagForNeverUpdate();

		pxObject->SetEntityModified();
		
		//If this object has no inputs or outputs , then we can destroy it since it has no further use
		if(!pxEntity->m_iNumInputNames && !pxEntity->m_iNumOutputNames)
		{
			Tools_Object_System::DeselectObject(pxObject);
			pxObject->Destroy();
		}
		else
		{
			pxObject->UpdateMesh();
		}

	}
}

bool Tools_Entity_TemplateInstance_Object::CanDetemplatify()
{
	Asura_Assert(ThisEntity->Type == TOT_Entity_TemplateInstance,"");
	Asura_ServerEntity_Template* pxAsuraEntity = (Asura_ServerEntity_Template*) ThisEntity->Entity;

	return !pxAsuraEntity->ShouldNeverUpdate();
}


void Tools_Entity_TemplateInstance_Object::PrintInfo(float& x,float& y)
{
	Tools_Entity_Template_Object::PrintInfo(x,y);

	Asura_ServerEntity_Template* pxAsuraEntity = (Asura_ServerEntity_Template*) Entity;

	//show the name of the template used
	Asura_Text_Output::Print(x,y++,0xffffffff,0,"Template : %s",pxAsuraEntity->m_xName.GetString());
}

Tools_Object* Tools_Entity_TemplateInstance_Object::CreateObjectOfThisType() const
{
	return CreateInstance(GetTemplateName());
}

void Tools_Entity_TemplateInstance_Object::Copy(const Tools_Object* pxObject)
{
	Tools_Entity_Template_Object::Copy(pxObject);

	const Tools_Entity_TemplateInstance_Object* pxFromObject = (const Tools_Entity_TemplateInstance_Object*) pxObject;

	Asura_ServerEntity_Template* pxFromEntity = (Asura_ServerEntity_Template*) pxFromObject->Entity;
	Asura_ServerEntity_Template* pxToEntity = (Asura_ServerEntity_Template*) Entity;

	//update the template name , and make all the objects again
	pxToEntity->m_xName = pxFromEntity->m_xName;
	pxToEntity->m_uReplacementSkinID = pxFromEntity->m_uReplacementSkinID;
	pxToEntity->FlagForUpdate();//force update
	UpdateFromTemplate();

	//destroy old outputs
	for(int i=0;i<pxToEntity->m_iNumOutputs;i++)
	{
		Asura_StaticMessage_System::DeleteMessageBlock(pxToEntity->m_pxOutputs[i].m_usMessageBlock);
	}
	delete [] pxToEntity->m_pxOutputs;
	pxToEntity->m_pxOutputs = 0;

	//copy outputs from other entity
	pxToEntity->m_iNumOutputs = pxFromEntity->m_iNumOutputs;

	if(pxToEntity->m_iNumOutputs)
	{
		pxToEntity->m_pxOutputs = new Asura_TemplateEntityMessage[pxToEntity->m_iNumOutputs];
		for(int i=0;i<pxToEntity->m_iNumOutputs;i++)
		{
			pxToEntity->m_pxOutputs[i].m_uNameHash = pxFromEntity->m_pxOutputs[i].m_uNameHash;
			pxToEntity->m_pxOutputs[i].m_usMessageBlock = Asura_StaticMessage_System::CopyMessageBlock(pxFromEntity->m_pxOutputs[i].m_usMessageBlock,pxFromObject->GetGuid(),GetGuid());
		}
	}

	//Copy initially triggered inputs
	u_int uNumInputsToTrigger = pxFromEntity->m_xInitialInputsToTrigger.Size();
	pxToEntity->m_xInitialInputsToTrigger.Resize(uNumInputsToTrigger);
	for(u_int i=0; i<uNumInputsToTrigger; i++)
	{
		pxToEntity->m_xInitialInputsToTrigger[i] = pxFromEntity->m_xInitialInputsToTrigger[i];
	}

	Tools_Object_System::CheckForMessageBlockOwnershipProblems();
}

void Tools_Entity_TemplateInstance_Object::DeleteDisowned_Phonon()
{
	//Check for any phonons that aren't whose parent doesn't know about them
	//and delete them.
	//This may happen if the sound/entity files got out of sync

	//Asura_Assert(!Tools_Mode_Sound::PhononList.size(),"Not expecting phonon objects to exist at this point")


	for(int i=0;i<Asura_Phonon_System::GetNumberOfPhonons();i++)
	{
		Asura_Phonon* pxPhonon = Asura_Phonon_System::GetPhononByIndex(i);
		if(!pxPhonon->IsMemberOfTemplateInstance()) continue;

		//find the template entity for this phonon
		bool bNeedToDelete = true;
		Asura_ServerEntity_Template* pxEntity = Asura_ServerEntity_Template::FindEntity(pxPhonon->GetTemplateGuid());
		if(pxEntity)
		{
			//make sure the entity agrees about owning this sound
			bNeedToDelete = !pxEntity->m_xChildIDHandler.Contains(Tools_EntityTemplate_ChildType_Phonon,pxPhonon->GetGuid());
		}

		if(bNeedToDelete)
		{
			Asura_Phonon_System::DeallocatePhonon(pxPhonon);
			i--;//need to go back one after deleting , since a new phonon will now occupy this slot
		}
	}
}

void Tools_Entity_TemplateInstance_Object::DeleteDisowned_Light()
{
	//Check for any lights whose parent doesn't know about them
	//and delete them.
	//This may happen if the light/entity files got out of sync

	List<u_int> xToBeDeleted;
	
	Asura_TrackingHandlePool<Tools_PhotonMapping_LightData>::Iterator xIterator = Tools_PhotonMapping_System::GetLightDataTrackingHandlePool()->GetIterator();

	for (xIterator.Begin(); !xIterator.End(); xIterator.Next())
	{
		Tools_PhotonMapping_TemplateData* pxLightTemplate = Tools_PhotonMapping_System::GetTemplateData(xIterator.GetCurrentHandle());
		if(!pxLightTemplate) continue;
		if(pxLightTemplate->m_uTemplateGuid == ASURA_GUID_UNREGISTERED) continue;

		//find the template entity for this light
		bool bNeedToDelete = true;
		Asura_ServerEntity_Template* pxEntity = Asura_ServerEntity_Template::FindEntity(pxLightTemplate->m_uTemplateGuid);
		if(pxEntity)
		{
			bNeedToDelete = !pxEntity->m_xChildIDHandler.Contains(Tools_EntityTemplate_ChildType_Light,pxLightTemplate->m_uID);
		}

		if(bNeedToDelete)
		{
			xToBeDeleted.add_entry(pxLightTemplate->m_uID);
		}
	}

	//delete the lights marked for deletion
	while(xToBeDeleted.size())
	{
		u_int uID = xToBeDeleted.first_entry();
		xToBeDeleted.delete_first_entry();

		Tools_Object* pxObject = Tools_PhotonMapping_LightSource_Object::GetLightObjectFromID(uID);
		if(pxObject)
		{
			pxObject->Destroy();
		}
		else
		{
			Asura_Handle iLightHandle = Tools_PhotonMapping_System::GetLightHandleFromID(uID);
			if(iLightHandle != ASURA_INVALID_HANDLE)
			{
				Tools_PhotonMapping_System::DestroyLightSource(iLightHandle);
				Tools_Mode_PhotonMapping::RelightEverything();
			}
		}
	}

}

void Tools_Entity_TemplateInstance_Object::DeleteDisowned_Corona()
{
	//Check for any corona whose parent doesn't know about them
	//and delete them.
	//This may happen if the light/entity files got out of sync

	List<u_int> xToBeDeleted;
	
	for(int i=0;i<Asura_StaticCorona_System::GetArraySize();i++)
	{
		if(!Asura_StaticCorona_System::IsValidCorona(i)) continue;

		Asura_StaticCorona* pxCorona = Asura_StaticCorona_System::GetCorona(i);
		if(!pxCorona) continue;
		if(pxCorona->m_uTemplateGuid == ASURA_GUID_UNREGISTERED) continue;

		//find the template entity for this corona
		bool bNeedToDelete = true;
		Asura_ServerEntity_Template* pxEntity = Asura_ServerEntity_Template::FindEntity(pxCorona->m_uTemplateGuid);
		if(pxEntity)
		{
			bNeedToDelete = !pxEntity->m_xChildIDHandler.Contains(Tools_EntityTemplate_ChildType_Corona,pxCorona->m_uID);
		}

		if(bNeedToDelete)
		{
			xToBeDeleted.add_entry(pxCorona->m_uID);
		}
	}

	//delete the coronas marked for deletion
	while(xToBeDeleted.size())
	{
		u_int uID = xToBeDeleted.first_entry();
		xToBeDeleted.delete_first_entry();

		Tools_Object* pxObject = Tools_Corona_Object::GetCoronaObjectFromID(uID);
		if(pxObject)
		{
			pxObject->Destroy();
		}
		else
		{
			int iIndex = Asura_StaticCorona_System::GetCoronaIndexFromID(uID);
			if(iIndex >= 0)
			{
				Asura_StaticCorona_System::DeallocateCorona(iIndex);
			}
		}
	}
}

void Tools_Entity_TemplateInstance_Object::DeleteDisowned_Attractor()
{
	#ifdef ASURA_USE_ATTRACTORS
	//Check for any attractor whose parent doesn't know about them
	//and delete them.
	//This may happen if the attractor/entity files got out of sync

	List<u_int> xToBeDeleted;
	
	for(Asura_Hashed_List_It<Tools_Attractor> xIt(Tools_Attractor_System::GetInstance()->GetAttractorList());!xIt.Done();xIt.Next())
	{
		const Tools_Attractor* pxAttractor = xIt.GetCurrent();
		if(pxAttractor->GetTemplateGuid() == ASURA_GUID_UNREGISTERED) continue;

		//find the template entity for this attractor
		bool bNeedToDelete = true;
		Asura_ServerEntity_Template* pxEntity = Asura_ServerEntity_Template::FindEntity(pxAttractor->GetTemplateGuid());
		if(pxEntity)
		{
			bNeedToDelete = !pxEntity->m_xChildIDHandler.Contains(Tools_EntityTemplate_ChildType_Attractor,pxAttractor->GetID());
		}

		if(bNeedToDelete)
		{
			xToBeDeleted.add_entry(pxAttractor->GetID());
		}
	}

	//delete the attractors marked for deletion
	while(xToBeDeleted.size())
	{
		u_int uID = xToBeDeleted.first_entry();
		xToBeDeleted.delete_first_entry();

		Tools_Attractor_System::GetInstance()->DeleteAttractorByID(uID,false);
	}
	#endif
}

void Tools_Entity_TemplateInstance_Object::DeleteDisowned_StreamingSound()
{
	//Check for any sound whose parent doesn't know about them
	//and delete them.
	//This may happen if the sound/entity files got out of sync

	List<u_int> xToBeDeleted;
	
	for(int i=0; i<Asura_StreamingBackgroundSound_System::GetNumberOfSounds(); i++)
	{
		const Asura_StreamingBackgroundSound* pxSound = Asura_StreamingBackgroundSound_System::GetSound(i);
		if(!pxSound->IsMemberOfTemplateInstance()) continue;

		//find the template entity for this sound
		bool bNeedToDelete = true;
		Asura_ServerEntity_Template* pxEntity = Asura_ServerEntity_Template::FindEntity(pxSound->m_uTemplateGuid);
		if(pxEntity)
		{
			bNeedToDelete = !pxEntity->m_xChildIDHandler.Contains(Tools_EntityTemplate_ChildType_StreamingSound,pxSound->m_uID);
		}

		if(bNeedToDelete)
		{
			xToBeDeleted.add_entry(pxSound->m_uID);
		}
	}

	//delete the sounds marked for deletion
	while(xToBeDeleted.size())
	{
		u_int uID = xToBeDeleted.first_entry();
		xToBeDeleted.delete_first_entry();

		Tools_Object* pxObject = Tools_StreamingBackgroundSound_Object::GetSoundObjectFromID(uID);
		if(pxObject)
		{
			pxObject->Destroy();
		}
		else
		{
			int iIndex = Asura_StreamingBackgroundSound_System::GetSoundIndexFromID(uID);
			if(iIndex >= 0)
			{
				Asura_StreamingBackgroundSound_System::DeleteSound(iIndex);
			}
		}
	}
}



void Tools_Entity_TemplateInstance_Object::FlagAllTemplateInstanceMembers()
{
	DeleteDisowned_Phonon();
	DeleteDisowned_Light();
	DeleteDisowned_Corona();
	DeleteDisowned_Attractor();
	DeleteDisowned_StreamingSound();


	//search the entity list for template instances
	for(Asura_Classified_List_It<Asura_Entity_Server> it(Asura_Entity_Server::GetList());!it.Done();it.Next())
	{
		Asura_Entity_Server* pxEntity = it.GetCurrent();
		//ignore entities marked for destruction
		if(pxEntity->IsDestroyed()) continue;
		//and those with non-tools guids
		if(pxEntity->GetGuid()<ASURA_GUID_TOOLSCREATEDENTITY_START || pxEntity->GetGuid()>ASURA_GUID_TOOLSCREATEDENTITY_END) continue;

		//is this a template entity?
		if(pxEntity->GetClassification() !=  AsuraEntityClass_Template) continue;

		Asura_ServerEntity_Template* pxTemplate = (Asura_ServerEntity_Template*) pxEntity;

		if(pxTemplate->m_bMaster) continue;
		if(pxTemplate->ShouldNeverUpdate()) continue;

		FlagTemplateInstanceMembers_Entity(pxTemplate);
		FlagTemplateInstanceMembers_Phonon(pxTemplate);
		FlagTemplateInstanceMembers_Light(pxTemplate);
		FlagTemplateInstanceMembers_Corona(pxTemplate);
		FlagTemplateInstanceMembers_Attractor(pxTemplate);
		FlagTemplateInstanceMembers_StreamingSound(pxTemplate);
	}

}

void Tools_Entity_TemplateInstance_Object::FlagTemplateInstanceMembers_Entity(Asura_ServerEntity_Template* pxTemplate)
{
	const Tools_EntityTemplate_ChildIDArray* pxArray = pxTemplate->m_xChildIDHandler.GetArray(Tools_EntityTemplate_ChildType_Entity);
	if(!pxArray) return;

	//find all the children , and mark them as template instances
	for(int i=0;i<pxArray->m_iNumIDs;i++)
	{
		Asura_Entity_Server* pxChild = Asura_Entity_Server::FindEntity(pxArray->m_puIDs[i]);
		if(pxChild && pxChild->ToolsData)
		{
			pxChild->ToolsData->m_uParentTemplate = pxTemplate->GetGuid();
		}
	}
}

void Tools_Entity_TemplateInstance_Object::FlagTemplateInstanceMembers_Phonon(Asura_ServerEntity_Template* pxTemplate)
{
	Tools_EntityTemplate_ChildIDArray* pxArray = pxTemplate->m_xChildIDHandler.GetArray(Tools_EntityTemplate_ChildType_Phonon);
	if(!pxArray) return;

	//Make sure each sound in the list agrees that this template owns it
	for(int i=0;i<pxArray->m_iNumIDs;i++)
	{
		Asura_Phonon* pxPhonon = Asura_Phonon_System::GetPhonon(pxArray->m_puIDs[i]);
		bool bOkay = false;
		if(pxPhonon)
		{
			if(pxPhonon->GetTemplateGuid() == pxTemplate->GetGuid())
			{
				bOkay = true;//all is in order
			}
		}
		if(!bOkay)
		{
			//template/sounds out of sync.. remove this guid from the list
			pxArray->m_puIDs[i] = pxArray->m_puIDs[pxArray->m_iNumIDs-1];
			pxArray->m_iNumIDs--;
			i--;
			pxTemplate->FlagForUpdate();
		}
	}
}

void Tools_Entity_TemplateInstance_Object::FlagTemplateInstanceMembers_Light(Asura_ServerEntity_Template* pxTemplate)
{
	Tools_EntityTemplate_ChildIDArray* pxArray = pxTemplate->m_xChildIDHandler.GetArray(Tools_EntityTemplate_ChildType_Light);
	if(!pxArray) return;

	//Make sure each light in the list agrees that this template owns it
	for(int i=0;i<pxArray->m_iNumIDs;i++)
	{
		Asura_Handle iHandle = Tools_PhotonMapping_System::GetLightHandleFromID(pxArray->m_puIDs[i]);
		
		bool bOkay = false;
		if(iHandle != ASURA_INVALID_HANDLE)
		{
			Tools_PhotonMapping_TemplateData* pxLightTemplate = Tools_PhotonMapping_System::GetTemplateData(iHandle);
			if(pxLightTemplate->m_uTemplateGuid == pxTemplate->GetGuid() &&
				pxLightTemplate->m_iTemplateVersion == pxTemplate->m_iTemplateVersion)
			{
				bOkay = true;//all is in order
			}
		}
		if(!bOkay)
		{
			//template/lights out of sync.. remove this guid from the list
			pxArray->m_puIDs[i] = pxArray->m_puIDs[pxArray->m_iNumIDs-1];
			pxArray->m_iNumIDs--;
			i--;
			pxTemplate->FlagForUpdate();
		}
	}
}

void Tools_Entity_TemplateInstance_Object::FlagTemplateInstanceMembers_Corona(Asura_ServerEntity_Template* pxTemplate)
{
	Tools_EntityTemplate_ChildIDArray* pxArray = pxTemplate->m_xChildIDHandler.GetArray(Tools_EntityTemplate_ChildType_Corona);
	if(!pxArray) return;

	//Make sure each corona in the list agrees that this template owns it
	for(int i=0;i<pxArray->m_iNumIDs;i++)
	{
		int iIndex = Asura_StaticCorona_System::GetCoronaIndexFromID(pxArray->m_puIDs[i]);
		
		bool bOkay = false;
		if(iIndex >= 0)
		{
			Asura_StaticCorona* pxCorona = Asura_StaticCorona_System::GetCorona(iIndex);
			if(pxCorona->m_uTemplateGuid == pxTemplate->GetGuid() &&
				pxCorona->m_iTemplateVersion == pxTemplate->m_iTemplateVersion)
			{
				bOkay = true;//all is in order
			}
		}
		if(!bOkay)
		{
			//template/coronas out of sync.. remove this guid from the list
			pxArray->m_puIDs[i] = pxArray->m_puIDs[pxArray->m_iNumIDs-1];
			pxArray->m_iNumIDs--;
			i--;
			pxTemplate->FlagForUpdate();
		}
	}
}

void Tools_Entity_TemplateInstance_Object::FlagTemplateInstanceMembers_Attractor(Asura_ServerEntity_Template* pxTemplate)
{
	#ifdef ASURA_USE_ATTRACTORS
	Tools_EntityTemplate_ChildIDArray* pxArray = pxTemplate->m_xChildIDHandler.GetArray(Tools_EntityTemplate_ChildType_Attractor);
	if(!pxArray) return;

	//Make sure each attractor in the list agrees that this template owns it
	for(int i=0;i<pxArray->m_iNumIDs;i++)
	{
		const Tools_Attractor* pxAttractor = Tools_Attractor_System::GetInstance()->GetAttractorByID(pxArray->m_puIDs[i]);
		
		bool bOkay = false;
		if(pxAttractor && pxAttractor->GetTemplateGuid() == pxTemplate->GetGuid() && 
			pxAttractor->GetTemplateVersion() == pxTemplate->m_iTemplateVersion)
		{
			bOkay = true;//all is in order
		}
		if(!bOkay)
		{
			//template/attractors out of sync.. remove this guid from the list
			pxArray->m_puIDs[i] = pxArray->m_puIDs[pxArray->m_iNumIDs-1];
			pxArray->m_iNumIDs--;
			i--;
			pxTemplate->FlagForUpdate();
		}
	}
	#endif
}

void Tools_Entity_TemplateInstance_Object::FlagTemplateInstanceMembers_StreamingSound(Asura_ServerEntity_Template* pxTemplate)
{
	Tools_EntityTemplate_ChildIDArray* pxArray = pxTemplate->m_xChildIDHandler.GetArray(Tools_EntityTemplate_ChildType_StreamingSound);
	if(!pxArray) return;

	//Make sure each sound in the list agrees that this template owns it
	for(int i=0;i<pxArray->m_iNumIDs;i++)
	{
		const Asura_StreamingBackgroundSound* pxSound = Asura_StreamingBackgroundSound_System::GetSoundFromID(pxArray->m_puIDs[i]);
		
		bool bOkay = false;
		if(pxSound && pxSound->m_uTemplateGuid == pxTemplate->GetGuid() && 
			pxSound->m_iTemplateVersion == pxTemplate->m_iTemplateVersion)
		{
			bOkay = true;//all is in order
		}
		if(!bOkay)
		{
			//template/sounds out of sync.. remove this guid from the list
			pxArray->m_puIDs[i] = pxArray->m_puIDs[pxArray->m_iNumIDs-1];
			pxArray->m_iNumIDs--;
			i--;
			pxTemplate->FlagForUpdate();
		}
	}
}



void Tools_Entity_TemplateInstance_Object::UnflagTemplateInstanceMembers()
{
	Asura_ServerEntity_Template* pxTemplate = (Asura_ServerEntity_Template*) Entity;

	UnflagTemplateInstanceMembers_Entity();
	UnflagTemplateInstanceMembers_Phonon();
	UnflagTemplateInstanceMembers_Light();
	UnflagTemplateInstanceMembers_Corona();
	UnflagTemplateInstanceMembers_Attractor();
	UnflagTemplateInstanceMembers_StreamingSound();

	pxTemplate->m_xChildIDHandler.Clear();

}

void Tools_Entity_TemplateInstance_Object::UnflagTemplateInstanceMembers_Entity()
{
	Asura_ServerEntity_Template* pxTemplate = (Asura_ServerEntity_Template*) Entity;

	const Tools_EntityTemplate_ChildIDArray* pxArray = pxTemplate->m_xChildIDHandler.GetArray(Tools_EntityTemplate_ChildType_Entity);
	if(!pxArray) return;

	//find all the children , and un-mark them as template instances
	for(int i=0;i<pxArray->m_iNumIDs;i++)
	{
		Asura_Entity_Server* pxChild = Asura_Entity_Server::FindEntity(pxArray->m_puIDs[i]);
		if(pxChild && pxChild->ToolsData)
		{
			pxChild->ToolsData->m_uParentTemplate = ASURA_GUID_UNREGISTERED;

			if(pxChild->ToolsData->ToolsObject)
			{
				//Cause the entity to be added to the entity browser tree
				Tools_SecondaryEntityFile_System::UpdateEntityInTree((Tools_Entity_Object*)pxChild->ToolsData->ToolsObject, false);
			}
		}
	}

}

void Tools_Entity_TemplateInstance_Object::UnflagTemplateInstanceMembers_Phonon()
{
	Asura_ServerEntity_Template* pxTemplate = (Asura_ServerEntity_Template*) Entity;

	const Tools_EntityTemplate_ChildIDArray* pxArray = pxTemplate->m_xChildIDHandler.GetArray(Tools_EntityTemplate_ChildType_Phonon);
	if(!pxArray) return;

	//sounds
	for(int i=0;i<pxArray->m_iNumIDs;i++)
	{
		Asura_Phonon* pxPhonon = Asura_Phonon_System::GetPhonon(pxArray->m_puIDs[i]);
		if(pxPhonon && pxPhonon->GetTemplateGuid() == pxTemplate->GetGuid())
		{
			pxPhonon->SetTemplateGuid(ASURA_GUID_UNREGISTERED);
			Tools_SaveAsura::SetSoundsModified();
		}
	}

}

void Tools_Entity_TemplateInstance_Object::UnflagTemplateInstanceMembers_Light()
{
	Asura_ServerEntity_Template* pxTemplate = (Asura_ServerEntity_Template*) Entity;

	const Tools_EntityTemplate_ChildIDArray* pxArray = pxTemplate->m_xChildIDHandler.GetArray(Tools_EntityTemplate_ChildType_Light);
	if(!pxArray) return;

	//lights
	for(int i=0;i<pxArray->m_iNumIDs;i++)
	{
		Asura_Handle iHandle = Tools_PhotonMapping_System::GetLightHandleFromID(pxArray->m_puIDs[i]);
		
		if(iHandle != ASURA_INVALID_HANDLE)
		{
			Tools_PhotonMapping_TemplateData* pxLightTemplate = Tools_PhotonMapping_System::GetTemplateData(iHandle);
			if(pxLightTemplate->m_uTemplateGuid == pxTemplate->GetGuid())
			{
				pxLightTemplate->m_uTemplateGuid = ASURA_GUID_UNREGISTERED;
				Tools_SaveAsura::SetGlobalIlluminationModified();
			}
		}
	}

}

void Tools_Entity_TemplateInstance_Object::UnflagTemplateInstanceMembers_Corona()
{
	Asura_ServerEntity_Template* pxTemplate = (Asura_ServerEntity_Template*) Entity;

	const Tools_EntityTemplate_ChildIDArray* pxArray = pxTemplate->m_xChildIDHandler.GetArray(Tools_EntityTemplate_ChildType_Corona);
	if(!pxArray) return;

	//coronas
	for(int i=0;i<pxArray->m_iNumIDs;i++)
	{
		int iIndex = Asura_StaticCorona_System::GetCoronaIndexFromID(pxArray->m_puIDs[i]);
		
		if(iIndex >= 0)
		{
			Asura_StaticCorona* pxCorona = Asura_StaticCorona_System::GetCorona(iIndex);
			if(pxCorona->m_uTemplateGuid == pxTemplate->GetGuid())
			{
				pxCorona->m_uTemplateGuid = ASURA_GUID_UNREGISTERED;
				Tools_SaveAsura::SetLightingModified();
			}
		}
	}
}

void Tools_Entity_TemplateInstance_Object::UnflagTemplateInstanceMembers_Attractor()
{
	#ifdef ASURA_USE_ATTRACTORS

	Asura_ServerEntity_Template* pxTemplate = (Asura_ServerEntity_Template*) Entity;

	const Tools_EntityTemplate_ChildIDArray* pxArray = pxTemplate->m_xChildIDHandler.GetArray(Tools_EntityTemplate_ChildType_Attractor);
	if(!pxArray) return;

	//attractors
	for(int i=0;i<pxArray->m_iNumIDs;i++)
	{
		Tools_Attractor* pxAttractor = Tools_Attractor_System::GetInstance()->GetAttractorByID(pxArray->m_puIDs[i]);
		
		if(pxAttractor)
		{
			if(pxAttractor->GetTemplateGuid() == pxTemplate->GetGuid())
			{
				pxAttractor->SetTemplateGuid(ASURA_GUID_UNREGISTERED);
				pxAttractor->SetModified();
			}
		}
	}
	#endif
}

void Tools_Entity_TemplateInstance_Object::UnflagTemplateInstanceMembers_StreamingSound()
{

	Asura_ServerEntity_Template* pxTemplate = (Asura_ServerEntity_Template*) Entity;

	const Tools_EntityTemplate_ChildIDArray* pxArray = pxTemplate->m_xChildIDHandler.GetArray(Tools_EntityTemplate_ChildType_StreamingSound);
	if(!pxArray) return;

	//sounds
	for(int i=0;i<pxArray->m_iNumIDs;i++)
	{
		Asura_StreamingBackgroundSound* pxSound = Asura_StreamingBackgroundSound_System::GetSoundFromID(pxArray->m_puIDs[i]);
		
		if(pxSound)
		{
			if(pxSound->m_uTemplateGuid == pxTemplate->GetGuid())
			{
				pxSound->m_uTemplateGuid = ASURA_GUID_UNREGISTERED;
				pxSound->m_iTemplateVersion = 0;
				Tools_SaveAsura::SetSoundsModified();
			}
		}
	}
}




void Tools_Entity_TemplateInstance_Object::AddContextMenuItems(class Tools_ContextMenu& xMenu, const class Tools_ContextMenu_Context& xContext)
{
	Tools_Entity_Template_Object::AddContextMenuItems(xMenu, xContext);

	Tools_ContextMenu_Item* pxItem;
	Tools_ContextMenu_Section* pxSection = xMenu.GetSection(TOOLS_CONTEXTMENU_SECTION_SELECTION);
	pxSection->AddSeparator();

	char szText[200];

	//add option for selecting all using this template
	if(xContext.m_bShiftHeld)
	{
		Asura_PrintfToBfr(szText,"Select template : %s",sizeof(szText),true,GetTemplateName());
		pxItem = new Tools_ContextMenu_Item_Function(szText, ContextMenu_SelectTemplate, this, 0);
		pxItem->SetID("Select by template");
		pxItem->SetHelpText("Deselects all template instances using this template");
		pxSection->AddItem(pxItem);
	}
	else if(xContext.m_bControlHeld)
	{
		Asura_PrintfToBfr(szText,"Select template : %s",sizeof(szText),true,GetTemplateName());
		pxItem = new Tools_ContextMenu_Item_Function(szText, ContextMenu_SelectTemplate, this, 1);
		pxItem->SetID("Select by template");
		pxItem->SetHelpText("Seselects all template instances using this template , adding to the current selection");
		pxSection->AddItem(pxItem);
	}
	else
	{
		Asura_PrintfToBfr(szText,"Select template : %s",sizeof(szText),true,GetTemplateName());
		pxItem = new Tools_ContextMenu_Item_Function(szText, ContextMenu_SelectTemplate, this, 2);
		pxItem->SetID("Select by template");
		pxItem->SetHelpText("Selects all template instances using this template , replacing the current selection");
		pxSection->AddItem(pxItem);
	}

}

void Tools_Entity_TemplateInstance_Object::ContextMenu_SelectTemplate(void* pObject, u_int uMode)
{
	Tools_Object* pxObject = (Tools_Object*) pObject;

	Asura_Assert(pxObject && pxObject->Type==TOT_Entity_TemplateInstance,"");
	Tools_Entity_TemplateInstance_Object* pxTemplateObject = (Tools_Entity_TemplateInstance_Object*) pxObject;

	//list all objects using this template
	List<Tools_Object*> xSelectionList;
	for(LIF<Tools_Entity_TemplateInstance_Object*> xObjectLIF(&s_xObjectList);!xObjectLIF.done();xObjectLIF.next())
	{
		if(xObjectLIF()->IsMemberOfTemplateInstance()) continue;//ignore instances that are members of other instances

		if(!stricmp(pxTemplateObject->GetTemplateName(),xObjectLIF()->GetTemplateName()))
		{
			xSelectionList.add_entry(xObjectLIF());
		}
	}
	if(xSelectionList.contains(pxObject))
	{
		xSelectionList.delete_entry(pxObject);
		xSelectionList.add_entry(pxObject);
	}
	
	
	//And (de)select them
	if(uMode == 0)
	{
		Tools_Object_System::DeselectObjects(xSelectionList);
	}
	else
	{
		Tools_Object_System::SelectObjects(xSelectionList, uMode == 1);
	}
}


Asura_Vector_3 Tools_Entity_TemplateInstance_Object::GetSnapPoint(const Asura_Vector_3& xPos) const
{

	float fMinDist = ASURA_LOS_MAX_DISTANCE;
	Asura_Vector_3 xBestSnapPoint = Position;

	//check all the child objects , and find the one with the nearest snap point
	for(CLIF<Tools_Object*> xChildLIF(&m_xChildList);!xChildLIF.done();xChildLIF.next())
	{
		Asura_Vector_3 xSnapPoint = xChildLIF()->GetSnapPoint(xPos);
		float fDist = (xSnapPoint - xPos).Magnitude();
		if(fDist<fMinDist)
		{
			fMinDist = fDist;
			xBestSnapPoint = xSnapPoint;
		}
	}

	return xBestSnapPoint;
}

void Tools_Entity_TemplateInstance_Object::GetBoundingBoxForSnapToVertex(Asura_Bounding_Box& xOutputBB) const
{
	//get all objects in the template (and child templates)
	List<Tools_Object*> xObjectList;
	//xObjectList.add_entry(this);
	ListChildObjects(xObjectList,true);
	

	Asura_Matrix_3x3 xInverseMat = Orientation;
	xInverseMat.Transpose();


	//For first pass , just use objects from physical objects
	for(int iPass=0;iPass<2;iPass++)
	{
		if(iPass==0)
			xOutputBB.Reset();
		else
			GetAccurateBoundingBox(xOutputBB);

		bool bFoundAny = false;
		
		//get combined bounding box of all child objects
		for(LIF<Tools_Object*> xObjectLIF(&xObjectList);!xObjectLIF.done();xObjectLIF.next())
		{
			if(iPass==0)
			{
				if(!Asura_ServerEntity_PhysicalObject::FindEntity(xObjectLIF()->GetGuid())) continue;
				if(xObjectLIF()->GetShapeOrSkinName().IsBlank()) continue;

			}
			bFoundAny = true;
			
			//get child object's box
			Asura_Bounding_Box xBox;
			xObjectLIF()->GetAccurateBoundingBox(xBox);

			//and make it relative to the instance object
			Asura_Matrix_3x3 xRelativeOrientation = xInverseMat * xObjectLIF()->Orientation;
			Asura_Vector_3 xRelativePosition = (xObjectLIF()->Position-Position)*xInverseMat;

			Asura_Bounding_Box xRotBB;
			xRotBB.FromOBB(xBox,xRelativeOrientation);
			xRotBB.Displace(xRelativePosition);

			xOutputBB.Set(xRotBB);

		}

		if(bFoundAny) break;
	}
}

void Tools_Entity_TemplateInstance_Object::HideTriggers( const bool bHide )
{
	bool bAnyPermanentlyVisibleChildren = false;
	HideTriggersRecursive(bHide, bAnyPermanentlyVisibleChildren);

	//If there are any children that are always visible , then it should be okay to hide the template instance object itself
	//RAR 03/01/2008
	//Not hiding the object fully though , since we still want the instance to appear in the find object overlay
	//despite being hidden
	//RAR 19/03/2008
	m_bHiddenForLOSAndRender =  bHide && bAnyPermanentlyVisibleChildren && s_bHideTriggers;
}

void Tools_Entity_TemplateInstance_Object::HideTriggersRecursive( const bool bHide, bool& bAnyPermanentlyVisible )
{
	for( LIF<Tools_Object*> xChildLIF( &m_xChildList ); !xChildLIF.done(); xChildLIF.next() )
	{
		Tools_Object* const pxObject = xChildLIF();
		if( pxObject->ShouldHideWithinTemplate() )
		{
			pxObject->SetHiddenFlag( ObjectHideFlag_TemplateTriggers, bHide && s_bHideTriggers );
		}
		else
		{
			bAnyPermanentlyVisible = true;
		}

		//If the child is a template instance , then do its children as well
		if(pxObject->Type == TOT_Entity_TemplateInstance)
		{
			Tools_Entity_TemplateInstance_Object* pxInstance = (Tools_Entity_TemplateInstance_Object*) pxObject;
			pxInstance->HideTriggersRecursive(bHide, bAnyPermanentlyVisible);
		}
	}
}


void Tools_Entity_TemplateInstance_Object::OnSelection()
{
	PARENT::OnSelection();
	HideTriggers(false);
}

void Tools_Entity_TemplateInstance_Object::OnDeselection()
{
	PARENT::OnDeselection();
	m_pxInputToTriggerListDialog = 0;

	//inform the children that this template is being deselected
	for(LIF<Tools_Object*> xChildLIF(&m_xChildList);!xChildLIF.done();xChildLIF.next())
	{
		xChildLIF()->OnParentDeselection();
	}

	HideTriggers(true);
}

void Tools_Entity_TemplateInstance_Object::OnGroupDeselection()
{
	//recurse over children
	for(LIF<Tools_Object*> xChildLIF(&m_xChildList);!xChildLIF.done();xChildLIF.next())
	{
		xChildLIF()->OnGroupDeselection();
	}
	//HideTriggers(true);	
}


void Tools_Entity_TemplateInstance_Object::SetBeingDestroyed()
{
	Tools_Entity_Template_Object::SetBeingDestroyed();

	//inform children that they are about to be killed
	for(LIF<Tools_Object*> xChildLIF(&m_xChildList);!xChildLIF.done();xChildLIF.next())
	{
		xChildLIF()->SetBeingDestroyed();
	}
}


/////////////////////////////////////////////////////////////////////////////////////////////////


//Overlay stuff

static const int g_iFolderTreeItemID = -2;

//search types
enum
{
	TEMPLATE_SEARCH_BEGINS,
	TEMPLATE_SEARCH_CONTAINS,
};

static const char* g_aszSearchOptions[] =
{
	"Begins With",
	"Contains",
};



bool Tools_Entity_TemplateInstance_Object::s_bHideTriggers			= true;
bool Tools_Entity_TemplateInstance_Object::s_bTemplateOverlayOpen	= false;
Asura_Vector_2 Tools_Entity_TemplateInstance_Object::s_xTemplateOverlayPosition(40,10);
Asura_Vector_2 Tools_Entity_TemplateInstance_Object::s_xTemplateListSize(0,0);
bool Tools_Entity_TemplateInstance_Object::s_bShowTemplatesInSubDirs = true;

static int g_iNumTemplateStrings = 0;
static const char** g_ppszTemplateStrings = 0;
static int* g_piTemplateStringToFileIndex = 0;
static bool g_bShowUsedOnly = false;
static bool g_bNeedToUpdateTemplateStrings = false;
static int g_iTemplateIndex = 0;
static Asura_List_Dialog* g_pxListDialog = 0;

Asura_NameTree g_xDirectoryNameTree;
static int g_iNumDirectoryStrings = 0;
static const char** g_ppszDirectoryStrings = 0;
static int g_iDirectoryIndex = 0;


static const char* g_pszNoTemplateString = "*No Templates*";
static const char* g_pszRootDirectoryString = "*Root*";

static HANDLE g_iSpawnedProcess = INVALID_HANDLE_VALUE;

struct Tools_TemplateFilename
{
	Asura_String m_xFilename;//minus extension
	Asura_String m_xDir;
	const char* m_szName;
};

static int g_iNumTemplates = 0;
static Tools_TemplateFilename* g_pxAllTemplateFiles = 0;

static int g_iSearchOption;
static Asura_String g_xSearchText;


class Tools_Overlay_Template : public Asura_Overlay
{
public :
	Tools_Overlay_Template(const char *szName, bool bAlwaysOnTop, Asura_Vector_2* pxPosition)
	:Asura_Overlay(szName, bAlwaysOnTop, pxPosition)
	{}

	~Tools_Overlay_Template()
	{
		Tools_Entity_TemplateInstance_Object::ClearupTemplateOverlay(IsClosingByCloseGadget());
	}
};


void Tools_Entity_TemplateInstance_Object::ToggleTemplateOverlay()
{
	if(TemplateOverlayIsOn())
	{
		s_bTemplateOverlayOpen = false;
		CloseTemplateOverlay();
	}
	else
	{
		s_bTemplateOverlayOpen = true;
		CreateTemplateOverlay();
	}
}

bool Tools_Entity_TemplateInstance_Object::TemplateOverlayIsOn()
{
	return Asura_Overlay_System::DoesOverlayExist(TOOLS_OVERLAY_ENTITYTEMPLATE);
}

void Tools_Entity_TemplateInstance_Object::CreateTemplateOverlay()
{
	if(!s_bTemplateOverlayOpen) return;
	if(Asura_Overlay_System::DoesOverlayExist(TOOLS_OVERLAY_ENTITYTEMPLATE)) return;

	BuildDirectoryStrings();

	BuildTemplateFilenameArray();
	BuildTemplateStrings(g_ppszDirectoryStrings[g_iDirectoryIndex]);
	g_iTemplateIndex = 0;

	Asura_Overlay* pxOverlay = new Tools_Overlay_Template("Templates",false,&s_xTemplateOverlayPosition);
	Asura_Vector_2 xOffset(1.0f,3.0f);
	Asura_Dialog* pxDialog;


	//heading for directories
	pxDialog = new Asura_Dialog("Directory :",xOffset);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::SmallSpacing();

	//and directory combo box
	int iNumLines = Asura_Maths::Min(15,g_iNumDirectoryStrings);
	pxDialog = new Asura_DropDown_List_Dialog(xOffset,g_ppszDirectoryStrings,&g_iDirectoryIndex,iNumLines,g_iNumDirectoryStrings,NULL,&g_bNeedToUpdateTemplateStrings);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();

	
	//add heading for list
	pxDialog = new Asura_Dialog("Templates :",xOffset);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::SmallSpacing();


	
	//and the list itself
	pxDialog = g_pxListDialog = new Asura_List_Dialog(xOffset,g_ppszTemplateStrings,&g_iTemplateIndex,15,g_iNumTemplateStrings,0);
	g_pxListDialog->SetDoubleClickFunction(CreateInstance_DoubleClick, 0);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y = pxDialog->m_xMaxCoord.y + 1.0f;

	pxDialog = new Asura_Resize_Dialog(g_pxListDialog, &s_xTemplateListSize);
	pxOverlay->AddDialog(pxDialog);

	//Search/Search Type
	pxDialog = new Asura_DropDown_List_Dialog(xOffset,g_aszSearchOptions,&g_iSearchOption,2,2,NULL,&g_bNeedToUpdateTemplateStrings,"");
	pxDialog->SetHelpText("Determines how the search text should be used for filtering the list.");
	pxDialog->SetRelativePosition(g_pxListDialog,true,false);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();


	pxDialog = new Asura_String_Dialog("",xOffset,&g_xSearchText,20,&g_bNeedToUpdateTemplateStrings);
	((Asura_String_Dialog*)pxDialog)->SetUpdateImmediatelyWhileTyping(true);
	pxDialog->SetHelpText("Filters the list so that it only shows templates beginning with/containing this string");
	pxDialog->SetRelativePosition(g_pxListDialog,true,false);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();


	
	pxDialog = new Asura_Function_Dialog("Create",xOffset,CreateInstance);
	pxDialog->SetHelpText("Creates an instance of the template currently selected in the list above.");
	pxDialog->SetRelativePosition(g_pxListDialog,true,false);
	pxOverlay->AddDialog(pxDialog);
	
	xOffset.x += 12.0f;
	pxDialog = new Asura_Function_Dialog("Apply",xOffset,ApplyTemplate);
	pxDialog->SetHelpText("Applies this template to all selected template instance objects.");
	pxDialog->SetRelativePosition(g_pxListDialog,true,false);
	pxOverlay->AddDialog(pxDialog);
	xOffset.x -= 12.0f;
	xOffset.y += Asura_Dialog::LargeSpacing();

	pxDialog = new Asura_Function_Dialog("Edit/View Template",xOffset,EditTemplate);
	pxDialog->SetHelpText("This opens the selected template file in another instance of tools.");
	pxDialog->SetRelativePosition(g_pxListDialog,true,false);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();


	pxDialog = new Asura_Tickbox_Dialog("Show used only",xOffset,&g_bShowUsedOnly , &g_bNeedToUpdateTemplateStrings);
	pxDialog->SetHelpText("If ticked then the template list will only show templates that have already been placed in this level.");
	pxDialog->SetRelativePosition(g_pxListDialog,true,false);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Tickbox_Dialog("Show templates in subdirectories",xOffset,&s_bShowTemplatesInSubDirs , &g_bNeedToUpdateTemplateStrings);
	pxDialog->SetHelpText("If ticked then the list will also contain templates that lie in subdirectories of the selected directory.");
	pxDialog->SetRelativePosition(g_pxListDialog,true,false);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();


	pxDialog = Tools_Entity_TemplateInstance_Object::CreateHideTemplateTriggersTickBox( xOffset );
	pxDialog->SetRelativePosition(g_pxListDialog,true,false);
	pxOverlay->AddDialog( pxDialog );
	xOffset.y = pxDialog->m_xMaxCoord.y + 1.0f;

	pxDialog = new Asura_Function_Dialog("Export Template",xOffset,Tools_Entity_TemplateMaster_Object::ExportSelectedAsTemplate);
	pxDialog->SetHelpText("Exports the selected entities as a template file. The template will be centered around the primary selected object.");
	pxDialog->SetRelativePosition(g_pxListDialog,true,false);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Reload Templates",xOffset,ReloadTemplates);
	pxDialog->SetHelpText("Forces all template files to be reloaded.");
	pxDialog->SetRelativePosition(g_pxListDialog,true,false);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Force Update",xOffset,ForceUpdate);
	pxDialog->SetHelpText("Forces all selected template instances to be updated from their templates.");
	pxDialog->SetRelativePosition(g_pxListDialog,true,false);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();


	pxOverlay->m_uID = TOOLS_OVERLAY_ENTITYTEMPLATE;

	Asura_Overlay_System::SubmitNewOverlay(pxOverlay);
}

void Tools_Entity_TemplateInstance_Object::ClearupTemplateOverlay(bool bClosedByUser)
{
	DeleteDirectoryStrings();
	DeleteTemplateStrings();
	DeleteTemplateFilenameArray();
	g_pxListDialog = 0;

	//Get rid of handle on any spawned process
	if(g_iSpawnedProcess != INVALID_HANDLE_VALUE)
	{
		CloseHandle(g_iSpawnedProcess);
		g_iSpawnedProcess = INVALID_HANDLE_VALUE;
	}

	if(bClosedByUser) s_bTemplateOverlayOpen = false;
}

void Tools_Entity_TemplateInstance_Object::CloseTemplateOverlay()
{
	Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_ENTITYTEMPLATE);
}

Asura_Dialog* Tools_Entity_TemplateInstance_Object::CreateHideTemplateTriggersTickBox( Asura_Vector_2 &xOffset )
{
	Asura_Tickbox_Dialog* pxHideTriggers = new Asura_Tickbox_Dialog( "Hide template triggers", xOffset, &s_bHideTriggers );
	pxHideTriggers->SetClickedFunction( HideTriggersTickboxClicked );
	pxHideTriggers->SetHelpText( "All template triggers are hidden if this is ticked." );
	return( pxHideTriggers );
}

void Tools_Entity_TemplateInstance_Object::BuildDirectoryStrings()
{
	//search for all template directories , and add their names to the name tree
	//want to search both locally , and on the network
	#ifdef ASURA_PC_ALLOW_LOAD_FROM_NETWORK
	for(int iNetwork = 0;iNetwork<2;iNetwork++)
	#endif // def ASURA_PC_ALLOW_LOAD_FROM_NETWORK
	{
		char szSearch[MAX_PATH];
		#ifdef ASURA_PC_ALLOW_LOAD_FROM_NETWORK
		if(iNetwork)
		{
			const char* pszNetworkPath = Asura_FileAccess::PlatformGetNetworkPath();
			if(!pszNetworkPath) continue;

			Asura_PrintfToBfr(szSearch,"%s\\Templates\\*.*",MAX_PATH,true,pszNetworkPath);
		}
		else
		#endif // def ASURA_PC_ALLOW_LOAD_FROM_NETWORK
		{
			strcpy(szSearch,"Templates\\*.*");
		}
		
		List<Asura_String> xFileList;

		Tools_FileSystem::FindDirectories(szSearch,xFileList,TOOLS_FILEFIND_RECURSIVE|TOOLS_FILEFIND_IGNOREBACKUPDIR);

		for(LIF<Asura_String> xFileLIF(&xFileList);!xFileLIF.done();xFileLIF.next())
		{
			const char* pszDirName = strstr(xFileLIF().GetString(),"Templates");
			if(pszDirName)
			{
				pszDirName += 10;//for Templates/ 
				if(*pszDirName)
				{
					//add this name to the list
					int iJunkIndex;
					g_xDirectoryNameTree.FindNodeIndexByName(pszDirName,true,iJunkIndex);
				}
			}
		}
	}

	//get the sorted list of names
	g_iNumDirectoryStrings = g_xDirectoryNameTree.GetNumberOfNodes()+1;
	g_ppszDirectoryStrings = new const char*[g_iNumDirectoryStrings];
	g_ppszDirectoryStrings[0] = g_pszRootDirectoryString;//first entry for root dir

	if(g_xDirectoryNameTree.GetNumberOfNodes())
	{
		g_xDirectoryNameTree.MakeSortedNameList(g_ppszDirectoryStrings+1);
	}

	if(g_iDirectoryIndex >= g_iNumDirectoryStrings)
	{
		g_iDirectoryIndex = 0;
	}
}

void Tools_Entity_TemplateInstance_Object::DeleteDirectoryStrings()
{
	delete [] g_ppszDirectoryStrings;
	g_xDirectoryNameTree.Reset();

	g_iNumDirectoryStrings = 0;
	g_ppszDirectoryStrings = 0;
}

static int SortTemplateNames(const void* p1, const void* p2)
{
	const Tools_TemplateFilename* px1 = (const Tools_TemplateFilename*) p1;
	const Tools_TemplateFilename* px2 = (const Tools_TemplateFilename*) p2;

	return stricmp(px1->m_szName, px2->m_szName);
}

void Tools_Entity_TemplateInstance_Object::BuildTemplateFilenameArray()
{
	DeleteTemplateFilenameArray();

	//search for all template files , and add their names to the name tree
	//want to search both locally , and on the network

	Asura_StringTree xStringTree;

	
#ifdef ASURA_PC_ALLOW_LOAD_FROM_NETWORK
	for(int iNetwork = 0;iNetwork<2;iNetwork++)
#endif // def ASURA_PC_ALLOW_LOAD_FROM_NETWORK
	{
		char szSearch[MAX_PATH];
		#ifdef ASURA_PC_ALLOW_LOAD_FROM_NETWORK
		if(iNetwork)
		{
			const char* pszNetworkPath = Asura_FileAccess::PlatformGetNetworkPath();
			if(!pszNetworkPath) continue;

			Asura_PrintfToBfr(szSearch,"%s\\Templates\\*.AsuraTemplate",MAX_PATH,true,pszNetworkPath);
		}
		else
		#endif // def ASURA_PC_ALLOW_LOAD_FROM_NETWORK
		{
			strcpy(szSearch,"Templates\\*.AsuraTemplate");
		}
		
		List<Asura_String> xFileList;

		Tools_FileSystem::FindFiles(szSearch,xFileList,TOOLS_FILEFIND_IGNOREBACKUPDIR | TOOLS_FILEFIND_RECURSIVE);

		for(LIF<Asura_String> xFileLIF(&xFileList);!xFileLIF.done();xFileLIF.next())
		{
			const char* szFilename = xFileLIF().GetString();
			//get filename relative to the templates directory
			const char* szRelativeFilename = Asura_FileAccess::PlatformRemoveNetworkOrExecutableDir(szFilename);
			if(!szRelativeFilename) continue;

			//skip past 'templates'
			if(strnicmp(szRelativeFilename,"Templates",9))
			{
				Asura_Assert(0,"Unexpected directory name");
				continue;
			}
			szRelativeFilename += 9;
			//skip past the \ or /
			if(*szRelativeFilename != '\\' && *szRelativeFilename != '/')
			{
				Asura_Assert(0, "Unexpected directory name");
				continue;
			}
			szRelativeFilename++;


						
			//add to the tree of template files
			int iJunkIndex;
			xStringTree.FindNodeIndexByName(szRelativeFilename,true,iJunkIndex);
		}
	}

	g_iNumTemplates = xStringTree.GetNumberOfNodes();
	if(!g_iNumTemplates) return;

	//get back all the filenames from the string tree
	const char** pszFilenames = new const char*[g_iNumTemplates];
	xStringTree.MakeSortedNameList(pszFilenames);

	//build an array to store the details of all the templates
	g_pxAllTemplateFiles = new Tools_TemplateFilename[g_iNumTemplates];
	char acBuffer[500];
	for(int i=0; i<g_iNumTemplates; i++)
	{
		//get the name without extension
		Asura_PrintfToBfr(acBuffer, "%s", sizeof(acBuffer), true, pszFilenames[i]);
		char* pcDot = strrchr(acBuffer, '.');
		if(pcDot) *pcDot = 0;
		
		g_pxAllTemplateFiles[i].m_xFilename = acBuffer;
		g_pxAllTemplateFiles[i].m_szName = Tools_FileSystem::StripPath(g_pxAllTemplateFiles[i].m_xFilename.GetString());

		//remove the name to get the directory
		char* pcSlash = strrchr(acBuffer, '\\');
		if(pcSlash)
		{
			*pcSlash = 0;
		}
		else
		{
			acBuffer[0] = 0;
		}
		g_pxAllTemplateFiles[i].m_xDir = acBuffer;

	}
	delete [] pszFilenames;

	//sort by template name
	Asura_SortAndSearch::Quicksort(g_pxAllTemplateFiles, g_iNumTemplates, SortTemplateNames);
	
}

void Tools_Entity_TemplateInstance_Object::DeleteTemplateFilenameArray()
{
	delete [] g_pxAllTemplateFiles;
	g_pxAllTemplateFiles = 0;
	g_iNumTemplates = 0;
}


void Tools_Entity_TemplateInstance_Object::BuildTemplateStrings(const char* pszDirectory)
{
	Asura_NameTree xUsedTemplates;
	
	if(g_bShowUsedOnly)
	{
		//search the entity list for template instance , and add their names to the list
		for(Asura_Classified_List_It<Asura_Entity_Server> it(Asura_Entity_Server::GetList());!it.Done();it.Next())
		{
			Asura_Entity_Server* pxEntity = it.GetCurrent();
			//ignore entities marked for destruction
			if(pxEntity->IsDestroyed()) continue;
			//and those with non-tools guids
			if(pxEntity->GetGuid()<ASURA_GUID_TOOLSCREATEDENTITY_START || pxEntity->GetGuid()>ASURA_GUID_TOOLSCREATEDENTITY_END) continue;
			
			//is this a template entity?
			if(pxEntity->GetClassification() !=  AsuraEntityClass_Template) continue;

			Asura_ServerEntity_Template* pxTemplate = (Asura_ServerEntity_Template*) pxEntity;

			if(pxTemplate->m_bMaster) continue;
			if(pxTemplate->m_xName.IsBlank()) continue;

			//add this name to the list
			int iJunkIndex;
			xUsedTemplates.FindNodeIndexByName(pxTemplate->m_xName,true,iJunkIndex);

		}
	}

	if(!pszDirectory || *pszDirectory == '*') pszDirectory = "";
	int iDirLength = strlen(pszDirectory);

	//prepare search string for wildcard search
	char acWildcardSearchString[100] = "";
	if(!g_xSearchText.IsBlank())
	{
		if(g_iSearchOption == TEMPLATE_SEARCH_CONTAINS)
		{
			Asura_PrintfToBfr(acWildcardSearchString,"*%s*",sizeof(acWildcardSearchString),true,g_xSearchText.GetString());
		}
		else
		{
			Asura_PrintfToBfr(acWildcardSearchString,"%s*",sizeof(acWildcardSearchString),true,g_xSearchText.GetString());
		}
	}
	
	if(g_iNumTemplates)
	{
		//allocate enough space for all the template names
		g_iNumTemplateStrings = 0;
		g_ppszTemplateStrings = new const char*[g_iNumTemplates];
		g_piTemplateStringToFileIndex = new int[g_iNumTemplates];

		//determine which ones should be shown
		for(int i=0; i<g_iNumTemplates; i++)
		{
			
			//check the template's directory
			const char* szDir = g_pxAllTemplateFiles[i].m_xDir.GetString();
			if(s_bShowTemplatesInSubDirs)
			{
				if(strnicmp(szDir, pszDirectory, iDirLength))
				{
					continue;
				}
				//make sure the next character is a '\' or the terminator , to ensure that the directory name isn't just
				//the start of the full name
				if(iDirLength && szDir[iDirLength] != 0 && szDir[iDirLength] != '\\')
				{
					continue;
				}
			}
			else
			{
				if(stricmp(szDir, pszDirectory))
				{
					continue;
				}
			}


			if(g_bShowUsedOnly)
			{
				//make sure this template is in use already
				int iJunkIndex;
				if(!xUsedTemplates.FindNodeIndexByName(g_pxAllTemplateFiles[i].m_xFilename.GetString(),false,iJunkIndex))
				{
					continue;	
				}
			}

			if(acWildcardSearchString[0])
			{
				//check the search string
				if(!Asura_StringUtil::CompareAgainstWildcardString(g_pxAllTemplateFiles[i].m_szName,acWildcardSearchString,true))
				{
					continue;
				}
			}

			g_ppszTemplateStrings[g_iNumTemplateStrings] = g_pxAllTemplateFiles[i].m_szName;
			g_piTemplateStringToFileIndex[g_iNumTemplateStrings] = i;
			g_iNumTemplateStrings++;
		}
	}
	if(!g_iNumTemplateStrings)
	{
		//empty list , so just create a dummy entry
		g_iNumTemplateStrings = 1;
		if(!g_ppszTemplateStrings)
		{
			g_ppszTemplateStrings = new const char*[1];
		}
		g_ppszTemplateStrings[0] = g_pszNoTemplateString;
	}
	
}

void Tools_Entity_TemplateInstance_Object::DeleteTemplateStrings()
{
	delete [] g_ppszTemplateStrings;
	delete [] g_piTemplateStringToFileIndex;

	g_iNumTemplateStrings = 0;
	g_ppszTemplateStrings = 0;
	g_piTemplateStringToFileIndex = 0;
}


bool Tools_Entity_TemplateInstance_Object::CanUseTemplate(const char* pszTemplateName)
{
	if(Tools_SaveAsura::GetFileType() == AsuraFile_Template)
	{
		//Currently editing a template file ,so we need to guard against it including itself
		//(otherwise things will get ugly)
		Tools_EntityTemplate* pxTemplate = Tools_EntityTemplate_System::GetTemplate(pszTemplateName);
		List<const char*> xNameList;
		xNameList.add_entry(pszTemplateName);
		if(!pxTemplate || !pxTemplate->GetDependencies(xNameList))
		{
			ShowMessageOverlay("Failed to determine which template files this template depends on.\nCreation of template instance aborted.");
			return false;
		}

		Asura_String xFilename = Tools_SaveAsura::GetTemplateFilename();
		//get the name
		char szName[MAX_PATH];
		strcpy(szName,xFilename.GetString());
		//remove the file extension
		char* pcDot = strrchr(szName,'.');
		if(pcDot) *pcDot = 0;

		//make relative to the templates directory
		strlwr(szName);
		char* pszMasterName = strstr(szName,"templates");
		if(pszMasterName)
		{
			pszMasterName += 10;//skip past templates/
		}
		else
		{
			pszMasterName = szName;
		}
		

		
		//compare the master template's name with those in the list
		for(LIF<const char*> xNameLIF(&xNameList);!xNameLIF.done();xNameLIF.next())
		{
			if(!stricmp(xNameLIF(),pszMasterName))
			{
				ShowMessageOverlay("Cannot create an instance of this template because it would result in this file having to include itself.\nCreation of template instance aborted.");
				return false;
			}
		}
	}
	return true;
}

void Tools_Entity_TemplateInstance_Object::CreateInstance()
{
	if(!s_bTemplateOverlayOpen) return;
	
	//get the index of the current selection
	if(!g_piTemplateStringToFileIndex) return;
	if(g_iTemplateIndex >= g_iNumTemplateStrings) return;

	int iFileIndex = g_piTemplateStringToFileIndex[g_iTemplateIndex];

	if ( iFileIndex >= 0 && iFileIndex < g_iNumTemplates)
	{
		const char* szName = g_pxAllTemplateFiles[iFileIndex].m_szName;
		const char* szFullName = g_pxAllTemplateFiles[iFileIndex].m_xFilename.GetString();

		Tools_Object* pxObject = CreateInstance(szFullName);

		if(pxObject)
		{
			
			Tools_SecondaryEntityFile_System::AddToCurrentFile(pxObject);
			
			Tools_Timed_Message_System::AddMessage(0xffffffff, "%s created",szName);

			Tools_SaveAsura::SetEntityModified();
		}
	}
	else
	{
		Tools_Timed_Message_System::AddMessage( Tools_Timed_Message_System::COLOUR_ERROR, "Invalid selection or directory contains no templates!" );
	}
}

Tools_Object* Tools_Entity_TemplateInstance_Object::CreateInstance(const char* pszTemplateName)
{
	if(!CanUseTemplate(pszTemplateName)) return 0;
	
	//make sure the template is valid
	if(!Tools_EntityTemplate_System::GetTemplate(pszTemplateName))
	{
		ShowMessageOverlay("Can't create instance of template '%s'\nEither failed to load ,or it isn't a valid template file..",pszTemplateName);
		return 0;
	}

	//initialise the tools data for the new template instance
	Asura_Guid uGuid = Asura_Entity_System::GetNextToolsCreatedGuid();
	//for default name , remove directory component if there is any
	const char* pszDefaultName = strrchr(pszTemplateName,'\\');
	if(pszDefaultName)
	{
		pszDefaultName++;//to skip the slash
	}
	else
	{
		pszDefaultName = pszTemplateName;
	}
	Tools_Entity_Data* pxToolsData = CreateToolsDataWithDefaultName(pszDefaultName,pszDefaultName,uGuid);
	if(!pxToolsData) return 0;

	pxToolsData->m_uLightSets = Tools_LightSets::GetLightSetsForNewObject(pxToolsData->Position);
	
	//create the asura entity
	Asura_ServerEntity_Template* pxTemplateEntity = new Asura_ServerEntity_Template();
	pxTemplateEntity->ToolsData = pxToolsData;
	pxTemplateEntity->m_xName = pszTemplateName;
	pxTemplateEntity->m_xPosition = pxToolsData->Position;


	//create the tools object , and select it
	Tools_Entity_TemplateInstance_Object* pxObject = new Tools_Entity_TemplateInstance_Object(pxTemplateEntity);
	Tools_Object_System::SelectObject(pxObject);

	pxObject->UpdateFromTemplate();
	Tools_Object_System::CheckForMessageBlockOwnershipProblems();


	return pxObject;
}

void Tools_Entity_TemplateInstance_Object::EditTemplate()
{
	if(!s_bTemplateOverlayOpen) return;
	
	//get the index of the current selection
	if(!g_piTemplateStringToFileIndex) return;
	if(g_iTemplateIndex >= g_iNumTemplateStrings) return;

	int iFileIndex = g_piTemplateStringToFileIndex[g_iTemplateIndex];
	
	if ( iFileIndex >= 0 && iFileIndex < g_iNumTemplates)
	{
		const char* szFullName = g_pxAllTemplateFiles[iFileIndex].m_xFilename.GetString();

		EditTemplate(szFullName);
	}
	else
	{
		Tools_Timed_Message_System::AddMessage( Tools_Timed_Message_System::COLOUR_ERROR, "Invalid selection or directory contains no templates!" );
	}
}
void Tools_Entity_TemplateInstance_Object::EditTemplate(const char* szTemplateName)
{
	//Confirm doing this , since it could be irritating doing it by mistake.
	if(!ShowYesNoOverlay("Edit '%s'?\nThis will open the template file in another instance of tools.",szTemplateName)) return;


	//need to grab the command line that was used to launch tools , and replace the level filename
	//with the name of the template file.
	Asura_String xCmdLine = GetCommandLine();

	//allocate space for new commandline (overestimating by a few miles)
	const int iTemplateFilenameSize = strlen(szTemplateName) + 50;
	char* szNewCommand = new char[xCmdLine.GetLength() + iTemplateFilenameSize];

	const char* pcSource = xCmdLine.GetString();
	char* pcDest = szNewCommand;

	//Copy the name of the executable
	bool bInQuotes = false;

	while(*pcSource)
	{
		if(*pcSource == ' ' && !bInQuotes)
		{
			//reached end
			*pcDest++ = *pcSource++;
			break;
		}
		else
		{
			if(*pcSource == '\"') bInQuotes = !bInQuotes;
			*pcDest++ = *pcSource++;
		}
	}

	//copy the template filename into the destination buffer
	Asura_PrintfToBfr(pcDest,"\"Templates\\%s.AsuraTemplate\"",iTemplateFilenameSize,true,szTemplateName);
	pcDest += strlen(pcDest);

	//skip the filename in the source
	while(*pcSource == ' ') pcSource++;

	bInQuotes = false;
	while(*pcSource)
	{
		if(*pcSource == ' ' && !bInQuotes)
		{
			//reached end
			break;
		}
		else
		{
			if(*pcSource == '\"') bInQuotes = !bInQuotes;
			pcSource++;
		}
	}

	//Copy the rest of the source into the destination buffer
	strcpy(pcDest,pcSource);

	//lose handle on any previous process
	if(g_iSpawnedProcess != INVALID_HANDLE_VALUE)
	{
		CloseHandle(g_iSpawnedProcess);
		g_iSpawnedProcess = INVALID_HANDLE_VALUE;
	}


    // Spawn the new tools process.
    {
        STARTUPINFO xSI;
        ZeroMemory( &xSI, sizeof(STARTUPINFO) );
        xSI.cb = sizeof(STARTUPINFO);

        PROCESS_INFORMATION xPI;
        ZeroMemory( &xPI, sizeof(PROCESS_INFORMATION) );


        if( CreateProcess(NULL, szNewCommand, NULL, NULL, FALSE, 0, NULL, NULL, &xSI, &xPI) )
        {

            g_iSpawnedProcess = xPI.hProcess;
            CloseHandle( xPI.hThread );

        }
        else
        {
            Asura_Assertf(0,( "Failed to start new tools instance. Reason: %s\n", Tools_OS::GetLastSystemError() ));
        }
    }

	delete [] szNewCommand;
}

void Tools_Entity_TemplateInstance_Object::ApplyTemplate()
{
	if(!s_bTemplateOverlayOpen) return;
	
	//get the index of the current selection
	if(!g_piTemplateStringToFileIndex) return;
	if(g_iTemplateIndex >= g_iNumTemplateStrings) return;

	int iFileIndex = g_piTemplateStringToFileIndex[g_iTemplateIndex];
	
	if ( iFileIndex >= 0 && iFileIndex < g_iNumTemplates)
	{
		const char* szFullName = g_pxAllTemplateFiles[iFileIndex].m_xFilename.GetString();

		ApplyTemplate(szFullName);
	}
	else
	{
		Tools_Timed_Message_System::AddMessage( Tools_Timed_Message_System::COLOUR_ERROR, "Invalid selection or directory contains no templates!" );
	}
}

void Tools_Entity_TemplateInstance_Object::ApplyTemplate(const char* pszTemplateName)
{
	//make sure some template instance objects are selected
	List<Tools_Entity_TemplateInstance_Object*> xSelectedList;	
	for(LIF<Tools_Entity_TemplateInstance_Object*> xObjectLIF(&s_xObjectList);!xObjectLIF.done();xObjectLIF.next())
	{
		if(xObjectLIF()->IsMemberOfTemplateInstance()) continue;//ignore instances that are members of other instances

		if(xObjectLIF()->IsGroupSelected())
		{
			xSelectedList.add_entry(xObjectLIF());
		}
	}

	if(!xSelectedList.size())
	{
		ShowMessageOverlay("One or more template instance objects must be selected first before a template can be applied");
		return;
	}

	//make sure its okay to use this template
	if(!CanUseTemplate(pszTemplateName)) return;

	//make sure the template is valid
	if(!Tools_EntityTemplate_System::GetTemplate(pszTemplateName))
	{
		ShowMessageOverlay("Unable to apply template '%s'\nEither failed to load ,or it isn't a valid template file.",pszTemplateName);
		return;
	}

	//apply to the selected objects
	for(LIF<Tools_Entity_TemplateInstance_Object*> xSelectedLIF(&xSelectedList);!xSelectedLIF.done();xSelectedLIF.next())
	{
		Asura_ServerEntity_Template* pxTemplateEntity = (Asura_ServerEntity_Template*) xSelectedLIF()->Entity;
		if(!stricmp(pxTemplateEntity->m_xName.GetString(),pszTemplateName)) continue;//already has template
		if(pxTemplateEntity->ShouldNeverUpdate()) continue;

		pxTemplateEntity->m_xName = pszTemplateName;
		pxTemplateEntity->FlagForUpdate();
		xSelectedLIF()->UpdateFromTemplate();
	}
	Tools_Object_System::CheckForMessageBlockOwnershipProblems();

	Tools_Timed_Message_System::AddMessage(0xffffffff, "%s applied to selection",pszTemplateName);
	
}


void Tools_Entity_TemplateInstance_Object::UpdateTemplateStrings()
{
	//get the current selection , in an attempt to preserve it
	Asura_String xSelected;
	if(g_iTemplateIndex < g_iNumTemplateStrings)
	{
		xSelected = g_ppszTemplateStrings[g_iTemplateIndex];
	}

	//rebuild the template string list
	DeleteTemplateStrings();
	BuildTemplateStrings(g_ppszDirectoryStrings[g_iDirectoryIndex]);

	//update the overlay
	Asura_Assert(g_pxListDialog,"");
	g_pxListDialog->ReplaceStrings(g_iNumTemplateStrings,g_ppszTemplateStrings);

	//search for the previous selection 
	int iIndex = 0;

	if(!xSelected.IsBlank())
	{
		for(int i=0;i<g_iNumTemplateStrings;i++)
		{
			if(!stricmp(g_ppszTemplateStrings[i],xSelected.GetString()))
			{
				iIndex = i;
				break;
			}
		}
	}
	g_iTemplateIndex = iIndex;
	g_bNeedToUpdateTemplateStrings = false;
	

}

void Tools_Entity_TemplateInstance_Object::UpdateTemplateOverlay()
{
	if(!g_pxListDialog) return;

	if(g_bNeedToUpdateTemplateStrings)
	{
		UpdateTemplateStrings();
	}

	//If another instance of tools has been opened for template editing , see if it has finished
	if(g_iSpawnedProcess != INVALID_HANDLE_VALUE)
	{
		if(WaitForSingleObject(g_iSpawnedProcess,0) == WAIT_OBJECT_0)
		{
			CloseHandle(g_iSpawnedProcess);
			g_iSpawnedProcess = INVALID_HANDLE_VALUE;

			//bring this back to the foreground , and update the templates
			ShowWindow(Asura_Windows::GetWindowHandle(),TRUE);
			SetForegroundWindow(Asura_Windows::GetWindowHandle());
			ReloadTemplates();
		}
	}
}


void Tools_Entity_TemplateInstance_Object::ListChildObjects(List<Tools_Object*>& xObjectList,bool bRecurse) const
{
	for(CLIF<Tools_Object*> xChildLIF(&m_xChildList);!xChildLIF.done();xChildLIF.next())
	{
		xObjectList.add_entry(xChildLIF());

		if(bRecurse)
		{
			//add any children of this object as well
			xChildLIF()->ListChildObjects(xObjectList,true);
		}

	}
}

bool Tools_Entity_TemplateInstance_Object::GetParameter(Asura_Hash_ID uParamID,class Tools_Parameter* pxParam) const
{

	Asura_ServerEntity_Template* pxAsuraEntity = (Asura_ServerEntity_Template*) Entity;

	switch(uParamID)
	{
		case g_uHash_Template_Skin :
			pxParam->SetHash(pxAsuraEntity->GetReplacementSkinID());
			break;

		case g_uHash_PhysicalObject_MergeWithEnv :
			pxParam->SetBool(pxAsuraEntity->m_bMergeWithEnvironment);
			break;


		default :
			return PARENT::GetParameter(uParamID, pxParam);
	}
	return true;
}


bool Tools_Entity_TemplateInstance_Object::SetParameter(Asura_Hash_ID uParamID,const class Tools_Parameter* pxParam)
{
	Asura_ServerEntity_Template* pxAsuraEntity = (Asura_ServerEntity_Template*) Entity;

	u_int uPlatforms = Entity->ToolsData->m_uPlatforms;
	u_int uLightSets = Entity->ToolsData->m_uLightSets;
	bool bEntityActive = Entity->GetEntityActiveIgnoringTemplate();
	bool bRetVal = true;

	switch(uParamID)
	{
		case g_uHash_Template_Skin :
			pxAsuraEntity->SetReplacementSkinID(pxParam->GetHash());
			m_bForceUpdate = true;
			break;

		case g_uHash_PhysicalObject_MergeWithEnv :
		{	
			u_int uFlag = pxParam->GetBool();
			if(pxAsuraEntity->m_bMergeWithEnvironment != uFlag)
			{
				pxAsuraEntity->m_bMergeWithEnvironment = pxParam->GetBool();
				m_bForceUpdate = true;
			}
			break;
		}

		default :
			bRetVal = PARENT::SetParameter(uParamID, pxParam);
	}
	
	//If the platforms, light sets, or activity have been changed , then we need to force an update of the template
	//so that the changes get propagated to the children
	if((uPlatforms != Entity->ToolsData->m_uPlatforms) || 
		(uLightSets != Entity->ToolsData->m_uLightSets) || 
		(bEntityActive != Entity->GetEntityActiveIgnoringTemplate()) )
	{
		Asura_ServerEntity_Template* pxAsuraEntity = (Asura_ServerEntity_Template*) Entity;
		pxAsuraEntity->FlagForUpdate();
	}	

	return bRetVal;
}


void Tools_Entity_TemplateInstance_Object::SetHiddenFlagIncludingChildren(int iHideFlag,bool bHide)
{
	Tools_Entity_Template_Object::SetHiddenFlagIncludingChildren(iHideFlag,bHide);

	for(LIF<Tools_Object*> xChildLIF(&m_xChildList);!xChildLIF.done();xChildLIF.next())
	{
		xChildLIF()->SetHiddenFlagIncludingChildren(iHideFlag,bHide);
	}
}


void Tools_Entity_TemplateInstance_Object::ReloadTemplates()
{
	//Invalidate cache of loaded templates
	Tools_EntityTemplate_System::InvalidateTemplates();

	//And force all instance to be updated.
	UpdateAllTemplates();
}

void Tools_Entity_TemplateInstance_Object::ForceUpdate()
{
	int iCount = 0;
	for(LIF<Tools_Entity_TemplateInstance_Object*> xObjectLIF(&s_xObjectList);!xObjectLIF.done();xObjectLIF.next())
	{
		Tools_Entity_TemplateInstance_Object* pxObject = xObjectLIF();
		
		if(pxObject->IsMemberOfTemplateInstance()) continue;//ignore instances that are members of other instances
		if(!pxObject->IsGroupSelected()) continue;

		Asura_ServerEntity_Template* pxEntity = (Asura_ServerEntity_Template*) pxObject->Entity;
		if(pxEntity->ShouldNeverUpdate()) continue;

		pxEntity->FlagForUpdate();
		pxObject->UpdateFromTemplate();	

		iCount++;
	}

	if(iCount)
	{
		Tools_Object_System::CheckForMessageBlockOwnershipProblems();
		Tools_Timed_Message_System::AddMessage(0xffffffff, "Updated %d template instance(s)",iCount);
	}
	else
	{
		Tools_Timed_Message_System::AddMessage(0xffffffff, "No updateable template instances are selected.");
	}
	
}

void Tools_Entity_TemplateInstance_Object::HideTriggersTickboxClicked()
{
	for( LIF< Tools_Entity_TemplateInstance_Object* > xObjectLIF( &s_xObjectList ); !xObjectLIF.done(); xObjectLIF.next() )
	{
		xObjectLIF()->UpdateTriggerVisibility();
	}
}



void Tools_Entity_TemplateInstance_Object::UpdateInputList()
{
	DeleteInputList();

	Asura_ServerEntity_Template* pxTemplate = (Asura_ServerEntity_Template*) Entity;

	if(pxTemplate->m_iNumInputNames)
	{
		m_iInputListSize = pxTemplate->m_iNumInputNames;
		m_pszInputList = new char*[m_iInputListSize];

		for(int i=0;i<m_iInputListSize; i++)
		{
			const char* szName = pxTemplate->m_pxInputNames[i].GetString();
			Asura_Assert(szName,"");

			m_pszInputList[i] = new char[strlen(szName)+1];
			strcpy(m_pszInputList[i], szName);
		}

	}
	else
	{
		m_iInputListSize = 1;
		m_pszInputList = new char*[1];
		m_pszInputList[0] = new char[7];
		strcpy(m_pszInputList[0], "*None*");
	}

	m_iInputSelection = Asura_Maths::Min(m_iInputListSize-1, m_iInputSelection);
}

void Tools_Entity_TemplateInstance_Object::DeleteInputList()
{
	for(int i=0; i<m_iInputListSize; i++)
	{
		delete [] m_pszInputList[i];
	}
	delete [] m_pszInputList;
	m_pszInputList = 0;
	m_iInputListSize = 0;
}

void Tools_Entity_TemplateInstance_Object::UpdateInputToTriggerList()
{
	DeleteInputToTriggerList();

	Asura_ServerEntity_Template* pxTemplate = (Asura_ServerEntity_Template*) Entity;

	if(pxTemplate->m_xInitialInputsToTrigger.Size())
	{
		m_iInputToTriggerListSize = pxTemplate->m_xInitialInputsToTrigger.Size();
		m_pszInputToTriggerList = new char*[m_iInputToTriggerListSize];

		for(int i=0;i<m_iInputToTriggerListSize; i++)
		{
			const char* szName = GetInputNameFromHash(pxTemplate->m_xInitialInputsToTrigger[i]);
			Asura_Assert(szName,"");

			m_pszInputToTriggerList[i] = new char[strlen(szName)+1];
			strcpy(m_pszInputToTriggerList[i], szName);
		}

	}
	else
	{
		m_iInputToTriggerListSize = 1;
		m_pszInputToTriggerList = new char*[1];
		m_pszInputToTriggerList[0] = new char[7];
		strcpy(m_pszInputToTriggerList[0], "*None*");
	}

	m_iInputToTriggerSelection = Asura_Maths::Min(m_iInputToTriggerListSize-1, m_iInputToTriggerSelection);

	if(m_pxInputToTriggerListDialog)
	{
		m_pxInputToTriggerListDialog->ReplaceStrings(m_iInputToTriggerListSize, m_pszInputToTriggerList);
	}
}

void Tools_Entity_TemplateInstance_Object::DeleteInputToTriggerList()
{
	for(int i=0; i<m_iInputToTriggerListSize; i++)
	{
		delete [] m_pszInputToTriggerList[i];
	}
	delete [] m_pszInputToTriggerList;
	m_pszInputToTriggerList = 0;
	m_iInputToTriggerListSize = 0;
}

void Tools_Entity_TemplateInstance_Object::AddTriggeredInput()
{
	Asura_ServerEntity_Template* pxTemplate = (Asura_ServerEntity_Template*) Entity;

	//get the selection from the input list
	if(m_iInputSelection<0 || m_iInputSelection >= pxTemplate->m_iNumInputNames) return;

	const char* szInput = pxTemplate->m_pxInputNames[m_iInputSelection].GetString();
	Asura_Hash_ID uInput = Asura_GetHashID(szInput);

	//add to the array
	u_int uNewSize = pxTemplate->m_xInitialInputsToTrigger.Size()+1;
	pxTemplate->m_xInitialInputsToTrigger.Resize(uNewSize);
	pxTemplate->m_xInitialInputsToTrigger[uNewSize-1] = uInput;

	CopyTriggeredInputListToSelection();

	SetEntityModified();

	UpdateInputToTriggerList();
}

bool Tools_Entity_TemplateInstance_Object::CanAddTriggeredInput()
{
	Asura_ServerEntity_Template* pxTemplate = (Asura_ServerEntity_Template*) Entity;
	return pxTemplate->m_iNumInputNames > 0;
}

void Tools_Entity_TemplateInstance_Object::RemoveTriggeredInput()
{
	Asura_ServerEntity_Template* pxTemplate = (Asura_ServerEntity_Template*) Entity;

	//make sure the selected index is in range
	if((u_int)m_iInputToTriggerSelection >= pxTemplate->m_xInitialInputsToTrigger.Size()) return;

	//remove this entry from the array , shuffling the rest down
	u_int uNewSize = pxTemplate->m_xInitialInputsToTrigger.Size() - 1;
	for(u_int i=m_iInputToTriggerSelection; i<uNewSize; i++)
	{
		pxTemplate->m_xInitialInputsToTrigger[i] = pxTemplate->m_xInitialInputsToTrigger[i+1];
	}
	pxTemplate->m_xInitialInputsToTrigger.Shrink(uNewSize);

	CopyTriggeredInputListToSelection();

	SetEntityModified();

	UpdateInputToTriggerList();
}

bool Tools_Entity_TemplateInstance_Object::CanRemoveTriggeredInput()
{
	Asura_ServerEntity_Template* pxTemplate = (Asura_ServerEntity_Template*) Entity;
	return pxTemplate->m_xInitialInputsToTrigger.Size() > 0;
}

void Tools_Entity_TemplateInstance_Object::CopyTriggeredInputListToSelection()
{
	const Asura_ServerEntity_Template* pxFromEntity = (Asura_ServerEntity_Template*) Entity;

	for(LIF<Tools_Entity_TemplateInstance_Object*> xObjectLIF(&s_xObjectList);!xObjectLIF.done();xObjectLIF.next())
	{
		if(xObjectLIF()->IsMemberOfTemplateInstance()) continue;//ignore instances that are members of other instances
		if(!xObjectLIF()->IsGroupSelected()) continue;

		Asura_ServerEntity_Template* pxToEntity = (Asura_ServerEntity_Template*) xObjectLIF()->Entity;
		if(pxToEntity == pxFromEntity) continue;

		//using the same template?
		if(pxToEntity->m_xName.Compare(pxFromEntity->m_xName)) continue;

		//replace the triggered inputs in the destination instance
		u_int uNumInputsToTrigger = pxFromEntity->m_xInitialInputsToTrigger.Size();
		pxToEntity->m_xInitialInputsToTrigger.Resize(uNumInputsToTrigger);
		for(u_int i=0; i<uNumInputsToTrigger; i++)
		{
			pxToEntity->m_xInitialInputsToTrigger[i] = pxFromEntity->m_xInitialInputsToTrigger[i];
		}

		Tools_SaveAsura::SetEntityModified(pxToEntity);
	}
}

bool Tools_Entity_TemplateInstance_Object::ShouldBeRendered()
{
	if(m_bHiddenForLOSAndRender) return false;
	return Tools_Entity_Template_Object::ShouldBeRendered();
}

bool Tools_Entity_TemplateInstance_Object::ShouldTestForLOS()
{
	if(m_bHiddenForLOSAndRender) return false;
	return Tools_Entity_Template_Object::ShouldTestForLOS();
}


void Tools_Entity_TemplateInstance_Object::Update()
{
	if(m_bForceUpdate)
	{
		m_bForceUpdate = false;
		
		Asura_ServerEntity_Template* pxAsuraEntity = (Asura_ServerEntity_Template*) Entity;
		pxAsuraEntity->FlagForUpdate();
		UpdateFromTemplate();
	}
}



