///////////////////////////////////////////////////////////////////////////////
//
// File   : Tools_FSFX_Overrides.h
// Author : Andy Southgate
// Date   : 21-Apr-2009
//
// Copyright (C) 2009 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "FSFX/Tools_FSFX_Overrides.h"

// Asura independent header-files...
#include "Core/Asura_SortAndSearch.h"
#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Overlay.h"
#include "Interface/Asura_Overlay_System.h"
#include "Interface/Asura_Table_Dialog.h"
#include "ChunkSystem/Asura_SaveBlock.h"
#include "ChunkSystem/Asura_Chunk_Loaders.h"
#include "ChunkSystem/Tools_Chunk_FSFXOverrides.h"
#include "FSFX/Asura_FSFX_System.h"
#include "Materials/Tools_Overlay_MaterialResponseOverrides.h"

// Tools header-files...
#include "Tools_Overlays.h"
#include "Misc/Tools_Timed_Messages.h"
#include "FSFX/Tools_Mode_FSFX.h"

#include "Particles/Overrides/Tools_UI_Overrides.h"


////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////

static const char*	g_szFSFXOverridesFilename	= "Misc\\FSFXOverrides.asr";

static const Asura_Vector_2	g_xTestMinimumCellSize(6.0f, 2.0f);
static Asura_Vector_2 g_xTableSize(80.0f, 21.0f);

////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Private functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Protected functions...
////////////////////////////////////////////////////////////////////////////////

bool Tools_UI_Overrides_FSFXInterface::ActivateOverrideMap(Asura_Hash_ID uOverrideMapHash )
{
	Asura_FSFX_System::EnableOverrideMap(uOverrideMapHash, true);
	return true;
}
bool Tools_UI_Overrides_FSFXInterface::CreateNewOverrideMap		(const Asura_String &xOverrideMapName)
{
	Asura_FSFX_Template_OverrideMap *pxMap = Asura_FSFX_System::AllocateOverrideMap(xOverrideMapName.GetHash());
	if (pxMap)
	{
		pxMap->SetName(xOverrideMapName.GetString());
		RefreshAll();
	}
	return pxMap != NULL;
}
bool Tools_UI_Overrides_FSFXInterface::CopyOverrideMap			(Asura_Hash_ID uFrom, const Asura_String &xTo)
{
	Asura_FSFX_Template_OverrideMap *pxNew  = Asura_FSFX_System::AllocateOverrideMap(xTo.GetHash());
	Asura_FSFX_Template_OverrideMap *pxOld  = Asura_FSFX_System::GetOverrideMapByHash(uFrom);

	if (pxNew && pxOld)
	{
		*pxNew = *pxOld;
		pxNew->SetName(xTo.GetString());
		RefreshAll();
		return true;
	}
	return false;
}
bool Tools_UI_Overrides_FSFXInterface::DeactivateOverrideMap		(Asura_Hash_ID uOverrideMapHash )
{
	Asura_FSFX_System::DisableOverrideMap(uOverrideMapHash);
	return true;
}
bool Tools_UI_Overrides_FSFXInterface::DeleteOverrideMap			(Asura_Hash_ID uOverrideMapHash )
{
	Asura_FSFX_System::FreeOverrideMap(uOverrideMapHash);
	RefreshAll();
	return true;
}
bool Tools_UI_Overrides_FSFXInterface::RenameOverrideMap			(Asura_Hash_ID uFrom, const Asura_String &xTo )
{
	return CopyOverrideMap(uFrom, xTo) && DeleteOverrideMap(uFrom);
}

bool Tools_UI_Overrides_FSFXInterface::IsOverrideMapActive		(Asura_Hash_ID uOverrideMapHash)
{
	return Asura_FSFX_System::IsOverrideMapEnabled(uOverrideMapHash);
}
bool Tools_UI_Overrides_FSFXInterface::IsItemOverriden			(Asura_Hash_ID uItemHash)
{
	Asura_FSFX_Template_ResourceSet_ConstMapIterator xIter(Asura_FSFX_System::GetResourceSet());

	for (;!xIter.Done(); xIter.Next())
	{
		if (xIter.GetCurrent()->GetOverride(uItemHash) != uItemHash)
			return true;
	}
	return false;
}

bool Tools_UI_Overrides_FSFXInterface::DoesOverrideMapExist		(Asura_Hash_ID uOverrideMapHash)
{
	return Asura_FSFX_System::GetOverrideMapByHash(uOverrideMapHash) != NULL;
}

static int CompareStringsCaseInsensitive( const void* p1, const void* p2 )
{
	const Asura_String*szOne = reinterpret_cast<const Asura_String*>(p1);
	const Asura_String*szTwo = reinterpret_cast<const Asura_String*>(p2);
	return stricmp(szOne->GetString(), szTwo->GetString());
}

bool Tools_UI_Overrides_FSFXInterface::GetEntryNames(Asura_String* &pxNameArray, int &iNumEntries)
{
	Asura_FSFX_System::GetTemplateNames(pxNameArray, iNumEntries);
	Asura_SortAndSearch::Quicksort(pxNameArray, iNumEntries, CompareStringsCaseInsensitive);
	return true;
}
bool Tools_UI_Overrides_FSFXInterface::GetOverrideNames(Asura_String* &pxOverrideNameArray, int &iNumOverrides)
{
	iNumOverrides = Asura_FSFX_System::GetNumOverrideMaps();
	pxOverrideNameArray = new Asura_String[iNumOverrides];
	int i = 0;
	for (Asura_FSFX_Template_ResourceSet_ConstMapIterator xIter(Asura_FSFX_System::GetResourceSet()); !xIter.Done(); ++i, xIter.Next())
	{
		pxOverrideNameArray[i] = xIter.GetCurrent()->GetName();
	}
	return true;
}

Asura_Hash_ID Tools_UI_Overrides_FSFXInterface::GetCurrentOverrideMap()
{
	Asura_FSFX_Template_ResourceSet_ConstMapIterator xIter(Asura_FSFX_System::GetResourceSet());

	for (; !xIter.Done(); xIter.Next())
	{
		if (Asura_FSFX_System::IsOverrideMapEnabled(xIter.GetCurrentHash()))
		{
			return xIter.GetCurrentHash();
		}
	}

	return  ASURA_HASH_ID_UNSET;
}

bool Tools_UI_Overrides_FSFXInterface::GetOverrideMapEntry		(Asura_Hash_ID uOverrideMapHash, Asura_Hash_ID uEntryHash, Asura_Hash_ID &uOverriddenEntryHash)
{
	Asura_FSFX_Template_OverrideMap *pxMap = Asura_FSFX_System::GetOverrideMapByHash(uOverrideMapHash);

	if (pxMap)
	{
		uOverriddenEntryHash = pxMap->GetOverride(uEntryHash);
		return true;
	}
	uOverriddenEntryHash = uEntryHash;
	return false;
}
bool Tools_UI_Overrides_FSFXInterface::RemoveOverrideMapEntry		(Asura_Hash_ID uOverrideMapHash, Asura_Hash_ID uEntryHash)
{
	Asura_FSFX_Template_OverrideMap *pxMap = Asura_FSFX_System::GetOverrideMapByHash(uOverrideMapHash);

	if (pxMap)
	{
		pxMap->RemoveOverride(uEntryHash);
		RefreshAll();
		return true;
	}
	return false;
}

bool Tools_UI_Overrides_FSFXInterface::SetOverrideMapEntry	(Asura_Hash_ID uOverrideMapHash, const Asura_String &xFrom, const Asura_String &xTo)
{
	Asura_FSFX_Template_OverrideMap *pxMap = Asura_FSFX_System::GetOverrideMapByHash(uOverrideMapHash);

	if (pxMap)
	{
		Asura_Hash_ID uHashFrom = xFrom.GetHash();
		Asura_Hash_ID uHashTo = xTo.GetHash();
		Asura_FSFX_Template_OverrideEntry *pxEntry = pxMap->GetOverrideEntry(uHashFrom);

		if (pxEntry)
		{
			if (uHashTo != uHashFrom)
			{
				pxEntry->SetNewName(xTo);
			}
			else
			{
				pxMap->RemoveOverride(uHashFrom);
			}
		} 
		else 
		{
			if (uHashTo != uHashFrom)
			{
				pxMap->CreateOverride(xFrom, xTo);

			}

		}

		Tools_FSFX_Overrides::RefreshFSFXCell(uOverrideMapHash, uHashFrom);

		return true;
	}
	return false;
}

void Tools_UI_Overrides_FSFXInterface::RefreshAll()
{
	Tools_FSFX_Overrides::RefreshAllFSFX();
}





////////////////////////////////////////////////////////////////////////////////
// Class        : Tools_FSFX_Overrides_Overlay
// Author       : Richard Rice
// Date         : 15/11/2007
//
// Description  : Derived overlay class for detecting if it is closed by the X button
//				  
////////////////////////////////////////////////////////////////////////////////

class Tools_FSFX_Overrides_Overlay : public Asura_Overlay
{
public :
	Tools_FSFX_Overrides_Overlay(Tools_FSFX_Overrides* pxOverrides, const char *szName, bool bAlwaysOnTop, Asura_Vector_2* pxPosition)
		:Asura_Overlay(szName, bAlwaysOnTop, pxPosition)
		, m_pxOverrides(pxOverrides)
	{}

	~Tools_FSFX_Overrides_Overlay()
	{
		m_pxOverrides->OnOverlayDeletion(this);
	}

	Tools_FSFX_Overrides* m_pxOverrides;
};


Tools_FSFX_Overrides* Tools_FSFX_Overrides::m_pxFirst = NULL;

Tools_FSFX_Overrides::Tools_FSFX_Overrides()
:	m_pxOverlay( NULL ), 
m_pxTabsDialog( NULL ),
m_xOverlayPosition( 6.0f, 6.0f ),
m_xOverridesUI(m_xOverridesUIInterface)
{
	m_pxNext = m_pxFirst;
	m_pxFirst = this;
}

Tools_FSFX_Overrides::~Tools_FSFX_Overrides()
{
	Tools_FSFX_Overrides** ppx = &m_pxFirst;
	while( *ppx != NULL )
	{
		Tools_FSFX_Overrides* px = *ppx;
		if( px == this )
		{
			//remove it from the chain
			*ppx = px->m_pxNext;
			break;
		}
		ppx = &px->m_pxNext;
	}

	// Close the dialog
	Close();
}

////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_FSFX_Overrides::CreateOverlay
// Author       : Andy Tate
// Date         : 26-Apr-2007
////////////////////////////////////////////////////////////////////////////////

void Tools_FSFX_Overrides::CreateOverlay(void)
{
	if( m_pxOverlay == NULL )
	{
//		Tools_Mode_FSFX::Verify(true);

		Asura_Assert( ( m_pxTabsDialog == NULL ), "" );
		Asura_Vector_2	xOffset(1.0f, 3.0f);
		Asura_Dialog* pxNewDialog;

		// Store the overlay...
		m_pxOverlay		= new Tools_FSFX_Overrides_Overlay(this, "FSFX Overrides", false, &m_xOverlayPosition);
		m_pxOverlay->m_uID	= TOOLS_OVERLAY_FSFX_OVERRIDES;

		// Only show materials with an override?

		AddDialogs( xOffset, g_xTableSize );

		// Save
		pxNewDialog = new Asura_Function_Dialog( "Save", xOffset, Save, (int)this );
		m_pxOverlay->AddDialog( pxNewDialog );
		xOffset.y += Asura_Dialog::DefaultSpacing();

		// Save and close...
		pxNewDialog = new Asura_Function_Dialog("Save + Close", xOffset, SaveAndClose, (int)this );
		m_pxOverlay->AddDialog( pxNewDialog );
		xOffset.y		+= Asura_Dialog::DefaultSpacing();

		//		pxNewDialog = new Asura_Resize_Dialog(m_pxSourceTableDialog, &g_xTableSize);
		//		AddDialog(pxNewDialog);

		// Finish off...
		//		m_pxOverlay->MakeDialogsRelativeToResizableDialog();
		Asura_Overlay_System::SubmitNewOverlay(m_pxOverlay);
	}
}


void Tools_FSFX_Overrides::AddDialogs( Asura_Vector_2& xOffset, const Asura_Vector_2& xTableSize )
{

	Asura_Tabs_Dialog* pxTabsDialog = new Asura_Tabs_Dialog(xOffset, 0, 0);
	m_pxOverlay->AddDialog(pxTabsDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();
	m_pxOverlay->SetCurrentParentDialog(pxTabsDialog);

	m_xOverridesUI.AddDialogs(m_pxOverlay, xOffset, xTableSize);


	m_xOverridesUI.Refresh();


	pxTabsDialog->SelectTab(0);


	m_pxOverlay->SetCurrentParentDialog(0);
}

////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_FSFX_Overrides::SaveAndClose
// Author       : Andy Tate
// Date         : 26-Apr-2007
////////////////////////////////////////////////////////////////////////////////

void Tools_FSFX_Overrides::SaveAndClose( void )
{
	// Save the file
	Save();
	// Close the dialog
	Close();
}

void Tools_FSFX_Overrides::Save( void )
{
	// Save the file to disc...
	if( Tools_Chunk_FSFXOverrides::Save( g_szFSFXOverridesFilename ) )
	{
		Asura_String	xMessage("Saved \"");

		xMessage	+= g_szFSFXOverridesFilename;
		xMessage	+= "\" successfully.";

		Tools_Timed_Message_System::AddMessage( 0x00FF00, xMessage.GetString() );
	}
	else
	{
		Asura_String	xMessage( "Failed to save \"");

		xMessage	+= g_szFSFXOverridesFilename;
		xMessage	+= "\" - write-protected?";

		Tools_Timed_Message_System::AddMessage( 0xFF0000, xMessage.GetString() );
	}
}

void Tools_FSFX_Overrides::Close( void )
{

	// Destroy our existing table...
	m_xOverridesUI.DestroyTable();

	// Close the window...
	if( m_pxOverlay != NULL )
	{
		m_pxOverlay->m_bShouldClose = true;
	}

	// Flag as closed...
	m_pxOverlay							= NULL;
}

void Tools_FSFX_Overrides::OnOverlayDeletion(Asura_Overlay* pxOverlay)
{
	//If the system still has a pointer to the overlay , then it hasn't cleared things up yet
	//and so needs to do SaveAndClose() to save and deallocate everything.
	//RAR 15/11/2007

	if(m_pxOverlay)
	{
		Asura_Assert(pxOverlay == m_pxOverlay, "Not deleting the expected overlay");
		SaveAndClose();
	}

	m_xOverridesUI.RemoveDialogs();
}

/// End of function 'Tools_FSFX_Overrides::SaveAndClose'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_FSFX_Overrides::RefreshAll
// Author       : Andy Tate
// Date         : 3-May-2007
////////////////////////////////////////////////////////////////////////////////

void Tools_FSFX_Overrides::RefreshAllFSFX( void )
{
	Tools_FSFX_Overrides* px = m_pxFirst;
	while( px != NULL )
	{
		px->m_xOverridesUI.DestroyTable();
		px->m_xOverridesUI.CreateTable();
		px = px->m_pxNext;
	}
}



void Tools_FSFX_Overrides::RefreshFSFXCell( Asura_Hash_ID uOverrideMapHash, Asura_Hash_ID uSourceHash  )
{
	for( Tools_FSFX_Overrides* px = m_pxFirst; px != NULL;  px = px->m_pxNext)
	{
		px->m_xOverridesUI.RefreshCell(uOverrideMapHash, uSourceHash);
	}
}


/// End of function 'Tools_FSFX_Overrides::RefreshAll'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_FSFX_Overrides::CreateTable
// Author       : Andy Tate
// Date         : 1-May-2007
////////////////////////////////////////////////////////////////////////////////



void Tools_FSFX_Overrides::Initialise(void)
{
	LoadAsuraFile( g_szFSFXOverridesFilename );
}

/// End of function 'Tools_FSFX_Overrides::Initialise'. ///




void Tools_FSFX_Overrides::Shutdown(void)
{
	if ( IsOpen() )
	{
		ToggleOverlay();
	}
}


void Tools_FSFX_Overrides::ToggleOverlay(void)
{
	if(IsOpen())
	{
		SaveAndClose();
	}
	else
	{
		CreateOverlay();
	}
}

