
///////////////////////////////////////////////////////////////////////////////
//
// File   : Tools_Overlay_FacialAnimation_EditFace.cpp
// Author : Andy Tate
// Date   : 9-May-2008
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

#if defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)

// This file's header...
#include "FacialAnimation/Overlays/Tools_Overlay_FacialAnimation_EditFace.h"

// Asura independent header-files...
#include "Animation/Asura_Animation.h"
#include "ChunkSystem/Asura_Chunk_Loaders.h"
#include "FacialAnimation/Asura_FacialAnimation_MuscleGroup.h"
#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Overlay_System.h"
#include "Interface/Asura_Tree_Dialog.h"

// Tools header-files...
#include "Tools_Overlays.h"
#include "FacialAnimation/Tools_FacialAnimation_Face.h"
#include "FacialAnimation/Tools_FacialAnimation_Face_MuscleGroup.h"
#include "FacialAnimation/Tools_FacialAnimation_System.h"
#include "FacialAnimation/Overlays/Tools_ModalOverlay_FacialAnimation_FaceTypeSelection.h"
#include "FacialAnimation/Overlays/Tools_ModalOverlay_FacialAnimation_MuscleGroupSelectControlType.h"
#include "Hierarchy/Tools_Animation_Files.h"
#include "Misc/Tools_Modal_Overlay.h"
#include "Misc/Tools_Timed_Messages.h"


////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////

static const char*	g_szNoneText	= "[None]";


////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////

const char* Tools_Overlay_FacialAnimation_EditFace::s_aszMuscleGroupColumnNames[MAX_MUSCLE_GROUP_COLUMNS] =
{
	"Muscle-group",	// MGC_NAME
	"Control-Type",	// MGC_CONTROL_TYPE
	"Animation(s)",	// MGC_ANIMATIONS
};

Tools_Overlay_FacialAnimation_EditFace*	Tools_Overlay_FacialAnimation_EditFace::s_pxThis	= NULL;


////////////////////////////////////////////////////////////////////////////////
// Private functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Protected functions...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Overlay_FacialAnimation_EditFace::AddOverlayItems
// Author       : Andy Tate
// Date         : 9-May-2008
////////////////////////////////////////////////////////////////////////////////

void Tools_Overlay_FacialAnimation_EditFace::AddOverlayItems(Asura_Vector_2& xOffset)
{
	const Asura_Vector_2	xInitialOffset(xOffset);

	Asura_Dialog*			pxNewDialogue;

	// Create the face-type binding...
	pxNewDialogue	= new Asura_Dialog("Face-type:", xOffset);
	AddDialog(pxNewDialogue);

	xOffset.y					+= Asura_Dialog::DefaultSpacing();
	m_pxSelectFaceTypeDialogue	 = new Asura_Function_Dialog("", xOffset, OnSelectFaceType, reinterpret_cast<int>(this));
	AddDialog(m_pxSelectFaceTypeDialogue);

	// Ensure that the face-type is up-to-date...
	UpdateFaceType();

	// Create the muscle-group list...
	xOffset.y				+= Asura_Dialog::LargeSpacing();
	m_pxMuscleGroupDialogue	 = new Asura_Table_Dialog (xOffset, 50.0f, 20.0f, true);
	xOffset.y				+= m_pxMuscleGroupDialogue->GetTableSize().y;
	AddDialog(m_pxMuscleGroupDialogue);

	// Ensure that our data is up-to-date...
	UpdateMuscleGroups();

	// Create the "Save & Close" button...
	xOffset.y		+= Asura_Dialog::LargeSpacing();
	pxNewDialogue	 = new Asura_Function_Dialog("Save & Close", xOffset, OnSaveAndCloseSelected, reinterpret_cast<int>(this));
	AddDialog(pxNewDialogue);
	pxNewDialogue->SetRelativePosition(m_pxMuscleGroupDialogue, true, false);
}

/// End of function 'Tools_Overlay_FacialAnimation_EditFace::AddOverlayItems'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Overlay_FacialAnimation_EditFace::UpdateMuscleGroups
// Author       : Andy Tate
// Date         : 9-May-2008
////////////////////////////////////////////////////////////////////////////////

void Tools_Overlay_FacialAnimation_EditFace::UpdateMuscleGroups(void)
{
	const Tools_FacialAnimation_Face::MuscleGroupContainer&	xMuscleGroupContainer	= m_xFace.GetMuscleGroupContainer();
	Tools_FacialAnimation_Face::MuscleGroupConstIterator	xMuscleGroupIterator(xMuscleGroupContainer);
	const Asura_Vector_2									xMinCellSize(1.0f, 1.0f);

	// Remove any existing muscle-groups...
	delete[](m_pszMuscleGroupNames);
	m_pxMuscleGroupDialogue->DestroyTable();

	// Create the new array of strings...
	m_iNumMuscleGroups		= xMuscleGroupContainer.GetNumEntries();

	if(m_iNumMuscleGroups > 0)
	{
		m_pszMuscleGroupNames	= new const char* [m_iNumMuscleGroups];

		for(int i = 0; i < m_iNumMuscleGroups; i++, xMuscleGroupIterator.Next())
		{
			const Tools_FacialAnimation_Face_MuscleGroup*	pxMuscleGroup	= xMuscleGroupIterator.GetCurrent();
			const Asura_String&								xName			= pxMuscleGroup->GetName();

			m_pszMuscleGroupNames[i]	= xName.GetString();
		}

		// Create the table itself...
		m_pxMuscleGroupDialogue->SetHidden(false);
		m_pxMuscleGroupDialogue->CreateTable
		(
			xMinCellSize,
			m_iNumMuscleGroups,
			MAX_MUSCLE_GROUP_COLUMNS,
			CreateMuscleGroupTableCell,
			&m_xFace,
			NULL,
			s_aszMuscleGroupColumnNames
		);
	}
	else
	{
		m_pszMuscleGroupNames	= NULL;
		m_pxMuscleGroupDialogue->SetHidden(true);
	}
}

/// End of function 'Tools_Overlay_FacialAnimation_EditFace::UpdateMuscleGroups'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Overlay_FacialAnimation_EditFace::UpdateFaceType
// Author       : Andy Tate
// Date         : 3-Jul-2008
////////////////////////////////////////////////////////////////////////////////

void Tools_Overlay_FacialAnimation_EditFace::UpdateFaceType(void)
{
	const Asura_String&	xFaceTypeName	= m_xFace.GetFaceTypeName();
	const char*			szFaceTypeName	= (xFaceTypeName.IsBlank())
										? g_szNoneText
										: xFaceTypeName.GetString();

	m_pxSelectFaceTypeDialogue->UpdateLabel(szFaceTypeName);
}

/// End of function 'Tools_Overlay_FacialAnimation_EditFace::UpdateFaceType'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Overlay_FacialAnimation_EditFace::Tools_Overlay_FacialAnimation_EditFace
// Author       : Andy Tate
// Date         : 9-May-2008
////////////////////////////////////////////////////////////////////////////////

Tools_Overlay_FacialAnimation_EditFace::Tools_Overlay_FacialAnimation_EditFace(const bool bNewFace,
																			   const Asura_String& xPath,
																			   Tools_FacialAnimation_Face& xFace)
: PARENT(xFace.GetName().GetString(), true)
, m_xPath(xPath)
, m_xFace(xFace)
, m_bChanged(bNewFace)
, m_iNumMuscleGroups(0)
, m_pszMuscleGroupNames(NULL)
, m_pxMuscleGroupDialogue(NULL)
{
	Asura_Vector_2	xOffset(1.0f, 3.0f);

	// Initialise the variables...
	m_uID			= TOOLS_OVERLAY_FACIALANIMATION_EDITFACE;
	s_pxThis	= this;

	// Finally, update the overlays...
	AddOverlayItems(xOffset);
}

/// End of function 'Tools_Overlay_FacialAnimation_EditFace::Tools_Overlay_FacialAnimation_EditFace'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Overlay_FacialAnimation_EditFace::~Tools_Overlay_FacialAnimation_EditFace
// Author       : Andy Tate
// Date         : 9-May-2008
////////////////////////////////////////////////////////////////////////////////

Tools_Overlay_FacialAnimation_EditFace::~Tools_Overlay_FacialAnimation_EditFace()
{
	delete[](m_pszMuscleGroupNames);
	s_pxThis	= NULL;
}

/// End of function 'Tools_Overlay_FacialAnimation_EditFace::~Tools_Overlay_FacialAnimation_EditFace'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Overlay_FacialAnimation_EditFace::CreateMuscleGroupTableCell
// Author       : Andy Tate
// Date         : 13-May-2008
////////////////////////////////////////////////////////////////////////////////

Asura_TableCell_Dialog* Tools_Overlay_FacialAnimation_EditFace::CreateMuscleGroupTableCell(Asura_Table_Dialog& xTableDialog,
																						   const Asura_Vector_2& xOffset,
																						   const Asura_Vector_2& xMinCellSize,
																						   const u_int uRow,
																						   const u_int uColumn,
																						   void* const pxUserData)
{
	const Tools_FacialAnimation_Face_MuscleGroup* pxMuscleGroup = s_pxThis->FindMuscleGroup(uRow);

	switch(uColumn)
	{
		case MGC_NAME:
		{
			const Asura_String&	xName	= pxMuscleGroup->GetName();
			const char*			szName	= xName.GetString();

			return(Asura_TableRowHeading_StandardDialog::Create(xTableDialog, szName, xOffset, xMinCellSize, uRow));
		}

		case MGC_CONTROL_TYPE:
		{
			const Asura_FacialAnimation_MuscleGroup::ControlType	eControlType	= pxMuscleGroup->GetControlType();
			const char*												szName			= Asura_FacialAnimation_MuscleGroup::Tools_GetControlTypeName(eControlType);

			return(Asura_TableCell_TextDialog::Create(xTableDialog, szName, xOffset, xMinCellSize, uRow, uColumn));
		}

		case MGC_ANIMATIONS:
		{
			return(new Tools_TableCell_MuscleGroupSelectAnimation_Dialog(xTableDialog, xOffset, xMinCellSize, uRow, uColumn, *s_pxThis));
		}

		default:
		{
			Asura_Assert(false, "Unexpected column found when creating muscle-group table.");
			return(NULL);
		}
	}		// End of switch.
}

/// End of function 'Tools_Overlay_FacialAnimation_EditFace::CreateMuscleGroupTableCell'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Overlay_FacialAnimation_EditFace::OnSelectFaceType
// Author       : Andy Tate
// Date         : 3-Jul-2008
////////////////////////////////////////////////////////////////////////////////

void Tools_Overlay_FacialAnimation_EditFace::OnSelectFaceType(int iUserData)
{
	Tools_Overlay_FacialAnimation_EditFace*	pxThis	= reinterpret_cast<Tools_Overlay_FacialAnimation_EditFace*>(iUserData);

	const char*	szFaceTypeDirectory		= Tools_FacialAnimation_System::GetTypeDirectoryName();
	const char*	szFaceTypeFileExtension	= Tools_FacialAnimation_System::GetTypeFileExtension();

	FileSelect_Overlay xSelectFaceType("Select face-type...", szFaceTypeDirectory, szFaceTypeFileExtension, false );


	if(xSelectFaceType.DoModal())
	{
		Asura_Hash_ID					uFaceTypeHash;
		Asura_String					xPath;

		xSelectFaceType.GetSelectedFile( uFaceTypeHash, xPath );

		// security check. Probably unnecessary, but one never know.
		Tools_FacialAnimation_FaceType*	pxFaceType	= Tools_FacialAnimation_System::FindOrLoadType( xPath );
		if( pxFaceType )
		{
			// Ok, I found the face, so use it.
			pxThis->SetCurrentFaceType(uFaceTypeHash);
		}
		else
		{
			// Something wrong is going on. It seems that system found the file existing, but it can't load it.
			ShowMessageOverlay("Failed to find the specified face-type \"%s\".", xPath.GetString());
		}
	}
}

/// End of function 'Tools_Overlay_FacialAnimation_EditFace::OnSelectFaceType'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Overlay_FacialAnimation_EditFace::OnSaveAndCloseSelected
// Author       : Andy Tate
// Date         : 8-May-2008
////////////////////////////////////////////////////////////////////////////////

void Tools_Overlay_FacialAnimation_EditFace::OnSaveAndCloseSelected(int iUserData)
{
	Tools_Overlay_FacialAnimation_EditFace*	pxThis	= reinterpret_cast<Tools_Overlay_FacialAnimation_EditFace*>(iUserData);

	if(pxThis->m_bChanged)
	{
		const Tools_FacialAnimation_Face&	xFace		= pxThis->m_xFace;
		const Asura_String					xFilename(Tools_FacialAnimation_System::CreateFaceFilename(pxThis->m_xPath, true));
		const char*							szFilename	= xFilename.GetString();

		// Attempt to save the file...
		if(Tools_FacialAnimation_System::WriteFaceFile(szFilename, xFace))
		{
			Tools_Timed_Message_System::AddMessage(Tools_Timed_Message_System::COLOUR_SUCCESS, "\"%s\" saved successfully.", szFilename);
		}
		else
		{
			Tools_Timed_Message_System::AddMessage(Tools_Timed_Message_System::COLOUR_ERROR, "Failed to save \"%s\".", szFilename);
		}
	}

	// Finally, close the overlay...
	CloseOverlay();
}

/// End of function 'Tools_Overlay_FacialAnimation_EditFace::OnSaveAndCloseSelected'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Overlay_FacialAnimation_EditFace::SetCurrentFaceType
// Author       : Andy Tate
// Date         : 3-Jul-2008
////////////////////////////////////////////////////////////////////////////////

void Tools_Overlay_FacialAnimation_EditFace::SetCurrentFaceType(const Asura_Hash_ID uFaceTypeHash)
{
	Tools_FacialAnimation_FaceType*	pxFaceType	= Tools_FacialAnimation_System::FindType(uFaceTypeHash);

	// Ensure that we have the correct face-type AND muscle-groups in this face...
	if(pxFaceType != NULL)
	{
		const Tools_FacialAnimation_FaceType::MuscleGroupContainer&	xMuscleGroupContainer	= pxFaceType->GetMuscleGroupContainer();
		const Asura_String&											xFaceTypeName			= pxFaceType->GetName();

		m_xFace.SetFaceTypeName(xFaceTypeName);

		for(Tools_FacialAnimation_FaceType::MuscleGroupConstIterator xIterator(xMuscleGroupContainer); !xIterator.Done(); xIterator.Next())
		{
			const Tools_FacialAnimation_FaceType_MuscleGroup*	pxFaceTypeMuscleGroup	= xIterator.GetCurrent();
			const Asura_String&									xMuscleGroupName		= pxFaceTypeMuscleGroup->GetName();
			const char*											szMuscleGroupName		= xMuscleGroupName.GetString();
			Tools_FacialAnimation_Face_MuscleGroup*				pxMuscleGroup			= m_xFace.FindMuscleGroup(szMuscleGroupName);

			if(pxMuscleGroup == NULL)
			{
				const Asura_FacialAnimation_MuscleGroup::ControlType	eControlType	= pxFaceTypeMuscleGroup->GetControlType();

				pxMuscleGroup	= m_xFace.CreateMuscleGroup(szMuscleGroupName, eControlType);
			}
		}
	}

	// Update the dialogues...
	UpdateFaceType();
	UpdateMuscleGroups();

	// Don't forget to tell the overlay that we've changed data...
	SetAsChanged();
}

/// End of function 'Tools_Overlay_FacialAnimation_EditFace::SetCurrentFaceType'. ///



////////////////////////////////////////////////////////////////////////////////
// Public functions...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Overlay_FacialAnimation_EditFace::CreateOverlay
// Author       : Andy Tate
// Date         : 9-May-2008
////////////////////////////////////////////////////////////////////////////////

void Tools_Overlay_FacialAnimation_EditFace::CreateOverlay(const bool bNewFace,
														   const Asura_String& xPath,
														   Tools_FacialAnimation_Face& xFace)
{
	if(Asura_Overlay_System::DoesOverlayExist(TOOLS_OVERLAY_FACIALANIMATION_EDITFACE))
	{
		Asura_Overlay_System::BringOverlayToTop(TOOLS_OVERLAY_FACIALANIMATION_EDITFACE);
	}
	else
	{
		Tools_Overlay_FacialAnimation_EditFace*	pxOverlay	= new Tools_Overlay_FacialAnimation_EditFace (bNewFace, xPath, xFace);

		Asura_Overlay_System::SubmitNewOverlay(pxOverlay, false, false);
	}
}

/// End of function 'Tools_Overlay_FacialAnimation_EditFace::CreateOverlay'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Overlay_FacialAnimation_EditFace::CloseOverlay
// Author       : Andy Tate
// Date         : 9-May-2008
////////////////////////////////////////////////////////////////////////////////

void Tools_Overlay_FacialAnimation_EditFace::CloseOverlay(void)
{
	Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_FACIALANIMATION_EDITFACE);
}

/// End of function 'Tools_Overlay_FacialAnimation_EditFace::CloseOverlay'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Overlay_FacialAnimation_EditFace::FindMuscleGroup
// Author       : Andy Tate
// Date         : 13-May-2008
////////////////////////////////////////////////////////////////////////////////

Tools_FacialAnimation_Face_MuscleGroup* Tools_Overlay_FacialAnimation_EditFace::FindMuscleGroup(const u_int uIndex)
{
	const char*								szName			= m_pszMuscleGroupNames[uIndex];
	Tools_FacialAnimation_Face_MuscleGroup*	pxMuscleGroup	= m_xFace.FindMuscleGroup(szName);

	return(pxMuscleGroup);
}

/// End of function 'Tools_Overlay_FacialAnimation_EditFace::FindMuscleGroup'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Overlay_FacialAnimation_EditFace::SetAsChanged
// Author       : Andy Tate
// Date         : 14-May-2008
////////////////////////////////////////////////////////////////////////////////

void Tools_Overlay_FacialAnimation_EditFace::SetAsChanged(void)
{
	m_bChanged	= true;

	if(m_pxMuscleGroupDialogue != NULL)
	{
		m_pxMuscleGroupDialogue->UpdateAll();
	}
}

/// End of function 'Tools_Overlay_FacialAnimation_EditFace::SetAsChanged'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_TableCell_MuscleGroupSelectAnimation_Dialog::Tools_TableCell_MuscleGroupSelectAnimation_Dialog
// Author       : Andy Tate
// Date         : 13-May-2008
////////////////////////////////////////////////////////////////////////////////

Tools_TableCell_MuscleGroupSelectAnimation_Dialog::Tools_TableCell_MuscleGroupSelectAnimation_Dialog(Asura_Table_Dialog& xTableDialogue,
																									 const Asura_Vector_2& xOffset,
																									 const Asura_Vector_2& xMinCellSize,
																									 const u_int uRow,
																									 const u_int uColumn,
																									 Tools_Overlay_FacialAnimation_EditFace& xEditFaceOverlay)
: PARENT(xTableDialogue, xOffset, xMinCellSize, uRow, uColumn)
, m_xEditFaceOverlay(xEditFaceOverlay)
{
	m_pxMuscleGroup	= xEditFaceOverlay.FindMuscleGroup(uRow);

	if(m_pxMuscleGroup != NULL)
	{
		const Asura_FacialAnimation_MuscleGroup::ControlType	eControlType	= m_pxMuscleGroup->GetControlType();
		Asura_Vector_2											xLocalOffset(0.0f, 0.0f);

		switch(eControlType)
		{
			case Asura_FacialAnimation_MuscleGroup::CT_SINGLE:
			case Asura_FacialAnimation_MuscleGroup::CT_BALANCE:
			{
				const Asura_String&	xAnimationName	= m_pxMuscleGroup->GetAnimationName(0);
				const char*			szAnimationName	= xAnimationName.GetString();

				m_apxSelectAnimationDialogues[0]	= new Asura_Function_Dialog (szAnimationName, xLocalOffset, OnSelectAnimation, reinterpret_cast<int>(this));
				AddDialogToPanel(m_apxSelectAnimationDialogues[0]);
				break;
			}

			case Asura_FacialAnimation_MuscleGroup::CT_SINGLE_PAIR:
			case Asura_FacialAnimation_MuscleGroup::CT_BALANCE_PAIR:
			{
				const Asura_String&	xHighAnimationName	= m_pxMuscleGroup->GetAnimationName(0);
				const char*			szHighAnimationName	= xHighAnimationName.GetString();
				const Asura_String&	xLowAnimationName	= m_pxMuscleGroup->GetAnimationName(1);
				const char*			szLowAnimationName	= xLowAnimationName.GetString();

				m_apxSelectAnimationDialogues[0]	= new Asura_Function_Dialog (szHighAnimationName, xLocalOffset, OnSelectHighAnimation, reinterpret_cast<int>(this));
				AddDialogToPanel(m_apxSelectAnimationDialogues[0]);

				xLocalOffset[1]						= m_apxSelectAnimationDialogues[0]->m_xMaxCoord[1];
				m_apxSelectAnimationDialogues[1]	= new Asura_Function_Dialog (szLowAnimationName, xLocalOffset, OnSelectLowAnimation, reinterpret_cast<int>(this));
				AddDialogToPanel(m_apxSelectAnimationDialogues[1]);
				break;
			}

			default:
			{
				Asura_Assert(false, "Unexpected control-type for this muscle-group.");
				break;
			}
		}		// End of switch.
	}
}

/// End of function 'Tools_TableCell_MuscleGroupSelectAnimation_Dialog::Tools_TableCell_MuscleGroupSelectAnimation_Dialog'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_TableCell_MuscleGroupSelectAnimation_Dialog::OnSelectAnimation
// Author       : Andy Tate
// Date         : 23-May-2008
////////////////////////////////////////////////////////////////////////////////

void Tools_TableCell_MuscleGroupSelectAnimation_Dialog::OnSelectAnimation(const u_int uIndex,
																		  const char* szMessage)
{
	Asura_String	xFilename;

	if(Tools_Animation_Files::BrowseForAsuraFile(szMessage, xFilename, true))
	{
		Asura_Dialog*		pxDialog	= m_apxSelectAnimationDialogues[uIndex];
		const Asura_String&	xOldName	= pxDialog->GetName();

		Asura_String		xNewName;

		xFilename.SplitFilename(NULL, &xNewName, NULL);

		if(xNewName != xOldName)
		{
			const char*					szFilename		= xFilename.GetString();
			const char*					szAnimationName	= xNewName.GetString();
			const Asura_Hash_ID			uAnimationHash	= xNewName.GetHash();
			const Asura_Hierarchy_Anim*	pxAnimation		= Asura_Animation_System::GetAnimByHash(uAnimationHash);

			// Ensure that we attempt to load the animation if we don't already have it in memory...
			if((pxAnimation == NULL) && !LoadAsuraFile(szFilename))
			{
				Tools_Timed_Message_System::AddMessage(Tools_Timed_Message_System::COLOUR_ERROR, "Failed to load \"%s\".", szFilename);
			}
			else
			{
				if(pxAnimation == NULL)
				{
					Tools_Timed_Message_System::AddMessage(Tools_Timed_Message_System::COLOUR_ERROR, "File \"%s\" does not contain an animation.", szFilename);
				}
				else
				{
					// Set the animation used by the muscle-group...
					m_pxMuscleGroup->SetAnimationName(uIndex, szAnimationName);

					// Update the dialogue, table-cell and overlay...
					pxDialog->UpdateLabel(szAnimationName);
					UpdateSize();
					m_xEditFaceOverlay.SetAsChanged();
				}
			}
		}
	}
}

/// End of function 'Tools_TableCell_MuscleGroupSelectAnimation_Dialog::OnSelectAnimation'. ///

#endif	//defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)


/// End of file. ///
