
///////////////////////////////////////////////////////////////////////////////
//
// File   : Tools_Overlay_FacialAnimation_EditFace.h
// Author : Andy Tate
// Date   : 9-May-2008
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __TOOLS_OVERLAY_FACIALANIMATION_EDITFACE_H__
#define __TOOLS_OVERLAY_FACIALANIMATION_EDITFACE_H__

#if defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "FacialAnimation/Asura_FacialAnimation_MuscleGroup.h"
#include "Interface/Asura_Overlay.h"
#include "Interface/Asura_Table_Dialog.h"


////////////////////////////////////////////////////////////////////////////////
// Forward class declarations...
////////////////////////////////////////////////////////////////////////////////

class Asura_Function_Dialog;
class Tools_FacialAnimation_Face;
class Tools_FacialAnimation_Face_MuscleGroup;
class Tools_TableCell_MuscleGroupSelectAnimation_Dialog;


////////////////////////////////////////////////////////////////////////////////
// Class declaration...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Class        : Tools_Overlay_FacialAnimation_EditFace
// Author       : Andy Tate
// Date         : 9-May-2008
//
// Description  : Class used for the overlay to edit face properties.
////////////////////////////////////////////////////////////////////////////////

class Tools_Overlay_FacialAnimation_EditFace : public Asura_Overlay
{
public:
	// Public functions...
	static void	CreateOverlay(const bool bNewFace, const Asura_String& xPath, Tools_FacialAnimation_Face& xFace);
	static void	CloseOverlay(void);

	static bool	OverlayExists(void)		{ return(s_pxThis != NULL); }

	// Accessor functions...
	Tools_FacialAnimation_Face_MuscleGroup*	FindMuscleGroup(const u_int uIndex);
	void									SetAsChanged(void);


protected:
	// Protected variables...

	// Protected functions...
	Tools_Overlay_FacialAnimation_EditFace(const bool bNewFace, const Asura_String& xPath, Tools_FacialAnimation_Face& xFace);
	virtual ~Tools_Overlay_FacialAnimation_EditFace();

	void	UpdateMuscleGroups(void);
	void	UpdateFaceType(void);

	void	AddOverlayItems(Asura_Vector_2& xOffset);

	static Asura_TableCell_Dialog* CreateMuscleGroupTableCell(Asura_Table_Dialog& xTableDialog, const Asura_Vector_2& xOffset, const Asura_Vector_2& xMinCellSize, const u_int uRow, const u_int uColumn, void* const pxUserData);

	// Event functions...
	static void	OnSelectFaceType(int iUserData);
	static void	OnSaveAndCloseSelected(int iUserData);

	// Accessor functions...
	void	SetCurrentFaceType(const Asura_Hash_ID uFaceTypeHash);


private:
	// Private enumerated types...
	enum MuscleGroupColumn
	{
		MGC_NAME,
		MGC_CONTROL_TYPE,
		MGC_ANIMATIONS,

		MAX_MUSCLE_GROUP_COLUMNS
	};

	// Private typedefs...
	typedef Asura_Overlay PARENT;

	// Private variables...
	Asura_String				m_xPath;					// The name of the face-file we're using.
	Tools_FacialAnimation_Face&	m_xFace;					// A reference to the face we're editing.
	Asura_Dialog*				m_pxSelectFaceTypeDialogue;	// A pointer to the dialogue used when selecting the face-type.

	bool						m_bChanged;					// Flag to say whether any data has changed on this overlay.
	int							m_iNumMuscleGroups;			// The number of muscle-groups in the list.
	char const**				m_pszMuscleGroupNames;		// Pointer to an array of names for the muscle-groups.
	Asura_Table_Dialog*			m_pxMuscleGroupDialogue;	// The muscle-group dialogue to be used.

	// Static variables...
	static const char*								s_aszMuscleGroupColumnNames[MAX_MUSCLE_GROUP_COLUMNS];	// Array of names for the columns in the muscle-group table.
	static Tools_Overlay_FacialAnimation_EditFace*	s_pxThis;												// We only ever create a single instance of this overlay.

	// Private functions...
	Tools_Overlay_FacialAnimation_EditFace(void);
	Tools_Overlay_FacialAnimation_EditFace(const Tools_Overlay_FacialAnimation_EditFace& xOther);

	Tools_Overlay_FacialAnimation_EditFace&	operator=(const Tools_Overlay_FacialAnimation_EditFace& xOther);

};   // End of class 'Tools_Overlay_FacialAnimation_EditFace'.



////////////////////////////////////////////////////////////////////////////////
// Class        : Tools_TableCell_MuscleGroupSelectAnimation_Dialog
// Author       : Andy Tate
// Date         : 13-May-2008
//
// Description  : Class used to create the dialogue for a particular cell in 
//				  the muscle-group animation table for facial-animations.
////////////////////////////////////////////////////////////////////////////////

class Tools_TableCell_MuscleGroupSelectAnimation_Dialog : public Asura_TableCell_Dialog
{
public:
	// Public functions...
	Tools_TableCell_MuscleGroupSelectAnimation_Dialog(Asura_Table_Dialog& xTableDialogue, const Asura_Vector_2& xOffset, const Asura_Vector_2& xMinCellSize, const u_int uRow, const u_int uColumn, Tools_Overlay_FacialAnimation_EditFace& xEditFaceOverlay);
//	virtual ~Tools_TableCell_MuscleGroupSelectAnimation_Dialog();

	// Overridden functions...
	virtual int	Compare(const Asura_TableCell_Dialog& xOther) const	{ return(0); }


protected:
	// Protected enumerated types...
	enum
	{
		MAX_ANIMATIONS	= Asura_FacialAnimation_MuscleGroup::MAX_ANIMATIONS
	};

	// Protected variables...
	Tools_Overlay_FacialAnimation_EditFace&	m_xEditFaceOverlay;								// A reference to the overlay we're a part of.
	Tools_FacialAnimation_Face_MuscleGroup*	m_pxMuscleGroup;								// A pointer to the muscle-group to use.
	Asura_Function_Dialog*					m_apxSelectAnimationDialogues[MAX_ANIMATIONS];	// A pointer to the dialogue used for animation selection.

	// Protected functions...
	void	OnSelectAnimation(const u_int uIndex, const char* szMessage);

	static void OnSelectAnimation(int iUserData)
	{
		Tools_TableCell_MuscleGroupSelectAnimation_Dialog*	pxThis	= (Tools_TableCell_MuscleGroupSelectAnimation_Dialog*)(iUserData);

		pxThis->OnSelectAnimation(0, "Select Animation");
	}

	static void OnSelectHighAnimation(int iUserData)
	{
		Tools_TableCell_MuscleGroupSelectAnimation_Dialog*	pxThis	= (Tools_TableCell_MuscleGroupSelectAnimation_Dialog*)(iUserData);

		pxThis->OnSelectAnimation(0, "Select High Animation");
	}

	static void OnSelectLowAnimation(int iUserData)
	{
		Tools_TableCell_MuscleGroupSelectAnimation_Dialog*	pxThis	= (Tools_TableCell_MuscleGroupSelectAnimation_Dialog*)(iUserData);

		pxThis->OnSelectAnimation(1, "Select Low Animation");
	}


private:
	// Private typedefs...
	typedef Asura_TableCell_Dialog PARENT;

	// Private variables...

	// Private functions...
	Tools_TableCell_MuscleGroupSelectAnimation_Dialog(void);
	Tools_TableCell_MuscleGroupSelectAnimation_Dialog(const Tools_TableCell_MuscleGroupSelectAnimation_Dialog& xOther);

	Tools_TableCell_MuscleGroupSelectAnimation_Dialog&	operator=(const Tools_TableCell_MuscleGroupSelectAnimation_Dialog& xOther);

};   // End of class 'Tools_TableCell_MuscleGroupSelectAnimation_Dialog'.

#endif	//defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)

#endif   // ifndef __TOOLS_OVERLAY_FACIALANIMATION_EDITFACE_H__


/// End of file. ///
