
///////////////////////////////////////////////////////////////////////////////
//
// File   : Tools_Overlay_FacialAnimation_EditFaceType.cpp
// Author : Andy Tate
// Date   : 13-Jun-2008
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

#if defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)

// This file's header...
#include "FacialAnimation/Overlays/Tools_Overlay_FacialAnimation_EditFaceType.h"

// Asura independent header-files...
#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Overlay_System.h"
#include "Interface/Asura_Table_Dialog.h"
#include "Interface/Asura_Tree_Dialog.h"

// Tools header-files...
#include "Tools_Overlays.h"
#include "FacialAnimation/Tools_FacialAnimation_FaceType.h"
#include "FacialAnimation/Tools_FacialAnimation_System.h"
#include "FacialAnimation/Overlays/Tools_ModalOverlay_FacialAnimation_MuscleGroupSelectControlType.h"
#include "Misc/Tools_Modal_Overlay.h"
#include "Misc/Tools_Timed_Messages.h"


////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////

const char* Tools_Overlay_FacialAnimation_EditFaceType::s_aszMuscleGroupColumnNames[MAX_MUSCLE_GROUP_COLUMNS] =
{
	"Muscle-group",	// MGC_NAME
	"Control-Type",	// MGC_CONTROL_TYPE
};

const char* Tools_Overlay_FacialAnimation_EditFaceType::s_aszMappingColumnNames[MAX_MAPPING_COLUMNS] =
{
	"Phoneme",		// MC_PHONEME
	"Face-Pose",	// MC_FACE_POSE
};

Tools_Overlay_FacialAnimation_EditFaceType*	Tools_Overlay_FacialAnimation_EditFaceType::s_pxThis	= NULL;


////////////////////////////////////////////////////////////////////////////////
// Private functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Protected functions...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Overlay_FacialAnimation_EditFaceType::AddOverlayItems
// Author       : Andy Tate
// Date         : 13-Jun-2008
////////////////////////////////////////////////////////////////////////////////

void Tools_Overlay_FacialAnimation_EditFaceType::AddOverlayItems(Asura_Vector_2& xOffset)
{
	const Asura_Vector_2	xInitialOffset(xOffset);

	Asura_Tabs_Dialog*		pxTabsDialogue;
	Asura_Dialog*			pxNewDialogue;

	// Create our tabs...
	pxTabsDialogue	= new Asura_Tabs_Dialog (xOffset);
	AddDialog(pxTabsDialogue);
	xOffset[1]	+= Asura_Dialog::LargeSpacing();

	// Create the phonemes tab...
	pxTabsDialogue->AddTab("Phonemes");
	{
		Asura_Vector_2	xLocalOffset(xOffset);

		// Add the dialogue used for phoneme selection...
		m_pxPhonemeDialogue	= new Asura_List_Dialog (xLocalOffset, m_pszPhonemeNames, &m_iCurrentPhoneme, 15, m_iNumPhonemes, NULL);
		pxTabsDialogue->AddDialog(m_pxPhonemeDialogue);

		// Update the data displayed by this list...
		UpdatePhonemes();

		// Create the phoneme manipulation buttons...
		xLocalOffset[1]	= m_pxPhonemeDialogue->m_xMaxCoord[1] + Asura_Dialog::DefaultSpacing();
		pxNewDialogue	= new Asura_Function_Dialog("Add phoneme", xLocalOffset, OnAddPhonemeSelected, &m_bChanged);
		pxTabsDialogue->AddDialog(pxNewDialogue);
		pxNewDialogue->SetRelativePosition(m_pxPhonemeDialogue, true, false);

		xLocalOffset[0]	+= 15.0f;
		pxNewDialogue	 = new Asura_Function_Dialog("Move Up", xLocalOffset, OnMoveSelectedPhonemeUp, &m_bChanged, CanMoveSelectedPhonemeUp);
		pxTabsDialogue->AddDialog(pxNewDialogue);
		pxNewDialogue->SetRelativePosition(m_pxPhonemeDialogue, true, false);
		xLocalOffset[0]	 = xInitialOffset[0];

		xLocalOffset[1]	+= Asura_Dialog::DefaultSpacing();
		pxNewDialogue	 = new Asura_Function_Dialog("Delete phoneme", xLocalOffset, OnDeletePhonemeSelected, &m_bChanged, HasPhonemeCurrentlySelected);
		pxTabsDialogue->AddDialog(pxNewDialogue);
		pxNewDialogue->SetRelativePosition(m_pxPhonemeDialogue, true, false);

		xLocalOffset[0]	+= 15.0f;
		pxNewDialogue	 = new Asura_Function_Dialog("Move Down", xLocalOffset, OnMoveSelectedPhonemeDown, &m_bChanged, CanMoveSelectedPhonemeDown);
		pxTabsDialogue->AddDialog(pxNewDialogue);
		pxNewDialogue->SetRelativePosition(m_pxPhonemeDialogue, true, false);
		xLocalOffset[0]	 = xInitialOffset[0];
	}

	// Create the muscle-groups tab...
	pxTabsDialogue->AddTab("Muscle-Groups");
	{
		Asura_Vector_2	xLocalOffset(xOffset);

		// Add the dialogue used for muscle-group selection...
		m_pxMuscleGroupDialogue	 = new Asura_Table_Dialog (xOffset, 50.0f, 20.0f, true);
		m_pxMuscleGroupDialogue->EnableRowSelection(false);
		pxTabsDialogue->AddDialog(m_pxMuscleGroupDialogue);

		// Update the data displayed by this list...
		UpdateMuscleGroups();

		// Create the muscle-group manipulation buttons...
		xLocalOffset[1]	= m_pxMuscleGroupDialogue->m_xMaxCoord[1] + Asura_Dialog::DefaultSpacing();
		pxNewDialogue	= new Asura_Function_Dialog("Add muscle-group", xLocalOffset, OnAddMuscleGroupSelected, &m_bChanged);
		pxTabsDialogue->AddDialog(pxNewDialogue);
		pxNewDialogue->SetRelativePosition(m_pxMuscleGroupDialogue, true, false);

		xLocalOffset[0]	+= 15.0f;
		pxNewDialogue	 = new Asura_Function_Dialog("Move Up", xLocalOffset, OnMoveSelectedMuscleGroupUp, &m_bChanged, CanMoveSelectedMuscleGroupUp);
		pxTabsDialogue->AddDialog(pxNewDialogue);
		pxNewDialogue->SetRelativePosition(m_pxMuscleGroupDialogue, true, false);
		xLocalOffset[0]	 = xInitialOffset[0];

		xLocalOffset[1]	+= Asura_Dialog::DefaultSpacing();
		pxNewDialogue	 = new Asura_Function_Dialog("Delete muscle-group", xLocalOffset, OnDeleteMuscleGroupSelected, &m_bChanged, HasMuscleGroupCurrentlySelected);
		pxTabsDialogue->AddDialog(pxNewDialogue);
		pxNewDialogue->SetRelativePosition(m_pxMuscleGroupDialogue, true, false);

		xLocalOffset[0]	+= 15.0f;
		pxNewDialogue	 = new Asura_Function_Dialog("Move Down", xLocalOffset, OnMoveSelectedMuscleGroupDown, &m_bChanged, CanMoveSelectedMuscleGroupDown);
		pxTabsDialogue->AddDialog(pxNewDialogue);
		pxNewDialogue->SetRelativePosition(m_pxMuscleGroupDialogue, true, false);
		xLocalOffset[0]	 = xInitialOffset[0];
	}

	// Create the phoneme to pose mappings tab...
	pxTabsDialogue->AddTab("Mappings");
	{
		Asura_Vector_2	xLocalOffset(xOffset);

		// Add the dialogue used for mapping selection...
		m_pxMappingDialogue	 = new Asura_Table_Dialog (xOffset, 50.0f, 20.0f, true);
		m_pxMappingDialogue->EnableRowSelection(false);
		pxTabsDialogue->AddDialog(m_pxMappingDialogue);

		// Update the data displayed by this list...
		UpdateMappings();
	}

	// Create the "Save & Close" button...
	xOffset[1]		= pxTabsDialogue->GetMaxCoords()[1] + Asura_Dialog::LargeSpacing();
	pxNewDialogue	= new Asura_Function_Dialog("Save & Close", xOffset, OnSaveAndCloseSelected);
	AddDialog(pxNewDialogue);
	pxNewDialogue->SetRelativePosition(pxTabsDialogue, true, false);

	// Start with the first tab being open...
	pxTabsDialogue->SelectTab(0);
}

/// End of function 'Tools_Overlay_FacialAnimation_EditFaceType::AddOverlayItems'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Overlay_FacialAnimation_EditFaceType::UpdateMuscleGroups
// Author       : Andy Tate
// Date         : 9-May-2008
////////////////////////////////////////////////////////////////////////////////

void Tools_Overlay_FacialAnimation_EditFaceType::UpdateMuscleGroups(void)
{
	const Tools_FacialAnimation_FaceType::MuscleGroupContainer&	xMuscleGroupContainer	= m_xType.GetMuscleGroupContainer();
	Tools_FacialAnimation_FaceType::MuscleGroupConstIterator	xMuscleGroupIterator(xMuscleGroupContainer);
	const Asura_Vector_2										xMinCellSize(1.0f, 1.0f);

	// Remove any existing muscle-groups...
	delete[](m_pszMuscleGroupNames);
	m_pxMuscleGroupDialogue->DestroyTable();

	// Create the new array of strings...
	m_iNumMuscleGroups	= m_xType.GetNumMuscleGroups();

	if(m_iNumMuscleGroups > 0)
	{
		m_pszMuscleGroupNames	= new const char* [m_iNumMuscleGroups];

		for(int i = 0; i < m_iNumMuscleGroups; i++, xMuscleGroupIterator.Next())
		{
			const Tools_FacialAnimation_FaceType_MuscleGroup*	pxMuscleGroup		= xMuscleGroupIterator.GetCurrent();
			const Asura_String&									xMuscleGroupName	= pxMuscleGroup->GetName();
			const char*											szMuscleGroupName	= xMuscleGroupName.GetString();

			m_pszMuscleGroupNames[i]	= szMuscleGroupName;
		}

		// Create the table itself...
		m_pxMuscleGroupDialogue->SetHidden(false);
		m_pxMuscleGroupDialogue->CreateTable
		(
			xMinCellSize,
			m_iNumMuscleGroups,
			MAX_MUSCLE_GROUP_COLUMNS,
			CreateMuscleGroupTableCell,
			&m_xType,
			NULL,
			s_aszMuscleGroupColumnNames
		);
	}
	else
	{
		m_pszMuscleGroupNames	= NULL;
		m_pxMuscleGroupDialogue->SetHidden(true);
	}
}

/// End of function 'Tools_Overlay_FacialAnimation_EditFaceType::UpdateMuscleGroups'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Overlay_FacialAnimation_EditFaceType::UpdatePhonemes
// Author       : Andy Tate
// Date         : 9-May-2008
////////////////////////////////////////////////////////////////////////////////

void Tools_Overlay_FacialAnimation_EditFaceType::UpdatePhonemes(void)
{
	const Tools_FacialAnimation_FaceType::PhonemeNameContainer&	xPhonemeNameContainer	= m_xType.GetPhonemeNameContainer();
	Tools_FacialAnimation_FaceType::PhonemeNameConstIterator	xPhonemeNameIterator(xPhonemeNameContainer);
	const Asura_Vector_2										xMinCellSize(1.0f, 1.0f);

	// Remove any existing muscle-groups...
	delete[](m_pszPhonemeNames);

	// Create the new array of strings...
	m_iNumPhonemes	= m_xType.GetNumPhonemeNames();

	if(m_iNumPhonemes > 0)
	{
		m_pszPhonemeNames	= new const char* [m_iNumPhonemes];

		for(int i = 0; i < m_iNumPhonemes; i++, xPhonemeNameIterator.Next())
		{
			const Asura_String*	pxPhonemeName	= xPhonemeNameIterator.GetCurrent();

			m_pszPhonemeNames[i]	= pxPhonemeName->GetString();
		}
	}
	else
	{
		m_pszPhonemeNames	= NULL;
	}

	m_pxPhonemeDialogue->ReplaceStrings(m_iNumPhonemes, m_pszPhonemeNames);
}

/// End of function 'Tools_Overlay_FacialAnimation_EditFaceType::UpdatePhonemes'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Overlay_FacialAnimation_EditFaceType::UpdateMappings
// Author       : Andy Tate
// Date         : 9-May-2008
////////////////////////////////////////////////////////////////////////////////

void Tools_Overlay_FacialAnimation_EditFaceType::UpdateMappings(void)
{
	const Asura_Vector_2	xMinCellSize(1.0f, 1.0f);

	m_pxMappingDialogue->DestroyTable();

	if(m_iNumPhonemes > 0)
	{
		// Create the table itself...
		m_pxMappingDialogue->SetHidden(false);
		m_pxMappingDialogue->CreateTable
		(
			xMinCellSize,
			m_iNumPhonemes,
			MAX_MAPPING_COLUMNS,
			CreateMappingTableCell,
			&m_xType,
			NULL,
			s_aszMappingColumnNames
		);
	}
	else
	{
		m_pxMappingDialogue->SetHidden(true);
	}
}

/// End of function 'Tools_Overlay_FacialAnimation_EditFaceType::UpdateMappings'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Overlay_FacialAnimation_EditFaceType::Tools_Overlay_FacialAnimation_EditFaceType
// Author       : Andy Tate
// Date         : 9-May-2008
////////////////////////////////////////////////////////////////////////////////

Tools_Overlay_FacialAnimation_EditFaceType::Tools_Overlay_FacialAnimation_EditFaceType(const bool bNewType,
																					   const Asura_String& xPath,
																					   Tools_FacialAnimation_FaceType& xType)
: PARENT(xType.GetName().GetString(), true)
, m_xPath(xPath)
, m_xType(xType)
, m_bChanged(bNewType)
, m_iNumMuscleGroups(0)
, m_pszMuscleGroupNames(NULL)
, m_pxMuscleGroupDialogue(NULL)
, m_iCurrentPhoneme(0)
, m_iNumPhonemes(0)
, m_pszPhonemeNames(NULL)
, m_pxPhonemeDialogue(NULL)
{
	Asura_Vector_2	xOffset(1.0f, 3.0f);

	// Initialise the variables...
	m_uID			= TOOLS_OVERLAY_FACIALANIMATION_EDITFACETYPE;
	s_pxThis	= this;

	// Finally, update the overlays...
	AddOverlayItems(xOffset);
}

/// End of function 'Tools_Overlay_FacialAnimation_EditFaceType::Tools_Overlay_FacialAnimation_EditFaceType'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Overlay_FacialAnimation_EditFaceType::~Tools_Overlay_FacialAnimation_EditFaceType
// Author       : Andy Tate
// Date         : 13-Jun-2008
////////////////////////////////////////////////////////////////////////////////

Tools_Overlay_FacialAnimation_EditFaceType::~Tools_Overlay_FacialAnimation_EditFaceType()
{
	delete[](m_pszMuscleGroupNames);
	delete[](m_pszPhonemeNames);
	s_pxThis	= NULL;
}

/// End of function 'Tools_Overlay_FacialAnimation_EditFaceType::~Tools_Overlay_FacialAnimation_EditFaceType'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Overlay_FacialAnimation_EditFaceType::CreateMuscleGroupTableCell
// Author       : Andy Tate
// Date         : 13-May-2008
////////////////////////////////////////////////////////////////////////////////

Asura_TableCell_Dialog* Tools_Overlay_FacialAnimation_EditFaceType::CreateMuscleGroupTableCell(Asura_Table_Dialog& xTableDialog,
																							   const Asura_Vector_2& xOffset,
																							   const Asura_Vector_2& xMinCellSize,
																							   const u_int uRow,
																							   const u_int uColumn,
																							   void* const pxUserData)
{
	const Tools_FacialAnimation_FaceType_MuscleGroup* pxMuscleGroup = s_pxThis->FindMuscleGroup(uRow);

	switch(uColumn)
	{
		case MGC_NAME:
		{
			const Asura_String&	xName	= pxMuscleGroup->GetName();
			const char*			szName	= xName.GetString();

			return(Asura_TableRowHeading_StandardDialog::Create(xTableDialog, szName, xOffset, xMinCellSize, uRow));
		}

		case MGC_CONTROL_TYPE:
		{
			const Asura_FacialAnimation_MuscleGroup::ControlType	eControlType	= pxMuscleGroup->GetControlType();
			const char*												szName			= Asura_FacialAnimation_MuscleGroup::Tools_GetControlTypeName(eControlType);

			return(Asura_TableCell_TextDialog::Create(xTableDialog, szName, xOffset, xMinCellSize, uRow, uColumn));
		}

		default:
		{
			Asura_Assert(false, "Unexpected column found when creating muscle-group table.");
			return(NULL);
		}
	}		// End of switch.
}

/// End of function 'Tools_Overlay_FacialAnimation_EditFaceType::CreateMuscleGroupTableCell'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Overlay_FacialAnimation_EditFaceType::CreateMappingTableCell
// Author       : Andy Tate
// Date         : 13-May-2008
////////////////////////////////////////////////////////////////////////////////

Asura_TableCell_Dialog* Tools_Overlay_FacialAnimation_EditFaceType::CreateMappingTableCell(Asura_Table_Dialog& xTableDialog,
																						   const Asura_Vector_2& xOffset,
																						   const Asura_Vector_2& xMinCellSize,
																						   const u_int uRow,
																						   const u_int uColumn,
																						   void* const pxUserData)
{
	switch(uColumn)
	{
		case MC_PHONEME:
		{
			const char*	szPhonemeName	= s_pxThis->m_pszPhonemeNames[uRow];

			return(Asura_TableRowHeading_StandardDialog::Create(xTableDialog, szPhonemeName, xOffset, xMinCellSize, uRow));
		}

		case MC_FACE_POSE:
		{
			return(new Tools_TableCell_FaceTypeSelectFacePose_Dialog(xTableDialog, xOffset, xMinCellSize, uRow, uColumn, *s_pxThis));
		}

		default:
		{
			Asura_Assert(false, "Unexpected column found when creating mapping table.");
			return(NULL);
		}
	}		// End of switch.
}

/// End of function 'Tools_Overlay_FacialAnimation_EditFaceType::CreateMappingTableCell'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Overlay_FacialAnimation_EditFaceType::OnAddMuscleGroupSelected
// Author       : Andy Tate
// Date         : 9-May-2008
////////////////////////////////////////////////////////////////////////////////

void Tools_Overlay_FacialAnimation_EditFaceType::OnAddMuscleGroupSelected(void)
{
	GetString_Overlay	xGetNameOverlay("Enter name for new muscle-group", "Muscle-group");

	if(xGetNameOverlay.DoModal())
	{
		const char*									szName			= xGetNameOverlay.GetString();
		Tools_FacialAnimation_FaceType&				xType			= s_pxThis->m_xType;
		Tools_FacialAnimation_FaceType_MuscleGroup*	pxMuscleGroup	= xType.FindMuscleGroup(szName);

		if(pxMuscleGroup == NULL)
		{
			Tools_ModalOverlay_FacialAnimation_MuscleGroupSelectControlType	xSelectControlType;

			if(xSelectControlType.DoModal())
			{
				const Asura_FacialAnimation_MuscleGroup::ControlType	eControlType	= xSelectControlType.GetControlType();

				pxMuscleGroup	= xType.CreateMuscleGroup(szName, eControlType);

				if(pxMuscleGroup == NULL)
				{
					ShowMessageOverlay("Failed to create a new muscle-group named \"%s\".", szName);
				}
				else
				{
					s_pxThis->UpdateMuscleGroups();
				}
			}
		}
		else
		{
			ShowMessageOverlay("Already have a muscle-group named \"%s\", try a different name.", szName);
		}
	}
}

/// End of function 'Tools_Overlay_FacialAnimation_EditFaceType::OnAddMuscleGroupSelected'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Overlay_FacialAnimation_EditFaceType::OnDeleteMuscleGroupSelected
// Author       : Andy Tate
// Date         : 9-May-2008
////////////////////////////////////////////////////////////////////////////////

void Tools_Overlay_FacialAnimation_EditFaceType::OnDeleteMuscleGroupSelected(void)
{
	const int	iIndex	= s_pxThis->m_pxMuscleGroupDialogue->GetCurrentlySelectedRow();

	if(iIndex >= 0)
	{
		const char*			szName	= s_pxThis->m_pszMuscleGroupNames[iIndex];
		const Asura_String	xQuestion(Asura_String("Do you wish to delete muscle-group \"") + szName + Asura_String("\"?"));
		YesNo_Overlay		xDoYouWishToDeleteMuscleGroupOverlay(xQuestion);

		if(xDoYouWishToDeleteMuscleGroupOverlay.DoModal())
		{
			Tools_FacialAnimation_FaceType&	xType	= s_pxThis->m_xType;

			xType.DestroyMuscleGroup(szName);
			s_pxThis->UpdateMuscleGroups();
		}
	}
}

/// End of function 'Tools_Overlay_FacialAnimation_EditFaceType::OnDeleteMuscleGroupSelected'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Overlay_FacialAnimation_EditFaceType::OnMoveSelectedMuscleGroupUp
// Author       : Andy Tate
// Date         : 27-May-2008
////////////////////////////////////////////////////////////////////////////////

void Tools_Overlay_FacialAnimation_EditFaceType::OnMoveSelectedMuscleGroupUp(void)
{
	Tools_FacialAnimation_FaceType&				xType			= s_pxThis->m_xType;

	const int									iIndex1			= s_pxThis->m_pxMuscleGroupDialogue->GetCurrentlySelectedRow();
	const char*									szName1			= s_pxThis->m_pszMuscleGroupNames[iIndex1];
	Tools_FacialAnimation_FaceType_MuscleGroup*	pxMuscleGroup1	= xType.FindMuscleGroup(szName1);

	const int									iIndex2			= iIndex1 - 1;
	const char*									szName2			= s_pxThis->m_pszMuscleGroupNames[iIndex2];
	Tools_FacialAnimation_FaceType_MuscleGroup*	pxMuscleGroup2	= xType.FindMuscleGroup(szName2);

	// Swap this row with the previous one...
	xType.SwapMuscleGroups(pxMuscleGroup1, pxMuscleGroup2);
	s_pxThis->UpdateMuscleGroups();
	s_pxThis->m_pxMuscleGroupDialogue->SetCurrentlySelectedRow(iIndex2);
}

/// End of function 'Tools_Overlay_FacialAnimation_EditFaceType::OnMoveSelectedMuscleGroupUp'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Overlay_FacialAnimation_EditFaceType::OnMoveSelectedMuscleGroupDown
// Author       : Andy Tate
// Date         : 27-May-2008
////////////////////////////////////////////////////////////////////////////////

void Tools_Overlay_FacialAnimation_EditFaceType::OnMoveSelectedMuscleGroupDown(void)
{
	Tools_FacialAnimation_FaceType&	xType	= s_pxThis->m_xType;

	const int									iIndex1			= s_pxThis->m_pxMuscleGroupDialogue->GetCurrentlySelectedRow();
	const char*									szName1			= s_pxThis->m_pszMuscleGroupNames[iIndex1];
	Tools_FacialAnimation_FaceType_MuscleGroup*	pxMuscleGroup1	= xType.FindMuscleGroup(szName1);

	const int									iIndex2			= iIndex1 + 1;
	const char*									szName2			= s_pxThis->m_pszMuscleGroupNames[iIndex2];
	Tools_FacialAnimation_FaceType_MuscleGroup*	pxMuscleGroup2	= xType.FindMuscleGroup(szName2);

	// Swap this row with the next one...
	xType.SwapMuscleGroups(pxMuscleGroup1, pxMuscleGroup2);
	s_pxThis->UpdateMuscleGroups();
	s_pxThis->m_pxMuscleGroupDialogue->SetCurrentlySelectedRow(iIndex2);
}

/// End of function 'Tools_Overlay_FacialAnimation_EditFaceType::OnMoveSelectedMuscleGroupDown'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Overlay_FacialAnimation_EditFaceType::OnAddPhonemeSelected
// Author       : Andy Tate
// Date         : 9-May-2008
////////////////////////////////////////////////////////////////////////////////

void Tools_Overlay_FacialAnimation_EditFaceType::OnAddPhonemeSelected(void)
{
	GetString_Overlay	xGetNameOverlay("Enter name for new phoneme", "Phoneme");

	if(xGetNameOverlay.DoModal())
	{
		const char*							szName			= xGetNameOverlay.GetString();
		Tools_FacialAnimation_FaceType&		xType			= s_pxThis->m_xType;
		Asura_String*						pxPhonemeName	= xType.FindPhonemeName(szName);

		if(pxPhonemeName == NULL)
		{
			pxPhonemeName	= xType.CreatePhonemeName(szName);

			if(pxPhonemeName == NULL)
			{
				ShowMessageOverlay("Failed to create a new phoneme named \"%s\".", szName);
			}
			else
			{
				s_pxThis->UpdatePhonemes();
			}
		}
		else
		{
			ShowMessageOverlay("Already have a phoneme named \"%s\", try a different name.", szName);
		}
	}
}

/// End of function 'Tools_Overlay_FacialAnimation_EditFaceType::OnAddPhonemeSelected'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Overlay_FacialAnimation_EditFaceType::OnDeletePhonemeSelected
// Author       : Andy Tate
// Date         : 9-May-2008
////////////////////////////////////////////////////////////////////////////////

void Tools_Overlay_FacialAnimation_EditFaceType::OnDeletePhonemeSelected(void)
{
	const int	iIndex	= s_pxThis->m_iCurrentPhoneme;

	if(iIndex >= 0)
	{
		const char*			szName	= s_pxThis->m_pszPhonemeNames[iIndex];
		const Asura_String	xQuestion(Asura_String("Do you wish to delete phoneme \"") + szName + Asura_String("\"?"));
		YesNo_Overlay		xDoYouWishToDeletePhonemeOverlay(xQuestion);

		if(xDoYouWishToDeletePhonemeOverlay.DoModal())
		{
			Tools_FacialAnimation_FaceType&	xType	= s_pxThis->m_xType;

			xType.DestroyPhonemeName(szName);
			s_pxThis->UpdatePhonemes();
		}
	}
}

/// End of function 'Tools_Overlay_FacialAnimation_EditFaceType::OnDeletePhonemeSelected'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Overlay_FacialAnimation_EditFaceType::OnMoveSelectedPhonemeUp
// Author       : Andy Tate
// Date         : 27-May-2008
////////////////////////////////////////////////////////////////////////////////

void Tools_Overlay_FacialAnimation_EditFaceType::OnMoveSelectedPhonemeUp(void)
{
	Tools_FacialAnimation_FaceType&	xType			= s_pxThis->m_xType;

	const int						iIndex1			= s_pxThis->m_iCurrentPhoneme;
	const char*						szName1			= s_pxThis->m_pszPhonemeNames[iIndex1];
	Asura_String*					pxPhonemeName1	= xType.FindPhonemeName(szName1);

	const int						iIndex2			= iIndex1 - 1;
	const char*						szName2			= s_pxThis->m_pszPhonemeNames[iIndex2];
	Asura_String*					pxPhonemeName2	= xType.FindPhonemeName(szName2);

	// Swap this row with the previous one...
	xType.SwapPhonemeNames(pxPhonemeName1, pxPhonemeName2);
	s_pxThis->UpdatePhonemes();

	s_pxThis->m_iCurrentPhoneme	= iIndex2;
}

/// End of function 'Tools_Overlay_FacialAnimation_EditFaceType::OnMoveSelectedPhonemeUp'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Overlay_FacialAnimation_EditFaceType::OnMoveSelectedPhonemeDown
// Author       : Andy Tate
// Date         : 27-May-2008
////////////////////////////////////////////////////////////////////////////////

void Tools_Overlay_FacialAnimation_EditFaceType::OnMoveSelectedPhonemeDown(void)
{
	Tools_FacialAnimation_FaceType&	xType			= s_pxThis->m_xType;

	const int						iIndex1			= s_pxThis->m_iCurrentPhoneme;
	const char*						szName1			= s_pxThis->m_pszPhonemeNames[iIndex1];
	Asura_String*					pxPhonemeName1	= xType.FindPhonemeName(szName1);

	const int						iIndex2			= iIndex1 + 1;
	const char*						szName2			= s_pxThis->m_pszPhonemeNames[iIndex2];
	Asura_String*					pxPhonemeName2	= xType.FindPhonemeName(szName2);

	// Swap this row with the next one...
	xType.SwapPhonemeNames(pxPhonemeName1, pxPhonemeName2);
	s_pxThis->UpdatePhonemes();

	s_pxThis->m_iCurrentPhoneme	= iIndex2;
}

/// End of function 'Tools_Overlay_FacialAnimation_EditFaceType::OnMoveSelectedPhonemeDown'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Overlay_FacialAnimation_EditFaceType::OnSaveAndCloseSelected
// Author       : Andy Tate
// Date         : 13-Jun-2008
////////////////////////////////////////////////////////////////////////////////

void Tools_Overlay_FacialAnimation_EditFaceType::OnSaveAndCloseSelected(void)
{
	if(s_pxThis->m_bChanged)
	{
		const Tools_FacialAnimation_FaceType&	xType		= s_pxThis->m_xType;
		const Asura_String						xFilename(Tools_FacialAnimation_System::CreateTypeFilename(s_pxThis->m_xPath, true));
		const char*								szFilename	= xFilename.GetString();

		// Attempt to save the file...
		if(Tools_FacialAnimation_System::WriteTypeFile(szFilename, xType))
		{
			Tools_Timed_Message_System::AddMessage(Tools_Timed_Message_System::COLOUR_SUCCESS, "\"%s\" saved successfully.", szFilename);
		}
		else
		{
			Tools_Timed_Message_System::AddMessage(Tools_Timed_Message_System::COLOUR_ERROR, "Failed to save \"%s\".", szFilename);
		}
	}

	// Finally, close the overlay...
	CloseOverlay();
}

/// End of function 'Tools_Overlay_FacialAnimation_EditFaceType::OnSaveAndCloseSelected'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Overlay_FacialAnimation_EditFaceType::HasMuscleGroupCurrentlySelected
// Author       : Andy Tate
// Date         : 13-May-2008
////////////////////////////////////////////////////////////////////////////////

bool Tools_Overlay_FacialAnimation_EditFaceType::HasMuscleGroupCurrentlySelected(void)
{
	const int	iNumRows	= s_pxThis->m_iNumMuscleGroups;
	const int	iIndex		= s_pxThis->m_pxMuscleGroupDialogue->GetCurrentlySelectedRow();

	return((iIndex >= 0) && (iIndex < iNumRows));
}

/// End of function 'Tools_Overlay_FacialAnimation_EditFaceType::HasMuscleGroupCurrentlySelected'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Overlay_FacialAnimation_EditFaceType::CanMoveSelectedMuscleGroupUp
// Author       : Andy Tate
// Date         : 27-May-2008
////////////////////////////////////////////////////////////////////////////////

bool Tools_Overlay_FacialAnimation_EditFaceType::CanMoveSelectedMuscleGroupUp(void)
{
	const int	iNumRows	= s_pxThis->m_iNumMuscleGroups;
	const int	iIndex		= s_pxThis->m_pxMuscleGroupDialogue->GetCurrentlySelectedRow();

	return((iIndex > 0) && (iIndex < iNumRows));
}

/// End of function 'Tools_Overlay_FacialAnimation_EditFaceType::CanMoveSelectedMuscleGroupUp'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Overlay_FacialAnimation_EditFaceType::CanMoveSelectedMuscleGroupDown
// Author       : Andy Tate
// Date         : 27-May-2008
////////////////////////////////////////////////////////////////////////////////

bool Tools_Overlay_FacialAnimation_EditFaceType::CanMoveSelectedMuscleGroupDown(void)
{
	const int	iNumRows	= s_pxThis->m_iNumMuscleGroups;
	const int	iIndex		= s_pxThis->m_pxMuscleGroupDialogue->GetCurrentlySelectedRow();

	return((iIndex >= 0) && (iIndex < (iNumRows - 1)));
}

/// End of function 'Tools_Overlay_FacialAnimation_EditFaceType::CanMoveSelectedMuscleGroupDown'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Overlay_FacialAnimation_EditFaceType::HasPhonemeCurrentlySelected
// Author       : Andy Tate
// Date         : 13-May-2008
////////////////////////////////////////////////////////////////////////////////

bool Tools_Overlay_FacialAnimation_EditFaceType::HasPhonemeCurrentlySelected(void)
{
	const int	iNumRows	= s_pxThis->m_iNumPhonemes;
	const int	iIndex		= s_pxThis->m_iCurrentPhoneme;

	return((iIndex >= 0) && (iIndex < iNumRows));
}

/// End of function 'Tools_Overlay_FacialAnimation_EditFaceType::HasPhonemeCurrentlySelected'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Overlay_FacialAnimation_EditFaceType::CanMoveSelectedPhonemeUp
// Author       : Andy Tate
// Date         : 27-May-2008
////////////////////////////////////////////////////////////////////////////////

bool Tools_Overlay_FacialAnimation_EditFaceType::CanMoveSelectedPhonemeUp(void)
{
	const int	iNumRows	= s_pxThis->m_iNumPhonemes;
	const int	iIndex		= s_pxThis->m_iCurrentPhoneme;

	return((iIndex > 0) && (iIndex < iNumRows));
}

/// End of function 'Tools_Overlay_FacialAnimation_EditFaceType::CanMoveSelectedPhonemeUp'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Overlay_FacialAnimation_EditFaceType::CanMoveSelectedPhonemeDown
// Author       : Andy Tate
// Date         : 27-May-2008
////////////////////////////////////////////////////////////////////////////////

bool Tools_Overlay_FacialAnimation_EditFaceType::CanMoveSelectedPhonemeDown(void)
{
	const int	iNumRows	= s_pxThis->m_iNumPhonemes;
	const int	iIndex		= s_pxThis->m_iCurrentPhoneme;

	return((iIndex >= 0) && (iIndex < (iNumRows - 1)));
}

/// End of function 'Tools_Overlay_FacialAnimation_EditFaceType::CanMoveSelectedPhonemeDown'. ///



////////////////////////////////////////////////////////////////////////////////
// Public functions...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Overlay_FacialAnimation_EditFaceType::CreateOverlay
// Author       : Andy Tate
// Date         : 9-May-2008
////////////////////////////////////////////////////////////////////////////////

void Tools_Overlay_FacialAnimation_EditFaceType::CreateOverlay(const bool bNewType,
															   const Asura_String& xPath,
															   Tools_FacialAnimation_FaceType& xType)
{
	if(Asura_Overlay_System::DoesOverlayExist(TOOLS_OVERLAY_FACIALANIMATION_EDITFACETYPE))
	{
		Asura_Overlay_System::BringOverlayToTop(TOOLS_OVERLAY_FACIALANIMATION_EDITFACETYPE);
	}
	else
	{
		Tools_Overlay_FacialAnimation_EditFaceType*	pxOverlay	= new Tools_Overlay_FacialAnimation_EditFaceType (bNewType, xPath, xType);

		Asura_Overlay_System::SubmitNewOverlay(pxOverlay, false, false);
	}
}

/// End of function 'Tools_Overlay_FacialAnimation_EditFaceType::CreateOverlay'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Overlay_FacialAnimation_EditFaceType::CloseOverlay
// Author       : Andy Tate
// Date         : 9-May-2008
////////////////////////////////////////////////////////////////////////////////

void Tools_Overlay_FacialAnimation_EditFaceType::CloseOverlay(void)
{
	Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_FACIALANIMATION_EDITFACETYPE);
}

/// End of function 'Tools_Overlay_FacialAnimation_EditFaceType::CloseOverlay'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_FacialAnimation_FaceType_MuscleGroup::FindMuscleGroup
// Author       : Andy Tate
// Date         : 13-May-2008
////////////////////////////////////////////////////////////////////////////////

Tools_FacialAnimation_FaceType_MuscleGroup* Tools_Overlay_FacialAnimation_EditFaceType::FindMuscleGroup(const u_int uIndex)
{
	const char*									szName			= m_pszMuscleGroupNames[uIndex];
	Tools_FacialAnimation_FaceType_MuscleGroup*	pxMuscleGroup	= m_xType.FindMuscleGroup(szName);

	return(pxMuscleGroup);
}

/// End of function 'Tools_FacialAnimation_FaceType_MuscleGroup::FindMuscleGroup'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Overlay_FacialAnimation_EditFaceType::SetAsChanged
// Author       : Andy Tate
// Date         : 8-Jul-2008
////////////////////////////////////////////////////////////////////////////////

void Tools_Overlay_FacialAnimation_EditFaceType::SetAsChanged(void)
{
	m_bChanged	= true;

	if(m_pxMuscleGroupDialogue != NULL)
	{
		m_pxMuscleGroupDialogue->UpdateAll();
	}

	if(m_pxMappingDialogue != NULL)
	{
		m_pxMappingDialogue->UpdateAll();
	}
}

/// End of function 'Tools_Overlay_FacialAnimation_EditFaceType::SetAsChanged'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_TableCell_FaceTypeSelectFacePose_Dialog::Tools_TableCell_FaceTypeSelectFacePose_Dialog
// Author       : Andy Tate
// Date         : 8-Jul-2008
////////////////////////////////////////////////////////////////////////////////

Tools_TableCell_FaceTypeSelectFacePose_Dialog::Tools_TableCell_FaceTypeSelectFacePose_Dialog(Asura_Table_Dialog& xTableDialogue,
																							 const Asura_Vector_2& xOffset,
																							 const Asura_Vector_2& xMinCellSize,
																							 const u_int uRow,
																							 const u_int uColumn,
																							 Tools_Overlay_FacialAnimation_EditFaceType& xEditFaceTypeOverlay)
: PARENT(xTableDialogue, xOffset, xMinCellSize, uRow, uColumn)
, m_xEditFaceTypeOverlay(xEditFaceTypeOverlay)
, m_xFaceType(xEditFaceTypeOverlay.GetFaceType())
, m_szPhonemeName(xEditFaceTypeOverlay.GetPhonemeName(uRow))
{
	const Asura_String*	pxFacePoseName	= m_xFaceType.FindFacePoseNameForPhonemeName(m_szPhonemeName);
	const char*			szFacePoseName	= ((pxFacePoseName == NULL) || pxFacePoseName->IsBlank())
										? "[None]"
										: pxFacePoseName->GetString();
	Asura_Vector_2		xLocalOffset(0.0f, 0.0f);

	m_pxFacePoseDialogue	= new Asura_Function_Dialog (szFacePoseName, xLocalOffset, OnSelectFacePose, reinterpret_cast<int>(this));
	AddDialogToPanel(m_pxFacePoseDialogue);
}

/// End of function 'Tools_TableCell_FaceTypeSelectFacePose_Dialog::Tools_TableCell_FaceTypeSelectFacePose_Dialog'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_TableCell_FaceTypeSelectFacePose_Dialog::OnSelectFacePose
// Author       : Andy Tate
// Date         : 8-Jul-2008
////////////////////////////////////////////////////////////////////////////////
void Tools_TableCell_FaceTypeSelectFacePose_Dialog::OnSelectFacePose(int iUserData)
{
	Tools_TableCell_FaceTypeSelectFacePose_Dialog*	pxThis	= reinterpret_cast<Tools_TableCell_FaceTypeSelectFacePose_Dialog*>(iUserData);

	const char*				szPoseDirectory		= Tools_FacialAnimation_System::GetPoseDirectoryName();
	const char*				szPoseFileExtension	= Tools_FacialAnimation_System::GetPoseFileExtension();

	FileSelect_Overlay		xSelectPose( "Select face-pose", szPoseDirectory, szPoseFileExtension );

	if(xSelectPose.DoModal())
	{
		Asura_Hash_ID uSelectedHashID;
		Asura_String xSelectedName;

		xSelectPose.GetSelectedFile( uSelectedHashID, xSelectedName );


		if( uSelectedHashID==ASURA_HASH_ID_UNSET )
		{
			pxThis->m_xFaceType.DestroyPhonemeToFacePoseMapping(pxThis->m_szPhonemeName);

			pxThis->m_pxFacePoseDialogue->UpdateLabel("[None]");
			pxThis->UpdateSize();

			pxThis->m_xEditFaceTypeOverlay.SetAsChanged();
		}
		else
		{
			Tools_FacialAnimation_FacePose*	pxPose	= Tools_FacialAnimation_System::FindOrLoadPose( xSelectedName );

			if(pxPose == NULL)
			{
				ShowMessageOverlay("Failed to find the specified face-pose \"%s\".", xSelectedName.GetString());
			}
			else
			{
				const Asura_String*	pxFacePoseName	= pxThis->m_xFaceType.CreatePhonemeToFacePoseMapping(pxThis->m_szPhonemeName, *pxPose);
				const char*			szFacePoseName	= pxFacePoseName->GetString();

				pxThis->m_pxFacePoseDialogue->UpdateLabel(szFacePoseName);
				pxThis->UpdateSize();

				pxThis->m_xEditFaceTypeOverlay.SetAsChanged();
			}
		}
	}
}

/// End of function 'Tools_TableCell_FaceTypeSelectFacePose_Dialog::OnSelectFacePose'. ///

#endif	//defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)


/// End of file. ///
