
///////////////////////////////////////////////////////////////////////////////
//
// File   : Tools_FacialAnimation_FacePose.cpp
// Author : Andy Tate
// Date   : 3-Jun-2008
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

#if defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)

// This file's header...
#include "FacialAnimation/Tools_FacialAnimation_FacePose.h"

// Asura independent header-files...
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "FacialAnimation/Asura_FacialAnimation_AnimationHandler.h"

// Tools header-files...
#include "FacialAnimation/Tools_FacialAnimation_System.h"


////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Version	Author	Comment
// ~~~~~~~	~~~~~~	~~~~~~~
// 1		ART		Created.
// 2		ART		Added binding to a particular face-type.
// 3		RAR		Added eye weights
////////////////////////////////////////////////////////////////////////////////
const u_int	Tools_FacialAnimation_FacePose::s_uChunkVersion	= 3;

////////////////////////////////////////////////////////////////////////////////
// Version	Author	Comment
// ~~~~~~~	~~~~~~	~~~~~~~
// 1		ART		Created.
////////////////////////////////////////////////////////////////////////////////
const u_int	Tools_FacialAnimation_MuscleGroupWeighting::s_uChunkVersion	= 1;


////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Private functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Protected functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Public functions...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_FacialAnimation_MuscleGroupWeighting::Tools_FacialAnimation_MuscleGroupWeighting
// Author       : Andy Tate
// Date         : 3-Jun-2008
////////////////////////////////////////////////////////////////////////////////

Tools_FacialAnimation_MuscleGroupWeighting::Tools_FacialAnimation_MuscleGroupWeighting(void)
: m_fWeight(0.0f)
, m_fBalance(0.0f)
{
}

/// End of function 'Tools_FacialAnimation_MuscleGroupWeighting::Tools_FacialAnimation_MuscleGroupWeighting'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_FacialAnimation_MuscleGroupWeighting::Tools_FacialAnimation_MuscleGroupWeighting
// Author       : Andy Tate
// Date         : 3-Jun-2008
////////////////////////////////////////////////////////////////////////////////

Tools_FacialAnimation_MuscleGroupWeighting::Tools_FacialAnimation_MuscleGroupWeighting(const Asura_Hash_ID uHash,
																					   Asura_Chunk_Stream& xStream)
{
	ReadFromChunkStream(xStream);
}

/// End of function 'Tools_FacialAnimation_MuscleGroupWeighting::Tools_FacialAnimation_MuscleGroupWeighting'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_FacialAnimation_MuscleGroupWeighting::ReadFromChunkStream
// Author       : Andy Tate
// Date         : 3-Jun-2008
////////////////////////////////////////////////////////////////////////////////

void Tools_FacialAnimation_MuscleGroupWeighting::ReadFromChunkStream(Asura_Chunk_Stream& xStream)
{
	u_int	uVersion;

	// First, read (and verify) the chunk-version.
	xStream >> uVersion;

	switch(uVersion)
	{
		case 1:
		{
			xStream >> m_xName;
			xStream >> m_fWeight;
			xStream >> m_fBalance;
			break;
		}

		default:
		{
			Asura_Assertf(false, ("Unexpected chunk-version %d", uVersion));
			break;
		}
	}		// End of switch.
}

/// End of function 'Tools_FacialAnimation_MuscleGroupWeighting::ReadFromChunkStream'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_FacialAnimation_MuscleGroupWeighting::WriteToChunkStream
// Author       : Andy Tate
// Date         : 3-Jun-2008
////////////////////////////////////////////////////////////////////////////////

void Tools_FacialAnimation_MuscleGroupWeighting::WriteToChunkStream(Asura_Chunk_Stream& xStream) const
{
	xStream << s_uChunkVersion;

	xStream << m_xName;
	xStream << m_fWeight;
	xStream << m_fBalance;
}

/// End of function 'Tools_FacialAnimation_MuscleGroupWeighting::WriteToChunkStream'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_FacialAnimation_FacePose::Tools_FacialAnimation_FacePose
// Author       : Andy Tate
// Date         : 3-Jun-2008
////////////////////////////////////////////////////////////////////////////////

Tools_FacialAnimation_FacePose::Tools_FacialAnimation_FacePose(void)
{
	SetDefaultEyeValues();
}

/// End of function 'Tools_FacialAnimation_FacePose::Tools_FacialAnimation_FacePose'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_FacialAnimation_FacePose::Tools_FacialAnimation_FacePose
// Author       : Andy Tate
// Date         : 3-Jun-2008
////////////////////////////////////////////////////////////////////////////////

Tools_FacialAnimation_FacePose::Tools_FacialAnimation_FacePose(const Asura_Hash_ID uHash,
															   Asura_Chunk_Stream& xStream)
{
	ReadFromChunkStream(xStream);
}

/// End of function 'Tools_FacialAnimation_FacePose::Tools_FacialAnimation_FacePose'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_FacialAnimation_FacePose::CopyFromAnimationHandler
// Author       : Andy Tate
// Date         : 3-Jun-2008
////////////////////////////////////////////////////////////////////////////////
#ifdef ASURA_MAIN
bool Tools_FacialAnimation_FacePose::CopyFromAnimationHandler(const Asura_FacialAnimation_AnimationHandler& xAnimationHandler)
{
	const Asura_Hash_ID					uFaceHash	= xAnimationHandler.GetFaceHash();
	const Tools_FacialAnimation_Face*	pxFace		= Tools_FacialAnimation_System::FindFace(uFaceHash);

	// Remove any existing data...
	m_xMuscleGroupWeightingContainer.Reset();

	if(pxFace == NULL)
	{
		Asura_Assert(false, "Failed to find the face used by this animation-handler.");
		return(false);
	}
	else
	{
		const Tools_FacialAnimation_Face::MuscleGroupContainer&	xMuscleGroupContainer	= pxFace->GetMuscleGroupContainer();

		Tools_FacialAnimation_Face::MuscleGroupConstIterator	xIterator(xMuscleGroupContainer);

		for(u_int uIndex = 0; !xIterator.Done(); xIterator.Next())
		{
			const Tools_FacialAnimation_Face_MuscleGroup*	pxMuscleGroup	= xIterator.GetCurrent();

			if(pxMuscleGroup != NULL)
			{
				const Asura_String&							xMuscleGroupName	= pxMuscleGroup->GetName();
				const Asura_Hash_ID							uMuscleGroupHash	= xMuscleGroupName.GetHash();
				const float									fWeight				= xAnimationHandler.Tools_GetWeight(uIndex);
				const float									fBalance			= xAnimationHandler.Tools_GetBalance(uIndex);
				Tools_FacialAnimation_MuscleGroupWeighting*	pxNewWeighting		= m_xMuscleGroupWeightingContainer.AddNode(uMuscleGroupHash);

				pxNewWeighting->SetName(xMuscleGroupName);
				pxNewWeighting->SetWeightAndBalance(fWeight, fBalance);
				uIndex++;
			}
		}

		if(pxFace->HasEyes())
		{
			for(u_int uIndex=0; uIndex<ASURA_FACIALANIMATION_EYE_NUMPARAM; uIndex++)
			{
				m_afEyeValues[uIndex] = xAnimationHandler.Tools_GetEyeValue(uIndex);
			}
		}

		return(true);
	}
}
#endif // ASURA_MAIN
/// End of function 'Tools_FacialAnimation_FacePose::CopyFromAnimationHandler'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_FacialAnimation_FacePose::CopyToAnimationHandler
// Author       : Andy Tate
// Date         : 3-Jun-2008
////////////////////////////////////////////////////////////////////////////////
#ifdef ASURA_MAIN
bool Tools_FacialAnimation_FacePose::CopyToAnimationHandler(Asura_FacialAnimation_AnimationHandler& xAnimationHandler) const
{
	const Asura_Hash_ID					uFaceHash	= xAnimationHandler.GetFaceHash();
	const Tools_FacialAnimation_Face*	pxFace		= Tools_FacialAnimation_System::FindFace(uFaceHash);

	if(pxFace == NULL)
	{
		Asura_Assert(false, "Failed to find the face used by this animation-handler.");
		return(false);
	}
	else
	{
		const Tools_FacialAnimation_Face::MuscleGroupContainer&	xMuscleGroupContainer	= pxFace->GetMuscleGroupContainer();

		Tools_FacialAnimation_Face::MuscleGroupConstIterator	xIterator(xMuscleGroupContainer);

		for(u_int uIndex = 0; !xIterator.Done(); xIterator.Next())
		{
			const Tools_FacialAnimation_Face_MuscleGroup*	pxMuscleGroup	= xIterator.GetCurrent();

			if(pxMuscleGroup != NULL)
			{
				const Asura_String&	xMuscleGroupName	= pxMuscleGroup->GetName();
				const Asura_Hash_ID	uMuscleGroupHash	= xMuscleGroupName.GetHash();

				const Tools_FacialAnimation_MuscleGroupWeighting*	pxWeighting	= m_xMuscleGroupWeightingContainer.GetDataByHash(uMuscleGroupHash);

				if(pxWeighting != NULL)
				{
					const float	fWeight		= pxWeighting->GetWeight();
					const float	fBalance	= pxWeighting->GetBalance();

					xAnimationHandler.Tools_SetWeightAndBalance(uIndex, fWeight, fBalance);
					uIndex++;
				}
			}
		}

		if(pxFace->HasEyes())
		{
			for(u_int uIndex=0; uIndex<ASURA_FACIALANIMATION_EYE_NUMPARAM; uIndex++)
			{
				*xAnimationHandler.Tools_GetEyeValuePointer(uIndex) = m_afEyeValues[uIndex];
			}
		}


		return(true);
	}
}
#endif //ASURA_MAIN
/// End of function 'Tools_FacialAnimation_FacePose::CopyToAnimationHandler'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_FacialAnimation_FacePose::ReadFromChunkStream
// Author       : Andy Tate
// Date         : 3-Jun-2008
////////////////////////////////////////////////////////////////////////////////

void Tools_FacialAnimation_FacePose::ReadFromChunkStream(Asura_Chunk_Stream& xStream)
{
	u_int	uVersion;

	// Reset any existing data...
	m_xMuscleGroupWeightingContainer.Reset();
	m_xFaceTypeName.MakeNull();

	// First, read (and verify) the chunk-version.
	xStream >> uVersion;

	SetDefaultEyeValues();
	switch(uVersion)
	{
		case 1:
		{
			u_int	uNumMuscleGroupWeightings;

			// Read the name followed by each weighting in-turn...
			xStream >> m_xName;
			xStream >> uNumMuscleGroupWeightings;

			for(u_int u = 0; u < uNumMuscleGroupWeightings; u++)
			{
				Asura_Hash_ID	uHash;

				xStream >> uHash;
				m_xMuscleGroupWeightingContainer.AddNode(uHash, xStream);
			}
			break;
		}

		case 2:
		{
			u_int	uNumMuscleGroupWeightings;

			// Remove any existing weightings...
			m_xMuscleGroupWeightingContainer.Reset();

			// Read the name followed by each weighting in-turn...
			xStream >> m_xName;
			xStream >> m_xFaceTypeName;
			xStream >> uNumMuscleGroupWeightings;

			for(u_int u = 0; u < uNumMuscleGroupWeightings; u++)
			{
				Asura_Hash_ID	uHash;

				xStream >> uHash;
				m_xMuscleGroupWeightingContainer.AddNode(uHash, xStream);
			}
			break;
		}

		case 3:
		{
			u_int	uNumMuscleGroupWeightings;

			// Remove any existing weightings...
			m_xMuscleGroupWeightingContainer.Reset();

			// Read the name followed by each weighting in-turn...
			xStream >> m_xName;
			xStream >> m_xFaceTypeName;
			xStream >> uNumMuscleGroupWeightings;

			for(u_int u = 0; u < uNumMuscleGroupWeightings; u++)
			{
				Asura_Hash_ID	uHash;

				xStream >> uHash;
				m_xMuscleGroupWeightingContainer.AddNode(uHash, xStream);
			}

			int iNumEyeValue;
			xStream >> iNumEyeValue;
			Asura_Assert(iNumEyeValue == ASURA_FACIALANIMATION_EYE_NUMPARAM, "Incorrect number of eye values saved in face pose");
			for(int i=0; i<iNumEyeValue; i++)
			{
				if(i<ASURA_FACIALANIMATION_EYE_NUMPARAM)
				{
					xStream >> m_afEyeValues[i];
				}
				else
				{
					xStream.Skip(sizeof(float));
				}
			}
			break;
		}


		default:
		{
			Asura_Assertf(false, ("Unexpected chunk-version %d", uVersion));
			break;
		}
	}		// End of switch.
}

/// End of function 'Tools_FacialAnimation_FacePose::ReadFromChunkStream'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_FacialAnimation_FacePose::WriteToChunkStream
// Author       : Andy Tate
// Date         : 3-Jun-2008
////////////////////////////////////////////////////////////////////////////////

void Tools_FacialAnimation_FacePose::WriteToChunkStream(Asura_Chunk_Stream& xStream) const
{
	const u_int	uNumMuscleGroupWeightings	= m_xMuscleGroupWeightingContainer.GetNumberOfNodes();

	// First write the chunk-version...
	xStream << s_uChunkVersion;

	xStream << m_xName;
	xStream << m_xFaceTypeName;
	xStream << uNumMuscleGroupWeightings;

	for(MuscleGroupWeightingConstIterator xIterator(m_xMuscleGroupWeightingContainer); !xIterator.Done(); xIterator.Next())
	{
		const Tools_FacialAnimation_MuscleGroupWeighting*	pxMuscleGroupWeighting	= xIterator.GetCurrent();
		const Asura_Hash_ID									uHash					= xIterator.GetCurrentHash();

		xStream << uHash;
		pxMuscleGroupWeighting->WriteToChunkStream(xStream);
	}

	xStream << (int) ASURA_FACIALANIMATION_EYE_NUMPARAM;
	for(int i=0; i<ASURA_FACIALANIMATION_EYE_NUMPARAM; i++)
	{
		xStream << m_afEyeValues[i];
	}

}

/// End of function 'Tools_FacialAnimation_FacePose::WriteToChunkStream'. ///


////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_FacialAnimation_FacePose::SetDefaultEyeValues
// Author       : Richard Rice
// Date         : 30-Oct-2008
////////////////////////////////////////////////////////////////////////////////

void Tools_FacialAnimation_FacePose::SetDefaultEyeValues()
{
	//Eye values default to zero , except for the target mix that needs to default to 1
	//And squint probably needs to default to 1 as well
	for(int i=0; i<ASURA_FACIALANIMATION_EYE_NUMPARAM; i++)
	{
		m_afEyeValues[i] = 0.0f;
	}
	m_afEyeValues[ASURA_FACIALANIMATION_EYE_TARGETMIX] = 1.0f;
	m_afEyeValues[ASURA_FACIALANIMATION_EYE_SQUINT] = 1.0f;
}


////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_FacialAnimation_FacePose::GetEyeValue
// Author       : Emil Popovic
// Date         : 2-Dec-2008
////////////////////////////////////////////////////////////////////////////////

float Tools_FacialAnimation_FacePose::GetEyeValue( u_int uEyeNumParam ) const
{
	Asura_Assertf( (uEyeNumParam<ASURA_FACIALANIMATION_EYE_NUMPARAM), ("Failed to get Facial Animation EyeValue. Access is out of range. Field has %d elements but I am accessing element %d.", ASURA_FACIALANIMATION_EYE_NUMPARAM, uEyeNumParam ) );

	return m_afEyeValues[uEyeNumParam];
}



/// End of function 'Tools_FacialAnimation_FacePose::SetDefaultEyeValues'. ///


#endif	//defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)


/// End of file. ///
