
///////////////////////////////////////////////////////////////////////////////
//
// File   : Tools_FacialAnimation_FacePose.h
// Author : Andy Tate
// Date   : 8-May-2008
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __TOOLS_FACIALANIMATION_FACEPOSE_H__
#define __TOOLS_FACIALANIMATION_FACEPOSE_H__

#if defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura_HashTree.h"
#include "Core/Asura_String.h"
#include "FacialAnimation/Asura_FacialAnimation_FaceAnimation.h"


////////////////////////////////////////////////////////////////////////////////
// Forward class declarations...
////////////////////////////////////////////////////////////////////////////////

class Asura_Chunk_Stream;
class Asura_FacialAnimation_AnimationHandler;


////////////////////////////////////////////////////////////////////////////////
// Class declaration...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Class        : Tools_FacialAnimation_MuscleGroupWeighting
// Author       : Andy Tate
// Date         : 8-May-2008
//
// Description  : Tools-class used to store the weightings of individual
//				  muscle-groups (used to make up a single face-pose).
////////////////////////////////////////////////////////////////////////////////

class Tools_FacialAnimation_MuscleGroupWeighting
{
public:
	// Public functions...
	Tools_FacialAnimation_MuscleGroupWeighting(void);
	Tools_FacialAnimation_MuscleGroupWeighting(const Asura_Hash_ID uHash, Asura_Chunk_Stream& xStream);
//	~Tools_FacialAnimation_MuscleGroupWeighting();

	// Chunk-stream functions...
	void	ReadFromChunkStream(Asura_Chunk_Stream& xStream);
	void	WriteToChunkStream(Asura_Chunk_Stream& xStream) const;

	// Accessor functions...
	void				SetName(const char* szName)			{ m_xName = szName; }
	void				SetName(const Asura_String& xName)	{ m_xName = xName; }
	const Asura_String&	GetName(void) const					{ return(m_xName); }

	void	SetWeightAndBalance(const float fWeight, const float fBalance)	{ m_fWeight = fWeight; m_fBalance = fBalance; }
	float	GetWeight(void) const											{ return(m_fWeight); }
	float	GetBalance(void) const											{ return(m_fBalance); }


protected:
	// Protected variables...

	// Protected functions...


private:
	// Private variables...
	Asura_String		m_xName;			// The name of the muscle-group.
	float				m_fWeight;			// The weighting of this muscle-group.
	float				m_fBalance;			// The balance value of this muscle-group (optional).

	static const u_int	s_uChunkVersion;	// Current version used for reading/writing data.

	// Private functions...
	Tools_FacialAnimation_MuscleGroupWeighting(const Tools_FacialAnimation_MuscleGroupWeighting& xOther);

	Tools_FacialAnimation_MuscleGroupWeighting&	operator=(const Tools_FacialAnimation_MuscleGroupWeighting& xOther);

};   // End of class 'Tools_FacialAnimation_MuscleGroupWeighting'.



////////////////////////////////////////////////////////////////////////////////
// Class        : Tools_FacialAnimation_FacePose
// Author       : Andy Tate
// Date         : 8-May-2008
//
// Description  : Class used to store a set of weightings for individual 
//				  muscle-groups, making up a single face-pose.
////////////////////////////////////////////////////////////////////////////////

class Tools_FacialAnimation_FacePose
{
public:
	// Public typedefs...
	typedef Asura_TemplateHashTree<Tools_FacialAnimation_MuscleGroupWeighting> MuscleGroupWeightingContainer;
	typedef Asura_TemplateHashTree_Iterator<Tools_FacialAnimation_MuscleGroupWeighting> MuscleGroupWeightingIterator;
	typedef Asura_TemplateHashTree_ConstIterator<Tools_FacialAnimation_MuscleGroupWeighting> MuscleGroupWeightingConstIterator;

	// Public functions...
	Tools_FacialAnimation_FacePose(const Asura_Hash_ID uHash, Asura_Chunk_Stream& xStream);
	Tools_FacialAnimation_FacePose(void);
//	~Tools_FacialAnimation_FacePose();

	// Copy functions...
	#ifdef ASURA_MAIN
	bool	CopyFromAnimationHandler(const Asura_FacialAnimation_AnimationHandler& xAnimationHandler);
	bool	CopyToAnimationHandler(Asura_FacialAnimation_AnimationHandler& xAnimationHandler) const;
	#endif

	// Chunk-stream functions...
	void	ReadFromChunkStream(Asura_Chunk_Stream& xStream);
	void	WriteToChunkStream(Asura_Chunk_Stream& xStream) const;

	// Accessor functions...
	void				SetName(const char* szName)			{ m_xName = szName; }
	void				SetName(const Asura_String& xName)	{ m_xName = xName; }
	const Asura_String&	GetName(void) const					{ return(m_xName); }

	void				SetFaceTypeName(const char* szName)			{ m_xFaceTypeName = szName; }
	void				SetFaceTypeName(const Asura_String& xName)	{ m_xFaceTypeName = xName; }
	const Asura_String&	GetFaceTypeName(void) const					{ return(m_xFaceTypeName); }

	u_int									GetNumMuscleGroupWeightings(void) const			{ return(m_xMuscleGroupWeightingContainer.GetNumberOfNodes()); }
	const MuscleGroupWeightingContainer&	GetMuscleGroupWeightingContainer(void) const	{ return(m_xMuscleGroupWeightingContainer); }

	float 				GetEyeValue( u_int uEyeNumParam ) const;

	//resets the eyes to the defaults
	void SetDefaultEyeValues();


protected:
	// Protected variables...

	// Protected functions...


private:
	// Private variables...
	Asura_String					m_xName;							// The name of this face-pose.
	Asura_String					m_xFaceTypeName;					// The name of the face-type this face-pose is bound to.
	MuscleGroupWeightingContainer	m_xMuscleGroupWeightingContainer;	// The weighting data for the individual muscle-groups making up this face-pose.

	float							m_afEyeValues[ASURA_FACIALANIMATION_EYE_NUMPARAM];//the eye weights for this pose

	static const u_int				s_uChunkVersion;					// Current version used for reading/writing data.

	// Private functions...
	Tools_FacialAnimation_FacePose(const Tools_FacialAnimation_FacePose& xOther);

	Tools_FacialAnimation_FacePose&	operator=(const Tools_FacialAnimation_FacePose& xOther);

};   // End of class 'Tools_FacialAnimation_FacePose'.

#endif	//defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)

#endif   // ifndef __TOOLS_FACIALANIMATION_FACEPOSE_H__


/// End of file. ///
