
///////////////////////////////////////////////////////////////////////////////
//
// File   : Tools_FacialAnimation_Face_MuscleGroup.cpp
// Author : Andy Tate
// Date   : 9-May-2008
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

#if defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)

// This file's header...
#include "FacialAnimation/Tools_FacialAnimation_Face_MuscleGroup.h"

// Asura independent header-files...
#include "Animation/Asura_Animation.h"
#include "ChunkSystem/Asura_Chunk_Loaders.h"
#include "ChunkSystem/Asura_Chunk_Stream.h"

// Tools header-files...
#include "Hierarchy/Tools_Animation_Files.h"
#include "Misc/Tools_Timed_Messages.h"


////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Version	Author	Comment
// ~~~~~~~	~~~~~~	~~~~~~~
// 1		ART		Created.
////////////////////////////////////////////////////////////////////////////////
const u_int	Tools_FacialAnimation_Face_MuscleGroup::s_uChunkVersion	= 1;


////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Private functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Protected functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Public functions...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_FacialAnimation_Face_MuscleGroup::Tools_FacialAnimation_Face_MuscleGroup
// Author       : Andy Tate
// Date         : 9-May-2008
////////////////////////////////////////////////////////////////////////////////

Tools_FacialAnimation_Face_MuscleGroup::Tools_FacialAnimation_Face_MuscleGroup(const char* szName,
																			   const Asura_FacialAnimation_MuscleGroup::ControlType eControlType)
: m_xName(szName)
, m_eControlType(eControlType)
{
	switch(eControlType)
	{
		case Asura_FacialAnimation_MuscleGroup::CT_SINGLE:
		case Asura_FacialAnimation_MuscleGroup::CT_BALANCE:
		{
			m_uNumAnimations	= 1;
			break;
		}

		case Asura_FacialAnimation_MuscleGroup::CT_SINGLE_PAIR:
		case Asura_FacialAnimation_MuscleGroup::CT_BALANCE_PAIR:
		{
			m_uNumAnimations	= 2;
			break;
		}

		default:
		{
			Asura_Assert(false, "Unexpected control-type specified.");
			break;
		}
	}		// End of switch.
}

/// End of function 'Tools_FacialAnimation_Face_MuscleGroup::Tools_FacialAnimation_Face_MuscleGroup'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_FacialAnimation_Face_MuscleGroup::Tools_FacialAnimation_Face_MuscleGroup
// Author       : Andy Tate
// Date         : 9-May-2008
////////////////////////////////////////////////////////////////////////////////

Tools_FacialAnimation_Face_MuscleGroup::Tools_FacialAnimation_Face_MuscleGroup(Asura_Chunk_Stream& xStream)
{
	ReadFromChunkStream(xStream);
}

/// End of function 'Tools_FacialAnimation_Face_MuscleGroup::Tools_FacialAnimation_Face_MuscleGroup'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_FacialAnimation_Face_MuscleGroup::Tools_FacialAnimation_Face_MuscleGroup
// Author       : Andy Tate
// Date         : 9-May-2008
////////////////////////////////////////////////////////////////////////////////

Tools_FacialAnimation_Face_MuscleGroup::Tools_FacialAnimation_Face_MuscleGroup(const Asura_Hash_ID uHash,
																	 Asura_Chunk_Stream& xStream)
{
	ReadFromChunkStream(xStream);
}

/// End of function 'Tools_FacialAnimation_Face_MuscleGroup::Tools_FacialAnimation_Face_MuscleGroup'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_FacialAnimation_Face_MuscleGroup::ReloadAsuraAnimations
// Author       : Andy Tate
// Date         : 23-May-2008
////////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_MAIN
u_int Tools_FacialAnimation_Face_MuscleGroup::ReloadAsuraAnimations(void)
{
	u_int	uNumAnimationsReloaded	= 0;

	for(u_int u = 0; u < m_uNumAnimations; u++)
	{
		const Asura_String&		xAnimationName	= m_axAnimationNames[u];
		const char*				szAnimationName	= xAnimationName.GetString();
		const Asura_Hash_ID		uAnimationHash	= xAnimationName.GetHash();
		Asura_Hierarchy_Anim*	pxHierarchyAnim	= Asura_Animation_System::GetAnimByHash(uAnimationHash);

		Asura_String			xFilename;

		// Attempt to re-initialise this animation...
		if(pxHierarchyAnim != NULL)
		{
			pxHierarchyAnim->m_bAllowOverwriteOnLoad	= true;
		}

		// Time to find the Asura-file and load the animation...
		if(Tools_Animation_Files::FindFile(szAnimationName, &xFilename))
		{
			const char*	szFilename	= xFilename.GetString();

			if(LoadAsuraFile(szFilename))
			{
				//Tools_Timed_Message_System::AddMessage(Tools_Timed_Message_System::COLOUR_SUCCESS, "Successfully loaded Asura-file \"%s\" for animation \"%s\".", szFilename, szAnimationName);
				uNumAnimationsReloaded++;
			}
			else
			{
				Tools_Timed_Message_System::AddMessage(Tools_Timed_Message_System::COLOUR_ERROR, "Failed to load Asura-file \"%s\" for animation \"%s\".", szFilename, szAnimationName);
			}
		}
		else
		{
			Tools_Timed_Message_System::AddMessage(Tools_Timed_Message_System::COLOUR_ERROR, "Failed to find Asura-file for animation \"%s\".", szAnimationName);
		}
	}

	return(uNumAnimationsReloaded);
}
#endif //ASURA_MAIN
/// End of function 'Tools_FacialAnimation_Face_MuscleGroup::ReloadAsuraAnimations'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_FacialAnimation_Face_MuscleGroup::ReadFromChunkStream
// Author       : Andy Tate
// Date         : 9-May-2008
////////////////////////////////////////////////////////////////////////////////

void Tools_FacialAnimation_Face_MuscleGroup::ReadFromChunkStream(Asura_Chunk_Stream& xStream)
{
	u_int	uVersion;

	// First, read (and verify) the chunk-version.
	xStream >> uVersion;

	switch(uVersion)
	{
		case 1:
		{
			u_int	uControlType;

			xStream >> m_xName;
			xStream >> uControlType;
			xStream >> m_uNumAnimations;

			for(u_int u = 0; u < m_uNumAnimations; u++)
			{
				xStream >> m_axAnimationNames[u];
			}

			m_eControlType	= static_cast<Asura_FacialAnimation_MuscleGroup::ControlType>(uControlType);
			break;
		}

		default:
		{
			Asura_Assertf(false, ("Unexpected chunk-version %d", uVersion));
			break;
		}
	}		// End of switch.
}

/// End of function 'Tools_FacialAnimation_Face_MuscleGroup::ReadFromChunkStream'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_FacialAnimation_Face_MuscleGroup::WriteToChunkStream
// Author       : Andy Tate
// Date         : 9-May-2008
////////////////////////////////////////////////////////////////////////////////

void Tools_FacialAnimation_Face_MuscleGroup::WriteToChunkStream(Asura_Chunk_Stream& xStream) const
{
	const u_int	uControlType	= m_eControlType;

	// First write the chunk-version...
	xStream << s_uChunkVersion;

	xStream << m_xName;
	xStream << uControlType;
	xStream << m_uNumAnimations;

	for(u_int u = 0; u < m_uNumAnimations; u++)
	{
		xStream << m_axAnimationNames[u];
	}
}

/// End of function 'Tools_FacialAnimation_Face_MuscleGroup::WriteToChunkStream'. ///

#endif	//defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)


/// End of file. ///
