
///////////////////////////////////////////////////////////////////////////////
//
// File   : Tools_FacialAnimation_LipsyncConversion.h
// Author : Andy Tate
// Date   : 6-Aug-2008
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __TOOLS_FACIALANIMATION_LIPSYNCCONVERSION_H__
#define __TOOLS_FACIALANIMATION_LIPSYNCCONVERSION_H__

#if defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Forward class declarations...
////////////////////////////////////////////////////////////////////////////////

class Tools_FacialAnimation_FaceAnimation;


////////////////////////////////////////////////////////////////////////////////
// Class declaration...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Class        : Tools_FacialAnimation_LipsyncConversion
// Author       : Andy Tate
// Date         : 6-Aug-2008
//
// Description  : Abstract class used to define the interface for lip-sync
//				  conversion from .WAV files to facial-animation.
////////////////////////////////////////////////////////////////////////////////

class Tools_FacialAnimation_LipsyncConversion
{
public:
	// Public functions...
	Tools_FacialAnimation_LipsyncConversion(void)		{}
	virtual ~Tools_FacialAnimation_LipsyncConversion()	{}

	// Abstract conversion functions...
	virtual	bool									BeginConversion(void)																					= 0;
	virtual Tools_FacialAnimation_FaceAnimation*	ConvertTextless(const char* szFaceTypeName, const char* szBaseFacePoseName, const char* szWAVFilename)	= 0;
	virtual bool									EndConversion(void)																						= 0;

	// Abstract accessor functions...
	virtual bool		IsConversionInProgress(void) const				= 0;

	virtual const u_int	GetNumPhonemes(void) const						= 0;
	virtual char const*	GetPhonemeName(const u_int uIndex) const		= 0;
	virtual char const*	GetPhonemeComments(const u_int uIndex) const	= 0;


protected:
	// Protected variables...

	// Protected functions...
	Tools_FacialAnimation_FaceAnimation*	BeginAnimationCreation(const char* szFaceTypeName, const char* szBaseFacePoseName, const char* szWAVFilename) const;
	void									EndAnimationCreation(Tools_FacialAnimation_FaceAnimation& xAnimation) const;


private:
	// Private variables...

	// Private functions...
	Tools_FacialAnimation_LipsyncConversion(const Tools_FacialAnimation_LipsyncConversion& xOther);

	Tools_FacialAnimation_LipsyncConversion&	operator=(const Tools_FacialAnimation_LipsyncConversion& xOther);

};   // End of class 'Tools_FacialAnimation_LipsyncConversion'.

#endif	//defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)

#endif   // ifndef __TOOLS_FACIALANIMATION_LIPSYNCCONVERSION_H__


/// End of file. ///
