
///////////////////////////////////////////////////////////////////////////////
//
// File   : Tools_FacialAnimation_System.cpp
// Author : Andy Tate
// Date   : 8-May-2008
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

#if defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)

// This file's header...
#include "FacialAnimation/Tools_FacialAnimation_System.h"

// Asura independent header-files...
#include "Animation/Asura_Animation.h"
#include "ChunkSystem/Asura_Chunk_FacialAnimation.h"
#include "ChunkSystem/Asura_Chunk_Hierarchy.h"
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "ChunkSystem/Asura_SaveBlock.h"
#include "Interface/Asura_Tree_Dialog.h"

// Tools header-files...
#include "ChunkSystem/Tools_Chunk_FacialAnimation.h"
#include "FacialAnimation/Tools_FacialAnimation_Face_MuscleGroup.h"
#include "Misc/Tools_Timed_Messages.h"
#include "PC/FileSystem/Tools_FileSystem.h"


////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////

Tools_FacialAnimation_System::TypeData		Tools_FacialAnimation_System::s_xTypeData;
Tools_FacialAnimation_System::FaceData		Tools_FacialAnimation_System::s_xFaceData;
Tools_FacialAnimation_System::AnimationData	Tools_FacialAnimation_System::s_xAnimationData;
Tools_FacialAnimation_System::PoseData		Tools_FacialAnimation_System::s_xPoseData;

Tools_FacialAnimation_LipsyncConversion*	Tools_FacialAnimation_System::s_pxLipsyncConversion	= NULL;

bool										Tools_FacialAnimation_System::s_bFaceFilenamesValid = false;
Asura_TemplateHashTree<Asura_String>		Tools_FacialAnimation_System::s_xFaceFilenames;
bool										Tools_FacialAnimation_System::s_bAnimationFilenamesValid = false;
Asura_TemplateHashTree<Asura_String>		Tools_FacialAnimation_System::s_xAnimationFilenames;



////////////////////////////////////////////////////////////////////////////////
// Private functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Protected functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Public functions...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_FacialAnimation_System::Initialise
// Author       : Andy Tate
// Date         : 8-May-2008
////////////////////////////////////////////////////////////////////////////////

void Tools_FacialAnimation_System::Initialise(void)
{
}

/// End of function 'Tools_FacialAnimation_System::Initialise'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_FacialAnimation_System::Shutdown
// Author       : Andy Tate
// Date         : 8-May-2008
////////////////////////////////////////////////////////////////////////////////

void Tools_FacialAnimation_System::Shutdown(void)
{
	s_xTypeData.Reset();
	s_xFaceData.Reset();
	s_xAnimationData.Reset();
	s_xPoseData.Reset();
	s_xFaceFilenames.Reset();
	s_bFaceFilenamesValid = false;
	s_xAnimationFilenames.Reset();
	s_bAnimationFilenamesValid = false;
}

/// End of function 'Tools_FacialAnimation_System::Shutdown'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_FacialAnimation_System::CreateType
// Author       : Andy Tate
// Date         : 9-May-2008
////////////////////////////////////////////////////////////////////////////////

Tools_FacialAnimation_FaceType* Tools_FacialAnimation_System::CreateType(const char* szName)
{
	const Asura_Hash_ID				uHash	= Asura_GetHashID(szName);
	Tools_FacialAnimation_FaceType*	pxType	= s_xTypeData.GetDataByHash(uHash);
	
	// If we've already have this face-type in memory, delete it...
	if(pxType != NULL)
	{
		s_xTypeData.DeleteNodeByHash(uHash);
	}

	// Create the new face-type...
	pxType	= s_xTypeData.AddNode(uHash);

	if(pxType == NULL)
	{
		Asura_Assert(false, "We've already got this face-type in memory.");
		return(NULL);
	}
	else
	{
		pxType->SetName(szName);
		return(pxType);
	}
}

/// End of function 'Tools_FacialAnimation_System::CreateType'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_FacialAnimation_System::CreateType
// Author       : Andy Tate
// Date         : 9-May-2008
////////////////////////////////////////////////////////////////////////////////

Tools_FacialAnimation_FaceType* Tools_FacialAnimation_System::CreateType(const Asura_Hash_ID uHash,
																		 Asura_Chunk_Stream& xStream)
{
	Tools_FacialAnimation_FaceType*	pxType	= s_xTypeData.GetDataByHash(uHash);

	// If we've already have this face-type in memory, delete it...
	if(pxType != NULL)
	{
		s_xTypeData.DeleteNodeByHash(uHash);
	}

	// Create the new face-type...
	pxType	= s_xTypeData.AddNode(uHash, xStream);

	if(pxType == NULL)
	{
		Asura_Assert(false, "We've already got this face-type in memory.");
		return(NULL);
	}
	else
	{
		return(pxType);
	}
}

/// End of function 'Tools_FacialAnimation_System::CreateType'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_FacialAnimation_System::FindOrLoadType
// Author       : Andy Tate
// Date         : 7-Aug-2008
////////////////////////////////////////////////////////////////////////////////

Tools_FacialAnimation_FaceType* Tools_FacialAnimation_System::FindOrLoadType(const char* szName)
{
	Tools_FacialAnimation_FaceType*	pxType	= FindType(szName);

	if(pxType == NULL)
	{
		Asura_String		xPattern(CreateTypeFilename(szName, true));
		List<Asura_String>	xFileList;

		Tools_FileSystem::FindFilesLocalAndNetwork(xPattern.GetString(), xFileList, TOOLS_FILEFIND_RECURSIVE);

		if(xFileList.size() == 0)
		{
			return(NULL);
		}
		else if(xFileList.size() > 1)
		{
			Asura_Assert(false, "Found more than one match for this file - that shouldn't be possible.");
			return(NULL);
		}
		else
		{
			const Asura_String&	xFilename	= xFileList.first_entry();
			const char*			szFilename	= xFilename.GetString();

			if(LoadAsuraFile(szFilename))
			{
				pxType	= FindType(szName);
				return(pxType);
			}
			else
			{
				return(NULL);
			}
		}
	}
	else
	{
		return(pxType);
	}
}

/// End of function 'Tools_FacialAnimation_System::FindOrLoadType'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_FacialAnimation_System::WriteTypeFile
// Author       : Andy Tate
// Date         : 9-May-2008
////////////////////////////////////////////////////////////////////////////////

bool Tools_FacialAnimation_System::WriteTypeFile(const char* szFilename,
												 const Tools_FacialAnimation_FaceType& xType)
{
	const Asura_String&				xName	= xType.GetName();
	const Asura_Hash_ID				uHash	= xName.GetHash();

	Asura_SaveBlock					xSaveBlock;

	Asura_Chunk_SaveBlock_Stream	xStream(&xSaveBlock);

	// Add the appropriate chunk to the save-block...
	xStream.SetSwapEndian(false);
	Tools_Chunk_FacialAnimation_FaceType::Create(uHash, xType, xStream);

	// Close the save-block and write it to the appropriate file...
	xSaveBlock.AddAsuraTerminator();
	Tools_FileSystem::EnsureDirectoryExists(szFilename);
	return(xSaveBlock.SaveFile(szFilename, false));
}

/// End of function 'Tools_FacialAnimation_System::WriteTypeFile'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_FacialAnimation_System::CreateFace
// Author       : Andy Tate
// Date         : 9-May-2008
////////////////////////////////////////////////////////////////////////////////

Tools_FacialAnimation_Face* Tools_FacialAnimation_System::CreateFace(const char* szName)
{
	const Asura_Hash_ID			uHash	= Asura_GetHashID(szName);
	Tools_FacialAnimation_Face*	pxFace	= s_xFaceData.GetDataByHash(uHash);
	
	// If we already have this face in memory, delete it...
	if(pxFace != NULL)
	{
		s_xFaceData.DeleteNodeByHash(uHash);
	}
	
	// Create the new face...
	pxFace	= s_xFaceData.AddNode(uHash);

	if(pxFace == NULL)
	{
		Asura_Assert(false, "We've already got this face in memory.");
		return(NULL);
	}
	else
	{
		pxFace->SetName(szName);
		return(pxFace);
	}
}

/// End of function 'Tools_FacialAnimation_System::CreateFace'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_FacialAnimation_System::CreateFace
// Author       : Andy Tate
// Date         : 9-May-2008
////////////////////////////////////////////////////////////////////////////////

Tools_FacialAnimation_Face* Tools_FacialAnimation_System::CreateFace(const Asura_Hash_ID uHash,
																	 Asura_Chunk_Stream& xStream)
{
	Tools_FacialAnimation_Face*	pxFace	= s_xFaceData.GetDataByHash(uHash);

	// If we already have this face in memory, delete it...
	if(pxFace != NULL)
	{
		s_xFaceData.DeleteNodeByHash(uHash);
	}

	// Create the new face...
	pxFace	= s_xFaceData.AddNode(uHash, xStream);

	if(pxFace == NULL)
	{
		Asura_Assert(false, "We've already got this face in memory.");
		return(NULL);
	}
	else
	{
		return(pxFace);
	}
}

/// End of function 'Tools_FacialAnimation_System::CreateFace'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_FacialAnimation_System::FindOrLoadFace
// Author       : Richard Rice
// Date         : 17-Nov-2008
////////////////////////////////////////////////////////////////////////////////

Tools_FacialAnimation_Face* Tools_FacialAnimation_System::FindOrLoadFace(const Asura_Hash_ID uHash)
{
	if(!uHash) return NULL;

	Tools_FacialAnimation_Face*	pxFace	= FindFace(uHash);

	if(pxFace == NULL)
	{
		//search for the face in the filename tree
		if(!s_bFaceFilenamesValid) UpdateFaceFilenames();
		const Asura_String* pxFilename = s_xFaceFilenames.GetDataByHash(uHash);

		if(pxFilename)
		{
			if(LoadAsuraFile(pxFilename->GetString()))
			{
				pxFace = FindFace(uHash);
				return(pxFace);
			}
			else
			{
				return(NULL);
			}
		}
		
		return NULL;
	}
	else
	{
		return(pxFace);
	}
}

/// End of function 'Tools_FacialAnimation_System::FindOrLoadFace'. ///




////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_FacialAnimation_System::WriteFaceFile
// Author       : Andy Tate
// Date         : 9-May-2008
////////////////////////////////////////////////////////////////////////////////

bool Tools_FacialAnimation_System::WriteFaceFile(const char* szFilename,
												 const Tools_FacialAnimation_Face& xFace)
{
	Asura_SaveBlock											xSaveBlock;

	const Asura_String&										xFaceName				= xFace.GetName();
	const Asura_Hash_ID										uFaceHash				= xFaceName.GetHash();
	const Tools_FacialAnimation_Face::MuscleGroupContainer&	xMuscleGroupContainer	= xFace.GetMuscleGroupContainer();

	Asura_Chunk_SaveBlock_Stream							xStream(&xSaveBlock, false);

	// Firstly, write-out all of the animation chunks associated with this face...
	for(Tools_FacialAnimation_Face::MuscleGroupConstIterator xIterator(xMuscleGroupContainer); !xIterator.Done(); xIterator.Next())
	{
		const Tools_FacialAnimation_Face_MuscleGroup*	pxMuscleGroup	= xIterator.GetCurrent();

		if(pxMuscleGroup != NULL)
		{
			const u_int	uNumAnimations	= pxMuscleGroup->GetNumAnimations();

			for(u_int u = 0; u < uNumAnimations; u++)
			{
				const Asura_String&	xAnimationName	= pxMuscleGroup->GetAnimationName(u);

				if(!xAnimationName.IsBlank())
				{
					const Asura_Hash_ID			uAnimationHash	= xAnimationName.GetHash();
					const Asura_Hierarchy_Anim*	pxAnimation		= Asura_Animation_System::GetAnimByHash(uAnimationHash);

					if(pxAnimation == NULL)
					{
						Asura_Assertf(false, ("Failed to find animation \"%s\" to be saved with this face.", xAnimationName.GetString()));
					}
					else
					{
						Asura_Chunk_Hierarchy_Anim::Create(pxAnimation, xStream);
					}
				}
			}
		}
	}

	// Write-out the face chunk...
	Tools_Chunk_FacialAnimation_Face::Create(uFaceHash, xFace, xStream);

	// Close the save-block and write it to the appropriate file...
	xSaveBlock.AddAsuraTerminator();
	Tools_FileSystem::EnsureDirectoryExists(szFilename);
	return(xSaveBlock.SaveFile(szFilename, false));
}

/// End of function 'Tools_FacialAnimation_System::WriteFaceFile'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_FacialAnimation_System::CreateAnimation
// Author       : Andy Tate
// Date         : 6-Aug-2008
////////////////////////////////////////////////////////////////////////////////

Tools_FacialAnimation_FaceAnimation* Tools_FacialAnimation_System::CreateAnimation(const char* szFaceTypeName,
																				   const char* szBaseFacePoseName,
																				   const char* szAnimationName,
																				   const char* szWAVFilename)
{
	const Asura_Hash_ID						uHash		= Asura_GetHashID(szAnimationName);
	Tools_FacialAnimation_FaceAnimation*	pxAnimation	= s_xAnimationData.GetDataByHash(uHash);
	
	// If we've already got this animation in memory, delete it...
	if(pxAnimation != NULL)
	{
		s_xAnimationData.DeleteNodeByHash(uHash);
	}

	// Create the new animation...
	pxAnimation	= s_xAnimationData.AddNode(uHash);

	if(pxAnimation == NULL)
	{
		return(NULL);
	}
	else
	{
		pxAnimation->SetName(szAnimationName);
		pxAnimation->SetFaceTypeName(szFaceTypeName);
		pxAnimation->SetBaseFacePoseName(szBaseFacePoseName);
		pxAnimation->SetWAVFilename(szWAVFilename);
		return(pxAnimation);
	}
}

/// End of function 'Tools_FacialAnimation_System::CreateAnimation'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_FacialAnimation_System::CreateAnimation
// Author       : Andy Tate
// Date         : 9-May-2008
////////////////////////////////////////////////////////////////////////////////

Tools_FacialAnimation_FaceAnimation* Tools_FacialAnimation_System::CreateAnimation(const Asura_Hash_ID uHash,
																				   Asura_Chunk_Stream& xStream)
{
	Tools_FacialAnimation_FaceAnimation*	pxAnimation	= s_xAnimationData.GetDataByHash(uHash);

	// If we've already got this animation in memory, delete it...
	if(pxAnimation != NULL)
	{
		s_xAnimationData.DeleteNodeByHash(uHash);
	}

	// Create the new animation...
	pxAnimation	= s_xAnimationData.AddNode(uHash, xStream);

	if(pxAnimation == NULL)
	{
		Asura_Assert(false, "We've already got this animation in memory.");
		return(NULL);
	}
	else
	{
		return(pxAnimation);
	}
}

/// End of function 'Tools_FacialAnimation_System::CreateAnimation'. ///




////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_FacialAnimation_System::FindOrLoadAnimation
// Author       : Richard Rice
// Date         : 18-Nov-2008
////////////////////////////////////////////////////////////////////////////////

Tools_FacialAnimation_FaceAnimation* Tools_FacialAnimation_System::FindOrLoadAnimation(const Asura_Hash_ID uHash)
{
	if(!uHash) return NULL;

	Tools_FacialAnimation_FaceAnimation*	pxAnimation	= FindAnimation(uHash);

	if(pxAnimation == NULL)
	{

		//search for the animation in the filename tree
		if(!s_bAnimationFilenamesValid) UpdateAnimationFilenames();
		const Asura_String* pxFilename = s_xAnimationFilenames.GetDataByHash(uHash);
		
		if(pxFilename)
		{
			if(LoadAsuraFile(pxFilename->GetString()))
			{
				pxAnimation = FindAnimation(uHash);
				return(pxAnimation);
			}
			else
			{
				return(NULL);
			}
		}

		return NULL;
	}
	else
	{
		return(pxAnimation);
	}
}

/// End of function 'Tools_FacialAnimation_System::FindOrLoadAnimation'. ///




////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_FacialAnimation_System::WriteAnimationFile
// Author       : Andy Tate
// Date         : 9-May-2008
////////////////////////////////////////////////////////////////////////////////

bool Tools_FacialAnimation_System::WriteAnimationFile(const char* szFilename,
													  const Asura_Hash_ID uAnimationHash,
													  const Tools_FacialAnimation_FaceAnimation& xToolsAnimation,
													  const Asura_FacialAnimation_FaceAnimation& xAsuraAnimation)
{
	Asura_SaveBlock					xSaveBlock;

	Asura_Chunk_SaveBlock_Stream	xStream(&xSaveBlock);

	// Add the appropriate chunk to the save-block...
	xStream.SetSwapEndian(false);
	Tools_Chunk_FacialAnimation_FaceAnimation::Create(uAnimationHash, xToolsAnimation, xStream);
	Asura_Chunk_FacialAnimation_FaceAnimation::Create(uAnimationHash, xAsuraAnimation, xStream);

	// Close the save-block and write it to the appropriate file...
	xSaveBlock.AddAsuraTerminator();
	Tools_FileSystem::EnsureDirectoryExists(szFilename);
	return(xSaveBlock.SaveFile(szFilename, false));
}

/// End of function 'Tools_FacialAnimation_System::WriteAnimationFile'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_FacialAnimation_System::CreatePose
// Author       : Andy Tate
// Date         : 9-May-2008
////////////////////////////////////////////////////////////////////////////////

Tools_FacialAnimation_FacePose* Tools_FacialAnimation_System::CreatePose(const char* szName)
{
	const Asura_Hash_ID				uHash	= Asura_GetHashID(szName);
	Tools_FacialAnimation_FacePose*	pxPose	= s_xPoseData.GetDataByHash(uHash);
	
	// If we've already got this face-pose in memory - delete it...
	if(pxPose != NULL)
	{
		s_xPoseData.DeleteNodeByHash(uHash);
	}

	// Create a new pose...
	pxPose	= s_xPoseData.AddNode(uHash);

	if(pxPose == NULL)
	{
		Asura_Assert(false, "We've already got this pose in memory.");
		return(NULL);
	}
	else
	{
		pxPose->SetName(szName);
		return(pxPose);
	}
}

/// End of function 'Tools_FacialAnimation_System::CreatePose'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_FacialAnimation_System::CreatePose
// Author       : Andy Tate
// Date         : 9-May-2008
////////////////////////////////////////////////////////////////////////////////

Tools_FacialAnimation_FacePose* Tools_FacialAnimation_System::CreatePose(const Asura_Hash_ID uHash,
																		 Asura_Chunk_Stream& xStream)
{
	Tools_FacialAnimation_FacePose*	pxPose	= s_xPoseData.GetDataByHash(uHash);

	// If we've already got this face-pose in memory - delete it...
	if(pxPose != NULL)
	{
		s_xPoseData.DeleteNodeByHash(uHash);
	}

	// Create a new pose...
	pxPose	= s_xPoseData.AddNode(uHash, xStream);

	if(pxPose == NULL)
	{
		Asura_Assert(false, "We've already got this pose in memory.");
		return(NULL);
	}
	else
	{
		return(pxPose);
	}
}

/// End of function 'Tools_FacialAnimation_System::CreatePose'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_FacialAnimation_System::FindOrLoadPose
// Author       : Andy Tate
// Date         : 7-Aug-2008
////////////////////////////////////////////////////////////////////////////////

Tools_FacialAnimation_FacePose* Tools_FacialAnimation_System::FindOrLoadPose(const char* szName)
{
	Tools_FacialAnimation_FacePose*	pxPose	= FindPose(szName);

	if(pxPose == NULL)
	{
		Asura_String		xPattern(CreatePoseFilename(szName, true));
		List<Asura_String>	xFileList;

		Tools_FileSystem::FindFilesLocalAndNetwork(xPattern.GetString(), xFileList, TOOLS_FILEFIND_RECURSIVE);

		if(xFileList.size() == 0)
		{
			return(NULL);
		}
		else if(xFileList.size() > 1)
		{
			Asura_Assert(false, "Found more than one match for this file - that shouldn't be possible.");
			return(NULL);
		}
		else
		{
			const Asura_String&	xFilename	= xFileList.first_entry();
			const char*			szFilename	= xFilename.GetString();

			if(LoadAsuraFile(szFilename))
			{
				pxPose	= FindPose(szName);
				return(pxPose);
			}
			else
			{
				return(NULL);
			}
		}
	}
	else
	{
		return(pxPose);
	}
}

/// End of function 'Tools_FacialAnimation_System::FindOrLoadPose'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_FacialAnimation_System::WritePoseFile
// Author       : Andy Tate
// Date         : 9-May-2008
////////////////////////////////////////////////////////////////////////////////

bool Tools_FacialAnimation_System::WritePoseFile(const char* szFilename,
												 const Tools_FacialAnimation_FacePose& xPose)
{
	const Asura_String&				xName	= xPose.GetName();
	const Asura_Hash_ID				uHash	= xName.GetHash();

	Asura_SaveBlock					xSaveBlock;

	Asura_Chunk_SaveBlock_Stream	xStream(&xSaveBlock);

	// Add the appropriate chunk to the save-block...
	xStream.SetSwapEndian(false);
	Tools_Chunk_FacialAnimation_FacePose::Create(uHash, xPose, xStream);

	// Close the save-block and write it to the appropriate file...
	xSaveBlock.AddAsuraTerminator();
	Tools_FileSystem::EnsureDirectoryExists(szFilename);
	return(xSaveBlock.SaveFile(szFilename, false));
}

/// End of function 'Tools_FacialAnimation_System::WritePoseFile'. ///


////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_FacialAnimation_System::FindOrLoadTypeFromTreeItem
// Author       : Andy Tate
// Date         : 20-May-2008
////////////////////////////////////////////////////////////////////////////////
#ifdef ASURA_MAIN
Tools_FacialAnimation_FaceType* Tools_FacialAnimation_System::FindOrLoadTypeFromTreeItem(const Asura_TreeDialog_Item& xItem,
																						 Asura_String& xPath)
{
	const Asura_Hash_ID	uHash	= xItem.GetHash();

	if(uHash == ASURA_HASH_ID_UNSET)
	{
		return(NULL);
	}
	else
	{
		Tools_FacialAnimation_FaceType*	pxType	= FindType(uHash);

		xItem.ConcatenatePath( xPath );

		if(pxType == NULL)
		{
			Asura_String	xFilename(Tools_FacialAnimation_System::CreateTypeFilename(xPath, true));
			const char*		szFilename	= xFilename.GetString();

			// Attempt to load this face-type file...
			if(LoadAsuraFile(szFilename))
			{
				pxType	= Tools_FacialAnimation_System::FindType(uHash);
			}
			else
			{
				Tools_Timed_Message_System::AddMessage(Tools_Timed_Message_System::COLOUR_ERROR, "Failed to load face-type \"%s\".", szFilename);
			}

			return(pxType);
		}
		else
		{
			return(pxType);
		}
	}
}
#endif   //ASURA_MAIN
/// End of function 'Tools_FacialAnimation_System::FindOrLoadTypeFromTreeItem'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_FacialAnimation_System::FindOrLoadPoseFromTreeItem
// Author       : Andy Tate
// Date         : 20-May-2008
////////////////////////////////////////////////////////////////////////////////
#ifdef ASURA_MAIN
Tools_FacialAnimation_FacePose* Tools_FacialAnimation_System::FindOrLoadPoseFromTreeItem(const Asura_TreeDialog_Item& xItem,
																						 Asura_String& xPath)
{
	const Asura_Hash_ID	uHash	= xItem.GetHash();

	if(uHash == ASURA_HASH_ID_UNSET)
	{
		return(NULL);
	}
	else
	{
		Tools_FacialAnimation_FacePose*	pxPose	= FindPose(uHash);

		xItem.ConcatenatePath( xPath );
		
		if(pxPose == NULL)
		{
			Asura_String	xFilename(Tools_FacialAnimation_System::CreatePoseFilename(xPath, true));
			const char*		szFilename	= xFilename.GetString();

			// Attempt to load this face-pose file...
			if(LoadAsuraFile(szFilename))
			{
				pxPose	= Tools_FacialAnimation_System::FindPose(uHash);
			}
			else
			{
				Tools_Timed_Message_System::AddMessage(Tools_Timed_Message_System::COLOUR_ERROR, "Failed to load face-type \"%s\".", szFilename);
			}

			return(pxPose);
		}
		else
		{
			return(pxPose);
		}
	}
}
#endif //ASURA_MAIN
/// End of function 'Tools_FacialAnimation_System::FindOrLoadPoseFromTreeItem'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_FacialAnimation_System::FindOrLoadFaceFromTreeItem
// Author       : Andy Tate
// Date         : 20-May-2008
////////////////////////////////////////////////////////////////////////////////
#ifdef ASURA_MAIN
Tools_FacialAnimation_Face* Tools_FacialAnimation_System::FindOrLoadFaceFromTreeItem(const Asura_TreeDialog_Item& xItem,
																					 Asura_String& xPath)
{
	const Asura_Hash_ID	uHash	= xItem.GetHash();

	if(uHash == ASURA_HASH_ID_UNSET)
	{
		return(NULL);
	}
	else
	{
		Tools_FacialAnimation_Face*	pxFace	= FindFace(uHash);

		xItem.ConcatenatePath( xPath );
		
		if(pxFace == NULL)
		{
			Asura_String	xFilename(Tools_FacialAnimation_System::CreateFaceFilename(xPath, true));
			const char*		szFilename	= xFilename.GetString();

			// Attempt to load this face file...
			if(LoadAsuraFile(szFilename))
			{
				pxFace	= Tools_FacialAnimation_System::FindFace(uHash);
			}
			else
			{
				Tools_Timed_Message_System::AddMessage(Tools_Timed_Message_System::COLOUR_ERROR, "Failed to load face \"%s\".", szFilename);
			}

			return(pxFace);
		}
		else
		{
			return(pxFace);
		}
	}
}
#endif //ASURA_MAIN
/// End of function 'Tools_FacialAnimation_System::FindOrLoadFaceFromTreeItem'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_FacialAnimation_System::ConvertSoundFilenameToFaceAnimationName
// Author       : Andy Tate
// Date         : 5-Sep-2008
////////////////////////////////////////////////////////////////////////////////

Asura_String Tools_FacialAnimation_System::ConvertSoundFilenameToFaceAnimationName(const char* szFilename)
{
	Asura_String	xRelativeFilename(szFilename);

	xRelativeFilename.Replace("Sounds\\", "");
	xRelativeFilename.StripExtension();
	return(xRelativeFilename);
}

/// End of function 'Tools_FacialAnimation_System::ConvertSoundFilenameToFaceAnimationName'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_FacialAnimation_System::UpdateFaceFilenames
// Author       : Richard Rice
// Date         : 19-Nov-2008
////////////////////////////////////////////////////////////////////////////////

void Tools_FacialAnimation_System::UpdateFaceFilenames()
{
	//Search for all the face files
	const char* szFaceDirectory		= GetFaceDirectoryName();
	const char* szFaceFileExtension	= GetFaceFileExtension();

	const u_int	uFileFlags = TOOLS_FILEFIND_RECURSIVE | TOOLS_FILEFIND_IGNOREBACKUPDIR;
	Asura_String		xDirectoryAndPattern(szFaceDirectory);
	List<Asura_String>	xList;

	xDirectoryAndPattern	+= "\\*.";
	xDirectoryAndPattern	+= szFaceFileExtension;

	Tools_FileSystem::FindFilesLocalAndNetwork(xDirectoryAndPattern.GetString(), xList, uFileFlags);

	//Add each to the face filename tree
	for(LIF<Asura_String> xFaceLIF(&xList); !xFaceLIF.done(); xFaceLIF.next())
	{
		const Asura_String& xFilename = xFaceLIF();
		Asura_String xFaceName;
		xFilename.SplitFilename(0,&xFaceName,0);

		Asura_Hash_ID uFaceHash = xFaceName.GetHash();

		bool bAlreadyExists;
		Asura_String* pxName = s_xFaceFilenames.AddOrFindNode(uFaceHash, &bAlreadyExists);
		Asura_Assert(pxName, "Error adding name to face filename tree");
		if(pxName)
		{
			if(bAlreadyExists)
			{
				//Already an entry for that hash, complain if the name doesn't match
				Asura_Assertf(!pxName->Compare(xFilename), ("Found existing entry in face tree for this hash value , but with a different name.\nExsiting name '%s'.\nNew name '%s'",pxName->GetString(), xFilename.GetString()));
			}
			else
			{
				//Got a new node , so update the name
				*pxName = xFilename;
			}
		}
	}
	s_bFaceFilenamesValid = true;
}
/// End of function 'Tools_FacialAnimation_System::UpdateFaceFilenames'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_FacialAnimation_System::UpdateAnimationFilenames
// Author       : Richard Rice
// Date         : 19-Nov-2008
////////////////////////////////////////////////////////////////////////////////

void Tools_FacialAnimation_System::UpdateAnimationFilenames()
{
	//Search for all the animation files
	const char* szAnimationDirectory		= GetAnimationDirectoryName();
	const char* szAnimationFileExtension	= GetAnimationFileExtension();
  	const int iDirectoryLength = strlen(szAnimationDirectory) + 1;

	const u_int	uFileFlags = TOOLS_FILEFIND_RECURSIVE | TOOLS_FILEFIND_IGNOREBACKUPDIR;
	Asura_String		xDirectoryAndPattern(szAnimationDirectory);
	List<Asura_String>	xList;

	xDirectoryAndPattern	+= "\\*.";
	xDirectoryAndPattern	+= szAnimationFileExtension;

	Tools_FileSystem::FindFilesLocalAndNetwork(xDirectoryAndPattern.GetString(), xList, uFileFlags);

	//Add each to the animation filename tree
	for(LIF<Asura_String> xAnimationLIF(&xList); !xAnimationLIF.done(); xAnimationLIF.next())
	{
		const Asura_String& xFilename = xAnimationLIF();
		Asura_Hash_ID uAnimationHash = Asura_GetHashID_IgnoreExtension(xFilename.GetString()+iDirectoryLength);

		bool bAlreadyExists;
		Asura_String* pxName = s_xAnimationFilenames.AddOrFindNode(uAnimationHash, &bAlreadyExists);
		Asura_Assert(pxName, "Error adding name to animation filename tree");
		if(pxName)
		{
			if(bAlreadyExists)
			{
				//Already an entry for that hash, complain if the name doesn't match
				Asura_Assertf(!pxName->Compare(xFilename), ("Found existing entry in animation tree for this hash value , but with a different name.\nExsiting name '%s'.\nNew name '%s'",pxName->GetString(), xFilename.GetString()));
			}
			else
			{
				//Got a new node , so update the name
				*pxName = xFilename;
			}
		}
	}
	s_bAnimationFilenamesValid = true;
}
/// End of function 'Tools_FacialAnimation_System::UpdateAnimationFilenames'. ///


#endif	//defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)


/// End of file. ///
