
///////////////////////////////////////////////////////////////////////////////
//
// File   : Tools_FacialAnimation_System.h
// Author : Andy Tate
// Date   : 8-May-2008
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __TOOLS_FACIALANIMATION_SYSTEM_H__
#define __TOOLS_FACIALANIMATION_SYSTEM_H__

#if defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura_HashTree.h"

#include "FacialAnimation/Tools_FacialAnimation_Face.h"
#include "FacialAnimation/Tools_FacialAnimation_FaceAnimation.h"
#include "FacialAnimation/Tools_FacialAnimation_FacePose.h"
#include "FacialAnimation/Tools_FacialAnimation_FaceType.h"


////////////////////////////////////////////////////////////////////////////////
// Forward class declarations...
////////////////////////////////////////////////////////////////////////////////

class Asura_TreeDialog_Item;
class Asura_TreeDialog_Tree;

class Tools_FacialAnimation_LipsyncConversion;


////////////////////////////////////////////////////////////////////////////////
// Class declaration...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Class        : Tools_FacialAnimation_System
// Author       : Andy Tate
// Date         : 8-May-2008
//
// Description  : Tools-class used to store the Tools-side data needed to 
//				  support the facial-animation system.
////////////////////////////////////////////////////////////////////////////////

class Tools_FacialAnimation_System
{
public:
	// Public functions...
	static void	Initialise(void);
	static void	Shutdown(void);

	// Type functions...
	static Tools_FacialAnimation_FaceType*	CreateType(const char* szName);
	static Tools_FacialAnimation_FaceType*	CreateType(const Asura_Hash_ID uHash, Asura_Chunk_Stream& xStream);

	static bool	DestroyType(const char* szName)	{ return(s_xTypeData.DeleteNodeByHash(Asura_GetHashID(szName))); }

	static Tools_FacialAnimation_FaceType*	FindType(const char* szName)		{ return(s_xTypeData.GetDataByName(szName)); }
	static Tools_FacialAnimation_FaceType*	FindType(const Asura_String& xName)	{ return(s_xTypeData.GetDataByHash(xName.GetHash())); }
	static Tools_FacialAnimation_FaceType*	FindType(const Asura_Hash_ID uHash)	{ return(s_xTypeData.GetDataByHash(uHash)); }

	static Tools_FacialAnimation_FaceType*	FindOrLoadType(const char* szName);
	static Tools_FacialAnimation_FaceType*	FindOrLoadType(const Asura_String& xName)	{ return(FindOrLoadType(xName.GetString())); }

	static bool	WriteTypeFile(const char* szFilename, const Tools_FacialAnimation_FaceType& xType);

	// Face functions...
	static Tools_FacialAnimation_Face*	CreateFace(const char* szName);
	static Tools_FacialAnimation_Face*	CreateFace(const Asura_Hash_ID uHash, Asura_Chunk_Stream& xStream);

	static bool	DestroyFace(const char* szName)	{ return(s_xFaceData.DeleteNodeByHash(Asura_GetHashID(szName))); }

	static Tools_FacialAnimation_Face*	FindFace(const char* szName)		{ return(s_xFaceData.GetDataByName(szName)); }
	static Tools_FacialAnimation_Face*	FindFace(const Asura_String& xName)	{ return(s_xFaceData.GetDataByHash(xName.GetHash())); }
	static Tools_FacialAnimation_Face*	FindFace(const Asura_Hash_ID uHash)	{ return(s_xFaceData.GetDataByHash(uHash)); }

	static Tools_FacialAnimation_Face*	FindOrLoadFace(const char* szName)			{ return FindOrLoadFace(Asura_GetHashID(szName)); }
	static Tools_FacialAnimation_Face*	FindOrLoadFace(const Asura_String& xName)	{ return(FindOrLoadFace(xName.GetHash())); }
	static Tools_FacialAnimation_Face*	FindOrLoadFace(const Asura_Hash_ID uHash);

	static bool	WriteFaceFile(const char* szFilename, const Tools_FacialAnimation_Face& xFace);

	// Animation functions...
	static Tools_FacialAnimation_FaceAnimation*	CreateAnimation(const char* szFaceTypeName, const char* szBaseFacePoseName, const char* szAnimationName, const char* szWAVFilename);
	static Tools_FacialAnimation_FaceAnimation*	CreateAnimation(const Asura_Hash_ID uHash, Asura_Chunk_Stream& xStream);

	static bool	DestroyAnimation(const char* szName)	{ return(s_xAnimationData.DeleteNodeByHash(Asura_GetHashID(szName))); }

	static Tools_FacialAnimation_FaceAnimation*	FindAnimation(const char* szName)			{ return(s_xAnimationData.GetDataByName(szName)); }
	static Tools_FacialAnimation_FaceAnimation*	FindAnimation(const Asura_String& xName)	{ return(s_xAnimationData.GetDataByHash(xName.GetHash())); }
	static Tools_FacialAnimation_FaceAnimation*	FindAnimation(const Asura_Hash_ID uHash)	{ return(s_xAnimationData.GetDataByHash(uHash)); }

	static Tools_FacialAnimation_FaceAnimation*	FindOrLoadAnimation(const char* szName)			{ return FindOrLoadAnimation(Asura_GetHashID(szName));}
	static Tools_FacialAnimation_FaceAnimation*	FindOrLoadAnimation(const Asura_String& xName)	{ return(FindOrLoadAnimation(xName.GetHash())); }
	static Tools_FacialAnimation_FaceAnimation*	FindOrLoadAnimation(const Asura_Hash_ID uHash);

	static bool	WriteAnimationFile(const char* szFilename, const Asura_Hash_ID uAnimationHash, const Tools_FacialAnimation_FaceAnimation& xToolsAnimation, const Asura_FacialAnimation_FaceAnimation& xAsuraAnimation);

	static int GetNumberOfAnimations() {return s_xAnimationData.GetNumberOfNodes();}
	static Tools_FacialAnimation_FaceAnimation* GetAnimationByIndex(int iIndex) {return s_xAnimationData.GetDataByIndex(iIndex);}

	// Pose functions...
	static Tools_FacialAnimation_FacePose*	CreatePose(const char* szName);
	static Tools_FacialAnimation_FacePose*	CreatePose(const Asura_Hash_ID uHash, Asura_Chunk_Stream& xStream);

	static bool	DestroyPose(const char* szName)	{ return(s_xPoseData.DeleteNodeByHash(Asura_GetHashID(szName))); }

	static Tools_FacialAnimation_FacePose*	FindPose(const char* szName)		{ return(s_xPoseData.GetDataByName(szName)); }
	static Tools_FacialAnimation_FacePose*	FindPose(const Asura_String& xName)	{ return(s_xPoseData.GetDataByHash(xName.GetHash())); }
	static Tools_FacialAnimation_FacePose*	FindPose(const Asura_Hash_ID uHash)	{ return(s_xPoseData.GetDataByHash(uHash)); }

	static Tools_FacialAnimation_FacePose*	FindOrLoadPose(const char* szName);
	static Tools_FacialAnimation_FacePose*	FindOrLoadPose(const Asura_String& xName)	{ return(FindOrLoadPose(xName.GetString())); }

	static bool	WritePoseFile(const char* szFilename, const Tools_FacialAnimation_FacePose& xPose);

	// Filename creation functions...
	static Asura_String	CreateTypeFilename(const char* szName, const bool bIncludePath)			{ return(CreateFilename(GetTypeDirectoryName(), szName, GetTypeFileExtension(), bIncludePath)); }
	static Asura_String	CreateTypeFilename(const Asura_String& xName, const bool bIncludePath)	{ return(CreateFilename(GetTypeDirectoryName(), xName.GetString(), GetTypeFileExtension(), bIncludePath)); }

	static Asura_String	CreateFaceFilename(const char* szName, const bool bIncludePath)			{ return(CreateFilename(GetFaceDirectoryName(), szName, GetFaceFileExtension(), bIncludePath)); }
	static Asura_String	CreateFaceFilename(const Asura_String& xName, const bool bIncludePath)	{ return(CreateFilename(GetFaceDirectoryName(), xName.GetString(), GetFaceFileExtension(), bIncludePath)); }

	static Asura_String	CreateAnimationFilename(const char* szName, const bool bIncludePath)		{ return(CreateFilename(GetAnimationDirectoryName(), szName, GetAnimationFileExtension(), bIncludePath)); }
	static Asura_String	CreateAnimationFilename(const Asura_String& xName, const bool bIncludePath)	{ return(CreateFilename(GetAnimationDirectoryName(), xName.GetString(), GetAnimationFileExtension(), bIncludePath)); }

	static Asura_String	CreatePoseFilename(const char* szName, const bool bIncludePath)			{ return(CreateFilename(GetPoseDirectoryName(), szName, GetPoseFileExtension(), bIncludePath)); }
	static Asura_String	CreatePoseFilename(const Asura_String& xName, const bool bIncludePath)	{ return(CreateFilename(GetPoseDirectoryName(), xName.GetString(), GetPoseFileExtension(), bIncludePath)); }

	// Accessor functions...
	static Asura_String	ConvertSoundFilenameToFaceAnimationName(const char* szFilename);

	static void										SetLipsyncConversion(Tools_FacialAnimation_LipsyncConversion* pxConversion)	{ s_pxLipsyncConversion = pxConversion; }
	static Tools_FacialAnimation_LipsyncConversion*	GetLipsyncConversion(void)													{ return(s_pxLipsyncConversion); }

	static Tools_FacialAnimation_Face*		FindOrLoadFaceFromTreeItem(const Asura_TreeDialog_Item& xItem, Asura_String& xPath);
	static Tools_FacialAnimation_FacePose*	FindOrLoadPoseFromTreeItem(const Asura_TreeDialog_Item& xItem, Asura_String& xPath);
	static Tools_FacialAnimation_FaceType*	FindOrLoadTypeFromTreeItem(const Asura_TreeDialog_Item& xItem, Asura_String& xPath);

	static const char*	GetTypeDirectoryName(void)	{ return("FacialAnimation\\Types"); }
	static const char*	GetTypeFileExtension(void)	{ return("FaceType"); }

	static const char*	GetFaceDirectoryName(void)	{ return("FacialAnimation\\Faces"); }
	static const char*	GetFaceFileExtension(void)	{ return("Face"); }

	static const char*	GetAnimationDirectoryName(void)	{ return("FacialAnimation\\Animations"); }
	static const char*	GetAnimationFileExtension(void)	{ return("FaceAnim"); }

	static const char*	GetPoseDirectoryName(void)	{ return("FacialAnimation\\Poses"); }
	static const char*	GetPoseFileExtension(void)	{ return("FacePose"); }



protected:
	// Protected enumerated types...
	enum
	{
		UNSET_ITEM		= 0x00000000,
		DIRECTORY_ITEM	= 0x00000001,
		FILE_ITEM		= 0x00000002,
	};

	// Protected variables...

	// Protected functions...
	static Asura_String	CreateFilename(const char* szDirectory, const char* szFilename, const char* szExtension, const bool bIncludePath)
	{
		Asura_String	xFilename(szFilename);

		// Ensure that the filename we're passed doesn't include an extension...
		xFilename.StripExtension();

		if(bIncludePath)
		{
			return(Asura_String(szDirectory) + "\\" + xFilename + "." + szExtension);
		}
		else
		{
			return(xFilename + "." + szExtension);
		}
	}


private:
	// Private typedefs...
	typedef Asura_TemplateHashTree<Tools_FacialAnimation_FaceType> TypeData;
	typedef Asura_TemplateHashTree<Tools_FacialAnimation_Face> FaceData;
	typedef Asura_TemplateHashTree<Tools_FacialAnimation_FaceAnimation> AnimationData;
	typedef Asura_TemplateHashTree<Tools_FacialAnimation_FacePose> PoseData;

	// Private variables...
	static TypeData									s_xTypeData;			// Data for all of the face-types we need to store.
	static FaceData									s_xFaceData;			// Data for all of the faces we need to store.
	static AnimationData							s_xAnimationData;		// Data for all of the animations for all of the faces.
	static PoseData									s_xPoseData;			// Data used for poses which are combined with automated .WAV conversion to produce an animation.

	static Tools_FacialAnimation_LipsyncConversion*	s_pxLipsyncConversion;	// A pointer to the class used to convert from .WAV files to facial-animations.

	static bool										s_bFaceFilenamesValid;
	static Asura_TemplateHashTree<Asura_String>		s_xFaceFilenames;			// Face file names found by doing a local/network search for files
	static bool										s_bAnimationFilenamesValid;
	static Asura_TemplateHashTree<Asura_String>		s_xAnimationFilenames;		// Animation file names found by doing a local/network search for files

	// Private functions...
	Tools_FacialAnimation_System(void);
	Tools_FacialAnimation_System(const Tools_FacialAnimation_System& xOther);
	~Tools_FacialAnimation_System();

	Tools_FacialAnimation_System&	operator=(const Tools_FacialAnimation_System& xOther);

	static void	UpdateFaceFilenames();		//Does a new search for face files
	static void	UpdateAnimationFilenames(); //Does a new search for animation files

};   // End of class 'Tools_FacialAnimation_System'.

#endif	//defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)

#endif   // ifndef __TOOLS_FACIALANIMATION_SYSTEM_H__


/// End of file. ///
