//
// Tools_CollBox_Object.cpp
//
// Created by Richard Rice - 26/03/2004
// 
// Copyright (C) 2004 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"
#include "Flags/Tools_CollBox_Object.h"

#include "Load&Save/Tools_SaveAsura.h"
#include "Objects/Tools_Object_System.h"
#include "Mesh/Asura_Mesh.h"
#include "Mesh/Asura_Mesh_CollMeshGenData.h"
#include "Misc/Tools_Render.h"
#include "Rendering/Asura_Module.h"

#include "MeshEdit/Tools_Mode_MeshEdit.h"


List<Tools_CollBox_Object*> Tools_CollBox_Object::s_xList;

Tools_CollBox_Object::Tools_CollBox_Object(int iModule,u_int uBoxID)
:Tools_Object()
{
	s_xList.add_entry(this);

	Mesh = 0;
	Position.Zero();
	Orientation.Identity();

	m_iModuleIndex = iModule;
	m_uBoxID = uBoxID;
	m_pxMesh = Asura_Module_System::GetMeshForModule(iModule);

	Asura_Mesh_CollBoxInfo* pxBox = GetCollisionBox();
	if(pxBox)
	{
		Orientation = pxBox->m_xOrient;
	}
	UpdateBoundingBox();
}

Tools_CollBox_Object::~Tools_CollBox_Object()
{
	s_xList.delete_entry(this);
}

void Tools_CollBox_Object::UpdatePositionInAsuraData(bool /*bUpdateLinkedObjects*/)
{
	Asura_Mesh_CollBoxInfo* pxBox = GetCollisionBox();
	if(!pxBox) return;

	pxBox->m_xOrient = Orientation;

	Tools_Mode_MeshEdit::InvalidateModuleCollisionMesh(m_iModuleIndex);
    Tools_SaveAsura::SetMainFileModified();

	UpdateBoundingBox();
}

struct Asura_Mesh_CollBoxInfo* Tools_CollBox_Object::GetCollisionBox()
{
	if(m_pxMesh && m_pxMesh->m_pxCollMeshGenData)
	{
		Asura_Mesh_CollMeshGenData* pxCollGen = m_pxMesh->m_pxCollMeshGenData;
		
		for(u_int i=0;i<pxCollGen->m_uNumberOfBoxes;i++)
		{
			if(m_uBoxID == pxCollGen->m_pxBoxes[i].m_uGroupID)
			{
				return &pxCollGen->m_pxBoxes[i];
			}
		}
	}
	return 0;
}

void Tools_CollBox_Object::OnSelection()
{
	Tools_Object_System::EnableAllControls(true);
}

void Tools_CollBox_Object::OnDeselection()
{
	Tools_Object_System::EnableAllControls(false);
}

void Tools_CollBox_Object::Render()
{
	if(m_bBoxValid)
	{
		Asura_Vector_3 xPosition;
		Asura_Module_System::GetPositionOfModule(m_iModuleIndex,xPosition);

		Tools_Render::RenderCube(m_xBox,xPosition,Orientation,0xff0000ff);
	}
}

void Tools_CollBox_Object::UpdateBoundingBox()
{
	m_xBox.Reset();
	m_bBoxValid = false;
	Asura_Mesh_CollMeshGenData* pxCollGenData = m_pxMesh->m_pxCollMeshGenData;

	if(!pxCollGenData || !pxCollGenData->m_pxPolygons) return;

	//search for all polygons that are part of the box , and extend the bounding box to include them
	for(int j=0;j<m_pxMesh->NumberOfPolygons;j++)
	{
		if(pxCollGenData->m_pxPolygons[j].m_usGroupID != (u_short) m_uBoxID)
		{
			continue;
		}

		//grab the verts , and add them to the box
		const Asura_Mesh_Polygon* pxPoly = &m_pxMesh->PolygonDataPtr[j];

		for(int k=0;k<pxPoly->NumberOfVertices();k++)
		{
			Asura_Vector_3 xVec = m_pxMesh->VertexDataPtr[pxPoly->VertexID[k]].Position;
			xVec = Orientation.ApplyTranspose(xVec);//vector needs to be rotated so as to be using the axes of the box

			m_xBox.Set(xVec);
		}
	}
	m_bBoxValid = true;

}

void Tools_CollBox_Object::DeleteAll()
{
	while(s_xList.size())
	{
		Tools_Object_System::DeleteObject(s_xList.first_entry());
	}
}
