#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef __TOOLS_MODE_FLAGS_H__
#define __TOOLS_MODE_FLAGS_H__

#include "Tools_Mode.h"
#include "Core/Asura_Handle.h"
#include "Misc/Tools_Polygon_Selection.h"
#include "Misc/Tools_Vertex_Selection.h"
#include "Misc/list_tem.hpp"

class Asura_Overlay;

///////////////////////////////////////////////////////////////////////////////////////
struct ModuleVertexPair;

struct FlagsUndoData
{
	enum UndoType
	{
		UndoType_PolyFlagsChanged,
		UndoType_VertFlagsChanged,
		UndoType_ProjectPolygonDataChanged,
		UndoType_ForceInt = 0xffffffff,
	};
	
	enum UndoType m_eUndoType;
	int m_iModuleIndex;
	union
	{
		int m_iVertexIndex;
		int m_iPolygonIndex;
	};
	union
	{
		int m_iFlags;
		u_int m_uFlags;
		u_int m_uData;
	};

};

class FlagsModeUndo : public FixedSizeUndo
{
public :
	FlagsModeUndo()
	:FixedSizeUndo(10000,sizeof(FlagsUndoData)){}

	void ProcessUndoData(void*);
	
	void LogPolyFlagsChanged(const PolygonID& polygon);
	void LogVertFlagsChanged(const ModuleVertexPair& xVertex);
	void LogProjectPolygonDataChanged(const PolygonID& polygon);
};

///////////////////////////////////////////////////////////////////////////////////////

class Tools_Mode_Flags : public Tools_Mode
{
public:
	Tools_Mode_Flags();
	virtual ~Tools_Mode_Flags();

	virtual void InitialiseMode();
	//initialise for external modes that want to create the overlay
	//doesn't add overlay options to menu or initialise polygon selection though
	static void InitialiseExMode(int iDefaultFlagValue);
	static void InitialiseForGI();

	virtual void Update();
	static void UpdateExMode( const bool bUpdateSelection,
							  const int iPolygonSelectionUpdateMode = PolygonSelectionUpdate_Standard,
							  const int iVertexSelectionUpdateMode = VertexSelectionUpdate_Standard );
	static void UpdateForGI();

	virtual void RenderWorld_PostToneMap();
	virtual void RenderUI();
	static void RenderPolyFlagText(float& fPrintX,float& fPrintY);
	static void RenderVertFlagText(float& fPrintX,float& fPrintY);
	static void RenderProjectPolygonDataText(float& fPrintX,float& fPrintY);
	
	virtual void ShutdownMode();
	static void ShutdownExMode();
	static void ShutdownForGI();

 	virtual bool IsModified();

	static void ApplyFlag(void);
	static void ApplyPolyFlag(void);
	static void ApplyVertFlag(void);
	static void ApplyProjectPolygonData(void);

	static void RemoveFlag(void);
	static void RemovePolyFlag(void);
	static void RemoveVertFlag(void);
	static void RemoveProjectPolygonData(void);

	static void ClearFlags();
	static void ClearPolyFlags();
	static void ClearVertFlags();
	static void ClearProjectPolygonData();

    static void RegenerateStepFlags( void );
    static void SetSelectPhysicsWalkableWithFloorFlag( bool bSelect ) { s_bSelectPhysicsWalkableWithFloorFlag = bSelect; }

	//logs current state of selection for undo
	static void PrepareUndo_PolyFlag();
	static void PrepareUndo_VertFlag();
	static void PrepareUndo_ProjectPolygonData();
	static void Undo();
	static void Redo();

	//gives each polygon a colour coded outline
    static void HighlightPolyFlags();
    static void HighlightVertFlags();
    static void HighlightProjectPolygonData();

	//for accessing polygon flag names outside of the mode
	static int GetNumberOfPolygonFlags();
	static int GetPolygonFlagValue(int iIndex);
	static const char* GetPolygonFlagName(int iIndex);
	static const char* GetPolygonFlagHelpText(int iIndex);

	//and similarly for the project specific flags (for convenience... just calls Tools_System functions)
	static int GetNumberOfProjectPolygonFlags();
	static int GetProjectPolygonFlagValue(int iIndex);
	static const char* GetProjectPolygonFlagName(int iIndex);
	static const char* GetProjectPolygonFlagHelpText(int iIndex);

	//for accessing vertex flag names outside of the mode
	static int GetNumberOfVertexFlags();
	static u_int GetVertexFlagValue(int iIndex);
	static const char* GetVertexFlagName(int iIndex);
	static const char* GetVertexFlagHelpText(int iIndex);

	//and similarly for the project specific flags (for convenience... just calls Tools_System functions)
	static int GetNumberOfProjectVertexFlags();
	static u_int GetProjectVertexFlagValue(int iIndex);
	static const char* GetProjectVertexFlagName(int iIndex);
	static const char* GetProjectVertexFlagHelpText(int iIndex);

	//Virtual functions for editing the project specific polygon data.
	//Get the number of different values being packed in
	virtual int GetNumberOfProjectPolygonDataValues() const {return 0;};
	//Get the name for the iIndex'th value
	virtual const char* GetProjectPolygonDataName(int iIndex) const {return 0;};
	//Get help text for the iIndex'th value
	virtual const char* GetProjectPolygonDataHelpText(int iIndex) const {return 0;};
	//Get the maximum allowed for the iIndex'th value
	virtual u_int GetProjectPolygonDataMax(int iIndex) const {return 0;};
	//Extracts the iIndex'th value from uData
	virtual u_int GetValueFromProjectPolygonData(int iIndex, u_int uData) const{return 0;};
	//Sets the iIndex'th value in uData and returns the result
	virtual u_int SetValueInProjectPolygonData(int iIndex,u_int uData,u_int uValue) const {return uData;};

	// (SB - 12/12/06) Virtual functions for project specific keyboard handling and addition of buttons to flags panes
	virtual void ProjectHandleKeyboard(void) { return; };
	virtual void ProjectAddItemsToFlagsPaneEarly(Asura_Overlay* const pxOverlay, Asura_Vector_2& xOffset) { return; };
	virtual void ProjectAddItemsToFlagsPane(Asura_Overlay* const pxOverlay, Asura_Vector_2& xOffset) { return; };
	virtual void ProjectAddItemsToVertexFlagsPaneEarly(Asura_Overlay* const pxOverlay, Asura_Vector_2& xOffset) { return; };
	virtual void ProjectAddItemsToVertexFlagsPane(Asura_Overlay* const pxOverlay, Asura_Vector_2& xOffset) { return; };
	virtual void ProjectAddItemsToPolyFlagsPaneEarly(Asura_Overlay* const pxOverlay, Asura_Vector_2& xOffset, int iIndex) { return; };
	virtual void ProjectAddItemsToPolyFlagsPane(Asura_Overlay* const pxOverlay, Asura_Vector_2& xOffset, int iIndex) { return; };

	static void ToggleFlagsOverlay(void);
	static bool FlagsOverlayIsOn(void);
	static void CreateFlagsOverlay(void);

	static int GetSelectedFlags() {return s_iSelectedPolyFlags;};

	enum SubMode
	{
		SUBMODE_POLYGON_FLAGS,
		SUBMODE_VERTEX_FLAGS,
		SUBMODE_PROJECTDATA,
		SUBMODE_ELEMENTS
	};

	static SubMode GetSubMode() {return s_eSubMode;}


#ifdef ASURA_SUPPORT_SKIN_ELEMENTS
	static void ApplyElement(void);
	static void UpdateElementVisibilities(void);
	static void ToggleElementVisibility(int iElement, int iUnused);
	static void ToggleElementsOverlay();
	static void ChangeVisibleElementSelection(int iSelectionType);
#endif


	virtual void HandleMeshChange(int iModuleIndex);

	//clears out all undos/redos
	static void ClearUndo();


protected:
	static void TransferModeData(int mode);

	//build combined project/common flag list
	static void BuildPolygonFlagArrays();
	static void BuildVertexFlagArrays();
	static void BuildProjectPolygonDataArrays();
	//and delete said list
	static void DeletePolygonFlagArrays();
	static void DeleteVertexFlagArrays();
	static void DeleteProjectPolygonDataArrays();

	static void HandleKeyboard(void);

	static void SetPolyFlag(int iFlag,bool bAdd);
	static void SetVertFlag(u_int uFlag,bool bAdd);
	static void SetProjectPolygonData(int iIndex,u_int uValue);


	static void CollisionMesh_ReplaceByBox();
	static void CollisionMesh_ReplaceByBox(int iModule,const List<int>& xPolyList);

	static void CollisionMesh_ResetPolygons();
	static void CollisionMesh_ResetPolygons(int iModule,const List<int>& xPolyList);	

	//for flag list control
	static bool IsPolyFlagSelected(int iIndex,int iUnused);
	static void TogglePolyFlagSelection(int iIndex,int iUnused);

	static bool IsVertFlagSelected(int iIndex,int iUnused);
	static void ToggleVertFlagSelection(int iIndex,int iUnused);

	static void DetermineCurrentSubMode();
	static void EnsureProjectPolygonDataIsInRange();
		
	static bool IsInFlagSubMode();


	///////////////////////////
	//data
	static Asura_Handle MenuHandle;

	static Asura_Vector_2 FlagsOverlayPosition;
	static bool	FlagsOverlayOpen;

	static Asura_Vector_2 s_xElementsOverlayPos;

	static int s_iPolyFlagSelectionIndex;//index of flag being edited
	static int s_iSelectedPolyFlags;

	static int s_iVertFlagSelectionIndex;//index of flag being edited
	static u_int s_uSelectedVertFlags;

	static int s_iProjectPolygonDataSelectionIndex;
	static u_int s_uProjectPolygonDataValue;

	static bool s_bHighlightFlag;
	static bool s_bAllowMultipleFlagSelection;
	static bool s_bHighlightOnlyIfHaveAllFlags;
    static bool s_bSelectPhysicsWalkableWithFloorFlag;
	static bool s_bShowHiddenPolysWithFlag;
	
	static int s_iNumberOfPolygonFlags;
	static char** s_pszPolygonFlagNames;
	static char** s_pszPolygonFlagHelp;
	static int* s_piPolygonFlagValues;

	static int s_iNumberOfVertexFlags;
	static char** s_pszVertexFlagNames;
	static char** s_pszVertexFlagHelp;
	static u_int* s_puVertexFlagValues;

	static int s_iNumberOfProjectPolygonDataValues;
	static char** s_pszProjectPolygonDataNames;
	static char** s_pszProjectPolygonDataHelp;


	static u_int s_uSelectedTab;
	static SubMode s_eSubMode;

	static SubMode s_aeTabToSubMode[4];

	static FlagsModeUndo FlagsUndo;

	static Tools_Mode_Flags* s_pxThis;

#ifdef ASURA_SUPPORT_SKIN_ELEMENTS
	static int s_iElementSelectionIndex;
#endif

};

#endif//__TOOLS_MODE_FLAGS_H__
