//
// Tools_GUIMenu_AnimEdit.cpp
//
// Created by Richard Rice - 12/04/2007
// 
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

#ifdef ASURA_USE_GUI_MENU_SYSTEM

#include "GUIMenu/Tools_GUIMenu_AnimEdit.h"

#include "Core/Asura_Timers.h"
#include "Core/Asura_UserInput.h"
#include "GUIMenu/Asura_GUIMenu_Animator.h"
#include "Interface/Asura_Overlay.h"
#include "Interface/Asura_Overlay_System.h"
#include "Interface/Asura_Dialog.h"
#include "Mesh/Asura_Mesh.h"

#include "Tools_Overlays.h"
#include "GUIMenu/Tools_GUIMenu_Root.h"
#include "GUIMenu/Tools_GUIMenu_System.h"
#include "GUIMenu/Tools_GUIMenu_Element_Image.h"
#include "Interface/Tools_ColourPicker.h"
#include "Load&Save/Tools_SaveAsura.h"
#include "Misc/Tools_Modal_Overlay.h"
#include "Texturing/Tools_PickTexture.h"
#include "Texturing/Tools_Texturing_Texture.h"
#include "Texturing/Tools_Texture_Management.h"


bool Tools_GUIMenu_AnimEdit::s_bOverlayOpen = false;
u_int Tools_GUIMenu_AnimEdit::s_uOverlayMinimised = 0;
Asura_Vector_2 Tools_GUIMenu_AnimEdit::s_xOverlayPosition(20,20);
Asura_Vector_2 Tools_GUIMenu_AnimEdit::s_xEditSeqOverlayPosition(40,20);

int Tools_GUIMenu_AnimEdit::s_iSelectedSeq = 0;
int Tools_GUIMenu_AnimEdit::s_iNumSeqNames = 0;
const char** Tools_GUIMenu_AnimEdit::s_pszSeqNames = 0;

Asura_GUIMenu_Animator_Sequence* Tools_GUIMenu_AnimEdit::s_pxEditedSeq = 0;

static const char* g_pszNoAnimString = "*None*";

int Tools_GUIMenu_AnimEdit::s_iSelectedFrame = 0;

bool Tools_GUIMenu_AnimEdit::s_bNeedUpdateFromSeq = false;
bool Tools_GUIMenu_AnimEdit::s_bChanged = false;
bool Tools_GUIMenu_AnimEdit::s_bDurationChanged = false;

float Tools_GUIMenu_AnimEdit::s_fSeqDuration;
u_int Tools_GUIMenu_AnimEdit::s_uSeqStateFlags;
bool Tools_GUIMenu_AnimEdit::s_bSeqLoop;
bool Tools_GUIMenu_AnimEdit::s_bSeqLoopBackAndForth;
bool Tools_GUIMenu_AnimEdit::s_bSeqPlayToEnd;
bool Tools_GUIMenu_AnimEdit::s_bSeqInterpolate;
bool Tools_GUIMenu_AnimEdit::s_bSeqRandom;
bool Tools_GUIMenu_AnimEdit::s_bGlobalTime;
bool Tools_GUIMenu_AnimEdit::s_bConsoleVarSmooth;

int Tools_GUIMenu_AnimEdit::s_iEaseType;

float Tools_GUIMenu_AnimEdit::s_fFrameTime;
bool Tools_GUIMenu_AnimEdit::s_bFrameTimeInSecs = false;

float Tools_GUIMenu_AnimEdit::s_fSmoothFrameTime;

Asura_Vector_2 Tools_GUIMenu_AnimEdit::s_xFrameData_Vector2;
float Tools_GUIMenu_AnimEdit::s_fFrameData;
Asura_GUIMenu_UV_Set Tools_GUIMenu_AnimEdit::s_xFrameData_UV;
Asura_RGBA Tools_GUIMenu_AnimEdit::s_xFrameData_RGBA;

Asura_String Tools_GUIMenu_AnimEdit::s_xNumFrameText;

int Tools_GUIMenu_AnimEdit::s_iTextureID = ASURA_TEXTURE_NONE;
int Tools_GUIMenu_AnimEdit::s_iTextureWidth = 512;
int Tools_GUIMenu_AnimEdit::s_iTextureHeight = 512;
bool Tools_GUIMenu_AnimEdit::s_bShowInPixels = true;
bool Tools_GUIMenu_AnimEdit::s_bUpdateEditSeqOverlay = false;
Tools_UV_Editor* Tools_GUIMenu_AnimEdit::s_pxUVEdit = 0;

bool Tools_GUIMenu_AnimEdit::s_bUseTimerOverride = false;
float Tools_GUIMenu_AnimEdit::s_fTimerOverride = 0.0f;

static const Asura_Hash_ID g_uDialog_LeftUV = 0xBE0AC8E8; //Asura_GetHashID("LeftUV")
static const Asura_Hash_ID g_uDialog_RightUV = 0x478FB9BD; //Asura_GetHashID("RightUV")
static const Asura_Hash_ID g_uDialog_TopUV =  0x0696CEB6; //Asura_GetHashID("TopUV")
static const Asura_Hash_ID g_uDialog_BottomUV = 0x808FCA0C; //Asura_GetHashID("BottomUV")

static const Asura_Char* g_apcAnimatorsEaseNames[] = 
{
	"None",
	"In/Out",
	"Out/In",
	"In",
	"Out",
};

void Tools_GUIMenu_AnimEdit::Initialise()
{
	Tools_GUIMenu_AnimEdit::CreateOverlay();
}

void Tools_GUIMenu_AnimEdit::Shutdown()
{
	Tools_GUIMenu_AnimEdit::CloseOverlays();
	s_pxEditedSeq = 0;
	FinishUVEdit();
}

void Tools_GUIMenu_AnimEdit::Update()
{
	if(s_bOverlayOpen && !Asura_Overlay_System::DoesOverlayExist(TOOLS_OVERLAY_GUIMENU_ANIMATOR))
	{
		//Should get here if the overlay was closed using the 'X'.
		//So use ToggleOverlays() to make sure everything else is cleared up.
		ToggleOverlay();
	}
	
	if(s_pxUVEdit) UpdateUVEdit();
	
	if(s_bNeedUpdateFromSeq)
	{
		UpdateOverlayVariablesFromSeq();
	}
	else if(s_bChanged || s_bDurationChanged)
	{
		UpdateSeqFromOverlayVariables();
		SetModified();
	}
	s_bNeedUpdateFromSeq = false;
	s_bDurationChanged = false;
	s_bChanged = false;

	if(s_bUpdateEditSeqOverlay)
	{
		UpdateEditSeqOverlay();
		s_bUpdateEditSeqOverlay = false;
	}

	//update preview timer
	if(s_pxEditedSeq && Asura_Overlay_System::DoesOverlayExist(TOOLS_OVERLAY_GUIMENU_ANIMATORSEQUENCE))
	{
		if(s_bUseTimerOverride)
		{
			s_fTimerOverride += Asura_Timers::GetGameTimeSlice();	
		}
	}
	else
	{
		s_pxEditedSeq = 0;
		FinishUVEdit();
	}
}

void Tools_GUIMenu_AnimEdit::Render()
{
	if(s_pxUVEdit) s_pxUVEdit->Render();	
}

void Tools_GUIMenu_AnimEdit::ToggleOverlay(void)
{
	s_bOverlayOpen = !s_bOverlayOpen;

	if (s_bOverlayOpen)
	{
		CreateOverlay();
	}
	else
	{
		CloseOverlays();
	}
}
bool Tools_GUIMenu_AnimEdit::OverlayIsOn(void)
{
	return s_bOverlayOpen;
}

void Tools_GUIMenu_AnimEdit::CreateOverlay(void)
{
	if(!s_bOverlayOpen) return;

	Asura_Overlay* pxOverlay;
	pxOverlay = new Asura_Overlay("Animators",false,&s_xOverlayPosition,&s_uOverlayMinimised);
	
	Asura_Dialog* pxDialog;
	Asura_Vector_2 xOffset = Asura_Vector_2(1.0f,3.0f);


	//add heading for anim list
	pxDialog = new Asura_Dialog("Animators :",xOffset);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::SmallSpacing();

	//add list of anim names
	BuildSeqNames();
	if(s_iSelectedSeq>=s_iNumSeqNames) s_iSelectedSeq = s_iNumSeqNames-1;
    	
	int iNumLines = Asura_Maths::Min(15,s_iNumSeqNames);
	pxDialog = new Asura_List_Dialog(xOffset,s_pszSeqNames,&s_iSelectedSeq,iNumLines,s_iNumSeqNames,0);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y = pxDialog->m_xMaxCoord.y + 1.0f;


	pxDialog = new Asura_Function_Dialog("Create",xOffset,CreateSeq);
	pxDialog->SetHelpText("Creates a new menu animator");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Duplicate",xOffset,DuplicateSeq);
	pxDialog->SetHelpText("Creates a copy of the selected animator.");
	pxDialog->SetIsEnabledFunction(IsSeqSelected);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Delete",xOffset,DeleteSeq);
	pxDialog->SetHelpText("Deletes the currently selected animator");
	pxDialog->SetIsEnabledFunction(IsSeqSelected);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Rename",xOffset,RenameSeq);
	pxDialog->SetHelpText("Renames the currently selected animator");
	pxDialog->SetIsEnabledFunction(IsSeqSelected);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();


	pxDialog = new Asura_Function_Dialog("Edit",xOffset,EditSeq);
	pxDialog->SetHelpText("Edits the currently selected animator");
	pxDialog->SetIsEnabledFunction(IsSeqSelected);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();

	pxDialog = new Asura_Function_Dialog("Apply",xOffset,ApplySeq);
	pxDialog->SetIsEnabledFunction(CanApplySeq);
	pxDialog->SetHelpText("Applies the current animator to the selected menu item");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();


//	pxDialog = new Asura_Function_Dialog("Restart All",xOffset,Asura_GUIMenu_AnimEdit_System::ResetAnimationControllers);
//	pxDialog->SetHelpText("Resets the timers on all texture animations to zero.\n");
//	pxOverlay->AddDialog(pxDialog);
//	xOffset.y += Asura_Dialog::DefaultSpacing();

		
	pxOverlay->m_uID = TOOLS_OVERLAY_GUIMENU_ANIMATOR;

	Asura_Overlay_System::SubmitNewOverlay(pxOverlay);
}


void Tools_GUIMenu_AnimEdit::CloseOverlays()
{
	Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_GUIMENU_ANIMATOR);
	CloseEditSeqOverlay();
	DeleteSeqNames();
	s_pxEditedSeq = 0;
}


static int SortSeqNames(const void* p1,const void* p2)
{
	return stricmp(*(const char**)p1,*(const char**)p2);
}

void Tools_GUIMenu_AnimEdit::BuildSeqNames()
{

	//delete the old strings
	DeleteSeqNames();

	
	if(Asura_GUIMenu_Animator_Storage::GetNumberOfSequences())	
	{
		//get all the anim names
		s_iNumSeqNames = Asura_GUIMenu_Animator_Storage::GetNumberOfSequences();
		s_pszSeqNames = new const char*[s_iNumSeqNames];

		for(int i=0;i<s_iNumSeqNames;i++)
		{
			Asura_GUIMenu_Animator_Sequence* pxAnim = Asura_GUIMenu_Animator_Storage::GetSequenceByIndex(i);
			s_pszSeqNames[i] = pxAnim->GetName();
		}
		
		//and sort them
		qsort(s_pszSeqNames,s_iNumSeqNames,sizeof(char*),SortSeqNames);

	}
	else
	{
		//no animations for the list
		s_iNumSeqNames = 1;
		s_pszSeqNames = new const char*[1];
		s_pszSeqNames[0] = g_pszNoAnimString;
	}
}

void Tools_GUIMenu_AnimEdit::DeleteSeqNames()
{
	delete [] s_pszSeqNames;
	s_pszSeqNames = 0;
	s_iNumSeqNames = 0;
}

void Tools_GUIMenu_AnimEdit::CreateSeq()
{
	Asura_Assert(s_bOverlayOpen,"");


	//ask for a name
	GetString_Overlay xOverlay("Name for animator sequence:","");

	if(!xOverlay.DoModal()) return;

	const char* szName = xOverlay.GetString();
	if(!*szName) return;

	//make sure the sequence doesn't already exist
	Asura_Hash_ID uHashID = Asura_GetHashID(szName);
	if(Asura_GUIMenu_Animator_Storage::GetSequenceByHash(uHashID))
	{
		ShowMessageOverlay("An animator sequence already exists with the same name/hash ID.");
		return;
	}

	//ask for the type of animator to create
	ListSelection_Overlay xListOverlay("Select animator type");
	// start at 1 because 0 is the invalid value.
	for(int i = 1; i < ASURA_GUIMENU_NUM_MODIFY_TYPES; i++)
	{
		ASURA_GUIMENU_MODIFY_TYPE eModifyType = static_cast<ASURA_GUIMENU_MODIFY_TYPE>(i);
		if (!IsModifierTypeSupported(eModifyType)) continue;

		const char* szName = Asura_GUIMenu_Modifier::GetModifyTypeName(eModifyType);
		if(szName)
		{
			xListOverlay.AddItem(szName, i);
		}
	}

	if(!xListOverlay.DoModal()) return;
	ASURA_GUIMENU_MODIFY_TYPE eSelectedModifyType = static_cast<ASURA_GUIMENU_MODIFY_TYPE>(xListOverlay.GetSelectionData());


	//create a default sequence
	Asura_GUIMenu_Animator_Sequence* pxSeq = new Asura_GUIMenu_Animator_Sequence(szName);
	pxSeq->Initialise(eSelectedModifyType);

	//re-create the overlay to update the list
	CloseOverlays();
	CreateOverlay();
		
	//Select the newly created anim from the list
	s_iSelectedSeq = GetListIndexFromName(szName);
	Asura_Assert(s_iSelectedSeq>=0,"Failed to find anim in list?");

	SetModified();

	//edit the newly created anim
	EditSeq();
}

void Tools_GUIMenu_AnimEdit::DeleteSeq()
{
	Asura_Assert(s_bOverlayOpen,"");
	
	if(!Asura_GUIMenu_Animator_Storage::GetNumberOfSequences()) return;

	const char* szAnimName = s_pszSeqNames[s_iSelectedSeq];

	//confirm deletion
	if(!ShowYesNoOverlay("Delete this animation?\n'%s'",szAnimName))
	{
		return;
	}

	//Remove references to the deleted sequence
	Tools_Mode_GUIMenu::HandleDeleteAnimator(szAnimName);

	//and delete
	Asura_GUIMenu_Animator_Sequence* pxSeq = Asura_GUIMenu_Animator_Storage::GetSequenceByHash(Asura_GetHashID(szAnimName));
	delete pxSeq;

	//update the overlays
	CloseOverlays();
	CreateOverlay();

	SetModified();
}

void Tools_GUIMenu_AnimEdit::DuplicateSeq()
{
#if 1
	Asura_Assert(s_bOverlayOpen,"");
	
	if(!Asura_GUIMenu_Animator_Storage::GetNumberOfSequences()) return;

	//get the anim being copied from
	const char* szSelectedName = s_pszSeqNames[s_iSelectedSeq];
	Asura_Hash_ID uSelectedHash = Asura_GetHashID(szSelectedName);
	Asura_GUIMenu_Animator_Sequence* pxSelectedSeq = Asura_GUIMenu_Animator_Storage::GetSequenceByHash(uSelectedHash);
	if(!pxSelectedSeq) return;

	//get name for copy
	GetString_Overlay xOverlay("Enter name for copy :","");

	if(!xOverlay.DoModal()) return;

	const char* szName = xOverlay.GetString();
	if(!*szName) return;

	//make sure the sequence doesn't already exist
	Asura_Hash_ID uHashID = Asura_GetHashID(szName);
	if(Asura_GUIMenu_Animator_Storage::GetSequenceByHash(uHashID))
	{
		ShowMessageOverlay("An animator sequence already exists with the same name/hash ID.");
		return;
	}
	//create a default sequence
	Asura_GUIMenu_Animator_Sequence* pxSeq = new Asura_GUIMenu_Animator_Sequence(szName);
	pxSeq->Copy(pxSelectedSeq);

	//re-create the overlay to update the list
	CloseOverlays();
	CreateOverlay();
		
	//Select the newly created anim from the list
	s_iSelectedSeq = GetListIndexFromName(szName);
	Asura_Assert(s_iSelectedSeq>=0,"Failed to find anim in list?");

	SetModified();

	//edit the newly created anim
	EditSeq();
#endif
	
}

bool Tools_GUIMenu_AnimEdit::IsSeqSelected()
{
	return Asura_GUIMenu_Animator_Storage::GetNumberOfSequences() > 0;
}

void Tools_GUIMenu_AnimEdit::ApplySeq()
{
	ApplySeq(false);
}

bool Tools_GUIMenu_AnimEdit::CanApplySeq()
{
	return ApplySeq(true);
}

bool Tools_GUIMenu_AnimEdit::ApplySeq(bool bJustCheckIfAllowed)
{
	//need a selected sequence
	Asura_Assert(s_bOverlayOpen,"");
	if(!Asura_GUIMenu_Animator_Storage::GetNumberOfSequences()) return false;

	const char* szAnimName = s_pszSeqNames[s_iSelectedSeq];
	Asura_GUIMenu_Animator_Sequence* pxSeq = Asura_GUIMenu_Animator_Storage::GetSequenceByHash(Asura_GetHashID(szAnimName));


	//and a selected menu item
	Tools_GUIMenu_Root* pxSelectedItem = Tools_Mode_GUIMenu::GetSelectedItem();
	if(!pxSelectedItem) return false;

	
	//make sure the sequence is allowed on the selected menu item
	if(!pxSelectedItem->IsAnimatorAllowed(pxSeq))	 return false;

	if(!bJustCheckIfAllowed)
	{
		pxSelectedItem->AddAnimator(pxSeq->GetName());
	}
	return true;
}




void Tools_GUIMenu_AnimEdit::EditSeq()
{
	Asura_Assert(s_bOverlayOpen,"");
	if(!Asura_GUIMenu_Animator_Storage::GetNumberOfSequences()) return;

	const char* szAnimName = s_pszSeqNames[s_iSelectedSeq];
	Asura_GUIMenu_Animator_Sequence* pxSeq = Asura_GUIMenu_Animator_Storage::GetSequenceByHash(Asura_GetHashID(szAnimName));

	if(pxSeq) EditSeq(pxSeq);
}

void Tools_GUIMenu_AnimEdit::EditSeq(Asura_GUIMenu_Animator_Sequence* pxSeq)
{
	CloseEditSeqOverlay();

	Asura_Assert(pxSeq,"");

	if(pxSeq != s_pxEditedSeq)
	{
		//and reset the edited frame
		s_iSelectedFrame = 0;
		s_pxEditedSeq = pxSeq;
	}
	//reset the preview timer
	s_fTimerOverride = 0.0f;

	UpdateOverlayVariablesFromSeq();

	char szTitle[30];
	strncpy(szTitle,pxSeq->GetName(),20);
	if(strlen(pxSeq->GetName())>=20)
	{
		strcpy(&szTitle[17],"...");
	}
	
	Asura_Overlay* pxOverlay;
	pxOverlay = new Asura_Overlay(szTitle,false,&s_xEditSeqOverlayPosition);

	Asura_Dialog* pxDialog;
	Asura_Vector_2 xOffset = Asura_Vector_2(1.0f,3.0f);

	//start of tabs
	Asura_Tabs_Dialog* pxTabsDialog = new Asura_Tabs_Dialog(xOffset, 0, 0);
	pxOverlay->AddDialog(pxTabsDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();
	pxOverlay->SetCurrentParentDialog(pxTabsDialog);
	
	float fTabsStartY = xOffset.y;
	float fTabsEndY = xOffset.y;


	//start of main tab
	pxTabsDialog->AddTab("Main");


	pxDialog = new Asura_Numerical_Dialog("Duration",xOffset,&s_fSeqDuration,0.1f,9999.0f,1.0f,"%.3f",&s_bDurationChanged);
	pxDialog->SetHelpText("Length of one loop of the sequence in seconds.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	
	pxDialog = new Asura_Tickbox_Dialog("Looped",xOffset,&s_bSeqLoop,&s_bChanged);	
	pxDialog->SetHelpText("Sequence loops");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Tickbox_Dialog("Loop Back And Forth",xOffset,&s_bSeqLoopBackAndForth,&s_bChanged);	
	pxDialog->SetHelpText("Sequence reverses upon reaching the end and goes forward again upon returning to the start.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Tickbox_Dialog("Play to end",xOffset,&s_bSeqPlayToEnd,&s_bChanged);	
	pxDialog->SetHelpText("Once started the sequence will play to its end regardless of any state changes.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Tickbox_Dialog("Interpolate",xOffset,&s_bSeqInterpolate,&s_bChanged);	
	pxDialog->SetHelpText("Interpolate between frames");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_DropDown_List_Dialog( xOffset,
		&g_apcAnimatorsEaseNames[0],
		&s_iEaseType,
		ASURA_GUIMENU_ANIMATOR_MAX_EASE_TYPES, 
		ASURA_GUIMENU_ANIMATOR_MAX_EASE_TYPES,
		NULL, &s_bChanged, "Ease Type");
	pxDialog->SetIsEnabledFunction(IsEnabled_EaseInOut);
	pxOverlay->AddDialog( pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Tickbox_Dialog("Global Timer",xOffset,&s_bGlobalTime,&s_bChanged);	
	pxDialog->SetHelpText("Use a global time to keep all animations in sync.");
	pxDialog->SetIsEnabledFunction(IsEnabled_GlobalTimer);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();
	
	pxDialog = new Asura_Dialog("Frame selection :",xOffset);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();


	pxDialog = new Asura_Numerical_Dialog("Frame",xOffset,&s_iSelectedFrame,-1,999,1,"%d",&s_bNeedUpdateFromSeq);
	pxDialog->SetHelpText("Selects the frame to edit");
	pxOverlay->AddDialog(pxDialog);

	xOffset.x = pxDialog->m_xMaxCoord.x;
	pxDialog = new Asura_Dynamic_Dialog(&s_xNumFrameText,xOffset);
	pxOverlay->AddDialog(pxDialog);
	xOffset.x = 1.0f;
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Add After",xOffset,CreateFrame);
	pxDialog->SetHelpText("Add a new frame after the current one.");
	pxOverlay->AddDialog(pxDialog);

	xOffset.x = pxDialog->m_xMaxCoord.x+1.0f;
	pxDialog = new Asura_Function_Dialog("Remove",xOffset,DeleteFrame);
	pxDialog->SetHelpText("Delete the current frame");
	pxDialog->SetIsEnabledFunction(IsEnabled_DeleteFrame);
	pxOverlay->AddDialog(pxDialog);
	xOffset.x = 1.0f;
	xOffset.y += Asura_Dialog::LargeSpacing();

	pxDialog = new Asura_Dialog("Frame data :",xOffset);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	//Add controls for editing the frame data for this animator type
	AddAnimatorFrameControlsToOverlay(pxOverlay,xOffset,pxSeq);
	xOffset.y += Asura_Dialog::LargeSpacing() - Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Evenly space frames",xOffset,EvenlySpaceFrames);
	pxDialog->SetHelpText("Automatically space out the timing of the frames of animation.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.x = 1.0f;
	xOffset.y += Asura_Dialog::LargeSpacing();

	pxDialog = new Asura_Tickbox_Dialog("Time In Secs",xOffset,&s_bFrameTimeInSecs,&s_bNeedUpdateFromSeq);
	pxDialog->SetHelpText("If on then frame times are measured in seconds, otherwise they are measured in the range 0 to 1");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();


	pxDialog = new Asura_Numerical_Dialog("Time",xOffset,&s_fFrameTime,0.0f,9999.0f,1,"%.3f",&s_bChanged);
	pxDialog->SetIsEnabledFunction(IsEnabled_FrameTime);
	pxDialog->SetHelpText("This is either measured as a value between 0 and 1 , or as a time in seconds depending on tickbox.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	//end of main tab
	if(xOffset.y > fTabsEndY) fTabsEndY = xOffset.y;

	//start of states tab
	pxTabsDialog->AddTab("States");
	xOffset.y = fTabsStartY;

	for(u_int i=0; i< Tools_GUIMenu_Root::GetNumMenuItemStates() ; i++)
	{
		pxDialog = new Asura_FlagTickbox_Dialog(Tools_GUIMenu_Root::GetMenuItemStateName(i),xOffset,&s_uSeqStateFlags,Tools_GUIMenu_Root::GetMenuItemStateValue(i),&s_bChanged);
		pxDialog->SetHelpText("Indicates whether the sequence applies during this state.");
		pxOverlay->AddDialog(pxDialog);
		xOffset.y += Asura_Dialog::DefaultSpacing();
	}
	
	//end of states tab
	if(xOffset.y > fTabsEndY) fTabsEndY = xOffset.y;

	//start of links tab
	pxTabsDialog->AddTab("Links");
	xOffset.y = fTabsStartY;

	// Add animation sequence console var to links tab
	u_int uConsoleVarType = Asura_CommandConsole_VarRepository::Var::TYPE_FLOAT;

	pxSeq->m_xConsoleVar.AddOverlayControls( pxOverlay, 
		xOffset, "Console Var", "The console variable that provides the time used by the animation", uConsoleVarType, &s_bChanged );

	// Add a tickbox to smooth out the console var
	pxDialog = new Asura_Tickbox_Dialog("Smooth",xOffset,&s_bConsoleVarSmooth,&s_bChanged);	
	pxDialog->SetIsEnabledFunction(IsEnabled_SmoothTickBox);
	pxDialog->SetHelpText("Smooth Console Var over time");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	// Smooth time for the console var
	pxDialog = new Asura_Numerical_Dialog("Smooth Time",xOffset,&s_fSmoothFrameTime,0.0f,9999.0f,1,"%.3f",&s_bChanged);
	pxDialog->SetIsEnabledFunction(IsEnabled_SmoothFrameTime);
	pxDialog->SetHelpText("This time in seconds over which the console var is smoothed between its min and max values.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	//end of links tab
	if(xOffset.y > fTabsEndY) fTabsEndY = xOffset.y;

	//end of tabs
	xOffset.y = fTabsEndY + 1.0f;
	pxOverlay->SetCurrentParentDialog(0);
	static u_int s_uSelectedTab = 0;
	pxTabsDialog->SetSelectedTabPointer(&s_uSelectedTab);

	pxDialog = new Asura_Function_Dialog("Play",xOffset,PlayAnim);
	pxDialog->SetHelpText("Plays this animation on all menu items that are using it.\nIf not being played then the currently edited frame will be shown instead.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Stop",xOffset,StopAnim);
	pxDialog->SetHelpText("Stops this animation on all menu items that are using it.\nIf not being played then the currently edited frame will be shown instead.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();


	pxOverlay->m_uID = TOOLS_OVERLAY_GUIMENU_ANIMATORSEQUENCE;
	Asura_Overlay_System::SubmitNewOverlay(pxOverlay,true);

	UpdateEditSeqOverlay();
}

void Tools_GUIMenu_AnimEdit::CloseEditSeqOverlay()
{
	Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_GUIMENU_ANIMATORSEQUENCE);
	FinishUVEdit();
}

void Tools_GUIMenu_AnimEdit::UpdateEditSeqOverlay()
{
	Asura_Overlay* pxOverlay = Asura_Overlay_System::GetOverlay(TOOLS_OVERLAY_GUIMENU_ANIMATORSEQUENCE);
	if(!pxOverlay) return;

	//update scale and format for UV editing controls
	const char* szUVPrintFormat = s_bShowInPixels ? "%.0f" : "%.3f";
	float fUVXScale = s_bShowInPixels ? (float) s_iTextureWidth : 1.0f;
	float fUVYScale = s_bShowInPixels ? (float) s_iTextureHeight : 1.0f;
	float fUVMin = -16.0f*Asura_Maths::Max(fUVXScale,fUVYScale);
	float fUVMax = -fUVMin;
	float fUVDelta = s_bShowInPixels ? 1.0f : 0.001f;

	Asura_Hash_ID auUVDialogID[4] = {g_uDialog_LeftUV,g_uDialog_TopUV,g_uDialog_RightUV,g_uDialog_BottomUV};
	for(int i=0;i<4;i++)
	{
		Asura_ScaledNumerical_Dialog* pxDialog = (Asura_ScaledNumerical_Dialog*) pxOverlay->GetDialog(auUVDialogID[i]);
		if(pxDialog)
		{
			pxDialog->SetScale((i&1) ? fUVYScale : fUVXScale, fUVMin, fUVMax);
			pxDialog->SetDelta(fUVDelta);
			pxDialog->SetPrintFormat(szUVPrintFormat);
		}
	}
	
}

void Tools_GUIMenu_AnimEdit::RenameSeq()
{

	Asura_Assert(s_bOverlayOpen,"");
	if(!Asura_GUIMenu_Animator_Storage::GetNumberOfSequences()) return;

	Asura_String xOldName = s_pszSeqNames[s_iSelectedSeq];
	Asura_Hash_ID uOldHash = Asura_GetHashID(xOldName.GetString());
	Asura_GUIMenu_Animator_Sequence* pxSeq = Asura_GUIMenu_Animator_Storage::GetSequenceByHash(uOldHash);

	//ask for the new name for the anim
	GetString_Overlay xOverlay("Enter new name for the sequence",xOldName.GetString());
	if(!xOverlay.DoModal()) return;

	//check the name is okay
	Asura_String xNewName = xOverlay.GetString();
	if(xNewName.IsBlank()) return;

	if(!strcmp(xNewName.GetString(),xOldName.GetString())) return;//name unchanged


	
	Asura_Hash_ID uNewHash = Asura_GetHashID(xNewName.GetString());
	if(uNewHash == uOldHash)
	{
		//Name is different , but has same hash (probably changed capitalisation.
		//So can get just update the name without doing anything else
		pxSeq->Rename(xNewName.GetString());

		//re-create the overlay to update the list
		CloseOverlays();
		CreateOverlay();
	}
	else
	{
		//hash already used?
		if(Asura_GUIMenu_Animator_Storage::GetSequenceByHash(uNewHash))
		{
			ShowMessageOverlay("A sequence with that name already exists");
			return;
		}

		//go ahead and rename then
		bool bOkay = Asura_GUIMenu_Animator_Storage::RenameSequence(xOldName.GetString(),xNewName.GetString());
		if(!bOkay)
		{
			Asura_Assert(0,"Unexpected error renaming sequence");
			return;
		}
		
		//re-create the overlay to update the list
		CloseOverlays();
		CreateOverlay();

	}

	//Update any menu items using the sequence
	Tools_Mode_GUIMenu::HandleRenameAnimator(xOldName.GetString(),xNewName.GetString());


	
		
	//Find the index under the new name
	s_iSelectedSeq = GetListIndexFromName(xNewName.GetString());
	Asura_Assert(s_iSelectedSeq>=0,"Failed to find sequence in list?");

	SetModified();
}


void Tools_GUIMenu_AnimEdit::CreateFrame()
{
	//find the sequence
	if(!s_pxEditedSeq) return;


	//create a new frame and select it
	s_pxEditedSeq->AddFrameAfter(s_iSelectedFrame);
	s_iSelectedFrame++;

	//and update the overlay
	UpdateOverlayVariablesFromSeq();

	SetModified();	
}

void Tools_GUIMenu_AnimEdit::EvenlySpaceFrames()
{
	if(!s_pxEditedSeq) return;

	// Count the number of frames
	u_int uNumFrames = s_pxEditedSeq->GetNumberOfFrames();

	if ( uNumFrames > 0 )
	{
		// Work out the spacing. The first frame always starts at 0.
		float fSpacing = 1.0f / (uNumFrames-1);

		// Set the timing for each frame
		float fTime = 0.0f;

		for ( u_int i = 0; i < uNumFrames; i++)
		{
			s_pxEditedSeq->SetFrameTime(i,fTime);
			fTime += fSpacing;
		}

		UpdateOverlayVariablesFromSeq();

		SetModified();	
	}
}

void Tools_GUIMenu_AnimEdit::DeleteFrame()
{
	if(!IsEnabled_DeleteFrame()) return;

	if(!ShowYesNoOverlay("Delete this frame?")) return;

	if(!s_pxEditedSeq) return;

	//delete the frame
	s_pxEditedSeq->DeleteFrame(s_iSelectedFrame);

	//and update the overlay
	UpdateOverlayVariablesFromSeq();

	SetModified();	
}

bool Tools_GUIMenu_AnimEdit::IsEnabled_DeleteFrame()
{
	if(!s_pxEditedSeq) return false;

	return s_pxEditedSeq->GetNumberOfFrames() > 1;
}

bool Tools_GUIMenu_AnimEdit::IsEnabled_GlobalTimer()
{
	if(!s_pxEditedSeq) return false;

	// Can only set global timer if the animation is looped in some way.
	return (s_pxEditedSeq->GetLoop() || s_pxEditedSeq->GetLoopBackAndForth());
}

bool Tools_GUIMenu_AnimEdit::IsEnabled_EaseInOut()
{
	if(!s_pxEditedSeq) return false;

	// Can only set ease in or out flag if interpolate is set
	return (s_pxEditedSeq->GetInterpolate());
}

// If a console var is selected then enabled the smooth tickbox
bool Tools_GUIMenu_AnimEdit::IsEnabled_SmoothTickBox()
{
	if ( s_pxEditedSeq == NULL ) return false;

	return s_pxEditedSeq->GetConsoleVar().GetHash() != ASURA_HASH_ID_UNSET;
}

void Tools_GUIMenu_AnimEdit::SetModified()
{
	Tools_GUIMenu_System::SetAnimatorsModified();
}


int Tools_GUIMenu_AnimEdit::GetListIndexFromName(const char* pszName)
{
	for(int i=0;i<s_iNumSeqNames;i++)
	{
		if(!stricmp(pszName,s_pszSeqNames[i]))
		{
			return i;
		}
	}
	//failed to find
	return -1;
}

void Tools_GUIMenu_AnimEdit::UpdateOverlayVariablesFromSeq()
{
	if(!s_pxEditedSeq) return;

	//grab the sequence variables
	s_fSeqDuration = s_pxEditedSeq->GetDuration();
	s_uSeqStateFlags = s_pxEditedSeq->GetStateFlags();
	s_bSeqLoop = s_pxEditedSeq->GetLoop();
	s_bSeqLoopBackAndForth = s_pxEditedSeq->GetLoopBackAndForth();
	s_bSeqPlayToEnd = s_pxEditedSeq->GetPlayToEnd();
	s_bSeqInterpolate = s_pxEditedSeq->GetInterpolate();
	s_bGlobalTime = s_pxEditedSeq->GetUseGlobalTime();

	s_iEaseType = s_pxEditedSeq->GetEaseType();

	s_bConsoleVarSmooth = s_pxEditedSeq->GetSmoothConsoleVar();

	s_fSmoothFrameTime = s_pxEditedSeq->GetSmoothFrameTime();

	//ensure frame in range
	if(s_iSelectedFrame < 0) s_iSelectedFrame = (int)s_pxEditedSeq->GetNumberOfFrames()-1;
	if(s_iSelectedFrame >= (int)s_pxEditedSeq->GetNumberOfFrames()) s_iSelectedFrame = 0;

	//grab the frame variables
	if(s_bFrameTimeInSecs)
	{
		s_fFrameTime = s_pxEditedSeq->GetFrameTime(s_iSelectedFrame)*s_fSeqDuration;
	}
	else
	{
		s_fFrameTime = s_pxEditedSeq->GetFrameTime(s_iSelectedFrame);
	}

	CopyFrameDataFromAsuraSequence(s_pxEditedSeq,s_iSelectedFrame);


	//update string indicating number of frames
	char szText[50];
	sprintf(szText,"(of %d)",s_pxEditedSeq->GetNumberOfFrames());
	s_xNumFrameText = szText;
}

void Tools_GUIMenu_AnimEdit::UpdateSeqFromOverlayVariables()
{
	if(!s_pxEditedSeq) return;

	//update the sequence variables
	s_pxEditedSeq->SetDuration(s_fSeqDuration);
	s_pxEditedSeq->SetStateFlags(s_uSeqStateFlags);

	if ((s_bSeqLoop && s_bSeqLoopBackAndForth) || (s_bSeqPlayToEnd && s_bSeqLoop) || (s_bSeqPlayToEnd && s_bSeqLoopBackAndForth))
	{
		//only one of these should be set.. so go with the one that was just changed
		if (!s_pxEditedSeq->GetLoop() && s_bSeqLoop) 
		{	
			s_bSeqLoopBackAndForth = false;
			s_bSeqPlayToEnd = false;
		}
		else if (!s_pxEditedSeq->GetLoopBackAndForth() && s_bSeqLoopBackAndForth)
		{
			s_bSeqLoop = false;
			s_bSeqPlayToEnd = false;
		}
		else
		{
			s_bSeqLoopBackAndForth = false;
			s_bSeqLoop = false;
		}
	}

	if ( s_bSeqLoop == false && s_bSeqLoopBackAndForth == false )
	{
		// Can only set global timer if the animation is looped in some way.
		// If animation is not looped then change the global time flag
		s_bGlobalTime = false;
	}

	s_pxEditedSeq->SetLoop(s_bSeqLoop);
	s_pxEditedSeq->SetLoopBackAndForth(s_bSeqLoopBackAndForth);
	s_pxEditedSeq->SetPlayToEnd(s_bSeqPlayToEnd);
	s_pxEditedSeq->SetInterpolate(s_bSeqInterpolate);
	s_pxEditedSeq->SetUseGlobalTime(s_bGlobalTime);
	s_pxEditedSeq->SetSmoothConsoleVar(s_bConsoleVarSmooth);
	s_pxEditedSeq->SetSmoothFrameTime(s_fSmoothFrameTime);

	s_pxEditedSeq->SetEaseType(static_cast<ASURA_GUIMENU_ANIMATOR_EASE_TYPE>(s_iEaseType));

	if(s_iSelectedFrame >= (int) s_pxEditedSeq->GetNumberOfFrames() || s_iSelectedFrame<0)
	{
		//frame out of range... forget about this and update from the anim instead
		UpdateOverlayVariablesFromSeq();
		return;
	}


	if(s_bFrameTimeInSecs)
	{
		if(s_bDurationChanged)
		{
			//If duration has changed we need to recalculate the display time in seconds
			s_fFrameTime = s_pxEditedSeq->GetFrameTime(s_iSelectedFrame)*s_fSeqDuration;
		}
		else
		{
			s_fFrameTime = Asura_Maths::ClampToRange(s_fFrameTime,0.0f,s_fSeqDuration);
			s_pxEditedSeq->SetFrameTime(s_iSelectedFrame, s_fFrameTime/s_fSeqDuration);
		}
	}
	else
	{
		s_fFrameTime = Asura_Maths::ClampToRange(s_fFrameTime,0.0f,1.0f);
		s_pxEditedSeq->SetFrameTime(s_iSelectedFrame, s_fFrameTime);
	}

	CopyFrameDataToAsuraSequence(s_pxEditedSeq,s_iSelectedFrame);
}


bool Tools_GUIMenu_AnimEdit::IsModifierTypeSupported(int iType)
{
	return	(iType == ASURA_GUIMENU_MODIFY_POSITION) |
			(iType == ASURA_GUIMENU_MODIFY_UVS) |
			(iType == ASURA_GUIMENU_MODIFY_TEXT_SCROLL_RATE) |
			(iType == ASURA_GUIMENU_MODIFY_TEXT_OFFSET_X) |
			(iType == ASURA_GUIMENU_MODIFY_TEXT_OFFSET_Y) |
			(iType == ASURA_GUIMENU_MODIFY_ALPHA_SCALED) |
			(iType == ASURA_GUIMENU_MODIFY_WIDTH_SCALED) |
			(iType == ASURA_GUIMENU_MODIFY_HEIGHT_SCALED) |
			(iType == ASURA_GUIMENU_MODIFY_FONT_SCALE) |
			(iType == ASURA_GUIMENU_MODIFY_COLOUR) |
			(iType == ASURA_GUIMENU_MODIFY_IMAGE_ROTATE) |
			(iType == ASURA_GUIMENU_MODIFY_POSITION_RELATIVE) |
			(iType == ASURA_GUIMENU_MODIFY_SIZE_SCALE);
}


void Tools_GUIMenu_AnimEdit::CopyFrameDataToAsuraSequence(Asura_GUIMenu_Animator_Sequence* pxSeq,int iFrame)
{
	switch(pxSeq->GetDataType())
	{
		case ASURA_GUIMENU_ANIMATOR_DATA_TYPE_VECTOR2 :
			pxSeq->SetFrameData(iFrame,s_xFrameData_Vector2);
			break;

		case ASURA_GUIMENU_ANIMATOR_DATA_TYPE_FLOAT :
			pxSeq->SetFrameData(iFrame,s_fFrameData);
			break;

		case ASURA_GUIMENU_ANIMATOR_DATA_TYPE_UVSET :
			pxSeq->SetFrameData(iFrame,s_xFrameData_UV);
			break;

		case ASURA_GUIMENU_ANIMATOR_DATA_TYPE_RGBA :
			pxSeq->SetFrameData(iFrame,s_xFrameData_RGBA);
			break;

		default :
			Asura_Assert(0,"Unhandled case in Tools_GUIMenu_AnimEdit::CopyFrameDataToAsuraSequence()");
			break;
	}
}

void Tools_GUIMenu_AnimEdit::CopyFrameDataFromAsuraSequence(const Asura_GUIMenu_Animator_Sequence* pxSeq,int iFrame)
{
	switch(pxSeq->GetDataType())
	{
		case ASURA_GUIMENU_ANIMATOR_DATA_TYPE_VECTOR2 :
			s_xFrameData_Vector2 = *(Asura_Vector_2*) pxSeq->GetFrameData(iFrame);
			break;

		case ASURA_GUIMENU_ANIMATOR_DATA_TYPE_FLOAT :
			s_fFrameData = *(float*) pxSeq->GetFrameData(iFrame);
			break;

		case ASURA_GUIMENU_ANIMATOR_DATA_TYPE_UVSET :
			s_xFrameData_UV = *(Asura_GUIMenu_UV_Set*) pxSeq->GetFrameData(iFrame);
			break;

		case ASURA_GUIMENU_ANIMATOR_DATA_TYPE_RGBA :
			s_xFrameData_RGBA = *(Asura_RGBA*) pxSeq->GetFrameData(iFrame);
			break;

		default :
			Asura_Assert(0,"Unhandled case in Tools_GUIMenu_AnimEdit::CopyFrameDataFromAsuraSequence()");
			break;
	}
}

void Tools_GUIMenu_AnimEdit::AddAnimatorFrameControlsToOverlay(Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset, Asura_GUIMenu_Animator_Sequence* pxSeq)
{
	Asura_Dialog* pxDialog;

	switch(pxSeq->GetModifyType())
	{
		case ASURA_GUIMENU_MODIFY_POSITION :
		{
			pxDialog = new Asura_Numerical_Dialog("X Offset",xOffset,&s_xFrameData_Vector2.x,-9999.0f,9999.0f,1.0f,"%.0f",&s_bChanged);
			pxDialog->SetHelpText("The offset to applied on this frame.");
			pxOverlay->AddDialog(pxDialog);
			xOffset.y += Asura_Dialog::DefaultSpacing();

			pxDialog = new Asura_Numerical_Dialog("Y Offset",xOffset,&s_xFrameData_Vector2.y,-9999.0f,9999.0f,1.0f,"%.0f",&s_bChanged);
			pxDialog->SetHelpText("The offset to applied on this frame.");
			pxOverlay->AddDialog(pxDialog);
			xOffset.y += Asura_Dialog::DefaultSpacing();
			break;
		}

		case ASURA_GUIMENU_MODIFY_POSITION_RELATIVE :
		{
			pxDialog = new Asura_Numerical_Dialog("X Offset",xOffset,&s_xFrameData_Vector2.x,-1.0f,1.0f,0.001f,"%1.3f",&s_bChanged);
			pxDialog->SetHelpText("The offset to applied on this frame in relative 0-1 offsets.  The 0-1 actual pixel offset is calculated based on the menu width.");
			pxOverlay->AddDialog(pxDialog);
			xOffset.y += Asura_Dialog::DefaultSpacing();

			pxDialog = new Asura_Numerical_Dialog("Y Offset",xOffset,&s_xFrameData_Vector2.y,-1.0f,1.0f,0.001f,"%1.3f",&s_bChanged);
			pxDialog->SetHelpText("The offset to applied on this frame in relative 0-1 offsets.  The 0-1 actual pixel offset is calculated based on the menu height.");
			pxOverlay->AddDialog(pxDialog);
			xOffset.y += Asura_Dialog::DefaultSpacing();
			break;
		}

		case ASURA_GUIMENU_MODIFY_WIDTH_SCALED :
		{
			pxDialog = new Asura_Numerical_Dialog("Width Scale",xOffset,&s_fFrameData,0.0f,99.0f,0.1f,"%.3f",&s_bChanged);
			pxDialog->SetHelpText("Multiplier to apply to the menu item's width.");
			pxOverlay->AddDialog(pxDialog);
			xOffset.y += Asura_Dialog::DefaultSpacing();
			break;
		}

		case ASURA_GUIMENU_MODIFY_HEIGHT_SCALED :
		{
			pxDialog = new Asura_Numerical_Dialog("Height Scale",xOffset,&s_fFrameData,0.0f,99.0f,0.1f,"%.3f",&s_bChanged);
			pxDialog->SetHelpText("Multiplier to apply to the menu item's height.");
			pxOverlay->AddDialog(pxDialog);
			xOffset.y += Asura_Dialog::DefaultSpacing();
			break;
		}

		case ASURA_GUIMENU_MODIFY_SIZE_SCALE :
		{
			pxDialog = new Asura_Numerical_Dialog("Width Scale",xOffset,&s_xFrameData_Vector2.x,0.0f,99.0f,0.1f,"%.3f",&s_bChanged);
			pxDialog->SetHelpText("Multiplier to apply to the menu item's width.");
			pxOverlay->AddDialog(pxDialog);
			xOffset.y += Asura_Dialog::DefaultSpacing();

			pxDialog = new Asura_Numerical_Dialog("Height Scale",xOffset,&s_xFrameData_Vector2.y,0.0f,99.0f,0.1f,"%.3f",&s_bChanged);
			pxDialog->SetHelpText("Multiplier to apply to the menu item's height.");
			pxOverlay->AddDialog(pxDialog);
			xOffset.y += Asura_Dialog::DefaultSpacing();
			break;
		}

		case ASURA_GUIMENU_MODIFY_ALPHA_SCALED :
		{
			pxDialog = new Asura_Numerical_Dialog("Alpha Scale",xOffset,&s_fFrameData,0.0f,1.0f,0.1f,"%.3f",&s_bChanged);
			pxDialog->SetHelpText("Multiplier to apply to the alpha value.");
			pxOverlay->AddDialog(pxDialog);
			xOffset.y += Asura_Dialog::DefaultSpacing();
			break;
		}

		case ASURA_GUIMENU_MODIFY_TEXT_SCROLL_RATE :
		{
			pxDialog = new Asura_Numerical_Dialog("Text Scroll Rate",xOffset,&s_fFrameData, 0.0f, 999.0f, 0.1f, "%.1f",&s_bChanged);
			pxOverlay->AddDialog(pxDialog);
			xOffset.y += Asura_Dialog::DefaultSpacing();
			break;
		}

		case ASURA_GUIMENU_MODIFY_TEXT_OFFSET_X :
		{
			pxDialog = new Asura_Numerical_Dialog("Text X Offset",xOffset,&s_fFrameData,-9999.0f,9999.0f,1.0f,"%.0f",&s_bChanged);
			pxOverlay->AddDialog(pxDialog);
			xOffset.y += Asura_Dialog::DefaultSpacing();
			break;
		}

		case ASURA_GUIMENU_MODIFY_TEXT_OFFSET_Y :
		{
			pxDialog = new Asura_Numerical_Dialog("Text Y Offset",xOffset,&s_fFrameData,-9999.0f,9999.0f,1.0f,"%.0f",&s_bChanged);
			pxOverlay->AddDialog(pxDialog);
			xOffset.y += Asura_Dialog::DefaultSpacing();
			break;
		}

		case ASURA_GUIMENU_MODIFY_FONT_SCALE :
		{
			pxDialog = new Asura_Numerical_Dialog( "Font Scale", xOffset, &s_fFrameData, -9999.0f, 9999.0f, 0.1f, "%2.3f", &s_bChanged);
			pxOverlay->AddDialog(pxDialog);
			xOffset.y += Asura_Dialog::DefaultSpacing();

			break;
		}

		case ASURA_GUIMENU_MODIFY_UVS:
		{
			pxDialog = new Asura_Function_Dialog("Select Texture",xOffset,SelectTexture);
			pxDialog->SetHelpText("This selects the texture that will be shown while editing UV coordinates. It is also used for determining the scale values to use when showing the coordinates in pixels.\nThis texture is purely for helping with editing , and has no affect on the sequence itself.");
			pxOverlay->AddDialog(pxDialog);
			xOffset.y += Asura_Dialog::DefaultSpacing();
			
			
			pxDialog = new Asura_Function_Dialog("Texture coordinates:", xOffset, ToggleUVEdit);
			pxOverlay->AddDialog(pxDialog);
			xOffset.y += Asura_Dialog::DefaultSpacing();

			pxDialog = new Asura_Tickbox_Dialog("Show pixel coordinates", xOffset, &s_bShowInPixels, &s_bUpdateEditSeqOverlay);
			pxDialog->SetHelpText("Determines whether coordinates are shown between 0 to 1 , or as pixel coordinates on the texture");
			pxOverlay->AddDialog(pxDialog);
			xOffset.y += Asura_Dialog::DefaultSpacing();



			pxDialog = new Asura_ScaledNumerical_Dialog("Left", xOffset, 1.0f, &s_xFrameData_UV.xUV0.u, 0.0f, 1.0f, .001f, "%.3f", &s_bChanged);
			pxDialog->m_iControlID = g_uDialog_LeftUV;
			pxOverlay->AddDialog(pxDialog);
			xOffset.y += Asura_Dialog::DefaultSpacing();

			pxDialog = new Asura_ScaledNumerical_Dialog("Right", xOffset, 1.0f, &s_xFrameData_UV.xUV1.u, 0.0f, 1.0f, .001f, "%.3f", &s_bChanged);
			pxDialog->m_iControlID = g_uDialog_RightUV;
			pxOverlay->AddDialog(pxDialog);
			xOffset.y += Asura_Dialog::DefaultSpacing();

			pxDialog = new Asura_ScaledNumerical_Dialog("Top", xOffset, 1.0f, &s_xFrameData_UV.xUV0.v, 0.0f, 1.0f, .001f, "%.3f", &s_bChanged);
			pxDialog->m_iControlID = g_uDialog_TopUV;
			pxOverlay->AddDialog(pxDialog);
			xOffset.y += Asura_Dialog::DefaultSpacing();

			pxDialog = new Asura_ScaledNumerical_Dialog("Bottom", xOffset, 1.0f, &s_xFrameData_UV.xUV1.v, 0.0f, 1.0f, .001f, "%.3f", &s_bChanged);
			pxDialog->m_iControlID = g_uDialog_BottomUV;
			pxOverlay->AddDialog(pxDialog);
			xOffset.y += Asura_Dialog::DefaultSpacing();

			break;
		}

		case ASURA_GUIMENU_MODIFY_COLOUR :
		{
			pxDialog = new Asura_Dialog("Colour",xOffset);
			pxOverlay->AddDialog(pxDialog);

			xOffset.x = pxDialog->m_xMaxCoord.x + 1.0f;
			Tools_ColourPicker_Dialog* pxColourDialog = new Tools_ColourPicker_Dialog("Colour",xOffset,Asura_Vector_2(1.5f,1.0f),&s_xFrameData_RGBA.m_fR,&s_xFrameData_RGBA.m_fG,&s_xFrameData_RGBA.m_fB,1.0f,&s_bChanged);
			pxOverlay->AddDialog(pxColourDialog);
			xOffset.y += Asura_Dialog::DefaultSpacing();
			xOffset.x = 1.0f;

			break;
		}

		case ASURA_GUIMENU_MODIFY_IMAGE_ROTATE :
		{
			pxDialog = new Asura_AngleNumerical_Dialog( "Rotation", xOffset, &s_fFrameData,-999.0f, 999.0f, 1.0f, "%3.3f", &s_bChanged);
			pxDialog->SetHelpText("Angle in degrees");
			pxOverlay->AddDialog(pxDialog);
			xOffset.y += Asura_Dialog::DefaultSpacing();
			break;
		}

		default :
			Asura_Assert(0,"Unhandled case in Tools_GUIMenu_AnimEdit::AddAnimatorFrameControlsToOverlay()");
			break;
	}
}

void Tools_GUIMenu_AnimEdit::PlayAnim()
{
	s_fTimerOverride = 0.0f;
	s_bUseTimerOverride = true;
}

void Tools_GUIMenu_AnimEdit::StopAnim()
{
	s_fTimerOverride = 0.0f;
	s_bUseTimerOverride = false;
}


bool Tools_GUIMenu_AnimEdit::IsToolsOverride(const Asura_GUIMenu_Animator_Sequence* pxSeq)
{
	//Is this sequence currently being edited?
	if(!pxSeq) return false;
	if(pxSeq != s_pxEditedSeq) return false;

	//Override isn't applied during menu preview
	if(Tools_Mode_GUIMenu::IsPreview()) return false;

	return true;
}

u_int Tools_GUIMenu_AnimEdit::GetFrameOverride()
{
	//Get the edited frame , if not playing the animation
	return s_bUseTimerOverride ? 0xffffffff : s_iSelectedFrame;
}

float Tools_GUIMenu_AnimEdit::GetTimerOverride()
{
	return s_fTimerOverride;
}


void Tools_GUIMenu_AnimEdit::ToggleUVEdit()
{
	if(s_pxUVEdit)
	{
		FinishUVEdit();
	}
	else
	{
		StartUVEdit();
	}
}


void Tools_GUIMenu_AnimEdit::StartUVEdit()
{
	//allocate the uv editing thing
	s_pxUVEdit = new Tools_UV_Editor;
	s_pxUVEdit->SetTextureID(s_iTextureID);

	//initalise the uv coordinates
	s_pxUVEdit->Nodes[0].x = s_xFrameData_UV.xUV0.u;
	s_pxUVEdit->Nodes[0].y = s_xFrameData_UV.xUV0.v;
	s_pxUVEdit->Nodes[1].x = s_xFrameData_UV.xUV1.u;
	s_pxUVEdit->Nodes[1].y = s_xFrameData_UV.xUV0.v;
	s_pxUVEdit->Nodes[2].x = s_xFrameData_UV.xUV1.u;
	s_pxUVEdit->Nodes[2].y = s_xFrameData_UV.xUV1.v;
	s_pxUVEdit->Nodes[3].x = s_xFrameData_UV.xUV0.u; 
	s_pxUVEdit->Nodes[3].y = s_xFrameData_UV.xUV1.v; 
	

	s_pxUVEdit->CreateMouseModeOverlay();
}

void Tools_GUIMenu_AnimEdit::FinishUVEdit()
{
	if(s_pxUVEdit)
	{
		s_pxUVEdit->CloseMouseModeOverlay();
		delete s_pxUVEdit;
		s_pxUVEdit = 0;
	}
}

void Tools_GUIMenu_AnimEdit::UpdateUVEdit()
{
	if(!s_pxUVEdit) return;

	s_pxUVEdit->HandleMouse();


	if(s_pxUVEdit->NodesMoved)
	{
		//make sure the nodes stay in rectangular formation
		s_pxUVEdit->EnsureNodesAreStillOkayForTextureWrap();
		
		s_pxUVEdit->Nodes[1].x = s_pxUVEdit->Nodes[2].x;
		s_pxUVEdit->Nodes[1].y = s_pxUVEdit->Nodes[0].y;

		s_pxUVEdit->Nodes[3].x = s_pxUVEdit->Nodes[0].x;
		s_pxUVEdit->Nodes[3].y = s_pxUVEdit->Nodes[2].y;


		s_pxUVEdit->NodesMoved = false;


		s_xFrameData_UV.xUV0.u = s_pxUVEdit->Nodes[0].x; 
		s_xFrameData_UV.xUV0.v  = s_pxUVEdit->Nodes[0].y; 
		s_xFrameData_UV.xUV1.u  = s_pxUVEdit->Nodes[2].x; 
		s_xFrameData_UV.xUV1.v  = s_pxUVEdit->Nodes[2].y;
		s_bChanged = true;
		

	}
	else
	{
		//make sure the uv coordinates being edited match those on the overlay
		s_pxUVEdit->Nodes[0].x = s_xFrameData_UV.xUV0.u;
		s_pxUVEdit->Nodes[0].y = s_xFrameData_UV.xUV0.v;
		s_pxUVEdit->Nodes[1].x = s_xFrameData_UV.xUV1.u;
		s_pxUVEdit->Nodes[1].y = s_xFrameData_UV.xUV0.v;
		s_pxUVEdit->Nodes[2].x = s_xFrameData_UV.xUV1.u;
		s_pxUVEdit->Nodes[2].y = s_xFrameData_UV.xUV1.v;
		s_pxUVEdit->Nodes[3].x = s_xFrameData_UV.xUV0.u; 
		s_pxUVEdit->Nodes[3].y = s_xFrameData_UV.xUV1.v; 
	}
}

void Tools_GUIMenu_AnimEdit::SelectTexture()
{
	s_iTextureID = Tools_GUIMenu_Element_Image::SelectTexture(s_iTextureID);

	//get the width/height for the scaling
	s_iTextureWidth = 512;
	s_iTextureHeight = 512;
	
	if(s_iTextureID != ASURA_TEXTURE_NONE)
	{
		Tools_Texture_Management::GetTextureSize(s_iTextureID,s_iTextureWidth,s_iTextureHeight);
	}
	s_bUpdateEditSeqOverlay = true;

	//update the texture in the UV editor
	if(s_pxUVEdit)
	{
		s_pxUVEdit->SetTextureID(s_iTextureID);
	}
}


bool Tools_GUIMenu_AnimEdit::IsUsingKey(int iKey)
{
	if(s_pxUVEdit)
	{
		if(iKey == ASURA_KEYID_MOUSEBUTTON1 ||
			iKey == ASURA_KEYID_MOUSEBUTTON2)
		{
			return true;
		}
	}
	return false;
}

#endif // ASURA_USE_GUI_MENU_SYSTEM