//
// Tools_GUIMenu_AnimEdit.h
//
// Created by Richard Rice - 12/04/2007
// 
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//


#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef __TOOLS_GUIMENU_ANIMEDIT_H__
#define __TOOLS_GUIMENU_ANIMEDIT_H__

#ifdef ASURA_USE_GUI_MENU_SYSTEM

#include "Core/Asura_Maths.h"
#include "Core/Asura_Hash.h"
#include "GUIMenu/Asura_GUIMenu_Animator.h" // for ASURA_GUIMENU_ANIMATOR_EASE_TYPE

class Asura_Overlay;
class Asura_GUIMenu_Animator_Sequence;
class Tools_GUIMenu_Root;
class Tools_UV_Editor;
struct Asura_GUIMenu_UV_Set;
class Asura_RGBA;

//Handles editing of Animator sequences for the GUIMenu system.

class Tools_GUIMenu_AnimEdit
{
public :

	static void Initialise();
	static void Shutdown();

	static void Update();
	static void Render();

	static void ToggleOverlay();
	static bool OverlayIsOn();
	static void CreateOverlay();

	static void CloseOverlays();

	//returns true if tools is currently overriding the playback state for this sequence
	static bool IsToolsOverride(const Asura_GUIMenu_Animator_Sequence* pxSeq);

	//gets the frame to use for anim override , or 0xffffffff if not setting the frame directly
	static u_int GetFrameOverride();
	//gets the timer to use for anim override.
	static float GetTimerOverride();

	//for sorting out clash in use of mouse buttons.. reports which keys the anim editing is currently making use of
	static bool IsUsingKey(int iKey);


//overlay stuff that gets saved to mode ini file
	static bool s_bOverlayOpen;
	static u_int s_uOverlayMinimised;
	static Asura_Vector_2 s_xOverlayPosition;
	static Asura_Vector_2 s_xEditSeqOverlayPosition;

private :
	//deals with the list of names for the overlay
	static void BuildSeqNames();
	static void DeleteSeqNames();

	//overlay functions
	static void CreateSeq();
	static void DeleteSeq();
	static void DuplicateSeq();
	static bool IsSeqSelected();

	static void ApplySeq();
	static bool CanApplySeq();
	static bool ApplySeq(bool bJustCheckIfAllowed);

	//opens overlay for editing selected anim
	static void EditSeq();
	static void EditSeq(Asura_GUIMenu_Animator_Sequence* pxSeq);
	//and closes
	static void CloseEditSeqOverlay();

	//update various controls on the sequence editing overlay
	static void UpdateEditSeqOverlay();

	static void RenameSeq();

	static void EvenlySpaceFrames();

	static void CreateFrame();
	static void DeleteFrame();
	static bool IsEnabled_DeleteFrame();

	static bool IsEnabled_GlobalTimer();

	static bool IsEnabled_EaseInOut();

	//set the file that contains the animations as modified
	static void SetModified();

	//gets the index of the anim from the list shown on the overlay.
	//(not the same as the index in the animation system).
	static int GetListIndexFromName(const char* pszName);

	//updates the copy of the variables used by the overlays from the Asura_GUIMenu_AnimEdit
	static void UpdateOverlayVariablesFromSeq();
	//and the reverse
	static void UpdateSeqFromOverlayVariables();

	//Deals with copying the overlay data to/from the sequence for the data that changes according to animator type.
	static void CopyFrameDataToAsuraSequence(Asura_GUIMenu_Animator_Sequence* pxSeq,int iFrame);
	static void CopyFrameDataFromAsuraSequence(const Asura_GUIMenu_Animator_Sequence* pxSeq,int iFrame);

	//Adds the overlay controls for the frame data
	static void AddAnimatorFrameControlsToOverlay(Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset, Asura_GUIMenu_Animator_Sequence* pxSeq);
 

	static bool IsEnabled_FrameTime(){return s_iSelectedFrame!=0;}

	// If the smooth tick box is set then enable the smooth time dialog
	static bool IsEnabled_SmoothFrameTime(){return s_bConsoleVarSmooth!=0;}

	// If a console var is selected then enabled the smooth tickbox
	static bool IsEnabled_SmoothTickBox();

	//Returns true for modifier types that are currently supported in animators
	static bool IsModifierTypeSupported(int iType);

	//starts/stops preview
	static void PlayAnim();
	static void StopAnim();

	static void ToggleUVEdit();
	static void StartUVEdit();
	static void FinishUVEdit();
	static void UpdateUVEdit();
	static void SelectTexture();



	static int s_iSelectedSeq;
	static int s_iNumSeqNames;
	static const char** s_pszSeqNames;

	static Asura_GUIMenu_Animator_Sequence* s_pxEditedSeq;

	static int s_iSelectedFrame;

	static class Asura_String s_xNumFrameText;

	static bool s_bNeedUpdateFromSeq;
	static bool s_bChanged;

	static float s_fSeqDuration;
	static u_int s_uSeqStateFlags;
	static bool	s_bSeqLoop;
	static bool	s_bSeqLoopBackAndForth;
	static bool s_bSeqPlayToEnd;
	static bool s_bSeqInterpolate;
	static bool s_bSeqRandom;
	static bool s_bDurationChanged;
	static bool s_bGlobalTime;
	static bool s_bConsoleVarSmooth;

	static int s_iEaseType;

	static float s_fFrameTime;
	static bool s_bFrameTimeInSecs;

	static float s_fSmoothFrameTime;


	//frame data for each of the animator data types
	static Asura_Vector_2 s_xFrameData_Vector2;
	static float s_fFrameData;
	static Asura_GUIMenu_UV_Set s_xFrameData_UV;
	static Asura_RGBA s_xFrameData_RGBA;

	//extra options while working with UVs
	static int s_iTextureID;
	static int s_iTextureWidth;
	static int s_iTextureHeight;
	static bool s_bShowInPixels;
	static bool s_bUpdateEditSeqOverlay;
	static Tools_UV_Editor* s_pxUVEdit;




	//For previewing the edited sequence
	static bool s_bUseTimerOverride;
	static float s_fTimerOverride;

};
#endif // ASURA_USE_GUI_MENU_SYSTEM

#endif //__TOOLS_GUIMENU_ANIMEDIT_H__
