//////////////////////////////////////////////////////////////////////////////
//
// Tools_GUIMenu_CommandConsole.cpp
//
// Copyright (c) 2007 Rebellion. All rights reserved.
//
// Description:
//
//     Tools interface for GUIMenu command console interaction
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

#ifdef ASURA_USE_GUI_MENU_SYSTEM

// This file's header.
#include "GUIMenu/Tools_GUIMenu_CommandConsole.h"

#include "Core/Asura_NameTree.h"
#include "GUIMenu/Asura_GUIMenu_CommandConsole.h"
#include "GUIMenu/Asura_GUIMenu_Menu_Page.h"
#include "GUIMenu/Asura_GUIMenu_Root.h"
#include "GUIMenu/Asura_GUIMenu_Widget_Base.h"
#include "GUIMenu/Asura_GUIMenu_System.h"
#include "Interface/Asura_CommandConsole.h"
#include "Interface/Asura_Overlay.h"

#include "GUIMenu/Tools_GUIMenu_Root.h"
#include "GUIMenu/Tools_Mode_GUIMenu.h"
#include "Misc/Tools_Modal_Overlay.h"


// G L O B A L S /////////////////////////////////////////////////////////////
static const char* g_aszDisplayConditions[NUM_ASURA_GUIMENU_CONDITIONS] = 
{
	" ",//leaving the none entry blank to make the dropdown list a bit tidier
	"=",
	">",
	">=",
	"<",
	"<=",
	"!="
};

static Asura_String g_xHelpText;

// I N T E R N A L   F U N C T I O N S ///////////////////////////////////////
static void GetHelpTextCallback( const Asura_UnicodeChar* wszHelpOutput )
{
	char acHelpText[256];
	Asura_PrintfToBfr(acHelpText, "%S", 256, true, wszHelpOutput );
	if (!Asura_StringUtil::Contains(acHelpText, "no help available"))
	{
		g_xHelpText += "\n";
		g_xHelpText += acHelpText;
		g_xHelpText += "\n";
	}
}

//////////////////////////////////////////////////////////////////////////////
// Asura_GUIMenu_ConsoleVar
//////////////////////////////////////////////////////////////////////////////
void Asura_GUIMenu_ConsoleVar::AddOverlayControls(Asura_Overlay* pxOverlay, 
												  Asura_Vector_2& xOffset,
												  const char* szLabel,
												  const char* szHelpText,
												  int iType, 
												  bool* pbChanged,
												  bool bEnable,
												  List<Asura_Dialog*>* pxDialogList /* = NULL */ )
{
	Asura_Dialog* pxDialog;

	pxDialog = new Tools_GUIMenu_ConsoleVar_Dialog( xOffset, szLabel,this, iType, szHelpText, pbChanged);
	pxDialog->SetEnabled(bEnable);
	if(szHelpText) pxDialog->SetHelpText(szHelpText);

	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	if(pxDialogList) pxDialogList->add_entry( pxDialog );
}

//////////////////////////////////////////////////////////////////////////////
// Asura_GUIMenu_ConsoleCmd
//////////////////////////////////////////////////////////////////////////////
void Asura_GUIMenu_ConsoleCmd::AddOverlayControls(Asura_Overlay* pxOverlay, 
												  Asura_Vector_2& xOffset,
												  const char* szLabel,
												  const char* szHelpText,
												  bool bAllowArgs,
												  bool* pbChanged,
												  bool bEnable,
												  List<Asura_Dialog*>* pxDialogList /* = NULL */ )
{
	Asura_Dialog* pxDialog;
	int iType = 0;
	pxDialog = new Tools_GUIMenu_ConsoleCmd_Dialog( xOffset, szLabel, &m_xConsoleCmdName, iType, szHelpText, pbChanged );
	pxDialog->SetEnabled(bEnable);
	if(szHelpText) pxDialog->SetHelpText(szHelpText);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();
	if(pxDialogList) pxDialogList->add_entry( pxDialog );

	if (bAllowArgs)
	{
		int iMaxArgsLen = 24;// arbitrary limit
		pxDialog = new Asura_String_Dialog( "ConsoleCmd args", xOffset, &m_xConsoleCmdArgs, iMaxArgsLen, pbChanged );
		pxDialog->SetEnabled(bEnable);
		pxOverlay->AddDialog(pxDialog);
		xOffset.y += Asura_Dialog::DefaultSpacing();

		if(pxDialogList) pxDialogList->add_entry( pxDialog );
	}
}

//////////////////////////////////////////////////////////////////////////////
// Asura_GUIMenu_ConditionVar
//////////////////////////////////////////////////////////////////////////////

void Asura_GUIMenu_ConditionVar::AddOverlayControls(class Asura_Overlay* pxOverlay,class Asura_Vector_2& xOffset, bool* pbChanged, bool bEnable, List<Asura_Dialog*>* pxDialogList /* = NULL */ )
{
	float fXOffset = xOffset.x;

	Asura_Dialog* pxDialog;
    	
	bool bVarEnabled = (bEnable && m_xCmd.IsBlank());
	bool bCmdEnabled = (bEnable && m_xVar.IsBlank());

	int iType = Asura_CommandConsole_VarRepository::Var::TYPE_INVALID;
	const char* szHelpText = "The console variable to use for the display condition.\nIf a display condition is set , then the menu item will only be visible when the result is true.\nAny conditions that are set to *None* will be removed when saved.";

	pxDialog = new Asura_Dialog("Var", xOffset);
	pxOverlay->AddDialog(pxDialog);
	if(pxDialogList) pxDialogList->add_entry( pxDialog );

	xOffset.x = pxDialog->m_xMaxCoord.x + 1.0f;
	m_xVar.AddOverlayControls( pxOverlay, xOffset, "", szHelpText, iType, pbChanged, bVarEnabled, pxDialogList);
	xOffset.x = fXOffset;

	pxDialog = new Asura_Dialog("Cmd", xOffset);
	pxOverlay->AddDialog(pxDialog);
	if(pxDialogList) pxDialogList->add_entry( pxDialog );

	xOffset.x = pxDialog->m_xMaxCoord.x + 1.0f;
	m_xCmd.AddOverlayControls( pxOverlay, xOffset, "", szHelpText, true, pbChanged, bCmdEnabled, pxDialogList);
	xOffset.x = fXOffset;

	pxDialog = new Asura_DropDown_List_Dialog(xOffset, g_aszDisplayConditions, (int*)&m_eCondition, NUM_ASURA_GUIMENU_CONDITIONS, NUM_ASURA_GUIMENU_CONDITIONS, NULL, pbChanged);
	if(pxDialogList) pxDialogList->add_entry( pxDialog );
	
	pxDialog->SetHelpText("The comparison to use for the display condition.\nIf a display condition is set , then the menu item will only be visible when the result is true.");
	if(bEnable)
		pxDialog->SetIsEnabledFunction(IsConsoleVarSelected,(int) this);
	else
		pxDialog->SetEnabled(false);
	pxOverlay->AddDialog(pxDialog);
	//xOffset.y += Asura_Dialog::DefaultSpacing();
	xOffset.x = pxDialog->m_xMaxCoord.x + 1.0f;

	int iMaxArgsLen = 12;// arbitrary limit (enough for MAX_INT)
	pxDialog = new Asura_String_Dialog( "", xOffset, &m_xValue, iMaxArgsLen, pbChanged );
	if(pxDialogList) pxDialogList->add_entry( pxDialog );

	pxDialog->SetHelpText("The value to compare the display condition against.\nIf a display condition is set , then the menu item will only be visible when the result is true.");
	if(bEnable)
		pxDialog->SetIsEnabledFunction(IsConsoleVarSelected,(int) this);
	else
		pxDialog->SetEnabled(false);
	pxDialog->SetIsHiddenFunction(IsWidgetOrPageHashConsoleVar,(int) this);
	pxOverlay->AddDialog(pxDialog);

	//special widget selection control for use when the console var is GUIMenu.ActiveWidget
	pxDialog = new Tools_GUIMenu_ConditionVar_Widget_Dialog(xOffset, &m_xValue, pbChanged);
	if(pxDialogList) pxDialogList->add_entry( pxDialog );

	pxDialog->SetHelpText("The widget to compare the display condition against.");
	if(bEnable)
		pxDialog->SetIsEnabledFunction(IsConsoleVarSelected,(int) this);
	else
		pxDialog->SetEnabled(false);
	pxDialog->SetIsHiddenFunction(IsNotWidgetHashConsoleVar,(int) this);
	pxOverlay->AddDialog(pxDialog);

	//special menupage selection control for use when the console var is GUIMenu.FocusedPage

	pxDialog = new Tools_GUIMenu_ConditionVar_Page_Dialog( xOffset, &m_xValue, pbChanged);
	if(pxDialogList) pxDialogList->add_entry( pxDialog );

	pxDialog->SetHelpText("The menu-page to compare the display condition against.");
	if(bEnable)
		pxDialog->SetIsEnabledFunction(IsConsoleVarSelected,(int) this);
	else
		pxDialog->SetEnabled(false);
	pxDialog->SetIsHiddenFunction(IsNotPageHashConsoleVar,(int) this);
	pxOverlay->AddDialog(pxDialog);

	xOffset.y += Asura_Dialog::LargeSpacing();
	xOffset.x = fXOffset;
}

bool Asura_GUIMenu_ConditionVar::IsConsoleVarSelected(int iThis)
{
	Asura_GUIMenu_ConditionVar* pxCondition = (Asura_GUIMenu_ConditionVar*) iThis;

	return !(pxCondition->m_xVar.IsBlank() && pxCondition->m_xCmd.IsBlank());
}

bool Asura_GUIMenu_ConditionVar::IsNotWidgetHashConsoleVar(int iThis)
{
	Asura_GUIMenu_ConditionVar* pxCondition = (Asura_GUIMenu_ConditionVar*) iThis;
	if(pxCondition->m_xVar.IsBlank()) return true;


	//check for active widget console var
	return (stricmp(pxCondition->m_xVar.GetName().GetString(),"GUIMenu.ActiveWidget")) &&
		   (stricmp(pxCondition->m_xVar.GetName().GetString(),"GUIMenu.LastActiveWidget"));
}

bool Asura_GUIMenu_ConditionVar::IsNotPageHashConsoleVar(int iThis)
{
	Asura_GUIMenu_ConditionVar* pxCondition = (Asura_GUIMenu_ConditionVar*) iThis;
	if(pxCondition->m_xVar.IsBlank()) return true;

	//check for focused page console var
	return stricmp(pxCondition->m_xVar.GetName().GetString(),"GUIMenu.FocusedPage")!=0;
}

//////////////////////////////////////////////////////////////////////////////
// Tools_GUIMenu_ConsoleVarOrCmd_Dialog
//////////////////////////////////////////////////////////////////////////////

int Tools_GUIMenu_ConsoleVarOrCmd_Dialog::s_iListValidID;

Tools_GUIMenu_ConsoleVarOrCmd_Dialog::Tools_GUIMenu_ConsoleVarOrCmd_Dialog(const char* szLabel, 
																		   const Asura_Vector_2& xOffset,
																			Asura_String* pxConsoleVar,
																			int iType,
																			const char* szBaseHelpText,
																			bool* pbModified)
:Asura_DropDown_List_Dialog(xOffset)
{
	m_iType = iType;
	m_bInitialised = false;
	m_pszConsoleVarList = 0;
	m_uConsoleVarListSize = 0;
	m_pxConsoleVarTree = 0;

	m_pbModified = pbModified;
	m_pxConsoleVar = pxConsoleVar;
	Asura_Assert(m_pxConsoleVar,"");

	m_bSelectionChanged = false;
	m_xConsoleVarBackup = *m_pxConsoleVar;

	// create single item list.
	m_uConsoleVarListSize = 1;
	m_pszConsoleVarList = new const char*[1];
	m_pszConsoleVarList[0] = "*New*";
	m_iSelectedIndex = 0;
	Initialise(xOffset,m_pszConsoleVarList,&m_iSelectedIndex,10,m_uConsoleVarListSize,NULL,&m_bSelectionChanged,szLabel);

	m_xBaseHelpText = szBaseHelpText;
	UpdateHelpText();
}

Tools_GUIMenu_ConsoleVarOrCmd_Dialog::~Tools_GUIMenu_ConsoleVarOrCmd_Dialog()
{
	DeleteConsoleVarList();
}


void Tools_GUIMenu_ConsoleVarOrCmd_Dialog::UpdateConsoleVarList()
{
	DeleteConsoleVarList();

	m_pxConsoleVarTree = new Asura_StringTree;

	//make sure the ConsoleVar being edited isn't blank
	if(m_pxConsoleVar->IsBlank())
	{
		*m_pxConsoleVar = ASURA_GUIMENU_CONSOLEVAR_NONE;
		m_xConsoleVarBackup = *m_pxConsoleVar;
	}
	//and add it to the tree
	int iJunkIndex;
	m_pxConsoleVarTree->FindNodeIndexByName(m_pxConsoleVar->GetString(),true,iJunkIndex);

	//add option for none
	m_pxConsoleVarTree->FindNodeIndexByName(ASURA_GUIMENU_CONSOLEVAR_NONE,true,iJunkIndex);


	// populate list using virtual function (for var or cmd)
	PopulateList();

	//Fill array with these strings , adding an entry for new category at the beginning
	m_uConsoleVarListSize = m_pxConsoleVarTree->GetNumberOfNodes()+1;
	m_pszConsoleVarList = new const char*[m_uConsoleVarListSize];
	m_pszConsoleVarList[0] = "*New*";
	m_pxConsoleVarTree->MakeSortedNameList(m_pszConsoleVarList+1);

	//find the index of the current string in the sorted list
	m_iSelectedIndex = 0;
	for(u_int i=1;i<m_uConsoleVarListSize;i++)
	{
		if(!m_pxConsoleVar->Compare(m_pszConsoleVarList[i]))
		{
			m_iSelectedIndex = (int) i;
			break;
		}
	}
	Asura_Assert(m_iSelectedIndex,"Failed to find category in list");

	if(m_bInitialised)
	{
		ReplaceStrings(m_uConsoleVarListSize,m_pszConsoleVarList);
	}

	m_iListValidID = s_iListValidID;
}

Tools_GUIMenu_ConsoleVar_Dialog::Tools_GUIMenu_ConsoleVar_Dialog(const Asura_Vector_2& xOffset,const char* szLabel,Asura_GUIMenu_ConsoleVar* pxVar,u_int uType,const char* szBaseHelpText,bool* pbModified)
:Tools_GUIMenu_ConsoleVarOrCmd_Dialog(szLabel,xOffset,&pxVar->GetName(),uType,szBaseHelpText,pbModified)
,m_pxVar(pxVar)
{
}

void Tools_GUIMenu_ConsoleVar_Dialog::PerFrameUpdate()
{
	Tools_GUIMenu_ConsoleVarOrCmd_Dialog::PerFrameUpdate();
	//Make sure the console var hash stays in sync with the console var name.
	//Could try just doing this when it changes , but it is less hassle just to do it each frame instead.
	m_pxVar->Update();
}


void Tools_GUIMenu_ConsoleVar_Dialog::PopulateList()
{
	int iJunkIndex;
	//TODO: add the ConsoleVars from all the loaded widgets?
//	for(u_int i=0;i<Asura_GUIMenu_Widget_Base::GetNumberOfWidgets();i++)
//	{
//		const Asura_GUIMenu_Widget_Base* pxWidget = Asura_GUIMenu_Widget_Base::GetWidgetByIndex(i);
//		const Asura_GUIMenu_ConsoleVar* pxVar = pxWidget->GetConsoleVar();
//		const char* szConsoleVar = pxVar->GetName();
//		int iType = pxVar->GetType();
//
//		if(szConsoleVar && *szConsoleVar && (eType == m_iType))
//		{
//			m_pxConsoleVarTree->FindNodeIndexByName(szConsoleVar,true,iJunkIndex);
//		}
//	}

	// add console vars from repository
	for( const Asura_CommandConsole_VarRepository::Var* pxVar = Asura_CommandConsole_VarRepository::GetRootNode(); pxVar; pxVar = pxVar->m_pxNext )
	{
		if ((pxVar->m_eType == m_iType) || (m_iType == Asura_CommandConsole_VarRepository::Var::TYPE_INVALID) )
		{
			char acName[128];
			Asura_PrintfToBfr( acName, "%S", 128, true, pxVar->m_wszName );
			m_pxConsoleVarTree->FindNodeIndexByName(acName,true,iJunkIndex);
		}
	}
}

void Tools_GUIMenu_ConsoleCmd_Dialog::PopulateList()
{
	int iJunkIndex;
	//TODO: add the ConsoleCmds from all the loaded widgets?
//	for(u_int i=0;i<Asura_GUIMenu_Widget_Base::GetNumberOfWidgets();i++)
//	{
//		const Asura_GUIMenu_Widget_Base* pxWidget = Asura_GUIMenu_Widget_Base::GetWidgetByIndex(i);
//		const Asura_GUIMenu_ConsoleCmd* pxCmd = pxWidget->GetConsoleCmd();
//		const char* szConsoleCmd = pxVar->GetName();
//		int iType = pxVar->GetType();
//
//		if(szConsoleCmd && *szConsoleCmd && (iType == m_iType))
//		{
//			m_pxConsoleVarTree->FindNodeIndexByName(szConsoleCmd,true,iJunkIndex);
//		}
//	}

	// add console cmds from repository
	const Asura_CommandConsole_CmdList::Node* pxNode = Asura_CommandConsole_CmdList::GetRootNode();
	while (pxNode)
	{
		char acName[128];
		Asura_PrintfToBfr( acName, "%S", 128, true, pxNode->GetName() );
		m_pxConsoleVarTree->FindNodeIndexByName(acName,true,iJunkIndex);

		pxNode = pxNode->GetNext();
	}
}

void Tools_GUIMenu_ConsoleVarOrCmd_Dialog::DeleteConsoleVarList()
{
	delete [] m_pszConsoleVarList;
	m_pszConsoleVarList = 0;
	m_uConsoleVarListSize = 0;
	delete m_pxConsoleVarTree;
	m_pxConsoleVarTree = 0;
}

void Tools_GUIMenu_ConsoleVarOrCmd_Dialog::OnSelectionChanged()
{
	if(m_iSelectedIndex)
	{
		//an existing ConsoleVar has been chosen
		*m_pxConsoleVar = m_pszConsoleVarList[m_iSelectedIndex];
		if(m_pbModified) *m_pbModified = true;
	}
	else
	{
		//The new ConsoleVar option has been chosen.
		//Ask for a new name
		GetString_Overlay xOverlay("New ConsoleVar (not yet hooked up in code):","");
		if(xOverlay.DoModal())
		{
			const char* szConsoleVar = xOverlay.GetString();
			if(*szConsoleVar)
			{
				*m_pxConsoleVar = szConsoleVar;
				if(m_pbModified) *m_pbModified = true;

				//Bump the list valid ID so that other ConsoleVar dialogs will be updated as well
				s_iListValidID++;
			}
		}
		//Update the list so that the new ConsoleVar (if any) is added , and so that the selection index
		//is put abck to a non-zero value.
		UpdateConsoleVarList();
	}
	m_xConsoleVarBackup = *m_pxConsoleVar;
	m_bSelectionChanged = false;

	UpdateHelpText();
}

void Tools_GUIMenu_ConsoleVarOrCmd_Dialog::UpdateHelpText()
{
	if ( m_pxConsoleVar )
	{
		if (m_pxConsoleVar->GetLength())
		{
			bool bReturn;
			Asura_UnicodeChar awcBuffer[256];
			Asura_PrintfToBfr( awcBuffer, L"help %S", 256, true, m_pxConsoleVar->GetString() );
			g_xHelpText = m_xBaseHelpText;
			g_xHelpText += "\n";
			if (m_pxConsoleVar->Compare(ASURA_GUIMENU_CONSOLEVAR_NONE))
			{
				Asura_CommandConsole::Execute( awcBuffer, &bReturn, GetHelpTextCallback );
			}
			SetHelpText( g_xHelpText.GetString() );
		}
	}
}


void Tools_GUIMenu_ConsoleVarOrCmd_Dialog::PerFrameUpdate()
{
	if (!m_bInitialised)
	{
		m_bInitialised = true;
		UpdateConsoleVarList();

		SetOnSelectCallback(OnSelectionChanged,(int) this);
	}

	//check for external changes to the ConsoleVar
	if(!m_bSelectionChanged && m_xConsoleVarBackup.Compare(*m_pxConsoleVar))
	{
		m_xConsoleVarBackup = *m_pxConsoleVar;
		UpdateConsoleVarList();
	}

	if(m_iListValidID != s_iListValidID && !IsHidden())
	{
		//update strings
		UpdateConsoleVarList();
	}

	Asura_DropDown_List_Dialog::PerFrameUpdate();
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*-----------------------------------------**
** Tools_GUIMenu_PickItem_DisplayCondition **
**-----------------------------------------*/

class Tools_GUIMenu_PickItem_DisplayCondition : public Tools_GUIMenu_PickItem
{
public :
	Tools_GUIMenu_PickItem_DisplayCondition(Tools_GUIMenu_ConditionVar_Widget_Dialog* pxDialog)
	{
		m_pxDialog = pxDialog;
	}

	virtual const char* GetPrompt(){return "Select widget for display condition";}

	virtual bool IsValidSelection(Tools_GUIMenu_Root* pxItem)
	{
		if(pxItem)
		{
			if ( pxItem->IsWidget() )
			{
				Asura_GUIMenu_Widget_Base* pxWidget = (Asura_GUIMenu_Widget_Base*) pxItem->GetAsuraData();
				return pxWidget->IsSelectable();
			}
			return false;
		}
		return true;
	}

	virtual void OnFinish(Tools_GUIMenu_Root* pxItem)
	{
		Asura_Assert(pxItem==0 || pxItem->IsWidget(),"");
		m_pxDialog->SetWidgetHash(pxItem ? pxItem->GetID() : 0);
	}

private :
	Tools_GUIMenu_ConditionVar_Widget_Dialog* m_pxDialog;
};

////////////////////////////////////////////////////////////////////////////////
/*------------------------------------------**
** Tools_GUIMenu_ConditionVar_Widget_Dialog **
**------------------------------------------*/

Tools_GUIMenu_ConditionVar_Widget_Dialog::Tools_GUIMenu_ConditionVar_Widget_Dialog(const Asura_Vector_2& xOffset,Asura_String* pxValue,bool* pbChanged)
:Asura_Function_Dialog("",xOffset,OnClicked,0)
{
	m_pxValue = pxValue;
	m_xValueBackup = *pxValue;
	m_pbChanged = pbChanged;

	//The callback user data is set here , rather than in the Asura_Function_Dialog constructor ,to avoid a warning
	SetCallback(OnClicked,(int)this);

	UpdateWidgetLabel();
}

void Tools_GUIMenu_ConditionVar_Widget_Dialog::PerFrameUpdate()
{
	Asura_Function_Dialog::PerFrameUpdate();

	//update the label if the value has changed externally
	if(m_xValueBackup != *m_pxValue)
	{
		m_xValueBackup = *m_pxValue;
		UpdateWidgetLabel();
	}
}

void Tools_GUIMenu_ConditionVar_Widget_Dialog::UpdateWidgetLabel()
{
	Asura_Hash_ID uWidgetHash = 0;
	Asura_StringUtil::GetIntValue(m_pxValue->GetString(),(int*)&uWidgetHash);
	

	char szText[100];
	if(uWidgetHash == 0)
	{
		Asura_PrintfToBfr(szText,"Widget : *None*",sizeof(szText),true);
	}
	else
	{
		const Tools_GUIMenu_Root* pxItem = Tools_Mode_GUIMenu::GetInstance()->FindItemByID(uWidgetHash);
	
		if(pxItem)
		{
			Asura_PrintfToBfr(szText,"Widget : %s",sizeof(szText),true,pxItem->GetName());
		}
		else
		{
			Asura_PrintfToBfr(szText,"Widget : *Unknown* (0x%x)",sizeof(szText),true,uWidgetHash);
		}
	}
	if(m_pxOverlay)
	{
		m_pxOverlay->UpdateDialogLabel(this,szText);
	}
	else
	{
		UpdateLabel(szText);
	}
}

void Tools_GUIMenu_ConditionVar_Widget_Dialog::OnClicked()
{
	//start widget selection
	Tools_GUIMenu_PickItem_DisplayCondition* pxPickItem = new Tools_GUIMenu_PickItem_DisplayCondition(this);
	Tools_Mode_GUIMenu::GetInstance()->StartPromptedSelection(pxPickItem);
}

void Tools_GUIMenu_ConditionVar_Widget_Dialog::SetWidgetHash(Asura_Hash_ID uHash)
{
	//convert the ID back to text
	char szBuffer[50];
	Asura_PrintfToBfr(szBuffer,"%d",sizeof(szBuffer),true,uHash);
	*m_pxValue = szBuffer;
	m_xValueBackup = szBuffer;

	UpdateWidgetLabel();

	if(m_pbChanged) *m_pbChanged = true;
}

Tools_GUIMenu_ConditionVar_Page_Dialog::Tools_GUIMenu_ConditionVar_Page_Dialog(const Asura_Vector_2& xOffset,Asura_String* pxValue,bool* pbChanged)
:Asura_Function_Dialog("",xOffset,OnClicked,0)
{
	m_pxValue = pxValue;
	m_xValueBackup = *pxValue;
	m_pbChanged = pbChanged;

	//The callback user data is set here , rather than in the Asura_Function_Dialog constructor ,to avoid a warning
	SetCallback(OnClicked,(int)this);

	UpdatePageLabel();
}

void Tools_GUIMenu_ConditionVar_Page_Dialog::PerFrameUpdate()
{
	Asura_Function_Dialog::PerFrameUpdate();

	//update the label if the value has changed externally
	if(m_xValueBackup != *m_pxValue)
	{
		m_xValueBackup = *m_pxValue;
		UpdatePageLabel();
	}
}

void Tools_GUIMenu_ConditionVar_Page_Dialog::UpdatePageLabel()
{
	Asura_Hash_ID uWidgetHash = 0;
	Asura_StringUtil::GetIntValue(m_pxValue->GetString(),(int*)&uWidgetHash);

	char szText[100];
	if(uWidgetHash == 0)
	{
		Asura_PrintfToBfr(szText,"Page : *None*",sizeof(szText),true);
	}
	else
	{
		const Tools_GUIMenu_Root* pxItem = Tools_Mode_GUIMenu::GetInstance()->FindItemByID(uWidgetHash);

		if(pxItem)
		{
			Asura_PrintfToBfr(szText,"Page : %s",sizeof(szText),true,pxItem->GetName());
		}
		else
		{
			Asura_PrintfToBfr(szText,"Page : *Unknown* (0x%x)",sizeof(szText),true,uWidgetHash);
		}
	}
	if(m_pxOverlay)
	{
		m_pxOverlay->UpdateDialogLabel(this,szText);
	}
	else
	{
		UpdateLabel(szText);
	}
}

void Tools_GUIMenu_ConditionVar_Page_Dialog::OnClicked()
{
	//start page selection
	if(m_pxValue)
	{
		ListSelection_Overlay xOverlay("Select Menu Page :");

		xOverlay.AddItem("*None*",ASURA_HASH_ID_UNSET);

		for(u_int uPage=0;uPage<Asura_GUIMenu_System::GetCurrentMenuListCount();uPage++)
		{
			Asura_GUIMenu_Menu_Page* pxPage = Asura_GUIMenu_System::GetMenuPageByIndex(uPage);
			if(pxPage)
			{
				xOverlay.AddItem(pxPage->GetNameAsChar(),pxPage->GetNameHash());
			}
			else
			{
				continue;
			}
		}

		if(!xOverlay.DoModal()) return;

		Asura_Char xIntString[20];
		Asura_PrintfToBfr(xIntString, "%d", 20, true, xOverlay.GetSelectionData());

		*m_pxValue = xIntString;
	}
}

void Tools_GUIMenu_ConditionVar_Page_Dialog::SetWidgetHash(Asura_Hash_ID uHash)
{
	//convert the ID back to text
	char szBuffer[50];
	Asura_PrintfToBfr(szBuffer,"%d",sizeof(szBuffer),true,uHash);
	*m_pxValue = szBuffer;
	m_xValueBackup = szBuffer;

	UpdatePageLabel();

	if(m_pbChanged) *m_pbChanged = true;
}
#endif // #ifdef ASURA_USE_GUI_MENU_SYSTEM
