//
// Tools_GUIMenu_Element_FMV.cpp
//
// Created by Martin Hutchings - 24/03/2009
// 
// Copyright (C) 2009 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"
#include "GUIMenu/Tools_GUIMenu_Element_FMV.h"

#ifdef ASURA_USE_GUI_MENU_SYSTEM
#include "Core/Asura_TextureManagement.h"
#include "Core/Asura_TiledTextureManagement.h"
#include "Core/Asura_UserInput.h"
#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Overlay.h"

#include "GUIMenu/Asura_GUIMenu_Element_Image.h"
#include "GUIMenu/Asura_GUIMenu_Element_FMV.h"
#include "GUIMenu/Asura_GUIMenu_System.h"
#include "GUIMenu/Asura_GUIMenu_Modifier.h"

#include "PC/FileSystem/Tools_FileSystem.h"
#include "Texturing/Tools_PickTexture.h"
#include "Texturing/Tools_Texturing_Texture.h"
#include "Texturing/Tools_Texture_Management.h"
#include "TextureTiler/Tools_PickTiledTexture.h"

static const Asura_Hash_ID g_uDialog_LeftUV = 0xBE0AC8E8; //Asura_GetHashID("LeftUV")
static const Asura_Hash_ID g_uDialog_RightUV = 0x478FB9BD; //Asura_GetHashID("RightUV")
static const Asura_Hash_ID g_uDialog_TopUV =  0x0696CEB6; //Asura_GetHashID("TopUV")
static const Asura_Hash_ID g_uDialog_BottomUV = 0x808FCA0C; //Asura_GetHashID("BottomUV")
static const Asura_Hash_ID g_uDialog_ModifierLeftUV = 0x172D0E7F; //Asura_GetHashID("ModifierLeftUV")
static const Asura_Hash_ID g_uDialog_ModifierRightUV = 0x12B62706; //Asura_GetHashID("ModifierRightUV")
static const Asura_Hash_ID g_uDialog_ModifierTopUV = 0x95D9FA3F; //Asura_GetHashID("ModifierTopUV")
static const Asura_Hash_ID g_uDialog_ModifierBottomUV = 0x1A3705E3; //Asura_GetHashID("ModifierBottomUV")

static const Asura_Hash_ID g_uDialog_Texture = 0xAB7DD51B; //Asura_GetHashID("Texture")
static const Asura_Hash_ID g_uDialog_ModifierTexture = 0x76A44264; //Asura_GetHashID("ModifierTexture")


Tools_GUIMenu_Element_FMV::Tools_GUIMenu_Element_FMV(Asura_GUIMenu_Element_FMV* pxAsuraFMV,Tools_GUIMenu_Root* pxToolsParent)
:Tools_GUIMenu_Element_Image(pxAsuraFMV,pxToolsParent)
,m_bFilenameUpdated(false)
{
	m_pxAsuraFMV = pxAsuraFMV;

	m_bFMVSettingsChanged = false;
}

Tools_GUIMenu_Element_FMV::~Tools_GUIMenu_Element_FMV()
{
}

void Tools_GUIMenu_Element_FMV::SetRectangle(const Asura_Bounding_Rectangle& xRect)
{
	PARENT::SetRectangle(xRect);
}


void Tools_GUIMenu_Element_FMV::OnDeselection()
{
	PARENT::OnDeselection();

	// Stop the FMV playing
	StopFMV();
}

void Tools_GUIMenu_Element_FMV::SelectFMV()
{
	Tools_FileSystem_DialogInfo	xFileSelector;

	xFileSelector.DefaultExtension		= "*.bik";
	xFileSelector.DefaultFilename		= "";
	xFileSelector.DefaultPath			= NULL;
	xFileSelector.FileFilter			= "Movie files (*.bik; *.wmv; *.avi)\0*.bik; *.wmv; *.avi;\0";
	xFileSelector.WindowTitle			= "Choose source FMV...";

	xFileSelector.AllowMultipleSelect	= false;

	const Asura_String	xSelectedFilename(Tools_FileSystem::GetLoadFilename(xFileSelector));

	if(!xSelectedFilename.IsBlank())
	{
		Asura_String	xWorkingDirectory;

		xWorkingDirectory	 = Asura_FileAccess::PlatformGetAsuraFilesDir();
		xWorkingDirectory	+= "\\FMV\\";

		if(xSelectedFilename.StartsWith(xWorkingDirectory.GetString()))
		{
			const u_int		uWorkingDirectoryLength	= xWorkingDirectory.GetLength();
			const u_int		uSelectedFilenameLength	= xSelectedFilename.GetLength();
			const u_int		uRelativeFilenameLength	= uSelectedFilenameLength - uWorkingDirectoryLength;

			Asura_String	xFilename(xSelectedFilename.SubString(uWorkingDirectoryLength, uRelativeFilenameLength));

			xFilename.StripExtension();

			m_xFMVName = xFilename;

			m_bFilenameUpdated = true;
		}
		else
		{
			Asura_String	xMessage;

			xMessage	 = "Chosen file is not under \"";
			xMessage	+= xWorkingDirectory;
			xMessage	+= "\").";

			ShowMessageOverlay(xMessage.GetString());
		}
	}
}

void Tools_GUIMenu_Element_FMV::PlayFMV()
{
	if( m_pxAsuraFMV )
	{
		m_pxAsuraFMV->Play();
	}
}

void Tools_GUIMenu_Element_FMV::StopFMV()
{
	if( m_pxAsuraFMV )
	{
		m_pxAsuraFMV->Stop();
	}
}

void Tools_GUIMenu_Element_FMV::RegisterFMV()
{
	// Register the changes to the FMV
	if( m_pxAsuraFMV )
	{
		// Stop FMV playing when changing settings
		StopFMV();
		m_pxAsuraFMV->RegisterFMV();
	}
}

void Tools_GUIMenu_Element_FMV::SelectFMVTextureCallback()
{
	SelectTextureCallback();

	// Register the change in texture to the FMV
	RegisterFMV();
}

void Tools_GUIMenu_Element_FMV::AddOverlayControls(Asura_Overlay* pxOverlay,Asura_Vector_2& xOffset)
{
	char szText[256];
	// Skip over parent and call grandparent function instead.
	// Don't want parent layout. Must over own unique layout.
	PARENT::PARENT::AddOverlayControls(pxOverlay,xOffset);

	Asura_Assert( m_pxAsuraFMV,"Pointer to asura image is NULL. Failed to add overlay controls" );

	if( m_pxAsuraFMV )
	{
		m_uSoundTrackIndex = m_pxAsuraFMV->GetSoundTrackIndex();
		m_bLoop = m_pxAsuraFMV->GetLoop();
		m_bSound = m_pxAsuraFMV->GetPlaySound();

		Asura_Dialog* pxDialog;

		xOffset.y += Asura_Dialog::DefaultSpacing();

		m_fRotation = m_pxAsuraFMV->GetRotation();

		pxDialog = new Asura_AngleNumerical_Dialog("Rotation", xOffset, &m_fRotation, -360.0f, 360.0f, 1.0f, "%3.3f", &m_bRotationChanged);
		pxDialog->SetIsEnabledFunction(CanEditRotation,(int)this);
		//	pxDialog->ControlID = g_uDialog_BottomUV;
		pxOverlay->AddDialog(pxDialog);
		xOffset.y += Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_FlagTickbox_Dialog("Apply Rotation before Scaling",xOffset,m_pxAsuraFMV->GetFlagsPtr(),
												Asura_GUIMenu_Element_Image::ASURA_GUIMENU_ELEMENT_IMAGE_LOCAL_FLAG_POST_ROTATE_SCALE,&m_bChanged);
		pxDialog->SetHelpText("If ticked, the image will be rotated prior to scaling.\nTherefore width and height will refer to screenspace,\nrather than texture space as normal.");
		pxDialog->SetIsEnabledFunction(CanEditRotation,(int)this);
		pxOverlay->AddDialog(pxDialog);
		xOffset.y += Asura_Dialog::LargeSpacing();

		//get text for fmv selection control		
		m_xFMVName = m_pxAsuraFMV->GetFMVFilename();
		if(m_xFMVName.IsBlank())
		{
			strcpy(szText,"FMV : <Not Set>");
		}
		else
		{
			Asura_PrintfToBfr(szText,"FMV : %s", 100, true, m_xFMVName.GetString());
		}
		pxDialog = new Asura_Function_Dialog(szText, xOffset, SelectFMV, (int)this);
		pxDialog->SetIsEnabledFunction(CanEditFMVFilename,(int)this);
		pxDialog->SetHelpText("Select the FMV filename to play.");
		pxOverlay->AddDialog(pxDialog);
		xOffset.y += Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_Function_Dialog("Texture : ",xOffset,SelectFMVTextureCallback,(int)this);
		pxDialog->SetIsEnabledFunction(CanEditTexture,(int)this);
		pxDialog->SetHelpText("Select the texture to use for this element");
		pxDialog->m_iControlID = g_uDialog_Texture;
		pxOverlay->AddDialog(pxDialog);
		xOffset.y += Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_Tickbox_Dialog("Loop", xOffset, &m_bLoop, &m_bFMVSettingsChanged);
		pxDialog->SetIsEnabledFunction(CanEditLoop,(int)this);
		pxDialog->SetHelpText("Determines whether the FMV loops");
		pxOverlay->AddDialog(pxDialog);
		xOffset.y += Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_Tickbox_Dialog("Play Sound", xOffset, &m_bSound, &m_bFMVSettingsChanged);
		pxDialog->SetIsEnabledFunction(CanEditSound,(int)this);
		pxDialog->SetHelpText("Determines whether the FMV plays sound");
		pxOverlay->AddDialog(pxDialog);
		xOffset.y += Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_Numerical_Dialog("Sound Track", xOffset, &m_uSoundTrackIndex, 0, 6, 1, "%d", &m_bFMVSettingsChanged);
		pxDialog->SetIsEnabledFunction(CanEditSoundTrack,(int)this);
		pxDialog->SetHelpText("Set the sound track to play from.");
		pxOverlay->AddDialog( pxDialog );
		xOffset.y += Asura_Dialog::DefaultSpacing();

		xOffset.y += Asura_Dialog::LargeSpacing();

		pxDialog = new Asura_Function_Dialog("Texture coordinates:", xOffset, ToggleUVEdit, (int) this);
		pxDialog->SetIsEnabledFunction(CanEditUVs,(int)this);
		pxOverlay->AddDialog(pxDialog);
		xOffset.y += Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_Tickbox_Dialog("Show pixel coords", xOffset, &s_bShowInPixels, &s_bShowInPixelsChanged);
		pxDialog->SetHelpText("Determines whether coordinates are shown between 0 to 1 , or as pixel coordinates on the texture");
		pxOverlay->AddDialog(pxDialog);
		xOffset.y += Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_ScaledNumerical_Dialog("Left", xOffset, 1.0f, &m_xUV.xUV0.u, 0.0f, 99.0f, .001f, "%.3f", &m_bChanged);
		pxDialog->SetIsEnabledFunction(CanEditUVs,(int)this);
		pxDialog->m_iControlID = g_uDialog_LeftUV;
		pxOverlay->AddDialog(pxDialog);
		xOffset.y += Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_ScaledNumerical_Dialog("Right", xOffset, 1.0f, &m_xUV.xUV1.u, 0.0f, 99.0f, .001f, "%.3f", &m_bChanged);
		pxDialog->SetIsEnabledFunction(CanEditUVs,(int)this);
		pxDialog->m_iControlID = g_uDialog_RightUV;
		pxOverlay->AddDialog(pxDialog);
		xOffset.y += Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_ScaledNumerical_Dialog("Top", xOffset, 1.0f, &m_xUV.xUV0.v, 0.0f, 99.0f, .001f, "%.3f", &m_bChanged);
		pxDialog->SetIsEnabledFunction(CanEditUVs,(int)this);
		pxDialog->m_iControlID = g_uDialog_TopUV;
		pxOverlay->AddDialog(pxDialog);
		xOffset.y += Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_ScaledNumerical_Dialog("Bottom", xOffset, 1.0f, &m_xUV.xUV1.v, 0.0f, 99.0f, .001f, "%.3f", &m_bChanged);
		pxDialog->SetIsEnabledFunction(CanEditUVs,(int)this);
		pxDialog->m_iControlID = g_uDialog_BottomUV;
		pxOverlay->AddDialog(pxDialog);
		xOffset.y += Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_FlagTickbox_Dialog("Lock UV to dims",xOffset,&m_uToolsFlags,ASURA_GUIMENU_TOOLS_FLAG_LOCKUVTODIMS,&m_bChanged);
		pxDialog->SetIsEnabledFunction(CanLockUVToDims,(int)this);
		pxDialog->SetHelpText("If ticked then the UV coordinates and width/height will be kept in sync while editing, so changing one will change the other.");
		pxOverlay->AddDialog(pxDialog);
		xOffset.y += Asura_Dialog::DefaultSpacing();

		xOffset.y += Asura_Dialog::DefaultSpacing();
		pxDialog = new Asura_StringDisplay_Dialog( ".:: Animator Control ::.", xOffset);
		pxOverlay->AddDialog(pxDialog);
		xOffset.y += Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_FlagTickbox_Dialog("Scale to origin",xOffset,m_pxAsuraFMV->GetFlagsPtr(),
												Asura_GUIMenu_Element_Image::ASURA_GUIMENU_ELEMENT_IMAGE_LOCAL_FLAG_SCALETOORIGIN,&m_bChanged);
		pxDialog->SetHelpText("If ticked then width or height scaling by animators will be centered on the position origin.");
		pxOverlay->AddDialog(pxDialog);

		xOffset.y += Asura_Dialog::LargeSpacing();

		pxDialog = new Asura_Function_Dialog("Play", xOffset, PlayFMV, (int)this);
		pxDialog->SetIsEnabledFunction( CanPlayStop,(int)this );
		pxOverlay->AddDialog(pxDialog);

		xOffset.y += Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_Function_Dialog("Stop", xOffset, StopFMV, (int)this);
		pxDialog->SetIsEnabledFunction( CanPlayStop,(int)this );
		pxOverlay->AddDialog(pxDialog);

		xOffset.y += Asura_Dialog::DefaultSpacing();
	}
}

bool Tools_GUIMenu_Element_FMV::CanPlayStop()
{
	if( m_pxAsuraFMV )
	{
		return m_pxAsuraFMV->IsFMVValid();
	}

	return false;
}

bool Tools_GUIMenu_Element_FMV::CanEditFMVFilename()
{
	if( m_pxAsuraFMV )
	{
		// Can only set the FMV filename if the object
		// hasn't been created from a style
		if ( m_pxAsuraFMV->GetStyleObjectHash() == ASURA_HASH_ID_UNSET )
		{
			return true;
		}
	}

	return false;
}

bool Tools_GUIMenu_Element_FMV::CanEditLoop()
{
	if( m_pxAsuraFMV )
	{
		// Can only set the loop flag if the object
		// hasn't been create from a style
		if ( m_pxAsuraFMV->GetStyleObjectHash() == ASURA_HASH_ID_UNSET )
		{
			return true;
		}
	}

	return false;
}

bool Tools_GUIMenu_Element_FMV::CanEditSound()
{
	if( m_pxAsuraFMV )
	{
		// Can only enable/disable sound if the object
		// hasn't been create from a style
		if ( m_pxAsuraFMV->GetStyleObjectHash() == ASURA_HASH_ID_UNSET )
		{
			return true;
		}
	}

	return false;
}

bool Tools_GUIMenu_Element_FMV::CanEditSoundTrack()
{
	if( m_pxAsuraFMV )
	{
		// Can only edit sound track if sound is enabled and the object
		// hasn't been create from a style
		if ( m_pxAsuraFMV->GetStyleObjectHash() == ASURA_HASH_ID_UNSET )
		{
			return m_bSound;
		}
	}

	return false;
}

bool Tools_GUIMenu_Element_FMV::CanCloseWhenComplete()
{
	if( m_pxAsuraFMV )
	{
		// The FMV can only close when it gets to the end if it not looping
		return (!m_bLoop);
	}

	return false;
}


void Tools_GUIMenu_Element_FMV::UpdateOverlayControls(Asura_Overlay* pxOverlay)
{
	PARENT::UpdateOverlayControls(pxOverlay);
}

bool Tools_GUIMenu_Element_FMV::UsesTexture(int iTexture) const
{
//	if(m_pxAsuraFMV->GetTexture() == (u_int)iTexture) return true;
// TODO
	return PARENT::UsesTexture(iTexture);
}

void Tools_GUIMenu_Element_FMV::CopyToAsura()
{
	PARENT::CopyToAsura();
}

void Tools_GUIMenu_Element_FMV::CopyFromAsura()
{
	PARENT::CopyFromAsura();
}

void Tools_GUIMenu_Element_FMV::Update()
{
	PARENT::Update();

	if ( m_bFilenameUpdated == true )
	{
		m_pxAsuraFMV->SetFMVFilename(m_xFMVName);

		RegisterFMV();

		SetModified();

		//update the FMV overlay
		DeleteOverlay();
		CreateOverlay();

		m_bFilenameUpdated = false;
	}

	if ( m_bFMVSettingsChanged )
	{
		m_pxAsuraFMV->SetLoop(m_bLoop);
		m_pxAsuraFMV->SetPlaySound(m_bSound);

		m_pxAsuraFMV->SetSoundTrackIndex(m_uSoundTrackIndex);

		RegisterFMV();
		SetModified();
		m_bFMVSettingsChanged = false;
	}
}

void Tools_GUIMenu_Element_FMV::RenderUI()
{
	PARENT::RenderUI();
}

bool Tools_GUIMenu_Element_FMV::IsUsingKey(int iKey)
{
	return PARENT::IsUsingKey(iKey);
}

void Tools_GUIMenu_Element_FMV::Copy(const Tools_GUIMenu_Root* pxSource)
{
	PARENT::Copy(pxSource);
}

const char* Tools_GUIMenu_Element_FMV::GetLocalDataFlagName(u_int uFlagValue) const
{
	switch(uFlagValue)
	{
		default :
			return PARENT::GetLocalDataFlagName(uFlagValue);
	}
}

// returns whether this object supports e.g. position, or UVs etc.
bool Tools_GUIMenu_Element_FMV::IsModifierTypeSupported( ASURA_GUIMENU_MODIFY_TYPE eType ) const
{
	switch (eType)
	{
		default:
			return PARENT::IsModifierTypeSupported(eType);
	}
}

void Tools_GUIMenu_Element_FMV::AddModifierOverlayControls(Asura_GUIMenu_Modifier* pxModifier, Asura_Vector_2& xOffset )
{
	switch(pxModifier->GetModifyType())
	{
		default :
			PARENT::AddModifierOverlayControls(pxModifier,xOffset);
			break;
	}
}

void Tools_GUIMenu_Element_FMV::InitialiseModifier(Asura_GUIMenu_Modifier* pxModifier)
{
	switch(pxModifier->GetModifyType())
	{
		default :
			PARENT::InitialiseModifier(pxModifier);
			break;
	}
}

void Tools_GUIMenu_Element_FMV::CleanupModifierEditing()
{
	PARENT::CleanupModifierEditing();
}

#endif //ASURA_USE_GUI_MENU_SYSTEM
