//
// Tools_GUIMenu_Element_Image.cpp
//
// Created by Richard Rice - 05/12/2006
// 
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"
#include "GUIMenu/Tools_GUIMenu_Element_Image.h"

#ifdef ASURA_USE_GUI_MENU_SYSTEM
#include "ChunkSystem/Asura_Chunk_Loaders.h"
#include "Core/Asura_TextureManagement.h"
#include "Core/Asura_TiledTextureManagement.h"
#include "Core/Asura_UserInput.h"
#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Overlay.h"

//#include "GUIMenu/Asura_GUIMenu_Root.h"
//#include "GUIMenu/Asura_GUIMenu_Element_Base.h"
#include "GUIMenu/Asura_GUIMenu_Element_Image.h"
#include "GUIMenu/Asura_GUIMenu_System.h"
#include "GUIMenu/Asura_GUIMenu_Modifier.h"
#include "Misc/Tools_Timed_Messages.h"
#include "PC/FileSystem/Tools_FileSystem.h"
#include "Texturing/Tools_PickTexture.h"
#include "Texturing/Tools_Texturing_Texture.h"
#include "Texturing/Tools_Texture_Management.h"
#include "TextureTiler/Tools_PickTiledTexture.h"

template<typename T> void swap( T& A, T& B )
{
	T tmp = A;

	A = B;
	B = tmp;
}

bool Tools_GUIMenu_Element_Image::s_bShowInPixels = true;
bool Tools_GUIMenu_Element_Image::s_bShowInPixelsChanged = false;

static const Asura_Hash_ID g_uDialog_LeftUV				= 0xBE0AC8E8; //Asura_GetHashID("LeftUV")
static const Asura_Hash_ID g_uDialog_RightUV			= 0x478FB9BD; //Asura_GetHashID("RightUV")
static const Asura_Hash_ID g_uDialog_TopUV				= 0x0696CEB6; //Asura_GetHashID("TopUV")
static const Asura_Hash_ID g_uDialog_BottomUV			= 0x808FCA0C; //Asura_GetHashID("BottomUV")
static const Asura_Hash_ID g_uDialog_ModifierLeftUV		= 0x172D0E7F; //Asura_GetHashID("ModifierLeftUV")
static const Asura_Hash_ID g_uDialog_ModifierRightUV	= 0x12B62706; //Asura_GetHashID("ModifierRightUV")
static const Asura_Hash_ID g_uDialog_ModifierTopUV		= 0x95D9FA3F; //Asura_GetHashID("ModifierTopUV")
static const Asura_Hash_ID g_uDialog_ModifierBottomUV	= 0x1A3705E3; //Asura_GetHashID("ModifierBottomUV")

static const Asura_Hash_ID g_uDialog_Texture = 0xAB7DD51B; //Asura_GetHashID("Texture")
static const Asura_Hash_ID g_uDialog_ModifierTexture = 0x76A44264; //Asura_GetHashID("ModifierTexture")


Tools_GUIMenu_Element_Image::Tools_GUIMenu_Element_Image(Asura_GUIMenu_Element_Image* pxAsuraImage,Tools_GUIMenu_Root* pxToolsParent)
:Tools_GUIMenu_Element_Base(pxAsuraImage,pxToolsParent)
,m_pxUVEditUVs(0)
,m_pxUVEdit(0)
,m_bEditingModifierUV(false)
,m_fRotation( 0.0)
,m_bRotationChanged(false)
,m_bAtemptedTileFileLoad(false)
{
	Asura_Assert( pxAsuraImage,"Pointer to proxy image object should not be NULL" );

	if( pxAsuraImage )
	{
		m_pxAsuraImage = pxAsuraImage;

		m_xUV.xUV0 = m_pxAsuraImage->GetUV( 0);
		m_xUV.xUV1 = m_pxAsuraImage->GetUV( 1);

		m_fRotation = m_pxAsuraImage->GetRotation();
		
		// fix up old data (pre chunk version 3) by trying to guess tiled texture name file from texture file name - NDB
		if( IsTile() && (m_pxAsuraImage->GetTileFileName().IsNull() || m_pxAsuraImage->GetTileFileName().IsBlank()) && !(m_pxAsuraImage->GetTextureFileName().IsBlank()) )
		{
			Asura_String xFileName( m_pxAsuraImage->GetTextureFileName() );

			xFileName.Replace( "_texture.tga",".asr" );

			if( Asura_FileAccess::FileExists( xFileName.GetString() ) )
			{
				m_pxAsuraImage->SetTileFileName( xFileName );
			}
		}

		// fix up old data that mistakenly contains full path to tile file - NDB
		if( !m_pxAsuraImage->GetTileFileName().IsNull() && !m_pxAsuraImage->GetTileFileName().IsBlank() )
		{
			m_pxAsuraImage->SetTileFileName( Asura_GUIMenu_Element_Image::StripProjectPathFromGraphicsFileName( m_pxAsuraImage->GetTileFileName() ) );
		}
	}
	
	UpdateFromTile();
	UpdateTextureSize();
}

Tools_GUIMenu_Element_Image::~Tools_GUIMenu_Element_Image()
{
	FinishUVEdit();	
}

void Tools_GUIMenu_Element_Image::SetRectangle(const Asura_Bounding_Rectangle& xRect)
{
	PARENT::SetRectangle(xRect);

	if((m_uToolsFlags & ASURA_GUIMENU_TOOLS_FLAG_LOCKUVTODIMS) && m_bTextureSizeValid)
	{
		//update UVs to match the width/height
		float fWidth = xRect.MaxX - xRect.MinX;
		float fHeight = xRect.MaxY - xRect.MinY;

		float fUVWidth = fWidth/m_iTextureWidth;
		float fUVHeight = fHeight/m_iTextureHeight;
		
		//negate width/height if needed to preserve mirroring
		if(m_xUV.xUV1.u < m_xUV.xUV0.u) fUVWidth = -fUVWidth;
		if(m_xUV.xUV1.v < m_xUV.xUV0.v) fUVHeight = -fUVHeight;


		m_xUV.xUV1.u = m_xUV.xUV0.u + fUVWidth;
		m_xUV.xUV1.v = m_xUV.xUV0.v + fUVHeight;

		m_pxAsuraImage->SetUV(0,m_xUV.xUV0);
		m_pxAsuraImage->SetUV(1,m_xUV.xUV1);
	}
}


void Tools_GUIMenu_Element_Image::OnDeselection()
{
	FinishUVEdit();
	PARENT::OnDeselection();
}

void Tools_GUIMenu_Element_Image::AddOverlayControls(Asura_Overlay* pxOverlay,Asura_Vector_2& xOffset)
{
	PARENT::AddOverlayControls(pxOverlay,xOffset);

	Asura_Assert( m_pxAsuraImage,"Pointer to asura image is NULL. Failed to add overlay controls" );

	if( m_pxAsuraImage )
	{
		Asura_Dialog* pxDialog;

		xOffset.y += Asura_Dialog::DefaultSpacing();

		m_fRotation = m_pxAsuraImage->GetRotation();

		pxDialog = new Asura_AngleNumerical_Dialog("Rotation", xOffset, &m_fRotation, -360.0f, 360.0f, 1.0f, "%3.3f", &m_bRotationChanged);
		pxDialog->SetIsEnabledFunction(CanEditRotation,(int)this);
		//	pxDialog->ControlID = g_uDialog_BottomUV;
		pxOverlay->AddDialog(pxDialog);
		xOffset.y += Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_FlagTickbox_Dialog("Apply Rotation before Scaling",xOffset,m_pxAsuraImage->GetFlagsPtr(),
												Asura_GUIMenu_Element_Image::ASURA_GUIMENU_ELEMENT_IMAGE_LOCAL_FLAG_POST_ROTATE_SCALE,&m_bChanged);
		pxDialog->SetHelpText("If ticked, the image will be rotated prior to scaling.\nTherefore width and height will refer to screenspace,\nrather than texture space as normal.");
		pxDialog->SetIsEnabledFunction(CanEditRotation,(int)this);
		pxOverlay->AddDialog(pxDialog);
		xOffset.y += Asura_Dialog::LargeSpacing();

		pxDialog = new Asura_Function_Dialog("Texture : ",xOffset,SelectTextureCallback,(int)this);
		pxDialog->SetIsEnabledFunction(CanEditTexture,(int)this);
		pxDialog->SetHelpText("Select the texture to use for this element");
		pxDialog->m_iControlID = g_uDialog_Texture;
		pxOverlay->AddDialog(pxDialog);
		xOffset.y += Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_Function_Dialog("Choose Tiled Texture",xOffset,SelectTiledTextureCallback,(int)this);
		pxDialog->SetIsEnabledFunction(CanEditTexture,(int)this);
		pxDialog->SetHelpText("Select the texture tile to use for this element");
		pxOverlay->AddDialog(pxDialog);
		xOffset.y += Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_Function_Dialog("Texture coordinates:", xOffset, ToggleUVEdit, (int) this);
		pxDialog->SetIsEnabledFunction(CanEditUVs,(int)this);
		pxOverlay->AddDialog(pxDialog);
		xOffset.y += Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_Tickbox_Dialog("Show pixel coords", xOffset, &s_bShowInPixels, &s_bShowInPixelsChanged);
		pxDialog->SetHelpText("Determines whether coordinates are shown between 0 to 1 , or as pixel coordinates on the texture");
		pxOverlay->AddDialog(pxDialog);
		xOffset.y += Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_ScaledNumerical_Dialog("Left", xOffset, 1.0f, &m_xUV.xUV0.u, 0.0f, 99.0f, .001f, "%.3f", &m_bChanged);
		pxDialog->SetIsEnabledFunction(CanEditUVs,(int)this);
		pxDialog->m_iControlID = g_uDialog_LeftUV;
		pxOverlay->AddDialog(pxDialog);
		xOffset.y += Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_ScaledNumerical_Dialog("Right", xOffset, 1.0f, &m_xUV.xUV1.u, 0.0f, 99.0f, .001f, "%.3f", &m_bChanged);
		pxDialog->SetIsEnabledFunction(CanEditUVs,(int)this);
		pxDialog->m_iControlID = g_uDialog_RightUV;
		pxOverlay->AddDialog(pxDialog);
		xOffset.y += Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_ScaledNumerical_Dialog("Top", xOffset, 1.0f, &m_xUV.xUV0.v, 0.0f, 99.0f, .001f, "%.3f", &m_bChanged);
		pxDialog->SetIsEnabledFunction(CanEditUVs,(int)this);
		pxDialog->m_iControlID = g_uDialog_TopUV;
		pxOverlay->AddDialog(pxDialog);
		xOffset.y += Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_ScaledNumerical_Dialog("Bottom", xOffset, 1.0f, &m_xUV.xUV1.v, 0.0f, 99.0f, .001f, "%.3f", &m_bChanged);
		pxDialog->SetIsEnabledFunction(CanEditUVs,(int)this);
		pxDialog->m_iControlID = g_uDialog_BottomUV;
		pxOverlay->AddDialog(pxDialog);
		xOffset.y += Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_FlagTickbox_Dialog("Lock UV to dims",xOffset,&m_uToolsFlags,ASURA_GUIMENU_TOOLS_FLAG_LOCKUVTODIMS,&m_bChanged);
		pxDialog->SetIsEnabledFunction(CanLockUVToDims,(int)this);
		pxDialog->SetHelpText("If ticked then the UV coordinates and width/height will be kept in sync while editing, so changing one will change the other.");
		pxOverlay->AddDialog(pxDialog);
		xOffset.y += Asura_Dialog::DefaultSpacing();

		xOffset.y += Asura_Dialog::DefaultSpacing();
		pxDialog = new Asura_StringDisplay_Dialog( ".:: Tile Control ::.", xOffset);
		pxDialog->SetIsEnabledFunction( IsTile,(int)this );
		pxOverlay->AddDialog(pxDialog);
		xOffset.y += Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_FlagTickbox_Dialog("Flip tile in X",xOffset,m_pxAsuraImage->GetFlagsPtr(),
												Asura_GUIMenu_Element_Image::ASURA_GUIMENU_ELEMENT_IMAGE_LOCAL_FLAG_INVERT_X,&m_bChanged);
		pxDialog->SetHelpText("If ticked, the min and max U coordinates of the selected tile are swapped");
		pxDialog->SetIsEnabledFunction( CanFlipTile,(int)this );
		pxOverlay->AddDialog(pxDialog);
		xOffset.y += Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_FlagTickbox_Dialog("Flip tile in Y",xOffset,m_pxAsuraImage->GetFlagsPtr(),
												Asura_GUIMenu_Element_Image::ASURA_GUIMENU_ELEMENT_IMAGE_LOCAL_FLAG_INVERT_Y,&m_bChanged);
		pxDialog->SetHelpText("If ticked, the min and max V coordinates of the selected tile are swapped");
		pxDialog->SetIsEnabledFunction( CanFlipTile,(int)this );
		pxOverlay->AddDialog(pxDialog);
		xOffset.y += Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_FlagTickbox_Dialog("Ignore Tile UV",xOffset,m_pxAsuraImage->GetFlagsPtr(),
												Asura_GUIMenu_Element_Image::ASURA_GUIMENU_ELEMENT_IMAGE_LOCAL_FLAG_IGNORETILEUV,&m_bChanged);
		pxDialog->SetHelpText("If ticked then tiled texture uv coordinates will be overriden by user defined uv values.");
		pxDialog->SetIsEnabledFunction( IsTile,(int)this );
		pxOverlay->AddDialog(pxDialog);
		xOffset.y += Asura_Dialog::DefaultSpacing();

		xOffset.y += Asura_Dialog::DefaultSpacing();
		pxDialog = new Asura_StringDisplay_Dialog( ".:: Animator Control ::.", xOffset);
		pxOverlay->AddDialog(pxDialog);
		xOffset.y += Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_FlagTickbox_Dialog("Scale to origin",xOffset,m_pxAsuraImage->GetFlagsPtr(),
												Asura_GUIMenu_Element_Image::ASURA_GUIMENU_ELEMENT_IMAGE_LOCAL_FLAG_SCALETOORIGIN,&m_bChanged);
		pxDialog->SetHelpText("If ticked then width or height scaling by animators will be centered on the position origin.");
		pxOverlay->AddDialog(pxDialog);
		xOffset.y += Asura_Dialog::DefaultSpacing();

		xOffset.y += Asura_Dialog::DefaultSpacing();
		pxDialog = new Asura_StringDisplay_Dialog( ".:: Resource Control ::.", xOffset);
		pxOverlay->AddDialog(pxDialog);
		xOffset.y += Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_FlagTickbox_Dialog("Export Resource",xOffset,&m_uToolsFlags,ASURA_GUIMENU_TOOLS_FLAG_EXPORTRESOURCES,&m_bChanged);
		pxDialog->SetHelpText("If ticked then any texture or tiled texture files refferenced by this element will be added to the GUI menu file at export.");
		pxOverlay->AddDialog(pxDialog);
		xOffset.y += Asura_Dialog::DefaultSpacing();
	}
}

void Tools_GUIMenu_Element_Image::UpdateOverlayControls(Asura_Overlay* pxOverlay)
{
	PARENT::UpdateOverlayControls(pxOverlay);
	
	//update texture selection control
	char	szText[100];
	
	const char* pcFileName(NULL);

	if( IsTile() )
	{
		// display tiled texture file name
		const Asura_String* pxTileFileName( NULL );
		
		if( m_pxAsuraImage->GetTileHash() != ASURA_HASH_ID_UNSET )
		{
			pxTileFileName = Asura_TiledTextureManagement::GetAsuraFileNameByHash( m_pxAsuraImage->GetTileHash() );
		}

		if( pxTileFileName )
		{
			pcFileName = Tools_FileSystem::StripPath( pxTileFileName->GetString() );
		}

		if( !pcFileName )
		{
			pcFileName = "*None*";
		}

		Asura_PrintfToBfr( szText,"Texture: %s (Tile)",sizeof(szText),true,pcFileName );
		
	}
	else
	{
		// display texture file name
		const Asura_String* pxFullFilename( Asura_Texture_Management::GetTextureName(m_pxAsuraImage->GetTexture()) );

		if( pxFullFilename && !pxFullFilename->IsNull() && !pxFullFilename->IsBlank() )
		{
			pcFileName = Tools_FileSystem::StripPath(pxFullFilename->GetString());
		}

		if( !pcFileName )
		{
			pcFileName = "*None*";
		}			

		Asura_PrintfToBfr( szText,"Texture: %s",sizeof(szText),true,pcFileName );
		
	}

	pxOverlay->UpdateDialogLabel(g_uDialog_Texture,szText);

	//and for the modifier texture
	if(pxOverlay->GetDialog(g_uDialog_ModifierTexture))
	{
		const int iTextureIndex( m_iModifierData );

		if(iTextureIndex == ASURA_TEXTURE_NONE)
		{
			strcpy(szText,"Texture: *None*");
		}
		else
		{
			const Asura_String* pxFullFilename = Asura_Texture_Management::GetTextureName(iTextureIndex);
			Asura_Assert(pxFullFilename,"");
			const char* szFilename = Tools_FileSystem::StripPath(pxFullFilename->GetString());

			Asura_PrintfToBfr(szText,"Texture: %s",sizeof(szText),true,szFilename);
		}
		pxOverlay->UpdateDialogLabel(g_uDialog_ModifierTexture,szText);
		
	}

	//update scale and format for UV editing controls
	const char* szUVPrintFormat = s_bShowInPixels ? "%.0f" : "%.3f";
	float fUVXScale = s_bShowInPixels ? static_cast<float>(m_iTextureWidth) : 1.0f;
	float fUVYScale = s_bShowInPixels ? static_cast<float>(m_iTextureHeight) : 1.0f;
	float fUVMin = 0.0f;
	float fUVMax = s_bShowInPixels ? 9999.0f : 99.0f;
	float fUVDelta = s_bShowInPixels ? 1.0f : 0.001f;

	Asura_Hash_ID auUVDialogID[8] = {g_uDialog_LeftUV,g_uDialog_TopUV,g_uDialog_RightUV,g_uDialog_BottomUV,
									g_uDialog_ModifierLeftUV,g_uDialog_ModifierTopUV,g_uDialog_ModifierRightUV,g_uDialog_ModifierBottomUV};
	for(int i=0;i<8;i++)
	{
		Asura_ScaledNumerical_Dialog* pxDialog = (Asura_ScaledNumerical_Dialog*) pxOverlay->GetDialog(auUVDialogID[i]);
		if(pxDialog)
		{
			pxDialog->SetScale((i&1) ? fUVYScale : fUVXScale, fUVMin, fUVMax);
			pxDialog->SetDelta(fUVDelta);
			pxDialog->SetPrintFormat(szUVPrintFormat);
		}
	}
}

bool Tools_GUIMenu_Element_Image::UsesTexture(int iTexture) const
{
	if(m_pxAsuraImage->GetTexture() == iTexture) return true;

	return PARENT::UsesTexture(iTexture);
}

void Tools_GUIMenu_Element_Image::SelectTextureCallback()
{
	Asura_String xTexName;
	int iTextureIndex = m_pxAsuraImage->GetTexture();

	//select a new texture
	iTextureIndex = SelectTexture(iTextureIndex);

	const Asura_String* pxFullFilename = Asura_Texture_Management::GetTextureName(iTextureIndex);
	
	if(pxFullFilename)
	{
		xTexName = *pxFullFilename;
	}

	m_pxAsuraImage->SetTile( ASURA_HASH_ID_UNSET );
	m_pxAsuraImage->SetTexture( xTexName, true);

	//update the overlay
	UpdateTextureSize();
	UpdateOverlay();

	//If doing UV editing , update the texture there as well
	if(m_pxUVEdit)
	{
		m_pxUVEdit->SetTextureID(iTextureIndex);
	}

	SetModified();
}

void Tools_GUIMenu_Element_Image::SelectTiledTextureCallback()
{
	int iCurrentTileIndex(0);

	if( IsTile() )
	{
		iCurrentTileIndex = Asura_TiledTextureManagement::GetHandleOfTile( m_pxAsuraImage->GetTileHash() );
	}

	const int iTileIndex( SelectTiledTexture(iCurrentTileIndex) );

	const Asura_TiledTextureInfo* pxTileInfo( Asura_TiledTextureManagement::GetTileInfoByID(iTileIndex) );
	
	Asura_Assert( pxTileInfo != NULL,"Tile not found in tile management" );

	if( pxTileInfo == NULL )
	{
		return;
	}

	m_pxAsuraImage->SetTile( pxTileInfo->m_uTileHashID );

	// set GUI menu mode UV from tile
	UpdateFromTile();
	
	//update the overlay
	UpdateTextureSize();
	UpdateOverlay();

	//If doing UV editing, update the texture there as well
	if(m_pxUVEdit)
	{
		m_pxUVEdit->SetTextureID( m_pxAsuraImage->GetTexture() );
	}

	SetModified();
}

void Tools_GUIMenu_Element_Image::SelectModifierTexture()
{
	m_iModifierData = SelectTexture(m_iModifierData);

	UpdateOverlay();
	m_bModifierChanged = true;
}

int Tools_GUIMenu_Element_Image::SelectTexture(int iInitialTexture, const char* szTitle /*=0*/)
{
	//build list of textures that are currently being used
	List<int> xTextureList;
	Asura_GUIMenu_System::TraverseUsedTextures(ListTexturesCallback,&xTextureList);

	if(!szTitle) szTitle = "Select texture for element";

	Tools_PickTexture::SetTextureFilterCallback(IsMenuTexture,&xTextureList);
	int iTextureIndex = Tools_PickTexture::PickTexture(szTitle,iInitialTexture,PICKTEXTURE_USECALLBACKFILTER,Asura_GUIMenu_Element_Image::GetDefaultTextureFlags());
	Tools_PickTexture::SetTextureFilterCallback(0);

	return (iTextureIndex == ASURA_TEXTURE_NONE) ? iInitialTexture : iTextureIndex;
}

int Tools_GUIMenu_Element_Image::SelectTiledTexture(int iInitialTile)
{
	const int iSelectedTile( Tools_PickTiledTexture::PickTiledTexture( "Choose A Tile For Menu Image",iInitialTile ) );

	return iSelectedTile;
}

void Tools_GUIMenu_Element_Image::CopyToAsura()
{
	PARENT::CopyToAsura();

	//update UVs
	m_pxAsuraImage->SetUV(0,m_xUV.xUV0);
	m_pxAsuraImage->SetUV(1,m_xUV.xUV1);

	if((m_uToolsFlags & ASURA_GUIMENU_TOOLS_FLAG_LOCKUVTODIMS) && m_bTextureSizeValid)
	{
		if(CanLockUVToDims())
		{
			//keep the width/height in sync with the UVs
			float fUVWidth = m_xUV.xUV1.u - m_xUV.xUV0.u;
			float fUVHeight = m_xUV.xUV1.v - m_xUV.xUV0.v;

			float fWidth = static_cast<float>( Asura_Maths::FloatToInt_Near(fUVWidth * m_iTextureWidth) );
			float fHeight = static_cast<float>( Asura_Maths::FloatToInt_Near(fUVHeight * m_iTextureHeight) );

			Asura_Bounding_Rectangle xRect = GetRectangle();
			xRect.MaxX = xRect.MinX + Asura_Maths::Abs(fWidth);
			xRect.MaxY = xRect.MinY + Asura_Maths::Abs(fHeight);

			if(xRect != GetRectangle())
			{
				PARENT::SetRectangle(xRect);
			}
		}
		else
		{
			m_uToolsFlags &=~ASURA_GUIMENU_TOOLS_FLAG_LOCKUVTODIMS;
			m_pxAsuraImage->SetToolsFlags(m_uToolsFlags);
		}
	} 

	m_pxAsuraImage->SetRotation( m_fRotation);

	// copy Tools instance of 'tools flags' to Asura instance of 'tools flags' (could be better) - NDB
	m_pxAsuraImage->SetToolsFlags( m_uToolsFlags );
}

void Tools_GUIMenu_Element_Image::CopyFromAsura()
{
	PARENT::CopyFromAsura();

	//update UVs
	m_xUV.xUV0 = m_pxAsuraImage->GetUV( 0);
	m_xUV.xUV1 = m_pxAsuraImage->GetUV( 1);
	
	UpdateFromTile();
	UpdateTextureSize();

	m_fRotation = m_pxAsuraImage->GetRotation();

	// copy Asura instance of 'tools flags' to Tools instance of 'tools flags' (could be better) - NDB
	m_uToolsFlags = m_pxAsuraImage->GetToolsFlags();
}


void Tools_GUIMenu_Element_Image::ToggleUVEdit()
{
	if(m_pxUVEdit && m_bEditingModifierUV) FinishUVEdit();
	
	if(m_pxUVEdit)
	{
		FinishUVEdit();
	}
	else
	{
		StartUVEdit(&m_xUV,false);
	}
}

void Tools_GUIMenu_Element_Image::ToggleModifierUVEdit()
{
	if(m_pxUVEdit && !m_bEditingModifierUV) FinishUVEdit();

	if(m_pxUVEdit)
	{
		FinishUVEdit();
	}
	else
	{
		StartUVEdit(&m_xModifierData_UV,true);
	}
}


void Tools_GUIMenu_Element_Image::StartUVEdit(Asura_GUIMenu_UV_Set* pxUVs,bool bModifierUV)
{
	//allocate the uv editing thing
	m_pxUVEdit = new Tools_UV_Editor;
	m_bEditingModifierUV = bModifierUV;
	m_pxUVEditUVs = pxUVs;
	const int iTextureID = m_pxAsuraImage->GetTexture();
	m_pxUVEdit->SetTextureID(iTextureID);

	//initalise the uv coordinates
	m_pxUVEdit->Nodes[0].x = m_pxUVEditUVs->xUV0.u;
	m_pxUVEdit->Nodes[0].y = m_pxUVEditUVs->xUV0.v;
	m_pxUVEdit->Nodes[1].x = m_pxUVEditUVs->xUV1.u;
	m_pxUVEdit->Nodes[1].y = m_pxUVEditUVs->xUV0.v;
	m_pxUVEdit->Nodes[2].x = m_pxUVEditUVs->xUV1.u;
	m_pxUVEdit->Nodes[2].y = m_pxUVEditUVs->xUV1.v;
	m_pxUVEdit->Nodes[3].x = m_pxUVEditUVs->xUV0.u; 
	m_pxUVEdit->Nodes[3].y = m_pxUVEditUVs->xUV1.v; 
	

	m_pxUVEdit->CreateMouseModeOverlay();
}

void Tools_GUIMenu_Element_Image::FinishUVEdit()
{
	if(m_pxUVEdit)
	{
		m_pxUVEdit->CloseMouseModeOverlay();
		delete m_pxUVEdit;
		m_pxUVEdit = 0;
	}
	m_pxUVEditUVs = 0;
	m_bEditingModifierUV = false;
}

void Tools_GUIMenu_Element_Image::UpdateUVEdit()
{
	if(!m_pxUVEdit) return;

	m_pxUVEdit->HandleMouse();


	if(m_pxUVEdit->NodesMoved)
	{
		//make sure the nodes stay in rectangular formation
		m_pxUVEdit->EnsureNodesAreStillOkayForTextureWrap();
		
		m_pxUVEdit->Nodes[1].x = m_pxUVEdit->Nodes[2].x;
		m_pxUVEdit->Nodes[1].y = m_pxUVEdit->Nodes[0].y;

		m_pxUVEdit->Nodes[3].x = m_pxUVEdit->Nodes[0].x;
		m_pxUVEdit->Nodes[3].y = m_pxUVEdit->Nodes[2].y;


		m_pxUVEdit->NodesMoved = false;


		m_pxUVEditUVs->xUV0.u = m_pxUVEdit->Nodes[0].x; 
		m_pxUVEditUVs->xUV0.v  = m_pxUVEdit->Nodes[0].y; 
		m_pxUVEditUVs->xUV1.u  = m_pxUVEdit->Nodes[2].x; 
		m_pxUVEditUVs->xUV1.v  = m_pxUVEdit->Nodes[2].y;
		if(m_bEditingModifierUV)
		{
			m_bModifierChanged = true;
		}
		else
		{
			m_bChanged = true;
		}

	}
	else
	{
		//make sure the uv coordinates being edited match those on the overlay
		m_pxUVEdit->Nodes[0].x = m_pxUVEditUVs->xUV0.u;
		m_pxUVEdit->Nodes[0].y = m_pxUVEditUVs->xUV0.v;
		m_pxUVEdit->Nodes[1].x = m_pxUVEditUVs->xUV1.u;
		m_pxUVEdit->Nodes[1].y = m_pxUVEditUVs->xUV0.v;
		m_pxUVEdit->Nodes[2].x = m_pxUVEditUVs->xUV1.u;
		m_pxUVEdit->Nodes[2].y = m_pxUVEditUVs->xUV1.v;
		m_pxUVEdit->Nodes[3].x = m_pxUVEditUVs->xUV0.u; 
		m_pxUVEdit->Nodes[3].y = m_pxUVEditUVs->xUV1.v; 
	}
}

void Tools_GUIMenu_Element_Image::RenderUVEdit()
{
	if(!m_pxUVEdit) return;
	m_pxUVEdit->Render();
}

void Tools_GUIMenu_Element_Image::Update()
{
	UpdateUVEdit();
	UpdateFromTile();

	PARENT::Update();

	if(s_bShowInPixelsChanged && IsSelected())
	{
		//need to switch the controls being displayed
		s_bShowInPixelsChanged = false;
		UpdateOverlay();
	}

	if( m_bRotationChanged)
	{
		m_pxAsuraImage->SetRotation( m_fRotation);
		m_bRotationChanged = false;
		m_bChanged = true;
		SetModified();
	}
}

void Tools_GUIMenu_Element_Image::RenderUI()
{
	PARENT::RenderUI();

	RenderUVEdit();
}

void Tools_GUIMenu_Element_Image::UpdateTextureSize()
{
	m_iTextureWidth = 512;
	m_iTextureHeight = 512;
	m_bTextureSizeValid = false;
	
	const int iTexture( m_pxAsuraImage->GetTexture() );

	if(iTexture != ASURA_TEXTURE_NONE)
	{
		m_bTextureSizeValid = Tools_Texture_Management::GetTextureSize(iTexture,m_iTextureWidth,m_iTextureHeight);
	}
}

void Tools_GUIMenu_Element_Image::UpdateFromTile()
{
	if( m_pxAsuraImage && IsTile() )
	{
		const Asura_TiledTextureInfo* pxTileInfo( Asura_TiledTextureManagement::GetTileInfoByHash( m_pxAsuraImage->GetTileHash() ) );
	
		// iff tile not found try loading tiled texture and repating search for tile 
		// note: tiled textures are streamed into GUI menu files at the begining of 
		// export so should always be loaded before the menu page - NDB
		if( !m_bAtemptedTileFileLoad && !pxTileInfo && !m_pxAsuraImage->GetTileFileName().IsNull() && !m_pxAsuraImage->GetTileFileName().IsBlank() )
		{
			// strip project path
			Asura_String xFileName( Asura_GUIMenu_Element_Image::StripProjectPathFromGraphicsFileName( m_pxAsuraImage->GetTileFileName() ) );
			
			// prepend asura files dir
			xFileName.TidyFilename();
			
			// atempt to load file and repeat search for tile
			Asura_ChunkLoading_System::LoadAsuraFile( xFileName.GetString() );
			pxTileInfo = Asura_TiledTextureManagement::GetTileInfoByHash( m_pxAsuraImage->GetTileHash() );		

			if( pxTileInfo )
			{
				// force image element to search for tile data again as tile has just been loaded - NDB
				m_pxAsuraImage->SetTile( m_pxAsuraImage->GetTileHash() );
			}
			else
			{
				Tools_Timed_Message_System::AddMessage( 0x00FFFFFF,"Could not find tile referenced by \"%s\" (in \"%s\")", m_pxAsuraImage->GenerateFullyQualifiedName().GetString(),m_pxAsuraImage->GetTileFileName() );
			}

			m_bAtemptedTileFileLoad = true;
		}

		Asura_Assertf( pxTileInfo,("Cannot find tiled texture referenced by GUI image element \"%s\"", m_pxAsuraImage->GenerateFullyQualifiedName().GetString() ) );
		
		if( pxTileInfo  )
		{
			// if texture index is unchanged then tile file must be unchanged, don't bother updating file name - NDB
			if( pxTileInfo->m_iIndexOfTextureUsed != m_pxAsuraImage->GetTexture() ) 
			{
				// set tile file name
				const Asura_String* const pxFileName( Asura_TiledTextureManagement::GetAsuraFileNameByHash( pxTileInfo->m_uTileHashID ) );

				if( pxFileName )
				{
					m_pxAsuraImage->SetTileFileName( *pxFileName );
				}

				m_pxAsuraImage->SetTextureFileName( "" );
			}

			m_pxAsuraImage->SetTile( pxTileInfo->m_uTileHashID );

			if( !m_pxAsuraImage->GetFlag( Asura_GUIMenu_Element_Image::ASURA_GUIMENU_ELEMENT_IMAGE_LOCAL_FLAG_IGNORETILEUV ) )
			{
				// set GUI menu Tools mode UV from image
				m_xUV.xUV0 = m_pxAsuraImage->GetUV(0);
				m_xUV.xUV1 = m_pxAsuraImage->GetUV(1);
			}

		}
	}
}

bool Tools_GUIMenu_Element_Image::IsUsingKey(int iKey)
{
	if(m_pxUVEdit)
	{
		if(iKey == ASURA_KEYID_MOUSEBUTTON1 ||
			iKey == ASURA_KEYID_MOUSEBUTTON2)
		{
			return true;
		}
	}
	return false;
}

void Tools_GUIMenu_Element_Image::Copy(const Tools_GUIMenu_Root* pxSource)
{
	PARENT::Copy(pxSource);

	if( pxSource && pxSource->GetType() == Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_ELEMENT_IMAGE )
	{
		const Tools_GUIMenu_Element_Image* pxSourceImage = static_cast<const Tools_GUIMenu_Element_Image*>( pxSource );

		//copy tools data
		m_iTextureWidth = pxSourceImage->m_iTextureWidth;
		m_iTextureHeight = pxSourceImage->m_iTextureHeight;
		m_xUV = pxSourceImage->m_xUV;
		m_fRotation = pxSourceImage->m_fRotation;
	}
}

bool Tools_GUIMenu_Element_Image::IsMenuTexture(int iTextureIndex,void* pList)
{
	List<int>* pxList = (List<int>*) pList;

	return pxList->contains(iTextureIndex);	
}

void Tools_GUIMenu_Element_Image::ListTexturesCallback(int iTextureIndex,void* pList)
{
	List<int>* pxList = (List<int>*) pList;

	if(iTextureIndex != ASURA_TEXTURE_NONE)
	{
		if(!pxList->contains(iTextureIndex))
		{
			pxList->add_entry(iTextureIndex);
		}
	}
}

const char* Tools_GUIMenu_Element_Image::GetLocalDataFlagName(u_int uFlagValue) const
{
	switch(uFlagValue)
	{
		case Asura_GUIMenu_Element_Image::ASURA_GUIMENU_ELEMENT_IMAGE_LOCAL_FLAG_TEXTURE :
			return "Texture";
			break;

		case Asura_GUIMenu_Element_Image::ASURA_GUIMENU_ELEMENT_IMAGE_LOCAL_FLAG_UVS :
			return "UVs";
			break;

		case Asura_GUIMenu_Element_Image::ASURA_GUIMENU_ELEMENT_IMAGE_LOCAL_FLAG_ROTATION :
			return "Rotation";
			break;

		default :
			return PARENT::GetLocalDataFlagName(uFlagValue);

	}
}

// returns whether this object supports e.g. position, or UVs etc.
bool Tools_GUIMenu_Element_Image::IsModifierTypeSupported( ASURA_GUIMENU_MODIFY_TYPE eType ) const
{
	switch (eType)
	{
		case ASURA_GUIMENU_MODIFY_TEXTURE:
		case ASURA_GUIMENU_MODIFY_UVS:
		case ASURA_GUIMENU_MODIFY_IMAGE_ROTATE:
			return true;
		default:
			return PARENT::IsModifierTypeSupported(eType);
	}
}

bool Tools_GUIMenu_Element_Image::CanEditTexture()
{
	return ShouldStyleUseLocalDataFor(Asura_GUIMenu_Element_Image::ASURA_GUIMENU_ELEMENT_IMAGE_LOCAL_FLAG_TEXTURE);
}

bool Tools_GUIMenu_Element_Image::CanEditUVs()
{
	bool bRtn(false);

	if( m_pxAsuraImage && ShouldStyleUseLocalDataFor(Asura_GUIMenu_Element_Image::ASURA_GUIMENU_ELEMENT_IMAGE_LOCAL_FLAG_UVS) )
	{
		bRtn = (IsTexture() || m_pxAsuraImage->GetFlag(Asura_GUIMenu_Element_Image::ASURA_GUIMENU_ELEMENT_IMAGE_LOCAL_FLAG_IGNORETILEUV) );
	}

	return bRtn;

}

bool Tools_GUIMenu_Element_Image::CanEditRotation()
{
	return ShouldStyleUseLocalDataFor(Asura_GUIMenu_Element_Image::ASURA_GUIMENU_ELEMENT_IMAGE_LOCAL_FLAG_ROTATION);
}

bool Tools_GUIMenu_Element_Image::CanFlipTile()
{
	return IsTile() && !CanEditUVs();
}

bool Tools_GUIMenu_Element_Image::IsTile()
{
	return m_pxAsuraImage && (m_pxAsuraImage->GetTileHash() != ASURA_HASH_ID_UNSET);
}

bool Tools_GUIMenu_Element_Image::IsTexture()
{
	return !IsTile();
}


bool Tools_GUIMenu_Element_Image::CanLockUVToDims()
{
	//Need to be able to edit UVs and width/height for this
	return ShouldStyleUseLocalDataFor(Asura_GUIMenu_Element_Image::ASURA_GUIMENU_ELEMENT_IMAGE_LOCAL_FLAG_UVS) &&
			ShouldStyleUseLocalDataFor(Asura_GUIMenu_Element_Base::ASURA_GUIMENU_ELEMENT_BASE_LOCAL_FLAG_WIDTH) &&
			ShouldStyleUseLocalDataFor(Asura_GUIMenu_Element_Base::ASURA_GUIMENU_ELEMENT_BASE_LOCAL_FLAG_HEIGHT);

}


void Tools_GUIMenu_Element_Image::AddModifierOverlayControls(Asura_GUIMenu_Modifier* pxModifier, Asura_Vector_2& xOffset )
{
	Asura_Dialog* pxDialog;

	switch(pxModifier->GetModifyType())
	{
		case ASURA_GUIMENU_MODIFY_UVS:
		{
			m_xModifierData_UV = *pxModifier->GetUVSetData();

			pxDialog = new Asura_Function_Dialog("Texture coordinates:", xOffset, ToggleModifierUVEdit, (int) this);
			AddModifierDialog(pxDialog);
			xOffset.y += Asura_Dialog::DefaultSpacing();

			pxDialog = new Asura_Tickbox_Dialog("Show pixel coordinates", xOffset, &s_bShowInPixels, &s_bShowInPixelsChanged);
			pxDialog->SetHelpText("Determines whether coordinates are shown between 0 to 1 , or as pixel coordinates on the texture");
			AddModifierDialog(pxDialog);
			xOffset.y += Asura_Dialog::DefaultSpacing();

			pxDialog = new Asura_ScaledNumerical_Dialog("Left", xOffset, 1.0f, &m_xModifierData_UV.xUV0.u, 0.0f, 1.0f, .001f, "%.3f", &m_bModifierChanged);
			pxDialog->m_iControlID = g_uDialog_ModifierLeftUV;
			AddModifierDialog(pxDialog);
			xOffset.y += Asura_Dialog::DefaultSpacing();

			pxDialog = new Asura_ScaledNumerical_Dialog("Right", xOffset, 1.0f, &m_xModifierData_UV.xUV1.u, 0.0f, 1.0f, .001f, "%.3f", &m_bModifierChanged);
			pxDialog->m_iControlID = g_uDialog_ModifierRightUV;
			AddModifierDialog(pxDialog);
			xOffset.y += Asura_Dialog::DefaultSpacing();

			pxDialog = new Asura_ScaledNumerical_Dialog("Top", xOffset, 1.0f, &m_xModifierData_UV.xUV0.v, 0.0f, 1.0f, .001f, "%.3f", &m_bModifierChanged);
			pxDialog->m_iControlID = g_uDialog_ModifierTopUV;
			AddModifierDialog(pxDialog);
			xOffset.y += Asura_Dialog::DefaultSpacing();

			pxDialog = new Asura_ScaledNumerical_Dialog("Bottom", xOffset, 1.0f, &m_xModifierData_UV.xUV1.v, 0.0f, 1.0f, .001f, "%.3f", &m_bModifierChanged);
			pxDialog->m_iControlID = g_uDialog_ModifierBottomUV;
			AddModifierDialog(pxDialog);
			xOffset.y += Asura_Dialog::DefaultSpacing();
			
		}
		break;

		case ASURA_GUIMENU_MODIFY_TEXTURE:
			m_iModifierData = pxModifier->GetTextureIndexData();

			pxDialog = new Asura_Function_Dialog("Texture : ",xOffset,SelectModifierTexture,(int)this);
			pxDialog->SetHelpText("Select the texture to use when the modifier is active");
			pxDialog->m_iControlID = g_uDialog_ModifierTexture;
			AddModifierDialog(pxDialog);
			xOffset.y += Asura_Dialog::DefaultSpacing();
			break;


		case ASURA_GUIMENU_MODIFY_IMAGE_ROTATE:
			m_fModifierData = pxModifier->GetFloatData();

			pxDialog = new Asura_AngleNumerical_Dialog("Rotation", xOffset, &m_fModifierData, -360.0f, 360.0f, 1.0f, "%3.3f", &m_bModifierChanged);
			pxDialog->SetHelpText("The rotation (in degrees) to use for the image when the modifier is active");
			AddModifierDialog(pxDialog);
			xOffset.y += Asura_Dialog::DefaultSpacing();
			break;

		default :
			PARENT::AddModifierOverlayControls(pxModifier,xOffset);
			break;
	}
}

void Tools_GUIMenu_Element_Image::InitialiseModifier(Asura_GUIMenu_Modifier* pxModifier)
{
	switch(pxModifier->GetModifyType())
	{
		case ASURA_GUIMENU_MODIFY_UVS:
		{
			Asura_GUIMenu_UV_Set xUVSet;
			xUVSet.xUV0 = m_pxAsuraImage->GetUV(0);
			xUVSet.xUV1 = m_pxAsuraImage->GetUV(1);
			pxModifier->SetData(xUVSet);
			break;
		}

		case ASURA_GUIMENU_MODIFY_TEXTURE:
			pxModifier->SetTextureIndexData(m_pxAsuraImage->GetTexture());
			break;
		
		case ASURA_GUIMENU_MODIFY_IMAGE_ROTATE:
			pxModifier->SetData( m_pxAsuraImage->GetRotation());
			break;

		default :
			PARENT::InitialiseModifier(pxModifier);
			break;
	}
}

void Tools_GUIMenu_Element_Image::CleanupModifierEditing()
{
	PARENT::CleanupModifierEditing();

	//If editing modifier UVs then stop
	if(m_bEditingModifierUV)
	{
		FinishUVEdit();
	}
}

bool Tools_GUIMenu_Element_Image::Verify()
{
	int	iTextureID;
	bool bRes;

	bRes = PARENT::Verify();

#ifdef ASURA_GUIMENU_STORE_NAMES
	if ( m_pxAsuraImage != NULL )
	{
		if ( IsTile() == false )
		{
			// Test for non-tiled texture being loaded
			Asura_String xTextureName = m_pxAsuraImage->GetTextureName();

			if ( xTextureName.IsNull() == false && xTextureName.IsBlank() == false )
			{
				iTextureID = Asura_Texture_Management::GetIndexFromHashID( xTextureName.GetHash_IgnoreExtension() );

				if( iTextureID == ASURA_TEXTURE_NONE ||
					Asura_Texture_Management::IsTextureLoaded(iTextureID) == false)
				{
					Tools_Mode_GUIMenu::AddVerifyMessage(this,"The texture %s for element image %s can't be found.",xTextureName.GetString(),GetName());
					bRes = false; // An error has occurred so return false anyway
				}
			}
		}
		else
		{	
			// Test if tiled texture is loaded
			const Asura_TiledTextureInfo* pxTileInfo( Asura_TiledTextureManagement::GetTileInfoByHash( m_pxAsuraImage->GetTileHash() ));

			if( pxTileInfo == NULL )
			{
				Tools_Mode_GUIMenu::AddVerifyMessage(this,"The tiled texture %s for tiled element image %s can't be found.", m_pxAsuraImage->GetTileFileName().GetString(),GetName());
				bRes = false; // An error has occurred so return false anyway
			}
		}
	}
#endif

	return bRes;
}


#endif //ASURA_USE_GUI_MENU_SYSTEM
