//
// Tools_GUIMenu_Element_Path.cpp
//
// Created by Richard Rice - 05/12/2006
// 
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

#include "GUIMenu/Tools_GUIMenu_Element_Path.h"

#ifdef ASURA_USE_GUI_MENU_SYSTEM
#include "Core/Asura_TextureManagement.h"
#include "Core/Asura_Timers.h"
#include "Core/Asura_UserInput.h"

#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Interface.h"
#include "Interface/Asura_Overlay.h"

//#include "GUIMenu/Asura_GUIMenu_Root.h"
//#include "GUIMenu/Asura_GUIMenu_Element_Base.h"
#include "GUIMenu/Asura_GUIMenu_Element_Path.h"

#include "Rendering/Asura_Render.h"

#include "GUIMenu/Tools_Mode_GUIMenu.h"


bool Tools_GUIMenu_Element_Path::s_bClickedForPathEditing = false;
bool Tools_GUIMenu_Element_Path::s_bDrag= false;
Asura_Vector_2 Tools_GUIMenu_Element_Path::s_xInitialClickPos;
bool Tools_GUIMenu_Element_Path::s_bEditPath = false;


Tools_GUIMenu_Element_Path::Tools_GUIMenu_Element_Path(Asura_GUIMenu_Element_Path* pxAsuraPath,Tools_GUIMenu_Root* pxToolsParent)
:Tools_GUIMenu_Element_Base(pxAsuraPath,pxToolsParent)
{
	m_pxAsuraPath = pxAsuraPath;
	m_iSelectedPoint = -1;
	
	//get the points from the asura data
	m_uNumPoints = m_pxAsuraPath->GetNumberOfPoints();
	m_pxLocalPoints = m_uNumPoints ? new Asura_Vector_2[m_uNumPoints] : 0;


	for(u_int i=0;i<m_uNumPoints;i++)
	{
		m_pxLocalPoints[i] = m_pxAsuraPath->GetPoint(i);
	}		

}

Tools_GUIMenu_Element_Path::~Tools_GUIMenu_Element_Path()
{
	delete [] m_pxLocalPoints;
}

void Tools_GUIMenu_Element_Path::InitialiseNewItem()
{
	PARENT::InitialiseNewItem();

	//create a default path
	delete [] m_pxLocalPoints;

	m_uNumPoints = 2;
	m_pxLocalPoints = new Asura_Vector_2[2];

	m_pxLocalPoints[0].x = 0.0f;
	m_pxLocalPoints[0].y = m_pxAsuraPath->GetUnscaledLocalBounds().GetHeight() / 2.0f;

	m_pxLocalPoints[1].x = m_pxAsuraPath->GetUnscaledLocalBounds().GetWidth();
	m_pxLocalPoints[1].y = m_pxAsuraPath->GetUnscaledLocalBounds().GetHeight() / 2.0f;

	m_pxAsuraPath->SetPointArray(m_pxLocalPoints,m_uNumPoints);
}

void Tools_GUIMenu_Element_Path::Update()
{
	PARENT::Update();

	if (m_pxAsuraPath)
	{
		if (IsSelected() && s_bEditPath && (Asura_Timers::GetFrameNumber() & 32) == 0)
		{
			m_pxAsuraPath->SetCurrentPathValue(1.0f);
		}
		else
		{
			m_pxAsuraPath->SetCurrentPathValue(0.0f);
		}
	}

	if(IsSelected() && s_bEditPath)
	{
		UpdatePathEdit();
	}
	else if(!s_bEditPath)
	{
		s_bClickedForPathEditing = false;
	}


}

void Tools_GUIMenu_Element_Path::RenderUI()
{
	PARENT::RenderUI();
	
	Asura_Bounding_Rectangle xRect = GetRectangle();

	if(m_uNumPoints)
	{
		//draw lines between the points
		u_int uNumSegments = (m_uNumPoints-1)*10;
		
		Asura_Vector_2 xPathPos = m_pxAsuraPath->GetCurrentPathPosition(0.0f);
		Asura_Vector_2 xEnd(xPathPos.x + xRect.MinX, xPathPos.y + xRect.MinY);
		xEnd = Tools_Mode_GUIMenu::GetInstance()->ConvertFromMenuCoords(xEnd);

		for(u_int i=0;i<uNumSegments;i++)
		{
			Asura_Vector_2 xStart = xEnd;
			xPathPos = m_pxAsuraPath->GetCurrentPathPosition((i+1)/(float)uNumSegments);
			xEnd.x = xPathPos.x + xRect.MinX;
			xEnd.y = xPathPos.y + xRect.MinY;
			xEnd = Tools_Mode_GUIMenu::GetInstance()->ConvertFromMenuCoords(xEnd);

   			Asura_Render::Render2DLine(xStart.x,xStart.y,xEnd.x,xEnd.y,0xffffffff);
		}


		//draw the points points
		static int iTextureIndex = -1;
		if(iTextureIndex == -1)
		{
			iTextureIndex = Asura_Texture_Management::AddTextureWithResetGroup("\\tools\\graphnode.tga", ASURA_TEXTURE_LOADED_PERMANENT);
			Asura_Texture_Management::LoadTextures();
		}

		Asura_Render::Overlay.TextureIndex = iTextureIndex;
		Asura_Render::Overlay.TranslucencyMode = ASURA_TRANSLUCENCY_NORMAL;
		Asura_Render::Overlay.SetRectUV(0.f, 0.f, 1.f, 1.f);


		for(u_int i=0;i<m_uNumPoints;i++)
		{
			//get points position in screen space
			Asura_Vector_2 xPos(m_pxLocalPoints[i].x + xRect.MinX, m_pxLocalPoints[i].y + xRect.MinY);
			xPos = Tools_Mode_GUIMenu::GetInstance()->ConvertFromMenuCoords(xPos);
	
			float fRadius = 4.0f;
			if(i == (u_int) m_iSelectedPoint) fRadius += 1.5f;

			Asura_Render::Overlay.SetRectXY(xPos.x - fRadius, xPos.y - fRadius, fRadius * 2, fRadius * 2);
			Asura_Render::Overlay.Colour = 0xffffffff;
			Asura_Render::Overlay.Render();	
		}
	}
}

void Tools_GUIMenu_Element_Path::OnDeselection()
{
	PARENT::OnDeselection();
	s_bEditPath = false;
}

void Tools_GUIMenu_Element_Path::SetRectangle(const Asura_Bounding_Rectangle& xRect)
{
	Asura_Bounding_Rectangle xOldRect = GetRectangle();

	if(m_uNumPoints)
	{
		float fOldWidth = xOldRect.GetWidth();
		float fOldHeight = xOldRect.GetHeight();
		float fWidth = xRect.GetWidth();
		float fHeight = xRect.GetHeight();

		if(fOldWidth != fWidth || fOldHeight != fHeight)
		{
			if(Tools_Mode_GUIMenu::ShouldResizeChildrenWithParent())
			{
				//scale the points if the rectangle is being resized
				for(u_int i=0;i<m_uNumPoints;i++)
				{
					if(fOldWidth)
					{
						m_pxLocalPoints[i].x *= fWidth/fOldWidth;
					}
					if(fOldHeight)
					{
						m_pxLocalPoints[i].y *= fHeight/fOldHeight;
					}
				}
			}
			else
			{
				//clamp points to rectangle	
				for(u_int i=0;i<m_uNumPoints;i++)
				{
					m_pxLocalPoints[i].x = Asura_Maths::Min(m_pxLocalPoints[i].x,fWidth);
					m_pxLocalPoints[i].y = Asura_Maths::Min(m_pxLocalPoints[i].y,fHeight);

				}
			}

			m_pxAsuraPath->SetPointArray(m_pxLocalPoints,m_uNumPoints);
		}
	}

	PARENT::SetRectangle(xRect);

}

void Tools_GUIMenu_Element_Path::AddOverlayControls(Asura_Overlay* pxOverlay,Asura_Vector_2& xOffset)
{
	PARENT::AddOverlayControls(pxOverlay,xOffset);

	Asura_Dialog* pxDialog;

	pxDialog = new Asura_Function_Dialog("Add Image",xOffset,AddElement_Image,(int)this);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Tickbox_Dialog("Edit/Preview Path",xOffset,&s_bEditPath);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();
}

void Tools_GUIMenu_Element_Path::UpdatePathEdit()
{
	
	if(Asura_Interface::UsingMenusOrOverlays())
	{
		s_bClickedForPathEditing = false;
		return;
	}

	
	
	Asura_Vector_2 xCursorPos = Tools_Mode_GUIMenu::GetInstance()->GetCursorPosInMenuCoords();
	
	if(Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_MOUSEBUTTON1))
	{
		if(Asura_UserInput::KeyPressed(ASURA_KEYID_SHIFT))
		{
			//shift + left click - create a new point
			CreatePoint();
			SetModified();
			m_bChanged = true;
		}
		else
		{
			//debounced left click selects the point under the cursor
			m_iSelectedPoint = GetPointFromPosition(xCursorPos);

			s_bClickedForPathEditing = true;
			s_bDrag = false;
			s_xInitialClickPos = xCursorPos;
		}

	}
	else if(Asura_UserInput::KeyPressed(ASURA_KEYID_MOUSEBUTTON1))
	{
		//mouse button is held down.. but only use if the initial click was picked up
		if(s_bClickedForPathEditing)
		{
			//has the cursor moved far enough for dragging to start?
			if(!s_bDrag)
			{
				Asura_Vector_2 xDiff = xCursorPos - s_xInitialClickPos;
				float fTol = 5.0f/Tools_Mode_GUIMenu::GetInstance()->GetZoom();
				if(Asura_Maths::Abs(xDiff.x)>fTol || Asura_Maths::Abs(xDiff.y)>fTol)
				{
					s_bDrag = true;
				}
			}

			if(s_bDrag && m_iSelectedPoint>=0)
			{
				SetPointPosition(m_iSelectedPoint,xCursorPos);
				SetModified();
				m_bChanged = true;
			}
		}
	}
	else
	{
		s_bClickedForPathEditing = false;
	}

	if(Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_DEL))
	{
		if(m_iSelectedPoint>=0)
		{
			DeletePoint(m_iSelectedPoint);
		}
	}
}

int Tools_GUIMenu_Element_Path::GetPointFromPosition(const Asura_Vector_2& xPos)
{
	int iNearPoint = -1;
	//maximum distance from point allowed for selecting. Need to divide by the zoom , since
	//the positions are in menu coords.
	float fNearDist = 6.0f/Tools_Mode_GUIMenu::GetInstance()->GetZoom();

	
	Asura_Bounding_Rectangle xRect = GetRectangle();
	
	for(u_int i=0;i<m_uNumPoints;i++)
	{
		//get point position in world menu coords
		Asura_Vector_2 xPointPos;
		xPointPos.x = m_pxLocalPoints[i].x + xRect.MinX;
		xPointPos.y = m_pxLocalPoints[i].y + xRect.MinY;

		float fDist = (xPointPos - xPos).Magnitude();
		if(fDist < fNearDist)
		{
			iNearPoint = i;
			fNearDist = fDist;
		}
	}

	return iNearPoint;
}

void Tools_GUIMenu_Element_Path::CreatePoint()
{
	Asura_Bounding_Rectangle xRect = GetRectangle();
	Asura_Vector_2 xCursorPos = Tools_Mode_GUIMenu::GetInstance()->GetCursorPosInMenuCoords();

	Asura_Vector_2 xLocalCursorPos;
	xLocalCursorPos.x = xCursorPos.x - xRect.MinX;
	xLocalCursorPos.y = xCursorPos.y - xRect.MinY;

	//determine which points to insert between
	u_int uInsertIndex = 0;
	float fBestScore = -1000.0f;

	for(u_int i=0;i<=m_uNumPoints;i++)
	{
		Asura_Vector_2 xDir1,xDir2;

		if(i==0)
		{
			xDir1 = m_pxLocalPoints[1] - m_pxLocalPoints[0];
			xDir2 = m_pxLocalPoints[0] - xLocalCursorPos;
		}
		else if(i == m_uNumPoints)
		{
			xDir1 = m_pxLocalPoints[i-1] - m_pxLocalPoints[i-2];
			xDir2 = xLocalCursorPos - m_pxLocalPoints[i-1];
		}
		else
		{
			xDir1 = xLocalCursorPos - m_pxLocalPoints[i-1];
			xDir2 = m_pxLocalPoints[i] - xLocalCursorPos;
		}

		if(xDir1.Magnitude()) xDir1.Normalise();
		if(xDir2.Magnitude()) xDir2.Normalise();

		float fScore = xDir1*xDir2;

		if(fScore > fBestScore)
		{
			fBestScore = fScore;
			uInsertIndex = i;
		}
 	}

	//create enlarged array for points
	Asura_Vector_2* pxLocalPoints = new Asura_Vector_2[m_uNumPoints+1];
	for(u_int i=0;i<m_uNumPoints;i++)
	{
		if( i>= uInsertIndex)
			pxLocalPoints[i+1] = m_pxLocalPoints[i];
		else
			pxLocalPoints[i] = m_pxLocalPoints[i];
		
	}

	delete [] m_pxLocalPoints;
	m_pxLocalPoints = pxLocalPoints;
	m_uNumPoints++;


	m_iSelectedPoint = (int) uInsertIndex;
	SetPointPosition(m_iSelectedPoint,xCursorPos);

	SetModified();
	
}

void Tools_GUIMenu_Element_Path::DeletePoint(int iIndex)
{
	Asura_Assert(iIndex>=0 && iIndex<(int)m_uNumPoints,"");

	//don't allow deletion below two points
	if(m_uNumPoints == 2) return;

	//remove from the tools copy of the points
	m_uNumPoints--;
	for(u_int i = (u_int)iIndex; i<m_uNumPoints; i++)
	{
		m_pxLocalPoints[i] = m_pxLocalPoints[i+1];		
	}

	//update the asura points
	m_pxAsuraPath->SetPointArray(m_pxLocalPoints,m_uNumPoints);

	if(m_iSelectedPoint >= (int)m_uNumPoints)
	{
		m_iSelectedPoint = (int)(m_uNumPoints-1);
	}

	UpdateChildrenRelativeToPath();

	
	SetModified();
}

void Tools_GUIMenu_Element_Path::SetPointPosition(int iIndex,const Asura_Vector_2& xPos)
{
	Asura_Assert(iIndex>=0 && iIndex<(int)m_uNumPoints,"");

	//snap position to grid if required
	Asura_Vector_2 xWorldPos;
	xWorldPos.x = Tools_Mode_GUIMenu::GetInstance()->SnapX(xPos.x);
	xWorldPos.y = Tools_Mode_GUIMenu::GetInstance()->SnapY(xPos.y);

	Asura_Bounding_Rectangle xRect = GetRectangle();
	//clamp to the extents of the element
	if(xWorldPos.x < xRect.MinX) xWorldPos.x = xRect.MinX;
	if(xWorldPos.x > xRect.MaxX) xWorldPos.x = xRect.MaxX;
	if(xWorldPos.y < xRect.MinY) xWorldPos.y = xRect.MinY;
	if(xWorldPos.y > xRect.MaxY) xWorldPos.y = xRect.MaxY;

	//convert to local coords and update in the points array
	m_pxLocalPoints[iIndex].x = xWorldPos.x - xRect.MinX;
	m_pxLocalPoints[iIndex].y = xWorldPos.y - xRect.MinY;

	//update asura path
	m_pxAsuraPath->SetPointArray(m_pxLocalPoints,m_uNumPoints);

	UpdateChildrenRelativeToPath();

	SetModified();

}

bool Tools_GUIMenu_Element_Path::IsUsingKey(int iKey)
{
	if(s_bEditPath)
	{
		if(iKey == ASURA_KEYID_MOUSEBUTTON1 ||
			iKey == ASURA_KEYID_MOUSEBUTTON2 ||
			iKey == ASURA_KEYID_DEL)
		{
			return true;
		}
	}
	return false;
}

void Tools_GUIMenu_Element_Path::Copy(const Tools_GUIMenu_Root* pxSource)
{
	PARENT::Copy(pxSource);

	const Tools_GUIMenu_Element_Path* pxSourcePath = (const Tools_GUIMenu_Element_Path*) pxSource;

	//copy tools data
	m_uNumPoints = pxSourcePath->m_uNumPoints;
	delete [] m_pxLocalPoints;
	m_pxLocalPoints = m_uNumPoints ? new Asura_Vector_2[m_uNumPoints] : 0;
	for(u_int i=0;i<m_uNumPoints;i++)
	{
		m_pxLocalPoints[i] = pxSourcePath->m_pxLocalPoints[i];
	}
	m_iSelectedPoint = pxSourcePath->m_iSelectedPoint;
}


bool Tools_GUIMenu_Element_Path::IsValidChildType(int iType) const
{
	//paths can have images as children
	return iType == Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_ELEMENT_IMAGE;

}

void Tools_GUIMenu_Element_Path::AddElement_Image()
{
	Tools_Mode_GUIMenu::GetInstance()->StartMenuItemPlacement(this,Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_ELEMENT_IMAGE);
}


void Tools_GUIMenu_Element_Path::UpdateChildrenRelativeToPath()
{
	GetAsuraData()->SetScaledWorldBounds(GetAsuraData()->GetScaledWorldBounds());
}

void Tools_GUIMenu_Element_Path::CopyFromAsura()
{
	PARENT::CopyFromAsura();

	//get the points from the asura data
	delete [] m_pxLocalPoints;
	m_uNumPoints = m_pxAsuraPath->GetNumberOfPoints();
	m_pxLocalPoints = m_uNumPoints ? new Asura_Vector_2[m_uNumPoints] : 0;


	for(u_int i=0;i<m_uNumPoints;i++)
	{
		m_pxLocalPoints[i] = m_pxAsuraPath->GetPoint(i);
	}		

	if(m_iSelectedPoint >= (int) m_uNumPoints) m_iSelectedPoint = -1;
}


#endif //ASURA_USE_GUI_MENU_SYSTEM
