// Tools_GUIMenu_Menu_Page.cpp
//
// Created by Richard Rice - 30/11/2006
// 
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"
#include "GUIMenu/Tools_GUIMenu_Menu_Page.h"

#ifdef ASURA_USE_GUI_MENU_SYSTEM
#include "Interface/Asura_Overlay.h"
#include "Interface/Asura_Dialog.h"

//#include "GUIMenu/Asura_GUIMenu_Message.h"
//#include "GUIMenu/Asura_GUIMenu_System.h"
#include "GUIMenu/Asura_GUIMenu_Menu_Page.h"

#include "GUIMenu/Tools_Mode_GUIMenu.h"
#include "GUIMenu/Tools_GUIMenu_System.h"
#include "GUIMenu/Tools_GUIMenu_Widget_Base.h"
#include "Load&Save/LevelExport/Tools_Level_Export.h"
#include "Misc/Tools_Modal_Overlay.h"
#include "Misc/Tools_Timed_Messages.h"

////////////////////////////////////////////////////////////////////////////////////////////////////////////

struct GUIMenuPlatformInfo
{
	Asura_Char m_acName[10];
	int m_iID;
};

static GUIMenuPlatformInfo g_axPlatformInfo[Tools_Level_Export::ExportType_NumPlatforms+1];
static Asura_Char* g_apcPlatformInfoNames[Tools_Level_Export::ExportType_NumPlatforms+1];

////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*--------------------------------------**
** Tools_GUIMenu_PickItem_InitialWidget **
**--------------------------------------*/

//For selecting the startup widget
class Tools_GUIMenu_PickItem_InitialWidget : public Tools_GUIMenu_PickItem
{
public :
	Tools_GUIMenu_PickItem_InitialWidget(Tools_GUIMenu_Menu_Page* pxMenu)
	{
		m_pxMenu = pxMenu;
	}

	virtual const char* GetPrompt(){return "Select the initial widget";}

	virtual bool IsValidSelection(Tools_GUIMenu_Root* pxItem)
	{
		//only allow widgets that are selectable
		if(pxItem)
		{
			if(pxItem->IsWidget())
			{
				return Tools_GUIMenu_Widget_Base::GetCanBeSelectable(pxItem);
			}
			return false;

		}
		return true;
	}

	virtual void OnFinish(Tools_GUIMenu_Root* pxItem)
	{
		Asura_Assert(pxItem==0 || pxItem->IsWidget(),"");

		m_pxMenu->SetInitialWidget(pxItem);
	}

private :
	Tools_GUIMenu_Menu_Page* m_pxMenu;
};

////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*--------------------------------**
** Tools_GUIMenu_Widget_Menu_Page **
**--------------------------------*/

Tools_GUIMenu_Menu_Page::Tools_GUIMenu_Menu_Page(Asura_GUIMenu_Menu_Page* pxAsuraMenuPage,Tools_GUIMenu_Root* pxToolsParent)
:Tools_GUIMenu_Root(pxAsuraMenuPage,pxToolsParent)
{
	m_pxAsuraMenuPage = pxAsuraMenuPage;

	m_uStartupWidgetID = m_pxAsuraMenuPage->GetStartupWidgetID();

	m_bCodeDrivenChanged = false;
}


void Tools_GUIMenu_Menu_Page::SetRectangle(const Asura_Bounding_Rectangle& xRect)
{
	// SK 30.04.07 - added for Gordon Hayes
	m_pxAsuraMenuPage->SetUnscaledLocalBounds(xRect);

}

bool Tools_GUIMenu_Menu_Page::ConfirmDeletion()
{
	return ShowYesNoOverlay("Delete menu '%s'?",GetName());
}

bool Tools_GUIMenu_Menu_Page::IsValidChildType(int iType) const
{
	//allow widgets
	if(iType >= Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_WIDGET_BEGIN && iType <=Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_WIDGET_END) return true;

	//and some elements
	return (iType == Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_ELEMENT_IMAGE) 
		|| (iType == Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_ELEMENT_TEXT)
		|| (iType == Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_ELEMENT_ANIMATION)
		|| (iType == Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_ELEMENT_STATUSTEXT)
		|| (iType == Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_ELEMENT_FMV);

}

void Tools_GUIMenu_Menu_Page::Copy(const Tools_GUIMenu_Root* pxSource)
{
	PARENT::Copy(pxSource);

	const Tools_GUIMenu_Menu_Page* pxSourceMenuPage = (const Tools_GUIMenu_Menu_Page*) pxSource;
	const Asura_GUIMenu_Menu_Page* pxSourceAsuraMenu = (const Asura_GUIMenu_Menu_Page*) pxSourceMenuPage->m_pxAsuraMenuPage;

	//Need to keep a local copy of the startup widget ID ,since the widget probably won't exist yet.
	//This will get fixed up when UpdateReferences is called
	m_uStartupWidgetID = pxSourceAsuraMenu->GetStartupWidgetID();  
}

// --------------------------------------------------------------------------------------------

void Tools_GUIMenu_Menu_Page::AddOverlayControls(Asura_Overlay* pxOverlay,Asura_Vector_2& xOffset)
{
	PARENT::AddOverlayControls(pxOverlay,xOffset);

	Asura_Dialog* pxDialog;

	xOffset.y += Asura_Dialog::SmallSpacing();



	pxDialog = new Asura_Function_Dialog("Transfer Menu",xOffset,TransferMenu,(int)this);
	pxDialog->SetHelpText("Moves the menu into a different file..");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Initial Widget : ",xOffset,EditInitialWidget,(int)this);
	pxDialog->SetHelpText("The widget that should be selected by default when the menu activates.");
	pxDialog->m_iControlID = Asura_GetHashID("InitalWidget");
	pxOverlay->AddDialog(pxDialog);
	//xOffset.y += Asura_Dialog::DefaultSpacing();
	xOffset.y += Asura_Dialog::LargeSpacing();

	//Tools_Mode_GUIMenu::RemoveReferencesToMenuItem(m_pxAsuraRoot->GetID());

	pxDialog = new Asura_FlagTickbox_Dialog( "Code Driven", xOffset, m_pxAsuraMenuPage->GetMenuPageFlagsPtr(), ASURA_GUIMENU_MENUPAGE_FLAG_CODE_UPDATE_AND_RENDER_ONLY, &m_bCodeDrivenChanged);
	pxDialog->SetIsEnabledFunction(CanEditMenuPageFlags,(int)this);
	pxDialog->SetHelpText("This flag causes the menu to never be Auto updated or rendered.\nThe menu page update & render calls must be called by the code specifically.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_FlagTickbox_Dialog( "Center Menu On Screen", xOffset, m_pxAsuraMenuPage->GetMenuPageFlagsPtr(), ASURA_GUIMENU_MENUPAGE_FLAG_CENTER_MENU_ON_SCREEN, &m_bChanged );
	pxDialog->SetIsEnabledFunction(CanEditMenuPageFlags,(int)this);
	pxDialog->SetHelpText("Causes the menu page to center itself on the screen when opened.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_FlagTickbox_Dialog( "Auto Resize", xOffset, m_pxAsuraMenuPage->GetMenuPageFlagsPtr(), ASURA_GUIMENU_MENUPAGE_FLAG_ALLOW_AUTO_RESIZE, &m_bChanged );
	pxDialog->SetIsEnabledFunction(CanEditMenuPageFlags,(int)this);
	pxDialog->SetHelpText("Causes the menu to be automatically resized to match the viewport's size.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_FlagTickbox_Dialog( "Render Widescreen Bars", xOffset, m_pxAsuraMenuPage->GetMenuPageFlagsPtr(), ASURA_GUIMENU_MENUPAGE_FLAG_RENDER_WIDESCREEN_BARS, &m_bChanged );
	pxDialog->SetIsEnabledFunction(CanEditMenuPageFlags,(int)this);
	pxDialog->SetHelpText("Causes the menu to render widescreen bars if it doesn't fit on the screen.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_FlagTickbox_Dialog( "Always render fullscreen", xOffset, m_pxAsuraMenuPage->GetMenuPageFlagsPtr(), ASURA_GUIMENU_MENUPAGE_FLAG_RENDER_TO_FULLSCREEN, &m_bChanged );
	pxDialog->SetIsEnabledFunction(CanEditMenuPageFlags,(int)this);
	pxDialog->SetHelpText("in a split screen game, this flag forces the menu to be displayed across the main screen and not in a sub window.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_FlagTickbox_Dialog( "Allow 'Back' to this menu", xOffset, m_pxAsuraMenuPage->GetMenuPageFlagsPtr(), ASURA_GUIMENU_MENUPAGE_FLAG_ALLOW_BACK_INTO_THIS, &m_bChanged );
	pxDialog->SetIsEnabledFunction(CanEditMenuPageFlags,(int)this);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_FlagTickbox_Dialog( "Allow 'Back' from this menu", xOffset, m_pxAsuraMenuPage->GetMenuPageFlagsPtr(), ASURA_GUIMENU_MENUPAGE_FLAG_ALLOW_BACK_FROM_THIS, &m_bChanged );
	pxDialog->SetIsEnabledFunction(CanEditMenuPageFlags,(int)this);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_FlagTickbox_Dialog( "Allow key press 'Back' from this menu", xOffset, m_pxAsuraMenuPage->GetMenuPageFlagsPtr(), ASURA_GUIMENU_MENUPAGE_FLAG_ALLOW_KEY_PRESS_BACK_FROM_THIS, &m_bChanged );
	pxDialog->SetIsEnabledFunction(CanEditMenuPageFlags,(int)this);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();


	pxDialog = new Asura_FlagTickbox_Dialog( "Close this Menu on 'Back'", xOffset, m_pxAsuraMenuPage->GetMenuPageFlagsPtr(), ASURA_GUIMENU_MENUPAGE_FLAG_CLOSE_THIS_MENU_ON_BACK, &m_bChanged );
	pxDialog->SetIsEnabledFunction(CanEditMenuPageFlags,(int)this);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_FlagTickbox_Dialog( "Should Pause", xOffset, m_pxAsuraMenuPage->GetMenuPageFlagsPtr(), ASURA_GUIMENU_MENUPAGE_FLAG_PAUSE, &m_bChanged );
	pxDialog->SetIsEnabledFunction(CanEditMenuPageFlags,(int)this);
	pxDialog->SetHelpText("Pauses the game if this menu is active");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();


	bool bEnableOnInitCommand = CanEditOnInitCommand();
	m_pxAsuraMenuPage->m_xOnInitCommand.AddOverlayControls( pxOverlay, xOffset, "OnInit Command", "Calls this command when the OnInit message is received by a menu page", true, &m_bChanged, bEnableOnInitCommand );

	bool bEnableUpdateCommand = CanEditOnUpdateCommand();
	m_pxAsuraMenuPage->m_xOnUpdateCommand.AddOverlayControls( pxOverlay, xOffset, "OnUpdate Command", "Calls this command when the a menu page is updated", true, &m_bChanged, bEnableUpdateCommand );

	bool bEnableOnDeInitCommand = CanEditOnDeInitCommand();
	m_pxAsuraMenuPage->m_xOnDeInitCommand.AddOverlayControls( pxOverlay, xOffset, "OnDeInit Command", "Calls this command when the OnDeInit message is received by a menu page", true, &m_bChanged, bEnableOnDeInitCommand  );
}

void Tools_GUIMenu_Menu_Page::UpdateOverlayControls(Asura_Overlay* pxOverlay)
{
	PARENT::UpdateOverlayControls(pxOverlay);

	//update initial widget selection control
	Asura_Hash_ID uID = m_pxAsuraMenuPage->GetStartupWidgetID();

	const char* szName;
	if(uID)
	{
		Tools_GUIMenu_Widget_Base* pxWidget = FindWidget(uID);

		szName = pxWidget ? pxWidget->GetName() : "*Unknown*";
	}
	else
	{
		szName = "*None*";
	}

	char szText[100];
	Asura_PrintfToBfr(szText,"Initial Widget : %s",sizeof(szText),true,szName);

	pxOverlay->UpdateDialogLabel(Asura_GetHashID("InitalWidget"),szText);
}

void Tools_GUIMenu_Menu_Page::Update( void)
{
	if (m_bResized)
	{
		if (m_fW == 640.0f
			&& m_fH == 528.0f)
		{
			m_pxAsuraMenuPage->GetVirtualSizePtr()->x = 640.0f;
			m_pxAsuraMenuPage->GetVirtualSizePtr()->y = 480.0f;
		}
		else if (m_pxAsuraMenuPage->GetUnscaledLocalBounds().GetWidth() == 640.0f
			&& m_pxAsuraMenuPage->GetUnscaledLocalBounds().GetHeight() == 528.0f
			&& m_pxAsuraMenuPage->GetVirtualSize().x == 640.0f
			&& m_pxAsuraMenuPage->GetVirtualSize().y == 480.0f)
		{
			m_pxAsuraMenuPage->GetVirtualSizePtr()->x = 0.0f;
			m_pxAsuraMenuPage->GetVirtualSizePtr()->y = 0.0f;
		}
	}

	if( m_bCodeDrivenChanged)
	{
		bool bTF = false;

		if( (m_pxAsuraMenuPage->GetMenuPageFlags()&ASURA_GUIMENU_MENUPAGE_FLAG_CODE_UPDATE_AND_RENDER_ONLY) != 0)
		{
			Tools_Mode_GUIMenu::RemoveReferencesToMenuItem( m_pxAsuraMenuPage->GetID());
			bTF = true;
		}

		Asura_GUIMenu_System::FlagMenuPageAsCodeDrivenByID( m_pxAsuraMenuPage->GetID(), bTF);

		m_bChanged = true;
		m_bCodeDrivenChanged = false;
	}

	if(m_bChanged)
	{
		// CLF (07.11.08) - Support for new flag allowing key press back functionality to be disabled
		if ((m_pxAsuraMenuPage->GetMenuPageFlags() & ASURA_GUIMENU_MENUPAGE_FLAG_ALLOW_BACK_FROM_THIS) == 0)
		{
			//Disable key press if back from this menu is disabled
			*m_pxAsuraMenuPage->GetMenuPageFlagsPtr() &= ~ASURA_GUIMENU_MENUPAGE_FLAG_ALLOW_KEY_PRESS_BACK_FROM_THIS;
		}
	}

	PARENT::Update();

	// Bug testing - Leaving this in as it may need to be added to catch
	// menu pages involuntary resizing while using the GUI menu tools.
	//{
	//	// check size of menu page for errors
	//	const Asura_Bounding_Rectangle xUnscaledbound = 
	//				m_pxAsuraMenuPage->GetUnscaledLocalBounds();

	//	if ( xUnscaledbound.MaxX > 1280.0f ||
	//		xUnscaledbound.MaxY > 720 )
	//	{
	//		// if tiled texture file does not exist for platform then attempt to create it
	//		if ( m_pxAsuraMenuPage->GetNameAsChar() != NULL)
	//		{
	//			Tools_Timed_Message_System::AddMessage( Tools_Timed_Message_System::COLOUR_WARNING,"WARNING : Menu Page Size is larger than 1280x720 \"%s\" ",m_pxAsuraMenuPage->GetNameAsChar() );
	//		}
	//	}
	//}
}


void Tools_GUIMenu_Menu_Page::EditInitialWidget()
{
	Tools_GUIMenu_PickItem_InitialWidget* pxPickItem = new Tools_GUIMenu_PickItem_InitialWidget(this);
	Tools_Mode_GUIMenu::GetInstance()->StartPromptedSelection(pxPickItem);
}

void Tools_GUIMenu_Menu_Page::SetInitialWidget(Tools_GUIMenu_Root* pxItem)
{
	Asura_Hash_ID uID = pxItem ? pxItem->GetAsuraData()->GetID() : 0;

	//update the menu
	m_pxAsuraMenuPage->SetStartupWidgetID(uID);
	m_uStartupWidgetID = uID;
	SetModified();

	//and update the overlay
	UpdateOverlay();
}


void Tools_GUIMenu_Menu_Page::UpdateReferences(Asura_Hash_ID uOldID,Asura_Hash_ID uNewID,bool bPostCopy)
{
	PARENT::UpdateReferences(uOldID, uNewID,bPostCopy);

	bool bAnyChange = false;

	if(m_uStartupWidgetID == uOldID)
	{
		m_pxAsuraMenuPage->SetStartupWidgetID(uNewID);
		m_uStartupWidgetID = uNewID;
		bAnyChange = true;
	}

	if(bAnyChange) SetModified();
}

void Tools_GUIMenu_Menu_Page::CheckRequiredReferences()
{
	PARENT::CheckRequiredReferences();

	//make sure the menu has a startup widget
	if(!m_pxAsuraMenuPage->GetStartupWidgetID())
	{
		//search for the first widget , and make that the startup widget
		for(CLIF<Tools_GUIMenu_Root*> xChildLIF(GetChildList());!xChildLIF.done();xChildLIF.next())
		{
			if(!xChildLIF()->IsWidget()) continue;

			Asura_Hash_ID uID = xChildLIF()->GetAsuraData()->GetID();
			m_pxAsuraMenuPage->SetStartupWidgetID(uID);
			m_uStartupWidgetID = uID;

			// (AAM - 27.11.2007) Used to call SetModified() here, but there's no point. That just
			// causes problems for people who don't have the file checked out

			break;
		}
	}
}

const char* Tools_GUIMenu_Menu_Page::GetLocalDataFlagName(u_int uFlagValue) const
{
	switch(uFlagValue)
	{
		case Asura_GUIMenu_Menu_Page::ASURA_GUIMENU_MENU_PAGE_LOCAL_FLAG_WIDTH :
			return "Width";
			break;

		case Asura_GUIMenu_Menu_Page::ASURA_GUIMENU_MENU_PAGE_LOCAL_FLAG_HEIGHT :
			return "Height";
			break;

		case Asura_GUIMenu_Menu_Page::ASURA_GUIMENU_MENU_PAGE_LOCAL_FLAG_MENUPAGE_FLAGS :
			return "MenuPage Flags";
			break;

		case Asura_GUIMenu_Menu_Page::ASURA_GUIMENU_MENU_PAGE_LOCAL_FLAG_ONINIT :
			return "On Init Command";
			break;

		case Asura_GUIMenu_Menu_Page::ASURA_GUIMENU_MENU_PAGE_LOCAL_FLAG_ONUPDATE :
			return "On Update Command";
			break;

		case Asura_GUIMenu_Menu_Page::ASURA_GUIMENU_MENU_PAGE_LOCAL_FLAG_ONDEINIT :
			return "On DeInit Command";
			break;

		default :
			return PARENT::GetLocalDataFlagName(uFlagValue);

	}
}

bool Tools_GUIMenu_Menu_Page::CanEditWidth() const
{
	return ShouldStyleUseLocalDataFor(Asura_GUIMenu_Menu_Page::ASURA_GUIMENU_MENU_PAGE_LOCAL_FLAG_WIDTH);
}

bool Tools_GUIMenu_Menu_Page::CanEditHeight() const
{
	return ShouldStyleUseLocalDataFor(Asura_GUIMenu_Menu_Page::ASURA_GUIMENU_MENU_PAGE_LOCAL_FLAG_HEIGHT);
}

bool Tools_GUIMenu_Menu_Page::CanEditMenuPageFlags()
{
	return ShouldStyleUseLocalDataFor(Asura_GUIMenu_Menu_Page::ASURA_GUIMENU_MENU_PAGE_LOCAL_FLAG_MENUPAGE_FLAGS);
}

bool Tools_GUIMenu_Menu_Page::CanEditOnInitCommand()
{
	return ShouldStyleUseLocalDataFor(Asura_GUIMenu_Menu_Page::ASURA_GUIMENU_MENU_PAGE_LOCAL_FLAG_ONINIT);
}

bool Tools_GUIMenu_Menu_Page::CanEditOnUpdateCommand()
{
	return ShouldStyleUseLocalDataFor(Asura_GUIMenu_Menu_Page::ASURA_GUIMENU_MENU_PAGE_LOCAL_FLAG_ONUPDATE);
}

bool Tools_GUIMenu_Menu_Page::CanEditOnDeInitCommand()
{
	return ShouldStyleUseLocalDataFor(Asura_GUIMenu_Menu_Page::ASURA_GUIMENU_MENU_PAGE_LOCAL_FLAG_ONDEINIT);
}

void Tools_GUIMenu_Menu_Page::TransferMenu()
{
	//make sure the current file is writeable
	if(!Tools_GUIMenu_System::CanSaveFile(m_pxAsuraMenuPage->GetFilename()))
	{
		ShowMessageOverlay("The menu can't be transferred to a different file because %s isn't writeable",m_pxAsuraMenuPage->GetFilename());
		return;
	}

	//populate a list overlay with all the filenames
	ListSelection_Overlay xOverlay("Transfer the menu to which file?");
	const Tools_GUIMenu_Root* pxRoot = Tools_Mode_GUIMenu::GetRootItem();

	for(CLIF<Tools_GUIMenu_Root*> xFileLIF(pxRoot->GetChildList()); !xFileLIF.done(); xFileLIF.next())
	{
		xOverlay.AddItem(xFileLIF()->GetName(),(int)xFileLIF());	
	}

	//ask for the file to use.
	if(!xOverlay.DoModal()) return;
	const char* szNewFile = xOverlay.GetSelectionString();
	Tools_GUIMenu_Root* pxFile = (Tools_GUIMenu_Root*) xOverlay.GetSelectionData();

	//the new file needs to be writeable
	if(!Tools_GUIMenu_System::CanSaveFile(szNewFile))
	{
		ShowMessageOverlay("The menu can't be transferred because %s isn't writeable",szNewFile);
		return;
	}

	//mark both files as modified
	Tools_GUIMenu_System::SetModified(m_pxAsuraMenuPage->GetFilename());
	Tools_GUIMenu_System::SetModified(szNewFile);

	m_pxAsuraMenuPage->SetFilename(szNewFile);

	//update the hierarchy of tools menu items , and the treeview
	GetParent()->RemoveChild(this);
	pxFile->AddChild(this);
	m_pxToolsParent = pxFile;
	Tools_Mode_GUIMenu::GetInstance()->UpdateParentInTree(this);

}


#endif //ASURA_USE_GUI_MENU_SYSTEM

