//
// Tools_GUIMenu_PreviewOverlay.h
//
// Created by Richard Rice - 12/03/2007
// 
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//


#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif


#ifndef _TOOLS_GUIMENU_PREVIEWOVERLAY_H_
#define _TOOLS_GUIMENU_PREVIEWOVERLAY_H_
#ifdef ASURA_USE_GUI_MENU_SYSTEM

#include "Core/Asura_Hash.h"
#include "Interface/Asura_CommandConsole.h"


class Tools_GUIMenu_ConsoleData;
template <class T> class Asura_TemplateHashTree;
template <class T> class List;

class Asura_Numerical_Dialog;

//Class for handling overlay that is used during menu preview.
//For now , its main purpose is for allowing summy versions of the console variables and commands to be edited.

class Tools_GUIMenu_PreviewOverlay
{
public :
	static void Initialise();
	static void Shutdown();

	//sets stuff up before preview ,and shows the overlay
	static void StartPreview(const class Tools_GUIMenu_Root* pxRoot);

	static void ShowOverlay();
	static void CloseOverlay();

	//per frame update while previewing menus
	static void Update();


	//called by menu items when listing variables/commands that they use
	static void RegisterConsoleVariable(const char* szName,int iType);
	static void RegisterConsoleCommand(const char* szName);

	//Get the console variable data from the name , if it has been registered with the tools system yet
	static Tools_GUIMenu_ConsoleData* GetConsoleVarData(const char* szName);

	//Mirrors of the equivalent fuctions in Asura_CommandConsole_VarRepository , to prevent data from being modified
	//while testing menus.
	static bool GetBoolValue( const char* szVarName );
	static int GetIntValue( const char* szVarName );
	static float GetFloatValue( const char* szVarName );
	static const Asura_UnicodeChar* GetUnicodeStringValue( const char* szVarName );
	static bool SetBoolValue( const char* szVarName, bool bValue );
	static bool SetIntValue( const char* szVarName, int iValue );
	static bool SetFloatValue( const char* szVarName, float fValue );
	static bool SetUnicodeStringValue( const char* szVarName, const Asura_UnicodeChar* wszValue );

	// functions to cope with not always wanting autoclick to happen when previewing
	static bool IsAutoclickEnabled() { return s_bEnableAutoclick; }
	static void NotifyAutoclick( const char* szWidgetName );

	// interface for user-specified preview text for populating listboxes.
	static const char* GetPopulationText() { return s_xPopulationText.GetString(); }

	static float GetPreviewSpeed() { return s_fSpeed; }

private : 
	static void BuildCurrentVariableList();
	static void DeleteCurrentVariableList();

	//Adds or gets existing data for console command/variable
	static Tools_GUIMenu_ConsoleData* AddConsoleVariable(const char* szName,int iType);
	static Tools_GUIMenu_ConsoleData* AddConsoleCommand(const char* szName);
	
	static void AddConsoleDataForCurrentMenu(Tools_GUIMenu_ConsoleData* pxData);

	static Asura_Hash_ID s_uCurrentMenu;//ID of menu that variables are being displayed for
	static Asura_TemplateHashTree<Tools_GUIMenu_ConsoleData> s_xConsoleDataTree;
	static List<Tools_GUIMenu_ConsoleData*> s_xConsoleDataList;//Sorted list of variables/commands used by current menu

	static List<Asura_Numerical_Dialog*> s_xConsoleDialogList; // List of numeric dialogs created for console commands

	static float s_fStepValue;
	static float s_fSpeed;

public :
	static Asura_Vector_2 s_xOverlayPosition;
	static Asura_Vector_2 s_xSubwindowSize;
	static u_int s_uOverlayMinimised;
	static bool s_bEnableAutoclick;
	static Asura_String s_xLastAutoclick;
	static u_int s_uLastAutoclickTime;
	static Asura_String s_xPopulationText;
	static bool s_bUseCursor;
	static bool s_bShowScreenBorder;
	static bool s_bShowSafeArea;
	static bool s_bClipToScreen;
	static bool s_bCentreScreen;
	static bool s_bUseVirtualResolution;

};


//For holding data for tools versions of console variables/commands
class Tools_GUIMenu_ConsoleData
{
public :
	Tools_GUIMenu_ConsoleData();
	~Tools_GUIMenu_ConsoleData();

	void UpdateUnicodeString();

	bool m_bCmd;//true if this represents a console command

	int m_iType;//matches the TYPE enumeration in Asura_CommandConsole_VarRepository
	Asura_String m_xName;// name of console variable/command

	Asura_String m_xString;//for string variables
	Asura_UnicodeChar* m_wszData;//unicode version of string variable (only valid while getting)

	union
	{
		bool m_bData; //for bool variables , or return value from console commsnds
		int m_iData; //for int variables
		float m_fData; //for float variables
	};

	const Asura_CommandConsole_VarRepository::Var* m_pxVar;
};


#endif //ASURA_USE_GUI_MENU_SYSTEM
#endif //_TOOLS_MODE_GUIMENU_H_
