// Tools_GUIMenu_Widget_Checkbox.cpp
//
// Created by Richard Rice - 01/12/2006
// 
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"
#include "GUIMenu/Tools_GUIMenu_Widget_CheckBox.h"

#ifdef ASURA_USE_GUI_MENU_SYSTEM

#include "GUIMenu/Asura_GUIMenu_Widget_CheckBox.h"
#include "Interface/Asura_CommandConsole.h"
#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Overlay.h"

#include "GUIMenu/Tools_GUIMenu_CommandConsole.h"
#include "GUIMenu/Tools_GUIMenu_PreviewOverlay.h"

Tools_GUIMenu_Widget_CheckBox::Tools_GUIMenu_Widget_CheckBox(Asura_GUIMenu_Widget_CheckBox* pxAsuraCheckBox,Tools_GUIMenu_Root* pxToolsParent)
:Tools_GUIMenu_Widget_Base(pxAsuraCheckBox,pxToolsParent)
{
	m_pxAsuraCheckBox = pxAsuraCheckBox;

}

void Tools_GUIMenu_Widget_CheckBox::AddChildConditionOverlayControls(Tools_GUIMenu_Root* pxItem,Asura_Overlay* pxOverlay,Asura_Vector_2& xOffset)
{
	Asura_Dialog* pxDialog;

	pxDialog = new Asura_Dialog("Condition : ",xOffset);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_FunctionTickbox_Dialog("Always visible",xOffset,pxItem,IsChecked_ConditionAlways,SetChecked_ConditionAlways);
	pxDialog->SetEnabled(pxItem->ShouldStyleUseLocalDataFor(Asura_GUIMenu_Root::ASURA_GUIMENU_ROOT_LOCAL_FLAG_DISPLAY_CONDITIONS));
	pxDialog->SetHelpText("Visible regardless of whether the checkbox is on or off.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_FunctionTickbox_Dialog("Visible when checked",xOffset,pxItem,IsChecked_ConditionOn,SetChecked_ConditionOn);
	pxDialog->SetEnabled(pxItem->ShouldStyleUseLocalDataFor(Asura_GUIMenu_Root::ASURA_GUIMENU_ROOT_LOCAL_FLAG_DISPLAY_CONDITIONS));
	pxDialog->SetHelpText("Only visible when the checkbox is checked.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_FunctionTickbox_Dialog("Visible when unchecked",xOffset,pxItem,IsChecked_ConditionOff,SetChecked_ConditionOff);
	pxDialog->SetEnabled(pxItem->ShouldStyleUseLocalDataFor(Asura_GUIMenu_Root::ASURA_GUIMENU_ROOT_LOCAL_FLAG_DISPLAY_CONDITIONS));
	pxDialog->SetHelpText("Only visible when the checkbox is unchecked.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();

}

void Tools_GUIMenu_Widget_CheckBox::AddOverlayControls(Asura_Overlay* pxOverlay,Asura_Vector_2& xOffset)
{
	Asura_Dialog* pxDialog;

	PARENT::AddOverlayControls(pxOverlay, xOffset);

	m_pxAsuraCheckBox->m_xConsoleVar.AddOverlayControls( pxOverlay, 
		xOffset, "Console Var", "The console variable that will be altered by this checkbox", Asura_CommandConsole_VarRepository::Var::TYPE_BOOL, &m_bChanged );

	pxDialog = new Asura_FlagTickbox_Dialog( "Use as left/right toggle control", xOffset, m_pxAsuraCheckBox->GetCheckboxFlagsAsPtr(), ASURA_GUIMENU_WIDGET_CHECKBOX_FLAGS_USE_LEFT_RIGHT_TOGGLE_CONTROL, &m_bChanged );
	pxDialog->SetIsEnabledFunction(CanEditCheckboxFlags,(int)this);
	pxDialog->SetHelpText("When this option is ticked, the checkbox control will use a left/right control method to change it's on/off value.\nThis means that the left/right navigation option won't be used.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

}

bool Tools_GUIMenu_Widget_CheckBox::IsChecked_ConditionAlways(void* pItem)
{
	Asura_GUIMenu_Root* pxAsuraItem = ((Tools_GUIMenu_Root*) pItem)->GetAsuraData();
	return pxAsuraItem->GetInternalDisplayConditionCompare() == ASURA_GUIMENU_CONDITION_NONE;
}

void Tools_GUIMenu_Widget_CheckBox::SetChecked_ConditionAlways(bool bSet,void* pItem)
{
	if(!bSet) return;
	Asura_GUIMenu_Root* pxAsuraItem = ((Tools_GUIMenu_Root*) pItem)->GetAsuraData();
	pxAsuraItem->SetInternalDisplayCondition(0, ASURA_GUIMENU_CONDITION_NONE);
	((Tools_GUIMenu_Root*) pItem)->SetModified();
}

bool Tools_GUIMenu_Widget_CheckBox::IsChecked_ConditionOn(void* pItem)
{
	Asura_GUIMenu_Root* pxAsuraItem = ((Tools_GUIMenu_Root*) pItem)->GetAsuraData();
	return pxAsuraItem->GetInternalDisplayConditionCompare() == ASURA_GUIMENU_CONDITION_EQUAL && pxAsuraItem->GetInternalDisplayConditionThreshold() == 1;
}

void Tools_GUIMenu_Widget_CheckBox::SetChecked_ConditionOn(bool bSet,void* pItem)
{
	if(!bSet) return;
	Asura_GUIMenu_Root* pxAsuraItem = ((Tools_GUIMenu_Root*) pItem)->GetAsuraData();
	pxAsuraItem->SetInternalDisplayCondition(1, ASURA_GUIMENU_CONDITION_EQUAL);
	((Tools_GUIMenu_Root*) pItem)->SetModified();
}

bool Tools_GUIMenu_Widget_CheckBox::IsChecked_ConditionOff(void* pItem)
{
	Asura_GUIMenu_Root* pxAsuraItem = ((Tools_GUIMenu_Root*) pItem)->GetAsuraData();
	return pxAsuraItem->GetInternalDisplayConditionCompare() == ASURA_GUIMENU_CONDITION_EQUAL && pxAsuraItem->GetInternalDisplayConditionThreshold() == 0;
}

void Tools_GUIMenu_Widget_CheckBox::SetChecked_ConditionOff(bool bSet,void* pItem)
{
	if(!bSet) return;
	Asura_GUIMenu_Root* pxAsuraItem = ((Tools_GUIMenu_Root*) pItem)->GetAsuraData();
	pxAsuraItem->SetInternalDisplayCondition(0, ASURA_GUIMENU_CONDITION_EQUAL);
	((Tools_GUIMenu_Root*) pItem)->SetModified();
}

u_int Tools_GUIMenu_Widget_CheckBox::GetFlagsToEdit() const
{
	return PARENT::GetFlagsToEdit() | ASURA_GUIMENU_ACTION_FLAG_NOTIFY_DATA_CHANGE_IMMEDIATE | ASURA_GUIMENU_ACTION_FLAG_NOTIFY_DATA_CHANGE_ON_EXIT;
}


void Tools_GUIMenu_Widget_CheckBox::AddToPreviewOverlay() const
{
	PARENT::AddToPreviewOverlay();

	if(!m_pxAsuraCheckBox->m_xConsoleVar.IsBlank())
	{
		Tools_GUIMenu_PreviewOverlay::RegisterConsoleVariable(m_pxAsuraCheckBox->m_xConsoleVar.GetName().GetString(),Asura_CommandConsole_VarRepository::Var::TYPE_BOOL);
	}

}

const char* Tools_GUIMenu_Widget_CheckBox::GetLocalDataFlagName(u_int uFlagValue) const
{
	switch(uFlagValue)
	{
		case Asura_GUIMenu_Widget_CheckBox::ASURA_GUIMENU_WIDGET_CHECKBOX_LOCAL_FLAG_FLAGS:
			return "Checkbox flags";
			break;

		default:
			return PARENT::GetLocalDataFlagName(uFlagValue);
	}
}

bool Tools_GUIMenu_Widget_CheckBox::CanEditCheckboxFlags()
{
	return ShouldStyleUseLocalDataFor(Asura_GUIMenu_Widget_CheckBox::ASURA_GUIMENU_WIDGET_CHECKBOX_LOCAL_FLAG_FLAGS);
}

#endif //ASURA_USE_GUI_MENU_SYSTEM
