//
// Tools_GUIMenu_Widget_ListBox.cpp
//
// Created by RDM - 24 April 2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//

/************/
/* Includes */
/************/

// main asura header
#include "Core/Asura.h"

// This file's header.
#include "GUIMenu/Tools_GUIMenu_Widget_ListBox.h"

#ifdef ASURA_USE_GUI_MENU_SYSTEM

// asura
#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Overlay.h"
#include "Interface/Asura_Overlay_System.h"
#include "GUIMenu/Asura_GUIMenu_Element_Image.h"
#include "GUIMenu/Asura_GUIMenu_Element_Path.h"
#include "GUIMenu/Asura_GUIMenu_Widget_ListBox.h"
#include "Rendering/Asura_Font_System.h"

// tools
#include "GUIMenu/Tools_GUIMenu_System.h"
#include "Interface/Tools_ColourPicker.h"
#include "Misc/Tools_Modal_Overlay.h"
#include "Misc/Tools_Timed_Messages.h"
#include "Tools_Overlays.h"

/********/
/* Data */
/********/

/*************/
/* Functions */
/*************/

// Tools_GUIMenu_Widget_ListBoxEntry ///////////////////////////////////////////////////////////////////////////

Tools_GUIMenu_Widget_ListBoxEntry::Tools_GUIMenu_Widget_ListBoxEntry(Asura_GUIMenu_Widget_ListBoxEntry* pxAsuraListBoxEntry,Tools_GUIMenu_Root* pxToolsParent)
: Tools_GUIMenu_Widget_Base( pxAsuraListBoxEntry, pxToolsParent )
, m_pxAsuraListBoxEntry( pxAsuraListBoxEntry )
{
	// NY - 26 August 2008 - some list box entries didn't import correctly from old data - this should fix their size up
	// in theory this means that an LBE with children won't be allowed to have zero size if it's children are bigger than it
	// however, that's basically impossible and not something taht would be useful even if it was
	if (pxAsuraListBoxEntry
		&& pxAsuraListBoxEntry->GetContainerCount()
		&& pxAsuraListBoxEntry->GetUnscaledLocalBounds().GetWidth() <= 0.0f
		&& pxAsuraListBoxEntry->GetUnscaledLocalBounds().GetHeight() <= 0.0f)
	{
		pxAsuraListBoxEntry->FixupBoundsFromChildren();
	}
}

void Tools_GUIMenu_Widget_ListBoxEntry::Update()
{
	PARENT::Update();
	if (m_pxAsuraListBoxEntry
		&& m_pxAsuraListBoxEntry->GetParentObject())
	{
		static_cast<Asura_GUIMenu_Widget_ListBox*>(m_pxAsuraListBoxEntry->GetParentObject())->UpdateEntryOffset();
	}

	// Mantis bug 64234. Always move the list box entry to the front so it renders on top
	// of any other widgets/elements
	MoveToFront();
}

void Tools_GUIMenu_Widget_ListBoxEntry::RenderUI()
{
	PARENT::RenderUI();

	if (m_pxToolsParent)
	{
		static_cast<Tools_GUIMenu_Widget_ListBox*>(m_pxToolsParent)->RenderOffset();
	}
}

void Tools_GUIMenu_Widget_ListBoxEntry::AddOverlayControls(Asura_Overlay* pxOverlay,Asura_Vector_2& xOffset)
{
	// skip basic widget controls
	Tools_GUIMenu_Root::AddOverlayControls( pxOverlay, xOffset );

	Asura_Dialog* pxDialog;

	pxDialog = new Asura_Function_Dialog("Add Text",xOffset,AddElement_Text,(int)this);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Add Image",xOffset,AddElement_Image,(int)this);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Add Path",xOffset,AddElement_Path,(int)this);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Add Anim",xOffset,AddElement_Anim,(int)this);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();
}


void Tools_GUIMenu_Widget_ListBoxEntry::Destroy()
{
	m_bBeingDestroyed = true;
	SetModified();

	
	if(m_pxAsuraListBoxEntry)
	{
		//Remove any references to this item
		Tools_Mode_GUIMenu::RemoveReferencesToMenuItem(m_pxAsuraListBoxEntry->GetID());
	}

	//destroy the children as well
	while(m_xChildren.size())
	{
		m_xChildren.first_entry()->Destroy();
	}

	// should never have tools objects for entries that aren't the entry template
	Asura_Assert( m_pxAsuraListBoxEntry->IsEntryTemplate(), "Created tools object for non-entry template entry!" );

	// no need to delete the asura data as it's a member variable of the list box itself

	//and the tools data
	delete this;
}

void Tools_GUIMenu_Widget_ListBoxEntry::SetName( const char* szNewName )
{
	if(!strcmp(szNewName,GetName()))
	{
		//name is unchanged ,except possibly by case.
		m_pxAsuraRoot->SetName(szNewName);
		m_xName = szNewName;
	}
	else
	{
		m_pxAsuraRoot->SetName(szNewName);
		m_xName = szNewName;

		//now need to recurse down updaing the IDs , and fixing up any links
		RecalculateID();
	}

	SetModified();

	if(IsSelected())
	{
		//update the overlay's title
		Asura_Overlay* pxOverlay = Asura_Overlay_System::GetOverlay(TOOLS_OVERLAY_GUIMENU_SELECTED);
		if(pxOverlay) pxOverlay->SetTitle(szNewName);
	}	

	//update the name on the tree view
	Tools_Mode_GUIMenu::GetInstance()->UpdateNameOnTree(this);
}

void Tools_GUIMenu_Widget_ListBoxEntry::AddParentOverlayControls( Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset, Tools_GUIMenu_Root* pxChild )
{
	Asura_GUIMenu_ConsoleCmd* pxPopFunc = m_pxAsuraListBoxEntry->GetPopulationFunctionForItem( pxChild->GetID() );
	if( pxPopFunc )
	{
		Asura_Dialog* pxDialog = new Asura_Dialog( ".:: Population Function ::.", xOffset );
		pxOverlay->AddDialog( pxDialog );
		xOffset.y += Asura_Dialog::DefaultSpacing();

		pxPopFunc->AddOverlayControls( pxOverlay, xOffset, "", "Console command used to set the images/anims/text for this entry.", false, &m_bChanged);
	}
	Asura_GUIMenu_ConsoleCmd* pxVisFunc = m_pxAsuraListBoxEntry->GetVisibilityFunctionForItem( pxChild->GetID() );
	if( pxVisFunc )
	{
		Asura_Dialog* pxDialog = new Asura_Dialog( ".:: Visibility Function ::.", xOffset );
		pxOverlay->AddDialog( pxDialog );
		xOffset.y += Asura_Dialog::DefaultSpacing();

		pxVisFunc->AddOverlayControls( pxOverlay, xOffset, "", "Console command used to determine whether to render this entry.", false, &m_bChanged);
	}
}

void Tools_GUIMenu_Widget_ListBoxEntry::CopyDataForThisStoredInParent(const Tools_GUIMenu_Root* pxSource, Tools_GUIMenu_Root* pxDest)
{
	PARENT::CopyDataForThisStoredInParent(pxSource,pxDest);

	//Is the source also a child of a listbox entry?
	if(!pxSource->GetParent()) return;
	if(pxSource->GetParent()->GetType() != Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_WIDGET_LISTBOXENTRY) return;

	//find the listbox for both source and dest
	if(!GetParent()) return;
	if(GetParent()->GetType() != Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_WIDGET_LISTBOX) return;
	Tools_GUIMenu_Widget_ListBox* pxDestListBox = (Tools_GUIMenu_Widget_ListBox*) m_pxToolsParent;

	if(!pxSource->GetParent()->GetParent()) return;
	if(pxSource->GetParent()->GetParent()->GetType() != Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_WIDGET_LISTBOX) return;
	const Tools_GUIMenu_Widget_ListBox* pxSourceListBox = (const Tools_GUIMenu_Widget_ListBox*) pxSource->GetParent()->GetParent();

	pxDestListBox->CopyPopulationFunction(pxSourceListBox,pxSource->GetID(),pxDest->GetID());
	pxDestListBox->CopyVisibilityFunction(pxSourceListBox,pxSource->GetID(),pxDest->GetID());

	
}

// Tools_GUIMenu_Widget_ListBox ///////////////////////////////////////////////////////////////////////////

Tools_GUIMenu_Widget_ListBox::Tools_GUIMenu_Widget_ListBox(Asura_GUIMenu_Widget_ListBox* pxAsuraListBox,Tools_GUIMenu_Root* pxToolsParent)
: Tools_GUIMenu_Widget_Base( pxAsuraListBox, pxToolsParent )
, m_pxAsuraListBox( pxAsuraListBox )
, m_uMaxAllowedEntries( m_pxAsuraListBox->GetMaxAllowedEntries() )
, m_uMaxDisplayedEntries( m_pxAsuraListBox->GetMaxDisplayedEntries() )
, m_pszImageNames(0)
, m_bDecImageChanged(false)
, m_iCurrentDecImageIndex(0)
, m_bIncImageChanged(false)
, m_iCurrentIncImageIndex(0)
, m_pszPathNames(0)
, m_bScrollBarChanged(false)
, m_iCurrentScrollBarIndex(0)
, m_bMaxActionsChanged( false )
, m_fInitialRepeatDelay( fASURA_GUIMENU_WIDGET_LISTBOX_CONTROL_INITIAL_REPEAT_DELAY )
, m_fSecondaryRepeatDelay( fASURA_GUIMENU_WIDGET_LISTBOX_CONTROL_SECONDARY_REPEAT_DELAY )
, m_bInputDelayChanged( false )
{
	// destroy any left over entries
	m_pxAsuraListBox->ClearAllEntries();
	m_pxAsuraListBox->DeallocateEntryList();

	if( m_pxAsuraListBox->m_xEntryTemplate.GetNamePointer()->IsBlank() )
	{
		Asura_String xTemplateName = GetName();
		xTemplateName += "_Entry";

		m_pxAsuraListBox->m_xEntryTemplate.SetName(xTemplateName);
		m_pxAsuraListBox->m_xEntryTemplate.GenerateHashID();
	}
	else if(m_pxAsuraListBox->m_xEntryTemplate.GetID() == 0)
	{
		m_pxAsuraListBox->m_xEntryTemplate.GenerateHashID();
	}

	m_bChangedOffsetOri = false;
	m_bChangedOffsetVector_Downwards = false;
	m_bChangedOffsetVector_Right = false;
	m_fOffsetOri = Asura_Maths::ArcTangent(m_pxAsuraListBox->GetOffsetVector().x, m_pxAsuraListBox->GetOffsetVector().y);
	m_bOffsetVector_Right = (m_pxAsuraListBox->GetOffsetVector().x > 0.9999f);
	m_bOffsetVector_Downwards = (m_pxAsuraListBox->GetOffsetVector().y > 0.9999f);

	m_bLoopSelection = m_pxAsuraListBox->m_bLoopSelection;
	m_bCarousel = m_pxAsuraListBox->m_bCarousel;
	m_bAllowSubScroll = m_pxAsuraListBox->m_bAllowSubScroll;

	m_pxToolsListBoxEntryTemplate = static_cast<Tools_GUIMenu_Widget_ListBoxEntry*> (Tools_Mode_GUIMenu::CreateToolsItemsForAsuraItem( &m_pxAsuraListBox->m_xEntryTemplate, this ));

}


Tools_GUIMenu_Widget_ListBox::~Tools_GUIMenu_Widget_ListBox()
{	
	delete [] m_pszImageNames;
	delete [] m_pszPathNames;
}

void Tools_GUIMenu_Widget_ListBox::InitialiseNewItem()
{
	PARENT::InitialiseNewItem();

	//should have allow for at least one entry
	m_uMaxAllowedEntries = 1;
	m_uMaxDisplayedEntries = 1;
	m_pxAsuraListBox->SetMaxAllowedEntries(m_uMaxAllowedEntries);
	m_pxAsuraListBox->SetMaxDisplayedEntries(m_uMaxDisplayedEntries);
}

void Tools_GUIMenu_Widget_ListBox::Update()
{
	PARENT::Update();

	if (m_pxAsuraListBox)
	{
		if (m_bChanged)
		{
			if (m_bCarousel && m_bAllowSubScroll)
			{
				// NY - 14 January 2009 - allow only one of these
				m_bCarousel = m_pxAsuraListBox->m_bAllowSubScroll;
				m_bAllowSubScroll = !m_bCarousel;
			}
			m_pxAsuraListBox->m_bLoopSelection = m_bLoopSelection;
			m_pxAsuraListBox->m_bCarousel = m_bCarousel;
			m_pxAsuraListBox->m_bAllowSubScroll = m_bAllowSubScroll;
		}
		// NY - 06 August 2008 - this can't be safely re-used once the template changes, so nuke it when editing
		m_pxAsuraListBox->DeallocateEntryList();

		if (m_bChangedOffsetVector_Downwards)
		{
			m_bChangedOffsetVector_Downwards = false;
			if (m_bOffsetVector_Downwards)
			{
				m_pxAsuraListBox->SetOffsetVector(Asura_Vector_2(0.0f, 1.0f));
				m_bOffsetVector_Right = false;
			}
			else
			{
				m_pxAsuraListBox->SetOffsetVector(Asura_Vector_2(1.0f, 0.0f));
				m_bOffsetVector_Right = true;
			}
			m_fOffsetOri = Asura_Maths::ArcTangent(m_pxAsuraListBox->GetOffsetVector().x, m_pxAsuraListBox->GetOffsetVector().y);
		}
		if (m_bChangedOffsetVector_Right)
		{
			m_bChangedOffsetVector_Right = false;
			if (m_bOffsetVector_Right)
			{
				m_pxAsuraListBox->SetOffsetVector(Asura_Vector_2(1.0f, 0.0f));
				m_bOffsetVector_Downwards = false;
			}
			else
			{
				m_pxAsuraListBox->SetOffsetVector(Asura_Vector_2(0.0f, 1.0f));
				m_bOffsetVector_Downwards = true;
			}
			m_fOffsetOri = Asura_Maths::ArcTangent(m_pxAsuraListBox->GetOffsetVector().x, m_pxAsuraListBox->GetOffsetVector().y);
		}
		if (m_bChangedOffsetOri)
		{
			m_bChangedOffsetOri = false;
			Asura_Vector_2 xOffsetVector;
			Asura_Maths::SinCos(m_fOffsetOri, xOffsetVector.x, xOffsetVector.y);
			m_pxAsuraListBox->SetOffsetVector(xOffsetVector);
			m_bOffsetVector_Right = (xOffsetVector.x > 0.9999f);
			m_bOffsetVector_Downwards = (xOffsetVector.y > 0.9999f);
		}
	}

	if( m_bMaxActionsChanged)
	{
		Tools_GUIMenu_Root::DeleteOverlay();
		Tools_GUIMenu_Root::CreateOverlay();
		m_bMaxActionsChanged = false;
		m_bChanged = true;
		SetModified();
	}

	if( m_bInputDelayChanged)
	{
		m_pxAsuraListBox->SetInitialControlRepeatDelay( m_fInitialRepeatDelay);
		m_pxAsuraListBox->SetSecondaryControlRepeatDelay( m_fSecondaryRepeatDelay);
		m_bInputDelayChanged = false;
		m_bChanged = true;
		SetModified();
	}

	if( m_bDecImageChanged)
	{
		Asura_GUIMenu_Root* pxItem;
		Asura_Hash_ID uHashItem;
		Asura_Hash_ID uHashName;
		int iCount;
		u_int u;
		const Asura_Char** ppszList = Tools_GUIMenu_System::GetAllWidgetElementNames( m_pxAsuraListBox->GetID(), &iCount, Tools_GUIMenu_System::ELEMENT_FILTER_ALLOW_IMAGES);

		m_pxAsuraListBox->SetDecImage( NULL);

		uHashName = Asura_GetHashID( ppszList[m_iCurrentDecImageIndex]);
		for( u=0; u<m_pxAsuraListBox->GetContainerCount(); u++)
		{
			pxItem = m_pxAsuraListBox->GetContainerItem( u);
			uHashItem = Asura_GetHashID( pxItem->GetNameAsChar());
		
			if( uHashName == uHashItem)
			{
				m_pxAsuraListBox->SetDecImage( static_cast<Asura_GUIMenu_Element_Image*>(pxItem));
				break;
			}
		}

		m_bDecImageChanged = false;
		m_bChanged = true;
		SetModified();
	}

	if( m_bIncImageChanged)
	{
		Asura_GUIMenu_Root* pxItem;
		Asura_Hash_ID uHashItem;
		Asura_Hash_ID uHashName;
		int iCount;
		u_int u;
		const Asura_Char** ppszList = Tools_GUIMenu_System::GetAllWidgetElementNames( m_pxAsuraListBox->GetID(), &iCount, Tools_GUIMenu_System::ELEMENT_FILTER_ALLOW_IMAGES);

		m_pxAsuraListBox->SetIncImage( NULL);

		uHashName = Asura_GetHashID( ppszList[m_iCurrentIncImageIndex]);
		for( u=0; u<m_pxAsuraListBox->GetContainerCount(); u++)
		{
			pxItem = m_pxAsuraListBox->GetContainerItem( u);
			uHashItem = Asura_GetHashID( pxItem->GetNameAsChar());

			if( uHashName == uHashItem)
			{
				m_pxAsuraListBox->SetIncImage( static_cast<Asura_GUIMenu_Element_Image*>(pxItem));
				break;
			}
		}


		m_bIncImageChanged = false;
		m_bChanged = true;
		SetModified();
	}

	if( m_bScrollBarChanged)
	{
		Asura_GUIMenu_Root* pxItem;
		Asura_Hash_ID uHashItem;
		Asura_Hash_ID uHashName;
		int iCount;
		u_int u;
		const Asura_Char** ppszList = Tools_GUIMenu_System::GetAllWidgetElementNames( m_pxAsuraListBox->GetID(), &iCount, Tools_GUIMenu_System::ELEMENT_FILTER_ALLOW_PATH);

		m_pxAsuraListBox->SetScrollBar( NULL);

		uHashName = Asura_GetHashID( ppszList[m_iCurrentScrollBarIndex]);
		for( u=0; u<m_pxAsuraListBox->GetContainerCount(); u++)
		{
			pxItem = m_pxAsuraListBox->GetContainerItem( u);
			uHashItem = Asura_GetHashID( pxItem->GetNameAsChar());

			if( uHashName == uHashItem)
			{
				m_pxAsuraListBox->SetScrollBar( static_cast<Asura_GUIMenu_Element_Path*>(pxItem));
				break;
			}
		}


		m_bScrollBarChanged = false;
		m_bChanged = true;
		SetModified();
	}

	if (m_pxAsuraListBox)
	{
		m_pxAsuraListBox->UpdateEntryOffset();
	}
}

void Tools_GUIMenu_Widget_ListBox::AddOverlayControls(Asura_Overlay* pxOverlay,Asura_Vector_2& xOffset)
{
	PARENT::AddOverlayControls(pxOverlay,xOffset);

	Asura_Dialog* pxDialog;

	xOffset.y += Asura_Dialog::SmallSpacing();

	pxDialog = new Asura_StringDisplay_Dialog( ".:: Button Press Repeat Settings ::.", xOffset);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	m_fInitialRepeatDelay = m_pxAsuraListBox->GetInitialControlRepeatDelay();
	m_fSecondaryRepeatDelay = m_pxAsuraListBox->GetSecondaryControlRepeatDelay();

	pxDialog = new Asura_Numerical_Dialog("Initial Repeat Delay", xOffset, &m_fInitialRepeatDelay, 0.0f, 10.0f, 0.1f, "%2.2f", &m_bInputDelayChanged);
	pxDialog->SetIsEnabledFunction(CanEditInputRepeat,(int)this);
	pxDialog->SetHelpText("Sets the initial repeat delay of a button press");
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Secondary Repeat Delay", xOffset, &m_fSecondaryRepeatDelay, 0.0f, 10.0f, 0.1f, "%2.2f", &m_bInputDelayChanged);
	pxDialog->SetIsEnabledFunction(CanEditInputRepeat,(int)this);
	pxDialog->SetHelpText("Sets the secondary repeat delay of a button press");
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();
	xOffset.y += Asura_Dialog::SmallSpacing();

	pxDialog = new Asura_Numerical_Dialog("Max Entries",xOffset,&m_uMaxAllowedEntries,1,9999,1,"%d",&m_bChanged);
	pxDialog->SetIsEnabledFunction(CanEditMaxEntries,(int)this);
	pxDialog->SetHelpText("Maximum number of entries allowed in the list box");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Max Displayed",xOffset,&m_uMaxDisplayedEntries,1,9999,1,"%d",&m_bChanged);
	pxDialog->SetIsEnabledFunction(CanEditMaxDisplayed,(int)this);
	pxDialog->SetHelpText("Maximum number of entries to display at once");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Tickbox_Dialog("Downwards (default)", xOffset, &m_bOffsetVector_Downwards, &m_bChangedOffsetVector_Downwards);
	pxDialog->SetHelpText("List goes downwards, first items at top");
	pxDialog->SetIsEnabledFunction(CanEditOffsetVector,(int)this);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Tickbox_Dialog("Rightwards", xOffset, &m_bOffsetVector_Right, &m_bChangedOffsetVector_Right);
	pxDialog->SetHelpText("List goes to the right, first items at left");
	pxDialog->SetIsEnabledFunction(CanEditOffsetVector,(int)this);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_AngleNumerical_Dialog("Offset Orientation", xOffset, &m_fOffsetOri, -180.0f, 180.0f, 1.0f, "%.0f", &m_bChangedOffsetOri);
	pxDialog->SetIsEnabledFunction(CanEditOffsetVector,(int)this);
	pxDialog->SetHelpText("Angle list runs at (0 is straight down)");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Tickbox_Dialog("Loop Selection",xOffset,&m_bLoopSelection,&m_bChanged);
	pxDialog->SetHelpText("Toggles whether the selection loops around (pressing up at the top will take you to the bottom and vice-versa).");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Tickbox_Dialog("Carousel",xOffset,&m_bCarousel,&m_bChanged);
	pxDialog->SetHelpText("Enables Carousel behaviour - current selection always in middle etc.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Tickbox_Dialog("Allow sub scrolling",xOffset,&m_bAllowSubScroll,&m_bChanged);
	pxDialog->SetHelpText("Allows scrolling by partial elements - useful if element bigger than listbox.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_StringDisplay_Dialog( ".:: Flash Details ::.", xOffset);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();


	Tools_ColourPicker_Dialog* pxColourDialog = new Tools_ColourPicker_Dialog( "Flash Colour",
																			   xOffset,
																			   Asura_Vector_2(1.5f,1.0f),
																			   &m_pxAsuraListBox->GetFlashColourPointer()->m_fR,
																			   &m_pxAsuraListBox->GetFlashColourPointer()->m_fG,
																			   &m_pxAsuraListBox->GetFlashColourPointer()->m_fB,
																			   1.0f,
																			   &m_bChanged);
	pxColourDialog->EnableAlphaEditing(&m_pxAsuraListBox->GetFlashColourPointer()->m_fA);
	pxDialog->SetIsEnabledFunction(CanEditFlashDetails,(int)this);
	pxOverlay->AddDialog(pxColourDialog);

	Asura_Vector_2 xDims = pxColourDialog->GetMaxCoords();
	xDims.y = 0;

	pxDialog = new Asura_Numerical_Dialog("Flash Time", xOffset + xDims, m_pxAsuraListBox->GetFlashTimePointer(), 0.0f, 30.0f, 0.01f, "%3.2f", &m_bChanged);
	pxDialog->SetIsEnabledFunction(CanEditFlashDetails,(int)this);
	pxDialog->SetHelpText("Specifies the length of time the flash stays active");
	pxOverlay->AddDialog( pxDialog );

	xOffset.y += Asura_Dialog::DefaultSpacing();
	xOffset.y += Asura_Dialog::DefaultSpacing();

	int iCount = 0;

	delete [] m_pszImageNames;
	const char** pszName = Tools_GUIMenu_System::GetAllWidgetElementNames( m_pxAsuraListBox->GetID(), &iCount, Tools_GUIMenu_System::ELEMENT_FILTER_ALLOW_IMAGES);
	m_pszImageNames = new const char*[iCount];
	for(int i = 0; i < iCount; ++i)
	{
		m_pszImageNames[i] = pszName[i];
	}

	m_iCurrentDecImageIndex = 0;
	m_iCurrentIncImageIndex = 0;
	m_iCurrentScrollBarIndex = 0;

	const Asura_GUIMenu_Element_Image* pxDecImage = m_pxAsuraListBox->GetDecImage();
	Asura_Hash_ID uHashItem;
	Asura_Hash_ID uHashName;
	const Asura_Char* pcName;

	if( pxDecImage)
	{
		pcName = pxDecImage->GetNameAsChar();
		uHashName = Asura_GetHashID( pcName);

		for(int i = 0; i < iCount; ++i)
		{
			uHashItem = Asura_GetHashID( m_pszImageNames[i]);

			if( uHashName == uHashItem)
			{
				m_iCurrentDecImageIndex = i;
				break;
			}
		}
	}

	pxDialog = new Asura_DropDown_List_Dialog( xOffset,
												m_pszImageNames,
												&m_iCurrentDecImageIndex,
												iCount, 
												iCount,
												NULL, &m_bDecImageChanged, ".:: Decrement Icon - Image Element ::.");

	pxOverlay->AddDialog( pxDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();


	const Asura_GUIMenu_Element_Image* pxIncImage = m_pxAsuraListBox->GetIncImage();

	if( pxIncImage)
	{
		pcName = pxIncImage->GetNameAsChar();
		uHashName = Asura_GetHashID( pcName);

		for(int i = 0; i < iCount; ++i)
		{
			uHashItem = Asura_GetHashID( m_pszImageNames[i]);

			if( uHashName == uHashItem)
			{
				m_iCurrentIncImageIndex = i;
				break;
			}
		}
	}

	pxDialog = new Asura_DropDown_List_Dialog( xOffset,
												m_pszImageNames,
												&m_iCurrentIncImageIndex,
												iCount, 
												iCount,
												NULL, &m_bIncImageChanged, ".:: Increment Icon - Image Element ::.");

	pxOverlay->AddDialog( pxDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();


	int iPathCount = 0;
	delete [] m_pszPathNames;
	pszName = Tools_GUIMenu_System::GetAllWidgetElementNames( m_pxAsuraListBox->GetID(), &iPathCount, Tools_GUIMenu_System::ELEMENT_FILTER_ALLOW_PATH);
	m_pszPathNames = new const char*[iPathCount];
	for(int i = 0; i < iPathCount; ++i)
	{
		m_pszPathNames[i] = pszName[i];
	}

	const Asura_GUIMenu_Element_Path* pxScrollBar = m_pxAsuraListBox->GetScrollBar();

	if( pxScrollBar)
	{
		pcName = pxScrollBar->GetNameAsChar();
		uHashName = Asura_GetHashID( pcName);

		for(int i = 0; i < iPathCount; ++i)
		{
			uHashItem = Asura_GetHashID( m_pszPathNames[i]);

			if( uHashName == uHashItem)
			{
				m_iCurrentScrollBarIndex = i;
				break;
			}
		}
	}

	pxDialog = new Asura_DropDown_List_Dialog( xOffset,
												m_pszPathNames,
												&m_iCurrentScrollBarIndex,
												iPathCount, 
												iPathCount,
												NULL, &m_bScrollBarChanged, ".:: Scroll Bar - Path Element ::.");
	pxDialog->SetHelpText("Adding a scrollbar causes the increment and decrement icons to act as scroll down and up.");

	pxOverlay->AddDialog( pxDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();

	// don't allow arguments to be edited as they must be the index of the selected listbox item
	m_pxAsuraListBox->m_xCommandManager.AddOverlayControls( pxOverlay, xOffset, false, false, &m_bChanged, &m_bMaxActionsChanged);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	m_pxAsuraListBox->m_xInitValueVar.AddOverlayControls( pxOverlay,  
													   xOffset,
													   "Initial value var",
													   "The console variable to take the initial value from\n(optional, can be same as current value var)",
													   Asura_CommandConsole_VarRepository::Var::TYPE_INT,
													   &m_bChanged);

	m_pxAsuraListBox->m_xCurrentValueVar.AddOverlayControls( pxOverlay,  
													   xOffset,
													   "Record to Var",
													   "The console variable to record the current value to\n(optional, can be same as initial value var)",
													   Asura_CommandConsole_VarRepository::Var::TYPE_INT,
													   &m_bChanged);

}

void Tools_GUIMenu_Widget_ListBox::CopyToAsura()
{
	PARENT::CopyToAsura();

	m_pxAsuraListBox->SetMaxAllowedEntries(m_uMaxAllowedEntries);
	m_pxAsuraListBox->SetMaxDisplayedEntries(m_uMaxDisplayedEntries);
	m_pxAsuraListBox->m_bLoopSelection = m_bLoopSelection;
	m_pxAsuraListBox->m_bCarousel = m_bCarousel;
	m_pxAsuraListBox->m_bAllowSubScroll = m_bAllowSubScroll;
}

void Tools_GUIMenu_Widget_ListBox::CopyFromAsura()
{
	PARENT::CopyFromAsura();

	m_uMaxAllowedEntries = m_pxAsuraListBox->GetMaxAllowedEntries();
	m_uMaxDisplayedEntries = m_pxAsuraListBox->GetMaxDisplayedEntries();
	m_bLoopSelection = m_pxAsuraListBox->m_bLoopSelection;
	m_bCarousel = m_pxAsuraListBox->m_bCarousel;
	m_bAllowSubScroll = m_pxAsuraListBox->m_bAllowSubScroll;
}

void Tools_GUIMenu_Widget_ListBox::SetRectangle(const Asura_Bounding_Rectangle& xRect)
{
	PARENT::SetRectangle(xRect);

	//update the max displayed entries so that the entry height is recalculated
	m_pxAsuraListBox->SetMaxDisplayedEntries(m_uMaxDisplayedEntries);
}

void Tools_GUIMenu_Widget_ListBox::Copy(const Tools_GUIMenu_Root* pxSource)
{
	const Tools_GUIMenu_Widget_ListBox* pxSourceListBox = static_cast<const Tools_GUIMenu_Widget_ListBox*> (pxSource);
	const Asura_GUIMenu_Widget_ListBox* pxSourceAsuraListBox = (const Asura_GUIMenu_Widget_ListBox*) pxSourceListBox->m_pxAsuraListBox;

#if 0
	// manually copy the entry template
	m_pxToolsListBoxEntryTemplate->Copy( pxSourceListBox->m_pxToolsListBoxEntryTemplate );

	//create copies of the entry template's children
	for(CLIF<Tools_GUIMenu_Root*> xChildLIF(pxSourceListBox->m_pxToolsListBoxEntryTemplate->GetChildList());!xChildLIF.done();xChildLIF.next())
	{
		Tools_Mode_GUIMenu::GetInstance()->CreateCopy(m_pxToolsListBoxEntryTemplate,xChildLIF(),0);
	}

	// copy population functions to for entry template children
	CLIF<Tools_GUIMenu_Root*> xSourceChildLIF(pxSourceListBox->m_pxToolsListBoxEntryTemplate->GetChildList());
	CLIF<Tools_GUIMenu_Root*> xDestChildLIF(m_pxToolsListBoxEntryTemplate->GetChildList());
	while( !xSourceChildLIF.done() )
	{
		Asura_GUIMenu_ConsoleCmd* pxDestCmd = m_pxAsuraListBox->GetPopulationFunctionForItem( xDestChildLIF()->GetID() );
		const Asura_GUIMenu_ConsoleCmd* pxSourceCmd = pxSourceListBox->m_pxAsuraListBox->GetPopulationFunctionForItem( xSourceChildLIF()->GetID() );
		
		Asura_Assert( pxDestCmd && pxSourceCmd, "Failed to copy population functions correctly." );
		if( pxDestCmd && pxSourceCmd )
		{
			pxDestCmd->Copy( pxSourceCmd );	
		}		
		
		xSourceChildLIF.next();		
		xDestChildLIF.next();
	}
#endif

	PARENT::Copy(pxSource);

	//copy tools data
	m_uMaxAllowedEntries = pxSourceListBox->m_pxAsuraListBox->m_uMaxAllowedEntries;
	m_uMaxDisplayedEntries = pxSourceListBox->m_pxAsuraListBox->m_uMaxDisplayedEntries;

	int iOrgCount = 0;
	int iCopyCount = 0;
	const Asura_Char** ppszOrgList = Tools_GUIMenu_System::GetAllWidgetElementNames( pxSourceAsuraListBox->GetID(), &iOrgCount, Tools_GUIMenu_System::ELEMENT_FILTER_ALLOW_IMAGES);
	Tools_GUIMenu_System::GetAllWidgetElementNames( m_pxAsuraListBox->GetID(), &iCopyCount, Tools_GUIMenu_System::ELEMENT_FILTER_ALLOW_IMAGES);

	// -- Calc original indecies for Dec/Inc values --
	Asura_Hash_ID uHashOrgDecName = ASURA_HASH_ID_UNSET;
	Asura_Hash_ID uHashOrgIncName = ASURA_HASH_ID_UNSET;
	int iOrgDecIndex = 999999;
	int iOrgIncIndex = 999999;

	if(	pxSourceAsuraListBox->m_pxDecImage != NULL)
	{
		uHashOrgDecName = Asura_GetHashID( pxSourceAsuraListBox->m_pxDecImage->GetNameAsChar());
	}
	if(	pxSourceAsuraListBox->m_pxIncImage != NULL)
	{
		uHashOrgIncName = Asura_GetHashID( pxSourceAsuraListBox->m_pxIncImage->GetNameAsChar());
	}

	Asura_Hash_ID uHashItemName;

	for( int i=0; i<iOrgCount; i++)
	{
		uHashItemName = Asura_GetHashID( ppszOrgList[i]);

		if( uHashItemName == uHashOrgDecName)
		{
			iOrgDecIndex = i-1;
		}

		if( uHashItemName == uHashOrgIncName)
		{
			iOrgIncIndex = i-1;
		}
	}


	if( iOrgDecIndex < iCopyCount)
	{
		Asura_GUIMenu_Root* pxItem = m_pxAsuraListBox->GetContainerItem( iOrgDecIndex);
		m_pxAsuraListBox->SetDecImage( static_cast<Asura_GUIMenu_Element_Image*>(pxItem));
	}

	if( iOrgIncIndex < iCopyCount)
	{
		Asura_GUIMenu_Root* pxItem = m_pxAsuraListBox->GetContainerItem( iOrgIncIndex);
		m_pxAsuraListBox->SetIncImage( static_cast<Asura_GUIMenu_Element_Image*>(pxItem));
	}

	int iOrgPathCount = 0;
	int iCopyPathCount = 0;
	const Asura_Char** ppszOrgPathList = Tools_GUIMenu_System::GetAllWidgetElementNames( pxSourceAsuraListBox->GetID(), &iOrgPathCount, Tools_GUIMenu_System::ELEMENT_FILTER_ALLOW_PATH);
	Tools_GUIMenu_System::GetAllWidgetElementNames( m_pxAsuraListBox->GetID(), &iCopyPathCount, Tools_GUIMenu_System::ELEMENT_FILTER_ALLOW_PATH);

	Asura_Hash_ID uHashOrgScrName = ASURA_HASH_ID_UNSET;
	int iOrgScrIndex = 999999;

	if(	pxSourceAsuraListBox->m_pxScrollBar != NULL)
	{
		uHashOrgScrName = Asura_GetHashID( pxSourceAsuraListBox->m_pxScrollBar->GetNameAsChar());
	}

	for( int i=0; i<iOrgPathCount; i++)
	{
		uHashItemName = Asura_GetHashID( ppszOrgPathList[i]);

		if( uHashItemName == uHashOrgScrName)
		{
			iOrgScrIndex = i-1;
		}
	}

	if( iOrgScrIndex < iCopyPathCount)
	{
		Asura_GUIMenu_Root* pxItem = m_pxAsuraListBox->GetContainerItem( iOrgScrIndex);
		m_pxAsuraListBox->SetScrollBar( static_cast<Asura_GUIMenu_Element_Path*>(pxItem));
	}
}

void Tools_GUIMenu_Widget_ListBox::CopyPopulationFunction(const Tools_GUIMenu_Widget_ListBox* pxSourceListBox, Asura_Hash_ID uSourceID, Asura_Hash_ID uDestID)
{
	Asura_GUIMenu_ConsoleCmd* pxDestCmd = m_pxAsuraListBox->GetPopulationFunctionForItem( uDestID );
	const Asura_GUIMenu_ConsoleCmd* pxSourceCmd = pxSourceListBox->m_pxAsuraListBox->GetPopulationFunctionForItem( uSourceID );
		
	Asura_Assert( pxDestCmd && pxSourceCmd, "Failed to copy population functions correctly." );
	if( pxDestCmd && pxSourceCmd )
	{
		pxDestCmd->Copy( pxSourceCmd );	
	}		
}

void Tools_GUIMenu_Widget_ListBox::CopyVisibilityFunction(const Tools_GUIMenu_Widget_ListBox* pxSourceListBox, Asura_Hash_ID uSourceID, Asura_Hash_ID uDestID)
{
	Asura_GUIMenu_ConsoleCmd* pxDestCmd = m_pxAsuraListBox->GetVisibilityFunctionForItem( uDestID );
	const Asura_GUIMenu_ConsoleCmd* pxSourceCmd = pxSourceListBox->m_pxAsuraListBox->GetVisibilityFunctionForItem( uSourceID );
		
	Asura_Assert( pxDestCmd && pxSourceCmd, "Failed to copy Visibility functions correctly." );
	if( pxDestCmd && pxSourceCmd )
	{
		pxDestCmd->Copy( pxSourceCmd );	
	}		
}

bool Tools_GUIMenu_Widget_ListBox::IsNavExitSupported(int iLink) const
{
	return PARENT::IsNavExitSupported(iLink);
}

u_int Tools_GUIMenu_Widget_ListBox::GetFlagsToEdit() const
{
	return PARENT::GetFlagsToEdit() | ASURA_GUIMENU_ACTION_FLAG_NOTIFY_DATA_CHANGE_IMMEDIATE | ASURA_GUIMENU_ACTION_FLAG_NOTIFY_DATA_CHANGE_ON_EXIT;
}

const char* Tools_GUIMenu_Widget_ListBox::GetLocalDataFlagName(u_int uFlagValue) const
{
	switch(uFlagValue)
	{
	case Asura_GUIMenu_Widget_ListBox::ASURA_GUIMENU_WIDGET_LISTBOX_LOCAL_FLAG_WIDTH :
		return "Width";
		break;

	case Asura_GUIMenu_Widget_ListBox::ASURA_GUIMENU_WIDGET_LISTBOX_LOCAL_FLAG_HEIGHT :
		return "Height";
		break;

	case Asura_GUIMenu_Widget_ListBox::ASURA_GUIMENU_WIDGET_LISTBOX_LOCAL_FLAG_MAX_ALLOWED_ENTRIES :
		return "Max Entries";
		break;

	case Asura_GUIMenu_Widget_ListBox::ASURA_GUIMENU_WIDGET_LISTBOX_LOCAL_FLAG_MAX_DISPLAY_ENTRIES :
		return "Max Displayed Entries";
		break;

	case Asura_GUIMenu_Widget_ListBox::ASURA_GUIMENU_WIDGET_LISTBOX_LOCAL_FLAG_FLASH_DETAILS :
		return "Flash details";
		break;

	case Asura_GUIMenu_Widget_ListBox::ASURA_GUIMENU_WIDGET_LISTBOX_LOCAL_FLAG_OFFSET_VECTOR :
		return "Direction";
		break;

	default :
		return PARENT::GetLocalDataFlagName(uFlagValue);
	}
}

bool Tools_GUIMenu_Widget_ListBox::CanEditMaxEntries()
{
	return ShouldStyleUseLocalDataFor(Asura_GUIMenu_Widget_ListBox::ASURA_GUIMENU_WIDGET_LISTBOX_LOCAL_FLAG_MAX_ALLOWED_ENTRIES);
}

bool Tools_GUIMenu_Widget_ListBox::CanEditMaxDisplayed()
{
	return ShouldStyleUseLocalDataFor(Asura_GUIMenu_Widget_ListBox::ASURA_GUIMENU_WIDGET_LISTBOX_LOCAL_FLAG_MAX_DISPLAY_ENTRIES);
}

bool Tools_GUIMenu_Widget_ListBox::CanEditWidth()
{
	return ShouldStyleUseLocalDataFor(Asura_GUIMenu_Widget_ListBox::ASURA_GUIMENU_WIDGET_LISTBOX_LOCAL_FLAG_WIDTH);
}

bool Tools_GUIMenu_Widget_ListBox::CanEditHeight()
{
	return ShouldStyleUseLocalDataFor(Asura_GUIMenu_Widget_ListBox::ASURA_GUIMENU_WIDGET_LISTBOX_LOCAL_FLAG_HEIGHT);
}

bool Tools_GUIMenu_Widget_ListBox::CanEditInputRepeat()
{
	return ShouldStyleUseLocalDataFor(Asura_GUIMenu_Widget_ListBox::ASURA_GUIMENU_WIDGET_LISTBOX_LOCAL_FLAG_CONTROL_REPEAT);
}

bool Tools_GUIMenu_Widget_ListBox::CanEditFlashDetails()
{
	return ShouldStyleUseLocalDataFor(Asura_GUIMenu_Widget_ListBox::ASURA_GUIMENU_WIDGET_LISTBOX_LOCAL_FLAG_FLASH_DETAILS);
}

bool Tools_GUIMenu_Widget_ListBox::CanEditOffsetVector()
{
	return ShouldStyleUseLocalDataFor(Asura_GUIMenu_Widget_ListBox::ASURA_GUIMENU_WIDGET_LISTBOX_LOCAL_FLAG_OFFSET_VECTOR);
}

void Tools_GUIMenu_Widget_ListBox::Rename()
{
	PARENT::Rename();

	Asura_String xTemplateName = GetName();
	xTemplateName += "_Entry";

	m_pxToolsListBoxEntryTemplate->SetName( xTemplateName.GetString() );
}

void Tools_GUIMenu_Widget_ListBox::RenderUI()
{
	PARENT::RenderUI();

	RenderOffset();
}

void Tools_GUIMenu_Widget_ListBox::RenderOffset()
{
	if (m_pxAsuraListBox && m_pxToolsListBoxEntryTemplate)
	{
		Asura_Bounding_Rectangle xRect(m_pxToolsListBoxEntryTemplate->GetRectangle());
		for (u_int u = 0; u < m_uMaxDisplayedEntries; ++u)
		{
			RenderRectangle(Tools_Mode_GUIMenu::GetInstance()->ConvertFromMenuCoords(xRect), 0x7f00ff00);
			xRect.Displace(m_pxAsuraListBox->GetOffsetVector() * m_pxAsuraListBox->GetEntryOffset());
		}
	}
}
#endif //ASURA_USE_GUI_MENU_SYSTEM
/***************/
/* End of File */
/***************/
