//
// Tools_GUIMenu_Widget_Numeric.h
//
// Created by Steven Keen - 21/02/2007
// 
// Copyright (C) 2007 Rebellion (Derby), All Rights Reserved.
//

#include "Core/Asura.h"
#include "Tools_GUIMenu_Widget_Numeric.h"

#ifdef ASURA_USE_GUI_MENU_SYSTEM
#include "Interface/Asura_CommandConsole.h"

#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Overlay.h"
#include "Interface/Tools_ColourPicker.h"

#include "GUIMenu/Asura_GUIMenu_System.h"
#include "GUIMenu/Asura_GUIMenu_Menu_Page.h"
#include "GUIMenu/Asura_GUIMenu_Widget_Numeric.h"

#include "GUIMenu/Tools_GUIMenu_Element_Text.h"
#include "GUIMenu/Tools_GUIMenu_Element_Image.h"
#include "GUIMenu/Tools_GUIMenu_CommandConsole.h"
#include "GUIMenu/Tools_GUIMenu_PreviewOverlay.h"

// ---------------------------------------------------------------------------------------

static const Asura_Char* g_apcTextOutputTypes[] =
{
	"Integer",
	"Floating Point"
};

static const u_int g_uNumTextOutputTypes = 2;

// ---------------------------------------------------------------------------------------

Tools_GUIMenu_Widget_Numeric::Tools_GUIMenu_Widget_Numeric(Asura_GUIMenu_Widget_Numeric* pxAsuraProgressBar,Tools_GUIMenu_Root* pxToolsParent)
:Tools_GUIMenu_Widget_Base(pxAsuraProgressBar,pxToolsParent)
{
	m_pxAsuraNumeric = pxAsuraProgressBar;

	m_fEditMin = 0.0f;
	m_fEditMax = 100.0f;
	m_fEditStep = 1.0f;

	m_bChangedMin =
	m_bChangedMax = 
	m_bChangedStep = false;
	m_bChangedConsoleVar = false;
	m_bChangedUseConsoleVarMinAndMax = false;

	m_iTextOutputTypeIndex = 0;
	m_bTextOutputTypeChanged = false;

	m_uPrecision = ASURA_GUIMENU_WIDGET_NUMERIC_DEFAULT_FLOATING_POINT_PRECISION;
	m_bChangedPrecision = false;

	m_xFlashColour.Set( 1.0f, 0.0f, 0.0f, 1.0f);
	m_bFlashColourChanged = false;

	m_fFlashTime = fASURA_GUIMENU_WIDGET_NUMERIC_FLASH_TIME_DEFAULT;
	m_bFlashTimeChanged = false;

	m_fInitialRepeatDelay = fASURA_GUIMENU_WIDGET_NUMERIC_CONTROL_INITIAL_REPEAT_DELAY;
	m_fSecondaryRepeatDelay = fASURA_GUIMENU_WIDGET_NUMERIC_CONTROL_SECONDARY_REPEAT_DELAY;
	m_bInputDelayChanged = false;

}

// -------------------------------------------------------------------------------------------------------

void Tools_GUIMenu_Widget_Numeric::InitialiseNewItem( void)
{
	PARENT::InitialiseNewItem();

	// SK 22.2.07 - I don't like doing this here but it'll do for now --
	// This sets up a basic 4 element numeric widget.
	// Element 1 - Text element that receives the changing numeric data as text strings.
	// Element 2 - text element that works as a label
	// Element 3 - Image element that will flash when the current value decreases
	// Element 4 - Image element that will flash when the current value increases

	Asura_Bounding_Rectangle xNumericBBox;
	Asura_Bounding_Rectangle xImageBBox;
	Asura_GUIMenu_Element_Text* pxChangingTextElement;
	Asura_GUIMenu_Element_Text* pxLabelTextElement;

	Asura_RGBA xColour;
	Asura_String xTexName( "\\GUI\\Slider-back.tga");

	Asura_String xFontName;
	float fFontHeight;

	Asura_UnicodeChar awszText[128];

	const Asura_Font* pxFont = Asura_Font_System::GetFont( Tools_GUIMenu_Element_Text::s_iMostRecentSelectedFont);
	if(pxFont)
	{
		xFontName = pxFont->m_szFileName;
		fFontHeight = pxFont->GetFontHeight();
	}



	// -- Rough bounding box --
	xNumericBBox.MinX = xNumericBBox.MinY = 0.0f;
	xNumericBBox.MaxX = 150.0f;
	xNumericBBox.MaxY = fFontHeight;//40.0f;

	// -- setup common data --
	xColour.Set( 1.0f, 1.0f, 1.0f, 1.0f);

	Asura_Vector_2 xPos( 0.0f, 0.0f);
	Asura_Bounding_Rectangle xDummyRect(0.0f,0.0f,0.0f,0.0f);


	//create and add changeable text element
	pxChangingTextElement = static_cast<Asura_GUIMenu_Element_Text*>(Asura_GUIMenu_System::Create(Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_ELEMENT_TEXT,xDummyRect));
	pxChangingTextElement->SetName( "Dynamic_Text");
	m_pxAsuraRoot->AddToThisContainer(pxChangingTextElement);
	pxChangingTextElement->GenerateHashID();
	

	// -- Setup changable text element --
	wcsncpy( &awszText[0], L"Empty", 128);
	pxChangingTextElement->ChangeFlag( ASURA_GUIMENU_ACTION_FLAG_ALL, true);

	pxChangingTextElement->SetTextRenderFlag( ASURA_GUIMENU_TEXT_JUSTIFY_RIGHT, true);
	pxChangingTextElement->SetTextRenderFlag( ASURA_GUIMENU_TEXT_JUSTIFY_VERTICAL_CENTRE, true);
	m_pxAsuraNumeric->SetFeedbackTextElement( pxChangingTextElement);
	pxChangingTextElement->SetText( &awszText[0]);
	Asura_Bounding_Rectangle xBounds = pxChangingTextElement->GetUnscaledLocalBounds();
	xBounds.SetBottomRight(xBounds.GetTopLeft() + xNumericBBox.GetDimension());
	pxChangingTextElement->SetUnscaledLocalBounds( xBounds);
	pxChangingTextElement->SetFontByName( xFontName);

	//create and add label text element
	pxLabelTextElement = static_cast<Asura_GUIMenu_Element_Text*>(Asura_GUIMenu_System::Create(Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_ELEMENT_TEXT,xDummyRect));
	pxLabelTextElement->SetName( "Label_Text");
	m_pxAsuraRoot->AddToThisContainer(pxLabelTextElement);
	pxLabelTextElement->GenerateHashID();

	// -- Setup label text element --
	wcsncpy( &awszText[0], L"Label", 128);

	pxLabelTextElement->ChangeFlag( ASURA_GUIMENU_ACTION_FLAG_ALL, true);
	xBounds = pxLabelTextElement->GetUnscaledLocalBounds();
	xBounds.SetBottomRight(xBounds.GetTopLeft() + xNumericBBox.GetDimension());
	pxLabelTextElement->SetUnscaledLocalBounds( xBounds);
	pxLabelTextElement->SetTextRenderFlag( ASURA_GUIMENU_TEXT_JUSTIFY_LEFT, true);
	pxLabelTextElement->SetTextRenderFlag( ASURA_GUIMENU_TEXT_JUSTIFY_VERTICAL_CENTRE, true);
	pxLabelTextElement->SetText( &awszText[0]);
	pxLabelTextElement->SetFontByName( xFontName);
	

	//create and add decrement image
	Asura_GUIMenu_Element_Image* pxDecImageElement = static_cast<Asura_GUIMenu_Element_Image*>(Asura_GUIMenu_System::Create(Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_ELEMENT_IMAGE,xDummyRect));
	pxDecImageElement->SetName( "Decrement_Image");
	m_pxAsuraRoot->AddToThisContainer(pxDecImageElement);
	pxDecImageElement->GenerateHashID();

	
	// -- Setup Decrement image for flashing --
	xImageBBox.MinX = xImageBBox.MinY = 0.0f;
	xImageBBox.MaxX = fFontHeight; xImageBBox.MaxY = fFontHeight;

	pxDecImageElement->ChangeFlag( ASURA_GUIMENU_ACTION_FLAG_ALL, true);
	xBounds.SetTopLeft(Asura_Vector_2(-fFontHeight, 0.0f));
	xBounds.SetBottomRight(xBounds.GetTopLeft() + xImageBBox.GetDimension());
	pxDecImageElement->SetUnscaledLocalBounds( xBounds);
	m_pxAsuraNumeric->SetDecrementImage( pxDecImageElement);

	//create and add decrement image
	Asura_GUIMenu_Element_Image* pxIncImageElement = static_cast<Asura_GUIMenu_Element_Image*>(Asura_GUIMenu_System::Create(Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_ELEMENT_IMAGE,xDummyRect));
	pxIncImageElement->SetName( "Increment_Image");
	m_pxAsuraRoot->AddToThisContainer(pxIncImageElement);
	pxIncImageElement->GenerateHashID();

	// -- Setup Increment Image for flashing --
	pxIncImageElement->ChangeFlag( ASURA_GUIMENU_ACTION_FLAG_ALL, true);
	xBounds.SetTopLeft(Asura_Vector_2(xNumericBBox.MaxX, 0.0f));
	xBounds.SetBottomRight(xBounds.GetTopLeft() + xImageBBox.GetDimension());
	pxDecImageElement->SetUnscaledLocalBounds( xBounds);
	m_pxAsuraNumeric->SetIncrementImage( static_cast<Asura_GUIMenu_Element_Image*>(pxIncImageElement));

	m_pxAsuraNumeric->SetFlashColour( 1.0f, 0.0f, 0.0f, 1.0f);
	m_pxAsuraNumeric->SetMaximumValue( 100.0f);
	m_pxAsuraNumeric->SetMinimumValue( 0.0f);
	m_pxAsuraNumeric->SetStepValue( 1.0f);

	//create the tools items for all the child elements.
	//doing this last so that the above change to the asura items will get
	//copied across to the tools items correctly.
	//RAR 05/09/2007
	Tools_Mode_GUIMenu::GetInstance()->AllocateToolsMenuItem(pxChangingTextElement,this);
	Tools_Mode_GUIMenu::GetInstance()->AllocateToolsMenuItem(pxLabelTextElement,this);
	Tools_Mode_GUIMenu::GetInstance()->AllocateToolsMenuItem(pxDecImageElement,this);
	Tools_Mode_GUIMenu::GetInstance()->AllocateToolsMenuItem(pxIncImageElement,this);


	CalculateExtents();
}


// -------------------------------------------------------------------------------------------------------

void Tools_GUIMenu_Widget_Numeric::Update( void)
{
	if(m_bChangedConsoleVar || m_bChangedUseConsoleVarMinAndMax)
	{
		UpdateFromConsoleVarIfRequired();
		m_bChangedConsoleVar = false;
		m_bChangedUseConsoleVarMinAndMax = false;
		m_bChanged = true;
	}


	PARENT::Update();

	if( m_bChangedMax)
	{
		if( m_pxAsuraNumeric->IsFeedbackTextInteger())
		{
			if( m_iEditMax <= m_iEditMin)
			{
				m_iEditMax = m_iEditMin + 1;
			}

			m_fEditMax = (float)m_iEditMax;
		}
		else
		{
			if( m_fEditMax <= m_fEditMin)
			{
				m_fEditMax = m_fEditMin + 0.001f;
			}
		}

		m_pxAsuraNumeric->SetMaximumValue( m_fEditMax);	

		m_bChangedMax = false;
		m_bChanged = true;
		SetModified();
	}

	if( m_bChangedMin)
	{
		if( m_pxAsuraNumeric->IsFeedbackTextInteger())
		{
			if( m_iEditMin >=m_iEditMax)
			{
				m_iEditMin = m_iEditMax - 1;
			}

			m_fEditMin = (float)m_iEditMin;
		}
		else
		{
			if( m_fEditMin >= m_fEditMax)
			{
				m_fEditMin = m_fEditMax - 0.001f;
			}
		}

		m_pxAsuraNumeric->SetMinimumValue( m_fEditMin);	

		m_bChangedMin = false;
		m_bChanged = true;
		SetModified();
	}

	if( m_bChangedStep)
	{
		if( m_pxAsuraNumeric->IsFeedbackTextInteger())
		{
			// MGH 22/8/2007: Don't bother checking the range if it is very large (as we get an overflow).
			if (m_iEditMax != INT_MAX && m_iEditMin != INT_MIN)
			{
				int iRange = m_iEditMax - m_iEditMin;

				if( m_iEditStep > iRange)
				{
					m_iEditStep = iRange;
				}
			}

			m_fEditStep = (float)m_iEditStep;
		}
		else
		{
			// MGH 22/8/2007: Don't bother checking the range if it is very large (as we get an overflow).
			if (m_fEditMax != FLT_MAX && m_fEditMin != -FLT_MAX)
			{
				float fRange = m_fEditMax - m_fEditMin;

				if( m_fEditStep > fRange)
				{
					m_fEditStep = fRange;
				}
			}
		}

		m_pxAsuraNumeric->SetStepValue( m_fEditStep);	

		m_bChangedStep = false;
		m_bChanged = true;
		SetModified();
	}

	if( m_bTextOutputTypeChanged)
	{
		switch( m_iTextOutputTypeIndex)
		{
			case 1:	// floating point output
				m_pxAsuraNumeric->SetFeedbackTextAsFloatingPoint();
				break;

			case 0:	//integer output
			default:
				m_pxAsuraNumeric->SetFeedbackTextAsInteger();
				break;
		}

		Tools_GUIMenu_Root::DeleteOverlay();
		Tools_GUIMenu_Root::CreateOverlay();

		m_pxAsuraNumeric->m_xConsoleVar.SetName( "" );

		m_bTextOutputTypeChanged = false;
		m_bChanged = true;
		SetModified();
	}

	if( m_bChangedPrecision)
	{
		m_pxAsuraNumeric->SetFeedbackTextFloatingPointPrecision( m_uPrecision);
		m_bChangedPrecision = false;
		m_bChanged = true;
		SetModified();
	}

	if(m_bFlashColourChanged)
	{
		m_pxAsuraNumeric->SetFlashColour( m_xFlashColour.m_fR, m_xFlashColour.m_fG, m_xFlashColour.m_fB, m_xFlashColour.m_fA);
		m_bFlashColourChanged = false;
		m_bChanged = true;
		SetModified();
	}

	if( m_bFlashTimeChanged)
	{
		m_pxAsuraNumeric->SetFlashTime( m_fFlashTime);
		m_bFlashTimeChanged = false;
		m_bChanged = true;
		SetModified();
	}

	if( m_bInputDelayChanged)
	{
		m_pxAsuraNumeric->SetInitialControlRepeatDelay( m_fInitialRepeatDelay);
		m_pxAsuraNumeric->SetSecondaryControlRepeatDelay( m_fSecondaryRepeatDelay);
		m_bInputDelayChanged = false;
		m_bChanged = true;
		SetModified();
	}

}

// -------------------------------------------------------------------------------------------------------

void Tools_GUIMenu_Widget_Numeric::UpdateReferences(Asura_Hash_ID uOldID,Asura_Hash_ID uNewID, bool bPostCopy)
{
	PARENT::UpdateReferences( uOldID, uNewID, bPostCopy);
}

// -------------------------------------------------------------------------------------------------------

void Tools_GUIMenu_Widget_Numeric::AddOverlayControls(Asura_Overlay* pxOverlay,Asura_Vector_2& xOffset)
{
	Asura_Dialog* pxDialog;

	PARENT::AddOverlayControls(pxOverlay,xOffset);

	AddRangeOverlayControls( pxOverlay, xOffset);

	pxDialog = new Asura_StringDisplay_Dialog( ".:: Button Press Repeat Settings ::.", xOffset);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	m_fInitialRepeatDelay = m_pxAsuraNumeric->GetInitialControlRepeatDelay();
	m_fSecondaryRepeatDelay = m_pxAsuraNumeric->GetSecondaryControlRepeatDelay();

	pxDialog = new Asura_Numerical_Dialog("Initial Repeat Delay", xOffset, &m_fInitialRepeatDelay, 0.0f, 10.0f, 0.1f, "%2.2f", &m_bInputDelayChanged);
	pxDialog->SetIsEnabledFunction(CanEditInputRepeat,(int)this);
	pxDialog->SetHelpText("Sets the initial repeat delay of a button press");
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Secondary Repeat Delay", xOffset, &m_fSecondaryRepeatDelay, 0.0f, 10.0f, 0.1f, "%2.2f", &m_bInputDelayChanged);
	pxDialog->SetIsEnabledFunction(CanEditInputRepeat,(int)this);
	pxDialog->SetHelpText("Sets the secondary repeat delay of a button press");
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_StringDisplay_Dialog( ".:: Text Output Details ::.", xOffset);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	if( m_pxAsuraNumeric->IsFeedbackTextInteger() == true)
	{
		m_iTextOutputTypeIndex = 0;
	}
	else if( m_pxAsuraNumeric->IsFeedbackTextFloatingPoint() == true)
	{
		m_iTextOutputTypeIndex = 1;
	}

	Asura_Vector_2 xPos = xOffset;

	pxDialog = new Asura_DropDown_List_Dialog( xPos,
											   &g_apcTextOutputTypes[0],
											   &m_iTextOutputTypeIndex,
											   g_uNumTextOutputTypes, 
											   g_uNumTextOutputTypes,
											   NULL, &m_bTextOutputTypeChanged);

	pxDialog->SetIsEnabledFunction(CanEditFeedback,(int)this);
	pxOverlay->AddDialog( pxDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();

	Asura_Vector_2 xDims = pxDialog->GetMaxCoords();

	xPos.x += xDims.x;

	m_uPrecision = m_pxAsuraNumeric->GetFeedbackTextFloatingPointPrecision();

	pxDialog = new Asura_Numerical_Dialog("Precision", xPos, &m_uPrecision, 0 , ASURA_GUIMENU_WIDGET_NUMERIC_PRECISION_MAX-1, 1, "%d", &m_bChangedPrecision);
	pxDialog->SetIsEnabledFunction(CanEditFeedback,(int)this);
	pxDialog->SetHelpText("Sets the floating point precision");
	pxOverlay->AddDialog( pxDialog );


	pxDialog = new Asura_StringDisplay_Dialog( ".:: Flash Details ::.", xOffset);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	xPos = xOffset;

	m_xFlashColour = m_pxAsuraNumeric->GetFlashColour();

	Tools_ColourPicker_Dialog* pxColourDialog = new Tools_ColourPicker_Dialog( "Flash Colour",
																			   xOffset,
																			   Asura_Vector_2(1.5f,1.0f),
																			   &m_xFlashColour.m_fR,
																			   &m_xFlashColour.m_fG,
																			   &m_xFlashColour.m_fB,
																			   1.0f,
																			   &m_bFlashColourChanged);
	pxColourDialog->EnableAlphaEditing(&m_xFlashColour.m_fA);
	pxDialog->SetIsEnabledFunction(CanEditFlash,(int)this);
	pxOverlay->AddDialog(pxColourDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	xDims = pxColourDialog->GetMaxCoords();
	xPos.x += xDims.x;

	m_fFlashTime = m_pxAsuraNumeric->GetFlashTime();

	pxDialog = new Asura_Numerical_Dialog("Flash Time", xPos, &m_fFlashTime, 0.0f, 30.0f, 0.01f, "%3.2f", &m_bFlashTimeChanged);
	pxDialog->SetIsEnabledFunction(CanEditFlash,(int)this);
	pxDialog->SetHelpText("Specifies the length of time the flash stays active");
	pxOverlay->AddDialog( pxDialog );

	u_int uConsoleVarType = Asura_CommandConsole_VarRepository::Var::TYPE_FLOAT;

	if( m_pxAsuraNumeric->IsFeedbackTextInteger() == true)
	{
		uConsoleVarType = Asura_CommandConsole_VarRepository::Var::TYPE_INT;
	}

	m_pxAsuraNumeric->m_xConsoleVar.AddOverlayControls( pxOverlay, 
		xOffset, "Console Var", "The console variable that will be altered by this control", uConsoleVarType, &m_bChangedConsoleVar );

	pxDialog = new Asura_FlagTickbox_Dialog( "Update on var change", xOffset, m_pxAsuraNumeric->GetNumericFlagsAsPtr(), ASURA_GUIMENU_WIDGET_NUMERIC_FLAGS_CONSOLEVAR_UPDATES, &m_bChanged );
	pxDialog->SetIsEnabledFunction(CanEditFeedback,(int)this);
	pxDialog->SetHelpText("When ticked, will update the numeric widgets value on the console variables data changing.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_FlagTickbox_Dialog( "Always in edit mode when highlighted", xOffset, m_pxAsuraNumeric->GetNumericFlagsAsPtr(), ASURA_GUIMENU_WIDGET_NUMERIC_FLAGS_ALWAYS_IN_EDIT_MODE_WHEN_HIGHLIGHTED, &m_bChanged );
	pxDialog->SetIsEnabledFunction(CanEditFeedback,(int)this);
	pxDialog->SetHelpText( "When this slider is selected it will go directly to edit mode without requiring a button press to activate it" );
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();
}

// -------------------------------------------------------------------------------------------------------

void Tools_GUIMenu_Widget_Numeric::AddRangeOverlayControls(Asura_Overlay* pxOverlay,Asura_Vector_2& xOffset)
{
	Asura_Dialog* pxDialog;

	m_bChangedMin =
	m_bChangedMax = 
	m_bChangedStep = false;

	m_fEditMin = m_pxAsuraNumeric->GetMinimumValue();
	m_fEditMax = m_pxAsuraNumeric->GetMaximumValue();
	m_fEditStep = m_pxAsuraNumeric->GetStepValue();

	// MGH 22/8/2007: Many console variables have ranges going up to INT_MAX or FLT_MAX.
	// Repeated casting/conversion between float and integer often causes these values to be corrupted.
	// Restore some sensible defaults.
	if (m_fEditMin < INT_MIN)
	{
		m_iEditMin = INT_MIN;
	}
	else
	{
		m_iEditMin = (int)m_fEditMin;
	}
	
	if (m_fEditMax > INT_MAX)
	{
		m_iEditMax = INT_MAX;
	}
	else
	{
		m_iEditMax = (int)m_fEditMax;
		if (m_iEditMax == INT_MIN)
		{
			// MGH 22/8/2007: Fix corrupted value.
			m_iEditMax = INT_MAX;
		}
	}

	m_iEditStep = (int)m_fEditStep;
	if( m_fEditStep > 0.0f && m_fEditStep < 1.0f)
	{
		m_iEditStep = 1;
	}

	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_StringDisplay_Dialog( ".:: Range Details ::.", xOffset);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_FlagTickbox_Dialog( "Use Console Var Min and Max", xOffset, m_pxAsuraNumeric->GetNumericFlagsAsPtr(), ASURA_GUIMENU_WIDGET_NUMERIC_FLAGS_USE_CONSOLEVAR_MIN_AND_MAX, &m_bChangedUseConsoleVarMinAndMax );
	pxDialog->SetHelpText("If ticked then the min and max values will be taken from the console variable when the menu gets initialised.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();


	if( m_pxAsuraNumeric->IsFeedbackTextInteger())
	{
		m_pxAsuraNumeric->SetMinAndMaxValue( (float)m_iEditMin, (float)m_iEditMax);
		m_pxAsuraNumeric->SetStepValue( (float)m_iEditStep);


		pxDialog = new Asura_Numerical_Dialog("Minimum", xOffset, &m_iEditMin, 0, INT_MAX, 1, "%d", &m_bChangedMin);
		pxDialog->SetIsEnabledFunction(CanEditMinimum,(int)this);
		pxDialog->SetHelpText("Range Minimum");
		pxDialog->m_iControlID = Asura_GetHashID("RangeMin");
		pxOverlay->AddDialog( pxDialog );
		xOffset.y += Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_Numerical_Dialog("Maximum", xOffset, &m_iEditMax, 0, INT_MAX, 1, "%d", &m_bChangedMax);
		pxDialog->SetIsEnabledFunction(CanEditMaximum,(int)this);
		pxDialog->SetHelpText("Range Maximum");
		pxDialog->m_iControlID = Asura_GetHashID("RangeMax");
		pxOverlay->AddDialog( pxDialog );
		xOffset.y += Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_Numerical_Dialog("Step", xOffset, &m_iEditStep, 0, 9999, 1, "%d", &m_bChangedStep);
		pxDialog->SetIsEnabledFunction(CanEditStep,(int)this);
		pxDialog->SetHelpText("Step Increment");
		pxDialog->m_iControlID = Asura_GetHashID("RangeStep");
		pxOverlay->AddDialog( pxDialog );
		xOffset.y += Asura_Dialog::LargeSpacing();
	}
	else
	{
		pxDialog = new Asura_Numerical_Dialog("Minimum", xOffset, &m_fEditMin, 0.0f, 9999999.0f, 0.001f, "%3.3f", &m_bChangedMin);
		pxDialog->SetIsEnabledFunction(CanEditMinimum,(int)this);
		pxDialog->SetHelpText("Range Minimum");
		pxDialog->m_iControlID = Asura_GetHashID("RangeMin");
		pxOverlay->AddDialog( pxDialog );
		xOffset.y += Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_Numerical_Dialog("Maximum", xOffset, &m_fEditMax, 0.0f, 9999999.0f, 0.001f, "%3.3f", &m_bChangedMax);
		pxDialog->SetIsEnabledFunction(CanEditMaximum,(int)this);
		pxDialog->SetHelpText("Range Maximum");
		pxDialog->m_iControlID = Asura_GetHashID("RangeMax");
		pxOverlay->AddDialog( pxDialog );
		xOffset.y += Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_Numerical_Dialog("Step", xOffset, &m_fEditStep, 0.0f, 9999.0f, 0.001f, "%3.3f", &m_bChangedStep);
		pxDialog->SetIsEnabledFunction(CanEditStep,(int)this);
		pxDialog->SetHelpText("Step Increment");
		pxDialog->m_iControlID = Asura_GetHashID("RangeStep");
		pxOverlay->AddDialog( pxDialog );
		xOffset.y += Asura_Dialog::LargeSpacing();
	}
}

// -------------------------------------------------------------------------------------------------------

void Tools_GUIMenu_Widget_Numeric::Copy(const Tools_GUIMenu_Root* pxSource)
{
	const Tools_GUIMenu_Widget_Numeric* pxSourceNumeric = (const Tools_GUIMenu_Widget_Numeric*) pxSource;

	switch( pxSourceNumeric->m_iTextOutputTypeIndex)
	{
	case 1:	// floating point output
		m_pxAsuraNumeric->SetFeedbackTextAsFloatingPoint();
		break;

	case 0:	//integer output
	default:
		m_pxAsuraNumeric->SetFeedbackTextAsInteger();
		break;
	}

	PARENT::Copy(pxSource);

	//copy tools data
	m_fEditMin = pxSourceNumeric->m_fEditMin;
	m_fEditMax = pxSourceNumeric->m_fEditMax;
	m_fEditStep = pxSourceNumeric->m_fEditStep;
	
	m_iTextOutputTypeIndex = pxSourceNumeric->m_iTextOutputTypeIndex;
	m_uPrecision = pxSourceNumeric->m_uPrecision;
	m_xFlashColour = pxSourceNumeric->m_xFlashColour;

	m_fFlashTime = pxSourceNumeric->m_fFlashTime;
}

u_int Tools_GUIMenu_Widget_Numeric::GetFlagsToEdit() const
{
	return PARENT::GetFlagsToEdit() | ASURA_GUIMENU_ACTION_FLAG_NOTIFY_DATA_CHANGE_IMMEDIATE | ASURA_GUIMENU_ACTION_FLAG_NOTIFY_DATA_CHANGE_ON_EXIT;
}

const char* Tools_GUIMenu_Widget_Numeric::GetLocalDataFlagName(u_int uFlagValue) const
{
	switch(uFlagValue)
	{
		case Asura_GUIMenu_Widget_Numeric::ASURA_GUIMENU_WIDGET_NUMERIC_LOCAL_FLAG_MINIMUM :
			return "Minimum";
			break;

		case Asura_GUIMenu_Widget_Numeric::ASURA_GUIMENU_WIDGET_NUMERIC_LOCAL_FLAG_MAXIMUM :
			return "Maximum";
			break;

		case Asura_GUIMenu_Widget_Numeric::ASURA_GUIMENU_WIDGET_NUMERIC_LOCAL_FLAG_STEP :
			return "Step";
			break;

		case Asura_GUIMenu_Widget_Numeric::ASURA_GUIMENU_WIDGET_NUMERIC_LOCAL_FLAG_FEEDBACK_DETAILS :
			return "Feedback Details";
			break;

		case Asura_GUIMenu_Widget_Numeric::ASURA_GUIMENU_WIDGET_NUMERIC_LOCAL_FLAG_FLASH_DETAILS :
			return "Flash Details";
			break;

		case Asura_GUIMenu_Widget_Numeric::ASURA_GUIMENU_WIDGET_NUMERIC_LOCAL_FLAG_CONTROL_REPEAT :
			return "Input Repeat Details";
			break;

		default :
			return PARENT::GetLocalDataFlagName(uFlagValue);

	}
}

bool Tools_GUIMenu_Widget_Numeric::CanEditMinimum()
{
	if(m_pxAsuraNumeric->IsNumericFlagSet(ASURA_GUIMENU_WIDGET_NUMERIC_FLAGS_USE_CONSOLEVAR_MIN_AND_MAX)) return false;
	return ShouldStyleUseLocalDataFor(Asura_GUIMenu_Widget_Numeric::ASURA_GUIMENU_WIDGET_NUMERIC_LOCAL_FLAG_MINIMUM);
}

bool Tools_GUIMenu_Widget_Numeric::CanEditMaximum()
{
	if(m_pxAsuraNumeric->IsNumericFlagSet(ASURA_GUIMENU_WIDGET_NUMERIC_FLAGS_USE_CONSOLEVAR_MIN_AND_MAX)) return false;
	return ShouldStyleUseLocalDataFor(Asura_GUIMenu_Widget_Numeric::ASURA_GUIMENU_WIDGET_NUMERIC_LOCAL_FLAG_MAXIMUM);
}

bool Tools_GUIMenu_Widget_Numeric::CanEditStep()
{
	return ShouldStyleUseLocalDataFor(Asura_GUIMenu_Widget_Numeric::ASURA_GUIMENU_WIDGET_NUMERIC_LOCAL_FLAG_STEP);
}

bool Tools_GUIMenu_Widget_Numeric::CanEditFeedback()
{
	return ShouldStyleUseLocalDataFor(Asura_GUIMenu_Widget_Numeric::ASURA_GUIMENU_WIDGET_NUMERIC_LOCAL_FLAG_FEEDBACK_DETAILS);
}

bool Tools_GUIMenu_Widget_Numeric::CanEditFlash()
{
	return ShouldStyleUseLocalDataFor(Asura_GUIMenu_Widget_Numeric::ASURA_GUIMENU_WIDGET_NUMERIC_LOCAL_FLAG_FLASH_DETAILS);
}

bool Tools_GUIMenu_Widget_Numeric::CanEditInputRepeat()
{
	return ShouldStyleUseLocalDataFor(Asura_GUIMenu_Widget_Numeric::ASURA_GUIMENU_WIDGET_NUMERIC_LOCAL_FLAG_CONTROL_REPEAT);
}

void Tools_GUIMenu_Widget_Numeric::AddToPreviewOverlay() const
{
	PARENT::AddToPreviewOverlay();

	if(!m_pxAsuraNumeric->m_xConsoleVar.IsBlank())
	{
		u_int uConsoleVarType = Asura_CommandConsole_VarRepository::Var::TYPE_FLOAT;

		if( m_pxAsuraNumeric->IsFeedbackTextInteger() == true)
		{
			uConsoleVarType = Asura_CommandConsole_VarRepository::Var::TYPE_INT;
		}
		
		Tools_GUIMenu_PreviewOverlay::RegisterConsoleVariable(m_pxAsuraNumeric->m_xConsoleVar.GetName().GetString(),uConsoleVarType);
	}

}

void Tools_GUIMenu_Widget_Numeric::UpdateFromConsoleVarIfRequired()
{
	if(m_pxAsuraNumeric->IsNumericFlagSet(ASURA_GUIMENU_WIDGET_NUMERIC_FLAGS_USE_CONSOLEVAR_MIN_AND_MAX))
	{
		//update min/max in asura data from console var
		m_pxAsuraNumeric->InitFromConsoleVar();

		//and update local copy of min and max
		m_fEditMin = m_pxAsuraNumeric->GetMinimumValue();
		m_fEditMax = m_pxAsuraNumeric->GetMaximumValue();

		m_iEditMin = (int)m_fEditMin;
		m_iEditMax = (int)m_fEditMax;

		if (m_iEditMax == INT_MIN)
		{
			// MGH 22/8/2007: Fix corrupted value.
			m_iEditMax = INT_MAX;
		}
	}
}


#endif //ASURA_USE_GUI_MENU_SYSTEM

