// Tools_GUIMenu_Widget_Slider.cpp
//
// Created by Richard Rice - 05/12/2006
// 
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"
#include "Tools_GUIMenu_Widget_ProgressBar.h"

#ifdef ASURA_USE_GUI_MENU_SYSTEM
#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Overlay.h"

#include "GUIMenu/Asura_GUIMenu_Menu_Page.h"
#include "GUIMenu/Asura_GUIMenu_Widget_ProgressBar.h"
#include "GUIMenu/Tools_GUIMenu_PreviewOverlay.h"

// ---------------------------------------------------------------------------------------

struct ProgressBarAxisNameAndValue
{
	const Asura_Char* m_szName;
	u_int m_uValue;
};

static ProgressBarAxisNameAndValue g_axFlags[] =
{
	{"Axis Move - None", ASURA_GUIMENU_PROGRESSBAR_AXIS_NONE},
	{"Axis Move - North", ASURA_GUIMENU_PROGRESSBAR_AXIS_NORTH},
	{"Axis Move - East", ASURA_GUIMENU_PROGRESSBAR_AXIS_EAST},
	{"Axis Move - South", ASURA_GUIMENU_PROGRESSBAR_AXIS_SOUTH},
	{"Axis Move - West", ASURA_GUIMENU_PROGRESSBAR_AXIS_WEST}
};

static const Asura_Char* g_apcFlagNames[] =
{
	g_axFlags[0].m_szName,
	g_axFlags[1].m_szName,
	g_axFlags[2].m_szName,
	g_axFlags[3].m_szName,
	g_axFlags[4].m_szName
};

static const u_int g_uNumMenuFlags = sizeof(g_axFlags) / sizeof(ProgressBarAxisNameAndValue);

// ---------------------------------------------------------------------------------------

Tools_GUIMenu_Widget_ProgressBar::Tools_GUIMenu_Widget_ProgressBar(Asura_GUIMenu_Widget_ProgressBar* pxAsuraProgressBar,Tools_GUIMenu_Root* pxToolsParent)
:Tools_GUIMenu_Widget_Base(pxAsuraProgressBar,pxToolsParent)
{
u_int i;

	m_pxAsuraProgressBar = pxAsuraProgressBar;

	m_fEditMin =
	m_fEditMax =
	m_fEditStep = 0.0f;

	m_bChangedMin =
	m_bChangedMax = 
	m_bChangedStep = false;

	m_bChangedConsoleVar = false;

	m_uOverlayImageCount = 0;

	for( i=0; i<ASURA_GUIMENU_MAX_PROGRESSBAR_IMAGE_OVERLAYS; i++)
	{
		m_abOverlayImageChanged[i] = false;
		m_aiOverlayImageIndex[i] = 0;

		m_aiOverlayAxisIndex[i] = 0;
	}

	m_ppcOverlayImageOptions = NULL;


}


Tools_GUIMenu_Widget_ProgressBar::~Tools_GUIMenu_Widget_ProgressBar( void)
{
	if( m_ppcOverlayImageOptions != NULL)
	{
		delete []m_ppcOverlayImageOptions;
		m_ppcOverlayImageOptions = NULL;
	}
}

// -------------------------------------------------------------------------------------------------------

void Tools_GUIMenu_Widget_ProgressBar::Update( void)
{
u_int i;
const Asura_Char* pcName;
Asura_GUIMenu_Root* pxItem = NULL;

	PARENT::Update();

	if( m_bChangedMax)
	{
		if( m_fEditMax > m_fEditMin)
		{	m_pxAsuraProgressBar->SetMaximumValue( m_fEditMax);	}
		else
		{
			m_fEditMax = m_fEditMin + 0.001f;
		}

		m_bChangedMax = false;
		m_bChanged = true;
		SetModified();
	}

	if( m_bChangedMin)
	{
		if( m_fEditMin < m_fEditMax)
		{	m_pxAsuraProgressBar->SetMinimumValue( m_fEditMin);	}
		else
		{
			m_fEditMin = m_fEditMax - 0.001f;
		}

		m_bChangedMin = false;
		m_bChanged = true;
		SetModified();
	}

	if( m_bChangedStep)
	{
		float fRange = m_fEditMax - m_fEditMin;

        if( m_fEditStep <= fRange)
		{	m_pxAsuraProgressBar->SetStepValue( m_fEditStep);	}
		else
		{
			m_fEditStep = fRange;
		}

		m_bChangedStep = false;
		m_bChanged = true;
		SetModified();
	}

	if( m_ppcOverlayImageOptions != NULL)
	{
		for( i=0; i<ASURA_GUIMENU_MAX_PROGRESSBAR_IMAGE_OVERLAYS; i++)
		{
			Asura_Hash_ID uHashID = ASURA_HASH_ID_UNSET;
			Asura_GUIMenu_ProgressBar_Axis_Movement eAxis = ASURA_GUIMENU_PROGRESSBAR_AXIS_NONE;
			u_int uFlags = 0;

			if( m_abOverlayImageChanged[i] == true)
			{
				m_abOverlayImageChanged[i] = false;
				// -- a dropdown box index has changed --
				Asura_GUIMenu_ProgressBar_Overlay_Info* pxInfo = m_pxAsuraProgressBar->GetActiveOverlayImageDataPtr( i);

				pcName = m_ppcOverlayImageOptions[m_aiOverlayImageIndex[i]];
				pxItem = FindImageElementMatchFromName( pcName);

				if( pxItem != NULL)
				{
					uHashID = pxItem->GetID();
					eAxis = pxInfo->m_eAxisMovement;
					uFlags = pxInfo->m_uOverlayFlags;
				}

				m_pxAsuraProgressBar->SetActiveOverlayImageData( i, uHashID, eAxis, uFlags);
				SetModified();
			}
		}
	}

	if( m_bChangedConsoleVar && m_pxAsuraProgressBar )
	{
		Asura_CommandConsole_VarRepository::Var* pxConsoleVar( Asura_CommandConsole_VarRepository::Find( m_pxAsuraProgressBar->GetConsoleVar().GetHash() ) );

		Asura_Assert( pxConsoleVar,"Failed to find console var" );

		if( pxConsoleVar )
		{
			m_pxAsuraProgressBar->GetConsoleVar().Update();
			m_pxAsuraProgressBar->SetUpdatePositionFromConsoleVar( !m_pxAsuraProgressBar->GetConsoleVar().IsBlank() );
			
			m_bChanged = true;		
			SetModified();
		}
	}
}

void Tools_GUIMenu_Widget_ProgressBar::CalculateExtents()
{
	

	Tools_GUIMenu_Widget_Base::CalculateExtents();

	for( int i(0); i<ASURA_GUIMENU_MAX_PROGRESSBAR_IMAGE_OVERLAYS; ++i )
	{
		Asura_GUIMenu_ProgressBar_Overlay_Info* pxInfo = m_pxAsuraProgressBar->GetActiveOverlayImageDataPtr( i);
		m_pxAsuraProgressBar->SetActiveOverlayImageData( i, pxInfo->m_uHashImageID, pxInfo->m_eAxisMovement, pxInfo->m_uOverlayFlags);
	}

}

// -------------------------------------------------------------------------------------------------------

void Tools_GUIMenu_Widget_ProgressBar::NotifyParentOnDeselection( void)
{
	int i;

	for( i=0; i<ASURA_GUIMENU_MAX_PROGRESSBAR_IMAGE_OVERLAYS; i++)
	{
		Asura_GUIMenu_ProgressBar_Overlay_Info* pxInfo = m_pxAsuraProgressBar->GetActiveOverlayImageDataPtr( i);
		m_pxAsuraProgressBar->SetActiveOverlayImageData( i, pxInfo->m_uHashImageID, pxInfo->m_eAxisMovement, pxInfo->m_uOverlayFlags);
	}

	// implemented here to trap when an overlay image is possibly changed in some way.
	// we can't detect when a change happens to a child but we can trap when a child is deselected.
	PARENT::NotifyParentOnDeselection();	
	
}

void Tools_GUIMenu_Widget_ProgressBar::AddToPreviewOverlay() const
{
	PARENT::AddToPreviewOverlay();

	Tools_GUIMenu_PreviewOverlay::RegisterConsoleVariable( m_pxAsuraProgressBar->GetConsoleVar().GetName().GetString(),m_pxAsuraProgressBar->GetConsoleVar().GetType() );
}

// -------------------------------------------------------------------------------------------------------

void Tools_GUIMenu_Widget_ProgressBar::UpdateReferences(Asura_Hash_ID uOldID,Asura_Hash_ID uNewID,bool bPostCopy)
{
u_int i;
	PARENT::UpdateReferences( uOldID, uNewID, bPostCopy);

	for( i=0; i<ASURA_GUIMENU_MAX_PROGRESSBAR_IMAGE_OVERLAYS; i++)
	{
		Asura_GUIMenu_ProgressBar_Overlay_Info* pxInfo = m_pxAsuraProgressBar->GetActiveOverlayImageDataPtr( i);
		if( pxInfo != NULL)
		{
			if( pxInfo->m_uHashImageID == uOldID)
			{
				pxInfo->m_uHashImageID = uNewID;
			}
		}
	}
}

// -------------------------------------------------------------------------------------------------------

void Tools_GUIMenu_Widget_ProgressBar::AddOverlayControls(Asura_Overlay* pxOverlay,Asura_Vector_2& xOffset)
{
	PARENT::AddOverlayControls(pxOverlay,xOffset);

	Asura_Dialog* pxDialog;

	m_fEditMin = m_pxAsuraProgressBar->GetMinimumValue();
	m_fEditMax = m_pxAsuraProgressBar->GetMaximumValue();
	m_fEditStep = m_pxAsuraProgressBar->GetStepValue();

	m_bChangedMin = false;
	m_bChangedMax = false;
	m_bChangedStep = false;
	m_bChangedConsoleVar = false;

	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_StringDisplay_Dialog( ".:: Range Information ::.", xOffset);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Minimum", xOffset, &m_fEditMin, 0.0f, 9999.0f, 0.001f, "%3.3f", &m_bChangedMin);
	pxDialog->SetIsEnabledFunction(CanEditMinimum,(int)this);
	pxDialog->SetHelpText("Range Minimum");
	pxDialog->m_iControlID = Asura_GetHashID("RangeMin");
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Maximum", xOffset, &m_fEditMax, 0.0f, 9999.0f, 0.001f, "%3.3f", &m_bChangedMax);
	pxDialog->SetIsEnabledFunction(CanEditMaximum,(int)this);
	pxDialog->SetHelpText("Range Maximum");
	pxDialog->m_iControlID = Asura_GetHashID("RangeMax");
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Step", xOffset, &m_fEditStep, 0.0f, 9999.0f, 0.001f, "%3.3f", &m_bChangedStep);
	pxDialog->SetIsEnabledFunction(CanEditStep,(int)this);
	pxDialog->SetHelpText("Step Increment");
	pxDialog->m_iControlID = Asura_GetHashID("RangeStep");
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::LargeSpacing();

	AddImageManipulationControls( pxOverlay, xOffset);

	m_pxAsuraProgressBar->GetConsoleVar().AddOverlayControls( pxOverlay, 
		xOffset, "Console Var", "Set the value displayed by the progress bar from a console var",
		Asura_CommandConsole_VarRepository::Var::TYPE_INVALID, &m_bChangedConsoleVar );
}

// -------------------------------------------------------------------------------------------------------

bool bOffsetAAA = true;

void Tools_GUIMenu_Widget_ProgressBar::AddImageManipulationControls(Asura_Overlay* pxOverlay,Asura_Vector_2& xOffset)
{
Asura_Dialog* pxDialog;
Asura_GUIMenu_ProgressBar_Overlay_Info* pxInfo;
u_int i;
int iIndex;

	// -- image manipulation settings --
	BuildImageManipulationList();

	pxDialog = new Asura_StringDisplay_Dialog( ".:: Image Manipulation ::.", xOffset);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	for( i=0; i<ASURA_GUIMENU_MAX_PROGRESSBAR_IMAGE_OVERLAYS; i++)
	{
		m_abOverlayImageChanged[i] = false;
		m_aiOverlayImageIndex[i] = 0;

		m_aiOverlayAxisIndex[i] = 0;

		// -- add options --
		pxInfo = m_pxAsuraProgressBar->GetActiveOverlayImageDataPtr( i);

		iIndex = FindIndexMatchforImageName( pxInfo->m_uHashImageID);
		if( iIndex != -1)
		{
			m_aiOverlayImageIndex[i] = iIndex;
		}

		if( pxInfo != NULL)
		{
			pxDialog = new Asura_FlagTickbox_Dialog( "Clip UVs On Sizing", xOffset, &pxInfo->m_uOverlayFlags, ASURA_GUIMENU_PROGRESSBAR_FLAGS_CLIP_OVERLAY);
			pxDialog->SetIsEnabledFunction(CanEditOverlays,(int)this);
			pxOverlay->AddDialog(pxDialog);
			xOffset.y += Asura_Dialog::DefaultSpacing();

			pxDialog = new Asura_DropDown_List_Dialog( xOffset,
													m_ppcOverlayImageOptions,
													&m_aiOverlayImageIndex[i],
													Asura_Maths::Min( (u_int)10, m_uOverlayImageCount), 
													m_uOverlayImageCount,
													NULL, &m_abOverlayImageChanged[i]);
			pxDialog->SetIsEnabledFunction(CanEditOverlays,(int)this);
			pxOverlay->AddDialog( pxDialog);

			Asura_Vector_2 xDims = pxDialog->GetMaxCoords();
			Asura_Vector_2 xPos = xOffset;

			xPos.x += xDims.x;

			Asura_GUIMenu_ProgressBar_Overlay_Info* pxInfo = m_pxAsuraProgressBar->GetActiveOverlayImageDataPtr( i);

			pxDialog = new Asura_DropDown_List_Dialog( xPos,
													   &g_apcFlagNames[0],
													   static_cast<int*>(static_cast<void*>(&pxInfo->m_eAxisMovement)),//ick
													   g_uNumMenuFlags, 
													   g_uNumMenuFlags,
													   NULL, &m_abOverlayImageChanged[i]);

			pxDialog->SetIsEnabledFunction(CanEditOverlays,(int)this);
			pxOverlay->AddDialog( pxDialog);
			xOffset.y += Asura_Dialog::LargeSpacing();
		}
	}
}

// -------------------------------------------------------------------------------------------------------

void Tools_GUIMenu_Widget_ProgressBar::BuildImageManipulationList()
{
u_int i;
Asura_GUIMenu_Root* pxItem;

	if( m_ppcOverlayImageOptions != NULL)
	{
		delete []m_ppcOverlayImageOptions;
		m_ppcOverlayImageOptions = NULL;
	}

	m_ppcOverlayImageOptions = new const Asura_Char*[m_pxAsuraProgressBar->GetContainerCount()+1];

	m_ppcOverlayImageOptions[0] = "None";
	m_uOverlayImageCount = 1;

	for( i = 0; i < m_pxAsuraProgressBar->GetContainerCount(); i++)
	{
		pxItem = m_pxAsuraProgressBar->GetContainerItem( i);

		if( pxItem->GetType() == Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_ELEMENT_IMAGE)
		{
			m_ppcOverlayImageOptions[m_uOverlayImageCount] = pxItem->GetNameAsChar();
			m_uOverlayImageCount++;
		}
	}
}

// -------------------------------------------------------------------------------------------------------

int Tools_GUIMenu_Widget_ProgressBar::FindIndexMatchforImageName( Asura_Hash_ID uHashID)
{
u_int i, j;
Asura_GUIMenu_Root* pxItem;
const Asura_String* pxItemName;
int iRetVal = -1;

	for( i=0; i<m_pxAsuraProgressBar->GetContainerCount(); i++)
	{
		pxItem = m_pxAsuraProgressBar->GetContainerItem( i);
		
		if( pxItem->GetID() == uHashID)
		{
			// -- found match, so find the string index in m_apcOverlayImageOptions
			pxItemName = pxItem->GetNamePointer();

			for( j=0; j<m_uOverlayImageCount; j++)
			{
				if( !pxItemName->Compare( m_ppcOverlayImageOptions[j]))
				{
					iRetVal = j;
					break;
				}
			}
		}
	}

	return iRetVal;
}

// -------------------------------------------------------------------------------------------------------

Asura_GUIMenu_Root* Tools_GUIMenu_Widget_ProgressBar::FindImageElementMatchFromName( const Asura_Char* pcName)
{ 
u_int i;
Asura_GUIMenu_Root* pxItem;
Asura_GUIMenu_Root* pxRetItem = NULL;
const Asura_String* pxItemName;

	for( i=0; i<m_pxAsuraProgressBar->GetContainerCount(); i++)
	{
		pxItem = m_pxAsuraProgressBar->GetContainerItem( i);

		pxItemName = pxItem->GetNamePointer();

		if( !pxItemName->Compare( pcName))
		{
			pxRetItem = pxItem;
			break;
		}
	}

	return pxRetItem;
}

// -------------------------------------------------------------------------------------------------------

void Tools_GUIMenu_Widget_ProgressBar::Copy(const Tools_GUIMenu_Root* pxSource)
{
	PARENT::Copy(pxSource);

	const Tools_GUIMenu_Widget_ProgressBar* pxSourcePB = (const Tools_GUIMenu_Widget_ProgressBar*) pxSource;

	//copy tools data
	m_fEditMin = pxSourcePB->m_fEditMin;
	m_fEditMax = pxSourcePB->m_fEditMax;
	m_fEditStep = pxSourcePB->m_fEditStep;

//	for(int i=0;i<ASURA_GUIMENU_MAX_PROGRESSBAR_IMAGE_OVERLAYS;i++)
//	{
//		m_aiOverlayImageIndex[i] = pxSourcePB->m_aiOverlayImageIndex[i];
//		m_aiOverlayAxisIndex[i] = pxSourcePB->m_aiOverlayAxisIndex[i];
//	}
}

u_int Tools_GUIMenu_Widget_ProgressBar::GetFlagsToEdit() const
{
	return PARENT::GetFlagsToEdit() | ASURA_GUIMENU_ACTION_FLAG_NOTIFY_DATA_CHANGE_IMMEDIATE | ASURA_GUIMENU_ACTION_FLAG_NOTIFY_DATA_CHANGE_ON_EXIT;
}

const char* Tools_GUIMenu_Widget_ProgressBar::GetLocalDataFlagName(u_int uFlagValue) const
{
	switch(uFlagValue)
	{
		case Asura_GUIMenu_Widget_ProgressBar::ASURA_GUIMENU_WIDGET_PROGRESSBAR_LOCAL_FLAG_MINIMUM :
			return "Minimum";
			break;

		case Asura_GUIMenu_Widget_ProgressBar::ASURA_GUIMENU_WIDGET_PROGRESSBAR_LOCAL_FLAG_MAXIMUM :
			return "Maximum";
			break;

		case Asura_GUIMenu_Widget_ProgressBar::ASURA_GUIMENU_WIDGET_PROGRESSBAR_LOCAL_FLAG_STEP :
			return "Step";
			break;

		case Asura_GUIMenu_Widget_ProgressBar::ASURA_GUIMENU_WIDGET_PROGRESSBAR_LOCAL_FLAG_OVERLAYS :
			return "Overlays";
			break;

		default :
			return PARENT::GetLocalDataFlagName(uFlagValue);

	}
}

bool Tools_GUIMenu_Widget_ProgressBar::CanEditMinimum()
{
	return ShouldStyleUseLocalDataFor(Asura_GUIMenu_Widget_ProgressBar::ASURA_GUIMENU_WIDGET_PROGRESSBAR_LOCAL_FLAG_MINIMUM);
}

bool Tools_GUIMenu_Widget_ProgressBar::CanEditMaximum()
{
	return ShouldStyleUseLocalDataFor(Asura_GUIMenu_Widget_ProgressBar::ASURA_GUIMENU_WIDGET_PROGRESSBAR_LOCAL_FLAG_MAXIMUM);
}

bool Tools_GUIMenu_Widget_ProgressBar::CanEditStep()
{
	return ShouldStyleUseLocalDataFor(Asura_GUIMenu_Widget_ProgressBar::ASURA_GUIMENU_WIDGET_PROGRESSBAR_LOCAL_FLAG_STEP);
}

bool Tools_GUIMenu_Widget_ProgressBar::CanEditOverlays()
{
	return ShouldStyleUseLocalDataFor(Asura_GUIMenu_Widget_ProgressBar::ASURA_GUIMENU_WIDGET_PROGRESSBAR_LOCAL_FLAG_OVERLAYS);
}

#endif //ASURA_USE_GUI_MENU_SYSTEM

