//
// Tools_GUIMenu_Widget_TextBox.cpp
//
// Created by Richard Rice - 05/12/2006
// 
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"
#include "GUIMenu/Tools_GUIMenu_Widget_TextBox.h"

#ifdef ASURA_USE_GUI_MENU_SYSTEM

// asura
#include "GUIMenu/Asura_GUIMenu_Widget_TextBox.h"
#include "GUIMenu/Asura_GUIMenu_Element_Text.h"
#include "Interface/Asura_CommandConsole.h"
#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Overlay.h"

// tools
#include "GUIMenu/Tools_Mode_GUIMenu.h"
#include "GUIMenu/Tools_GUIMenu_PreviewOverlay.h"

Tools_GUIMenu_Widget_TextBox::Tools_GUIMenu_Widget_TextBox(Asura_GUIMenu_Widget_TextBox* pxAsuraTextBox,Tools_GUIMenu_Root* pxToolsParent)
:Tools_GUIMenu_Widget_Base(pxAsuraTextBox,pxToolsParent)
{
	Asura_Assert(pxAsuraTextBox, "null pointer");
	m_pxAsuraTextBox = pxAsuraTextBox;
	m_uMaxTextEntryLength = m_pxAsuraTextBox->GetTextEntryMaxChars();
}

void Tools_GUIMenu_Widget_TextBox::InitialiseNewItem()
{
	PARENT::InitialiseNewItem();

	//Create a text element to go with the text box
	Asura_Vector_2 xPos = m_pxAsuraTextBox->GetScaledWorldPos();
	Tools_GUIMenu_Root* pxItem = Tools_Mode_GUIMenu::GetInstance()->CreateItem(this,Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_ELEMENT_TEXT,xPos);
	if(pxItem)
	{

		Asura_GUIMenu_Root* pxRoot = pxItem->GetAsuraData();
		Asura_GUIMenu_Element_Text* pxText = static_cast<Asura_GUIMenu_Element_Text*>(pxRoot);
		pxText->SetText( L"This is example text set as active for a TextBox widget. You should be able to scroll up and down by holding down the 'X' button and pressing up and down on the D-Pad.");
		m_pxAsuraTextBox->SetActiveText(pxItem->GetAsuraData()->GetID());
	}
}

bool Tools_GUIMenu_Widget_TextBox::CanEditTextEntryDetails()
{
	return ShouldStyleUseLocalDataFor(Asura_GUIMenu_Widget_TextBox::ASURA_GUIMENU_WIDGET_TEXTBOX_LOCAL_FLAG_TEXTENTRY);
}

bool Tools_GUIMenu_Widget_TextBox::CanEditIsPasswordField()
{
	return m_pxAsuraTextBox->IsTextEntryBox();
}


void Tools_GUIMenu_Widget_TextBox::CheckRequiredReferences()
{
	PARENT::CheckRequiredReferences();

	//make sure the text box knows about a child text element
	if(!m_pxAsuraTextBox->GetActiveTextHashID())
	{
		//search for a child text element
		for(u_int i=0;i<m_pxAsuraTextBox->GetContainerCount();i++)		
		{
			Asura_GUIMenu_Root* pxChild = m_pxAsuraTextBox->GetContainerItem(i);
			if(pxChild->GetType() == Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_ELEMENT_TEXT)
			{
				m_pxAsuraTextBox->SetActiveText(pxChild->GetID());
				SetModified();
				break;
			}
		}
	}

}

void Tools_GUIMenu_Widget_TextBox::AddOverlayControls(Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset)
{
	PARENT::AddOverlayControls( pxOverlay, xOffset );

	Asura_Dialog* pxDialog;

	pxDialog = new Asura_Divider_Dialog( xOffset );
	pxOverlay->AddDialog(pxDialog);

	pxDialog = new Asura_Numerical_Dialog( "Text Entry Max Length", xOffset, &m_uMaxTextEntryLength, 1, 128, 1, "%u", &m_bChanged );
	pxDialog->SetHelpText("max number of characters that can be input in this box. leave as 0 if it's not a text-entry field");
	pxDialog->SetIsEnabledFunction(CanEditTextEntryDetails,(int)this);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	m_pxAsuraTextBox->GetConsoleVar().AddOverlayControls( pxOverlay, 
		xOffset, "Console Var", "The console variable that should be altered by this control",Asura_CommandConsole_VarRepository::Var::TYPE_STRING, &m_bChanged );

	pxDialog = new Asura_Tickbox_Dialog( "Is Password Field", xOffset, &m_pxAsuraTextBox->m_bIsPasswordField, &m_bChanged);
	pxDialog->SetHelpText("displays text entry fields as a row of ****s");
	pxDialog->SetIsEnabledFunction(CanEditIsPasswordField,(int)this);
	pxOverlay->AddDialog( pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Tickbox_Dialog( "Allow Blank String", xOffset, &m_pxAsuraTextBox->m_bAllowBlankString, &m_bChanged);
	pxDialog->SetHelpText("if ticked, blank strings are replaced by specified text ID");
	pxDialog->SetIsEnabledFunction(CanEditIsPasswordField,(int)this);	// CanEditIsPasswordField returns true iff is textentry box
	pxOverlay->AddDialog( pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();
}


const char* Tools_GUIMenu_Widget_TextBox::GetLocalDataFlagName(u_int uFlagValue) const
{
	switch(uFlagValue)
	{
		case Asura_GUIMenu_Widget_TextBox::ASURA_GUIMENU_WIDGET_TEXTBOX_LOCAL_FLAG_TEXTENTRY :
			return "Text Entry Field";
		default :
			return PARENT::GetLocalDataFlagName(uFlagValue);

	}
}


void Tools_GUIMenu_Widget_TextBox::Copy(const Tools_GUIMenu_Root* pxSource)
{
	PARENT::Copy(pxSource);

	const Tools_GUIMenu_Widget_TextBox* pxSourceTextBox = (const Tools_GUIMenu_Widget_TextBox*) pxSource;
	const Asura_GUIMenu_Widget_TextBox* pxSourceAsuraTextBox = pxSourceTextBox->m_pxAsuraTextBox;

	m_uMaxTextEntryLength = pxSourceAsuraTextBox->GetTextEntryMaxChars();
}

void Tools_GUIMenu_Widget_TextBox::Update()
{
	if (m_uMaxTextEntryLength != m_pxAsuraTextBox->GetTextEntryMaxChars())
	{
		m_pxAsuraTextBox->SetTextEntryMaxChars(m_uMaxTextEntryLength);
	}

	PARENT::Update();
}

void Tools_GUIMenu_Widget_TextBox::AddToPreviewOverlay() const
{
	PARENT::AddToPreviewOverlay();

	if(!m_pxAsuraTextBox->GetConsoleVar().IsBlank())
	{
		Tools_GUIMenu_PreviewOverlay::RegisterConsoleVariable(m_pxAsuraTextBox->GetConsoleVar().GetName().GetString(),Asura_CommandConsole_VarRepository::Var::TYPE_STRING);
	}

}


#endif //ASURA_USE_GUI_MENU_SYSTEM
