//
// Tools_HelperObject_Object.cpp
//
// Created by Richard Rice - 31/08/2006
// 
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"
#include "HelperObject/Tools_HelperObject_Object.h"

#include "Camera/Asura_Camera.h"
#include "Interface/Asura_Overlay_System.h"
#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Overlay.h"
#include "Mesh/Asura_Mesh_System.h"

#include "Tools_Overlays.h"
#include "HelperObject/Tools_HelperObject_System.h"
#include "Load&Save/Tools_SaveAsura.h"
#include "Misc/Tools_Render.h"
#include "Objects/Tools_Object_System.h"
#include "Objects/Tools_VolumeEdit_Object.h"
#include "Objects/Tools_ObjectSystem_Param.h"


//declarations for params that get shared between all selected objects
TOOLS_DECLARE_SHARED_PARAM(bool,		0x3A0B036D,	HelperObject_ImportIntoMax)
TOOLS_DECLARE_SHARED_PARAM(Asura_String,0xB7CE320E,	HelperObject_File)


Asura_Vector_2 Tools_HelperObject_Object::s_xOverlayPosition(10.0f,10.0f);

static bool g_bHideInNonObjectMode = true;
static bool g_bHideInObjectMode = false;


static Asura_String g_xDefaultFile;//default file for new objects

Tools_HelperObject_Object::Tools_HelperObject_Object(Tools_HelperObject* pxHelper)
:Tools_Object(TOT_HelperObject)
{
	m_pxHelper = pxHelper;
	m_uVolumeEditID = 0;
	m_bChanged = false;

	SetMesh("ToolsShape_Helper");

	Position = m_pxHelper->m_xPosition;
	Orientation = m_pxHelper->m_xOrientation;
	
}

Tools_HelperObject_Object::~Tools_HelperObject_Object()
{
	if(m_uVolumeEditID) Tools_VolumeEdit_Object::Delete(m_uVolumeEditID);
}


void Tools_HelperObject_Object::Destroy()
{
	SetModified();
	Tools_HelperObject_System::DeleteHelperObject(m_pxHelper);

	Tools_Object::Destroy();
}

void Tools_HelperObject_Object::OnSelection()
{
	CreateOverlay();
	Tools_Object_System::SetCommentText(m_pxHelper->m_xComment.GetString(),this);
}

void Tools_HelperObject_Object::OnDeselection()
{
	DeleteOverlay();
	Tools_Object_System::ClearCommentText(this);
}

void Tools_HelperObject_Object::CreateOverlay()
{
	Asura_Overlay* pxOverlay;
	Asura_Dialog* pxDialog;

	pxOverlay = new Asura_Overlay(GetName(), false, &s_xOverlayPosition);
	Asura_Vector_2 xOffset = Asura_Vector_2(1.0f,3.0f);

	pxDialog = new Asura_Function_Dialog("File : ",&g_xParam_HelperObject_File,xOffset,SelectFile,(int)this);
	pxDialog->SetHelpText("Sets the files for the selected objects.\nAlso determines the default file for new objects.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Rename",xOffset,Rename,(int)this);
	pxDialog->SetHelpText("Rename this object");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();


	pxDialog = new Asura_Function_Dialog("Edit Comment",xOffset,EditComment,(int)this);
	pxOverlay->AddDialog(pxDialog);

	xOffset.x += 12.0f;
	pxDialog = new Asura_Tickbox_Dialog("Show",xOffset,&Tools_Object_System::s_bShowComment);
	pxDialog->SetHelpText("Displays the comment (if any) in the corner of the screen");
  	pxOverlay->AddDialog(pxDialog);
	xOffset.x -= 12.0f;
	xOffset.y += Asura_Dialog::DefaultSpacing();


	pxDialog = new Asura_Function_Dialog("Edit Volume",xOffset,EditVolume,(int)this);
	pxDialog->SetHelpText("Edit optional volume for this object.\nThe volume will be shown whenever this object is selected.");
	pxDialog->SetIsEnabledFunction(Tools_Object_System::AreControlsEnabled);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();

	pxDialog = new Asura_SharedTickbox_Dialog("Max/Maya",xOffset,&g_xParam_HelperObject_ImportIntoMax);
	pxDialog->SetHelpText("This object will be imported into Max/Maya as a dummy object.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();


	pxOverlay->m_uID = TOOLS_OVERLAY_SELECTEDOBJECT;

	Asura_Overlay_System::SubmitNewOverlay(pxOverlay);
}

void Tools_HelperObject_Object::DeleteOverlay()
{
	Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_SELECTEDOBJECT);
}

void Tools_HelperObject_Object::Update()
{
	if(m_bChanged)
	{
		m_bChanged = false;
		SetModified();
	}
}

void Tools_HelperObject_Object::UpdateWhileSelected()
{
	//deselect if the overlay is closed
	if(!Asura_Overlay_System::DoesOverlayExist(TOOLS_OVERLAY_SELECTEDOBJECT))
	{
		Tools_Object_System::DeselectObject(this);
	}
}

void Tools_HelperObject_Object::Render()
{
	Tools_Object::Render();

	//show the volume , if there is one , and it isn't being edited
	if(IsSelected() || Tools_Object_System::IsObjectUnderCursor(this))
	{
		if(m_pxHelper->m_xVolume.m_xBox.GetVolume()>0.0f && !Tools_VolumeEdit_Object::Find(m_uVolumeEditID))
		{
			Asura_Bounding_Box xRelativeBox = m_pxHelper->m_xVolume.m_xBox;
			Asura_Vector_3 xCentre = xRelativeBox.CalculateCentre();
			xRelativeBox.Displace(-xCentre);

			Asura_Matrix_3x3 xMat;
			m_pxHelper->m_xVolume.m_xOrient.ToMatrix(xMat);
			
			Tools_Render::RenderCube(xRelativeBox,xCentre,xMat,0xff0000ff,true);
			Tools_Render::RenderCube(xRelativeBox,xCentre,xMat,0x3f0000ff,false);
		}
	}
}


void Tools_HelperObject_Object::Rename()
{
	if(Tools_Object_System::AskForName(m_pxHelper->m_xName,"Rename object",this,false,GetMyNamePrefix()))
	{
		SetModified();

		//update the overlay
		Asura_Overlay* pxOverlay = Asura_Overlay_System::GetOverlay(TOOLS_OVERLAY_SELECTEDOBJECT);
		if(pxOverlay) pxOverlay->SetTitle(GetName());
	}
}

void Tools_HelperObject_Object::EditComment()
{
	GetString_Overlay xOverlay("Comment Text :",m_pxHelper->m_xComment.GetString(),1000);
	if(xOverlay.DoModal())
	{
		m_pxHelper->m_xComment = xOverlay.GetString();
		SetModified();

		Tools_Object_System::SetCommentText(m_pxHelper->m_xComment.GetString(),this);
	}
}


void Tools_HelperObject_Object::EditVolume()
{
	//see if the volume edit object already exists
	Tools_VolumeEdit_Object* pxVolumeObject = Tools_VolumeEdit_Object::Find(m_uVolumeEditID);

	if(!pxVolumeObject)
	{
		Asura_Bounding_Box* pxBB = &m_pxHelper->m_xVolume.m_xBox;

		//set a default value for the bounding box if needed
		if(pxBB->GetVolume()==0)
		{
			pxBB->Reset();
			pxBB->Set(Position);
			pxBB->Grow(0.5f);
		}

		char acName[200];
		Asura_PrintfToBfr(acName,"%s Volume",200,true,GetName());

		pxVolumeObject = new Tools_VolumeEdit_Object(acName,pxBB,0,&m_bChanged);
		pxVolumeObject->SetVolumeOrientation(&m_pxHelper->m_xVolume.m_xOrient);
		pxVolumeObject->SetOwner(this);
		m_uVolumeEditID = pxVolumeObject->GetID();

	}

	Tools_Object_System::SelectObject(pxVolumeObject);
}


void Tools_HelperObject_Object::Create()
{
	//get a name for the object
	Asura_String m_xName;
	if(!Tools_Object_System::AskForName(m_xName,"Name helper object", 0, false, Tools_HelperObject_Object::GetNamePrefix())) return;

	//create a new object at the camera
	Tools_HelperObject* pxHelper = Tools_HelperObject_System::AllocateHelperObject();
	pxHelper->m_bIsNew = true;
	pxHelper->m_xName = m_xName;
	pxHelper->m_xPosition = *Asura_Camera::GetPosition();
	pxHelper->m_xOrientation = *Asura_Camera::GetOrientation();

	//assign a default file
	if(g_xDefaultFile.IsBlank())
	{
		Asura_String xAsuraFilename = Tools_SaveAsura::GetMainFilename();
		xAsuraFilename.SplitFilename(0,&pxHelper->m_xFile,0);
	}
	else
	{
		pxHelper->m_xFile = g_xDefaultFile;
	}

	//Create a tools object for this , and select it
	Tools_HelperObject_Object* pxObject = new Tools_HelperObject_Object(pxHelper);
	Tools_Object_System::SelectObject(pxObject);

	pxObject->SetModified();
	
}

void Tools_HelperObject_Object::CreateToolsObjects()
{
	for(CLIF<Tools_HelperObject*> xHelperLIF(Tools_HelperObject_System::GetList());!xHelperLIF.done();xHelperLIF.next())
	{																											
		new Tools_HelperObject_Object(xHelperLIF());
	}

	//Set initial visibilty. The objects should be being created while in a non-object mode
	Tools_Object_System::SetTypeHidden(TOT_HelperObject,ObjectHideFlag_HideByTypeOverlay,g_bHideInNonObjectMode);
}

void Tools_HelperObject_Object::DeleteToolsObjects()
{
	List<Tools_Object*> xObjectList;
	Tools_Object_System::ListObjectsOfType(TOT_HelperObject,xObjectList);

	for(LIF<Tools_Object*> xObjectLIF(&xObjectList);!xObjectLIF.done();xObjectLIF.next())
	{
		Tools_Object_System::DeleteObject(xObjectLIF());
	}
}

void Tools_HelperObject_Object::SetModified()
{
	Tools_HelperObject_System::SetModified(m_pxHelper);
}

void Tools_HelperObject_Object::SelectFile()
{
	ListSelection_Overlay xListOverlay("Select File");

	int iJunkIndex;
	Asura_NameTree xFileTree;

	for(CLIF<Tools_HelperObject*> xHelperLIF(Tools_HelperObject_System::GetList());!xHelperLIF.done();xHelperLIF.next())
	{
		const char* szFilename = xHelperLIF()->m_xFile.GetString();
		if(!xFileTree.FindNodeIndexByName(szFilename,true,iJunkIndex))
		{
			bool bDefaultSelection = false;
			if(!stricmp(szFilename,m_pxHelper->m_xFile.GetString()))
			{
				bDefaultSelection = true;
			}
			xListOverlay.AddItem(szFilename,0,bDefaultSelection);
		}
	}
	//add an option for creating a new file
	const char* szNewFileString = "*New File*";
	xListOverlay.AddItem(szNewFileString);
		 
	if(!xListOverlay.DoModal()) return;

	Asura_String xFilename = xListOverlay.GetSelectionString();

	if(!stricmp(szNewFileString,xFilename.GetString()))
	{
		//the option for creating a new file was selected
		GetString_Overlay xStringOverlay("Enter new filename (without path/extension)",m_pxHelper->m_xFile.GetString());
		if(!xStringOverlay.DoModal()) return;

		xFilename = xStringOverlay.GetString();
		if(xFilename.IsBlank()) return;
	}
	
	//make sure the target file is writeable
	if(!Tools_HelperObject_System::CanSaveToFile(xFilename.GetString()))
	{
		ShowMessageOverlay("Can't transfer objects since target file isn't writeable.");
		return;
	}

	//Apply the changes to all of the selected helper objects
	int iFailedCount = 0;
	for(LIF<Tools_Object*> xSelectedLIF(&Tools_Object_System::SelectedObjectList);!xSelectedLIF.done();xSelectedLIF.next())
	{
		if(xSelectedLIF()->Type != TOT_HelperObject) continue;

		Tools_HelperObject_Object* pxObject = (Tools_HelperObject_Object*) xSelectedLIF();

		//make sure the source file is writeable
		if(!pxObject->m_pxHelper->m_bIsNew)
		{
			if(!Tools_HelperObject_System::CanSaveToFile(pxObject->m_pxHelper->m_xFile.GetString()))
			{
				iFailedCount++;
				continue;
			}
		}

		//mark as modified so the source file is modified
		pxObject->SetModified();

		//then move and make the new file modified
		pxObject->m_pxHelper->m_xFile = xFilename;
		pxObject->SetModified();
	}

	if(iFailedCount)
	{
		ShowMessageOverlay("Failed to transfer %d object(s) , because they were in read-only files.",iFailedCount);
	}

	//Change the default file
	g_xDefaultFile = xFilename;
}


bool Tools_HelperObject_Object::GetParameter(Asura_Hash_ID uParamID,class Tools_Parameter* pxParam) const
{
	switch(uParamID)
	{
		case g_uHash_HelperObject_ImportIntoMax :
			pxParam->SetBool(m_pxHelper->m_bImportIntoMax);
			break;

		case g_uHash_HelperObject_File :
			pxParam->SetString(m_pxHelper->m_xFile.GetString());
			break;
		
		default :
			return false;
	}

	return true;
}

bool Tools_HelperObject_Object::SetParameter(Asura_Hash_ID uParamID,const class Tools_Parameter* pxParam)
{
	switch(uParamID)
	{
		case g_uHash_HelperObject_ImportIntoMax :
			m_pxHelper->m_bImportIntoMax = pxParam->GetBool();
			break;


		default :
			return false;
	}

	m_bChanged = true;
	return true;
}


void Tools_HelperObject_Object::UpdateVisibility(bool bObjectControlsEnabled)
{
	if(bObjectControlsEnabled)
	{
		//update hidden state for non-object modes
		g_bHideInNonObjectMode = Tools_Object_System::GetTypeHiddenFlag(TOT_HelperObject,ObjectHideFlag_HideByTypeOverlay);

		//and set hidden hidden state for object type modes
		Tools_Object_System::SetTypeHidden(TOT_HelperObject,ObjectHideFlag_HideByTypeOverlay,g_bHideInObjectMode);
	}
	else
	{
		//update hidden state for object mode
		g_bHideInObjectMode = Tools_Object_System::GetTypeHiddenFlag(TOT_HelperObject,ObjectHideFlag_HideByTypeOverlay);

		//and set hidden hidden state for non object modes
		Tools_Object_System::SetTypeHidden(TOT_HelperObject,ObjectHideFlag_HideByTypeOverlay,g_bHideInNonObjectMode);
	}
}