
///////////////////////////////////////////////////////////////////////////////
//
// File   : Tools_Overlay_Hierarchy_ViewFacialAnimation.cpp
// Author : Andy Tate
// Date   : 20-May-2008
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

#if defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)

// This file's header...
#include "Hierarchy/Overlays/Tools_Overlay_Hierarchy_ViewFacialAnimation.h"

// Asura independent header-files...
#include "Animation/Asura_Animation.h"
#include "Interface/Asura_Dialog.h"
#include "ChunkSystem/Asura_Chunk_Loaders.h"
#include "FacialAnimation/Asura_FacialAnimation_System.h"
#include "Interface/Asura_Overlay_System.h"
#include "Interface/Asura_Tree_Dialog.h"
#include "Sound/Asura_Dialogue.h"
#include "Sound/Asura_Dialogue_System.h"

// Tools header-files...
#include "Tools_Overlays.h"
#include "FacialAnimation/Tools_FacialAnimation_AnimationHandler_HierarchyMode.h"
#include "FacialAnimation/Tools_FacialAnimation_System.h"
#include "Hierarchy/Tools_Mode_Hierarchy.h"
#include "Misc/Tools_Modal_Overlay.h"
#include "Misc/Tools_Timed_Messages.h"
#include "PC/FileSystem/Tools_FileSystem.h"
#include "Sound/Tools_PickDialogueLine_Overlay.h"



////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////

const char* Tools_Overlay_Hierarchy_ViewFacialAnimation::s_aszTableColumnNames[MAX_TABLE_COLUMNS] =
{
	"Muscle-group",	// TC_MUSCLE_GROUP
	"Controls",		// TC_CONTROLS
};

float	Tools_TableCell_MuscleGroupApplyControlType_Dialog::s_fUnusedWeight	= 0.0f;
float	Tools_TableCell_MuscleGroupApplyControlType_Dialog::s_fUnusedBalance	= 0.0f;
float	Tools_TableCell_EyeValueApplyControlType_Dialog::s_fUnusedWeight	= 0.0f;

static const char*	g_szDefaultFaceText	= "Select Face...";

static const char* g_aszEyeValueNames[ASURA_FACIALANIMATION_EYE_NUMPARAM] =
{
	"Eye Left/Right",
	"Eye Up/Down",
	"Eye Focus",
	"Eye Close",
	"Eye Squint",
	"Eye Balance",
	"Eye Target Mix",
};


////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////

Tools_Overlay_Hierarchy_ViewFacialAnimation*	Tools_Overlay_Hierarchy_ViewFacialAnimation::s_pxThis	= NULL;

static float	g_fMinWeightValue		= 0.0f;
static float	g_fMaxWeightValue		= 1.0f;
static float	g_fMinPairWeightValue	= -1.0f;
static float	g_fMaxPairWeightValue	= 1.0f;

static float	g_fMinBalanceValue		= -1.0f;
static float	g_fMaxBalanceValue		= 1.0f;
static float	g_fMinBalancePairValue	= -1.0f;
static float	g_fMaxBalancePairValue	= 1.0f;


////////////////////////////////////////////////////////////////////////////////
// Private functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Protected functions...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Overlay_Hierarchy_ViewFacialAnimation::AddOverlayControls
// Author       : Andy Tate
// Date         : 20-May-2008
////////////////////////////////////////////////////////////////////////////////

void Tools_Overlay_Hierarchy_ViewFacialAnimation::AddOverlayControls(Asura_Vector_2& xOffset)
{
	const Asura_Vector_2	xMaxTableSize(60.0f, 30.0f);
	const Asura_Vector_2	xInitialOffset(xOffset);

	Asura_Dialog*			pxNewDialogue;

	// Allow selection of the face to use...
	m_pxSelectFaceDialogue	= new Asura_Function_Dialog (g_szDefaultFaceText, xOffset, OnSelectFaceSelected, reinterpret_cast<int>(this));
	m_pxSelectFaceDialogue->SetIsEnabledFunction(CanSelectFace, reinterpret_cast<int>(this));
	m_pxSelectFaceDialogue->SetHelpText("Allows selection of the face to when viewing facial-animation on the current skin.");
	AddDialog(m_pxSelectFaceDialogue);

#if 0
	// Create the "Reload Animations" button - NOTE: This is temporary, until we have exports from Max...
	xOffset[0]		= xInitialOffset[0] + 16.0f;
	pxNewDialogue	= new Asura_Function_Dialog("Reload Animations", xOffset, OnReloadAnimationsSelected, reinterpret_cast<int>(this));
	pxNewDialogue->SetIsEnabledFunction(CanBeAppliedToSkin, reinterpret_cast<int>(this));
	AddDialog(pxNewDialogue);

	xOffset[0]	= xInitialOffset[0];
#endif

	// Handle the playing of test dialogue...
	xOffset[1]		+= Asura_Dialog::LargeSpacing();
	pxNewDialogue	 = new Asura_Function_Dialog ("Play Dialogue Line", xOffset, OnPlayDialogueLineSelected, reinterpret_cast<int>(this));
	pxNewDialogue->SetIsEnabledFunction(CanPlayDialogueLine, reinterpret_cast<int>(this));
	pxNewDialogue->SetHelpText("Allows a line of dialogue to be played, any facial-animation data will be played on the current face (if appropriate).");
	AddDialog(pxNewDialogue);

	xOffset[0]		= xInitialOffset[0] + 16.0f;
	pxNewDialogue	= new Asura_Function_Dialog ("Stop Dialogue Line", xOffset, OnStopDialogueLineSelected, reinterpret_cast<int>(this));
	pxNewDialogue->SetIsEnabledFunction(CanStopDialogueLine, reinterpret_cast<int>(this));
	pxNewDialogue->SetHelpText("Stops the line of dialogue currently playing, any facial-animation data will also be stopped on the current face (if appropriate).");
	AddDialog(pxNewDialogue);

	// play an facial anim directly
	xOffset[0]		 = xInitialOffset[0];
	xOffset[1]		+= Asura_Dialog::LargeSpacing();
	pxNewDialogue	 = new Asura_Function_Dialog("Play Facial Animation", xOffset, OnPlayFacialAnimationSelected, reinterpret_cast<int>(this));
	pxNewDialogue->SetIsEnabledFunction(CanPlayFacialAnimation, reinterpret_cast<int>(this));
	pxNewDialogue->SetHelpText("Allows a specific facial animation to be played, any facial-animation data will be played on the current face (if appropriate).");
	AddDialog(pxNewDialogue);

	xOffset[0]		 = xInitialOffset[0] + 16.0f;
	pxNewDialogue	 = new Asura_Function_Dialog("Stop Facial Animation", xOffset, OnStopFacialAnimationSelected, reinterpret_cast<int>(this));
	pxNewDialogue->SetIsEnabledFunction(CanStopFacialAnimation, reinterpret_cast<int>(this));
	pxNewDialogue->SetHelpText("Allows a specific facial animation to be played, any facial-animation data will be played on the current face (if appropriate).");
	AddDialog(pxNewDialogue);

	// Handle application of this face to the skin...
	xOffset[0]				 = xInitialOffset[0];
	xOffset[1]				+= Asura_Dialog::LargeSpacing();
	m_pxApplyToSkinDialogue	 = new Asura_Tickbox_Dialog ("Apply To Skin", xOffset, &m_bApplyToSkin);
	m_pxApplyToSkinDialogue->SetIsEnabledFunction(CanBeAppliedToSkin, reinterpret_cast<int>(this));
	m_pxApplyToSkinDialogue->SetClickedFunction(OnApplyToSkinSelected, reinterpret_cast<int>(this));
	m_pxApplyToSkinDialogue->SetHelpText("Allows the specified facial-animation data to be applied to the current skin.");
	AddDialog(m_pxApplyToSkinDialogue);

	xOffset[0]		= xInitialOffset[0] + 16.0f;
	pxNewDialogue	= new Asura_Function_Dialog ("Reset to Defaults", xOffset, OnResetWeightsAndBalancesSelected, reinterpret_cast<int>(this));
	pxNewDialogue->SetIsEnabledFunction(IsFacialAnimationAppliedToSkin, reinterpret_cast<int>(this));
	pxNewDialogue->SetHelpText("Resets the weight and balance data to its default values.");
	AddDialog(pxNewDialogue);

	xOffset[0]		= xInitialOffset[0] + 32.0f;
	pxNewDialogue	= new Asura_Function_Dialog ("Load Pose", xOffset, OnLoadPoseSelected, reinterpret_cast<int>(this));
	pxNewDialogue->SetIsEnabledFunction(IsFacialAnimationAppliedToSkin, reinterpret_cast<int>(this));
	pxNewDialogue->SetHelpText("Loads a specific pose for the current facial-animation.");
	AddDialog(pxNewDialogue);

	// Create the table of controls...
	xOffset[0]			 = xInitialOffset[0];
	xOffset[1]			+= Asura_Dialog::LargeSpacing();
	m_pxTableDialogue	 = new Asura_Table_Dialog (xOffset, xMaxTableSize[0], xMaxTableSize[1], true);
	m_pxTableDialogue->SetIsEnabledFunction(IsFacialAnimationAppliedToSkin, reinterpret_cast<int>(this));
	AddDialog(m_pxTableDialogue);

	// Allow saving/loading of expression face-pose data...
	xOffset[1]		= m_pxTableDialogue->m_xMaxCoord[1] + Asura_Dialog::DefaultSpacing();
	pxNewDialogue	= new Asura_Function_Dialog ("Save as Pose", xOffset, OnSaveAsPoseSelected, reinterpret_cast<int>(this));
	pxNewDialogue->SetIsEnabledFunction(IsFacialAnimationAppliedToSkin, reinterpret_cast<int>(this));
	pxNewDialogue->SetHelpText("Allows the current set of controls to be saved as an expression to be applied to the current skin.");
	AddDialog(pxNewDialogue);
	pxNewDialogue->SetRelativePosition(m_pxTableDialogue, true, false);

	// Update the face...
	SetCurrentFace(m_uCurrentFaceHash, false);
}

/// End of function 'Tools_Overlay_Hierarchy_ViewFacialAnimation::AddOverlayControls'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Overlay_Hierarchy_ViewFacialAnimation::SetCurrentFace
// Author       : Andy Tate
// Date         : 20-May-2008
////////////////////////////////////////////////////////////////////////////////

void Tools_Overlay_Hierarchy_ViewFacialAnimation::SetCurrentFace(const Asura_Hash_ID uFaceHash,
																 const bool bUpdateAnimationHandler)
{
	const Tools_FacialAnimation_Face*	pxFace	= Tools_FacialAnimation_System::FindFace(uFaceHash);

	// Remove any existing data...
	m_pxTableDialogue->DestroyTable();
	delete[](m_pszMuscleGroupNames);

	if(m_pxAnimationHandler != NULL)
	{
		Asura_FacialAnimation_System::RemoveAnimationHandler(ASURA_GUID_TOOLSUSER);
		delete(m_pxAnimationHandler);
	}

	if(m_iDialogueLineHandle != ASURA_INVALID_HANDLE)
	{
		Asura_Dialogue_System::Stop(m_iDialogueLineHandle);
		m_iDialogueLineHandle	= ASURA_INVALID_HANDLE;
	}

	// Set the face we're going to use...
	m_uCurrentFaceHash		= uFaceHash;
	m_pxAnimationHandler	= NULL;

	if(pxFace == NULL)
	{
		m_pszMuscleGroupNames	= NULL;

		m_pxSelectFaceDialogue->UpdateLabel(g_szDefaultFaceText);
		m_pxTableDialogue->SetHidden(true);
	}
	else
	{
		Tools_FacialAnimation_Face::MuscleGroupConstIterator	xIterator(pxFace->GetMuscleGroupContainer());
		const Asura_Vector_2									xMinTableCellSize(1.0f, 1.0f);

		const Asura_String&										xFaceName	= pxFace->GetName();
		const char*												szFaceName	= xFaceName.GetString();

		// Re-initialise the muscle-group data...
		m_uNumMuscleGroups		= pxFace->GetNumMuscleGroups();
		m_pszMuscleGroupNames	= new const char* [m_uNumMuscleGroups];
		m_pxAnimationHandler	= (bUpdateAnimationHandler) ? new Tools_FacialAnimation_AnimationHandler_HierarchyMode (uFaceHash) : NULL;
		m_uNumEyeValues 		= pxFace->HasEyes() ? ASURA_FACIALANIMATION_EYE_NUMPARAM : 0;

		if(m_pxAnimationHandler != NULL)
		{
			Asura_FacialAnimation_System::AddAnimationHandler(ASURA_GUID_TOOLSUSER, m_pxAnimationHandler);
			//Make sure that any facial animation associated with the current animation in hierarchy mode is played
			PlayLinkedFacialAnimation(Tools_Mode_Hierarchy::GetCurrentAnimation());
		}

		for(u_int u = 0; u < m_uNumMuscleGroups; u++, xIterator.Next())
		{
			const Tools_FacialAnimation_Face_MuscleGroup*	pxMuscleGroup		= xIterator.GetCurrent();
			const Asura_String&							xMuscleGroupName	= pxMuscleGroup->GetName();
			const char*									szMuscleGroupName	= xMuscleGroupName.GetString();

			m_pszMuscleGroupNames[u]	= szMuscleGroupName;

			if(bUpdateAnimationHandler)
			{
				const u_int	uNumAnimations	= pxMuscleGroup->GetNumAnimations();

				// Validate our animations...
				for(u_int u = 0; u < uNumAnimations; u++)
				{
					const Asura_String&	xAnimationName		= pxMuscleGroup->GetAnimationName(u);

					if(xAnimationName.IsBlank())
					{
						Tools_Timed_Message_System::AddMessage(Tools_Timed_Message_System::COLOUR_ERROR, "Muscle-group \"%s\" has no animation specified.", szMuscleGroupName);
					}
					else
					{
						const char*	szAnimationName	= xAnimationName.GetString();

						if(Asura_Animation_System::GetAnim(szAnimationName) == NULL)
						{
							Tools_Timed_Message_System::AddMessage(Tools_Timed_Message_System::COLOUR_ERROR, "Failed to find animation \"%s\" for muscle-group \"%s\".", szAnimationName, szMuscleGroupName);
						}
					}
				}
			}
		}

		// Update the dialogues...
		m_pxSelectFaceDialogue->UpdateLabel(szFaceName);
		m_pxTableDialogue->SetHidden(false);
		m_pxTableDialogue->CreateTable
			(
			xMinTableCellSize,
			m_uNumMuscleGroups + m_uNumEyeValues,
			MAX_TABLE_COLUMNS,
			CreateTableCell,
			this,
			NULL,
			s_aszTableColumnNames
		);
	}
}

/// End of function 'Tools_Overlay_Hierarchy_ViewFacialAnimation::SetCurrentFace'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Overlay_Hierarchy_ViewFacialAnimation::GetCurrentFace
// Author       : Andy Tate
// Date         : 22-May-2008
////////////////////////////////////////////////////////////////////////////////

const Tools_FacialAnimation_Face* Tools_Overlay_Hierarchy_ViewFacialAnimation::GetCurrentFace(void) const
{
	return(Tools_FacialAnimation_System::FindFace(m_uCurrentFaceHash));
}

/// End of function 'Tools_Overlay_Hierarchy_ViewFacialAnimation::GetCurrentFace'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Overlay_Hierarchy_ViewFacialAnimation::Tools_Overlay_Hierarchy_ViewFacialAnimation
// Author       : Andy Tate
// Date         : 20-May-2008
////////////////////////////////////////////////////////////////////////////////

Tools_Overlay_Hierarchy_ViewFacialAnimation::Tools_Overlay_Hierarchy_ViewFacialAnimation(void)
: PARENT("View Facial-Animation", false)
, m_uCurrentFaceHash(ASURA_HASH_ID_UNSET)
, m_bApplyToSkin(false)
, m_uNumMuscleGroups(0)
, m_pszMuscleGroupNames(NULL)
, m_pxAnimationHandler(NULL)
, m_iDialogueLineHandle(ASURA_INVALID_HANDLE)
, m_uFaceAnimHashId( ASURA_HASH_ID_UNSET )
, m_xAnimString( NULL )
{
	Asura_Vector_2	xOffset(1.0f, 3.0f);

	// Initialise the variables...
	m_uID			= TOOLS_OVERLAY_HIERARCHY_VIEWFACIALANIMATION;
	s_pxThis	= this;

	// Update the overlays...
	AddOverlayControls(xOffset);
}

/// End of function 'Tools_Overlay_Hierarchy_ViewFacialAnimation::Tools_Overlay_Hierarchy_ViewFacialAnimation'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Overlay_Hierarchy_ViewFacialAnimation::~Tools_Overlay_Hierarchy_ViewFacialAnimation
// Author       : Andy Tate
// Date         : 20-May-2008
////////////////////////////////////////////////////////////////////////////////

Tools_Overlay_Hierarchy_ViewFacialAnimation::~Tools_Overlay_Hierarchy_ViewFacialAnimation()
{
	// Stop any dialogue that's playing...
	if(m_iDialogueLineHandle != ASURA_INVALID_HANDLE)
	{
		Asura_Dialogue_System::Stop(m_iDialogueLineHandle);
	}

	// Free-up the previously allocated memory...
	delete[](m_pszMuscleGroupNames);

	if(m_pxAnimationHandler != NULL)
	{
		Asura_FacialAnimation_System::RemoveAnimationHandler(ASURA_GUID_TOOLSUSER);
		delete(m_pxAnimationHandler);
	}

	// Reset the variables...
	s_pxThis	= NULL;
}

/// End of function 'Tools_Overlay_Hierarchy_ViewFacialAnimation::~Tools_Overlay_Hierarchy_ViewFacialAnimation'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Overlay_Hierarchy_ViewFacialAnimation::CreateTableCell
// Author       : Andy Tate
// Date         : 20-May-2008
////////////////////////////////////////////////////////////////////////////////

Asura_TableCell_Dialog* Tools_Overlay_Hierarchy_ViewFacialAnimation::CreateTableCell(Asura_Table_Dialog& xTableDialog,
																					 const Asura_Vector_2& xOffset,
																					 const Asura_Vector_2& xMinCellSize,
																					 const u_int uRow,
																					 const u_int uColumn,
																					 void* const pxUserData)
{
	Tools_Overlay_Hierarchy_ViewFacialAnimation*	pxThis				= static_cast<Tools_Overlay_Hierarchy_ViewFacialAnimation*>(pxUserData);

	//Determine whether this row is for the muscle groups or the eye values
	bool bEye = uRow>=pxThis->m_uNumMuscleGroups;

	if(bEye)
	{
		u_int uEyeValueIndex = uRow-pxThis->m_uNumMuscleGroups;

		switch(uColumn)
		{
			case TC_MUSCLE_GROUP:
			{
				return(Asura_TableRowHeading_StandardDialog::Create(xTableDialog, g_aszEyeValueNames[uEyeValueIndex], xOffset, xMinCellSize, uRow));
			}

			case TC_CONTROLS:
			{
				return(new Tools_TableCell_EyeValueApplyControlType_Dialog(uEyeValueIndex, xTableDialog, xOffset, xMinCellSize, uRow, uColumn, *pxThis));
			}

			default:
			{
				Asura_Assert(false, "Unexpected column being used when creating view facial-animation table.");
				return(NULL);
			}
		}		// End of switch
	}
	else
	{
		const char* szMuscleGroupName = pxThis->m_pszMuscleGroupNames[uRow];

		switch(uColumn)
		{
			case TC_MUSCLE_GROUP:
			{
				return(Asura_TableRowHeading_StandardDialog::Create(xTableDialog, szMuscleGroupName, xOffset, xMinCellSize, uRow));
			}

			case TC_CONTROLS:
			{
				const Tools_FacialAnimation_Face*				pxFace			= pxThis->GetCurrentFace();
				const Tools_FacialAnimation_Face_MuscleGroup*	pxMuscleGroup	= pxFace->FindMuscleGroup(szMuscleGroupName);

				if(pxMuscleGroup == NULL)
				{
					Asura_Assertf(false, ("Failed to find muscle-group \"%s\" when creating table.", szMuscleGroupName));
					return(NULL);
				}
				else
				{
					return(new Tools_TableCell_MuscleGroupApplyControlType_Dialog(*pxMuscleGroup, xTableDialog, xOffset, xMinCellSize, uRow, uColumn, *pxThis));
				}
			}

			default:
			{
				Asura_Assert(false, "Unexpected column being used when creating view facial-animation table.");
				return(NULL);
			}
		}		// End of switch
	}
}

/// End of function 'Tools_Overlay_Hierarchy_ViewFacialAnimation::CreateTableCell'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Overlay_Hierarchy_ViewFacialAnimation::OnSelectFaceSelected
// Author       : Andy Tate
// Date         : 20-May-2008
////////////////////////////////////////////////////////////////////////////////

void Tools_Overlay_Hierarchy_ViewFacialAnimation::OnSelectFaceSelected(int iUserData)
{
	Tools_Overlay_Hierarchy_ViewFacialAnimation* pxThis = reinterpret_cast<Tools_Overlay_Hierarchy_ViewFacialAnimation*>(iUserData);

	const char*				szFaceDirectory		= Tools_FacialAnimation_System::GetFaceDirectoryName();
	const char*				szFaceFileExtension	= Tools_FacialAnimation_System::GetFaceFileExtension();

	FileSelect_Overlay		xSelectFace( "Select face", szFaceDirectory, szFaceFileExtension, false );

	xSelectFace.SelectFile( pxThis->GetCurrentFaceHash(), false );

	if( xSelectFace.DoModal() )
	{
		Asura_Hash_ID uSelectedFaceHash;
		Asura_String  xSelectedFacePath;

		xSelectFace.GetSelectedFile( uSelectedFaceHash, xSelectedFacePath );

		Tools_FacialAnimation_Face*	pxFace = Tools_FacialAnimation_System::FindOrLoadFace( uSelectedFaceHash );
		if( pxFace )
		{
			// Finally, update the selected face...
			pxThis->SetCurrentFace(uSelectedFaceHash, true);
		}
		else
		{
			ShowMessageOverlay("Failed to find the specified face \"%s\".", xSelectedFacePath.GetString());
		}
	}
}


/// End of function 'Tools_Overlay_Hierarchy_ViewFacialAnimation::OnSelectFaceSelected'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Overlay_Hierarchy_ViewFacialAnimation::OnReloadAnimationsSelected
// Author       : Andy Tate
// Date         : 23-May-2008
////////////////////////////////////////////////////////////////////////////////

void Tools_Overlay_Hierarchy_ViewFacialAnimation::OnReloadAnimationsSelected(int iUserData)
{
	Tools_Overlay_Hierarchy_ViewFacialAnimation*	pxThis		= reinterpret_cast<Tools_Overlay_Hierarchy_ViewFacialAnimation*>(iUserData);
	const Asura_Hash_ID								uFaceHash	= pxThis->m_uCurrentFaceHash;
	Tools_FacialAnimation_Face*						pxFace		= Tools_FacialAnimation_System::FindFace(uFaceHash);

	if(pxFace != NULL)
	{
		const u_int	uNumAnimationsReloaded	= pxFace->ReloadAsuraAnimations();

		char		szMessage[256];

		Asura_PrintfToBfr(szMessage, "Successfully reloaded %d animations.", 256, true, uNumAnimationsReloaded);

		{
			Message_Overlay	xFinishedOverlay(szMessage);

			// Output a message before updating the face and returning...
			xFinishedOverlay.DoModal();

			pxThis->SetCurrentFace(uFaceHash, true);
		}
	}
}

/// End of function 'Tools_Overlay_Hierarchy_ViewFacialAnimation::OnReloadAnimationsSelected'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Overlay_Hierarchy_ViewFacialAnimation::OnPlayDialogueLineSelected
// Author       : Andy Tate
// Date         : 7-Aug-2008
////////////////////////////////////////////////////////////////////////////////

void Tools_Overlay_Hierarchy_ViewFacialAnimation::OnPlayDialogueLineSelected(int iUserData)
{
	const Asura_Hash_ID	uDialogueLineHash	= Tools_PickDialogueLine_Overlay::PickLine("Select Dialogue Line to Play");

	if(uDialogueLineHash != ASURA_HASH_ID_UNSET)
	{
		const Asura_Dialogue_Line*	pxDialogueLine	= Asura_Dialogue_Storage::FindLine(uDialogueLineHash);

		if(pxDialogueLine != NULL)
		{
			Tools_Overlay_Hierarchy_ViewFacialAnimation*	pxThis		= reinterpret_cast<Tools_Overlay_Hierarchy_ViewFacialAnimation*>(iUserData);

			Asura_Dialogue_System::PlayLine(pxDialogueLine, ASURA_GUID_TOOLSUSER, &(pxThis->m_iDialogueLineHandle));
		}
	}
}
/// End of function 'Tools_Overlay_Hierarchy_ViewFacialAnimation::OnPlayDialogueLineSelected'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Overlay_Hierarchy_ViewFacialAnimation::OnPlayFacialAnimationSelected
// Author       : Gaz Thomas
// Date         : 7-Aug-2008
////////////////////////////////////////////////////////////////////////////////
void Tools_Overlay_Hierarchy_ViewFacialAnimation::OnPlayFacialAnimationSelected(int iUserData)
{
	Tools_Overlay_Hierarchy_ViewFacialAnimation*	pxThis	= reinterpret_cast<Tools_Overlay_Hierarchy_ViewFacialAnimation*>(iUserData);
	const char*		szDirectory		= Tools_FacialAnimation_System::GetAnimationDirectoryName();
	const char*		szFileExtension	= Tools_FacialAnimation_System::GetAnimationFileExtension();

	FileSelect_Overlay xSelectDialog( "Choose Facial Animation", szDirectory, szFileExtension ); 

	xSelectDialog.SelectFile( pxThis->m_uFaceAnimHashId, true );

	if ( xSelectDialog.DoModal() )
	{
		if (pxThis->m_pxAnimationHandler)
		{
			pxThis->m_pxAnimationHandler->HierarchyMode_StopFacialAnimation();

			Asura_Hash_ID uHashId = ASURA_HASH_ID_UNSET;
			Asura_String  xValue;
			xSelectDialog.GetSelectedFileWithPath( uHashId, xValue );

			if ( uHashId != ASURA_HASH_ID_UNSET )
			{
				pxThis->m_uFaceAnimHashId = uHashId;
			}
		
			Tools_FacialAnimation_FaceAnimation* pxFaceAnim = Tools_FacialAnimation_System::FindOrLoadAnimation( pxThis->m_uFaceAnimHashId );

			if ( pxFaceAnim )
			{
				pxThis->m_pxAnimationHandler->HierarchyMode_PlayFacialAnimation( pxThis->m_uFaceAnimHashId, true );
			}
		}
	}
}
/// End of function 'Tools_Overlay_Hierarchy_ViewFacialAnimation::OnPlayFacialAnimationSelected'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Overlay_Hierarchy_ViewFacialAnimation::OnStopFacialAnimationSelected
// Author       : Gaz Thomas
// Date         : 7-Aug-2008
////////////////////////////////////////////////////////////////////////////////
void Tools_Overlay_Hierarchy_ViewFacialAnimation::OnStopFacialAnimationSelected(int iUserData)
{
	Tools_Overlay_Hierarchy_ViewFacialAnimation*	pxThis	= reinterpret_cast<Tools_Overlay_Hierarchy_ViewFacialAnimation*>(iUserData);

	pxThis->m_pxAnimationHandler->HierarchyMode_StopFacialAnimation();

	pxThis->m_uFaceAnimHashId = ASURA_HASH_ID_UNSET;
}





////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Overlay_Hierarchy_ViewFacialAnimation::OnStopDialogueLineSelected
// Author       : Andy Tate
// Date         : 7-Aug-2008
////////////////////////////////////////////////////////////////////////////////

void Tools_Overlay_Hierarchy_ViewFacialAnimation::OnStopDialogueLineSelected(int iUserData)
{
	Tools_Overlay_Hierarchy_ViewFacialAnimation*	pxThis	= reinterpret_cast<Tools_Overlay_Hierarchy_ViewFacialAnimation*>(iUserData);

	Asura_Dialogue_System::Stop(pxThis->m_iDialogueLineHandle);
}

/// End of function 'Tools_Overlay_Hierarchy_ViewFacialAnimation::OnStopDialogueLineSelected'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Overlay_Hierarchy_ViewFacialAnimation::OnApplyToSkinSelected
// Author       : Andy Tate
// Date         : 4-Jun-2008
////////////////////////////////////////////////////////////////////////////////

void Tools_Overlay_Hierarchy_ViewFacialAnimation::OnApplyToSkinSelected(int iUserData)
{
	Tools_Overlay_Hierarchy_ViewFacialAnimation*	pxThis		= reinterpret_cast<Tools_Overlay_Hierarchy_ViewFacialAnimation*>(iUserData);
	const Asura_Hash_ID								uFaceHash	= pxThis->m_uCurrentFaceHash;

	pxThis->SetCurrentFace(uFaceHash, true);
}

/// End of function 'Tools_Overlay_Hierarchy_ViewFacialAnimation::OnApplyToSkinSelected'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Overlay_Hierarchy_ViewFacialAnimation::OnResetWeightsAndBalancesSelected
// Author       : Andy Tate
// Date         : 23-May-2008
////////////////////////////////////////////////////////////////////////////////

void Tools_Overlay_Hierarchy_ViewFacialAnimation::OnResetWeightsAndBalancesSelected(int iUserData)
{
	Tools_Overlay_Hierarchy_ViewFacialAnimation*	pxThis				= reinterpret_cast<Tools_Overlay_Hierarchy_ViewFacialAnimation*>(iUserData);
	Asura_FacialAnimation_AnimationHandler*			pxAnimationHandler	= pxThis->m_pxAnimationHandler;

	if(pxAnimationHandler != NULL)
	{
		pxAnimationHandler->ZeroWeightsAndBalances();
	}
}

/// End of function 'Tools_Overlay_Hierarchy_ViewFacialAnimation::OnResetWeightsAndBalancesSelected'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Overlay_Hierarchy_ViewFacialAnimation::OnLoadPoseSelected
// Author       : Andy Tate
// Date         : 3-Jun-2008
////////////////////////////////////////////////////////////////////////////////

void Tools_Overlay_Hierarchy_ViewFacialAnimation::OnLoadPoseSelected(int iUserData)
{
	const Asura_Char*			szDefaultPath		= Tools_FacialAnimation_System::GetPoseDirectoryName();
	const Asura_Char*			szDefaultExtension	= Tools_FacialAnimation_System::GetPoseFileExtension();

	const Asura_String			xFileFilter(Tools_FacialAnimation_System::CreatePoseFilename("*", false));

	const Asura_Char*			szFileFilter		= xFileFilter.GetString();

	Tools_FileSystem_DialogInfo	xDialogueInfo("Select filename for expression", szDefaultPath, szFileFilter, szDefaultExtension, szFileFilter);

	const Asura_String			xSelectedFilename(Tools_FileSystem::GetLoadFilename(xDialogueInfo));

	if(!xSelectedFilename.IsBlank())
	{
		Asura_String	xPoseName;

		if(xSelectedFilename.SplitFilename(NULL, &xPoseName, NULL))
		{
			const char*	szPoseName			= xPoseName.GetString();
			const char*	szSelectedFilename	= xSelectedFilename.GetString();

			// Remove any existing pose before loading the specified one...
			Tools_FacialAnimation_System::DestroyPose(szPoseName);

			if(LoadAsuraFile(szSelectedFilename))
			{
				const Tools_FacialAnimation_FacePose*	pxPose	= Tools_FacialAnimation_System::FindPose(szPoseName);

				if(pxPose == NULL)
				{
					Message_Overlay	xErrorOverlay("Failed to find pose");

					xErrorOverlay.DoModal();
				}
				else
				{
					Tools_Overlay_Hierarchy_ViewFacialAnimation*	pxThis				= reinterpret_cast<Tools_Overlay_Hierarchy_ViewFacialAnimation*>(iUserData);
					Asura_FacialAnimation_AnimationHandler*			pxAnimationHandler	= pxThis->m_pxAnimationHandler;

					pxPose->CopyToAnimationHandler(*pxAnimationHandler);
				}
			}
		}
		else
		{
			Message_Overlay	xErrorOverlay("Failed to load pose");

			xErrorOverlay.DoModal();
		}
	}
}

/// End of function 'Tools_Overlay_Hierarchy_ViewFacialAnimation::OnLoadPoseSelected'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Overlay_Hierarchy_ViewFacialAnimation::OnSaveAsPoseSelected
// Author       : Andy Tate
// Date         : 3-Jun-2008
////////////////////////////////////////////////////////////////////////////////

void Tools_Overlay_Hierarchy_ViewFacialAnimation::OnSaveAsPoseSelected(int iUserData)
{
	const Asura_Char*			szDefaultPath		= Tools_FacialAnimation_System::GetPoseDirectoryName();
	const Asura_Char*			szDefaultExtension	= Tools_FacialAnimation_System::GetPoseFileExtension();

	const Asura_String			xDefaultFilename(Tools_FacialAnimation_System::CreatePoseFilename("*", false));
	const Asura_String			xFileFilter(Tools_FacialAnimation_System::CreatePoseFilename("*", false));

	const Asura_Char*			szDefaultFilename	= xDefaultFilename.GetString();
	const Asura_Char*			szFileFilter		= xFileFilter.GetString();

	Tools_FileSystem_DialogInfo	xDialogueInfo("Select filename for expression", szDefaultPath, szDefaultFilename, szDefaultExtension, szFileFilter);

	const Asura_String			xSelectedFilename(Tools_FileSystem::GetSaveFilename(xDialogueInfo));

	if(!xSelectedFilename.IsBlank())
	{
		Asura_String	xPoseName;

		if(xSelectedFilename.SplitFilename(NULL, &xPoseName, NULL))
		{
			const char*						szPoseName	= xPoseName.GetString();
			Tools_FacialAnimation_FacePose*	pxPose		= Tools_FacialAnimation_System::CreatePose(szPoseName);

			if(pxPose == NULL)
			{
				Message_Overlay	xErrorOverlay("Failed to create expression");

				xErrorOverlay.DoModal();
			}
			else
			{
				const Tools_Overlay_Hierarchy_ViewFacialAnimation*	pxThis				= reinterpret_cast<Tools_Overlay_Hierarchy_ViewFacialAnimation*>(iUserData);
				const Asura_FacialAnimation_AnimationHandler*		pxAnimationHandler	= pxThis->m_pxAnimationHandler;
				const char*											szSelectedFilename	= xSelectedFilename.GetString();
				const Asura_Hash_ID									uFaceHash			= pxThis->m_uCurrentFaceHash;
				const Tools_FacialAnimation_Face*					pxFace				= Tools_FacialAnimation_System::FindFace(uFaceHash);

				// Update the face-pose from the current data in our animation-handler...
				pxPose->CopyFromAnimationHandler(*pxAnimationHandler);

				// Ensure that we're matched to the correct face-type...
				if(pxFace != NULL)
				{
					const Asura_String&	xFaceTypeName	= pxFace->GetFaceTypeName();

					pxPose->SetFaceTypeName(xFaceTypeName);
				}

				// Attempt to save the file...
				if(Tools_FacialAnimation_System::WritePoseFile(szSelectedFilename, *pxPose))
				{
					Tools_Timed_Message_System::AddMessage(Tools_Timed_Message_System::COLOUR_SUCCESS, "\"%s\" saved successfully.", szSelectedFilename);
				}
				else
				{
					Tools_Timed_Message_System::AddMessage(Tools_Timed_Message_System::COLOUR_ERROR, "Failed to save \"%s\".", szSelectedFilename);
				}
			}
		}
	}
}

/// End of function 'Tools_Overlay_Hierarchy_ViewFacialAnimation::OnSaveAsPoseSelected'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Overlay_Hierarchy_ViewFacialAnimation::CanSelectFace
// Author       : Andy Tate
// Date         : 28-Aug-2008
////////////////////////////////////////////////////////////////////////////////

bool Tools_Overlay_Hierarchy_ViewFacialAnimation::CanSelectFace(int iUserData)
{
	const Asura_Hierarchy_Skin*	pxSkin	= Tools_Mode_Hierarchy::GetCurrentSkin();

	return(pxSkin != NULL);
}

/// End of function 'Tools_Overlay_Hierarchy_ViewFacialAnimation::CanSelectFace'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Overlay_Hierarchy_ViewFacialAnimation::CanPlayDialogueLine
// Author       : Andy Tate
// Date         : 7-Aug-2008
////////////////////////////////////////////////////////////////////////////////

bool Tools_Overlay_Hierarchy_ViewFacialAnimation::CanPlayDialogueLine(int iUserData)
{
	const Tools_Overlay_Hierarchy_ViewFacialAnimation*	pxThis	= reinterpret_cast<Tools_Overlay_Hierarchy_ViewFacialAnimation*>(iUserData);
	const Tools_FacialAnimation_Face*					pxFace	= pxThis->GetCurrentFace();
	const Asura_Hierarchy_Skin*							pxSkin	= Tools_Mode_Hierarchy::GetCurrentSkin();

	return((pxThis->m_iDialogueLineHandle == ASURA_INVALID_HANDLE) && (pxSkin != NULL) && (pxFace != NULL) && !pxThis->m_bApplyToSkin);
}

/// End of function 'Tools_Overlay_Hierarchy_ViewFacialAnimation::CanPlayDialogueLine'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Overlay_Hierarchy_ViewFacialAnimation::CanStopDialogueLine
// Author       : Andy Tate
// Date         : 7-Aug-2008
////////////////////////////////////////////////////////////////////////////////

bool Tools_Overlay_Hierarchy_ViewFacialAnimation::CanStopDialogueLine(int iUserData)
{
	const Tools_Overlay_Hierarchy_ViewFacialAnimation*	pxThis	= reinterpret_cast<Tools_Overlay_Hierarchy_ViewFacialAnimation*>(iUserData);
	const Tools_FacialAnimation_Face*					pxFace	= pxThis->GetCurrentFace();
	const Asura_Hierarchy_Skin*							pxSkin	= Tools_Mode_Hierarchy::GetCurrentSkin();

	return((pxThis->m_iDialogueLineHandle != ASURA_INVALID_HANDLE) && (pxSkin != NULL) && (pxFace != NULL) && !pxThis->m_bApplyToSkin);
}

/// End of function 'Tools_Overlay_Hierarchy_ViewFacialAnimation::CanStopDialogueLine'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Overlay_Hierarchy_ViewFacialAnimation::CanBeAppliedToSkin
// Author       : Andy Tate
// Date         : 21-May-2008
////////////////////////////////////////////////////////////////////////////////

bool Tools_Overlay_Hierarchy_ViewFacialAnimation::CanBeAppliedToSkin(int iUserData)
{
	const Tools_Overlay_Hierarchy_ViewFacialAnimation*	pxThis	= reinterpret_cast<Tools_Overlay_Hierarchy_ViewFacialAnimation*>(iUserData);
	const Tools_FacialAnimation_Face*					pxFace	= pxThis->GetCurrentFace();
	const Asura_Hierarchy_Skin*							pxSkin	= Tools_Mode_Hierarchy::GetCurrentSkin();

	return((pxThis->m_iDialogueLineHandle == ASURA_INVALID_HANDLE) && (pxSkin != NULL) && (pxFace != NULL));
}

/// End of function 'Tools_Overlay_Hierarchy_ViewFacialAnimation::CanBeAppliedToSkin'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Overlay_Hierarchy_ViewFacialAnimation::CanPlayFacialAnimation
// Author       : Gaz Thoams
// Date         : 26-March-2008
////////////////////////////////////////////////////////////////////////////////

bool Tools_Overlay_Hierarchy_ViewFacialAnimation::CanPlayFacialAnimation(int iUserData)
{
	const Tools_Overlay_Hierarchy_ViewFacialAnimation*	pxThis	= reinterpret_cast<Tools_Overlay_Hierarchy_ViewFacialAnimation*>(iUserData);
	const Tools_FacialAnimation_Face*					pxFace	= pxThis->GetCurrentFace();
	const Asura_Hierarchy_Skin*							pxSkin	= Tools_Mode_Hierarchy::GetCurrentSkin();
	
	return ( (pxSkin != NULL) && (pxFace != NULL) );
}

/// End of function 'Tools_Overlay_Hierarchy_ViewFacialAnimation::CanPlayFacialAnimation'. ///

bool Tools_Overlay_Hierarchy_ViewFacialAnimation::CanStopFacialAnimation(int iUserData)
{
	const Tools_Overlay_Hierarchy_ViewFacialAnimation*	pxThis	= reinterpret_cast<Tools_Overlay_Hierarchy_ViewFacialAnimation*>(iUserData);
	const Tools_FacialAnimation_Face*					pxFace	= pxThis->GetCurrentFace();
	const Asura_Hierarchy_Skin*							pxSkin	= Tools_Mode_Hierarchy::GetCurrentSkin();

	return ( (pxSkin != NULL) && (pxFace != NULL) && (pxThis->m_uFaceAnimHashId != ASURA_HASH_ID_UNSET) );
}



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Overlay_Hierarchy_ViewFacialAnimation::IsFacialAnimationAppliedToSkin
// Author       : Andy Tate
// Date         : 21-May-2008
////////////////////////////////////////////////////////////////////////////////

bool Tools_Overlay_Hierarchy_ViewFacialAnimation::IsFacialAnimationAppliedToSkin(int iUserData)
{
	const Tools_Overlay_Hierarchy_ViewFacialAnimation*	pxThis	= reinterpret_cast<Tools_Overlay_Hierarchy_ViewFacialAnimation*>(iUserData);

	return(pxThis->m_bApplyToSkin);
}

/// End of function 'Tools_Overlay_Hierarchy_ViewFacialAnimation::IsFacialAnimationAppliedToSkin'. ///



////////////////////////////////////////////////////////////////////////////////
// Public functions...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Overlay_Hierarchy_ViewFacialAnimation::CreateOverlay
// Author       : Andy Tate
// Date         : 20-May-2008
////////////////////////////////////////////////////////////////////////////////

void Tools_Overlay_Hierarchy_ViewFacialAnimation::CreateOverlay(void)
{
	if(Asura_Overlay_System::DoesOverlayExist(TOOLS_OVERLAY_HIERARCHY_VIEWFACIALANIMATION))
	{
		Asura_Overlay_System::BringOverlayToTop(TOOLS_OVERLAY_HIERARCHY_VIEWFACIALANIMATION);
	}
	else
	{
		Tools_Overlay_Hierarchy_ViewFacialAnimation*	pxOverlay	= new Tools_Overlay_Hierarchy_ViewFacialAnimation;

		Asura_Overlay_System::SubmitNewOverlay(pxOverlay);
	}
}

/// End of function 'Tools_Overlay_Hierarchy_ViewFacialAnimation::CreateOverlay'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Overlay_Hierarchy_ViewFacialAnimation::CloseOverlay
// Author       : Andy Tate
// Date         : 20-May-2008
////////////////////////////////////////////////////////////////////////////////

void Tools_Overlay_Hierarchy_ViewFacialAnimation::CloseOverlay(void)
{
	Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_HIERARCHY_VIEWFACIALANIMATION);
}

/// End of function 'Tools_Overlay_Hierarchy_ViewFacialAnimation::CloseOverlay'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Overlay_Hierarchy_ViewFacialAnimation::PerFrameUpdate
// Author       : Andy Tate
// Date         : 22-May-2008
////////////////////////////////////////////////////////////////////////////////

void Tools_Overlay_Hierarchy_ViewFacialAnimation::PerFrameUpdate(void)
{
	const bool	bDialoguePlaying	= (m_iDialogueLineHandle != ASURA_INVALID_HANDLE);

	// Update the animation-handler ourselves...
	if(m_pxAnimationHandler != NULL)
	{
		const bool	bFaceAnimationPlaying	= m_pxAnimationHandler->IsPlayingFaceAnimation();

		m_pxAnimationHandler->SetActive(m_bApplyToSkin || bDialoguePlaying || bFaceAnimationPlaying);
	}

	// Check whether we've finished playing dialogue...
	if(bDialoguePlaying)
	{
		if(!Asura_Dialogue_System::IsPlaying(m_iDialogueLineHandle))
		{
			Asura_Dialogue_System::FinishedWithHandle(m_iDialogueLineHandle);
			m_iDialogueLineHandle	= ASURA_INVALID_HANDLE;
		}
	}

	// Finally, call the base-function...
	PARENT::PerFrameUpdate();
}

/// End of function 'Tools_Overlay_Hierarchy_ViewFacialAnimation::PerFrameUpdate'. ///


////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Overlay_Hierarchy_ViewFacialAnimation::PlayLinkedFacialAnimation
// Author       : Richard Rice
// Date         : 15-Dec-2008
////////////////////////////////////////////////////////////////////////////////

void Tools_Overlay_Hierarchy_ViewFacialAnimation::PlayLinkedFacialAnimation(const Asura_Hierarchy_Anim* pxAnim)
{
	if(!s_pxThis) return;
	if(!s_pxThis->m_pxAnimationHandler) return;

	if(pxAnim && pxAnim->HasFacialAnimation())
	{
		s_pxThis->m_pxAnimationHandler->HierarchyMode_PlayFacialAnimation(pxAnim->GetHashID());
	}
	else
	{
		//Either not playing an animation or the animation doesn't have a facial animation , so stop the current
		//facial animation that is linked to the body animation.
		s_pxThis->m_pxAnimationHandler->HierarchyMode_StopFacialAnimation();
	}
}

/// End of function 'Tools_Overlay_Hierarchy_ViewFacialAnimation::PlayLinkedFacialAnimation'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_TableCell_MuscleGroupApplyControlType_Dialog::Tools_TableCell_MuscleGroupApplyControlType_Dialog
// Author       : Andy Tate
// Date         : 21-May-2008
////////////////////////////////////////////////////////////////////////////////

Tools_TableCell_MuscleGroupApplyControlType_Dialog::Tools_TableCell_MuscleGroupApplyControlType_Dialog(const Tools_FacialAnimation_Face_MuscleGroup& xMuscleGroup,
																									   Asura_Table_Dialog& xTableDialogue,
																									   const Asura_Vector_2& xOffset,
																									   const Asura_Vector_2& xMinCellSize,
																									   const u_int uRow,
																									   const u_int uColumn, 
																									   Tools_Overlay_Hierarchy_ViewFacialAnimation& xOverlay)
: PARENT(xTableDialogue, xOffset, xMinCellSize, uRow, uColumn)
, m_xOverlay(xOverlay)
, m_xMuscleGroup(xMuscleGroup)
{
	const Asura_FacialAnimation_MuscleGroup::ControlType	eControlType		= m_xMuscleGroup.GetControlType();
	Asura_FacialAnimation_AnimationHandler*					pxAnimationHandler	= xOverlay.GetAnimationHandler();
	float*													pfWeight			= (pxAnimationHandler == NULL) ? &s_fUnusedWeight : pxAnimationHandler->Tools_GetWeightPointer(uRow);
	float*													pfBalance			= (pxAnimationHandler == NULL) ? &s_fUnusedBalance : pxAnimationHandler->Tools_GetBalancePointer(uRow);

	Asura_Vector_2											xLocalOffset(0.0f, 0.0f);

	Asura_Dialog*											pxNewDialogue;

	// Create the dialogues for the appropriate type of control...
	switch(eControlType)
	{
		case Asura_FacialAnimation_MuscleGroup::CT_SINGLE:
		{
			pxNewDialogue		= new Asura_Numerical_Dialog ("Weight", xLocalOffset, pfWeight, g_fMinWeightValue, g_fMaxWeightValue, 0.100f, "%.3f");
			AddDialogToPanel(pxNewDialogue);
			break;
		}

		case Asura_FacialAnimation_MuscleGroup::CT_SINGLE_PAIR:
		{
			pxNewDialogue		= new Asura_Numerical_Dialog ("Weight", xLocalOffset, pfWeight, g_fMinPairWeightValue, g_fMaxPairWeightValue, 0.100f, "%.2f");
			AddDialogToPanel(pxNewDialogue);
			break;
		}

		case Asura_FacialAnimation_MuscleGroup::CT_BALANCE:
		{
			pxNewDialogue		= new Asura_Numerical_Dialog ("Weight", xLocalOffset, pfWeight, g_fMinWeightValue, g_fMaxWeightValue, 0.100f, "%.3f");
			AddDialogToPanel(pxNewDialogue);

			xLocalOffset[0]		= pxNewDialogue->m_xMaxCoord[0] + 3.0f;
			pxNewDialogue		= new Asura_NumberSlider_Dialog (xLocalOffset, pfBalance, &g_fMinBalanceValue, &g_fMaxBalanceValue, 10.0f, "");
			AddDialogToPanel(pxNewDialogue);
			break;
		}

		case Asura_FacialAnimation_MuscleGroup::CT_BALANCE_PAIR:
		{
			pxNewDialogue		= new Asura_Numerical_Dialog ("Weight", xLocalOffset, pfWeight, g_fMinPairWeightValue, g_fMaxPairWeightValue, 0.100f, "%.2f");
			AddDialogToPanel(pxNewDialogue);

			xLocalOffset[0]		= pxNewDialogue->m_xMaxCoord[0] + 3.0f;
			pxNewDialogue		= new Asura_NumberSlider_Dialog (xLocalOffset, pfBalance, &g_fMinBalanceValue, &g_fMaxBalanceValue, 10.0f, "");
			AddDialogToPanel(pxNewDialogue);
			break;
		}

		default:
		{
			Asura_Assert(false, "Unexpected control-type found for this muscle-group.");
			break;
		}
	}		// End of switch.
}

/// End of function 'Tools_TableCell_MuscleGroupApplyControlType_Dialog::Tools_TableCell_MuscleGroupApplyControlType_Dialog'. ///


////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_TableCell_EyeValueApplyControlType_Dialog::Tools_TableCell_EyeValueApplyControlType_Dialog
// Author       : Richard Rice
// Date         : 30-Oct-2008
////////////////////////////////////////////////////////////////////////////////

Tools_TableCell_EyeValueApplyControlType_Dialog::Tools_TableCell_EyeValueApplyControlType_Dialog(u_int uEyeValueIndex,
																									   Asura_Table_Dialog& xTableDialogue,
																									   const Asura_Vector_2& xOffset,
																									   const Asura_Vector_2& xMinCellSize,
																									   const u_int uRow,
																									   const u_int uColumn, 
																									   Tools_Overlay_Hierarchy_ViewFacialAnimation& xOverlay)
: PARENT(xTableDialogue, xOffset, xMinCellSize, uRow, uColumn)
, m_xOverlay(xOverlay)
, m_uEyeValueIndex(uEyeValueIndex)
{
	Asura_FacialAnimation_AnimationHandler*					pxAnimationHandler	= xOverlay.GetAnimationHandler();
	float*													pfWeight			= (pxAnimationHandler == NULL) ? &s_fUnusedWeight : pxAnimationHandler->Tools_GetEyeValuePointer(m_uEyeValueIndex);
	if(!pfWeight) pfWeight = &s_fUnusedWeight;

	Asura_Vector_2											xLocalOffset(0.0f, 0.0f);

	Asura_Numerical_Dialog*											pxNewDialogue;

	static float s_afEyeValueMin[ASURA_FACIALANIMATION_EYE_NUMPARAM] =
	{
		-1.0f,	//ASURA_FACIALANIMATION_EYE_LEFTRIGHT = 0,
		-1.0f,	//ASURA_FACIALANIMATION_EYE_UPDOWN = 1,
		0.0f, 	//ASURA_FACIALANIMATION_EYE_FOCUS = 2,
		-1.0f,	//ASURA_FACIALANIMATION_EYE_CLOSE = 3,
		0.0f, 	//ASURA_FACIALANIMATION_EYE_SQUINT = 4,
		-1.0f,	//ASURA_FACIALANIMATION_EYE_BALANCE = 5,
		0.0f, 	//ASURA_FACIALANIMATION_EYE_TARGETMIX = 6,
	};



	pxNewDialogue = new Asura_Numerical_Dialog ("Weight", xLocalOffset, pfWeight, s_afEyeValueMin[m_uEyeValueIndex], 1.0f, 0.100f, "%.2f");
	pxNewDialogue->SetBoxLength(5);
	AddDialogToPanel(pxNewDialogue);
}

/// End of function 'Tools_TableCell_EyeValueApplyControlType_Dialog::Tools_TableCell_EyeValueApplyControlType_Dialog'. ///


#endif	//defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)


/// End of file. ///
