
//
// Tools_Animation_Files.cpp
//
// Created by Richard Rice - 15/08/2002
// 
// Copyright (C) 2002 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"
#include "Hierarchy/Tools_Animation_Files.h"

#include "Animation/Asura_Animation.h"
#include "Animation/Asura_Hierarchy.h"
#ifndef ASURA_DISABLE_ANIM_TABLE_CODE
#include "Animation/Asura_AnimTable_System.h"
#endif
#include "Core/Asura_TextureManagement.h"
#include "ChunkSystem/Asura_Chunk_Loaders.h"
#include "Debugging/Asura_Logfile.h"
#include "Debugging/Asura_Assert.h"
#include "Entity/PhysicalObject/Asura_ServerEntity_PhysicalObject.h"
#include "FileAccess/Asura_FileAccess.h"
#ifdef ASURA_SUPPORT_COMMAND_CONSOLE
#include "Interface/Asura_CommandConsole.h"
#include "Interface/Asura_CommandConsole_IO.h"
#endif
#include "Mesh/Asura_Mesh.h"
#include "Mesh/Asura_Mesh_System.h"

#if !defined(ASURA_PC_DISABLE_PRESM3_SMOOTH_SKIN_RENDERER) || !ASURA_PC_DISABLE_PRESM3_SMOOTH_SKIN_RENDERER
#include "Rendering/Asura_PlatformRenderCharacter.h"
#endif
#if defined(ASURA_PC_USE_SHADER_MODEL_3_SMOOTH_SKIN_RENDERER) && ASURA_PC_USE_SHADER_MODEL_3_SMOOTH_SKIN_RENDERER
#include "Rendering/Asura_Direct3D_SmoothSkinRenderer.h"
#endif
#include "Rendering/Asura_Platform_RenderObjectHierarchy.h"
#include "Sound/Asura_Audio_System.h"

#include "Tools_System.h"
#include "Hierarchy/Tools_Mode_hierarchy.h"
#include "Load&Save/Tools_AsuraFileContents.h"
#include "Load&Save/Tools_SaveAsura.h"
#include "Mesh/Tools_PlatformMesh_System.h"
#include "Misc/Tools_Modal_Overlay.h"
#include "Misc/Tools_Timed_Messages.h"
#include "Misc/Tools_IniFile.h"
#include "Misc/Tools_Platform.h"
#include "PC/FileSystem/Tools_FileSystem.h"
#include "Sound/Tools_PickSound.h"

static const char* g_pszChooseDirectoryString = "*Choose Directory*";
static const char* g_pszChooseCategoryString = "*Choose Category*";
static const char* g_pszDisplayAllString = "*Display All*";
static const char* g_pszCutsceneSpecificString = "*Cutscene Specific*";
static const char* g_pszBrowseDiskString = "*Browse Disk*";
static const char* g_pszNone = "*None*";

Tools_Animation_Files* Tools_Animation_Files::s_xThis = 0;

Tools_Animation_Files::Tools_Animation_Files()
{
	s_xThis = this;
	m_bIniFileLoaded = false;
	m_pxCurrentSkinCat = 0;
}

Tools_Animation_Files::~Tools_Animation_Files()
{
	while(m_xDirectoryList.size())
	{
		delete m_xDirectoryList.first_entry();
		m_xDirectoryList.delete_first_entry();
	}
	while(m_xSkinList.size())
	{
		delete m_xSkinList.first_entry();
		m_xSkinList.delete_first_entry();
	}
	while(m_xSkinCatList.size())
	{
		delete m_xSkinCatList.first_entry();
		m_xSkinCatList.delete_first_entry();
	}
	while(m_xAnimCatList.size())
	{
		delete m_xAnimCatList.first_entry();
		m_xAnimCatList.delete_first_entry();
	}

	s_xThis = 0;
}

void Tools_Animation_Files::UpdateDirectoryList()
{
	while(m_xDirectoryList.size())
	{
		delete m_xDirectoryList.first_entry();
		m_xDirectoryList.delete_first_entry();
	}

	char szSearch[MAX_PATH];
	for(int iNetwork=0;iNetwork<2;iNetwork++)
	{
		#ifdef ASURA_PC_ALLOW_LOAD_FROM_NETWORK
		const char* pszPath = iNetwork ? Asura_FileAccess::PlatformGetNetworkPath() : ".";
		#else
		const char* pszPath = ".";
		#endif
		
		for(int i=0;i<GetNumberOfRootDirectories();i++)
		{
			//add all the directories of the form Skinname\Anims
			sprintf(szSearch,"%s\\%s\\*.*",pszPath,GetDirectory(i));
			WIN32_FIND_DATA wfd;
			HANDLE hFindFile = ::FindFirstFile(szSearch,&wfd);

			char szTemp[MAX_PATH];
			if(INVALID_HANDLE_VALUE != hFindFile)
			{
				do
				{
					if(wfd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
					{
						sprintf(szTemp,"%s\\%s\\%s\\anims",pszPath,GetDirectory(i),wfd.cFileName);
						//see if this directory exists
						DWORD attr = GetFileAttributes(szTemp);
						if(attr != 0xffffffff && (attr & FILE_ATTRIBUTE_DIRECTORY))
						{
							//add this directory
							sprintf(szTemp,"%s\\%s\\anims",GetDirectory(i),wfd.cFileName);
							AddDirectory(szTemp,wfd.cFileName);
							continue;
						}
						//also add if there is an AnimDirs.ini file
						sprintf(szTemp,"%s\\%s\\%s\\AnimDirs.ini",pszPath,GetDirectory(i),wfd.cFileName);
						attr = GetFileAttributes(szTemp);
						if(attr != 0xffffffff)
						{
							//add this directory
							sprintf(szTemp,"%s\\%s\\anims",GetDirectory(i),wfd.cFileName);
							AddDirectory(szTemp,wfd.cFileName);
						}
						

					}
				}while(::FindNextFile(hFindFile,&wfd));

				::FindClose(hFindFile);
			}

			//add all the subdirectorries of Anims_Body_Types
			sprintf(szSearch,"%s\\%s\\Anims_Body_Types\\*.*",pszPath,GetDirectory(i));
			hFindFile = ::FindFirstFile(szSearch,&wfd);

			if(INVALID_HANDLE_VALUE != hFindFile)
			{
				do
				{
					if(wfd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
					{
						if(wfd.cFileName[0] == '.') continue;
						
						//add this directory
						sprintf(szTemp,"%s\\Anims_Body_Types\\%s",GetDirectory(i),wfd.cFileName);
						AddDirectory(szTemp,wfd.cFileName);
					}
				}while(::FindNextFile(hFindFile,&wfd));

				::FindClose(hFindFile);
			}

			//add all the subdirectorries of Anims_Specialised
			sprintf(szSearch,"%s\\%s\\Anims_Specialised\\*.*",pszPath,GetDirectory(i));
			hFindFile = ::FindFirstFile(szSearch,&wfd);

			if(INVALID_HANDLE_VALUE != hFindFile)
			{
				do
				{
					if(wfd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
					{
						if(wfd.cFileName[0] == '.') continue;
						
						//add this directory
						sprintf(szTemp,"%s\\Anims_Specialised\\%s",GetDirectory(i),wfd.cFileName);
						AddDirectory(szTemp,wfd.cFileName);
					}
				}while(::FindNextFile(hFindFile,&wfd));

				::FindClose(hFindFile);
			}
		}
	}
}

void Tools_Animation_Files::AddDirectory(const char* pszDir,const char* pszName)
{
	//make sure the direcyory hasn't already been added
	for(LIF<ToolsAnimationDirectory*> xDirLIF(&m_xDirectoryList);!xDirLIF.done();xDirLIF.next())
	{
		if(!stricmp(xDirLIF()->m_xDirectory.GetString(),pszDir))
		{
			return;
		}
	}	

	ToolsAnimationDirectory* pxDir = new ToolsAnimationDirectory;
	pxDir->m_xDirectory = pszDir;
	pxDir->m_xName = pszName;
	m_xDirectoryList.add_entry(pxDir);
	
}

void Tools_Animation_Files::UpdateSkinList()
{
	while(m_xSkinList.size())
	{
		delete m_xSkinList.first_entry();
		m_xSkinList.delete_first_entry();
	}


	for(int iNetwork=0;iNetwork<2;iNetwork++)
	{
		#ifdef ASURA_PC_ALLOW_LOAD_FROM_NETWORK
		const char* pszPath = iNetwork ? Asura_FileAccess::PlatformGetNetworkPath() : ".";
		#else
		const char* pszPath = ".";
		#endif

		for(int i=0;i<GetNumberOfRootDirectories();i++)
		{
			//look for subdirectories of Characters
			WIN32_FIND_DATA wfd;
			char szSearchMain[MAX_PATH];
			sprintf(szSearchMain,"%s\\%s\\*.*",pszPath,GetDirectory(i));
			HANDLE hFindFile = ::FindFirstFile(szSearchMain,&wfd);

			if(INVALID_HANDLE_VALUE != hFindFile)
			{
				do
				{
					if(wfd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
					{
						if(wfd.cFileName[0]=='.') continue;
						
						//get all the files in this directory
						char szSearch[MAX_PATH];
						sprintf(szSearch,"%s\\%s\\%s\\*.asr",pszPath,GetDirectory(i),wfd.cFileName);
							
						WIN32_FIND_DATA wfd2;
						HANDLE hFindFile2 = ::FindFirstFile(szSearch,&wfd2);

						if(INVALID_HANDLE_VALUE != hFindFile2)
						{
							do
							{
								if(!(wfd2.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY))
								{
									char szTemp[MAX_PATH];
									sprintf(szTemp,"%s\\%s\\%s",GetDirectory(i),wfd.cFileName,wfd2.cFileName);
									AddToSkinList(szTemp);

								}
							}while(::FindNextFile(hFindFile2,&wfd2));

							::FindClose(hFindFile2);
						}
						
					}
				}while(::FindNextFile(hFindFile,&wfd));

				::FindClose(hFindFile);
			}
		}
	}

}

void Tools_Animation_Files::AddToSkinList(const char* pszFilename)
{
	Asura_String xFilename = pszFilename;
	Asura_String xPath,xName;
	xFilename.SplitFilename(&xPath,&xName,0);
	
	
	//see if its already in the list
	for(LIF<ToolsSkinFile*> xSkinLIF(&m_xSkinList);!xSkinLIF.done();xSkinLIF.next())
	{
		if(!stricmp(xSkinLIF()->m_xName.GetString(),xName.GetString()))
		{
			return;
		}
	}
	
	// JAT 19/04/07 ignore the file if it isn't a skin file 
	// RAR 19/05/08 Skipping this since its fairly slow.
	//Tools_AsuraFileContents xContents;
	//if ( xContents.GetAsuraFileType( pszFilename ) != AsuraFile_Skin )
	//{
	//	return;
	//}

	ToolsSkinFile* pxSkin = new ToolsSkinFile;
	pxSkin->m_xFilename = pszFilename;
	pxSkin->m_xName = xName;

		//if there is an animation text file , use that as the default animation 'directory'
	char szAnimList[MAX_PATH];
	char szAnimListNet[MAX_PATH];
	char szAnimDir[MAX_PATH];
	sprintf(szAnimList,"%s\\AnimationList.txt",xPath.GetString());
	Asura_FileAccess::PlatformGetNetworkFilename(szAnimList,szAnimListNet,MAX_PATH);

	HANDLE xHandle = CreateFile(szAnimListNet,GENERIC_READ,FILE_SHARE_READ,0,OPEN_EXISTING,0,0);
	if(xHandle!=INVALID_HANDLE_VALUE)
	{

		strcpy(szAnimDir,szAnimListNet);
		CloseHandle(xHandle);
	}
	else
	{
		sprintf(szAnimDir,"%s\\anims",xPath.GetString());
	}

	pxSkin->m_xDefaultAnimDir = szAnimDir;

	m_xSkinList.add_entry(pxSkin);
}


const List<ToolsAnimationDirectory*>* Tools_Animation_Files::GetDirectoryList()
{
	Asura_Assert(s_xThis,"");
	return s_xThis->GetDirectoryListMain();
}


const List<ToolsAnimationDirectory*>* Tools_Animation_Files::GetDirectoryListMain()
{
	if(!m_xDirectoryList.size()) UpdateDirectoryList();

	return &m_xDirectoryList;
}

const List<ToolsSkinFile*>* Tools_Animation_Files::GetSkinList()
{
	Asura_Assert(s_xThis,"");
	return s_xThis->GetSkinListMain();
}

const List<ToolsSkinFile*>* Tools_Animation_Files::GetSkinListMain()
{
	if(!m_xSkinList.size()) UpdateSkinList();

	return &m_xSkinList;
}



void Tools_Animation_Files::ListAnimations(const char* pszDir,List<Asura_String>* pxOutputAnimNameList,List<Asura_String>* pxOutputFilenameList,bool bRecurse)
{
	Asura_Assert(s_xThis,"");
	s_xThis->ListAnimationsMain(pszDir,pxOutputAnimNameList,pxOutputFilenameList,bRecurse);
}

void Tools_Animation_Files::ListAnimationsMain(const char* pszDir,List<Asura_String>* pxOutputAnimNameList,List<Asura_String>* pxOutputFilenameList, bool bRecurse)
{
	if(!pszDir) return;
	
	if(pszDir[0]=='@')
	{
		ListAnimationsInCategory(pszDir+1,pxOutputAnimNameList,pxOutputFilenameList);
		return;
	}
	else if(strstr(pszDir,".txt"))
	{
		ListAnimationsInTextFile(pszDir,pxOutputAnimNameList,pxOutputFilenameList);
		return;
	}
	
	
	char szFilename[MAX_PATH];
	char szSearch[MAX_PATH];
	char szName[MAX_PATH];
	HANDLE hFindFile;
	WIN32_FIND_DATA wfd;

	
	//search this directory for .asr files
	for(int iNetwork=0;iNetwork<2;iNetwork++)
	{
		#ifdef ASURA_PC_ALLOW_LOAD_FROM_NETWORK
		const char* pszPath = iNetwork ? Asura_FileAccess::PlatformGetNetworkPath() : ".";
		#else
		const char* pszPath = ".";
		#endif
		
		sprintf(szSearch,"%s\\%s\\*.asr",pszPath,pszDir);
   		hFindFile = ::FindFirstFile(szSearch,&wfd);

		if(INVALID_HANDLE_VALUE != hFindFile)
		{
			do
			{	
				//get the anim name
				strcpy(szName,wfd.cFileName);
				char* pcDotPos = strrchr(szName,'.');
				if(pcDotPos) *pcDotPos = 0;

				//and relative filename
				sprintf(szFilename,"%s\\%s",pszDir,wfd.cFileName);

				//Has this anim already been added
				bool bAlreadyAdded = false;
				if(pxOutputFilenameList)
				{
					for(LIF<Asura_String> xNameLIF(pxOutputFilenameList);!xNameLIF.done();xNameLIF.next())
					{
						if(!stricmp(xNameLIF().GetString(),szFilename))
						{
							bAlreadyAdded = true;
							break;
						}
					}
				}
				else if(pxOutputAnimNameList)
				{
					for(LIF<Asura_String> xNameLIF(pxOutputAnimNameList);!xNameLIF.done();xNameLIF.next())
					{
						if(!stricmp(xNameLIF().GetString(),szName))
						{
							bAlreadyAdded = true;
							break;
						}
					}
				}


				if(!bAlreadyAdded)
				{
					if(pxOutputAnimNameList)
					{
						pxOutputAnimNameList->add_entry(szName);
					}
					if(pxOutputFilenameList)
					{
						pxOutputFilenameList->add_entry(szFilename);
					}
				}
			}
			while(::FindNextFile(hFindFile,&wfd));
			::FindClose(hFindFile);
		}
	}
	
	if(bRecurse)
	{
		//search for subdirectories of this directory
		sprintf(szFilename,"%s\\",pszDir);
		int iDirLength = strlen(pszDir);

		List<Asura_String> xDirsChecked;

		for(int iNetwork=0;iNetwork<2;iNetwork++)
		{
			#ifdef ASURA_PC_ALLOW_LOAD_FROM_NETWORK
			const char* pszPath = iNetwork ? Asura_FileAccess::PlatformGetNetworkPath() : ".";
			#else
			const char* pszPath = ".";
			#endif
			
			sprintf(szSearch,"%s\\%s\\*.*",pszPath,pszDir);
			hFindFile = ::FindFirstFile(szSearch,&wfd);

			if(INVALID_HANDLE_VALUE != hFindFile)
			{
				do
				{	
					//only interested in directories
					if(!(wfd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)) continue;
					//ignore the '.' and '..' directories
					if(wfd.cFileName[0]=='.') continue;
					
					if(iNetwork)
					{
						//make sure this directory wasn't already considered when lookin locally
						bool bChecked = false;
						for(LIF<Asura_String> xDirLIF(&xDirsChecked);!xDirLIF.done();xDirLIF.next())
						{
							if(!stricmp(xDirLIF().GetString(),wfd.cFileName))
							{
								bChecked = true;
								break;
							}
						}
						if(bChecked) continue;
					}
					else
					{
						xDirsChecked.add_entry(wfd.cFileName);
					}
					
					strcpy(&szFilename[iDirLength+1],wfd.cFileName);
					ListAnimationsMain(szFilename,pxOutputAnimNameList,pxOutputFilenameList,true);
				
				}
				while(::FindNextFile(hFindFile,&wfd));
				::FindClose(hFindFile);
			}
		}
	}
}

void Tools_Animation_Files::ListAnimationsInTextFile(const char* pszTextFile,List<Asura_String>* pxOutputAnimNameList,List<Asura_String>* pxOutputFilenameList)
{
	Tools_IniFile xFile(pszTextFile);
	if(!xFile.LoadIniFile()) return;

	char szTemp[MAX_PATH];

	while(xFile.GetNextIniTextLine())
	{
		//Is this entry a list of files that needs to be searched (*.asr) , or a single filename?
		if(strchr(xFile.LineBuffer,'*'))
		{
			//do a search for the files based on this entry
			WIN32_FIND_DATA wfd;
			HANDLE hFindFile = ::FindFirstFile(xFile.LineBuffer,&wfd);

			if(INVALID_HANDLE_VALUE != hFindFile)
			{
				do
				{	
					if(wfd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) continue;
		
					if(pxOutputAnimNameList)
					{
						strcpy(szTemp,wfd.cFileName);
						//remove the file extension
						char* pcDotPos = strrchr(szTemp,'.');
						if(pcDotPos) *pcDotPos = 0;
						pxOutputAnimNameList->add_entry(szTemp);
					}
					if(pxOutputFilenameList)
					{
						strcpy(szTemp,xFile.LineBuffer);
						char* pcSlash = strrchr(szTemp,'\\');
						if(!pcSlash) pcSlash = strrchr(szTemp,'/');
						strcpy(pcSlash ? pcSlash+1 : szTemp,wfd.cFileName);

						pxOutputFilenameList->add_entry(szTemp);
					}
				}
				while(::FindNextFile(hFindFile,&wfd));
				::FindClose(hFindFile);
			}
		}
		else
		{
			//see if the named file exists
			HANDLE xHandle = CreateFile(xFile.LineBuffer,GENERIC_READ,FILE_SHARE_READ,0,OPEN_EXISTING,0,0);
			if(xHandle!=INVALID_HANDLE_VALUE)
			{
				CloseHandle(xHandle);
				if(pxOutputFilenameList)
				{
					pxOutputFilenameList->add_entry(xFile.LineBuffer);
				}
				if(pxOutputAnimNameList)
				{
					strcpy(szTemp,Tools_FileSystem::StripPath(xFile.LineBuffer));
					char* pcDot = strrchr(szTemp,'.');
					if(pcDot) *pcDot = 0;
					pxOutputAnimNameList->add_entry(szTemp);
				}
			}
						
		}
	}
}

void Tools_Animation_Files::ListAnimationsInCategory(const char* pszCategory,List<Asura_String>* pxOutputAnimNameList,List<Asura_String>* pxOutputFilenameList)
{
	Asura_Assert(pszCategory,"");

	//find the anim cat
	ToolsAnimCategory* pxAnimCat = 0;
	for(LIF<ToolsAnimCategory*> xCatLIF(&m_xAnimCatList);!xCatLIF.done();xCatLIF.next())
	{
		if(!stricmp(xCatLIF()->m_xName.GetString(),pszCategory))
		{
			//found it
			pxAnimCat = xCatLIF();
			break;
		}
	}
	if(!pxAnimCat) return;


	for(LIF<Asura_String> xAnimLIF(&pxAnimCat->m_xAnimList);!xAnimLIF.done();xAnimLIF.next())
	{
		Asura_String xFilename;
		if(FindFile(xAnimLIF().GetString(),&xFilename))
		{
			if(pxOutputAnimNameList) pxOutputAnimNameList->add_entry(xAnimLIF());			
			if(pxOutputFilenameList) pxOutputFilenameList->add_entry(xFilename);
		}
	}
}


const char* Tools_Animation_Files::GetDefaultAnimDirForSkin(const char* pszSkin)
{
	Asura_Assert(s_xThis,"");
	return s_xThis->GetDefaultAnimDirForSkinMain(pszSkin);
}

const char* Tools_Animation_Files::GetDefaultAnimDirForSkinMain(const char* pszSkin)
{
	if(!pszSkin) return 0;
	
	if(!m_xSkinList.size()) UpdateSkinList();

	for(LIF<ToolsSkinFile*> xSkinLIF(&m_xSkinList);!xSkinLIF.done();xSkinLIF.next())
	{
		if(!stricmp(pszSkin,xSkinLIF()->m_xName.GetString()))
		{
			return xSkinLIF()->m_xDefaultAnimDir.GetString();
		}
	}
	return 0;
}


bool Tools_Animation_Files::ChooseSkin(const Asura_Hierarchy_Skin** ppxOutputSkin)
{
	Asura_Assert(ppxOutputSkin,"");
	Asura_Assert(s_xThis, "");

	return s_xThis->ChooseSkinMain(ppxOutputSkin, NULL, NULL, false);
}

bool Tools_Animation_Files::ChooseSkin_AllowNone(const Asura_Hierarchy_Skin** ppxOutputSkin)
{
	Asura_Assert(ppxOutputSkin,"");
	Asura_Assert(s_xThis, "");

	return s_xThis->ChooseSkinMain(ppxOutputSkin, NULL, NULL, true);
}


bool Tools_Animation_Files::ChooseAnimation(const Asura_Hierarchy_Skin* pxSkin,const Asura_Hierarchy_Anim** ppxOutputAnim,bool bOutputIsAlsoDefault)
{
	//Asura_Assert(pxSkin,"");
	Asura_Assert(ppxOutputAnim,"");

	const Asura_Hierarchy_Anim* pxDefault = bOutputIsAlsoDefault ? *ppxOutputAnim : 0;
	
	Asura_Hierarchy_Anim* pxAnim = ChooseAnimation(pxSkin ? pxSkin->Name.GetString() : 0,0,0,0,pxDefault);
	if(!pxAnim)
	{
		return false;
	}

	
	*ppxOutputAnim = pxAnim;
	return true;

}

bool Tools_Animation_Files::BrowseForAsuraFile(const char* pszTitle,Asura_String& xFilename,bool bAnim, const char* szDefaultPath /*=0*/)
{
	Asura_Assert(pszTitle,"");

	static Asura_String s_xDefaultSkinPath = "Characters";
	static Asura_String s_xDefaultAnimPath = "Characters";

	Asura_String* pxDefaultPath = bAnim ? &s_xDefaultAnimPath : &s_xDefaultSkinPath;

	if(!szDefaultPath || !*szDefaultPath) szDefaultPath = pxDefaultPath->GetString();

	Tools_FileSystem_DialogInfo	xFileSelector;

	xFileSelector.DefaultExtension		= ".ASR";
	xFileSelector.DefaultFilename		= "";
	xFileSelector.DefaultPath			= szDefaultPath;
	xFileSelector.FileFilter			= "Asura Files\0*.ASR;\0";
	xFileSelector.WindowTitle			= pszTitle;
	xFileSelector.AllowMultipleSelect	= false;

	xFilename = Tools_FileSystem::GetLoadFilename( xFileSelector );

	if(xFilename.IsNull()) return false;

	//remember the most recent directory selected		
	xFilename.SplitFilename(pxDefaultPath,0,0);
	return true;
}

void Tools_Animation_Files::LoadAnimationIniFile()
{
	if(m_bIniFileLoaded) return;
	m_bIniFileLoaded = true;

	//load the ini file
	const char* pszFilename = "ToolsIniFiles\\AnimationCategories.ini";
	Asura_Logfile_Retail((TOOLS_DEBUGGING_LOGFILE,"Loading %s",pszFilename));

	Tools_IniFile xIni(pszFilename);
	if(!xIni.LoadIniFile())
	{
		Asura_Logfile_Retail((TOOLS_DEBUGGING_LOGFILE,"Failed to load %s",pszFilename));
		return;
	}
	


	//find the definition of anim categories
	if(xIni.GetIniTextSection("Anim Categories",true))
	{
		//get a list of anim category names
		List<char*> xAnimSectionList = xIni.GetSectionNameList();
		for(LIF<char*> xAnimSectionLIF(&xAnimSectionList);!xAnimSectionLIF.done();xAnimSectionLIF.next())
		{
			xIni.GetIniTextSection(xAnimSectionLIF());
						
			//allocate the anim category
			ToolsAnimCategory* pxAnimCat = new ToolsAnimCategory;
			pxAnimCat->m_xName = xAnimSectionLIF();

			//read in the list of animations
			do
			{
				Asura_String xAnim;
				if(xIni.GetString(xAnim))
				{
					pxAnimCat->m_xAnimList.add_entry(xAnim);
				}
			}while(xIni.GetNextIniTextLine());

			m_xAnimCatList.add_entry(pxAnimCat);

			
		}

		//step out of the anim categories
		xIni.GetParentSection();

	}
	else
	{
		Asura_Logfile_Retail((TOOLS_DEBUGGING_LOGFILE,"Failed to find [Anim Categories] section"));
	}
	
	//find the definition of skin categories
	if(xIni.GetIniTextSection("Skin Categories",true))
	{
		//get a list of skin category names
		List<char*> xSkinSectionList = xIni.GetSectionNameList();
		for(LIF<char*> xSkinSectionLIF(&xSkinSectionList);!xSkinSectionLIF.done();xSkinSectionLIF.next())
		{
			xIni.GetIniTextSection(xSkinSectionLIF(),true);
			
			//allocate the skin category
			ToolsSkinCategory* pxSkinCat = new ToolsSkinCategory;
			pxSkinCat->m_xName = xSkinSectionLIF();

			//find the section with the list of skins
			if(xIni.GetIniTextSection("Skins"))
			{
				do
				{
					Asura_String xSkin;
					if(xIni.GetString(xSkin))
					{
						pxSkinCat->m_xSkinList.add_entry(xSkin);
					}
				}while(xIni.GetNextIniTextLine());
			}
			else
			{
				Asura_Logfile_Retail((TOOLS_DEBUGGING_LOGFILE,"Failed to find [Skins] section for %s",xSkinSectionLIF()));
			}

			//find the section with the list of anims
			if(xIni.GetIniTextSection("Anim Categories"))
			{
				do
				{
					Asura_String xAnim;
					if(xIni.GetString(xAnim))
					{
						//find the anim category
						ToolsAnimCategory* pxAnimCat = 0;
						for(LIF<ToolsAnimCategory*> xAnimCatLIF(&m_xAnimCatList);!xAnimCatLIF.done();xAnimCatLIF.next())
						{
							if(!stricmp(xAnimCatLIF()->m_xName.GetString(),xAnim.GetString()))
							{
								pxAnimCat = xAnimCatLIF();
								break;
							}
						}
						if(pxAnimCat)
						{
							pxSkinCat->m_xAnimCatList.add_entry(pxAnimCat);	
						}
						else
						{
							Asura_Logfile_Retail((TOOLS_DEBUGGING_LOGFILE,"Anim categoty %s doesn't exist",xAnim.GetString()));
						}
					}
				}while(xIni.GetNextIniTextLine());
			}
			else
			{
				Asura_Logfile_Retail((TOOLS_DEBUGGING_LOGFILE,"Failed to find [Anim Categories] section for %s",xSkinSectionLIF()));
			}

			//step out of this category
			xIni.GetParentSection();
			m_xSkinCatList.add_entry(pxSkinCat);
		}
		
		//step out of the skin categories
		xIni.GetParentSection();
	}
	else
	{
		Asura_Logfile_Retail((TOOLS_DEBUGGING_LOGFILE,"Failed to find [Skin Categories] section"));
	}

}

Asura_Hierarchy_Skin* Tools_Animation_Files::ChooseSkin(Asura_String* pxSkinName,Asura_String* pxSkinFilename)
{
	Asura_Assert(s_xThis,"");
	const Asura_Hierarchy_Skin* pxSkin = 0;
	bool bOkay = s_xThis->ChooseSkinMain(&pxSkin, pxSkinName,pxSkinFilename,false);

	return bOkay ? const_cast<Asura_Hierarchy_Skin*>(pxSkin) : 0;
}

bool Tools_Animation_Files::ChooseSkinMain(const Asura_Hierarchy_Skin** ppxSkin, Asura_String* pxSkinName,Asura_String* pxSkinFilename, bool bAllowNone)
{
	LoadAnimationIniFile();


	
	//set up initial state
	m_pxCurrentSkinCat = 0;
	if(m_xSkinCatList.size())
	{
		m_eSkinSelectionState = SSS_SELECT_CATEGORY;
	}	
	else
	{
		m_eSkinSelectionState = SSS_DISPLAY_ALL;
	}

	Asura_String xSkinName,xSkinFilename;
	
	while(m_eSkinSelectionState != SSS_CANCEL && m_eSkinSelectionState != SSS_OKAY)
	{
		switch(m_eSkinSelectionState)
		{
			case SSS_SELECT_CATEGORY :
				ChooseSkinCategory();
				break;

			case SSS_SELECT_FROM_CATEGORY :
				ChooseSkinFromCategory(&xSkinName,&xSkinFilename);
				break;

			case SSS_DISPLAY_ALL :
				ChooseSkinDisplayAll(&xSkinName,&xSkinFilename,pxSkinName,bAllowNone);
				break;

			case SSS_CUTSCENE_SPECIFIC :
				ChooseSkinCutsceneSpecific(&xSkinName,&xSkinFilename,bAllowNone);
				break;

			case SSS_BROWSE_DISK :
				ChooseSkinBrowseDisk(&xSkinName,&xSkinFilename,pxSkinName,pxSkinFilename);
				break;


			default :
				Asura_Assert(0,"Unexpected state");
				m_eSkinSelectionState = SSS_CANCEL;
		}
	}

	Asura_Hierarchy_Skin* pxSkin = 0;
	bool bOkay = (m_eSkinSelectionState == SSS_OKAY);
	if(bOkay)
	{
		//attempt to find the skin
		if(!(bAllowNone && xSkinName.IsBlank()))
		{
			pxSkin = Asura_Animation_System::GetSkin(xSkinName.GetString());
			if(!pxSkin)
			{
				//make sure the file is a skin file
				Tools_AsuraFileContents xFileContents;
				if(!xFileContents.LoadAsuraFile(xSkinFilename.GetString()))
				{
					Tools_Timed_Message_System::AddMessage(0xff0000,"Failed to load %s",xSkinFilename.GetString());
					return 0;
				}
				if(!xFileContents.GetSkinList()->size())
				{
					Tools_Timed_Message_System::AddMessage(0xff0000,"%s isn't a skin file",xSkinFilename.GetString());
					return 0;
				}
				
				//try loading the skin
				LoadAsuraFile(xSkinFilename.GetString());
				Asura_Texture_Management::LoadTextures();

				pxSkin= Asura_Animation_System::GetSkin(xSkinName.GetString());
				if(!pxSkin)
				{
					Tools_Timed_Message_System::AddMessage(0xff0000,"Failed to get skin '%s'",xSkinName.GetString());
					Tools_Timed_Message_System::AddMessage(0xff0000,"Skin name doesn't match filename?");
				}
				else
				{
					LoadSoundsForSkin(pxSkin);
				}
			}
			if(!pxSkin)
			{
				bOkay = false;
			}

		}
	}
	if(bOkay)
	{
		if(ppxSkin) *ppxSkin = pxSkin;
		if(pxSkin)
		{
			if(pxSkinName) *pxSkinName = xSkinName;
			if(pxSkinFilename) *pxSkinFilename = xSkinFilename;
		}
		else
		{
			if(pxSkinName) pxSkinName->MakeNull();
			if(pxSkinFilename) pxSkinFilename->MakeNull();
		}
	}

	return bOkay;

}

void Tools_Animation_Files::ChooseSkinCategory()
{
	ListSelection_Overlay xOverlay("Choose Skin Category");
	for(LIF<ToolsSkinCategory*> xSkinCatLIF(&m_xSkinCatList);!xSkinCatLIF.done();xSkinCatLIF.next())
	{
		xOverlay.AddItem(xSkinCatLIF()->m_xName.GetString(),(int)xSkinCatLIF());
	}

	//add display all option
	xOverlay.AddItem(g_pszDisplayAllString,0);
	if(m_xCutsceneDirectory.GetLength())
	{
		xOverlay.AddItem(g_pszCutsceneSpecificString,0);
	}
	xOverlay.AddItem(g_pszBrowseDiskString,0);

	if(!xOverlay.DoModal())
	{
		m_eSkinSelectionState = SSS_CANCEL;
		return;
	}

	ToolsSkinCategory* pxSkinCat = (ToolsSkinCategory*)xOverlay.GetSelectionData();
	if(pxSkinCat)
	{
		m_pxCurrentSkinCat = pxSkinCat;
		m_eSkinSelectionState = SSS_SELECT_FROM_CATEGORY;

	}
	else
	{
		UpdateSkinSelectionState(xOverlay.GetSelectionString());
	}
}

void Tools_Animation_Files::ChooseSkinFromCategory(Asura_String* pxSkinName,Asura_String* pxSkinFilename)
{
	Asura_Assert(m_pxCurrentSkinCat,"");

	ListSelection_Overlay xOverlay("Select Skin");
	for(LIF<Asura_String> xSkinLIF(&m_pxCurrentSkinCat->m_xSkinList);!xSkinLIF.done();xSkinLIF.next())
	{
		xOverlay.AddItem(xSkinLIF().GetString());
	}

	//add select category option
	xOverlay.AddItem(g_pszChooseCategoryString,0);
	xOverlay.AddItem(g_pszDisplayAllString,0);
	if(m_xCutsceneDirectory.GetLength())
	{
		xOverlay.AddItem(g_pszCutsceneSpecificString,0);
	}
	xOverlay.AddItem(g_pszBrowseDiskString,0);

	if(!xOverlay.DoModal())
	{
		m_eSkinSelectionState = SSS_CANCEL;
		return;
	}

	Asura_String xTemp = xOverlay.GetSelectionString();

	//check for the select category , and display all options
	if(xTemp.GetString()[0] == '*')
	{
		UpdateSkinSelectionState(xTemp.GetString());
		return;
	}

	Asura_String xFilename;
	if(!FindFile(xTemp.GetString(),&xFilename))
	{
		Asura_Assert(0,"Failed to find skin file");
		return;
	}

	//fill in the output strings
	if(pxSkinName) *pxSkinName = xTemp;
	if(pxSkinFilename) *pxSkinFilename = xFilename;

	m_eSkinSelectionState = SSS_OKAY;

}

void Tools_Animation_Files::ChooseSkinDisplayAll(Asura_String* pxSkinName,Asura_String* pxSkinFilename, const Asura_String* pxInitialSkin, bool bAllowNone)
{
	const List<ToolsSkinFile*>* pxSkinList = GetSkinList();

	ListSelection_Overlay xOverlay("Select Skin");
	for(CLIF<ToolsSkinFile*> xSkinLIF(pxSkinList);!xSkinLIF.done();xSkinLIF.next())
	{
		bool bInitialSelection = false;
		if(pxInitialSkin && !pxInitialSkin->IsBlank() && !pxInitialSkin->Compare(xSkinLIF()->m_xName))
		{
			bInitialSelection = true;
		}
		
		xOverlay.AddItem(xSkinLIF()->m_xName.GetString(),(int)xSkinLIF(),bInitialSelection);
	}


	//add select category option
	xOverlay.AddItem(g_pszChooseCategoryString,0);
	if(m_xCutsceneDirectory.GetLength())
	{
		xOverlay.AddItem(g_pszCutsceneSpecificString,0);
	}
	xOverlay.AddItem(g_pszBrowseDiskString,0);
	if(bAllowNone)
	{
		xOverlay.AddItem(g_pszNone,0);
	}


	if(!xOverlay.DoModal())
	{
		m_eSkinSelectionState = SSS_CANCEL;
		return;
	}

	ToolsSkinFile* pxSkin = (ToolsSkinFile*) xOverlay.GetSelectionData();
	if(pxSkin)
	{
		if(pxSkinName) *pxSkinName = pxSkin->m_xName;
		if(pxSkinFilename) * pxSkinFilename = pxSkin->m_xFilename;
		m_eSkinSelectionState = SSS_OKAY;
	}
	else if(!stricmp(xOverlay.GetSelectionString(),g_pszNone))
	{
		if(pxSkinName) pxSkinName->MakeNull();
		if(pxSkinFilename) pxSkinFilename->MakeNull();
		m_eSkinSelectionState = SSS_OKAY;
	}
	else
	{
		UpdateSkinSelectionState(xOverlay.GetSelectionString());
	}
	
}

void Tools_Animation_Files::ChooseSkinCutsceneSpecific(Asura_String* pxSkinName,Asura_String* pxSkinFilename, bool bAllowNone)
{
	List<Asura_String> xSkinList;
	List<Asura_String> xFilenameList;

	//list the anims in the cutscene dir
	char szDir[MAX_PATH];
	sprintf(szDir,"Cutscenes\\%s\\Props",m_xCutsceneDirectory.GetString());
	ListAnimations(szDir,&xSkinList,&xFilenameList,false);
	
	Asura_Assert(xSkinList.size() == xFilenameList.size(),"");

	ListSelection_Overlay xOverlay("Select Skin");

	//add all the anims to the overlay
	LIF<Asura_String> xSkinLIF(&xSkinList);
	LIF<Asura_String> xFilenameLIF(&xFilenameList);
	for(;!xSkinLIF.done();xSkinLIF.next(),xFilenameLIF.next())
	{
		xOverlay.AddItem(xSkinLIF().GetString(),(int)xFilenameLIF().GetString());
	}

	//add select category option
	xOverlay.AddItem(g_pszChooseCategoryString,0);
	xOverlay.AddItem(g_pszDisplayAllString,0);
	xOverlay.AddItem(g_pszBrowseDiskString,0);
	if(bAllowNone)
	{
		xOverlay.AddItem(g_pszNone,0);
	}


	if(!xOverlay.DoModal())
	{
		m_eSkinSelectionState = SSS_CANCEL;
		return;
	}

	Asura_String xTemp = xOverlay.GetSelectionString();

	//check for the select category , and display all options
	if(xTemp.GetString()[0] == '*')
	{
		if(!stricmp(xOverlay.GetSelectionString(),g_pszNone))
		{
			if(pxSkinName) pxSkinName->MakeNull();
			if(pxSkinFilename) pxSkinFilename->MakeNull();
			m_eSkinSelectionState = SSS_OKAY;
		}
		else
		{
			UpdateSkinSelectionState(xTemp.GetString());
		}
		return;
	}

	if(pxSkinName) *pxSkinName = xOverlay.GetSelectionString();
	if(pxSkinFilename) *pxSkinFilename = (char*)xOverlay.GetSelectionData();
}

void Tools_Animation_Files::ChooseSkinBrowseDisk(Asura_String* pxSkinName,Asura_String* pxSkinFilename, const Asura_String* pxInitialName, const Asura_String* pxInitialFile)
{
	//get the default path from the initial skin or filename , if available
	Asura_String xDefaultPath;
	if(pxInitialFile && !pxInitialFile->IsBlank())
	{
		pxInitialFile->SplitFilename(&xDefaultPath,0,0);
	}
	else if(pxInitialName && !pxInitialName->IsBlank())
	{
		Asura_String xInitialFile;
		if(FindFile(pxInitialName->GetString(),&xInitialFile))
		{
			xInitialFile.SplitFilename(&xDefaultPath,0,0);
		}
	}


	Asura_String xName,xFilename,xFilename2;
	if(!BrowseForAsuraFile("Select Skin",xFilename,false,xDefaultPath.GetString()))
	{
		m_eSkinSelectionState = SSS_CANCEL;
		return;
	}
	//determine skin name from filename
	xFilename.SplitFilename(0,&xName,0);

	if(!FindFile(xName.GetString(),&xFilename2))
	{
		m_eSkinSelectionState = SSS_CANCEL;
		Tools_Timed_Message_System::AddMessage(0xff0000,"File must be in the characters or cutscenes directories");
		return;
	}

	//strlwr(xFilename.GetString());
	//strlwr(xFilename2.GetString());
	//if(!xFilename.Contains(xFilename2.GetString()))
	//{
	//	Tools_Timed_Message_System::AddMessage(0xffff00,"Using %s instead.",xFilename2.GetString());
	//}

	if(pxSkinName) *pxSkinName = xName;
	if(pxSkinFilename) *pxSkinFilename = Asura_FileAccess::PlatformRemoveNetworkOrExecutableDir(xFilename.GetString());

	m_eSkinSelectionState = SSS_OKAY;
}

void Tools_Animation_Files::UpdateSkinSelectionState(const char* pszOption)
{
	if(!stricmp(pszOption,g_pszChooseCategoryString))
	{
		m_eSkinSelectionState = SSS_SELECT_CATEGORY;
	
	}
	else if(!stricmp(pszOption,g_pszDisplayAllString))
	{
		m_eSkinSelectionState = SSS_DISPLAY_ALL;
	}
	else if(!stricmp(pszOption,g_pszCutsceneSpecificString))
	{
		m_eSkinSelectionState = SSS_CUTSCENE_SPECIFIC;
	}
	else if(!stricmp(pszOption,g_pszBrowseDiskString))
	{
		m_eSkinSelectionState = SSS_BROWSE_DISK;
	}
}

Asura_Hierarchy_Anim* Tools_Animation_Files::ChooseAnimation(const char* pszSkinName,Asura_String* pxDir,Asura_String* pxAnimName,Asura_String* pxAnimFilename,const Asura_Hierarchy_Anim* pxDefaultAnim,bool bAllowNone,bool* pbWasCancelled)
{
	Asura_Assert(s_xThis,"");
	const Asura_Hierarchy_Anim* pxAnim = s_xThis->ChooseAnimationMain(pszSkinName,pxDir,pxAnimName,pxAnimFilename,pxDefaultAnim,bAllowNone,pbWasCancelled);

	//Urgh , constness in a mess now , going to have to cast it away. RAR 13/12/2005
	return const_cast<Asura_Hierarchy_Anim*>(pxAnim);
}

const Asura_Hierarchy_Anim* Tools_Animation_Files::ChooseAnimationMain(const char* pszSkinName,Asura_String* pxDir,Asura_String* pxAnimName,Asura_String* pxAnimFilename,const Asura_Hierarchy_Anim* pxDefaultAnim,bool bAllowNone,bool* pbWasCancelled)
{
	if(!m_bIniFileLoaded) LoadAnimationIniFile();

	Asura_String xDir = pxDir ? *pxDir : "";

	//Set a default directory string if needed.
	//Also make sure it doesn't start as *None*
	if(xDir.IsBlank() || !stricmp(xDir.GetString(),g_pszNone))
	{
		//pick a default dir/anim category
		if(m_xAnimCatList.size())
		{
			xDir = g_pszChooseCategoryString;
		}
		else
		{
			xDir = GetDefaultAnimDirForSkin(pszSkinName);
			if(xDir.IsBlank())
			{
				xDir = g_pszChooseDirectoryString;
			}
		}
	}
	Asura_String xAnimName;
	Asura_String xAnimFilename;

	if(pbWasCancelled) *pbWasCancelled = true;

	do
	{
		if(!stricmp(xDir.GetString(),g_pszNone))
		{
			break;
		}
		else if(!stricmp(xDir.GetString(),g_pszChooseCategoryString))
		{
			if(!ChooseAnimCategory(pszSkinName,&xDir)) return pxDefaultAnim;
		}
		else if(!stricmp(xDir.GetString(),g_pszChooseDirectoryString))
		{
			if(!ChooseAnimDirectory(pszSkinName,&xDir,bAllowNone)) return pxDefaultAnim;
		}
		else if(!stricmp(xDir.GetString(),g_pszCutsceneSpecificString))
		{
			if(!ChooseAnimFromCutsceneDirectory(&xDir,&xAnimName,&xAnimFilename,pxDefaultAnim)) return pxDefaultAnim;
			if(!stricmp(xDir.GetString(),g_pszCutsceneSpecificString)) break;
		}
		else if(!stricmp(xDir.GetString(),g_pszBrowseDiskString))
		{
			xDir = "";
			if(!ChooseAnimBrowseDisk(&xAnimName,&xAnimFilename, pxDefaultAnim)) return pxDefaultAnim;
			//get the directory of the chosen anim.
			Asura_String xRelPath = Asura_FileAccess::PlatformRemoveNetworkOrExecutableDir(xAnimFilename.GetString());
			xRelPath.SplitFilename(&xDir,0,0);
			break;
		}
		else
		{
			if(xDir.GetString()[0]=='@')
			{
				if(!ChooseAnimFromCategory(&xDir,&xAnimName,&xAnimFilename,pxDefaultAnim,bAllowNone)) return pxDefaultAnim;
			}
			else
			{
				if(!ChooseAnimFromDirectory(&xDir,&xAnimName,&xAnimFilename,pxDefaultAnim,bAllowNone)) return pxDefaultAnim;
			}
			if(xDir.GetString()[0] != '*') break;
		}

	}while(true);
	
	if(!stricmp(xDir.GetString(),g_pszNone))
	{
		xAnimName.MakeNull();
		xAnimFilename.MakeNull();	
	}
	else
	{
		if(pxDir) *pxDir = xDir;
	}

	Asura_Hierarchy_Anim* pxAnim;
	if(bAllowNone && xAnimName.IsBlank())
	{
		//selected 'none'
		pxAnim = 0;
		if(pbWasCancelled) *pbWasCancelled = false;
	}
	else
	{
		//find/load the animation
		pxAnim = Asura_Animation_System::GetAnim(xAnimName.GetString());
		if(!pxAnim)
		{
			//make sure the file is an anim file
			Tools_AsuraFileContents xFileContents;
			if(!xFileContents.LoadAsuraFile(xAnimFilename.GetString()))
			{
				Tools_Timed_Message_System::AddMessage(0xff0000,"Failed to load %s",xAnimFilename.GetString());
				return pxDefaultAnim;
			}
			if(!xFileContents.GetAnimList()->size())
			{
				Tools_Timed_Message_System::AddMessage(0xff0000,"%s isn't an anim file",xAnimFilename.GetString());
				return pxDefaultAnim;
			}
			
			
			//Try loading the anim
			LoadAsuraFile(xAnimFilename.GetString());
			
			pxAnim = Asura_Animation_System::GetAnim(xAnimName.GetString());
			if(!pxAnim)
			{
				Tools_Timed_Message_System::AddMessage(0xff0000,"Failed to get anim '%s'",xAnimName.GetString());
				Tools_Timed_Message_System::AddMessage(0xff0000,"Anim name doesn't match filename?");
				return pxDefaultAnim;
			}
			else
			{
				LoadSoundsForAnim(pxAnim);
			}
		}
		if(pxAnim)
		{
			if(pbWasCancelled) *pbWasCancelled = false;
		}
	}

	if(pxAnim)
	{
		if(pxAnimName) *pxAnimName = xAnimName;
		if(pxAnimFilename) *pxAnimFilename = xAnimFilename;
	}

	return pxAnim;
}


bool Tools_Animation_Files::ChooseAnimCategory(const char* pszSkinName,Asura_String* pxDir)
{
	if(!pszSkinName) return false;
	Asura_Assert(pxDir,"");

	if(!m_xAnimCatList.size())
	{
		*pxDir = g_pszChooseDirectoryString;
		return true;
	}

	//get the anim categories
	List<const char*> xCatList;
	ListAnimCategoriesForSkin(pszSkinName,xCatList);

	ListSelection_Overlay xOverlay("Choose Anim Category");

	for(LIF<const char*> xCatLIF(&xCatList);!xCatLIF.done();xCatLIF.next())
	{
		xOverlay.AddItem(xCatLIF());
	}
	//add entry for choosing directory
	xOverlay.AddItem(g_pszBrowseDiskString);
	xOverlay.AddItem(g_pszChooseDirectoryString);
	if(m_xCutsceneDirectory.GetLength())
	{
		xOverlay.AddItem(g_pszCutsceneSpecificString,0);
	}


	if(!xOverlay.DoModal()) return false;

	Asura_String xTemp = xOverlay.GetSelectionString();

	if(xTemp.GetString()[0] != '*')
	{
		//tack an @ on the front to indicate that this is a category ,and not a proper dir
		*pxDir = "@";
		*pxDir += xTemp.GetString();
	}
	else
	{
		*pxDir = xTemp;
	}

	return true;
}

bool Tools_Animation_Files::ChooseAnimDirectory(const char* pszSkinName,Asura_String* pxDir,bool bAllowNone)
{
	Asura_Assert(pxDir,"");

	const List<ToolsAnimationDirectory*>* pxDirList = GetDirectoryList();

	ListSelection_Overlay xOverlay("Choose Anim Directory");

	//add all the dirs to the overlay
	for(CLIF<ToolsAnimationDirectory*> xDirLIF(pxDirList);!xDirLIF.done();xDirLIF.next())
	{
		xOverlay.AddItem(xDirLIF()->m_xName.GetString(),(int)xDirLIF());
	}

	//add entry for choose category
	if(m_xAnimCatList.size())
	{
		xOverlay.AddItem(g_pszChooseCategoryString,0);
	}

	if(m_xCutsceneDirectory.GetLength())
	{
		xOverlay.AddItem(g_pszCutsceneSpecificString,0);
	}
	xOverlay.AddItem(g_pszBrowseDiskString);
	if(bAllowNone)
	{
		xOverlay.AddItem(g_pszNone,0);
	}


	if(!xOverlay.DoModal()) return false;

	ToolsAnimationDirectory* pxAnimDir = (ToolsAnimationDirectory*) xOverlay.GetSelectionData();
	if(pxAnimDir)
	{
		*pxDir = pxAnimDir->m_xDirectory;
	}
	else
	{
		//string should be the special choose category string
		*pxDir = xOverlay.GetSelectionString();
	}

	return true;
}

bool Tools_Animation_Files::ChooseAnimFromCategory(Asura_String* pxDir,Asura_String* pxAnimName,Asura_String* pxAnimFilename,const Asura_Hierarchy_Anim* pxDefaultAnim,bool bAllowNone)
{
	Asura_Assert(pxDir,"");
	Asura_Assert(pxDir->GetString()[0] == '@',"");

	//find the anim cat
	ToolsAnimCategory* pxAnimCat = 0;
	for(LIF<ToolsAnimCategory*> xCatLIF(&m_xAnimCatList);!xCatLIF.done();xCatLIF.next())
	{
		if(!stricmp(xCatLIF()->m_xName.GetString(),&pxDir->GetString()[1]))
		{
			//found it
			pxAnimCat = xCatLIF();
			break;
		}
	}
	
	Asura_Assert(pxAnimCat,"");
		
	//add the animations from this category to an overlay
	ListSelection_Overlay xOverlay("Choose Animation");

	for(LIF<Asura_String> xAnimLIF(&pxAnimCat->m_xAnimList);!xAnimLIF.done();xAnimLIF.next())
	{
		bool bDefaultSelection = false;
		if(pxDefaultAnim && !pxDefaultAnim->Name.Compare(xAnimLIF()))
		{
			bDefaultSelection = true;
		}
		xOverlay.AddItem(xAnimLIF().GetString());
	}

	//add the choose dir and choose category options
	if(m_xAnimCatList.size())
	{
		xOverlay.AddItem(g_pszChooseCategoryString,0);
	}
	xOverlay.AddItem(g_pszChooseDirectoryString,0);
	if(m_xCutsceneDirectory.GetLength())
	{
		xOverlay.AddItem(g_pszCutsceneSpecificString,0);
	}
	xOverlay.AddItem(g_pszBrowseDiskString);
	if(bAllowNone)
	{
		xOverlay.AddItem(g_pszNone,(int)g_pszNone,!pxDefaultAnim);
	}



	do
	{
		if(!xOverlay.DoModal()) return false;

		Asura_String xName = xOverlay.GetSelectionString();
		if(xName.GetString()[0]=='*')
		{
			if(!xName.Compare(g_pszNone))
			{
				if(pxAnimName) pxAnimName->MakeNull();
				if(pxAnimFilename) pxAnimFilename->MakeNull();
			}
			else
			{
				*pxDir = xName;//special select category/directory option
			}
			return true;
		}

		Asura_String xFilename;
		if(!FindFile(xName.GetString(),&xFilename))
		{
			Asura_Assert(0,"Failed to find anim file");
			continue;	
		}

		if(pxAnimName) *pxAnimName = xName;
		if(pxAnimFilename) *pxAnimFilename = xFilename;
		break;
	}while(true);
	
	return true;
}


void Tools_Animation_Files::GetChooseAnimListFromDirectory(const char* pszDir,List<Asura_String>* pxOutputAnimNameList,List<Asura_String>* pxOutputFilenameList)
{
	if(!pszDir) return;

	//get a list of animations directly in this directory
	ListAnimations(pszDir,pxOutputAnimNameList,pxOutputFilenameList);

	if(!stricmp("anims",Tools_FileSystem::StripPath(pszDir)))
	{
		//If dealing with an 'anims' directory , check for a AnimDirs.ini file in the parent
		//directory , listing additional directories to check.
		char szDir[MAX_PATH];
		strcpy(szDir,pszDir);
		int iLength = strlen(szDir);

		strcpy(&szDir[iLength-5],"AnimDirs.ini");
		Tools_IniFile xIni(szDir);
		if(xIni.LoadIniFile())
		{
			while(xIni.GetNextIniTextLine())
			{
				ListAnimations(xIni.LineBuffer,pxOutputAnimNameList,pxOutputFilenameList);
			}
		}
		
	}
}

bool Tools_Animation_Files::ChooseAnimFromDirectory(Asura_String* pxDir,Asura_String* pxAnimName,Asura_String* pxAnimFilename,const Asura_Hierarchy_Anim* pxDefaultAnim,bool bAllowNone)
{
	Asura_Assert(pxDir,"");

	//get a list of animations for this directory
	List<Asura_String> xNameList;
	List<Asura_String> xFilenameList;
	GetChooseAnimListFromDirectory(pxDir->GetString(),&xNameList,&xFilenameList);
	Asura_Assert(xNameList.size()==xFilenameList.size(),"");


	ListSelection_Overlay xOverlay("Choose Animation");

	//add to the overlay
	LIF<Asura_String> xNameLIF(&xNameList);
	LIF<Asura_String> xFilenameLIF(&xFilenameList);
	
	for(;!xNameLIF.done();xNameLIF.next(),xFilenameLIF.next())
	{
		bool bDefaultSelection = false;
		if(pxDefaultAnim && !pxDefaultAnim->Name.Compare(xNameLIF()))
		{
			bDefaultSelection = true;
		}
		
		xOverlay.AddItem(xNameLIF().GetString(),(int)xFilenameLIF().GetString(),bDefaultSelection);
	} 

	//add the choose dir and choose category options
	if(m_xAnimCatList.size())
	{
		xOverlay.AddItem(g_pszChooseCategoryString,0);
	}
	xOverlay.AddItem(g_pszChooseDirectoryString,0);
	if(m_xCutsceneDirectory.GetLength())
	{
		xOverlay.AddItem(g_pszCutsceneSpecificString,0);
	}
	xOverlay.AddItem(g_pszBrowseDiskString);
	if(bAllowNone)
	{
		xOverlay.AddItem(g_pszNone,(int)g_pszNone,!pxDefaultAnim);
	}


	if(!xOverlay.DoModal()) return false;

	const char* pszFilename = (const char*) xOverlay.GetSelectionData();
	if(pszFilename)
	{
		if(pszFilename == g_pszNone)
		{
			if(pxAnimName) pxAnimName->MakeNull();
			if(pxAnimFilename) pxAnimFilename->MakeNull();
		}
		else
		{
			if(pxAnimName) *pxAnimName = xOverlay.GetSelectionString();
			if(pxAnimFilename) *pxAnimFilename = pszFilename;
		}
	}
	else
	{
		//chose select category/select directory
		*pxDir = xOverlay.GetSelectionString();
	}

	return true;
}

bool Tools_Animation_Files::ChooseAnimFromCutsceneDirectory(Asura_String* pxDir,Asura_String* pxAnimName,Asura_String* pxAnimFilename,const Asura_Hierarchy_Anim* pxDefaultAnim)
{
	Asura_Assert(pxDir,"");
	ListSelection_Overlay xOverlay("Choose Animation");

	List<Asura_String> xAnimList;
	List<Asura_String> xFilenameList;

	//list the anims in the cutscene dir
	char szDir[MAX_PATH];
	sprintf(szDir,"Cutscenes\\%s",m_xCutsceneDirectory.GetString());
	ListAnimations(szDir,&xAnimList,&xFilenameList,false);
	
	Asura_Assert(xAnimList.size() == xFilenameList.size(),"");

	//add all the anims to the overlay
	LIF<Asura_String> xAnimLIF(&xAnimList);
	LIF<Asura_String> xFilenameLIF(&xFilenameList);
	for(;!xAnimLIF.done();xAnimLIF.next(),xFilenameLIF.next())
	{
		bool bDefaultSelection = false;
		if(pxDefaultAnim && !pxDefaultAnim->Name.Compare(xAnimLIF()))
		{
			bDefaultSelection = true;
		}
		
		xOverlay.AddItem(xAnimLIF().GetString(),(int)xFilenameLIF().GetString());
	}
	xOverlay.AddItem(g_pszBrowseDiskString);

	//add the choose dir and choose category options
	if(m_xAnimCatList.size())
	{
		xOverlay.AddItem(g_pszChooseCategoryString,0);
	}
	xOverlay.AddItem(g_pszChooseDirectoryString,0);
		

	//show the overlay
	if(!xOverlay.DoModal()) return false;

	if(xOverlay.GetSelectionData())
	{

		if(pxAnimName) *pxAnimName = xOverlay.GetSelectionString();
		if(pxAnimFilename) *pxAnimFilename = (char*)xOverlay.GetSelectionData();
	}
	else
	{
		//chose select category/select directory
		*pxDir = xOverlay.GetSelectionString();
	}
	return true;
}

bool Tools_Animation_Files::ChooseAnimBrowseDisk(Asura_String* pxAnimName,Asura_String* pxAnimFilename, const Asura_Hierarchy_Anim* pxInitialAnim)
{
	Asura_String xDefaultPath;
	if(pxInitialAnim)
	{
		Asura_String xInitialFile;
		if(FindFile(pxInitialAnim->Name.GetString(),&xInitialFile))
		{
			xInitialFile.SplitFilename(&xDefaultPath,0,0);
		}
	}

	Asura_String xName,xFilename,xFilename2;
	if(!BrowseForAsuraFile("Select Anim",xFilename,true, xDefaultPath.GetString()))
	{
		return false;
	}
	//determine anim name from filename
	xFilename.SplitFilename(0,&xName,0);

	if(!FindFile(xName.GetString(),&xFilename2))
	{
		Tools_Timed_Message_System::AddMessage(0xff0000,"File must be in the characters or cutscenes directories");
		return false;
	}

	//strlwr(xFilename.GetString());
	//strlwr(xFilename2.GetString());
	//if(!xFilename.Contains(xFilename2.GetString()))
	//{
	//	Tools_Timed_Message_System::AddMessage(0xffff00,"Using %s instead.",xFilename2.GetString());
	//}


	if(pxAnimName) *pxAnimName = xName;
	if(pxAnimFilename) *pxAnimFilename = Asura_FileAccess::PlatformRemoveNetworkOrExecutableDir(xFilename.GetString());

	return true;
}


void Tools_Animation_Files::ListAnimCategoriesForSkin(const char* pszSkinName,List<const char*>& xCatList)
{
	while(xCatList.size()) xCatList.delete_first_entry();

	//search for the skin category using this skin
	for(LIF<ToolsSkinCategory*> xSkinCatLIF(&m_xSkinCatList);!xSkinCatLIF.done();xSkinCatLIF.next())	
	{
		ToolsSkinCategory* pxSkinCat = xSkinCatLIF();
		bool bSkinInThisCat = false;
		for(LIF<Asura_String> xSkinLIF(&pxSkinCat->m_xSkinList);!xSkinLIF.done();xSkinLIF.next())
		{
			if(!stricmp(xSkinLIF().GetString(),pszSkinName))
			{
				bSkinInThisCat = true;
				break;
			}
		}
		if(bSkinInThisCat)
		{
			//add the anim cats for this skin cat
			for(LIF<ToolsAnimCategory*> xAnimLIF(&pxSkinCat->m_xAnimCatList);!xAnimLIF.done();xAnimLIF.next())
			{
				bool bAdd = true;
				//make sure this hasn't already been added
				for(LIF<const char*> xCatLIF(&xCatList);!xCatLIF.done();xCatLIF.next())
				{
					if(!stricmp(xCatLIF(),xAnimLIF()->m_xName.GetString()))
					{
						bAdd = false;
						break;
					}
				}
				if(bAdd)
				{
					xCatList.add_entry(xAnimLIF()->m_xName.GetString());
				}
			}
		}
	}

	//if no catgories were found for this skin , show all of them
	if(!xCatList.size())
	{
		for(LIF<ToolsAnimCategory*> xAnimCatLIF(&m_xAnimCatList);!xAnimCatLIF.done();xAnimCatLIF.next())
		{
			xCatList.add_entry(xAnimCatLIF()->m_xName.GetString());
		}
	}
}


bool Tools_Animation_Files::FindFile(const char* pszName,Asura_String* pOutputFilename)
{
	Asura_Assert(s_xThis,"");
	return s_xThis->FindFileMain(pszName,pOutputFilename);
}

bool Tools_Animation_Files::FindFileMain(const char* pszName,Asura_String* pOutputFilename)
{
	Asura_Assert(pszName,"");
	Asura_Assert(pOutputFilename,"");

	bool bSuccess = false;
	
	
	for(int i=0;i<GetNumberOfRootDirectories() && !bSuccess;i++)
	{
		bSuccess = Tools_SaveAsura::FindAnimationRecursive(GetDirectory(i),pszName,*pOutputFilename);
	}
	if(!bSuccess) bSuccess = Tools_SaveAsura::FindAnimationRecursive("Cutscenes",pszName,*pOutputFilename);
	if(!bSuccess) bSuccess = Tools_SaveAsura::FindAnimationRecursive("Objects",pszName,*pOutputFilename);


	#ifdef ASURA_PC_ALLOW_LOAD_FROM_NETWORK
	//try looking for the file across the network as well
	char szDir[MAX_PATH];
	for(int i=0;i<GetNumberOfRootDirectories() && !bSuccess;i++)
	{
		Asura_PrintfToBfr(szDir,"%s\\%s",MAX_PATH,true,Asura_FileAccess::PlatformGetNetworkPath(),GetDirectory(i));
		bSuccess = Tools_SaveAsura::FindAnimationRecursive(szDir,pszName,*pOutputFilename);
	}
	if(!bSuccess)
	{
		Asura_PrintfToBfr(szDir,"%s\\%s",MAX_PATH,true,Asura_FileAccess::PlatformGetNetworkPath(),"Cutscenes");
		bSuccess = Tools_SaveAsura::FindAnimationRecursive(szDir,pszName,*pOutputFilename);
	}
	if(!bSuccess)
	{
		Asura_PrintfToBfr(szDir,"%s\\%s",MAX_PATH,true,Asura_FileAccess::PlatformGetNetworkPath(),"Objects");
		bSuccess = Tools_SaveAsura::FindAnimationRecursive(szDir,pszName,*pOutputFilename);
	}
	#endif
	
	
	return bSuccess;
}



void Tools_Animation_Files::SetCutsceneDirectory(const char* pszDir)
{
	Asura_Assert(s_xThis,"");
	if(pszDir)
	{
		s_xThis->m_xCutsceneDirectory = pszDir;
	}
	else
	{
		s_xThis->m_xCutsceneDirectory.MakeNull();
	}
}

int Tools_Animation_Files::GetNumberOfRootDirectories()
{
	return 2;
}

const char* Tools_Animation_Files::GetDirectory(int iIndex)
{
	switch(iIndex)
	{
		case 0:
			return "Characters";
		case 1:
			return "Vehicles";

	}
	Asura_Assert(0,"Invalid Index");
	return "Characters";
}



bool Tools_Animation_Files::IsSkinMeshLoaded(const Asura_Hierarchy_Skin* pxSkin)
{
	Asura_Assert(pxSkin,"");

	if(pxSkin->IsSkinnedModel())
	{
		Asura_Mesh* pxMesh = Asura_Mesh_System::GetMesh(pxSkin->Name.GetString());
		return pxMesh != 0;
	}
	else
	{
		//need to check each hierarchy section... assume the meshes are loaded if they
		//exist for any section
		for(int i=0;i<pxSkin->NumberOfBones;i++)
		{
			if(pxSkin->SectionData[i].Mesh)
			{
				return true;
			}
		}
		//no meshes found.... check to see if any were expected
		if(!Asura_Direct3D_ObjectHierarchyRenderer::DoesSkinHaveAnyTriangles(pxSkin))
		{	
			//no mesh needed
			return true;
		}

	}
	return false;
}

#if 0
static bool ChunkLoadFilter_MeshesOnly(enum ASURA_CHUNKID eChunkID)
{
	switch(eChunkID)
	{
		case ASURA_CHUNK_MESH_VERTICES:
		case ASURA_CHUNK_MESH_POLYGONS:
		case ASURA_CHUNK_MESH_NORMALS:
		case ASURA_CHUNK_MESH_TEXTUREINFO:
		case ASURA_CHUNK_MESH_SMOOTHINGGROUP:
		case ASURA_CHUNK_MESH_POLYGONFLAGS:
		case ASURA_CHUNK_MESH_LOD:
		case ASURA_CHUNK_MESH_MISC:
		case ASURA_CHUNK_TEXTURENAMES:
		case ASURA_CHUNK_TEXTUREFLAGS:
		case ASURA_CHUNK_MATERIAL :
		case ASURA_CHUNK_MATERIALNAMES :
			return true;
			break;
		
		default :
			break;
	}
	return false;
}
#endif

bool Tools_Animation_Files::LoadMeshForSkin(Asura_Hierarchy_Skin* pxSkin)
{
	Asura_Assert(pxSkin,"");
	
	//If the meshes are already loaded , then no need to do anything
	if(IsSkinMeshLoaded(pxSkin)) return true;
	

	//try to determine the filename for the skin file
	Asura_String xFilename;
	if(!FindFile(pxSkin->Name.GetString(),&xFilename))
	{
		return false;
	}

	int iOldNumberOfBones = pxSkin->NumberOfBones;
	int iOldNumberOfVertices = pxSkin->NumberOfVertices;

	//Need to allow the skin to be overwritten because the loaded mesh data may be from
	//a different version , and so have a different number of vertices
	pxSkin->m_bAllowOverwriteOnLoad = true;
	bool bReadOkay = LoadAsuraFile(xFilename.GetString());
	pxSkin->m_bAllowOverwriteOnLoad = false;

	if(!bReadOkay) return false;

	//If the number of bones or vertices has changed , then the old render data won't be safe to use,
	//so we'll need to regenerate it. Its possible that the render data may be out of date even
	//if these match (causing it to look wrong) , however I don't think this will cause a crash , and
	//it should save us from having to recalculate it more than we really need to. RAR 14/11/203
	if(iOldNumberOfVertices != pxSkin->NumberOfVertices || iOldNumberOfBones != pxSkin->NumberOfBones)
	{
#if !defined(ASURA_PC_DISABLE_PRESM3_SMOOTH_SKIN_RENDERER) || !ASURA_PC_DISABLE_PRESM3_SMOOTH_SKIN_RENDERER
		Asura_Direct3D_CharacterRender_Handler&			xCharacterHandler		= Asura_Direct3D_CharacterRenderer::GetCurrentHandler();
#endif
		Asura_Direct3D_ObjectHierarchyRender_Handler&	xObjectHierachyHandler	= Asura_Direct3D_ObjectHierarchyRenderer::GetCurrentHandler();

#if !defined(ASURA_PC_DISABLE_PRESM3_SMOOTH_SKIN_RENDERER) || !ASURA_PC_DISABLE_PRESM3_SMOOTH_SKIN_RENDERER
		xCharacterHandler.DeleteCharacterMesh(pxSkin);
#endif
		xObjectHierachyHandler.DeleteHierarchy(pxSkin);		
	}

	
	//finally need to make sure the mesh pointers are up to date in the skin
	if(pxSkin->IsSkinnedModel())
	{
		pxSkin->GetMesh();		
	}
	else
	{
		for(int i=0;i<pxSkin->NumberOfBones;i++)
		{
			Asura_String xMeshName = pxSkin->Name;
			xMeshName += "_";
			xMeshName += pxSkin->SectionData[i].SectionName;
			
			
			pxSkin->SectionData[i].Mesh = Asura_Mesh_System::GetMesh(xMeshName);
		}
	}

	return true;
}

void Tools_Animation_Files::LoadMeshesForAllSkins()
{
	for(int i=0;i<Asura_Animation_System::GetNumberOfSkins();i++)
	{
		LoadMeshForSkin(Asura_Animation_System::GetSkin(i));
	}
}



void Tools_Animation_Files::LoadSoundsForSkin(const Asura_Hierarchy_Skin* pxSkin)
{
	Asura_Assert(pxSkin,"");
	for(int i=0;i<pxSkin->m_iNumberOfSoundDefns;i++)
	{
		if(pxSkin->m_pxSoundDefns[i].m_xSoundName.IsBlank()) continue;

		#if defined ASURA_USE_AUDIO_SYSTEM
			Asura_Audio_System::LoadWave(pxSkin->m_pxSoundDefns[i].m_xSoundName.GetString(),false);
		#else
			Tools_PickSound::GetSoundIDForFilename(pxSkin->m_pxSoundDefns[i].m_xSoundName.GetString(),ASURA_SID_HIERARCHY_MIN,ASURA_SID_HIERARCHY_MAX);
		#endif
	}
}

void Tools_Animation_Files::LoadSoundsForAnim(const Asura_Hierarchy_Anim* pxAnim)
{
	Asura_Assert(pxAnim,"");
	for(int i=0;i<pxAnim->m_iNumberOfSoundDefns;i++)
	{
		if(pxAnim->m_pxSoundDefns[i].m_xSoundName.IsBlank()) continue;

		#if defined ASURA_USE_AUDIO_SYSTEM
			Asura_Audio_System::LoadWave(pxAnim->m_pxSoundDefns[i].m_xSoundName.GetString(),false);
		#else
			Tools_PickSound::GetSoundIDForFilename(pxAnim->m_pxSoundDefns[i].m_xSoundName.GetString(),ASURA_SID_HIERARCHY_MIN,ASURA_SID_HIERARCHY_MAX);
		#endif
	}
}



void Tools_Animation_Files::LoadSoundsForAllSkinsAndAnims()
{
	for(int i=0;i<Asura_Animation_System::GetNumberOfSkins();i++)
	{
		LoadSoundsForSkin(Asura_Animation_System::GetSkin(i));
	}

	for(int i=0;i<Asura_Animation_System::GetNumberOfAnims();i++)
	{
		LoadSoundsForAnim(Asura_Animation_System::GetAnim(i));
	}

}

void Tools_Animation_Files::LoadUncompressedAnimation(Asura_Hierarchy_Anim* pxAnim,const char* pszFilename /*= NULL*/)
{
	//If the uncompressed frames already exist , not need to do anything
	if(pxAnim->BoneAnims) return;

	//try to determine the filename for the anim file (if one wasn't provided)
	Asura_String xFilename = pszFilename;
	if(xFilename.IsBlank())
	{
		if(!FindFile(pxAnim->Name.GetString(),&xFilename))
		{
			return;
		}
	}


	//attempt to load the file
	pxAnim->m_bAllowOverwriteOnLoad = true;
	LoadAsuraFile(xFilename.GetString());
	pxAnim->m_bAllowOverwriteOnLoad = false;
		
}


void Tools_Animation_Files::ReloadAnimations(Tools_Animation_Files_ReloadOptions eReloadType)
{
	//Hierarchy mode may get upset if the edited anim is reloaded , so allow it to take evasive action first.
	Tools_Mode_Hierarchy::PreReloadAnims();
	
	bool bOverwriteAnimsModifiedInTools = false;
	bool bShowOverlay = true;

	int iLoaded = 0;
	int iIgnored = 0;
	int iNotAnim = 0;

 	#ifdef ASURA_SUPPORT_PLATFORM_HASHED_FILENAMES
	int iDesiredPlatform = Tools_PlatformMesh_System::GetPlatform();
	if(iDesiredPlatform<0) iDesiredPlatform = TOOLS_PLATFORM_PC;
	#endif

	for(int i=0;i<Asura_Animation_System::GetNumberOfAnims();i++)
	{
		Asura_Hierarchy_Anim* pxAnim = Asura_Animation_System::GetAnim(i);

		if(eReloadType == TOOLS_ANIMATION_RELOAD_PLATFORMSPECIFIC)
		{
		 	#ifdef ASURA_SUPPORT_PLATFORM_HASHED_FILENAMES
			//If the anim already matches the desired platform , then there is no need to do anything
			if(pxAnim->m_iPlatform == iDesiredPlatform) continue;
			//If using a default platform , then we need to check if there is a platform specific version for this platform
			if(pxAnim->m_iPlatform == -1)
			{
				if(!Asura_HashedFilename_System::DoesStrippedFilenameHaveVersionForPlatform(pxAnim->m_uHashID,iDesiredPlatform))
				{
					//default platform is okay then
					continue;
				}
			}

			#else
			continue;
			#endif
		}

		//get the filename for the anim
		const char* szFilename = Asura_HashedFilename_System::GetFullFilename(pxAnim->m_uHashID,false);
		if(!szFilename)
		{
			if(Tools_System::IsActive())
			{
				Tools_Timed_Message_System::AddMessage(0xff0000,"Failed to find file for %s",pxAnim->Name.GetString());
			}
			#ifdef ASURA_SUPPORT_COMMAND_CONSOLE
			else
			{
		        Asura_CommandConsole::Printf( L"Failed to find file for %S",pxAnim->Name.GetString() );
			}
			#endif
			continue;
		}

		char szNetFilename[MAX_PATH];
		Asura_FileAccess::PlatformGetNetworkFilename(szFilename,szNetFilename,MAX_PATH);

		if(eReloadType == TOOLS_ANIMATION_RELOAD_FILEMODIFIED)
		{
			//skip if the modified time is unchanged
			FILETIME xModifiedTime;
			if(Tools_FileSystem::GetModifiedTime(szNetFilename,&xModifiedTime))
			{
				if(Tools_FileSystem::ApproxCompareFileTime(&xModifiedTime,&pxAnim->m_xModifiedTime) == 0)
				{
					iIgnored++;
					continue;
				}
			}
		}

		if(Tools_System::IsActive())
		{
			//See if file has been modfied in tools
			if(Tools_SaveAsura::IsAsuraFileModified(szFilename))
			{
				if(bShowOverlay)
				{
					char szMessage[500];
					Asura_PrintfToBfr(szMessage,"Anim %s has been modified in tools.\nWould you like to reload it?",sizeof(szMessage),true,pxAnim->Name.GetString());
					YesNoAll_Overlay xOverlay(szMessage);

					bOverwriteAnimsModifiedInTools = xOverlay.DoModal();
					bShowOverlay = !xOverlay.AllSelected();
				}
				if(bOverwriteAnimsModifiedInTools)
				{
					Tools_SaveAsura::SetAsuraFileModified(szFilename,false);
				}
				else
				{
					continue;
				}
			}
		}

		//Make sure this is an anim file
		//(Prevents animating fragments from being reloaded in this way , but they
		//would probably cause problems anyway).
		if(Tools_AsuraFileContents::GetAsuraFileType(szNetFilename,false) == AsuraFile_Anim)
		{
			//attempt to load the file
			pxAnim->m_bAllowOverwriteOnLoad = true;
			LoadAsuraFile(szNetFilename);
			pxAnim->m_bAllowOverwriteOnLoad = false;
		}
		else
		{
			iNotAnim++;
		}
		iLoaded++;
	}

	if(Tools_System::IsActive())
	{
		if(iLoaded)  Tools_Timed_Message_System::AddMessage(0xffffff, "Anims Reloaded : %d", iLoaded);
		if(iIgnored) Tools_Timed_Message_System::AddMessage(0xffffff, "Anims Ignored (unmodified) : %d", iIgnored);
		if(iNotAnim)Tools_Timed_Message_System::AddMessage(0xffffff, "Anims Skipped (not in anim files) : %d", iNotAnim);
	}
	#ifdef ASURA_SUPPORT_COMMAND_CONSOLE
	else
	{
		if(iLoaded) Asura_CommandConsole::Printf( L"Anims Reloaded : %d", iLoaded);
		if(iIgnored) Asura_CommandConsole::Printf( L"Anims Ignored (unmodified) : %d", iIgnored);
		if(iNotAnim )Asura_CommandConsole::Printf( L"Anims Skipped (not in anim files) : %d", iNotAnim);
	}
	#endif

	Tools_Mode_Hierarchy::PostReloadAnims();

}

#ifdef ASURA_SUPPORT_COMMAND_CONSOLE
bool Tools_Animation_Files::ConsoleCmd_ReloadAllAnimations(bool bHelp)
{
	if(bHelp)
	{
        Asura_CommandConsole::Printf( L"Reloads all animations from the individual animation files." );
	}
	else
	{
		ReloadAllAnimations();
	}
	return true;
}
#endif //ASURA_SUPPORT_COMMAND_CONSOLE


void Tools_Animation_Files::ReloadSkins(Tools_Animation_Files_ReloadOptions eReloadType)
{


	//Hierarchy mode may get upset if the edited anim is reloaded , so allow it to take evasive action first.
	Tools_Mode_Hierarchy::PreReloadAnims();
	
	bool bOverwriteSkinsModifiedInTools = false;
	bool bShowOverlay = true;

	int iLoaded = 0;
	int iIgnored = 0;
	int iNotSkin = 0;

 	#ifdef ASURA_SUPPORT_PLATFORM_HASHED_FILENAMES
	int iDesiredPlatform = Tools_PlatformMesh_System::GetPlatform();
	if(iDesiredPlatform<0) iDesiredPlatform = TOOLS_PLATFORM_PC;
	#endif

	for(int i=0;i<Asura_Animation_System::GetNumberOfSkins();i++)
	{
		Asura_Hierarchy_Skin* pxSkin = Asura_Animation_System::GetSkin(i);

		if(eReloadType == TOOLS_ANIMATION_RELOAD_PLATFORMSPECIFIC)
		{
		 	#ifdef ASURA_SUPPORT_PLATFORM_HASHED_FILENAMES
			//If the skin already matches the desired platform , then there is no need to do anything
			if(pxSkin->m_iPlatform == iDesiredPlatform) continue;
			//If using a default platform , then we need to check if there is a platform specific version for this platform
			if(pxSkin->m_iPlatform == -1)
			{
				if(!Asura_HashedFilename_System::DoesStrippedFilenameHaveVersionForPlatform(pxSkin->GetHashID(),iDesiredPlatform))
				{
					//default platform is okay then
					continue;
				}
			}

			#else
			continue;
			#endif
		}


		//get the filename for the skin
		const char* szFilename = Asura_HashedFilename_System::GetFullFilename(pxSkin->GetHashID(),false);
		if(!szFilename)
		{
			if(Tools_System::IsActive())
			{
				Tools_Timed_Message_System::AddMessage(0xff0000,"Failed to find file for %s",pxSkin->Name.GetString());
			}
			#ifdef ASURA_SUPPORT_COMMAND_CONSOLE
			else
			{
		        Asura_CommandConsole::Printf( L"Failed to find file for %S",pxSkin->Name.GetString() );
			}
			#endif
			continue;
		}

		char szNetFilename[MAX_PATH];
		Asura_FileAccess::PlatformGetNetworkFilename(szFilename,szNetFilename,MAX_PATH);

		if(eReloadType == TOOLS_ANIMATION_RELOAD_FILEMODIFIED)
		{
			//skip if the modified time is unchanged
			FILETIME xModifiedTime;
			if(Tools_FileSystem::GetModifiedTime(szNetFilename,&xModifiedTime))
			{
				if(Tools_FileSystem::ApproxCompareFileTime(&xModifiedTime,&pxSkin->m_xModifiedTime) == 0)
				{
					iIgnored++;
					continue;
				}
			}
		}

		if(Tools_System::IsActive())
		{
			//See if file has been modfied in tools
			if(Tools_SaveAsura::IsAsuraFileModified(szFilename))
			{
				if(bShowOverlay)
				{
					char szMessage[500];
					Asura_PrintfToBfr(szMessage,"Skin %s has been modified in tools.\nWould you like to reload it?",sizeof(szMessage),true,pxSkin->Name.GetString());
					YesNoAll_Overlay xOverlay(szMessage);

					bOverwriteSkinsModifiedInTools = xOverlay.DoModal();
					bShowOverlay = !xOverlay.AllSelected();
				}
				if(bOverwriteSkinsModifiedInTools)
				{
					Tools_SaveAsura::SetAsuraFileModified(szFilename,false);
				}
				else
				{
					continue;
				}
			}
		}

		if(pxSkin == Tools_SaveAsura::GetSkin())
		{
			//If tools was loaded by editing a skin ,then reloading it safely is a whole lot more of a pain.
			//Too troublesome to deal with for now , so just say no.
			//RAR 26/04/2007
			Asura_Assert(0,"Attempting to reload the skin file that was used to enter tools.\nThis isn't currently allowed.");
			continue;
		}

		//Make sure this is a skin file
		//(Prevents animating fragments from being reloaded in this way , but they
		//would probably cause problems anyway).
		if(Tools_AsuraFileContents::GetAsuraFileType(szNetFilename,false) == AsuraFile_Skin)
		{
			//invalidate the render data for the skin
			if(pxSkin->IsSkinnedModel())
			{
				#if !defined(ASURA_PC_DISABLE_PRESM3_SMOOTH_SKIN_RENDERER) || !ASURA_PC_DISABLE_PRESM3_SMOOTH_SKIN_RENDERER
				Asura_Direct3D_CharacterRenderer::ResetAllLODs(pxSkin);
				#endif
				#if defined(ASURA_PC_USE_SHADER_MODEL_3_SMOOTH_SKIN_RENDERER) && ASURA_PC_USE_SHADER_MODEL_3_SMOOTH_SKIN_RENDERER
				Asura_Direct3D_SmoothSkinRenderer::ResetAllLODs(pxSkin);
				#endif
			}
			else
			{
				Asura_Direct3D_ObjectHierarchyRenderer::ResetAllLODs(pxSkin);
			}

			
			//Get all the meshes used by the skin , and remove the references to them
			//RAR 07/01/2008
			List<Asura_Mesh*> xMeshList;
			Asura_Hierarchy_Skin* pxLODSkin = pxSkin;
			while(pxLODSkin)
			{
				if(pxLODSkin->IsSkinnedModel())
				{
					xMeshList.add_entry(pxLODSkin->GetMesh());
					pxLODSkin->ClearMeshPointer();
				}
				else
				{
					for(int i=0;i<pxLODSkin->NumberOfBones;i++)
					{
						xMeshList.add_entry(pxLODSkin->SectionData[i].Mesh);
						pxLODSkin->SectionData[i].Mesh = 0;
					}
				}
				pxLODSkin = pxLODSkin->m_pxNextLowerSkin;
			}

			//Delete these meshes, and get a list of old-style materials that were being used
			List<int> xMaterialList;
			for(;xMeshList.size(); xMeshList.delete_first_entry())
			{
				Asura_Mesh* pxMesh = xMeshList.first_entry();

				if(pxMesh && pxMesh->TextureDataPtr)
				{
					for(int i=0;i<pxMesh->NumberOfPolygons;i++)
					{
						int iMatID = pxMesh->TextureDataPtr[i].m_iMaterialID;
						if(iMatID != ASURA_MATERIAL_NONE)
						{
							if(!xMaterialList.contains(iMatID)) xMaterialList.add_entry(iMatID);
						}
					}
				}
				Asura_Mesh_System::DeleteMesh(pxMesh);

			}

			//Delete all the old-style materials used. Because of the way each file creates its own materials
			//on load , it should be safe to do so without worrying about other skins sharing the same materials.
			//This needs to be done even if using the material response system , since that creates the old materials as well
			//RAR 07/01/2008
			for(;xMaterialList.size(); xMaterialList.delete_first_entry())
			{
				int iMatID = xMaterialList.first_entry();
				Asura_Material_System::DestroyMaterial(iMatID);
			}
			
			
			//attempt to load the file
			pxSkin->m_bAllowOverwriteOnLoad = true;
			LoadAsuraFile(szNetFilename);
			pxSkin->m_bAllowOverwriteOnLoad = false;
		}
		else
		{
			iNotSkin++;
		}
		iLoaded++;
	}

	if(Tools_System::IsActive())
	{
		if(iLoaded)  Tools_Timed_Message_System::AddMessage(0xffffff, "Skins Reloaded : %d", iLoaded);
		if(iIgnored) Tools_Timed_Message_System::AddMessage(0xffffff, "Skins Ignored (unmodified) : %d", iIgnored);
		if(iNotSkin)Tools_Timed_Message_System::AddMessage(0xffffff, "Skins Skipped (not in skin files) : %d", iNotSkin);
	}
	#ifdef ASURA_SUPPORT_COMMAND_CONSOLE
	else
	{
		if(iLoaded) Asura_CommandConsole::Printf( L"Skins Reloaded : %d", iLoaded);
		if(iIgnored) Asura_CommandConsole::Printf( L"Skins Ignored (unmodified) : %d", iIgnored);
		if(iNotSkin )Asura_CommandConsole::Printf( L"Skins Skipped (not in skin files) : %d", iNotSkin);
	}
	#endif

	if(iLoaded)
	{
		//Update entity collision objects , since they may have stored pointers to collision meshes
		//in skins that have been reloaded.
		Asura_ServerEntity_PhysicalObject::ResetAllCollisionObjects();
	}

	Tools_Mode_Hierarchy::PostReloadSkins();

}

#ifdef ASURA_SUPPORT_COMMAND_CONSOLE
bool Tools_Animation_Files::ConsoleCmd_ReloadAllSkins(bool bHelp)
{
	if(bHelp)
	{
        Asura_CommandConsole::Printf( L"Reloads all skins from the individual animation files." );
	}
	else
	{
		ReloadAllSkins();
	}
	return true;
}
#endif //ASURA_SUPPORT_COMMAND_CONSOLE


Asura_Hierarchy_Skin* Tools_Animation_Files::GetSkinFromID(Asura_Hash_ID uSkinID,Asura_Hash_ID uFileID,bool bShowMessages)
{

	//is the skin loaded?
	Asura_Hierarchy_Skin* pxSkin = Asura_Animation_System::GetSkinByHash(uSkinID);
	if(!pxSkin)
	{
		if(uFileID == ASURA_HASH_ID_UNSET) uFileID = uSkinID;

		//determine the filename from the hash id
		if(Asura_HashedFilename_System::DoesStrippedFilenameHavePlatformVersions(uSkinID))
		{
			//If there are platform specific versions of the file , then do the lookup based on the skin ID to make
			//sure the appropriate one for the platform is found.
			//It might be better to always ignore the file ID , but there could be problems with existing projects
			//if they have a file appearing in more than one place in the directories being searched.
			//RAR 25/04/2007
			uFileID = uSkinID;
		}

		const char* pxFilename = Asura_HashedFilename_System::GetFullFilename(uFileID, false);
		if(!pxFilename)
		{
			if(bShowMessages)
			{
				Tools_Timed_Message_System::AddMessage(0xff0000,"Failed to determine filename for skin");
			}
			return 0;
		}

		//try loading
		if(!LoadAsuraFile(pxFilename))
		{
			if(bShowMessages)
			{
				Tools_Timed_Message_System::AddMessage(0xff0000,"Failed to load %s",pxFilename);
			}
			return 0;
		}

		//skin there now?
		pxSkin = Asura_Animation_System::GetSkinByHash(uSkinID);
		if(!pxSkin)
		{
			if(bShowMessages)
			{
				Tools_Timed_Message_System::AddMessage(0xff0000,"Failed to find skin in %s",pxFilename);
			}
			return 0;
		}
		//make sure the textures are loaded
		Asura_Texture_Management::LoadTextures();

	}	
	return pxSkin;
}


Asura_Hierarchy_Anim* Tools_Animation_Files::GetAnimFromID(Asura_Hash_ID uAnimID,Asura_Hash_ID uFileID,bool bShowMessages)
{
	if(!uAnimID) return 0;

	Asura_Hierarchy_Anim* pxAnim = Asura_Animation_System::GetAnimByHash(uAnimID);
	if(!pxAnim)
	{
		if(uFileID == ASURA_HASH_ID_UNSET) uFileID = uAnimID;

		if(Asura_HashedFilename_System::DoesStrippedFilenameHavePlatformVersions(uAnimID))
		{
			//If there are platform specific versions of the file , then do the lookup based on the anim ID to make
			//sure the appropriate one for the platform is found.
			//It might be better to always ignore the file ID , but there could be problems with existing projects
			//if they have a file appearing in more than one place in the directories being searched.
			//RAR 25/04/2007
			uFileID = uAnimID;
		}


		//determine the filename from the hash id
		const char* pxFilename = Asura_HashedFilename_System::GetFullFilename(uFileID, false);
		if(!pxFilename)
		{
			if(bShowMessages)
			{
				Tools_Timed_Message_System::AddMessage(0xff0000,"Failed to determine filename for animation ID 0x%X", uAnimID );
			}
			return 0;
		}

		//try loading
		if(!LoadAsuraFile(pxFilename))
		{
			if(bShowMessages)
			{
				Tools_Timed_Message_System::AddMessage(0xff0000,"Failed to load %s",pxFilename);
			}
			return 0;
		}

		//Animation there now?
		pxAnim = Asura_Animation_System::GetAnimByHash(uAnimID);
		if(!pxAnim)
		{
			if(bShowMessages)
			{
				Tools_Timed_Message_System::AddMessage(0xff0000,"Failed to find animation in %s",pxFilename);
			}
			return 0;
		}
	}
	return pxAnim;
}


#ifndef ASURA_DISABLE_ANIM_TABLE_CODE
Asura_AnimTable* Tools_Animation_Files::GetAnimTable(Asura_Hash_ID uHash)
{
	return GetAnimTable(uHash,0);
}

Asura_AnimTable* Tools_Animation_Files::GetAnimTable(const char* szName)
{
	return GetAnimTable(0,szName);
}

Asura_AnimTable* Tools_Animation_Files::GetAnimTable(Asura_Hash_ID uHashID,const char* szName)
{
	if(szName && !uHashID) uHashID = Asura_GetHashID(szName);
	
	//Is the table already loaded?
	Asura_AnimTable* pxTable = Asura_AnimTable_System::GetAnimTable(uHashID);
	if(pxTable) return pxTable;

	//If tools isn't initialised , we are out of luck for the rest
	if(!s_xThis) return 0;

	//also no point in continuing if there isn't any information on how to generate the animtable names
	if(!s_xThis->GetNumberOfAnimTableFilePostFixes()) return 0;

	if(!szName)
	{
		//need to determine the base animtable name , to be able to generate the filenames.
		//basically we need to hope that there is a matching asr file , so we can get the name from the hashed filename system
		szName = Asura_HashedFilename_System::GetFilename(uHashID,false);
		if(!uHashID)
		{
			Asura_Assert(0,"Failed to determine animation table name from hash ID");
			return 0;
		}
	}

	char szTempFilename[MAX_PATH];

	//attempt to find each file that makes up the anim table
	for(int i=0;i<s_xThis->GetNumberOfAnimTableFilePostFixes();i++)
	{
		const char* szPostFix = s_xThis->GetAnimTableFilePostFix(i);
		Asura_Assert(szPostFix,"");
		Asura_PrintfToBfr(szTempFilename,"%s_%s",sizeof(szTempFilename),true,szName,szPostFix);

		//try to get a full filename from the hashed filename system
		const char* szFilename = Asura_HashedFilename_System::GetFullFilename(Asura_GetHashID(szTempFilename),false);
		if(!szFilename) continue;

		//Load the file if it appears to be an animation table
		if(Tools_AsuraFileContents::GetAsuraFileType(szFilename) == AsuraFile_AnimTable)
		{
			LoadAsuraFile(szFilename);
		}
	}

	return Asura_AnimTable_System::GetAnimTable(uHashID);

}

#endif //ifndef ASURA_DISABLE_ANIM_TABLE_CODE

void Tools_Animation_Files::GetStandardRestAnim(Asura_String& xStandardRestAnimFile, Asura_String& xStandardRestAnimPath)
{
	s_xThis->GetStandardRestAnimMain(xStandardRestAnimFile, xStandardRestAnimPath);
}

void Tools_Animation_Files::GetStandardRestAnimMain(Asura_String& xStandardRestAnimFile, Asura_String& xStandardRestAnimPath)
{
	xStandardRestAnimFile = "";
	xStandardRestAnimPath = "";
}

