//
// Tools_Dialog_PickSkin.cpp
//
// Created by Richard Rice - 10/12/2008
// 
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"
#include "Hierarchy/Tools_Dialog_PickSkin.h"

#include "Animation/Asura_Hierarchy.h"
#include "Interface/Asura_Overlay.h"

#include "Hierarchy/Tools_Animation_Files.h"


Tools_Dialog_PickSkin::Tools_Dialog_PickSkin(const char* szPrefix,const Asura_Vector_2& xOffset,Asura_Hash_ID* puSkinID,bool* pbModified /*= 0*/)
:Asura_Function_Dialog(szPrefix,xOffset,OnClicked,0)
{
	m_xPrefix = szPrefix;
	m_puSkinID = puSkinID;
	m_puSharedSkinID = 0;
	m_uSkinIDBackup = GetSkinID();
	m_pbModified = pbModified;

	//The callback user data is set here , rather than in the Asura_Function_Dialog constructor ,to avoid a warning
	SetCallback(OnClicked,(int)this);

	UpdateText();
}

Tools_Dialog_PickSkin::Tools_Dialog_PickSkin(const char* szPrefix,const Asura_Vector_2& xOffset,Asura_Dialog_SharedValue<Asura_Hash_ID>* puSharedSkinID,bool* pbModified /*= 0*/)
:Asura_Function_Dialog(szPrefix,xOffset,OnClicked,0)
{
	m_xPrefix = szPrefix;
	m_puSkinID = 0;
	m_puSharedSkinID = puSharedSkinID;
	m_uSkinIDBackup = GetSkinID();
	m_pbModified = pbModified;
	SetUseSharedDialogColour(true);

	//The callback user data is set here , rather than in the Asura_Function_Dialog constructor ,to avoid a warning
	SetCallback(OnClicked,(int)this);

	UpdateText();
}


void Tools_Dialog_PickSkin::PerFrameUpdate()
{
	Asura_Function_Dialog::PerFrameUpdate();

	//update the label if the skin ID has changed externally
	Asura_Hash_ID uSkinID = GetSkinID();
	if(m_uSkinIDBackup != uSkinID)
	{
		m_uSkinIDBackup = uSkinID;
		UpdateText();
	}
}

void Tools_Dialog_PickSkin::UpdateText()
{
	bool bMixedValue;
	Asura_Hash_ID uSkinID = GetSkinID(&bMixedValue);
	SetMixedSharedValue(bMixedValue);
	

	char szText[100];
	if(uSkinID == 0)
	{
		Asura_PrintfToBfr(szText,"%s*None*",sizeof(szText),true,m_xPrefix.GetString());
	}
	else
	{
		const Asura_Hierarchy_Skin* pxSkin = Tools_Animation_Files::GetSkinFromID(uSkinID, 0 , false);

		if(pxSkin)
		{
			Asura_PrintfToBfr(szText,"%s%s",sizeof(szText),true,m_xPrefix.GetString(),pxSkin->GetName());
		}
		else
		{
			// PAZ 09/03/2009
			if( m_puSkinID == NULL )
			{
				Asura_Assert( false, "Tools_Dialog_PickSkin::UpdateText, m_puSkinID == NULL" );
				Asura_PrintfToBfr( szText, "%s*NULL*", sizeof( szText ), true, m_xPrefix.GetString() );
			}
			else
			{
				Asura_PrintfToBfr(szText,"%s*Unknown* (0x%x)",sizeof(szText),true,m_xPrefix.GetString(),*m_puSkinID);
			}
		}
	}
	if(m_pxOverlay)
	{
		m_pxOverlay->UpdateDialogLabel(this,szText);
	}
	else
	{
		UpdateLabel(szText);
	}
}


void Tools_Dialog_PickSkin::OnClicked()
{
	const Asura_Hierarchy_Skin* pxSkin = 0;
	if(Tools_Animation_Files::ChooseSkin_AllowNone(&pxSkin))
	{
		SetSkinID(pxSkin ? pxSkin->GetHashID() : 0);
	}

}

Asura_Hash_ID Tools_Dialog_PickSkin::GetSkinID(bool* pbMixedValue) const
{
	if(m_puSkinID)
	{
		if(pbMixedValue) *pbMixedValue = false;
		return *m_puSkinID;
	}
	else
	{
		Asura_Assert(m_puSharedSkinID, "");
		Asura_Hash_ID uSkinID = 0;
		m_puSharedSkinID->GetValue(&uSkinID, pbMixedValue);

		return uSkinID;
	}
}

void Tools_Dialog_PickSkin::SetSkinID(Asura_Hash_ID uSkinID)
{
	m_uSkinIDBackup = uSkinID;	
	if(m_pbModified) *m_pbModified = true;
	if(m_puSkinID)
	{
		*m_puSkinID = uSkinID;
	}
	if(m_puSharedSkinID)
	{
		m_puSharedSkinID->SetValue(uSkinID);
	}
	UpdateText();

}
