//
// Tools_SectionBoundingBox_Object.cpp
//
// Created by Richard Rice - 16/07/2004
// 
// Copyright (C) 2004 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"
#include "Hierarchy/Tools_SectionBoundingBox_Object.h"

#include "Animation/Asura_Animation.h"
#include "Animation/Asura_Hierarchy.h"
#include "Camera/Asura_Camera.h"
#include "Core/Asura_LOS.h"
#include "Core/Asura_UserInput.h"
#include "Core/Asura_Timers.h"
#include "Interface/Asura_Overlay_System.h"
#include "Interface/Asura_Overlay.h"
#include "Interface/Asura_Dialog.h"
#include "Mesh/Asura_CollisionMesh.h"
#include "Rendering/Asura_Text_Output.h"
#include "Rendering/Asura_Render.h"
#include "Rendering/Asura_Font_System.h"



#include "Tools_Overlays.h"
#include "Hierarchy/Tools_Mode_Hierarchy.h"
#include "Misc/Tools_Render.h"



Asura_Vector_2 Tools_SectionBoundingBox_Object::s_xOverlayPosition(10,10);
Tools_SectionBoundingBox_Object* Tools_SectionBoundingBox_Object::s_pxThis = 0;


Tools_SectionBoundingBox_Object::Tools_SectionBoundingBox_Object(Asura_Hierarchy_Skin* pxSkin,int iSection)
:Tools_Object(TOT_SectionBoundingBox)
{
	m_pxSkin = pxSkin;
	m_iSection = iSection;

	Asura_Assert(m_pxSkin->SectionData,"");

	//get the name from the section data
	m_xName = m_pxSkin->SectionData[iSection].SectionName;

	if(!m_pxSkin->m_pxSectionBoundingBoxData) m_pxSkin->InitSectionBoundingBoxes();

	//get the bounding box
	m_bManualBox = false;
	m_xBox = m_pxSkin->m_pxSectionBoundingBoxData[iSection].m_xBB;
	m_bValid = m_pxSkin->m_pxSectionBoundingBoxData[iSection].m_uValid;

	if(m_pxSkin->m_pxManualSectionBoundingBoxData)
	{
		if(m_pxSkin->m_pxManualSectionBoundingBoxData[iSection].m_uValid)
		{
			m_bManualBox = true;
		}
	}
	if(!m_bValid)
	{
		m_xBox.Zero();
	}

	m_bMinChanged = false;
	m_bMaxChanged = false;
	m_bChanged = false;
	m_bCollisionMesh = m_pxSkin->SectionData[m_iSection].m_pxCollisionMesh ? true : false;
	//Default to showing the collision mesh for skinned models that have them
	m_bShowCollisionMesh = m_bCollisionMesh && m_pxSkin->IsSkinnedModel();;
	m_eSelectedFace = FACE_NONE;
}

void Tools_SectionBoundingBox_Object::UpdatePosition(const Asura_Vector_3& xPos,const Asura_Matrix_3x3& xMat)
{
	Position = xPos;
	Orientation = xMat;
}

DEBUGVAR u_int dbg_auBBColours[4] = { 0xffffff00, 0xff00ffff, 0xffff00ff, 0xff7fff7f };
void Tools_SectionBoundingBox_Object::Render()
{
	u_int uColour = dbg_auBBColours[m_iSection&3];
	if(IsSelected())
	{
		uColour = 0xffff0000;
	}
	else if(m_bManualBox && !m_bShowCollisionMesh)
	{
		uColour = 0xffafafaf;
	}

	u_int uSelectedFaceColour = uColour;
	if(IsSelected())
	{
		uSelectedFaceColour = 0xffffcfcf;
	}

	if(m_bShowCollisionMesh)
	{
		Asura_CollisionMesh* pxCollMesh = m_pxSkin->SectionData[m_iSection].m_pxCollisionMesh;
		Asura_Assert(pxCollMesh,"");
		Tools_Render::RenderCollisionMesh(pxCollMesh,&Position,&Orientation,uColour);
	}
	else
	{
		Tools_Render::RenderCube(m_xBox, Position, Orientation, uColour,true,m_eSelectedFace,uSelectedFaceColour);
	}

//	Asura_Bounding_Box xBB = m_xBox;		
//	Asura_Vector_3 xPos = xBB.CalculateCentre();	
//	xPos = (xPos * Orientation) + Position;
//
//	Asura_Camera::ProjectWorldCoordsToScreenSpace( xPos, true );
//	if( xPos.z > 0.0f )
//	{
//		Asura_Font_System::Print( xPos.x, xPos.y, 2.0f, 0xffffffff, 0, L"%d", m_iSection );	
//	}	
	
	if ( IsSelected() || IsGroupSelected() )
	{
		Asura_Render::RenderAxes( Position, Orientation, 0.4f );
	}
}

void Tools_SectionBoundingBox_Object::UpdateWhileSelected()
{

	if(!Asura_Overlay_System::OverlaysHaveKeyboardFocus() && m_pxSkin->IsSkinnedModel())
	{
  		if(!Asura_UserInput::KeyPressed(ASURA_KEYID_ALT) && !Asura_UserInput::KeyPressed(ASURA_KEYID_CTRL))
		{
            if ( !Asura_UserInput::KeyPressed( ASURA_KEYID_SHIFT ) )
            {

			    if(!m_bShowCollisionMesh)
				{
			    	if(Asura_UserInput::DebouncedKeyPressed('1'))
			    	{
					    m_eSelectedFace = FACE_MIN_X;
			    	}
			    	if(Asura_UserInput::DebouncedKeyPressed('2'))
			    	{
					    m_eSelectedFace = FACE_MAX_X;
			    	}
			    	if(Asura_UserInput::DebouncedKeyPressed('3'))
			    	{
					    m_eSelectedFace = FACE_MIN_Y;
			    	}
			    	if(Asura_UserInput::DebouncedKeyPressed('4'))
			    	{
					    m_eSelectedFace = FACE_MAX_Y;
			    	}
			    	if(Asura_UserInput::DebouncedKeyPressed('5'))
			    	{
					    m_eSelectedFace = FACE_MIN_Z;
			    	}
			    	if(Asura_UserInput::DebouncedKeyPressed('6'))
			    	{
					    m_eSelectedFace = FACE_MAX_Z;
			    	}
                	if( Asura_UserInput::DebouncedKeyPressed('7') ||
                	    Asura_UserInput::DebouncedKeyPressed('8') )
                	{
					    m_eSelectedFace = FACE_NONE;
                	}
				}

            }
			//increase/decrease side.
			if(m_eSelectedFace != FACE_NONE && !m_bShowCollisionMesh)
			{
				float fAdjustment = 0;
				if(Asura_UserInput::KeyPressed('0'))
				{
					fAdjustment = Asura_Timers::GetFrameTime()/5.0f;
				}
				if(Asura_UserInput::KeyPressed('9'))
				{
					fAdjustment = -Asura_Timers::GetFrameTime()/5.0f;
				}
				if(fAdjustment)
				{
					if(Asura_UserInput::KeyPressed(ASURA_KEYID_SHIFT))
					{
						fAdjustment*=10.0f;
					}
					switch(m_eSelectedFace)
					{
						case FACE_MIN_X :
							m_xBox.MinX -= fAdjustment;
							m_bMinChanged = true;
							break;
						case FACE_MAX_X :
							m_xBox.MaxX += fAdjustment;
							m_bMaxChanged = true;
							break;
						case FACE_MIN_Y :
							m_xBox.MinY -= fAdjustment;
							m_bMinChanged = true;
							break;
						case FACE_MAX_Y :
							m_xBox.MaxY += fAdjustment;
							m_bMaxChanged = true;
							break;
						case FACE_MIN_Z :
							m_xBox.MinZ -= fAdjustment;
							m_bMinChanged = true;
							break;
						case FACE_MAX_Z :
							m_xBox.MaxZ += fAdjustment;
							m_bMaxChanged = true;
							break;
					}

				}
			}
		}

	}

	if(m_bMinChanged)
	{
		//make sure the maximums stay greater than the minimumes
		if(m_xBox.MinX > m_xBox.MaxX) m_xBox.MinX = m_xBox.MaxX;
		if(m_xBox.MinY > m_xBox.MaxY) m_xBox.MinY = m_xBox.MaxY;
		if(m_xBox.MinZ > m_xBox.MaxZ) m_xBox.MinZ = m_xBox.MaxZ;
		m_bMinChanged = false;
		m_bChanged = true;
	}
	if(m_bMaxChanged)
	{
		//make sure the maximums stay greater than the minimumes
		if(m_xBox.MinX > m_xBox.MaxX) m_xBox.MaxX = m_xBox.MinX;
		if(m_xBox.MinY > m_xBox.MaxY) m_xBox.MaxY = m_xBox.MinY;
		if(m_xBox.MinZ > m_xBox.MaxZ) m_xBox.MaxZ = m_xBox.MinZ;
		m_bMaxChanged = false;
		m_bChanged = true;
	}


	if(m_bChanged)
	{
		if(!m_pxSkin->m_pxManualSectionBoundingBoxData)
		{
			Asura_ResourcePushType(ASURA_RESOURCE_ANIMATION);
			{
				Asura_ResourceAllocateArray(m_pxSkin->m_pxManualSectionBoundingBoxData, m_pxSkin->NumberOfBones);
			}
			Asura_ResourcePopType();
		}
		
		m_pxSkin->m_pxManualSectionBoundingBoxData[m_iSection].m_xBB = m_xBox;
		m_pxSkin->m_pxManualSectionBoundingBoxData[m_iSection].m_uValid = 1;
		m_pxSkin->m_pxSectionBoundingBoxData[m_iSection].m_xBB = m_xBox;
		m_pxSkin->m_pxSectionBoundingBoxData[m_iSection].m_uValid = m_xBox.GetVolume() > 0.0f;

		m_bManualBox = true;
		m_bValid = true;
		m_bChanged = false;

		Tools_Mode_Hierarchy::SetSkinModified();
	}
}

void Tools_SectionBoundingBox_Object::PrintInfo(float& x,float& y)
{
	Tools_Object::PrintInfo(x,y);
	Asura_Text_Output::Print(x,y++,0xffffffff,0,"Bone ID: %d",m_iSection);

	Asura_Text_Output::Print(x,y++,0xffffffff,0,"Collision Mesh : %s",m_bCollisionMesh ? "Yes" : "No");
	Asura_Text_Output::Print(x,y++,0xffffffff,0,"Manual BB      : %s",m_bManualBox ? "Yes" : "No");
}


void Tools_SectionBoundingBox_Object::OnSelection()
{
	s_pxThis = this;
	CreateOverlay();
}

void Tools_SectionBoundingBox_Object::OnDeselection()
{
	DeleteOverlay();
	s_pxThis = 0;
}

void Tools_SectionBoundingBox_Object::CreateOverlay()
{
	Asura_Overlay* pxOverlay;
	pxOverlay = new Asura_Overlay(GetTypeDescription(),false,&s_xOverlayPosition);

	
	Asura_Dialog* pxDialog;
	Asura_Vector_2 xOffset = Asura_Vector_2(1.0f,3.0f);

	pxDialog = new Asura_Numerical_Dialog("1: Min X",xOffset,&m_xBox.MinX,-99.0f,99.0f,0.01f,"%.3f",&m_bMinChanged);
	pxOverlay->AddDialog(pxDialog);
	pxDialog->SetIsEnabledFunction(IsEnabled_BBEdit);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("2: Max X",xOffset,&m_xBox.MaxX,-99.0f,99.0f,0.01f,"%.3f",&m_bMaxChanged);
	pxOverlay->AddDialog(pxDialog);
	pxDialog->SetIsEnabledFunction(IsEnabled_BBEdit);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("3: Min Y",xOffset,&m_xBox.MinY,-99.0f,99.0f,0.01f,"%.3f",&m_bMinChanged);
	pxOverlay->AddDialog(pxDialog);
	pxDialog->SetIsEnabledFunction(IsEnabled_BBEdit);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("4: Max Y",xOffset,&m_xBox.MaxY,-99.0f,99.0f,0.01f,"%.3f",&m_bMaxChanged);
	pxOverlay->AddDialog(pxDialog);
	pxDialog->SetIsEnabledFunction(IsEnabled_BBEdit);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("5: Min Z",xOffset,&m_xBox.MinZ,-99.0f,99.0f,0.01f,"%.3f",&m_bMinChanged);
	pxOverlay->AddDialog(pxDialog);
	pxDialog->SetIsEnabledFunction(IsEnabled_BBEdit);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("6: Max Z",xOffset,&m_xBox.MaxZ,-99.0f,99.0f,0.01f,"%.3f",&m_bMaxChanged);
	pxOverlay->AddDialog(pxDialog);
	pxDialog->SetIsEnabledFunction(IsEnabled_BBEdit);
	xOffset.y+=Asura_Dialog::LargeSpacing();

	pxDialog = new Asura_Function_Dialog("Reset",xOffset,_Reset);
	pxDialog->SetHelpText("This gets rid of any manual changes to this section's bounding box , and causes the automatically calculated one to be used instead.");
	pxDialog->SetIsEnabledFunction(IsEnabled_BBEdit);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Disable Box",xOffset,_DisableBox);
	pxDialog->SetHelpText("Disables the bounding box for this section by setting all the extents to zero.");
	pxDialog->SetIsEnabledFunction(IsEnabled_BBEdit);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Dialog("9/0 : Adjusts selected face",xOffset);
	pxDialog->SetHelpText("Select a face by pressing the appropriate number from 1 to 6.\nThen press 9 to move the face in , and 0 to move out.");
	pxDialog->SetIsEnabledFunction(IsEnabled_BBEdit);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::LargeSpacing();

	if(m_bCollisionMesh)
	{
		pxDialog = new Asura_Tickbox_Dialog("Show Coll Mesh",xOffset,&m_bShowCollisionMesh);
		pxDialog->SetHelpText("Show the section collision mesh that is used for LOS tests.\nUnticking this allows the manual section BBs to be viewed/edited , but they are largely unused unless accessed by project code.\nThe manual BBs will only be used for calcualting the skins overall bounding box.");
		pxOverlay->AddDialog(pxDialog);
		xOffset.y+=Asura_Dialog::DefaultSpacing();
	}
	
	pxOverlay->m_uID = TOOLS_OVERLAY_SELECTEDOBJECT;

 	Asura_Overlay_System::SubmitNewOverlay(pxOverlay,false);
}

void Tools_SectionBoundingBox_Object::DeleteOverlay()
{
	Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_SELECTEDOBJECT);
}


bool Tools_SectionBoundingBox_Object::ObjectIsInLOS(const Asura_Vector_3& xAlpha, const Asura_Vector_3& xBeta, float& fLowest)
{
	if(m_bShowCollisionMesh)
	{
		Asura_CollisionMesh* pxCollMesh = m_pxSkin->SectionData[m_iSection].m_pxCollisionMesh;
		Asura_Assert(pxCollMesh,"");
		Asura_LOS_InputData xInput(xAlpha,xBeta,ASURA_GUID_UNREGISTERED,fLowest);
		Asura_LOS_OutputData xOutput;
		xOutput.m_fDistanceToIntersection = fLowest;
		if(pxCollMesh->LOSTestPolygons(Position,Orientation,&xInput,&xOutput))
		{
			fLowest = xOutput.m_fDistanceToIntersection;			
			return true;
		}
		return false;

	}
	else
	{
		if(!m_bValid) return false;

		//rotate the positions to be relative to the bounding box
		Asura_Vector_3 xRelAlpha = Orientation.ApplyTranspose(xAlpha - Position);
		Asura_Vector_3 xRelBeta = Orientation.ApplyTranspose(xBeta);


		//generate points for cube
		Asura_Vector_3 axVerts[8];
		for(int i=0;i<8;i++)
		{
			axVerts[i].x = (i&1) ? m_xBox.MaxX : m_xBox.MinX;
			axVerts[i].y = (i&2) ? m_xBox.MaxY : m_xBox.MinY;
			axVerts[i].z = (i&4) ? m_xBox.MaxZ : m_xBox.MinZ;
		}

		//triangle indeces
		int aiIndeces[12][3] =
		{
			{0,4,6},//min x
			{0,6,2},
			{3,7,5},//max x
			{3,5,1},
			{1,5,4},//min y
			{1,4,0},
			{2,6,7},//max y
			{2,7,3},
			{0,2,3},//min z
			{0,3,1},
			{4,6,7},//max z
			{4,7,5}
		};

		//do los tests against the bounding box sides
		bool bInLOS = false;
		for(int i=0;i<12;i++)
		{
			Asura_Vector_3& xV0 = axVerts[aiIndeces[i][0]];
			Asura_Vector_3& xV1 = axVerts[aiIndeces[i][1]];
			Asura_Vector_3& xV2 = axVerts[aiIndeces[i][2]];
			bool bFacingToward;

			float fDist = Asura_Maths::RayIntersectsTri(xRelAlpha,xRelBeta,xV0,xV1,xV2,bFacingToward);
			if(fDist >= 0.0f && fDist < fLowest)
			{
				fLowest = fDist;
				bInLOS = true;
			}
		}												   
	

		return bInLOS;
	}
}

void Tools_SectionBoundingBox_Object::Reset()
{
	if(m_bManualBox)
	{
		//reset this box to the automatic values
		m_pxSkin->m_pxManualSectionBoundingBoxData[m_iSection].m_uValid = 0;
		m_bManualBox = false;

		//recalculate by deleting and initialising the boxes
		Asura_ResourcePushType(ASURA_RESOURCE_ANIMATION);
		{
			Asura_ResourceDeallocateArray(m_pxSkin->m_pxSectionBoundingBoxData);
		}
		Asura_ResourcePopType();
		m_pxSkin->m_pxSectionBoundingBoxData = 0;
		m_pxSkin->InitSectionBoundingBoxes();

		//grab the values again
		m_xBox = m_pxSkin->m_pxSectionBoundingBoxData[m_iSection].m_xBB;
		m_bValid = m_pxSkin->m_pxSectionBoundingBoxData[m_iSection].m_uValid;
		if(!m_bValid)
		{
			m_xBox.Zero();
		}

		Tools_Mode_Hierarchy::SetSkinModified();
	}
}

void Tools_SectionBoundingBox_Object::DisableBox()
{
	m_xBox.Zero();
	m_bChanged = true;	
}


