//
// Tools_IPImage.cpp
//
// Copyright (C) 2003 Rebellion, All Rights Reserved.
//


#include "Core/Asura.h"
#include "ImageProcessing/Tools_IPImage.h"

#include <string.h>

void Tools_IPImage::Destroy()
{
	if (aColourData) delete[] aColourData;
	if (aColourRows) delete[] aColourRows;
	if (aIndexData) delete[] aIndexData;
	if (aIndexRows) delete[] aIndexRows;
	if (aAlphaData) delete[] aAlphaData;
	if (aAlphaRows) delete[] aAlphaRows;
}

void Tools_IPImage::Init()
{
	aColourData = NULL;
	aColourRows = NULL;
	aIndexData = NULL;
	aIndexRows = NULL;
	aAlphaData = NULL;
	aAlphaRows = NULL;
	bHasMask = false;
}

void Tools_IPImage::CopyFrom(Tools_IPImage const & rImage)
{
	if (!rImage.IsValid()) return;
	
	width = rImage.width;
	height = rImage.height;
	palette_size = rImage.palette_size;
	
	Create();
	
	if (NOT_PALETTIZED == palette_size)
	{
		memcpy(aColourData,rImage.aColourData,width*height*sizeof(IPColour));
	}
	else
	{
		memcpy(aIndexData,rImage.aIndexData,width*height*sizeof(IPIndex));
			
		memcpy(aColourData,rImage.aColourData,palette_size*sizeof(IPColour));
	}
	
	bHasMask = rImage.bHasMask;
	colourMask = rImage.colourMask;
	indexMask = rImage.indexMask;
	
	if (rImage.HasAlphaChannel())
	{
		CreateAlphaChannel();
		
		memcpy(aAlphaData,rImage.aAlphaData,width*height*sizeof(IPAlphaValue));
	}
}

void Tools_IPImage::Create()
{
	if (NOT_PALETTIZED == palette_size)
	{
		aColourRows = new IPColour * [height];
		aColourData = new IPColour [width*height];
		for (unsigned i=0,j=0; i<height; ++i,j+=width)
		{
			aColourRows[i] = &aColourData[j];
		}
	}
	else
	{
		aIndexRows = new IPIndex * [height];
		aIndexData = new IPIndex [width*height];
		for (unsigned i=0,j=0; i<height; ++i,j+=width)
		{
			aIndexRows[i] = &aIndexData[j];
		}
		
		aColourData = new IPColour [palette_size];
	}
	bModified = true;
}

static inline IPColour const & GetAt(Tools_IPImage const & img,unsigned x,unsigned y)
{
	if (Tools_IPImage::NOT_PALETTIZED == img.GetPaletteSize())
		return img.GetColourRow(y)[x];
	else
		return img.GetPalette()[img.GetIndexRow(y)[x]];
}

static IPAlphaValue ip_ipavDefault = 0xFF;
static inline IPAlphaValue const & GetAlphaAt(Tools_IPImage const & img, unsigned x, unsigned y)
{
	if (img.HasAlphaChannel())
		return img.GetAlphaRow(y)[x];
	else
		return ip_ipavDefault;
}

bool Tools_IPImage::operator == (Tools_IPImage const & cmp) const
{
	if (!IsValid() || !cmp.IsValid()) return false;
	
	if (height != cmp.height || width != cmp.width) return false;
	
	for (unsigned y=0; y<height; ++y)
	{
		for (unsigned x=0; x<width; ++x)
		{
			if (GetAt(*this,x,y) != GetAt(cmp,x,y) || GetAlphaAt(*this,x,y) != GetAlphaAt(cmp,x,y))
			{
				return false;
			}
		}
	}
	return true;
}

template <class T>
	void swap(T & x, T & y)
	{
		T tmp = x;
		x = y;
		y = tmp;
	}

bool Tools_IPImage::SwapIndeces(IPIndex index1, IPIndex index2)
{
	if (!IsValid() || !IsPalettized()) return false;
	if (index1 >= GetPaletteSize() || index2 >= GetPaletteSize()) return false;
	
	swap(GetPalette()[index1],GetPalette()[index2]);
	
	if (HasMask())
	{
		if (GetMaskIndex()==index1)
			SetMaskIndex(index2);
		else if (GetMaskIndex()==index2)
			SetMaskIndex(index1);
	}
	
	for (unsigned y=0; y<height; ++y)
	{
		for (unsigned x=0; x<width; ++x)
		{
			if (GetIndexRow(y)[x]==index1)
				GetIndexRow(y)[x]=index2;
			else if (GetIndexRow(y)[x]==index2)
				GetIndexRow(y)[x]=index1;
		}
	}
	
	return true;
}

void Tools_IPImage::CreateAlphaChannel()
{
	if (!IsValid()) return;
	
	aAlphaRows = new IPAlphaValue * [height];
	aAlphaData = new IPAlphaValue [width*height];
	for (unsigned i=0,j=0; i<height; ++i,j+=width)
	{
		aAlphaRows[i] = &aAlphaData[j];
	}
	
	bModified = true;
}

void Tools_IPImage::RemoveAlphaChannel()
{
	if (!IsValid()) return;
	
	if (aAlphaData) delete[] aAlphaData;
	if (aAlphaRows) delete[] aAlphaRows;
	aAlphaData = NULL;
	aAlphaRows = NULL;
	
	bModified = true;
}
