//
// Tools_RGBTypes.h
//
// Copyright (C) 2003 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef _INCLUDED_RGBTYPES_HPP_
#define _INCLUDED_RGBTYPES_HPP_

template<class T>
struct RGBTriple;

template<class T>
struct RGBTriple
{
	RGBTriple() : r(0), g(0), b(0) {}
	
	#if defined(__WATCOMC__) && (__WATCOMC__<=1100)
		#define RGBT_INSTANTIATE_CONSTRUCTOR(W) \
			RGBTriple(RGBTriple<W> const & p) : r(static_cast<T>(p.r)), g(static_cast<T>(p.g)), b(static_cast<T>(p.b)) {}
		RGBT_INSTANTIATE_CONSTRUCTOR(unsigned char)
		RGBT_INSTANTIATE_CONSTRUCTOR(signed char)
		RGBT_INSTANTIATE_CONSTRUCTOR(unsigned short)
		RGBT_INSTANTIATE_CONSTRUCTOR(signed short)
		RGBT_INSTANTIATE_CONSTRUCTOR(unsigned)
		RGBT_INSTANTIATE_CONSTRUCTOR(signed)
		RGBT_INSTANTIATE_CONSTRUCTOR(unsigned long)
		RGBT_INSTANTIATE_CONSTRUCTOR(signed long)
		RGBT_INSTANTIATE_CONSTRUCTOR(float)
		RGBT_INSTANTIATE_CONSTRUCTOR(double)
	#else
		template<class W>
			RGBTriple(RGBTriple<W> const & p) : r(static_cast<T>(p.r)), g(static_cast<T>(p.g)), b(static_cast<T>(p.b)) {}
	#endif
	RGBTriple(T const r, T const g, T const b) : r(r), g(g), b(b) {}
	
	//inline operator PG_Pixel (void) const { return PG_Pixel(r,g,b); }
	
	T r,g,b;

	inline RGBTriple & operator += (RGBTriple const & d) { r += d.r; g += d.g; b += d.b; return *this; }
	inline RGBTriple & operator -= (RGBTriple const & d) { r -= d.r; g -= d.g; b -= d.b; return *this; }
	inline RGBTriple & operator *= (T const n) { r *= n; g *= n; b *= n; return *this; }
	inline RGBTriple & operator *= (RGBTriple const & d) { r *= d.r; g *= d.g; b *= d.b; return *this; }
	inline RGBTriple & operator /= (T const n) { r /= n; g /= n; b /= n; return *this; }
	inline RGBTriple & operator /= (RGBTriple const & d) { r /= d.r; g /= d.g; b /= d.b; return *this; }

	inline RGBTriple operator + (RGBTriple const & d) const { return RGBTriple(r+d.r,g+d.g,b+d.b); }
	inline RGBTriple operator - (RGBTriple const & d) const { return RGBTriple(r-d.r,g-d.g,b-d.b); }
	inline RGBTriple operator * (T const n) const { return RGBTriple(r*n,g*n,b*n); }
	inline RGBTriple operator * (RGBTriple const & d) const { return RGBTriple(r*d.r,g*d.g,b*d.b); }
	inline RGBTriple operator / (T const n) const { return RGBTriple(r/n,g/n,b/n); }
	inline RGBTriple operator / (RGBTriple const & d) const { return RGBTriple(r/d.r,g/d.g,b/d.b); }
	
	inline bool operator == (RGBTriple const & d) const { return r==d.r && g==d.g && b==d.b; }
	inline bool operator != (RGBTriple const & d) const { return ! operator == (d); }
	inline bool operator < (RGBTriple const & d) const { return r*299+g*587+b*114 < d.r*299+d.g*587+d.b*114; }
	inline bool operator >= (RGBTriple const & d) const { return ! operator < (d); }
	inline bool operator <= (RGBTriple const & d) const { return operator == (d) || operator < (d); }
	inline bool operator > (RGBTriple const & d) const { return ! operator <= (d); }
};

template<class T, class N>
struct RGBQuad : public RGBTriple<T>
{
	RGBQuad(T const r,T const g,T const b,N const n) : RGBTriple<T>(r,g,b), n(n) {}
	RGBQuad(T const r,T const g,T const b) : RGBTriple<T>(r,g,b) {}
	RGBQuad(RGBTriple<T> const & p,N const n) : RGBTriple<T>(p), n(n) {}
	RGBQuad(RGBTriple<T> const & p) : RGBTriple<T>(p) {}
	RGBQuad(){}

	inline RGBQuad operator + (RGBQuad const & a) const { return RGBQuad(RGBTriple<T>::operator + (a), n + a.n); }
	inline RGBQuad & operator += (RGBQuad const & a) { RGBTriple<T>::operator += (a); n += a.n; return *this; }
	inline RGBQuad & operator += (N const nn) { n += nn; return *this; }

	N n;
};

#endif
