#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef __TOOLS_TIM2_WRITER_H__
#define __TOOLS_TIM2_WRITER_H__

// in middleware.
#include "NVidia/DDS_Utilities/dxtlib.h"

//for writeing 24 bit bmps as well as tims
//#define TOOLS_TIM2_WRITEBMPS

#define TIM2_MAX_MIPMAPS 7 //(including original image)

// (MP - 05.03.2003) Sometimes we need to generate a lot of mipmaps. Some
// source textures are at ridiculous resolutions wrt to the PS2 output
// version (eg. 2048x2048 versus 128x128.)
#define TOOLS_TIM2_MAX_MIPMAPS_TO_GENERATE (10)

//storage for 1 mip map level
struct Tools_Tim2_ImageData
{
	Tools_Tim2_ImageData(){Width=0;Height=0;PixelData=0;TimDataSize = 0;}
	~Tools_Tim2_ImageData(){delete [] PixelData;}
	
	int Width;
	int Height;
	unsigned int* PixelData;
	int TimDataSize;//size of texture data in file
};

struct TextureConversionData;

class Tools_Tim2_Writer
{
public :
	Tools_Tim2_Writer();
	Tools_Tim2_Writer(int iWidth,int iHeight,int iPitch,void* pBits,const TextureConversionData& xConversionData);
	~Tools_Tim2_Writer();

	void Initialise(int iWidth,int iHeight,int iPitch,void* pBits,const TextureConversionData& xConversionData);

	enum ALPHA_TYPES {ALPHA_NONE,ALPHA_1BIT,ALPHA_8BIT};


	//rescales the main image to the specified dimensions.
	void ScaleImage(int iWidth,int iHeight);


	bool WriteOutFile(const char* filename,int numMipLevelsToGenerate,int numMipLevelsToDrop,bool bRescaleAlpha=true);


    static bool GetDataSizeFromExistingFile( const char* szFilename, u_int* puDataSize );
    static bool GetImgProportionsFromExistingFile( const char* szFilename, u_int* puWidth, u_int* puHeight );
    static bool GetAlphaTypeFromExistingFile( const char* szFilename, ALPHA_TYPES* peAlphaType );

	static bool ReadHeadersFromFile( const char* szFilename, struct TIM2_FILEHEADER* pxFileHdr, struct TIM2_PICTUREHEADER* pxPicHdr, struct AsuraTim2UserData* pxUserData = 0);
	static bool ReadHeadersFromBuffer( const char* pcBuffer, u_int uBufferSize,struct TIM2_FILEHEADER* pxFileHdr,struct  TIM2_PICTUREHEADER* pxPicHdr, struct AsuraTim2UserData* pxUserData = 0);

	//If swizzled , this restores the original width,height and image type in the picture header
	static void DeSwizzleHeaders(struct  TIM2_PICTUREHEADER* pxPicHdr, struct AsuraTim2UserData* pxUserData);


	//Get the size of the mip map header for a given number of mip maps
	static int GetMipMapHeaderSize(int iNumMipMaps);


private :

	//passes the top level image , in 8888 ARGB
	void SetMainImage(int iWidth,int iHeight,int iPitch,void* pBits);

	#ifdef TOOLS_TIM2_WRITEBMPS
	void WriteBitmaps(const char* pszTimName);
	#endif

	void Sharpen();

	int NumMipMaps;//including main image
	//pixel data for themain image , and generated mip maps
	Tools_Tim2_ImageData Images[TOOLS_TIM2_MAX_MIPMAPS_TO_GENERATE];

	u_int m_uPaletteSize;
	int* Palette;

	ALPHA_TYPES AlphaType;
	ALPHA_TYPES ActualAlpha;

	int PixelFormat;
	int PaletteFormat;
	bool AlphaIsUsedAsNormalAlpha;

	int TextureFlags;
	unsigned int m_uMaterialFlags;
	bool Swizzle;

	//Causes a fixed palette to be used , as defined by SetFixedPalette
	bool m_bUseFixedPalette;
	//For simpsons... causes the end palette to contain index values in the red component , rather
	//than proper colour values.
	bool m_bWritePaletteAsRedScaleIndices;

	//Prevents the use of dithering during palette generation.
	bool m_bDisableDithering;

	//If non-zero may cause extra mip levels to be dropped to reduce texture size to below this
	u_int m_uMaximumTextureSize;



	bool GenerateMipMaps( int maxImages, int numMipLevelsToDrop, u_int uImgType );
	void GenerateAlphaMipMapsForSelfIllumination();
	void MoireFix();
	bool GeneratePalette(int size,int clutType);
	void ConvertPixelData(int imageType,bool bRescaleAlpha=true);


	static Tools_Tim2_Writer* ThisWriter;
	static NV_ERROR_CODE MipMapCallback(const void* pPixelData, size_t uCount, const MIPMapData* pxMipMapData, void* pUserData);

};


struct Tim2_RGB16_Pixel
{
	unsigned short r:5;
	unsigned short g:5;
	unsigned short b:5;
	unsigned short a:1;
};
struct Tim2_RGB24_Pixel
{
	unsigned char r;
	unsigned char g;
	unsigned char b;
};

struct Tim2_RGB32_Pixel
{
	unsigned int r:8;
	unsigned int g:8;
	unsigned int b:8;
	unsigned int a:8;
};


#define TIM2_ALIGN_16 0 //for FormatId
#define TIM2_ALIGN_128 1 //for FormatId

//pixel formats
#define TIM2_RGB16 1
#define TIM2_RGB24 2
#define TIM2_RGB32 3
#define TIM2_IDTEX4 4
#define TIM2_IDTEX8 5

struct TIM2_FILEHEADER {
	char    FileId[4];      // File ID (always "T", "I", "M", "2")
	unsigned __int8  FormatVersion;  // File format version number
	unsigned __int8  FormatId;       // Format ID
	unsigned __int16 Pictures;       // Number of picture data parts
	char    Reserved[8];    // Padding (always 0x00)
} ;



struct TIM2_PICTUREHEADER{
	unsigned __int32   TotalSize;      // Total size in bytes of all picture data
	unsigned __int32   ClutSize;       // Size in bytes of CLUT data part
	unsigned __int32   ImageSize;      // Size in bytes of image data part
	unsigned __int16 HeaderSize;     // Size in bytes of header part
	unsigned __int16 ClutColors;     // Total number of colors in CLUT data part
	unsigned __int8  PictFormat;     // Picture format ID (always 0)
	unsigned __int8  MipMapTextures; // Number of MIPMAP textures
	unsigned __int8  ClutType;       // CLUT data part type
	unsigned __int8  ImageType;      // Image data part type
	unsigned __int16 ImageWidth;     // Picture width
	unsigned __int16 ImageHeight;    // Picture height
	unsigned __int64  GsTex0;         // GS TEX0 register data
	unsigned __int64  GsTex1;         // GS TEX1 register data
	unsigned __int32   GsRegs;         // GS TEXA, FBA, PABE register data
	unsigned __int32   GsTexClut;      // GS TEXCLUT register data
} ;

typedef struct tagTIM2_MIPMAPHEADER {
	unsigned __int64  GsMiptbp1;      // Value set in the GS MIPTBP1 register
	unsigned __int64  GsMiptbp2;      // Value set in the GS MIPTBP2 register
	unsigned __int32  Size[TIM2_MAX_MIPMAPS];        // Array indicating the size in bytes of
	char Padding[4];
                        // the texture data
} TIM2_MIPMAPHEADER;

#define ASURA_TIM2_FILE_ID (('T')+('I'<<8)+('M'<<16)+('2'<<24))
#define ASURA_TIM2_USERDATA_ID (('A')+('S'<<8)+('U'<<16)+('R'<<24))

struct AsuraTim2UserData
{
	bool IsValid() const{return Id == ASURA_TIM2_USERDATA_ID;}

	int Id; // ('A')+('S'<<8)+('U'<<16)+('R'<<24)
	unsigned int Swizzled:1;
	unsigned int Pad:31;
};
#endif
