//
// Tools_Overlay_EnlargedGraph.cpp
//
// Created by RAR - 05/02/2009 
//
// Copyright (C) 2009 Rebellion, All Rights Reserved.
//


#include "Core/Asura.h"
#include "Interface/Tools_Overlay_EnlargedGraph.h"

#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Overlay_System.h"

#include "Tools_Overlays.h"
#include "Load&Save/Tools_ModeDataFile.h"

Asura_Vector_2 Tools_Overlay_EnlargedGraph::s_xOverlayPosition(10.0f,10.0f);
Asura_Vector_2 Tools_Overlay_EnlargedGraph::s_xGraphSize(600.0f, 300.0f);

void Tools_Overlay_EnlargedGraph::Create(const Asura_Dialog* pxGraphSource, Asura_Dialog* pxGraphCopy)
{
	//Close any previousnoverlay
	Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_ENLARGEDGRAPH);
	
	//and create the new one
	LoadSettings();
	const char* szName = pxGraphCopy->GetName().GetString();
	if(!szName || !*szName) szName = "Graph";
	new Tools_Overlay_EnlargedGraph(szName, pxGraphSource, pxGraphCopy);
}

void Tools_Overlay_EnlargedGraph::CloseOverlay(const Asura_Dialog* pxGraphSource)
{
	//see if the overlay exists
	Asura_Overlay* pxOverlay = Asura_Overlay_System::GetOverlay(TOOLS_OVERLAY_ENLARGEDGRAPH);
	if(!pxOverlay) return;

	//see if it was opened for this dialog
	Tools_Overlay_EnlargedGraph* pxGraphOverlay = (Tools_Overlay_EnlargedGraph*) pxOverlay;
	if(pxGraphOverlay->m_pxGraphSource != pxGraphSource) return;

	Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_ENLARGEDGRAPH);
}


Tools_Overlay_EnlargedGraph::Tools_Overlay_EnlargedGraph(const char* szName, const Asura_Dialog* pxGraphSource, Asura_Dialog* pxGraphCopy)
:Asura_Overlay(szName, false, &s_xOverlayPosition)
,m_pxGraphDialog(pxGraphCopy)
,m_pxGraphSource(pxGraphSource)
{
	AddDialog(pxGraphCopy);

	Asura_Dialog* pxDialog = new Asura_Resize_Dialog(pxGraphCopy, &s_xGraphSize);
	AddDialog(pxDialog);

	m_uID = TOOLS_OVERLAY_ENLARGEDGRAPH;
	Asura_Overlay_System::SubmitNewOverlay(this);
}

Tools_Overlay_EnlargedGraph::~Tools_Overlay_EnlargedGraph()
{
	SaveSettings();
}

void Tools_Overlay_EnlargedGraph::PerFrameUpdate()
{
	//Update enabled status based on the original graph
	m_pxGraphDialog->SetEnabled(m_pxGraphSource->IsEnabled());

	Asura_Overlay::PerFrameUpdate();
}

void Tools_Overlay_EnlargedGraph::LoadSettings()
{
	LoadOrSaveSettings(TOOLSMODEDATAFILE_LOAD);
}

void Tools_Overlay_EnlargedGraph::SaveSettings()
{
	LoadOrSaveSettings(TOOLSMODEDATAFILE_SAVE);
}

void Tools_Overlay_EnlargedGraph::LoadOrSaveSettings(int iMode)
{
	//open the file
	Tools_ModeDataFile xFile("GraphOverlay.dat",iMode);

	//tranfer the data
	//(none of these should be removed , and new data should go on the end, otherwise
	//the files will get screwed up)
	xFile.Transfer(s_xOverlayPosition);
	xFile.Transfer(s_xGraphSize);
}

