//
// Tools_GlobalIllumination_EntitySamples.h
//
// Created by KJFL - 2004-06-25 16:16:33
//
// Copyright (C) 2004 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif

#ifndef __TOOLS_GLOBALILLUMINATION_ENTITYSAMPLES_H__
#define __TOOLS_GLOBALILLUMINATION_ENTITYSAMPLES_H__


#include "Core/Asura_Hash.h"
#include "Core/Asura_Maths.h"

class Tools_GlobalIllumination_EntitySample
{
	public:
		
		bool operator==( const Tools_GlobalIllumination_EntitySample& xOther ) const
        {
             // Quick rejection using hash
            if( m_uHash != xOther.m_uHash ) { return false; }

            // Hash will probably be sufficient but we'd better do a proper check
            return m_xPosition.ApproximatelyEquals(xOther.m_xPosition);
		}

		void SetPositionAndHash( const Asura_Vector_3& xPosition )
		{
			m_xPosition = xPosition;

			// We want to group close vertices together with the same hash (so we can tell if vertices are
			// nearly the same, rather than just exactly the same), so we'll offset off the origin and round
			// to the nearest 10m (which puts things in reasonable buckets). The offset copes with artists
			// habit of building to a grid, meaning verts might be on a round-up/down boundary.
			//
			// Testing on levels vs a very slow exact-n-squared-comparision gave exactly the same results.
			//
			int aiHashData[3];
			aiHashData[0] = Asura_Maths::FloatToInt_Near( (xPosition.x + 0.15921f) * 10.0f);
			aiHashData[1] = Asura_Maths::FloatToInt_Near( (xPosition.y + 0.15921f) * 10.0f);
			aiHashData[2] = Asura_Maths::FloatToInt_Near( (xPosition.z + 0.15921f) * 10.0f);

			m_uHash = Asura_GetHashID_Binary( aiHashData, sizeof(aiHashData) );
		}
		
		Asura_Hash_ID GetHashID( void ) const 
		{
			return m_uHash;
		}

		inline const Asura_Vector_3 GetPosition( void ) const
		{
			return m_xPosition;
		}
			
		inline const u_int GetPlaneAxis(void) const
		{
			return m_uSplittingPlane;
		}

		inline void SetSplittingPlane(u_int uSplittingPlane)
		{
			m_uSplittingPlane = (u_short)uSplittingPlane;
		}

		u_int									m_uSplittingPlane;

		Asura_Vector_3							m_xPosition;
		Asura_Hash_ID							m_uHash;

		Asura_SphericalHarmonics_Order2_RGB		m_xSH;

		u_int									m_uModule;
		u_int									m_uVertex;
		u_int									m_uFrameStamp;
};



class Tools_GlobalIllumination_EntitySamples
{
	public:
		static void Initialise( void );
		static void Shutdown( void );
		static void Reset( void );

		static void GetNearestSample( const Asura_Vector_3& xPosition, Asura_SphericalHarmonics_Order2_RGB& xSH );

		
		static void ConstructSamples( void );

		static void LightingMayHaveChanged( void );


		// functions called by Asura_GI_Solution when initialising data
		static u_int GetNumberOfSamples( void )
		{
			return s_uNumberOfSamples;
		}

		static const Tools_GlobalIllumination_EntitySample& GetSample( u_int uSample );

		static u_int GetFrameStamp(){return s_uFrameStamp;}


	private:


		static bool ShouldMakeSampleAtVertex( u_int uModule, u_int uVertex );


		static void ConstructLeftBalancedTree(void);

		static void BalanceSegment(Tools_GlobalIllumination_EntitySample* pxBalancedArray, Tools_GlobalIllumination_EntitySample** ppxOriginalArray, const u_int uIndex, const u_int uStart, const u_int uEnd);

		static void MedianSplit(Tools_GlobalIllumination_EntitySample** ppxArray, const u_int uStart, const u_int uEnd, const u_int uMedian, const u_int uAxis);

				
		
		static Tools_GlobalIllumination_EntitySample*	s_pxSamples;

		static Tools_GlobalIllumination_EntitySample*	s_pxSortedSamples;

		static u_int									s_uNumberOfSamples;

		static u_int									s_uFrameStamp;

		static bool										s_bValid;


		// sample proximity test data

		static Tools_GlobalIllumination_EntitySample**	s_ppxProximalSamples;
		static float*									s_fProximalSampleDistancesSquared;
		static float									s_fMaxDistanceSquared;
		static u_int									s_uMostDistantSample;
		

};



#endif//__TOOLS_GLOBALILLUMINATION_ENTITYSAMPLES_H__
