//
// Tools_Mode_PhotonMapping.cpp
//
// Created by KJL - 09/01/2004 16:04:25
//
// Copyright (C) 2004 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

// This file's header.
#include "Lighting/Tools_Mode_PhotonMapping.h"
#include "Lighting/Tools_EnvironmentalLightTransport.h"
#include "Lighting/Tools_Environment_LOS.h"
#include "Lighting/Tools_PhotonMapping_System.h"
#include "Lighting/Tools_PhotonMapping_System_Internal.h"
#include "Lighting/Tools_PhotonMapping_LightSource.h"
#include "Lighting/Tools_PhotonMapping_LightSource_Object.h"
#include "Lighting/Tools_PhotonMapping_PolygonLightSource.h"
#include "Lighting/Tools_PhotonMapping_PolygonLightSource_Object.h"
#include "Lighting/Tools_PhotonMapping_VertexSampleStamp.h"
#include "Lighting/Tools_GlobalIllumination_LightProbe.h"
#include "Lighting/Tools_GlobalIllumination_EntitySamples.h"
#include "Lighting/Tools_Corona_Object.h"
#include "Lighting/Tools_LightSets.h"

#include "ChunkSystem/Asura_Chunk_Loaders.h"
#include "ChunkSystem/Asura_Chunk_GlobalIllumination.h"
#include "ChunkSystem/Asura_Chunk_Corona.h"
#include "ChunkSystem/Asura_Chunk_Texture.h"
#include "Core/Asura_UserInput.h"
#include "Debugging/Asura_CPUProfile_Renderer.h"
#include "Debugging/Asura_Debug_Timers.h"
#include "Light/Asura_GI_Solution.h"

#include "Entity/Tools_Entity_Object.h"
#include "Entity/Tools_Entity_PhysicsObject.h"

#ifdef ASURA_USE_CONTAINER_SYSTEM
#include "Entity/Container/Tools_Entity_Container.h"
#endif //#ifdef ASURA_USE_CONTAINER_SYSTEM

#include "Load&Save/Tools_SaveAsura.h"
#include "Load&Save/Tools_BatchProcessing.h"

#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Overlay_System.h"
#include "Interface/Asura_Overlay.h"
#include "Interface/Asura_Menubar_System.h"
#include "Interface/Asura_Menubar.h"
#include "Interface/Tools_ColourPicker.h"

#include "Camera/Asura_Camera.h"

// for forcing mesh relight
#include "Rendering/Asura_Module.h"
#include "Rendering/Asura_Text_Output.h"
#include "Mesh/Asura_Mesh.h"

#include "Flags/Tools_Mode_Flags.h"
#include "Load&Save/Tools_ModeDataFile.h"
#include "Load&Save/Tools_AsuraFileContents.h"
#include "Misc/Tools_Timed_Messages.h"
#include "Misc/Tools_Render_Options.h"
#include "Misc/Tools_Render.h"
#include "Misc/Tools_Polygon_Selection.h"
#include "Misc/Tools_Help.h"
#include "Objects/Tools_Object_System.h"
#include "PC/FileSystem/Tools_FileSystem.h"

#include "Texturing/Tools_Texturing_Find.h"

#include "Tools_System.h"
#include "Tools_Overlays.h"

#if defined( ASURA_USE_BOTANICALS)
	#include "Botanical/Asura_Direct3D_Botanicals_Game_Render.h"
#endif
#if defined( ASURA_USE_CARPET)
	#include "Carpet/Asura_Direct3D_Carpet_Game_Render.h"
	#include "Carpet/Asura_Carpet_MeshManager.h"
#endif


///////////////////////////////////////////////////////////////////////////////////////
/*--------------**
** Help Strings **
**--------------*/

static const char* s_apszGIModeHelpStrings [] =
{
	"P :           Toggle polygon/object selection",
	"1 :           Recalculate photons for the selected lights",
	"",
	"While polygon selection is activated :",
	"F :           Apply the GI Sample flag",
	"Shift+F :     Remove the GI Sample flag",
	"< :           Undo flag changes",
	"> :           Redo flag changes",
	NULL
};

static const char* s_szResampleHelp =	"Lighting information is compressed using 'spherical harmonics'. As this is a lossy compression errors can creep in, "
										"e.g. very bright lights causing 'negative' colours on the sides of objects facing away from the light. By tweaking "
										"the 'ambient' and 'directional' scale values below effects such this can be minimised by rebalancing the contributions "
										"- at the cost of contrast. The scale factors are saved per level, and unless you use the 'Disable Relight' tickbox below "
										"the level lighting will update automatically as the values are changed. However, you may need to 'Recalc Entity GI Solution' "
										"manually before in-game objects see any effect.";

///////////////////////////////////////////////////////////////////////////////////////////////////
/*--------------------------------------------**
** Tools_PhotonMapping_RecalculateAll_Overlay **
**--------------------------------------------*/

//Overlay for showing options for use during RecalculateAll
class Tools_PhotonMapping_RecalculateAll_Overlay : public Tools_Modal_Overlay
{
public :
	Tools_PhotonMapping_RecalculateAll_Overlay();	
	bool DoModal();

private :
	virtual void InitialiseOverlay(class Asura_Overlay* pxOverlay);
	virtual void Update();

	static void OkaySelected(int iThis){((Tools_PhotonMapping_RecalculateAll_Overlay*)iThis)->m_bReturnValue = true;}
	static void CancelSelected(int iThis){((Tools_PhotonMapping_RecalculateAll_Overlay*)iThis)->m_bReturnValue = false;}


	bool m_bReturnValue;
	bool m_bMinChanged;
	bool m_bMaxChanged;

};

Tools_PhotonMapping_RecalculateAll_Overlay::Tools_PhotonMapping_RecalculateAll_Overlay()
:Tools_Modal_Overlay("Recalculate All")
{
}

bool Tools_PhotonMapping_RecalculateAll_Overlay::DoModal()
{
	m_bReturnValue = false;
	ShowOverlay();
	return m_bReturnValue;
}

void Tools_PhotonMapping_RecalculateAll_Overlay::InitialiseOverlay(class Asura_Overlay* pxOverlay)
{
	Asura_Vector_2 xOffset = Asura_Vector_2(1.0f,3.0f);
	Asura_Dialog* pxDialog;

	m_bMinChanged = false;
	m_bMaxChanged = false;

	pxDialog = new Asura_Numerical_Dialog("Min Accuracy",xOffset,&Tools_Mode_PhotonMapping::s_uRecalcAll_MinAccuracy,1,TOOLS_PHOTONMAPPING_MAX_ACCURACY,1,"%d",&m_bMinChanged);
	pxDialog->SetHelpText("Minimum accuracy setting to use when recalculating lights.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Max Accuracy",xOffset,&Tools_Mode_PhotonMapping::s_uRecalcAll_MaxAccuracy,1,TOOLS_PHOTONMAPPING_MAX_ACCURACY,1,"%d",&m_bMaxChanged);
	pxDialog->SetHelpText("Maximum accuracy setting to use when recalculating lights.\nIf a light has already been calculated to a greater accuracy level , then it won't be recalculated unless 'Force Recalculate All' is used.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Tickbox_Dialog("Quick ELT",xOffset,&Tools_Mode_PhotonMapping::s_bRecalcAll_QuickELT);
	pxDialog->SetHelpText("Calculates the ELT using the fast version.\nIf a valid full ELT already exists , then it will be left alone.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::LargeSpacing();


	pxDialog = new Asura_Tickbox_Dialog("Force Recalculate All",xOffset,&Tools_Mode_PhotonMapping::s_bRecalcAll_ForceRecalcAll);
	pxDialog->SetHelpText("Forces everything to be recalculated regardless of whether they appear to be valid.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::LargeSpacing();
	

	pxDialog = new Asura_Function_Dialog("Okay",xOffset,OkaySelected,(int)this,&pxOverlay->m_bShouldClose);
	pxOverlay->AddDialog(pxDialog);
	xOffset.x+=6.0f;

	pxDialog = new Asura_Function_Dialog("Cancel",xOffset,CancelSelected,(int)this,&pxOverlay->m_bShouldClose);
	pxOverlay->AddDialog(pxDialog);
	
} 

void Tools_PhotonMapping_RecalculateAll_Overlay::Update()
{
	if(m_bMinChanged)
	{
		Tools_Mode_PhotonMapping::s_uRecalcAll_MaxAccuracy = Asura_Maths::Max(Tools_Mode_PhotonMapping::s_uRecalcAll_MinAccuracy,Tools_Mode_PhotonMapping::s_uRecalcAll_MaxAccuracy);
		m_bMinChanged = false;
	}
	if(m_bMaxChanged)
	{
		Tools_Mode_PhotonMapping::s_uRecalcAll_MinAccuracy = Asura_Maths::Min(Tools_Mode_PhotonMapping::s_uRecalcAll_MinAccuracy,Tools_Mode_PhotonMapping::s_uRecalcAll_MaxAccuracy);
		m_bMaxChanged = false;
	}
}


///////////////////////////////////////////////////////////////////////////////////////////////////
/*--------------------------**
** Tools_Mode_PhotonMapping **
**--------------------------*/

Asura_Vector_2 Tools_Mode_PhotonMapping::s_xModeOverlayPosition(0,0);
Asura_Vector_2 Tools_Mode_PhotonMapping::s_xLightPropertiesOverlayPosition(0,20.0f);
Asura_Handle	Tools_Mode_PhotonMapping::s_iCurrentLightSourceHandle;

// need sensible defaults since no longer set in Initialise
bool Tools_Mode_PhotonMapping::s_bHideStaticObjects = false;
bool Tools_Mode_PhotonMapping::s_bHideStaticObjectsChanged = false;

bool Tools_Mode_PhotonMapping::s_bHideContainerObjects = false;
bool Tools_Mode_PhotonMapping::s_bHideContainerObjectsChanged = false;

bool Tools_Mode_PhotonMapping::s_bHideIcons = false;

bool Tools_Mode_PhotonMapping::s_bShowLightRadius = false;


u_int Tools_Mode_PhotonMapping::s_uRecalcAll_MinAccuracy = 1;
u_int Tools_Mode_PhotonMapping::s_uRecalcAll_MaxAccuracy = TOOLS_PHOTONMAPPING_MAX_ACCURACY;
bool Tools_Mode_PhotonMapping::s_bRecalcAll_ForceRecalcAll = false;
bool Tools_Mode_PhotonMapping::s_bRecalcAll_QuickELT = false;

bool Tools_Mode_PhotonMapping::s_bPolygonSelection = false;
bool Tools_Mode_PhotonMapping::s_bShowGISampleFlag = false;

Asura_Vector_2 Tools_Mode_PhotonMapping::s_xLightScaleOverlayPosition(25.0f,2.0f);
float Tools_Mode_PhotonMapping::s_fLightScale_Power = 1.0f;
float Tools_Mode_PhotonMapping::s_fLightScale_Red = 1.0f;
float Tools_Mode_PhotonMapping::s_fLightScale_Green = 1.0f;
float Tools_Mode_PhotonMapping::s_fLightScale_Blue = 1.0f;
bool Tools_Mode_PhotonMapping::s_bLightScale_PolygonLights = true;
bool Tools_Mode_PhotonMapping::s_bLightScale_ParallelLights = true;
bool Tools_Mode_PhotonMapping::s_bLightScale_StandardLights = true;
bool Tools_Mode_PhotonMapping::s_bLightScale_SelectedOnly = false;

bool Tools_Mode_PhotonMapping::s_bInvertColourForShadowLights = true;


Asura_RGBA Tools_Mode_PhotonMapping::s_xGlobalAmbience;
bool Tools_Mode_PhotonMapping::s_bGlobalAmbienceChanged;

bool Tools_Mode_PhotonMapping::s_bDisableRelight = false;


bool Tools_Mode_PhotonMapping::s_bELTParamsChanged = false;
u_int Tools_Mode_PhotonMapping::s_uDirectLightingExtraSharpenFactor;
float Tools_Mode_PhotonMapping::s_fDirectLightingExtraIntensityFactor;
float Tools_Mode_PhotonMapping::s_fInterreflectedScaleFactor;

// User tweakable rescale factors to improve spherical harmonic downsampling
bool Tools_Mode_PhotonMapping::s_bSHResampleParamsChanged = false;
float Tools_Mode_PhotonMapping::s_fSHZerothOrderResampleScale = 1.0f;
float Tools_Mode_PhotonMapping::s_fSHFirstOrderResampleScale = 1.0f;

bool  Tools_Mode_PhotonMapping::s_bRelightRequired = false;
bool  Tools_Mode_PhotonMapping::s_bIgnoreDisableRelightOnNextRelight = false;

Asura_Handle Tools_Mode_PhotonMapping::s_iMenuHandle = ASURA_INVALID_HANDLE;

#ifdef ASURA_USE_TASK_SYSTEM
// KJFL - 8 August 2008 - a test bool controlled by a GUI tickbox
bool Tools_Mode_PhotonMapping::s_bUseMultithreading = true;
#endif

void Tools_Mode_PhotonMapping::Initialise(void)
{
	//load values from .dat file
	TransferModeData(TOOLSMODEDATAFILE_LOAD);
	
	s_xGlobalAmbience = Tools_PhotonMapping_System::GetGlobalAmbience();

	s_bELTParamsChanged = false;
	s_uDirectLightingExtraSharpenFactor = Tools_EnvironmentalLightTransport_Handler::GetDirectLightingExtraSharpenFactor();
	s_fDirectLightingExtraIntensityFactor = Tools_EnvironmentalLightTransport_Handler::GetDirectLightingExtraIntensityFactor();
	s_fInterreflectedScaleFactor = Tools_EnvironmentalLightTransport_Handler::GetInterreflectedScaleFactor();

	s_bSHResampleParamsChanged = false;
	s_fSHZerothOrderResampleScale = Tools_PhotonMapping_System::GetSHZerothOrderResampleScale();
	s_fSHFirstOrderResampleScale = Tools_PhotonMapping_System::GetSHFirstOrderResampleScale();
	
	// KJFL - 2004-07-15 17:01:20 - originally defaulted to showing static objects every
	// time you enter the mode, but the artists will probably prefer to keep it the same
	// between mode changes.

	//	s_bHideStaticObjects = false;
	//	s_bHideStaticObjectsChanged = false;

	CreateModeOverlay();

	s_iCurrentLightSourceHandle = ASURA_INVALID_HANDLE;

	Tools_Polygon_Selection::Activate();

	Tools_Object_System::EnableAllControls(true);
	Tools_Object_System::SetSelectionFilter(TOT_PhotonMapping_LightSource);
	Tools_Object_System::SetSelectionFilter(TOT_PhotonMapping_PolygonLightSource);
	Tools_Object_System::SetSelectionFilter(TOT_LightingTest);
	Tools_Object_System::SetSelectionFilter(TOT_Corona);
	Tools_Object_System::SetSelectionFilter(TOT_ToolsPhoton);

	//create tools objects for coronas...need to do this before the static objects to avoid
	//problems with template updated
	for(int i=0;i<Asura_StaticCorona_System::GetArraySize();i++)
	{
		if(Asura_StaticCorona_System::IsValidCorona(i))
		{
			new Tools_Corona_Object(i);
		}
	}


	//create all the static objects
	Tools_Entity_Object::CreateAllObjects();

	//and hide them
	for(LIF<Tools_Object*> xObjectLIF(&Tools_Object_System::ObjectList);!xObjectLIF.done();xObjectLIF.next())
	{
		if(xObjectLIF()->Type == TOT_PhotonMapping_LightSource || 
		   xObjectLIF()->Type == TOT_LightingTest || 
		   xObjectLIF()->Type == TOT_Corona ||
		   xObjectLIF()->Type == TOT_PhotonMapping_PolygonLightSource ||
		   xObjectLIF()->Type == TOT_Camera ||
		   xObjectLIF()->ShouldPersistBetweenModes() ) continue;

		xObjectLIF()->HiddenFlags |= ObjectHideFlag_Mode;
	}

	//update visibility of objects
	UpdateStaticObjectVisibility();
	UpdateContainerObjectVisibility();


	// after the tools objects etc have been created, reset/initialise the various lighting systems
	// It's imperative the LOS system gets initialised after templated object creation.

	Tools_Environment_LOS::Reset();


	{
		Asura_TrackingHandlePool<Tools_PhotonMapping_LightData>::Iterator xIterator = Tools_PhotonMapping_System::GetLightDataTrackingHandlePool()->GetIterator();

		for (xIterator.Begin(); !xIterator.End(); xIterator.Next())
		{
			new Tools_PhotonMapping_LightSource_Object(xIterator.GetCurrentHandle());
		}
	}
	//create tools objects for polygon lighta
	Tools_PhotonMapping_System::SyncPolygonLightsWithMeshes();
	{
		Asura_TrackingHandlePool<Tools_PhotonMapping_PolygonLightData>::Iterator xIterator = Tools_PhotonMapping_System::GetPolygonLightDataTrackingHandlePool()->GetIterator();

		for (xIterator.Begin(); !xIterator.End(); xIterator.Next())
		{
			new Tools_PhotonMapping_PolygonLightSource_Object(xIterator.GetCurrentHandle());
		}
	}
	Tools_PhotonMapping_LightSource_Object::SetLightSourceObjectsCreated(true);


	Tools_PhotonMapping_VertexSampleStamp::Initialise();

	if(!Tools_Render_Options::IsUsingGlobalIllumination())
	{
		RelightEverything();
	}

	Tools_GlobalIllumination_EntitySamples::Reset();

	Tools_GlobalIllumination_LightProbe::LoadFavoriteShapeList();

	Tools_Mode_Flags::InitialiseForGI();

	Tools_Help::AddHelpTopic("GI Mode",s_apszGIModeHelpStrings);

	//create the menubar
	s_iMenuHandle = Asura_Menubar_System::CreateMenuSection();
	Asura_Menubar_Title* pxTitle;

	pxTitle = Asura_Menubar_System::AddTitle(s_iMenuHandle,"Overlays");
	pxTitle->AddItem("Selection",Tools_Polygon_Selection::ToggleSelectionOverlay,Tools_Polygon_Selection::SelectionOverlayIsOn);
	pxTitle->AddItem("Find",Tools_Texturing_Find::ToggleFindOverlay,Tools_Texturing_Find::FindOverlayIsOn);

	Tools_Object_System::SetTextRenderPos(1.0f,7.0f);//move object system text down a bit to leave room for this mode's text

}

void Tools_Mode_PhotonMapping::Shutdown(void)
{
	//save values to .dat file
	TransferModeData(TOOLSMODEDATAFILE_SAVE);

	// flag the lighting as invalid so that if the user isn't using GI in the other modes, the
	// lighting will default back to the older scheme
	// Note that this could be removed once we go over to the GI solution by default

	// Also if relighting was being disabled , force recalc now, RAR 20/02/2006
	if(!Tools_Render_Options::IsUsingGlobalIllumination() || s_bDisableRelight)
	{
		RelightEverything();
	}
	s_bDisableRelight = false;

	Tools_Help::RemoveHelpTopic("GI Mode");
	
	Tools_Mode_Flags::ShutdownForGI();


	//shutdown the menu
	if(s_iMenuHandle != ASURA_INVALID_HANDLE)
	{
		Asura_Menubar_System::DestroyMenuSection(s_iMenuHandle);
		s_iMenuHandle = ASURA_INVALID_HANDLE;
	}

	Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_MODE);
	Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_LIGHTPROPERTIES);
	Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_LIGHTSCALE);

	Tools_Object_System::DeleteAllModeSpecific();
	Tools_PhotonMapping_LightSource_Object::SetLightSourceObjectsCreated(false);

	//RAR 06/07/04 - Clear selection filter so objects can be selected in other modes.
	Tools_Object_System::ClearSelectionFilter();
	//RAR 06/07/04 - Disable object system controls
	Tools_Object_System::EnableAllControls(false);

	Tools_Polygon_Selection::Deactivate();

	Tools_PhotonMapping_VertexSampleStamp::Shutdown();

	Tools_GlobalIllumination_LightProbe::DeleteFavoriteShapeList();

	//unhide lights/coronas if they were hidden
	Tools_Object_System::SetTypeHidden(TOT_PhotonMapping_PolygonLightSource,ObjectHideFlag_Mode,false);
	Tools_Object_System::SetTypeHidden(TOT_PhotonMapping_LightSource,ObjectHideFlag_Mode,false);
	Tools_Object_System::SetTypeHidden(TOT_Corona,ObjectHideFlag_Mode,false);

	Tools_Object_System::ResetTextRenderPos();

}

void Tools_Mode_PhotonMapping::TransferModeData(int mode)
{
	//open the file
	Tools_ModeDataFile xFile("PhotonMappingMode.dat",mode);

	//tranfer the data
	//(none of these should be removed , and new data should go on the end, otherwise
	//the files will get screwed up)
	xFile.Transfer(s_xModeOverlayPosition);
	xFile.Transfer(s_xLightPropertiesOverlayPosition);
	xFile.Transfer(s_bInvertColourForShadowLights);
}


void Tools_Mode_PhotonMapping::QuickSave(void)
{
	Tools_SaveAsura::SaveEnvironmentalLightTransport();
	Tools_SaveAsura::SaveGlobalIllumination();

	//Some other files may have become modified when entering this mode , as a result of templates
	//being updated. So save those if required.
	if(Tools_SaveAsura::IsLightingModified()) Tools_SaveAsura::SaveLights();
	if(Tools_SaveAsura::IsEntityModified()) Tools_SaveAsura::SaveEntities();
	if(Tools_SaveAsura::IsMainFileModified()) Tools_SaveAsura::QuickSaveFile();
	if(Tools_SaveAsura::IsSoundsModified()) Tools_SaveAsura::SaveSounds();

}
bool Tools_Mode_PhotonMapping::IsModified()
{
	return	Tools_SaveAsura::IsEnvironmentalLightTransportModified() || 
			Tools_SaveAsura::IsGlobalIlluminationModified() ||
			Tools_SaveAsura::IsLightingModified();
}



void Tools_Mode_PhotonMapping::Update(void)
{
//	if (g_bNumberOfBouncesChanged)
//	{
//		Tools_PhotonMapping_System::SetNumberOfBounces(Asura_Maths::FloatToInt(g_fNumberOfBounces));
//		g_bNumberOfBouncesChanged = false;
//	}
//
//	if (g_bNumberOfPhotonCastsChanged)
//	{
//		Tools_PhotonMapping_System::SetNumberOfPhotonCast(Asura_Maths::FloatToInt(g_fNumberOfPhotonCasts));
//		g_bNumberOfPhotonCastsChanged = false;
//	}
//
//	if (g_bSampleRadiusChanged)
//	{
//		Tools_PhotonMapping_System::SetSampleRadius(g_fSampleRadius);
//		g_bSampleRadiusChanged = false;
//	}
//
//	if (g_bParallelBounceScaleChanged)
//	{
//		Tools_PhotonMapping_System::SetParallelBounceScale(g_fParallelBounceScale);
//		g_bParallelBounceScaleChanged = false;
//		RelightEverything();
//	}

	
	if(s_bELTParamsChanged)
	{
		Tools_EnvironmentalLightTransport_Handler::SetDirectLightingExtraSharpenFactor(s_uDirectLightingExtraSharpenFactor);
		Tools_EnvironmentalLightTransport_Handler::SetDirectLightingExtraIntensityFactor(s_fDirectLightingExtraIntensityFactor);
		Tools_EnvironmentalLightTransport_Handler::SetInterreflectedScaleFactor(s_fInterreflectedScaleFactor);
		Tools_EnvironmentalLightTransport::Invalidate();
		Tools_SaveAsura::SetEnvironmentalLightTransportModified();
		s_bELTParamsChanged = false;
	}

	if ( s_bSHResampleParamsChanged )
	{
		Tools_PhotonMapping_System::SetSHZerothOrderResampleScale( s_fSHZerothOrderResampleScale );
		Tools_PhotonMapping_System::SetSHFirstOrderResampleScale( s_fSHFirstOrderResampleScale );
		s_bSHResampleParamsChanged = false;
		s_bRelightRequired = true;
		Tools_SaveAsura::SetGlobalIlluminationModified();
	}
	
	
	if(s_bGlobalAmbienceChanged)
	{
		s_bGlobalAmbienceChanged = false;
		Asura_GI_Solution::ApplyGlobalAmbienceDelta(s_xGlobalAmbience-Tools_PhotonMapping_System::GetGlobalAmbience());
		Tools_PhotonMapping_System::SetGlobalAmbience(s_xGlobalAmbience);
		Tools_Mode_PhotonMapping::RelightEverything(false);
		Tools_SaveAsura::SetGlobalIlluminationModified();
	}


	//Get some values back again ,since they may have changed via the light set overlay
	s_xGlobalAmbience = Tools_PhotonMapping_System::GetGlobalAmbience();



	HandleKeyboard();

	//polygon or object selection currently active?
	if(s_bPolygonSelection)
	{
		Tools_Polygon_Selection::Update();
		
		Tools_Object_System::EnableAllControls(false);

		Tools_Mode_Flags::UpdateForGI();
	}
	else
	{
		Tools_Polygon_Selection::Update(PolygonSelectionUpdate_NoSelection);
		Tools_Object_System::EnableAllControls(true);
	}

	

	if (s_bHideStaticObjectsChanged)
	{
		UpdateStaticObjectVisibility();
	}

	if (s_bHideContainerObjectsChanged)
	{
		UpdateContainerObjectVisibility();
	}

	//update visibility of lights
	Tools_Object_System::SetTypeHidden(TOT_PhotonMapping_LightSource,ObjectHideFlag_Mode,s_bHideIcons);
	Tools_Object_System::SetTypeHidden(TOT_PhotonMapping_PolygonLightSource,ObjectHideFlag_Mode,s_bHideIcons);
	Tools_Object_System::SetTypeHidden(TOT_Corona,ObjectHideFlag_Mode,s_bHideIcons);


	if (s_bRelightRequired)
	{			
		RelightEverythingNow(s_bIgnoreDisableRelightOnNextRelight);
		s_bRelightRequired = false;
		s_bIgnoreDisableRelightOnNextRelight=false;
	}

}

void Tools_Mode_PhotonMapping::RenderWorld_PostToneMap()
{
	//possibly highlight a module
	int iModuleToHighlight = -1;

	//if a polygon light is selected , show the module it uses
	if(Tools_Object_System::SelectedObject && Tools_Object_System::SelectedObject->Type == TOT_PhotonMapping_PolygonLightSource)
	{
		Tools_PhotonMapping_PolygonLightSource_Object* pxObject = (Tools_PhotonMapping_PolygonLightSource_Object*) Tools_Object_System::SelectedObject;
		iModuleToHighlight = pxObject->GetModuleIndex();
	}

	//if no light selected , then highlight the module that has some polygons selected
	if(s_bPolygonSelection && iModuleToHighlight == -1)
	{
		//get index of one of modules selected
		for(int i=0;i<Asura_Module_System::GetNumberOfModules();i++)
		{
			if(Tools_Polygon_Selection::CountSelectedInModule(i))
			{
				iModuleToHighlight = i;
				break;
			}
		}
	}

	if(iModuleToHighlight != -1)
	{
		//do the highlight
		//using the per-polygon highlighting function so that polygon selection
		//will show up correctly over the top
		const Asura_Mesh* pxMesh = Asura_Module_System::GetMeshForModule(iModuleToHighlight);
		for(int i=0;i<pxMesh->NumberOfPolygons;i++)
		{
			Tools_Render::RenderPolygonOutline(i,iModuleToHighlight,0x3f7f7fff);
		}
	}

	if(s_bShowGISampleFlag)
	{
		Tools_Mode_Flags::HighlightPolyFlags();
	}

	if(s_bPolygonSelection)
	{
		Tools_Polygon_Selection::Render();
	}
}

void Tools_Mode_PhotonMapping::RenderUI()
{
	if(s_bPolygonSelection)
	{
		Tools_Polygon_Selection::RenderUI();
	}
	
	
	int iMemForGISamples = Tools_GlobalIllumination_EntitySamples::GetNumberOfSamples()*sizeof(Asura_GI_IrradianceSample);
	Asura_Text_Output::Print(1.0f,3.0f,0xffffffff,0,"GI Sample Memory : %d K",(iMemForGISamples+1023)/1024);

	//show ELT validity
	const char* pszELTString;
	if(!Tools_EnvironmentalLightTransport::DoesEnvironmentMatch())
	{
		pszELTString = "Invalid";
	}
	else if(Tools_EnvironmentalLightTransport::IsQuickData())
	{
		pszELTString = "Valid (Quick)";
	}
	else
	{
		pszELTString = "Valid";
	}
	Asura_Text_Output::Print(1.0f,4.0f,0xffffffff,0,"ELT : %s",pszELTString);

	//show photon map validity
	Tools_PhotonMap_Validity ePhotonValid = Tools_PhotonMapping_System::CheckPhotonMapsAreValid();
	switch(ePhotonValid)
	{
		case TOOLS_PHOTONMAP_INVALID :
			Asura_Text_Output::Print(1.0f,5.0f,0xffffffff,0,"Photon Maps : Invalid");
			break;
		case TOOLS_PHOTONMAP_REDUCEDACCURACY :
			Asura_Text_Output::Print(1.0f,5.0f,0xffffffff,0,"Photon Maps : Valid (Reduced Accuracy)");
			break;
		case TOOLS_PHOTONMAP_VALID :
			Asura_Text_Output::Print(1.0f,5.0f,0xffffffff,0,"Photon Maps : Valid");
			break;
	}

	Asura_Text_Output::Print(1.0f,6.0f,0xffffffff,0,"GI Samples : %s",Asura_GI_Solution::AreSamplesValid() ? "Valid" : "Invalid");


}

void Tools_Mode_PhotonMapping::HandleKeyboard()
{
	if(Asura_Overlay_System::OverlaysHaveKeyboardFocus()) return;
 	if(Asura_UserInput::KeyPressed(ASURA_KEYID_ALT)) return;
 	if(Asura_UserInput::KeyPressed(ASURA_KEYID_CTRL))
 	{
	}
	else if(Asura_UserInput::KeyPressed(ASURA_KEYID_SHIFT))
	{
	}
	else
	{
		if(Asura_UserInput::DebouncedKeyPressed('1'))
		{
			Tools_Mode_PhotonMapping::RecalcPhotonMapsForSelectedLights();
		}
		if(Asura_UserInput::DebouncedKeyPressed('h'))
		{
			s_bHideIcons = !s_bHideIcons;
		}
		if(Asura_UserInput::DebouncedKeyPressed('p'))
		{
			s_bPolygonSelection = !s_bPolygonSelection;
		}
	}
}

void Tools_Mode_PhotonMapping::RecalcEverything(void)
{
	Tools_PhotonMapping_System::RecalculateEverything();
	RelightEverything();
	
	if(Asura_UserInput::KeyPressed(ASURA_KEYID_ESCAPE)) return;
	
	if (!Tools_BatchProcessing::IsGlobalIllumination())
	{
		if(ShowYesNoOverlay("Save these changes now?"))
		{
			Tools_SaveAsura::SaveGlobalIllumination();
			Tools_SaveAsura::SaveMissionGlobalIllumination();
		}
	}

}

void Tools_Mode_PhotonMapping::RecalcAllInvalidLights(void)
{
	if (Tools_PhotonMapping_System::RecalculateAllInvalidLights())
	{
		RelightEverything();
		if(Asura_UserInput::KeyPressed(ASURA_KEYID_ESCAPE)) return;
		if (!Tools_BatchProcessing::IsGlobalIllumination())
		{
			if(ShowYesNoOverlay("Save these changes now?"))
			{
				Tools_SaveAsura::SaveGlobalIllumination();
				Tools_SaveAsura::SaveMissionGlobalIllumination();
			}
		}
	}
}

void Tools_Mode_PhotonMapping::QuickRecalcAllInvalidLights()
{
	//ask for maximum accuracy level to use
	static float s_fAccuracy = 1.0f;
	GetNumber_Overlay xOverlay("Maximum accuracy level to use :",s_fAccuracy,1.0f,(float)TOOLS_PHOTONMAPPING_MAX_ACCURACY,"%.0f");
	if(!xOverlay.DoModal()) return;

	s_fAccuracy = xOverlay.GetNumber();
	if (Tools_PhotonMapping_System::RecalculateAllInvalidLights(1,(u_int)s_fAccuracy))
	{
		RelightEverything();
		if(Asura_UserInput::KeyPressed(ASURA_KEYID_ESCAPE)) return;
		if (!Tools_BatchProcessing::IsGlobalIllumination())
		{
			if(ShowYesNoOverlay("Save these changes now?"))
			{
				Tools_SaveAsura::SaveGlobalIllumination();
				Tools_SaveAsura::SaveMissionGlobalIllumination();
			}
		}
	}
	
}

void Tools_Mode_PhotonMapping::RecalculateAll()
{
	//show dialog box asking for some settings
	Tools_PhotonMapping_RecalculateAll_Overlay xOverlay;
	if(!xOverlay.DoModal()) return;

	RecalculateAllMain();

	//And save the changes if desired
	if(Asura_UserInput::KeyPressed(ASURA_KEYID_ESCAPE)) return;

	if(ShowYesNoOverlay("Save these changes now?"))
	{
		Tools_SaveAsura::SaveEnvironmentalLightTransport();
		Tools_SaveAsura::SaveGlobalIllumination();
		Tools_SaveAsura::SaveMissionGlobalIllumination();
	}

}

void Tools_Mode_PhotonMapping::RecalculateAllMain()
{
	ASURA_CPU_PROFILE_SCOPED_TIMER( "Tools_Mode_PhotonMapping::RecalculateAllMain");
	
	//recalculate ELT
	bool bRecalcELT = s_bRecalcAll_ForceRecalcAll || !Tools_EnvironmentalLightTransport::IsValid();
	if(Tools_EnvironmentalLightTransport::IsQuickData() && !s_bRecalcAll_QuickELT)
	{
		//current ELT is the quick version , and we want full version
		bRecalcELT = true;
	}
	
	if(bRecalcELT)
	{
		ASURA_CPU_PROFILE_SCOPED_TIMER( "Tools_Mode_PhotonMapping::RecalculateAllMain ELT");
	
		if(s_bRecalcAll_QuickELT)
		{
			Tools_EnvironmentalLightTransport::ProcessEnvironment(true);
		}
		else
		{
			Tools_EnvironmentalLightTransport::ProcessEnvironment();
		}
	}
	if(Asura_UserInput::KeyPressed(ASURA_KEYID_ESCAPE)) return;

	u_int uMinAccuracy = Asura_Maths::ClampToRange(s_uRecalcAll_MinAccuracy,1u,TOOLS_PHOTONMAPPING_MAX_ACCURACY);
	u_int uMaxAccuracy = Asura_Maths::ClampToRange(s_uRecalcAll_MaxAccuracy,1u,TOOLS_PHOTONMAPPING_MAX_ACCURACY);

	
	//recalculate photon maps
	if(s_bRecalcAll_ForceRecalcAll)
	{
		Tools_PhotonMapping_System::RecalculateEverything(uMinAccuracy,uMaxAccuracy);
	}
	else
	{
		Tools_PhotonMapping_System::RecalculateAllInvalidLights(uMinAccuracy,uMaxAccuracy);
	}
	if(Asura_UserInput::KeyPressed(ASURA_KEYID_ESCAPE)) return;

	//recalculate entity samples
	Asura_GI_Solution::Tools_InitialiseFromPhotonMappingSystem();
	if(Asura_UserInput::KeyPressed(ASURA_KEYID_ESCAPE)) return;

	//flag prelighting for meshes as invalid
	RelightEverything();
}

void Tools_Mode_PhotonMapping::RecalculateEntityGISolution()
{
	Asura_GI_Solution::Tools_InitialiseFromPhotonMappingSystem();

	if(Asura_UserInput::KeyPressed(ASURA_KEYID_ESCAPE)) return;
	if (!Tools_BatchProcessing::IsGlobalIllumination())
	{
		if(ShowYesNoOverlay("Save these changes now?"))
		{
			Tools_SaveAsura::SaveGlobalIllumination();
			Tools_SaveAsura::SaveMissionGlobalIllumination();
		}
	}

}


void Tools_Mode_PhotonMapping::RecalcPhotonMapsForSelectedLights()
{
	List<Asura_Handle> xLightList;
	List<Asura_Handle> xPolygonLightList;

	// pause profiling to avoid assert
	ASURA_CPUPROFILE_SCOPED_PAUSE

	//get the handles of all the selected lights
	for(LIF<Tools_Object*> xObjectLIF(&Tools_Object_System::SelectedObjectList);!xObjectLIF.done();xObjectLIF.next())
	{
		if(xObjectLIF()->Type == TOT_PhotonMapping_LightSource)
		{
			Tools_PhotonMapping_LightSource_Object* pxObject = (Tools_PhotonMapping_LightSource_Object*) xObjectLIF();
			if(!pxObject->m_bParallelSource)
			{
				xLightList.add_entry(pxObject->m_iLightSourceHandle);
			}
		}
		else if(xObjectLIF()->Type == TOT_PhotonMapping_PolygonLightSource)
		{
			Tools_PhotonMapping_PolygonLightSource_Object* pxObject = (Tools_PhotonMapping_PolygonLightSource_Object*) xObjectLIF();
			xPolygonLightList.add_entry(pxObject->m_iLightSourceHandle);
		}
	}
	//Redo the lighting for any lights we found
	if(xLightList.size() || xPolygonLightList.size())
	{
			
		Tools_SaveAsura::SetGlobalIlluminationModified();
		Tools_SaveAsura::SetMissionGlobalIlluminationModified();
		Tools_PhotonMapping_System::RecalculatePhotonMapForLights(xLightList,xPolygonLightList);
		RelightEverything();
	}
}

void Tools_Mode_PhotonMapping::RecalcELT(void)
{
	Tools_EnvironmentalLightTransport::ProcessEnvironment();
	RelightEverything();

	if(Asura_UserInput::KeyPressed(ASURA_KEYID_ESCAPE)) return;

	if (!Tools_BatchProcessing::IsGlobalIllumination())
	{
		if(ShowYesNoOverlay("Save these changes now?"))
		{
			Tools_SaveAsura::SaveEnvironmentalLightTransport();
		}
	}
}

void Tools_Mode_PhotonMapping::QuickRecalcELT(void)
{
	Tools_EnvironmentalLightTransport::ProcessEnvironment(true);
	RelightEverything();
	if(Asura_UserInput::KeyPressed(ASURA_KEYID_ESCAPE)) return;

	if (!Tools_BatchProcessing::IsGlobalIllumination())
	{
		if(ShowYesNoOverlay("Save these changes now?"))
		{
			Tools_SaveAsura::SaveEnvironmentalLightTransport();
		}
	}
}


void Tools_Mode_PhotonMapping::RelightEverything(bool bIgnoreDisableRelight)
{
	ASURA_CPU_PROFILE_SCOPED_TIMER( "Tools_Mode_PhotonMapping::RelightEverything");

	// DMR 12-06-07
	// This was getting called more than once per frame, causing extreme slowness
	// with the already not-exactly-fast carpet relighting. So I'm just going to set
	// a flag here, and do the actual relight once in Update(), if we're in GI mode	

	if (Tools_System::GetCurrentMode()!=TOOLS_MODE_PHOTONMAPPING)
	{
		RelightEverythingNow(bIgnoreDisableRelight);
	}
	else
	{
		if (bIgnoreDisableRelight)
		    s_bIgnoreDisableRelightOnNextRelight = true;

		s_bRelightRequired = true;
	}
}



void Tools_Mode_PhotonMapping::RelightEverythingNow(bool bIgnoreDisableRelight)
{
	if(s_bDisableRelight && !bIgnoreDisableRelight) return; 

	ASURA_CPU_PROFILE_SCOPED_TIMER( "Tools_Mode_PhotonMapping::RelightEverythingNow");

	Tools_GlobalIllumination_EntitySamples::LightingMayHaveChanged();
	Tools_PhotonMapping_System::CalculateEnvironmentIrradiance();
	{
		for (int m=0; m<Asura_Module_System::GetNumberOfModules(); ++m)
		{
			Asura_Mesh* mesh = Asura_Module_System::GetMeshForModule(m);
			mesh->PrelightingDataValid = false;
		}
	}
	
#if defined( ASURA_USE_BOTANICALS)
	if( true == Asura_Direct3D_Botanicals_Game_Render::HasDataSet())
	{
		Asura_Direct3D_Botanicals_Game_Render::RelightEverything();
	}
#endif
#if defined( ASURA_USE_CARPET)
	if( true == Asura_Direct3D_Carpet_Game_Render::HasDataSet())
	{
		Asura_Direct3D_Carpet_Game_Render::RelightEverything();
	}
	Asura_Carpet_MeshManager::UpdateLighting();
#endif
}


void Tools_Mode_PhotonMapping::CreateLightSource(void)
{
	Tools_PhotonMapping_LightSource_Object* pxObject = Tools_PhotonMapping_LightSource_Object::CreateLightSource(false);
	Tools_Object_System::SelectObject(pxObject);
}

void Tools_Mode_PhotonMapping::CreateParallelLightSource(void)
{
	Tools_PhotonMapping_LightSource_Object* pxObject = Tools_PhotonMapping_LightSource_Object::CreateLightSource(true);
	Tools_Object_System::SelectObject(pxObject);
}

void Tools_Mode_PhotonMapping::CreatePolygonLightSource()
{
	Tools_PhotonMapping_PolygonLightSource_Object* pxObject = Tools_PhotonMapping_PolygonLightSource_Object::CreateLightSource();
	if(pxObject)
	{
		Tools_Object_System::SelectObject(pxObject);
	}
}


void Tools_Mode_PhotonMapping::DestroyCurrentLightSource(void)
{
	// KJL - 22/01/2004 10:31:11 - there are different ways of destroying lights (e.g. you can block
	// select a group of lights and press 'delete') so all the destruction handling is done by the 
	// light source tools objects.
	Tools_Object_System::DestroySelected();

	Tools_SaveAsura::SetGlobalIlluminationModified();
	Tools_SaveAsura::SetMissionGlobalIlluminationModified();
}


void Tools_Mode_PhotonMapping::CreateModeOverlay(void)
{
	Asura_Overlay* pxOverlay;
	pxOverlay = new Asura_Overlay("Photon Mapping Mode",false,&s_xModeOverlayPosition);
	Asura_Vector_2 xOffset = Asura_Vector_2(1.0f,3.0f);
	
	Asura_Dialog* pxDialog;
	
	pxDialog = new Asura_Dialog("Global Ambience :",xOffset);
	pxOverlay->AddDialog(pxDialog);

	xOffset.x = pxDialog->m_xMaxCoord.x + 1.0f;
	pxDialog = new Tools_ColourPicker_Dialog("Global Ambience",xOffset,Asura_Vector_2(1.5f,1.0f),&s_xGlobalAmbience.m_fR,&s_xGlobalAmbience.m_fG,&s_xGlobalAmbience.m_fB,0.5f,&s_bGlobalAmbienceChanged);
	pxDialog->SetHelpText("Sets the base light value for the environment.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();
	xOffset.x = 1.0f;

	if(Tools_LightSets::IsUsingLightSets())
	{
		pxDialog = new Asura_Function_Dialog("Apply To All Light Sets", xOffset, ApplyGlobalAmbienceToAllLightSets);
		pxDialog->SetHelpText("Applies the current global ambience to all light sets");
		pxOverlay->AddDialog(pxDialog);
		xOffset.y += Asura_Dialog::DefaultSpacing();
	}
  	xOffset.y += Asura_Dialog::LargeSpacing()-Asura_Dialog::DefaultSpacing();


	for(u_int i=0; i<Tools_LightSets::GetNumGroups(); i++)
	{
		Asura_Dialog* pxLightSetDialog = pxDialog = new Tools_LightSet_Dialog(xOffset,i,0,0, Tools_LightSets::GetGroupName(i));
		pxDialog->SetHelpText("Sets the current light set to view.\nOnly light sets that have been marked as used can be selected.");
		pxOverlay->AddDialog(pxDialog);

		//put tickbox off to the right of the drop down list
		if(i==0)
		{
			pxDialog = new Asura_Tickbox_Dialog("Hide Other",xOffset, Tools_LightSets::GetHideObjectsInOtherSetsPtr());
			pxDialog->SetHelpText("Hides all lights that aren't part of the current light set.\nThis is equivalent to 'Hide Objects In Other Sets' on the light sets overlay.");
			pxOverlay->AddDialog(pxDialog);
			pxDialog->SetRelativePosition(pxLightSetDialog,Asura_Vector_2(6.0f,0.0f),false,true);
		}

		xOffset.y+=Asura_Dialog::DefaultSpacing();
	}
	xOffset.y += Asura_Dialog::LargeSpacing()-Asura_Dialog::DefaultSpacing();


	//Tab control
	static char* s_aszTabs[6] = {"Create","Recalc","Flagging","ELT","Misc","Resample"};
	Asura_Tabs_Dialog* pxTabsDialog = new Asura_Tabs_Dialog(xOffset,s_aszTabs,6,25);
	pxOverlay->AddDialog(pxTabsDialog);
	xOffset.y = pxTabsDialog->m_xMaxCoord.y + 1.0f;

	float fTabsStartY = xOffset.y;
	float fTabsEndY = xOffset.y;

	pxOverlay->SetCurrentParentDialog(pxTabsDialog);

	//Light creation tab
	pxTabsDialog->SelectTab(0);
		
	
	pxDialog = new Asura_Function_Dialog("Create Light",xOffset,CreateLightSource);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Create Parallel Light",xOffset,CreateParallelLightSource);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Create Polygon Light",xOffset,CreatePolygonLightSource);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();


	pxDialog = new Asura_Function_Dialog("Create Corona",xOffset,Tools_Corona_Object::_CreateCorona);
	pxDialog->SetHelpText("Create a free standing corona.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Create GI Probe",xOffset,CreateLightProbeObject);
	pxDialog->SetHelpText("Creates a test object for previewing the entity lighting.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	Tools_Object_System::AddMissionFileTickbox(pxOverlay, xOffset);
	xOffset.y+=Asura_Dialog::LargeSpacing() - Asura_Dialog::DefaultSpacing();

	//add copy/paste controls
	Tools_Object_System::AddCopyOverlayControls(pxOverlay,xOffset);
	xOffset.y+=Asura_Dialog::LargeSpacing()-Asura_Dialog::DefaultSpacing();

	//End of light creation tab
	if(xOffset.y > fTabsEndY) fTabsEndY = xOffset.y;

	//Light recalc tab
	pxTabsDialog->SelectTab(1);
	xOffset.y = fTabsStartY;

	pxDialog = new Asura_Function_Dialog("Recalc ALL",xOffset,RecalculateAll);
	pxDialog->SetHelpText("Recalculate all invalid lighting data , allowing the accuracy settings to be overidden if desired.\nThis will perform all the recalculate options below (as appropriate).");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::LargeSpacing();

	xOffset.x+= 1.0f;	

	pxDialog = new Asura_Function_Dialog("Recalc Quick Approx. ELT",xOffset,QuickRecalcELT);
	pxDialog->SetHelpText("Does an approximate recalculation of the ELT data , which is required by parallel lights.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Recalc ELT",xOffset,RecalcELT);
	pxDialog->SetHelpText("Does a full recalculation of the ELT data , which is required by parallel lights.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::LargeSpacing();


	
	pxDialog = new Asura_Function_Dialog("Quick Recalc All Invalid LightSources",xOffset,QuickRecalcAllInvalidLights);
	pxDialog->SetHelpText("Recalculates the lighting for all non-parallel lights that are currently flagged as invalid , but using a lower accuracy setting.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Recalc All Invalid LightSources",xOffset,RecalcAllInvalidLights);
	pxDialog->SetHelpText("Recalculates the lighting for all non-parallel lights that are currently flagged as invalid.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Recalc All LightSources",xOffset,RecalcEverything);
	pxDialog->SetHelpText("Recalculates the lighting for all non-parallel lights regardless of validity.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::LargeSpacing();
//

	pxDialog = new Asura_Function_Dialog("Recalc Entity GI Solution",xOffset,RecalculateEntityGISolution);
	pxDialog->SetHelpText("Recalculates the entity lighting data.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Relight Mesh",xOffset,RelightEverything);
	pxDialog->SetHelpText("Recalculates the prelighting on all meshes.\nThis is done automatically by all the other recalculate options anyway.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::LargeSpacing();
	xOffset.x -= 1.0f;

	//End of light recalc tab
	if(xOffset.y > fTabsEndY) fTabsEndY = xOffset.y;

	//Flagging tab
	pxTabsDialog->SelectTab(2);
	xOffset.y = fTabsStartY;

	pxDialog = new Asura_Function_Dialog("Apply GI Sample Flag",xOffset,Tools_Mode_Flags::ApplyFlag);
	pxDialog->SetHelpText("Adds the GI sample flag to the selected polygons");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Remove GI Sample Flag",xOffset,Tools_Mode_Flags::RemoveFlag);
	pxDialog->SetHelpText("Removes the GI sample flag from the selected polygons");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Tickbox_Dialog("Show GI Sample Flag",xOffset,&s_bShowGISampleFlag);
	pxDialog->SetHelpText("Highlights the polygons with the GI sample flag in green");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::LargeSpacing();

	//End of flagging tab
	if(xOffset.y > fTabsEndY) fTabsEndY = xOffset.y;


	//ELT tab
	pxTabsDialog->SelectTab(3);
	xOffset.y = fTabsStartY;

	pxDialog = new Asura_Numerical_Dialog("Direct Light Sharpen",xOffset,&s_uDirectLightingExtraSharpenFactor, 0,9,1,"  %d  ",&s_bELTParamsChanged);
	pxDialog->SetHelpText("Sharpens the effect of direct light , causing the light to fade faster on polygons that don't point directly towards the infinite lights.\nNote that this isn't a linear scale.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();
	

	pxDialog = new Asura_Numerical_Dialog("ELT Scale",xOffset,&s_fDirectLightingExtraIntensityFactor, 0,100.0f,1.0f,"%0.2f",&s_bELTParamsChanged);
	pxDialog->SetHelpText("Acts as a multiplier to all infinite lights.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Interreflected Scale",xOffset,&s_fInterreflectedScaleFactor, 0,100.0f,1.0f,"%0.2f",&s_bELTParamsChanged);
	pxDialog->SetHelpText("Multiplier applied to the intensity of the bounced light when calculating the ELT.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::LargeSpacing();

	pxDialog = new Asura_Function_Dialog("Recalc Quick Approx. ELT",xOffset,QuickRecalcELT);
	pxDialog->SetHelpText("Does an approximate recalculation of the ELT data , which is required by parallel lights.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Recalc ELT",xOffset,RecalcELT);
	pxDialog->SetHelpText("Does a full recalculation of the ELT data , which is required by parallel lights.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();



	//End of ELT tab
	if(xOffset.y > fTabsEndY) fTabsEndY = xOffset.y;
	
	//Misc tab
	pxTabsDialog->SelectTab(4);
	xOffset.y = fTabsStartY;

	pxDialog = new Asura_Function_Dialog("Scale Lights",xOffset,CreateLightScaleOverlay);
	pxDialog->SetHelpText("Brings up an overlay for applying a global brightness scale to all existing lights.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::LargeSpacing();

	pxDialog = new Asura_Function_Dialog("Import Lights",xOffset,ImportLights);
	pxDialog->SetHelpText("Imports and selects all the lights from another GI file.\nAny lights that are in the same position as existing lights will be ignored.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Import Coronas",xOffset,ImportCoronas);
	pxDialog->SetHelpText("Imports and selects all the coronas from another LIT file.\nAny coronas that are in the same position as existing coronas will be ignored.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::LargeSpacing();

	//End of misc tab
	if(xOffset.y > fTabsEndY) fTabsEndY = xOffset.y;

	//Resample tab
	pxTabsDialog->SelectTab(5);
	xOffset.y = fTabsStartY;

	pxDialog = new Asura_Numerical_Dialog("Resample Ambient Scale",xOffset,&s_fSHZerothOrderResampleScale, 0,2.0f,0.01f,"%0.2f",&s_bSHResampleParamsChanged);
	pxDialog->SetHelpText(s_szResampleHelp);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Resample Directional Scale",xOffset,&s_fSHFirstOrderResampleScale, 0,2.0f,0.01f,"%0.2f",&s_bSHResampleParamsChanged);
	pxDialog->SetHelpText(s_szResampleHelp);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::LargeSpacing();


	//End of resample tab
	if(xOffset.y > fTabsEndY) fTabsEndY = xOffset.y;


	//End of tabs
	pxOverlay->SetCurrentParentDialog(0);
	static u_int s_uSelectedTab = 0;
	pxTabsDialog->SetSelectedTabPointer(&s_uSelectedTab);
	xOffset.y = fTabsEndY + 1.0f;


	Asura_Tickbox_Dialog* pxTickDialog = new Asura_Tickbox_Dialog("Disable Relight",xOffset,&s_bDisableRelight);
	pxTickDialog->SetClickedFunction(DisableRelightClicked);
	pxTickDialog->SetHelpText("Prevents prelighting from being invalidated and recalculated in most cases.\nAllows the tools to be more responsive ,but the lighting won't be correct.");
	pxOverlay->AddDialog(pxTickDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Tickbox_Dialog("Hide Static Objects",xOffset,&s_bHideStaticObjects,&s_bHideStaticObjectsChanged);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

#ifdef ASURA_USE_CONTAINER_SYSTEM
	pxDialog = new Asura_Tickbox_Dialog("Hide Container Objects", xOffset, &s_bHideContainerObjects, &s_bHideContainerObjectsChanged);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();
#endif //ASURA_USE_CONTAINER_SYSTEM

	pxDialog = new Asura_Tickbox_Dialog("Hide Icons [H]",xOffset,&(s_bHideIcons));
	pxDialog->SetHelpText("Hides the icons/objects for the lights.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();
	

	pxDialog = new Asura_Tickbox_Dialog("Polygon Selection [P]",xOffset,&s_bPolygonSelection);
	pxDialog->SetHelpText("Enables polygon selection for use with creation of emissive polygon lights.\nWhile active object selection controls will be disabled.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

#ifdef ASURA_USE_TASK_SYSTEM
	pxDialog = new Asura_Tickbox_Dialog("Use Multithreading",xOffset,&s_bUseMultithreading);
	pxDialog->SetHelpText("Turns on multithreading - this is a test feature which will let the ELT calculation run multithreaded. If successful it will be turned on all the time.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();
#endif

	pxOverlay->m_uID = TOOLS_OVERLAY_MODE;

	Asura_Overlay_System::SubmitNewOverlay(pxOverlay,false);
}

void Tools_Mode_PhotonMapping::CreateLightProbeObject()
{
	Tools_GlobalIllumination_LightProbe* pxObject = new Tools_GlobalIllumination_LightProbe(Asura_Camera::GetPosition(),Asura_Camera::GetOrientation());
	Tools_Object_System::SelectObject(pxObject);
	Tools_Object_System::StartSnapToVertex();
}



void Tools_Mode_PhotonMapping::UpdateStaticObjectVisibility(void)
{
	Asura_Hashed_List_It<Tools_Entity_PhysicsObject> xIterator(Tools_Entity_PhysicsObject::GetList());

	for(;!xIterator.Done();xIterator.Next())
	{
		if(s_bHideStaticObjects)
			xIterator.GetCurrent()->HiddenFlags |= ObjectHideFlag_Mode;
		else	
			xIterator.GetCurrent()->HiddenFlags &=~ ObjectHideFlag_Mode;
	}

	s_bHideStaticObjectsChanged = false;
}

void Tools_Mode_PhotonMapping::UpdateContainerObjectVisibility()
{
#ifdef ASURA_USE_CONTAINER_SYSTEM
	Asura_Hashed_List_It<Tools_Entity_Container> xIterator(Tools_Entity_Container::GetList());

	for( ;!xIterator.Done(); xIterator.Next())
	{
		if (s_bHideContainerObjects)
		{
			xIterator.GetCurrent()->HiddenFlags |= ObjectHideFlag_Mode;
		}
		else
		{
			xIterator.GetCurrent()->HiddenFlags &=~ ObjectHideFlag_Mode;
		}
	}
#endif //ASURA_USE_CONTAINER_SYSTEM

	s_bHideContainerObjectsChanged = false;
}


void Tools_Mode_PhotonMapping::CreateLightScaleOverlay()
{
	if(Asura_Overlay_System::DoesOverlayExist(TOOLS_OVERLAY_LIGHTSCALE)) return;
	
	//set scaling values back to 1
	s_fLightScale_Power = 1.0f;
	s_fLightScale_Red = 1.0f;
	s_fLightScale_Green = 1.0f;
	s_fLightScale_Blue = 1.0f;


	//take a copy of the current light values for each object
	Tools_PhotonMapping_LightSource_Object::UpdateInitialLightValues();
	Tools_PhotonMapping_PolygonLightSource_Object::UpdateInitialLightValues();
	
	
	Asura_Overlay* pxOverlay;
	pxOverlay = new Asura_Overlay("Global Light Scale",false,&s_xLightScaleOverlayPosition);
	Asura_Vector_2 xOffset = Asura_Vector_2(1.0f,3.0f);

	Asura_Dialog* pxDialog;

	pxDialog = new Asura_Numerical_Dialog("Power Scale",xOffset,&s_fLightScale_Power, 0,100.0f,0.01f,"%0.2f");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Red Scale",xOffset,&s_fLightScale_Red, 0,100.0f,0.01f,"%0.2f");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Green Scale",xOffset,&s_fLightScale_Green, 0,100.0f,0.01f,"%0.2f");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Blue Scale",xOffset,&s_fLightScale_Blue, 0,100.0f,0.01f,"%0.2f");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::LargeSpacing();

	pxDialog = new Asura_Tickbox_Dialog("Scale Parallel Lights",xOffset,&s_bLightScale_ParallelLights);
	pxDialog->SetHelpText("The scale values will be applied to infinite parallel lights.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Tickbox_Dialog("Scale Standard Lights",xOffset,&s_bLightScale_StandardLights);
	pxDialog->SetHelpText("The scale values will be applied to standard lights (ie not parallel and not polygon).");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Tickbox_Dialog("Scale Polygon Lights",xOffset,&s_bLightScale_PolygonLights);
	pxDialog->SetHelpText("The scale values will be applied to polygon lights.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Tickbox_Dialog("Selected Only", xOffset, &s_bLightScale_SelectedOnly);
	pxDialog->SetHelpText("The scale values will only be applied to the selected lights");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::LargeSpacing();

		
	pxDialog = new Asura_Function_Dialog("Apply",xOffset,ApplyLightScale);
	pxDialog->SetHelpText("Update the lights using the current scale values.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.x = pxDialog->m_xMaxCoord.x + 1.0f;

	pxDialog = new Asura_Function_Dialog("Accept",xOffset,AcceptLightScale,&pxOverlay->m_bShouldClose);
	pxDialog->SetHelpText("Closes this overlay , and leaves the lights as they currently appear.\n(ie they will use the last scale values that were 'applied').");
	pxOverlay->AddDialog(pxDialog);
	xOffset.x = pxDialog->m_xMaxCoord.x + 1.0f;
	

	pxDialog = new Asura_Function_Dialog("Cancel",xOffset,CancelLightScale,&pxOverlay->m_bShouldClose);
	pxDialog->SetHelpText("Closes this overlay , and resets the lights to their original values.");
	pxOverlay->AddDialog(pxDialog);
	

	pxOverlay->m_uID = TOOLS_OVERLAY_LIGHTSCALE;

	Asura_Overlay_System::SubmitNewOverlay(pxOverlay,false);
}



void Tools_Mode_PhotonMapping::ApplyLightScale()
{
	Tools_PhotonMapping_LightSource_Object::ApplyLightScaleValues();
	Tools_PhotonMapping_PolygonLightSource_Object::ApplyLightScaleValues();
}


void Tools_Mode_PhotonMapping::AcceptLightScale()
{
}

void Tools_Mode_PhotonMapping::CancelLightScale()
{
	s_fLightScale_Power = 1.0f;
	s_fLightScale_Red = 1.0f;
	s_fLightScale_Green = 1.0f;
	s_fLightScale_Blue = 1.0f;

	ApplyLightScale();
}



void Tools_Mode_PhotonMapping::ImportCoronas()
{
	Tools_FileSystem_DialogInfo	xFileSelector;

	xFileSelector.DefaultExtension		= ".lit";
	xFileSelector.DefaultFilename		= "";
	xFileSelector.DefaultPath			= "";
	xFileSelector.FileFilter			= "LIT Files\0*.lit;\0";
	xFileSelector.WindowTitle			= "Select file to import coronas from";
	xFileSelector.AllowMultipleSelect	= false;

	Asura_String xFilename = Tools_FileSystem::GetLoadFilename( xFileSelector );

	if(xFilename.IsNull()) return;

	//load the chunks
	Tools_AsuraFileChunkLister xChunkLister;
	if(!xChunkLister.LoadAsuraFile(xFilename.GetString()))
	{
		ShowMessageOverlay("Failed to load %s",xFilename.GetString());
		return;
	}

	//search for the Corona chunk
	const Asura_Chunk_Corona* pxCoronaChunk = (const Asura_Chunk_Corona*) xChunkLister.GetChunkByID(ASURA_CHUNK_CORONA);

	if(!pxCoronaChunk)
	{
		ShowMessageOverlay("Failed to find corona chunk in file");
		return;
	}
	//and texture chunk
	const Asura_Chunk_TextureNames* pxTextureChunk = (const Asura_Chunk_TextureNames*) xChunkLister.GetChunkByID(ASURA_CHUNK_TEXTURENAMES);
	if(!pxTextureChunk)
	{
		ShowMessageOverlay("Failed to find texture chunk in file");
		return;
	}
	

	//deal with texture chunk so index conversions will work for coronas
	pxTextureChunk->Process((const char*)(pxTextureChunk+1));


	//create all the coronas from the chunk
	pxCoronaChunk->ImportFromChunkBuffer((const char*)(pxCoronaChunk+1),0,0);

	List<Tools_Object*> xCoronaList;
	
	//create tools objects for all the new coronas
	for(int i=0;i<Asura_StaticCorona_System::GetArraySize();i++)
	{
		if(Asura_StaticCorona_System::IsValidCorona(i))
		{
			//does this corona already have a tools object?
			Asura_StaticCorona* pxCorona = Asura_StaticCorona_System::GetCorona(i);	
			if(Tools_Corona_Object::GetCoronaObjectFromID(pxCorona->m_uID)) continue;


			//is the corona a copy of an already existing corona?
			//(base this on position)
			bool bDuplicate = false;
			for(int j=0;j<Asura_StaticCorona_System::GetArraySize();j++)
			{
				if(i==j) continue;
				if(!Asura_StaticCorona_System::IsValidCorona(j)) continue;

				Asura_StaticCorona* pxCorona2 = Asura_StaticCorona_System::GetCorona(j);

				if(pxCorona->m_xPosition == pxCorona2->m_xPosition)
				{
					bDuplicate = true;
					break;
				}


			}

			if(bDuplicate)
			{
				//This corona is a copy of an existing one , so delete it
				Asura_StaticCorona_System::DeallocateCorona(i);
			}
			else
			{
				//This is a new corona , so create a tools object for it.
				xCoronaList.add_entry(new Tools_Corona_Object(i));
			}


		}
	}

	//select the newly created coronas
	Tools_Object_System::SelectObjects(xCoronaList,false);
	
	Tools_Timed_Message_System::AddMessage(0xffffff,"%d coronas imported.",xCoronaList.size());
}



void Tools_Mode_PhotonMapping::ImportLights()
{
	Tools_FileSystem_DialogInfo	xFileSelector;

	xFileSelector.DefaultExtension		= ".gi";
	xFileSelector.DefaultFilename		= "";
	xFileSelector.DefaultPath			= "";
	xFileSelector.FileFilter			= "GI Files\0*.gi;\0";
	xFileSelector.WindowTitle			= "Select file to import lights from";
	xFileSelector.AllowMultipleSelect	= false;

	Asura_String xFilename = Tools_FileSystem::GetLoadFilename( xFileSelector );

	if(xFilename.IsNull()) return;

	//load the chunks
	Tools_AsuraFileChunkLister xChunkLister;
	if(!xChunkLister.LoadAsuraFile(xFilename.GetString()))
	{
		ShowMessageOverlay("Failed to load %s",xFilename.GetString());
		return;
	}

	//search for the GI chunk
	const Asura_Chunk_GlobalIllumination* pxLightChunk = (const Asura_Chunk_GlobalIllumination*) xChunkLister.GetChunkByID(ASURA_CHUNK_GLOBALILLUMINATION);

	if(!pxLightChunk)
	{
		ShowMessageOverlay("Failed to find light chunk in file");
		return;
	}

	//create all the lights from the chunk
	pxLightChunk->ImportFromChunkBuffer((const char*)(pxLightChunk+1),0,0);

	//create tools objects for all the new lights
	List<Tools_Object*> xLightList;
	List<Asura_Handle> xLightsToDestroy;

	Asura_TrackingHandlePool<Tools_PhotonMapping_LightData>::Iterator xIterator = Tools_PhotonMapping_System::GetLightDataTrackingHandlePool()->GetIterator();

	for (xIterator.Begin(); !xIterator.End(); xIterator.Next())
	{
		Asura_Handle iHandle = xIterator.GetCurrentHandle();
		Tools_PhotonMapping_TemplateData* pxTemplateData = Tools_PhotonMapping_System::GetTemplateData(iHandle);
		//does a tools object exist for this light yet?
		if(Tools_PhotonMapping_LightSource_Object::GetLightObjectFromID(pxTemplateData->m_uID)) continue;

		Tools_PhotonMapping_LightSource* pxLightSource = Tools_PhotonMapping_System::GetLightSource(iHandle);

		//is the light a copy of an already existing light?
		//(base this on position)
		bool bDuplicate = false;
		Asura_TrackingHandlePool<Tools_PhotonMapping_LightData>::Iterator xIterator2 = Tools_PhotonMapping_System::GetLightDataTrackingHandlePool()->GetIterator();

		for (xIterator2.Begin(); !xIterator2.End(); xIterator2.Next())
		{
			Asura_Handle iHandle2 = xIterator2.GetCurrentHandle();
			if(xIterator.GetCurrentHandle() == iHandle2) continue;
			if(xLightsToDestroy.contains(iHandle2)) continue;
			
			Tools_PhotonMapping_LightSource* pxLightSource2 = Tools_PhotonMapping_System::GetLightSource(iHandle2);

			if(pxLightSource->GetPosition() == pxLightSource2->GetPosition())
			{
				bDuplicate = true;
				break;
			}

		}

		if(bDuplicate)
		{
			//Light is a copy of an existing one... add to list to be destroyed
			xLightsToDestroy.add_entry(iHandle);
		}
		else
		{
			//This is a new light... create a tools object for it
			xLightList.add_entry(new Tools_PhotonMapping_LightSource_Object(xIterator.GetCurrentHandle()));
		}
	}

	//destroy lights marked for killing
	for(LIF<Asura_Handle> xDestroyLIF(&xLightsToDestroy);!xDestroyLIF.done();xDestroyLIF.next())
	{
		Tools_PhotonMapping_System::DestroyLightSource(xDestroyLIF());
	}

	//select the newly created lights
	Tools_Object_System::SelectObjects(xLightList,false);
	
	Tools_Timed_Message_System::AddMessage(0xffffff,"%d lights imported.",xLightList.size());
}


bool Tools_Mode_PhotonMapping::SelectedLightHasInvertedColour()
{
	//check light source objects
	if(Tools_PhotonMapping_LightSource_Object::s_pxThis) return Tools_PhotonMapping_LightSource_Object::s_pxThis->m_bColoursInverted;
	//check polygon light source objects
	if(Tools_PhotonMapping_PolygonLightSource_Object::s_pxThis) return Tools_PhotonMapping_PolygonLightSource_Object::s_pxThis->m_bColoursInverted;

	return false;
}


void Tools_Mode_PhotonMapping::DisableRelightClicked()
{
	//if being switched off , make sure lighting is now up to date
	if(!s_bDisableRelight) RelightEverything();

}

void Tools_Mode_PhotonMapping::HandleMeshChange(int iModuleIndex)
{
	Tools_Mode_Flags::ClearUndo();
}

void Tools_Mode_PhotonMapping::ApplyGlobalAmbienceToAllLightSets()
{
	bool bOkay = ShowYesNoOverlay("Copy the current global ambient value to all other light sets?");
	if(!bOkay) return;


	Asura_RGBA xGlobalAmbience = s_xGlobalAmbience;
	u_int uInitialLightSets = Tools_LightSets::GetCurrentLightSets();

	//Apply the current global ambience to each used light set combination
	for(Tools_UsedLightSetIterator xIt; !xIt.Done(); xIt.Next())
	{
		Tools_LightSets::SetCurrentLightSets(xIt.GetCurrent(), false);

		Asura_GI_Solution::ApplyGlobalAmbienceDelta(xGlobalAmbience-Tools_PhotonMapping_System::GetGlobalAmbience());
		Tools_PhotonMapping_System::SetGlobalAmbience(xGlobalAmbience);
	}
	
	//and restore the original light sets
	Tools_LightSets::SetCurrentLightSets(uInitialLightSets, false);
	Tools_SaveAsura::SetGlobalIlluminationModified();
}