//
// Tools_PhotonMapping_DirectEstimation.h
//
// Created by KJFL - 2004-05-06 11:30:58
//
// Copyright (C) 2004 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif

#ifndef __TOOLS_PHOTONMAPPING_DIRECTESTIMATION_H__
#define __TOOLS_PHOTONMAPPING_DIRECTESTIMATION_H__

#include "Core/Asura_Maths.h"

#include "Core/Asura_Atomic.h"
#include "Rendering/Asura_Module.h"
#include "Mesh/Asura_Mesh.h"
class Tools_PhotonMap_DirectEstimation_Sample
{
	public:
		Tools_PhotonMap_DirectEstimation_Sample()
		{
			m_xIrradiance.Zero();
		}

		Asura_SphericalHarmonics_Order3 m_xIrradiance;
};

class Tools_PhotonMap_DirectEstimation_Module
{
	public:

		Tools_PhotonMap_DirectEstimation_Module()
		{
			m_uNumberOfSamples = 0;
			m_pxSamples = 0;
			m_puSampleIndices = 0;
		}
		~Tools_PhotonMap_DirectEstimation_Module()
		{
			delete [] m_pxSamples;
			delete [] m_puSampleIndices;
		}
		
		void Reset( u_int uNumberOfSamples = 0 )
		{
			if (uNumberOfSamples)
			{
				if ( uNumberOfSamples != m_uNumberOfSamples )
				{
					Asura_Assert(m_uNumberOfSamples==0,"Not expecting to have previously allocated samples");
					
					delete [] m_pxSamples;
					delete [] m_puSampleIndices;
				
					m_pxSamples = new Tools_PhotonMap_DirectEstimation_Sample[uNumberOfSamples]; 

					m_puSampleIndices = new u_int[uNumberOfSamples];
					memset(m_puSampleIndices,0xffffffff,sizeof(u_int)*uNumberOfSamples);

					// set this last, as it's the thing we check before locking
					m_uNumberOfSamples = uNumberOfSamples;
				}
			}
			else
			{
				delete [] m_pxSamples;
				delete [] m_puSampleIndices;
				
				m_pxSamples = 0;
				m_puSampleIndices = 0;
				
				m_uNumberOfSamples = uNumberOfSamples;
			}
		}

		Tools_PhotonMap_DirectEstimation_Sample& AddSample( class Tools_PhotonMap_DirectEstimation* pxPhotonMap,const u_int uSample, const u_int uTotalNumberOfSamples );

		volatile u_int								m_uNumberOfSamples;
		Tools_PhotonMap_DirectEstimation_Sample*	m_pxSamples;//temporary storage of samples while calculating them
		u_int* 										m_puSampleIndices;//indeces into samples in Tools_PhotonMap_DirectEstimation

	
};

class Tools_PhotonMap_DirectEstimation
{
	public:
		Tools_PhotonMap_DirectEstimation()
		{
			m_pxModules = 0;
			m_uNumberOfModules = 0;

			m_uNumberOfSamples = 0;
			m_pxSamples = 0;
		}
		~Tools_PhotonMap_DirectEstimation()
		{
			delete [] m_pxModules;
			delete [] m_pxSamples;
		}

		//! used by the chunksystem when saving
		u_int GetSizeInBytes( void ) const
		{
			u_int uSize = 0;
			
			if (!m_pxModules) return 0;
			
#if !defined( _WIN64)
		ASURA_STATIC_ASSERT( (sizeof(Tools_PhotonMap_DirectEstimation_Module) == 0xC));
#else
		ASURA_STATIC_ASSERT( (sizeof(Tools_PhotonMap_DirectEstimation_Module) == 0x18));
#endif
			//uSize += m_uNumberOfModules * sizeof ( Tools_PhotonMap_DirectEstimation_Module );
			uSize += m_uNumberOfModules * 0xC;
			
			uSize += m_uNumberOfSamples * sizeof ( Tools_PhotonMap_DirectEstimation_Sample );
			uSize += sizeof(u_int);//m_uNumberOfSamples

			for (u_int u=0; u<m_uNumberOfModules; u++)
			{
				uSize += m_pxModules[u].m_uNumberOfSamples * sizeof ( u_int );
			}
			return uSize;
		}

		void Clear()
		{
			delete [] m_pxModules;
			m_pxModules = 0;

			delete [] m_pxSamples;
			m_pxSamples = 0;

			m_uNumberOfModules = 0;
			m_uNumberOfSamples = 0;
		}

		void Reset( u_int uNumberOfModules )
		{
			if ( (uNumberOfModules != m_uNumberOfModules) && (m_pxModules != 0) )
			{
				delete [] m_pxModules;
				m_pxModules = 0;
			}

			m_uNumberOfModules = uNumberOfModules;

			delete [] m_pxSamples;
			m_pxSamples = 0;
			m_uNumberOfSamples = 0;


			if (!m_pxModules)
			{
				m_pxModules = new Tools_PhotonMap_DirectEstimation_Module[uNumberOfModules];
			}
		
			for (u_int u=0; u<m_uNumberOfModules; u++)
			{
				m_pxModules[u].Reset(); //Asura_Module_System::GetMeshForModule(u)->NumberOfVertices
			}
		}

		bool ProcessFromChunk(Asura_Handle hFile,u_int uNumberOfModules ,u_int uPhotonMapVersion);
		static void SkipFromChunkBuffer(const char*& pcChunkBuffer,u_int uNumberOfModules ,u_int uPhotonMapVersion);
		void SaveToChunkStream(class Asura_Chunk_Stream& xStream) const;	


		//Checks to see if vertex and module index are in bounds of the data that the light has
		//(which may be paritally out of date)
		bool IsVertexAndModuleIndexInRange( const u_int uVertex, const u_int uModule ) const
		{
			if(!m_pxModules || !m_pxSamples) return true;//returning true to stop lights that haven't been done at all yet from messing things up big time
			if(uModule >= m_uNumberOfModules) return false;
			if (!m_pxModules[uModule].m_puSampleIndices) return true;
			if(uVertex>=m_pxModules[uModule].m_uNumberOfSamples) return false;
			return true;

		}
		
		inline const Tools_PhotonMap_DirectEstimation_Sample* GetSample( const u_int uVertex, const u_int uModule ) const
		{
			if (!m_pxModules || !m_pxSamples) return 0;
			if(uModule >= m_uNumberOfModules) return 0;
			if (!m_pxModules[uModule].m_puSampleIndices) return 0;
			if(uVertex>=m_pxModules[uModule].m_uNumberOfSamples) return 0;

			u_int uIndex = m_pxModules[uModule].m_puSampleIndices[uVertex];
			if(uIndex == 0xffffffff) return 0;

			return &m_pxSamples[uIndex];
		}
#if 0
		void AddPhotonHit(const Asura_Vector_3& xPosition, const Asura_Vector_3& xDirection, const float fEnergy, const float fBandwidth);
#else
		void AddPhotonHit(const u_int uPolyIndex, const u_int uModuleIndex, const Asura_Vector_3& xPosition, const Asura_Vector_3& xDirection, const float fEnergy, const float fBandwidth );
		void AddPhotonHit(const u_int uPolyIndex, const u_int uModuleIndex, const Asura_Vector_3& xPosition, const Asura_Vector_3& xDirection, const float fEnergy, const float fBandwidth, bool bUseStamp );

#endif

		u_int AllocateSampleIndex()
		{
			Asura_Assert(!m_pxSamples,"Shouldn't be doing this once the samples have been allocated");
			return Asura_Atomic::Increment(m_uNumberOfSamples)-1;
		}

		//copies all the individual samples from the module data into the single array here
		void TransferSamplesFromModules();

		//void AddPhotonHit(const u_int uPolyIndex, const u_int uModuleIndex, const Asura_Vector_3& xPosition, const Asura_Vector_3& xDirection, const float fEnergy, const float fBandwidth);


		Tools_PhotonMap_DirectEstimation_Module*	m_pxModules;
		u_int										m_uNumberOfModules;

		u_int										m_uNumberOfSamples;
		Tools_PhotonMap_DirectEstimation_Sample*	m_pxSamples;



	private:
		void AddPhotonHit_Recursive(const u_int uPolyIndex, const u_int uModuleIndex, const Asura_Vector_3& xPosition, const Asura_Vector_3& xDirection, const float fEnergy, const float fBandwidth);

};


#endif//__TOOLS_PHOTONMAPPING_DIRECTESTIMATION_H__
