//
// Tools_PhotonMapping_PolygonLightSource.cpp
//
// Created by RAR - 28/09/2004
//
// Copyright (C) 2004 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"
#include "Lighting/Tools_PhotonMapping_PolygonLightSource.h"

#include "Mesh/Asura_Mesh.h"
#include "Rendering/Asura_Module.h"

static u_int g_uRootNumberOfSamples;
static u_int g_uNumberOfSamples;
static float g_fOneOverNumberOfSamples;


void Tools_PhotonMapping_PolygonLightSource::UpdatePolygonList()
{
	if(m_iModule<=0 || m_iModule>=Asura_Module_System::GetNumberOfModules())
	{
		m_iNumPolygons = 0;
		delete [] m_pxPolyData;
		m_pxPolyData = 0;
	}
	
	Asura_Mesh* pxMesh = Asura_Module_System::GetMeshForModule(m_iModule);

	//count the number of polygons used
	int iNumPolygons = 0;
	if(pxMesh->m_puPolygonLightIDs)
	{
		for(int i=0;i<pxMesh->NumberOfPolygons;i++)
		{
			if(pxMesh->m_puPolygonLightIDs[i] == m_uLightID)
			{
				iNumPolygons++;
			}
		}
	}
	
	if(iNumPolygons>m_iNumPolygons)
	{
		//allocate space for the polygons
		delete [] m_pxPolyData;
		m_pxPolyData = new Tools_PhotonMapping_PolygonData[iNumPolygons];
	}
	m_iNumPolygons = iNumPolygons;
	if(!m_iNumPolygons)
	{
		//No polygons , so this won't produce any light , but will be valid for all that
		SetHasPhotonMap(false);
		SetPhotonMapValidity(5);
		return;
	}

	if(!pxMesh->m_bFaceNormalsValid) pxMesh->CreateFaceNormals();

	//go through the polygon list again and get the indeces
	int iIndex = 0;
	m_fTotalArea = 0;
	Tools_PhotonMapping_PolygonData* pxPolyData = m_pxPolyData;
	for(int i=0;i<pxMesh->NumberOfPolygons;i++)
	{
		if(pxMesh->m_puPolygonLightIDs[i] == m_uLightID)
		{
			m_fTotalArea += pxMesh->CalculatePolyArea(i);
						
			pxPolyData->m_iIndex = i;
			pxPolyData->m_fTotalArea = m_fTotalArea;

			//get normal and 'x' and 'y' axes for polygon
			const Asura_Vector_3& xNorm = pxMesh->NormalsDataPtr[i].FaceNormal;
			Asura_Matrix_3x3 xMat;
			Asura_Maths::OrientationFromDirection(xMat,xNorm);

			pxPolyData->m_xNormal.x = xMat.e13;
			pxPolyData->m_xNormal.y = xMat.e23;
			pxPolyData->m_xNormal.z = xMat.e33;
			pxPolyData->m_xAxis1.x = xMat.e11;
			pxPolyData->m_xAxis1.y = xMat.e21;
			pxPolyData->m_xAxis1.z = xMat.e31;
			pxPolyData->m_xAxis2.x = xMat.e12;
			pxPolyData->m_xAxis2.y = xMat.e22;
			pxPolyData->m_xAxis2.z = xMat.e32;

			pxPolyData++;
			iIndex++;
		}
	}
}

bool Tools_PhotonMapping_PolygonLightSource::ShouldIgnoreLight()
{
	//probably not an ideal place to calculate this , but will do for now
	UpdatePolygonList();

	return !m_iNumPolygons;
}

void Tools_PhotonMapping_PolygonLightSource::SetStratificationLevels(u_int uNumberOfSamples)
{
	g_uRootNumberOfSamples = uNumberOfSamples;
	g_uNumberOfSamples = uNumberOfSamples*uNumberOfSamples;
	g_fOneOverNumberOfSamples = 1.0f/g_uNumberOfSamples;
}

void Tools_PhotonMapping_PolygonLightSource::GetStratifiedPhotonPositionAndDirection(Asura_Vector_3& xPosition, Asura_Vector_3& xDirection, u_int uX, u_int uY)
{
	u_int uIndex = uY*g_uRootNumberOfSamples + uX;
	float fProgress = (uIndex*g_fOneOverNumberOfSamples)*m_fTotalArea;

	Asura_Mesh* pxMesh = Asura_Module_System::GetMeshForModule(m_iModule);

	//determine current polygon
	int iIndex;
	for(iIndex=0;iIndex<m_iNumPolygons-1;iIndex++)
	{
		if(fProgress<m_pxPolyData[iIndex].m_fTotalArea) break;				
	}

	const Asura_Mesh_Polygon* pxPoly = &pxMesh->PolygonDataPtr[m_pxPolyData[iIndex].m_iIndex];

	int v0,v1,v2;
	//get a random point on the polygon
	float fScale1 = Asura_Random::GetFloat();
	float fScale2 = Asura_Random::GetFloat();

	if(fScale1+fScale2 <=1.0f)
	{
		//take point in first triangle
		v0 = pxPoly->VertexID[0];
		v1 = pxPoly->VertexID[1];
		v2 = pxPoly->VertexID[2];
	}
	else
	{
		//if this is a quad take point froms second half
		if(pxPoly->IsQuad())
		{
			v0 = pxPoly->VertexID[2];
			v1 = pxPoly->VertexID[3];
			v2 = pxPoly->VertexID[0];
	
		}
		else
		{
			v0 = pxPoly->VertexID[0];
			v1 = pxPoly->VertexID[1];
			v2 = pxPoly->VertexID[2];
		}
		fScale1 = 1.0f-fScale1;
		fScale2 = 1.0f-fScale2;
	}

	const Asura_Vector_3& xVert0 = pxMesh->VertexDataPtr[v0].Position;
	const Asura_Vector_3& xVert1 = pxMesh->VertexDataPtr[v1].Position;
	const Asura_Vector_3& xVert2 = pxMesh->VertexDataPtr[v2].Position;

	xPosition = xVert0*(1.0f-(fScale1+fScale2)) + xVert1*fScale1 + xVert2*fScale2;
	xPosition += Asura_Module_System::GetPositionOfModule(m_iModule);

	//offset position slightly by normal
	xPosition += m_pxPolyData[iIndex].m_xNormal*0.01f;

	//get direction from normal
	if(m_fAngle)
	{
		//generate a random direction within the spread value
		float fAngleRad = m_fAngle*(Asura_Maths::Pi/180.0f);

		float fTheta = Asura_Random::GetFloat(0,Asura_Maths::Pi*2.0f);
		float fPhiSquare = Asura_Random::GetFloat(0,fAngleRad*fAngleRad);
		float fPhi = Asura_Maths::Sqrt(fPhiSquare);

	
		float fSineTheta = Asura_Maths::Sine(fTheta);
		float fCosineTheta = Asura_Maths::Cosine(fTheta);
		float fSinePhi = Asura_Maths::Sine(fPhi);
		float fCosinePhi = Asura_Maths::Cosine(fPhi);
		
		xDirection = (m_pxPolyData[iIndex].m_xNormal* fCosinePhi)
					+ (m_pxPolyData[iIndex].m_xAxis1 * (fSinePhi * fCosineTheta))
					+ (m_pxPolyData[iIndex].m_xAxis2 * (fSinePhi * fSineTheta));
	}
	else
	{
		xDirection = m_pxPolyData[iIndex].m_xNormal;
	}
}


float Tools_PhotonMapping_PolygonLightSource::GetStratifiedPhotonEnergy()
{
	return g_fOneOverNumberOfSamples * m_fTotalArea;
}

bool Tools_PhotonMapping_PolygonLightSource::SyncWithMesh()
{
	if(m_iModule<0 || m_iModule >= Asura_Module_System::GetNumberOfModules()) return false;

	Asura_Mesh* pxMesh = Asura_Module_System::GetMeshForModule(m_iModule);

	//search the mesh's lighting data for one with the correct ID
	Asura_Mesh_PolygonLight* pxLightData = 0;
	for(int i=0;i<pxMesh->m_iNumberOfPolygonLights;i++)
	{
		if(pxMesh->m_pxPolygonLights[i].m_uLightID == m_uLightID)
		{
			pxLightData = &pxMesh->m_pxPolygonLights[i];
			break;
		}
	}
	if(!pxLightData) return false;


	//Update settings from the mesh
	m_xColour = pxLightData->m_xColour;
	m_fPower = pxLightData->m_fPower;
	m_fAngle = pxLightData->m_fAngle;
	m_uAccuracy = pxLightData->m_uAccuracy;
	m_bBleached = pxLightData->m_bBleached;
	m_bShortRange = pxLightData->m_bShortRange;
	m_bUseRange = pxLightData->m_bUseRange;
	m_fInnerRange = pxLightData->m_fInnerRange;
	m_fOuterRange = pxLightData->m_fOuterRange;
	m_fReflectionChance = pxLightData->m_fReflectionChance;
	m_uLightSets = pxLightData->m_uLightSets;
	
	return true;
}

void Tools_PhotonMapping_PolygonLightSource::InitialiseFromMeshData(int iModule,struct Asura_Mesh_PolygonLight* pxLightData)
{
	m_iModule = iModule;
	m_uLightID = pxLightData->m_uLightID;
	m_xColour = pxLightData->m_xColour;
	m_fPower = pxLightData->m_fPower;
	m_fAngle = pxLightData->m_fAngle;
	m_uAccuracy = pxLightData->m_uAccuracy;
	m_bBleached = pxLightData->m_bBleached;
	m_bShortRange = pxLightData->m_bShortRange;
	m_bUseRange = pxLightData->m_bUseRange;
	m_fInnerRange = pxLightData->m_fInnerRange;
	m_fOuterRange = pxLightData->m_fOuterRange;
	m_fReflectionChance = pxLightData->m_fReflectionChance;
	m_uLightSets = pxLightData->m_uLightSets;

	m_xOrientation.Identity();
	
	Asura_Mesh* pxMesh = Asura_Module_System::GetMeshForModule(iModule);
	
	//choose a default location by looking at the positions of th polygons involved
	m_xPosition.Zero();

	if(pxMesh->m_puPolygonLightIDs)
	{
		for(int i=0;i<pxMesh->NumberOfPolygons;i++)
		{
			if(pxMesh->m_puPolygonLightIDs[i] != m_uLightID) continue;
			
			//found a polygon , position the light slightly in front of it
			Asura_Mesh_Polygon* pxPoly = &pxMesh->PolygonDataPtr[i];
			for(int j=0;j<pxPoly->NumberOfVertices();j++)
			{
				m_xPosition += pxMesh->VertexDataPtr[pxPoly->VertexID[j]].Position;
			}
			m_xPosition /= (float)pxPoly->NumberOfVertices();

			if(!pxMesh->m_bFaceNormalsValid) pxMesh->CreateFaceNormals();

			m_xPosition += pxMesh->NormalsDataPtr[i].FaceNormal*0.25f;
			break;
		}
	}
	//offset by the position of the module
	m_xPosition += Asura_Module_System::GetPositionOfModule(iModule);
	
}
