//
// Tools_PhotonMapping_PolygonLightSource_Object.cpp
//
// Created by RAR - 28/09/2004
//
// Copyright (C) 2004 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"
#include "Lighting/Tools_PhotonMapping_PolygonLightSource_Object.h"

#include "Camera/Asura_Camera.h"
#include "Core/Asura_UserInput.h"
#include "Debugging/Asura_Assert.h"
#include "Interface/Asura_Overlay_System.h"
#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Overlay.h"
#include "Mesh/Asura_Mesh_System.h"
#include "Mesh/Asura_Mesh.h"
#include "Rendering/Asura_Module.h"

#include "Tools_Overlays.h"

#include "Interface/Tools_ColourPicker.h"
#include "Lighting/Tools_Mode_PhotonMapping.h"
#include "Lighting/Tools_PhotonMapping_System.h"
#include "Lighting/Tools_PhotonMapping_PolygonLightSource.h"
#include "Lighting/Tools_LightSets.h"
#include "Load&Save/Tools_SaveAsura.h"
#include "Misc/Tools_Polygon_Selection.h"
#include "Misc/Tools_Modal_Overlay.h"
#include "Misc/Tools_Render.h"
#include "Misc/Tools_Timed_Messages.h"
#include "Objects/Tools_ObjectSystem_Param.h"


static const char* szPHOTONSTATUS_BLANK		= "";
static const char* szPHOTONSTATUS_UPTODATE	= "Photons are up-to-date";	
static const char* szPHOTONSTATUS_INVALID	= "Photons need recalc";

static Asura_String g_xLightStatusString;

List<Tools_PhotonMapping_PolygonLightSource_Object*> Tools_PhotonMapping_PolygonLightSource_Object::s_xObjectList;
Tools_PhotonMapping_PolygonLightSource_Object* Tools_PhotonMapping_PolygonLightSource_Object::s_pxThis = 0;
bool Tools_PhotonMapping_PolygonLightSource_Object::s_bHighlightCurrent = true;
bool Tools_PhotonMapping_PolygonLightSource_Object::s_bHighlightAll = false;

//////////////////////


//These use the hash values as those for normal lights , to allow both types of objects
//to be altered at the same time.
TOOLS_DECLARE_SHARED_PARAM(bool,	0x522A5A8F,	LightSource_ShortRange)
TOOLS_DECLARE_SHARED_PARAM(bool,	0xCBA1C3E4,	LightSource_UseRange)
TOOLS_DECLARE_SHARED_PARAM(bool,	0xA2B112F8,	LightSource_Bleached)
TOOLS_DECLARE_SHARED_PARAM(int,		0x5C6EACC7,	LightSource_Accuracy)
TOOLS_DECLARE_SHARED_PARAM(float,	0xECA3D7F7,	LightSource_Power)
TOOLS_DECLARE_SHARED_PARAM(float,	0x25EAA8F5,	LightSource_InnerRange)
TOOLS_DECLARE_SHARED_PARAM(float,	0x5CC965F0,	LightSource_OuterRange)
TOOLS_DECLARE_SHARED_PARAM(float,	0x88DEBB7B,	PolygonLightSource_Spread)
TOOLS_DECLARE_SHARED_PARAM(float,	0x6A5E38ED,	LightSource_ReflectionChance)

TOOLS_DECLARE_SHARED_PARAM(float,	0xD8D21C03,	LightSource_Red)
TOOLS_DECLARE_SHARED_PARAM(float,	0xEC261DF5,	LightSource_Green)
TOOLS_DECLARE_SHARED_PARAM(float,	0x416A3B28,	LightSource_Blue)
TOOLS_DECLARE_SHARED_PARAM(float,	0xD8D1F86A,	LightSource_Hue)
TOOLS_DECLARE_SHARED_PARAM(float,	0xE6133420,	LightSource_Saturation)
TOOLS_DECLARE_SHARED_PARAM(float,	0xDA78CAEA,	LightSource_Luminance)

static const Asura_Hash_ID g_uHash_LightSource_LightSet0 = 0xBD2D1016; 
static const Asura_Hash_ID g_uHash_LightSource_LightSetLast = g_uHash_LightSource_LightSet0+TOOLS_MAX_LIGHT_SETS-1; 
static Tools_ObjectSystem_SharedValue<bool> g_axParam_LightSource_LightSet[TOOLS_MAX_LIGHT_SETS];


Tools_PhotonMapping_PolygonLightSource_Object::Tools_PhotonMapping_PolygonLightSource_Object(const Asura_Handle iLightSourceHandle)
:Tools_Object(TOT_PhotonMapping_PolygonLightSource)
{
	m_iLightSourceHandle = iLightSourceHandle;
	m_bColoursInverted = false;

	Mesh = Asura_Mesh_System::GetMesh("ToolsShape_Bulb");

	Tools_PhotonMapping_PolygonLightSource* pxLightSource = Tools_PhotonMapping_System::GetPolygonLightSource(m_iLightSourceHandle);

	if (pxLightSource)
	{
		Asura_Vector_3 xColour = pxLightSource->GetColour();

		m_fPower = pxLightSource->GetPower();
		
		if(m_fPower<0 && Tools_Mode_PhotonMapping::s_bInvertColourForShadowLights)
		{
			m_bColoursInverted = true;
			xColour.x = 255.0f - xColour.x;			
			xColour.y = 255.0f - xColour.y;			
			xColour.z = 255.0f - xColour.z;			
		}

		m_xColourPair.Initialise(xColour.x,xColour.y,xColour.z);

		
		m_fAngle = pxLightSource->GetAngle();

		Position = pxLightSource->GetPosition();
		Orientation = pxLightSource->GetOrientation();
		
		m_bBleachedSource = pxLightSource->IsBleached();
		m_bShortRange = pxLightSource->IsShortRange();
	
		m_iAccuracy = pxLightSource->GetAccuracy();

		m_bUseRange = pxLightSource->GetUseRange();
		m_fInnerRange = pxLightSource->GetInnerRange();
		m_fOuterRange = pxLightSource->GetOuterRange();
		m_fReflectionChance = pxLightSource->GetReflectionChance();
		m_uLightSets = pxLightSource->GetLightSets();

		m_fInitialPower = m_fPower;
		m_xInitialColour = xColour;

	}

	//Check to see if the polygons used are from merged entity polygons.
	//If they are , then this light shouldn't be selectable
	m_bUsesEntityPolygons = false;
	if(pxLightSource)
	{
		int iModuleIndex = pxLightSource->GetModuleIndex();
		if(iModuleIndex>=0 || iModuleIndex < Asura_Module_System::GetNumberOfModules()) 
		{

			Asura_Mesh* pxMesh = Asura_Module_System::GetMeshForModule(pxLightSource->GetModuleIndex());
			if(pxMesh->m_puPolygonLightIDs && pxMesh->m_pxPolygonGuids)
			{
				for(int i=0;i<pxMesh->NumberOfPolygons;i++)
				{
					if(pxMesh->m_puPolygonLightIDs[i] == pxLightSource->GetLightID())
					{
						m_bUsesEntityPolygons = (pxMesh->m_pxPolygonGuids[i] != ASURA_GUID_UNREGISTERED);
						break;
					}
				}
			}
		}		
	}


	
	UpdateName();

	m_bChanged = false;
	m_bChanged_NoRelightNeeded = false;
	m_bInnerRangeChanged = false;
	m_bOuterRangeChanged = false;

	s_xObjectList.add_entry(this);
}

Tools_PhotonMapping_PolygonLightSource_Object::~Tools_PhotonMapping_PolygonLightSource_Object()
{
	s_xObjectList.delete_entry(this);
}


void Tools_PhotonMapping_PolygonLightSource_Object::OnSelection()
{
	s_pxThis = this;
	Tools_Object::OnSelection();
	Tools_PhotonMapping_PolygonLightSource* pxLightSource = Tools_PhotonMapping_System::GetPolygonLightSource(m_iLightSourceHandle);

	if (pxLightSource)
	{
		if (Orientation != pxLightSource->GetOrientation())
		{
			pxLightSource->SetOrientation(Orientation);
		}
		
		if (Position != pxLightSource->GetPosition())
		{
			pxLightSource->SetPosition(Position);
		}
		
		//will be set correctly in the per-frame update
		g_xLightStatusString = "";
	}

	CreateLightPropertiesOverlay();
}


void Tools_PhotonMapping_PolygonLightSource_Object::OnDeselection()
{
	m_xColourPair.Update();

	Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_LIGHTPROPERTIES);

	Tools_Object::OnDeselection();

	s_pxThis = 0;
}




void Tools_PhotonMapping_PolygonLightSource_Object::CreateLightPropertiesOverlay()
{
	Asura_Overlay* pxOverlay;
	Asura_Dialog* pxDialog;

	pxOverlay = new Asura_Overlay(GetName(), false, &Tools_Mode_PhotonMapping::s_xLightPropertiesOverlayPosition);
	Asura_Vector_2 xOffset = Asura_Vector_2(1.0f,3.0f);


	//Tab control
	static char* s_aszTabs[2] = {"Main","Light Sets"};
	Asura_Tabs_Dialog* pxTabsDialog = new Asura_Tabs_Dialog(xOffset,s_aszTabs,2);
	pxOverlay->AddDialog(pxTabsDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();

	float fTabsStartY = xOffset.y;
	float fTabsEndY = xOffset.y;

	pxOverlay->SetCurrentParentDialog(pxTabsDialog);	


	//Main Tab
	pxTabsDialog->SelectTab(0);


	pxDialog = new Asura_Dynamic_Dialog( &g_xLightStatusString , xOffset );
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Recalc Photons [1]",xOffset,Tools_Mode_PhotonMapping::RecalcPhotonMapsForSelectedLights);
	pxDialog->SetHelpText("Recalculates the lighting for the currently selected lights.\nShortcut: 1");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::LargeSpacing();

	pxDialog = new Asura_Function_Dialog("Add Polygons",xOffset,AddSelectedPolygons);
	pxDialog->SetHelpText("Add the selected polygons to this light. Only polgons within the same module may be added.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Remove Polygons",xOffset,RemoveSelectedPolygons);
	pxDialog->SetHelpText("Remove the selected polygons from this (or other) lights.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::LargeSpacing();

	pxDialog = new Asura_SharedTickbox_Dialog("Short Range",xOffset,&g_xParam_LightSource_ShortRange);
	pxDialog->SetHelpText("This will make a light that is more accurate at short ranges , and less prone to bleeding through polygons.\nThe power setting effectively uses a different scale when this is used.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Accuracy",xOffset,&g_xParam_LightSource_Accuracy, 1, TOOLS_PHOTONMAPPING_MAX_ACCURACY, 1,"%d");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::LargeSpacing();

	pxDialog = new Asura_Tickbox_Dialog("Invert When Negative",xOffset,&Tools_Mode_PhotonMapping::s_bInvertColourForShadowLights);
	pxDialog->SetHelpText("If ticked then the displayed colours will be inverted when dealing with lights that have negative power (ie ,it will indicate what colour will be left behind).");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();


	xOffset.x += 12.0f;
	pxDialog = new Tools_ColourPicker_Dialog("Light Colour",xOffset,Asura_Vector_2(4.0f,3.0f),&g_xParam_LightSource_Red,&g_xParam_LightSource_Green,&g_xParam_LightSource_Blue,255.0f,NULL);
	pxOverlay->AddDialog(pxDialog);
	xOffset.x -= 12.0f;

	pxDialog = new Asura_Numerical_Dialog("Red",xOffset,&g_xParam_LightSource_Red, 0,255.0f,1.0f,"%0.0f");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Green",xOffset,&g_xParam_LightSource_Green, 0,255.0f,1.0f,"%0.0f");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Blue",xOffset,&g_xParam_LightSource_Blue, 0,255.0f,1.0f,"%0.0f");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::LargeSpacing();

	pxDialog = new Asura_Numerical_Dialog("Hue",xOffset,&g_xParam_LightSource_Hue, 0,100.0f,1.0f,"%0.0f");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Saturation",xOffset,&g_xParam_LightSource_Saturation, 0,100.0f,1.0f,"%0.0f");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Luminance",xOffset,&g_xParam_LightSource_Luminance, 0,100.0f,1.0f,"%0.0f");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::LargeSpacing();

	pxDialog = new Asura_Numerical_Dialog("Power",xOffset,&g_xParam_LightSource_Power, -9999999.0f,9999999.0f,0.1f,"%0.3f");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::LargeSpacing();

	pxDialog = new Asura_Numerical_Dialog("Spread",xOffset,&g_xParam_PolygonLightSource_Spread,0.0f,90.0f,10.0f,"%.1f");
	pxDialog->SetHelpText("Spread of light in degrees , determines how much away from parallel rays can be cast");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::LargeSpacing();

	const char* pszRangeHelpText = "If 'Use Range' is enabled then the distance that photons can travel are limited by the outer range value.\nThe photons will have maximum brightness until they have travelled the 'Inner Range' distance , and will then have a linear falloff until the Outer Range is hit.";

	pxDialog = new Asura_SharedTickbox_Dialog("Use Range",xOffset,&g_xParam_LightSource_UseRange);
	pxDialog->SetHelpText(pszRangeHelpText);
	pxOverlay->AddDialog(pxDialog);

	xOffset.x = pxDialog->m_xMaxCoord.x + 1.0f;
	pxDialog = new Asura_Tickbox_Dialog("Show",xOffset,&Tools_Mode_PhotonMapping::s_bShowLightRadius);
	pxDialog->SetHelpText("Renders a sphere indicating the range of the selected light. (If the light is flagged as using range).");
	pxOverlay->AddDialog(pxDialog);
	xOffset.x = 1.0f;
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Inner Range",xOffset,&g_xParam_LightSource_InnerRange,0.0f,9999.0f,1.0f,"%.1f");
	pxDialog->SetHelpText(pszRangeHelpText);
	pxDialog->SetIsEnabledFunction(IsEnabled_UsesRange);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Outer Range",xOffset,&g_xParam_LightSource_OuterRange,0.0f,9999.0f,1.0f,"%.1f");
	pxDialog->SetHelpText(pszRangeHelpText);
	pxDialog->SetIsEnabledFunction(IsEnabled_UsesRange);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::LargeSpacing();

	pxDialog = new Asura_Numerical_Dialog("Bounce Chance",xOffset,&g_xParam_LightSource_ReflectionChance,0.0f,0.99f,0.01f,"%.2f");
	pxDialog->SetHelpText("Chance of photons emitted by this light bouncing when they hit a polygon.\nNote that if this is set too high , then the overall brightness will fall ,since many photons will exceed the maximum number of bounces.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	
	pxDialog = new Asura_SharedTickbox_Dialog("Bleached",xOffset,&g_xParam_LightSource_Bleached);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::LargeSpacing();

	pxDialog = new Asura_Tickbox_Dialog("Highlight Current",xOffset,&(s_bHighlightCurrent));
	pxDialog->SetHelpText("Highlights all polygons belonging to this light with a green outline");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Tickbox_Dialog("Highlight All",xOffset,&(s_bHighlightAll));
	pxDialog->SetHelpText("Highlights all polygons that belong to a light with a blue outline");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Tickbox_Dialog("Polygon Selection [P]",xOffset,&Tools_Mode_PhotonMapping::s_bPolygonSelection);
	pxDialog->SetHelpText("Enables polygon selection for use with creation of emissive polygon lights.\nWhile active object selection controls will be disabled.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();


	//End of main tab
	if(xOffset.y > fTabsEndY) fTabsEndY = xOffset.y;

	//Light set tab
	pxTabsDialog->SelectTab(1);
	xOffset.y = fTabsStartY;

	//add a checkbox for each light set
	for(u_int i=0;i<TOOLS_MAX_LIGHT_SETS;i++)
	{
		const char* szSetName = Tools_LightSets::GetLightSetNameByIndex(i);
		if(!szSetName) continue;

		g_axParam_LightSource_LightSet[i].SetParamID(g_uHash_LightSource_LightSet0+i);

		pxDialog = new Asura_SharedTickbox_Dialog(szSetName,xOffset,&g_axParam_LightSource_LightSet[i]);
		pxDialog->SetHelpText("Enables this light for this light set.");
		pxOverlay->AddDialog(pxDialog);
		xOffset.y+=Asura_Dialog::DefaultSpacing();
	}

	//End of light set tab
	if(xOffset.y > fTabsEndY) fTabsEndY = xOffset.y;

	//End of tabs
	pxOverlay->SetCurrentParentDialog(0);
	static u_int s_uSelectedTab = 0;
	pxTabsDialog->SetSelectedTabPointer(&s_uSelectedTab);
	xOffset.y = fTabsEndY + 1.0f;

	//set things up so that right clicking on the overlay will be equivalent to right clicking on the object
	//RAR 06/11/2007
	pxOverlay->SetContextMenuCallback(Tools_Object_System::AddContextMenuItemsFromOverlay, this);


	pxOverlay->m_uID = TOOLS_OVERLAY_LIGHTPROPERTIES;
	Asura_Overlay_System::SubmitNewOverlay(pxOverlay,false);
}

void Tools_PhotonMapping_PolygonLightSource_Object::UpdateWhileSelected()
{
	//handle inverting colours for negative lights
	UpdateInvertedColourStatus();

	m_xColourPair.Update();

	if(m_bInnerRangeChanged)
	{
		m_fOuterRange = Asura_Maths::Max(m_fInnerRange,m_fOuterRange);
		m_bInnerRangeChanged = false;
		m_bChanged_NoRelightNeeded =true;
	}
	if(m_bOuterRangeChanged)
	{
		m_fInnerRange = Asura_Maths::Min(m_fInnerRange,m_fOuterRange);
		m_bOuterRangeChanged = false;
		m_bChanged_NoRelightNeeded =true;
	}

	//copy any changes to the asura light data
	UpdateAsuraLight();

	Tools_PhotonMapping_PolygonLightSource* pxLightSource = Tools_PhotonMapping_System::GetPolygonLightSource(m_iLightSourceHandle);
	if (pxLightSource)
	{
		if (pxLightSource->IsPhotonMapCompletelyValid())
		{
			g_xLightStatusString = szPHOTONSTATUS_UPTODATE;
		}
		else if(pxLightSource->IsPhotonMapPartiallyValid())
		{
			char szText[100];
			Asura_PrintfToBfr(szText,"Partially valid (Acc %d)",100,true,pxLightSource->GetPhotonMapValidity());
			g_xLightStatusString = szText;
		}
		else
		{
			g_xLightStatusString = szPHOTONSTATUS_INVALID;
		}
	}

	UpdateIfChanged();

}

void Tools_PhotonMapping_PolygonLightSource_Object::UpdateAsuraLight()
{
	Tools_PhotonMapping_PolygonLightSource* pxLightSource = Tools_PhotonMapping_System::GetPolygonLightSource(m_iLightSourceHandle);

	if (pxLightSource)
	{

		if (m_iAccuracy != static_cast<int>(pxLightSource->GetAccuracy()))
		{
			pxLightSource->SetAccuracy(m_iAccuracy);
		}


		Asura_Vector_3 xColour(m_xColourPair.m_xRGB.m_fR,m_xColourPair.m_xRGB.m_fG,m_xColourPair.m_xRGB.m_fB);

		if(m_bColoursInverted)
		{
			xColour.x = 255.0f - xColour.x;
			xColour.y = 255.0f - xColour.y;
			xColour.z = 255.0f - xColour.z;
		}
		
		pxLightSource->SetColour(xColour);
		pxLightSource->SetPower(m_fPower);
		pxLightSource->SetAngle(m_fAngle);
		pxLightSource->SetBleached(m_bBleachedSource);
		pxLightSource->SetShortRange(m_bShortRange);
		pxLightSource->SetUseRange(m_bUseRange);
		pxLightSource->SetInnerRange(m_fInnerRange);
		pxLightSource->SetOuterRange(m_fOuterRange);
		pxLightSource->SetReflectionChance(m_fReflectionChance);
		pxLightSource->SetLightSets(m_uLightSets);


		//also update the data in the mesh
		Asura_Mesh_PolygonLight* pxLightInfo = GetLightDataFromMesh();
		if(pxLightInfo)
		{
			pxLightInfo->m_xColour = pxLightSource->GetColour();
			pxLightInfo->m_fPower = pxLightSource->GetPower();
			pxLightInfo->m_fAngle = pxLightSource->GetAngle();
			pxLightInfo->m_uAccuracy = pxLightSource->GetAccuracy();
			pxLightInfo->m_bBleached = pxLightSource->IsBleached();
			pxLightInfo->m_bShortRange = pxLightSource->IsShortRange();
			pxLightInfo->m_bUseRange = pxLightSource->GetUseRange();
			pxLightInfo->m_fInnerRange = pxLightSource->GetInnerRange();
			pxLightInfo->m_fOuterRange = pxLightSource->GetOuterRange();
			pxLightInfo->m_fReflectionChance = pxLightSource->GetReflectionChance();
			pxLightInfo->m_uLightSets = pxLightSource->GetLightSets();
			pxLightInfo->m_uLightSets = pxLightSource->GetLightSets();
		}

	}
}

void Tools_PhotonMapping_PolygonLightSource_Object::UpdateInvertedColourStatus() const
{
	bool bShouldInvert = m_fPower<0 && Tools_Mode_PhotonMapping::s_bInvertColourForShadowLights;
	if(bShouldInvert != m_bColoursInverted)
	{
		m_bColoursInverted = !m_bColoursInverted;
		m_xColourPair.m_xRGB.m_fR = 255.0f - m_xColourPair.m_xRGB.m_fR;
		m_xColourPair.m_xRGB.m_fG = 255.0f - m_xColourPair.m_xRGB.m_fG;
		m_xColourPair.m_xRGB.m_fB = 255.0f - m_xColourPair.m_xRGB.m_fB;

		m_xInitialColour.x = 255.0f-m_xInitialColour.x;
		m_xInitialColour.y = 255.0f-m_xInitialColour.y;
		m_xInitialColour.z = 255.0f-m_xInitialColour.z;

		m_xColourPair.Update();
	}
	
}


Asura_Mesh_PolygonLight* Tools_PhotonMapping_PolygonLightSource_Object::GetLightDataFromMesh()
{
	Tools_PhotonMapping_PolygonLightSource* pxLightSource = Tools_PhotonMapping_System::GetPolygonLightSource(m_iLightSourceHandle);
	if (!pxLightSource) return 0;

	int iModuleIndex = pxLightSource->GetModuleIndex();
	if(iModuleIndex<0 || iModuleIndex >= Asura_Module_System::GetNumberOfModules()) return 0;

	Asura_Mesh* pxMesh = Asura_Module_System::GetMeshForModule(pxLightSource->GetModuleIndex());

	//search the light data array for the entry for this light
	for(int i=0;i<pxMesh->m_iNumberOfPolygonLights;i++)
	{
		if(pxMesh->m_pxPolygonLights[i].m_uLightID == pxLightSource->GetLightID())
		{
			return &pxMesh->m_pxPolygonLights[i];
		}
	}
	return 0;
}


void Tools_PhotonMapping_PolygonLightSource_Object::Destroy()
{

	Tools_SaveAsura::SetGlobalIlluminationModified();


	//unflag any polygons used by this light
	Tools_PhotonMapping_PolygonLightSource* pxLightSource = Tools_PhotonMapping_System::GetPolygonLightSource(m_iLightSourceHandle);
	if(pxLightSource)
	{
		int iModuleIndex = pxLightSource->GetModuleIndex();
		u_int uLightID = pxLightSource->GetLightID();

		if(iModuleIndex>=0 && iModuleIndex < Asura_Module_System::GetNumberOfModules())
		{
			Asura_Mesh* pxMesh = Asura_Module_System::GetMeshForModule(iModuleIndex);
			if(pxMesh->m_puPolygonLightIDs)
			{
				for(int i=0;i<pxMesh->NumberOfPolygons;i++)
				{
					if(pxMesh->m_puPolygonLightIDs[i] == uLightID)
					{
						pxMesh->m_puPolygonLightIDs[i] = 0;
					}
				}
			}
			for(int i=0;i<pxMesh->m_iNumberOfPolygonLights;i++)
			{
				if(pxMesh->m_pxPolygonLights[i].m_uLightID == uLightID)
				{
					pxMesh->m_pxPolygonLights[i] = pxMesh->m_pxPolygonLights[pxMesh->m_iNumberOfPolygonLights-1];
					pxMesh->m_iNumberOfPolygonLights--;
					if(!pxMesh->m_iNumberOfPolygonLights)
					{
						delete [] pxMesh->m_pxPolygonLights;
						pxMesh->m_pxPolygonLights = 0;
					}
					break;
				}
			}
			Tools_SaveAsura::SetModuleModified(iModuleIndex);

		}
	
	}


	Tools_PhotonMapping_System::DestroyPolygonLightSource(m_iLightSourceHandle);
	Tools_Object::Destroy();

	// force a mesh relight to take account of the changes
	Tools_Mode_PhotonMapping::RelightEverything(false);
}

void Tools_PhotonMapping_PolygonLightSource_Object::Render()
{
	Tools_Object::Render();
	
	if(IsSelected() && Tools_Mode_PhotonMapping::s_bShowLightRadius && m_bUseRange)
	{
		//show the light's inner and outer range
		extern void Render_Sphere( const Asura_Vector_3& xCentre, float fRadius, int iColour );
		if(m_fInnerRange)
		{
			Render_Sphere(Position,m_fInnerRange,0x7f0000ff);
		}
		if(m_fOuterRange)
		{
			Render_Sphere(Position,m_fOuterRange,0x7fff0000);
		}
	}

	Tools_PhotonMapping_PolygonLightSource* pxLightSource = Tools_PhotonMapping_System::GetPolygonLightSource(m_iLightSourceHandle);
	if (!pxLightSource) return;

	if(IsSelected() && (s_bHighlightCurrent || s_bHighlightAll))
	{
		int iModuleIndex = pxLightSource->GetModuleIndex();
		u_int uLightID = pxLightSource->GetLightID();
	
		//deal with polygon highlights
		for(int i=0;i<Asura_Module_System::GetNumberOfModules();i++)
		{
			if(!s_bHighlightAll && (i != iModuleIndex)) continue;
			Asura_Mesh* pxMesh = Asura_Module_System::GetMeshForModule(i);

			if(!pxMesh->m_puPolygonLightIDs) continue;//mesh has no lights , so skip it

			for(int j=0;j<pxMesh->NumberOfPolygons;j++)
			{
				if(!pxMesh->m_puPolygonLightIDs[j]) continue;

				if(s_bHighlightCurrent && pxMesh->m_puPolygonLightIDs[j] == uLightID)
				{
					//polygon used by this light
					Tools_Render::RenderPolygonOutline(j,i,0xff00ff00);//green
				}
				else if(s_bHighlightAll)
				{
					Tools_Render::RenderPolygonOutline(j,i,0xff0000ff);//blue
				}
			}
		}
	}
}


void Tools_PhotonMapping_PolygonLightSource_Object::UpdateIfChanged()
{
	if(m_bChanged || m_bChanged_NoRelightNeeded)
	{
		Tools_SaveAsura::SetGlobalIlluminationModified();

		Tools_PhotonMapping_PolygonLightSource* pxLightSource = Tools_PhotonMapping_System::GetPolygonLightSource(m_iLightSourceHandle);
		if(pxLightSource)
		{
			Tools_SaveAsura::SetModuleModified(pxLightSource->GetModuleIndex());
		}
	}

	if (m_bChanged)
	{
		Tools_Mode_PhotonMapping::RelightEverything(false);
	}
	m_bChanged = false;
	m_bChanged_NoRelightNeeded = false;

}

void Tools_PhotonMapping_PolygonLightSource_Object::UpdateName()
{

	m_xName = "Light";

	Tools_PhotonMapping_PolygonLightSource* pxLightSource = Tools_PhotonMapping_System::GetPolygonLightSource(m_iLightSourceHandle);
	if (pxLightSource)
	{
		int iModuleIndex = pxLightSource->GetModuleIndex();
		if(iModuleIndex>=0 || iModuleIndex < Asura_Module_System::GetNumberOfModules())
		{
			Asura_Mesh* pxMesh = Asura_Module_System::GetMeshForModule(pxLightSource->GetModuleIndex());

			char szName[100];
			Asura_PrintfToBfr(szName,"Light_%s_%d",100,true,pxMesh->Name.GetString(),pxLightSource->GetLightID());
			m_xName = szName;
		}
	}
	
}

void Tools_PhotonMapping_PolygonLightSource_Object::AddSelectedPolygons()
{
	if(s_pxThis)
	{
		s_pxThis->AddSelectedPolygons(false,true);
	}
}

void Tools_PhotonMapping_PolygonLightSource_Object::AddSelectedPolygons(bool bTransferUsedPolys,bool bAskAboutTransfer)
{
	Tools_PhotonMapping_PolygonLightSource* pxLightSource = Tools_PhotonMapping_System::GetPolygonLightSource(m_iLightSourceHandle);
	if (!pxLightSource) return;

	int iModuleIndex = pxLightSource->GetModuleIndex();
	if(iModuleIndex<0 || iModuleIndex>= Asura_Module_System::GetNumberOfModules()) return;

	Asura_Mesh* pxMesh = Asura_Module_System::GetMeshForModule(iModuleIndex);

	//space been allocated for light IDs yet?
	if(!pxMesh->m_puPolygonLightIDs)
	{
		//allocate and initialise them
		pxMesh->m_puPolygonLightIDs = new u_int[pxMesh->NumberOfPolygons];
		for(int i=0;i<pxMesh->NumberOfPolygons;i++)
		{
			pxMesh->m_puPolygonLightIDs[i] = 0;
		}
	}

	u_int uLightID = pxLightSource->GetLightID();

	//iterate over the selected polygons , marking them as belonging to this light
	PolygonID xPoly;
	Tools_Polygon_Selection::StartIteration();

	int iCount = 0;
	int iWrongModuleCount = 0;
	while(Tools_Polygon_Selection::GetNextSelected(&xPoly))
	{
		if(xPoly.ModuleIndex != iModuleIndex)
		{
			iWrongModuleCount++;
			continue;
		}

		if(pxMesh->m_puPolygonLightIDs[xPoly.PolyIndex] && pxMesh->m_puPolygonLightIDs[xPoly.PolyIndex]!=uLightID)
		{
			//polygon used by another light , okay to transfer it?
			if(bAskAboutTransfer)
			{
				bAskAboutTransfer = false;
				bTransferUsedPolys = ShowYesNoOverlay("Some of the polygons are used by another light. Transfer them to this light?");
			}
			if(!bTransferUsedPolys) continue;

			//invalidate photon map for the light that is losing polyons
			Tools_PhotonMapping_PolygonLightSource* pxOtherLightSource = Tools_PhotonMapping_System::GetPolygonLightSourceFromID(iModuleIndex,pxMesh->m_puPolygonLightIDs[xPoly.PolyIndex]);
			if(pxOtherLightSource) pxOtherLightSource->SetPhotonMapValidity(0);
		}
		if(pxMesh->m_puPolygonLightIDs[xPoly.PolyIndex] != uLightID)
		{
			pxMesh->m_puPolygonLightIDs[xPoly.PolyIndex] = uLightID;
			pxLightSource->SetPhotonMapValidity(0);
			iCount++;
		}
	}

	//TBD , deal with lights that have lost all their polygons

	Tools_SaveAsura::SetModuleModified(iModuleIndex);
	Tools_SaveAsura::SetGlobalIlluminationModified();

	Tools_Timed_Message_System::AddMessage(0xffffff,"%d polygon(s) added to the light.",iCount);
	if(iWrongModuleCount)
	{
		Tools_Timed_Message_System::AddMessage(0xff0000,"%d polygon(s) were ignored because they weren't in the correct module.",iWrongModuleCount);
	}

}

void Tools_PhotonMapping_PolygonLightSource_Object::RemoveSelectedPolygons()
{
	if(s_pxThis) s_pxThis->RemoveSelectedPolygonsMain();
}

void Tools_PhotonMapping_PolygonLightSource_Object::RemoveSelectedPolygonsMain()
{
	Tools_PhotonMapping_PolygonLightSource* pxLightSource = Tools_PhotonMapping_System::GetPolygonLightSource(m_iLightSourceHandle);
	if (!pxLightSource) return;

	int iModuleIndex = pxLightSource->GetModuleIndex();
	u_int uLightID = pxLightSource->GetLightID();
	
	bool bAskAboutOtherLights = true;
	bool bRemoveFromOtherLights = false;
	int iCount=0;

	PolygonID xPoly;
	Tools_Polygon_Selection::StartIteration();

	while(Tools_Polygon_Selection::GetNextSelected(&xPoly))
	{
		Asura_Mesh* pxMesh = Asura_Module_System::GetMeshForModule(xPoly.ModuleIndex);
		//does the polygon belong to any light?
		if(!pxMesh->m_puPolygonLightIDs) continue;
		if(!pxMesh->m_puPolygonLightIDs[xPoly.PolyIndex]) continue;

		//does the polygon belong to a different light?
		if(xPoly.ModuleIndex!=iModuleIndex || pxMesh->m_puPolygonLightIDs[xPoly.PolyIndex]!=uLightID)
		{
			if(bAskAboutOtherLights)
			{
				bAskAboutOtherLights = false;
				bRemoveFromOtherLights = ShowYesNoOverlay("Some of the selected polygons belong to other lights.\nRemove these as well?.");
			}
			if(!bRemoveFromOtherLights) continue;
		}
		//invalidate the photon map of the light that is losing its polygons
		Tools_PhotonMapping_PolygonLightSource* pxOtherLightSource = Tools_PhotonMapping_System::GetPolygonLightSourceFromID(xPoly.ModuleIndex,pxMesh->m_puPolygonLightIDs[xPoly.PolyIndex]);
		if(pxOtherLightSource) pxOtherLightSource->SetPhotonMapValidity(0);

		//reset the polygon ID
		pxMesh->m_puPolygonLightIDs[xPoly.PolyIndex] = 0;
		iCount++;


	}

	Tools_SaveAsura::SetModuleModified(iModuleIndex);
	Tools_SaveAsura::SetGlobalIlluminationModified();

	//TBD , deal with lights that have lost all their polygons
	Tools_Timed_Message_System::AddMessage(0xffffff,"%d polygon(s) removed.",iCount);
}


Tools_PhotonMapping_PolygonLightSource_Object* Tools_PhotonMapping_PolygonLightSource_Object::CreateLightSource()
{
	if(Tools_Polygon_Selection::NumSelectedModules == 0)
	{
		ShowMessageOverlay("Polygons for the light need to be selected first");
		return 0;
	}
	if(Tools_Polygon_Selection::NumSelectedModules > 1)
	{
		ShowMessageOverlay("Polygons have been selected from %d modules. Each light must consist of polygons from just one module.",Tools_Polygon_Selection::NumSelectedModules);
		return 0;
	}

	//determine the module selected , and check to see if any of the polygons are already assigned
	//to a light

	int iModuleIndex = -1;
	Asura_Mesh* pxMesh = 0;
	int iPolysAlreadyUsed = 0;
	int iPolysNotYetUsed = 0;

	PolygonID xPoly;
	Tools_Polygon_Selection::StartIteration();

	while(Tools_Polygon_Selection::GetNextSelected(&xPoly))
	{
		if(iModuleIndex == -1)
		{
			iModuleIndex = xPoly.ModuleIndex;
			pxMesh = Asura_Module_System::GetMeshForModule(xPoly.ModuleIndex);
		}
		Asura_Assert(iModuleIndex == xPoly.ModuleIndex,"");

		if(pxMesh->m_puPolygonLightIDs && pxMesh->m_puPolygonLightIDs[xPoly.PolyIndex])
		{
			iPolysAlreadyUsed++;	
		}
		else
		{
			iPolysNotYetUsed++;
		}
	}
	Asura_Assert(pxMesh,"");

	
	bool bTransferPolys = false;
	if(iPolysAlreadyUsed)
	{
		//check whether to go ahead and transfer the polygons
		bTransferPolys = ShowYesNoOverlay("%d polys already assigned to a light. Transfer them to this light?",iPolysAlreadyUsed);
	}

	if(!iPolysNotYetUsed && !bTransferPolys) return 0;


	Asura_Handle iHandle = Tools_PhotonMapping_System::CreatePolygonLightSource();

	if (iHandle == ASURA_INVALID_HANDLE)
	{
		return 0;
	}

	//scan mesh for an unused light ID
	u_int uLightID = 1;
	for(int i=0;i<pxMesh->m_iNumberOfPolygonLights;i++)
	{
		uLightID = Asura_Maths::Max(uLightID,pxMesh->m_pxPolygonLights[i].m_uLightID+1);
	}

	Tools_PhotonMapping_PolygonLightSource* pxLightSource = Tools_PhotonMapping_System::GetPolygonLightSource(iHandle);

	//fill in the module and light ID
	pxLightSource->SetModuleIndex(iModuleIndex);
	pxLightSource->SetLightID(uLightID);

	//Set the initial light sets
	pxLightSource->SetLightSets(Tools_LightSets::GetLightSetsForNewObject(iModuleIndex));

	//const Asura_Vector_3& xPosition = *Asura_Camera::GetPosition();
	//const Asura_Matrix_3x3& xOrientation = *Asura_Camera::GetOrientation();

	Tools_PhotonMapping_PolygonLightSource_Object* pxObject = new Tools_PhotonMapping_PolygonLightSource_Object(iHandle);
	//pxObject->Position = xPosition;
	//pxObject->Orientation = xOrientation;

	Tools_SaveAsura::SetGlobalIlluminationModified();
	Tools_SaveAsura::SetModuleModified(iModuleIndex);

	//allocate enlarged array of light info for mesh
	Asura_Mesh_PolygonLight* pxPolygonLights = new Asura_Mesh_PolygonLight[pxMesh->m_iNumberOfPolygonLights+1];
	for(int i=0;i<pxMesh->m_iNumberOfPolygonLights;i++)
	{
		pxPolygonLights[i] = pxMesh->m_pxPolygonLights[i];
	}
	delete [] pxMesh->m_pxPolygonLights;
	pxMesh->m_pxPolygonLights = pxPolygonLights;
	pxMesh->m_iNumberOfPolygonLights++;

	//fill in details for this light
	Asura_Mesh_PolygonLight* pxLightInfo = &pxMesh->m_pxPolygonLights[pxMesh->m_iNumberOfPolygonLights-1];
	pxLightInfo->m_uLightID = pxLightSource->GetLightID();
	pxLightInfo->m_xColour = pxLightSource->GetColour();
	pxLightInfo->m_fPower = pxLightSource->GetPower();
	pxLightInfo->m_fAngle = pxLightSource->GetAngle();
	pxLightInfo->m_uAccuracy = pxLightSource->GetAccuracy();
	pxLightInfo->m_bBleached = pxLightSource->IsBleached();
	pxLightInfo->m_bShortRange = pxLightSource->IsShortRange();
	pxLightInfo->m_bUseRange = pxLightSource->GetUseRange();
	pxLightInfo->m_fInnerRange = pxLightSource->GetInnerRange();
	pxLightInfo->m_fOuterRange = pxLightSource->GetOuterRange();
	pxLightInfo->m_fReflectionChance = pxLightSource->GetReflectionChance();
	pxLightInfo->m_uLightSets = pxLightSource->GetLightSets();

	//add the polgons to this light
	pxObject->AddSelectedPolygons(bTransferPolys,false);

	//this should set a suitable default position for the light
	pxLightSource->InitialiseFromMeshData(iModuleIndex,pxLightInfo);
	pxObject->Position = pxLightSource->GetPosition();
	pxObject->Orientation = pxLightSource->GetOrientation();

	return pxObject;
}


bool Tools_PhotonMapping_PolygonLightSource_Object::IsSelectable()
{
	return (!m_bUsesEntityPolygons) && Tools_Object::IsSelectable();
}

bool Tools_PhotonMapping_PolygonLightSource_Object::IsHidden()
{
	//Light sets can be hidden by the 'hide by type' overlay
	if(Tools_Object_System::IsHideByLightSetEnabled())
	{
		if(!(m_uLightSets & Tools_Object_System::GetVisibleLightSets())) return true;
	}

	if(Tools_LightSets::ShouldHideObjectsInOtherSets())
	{
		if(!Tools_LightSets::IsMemberOfCurrentLightSets(m_uLightSets)) return true;
	}

	return Tools_Object::IsHidden();
}


int Tools_PhotonMapping_PolygonLightSource_Object::GetModuleIndex()
{
	Tools_PhotonMapping_PolygonLightSource* pxLightSource = Tools_PhotonMapping_System::GetPolygonLightSource(m_iLightSourceHandle);
	if (!pxLightSource) return -1;

	int iModuleIndex = pxLightSource->GetModuleIndex();
	if(iModuleIndex<0 || iModuleIndex >= Asura_Module_System::GetNumberOfModules()) return -1;

	return iModuleIndex;
}


void Tools_PhotonMapping_PolygonLightSource_Object::Copy(const Tools_Object* pxObject)
{
	const Tools_PhotonMapping_PolygonLightSource_Object* pxFrom = (const Tools_PhotonMapping_PolygonLightSource_Object*) pxObject;

	//copy from the other tools object
	m_fPower = pxFrom->m_fPower;
	m_fInitialPower = pxFrom->m_fInitialPower;
	m_fAngle = pxFrom->m_fAngle;
	m_xColourPair = pxFrom->m_xColourPair;
	m_xInitialColour = pxFrom->m_xInitialColour;
	m_bBleachedSource = pxFrom->m_bBleachedSource;
	m_bShortRange = pxFrom->m_bShortRange;
	m_iAccuracy = pxFrom->m_iAccuracy;
	m_bUseRange = pxFrom->m_bUseRange;
	m_fInnerRange = pxFrom->m_fInnerRange;
	m_fOuterRange = pxFrom->m_fOuterRange;
	m_fReflectionChance = pxFrom->m_fReflectionChance;
	m_uLightSets = pxFrom->m_uLightSets;
	m_bColoursInverted = pxFrom->m_bColoursInverted;

	//update the changes in the asura light data
	m_bChanged = true;
	m_bChanged_NoRelightNeeded = true;
	UpdateAsuraLight();
	UpdateIfChanged();

}

Tools_Object* Tools_PhotonMapping_PolygonLightSource_Object::CreateObjectOfThisType() const
{
	return CreateLightSource();	
}

void Tools_PhotonMapping_PolygonLightSource_Object::UpdateInitialLightValues()
{
	for(LIF<Tools_PhotonMapping_PolygonLightSource_Object*> xObjectLIF(&s_xObjectList);!xObjectLIF.done();xObjectLIF.next())
	{
		Tools_PhotonMapping_PolygonLightSource_Object* pxLight = xObjectLIF();
		
		pxLight->m_fInitialPower = pxLight->m_fPower;
		pxLight->m_xInitialColour.x = pxLight->m_xColourPair.m_xRGB.m_fR;
		pxLight->m_xInitialColour.y = pxLight->m_xColourPair.m_xRGB.m_fG;
		pxLight->m_xInitialColour.z = pxLight->m_xColourPair.m_xRGB.m_fB;
	}
}

bool Tools_PhotonMapping_PolygonLightSource_Object::IsEnabled_UsesRange()
{
	if(!s_pxThis) return false;
	return s_pxThis->m_bUseRange;
}

void Tools_PhotonMapping_PolygonLightSource_Object::ApplyLightScaleValues()
{
	for(LIF<Tools_PhotonMapping_PolygonLightSource_Object*> xObjectLIF(&s_xObjectList);!xObjectLIF.done();xObjectLIF.next())
	{
		Tools_PhotonMapping_PolygonLightSource_Object* pxLight = xObjectLIF();
	
		//determine if the scale should be applied to this light
		bool bApplyScale = true;

		if(Tools_Mode_PhotonMapping::s_bLightScale_SelectedOnly)
		{
			if(!pxLight->IsSelected() && !pxLight->IsGroupSelected())
			{
				bApplyScale = false;
			}
		}
		if(!Tools_Mode_PhotonMapping::s_bLightScale_PolygonLights)
		{
			bApplyScale = false;
		}

		//determine the new colour and power values
		Asura_Vector_3 xColour = pxLight->m_xInitialColour;
		float fPower = pxLight->m_fInitialPower;

		if(bApplyScale)
		{
			fPower *= Tools_Mode_PhotonMapping::s_fLightScale_Power;

			if((fPower<0) && !pxLight->m_bColoursInverted)
			{
				//for negative lights do the scaling on invered values
				xColour.x = 255.0f - xColour.x;
				xColour.y = 255.0f - xColour.y;
				xColour.z = 255.0f - xColour.z;
			}

			xColour.x *= Tools_Mode_PhotonMapping::s_fLightScale_Red; 
			xColour.y *= Tools_Mode_PhotonMapping::s_fLightScale_Green; 
			xColour.z *= Tools_Mode_PhotonMapping::s_fLightScale_Blue; 

			if((fPower<0) && !pxLight->m_bColoursInverted)
			{
				//invert back
				xColour.x = 255.0f - xColour.x;
				xColour.y = 255.0f - xColour.y;
				xColour.z = 255.0f - xColour.z;
			}

			xColour.x = Asura_Maths::Min(xColour.x,255.0f);
			xColour.y = Asura_Maths::Min(xColour.y,255.0f);
			xColour.z = Asura_Maths::Min(xColour.z,255.0f);
		}

		//apply these values if different
		if(fPower != pxLight->m_fPower || 
			xColour.x != pxLight->m_xColourPair.m_xRGB.m_fR ||
			xColour.y != pxLight->m_xColourPair.m_xRGB.m_fG ||
			xColour.z != pxLight->m_xColourPair.m_xRGB.m_fB)
		{
			pxLight->m_fPower = fPower;
			pxLight->m_xColourPair.Initialise(xColour.x,xColour.y,xColour.z);
			pxLight->m_bChanged = true;
			pxLight->UpdateAsuraLight();
			pxLight->UpdateIfChanged();
		}
			 
	}
}



bool Tools_PhotonMapping_PolygonLightSource_Object::GetParameter(Asura_Hash_ID uParamID,class Tools_Parameter* pxParam) const
{
	bool bColoursInverted = Tools_Mode_PhotonMapping::SelectedLightHasInvertedColour();

	if(uParamID>=g_uHash_LightSource_LightSet0 && uParamID<=g_uHash_LightSource_LightSetLast)
	{
		u_int uBit = 1 << (uParamID - g_uHash_LightSource_LightSet0);
		pxParam->SetBool((m_uLightSets & uBit) ? true : false);
	}
	else
	{
		switch(uParamID)
		{
			case g_uHash_LightSource_ShortRange :
				pxParam->SetBool(m_bShortRange);
				break;
			case g_uHash_LightSource_UseRange :
				pxParam->SetBool(m_bUseRange);
				break;
			case g_uHash_LightSource_Bleached :
				pxParam->SetBool(m_bBleachedSource);
				break;

			case g_uHash_LightSource_Accuracy :
				pxParam->SetInt(m_iAccuracy);
				break;

			case g_uHash_LightSource_Power :
				pxParam->SetFloat(m_fPower);
				break;

			case g_uHash_LightSource_InnerRange :
				pxParam->SetFloat(m_fInnerRange);
				break;

			case g_uHash_LightSource_OuterRange :
				pxParam->SetFloat(m_fOuterRange);
				break;

			case g_uHash_PolygonLightSource_Spread :
				pxParam->SetFloat(m_fAngle);
				break;

			case g_uHash_LightSource_Red :
				//To avoid getting colours messed up , only allow getting/setting of
				//the if the inverted state matches that used by the main selection.
				UpdateInvertedColourStatus();
				if(bColoursInverted != m_bColoursInverted) return false;

				pxParam->SetFloat(m_xColourPair.m_xRGB.m_fR);
				break;

			case g_uHash_LightSource_Green :
				UpdateInvertedColourStatus();
				if(bColoursInverted != m_bColoursInverted) return false;
				pxParam->SetFloat(m_xColourPair.m_xRGB.m_fG);
				break;

			case g_uHash_LightSource_Blue :
				UpdateInvertedColourStatus();
				if(bColoursInverted != m_bColoursInverted) return false;
				pxParam->SetFloat(m_xColourPair.m_xRGB.m_fB);
				break;

			case g_uHash_LightSource_Hue :
				UpdateInvertedColourStatus();
				if(bColoursInverted != m_bColoursInverted) return false;
				pxParam->SetFloat(m_xColourPair.m_xHSV.m_fH);
				break;

			case g_uHash_LightSource_Saturation :
				UpdateInvertedColourStatus();
				if(bColoursInverted != m_bColoursInverted) return false;
				pxParam->SetFloat(m_xColourPair.m_xHSV.m_fS);
				break;

			case g_uHash_LightSource_Luminance :
				UpdateInvertedColourStatus();
				if(bColoursInverted != m_bColoursInverted) return false;
				pxParam->SetFloat(m_xColourPair.m_xHSV.m_fV);
				break;

			case g_uHash_LightSource_ReflectionChance :
				pxParam->SetFloat(m_fReflectionChance);
				break;

			default :
				return false;
			
		}
	}

	return true;
}

bool Tools_PhotonMapping_PolygonLightSource_Object::SetParameter(Asura_Hash_ID uParamID,const class Tools_Parameter* pxParam)
{
	bool bColoursInverted = Tools_Mode_PhotonMapping::SelectedLightHasInvertedColour();

	if(uParamID>=g_uHash_LightSource_LightSet0 && uParamID<=g_uHash_LightSource_LightSetLast)
	{
		u_int uBit = 1 << (uParamID - g_uHash_LightSource_LightSet0);
		bool bSet = pxParam->GetBool();
		if(bSet)
			m_uLightSets |= uBit;
		else
			m_uLightSets &=~ uBit;

		//Need for relight depends on whether the light set being altered is the one being shown
		if(uBit & Tools_LightSets::GetCurrentLightSets())
			m_bChanged = true;
		else
			m_bChanged_NoRelightNeeded = true;
	}
	else
	{
		switch(uParamID)
		{
			case g_uHash_LightSource_ShortRange :
				m_bShortRange = pxParam->GetBool();
				m_bChanged_NoRelightNeeded = true;
				break;

			case g_uHash_LightSource_UseRange :
				m_bUseRange = pxParam->GetBool();
				m_bChanged_NoRelightNeeded = true;
				break;

			case g_uHash_LightSource_Bleached :
				m_bBleachedSource = pxParam->GetBool();
				m_bChanged = true;
				break;

			case g_uHash_LightSource_Accuracy :
				m_iAccuracy = pxParam->GetInt();
				m_bChanged_NoRelightNeeded =true;
				break;

			case g_uHash_LightSource_Power :
				m_fPower = pxParam->GetFloat();
				m_bChanged = true;
				break;

			case g_uHash_LightSource_InnerRange :
				m_fInnerRange = pxParam->GetFloat();
				m_fOuterRange = Asura_Maths::Max(m_fInnerRange,m_fOuterRange);
				m_bChanged_NoRelightNeeded =true;
				break;

			case g_uHash_LightSource_OuterRange :
				m_fOuterRange = pxParam->GetFloat();
				m_fInnerRange = Asura_Maths::Min(m_fInnerRange,m_fOuterRange);
				m_bChanged_NoRelightNeeded =true;
				break;

			case g_uHash_PolygonLightSource_Spread :
				m_fAngle = pxParam->GetFloat();
				m_bChanged_NoRelightNeeded = true;
				break;

			case g_uHash_LightSource_Red :
				//To avoid getting colours messed up , only allow getting/setting of
				//the if the inverted state matches that used by the main selection.
				UpdateInvertedColourStatus();
				if(bColoursInverted != m_bColoursInverted) return false;
				
				m_xColourPair.m_xRGB.m_fR = pxParam->GetFloat();
				m_xColourPair.Update();
				m_bChanged = true;
				break;

			case g_uHash_LightSource_Green :
				UpdateInvertedColourStatus();
				if(bColoursInverted != m_bColoursInverted) return false;

				m_xColourPair.m_xRGB.m_fG = pxParam->GetFloat();
				m_xColourPair.Update();
				m_bChanged = true;
				break;

			case g_uHash_LightSource_Blue :
				UpdateInvertedColourStatus();
				if(bColoursInverted != m_bColoursInverted) return false;

				m_xColourPair.m_xRGB.m_fB = pxParam->GetFloat();
				m_xColourPair.Update();
				m_bChanged = true;
				break;

			case g_uHash_LightSource_Hue :
				UpdateInvertedColourStatus();
				if(bColoursInverted != m_bColoursInverted) return false;

				m_xColourPair.m_xHSV.m_fH = pxParam->GetFloat();
				m_xColourPair.Update();
				m_bChanged = true;
				break;

			case g_uHash_LightSource_Saturation :
				UpdateInvertedColourStatus();
				if(bColoursInverted != m_bColoursInverted) return false;

				m_xColourPair.m_xHSV.m_fS = pxParam->GetFloat();
				m_xColourPair.Update();
				m_bChanged = true;
				break;

			case g_uHash_LightSource_Luminance :
				UpdateInvertedColourStatus();
				if(bColoursInverted != m_bColoursInverted) return false;

				m_xColourPair.m_xHSV.m_fV = pxParam->GetFloat();
				m_xColourPair.Update();
				m_bChanged = true;
				break;

			case g_uHash_LightSource_ReflectionChance :
				m_fReflectionChance = pxParam->GetFloat();
				m_bChanged_NoRelightNeeded = true;
				break;


			default :
				return false;		
		}
	}

	UpdateAsuraLight();
	UpdateIfChanged();	

	return true;
}
