//
// Tools_Photon_Object.h
//
// Created by Richard Rice - 12/03/2003
// 
// Copyright (C) 2003 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef _TOOLS_PHOTON_OBJECT_H_
#define _TOOLS_PHOTON_OBJECT_H_

#include "Objects/Tools_Object.h"

//Object for creating/editing a temporary light in tools

class Tools_Photon_Object : public Tools_Object
{
		friend class Tools_Chunk_ToolsLights;

	public :		
		Tools_Photon_Object(const Asura_Vector_3* pxPosition,const Asura_Matrix_3x3* pxOrientation);
		~Tools_Photon_Object();

		virtual void UpdatePositionInAsuraData(bool /*bUpdateLinkedObjects*/);
		virtual void Render();

		virtual const char* GetName(){return "Light";}
		virtual const char* GetTypeDescription(){return "Tools Light";}
		virtual bool ShouldPersistBetweenModes() const {return true;}

		virtual void OnSelection();
		virtual void OnDeselection();
		virtual void UpdateWhileSelected();

		static Tools_Photon_Object* CreateLight( const Asura_Vector_3& xPosition, const Asura_Matrix_3x3& xOrientation );
		static void CreateToolsObjects();
		static void DeleteToolsObjects();
		static void SaveToolsObjects();

		static bool GeneralOverlayIsOpen(){return s_bGeneralOverlayOpen;}
		static void CreateGeneralOverlay();
		static void ToggleGeneralOverlay();

		void SetLightColour(const Asura_Vector_3& xColour);
		void SetLightBrightness(const float fBrightness) { m_fBrightness = fBrightness; }
		void SetLightRange(const float fRange) { m_fRange = fRange; }

		void UpdatePhoton();

	private :
		void CreatePropertiesOverlay();
		void DeletePropertiesOverlay();

		static void CreateLight();
		static void ShowLightsClicked();

		//data
		float m_fRed;
		float m_fGreen;
		float m_fBlue;
		float m_fBrightness;
		float m_fRange;
		bool m_bActive;

		class Asura_Photon* m_pxPhoton;

		bool m_bChanged;

		//static data for overlays
		static Asura_Vector_2 s_xPropertiesOverlayPosition;

		static bool s_bGeneralOverlayOpen;
		static u_int s_uGeneralOverlayMinimised;
		static Asura_Vector_2 s_xGeneralOverlayPosition;

		static bool s_bShowLights;
};

#endif //_TOOLS_PHOTON_OBJECT_H_
