//////////////////////////////////////////////////////////////////////////////
//
// Tools_LipSync.cpp
//
// Created by Stuart Middleton - 20 June 2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

#ifdef ASURA_USE_ARTICULATION_SYSTEM

#include "Tools_LipSync.h"

#include "LipSync/Asura_Lipsync_Manager.h"
#include "Annosoft/lib/liblipsync.h"
#include "Annosoft/lib/libarticulator.h"
#include "Annosoft/lib/liblipsync_license.h"
#include "ChunkSystem/Asura_SaveBlock.h"
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "Misc/Tools_Timed_Messages.h"
#include "ChunkSystem/Asura_chunk_lipsync.h"
#include "Sound/Asura_Dialogue.h"
#include "Core/Asura_LocalisedText.h"
#include "pc/FileSystem/Tools_FileSystem.h"

// The following code uses the Annosoft SDK and therefore cannot always conform to coding standards
char*	Tools_LipSync::g_pszLexData[ASURA_LT_L_NUM_LANGUAGES];

// Keep a list of dictionary files. These are a couple of meg each and only get loaded when you do a conversion but it is so much faster to
// keep them in memory. The logic being if you're converting one line you're probably going to convert some more this session.

//Get filename of local or network file
bool Tools_LipSync::GetFullPath(char* szFilename, char* szOut, char* szAddPath, bool bLocal)
{
	Asura_StringBuffer<512> szFullPathName;
	Asura_StringBuffer<32> szDrive;
	Asura_StringBuffer<512> szDir;
	Asura_StringBuffer<128> szFile;
	Asura_StringBuffer<32> szExt;

	if(!bLocal)
	{
	#ifdef ASURA_PC_ALLOW_LOAD_FROM_NETWORK
		Asura_FileAccess::PlatformGetNetworkFilenameOrGetLocalCopy(szFilename, (char*)szFullPathName.GetString(), 512);
	#else
		GetCurrentDirectory( szFullPathName.GetMaxLength()-1, (char*)szFullPathName.GetString() );
		szFullPathName.Cat("\\");
		szFullPathName.Cat(szFilename);
	#endif //ASURA_PC_ALLOW_LOAD_FROM_NETWORK
	}
	else
	{
		GetCurrentDirectory( szFullPathName.GetMaxLength()-1, (char*)szFullPathName.GetString() );
		szFullPathName.Cat("\\");
		szFullPathName.Cat(szFilename);
	}

	if(szAddPath != NULL)
	{
		_splitpath((char*)szFullPathName.GetString(), (char*)szDrive.GetString(), (char*)szDir.GetString(), (char*)szFile.GetString(), (char*)szExt.GetString());

		szFullPathName.Copy(szDrive.GetString());
		szFullPathName.Cat(szDir.GetString());
		szFullPathName.Cat(szAddPath);
		szFullPathName.Cat(szFile.GetString());
		szFullPathName.Cat(szExt.GetString());
	}

	strcpy(szOut, szFullPathName.GetString());

	return true;
}

char* Tools_LipSync::ReadTextFileIntoMemory(char* szFilename, long& nBytes)
{
	char* pszData;
	HANDLE hFile = CreateFile(szFilename, GENERIC_READ, FILE_SHARE_READ, 0, OPEN_EXISTING, FILE_FLAG_RANDOM_ACCESS, 0);

	if(hFile == INVALID_HANDLE_VALUE)
	{
		Asura_Assertf(false, ("Failed to open \"%s\" for lip-syncing.", szFilename));
		return(NULL);
	}
	else
	{
		nBytes = GetFileSize(hFile, NULL);
		pszData = new char[nBytes];
		ReadFile(hFile, pszData, nBytes, (LPDWORD)&nBytes, NULL);
		CloseHandle(hFile);
		return pszData;
	}
}

bool Tools_LipSync::DoesFileExist(char* szFilename)
{
	HANDLE hFile = CreateFile(szFilename, GENERIC_READ, FILE_SHARE_READ, 0, OPEN_EXISTING, FILE_FLAG_RANDOM_ACCESS, 0);

	if(hFile != INVALID_HANDLE_VALUE)
	{
		CloseHandle(hFile);
		return true;
	}
	return false;
}

void Tools_LipSync::CreateFolder(char* szFolderName)
{
	Asura_StringBuffer<512> szFullPathName;
	Asura_StringBuffer<32> szDrive;
	Asura_StringBuffer<512> szDir;

	_splitpath((char*)szFolderName, (char*)szDrive.GetString(), (char*)szDir.GetString(), NULL, NULL);

	szFullPathName.Copy(szDrive.GetString());
	szFullPathName.Cat(szDir.GetString());

	CreateDirectory(szFullPathName.GetString(), 0);
}

void Tools_LipSync::GetLanguageVersionOfPath(char* szNew, const char* szOrig, int iLanguage)
{
	strcpy(szNew, szOrig);

	if(iLanguage>= 0 && iLanguage != ASURA_LT_L_ENGLISH)
	{
		//Search for a directory named 'en' in the filename
		char* pcPos = &szNew[0];
		while(*pcPos)
		{
			if((pcPos[0] == '\\' || pcPos[0] == '/') &&
				(pcPos[1] == 'e'  || pcPos[1] == 'E') &&
				(pcPos[2] == 'n'  || pcPos[2] == 'N') &&
				(pcPos[3] == '\\' || pcPos[3] == '/'))
			{
				pcPos++;//advance to the 'e'
				break;
			}
			pcPos++;
		}

		if(*pcPos)
		{
			//replace by the code for the current language
			const char* szLangCode = Asura_LocalisedText::GetTwoCharCodeForLang((Asura_LocalisedText_Language)iLanguage);
			pcPos[0] = szLangCode[0];
			pcPos[1] = szLangCode[1];
		}
	}
}

void Tools_LipSync::CreateArticulationData(const Asura_Dialogue_Line* pxDialogueLine, const Asura_Hash_ID uHash, bool bTextless, bool bRegen)
{
	CLipSyncAccousticHMM*		pxHmm = NULL;				// hidden markov model
	ITextlessPhnRecognizer*		pxRecog = NULL;				// recognizer object
	IObservationStream*			pxObs = NULL;				// audio data stream
	ISyncResultsCollection*		pxSyncResults = NULL;	// sync marker results
	ILipSyncTranscriber*			pxTxScribe = NULL;			// text to phoneme file
	ITextBasedPhnRecognizer*	pxTBRecog = NULL;			// text based recogniser object
	long			nHmmBytes;						// hmm loading variable                        
	char*		pszHmmData = NULL;		// hmm loading variable
	long			nLexBytes;							// lex loading variable                        
	char			szError[320];						// error string
	serror		err = kNoError;					// error code

	char szEngWavName[512];
	char szEngAsrName[512];
	char szWavName[512];
	char szAsrName[512];
	char szTempName[512];

	const char* szName = pxDialogueLine->GetSoundFilename();

 	if(!szName)
	{	return ;}

	// Make the wav name and asr name from the passed sample name

	strcpy(szTempName, szName);
	GetFullPath(szTempName, szEngWavName, NULL, false);

	strcpy(szTempName, szName);
	szTempName[strlen(szTempName)-4] = (char)0;
	strcat(szTempName, ".asr");
	GetFullPath(szTempName, szEngAsrName, "Lipsync\\", true);


	for(int iLang = 0; iLang < ASURA_LT_L_NUM_LANGUAGES; iLang++)
	{
		const Asura_UnicodeChar* wszText = pxDialogueLine->GetTextForLanguage(iLang);
		char* szAscii = NULL;

		if(!bTextless)
		{
			int iLen = 0;
			if(wszText != NULL)
			{	
				// Possibility of triple char per unichar +2*triple on end JTBS
				iLen = (int)wcslen(wszText);
				if(iLen != 0)
				{	iLen = (iLen * 3) + 6;}
			}		

			if(!(iLen == 0 || wszText == NULL))
			{
				szAscii = new char[iLen];
				Asura_UnicodeToUTF8(wszText, szAscii);
			}
		}

		GetLanguageVersionOfPath(szWavName, szEngWavName, iLang);
		GetLanguageVersionOfPath(szAsrName, szEngAsrName, iLang);

		if(!DoesFileExist(szWavName))
		{
			if(szAscii)
			{	delete[] szAscii;}
			continue;
		}

		if(bRegen)
		{
			if(!DoesFileExist(szEngAsrName))
			{
				if(szAscii)
				{	delete[] szAscii;}
				return;
			}
		}


		CreateFolder(szAsrName);

		//Step 1)
		//Create a CAccousticHMM from an HMM file. You can also use resources.
		//The HMM files are located in the \models directory.
		//Which HMM? see HMM File Format.

		GetFullPath("Misc\\Annosoft\\models\\HiQ40_20_38generic.hmm", szTempName, NULL, false);
		pszHmmData = ReadTextFileIntoMemory(szTempName, nHmmBytes);
		if(pszHmmData == NULL)
		{	continue;}
		err = ::CreateAccHMM(pszHmmData, nHmmBytes, szError, &pxHmm);
		Asura_Assert(err == kNoError, szError);
		delete[] pszHmmData;
		pszHmmData = NULL;

		bool bUseText = false;

		if(szAscii)
		{
			bUseText = true;
			if(g_pszLexData[iLang] == NULL)
			{
				//Step 2) Load the ALEX (.Alex Files) file for your language:
				strcpy(szTempName, "Misc\\Annosoft\\dicts\\");
				strcat(szTempName, Asura_LocalisedText::GetTwoCharCodeForLang((Asura_LocalisedText_Language)iLang));
				strcat(szTempName, "_lexicon.alex");
				GetFullPath(szTempName, szTempName, NULL, false);
				g_pszLexData[iLang] = ReadTextFileIntoMemory(szTempName, nLexBytes);

				Asura_Assertf(g_pszLexData[iLang] != NULL, ("Cannot load dictionary file for language %s", Asura_LocalisedText::GetTwoCharCodeForLang((Asura_LocalisedText_Language)iLang)));
			}

			err = ::CreateDefaultTranscriber(g_pszLexData[iLang], nLexBytes, &pxTxScribe);
			Asura_Assert(err == kNoError,GetLipsyncErrorMessage(err));
		}

		// if not got text translation or has got translation and we haven't got a dictionary use the generic non text recognizer
		if( !szAscii || (szAscii && (g_pszLexData[iLang] == NULL)) )
		{
			//Step 2) Load the trigrammer into the HMM (optional but recommended)
			GetFullPath("Misc\\Annosoft\\dicts\\anno.trig", szTempName, NULL, false);
			err = ::LoadTrigramFileIntoHMM(pxHmm, szTempName, false);
			Asura_Assert(err == kNoError,GetLipsyncErrorMessage(err));
			bUseText = false;
		}

		//Step 3) Create the Observation stream based on your audio file
		err = ::CreateObservationStreamFromAudioFile(szWavName, pxHmm, &pxObs);
		Asura_Assert(err == kNoError,GetLipsyncErrorMessage(err));

		// Do both in case text based fails then we can do textless afterwards

		//Step 4) Create the Text based phoneme recognizer
//		err = ::CreateTextBasedRecognizer(&pxTBRecog); 
//		Asura_Assert(err == kNoError,GetLipsyncErrorMessage(err));

		//Step 4) Create the Textless phoneme recognizer
		err = ::CreateTextlessPhnRecognizer(&pxRecog);
		Asura_Assert(err == kNoError,GetLipsyncErrorMessage(err));

		//Step 5) Create the ISyncResultsCollection. The recognition results will be returned in this collection object for us to use.
		err = CreateSyncResultsCollection(ISyncResultsCollection::opt_intensity_as_morph, &pxSyncResults);
		Asura_Assert(err == kNoError,GetLipsyncErrorMessage(err));

		if(bUseText)
		{
			//Step 6) perform the recognition (with text)
			err = pxTBRecog->RecognizePhonemes(pxObs, pxHmm, szAscii, pxTxScribe, NULL, pxSyncResults);
			Asura_Assert(err == kNoError,GetLipsyncErrorMessage(err));

			if(err != kNoError)
			{
				//Step 6) perform the recognition
				err = pxRecog->RecognizePhonemes(pxObs, pxHmm, NULL, pxSyncResults);
				Asura_Assert(err == kNoError,GetLipsyncErrorMessage(err));
			}
		}
		else
		{
			//Step 6) perform the recognition
			err = pxRecog->RecognizePhonemes(pxObs, pxHmm, NULL, pxSyncResults);
			Asura_Assert(err == kNoError,GetLipsyncErrorMessage(err));
		}

		//Step 7) Do Something with the results. The IPhnMixtureArticulator can also do some great things with the data.
		IPhnMixtureArticulator *pxArt = NULL;
		err = CreatePhonemeArticulator(&pxArt, 0);
		Asura_Assert(err == kNoError,"");
		pxArt->SetMaxFrameDelta(0.25);
		pxArt->GenerateArticulations(pxSyncResults, NULL);

		Asura_SaveBlock xSaveBlock;
		Asura_Chunk_SaveBlock_Stream xStream(&xSaveBlock);

		Asura_LipSync_Articulation* pxLipArt = CreateArticulationFromAnnosoftPhonemeData(pxArt, uHash);

		Asura_Chunk_LipSync::Create(xStream, pxLipArt);

		xSaveBlock.AddAsuraTerminator();

		if(!xSaveBlock.SaveFile(szAsrName, false))
		{
			/* Display message for failed saved sequence */
			Asura_String xMessageString = "Failed to save ";
			xMessageString += szAsrName;
			Tools_Timed_Message_System::AddMessage(0xFFFF0000, xMessageString.GetString());
		}
		else
		{
			/* Display message for successfully exported sequences */
			Asura_String xMessageString = "Exported ";
			xMessageString += szAsrName;
			Tools_Timed_Message_System::AddMessage(0xFF00FF00, xMessageString.GetString());

			if(iLang == ASURA_LT_L_ENGLISH)
			{
				// TODO - get the data in without deleting and reloadin it.

				// Force a delete of the articulation so it loads back in the new version we've just created
				Asura_LipSync_Manager::DeleteArticulationByHash(uHash);

				// Load the file back in so its in our system
				Asura_ChunkLoading_System::LoadAsuraFile(szAsrName);
			}
		}

		delete pxLipArt;
		pxArt->Release();

		DestroySyncResultsCollection(pxSyncResults);
		DestroyAccHMM(pxHmm);
		DestroyTextlessPhnRecognizer(pxRecog);
		DestroyTextBasedRecognizer(pxTBRecog);
		DestroyObservationStream(pxObs);
		if(szAscii)
		{	delete[] szAscii;}
	}
}

Asura_LipSync_Articulation* Tools_LipSync::CreateArticulationFromAnnosoftPhonemeData(IPhnMixtureArticulator* pxArt, Asura_Hash_ID uHash)
{
	Asura_LipSync_Articulation* pxLipArt = new Asura_LipSync_Articulation();

	pxLipArt->m_uHash = uHash;

	CPhonemeMixtureArticulation *pxCur, *pxEnd;
	pxCur = pxArt->begin();
	pxEnd = pxArt->end();
	int total = 0;

	while (pxCur != pxEnd)
	{	total++;	pxCur++;}

	pxLipArt->m_uFrames = total;
	pxLipArt->m_pxFrameData = new Asura_LipSync_Articulation_FrameData[total];

	pxCur = pxArt->begin();
	pxEnd = pxArt->end();
	int iCount = 0;
	while (pxCur != pxEnd)
	{
		pxLipArt->m_pxFrameData[iCount].m_uMSStart = pxCur->msStart;
		pxLipArt->m_pxFrameData[iCount].m_uMSEnd = pxCur->msEnd;
		pxLipArt->m_pxFrameData[iCount].m_uNumConsts = pxCur->m_nConstituents;

		pxLipArt->m_pxFrameData[iCount].m_pxPhononData = new Asura_LipSync_Articulation_PhononData[pxCur->m_nConstituents];

		for(int i=0; i<pxCur->m_nConstituents; i++)
		{
			u_int uPhon = pxCur->m_constituents[i].strPhoneme[0] + (pxCur->m_constituents[i].strPhoneme[1] << 8);
			pxLipArt->m_pxFrameData[iCount].m_pxPhononData[i].m_uPhoneme = Asura_LipSync_Manager::GetPhonemeIndex(uPhon);
			pxLipArt->m_pxFrameData[iCount].m_pxPhononData[i].m_fWeight = pxCur->m_constituents[i].weight;
		}

		pxCur++;
		iCount++;
	}

	return pxLipArt;
}

void Tools_LipSync::Initialise()
{
	ILibLipsyncRegistrar* pRegistrar = NULL;
	LipsyncGetInfoMgr(&pRegistrar);
	Asura_Assert(pRegistrar, "Failed to get Annosoft Lipsync registra information");
	if (pRegistrar)
	{
// SM - 04 February 2008 20:22:33 - This is our SS2 licence. I was under the impression it was a company wide license.
		// I am therefore going to put it in tools so that other projects can use it. If we require a new licence per project
		// We will have to have a project version of this file.

		pRegistrar->SetUserName("ALIEN_V_PRED_3");
		pRegistrar->SetCompanyName("REBELLION");
		pRegistrar->SetLicenseKey("6D54-5B49-4E40-4E49-5B3C-5159-7049-5153-5F5A-405A-5F53-515C-4A4D-533F-5872-5E23-4127-5925");
		// if you have purchased or otherwise recieved license
		// to additional language processors for text based lipsync
		// annosoft will give you specific keys for each language
		// you will need to register those keys with the registrar
		// in order to use the language
		//pRegistrar->AddLanguageKey("3837-3C55-700F-2437-5056-383A-3B5D-5D09-0309-415B-313A-3E41-5B3B-3539-4D21-091A-0923-2109");	// French
		//pRegistrar->AddLanguageKey("3834-3F55-700F-2734-5056-3839-385D-5D09-000A-415B-3139-3D41-5B3B-363A-4D21-0919-0A25-6E54");		// German
		//pRegistrar->AddLanguageKey("3E34-3D55-7009-2736-5056-3E39-3A5D-5D0F-0008-415B-3739-3F41-5B3D-3638-4D21-0F19-0823-250F");		// Italian
		//pRegistrar->AddLanguageKey("3C33-3C55-700B-2037-5056-3C3E-3B5D-5D0D-0709-415B-353E-3E41-5B3F-3139-4D21-0D1E-0923-230D");	// Spanish
		pRegistrar->Release();
	}


	// Clear out the dictionary file pointers
	for(int i=0; i<ASURA_LT_L_NUM_LANGUAGES; i++)
	{	g_pszLexData[i] = NULL;}

}

void Tools_LipSync::Shutdown()
{
	// delete all the dictionaries
	for(int i=0; i<ASURA_LT_L_NUM_LANGUAGES; i++)
	{
		if(g_pszLexData[i] != NULL)
		{
			delete[] g_pszLexData[i];
		}
	}

}

#endif
