#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef __TOOLS_LEVEL_EXPORT_H_
#define __TOOLS_LEVEL_EXPORT_H_

// Engine independent.
#include "Core/Asura_String.h"
#include "Core/Asura_HashTree.h"
#include "Core/Asura_Material_System.h"
#include "Core/Asura_Maths.h"
#include "Core/Asura_NameTree.h"
#include "Core/Asura_Timers.h"
#if ( defined ASURA_MAIN ) && !( defined ASURA_USE_AUDIO_SYSTEM )
#include "Sound/Asura_ProjectSoundIDs.h"
#endif

// Tools.
#include "Misc/list_tem.hpp"
#include "Misc/Tools_DynamicArray.h"
#include "Misc/Tools_Platform.h"

class Asura_AnimTable;
class Asura_Blueprint;
class Asura_Chunk_Header;
class Asura_Chunk_Stream;
class Asura_FacialAnimation_Face;
class Asura_FacialAnimation_FaceAnimation;
class Asura_NameTree;
struct TextureConversionData;
class Asura_Mesh;
class Asura_Mesh_TextureInfo;
struct Tools_IniFile;
class Asura_Hierarchy_Anim;
class Asura_Hierarchy_Skin;
class Asura_Chunk_Memory_Stream;
class Asura_SaveBlock;

struct SoundExportData;
struct Asura_SoundMapping;
struct Tools_SoundProperties;
class Asura_Sound_Event;
class Asura_Sound_Sample;
class Asura_Dialogue_Event;
class Asura_Dialogue_Line_Template;
class Axon_Gamescene_AnimTemplate;

template<class T> class Asura_List;

//  AAB:  (14/11/2003 12:31:28):  Moved from cpp
struct ResourceFileListEntry
{
	ResourceFileListEntry()
	{
		this->bFileAddedOK = false;
		this->iByteOffset = 0;
		this->iNumBytes = 0;
	}
	
    bool bFileAddedOK;

    Asura_String xFilename;
    int iByteOffset;
    int iNumBytes;
};

#ifdef ASURA_USE_AUDIO_SYSTEM

// Usedfor exporting audio system streaming chunks
struct Tools_StreamingSoundExportInfo
{
	Asura_String m_xFilename;
	Asura_String m_xExportName;
	bool m_bLooping;
};

#endif

// C O N S T S /////////////////////////////////////////
static const float TOOLS_DEFAULT_ANIM_ANGLE_TOLERANCE = 0.4f;
static const float TOOLS_DEFAULT_ANIM_POSITION_TOLERANCE = 0.01f;

class Tools_Level_Export
{
	public :
		//Values from this enumeration get saved out , so existing entries shouldn't
		//be changed. RAR 01/04/2005
		//Now set to match enumeration in Tools_Platform. RAR 11/08/2006
		enum EXPORT_TYPE
		{
			Export_PC = TOOLS_PLATFORM_PC,
			Export_PS2 = TOOLS_PLATFORM_PS2,
			// (MH - 08.05.2008) Xbox is no longer supported.
			//Export_XBox = TOOLS_PLATFORM_XBOX,
			// (KOR - 14/10/2008) Nor GameCube
			//Export_GC = TOOLS_PLATFORM_GC,
			Export_PSP = TOOLS_PLATFORM_PSP,
			Export_PS3 = TOOLS_PLATFORM_PS3,
			Export_Xbox360 = TOOLS_PLATFORM_XBOX360,
			Export_Wii = TOOLS_PLATFORM_WII,

			ExportType_NumPlatforms = TOOLS_PLATFORM_NUM_PLATFORMS,
		};

		// please keep in sync with Tools_Level_Export::s_aszExportStageText
		enum EXPORT_STAGE
		{
			EXPORT_STAGE_UNDEFINED = 0,				// no stage set yet
			EXPORT_STAGE_END,						// force end of current stage
			EXPORT_STAGE_INITIALISING,				// "Initialising..."
			EXPORT_STAGE_START_EXPORT,				// "Beginning export..."
			EXPORT_STAGE_GENERATING_WAYPOINTS_SUBSET, // "Generating subset of waypoints to export..."
			EXPORT_STAGE_TEMPORARY_FILES_SAVE,		// "Saving temporary files for export..."
			EXPORT_STAGE_COMPILE_INCLUDE_FILE_LIST,	// "Compiling list of files to include..."
			EXPORT_STAGE_LOADING_FILES,				// "Loading files..." for fake export
			EXPORT_STAGE_ADJUSTING_UV,				// "Adjusting UVs..."
			EXPORT_STAGE_ADDING_ENVIRONMENT_MATERIALS, // "Adding environment materials..."
			EXPORT_STAGE_FINDING_ENVIRONMENT_TEXTURES, // "Finding environment textures..."
			EXPORT_STAGE_FINDING_MESHLESS_ENTITY_TEXTURES, // "Finding meshless entity textures..."
			EXPORT_STAGE_FINDING_CORONA_TEXTURES,	// "Finding corona textures..."
			EXPORT_STAGE_FINDING_BILLBOARD_PLANT_TEXTURES, // "Finding billboard plant textures..."
			EXPORT_STAGE_FINDING_CARPET_TEXTURES,	// "Finding carpet textures..."
			EXPORT_STAGE_ADDING_ENVIRONMENT_TEXTURES, // "Adding environment textures..."
			EXPORT_STAGE_ADDING_SKIN_TEXTURES,		// "Adding skin textures..."
			EXPORT_STAGE_ADDING_SKIN,				// "Adding skin "hello.tga"..."
			EXPORT_STAGE_ADDING_SHAPE_TEXTURES,		// "Adding shape textures..."
			EXPORT_STAGE_ADDING_BOTANICAL_TEXTURES,	// "Adding botanical textures..."
			EXPORT_STAGE_STRIPIFYING_ENVIRONMENT,	// "Stripifying environment..." // this includes "Subdividing, optimizing, stripifying environment...", 
			EXPORT_STAGE_GROUP_ENVIRONMENT_VERTS,	// "Grouping environment vertices by position"
			EXPORT_STAGE_WATER_COLLECTING_MESHES,	// "Water: Collecting meshes:"
			EXPORT_STAGE_WATER_SUBDIVIDING_MESHES,	// "Water: Subdividing meshes:"
			EXPORT_STAGE_WATER_SPLITTING_QUADS,		// "Water: Splitting quads:"
			EXPORT_STAGE_WATER_STRIPIFYING_MESHES,	// "Water: Stripifying meshes:"
			EXPORT_STAGE_PS2_WRITING_STRIPPED_ENV,	// "PS2 Writing stripped env..."
			EXPORT_STAGE_COMPILING_ENVIRONMENT_SHADERS, // "Compiling Environment shaders..."
			EXPORT_STAGE_SORTING_STRIPS,			// "Sorting strips based on shader and material hash..."
			EXPORT_STAGE_EXPORTING_ENVIRONMENT,		// "Exporting environment..."
			EXPORT_STAGE_BOTANICALS_FIND_USED_TEMPLATES_AND_SHAPES,	// "Botanicals: Finding used templates and shapes..."
			EXPORT_STAGE_BOTANICALS_CALC_SHAPE_EXPORT_SIZE,	// "Botanicals: calculating shape export size..."
			EXPORT_STAGE_BOTANICALS_EXPORTING_SHAPES,	// "Botanicals: Exporting shapes..."
			EXPORT_STAGE_BOTANICALS_EXPORTING_TEMPLATES, // "Botanicals: Exporting templates..."
			EXPORT_STAGE_ADDING_SHAPES,				// "Adding shape "shape name"..."
			EXPORT_STAGE_CREATING_PHYSICS_ZONES, // "Creating physics zones"
			EXPORT_STAGE_SAVING_ENVIRONMENT_TO_FILE, // "Saving environment to file"
			EXPORT_STAGE_COMPILING_MATERIAL_RESPONSE_PIXEL_SHADERS, // "Compiling material response pixel shaders..."
			EXPORT_STAGE_DIALOGUE_EVENT_SOUNDS,		// "Dialogue Event Sounds"
			EXPORT_STAGE_ASURA_FILES,				// "Asura Files";
			EXPORT_STAGE_EXPORT_SOUNDS_FROM_INI_FILE,// "Export Sounds From Ini File";
			EXPORT_STAGE_STREAMING_TEXTURES,		// "Streaming Textures";
			EXPORT_STAGE_SKYBOX_TEXTURES,			// "Skybox Textures";
			EXPORT_STAGE_ADD_MISC_TEXTURES,			// "add misc Textures";
			EXPORT_STAGE_HIERARCHY_SOUNDS,			// "Hierarchy Sounds";
			EXPORT_STAGE_ENVIRONMENT_SOUNDS,		// "Environment Sounds";
			EXPORT_STAGE_ENTITY_SOUNDS,				// "Entity Sounds";
			EXPORT_STAGE_SHAPES,					// "Shapes";
			EXPORT_STAGE_NON_ENVIRONMENTAL_SOUNDS,	// "Non-Environmental Sounds";
			EXPORT_STAGE_LOADING_OLD_EXPORT_FILE,	// "Loading Old Export File";
			EXPORT_STAGE_LOCALISED_SOUNDS_NON_STREAMED, // "Localised Sounds (non-streamed)";
			EXPORT_STAGE_LOCALISED_SOUNDS_STREAMED,	// "Localised Sounds (streamed)";
			EXPORT_STAGE_SOUND_EVENT_SOUNDS,		// "Sound Event Sounds";
			EXPORT_STAGE_SOUND_EVENTS_TOOLSGMSND,	// "Sound Events (ToolsGmSnd.asr)";
			EXPORT_STAGE_SOUND_EVENTS_INDIVIDUAL,	// "Sound Events (Individual)";
			EXPORT_STAGE_STREAMING_SOUNDS,			// "Streaming Sounds";
			EXPORT_STAGE_COMPRESSING_FILE,			// "Compressing File";

			EXPORT_STAGE_OVERALL,
			EXPORT_STAGE_TOTAL
		};

		struct ExportStageStats
		{
			bool m_bUsed;
			Asura_Char* m_pszText;
			Asura_TickCount m_xTimeStart;
			Asura_TickCount m_xStageTime;			// total time to run this stage
		};

		Tools_Level_Export();
		virtual ~Tools_Level_Export();

		static void Update();

		static void ExportLevel(int iPlatform);
		//Calls ExportLevel() for each platform that has been selected
		static void ExportLevelForEachPlatform();
		static bool IsEnabled_ExportLevel(){return s_uPlatformsToExport!=0;}
		static bool IsPlatformSetToExport(EXPORT_TYPE ePlatform){return (s_uPlatformsToExport & (1<<ePlatform)) != 0;}

		// Note - these are obsolete and should be removed once we confirm no current project is calling them.
		static void ExportPCLevel() {ExportLevel(Export_PC);}
		static void ExportPS2Level() {ExportLevel(Export_PS2);}
		static void ExportPSPLevel() {ExportLevel(Export_PSP);}
		static void ExportPS3Level() {ExportLevel(Export_PS3);}
		static void ExportXbox360Level() {ExportLevel(Export_Xbox360);}

		static void ExportCharacters(int iPlatform);
		// Obsolete - see ExportLevel comment above 
		static void ExportPCCharacters() {ExportCharacters(Export_PC);}
		static void ExportPS2Characters() {ExportCharacters(Export_PS2);}
		static void ExportPSPCharacters() {ExportCharacters(Export_PSP);}
		static void ExportPS3Characters() {ExportCharacters(Export_PS3);}
		static void ExportXbox360Characters() {ExportCharacters(Export_Xbox360);}

		//Does skybox export for all platforms supported by project
		static void ExportSkybox_AllPlatforms();

		static void ExportGlobalSoundEvents();
		static void ExportGlobalSoundEvents(int iPlatform);
		static void ExportGlobalSoundEventsForTools();

		#ifdef ASURA_USE_AUDIO_SYSTEM
		static void ExportStreamingSounds();
		static void ExportStreamingSounds( const int iPlatform );
		#endif

		static void ExportCommonFile();
		static void ExportCommonFile(int iPlatform);

		#ifdef ASURA_USE_GUI_MENU_SYSTEM
		//exports all the global menu file
		static void ExportGUIMenus();
		//exports the specified menu file for one or more platforms
		static void ExportGUIMenus(const char* szMenuFile);
		//exports the specified menu file (or all if szMenuFile is NULL) for the selected platform
		static void ExportGUIMenus(int iPlatform,const char* szMenuFile);
		#endif

		static void BatchExport_ExportLevel();
		virtual void BatchExport_GetIdentifiers( List<const char*>* pxIdentifiers );	

		//This performs the parts of the level export required to load any files that will be needed
		//when using Quit To Game. The .pc file won't get generated however , and hopefully I can skip
		//some of the slower parts of the process. RAR 08/05/03
		static void DoFakeLevelExport();

		//Loads asura files listed in an ini file  (files should be under the [Asura Files] section header).
		//This isn't really meant for use during level exports , and is only put here since it is related
		//to AddAsuraFilesFromTextFile(). RAR 30/03/2006
		//Returns false if file fails to load , or no [Asura Files] section is found
		static bool LoadAsuraFilesFromTextFile(const char* pszFilename);

        static bool AddFileToExportList(List<Asura_String>& xNameList, const Asura_String& xName, bool bIsSkinFile);
		//Version that doesn't require the list to be passed around (since the export system knows
		//about the correct list anyway).
		static bool AddFileToExportList(const Asura_String& xName, bool bIsSkinFile);
		//Adds all the files that match the given filter in the directory (and subdiretories) to the export list
		//This also stores the names of any skin files found in m_pxSkinFileList
		static void AddDirToExportList(List<Asura_String>& xNameList,const char* pszDirectory,Asura_String xFilter="*.asr",bool bWarnOnEmpty = true);		
		//Adds all the animations and skins in the directory (and subdiretories) to the export list
		//This also stores the names of the skin files in m_pxSkinFileList
		static void AddAnimDirToExportList(List<Asura_String>& xNameList,const char* pszDirectory,bool bWarnOnEmpty = true, const bool bAllFilesAreAnims = false);
		//Adds a single animation by name or hash
		//Skin must be in one of the directories dealt with by the hashed filename system
		static void AddSkinToExportList(const char* szName);
		static void AddSkinToExportList(Asura_Hash_ID uSkinHash);

		//Adds a single animation by name or hash
		//Anim must be in one of the directories dealt with by the hashed filename system
		static void AddAnimToExportList(const char* szName);
		static void AddAnimToExportList(Asura_Hash_ID uAnimHash);

		//Adds the animation table and all animations that it refers to.
		//The anim table will be loaded if needed if Tools_Animation_Files::GetAnimTableFilePostFix() and
		//Tools_Animation_Files::GetNumberOfAnimTableFilePostFixes() have been implemented.
		static void AddAnimTableToExportList(const char* szName);
		static void AddAnimTableToExportList(Asura_Hash_ID uTableHash);
		static void AddAnimTableToIgnore(Asura_Hash_ID uTableHash);

		//adds skin and matching anim table and anims to the export list
		static void AddSkinAndAnimsToExportList(const char* szName);
		static void AddSkinAndAnimsToExportList(Asura_Hash_ID uSkinAndTableHash);

		//causes the .sky file to be added to export list
		static void AddSkyboxFilesToExport();


		//Adds the sound to the those to be included in the level. (Most entity , environment , skin and anim sounds are
		//handled by their own means however).
		//If bOnlyUsedByHash is set then no sound mapping will be created for the sound ID , should
		//a sound with the filename already have been exported.
		static void AddSoundToExportList(int iSoundID,bool bOnlyUsedByHash = false);

		//Adds sounds listed in ini file to export list. Returns false if the ini file isn't loaded.
		//The optional callback function can be used to select a subset from the ini file , and should return
		//true if a sound should be included in the export.
		static bool AddIniFileSoundsToExportList(const char* szIniFilename,bool (*pfnShouldExport)(int iSoundID) = 0);

		//Adds a sound to the already exported list , so that any subsequent attempt to export it will
		//just create a sound mapping to it.
		static void AddSoundToIgnore(int iSoundID,const char* szFilename);

		//Adds the event along with any child events and samples , and any wav files used by them.
		static void AddSoundEventToExportList(Asura_Hash_ID uEventHash, const char* szDebugText = NULL);
		static void AddSoundEventToExportList(const Asura_Sound_Event* pxEvent);

		//Adds sample and its wav file to the export
		static void AddSoundSampleToExportList(Asura_Hash_ID uSampleHash);
		#ifdef ASURA_USE_DIALOGUE_SYSTEM
		//Adds the event along with any child events and lines , and any wav files used by them.
		static void AddDialogueEventToExportList(Asura_Hash_ID uEventHash);
		static void AddDialogueEventToExportList(const Asura_Dialogue_Event* pxEvent);

		//Adds line and its wav file to the export
		static void AddDialogueLineToExportList(Asura_Hash_ID uLineHash);
		static void AddDialogueLineTemplateToExportList(const Asura_Dialogue_Line_Template* pxLineTemplate);
		#endif

		#if defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)
		// Add the facial-animation data to the export...
		static bool AddFaceToExportList(const char* szFaceName, const Asura_Hash_ID uFaceHash);
		static bool AddFaceAnimationToExportList(const char* szFaceAnimationName, const Asura_Hash_ID uFaceAnimationHash)			{ return(AddFaceAnimationToExportList(szFaceAnimationName, uFaceAnimationHash, This->m_pxFaceAnimationContainer)); }
		static bool AddLocalisedFaceAnimationToExportList(const char* szFaceAnimationName, const Asura_Hash_ID uFaceAnimationHash)	{ return(AddFaceAnimationToExportList(szFaceAnimationName, uFaceAnimationHash, This->m_pxLocalisedFaceAnimationContainer)); }
		#endif	//defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)

		//Adds blueprint type to list for export
		static void AddBlueprintTypeToExportList(Asura_Hash_ID uBlueprintType);
		static bool ShouldExportBlueprint_Static(const Asura_Blueprint* pxBlueprint, const bool bExportChildBlueprints);

		#ifdef ASURA_USE_AXON
		//Adds animation templates to list for export
		//This is currently an all or nothing affair
		static void AddAnimTemplatesToExportList();
		#endif

		#ifdef ASURA_USE_STREAMING
		//Adds the global meta music. Used by the common sound export.
		static void AddGlobalMetaMusicToExportList();
		#endif

		#ifdef ASURA_USE_GUI_MENU_SYSTEM
		//Adds all menus that are assigned to the file to the export list.
		//File is the filename without path/extension
		virtual void AddGUIMenuFileToExportList(const char* szMenuFile);
		#endif

		//Add a texture to the export list. Unlike the other AddTexture functions ,this one can be used
		//ealier , before writing to the file has started.
		static void AddTextureToExportList(int iTextureID);

		//Similar to the above, but for GUIMenu textures, which shouldn't be sRGB corrected
		static void AddTextureToGUIMenuList(int iTextureID);

		//Checks to make sure that a texture is okay for use within the streaming textures file
		static bool IsTextureOkayForStreaming(int iTextureID);

#if defined(ASURA_USE_MATERIAL_RESPONSE_SYSTEM)
		// Add the material-response override data to the export list...
		static void AddMaterialResponseOverridesToExportList();

		// Functions to set and reset the material-response overrides for export...
		void SetMaterialResponseOverridesForExport(const u_int uNumOverrides, const Asura_Hash_ID* pxOverrides)
		{
			m_uNumMaterialResponseOverridesForExport	= uNumOverrides;
			m_pxMaterialResponseOverridesForExport		= pxOverrides;
		}
		void ResetMaterialResponseOverridesForExport(void)
		{
			m_uNumMaterialResponseOverridesForExport	= 0;
			m_pxMaterialResponseOverridesForExport		= NULL;
		}

		static void AddMRPShaders( const Asura_Hash_ID uMaterial1, const Asura_Hash_ID uMaterial2,
			const bool bSkin );
#endif	// defined(ASURA_USE_MATERIAL_RESPONSE_SYSTEM)

		static void AddPFXOverridesToExportList();
		static void AddFSFXOverridesToExportList();

		static void CreateOverlay();

		static void CreateAnimationOverlay();

		static void AddAdvanceOptionsOverlayControls(class Asura_Overlay* pxOverlay,class Asura_Vector_2& xOffset);

		//Adds controls for selecting the languages to export
		static void AddLanguageOverlayControls(class Asura_Overlay* pxOverlay,class Asura_Vector_2& xOffset);


		enum MessageType
		{
			MT_INFORMATION,
			MT_WARNING,
			MT_ERROR,
			MT_LOGONLY,
			MT_HEADING,
			MT_LOGANDMESSAGE,	// output to Message Log and logfile
			MT_MESSAGELOGONLY,	// output to Message Log only
		};

		static void AddMessage( MessageType eMsgType, const char* szMsgFormat, ... );
		static void ClearMessageDisplayHeading(){s_xMessageDisplayHeading.MakeNull();}

        static inline void RecordError( void ) { s_iNumErrors++; }
        static int GetNumErrors( void ) { return s_iNumErrors; }

        static bool IsAbortFlagSet( void ) { return s_bAbort || s_bUserAbort; }
        static bool IsUserAbortFlagSet( void ) { return s_bUserAbort; }
        static void SetAbortFlag( void ) 
		{ 
			s_bAbort = true; 
		}
        static void UpdateStatus( EXPORT_STAGE eStage, const char* szHeading, int iPercent) { s_xHeading = szHeading; s_iPercent = iPercent; s_eThisStage = eStage; This->UpdateProgress(); }

	
        static int GetUntexturedMaterialIndex(){return This->m_iUntexturedMaterialIndex;}
        
        static char* s_szLogPath;
        static char* s_szErrorLogPath;
        static char* s_szSoundLogPath;
        static char* s_szBlueprintLogPath;

		//Attempts to load the sound files that would be added to the level export as a reult of the 
		//ini files in Sounds\\IniFiles. Used for getting sounds loaded when dealing with levels that
		//haven't been exported yet.
		//It should be okay to use this when tools haven't been initialised
		static void LoadIniFileSounds();

		//generates a dds file if the existing one isn't up to date.
		//intended for use before tools are initialised
		static bool GenerateDDSFile(const char* pszInputName,const char* pszOutputName,int iIndex);
        static bool CreateDDSFile(const char* inputName,const char* outputName,int index, const TextureConversionData& xConversionData);

        // Currently these functions are quite slow (generating the
        // texture in question or accessing the TIM2 file if it's up to date)
        // and assume you do some form of caching of the results externally
        // if speed is an issue.
        static u_int GetTIM2DataSize( int iTexID, bool bQuiet = true );
        static bool  GetTIM2Proportions( int iTexID, u_int* puWidth, u_int* puHeight, bool bQuiet = true );
        static bool  DoesTIM2Have8BitAlpha( int iTexID, bool* pbHas8BitAlpha, bool bQuiet = true );

        static bool IsPS2MultiplayerExport( void ) { return s_bPS2MultiplayerExport; }
        static bool ShouldIncludeDualTextureData( void ) { return s_bIncludeDualTexturingData; }
        static bool IsLevelExport( void ) { return s_bIsLevelExport; }
        static bool IsExport( void ) { return s_bIsExport; }
		static bool IsCharacterExport( void ) { return s_bIsCharacterExport; }
        static bool IsSkyboxExport( void ) { return s_bIsSkyboxExport; }
		static bool IsDoingFakeLevelExport() { return This ? This->m_bFakeLevelExport : false;}
		static bool IsGlobalSoundEventExport() { return s_bIsGlobalSoundEventExport; }
		static bool IsGUIMenuExport() { return s_bIsGUIMenuExport; }
		static bool IsGUIMenuAuidoExport(void) { return(s_bIsGUIMenuAudioExport); }
		static bool IsCommonFileExport() {return s_bIsCommonFileExport;}

		static void SetIsExport(bool bExport) {s_bIsExport = bExport;}
		static void SetForceTextureGeneration(bool bForce) { s_bForceTextureGeneration = bForce; }

		// Compression accessors...
		static void DisableCompression(void)	{ s_bCompressionDisabled = true; }
		static void EnableCompression(void)		{ s_bCompressionDisabled = false; }
		static bool IsCompressionDisabled(void)	{ return(s_bCompressionDisabled); }

		//Gets value to be subtracted from all positions during export , so as to recentre the level.
		//Outside of proper level exports , this returns a zero vector
		static const Asura_Vector_3& GetExportCentre() {return (IsLevelExport() && !IsDoingFakeLevelExport()) ? s_xExportCentre : Asura_Maths::ZeroVector3;}

		//Sets centre for level export , which will be subtracted from all positions during the export
		static void SetExportCentre(const Asura_Vector_3& xCentre){s_xExportCentre = xCentre;}

		//For applying gamma correction to lighting values during export
		//Scale a prelighting colour value
		static u_int ScalePrelightValue(u_int uColour);
		//Scale brightness for lights
		static float ScaleLightBrightness(float fBrightness);

		static bool ExportFullBright(){return s_bFullBrightPrelighting;}


		//Clears progress messages from DDS generation if needed
		static void ClearProgressMessage();

		//Alter uv coordinates of mesh to be in acceptable ranges , and suitable for strips
		static void AdjustUVCoordinates(Asura_Mesh* pxMesh);

		static class Tools_EnumerationReader* GetSoundIDEnumeration()
		{return This->LoadSoundIDEnumeration();}

		static bool ShouldExportEntityClass(int iClass)
		{return !s_bFilterEntityExport || !s_xEntityClassesToIgnore.contains(iClass);}

        virtual u_int GetDefaultCharacterTextureFlags( void );

		//gets the name of the platform from an EXPORT_TYPE value
		static const char* GetPlatformName(int iPlatform);

		//gets the name of the platform graphics directory from an EXPORT_TYPE value
		static const char* GetPlatformGraphicsDirectoryForFileExport( int iPlatform );

		//Static version of function made available so that shape export can check
		//which chunks should be allowed.
		static bool ShouldChunkBeExported_Static(int iChunkID);
		static bool ModifyChunkBeforeOutput_Static(const Asura_String& xFilename,const Asura_Chunk_Header* pxChunk,char** ppcOutputChunkBuffer);

		static bool ShouldExportBigEndian( void )
		{
			return (s_bIsExport) && (Tools_Platform::IsBigEndian(s_eExportType));
		}

		static bool ShouldDebugColourSubstrips() { return s_bDebugColourSubstrips; }

		//Gets the texture filename to use during export , possibly from the platform directories
		static bool GetTextureFilenameForExport(const char* pszInputName,const char* pszPlatformExtension,Asura_String& xOutputName,bool* pbUsingPlatformDir);

		//Gets the light set that is going to be exported
		static u_int GetCurrentLightSetsToExport(){return s_uCurrentLightSetsToExport;}
		//Sets the light sets for the current export. This needs to be done before StartLevelExport()
		static void SetCurrentLightSetsToExport(u_int uLightSets);

		static EXPORT_TYPE s_eExportType;
	
		static void SetForcedBotanicalTextureFlagsPS2(int iVal)	//CSB 28/04/06
		{
			s_iForcedBotanicalTextureFlagsPS2 = iVal;
		}


		//returns true if platform is supported by the project.
		static bool IsPlatformSupported(EXPORT_TYPE ePlatform) {Asura_Assert(This,"");return This->IsPlatformAllowed(ePlatform);}
		//Should return true for any platforms supported by project
		virtual bool IsPlatformAllowed(EXPORT_TYPE ePlatform) const {return true;}

		//Should return true for any languages supported by project. Default implementation is just for english
		virtual bool IsLanguageAllowed(int iLanguage) const;
		static bool Static_IsLanguageAllowed(int iLanguage) {Asura_Assert(This,"");return This->IsLanguageAllowed(iLanguage);}

		static char* GetCommonExportFile() { return "ToolsIniFiles\\Common_Export.txt"; }
#ifdef ASURA_USING_ALL_LEVEL_EXPORT
		static void AddPFXToIgnore(const Asura_Hash_ID uPFXToIgnore);
#endif

		//Gets the subdirectory to use for temporary files in the export
		static const char* GetTempDirectory();

		//deletes the files in the temporary directory
		static void DeleteTemporaryFiles();

		//If a level export is in process this returns true if the module belongs to the light set being exported.
		//If not a level export it returns true for all modules.
		static bool ShouldIncludeModuleInExport(int iModuleIndex);

		//Returns true if ShouldIncludeModuleInExport() is true for all modules
		static bool IsExportingAllModules();

		//true if doing a mission export that contains just the environment
		static bool MissionExport_IsExportingEnvironmentOnly(){return s_bMission_IsExportingEnvironment;}
		//true if doing a mission export that contains just the mission
		static bool MissionExport_IsExportingMissionOnly(){return s_bMission_IsExportingMission;}

		bool ConvertSound(const char* pszWavFilename, int iFrequency, bool bLooped, Asura_String* xOutputName, bool bStreaming);

		static void SetWiiStreamsUseBink(bool bUseBink) { s_bWiiStreamsUseBink = bUseBink; }

	private :
		static void InitialiseTempDirectory();
		static void CloseTempDirectory();
		static int CompareExportStageStatsElement( const void* pData1, const void* pData2 );

	protected :

		// Override this to control the weld tolerance - Added to fix SS2 weapon problems
		virtual float GetPS2WeldToleranceForSkins();			

		//gets a list of filenames of asura files to be added for a level export
		virtual void ListAsuraFilesForExport(List<Asura_String>& nameList);
		virtual void AddAsuraFilesFromTextFiles();

		//adds all the ragdoll descriptions for the common file export
		void AddRagdollDescriptionsToExportList();

		//add files for the common file export
		virtual void AddCommonExportFilesToList(List<Asura_String>& xNameList);

		//whether or not to display the 'press any key to continue' prompt after an export
		virtual bool ShouldPauseAtEndOfExport( void ) const;
		//Deals with waiting for a key press at the end of export
		void PauseAtEndOfExport();

		// Whether this animation should be exported uncompressed OR have keyframes removed...
		virtual bool ShouldExportAnimationUncompressed(const Asura_Hierarchy_Anim& xAnimation) const;
		virtual bool ShouldExportAnimationWithoutRemovingKeyframes(const Asura_Hierarchy_Anim& xAnimation) const;

		// Couple of virtual functions to control whether botanical and carpet textures are added
		// during the AddEnvironmentTextures phase of the export process.
		virtual bool ShouldExportBotanicalTextures() const { return true; }
		virtual bool ShouldExportCarpetTextures() const { return true; }

		//adds asura files listed in an ini file to the export list.
		//(files should be under the [Asura Files] and [Textures] section heading)
		//Flags can be set with "Flags=0x????????" in the ini file for all following textures in the [Textures] section
		void AddAsuraFilesFromTextFile(const char* pszFilename, const bool bAllowMaterials=false);

		//Adds all the files that match the given filter in the directory (and subdiretories) to the export list
		//This also stores the names of the any skin files found in m_pxSkinFileList
		void AddDirToExportListMain(List<Asura_String>& xNameList,const char* pszDirectory,Asura_String xFilter="*.asr",bool bWarnOnEmpty = true) ;

		//Adds all the animations and skins in the directory (and subdiretories) to the export list
		//This also stores the names of the skin files in m_pxSkinFileList
		void AddAnimDirToExportListMain(List<Asura_String>& xNameList,const char* pszDirectory, bool bWarnOnEmpty = true, const bool bAllFilesAreAnims = false) ;

		//Adds a single skin by name or hash
		void AddSkinToExportListMain(Asura_Hash_ID uSkinHash,const char* szName);

		//Adds a single animation by name or hash
		void AddAnimToExportListMain(Asura_Hash_ID uAnimHash,const char* szName);

		//Adds the animation table and all animations that it refers to.
		void AddAnimTableToExportListMain(Asura_Hash_ID uTableHash,const char* szName);

		//Adds an anim table to the list of anim tables to ignore. Will remove a table if it's already been set for export
		void AddAnimTableToIgnoreMain(Asura_Hash_ID uTableHash);

		//adds directory to the list of dirs dealt with , without actually adding any of the files
		//contained within it
		void AddAnimDirToIgnore(const char* pszDirectory);
		void AddFileToIgnore(const char* pszFilename);

		//calls AddFileToIgnore for each file in [asura files] section of text file.
		void IgnoreAsuraFilesInTextFile(const char* szFilename);

		//Prevents texture from being exported by adding it to the list that have already been done.
		void AddTextureToIgnore(const char* szFilename);

		//Prevents events/samples from being exported by adding it to the list that have already been done.
		void AddSoundEventToIgnore(Asura_Hash_ID uEventHash);
		void AddSoundEventToIgnore(const Asura_Sound_Event* pxEvent);
		void AddSoundSampleToIgnore(Asura_Hash_ID uSampleHash);
		void AddSoundSampleToIgnore(const Asura_Sound_Sample* pxSample);

		void AddBlueprintTypeToIgnore(Asura_Hash_ID uBlueprintType);
		virtual bool ShouldExportBlueprint(const Asura_Blueprint* pxBlueprint, const bool bExportChildBlueprints);

		#ifdef ASURA_USE_ANIMATION_IK_SOLVER
		//Marks the IK templates for inclusion in this export
		void AddIKTemplatesToExport(bool bAdd) {m_bIncludeIKTemplates = bAdd;}
		#endif


		//Override to filter out files from being added by AddFileToExportList
		virtual bool Project_ShouldIgnoreFile(const char* pszFilename){return false;}

		//Adds all cutscene animation/skin files to file list , assuming they should be exported
		void AddMainFileCutsceneFilesToList(List<Asura_String>& xNameList);
		//Adds animation/skin files for single cutscene (bypassing checks for whether cutscene stuff should be exported)
		virtual void AddCutsceneFilesToList(List<Asura_String>& xNameList,class Asura_Cutscene* pxCutscene);
        bool AddFileToList(List<Asura_String>& xNameList, const Asura_String& xName, bool bIsSkinFile);

        virtual void AddInGameFilesToList( List<Asura_String>* pxList, const Asura_String& xEnvFilename );

		//Adds an asura file to a list of filenames of files that will then be loaded automatically
		//when the exported level is loaded.
		void AddSeperatelyLoadedFile(const char* pszFilename);

		//Used for filtering out unwanted chunks during level export.
		//Derived classes should return false if the base class does so. (unless you really want the extra junk...)
		virtual bool ShouldChunkBeExported(int id);

		//gets the filename/path to use for the exported level file
        virtual const char* GetPlatformExtension( void );
		virtual void GetExportedLevelFilename(const Asura_String& environmentFilename,Asura_String& outputFilename);
		#if defined(ASURA_USE_TEXTURE_STREAMING)
		virtual void GetExportedLevelTextureStreamingFilename(const Asura_String& xEnvironmentFilename, Asura_String& xOutputFilename);
		#endif	//defined(ASURA_USE_TEXTURE_STREAMING)
        virtual void GetExportedCharacterDir(Asura_String& outputDir, bool bCreate);
        virtual void GetExportedCutsceneFilename( const Asura_String& xEnvFilename, const Asura_String& xCutsceneName, Asura_String* pxOutputFilename );
        virtual void GetExportedInGameFilesFilename( const Asura_String& xEnvFilename, Asura_String* pxOutputFilename );
		virtual void GetExportedSkyboxFilename(const Asura_String& xEnvFilename,Asura_String& xOutputFilename);
		virtual void GetGlobalSoundEventFilename(Asura_String& xOutputFilename);
		virtual void GetCommonExportFilename(Asura_String& xOutputFilename);
		#ifdef ASURA_USE_GUI_MENU_SYSTEM
		virtual void GetGUIMenuFilename(const char* szMenuFile,Asura_String& xOutputFilename);
		virtual void GetGUIMenuAudioFilename(const char* szMenuFile,Asura_String& xOutputFilename);
		#endif
		#ifdef ASURA_USE_AUDIO_SYSTEM
		virtual void GetStreamingSoundsFilename( Asura_String& xOutputFilename );
		#endif

		bool IsExportedLevelFileWriteable(bool bShowMessage=true);
		bool IsExportedSkyboxFileWriteable(bool bShowMessage=true);
		bool IsGlobalSoundEventFileWriteable(bool bShowMessage=true);
		#ifdef ASURA_USE_GUI_MENU_SYSTEM
		bool IsGUIMenuFileWriteable(const char* szMenuFile,bool bShowMessage=true);
		bool IsGUIMenuAudioFileWriteable(const char* szMenuFile, bool bShowMessage=true);

		bool AddGUIMenuAudioData( HANDLE pSaveHandle);
		#endif

		//Checks specified file , plus all language variations of it
		bool IsWriteableIncludingLanguageFiles(const char* szMainFile,bool bShowMessage, bool bOnlyCheckLanguageFiles = false);


		//If the function returns true , then a pointer to a modified version of the chunk is filled in at the chunk buffer
		virtual bool ModifyChunkBeforeOutput(const Asura_String& xFilename,const Asura_Chunk_Header* pxChunk,char** ppcOutputChunkBuffer);


		//for setting whether subdivision is done on waypoints,camera tracks , or whatever
		virtual void InitialiseSubdivision();

		//add project specific controls to the level export overlay
		virtual void AddOverlayControls(class Asura_Overlay* pxOverlay,class Asura_Vector_2& xOffset){};
		virtual void AddCharacterExportOverlayControls(Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset);

		static void AddPlatformExportDialogs(Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset, void(*pfnCallback)(int));
		//Adds tickboxes for enabling/disabling each platform for export
		static void AddPlatformExportTickboxes(Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset);

		// override for projects that want to hide some platforms.
		virtual bool ShowAllExportOptions() const {return true;}
		// Allow project specific batch commands.
		virtual void BatchExport_ProcessIdentifiers(Tools_IniFile* pxBatchIniFile) {}
		virtual bool BatchExport_ProjectExport() {return false;} // return true to prevent further exports.

		virtual void ExportLevel( void );
        virtual void StartLevelExport( void );
        virtual void EndLevelExport( void );
		//Does some checks to determine whether level export should go ahead. Return false to abort the export.
		//(Despite the name , this happens immediately after StartLevelExport)		
		virtual bool PreLevelExportChecks();


        virtual bool StartCharacterExport( const Asura_String& xDir );
        virtual void EndCharacterExport( void );

		//functions for exporting just the skybox from the level.
		//Note objects merged into skybox not supported yet. RAR 09/03/2006
		virtual void ExportSkybox();
        virtual void StartSkyboxExport( void );
        virtual void EndSkyboxExport( void );

		//global sound event export
		virtual void ExportGlobalSoundEventsMain(bool bToolsExport);
		virtual void StartGlobalSoundEventExport();
		virtual void EndGlobalSoundEventExport();

		virtual void ExportCommonFileMain();
		virtual void StartCommonFileExport();
		virtual void EndCommonFileExport();
		//return true to remove the UI options for exporting the common file for individual platforms
		virtual bool OnlyAllowCommonExportForAllPlatforms() {return false;}

		//global menu export
		#ifdef ASURA_USE_GUI_MENU_SYSTEM
		virtual void ExportGUIMenusMain(const char* szMenuFile);
		virtual void ExportGUIAudioMain(const char* szMenuAudioFile);
		virtual void StartGUIMenuExport();
		virtual void EndGUIMenuExport();
		virtual void StartGUIMenuAudioExport();
		virtual void EndGUIMenuAudioExport();
		#endif

		#ifdef ASURA_USE_AUDIO_SYSTEM
		virtual void ExportStreamingSoundsMain();

		virtual void CreateStreamingSoundsChunk(Asura_Chunk_Stream& xStream,
			const Asura_List<Tools_StreamingSoundExportInfo>& xTree, const bool bDialogue);
		#endif

		//gets a list of all directories to export when using the export all character file option
		virtual void GetCharacterDirectories(List<Asura_String>& xDirectoryList);

		//add all files to be included in export to export list. By defaul just includes everything in the given directory
		virtual void AddFilesForCharacterExport(List<Asura_String>& xFileList,const char* szDirectory);

		//Determines whether exported character and level files should use the huffman compression
		virtual bool ShouldCompressExportedFiles(){return((s_eExportType == Export_PC) || (s_eExportType == Export_Xbox360) || (s_eExportType == Export_Wii));}

		//Override and return true , if the sounds from the ini files should be added when doing character exports
		virtual bool ShouldSoundsBeIncludedInCharacterExports(){return false;}
		//Override and return true if you don't want non-environmental sounds in the export.
		virtual bool SkipNonEnvironmentalSounds() {return false;}
        // Override to specify default sample rate for specified sound, when
        // not found in .ini files, etc.
        virtual int GetDefaultMaxSoundFrequency( const char* szWavFilename );

		//Determines whether sounds listed in the ini files should be added for ignored asura files during
		//environment exports. By default they get added if sounds aren't being added to character exports (on
		//the principle that they need to be added to one or the other typically).   RAR 23/05/2006
		virtual bool ShouldAddSoundsForIgnoredAsuraFiles(){return !ShouldSoundsBeIncludedInCharacterExports();}

		//This should be overridden to enable/disable shapes on an export-type (or whatever) basis.
		//(Using the functions in Tools_ShapeTable for handling this)
		virtual void FlagShapeTableEntriesForExport() {};

		//Allows a project to make changes to the texture conversion parameters.
		//This is called as the final step before checking whether the converted file is up to date.
		virtual void Project_ModifyTextureConversionSettings(struct TextureConversionData* pxConvertData,int iTextureID, const char* szTextureName){};

		
		//Allows a project to make changes to the sound conversion parameters.
		//This is called as the final step before checking whether the converted file is up to date.
		virtual void Project_ModifySoundConversionSettings(struct ToolsSoundConversionData* pxConvertData, const char* szWavFilename, bool bStreaming){};


		//GetSharedLanguageFilename() should be overidden if several level files are intended to make use of the the same localised
		//files for dialogue lines and sounds. If used then the export process will check the existing file for dialogue lines , and
		//add those to the current export , so in the end the file should contain all the resources used by all the levels that are sharing it.
		//Note that there is nothing at the moment for detecting when resources are no longer required in a file , so it may
		//be advisable to delete them on occasion if they build up too much junk.
		//Back to the function at hand...
		//This should return true if this export is using a shared language file , and fill in xOutputLanguageFilename to
		//give the base name of the file to use. The extension used is irrelvant , since it will be replaced anyway.
		//Note that this may be called before the export gets fully underway , so checks for determining the type
		//of export (common,level etc) may not work.
		//RAR 10/07/2007
		virtual bool GetSharedLanguageFilename(const char* szMainExportFile,Asura_String& xOutputLanguageFilename) const {return false;}

		//returns true if the texture is one of the skybox textures
		bool IsSkyboxTexture(int iTextureIndx);

		//Adds file containing shape meshes to export
		void AddShapeFileToExportList(const char* pszFilename);

		//  AAB:  (14/11/2003 12:31:38):  Made protected, so that derived classes can use them.
		void SetMainHeading(const char* szHeading) { s_szMainHeading = szHeading; }
		void StartExport();
		void EndExport(bool bContinuePrompt = true);
		void WriteExportSummary();
		void ResetExportSummaryStats();
		virtual void StartFileExport();
		virtual void EndFileExport();
		bool AddAsuraFile( HANDLE hFile, const Asura_String& filename, unsigned uIndexWithinFileList, bool bMissingFileIsError = false );
		bool AddAsuraFileMain( HANDLE hFile, const Asura_String& filename, unsigned uIndexWithinFileList, bool bMissingFileIsError);

		void CompressFile(const char* filename);
		void CompressFileReport( const char* filename, const u_int uUncompressedFileSize, const u_int uCompressedFileSize );

		static bool ProcessExportList( const char* szDirectory, List<Asura_String>* pxSkinFileList, List<Asura_String>* pxAnimFileList );
		void AddFileToSkinList(const Asura_String& xName);
		void AddFileToAnimList(const Asura_String& xName);
		void OpenExportLog(const char* pszExportedFileName);
		void CloseExportLog();
		void UpdateProgress();


		bool IsFakeLevelExport() const {return m_bFakeLevelExport;}

		const Asura_String& GetEnvironmentFilename() const { return m_xEnvironmentFileName; }
		bool ExportCharacterFile( const char* directory, const char* szOutputFilename = NULL );

		#if defined ( ASURA_USE_TEXTURE_STREAMING )
		bool ExportStreamingTexturesFile( void );
		virtual bool ShouldExportStreamingTexturesFile() const { return !m_bFakeLevelExport && !MissionExport_IsExportingMissionOnly(); }
		#endif

		// Function to allow projects to associate extra textures/materials with a skin.
		// The list starts with the default textures/materials for the skin , and can be emptied
		// if the defaults aren't wanted.
		#ifdef ASURA_USE_MATERIAL_RESPONSE_SYSTEM
		virtual void Project_AddMaterialsForSkin(List<Asura_Hash_ID>& xMaterialList, Asura_Hierarchy_Skin* pxSkin, HANDLE xSaveHandle) {}
		#else
		virtual void Project_AddTexturesForSkin(List<int>& xTextureIndices, Asura_Hierarchy_Skin* pxSkin, HANDLE xSaveHandle=NULL) {}
		#endif
		void LogAnimationExportStats();

		//This should call AddGameSoundsIniFile() for any ini files that list permanently
		//loaded game sounds
		virtual void Project_GetGameSoundList(){}

		//Adds files listed in the inifile as 'already exported'. This allows environmental sounds
		//etc to be mapped to those sound IDs , instead of the file being included again.
		void AddGameSoundsIniFile(const char* pszFilename);

		// Adam A. Brown (25.10.2004 14:21:43) : Added accessor functions an protectorised some stuff
		//to allow flexible use of export functionality for project-specific purposes.
		List<Asura_String>* GetFileList() { return &m_xAsuraFileList; }
		
		//Exports the files in m_xAsuraFileList. If its an environment , then some extra
		//textures and sounds get chucked in at the same time
		virtual void ExportFilesInList(const char* pszFilename,bool bEnvironment);

		//Causes the shapes used by static objects/cutscenes (the ones that get shown in the shape table
		//to the artists) to be included in the export
		//Needs to be done after StartFileExport and before ExportFilesInList and AddFragmentsToExport
		void SetIncludeEntityShapes(bool bInclude){m_bIncludeEntityShapes = bInclude;}
		//Causes the 'common' shapes to be included in the export. These are the shapes that
		//appear in the [SHAPES] section of the common_export.txt file etc.
		//Needs to be done after StartFileExport and before ExportFilesInList and AddFragmentsToExport
		void SetIncludeCommonShapes(bool bInclude){m_bIncludeCommonShapes = bInclude;}

		//enables inclusion of sounds associated with skins and shapes.
		void SetIncludeSkinAndShapeSounds(bool bInclude){m_bIncludeSkinAndShapeSounds = bInclude;}

		//Adds fragment files that are required by the shapes currently set for export
		void AddFragmentsToExport();

		//If using streaming , it is assumed that the skybox will be done separately
		#ifdef ASURA_USE_STREAMING
		virtual bool ShouldIncludeSkyboxInLevelExport( bool bEnvironmentExport ) { return false;}
		#else 
		virtual bool ShouldIncludeSkyboxInLevelExport( bool bEnvironmentExport ) { return bEnvironmentExport; }
		#endif


		//Projects might want to use different hierarchy sound ranges for weapon or character exports
		#if ( defined ASURA_MAIN ) && !( defined ASURA_USE_AUDIO_SYSTEM )//(prevents inclusion by resourcepacker etc)
		virtual Asura_Sound_ID GetMinHierarchySoundID() const {return ASURA_SID_HIERARCHY_MIN;}
		virtual Asura_Sound_ID GetMaxHierarchySoundID() const {return ASURA_SID_HIERARCHY_MAX;}
		#endif

		virtual bool ShouldCutsceneBeExportedToThisPlatform(Asura_Cutscene* pxCutscene);

		virtual bool ShouldIgnoreAlwaysExportPFXFlag( void ) const { return false; }
		
#ifdef ASURA_USE_MATERIAL_RESPONSE_SYSTEM
		void AddMaterialToExport(HANDLE xSaveHandle, const Asura_Hash_ID uMaterialID, Asura_HashTree& xMaterialExportTree, Asura_HashTree& xTextureExportTree, bool bIncludeTextures);
		void AddMaterialToExport( HANDLE xSaveHandle, const Asura_Hash_ID uMaterialID, bool bIncludeTextures  = true );

		//Prevents the material and any textures it uses from being exported.
		//Unlike some if the other 'Ignore' functions this really needs to be done before the material
		//gets added to supress it properly , since if done afterwards it will be too late to prevent
		//the texture export.
		void AddMaterialToIgnore(Asura_Hash_ID uMaterialID);

		static void BeginMRPShaderExport();
		static void EndMRPShaderExport( const HANDLE xExportHandle );

		static void AddMRPShader_Internal( const Asura_Hash_ID uMaterial1,
			const Asura_Hash_ID uMaterial2, const bool bSkin );
#endif

		void AddEnvironmentTextures(HANDLE saveHandle);
		void ExcludeEnvironmentTextures();
		void DealWithEnvironmentTextures(HANDLE saveHandle, bool bExclude);
		void AddSkyboxTextures(HANDLE pSaveHandle);
		void AddSkinTextures(HANDLE pSaveHandle);
		void AddTexturesForSkin(HANDLE saveHandle,const Asura_String& xFilename);
		void AddTexturesForSkin(HANDLE saveHandle,class Asura_Hierarchy_Skin* skin);

		bool AddTexture(HANDLE saveHandle, const Asura_String& filename,int index,bool bSRGB = true);
		void CheckForInvalidTextureFlags( const Asura_String& filename, int index );
        bool GenerateTextureIfNecessary( const Asura_String& xName,
                                         int iTexID,
                                         bool bQuiet,
										 bool bSRGB,
                                         Asura_String* pxOutputFilename = NULL,
                                         TextureConversionData* pxOutputConversionData = NULL);

		//Adds textures that have been queued up by AddTextureToExportList
		void AddMiscTextures(HANDLE pSaveHandle);

		//Callback used when adding entity textures
		static void AddEntityTextureToExportList(int iTextureID, void* pNotUsed);

		void AddSounds(HANDLE saveHandle,const char* pszHeading,int iMinID,int iMaxID);
        void AddSound( HANDLE saveHandle, const Asura_String& filename, int iSoundID, int iFrequency );
		void AddSound(HANDLE pSaveHandle,const SoundExportData* pxSound);
		void AddSound(HANDLE pSaveHandle,const SoundExportData* pxSound, int iLanguage);
		bool TargetPlatformSupportsStreamedSounds();
		void AddStreamedSoundToList(const SoundExportData* pxSound, const char* szLocalisedFilename, Asura_Hash_ID uFilenameHash);
		void AddStreamedSoundsFromList(HANDLE pSaveHandle);

		//Adds any localised sounds that have been queued up
		void AddLocalisedSounds(HANDLE pSaveHandle,int iLanguage);
		//Adds any localised streamed sounds that have been queued up
		void AddLocalisedStreamedSounds(HANDLE pSaveHandle,int iLanguage);

		//For adding the keyframe sounds to export
		void AddSkinSounds(HANDLE pSaveHandle);
		void AddSoundsForSkin(HANDLE pSaveHandle,const Asura_String& xFilename);
		void AddSoundsForSkin(HANDLE pSaveHandle,const Asura_Hierarchy_Skin* pxSkin);
		void AddSoundsForAnim(HANDLE pSaveHandle,const Asura_String& xFilename);
		void AddSoundsForAnim(HANDLE pSaveHandle,const class Asura_Hierarchy_Anim* pxAnim);


		//Adds sounds used by phonons and shape sounds(which fall in the same range)
		void AddEnvironmentalSounds(HANDLE pSaveHandle);

		//deals with adding sounds with ids outside of the environmental range
		void BuildNonEnvironmentalSoundList(const List<Asura_String>& xNameList);
		void AddNonEnvironmentalSounds(HANDLE pSaveHandle);

		void AddShapes( HANDLE pSaveHandle );
		void AddShapeTextures( HANDLE pSaveHandle );

		//deals with the tables that have been added to the export list
		void AddAnimationTables(HANDLE pSaveHandle);

		void AddSoundMappingResource(HANDLE hFile);	
		void AddSoundPropertiesResource(HANDLE hFile);	
		void AddFilenameResources(HANDLE hFile);
		void AddFilenameResource(HANDLE hFile, const char* szFilename, bool bLocalisedFile);

		//Adds any sound events , samples , and their wav files that have been queued up
		bool AddSoundEvents(HANDLE hFile);
		//Adds all global sound events to export
		void AddGlobalSoundEventsToExportList();
		//Excludes global sound events from level exports
		void ExcludeGlobalSoundEvents();

		//Used by fake level export to force all the sound events to be loaded.
		void LoadSoundEventWavs();
		
		#ifdef ASURA_USE_DIALOGUE_SYSTEM
		//Adds any dialogue events , lines , and their wav files that have been queued up
		bool AddDialogueEvents(HANDLE pSaveHandle);
		//Adds the localised parts of dialogue events to the file
		bool AddLocalisedDialogueEvents(HANDLE pSaveHandle,int iLanguage);
		//Adds all global dialogue events to export
		void AddGlobalDialogueEventsToExportList();
		//Excludes global dialogue events from level exports
		void ExcludeGlobalDialogueEvents();
		// Project-overridable function to skip the sound file of a dialogue event. Base class version just checks the m_bNoWavForPSP flag,
		virtual bool GetExportDialogueLineSound(const Asura_Dialogue_Line_Template* pxTemplate) const;
		#endif

		#if defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)
		// Typedefs for facial-animation data-containers...
		typedef Asura_TemplateHashTree<const Asura_FacialAnimation_Face*> FaceContainer;
		typedef Asura_TemplateHashTree_Iterator<const Asura_FacialAnimation_Face*> FaceIterator;
		typedef Asura_TemplateHashTree_ConstIterator<const Asura_FacialAnimation_Face*> FaceConstIterator;

		typedef Asura_TemplateHashTree<const Asura_FacialAnimation_FaceAnimation*> FaceAnimationContainer;
		typedef Asura_TemplateHashTree_Iterator<const Asura_FacialAnimation_FaceAnimation*> FaceAnimationIterator;
		typedef Asura_TemplateHashTree_ConstIterator<const Asura_FacialAnimation_FaceAnimation*> FaceAnimationConstIterator;

		// Add the facial-animation data to the export...
		static bool AddFaceAnimationToExportList(const char* szFaceAnimationName, const Asura_Hash_ID uFaceAnimationHash, FaceAnimationContainer* pxFaceAnimationContainer);
		//Adds any facial-animation data that has been queued up
		bool AddFacialAnimationData(HANDLE pSaveHandle);
		//Adds the localised parts of dialogue events to the file
		bool AddLocalisedFacialAnimationData(HANDLE pSaveHandle,int iLanguage);
		//Checks for any combined animatin/facial animation files that have been added to the export list , 
		//and adds the associated facial animations.
		void AddFaceAnimsUsedByStandardAnimsToExportList();

		#endif	//defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)

		//Adds any blueprints that have been queued up
		bool AddBlueprints(HANDLE hFile);

		#ifdef ASURA_USE_CONTAINER_SYSTEM
		bool AddContainers(HANDLE hFile);
		#endif //ASURA_USE_CONTAINER_SYSTEM

		#ifdef ASURA_USE_MATERIAL_RESPONSE_SYSTEM
		bool AddMaterialResponses(HANDLE hFile);
		#endif

		#ifdef ASURA_USE_GUI_MENU_SYSTEM
		//Adds any menus that have been queued up for export
		bool AddGUIMenus(HANDLE hFile);
		#endif


		bool LeaveSpaceForFileInfoChunk( HANDLE hFile );
		bool FillOutFileInfoChunk( HANDLE hFile, u_int uFileSize );

		//Called at end of ExportFilesInList to create any language specific files if needed.
		void ExportLanguageFiles(const char* szMainFilename);
        		
		//Flags export as needing the localised files , and creates a resource chunk for loading the localised
		//file if needed.
		void SetCreateLanguageFiles(HANDLE pSaveHandle);

		//For use when using localised files that are shared across several level exports.
		//This checks the existing file for any dialogue lines that are being used, and adds them
		//to the current export.
		void GetResourcesFromExistingLanguageFile(const char* szMainExportFilename);
        		

        static bool s_bPS2MultiplayerExport;

		void TransferIniFileData(int iMode);

		bool CheckPrelightingIsCorrect();
		virtual void ExportAllCharacterFiles();
		virtual void ExportSingleCharacterFile( void );

		void UpdateExportedLevel();

		//Updates the files in m_xAsuraFileList in an existing exported level file.
		void UpdateFilesInList(const char* pszFilename);

		//Loads the files in m_xAsuraFileList for fake level export
		void LoadFilesInList();
		
		void AdjustAllUVCoordinates();
		void AdjustUVCoordinatesMain(Asura_Mesh* pxMesh,int* piResetPolygonCount = 0,int* piDodgyPolygonCount = 0);
		void AdjustUVCoordinates(class Asura_Hierarchy_Skin* pxSkin);


		void PreAddAsuraFile( HANDLE hFile, const Asura_String& filename);
		bool PostAddAsuraFile( HANDLE hFile, const Asura_String& filename);
		
		bool CreateTim2File(const char* inputName,const char* outputName,int index, const TextureConversionData& xConversionData, int iWidthOveride, int iHeightOveride );
		bool CreateTPLFile(const char* inputName,const char* outputName,int index, const TextureConversionData& xConversionData );

		//returns true if texture has values other than 255 for alpha
		bool DoesTextureHaveAlphaChannel(const char* pszFilename);
		
		bool IsTextureUsedOnEnvironment(int index);

		//Gets the overides for the tim2 sizes from the ini file. Returns false if the size isn't overridden
		bool GetPS2TextureSizeOverride(int iTextureID,int* piOutputWidth,int* piOutputHeight,int* piPixelFormat,int* piPaletteFormat);
		void GetUVScaleForPS2Skin(const Asura_Hierarchy_Skin* pxSkin,float * pfUScale,float* pfVScale);

		//display a single message on black background
		static void DisplayProgressMessage(const char* message);

		void WriteDTXnFile(DWORD count, void * buffer);

		static bool DirectoryIsAnimOnly(const char* directory);

		bool AddStripifiedSkin( HANDLE pSaveHandle, Asura_Hierarchy_Skin* pxSkin );
		void PadToQWordBoundary(HANDLE pSaveHandle);
		void AddStripifiedEnvironment(HANDLE pSaveHandle);

        bool AddSkinSectionBBChunk( HANDLE hFile, const Asura_String& xSkinName, Asura_Hierarchy_Skin* pxSkin );

		bool IsSkinFile(const Asura_String& xFilename);
		bool IsEnvironmentFile(const Asura_String& xFilename);

		static bool ConversionDataFileMatches(const char* textureFilename,const struct TextureConversionData* convertData);
		static void CreateConversionDataFile(const char* textureFilename,const struct TextureConversionData* convertData);

		bool ShouldCutsceneBeExportedToOwnFile(Asura_Cutscene* pxCutscene);


		//loads sound IDs from Asura_ProjectSoundIDs.h (if not already loaded)
		class Tools_EnumerationReader* LoadSoundIDEnumeration();

		//Adds sounds listed in ini file to  list. Returns false if the ini file isn't loaded.
		//The optional callback function can be used to select a subset from the ini file , and should return
		//true if a sound should be included in the export.
		static bool AddSoundsFromIniFile(const char* pszFilename,class Tools_EnumerationReader* pxSoundEnum,List<SoundExportData*>& xSoundList,bool (*pfnShouldExport)(int iSoundID) = 0);
		
	

		unsigned int ExportBigEndianAnimation(Asura_Chunk_Header* pxChunk, HANDLE hFile);
		static bool AddBigEndianSkinChunks(HANDLE hFile, Asura_Hierarchy_Skin* pxSkin);

		//Writes an animation out directly to the export , compressing if needed
		static u_int AddAnimationToExport(HANDLE pSaveHandle, const Asura_Hierarchy_Anim* pxAnim);			
		static u_int AddAnimationToExport(HANDLE pSaveHandle, const class Asura_Chunk_Hierarchy_Anim* pxHierAnimChunk);			


		#ifdef ASURA_USE_WATER
		void AddWaterPackets(HANDLE hFile);
		#endif

		#if defined( ASURA_USE_CARPET )
		bool AddCarpetMeshData( HANDLE pSaveHandle );
		#endif // #if defined( ASURA_USE_CARPET )

		#if defined( ASURA_USE_BOTANICALS )
		void AddBotanicalsTextures( HANDLE pSaveHandle );
		bool AddBotanicalsData( HANDLE pSaveHandle );
		#endif // #if defined( ASURA_USE_BOTANICALS )

		#ifdef ASURA_USE_STATIC_DECAL_SYSTEM
		bool AddStaticDecalsData(HANDLE pSaveHandle);
		#endif

#ifdef ASURA_USE_ATTRACTORS
		bool AddAttractorData( HANDLE pSaveHandle );
#endif

#if defined ASURA_USE_REVERBVOLUME_SYSTEM && defined ASURA_USE_AUDIO_SYSTEM
		bool AddReverbs( HANDLE pSaveHandle );
#endif

		// Functions to add various types of chunked data to the specified file...
		bool AddChunkStream(Asura_Chunk_Memory_Stream& xStream, HANDLE pSaveHandle);
		bool AddSaveBlock(Asura_SaveBlock& xSaveBlock, HANDLE pSaveHandle);

		//For determing which export options should be enabled
		bool ShouldExportLevelBeEnabled(EXPORT_TYPE ePlatform);

		//Exports any language specific bits that were picked up during ExportFilesInList()
		void ExportLanguageFile(const char* szMainFilename,int iLanguage);
		void GetLanguageFilename(const char* szMainFilename,int iLanguage,Asura_String& xOutputFilename);

		//callbacks for language export options
		static void EnableAllLanguagesForExport();
		static void DisableAllLanguagesForExport();

		//callbacks for light set export options
		static void EnableAllLightSetsForExport(){s_uLightSetsToExport = 0xffffffff;}
		static void DisableAllLightSetsForExport(){s_uLightSetsToExport = 0;}

		//callbacks for platform export options
		static void EnableAllPlatformsForExport();
		static void DisableAllPlatformsForExport(){s_uPlatformsToExport = 0;}


		//Scans through list of asura filenames ,and removes any that appear to be intended for other platforms.
		void RemoveFilesForOtherPlatforms(List<Asura_String>& xFileList);

		static bool IsEnabled_MaximumTextureSize(){return s_bUseMaximumTextureSize;}



		static bool s_bAbort;
		static bool s_bUserAbort;
		static int s_iPercent;
		static const char* s_szMainHeading;
		static int s_iNumErrors;
		static EXPORT_STAGE s_eThisStage;
		static EXPORT_STAGE s_eCurrentStage;

		static bool s_bSubdivideMeshes;
		static bool s_bSwizzleTims;
		static bool s_bRecalculatePrelighting;
		static float s_fFlatSubdivideSize;
		static float s_fCurveSubdivideSize;
		static bool s_bDontExportCutscenes;
        static bool s_bIncludeDualTexturingData;
		static bool s_bFullBrightPrelighting;
		static bool s_bExportAllCharacters;//for animation export overlay. export current character/all characters
        static bool s_bIsLevelExport;
		static bool s_bIsSkyboxExport;
		static bool s_bIsGlobalSoundEventExport;
		static bool s_bIsCommonFileExport;
		static bool s_bIsGUIMenuExport;
		static bool s_bIsGUIMenuAudioExport;
		static bool s_bIsExport;
		static bool s_bIsCharacterExport;
		static bool s_bUpdateExistingFile;
		static bool s_bDisplayingProgressMessage;
		static bool s_bDebugColourSubstrips;
		static bool s_bForceTextureGeneration;
		static bool s_bUseNewNVidiaExport;
		static bool s_bWiiStreamsUseBink;
		static bool s_bCompressionDisabled;

		static Asura_Vector_3 s_xExportCentre;

		static bool s_bFilterEntityExport;
		static List<int> s_xEntityClassesToIgnore;//list of entity classifications not to export

		
		static bool s_bOverrideFrameRemovalSettings;
		static float s_fAnimPositionTolerance;
		static float s_fAnimAngleTolerance;
		static float s_fMinGCTexRes;
		static float s_fMaxMipDrops;

		static u_int s_uPlatformsToExport;//bitfield of platforms to be exported to

		static u_int s_uCurrentLightSetsToExport;
		static u_int s_uLightSetsToExport;//bitfield of all lightsets to be exported
		static u_int s_uInitialLightSets;

		//Stores user selection of languages to export (for localised dialogue events)
		static bool s_abExportLanguage[];
		static bool s_bLanguageArrayInitialised;
		static bool s_bOnlyExportLocalisedGlobalSounds;//for global sound exports, skip exporting the non-localised part

		//Override for clamping texture sizes upon export.
		static u_int s_uMaximumTextureSize;
		static bool s_bUseMaximumTextureSize;


		//stores a message that will only get displayed if some other message is subsequently displayed
		static Asura_String s_xMessageDisplayHeading;

		static int s_iForcedBotanicalTextureFlagsPS2;	//CSB 28/04/06
		static Asura_Char* s_aszExportStageText[EXPORT_STAGE_TOTAL];

		int m_iNumberOfUntexturedPolygons;
		int m_iNumberOfPolygonsWithMissingTexture;
		int m_iUntexturedTextureIndex;
		int m_iUntexturedMaterialIndex;

		//for keeping track of frames removed during export
		int m_iTotalAnimationFramesBefore;
		int m_iTotalAnimationFramesAfter;
		int m_iTotalAnimationSizeBefore;
		int m_iTotalAnimationSizeAfter;
	
		//list of asura files to be included in export
		List<Asura_String> m_xAsuraFileList;
		class Asura_StringTree* m_pxAsuraFileStringTree;
		class Asura_StringTree* m_pxSkinFileStringTree;
		class Asura_StringTree* m_pxShapeFileStringTree;

#ifdef ASURA_USE_MATERIAL_RESPONSE_SYSTEM
		// list of parent materials that will be exported
		Asura_HashTree* m_pxMaterialsToExport;
		// list of slaves that won't have their responses exported
		Asura_HashTree* m_pxSlaveMaterialsToExport;
		// list of materials that have had their textures added to the export list
		Asura_HashTree* m_pxMaterialsWithExportedTexturesTree;

		//list of materials to be excluded from export
		Asura_HashTree* m_pxMaterialsToIgnore;

		Asura_NameTree* m_pxMaterialsFromCommonExport;
		Asura_HashTree* m_pxOverrideMaterialsToExport;

		// Variables to allow a list of material-response overrides to be specified for export...
		u_int					m_uNumMaterialResponseOverridesForExport;
		const Asura_Hash_ID*	m_pxMaterialResponseOverridesForExport;
#endif

		//list of textures that have been added , to avoid duplicates
		Asura_NameTree* m_pxExportedTexturesList;
		//List of textures that will be added during the AddMiscTextures stage
		List<int> m_xMiscTextureList;
		//List of GUIMenu textures. Separated from the above because overlay quads shouldn't be sRGB-corrected
		List<int> m_xGUIMenuTextureList;
		//list of textures being used by skins
		Asura_NameTree* m_pxSkinTextureList;
		//list of skin files , these will need special handling during AddAsuraFile
		Asura_NameTree* m_pxSkinFileList;
		Asura_NameTree* m_pxAnimFileList;
		//list of animation directories that have been dealt with , to prevent searching them multiple times
		Asura_NameTree* m_pxAnimDirList;
		//list of directories to ignore when adding asura files
		List<Asura_String> m_xDirsToIgnore;
		List<Asura_String> m_xFilesToIgnore;
		//name of the env file for special handling in AddAsuraFile;
		Asura_String m_xEnvironmentFileName;

		//store of things that were added to the environment , so cutscene export knows not to bother
		List<Asura_String> m_xEnvAsuraFileList;
		Asura_NameTree* m_pxEnvExportedTexturesList;

		//list of filenames to be added in a resource chunk in the exported level
		List<Asura_String> m_xSeperatelyLoadedFiles;
				
		List<Asura_String> m_xShapeFilenames;
		List<Asura_Mesh*> m_xShapeMeshList;

		//list of extra sounds to be exported (with SIDs outside of the environmental range)
		List<SoundExportData*> m_xSoundList;

		//list of sounds exported so far , for determing sound mapping
		List<SoundExportData*> m_xExportedSoundList;
		List<Asura_SoundMapping> m_xSoundMappingList;
		List<Tools_SoundProperties> m_xSoundPropertiesList;

		//List of sounds that will be included in the localised part of the export.
		List<const SoundExportData*> m_xLocalisedSoundList;

		//List of sounds that we will try to include in the localised streamed sounds part of the export.
		List<const SoundExportData*> m_xLocalisedStreamedSoundList;
		//List of such sounds that will actually be included, with the filename set to the converted file.
		List<const SoundExportData*> m_xLocalisedStreamedSoundExportList;

		//sound events and samples to export
		//list to export
		List<const class Asura_Sound_Event*> m_xSoundEventList;
		List<const class Asura_Sound_Sample*> m_xSoundSampleList;
		//hash tree of events/samples that have already been dealt with
		Asura_HashTree* m_pxSoundEventTree;
		Asura_HashTree* m_pxSoundSampleTree;

		#ifdef ASURA_USE_DIALOGUE_SYSTEM
		//list to export
		List<const class Asura_Dialogue_Event*> m_xDialogueEventList;
		List<class Asura_Dialogue_Line*> m_xDialogueLineList;
		List<const class Asura_Dialogue_Line_Template*> m_xDialogueLineTemplateList;
		//hash tree of events/liness that have already been dealt with
		Asura_HashTree* m_pxDialogueEventTree;
		Asura_HashTree* m_pxDialogueLineTree;
		Asura_HashTree* m_pxDialogueLineTemplateTree;
		#endif

		#if defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)
		// Containers for the facial-animation data to export...
		FaceContainer*			m_pxFaceContainer;
		FaceAnimationContainer*	m_pxFaceAnimationContainer;
		FaceAnimationContainer*	m_pxLocalisedFaceAnimationContainer;
		#endif	//defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)

		//list to export
		List<Asura_Hash_ID> m_xBlueprintTypeList;
		//hash tree of blueprints that have already been dealt with
		Asura_HashTree* m_pxBlueprintTypeTree;

		//menus to export
		#ifdef ASURA_USE_GUI_MENU_SYSTEM
		List<const class Asura_GUIMenu_Menu_Page*> m_xGUIMenuList;
		#endif

		//true if IK templates should be included in this export
		#ifdef ASURA_USE_ANIMATION_IK_SOLVER
		bool m_bIncludeIKTemplates;
		#endif

		ExportStageStats m_xExportStageStatsList[EXPORT_STAGE_TOTAL];

		//list of skin hashes that have been dealt with (may not include files added by all methods)
		Asura_HashTree* m_pxSkinHashTree;

		//list of anim hashes that have been dealt with (may not include files added by all methods)
		Asura_HashTree* m_pxAnimHashTree;

		//the anim tables that have been dealt with
		Asura_HashTree* m_pxAnimTableHashTree;
		//list of anim tables that will need to have chunks adding
		List<const Asura_AnimTable*> m_xAnimTableList;

		class Tools_EnumerationReader* m_pxSoundEnum;

		bool m_bFakeLevelExport;
		bool m_bIsCharacterExport;
		bool m_bIsToolsSoundExport; //Indicates a tools sound event export (only meaningful during a global sound event export)

		bool m_bIncludeEntityShapes;
		bool m_bIncludeCommonShapes;
		bool m_bIncludeSkinAndShapeSounds;

		bool m_bCreateLanguageFiles; //Set to true during file exports if extra language specific files should be created
		bool m_bExportingLanguageFile; //Set while in the middle of exporting the language files
		bool m_bUsingSharedLanguageFile; //Indicates that the localised files are shared between several exports , and so won't match the export's filename
		Asura_String m_xSharedLanguageFilename;

		bool m_bAddingStreamedSounds;//Set while in the middle of exporting streamed sounds.

		Asura_String m_xLevelNameBackup;

		struct Tools_IniFile* m_pxPS2TextureSizeIni;


		enum DialogID
		{
			DialogID_CreateSeperateCutsceneSkinFiles,
			DialogID_PositionTolerance,
			DialogID_AngleTolerance,

		};

		#if defined ASURA_USE_BOTANICALS
		Asura_NameTree m_xBotanicalsTextures;
		Tools_DynamicArray< Asura_Material > m_xBotanicalsMtlList;
		int m_iNumBotanicalsMtls;
		int m_aiBotanicalsMtlConvList[ ASURA_MAX_NO_OF_MATERIALS ];
		#endif // defined ASURA_USE_BOTANICALS

		#ifdef ASURA_USE_MATERIAL_RESPONSE_SYSTEM
		//Used to determine whether the texture names , texture flags , and material chunks should be copied into the export
		bool m_bIncludeOldMaterialChunks;
		#endif

		//export options for when editing mission files
		static bool s_bMission_ShouldExportEnvironment;
		static bool s_bMission_ShouldExportMission;

		//type of mission export currently being done
		static bool s_bMission_IsExportingEnvironment;
		static bool s_bMission_IsExportingMission;

		static HANDLE s_pDDSHandle;

private:
		static Asura_String s_xHeading;

public :

		//'this' substitute for all the static functions
		static Tools_Level_Export* This;

};



struct SoundExportData
{
	SoundExportData()
	{
		m_iFrequency = 0;
		m_bOnlyUsedByHash = false;
		m_bLocalise = false;
		m_bStreamed = false;
		m_bLooping = false;

		m_fPitchScale = -1.0f;
		m_fVolumeScale = -1.0f;
		m_fDefaultInnerRadius = -1.0f;
		m_fDefaultOuterRadius = -1.0f;
	}

	bool HasSoundProperties() const
	{
		return m_fPitchScale != -1.0f ||
				m_fVolumeScale != -1.0f ||
				m_fDefaultInnerRadius != -1.0f ||
				m_fDefaultOuterRadius != -1.0f;

	}
	
	int m_iID;
	Asura_String m_xFilename;
	int m_iFrequency;
	bool m_bOnlyUsedByHash;
	bool m_bLocalise;//set to true if the sound has different localised versions
	bool m_bStreamed;//set to true if the sound is streamed rather than memory-resident
	bool m_bLooping; //set to true if the sound loops

	float m_fPitchScale;
	float m_fVolumeScale;
	float m_fDefaultInnerRadius;
	float m_fDefaultOuterRadius;
};

#endif //__TOOLS_LEVEL_EXPORT_H_
