/**
*
* \file Tools_Level_Export.cpp
*
* Created by Kevin Lea - 29/11/2002 11:23:31
* Splitted into separate file 12/11/2008
*
* Copyright (C) 2002 Rebellion, All Rights Reserved.
*
* \brief
*
**/

#include "Core/Asura.h"
#include "Load&Save/LevelExport/Tools_Level_Export.h"

//Asura
#include "ChunkSystem/Asura_Chunk_Loaders.h"
#include "FileAccess/Asura_FileAccess.h"

// Tools
#include "PC/OS/Tools_OS.h"
#include "PC/Tools_PC_Export.h"
#include "PS3/Tools_PS3_Export.h"
#include "Skybox/Tools_Mode_Skybox.h"
#include "Wii/Tools_Wii_Textures.h"
#include "Xbox360/Tools_Xbox360_Export.h"

#if ASURA_USE_GUI_MENU_SYSTEM
//#include "GUIMenu/Asura_GUIMenu_Message.h"
//#include "GUIMenu/Asura_GUIMenu_Menu_Page.h"
#include "GUIMenu/Asura_GUIMenu_System.h"
#include "GUIMenu/Tools_GUIMenu_System.h"	
#endif

#ifdef ASURA_USE_CONTAINER_SYSTEM
#include "ChunkSystem/Asura_Chunk_Container.h"
#include "Entity/Container/Asura_Container_System.h"
#endif //ASURA_USE_CONTAINER_SYSTEM

// F U N C T I O N S /////////////////////////////////////////////////////////////
extern bool FillOutResourceFileListChunk( HANDLE hFile );
extern bool LeaveSpaceForResourceFileListChunk( HANDLE hFile );

// G L O B A L S ////////////////////////////////////////
extern u_int g_uAsuraFilesInExport;
extern ResourceFileListEntry* g_axResourceFileListEntries;
extern bool g_bPreparedHashMapFile;

void Tools_Level_Export::AddFilesForCharacterExport(List<Asura_String>& xFileList,const char* szDirectory)
{
	AddAnimDirToExportList(xFileList,szDirectory);
}

bool Tools_Level_Export::ExportCharacterFile( const char* szDirectory, const char* szOutputFilename )
{
	HANDLE saveHandle = INVALID_HANDLE_VALUE;
    DWORD bytesWritten;
	bool bOK = true;

	s_bIsCharacterExport = true;
   	#define CHECK( expr ) \
   	    if( !(expr) ) { bOK = false; goto End; }
	
    //get a list of asura files to add
	AddFilesForCharacterExport(m_xAsuraFileList,szDirectory);

	if(!m_xAsuraFileList.size() && !m_xMiscTextureList.size())
	{
		AddMessage(MT_INFORMATION,"No files found.");
		return true;
	}
	
	// (MH - 12.08.2002) this has to go here because of the evil goto
    Asura_String xOutputDir;


	//get the filename from the last part of the directory name
	char filename[MAX_PATH];

    {
        GetExportedCharacterDir( xOutputDir, true ); // create if not exist

		const char* const szFileExtension = ShouldExportBigEndian() ? ASURA_STANDARD_FILE_EXTENSION_BE : ASURA_STANDARD_FILE_EXTENSION_LE;

        {
	        const char* slash = strrchr(szDirectory,'\\');
	        if(!slash) slash = strrchr(szDirectory,'/');
	        if(!slash || !*(slash+1))
	        {
		        AddMessage( MT_ERROR, "Error choosing name for character file." );
                CHECK( false );
	        }

            Asura_PrintfToBfr( filename, "%s\\%s.%s", MAX_PATH, true, xOutputDir.GetString(), slash+1, szFileExtension );
        }

        if( szOutputFilename )
        {
            Asura_Char szNewFilename[MAX_PATH];
            const u_int uLen = Asura_PrintfToBfr( szNewFilename, "%s\\%s.%s", MAX_PATH, true, xOutputDir.GetString(), szOutputFilename, szFileExtension );

            static Asura_Char szLog[MAX_PATH];
            if( !g_bPreparedHashMapFile )
            {
                Asura_PrintfToBfr( szLog, "%s\\hashed_name_to_src_name_map.txt", MAX_PATH, true, xOutputDir.GetString() );
                g_bPreparedHashMapFile = true;
            }

	        HANDLE h = CreateFile( szLog, GENERIC_WRITE, 0, 0, OPEN_ALWAYS, 0, 0 );
            if( INVALID_HANDLE_VALUE != h )
            {
                // We want to append.
                SetFilePointer( h, 0, NULL, FILE_END );

                DWORD uIgnored;
                WriteFile( h, szNewFilename, uLen, &uIgnored, NULL );
                WriteFile( h, " -> ", 4, &uIgnored, NULL );
                WriteFile( h, szDirectory, strlen(szDirectory), &uIgnored, NULL );
                WriteFile( h, "\n", 1, &uIgnored, NULL );
                CloseHandle( h );
            }

            strncpy( filename, szNewFilename, uLen + 1 );
        }
    }
		
	Tools_FileSystem::EnsureDirectoryExists( filename);
	
	//open the file to write to
	saveHandle = CreateFile(filename,GENERIC_WRITE,0,0,CREATE_ALWAYS,0,0);
	if(saveHandle == INVALID_HANDLE_VALUE)
	{
		AddMessage( MT_ERROR, "Failed to open \"%s\" for writing.", filename );
		CHECK( false );
	}

	#ifdef ASURA_USE_MATERIAL_RESPONSE_SYSTEM
		BeginMRPShaderExport();
	#endif

	g_uAsuraFilesInExport = m_xAsuraFileList.size();
	
	g_axResourceFileListEntries = new ResourceFileListEntry[g_uAsuraFilesInExport];

    // Fill out list of names of Asura files we're going to output.
	{
		unsigned u = 0;
		for(LIF<Asura_String> xFileLIF(&m_xAsuraFileList);!xFileLIF.done();xFileLIF.next())
		{
			g_axResourceFileListEntries[u++].xFilename = Asura_FileAccess::PlatformRemoveNetworkOrExecutableDir( xFileLIF().GetString() );
		}
	}

	//write the header
	WriteFile(saveHandle,ASURA_UNCOMPRESSED_FILE_ID,ASURA_FILE_ID_NUM_CHARS,&bytesWritten,0);

   	CHECK( LeaveSpaceForFileInfoChunk(saveHandle) );
    CHECK( LeaveSpaceForResourceFileListChunk(saveHandle) );

	
	if(ShouldSoundsBeIncludedInCharacterExports())
	{
		//add sounds
		BuildNonEnvironmentalSoundList(m_xAsuraFileList);
		AddNonEnvironmentalSounds(saveHandle);
		AddSkinSounds(saveHandle);

		//write out the sound mappings
		AddSoundMappingResource(saveHandle);

		//This needs to be done after the other sounds , just in case any of the wav files being exported
		//have a non-dynamic sound ID given to them from elsewhere. RAR 13/07/2006
		CHECK(AddSoundEvents(saveHandle));
		#ifdef ASURA_USE_DIALOGUE_SYSTEM
		CHECK(AddDialogueEvents(saveHandle));
		#endif

		#if defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)
		CHECK(AddFacialAnimationData(saveHandle));
		#endif	//defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)
	}

	if(s_eExportType == Export_Wii)
	{
		// Write out alpha textures
		Tools_Wii_Textures::BeginExport(filename);
	}

	//add any extra textures...
	AddSkinTextures(saveHandle);
	AddMiscTextures(saveHandle);
	
	if(s_eExportType == Export_Wii)
	{
		// Write out alpha textures
		Tools_Wii_Textures::EndExport(saveHandle);
	}

	//add all the asura files
	{
		unsigned uNumAsuraFilesAdded = 0;
		for(LIF<Asura_String> xFileLIF(&m_xAsuraFileList);!xFileLIF.done();xFileLIF.next())
		{
			CHECK( AddAsuraFile(saveHandle, xFileLIF(), uNumAsuraFilesAdded++) );
		}
		Asura_Assert( g_uAsuraFilesInExport == uNumAsuraFilesAdded, "" );
	}
	
	// In material response builds, add the materials too
	#if defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM
		{
			CHECK( AddMaterialResponses( saveHandle ) );
		}
	#endif

	#ifdef ASURA_USE_MATERIAL_RESPONSE_SYSTEM
		EndMRPShaderExport( saveHandle );
	#endif

	//Add the terminator
    {
	    int Terminator;
	    Terminator = 0;
	    BOOL bOK = WriteFile(saveHandle,&Terminator,4,&bytesWritten,0);
        if( !bOK || 4 != bytesWritten )
        {
            AddMessage( MT_ERROR, "ERROR: %s.", Tools_OS::GetLastSystemError() );
            CHECK( false );
        }
    }

	//Write out file info chunk so that the file type gets saved out.
	//Leaving the file size as 0 , so that the old behaviour of not altering
	//the loading bar is preserved
   	CHECK( FillOutFileInfoChunk(saveHandle, 0) );
    // Fill out the resource file list chunk now.
    CHECK( FillOutResourceFileListChunk(saveHandle) );

End :
    // Clear the progress message.
	s_bIsCharacterExport = false;

	UpdateStatus(EXPORT_STAGE_END, NULL, 0);

	if( saveHandle != INVALID_HANDLE_VALUE )
    {
	    CloseHandle(saveHandle);
    }

	delete [] g_axResourceFileListEntries;
	g_axResourceFileListEntries = 0;

	if( bOK )
	{
		if(!IsCompressionDisabled() && ShouldCompressExportedFiles())
		{
			AddMessage( MT_INFORMATION, "Compressing \"%s\"...", filename );
			CompressFile(filename);
		}
		
		AddMessage( MT_INFORMATION, "Successfully exported \"%s\".",filename);
	}
	else
	{
		DeleteFile(filename);
        AddMessage( MT_ERROR, "Failed to export \"%s\".", filename );
	}

	//Export language files if needed
	ExportLanguageFiles(filename);

	#undef CHECK

    return bOK;
}

void Tools_Level_Export::ExportCharacters(int iPlatform)
{
	s_eExportType = (EXPORT_TYPE)iPlatform;

	char szHeading[100];
	Asura_PrintfToBfr(szHeading,"%s Character Export",sizeof(szHeading),true,GetPlatformName(iPlatform));
	s_szMainHeading = szHeading;

	Asura_Assert(This,"");
	if(s_bExportAllCharacters)
		This->ExportAllCharacterFiles();
	else
		This->ExportSingleCharacterFile();

	//Reset type to PC at the end to be on the safe side.
	s_eExportType = Export_PC;

}

void Tools_Level_Export::GetExportedCharacterDir(Asura_String& xOutputDir, bool bCreate)
{
	xOutputDir = GetPlatformName( s_eExportType );
	xOutputDir += "Chars";

		if (bCreate)
        {
		CreateDirectory( xOutputDir.GetString(), NULL );
	}
}


void Tools_Level_Export::GetCharacterDirectories(List<Asura_String>& xDirectoryList)
{
    char szSearch[MAX_PATH];

    Asura_FileAccess::PlatformGetNetworkDir( "Characters", szSearch, MAX_PATH );
    strcat( szSearch, "\\*.*" );
	Tools_FileSystem::FindDirectories( szSearch, xDirectoryList );

	for(LIF<Asura_String> dlif(&xDirectoryList);!dlif.done();)
	{
        const char* szLastSlash = strrchr( dlif().GetString(), '\\' );
        if( szLastSlash && (!stricmp(szLastSlash + 1, "Anims_Body_Types") || !stricmp(szLastSlash + 1, "Anims_Specialised")) )
        {
			dlif.delete_current();//ignore these 2 directories
		}
		else
		{
			dlif.next();
		}
	}

    Asura_FileAccess::PlatformGetNetworkDir( "Characters\\Anims_Body_Types", szSearch, MAX_PATH );
    strcat( szSearch, "\\*.*" );
	Tools_FileSystem::FindDirectories( szSearch, xDirectoryList );

    Asura_FileAccess::PlatformGetNetworkDir( "Characters\\Anims_Specialised", szSearch, MAX_PATH );
    strcat( szSearch, "\\*.*" );
	Tools_FileSystem::FindDirectories( szSearch, xDirectoryList );
}

void Tools_Level_Export::ExportAllCharacterFiles()
{
	//get a list of directories
	List<Asura_String> directoryList;

	GetCharacterDirectories(directoryList);


    // Check all entries in output filename list are unique.
    if( Export_PS2 == s_eExportType )
    {
        for( int i = 0; i < directoryList.size(); ++i )
        {
			#if defined ASURA_ASSERT
				const u_int uHash = Asura_GetHashID( directoryList[i].GetString() );
			#endif

            for( int j = 0; j < directoryList.size(); ++j )
            {
                if( i != j )
                {
                    Asura_Assert( uHash != Asura_GetHashID(directoryList[j].GetString()), "Hash clash!" );
                }
            }
        }
    }

    {
        Asura_Char szOutputFilename[8 + 1];

	    StartExport();
        for( int i = 0; !IsAbortFlagSet() && i < directoryList.size(); ++i )
        {
		    StartFileExport();

            if( StartCharacterExport(directoryList[i]) )
            {
                const char* szRawPath = directoryList[i].GetString();
                const char* szRelPath = Asura_FileAccess::PlatformRemoveNetworkOrExecutableDir( szRawPath );
				OpenExportLog(szRawPath);

		        AddMessage( MT_INFORMATION, "Exporting %s.", szRelPath );
                
                if( Export_PS2 == s_eExportType )
                {
                    // Generate 8.3 output filename for PS2 export.
                    Asura_PrintfToBfr( szOutputFilename, "%08x", 8 + 1, true, Asura_GetHashID(szRelPath) );
                    ExportCharacterFile( szRawPath, szOutputFilename );
                }
                else
                {
                    ExportCharacterFile( szRawPath );
                }

				LogAnimationExportStats();
			    EndFileExport();
				CloseExportLog();
        
                EndCharacterExport();
            }

	    }
	    EndExport();
    }
}

void Tools_Level_Export::ExportSingleCharacterFile( void )
{
	//Ask for a directory to export characters from
	char szRootDir[MAX_PATH];
	#ifdef ASURA_PC_ALLOW_LOAD_FROM_NETWORK
	const char* pszNetworkPath = Asura_FileAccess::PlatformGetNetworkPath();
	if(pszNetworkPath && *pszNetworkPath)
	{
		strcpy(szRootDir,pszNetworkPath);
	}
	else
	#endif // def ASURA_PC_ALLOW_LOAD_FROM_NETWORK
	{
		GetCurrentDirectory(MAX_PATH,szRootDir);
	}

	Tools_FileSystem_SelectFolderInfo xInfo;
	xInfo.StartDirectory = szRootDir;
	xInfo.InitialSelection = "Characters";
	xInfo.DisplayText = "Select directory to export to character file";

	if(!Tools_FileSystem::SelectFolder(xInfo))
	{
		return;
	}

    StartExport();
    
    if( !IsAbortFlagSet() )
    {
        Asura_String xDir = xInfo.SelectedFolder;

        StartFileExport();

        if( StartCharacterExport(xDir) )
        {
            const char* szRawPath = xDir.GetString();
            const char* szRelPath = Asura_FileAccess::PlatformRemoveNetworkOrExecutableDir( szRawPath );

			OpenExportLog(szRawPath);

		    AddMessage( MT_INFORMATION, "Exporting %s.", szRelPath );
            
            if( Export_PS2 == s_eExportType )
            {
                // Generate 8.3 output filename for PS2 export.
                Asura_Char szOutputFilename[8 + 1];
                Asura_PrintfToBfr( szOutputFilename, "%08x", 8 + 1, true, Asura_GetHashID(szRelPath) );

                ExportCharacterFile( szRawPath, szOutputFilename );
            }
            else
            {
                ExportCharacterFile( szRawPath );
            }
		  	LogAnimationExportStats();
			CloseExportLog();
            EndCharacterExport();
        }
        
        EndFileExport();
    }

	EndExport();
}


bool Tools_Level_Export::StartCharacterExport( const Asura_String& )
{
	m_iTotalAnimationFramesBefore = 0;
	m_iTotalAnimationFramesAfter = 0;
	m_iTotalAnimationSizeBefore = 0;
	m_iTotalAnimationSizeAfter = 0;
	m_bIsCharacterExport = true;
    return true;
}

void Tools_Level_Export::EndCharacterExport( void )
{
	m_bIsCharacterExport = false;
}


/// End of file. ///