/**
*
* \file Tools_Level_Export.cpp
*
* Created by Kevin Lea - 29/11/2002 11:23:31
* Splitted into separate file 12/11/2008
*
* Copyright (C) 2002 Rebellion, All Rights Reserved.
*
* \brief
*
**/

#include "Core/Asura.h"

#include "Load&Save/LevelExport/Tools_Level_Export.h"

// Asura
#include "ChunkSystem/Asura_Chunk_Hierarchy.h"
#include "ChunkSystem/Asura_Chunk_Lights.h"
#include "ChunkSystem/Asura_Chunk_Material.h"
#include "ChunkSystem/Asura_Chunk_ResourceFileList.h"
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "ChunkSystem/Asura_Chunk_Texture.h"
#include "ChunkSystem/Asura_SaveBlock.h"
#include "Light/Asura_Light.h"

// Tools
#include "Entity/Tools_Entity_PhysicalObject.h"
#include "Load&Save/Tools_SaveAsura.h"

#if ASURA_USE_GUI_MENU_SYSTEM
//#include "GUIMenu/Asura_GUIMenu_Message.h"
//#include "GUIMenu/Asura_GUIMenu_Menu_Page.h"
#include "GUIMenu/Asura_GUIMenu_System.h"
#include "GUIMenu/Tools_GUIMenu_System.h"	
#endif

#ifdef ASURA_USE_DECAL_MANAGER
#include "SpecialFX/Asura_Decal_Manager.h"
#include "ChunkSystem/Asura_Chunk_Decals.h"
#endif

// G L O B A L S ////////////////////////////////////////
extern u_int g_uAsuraFilesInExport;
extern ResourceFileListEntry* g_axResourceFileListEntries;
extern unsigned g_uBytesReservedForFileListChunk;
extern unsigned g_uFileListChunkStartFilePtr;
extern unsigned g_uBytesReservedForFileInfoChunk;
extern unsigned g_uFileInfoChunkStartFilePtr;

// F U N C T I O N S /////////////////////////////////////////////////////////////
extern BOOL WriteFileString( HANDLE hFile, const Asura_String &xString, DWORD* puBytesWritten);

//  AAB:  (14/11/2003 14:25:40):  Removed static, so this function can be used in derived
// classes of Tools_Level_Export
bool LeaveSpaceForResourceFileListChunk( HANDLE hFile )
{
    if(!g_uAsuraFilesInExport)
	{
		//no need for chunk in this case. RAR 14/07/2006
		g_uBytesReservedForFileListChunk = 0;
		return true;
	}
    
    // Determine max amount of space we'll be needing.
    g_uBytesReservedForFileListChunk = sizeof(Asura_Chunk_ResourceFileList);
    for( unsigned u = 0; u < g_uAsuraFilesInExport; ++u )
    {
        // Should have filled out filenames before calling this function.
        Asura_Assert( !g_axResourceFileListEntries[u].xFilename.IsBlank(), "" );

        g_uBytesReservedForFileListChunk += (g_axResourceFileListEntries[u].xFilename.GetSizeInInts() << 2) + 12;
    }

    // Record file pointer value now.
    g_uFileListChunkStartFilePtr = SetFilePointer( hFile, 0, NULL, FILE_CURRENT );
    if( INVALID_SET_FILE_POINTER == g_uFileListChunkStartFilePtr )
    {
        Tools_Level_Export::AddMessage( Tools_Level_Export::MT_ERROR, "I/O error. Export aborted." );
        return false;
    }

    // Move file pointer on beyond the space we're leaving for the file list
    // chunk.
    {
        DWORD uNewPtrVal = SetFilePointer( hFile, g_uBytesReservedForFileListChunk, NULL, FILE_CURRENT );
        if( INVALID_SET_FILE_POINTER == uNewPtrVal )
        {
            Tools_Level_Export::AddMessage( Tools_Level_Export::MT_ERROR, "I/O error. Export aborted." );
            return false;
        }
    }


    // Okay.
    return true;
}

//  AAB:  (14/11/2003 14:25:40):  Removed static, so this function can be used in derived
// classes of Tools_Level_Export
bool FillOutResourceFileListChunk( HANDLE hFile )
{
    if(!g_uAsuraFilesInExport)
	{
		//no need for chunk in this case. RAR 14/07/2006
		return true;
	}

    // Verify that LeaveSpaceForResourceFileListChunk() was called.
    Asura_Assert( g_uBytesReservedForFileListChunk, "" );

    // Record cur pos of file ptr.
    const DWORD uStoredFilePtrPos = SetFilePointer( hFile, 0, NULL, FILE_CURRENT );
    if( INVALID_SET_FILE_POINTER == uStoredFilePtrPos )
    {
        Tools_Level_Export::AddMessage( Tools_Level_Export::MT_ERROR, "I/O error. Export aborted." );
        return false;
    }

    // Move file ptr back to start of space we left for the chunk.
    {
        DWORD uNewPtrVal = SetFilePointer( hFile, g_uFileListChunkStartFilePtr, NULL, FILE_BEGIN );
        if( INVALID_SET_FILE_POINTER == uNewPtrVal )
        {
            Tools_Level_Export::AddMessage( Tools_Level_Export::MT_ERROR, "I/O error. Export aborted." );
            return false;
        }
    }

    // Count number of files and calculate the actual amount of data we'll
    // need to write.
    unsigned uActualNumFiles = 0;
    unsigned uActualDataSize = sizeof(Asura_Chunk_ResourceFileList);
    for( unsigned u = 0; u < g_uAsuraFilesInExport; ++u )
    {
        if( g_axResourceFileListEntries[u].bFileAddedOK )
        {
            ++uActualNumFiles;
            uActualDataSize += (g_axResourceFileListEntries[u].xFilename.GetSizeInInts() << 2) + 12;
        }
    }
    Asura_Assert( uActualDataSize <= g_uBytesReservedForFileListChunk, "" );

    unsigned uTotalNumBytesWritten = 0;
    #define OUTPUT( data_ptr, num_bytes ) \
    { \
        DWORD uNumBytesWritten; \
        BOOL bOK = WriteFile( hFile, (data_ptr), (num_bytes), &uNumBytesWritten, NULL ); \
        if( !bOK || (DWORD)(num_bytes) != uNumBytesWritten ) \
        { \
            if( ERROR_HANDLE_DISK_FULL == GetLastError() ) \
            { \
                Tools_Level_Export::AddMessage( Tools_Level_Export::MT_ERROR, "Disk full. Export aborted." ); \
            } \
            else \
            { \
                Tools_Level_Export::AddMessage( Tools_Level_Export::MT_ERROR, "I/O error. Export aborted." ); \
            } \
            return false; \
        } \
        else \
        { \
            uTotalNumBytesWritten += (num_bytes); \
        } \
    }

    #define OUTPUT_STRING( data_ptr) \
    { \
        DWORD uNumBytesWritten; \
        BOOL bOK = WriteFileString( hFile, (data_ptr), &uNumBytesWritten); \
        if( !bOK || (DWORD)((data_ptr.GetSizeInInts() << 2)) != uNumBytesWritten ) \
        { \
            if( ERROR_HANDLE_DISK_FULL == GetLastError() ) \
            { \
                Tools_Level_Export::AddMessage( Tools_Level_Export::MT_ERROR, "Disk full. Export aborted." ); \
            } \
            else \
            { \
                Tools_Level_Export::AddMessage( Tools_Level_Export::MT_ERROR, "I/O error. Export aborted." ); \
            } \
            return false; \
        } \
        else \
        { \
            uTotalNumBytesWritten += (uNumBytesWritten); \
        } \
    }

    // Write chunk hdr. Note that we specify the size as the maximum so that
    // anything reading the chunk can skip over the padding at the end.
    Asura_Chunk_ResourceFileList xFileListHdr;
	xFileListHdr.Create(g_uBytesReservedForFileListChunk, uActualNumFiles);
    OUTPUT( &xFileListHdr, sizeof(Asura_Chunk_ResourceFileList) );

    // Output each entry.
    const int iOne = 1;
    for( unsigned u = 0; u < g_uAsuraFilesInExport; ++u )
    {
        if( g_axResourceFileListEntries[u].bFileAddedOK )
        {
            //OUTPUT( g_axResourceFileListEntries[u].xFilename.GetString(), (g_axResourceFileListEntries[u].xFilename.GetSizeInInts() << 2) );
            OUTPUT_STRING( g_axResourceFileListEntries[u].xFilename);
            
            OUTPUT( &g_axResourceFileListEntries[u].iByteOffset, 4 );
            OUTPUT( &g_axResourceFileListEntries[u].iNumBytes, 4 );
            OUTPUT( &iOne, 4 );
        }
    }

    // Verify we wrote as much data as expected.
    Asura_Assert( uTotalNumBytesWritten == uActualDataSize, "" );

    // Probably a good idea to fill the padding with something obvious.
    {
        unsigned uNumPadBytes = g_uBytesReservedForFileListChunk - uActualDataSize;

        // Should be an integral number of words left to pad.
        Asura_Assert( !(uNumPadBytes & 3), "" );

        unsigned uNumPadWords = (uNumPadBytes >> 2);
        while( uNumPadWords-- )
        {
            const unsigned uPadValue = 0xdeadbeef;
            OUTPUT( &uPadValue, 4 );
        }
    }

    #undef OUTPUT
    #undef OUTPUT_STRING

    // Return file ptr to where it was before.
    {
        DWORD uNewPtrVal = SetFilePointer( hFile, uStoredFilePtrPos, NULL, FILE_BEGIN );
        if( INVALID_SET_FILE_POINTER == uNewPtrVal )
        {
            Tools_Level_Export::AddMessage( Tools_Level_Export::MT_ERROR, "I/O error. Export aborted." );
            return false;
        }
    }

    // Okay.
    return true;
}



// TOOLS_LEVEL_EXPORT CHUNK RELATED FUNCTIONS /////////////

bool Tools_Level_Export::LeaveSpaceForFileInfoChunk( HANDLE hFile )
{
    // Determine amount of space we'll be needing.
    g_uBytesReservedForFileInfoChunk = Asura_Chunk_FileInfo::GetChunkSize();
    
    // Record file pointer value now.
    g_uFileInfoChunkStartFilePtr = SetFilePointer( hFile, 0, NULL, FILE_CURRENT );
    if( INVALID_SET_FILE_POINTER == g_uFileInfoChunkStartFilePtr )
    {
        Tools_Level_Export::AddMessage( Tools_Level_Export::MT_ERROR, "I/O error. Export aborted." );
        return false;
    }

    // Move file pointer on beyond the space we're leaving for this chunk
    // chunk.
    {
        DWORD uNewPtrVal = SetFilePointer( hFile, g_uBytesReservedForFileInfoChunk, NULL, FILE_CURRENT );
        if( INVALID_SET_FILE_POINTER == uNewPtrVal )
        {
            Tools_Level_Export::AddMessage( Tools_Level_Export::MT_ERROR, "I/O error. Export aborted." );
            return false;
        }
    }

    // Okay.
    return true;
}

bool Tools_Level_Export::FillOutFileInfoChunk( HANDLE hFile, u_int uFileSize )
{
    // Verify that LeaveSpaceForFileInfoChunk() was called.
    Asura_Assert( g_uBytesReservedForFileInfoChunk, "" );

    // Record cur pos of file ptr.
    const DWORD uStoredFilePtrPos = SetFilePointer( hFile, 0, NULL, FILE_CURRENT );
    if( INVALID_SET_FILE_POINTER == uStoredFilePtrPos )
    {
        Tools_Level_Export::AddMessage( Tools_Level_Export::MT_ERROR, "I/O error. Export aborted." );
        return false;
    }

    // Move file ptr back to start of space we left for the chunk.
    {
        DWORD uNewPtrVal = SetFilePointer( hFile, g_uFileInfoChunkStartFilePtr, NULL, FILE_BEGIN );
        if( INVALID_SET_FILE_POINTER == uNewPtrVal )
        {
            Tools_Level_Export::AddMessage( Tools_Level_Export::MT_ERROR, "I/O error. Export aborted." );
            return false;
        }
    }

    char* pcBuffer = new char[g_uBytesReservedForFileInfoChunk];
    if (pcBuffer == 0)
    {
        Asura_Assert(0, "out of memory");
        return false;
    }

	Asura_Chunk_FileInfo::Create(pcBuffer, uFileSize, AsuraFile_Exported, (m_bCreateLanguageFiles && !m_bExportingLanguageFile && !m_bUsingSharedLanguageFile));
    
    unsigned uActualDataSize = g_uBytesReservedForFileInfoChunk;
    unsigned uTotalNumBytesWritten = 0;
    #define OUTPUT( data_ptr, num_bytes ) \
    { \
        DWORD uNumBytesWritten; \
        BOOL bOK = WriteFile( hFile, (data_ptr), (num_bytes), &uNumBytesWritten, NULL ); \
        if( !bOK || (DWORD)(num_bytes) != uNumBytesWritten ) \
        { \
            if( ERROR_HANDLE_DISK_FULL == GetLastError() ) \
            { \
                Tools_Level_Export::AddMessage( Tools_Level_Export::MT_ERROR, "Disk full. Export aborted." ); \
            } \
            else \
            { \
                Tools_Level_Export::AddMessage( Tools_Level_Export::MT_ERROR, "I/O error. Export aborted." ); \
            } \
            return false; \
        } \
        else \
        { \
            uTotalNumBytesWritten += (num_bytes); \
        } \
    }

    OUTPUT( pcBuffer, uActualDataSize );
    delete [] pcBuffer;

    // Verify we wrote as much data as expected.
    Asura_Assert( uTotalNumBytesWritten == uActualDataSize, "" );

    // Probably a good idea to fill the padding with something obvious.
	if(g_uBytesReservedForFileListChunk)
    {
        unsigned uNumPadBytes = g_uBytesReservedForFileListChunk - uActualDataSize;

        // Should be an integral number of words left to pad.
        Asura_Assert( !(uNumPadBytes & 3), "" );

        unsigned uNumPadWords = (uNumPadBytes >> 2);
        while( uNumPadWords-- )
        {
            const unsigned uPadValue = 0xdeadbeef;
            OUTPUT( &uPadValue, 4 );
        }
    }

    #undef OUTPUT

    // Return file ptr to where it was before.
    {
        DWORD uNewPtrVal = SetFilePointer( hFile, uStoredFilePtrPos, NULL, FILE_BEGIN );
        if( INVALID_SET_FILE_POINTER == uNewPtrVal )
        {
            Tools_Level_Export::AddMessage( Tools_Level_Export::MT_ERROR, "I/O error. Export aborted." );
            return false;
        }
    }

    // Okay.
    return true;
}


////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Level_Export::AddChunkStream
// Author       : Andy Tate
// Date         : 5-Sep-2008
////////////////////////////////////////////////////////////////////////////////
bool Tools_Level_Export::AddChunkStream(Asura_Chunk_Memory_Stream& xStream, HANDLE pSaveHandle)
{
	const char*	pcStartAddress	= static_cast<const char*>(xStream.GetStartAddress());
	const char*	pcEndAddress	= static_cast<const char*>(xStream.GetAddress());

	DWORD		dwDataSize		= pcEndAddress - pcStartAddress;

	DWORD		dwBytesWritten;

	// Write the stream out...
	WriteFile(pSaveHandle, pcStartAddress, dwDataSize, &dwBytesWritten, NULL);

	if(dwBytesWritten == dwDataSize)
	{
		return(true);
	}
	else
	{
		AddMessage(MT_ERROR, "I/O error.");
		return(false);
	}
}

/// End of function 'Tools_Level_Export::AddChunkStream'. ///

bool Tools_Level_Export::ShouldChunkBeExported_Static(int iChunkID)
{
	Asura_Assert(This,"Tools_Level_Export not allocated");
	return This->ShouldChunkBeExported(iChunkID);
}

bool Tools_Level_Export::ShouldChunkBeExported(int id)
{
	// KJFL - 2004-07-20 - first check for chunks which should never be exported, irrespective of platform
	switch( id )
	{
		// by definition chunks only used in the tools should be included here
		case ASURA_CHUNK_LIGHTVISIBILITIES:
  		case ASURA_CHUNK_MESH_LOD:
		case ASURA_CHUNK_MESH_MISC:
		case ASURA_CHUNK_MESH_NORMALS:
		case ASURA_CHUNK_MESH_POLYGONFLAGS:
		case ASURA_CHUNK_MESH_POLYGONS:
		case ASURA_CHUNK_MESH_SMOOTHINGGROUP:
		case ASURA_CHUNK_MESH_TEXTUREINFO:
		case ASURA_CHUNK_MESH_VERTICES:
		case ASURA_CHUNK_MODULENAMES:
		case ASURA_CHUNK_STREAMINGMODULENAMES:
		case ASURA_CHUNK_MATERIALNAMES:
		case ASURA_CHUNK_ENTITY_TOOLSDATA:
		case ASURA_CHUNK_ENTITY_NEXTGUID:
		case ASURA_CHUNK_LIGHTSCALE:
		case ASURA_CHUNK_SOUNDNAMES:
		case ASURA_CHUNK_MESH_COLLMESHGENDATA:
		case ASURA_CHUNK_GLOBALILLUMINATION:
		case ASURA_CHUNK_ENVIRONMENTALLIGHTTRANSPORT:		
		case ASURA_CHUNK_HIERARCHY_MANUALSECTIONBBS:	
		case ASURA_CHUNK_POLYGONLIGHTS:
		case ASURA_CHUNK_LIGHTS:
		case ASURA_CHUNK_FILEINFO ://this chunk is created for exports , but shouldn't be copied from other files
		case ASURA_CHUNK_RESOURCEFILELIST ://this chunk is created for exports , but shouldn't be copied from other files
		case ASURA_CHUNK_BOTANICALS_TEMPLATES:
		case ASURA_CHUNK_BOTANICALS_INSTANCES:
		case ASURA_CHUNK_BOTANICALS_STYLES:
		case ASURA_CHUNK_BOTANICALS_DEFINITION:
		case ASURA_CHUNK_CARPET_MESH:
		case ASURA_CHUNK_CARPET_STYLES:
		// JGP - painted vertex colours currently don't end up in game data. Though next-gen they should do probably.
		case ASURA_CHUNK_MESH_PAINTEDCOLOURS:
		case ASURA_CHUNK_MESH_COLOURS:
		case TOOLS_CHUNK_MATERIAL_RESPONSE_NAMES:
		case TOOLS_CHUNK_CONTAINER_FILE:
		case TOOLS_CHUNK_CONTAINER_GROUP:
		case TOOLS_CHUNK_CONTAINER:
		case ASURA_CHUNK_FACIALANIMATION_FACEANIMATION://Any of these in animation files will be added separately by the export code
		case TOOLS_CHUNK_FACIALANIMATION_FACEANIMATION:
		{
			return false;
		}

		case ASURA_CHUNK_WATERCONTROL :
		case ASURA_CHUNK_WATERPACKET :
		{
			//RAR 11/01/2005 - For the moment water is only available on PS2
			// DH Allow for PC as well
			return (s_eExportType == Export_PS2) || (s_eExportType == Export_PC);
		}

		#ifdef ASURA_USE_MATERIAL_RESPONSE_SYSTEM
		case ASURA_CHUNK_TEXTURENAMES :
		case ASURA_CHUNK_TEXTUREFLAGS :
		//case ASURA_CHUNK_MATERIAL : //Collision meshes still refer to the material chunk after all. RAR 12/04/2007
		{
			return m_bIncludeOldMaterialChunks;
		}
		#endif


		default: // okay, we'll carry on and check out remaining chunks on a per-platform basis
			break;
		
	}
	


	switch (s_eExportType)
	{

		case Export_PC:
		{
			switch( id )
			{
				// any PC rejected chunks would go here...
				case ASURA_CHUNK_PSP_BLUR:
				case ASURA_CHUNK_PSP_FILTER:
					return false;
 
				default:
					break;
			
			}

			break;
		}

		case Export_PS2:
		{
			switch( id )
			{
				// any PS2 rejected chunks would go here...
				case ASURA_CHUNK_PSP_BLUR:
				case ASURA_CHUNK_PSP_FILTER:
					return false;

				default:
					break;
			}
			
			break;
		}

		case Export_PSP:
		{
			switch( id )
			{
				case ASURA_CHUNK_PARTICLETEMPLATES:
				// Not used in any PSP projects
				case ASURA_CHUNK_BILLBOARDPLANTS_TEMPLATES:
				case ASURA_CHUNK_BILLBOARDPLANTS_BATCHES:
				{
					return false;
				}

				default:
					break;
			}
			
			break;
		}

		
		case Export_PS3:
		{
			switch( id )
			{
				case ASURA_CHUNK_PARTICLETEMPLATES:
				case ASURA_CHUNK_BILLBOARDPLANTS_TEMPLATES:
				case ASURA_CHUNK_BILLBOARDPLANTS_BATCHES:
				case ASURA_CHUNK_PSP_BLUR:
				case ASURA_CHUNK_PSP_FILTER:
					return false;

				default:
					break;
			}
			
			break;
		}

		case Export_Xbox360:
		{
			switch( id )
			{
				// any Xbox 360 rejected chunks would go here...
				case ASURA_CHUNK_PSP_BLUR:
				case ASURA_CHUNK_PSP_FILTER:
					return false;

				default:
					break;
			}

			break;
		}

		case Export_Wii:
		{
			switch( id )
			{
				// any GC rejected chunks would go here...
				case ASURA_CHUNK_PSP_BLUR:
				case ASURA_CHUNK_PSP_FILTER:
					return false;

				default:
					break;
			}

			break;
		}

		default:
		{
			Asura_Assert( false, "Unknown platform!" );
			return false;
		}

	}

	return true;
}

bool Tools_Level_Export::ModifyChunkBeforeOutput_Static(const Asura_String& xFilename,const Asura_Chunk_Header* pxChunk,char** ppcOutputChunkBuffer)
{
	Asura_Assert(This,"Tools_Level_Export not allocated");
	return This->ModifyChunkBeforeOutput(xFilename,pxChunk,ppcOutputChunkBuffer);
}

bool Tools_Level_Export::ModifyChunkBeforeOutput(const Asura_String& xFilename,const Asura_Chunk_Header* pxChunk,char** ppcOutputChunkBuffer)
{
	Asura_Assert(pxChunk,"");
	Asura_Assert(ppcOutputChunkBuffer,"");

	//Get a copy of the header . This will flip it if needed for GC
	Asura_Chunk_Header xChunkHeader(pxChunk->ID, pxChunk->Size, pxChunk->Version, pxChunk->Flags);
    
	if(xChunkHeader.ID == ASURA_CHUNK_TEXTURENAMES && s_eExportType == Export_PS2)
	{
		const Asura_Chunk_TextureNames* pxOldChunk = (const Asura_Chunk_TextureNames*) pxChunk;

		bool bNeedToModify = false;
		bool bEnvironment = IsEnvironmentFile(xFilename);
		if(bEnvironment)
		{
			if(m_iNumberOfUntexturedPolygons||m_iNumberOfPolygonsWithMissingTexture)
			{
				//There were untextured polygons in environment , so need to add untextured
				//texture to the chunk
				bNeedToModify = true;
			}
		}
		else if(pxOldChunk->NumberOfTextures == 0)
		{
			//for non-environment we need to add an untextured entry if no textures exist
			bNeedToModify = true;
		}
				
		if(!bNeedToModify)
		{
			return false;
		}
		//Need to alter the texture names chunk to add a fake texture for 'untextured'
		//Can only do this if it is a version that we recognise
		if(!(pxChunk->Version==2 || pxChunk->Version==3))
		{
			Asura_Assert(pxChunk->Version==2 || pxChunk->Version == 3,"Modify chunk code only supports versions 2 and 3 of texture names chunk");
			return false;
		}

		//allocate a new chunk , and copy over the contents of the old
		int iChunkSize = pxChunk->Size + 12;
		Asura_Chunk_TextureNames* pxNewChunk = (Asura_Chunk_TextureNames*) new char[iChunkSize];
		memcpy(pxNewChunk,pxChunk,pxChunk->Size);

		pxNewChunk->Size = iChunkSize;
		pxNewChunk->NumberOfTextures++;

		//add the fake texture name to the end
		char* pcPos = (char*)pxNewChunk;
		pcPos += pxChunk->Size;
		strcpy(pcPos,"Untextured!");
		
		*ppcOutputChunkBuffer = (char*) pxNewChunk;

		if(bEnvironment)
		{
			m_iUntexturedTextureIndex = pxNewChunk->NumberOfTextures - 1;
		}

		return true;
	}
	else if( xChunkHeader.ID == ASURA_CHUNK_MATERIAL )
	{
		const Asura_Chunk_Material* pxMaterialChunk = static_cast<const Asura_Chunk_Material*>( pxChunk );

#ifdef ASURA_USE_MATERIAL_RESPONSE_SYSTEM
		if(m_bIncludeOldMaterialChunks) return false;

		//Don't really want the old material data , but its still required for the collision meshes.
		//So go through and replace all the texture indeces to -1 to prevent the need for the texture names chunk
		//RAR 12/04/2007

		//Any projects with the material response system should be on the more recent material chunk versions by now.
		int iVersion = pxMaterialChunk->Version;
		ASURA_CHUNK_VERIFY(iVersion,int);
		if( iVersion < 4 || iVersion > 6)
		{
			Asura_Assertf( false, ("Modify chunk code only supports versions 4 to 6 of the material chunk.\nThis chunk's version is %d", iVersion) );
			return false;
		}

		int iChunkSize = pxMaterialChunk->Size;
		ASURA_CHUNK_VERIFY(iChunkSize,int);

		Asura_Chunk_Material* pxNewMaterialChunk = reinterpret_cast<Asura_Chunk_Material*>( new char[iChunkSize] );
        if( !pxNewMaterialChunk ) { return false; }

        memcpy( pxNewMaterialChunk, pxMaterialChunk, iChunkSize );

		int iNumMaterials = pxNewMaterialChunk->m_iNumberOfMaterials;
		ASURA_CHUNK_VERIFY(iNumMaterials,int);

		//skip past the chunk header
		int* piPos = (int*) (pxNewMaterialChunk+1);

		//replace all texture indeces by -1
		int iTexturesPerMaterial = 4; // Base textures and normal maps
		int iExtraIntsPerMaterial = 3; // Material flags, project material flags and texture anim hash
		if (iVersion >= 5) { iTexturesPerMaterial += 2; } // Specular maps
		if (iVersion >= 6) { iExtraIntsPerMaterial += 1; } // Specular colour

		for(int i=0;i<iNumMaterials;i++)
		{
			for(int j=0;j<iTexturesPerMaterial;j++)
			{
				*piPos++ = -1;
			}

			piPos += iExtraIntsPerMaterial;
		}

		//should have reached the end of the chunk
		Asura_Assert(((int)piPos-(int)pxNewMaterialChunk) == iChunkSize,"Error modifiying material chunk");
			
		*ppcOutputChunkBuffer = reinterpret_cast<char*>( pxNewMaterialChunk );
		return true;

#else  //ASURA_USE_MATERIAL_RESPONSE_SYSTEM
       
        bool bRemoveDualTextureData = ( pxMaterialChunk->Version >= 2 && !ShouldIncludeDualTextureData() );
        int  iTextureIdxToAdd       = ASURA_TEXTURE_NONE;

		bool bEnvironment = IsEnvironmentFile( xFilename );

        // Currently only require this export fix on PS2.
        if( Export_PS2 == s_eExportType )
        {
		    if( bEnvironment )
		    {
			    if( m_iNumberOfUntexturedPolygons || m_iNumberOfPolygonsWithMissingTexture )
			    {
				    //environment material chunk
				    //we need to add an untextured material
				    Asura_Assert( ASURA_TEXTURE_NONE != m_iUntexturedTextureIndex, "Fake texture index for untextured polygons should have been generated already" );
				    iTextureIdxToAdd = m_iUntexturedTextureIndex;
			    }
		    }
		    else if( pxMaterialChunk->m_iNumberOfMaterials == 0 )
		    {
			    //non-environment material chunk.
			    //we need to add an untextured material since non exist
			    iTextureIdxToAdd = 0;
		    }
        }

		if( !bRemoveDualTextureData && ASURA_TEXTURE_NONE == iTextureIdxToAdd )
		{
			return false;
		}

		// Can only modify data if it is a version that we recognise.
		if( pxMaterialChunk->Version > 6 )
		{
			Asura_Assert( false, "Modify chunk code only supports up to version 6 of the material chunk. Ignoring this assertion will result in PS2 levels crashing. Do NOT ignore!" );
			return false;
		}

		int iNewChunkSize = pxMaterialChunk->Size;
        
        if( ASURA_TEXTURE_NONE != iTextureIdxToAdd )
        {
            iNewChunkSize += 8;                                         // Base texture ID and flags.
            if( pxMaterialChunk->Version >= 1 ) { iNewChunkSize += 4; } // Project flags.
            if( pxMaterialChunk->Version >= 2 ) { iNewChunkSize += 4; } // Second texture ID.
			if( pxMaterialChunk->Version >= 3 ) { iNewChunkSize += 8; } // Normal map texture ID per texture.
			if( pxMaterialChunk->Version >= 4 ) { iNewChunkSize += 4; } // Texture Anim Hash.
			if( pxMaterialChunk->Version >= 5 ) { iNewChunkSize += 8; } // VGP 9Feb2006 Specular map texture ID per texture.
			if( pxMaterialChunk->Version >= 6 ) { iNewChunkSize += 4; } // Specular colour
        }
		
		Asura_Chunk_Material* pxNewMaterialChunk = reinterpret_cast<Asura_Chunk_Material*>( new char[iNewChunkSize] );
        if( !pxNewMaterialChunk ) { return false; }

        memcpy( pxNewMaterialChunk, pxMaterialChunk, pxMaterialChunk->Size );

        if( bRemoveDualTextureData )
        {
            int* piDst = reinterpret_cast<int*>( pxNewMaterialChunk + 1 );
            for( u_int u = pxMaterialChunk->m_iNumberOfMaterials; u--; )
            {
                // Skip base texture ID.
                ++piDst;

				if ( pxMaterialChunk->Version >= 3 )
				{
					// Skip normal map 1
					++piDst;
				}

				if ( pxMaterialChunk->Version >= 5 )
				{
					// Skip specular map 1
					++piDst;
				}

				// Trash second texture ID.
                *piDst++ = ASURA_TEXTURE_NONE;

				if ( pxMaterialChunk->Version >= 3 )
				{
					// Skip normal map 2
					++piDst;
				}

				if ( pxMaterialChunk->Version >= 5 )
				{
					// Skip specular map 2
					++piDst;
				}

				// Skip material and project flags;
                piDst += 2;
				
				if( pxMaterialChunk->Version >= 4)
				{
					//Skip texture anim
					++piDst;
				}

				if( pxMaterialChunk->Version >= 6)
				{
					//Skip specular colour
					++piDst;
				}
            }

            Asura_Assert( piDst == reinterpret_cast<int*>(reinterpret_cast<char*>(pxNewMaterialChunk) + pxMaterialChunk->Size), "" );
        }

        if( ASURA_TEXTURE_NONE != iTextureIdxToAdd )
        {
            // Fix up header data.
		    pxNewMaterialChunk->Size = iNewChunkSize;
		    pxNewMaterialChunk->m_iNumberOfMaterials++;

		    //add the fake material to the end
		    char* pcPos = (char*)pxNewMaterialChunk;
		    pcPos += pxMaterialChunk->Size;

		    *(int*)pcPos = iTextureIdxToAdd;
		    pcPos += 4;

			if (pxMaterialChunk->Version >= 3) //normal map texture IDs added in version 3
			{
			    *(int*)pcPos = ASURA_TEXTURE_NONE;
			    pcPos += 4;
			}

			if (pxMaterialChunk->Version >= 5) // VGP 9Feb2006 specular map texture IDs added in version 5
			{
			    *(int*)pcPos = ASURA_TEXTURE_NONE;
			    pcPos += 4;
			}

			if( pxMaterialChunk->Version >= 2 )//second texture index added in version 2
		    {
			    *(int*)pcPos = ASURA_TEXTURE_NONE;
			    pcPos += 4;
		    }

			if (pxMaterialChunk->Version >= 3) //normal map texture IDs added in version 3
			{
			    *(int*)pcPos = ASURA_TEXTURE_NONE;
			    pcPos += 4;
			}

			if (pxMaterialChunk->Version >= 5) // VGP 9Feb2006 specular map texture IDs added in version 5
			{
			    *(int*)pcPos = ASURA_TEXTURE_NONE;
			    pcPos += 4;
			}

			*(int*)pcPos = 0;//material flags
		    pcPos += 4;

		    if( pxMaterialChunk->Version >= 1 ) //project flags added in version 1
		    {
			    *(int*)pcPos = 0;//project flags
			    pcPos += 4;
		    }

			if( pxMaterialChunk->Version >=4 )
			{
			    *(Asura_Hash_ID*)pcPos = 0;//texture anim hash
			    pcPos += 4;
				
			}

			if( pxMaterialChunk->Version >= 6 )
			{
				*(int*)pcPos = 0xffffffff;//specular colour
				pcPos += 4;
			}
            Asura_Assert( pcPos == reinterpret_cast<char*>(pxNewMaterialChunk) + iNewChunkSize, "" );
        }

		*ppcOutputChunkBuffer = reinterpret_cast<char*>( pxNewMaterialChunk );

		if( bEnvironment && ASURA_TEXTURE_NONE != iTextureIdxToAdd )
		{
			m_iUntexturedMaterialIndex = pxNewMaterialChunk->m_iNumberOfMaterials - 1;
		}

		return true;

#endif	//#else ASURA_USE_MATERIAL_RESPONSE_SYSTEM
	}
	// KJL - 17/05/2003 13:59:27 - altered the if below so non-entity lights are always ignored on export
    else if( ASURA_CHUNK_LIGHTS == xChunkHeader.ID )
    {
		//This section needs to deal with both big endian and little endian chunks , so any reading/writing
		//to the buffer will need to be flipped appropriately. RAR 28/08/03
		if(xChunkHeader.Version != 5)
		{
			Asura_Assert(0,"Modify chunk only works with version 5 of the light chunk");
			return false;
		}

		const int iExtraHeaderBytes = 28;//fixed bytes after Asura_Chunk_Header , but before the light data

        const Asura_Chunk_Lights* pxSrcChunkHdr = reinterpret_cast<const Asura_Chunk_Lights*>( pxChunk );

        //get to the chunk buffer after the light chunk header
        const char* pcSrcChunkBuffer = reinterpret_cast<const char*>( pxSrcChunkHdr + 1 );
		//and then skip past to the light data
        const Asura_Light* pxSrcLightData = reinterpret_cast<const Asura_Light*>( pcSrcChunkBuffer+iExtraHeaderBytes );


		int iNumberOfLights = pxSrcChunkHdr->NumberOfLights;
		ASURA_CHUNK_VERIFY(iNumberOfLights,int); //(big endian->little endian)

        // Count reqd lights.
        int iReqdLights = 0;
        for( int i = 0; i < iNumberOfLights; ++i )
        {
            u_int uFlags = pxSrcLightData[i].m_uFlags;
			ASURA_CHUNK_BITFIELD_VERIFY(uFlags,u_int);	//(big endian->little endian)

            if( uFlags & (AsuraLightFlag_HasCorona|AsuraLightFlag_AffectsEntities) )
            {
                ++iReqdLights;
            }
        }

        // Allocate new chunk.
        u_int uNewChunkSize  = sizeof(Asura_Chunk_Lights) + iReqdLights * sizeof(Asura_Light) + iExtraHeaderBytes;
        char* pcNewChunkData = new char[uNewChunkSize];

        // Setup new chunk hdr ptr.
        Asura_Chunk_Lights* pxNewChunkHdr = reinterpret_cast<Asura_Chunk_Lights*>( pcNewChunkData );
        
        // Write new chunk hdr.
        {
            // Most of the hdr will be the same.
            *pxNewChunkHdr = *pxSrcChunkHdr;

            // Correct light count and chunk size.
            pxNewChunkHdr->NumberOfLights = iReqdLights;
			ASURA_CHUNK_VERIFY(pxNewChunkHdr->NumberOfLights,int); //(little endian->big endian)

            pxNewChunkHdr->Size           = uNewChunkSize;
			ASURA_CHUNK_VERIFY(pxNewChunkHdr->Size,int); //(little endian->big endian)
        }

		char* pcDestChunkBuffer = reinterpret_cast<char*>( pxNewChunkHdr + 1 );
		memcpy(pcDestChunkBuffer,pcSrcChunkBuffer,iExtraHeaderBytes);

        // Output new light data.
        Asura_Light* pxOut = reinterpret_cast<Asura_Light*>( pcDestChunkBuffer + iExtraHeaderBytes );
        for( int i = 0; i < iNumberOfLights; ++i )
        {
            u_int uFlags = pxSrcLightData[i].m_uFlags;
			ASURA_CHUNK_BITFIELD_VERIFY(uFlags,u_int); //(big endian->little endian)

            if( uFlags & (AsuraLightFlag_HasCorona|AsuraLightFlag_AffectsEntities) )
            {
                *pxOut++ = pxSrcLightData[i];
            }
        }
        Asura_Assert( pxOut - reinterpret_cast<Asura_Light*>(pcDestChunkBuffer + iExtraHeaderBytes) == iReqdLights, "" );

        *ppcOutputChunkBuffer = pcNewChunkData;
		return true;
    }
    else if( ASURA_CHUNK_HIERARCHY_SKIN == xChunkHeader.ID )
	{
		//load and resave the skin data to strip out tools-only stuff
		//If doing big-endian exports , then any skin data will have been saved out again correctly already
		if(ShouldExportBigEndian()) return false;

		//read in to a temporary skin
		const Asura_Chunk_Hierarchy_Skin* pxSkinChunk = (const Asura_Chunk_Hierarchy_Skin*) pxChunk;
		Asura_Chunk_Memory_Stream xReadStream(pxSkinChunk->Size - sizeof(Asura_Chunk_Hierarchy_Skin), pxSkinChunk+1);

		Asura_Hierarchy_Skin xSkin;
		if(!pxSkinChunk->Process(xReadStream, &xSkin, false)) return false;

		//save out the skin to a new chunk
		Asura_Chunk_Memory_Stream xWriteStream(pxSkinChunk->Size);
		Asura_Chunk_Hierarchy_Skin::Create( xWriteStream, &xSkin, 0, false);

		//grab the memory buffer , and return it as the modified chunk
		*ppcOutputChunkBuffer = (char*) xWriteStream.TakeOwnershipOfMemory();
		return true;
	}
    else if( ASURA_CHUNK_HIERARCHY_SKINLOD == xChunkHeader.ID )
	{
		//load and resave the skin data to strip out tools-only stuff (which is most of the chunk in this case)
		//If doing big-endian exports , then any skin data will have been saved out again correctly already
		if(ShouldExportBigEndian()) return false;

		//read in to a temporary skin
		const Asura_Chunk_Hierarchy_SkinLOD* pxSkinChunk = (const Asura_Chunk_Hierarchy_SkinLOD*) pxChunk;
		Asura_Chunk_Memory_Stream xReadStream(pxSkinChunk->Size - sizeof(Asura_Chunk_Hierarchy_SkinLOD), pxSkinChunk+1);

		Asura_Hierarchy_Skin xSkin;
		Asura_String xRootSkinName;
		if(!pxSkinChunk->Process(xReadStream, &xSkin, &xRootSkinName)) return false;

		//save out the skin to a new chunk
		Asura_Chunk_Memory_Stream xWriteStream(pxSkinChunk->Size);
		Asura_Chunk_Hierarchy_SkinLOD::Create( xWriteStream, &xSkin, xRootSkinName);

		//grab the memory buffer , and return it as the modified chunk
		*ppcOutputChunkBuffer = (char*) xWriteStream.TakeOwnershipOfMemory();
		return true;
	}
	#if 1
	else if(ASURA_CHUNK_HIERARCHY_ANIM == xChunkHeader.ID  && Export_Wii != s_eExportType)
	{
		// KJL - 28/04/2003 09:16:27 - frame removal always on, now it's just a question of overriding
		// the default settings

		m_iTotalAnimationSizeBefore += pxChunk->Size;
		//extract the animation from the chunk
		Asura_Hierarchy_Anim* pxAnim = ((Asura_Chunk_Hierarchy_Anim*)pxChunk)->Process();
		if(!pxAnim)
		{
			m_iTotalAnimationSizeAfter += pxChunk->Size;
			return false;
		}

		//strip out unwanted keyframes
		int iFramesBefore,iFramesAfter;
		if(ShouldExportAnimationWithoutRemovingKeyframes(*pxAnim))
		{
			iFramesBefore	= pxAnim->CalculateTotalKeyFrames();
			iFramesAfter	= iFramesBefore;
		}
		else
		{
		pxAnim->RemoveUnneededKeyFrames(s_fAnimPositionTolerance,s_fAnimAngleTolerance,&iFramesBefore,&iFramesAfter);
		}

		m_iTotalAnimationFramesBefore += iFramesBefore;
		m_iTotalAnimationFramesAfter += iFramesAfter;

		#if 0 // Old uncompressed anim data
		if(iFramesBefore != iFramesAfter)
		{
			//Create a new chunk
			int iChunkSize = Asura_Chunk_Hierarchy_Anim::GetChunkBufferSize(pxAnim);
			char* pcNewChunkData = new char[iChunkSize];
			Asura_Chunk_Hierarchy_Anim::Create(pxAnim,pcNewChunkData,iChunkSize);

			m_iTotalAnimationSizeAfter += iChunkSize;

        	*ppcOutputChunkBuffer = pcNewChunkData;
		}
		else
		{
			m_iTotalAnimationSizeAfter += pxChunk->Size;
		}

		//kill off the animation used for processing
		delete pxAnim;

		return iFramesBefore != iFramesAfter;

		#else // New compressed anim data
		{

			if(ShouldExportAnimationUncompressed(*pxAnim))
			{
				AddMessage(MT_LOGONLY,"Exporting Uncompressed Anim - '%s'",pxAnim->Name.GetString());

				int iChunkSize = Asura_Chunk_Hierarchy_Anim::GetChunkBufferSize(pxAnim);
				char* pcNewChunkData = new char[iChunkSize];
				Asura_Chunk_Hierarchy_Anim::Create(pxAnim,pcNewChunkData,iChunkSize);
				
				*ppcOutputChunkBuffer = pcNewChunkData;
			}
			else
			{
				pxAnim->MakeCompressedKeyFrames();
				int iChunkSize = Asura_Chunk_Hierarchy_CompressedAnim::GetChunkBufferSize(pxAnim);
				char* pcNewChunkData = new char[iChunkSize];
				Asura_Chunk_Hierarchy_CompressedAnim::Create(pxAnim,pcNewChunkData);
				pxAnim->DeleteCompressedKeyFrames();

				*ppcOutputChunkBuffer = pcNewChunkData;
			}
		}

		//kill off the animation used for processing
		delete pxAnim;

		return true;

		#endif
	}
	#endif
	// KOR - unfortunately the chunk version has gone up to 7, and the structure of the chunk has changed significantly
	// meaning that this will be difficult to patch up. However, this section is only needed for stripping out cutscenes
	// that will be replaced by FMV on certain platforms, which isn't used much anymore, as far as I know.
#if 0
	else if(ASURA_CHUNK_ENTITY == xChunkHeader.ID && (s_eExportType == Export_PS2 || s_eExportType == Export_Wii))
	{
		//This section needs to deal with both big endian and little endian chunks , so any reading/writing
		//to the buffer will need to be flipped appropriately. RAR 28/08/03

		//may need to modify cutscene controllers for PS2/GC level exports
		#ifdef ASURA_IGNORE_ENTITY_PROJECT_DATA
		if(xChunkHeader.Version !=0)
		{
			Asura_Assert(0,"Modify chunk only works with version 0 of entity chunks");
			return false;
		}
		#else
		if(xChunkHeader.Version >1)
		{
			Asura_Assert(0,"Modify chunk only works with version 0 and 1 of entity chunks");
			return false;
		}
		#endif

		Asura_Chunk_Entity* pxEntityChunk = (Asura_Chunk_Entity*)pxChunk;

		u_short usClassification = pxEntityChunk->Classification;
		ASURA_CHUNK_VERIFY(usClassification,u_short);
		
		if(usClassification != AsuraEntityClass_CutsceneController)	
		{
			return false;
		}

		char* pcBuffer = (char*)(pxEntityChunk+1) ;

		#ifndef ASURA_IGNORE_ENTITY_PROJECT_DATA
		bool bHasProjectData = (pxEntityChunk->Version >= 1) && pxEntityChunk->m_ucHasProjectData;
		if(bHasProjectData)
		{
			//skip past the project data.
			//easiest way to do this would appear to be to create the project data ,and see
			//what postition the chunkbuffer is at afterwards. RAR 15/02/2006
			Asura_Chunk_Memory_Stream	xStream (pxEntityChunk->Size-sizeof(Asura_Chunk_Entity) , reinterpret_cast<const void*>(pcBuffer));

			//one short lived project data coming up...
			delete Asura_Entity_Server::Project_CreateProjectData (pxEntityChunk->Classification, xStream);

			pcBuffer = (char*) xStream.GetAddress();
		}
		#endif   //	 !defined  ASURA_IGNORE_ENTITY_PROJECT_DATA


		//check the cutscene name
		u_int uVersion;
		ASURA_CHUNK_EXTRACT( pcBuffer, uVersion, u_int );
		ASURA_CHUNK_VERIFY(uVersion,u_int);

		if(uVersion!=6)
		{
			//Now that the files are all saved under a temporary name , we should always
			//be getting the most recent version of the chunk. RAR 09/06/03
			Asura_Assert(0,"Modify chunk only recognises version 6 of the cutscene controller chunk");
			return false;
		}
		
		Asura_String xName;
		ASURA_CHUNK_EXTRACT_STRING(pcBuffer, xName);

		//find the cutscene
		Asura_Cutscene* pxCutscene = Asura_Cutscene_System::GetCutscene(xName.GetString());
		if(!pxCutscene) return false;
		if(ShouldCutsceneBeExportedToThisPlatform(pxCutscene)) return false;

		//copy and modify the chunk
		int iChunkSize = xChunkHeader.Size;
		char* pcNewChunkBuffer = new char[iChunkSize];
		memcpy(pcNewChunkBuffer,pxChunk,iChunkSize);


		int iFlagsOffset =  iChunkSize-12;

		u_int* puFlags = (u_int*)(pcNewChunkBuffer+(iFlagsOffset));

		ASURA_CHUNK_BITFIELD_VERIFY(*puFlags,u_int); //flip from big endian if needed
		*puFlags |= AsuraCutsceneControllerFlag_TriggerMessagesOnly;
		ASURA_CHUNK_BITFIELD_VERIFY(*puFlags,u_int); //and flip back
		

		*ppcOutputChunkBuffer = pcNewChunkBuffer;
		return true;
	}
#endif

	return false;
}


bool Tools_Level_Export::AddSkinSectionBBChunk( HANDLE hFile, const Asura_String& xSkinName, Asura_Hierarchy_Skin* pxSkin )
{
    if( !pxSkin->m_pxSectionBoundingBoxData )
    {
        pxSkin->InitSectionBoundingBoxes();
    }

	Asura_SaveBlock xSaveBlock;
    if( !Asura_Chunk_Hierarchy_SectionBBs::Create(pxSkin, xSkinName, &xSaveBlock) )
    {
        return false;
    }

    u_int uDataSize = xSaveBlock.GetSize() - ASURA_FILE_ID_NUM_CHARS;

    DWORD uBytesWritten;
	BOOL bOK = WriteFile( hFile, xSaveBlock.GetStartPtr() + ASURA_FILE_ID_NUM_CHARS, uDataSize, &uBytesWritten, NULL );

    return( bOK && uBytesWritten == uDataSize );
}

bool Tools_Level_Export::AddBigEndianSkinChunks(HANDLE hFile, Asura_Hierarchy_Skin* pxSkin)
{
	if(!pxSkin)
		return false;

	Asura_SaveBlock xSaveBlock;
	Tools_SaveAsura::Static_AddChunksToSkinFile(pxSkin, &xSaveBlock);

	return ( -1 != xSaveBlock.WriteToFile(hFile));
}
/// End of file. ///