//
// Tools_AsrFileManager.h
//
// Created by Richard Rice - 18/10/2007
// 
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//


#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef _TOOLS_ASRFILEMANAGER_H_
#define _TOOLS_ASRFILEMANAGER_H_

class Tools_TempLoadModule;
class Asura_Module;
class Asura_Environment_Module;
class Tools_AsrFileState;
template<class T> class List;
class Asura_Mesh;

//For managing environments that have had their main .asr split into several pieces
class Tools_AsrFileManager
{
	friend class Tools_Overlay_AsrFileManager;

public :
	static void Shutdown();
	static void Reset();

	static bool LoadMainAsuraFile( const char* szFilename);
	static bool IsUsingMultipleAsrFiles(){return s_bUsingMultipleAsrFiles;}

	//used by Asura_Chunk_Env to allocate storage for extra modules being loaded by that chunk
	static void AllocateModuleStorage(int iNumModules);
	//used by Asura_Chunk_Env to get a pointer to the storage space for the next module
	static Tools_TempLoadModule* GetNextModule();

	//flag the file containing the given mesh as modified
	static void SetAsrModified(const Asura_Mesh* pxMesh);
	//flag as modified using the stripped filename
	static void SetAsrModified(const char* szStrippedName);
	//set all the env asr files as modified
	static void SetAllAsrModified();

	//sets the text file containing the list of asr files to be modified
	static void SetFileListFileModified();
	static void GetFileListFilename(Asura_String& xFilename);
	static bool CanSaveFileListFile(bool bShowMessage);


	static bool IsModified();
	//adds all the env asr files to the list to check for external changes
	static void SetFilesToCheckForExternalModification();
	static void FlagEnvironmentFilesAsLoaded();

	static void SaveAll();

	static void SaveModified();

	//used by Tools_SaveAsura to make backups of the asr files before saving
	static void MakeBackups();

	//Associates all the environment meshes with the new filename
	static void PreSaveAllAs(const char* szNewFilename);

	//For dealing with the overlay for moving meshes between files
	static void CreateOverlay();
	static void CloseOverlay();
	static void ToggleOverlay();
	static bool IsOverlayOpen();

	//updates hidden state of modules based on the files that they are in
	static void UpdateModuleVisibility();


private :

	static Tools_AsrFileState* AddToFileList(const char* szFilename, bool bMainFile);

	static Tools_AsrFileState* GetFile(const char* szStrippedName);

	static void SaveFileListFile();



	///////////
	
	static List<Tools_AsrFileState*> s_xFileList;
	static Tools_AsrFileState* s_pxMainFile;

	static bool s_bFileListFileModified;

	static bool s_bUsingMultipleAsrFiles;

	static int s_iNumModulesAllocated;
	static int s_iNumModulesLoaded;
	static Tools_TempLoadModule** s_ppxModuleStorage;

public :
	static Asura_Vector_2 s_xOverlayPosition;
	static Asura_Vector_2 s_xUserSetTreeSize;

};

//for storing the module data upon load until the environment can be allocated
class Tools_TempLoadModule
{
public :
	Tools_TempLoadModule();
	~Tools_TempLoadModule();


	Asura_Module* m_pxModule;
	Asura_Environment_Module* m_pxEnvModule;
};

class Tools_AsrFileState
{
public :
	Asura_String m_xRelativeName;//filename relative to the exe's dir
	Asura_String m_xStrippedName;//filename without path
	bool m_bModified;
	bool m_bMainFile;//the asr file whose name matches all the ancillary files
	bool m_bToBeDeleted;//indicates that this file is to be deleted when next saved
	bool m_bVisible;//determines whether the modules in this file should be visible
};


#endif //_TOOLS_ASRFILEMANAGER_H_