//
// Tools_AsuraFileContents.h
//
// Created by Richard Rice - 31/01/03
// 
// Copyright (C) 2003 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef __TOOLS_ASURA_FILECONTENTS_H__
#define __TOOLS_ASURA_FILECONTENTS_H__

#include "Core/Asura_Hash.h"
#include "Core/Asura_Handle.h"
#include "Core/Asura_String.h"
#include "Misc/list_tem.hpp"



//For opening an asura file to find out whether it contains environment,animations , skins etc
//without the contents being processed by the main asura code.
class Asura_Chunk_Header;

class Tools_AsuraFileContents
{
public :
	Tools_AsuraFileContents();
	~Tools_AsuraFileContents();

	bool LoadAsuraFile(const char* pszFilename,bool bCheckNetwork = true);

	bool IsEnvironment(){return m_bEnvironment;}
	bool IsFragmentFile(){return m_xFragmentList.size()!=0;}
	
	//Gets file type taken from AsuraFileType enumeration in Asura_Chunk_Loaders.
	//If no file info chunk exists , then the type is guessed based on the chunks found
	enum AsuraFileType GetFileType() const;


	const List<Asura_String>* GetMeshList(){return &m_xMeshList;}
	const List<Asura_String>* GetSkinList(){return &m_xSkinList;}
	const List<Asura_String>* GetAnimList(){return &m_xAnimList;}
	const List<Asura_Hash_ID>* GetFragmentList(){return &m_xFragmentList;}


	static bool IsEnvironmentFile(const char* pszFilename,bool bCheckNetwork = true);
	static enum AsuraFileType  GetAsuraFileType(const char* pszFilename,bool bCheckNetwork = true);


protected :
	void DeleteAll();

	void ProcessChunk(const class Asura_Chunk_Hierarchy_Anim* pxChunk, Asura_Handle iHandle);
	void ProcessChunk(const class Asura_Chunk_Hierarchy_Skin* pxChunk, Asura_Handle iHandle);
	void ProcessChunk(const class Asura_Chunk_Mesh_Vertices* pxChunk, Asura_Handle iHandle);
	void ProcessChunk(const class Asura_Chunk_Fragment* pxChunk, Asura_Handle iHandle);
	void ProcessChunk(const class Asura_Chunk_FileInfo* pxChunk, Asura_Handle iHandle);

	enum AsuraFileType GuessFileType() const;

	bool m_bDetermineFileTypeOnly;
	bool m_bEnvironment;
	int m_iFileType;
	List<Asura_String> m_xMeshList;
	List<Asura_String> m_xSkinList;
	List<Asura_String> m_xAnimList;
	List<Asura_Hash_ID> m_xFragmentList;
	Asura_String m_xFilename;
	int m_iNumAnimTables;
	int m_iNumTiledTexture;

};


class Tools_AsuraFileChunkLister
{
public :
	Tools_AsuraFileChunkLister();	
	~Tools_AsuraFileChunkLister();	

	bool LoadAsuraFile(const char* pszFilename,bool bCheckNetwork = true,bool bFailureHandledExternally = false);

	//loads the asura file , only reading in chunks with IDs contained in piChunkIDArray
	bool LoadAsuraFile(const char* pszFilename, const int* piChunkIDArray, int iChunkIDArraySize,bool bCheckNetwork = true,bool bFailureHandledExternally = false);


	const List<const Asura_Chunk_Header*>* GetChunkList() const {return &m_xChunkList;}

	const Asura_Chunk_Header* GetChunkByID(int iChunkID);

	void ListChunksByID(int iChunkID,List<const Asura_Chunk_Header*>& xOutputList);

	//Note that the chunks are left as big endian if the file is big endian , so any calls to ListChunksByID() and
	//GetChunkByID() will need to swap the IDs as required.
	bool IsBigEndian() const {return m_bBigEndian;}

protected  :
	void DeleteAll();

	List<const Asura_Chunk_Header*> m_xChunkList;
	char* m_pcFileBuffer;
	bool m_bBigEndian;
	bool m_bIndividualChunkAllocations;
};


#endif //__TOOLS_ASURA_FILECONTENTS_H__
