//
// Tools_BatchProcessing.h
//
// Created by Richard Rice - 01/10/2003
// 
// Copyright (C) 2003 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef __TOOLS_BATCH_EXPORT_H_
#define __TOOLS_BATCH_EXPORT_H_

#include "Core/Asura_String.h"
template<class T> class List;


#define TOOLS_BATCH_PROCESS_LOGFILE "logfiles\\batch_process.txt"
#define TOOLS_EXIT_CODE_BATCH_ABORTED (2)


class Tools_BatchProcessing
{
public :
	//checks whether the file has the filename of a batch processing file
	static bool IsBatchFile(const char* pszFilename);
	static void SetConsoleMode( const bool bIsConsoleMode) { s_bIsConsoleMode = bIsConsoleMode;}

	//initialises batch export , setting the level to load into DefaultFileToLoad
	static bool Initialise( const char* szIniName );
	static void DoBatchProcess();
	//Starts processing the next file in the ini file
	static int NextFile();
    static bool IsFirstFile( void ) { return s_bIsFirstFile; }
	static void SetFirstFile( const bool bSetFirstBatch) { s_bIsFirstFile = bSetFirstBatch; }

	static bool IsConsoleMode() { return( s_bIsConsoleMode);}
	static bool IsBatch();
	static bool IsLevelExport(){return s_eBatchType == BATCH_LEVELEXPORT;}
	static bool IsWaypointing(){return s_eBatchType == BATCH_WAYPOINTING;}
	static bool IsGlobalIllumination() {return s_eBatchType == BATCH_GLOBAL_ILLUMINATION;}

	static void Abort(){s_bAbortExport = true; Asura_Assert( false, "Tools Batch Processing Aborted");}
	static bool	IsAborted(){return s_bAbortExport;}

	static struct Tools_IniFile* GetIniFile();
	static int GetSectionIndex(){return s_iBatchIndex;}

private :

	enum BatchType
	{
		BATCH_NONE,
		BATCH_LEVELEXPORT,
		BATCH_WAYPOINTING,
		BATCH_GLOBAL_ILLUMINATION,
	};
	
	static enum BatchType GetBatchFileType(const char* pszFilename);
	static bool VerifyIniFile();
	
    static void Project_OnFinishLastFile( void );
	static void Shutdown();

	static bool GetNextValidBatchStage(bool bCheckCurrent);

	static void ExecuteCommandOnCompletion();


	static enum BatchType s_eBatchType;
	static bool s_bAbortExport;
	static int s_iBatchIndex;
	static int s_iBatchFileIndex;
    static bool s_bIsFirstFile;
	static Asura_String s_xCommandOnCompletion;
    
    static bool s_bIsConsoleMode;

	static List<Asura_String> s_xIniFilenameList;

};


#endif //__TOOLS_BATCH_EXPORT_H_
