//
// Tools_GUIMenu_Export.cpp
//
// Created by Richard Rice - 03/02/2007
// 
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

#ifdef ASURA_USE_GUI_MENU_SYSTEM
#include "Load&Save/LevelExport/Tools_Level_Export.h"

//asura
#include "ChunkSystem/Asura_SaveBlock.h"
#include "ChunkSystem/Asura_Chunk_GUIMenu.h"
#include "GUIMenu/Asura_GUIMenu_Audio.h"
#include "GUIMenu/Asura_GUIMenu_Menu_Page.h"
#include "GUIMenu/Asura_GUIMenu_System.h"

//tools
#include "GUIMenu/Tools_GUIMenu_System.h"
#include "GUIMenu/Tools_Mode_GUIMenu.h"
#include "Misc/Tools_Modal_Overlay.h"
#include "PC/FileSystem/Tools_FileSystem.h"

// -- these are prototypes, so this cpp file can access them in the Tools_Level_Export.cpp
bool LeaveSpaceForResourceFileListChunk( HANDLE hFile );
bool FillOutResourceFileListChunk( HANDLE hFile );


static const int g_iPLATFORM_ALL = -1;

void Tools_Level_Export::ExportGUIMenus()
{
	ExportGUIMenus(NULL);
}

void Tools_Level_Export::ExportGUIMenus(const char* szMenuFile)
{
	if(!This) return;

	bool bVerify =  Tools_Mode_GUIMenu::VerifyBeforeExport();

	if ( bVerify == false )
	{
		bool bExport =  ShowYesNoOverlay("Verification Errors\n\nThere are errors in the data. Do you want to continue with the export?");

		if ( bExport == false )
		{
			return;
		}
	}

	//ask which platform to export for
	ListSelection_Overlay xOverlay("Export menus for :");

	xOverlay.AddItem("*All*",g_iPLATFORM_ALL);

	for(int iPlatform=0;iPlatform<ExportType_NumPlatforms;iPlatform++)
	{
		EXPORT_TYPE ePlatform = (EXPORT_TYPE) iPlatform;
		if(!This->IsPlatformAllowed(ePlatform)) continue;

		xOverlay.AddItem(GetPlatformName(iPlatform),iPlatform);
	}

	if(!xOverlay.DoModal()) return;

	//export the selected platform(s)
	int iSelectedPlatform = xOverlay.GetSelectionData();
	for(int iPlatform=0;iPlatform<ExportType_NumPlatforms;iPlatform++)
	{
		if(iSelectedPlatform == iPlatform || iSelectedPlatform == g_iPLATFORM_ALL)
		{

			EXPORT_TYPE ePlatform = (EXPORT_TYPE) iPlatform;

			if(!This->IsPlatformAllowed(ePlatform)) continue;

			ExportGUIMenus(ePlatform,szMenuFile);
		}
	}
}

void Tools_Level_Export::ExportGUIMenus(int iPlatform,const char* szMenuFile)
{
	Asura_Assert(This,"");

	EXPORT_TYPE eOldExportType = s_eExportType;
	s_eExportType = (EXPORT_TYPE) iPlatform;
	
	if(szMenuFile)
	{
		This->ExportGUIMenusMain(szMenuFile);
	}
	else
	{
		//check all the menus , and find the files to export
		Asura_NameTree xFilesDone;

		for(u_int i=0;i<Asura_GUIMenu_System::GetCurrentMenuListCount();++i )
		{
			Asura_GUIMenu_Menu_Page* pxMenuPage(Asura_GUIMenu_System::GetMenuPageByIndex( i));

			//ignore menus that are menu containers
			if(pxMenuPage->GetToolsFlags() & ASURA_GUIMENU_TOOLS_FLAG_STYLECONTAINER) continue;

			//ignore menus that aren't for this platform
			if(!pxMenuPage->ShouldExportToPlatform(iPlatform)) continue;

			//ignore level specific menus
			const char* szFilename = pxMenuPage->GetFilename();
			if(!szFilename || !*szFilename) continue;

			if(!stricmp(Tools_GUIMenu_System::GetLevelSpecificFile(),szFilename)) continue;

			//already dealt with this file?
			int iJunkIndex;
			if(xFilesDone.FindNodeIndexByName(szFilename,true,iJunkIndex)) continue;

			//export all menus in the same file
			This->ExportGUIMenusMain(szFilename);
		}

		// -- export the gui audio file --
		This->ExportGUIAudioMain( "GUIAudio");
	}

	s_eExportType = eOldExportType;
}

void Tools_Level_Export::ExportGUIMenusMain(const char* szMenuFile)
{
	//Early check to make sure destination files are writeable. RAR 17/01/2006
	if(!IsGUIMenuFileWriteable(szMenuFile))
	{
		return;
	}

	char szHeading[100];
	Asura_PrintfToBfr(szHeading,"Menu Export - %s",sizeof(szHeading),true,szMenuFile);

	s_szMainHeading = szHeading;

	StartExport();
	OpenExportLog(szMenuFile);


	StartGUIMenuExport();
	if(!IsAbortFlagSet())
	{
		StartFileExport();
	}

	if(!IsAbortFlagSet())
	{
		AddGUIMenuFileToExportList(szMenuFile);
		

		Asura_String xExportFilename;
		GetGUIMenuFilename(szMenuFile,xExportFilename);
		
		ExportFilesInList( xExportFilename.GetString(),false);
	}

	
	EndFileExport();
	EndGUIMenuExport();
	CloseExportLog();

	EndExport(false);
}

void Tools_Level_Export::ExportGUIAudioMain(const char* szMenuAudioFile)
{
	//Early check to make sure destination files are writeable. RAR 17/01/2006
	if(!IsGUIMenuAudioFileWriteable(szMenuAudioFile))
	{
		return;
	}

	char szHeading[100];
	Asura_PrintfToBfr(szHeading,"Menu Audio Export - %s",sizeof(szHeading),true,szMenuAudioFile);

	s_szMainHeading = szHeading;

	StartExport();
	OpenExportLog(szMenuAudioFile);


	StartGUIMenuAudioExport();
	if(!IsAbortFlagSet())
	{
		StartFileExport();
	}

	if(!IsAbortFlagSet())
	{
		Asura_String	xExportFilename;

		Asura_GUIMenu_Audio::AddFilesToExportList();

		GetGUIMenuAudioFilename(szMenuAudioFile, xExportFilename);
		ExportFilesInList(xExportFilename.GetString(), false);
	}

	EndFileExport();
	EndGUIMenuAudioExport();
	CloseExportLog();

	EndExport(false);
}

void Tools_Level_Export::StartGUIMenuExport()
{
    s_bIsGUIMenuExport = true;
}

void Tools_Level_Export::EndGUIMenuExport()
{
	DeleteTemporaryFiles();
    s_bIsGUIMenuExport = false;
}

void Tools_Level_Export::StartGUIMenuAudioExport()
{
	s_bIsGUIMenuExport = true;
	s_bIsGUIMenuAudioExport	= true;
}

void Tools_Level_Export::EndGUIMenuAudioExport()
{
	DeleteTemporaryFiles();
	s_bIsGUIMenuExport = false;
	s_bIsGUIMenuAudioExport	= false;
}

bool Tools_Level_Export::IsGUIMenuFileWriteable(const char* szMenuFile,bool bShowMessage)
{
	Asura_String xFilename;
	GetGUIMenuFilename(szMenuFile,xFilename);

	return IsWriteableIncludingLanguageFiles(xFilename.GetString(),bShowMessage);
}

void Tools_Level_Export::GetGUIMenuFilename(const char* szMenuFile,Asura_String& xOutputFilename)
{
	char acFilename[MAX_PATH];

	const char* szPlatformName = GetPlatformName(s_eExportType);
	if (szPlatformName)
	{
		Asura_PrintfToBfr(acFilename, "..\\%s_Release\\GUIMenu\\%s.gui%s", MAX_PATH, true, 
			szPlatformName, szMenuFile, ShouldExportBigEndian() ? ASURA_STANDARD_FILE_EXTENSION_BE_SUFFIX : "");
		xOutputFilename = acFilename;
	}
	else
	{
		xOutputFilename = szMenuFile;
		xOutputFilename += ".unknown_platform";
	}
}

bool Tools_Level_Export::IsGUIMenuAudioFileWriteable(const char* szMenuFile, bool bShowMessage)
{
	Asura_String xFilename;
	GetGUIMenuAudioFilename(szMenuFile,xFilename);

	return IsWriteableIncludingLanguageFiles(xFilename.GetString(),bShowMessage);
}

void Tools_Level_Export::GetGUIMenuAudioFilename(const char* szMenuFile,Asura_String& xOutputFilename)
{
	char acFilename[MAX_PATH];

	const char* szPlatformName = GetPlatformName(s_eExportType);
	if (szPlatformName)
	{
		Asura_PrintfToBfr(acFilename, "..\\%s_Release\\GUIMenu\\%s.asr%s", MAX_PATH, true, 
			szPlatformName, szMenuFile, ShouldExportBigEndian() ? ASURA_STANDARD_FILE_EXTENSION_BE_SUFFIX : "");
		xOutputFilename = acFilename;
	}
	else
	{
		xOutputFilename = szMenuFile;
		xOutputFilename += ".unknown_platform";
	}
}

void Tools_Level_Export::AddGUIMenuFileToExportList(const char* szMenuFile)
{
	//add all menus marked as being in this file
	for(u_int i=0;i<Asura_GUIMenu_System::GetCurrentMenuListCount();++i )
	{
		Asura_GUIMenu_Menu_Page* pxMenuPage(Asura_GUIMenu_System::GetMenuPageByIndex( i));

		//ignore menus that are menu containers
		if(pxMenuPage->GetToolsFlags() & ASURA_GUIMENU_TOOLS_FLAG_STYLECONTAINER) continue;

		//ignore menus that aren't for this platform
		if(!pxMenuPage->ShouldExportToPlatform(s_eExportType)) continue;

		const char* szFilename = pxMenuPage->GetFilename();
		if(!szFilename || !*szFilename) continue;

		if(stricmp(szFilename,szMenuFile)) continue;

		//already added this menu?
		if(m_xGUIMenuList.contains(pxMenuPage)) continue;

		m_xGUIMenuList.add_entry(pxMenuPage);

		//add any textures used by the menus
		Asura_GUIMenu_System::AddTexturesToExportList(pxMenuPage);

		// add any other files used by the menus (anims, skins, etc.)
		Asura_GUIMenu_System::AddFilesToExportList(pxMenuPage);
	}
}


bool Tools_Level_Export::AddGUIMenus(HANDLE pSaveHandle)
{
	if(!m_xGUIMenuList.size()) return true;

	if(IsAbortFlagSet()) return false;

	//Make sure any style instances are up to date before the export.
	//RAR 17/05/2007
	Tools_Mode_GUIMenu::DoStyleUpdateIfRequired();


	//Fill in a save block containing the menu chunks
	Asura_SaveBlock xSaveBlock;

    // (MH - 04.04.2007) temporary solution to not knowing which anim sequences are needed:
	// put all sequences at the start of every file. If already loaded they will be skipped
	// when the file is loaded in.
	Asura_Chunk_GUIMenu_AnimatorSequences::Create(&xSaveBlock);

	//write out chunks for the text pages used
	Asura_HashTree xTextPageTree;
	for(LIF<const Asura_GUIMenu_Menu_Page*> xMenuLIF(&m_xGUIMenuList);!xMenuLIF.done();xMenuLIF.next())
	{
		const Asura_GUIMenu_Menu_Page* pxPage = xMenuLIF();
		if( pxPage )
		{
			pxPage->ListUsedTextPages(xTextPageTree);
		}		
	}

	if(xTextPageTree.GetNumberOfNodes())
	{
		//make a chunk for each text page used
		Asura_Hash_ID* puTextPage = new Asura_Hash_ID[xTextPageTree.GetNumberOfNodes()];
		xTextPageTree.MakeHashList(puTextPage);
		
		for(int i=0; i<xTextPageTree.GetNumberOfNodes(); i++)
		{			
			Asura_Chunk_GUIMenu_TextPage::Create(puTextPage[i], &xSaveBlock);
		}

		delete [] puTextPage;
	}



	for(LIF<const Asura_GUIMenu_Menu_Page*> xMenuLIF(&m_xGUIMenuList);!xMenuLIF.done();xMenuLIF.next())
	{
		Asura_Chunk_GUIMenu::Create(xMenuLIF(),&xSaveBlock);
	}

	//write the chunks to the file
    u_int uDataSize = xSaveBlock.GetSize() - ASURA_FILE_ID_NUM_CHARS;

    DWORD uBytesWritten;
	WriteFile( pSaveHandle, xSaveBlock.GetStartPtr() + ASURA_FILE_ID_NUM_CHARS, uDataSize, &uBytesWritten, NULL );

	if(uBytesWritten != uDataSize)
	{
        AddMessage( MT_ERROR, "I/O error." );
		return false;
	}

	return true;
}

bool Tools_Level_Export::AddGUIMenuAudioData( HANDLE pSaveHandle)
{
	if(IsAbortFlagSet()) return false;

	//Fill in a save block containing the menu chunks
	Asura_SaveBlock xSaveBlock;
	
	Asura_Chunk_GUIMenu_Audio::Create(&xSaveBlock);

	//write the chunks to the file
	u_int uDataSize = xSaveBlock.GetSize() - ASURA_FILE_ID_NUM_CHARS;

	DWORD uBytesWritten;
	WriteFile( pSaveHandle, xSaveBlock.GetStartPtr() + ASURA_FILE_ID_NUM_CHARS, uDataSize, &uBytesWritten, NULL );

	if(uBytesWritten != uDataSize)
	{
		AddMessage( MT_ERROR, "I/O error." );
		return false;
	}

	return true;
}


#endif //ASURA_USE_GUIMENU_SYSTEM
