//
// Tools_LoadedFileList.cpp
//
// Created by Richard Rice - 08/05/2003
// 
// Copyright (C) 2003 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"
#include "Load&Save/Tools_LoadedFileList.h"

#include "Core/Asura_NameTree.h"
#include "Core/Asura_HashTree.h"
#include "Debugging/Asura_Assert.h"
#include "FileAccess/Asura_FileAccess.h"
#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Overlay.h"

#include "Load&Save/LevelExport/Tools_Level_Export.h"
#include "Misc/Tools_Modal_Overlay.h"
#include "PC/FileSystem/Tools_FileSystem.h"

Asura_StringTree* Tools_LoadedFileList::s_pxLoadedFileTree = NULL;
Asura_TemplateHashTree<Tools_LoadedFile>* Tools_LoadedFileList::s_pxFileTree = NULL;

//////////////////////////////////////////////////////////////////////////////////////////////
struct Tools_LoadedFile
{
	Tools_LoadedFile(){}	
	Tools_LoadedFile(const char* pszFilename)
	:m_xRelativeFilename(pszFilename)
	,m_bExternallyModified(false)
	,m_bLocked(false)
	,m_bUnlock(false)
	,m_bTrackExternalModification(false)
	{
		memset(&m_xModifiedTime,0,sizeof(FILETIME));
	}

	Asura_String m_xRelativeFilename;
	FILETIME m_xModifiedTime;
	bool m_bExternallyModified;
	bool m_bLocked;
	bool m_bUnlock;
	bool m_bTrackExternalModification;
};
//////////////////////////////////////////////////////////////////////////////////////////////
class ToolsUnlockFile_Overlay : public Tools_BaseMessage_Overlay
{
public :
	ToolsUnlockFile_Overlay(Asura_TemplateHashTree<Tools_LoadedFile>* pxFileTree)
	:Tools_BaseMessage_Overlay("These files have been modified outside of tools.\nSelect the ones you want to be able to be enabled for saving.\n\n*** WARNING ***\n\nAny changes made by whoever last checked in the files will most likely be lost.")
	{
		m_pxFileTree = pxFileTree;
	}

	protected :
		virtual void InitialiseOverlay(class Asura_Overlay* pxOverlay,Asura_Vector_2& xOffset);

	private :
		static void AddControlIfLocked(Asura_Hash_ID uHash,Tools_LoadedFile* pxFile);
		static void UnlockIfFlagged(Asura_Hash_ID uHash,Tools_LoadedFile* pxFile);
		static void OkaySelected(int iThis);
		static void CancelSelected(){};
	

		Asura_TemplateHashTree<Tools_LoadedFile>* m_pxFileTree;

		static Asura_Overlay* s_pxOverlay;
		static Asura_Vector_2 s_xOffset;
	
};

Asura_Overlay* ToolsUnlockFile_Overlay::s_pxOverlay;
Asura_Vector_2 ToolsUnlockFile_Overlay::s_xOffset;

void ToolsUnlockFile_Overlay::InitialiseOverlay(class Asura_Overlay* pxOverlay,Asura_Vector_2& xOffset)
{
	s_pxOverlay = pxOverlay;
	s_xOffset = xOffset;

	//add check boxes for the modified files
	m_pxFileTree->Traverse(AddControlIfLocked);
	xOffset = s_xOffset;

	//add okay and cancel buttons
	xOffset.y += 1.0f;
	Asura_Dialog* pxDialog = new Asura_Function_Dialog("Okay",xOffset,OkaySelected,(int)this,&pxOverlay->m_bShouldClose);
	pxOverlay->AddDialog(pxDialog);
	xOffset.x = pxDialog->m_xMaxCoord.x + 2.0f;

	pxDialog = new Asura_Function_Dialog("Cancel",xOffset,CancelSelected,&pxOverlay->m_bShouldClose);
	pxOverlay->AddDialog(pxDialog);
	xOffset.x = 1.0f;

}

void ToolsUnlockFile_Overlay::AddControlIfLocked(Asura_Hash_ID uHash,Tools_LoadedFile* pxFile)
{
	if(pxFile->m_bLocked)
	{
		pxFile->m_bUnlock = false;
		Asura_Dialog* pxDialog = new Asura_Tickbox_Dialog(Tools_FileSystem::StripPath(pxFile->m_xRelativeFilename.GetString()),s_xOffset,&pxFile->m_bUnlock);
		s_pxOverlay->AddDialog(pxDialog);
		s_xOffset.y += Asura_Dialog::DefaultSpacing();
	}
}

void ToolsUnlockFile_Overlay::OkaySelected(int iThis)
{
	ToolsUnlockFile_Overlay* pxThis = (ToolsUnlockFile_Overlay*) iThis;

	//traverse the file tree unlocking the flagged files
	pxThis->m_pxFileTree->Traverse(UnlockIfFlagged);
}

void ToolsUnlockFile_Overlay::UnlockIfFlagged(Asura_Hash_ID uHash,Tools_LoadedFile* pxFile)
{
	if(pxFile->m_bUnlock)
	{
		pxFile->m_bLocked = false;
		pxFile->m_bUnlock = false;
	}
}

//////////////////////////////////////////////////////////////////////////////////////////////

void Tools_LoadedFileList::Initialise()
{
	s_pxLoadedFileTree = new Asura_StringTree;
	s_pxFileTree = new Asura_TemplateHashTree<Tools_LoadedFile>;
}

void Tools_LoadedFileList::Shutdown()
{
	delete s_pxLoadedFileTree;
	s_pxLoadedFileTree = 0;
	delete s_pxFileTree;
	s_pxFileTree = 0;
}

void Tools_LoadedFileList::Reset()
{
	Asura_Assert(s_pxLoadedFileTree,"Not initialised");
	s_pxLoadedFileTree->Reset();
	s_pxFileTree->Reset();
}

void Tools_LoadedFileList::AddFile(const char* pszFilename,bool bStoreModifiedTime /*= true*/)
{
	Asura_Assert(s_pxLoadedFileTree,"Not initialised");

	Asura_String xRelativePath = Asura_FileAccess::PlatformRemoveNetworkOrExecutableDir( pszFilename );

    // Paths sometimes come through with forward slashes instead of
    // backslashes. We should regard those as the same so let's just always
    // use backslashes internally.
    xRelativePath.ReplaceAllChars( '/', '\\' );

	//add this filename to the name tree
	int iJunkIndex;
	s_pxLoadedFileTree->FindNodeIndexByName( xRelativePath.GetString(), true, iJunkIndex );

	if(bStoreModifiedTime)
	{
		StoreModifiedTime(pszFilename);
		//TrackExternalModification(pszFilename);
	}
}

bool Tools_LoadedFileList::IsFileLoaded(const char* pszFilename)
{
	Asura_Assert(s_pxLoadedFileTree,"Not initialised");

    // See comments in AddFile() above.
	Asura_String xRelativePath = Asura_FileAccess::PlatformRemoveNetworkOrExecutableDir( pszFilename );
    xRelativePath.ReplaceAllChars( '/', '\\' );

	int iJunkIndex;
	return s_pxLoadedFileTree->FindNodeIndexByName( xRelativePath.GetString(), false, iJunkIndex );
}

void Tools_LoadedFileList::StoreModifiedTime(const char* pszFilename)
{
	Asura_Assert(s_pxFileTree,"Not initialised");	

	const char* pszRelativeName = Asura_FileAccess::PlatformRemoveNetworkOrExecutableDir(pszFilename);
	//entry already exist?
	Tools_LoadedFile* pxFile = s_pxFileTree->GetDataByName(pszRelativeName);
	if(pxFile) return;
	
	//need to add then
	s_pxFileTree->AddNode(Asura_GetHashID(pszRelativeName),Tools_LoadedFile(pszRelativeName));
	pxFile = s_pxFileTree->GetDataByName(pszRelativeName);
	pxFile->m_bTrackExternalModification = false;

	Asura_Assert(pxFile,"Failed to find entry just added to tree");
	Asura_Assert(!stricmp(pxFile->m_xRelativeFilename.GetString(),pszRelativeName),"Two files with same hash ID?");
	
	UpdateModifiedTime(pszFilename,true);		

}

void Tools_LoadedFileList::TrackExternalModification(const char* pszFilename)
{
	Asura_Assert(!Tools_Level_Export::IsLevelExport(),"");//filesnames are changed during level exports , so best not to add files at that point

	Asura_Assert(s_pxFileTree,"Not initialised");	

	const char* pszRelativeName = Asura_FileAccess::PlatformRemoveNetworkOrExecutableDir(pszFilename);
	//entry already exist?
	Tools_LoadedFile* pxFile = s_pxFileTree->GetDataByName(pszRelativeName);
	if(pxFile)
	{
		pxFile->m_bTrackExternalModification = true;
		return;
	}
	
	//need to add then
	s_pxFileTree->AddNode(Asura_GetHashID(pszRelativeName),Tools_LoadedFile(pszRelativeName));
	pxFile = s_pxFileTree->GetDataByName(pszRelativeName);
	pxFile->m_bTrackExternalModification = true;

	Asura_Assert(pxFile,"Failed to find entry just added to tree");
	Asura_Assert(!stricmp(pxFile->m_xRelativeFilename.GetString(),pszRelativeName),"Two files with same hash ID?");
	
	UpdateModifiedTime(pszFilename,true);		

}

void Tools_LoadedFileList::UpdateModifiedTime(const char* pszFilename,bool bCheckNetwork)
{
	if(Tools_Level_Export::IsLevelExport()) return;//ignore during level exports , since files are saved to different directories
	Asura_Assert(s_pxFileTree,"Not initialised");	

	const char* pszRelativeName = Asura_FileAccess::PlatformRemoveNetworkOrExecutableDir(pszFilename);
	//entry exist?
	Tools_LoadedFile* pxFile = s_pxFileTree->GetDataByName(pszRelativeName);
	if(!pxFile) return;

	//convert to network path if needed
	char szFullFilename[MAX_PATH];
	const char* pszFileToCheck = pszRelativeName;
	if(bCheckNetwork)
	{
		Asura_FileAccess::PlatformGetNetworkFilename(pszRelativeName,szFullFilename,MAX_PATH);
		pszFileToCheck = pszRelativeName;
	}

	WIN32_FILE_ATTRIBUTE_DATA xFileAttribs = { 0};
	BOOL bAttribStatus = GetFileAttributesEx( pszFileToCheck, GetFileExInfoStandard, &xFileAttribs);
	if( 0 != bAttribStatus)
	{
	    pxFile->m_xModifiedTime = xFileAttribs.ftLastWriteTime;
		pxFile->m_bExternallyModified = false;
		pxFile->m_bLocked = false;
	}
	//else file may not exist yet , which may not be a problem
}

bool Tools_LoadedFileList::IsLockedBecauseOfExternalModification(const char* pszFilename)
{
	if(Tools_Level_Export::IsLevelExport()) return false;//ignore during level exports , since files are saved to different directories
	Asura_Assert(s_pxFileTree,"Not initialised");	

	//find entry for file
	const char* pszRelativeName = Asura_FileAccess::PlatformRemoveNetworkOrExecutableDir(pszFilename);
	Tools_LoadedFile* pxFile = s_pxFileTree->GetDataByName(pszRelativeName);
	if(!pxFile) return false;
	if(!pxFile->m_bTrackExternalModification) return false;

	//previously found to be modified?
	if(pxFile->m_bExternallyModified)
	{
		return pxFile->m_bLocked;
	}


	//if the file is read-only , it doesn't matter
	if(Tools_FileSystem::IsReadOnly(pszRelativeName)) return false;


	//get the modified time
	//If the local copy doesn't exist , then we don't need to know the modified time , since
	//saving won't be allowed anyway (if the file exists at all)

	WIN32_FILE_ATTRIBUTE_DATA xFileAttribs = { 0};
	BOOL bAttribStatus = GetFileAttributesEx( pszRelativeName, GetFileExInfoStandard, &xFileAttribs);

	if( bAttribStatus )
	{
		if(Tools_FileSystem::ApproxCompareFileTime(&xFileAttribs.ftLastWriteTime,&pxFile->m_xModifiedTime))
		{
			//file has been modified
			pxFile->m_bExternallyModified = true;
			pxFile->m_bLocked = true;
		}
	}

	return pxFile->m_bLocked;
}

static int g_iNumModifiedFilesFound = 0;

static void CheckIfModified(Asura_Hash_ID uHash,Tools_LoadedFile* pxFile)
{
	if(!pxFile->m_bTrackExternalModification) return;
	if(pxFile->m_bExternallyModified) return;//previously found to be modified , so no need to check again	

	WIN32_FILE_ATTRIBUTE_DATA xFileAttribs = { 0};
	BOOL bAttributesStatus = GetFileAttributesEx( pxFile->m_xRelativeFilename.GetString(), GetFileExInfoStandard, &xFileAttribs);
	if( 0 != bAttributesStatus)
	{
		if( 0 != (FILE_ATTRIBUTE_READONLY & xFileAttribs.dwFileAttributes))
		{
			return;
		}
 
	    FILETIME xFiletime = xFileAttribs.ftLastWriteTime;

		if(Tools_FileSystem::ApproxCompareFileTime(&xFiletime,&pxFile->m_xModifiedTime))
		{
			//file has been modified
			pxFile->m_bExternallyModified = true;
			pxFile->m_bLocked = true;
			g_iNumModifiedFilesFound++;
		}
	}
}

void Tools_LoadedFileList::CheckForExternallyModifiedFiles(bool bDisplayMessages)
{
	Asura_Assert(s_pxFileTree,"Not initialised");

	g_iNumModifiedFilesFound = 0;
	s_pxFileTree->Traverse(CheckIfModified);

	if(g_iNumModifiedFilesFound && bDisplayMessages)
	{
		ShowMessageOverlay("%d file(s) were modified outside of tools.\nYou won't be allowed to save over these unless you unlock them first.\n(File->Unlock Externally Modified).",g_iNumModifiedFilesFound);
	}
}

static int g_iNumLockedFilesFound = 0;
static void CheckIfLocked(Asura_Hash_ID uHash,Tools_LoadedFile* pxFile)
{
	CheckIfModified(uHash,pxFile);
	if(pxFile->m_bLocked) g_iNumLockedFilesFound++;
}


void Tools_LoadedFileList::UnlockFiles()
{
	Asura_Assert(s_pxFileTree,"Not initialised");

	g_iNumLockedFilesFound = 0;
	s_pxFileTree->Traverse(CheckIfLocked);

	if(!g_iNumLockedFilesFound)
	{
		ShowMessageOverlay("No files have been modified outside of tools.\nThere is no need to unlock them.");
		return;
	}
	else
	{
		//show the unlocking overlay thing
		ToolsUnlockFile_Overlay xOverlay(s_pxFileTree);
		xOverlay.ShowOverlay();
	}

}
