//
// Tools_OptionsFile.cpp
//
// Created by Kevin Lea - 05/10/2001 16:51:50
// 
// Copyright (C) 2001 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"
#include "Load&Save/Tools_OptionsFile.h"

//asura
#include "Camera/Asura_Camera.h"
#include "Interface/Asura_Overlay_System.h"
#include "Interface/Asura_Cursor.h"
#include "Rendering/Asura_Render.h"
#include "Rendering/Asura_Direct3D_UserOptions.h"
#include "Core/Asura_Windows.h"

//tools
#include "Tools_System.h"
#include "Load&Save/Tools_SaveAsura.h"
#include "Mesh/Tools_PlatformMesh_System.h"
#include "Misc/Tools_Render_Options.h"
#include "Misc/Tools_Render.h"
#include "Movement/Tools_Movement.h"
#include "PC/FileSystem/Tools_FileSystem.h"
#include "Rendering/Tools_RenderCollisionMesh.h"
#include "Sound/Tools_Sound_System.h"



////////////////////////////////////////////////////////////////////////////////////////////////
/*------------------**
** Tools_OptionsFile **
**------------------*/



//local store of file getting settings
static bool s_bFileAccess_GetExistingFiles = false;
static float s_fFileAccess_GetNewFiles_MinSize = 0.0f;
static float s_fFileAccess_GetNewFiles_MaxSize = 0.0f;


Tools_OptionsFile* Tools_OptionsFile::s_pxThis = 0;


Tools_OptionsFile::Tools_OptionsFile()
{
	Asura_Assert(!s_pxThis,"Tools_OptionsFile already allocated");
	s_pxThis = this;
}

Tools_OptionsFile::~Tools_OptionsFile()
{
	s_pxThis = 0;
}


void Tools_OptionsFile::Load()
{
	#define BOOL_READ(string,var) GetBoolFromIniFileBuffer(string,buffer,var)
	#define INT_READ(string,var) GetIntFromIniFileBuffer(string,buffer,var)
	#define FLOAT_READ(string,var) GetFloatFromIniFileBuffer(string,buffer,var)
	
	//build the filename
	char filename[MAX_PATH];
	sprintf(filename,"%s\\AsuraTools.ini",ToolsIniDataDirectory);

	//open the file
	HANDLE handle=CreateFile(filename,GENERIC_READ, 0, 0, OPEN_EXISTING,FILE_FLAG_RANDOM_ACCESS, 0);
	if(handle!=INVALID_HANDLE_VALUE)
	{
		char* buffer;
		int file_size;
		unsigned long bytes_read;
	
		//copy the file contents into a buffer
		file_size= GetFileSize(handle,0);
		buffer=new char[file_size+1];
		ReadFile(handle,buffer,file_size,&bytes_read,0);
		CloseHandle(handle);

		float fCursorSpeed = 1.0f;

		BOOL_READ("Camera Relative Rotation",Tools_Movement::UseRelativeRotation);
		BOOL_READ("Invert Y Axis",Tools_Movement::InvertYAxis);
		FLOAT_READ("Mouse Rotation Multipier",Tools_Movement::MouseRotationMultiplier);
        FLOAT_READ("Camera Movement Scale",Tools_Movement::CameraMovementScale);
        FLOAT_READ("Camera Rotation Scale",Tools_Movement::CameraRotationScale);
        FLOAT_READ("Object Movement Scale",Tools_Movement::ObjectMovementScale);
        FLOAT_READ("Object Rotation Scale",Tools_Movement::ObjectRotationScale);
		BOOL_READ("Hardware Mouse",Tools_System::RequestHardwareMouse);
		FLOAT_READ("Cursor Speed",fCursorSpeed);
		BOOL_READ("Use Vertical Field Of View",Tools_Render_Options::DefaultFieldOfViewIsVertical);
		FLOAT_READ("Field Of View",Tools_Render_Options::DefaultFieldOfView);
		BOOL_READ("Use View Distance",Tools_Render_Options::UseViewDistance);
		FLOAT_READ("View Distance",Tools_Render_Options::ViewDistance);
		BOOL_READ("Use Icon Distance",Tools_Render_Options::s_bUseRenderAsIconDistance);
		FLOAT_READ("Icon Distance",Tools_Render_Options::s_fRenderAsIconDistance);
		FLOAT_READ("Icon Size",Tools_Render_Options::s_fDistantIconSize);
		BOOL_READ("Use Player Height",Tools_Render_Options::s_bUsePlayerHeight);
		FLOAT_READ("Player Height",Tools_Render_Options::s_fPlayerHeight);
		BOOL_READ("Dont Load Wavs",Tools_Sound_System::s_bDontLoadWavs);
		BOOL_READ("Load Wavs On Quit To Game",Tools_Sound_System::s_bLoadWavsOnQuitToGame);
		BOOL_READ("Mute Phonons",Tools_Sound_System::s_bMutePhonons);
		BOOL_READ("Mute Streaming",Tools_Sound_System::s_bMuteStreaming);
		FLOAT_READ("Gamma",Tools_Render_Options::s_fGamma);
		BOOL_READ("Don't Compress Textures On Load",Tools_System::s_bDontCompressTexturesOnLoad);
		FLOAT_READ("Flat Subdivision Size",Tools_Render_Options::s_fFlatSubdivideSize);
		FLOAT_READ("Curved Subdivision Size",Tools_Render_Options::s_fCurvedSubdivideSize);
		BOOL_READ("Prelight Subdivided Meshes",Tools_Render_Options::s_bPrelightSubdividedMeshes);

		int iRed, iGreen, iBlue;
		if( INT_READ("Overlay Colour Red",iRed) && 
			INT_READ("Overlay Colour Green",iGreen) && 
			INT_READ("Overlay Colour Blue",iBlue) )
		{
			Asura_Overlay_System::s_uOverlayColour = (iRed << 16) | (iGreen << 8) | iBlue;
		}
		
		if(!INT_READ("Faded Overlay Alpha", *(int*)&Asura_Overlay_System::s_uFadedOverlayAlpha))
		{
			bool bUseTranslucentOverlays = true;
			BOOL_READ("Use Translucent Overlays",bUseTranslucentOverlays);
			Asura_Overlay_System::s_uFadedOverlayAlpha = bUseTranslucentOverlays ? 127 : 255;
		}

		int iAlpha;
		if( INT_READ("Wireframe Colour Red",iRed) && 
			INT_READ("Wireframe Colour Green",iGreen) && 
			INT_READ("Wireframe Colour Blue",iBlue) &&
			INT_READ("Wireframe Colour Alpha",iAlpha) )
		{
			Tools_Render_Options::SetWireframeColour ((iAlpha << 24) | (iRed << 16) | (iGreen << 8) | iBlue);
		}
		BOOL_READ( "Use 3dsMax Colours", Tools_Render_Options::s_bUseMaxWireframeColour);


		bool bIdleWhileMinimised = true;
		BOOL_READ("Idle While Minimised", bIdleWhileMinimised);
		Tools_System::SetIdleWhileMinimised(bIdleWhileMinimised);
		
		//RAR GI always on now. 22/10/2004
		//BOOL_READ("Use Global Illumination",Tools_Render_Options::s_bUseGlobalIllumination);

		BOOL_READ("Glow Effect Enabled",Tools_Render_Options::s_bGlowEffectEnabled);
		Tools_Render_Options::s_bGlowEffectEnabledChanged = true;

		//frame rate options		
		BOOL_READ("Use Max Frame Rate",Tools_Render_Options::s_bClampFrameRate);
		FLOAT_READ("Max Frame Rate",Tools_Render_Options::s_fMaxFrameRate);

		bool bSwapButtons = false;
		BOOL_READ("Swap Mouse Buttons",bSwapButtons);
		Tools_System::SetSwapMouseButtons(bSwapButtons);

		bool bUseWindowsCursor = true;
		BOOL_READ("Use Windows Cursor",bUseWindowsCursor);
		Tools_System::SetUseWindowsCursor(bUseWindowsCursor);

		bool bRestrictMouseToWindowInSubwindowTools;
		if(BOOL_READ("Restrict Mouse To Window",bRestrictMouseToWindowInSubwindowTools))
		{
			Tools_System::SetRestrictMouseToWindowInSubwindowTools(bRestrictMouseToWindowInSubwindowTools);
		}

		bool bRestrictMouseToWindowInSubwindowGame;
	    if(BOOL_READ("Restrict Mouse To Window In Game",bRestrictMouseToWindowInSubwindowGame))
		{
			Tools_System::SetRestrictMouseToWindowInSubwindowGame(bRestrictMouseToWindowInSubwindowGame);
		}

		bool bRestrictMouseToWindowInFullScreenTools;
		if(BOOL_READ("Restrict Mouse To Window Fullscreen",bRestrictMouseToWindowInFullScreenTools))
		{
			Tools_System::SetRestrictMouseToWindowInFullScreenTools(bRestrictMouseToWindowInFullScreenTools);
		}

		bool bRestrictMouseToWindowInFullScreenGame;
		if(BOOL_READ("Restrict Mouse To Window In Game Fullscreen",bRestrictMouseToWindowInFullScreenGame))
		{
			Tools_System::SetRestrictMouseToWindowInFullScreenGame(bRestrictMouseToWindowInFullScreenGame);
		}


		
	
		float fTemp;
		if(FLOAT_READ("CMesh Z Penetration",fTemp))
		{
			Tools_RenderCollisionMesh_System::SetZPenetration(fTemp);
		}

		BOOL_READ("Always Get Existing Files",s_bFileAccess_GetExistingFiles);
		FLOAT_READ("Get Files (Min)",s_fFileAccess_GetNewFiles_MinSize);
		FLOAT_READ("Get Files (Max)",s_fFileAccess_GetNewFiles_MaxSize);


		if(Tools_Render_Options::s_fGamma!=0.5f) Tools_Render_Options::s_bGammaChanged = true;
		Asura_Cursor::SetCursorSpeed(fCursorSpeed);

		
		//Tools_Movement
		bool bAltMouseRotate = Tools_Movement::IsUsingAltForMouseRotate();
		BOOL_READ("Alt Mouse Rotate",bAltMouseRotate);
		Tools_Movement::SetUseAltForMouseRotate(bAltMouseRotate);

		bool bMMBToggled = Tools_Movement::IsMMBToggled();
		BOOL_READ("Middle Button Is Toggled",bMMBToggled);
		Tools_Movement::SetMMBToggled(bMMBToggled);

		bool bMouseWheelSpeed = Tools_Movement::IsUsingMouseWheelForSpeed();
		BOOL_READ("Mouse Wheel Speed",bMouseWheelSpeed);
		Tools_Movement::SetUseMouseWheelForSpeed(bMouseWheelSpeed);

		//Tools_SaveAsura
		int iBackupsToMake = Tools_SaveAsura::GetNumBackupsToMake();
		INT_READ("Number Of Backups",iBackupsToMake);
		Tools_SaveAsura::SetNumBackupsToMake(iBackupsToMake);

		int iBackupInterval = Tools_SaveAsura::GetBackupInterval();
		INT_READ("Backup Interval",iBackupInterval);
		Tools_SaveAsura::SetBackupInterval(iBackupInterval);

		//Tools_PlatformMesh_System
		bool bTemp = Tools_PlatformMesh_System::GetReloadFilesWhenChangingPlatform();
		BOOL_READ("Reload Files When Changing Platform",bTemp);
		Tools_PlatformMesh_System::SetReloadFilesWhenChangingPlatform(bTemp);

#ifdef ASURA_USE_HDR

		// HDR settings
		FLOAT_READ("Key Value", Tools_Render_Options::s_xHDRSettings.m_fKeyValue);
		FLOAT_READ("Adaptation Rate", Tools_Render_Options::s_xHDRSettings.m_fAdaptationRate);
		FLOAT_READ("Min Luminance", Tools_Render_Options::s_xHDRSettings.m_fMinLuminance);
		FLOAT_READ("Max Luminance", Tools_Render_Options::s_xHDRSettings.m_fMaxLuminance);
		FLOAT_READ("White Point", Tools_Render_Options::s_xHDRSettings.m_fWhitePoint);
		FLOAT_READ("Black Point", Tools_Render_Options::s_xHDRSettings.m_fBlackPoint);
		FLOAT_READ("Fixed Luminance", Tools_Render_Options::s_xHDRSettings.m_fFixedLuminance);
		BOOL_READ("Dynamic Exposure", Tools_Render_Options::s_xHDRSettings.m_bDynamicExposure);

#endif // ASURA_USE_HDR

		if (s_pxThis)
		{
			// (AW - 18.09.2003) - Not safe to assume that s_pxThis has been allocated
			// as some non-tools applications might use this function.
			s_pxThis->LoadProjectSpecificOptions(buffer);
		}


	
   
		delete [] buffer;
	}


	Tools_Render_Options::UseDefaultFieldOfView();
	
}

void Tools_OptionsFile::LoadDisplaySettings()
{
	#define BOOL_READ(string,var) GetBoolFromIniFileBuffer(string,buffer,var)
	#define INT_READ(string,var) GetIntFromIniFileBuffer(string,buffer,var)
	#define FLOAT_READ(string,var) GetFloatFromIniFileBuffer(string,buffer,var)

	//build the filename
	char filename[MAX_PATH];
	sprintf(filename,"%s\\AsuraTools.ini",ToolsIniDataDirectory);

	//open the file
	HANDLE handle=CreateFile(filename,GENERIC_READ, 0, 0, OPEN_EXISTING,FILE_FLAG_RANDOM_ACCESS, 0);
	if(handle!=INVALID_HANDLE_VALUE)
	{
		char* buffer;
		int file_size;
		unsigned long bytes_read;

		//copy the file contents into a buffer
		file_size= GetFileSize(handle,0);
		buffer=new char[file_size+1];
		ReadFile(handle,buffer,file_size,&bytes_read,0);
		CloseHandle(handle);


		int iOutput;
/*
		if(GetIntFromIniFileBuffer("IsWindowed", buffer, iOutput))
		{
			const bool bWasRunningFullscreen = Asura_Windows::RunningFullScreen();
			const bool bWantToRunFullscreen = (iOutput == 0) ? true : false;

			if(bWasRunningFullscreen != bWantToRunFullscreen)
			{
				bool bSubwindowSpecifiedOnCommandLine, bFullscreenSpecifiedOnCommandLine;

				Asura_Windows::GetCommandLineWindowStateFlags(bSubwindowSpecifiedOnCommandLine, bFullscreenSpecifiedOnCommandLine);

				if((bSubwindowSpecifiedOnCommandLine && (bWantToRunFullscreen == false)) ||
					(bFullscreenSpecifiedOnCommandLine && (bWantToRunFullscreen == true)) ||
					(!bSubwindowSpecifiedOnCommandLine && !bFullscreenSpecifiedOnCommandLine))
				{
					Asura_Windows::DestroyWindows();
					Asura_Windows::SetRunningFullScreen(bWantToRunFullscreen);
					Asura_Windows::InitialiseWindow(Asura_Windows::GetWindowhInstance(), SW_SHOWNORMAL);
					Asura_Direct3D::RecalcPresentParameters();
					//Asura_Direct3D::RestoreAll(true);
					SetForegroundWindow(Asura_Windows::GetWindowHandle());
					Tools_Movement::CancelToggledMMB();
					Tools_System::UpdateRestrictMouseToWindow();
				}
			}
		}
*/
		bool bIsWindowMaximised = false;
		GetBoolFromIniFileBuffer("Windowed Maximised", buffer, bIsWindowMaximised);
		int iScreenX = 0;
		int iScreenY = 0;
		int iScreenWidth = 0;
		int iScreenHeight = 0;
		GetIntFromIniFileBuffer("Windowed X", buffer, iScreenX);
		GetIntFromIniFileBuffer("Windowed Y", buffer, iScreenY);
		GetIntFromIniFileBuffer("Windowed Width", buffer, iScreenWidth);
		GetIntFromIniFileBuffer("Windowed Height", buffer, iScreenHeight);

		// If window position or size has been specified on the command line, these functions will change the parameters
		Asura_Windows::GetCommandLineWindowPosition(iScreenX, iScreenY);
		Asura_Windows::GetCommandLineWindowSize(iScreenWidth, iScreenHeight);

		Asura_Windows::SetWindowPosition(iScreenX, iScreenY);
		Asura_Windows::SetWindowedXPos(iScreenX);
		Asura_Windows::SetWindowedYPos(iScreenY);

		if(bIsWindowMaximised)
		{
			Asura_Windows::RequestMaximiseWindow();
		}

		if((iScreenWidth > 0) && (iScreenHeight > 0))
		{
			Asura_Direct3D_UserOptions::GetCurrentOptions().SetBackBufferWidth(iScreenWidth, 0, true, false);
			Asura_Direct3D_UserOptions::GetCurrentOptions().SetBackBufferHeight(iScreenHeight, 0, true, false);

			Asura_Windows::SetWindowedWidth(iScreenWidth);
			Asura_Windows::SetWindowedHeight(iScreenHeight);
		}

		for(u_int uScreen = 0; uScreen < 4; ++uScreen)
		{
			char szEntryName[200];
			Asura_PrintfToBfr(szEntryName, "Screen %u Width", 200, true, uScreen);
			if(GetIntFromIniFileBuffer(szEntryName, buffer, iOutput))
			{
				Asura_Direct3D_UserOptions::GetCurrentOptions().SetBackBufferWidth(iOutput, uScreen, false, true);
			}
			Asura_PrintfToBfr(szEntryName, "Screen %u Height", 200, true, uScreen);
			if(GetIntFromIniFileBuffer(szEntryName, buffer, iOutput))
			{
				Asura_Direct3D_UserOptions::GetCurrentOptions().SetBackBufferHeight(iOutput, uScreen, false, true);
			}
		}

#ifdef ASURA_SUPPORT_MULTIPLE_MONITORS
		bool bMultiheadEnabled = Asura_Direct3D_UserOptions::GetMultiheadEnabled();
		BOOL_READ("Multiple Monitors Enabled", bMultiheadEnabled);
		Asura_Direct3D_UserOptions::SetMultiheadEnabled(bMultiheadEnabled);
		Tools_Render_Options::s_bUseMultipleMonitors = bMultiheadEnabled;
#endif

		bool bUseSkinnedUI = Asura_Overlay_System::UseSkinnedUI();
		BOOL_READ("Use Skinned UI",bUseSkinnedUI);
		Asura_Overlay_System::SetUseSkinnedUI(bUseSkinnedUI);

		delete [] buffer;
	}
}

void Tools_OptionsFile::LoadSubwindowFlag(bool& bSubwindowFlag)
{

	//build the filename
	char filename[MAX_PATH];
	sprintf(filename,"%s\\AsuraTools.ini",ToolsIniDataDirectory);

	//open the file
	HANDLE handle=CreateFile(filename,GENERIC_READ, 0, 0, OPEN_EXISTING,FILE_FLAG_RANDOM_ACCESS, 0);
	if(handle!=INVALID_HANDLE_VALUE)
	{
		char* buffer;
		int file_size;
		unsigned long bytes_read;

		//copy the file contents into a buffer
		file_size= GetFileSize(handle,0);
		buffer=new char[file_size+1];
		ReadFile(handle,buffer,file_size,&bytes_read,0);
		CloseHandle(handle);

		GetBoolFromIniFileBuffer("IsWindowed", buffer, bSubwindowFlag);

		delete [] buffer;
	}
}


void Tools_OptionsFile::Save()
{
	#define BOOL_WRITE(string,var) fprintf(file,"%s = %s\n",string,var ? "Yes" : "No")
	#define INT_WRITE(string,var) fprintf(file,"%s = %d\n",string,var)
	#define FLOAT_WRITE(string,var) fprintf(file,"%s = %f\n",string,var)

	//make sure the directory exists
	CreateDirectory(ToolsIniDataDirectory,0);

	//build the filename
	char filename[MAX_PATH];
	sprintf(filename,"%s\\AsuraTools.ini",ToolsIniDataDirectory);

	Tools_FileSystem::MakeWriteable(filename);

	FILE* file = fopen(filename,"wt");
	if(file)
	{
		fprintf(file,"##Movement Settings##\n");
		BOOL_WRITE("Camera Relative Rotation",Tools_Movement::UseRelativeRotation);
		BOOL_WRITE("Invert Y Axis",Tools_Movement::InvertYAxis);
		BOOL_WRITE("Alt Mouse Rotate",Tools_Movement::IsUsingAltForMouseRotate());
		BOOL_WRITE("Mouse Wheel Speed",Tools_Movement::IsUsingMouseWheelForSpeed());
		BOOL_WRITE("Middle Button Is Toggled",Tools_Movement::IsMMBToggled());
		BOOL_WRITE("Swap Mouse Buttons",Tools_System::IsSwappingMouseButtons());
		BOOL_WRITE("Use Windows Cursor",Tools_System::GetUseWindowsCursor());
		BOOL_WRITE("Restrict Mouse To Window",Tools_System::IsRestrictMouseToWindowInSubwindowTools());
		BOOL_WRITE("Restrict Mouse To Window In Game",Tools_System::IsRestrictMouseToWindowInSubwindowGame());
		BOOL_WRITE("Restrict Mouse To Window Fullscreen",Tools_System::IsRestrictMouseToWindowInFullScreenTools());
		BOOL_WRITE("Restrict Mouse To Window In Game Fullscreen",Tools_System::IsRestrictMouseToWindowInFullScreenGame());
		FLOAT_WRITE("Mouse Rotation Multipier",Tools_Movement::MouseRotationMultiplier);
        FLOAT_WRITE("Camera Movement Scale",Tools_Movement::CameraMovementScale);
        FLOAT_WRITE("Camera Rotation Scale",Tools_Movement::CameraRotationScale);
        FLOAT_WRITE("Object Movement Scale",Tools_Movement::ObjectMovementScale);
        FLOAT_WRITE("Object Rotation Scale",Tools_Movement::ObjectRotationScale);

        fprintf(file,"\n##Not Movement Settings....##\n");
		BOOL_WRITE("Hardware Mouse",Tools_System::RequestHardwareMouse);
		FLOAT_WRITE("Cursor Speed",Asura_Cursor::GetCursorSpeed());
		BOOL_WRITE("Use Vertical Field Of View",Tools_Render_Options::FieldOfViewIsVertical);
		if(Tools_Render_Options::FieldOfViewIsVertical)
		{
			FLOAT_WRITE("Field Of View",Tools_Render_Options::VerticalFieldOfViewInDegress);
		}
		else
		{
			FLOAT_WRITE("Field Of View",Tools_Render_Options::HorizontalFieldOfViewInDegress);
		}
		BOOL_WRITE("Use View Distance",Tools_Render_Options::UseViewDistance);
		FLOAT_WRITE("View Distance",Tools_Render_Options::ViewDistance);
		BOOL_WRITE("Use Icon Distance",Tools_Render_Options::s_bUseRenderAsIconDistance);
		FLOAT_WRITE("Icon Distance",Tools_Render_Options::s_fRenderAsIconDistance);
		FLOAT_WRITE("Icon Size",Tools_Render_Options::s_fDistantIconSize);
		FLOAT_WRITE("Gamma",Tools_Render_Options::s_fGamma);
		BOOL_WRITE("Dont Load Wavs",Tools_Sound_System::s_bDontLoadWavs);
		BOOL_WRITE("Load Wavs On Quit To Game",Tools_Sound_System::s_bLoadWavsOnQuitToGame);
		BOOL_WRITE("Mute Phonons",Tools_Sound_System::s_bMutePhonons);
		BOOL_WRITE("Mute Streaming",Tools_Sound_System::s_bMuteStreaming);
		BOOL_WRITE("Use Player Height",Tools_Render_Options::s_bUsePlayerHeight);
		FLOAT_WRITE("Player Height",Tools_Render_Options::s_fPlayerHeight);
		BOOL_WRITE("Don't Compress Textures On Load",Tools_System::s_bDontCompressTexturesOnLoad);
		BOOL_WRITE("Use Max Frame Rate",Tools_Render_Options::s_bClampFrameRate);
		FLOAT_WRITE("Max Frame Rate",Tools_Render_Options::s_fMaxFrameRate);
		FLOAT_WRITE("CMesh Z Penetration",Tools_RenderCollisionMesh_System::GetZPenetration());
		BOOL_WRITE("Reload Files When Changing Platform",Tools_PlatformMesh_System::GetReloadFilesWhenChangingPlatform());
		BOOL_WRITE("Idle While Minimised", Tools_System::GetIdleWhileMinimised());
		
		fprintf(file,"\n##Backup Options##\n");
		INT_WRITE("Number Of Backups",Tools_SaveAsura::GetNumBackupsToMake());
		INT_WRITE("Backup Interval",Tools_SaveAsura::GetBackupInterval());
		
		
		fprintf(file,"\n##Subdivision##\n");
		FLOAT_WRITE("Flat Subdivision Size",Tools_Render_Options::s_fFlatSubdivideSize);
		FLOAT_WRITE("Curved Subdivision Size",Tools_Render_Options::s_fCurvedSubdivideSize);
		BOOL_WRITE("Prelight Subdivided Meshes",Tools_Render_Options::s_bPrelightSubdividedMeshes);
		
		//RAR GI always on now. 22/10/2004
		//BOOL_WRITE("Use Global Illumination",Tools_Render_Options::s_bUseGlobalIllumination);

		BOOL_WRITE("Glow Effect Enabled",Tools_Render_Options::s_bGlowEffectEnabled);
		
		fprintf(file,"\n##Overlay Colours##\n");
		INT_WRITE( "Overlay Colour Red",   (Asura_Overlay_System::s_uOverlayColour >> 16) );
		INT_WRITE( "Overlay Colour Green", (Asura_Overlay_System::s_uOverlayColour >> 8) & 0xff );
		INT_WRITE( "Overlay Colour Blue",  (Asura_Overlay_System::s_uOverlayColour & 0xff) );
		INT_WRITE("Faded Overlay Alpha",Asura_Overlay_System::s_uFadedOverlayAlpha);

		fprintf(file,"\n##Use Skinned UI##\n");
		BOOL_WRITE("Use Skinned UI",Asura_Overlay_System::UseSkinnedUI());

		u_int uWireframeColour = Tools_Render_Options::GetWireframeColour();
		fprintf(file,"\n##Wireframe Colours##\n");
		INT_WRITE( "Wireframe Colour Red",  (uWireframeColour >> 16) & 0xff);
		INT_WRITE( "Wireframe Colour Green", (uWireframeColour >> 8) & 0xff );
		INT_WRITE( "Wireframe Colour Blue",  (uWireframeColour & 0xff) );
		INT_WRITE( "Wireframe Colour Alpha", (uWireframeColour >> 24) & 0xff);
		BOOL_WRITE( "Use 3dsMax Colours", Tools_Render_Options::s_bUseMaxWireframeColour);

		fprintf(file,"\n##Fetching network files##\n");
		fprintf(file,"This setting will cause a \'get\' of any out of date file , if there is an existing read-only local copy.\n");
		BOOL_WRITE("Always Get Existing Files",s_bFileAccess_GetExistingFiles);
		fprintf(file,"\n#The next two settings will cause files to be got if their size is within the specified range ,\n");
		fprintf(file,"#and if their isn't a local copy or the local copy is read-only and out of date.\n");
		fprintf(file,"#These work independently of the option for always getting existing files.\n");
		fprintf(file,"#The file sizes are measure in megabytes\n");
		FLOAT_WRITE("Get Files (Min)",s_fFileAccess_GetNewFiles_MinSize);
		FLOAT_WRITE("Get Files (Max)",s_fFileAccess_GetNewFiles_MaxSize);
		
		fprintf(file,"\n##Display settings##\n");
		INT_WRITE("IsWindowed", !Asura_Windows::RunningFullScreen());
		BOOL_WRITE("Windowed Maximised", Asura_Windows::IsMaximised());
		INT_WRITE("Windowed X", Asura_Windows::GetWindowedXPos());
		INT_WRITE("Windowed Y", Asura_Windows::GetWindowedYPos());
		INT_WRITE("Windowed Width", Asura_Windows::GetWindowedWidth());
		INT_WRITE("Windowed Height", Asura_Windows::GetWindowedHeight());
		for(u_int uScreen = 0; uScreen < 4; ++uScreen)
		{
			char szEntryName[200];
			Asura_PrintfToBfr(szEntryName, "Screen %u Width", 200, true, uScreen);
			INT_WRITE(szEntryName, Asura_Direct3D_UserOptions::GetCurrentOptions().GetBackBufferWidth(uScreen));
			Asura_PrintfToBfr(szEntryName, "Screen %u Height", 200, true, uScreen);
			INT_WRITE(szEntryName, Asura_Direct3D_UserOptions::GetCurrentOptions().GetBackBufferHeight(uScreen));
		}

#ifdef ASURA_SUPPORT_MULTIPLE_MONITORS
		BOOL_WRITE("Multiple Monitors Enabled", Tools_Render_Options::s_bUseMultipleMonitors);
#endif

#ifdef ASURA_USE_HDR

		// HDR settings
		fprintf(file,"\n##HDR settings##\n");
		FLOAT_WRITE("Key Value", Tools_Render_Options::s_xHDRSettings.m_fKeyValue);
		FLOAT_WRITE("Adaptation Rate", Tools_Render_Options::s_xHDRSettings.m_fAdaptationRate);
		FLOAT_WRITE("Min Luminance", Tools_Render_Options::s_xHDRSettings.m_fMinLuminance);
		FLOAT_WRITE("Max Luminance", Tools_Render_Options::s_xHDRSettings.m_fMaxLuminance);
		FLOAT_WRITE("White Point", Tools_Render_Options::s_xHDRSettings.m_fWhitePoint);
		FLOAT_WRITE("Black Point", Tools_Render_Options::s_xHDRSettings.m_fBlackPoint);
		FLOAT_WRITE("Fixed Luminance", Tools_Render_Options::s_xHDRSettings.m_fFixedLuminance);
		BOOL_WRITE("Dynamic Exposure", Tools_Render_Options::s_xHDRSettings.m_bDynamicExposure);

#endif // ASURA_USE_HDR

		Asura_Assert(s_pxThis, "Tools_OptionsFile not allocated!");
		if (s_pxThis)
		{
			s_pxThis->SaveProjectSpecificOptions(file);
		}

		fclose(file);
	}
}


bool Tools_OptionsFile::GetStringFromIniFileBuffer(const char* identifier,const char* buffer,Asura_String& output)
{
	char temp_string[200];
	char* stringPtr=&temp_string[0];
	int id_length=strlen(identifier);
	
	while(*buffer)
	{
		//skip past white space at the beginning of the line
		while(*buffer)
		{
			if(*buffer==' ' || *buffer=='\t') buffer++;
			else break;
		}
		
		//see if the string matches the identifier
		if(!_strnicmp(buffer,identifier,id_length))
		{
			//skip past the identifier
			buffer+=id_length;

			
			//skip past white space
			while(*buffer)
			{
				if(*buffer==' ' || *buffer=='\t') buffer++;
				else break;
			}

			//have we found the equals sign?
			if(*buffer=='=')
			{
				//skip past the sign
				buffer++;
				//skip past white space
				while(*buffer)
				{
					if(*buffer==' ' || *buffer=='\t') buffer++;
					else break;
				}
			
				//now copy the text up until the end of the line
				char* lastNonSpace=0;
				int length = 0;
				while(*buffer && length<200)
				{
					if(*buffer=='\n' || *buffer=='\r')
					{
						*stringPtr=0;
						break;
					}
					else if(*buffer==' ')
					{
						*stringPtr++=*buffer++;
						length++;
					}
					else
					{
						lastNonSpace=stringPtr;
						*stringPtr++=*buffer++;
						length++;
					}
				}
				if(length==200) return false;//too long

				//finally get rid of trailing spaces on the string
				if(lastNonSpace)
				{
					lastNonSpace++;
					*lastNonSpace=0;
				}

				
				//copy the string to the destination
				output = temp_string;

				return true;

			}
			
			
		}

		//skip to next line
		while(*buffer && *buffer!='\r' && *buffer!='\n') buffer++;
		
		if(*buffer=='\r')buffer++;
		if(*buffer=='\n')buffer++;
	}

	return false;
}

bool Tools_OptionsFile::GetBoolFromIniFileBuffer(const char* identifier,const char* buffer,bool& output)
{
	Asura_String temp_string;
	if(GetStringFromIniFileBuffer(identifier,buffer,temp_string))
	{
		if(!_stricmp(temp_string.GetString(),"Yes") || !_stricmp(temp_string.GetString(),"True"))
		{
			output = true;
			return true;
		}
		if(!_stricmp(temp_string.GetString(),"No") || !_stricmp(temp_string.GetString(),"False"))
		{
			output= false;
			return true;
		}
		int temp;
		if(sscanf(temp_string.GetString(),"%d",&temp) == 1)
		{
			output = temp >0;
			return true;
		}
	}
	return false;
}

bool Tools_OptionsFile::GetIntFromIniFileBuffer(const char* identifier,const char* buffer,int& output)
{
	Asura_String temp_string;
	if(GetStringFromIniFileBuffer(identifier,buffer,temp_string))
	{
		return (sscanf(temp_string.GetString(),"%d",&output) == 1);
	}
	return false;
}

bool Tools_OptionsFile::GetFloatFromIniFileBuffer(const char* identifier,const char* buffer,float& output)
{
	Asura_String temp_string;
	if(GetStringFromIniFileBuffer(identifier,buffer,temp_string))
	{
		return (sscanf(temp_string.GetString(),"%f",&output) == 1);
	}
	return false;
}
