//
// Tools_ResourceExportLog.h
//
// Created by Richard Rice - 30/05/2008
// 
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef __TOOLS_RESOURCEEXPORTLOG_H__
#define __TOOLS_RESOURCEEXPORTLOG_H__


struct Tools_ResourceExportLogEntry;
#define TOOLS_RESOURCEXPORTLOG_STACKSIZE 100


//For creating a log file that shows a hierarchy of what is causing what to be added
class Tools_ResourceExportLog
{
public :
	//open and clear the log file
	static void OpenLog(const char* szFilename);
	//'close' the file
	static void CloseLog();
	//clears the log file again if open
	static void ResetLog();
	
	//Add to the stack of dependencies.
	//szType and sszName need to stay valid until removed
	//If bLogNow is true then everything that hasn't yet been logged will be written out
	//Setting to false allow things to be added to the stack in case they are needed , without
	//writing them out if they don't cause anything else to be added.
	static void AddToStack(const char* szType, const char* szName, bool bLogNow);
	//Removes the most recent entry from the stack
	static void RemoveFromStack();

	//forces everything in the stack to be written out now
	static void ForceOutputToLog();

	static bool IsOpen() {return !s_xLogName.IsBlank();}

private :
	static Asura_String s_xLogName;
	static int s_iStackDepth;
	static int s_iHighestLoggedEntry;
	static Tools_ResourceExportLogEntry s_axStack[TOOLS_RESOURCEXPORTLOG_STACKSIZE];


};


//This , and the following macro , cause an entry to be added to the resource export log stack while the
//instance of the class stays in scope.
class Tools_ResourceExportLog_ScopedEntry
{
public :
	Tools_ResourceExportLog_ScopedEntry(const char* szType, const char* szName, bool bLogNow)
	{
		Tools_ResourceExportLog::AddToStack(szType, szName, bLogNow);
	}
	~Tools_ResourceExportLog_ScopedEntry()
	{
		Tools_ResourceExportLog::RemoveFromStack();
	}
};

#define TOOLS_RESOURCEEXPORTLOG(szType, szName, bLogNow) Tools_ResourceExportLog_ScopedEntry __xResourceExportLogEntry ## __LINE__(szType, szName, bLogNow)



#endif //__TOOLS_RESOURCEEXPORTLOG_H__
