//
// Tools_SaveAsura.cpp
//
// Created by Kevin Lea - 05/10/2001 16:52:05
// 
// Copyright (C) 2001 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"
#include "Load&Save/Tools_SaveAsura.h"

#include "Animation/Asura_Animation.h"
#include "Animation/Asura_Hierarchy.h"
#include "Animation/Asura_TextureAnimation.h"
#include "Blueprint/Asura_Blueprint_System.h"
#include "Camera/Asura_Camera.h"
#ifdef ASURA_USE_BILLBOARD_PLANTS
#include "ChunkSystem/Asura_Chunk_BillboardPlant_Batches.h"
#endif
#include "ChunkSystem/Asura_Chunk_Corona.h"
#include "ChunkSystem/Asura_Chunk_Definitions.h"
#include "ChunkSystem/Asura_Chunk_DSP.h"
#ifdef ASURA_SUPPORT_DYNAMIC_MUSIC_SYSTEM
#include "ChunkSystem/Asura_Chunk_DynamicMusic.h"
#endif
#include "ChunkSystem/Asura_Chunk_EnvironmentLBAABBTree.h"
#include "ChunkSystem/Asura_Chunk_EnvironmentalLightTransport.h"
#include "ChunkSystem/Asura_Chunk_Entity.h"
#include "ChunkSystem/Asura_Chunk_Fragment.h"
#include "ChunkSystem/Asura_Chunk_GlobalIllumination.h"
#include "ChunkSystem/Asura_Chunk_Loaders.h"
#include "ChunkSystem/Asura_Chunk_Material.h"
#include "ChunkSystem/Asura_Chunk_Navigation.h"
#include "ChunkSystem/Asura_Chunk_RenderSettings.h"
#include "ChunkSystem/Tools_Chunk_StaticDecals.h"
#include "ChunkSystem/Tools_Chunk_Skybox.h"
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "ChunkSystem/Asura_Chunk_Stream_WindowsFile.h"
#include "ChunkSystem/Asura_Chunk_TextureAnimation.h"
#if defined( ASURA_USE_NEW_VISIBILITY_SYSTEM )
#include "ChunkSystem/Asura_Chunk_Visibility.h"
#endif
#ifdef ASURA_USE_WATER
#include "ChunkSystem/Asura_Chunk_Water.h"
#endif
#ifdef ASURA_USE_BOTANICALS
#include "ChunkSystem/Asura_Chunk_Botanicals_Instances.h"
#include "ChunkSystem/Asura_Chunk_Carpet_Mesh.h"
#include "BotanicalCreation/Tools_Mode_BotanicalCreation.h"
#endif // #ifdef ASURA_USE_BOTANICALS
#ifdef ASURA_USE_PSP_BLUR
#include "ChunkSystem/Asura_Chunk_PSP_Blur.h"
#endif
#ifdef ASURA_USE_PSP_FILTER
#include "ChunkSystem/Asura_Chunk_PSP_Filter.h"
#endif
#ifdef ASURA_USE_REVERBVOLUME_SYSTEM
#include "ChunkSystem/Asura_Chunk_ReverbVolumes.h"
#endif
#ifdef ASURA_USE_FACIAL_ANIMATION_SYSTEM
#include "ChunkSystem/Asura_Chunk_FacialAnimation.h"
#endif
#include "ChunkSystem/Asura_Huffman.h"
#include "ChunkSystem/Asura_Saveblock.h"
#include "Core/Asura_Material_System.h"
#include "Core/Asura_TextureManagement.h"
#include "Core/Asura_Timers.h"
#include "Rendering/Asura_Render.h"
#include "Debugging/Asura_Assert.h"
#include "Debugging/Asura_Debug_Timers.h"
#include "Entity/Asura_Entity.h"
#include "Entity/Asura_StaticMessage_System.h"
#include "Entity/EntityTemplate/Asura_ServerEntity_Template.h"
#ifdef ASURA_USE_FACIAL_ANIMATION_SYSTEM
#include "FacialAnimation/Asura_FacialAnimation_System.h"
#endif
#include "FileAccess/Asura_FileAccess.h"
#include "Interface/Asura_Overlay_System.h"
#include "Interface/Asura_Overlay.h"
#include "Interface/Asura_Dialog.h"
#include "Light/Asura_Light.h"
#include "Mesh/Asura_CollisionMesh.h"
#include "Mesh/Asura_Env.h"
#include "Mesh/Asura_Env_System.h"
#include "Mesh/Asura_Mesh.h"
#include "Mesh/Asura_Mesh_System.h"
#include "Rendering/Asura_Module.h"
#include "Shapes/Asura_Shape_System.h"
#include "Sound/Asura_Phonon.h"
#include "Sound/Asura_Sound.h"
#include "Sound/Asura_MetaMusic_System.h"
#include "SpecialFX/Asura_Skybox.h"
#include "SpecialFX/Asura_Fragment_System.h"
#include "SpecialFX/Asura_Weather_System.h"
#include "SpecialFX/Asura_StaticCorona_System.h"
#if defined(ASURA_USE_STREAMING)
#include "Streaming/Asura_Streaming_System.h"
#endif	// defined(ASURA_USE_STREAMING)

#ifdef ASURA_USE_ANIMATION_IK_SOLVER
#include "Animation/IK/Tools_Animation_IK_Template_System.h"
#endif
#include "Blueprint/Tools_Mode_Blueprint.h"
#ifdef ASURA_USE_FACIAL_ANIMATION_SYSTEM
#include "ChunkSystem/Tools_Chunk_FacialAnimation.h"
#endif
#include "Cutscene/Tools_Mode_Cutscene.h"
#include "Cutscene/Tools_Cutscene_Object.h"

#ifdef ASURA_USE_CONTAINER_SYSTEM
#include "Entity/Container/Tools_Container_System.h"
#endif //#ifdef ASURA_USE_CONTAINER_SYSTEM

#include "Entity/Tools_Entity_Simulation.h"
#include "Entity/Tools_SecondaryEntityFile_System.h"
#include "Entity/Tools_Entity_PhysicalObject.h"
#include "EntityTemplate/Tools_EntityTemplate_System.h"
#include "Entity/Tools_Entity_Data.h"
#ifdef ASURA_USE_FACIAL_ANIMATION_SYSTEM
#include "FacialAnimation/Tools_FacialAnimation_System.h"
#endif
#include "HelperObject/Tools_HelperObject_System.h"
#include "Hierarchy/Tools_Animation_Files.h"
#include "Load&Save/Tools_AsuraFileContents.h"
#include "Load&Save/Tools_AsrFileManager.h"
#include "Load&Save/Tools_LoadedFileList.h"
#include "Load&Save/Tools_Zlib.h"
#if defined(ASURA_USE_MATERIAL_RESPONSE_SYSTEM)
#include "Materials/Tools_Overlay_MaterialResponseOverrides.h"
#endif
#include "Mesh/Tools_PlatformMesh_System.h"
#include "Misc/Tools_IniFile.h"
#include "Misc/Tools_Timed_Messages.h"
#include "Misc/Tools_Render.h"
#include "Misc/Tools_Render_Options.h"
#include "TextureTiler/Tools_Mode_TextureTiler.h"
#include "Particles/Tools_Mode_PFX.h"
#include "PC/FileSystem/Tools_FileSystem.h"
#include "Regions/Tools_Mode_Regions.h"
#include "Shapes/Tools_Mode_ShapeData.h"
#include "Shapes/Tools_Shapes.h"
#include "Sound/Tools_Audio_DSP.h"
#include "Sound/Tools_Overlay_DynamicMusic.h"
#include "Sound/Tools_Sound_PS2Settings.h"
#include "Sound/Tools_Sound_System.h"
#include "Sound/Tools_Mode_MetaMusic.h"
#include "Sound/Tools_SoundEvent_System.h"
#ifdef ASURA_USE_DIALOGUE_SYSTEM
#include "Sound/Tools_Dialogue_System.h"
#endif
#ifdef ASURA_USE_STREAMING
#include "Streaming/Tools_Mode_Streaming.h"
#include "Streaming/Tools_Streaming_System.h"
#endif
#include "Texturing/Tools_Texturing_MaterialResponse.h"
#include "Tools_System.h"

#ifdef ASURA_USE_AXON
#include "AnimTemplates/Tools_Mode_AnimTemplates.h"
#endif

#ifdef ASURA_USE_BOTANICALS
#include "Botanicals/Asura_Botanicals_TemplateManager.h"
#include "Botanicals/Asura_Botanicals_InstanceManager.h"
#include "Botanicals/Tools_Mode_Botanicals.h"
#include "BotanicalCreation/Tools_Mode_BotanicalCreation.h"
#endif // #ifdef ASURA_USE_BOTANICALS

#ifdef ASURA_USE_CARPET
#include "Carpet/Asura_Carpet_StyleManager.h"
#include "Carpet/Asura_Carpet_MeshManager.h"
#endif // #ifdef ASURA_USE_CARPET

#ifdef ASURA_USE_STATIC_DECAL_SYSTEM
#include "Decals/Tools_StaticDecal_System.h"
#endif
#ifdef ASURA_USE_DECAL_MANAGER
#include "Decals/Tools_Overlay_Decals.h"
#endif

#ifdef ASURA_USE_GUI_MENU_SYSTEM
//#include "GUIMenu/Asura_GUIMenu_Message.h"
#include "GUIMenu/Asura_GUIMenu_System.h"
//#include "GUIMenu/Asura_GUIMenu_Menu_Page.h"

#include "GUIMenu/Tools_GUIMenu_System.h"
#endif // ASURA_USE_GUI_MENU_SYSTEM

#if defined (ASURA_USE_MATERIAL_RESPONSE_SYSTEM)
#include "Materials/Tools_MaterialResponse_System.h"
#include "Texturing/Tools_Texturing_MaterialResponse.h"
#endif

#include "TextureTiler/Tools_Mode_TextureTiler.h"


#define FILTER_CHUNK( chunkID, createFunctionCall ) \
    if( !Tools_Level_Export::IsExport() || Tools_Level_Export::ShouldChunkBeExported_Static(chunkID) ) \
    { \
        createFunctionCall; \
    }

// Kevin Lea - 08/08/2001 15:08:11 - extern to variable defined in Dredd_LoadEnvironment
extern Asura_String EnvironmentFilename;
static Asura_String g_xRealFilename;

Tools_SaveAsura* Tools_SaveAsura::This = NULL;

Asura_NameTree* Tools_SaveAsura::EnvironmentTextureTree;

Asura_String Tools_SaveAsura::NameWithoutExtension;
char Tools_SaveAsura::GetFilenameReturnString[MAX_PATH];

bool Tools_SaveAsura::LightingModified = false;
bool Tools_SaveAsura::s_bMissionLightingModified = false;
bool Tools_SaveAsura::s_bEnvironmentalLightTransportModified = false;
bool Tools_SaveAsura::s_bGlobalIlluminationModified = false;
bool Tools_SaveAsura::s_bMissionGlobalIlluminationModified = false;
bool Tools_SaveAsura::ModuleLinkingModified = false;
bool Tools_SaveAsura::SkyboxModified = false;
bool Tools_SaveAsura::SoundsModified = false;
bool Tools_SaveAsura::s_bMissionSoundsModified = false;
bool Tools_SaveAsura::CutscenesModified = false;
bool Tools_SaveAsura::EntityModified = false;
bool Tools_SaveAsura::ShapesModified = false;
bool Tools_SaveAsura::WaypointingModified = false;

#ifdef ASURA_USE_BILLBOARD_PLANTS
bool Tools_SaveAsura::BillboardPlantsModified = false;
#endif
#ifdef ASURA_USE_STREAMING
bool Tools_SaveAsura::s_bStreamingConnectionsModified = false;
#endif
#ifdef ASURA_USE_ATTRACTORS
bool Tools_SaveAsura::s_bAttractorsModified = false;
bool Tools_SaveAsura::s_bMissionAttractorsModified = false;
#endif
bool Tools_SaveAsura::s_bBlueprintsModified = false;
#ifdef ASURA_USE_AXON
bool Tools_SaveAsura::s_bAnimTemplatesModified = false;
#endif
#ifdef ASURA_USE_STREAMING
bool Tools_SaveAsura::s_bGlobalMetaMusicModified = false;
#endif
#ifdef ASURA_USE_CARPET
bool Tools_SaveAsura::s_bCarpetModified = false;
#endif
#ifdef ASURA_USE_BOTANICALS
bool Tools_SaveAsura::s_bBotanicalsModified = false;
bool Tools_SaveAsura::s_bBotanicalCreationModified = false;
#endif
#ifdef ASURA_USE_STATIC_DECAL_SYSTEM
bool Tools_SaveAsura::s_bStaticDecalsModified = false;
#endif
#ifdef ASURA_USE_DECAL_MANAGER
bool Tools_SaveAsura::s_bDecalTemplatesModified = false;
#endif
#ifdef ASURA_USE_REVERBVOLUME_SYSTEM
bool Tools_SaveAsura::s_bReverbVolumesModified = false;
#endif

bool Tools_SaveAsura::s_bTextureTilerModified = false;

enum AsuraFileType Tools_SaveAsura::FileType = AsuraFile_Environment;
Asura_Hierarchy_Skin* Tools_SaveAsura::s_pxSkin = 0;
Asura_Hierarchy_Anim* Tools_SaveAsura::s_pxAnim = 0;
Asura_FragmentType* Tools_SaveAsura::s_pxFragmentType = 0;

List<Asura_String> Tools_SaveAsura::s_xModifiedAsuraFileList;

Asura_String Tools_SaveAsura::s_xTemplateFilename;
Asura_String Tools_SaveAsura::s_xTextureTilerFilename;
Asura_String Tools_SaveAsura::s_xMissionFilenameWithoutExtension;
bool Tools_SaveAsura::s_bMission_UseMissionLighting = true;
bool Tools_SaveAsura::s_bMission_UseMissionGlobalIllumination = true;

int Tools_SaveAsura::s_iNumBackupsToMake = 3;
int Tools_SaveAsura::s_iBackupInterval = 30;

const Asura_ServerEntity_Template* Tools_SaveAsura::s_pxExportedTemplate = 0;



Tools_SaveAsura::Tools_SaveAsura()
{
	This=this;
	s_pxSkin = 0;
	s_pxAnim = 0;
	s_pxFragmentType = 0;
	s_pxExportedTemplate = 0;
}

void Tools_SaveAsura::Shutdown()
{
	while(s_xModifiedAsuraFileList.size())
	{
		s_xModifiedAsuraFileList.delete_first_entry();
	}
}

void Tools_SaveAsura::SetRealFilename( const Asura_String& xRealFilename )
{
    g_xRealFilename = xRealFilename;
}

const char* Tools_SaveAsura::GetRealFilename()
{
	if(!g_xRealFilename.IsBlank())
	{
		return g_xRealFilename.GetString();
	}
	else
	{
		return EnvironmentFilename.GetString();
	}
}


void Tools_SaveAsura::SetFilename(const char* pszMainFilename)
{
	EnvironmentFilename = pszMainFilename;	
	UpdateFilename(EnvironmentFilename);
}

void Tools_SaveAsura::UpdateFilename(const Asura_String& Filename)
{
	//get the filename without extension
	Asura_Char filename[MAX_PATH];
	strcpy(filename,Filename.GetString());
	char* dotpos = strrchr(filename,'.');
	if(dotpos) *dotpos = 0;

	NameWithoutExtension = filename;

}



void Tools_SaveAsura::GenerateEnvironmentForShapes()
{
#if defined(ASURA_USE_STREAMING)
	Asura_Environment&		xEnvironment	= Asura_Environment_System::GetCurrentEnvironment();
	Asura_Module_Handler&	xModuleHandler	= Asura_Module_System::GetCurrentHandler();
	Asura_Environment*		pxEnvironment	= &xEnvironment;
	Asura_Module_Handler*	pxModuleHandler	= &xModuleHandler;
#else
	const char*				szName			= "Shapes";
	Asura_Environment*		pxEnvironment	= Asura_Environment_System::GetEnvironment(szName);
	Asura_Module_Handler&	xModuleHandler	= Asura_Module_System::GetCurrentHandler();
	Asura_Module_Handler*	pxModuleHandler	= &xModuleHandler;
#endif	// defined(ASURA_USE_STREAMING)

	Asura_Assert(pxEnvironment,"");

	if((FileType == AsuraFile_Skin || FileType == AsuraFile_AnimFragment)&& s_pxSkin->IsSkinnedModel() && !s_pxSkin->m_bManualLOD)
	{
		//for skinned models , just show the highest LOD mesh
		//(except for manually generated LODs which will be shown)
		pxEnvironment->CreateModules(1);
    	pxModuleHandler->AllocateModuleStorage( *pxEnvironment );

		Asura_Vector_3 xPosition(0,0,0);

		Asura_Mesh* pxMesh = s_pxSkin->GetMesh();
		if( !pxMesh->BoundingBoxDataValid )
		{
			pxMesh->CalculateBoundingBox();
		}

		pxModuleHandler->SetupModuleMeshForFakeEnv( 0, pxMesh );

		pxEnvironment->Modules[0].m_uRegion = 0;
		pxEnvironment->Modules[0].m_uReqdRegions = (1 << 0);

		pxModuleHandler->SetupModulePosForFakeEnv( 0, xPosition );
		
	}
	else
	{
		pxEnvironment->CreateModules(Asura_Mesh_System::GetNumberOfMeshes());
    	pxModuleHandler->AllocateModuleStorage( *pxEnvironment );

		Asura_Vector_3 xPosition(0,0,0);

		//since no environment data exists , put all the shapes in a line
		for(int i=0;i<Asura_Mesh_System::GetNumberOfMeshes();i++)
		{
    	    Asura_Mesh* pxMesh = Asura_Mesh_System::GetMesh( i );
			if( !pxMesh->BoundingBoxDataValid )
			{
				pxMesh->CalculateBoundingBox();
			}

    	    pxModuleHandler->SetupModuleMeshForFakeEnv( i, pxMesh );

			pxEnvironment->Modules[i].m_uRegion = 0;
    	    pxEnvironment->Modules[i].m_uReqdRegions = (1 << 0);

			if(i) xPosition.x += -pxMesh->BoundingBox.MinX;
    	    pxModuleHandler->SetupModulePosForFakeEnv( i, xPosition );
			xPosition.x += pxMesh->BoundingBox.MaxX + 1.0f;
		}
	}

	//if this is a skin file containing an object hierarchy , position the shapes in the restpose
	//(Disabled since its not that useful without the ability to set the orientations RAR 27/04/2005)
	#if 0
	if(FileType == AsuraFile_Skin || FileType == AsuraFile_AnimFragment)
	{
		Asura_Assert(s_pxSkin,"");
		if(!s_pxSkin->IsSkinnedModel())
		{
			Asura_Assert(s_pxSkin->BoneRestPosition,"");
			Asura_Assert(s_pxSkin->BoneRestOrientation,"");
			Asura_Assert(s_pxSkin->SectionData,"");

            Asura_Hierarchy_System::s_pxSkin = s_pxSkin;

			extern void CalcRestPoseBone(int bone, const Asura_Matrix_3x3& m, const Asura_Vector_3& p);
            CalcRestPoseBone(0, Asura_Maths::IdentityMatrix3x3, Asura_Maths::ZeroVector3);

			Asura_Hierarchy_Skin* pxLODSkin = s_pxSkin;
			Asura_Vector_3 xOffset(0,0,0);
			while(pxLODSkin)
			{
				for(int i=0;i<s_pxSkin->NumberOfBones;i++)
				{
					if(pxLODSkin->SectionData[i].Mesh)
					{
						//search for the 'module' using this mesh
						for(int j=0;j<Asura_Mesh_System::GetNumberOfMeshes();j++)
						{
            	            if( Asura_Module_System::GetMeshForModule(j) == pxLODSkin->SectionData[i].Mesh )
            	            {
							    Asura_Module_System::SetupModulePosForFakeEnv( j, s_pxSkin->BoneRestPosition[i]+xOffset);
							    break;
            	            }
            	        }
					}
				}
				pxLODSkin = pxLODSkin->m_pxNextLowerSkin;
				xOffset.x += 5.0f;
			}
		}
	}
	#endif

	if(FileType == AsuraFile_Fragment)
	{
		//for fragment files , set up all the shapes in the positions listed in the fragment data
		Asura_Assert(s_pxFragmentType,"");

		for(int i=0;i<Asura_Mesh_System::GetNumberOfMeshes();i++)
		{
			//get the hash id for the mesh
			Asura_Hash_ID uHash = Asura_GetHashID( (Asura_Module_System::GetMeshForModule(i))->Name.GetString() );

			//search through the fragments for one using this mesh
			for(int j=0;j<s_pxFragmentType->m_iNumberOfFragments;j++)
			{
				Asura_Fragment* pxFragment = &s_pxFragmentType->m_pxFragments[j];
				if(pxFragment->m_uShapeHash == uHash)
				{
					pxModuleHandler->SetupModulePosForFakeEnv( i, pxFragment->m_xPosition );
					break;
				}
			}
		}

	}
}

void Tools_SaveAsura::LoadEnvironment()
{
	ASURA_CPU_PROFILE_SCOPED_TIMER( "Tools_SaveAsura::LoadEnvironment");
	
	extern char* DefaultFileToLoad;
	
	if (!*DefaultFileToLoad)
	{
		DefaultFileToLoad = "\"Environments\\Bank\\bank.asr\"";
	}
		
	bool loadedFile = false;

	if(*DefaultFileToLoad=='"') 
	{
		char* ptr = DefaultFileToLoad;
		while(*++ptr!='"');
		*ptr=0;
		DefaultFileToLoad++;
	}

	{
#if defined( ASURA_CPU_PROFILE_SCOPED_TIMER)
		Asura_String xDebugTimerText = "Tools_SaveAsura::LoadEnvironment Load File '";
		xDebugTimerText +=  DefaultFileToLoad;
		xDebugTimerText += "'";
		ASURA_CPU_PROFILE_SCOPED_TIMER( xDebugTimerText.GetString());
#endif

		loadedFile = Tools_AsrFileManager::LoadMainAsuraFile(DefaultFileToLoad);
	}
			
	EnvironmentFilename=DefaultFileToLoad;

	UpdateFilename(EnvironmentFilename);

	//update module linking filename
	extern char ModuleFilename[MAX_PATH];
	strcpy(ModuleFilename,GetModuleLinkingFilename());
	
	
	if(Asura_Module_System::GetNumberOfModules() == 0)
	{
		ASURA_CPU_PROFILE_SCOPED_TIMER( "Tools_SaveAsura::LoadEnvironment NoModules");
		
		//Not an environment.
		Tools_AsuraFileContents xContents;

		#if defined ASURA_ASSERT
			bool bLoaded =
		#endif
			xContents.LoadAsuraFile(EnvironmentFilename.GetString());

		Asura_Assertf(bLoaded,("Error loading %s to determine contents",EnvironmentFilename.GetString()));

	
		
		if(xContents.GetSkinList()->size())
		{
			//this file had some skins
			s_pxSkin = Asura_Animation_System::GetSkin(xContents.GetSkinList()->first_entry().GetString());

		}
		if(xContents.GetAnimList()->size())
		{
			//this file had some anims
			s_pxAnim = Asura_Animation_System::GetAnim(xContents.GetAnimList()->first_entry().GetString());

		}
		if(xContents.GetFragmentList()->size())
		{
			//this file had some fragments
			s_pxFragmentType = Asura_Fragment_System::GetFragmentType(xContents.GetFragmentList()->first_entry());
		}
		else
		{
			FileType = AsuraFile_Shapes;
		}

		if(s_pxFragmentType)
		{
			if(s_pxSkin && s_pxAnim)
			{
				FileType = AsuraFile_AnimFragment;
			}
			else
			{
				FileType = AsuraFile_Fragment;
			}
		}
		else if(s_pxSkin)
		{
			FileType = AsuraFile_Skin;
		}
		else
		{
			FileType = AsuraFile_Shapes;
		}

		//Generate a fake environement for viewing the shapes
		GenerateEnvironmentForShapes();
	}

	if( (FileType != AsuraFile_Template) && (FileType != AsuraFile_TextureTiler) )
	{
		//confirm that the file type guessed from what was loaded corresponds with
		//that listed in the file (unless it was changed to provide a default environment 
		//for modes which don't proide their own environment - NDB)

		#if defined ASURA_ASSERT
			AsuraFileType eTypeFromFile =
		#endif
			Tools_AsuraFileContents::GetAsuraFileType(EnvironmentFilename.GetString());

		Asura_Assertf(FileType == eTypeFromFile, ("General confusion about exactly what type of asura file '%s' is...", EnvironmentFilename.GetString()));
	}
	
	
	//Check the file extension
	//If this isn't one of the concatenated .pc files then we need to
	//load the various ancillary files as well
	const char* extension = strrchr(EnvironmentFilename.GetString(),'.');

	//If this isn't an environment file then we shouldn't be loading any extra files.
	//In particular avoid loading the common export shapes (to avoid complications when trying to edit one of the shapes contained therein)
	//RAR 1/11/02
	//(On the other hand , its too late doing anything anout it here for anything other than the plain tools..)
	if( (FileType==AsuraFile_Environment) || 
		(FileType == AsuraFile_Template)  || 
		(FileType == AsuraFile_TextureTiler) )
	{
		if(!extension || stricmp(extension,".pc"))
		{
			Asura_Assert(This,"");
			This->LoadAncillaryFiles();
		}
	}
	
	// (AAM - 12.12.2006) Need some lighting data to see material responses. This can go when
	// we add some sort of default rig
	#if defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM
		if( FileType == AsuraFile_Shapes || FileType == AsuraFile_Skin )
		{
			ASURA_CPU_PROFILE_SCOPED_TIMER( "Tools_SaveAsura::LoadEnvironment Lighting");
			
			LoadAsuraFile( GetLightingFilename() );
			LoadAsuraFile( GetEnvironmentalLightTransportFilename() );
			LoadAsuraFile( GetGlobalIlluminationFilename() );
		}
	#endif

	if (!loadedFile)
	{
		return;
	}


	Asura_Module_Handler&	xHandler	= Asura_Module_System::GetCurrentHandler();
	xHandler.PreLoad();
	xHandler.Load();
	{
		Asura_Environment* pxEnvironment = Asura_Environment_System::GetEnvironment();

		Asura_Camera::SetPosition(pxEnvironment->GetStartPosition());
		Asura_Camera::SetOrientation(pxEnvironment->GetStartOrientation());
	}
	
	{
		ASURA_CPU_PROFILE_SCOPED_TIMER( "Tools_SaveAsura::LoadEnvironment Textures");
		Asura_Texture_Management::LoadTextures();
	}


	if(FileType!=AsuraFile_Environment)
	{
		//default to ignoring module links if not an environment
		Tools_Render_Options::SetUseModuleLinksForStandardRendering(false);
	}
}

void Tools_SaveAsura::SaveAllAs(void)
{
	ASURA_CPU_PROFILE_SCOPED_TIMER( "Tools_SaveAsura::SaveAllAs");
	
	Asura_Assert(This,"");

	Tools_FileSystem_DialogInfo info;

	info.DefaultExtension = "asr";

	info.DefaultFilename = EnvironmentFilename.GetString();

	info.DefaultPath = "Environments";
	info.FileFilter = "Asura Files\0*.asr\0";
	info.WindowTitle = "Save Asura File as...";

	Asura_Char* name = Tools_FileSystem::GetSaveFilename(info);
	
	if (!name)
	{
		return;
	}
	EnvironmentFilename = name;

	UpdateFilename(EnvironmentFilename);
	Tools_AsrFileManager::PreSaveAllAs(EnvironmentFilename.GetString());

	if (Tools_System::CurrentModeClass)
		Tools_System::CurrentModeClass->PreSave();


	SaveMainAsrFile(name);
	if(FileType == AsuraFile_Environment)
	{
		This->SaveAncillaryFiles(true);
	}



	if (Tools_System::CurrentModeClass)
		Tools_System::CurrentModeClass->PostSave();

	//Add the new environment files to the list of files that have already been loaded , to prevent
	//them from being loaded when quitting to game
	FlagEnvironmentFilesAsLoaded();
	//Also flag them to be checked for external modification. Probably not really such an issue
	//with new files really , since they are aren't going to be checked out half-way through the tools session.
	SetFilesToCheckForExternalModification();

}

void Tools_SaveAsura::SaveAll(bool bSaveAll)
{
	ASURA_CPU_PROFILE_SCOPED_TIMER( "Tools_SaveAsura::SaveAll");
	
	Asura_Assert(This,"");

	if (Tools_System::CurrentModeClass)
		Tools_System::CurrentModeClass->PreSave();

	if( FileType == AsuraFile_Template )
	{
		if(!Tools_Level_Export::IsExport())
		{
			//Haven't got a decent way to check whether the template file itself needs to be saved at the moment , 
			//so just cheat and save if anything is modified. RAR 26/06/2007
			if(bSaveAll || This->IsModified())
			{
				ASURA_CPU_PROFILE_SCOPED_TIMER( "Tools_SaveAsura::SaveAll Temp");
				SaveTemplateFile();
			}
		}
	}
	else
	{
		if(bSaveAll)
		{
			ASURA_CPU_PROFILE_SCOPED_TIMER( "Tools_SaveAsura::SaveAll Tools_AsrFileManager::SaveAll");
			Tools_AsrFileManager::SaveAll();
		}
		else
		{
			ASURA_CPU_PROFILE_SCOPED_TIMER( "Tools_SaveAsura::SaveAll Tools_AsrFileManager::SaveModified");
			Tools_AsrFileManager::SaveModified();
		}
		
		if(FileType == AsuraFile_Environment)
		{
			ASURA_CPU_PROFILE_SCOPED_TIMER( "Tools_SaveAsura::SaveAll Ancillary");
			This->SaveAncillaryFiles(bSaveAll);
		}
	}

	if(!Tools_Level_Export::IsExport())
	{
		ASURA_CPU_PROFILE_SCOPED_TIMER( "Tools_SaveAsura::SaveAll Modified");
		SaveModifiedAsuraFiles();
	}

	// Save PFX templates
	SavePFXTemps();

	//Save various global files if not doing an export.
	if(!Tools_Level_Export::IsExport())
	{
		ASURA_CPU_PROFILE_SCOPED_TIMER( "Tools_SaveAsura::SaveAll Global");
		//Save sound events
		if(bSaveAll)
			Tools_SoundEvent_System::SaveAll();
		else
			Tools_SoundEvent_System::SaveModified();

		//save dialogue files
		#ifdef ASURA_USE_DIALOGUE_SYSTEM
		if(bSaveAll)
			Tools_Dialogue_System::SaveAll();
		else
			Tools_Dialogue_System::SaveModified();
		#endif

		if(bSaveAll)
			Tools_HelperObject_System::SaveAll();
		else
			Tools_HelperObject_System::SaveModified();

		if(bSaveAll || AreBlueprintsModified())
		{
	   		SaveBlueprints();
		}

		#ifdef ASURA_USE_AXON
	    if(bSaveAll || AreAnimTemplatesModified())
		{
	    	SaveAnimTemplates();
		}
		#endif

		#ifdef ASURA_USE_STREAMING
		if(bSaveAll ||IsGlobalMetaMusicModified())
		{
			SaveGlobalMetaMusic();
		}
		#endif

		#ifdef ASURA_USE_MATERIAL_RESPONSE_SYSTEM
		Tools_Texturing_MaterialResponse::SaveChangedMaterials();
		#endif

		#ifdef ASURA_USE_GUI_MENU_SYSTEM
		if(bSaveAll)
			Tools_GUIMenu_System::SaveAll();
		else
			Tools_GUIMenu_System::SaveModified();
		#endif

#ifdef ASURA_USE_DECAL_MANAGER
		if (bSaveAll || IsDecalTemplatesModified())
		{
			SaveDecalTemplates();
		}
#endif

#ifdef ASURA_USE_CONTAINER_SYSTEM
		if (bSaveAll || Tools_Container_System::IsModified())
		{
			Tools_Container_System::SaveModified();
		}
#endif //#ifdef ASURA_USE_CONTAINER_SYSTEM

		#ifdef ASURA_USE_ANIMATION_IK_SOLVER
		if(Tools_Animation_IK_Template_System::IsModified())
		{
			Tools_Animation_IK_Template_System::SaveFile();
		}
		#endif

		#ifdef ASURA_USE_AUDIO_SYSTEM
		if(Tools_Audio_DSP::Modified())
		{
			Asura_Chunk_DSP::SaveToolsData();
		}

		if(Tools_Sound_System::IsStreamingModified())
		{
			Tools_Sound_System::SaveStreaming();
		}

		if(Tools_Sound_System::IsReverbModified())
		{
			Tools_Sound_System::SaveReverb();
		}
		#endif

		#if defined(ASURA_USE_MATERIAL_RESPONSE_SYSTEM)
		if(Tools_Overlay_MaterialResponseOverrides::IsModified())
		{
			Tools_Overlay_MaterialResponseOverrides::Save();
		}
		#endif

	}



	if (Tools_System::CurrentModeClass)
	{
		ASURA_CPU_PROFILE_SCOPED_TIMER( "Tools_SaveAsura::SaveAll Post");
		Tools_System::CurrentModeClass->PostSave();
	}
}

void Tools_SaveAsura::SaveAncillaryFiles(bool bSaveAll)
{
	ASURA_CPU_PROFILE_SCOPED_TIMER( "Tools_SaveAsura::SaveAncillaryFiles");

	// Kevin Lea - 28/09/2001 12:09:33 - module linking
    // (MP - 15.05.2003) Don't force region requirements to be recalculated
    // because we'll have done that in module linking mode's PreSave()
    // override if necessary.
	if(bSaveAll || IsModuleLinkingModified())
	{
		SaveModuleLinking( false );
	}

	// Kevin Lea - 28/09/2001 12:09:56 - lighting
	if(bSaveAll || IsLightingModified())
	{
		SaveLights();
	}
	if(bSaveAll || IsMissionLightingModified())
	{
		SaveMissionLights();
	}

	// KJFL - 2004-04-27 - environmental light transport
	if(bSaveAll || IsEnvironmentalLightTransportModified())
	{
		SaveEnvironmentalLightTransport();
	}

	// KJFL - 2004-06-09 16:59:47 - and Global Illumination too
	if(bSaveAll || IsGlobalIlluminationModified())
	{
		SaveGlobalIllumination();
	}
	if(bSaveAll || IsMissionGlobalIlluminationModified())
	{
		SaveMissionGlobalIllumination();
	}
	
	// Kevin Lea - 28/09/2001 12:19:28 - sounds
	if(bSaveAll || IsSoundsModified())
	{
		SaveSounds();
	}
	if(bSaveAll || IsMissionSoundsModified())
	{
		SaveMissionSounds();
	}


	// Kevin Lea - 03/10/2001 16:50:27 - skybox
	if(bSaveAll || IsSkyboxModified())
	{
		SaveSkybox();
	}

	//Save entities 
	if(bSaveAll || IsEntityModified())
	{
		SaveEntities();
	}

	//Save cutscenes
	if(bSaveAll || IsCutscenesModified())
	{
		SaveCutscenes();
	}
	
	/// Kevin Lea - 26/06/2002 13:07:14 - waypointing
	if(bSaveAll || IsWaypointingModified())
	{
		SaveWaypointing();
	}

#ifdef ASURA_USE_BILLBOARD_PLANTS
	// Billboard Plants.
	if(bSaveAll || IsBillboardPlantsModified())
	{
		SaveBillboardPlants();
	}
#endif 

	// DMR - Botanicals + Carpet
#ifdef ASURA_USE_BOTANICALS
	if(bSaveAll || IsBotanicalsModified())
	{
		SaveBotanicals();
	}

	if(bSaveAll || IsBotanicalCreationModified())
	{
		SaveBotanicalCreation();
	}
#endif

#ifdef ASURA_USE_CARPET
	if(bSaveAll || IsCarpetModified())
	{
		SaveCarpet();
	}
#endif

#ifdef ASURA_USE_STATIC_DECAL_SYSTEM
	if(bSaveAll || IsStaticDecalsModified())
	{
		SaveStaticDecals();
	}
#endif



	// JGP - Attractors
#ifdef ASURA_USE_ATTRACTORS
	if(bSaveAll || IsAttractorsModified())
	{
		SaveAttractors();
	}
	if(bSaveAll || IsMissionAttractorsModified())
	{
		SaveMissionAttractors();
	}
#endif


#ifdef ASURA_USE_STREAMING
	if(bSaveAll || IsStreamingConnectionsModified())
	{
		SaveStreamingConnections();
	}
#endif

#ifdef ASURA_USE_REVERBVOLUME_SYSTEM
	if(bSaveAll || IsReverbVolumesModified())
	{
		SaveReverbVolumes();
	}
#endif
}

void Tools_SaveAsura::LoadAncillaryFiles()
{
	ASURA_CPU_PROFILE_SCOPED_TIMER( "Tools_SaveAsura::LoadAncillaryFiles");

	//(module linking isn't loaded here , because it has to be awkward)

    // (MP - 22.06.2006) Initialise the shape table stuff early, in case any
    // of the below ends up loading shapes.
	Tools_ShapeTable::Initialise();

	{
		ASURA_CPU_PROFILE_SCOPED_TIMER( "Tools_SaveAsura::LoadAncillaryFiles Lighting");

		LoadAsuraFile(GetLightingFilename());
		if(IsUsingMissionLighting())
		{
			LoadAsuraFile(GetMissionLightingFilename());
		}
		
		LoadAsuraFile(GetEnvironmentalLightTransportFilename());
		if(IsUsingMissionGlobalIllumination())
		{
			LoadAsuraFile(GetMissionGlobalIlluminationFilename());
		}
		LoadAsuraFile(GetGlobalIlluminationFilename());
	}

	if( (FileType != AsuraFile_Template) && (FileType != AsuraFile_TextureTiler) )
	{
		ASURA_CPU_PROFILE_SCOPED_TIMER( "Tools_SaveAsura::LoadAncillaryFiles SoundsFile");
		
		LoadAsuraFile(GetSoundsFilename());
		if(IsEditingMission())
		{
			LoadAsuraFile(GetMissionSoundsFilename());
		}
	}
#ifdef ASURA_USE_DIALOGUE_SYSTEM
	Tools_Dialogue_System::LoadLevelSpecificFile();
#endif

	LoadAsuraFile(GetNavigationFilename());	
	LoadAsuraFile(GetSkyboxFilename());
	if( (FileType != AsuraFile_Template) && (FileType != AsuraFile_TextureTiler) )
	{
		LoadAsuraFile(GetCutscenesFilename());
		LoadAsuraFile(GetEntityFilename());
	}
	else
	{
		LoadAsuraFile(Tools_SaveAsura::GetTemplateFilename());
	}

	{
		ASURA_CPU_PROFILE_SCOPED_TIMER( "Tools_SaveAsura::LoadAncillaryFiles BuildShapeTable");
		
		// (MH - 18.09.2002) load the shape table
		Tools_ShapeTable::BuildShapeTable();
	}
	{
		ASURA_CPU_PROFILE_SCOPED_TIMER( "Tools_SaveAsura::LoadAncillaryFiles LoadExportShapes");
		
		Tools_ShapeTable::LoadExportShapes();
	}

#ifdef ASURA_USE_BILLBOARD_PLANTS
	LoadAsuraFile(GetBillboardPlantsFilename());
#endif


#ifdef ASURA_USE_BOTANICALS
	{
		ASURA_CPU_PROFILE_SCOPED_TIMER( "Tools_SaveAsura::LoadAncillaryFiles Botanicals");
		
		// Load the botanical instances here	
		LoadAsuraFile(GetBotanicalsTemplatesFilename());
		LoadAsuraFile(GetBotanicalsStylesFilename());
		LoadAsuraFile(GetBotanicalsInstancesFilename());

		// In editor mode, this loads the shapes
		Asura_Botanicals_TemplateManager::LoadResources();
	}
#endif //ASURA_USE_BOTANICALS

#ifdef ASURA_USE_ATTRACTORS
	if( (FileType != AsuraFile_Template) && (FileType != AsuraFile_TextureTiler) )
	{
		LoadAsuraFile( GetAttractorsFilename() );
		if(IsEditingMission())
		{
			LoadAsuraFile( GetMissionAttractorsFilename() );
		}
	}
#endif

#ifdef ASURA_USE_CARPET
	{
		ASURA_CPU_PROFILE_SCOPED_TIMER( "Tools_SaveAsura::LoadAncillaryFiles Carpet");
		
		LoadAsuraFile(GetCarpetStylesFilename());
		Asura_Carpet_StyleManager::LoadResources();
		LoadAsuraFile(GetCarpetMeshFilename());
	}
#endif

#ifdef ASURA_USE_STATIC_DECAL_SYSTEM
	{
		ASURA_CPU_PROFILE_SCOPED_TIMER( "Tools_SaveAsura::LoadAncillaryFiles StaticDecals");
		LoadAsuraFile(GetStaticDecalsFilename());
	}
#endif

#ifdef ASURA_USE_STREAMING
	LoadAsuraFile(GetStreamingConnectionsFilename());
#endif

#ifdef ASURA_USE_GUI_MENU_SYSTEM
	Tools_GUIMenu_System::LoadLevelSpecificFile();
#endif

#ifdef ASURA_USE_REVERBVOLUME_SYSTEM
	LoadAsuraFile(GetReverbVolumesFilename());
#endif

	{
		ASURA_CPU_PROFILE_SCOPED_TIMER( "Tools_SaveAsura::LoadAncillaryFiles LoadAllShapes");
		
		// (MH - 18.09.2002) setup the shapes
		Asura_Shape_System::LoadAllShapes();
    }
}



void Tools_SaveAsura::QuickSaveFile(void)
{
	Tools_AsrFileManager::SaveAll();
}

void Tools_SaveAsura::SetMainFileModified(bool modified /*=true*/)
{
	if( (FileType == AsuraFile_Template) || (FileType == AsuraFile_TextureTiler) )
	{
		//ignore changes to this file for template files
		return;
	}

	Tools_AsrFileManager::SetAllAsrModified();
}

void Tools_SaveAsura::SetMeshModified(const Asura_Mesh* pxMesh)
{
	if( (FileType == AsuraFile_Template) || (FileType == AsuraFile_TextureTiler) )
	{
		//ignore changes to this file for template files
		return;
	}

	Tools_AsrFileManager::SetAsrModified(pxMesh);
}

void Tools_SaveAsura::SetModuleModified(int iModuleIndex)
{
	if( (FileType == AsuraFile_Template) || (FileType == AsuraFile_TextureTiler) )
	{
		//ignore changes to this file for template files
		return;
	}

	if(iModuleIndex<0 || iModuleIndex >= Asura_Module_System::GetNumberOfModules())
	{
		Asura_Assert(0,"Invalid module index for SetModuleModified()");
		return;
	}


	const Asura_Mesh* pxMesh = Asura_Module_System::GetMeshForModule(iModuleIndex);

	Tools_AsrFileManager::SetAsrModified(pxMesh);
}

bool Tools_SaveAsura::IsMainFileModified()
{
	return Tools_AsrFileManager::IsModified();
}

void Tools_SaveAsura::SetLightingModified(bool modified /*=true*/)
{
	if(modified && !LightingModified) WarnIfReadOnly(GetLightingFilename());
	LightingModified = modified;
}

void Tools_SaveAsura::SetMissionLightingModified(bool modified /*=true*/)
{
	if(!IsUsingMissionLighting()) return;

	if(modified && !s_bMissionLightingModified) WarnIfReadOnly(GetMissionLightingFilename());
	s_bMissionLightingModified = modified;
}


void Tools_SaveAsura::SetEnvironmentalLightTransportModified( bool bModified /*=true*/ )
{
	if( (FileType == AsuraFile_Template) || (FileType == AsuraFile_TextureTiler) )
	{
		//ignore changes to this file for template files
		return;
	}

	if(bModified && !s_bEnvironmentalLightTransportModified)
	{
		WarnIfReadOnly(GetEnvironmentalLightTransportFilename());
	}

	s_bEnvironmentalLightTransportModified = bModified;
}

void Tools_SaveAsura::SetGlobalIlluminationModified( bool bModified /*=true*/ )
{

	if(bModified && !s_bGlobalIlluminationModified)
	{
		WarnIfReadOnly(GetGlobalIlluminationFilename());
	}

	s_bGlobalIlluminationModified = bModified;
}

void Tools_SaveAsura::SetMissionGlobalIlluminationModified( bool bModified /*=true*/ )
{
	if(!IsUsingMissionGlobalIllumination()) return;

	if(bModified && !s_bMissionGlobalIlluminationModified)
	{
		WarnIfReadOnly(GetMissionGlobalIlluminationFilename());
	}

	s_bMissionGlobalIlluminationModified = bModified;
}


void Tools_SaveAsura::SetModuleLinkingModified(bool modified /*=true*/)
{
	if( (FileType == AsuraFile_Template) || (FileType == AsuraFile_TextureTiler) )
	{
		//ignore changes to this file for template files
		return;
	}
	if(modified && !ModuleLinkingModified) WarnIfReadOnly(GetModuleLinkingFilename());
	ModuleLinkingModified = modified;
}

void Tools_SaveAsura::SetSkyboxModified(bool modified /*=true*/)
{
	if( (FileType != AsuraFile_Environment))
	{
		//ignore changes to this if not an environment file
		return;
	}
	if(modified && !SkyboxModified) WarnIfReadOnly(GetSkyboxFilename());
	SkyboxModified = modified;
}

void Tools_SaveAsura::SetSoundsModified(bool modified /*=true*/)
{
	
	if(modified && !SoundsModified) WarnIfReadOnly(GetSoundsFilename());
	SoundsModified = modified;
}

void Tools_SaveAsura::SetMissionSoundsModified(bool bModified /*=true*/)
{
	if(!IsEditingMission()) return;
	
	if(bModified && !s_bMissionSoundsModified) WarnIfReadOnly(GetMissionSoundsFilename());
	s_bMissionSoundsModified = bModified;
}

void Tools_SaveAsura::SetDynamicMusicModified(bool bModified /*= true*/)
{
	if(IsEditingMission())
	{
		SetMissionSoundsModified(bModified);
	}
	else
	{
		SetSoundsModified(bModified);
	}
}

void Tools_SaveAsura::SetCutscenesModified(bool modified /*=true*/)
{
	if( (FileType == AsuraFile_Template) || (FileType == AsuraFile_TextureTiler) )
	{
		//ignore changes to this file for template files
		return;
	}
	if(modified && !CutscenesModified) WarnIfReadOnly(GetCutscenesFilename());
	CutscenesModified = modified;

}



void Tools_SaveAsura::SetEntityModified(bool modified /*=true*/)
{
	if(modified && !EntityModified)
	{
		WarnIfReadOnly(GetEntityFilename());
	}
	EntityModified = modified;
}

void Tools_SaveAsura::SetEntityModified(const Asura_Entity_Server* pxEntity)
{
	Asura_Assert(pxEntity, "");

	//Flag the file that contains the entity as modified.
	if(pxEntity && pxEntity->ToolsData)
	{
		Tools_SecondaryEntityFile_System::SetFileModified(pxEntity->ToolsData->m_xSecondaryEntityFilename.GetString());
	}
	else
	{
		SetEntityModified();
	}
}


void Tools_SaveAsura::SetWaypointingModified(bool modified /*=true*/)
{
	if( (FileType == AsuraFile_Template) || (FileType == AsuraFile_TextureTiler) )
	{
		//ignore changes to this file for template files
		return;
	}
	if(modified && !WaypointingModified) WarnIfReadOnly(GetNavigationFilename());
	WaypointingModified = modified;
}

void Tools_SaveAsura::SetShapesModified(bool bModified /*=true*/)
{
	#if 0 //redundant now. RAR 26/02/2004
	if(bModified && !ShapesModified) WarnIfReadOnly(GetShapesFilename());
	ShapesModified = bModified;
	#endif
}

#ifdef ASURA_USE_BILLBOARD_PLANTS
void Tools_SaveAsura::SetBillboardPlantsModified(bool modified /*=true*/)
{
	if( (FileType == AsuraFile_Template) || (FileType == AsuraFile_TextureTiler) )
	{
		//ignore changes to this file for template files
		return;
	}
	if (modified && !BillboardPlantsModified) WarnIfReadOnly(GetBillboardPlantsFilename());
	BillboardPlantsModified = modified;
}
#endif //ASURA_USE_BILLBOARD_PLANTS

#ifdef ASURA_USE_STREAMING
void Tools_SaveAsura::SetStreamingConnectionsModified(bool bModified)
{
	if( (FileType != AsuraFile_Environment))
	{
		//File is only edited for environments
		return;
	}
	if (bModified && !s_bStreamingConnectionsModified) WarnIfReadOnly(GetStreamingConnectionsFilename());
	s_bStreamingConnectionsModified = bModified;;
}
#endif // ASURA_USE_STREAMING

#ifdef ASURA_USE_ATTRACTORS
void Tools_SaveAsura::SetAttractorsModified(bool bModified)
{
	if(FileType != AsuraFile_Environment && FileType != AsuraFile_Template)
	{
		//File is only edited for environments and templates
		return;
	}
	if (bModified && !s_bAttractorsModified) WarnIfReadOnly(GetAttractorsFilename());
	s_bAttractorsModified = bModified;;
}

void Tools_SaveAsura::SetMissionAttractorsModified(bool bModified)
{
	if(!IsEditingMission()) return;

	if (bModified && !s_bMissionAttractorsModified) WarnIfReadOnly(GetMissionAttractorsFilename());
	s_bMissionAttractorsModified = bModified;;
}

#endif // ASURA_USE_ATTRACTORS

void Tools_SaveAsura::SetBlueprintsModified(bool bModified /*=true*/)
{
    s_bBlueprintsModified = bModified;
}

#ifdef ASURA_USE_AXON
void Tools_SaveAsura::SetAnimTemplatesModified(bool bModified /*=true*/)
{
    s_bAnimTemplatesModified = bModified;
}
#endif

#ifdef ASURA_USE_STREAMING
void Tools_SaveAsura::SetGlobalMetaMusicModified(bool bModified /*=true*/)
{
	if(bModified && !s_bGlobalMetaMusicModified) WarnIfReadOnly(GetGlobalMetaMusicFilename());
	s_bGlobalMetaMusicModified = bModified;
}
#endif 

#ifdef ASURA_USE_BOTANICALS
void Tools_SaveAsura::SetBotanicalsModified(bool bModified /*=true*/)
{
	if(bModified && !s_bBotanicalsModified)
	{
		WarnIfReadOnly(GetBotanicalsTemplatesFilename());
		WarnIfReadOnly(GetBotanicalsStylesFilename());
		WarnIfReadOnly(GetBotanicalsInstancesFilename());
	}
	s_bBotanicalsModified = bModified;
}


void Tools_SaveAsura::SetBotanicalCreationModified(bool bModified /*=true*/)
{
	if(bModified && !s_bBotanicalCreationModified)
	{
		if (Tools_Mode_BotanicalCreation::GetCurrentDefinitionFilename())
		{
			Asura_String xFileName = Tools_Mode_BotanicalCreation::GetCurrentDefinitionFilename();
			xFileName+= ".btd";			
			WarnIfReadOnly(xFileName.GetString());
		}
	}
	
	s_bBotanicalCreationModified = bModified;
}
#endif

#ifdef ASURA_USE_CARPET
void Tools_SaveAsura::SetCarpetModified(bool bModified /*=true*/)
{
	if(bModified && !s_bCarpetModified)
	{
		WarnIfReadOnly(GetCarpetStylesFilename());
		WarnIfReadOnly(GetCarpetMeshFilename());
	}
	s_bCarpetModified = bModified;
}
#endif

#ifdef ASURA_USE_STATIC_DECAL_SYSTEM
void Tools_SaveAsura::SetStaticDecalsModified(bool bModified /*=true*/)
{
	if(bModified && !s_bStaticDecalsModified)
	{
		WarnIfReadOnly(GetStaticDecalsFilename());		
	}
	s_bStaticDecalsModified = bModified;
}
#endif

#ifdef ASURA_USE_DECAL_MANAGER
void Tools_SaveAsura::SetDecalTemplatesModified(bool bModified /*=true*/)
{
	if(bModified && !s_bDecalTemplatesModified)
	{
		WarnIfReadOnly(Tools_Overlay_Decals::GetDecalTemplatesFilename());		
	}
	s_bDecalTemplatesModified = bModified;
}
#endif

void Tools_SaveAsura::SetTextureTilerModified( bool bModified /*=true*/)
{
	s_bTextureTilerModified = bModified;
}

#ifdef ASURA_USE_REVERBVOLUME_SYSTEM
void Tools_SaveAsura::SetReverbVolumesModified(bool bModified)
{
	if(FileType != AsuraFile_Environment)
	{
		//File is only edited for environments
		return;
	}
	if (bModified && !s_bReverbVolumesModified) WarnIfReadOnly(GetReverbVolumesFilename());
	s_bReverbVolumesModified = bModified;;
}
#endif // ASURA_USE_REVERBVOLUME_SYSTEM


bool Tools_SaveAsura::WarnIfReadOnly(const char* filename)
{
	Asura_Assert(filename,"");

	DWORD attributes = GetFileAttributes(filename);
	if((attributes & FILE_ATTRIBUTE_READONLY) && attributes!=0xffffffff)
	{
		Tools_Timed_Message_System::AddMessageRemovingDuplicate(0xffff00,"Warning : %s is read-only",filename);
		return false;
	}
	else if(Asura_FileAccess::PlatformUsingNetworkFile(filename))
	{
		Tools_Timed_Message_System::AddMessageRemovingDuplicate(0xffff00,"Warning : %s was loaded from the network",filename);
		return false;
	}
	return true;
}

bool Tools_SaveAsura::CanSaveFile(const char* szFilename,bool bDisplayMessage)
{
	DWORD attributes = GetFileAttributes(szFilename);
	if((attributes & FILE_ATTRIBUTE_READONLY) && attributes!=0xffffffff)
	{
		if(bDisplayMessage)
		{
			Tools_Timed_Message_System::AddMessage(0xff0000,"Error : %s is read-only",Tools_FileSystem::StripPath(szFilename));
		}
		return false;
	}
	else if(Asura_FileAccess::PlatformUsingNetworkFile(szFilename))
	{
		if(bDisplayMessage)
		{
			Tools_Timed_Message_System::AddMessage(0xff0000,"Error : %s loaded from network. Saving not allowed.",Tools_FileSystem::StripPath(szFilename));
		}
		return false;
	}
	else if(Tools_LoadedFileList::IsLockedBecauseOfExternalModification(szFilename))
	{
		if(bDisplayMessage)
		{
			Tools_Timed_Message_System::AddMessage(0xff0000,"Error : %s externally modified. Saving not allowed.",Tools_FileSystem::StripPath(szFilename));
		}
		return false;
	}
	return true;	
}




char* Tools_SaveAsura::GetMainFilename()
{
	static Asura_String s_xFilename;
	Asura_String xFilename;

	xFilename = NameWithoutExtension;
	if( Tools_Level_Export::ShouldExportBigEndian() )
	{
		xFilename += "." ASURA_STANDARD_FILE_EXTENSION_BE;
	}
	else
	{
		xFilename += "." ASURA_STANDARD_FILE_EXTENSION_LE;
	}
	if (s_xFilename != xFilename)
	{
		s_xFilename = xFilename;
	}
	return s_xFilename.GetString();
}

char* Tools_SaveAsura::GetLightingFilename()
{
	static Asura_String s_xFilename;
	s_xFilename = NameWithoutExtension;
	if( Tools_Level_Export::ShouldExportBigEndian() )
	{
		s_xFilename += ".lit" ASURA_STANDARD_FILE_EXTENSION_BE_SUFFIX;
	}
	else
	{
		s_xFilename += ".lit";
	}
	return s_xFilename.GetString();
	
}

char* Tools_SaveAsura::GetMissionLightingFilename()
{
	Asura_Assert(IsEditingMission(),"");

	static Asura_String s_xFilename;
	s_xFilename = s_xMissionFilenameWithoutExtension;
	if( Tools_Level_Export::ShouldExportBigEndian() )
	{
		s_xFilename += ".lit" ASURA_STANDARD_FILE_EXTENSION_BE_SUFFIX;
	}
	else
	{
		s_xFilename += ".lit";
	}
	return s_xFilename.GetString();
	
}


const char* Tools_SaveAsura::GetEnvironmentalLightTransportFilename()
{
	static Asura_String xFileName;
	
	xFileName = NameWithoutExtension;
	
	if( Tools_Level_Export::ShouldExportBigEndian() )
	{
		xFileName += ".elt" ASURA_STANDARD_FILE_EXTENSION_BE_SUFFIX;
	}
	else
	{
		xFileName += ".elt";
	}

	return xFileName.GetString();
}

const char* Tools_SaveAsura::GetGlobalIlluminationFilename()
{
	static Asura_String xFileName;
	
	xFileName = NameWithoutExtension;

	if( Tools_Level_Export::ShouldExportBigEndian() )
	{
		xFileName += ".gi" ASURA_STANDARD_FILE_EXTENSION_BE_SUFFIX;
	}
	else
	{
		xFileName += ".gi";
	}

	return xFileName.GetString();
}

const char* Tools_SaveAsura::GetMissionGlobalIlluminationFilename()
{
	Asura_Assert(IsEditingMission(),"");

	static Asura_String xFileName;
	
	xFileName = s_xMissionFilenameWithoutExtension;

	if( Tools_Level_Export::ShouldExportBigEndian() )
	{
		xFileName += ".gi" ASURA_STANDARD_FILE_EXTENSION_BE_SUFFIX;
	}
	else
	{
		xFileName += ".gi";
	}

	return xFileName.GetString();
}



char* Tools_SaveAsura::GetModuleLinkingFilename()
{
	static Asura_String s_xFilename;
	s_xFilename = NameWithoutExtension;
	if( Tools_Level_Export::ShouldExportBigEndian() )
	{
		s_xFilename += ".mod" ASURA_STANDARD_FILE_EXTENSION_BE_SUFFIX;
	}
	else
	{
		s_xFilename += ".mod";
	}
	return s_xFilename.GetString();

}

char* Tools_SaveAsura::GetSkyboxFilename()
{
	static Asura_String s_xFilename;
	s_xFilename = NameWithoutExtension;
	if( Tools_Level_Export::ShouldExportBigEndian() )
	{
		s_xFilename += ".sky" ASURA_STANDARD_FILE_EXTENSION_BE_SUFFIX;
	}
	else
	{
		s_xFilename += ".sky";
	}
	return s_xFilename.GetString();

}

char* Tools_SaveAsura::GetSoundsFilename()
{
	static Asura_String s_xFilename;
	s_xFilename = NameWithoutExtension;
	if( Tools_Level_Export::ShouldExportBigEndian() )
	{
		s_xFilename += ".snd" ASURA_STANDARD_FILE_EXTENSION_BE_SUFFIX;
	}
	else
	{
		s_xFilename += ".snd";
	}
	return s_xFilename.GetString();
}

char* Tools_SaveAsura::GetMissionSoundsFilename()
{
	Asura_Assert(IsEditingMission(),"");

	static Asura_String s_xFilename;
	s_xFilename = s_xMissionFilenameWithoutExtension;
	if( Tools_Level_Export::ShouldExportBigEndian() )
	{
		s_xFilename += ".snd" ASURA_STANDARD_FILE_EXTENSION_BE_SUFFIX;
	}
	else
	{
		s_xFilename += ".snd";
	}
	return s_xFilename.GetString();
}


char* Tools_SaveAsura::GetCutscenesFilename()
{
	static Asura_String s_xFilename;
	if(!s_xMissionFilenameWithoutExtension.IsBlank() && !Tools_Level_Export::IsExport())
	{
		s_xFilename = s_xMissionFilenameWithoutExtension;
	}
	else
	{
		s_xFilename = NameWithoutExtension;
	}
	if( Tools_Level_Export::ShouldExportBigEndian() )
	{
		s_xFilename += ".cut" ASURA_STANDARD_FILE_EXTENSION_BE_SUFFIX;
	}
	else
	{
		s_xFilename += ".cut";
	}
	return s_xFilename.GetString();
}

char* Tools_SaveAsura::GetEntityFilename()
{
	static Asura_String s_xFilename;
	if(!s_xMissionFilenameWithoutExtension.IsBlank() && !Tools_Level_Export::IsExport())
	{
		s_xFilename = s_xMissionFilenameWithoutExtension;
	}
	else
	{
		s_xFilename = NameWithoutExtension;
	}
	if( Tools_Level_Export::ShouldExportBigEndian() )
	{
		s_xFilename += ".ent" ASURA_STANDARD_FILE_EXTENSION_BE_SUFFIX;
	}
	else
	{
		s_xFilename += ".ent";
	}
	
	return s_xFilename.GetString();
}

char* Tools_SaveAsura::GetShapesFilename()
{
	static Asura_String s_xFilename;
	s_xFilename = NameWithoutExtension;
	if( Tools_Level_Export::ShouldExportBigEndian() )
	{
		s_xFilename += ".shp" ASURA_STANDARD_FILE_EXTENSION_BE_SUFFIX;
	}
	else
	{
		s_xFilename += ".shp";
	}
	return s_xFilename.GetString();
}

char* Tools_SaveAsura::GetNavigationFilename()
{
	static Asura_String s_xFilename;
	if(IsEditingMission() && !Tools_Level_Export::IsExport())
	{
		//Missions use a nav file in the mission directoty
		//(not done during exports , since we want to save to LevelExportTemp in that case)
		s_xFilename = s_xMissionFilenameWithoutExtension;
	}
	else
	{
		s_xFilename = NameWithoutExtension;
	}

	if( Tools_Level_Export::ShouldExportBigEndian() )
	{
		s_xFilename += ".nav" ASURA_STANDARD_FILE_EXTENSION_BE_SUFFIX;
	}
	else
	{
		s_xFilename += ".nav";
	}
	return s_xFilename.GetString();
}

#ifdef ASURA_USE_BILLBOARD_PLANTS
char* Tools_SaveAsura::GetBillboardPlantsFilename()
{
	static Asura_String s_xFilename;
	s_xFilename = NameWithoutExtension;
	if( Tools_Level_Export::ShouldExportBigEndian() )
	{
		s_xFilename += ".plt" ASURA_STANDARD_FILE_EXTENSION_BE_SUFFIX;
	}
	else
	{
		s_xFilename += ".plt";
	}
	return s_xFilename.GetString();
}
#endif




#ifdef ASURA_USE_STREAMING
const char* Tools_SaveAsura::GetStreamingConnectionsFilename()
{
	static Asura_String s_xFilename;
	s_xFilename = NameWithoutExtension;

	if( Tools_Level_Export::ShouldExportBigEndian() )
	{
		s_xFilename += ".con" ASURA_STANDARD_FILE_EXTENSION_BE_SUFFIX;
	}
	else
	{
		s_xFilename += ".con";
	}

	return s_xFilename.GetString();


}
#endif // ASURA_USE_STREAMING

const char* Tools_SaveAsura::GetAllLevelsExportFilename()
{
	return "Environments\\All_Levels_Export.txt";
}

const char* Tools_SaveAsura::GetLevelSpecificExportFilename()
{
	static Asura_String s_xFilename;

	//build the filename for the export list for this level
	Asura_String xPath,xFilename;
    if( g_xRealFilename.IsBlank() )
    {
	    EnvironmentFilename.SplitFilename(&xPath,&xFilename,NULL);
    }
    else
    {
	    g_xRealFilename.SplitFilename(&xPath,&xFilename,NULL);
    }

	s_xFilename	= xPath;
	s_xFilename	+= "\\";
	s_xFilename	+= xFilename;
	s_xFilename	+= "_Export.txt";

	return s_xFilename.GetString();	

}

bool Tools_SaveAsura::CanSaveShapes()
{
	#if 0 //redundant now. RAR 26/02/2004
	//check .shp file
	if(!CanSaveFile(GetShapesFilename(),false)) return false;
	//and .stb
	Asura_String xTableName;
	Tools_ShapeTable::GetTableFilename(xTableName);
	if(!CanSaveFile(xTableName.GetString(),false)) return false;
	#endif

	return true;
}

bool Tools_SaveAsura::IsModified()
{
	if(FileType == AsuraFile_Environment)
	{
		//The ancillary files only get saved when editing an environment ,so ignore the modified flag if dealing
		//with other types of .asr files
		if(IsLightingModified() ||
		   IsEnvironmentalLightTransportModified() ||
		   IsGlobalIlluminationModified() ||
		   IsModuleLinkingModified() ||
		   IsSkyboxModified() ||
		   IsSoundsModified() ||
		   IsCutscenesModified() ||
		   IsEntityModified() ||
#ifdef ASURA_USE_BILLBOARD_PLANTS
		   IsBillboardPlantsModified() ||
#endif
#ifdef ASURA_USE_STREAMING
		   IsStreamingConnectionsModified() ||
#endif
#ifdef ASURA_USE_ATTRACTORS
		   IsAttractorsModified() ||
#endif
#ifdef ASURA_USE_CARPET
		   IsCarpetModified() ||
#endif
#ifdef ASURA_USE_BOTANICALS
		   IsBotanicalsModified() ||
		   IsBotanicalCreationModified() ||
#endif
#ifdef ASURA_USE_STATIC_DECAL_SYSTEM
		   IsStaticDecalsModified() ||
#endif
#ifdef ASURA_USE_DECAL_MANAGER
		   IsDecalTemplatesModified() ||
#endif
#ifdef ASURA_USE_REVERBVOLUME_SYSTEM
		   IsReverbVolumesModified() ||
#endif
#ifdef ASURA_USE_AXON
		   AreAnimTemplatesModified() ||
#endif
		   IsWaypointingModified()
		)
		{
			return true;
		}
		if(IsEditingMission())
		{
			if(IsMissionGlobalIlluminationModified() ||
			   IsMissionLightingModified() ||
#ifdef ASURA_USE_ATTRACTORS
			   IsMissionAttractorsModified() ||
#endif

			   IsMissionSoundsModified())
			{
				return true;
			}
		}
		
	}
	if(FileType == AsuraFile_Template)
	{
		//Some ancillary files are effectively included in template files , so check for them being modified
		if(IsLightingModified() ||
		   IsGlobalIlluminationModified() ||
		   IsSoundsModified() ||
#ifdef ASURA_USE_ATTRACTORS
			IsAttractorsModified() ||
#endif
		   IsEntityModified()
		)
		{
			return true;
		}
	}

	return	AreAsuraFilesModified() ||
		    Tools_SoundEvent_System::IsModified() ||
#ifdef ASURA_USE_DIALOGUE_SYSTEM
			Tools_Dialogue_System::IsModified() ||
#endif
#ifdef ASURA_USE_GUI_MENU_SYSTEM
			Tools_GUIMenu_System::IsModified() ||
			Tools_GUIMenu_System::IsAudioModified() ||
#endif
			Tools_HelperObject_System::IsModified() ||
			AreBlueprintsModified() ||
#ifdef ASURA_USE_MATERIAL_RESPONSE_SYSTEM
			Tools_Texturing_MaterialResponse::IsModified() ||
#endif
#ifdef ASURA_USE_STREAMING
			IsGlobalMetaMusicModified() ||
#endif
			Tools_Mode_PFX::IsPFXModified() ||
			IsTextureTilerModified() ||
#ifdef ASURA_USE_ANIMATION_IK_SOLVER
			Tools_Animation_IK_Template_System::IsModified() ||
#endif
#ifdef ASURA_USE_AUDIO_SYSTEM
			Tools_Audio_DSP::Modified() ||
			Tools_Sound_System::IsStreamingModified() ||
			Tools_Sound_System::IsReverbModified() ||
#endif
#ifdef ASURA_SUPPORT_DYNAMIC_MUSIC_SYSTEM
			Tools_Overlay_DynamicMusic::IsModified() ||
#endif
#ifdef ASURA_USE_AXON
			AreAnimTemplatesModified() ||
#endif
#if defined(ASURA_USE_MATERIAL_RESPONSE_SYSTEM)
			Tools_Overlay_MaterialResponseOverrides::IsModified() ||
#endif
			Tools_AsrFileManager::IsModified();
}

bool Tools_SaveAsura::SaveMainAsrFile(const char* szFilename)
{
	ASURA_CPU_PROFILE_SCOPED_TIMER( "Tools_SaveAsura::SaveMainAsrFile");



	if(!CanSaveFile(szFilename,true))
	{
		return false;
	}

	{
		ASURA_CPU_PROFILE_SCOPED_TIMER( "Tools_SaveAsura::OutputFile MakeBackup");

		MakeBackupIfNeeded();
	}

	//Open the file for writing
	Asura_Chunk_Stream_WindowsFile xStream;
	if(!xStream.CreateFile(szFilename, true))
	{
		Tools_Timed_Message_System::AddMessage(0xff0000,"Unable to save %s! (Write-protected file?)",szFilename);
		return false;
	}



	if(!Tools_Level_Export::IsExport())
	{
		//Use the default meshes when doing a normal save. RAR 16/03/2007
		Tools_PlatformMesh_System::PushPlatform(iTOOLS_PLATFORMMESH_DEFAULT,true);
	}
	


	{
		ASURA_CPU_PROFILE_SCOPED_TIMER( "Tools_SaveAsura::OutputFile AddChunks");
		
		Asura_Chunk_FileInfo::Create(xStream,0,FileType);
		This->AddChunksToMainFile(szFilename, xStream);

	}

	if(!Tools_Level_Export::IsExport())
	{
		//Reset to using the platform chosen by the user
		Tools_PlatformMesh_System::PopPlatform(true);
	}
	
	
	if(xStream.CloseFile())
	{
		#ifdef ASURA_WRITE_ZLIB
		Tools_Zlib::CompressFile(szFilename);
		#endif

		Tools_Timed_Message_System::AddMessage(0x00ff00,"Saved %s successfully.",Tools_FileSystem::StripPath(szFilename));
		if(!Tools_Level_Export::IsExport())
		{
			Tools_LoadedFileList::UpdateModifiedTime(szFilename);
		}
		return true;
	}
	else
	{
		Tools_Timed_Message_System::AddMessage(0xff0000,"Error writing %s.",Tools_FileSystem::StripPath(szFilename));
		return false;
	}

}

void Tools_SaveAsura::AddChunksToMainFile(const char* pszFilename, Asura_Chunk_Stream& xStream)
{
	ASURA_CPU_PROFILE_SCOPED_TIMER( "Tools_SaveAsura::AddChunksToMainFile");
	

	const char* szMeshStrippedFilename = 0;//the meshes that should be exported when dealing with multiple asr files
	if(FileType == AsuraFile_Environment && !Tools_Level_Export::IsExport() && Tools_AsrFileManager::IsUsingMultipleAsrFiles())
	{
		szMeshStrippedFilename = Tools_FileSystem::StripPath(pszFilename);
	}


	//get a list of meshes that need to be saved
	List<Asura_Mesh*> xMeshList;


	if(FileType == AsuraFile_Environment && !szMeshStrippedFilename)
	{
		ASURA_CPU_PROFILE_SCOPED_TIMER( "Tools_SaveAsura::AddChunksToMainFile Collision");
	
		//Several of the chunks were forcing the collision meshes to be invalidated and recalcuated. I'm not entirely
		//convinced that this is necessary every time a save has done , but I've moved it to a single instance here ,so
		//it will only be done once per save.
		//RAR 09/06/2006
		Asura_Module_System::InvalidateAllCollisionData();
		Asura_Module_System::UpdateInvalidCollisionMeshes();
	}
	


	if(FileType == AsuraFile_Skin || FileType == AsuraFile_AnimFragment)
	{
		ASURA_CPU_PROFILE_SCOPED_TIMER( "Tools_SaveAsura::AddChunksToMainFile Skins");
	
		//for skins , add the meshes needed by each detail level
		Asura_Assert(s_pxSkin,"");
		ListMeshesForSkin(s_pxSkin,xMeshList);
		
	}
	else
	{
		ASURA_CPU_PROFILE_SCOPED_TIMER( "Tools_SaveAsura::AddChunksToMainFile Modules");
	
		//add all 'module' meshes
		int iNumberOfMeshes = Asura_Module_System::GetNumberOfModules();

		for (int i=0; i<iNumberOfMeshes; i++)
		{
			Asura_Mesh* pxMesh = Asura_Module_System::GetMeshForModule(i);

			if(szMeshStrippedFilename && stricmp(szMeshStrippedFilename,pxMesh->m_xStrippedFilename.GetString()))
			{
				//ignore meshes that aren't supposed to be in this file
				continue;
			}
			
			if(Tools_Level_Export::IsExport())
			{
				//During exports just take the currently selected platform mesh
				xMeshList.add_entry(pxMesh);
			}
			else
			{
				//Outside of exports we need to save all the platform meshes
				for(Asura_Mesh* pxPlatformMesh = pxMesh->GetDefaultPlatformMesh(); pxPlatformMesh; pxPlatformMesh = pxPlatformMesh->m_pxNextPlatformMesh)
				{
					xMeshList.add_entry(pxPlatformMesh);
				}
			}
		}
	}

	{
		ASURA_CPU_PROFILE_SCOPED_TIMER( "Tools_SaveAsura::AddChunksToMainFile MeshAndTextures");
	
		AddMeshesAndTexturesToSaveBlock(xMeshList,xStream,pszFilename,FileType,szMeshStrippedFilename);
	}

	if(FileType == AsuraFile_Skin || FileType == AsuraFile_AnimFragment)
	{
		Asura_Assert(s_pxSkin,"");

		Asura_Chunk_Hierarchy_Skin::Create( xStream, s_pxSkin, &xMeshList, true );
		Asura_Chunk_Hierarchy_MarkerPoints::Create(s_pxSkin, s_pxSkin->Name, xStream);
		Asura_Chunk_Hierarchy_Sounds::Create(s_pxSkin, xStream);
		Asura_Chunk_Hierarchy_ManualSectionBBs::Create(s_pxSkin, xStream);
		
	}

	if(FileType == AsuraFile_Fragment || FileType == AsuraFile_AnimFragment)
	{
		Asura_Assert(s_pxFragmentType,"");

		Asura_Chunk_Fragment::Create(s_pxFragmentType,xStream);
	}

	if(FileType == AsuraFile_AnimFragment)
	{
		Asura_Assert(s_pxAnim,"");
		//save the anim
		Asura_Chunk_Hierarchy_Anim::Create(s_pxAnim, xStream);
		Asura_Chunk_Hierarchy_Sounds::Create(s_pxAnim, xStream);
	}
	
	if( FileType == AsuraFile_Environment )
	{
		ASURA_CPU_PROFILE_SCOPED_TIMER( "Tools_SaveAsura::AddChunksToMainFile Environment");
	
		Asura_Chunk_EnvironmentLBAABBTrees::Create( xStream );
	}

}

void Tools_SaveAsura::ListMeshesForSkin(const Asura_Hierarchy_Skin* pxSkin,List<Asura_Mesh*>& xMeshList)
{
	Asura_Assert(pxSkin,"");

	while(pxSkin)
	{
		if(pxSkin->IsSkinnedModel())
		{
			Asura_Mesh* pxMesh = Asura_Mesh_System::GetMesh(pxSkin->Name);
			if(pxMesh && !xMeshList.contains(pxMesh))
			{
				xMeshList.add_entry(pxMesh);
			}
		}
		else
		{
			for(int i=0;i<pxSkin->NumberOfBones;i++)
			{
				if(pxSkin->SectionData[i].Mesh  && !xMeshList.contains(pxSkin->SectionData[i].Mesh))
				{
					xMeshList.add_entry(pxSkin->SectionData[i].Mesh);
				}
			}
		}
		pxSkin = pxSkin->m_pxNextLowerSkin;

	}
}

static bool MaterialsAreEquivalent(int iMaterialID1,int iMaterialID2)
{
	Asura_Material* pxMat1 = Asura_Material_System::GetMaterial(iMaterialID1);
	Asura_Material* pxMat2 = Asura_Material_System::GetMaterial(iMaterialID2);

	return Asura_Material::MaterialsAreEquivalent(pxMat1,pxMat2,false);
}

u_int Tools_SaveAsura::PrepareMeshMaterialConversionLists( const List<Asura_Mesh*>& xMeshList, int* piToMesh, int* piFromMesh )
{

    Asura_Assert( piToMesh, "" );
    for( int i = 0; i < ASURA_MAX_NO_OF_MATERIALS; ++i ) { piToMesh[i] = ASURA_MATERIAL_NONE; }

	//Allocate from mesh array for help with removing duplicate materials , even
	//if the array isn't required by the caller. RAR 14/06/2006
	int* piFromMeshInternal = 0;
	if(!piFromMesh)
	{
		piFromMesh = piFromMeshInternal = new int[ASURA_MAX_NO_OF_MATERIALS];

	}

    if( piFromMesh )
    {
        for( int i = 0; i < ASURA_MAX_NO_OF_MATERIALS; ++i ) { piFromMesh[i] = ASURA_MATERIAL_NONE; }
    }

    u_int uNumMaterialsUsed = 0;
    for( CLIF<Asura_Mesh*> xIt(&xMeshList); !xIt.done(); xIt.next() )
    {
        const Asura_Mesh* pxMesh = xIt();
        for( int iPoly = 0; iPoly < pxMesh->NumberOfPolygons; ++iPoly )
        {
            int iMaterialID = pxMesh->TextureDataPtr[iPoly].m_iMaterialID;
            
            // ASURA_MATERIAL_NONE isn't converted.
            if( ASURA_MATERIAL_NONE == iMaterialID ) { continue; }

            int iMaterialArrayIdx = Asura_Material_System::TranslateMaterialIDToArrayIdx( iMaterialID );

            // Seen this material already?
            if( ASURA_MATERIAL_NONE != piToMesh[iMaterialArrayIdx] ) { continue; }

			//new material..but is it equivalent to an earlier one?
			u_int uMatIndex = uNumMaterialsUsed;
			if(piFromMesh)
			{
				for(uMatIndex = 0;uMatIndex<uNumMaterialsUsed;uMatIndex++)
				{
					if(MaterialsAreEquivalent(iMaterialID,piFromMesh[uMatIndex]))
					{
						//same as existing material
						break;
					}
				}
			}
            piToMesh[iMaterialArrayIdx] = uMatIndex;
			if(uMatIndex == uNumMaterialsUsed)
			{
            	if( piFromMesh ) { piFromMesh[uNumMaterialsUsed] = iMaterialID; }
            	++uNumMaterialsUsed;
			}
        }
    }

	//Check for materials used by collision meshes
	//Doing this after the normal meshes to avoid any compatibilty problems with
	//some of the other export code which assumes the materials are converted in a certain way
	//RAR 09/08/04
    for( CLIF<Asura_Mesh*> xIt(&xMeshList); !xIt.done(); xIt.next() )
    {
        const Asura_Mesh* pxMesh = xIt();
		
		for(int iColl=0;iColl<2;iColl++)
		{
			const Asura_CollisionMesh* pxCollMesh = iColl ? pxMesh->m_pxCollisionMesh : pxMesh->m_pxMergedEntityCollisionMesh;
			if(!pxCollMesh) continue;
        	for( u_int iPoly = 0; iPoly < pxCollMesh->GetNumberOfPolys(); ++iPoly )
        	{
        	    int iMaterialID = pxCollMesh->GetMaterialID(iPoly);
        	    
        	    // ASURA_MATERIAL_NONE isn't converted.
        	    if( ASURA_MATERIAL_NONE == iMaterialID ) { continue; }

                int iMaterialArrayIdx = Asura_Material_System::TranslateMaterialIDToArrayIdx( iMaterialID );

        	    // Seen this material already?
        	    if( -1 != piToMesh[iMaterialArrayIdx] ) { continue; }

				//new material..but is it equivalent to an earlier one?
				u_int uMatIndex = uNumMaterialsUsed;
				if(piFromMesh)
				{
					for(uMatIndex = 0;uMatIndex<uNumMaterialsUsed;uMatIndex++)
					{
						if(MaterialsAreEquivalent(iMaterialID,piFromMesh[uMatIndex]))
						{
							//same as existing material
							break;
						}
					}
				}
	            piToMesh[iMaterialArrayIdx] = uMatIndex;
				if(uMatIndex == uNumMaterialsUsed)
				{
	            	if( piFromMesh ) { piFromMesh[uNumMaterialsUsed] = iMaterialID; }
	            	++uNumMaterialsUsed;
				}


        	}
		}
    }

	delete [] piFromMeshInternal;
	 

    return uNumMaterialsUsed;
}


void Tools_SaveAsura::AddMeshesAndTexturesToSaveBlock(const List<Asura_Mesh*>& xMeshList,Asura_Chunk_Stream& xStream,const char* pszFileName,AsuraFileType eFileType,const char* szMeshStrippedFilename)
{
	ASURA_CPU_PROFILE_SCOPED_TIMER( "Tools_SaveAsura::AddMeshesAndTexturesToSaveBlock");
	
	#if defined (ASURA_USE_MATERIAL_RESPONSE_SYSTEM)
	//Update the old-style materials to prevent references to old textures from hanging around
	//In the future it may be better just to convert all the IDs to ASUAR_MATERIAL_NONE rather than trying
	//to keep hold of the old materials
	//RAR 30/07/2007
	//Also deal with finding out which materials are used at this point.
	//RAR 21/09/2007
	Asura_HashTree xUsedMaterialResponseTree;													
	Tools_Texturing_MaterialResponse::UpdateOldMaterialsFromMaterialResponseAndGetUsedMaterials(xMeshList, xUsedMaterialResponseTree);
	#endif


	//create a new (empty) name tree for storing the used textures	
	Asura_NameTree* pxOutputTextureTree = new Asura_NameTree;
			
	//create and initialise index conversion arrays
	int* piFromTextureIndexConversion = new int[ASURA_MAX_NO_OF_TEXTURES];
	int * TextureFlags = new int[ASURA_MAX_NO_OF_TEXTURES];
	for (int i=0; i<ASURA_MAX_NO_OF_TEXTURES; i++)
	{
		piFromTextureIndexConversion[i] = -1;	
		TextureFlags[i] = 0;
	}

   
    int* piMaterialIndexToIDConversion = new int[ASURA_MAX_NO_OF_MATERIALS];  
    int* piMaterialIDToIndexConversion = new int[ASURA_MAX_NO_OF_MATERIALS];
    int iNumberOfMaterialsUsed = PrepareMeshMaterialConversionLists( xMeshList, piMaterialIDToIndexConversion, piMaterialIndexToIDConversion );



	//allocate materials to be saved
	Asura_Material* pxMaterials = new Asura_Material[iNumberOfMaterialsUsed];

	//copy all the materials used , and add the texutres used by them to the name tree
	for(int i=0;i<iNumberOfMaterialsUsed;i++)
	{
		//get the material
		int iMaterialID = piMaterialIndexToIDConversion[i];
		pxMaterials[i] = *Asura_Material_System::GetMaterial(iMaterialID);
		
		//convert the texture indeces
		for(int j=0;j<ASURA_MATERIAL_MAXTEXTURES;j++)
		{
			int iTextureID = pxMaterials[i].m_iTextureID[j];
			if(iTextureID != ASURA_TEXTURE_NONE)
			{
				if(piFromTextureIndexConversion[iTextureID] == -1)
				{
					const Asura_String* pxTextureName = Asura_Texture_Management::GetTextureName(iTextureID);
					Asura_Assert(pxTextureName,"");
					
					int iNewIndex;
					pxOutputTextureTree->FindNodeIndexByName(pxTextureName->GetString(),true,iNewIndex);
					piFromTextureIndexConversion[iTextureID] = iNewIndex;
					TextureFlags[iNewIndex] = Asura_Texture_Management::GetTextureFlags(iTextureID);
				
				}
				pxMaterials[i].m_iTextureID[j] = piFromTextureIndexConversion[iTextureID];
			}

			//normal maps
			iTextureID = pxMaterials[i].m_iNormalMapID[j];
			if(iTextureID != ASURA_TEXTURE_NONE)
			{
				if(piFromTextureIndexConversion[iTextureID] == -1)
				{
					const Asura_String* pxTextureName = Asura_Texture_Management::GetTextureName(iTextureID);
					Asura_Assert(pxTextureName,"");
					
					int iNewIndex;
					pxOutputTextureTree->FindNodeIndexByName(pxTextureName->GetString(),true,iNewIndex);
					piFromTextureIndexConversion[iTextureID] = iNewIndex;
					TextureFlags[iNewIndex] = Asura_Texture_Management::GetTextureFlags(iTextureID);
				
				}
				pxMaterials[i].m_iNormalMapID[j] = piFromTextureIndexConversion[iTextureID];
			}

			//VGP 14Feb2006 Specular maps
			iTextureID = pxMaterials[i].m_iSpecularMapID[j];
			if(iTextureID != ASURA_TEXTURE_NONE)
			{
				if(piFromTextureIndexConversion[iTextureID] == -1)
				{
					const Asura_String* pxTextureName = Asura_Texture_Management::GetTextureName(iTextureID);
					Asura_Assert(pxTextureName,"");
					
					int iNewIndex;
					pxOutputTextureTree->FindNodeIndexByName(pxTextureName->GetString(),true,iNewIndex);
					piFromTextureIndexConversion[iTextureID] = iNewIndex;
					TextureFlags[iNewIndex] = Asura_Texture_Management::GetTextureFlags(iTextureID);
				
				}
				pxMaterials[i].m_iSpecularMapID[j] = piFromTextureIndexConversion[iTextureID];
			}
		}
	}
	
	//get a list of anims used by the materials
	List<Asura_Hash_ID> xAnimList;
	for(int i=0;i<iNumberOfMaterialsUsed;i++)
	{
		if(!xAnimList.contains(pxMaterials[i].m_uAnimHash))
		{
			xAnimList.add_entry(pxMaterials[i].m_uAnimHash);
		}
	}

	#if defined (ASURA_USE_MATERIAL_RESPONSE_SYSTEM)
	//Add the texture anims used by material responses used by the meshes
	//Also add any material response textures to the used texture list so that a texture flags
	//chunk will be saved for them.
	if(xUsedMaterialResponseTree.GetNumberOfNodes())
	{
		int iNumMaterialResponse = xUsedMaterialResponseTree.GetNumberOfNodes();

		Asura_Hash_ID* puMaterialIDs = new Asura_Hash_ID[iNumMaterialResponse];
		xUsedMaterialResponseTree.MakeHashList(puMaterialIDs);

		for(int i=0; i<iNumMaterialResponse; i++)
		{
			const Tools_MaterialResponse* const pxMaterial = Tools_MaterialResponse_System::GetMaterialByHashID(puMaterialIDs[i]);

			if(pxMaterial)
			{
				//add texture anims
				Asura_Hash_ID xAnimHash = pxMaterial->GetTextureAnim();

				if ( xAnimHash != ASURA_HASH_ID_UNSET )
				{
					if( !xAnimList.contains( xAnimHash ) )
					{
						xAnimList.add_entry( xAnimHash );
					}
				}

				//add to texture list
				for(int iTexSlot=0; iTexSlot<Asura_MaterialResponse::NUMBER_OF_TEXTURE_SLOTS; iTexSlot++)
				{
					int iTextureID = pxMaterial->GetTexture(iTexSlot);
					if(iTextureID != ASURA_TEXTURE_NONE)
					{
						if(piFromTextureIndexConversion[iTextureID] == -1)
						{
							const Asura_String* pxTextureName = Asura_Texture_Management::GetTextureName(iTextureID);
							Asura_Assert(pxTextureName,"");
							
							int iNewIndex;
							pxOutputTextureTree->FindNodeIndexByName(pxTextureName->GetString(),true,iNewIndex);
							piFromTextureIndexConversion[iTextureID] = iNewIndex;
							TextureFlags[iNewIndex] = Asura_Texture_Management::GetTextureFlags(iTextureID);
						
						}
					}
					
				}
			}
		}		

		delete [] puMaterialIDs;
	}
	#endif // defined (ASURA_USE_MATERIAL_RESPONSE_SYSTEM)

	if(Tools_Level_Export::IsExport() && !Tools_Level_Export::ShouldChunkBeExported_Static(ASURA_CHUNK_TEXTURENAMES))
	{
		//If not including the texture names chunk , then the texture indeces in the material chunk need to
		//be replaced by -1's.
		//RAR 12/04/2007
		for(int i=0;i<iNumberOfMaterialsUsed;i++)
		{
			for(int j=0;j<ASURA_MATERIAL_MAXTEXTURES;j++)
			{
				pxMaterials[i].m_iTextureID[j] = ASURA_TEXTURE_NONE;
				pxMaterials[i].m_iNormalMapID[j] = ASURA_TEXTURE_NONE;
				pxMaterials[i].m_iSpecularMapID[j] = ASURA_TEXTURE_NONE;
			}
		}

	}

	//the name tree now has all the environment textures , so save it out
    FILTER_CHUNK( ASURA_CHUNK_TEXTURENAMES, Asura_Chunk_TextureNames::Create(pxOutputTextureTree, xStream) );

	//save the texture flags
    FILTER_CHUNK( ASURA_CHUNK_TEXTUREFLAGS, Asura_Chunk_TextureFlags::Create(pxOutputTextureTree->GetNumberOfNodes(), TextureFlags, xStream) );
	delete [] TextureFlags;

	//save the materials
    FILTER_CHUNK( ASURA_CHUNK_MATERIAL, Asura_Chunk_Material::Create(iNumberOfMaterialsUsed, pxMaterials, xStream) );
    FILTER_CHUNK( ASURA_CHUNK_MATERIALNAMES, Asura_Chunk_MaterialNames::Create(iNumberOfMaterialsUsed, pxMaterials, xStream) );
	delete [] pxMaterials;

	//save texture anims
	for(LIF<Asura_Hash_ID> xAnimLIF(&xAnimList);!xAnimLIF.done();xAnimLIF.next())
	{
		const Asura_TextureAnimation* pxAnim = Asura_TextureAnimation_System::GetAnimationByHash(xAnimLIF());
		if(pxAnim)
		{
            FILTER_CHUNK( ASURA_CHUNK_TEXTUREANIMATION, Asura_Chunk_TextureAnimation::Create(xStream, pxAnim) );
		}
	}

	#if defined (ASURA_USE_MATERIAL_RESPONSE_SYSTEM)
    FILTER_CHUNK( TOOLS_CHUNK_MATERIAL_RESPONSE_NAMES, Tools_Chunk_MaterialResponseNames::Create(xUsedMaterialResponseTree, xStream) );
	#endif


	if(eFileType == AsuraFile_Environment)
	{
		ASURA_CPU_PROFILE_SCOPED_TIMER( "Tools_SaveAsura::AddMeshesAndTexturesToSaveBlock Physics");
		
        FILTER_CHUNK( ASURA_CHUNK_PHYSICSENVIRONMENT, Asura_Chunk_PhysicsEnvironment::Create(xStream,piMaterialIDToIndexConversion) );
	}

	{
		ASURA_CPU_PROFILE_SCOPED_TIMER( "Tools_SaveAsura::AddMeshesAndTexturesToSaveBlock Meshes");
		
		// Kevin Lea - 25/06/2001 17:28:05 - save environment and meshes
		for(CLIF<Asura_Mesh*> xMeshLIF(&xMeshList);!xMeshLIF.done();xMeshLIF.next())
		{
			Asura_Mesh* mesh = xMeshLIF();
	    
			FILTER_CHUNK( ASURA_CHUNK_MESH_VERTICES, Asura_Chunk_Mesh_Vertices::Create(mesh, xStream) );
			FILTER_CHUNK( ASURA_CHUNK_MESH_POLYGONS, Asura_Chunk_Mesh_Polygons::Create(mesh, xStream) );
			FILTER_CHUNK( ASURA_CHUNK_MESH_PAINTEDCOLOURS, Asura_Chunk_Mesh_PaintedColours::Create( mesh, xStream ) );
			if(!mesh->AreNormalsValid()) mesh->CreateNormals();
			FILTER_CHUNK( ASURA_CHUNK_MESH_NORMALS, Asura_Chunk_Mesh_Normals::Create(mesh, xStream) );
			FILTER_CHUNK( ASURA_CHUNK_MESH_TEXTUREINFO, Asura_Chunk_Mesh_TextureInfo::Create(mesh, xStream, piMaterialIDToIndexConversion) );
	    	
			FILTER_CHUNK( ASURA_CHUNK_MESH_SMOOTHINGGROUP, Asura_Chunk_Mesh_SmoothingGroup::Create(mesh, xStream) );
			FILTER_CHUNK( ASURA_CHUNK_MESH_POLYGONFLAGS, Asura_Chunk_Mesh_PolygonFlags::Create(mesh, xStream) );
			FILTER_CHUNK( ASURA_CHUNK_MESH_MISC, Asura_Chunk_Mesh_Misc::Create(mesh, xStream, piMaterialIDToIndexConversion) );
		}
	}

	{
		ASURA_CPU_PROFILE_SCOPED_TIMER( "Tools_SaveAsura::AddMeshesAndTexturesToSaveBlock MeshLOD");
		//add any mesh LOD data
		for(CLIF<Asura_Mesh*> xMeshLIF(&xMeshList);!xMeshLIF.done();xMeshLIF.next())
		{
			const Asura_Mesh* pxMesh = xMeshLIF();
			if(pxMesh->IsDetailMesh())
			{
				FILTER_CHUNK( ASURA_CHUNK_MESH_LOD, Asura_Chunk_Mesh_LOD::Create(pxMesh, xStream) );
			}
		}
	}

	// Save the module data.
	if(eFileType == AsuraFile_Environment)
	{
		ASURA_CPU_PROFILE_SCOPED_TIMER( "Tools_SaveAsura::AddMeshesAndTexturesToSaveBlock Module List");
		const Asura_Environment* pxEnvironment = Asura_Environment_System::GetEnvironment();	
    	
		FILTER_CHUNK( ASURA_CHUNK_ENVIRONMENT_MODULELIST,
                      Asura_Chunk_Environment_ModuleList::Create(pxEnvironment, xStream, piMaterialIDToIndexConversion, szMeshStrippedFilename) );
    
		//Start positions get saved to asura file if we have yet to be able to move
		//them to the entity file.
		if(Asura_Chunk_StartPosition::s_eStartPositionFile == Asura_Chunk_StartPosition::SPF_ASURA)
		{
			//save off the start positions , in reverse order so that the loaders know how large to make the array
			for(int j=pxEnvironment->NumberOfStartPositions-1;j>=0;j--)
			{
				FILTER_CHUNK( ASURA_CHUNK_STARTPOSITION, Asura_Chunk_StartPosition::Create(&pxEnvironment->StartPositions[j], j, xStream) );
			}
		}
	}
	
    if( !Tools_Level_Export::IsExport() && eFileType == AsuraFile_Shapes )
	{
		ASURA_CPU_PROFILE_SCOPED_TIMER( "Tools_SaveAsura::AddMeshesAndTexturesToSaveBlock ShapeData");
		
        FILTER_CHUNK( ASURA_CHUNK_SHAPEDATA,
                      Asura_Chunk_ShapeData::Create(Tools_Mode_ShapeData::GetFlags(), Tools_Mode_ShapeData::GetBB(), Tools_Mode_ShapeData::GetHashID(), xStream) );
	}


	delete [] piMaterialIndexToIDConversion;
	delete [] piMaterialIDToIndexConversion;
	delete [] piFromTextureIndexConversion;
    	
	if(eFileType == AsuraFile_Environment && pszFileName)
	{
		ASURA_CPU_PROFILE_SCOPED_TIMER( "Tools_SaveAsura::AddMeshesAndTexturesToSaveBlock UsedTextureLog");

		int n = pxOutputTextureTree->GetNumberOfNodes();
		const Asura_String** nameList = new const Asura_String*[n];
		pxOutputTextureTree->MakeNameList(nameList);
    
		//build the filename for the texture log by appending _UsedTexturesLog.txt onto the
		//name of the asura file
		char filename[MAX_PATH];
		strcpy(filename,pszFileName);
		char* dotpos = strrchr(filename,'.');
		Asura_Assert(dotpos,"");
		strcpy(dotpos,"_UsedTexturesLog.txt");
		
		FILE* file = fopen(filename,"w");

		if (file)
		{
			fprintf(file,"Environment Textures used in %s:\n\n",pszFileName);

			int t = 1;
			for(int i=0; i<n; i++)
			{
    			
				fprintf(file,"<%d> <%s>\n",t,nameList[i]->GetString());
				t++;
			}

			fclose(file);
		}
    	
		delete [] nameList;
	}

	delete pxOutputTextureTree;
}

extern bool SaveLights(const char* filename,bool bMissionFile);
void Tools_SaveAsura::SaveLights()
{
	if( (FileType == AsuraFile_Template) || (FileType == AsuraFile_TextureTiler) )
	{
		//ignore changes to this file for template files
		return;
	}


	MakeBackupIfNeeded();
	
	
	
	const char* szFilename = GetLightingFilename();
	if(!CanSaveFile(szFilename,true))return;

	if(::SaveLights(szFilename,false))
	{
		Tools_Timed_Message_System::AddMessage(0x00ff00,"Saved Lighting successfully.");
		Tools_LoadedFileList::UpdateModifiedTime(szFilename);

		//reset the modified flag
		LightingModified = false;
	}
	else
	{
		Tools_Timed_Message_System::AddMessage(0xff0000,"Failed to save Lighting. (Write-protected file?)");
	}
}

void Tools_SaveAsura::SaveMissionLights()
{
	if(!IsUsingMissionLighting()) return;
	if(Tools_Level_Export::IsExport()) return;

	MakeBackupIfNeeded();
	
	
	
	const char* szFilename = GetMissionLightingFilename();
	if(!CanSaveFile(szFilename,true))return;

	if(::SaveLights(szFilename,true))
	{
		Tools_Timed_Message_System::AddMessage(0x00ff00,"Saved mission .lit successfully.");
		if(!Tools_Level_Export::IsExport())
		{
			Tools_LoadedFileList::UpdateModifiedTime(szFilename);
	
			//reset the modified flag
			s_bMissionLightingModified = false;
		}
	}
	else
	{
		Tools_Timed_Message_System::AddMessage(0xff0000,"Failed to save mission .lit. (Write-protected file?)");
	}
}


void Tools_SaveAsura::SaveEnvironmentalLightTransport()
{
	if( (FileType == AsuraFile_Template) || (FileType == AsuraFile_TextureTiler) )
	{
		//ignore changes to this file for template files
		return;
	}

	MakeBackupIfNeeded();


	const char* szFilename = GetEnvironmentalLightTransportFilename();

	if ( !CanSaveFile(szFilename,true) ) return;

	if(!Tools_Level_Export::IsExport())
	{
		//Use the default meshes when doing a normal save. RAR 16/03/2007
		Tools_PlatformMesh_System::PushPlatform(iTOOLS_PLATFORMMESH_DEFAULT,true);
	}


	if ( Asura_Chunk_EnvironmentalLightTransport::SaveFile(szFilename) )
	{
		Tools_Timed_Message_System::AddMessage(0x00ff00,"Saved .ELT successfully.");
		if(!Tools_Level_Export::IsExport())
		{
			Tools_LoadedFileList::UpdateModifiedTime(szFilename);
			//reset the modified flag
			s_bEnvironmentalLightTransportModified = false;
		}
	}
	else
	{
		Tools_Timed_Message_System::AddMessage(0xff0000,"Failed to save .ELT (Write-protected file?)");
	}

	if(!Tools_Level_Export::IsExport())
	{
		//Reset to using the platform chosen by the user
		Tools_PlatformMesh_System::PopPlatform(true);
	}

}

void Tools_SaveAsura::SaveGlobalIllumination()
{
	if( (FileType == AsuraFile_Template) || (FileType == AsuraFile_TextureTiler) )
	{
		//ignore changes to this file for template files
		return;
	}

	MakeBackupIfNeeded();


	const char* szFilename = GetGlobalIlluminationFilename();

	if ( !CanSaveFile(szFilename,true) ) return;

	if(!Tools_Level_Export::IsExport())
	{
		//Use the default meshes when doing a normal save. RAR 16/03/2007
		Tools_PlatformMesh_System::PushPlatform(iTOOLS_PLATFORMMESH_DEFAULT,true);
	}


	if ( Asura_Chunk_GlobalIllumination::SaveFile(szFilename,false) )
	{
		Tools_Timed_Message_System::AddMessage(0x00ff00,"Saved .GI successfully.");
		if(!Tools_Level_Export::IsExport())
		{
			Tools_LoadedFileList::UpdateModifiedTime(szFilename);
			//reset the modified flag
			s_bGlobalIlluminationModified = false;
		}
	}
	else
	{
		Tools_Timed_Message_System::AddMessage(0xff0000,"Failed to save .GI (Write-protected file?)");
	}

	if(!Tools_Level_Export::IsExport())
	{
		//Reset to using the platform chosen by the user
		Tools_PlatformMesh_System::PopPlatform(true);
	}
}

void Tools_SaveAsura::SaveMissionGlobalIllumination()
{
	if(!IsUsingMissionGlobalIllumination()) return;
	if(Tools_Level_Export::IsExport()) return;

	MakeBackupIfNeeded();


	const char* szFilename = GetMissionGlobalIlluminationFilename();

	if ( !CanSaveFile(szFilename,true) ) return;


	if ( Asura_Chunk_GlobalIllumination::SaveFile(szFilename,true) )
	{
		Tools_Timed_Message_System::AddMessage(0x00ff00,"Saved mission .GI successfully.");
		if(!Tools_Level_Export::IsExport())
		{
			Tools_LoadedFileList::UpdateModifiedTime(szFilename);
		
			//reset the modified flag
			s_bMissionGlobalIlluminationModified = false;
		}
	}
	else
	{
		Tools_Timed_Message_System::AddMessage(0xff0000,"Failed to save mission .GI (Write-protected file?)");
	}
}




void Tools_SaveAsura::SaveModuleLinking( bool bForceRegionRequirementUpdate /*=true*/ )
{
	if( (FileType == AsuraFile_Template) || (FileType == AsuraFile_TextureTiler) )
	{
		//ignore changes to this file for template files
		return;
	}
	MakeBackupIfNeeded();

	if(!Tools_Level_Export::IsExport())
	{
		//Use the default meshes when doing a normal save. RAR 16/03/2007
		Tools_PlatformMesh_System::PushPlatform(iTOOLS_PLATFORMMESH_DEFAULT,true);
	}
	
	const char* szFilename = GetModuleLinkingFilename();
	if( CanSaveFile(szFilename, true) )
    {
		Asura_SaveBlock xSaveBlock;

	    Asura_Module_System::Save( &xSaveBlock );

#if defined( ASURA_USE_NEW_VISIBILITY_SYSTEM )
		// add visibility chunks to the .mod file here
		Asura_Chunk_SaveBlock_Stream xStream( &xSaveBlock );

		Asura_Chunk_Visibility::Create( xStream );
#endif
		xSaveBlock.AddAsuraTerminator();

		if ( xSaveBlock.SaveFile(szFilename,true) )
	    {
#ifdef ASURA_USE_NEW_VISIBILITY_SYSTEM
            Tools_Timed_Message_System::AddMessage( 0x00FF00, "Saved visibility data successfully (%s)", Tools_FileSystem::StripPath(szFilename) );
#else
		    Tools_Timed_Message_System::AddMessage(0x00ff00,"Saved Module Linking successfully.");
#endif

			if(!Tools_Level_Export::IsExport())
			{
				Tools_LoadedFileList::UpdateModifiedTime(szFilename);
		    	//reset the modified flag
			    ModuleLinkingModified = false;
			}
	    }
	    else
	    {
#ifdef ASURA_USE_NEW_VISIBILITY_SYSTEM
            Tools_Timed_Message_System::AddMessage( 0x00FF00, "Failed to save visibility data (%s)", Tools_FileSystem::StripPath(szFilename) );
#else
		    Tools_Timed_Message_System::AddMessage(0xff0000,"Failed to Save Module Linking.");
#endif
	    }
    }

	if(!Tools_Level_Export::IsExport())
	{
		//Reset to using the platform chosen by the user
		Tools_PlatformMesh_System::PopPlatform(true);
	}


    if( bForceRegionRequirementUpdate )
    {
        Tools_Mode_Regions::DealWithModifiedModuleLinks();
    }
}

extern bool SaveSkybox(const char* filename);

void Tools_SaveAsura::SaveSkybox()
{
	Asura_Assert(This,"");
	if(FileType != AsuraFile_Environment)
	{
		//ignore changes to this if not an environment file
		return;
	}

	MakeBackupIfNeeded();

	const char* szFilename = GetSkyboxFilename();
	if(!CanSaveFile(szFilename,true))return;

	Asura_SaveBlock xSaveBlock;
	This->AddChunksToSkyboxFile(&xSaveBlock);
	xSaveBlock.AddAsuraTerminator();

	if(xSaveBlock.SaveFile(szFilename,true))
	{
		Tools_Timed_Message_System::AddMessage(0x00ff00,"Saved Skybox successfully.");
		if(!Tools_Level_Export::IsExport())
		{
			Tools_LoadedFileList::UpdateModifiedTime(szFilename);
			//reset the modified flag
			SkyboxModified = false;
		}
	}
	else
	{
		Tools_Timed_Message_System::AddMessage(0xff0000,"Failed to save Skybox. (Write-protected file?)");
	}
}

void Tools_SaveAsura::AddChunksToSkyboxFile(Asura_SaveBlock* pxSaveBlock)
{
	//get list of texture names , and flags for textures
	const int iMaxTextures = 10;//ought to be more than enough
	Asura_NameTree xTextureTree(iMaxTextures);
	int aiTextureFlags[iMaxTextures];
	int iNumTextures = 0;

	//deal with skybox textures
	for(int i=0;i<Asura_Skybox::ASURA_SKYBOX_NUM_TEXTURES;i++)
	{
		Asura_String xTextureName = Asura_Skybox::GetTexture(i);
		if(xTextureName.IsBlank()) continue;

		int iOldIndex = Asura_Texture_Management::GetTextureIndex(xTextureName);
	   	if(iOldIndex != ASURA_TEXTURE_NONE)
		{
			Asura_Assert(iNumTextures<iMaxTextures,"");

			int iNewIndex;
		   	if(!xTextureTree.FindNodeIndexByName(xTextureName.GetString(),true,iNewIndex))
			{
		   		aiTextureFlags[iNewIndex] = Asura_Texture_Management::GetTextureFlags(iOldIndex);
				iNumTextures++;
			}
		}
	}
	//and the reflectance texture
	{
		Asura_String xTextureName = Asura_Render::GetReflectanceMapName();

		if(!xTextureName.IsBlank())
		{
			int iOldIndex = Asura_Texture_Management::GetTextureIndex(xTextureName);
	   		if(iOldIndex != ASURA_TEXTURE_NONE)
			{
				Asura_Assert(iNumTextures<iMaxTextures,"");

				int iNewIndex;
		   		if(!xTextureTree.FindNodeIndexByName(xTextureName.GetString(),true,iNewIndex))
				{
		   			aiTextureFlags[iNewIndex] = Asura_Texture_Management::GetTextureFlags(iOldIndex);
					iNumTextures++;
				}
			}
		}
	}
	

	//create texture names and texture flag chunks
	Asura_Chunk_TextureNames::Create(&xTextureTree,pxSaveBlock);
	Asura_Chunk_TextureFlags::Create(xTextureTree.GetNumberOfNodes(),aiTextureFlags,pxSaveBlock);
	
	
	Asura_Chunk_RenderSettings::Create(pxSaveBlock);

	Asura_Chunk_SaveBlock_Stream	xChunkStream (pxSaveBlock);	// ART - Move to top of file when we sort out "Asura_Chunk_RenderSettings"...
	Tools_Chunk_SkyBox::Create(xChunkStream);
		
	#ifdef ASURA_SUPPORT_WEATHER_SYSTEM
	/// Richard May - 18/12/2002 12:45:03 - sky file now also includes weather data
	/// Kevin Lea - 18/12/2002 18:30:56 - but only if ASURA_SUPPORT_WEATHER_SYSTEM is defined ;)
	Asura_Chunk_WeatherSystem::Create(pxSaveBlock);
    Asura_Chunk_Fog::Create( pxSaveBlock );
	#endif
	
	#ifdef ASURA_USE_WATER
	Asura_Chunk_WaterControl::Create(pxSaveBlock);
	#endif 

	#ifdef ASURA_USE_PSP_BLUR
	Asura_Chunk_PSP_Blur::Create(pxSaveBlock);
	#endif

	#ifdef ASURA_USE_PSP_FILTER
	Asura_Chunk_PSP_Filter::Create(pxSaveBlock);
	#endif

}


void Tools_SaveAsura::SaveSounds()
{
	if( (FileType == AsuraFile_Template) || (FileType == AsuraFile_TextureTiler) )
	{
		//ignore changes to this file for template files
		return;
	}

	MakeBackupIfNeeded();

	const char* szFilename = GetSoundsFilename();
	if(!CanSaveFile(szFilename,true))return;
	
	Asura_SaveBlock xSaveBlock;

	
	if(!Tools_Level_Export::IsExport())
	{
		//Use the default meshes when doing a normal save. RAR 16/03/2007
		Tools_PlatformMesh_System::PushPlatform(iTOOLS_PLATFORMMESH_DEFAULT,true);
	}
        
    
	//create all the chunks
	Asura_Chunk_SoundEvent_Phonon::Create(&xSaveBlock,false);
	
	//save reverb settings
	Asura_Chunk_ModuleReverb::Create(&xSaveBlock);
	//save streaming sounds
	Asura_Chunk_StreamingBackgroundSound::Create(&xSaveBlock);
	//save meta music tracks
	#ifndef ASURA_SUPPORT_DYNAMIC_MUSIC_SYSTEM
	Asura_Chunk_MetaMusic::Create(&xSaveBlock);
	#endif
	#ifdef ASURA_SUPPORT_DYNAMIC_MUSIC_SYSTEM
	//Dynamic music is saved out to the mission specific sound file  if doing mission files.
	//(Except during exports , in which case the 'main sound file' is used to store everything for the export
	//RAR 23/01/2009
	if(Tools_Level_Export::IsExport() || !IsEditingMission())
	{
		//save dynamic music tracks
		Asura_Chunk_DynamicMusic::Create(&xSaveBlock);
	}
	#endif

	Asura_Chunk_Entity_NextGuid::Create(TOOLS_GUIDRANGE_SOUND, xSaveBlock);
	
	//save sound controller entities
	//(during level exports these will be bundled with the entity file instead)
	if(!Tools_Level_Export::IsExport())
	{
		for(Asura_Classified_List_It<Asura_Entity_Server> xIt(Asura_Entity_Server::GetList());!xIt.Done();xIt.Next())
		{
			Asura_Entity_Server* pxEntity = xIt.GetCurrent();
			//ignore entities marked for destruction
			if(pxEntity->IsDestroyed()) continue;
			if(pxEntity->GetGuid()<ASURA_GUID_TOOLS_SOUNDFILE_START || pxEntity->GetGuid()>ASURA_GUID_TOOLS_SOUNDFILE_END) continue;

			
			//save this entity
			Asura_Chunk_Entity::Create(pxEntity,xSaveBlock);
			//and its tools data
			Asura_Chunk_Entity_ToolsData::Create(pxEntity, xSaveBlock);
		}
	}

	
	xSaveBlock.AddAsuraTerminator();

	if(!Tools_Level_Export::IsExport())
	{
		//Reset to using the platform chosen by the user
		Tools_PlatformMesh_System::PopPlatform(true);
	}


	if(xSaveBlock.SaveFile(szFilename,true))
	{
		Tools_Timed_Message_System::AddMessage(0x00ff00,"Saved Sounds successfully.");

		if(!Tools_Level_Export::IsExport())
		{
			Tools_LoadedFileList::UpdateModifiedTime(szFilename);

			//reset the modified flag
			SoundsModified = false;
		}
	}
	else
	{
		Tools_Timed_Message_System::AddMessage(0xff0000,"Failed to save Sounds. (Write-protected file?)");
	}

	//also save out changes to PS2 sound settings
	if(!Tools_Level_Export::IsExport())
	{
		if(Tools_Sound_PS2Settings::IsModified() && !Tools_Sound_PS2Settings::Save())
		{
			Tools_Timed_Message_System::AddMessage(0xff0000,"Failed to save %s. (Write-protected file?)",PS2SettingsFilename);
		}
	}
}

void Tools_SaveAsura::SaveMissionSounds()
{
	if(!IsEditingMission()) return;
	if(Tools_Level_Export::IsExport()) return;

	MakeBackupIfNeeded();

	const char* szFilename = GetMissionSoundsFilename();
	if(!CanSaveFile(szFilename,true))return;
	
	Asura_SaveBlock xSaveBlock;
	
	Asura_Chunk_SoundEvent_Phonon::Create(&xSaveBlock,true);

	#ifdef ASURA_SUPPORT_DYNAMIC_MUSIC_SYSTEM
	//save dynamic music tracks
	Asura_Chunk_DynamicMusic::Create(&xSaveBlock);
	#endif

	Asura_Chunk_Entity_NextGuid::Create(TOOLS_GUIDRANGE_MISSIONSOUND, xSaveBlock);

	//save sound controller entities
	//(during level exports these will be bundled with the entity file instead)
	if(!Tools_Level_Export::IsExport())
	{
		for(Asura_Classified_List_It<Asura_Entity_Server> xIt(Asura_Entity_Server::GetList());!xIt.Done();xIt.Next())
		{
			Asura_Entity_Server* pxEntity = xIt.GetCurrent();
			//ignore entities marked for destruction
			if(pxEntity->IsDestroyed()) continue;
			if(pxEntity->GetGuid()<ASURA_GUID_TOOLS_MISSIONSOUNDFILE_START || pxEntity->GetGuid()>ASURA_GUID_TOOLS_MISSIONSOUNDFILE_END) continue;
			
			//save this entity
			Asura_Chunk_Entity::Create(pxEntity,xSaveBlock);
			//and its tools data
			Asura_Chunk_Entity_ToolsData::Create(pxEntity, xSaveBlock);
		}
	}

	xSaveBlock.AddAsuraTerminator();



	if(xSaveBlock.SaveFile(szFilename,true))
	{
		Tools_Timed_Message_System::AddMessage(0x00ff00,"Saved mission sounds successfully.");
		if(!Tools_Level_Export::IsExport())
		{
			Tools_LoadedFileList::UpdateModifiedTime(szFilename);
			//reset the modified flag
			s_bMissionSoundsModified = false;
		}
	}
	else
	{
		Tools_Timed_Message_System::AddMessage(0xff0000,"Failed to save mission sounds. (Write-protected file?)");
	}

	//also save out changes to PS2 sound settings
	if(Tools_Sound_PS2Settings::IsModified() && !Tools_Sound_PS2Settings::Save())
	{
		Tools_Timed_Message_System::AddMessage(0xff0000,"Failed to save %s. (Write-protected file?)",PS2SettingsFilename);
	}
}


extern bool SaveCutscenes(const char* filename);
void Tools_SaveAsura::SaveCutscenes()
{
	if( (FileType == AsuraFile_Template) || (FileType == AsuraFile_TextureTiler) )
	{
		//ignore changes to this file for template files
		return;
	}
	MakeBackupIfNeeded();
	
	const char* szFilename = GetCutscenesFilename();
	if(!CanSaveFile(szFilename,true))return;

	if(Asura_Chunk_Cutscene::s_bCutscenesLoadedFromEntityFile)
	{
		//transfering cutscenes from .ent to .cut file.
		//must be able to save both files at the same time
		//RAR 28/03/03
		szFilename = GetEntityFilename();
		if(!CanSaveFile(szFilename,true))
		{
			Message_Overlay xOverlay("Aborting saving of .cut file , because you don't have the .ent file checked out");
			xOverlay.DoModal();
			return;
			
		}
		Asura_Chunk_Cutscene::s_bCutscenesLoadedFromEntityFile = false;
		SaveEntities();
	}

	if(Tools_Mode_Cutscene::CurrentCutscene)
	{

		//make sure the edited cutscene is up to date in the asura data
		Tools_Mode_Cutscene::CurrentCutscene->UpdateTracks();

		//update any matching points between cutscenes
		Tools_Mode_Cutscene::s_pxThis->UpdateLinkedCutscenes();
	}
	bool bSuccess = true;
    
	//save cutscenes file
    if(::SaveCutscenes(GetCutscenesFilename()))
	{
		Tools_Timed_Message_System::AddMessage(0x00ff00,"Saved Cutscenes successfully.");
		if(!Tools_Level_Export::IsExport())
		{
			Tools_LoadedFileList::UpdateModifiedTime(GetCutscenesFilename());
		}
	}
	else
	{
		Tools_Timed_Message_System::AddMessage(0xff0000,"Failed to save Cutscenes. (Write-protected file?)");
		bSuccess = false;
	}

	if(bSuccess)
	{
		if(!Tools_Level_Export::IsExport())
		{
			//reset the modified flag
			CutscenesModified = false;
		}
	}
}

void Tools_SaveAsura::SaveEntities()
{
	if( (FileType == AsuraFile_Template) || (FileType == AsuraFile_TextureTiler) )
	{
		//ignore changes to this file for template files
		return;
	}

	Tools_Entity_Simulation::End();

	MakeBackupIfNeeded();

	if(!Tools_Level_Export::IsLevelExport())
	{
		//save secondary entity files
		Tools_SecondaryEntityFile_System::SaveFiles();
	}
	
	const char* szFilename = GetEntityFilename();
	if(!CanSaveFile(szFilename,true))return;
	bool bSuccess = true;

	if(Asura_Chunk_Cutscene::s_bCutscenesLoadedFromEntityFile)
	{
		//transfering cutscenes from .ent to .cut file.
		//must be able to save both files at the same time
		//RAR 28/03/03
		szFilename = GetCutscenesFilename();
		if(!CanSaveFile(szFilename,true))
		{
			Message_Overlay xOverlay("Aborting saving of .ent file , because you don't have the .cut file checked out");
			xOverlay.DoModal();
			return;
			
		}
		Asura_Chunk_Cutscene::s_bCutscenesLoadedFromEntityFile = false;
		SaveCutscenes();
	}

    Asura_SaveBlock xSaveBlock;
	This->AddChunksToEntityFile(&xSaveBlock);
	xSaveBlock.AddAsuraTerminator();
    
    if(xSaveBlock.SaveFile(szFilename,true))
	{
		Tools_Timed_Message_System::AddMessage(0x00ff00,"Saved Entities successfully.");
		if(!Tools_Level_Export::IsExport())
		{
			Tools_LoadedFileList::UpdateModifiedTime(szFilename);
		}
	}
	else
	{
		Tools_Timed_Message_System::AddMessage(0xff0000,"Failed to save Entities. (Write-protected file?)");
		bSuccess = false;
	}

	if(bSuccess)
	{
		if(!Tools_Level_Export::IsExport())
		{
			//reset the modified flag
			EntityModified = false;
		}
		
	}
	
}

void Tools_SaveAsura::SaveTemplateFile()
{
	if(FileType != AsuraFile_Template)
	{
		Asura_Assert(FileType == AsuraFile_Template,"Not editing a template file.");
		return;
	}
	
	MakeBackupIfNeeded();
	
	const char* szFilename = s_xTemplateFilename.GetString();
	if(!CanSaveFile(szFilename,true))return;

    
    SaveTemplateFile(0,szFilename);
}

bool Tools_SaveAsura::SaveTemplateFile(Asura_ServerEntity_Template* pxTemplateEntity,const char* szFilename)
{
	if(!CanSaveFile(szFilename,true))return false;

	//When saving a template file normally , pxTemplateEntity will be null , and all sounds and entities etc should be saved.
	//However , when exporting entities from a level as a template file , it will be necessary to save a subset of these.
	//RAR 07/06/2006
	Asura_Assert(pxTemplateEntity || FileType == AsuraFile_Template,"Expecting a specific template entity to save , if not editing a template file");

    Asura_SaveBlock xSaveBlock;
	Asura_Chunk_FileInfo::Create(&xSaveBlock,0,AsuraFile_Template);

	SetTemplateBeingExported(pxTemplateEntity);

	//add chunks for sound file
    // build tree of used sounds
	if(!pxTemplateEntity)
	{
		Asura_Chunk_SoundEvent_Phonon::Create(&xSaveBlock,false);
		Asura_Chunk_StreamingBackgroundSound::Create(&xSaveBlock);
	}
		
	//add the entity chunks
	This->AddChunksToEntityFile(&xSaveBlock);


	if(!pxTemplateEntity)
	{
		//add light chunks
		Asura_Chunk_GlobalIllumination::Create(xSaveBlock,true,false);
	

		//add corona chunks
		//save textures used by coronas
		Asura_NameTree xTextureTree;
		Asura_StaticCorona_System::BuildTextureTree(&xTextureTree);
		if(xTextureTree.GetNumberOfNodes())
		{
			Asura_Chunk_TextureNames::Create(&xTextureTree,&xSaveBlock);
		}

		//and save the coronas themselves
		Asura_Chunk_Corona::Create(&xSaveBlock,true,false);
	}

	#ifdef ASURA_USE_ATTRACTORS
	//add attractor chunk
	Asura_Chunk_Tools_Attractors::Create(&xSaveBlock,false);
	#endif

	
	xSaveBlock.AddAsuraTerminator();

	SetTemplateBeingExported(0);
    
    bool bOkay = xSaveBlock.SaveFile(szFilename,true);

    if(bOkay)
	{
		Tools_Timed_Message_System::AddMessage(0x00ff00,"Saved template file successfully.");
		if(!Tools_Level_Export::IsExport())
		{
			Tools_LoadedFileList::UpdateModifiedTime(szFilename);
			//reset the modified flags
			SetLightingModified(false);
			SetGlobalIlluminationModified(false);
			SetSoundsModified(false);
			SetEntityModified(false);
			#ifdef ASURA_USE_ATTRACTORS
			SetAttractorsModified(false);
			#endif
		}

	}
	else
	{
		Tools_Timed_Message_System::AddMessage(0xff0000,"Failed to save %s. (Write-protected file?)",szFilename);
	}
	return bOkay;
}

void Tools_SaveAsura::SavePFXTemps()
{
	Tools_Mode_PFX::SaveAllTemplates();
}

#ifndef ASURA_USE_AUDIO_SYSTEM
static void AddEntitySoundToTree(int iSoundID,void* pTree)
{
	// This function is for sound name chunk creation and not relevant for the new audio system
	
	Asura_NameTree* pxNameTree = (Asura_NameTree*) pTree;
	Asura_Assert(pxNameTree,"");

	if(iSoundID>=ASURA_SID_ENTITY_MIN && iSoundID<=ASURA_SID_ENTITY_MAX)
	{
		const char* pszName = Asura_Sound_System::GetFileNameFromSoundID((Asura_Sound_ID)iSoundID);
		if(pszName)
		{
			int iIndex;
			// add name to tree
			pxNameTree->FindNodeIndexByName(pszName, true, iIndex);
		}
		else
		{
			Asura_Assertf(pszName,("Failed to find filename for entity sound %d",iSoundID));
		}

	}
}
#endif	// #ifndef ASURA_USE_AUDIO_SYSTEM

void Tools_SaveAsura::AddChunksToEntityFile(class Asura_SaveBlock* pxSaveBlock)
{
	AsuraFileType eFileType = GetFileType();
	//exporting a template file?
	const Asura_ServerEntity_Template* pxTemplateEntity = GetTemplateBeingExported();
	if(pxTemplateEntity)
	{
		eFileType = AsuraFile_Template;
	}


	if(eFileType == AsuraFile_Environment)
	{
		if(!Tools_Level_Export::IsLevelExport())
		{
			//add details of secondary entity files
			Tools_SecondaryEntityFile_System::AddChunksToFile(pxSaveBlock);
		}
		
		//save the start positions , as long as they aren't being saved to the asura file
		if(Asura_Chunk_StartPosition::s_eStartPositionFile != Asura_Chunk_StartPosition::SPF_ASURA)
		{
			Asura_Environment* pxEnv = Asura_Environment_System::GetEnvironment();	
			//save off the start positions , in reverse order so that the loaders know how large to make the array
			for(int j=pxEnv->NumberOfStartPositions-1;j>=0;j--)
			{
				Asura_Chunk_StartPosition::Create(&pxEnv->StartPositions[j],j,pxSaveBlock);
			}
		}
	}

	
	//save the static messages
	Asura_Chunk_StaticMessages::Create(*pxSaveBlock,ASURA_STATICMESSAGE_ENTITYSET);

	Asura_NameTree xTextureTree;
	BuildEntityTextureTree(&xTextureTree,ASURA_GUID_TOOLS_ENTITYFILE_START,ASURA_GUID_TOOLS_ENTITYFILE_END);

	if(xTextureTree.GetNumberOfNodes())
	{
		Asura_Chunk_TextureNames::Create(&xTextureTree,pxSaveBlock);
	}


    Asura_NameTree xEntitySoundTree;
	
	Asura_Chunk_Entity_NextGuid::Create(TOOLS_GUIDRANGE_ENTITY, *pxSaveBlock);

	//RAR 08/03/2006 Get entity offsets for level exports
	//RAR 07/06/2006 Also used for exporting entities to template files

	Asura_Vector_3 xPositionOffset = Asura_Maths::ZeroVector3;
	if(pxTemplateEntity)
	{
		xPositionOffset = pxTemplateEntity->GetPosition();
	}
	else
	{
		xPositionOffset = Tools_Level_Export::GetExportCentre();
	}

	Tools_EntityTemplateInstance_ApplyOffset xSubtractOffset(-xPositionOffset);
	Tools_EntityTemplateInstance_ApplyOffset xAddOffset(xPositionOffset);
	bool bApplyOffset = (xPositionOffset != Asura_Maths::ZeroVector3);


	//and the entities
	for(Asura_Classified_List_It<Asura_Entity_Server> it(Asura_Entity_Server::GetList());!it.Done();it.Next())
	{
		Asura_Entity_Server* pxEntity = it.GetCurrent();
		//ignore entities marked for destruction
		if(pxEntity->IsDestroyed()) continue;
		
		//check that the guid is one that belongs with this file
		bool bAcceptGuid = false;
		Asura_Guid uGuid = pxEntity->GetGuid();	
	
		bAcceptGuid = (uGuid>=ASURA_GUID_TOOLS_ENTITYFILE_START && uGuid<=ASURA_GUID_TOOLS_ENTITYFILE_END);
		if(Tools_Level_Export::IsExport())
		{
			//also include the sound file entities here during exports
			bAcceptGuid |= (uGuid>=ASURA_GUID_TOOLS_SOUNDFILE_START && uGuid<=ASURA_GUID_TOOLS_SOUNDFILE_END);
			bAcceptGuid |= (uGuid>=ASURA_GUID_TOOLS_MISSIONSOUNDFILE_START && uGuid<=ASURA_GUID_TOOLS_MISSIONSOUNDFILE_END);
		}
		if(!bAcceptGuid) continue;

		if(pxTemplateEntity)
		{
			//ignore entities that aren't part of the template being exported.
			if(pxEntity != pxTemplateEntity && !pxTemplateEntity->EntityBelongsToTemplate(pxEntity->GetGuid()))
			{
				continue;
			}
		}

		//recentre entity if required
		if(bApplyOffset)
		{
			if(pxEntity->ToolsData)
			{
				pxEntity->ToolsData->Position -= xPositionOffset;
			}
			pxEntity->ApplyChangesForTemplateInstance(&xSubtractOffset);
		}
		
		//save this entity
		Asura_Chunk_Entity::Create((Asura_Entity_Server*)pxEntity,*pxSaveBlock);

		//and its tools data
		Asura_Chunk_Entity_ToolsData::Create((Asura_Entity_Server*)pxEntity,*pxSaveBlock,pxTemplateEntity!=0);

		//undo changes to position
		if(bApplyOffset)
		{
			if(pxEntity->ToolsData)
			{
				pxEntity->ToolsData->Position += xPositionOffset;
			}
			pxEntity->ApplyChangesForTemplateInstance(&xAddOffset);
		}

		// We dont care about sound names with the new audio system
		#ifndef ASURA_USE_AUDIO_SYSTEM
		//add sounds used by this pxEntity to the list
		pxEntity->TraverseUsedSounds(AddEntitySoundToTree,&xEntitySoundTree);
		#endif

	}

	#ifndef ASURA_USE_AUDIO_SYSTEM
    //save out sounds used by entities
    Asura_Chunk_SoundNames::Create(&xEntitySoundTree,ASURA_SID_ENTITY_MIN,ASURA_SID_ENTITY_MAX,pxSaveBlock);
	#endif
}


bool Tools_SaveAsura::AddChunksToWaypointingFile( Asura_SaveBlock* pxSaveBlock )
{
    Asura_Chunk_SaveBlock_Stream xStream( pxSaveBlock );

    if ( !Asura_Chunk_Navigation::Create( xStream ) ) { return false; }
    if ( !Asura_Chunk_WaypointingSettings::Create( xStream ) ) { return false; }
    return true;
}


void Tools_SaveAsura::SaveShapes()
{
	//obsolete. RAR 23/03/2005
}

void Tools_SaveAsura::SaveWaypointing()
{
	if ( (FileType == AsuraFile_Template) || (FileType == AsuraFile_TextureTiler) )
	{
		//ignore changes to this file for template files
		return;
	}

	ASURA_CPU_PROFILE_SCOPED_TIMER( "Tools_SaveAsura::SaveWaypointing");

	MakeBackupIfNeeded();

	const char* szFilename = GetNavigationFilename();
	if ( !CanSaveFile( szFilename, true ) ) return;
	
	Asura_SaveBlock xSaveBlock;
    if ( !Static_AddChunksToWaypointingFile( &xSaveBlock ) ) { return; }
    if ( !xSaveBlock.AddAsuraTerminator() ) { return; }

    if ( xSaveBlock.SaveFile( szFilename, true ) )
	{
		Tools_Timed_Message_System::AddMessage(0x00ff00,"Saved Waypointing successfully.");
		if(!Tools_Level_Export::IsExport())
		{
			Tools_LoadedFileList::UpdateModifiedTime(szFilename);
			WaypointingModified = false;
		}
	}
	else
	{
		Tools_Timed_Message_System::AddMessage(0xff0000,"Failed to save Waypointing. (Write-protected file?)");
	}
}

#ifdef ASURA_USE_BILLBOARD_PLANTS
void Tools_SaveAsura::SaveBillboardPlants()
{
	if( (FileType == AsuraFile_Template) || (FileType == AsuraFile_TextureTiler) )
	{
		//ignore changes to this file for template files
		return;
	}

	MakeBackupIfNeeded();
	const char* szFilename = GetBillboardPlantsFilename();
	if (!CanSaveFile(szFilename, true)) return;
	if (Asura_Chunk_BillboardPlant_Batches::Create(szFilename))
	{
		Tools_Timed_Message_System::AddMessage(0x00ff00,"Saved billboard plant batches successfully.");
		if(!Tools_Level_Export::IsExport())
		{
			Tools_LoadedFileList::UpdateModifiedTime(szFilename);
			BillboardPlantsModified = false;
		}
	}
	else
	{
		Tools_Timed_Message_System::AddMessage(0xff0000,"Failed to save billboard plant batches. (Write-protected file?)");
	}
	// TO BE DONE - FINISHED 
}
#endif //ASURA_USE_BILLBOARD_PLANTS

#ifdef ASURA_USE_BOTANICALS
void Tools_SaveAsura::SaveBotanicals(void)
{
	bool bAnyFailed = false;

	// Save the templates	
	if (Asura_Botanicals_TemplateManager::Save(GetBotanicalsTemplatesFilename(),GetBotanicalsStylesFilename()))
	{	
		Tools_Timed_Message_System::AddMessage(0x00ff00,"Saved botanical templates successfully.");
	}
	else
	{
		Tools_Timed_Message_System::AddMessage(0xff0000, "Failed to save botanical templates (Write-protected file?)");
		bAnyFailed = true;
	}

	if (Asura_Chunk_Botanicals_Instances::Create(GetBotanicalsInstancesFilename()))
	{	
		Tools_Timed_Message_System::AddMessage(0x00ff00,"Saved botanicals successfully.");		
	}
	else
	{
		Tools_Timed_Message_System::AddMessage(0xff0000, "Failed to save botanicals (Write-protected file?)");
		bAnyFailed = true;
	}	

	if(!bAnyFailed)
	{
		s_bBotanicalsModified = false;
	}
}


void Tools_SaveAsura::SaveBotanicalCreation()
{
	bool bFailed = false;

	if(!Tools_Level_Export::IsExport())
	{
		if (!Tools_Mode_BotanicalCreation::Save())
		{	
			// The chunk save function contains it's own error message, and that's a better one - it gives the filename
			// Tools_Timed_Message_System::AddMessage(0xff0000, "Failed to save botanical definition (Write-protected file?)");
			bFailed = true;
		}
	}

	if (!bFailed)
	{
		if(!Tools_Level_Export::IsExport())
		{
			s_bBotanicalCreationModified = false;
		}
	}
}
#endif //ASURA_USE_BOTANICALS

#ifdef ASURA_USE_CARPET
void Tools_SaveAsura::SaveCarpet(void)
{
	bool bSavedStyle = false;
	bool bSavedMesh = false;
	
	// Save the templates	
	if(CanSaveFile(GetCarpetStylesFilename(),true))
	{
		if (Asura_Carpet_StyleManager::Save(GetCarpetStylesFilename()))
		{	
			Tools_Timed_Message_System::AddMessage(0x00ff00,"Saved carpet styles successfully.");
			bSavedStyle = true;
		}
		else
		{
			Tools_Timed_Message_System::AddMessage(0xff0000, "Failed to save carpet styles (Write-protected file?)");
		}
	}

	if(CanSaveFile(GetCarpetMeshFilename(),true))
	{
		if (Asura_Chunk_Carpet_Mesh::Create(GetCarpetMeshFilename()))
		{	
			Tools_Timed_Message_System::AddMessage(0x00ff00,"Saved carpet mesh successfully.");		
			bSavedMesh = true;
		}
		else
		{
			Tools_Timed_Message_System::AddMessage(0xff0000, "Failed to save carpet mesh (Write-protected file?)");
		}	
	}

	//Currently only got the one modified state tracking both files , so only reset if both files could
	//be saved.
	//RAR 26/06/2007
	if(bSavedStyle && bSavedMesh)
	{
		if(!Tools_Level_Export::IsExport())
		{
			s_bCarpetModified = false;
		}
	}

}
#endif //ASURA_USE_CARPET


#ifdef ASURA_USE_STATIC_DECAL_SYSTEM
void Tools_SaveAsura::SaveStaticDecals(void)
{
	bool bSaved = false;

	if(CanSaveFile(GetStaticDecalsFilename(),true))
	{
		if (Tools_Chunk_StaticDecals::Create(GetStaticDecalsFilename()))
		{	
			Tools_Timed_Message_System::AddMessage(0x00ff00,"Saved static decals successfully.");
			bSaved = true;
		}
		else
		{
			Tools_Timed_Message_System::AddMessage(0xff0000, "Failed to save static decals (Write-protected file?)");
		}
	}

	if (bSaved)
	{
		if(!Tools_Level_Export::IsExport())
		{
			s_bStaticDecalsModified = false;
		}
	}
}
#endif

#ifdef ASURA_USE_DECAL_MANAGER
void Tools_SaveAsura::SaveDecalTemplates(void)
{
	bool bSaved = false;

	if(CanSaveFile(Tools_Overlay_Decals::GetDecalTemplatesFilename(),true))
	{
		if (Tools_Overlay_Decals::SaveDecalTemplates())
		{	
			Tools_Timed_Message_System::AddMessage(0x00ff00,"Saved decal templates successfully.");
			bSaved = true;
		}
		else
		{
			Tools_Timed_Message_System::AddMessage(0xff0000, "Failed to save decal templates (Write-protected file?)");
		}
	}

	if (bSaved)
	{
		if(!Tools_Level_Export::IsExport())
		{
			s_bDecalTemplatesModified = false;
		}
	}
}
#endif


#ifdef ASURA_USE_ATTRACTORS

void Tools_SaveAsura::SaveAttractors()
{
	if(FileType != AsuraFile_Environment) return;
	MakeBackupIfNeeded();

	const char* szFilename = GetAttractorsFilename();
	if(!CanSaveFile(szFilename,true))return;


	Asura_SaveBlock xSaveBlock;
	Asura_Chunk_Tools_Attractors::Create(&xSaveBlock,false);
	xSaveBlock.AddAsuraTerminator();
		

    if(xSaveBlock.SaveFile(szFilename,false))
	{	
		Tools_Timed_Message_System::AddMessage(0x00ff00,"Saved env hooks successfully.");		
		if(!Tools_Level_Export::IsExport())
		{
			Tools_LoadedFileList::UpdateModifiedTime(szFilename);
			s_bAttractorsModified = false;
		}
	}
	else
	{
		Tools_Timed_Message_System::AddMessage(0xff0000, "Failed to save env hooks (Write-protected file?)");
	}	
}

void Tools_SaveAsura::SaveMissionAttractors()
{
	if(!IsEditingMission()) return;
	if(Tools_Level_Export::IsExport()) return;

	MakeBackupIfNeeded();

	const char* szFilename = GetMissionAttractorsFilename();
	if(!CanSaveFile(szFilename,true))return;


	Asura_SaveBlock xSaveBlock;
	Asura_Chunk_Tools_Attractors::Create(&xSaveBlock,true);
	xSaveBlock.AddAsuraTerminator();
		

    if(xSaveBlock.SaveFile(szFilename,false))
	{	
		Tools_Timed_Message_System::AddMessage(0x00ff00,"Saved mission env hooks successfully.");		
		if(!Tools_Level_Export::IsExport())
		{
			Tools_LoadedFileList::UpdateModifiedTime(szFilename);
			s_bMissionAttractorsModified = false;
		}
	}
	else
	{
		Tools_Timed_Message_System::AddMessage(0xff0000, "Failed to save mission env hooks (Write-protected file?)");
	}	
}


#endif //ASURA_USE_ATTRACTORS

#ifdef ASURA_USE_STREAMING
void Tools_SaveAsura::SaveStreamingConnections()
{
	if(FileType != AsuraFile_Environment) return;
	MakeBackupIfNeeded();

	const char* szFilename = GetStreamingConnectionsFilename();
	if(!CanSaveFile(szFilename,true))return;

    Asura_SaveBlock xSaveBlock;
	
	//add the one chunk that belongs to this file
	const Asura_Streaming_Zone* pxZone = Tools_Streaming_System::GetStreamingZoneForThisLevel();
	Asura_Chunk_StreamingConnections::Create(pxZone,&xSaveBlock);

	xSaveBlock.AddAsuraTerminator();
    
    if(xSaveBlock.SaveFile(szFilename,true))
	{
		Tools_Timed_Message_System::AddMessage(0x00ff00,"Saved streaming connections file successfully.");
		if(!Tools_Level_Export::IsExport())
		{
			Tools_LoadedFileList::UpdateModifiedTime(szFilename);
			//reset the modified flag
			s_bStreamingConnectionsModified = false;
		}

	}
	else
	{
		Tools_Timed_Message_System::AddMessage(0xff0000,"Failed to save streaming connections. (Write-protected file?)");
	}
}
#endif //ASURA_USE_STREAMING

void Tools_SaveAsura::SaveBlueprints()
{
	if( ( FileType == AsuraFile_Template ) || (FileType == AsuraFile_TextureTiler) )
	{
		//ignore changes to this file for template files
		return;
	}
    Tools_Mode_Blueprint::SaveAll();
}

#ifdef ASURA_USE_AXON
void Tools_SaveAsura::SaveAnimTemplates()
{
	if( ( FileType == AsuraFile_Template ) || (FileType == AsuraFile_TextureTiler) )
	{
		//ignore changes to this file for template files
		return;
	}
	Tools_AnimTemplates::SaveAll();
}
#endif

#ifdef ASURA_USE_STREAMING
void Tools_SaveAsura::SaveGlobalMetaMusic()
{
	#ifndef ASURA_SUPPORT_DYNAMIC_MUSIC_SYSTEM
	const char* szFilename = GetGlobalMetaMusicFilename();
	if(!CanSaveFile(szFilename,true)) return;

	Asura_MetaMusic_Handler* pxHandler = Tools_Mode_MetaMusic::GetGlobalHandler();

	//If there are no tracks , and the file isn't modified , then don't bother trying to create the file
	//This should prevent lots of empty files from being created as a result of using SaveAll()
	if(!pxHandler->GetNumberOfTracks() && !s_bGlobalMetaMusicModified) return;

	//add the common meta music tracks to a save block
	Asura_SaveBlock xSaveBlock;
	Asura_Chunk_MetaMusic::Create(&xSaveBlock,pxHandler);
	xSaveBlock.AddAsuraTerminator();

	if(xSaveBlock.SaveFile(szFilename,true))
	{
		Tools_Timed_Message_System::AddMessage(0x00ff00,"Saved global meta music successfully.");
		if(!Tools_Level_Export::IsExport())
		{
			Tools_LoadedFileList::UpdateModifiedTime(szFilename);
			//reset the modified flag
			s_bGlobalMetaMusicModified = false;
		}
	}
	else
	{
		Tools_Timed_Message_System::AddMessage(0xff0000,"Failed to save global meta music. (Write-protected file?)");
	}
	#endif
}
#endif //ASURA_USE_STREAMING

#ifdef ASURA_USE_REVERBVOLUME_SYSTEM

void Tools_SaveAsura::SaveReverbVolumes()
{
	if(FileType != AsuraFile_Environment) return;
	MakeBackupIfNeeded();

	const char* szFilename = GetReverbVolumesFilename();
	if(!CanSaveFile(szFilename,true))return;


	Asura_SaveBlock xSaveBlock;
	Asura_Chunk_ReverbVolumes::Create(&xSaveBlock);
	xSaveBlock.AddAsuraTerminator();
		

    if(xSaveBlock.SaveFile(szFilename,false))
	{	
		Tools_Timed_Message_System::AddMessage(0x00ff00,"Saved reverb volumes successfully.");		
		if(!Tools_Level_Export::IsExport())
		{
			Tools_LoadedFileList::UpdateModifiedTime(szFilename);
			s_bReverbVolumesModified = false;
		}
	}
	else
	{
		Tools_Timed_Message_System::AddMessage(0xff0000, "Failed to save reverb volumes (Write-protected file?)");
	}	
}

#endif //ASURA_USE_REVERBVOLUME_SYSTEM




void Tools_SaveAsura::SetAnimationModified(Asura_Hierarchy_Anim* pxAnim,const char* pszFilename)
{
	SetAsuraFileModified(pszFilename);
}



void Tools_SaveAsura::AddChunksToAnimFile(Asura_Hierarchy_Anim* pxAnim,class Asura_SaveBlock* pxSaveBlock)
{
	Asura_Chunk_Hierarchy_Anim::Create(pxAnim,pxSaveBlock);
	Asura_Chunk_Hierarchy_Sounds::Create(pxAnim, pxSaveBlock);

	#ifdef ASURA_USE_FACIAL_ANIMATION_SYSTEM
	if(pxAnim->HasFacialAnimation())
	{
		//Search for a matching face anim and save that as well
		Asura_Hash_ID uHash = pxAnim->GetHashID();

		const Tools_FacialAnimation_FaceAnimation* pxToolsAnim = Tools_FacialAnimation_System::FindAnimation(uHash);
		const Asura_FacialAnimation_FaceAnimation* pxAsuraAnim = Asura_FacialAnimation_System::FindFaceAnimation(uHash);
		Asura_Assertf(pxToolsAnim && pxAsuraAnim, ("Failed to find the face animation to save with animation %s",pxAnim->Name.GetString()));
		
		if(pxToolsAnim && pxAsuraAnim)
		{
			Asura_Chunk_SaveBlock_Stream	xStream(pxSaveBlock);

			Tools_Chunk_FacialAnimation_FaceAnimation::Create(uHash, *pxToolsAnim, xStream);
			Asura_Chunk_FacialAnimation_FaceAnimation::Create(uHash, *pxAsuraAnim, xStream);
		}		
	}
	#endif
}

bool Tools_SaveAsura::FindAnimationRecursive(const char* directory,const char* animName,Asura_String& outputFileName)
{
	char filename[MAX_PATH];
	char search[MAX_PATH];
	int dirLength = strlen(directory);

	//does the file exist in this directory?
	// (MH - 18.10.2002) try not to end up with / at the beginning of filename.
    // This will make the paths relative to z:\ or whatever instead of the working directory.
	if(!(directory[0] == '\\' && directory[1] == '\\'))//RAR 01\09\03  Allow \\ at beginning , to deal with network paths
	{
    	Asura_Assert(directory[0] != '\\' && directory[0] != '/', "directory not relative to working dir");
	}
    Asura_Assert(animName[0] != '\\' && animName[0] != '/', "animName contains '/' or '\\'");
    if (dirLength)
    {
        sprintf(filename,"%s\\%s.asr",directory,animName);
    }
    else
    {
        sprintf(filename, "%s.asr", animName);
    }
	if(GetFileAttributes(filename)!=0xffffffff)
	{
		outputFileName = filename;
		return true;
	}

	//search subdirectories
    // (MH - 18.10.2002) try not to end up with / at the beginning of filename.
	if (dirLength)
    {
        sprintf(search,"%s\\*.*",directory);
    }
    else
    {
        strcpy(search, "*.*");
    }
	WIN32_FIND_DATA wfd;
	HANDLE hFindFile = ::FindFirstFile(search,&wfd);

	if(INVALID_HANDLE_VALUE != hFindFile)
	{
		do
		{	
			//only interested in directories
			if(!(wfd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)) continue;
			//ignore the '.' and '..' directories
			if(wfd.cFileName[0]=='.') continue;
			
			// (MH - 18.10.2002) try not to end up with / at the beginning of filename.
	        if (dirLength)
            {
                Asura_Assert(filename[dirLength] == '\\', "MH: expecting '\\'");
                strcpy(&filename[dirLength+1],wfd.cFileName);
            }
            else
            {
                strcpy(filename,wfd.cFileName);
            }
			if(FindAnimationRecursive(filename,animName,outputFileName))
			{
				::FindClose(hFindFile);
				return true;
			}

			
		}
		while(::FindNextFile(hFindFile,&wfd));
		::FindClose(hFindFile);
	}

	return false;
}

void Tools_SaveAsura::SetSkinModified(Asura_Hierarchy_Skin* pxSkin,const char* pszFilename)
{
	SetAsuraFileModified(pszFilename);
}

void Tools_SaveAsura::AddChunksToSkinFile(Asura_Hierarchy_Skin* pxSkin,Asura_SaveBlock* pxSaveBlock)
{
	//get a list of meshes needed for this skin
	List<Asura_Mesh*> xMeshList;
	ListMeshesForSkin(pxSkin,xMeshList);

	Asura_Chunk_SaveBlock_Stream xStream(pxSaveBlock);
	AddMeshesAndTexturesToSaveBlock(xMeshList,xStream,0,AsuraFile_Skin,NULL);

	FILTER_CHUNK( ASURA_CHUNK_HIERARCHY_SKIN, Asura_Chunk_Hierarchy_Skin::Create(pxSaveBlock, pxSkin, &xMeshList, true) );
    FILTER_CHUNK( ASURA_CHUNK_HIERARCHY_MARKERPOINTS, Asura_Chunk_Hierarchy_MarkerPoints::Create(pxSkin, pxSkin->Name, pxSaveBlock) );
	FILTER_CHUNK( ASURA_CHUNK_HIERARCHY_SOUNDS, Asura_Chunk_Hierarchy_Sounds::Create(pxSkin, pxSaveBlock) );
	FILTER_CHUNK( ASURA_CHUNK_HIERARCHY_MANUALSECTIONBBS, Asura_Chunk_Hierarchy_ManualSectionBBs::Create(pxSkin, pxSaveBlock) );
}

void Tools_SaveAsura::SetAsuraFileModified(const char* pszFilename,bool bModified)
{
	//file already in modified list?
	for(LIF<Asura_String> xModLIF(&s_xModifiedAsuraFileList);!xModLIF.done();xModLIF.next())
	{
		if(!stricmp(pszFilename,xModLIF().GetString()))
		{
			if(!bModified)
			{
				//remove from the list
				xModLIF.delete_current();
			}
			return;
		}
	}
	if(bModified)
	{
		//add an entry to the modified list
		s_xModifiedAsuraFileList.add_entry(pszFilename);

		WarnIfReadOnly(pszFilename);
	}
}

void Tools_SaveAsura::ClearModifiedAsuraFileList()
{
	while(s_xModifiedAsuraFileList.size())
	{
		s_xModifiedAsuraFileList.delete_first_entry();
	}
}

bool Tools_SaveAsura::IsAsuraFileModified(const char* szFilename)
{
	const char* szRelativeFilename = Asura_FileAccess::PlatformRemoveNetworkOrExecutableDir(szFilename);

	for(LIF<Asura_String> xModLIF(&s_xModifiedAsuraFileList);!xModLIF.done();xModLIF.next())
	{
		if(!stricmp(szRelativeFilename,xModLIF().GetString()))
		{
			return true;
		}
	}
	return false;
}

void Tools_SaveAsura::SaveModifiedAsuraFiles()
{
	Asura_Assert(This,"");
    	
	//We only ever want to save these files as little endian , so make sure the platform type is set to PC.
	//RAR 18/04/2006
	enum Tools_Level_Export::EXPORT_TYPE eOriginalExportType = Tools_Level_Export::s_eExportType;
	Tools_Level_Export::s_eExportType = Tools_Level_Export::Export_PC;
	bool bWasExport = Tools_Level_Export::IsExport();
	Tools_Level_Export::SetIsExport(false);
	
	
	for(LIF<Asura_String> xModLIF(&s_xModifiedAsuraFileList);!xModLIF.done();)
	{
		const char* pszFilename = xModLIF().GetString();		
		if(Tools_SaveAsura::CanSaveFile(pszFilename,true))
		{
			if(This->SaveAsuraFile(xModLIF().GetString()))
			{
				Tools_Timed_Message_System::AddMessage(0x00ff00,"Saved %s.",pszFilename);
				xModLIF.delete_current();						
			}
			else
			{
				Tools_Timed_Message_System::AddMessage(0xff0000,"Failed to save %s.",pszFilename);
				xModLIF.next();						
			}
		}
		else
		{
			xModLIF.next();						
		}
	}

	//Restore current platform. RAR 18/04/2006
	Tools_Level_Export::s_eExportType = eOriginalExportType;
	Tools_Level_Export::SetIsExport(bWasExport);
}


bool Tools_SaveAsura::SaveAsuraFile(const char* pszFilename)
{
	if(Tools_Level_Export::IsExport())
	{
		Asura_Assert(0,"Not expecting this to be called during an export.");
		return false;
	}
	
	if(!Tools_SaveAsura::CanSaveFile(pszFilename,true))
	{
		return false;
	}


	//need to determine what type of file this is
	Tools_AsuraFileContents xContents;
	if(!xContents.LoadAsuraFile(pszFilename))
	{
		Asura_Assertf(false,("Failed to determine the file type (skin/anim etc) for %s",pszFilename));
		return false;
	}

	//Is this one of the file types we can deal with?
	AsuraFileType eFileType =  xContents.GetFileType();
	if(	eFileType != AsuraFile_Shapes && 
		eFileType != AsuraFile_Anim &&
		eFileType != AsuraFile_Skin &&
		eFileType != AsuraFile_Fragment &&
		eFileType != AsuraFile_AnimFragment)
	{
		Asura_Assertf(0,("Unexpected asura file type being saved by SaveAsuraFile.\nFilename %s",pszFilename));
	}

	//add chunks for everything in the file
	Asura_SaveBlock xSaveBlock;

	//file info
	Asura_Chunk_FileInfo::Create(&xSaveBlock,0,eFileType);

	//skins
	for(CLIF<Asura_String> xSkinLIF(xContents.GetSkinList());!xSkinLIF.done();xSkinLIF.next())
	{
		//find the skin
		Asura_Hierarchy_Skin* pxSkin = Asura_Animation_System::GetSkin(xSkinLIF().GetString());
		if(!pxSkin)
		{
			Tools_Timed_Message_System::AddMessage(0xff0000,"Failed to find skin to save to %s.",pszFilename);
			return false;
		}

		AddChunksToSkinFile(pxSkin,&xSaveBlock);

	}

	//anims
	for(CLIF<Asura_String> xAnimLIF(xContents.GetAnimList());!xAnimLIF.done();xAnimLIF.next())
	{
		//find the anim
		Asura_Hierarchy_Anim* pxAnim = Asura_Animation_System::GetAnim(xAnimLIF().GetString());
		if(!pxAnim)
		{
			Tools_Timed_Message_System::AddMessage(0xff0000,"Failed to find anim to save to %s.",pszFilename);
			return false;
		}

		AddChunksToAnimFile(pxAnim,&xSaveBlock);
	}

	//fragments
	for(CLIF<Asura_Hash_ID> xFragmentLIF(xContents.GetFragmentList());!xFragmentLIF.done();xFragmentLIF.next())
	{
		//find the skin
		Asura_FragmentType* pxFragment = Asura_Fragment_System::GetFragmentType(xFragmentLIF());
		if(!pxFragment)
		{
			Tools_Timed_Message_System::AddMessage(0xff0000,"Failed to find fragment type to save to %s.",pszFilename);
			return false;
		}

		if(!pxFragment->m_bAnimatedFragment)
		{
			//only animated fragments dealt with at the moment
			Asura_Assert(0,"Can't save non-animated fragment files with this function");
			return false;
		}
		Asura_Chunk_Fragment::Create(pxFragment,&xSaveBlock);
	}
	xSaveBlock.AddAsuraTerminator();

	if(xSaveBlock.SaveFile(pszFilename,true))
	{
		Tools_LoadedFileList::UpdateModifiedTime(pszFilename);

		FILETIME xModifiedTime;
		if(Tools_FileSystem::GetModifiedTime(pszFilename,&xModifiedTime))
		{
			//update modified times stored in anims. RAR 24/05/2006
			for(CLIF<Asura_String> xAnimLIF(xContents.GetAnimList());!xAnimLIF.done();xAnimLIF.next())
			{
				//find the anim
				Asura_Hierarchy_Anim* pxAnim = Asura_Animation_System::GetAnim(xAnimLIF().GetString());
				if(!pxAnim) continue;
				pxAnim->m_xModifiedTime = xModifiedTime;
			}
		}

		return true;
	}
	return false;

}



bool Tools_SaveAsura::TextureAllowedForThisFile(Asura_String& name)
{
	static bool FirstTime = true;
	static bool IsToolsFile = false;

	//first time this gets called we need to check the asura file that was loaded to see 
	//if it is the tools shapes file
	if(FirstTime)
	{
		char filename[MAX_PATH];
		strcpy(filename,EnvironmentFilename.GetString());
		_strlwr(filename);
		if(strstr(filename,"toolsshapes"))
		{
			IsToolsFile = true;
		}
	}

	if(IsToolsFile)
	{
		return name.Contains("\\tools\\");
	}
	else if(FileType == AsuraFile_Environment)
	{
		return name.Contains("environments");
	}
	else
	{
		return name.Contains("objects") || name.Contains("characters") || name.Contains("vehicles");
	}
}

bool Tools_SaveAsura::TextureIsUsedByModules(int iIndex)
{
	for(int i=0;i<Asura_Module_System::GetNumberOfModules();i++)
	{
		Asura_Mesh* pxMesh = Asura_Module_System::GetMeshForModule(i);
		if(!pxMesh->TextureDataValid) continue;

		for(int j=0;j<pxMesh->NumberOfPolygons;j++)
		{
			for(int iChan=0;iChan<ASURA_MATERIAL_MAXTEXTURES;iChan++)
			{
				if(Asura_Material_System::GetTextureID(pxMesh->TextureDataPtr[j].m_iMaterialID,iChan) == iIndex) return true;
				if(Asura_Material_System::GetNormalMapID(pxMesh->TextureDataPtr[j].m_iMaterialID,iChan) == iIndex) return true;
				if(Asura_Material_System::GetSpecularMapID(pxMesh->TextureDataPtr[j].m_iMaterialID,iChan) == iIndex) return true;
			}
		}

	}
	return false;
}


void Tools_SaveAsura::MakeBackupIfNeeded()
{
	if(!s_iNumBackupsToMake) return;
	
	ASURA_CPU_PROFILE_SCOPED_TIMER( "Tools_SaveAsura::MakeBackupIfNeeded");

	static bool s_bFirstTime = true;
	static u_int s_uFramestamp = 0;
	static u_int s_uLastBackupTime = 0;

	//First time through we always want to do a backup , on subsequent occasions we may
	//want to if enough time has passed
	if(s_bFirstTime)
	{
		s_bFirstTime = false;
	}
	else
	{
		//Make sure we haven't already checked this frame
		//(This is to prevent a backup being made halfway through saving a batch of files)
		if(s_uFramestamp == Asura_Timers::GetFrameNumber()) return;
		s_uFramestamp = Asura_Timers::GetFrameNumber();

		if((GetTickCount() - s_uLastBackupTime) < (u_int)(60*1000*s_iBackupInterval)) return;
	}

	s_uLastBackupTime = GetTickCount();


	if(FileType == AsuraFile_Template)
	{
		//just need to backup the template file itself in this case
		BackupFile(s_xTemplateFilename.GetString());
	}
	else
	{
		Asura_Assert(This,"");
		This->MakeBackups();	
	}
	
}

void Tools_SaveAsura::MakeBackups()
{
	//The main asura file
	Tools_AsrFileManager::MakeBackups();

	//The lights
	BackupFile(GetLightingFilename());
	BackupFile(GetGlobalIlluminationFilename());
	if(IsUsingMissionLighting())
	{
		BackupFile(GetMissionLightingFilename());
	}
	if(IsUsingMissionGlobalIllumination())
	{
		BackupFile(GetMissionGlobalIlluminationFilename());
	}

	//The sounds
	BackupFile(GetSoundsFilename());
	if(IsEditingMission())
	{
		BackupFile(GetMissionSoundsFilename());
	}

	//The waypoints
	BackupFile(GetNavigationFilename());

	//The skybox
	BackupFile(GetSkyboxFilename());

	//The cutscenes
	BackupFile(GetCutscenesFilename());

	//The module linking
	BackupFile(GetModuleLinkingFilename());

	//The entities
	BackupFile(GetEntityFilename());
	Tools_SecondaryEntityFile_System::MakeBackups();

#ifdef ASURA_USE_BILLBOARD_PLANTS
	//The billboard plants
	BackupFile(GetBillboardPlantsFilename());
#endif

#ifdef ASURA_USE_STREAMING
	BackupFile(GetStreamingConnectionsFilename());
#endif

#ifdef ASURA_USE_ATTRACTORS
	BackupFile(GetAttractorsFilename());
	if(IsEditingMission())
	{
		BackupFile(GetMissionAttractorsFilename());
	}
#endif

#ifdef ASURA_USE_REVERBVOLUME_SYSTEM
	BackupFile(GetReverbVolumesFilename());
#endif
}

void Tools_SaveAsura::BackupFile(const char* fullFilename)
{
	if(!s_iNumBackupsToMake) return;

	//see if the file is read-only
	DWORD attr = GetFileAttributes(fullFilename);
	if(attr & FILE_ATTRIBUTE_READONLY)
	{
		//file is readonly , or doesn't exist (attr==0xffffffff)
		//either way , no backup is needed
		return;
	}
	
	//split up the filename
	Asura_String filenameCopy = fullFilename;
	Asura_String path,filename,extension;
	filenameCopy.SplitFilename(&path,&filename,&extension);

	//determine the name of the backup directory
	char BackupDir[MAX_PATH];
	if(path.GetString())
	{
		sprintf(BackupDir,"%s\\Backup",path.GetString());
	}
	else
	{
		strcpy(BackupDir,"Backup");
	}
	
	//create it
	CreateDirectory(BackupDir,0);

	char fromName[MAX_PATH];
	char toName[MAX_PATH];

	//rename the previous backup files
	for(int i=s_iNumBackupsToMake;i;i--)
	{
		sprintf(toName,"%s\\%s_B%d.%s",BackupDir,filename.GetString(),i,extension.GetString());
		if(i-1)sprintf(fromName,"%s\\%s_B%d.%s",BackupDir,filename.GetString(),i-1,extension.GetString());
		

		DWORD attr = GetFileAttributes(toName);
		if(attr != 0xffffffff)
		{
			attr &=~ FILE_ATTRIBUTE_READONLY;
			SetFileAttributes(toName,attr);
		}
		DeleteFile(toName);
		if(i-1)
		{
			MoveFile(fromName,toName);
		}
		else
		{
			CopyFile(fullFilename,toName,FALSE);
		}
	}
}

bool Tools_SaveAsura::GetTemplateEnvironmentName(const char* szTemplateName, char* pcOutputBuffer, int iBufferSize)
{
	//Check the ini file to determine which environment should be loaded

	Tools_IniFile xEntityIni("ToolsIniFiles\\EntityTemplate.ini");

	if(!xEntityIni.LoadIniFile())
	{
		Asura_Assert(0,"Failed to load ToolsIniFiles\\EntityTemplate.ini.\nUnable to determine what evironment to use for template file.");
		return false;
	}


	Asura_String xEnvName;
	if(!szTemplateName || !*szTemplateName || !xEntityIni.GetStringInCurrentSection(szTemplateName,xEnvName))
	{
		//no entry for this template , try for the default
		if(!xEntityIni.GetStringInCurrentSection("Default",xEnvName))
		{
			//failed to find default entry either
			Asura_Assert(0,"Failed to find level file to use in ToolsIniFiles\\EntityTemplate.ini.");
			return false;
		}
	}
	Asura_PrintfToBfr(pcOutputBuffer,"%s", iBufferSize,true,xEnvName.GetString());
	return true;
}

// methods functionality extended to deal with TextureTiler mode which behaves in a similar maner to Template Mode.
// Methods name will  likely change to InitialiseSubstituteEnvironmentLoaIfNeeded()- NDB
void Tools_SaveAsura::InitialiseTemplateLoadingIfNeeded()
{
	extern char* DefaultFileToLoad;

	static char s_acNewFileToLoad[MAX_PATH];

	if(DefaultFileToLoad)
	{
		char* pcDot = strrchr(DefaultFileToLoad,'.');
	
		if(pcDot && !stricmp(pcDot,".AsuraTemplate"))
		{
			//template file selected , store the filename , and set the test environment as the file to load
			FileType = AsuraFile_Template;
			s_xTemplateFilename = DefaultFileToLoad;


			Asura_String xTemplateName;
			s_xTemplateFilename.SplitFilename(0,&xTemplateName,0);

			if(GetTemplateEnvironmentName(xTemplateName.GetString(), s_acNewFileToLoad, sizeof(s_acNewFileToLoad)))
			{
				DefaultFileToLoad = s_acNewFileToLoad;	
			}
		}
		else if(pcDot && !stricmp(pcDot,".Mission"))
		{
			//Mission file selected , open it up to find the name of the asura file to load
			Asura_String xEnvName;
			if(!ReadMissionIniFile(DefaultFileToLoad,xEnvName))
			{
				return;
			}

			//store the mission file name (minus extension)
			char acFilenameCopy[MAX_PATH];
			Asura_PrintfToBfr(acFilenameCopy, "%s", sizeof(acFilenameCopy), true, DefaultFileToLoad);
			char* pcExtension = strrchr(acFilenameCopy,'.');
			if(pcExtension) *pcExtension = 0;

			s_xMissionFilenameWithoutExtension = acFilenameCopy;

			//set the asura file to be loaded
			Asura_PrintfToBfr(s_acNewFileToLoad,"%s",sizeof(s_acNewFileToLoad),true,xEnvName.GetString());
			DefaultFileToLoad = s_acNewFileToLoad;	
		}
		else if( Tools_AsuraFileContents::GetAsuraFileType( DefaultFileToLoad ) == AsuraFile_TextureTiler )
		{
			FileType = AsuraFile_TextureTiler;
			Tools_Mode_TextureTiler::SetDefaultFileToLoad( DefaultFileToLoad );

			Asura_String xEnvName("Environments\\JustAPlane\\JustAPlane.asr");
		
			Asura_PrintfToBfr(s_acNewFileToLoad,"%s",sizeof(s_acNewFileToLoad),true,xEnvName.GetString());
			DefaultFileToLoad = s_acNewFileToLoad;	

			Tools_System::SetDefaultMode( AsuraFile_TextureTiler );
		}
		#ifdef ASURA_USE_GUI_MENU_SYSTEM
		else if(pcDot && !stricmp(pcDot,".gui"))
		{
			//If a .gui file is being loaded from the GUIMenu directory , then load the default template environment instead
			const char* szRelativeFilename =  Asura_FileAccess::PlatformRemoveNetworkOrExecutableDir(DefaultFileToLoad);
			if(!strnicmp(szRelativeFilename,"GUIMenu",7))
			{
				if(GetTemplateEnvironmentName( "", s_acNewFileToLoad, sizeof(s_acNewFileToLoad)))
				{
					DefaultFileToLoad = s_acNewFileToLoad;	
					Tools_System::SetDefaultMode( ".gui" );
				}
			}
		}
		#endif
	}
}

bool Tools_SaveAsura::ReadMissionIniFile(const char* szFilename, Asura_String& xOutputEnvName)
{
	const char* aszKnownIdentifiers[] =
	{
		"Asura_File",
		"Use_Mission_LIT",
		"Use_Mission_GI",
	};
	const int iNumKnownIdentifiers = sizeof(aszKnownIdentifiers) / sizeof(const char*);

	Tools_IniFile xIni(szFilename);

	if(!xIni.LoadIniFile())
	{
		Asura_Assertf(0,("Failed to load %s.",szFilename));
		return false;
	}

	//warn if any unrecognised entries are found int the file
	List<Asura_String> xIdentifierList;
	xIni.ListIdentifiersInCurrentSection(xIdentifierList);

	while(xIdentifierList.size())
	{
		const Asura_String& xIdentifier = xIdentifierList.first_entry();

		bool bOkay =  false;
		
		for(int i=0; i<iNumKnownIdentifiers; i++)
		{
			if( !stricmp(xIdentifier.GetString(), aszKnownIdentifiers[i]) )
			{
				bOkay = true;
				break;				
			}
		}

		if(!bOkay)
		{
			Asura_Assertf(0,("Unrecognized identifer '%s' found in misison file '%s'.",xIdentifier.GetString(),szFilename));
		}

		xIdentifierList.delete_first_entry();
	}

	//get the name of the asura file to load
	if(!xIni.GetStringInCurrentSection("Asura_File",xOutputEnvName))
	{
		Asura_Assertf(0,("Failed to find Asura_File entry in %s",szFilename));
		return false;
	}

	//and get some other optional values
	xIni.GetBoolInCurrentSection("Use_Mission_LIT",s_bMission_UseMissionLighting);
	xIni.GetBoolInCurrentSection("Use_Mission_GI",s_bMission_UseMissionGlobalIllumination);

	return true;

}


void Tools_SaveAsura::HandleTransferOfStartPositionFromAsuraFileToEntityFile()
{
	if(GetFileType() != AsuraFile_Environment) return;		
	
	if(Asura_Chunk_StartPosition::s_eStartPositionFile != Asura_Chunk_StartPosition::SPF_ASURA)
	{
		//if they weren't loaded from the asura file , then nothing needs to be done.
		return;
	}

	//see if both asura and entity files are available for writing
	if(!CanSaveEntity() || !CanSaveFile(EnvironmentFilename.GetString(),false))
	{	
		return;
	}

	//change the location for the start positions ,and save both files to do the transfer
	Asura_Chunk_StartPosition::s_eStartPositionFile = Asura_Chunk_StartPosition::SPF_ENTITY;

	QuickSaveFile();
	SaveEntities();

	ShowMessageOverlay("Start positions have been moved from the asura file to the entity file.\nPlease make sure both get checked in at the same time.");
}

void Tools_SaveAsura::SetFilesToCheckForExternalModificationMain()
{
	if(FileType == AsuraFile_Template)
	{
		Tools_LoadedFileList::TrackExternalModification(s_xTemplateFilename.GetString());
	}
	else
	{
		Tools_AsrFileManager::SetFilesToCheckForExternalModification();

		if(FileType == AsuraFile_Environment)
		{
			Tools_LoadedFileList::TrackExternalModification(GetModuleLinkingFilename());
			Tools_LoadedFileList::TrackExternalModification(GetLightingFilename());
			Tools_LoadedFileList::TrackExternalModification(GetEnvironmentalLightTransportFilename());
			Tools_LoadedFileList::TrackExternalModification(GetGlobalIlluminationFilename());
			Tools_LoadedFileList::TrackExternalModification(GetSoundsFilename());
			Tools_LoadedFileList::TrackExternalModification(GetSkyboxFilename());
			Tools_LoadedFileList::TrackExternalModification(GetEntityFilename());
			Tools_LoadedFileList::TrackExternalModification(GetCutscenesFilename());
			Tools_LoadedFileList::TrackExternalModification(GetNavigationFilename());
			#ifdef ASURA_USE_STREAMING
			Tools_LoadedFileList::TrackExternalModification(GetStreamingConnectionsFilename());
			#endif

			#ifdef ASURA_USE_ATTRACTORS
			Tools_LoadedFileList::TrackExternalModification(GetAttractorsFilename());
			#endif
			
			#ifdef ASURA_USE_BILLBOARD_PLANTS
			Tools_LoadedFileList::TrackExternalModification(GetBillboardPlantsFilename());
			#endif

			#ifdef ASURA_USE_REVERBVOLUME_SYSTEM
			Tools_LoadedFileList::TrackExternalModification(GetReverbVolumesFilename());
			#endif

			if(IsEditingMission())
			{
				if(IsUsingMissionLighting())
				{
					Tools_LoadedFileList::TrackExternalModification(GetMissionLightingFilename());
				}
				if(IsUsingMissionGlobalIllumination())
				{
					Tools_LoadedFileList::TrackExternalModification(GetMissionGlobalIlluminationFilename());
				}
				Tools_LoadedFileList::TrackExternalModification(GetMissionSoundsFilename());
				#ifdef ASURA_USE_ATTRACTORS
				Tools_LoadedFileList::TrackExternalModification(GetMissionAttractorsFilename());
				#endif

			}
		}

		Tools_SoundEvent_System::SetFilesToCheckForExternalModification();

		#ifdef ASURA_USE_DIALOGUE_SYSTEM
		Tools_Dialogue_System::SetFilesToCheckForExternalModification();
		#endif

		#ifdef ASURA_USE_GUI_MENU_SYSTEM
		Tools_GUIMenu_System::SetFilesToCheckForExternalModification();
		#endif

		#ifdef ASURA_USE_STREAMING
		Tools_LoadedFileList::TrackExternalModification(GetGlobalMetaMusicFilename());
		#endif

		#ifdef ASURA_USE_AXON
		Tools_AnimTemplates::SetFilesToCheckForExternalModification();
		#endif
		
		#ifdef ASURA_USE_ANIMATION_IK_SOLVER
		Tools_LoadedFileList::TrackExternalModification(Tools_Animation_IK_Template_System::GetIKTemplateFilename());
		#endif

        // Cycle through all of the blueprint types and add their files to the list of files to track.
		for ( Asura_Blueprint_Type_ConstIterator xIterator; !xIterator.Done(); xIterator.Next() )
		{
            const Asura_Hash_ID uTypeHashID = xIterator.GetCurrentHash();
			const Asura_String* pxFileName  = Asura_Blueprint_System::GetTypeFileName( uTypeHashID );
            if ( pxFileName && !pxFileName->IsBlank() )
            {
                Tools_LoadedFileList::TrackExternalModification( pxFileName->GetString() );
            }
        }
	}
}

void Tools_SaveAsura::FlagEnvironmentFilesAsLoadedMain()
{
	if(FileType == AsuraFile_Template)
	{
		Tools_LoadedFileList::AddFile(s_xTemplateFilename.GetString());
	}
	else
	{
		Tools_AsrFileManager::FlagEnvironmentFilesAsLoaded();

		if(FileType == AsuraFile_Environment)
		{
			Tools_LoadedFileList::AddFile(GetModuleLinkingFilename());
			Tools_LoadedFileList::AddFile(GetLightingFilename());
			Tools_LoadedFileList::AddFile(GetEnvironmentalLightTransportFilename());
			Tools_LoadedFileList::AddFile(GetGlobalIlluminationFilename());
			Tools_LoadedFileList::AddFile(GetSoundsFilename());
			Tools_LoadedFileList::AddFile(GetSkyboxFilename());
			Tools_LoadedFileList::AddFile(GetEntityFilename());
			Tools_LoadedFileList::AddFile(GetCutscenesFilename());
			Tools_LoadedFileList::AddFile(GetNavigationFilename());
			#ifdef ASURA_USE_STREAMING
			Tools_LoadedFileList::AddFile(GetStreamingConnectionsFilename());
			#endif

			#ifdef ASURA_USE_ATTRACTORS
			Tools_LoadedFileList::AddFile(GetAttractorsFilename());
			#endif
			
			#ifdef ASURA_USE_BILLBOARD_PLANTS
			Tools_LoadedFileList::AddFile(GetBillboardPlantsFilename());
			#endif

			#ifdef ASURA_USE_REVERBVOLUME_SYSTEM
			Tools_LoadedFileList::AddFile(GetReverbVolumesFilename());
			#endif

			if(IsEditingMission())
			{
				if(IsUsingMissionLighting())
				{
					Tools_LoadedFileList::AddFile(GetMissionLightingFilename());
				}
				if(IsUsingMissionGlobalIllumination())
				{
					Tools_LoadedFileList::AddFile(GetMissionGlobalIlluminationFilename());
				}
				Tools_LoadedFileList::AddFile(GetMissionSoundsFilename());
				#ifdef ASURA_USE_ATTRACTORS
				Tools_LoadedFileList::AddFile(GetMissionAttractorsFilename());
				#endif

			}
		}
	}
}


#ifdef ASURA_USE_ATTRACTORS

char* Tools_SaveAsura::GetAttractorsFilename()
{
	static Asura_String s_xFilename;
	s_xFilename = NameWithoutExtension;
	
	s_xFilename += ( Tools_Level_Export::ShouldExportBigEndian() ) ? ".atr" ASURA_STANDARD_FILE_EXTENSION_BE_SUFFIX : ".atr";
	return s_xFilename.GetString();
}

char* Tools_SaveAsura::GetMissionAttractorsFilename()
{
	Asura_Assert(IsEditingMission(),"");

	static Asura_String s_xFilename;
	s_xFilename = s_xMissionFilenameWithoutExtension;
	
	s_xFilename += ( Tools_Level_Export::ShouldExportBigEndian() ) ? ".atr" ASURA_STANDARD_FILE_EXTENSION_BE_SUFFIX : ".atr";
	return s_xFilename.GetString();
}

#endif


#ifdef ASURA_USE_BOTANICALS
char* Tools_SaveAsura::GetBotanicalsInstancesFilename()
{
	static Asura_String s_xFilename;
	s_xFilename = NameWithoutExtension;
	if( Tools_Level_Export::ShouldExportBigEndian() )
	{
		s_xFilename += ".bti" ASURA_STANDARD_FILE_EXTENSION_BE_SUFFIX;
	}
	else
	{
		s_xFilename += ".bti";
	}
	
	return s_xFilename.GetString();
}

char* Tools_SaveAsura::GetBotanicalsTemplatesFilename()
{
	static Asura_String s_xFilename;
	s_xFilename = NameWithoutExtension;
	if( Tools_Level_Export::ShouldExportBigEndian() )
	{
		s_xFilename += ".btt" ASURA_STANDARD_FILE_EXTENSION_BE_SUFFIX;
	}
	else
	{
		s_xFilename += ".btt";
	}
	
	return s_xFilename.GetString();
}

char* Tools_SaveAsura::GetBotanicalsStylesFilename()
{
	static Asura_String s_xFilename;
	s_xFilename = NameWithoutExtension;
	if( Tools_Level_Export::ShouldExportBigEndian() )
	{
		s_xFilename += ".bts" ASURA_STANDARD_FILE_EXTENSION_BE_SUFFIX;
	}
	else
	{
		s_xFilename += ".bts";
	}
	
	return s_xFilename.GetString();
}
#endif // #ifdef ASURA_USE_BOTANICALS


#ifdef ASURA_USE_CARPET
char* Tools_SaveAsura::GetCarpetMeshFilename()
{
	static Asura_String s_xFilename;
	s_xFilename = NameWithoutExtension;
	if( Tools_Level_Export::ShouldExportBigEndian() )
	{
		s_xFilename += ".cpm" ASURA_STANDARD_FILE_EXTENSION_BE_SUFFIX;
	}
	else
	{
		s_xFilename += ".cpm";
	}
	
	return s_xFilename.GetString();
}


char* Tools_SaveAsura::GetCarpetStylesFilename()
{
	static Asura_String s_xFilename;
	s_xFilename = NameWithoutExtension;
	if( Tools_Level_Export::ShouldExportBigEndian() )
	{
		s_xFilename += ".cps" ASURA_STANDARD_FILE_EXTENSION_BE_SUFFIX;
	}
	else
	{
		s_xFilename += ".cps";
	}
	
	return s_xFilename.GetString();
}
#endif // #ifdef ASURA_USE_CARPET


#ifdef ASURA_USE_STATIC_DECAL_SYSTEM
char* Tools_SaveAsura::GetStaticDecalsFilename()
{
	static Asura_String s_xFilename;
	s_xFilename = NameWithoutExtension;
	if( Tools_Level_Export::ShouldExportBigEndian() )
	{
		s_xFilename += ".dec" ASURA_STANDARD_FILE_EXTENSION_BE_SUFFIX;
	}
	else
	{
		s_xFilename += ".dec";
	}

	return s_xFilename.GetString();
}
#endif

#ifdef ASURA_USE_REVERBVOLUME_SYSTEM

const char* Tools_SaveAsura::GetReverbVolumesFilename()
{
	static Asura_String s_xFilename;
	s_xFilename = NameWithoutExtension;
	
	s_xFilename += ( Tools_Level_Export::ShouldExportBigEndian() ) ? ".rvb" ASURA_STANDARD_FILE_EXTENSION_BE_SUFFIX : ".rvb";
	return s_xFilename.GetString();
}

#endif



void Tools_SaveAsura::AddOverlayControls(Asura_Overlay* pxOverlay,Asura_Vector_2& xOffset)
{
	Asura_Dialog* pxDialog;

	pxDialog = new Asura_Numerical_Dialog("Num Backups",xOffset,&s_iNumBackupsToMake,0,999,1,"%d");
	pxDialog->SetHelpText("Sets the number of backups of each file that will be kept.\nNote that reducing this value won't cause any old backups to be removed.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Backup Interval",xOffset,&s_iBackupInterval,0,999,1,"%d");
	pxDialog->SetHelpText("How frequently (in minutes) that backups will be taken.\nThe first backup occurs on the first save.\nSubsequent backups will take place when saving if enough time has passed.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();
}

