//
// Tools_SaveAsura.h
//
// Created by Kevin Lea - 18/07/2001 14:56:07
// 
// Copyright (C) 2001 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef __TOOLS_SAVEASURA_H__
#define __TOOLS_SAVEASURA_H__

#include "ChunkSystem/Asura_Chunk_FileInfo.h"
#include "Core/Asura_String.h"
#include "Misc/list_tem.hpp"

#define TOOLS_DEBUGGING_LOGFILE "Logfiles\\ToolsDebugging.txt"


class Asura_Mesh;
class Asura_FragmentType;
class Asura_ServerEntity_Template;
class Asura_SaveBlock;

class Tools_SaveAsura
{
	friend class Tools_AsrFileManager;

public:
	Tools_SaveAsura();
	virtual ~Tools_SaveAsura(){This=NULL;}

	static void Shutdown();

	static void SaveAll(void) {SaveAll(true);}
	static void SaveModified(void) {SaveAll(false);}
	static void SaveAllAs(void);
	static void QuickSaveFile(void);

	static void SetFilename(const char* pszMainFilename);
    static void SetRealFilename( const Asura_String& xRealFilename );
	static const char* GetRealFilename();

	static void LoadEnvironment();

	static void SaveLights();
	static void SaveMissionLights();
	static void SaveEnvironmentalLightTransport();
	static void SaveGlobalIllumination();
	static void SaveMissionGlobalIllumination();
	static void SaveModuleLinking( bool bForceRegionRequirementUpdate = true );
	static void SaveSkybox();
	static void SaveSounds();
	static void SaveMissionSounds();
	static void SaveCutscenes();
	static void SaveEntities();
	static void SaveShapes();
	static void SaveWaypointing();
#ifdef ASURA_USE_BILLBOARD_PLANTS
	static void SaveBillboardPlants();
#endif //ASURA_USE_BILLBOARD_PLANTS
#ifdef ASURA_USE_BOTANICALS
	static void SaveBotanicals();
	static void SaveBotanicalCreation();
#endif // #ifdef ASURA_USE_BOTANICALS
#ifdef ASURA_USE_ATTRACTORS
	static void SaveAttractors();
	static void SaveMissionAttractors();
#endif // ASURA_USE_ATTRACTORS
#ifdef ASURA_USE_CARPET
	static void SaveCarpet();
#endif // #ifdef ASURA_USE_CARPET
#ifdef ASURA_USE_STATIC_DECAL_SYSTEM
	static void SaveStaticDecals();
#endif
#ifdef ASURA_USE_DECAL_MANAGER
	static void SaveDecalTemplates();
#endif
#ifdef ASURA_USE_STREAMING
	static void SaveStreamingConnections();
#endif
    static void SaveBlueprints();
#ifdef ASURA_USE_AXON
    static void SaveAnimTemplates();
#endif
#ifdef ASURA_USE_STREAMING
	static void SaveGlobalMetaMusic();
#endif
#ifdef ASURA_USE_REVERBVOLUME_SYSTEM
	static void SaveReverbVolumes();
#endif

	static void SaveModifiedAsuraFiles();//saves any additional skins/anims that may have been altered
	static void SaveTemplateFile();
	static bool SaveTemplateFile(Asura_ServerEntity_Template* pxTemplateEntity,const char* szFilename);
	static void SavePFXTemps();

	static void SetLightingModified(bool modified = true);
	static void SetMissionLightingModified(bool modified = true);
	static void SetEnvironmentalLightTransportModified( bool bModified = true );
	static void SetGlobalIlluminationModified( bool bModified = true );
	static void SetMissionGlobalIlluminationModified( bool bModified = true );
	static void SetModuleLinkingModified(bool modified = true);
	static void SetSkyboxModified(bool modified = true);
	static void SetSoundsModified(bool modified = true);
	static void SetMissionSoundsModified(bool modified = true);
	static void SetDynamicMusicModified(bool bModified = true);//Dynamic music is stored in one of the sounds files , so it will be setting one of those as modified
	static void SetCutscenesModified(bool modified = true);
	static void SetMainFileModified(bool modified = true);
	static void SetMeshModified(const Asura_Mesh* pxMesh);
	static void SetModuleModified(int iModuleIndex);
	static void SetEntityModified(bool modified = true);
	static void SetEntityModified(const class Asura_Entity_Server* pxEntity);
	static void SetShapesModified(bool modified = true);
	static void SetAnimationModified(class Asura_Hierarchy_Anim* anim,const char* pszFilename);
	static void SetSkinModified(class Asura_Hierarchy_Skin* pxSkin,const char* pszFilename);
	static void SetAsuraFileModified(const char* pszFilename,bool bModified = true);
	static void SetWaypointingModified(bool modified = true);
#ifdef ASURA_USE_BILLBOARD_PLANTS
	static void SetBillboardPlantsModified(bool modified = true);
#endif	//ASURA_USE_BILLBOARD_PLANTS
#ifdef ASURA_USE_STREAMING
	static void SetStreamingConnectionsModified(bool bModified = true);
#endif
#ifdef ASURA_USE_ATTRACTORS
	static void SetAttractorsModified(bool bModified = true);
	static void SetMissionAttractorsModified(bool bModified = true);
#endif
    static void SetBlueprintsModified(bool bModified = true);
#ifdef ASURA_USE_AXON
    static void SetAnimTemplatesModified(bool bModified = true);
#endif
#ifdef ASURA_USE_STREAMING
	static void SetGlobalMetaMusicModified(bool bModified = true);
#endif
#ifdef ASURA_USE_CARPET
	static void SetCarpetModified(bool bModified = true);
#endif
#ifdef ASURA_USE_BOTANICALS
	static void SetBotanicalsModified(bool bModified = true);
	static void SetBotanicalCreationModified(bool bModified = true);
#endif
#ifdef ASURA_USE_STATIC_DECAL_SYSTEM
	static void SetStaticDecalsModified(bool bModified = true);
#endif
#ifdef ASURA_USE_DECAL_MANAGER
	static void SetDecalTemplatesModified(bool bModified = true);
#endif
#ifdef ASURA_USE_REVERBVOLUME_SYSTEM
	static void SetReverbVolumesModified(bool bModified = true);
#endif

	static void SetTextureTilerModified(bool bModified = true);

	static bool IsLightingModified(){return LightingModified;}
	static bool IsMissionLightingModified(){return s_bMissionLightingModified && IsUsingMissionLighting();}
	static bool IsEnvironmentalLightTransportModified() { return s_bEnvironmentalLightTransportModified; }
	static bool IsGlobalIlluminationModified() { return s_bGlobalIlluminationModified; }
	static bool IsMissionGlobalIlluminationModified() { return s_bMissionGlobalIlluminationModified && IsUsingMissionGlobalIllumination(); }
	static bool IsModuleLinkingModified(){return ModuleLinkingModified;}
	static bool IsSkyboxModified(){return SkyboxModified;}
	static bool IsSoundsModified(){return SoundsModified;}
	static bool IsMissionSoundsModified(){return s_bMissionSoundsModified;}
	static bool IsCutscenesModified(){return CutscenesModified;}
	static bool IsMainFileModified();
	static bool IsEntityModified(){return EntityModified;}
	static bool IsShapesModified(){return ShapesModified;}
	static bool IsWaypointingModified(){return WaypointingModified;}
	static bool AreAsuraFilesModified(){return s_xModifiedAsuraFileList.size()!=0;}
	static int GetNumModifiedAsuraFiles(){return s_xModifiedAsuraFileList.size();}
	static void ClearModifiedAsuraFileList();
	static bool IsAsuraFileModified(const char* szFilename);//Note that this only checks skins/anims etc , and not the level files
#ifdef ASURA_USE_BILLBOARD_PLANTS
	static bool IsBillboardPlantsModified(){return BillboardPlantsModified;}
#endif //ASURA_USE_BILLBOARD_PLANTS
#ifdef ASURA_USE_STREAMING
	static bool IsStreamingConnectionsModified(){return s_bStreamingConnectionsModified;}
#endif
#ifdef ASURA_USE_ATTRACTORS
	static bool IsAttractorsModified(){return s_bAttractorsModified;}
	static bool IsMissionAttractorsModified(){return s_bMissionAttractorsModified;}
#endif
    static bool AreBlueprintsModified() { return s_bBlueprintsModified; }
#ifdef ASURA_USE_AXON
    static bool AreAnimTemplatesModified() { return s_bAnimTemplatesModified; }
#endif
#ifdef ASURA_USE_STREAMING
	static bool IsGlobalMetaMusicModified() { return s_bGlobalMetaMusicModified; }
#endif
#ifdef ASURA_USE_CARPET
	static bool IsCarpetModified() {return s_bCarpetModified;}
#endif
#ifdef ASURA_USE_BOTANICALS
	static bool IsBotanicalsModified() {return s_bBotanicalsModified;}
	static bool IsBotanicalCreationModified() {return s_bBotanicalCreationModified;}
#endif
#ifdef ASURA_USE_STATIC_DECAL_SYSTEM
	static bool IsStaticDecalsModified() {return s_bStaticDecalsModified;}
#endif
#ifdef ASURA_USE_DECAL_MANAGER
	static bool IsDecalTemplatesModified() {return s_bDecalTemplatesModified;}
#endif
	
	static bool IsTextureTilerModified() {return s_bTextureTilerModified;}

#ifdef ASURA_USE_REVERBVOLUME_SYSTEM
	static bool IsReverbVolumesModified() {return s_bReverbVolumesModified;}
#endif


	//checks all the individual modified flags to see if any changes have been made
	virtual bool IsModified();

	static void UpdateFilename(const Asura_String& Filename);

	static char* GetMainFilename();
	static char* GetLightingFilename();	  
	static char* GetMissionLightingFilename();	  
	static const char* GetEnvironmentalLightTransportFilename();
	static const char* GetGlobalIlluminationFilename();
	static const char* GetMissionGlobalIlluminationFilename();
	static char* GetModuleLinkingFilename();
	static char* GetSkyboxFilename();
	static char* GetSoundsFilename();
	static char* GetMissionSoundsFilename();
	static char* GetCutscenesFilename();
	static char* GetEntityFilename();
	static char* GetShapesFilename();
	static char* GetNavigationFilename();
#ifdef ASURA_USE_BOTANICALS
	static char* GetBotanicalsInstancesFilename();
	static char* GetBotanicalsTemplatesFilename();
	static char* GetBotanicalsStylesFilename();
#endif // #ifdef ASURA_USE_BOTANICALS

#ifdef ASURA_USE_ATTRACTORS
	static char* GetAttractorsFilename();
	static char* GetMissionAttractorsFilename();
#endif // ASURA_USE_ATTRACTORS

#ifdef ASURA_USE_CARPET
	static char* GetCarpetMeshFilename();
	static char* GetCarpetStylesFilename();
#endif // #ifdef ASURA_USE_CARPET
#ifdef ASURA_USE_STATIC_DECAL_SYSTEM
	static char* GetStaticDecalsFilename();
#endif

#ifdef ASURA_USE_REVERBVOLUME_SYSTEM
	static const char* GetReverbVolumesFilename();
#endif


	static const char* GetNameWithoutExtension() { return NameWithoutExtension.GetString(); };
	static const char* GetMissionFilenameWithoutExtension() { return s_xMissionFilenameWithoutExtension.GetString(); }
#ifdef ASURA_USE_BILLBOARD_PLANTS
	static char* GetBillboardPlantsFilename();
#endif
#ifdef ASURA_USE_STREAMING
	static const char* GetStreamingConnectionsFilename();
#endif
#ifdef ASURA_USE_STREAMING
	static const char* GetGlobalMetaMusicFilename() {return "Sounds\\GlobalMetaMusic.asr"; }
#endif


	//get the filename for the list of files that should be added to all exports
	static const char* GetAllLevelsExportFilename();
	//get the filename for the list of files that should be added to this level
	static const char* GetLevelSpecificExportFilename();


	static bool CanSaveEntity(){return CanSaveFile(GetEntityFilename(),false);}
	static bool CanSaveCutscenes(){return CanSaveFile(GetCutscenesFilename(),false);}
	static bool CanSaveShapes();
	

	static enum AsuraFileType GetFileType(){return FileType;}

	static Asura_Hierarchy_Skin* GetSkin() { return s_pxSkin; }
	static Asura_Hierarchy_Anim* GetAnim() { return s_pxAnim; }
	static Asura_FragmentType* GetFragmentType() { return s_pxFragmentType; }
	
	//checks that a texture is in the environments directory
	//(or in the case of the tools shapes in the tools diorectory)
	static bool TextureAllowedForThisFile(Asura_String& name);

	//checks all the 'modules' to see if the texture is used
	static bool TextureIsUsedByModules(int iIndex);

	static bool FindAnimationRecursive(const char* directory,const char* animName,Asura_String& outputFileName);

	

	//Shows warning if not allowed to save file (read-only , or using network copy).
	//Returns true if file is okay.
	static bool WarnIfReadOnly(const char* filename);
	static bool CanSaveFile(const char* szFilename,bool bDisplayMessage);

	static void InitialiseTemplateLoadingIfNeeded();
	static bool IsTemplateEditing(){return !s_xTemplateFilename.IsBlank();}
	static const char* GetTemplateFilename(){return s_xTemplateFilename.GetString();}

	static bool IsEditingMission(){return !s_xMissionFilenameWithoutExtension.IsBlank();}
	static bool IsUsingMissionLighting() {return IsEditingMission() && s_bMission_UseMissionLighting;}
	static bool IsUsingMissionGlobalIllumination() {return IsEditingMission() && s_bMission_UseMissionGlobalIllumination;}

    static u_int PrepareMeshMaterialConversionLists( const List<Asura_Mesh*>& xMeshList, int* piToMesh, int* piFromMesh );

	static void HandleTransferOfStartPositionFromAsuraFileToEntityFile();

	static void Static_AddChunksToEntityFile( Asura_SaveBlock* pxSaveBlock )
    {
        This->AddChunksToEntityFile( pxSaveBlock );
    }
    static void Static_AddChunksToSkinFile( Asura_Hierarchy_Skin* pxSkin, Asura_SaveBlock* pxSaveBlock )
    {
        This->AddChunksToSkinFile( pxSkin, pxSaveBlock );
    }
    static void Static_AddMeshesAndTexturesToSaveBlock( const List<Asura_Mesh*>& xMeshList, Asura_Chunk_Stream& xStream, const char* szFileName, AsuraFileType eFileType )
    {
        This->AddMeshesAndTexturesToSaveBlock( xMeshList, xStream, szFileName, eFileType, NULL );
    }
    static bool Static_AddChunksToWaypointingFile( Asura_SaveBlock* pxSaveBlock )
    {
        return This->AddChunksToWaypointingFile( pxSaveBlock );
    }

	static void SetFilesToCheckForExternalModification(){This->SetFilesToCheckForExternalModificationMain();}
	//Adds the main .asr and ancillary files to Tools_LoadedFileList
	static void FlagEnvironmentFilesAsLoaded(){This->FlagEnvironmentFilesAsLoadedMain();}

	//lists the meshes used by a skin , in the order that they would be saved to the skin file
	static void ListMeshesForSkin(const Asura_Hierarchy_Skin* pxSkin,List<Asura_Mesh*>& xMeshList);

	//Backup options
	static int GetNumBackupsToMake() {return s_iNumBackupsToMake;}
	static void SetNumBackupsToMake(int iVal) {s_iNumBackupsToMake = iVal;}

	static int GetBackupInterval() {return s_iBackupInterval;}
	static void SetBackupInterval(int iVal) {s_iBackupInterval = iVal;}

	//creates a backup of a file in the backups directory if it can be saved
	static void BackupFile(const char* fullFilename);



	//adds controls to general options overlay
	static void AddOverlayControls(class Asura_Overlay* pxOverlay,class Asura_Vector_2& xOffset);

	//Used when exporting a set of entities to a template file
	//(Doesn't apply when the main file is a template file however)
	static const Asura_ServerEntity_Template* GetTemplateBeingExported() {return s_pxExportedTemplate;}
	static void SetTemplateBeingExported(const Asura_ServerEntity_Template* pxTemplateEntity) {s_pxExportedTemplate = pxTemplateEntity;}

protected:

	//save asura files in addition to the main one
	//If bSaveAll is false then only the modified files should be saved
	virtual void SaveAncillaryFiles(bool bSaveAll);
	//load asura files in addition to the main one
	virtual void LoadAncillaryFiles(void);

	virtual void MakeBackups();

	static void MakeBackupIfNeeded();

	//overide to add project specific chunks to skybox file
	virtual void AddChunksToSkyboxFile(class Asura_SaveBlock* xSaveBlock);

	//overide to add project specific chunks to animation files
	virtual void AddChunksToAnimFile(Asura_Hierarchy_Anim* pxAnim, Asura_SaveBlock* xSaveBlock);
	virtual void AddChunksToSkinFile(Asura_Hierarchy_Skin* pxSkin, Asura_SaveBlock* xSaveBlock);

	virtual void AddChunksToMainFile(const char* pszFilename, Asura_Chunk_Stream& xStream);
	virtual void AddChunksToEntityFile(Asura_SaveBlock* pxSaveBlock);
    virtual bool AddChunksToWaypointingFile( Asura_SaveBlock* pxSaveBlock );

	//Tells Tools_LoadedFileList which files should be tracked for external modification.
	virtual void SetFilesToCheckForExternalModificationMain();
	//Adds the main .asr and ancillary files to Tools_LoadedFileList
	virtual void FlagEnvironmentFilesAsLoadedMain();

	
private :
	static void SaveAll(bool bSaveAll);

	static bool SaveMainAsrFile(const char* szFilename);

	
	static void GenerateEnvironmentForShapes();

	static void AddMeshesAndTexturesToSaveBlock(const List<Asura_Mesh*>& xMeshListt,Asura_Chunk_Stream& xStream,const char* pszFileName,AsuraFileType eFileType,const char* szMeshStrippedFilename);

	//saves over an existing skin/anim/fragment file
	bool SaveAsuraFile(const char* pszFilename);

	//Reads in settings from a .mission file.
	static bool ReadMissionIniFile(const char* szFilename, Asura_String& xOutputEnvName);

	//Checks ToolsIniFiles\EntityTemplate.ini and gets the environment to use for a given template
	static bool GetTemplateEnvironmentName(const char* szTemplateName, char* pcOutputBuffer, int iBufferSize);


	static class Asura_NameTree* EnvironmentTextureTree;

	//flags indicating which files have been modified
	static bool LightingModified;
	static bool s_bMissionLightingModified;
	static bool s_bEnvironmentalLightTransportModified;
	static bool s_bGlobalIlluminationModified;
	static bool s_bMissionGlobalIlluminationModified;
	static bool ModuleLinkingModified;
	static bool SkyboxModified;
	static bool SoundsModified;
	static bool s_bMissionSoundsModified;
	static bool CutscenesModified;
	static bool EntityModified;
	static bool ShapesModified;
	static bool WaypointingModified;
#ifdef ASURA_USE_BILLBOARD_PLANTS
	static bool BillboardPlantsModified;
#endif //ASURA_USE_BILLBOARD_PLANTS
#ifdef ASURA_USE_STREAMING
	static bool s_bStreamingConnectionsModified;
#endif
#ifdef ASURA_USE_ATTRACTORS
	static bool s_bAttractorsModified;
	static bool s_bMissionAttractorsModified;
#endif
    static bool s_bBlueprintsModified;
#ifdef ASURA_USE_AXON
    static bool s_bAnimTemplatesModified;
#endif
#ifdef ASURA_USE_STREAMING
	static bool s_bGlobalMetaMusicModified;
#endif 
#ifdef ASURA_USE_CARPET
	static bool s_bCarpetModified;
#endif
#ifdef ASURA_USE_BOTANICALS
	static bool s_bBotanicalsModified;
	static bool s_bBotanicalCreationModified;
#endif
#ifdef ASURA_USE_STATIC_DECAL_SYSTEM
	static bool s_bStaticDecalsModified;
#endif
#ifdef ASURA_USE_DECAL_MANAGER
	static bool s_bDecalTemplatesModified;
#endif
#ifdef ASURA_USE_REVERBVOLUME_SYSTEM
	static bool s_bReverbVolumesModified;
#endif

	static bool s_bTextureTilerModified;

	static List<Asura_String> s_xModifiedAsuraFileList;

	static enum AsuraFileType FileType;
	static Asura_Hierarchy_Skin* s_pxSkin;//pointer to skin if this is a skin file
	static Asura_Hierarchy_Anim* s_pxAnim;//pointer to anim in main file (if any)
	static Asura_FragmentType* s_pxFragmentType;//pointer to fragment type , if this is a fragment file

	static Asura_String s_xTemplateFilename;
	static Asura_String s_xTextureTilerFilename;
	static Asura_String s_xMissionFilenameWithoutExtension;
	static bool s_bMission_UseMissionLighting;
	static bool s_bMission_UseMissionGlobalIllumination;

	//Backup options
	static int s_iNumBackupsToMake;
	static int s_iBackupInterval;

	//When exporting a set of entities from a level as a template , this stores the pointer to the master entity , 
	//so that the individual save functions can save out the correct chunks
	static const Asura_ServerEntity_Template* s_pxExportedTemplate;


protected :
	static Asura_String NameWithoutExtension;
	static char GetFilenameReturnString[MAX_PATH];

public :
	//'this' substitute for all the static functions
	static Tools_SaveAsura* This;
};




#endif//__TOOLS_SAVEASURA_H__
