//
// Tools_SoundResource.cpp
//
// Created by Richard Rice - 30/04/03
// 
// Copyright (C) 2003 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"
#include "Load&Save/Tools_SoundResource.h"

#include "Core/Asura_String.h"
#include "ChunkSystem/Asura_Chunk_Loaders.h"
#ifdef ASURA_TOOLS
#include "ChunkSystem/Asura_Chunk_Macros.h"
#endif
#include "ChunkSystem/Asura_Chunk_ResourceFile.h"

#include "Misc/list_tem.hpp"


int Tools_SoundResource::GetSoundMappingResourceSize(const char* pszResourceName,const List<Asura_SoundMapping>& xMappingList)
{
	if(!xMappingList.size()) return 0;

	Asura_String xArchiveName = pszResourceName;

	int iDataSize = sizeof(int);    // version
	iDataSize += sizeof(int);       // number of sounds
	iDataSize += xMappingList.size() * sizeof(Asura_SoundMapping);
	int iTotalSize = (7*sizeof(int)) + (xArchiveName.GetSizeInInts()*4) + (iDataSize);

	return iTotalSize;

}

void Tools_SoundResource::CreateSoundMappingResource(const char* pszResourceName,const List<Asura_SoundMapping>& xMappingList,char* pcBufferStart)
{
	if(!xMappingList.size()) return;

	Asura_Assert(pcBufferStart,"");

	Asura_String xArchiveName = pszResourceName;

	int iDataSize = sizeof(int);    // version
	iDataSize += sizeof(int);       // number of sounds
	iDataSize += xMappingList.size() * sizeof(Asura_SoundMapping);
	int iTotalSize = (7*sizeof(int)) + (xArchiveName.GetSizeInInts()*4) + (iDataSize);


	int* piArchive = reinterpret_cast<int*>( pcBufferStart );
	*piArchive = ASURA_CHUNK_RESOURCEFILE;
	ASURA_CHUNK_VERIFY(*piArchive, int);
    piArchive++;
	*piArchive = iTotalSize; // Size
	ASURA_CHUNK_VERIFY(*piArchive, int);
    piArchive++;
	*piArchive = 1; // Version
	ASURA_CHUNK_VERIFY(*piArchive, int);
    piArchive++;
	*piArchive = 0; // Flags
	ASURA_CHUNK_VERIFY(*piArchive, int);
    piArchive++;

	*piArchive = ASURA_RESOURCEFILE_TYPE_SOUNDMAPPING; // Type
	ASURA_CHUNK_VERIFY(*piArchive, int);
    piArchive++;
	*piArchive = 0; // SubType
	ASURA_CHUNK_VERIFY(*piArchive, int);
    piArchive++;
	*piArchive = iDataSize; // Filesize
	ASURA_CHUNK_VERIFY(*piArchive, int);
    piArchive++;

	// add archivename (padded to int)
	piArchive = xArchiveName.IntCopy(piArchive);

	// start of sound mapping header
	*piArchive = ASURA_RESOURCEFILE_SOUNDMAPPING_VERSION; //version
	ASURA_CHUNK_VERIFY(*piArchive, int);
    piArchive++;
	*piArchive = xMappingList.size();
	ASURA_CHUNK_VERIFY(*piArchive, int);
    piArchive++;

	char* pcBuffer = (char*) piArchive;

	for(CLIF<Asura_SoundMapping> xMappingLIF(&xMappingList);!xMappingLIF.done();xMappingLIF.next())
	{
		const Asura_SoundMapping& xMapping = xMappingLIF();

		ASURA_CHUNK_FILLANDVERIFY(pcBuffer,xMapping.m_iNewSoundID,int);
		ASURA_CHUNK_FILLANDVERIFY(pcBuffer,xMapping.m_iExistingSoundID,int);
		ASURA_CHUNK_FILLANDVERIFY(pcBuffer,xMapping.m_uExistingSoundHash,Asura_Hash_ID);
	}


	Asura_Assert(pcBufferStart+iTotalSize == pcBuffer, "");
}

int Tools_SoundResource::GetSoundPropertiesResourceSize(const char* pszResourceName,const List<Tools_SoundProperties>& xPropertiesList)
{
	if(!xPropertiesList.size()) return 0;

	Asura_String xArchiveName = pszResourceName;

	int iDataSize = sizeof(int);    // version
	iDataSize += sizeof(int);       // number of sounds
	iDataSize += xPropertiesList.size() * sizeof(Tools_SoundProperties);
	int iTotalSize = (7*sizeof(int)) + (xArchiveName.GetSizeInInts()*4) + (iDataSize);

	return iTotalSize;

}

void Tools_SoundResource::CreateSoundPropertiesResource(const char* pszResourceName,const List<Tools_SoundProperties>& xPropertiesList,char* pcBufferStart)
{
	if(!xPropertiesList.size()) return;

	Asura_Assert(pcBufferStart,"");

	Asura_String xArchiveName = pszResourceName;

	int iDataSize = sizeof(int);    // version
	iDataSize += sizeof(int);       // number of sounds
	iDataSize += xPropertiesList.size() * sizeof(Tools_SoundProperties);
	int iTotalSize = (7*sizeof(int)) + (xArchiveName.GetSizeInInts()*4) + (iDataSize);


	int* piArchive = reinterpret_cast<int*>( pcBufferStart );
	*piArchive = ASURA_CHUNK_RESOURCEFILE;
	ASURA_CHUNK_VERIFY(*piArchive, int);
    piArchive++;
	*piArchive = iTotalSize; // Size
	ASURA_CHUNK_VERIFY(*piArchive, int);
    piArchive++;
	*piArchive = 1; // Version
	ASURA_CHUNK_VERIFY(*piArchive, int);
    piArchive++;
	*piArchive = 0; // Flags
	ASURA_CHUNK_VERIFY(*piArchive, int);
    piArchive++;

	*piArchive = ASURA_RESOURCEFILE_TYPE_SOUNDPROPERTIES; // Type
	ASURA_CHUNK_VERIFY(*piArchive, int);
    piArchive++;
	*piArchive = 0; // SubType
	ASURA_CHUNK_VERIFY(*piArchive, int);
    piArchive++;
	*piArchive = iDataSize; // Filesize
	ASURA_CHUNK_VERIFY(*piArchive, int);
    piArchive++;

	// add archivename (padded to int)
	piArchive = xArchiveName.IntCopy(piArchive);

	// start of sound properties header
	*piArchive = 0; //version
	ASURA_CHUNK_VERIFY(*piArchive, int);
    piArchive++;
	*piArchive = xPropertiesList.size();
	ASURA_CHUNK_VERIFY(*piArchive, int);
    piArchive++;

	char* pcBuffer = (char*) piArchive;

	for(CLIF<Tools_SoundProperties> xPropertiesLIF(&xPropertiesList);!xPropertiesLIF.done();xPropertiesLIF.next())
	{
		const Tools_SoundProperties& xProperties = xPropertiesLIF();

		ASURA_CHUNK_FILLANDVERIFY(pcBuffer,xProperties.m_iSoundID,int);
		ASURA_CHUNK_FILLANDVERIFY(pcBuffer,xProperties.m_fPitchScale,float);
		ASURA_CHUNK_FILLANDVERIFY(pcBuffer,xProperties.m_fVolumeScale,float);
		ASURA_CHUNK_FILLANDVERIFY(pcBuffer,xProperties.m_fDefaultInnerRadius,float);
		ASURA_CHUNK_FILLANDVERIFY(pcBuffer,xProperties.m_fDefaultOuterRadius,float);
	}

	Asura_Assert(pcBufferStart+iTotalSize == pcBuffer, "");
}

