//
// Tools_Sound_Export.h
//
// Created by Richard Rice - 23/05/03
// 
// Copyright (C) 2003 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef __TOOLS_SOUND_EXPORT_H__
#define __TOOLS_SOUND_EXPORT_H__

struct ToolsSoundConversionData_V6 
{
	int m_iMaxFrequency;
	bool m_bLoop;
	u_int m_uFirstSample;//for vag conversion , index of first vag sample to go into file.
	u_int m_uMaxSamples;//for vag conversion , maximum number of samples to use.(0 == no max)
	bool m_bLoadImmediate;// deprecated
	float m_fMinLength;// for vag conversion, pad end with silence
};

struct ToolsSoundConversionData : ToolsSoundConversionData_V6
{
	ToolsSoundConversionData();
	
	u_int m_uAT3BitRate;// For PSP AT3 conversion - bit rate in kbps (Note that at3tool only supports a limited set of values for this)
};

struct Tools_SoundResampleData
{
	u_int m_uNumberOfChannels;
	u_int m_uBitsPerSample;
	u_int m_uNumberOfSamples;
};

// -----------------------------------------------------------------------------
// -- This is for the R8Brain converter dll ------------------------------------

#ifndef DISABLE_R8BRAIN_DLL
#pragma pack(push)
#pragma pack(1)
struct R8BError // Should be 'packed' (align=1)
{
	bool bWasError;
	char szErrMsg[128];
};

struct R8BInputFormat // Should be 'packed'
{
	void* pBuffer;
	int iRate;
	int iBitDepth;
	int iChannels;
	int IEEE;
	int iLength;
};
#pragma pack(pop)

typedef void( *TR8BCallback )( int iPosition, int iMax, void* p );
typedef void (*r8b_getWAVInfo)( char* pszInFileName, int* piRate, int* piDepth, R8BError* pxError);
typedef void (*r8b_execute)( char* pszInFileName, char* pszOutFileName, int iOutRate, int iOutBitDepth, int iOutQuality, int iPreallocate, int* CancelFlag, TR8BCallback Callback, void* p, R8BError* pxError);
typedef void (*r8b_executeMem)( R8BInputFormat* pxInFormat, void* pOutBuffer, int iOutLength, int iOutRate, int iOutBitDepth, int iOutQuality, int* CancelFlag, TR8BCallback Callback, void* p, R8BError* pxError, int* piActualOut);

#endif	// DISABLE_R8BRAIN_DLL
// -----------------------------------------------------------------------------

class Tools_Sound_Export
{
public :
	//Converts the input wave file to a vag file using the provided conversion settings.
	//If the file already exists and is up to date , then it is left alone.
	//Returns true if the export was successful , or wasn't required
	static bool ExportVagFile(const char* pszInputFilename,const char* pszOutputFilename,const struct ToolsSoundConversionData* pxConvertData,u_int* puFailureCode = 0,float fMinLength = 0.0f);
	static bool ExportDSPFile(const char* szInputFilename, const char* szOutputFilename, const struct ToolsSoundConversionData* pxConvertData,u_int* puFailureCode = 0);
	static bool ExportXMAFile(const char* szInputFilename, const char* szOutputFilename, const struct ToolsSoundConversionData* pxConvertData,u_int* puFailureCode = 0);
	static int  ExportMSFFile(const char* szInputFilename, const char* szOutputFilename, const ToolsSoundConversionData* pxConvertData);
	static bool ExportAT3File(const char* szInputFilename, const char* szOutputFilename, const struct ToolsSoundConversionData* pxConvertData,u_int* puFailureCode = 0);
	static bool ExportBinkFile(const char* szInputFilename, const char* szOutputFilename, const struct ToolsSoundConversionData* pxConvertData,u_int* puFailureCode = 0);
	static bool ExportxWMAFile(const char* szInputFilename, const char* szOutputFilename, const struct ToolsSoundConversionData* pxConvertData,u_int* puFailureCode = 0);
	// Currently just uses a basic approach of dropping samples as needed when downsampling.
	static bool Resample_Sound(const Tools_SoundResampleData& xInputProperties,const Tools_SoundResampleData& xOutputProperties,const void* pInputBuffer,void* pOutputBuffer);

#ifndef DISABLE_R8BRAIN_DLL
	static void* ResampleSound_R8Brain( R8BInputFormat* pxInputFormat, int iNewSampleRate, int iNewBitsDepth, int* piOutSize);
#endif // DISABLE_R8BRAIN_DLL

	static void SetProjectVersionNumber( const unsigned short usProjectVersion )	{ s_usProjectVersion = usProjectVersion; }

private :
	//Enumeration used by DoesFileNeedExporting to determine which settings matter
	enum ConversionType
	{
	   	SOUND_CONVERSION_DEFAULT,//Used for anything that is happy to go along with the default options
		SOUND_CONVERSION_AT3,
	};

	//checks to see whether the export settings have changed since the last export (or if the input sound file has changed)
	static bool DoesFileNeedExporting(const char* pszInputFilename,const char* pszOutputFilename,const struct ToolsSoundConversionData* pxConvertData, struct ToolsSoundExportFile* pxSavedExportData,ConversionType eType);
	// Checks if the file needs exporting and grabs the shadow conversion if that is up to date.
	// Returns true if the shadow version is up to date and therefore no further work is needed.
	static bool UpdateFromShadow(const char* szInputFilename, const char* szOutputFilename, const ToolsSoundConversionData* pxConvertData,ConversionType eType);
	static void OnExportFinish(const char* szInputFilename, const char* szOutputFilename, const ToolsSoundConversionData* pxConvertData);
	//creates the .dat file containing the export details
	static void WriteConversionData(const char* pszInputFilename,const char* pszOutputFilename,const struct ToolsSoundConversionData* pxConvertData,bool bFirstSampleTooBig);

#ifndef DISABLE_R8BRAIN_DLL
	static bool Init_R8Brain_Dll( void);
	static bool Shutdown_R8Brain_Dll( void);

	static HINSTANCE		m_hDllInstance_R8B;
	static r8b_executeMem	m_fnR8B_ExecuteMem;
	static r8b_execute		m_fnR8B_Execute;
	static r8b_getWAVInfo	m_fnR8B_GetWavInfo;

	friend class Tools_R8Brain_Cleanup;

#endif // DISABLE_R8BRAIN_DLL

	static unsigned short s_usProjectVersion;
};

#define TOOLS_DEFAULT_MAX_PS2_FREQUENCY 16000

#endif //__TOOLS_SOUND_EXPORT_H__
