//
// Tools_TextureConversionData.h
//
// Created by KOR - 13/12/2005
//
// Copyright (C) 2005 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif

#ifndef __TOOLS_TEXTURECONVERSIONDATA_H__
#define __TOOLS_TEXTURECONVERSIONDATA_H__

#include "Load&Save/LevelExport/Tools_Level_Export.h"

//saved data to go along side converted textures , to indicate parameters used when doing the conversions
struct TextureConversionDataBase
{
	union
	{
		u_short m_usVersion;
		struct
		{
			u_char	m_ucVersion;
			u_char	m_ucPlatformVersion;
		};
	};
	//Can be bumped by projects to force all their textures to be regenerated if needed. RAR 26/10/2006
	//Unfortunately , it appears that it isn't safe to bump this by hand for PC in Project_ModifyTextureConversionSettings()
	//because it won't get done when entering tools.
	//So for the PC this needs to be increased by changing TOOLS_PROJECT_TEXTURE_CONVERSION_DATA_VERSION instead
	//RAR 09/03/2007
	u_short m_usProjectVersion;

	int m_iTextureFlags;

	//Sets palette to be used when using FixedPalette option
	static void SetFixedPalette(u_int uSize,const u_int* puPalette);
	//Get the current fixed palette
	static void GetFixedPalette(u_int& uSize, const u_int*& puPalette);

	
};

struct TextureConversionData_V2 : TextureConversionDataBase
{
	bool m_bSwizzled;
};

struct TextureConversionData_V4 : TextureConversionData_V2
{
	unsigned int m_uMaterialFlags;
};

struct TextureConversionData_V7 : TextureConversionData_V4
{
	bool m_bUnpalettised;//obsolete
};

struct TextureConversionData_V11 : TextureConversionData_V7
{
	//overrides for tim2 generation
	int m_iPixelFormat;
	int m_iPaletteFormat;
};

struct TextureConversionData_V16 : TextureConversionData_V11
{
	// For PS3, PSP (and GC...)
	int m_uMaxMipDrops;
};

struct TextureConversionData_V17 : TextureConversionData_V16
{
	//Causes a fixed palette to be used , as defined by SetFixedPalette
	bool m_bUseFixedPalette;

	//For simpsons... causes the end palette to contain index values in the red component , rather
	//than proper colour values.
	bool m_bWritePaletteAsRedScaleIndices;

};

struct TextureConversionData_V18 : TextureConversionData_V17
{
	//Prevents the use of dithering during palette generation.
	bool m_bDisableDithering;

};

struct TextureConversionData_V19 : TextureConversionData_V18
{
	u_int m_uTextureUsageFlags;//texture usage flags taken from the material response system


};

struct TextureConversionData_V20 : TextureConversionData_V19
{
	u_int m_uMaximumTextureSize;//forces textures to be halved in size until no larger than this value (if non-zero)

};

struct TextureConversionData : TextureConversionData_V20
{
	bool m_bSRGB; // Texture is stored in gamma 2.2 space

	//Sets the version and zeroes the rest
	void Initialise( Tools_Level_Export::EXPORT_TYPE eExportType );

	//zeroes various flags etc that don't affect the conversion for a given platform
	void ClearUnusedEntriesForPlatform(Tools_Level_Export::EXPORT_TYPE eExportType);

	//Gets a hash value based on the contents of the conversion data , for use in building the filename
	//for the network cache.
	Asura_Hash_ID GetHash() const;
};



#define TEXTURE_CONVERSION_DATA_VERSION 22

//This define can be increased on a project basic to force all the texturs to be regenerated
#ifndef TOOLS_PROJECT_TEXTURE_CONVERSION_DATA_VERSION
#define TOOLS_PROJECT_TEXTURE_CONVERSION_DATA_VERSION 0
#endif

#define TOOLS_PLATFORM_PC_TEXTURE_CONVERSION_DATA_VERSION		0

#define TOOLS_PLATFORM_PS2_TEXTURE_CONVERSION_DATA_VERSION		0

#define TOOLS_PLATFORM_XBox_TEXTURE_CONVERSION_DATA_VERSION		0

#define TOOLS_PLATFORM_PSP_TEXTURE_CONVERSION_DATA_VERSION		0

#define TOOLS_PLATFORM_PS3_TEXTURE_CONVERSION_DATA_VERSION		5

#define TOOLS_PLATFORM_Xbox360_TEXTURE_CONVERSION_DATA_VERSION	3

#define TOOLS_PLATFORM_Wii_TEXTURE_CONVERSION_DATA_VERSION		0

#endif//__TOOLS_TEXTURECONVERSIONDATA_H__
