
///////////////////////////////////////////////////////////////////////////////
//
// File   : Tools_Overlay_MaterialResponseOverrides.cpp
// Author : Andy Tate
// Date   : 26-Apr-2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"


// This file's header...
#include "Materials/Tools_Overlay_MaterialResponseOverrides.h"

// Asura independent header-files...
#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Overlay.h"
#include "Interface/Asura_Overlay_System.h"
#include "Interface/Asura_Table_Dialog.h"
#include "ChunkSystem/Asura_SaveBlock.h"

// Tools header-files...
#include "Tools_Overlays.h"
#include "Load&Save/Tools_SaveAsura.h"
#include "Misc/Tools_Timed_Messages.h"
#if defined(ASURA_USE_MATERIAL_RESPONSE_SYSTEM)
#include "Core/Asura_MaterialResponse_System.h"
#include "Materials/Tools_MaterialResponse_System.h"
#include "ChunkSystem/Tools_Chunk_MaterialResponseOverrides.h"

////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////

static const char*	g_szMaterialResponseOverridesFilename	= "Misc\\MaterialResponseOverrides.asr";

static const Asura_Vector_2	g_xTestMinimumCellSize(6.0f, 2.0f);
static Asura_Vector_2 g_xTableSize(80.0f, 21.0f);

////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////
bool Tools_Overlay_MaterialResponseOverrides::s_bModified = false;

////////////////////////////////////////////////////////////////////////////////
// Private functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Protected functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Class        : Tools_Overlay_MaterialResponseOverrides_Overlay
// Author       : Richard Rice
// Date         : 15/11/2007
//
// Description  : Derived overlay class for detecting if it is closed by the X button
//				  
////////////////////////////////////////////////////////////////////////////////

class Tools_Overlay_MaterialResponseOverrides_Overlay : public Asura_Overlay
{
public :
	Tools_Overlay_MaterialResponseOverrides_Overlay(Tools_Overlay_MaterialResponseOverrides* pxOverrides, const char *szName, bool bAlwaysOnTop, Asura_Vector_2* pxPosition)
	:Asura_Overlay(szName, bAlwaysOnTop, pxPosition)
	, m_pxOverrides(pxOverrides)
	{}

	~Tools_Overlay_MaterialResponseOverrides_Overlay()
	{
		m_pxOverrides->OnOverlayDeletion(this);
	}

	Tools_Overlay_MaterialResponseOverrides* m_pxOverrides;
};


////////////////////////////////////////////////////////////////////////////////
// Class		: Tools_Overlay_MaterialResponseOverridesFilters
// Author       : Mark Parry
// Date         : 04-Jul-2007
////////////////////////////////////////////////////////////////////////////////

class FilterStringArray
{
public:
	FilterStringArray();
	~FilterStringArray();

	bool Allocate( const int iSize );
	void Free( void );
	bool Add( const char* const szString, const bool bCheckAlreadyAdded = true );
	const char* GetEntry( const int iEntry, const char* const szInvalid = "" );
	bool IsValid( void )					const	{ return( m_bValid ); }
	const char** GetArray( void )			const	{ return( (const char**)m_pszStrings ); }
	int GetCount( void )					const	{ return( m_iIndex ); }
	int GetSize( void )						const	{ return( m_iSize); }
	int Find( const char* const szString, const int iStartIndex = 0 )	const;

private:
	int m_iSize;
	int m_iIndex;
	char** m_pszStrings;
	bool m_bValid;
};

FilterStringArray::FilterStringArray()
: m_iSize( 0 )
, m_iIndex( 0 )
, m_pszStrings( NULL )
, m_bValid( false )
{
}

FilterStringArray::~FilterStringArray()
{
	Free();
}

bool FilterStringArray::Allocate( const int iSize )
{
	Free();
	if( iSize <= 0 )
	{
		return( false );
	}
	m_pszStrings = new char*[ iSize ];
	if( m_pszStrings == NULL )
	{
		return( false );
	}
	m_iSize		= iSize;
	m_iIndex	= 0;
	m_bValid	= true;
	return( true );
}

void FilterStringArray::Free( void )
{
	if( m_pszStrings != NULL )
	{
		for( int i = 0; i < m_iIndex; i++ )
		{
			delete [] m_pszStrings[ i ];
		}
		delete [] m_pszStrings;
		m_pszStrings = NULL;
	}
	m_iSize		= 0;
	m_iIndex	= 0;
	m_bValid	= false;
}
bool FilterStringArray::Add( const char* const szString, const bool bCheckAlreadyAdded /* = true  */ )
{
	if( !m_bValid )
	{
		return( false );
	}
	if( ( szString == NULL ) || ( szString[ 0 ] == 0 ) )
	{
		//nothing to add
		return( true );
	}
	if( bCheckAlreadyAdded )
	{
		for( int i = 0; i < m_iIndex; i++ )
		{
			if( stricmp( szString, m_pszStrings[ i ] ) == 0 )
			{
				//already added
				return( true );
			}
		}
	}
	m_pszStrings[ m_iIndex ] = new char[ strlen( szString ) + 1 ];
	if( m_pszStrings[ m_iIndex ] == NULL )
	{
		m_bValid = false;
		return( false );
	}
	strcpy( m_pszStrings[ m_iIndex ], szString );
	m_iIndex++;
	return( true );
}

const char* FilterStringArray::GetEntry( const int iEntry, const char* const szInvalid /* = "" */ )
{
	if( ( iEntry < 0 ) || ( iEntry >= m_iIndex ) )
	{
		return( szInvalid );
	}
	return( m_pszStrings[ iEntry ] );
}

int FilterStringArray::Find( const char* const szString, const int iStartIndex /*= 0*/ ) const
{
	if( ( szString == NULL ) || ( szString[ 0 ] == 0 ) )
	{
		return( -1 );
	}
	for( int i = iStartIndex; i < m_iIndex; i++ )
	{
		if( stricmp( szString, m_pszStrings[ i ] ) == 0 )
		{
			//already added
			return( i );
		}
	}
	return( -1 );
}

////////////////////////////////////////////////////////////////////////////////

class Tools_Overlay_MaterialResponseOverridesFilter
{
public:
	Tools_Overlay_MaterialResponseOverridesFilter();
	bool Init( Asura_Overlay* const pxOverlay, Asura_Tabs_Dialog* const pxTabsDialog, Asura_Vector_2 &xOffset, void (*pfnChanged)( int iUserData ), const int iUserData, bool( *pfnIsEnabled )( void ) );
	bool HasChanged( void )				const	{ return( m_iIndex != m_iLastIndex ); }
	void ResetChanged( void )					{ m_iLastIndex = m_iIndex; }
	const char* GetFilter( void );
	bool Populate( const int iNumMaterials, const char* const* pszMaterialNameArray, const char* const szPrefix );
	void Clear( void );
	int FindFilter( const char* const szFilters )	const;
	void SetIndex( const int iIndex );
	void SetIndex( const char* const szFilters );
 
private:
	Asura_DropDown_List_Dialog* m_pxDropDownList;
	int m_iIndex;
	int m_iLastIndex;
	FilterStringArray m_xFiltersStringArray;

	static const int s_iDefaultListSize;
	static const char* const s_pszDefaultList[];
};

const char* const Tools_Overlay_MaterialResponseOverridesFilter::s_pszDefaultList[] =
{
	"All",
};
const int Tools_Overlay_MaterialResponseOverridesFilter::s_iDefaultListSize = sizeof( s_pszDefaultList ) / sizeof( s_pszDefaultList [ 0 ]);

Tools_Overlay_MaterialResponseOverridesFilter::Tools_Overlay_MaterialResponseOverridesFilter()
: m_pxDropDownList( NULL )
, m_iIndex( 0 )
, m_iLastIndex( -1 )
{
}

bool Tools_Overlay_MaterialResponseOverridesFilter::Init( Asura_Overlay* const pxOverlay, Asura_Tabs_Dialog* const pxTabsDialog, Asura_Vector_2 &xOffset, void (*pfnChanged)( int iUserData ), const int iUserData, bool( *pfnIsEnabled )( void ) )
{
	//add the filter drop-downs
	m_pxDropDownList = new Asura_DropDown_List_Dialog( xOffset, s_pszDefaultList, &m_iIndex, 10, s_iDefaultListSize, NULL, NULL, NULL, true );
	if( m_pxDropDownList == NULL )
	{
		return( false );
	}
	xOffset.y += Asura_Dialog::DefaultSpacing();
	if( pxOverlay != NULL )
	{
		pxOverlay->AddDialog( m_pxDropDownList );
	}
	if( pxTabsDialog != NULL )
	{
		pxTabsDialog->AddDialog( m_pxDropDownList );
	}
	m_pxDropDownList->SetOnSelectCallback( pfnChanged, iUserData );
	m_pxDropDownList->SetIsEnabledFunction( pfnIsEnabled );
	return( true );
}
const char* Tools_Overlay_MaterialResponseOverridesFilter::GetFilter( void )
{
	if( m_iIndex == 0 )
	{
		return( "" );
	}
	return( m_xFiltersStringArray.GetEntry( m_iIndex, "" ) );
}

bool Tools_Overlay_MaterialResponseOverridesFilter::Populate( const int iNumMaterials, const char* const* pszMaterialNameArray, const char* const szPrefix )
{
	Clear();

	int i;
	FilterStringArray xFiltersStringArray;
	if( ( iNumMaterials > 0 ) && ( pszMaterialNameArray != NULL ) )
	{
		xFiltersStringArray.Allocate( iNumMaterials );
		for( i = 0; i < iNumMaterials; i++ )
		{
			//copy up to the first slash
			const char* szMaterialName = pszMaterialNameArray[ i ];

			//does the material name have to have a specific prefix?
			if( ( szPrefix != NULL) && ( szPrefix[ 0 ] != 0 ) )
			{
				if( !Asura_StringUtil::StartsWith( szMaterialName, szPrefix, true ) )
				{	
					//ignore this material, the prefix does not match
					continue;
				}
				szMaterialName += strlen( szPrefix );
				if( ( szMaterialName[ 0 ] != '\\' ) && ( szMaterialName[ 0 ] != '/' ) )
				{
					continue;
				}
				while( ( szMaterialName[ 0 ] == '\\' ) || ( szMaterialName[ 0 ] == '/' ) )
				{
					szMaterialName++;
				}
			}

			Asura_String xFilter;
			while( ( szMaterialName[ 0 ] != 0 ) && ( szMaterialName[ 0 ] != '\\' ) && ( szMaterialName[ 0 ] != '/' ) )
			{
				xFilter += szMaterialName[ 0 ];
				szMaterialName++;
			}
			if( ( szMaterialName[ 0 ] != '\\' ) && ( szMaterialName[ 0 ] != '/' ) )
			{
				continue;
			}

			if( xFilter.GetLength() > 0 )
			{
				xFiltersStringArray.Add( xFilter.GetString(), true );
			}
		}
		if( !xFiltersStringArray.IsValid() )
		{
			return( false );
		}
	}

	const int iFilterCount = xFiltersStringArray.GetCount();
	m_xFiltersStringArray.Allocate( s_iDefaultListSize + iFilterCount );
	for( i = 0; i < s_iDefaultListSize; i++ )
	{
		m_xFiltersStringArray.Add( s_pszDefaultList[ i ] );
	}
	for( i = 0; i < iFilterCount; i++ )
	{
		m_xFiltersStringArray.Add( xFiltersStringArray.GetEntry( i ) );
	}

	if( !m_xFiltersStringArray.IsValid() )
	{
		m_xFiltersStringArray.Free();
		return( false );
	}

	m_pxDropDownList->ReplaceStrings( m_xFiltersStringArray.GetCount(), m_xFiltersStringArray.GetArray() );
	return( true );
}

void Tools_Overlay_MaterialResponseOverridesFilter::Clear( void )
{
	m_pxDropDownList->SelectEntry( 0 );
	m_pxDropDownList->ReplaceStrings( s_iDefaultListSize, s_pszDefaultList );
	m_xFiltersStringArray.Free();
}

int Tools_Overlay_MaterialResponseOverridesFilter::FindFilter( const char* const szFilters ) const
{
	if( ( szFilters == NULL ) || ( szFilters[ 0 ] == 0 ) )
	{
		return( 0 );
	}
	return( m_xFiltersStringArray.Find( szFilters, 1 ) );
}
void Tools_Overlay_MaterialResponseOverridesFilter::SetIndex( const int iIndex )
{
	if( ( iIndex >= 0 ) && ( iIndex < m_xFiltersStringArray.GetCount() ) )
	{
		m_iIndex = iIndex;
	}
}
void Tools_Overlay_MaterialResponseOverridesFilter::SetIndex( const char* const szString )
{
	SetIndex( FindFilter( szString ) );
}

////////////////////////////////////////////////////////////////////////////////

class Tools_Overlay_MaterialResponseOverridesFilters
{
public:
	Tools_Overlay_MaterialResponseOverridesFilters( Asura_Overlay* const pxOverlay, Asura_Tabs_Dialog* const pxTabsDialog, Asura_Vector_2 &xOffset, void( *pfnChanged )( void* const pxUserData ), void* const pxChangedUserData, bool( *pfnIsEnabled )( void ) = NULL );

	bool PopulateFilters( const bool bForce = false );
	bool MatchesFilters( const char* const szString );
	bool SetDefaults( Tools_Overlay_MaterialResponseOverridesFilters* const pxOverridesFilters );

private:

	static void Changed( int iUserData );
	void Changed( void );
	void( *m_pfnChanged )( void* const pxUserData );
	void* const m_pxChangedUserData;

	Tools_Overlay_MaterialResponseOverridesFilter m_xFilterList1;
	Tools_Overlay_MaterialResponseOverridesFilter m_xFilterList2;
};

Tools_Overlay_MaterialResponseOverridesFilters::Tools_Overlay_MaterialResponseOverridesFilters( Asura_Overlay* const pxOverlay, Asura_Tabs_Dialog* const pxTabsDialog, Asura_Vector_2 &xOffset, void( *pfnChanged )( void* const pxUserData ), void* const pxChangedUserData, bool( *pfnIsEnabled )( void ) /* = NULL */ )
: m_pfnChanged( pfnChanged )
, m_pxChangedUserData( pxChangedUserData )
{
	m_xFilterList1.Init( pxOverlay, pxTabsDialog, xOffset, Changed, (int)this, pfnIsEnabled );
	xOffset.y	+= Asura_Dialog::DefaultSpacing();
	m_xFilterList2.Init( pxOverlay, pxTabsDialog, xOffset, Changed, (int)this, pfnIsEnabled );

	PopulateFilters( true );
}

void Tools_Overlay_MaterialResponseOverridesFilters::Changed( int iUserData )
{
	Tools_Overlay_MaterialResponseOverridesFilters* pxThis = (Tools_Overlay_MaterialResponseOverridesFilters*)iUserData;
	if( pxThis->PopulateFilters() )
	{
		if( pxThis->m_pfnChanged != NULL )
		{
			pxThis->m_pfnChanged( pxThis->m_pxChangedUserData );
		}
	}
}

bool Tools_Overlay_MaterialResponseOverridesFilters::PopulateFilters( const bool bForce /* = false  */)
{
	const bool bChanged1 = m_xFilterList1.HasChanged();
	const bool bChanged2 = m_xFilterList2.HasChanged();

	if( !bForce )
	{
		if ( !bChanged1 )
		{
			// just flag to re-populate the list if the second list has changed
			m_xFilterList2.ResetChanged();
			return( bChanged2 );
		}
	}

	int iNumMaterials = 0;
	const char** pszMaterialNameArray = NULL;
	Tools_MaterialResponse_System::GetMaterialNames( iNumMaterials, pszMaterialNameArray );

	if( bForce )
	{
		m_xFilterList1.Populate( iNumMaterials, pszMaterialNameArray, "" );
	}
	if( ( bForce ) || ( bChanged1 ) )
	{
		m_xFilterList2.Populate( iNumMaterials, pszMaterialNameArray, m_xFilterList1.GetFilter() );
	}

	m_xFilterList1.ResetChanged();
	m_xFilterList2.ResetChanged();

	delete [] pszMaterialNameArray;
	return( true );
}

bool Tools_Overlay_MaterialResponseOverridesFilters::MatchesFilters( const char* const szString )
{
	const char* sz = ( szString == NULL ) ? "" : szString;
	const char* szFilter = m_xFilterList1.GetFilter();
	if( ( szFilter != NULL ) && ( szFilter[ 0 ] != 0 ) )
	{
		if( !Asura_StringUtil::StartsWith( sz, szFilter, true ) )
		{	
			return( false );
		}
		sz += strlen( szFilter );
		if( ( sz[ 0 ] != '\\' ) && ( sz[ 0 ] != '/' ) )
		{
			return( false );
		}
		while( ( sz[ 0 ] == '\\' ) || ( sz[ 0 ] == '/' ) )
		{
			sz++;
		}
	}

	szFilter = m_xFilterList2.GetFilter();
	if( ( szFilter != NULL ) && ( szFilter[ 0 ] != 0 ) )
	{
		if( !Asura_StringUtil::StartsWith( sz, szFilter, true ) )
		{	
			return( false );
		}
		sz += strlen( szFilter );
		if( ( sz[ 0 ] != '\\' ) && ( sz[ 0 ] != '/' ) )
		{
			return( false );
		}
	}
	return( true );
}

bool Tools_Overlay_MaterialResponseOverridesFilters::SetDefaults( Tools_Overlay_MaterialResponseOverridesFilters* const pxOverridesFilters )
{
	if( pxOverridesFilters == NULL )
	{
		return( false );
	}

	bool bChanged = false;
	m_xFilterList1.SetIndex( pxOverridesFilters->m_xFilterList1.GetFilter() );
	if( PopulateFilters() )
	{
		bChanged = true;
	}
	m_xFilterList2.SetIndex( pxOverridesFilters->m_xFilterList2.GetFilter() );
	if( PopulateFilters() )
	{
		bChanged = true;
	}
	if( bChanged )
	{
		if( m_pfnChanged != NULL )
		{
			m_pfnChanged( m_pxChangedUserData );
		}
	}
	return( true );
}

////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Overlay_MaterialResponseOverrides::Tools_Overlay_MaterialResponseOverrides
// Author       : Mark Parry
// Date         : 09-July-2007
////////////////////////////////////////////////////////////////////////////////

class SelectCurrentOverrideDialog : public Asura_Function_Dialog
{
public:
	SelectCurrentOverrideDialog(
		Tools_Overlay_MaterialResponseOverrides* const pxParent,
		const Asura_Vector_2& xOffset,
		void(*pfnCallback)(int),
		int iUserData,
		bool(*pfnIsEnabled)(void) );

	void Update( const Asura_Vector_2& xOrigin );

private:
	typedef Asura_Function_Dialog PARENT;

	Tools_Overlay_MaterialResponseOverrides* const m_pxParent;
};

SelectCurrentOverrideDialog::SelectCurrentOverrideDialog(
							Tools_Overlay_MaterialResponseOverrides* const pxParent,
							const Asura_Vector_2& xOffset,
							void(*pfnCallback)(int),
							int iUserData,
							bool(*pfnIsEnabled)(void) )
: m_pxParent( pxParent )
, Asura_Function_Dialog( "", xOffset, pfnCallback, iUserData, NULL, pfnIsEnabled )
{
}

void SelectCurrentOverrideDialog::Update( const Asura_Vector_2& xOrigin )
{
	if( m_pxParent != NULL )
	{
		const Asura_Hash_ID	uCurrentHash		= Asura_MaterialResponse_System::GetCurrentOverrideMapHash();
		const int			iCurrentIndex		= m_pxParent->GetIndexForOverride( uCurrentHash );
		const char*			szCurrentOverride	= ( iCurrentIndex >= 0 ) ? m_pxParent->GetOverrideNameArray()[ iCurrentIndex ] : "*** None ***";
		char				szText[256];
		Asura_PrintfToBfr( szText, "Select Current Override: %s", 256, true, szCurrentOverride );
		UpdateLabel( szText );
	}

	PARENT::Update( xOrigin );
}

Tools_Overlay_MaterialResponseOverrides* Tools_Overlay_MaterialResponseOverrides::m_pxFirst = NULL;

Tools_Overlay_MaterialResponseOverrides::Tools_Overlay_MaterialResponseOverrides()
: m_pxOverlay( NULL )
, m_pxTabsDialog( NULL )
, m_pxTableDialog( NULL )
, m_pbEnableTableRows( NULL )
, m_pxNoOverridesDialog( NULL )
, m_pxCurrentOverrideDialog( NULL )
, m_xOverlayPosition( 6.0f, 6.0f )
, m_pszMaterialNameArray( NULL )
, m_iNumMaterials( 0 )
, m_pxMaterialHeadingArray( NULL )
, m_puMaterialHashArray( NULL )
, m_pszOverrideNameArray( NULL )
, m_pxOverrideHeadingArray( NULL )
, m_puOverrideHashArray( NULL )
, m_iNumOverrides( 0 )
, m_pxOverridesFilters( NULL )
, m_iTabsDialogMaterialIndex( -1 )
, m_bOnlyShowMaterialsWithAnOverride( false )
, m_bLastOnlyShowMaterialsWithAnOverride( false )
{
	m_pxNext = m_pxFirst;
	m_pxFirst = this;
}

Tools_Overlay_MaterialResponseOverrides::~Tools_Overlay_MaterialResponseOverrides()
{
	Tools_Overlay_MaterialResponseOverrides** ppx = &m_pxFirst;
	while( *ppx != NULL )
	{
		Tools_Overlay_MaterialResponseOverrides* px = *ppx;
		if( px == this )
		{
			//remove it from the chain
			*ppx = px->m_pxNext;
			break;
		}
		ppx = &px->m_pxNext;
	}

	// Close the dialog
	Close();
}

////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Overlay_MaterialResponseOverrides::CreateOverlay
// Author       : Andy Tate
// Date         : 26-Apr-2007
////////////////////////////////////////////////////////////////////////////////

void Tools_Overlay_MaterialResponseOverrides::CreateOverlay(void)
{
	if( m_pxOverlay == NULL )
	{
		Asura_Assert( ( m_pxTabsDialog == NULL ), "" );
		Asura_Vector_2	xOffset(1.0f, 3.0f);
		Asura_Dialog* pxNewDialog;
		Asura_Tickbox_Dialog* pxNewTickbox;

		// Store the overlay...
		m_pxOverlay		= new Tools_Overlay_MaterialResponseOverrides_Overlay(this, "Material Response Overrides", false, &m_xOverlayPosition);
		m_pxOverlay->m_uID	= TOOLS_OVERLAY_MATERIAL_RESPONSE_OVERRIDES;

		// Only show materials with an override?
		pxNewTickbox = new Asura_Tickbox_Dialog( "Only show materials with an override", xOffset, &m_bOnlyShowMaterialsWithAnOverride );
		pxNewTickbox->SetClickedFunction( OnlyShowMaterialsWithAnOverrideClicked );
		pxNewTickbox->SetIsEnabledFunction( AreAnyOverridesPresent );
		AddDialog( pxNewTickbox );
		xOffset.y = pxNewTickbox->m_xMaxCoord.y + Asura_Tickbox_Dialog::DefaultSpacing();

		// Filters...
		m_pxOverridesFilters = new Tools_Overlay_MaterialResponseOverridesFilters( m_pxOverlay, m_pxTabsDialog, xOffset, FiltersChanged, this, AreAnyOverridesPresent );
		xOffset.y += Asura_Dialog::DefaultSpacing();

		AddDialogs( xOffset, g_xTableSize );

		// Save
		pxNewDialog = new Asura_Function_Dialog( "Save", xOffset, Save);
		AddDialog( pxNewDialog );
		xOffset.y += Asura_Dialog::DefaultSpacing();

		// Save and close...
		pxNewDialog = new Asura_Function_Dialog("Save + Close", xOffset, SaveAndClose, (int)this );
		AddDialog( pxNewDialog );
		xOffset.y		+= Asura_Dialog::DefaultSpacing();

		pxNewDialog = new Asura_Resize_Dialog(m_pxTableDialog, &g_xTableSize);
		AddDialog(pxNewDialog);

		// Finish off...
		m_pxOverlay->MakeDialogsRelativeToResizableDialog();
		Asura_Overlay_System::SubmitNewOverlay(m_pxOverlay);
	}
}

void Tools_Overlay_MaterialResponseOverrides::TabsDialog( const char* const szMaterialName, Asura_Tabs_Dialog* const pxTabsDialog, Asura_Vector_2& xOffset, const Asura_Vector_2& xTableSize )
{
	Asura_Dialog* pxNewDialog;

	Asura_Assert( ( m_pxTabsDialog == NULL ), "" );
	Asura_Assert( ( m_pxOverlay == NULL ), "" );
	m_iTabsDialogMaterialIndex = -1;
	m_pxTabsDialog = pxTabsDialog;
	AddDialogs( xOffset, xTableSize );

	// Save
	pxNewDialog = new Asura_Function_Dialog( "Save", xOffset, Save);
	AddDialog( pxNewDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	for( int i = 0; i < m_iNumMaterials; i++ )
	{
		if( stricmp( m_pszMaterialNameArray[ i ], szMaterialName ) == 0 )
		{
			m_iTabsDialogMaterialIndex = i;
		}
	}
	ApplyFilters();
}

void Tools_Overlay_MaterialResponseOverrides::AddDialogs( Asura_Vector_2& xOffset, const Asura_Vector_2& xTableSize )
{
	Asura_Dialog*	pxNewDialog;

	// Table...
	m_pxNoOverridesDialog	= new Asura_StringDisplay_Dialog( "There are currently no material-response overrides.", xOffset );
	AddDialog( m_pxNoOverridesDialog );

	m_pxTableDialog	= new Asura_Table_Dialog( xOffset, xTableSize.x, xTableSize.y, false );
	m_pxTableDialog->SetIsRowEnabledCallBack( IsTableRowEnabledCallBack, this );
	m_pxTableDialog->SetGetColourCallBack( TableGetColourCallBack, this );
	AddDialog( m_pxTableDialog );
	CreateTable();
	xOffset.y = m_pxTableDialog->m_xMaxCoord.y + Asura_Dialog::DefaultSpacing();


	// Other dialogs...
	pxNewDialog	 = new Asura_Function_Dialog( "Create New Override", xOffset, CreateNewOverride, (int)this );
	AddDialog( pxNewDialog );
	xOffset.y		+= Asura_Dialog::DefaultSpacing();

	m_pxCurrentOverrideDialog	 = new SelectCurrentOverrideDialog( this, xOffset, SelectCurrentOverride, (int)this, AreAnyOverridesPresent );
	AddDialog( m_pxCurrentOverrideDialog );
	xOffset.y		+= Asura_Dialog::DefaultSpacing();

	pxNewDialog	= new Asura_Function_Dialog( "Delete Current Override", xOffset, DeleteCurrentOverride, (int)this, NULL, IsCurrentOverrideValid );
	AddDialog( pxNewDialog );
	xOffset.y		+= Asura_Dialog::DefaultSpacing();

	pxNewDialog	= new Asura_Function_Dialog( "Copy Current Override", xOffset, CopyCurrentOverride, (int)this, NULL, IsCurrentOverrideValid );
	AddDialog( pxNewDialog );
	xOffset.y		+= Asura_Dialog::DefaultSpacing();

	pxNewDialog	= new Asura_Function_Dialog( "Rename Current Override", xOffset, RenameCurrentOverride, (int)this, NULL, IsCurrentOverrideValid );
	AddDialog( pxNewDialog );
	xOffset.y		+= Asura_Dialog::LargeSpacing();
}

void Tools_Overlay_MaterialResponseOverrides::AddDialog( Asura_Dialog* const pxDialog )
{
	if( m_pxOverlay != NULL )
	{
		m_pxOverlay->AddDialog( pxDialog );
	}
	if( m_pxTabsDialog != NULL )
	{
		m_pxTabsDialog->AddDialog( pxDialog );
	}
}

/// End of function 'Tools_Overlay_MaterialResponseOverrides::CreateOverlay'. ///

void Tools_Overlay_MaterialResponseOverrides::OnlyShowMaterialsWithAnOverrideClicked( void )
{
	Tools_Overlay_MaterialResponseOverrides* px = m_pxFirst;
	while( px != NULL )
	{
		if( px->m_bOnlyShowMaterialsWithAnOverride != px->m_bLastOnlyShowMaterialsWithAnOverride )
		{
			px->ApplyFilters();
		}
		px = px->m_pxNext;
	}
}

void Tools_Overlay_MaterialResponseOverrides::FiltersChanged( void* const pxUserData )
{
	Tools_Overlay_MaterialResponseOverrides* pxThis = (Tools_Overlay_MaterialResponseOverrides*)pxUserData;
	pxThis->ApplyFilters();
}

void Tools_Overlay_MaterialResponseOverrides::ApplyFilters( void )
{
	delete [] m_pbEnableTableRows;
	m_pbEnableTableRows = NULL;
	if ( m_pxTableDialog != NULL )
	{
		const u_int uRows			= m_pxTableDialog->GetNumRows();
		const u_int uNumMaterials	= (u_int)m_iNumMaterials;
		if( uRows == uNumMaterials )
		{
			if( m_iTabsDialogMaterialIndex != -1 )
			{
				m_pbEnableTableRows = new bool[ uRows ];
				if( m_pbEnableTableRows != NULL )
				{
					for( u_int uRow = 0; uRow < uRows; uRow++ )
					{
						m_pbEnableTableRows[ uRow ] = ( uRow == (u_int)m_iTabsDialogMaterialIndex );
					}
				}
			}
			else if( ( m_pxOverridesFilters != NULL ) && ( m_pszMaterialNameArray != NULL ) )
			{
				m_pbEnableTableRows = new bool[ uRows ];
				if( m_pbEnableTableRows != NULL )
				{
					for( u_int uRow = 0; uRow < uRows; uRow++ )
					{
						bool bEnableRow = m_pxOverridesFilters->MatchesFilters( m_pszMaterialNameArray[ uRow ] );
						if( ( bEnableRow ) && ( m_bOnlyShowMaterialsWithAnOverride ) )
						{
							bEnableRow = DoesMaterialHaveOverride( m_puMaterialHashArray[ uRow ] );
						}
						m_pbEnableTableRows[ uRow ] = bEnableRow;
					}
				}
			}
		}
	}
	m_bLastOnlyShowMaterialsWithAnOverride = m_bOnlyShowMaterialsWithAnOverride;
}

bool Tools_Overlay_MaterialResponseOverrides::DoesMaterialHaveOverride( const Asura_Hash_ID uMaterialHash )
{
	for( int i = 0; i < m_iNumOverrides; i++ )
	{
		Asura_MaterialResponse_OverrideMap* pxOverrideMap = Asura_MaterialResponse_System::GetOverrideMap( m_puOverrideHashArray[ i ] );
		if( pxOverrideMap != NULL )
		{
			if( pxOverrideMap->GetOverride( uMaterialHash ) )
			{
				return( true );
			}
		}
	}
	return( false );
}

bool Tools_Overlay_MaterialResponseOverrides::IsTableRowEnabledCallBack( const u_int uRow, void* const pxUserData )
{
	Tools_Overlay_MaterialResponseOverrides* pxThis = (Tools_Overlay_MaterialResponseOverrides*)pxUserData;
	if( ( pxThis->m_pbEnableTableRows != NULL ) && ( uRow < (u_int)pxThis->m_iNumMaterials ) )
	{
		return( pxThis->m_pbEnableTableRows[ uRow ] );
	}
	return( true );
}

unsigned Tools_Overlay_MaterialResponseOverrides::TableGetColourCallBack( const Asura_Table_Dialog& xTableDialog, void* const pxUserData,
																		  const Asura_Dialog* const pxDialog, const unsigned uColour,
																		  const u_int uRow, const u_int uColumn )
{
	Tools_Overlay_MaterialResponseOverrides* pxThis = (Tools_Overlay_MaterialResponseOverrides*)pxUserData;
	return( pxThis->ChangeTableColumnColour( uColour, uColumn ) );
}

unsigned Tools_Overlay_MaterialResponseOverrides::ChangeTableColumnColour( const unsigned uColour, const u_int uColumn )
{
	if( ( m_puOverrideHashArray != NULL ) &&
		( uColumn < (u_int)m_iNumOverrides ) &&
		( m_puOverrideHashArray[ uColumn ] == Asura_MaterialResponse_System::GetCurrentOverrideMapHash() ) )
	{
		return( uColour & 0xffffff00 );		// make it yellow
	}
	return( uColour );
}

////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Overlay_MaterialResponseOverrides::SaveAndClose
// Author       : Andy Tate
// Date         : 26-Apr-2007
////////////////////////////////////////////////////////////////////////////////

void Tools_Overlay_MaterialResponseOverrides::SaveAndClose( void )
{
	// Save the file
	Save();
	// Close the dialog
	Close();
}

void Tools_Overlay_MaterialResponseOverrides::Save( void )
{
	// Save the file to disc...
	if( Tools_Chunk_MaterialResponseOverrides::Save( g_szMaterialResponseOverridesFilename ) )
	{
		s_bModified = false;

		Asura_String	xMessage("Saved \"");

		xMessage	+= g_szMaterialResponseOverridesFilename;
		xMessage	+= "\" successfully.";

		Tools_Timed_Message_System::AddMessage( 0x00FF00, xMessage.GetString() );
	}
	else
	{
		Asura_String	xMessage( "Failed to save \"");

		xMessage	+= g_szMaterialResponseOverridesFilename;
		xMessage	+= "\" - write-protected?";

		Tools_Timed_Message_System::AddMessage( 0xFF0000, xMessage.GetString() );
	}
}

void Tools_Overlay_MaterialResponseOverrides::Close( void )
{
	delete m_pxOverridesFilters;
	m_pxOverridesFilters = NULL;

	// Destroy our existing table...
	DestroyTable();

	// Close the window...
	if( m_pxOverlay != NULL )
	{
		m_pxOverlay->m_bShouldClose = true;
	}

	// Flag as closed...
	m_pxCurrentOverrideDialog	= NULL;
	m_pxTableDialog				= NULL;
	m_pxNoOverridesDialog		= NULL;
	m_pxOverlay					= NULL;
}

void Tools_Overlay_MaterialResponseOverrides::SetModified()
{
	if(!s_bModified)
	{
		Tools_SaveAsura::WarnIfReadOnly(g_szMaterialResponseOverridesFilename);
		s_bModified =  true;
	}
}

void Tools_Overlay_MaterialResponseOverrides::OnOverlayDeletion(Asura_Overlay* pxOverlay)
{
	//If the system still has a pointer to the overlay , then it hasn't cleared things up yet
	//and so needs to do SaveAndClose() to save and deallocate everything.
	//RAR 15/11/2007
	if(m_pxOverlay)
	{
		Asura_Assert(pxOverlay == m_pxOverlay, "Not deleting the expected overlay");
		SaveAndClose();
	}
}

/// End of function 'Tools_Overlay_MaterialResponseOverrides::SaveAndClose'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Overlay_MaterialResponseOverrides::SelectCurrentOverride
// Author       : Andy Tate
// Date         : 3-May-2007
////////////////////////////////////////////////////////////////////////////////

void Tools_Overlay_MaterialResponseOverrides::SelectCurrentOverride(void)
{
	const char* const*								pszOverrideNameArray	= GetOverrideNameArray();
	const Asura_Hash_ID*							puOverrideHashArray		= GetOverrideHashArray();
	const int										iNumOverrides			= GetNumOverrides();

	const Asura_Hash_ID								uCurrentHash			= Asura_MaterialResponse_System::GetCurrentOverrideMapHash();
	const int										iOverrideIndex			= GetIndexForOverride(uCurrentHash, 0);

	Tools_Modal_Overlay_SelectCurrentOverrideMap	xSelectionOverlay("Select current override", iOverrideIndex, pszOverrideNameArray, iNumOverrides);

	if(xSelectionOverlay.DoModal())
	{
		const int	iNewOverrideIndex	= xSelectionOverlay.GetOverrideIndex();

		if(iNewOverrideIndex >= 0)
		{
			const Asura_Hash_ID	uNewOverrideHash	= puOverrideHashArray[iNewOverrideIndex];
			Asura_MaterialResponse_System::SetCurrentOverrideMap(uNewOverrideHash);
		}
		else
		{
			Asura_MaterialResponse_System::ResetCurrentOverrideMap();
		}
	}
}

/// End of function 'Tools_Overlay_MaterialResponseOverrides::SelectCurrentOverride'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Overlay_MaterialResponseOverrides::DeleteCurrentOverride
// Author       : Andy Tate
// Date         : 3-May-2007
////////////////////////////////////////////////////////////////////////////////

void Tools_Overlay_MaterialResponseOverrides::DeleteCurrentOverride( void )
{
	const Asura_Hash_ID	uCurrentHash	= Asura_MaterialResponse_System::GetCurrentOverrideMapHash();
	const int			iOverrideIndex	= GetIndexForOverride(uCurrentHash, -1);

	if(iOverrideIndex >= 0)
	{
		const char*		szOverrideName	= m_pszOverrideNameArray[iOverrideIndex];
		Asura_String	xMessageString("Are you sure you want to delete the \"");

		xMessageString	+= szOverrideName;
		xMessageString	+= "\" override?";

		{
			YesNo_Overlay	xSelectionOverlay(xMessageString);

			if(xSelectionOverlay.DoModal())
			{
				// Remove the override map...
				Asura_MaterialResponse_System::ResetCurrentOverrideMap();
				Asura_MaterialResponse_System::DestroyOverrideMap(uCurrentHash);

				// Refresh the columns...
				Tools_Overlay_MaterialResponseOverrides::RefreshAll();
				SetModified();

			}
		}
	}
	else
	{
		Asura_Assert(false, "Cannot find index for specified override.");
	}
}

/// End of function 'Tools_Overlay_MaterialResponseOverrides::DeleteCurrentOverride'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Overlay_MaterialResponseOverrides::CopyCurrentOverride
// Author       : Mark Parry
// Date         : 10-July-2007
////////////////////////////////////////////////////////////////////////////////

void Tools_Overlay_MaterialResponseOverrides::CopyCurrentOverride( void )
{
	const Asura_Hash_ID	uCurrentHash	= Asura_MaterialResponse_System::GetCurrentOverrideMapHash();
	const int			iOverrideIndex	= GetIndexForOverride( uCurrentHash, -1 );
	if( iOverrideIndex < 0 )
	{
		Asura_Assert(false, "Cannot find index for specified override.");
		return;
	}
	const Asura_MaterialResponse_OverrideMap* const pxCurrentOverrideMap = Asura_MaterialResponse_System::GetOverrideMap( uCurrentHash );

	bool bAwaitingValidOverride	= true;
	while( bAwaitingValidOverride )
	{
		Asura_String xTitle;
		xTitle += "Copy '";
		xTitle += m_pszOverrideNameArray[ iOverrideIndex ];
		xTitle += "' as...";
		Asura_String xOverrideName = GetOverrideName();

		GetString_Overlay xOverlay( xTitle.GetString(), xOverrideName.GetString() );
		if( xOverlay.DoModal() )
		{
			const char*	szNewOverrideMapName = xOverlay.GetString();
			Asura_Hash_ID uNewOverrideMapNameHash = Asura_GetHashID( szNewOverrideMapName );

            // Check if there's already some other override map with this
            // name.
			Asura_MaterialResponse_OverrideMap*	pxNewOverrideMap = Asura_MaterialResponse_System::GetOverrideMap( uNewOverrideMapNameHash );
			if( pxNewOverrideMap == NULL )
			{
				bAwaitingValidOverride	= false;

                const u_int uNumOverrides = pxCurrentOverrideMap->GetNumOverrides();

				pxNewOverrideMap = Asura_MaterialResponse_System::CreateOverrideMap( uNewOverrideMapNameHash, uNumOverrides );
				pxNewOverrideMap->SetName( szNewOverrideMapName );

                // Copy each override across.
                Asura_Hash_ID* puMaterialHashIDsToOverride = pxCurrentOverrideMap->CreateMaterialHashIDsToOverrideArray();
                for( u_int u = 0; u < uNumOverrides; ++u )
                {
                    const Asura_Hash_ID uMaterialHash = puMaterialHashIDsToOverride[u];
                    const Asura_MaterialResponse_Override* pxExistingOverride = pxCurrentOverrideMap->GetOverride( uMaterialHash );

                    Asura_MaterialResponse_Override* pxNewOverride = pxNewOverrideMap->CreateOverride( uMaterialHash );
					pxNewOverride->SetOverrideName( pxExistingOverride->GetOverrideName() );
					pxNewOverride->SetOverrideHashID( pxExistingOverride->GetOverrideHashID() );
                }
                pxCurrentOverrideMap->DestroyMaterialHashIDsToOverrideArray( puMaterialHashIDsToOverride );

				Tools_Overlay_MaterialResponseOverrides::RefreshAll();
				SetModified();
			}
		}
		else
		{
			bAwaitingValidOverride	= false;
		}
	}

}

/// End of function 'Tools_Overlay_MaterialResponseOverrides::CopyCurrentOverride'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Overlay_MaterialResponseOverrides::RenameCurrentOverride
// Author       : Mark Parry
// Date         : 10-July-2007
////////////////////////////////////////////////////////////////////////////////

void Tools_Overlay_MaterialResponseOverrides::RenameCurrentOverride( void )
{
	const Asura_Hash_ID	uCurrentHash	= Asura_MaterialResponse_System::GetCurrentOverrideMapHash();
	const int			iOverrideIndex	= GetIndexForOverride( uCurrentHash, -1 );
	if( iOverrideIndex < 0 )
	{
		Asura_Assert(false, "Cannot find index for specified override.");
		return;
	}

	const Asura_MaterialResponse_OverrideMap* const pxCurrentOverrideMap = Asura_MaterialResponse_System::GetOverrideMap( uCurrentHash );

	bool bAwaitingValidOverride	= true;
	while( bAwaitingValidOverride )
	{
		Asura_String xTitle;
		xTitle += "Rename '";
		xTitle += m_pszOverrideNameArray[ iOverrideIndex ];
		xTitle += "' to...";
		Asura_String xOverrideName = GetOverrideName();

		GetString_Overlay xOverlay( xTitle.GetString(), xOverrideName.GetString() );
		if( xOverlay.DoModal() )
		{
			const char*	szNewOverrideMapName = xOverlay.GetString();
			Asura_Hash_ID uNewOverrideMapNameHash = Asura_GetHashID( szNewOverrideMapName );

            // Check if there's already some other override map with this
            // name.
			Asura_MaterialResponse_OverrideMap*	pxNewOverrideMap = Asura_MaterialResponse_System::GetOverrideMap( uNewOverrideMapNameHash );
			if( pxNewOverrideMap == NULL )
			{
				bAwaitingValidOverride	= false;

                const u_int uNumOverrides = pxCurrentOverrideMap->GetNumOverrides();

				pxNewOverrideMap = Asura_MaterialResponse_System::CreateOverrideMap( uNewOverrideMapNameHash, uNumOverrides );
				pxNewOverrideMap->SetName( szNewOverrideMapName );

                // Copy each override across.
                Asura_Hash_ID* puMaterialHashIDsToOverride = pxCurrentOverrideMap->CreateMaterialHashIDsToOverrideArray();
                for( u_int u = 0; u < uNumOverrides; ++u )
                {
                    const Asura_Hash_ID uMaterialHash = puMaterialHashIDsToOverride[u];
                    const Asura_MaterialResponse_Override* pxExistingOverride = pxCurrentOverrideMap->GetOverride( uMaterialHash );

                    Asura_MaterialResponse_Override* pxNewOverride = pxNewOverrideMap->CreateOverride( uMaterialHash );
					pxNewOverride->SetOverrideName( pxExistingOverride->GetOverrideName() );
					pxNewOverride->SetOverrideHashID( pxExistingOverride->GetOverrideHashID() );
                }
                pxCurrentOverrideMap->DestroyMaterialHashIDsToOverrideArray( puMaterialHashIDsToOverride );

				Asura_MaterialResponse_System::SetCurrentOverrideMap( uNewOverrideMapNameHash );
				Asura_MaterialResponse_System::DestroyOverrideMap( uCurrentHash );
				Tools_Overlay_MaterialResponseOverrides::RefreshAll();
				SetModified();
			}
		}
		else
		{
			bAwaitingValidOverride	= false;
		}
	}

}

/// End of function 'Tools_Overlay_MaterialResponseOverrides::RenameCurrentOverride'. ///




////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Overlay_MaterialResponseOverrides::AreAnyOverridesPresent
// Author       : Andy Tate
// Date         : 3-May-2007
////////////////////////////////////////////////////////////////////////////////

bool Tools_Overlay_MaterialResponseOverrides::AreAnyOverridesPresent( void )
{
	return( Asura_MaterialResponse_System::GetNumOverrideMaps() > 0 );
}

/// End of function 'Tools_Overlay_MaterialResponseOverrides::AreAnyOverridesPresent'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Overlay_MaterialResponseOverrides::IsCurrentOverrideValid
// Author       : Andy Tate
// Date         : 3-May-2007
////////////////////////////////////////////////////////////////////////////////

bool Tools_Overlay_MaterialResponseOverrides::IsCurrentOverrideValid( void )
{
	const Asura_MaterialResponse_OverrideMap* const pxOverrideMap = Asura_MaterialResponse_System::GetCurrentOverrideMap();
	return( pxOverrideMap != NULL );
}

/// End of function 'Tools_Overlay_MaterialResponseOverrides::IsCurrentOverrideValid'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Overlay_MaterialResponseMaterials::GetIndexForMaterial
// Author       : Andy Tate
// Date         : 3-May-2007
////////////////////////////////////////////////////////////////////////////////

int Tools_Overlay_MaterialResponseOverrides::GetIndexForMaterial( const Asura_Hash_ID uHash, const int iDefaultIndex ) const
{
	for( int i = 0; i < m_iNumMaterials; i++ )
	{
		if( uHash == m_puMaterialHashArray[ i ] )
		{
			return( i );
		}
	}
	return( iDefaultIndex );
}

/// End of function 'Tools_Overlay_MaterialResponseMaterials::GetIndexForMaterial'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Overlay_MaterialResponseOverrides::GetIndexForOverride
// Author       : Andy Tate
// Date         : 3-May-2007
////////////////////////////////////////////////////////////////////////////////

int Tools_Overlay_MaterialResponseOverrides::GetIndexForOverride( const Asura_Hash_ID uHash, const int iDefaultIndex ) const
{
	for( int i = 0; i < m_iNumOverrides; i++ )
	{
		if( uHash == m_puOverrideHashArray[ i ] )
		{
			return( i );
		}
	}
	return( iDefaultIndex );
}

/// End of function 'Tools_Overlay_MaterialResponseOverrides::GetIndexForOverride'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Overlay_MaterialResponseOverrides::CreateNewOverride
// Author       : Andy Tate
// Date         : 3-May-2007
////////////////////////////////////////////////////////////////////////////////

Asura_String Tools_Overlay_MaterialResponseOverrides::GetOverrideName( void )
{
	Asura_MaterialResponse_OverrideMap*	pxOverrideMap	= NULL;
	Asura_Hash_ID						uOverrideHash;
	char szOverrideMapName[ 256 ];
	for( int i = 1; i < 1000; i++ )
	{
		Asura_PrintfToBfr( szOverrideMapName, "Override%d", sizeof( szOverrideMapName ), true, i );
		uOverrideHash	= Asura_GetHashID( szOverrideMapName );
		pxOverrideMap = Asura_MaterialResponse_System::GetOverrideMap( uOverrideHash );
		if( pxOverrideMap == NULL )
		{
			return( Asura_String( szOverrideMapName ) );
		}
	}
	return( Asura_String( "Override" ) );
}

void Tools_Overlay_MaterialResponseOverrides::CreateNewOverride( void )
{
	bool	bAwaitingValidOverride	= true;
	while( bAwaitingValidOverride )
	{
		Asura_String xOverrideName = GetOverrideName();
		GetString_Overlay xOverlay( "Enter override name", xOverrideName.GetString() );
		if( xOverlay.DoModal() )
		{
			const char*	szOverrideName							= xOverlay.GetString();
			Asura_Hash_ID						uOverrideHash	= Asura_GetHashID( szOverrideName );

            // Check if there's already some other override map with this
            // name.
            Asura_MaterialResponse_OverrideMap*	pxOverrideMap	= Asura_MaterialResponse_System::GetOverrideMap( uOverrideHash );
			if( pxOverrideMap == NULL )
			{
				bAwaitingValidOverride	= false;

                // Create the override with an arbitrary expected number of
                // overrides. It doesn't really matter - it doesn't limit the
                // capacity, just speeds up queries a bit if it's accurate,
                // and it'll get corrected when the data's saved anyway.
				pxOverrideMap = Asura_MaterialResponse_System::CreateOverrideMap( uOverrideHash, 100 );
				pxOverrideMap->SetName(szOverrideName);

				Tools_Overlay_MaterialResponseOverrides::RefreshAll();
				SetModified();
			}
		}
		else
		{
			bAwaitingValidOverride	= false;
		}
	}
}

/// End of function 'Tools_Overlay_MaterialResponseOverrides::CreateNewOverride'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Overlay_MaterialResponseOverrides::RefreshAll
// Author       : Andy Tate
// Date         : 3-May-2007
////////////////////////////////////////////////////////////////////////////////

void Tools_Overlay_MaterialResponseOverrides::RefreshAll( void )
{
	Tools_Overlay_MaterialResponseOverrides* px = m_pxFirst;
	while( px != NULL )
	{
		px->DestroyTable();
		px->CreateTable();
		px = px->m_pxNext;
	}
}

void Tools_Overlay_MaterialResponseOverrides::RefreshAllCell( const u_int uRow, const u_int uColumn )
{
	Tools_Overlay_MaterialResponseOverrides* px = m_pxFirst;
	while( px != NULL )
	{
		Asura_Table_Dialog* const pxTableDialog = px->m_pxTableDialog;
		if( pxTableDialog != NULL )
		{
			Asura_TableCell_Dialog* const pxCell = pxTableDialog->GetCell( uRow, uColumn );
			if( pxCell != NULL )
			{
				( (Tools_TableCell_MaterialResponseOverride_Dialog*) pxCell )->RefreshDialog();
			}
		}
		px = px->m_pxNext;
	}
}

/// End of function 'Tools_Overlay_MaterialResponseOverrides::RefreshAll'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Overlay_MaterialResponseOverrides::CreateTable
// Author       : Andy Tate
// Date         : 1-May-2007
////////////////////////////////////////////////////////////////////////////////

void Tools_Overlay_MaterialResponseOverrides::CreateTable(void)
{
	// Update the lists of material and override names...
	int i;
	Tools_MaterialResponse_System::GetMaterialNames( m_iNumMaterials, m_pszMaterialNameArray);

	m_iNumOverrides			= Asura_MaterialResponse_System::GetNumOverrideMaps();
	m_pszOverrideNameArray	= Asura_MaterialResponse_System::CreateOverrideMapNameArray();

	// Create the custom headings...
	m_pxMaterialHeadingArray	= new Asura_String[ m_iNumMaterials ];
	m_pxOverrideHeadingArray	= new Asura_String[ m_iNumOverrides ];

	for( i = 0 ; i < m_iNumMaterials; i++ )
	{
		const Asura_String	xFullPath( m_pszMaterialNameArray[ i ] );

		Asura_String		xPath;
		Asura_String		xFilename;
		Asura_String		xExtension;

		xFullPath.SplitFilename(&xPath, &xFilename, &xExtension);

		if( xPath.IsBlank() )
		{
			m_pxMaterialHeadingArray[i]	= xFilename;
		}
		else
		{
			m_pxMaterialHeadingArray[i]	 = "(";
			m_pxMaterialHeadingArray[i]	+= xPath;
			m_pxMaterialHeadingArray[i]	+= ")\n";
			m_pxMaterialHeadingArray[i]	+= xFilename;
		}

		if( !xExtension.IsBlank() )
		{
			m_pxMaterialHeadingArray[i]	+= ".";
			m_pxMaterialHeadingArray[i]	+= xExtension;
		}
	}

	for( i = 0; i < m_iNumOverrides; i++ )
	{
		m_pxOverrideHeadingArray[i]	= m_pszOverrideNameArray[i];
	}

	// Create the hash arrays...
	m_puMaterialHashArray	= new Asura_Hash_ID[ m_iNumMaterials ];
	m_puOverrideHashArray	= new Asura_Hash_ID[ m_iNumOverrides ];

	for( i = 0; i < m_iNumMaterials; i++ )
	{
		m_puMaterialHashArray[ i ]	= Asura_GetHashID( m_pszMaterialNameArray[ i ] );
	}

	for( i = 0; i < m_iNumOverrides; i++ )
	{
		m_puOverrideHashArray[i]	= Asura_GetHashID( m_pszOverrideNameArray[i] );
	}

	// Create the table...
	if( ( m_iNumMaterials > 0 ) && ( m_iNumOverrides > 0 ) && ( m_pxTableDialog != NULL ) )
	{
		const bool bUseMaterialHeadings = ( m_pxTabsDialog == NULL );
		Asura_String* pxHeadings = ( bUseMaterialHeadings ) ? m_pxMaterialHeadingArray : NULL;
		m_pxTableDialog->CreateTable( g_xTestMinimumCellSize, m_iNumMaterials, m_iNumOverrides, CreateCellDialog, this, pxHeadings, m_pxOverrideHeadingArray );
		if( bUseMaterialHeadings )
		{
			m_pxTableDialog->SortRowsUsingHeadings( false );
		}
		m_pxTableDialog->SortColumnsUsingHeadings( false );
		if( m_pxNoOverridesDialog != NULL )
		{
			m_pxNoOverridesDialog->SetHidden( true );
		}
	}
	else
	{
		if( m_pxNoOverridesDialog != NULL )
		{
			m_pxNoOverridesDialog->SetHidden( false );
		}
	}
	ApplyFilters();
}

/// End of function 'Tools_Overlay_MaterialResponseOverrides::CreateTable'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Overlay_MaterialResponseOverrides::DestroyTable
// Author       : Andy Tate
// Date         : 1-May-2007
////////////////////////////////////////////////////////////////////////////////

void Tools_Overlay_MaterialResponseOverrides::DestroyTable(void)
{
	// Remove the current table...
	if( m_pxTableDialog != NULL )
	{
		m_pxTableDialog->DestroyTable();
	}

	// Tidy-up the data...
	delete[]( m_pszMaterialNameArray );
	delete[]( m_pxMaterialHeadingArray );
	delete[]( m_puMaterialHashArray );

	Asura_MaterialResponse_System::DestroyOverrideMapNameArray( m_pszOverrideNameArray );
	delete[]( m_pxOverrideHeadingArray );
	delete[]( m_puOverrideHashArray );

	// Reset the variables...
	m_pszMaterialNameArray		= NULL;
	m_pxMaterialHeadingArray	= NULL;
	m_puMaterialHashArray		= NULL;
	m_iNumMaterials				= 0;

	m_pszOverrideNameArray		= NULL;
	m_pxOverrideHeadingArray	= NULL;
	m_puOverrideHashArray		= NULL;
	m_iNumOverrides				= 0;

	delete[]( m_pbEnableTableRows );
	m_pbEnableTableRows			= NULL;
}

/// End of function 'Tools_Overlay_MaterialResponseOverrides::DestroyTable'. ///



////////////////////////////////////////////////////////////////////////////////

class Tools_TableCell_MaterialResponseOverride_MaterialDialog : public Asura_Function_Dialog
{
public:
	Tools_TableCell_MaterialResponseOverride_MaterialDialog( 
		Tools_Overlay_MaterialResponseOverrides* const pxParent,
		const u_int uColumn,
		const char* szName,
		const Asura_Vector_2& xOffset,
		void(*pfnCallback)(int),
		int iUserData)
	: PARENT( szName, xOffset, pfnCallback, iUserData )
	, m_pxParent( pxParent )
	, m_iColumn( uColumn )
	{
	}

	virtual unsigned GetColour( void )			const	{ return( m_pxParent->ChangeTableColumnColour( PARENT::GetColour(), m_iColumn ) ); }

private:
	typedef Asura_Function_Dialog PARENT;
	Tools_Overlay_MaterialResponseOverrides* const m_pxParent;
	const u_int m_iColumn;
};

////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_TableCell_MaterialResponseOverride_Dialog::SelectNewMaterial
// Author       : Andy Tate
// Date         : 1-May-2007
////////////////////////////////////////////////////////////////////////////////

void Tools_TableCell_MaterialResponseOverride_Dialog::SelectNewMaterial( void )
{
	const char* const*									pszOverrideNameArray	= m_pxParent->GetOverrideNameArray();
	const Asura_Hash_ID*								puOverrideHashArray		= m_pxParent->GetOverrideHashArray();
	const int											iNumOverrides			= m_pxParent->GetNumOverrides();

	const int											iOverrideIndex			= m_iColumn;
	const Asura_Hash_ID									uOverrideHash			= puOverrideHashArray[iOverrideIndex];
	Asura_MaterialResponse_OverrideMap*					pxOverrideMap			= Asura_MaterialResponse_System::GetOverrideMap(uOverrideHash);

	if( pxOverrideMap == NULL )
	{
		Asura_Assert(false, "Cannot find override-map to set override in.");
	}
	else
	{
		const char* const*						pszMaterialNameArray	= m_pxParent->GetMaterialNameArray();
		const Asura_Hash_ID*					puMaterialHashArray		= m_pxParent->GetMaterialHashArray();
		const int								iNumMaterials			= m_pxParent->GetNumMaterials();

		const int								iMaterialIndex			= m_iRow;
		const Asura_Hash_ID						uMaterialHash			= puMaterialHashArray[iMaterialIndex];
		const Asura_MaterialResponse_Override*	pxOverride				= pxOverrideMap->GetOverride(uMaterialHash);
		const Asura_Hash_ID						uOverrideMaterialHash	= ( pxOverride == NULL ) ? ASURA_HASH_ID_UNSET : pxOverride->GetOverrideHashID();

		int										iOverrideMaterialIndex	= m_pxParent->GetIndexForMaterial(uOverrideMaterialHash, iMaterialIndex);

		Tools_Modal_Overlay_SelectMaterial		xSelectionOverlay( m_pxParent, "Select override", iMaterialIndex, iOverrideMaterialIndex, pszMaterialNameArray, iNumMaterials, iOverrideIndex, pszOverrideNameArray, iNumOverrides );

		if( xSelectionOverlay.DoModal() )
		{
			Asura_MaterialResponse_Override*	pxOverride			= pxOverrideMap->GetOverride(uMaterialHash);
			const int							iNewMaterialIndex	= xSelectionOverlay.GetMaterialIndex();

			// Remove the previous override for this material (if there was one)...
			if(pxOverride != NULL)
			{
				pxOverrideMap->DestroyOverride(uMaterialHash);
			}

			// Add the new material to this override-map...
			if( iNewMaterialIndex >= 0 )
			{
				const char*			szNewMaterialName	= pszMaterialNameArray[iNewMaterialIndex];
				const Asura_Hash_ID	uNewMaterialHash	= puMaterialHashArray[iNewMaterialIndex];

				pxOverride	= pxOverrideMap->CreateOverride(uMaterialHash);

				pxOverride->SetOverrideHashID(uNewMaterialHash);
				pxOverride->SetOverrideName(szNewMaterialName);
			}

			// Ensure we're up-to-date...
			Tools_Overlay_MaterialResponseOverrides::RefreshAllCell( m_iRow, m_iColumn );
			Tools_Overlay_MaterialResponseOverrides::SetModified();
		}
	}
}

/// End of function 'Tools_TableCell_MaterialResponseOverride_Dialog::SelectNewMaterial'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_TableCell_MaterialResponseOverride_Dialog::RefreshDialog
// Author       : Andy Tate
// Date         : 3-May-2007
////////////////////////////////////////////////////////////////////////////////

void Tools_TableCell_MaterialResponseOverride_Dialog::RefreshDialog(void)
{
	const Asura_Hash_ID*						puOverrideHashArray	= m_pxParent->GetOverrideHashArray();
	const Asura_Hash_ID							uOverrideHash		= puOverrideHashArray[m_iColumn];
	const Asura_MaterialResponse_OverrideMap*	pxOverrideMap		= Asura_MaterialResponse_System::GetOverrideMap(uOverrideHash);

	if(pxOverrideMap == NULL)
	{
		Asura_Assert(false, "Cannot find specified override-map.");
	}
	else
	{
		const Asura_Hash_ID*					puMaterialHashArray	= m_pxParent->GetMaterialHashArray();
		const Asura_Hash_ID						uMaterialHash		= puMaterialHashArray[m_iRow];
		const Asura_MaterialResponse_Override*	pxOverride			= pxOverrideMap->GetOverride(uMaterialHash);

		if(pxOverride == NULL)
		{
			m_pxMaterialDialog->UpdateLabel("*** none ***");
		}
		else
		{
			const Asura_String*	pxMaterialHeadings			= m_pxParent->GetMaterialHeadingArray();
			const Asura_Hash_ID	uOverriddenMaterialHash		= pxOverride->GetOverrideHashID();
			const int			iOverriddenMaterialIndex	= m_pxParent->GetIndexForMaterial(uOverriddenMaterialHash);
			const char*			szOverriddenMaterialName	= (iOverriddenMaterialIndex >= 0)
															? pxMaterialHeadings[iOverriddenMaterialIndex].GetString()
															: "???";

			m_pxMaterialDialog->UpdateLabel(szOverriddenMaterialName);
			Tools_MaterialResponse_System::GetMaterialByHashID(uOverriddenMaterialHash, &m_iCurrentMaterial);
		}

		// Ensure our cell is big enough...
		UpdateSize();
	}
}

/// End of function 'Tools_TableCell_MaterialResponseOverride_Dialog::RefreshDialog'. ///






////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Modal_Overlay_SelectMaterial::AcceptMaterial
// Author       : Andy Tate
// Date         : 1-May-2007
////////////////////////////////////////////////////////////////////////////////

void Tools_Modal_Overlay_SelectMaterial::AcceptMaterial( int iUserData )
{
	Tools_Modal_Overlay_SelectMaterial* pxThis = (Tools_Modal_Overlay_SelectMaterial*)iUserData;
	pxThis->m_bAcceptMaterial = true;
}

/// End of function 'Tools_Modal_Overlay_SelectMaterial::AcceptMaterial'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Modal_Overlay_SelectMaterial::CancelMaterial
// Author       : Andy Tate
// Date         : 1-May-2007
////////////////////////////////////////////////////////////////////////////////

void Tools_Modal_Overlay_SelectMaterial::CancelMaterial( int iUserData )
{
	Tools_Modal_Overlay_SelectMaterial* pxThis = (Tools_Modal_Overlay_SelectMaterial*)iUserData;
	pxThis->m_bAcceptMaterial = false;
}

/// End of function 'Tools_Modal_Overlay_SelectMaterial::CancelMaterial'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Modal_Overlay_SelectMaterial::DisableMaterial
// Author       : Andy Tate
// Date         : 1-May-2007
////////////////////////////////////////////////////////////////////////////////

void Tools_Modal_Overlay_SelectMaterial::DisableMaterial( int iUserData )
{
	Tools_Modal_Overlay_SelectMaterial* pxThis = (Tools_Modal_Overlay_SelectMaterial*)iUserData;
	pxThis->m_iMaterialIndex	= -1;
	pxThis->m_bAcceptMaterial	= true;
}

/// End of function 'Tools_Modal_Overlay_SelectMaterial::DisableMaterial'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Modal_Overlay_SelectMaterial::InitialiseOverlay
// Author       : Andy Tate
// Date         : 1-May-2007
////////////////////////////////////////////////////////////////////////////////

void Tools_Modal_Overlay_SelectMaterial::InitialiseOverlay(Asura_Overlay* pxOverlay)
{
	Asura_Vector_2	xOffset(1.0f, 3.0f);
	Asura_Dialog*	pxNewDialog;

	// Initialise the dialogs...
	delete m_pxOverridesFilters;
	m_pxOverridesFilters = new Tools_Overlay_MaterialResponseOverridesFilters( pxOverlay, NULL, xOffset, FiltersChanged, this );
	m_pxOverridesFilters->SetDefaults( m_pxParent->GetOverridesFilters() );
	xOffset.y	+= Asura_Dialog::DefaultSpacing();

	pxNewDialog	= new Asura_StringDisplay_Dialog(m_xMaterialString.GetString(), xOffset);
	pxOverlay->AddDialog( pxNewDialog );
	xOffset.y	+= Asura_Dialog::DefaultSpacing();

	pxNewDialog	= new Asura_StringDisplay_Dialog(m_xOverrideNameString.GetString(), xOffset);
	pxOverlay->AddDialog( pxNewDialog );
	xOffset.y	+= Asura_Dialog::DefaultSpacing();

	pxNewDialog	= new Asura_StringDisplay_Dialog(m_xOverrideMaterialString.GetString(), xOffset);
	pxOverlay->AddDialog( pxNewDialog );
	xOffset.y	+= Asura_Dialog::LargeSpacing();

	m_pxMaterialNamesDropDown = new Asura_DropDown_List_Dialog( xOffset, m_pszFilteredMaterialNames, &m_iFilteredMaterialIndex, 10, m_iNumFilteredMaterials, NULL, NULL, NULL, true );
	pxOverlay->AddDialog( m_pxMaterialNamesDropDown );
	xOffset.y 	+= Asura_Dialog::LargeSpacing();

	pxNewDialog	= new Asura_Function_Dialog("Accept", xOffset, AcceptMaterial, (int)this, &pxOverlay->m_bShouldClose);
	pxOverlay->AddDialog( pxNewDialog );
	xOffset.y	+= Asura_Dialog::DefaultSpacing();

	pxNewDialog	= new Asura_Function_Dialog("Cancel", xOffset, CancelMaterial, (int)this, &pxOverlay->m_bShouldClose);
	pxOverlay->AddDialog( pxNewDialog );
	xOffset.y	+= Asura_Dialog::DefaultSpacing();

	pxNewDialog	= new Asura_Function_Dialog("Disable", xOffset, DisableMaterial, (int)this, &pxOverlay->m_bShouldClose);
	pxOverlay->AddDialog( pxNewDialog );
	xOffset.y	+= Asura_Dialog::DefaultSpacing();

	ApplyFilters();
}

/// End of function 'Tools_Modal_Overlay_SelectMaterial::InitialiseOverlay'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : FiltersChanged & ApplyFilters
// Author       : Mark Parry
// Date         : 6-July-2007
////////////////////////////////////////////////////////////////////////////////

void Tools_Modal_Overlay_SelectMaterial::FiltersChanged( void* const pxUserData )
{
	Tools_Modal_Overlay_SelectMaterial* pxThis = (Tools_Modal_Overlay_SelectMaterial*)pxUserData;
	pxThis->ApplyFilters();
}

void Tools_Modal_Overlay_SelectMaterial::ApplyFilters( void )
{
	if( ( m_pxMaterialNamesDropDown != NULL ) && ( m_pxOverridesFilters != NULL ) && ( m_pszMaterialNames != NULL ) )
	{
		int iSelect = -1;
		m_iNumFilteredMaterials = 0;
		for( int i = 0; i < m_iNumMaterials; i++ )
		{
			const char* const szMaterialName = m_pszMaterialNames[ i ];
			if( m_pxOverridesFilters->MatchesFilters( szMaterialName ) )
			{
				if( i == m_iFilteredMaterialIndex )
				{
					iSelect = m_iNumFilteredMaterials;
				}
				if( ( iSelect == -1 ) && ( i == m_iMaterialIndex ) )
				{
					iSelect = m_iNumFilteredMaterials;
				}
				m_pszFilteredMaterialNames[ m_iNumFilteredMaterials ] = szMaterialName;
				m_piFilteredMaterialIndexesLookup[ m_iNumFilteredMaterials ] = i;
				m_iNumFilteredMaterials++;
			}
		}
		m_pxMaterialNamesDropDown->SelectEntry( 0 );
		m_pxMaterialNamesDropDown->ReplaceStrings( m_iNumFilteredMaterials, m_pszFilteredMaterialNames );
		m_pxMaterialNamesDropDown->SelectEntry( ( iSelect == -1 ) ? 0 : iSelect );
	}
}

////////////////////////////////////////////////////////////////////////////////
// Public functions...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Overlay_MaterialResponseOverrides::Initialise
// Author       : Andy Tate
// Date         : 26-Apr-2007
////////////////////////////////////////////////////////////////////////////////

void Tools_Overlay_MaterialResponseOverrides::Initialise(void)
{
	LoadAsuraFile( g_szMaterialResponseOverridesFilename );
}

/// End of function 'Tools_Overlay_MaterialResponseOverrides::Initialise'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Overlay_MaterialResponseOverrides::Shutdown
// Author       : Andy Tate
// Date         : 26-Apr-2007
////////////////////////////////////////////////////////////////////////////////

void Tools_Overlay_MaterialResponseOverrides::Shutdown(void)
{
	if ( IsOpen() )
	{
		ToggleOverlay();
	}
}

/// End of function 'Tools_Overlay_MaterialResponseOverrides::Shutdown'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Overlay_MaterialResponseOverrides::ToggleOverlay
// Author       : Andy Tate
// Date         : 26-Apr-2007
////////////////////////////////////////////////////////////////////////////////

void Tools_Overlay_MaterialResponseOverrides::ToggleOverlay(void)
{
	if(IsOpen())
	{
		SaveAndClose();
	}
	else
	{
		CreateOverlay();
	}
}

/// End of function 'Tools_Overlay_MaterialResponseOverrides::ToggleOverlay'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Overlay_MaterialResponseOverrides::CreateCellDialog
// Author       : Andy Tate
// Date         : 30-Apr-2007
////////////////////////////////////////////////////////////////////////////////

Asura_TableCell_Dialog* Tools_Overlay_MaterialResponseOverrides::CreateCellDialog(
	Asura_Table_Dialog& xTableDialog,
	const Asura_Vector_2& xOffset,
	const Asura_Vector_2& xMinCellSize,
	const u_int uRow,
	const u_int uColumn,
	void* const pxUserData )
{
	Tools_Overlay_MaterialResponseOverrides* pxThis = (Tools_Overlay_MaterialResponseOverrides*)pxUserData;

	return( new Tools_TableCell_MaterialResponseOverride_Dialog( xTableDialog, pxThis, xOffset, xMinCellSize, uRow, uColumn ) );
}

/// End of function 'Tools_Overlay_MaterialResponseOverrides::CreateCellDialog'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_TableCell_MaterialResponseOverride_Dialog::Tools_TableCell_MaterialResponseOverride_Dialog
// Author       : Andy Tate
// Date         : 30-Apr-2007
////////////////////////////////////////////////////////////////////////////////

Tools_TableCell_MaterialResponseOverride_Dialog::Tools_TableCell_MaterialResponseOverride_Dialog( Asura_Table_Dialog& xTableDialogue,
																								  Tools_Overlay_MaterialResponseOverrides* const pxParent,
																								  const Asura_Vector_2& xOffset, const Asura_Vector_2& xMinCellSize,
																								  const u_int uRow, const u_int uColumn )
: PARENT( xTableDialogue, xOffset, xMinCellSize, uRow, uColumn )
, m_pxParent( pxParent )
, m_iCurrentMaterial(0)
{
	Asura_Vector_2	xLocalOffset(0.0f, 0.0f);

	// Create the dialog...
	m_pxMaterialDialog = new Tools_TableCell_MaterialResponseOverride_MaterialDialog( pxParent, uColumn, "", xLocalOffset, SelectNewMaterial, (int)(this) );
	AddDialogToPanel( m_pxMaterialDialog );
	xLocalOffset[0]	+= m_pxMaterialDialog->m_xMaxCoord[0];

	// Refresh the text...
	RefreshDialog();
}

/// End of function 'Tools_TableCell_MaterialResponseOverride_Dialog::Tools_TableCell_MaterialResponseOverride_Dialog'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_TableCell_MaterialResponseOverride_Dialog::Compare
// Author       : Andy Tate
// Date         : 8-May-2007
////////////////////////////////////////////////////////////////////////////////

int Tools_TableCell_MaterialResponseOverride_Dialog::Compare(const Asura_TableCell_Dialog& xOther) const
{
	const Tools_TableCell_MaterialResponseOverride_Dialog*	pxOtherMaterialResponseOverrideDialog	= static_cast<const Tools_TableCell_MaterialResponseOverride_Dialog*>(&xOther);
	const Asura_Dialog*										pxMySortDialog							= m_pxMaterialDialog;
	const Asura_Dialog*										pxOtherSortDialog						= pxOtherMaterialResponseOverrideDialog->m_pxMaterialDialog;

	return(pxMySortDialog->m_xName.Compare(pxOtherSortDialog->m_xName));
}

/// End of function 'Tools_TableCell_MaterialResponseOverride_Dialog::Compare'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_TableCell_MaterialResponseOverride_Dialog::Update
// Author       : Andy Tate
// Date         : 1-May-2007
////////////////////////////////////////////////////////////////////////////////

void Tools_TableCell_MaterialResponseOverride_Dialog::Update(const Asura_Vector_2& xOrigin)
{
	// Call the base-function...
	PARENT::Update(xOrigin);
}

/// End of function 'Tools_TableCell_MaterialResponseOverride_Dialog::Update'. ///








////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Modal_Overlay_SelectMaterial::Tools_Modal_Overlay_SelectMaterial
// Author       : Andy Tate
// Date         : 1-May-2007
////////////////////////////////////////////////////////////////////////////////

Tools_Modal_Overlay_SelectMaterial::Tools_Modal_Overlay_SelectMaterial(
	Tools_Overlay_MaterialResponseOverrides* const pxParent,
	const char* const szTitle,
	const int iMaterialIndex,
	const int iOverrideMaterialIndex,
	const char* const* pszMaterialNames,
	const int iNumMaterials,
	const int iOverrideIndex,
	const char* const* pszOverrideNames,
	const int iNumOverrides)
: PARENT(szTitle)
, m_pxParent( pxParent )
, m_bAcceptMaterial(false)
, m_pxOverridesFilters( NULL )
, m_pxMaterialNamesDropDown( NULL )
, m_pszFilteredMaterialNames( NULL )
, m_iNumFilteredMaterials( 0 )
, m_iFilteredMaterialIndex( 0 )
, m_piFilteredMaterialIndexesLookup( NULL )
, m_iMaterialIndex(iMaterialIndex)
, m_pszMaterialNames(pszMaterialNames)
, m_iNumMaterials(iNumMaterials)
, m_xMaterialString("Material: ")
, m_iOverrideIndex(iOverrideIndex)
, m_pszOverrideNames(pszOverrideNames)
, m_iNumOverrides(iNumOverrides)
, m_xOverrideNameString("Override Name: ")
, m_xOverrideMaterialString("Override Material: ")
{
	m_xMaterialString			+= pszMaterialNames[ m_iMaterialIndex ];
	m_xOverrideNameString		+= pszOverrideNames[ m_iOverrideIndex ];
	m_xOverrideMaterialString	+= pszMaterialNames[ iOverrideMaterialIndex ];

	// initially unfiltered, so just copy the current materials
	m_pszFilteredMaterialNames = new const char*[ m_iNumMaterials ];
	Asura_Assert( ( m_pszFilteredMaterialNames != NULL ), "" );

	m_piFilteredMaterialIndexesLookup = new int[ m_iNumMaterials ];
	Asura_Assert( ( m_piFilteredMaterialIndexesLookup != NULL ), "" );

	for( int i = 0; i < m_iNumMaterials; i++ )
	{
		if( m_pszFilteredMaterialNames != NULL )
		{
			m_pszFilteredMaterialNames[ i ] = m_pszMaterialNames[ i ];
		}
		if( m_piFilteredMaterialIndexesLookup != NULL )
		{
			m_piFilteredMaterialIndexesLookup[ i ] = i;
		}
	}
	m_iFilteredMaterialIndex	= m_iMaterialIndex;
	m_iNumFilteredMaterials		= m_iNumMaterials;
}

/// End of function 'Tools_Modal_Overlay_SelectMaterial::Tools_Modal_Overlay_SelectMaterial'. ///

Tools_Modal_Overlay_SelectMaterial::~Tools_Modal_Overlay_SelectMaterial()
{
	delete m_pxOverridesFilters;
	m_pxOverridesFilters = NULL;

	delete [] m_pszFilteredMaterialNames;
	m_pszFilteredMaterialNames = NULL;

	delete [] m_piFilteredMaterialIndexesLookup;
	m_piFilteredMaterialIndexesLookup = NULL;
}

int	Tools_Modal_Overlay_SelectMaterial::GetMaterialIndex(void) const
{
	if( m_iMaterialIndex == -1 )
	{
		// override has been disabled
		return( m_iMaterialIndex );
	}
	return( m_piFilteredMaterialIndexesLookup[ m_iFilteredMaterialIndex ] );
}

////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Modal_Overlay_SelectMaterial::DoModal
// Author       : Andy Tate
// Date         : 1-May-2007
////////////////////////////////////////////////////////////////////////////////

bool Tools_Modal_Overlay_SelectMaterial::DoModal(void)
{
	ShowOverlay();
	return( m_bAcceptMaterial );
}

/// End of function 'Tools_Modal_Overlay_SelectMaterial::DoModal'. ///

#endif	// defined(ASURA_USE_MATERIAL_RESPONSE_SYSTEM)

////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Modal_Overlay_SelectCurrentOverrideMap::Tools_Modal_Overlay_SelectCurrentOverrideMap
// Author       : Andy Tate
// Date         : 1-May-2007
////////////////////////////////////////////////////////////////////////////////

Tools_Modal_Overlay_SelectCurrentOverrideMap::Tools_Modal_Overlay_SelectCurrentOverrideMap(const char* szTitle,
																						   const int iOverrideIndex,
																						   const char* const* pszOverrideNames,
																						   const int iNumOverrides)
																						   : PARENT(szTitle)
																						   , m_bAcceptOverride(false)
																						   , m_iOverrideIndex(iOverrideIndex)
																						   , m_pszOverrideNames(pszOverrideNames)
																						   , m_iNumOverrides(iNumOverrides)
{
}

/// End of function 'Tools_Modal_Overlay_SelectCurrentOverrideMap::Tools_Modal_Overlay_SelectCurrentOverrideMap'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Modal_Overlay_SelectCurrentOverrideMap::DoModal
// Author       : Andy Tate
// Date         : 1-May-2007
////////////////////////////////////////////////////////////////////////////////

bool Tools_Modal_Overlay_SelectCurrentOverrideMap::DoModal(void)
{
	ShowOverlay();
	return(m_bAcceptOverride);
}

/// End of function 'Tools_Modal_Overlay_SelectCurrentOverrideMap::DoModal'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Modal_Overlay_SelectCurrentOverrideMap::AcceptOverride
// Author       : Andy Tate
// Date         : 1-May-2007
////////////////////////////////////////////////////////////////////////////////

void Tools_Modal_Overlay_SelectCurrentOverrideMap::AcceptOverride( int iUserData )
{
	Tools_Modal_Overlay_SelectCurrentOverrideMap* pxThis = (Tools_Modal_Overlay_SelectCurrentOverrideMap*)iUserData;
	pxThis->m_bAcceptOverride = true;
}

/// End of function 'Tools_Modal_Overlay_SelectCurrentOverrideMap::AcceptOverride'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Modal_Overlay_SelectCurrentOverrideMap::CancelOverride
// Author       : Andy Tate
// Date         : 1-May-2007
////////////////////////////////////////////////////////////////////////////////

void Tools_Modal_Overlay_SelectCurrentOverrideMap::CancelOverride( int iUserData )
{
	Tools_Modal_Overlay_SelectCurrentOverrideMap* pxThis = (Tools_Modal_Overlay_SelectCurrentOverrideMap*)iUserData;
	pxThis->m_bAcceptOverride = false;
}

/// End of function 'Tools_Modal_Overlay_SelectCurrentOverrideMap::CancelOverride'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Modal_Overlay_SelectCurrentOverrideMap::DisableOverride
// Author       : Andy Tate
// Date         : 1-May-2007
////////////////////////////////////////////////////////////////////////////////

void Tools_Modal_Overlay_SelectCurrentOverrideMap::DisableOverride( int iUserData )
{
	Tools_Modal_Overlay_SelectCurrentOverrideMap* pxThis = (Tools_Modal_Overlay_SelectCurrentOverrideMap*)iUserData;
	pxThis->m_iOverrideIndex	= -1;
	pxThis->m_bAcceptOverride	= true;
}

/// End of function 'Tools_Modal_Overlay_SelectCurrentOverrideMap::DisableOverride'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Modal_Overlay_SelectCurrentOverrideMap::InitialiseOverlay
// Author       : Andy Tate
// Date         : 1-May-2007
////////////////////////////////////////////////////////////////////////////////

void Tools_Modal_Overlay_SelectCurrentOverrideMap::InitialiseOverlay(Asura_Overlay* pxOverlay)
{
	Asura_Vector_2	xOffset(1.0f, 3.0f);
	Asura_Dialog*	pxNewDialog;

	// Initialise the dialogs...
	pxNewDialog	= new Asura_DropDown_List_Dialog(xOffset, m_pszOverrideNames, &m_iOverrideIndex, 10, m_iNumOverrides, NULL, NULL, NULL, true );
	pxOverlay->AddDialog( pxNewDialog );
	xOffset.y	+= Asura_Dialog::LargeSpacing();

	pxNewDialog	= new Asura_Function_Dialog("Accept", xOffset, AcceptOverride, (int)this, &pxOverlay->m_bShouldClose);
	pxOverlay->AddDialog( pxNewDialog );
	xOffset.y	+= Asura_Dialog::DefaultSpacing();

	pxNewDialog	= new Asura_Function_Dialog("Cancel", xOffset, CancelOverride, (int)this, &pxOverlay->m_bShouldClose);
	pxOverlay->AddDialog( pxNewDialog );
	xOffset.y	+= Asura_Dialog::DefaultSpacing();

	pxNewDialog	= new Asura_Function_Dialog("Disable", xOffset, DisableOverride, (int)this, &pxOverlay->m_bShouldClose);
	pxOverlay->AddDialog( pxNewDialog );
	xOffset.y	+= Asura_Dialog::DefaultSpacing();
}

/// End of function 'Tools_Modal_Overlay_SelectCurrentOverrideMap::InitialiseOverlay'. ///


/// End of file. ///
