
///////////////////////////////////////////////////////////////////////////////
//
// File   : Tools_Overlay_MaterialResponseOverrides.h
// Author : Andy Tate
// Date   : 26-Apr-2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __TOOLS_OVERLAY_MATERIALRESPONSEOVERRIDES_H__
#define __TOOLS_OVERLAY_MATERIALRESPONSEOVERRIDES_H__


////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura_Maths.h"
#include "Interface/Asura_Table_Dialog.h"

#include "Misc/Tools_Modal_Overlay.h"
#if defined(ASURA_USE_MATERIAL_RESPONSE_SYSTEM)


////////////////////////////////////////////////////////////////////////////////
// Forward class declarations...
////////////////////////////////////////////////////////////////////////////////

class Asura_Overlay;
class Tools_Overlay_MaterialResponseOverridesFilters;
class Tools_TableCell_MaterialResponseOverride_MaterialDialog;

////////////////////////////////////////////////////////////////////////////////
// Class declaration...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Class        : Tools_Overlay_MaterialResponseOverrides
// Author       : Andy Tate
// Date         : 26-Apr-2007
//
// Description  : Static class used to handle the overlay for overriding the
//				  material-response system.
////////////////////////////////////////////////////////////////////////////////

class Tools_Overlay_MaterialResponseOverrides
{
public:
	Tools_Overlay_MaterialResponseOverrides();
	~Tools_Overlay_MaterialResponseOverrides();

	// Public functions...
	void	Initialise		( void );
	void	Shutdown		( void );
	void	ToggleOverlay	( void );
	void	TabsDialog		( const char* const szMaterialName, Asura_Tabs_Dialog* const pxTabsDialog, Asura_Vector_2& xOffset, const Asura_Vector_2& xTableSize );

	// Accessor functions...
	const char* const*		GetMaterialNameArray	( void )	const	{ return( m_pszMaterialNameArray ); }
	const Asura_String*		GetMaterialHeadingArray	( void )	const	{ return( m_pxMaterialHeadingArray ); }
	const Asura_Hash_ID*	GetMaterialHashArray	( void )	const	{ return( m_puMaterialHashArray ); }
	int						GetNumMaterials			( void )	const	{ return( m_iNumMaterials ); }

	const char* const*		GetOverrideNameArray	( void )	const	{ return( m_pszOverrideNameArray ); }
	const Asura_String*		GetOverrideHeadingArray	( void )	const	{ return( m_pxOverrideHeadingArray ); }
	const Asura_Hash_ID*	GetOverrideHashArray	( void )	const	{ return( m_puOverrideHashArray ); }
	int						GetNumOverrides			( void )	const	{ return( m_iNumOverrides ); }

	int						GetIndexForMaterial		( const Asura_Hash_ID uHash, const int iDefaultIndex = -1 )	const;
	int						GetIndexForOverride		( const Asura_Hash_ID uHash, const int iDefaultIndex = -1 )	const;
	Asura_Table_Dialog*		GetTableDialog			( void )	const	{ return( m_pxTableDialog ); }

	bool					IsOpen					( void )	const	{ return( m_pxOverlay != NULL ); }

	Tools_Overlay_MaterialResponseOverridesFilters* GetOverridesFilters( void)	const	{ return( m_pxOverridesFilters ); }

	static	void	RefreshAll( void );
	static	void	RefreshAllCell( const u_int uRow, const u_int uColumn );

	//Called by the overlay when it gets deleted.
	void OnOverlayDeletion(Asura_Overlay* pxOverlay);

	static void SetModified();
	static bool IsModified() {return s_bModified;}

	static void Save();

protected:
	// Protected variables...

	// Protected functions...
	static	bool	AreAnyOverridesPresent	( void );
	static	bool	IsCurrentOverrideValid	( void );

	static	void	SelectCurrentOverride	( int iUserData )		{ ( ( Tools_Overlay_MaterialResponseOverrides* ) iUserData )->SelectCurrentOverride(); }
	static	void	DeleteCurrentOverride	( int iUserData )		{ ( ( Tools_Overlay_MaterialResponseOverrides* ) iUserData )->DeleteCurrentOverride(); }
	static	void	CopyCurrentOverride		( int iUserData )		{ ( ( Tools_Overlay_MaterialResponseOverrides* ) iUserData )->CopyCurrentOverride(); }
	static	void	RenameCurrentOverride	( int iUserData )		{ ( ( Tools_Overlay_MaterialResponseOverrides* ) iUserData )->RenameCurrentOverride(); }
	
	static	void	SaveAndClose			( int iUserData )		{ ( ( Tools_Overlay_MaterialResponseOverrides* ) iUserData )->SaveAndClose(); }

	void	SelectCurrentOverride	( void );
	void	DeleteCurrentOverride	( void );
	void	CopyCurrentOverride		( void );
	void	RenameCurrentOverride	( void );
	void	SaveAndClose			( void );
	void	Close					( void );

	void	CreateOverlay			( void );
	void	AddDialogs				( Asura_Vector_2& xOffset, const Asura_Vector_2& xTableSize );
	void	AddDialog				( Asura_Dialog* const pxDialog );

	// Table manipulation functions...
	static void	CreateNewOverride( int iUserData )					{ ( ( Tools_Overlay_MaterialResponseOverrides* ) iUserData )->CreateNewOverride(); }
	static Asura_String GetOverrideName( void );

	void	CreateNewOverride	( void );
	void	DestroyTable		( void );
	void	CreateTable			( void );

	// Callback functions...
	static Asura_TableCell_Dialog* CreateCellDialog( Asura_Table_Dialog& xTableDialog, const Asura_Vector_2& xOffset, const Asura_Vector_2& xMinCellSize, const u_int uRow, const u_int uColumn, void* const pxUserData );


private:
	// Private variables...
	Asura_Overlay*		m_pxOverlay;							// A pointer to the current overlay.
	Asura_Tabs_Dialog*	m_pxTabsDialog;							// A pointer to the current tabs dialog.
	Asura_Table_Dialog*	m_pxTableDialog;						// A pointer to the current table dialog.
	bool*				m_pbEnableTableRows;					// Enable / Disable rows in the table
	Asura_Dialog*		m_pxNoOverridesDialog;					// A pointer to the dialog used to tell you we currently have no overrides.
	Asura_Dialog*		m_pxCurrentOverrideDialog;				// A pointer to the dialog used when overriding the current material-response settings.
	Asura_Vector_2		m_xOverlayPosition;						// The position of the overlay.

	const char**		m_pszMaterialNameArray;					// Array of material names.
	Asura_String*		m_pxMaterialHeadingArray;				// Array of headings for the materials.
	Asura_Hash_ID*		m_puMaterialHashArray;					// Array of material hash-IDs.
	int					m_iNumMaterials;						// The number of materials present.

	const char**		m_pszOverrideNameArray;					// Array of override names.
	Asura_String*		m_pxOverrideHeadingArray;				// Array of headings for the overrides.
	Asura_Hash_ID*		m_puOverrideHashArray;					// Array of override hash-IDs.
	int					m_iNumOverrides;						// The number of overrides present.
	int					m_iTabsDialogMaterialIndex;				// The material index for a tabs dialog
	bool				m_bOnlyShowMaterialsWithAnOverride;		// Show only materials with an override
	bool				m_bLastOnlyShowMaterialsWithAnOverride;	// Show only materials with an override

	Tools_Overlay_MaterialResponseOverridesFilters* m_pxOverridesFilters;
	static void OnlyShowMaterialsWithAnOverrideClicked( void );
	static void FiltersChanged( void* const pxUserData );
	void ApplyFilters( void );
	bool DoesMaterialHaveOverride( const Asura_Hash_ID uMaterialHash );
	static bool IsTableRowEnabledCallBack( const u_int uRow, void* const pxUserData );
	static unsigned TableGetColourCallBack( const Asura_Table_Dialog& xTableDialog, void* const pxUserData, const Asura_Dialog* const pxDialog, const unsigned uColour, const u_int uRow, const u_int uColumn );

	friend class Tools_TableCell_MaterialResponseOverride_MaterialDialog;
	unsigned ChangeTableColumnColour( const unsigned uColour, const u_int uColumn );

	// Private functions...
	Tools_Overlay_MaterialResponseOverrides  (const Tools_Overlay_MaterialResponseOverrides& xOther);
	Tools_Overlay_MaterialResponseOverrides&	operator=	(const Tools_Overlay_MaterialResponseOverrides& xOther);

	static Tools_Overlay_MaterialResponseOverrides* m_pxFirst;
	Tools_Overlay_MaterialResponseOverrides* m_pxNext;

	static bool s_bModified;

};   // End of class 'Tools_Overlay_MaterialResponseOverrides'.




////////////////////////////////////////////////////////////////////////////////
// Class        : Tools_TableCell_MaterialResponseOverride_Dialog
// Author       : Andy Tate
// Date         : 30-Apr-2007
//
// Description  : Class to handle the overriding of an individual material-response
//				  value.
////////////////////////////////////////////////////////////////////////////////

class Tools_TableCell_MaterialResponseOverride_Dialog : public Asura_TableCell_Dialog
{
public:
	// Public functions...
	Tools_TableCell_MaterialResponseOverride_Dialog( Asura_Table_Dialog& xTableDialogue, Tools_Overlay_MaterialResponseOverrides* const pxParent, const Asura_Vector_2& xOffset, const Asura_Vector_2& xMinCellSize, const u_int uRow, const u_int uColumn );
//	virtual ~Tools_TableCell_MaterialResponseOverride_Dialog ();

	// Overridden functions...
	virtual int		Compare	(const Asura_TableCell_Dialog& xOther) const;
	virtual void	Update	(const Asura_Vector_2& xOrigin);

	void	RefreshDialog			( void );

protected:
	// Protected variables...

	// Protected functions...
	static void	SelectNewMaterial	( int iUserData )			{ ( ( Tools_TableCell_MaterialResponseOverride_Dialog* ) iUserData )->SelectNewMaterial(); }
	void	SelectNewMaterial		( void );

private:
	// Private typedefs...
	typedef Asura_TableCell_Dialog PARENT;

	// Private variables...
	int															m_iCurrentMaterial;	// The index of the current material.
	Tools_TableCell_MaterialResponseOverride_MaterialDialog*	m_pxMaterialDialog;	// A pointer to the dialog used.

	// Private functions...
	Tools_TableCell_MaterialResponseOverride_Dialog  (void);
	Tools_TableCell_MaterialResponseOverride_Dialog  (const Tools_TableCell_MaterialResponseOverride_Dialog& xOther);

	Tools_TableCell_MaterialResponseOverride_Dialog&	operator=	(const Tools_TableCell_MaterialResponseOverride_Dialog& xOther);

	Tools_Overlay_MaterialResponseOverrides* m_pxParent;

};   // End of class 'Tools_TableCell_MaterialResponseOverride_Dialog'.




////////////////////////////////////////////////////////////////////////////////
// Class        : Tools_Modal_Overlay_SelectMaterial
// Author       : Andy Tate
// Date         : 1-May-2007
//
// Description  : Class to handle the selection of a specific material for use
//				  when overriding existing materials.
////////////////////////////////////////////////////////////////////////////////

class Tools_Modal_Overlay_SelectMaterial : public Tools_Modal_Overlay
{
public:
	// Public functions...
	Tools_Modal_Overlay_SelectMaterial( Tools_Overlay_MaterialResponseOverrides* const pxParent, const char* const szTitle, const int iMaterialIndex, const int iOverrideMaterialIndex, const char* const* pszMaterialNames, const int iNumMaterials, const int iOverrideIndex, const char* const* pszOverrideNames, const int iNumOverrides);
	virtual ~Tools_Modal_Overlay_SelectMaterial();

	// Function to handle modal overlay...
	bool DoModal	(void);

	// Accessor functions...
	int	GetMaterialIndex	(void) const;


protected:
	// Protected variables...

	// Protected functions...
	static void	AcceptMaterial	( int iUserData );
	static void	CancelMaterial	( int iUserData );
	static void	DisableMaterial	( int iUserData );

	// Overridden functions...
	virtual void	InitialiseOverlay	(Asura_Overlay* pxOverlay);
//	virtual void	Update				(void);


private:
	// Private typedefs...
	typedef Tools_Modal_Overlay PARENT;

	Tools_Overlay_MaterialResponseOverrides*	m_pxParent;

	// Private variables...
	bool										m_bAcceptMaterial;			// Flag to allow us to accept the material we've chosen.

	Tools_Overlay_MaterialResponseOverridesFilters* m_pxOverridesFilters;
	Asura_DropDown_List_Dialog*					m_pxMaterialNamesDropDown;
	const char**								m_pszFilteredMaterialNames;
	int											m_iNumFilteredMaterials;
	int											m_iFilteredMaterialIndex;
	int*										m_piFilteredMaterialIndexesLookup;

	int											m_iMaterialIndex;			// The index of the current material.
	const char* const*							m_pszMaterialNames;			// An array of pointers to the names of materials to use.
	int											m_iNumMaterials;			// The number of materials in the array.
	Asura_String								m_xMaterialString;			// The string used to display the material information.

	int											m_iOverrideIndex;			// The index of the current override.
	const char* const*							m_pszOverrideNames;			// An array of pointers to the names of overrides to use.
	int											m_iNumOverrides;			// The number of overrides in the array.
	Asura_String								m_xOverrideNameString;		// The string used to display the override name information.
	Asura_String								m_xOverrideMaterialString;	// The string used to display the override name information.

	// Private functions...
	Tools_Modal_Overlay_SelectMaterial  (void);
	Tools_Modal_Overlay_SelectMaterial  (const Tools_Modal_Overlay_SelectMaterial& xOther);

	Tools_Modal_Overlay_SelectMaterial&	operator=	(const Tools_Modal_Overlay_SelectMaterial& xOther);

	static void FiltersChanged( void* const pxUserData );
	void ApplyFilters( void );

};   // End of class 'Tools_Modal_Overlay_SelectMaterial'.


#endif	// defined(ASURA_USE_MATERIAL_RESPONSE_SYSTEM)

////////////////////////////////////////////////////////////////////////////////
// Class        : Tools_Modal_Overlay_SelectCurrentOverrideMap
// Author       : Andy Tate
// Date         : 1-May-2007
//
// Description  : Class to handle the selection of a specific material-response
//				  override map to use.
////////////////////////////////////////////////////////////////////////////////

class Tools_Modal_Overlay_SelectCurrentOverrideMap : public Tools_Modal_Overlay
{
public:
	// Public functions...
	Tools_Modal_Overlay_SelectCurrentOverrideMap          (const char* szTitle, const int iOverrideIndex, const char* const* pszOverrideNames, const int iNumOverrides);
	//	virtual ~Tools_Modal_Overlay_SelectCurrentOverrideMap ();

	// Function to handle modal overlay...
	bool DoModal	(void);

	// Accessor functions...
	int	GetOverrideIndex	(void) const	{ return(m_iOverrideIndex); }


protected:
	// Protected variables...

	// Protected functions...
	static void	AcceptOverride	( int iUserData );
	static void	CancelOverride	( int iUserData );
	static void	DisableOverride	( int iUserData );

	// Overridden functions...
	virtual void	InitialiseOverlay	(Asura_Overlay* pxOverlay);
	//	virtual void	Update				(void);


private:
	// Private typedefs...
	typedef Tools_Modal_Overlay PARENT;

	// Private variables...
	bool													m_bAcceptOverride;		// Flag to allow us to accept the material we've chosen.

	int														m_iOverrideIndex;		// The index of the current override.
	const char* const*										m_pszOverrideNames;		// An array of pointers to the names of overrides to use.
	int														m_iNumOverrides;		// The number of overrides in the array.

	// Private functions...
	Tools_Modal_Overlay_SelectCurrentOverrideMap  (void);
	Tools_Modal_Overlay_SelectCurrentOverrideMap  (const Tools_Modal_Overlay_SelectCurrentOverrideMap& xOther);

	Tools_Modal_Overlay_SelectCurrentOverrideMap&	operator=	(const Tools_Modal_Overlay_SelectCurrentOverrideMap& xOther);

};   // End of class 'Tools_Modal_Overlay_SelectCurrentOverrideMap'.


#endif	// ifndef __TOOLS_OVERLAY_MATERIALRESPONSEOVERRIDES_H__


/// End of file. ///
