//
// Tools_PlatformMesh_System.h
//
// Created by Richard Rice - 13/03/2007
// 
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//


#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif


#ifndef _TOOLS_PLATFORMMESH_SYSTEM_H_
#define _TOOLS_PLATFORMMESH_SYSTEM_H_

static const int iTOOLS_PLATFORMMESH_DEFAULT = -1; //Indicates that the default meshes should be used , rather than checking for particular platforms

//Handles switching between the meshes used by each platform while in tools
class Tools_PlatformMesh_System
{
public :
	//static void Initialise();
	//static void Shutdown();


	//switches the module system to be using meshes for the specific platform
	//Setting bTemp to true indicates that the platform is just being changed briefly for saving , and
	//so it shouldn't be necessary to invalidate selections etc.
	static void PushPlatform(int iPlatform,bool bTemp);
	static void PopPlatform(bool bTemp);

	//gets the current platform being previewed
	static int GetPlatform(bool bAllowDefault = true);

	//Adds a drop list for selecting the current platform being viewed
	static void AddOverlayControls(class Asura_Overlay* pxOverlay,class Asura_Vector_2& xOffset,bool bAddUpdateSkinsDialog=true);

	//Reloads platform specific skins/anims if any platform changing has been done
	//Only does anything if using ASURA_SUPPORT_PLATFORM_HASHED_FILENAMES.
	static void ReloadFilesIfNeeded();

	static bool GetReloadFilesWhenChangingPlatform() {return s_bReloadFilesWhenChangingPlatform;}
	static void SetReloadFilesWhenChangingPlatform(bool bSet) {s_bReloadFilesWhenChangingPlatform = bSet;}


private :

	//switches the module system to be using meshes for the specific platform
	//Setting bTemp to true indicates that the platform is just being changed briefly for saving , and
	//so it shouldn't be necessary to invalidate selections etc.
	static void SetPlatform(int iPlatform,bool bTemp);

	static void SelectedPlatformChanged();

	static void ReloadSkinsAndAnims();
	static void ReloadSounds();

	static int s_iPlatform;//Current platform being used. Uses enumeration from Tools_Platform together with a special value for default.
	static int s_iSelectedPlatform;//Platform selected by user
	static int s_iDropListIndex;//selection from dropdown list
	static bool s_bNeedToReloadSkinsAndAnims;//true if the platform has been changed , and the files haven't been reloaded yet.
	static bool s_bNeedToReloadSounds;

	static bool s_bReloadFilesWhenChangingPlatform;//If set then the system will automatiaclly try to reload skins/anims when the user changes platform
};

#endif //_TOOLS_PLATFORMMESH_SYSTEM_H_
