#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef __TOOLS_MODE_MESHEDIT_H_
#define __TOOLS_MODE_MESHEDIT_H_

#include "Core/Asura_Handle.h"
#include "Core/Asura_Maths.h"
#include "Network/Asura_Guid.h"
#include "Tools_Mode.h"
#include "Misc/list_tem.hpp"
#include "Misc/Tools_Vertex_Selection.h"

class Asura_Mesh;
class Asura_Overlay;
class Asura_String;
class Asura_Hierarchy_Skin;

/*
struct ModuleVertexPair
{
	int iModuleIndex;
	int iVertexIndex;
};
*/

//flags for merge object into module
#define TOOLS_MERGEOBJECT_IGNORERENDERMESH 0x00000001
#define TOOLS_MERGEOBJECT_IGNORECOLLISIONMESH 0x00000002


class Tools_Mode_MeshEdit : public Tools_Mode
{
public:
	Tools_Mode_MeshEdit() { s_pxThis = this; }
	virtual ~Tools_Mode_MeshEdit() { s_pxThis = NULL; }

	virtual void InitialiseMode();
	virtual void ShutdownMode();

	virtual void Update();
	void RenderWorld_PostToneMap();
	void RenderUI();

	virtual bool IsModified();
	virtual void QuickSave();

	void PreShutdown();

	virtual void HandleMeshChange(int iModuleIndex);


	// Made these functions public, so that I can use them to set up glass.aw.
	//piOrdering is an optional pointer to an array of 4 ints that will contain the order in which the vertices should be taken
	static bool WillSelectedVerticesMakeAQuad(const Asura_Vector_3*pxVertices,int* piOrdering = 0);	// Expects an array of 4 vertices.
	static void RenderVertexSelection(Asura_Vector_3& position);

	static void UnmergePolygons(int iModuleIndex,bool* pbUnmergeArray);

	
	//Adds all the polygons in the specified object mesh to the mesh for the module.
	//If uObjectGuid is set , then the mesh will remember the owner of the polygons so
	//they can be removed later
	//Returns false if it fails because there are too man polygons/vertice being added
	static bool MergeObjectIntoModule(int iModuleIndex,Asura_Mesh* pxObjectMesh,const Asura_Matrix_3x3& xObjectOrient,const Asura_Vector_3& xObjectPosition,Asura_Guid uObjectGuid = ASURA_GUID_UNREGISTERED,u_int uObjectChecksum = 0, float fScale = 1.0f);
	static bool MergeObjectIntoModule(int iModuleIndex,Asura_Mesh* pxObjectMesh,const Asura_Matrix_3x3& xObjectOrient,const Asura_Vector_3& xObjectPosition,Asura_Guid uObjectGuid,u_int uObjectChecksum, const Asura_Vector_3& xScale,u_int uPolyFlagsToAdd,u_int uMergeFlags);

	static bool MergeSkinIntoModule(int iModuleIndex,Asura_Hierarchy_Skin* pxSkin,const Asura_Matrix_3x3& xObjectOrient,const Asura_Vector_3& xObjectPosition,Asura_Guid uObjectGuid,u_int uObjectChecksum, const Asura_Vector_3& xScale,u_int uPolyFlagsToAdd,u_int uMergeFlags);

	//Checks to make sure that there will be enough room to merge all the listed meshes into the module.
	static bool CanMergeMeshesIntoModule(int iModuleInde, const List<Asura_Mesh*>& xMeshList, u_int uMergeFlags, bool bPermanent);

	//Determines a suitable module to merge object into. Returns -1 if it fails
	//Position and centre are both relative to the world
	static int GetModuleForObjectMerge(const Asura_Vector_3& xPosition,const Asura_Vector_3& xCentre);

	//Deletes any polgons that have been merged into the environment for the entity specified
	static void DeleteEntityPolygons(Asura_Guid uGuid);
	//Deletes all entity polygons that have been merged into the environment
	static void DeleteAllEntityPolygons();

	static void InvalidateModuleCollisionMesh(int iModuleIndex);

	enum CopyMeshFlags
	{
		CMNoTextures			= 0x00000001,
		CMNoSmoothingGroups		= 0x00000002,
		CMNoPolyFlags			= 0x00000004,
		CMNoPolyGuids			= 0x00000008,
		CMNoPolyLights			= 0x00000010,
		CMNoPrelighting			= 0x00000020,
		CMNoProjectPolyData	 	= 0x00000040,
		CMNoVertFlags			= 0x00000080,
	};

	/*****************************
	// Create a copy of a mesh. This is specifically aimed at mesh processing type stuff - creating a copy that
	// only has polys flagged in particular ways, or only ones facing a particular direction, etc
	// As such, it assumes nothing about the validity of the data on input or output - it just does a
	// straight copy with filtering.
	//
	// pfnPolyTestCallback should either be null (copy all polygons) or should be a pointer to a function
	// that takes a mesh pointer and polygon index within that mesh, and returns true if that polygon should be
	// copied to the output mesh.
	//
	// Use the uFlags parameter to skip parts of the copy - e.g. if you're going to recalculate normals/prelighting afterwards
	// then don't bother copying them.
	//
	// TODO: Normals, collision data, and merged entity data is currently not copied across because I'm not 
	//       sure if it should be or not (and I don't currently need it). Needs RAR or someone who knows the 
	//       implications to take a look.
	*********************/

	static void CopyMesh(Asura_Mesh* pxInputMesh, Asura_Mesh* pxOutputMesh, bool (*pfnPolyTestCallback)(Asura_Mesh*, int) = 0, u_int uFlags = 0);

	/******************************
	// Merge two meshes together. Takes the same flags as CopyMesh, and assumes that the meshes are efficient -
	// that is, it just concatenates things with no regard to unused data. If you need to optimize, you'll have
	// to do it on the result.
	******************************/
	static void MergeMeshes(Asura_Mesh* pxOutputMesh, Asura_Mesh* pxMesh1, Asura_Mesh* pxMesh2, u_int uFlags = 0);

	static void WeldVertices(Asura_Mesh* pxMesh, float fEpsilon = 1.0e-6f);

	static bool MergePolygons(Asura_Mesh* Mesh,List<int>& MergeList,int** OutputPolyConv);

	static bool UnmergePolygons(Asura_Mesh* Mesh, bool (*pfnPolyTestCallback)(Asura_Mesh*, int), int** OutputPolyConv1, int** OutputPolyConv2);

	static bool DeleteDegenerateTriangles(Asura_Mesh* pxMesh);

protected:
	virtual void AddOverlayControls( Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset );

private:
	//loads/saves data for this mode
	static void TransferModeData(int mode);

	static void ToggleMeshEditOverlay();
	static bool MeshEditOverlayIsOn();
	static void CreateMeshEditOverlay();

	static void HandleKeyboard();

	static void CreatePolygonFromSelectedVertices();
	static void CreatePolygon(int iModuleIndex,Asura_Mesh* pxMesh,int iNumVerts,int* piVerts);

	//Add some loose vertices to a module. Positions are in world space
	static bool AddVerticesToModule(int iModuleIndex,int iNumVertsToAdd,const Asura_Vector_3* pxVertices);

	//Deletes marked vertices from module. Only affects polygons that aren't used by any polygons
	static int DeleteVerticesFromModule(int iModuleIndex,const bool* pbDeleteArray);

	static void MoveVerticesOrPolygons();
	static void MoveVertices();
	static void MovePolygons();

	static void FlattenVertices();

	static int DeletePolygonsInModule(int iModule, Asura_Mesh* pxMesh, bool* pbDeleteArray, bool* pbModuleWasDeleted);
	

	static bool DeletePolygons(Asura_Mesh* Mesh,bool* DeleteArray,int** OutputPolyConv);
	static bool UnmergePolygons(Asura_Mesh* Mesh,bool* UnmergeArray,int** OutputPolyConv1,int** OutputPolyConv2);

	static void UnmergeSelectedPolygons();
	static void MergeSelectedPolygons();
	static void FlipSelectedPolygons();
	static void MergeSelectedModules();

	static void FlagPolygonsForDeletion();
	static void UnflagPolygonsForDeletion();
	static void DestroyFlaggedPolygons();
	static void DeleteModule(int iModuleIndex);

	static bool CanPolygonsBeMerged(Asura_Mesh* Mesh,int PolyIndex1,int PolyIndex2);
	static float GetAngleForPolygonMerge(Asura_Mesh* Mesh,int PolyIndex1,int PolyIndex2);

	//splits a polygon in 2  ,to fix tears at T junctions
	static bool SplitPolygonEdge(int iModuleIndex,int iPolyIndex,const Asura_Vector_3& xSplitPoint);
	static void FixTJunction();
	static void FixTJunction_PolygonSelected(int iModuleIndex,int iPolygonIndex);
	static void FixTJunction_VertexSelected(int iModuleIndex,int iVertexIndex);

	static void TearFix();
	static void TearFix_Vertex1Selected(int iModuleIndex,int iVertexIndex);
	static void TearFix_Vertex2Selected(int iModuleIndex,int iVertexIndex);
	static bool	TearFix_Vertex2Selectable(int iModuleIndex,int iVertexIndex);

	static void SplitEdge();
	static void SplitEdge_EdgeSelected(struct Tools_LOS_EdgeOutput xOutput);

	static void TurnEdge();
	static void TurnEdge_EdgeSelected(struct Tools_LOS_EdgeOutput xOutput);
	static void TurnEdgeBetweenPolys(const Tools_LOS_EdgeOutput& xEdge);
	static void TurnEdgeWithinQuad(const Tools_LOS_EdgeOutput& xEdge);

	static void AutomaticTearFix();

	static void MergeObjectIntoModule();
	static void MergeObjectIntoModule_ObjectSelected(class Tools_Object* pxObject);
	static void MergeObjectIntoModule_ModuleSelected(int iModuleIndex);
	
	static void CreateVertex();
	static void CreateVertex_ModuleSelected(int iModuleIndex);

	static void DeleteSelectedVertices();

private:
	static Tools_Mode_MeshEdit* s_pxThis;

	static Asura_Handle MenuHandle;

	static Asura_Vector_2 MeshEditOverlayPosition;
	static bool	MeshEditOverlayOpen;

	//for polygon creation
	static bool SelectingVertices;

	//automatic tearfix
	static float s_fTearFixTolerance;

	static bool s_bSuppressConfirmation;
};

#endif //__TOOLS_MODE_MESHEDIT_H_
