//
// Tools_CharacterProcessor.cpp
//
// Created by BM - 18/03/2003 12:36:48
//
// Copyright (C) 2003 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

// This file's header.
#include "MeshProcessing/Tools_CharacterProcessor.h"

#include "Mesh/Asura_Mesh.h"
#include "Animation/Asura_Hierarchy.h"
#include "MeshProcessing/Asura_TriStripper.h"
#include "Core/Asura_Material_System.h"

Tools_CharacterProcessor_Strip*	Tools_CharacterProcessor::s_pxProcessedStrip;
u_int							Tools_CharacterProcessor::s_uTotalNumberOfVertices;
u_int							Tools_CharacterProcessor::s_uTotalNumberOfIndices;
bool							Tools_CharacterProcessor::s_bBeenInitialised = false;

// The initialise function takes character mesh and skin and processes them
// into a single strip - the shutdown function cleans up all the memory created
// during the process
bool Tools_CharacterProcessor::Initialise(Asura_Mesh* xMesh, const Asura_Hierarchy_Skin* xSkin)
{
	if(!xMesh || !xSkin)
	{
		Asura_Assert(xMesh,"no mesh to process");
		Asura_Assert(xSkin,"no skin to process");
		return false;
	}
	Asura_Hierarchy_Skin* pxSkin = (Asura_Hierarchy_Skin* )xSkin;
	pxSkin->CalculateSkinNormalData();

//	Tools_Level_Export::UpdateStatus("Stripifying Character", 0);
	
	//Allow for enough space in case all triangles are double sided. RAR 30/01/2004
	u_int uTotalIndices = xMesh->NumberOfTriangles*3*2;

	u_int uNumberOfUniqueVertices = 0;
	u_int uIndex = 0;
	u_int uNumberOfTriangles = 0;
	bool bHasMaterial = false;

	Tools_CharacterProcessor_Vertex* pxWorkspaceVertices = new Tools_CharacterProcessor_Vertex[uTotalIndices];
	unsigned short* usIndexData = new unsigned short[uTotalIndices];

	// fill up the workspace vertex buffer
	for (int p=0; p<xMesh->NumberOfPolygons; p++)
	{	
		int iIndexNumber[4];

		int iPolyFlags = 0;

		if (xMesh->PolygonFlagDataPtr)
		{
			iPolyFlags = xMesh->PolygonFlagDataPtr[p].Flags;
		}

		if (iPolyFlags & AsuraPolygonFlag_DontRender) continue;

		bool bDoubleSided = (iPolyFlags & AsuraPolygonFlag_DoubleSided) ? true : false;

		if(xMesh->TextureDataPtr[p].m_iMaterialID != ASURA_MATERIAL_NONE)
		{
			bHasMaterial = true;
		}

		//If double sided we will need to do 2 passes of adding the vertices , with
		//normals and vertex ordering being reversed for the second side of the polygons
		//RAR 30/01/2004
		for( int iMirror=0;iMirror < (bDoubleSided ? 2 : 1);iMirror++)
		{
			int v=0;
			do
			{
		
				int vID =  xMesh->PolygonDataPtr[p].VertexID[v];
				Asura_Vector_3& xPosition = xMesh->VertexDataPtr[vID].Position;
				Asura_Vector_3& xNormal = xMesh->NormalsDataPtr[p].VertexNormal[v];

				Tools_CharacterProcessor_Vertex xVertexData;

				xVertexData.m_X = xPosition.x;
				xVertexData.m_Y = xPosition.y;
				xVertexData.m_Z = xPosition.z;

				xVertexData.m_NX = xNormal.x;
				xVertexData.m_NY = xNormal.y;
				xVertexData.m_NZ = xNormal.z;
				if(iMirror)
				{
					xVertexData.m_NX  *= -1.0f; 
					xVertexData.m_NY  *= -1.0f; 
					xVertexData.m_NZ  *= -1.0f; 
				}

				const Asura_Mesh_TextureInfo* pxTexInfo = xMesh->GetTextureInfoForExport(p);
				xVertexData.m_fU = pxTexInfo->U[0][v];
				xVertexData.m_fV = pxTexInfo->V[0][v];

				bool lastbone = false;
				for (int b=0; b<4; b++)
				{
					if (xSkin->VertexData[vID].Weight[b]>0 && !lastbone)
					{
						xVertexData.m_afWeight[b] = xSkin->VertexData[vID].Weight[b];
						// careful of the * 3 hack... 
						xVertexData.m_acIndex[b] = (float)(xSkin->VertexData[vID].BoneID[b]*3.0f); //Kev's *3 hack here
					}
					else
					{
						lastbone = true;
						xVertexData.m_afWeight[b] = 0;
						xVertexData.m_acIndex[b] = 0;
					}
				}

				int i = uNumberOfUniqueVertices;
				bool bVertexNotUnique = false;
				while(i)
				{
					i--;

					if(pxWorkspaceVertices[i] == xVertexData)
					{
						bVertexNotUnique = true;
						break;
					}
				}
	
				if (bVertexNotUnique)
				{
					iIndexNumber[v] = i;
				}
				else
				{
					iIndexNumber[v] = uNumberOfUniqueVertices;

					pxWorkspaceVertices[uNumberOfUniqueVertices] = xVertexData;
					uNumberOfUniqueVertices++;
				}
				v++;
			}
			while(v<4 && xMesh->PolygonDataPtr[p].VertexID[v]!=0xffff);

			if(iMirror)
			{
				//reverse of double sided polygon , so reverse the vertex ordering.
				usIndexData[uIndex++] = (unsigned short)(iIndexNumber[2]);
				usIndexData[uIndex++] = (unsigned short)(iIndexNumber[1]);
				usIndexData[uIndex++] = (unsigned short)(iIndexNumber[0]);
	
				if (xMesh->PolygonDataPtr[p].IsQuad())
				{
					usIndexData[uIndex++] = (unsigned short)(iIndexNumber[0]);
					usIndexData[uIndex++] = (unsigned short)(iIndexNumber[3]);
					usIndexData[uIndex++] = (unsigned short)(iIndexNumber[2]);
				}
			}
			else
			{
				usIndexData[uIndex++] = (unsigned short)(iIndexNumber[0]);
				usIndexData[uIndex++] = (unsigned short)(iIndexNumber[1]);
				usIndexData[uIndex++] = (unsigned short)(iIndexNumber[2]);
	
				if (xMesh->PolygonDataPtr[p].IsQuad())
				{
					usIndexData[uIndex++] = (unsigned short)(iIndexNumber[2]);
					usIndexData[uIndex++] = (unsigned short)(iIndexNumber[3]);
					usIndexData[uIndex++] = (unsigned short)(iIndexNumber[0]);
				}
			}
			uNumberOfTriangles += xMesh->PolygonDataPtr[p].IsQuad() ? 2 : 1;

		}
	
	}

	//test
	for(u_int i=0; i<uIndex; i++)
	{
		Asura_Assert(usIndexData[i]>=0,"");
		Asura_Assert(usIndexData[i]<uNumberOfUniqueVertices,"");
	}

	u_int uNumberOfVertices = uNumberOfUniqueVertices;

    Tools_TriStripper::Initialise();
    Tools_TriStripper::SetOutputType( Tools_TriStripper::OUTPUT_TRISTRIP_WITH_RESET_INDICES );

	u_short* pusStripIndices;
    u_int uNumberOfStrippedIndices = Tools_TriStripper::CreateStrips( uNumberOfTriangles, usIndexData, &pusStripIndices );

	u_short* pusPermutationTable;
    Tools_TriStripper::ComputeVertexPermutation( uNumberOfStrippedIndices, pusStripIndices, uNumberOfVertices, &pusPermutationTable );

    Tools_TriStripper::Shutdown();

	uNumberOfTriangles = uNumberOfStrippedIndices-2;

	// create new strip for putting data in
	s_pxProcessedStrip = new Tools_CharacterProcessor_Strip;
	s_pxProcessedStrip->m_pxVertices = new Tools_CharacterProcessor_Vertex[uNumberOfVertices];
	s_pxProcessedStrip->m_pusIndices = pusStripIndices;
	
	// copy all the indexed vertex data
	for(u_int i=0; i<uNumberOfUniqueVertices; i++)
	{	
		u_short usIndex = pusPermutationTable[i];
		s_pxProcessedStrip->m_pxVertices[i].m_X = pxWorkspaceVertices[usIndex].m_X;
		s_pxProcessedStrip->m_pxVertices[i].m_Y = pxWorkspaceVertices[usIndex].m_Y;
		s_pxProcessedStrip->m_pxVertices[i].m_Z = pxWorkspaceVertices[usIndex].m_Z;
		
		s_pxProcessedStrip->m_pxVertices[i].m_NX = pxWorkspaceVertices[usIndex].m_NX;
		s_pxProcessedStrip->m_pxVertices[i].m_NY = pxWorkspaceVertices[usIndex].m_NY;
		s_pxProcessedStrip->m_pxVertices[i].m_NZ = pxWorkspaceVertices[usIndex].m_NZ;

		s_pxProcessedStrip->m_pxVertices[i].m_fU = pxWorkspaceVertices[usIndex].m_fU;
		s_pxProcessedStrip->m_pxVertices[i].m_fV = pxWorkspaceVertices[usIndex].m_fV;
		
		s_pxProcessedStrip->m_pxVertices[i].m_afWeight[0] = pxWorkspaceVertices[usIndex].m_afWeight[0];
		s_pxProcessedStrip->m_pxVertices[i].m_afWeight[1] = pxWorkspaceVertices[usIndex].m_afWeight[1];
		s_pxProcessedStrip->m_pxVertices[i].m_afWeight[2] = pxWorkspaceVertices[usIndex].m_afWeight[2];
		s_pxProcessedStrip->m_pxVertices[i].m_afWeight[3] = pxWorkspaceVertices[usIndex].m_afWeight[3];

		s_pxProcessedStrip->m_pxVertices[i].m_acIndex[0] = pxWorkspaceVertices[usIndex].m_acIndex[0];
		s_pxProcessedStrip->m_pxVertices[i].m_acIndex[1] = pxWorkspaceVertices[usIndex].m_acIndex[1];
		s_pxProcessedStrip->m_pxVertices[i].m_acIndex[2] = pxWorkspaceVertices[usIndex].m_acIndex[2];
		s_pxProcessedStrip->m_pxVertices[i].m_acIndex[3] = pxWorkspaceVertices[usIndex].m_acIndex[3];
	}	

	s_pxProcessedStrip->m_iMaterialID = xMesh->TextureDataPtr[0].m_iMaterialID;	
	s_pxProcessedStrip->m_uNumberOfIndices = uNumberOfStrippedIndices;
	s_pxProcessedStrip->m_uNumberOfTriangles = uNumberOfTriangles;
	s_pxProcessedStrip->m_uNumberOfVertices = uNumberOfVertices;

	s_bBeenInitialised = true;

	// BM - 29/03/2003 14:54:01 - We only have one texture per character at 
	// the moment. If we ever have more than one this will need to be changed.
	// The conversion mechanism means that the exported material index will be 0
	s_pxProcessedStrip->m_iMaterialID = bHasMaterial ? 0 : -1;
	
//	Tools_Level_Export::UpdateStatus("Stripifying Character", 100);

	// delete temp data
	delete [] usIndexData;
	delete [] pxWorkspaceVertices;

	delete [] pusPermutationTable;


	return true;
}


void Tools_CharacterProcessor::Shutdown(void)
{
	if (s_bBeenInitialised)
	{
			delete s_pxProcessedStrip;

			s_bBeenInitialised = false;
	}
	else
	{
		Asura_Assert( s_bBeenInitialised, "Not yet Initialised!" );
	}	
}

const Tools_CharacterProcessor_Strip* Tools_CharacterProcessor::GetProcessedStrip()
{
	if (s_bBeenInitialised)
	{
			return s_pxProcessedStrip;
	}
	else
	{
		Asura_Assert( s_bBeenInitialised, "Not yet Initialised!" );
		return 0;
	}
}

u_int Tools_CharacterProcessor::GetTotalNumberOfVertices()
{
	if (s_bBeenInitialised)
	{
			return s_pxProcessedStrip->m_uNumberOfVertices;
	}
	else
	{
		Asura_Assert( s_bBeenInitialised, "Not yet Initialised!" );
		return 0;
	}
}

u_int Tools_CharacterProcessor::GetTotalNumberOfIndices()
{
	if (s_bBeenInitialised)
	{
			return s_pxProcessedStrip->m_uNumberOfIndices;
	}
	else
	{
		Asura_Assert( s_bBeenInitialised, "Not yet Initialised!" );
		return 0;
	}
}
