//
// Tools_CharacterProcessor_NormalMapped.cpp
//
// Created by KJFL - 2004-07-26 16:08:05
//
// Copyright (C) 2004 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

// This file's header.
#include "MeshProcessing/Tools_CharacterProcessor_NormalMapped.h"

#include "Mesh/Asura_Mesh.h"
#include "Animation/Asura_Hierarchy.h"
#include "MeshProcessing/Asura_TriStripper.h"
#include "MeshProcessing/Tools_MeshRestartIndex.h"
#include "MeshProcessing/Tools_TangentSpace.h"
#include "Core/Asura_Material_System.h"

Tools_CharacterProcessor_Strip_NormalMapped*	Tools_CharacterProcessor_NormalMapped::s_pxProcessedStrip;
u_int											Tools_CharacterProcessor_NormalMapped::s_uTotalNumberOfVertices;
u_int											Tools_CharacterProcessor_NormalMapped::s_uTotalNumberOfIndices;
	bool										Tools_CharacterProcessor_NormalMapped::s_bBeenInitialised = false;
		
// The initialise function takes character mesh and skin and processes them
// into a single strip - the shutdown function cleans up all the memory created
// during the process
bool Tools_CharacterProcessor_NormalMapped::Initialise(Asura_Mesh* pxMesh, const Asura_Hierarchy_Skin* xSkin)
{
	if(!pxMesh || !xSkin)
	{
		Asura_Assert(pxMesh,"no mesh to process");
		Asura_Assert(xSkin,"no skin to process");
		return false;
	}
	Asura_Hierarchy_Skin* pxSkin = (Asura_Hierarchy_Skin* )xSkin;
	pxSkin->CalculateSkinNormalData();

//	Tools_Level_Export::UpdateStatus("Stripifying Character", 0);
	
	//Allow for enough space in case all triangles are double sided. RAR 30/01/2004
	u_int uTotalIndices = pxMesh->NumberOfTriangles*3*2;

	u_int uNumberOfUniqueVertices = 0;
	u_int uIndex = 0;
	u_int uNumberOfTriangles = 0;
	bool bHasMaterial = false;

	Tools_CharacterProcessor_Vertex_NormalMapped* pxWorkspaceVertices = new Tools_CharacterProcessor_Vertex_NormalMapped[uTotalIndices];
	unsigned short* usIndexData = new unsigned short[uTotalIndices];

	// fill up the workspace vertex buffer
	for (int p=0; p<pxMesh->NumberOfPolygons; p++)
	{	
		int iIndexNumber[4];

		int iPolyFlags = 0;

		if (pxMesh->PolygonFlagDataPtr)
		{
			iPolyFlags = pxMesh->PolygonFlagDataPtr[p].Flags;
		}

		if (iPolyFlags & AsuraPolygonFlag_DontRender) continue;

		bool bDoubleSided = (iPolyFlags & AsuraPolygonFlag_DoubleSided) ? true : false;

		if(pxMesh->TextureDataPtr[p].m_iMaterialID != ASURA_MATERIAL_NONE)
		{
			bHasMaterial = true;
		}

		// Calculate the tangent and binormal, per polygon (but store results per poly)
		Asura_Vector_3 xTangent;
		Asura_Vector_3 xBinormal;

		CalculateTangentSpaceVectors(pxMesh, p, xTangent, xBinormal);

		//If double sided we will need to do 2 passes of adding the vertices , with
		//normals and vertex ordering being reversed for the second side of the polygons
		//RAR 30/01/2004
		for( int iMirror=0;iMirror < (bDoubleSided ? 2 : 1);iMirror++)
		{
			int v=0;
			do
			{
		
				int vID =  pxMesh->PolygonDataPtr[p].VertexID[v];
				Asura_Vector_3& xPosition = pxMesh->VertexDataPtr[vID].Position;
				Asura_Vector_3& xNormal = pxMesh->NormalsDataPtr[p].VertexNormal[v];

				Tools_CharacterProcessor_Vertex_NormalMapped xVertexData;

				xVertexData.m_uColour	= pxMesh->PrelightingDataValid ? pxMesh->PrelightingDataPtr[p].Colour[v] : 0xffffffff;
				xVertexData.m_xPosition	= xPosition;
				xVertexData.m_xNormal	= xNormal;
				xVertexData.m_xTangent	= Tools_TangentSpace::ClampPerpendicularToNormal (xNormal, xTangent, pxMesh->Name.GetString());
				xVertexData.m_xBinormal	= Tools_TangentSpace::ClampPerpendicularToNormal (xNormal, xBinormal, pxMesh->Name.GetString());

				if(iMirror)
				{
					xVertexData.m_xNormal *= -1.f;
				}

				const Asura_Mesh_TextureInfo* pxTexInfo = pxMesh->GetTextureInfoForExport(p);
				xVertexData.m_fU = pxTexInfo->U[0][v];
				xVertexData.m_fV = pxTexInfo->V[0][v];

				bool lastbone = false;
				for (int b=0; b<4; b++)
				{
					if (xSkin->VertexData[vID].Weight[b]>0 && !lastbone)
					{
						xVertexData.m_afWeight[b] = xSkin->VertexData[vID].Weight[b];
						// careful of the * 3 hack... 
						xVertexData.m_afIndex[b] = (float)(xSkin->VertexData[vID].BoneID[b]*3.0f); //Kev's *3 hack here
					}
					else
					{
						lastbone = true;
						xVertexData.m_afWeight[b] = 0;
						xVertexData.m_afIndex[b] = 0;
					}
				}

				int i = uNumberOfUniqueVertices;
				bool bVertexNotUnique = false;
				while(i)
				{
					i--;

					if(pxWorkspaceVertices[i] == xVertexData)
					{
						bVertexNotUnique = true;
						break;
					}
				}
	
				if (bVertexNotUnique)
				{
					iIndexNumber[v] = i;
					pxWorkspaceVertices[i].m_xTangent += xVertexData.m_xTangent;
					pxWorkspaceVertices[i].m_xBinormal += xVertexData.m_xBinormal; 
				}
				else
				{
					iIndexNumber[v] = uNumberOfUniqueVertices;

					pxWorkspaceVertices[uNumberOfUniqueVertices] = xVertexData;
					uNumberOfUniqueVertices++;
				}
				v++;
			}
			while(v<4 && pxMesh->PolygonDataPtr[p].VertexID[v]!=0xffff);

			if(iMirror)
			{
				//reverse of double sided polygon , so reverse the vertex ordering.
				usIndexData[uIndex++] = (unsigned short)(iIndexNumber[2]);
				usIndexData[uIndex++] = (unsigned short)(iIndexNumber[1]);
				usIndexData[uIndex++] = (unsigned short)(iIndexNumber[0]);
	
				if (pxMesh->PolygonDataPtr[p].IsQuad())
				{
					usIndexData[uIndex++] = (unsigned short)(iIndexNumber[0]);
					usIndexData[uIndex++] = (unsigned short)(iIndexNumber[3]);
					usIndexData[uIndex++] = (unsigned short)(iIndexNumber[2]);
				}
			}
			else
			{
				usIndexData[uIndex++] = (unsigned short)(iIndexNumber[0]);
				usIndexData[uIndex++] = (unsigned short)(iIndexNumber[1]);
				usIndexData[uIndex++] = (unsigned short)(iIndexNumber[2]);
	
				if (pxMesh->PolygonDataPtr[p].IsQuad())
				{
					usIndexData[uIndex++] = (unsigned short)(iIndexNumber[2]);
					usIndexData[uIndex++] = (unsigned short)(iIndexNumber[3]);
					usIndexData[uIndex++] = (unsigned short)(iIndexNumber[0]);
				}
			}
			uNumberOfTriangles += pxMesh->PolygonDataPtr[p].IsQuad() ? 2 : 1;

		}
	
	}

	//test
	for(u_int i=0; i<uIndex; i++)
	{
		Asura_Assert(usIndexData[i]>=0,"");
		Asura_Assert(usIndexData[i]<uNumberOfUniqueVertices,"");
	}

	u_int uNumberOfVertices = uNumberOfUniqueVertices;

    Tools_TriStripper::Initialise();
	Tools_TriStripper::SetOutputType( Tools_TriStripper::OUTPUT_TRISTRIP_WITH_RESET_INDICES );

	u_short* pusStripIndices;
    u_int uNumberOfStrippedIndices = Tools_TriStripper::CreateStrips( uNumberOfTriangles, usIndexData, &pusStripIndices );

	u_short* pusPermutationTable;
	Tools_TriStripper::ComputeVertexPermutation( uNumberOfStrippedIndices, pusStripIndices, uNumberOfVertices, &pusPermutationTable );

    Tools_TriStripper::Shutdown();

	uNumberOfTriangles = uNumberOfStrippedIndices-2;

	// create new strip for putting data in
	s_pxProcessedStrip = new Tools_CharacterProcessor_Strip_NormalMapped;
	s_pxProcessedStrip->m_pxVertices = new Tools_CharacterProcessor_Vertex_NormalMapped[uNumberOfVertices];
	s_pxProcessedStrip->m_pusIndices = pusStripIndices;

	// copy all the indexed vertex data
	for(u_int i=0; i<uNumberOfUniqueVertices; i++)
	{	
		u_short usIndex = pusPermutationTable[i];
		s_pxProcessedStrip->m_pxVertices[i].m_xPosition = pxWorkspaceVertices[usIndex].m_xPosition;
		s_pxProcessedStrip->m_pxVertices[i].m_xNormal = pxWorkspaceVertices[usIndex].m_xNormal;

		s_pxProcessedStrip->m_pxVertices[i].m_xTangent = pxWorkspaceVertices[usIndex].m_xTangent;
		s_pxProcessedStrip->m_pxVertices[i].m_xTangent.Normalise();

		s_pxProcessedStrip->m_pxVertices[i].m_xBinormal = pxWorkspaceVertices[usIndex].m_xBinormal;
		s_pxProcessedStrip->m_pxVertices[i].m_xBinormal.Normalise();

		s_pxProcessedStrip->m_pxVertices[i].m_fU = pxWorkspaceVertices[usIndex].m_fU;
		s_pxProcessedStrip->m_pxVertices[i].m_fV = pxWorkspaceVertices[usIndex].m_fV;
		
		s_pxProcessedStrip->m_pxVertices[i].m_afWeight[0] = pxWorkspaceVertices[usIndex].m_afWeight[0];
		s_pxProcessedStrip->m_pxVertices[i].m_afWeight[1] = pxWorkspaceVertices[usIndex].m_afWeight[1];
		s_pxProcessedStrip->m_pxVertices[i].m_afWeight[2] = pxWorkspaceVertices[usIndex].m_afWeight[2];
		s_pxProcessedStrip->m_pxVertices[i].m_afWeight[3] = pxWorkspaceVertices[usIndex].m_afWeight[3];

		s_pxProcessedStrip->m_pxVertices[i].m_afIndex[0] = pxWorkspaceVertices[usIndex].m_afIndex[0];
		s_pxProcessedStrip->m_pxVertices[i].m_afIndex[1] = pxWorkspaceVertices[usIndex].m_afIndex[1];
		s_pxProcessedStrip->m_pxVertices[i].m_afIndex[2] = pxWorkspaceVertices[usIndex].m_afIndex[2];
		s_pxProcessedStrip->m_pxVertices[i].m_afIndex[3] = pxWorkspaceVertices[usIndex].m_afIndex[3];

		s_pxProcessedStrip->m_pxVertices[i].m_uColour = pxWorkspaceVertices[usIndex].m_uColour;
	}	

	s_pxProcessedStrip->m_iMaterialID = pxMesh->TextureDataPtr[0].m_iMaterialID;	
	s_pxProcessedStrip->m_uNumberOfIndices = uNumberOfStrippedIndices;
	s_pxProcessedStrip->m_uNumberOfTriangles = uNumberOfTriangles;
	s_pxProcessedStrip->m_uNumberOfVertices = uNumberOfVertices;

	s_bBeenInitialised = true;

	// BM - 29/03/2003 14:54:01 - We only have one texture per character at 
	// the moment. If we ever have more than one this will need to be changed.
	// The conversion mechanism means that the exported material index will be 0
	s_pxProcessedStrip->m_iMaterialID = bHasMaterial ? 0 : -1;

	s_pxProcessedStrip->m_bHasColour = pxMesh->PrelightingDataValid;
	
//	Tools_Level_Export::UpdateStatus("Stripifying Character", 100);

	// delete temp data
	delete [] usIndexData;
	delete [] pxWorkspaceVertices;

	delete [] pusPermutationTable;


	return true;
}


void Tools_CharacterProcessor_NormalMapped::Shutdown(void)
{
	if (s_bBeenInitialised)
	{
			delete s_pxProcessedStrip;

			s_bBeenInitialised = false;
	}
	else
	{
		Asura_Assert( s_bBeenInitialised, "Not yet Initialised!" );
	}	
}

const Tools_CharacterProcessor_Strip_NormalMapped* Tools_CharacterProcessor_NormalMapped::GetProcessedStrip()
{
	if (s_bBeenInitialised)
	{
			return s_pxProcessedStrip;
	}
	else
	{
		Asura_Assert( s_bBeenInitialised, "Not yet Initialised!" );
		return 0;
	}
}

u_int Tools_CharacterProcessor_NormalMapped::GetTotalNumberOfVertices()
{
	if (s_bBeenInitialised)
	{
			return s_pxProcessedStrip->m_uNumberOfVertices;
	}
	else
	{
		Asura_Assert( s_bBeenInitialised, "Not yet Initialised!" );
		return 0;
	}
}

u_int Tools_CharacterProcessor_NormalMapped::GetTotalNumberOfIndices()
{
	if (s_bBeenInitialised)
	{
			return s_pxProcessedStrip->m_uNumberOfIndices;
	}
	else
	{
		Asura_Assert( s_bBeenInitialised, "Not yet Initialised!" );
		return 0;
	}
}

void Tools_CharacterProcessor_NormalMapped::CalculateTangentSpaceVectors(const class Asura_Mesh* pxMesh, u_int uPoly, Asura_Vector_3& xTangent, Asura_Vector_3& xBinormal)
{
	Asura_Vector_3	axPositions [3];
	Asura_Vector_2	axTextureCoords [3];


	// Initialise the vertex-data...

	for (u_int u = 0; u < 3; u++)
	{
		u_int	uVertexID	= pxMesh->PolygonDataPtr [uPoly].VertexID [u];


		// Fill in the data for this vertex...

		axPositions [u]			= pxMesh->VertexDataPtr [uVertexID].Position;

		const Asura_Mesh_TextureInfo* pxTexInfo = pxMesh->GetTextureInfoForExport(uPoly);
		axTextureCoords [u].x	= pxTexInfo->U [0][u];
		axTextureCoords [u].y	= 1.0f - pxTexInfo->V [0][u];
	}


	// Calculate the tangent AND binormal...

	Tools_TangentSpace::CalculateTangentAndBinormal	(axPositions, axTextureCoords, xTangent, xBinormal);
}
