//
// Tools_EnvironmentProcessor_SM3.h
//
// Created by KJFL - 23 April 2009
//
// Copyright (C) 2009 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __TOOLS_ENVIRONMENTPROCESSOR_SM3_H__
#define __TOOLS_ENVIRONMENTPROCESSOR_SM3_H__

#include "Core/Asura_Maths.h"
#include "Mesh/Asura_Mesh.h"

#include "Misc/Tools_HashTable.h"

class Tools_EP_Module;
class Tools_EP_Packet;

class Tools_EnvironmentProcessor_SM3
{
	public:
		
		Tools_EnvironmentProcessor_SM3();

		~Tools_EnvironmentProcessor_SM3();

		u_int GetNumberOfPackets() const { return m_uNumberOfPackets; }

		Tools_EP_Packet* GetPacketArray() const { return m_pxPackets; }

		u_int GetTotalNumberOfVertices() const { return m_uTotalNumberOfVertices; }
		u_int GetTotalNumberOfIndices() const { return m_uTotalNumberOfIndices; }

		u_int GetNumberOfVertexBuffers() const { return m_uNumberOfVertexBuffers; } 
		u_int GetSizeOfVertexBuffer( u_int u ) const { return m_puVertexBufferSizes[u]; }

		u_int GetNumberOfIndexBuffers() const { return m_uNumberOfIndexBuffers; } 
		u_int GetSizeOfIndexBuffer( u_int u ) const { return m_puIndexBufferSizes[u]; }


	private:

		void ProcessModule( int iModuleIndex );
		void SortPackets( void );

		Tools_EP_Module*	m_pxModules;

		Tools_EP_Packet*	m_pxPackets;
		u_int				m_uNumberOfPackets;

		u_int				m_uTotalNumberOfIndices;
		u_int				m_uTotalNumberOfVertices;

		u_int				m_uNumberOfIndexBuffers;
		u_int				m_uNumberOfVertexBuffers;

		u_int*				m_puVertexBufferSizes;
		u_int*				m_puIndexBufferSizes;


};


class Tools_EP_Vertex
{
	public:
	
		Asura_Hash_ID GetHashID() const { return m_xHash; } 

		//bool operator==( const Tools_EP_Vertex& xOther ) const

		bool ApproximatelyEquals( const Tools_EP_Vertex& xOther, const bool bDualMaterial ) const
		{
#if 0
			if ( m_xHash != xOther.m_xHash ) return false;

			if	(  ( m_uDiffuseRGBA != xOther.m_uDiffuseRGBA )
				|| ( m_fU != xOther.m_fU ) 
				|| ( m_fV != xOther.m_fV )
				|| ( m_fU2 != xOther.m_fU2 ) 
				|| ( m_fV2 != xOther.m_fV2 )
				|| ( m_xPosition != xOther.m_xPosition ) 
				|| ( m_xNormal != xOther.m_xNormal ) 
				|| ( m_xBinormal != xOther.m_xBinormal ) 
				|| ( m_xTangent != xOther.m_xTangent ) 
				|| ( m_xBinormal2 != xOther.m_xBinormal2 ) 
				|| ( m_xTangent2 != xOther.m_xTangent2 ) )
			{
				return false;
			}

			for (u_int u=0; u<Asura_SphericalHarmonics_Order2_RGB::NUMBER_OF_COEFFICIENTS; ++u)
			{
				if ( m_xDiffuseSH.m_afCoefficient[u] != xOther.m_xDiffuseSH.m_afCoefficient[u] )
				{
					return false;
				}
			}
#endif

			const float fPosTol = 0.0025f * 0.0025f; // +/- 0.25cm.
		    const float fNrmTol = 0.996194698092f;   // +/- 5 (where this value is cos 5)

			if ( !m_xPosition.ApproximatelyEquals(xOther.m_xPosition, fPosTol ) ) return false;
			//if ( !m_xNormal.ApproximatelyEquals(xOther.m_xNormal) ) return false;
#if 01
			if ( m_xNormal * xOther.m_xNormal < fNrmTol )
			if(!(m_xNormal.ApproximatelyEquals(Asura_Maths::ZeroVector3) && xOther.m_xNormal.ApproximatelyEquals(Asura_Maths::ZeroVector3)))
			{
				return false;
			}	

			if ( m_xBinormal * xOther.m_xBinormal < fNrmTol )
			{
				if(!(m_xBinormal.ApproximatelyEquals(Asura_Maths::ZeroVector3) && xOther.m_xBinormal.ApproximatelyEquals(Asura_Maths::ZeroVector3)))
				{
					return false;
				}	
			}
			
			if ( m_xTangent * xOther.m_xTangent < fNrmTol )
			{
				if(!(m_xTangent.ApproximatelyEquals(Asura_Maths::ZeroVector3) && xOther.m_xTangent.ApproximatelyEquals(Asura_Maths::ZeroVector3)))
				{
					return false;
				}	
			}
			
			const float fUVTol = 0.01f;
			
			if ( bDualMaterial)
			{
				if ( m_xBinormal2 * xOther.m_xBinormal2 < fNrmTol )
				{
					if(!(m_xBinormal2.ApproximatelyEquals(Asura_Maths::ZeroVector3) && xOther.m_xBinormal2.ApproximatelyEquals(Asura_Maths::ZeroVector3)))
					{
						return false;
					}	
				}
				
				if ( m_xTangent2 * xOther.m_xTangent2 < fNrmTol )
				{
					if(!(m_xTangent2.ApproximatelyEquals(Asura_Maths::ZeroVector3) && xOther.m_xTangent2.ApproximatelyEquals(Asura_Maths::ZeroVector3)))
					{
						return false;
					}	
				}

							
				if ( ( Asura_Maths::Abs(m_fU2 - xOther.m_fU2 ) > fUVTol ) 
					|| ( Asura_Maths::Abs(m_fV2 - xOther.m_fV2 ) > fUVTol ) )
				{
					return false;
				}
			
			}
#endif



			if	(  ( Asura_Maths::Abs( (int)(m_uDiffuseRGBA>>24) - (int)(xOther.m_uDiffuseRGBA>>24) ) > 2 )
				|| ( Asura_Maths::Abs(m_fU - xOther.m_fU ) > fUVTol )
				|| ( Asura_Maths::Abs(m_fV - xOther.m_fV ) > fUVTol ) )
			{
				return false;
			}


			
			for (u_int u=0; u<Asura_SphericalHarmonics_Order2_RGB::NUMBER_OF_COEFFICIENTS; ++u)
			{
				if ( Asura_Maths::Abs( m_xDiffuseSH.m_afCoefficient[u] - xOther.m_xDiffuseSH.m_afCoefficient[u]) > 0.01f )
				{
					return false;
				}
			}

			return true;
		}

		void SetHashID()
		{
			m_xHash = 0;

			m_xHash = Asura_GetHashID_Binary( this, sizeof(this) );
		}
	
	   		
		Asura_Vector_3						m_xPosition;
		
		Asura_Vector_3						m_xNormal;
		Asura_Vector_3						m_xTangent;
		Asura_Vector_3						m_xBinormal;
		Asura_Vector_3						m_xTangent2;
		Asura_Vector_3						m_xBinormal2;

		float								m_fU;
		float								m_fV;
		float								m_fU2;
		float								m_fV2;

		u_int								m_uDiffuseRGBA;

		Asura_SphericalHarmonics_Order2_RGB m_xDiffuseSH;

		Asura_Hash_ID						m_xHash;
};


class Tools_EP_Packet
{
	public:

		Tools_EP_Packet()
		{
			m_pxVertices = 0;
			m_pusIndices = 0;
			m_pxNextPacket = 0;

			m_pxVertexHashTable = 0;
		}
		
		Tools_EP_Packet( const Asura_Hash_ID* puHashes, int iModuleIndex )
		{
			for ( u_int u=0; u<TOOLS_NUM_MATERIALRESPONSE_PER_POLY; ++u )
			{
				m_auMaterialResponseHashID[u] = puHashes[u];
			}

			m_uNumberOfIndices = 0;
			m_uNumberOfTriangles = 0;
			m_uNumberOfVertices = 0;
			
			m_uOriginalModule = static_cast<u_int>(iModuleIndex);

			m_pxVertices = 0;
			m_pusIndices = 0;

			m_pxNextPacket = 0;

			m_pxVertexHashTable = new Tools_HashTable<Tools_EP_Vertex>(256);
		}
		
		~Tools_EP_Packet()
		{
			delete [] m_pxVertices;
			delete [] m_pusIndices;
			
			delete m_pxNextPacket;

			delete m_pxVertexHashTable;
		}

		bool CompareHashes( const Asura_Hash_ID* puHashes ) const
		{
			for ( u_int u=0; u<TOOLS_NUM_MATERIALRESPONSE_PER_POLY; ++u )
			{
				if ( puHashes[u] != m_auMaterialResponseHashID[u] ) return false;
			}

			return true;
		}
		
		u_int AddVertex( const Tools_EP_Vertex& xVertex, bool bDualMaterial )
		{
#if 1
			for (u_int u=0; u<m_uNumberOfVertices; ++u)
			{
				if (m_pxVertices[u].ApproximatelyEquals(xVertex,bDualMaterial))
				{
					return u;
				}
			}
			
			return m_uNumberOfVertices++;

#endif
#if 0
			const Tools_EP_Vertex* pxUsedSlot = m_pxVertexHashTable->Insert( &xVertex );

			// If it wasn't a duplicate, the vertex is in the right position
			// so we return the index, and then increment the count.
			if( &xVertex == pxUsedSlot ) 
			{
				return m_uNumberOfVertices++; 
			}

			// Otherwise we're reusing a previous vertex, and we need to
			// find the index from the pointers.
			return (u_int)(pxUsedSlot-m_pxVertices);
#endif
		}

		void AddIndex( u_int uIndex )
		{
			m_pusIndices[ m_uNumberOfIndices++ ] = static_cast<u_short>(uIndex);
		}

		Asura_Hash_ID		m_auMaterialResponseHashID[TOOLS_NUM_MATERIALRESPONSE_PER_POLY];

		u_int				m_uNumberOfIndices;
		u_int				m_uNumberOfTriangles;
		u_int				m_uNumberOfVertices;
		u_int				m_uOriginalModule;
		
		Tools_EP_Vertex*	m_pxVertices;
		u_short*			m_pusIndices;

		u_int				m_uVertexBuffer;
		u_int				m_uVertexBuffer_Offset;

		u_int				m_uIndexBuffer;
		u_int				m_uIndexBuffer_Offset;

		Tools_EP_Packet*	m_pxNextPacket;

		Tools_HashTable<Tools_EP_Vertex>* m_pxVertexHashTable;
};

class Tools_EP_Module
{
	public:

		Tools_EP_Module()
		{
			m_pxFirstPacket = 0;
			m_uNumberOfPackets = 0;
		}

		~Tools_EP_Module()
		{
			delete m_pxFirstPacket;
		}

		Tools_EP_Packet*	m_pxFirstPacket;
		u_int				m_uNumberOfPackets;
};

#endif//__TOOLS_ENVIRONMENTPROCESSOR_SM3_H__

