//
// Tools_ObjectProcessor_NormalMapped.h
//
// Created by AJG - 22/06/2005
//
// Copyright (C) 2005 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif

#ifndef __TOOLS_OBJECTPROCESSOR_NORMALMAPPED_H__
#define __TOOLS_OBJECTPROCESSOR_NORMALMAPPED_H__

#include "Core/Asura_Maths.h"

#include "MeshProcessing/Asura_TriStripper.h"

class Asura_Mesh;
class Asura_Hierarchy_Skin;

const float fTOOLS_OBJECTPROCESSOR_NORMALMAPPED_VERTEX_EPSILON_SQD = 0.005f * 0.005f;

class Tools_ObjectProcessor_NormalMapped_Vertex
{
	public:
		Asura_Vector_3 m_xPosition;
		Asura_Vector_3 m_xNormal;
		Asura_Vector_3 m_xTangent;
		Asura_Vector_3 m_xBinormal;
		float			m_fU;
		float			m_fV;
		float			m_fU2;
		float			m_fV2;
		float			m_fAlpha;

		inline bool operator == ( const Tools_ObjectProcessor_NormalMapped_Vertex& v ) const
		{
			Asura_Vector_2 xUVDelta1( m_fU, m_fV );
			xUVDelta1.x -= v.m_fU;
			xUVDelta1.y -= v.m_fV;
			Asura_Vector_2 xUVDelta2( m_fU2, m_fV2 );
			xUVDelta2.x -= v.m_fU2;
			xUVDelta2.y -= v.m_fV2;
			// Normal, tangent and binormal should be created orthogonal, so only need to check if one is different

			return xUVDelta1.MagnitudeSquared() < fTOOLS_OBJECTPROCESSOR_NORMALMAPPED_VERTEX_EPSILON_SQD
 				&& xUVDelta2.MagnitudeSquared() < fTOOLS_OBJECTPROCESSOR_NORMALMAPPED_VERTEX_EPSILON_SQD
				&& m_xPosition.ApproximatelyEquals( v.m_xPosition )
				&& m_xNormal.ApproximatelyEquals( v.m_xNormal );
		}
};

class Tools_ObjectProcessor_NormalMapped_Strip
{
	public:
		
		Tools_ObjectProcessor_NormalMapped_Strip()
		{
			m_pxVertices = 0;
			m_pusIndices = 0;
		}
		
		~Tools_ObjectProcessor_NormalMapped_Strip()
		{
			delete [] m_pxVertices;
			delete [] m_pusIndices;
		}

		u_int										m_uNumberOfIndices;
		u_int										m_uNumberOfTriangles;
		u_int										m_uNumberOfVertices;
		Asura_Vector_3								m_xBBMin;
		Asura_Vector_3								m_xBBMax;

		// Bit awkward - botanicals don't use the material response system yet,
		// but can be processed as shapes, so we need both items kicking around
		Asura_Hash_ID								m_auMaterialResponseHashID[2];
		int											m_iMaterialID;

		Tools_ObjectProcessor_NormalMapped_Vertex*	m_pxVertices;
		u_short*									m_pusIndices;
};



class Tools_ObjectProcessor_NormalMapped
{
	public:
		static bool Initialise(Asura_Mesh* pxMesh,
							   int iMaterialIndex,
							   bool bStripify = true,
							   Tools_TriStripper::OUTPUT eOutputType = Tools_TriStripper::OUTPUT_TRISTRIP_WITH_RESET_INDICES,
							   Asura_Hash_ID uMaterialResponseHashID1 = ASURA_HASH_ID_UNSET,
							   Asura_Hash_ID uMaterialResponseHashID2 = ASURA_HASH_ID_UNSET);

		static void Shutdown(void);

		static const Tools_ObjectProcessor_NormalMapped_Strip* GetProcessedStrip(void);
		static u_int GetTotalNumberOfVertices(void);
		static u_int GetTotalNumberOfIndices(void);
		static u_int GetTotalNumberOfStrips(void);
		
		static Tools_ObjectProcessor_NormalMapped_Strip*	s_pxProcessedStrip;
		static u_int										s_uTotalNumberOfVertices;
		static u_int										s_uTotalNumberOfIndices;
		static u_int										s_uTotalNumberOfTriangles;
		static u_int										s_uNumberOfStrips;
		static bool											s_bBeenInitialised;
	
	private:

		static bool MeshProcess(Asura_Mesh* pxMesh, u_int uStrip);

		static void CalculateTangentSpaceVectors(const class Asura_Mesh* pxMesh, u_int uPoly, Asura_Vector_3& xTangent, Asura_Vector_3& xBinormal);
};

#endif//__TOOLS_OBJECTPROCESSOR_NORMALMAPPED_H__
