//////////////////////////////////////////////////////////////////////////////
//
// Tools_SmoothSkin_Processor_PS2.cpp
//
// Copyright (c) 2004 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header.
#include "MeshProcessing/Tools_SmoothSkin_Processor_PS2.h"

// Stdlib independent.
#include "Core/Asura_MemoryManagement_DebugOff.h" // Std headers define their own new and delete operators and our macros break them.
#pragma warning( push )
#pragma warning( disable : 4702 ) // unreachable code
#include <algorithm>
#pragma warning( pop )
#include "Core/Asura_MemoryManagement_DebugOn.h"

// Stdlib platform-specific.
#include <windows.h>

// Engine independent.
#include "Animation/Asura_Hierarchy.h"
#include "Core/Asura_Material_System.h"
#include "Core/Asura_TextureManagement.h"
#include "Debugging/Asura_Assert.h"

// Engine PS2-specific.
#include "PS2/Animation/Asura_PS2_SkinTypes.h"
#include "PS2/ChunkSystem/Asura_Chunk_ResourceFile_PS2.h"
#include "PS2/ChunkSystem/Asura_Chunk_ResourceFile_PS2_StrippedSmoothSkin.h"

// Tools.
#include "Load&Save/LevelExport/Tools_Level_Export.h"
#include "MeshProcessing/Tools_SmoothSkin_Processor.h"
#include "PS2/PS2Strip/NVTriStrip/NvTriStripObjects.h"
#include "PS2/PS2Strip/PS2Strip_GeometryTypes.h"
#include "PS2/PS2Strip/PS2Strip_PS2Stuff.h"
#include "PS2/PS2Strip/PS2Strip_ToolsInterface.h"
#include "PS2/PS2Strip/Tools_PS2_DMAChainWriter.h"

// M A C R O S ///////////////////////////////////////////////////////////////

#define WRITE_DATA( file_handle, cumulative_count, data_ptr, num_bytes ) \
{ \
    const int iNumBytes = static_cast<int>( num_bytes ); \
    Asura_Assert( iNumBytes > 0, "" ); \
    DWORD uWritten; \
    if( !WriteFile((file_handle), (data_ptr), iNumBytes, &uWritten, NULL) || static_cast<int>(uWritten) != iNumBytes ) \
    { \
        PS2Strip::LogError( "ERROR: Failed to write all bytes to file. Reason: %s\n", PS2Strip::GetLastSystemError() ); \
        return false; \
    } \
    (cumulative_count) += iNumBytes; \
}

#define SKIP_DATA( file_handle, cumulative_count, num_bytes ) \
{ \
    const int iNumBytes = static_cast<int>( num_bytes ); \
    Asura_Assert( iNumBytes > 0, "" ); \
    SetFilePointer( (file_handle), iNumBytes, NULL, FILE_CURRENT ); \
    (cumulative_count) += iNumBytes; \
}

// C L A S S E S /////////////////////////////////////////////////////////////

class MaterialIdxCmp
{

    public:

        // C O N S T R U C T O R S   /   D E S T R U C T O R S ///////////////

        MaterialIdxCmp( const Tools_SmoothSkin_Processor* pxSrcProcessor ) : m_pxSrcProcessor( pxSrcProcessor ) { }

        // O P E R A T O R S /////////////////////////////////////////////////

        bool operator()( u_int uMaterialIdx0, u_int uMaterialIdx1 ) const
        {
            ASURA_PS2_SMOOTH_SKIN_RENDERPASS aeRenderPasses[2];

            for( u_int u = 0; u < 2; ++u )
            {
                const Tools_SmoothSkin_Material& xMat = m_pxSrcProcessor->GetMaterial( u ? uMaterialIdx1 : uMaterialIdx0 );
            
                const u_int uMaterialFlags = Asura_Material_System::GetMaterialFlags( xMat.GetMaterialID() );

                if( uMaterialFlags & AsuraMaterialFlag_PS2AdditiveAlpha )
                {
                    aeRenderPasses[u] = ASURA_PS2_SMOOTH_SKIN_RENDERPASS_ADDITIVE_ALPHA;
                }
                else if( uMaterialFlags & AsuraMaterialFlag_Transparent )
                {
                    bool bHas8BitAlpha = false;
                    Tools_Level_Export::DoesTIM2Have8BitAlpha( Asura_Material_System::GetTextureID(xMat.GetMaterialID()), &bHas8BitAlpha );
                    aeRenderPasses[u] = ( bHas8BitAlpha ? ASURA_PS2_SMOOTH_SKIN_RENDERPASS_TRANSLUCENT : ASURA_PS2_SMOOTH_SKIN_RENDERPASS_ON_OFF_TRANSPARENCY );
                }
                else
                {
                    aeRenderPasses[u] = ASURA_PS2_SMOOTH_SKIN_RENDERPASS_OPAQUE;
                }
            }

            return( aeRenderPasses[0] < aeRenderPasses[1] );
        }

    private:
        
        // S T A T E /////////////////////////////////////////////////////////

        const Tools_SmoothSkin_Processor* m_pxSrcProcessor;

};
// End class MaterialIdxCmp.

class ProcessedStrip
{

    public:

        // C L A S S E S /////////////////////////////////////////////////////

        class Cmp;

        // C O N S T R U C T O R S   /   D E S T R U C T O R S ///////////////

        ProcessedStrip( void );
        ~ProcessedStrip( void );

        // O P E R A T O R S /////////////////////////////////////////////////

        bool operator < ( const ProcessedStrip& xOther ) const { return m_uLength < xOther.m_uLength; }

        // M E T H O D S /////////////////////////////////////////////////////

        bool Initialise( u_int uMaxLength );

        // S T A T E /////////////////////////////////////////////////////////

        u_short* m_pusIndices;           // Index back into main vertex list in src skin.
        bool*    m_pbADCs;               // Each entry is true iff ADC bit should be set for that vertex.
        u_int    m_uLength;              // In vertices.
        u_int    m_uMaxWeightingsPerVtx;

};
// End class ProcessedStrip.

class ProcessedStrip::Cmp
{

    public:

        bool operator()( const ProcessedStrip* pxProcessedStripA, const ProcessedStrip* pxProcessedStripB ) const
        {
            if( pxProcessedStripA->m_uLength == pxProcessedStripB->m_uLength )
            {
                return( pxProcessedStripA->m_uMaxWeightingsPerVtx < pxProcessedStripB->m_uMaxWeightingsPerVtx );
            }
            else
            {
                return( pxProcessedStripA->m_uLength < pxProcessedStripB->m_uLength );
            }
        }

};
// End class ProcessedStrip::Cmp.

//// Requires that T defines a < operator. Internally the heap stores pointers
//// to the inserted data, rather than copying it.
//template< typename T >
//class MinHeap
//{
//
//  public:
//
//      // C O N S T R U C T O R S   /   D E S T R U C T O R S ///////////////
//
//      MinHeap( u_int uCapacity );
//      ~MinHeap( void );
//
//      // M E T H O D S /////////////////////////////////////////////////////
//
//      bool Insert( T* pxT );
//
//      T* RemoveMin( void );
//      void RemoveAll( void ) { m_uNumElements = 0; }
//
//      u_int GetNumElements( void ) const { return m_uNumElements; }
//      u_int GetElementIdx( const T* pxT ) const;
//      T* GetMin( void ) const { return( m_uNumElements ? m_ppxElements[0] : NULL ); }
//
//      void SetElementModified( T* pxT );
//      void SetMinModified( void ) { SiftDown( 0 ); }
//
//  private:
//
//      // M E T H O D S /////////////////////////////////////////////////////
//
//      void SiftDown( u_int uElementIdx );
//      void SiftUp( u_int uElementIdx );
//
//      // S T A T E /////////////////////////////////////////////////////////
//
//      T**   m_ppxElements;
//      u_int m_uCapacity;
//      u_int m_uNumElements;
//
//};
//// End class MinHeap.

class Tools_SmoothSkin_PS2_Processor::SubElementStripData
{

    public:

        // C O N S T R U C T O R S   /   D E S T R U C T O R S ///////////////

        SubElementStripData( void );
        ~SubElementStripData( void );

        // M E T H O D S /////////////////////////////////////////////////////

        bool ProcessStripifierOutput( const Tools_SmoothSkin_PS2_Processor* pxProcessor, u_int uMaxStripLength, u_int uTotalInputVerts );

        // S T A T E /////////////////////////////////////////////////////////

        u_int                      m_uElementIdx;
        NVTriStrip::NvStripInfoVec m_xUnprocessedStrips;
        ProcessedStrip*            m_pxProcessedStrips;
        ProcessedStrip**           m_ppxSortedProcessedStrips;
        u_int                      m_uNumProcessedStrips;

};
// End class Tools_SmoothSkin_PS2_Processor::SubElementStripData.

// P R O T O T Y P E S   A N D   I N L I N E   F U N C T I O N S /////////////

static void  AppendIndicesToProcessedStrip( ProcessedStrip* pxProcessedStrip, const u_int* puVtxIndices, u_int uNumVerts );
static u_int CountMaxWeightingsForVtxIndices( const Tools_SmoothSkin_Processor* pxSrcProcessor, const u_int* puVtxIndices, u_int uNumVerts );
static void  PrepareVtxIndicesFromUnprocessedStrip( const NVTriStrip::NvStripInfo* pxInputStrip, u_int* puIdxOut );

// Returns ceil(A/B)
static inline u_int DivideAndRoundUp( u_int uA, u_int uB )
{
    return( (uA + (uB - 1)) / uB );
}

static const char* GetMaterialName( int iMaterialID )
{
    const char* szMaterialName;

    if( ASURA_MATERIAL_NONE == iMaterialID )
    {
        szMaterialName = "-none-";
    }
    else
    {
        szMaterialName = Asura_Material_System::GetMaterial(iMaterialID)->m_xMaterialName.GetString();
        if( !szMaterialName ) { szMaterialName = "-unnamed-"; }
    }

    return szMaterialName;
}

// C L A S S   I M P L E M E N T A T I O N S /////////////////////////////////

//
// ProcessedStrip.
//


ProcessedStrip::ProcessedStrip( void )
    : m_pusIndices( NULL )
    , m_pbADCs( NULL )
    , m_uLength( 0 )
    , m_uMaxWeightingsPerVtx( 0 )
{
}

ProcessedStrip::~ProcessedStrip( void )
{
    delete[] m_pbADCs;
    delete[] m_pusIndices;
}

bool ProcessedStrip::Initialise( u_int uMaxLength )
{
    Asura_Assert( !m_pusIndices && !m_pbADCs, "" );

    m_pusIndices = new u_short[uMaxLength];
    m_pbADCs     = new bool[uMaxLength];

    return( m_pusIndices && m_pbADCs );
}

////
//// MinHeap.
////
//
//template< typename T >
//MinHeap<T>::MinHeap( u_int uCapacity )
//{
//  m_ppxElements  = new T*[uCapacity];
//  m_uCapacity    = ( m_ppxElements ? uCapacity : 0 );
//  m_uNumElements = 0;
//}
//
//template< typename T >
//MinHeap<T>::~MinHeap( void )
//{
//  delete[] m_ppxElements;
//}
//
//template< typename T >
//u_int MinHeap<T>::GetElementIdx( const T* pxT ) const
//{
//  if( !m_uNumElements ) { return m_uNumElements; } // Obviously can't be in an empty heap.
//
//  u_int uElementIdx = 0;
//  while( true )
//  {
//      const T* pxThisElement = m_ppxElements[uElementIdx];
//
//      // Ignore this entire branch if the element we're searching for is
//      // less than the root of it.
//      if( !(*pxT < *pxThisElement) )
//      {
//          // Match?
//          if( pxT == pxThisElement ) { return uElementIdx; }
//
//          // Recurse to left child, if it exists.
//          u_int uLeftChild = (uElementIdx << 1) + 1;
//          if( uLeftChild < m_uNumElements )
//          {
//              uElementIdx = uLeftChild;
//              continue;
//          }
//      }
//
//      // Head back up until we either reach the root or find a parent that
//      // we're in the left branch of.
//      while( true )
//      {
//          // If we're back up to the root then we're done and the element
//          // is not in the heap.
//          if( !uElementIdx ) { return m_uNumElements; }
//
//          // If we're a left child, head across to our sibling, if it
//          // exists.
//          if( uElementIdx & 1 )
//          {
//              ++uElementIdx;
//              if( uElementIdx < m_uNumElements )
//              {
//                  break;
//              }
//          }
//
//          // Otherwise just move up to parent and repeat.
//          uElementIdx = (uElementIdx - 1) >> 1;
//      }
//  }
//}
//
//template< typename T >
//bool MinHeap<T>::Insert( T* pxNewElement )
//{
//  if( m_uNumElements >= m_uCapacity ) { return false; }
//
//  m_ppxElements[m_uNumElements++] = pxNewElement;
//  SiftUp( m_uNumElements - 1 );
//  return true;
//}
//
//template< typename T >
//T* MinHeap<T>::RemoveMin( void )
//{
//  if( !m_uNumElements ) { return NULL; }
//
//  T* const pxOldMin = m_ppxElements[0];
//
//  if( m_uNumElements > 1 )
//  {
//      m_ppxElements[0] = m_ppxElements[--m_uNumElements];
//      SiftDown( 0 );
//  }
//
//  return pxOldMin;
//}
//
//template< typename T >
//void MinHeap<T>::SetElementModified( T* pxT )
//{
//  // Find it in the heap. Currently we assume that callers will not attempt
//  // to SetModified() on an item that is not currently in the heap.
//  u_int uElementIdx = GetElementIdx( pxT );
//  Asura_Assert( uElementIdx != m_uNumElements, "" );
//
//  SiftUp( uElementIdx );
//  SiftDown( uElementIdx );
//}
//
//template< typename T >
//void MinHeap<T>::SiftDown( u_int uElementIdx )
//{
//  while( uElementIdx + 1 < m_uNumElements )
//  {
//      u_int uSmallest = uElementIdx;
//      
//      // Find the smallest out of the current element and its two children.
//      u_int uLeftChild = (uElementIdx << 1) + 1;
//      if( uLeftChild < m_uNumElements )
//      {
//          if( *m_ppxElements[uLeftChild] < *m_ppxElements[uElementIdx] )
//          {
//              uSmallest = uLeftChild;
//          }
//
//          u_int uRightChild = uLeftChild + 1;
//          if( uRightChild < m_uNumElements && *m_ppxElements[uRightChild] < *m_ppxElements[uSmallest] )
//          {
//              uSmallest = uRightChild;
//          }
//      }
//
//      // We're done if we've reached the end of the heap or if both
//      // children are larger.
//      if( uSmallest == uElementIdx ) { break; }
//
//      // Swap the smaller child with the current element.
//      {
//          T* const pxTmp = m_ppxElements[uElementIdx];
//          m_ppxElements[uElementIdx] = m_ppxElements[uSmallest];
//          m_ppxElements[uSmallest] = pxTmp;
//      }
//
//      // Recurse down that child's branch.
//      uElementIdx = uSmallest;
//  }
//}
//
//template< typename T >
//void MinHeap<T>::SiftUp( u_int uElementIdx )
//{
//  T* const pxElement = m_ppxElements[uElementIdx];
//  while( uElementIdx )
//  {
//      u_int uParent = (uElementIdx - 1) >> 1;
//
//      // We're done if the parent element is already smaller than the
//      // current one.
//      if( *m_ppxElements[uParent] < *pxElement ) { break; }
//
//      // Swap them around.
//      m_ppxElements[uElementIdx] = m_ppxElements[uParent];
//      m_ppxElements[uParent] = pxElement;
//
//      // And recurse upwards.
//      uElementIdx = uParent;
//  }
//}

//
// Tools_SmoothSkin_PS2_Processor::SubElementStripData.
//

Tools_SmoothSkin_PS2_Processor::SubElementStripData::SubElementStripData( void )
    : m_pxProcessedStrips( NULL )
    , m_ppxSortedProcessedStrips( NULL )
    , m_uNumProcessedStrips( 0 )
{
}

Tools_SmoothSkin_PS2_Processor::SubElementStripData::~SubElementStripData( void )
{
    delete[] m_ppxSortedProcessedStrips;

    if( !m_xUnprocessedStrips.empty() )
    {
        for( u_int uStrip = m_xUnprocessedStrips.size(); uStrip--; )
        {
            for( u_int uFace = m_xUnprocessedStrips[uStrip]->m_faces.size(); uFace--; )
            {
                delete m_xUnprocessedStrips[uStrip]->m_faces[uFace];
            }
            delete m_xUnprocessedStrips[uStrip];
        }
    }

    delete[] m_pxProcessedStrips;
}

bool Tools_SmoothSkin_PS2_Processor::SubElementStripData::ProcessStripifierOutput( const Tools_SmoothSkin_PS2_Processor* pxProcessor,
                                                                                   u_int uMaxStripLength,
                                                                                   u_int uTotalInputVerts )
{
    const u_int uNumInputStrips = m_xUnprocessedStrips.size();

    // Shouldn't need to be anywhere near this many output strips but it's
    // easier and quicker to just preallocate the maximum we'll need.
    m_pxProcessedStrips = new ProcessedStrip[uNumInputStrips];
    if( !m_pxProcessedStrips ) { return false; }

    // Estimate how many output strips we'll actually need. Include a small
    // fudge factor to account for the likelihood that we won't manage to
    // optimally combine the input strips.
    u_int uExpectedNumOutputStrips = DivideAndRoundUp( uTotalInputVerts, uMaxStripLength - 1 );
    u_int uExpectedVertsPerOutputStrip = Asura_Maths::Min( DivideAndRoundUp(uTotalInputVerts, uExpectedNumOutputStrips), uMaxStripLength );

    u_int uTotalVertsInInputStrips  = 0;
    u_int uTotalVertsInOutputStrips = 0; // This will include those inserted with ADC bits for strip joining purposes.
    u_int uTotalTrisInOutputStrips  = 0; // This doesn't include degenerates.

    // Temporary array for use for preparing the list of vertex indices from
    // the unprocessed strip.
    u_int* puVtxIndices = new u_int[pxProcessor->GetMaxStripLength()];

    for( NVTriStrip::NvStripInfoVec::const_iterator xIt = m_xUnprocessedStrips.begin(); xIt != m_xUnprocessedStrips.end(); ++xIt )
    {
        const NVTriStrip::NvStripInfo* pxInputStrip = *xIt;
        const u_int uVertsInInputStrip = 2 + pxInputStrip->m_faces.size();

        uTotalVertsInInputStrips += uVertsInInputStrip;

        PrepareVtxIndicesFromUnprocessedStrip( pxInputStrip, puVtxIndices );

        u_int uMaxWeightings = CountMaxWeightingsForVtxIndices( pxProcessor->GetSrcProcessor(), puVtxIndices, uVertsInInputStrip );

        ProcessedStrip* pxStripToUse = NULL;

        // Look for another strip to append to. Use the shortest viable strip
        // with an equal or higher maximum number of weightings per vertex -
        // preferably with the same number of weightings.
        ProcessedStrip* pxOtherStrip = &m_pxProcessedStrips[0];
        for( u_int u = m_uNumProcessedStrips; u--; ++pxOtherStrip )
        {
            if( pxOtherStrip->m_uMaxWeightingsPerVtx < uMaxWeightings ) { continue; }

            u_int uMax = ( m_uNumProcessedStrips < uExpectedNumOutputStrips ? uExpectedVertsPerOutputStrip : uMaxStripLength );
            if( pxOtherStrip->m_uLength + uVertsInInputStrip > uMax ) { continue; }

            // Ok, this one is viable. Accept if it's got less weightings -
            // ie. so it's closer to the source unprocessed strip - or if
            // it's got the same but is shorter than the previous one we've
            // found. Obviously we should also accept it if it's the first
            // one we've found.
            // 
            if( !pxStripToUse
                ||
                pxOtherStrip->m_uMaxWeightingsPerVtx < pxStripToUse->m_uMaxWeightingsPerVtx
                ||
                (pxOtherStrip->m_uMaxWeightingsPerVtx == pxStripToUse->m_uMaxWeightingsPerVtx && pxOtherStrip->m_uLength < pxStripToUse->m_uLength ) )
            {
                pxStripToUse = pxOtherStrip;
            }
        }

        // Need to start a new one?
        if( !pxStripToUse )
        {
            Asura_Assert( m_uNumProcessedStrips < uNumInputStrips, "" );
            pxStripToUse = &m_pxProcessedStrips[m_uNumProcessedStrips++];
            pxStripToUse->Initialise( uMaxStripLength );
        }

        // Append.
        AppendIndicesToProcessedStrip( pxStripToUse, puVtxIndices, uVertsInInputStrip );
        Asura_Assert( pxStripToUse->m_uLength <= uMaxStripLength, "" );
        
        // Update max weightings for strip.
        pxStripToUse->m_uMaxWeightingsPerVtx = Asura_Maths::Max( pxStripToUse->m_uMaxWeightingsPerVtx, uMaxWeightings );
    }

    delete[] puVtxIndices;

    m_ppxSortedProcessedStrips = new ProcessedStrip*[m_uNumProcessedStrips];
    if( !m_ppxSortedProcessedStrips ) { return false; }

    // Sort the strips. We want a normal distribution of lengths since this
    // will help hide stalls at runtime (since the UNPACKing / rendering at
    // either end will take less time.) We'll also calculate a total vertex
    // count while we're busy examining the strips anyway.
    {
        // The easiest way to do this is to use another temporary buffer.
        // Set that up. Count the total vertices at this point too.
        ProcessedStrip** ppxStripsTemp = new ProcessedStrip*[m_uNumProcessedStrips];
        if( !ppxStripsTemp ) { return false; }
        for( u_int u = 0; u < m_uNumProcessedStrips; ++u )
        {
            ppxStripsTemp[u] = &m_pxProcessedStrips[u];
            uTotalVertsInOutputStrips += m_pxProcessedStrips[u].m_uLength;
            uTotalTrisInOutputStrips += m_pxProcessedStrips[u].m_uLength - 2;
        }

        // Now sort the temporary buffer by ascending length.
        std::sort( &ppxStripsTemp[0], &ppxStripsTemp[m_uNumProcessedStrips], ProcessedStrip::Cmp() );

        // Use this ordering to write out our normal distribution. This is
        // very simple: just alternate between the ends of the array as we
        // copy out the ptrs.
        for( u_int u = 0; u < m_uNumProcessedStrips; ++u )
        {
            u_int uOdd    = u & 1;
            u_int uOffset = (u >> 1) + uOdd;
            u_int uOutIdx = ( uOdd ? m_uNumProcessedStrips - uOffset : uOffset );

            m_ppxSortedProcessedStrips[uOutIdx] = ppxStripsTemp[u];
        }

        // Lose the temporary bfr.
        delete[] ppxStripsTemp;
    }

    // Log progress.
    float fInVertAvg = static_cast<float>(uTotalVertsInInputStrips) / uNumInputStrips;
    float fOutVertAvg = static_cast<float>(uTotalVertsInOutputStrips) / m_uNumProcessedStrips;
    float fOutTriAvg = static_cast<float>(uTotalTrisInOutputStrips) / m_uNumProcessedStrips;
    PS2Strip::LogInfo( "    (Element %u: %u input strips (avg %.1f verts / %.1f tris) -> %u output (avg %.1f verts / %.1f tris).)\n",
                       m_uElementIdx,
                       uNumInputStrips,
                       fInVertAvg,
                       fInVertAvg - 2.0f,
                       m_uNumProcessedStrips,
                       fOutVertAvg,
                       fOutTriAvg );

    return true;
}

//
// Tools_SmoothSkin_PS2_Processor.
//

Tools_SmoothSkin_PS2_Processor::Tools_SmoothSkin_PS2_Processor( void )
    : m_pxSrcProcessor( NULL )
    , m_uMaxStripLength( 0 )
    , m_pxStripData( NULL )
    , m_puSortedMaterials( NULL )
    , m_fUScale( 1.0f )
    , m_fVScale( 1.0f )
{
}

Tools_SmoothSkin_PS2_Processor::~Tools_SmoothSkin_PS2_Processor( void )
{
    delete[] m_puSortedMaterials;
    delete[] m_pxStripData;
}

bool Tools_SmoothSkin_PS2_Processor::Initialise( const Tools_SmoothSkin_Processor* pxSrcProcessor )
{
    bool bOK = false;

    m_pxSrcProcessor = pxSrcProcessor;

    PS2Strip::LogInfo( "    Beginning PS2-specific processing...\n" );

    // We'll stripify a single sub-element at a time. This is quicker, since
    // the stripifier currently has some nasty n stuff in it, and we know
    // the strips can't be shared between them anyway.

    PS2Strip_Tri* pxTris = NULL;

    const u_int uNumMaterials = m_pxSrcProcessor->GetNumMaterials();
    const u_int uNumElements  = m_pxSrcProcessor->GetNumElements();

    // Quick pass through the vertices to check for invalid positions.
    for( u_int uMaterial = 0; uMaterial < uNumMaterials; ++uMaterial )
    {
        const Tools_SmoothSkin_Material& xMaterial = m_pxSrcProcessor->GetMaterial( uMaterial );
        for( u_int uSubElement = 0; uSubElement < uNumElements; ++uSubElement )
        {
            const Tools_SmoothSkin_SubElement& xSubElement = xMaterial.GetSubElement( uSubElement );

            const u_short* pusIndices = xSubElement.GetIndices();
			for( u_int u = xSubElement.GetNumTris() * 3; u--; )
            {
                const Asura_Vector_3& xVtxPos = m_pxSrcProcessor->GetVertex(*pusIndices++).GetPos();
                for( u_int uAxis = 0; uAxis < 3; ++uAxis )
                {
                    // Output format is 1:3:12 so our range is
                    // [-7.999755859375, 7.999755859375]. Let's just use
                    // (-8, 8) instead.
                    if( xVtxPos[uAxis] <= -8.0 || 8.0f <= xVtxPos[uAxis] )
                    {
                        PS2Strip::LogError( "ERROR: Skin '%s' has vertices outside supported range of (-8, 8).\n",m_pxSrcProcessor->GetInputSkin()->Name.GetString() );
						
                        return false;
                    }
                }
            }
        }
    }

    // Find max tris in a sub-element. We can then just allocate a single 
    // temporary array for use as input to the stripifier.
    u_int uMaxTris = 0;
    for( u_int uMaterial = 0; uMaterial < uNumMaterials; ++uMaterial )
    {
        const Tools_SmoothSkin_Material& xMaterial = m_pxSrcProcessor->GetMaterial( uMaterial );
        for( u_int uSubElement = 0; uSubElement < uNumElements; ++uSubElement )
        {
            uMaxTris = Asura_Maths::Max( uMaxTris, xMaterial.GetSubElement(uSubElement).GetNumTris() );
        }
    }

    pxTris        = new PS2Strip_Tri[uMaxTris];
    m_pxStripData = new SubElementStripData[uNumMaterials * uNumElements];

    if( !pxTris || !m_pxStripData ) { goto InitCleanUp; }

    // The actual stripfication process.
    {
        NVTriStrip::NvStripifier xStripifier;

        m_uMaxStripLength = PS2Strip::GetMaxStripLength( m_pxSrcProcessor->GetInputSkin() );

        SubElementStripData* pxOut = &m_pxStripData[0];
        for( u_int uMaterial = 0; uMaterial < uNumMaterials; ++uMaterial )
        {
            const Tools_SmoothSkin_Material& xMaterial = m_pxSrcProcessor->GetMaterial( uMaterial );

            // Log progress.
            {
                int iMaterialID = xMaterial.GetMaterialID();
                PS2Strip::LogInfo( "    Generating strips for material %i (%s)...\n", iMaterialID, GetMaterialName(iMaterialID) );
            }

            for( u_int uSubElement = 0; uSubElement < uNumElements; ++pxOut, ++uSubElement )
            {
                const Tools_SmoothSkin_SubElement& xSubElement = xMaterial.GetSubElement( uSubElement );
                pxOut->m_uElementIdx = pxSrcProcessor->GetElementID(uSubElement);

                const u_int uNumTris = xSubElement.GetNumTris();

                // Ignore elements that don't use this material.
                if( !uNumTris ) { continue; }

                u_int uHighestVtxIdx = 0;

                // Fill out tri array.
                const u_short* pusIndices = xSubElement.GetIndices();
                for( u_int uTri = 0; uTri < uNumTris; ++uTri )
                {
                    u_int uV0 = *pusIndices++;
                    u_int uV1 = *pusIndices++;
                    u_int uV2 = *pusIndices++;

                    pxTris[uTri].SetVtxIndices( uV0, uV1, uV2 );

                    uHighestVtxIdx = Asura_Maths::Max( uHighestVtxIdx, uV0 );
                    uHighestVtxIdx = Asura_Maths::Max( uHighestVtxIdx, uV1 );
                    uHighestVtxIdx = Asura_Maths::Max( uHighestVtxIdx, uV2 );
                }

                // Pass off to stripifier.
                u_int uTotalVerts;
                xStripifier.Stripify( pxTris, uNumTris, m_uMaxStripLength, uHighestVtxIdx, true, &pxOut->m_xUnprocessedStrips, &uTotalVerts );

                // Process that raw output.
                if( !pxOut->ProcessStripifierOutput(this, m_uMaxStripLength, uTotalVerts) ) { goto InitCleanUp; }
            }
        }
    }
    
    // Prepare a sorted list of the materials.
    {
        m_puSortedMaterials = new u_int[uNumMaterials];
        if( !m_puSortedMaterials ) { goto InitCleanUp; }

        for( u_int u = 0; u < uNumMaterials; ++u ) { m_puSortedMaterials[u] = u; }

        std::sort( &m_puSortedMaterials[0], &m_puSortedMaterials[uNumMaterials], MaterialIdxCmp(m_pxSrcProcessor) );
    }

    // Report some statistics.
    {
        u_int auStripsWithMaxWeighting[4] = { 0, 0, 0, 0 };

        const SubElementStripData* pxStripData = m_pxStripData;
        for( u_int u = uNumMaterials * uNumElements; u--; ++pxStripData )
        {
            for( u_int uStrip = 0; uStrip < pxStripData->m_uNumProcessedStrips; ++uStrip )
            {
                ++auStripsWithMaxWeighting[pxStripData->m_pxProcessedStrips[uStrip].m_uMaxWeightingsPerVtx - 1];
            }
        }
        
        PS2Strip::LogInfo( "    Num strips with each max weighting count:\n" );
        for( u_int u = 0; u < 4; ++u )
        {
            PS2Strip::LogInfo( "        %u: %u\n", u + 1, auStripsWithMaxWeighting[u] );
        }
    }

    bOK = true;

    PS2Strip::LogInfo( "    Done.\n" );

InitCleanUp:

    delete[] pxTris;

    return bOK;
}

bool Tools_SmoothSkin_PS2_Processor::OutputToChunk( HANDLE pvFile, const char *szSkinName ) const
{
    if( INVALID_HANDLE_VALUE == pvFile ) { return false; }

    PS2Strip::LogInfo( "    Beginning PS2 output...\n" );

    int iTotalBytesWritten = 0;

    // Leave space for chunk header. We'll fill that in later.
    SKIP_DATA( pvFile, iTotalBytesWritten, sizeof(Asura_Chunk_ResourceFile_PS2_StrippedSmoothSkin) );

    // Output resource filename.
    {
        const char* szResourceFilename = "PS2Skin";
        WRITE_DATA( pvFile, iTotalBytesWritten, szResourceFilename, (strlen(szResourceFilename) + 4) & ~3 );
    }

    const int iBytesWrittenBeforeResourceData = iTotalBytesWritten;

    // Output skin name.
    {
        WRITE_DATA( pvFile, iTotalBytesWritten, szSkinName, (strlen(szSkinName) + 4) & ~3 );
    }

    const u_int uNumMaterials = m_pxSrcProcessor->GetNumMaterials();
    const u_int uNumElements  = m_pxSrcProcessor->GetNumElements();

    // Write some info about the skin to the logfile.
    {
        PS2Strip::LogInfo( "    Skin has %u element(s).\n", uNumElements );
        // TODO - Element names?

        PS2Strip::LogInfo( "    Skin has %u material(s):\n", uNumMaterials );
        for( u_int u = 0; u < uNumMaterials; ++u )
        {
            const Tools_SmoothSkin_Material& xMaterial = m_pxSrcProcessor->GetMaterial( m_puSortedMaterials[u] );

            int iMaterialID = xMaterial.GetMaterialID();
            PS2Strip::LogInfo( "        %i - %s\n", iMaterialID, GetMaterialName(iMaterialID) );

            int iTexID = Asura_Material_System::GetTextureID( iMaterialID );
            u_int uMaterialFlags = ( ASURA_MATERIAL_NONE == iMaterialID ? 0 : Asura_Material_System::GetMaterialFlags(iMaterialID) );
            const char* szTexName = ( ASURA_TEXTURE_NONE == iTexID ? "-none-" : Asura_Texture_Management::GetTextureName(iTexID)->GetString() );
            PS2Strip::LogInfo( "        (index within skin: %i; flags: 0x%08X; texture: %i (%s))\n", xMaterial.GetMaterialIDWithinSkin(), uMaterialFlags, iTexID, szTexName );
        }
    }

    const int iBytesWrittenBeforeSkinHdr = iTotalBytesWritten;

    u_int uBytesForHdrs = 0;
    u_int uBytesForHdrsIncPadding;

    // Calculate how much space will be required for all the header info -
    // the skin, each material and all the sub-element headers.
    {
        // Skin hdr.
        uBytesForHdrs += sizeof(Asura_PS2_SmoothSkin_SkinHdr) + uNumMaterials * sizeof(int);

        // Each material hdr and the corresponding sub-elements hdrs.
        for( u_int uMaterialIdx = 0; uMaterialIdx < uNumMaterials; ++uMaterialIdx )
        {
            uBytesForHdrs += sizeof(Asura_PS2_SmoothSkin_MaterialHdr);

            const Tools_SmoothSkin_Material& xMaterial = m_pxSrcProcessor->GetMaterial( m_puSortedMaterials[uMaterialIdx] );
            for( u_int u = 0; u < uNumElements; ++u )
            {
                if( xMaterial.GetSubElement(u).GetNumTris() ) { uBytesForHdrs += sizeof(Asura_PS2_SmoothSkin_SubElementHdr); }
            }
        }

        // That needs to be aligned to a 16-byte boundary since DMA chains
        // will immediately follow.
        uBytesForHdrsIncPadding = (uBytesForHdrs + 15) & ~15;
    }

    // Reserve that space and allocate a temporary buffer for accumulating
    // the data we're going to write into it.
    SKIP_DATA( pvFile, iTotalBytesWritten, uBytesForHdrsIncPadding );
    u_char* pucHdrBfr = new u_char[uBytesForHdrs];
    Asura_PS2_SmoothSkin_MaterialHdr* pxMatHdrOut;

    // Setup skin header in temporary buffer and prepare output buffer for
    // material hdrs.
    {
        Asura_PS2_SmoothSkin_SkinHdr* pxSkinHdr = reinterpret_cast<Asura_PS2_SmoothSkin_SkinHdr*>( pucHdrBfr );

        pxSkinHdr->usNumMaterials                  = static_cast<u_short>( uNumMaterials );
        pxSkinHdr->usNumElements                   = static_cast<u_short>( uNumElements );
        pxSkinHdr->usMaxVertsPerStrip              = static_cast<u_short>( m_uMaxStripLength );
        pxSkinHdr->usQWOffsetToFirstStripDataChain = static_cast<u_short>( uBytesForHdrsIncPadding >> 4 );
        
        int* piMaterialIDs = reinterpret_cast<int*>( pxSkinHdr + 1 );
        for( u_int u = 0; u < uNumMaterials; ++u )
        {
            piMaterialIDs[u] = m_pxSrcProcessor->GetMaterial(m_puSortedMaterials[u]).GetMaterialIDWithinSkin();
        }

        pxMatHdrOut = reinterpret_cast<Asura_PS2_SmoothSkin_MaterialHdr*>( piMaterialIDs + uNumMaterials );
    }

    Tools_PS2_DMAChainWriter xChainWriter;
    if( !xChainWriter.Init(pvFile) ) { return false; }

    // Output each material.
    for( u_int u = 0; u < uNumMaterials; ++u )
    {
        const u_int uMaterialIdx = m_puSortedMaterials[u];

        const Tools_SmoothSkin_Material& xMaterial = m_pxSrcProcessor->GetMaterial( uMaterialIdx );

        // Log progress.
        {
            int iMaterialID = xMaterial.GetMaterialID();
            PS2Strip::LogInfo( "    Writing out data for material %i (%s)...\n", iMaterialID, GetMaterialName(iMaterialID) );
        }

        // Count non-empty sub-elements.
        u_int uNumNonEmptySubElements = 0;
        for( u_int u = 0; u < m_pxSrcProcessor->GetNumElements(); ++u )
        {
            if( xMaterial.GetSubElement(u).GetNumTris() ) { ++uNumNonEmptySubElements; }
        }

        // Should have been eliminated earlier if there aren't any.
        Asura_Assert( uNumNonEmptySubElements, "" );

        // Write out material header.
        pxMatHdrOut->uNumElementsUsingThisMaterial = uNumNonEmptySubElements;
        ++pxMatHdrOut;

        // Output each non-empty sub-element.
        Asura_PS2_SmoothSkin_SubElementHdr* pxSubElementHdrOut = reinterpret_cast<Asura_PS2_SmoothSkin_SubElementHdr*>( pxMatHdrOut );
        for( u_int uElementIdx = 0; uElementIdx < uNumElements; ++uElementIdx )
        {
            const Tools_SmoothSkin_SubElement& xSubElement = xMaterial.GetSubElement( uElementIdx );

            const u_int uNumTris = xSubElement.GetNumTris();
            if( !uNumTris ) { continue; }

            const SubElementStripData& xStripData = m_pxStripData[uMaterialIdx * uNumElements + uElementIdx];

            // Output the DMA chain for this sub-element.
            if( !OutputStripDataChain(xChainWriter, xStripData) ) { return false; }

            const u_int uBytesForThisSE = xChainWriter.GetTotalBytesWrittenForLastChain();
            Asura_Assert( !(uBytesForThisSE & 15), "" );
            iTotalBytesWritten += uBytesForThisSE;
            PS2Strip::LogInfo( "        (Wrote %u bytes for element %u.)\n", uBytesForThisSE, uElementIdx );

            // Write out sub-element header.
            {
                pxSubElementHdrOut->usElementIdx           = static_cast<u_short>( uElementIdx );
                pxSubElementHdrOut->usTriCount             = static_cast<u_short>( xSubElement.GetNumTris() );
                pxSubElementHdrOut->uByteOffsetToNextChain = uBytesForThisSE;

                ++pxSubElementHdrOut;
            }
        }

        // Fix up material hdr ptr.
        pxMatHdrOut = reinterpret_cast<Asura_PS2_SmoothSkin_MaterialHdr*>( pxSubElementHdrOut );
    }

    // Verify that we've got the expected amount of hdr data.
    Asura_Assert( static_cast<u_int>(reinterpret_cast<u_char*>(pxMatHdrOut) - pucHdrBfr) == uBytesForHdrs, "" );

    const int iResourceDataSize = iTotalBytesWritten - iBytesWrittenBeforeResourceData;

    // Go back and fill out chunk hdr.
    {
        Asura_Chunk_ResourceFile_PS2_StrippedSmoothSkin xChunkHdr;
        xChunkHdr.ID       = ASURA_CHUNK_RESOURCEFILE;
        xChunkHdr.Size     = iTotalBytesWritten;
        xChunkHdr.Version  = ASURA_PS2_SMOOTH_SKIN_FILE_FORMAT_VERSION;
        xChunkHdr.Flags    = 0;
        xChunkHdr.Type     = ASURA_RESOURCEFILE_TYPE_PLATFORMSPECIFIC;
        xChunkHdr.SubType  = ASURA_RESOURCEFILE_TYPE_PS2_STRIPPEDSMOOTHSKIN;
        xChunkHdr.FileSize = iResourceDataSize;

        int iIgnore = 0;
        SetFilePointer( pvFile, -iTotalBytesWritten, NULL, FILE_CURRENT );
        WRITE_DATA( pvFile, iIgnore, &xChunkHdr, sizeof(Asura_Chunk_ResourceFile_PS2_StrippedSmoothSkin) );
        SetFilePointer( pvFile, iTotalBytesWritten - sizeof(Asura_Chunk_ResourceFile_PS2_StrippedSmoothSkin), NULL, FILE_CURRENT );
    }

    // Write out the accumulated hdr data.
    {
        int iIgnore = 0;
        SetFilePointer( pvFile, -(iTotalBytesWritten - iBytesWrittenBeforeSkinHdr), NULL, FILE_CURRENT );
        WRITE_DATA( pvFile, iIgnore, pucHdrBfr, uBytesForHdrs );
        SetFilePointer( pvFile, iTotalBytesWritten - (iBytesWrittenBeforeSkinHdr + uBytesForHdrs), NULL, FILE_CURRENT );
    }

    SetEndOfFile( pvFile );

    // Deallocate temporary hdr bfr.
    delete[] pucHdrBfr;

    PS2Strip::LogInfo( "    Done. Wrote total of %i bytes.\n", iResourceDataSize );
    return true;
}

bool Tools_SmoothSkin_PS2_Processor::OutputStripDataChain( Tools_PS2_DMAChainWriter& xChainWriter, const SubElementStripData& xStripData ) const
{
#define CHECK( e ) { if( !(e) ) { bOK = false; goto CleanUp; } }

    bool bOK = true;

    // Temporary buffer for use with preparing packed data. Allow up to 8
    // bytes per vertex.
    u_char*  puc8BitData  = new u_char[m_uMaxStripLength << 3];
    u_short* pus16BitData = reinterpret_cast<u_short*>( puc8BitData );

    CHECK( puc8BitData );

    CHECK( xChainWriter.BeginChain() );

    // Just setup CYCLE in advance; it's the same for all UNPACKs.
    CHECK( xChainWriter.AddVifCode(PS2STRIP_VIF1_SET_STCYCL(1, 5, false)) );

    // Setup MASK. This is only used for the UNPACK at the end of each strip
    // to write the max vertex weightings value.
    {
        const PS2Strip_VIF1_Mask xMask =
        {
            3, 3, 3, 0, // Mask everything except w component.
            3, 3, 3, 3, // Only the first matters since WL (in the CYCLE register) is 1. Just set to mask.
            3, 3, 3, 3, //                          "
            3, 3, 3, 3, //                          "
        };

        CHECK( xChainWriter.AddVifCode(PS2STRIP_VIF1_SET_STMASK(false)) );
        CHECK( xChainWriter.AddDataWord(reinterpret_cast<const u_int&>(xMask)) );
    }

    for( u_int uStrip = 0; uStrip < xStripData.m_uNumProcessedStrips; ++uStrip )
    {
        const ProcessedStrip* pxStrip = xStripData.m_ppxSortedProcessedStrips[uStrip];
        const u_int uNumVerts = pxStrip->m_uLength;
        const u_int uMaxWeightings = pxStrip->m_uMaxWeightingsPerVtx;

        // Normals.
        {
            CHECK( xChainWriter.AddVifCode(PS2STRIP_VIF1_SET_UNPACKR(0, uNumVerts, PS2STRIP_VIF1_CMD_V4_5, false)) );

            PS2Strip_VIF1_V4_5_Data xV4_5;
            for( u_int uVert = 0; uVert < uNumVerts; ++uVert )
            {
                const Asura_Vector_3& xNrm = m_pxSrcProcessor->GetVertex(pxStrip->m_pusIndices[uVert]).GetNormal();
                xV4_5.usX = Tools_PS2Strip_ConvertFloatForV45( xNrm.x );
                xV4_5.usY = Tools_PS2Strip_ConvertFloatForV45( xNrm.y );
                xV4_5.usZ = Tools_PS2Strip_ConvertFloatForV45( xNrm.z );

                xV4_5.usW = pxStrip->m_pbADCs[uVert];

                pus16BitData[uVert] = xV4_5.usValue;
            }

            CHECK( xChainWriter.AddData(pus16BitData, uNumVerts * 2) );
        }
        
        // Bone indices.
        {
            u_int uCMD = PS2STRIP_VIF1_CMD_S_8;
            switch( uMaxWeightings )
            {
                case 2: uCMD = PS2STRIP_VIF1_CMD_V2_8; break;
                case 3: uCMD = PS2STRIP_VIF1_CMD_V3_8; break;
                case 4: uCMD = PS2STRIP_VIF1_CMD_V4_8; break;
            }

            CHECK( xChainWriter.AddVifCode(PS2STRIP_VIF1_SET_UNPACKR(1, uNumVerts, uCMD, false) | PS2STRIP_VIF1_UNPACK_USN_FLAG) );

            u_char* pucOut = puc8BitData;
            for( u_int uVert = 0; uVert < uNumVerts; ++uVert )
            {
                const u_char* pucBoneIndices = m_pxSrcProcessor->GetVertex(pxStrip->m_pusIndices[uVert]).GetBoneIndices();
                for( u_int uWeighting = 0; uWeighting < uMaxWeightings; ++uWeighting )
                {
                    // Offset bone indices by minus 1, since we skip the
                    // irrelevant bone 0 when rendering on PS2. Also, scale
                    // by 4, to give indices we can use directly on PS2
                    // (since we have 4 QWs per bone matrix.) For unused
                    // weightings, use an invalid value, to aid debugging.
                    // The weight will be set to 0 anyway so it won't affect
                    // anything at runtime.
                    const u_int uBoneIdx = pucBoneIndices[uWeighting];
                    *pucOut++ = static_cast<u_char>( uBoneIdx ? (uBoneIdx - 1) << 2u : 0xFFu );
                }
            }

            CHECK( xChainWriter.AddData(puc8BitData, uMaxWeightings * uNumVerts * 1) );
        }

        // Bone weights.
        {
            u_int uCMD = PS2STRIP_VIF1_CMD_S_16;
            switch( uMaxWeightings )
            {
                case 2: uCMD = PS2STRIP_VIF1_CMD_V2_16; break;
                case 3: uCMD = PS2STRIP_VIF1_CMD_V3_16; break;
                case 4: uCMD = PS2STRIP_VIF1_CMD_V4_16; break;
            }

            CHECK( xChainWriter.AddVifCode(PS2STRIP_VIF1_SET_UNPACKR(2, uNumVerts, uCMD, false)) );

            u_short* pusOut = pus16BitData;
            for( u_int uVert = 0; uVert < uNumVerts; ++uVert )
            {
                const float* pfWeights = m_pxSrcProcessor->GetVertex(pxStrip->m_pusIndices[uVert]).GetBoneWeights();

                for( u_int uWeighting = 0; uWeighting < uMaxWeightings; ++uWeighting )
                {
                    *pusOut++ = Tools_PS2Strip_ConvertFloatTo16BitFixed( 15, pfWeights[uWeighting] );
                }
            }

            CHECK( xChainWriter.AddData(pus16BitData, uMaxWeightings * uNumVerts * 2) );
        }

        // Vertex positions.
        {
            CHECK( xChainWriter.AddVifCode(PS2STRIP_VIF1_SET_UNPACKR(3, uNumVerts, PS2STRIP_VIF1_CMD_V3_16, false)) );

            u_short* pusOut = pus16BitData;
            for( u_int uVert = 0; uVert < uNumVerts; ++uVert )
            {
                const Asura_Vector_3& xPos = m_pxSrcProcessor->GetVertex(pxStrip->m_pusIndices[uVert]).GetPos();
                for( u_int u = 0; u < 3; ++u )
                {
                    *pusOut++ = Tools_PS2Strip_ConvertFloatTo16BitFixed( 12, xPos[u] );
                }
            }

            CHECK( xChainWriter.AddData(pus16BitData, 3 * uNumVerts * 2) );
        }

        // STs.
        {
            CHECK( xChainWriter.AddVifCode(PS2STRIP_VIF1_SET_UNPACKR(4, uNumVerts, PS2STRIP_VIF1_CMD_V2_16, false)) );

            u_short* pusOut = pus16BitData;
            for( u_int uVert = 0; uVert < uNumVerts; ++uVert )
            {
                const Tools_SmoothSkin_Vertex& xVtx = m_pxSrcProcessor->GetVertex( pxStrip->m_pusIndices[uVert] );
                *pusOut++ = Tools_PS2Strip_ConvertFloatTo16BitFixed( 12, m_fUScale * xVtx.GetU() );
                *pusOut++ = Tools_PS2Strip_ConvertFloatTo16BitFixed( 12, m_fVScale * xVtx.GetV() );
            }
            
            CHECK( xChainWriter.AddData(pus16BitData, 2 * uNumVerts * 2) );
        }

        // Pass max weightings per vertex for the strip in the unused w
        // component of the first ST QW. Note that we've setup the MASK
        // register above, before starting to write data for the individual
        // strips, since this is the only place it's used.
        CHECK( xChainWriter.AddVifCode(PS2STRIP_VIF1_SET_UNPACKR(4, 1, PS2STRIP_VIF1_CMD_S_32 | PS2STRIP_VIF1_MASK_PROCESS_ENABLE, false)) );
        CHECK( xChainWriter.AddDataWord(uMaxWeightings) );

        // Pass vertex count in ITOPS.
        CHECK( xChainWriter.AddVifCode(PS2STRIP_VIF1_SET_ITOP(uNumVerts, false)) );

        // Finally, kick off the microprogram to render this strip.
        CHECK( xChainWriter.AddVifCode(PS2STRIP_VIF1_SET_MSCNT(false)) );
    }
    // End loop through strips.

    CHECK( xChainWriter.EndChain() );

CleanUp:
    delete[] pus16BitData;

    return bOK;

#undef CHECK
}

void Tools_SmoothSkin_PS2_Processor::SetUVScales( float fUScale, float fVScale )
{
    m_fUScale = fUScale;
    m_fVScale = fVScale;
}

// F U N C T I O N   I M P L E M E N T A T I O N S ///////////////////////////

static void AppendIndicesToProcessedStrip( ProcessedStrip* pxProcessedStrip, const u_int* puVtxIndices, u_int uNumVerts )
{
    u_short* pusIdxOut = &pxProcessedStrip->m_pusIndices[pxProcessedStrip->m_uLength];
    bool*    pbADCOut  = &pxProcessedStrip->m_pbADCs[pxProcessedStrip->m_uLength];

    for( u_int u = 0; u < uNumVerts; ++u )
    {
        *pusIdxOut++ = static_cast<u_short>( *puVtxIndices++ );
        *pbADCOut++  = ( u < 2 );
    }

    pxProcessedStrip->m_uLength += uNumVerts;
}

static u_int CountMaxWeightingsForVtxIndices( const Tools_SmoothSkin_Processor* pxSrcProcessor, const u_int* puVtxIndices, u_int uNumVerts )
{
    u_int uMaxWeightings = 0;

    while( uNumVerts-- )
    {
        uMaxWeightings = Asura_Maths::Max( uMaxWeightings, pxSrcProcessor->GetVertex(*puVtxIndices++).GetNumWeightings() );
    }

    return uMaxWeightings;
}

static void PrepareVtxIndicesFromUnprocessedStrip( const NVTriStrip::NvStripInfo* pxInputStrip, u_int* puIdxOut )
{
    const NVTriStrip::NvFaceInfoVec& xFaces = pxInputStrip->m_faces;
    const u_int uNumTris = xFaces.size();

    // First, append all three vertices of the first tri.
    const NVTriStrip::NvFaceInfo* pxTri0 = xFaces[0];
    for( u_int u = 0; u < 3; ++u )
    {
        *puIdxOut++ = pxTri0->m_av[u];
    }

    // For the remainder of the tris, just append the unique vertex wrt to
    // the previous tri.
    NVTriStrip::NvFaceInfo xLastTri = *pxTri0;
    for( u_int uTri = 1; uTri < uNumTris; ++uTri )
    {
        int iUniqueVert = NVTriStrip::NvStripifier::GetUniqueVertexInB( &xLastTri, xFaces[uTri] );

        if( -1 == iUniqueVert ) { continue; } // NB: This would be an issue if we ever need to care about winding order.

        *puIdxOut++ = iUniqueVert;

        xLastTri.m_v0 = xLastTri.m_v1;
        xLastTri.m_v1 = xLastTri.m_v2;
        xLastTri.m_v2 = iUniqueVert;
    }
}
