//////////////////////////////////////////////////////////////////////////////
//
// Tools_SmoothSkin_Processor_PSP.h
//
// Copyright (c) 2004 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef ___TOOLS_SMOOTHSKIN_PROCESSOR_PSP_H___
#define ___TOOLS_SMOOTHSKIN_PROCESSOR_PSP_H___ 1

// I N C L U D E S ///////////////////////////////////////////////////////////

// Engine independent.
#include "Core/Asura_Maths.h"

// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////////

class Tools_SmoothSkin_Processor;
class Tools_SmoothSkin_SubElement;
class Tools_SmoothSkin_Vertex;

class Tools_SmoothSkin_PSP_BoneSet;
class Tools_SmoothSkin_PSP_Group;
class Tools_SmoothSkin_PSP_Vertex;

// C L A S S E S /////////////////////////////////////////////////////////////

class Tools_SmoothSkin_PSP_Processor
{
    public:

        // C O N S T R U C T O R S   /   D E S T R U C T O R S ///////////////

        Tools_SmoothSkin_PSP_Processor( void );
        ~Tools_SmoothSkin_PSP_Processor();

        // M E T H O D S /////////////////////////////////////////////////////

        // Initialisation.
        bool Initialise( const Tools_SmoothSkin_Processor* pxSrcProcessor );
        
        // Query methods.
        u_int GetNumberOfGroups( void ) const  { return m_uNumberOfGroups; }
        const Tools_SmoothSkin_PSP_Group* GetGroup( const u_int uGroup ) const;
        u_int GetTotalNumberOfVertices( void ) const;       
        u_int GetTotalNumberOfIndices( void ) const;
    
    private:

		void ReorderBonesToMinimiseRenderingStateChanges( class Tools_SmoothSkin_PSP_Group& xCurrentGroup, const class Tools_SmoothSkin_PSP_Group& xComparisonGroup );

        // S T A T E /////////////////////////////////////////////////////////
        
        const Tools_SmoothSkin_Processor* m_pxSrcProcessor;
        u_int                             m_uNumberOfGroups;
        Tools_SmoothSkin_PSP_Group*       m_pxGroups;
        
};
// End class Tools_SmoothSkin_PSP_Processor.

class Tools_SmoothSkin_PSP_Group 
{
    public:
        
        Tools_SmoothSkin_PSP_Group();
        ~Tools_SmoothSkin_PSP_Group();
        
        int                          m_iMaterial;
        u_char                       m_aucBoneIDs[4];

        u_int                        m_uNumberOfIndices;
        u_int*                       m_puIndices;

        u_int                        m_uNumberOfVertices;
        Tools_SmoothSkin_PSP_Vertex* m_pxVertices;
};

// This is the generic version that the data is copied from
class Tools_SmoothSkin_PSP_Vertex
{
    public:

        void Set( const Tools_SmoothSkin_Vertex& pxVertex, const u_char* pucBoneIDs);
        
        // order of members to match layout on PSP determined by GPU
		// (SB - 13/03/07) Actually this now gets selectively exported depending on whether we want colours or normals
        u_char			m_aucWeight[4];
        u_short			m_usU;
        u_short			m_usV;
		u_int			m_uRGBAColour;
        short			m_asNormal[3];
        short			m_asPosition[3];
};

// And these are the two actual vertex types that will be written out
class Tools_SmoothSkin_PSP_Vertex_Coloured
{
	public:

		// order of member determined by PSP GPU
		u_char			m_aucWeight[4];		// 4	4
		u_short			m_usU;				// 2	2
		u_short			m_usV;				// 2	8
		u_int			m_uRGBAColour;		// 4	12
		short			m_asPosition[3];	// 6	18
};

class Tools_SmoothSkin_PSP_Vertex_Normal
{
	public:
		
		// order of member determined by PSP GPU
		u_char			m_aucWeight[4];		// 4	4
		u_short			m_usU;				// 2	2
		u_short			m_usV;				// 2	8
		short			m_asNormal[3];		// 6	14
		short			m_asPosition[3];	// 6	20
};

class Tools_SmoothSkin_PSP_BoneSet
{
    public:
        enum { MAXIMUM_BONES_IN_SET = 4 };

        Tools_SmoothSkin_PSP_BoneSet()
        {
            for (u_int u=0; u<MAXIMUM_BONES_IN_SET; u++)
            {
                m_auBone[u] = 0;
            }


            m_uNumberOfUsedBones = 0;
            m_uNumberOfTrianglesAdded = 0;
            m_uMaterialIdx = 0;
        }
        
		bool Tools_SmoothSkin_PSP_BoneSet::AddBonesOfTriUsingMostImportantWeightingsOnly( u_int uTri, const Tools_SmoothSkin_Processor* pxSrcProcessor, const Tools_SmoothSkin_SubElement& xSubElement );

        bool AddBonesOfTri( u_int uTri, const Tools_SmoothSkin_Processor* pxSrcProcessor, const Tools_SmoothSkin_SubElement& xSubElement );

        u_int GetNumberOfTris( void ) const
        {
            return m_uNumberOfTrianglesAdded;
        }

        void SetMaterialIndex( u_int uMaterialIdx )
        {
            m_uMaterialIdx = uMaterialIdx;
        }

        u_int GetMaterialIndex( void ) const
        {
            return m_uMaterialIdx;
        }

        u_int GetBone( u_int uBoneNumber ) const
        {
            return m_auBone[uBoneNumber];
        }
    private:

        bool AddBone(const u_int uBone)
        {
            // bone zero can be ignored
            if (!uBone) return true;

            // is bone already present? if so we're done
            for (u_int u = 0; u < m_uNumberOfUsedBones; u++)
            {
                if (uBone == m_auBone[u]) return true;
            }

            // It's a new bone - do we have space for it?
            if (MAXIMUM_BONES_IN_SET == m_uNumberOfUsedBones)
            {
                return false;
            }

            // add it in then
            m_auBone[m_uNumberOfUsedBones++] = uBone;

            return true;
        }

        u_int   m_auBone[MAXIMUM_BONES_IN_SET];
        u_int   m_uNumberOfUsedBones;

        u_int   m_uNumberOfTrianglesAdded;
        u_int   m_uMaterialIdx;
};

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

#endif // #ifndef ___TOOLS_SMOOTHSKIN_PROCESSOR_PSP_H___
