/**
*
* \file Tools_Colour.h
*
* Created by Kevin Lea - 24/01/2003 11:37:06
*
* Copyright (C) 2002 Rebellion, All Rights Reserved.
*
* \brief
*
**/
#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif

#ifndef __TOOLS_COLOUR_H__
#define __TOOLS_COLOUR_H__

#include "Core/Asura_Maths.h"

class Tools_Colour_RGB;
class Tools_Colour_HSV;

class Tools_Colour_RGB
{
	public:
		float m_fR;
		float m_fG;
		float m_fB;

		void ToHSV(Tools_Colour_HSV& xHSV) const;

		inline bool operator != ( const Tools_Colour_RGB& x) const
		{
			return (x.m_fR != m_fR || x.m_fG != m_fG || x.m_fB != m_fB);
		}
		
		inline u_int GetAsUint(void) const  
		{
			return (Asura_Maths::FloatToInt(m_fR)<<16) + (Asura_Maths::FloatToInt(m_fG)<<8) + Asura_Maths::FloatToInt(m_fB);
		}
};

class Tools_Colour_HSV
{
	public:
		float m_fH;
		float m_fS;
		float m_fV;

		inline bool operator != ( const Tools_Colour_HSV& x) const
		{
			return (x.m_fH != m_fH || x.m_fS != m_fS || x.m_fV != m_fV);
		}

		void ToRGB(Tools_Colour_RGB& xRGB) const;
};


class Tools_Colour_Pair
{
	public:
		
		//! Both m_xHSV and m_xRGB are public so that the overlay systems can directly access
		//! their data using float pointers. This means they don't have access functions.
		Tools_Colour_HSV m_xHSV;
		Tools_Colour_RGB m_xRGB;

		void Initialise(float fR, float fG, float fB)
		{
			m_xRGB.m_fR = m_xMemoryRGB.m_fR = fR;
			m_xRGB.m_fG = m_xMemoryRGB.m_fG = fG;
			m_xRGB.m_fB = m_xMemoryRGB.m_fB = fB;

			m_xMemoryRGB.ToHSV(m_xHSV);
			m_xMemoryHSV = m_xHSV;
		}
		
		//! Deprecated function, still used in tools in places - better to use the void Update(void)
		//! in conjunction with direct access to m_xRGB.
		void Update(float& fR, float& fG, float& fB)
		{
			m_xRGB.m_fR = fR;
			m_xRGB.m_fG = fG;
			m_xRGB.m_fB = fB;

			Update();

			fR = m_xRGB.m_fR;
			fG = m_xRGB.m_fG;
			fB = m_xRGB.m_fB;
		}

		//! Newer Update fn that should be used in preference to the one above.
		//! If the user has changed either m_xHSV or m_xRGB, the other one will
		//! be changed to match. If both are changed, RGB is taken as the
		//! authoritive version.
		void Update(void)
		{
			if (m_xRGB != m_xMemoryRGB)
			{
				// then someone has changed RGB values - update HSV
				m_xMemoryRGB = m_xRGB;
				m_xMemoryRGB.ToHSV(m_xHSV);
				m_xMemoryHSV = m_xHSV;
			}
			else if (m_xHSV != m_xMemoryHSV)
			{
				// then update the RGB values from the HSV ones
				m_xMemoryHSV = m_xHSV;
				m_xMemoryHSV.ToRGB(m_xRGB);
				m_xMemoryRGB = m_xRGB;
				
			}
		}

	private:

		Tools_Colour_RGB m_xMemoryRGB;
		Tools_Colour_HSV m_xMemoryHSV;
};

#endif//__TOOLS_COLOUR_H__
