#include "Core/Asura.h"
#ifdef ASURA_PC_ALLOW_LOAD_FROM_NETWORK
#include "FileAccess/Asura_FileAccess.h"
#endif


#include "Misc/Tools_EnumerationReader.h"



bool Tools_EnumerationReader::ReadEnumeration(const char* filename,const char* enumeration,bool bCheckNetwork /*=true*/)
{
	ErrorMessage[0] = 0;


	#ifdef ASURA_PC_ALLOW_LOAD_FROM_NETWORK
	char szFilename[MAX_PATH];
	if(bCheckNetwork)
	{
		Asura_FileAccess::PlatformGetNetworkFilename(filename,szFilename,MAX_PATH);
		filename = szFilename;
	}
	#endif

	//load the file
	HANDLE file=CreateFile(filename,GENERIC_READ, FILE_SHARE_READ, 0, OPEN_EXISTING,FILE_FLAG_RANDOM_ACCESS, 0);
	if(file==INVALID_HANDLE_VALUE)
	{
		sprintf(ErrorMessage,"Failed to open %s",filename);
		return false;
	}
	
	int file_size;
	unsigned long bytes_read;
	char* buffer;

	//copy the file contents into a buffer
	file_size= GetFileSize(file,0);
	buffer=new char[file_size+1];
	buffer[file_size] = 0;
	ReadFile(file,buffer,file_size,&bytes_read,0);
	CloseHandle(file);


	//convert any comments to spaces
	RemoveComments(buffer);

	//look for the enumeration name
	char* start = strstr(buffer,enumeration);
	char* end = 0;
	if(!start)
	{
		delete [] buffer;
		sprintf(ErrorMessage,"Failed to find enumeration '%s' in file",enumeration);
		return false;
	}

	//and get the start and end braces
	start = strchr(start,'{');
	if(start) end = strchr(start,'}');

	if(!start || !end)
	{
		strcpy(ErrorMessage,"Error finding start/end of enumeration");
		delete [] buffer;
		return false;
	}

	char* position = start+1;
	*end = 0;

	//start reading in the enumeration
	int nextValue = 0;
	char enumerationString[500];

	do
	{
		SkipWhiteSpaceOnLine(position);
		if(*position && *position!='\n' && *position!='\r' && *position!='#')
		{
			//this line appears to have an enumeration value on it
			int length = 0;
			while(*position && *position!=' ' && *position!=',' && *position!='\t' && *position!='\n' && *position!='\r' && *position != '=')
			{
				enumerationString[length++] = *position++;
			}
			enumerationString[length++] = 0;

			//see if there is an '= value' part to this line
			SkipWhiteSpaceOnLine(position);
			int value = 0;
			if(*position == '=')
			{
				position++;	
				SkipWhiteSpaceOnLine(position);

				bool okay;
				//read in the number. support hex and decimal
				if(position[0] == '0' && position[1] == 'x')
				{
					okay = sscanf(position,"%x",&value)==1 ;
				}
				else
				{
					okay = sscanf(position,"%d",&value)==1;
				}
				if(okay)
				{
					nextValue = value+1;
				}
				else
				{
					sprintf(ErrorMessage,"Error getting value for '%s'",enumerationString);
					delete [] buffer;
					return false;
				}
			
			}
			else
			{
				value = nextValue++;
			}
			//add to the tree
			EnumerationNameTree.InsertNode(enumerationString,value);
		}
		GotoNextLine(position);
	
	}while(*position);

	delete [] buffer;

	return true;
}

bool Tools_EnumerationReader::GetEnumerationValue(const char* valueName,int& outputValue)
{
	return EnumerationNameTree.FindNodeIndexByName(valueName,false,outputValue);
}

const char* Tools_EnumerationReader::GetEnumerationValueName( int iValue )
{
    const Asura_String* pxName = EnumerationNameTree.FindNodeNameByIndex( iValue );
    return( pxName ? pxName->GetString() : NULL );
}

void Tools_EnumerationReader::RemoveComments(char* buffer)
{
	do
	{
		if(*buffer!='/')
		{
			*buffer++;
			continue;
		}

		if(buffer[1]=='/')
		{
			//found a single line comment , replace the remains of this line with spaces
			while(*buffer && *buffer!='\n' && *buffer!='\r') *buffer++ = ' ';
		}
		else if(buffer[1] == '*')
		{
			//found the start of a multiline comment
			*buffer++ = ' ';//blank the start
			*buffer++ = ' ';

			//replace everything with spaces till we  find the end
			while(*buffer)
			{
				if(buffer[0] == '*' && buffer[1] == '/')
				{
					//reached the end of the comment
					*buffer++ = ' ';
					*buffer++ = ' ';
					break;
				}
				else if(*buffer=='\n' || *buffer == '\r')
				{
					buffer++;//leave new line characters alone
				}
				else
				{
					*buffer++ = ' ';					
				}
			}
		}
	}
	while(*buffer);
}

void Tools_EnumerationReader::SkipWhiteSpaceOnLine(char*& buffer)
{
	while(*buffer == ' ' || *buffer == '\t') buffer++;
}

void Tools_EnumerationReader::GotoNextLine(char*& buffer)
{
	while(*buffer && *buffer!='\n' && *buffer!='\r')
	{
		buffer++;
	}
	
	if(*buffer == '\r') buffer++;
	if(*buffer == '\n') buffer++;
}
