//
// Tools_Flag_IniFile.h
//
// Created by Richard Rice - 09/07/2002
// 
// Copyright (C) 2002 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef _TOOLS_FLAG_INIFILE_H_
#define _TOOLS_FLAG_INIFILE_H_

#include "Core/Asura_String.h"
#include "Misc/list_tem.hpp"

class Tools_Flag_List;

//for getting sets of flag names defined in a text file
class Tools_Flag_IniFile
{
public :
	Tools_Flag_IniFile(const char* fileName);
	~Tools_Flag_IniFile();

	//Gets the list of flags that appear under the specified section name.
	//Will return 0 if the section wasn't found (file not loaded sucessfully/section doesn't exist)
	//or if no flags are defined for that section
	Tools_Flag_List* GetFlagList(const char* sectionName);

private :
	List<class Tools_Flag_List*> AllFlagLists;
	struct Tools_IniFile* IniFile;

};


struct NameAndValue
{
	int Value;
	Asura_String Name;
};


class Tools_Value_List
{
public :
	Tools_Value_List(const char* sectionName);
	Tools_Value_List(const char* sectionName,struct Tools_IniFile* IniFile);

	bool SelectValue(const char* title,int* value);
	const char* GetName(int value,const char* unknownString = "<Unknown>");
	bool GetValueFromName(const char* szName, int& iOutput) const;

	Asura_String SectionName;
	List<NameAndValue> ValueList;
};

class Tools_Flag_List : public Tools_Value_List
{
public :
	Tools_Flag_List(const char* sectionName,struct Tools_IniFile* IniFile)
	:Tools_Value_List(sectionName,IniFile){}
	//constructor from a null terminated array of strings.
	//values are assigned 1,2,4,8,16,...
	Tools_Flag_List(const char* sectionName,const char** strings);
	
	const char* GetFlagName(int flagValue);
};

class Tools_Enumeration_List : public Tools_Value_List
{
public :
	Tools_Enumeration_List(const char* sectionName,struct Tools_IniFile* IniFile)
	:Tools_Value_List(sectionName,IniFile){}

	//constructor from a null terminated array of strings.
	//values are assigned 0,1,2,3,4,...
	Tools_Enumeration_List(const char* sectionName,const char** strings);

};

#endif //_TOOLS_FLAG_INIFILE_H_
