#include "Core/Asura.h"
#include "Misc/Tools_LOS.h"

#include "Rendering/Asura_Module.h"
#include "Mesh/Asura_Mesh.h"
#include "Interface/Asura_Cursor.h"
#include "Misc/Tools_Render_Options.h"
#include "Camera/Asura_Camera.h"
#include "Core/Asura_Los.h"

#include "Core/Asura_Material_System.h"
//#include "Asura_TextureManagement.h"

#include "Misc/Tools_Render_Options.h"
#include "Misc/Tools_Adjacency.h"
#include "Streaming/Tools_Streaming_System.h"




extern bool DrawUnlinkedModules;
bool Tools_LOS::s_bIgnoreTransparent = false;
bool Tools_LOS::s_bIgnoreBFC = false;
bool Tools_LOS::s_bIgnoreModuleLinking = false;
u_int Tools_LOS::s_uPolygonFlagsToIgnore = 0;
bool (*Tools_LOS::s_pfnModuleFilterCallback)(int iModuleIndex) = 0;
//This is basically the same as Asura_Module_System::FindPolyInLOS , except that it ignores any polygons
//that aren't being drawn

bool Tools_LOS::FindPolyInLOS(const Asura_Vector_3& alpha, const Asura_Vector_3& beta,int& polyIndex,int& moduleIndex,float& distance,bool bIncludeAllPolys,Asura_Hash_ID* puOutputZoneHash)
{
	if(puOutputZoneHash)
	{
		float fDistance = ASURA_LOS_MAX_DISTANCE;
		bool bFoundAny = false;

		for(Tools_Streaming_Iterator xIt;!xIt.Done();xIt.Next())
		{
			#ifdef ASURA_USE_STREAMING
			if(!Tools_Streaming_System::IsSetToMainZone() && Tools_Render_Options::IsHidingOtherZones() && !bIncludeAllPolys)
			{
				//Don't do LOS against other zones if they are hidden. RAR 11/04/2006
				continue;
			}
			#endif
			
			bool bFound = FindPolyInLOSInCurrentZone(alpha,beta,polyIndex,moduleIndex,fDistance,fDistance,bIncludeAllPolys);
			if(bFound)
			{
				*puOutputZoneHash = xIt.GetHash();
				distance = fDistance;
				bFoundAny = true;
			}
		}
		return bFoundAny;
	}
	else
	{
		return FindPolyInLOSInCurrentZone(alpha,beta,polyIndex,moduleIndex,distance,ASURA_LOS_MAX_DISTANCE,bIncludeAllPolys);
	}
}

bool Tools_LOS::FindPolyInLOSInCurrentZone(const Asura_Vector_3& alpha, const Asura_Vector_3& beta,int& polyIndex,int& moduleIndex,float& fOutputDistance,float fDistanceToCheck,bool bIncludeAllPolys)
{
	//return Asura_Module_System::FindPolyInLOS(alpha,beta,polyIndex,moduleIndex);
	float lowest = fDistanceToCheck;	
	int visibleModule=-1;
	int NumberOfModules = Asura_Module_System::GetNumberOfModules();


	for (int mn=0; mn<NumberOfModules; mn++)
	{
		//ignore modules that aren't visible
		if (!bIncludeAllPolys && !s_bIgnoreModuleLinking)
		{
			if(!DrawUnlinkedModules)
			{
				if(!Asura_Module_System::IsModuleVisible(mn)) continue;
			}
			if(!Tools_Render_Options::IsModuleWithinViewDistance(mn)) continue;
		}
		if(s_pfnModuleFilterCallback && !s_pfnModuleFilterCallback(mn)) continue;
		
		//skip hidden modules
		if(!bIncludeAllPolys &&	Tools_Render_Options::IsHidden(mn))
		{
			continue;
		}


		Asura_Mesh *mesh = Asura_Module_System::GetMeshForModule(mn);

		
		
		{
			const Asura_Vector_3& centre = Asura_Module_System::GetCentreOfModule( mn );
			float radius = Asura_Module_System::GetRadiusOfModule(mn);

			if (!Asura_Maths::RayIntersectsSphere(alpha,beta,centre,radius)) continue;
			
			float mindist = (alpha-centre).Magnitude() - radius;

			if (lowest<mindist) continue;

		}
		Asura_Vector_3 position;
		Asura_Module_System::GetPositionOfModule(mn,position);
			
		for (int i=0; i<mesh->NumberOfPolygons; i++)
		{
			//check for the hidden flag
			if(!bIncludeAllPolys &&
				Tools_Render_Options::IsHidden(mn,i))
				continue;

			if(s_bIgnoreTransparent && mesh->TextureDataValid)
			{
				if(Asura_Material_System::IsMaterialTransparent(mesh->TextureDataPtr[i].m_iMaterialID))
				//if(Asura_Texture_Management::IsTextureTransparent(mesh->TextureDataPtr[i].TextureID))
				{
					continue;
				}
			}
			
			int iFlags = mesh->PolygonFlagDataPtr ? mesh->PolygonFlagDataPtr[i].Flags : 0;
			bool bDoubleSided = (iFlags & AsuraPolygonFlag_DoubleSided) || Tools_Render_Options::NoBackFaceCull || s_bIgnoreBFC;

			if(iFlags & s_uPolygonFlagsToIgnore) continue;
			
			{
				bool facingToward;
				Asura_Vector_3 v0 = mesh->VertexDataPtr[mesh->PolygonDataPtr[i].VertexID[0]].Position+position;
				Asura_Vector_3 v1 = mesh->VertexDataPtr[mesh->PolygonDataPtr[i].VertexID[1]].Position+position;
				Asura_Vector_3 v2 = mesh->VertexDataPtr[mesh->PolygonDataPtr[i].VertexID[2]].Position+position;

				float d = Asura_Maths::RayIntersectsTri(alpha,beta,v0,v1,v2,facingToward);

				if (d>=0.0f && d<lowest)
				{
				
					if (facingToward || bDoubleSided) // need to check for translucent polys!
					{
						lowest=d;

						visibleModule = mn;	
						polyIndex = i;
						moduleIndex = mn;
						fOutputDistance = lowest;
					}

					
				}
			}
			if (mesh->PolygonDataPtr[i].IsQuad())
			{
				bool facingToward;
				Asura_Vector_3 v0 = mesh->VertexDataPtr[mesh->PolygonDataPtr[i].VertexID[2]].Position+position;
				Asura_Vector_3 v1 = mesh->VertexDataPtr[mesh->PolygonDataPtr[i].VertexID[3]].Position+position;
				Asura_Vector_3 v2 = mesh->VertexDataPtr[mesh->PolygonDataPtr[i].VertexID[0]].Position+position;

				float d = Asura_Maths::RayIntersectsTri(alpha,beta,v0,v1,v2,facingToward);

				if (d>=0.0f && d<lowest)
				{
				
					if (facingToward || bDoubleSided) // need to check for translucent polys!
					{
						lowest=d;

						visibleModule = mn;	
						polyIndex = i;
						moduleIndex = mn;
						fOutputDistance = lowest;
					}

					
				}
			}
		
		}

	}

	return (visibleModule!=-1);

}

bool Tools_LOS::FindPolyInLOS(const Asura_Vector_3& alpha, const Asura_Vector_3& beta,int& polyIndex,int& moduleIndex,Asura_Hash_ID* puOutputZoneHash)
{
	float junk;
	return FindPolyInLOS(alpha,beta,polyIndex,moduleIndex,junk,false,puOutputZoneHash);
}


void Tools_LOS::GetCursorLOSOriginAndDir( Asura_Vector_3* pxOrigin, Asura_Vector_3* pxDir )
{
    *pxOrigin = *Asura_Camera::GetPosition();

    Asura_Vector_3 xDir;
	Asura_Cursor::GetLOSDirection( xDir );
    xDir = xDir * (*Asura_Camera::GetOrientation());
    *pxDir = xDir;
}


bool Tools_LOS::FindPolyAndPositionUnderCursor(int& polyIndex,int& moduleIndex,Asura_Vector_3& position,Asura_Hash_ID* puOutputZoneHash /*= 0*/)
{
	Asura_Vector_3 xOrigin, xDir;
    GetCursorLOSOriginAndDir( &xOrigin, &xDir );

	//find the polygon in that direction
	float distance;
	if(!FindPolyInLOS(xOrigin,xDir,polyIndex,moduleIndex,distance,false,puOutputZoneHash)) return false;

	//work out the position
	position = xOrigin + xDir*distance;
	
	return true; 

}

bool Tools_LOS::FindModuleUnderCursor(int& moduleIndex,Asura_Hash_ID* puOutputZoneHash)
{
	Asura_Vector_3 xOrigin, xDir;
    GetCursorLOSOriginAndDir( &xOrigin, &xDir );

	int junkIndex;
	return FindPolyInLOS(xOrigin,xDir,junkIndex,moduleIndex,puOutputZoneHash);
}


bool Tools_LOS::FindVertInLOS(const Asura_Vector_3& alpha, const Asura_Vector_3& beta,int& vertIndex,int& moduleIndex,bool (*pfnVertexFilter)(int iModuleIndex,int iVertexIndex),bool bIncludeLooseVerts)
{
	//first find the polygon in that direction
	float distance = Asura_Maths::LargeFloat;
	int polyIndex;
	bool bFoundPoly = FindPolyInLOS(alpha,beta,polyIndex,moduleIndex,distance);

	if(bIncludeLooseVerts)
	{
		//See if there were any closer loose vertices
		bool bFoundVert = FindLooseVertexInLOS(alpha,beta,distance,vertIndex,moduleIndex);

		//If a vertex was found , go with it
		if(bFoundVert) return true;
	}

	if(!bFoundPoly) return false;

	//get the point of intersection
	Asura_Vector_3 intersection = alpha+beta*distance;

	//need to make relative to the module
	Asura_Vector_3 position;
	Asura_Module_System::GetPositionOfModule(moduleIndex,position);
	intersection-=position;

	Asura_Mesh* mesh = Asura_Module_System::GetMeshForModule(moduleIndex);

	#if 0
	//check each vertex in the polygon , and find the one closest to the intersection point
	Asura_Mesh_Polygon* poly = &mesh->PolygonDataPtr[polyIndex];
	float lowest = ASURA_LOS_MAX_DISTANCE;

	for(int i=0;i<poly->NumberOfVertices();i++)
	{
		float d = (mesh->VertexDataPtr[poly->VertexID[i]].Position - intersection).Magnitude();
		if(d<lowest)
		{
			lowest = d;
			vertIndex = poly->VertexID[i];
		}
	}
	#else
	//check each vertex in the mesh , and find the one closest to the intersection point
	float lowest = ASURA_LOS_MAX_DISTANCE;

	for(int i=0;i<mesh->NumberOfVertices;i++)
	{
		if(!pfnVertexFilter || pfnVertexFilter(moduleIndex,i))
		{
			float d = (mesh->VertexDataPtr[i].Position - intersection).Magnitude();
			if(d<lowest)
			{
				lowest = d;
				vertIndex = i;
			}
		}
	}
	#endif

	return lowest != ASURA_LOS_MAX_DISTANCE;
}

bool Tools_LOS::FindVertUnderCursor(int& vertIndex,int& moduleIndex,bool (*pfnVertexFilter)(int iModuleIndex,int iVertexIndex),bool bIncludeLooseVerts)
{
	Asura_Vector_3 xOrigin, xDir;
    GetCursorLOSOriginAndDir( &xOrigin, &xDir );

	return FindVertInLOS(xOrigin,xDir,vertIndex,moduleIndex,pfnVertexFilter,bIncludeLooseVerts);
}

bool Tools_LOS::FindEdgeUnderCursor(struct Tools_LOS_EdgeOutput& xOutput, bool bAllowDiagonalInQuad)
{
	//first find the polygon and position under the cursor
	Asura_Vector_3 xPosition;
	int iModule;
	int iPoly;
	if(!FindPolyAndPositionUnderCursor(iPoly,iModule,xPosition)) return false;

	//Find the polygon edge closest to this
	float fNearDist = Asura_Maths::LargeFloat;
	int iNearEdge = 0;
	bool bNearEdgeIsDiagonal = false;

	Asura_Mesh* pxMesh = Asura_Module_System::GetMeshForModule(iModule);
	Asura_Vector_3 xModulePos;
	Asura_Module_System::GetPositionOfModule(iModule,xModulePos);
	xPosition -= xModulePos;

	if(!pxMesh->m_bFaceNormalsValid) pxMesh->CreateFaceNormals();
	const Asura_Vector_3& xFaceNormal = pxMesh->NormalsDataPtr[iPoly].FaceNormal; 

	const Asura_Mesh_Polygon* pxPoly = &pxMesh->PolygonDataPtr[iPoly];
	int iNumVerts = pxPoly->NumberOfVertices();
	for(int i=0;i<iNumVerts;i++)
	{
		const Asura_Vector_3& xVert1 = pxMesh->VertexDataPtr[pxPoly->VertexID[i]].Position;
		const Asura_Vector_3& xVert2 = pxMesh->VertexDataPtr[pxPoly->VertexID[(i+1)%iNumVerts]].Position;

		//get a normal in the direction of the edge
		Asura_Vector_3 xEdgeNormal = xVert1-xVert2;
		xEdgeNormal.Normalise();

		//and a perpendicular lying on the polygon
		Asura_Vector_3 xPerp = xEdgeNormal^xFaceNormal;

		float fDist = Asura_Maths::Abs((xPosition - xVert1)*xPerp);

		if(fDist<fNearDist)
		{
			fNearDist = fDist;
			iNearEdge = i;
		}
	}
	if(iNumVerts==4 && bAllowDiagonalInQuad)
	{
		//Check the edge where the quad will be split into triangles
		const Asura_Vector_3& xVert1 = pxMesh->VertexDataPtr[pxPoly->VertexID[0]].Position;
		const Asura_Vector_3& xVert2 = pxMesh->VertexDataPtr[pxPoly->VertexID[2]].Position;

		//get a normal in the direction of the edge
		Asura_Vector_3 xEdgeNormal = xVert1-xVert2;
		xEdgeNormal.Normalise();

		//and a perpendicular lying on the polygon
		Asura_Vector_3 xPerp = xEdgeNormal^xFaceNormal;

		float fDist = Asura_Maths::Abs((xPosition - xVert1)*xPerp);

		if(fDist<fNearDist)
		{
			fNearDist = fDist;
			iNearEdge = 0;
			bNearEdgeIsDiagonal = true;
		}

	}

	//fill in the details so far
	if(bNearEdgeIsDiagonal)
	{
		xOutput.m_axEdge[0].m_iModuleIndex = iModule;
		xOutput.m_axEdge[0].m_iPolygonIndex = iPoly;
		xOutput.m_axEdge[0].m_aiVertexIndex[0] = pxPoly->VertexID[0];
		xOutput.m_axEdge[0].m_aiVertexIndex[1] = pxPoly->VertexID[2];

		xOutput.m_axEdge[1].m_iModuleIndex = -1;
		xOutput.m_axEdge[1].m_iPolygonIndex = -1;
		xOutput.m_axEdge[1].m_aiVertexIndex[0] = -1;
		xOutput.m_axEdge[1].m_aiVertexIndex[1] = -1;

		xOutput.m_bDiagonalInQuad = true;
	}
	else
	{
		xOutput.m_axEdge[0].m_iModuleIndex = iModule;
		xOutput.m_axEdge[0].m_iPolygonIndex = iPoly;
		xOutput.m_axEdge[0].m_aiVertexIndex[0] = pxPoly->VertexID[iNearEdge];
		xOutput.m_axEdge[0].m_aiVertexIndex[1] = pxPoly->VertexID[(iNearEdge+1)%iNumVerts];

		xOutput.m_axEdge[1].m_iModuleIndex = -1;
		xOutput.m_axEdge[1].m_iPolygonIndex = -1;
		xOutput.m_axEdge[1].m_aiVertexIndex[0] = -1;
		xOutput.m_axEdge[1].m_aiVertexIndex[1] = -1;

		xOutput.m_bDiagonalInQuad = false;


		//get the adjacent polygon
		const Tools_Polygon_Adjacency* pxAdjList = Tools_Adjacency::GetPolygonAdjacencies(iModule,iPoly);
		for(const Tools_Adjacent_Polygon* pxAdj = pxAdjList->GetFirstAdjacency(); pxAdj; pxAdj = pxAdj->GetNext())
		{
			if(pxAdj->IsBetweenZones()) continue;
			
			if(pxAdj->GetNumSharedVertices() != 2) continue;

			if(pxAdj->IsBetweenModules())
			{
				//search for matching verts in the second polygon
				//in a different module ,so compare the vert positions
				const int iAdjModuleIndex = pxAdj->GetModuleIndex();
				const int iAdjPolygonIndex = pxAdj->GetPolygonIndex();
				const Asura_Mesh* pxMesh2 = Asura_Module_System::GetMeshForModule(pxAdj->GetModuleIndex());
				const Asura_Mesh_Polygon* pxPoly2 = &pxMesh2->PolygonDataPtr[iAdjPolygonIndex];

							
				int aiVerts[2] = {-1,-1};
				
				for(int j=0;j<2;j++)
				{
					Asura_Vector_3 xVertPos = pxMesh->VertexDataPtr[xOutput.m_axEdge[0].m_aiVertexIndex[j]].Position + 
												Asura_Module_System::GetPositionOfModule(iModule);

					int iNumVerts = pxPoly2->NumberOfVertices();
					for(int k=0; k<iNumVerts; k++)
					{
						int iVert = pxPoly2->VertexID[k];
						Asura_Vector_3 xVertPos2 = pxMesh2->VertexDataPtr[iVert].Position +
													Asura_Module_System::GetPositionOfModule(iAdjModuleIndex);

						if(Tools_Adjacency::IsDistanceWithinTolerance(xVertPos2 - xVertPos))
						{
							aiVerts[j] = iVert;
							break;
						}
					}
				}
				if(aiVerts[0] != -1 && aiVerts[1] != -1 && aiVerts[0] != aiVerts[1])
				{
					//found a suitable match
					xOutput.m_axEdge[1].m_iModuleIndex = iAdjModuleIndex;
					xOutput.m_axEdge[1].m_iPolygonIndex = iAdjPolygonIndex;
					xOutput.m_axEdge[1].m_aiVertexIndex[0] = aiVerts[0];
					xOutput.m_axEdge[1].m_aiVertexIndex[1] = aiVerts[1];

					break;				
				}

			}
			else
			{
				//search for matching verts in the second polygon
				//in same module , so can just check for the vertex index being the same
				const int iAdjPolygonIndex = pxAdj->GetPolygonIndex();
				const Asura_Mesh_Polygon* pxPoly2 = &pxMesh->PolygonDataPtr[iAdjPolygonIndex];

							
				int aiVerts[2] = {-1,-1};
				
				for(int j=0;j<2;j++)
				{
					int iNumVerts = pxPoly2->NumberOfVertices();
					for(int k=0; k<iNumVerts; k++)
					{
						int iVert = pxPoly2->VertexID[k];

						if(iVert == xOutput.m_axEdge[0].m_aiVertexIndex[j])
						{
							aiVerts[j] = iVert;
							break;
						}
					}
				}
				if(aiVerts[0] != -1 && aiVerts[1] != -1 && aiVerts[0] != aiVerts[1])
				{
					//found a suitable match
					xOutput.m_axEdge[1].m_iModuleIndex = iModule;
					xOutput.m_axEdge[1].m_iPolygonIndex = iAdjPolygonIndex;
					xOutput.m_axEdge[1].m_aiVertexIndex[0] = aiVerts[0];
					xOutput.m_axEdge[1].m_aiVertexIndex[1] = aiVerts[1];

					break;				
				}
			}
		}
	}

	return true;
}

bool Tools_LOS::FindLooseVertexInLOS(const class Asura_Vector_3& xAlpha, const class Asura_Vector_3& xBeta,float fDistanceToCheck,int& iVertexIndex,int& iModuleIndex)
{
	float fLowest = fDistanceToCheck;	
	int NumberOfModules = Asura_Module_System::GetNumberOfModules();
	bool bFoundAny = false;

	Asura_Vector_3 xCameraPos = *Asura_Camera::GetPosition();


	for (int mn=0; mn<NumberOfModules; mn++)
	{
		//ignore modules that aren't visible
		if (!s_bIgnoreModuleLinking)
		{
			if(!DrawUnlinkedModules)
			{
				if(!Asura_Module_System::IsModuleVisible(mn)) continue;
			}
			if(!Tools_Render_Options::IsModuleWithinViewDistance(mn)) continue;
		}

		if(s_pfnModuleFilterCallback && !s_pfnModuleFilterCallback(mn)) continue;

		//ignore if there are no loose verts
		int iNumLooseVerts = Tools_Adjacency::GetNumLooseVertices(mn);
		if(!iNumLooseVerts) continue;


		Asura_Mesh *pxMesh = Asura_Module_System::GetMeshForModule(mn);
		
		//See if the ray intersects the module
		{
			const Asura_Vector_3& xCentre = Asura_Module_System::GetCentreOfModule( mn );
			float fRadius = Asura_Module_System::GetRadiusOfModule(mn);

			if (!Asura_Maths::RayIntersectsSphere(xAlpha,xBeta,xCentre,fRadius)) continue;
			
			float fMinDist = (xAlpha-xCentre).Magnitude() - fRadius;

			if (fLowest<fMinDist) continue;

		}

		const Asura_Vector_3 xModulePos = Asura_Module_System::GetPositionOfModule(mn);

		//do a sphere test against each loose vertex
		for(int i=0;i<iNumLooseVerts;i++)
		{
			int iVertIndex = Tools_Adjacency::GetLooseVertex(mn,i);

			Asura_Vector_3 xCentre = pxMesh->VertexDataPtr[iVertIndex].Position + xModulePos;

			//Determine radius to use for vertex. (Based on radius chosen in Tools_Render::RenderVertexOutline)
			float fRadius = 0.07f;
			float fDist = (xCentre - xCameraPos).Magnitude();
			if(fDist < 10.0f)
			{
				if(fDist < 0.1f) fDist = 0.1f;
				fRadius *= fDist/10.0f;	
			}

			if (!Asura_Maths::RayIntersectsSphere(xAlpha,xBeta,xCentre,fRadius)) continue;
			
			float fMinDist = (xAlpha-xCentre).Magnitude() - fRadius;

			if(fMinDist < fLowest)
			{
				fLowest = fMinDist;
				iModuleIndex = mn;
				iVertexIndex = iVertIndex;
				bFoundAny = true;
			}
		}		

	}

	return (bFoundAny);
}



bool Tools_LOS::FindCollModuleAndPositionUnderCursor(int& iModuleIndex,Asura_Vector_3& xPosition,Asura_Vector_3* pxOutputNormal,Asura_Hash_ID* puOutputZoneHash /*= 0*/)
{
	Asura_Vector_3 xOrigin, xDir;
    GetCursorLOSOriginAndDir( &xOrigin, &xDir );

	//find the polygon in that direction
	float fDistance;
	if(!FindCollModuleInLOS(xOrigin,xDir,iModuleIndex,fDistance,pxOutputNormal,false,puOutputZoneHash)) return false;

	//work out the position
	xPosition = xOrigin + xDir*fDistance;
	
	return true; 
}

bool Tools_LOS::FindCollModuleInLOS(const Asura_Vector_3& xAlpha, const Asura_Vector_3& xBeta,int& iModuleIndex,float& fOutputDistance,Asura_Vector_3* pxOutputNormal,bool bIncludeAllPolys,Asura_Hash_ID* puOutputZoneHash)
{
	if(puOutputZoneHash)
	{
		float fDistance = ASURA_LOS_MAX_DISTANCE;
		bool bFoundAny = false;

		for(Tools_Streaming_Iterator xIt;!xIt.Done();xIt.Next())
		{
			#ifdef ASURA_USE_STREAMING
			if(!Tools_Streaming_System::IsSetToMainZone() && Tools_Render_Options::IsHidingOtherZones() && !bIncludeAllPolys)
			{
				//Don't do LOS against other zones if they are hidden. RAR 11/04/2006
				continue;
			}
			#endif
			
			bool bFound = FindCollModuleInLOSInCurrentZone(xAlpha,xBeta,iModuleIndex,fDistance,pxOutputNormal,fDistance,bIncludeAllPolys);
			if(bFound)
			{
				*puOutputZoneHash = xIt.GetHash();
				fOutputDistance = fDistance;
				bFoundAny = true;
			}
		}
		return bFoundAny;
	}
	else
	{
		return FindCollModuleInLOSInCurrentZone(xAlpha,xBeta,iModuleIndex,fOutputDistance,pxOutputNormal,ASURA_LOS_MAX_DISTANCE,bIncludeAllPolys);
	}
}


bool Tools_LOS::FindCollModuleInLOSInCurrentZone(const Asura_Vector_3& xAlpha, const Asura_Vector_3& xBeta,int& iModuleIndex,float& fOutputDistance,Asura_Vector_3* pxOutputNormal,float fDistanceToCheck,bool bIncludeAllPolys)
{
	int iNumberOfModules = Asura_Module_System::GetNumberOfModules();

	Asura_LOS_InputData xLOSInput(xAlpha,xBeta,ASURA_GUID_UNREGISTERED,fDistanceToCheck,false,s_uPolygonFlagsToIgnore);
	Asura_LOS_OutputData xLOSOutput;
	xLOSOutput.Reset(fDistanceToCheck);

	int iBestModule = -1;

	for (int iModule=0; iModule<iNumberOfModules; iModule++)
	{
		//ignore modules that aren't visible
		if (!bIncludeAllPolys && !s_bIgnoreModuleLinking)
		{
			if(!DrawUnlinkedModules)
			{
				if(!Asura_Module_System::IsModuleVisible(iModule)) continue;
			}
			if(!Tools_Render_Options::IsModuleWithinViewDistance(iModule)) continue;
		}
		
		if(s_pfnModuleFilterCallback && !s_pfnModuleFilterCallback(iModule)) continue;


		//LOS test against this module's collision mesh
		const Asura_CollisionMesh *pxCMesh = Asura_Module_System::GetCollisionMeshForModule(iModule);
		Asura_Assert(pxCMesh,"");

		Asura_Vector_3 xModulePos = Asura_Module_System::GetPositionOfModule(iModule);

		if(pxCMesh->LOSTestPolygons(xModulePos,Asura_Maths::IdentityMatrix3x3,&xLOSInput,&xLOSOutput))
		{
			iBestModule = iModule;
		}
	}

	if(iBestModule != -1)
	{
		iModuleIndex = iBestModule;
		fOutputDistance = xLOSOutput.m_fDistanceToIntersection;
		if(pxOutputNormal) *pxOutputNormal = xLOSOutput.m_xNormal;
	}

	return iBestModule != -1;
}
