//
// Tools_Platform.cpp
//
// Created by Richard Rice - 11/08/2006
// 
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"
#include "Misc/Tools_Platform.h"

#include "Debugging/Asura_Assert.h"

static const char* g_aszPlatformNames[] =
{
	"PC",		//TOOLS_PLATFORM_PC
	"PS2",		//TOOLS_PLATFORM_PS2
	"XBox",		//TOOLS_PLATFORM_XBOX
	"GC",		//TOOLS_PLATFORM_GC
	"PSP",		//TOOLS_PLATFORM_PSP
	"PS3",		//TOOLS_PLATFORM_PS3
	"Xbox360",	//TOOLS_PLATFORM_XBOX360
	"Wii",		//TOOLS_PLATFORM_WII
};

//string lengths of the platform names.
static const int g_aiPlatformNameLength[] =
{
	2,//"PC"
	3,//"PS2"
	4,//"XBox"
	2,//"GC"
	3,//"PSP
	3,//"PS3"
	7,//"Xbox360"
	3,//"Wii"
};

static const char* g_aszPlatformExtensions[] =
{
	".pc",		//TOOLS_PLATFORM_PC
	".ps2",		//TOOLS_PLATFORM_PS2
	".xbx",		//TOOLS_PLATFORM_XBOX
	".gc",		//TOOLS_PLATFORM_GC
	".psp",		//TOOLS_PLATFORM_PSP
	".ps3",		//TOOLS_PLATFORM_PS3
	".xbox360",	//TOOLS_PLATFORM_XBOX360
	".wii",		//TOOLS_PLATFORM_WII
};



const char* Tools_Platform::GetPlatformName(int iPlatform)
{
	if(iPlatform >=0 && iPlatform < TOOLS_PLATFORM_NUM_PLATFORMS)
	{
		return g_aszPlatformNames[iPlatform];
	}
	else
	{
		return "*Unknown*";
	}
}

const char* Tools_Platform::GetPlatformExtension(int iPlatform)
{
	if(iPlatform >=0 && iPlatform < TOOLS_PLATFORM_NUM_PLATFORMS)
	{
		return g_aszPlatformExtensions[iPlatform];
	}

    Asura_Assert( false, "Unrecognised platform" );
    return NULL;
}

int Tools_Platform::GetPlatformFromPath(const char* szPath)
{
	int iPlatform;
	const char* pcPlatformPos;
	GetPlatformFromPath(szPath,iPlatform,pcPlatformPos);
	return iPlatform;
}

void Tools_Platform::GetPlatformFromPath(const char* szPath,int& iOutputPlatform,const char*& pcPlatformPos)
{
	const char* pcDir = szPath;
	while(*pcDir)
	{
		//skip past the slashes
		while(*pcDir == '\\' || *pcDir =='/') pcDir++;

		//get the length of the directory name
		int iLength = strcspn(pcDir,"\\/");

		//compare against each platform name
		for(int i=0;i<TOOLS_PLATFORM_NUM_PLATFORMS;i++)
		{
			if(iLength != g_aiPlatformNameLength[i]) continue;

			if(!strnicmp(pcDir,g_aszPlatformNames[i],iLength))
			{
				iOutputPlatform = i;
				pcPlatformPos = pcDir;
				return;
			}
		}
		pcDir += iLength;
	}

	iOutputPlatform = -1;
	pcPlatformPos = 0;
}


int Tools_Platform::RemovePlatformFromPath(const char* szPath, char* szOutputBuffer)
{
	int iPlatform;
	const char* pcPlatformPos;
	GetPlatformFromPath(szPath,iPlatform,pcPlatformPos);

	if(iPlatform == -1)
	{
		strcpy(szOutputBuffer,szPath);
	}
	else
	{
		//copy the part up to the platform
		int iStartLength = pcPlatformPos - szPath;
		strncpy(szOutputBuffer,szPath,iStartLength);

		//skip the platform name
		const char* pcSource = pcPlatformPos;
		pcSource += g_aiPlatformNameLength[iPlatform];
		while(*pcSource == '\\' || *pcSource =='/') pcSource++;

		char* pcDest = szOutputBuffer+iStartLength;

		//copy the rest
		strcpy(pcDest,pcSource);
	}

	return iPlatform;
}
