//
// Tools_Render.h
//
// Created by Kevin Lea - 11/09/2001 10:33:14
// 
// Copyright (C) 2001 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef __TOOLS_RENDER_H__
#define __TOOLS_RENDER_H__


#include "Core/Asura_Maths.h"
#include "Core/Asura_Hash.h"

class Tools_Render
{
	public:
		static void Initialise();
		static void Shutdown();

		static void BeginRenderWorld();
		static void EndRenderWorld();

		static void RenderEnvironment(void);
		static void RenderEnvironmentForCurrentZone(void);

		enum PolygonOutlineMode
		{
			POM_Default,
			POM_ForceWireframe,
			POM_ForceSolid,
		};

		
		static void RenderPolygonOutline(class Asura_Mesh* mesh,const int polyIndex,const Asura_Vector_3* position,const Asura_Matrix_3x3* orientation,unsigned int colour, PolygonOutlineMode mode, float fScale = 1.0f);
		static void RenderPolygonOutline(class Asura_Mesh* mesh,const int polyIndex,const Asura_Vector_3* position,const Asura_Matrix_3x3* orientation,unsigned int colour, PolygonOutlineMode mode, const Asura_Vector_3& xScale);
		static void RenderPolygonOutline(const int polyIndex, const int moduleIndex, unsigned int colour, bool forceWireframe=false);
		static void RenderMeshOutline(class Asura_Mesh* mesh,const Asura_Vector_3* position,const Asura_Matrix_3x3* orientation,unsigned int colour, bool forceWireframe = false, float fScale = 1.0f);
		static void RenderMeshOutline(class Asura_Mesh* mesh,const Asura_Vector_3* position,const Asura_Matrix_3x3* orientation,unsigned int colour, bool forceWireframe, const Asura_Vector_3& xScale);
        static void RenderMeshOutline( int iModuleIndex, u_int uColour, bool bForceWireframe = false, bool bIgnoreModuleLinking = false );
		static void RenderMeshOutlineByZone(Asura_Hash_ID uZoneHash,int iModuleIndex,unsigned int colour, bool forceWireframe = false, bool bIgnoreModuleLinking = false);
		static void RenderModuleOutline(const int moduleIndex,unsigned int colour, bool wireframe);//draws a bounding box for a module
		static void RenderModuleOutline(const int iModuleIndex,u_int uFaceColour,u_int uEdgeColour);
		static void RenderVertexOutline(int iModuleIndex,int iVertexIndex,unsigned int uColour);
		static void RenderCube(const class Asura_Bounding_Box& box,unsigned int colour, bool wireframe,int iHighlightSide = -1,int iHighlightColour = 0xffff0000);
		static void RenderCube(const class Asura_Bounding_Box& box,const Asura_Vector_3& xPosition,const Asura_Matrix_3x3& xOrient,unsigned int colour, bool wireframe,int iHighlightSide = -1,int iHighlightColour = 0xffff0000);
		static void RenderCube(const Asura_Bounding_Box& xBox, const Asura_Vector_3& xPosition, const Asura_Matrix_3x3& xOrient, unsigned int uColour = 0xffffffff);
		static void RenderEdge(int iModuleIndex,int iVert1,int iVert2, u_int uColour);
		static void RenderVertexPerPoly(int iModuleIndex,int iPoly,int iVert, u_int uColour);
		static void RenderCone(const Asura_Vector_3& xPos,const Asura_Matrix_3x3& xOrient,float fSize,Asura_Angle fAngle,u_int uColour,bool bWireframe);
		static void RenderSpheroid(const Asura_Vector_3& xPos,const Asura_Matrix_3x3& xOrient,const Asura_Vector_3& xExtents,u_int uColour,bool bWireframe, bool bPlain = false);
		// Render Cylinder - endpoints are in object space.
		static void RenderCylinder( const Asura_Vector_3& xPos1, const Asura_Vector_3& xPos2, float fRadius, u_int uColour, bool bWireframe );
		// Extended version of cylinder with colours for each section
		static void RenderCylinder( const Asura_Vector_3& xPos1, const Asura_Vector_3& xPos2, float fRadius, u_int uEndColour1, u_int uEndColour2, u_int uSideColour, bool bWireframe );
		static void RenderCollisionMesh(const class Asura_CollisionMesh* pxCollMesh,const Asura_Vector_3* pxPosition,const Asura_Matrix_3x3* pxOrient,u_int uColour);

		static void AddIconToBatch(const Asura_Vector_3& xPosition, int iTextureID, float fSize, u_int uColour = 0xffffffff);
		static void RenderBatchedIcons();

		static void ToggleLighting();//toggles between using lighting , and drawing with full brightness
		static bool LightingIsOn(){return UseLighting;}
		static void UpdateAmbient();//updates Asura_Lighting_System ambient values (if lighting is on)

#if defined( ASURA_TARGET_SM3 )
		static void ToggleTexturing( void ) { s_bUseTexturing ^= true; }
		static bool TexturingIsOn( void ){ return s_bUseTexturing; }

		static void ToggleShowPerPixelNormals( void ) { s_bShowPerPixelNormals ^= true; }
		static bool ShowPerPixelNormalsIsOn( void ){ return s_bShowPerPixelNormals; }
#endif // #if defined( ASURA_TARGET_SM3 )

		static bool HighlightPolyOrObjectUnderCursor;

		static void UseCameraForModuleLinking(){s_eModuleLinkMode = MODULELINK_USECAMERA;}

		static void SetPositionForModuleLinking(const Asura_Vector_3& xPosition);
		static void SetModuleForModuleLinking(int iIndex);

		static bool SetVideoMode(int iWidth,int iHeight,bool bFSAA = false);

	private :
		static bool UseLighting;
#if defined( ASURA_TARGET_SM3 )
		static bool s_bUseTexturing;
		static bool s_bShowPerPixelNormals;
#endif // #if defined( ASURA_TARGET_SM3 )

		static void RenderModule(int iPass,int moduleIndex, class Asura_Mesh& mesh, class Asura_Vector_3& position);
		static void RenderCoronas();

		//used by module linking mode to show the module linking from a different module that the one
		//that the camera is in.
		enum ModuleLinkMode
		{
			MODULELINK_USECAMERA,
			MODULELINK_USEPOSITION,
			MODULELINK_USEINDEX,
		};
		static enum ModuleLinkMode s_eModuleLinkMode;
		static Asura_Vector_3 s_xPositionForModuleLinking;
		static int s_iModuleForModuleLinking;

		static int* s_piModulesToRender;
		static int s_iModulesToRenderSize;
};

#endif//__TOOLS_RENDER_H__
