#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef __TOOLS_RENDER_OPTIONS_H_
#define __TOOLS_RENDER_OPTIONS_H_

#ifdef ASURA_USE_HDR
	#include "Camera/Asura_Camera.h"
#endif

#include "Core/Asura_Hash.h"
#include "Core/Asura_Maths.h"

class Asura_Mesh;
class Tools_Module_Render_Options;
class Tools_Polygon_Render_Options;
//////////////////////////////////////////////////////////////////////////////////////////////////
#define ToolsRenderFlag_PolygonSelectionHide	0x00000001
#define ToolsRenderFlag_ModuleSelectionHide		0x00000002
#define ToolsRenderFlag_ModeHide				0x00000004
#define ToolsRenderFlag_AsrFileManagerHide      0x00000008
#define ToolsRenderFlag_ToBeDestroyed			0x00010000 

enum Tools_SubdivisionMode
{
	TOOLS_SUBDIVIDE_NONE,
	TOOLS_SUBDIVIDE_EVENLY,
	TOOLS_SUBDIVIDE_NEARWAYPOINTS,
};

enum Tools_WireframeMode
{
	TOOLS_WIREFRAME_OFF,//no wireframe
	TOOLS_WIREFRAME_ON,//wireframe only
	TOOLS_WIREFRAME_MIXED,//textured mesh with wireframe on top
};


enum Tools_PSPRenderMode
{
	TOOLS_PSPRENDERMODE_INACTIVE,
	TOOLS_PSPRENDERMODE_ACTUAL_SIZE,
	TOOLS_PSPRENDERMODE_SCREEN_WIDTH,

	TOOLS_PSPRENDERMODE_MAX_MODES
};

//Methods used for highlighting selected polygons etc
enum Tools_HighlightMode
{
	TOOLS_HIGHLIGHT_WIREFRAME,
	TOOLS_HIGHLIGHT_SOLID,
	TOOLS_HIGHLIGHT_TRANSLUCENT,
};



//////////////////////////////////////////////////////////////////////////////////////////////////

//This is used for setting special render option flags on a per polygon basis for modules
class Tools_Render_Options
{
	public :
		static void Initialise();
		static void Shutdown();
		static void FinalShutdown();

		static void Update();
		static void Render();

		static void AddRenderFlagPolygon(int iModule,int iPolygon,int iFlags);
		static void RemoveRenderFlagPolygon(int iModule,int iPolygon,int iFlags);
		static void RemoveRenderFlagFromAllPolygonsInModule(int iModule,int iFlags);

		static void AddRenderFlagModule(int iModule,int iFlags);
		static void RemoveRenderFlagModule(int iModule,int iFlags);

		static int GetRenderFlags(int iModule);
		static int GetRenderFlags(int iModule,int iPolygon);

		static bool IsHidden(int iModule);
		static bool IsHidden(int iModule,int Polygon);
		static bool IsHidden(const Asura_Mesh* pxMesh,int iPolygon);

		//returns true if all the module polygons are hidden , regardless of the overall module hidden state.
		static bool AreAllPolygonsInModuleHidden(int iModule);


		static void HandlePolygonConversion(int module,int* PolyConv1,int* PolyConv2 = 0);
		static void HandleMeshChange(int iModuleIndex);//Used to cope with meshes being replaced by completely different ones
		static void DeleteModule(int iModuleIndex);


		static void AddOverlayControls(class Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset, int iTab);
		

		static bool IsModuleWithinViewDistance(int module);
		static bool IsPointWithinViewDistance(const Asura_Vector_3& point);

		static bool IsUseRenderAsIconDistance() {return s_bUseRenderAsIconDistance;}
		static float GetRenderAsIconDistance() {return s_fRenderAsIconDistance;}
		static float GetDistantIconSize() {return s_fDistantIconSize;}

		static void UpdateFieldOfView();
		static void UseDefaultFieldOfView();

		//Allow individual flags to be switched on and off. This allows the hide flags set by polygon selection
		//to remain dormant while outside of modes in which it can be used
		static void ActivateHideFlag(int iFlags);
		static void DeactivateHideFlag(int iFlags);
		static int GetHideFlags(){return s_iActiveHideFlags;}

		//Sets polygon flags that should cause polygons to be hidden.
		//At the moment Dont Render ,No Collision , Ignore By LOS are controlled automatically
		static void SetPolygonFlagToHide(int iFlags,bool bAdd);
		static void SetMaskedPolygonFlagsToHide(int iFlag,int iMask);
		static int GetPolygonFlagsToHide(){return s_iPolygonFlagsToHide;}

		//activates/deactivates the Don't Render polygon flag
		static void SetIgnoreDontRenderFlag(bool bSet){s_bIgnoreDontRenderFlag_Mode = bSet;}

		//Sets polygon flags that should cause a polygon to be visible , regardless of the don't render flag
		static void SetAlwaysVisiblePolygonFlags(int iFlags);
		static int GetAlwaysVisiblePolygonFlags(){return s_iAlwaysVisiblePolygonFlags;}

		//recalulcates whether a polygon should be visible
		static void UpdatePolygonVisibility(int iModule,int iPolygon);
		static void UpdateAllPolygonVisibilities();

		static float GetPlayerHeight();
		//If player height hasn't been overridden by user , this moves the height up through the levels
		//towards the stand height. Returns false if already standing
		static bool MoveTowardsStandPosture();
		//Similarly towards the prone height
		static bool MoveTowardsPronePosture();

		static bool IsEnvironmentHidden() {return HideEnvironment;}

		static bool ShowDestroyedPolygons;
		static bool HideEnvironment;

		static bool DefaultFieldOfViewIsVertical;
		static Asura_Angle DefaultFieldOfView;

        static bool NoBackFaceCull;

        static Tools_SubdivisionMode GetSubdivisionMode( void ) { return s_eSubdivideMode; }
		static void SetSubdivisionMode(enum Tools_SubdivisionMode eMode);
		static void ApplySubdivisionMode();

//        static bool ShouldSubdivideForPS2TextureSize( void ) { return s_bShouldSubdivideForPS2TextureSize; }


		static void ToggleFrameRateClamped() { s_bClampFrameRate = !s_bClampFrameRate; }
		static bool IsFrameRateClamped(){return s_bClampFrameRate;}
		static float GetMaxFrameRate(){return s_fMaxFrameRate;}

		static bool IsInPSPMode(void)							{ return (s_iPSPRenderMode != TOOLS_PSPRENDERMODE_INACTIVE); }
		static bool IsPSPModeActualSize(void)					{ return (s_iPSPRenderMode == TOOLS_PSPRENDERMODE_ACTUAL_SIZE); }
		static void TogglePSPMode(void);
		static void SetPSPMode(const Tools_PSPRenderMode eMode);
		static void PSPModeChanged(void);

		static bool IsUsingGlobalIllumination() {return s_bUseGlobalIllumination;}

		static bool IsGlowEffectEnabled() { return s_bGlowEffectEnabled; }

		//Are polygons that have been merged from placed obejcts being hidden?
		static bool IsHidingObjectPolygons(){return s_bHideObjectPolygons && !s_bDisableHideObjectPolygonFlag;}
		//For disabling the hide object polygon flag while in object placement mode
		static void SetDisableHideObjectPolygonFlag(bool bDisable);
		static bool IsHideObjectPolygonFlagEnabled(){return !s_bDisableHideObjectPolygonFlag;}


		static void SetUseDiscoLights(bool bUse);
		static bool GetUseDiscoLights(){ return s_bDiscoLights;}

		static Tools_WireframeMode GetWireframeMode(){return s_eWireframeMode;}
		static void NextWireframeMode();

		//Gets wireframe colour to use for module. If -1 then the generic wireframe colour is got
		static u_int GetWireframeColour(bool bMixedWireframe = true,int iModule = -1);
		static void SetWireframeColour(u_int uColour) {s_uWireframeColour = uColour;}

		//Sets the highlight mode used on selected polygons etc.
		static Tools_HighlightMode GetPolygonHighlightMode(){return s_ePolygonHighlightMode;}
		static void SetPolygonHighlightMode(Tools_HighlightMode eMode) { s_ePolygonHighlightMode = eMode;}
		static void NextPolygonHighlightMode();

		//Overlay scaling for special modes (like PSP rendering mode)
		static void  EnableOverlayScaling(bool bScale){s_bEnableOverlayScaling=bScale;}
		static bool  IsOverlayScalingEnabled();
		static float GetOverlayScale();

#ifndef ASURA_USE_VISIBILITY_SYSTEM
		//toggles use of module links for current rendering mode (normal or collision mesh)
		static void ToggleUseModLinks();
#endif

		//Use module links for standard (not coll mesh) rendering
		static void SetUseModuleLinksForStandardRendering(bool bUse){s_bUseModuleLinks = bUse;}

		//to allow Tools_TextureAnimation to share the same tickbox
		static void AddDisableTextureAnimControl(class Asura_Overlay* pxOverlay,Asura_Vector_2& xOffset);


		// Allow cut-down botanicals draw distance for better performance
		static bool IsBotanicalsRangeLimited() {return s_bLimitBotanicalsRange;}
		static float GetBotanicalsRange()	   {return s_fBotanicalsRange;}

		#ifdef ASURA_USE_STREAMING
		static bool IsHidingOtherZones(){return s_bHideOtherZones;}
		#endif

		static bool GetDisplayCollisionPolys() {return s_bDisplayCollisionPolys;}

		//Indicates that the tools mode will control the active state of the PFX entities as required.
		static void SetPFXControlledByMode(bool bSet);
		// Get show PFX setting.
		static bool GetShowPFX() { return s_bShowPFX; }

#ifdef ASURA_SUPPORT_WEATHER_SYSTEM
		//Enables sun glare , if it has been switched on for the level.
		static void SetModeEnableSunGlare(bool bSet){s_bModeEnableSunGlare = bSet;}
#endif

#ifdef ASURA_USE_FSFX
		//Indicates that the tools mode will control current FSFX template.
		static void SetFSFXControlledByMode(bool bSet);
		static void UpdateFSFXSettings();
#endif

#ifdef ASURA_USE_HDR
		static void GetHDRSettings( Asura_CameraData::HDRSettings& xSettings )
		{
			xSettings = s_xHDRSettings;
		}
#endif

#ifdef ASURA_TARGET_SM3
		static bool ShouldForceDynamicLightsOn(){return s_bForceDynamicLightsOn;}
		static void SetForceDynamicLightsOn(bool bSet);
#endif

		static bool GetEnableEffectVolume()	{return s_bEnableEffectVolume;}

	private :

		static bool IsShowPFXTickboxEnabled(){return !s_bPFXControlledByMode;}

#ifdef ASURA_SUPPORT_WEATHER_SYSTEM
		static bool IsEnableSunGlareTickboxEnabled(){return !s_bModeEnableSunGlare;}
#endif

#ifdef ASURA_USE_FSFX
		static bool IsFSFXControlEnabled(){return !s_bFSFXControlledByMode;}
#endif

		static void UpdateDisplaySettings();

		static void SetGamma(float fGamma);

		//callbacks for physics debug tickbox
		static bool IsShowPhysicsDebugInfo(void* pJunk);
		static void SetShowPhysicsDebugInfo(bool bSet,void* pJunk);

#ifdef ASURA_USE_HDR
		static bool IsUsingFixedExposure() { return !s_xHDRSettings.m_bDynamicExposure; }
		static bool IsUsingDynamicExposure() { return s_xHDRSettings.m_bDynamicExposure; }
		static void OnCaptureLuminance();
#endif

		//data
		static int NumberOfModules;
		static Tools_Module_Render_Options** ModuleData;

		static int * s_piMeshToModuleIndex;


		static bool FieldOfViewIsVertical;
		static float HorizontalFieldOfViewInDegress;
		static float VerticalFieldOfViewInDegress;
		static bool FieldOfViewChanged;

		static bool UseViewDistance;
		static float ViewDistance;

		static bool s_bUseRenderAsIconDistance;
		static float s_fRenderAsIconDistance;
		static float s_fDistantIconSize;

		static float s_fGamma;
		static bool s_bGammaChanged;

		static bool s_bHighlightDontExportPolygons;

		static int s_iActiveHideFlags;
		static int s_iAlwaysVisiblePolygonFlags;//flags that should cause polygon to be flagged even if marked don't render
		static int s_iPolygonFlagsToHide;//polygon flags that should cause polygon to be hidden(if not excluded by above)

		static bool s_bUsePlayerHeight;
		static float s_fPlayerHeight;
		
		static bool s_bDisplayCollisionPolys;

		enum PLAYER_POSTURE {POSTURE_STAND,POSTURE_CROUCH,POSTURE_PRONE} ;
		static int s_iPlayerPosture;

		//For getting polygons marked as 'Don't Render' drawn.
		static bool s_bIgnoreDontRenderFlag_User;//set by the checkbox on the render options overlay
		static bool s_bIgnoreDontRenderFlag_Mode;//set by individual modes , as needed

		//subdivision options
		static enum Tools_SubdivisionMode s_eSubdivideMode;
		static float s_fCurvedSubdivideSize;
		static float s_fFlatSubdivideSize;
		static bool s_bPrelightSubdividedMeshes;
//        static bool s_bShouldSubdivideForPS2TextureSize;

		static bool s_bRenderAllWhenInNoModule;
		static bool s_bNoBackFaceCullChanged;

		static bool s_bTestDualTextureRenderer;

		static bool s_bClampFrameRate;
		static float s_fMaxFrameRate;

		static int s_iPSPRenderMode;
		
		static bool s_bUseGlobalIllumination;
		static bool s_bUseGlobalIlluminationChanged;

		static bool s_bGlowEffectEnabled;
		static bool s_bGlowEffectEnabledChanged;

		static bool s_bHideObjectPolygons;
		static bool s_bHideObjectPolygonsChanged;
		static bool s_bDisableHideObjectPolygonFlag;

		static bool s_bHideOtherZones;

		static float s_fCursorSpeed;
		static bool s_bCursorSpeedChanged;

		static bool s_bShowHighPolyModules;
		static int s_iHighPolyModuleThreshold;

		static bool s_bDiscoLights;//fake lighting for showing up smoothing

		static Tools_WireframeMode s_eWireframeMode;
		static u_int s_uWireframeColour;
		static bool s_bUseMaxWireframeColour;

		static Tools_HighlightMode s_ePolygonHighlightMode;

		static bool s_bUseModuleLinks;
		static bool s_bUseModuleLinksCMesh;

		static bool s_bEnableTextureAnims;
		static bool s_bEnableEffectVolume;


		static bool  s_bLimitBotanicalsRange;
		static float s_fBotanicalsRange;

		static bool s_bShowPFX;
		static bool s_bShowPFXChanged;
		static bool s_bPFXControlledByMode;

#ifdef ASURA_SUPPORT_WEATHER_SYSTEM
		static bool s_bUserEnableSunGlare;
		static bool s_bModeEnableSunGlare;
#endif

#ifdef ASURA_USE_FSFX
		static Asura_Hash_ID s_uFSFXHash;
		static float s_fFSFXIntensity;
		static bool s_bFSFXChanged;
		static bool s_bFSFXControlledByMode;
#endif

#ifdef ASURA_TARGET_SM3
		static bool s_bForceDynamicLightsOn;
#endif


#ifdef ASURA_SUPPORT_MULTIPLE_MONITORS
		static bool s_bUseMultipleMonitors;
		static bool s_bUseMultipleMonitorsChanged;
		static bool s_bUseMultipleMonitorsWarningEmitted;
#endif


		static bool s_bUseSkinnedUI;
		static bool s_bUseSkinnedUIChanged;

		static int* s_piSelectedResolutions;
		static Asura_Char* s_pszScreenNames;
		
		friend class Tools_OptionsFile;

		static bool IsHorizontalFOVEnabled (void)	{ return(!FieldOfViewIsVertical); }
		static bool IsVerticalFOVEnabled (void)		{ return(FieldOfViewIsVertical); }
#if defined(ASURA_USE_SCREENSHOT_SYSTEM)
		static bool IsHiResScreenshotModeEnabled(void)		{ return(s_bEnableHiResScreenshotMode); }
#endif	//defined(ASURA_USE_SCREENSHOT_SYSTEM)

		static bool  s_bEnableOverlayScaling;

#ifdef ASURA_USE_HDR
		static Asura_CameraData::HDRSettings s_xHDRSettings;
#endif

#if defined(ASURA_USE_SCREENSHOT_SYSTEM)
		static bool	s_bEnableHiResScreenshotMode;
		static bool	s_bDisableMipMaps;

		static int	s_iHiResScreenshotMagnification;
		static bool	s_bHiResScreenshotMagnificationChanged;
#endif	//defined(ASURA_USE_SCREENSHOT_SYSTEM)
};
//////////////////////////////////////////////////////////////////////////////////////////////////

//render option data for one module
class Tools_Module_Render_Options
{
	public :
		Tools_Module_Render_Options(int moduleIndex,Asura_Mesh* mesh);
		~Tools_Module_Render_Options();

		void AddRenderFlagModule(int iFlags);
		void RemoveRenderFlagModule(int iFlags);

		void AddRenderFlagPolygon(int iPolygon,int iFlags);
		void RemoveRenderFlagPolygon(int iPolygon,int iFlags);

		void HandlePolygonConversion(Asura_Mesh* mesh,int* PolyConv1,int* PolyConv2);
		void HandleMeshChange(Asura_Mesh* pxMesh);

		void UpdatePolygonVisibility(int iPolygon);
		bool IsPolygonHidden(int iPolygon);


		//data
		int ModuleIndex;
		Asura_Mesh* Mesh;
		int NumberOfPolygons;
		Tools_Polygon_Render_Options* PolygonData;
		bool WithinViewDistance;//
		int m_iRenderFlags;
		int m_iNumVisiblePolygons;
};
//////////////////////////////////////////////////////////////////////////////////////////////////

//render options for one polygon
class Tools_Polygon_Render_Options
{
	public :
		Tools_Polygon_Render_Options();
		int m_iRenderFlags; //uses the ToolsRenderFlag flags
		bool m_bHidden;
};



#endif //__TOOLS_RENDER_OPTIONS_H_
