#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef _Tools_Timed_Messages_h
#define _Tools_Timed_Messages_h

#include <stdarg.h> 

/*
For displaying messages that will last for a few seconds before fading out
*/
struct Timed_Message;
class Tools_Timed_Message_System
{
	public :
        
        // Some helpful colour constants.
        enum
        {
            COLOUR_DEFAULT = 0xFFFFFFu,      // White.
            COLOUR_ERROR   = 0xFF0000u,      // Red.
            COLOUR_INFO    = COLOUR_DEFAULT,
            COLOUR_WARNING = 0xFFFF00u,      // Yellow.
			COLOUR_SUCCESS = 0x00FF00u,      // Green
        };

		static void Initialise();
		static void Render();
		static void Shutdown();

		static void AddMessage(int iColour,const char* szFormat, ...);
		//Adds a message that won't be logged by Tools_MessageLog
		static void AddUnloggedMessage(int iColour,const char* szFormat, ...);

		//As AddMessage but deletes any matching message that is already being displayed
		static void AddMessageRemovingDuplicate(int iColour,const char* szFormat, ...);

		static void DisableTimer();
		static void EnableTimer();

		//If this is set to true then any messages posted while tools isn't active will be
		//to the console instead.
		static void SetRedirectToConsole(bool bSet) {s_bRedirectToConsole = bSet;}
		static bool GetRedirectToConsole(){return s_bRedirectToConsole;}
		static void SetForceHideMessageSystem (bool flag) { s_bForceHideMessages = flag; }

	private :
		static void AddMessageInternal(bool bLog,int iColour, bool bRemoveDuplicate,const char* szFormat, va_list xArgs);

		static bool TimerEnabled;
		static bool s_bRedirectToConsole;
		static bool s_bInitialised;
		static bool s_bForceHideMessages;
};

#endif
