//
// Tools_VertexPerPoly_Selection.h
//
// Created by JGP 22/06/06
//
// Copyright (C) 2006 Rebellion (Derby), All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif


#ifndef __TOOLS_VERTEXPERPOLY_SELECTION_H__
#define __TOOLS_VERTEXPERPOLY_SELECTION_H__


#include "Tools_Undo.h"
#include "Core/Asura_Maths.h"
#include "Misc/list_tem.hpp"
#include "Misc/Tools_Polygon_Selection.h"


struct ModuleVertexPerPolyTriplet
{

	ModuleVertexPerPolyTriplet() 
	{
	}

	ModuleVertexPerPolyTriplet( int m, int p, int v ) 
	{ 
		m_iModuleIndex = m; 
		m_iPolyIndex = p; 
		m_iVertexIndex = v; 
	}

	bool operator ==( const ModuleVertexPerPolyTriplet& xOther ) 
	{
		return ( ( xOther.m_iModuleIndex == m_iModuleIndex ) && ( xOther.m_iPolyIndex == m_iPolyIndex ) && ( xOther.m_iVertexIndex == m_iVertexIndex ) );
	}

	bool operator !=( const ModuleVertexPerPolyTriplet& xOther ) 
	{
		return ( !( *this == xOther ) );
	}

	int m_iModuleIndex;
	int m_iPolyIndex;
	int m_iVertexIndex;

};


struct VertexPerPolySelectionUndoData : public ModuleVertexPerPolyTriplet
{

	enum UndoType
	{
		UndoType_Added,
		UndoType_Removed,
		UndoType_ForceInt = 0xffffffff,
	};
	
	enum UndoType m_eUndoType;

};


class VertexPerPolySelectionUndo : public FixedSizeUndo
{
public:

	VertexPerPolySelectionUndo()
		: FixedSizeUndo( 10000, sizeof( VertexPerPolySelectionUndoData ) )
	{
	}

	void ProcessUndoData( void* pData );
	
	void LogVertexPerPolySelected( int iModule, int iPoly, int iVert );
	void LogVertexPerPolyDeselected( int iModule, int iPoly, int iVert );

};


enum VertexPerPolySelectionMode
{
	VPSelectionMode_VertexPerPoly,
	VPSelectionMode_Polygon,
	VPSelectionMode_Object,
};


#define VertexPerPolySelectionUpdate_Standard 0
#define VertexPerPolySelectionUpdate_NoMouseSelection 1
#define VertexPerPolySelectionUpdate_NoSelection 2


class Tools_VertexPerPoly_Selection
{

public: 

	static void Initialise();
	static void Shutdown();
	
	static void Activate();
	static void Deactivate();

	static void Update( int iUpdateMode = VertexPerPolySelectionUpdate_Standard );
	static void Render();

	static bool IsActive() { return s_bActive; }

	static void SelectVertexPerPoly( int iModule, int iPoly, int iVert, const bool bAllowUndo = true ); 
	static void DeselectVertexPerPoly( int iModule, int iPoly, int iVert, const bool bAllowUndo = true );
	static void SelectPolygon( int iModuleIndex, int iPolygonIndex ); 
	static void DeselectPolygon( int iModuleIndex, int iPolygonIndex );
	static void SelectObject( int iModuleIndex ); 
	static void DeselectObject( int iModuleIndex );
	static void SelectAll();
	static void DeselectAll();

	static void HandleMeshChange(int iModuleIndex);
	static void DeleteModule(int iModuleIndex);

	//iterating through the selection
	
	static List<ModuleVertexPerPolyTriplet>* GetList() { return &s_xSelectionList; }
	static const int CountSelectedVertexPerPolys();

	// overlay

	static void ToggleSelectionOverlay();
	static bool IsSelectionOverlayOn();

private:

	//selection overlay functions

	static void CreateSelectionOverlay();
	static void CloseSelectionOverlay();
	
	static bool GetVertexPerPolyUnderMouse( ModuleVertexPerPolyTriplet& xVertexPerPoly );
	static bool GetHighlitVertexPerPoly( ModuleVertexPerPolyTriplet& xVertexPerPoly );

	static void ClearUndo() { s_xSelectionUndo.FlushUndoBuffer(); }

	static void HighlightSelection();													// red outline
   	static void HighlightVertexPerPoly( ModuleVertexPerPolyTriplet& xVertexPerPoly );	// pulsing white outline
   	static void HighlightPolygon( PolygonID& xPolygon );								// pulsing white outline

	static void Undo();	
	static void Redo();	

	static void HandleKeyboard();
	static void HandleMouseSelection();
	static void TransferModeData( int iMode );

	// variables

	static bool s_bShowSelection; 
	static bool s_bSelectionLocked;
	static int s_iSelectionMode;								// value from enum VertexPerPolySelectionMode
	static int s_iSelectionModeListOffset;						// just used since I need a paramater for the list overlay thingy

	static Asura_Vector_2 s_xSelectionOverlayPosition;
	static bool s_bSelectionOverlayOpen;
	static u_int s_uSelectionOverlayMinimised;
	
	static List<ModuleVertexPerPolyTriplet> s_xSelectionList;

	static VertexPerPolySelectionUndo s_xSelectionUndo;
	
	static bool s_bButtonDownForSelection;						// used to determine whether the mouse button was pressed to start polygon selection
	static ModuleVertexPerPolyTriplet s_xHighlitVertexPerPoly;	// VertexPerPoly mouse is currently over
	static PolygonID s_xHighlitPolygon;							// polygon mouse is currently over

	static bool s_bActive;

};



#endif // __TOOLS_VERTEXPERPOLY_SELECTION_H__
