//
// Tools_Mode_Module.h
//
// Created by Richard Rice - 21/03/2003
// 
// Copyright (C) 2003 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef __TOOLS_MODE_MODULE_H__
#define __TOOLS_MODE_MODULE_H__

#include "Core/Asura_Handle.h"
#include "Core/Asura_Maths.h"

#include "Tools_Mode.h"
#include "Tools_Undo.h"

///////////////////////////////////////////////////////////////////////////////////////
struct ModuleUndoData
{
	enum UndoType
	{
		UndoType_FlagsChanged,
		UndoType_LightSetsChanged,
		UndoType_ForceInt = 0xffffffff,
	};
	
	enum UndoType m_eUndoType;
	int m_iModuleIndex;
	u_int m_uFlags;
};

class ModuleModeUndo : public FixedSizeUndo
{
public :
	ModuleModeUndo()
	:FixedSizeUndo(10000,sizeof(ModuleUndoData)){}

	void ProcessUndoData(void*);
	
	void LogFlagsChanged(int iModuleIndex);
	void LogLightSetsChanged(int iModuleIndex);
};

///////////////////////////////////////////////////////////////////////////////////////


class Tools_Mode_Module : public Tools_Mode
{
    public:

        Tools_Mode_Module( void );
        ~Tools_Mode_Module( void );


        virtual void Update(void);
		virtual void RenderWorld_PostToneMap(void);
        virtual void RenderUI(void);
    
        virtual bool IsModified( void );
		virtual void QuickSave(void);

		virtual void InitialiseMode();
		virtual void ShutdownMode();

		enum SubMode
		{
			SUBMODE_MODULE_FLAGS,
			SUBMODE_LIGHTSETS
		};



    protected:
		//loads/saves data for this mode
		void TransferModeData(int iMode);


		static void ToggleModuleOverlay(void);
		static bool ModuleOverlayIsOn(void);
		void CreateModuleOverlay(void);


        void HandleKeyboard(void);

		void RenameModule();

		static void ApplyFlag(int iThis){((Tools_Mode_Module*)iThis)->ApplyFlag();}
		static void RemoveFlag(int iThis){((Tools_Mode_Module*)iThis)->RemoveFlag();}

		void ApplyFlag();
		void RemoveFlag();
		void SetFlag(int iFlag,bool bAdd);
		void SetLightSets(u_int uLightSets, u_int uMask);


		static void Undo(int iThis){((Tools_Mode_Module*)iThis)->Undo();}
		static void Redo(int iThis){((Tools_Mode_Module*)iThis)->Redo();}

		void Undo();
		void Redo();
		void PrepareUndo();

		void UpdateSubModeFromSelectedTab();

		///////////////////////////
		//data
		Asura_Handle m_iMenuHandle;

		Asura_Vector_2 m_xModuleOverlayPosition;
		bool m_bModuleOverlayOpen;

		int m_iCurrentFlagIndex;

		//for light set editing
		u_int m_uSelectedLightSets;
		bool m_bReplaceLightSets;



		enum SubMode m_eSubMode;
		static u_int s_uSelectedTab;

 
		ModuleModeUndo m_xModuleUndo;

		static Tools_Mode_Module* s_pxThis;

};

#endif //__TOOLS_MODE_MODULE_H__
