//
// Tools_Mode_ModuleLinking.cpp
//
// Created by Kevin Lea - 23/07/2001 14:03:08
// 
// Copyright (C) 2001 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

#include "AI/Asura_AI_Navigation.h"
#include "Camera/Asura_Camera.h"
#include "Core/Asura_UserInput.h"
#include "Core/Asura_Timers.h"
#include "Core/Asura_Los.h"
#include "Debugging/Asura_Assert.h"
#include "Interface/Asura_Cursor.h"
#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Overlay_System.h"
#include "Interface/Asura_Menubar_System.h"
#include "Interface/Asura_Menubar.h"
#include "Interface/Asura_Overlay.h"
#include "Interface/Asura_Interface.h"
#include "Mesh/Asura_Mesh.h"
#include "Mesh/Asura_Env_System.h"
#include "Mesh/Asura_Env.h"
#include "Rendering/Asura_Module.h"
#include "Rendering/Asura_Text_Output.h"
#include "Rendering/Asura_Render.h"
#include "Rendering/Asura_Viewport.h"


#include "Tools_System.h"
#include "Tools_Overlays.h"
#include "Entity/Tools_Entity_Object.h"
#include "Entity/Tools_Entity_PhysicsObject.h"
#include "Interface/Tools_ModuleSelect_Overlay.h"
#include "Load&Save/Tools_BatchProcessing.h"
#include "Load&Save/Tools_ModeDataFile.h"
#include "Load&Save/Tools_SaveAsura.h"
#include "Misc/Tools_Timed_Messages.h"
#include "Misc/Tools_Help.h"
#include "Misc/list_tem.hpp"
#include "Misc/Tools_LOS.h"
#include "Misc/Tools_Render.h"
#include "Misc/Tools_Modal_Overlay.h"
#include "Misc/Tools_Render_Options.h"
#include "ModuleLinking/Tools_Mode_ModuleLinking.h"
#include "Objects/Tools_Object_System.h"
#include "Regions/Tools_Mode_Regions.h"
#include "Streaming/Tools_Streaming_System.h"

#define TOOLS_RECOMMENDED_LINKED_IN_QUADS_LIMIT 4000


///////////////////////////////////////////////////////////////////////////////////////
/*--------------**
** Help Strings **
**--------------*/
static const char* ModuleLinkingHelpStrings [] =
{
	"LMB :              Add module link",
	"Shift+LMB :        Remove module link",
	"M :                Toggle 'Add Links' (semi-automatic linking)",
	"N :                Toggle 'Auto Set Base'",
	"Ctrl+S :           Quicksave",
	"B :                Set base module to module under cursor",
	NULL
};


///////////////////////////////////////////////////////////////////////////////////////

//static member variables
Asura_Handle					Tools_Mode_ModuleLinking::MenuHandle;

Asura_Vector_2					Tools_Mode_ModuleLinking::ModuleLinkOverlayPosition = Asura_Vector_2(3.0f,28.0f);
bool							Tools_Mode_ModuleLinking::ModuleLinkOverlayOpen = true;

bool							Tools_Mode_ModuleLinking::AddingLinks;
bool							Tools_Mode_ModuleLinking::AutoSetBaseModule = true;
bool							Tools_Mode_ModuleLinking::s_bIncludeStaticObjects = false;
bool							Tools_Mode_ModuleLinking::s_bViewStaticObjects = false;
bool							Tools_Mode_ModuleLinking::s_bStaticObjectsCreated = false;
bool							Tools_Mode_ModuleLinking::s_bStaticObjectsVisible = false;
bool							Tools_Mode_ModuleLinking::s_bSelectModuleUnderCamera = false;
bool							Tools_Mode_ModuleLinking::s_bIgnoreTransparentPolygons = false;
bool							Tools_Mode_ModuleLinking::s_bAddingLinksUsingMouse = false;
int								Tools_Mode_ModuleLinking::BaseModuleIndex = -1;
int								Tools_Mode_ModuleLinking::s_iLinkDirection = TOOLS_MODULELINK_NORMAL;
bool							Tools_Mode_ModuleLinking::s_bHighlightCursorModule = true;
bool							Tools_Mode_ModuleLinking::s_bShowCursorModuleName = true;
bool							Tools_Mode_ModuleLinking::s_bShowBaseModuleExtents = false;
float							Tools_Mode_ModuleLinking::s_fMaxLinkRange = 100.0f;
bool							Tools_Mode_ModuleLinking::s_bUseMaxLinkRange = false;


extern bool DrawUnlinkedModules;
void Tools_Mode_ModuleLinking::Initialise(void)
{
	DrawUnlinkedModules = true;
	AddingLinks = false;
	BaseModuleIndex = -1;
	s_bAddingLinksUsingMouse = false;

	//load values from .dat file
	TransferModeData(TOOLSMODEDATAFILE_LOAD);
	
	//create the menubar
	MenuHandle = Asura_Menubar_System::CreateMenuSection();
	Asura_Menubar_Title* title;

	title = Asura_Menubar_System::AddTitle(MenuHandle,"Overlays");
	title->AddItem("Module Linking",ToggleModuleLinkOverlay,ModuleLinkOverlayIsOn);

	//create overlays
	CreateModuleLinkOverlay();


	//add help
	Tools_Help::AddHelpTopic("Module Linking Mode",ModuleLinkingHelpStrings);
	
	s_bStaticObjectsVisible = false;
	s_bStaticObjectsCreated = false;
	UpdateStaticObjects();
}

void Tools_Mode_ModuleLinking::Shutdown(void)
{
	//save values to .dat file
	TransferModeData(TOOLSMODEDATAFILE_SAVE);

	Tools_Help::RemoveHelpTopic("Module Linking Mode");
	//shutdown the menu
	if(MenuHandle != ASURA_INVALID_HANDLE)
	{
		Asura_Menubar_System::DestroyMenuSection(MenuHandle);
		MenuHandle = ASURA_INVALID_HANDLE;
	}
	
	//close overlays
	Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_MODULELINKING);
	
	if (s_bStaticObjectsCreated)
	{
		//delete all the objects
		Tools_Object_System::DeleteAllModeSpecific();
	}

	DrawUnlinkedModules = false;
	Tools_Render::UseCameraForModuleLinking();
}

void Tools_Mode_ModuleLinking::TransferModeData(int mode)
{
	//open the file
	Tools_ModeDataFile file("ModuleLinkMode.dat",mode);

	//tranfer the data
	//(none of these should be removed , and new data should go on the end, otherwise
	//the files will get screwed up)
	file.Transfer(ModuleLinkOverlayPosition);
	file.Transfer(ModuleLinkOverlayOpen);
	file.Transfer(s_bShowCursorModuleName);
	file.Transfer(s_bHighlightCursorModule);
	file.Transfer(s_bUseMaxLinkRange);
	file.Transfer(s_fMaxLinkRange);
}

void Tools_Mode_ModuleLinking::Update(void)
{
	
	HandleKeyboard();
	
	if(AutoSetBaseModule)
	{
		if(s_bSelectModuleUnderCamera)
		{
			//Do a line of sight test down to find the module , and show the module linking from there
			const Asura_Vector_3* pxCameraPosition = Asura_Camera::GetPosition();
			Asura_Vector_3 xDirection(0,1,0);
			int iModuleIndex;
			int iJunk;
			float fJunk;
			Tools_LOS::SetIgnoreModuleLinking(true);
			if(Tools_LOS::FindPolyInLOS(*pxCameraPosition,xDirection,iJunk,iModuleIndex,fJunk,false) &&
			   !Asura_Environment_System::GetEnvironment()->IsSceneryModule(iModuleIndex) &&
			   !Asura_Environment_System::GetEnvironment()->IsNeverCameraModule(iModuleIndex))
			{
				BaseModuleIndex = iModuleIndex;
				Tools_Render::SetModuleForModuleLinking(BaseModuleIndex);
			}
			else
			{
				BaseModuleIndex = -1;
				Tools_Render::UseCameraForModuleLinking();
			}
			Tools_LOS::SetIgnoreModuleLinking(false);


		}
		else
		{
			//work out which module we are in
			BaseModuleIndex = Asura_Module_System::GetCameraModuleIndexFromPosition(*Asura_Camera::GetPosition());
			Tools_Render::SetPositionForModuleLinking(*Asura_Camera::GetPosition());
		}
	}
	else
	{
		if(BaseModuleIndex==-1)
		{
			Tools_Render::UseCameraForModuleLinking();
		}
		else
		{
			Tools_Render::SetModuleForModuleLinking(BaseModuleIndex);
		}

	}

	UpdateStaticObjects();
	
	if (!Asura_Interface::UsingMenusOrOverlays())
	{
		if(Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_MOUSEBUTTON1))
		{
			//mouse button has been clicked while not on menus or overlays , so allow it to be used
			//for adding links as long as it is held. RAR 07/04/03
			s_bAddingLinksUsingMouse = true;
		}
		
		
		if(Asura_UserInput::KeyPressed(ASURA_KEYID_MOUSEBUTTON1) && s_bAddingLinksUsingMouse)
		{
			//link/unlink the module the mouse is over
			//get the z direction for the mouse
			if(s_bIgnoreTransparentPolygons) Tools_LOS::SetIgnoreTransparent(true);
			Asura_Hash_ID uZoneHash;
			int iModule;
			if(Tools_LOS::FindModuleUnderCursor(iModule,&uZoneHash))
			{
				SetModuleLink(iModule,uZoneHash,!Asura_UserInput::KeyPressed(ASURA_KEYID_SHIFT),s_iLinkDirection);
			}
			if(s_bIgnoreTransparentPolygons) Tools_LOS::SetIgnoreTransparent(false);
			
		}
	}

	if(!Asura_UserInput::KeyPressed(ASURA_KEYID_MOUSEBUTTON1))
	{
		s_bAddingLinksUsingMouse = false;
	}
	
	if (AddingLinks)
	{
		AddModuleLinks();
	}

	//update enabled status of checkboxes according to current settings
	Asura_Overlay* pxOverlay = Asura_Overlay_System::GetOverlay(TOOLS_OVERLAY_MODULELINKING);
	if(pxOverlay)
	{
		Asura_Dialog* pxDialog;
		
		pxDialog = pxOverlay->GetDialog(DialogID_SelectModuleUnderCamera);
		if(pxDialog)
		{
			pxDialog->SetEnabled(AutoSetBaseModule);
		}
	}


}

void Tools_Mode_ModuleLinking::RenderWorld_PostToneMap(void)
{
	//highlight the base module
	if(BaseModuleIndex!=-1)
	{
		if(AutoSetBaseModule)
		{
			//Make sure polygon outlines don't get done as wireframe
			Tools_HighlightMode eHighlightBackup = Tools_Render_Options::GetPolygonHighlightMode();
			Tools_Render_Options::SetPolygonHighlightMode(TOOLS_HIGHLIGHT_SOLID);

			const Asura_Mesh* pxMesh = Asura_Module_System::GetMeshForModule(BaseModuleIndex);
			for(int i=0;i<pxMesh->NumberOfPolygons;i++)
			{
				Tools_Render::RenderPolygonOutline(i,BaseModuleIndex,0x2affffff);
			}

			Tools_Render_Options::SetPolygonHighlightMode(eHighlightBackup);
		}
		else
		{
			Tools_Render::RenderMeshOutline(BaseModuleIndex,0xffff0000);
		}
		
		if(s_bShowBaseModuleExtents)
		{
			Tools_Render::RenderModuleOutline(BaseModuleIndex,0x70ffffff,0xffffffff);
		}
	}

	if(!Asura_Interface::UsingMenusOrOverlays())
	{
		//highlight module under cursor
		int iModuleIndex;
		Asura_Hash_ID uZoneHash;
	  	bool bOverModule = Tools_LOS::FindModuleUnderCursor(iModuleIndex,&uZoneHash);
		if(bOverModule)
		{
			bool bRenderOutline = s_bHighlightCursorModule;
			if(iModuleIndex == BaseModuleIndex)
			{
				#ifdef ASURA_USE_STREAMING
				if(uZoneHash == Tools_Streaming_System::GetHashForThisLevel())
				#endif
				{
					bRenderOutline = false;
				}
			}
			
			if(bRenderOutline)
			{
				Tools_Render::RenderMeshOutlineByZone(uZoneHash,iModuleIndex,0xff7f7f7f);
			}
		}
	}
}

void Tools_Mode_ModuleLinking::RenderUI(void)
{
	float x = 2;
	float y = 5;

	//show the name of the base module
	if(BaseModuleIndex!=-1)
	{
		Asura_Mesh* mesh = Asura_Module_System::GetMeshForModule(BaseModuleIndex);
		Asura_Text_Output::Print(x,y++,0xffffffff,0,"Base Module : %s",mesh->Name.GetString());
	}
	if(!AutoSetBaseModule && BaseModuleIndex!=-1)
	{
		//see if camera is inside the base module
		Asura_Vector_3 location; 
		Asura_Module_System::GetPositionOfModule(BaseModuleIndex,location);
		Asura_Vector_3 point = *Asura_Camera::GetPosition() - location;

		//get the mesh for the module , to see if the camera is within it
		Asura_Mesh* mesh = Asura_Module_System::GetMeshForModule(BaseModuleIndex);
		
		
		float fPrintX = (float)Asura_Viewport::GetWidth()/2;
		float fPrintY = 0;
		Asura_Text_Output::ConvertPixelsToPrintCoords(fPrintX,fPrintY);
		fPrintY = y-1;

	
		
		if (mesh->BoundingBox.Contains(point))
		{
			Asura_Text_Output::Print(fPrintX,fPrintY,0xff00ff00,ASURA_TEXT_FORMAT_CENTREJUSTIFIED,"Inside Base Module");
		}
		else
		{
			Asura_Text_Output::Print(fPrintX,fPrintY,0xffff0000,ASURA_TEXT_FORMAT_CENTREJUSTIFIED,"Outside Base Module");
		}
	}
	y++;
	
	
	//show the name if the module(s) that the camera is currently in
	for (int m=0; m<Asura_Module_System::GetNumberOfModules(); m++)
	{
		if(Asura_Environment_System::GetEnvironment()->IsSceneryModule(m) || Asura_Environment_System::GetEnvironment()->IsNeverCameraModule(m))
		{
			//ignore scenery modules
			continue;
		}
		bool bExterior = Asura_Environment_System::GetEnvironment()->IsExteriorModule(m);


		Asura_Vector_3 location; 
		Asura_Module_System::GetPositionOfModule(m,location);
		Asura_Vector_3 point = *Asura_Camera::GetPosition() - location;

		//get the mesh for the module , to see if the camera is within it
		Asura_Mesh* mesh = Asura_Module_System::GetMeshForModule(m);
		
		if (mesh->BoundingBox.Contains(point))
		{
			Asura_Text_Output::Print(x,y++,0xffffffff,0,"In Module : %s%s",mesh->Name.GetString(),bExterior ? " (Ext)" : "");
		}
	}

	// Kevin Lea - 21/09/2001 17:36:19 - print the number of quads linked in from the camera's position
	// (draw this in red if the level designer is naughty)
	{
		int numberOfQuads = Asura_Module_System::GetNumberOfLinkedInPolys();
		int numberOfTris = Asura_Module_System::GetNumberOfLinkedInTriangles();

				
		unsigned int colour;
		
		if (numberOfQuads>TOOLS_RECOMMENDED_LINKED_IN_QUADS_LIMIT)
		{
			colour = 0xffff0000;
		}
		else
		{
			colour = 0xffffffff;
		}

		Asura_Text_Output::Print(x,y++,colour,0,"Polys Linked In: %d",numberOfQuads);
		Asura_Text_Output::Print(x,y++,colour,0,"Tris Linked In: %d",numberOfTris);
		
		y++;
	}



	Asura_Text_Output::Print(x,y++,0xffffffff,0,"Left click : Add link");
	Asura_Text_Output::Print(x,y++,0xffffffff,0,"Shift + Left click : Remove link");
	Asura_Text_Output::Print(x,y++,0xffffffff,0,"Ctrl+S : Save");
	y++;

	if(AddingLinks)
	{
		Asura_Text_Output::Print(x,y++,0xffffffff,0,"Adding Links");
	}

	//Show name of module under the cursor if required.
	if(!Asura_Interface::UsingMenusOrOverlays())
	{
		if(s_bShowCursorModuleName)
		{
			int iModuleIndex;
			Asura_Hash_ID uZoneHash;
		  	bool bOverModule = Tools_LOS::FindModuleUnderCursor(iModuleIndex,&uZoneHash);
			if(bOverModule)
			{
				#ifdef ASURA_USE_STREAMING
				Asura_Module_Handler* pxHandler = Asura_Module_System::GetHandler(uZoneHash);
				#else
				Asura_Module_Handler* pxHandler = &Asura_Module_System::GetCurrentHandler();;
				#endif
						
				//show module name next to cursor
				const Asura_Mesh* pxModuleMesh = pxHandler->GetMeshForModule( iModuleIndex );
				float x,y;
				Asura_Cursor::GetScreenCoords(x,y);
				x++;
				y--;
				Asura_Text_Output::Print( x, y++, 0xff00ff00, 0, "%s", pxModuleMesh->Name.GetString() );
			}
		}
	}
}

void Tools_Mode_ModuleLinking::QuickSave(void)
{
	Tools_SaveAsura::SaveModuleLinking( IsModified() );
}
bool Tools_Mode_ModuleLinking::IsModified()
{
	return Tools_SaveAsura::IsModuleLinkingModified();
}

void Tools_Mode_ModuleLinking::ToggleModuleLinkOverlay(void)
{
	ModuleLinkOverlayOpen = !ModuleLinkOverlayOpen;

	if (ModuleLinkOverlayOpen)
	{
		CreateModuleLinkOverlay();
	}
	else
	{
		Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_MODULELINKING);
	}
}
bool Tools_Mode_ModuleLinking::ModuleLinkOverlayIsOn(void)
{
	return ModuleLinkOverlayOpen;
}

void Tools_Mode_ModuleLinking::CreateModuleLinkOverlay(void)
{
	if(!ModuleLinkOverlayOpen) return;

	Asura_Overlay* newOverlay;
	newOverlay = new Asura_Overlay("Module Linking",false,&ModuleLinkOverlayPosition);
	
	Asura_Dialog* newDialog;
	Asura_Vector_2 offset = Asura_Vector_2(1.0f,3.0f);
			
	static char* s_apszLinkDirections[] =
	{
		"Normal",
		"Reversed",
		"Both Ways",
	};
	static int s_iJunkOffset = 0;

	//heading for link direction list box
	newDialog = new Asura_Dialog("Link Direction :",offset);
	newOverlay->AddDialog(newDialog);
	offset.y+=Asura_Dialog::SmallSpacing();

	//link direction list box
	newDialog = new Asura_List_Dialog(offset,s_apszLinkDirections,&s_iJunkOffset,&s_iLinkDirection,3,3,NULL);
	newDialog->SetHelpText(	"Normal: Module clicked on will be added to visibilty list of base module.\n"
							"Reversed : Base module will be made visible to clicked module.\n"
							"Both Ways : Base module and clicked module will be made visible to eachother.\n");
	newOverlay->AddDialog(newDialog);
	offset.y+=4.0f;

	
	newDialog = new Asura_Function_Dialog("Set Base From List",offset,SetBaseModuleFromList);
	newDialog->SetHelpText("Select the base module from a list of all modules in the level.");
	newOverlay->AddDialog(newDialog);
	offset.y+=Asura_Dialog::DefaultSpacing();

//	newDialog = new Asura_Tickbox_Dialog("Show Module Linking (F4)",offset,&(Asura_Module_System::UseModLinks));
//	newDialog->SetHelpText("Causes only the linked in modules to be drawn.\nIf unticked then all modules will be shown.");
//	newOverlay->AddDialog(newDialog);
//	offset.y+=Asura_Dialog::DefaultSpacing();

	newDialog = new Asura_Tickbox_Dialog("Draw Unlinked Modules (F5)",offset,&DrawUnlinkedModules);
	newDialog->SetHelpText("Forces unlinked modules to be drawn , but as green polygons.");
	newOverlay->AddDialog(newDialog);
	offset.y+=Asura_Dialog::DefaultSpacing();

	newDialog = new Asura_Tickbox_Dialog("Highlight Cursor Module",offset,&s_bHighlightCursorModule);
	newDialog->SetHelpText("Draws wireframe highlight for the module under the cursor");
	newOverlay->AddDialog(newDialog);
	offset.y+=Asura_Dialog::DefaultSpacing();

	newDialog = new Asura_Tickbox_Dialog("Cursor Module Name",offset,&s_bShowCursorModuleName);
	newDialog->SetHelpText("Displays the name of the module under the cursor next to it.");
	newOverlay->AddDialog(newDialog);
	offset.y+=Asura_Dialog::DefaultSpacing();

	newDialog = new Asura_Tickbox_Dialog("Show Base Module Extents",offset,&s_bShowBaseModuleExtents);
	newDialog->SetHelpText("Renders the bounding box of the base module");
	newOverlay->AddDialog(newDialog);
	offset.y+=Asura_Dialog::LargeSpacing();

//	newDialog = new Asura_Tickbox_Dialog("Use Dynamics (F9)",offset,&(Tools_System::DynamicsForCamera));
//	newDialog->SetHelpText("Use dynamics/collisions for moving the camera around.");
//	newOverlay->AddDialog(newDialog);
//	offset.y+=Asura_Dialog::DefaultSpacing();

	newDialog = new Asura_Tickbox_Dialog("Add Module Links (M)",offset,&(AddingLinks));
	newDialog->SetHelpText("Continously performs random LOS tests from the current position , linking in any modules that are hit.");
	newOverlay->AddDialog(newDialog);
	offset.y+=Asura_Dialog::DefaultSpacing();

	newDialog = new Asura_Tickbox_Dialog("Auto Set Base Module (N)",offset,&(AutoSetBaseModule));
	newDialog->SetHelpText("Automatically set the base module (edited module) to be whatever module the camera is in.");
	newOverlay->AddDialog(newDialog);
	offset.y+=Asura_Dialog::DefaultSpacing();

	newDialog = new Asura_Tickbox_Dialog("Select Module Under Camera",offset,&(s_bSelectModuleUnderCamera));
	newDialog->SetHelpText("Modified behaviour of Auto Set Base Module.\nModule is chosen by selecting the one beneath the camera , instead of checking the bounding box.");
	newDialog->m_iControlID = DialogID_SelectModuleUnderCamera;
	newOverlay->AddDialog(newDialog);
	offset.y+=Asura_Dialog::DefaultSpacing();

	newDialog = new Asura_Tickbox_Dialog("Ignore Transparent Polygons",offset,&(s_bIgnoreTransparentPolygons));
	newDialog->SetHelpText("Prevents transparent polygons from blocking LOS tests when adding module links.");
	newOverlay->AddDialog(newDialog);
	offset.y+=Asura_Dialog::LargeSpacing();

	newDialog = new Asura_Tickbox_Dialog("Include static objects",offset,&(s_bIncludeStaticObjects));
	newDialog->SetHelpText("Allows modules ot be linked in if any static object in that module is hit by an LOS test.");
	newOverlay->AddDialog(newDialog);
	offset.y+=Asura_Dialog::DefaultSpacing();
	
	newDialog = new Asura_Tickbox_Dialog("View static objects",offset,&(s_bViewStaticObjects));
	newDialog->SetHelpText("Unhides the static objects.");
	newOverlay->AddDialog(newDialog);
	offset.y+=Asura_Dialog::LargeSpacing();

	newDialog = new Asura_Numerical_Dialog("Max Link Range",offset,&s_fMaxLinkRange,0,9999.0f,1.0f,"%.0f");
	newDialog->SetHelpText("The maximum distance apart that modules should be linked.\nThis is enabled for module linking if the 'Use' checkbox is ticked.\nIt is also the distance used by the 'Clear Links By Range' button.");
	newOverlay->AddDialog(newDialog);
	offset.x = newDialog->m_xMaxCoord.x + 1.0f;

	newDialog = new Asura_Tickbox_Dialog("Use",offset,&s_bUseMaxLinkRange);
	newDialog->SetHelpText("Enables 'Max Link Range' for module linking.");
	newOverlay->AddDialog(newDialog);
	offset.x = 1.0f;
	offset.y+=Asura_Dialog::DefaultSpacing();

	newDialog = new Asura_Function_Dialog("Clear Links By Range",offset,ClearLinksByRange);
	newDialog->SetHelpText("Removes all module links for modules that are further apart than the distance specified by Max Link Range.");
	newOverlay->AddDialog(newDialog);
	offset.y+=Asura_Dialog::LargeSpacing();

	
	newDialog = new Asura_Function_Dialog("Show Linked Polygon Counts",offset,ShowModulePolygonCounts);
	newDialog->SetHelpText("Brings up a screen providing a summary of linked in polygon counts for all modules in the level..");
	newOverlay->AddDialog(newDialog);
	offset.y+=Asura_Dialog::DefaultSpacing();
	
	newDialog = new Asura_Function_Dialog("Clear Links",offset,ClearLinks);
	newDialog->SetHelpText("Removes all module linking from the current base module.");
	newOverlay->AddDialog(newDialog);
	offset.y+=Asura_Dialog::DefaultSpacing();
	
	newDialog = new Asura_Function_Dialog("Clear All Links",offset,ClearAllLinks);
	newDialog->SetHelpText("Removes all module linking from all modules.");
	newOverlay->AddDialog(newDialog);
	offset.y+=Asura_Dialog::DefaultSpacing();

	newDialog = new Asura_Function_Dialog("Auto-Link",offset,AutoModLink);
	newDialog->SetHelpText("Automatically calculates module links by performing random LOS tests from the waypoints.");
	newOverlay->AddDialog(newDialog);
	offset.y+=Asura_Dialog::DefaultSpacing();

	
	newOverlay->m_uID = TOOLS_OVERLAY_MODULELINKING;

	Asura_Overlay_System::SubmitNewOverlay(newOverlay,false);
}

void Tools_Mode_ModuleLinking::HandleKeyboard()
{
	if(Asura_Overlay_System::OverlaysHaveKeyboardFocus()) return;
	if(Asura_UserInput::KeyPressed(ASURA_KEYID_ALT)) return;
 	if(Asura_UserInput::KeyPressed(ASURA_KEYID_CTRL))
 	{
	}
	else if(Asura_UserInput::KeyPressed(ASURA_KEYID_SHIFT))
	{
	}
	else
	{
		if (Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_F5))
		{
			DrawUnlinkedModules = !DrawUnlinkedModules;
		}
		if (Asura_UserInput::DebouncedKeyPressed('m'))
		{
			AddingLinks = !AddingLinks;
		}
		if (Asura_UserInput::DebouncedKeyPressed('n'))
		{
			AutoSetBaseModule = !AutoSetBaseModule;
		}
		if (Asura_UserInput::DebouncedKeyPressed('b'))
		{
			SetBaseModule();
		}
	}
}

bool Tools_Mode_ModuleLinking::TestAgainstStaticObjects(const Asura_Vector_3& xAlpha, const Asura_Vector_3& xBeta, int &iModuleIndex, const float fDistance)
{
	Asura_Assert(s_bIncludeStaticObjects, "");
	float fLowest = fDistance;
	// Set a maximum for objects here!
	if (fLowest > 100.0f)
		fLowest = 100.0f;

	const Tools_Object* pxObject = NULL;
	for( CLIF<Tools_Object*> olif(&Tools_Object_System::ObjectList);!olif.done();olif.next() )
	{
//		if(olif()->IsHidden()) continue;

		if(olif()->UseObjectInModuleLinking()) 
			continue;

		if(olif()->Type != TOT_Entity_PhysicsObject)
			continue;

        const Tools_Entity_PhysicsObject* pxPhysicsObj = Tools_Entity_PhysicsObject::FindEntity( olif()->GetGuid() );
        if( !pxPhysicsObj || !pxPhysicsObj->IsStatic() ) { continue; }

		if(olif()->ObjectIsInLOS(xAlpha,xBeta,fLowest))
		{
			pxObject = olif();
		}
	}

	bool bRet = false;
	if (pxObject)
	{
		Asura_Vector_3 xPos = xAlpha + (xBeta * fLowest);
		iModuleIndex = Asura_Module_System::GetModuleIndexFromPosition(xPos);
		if (iModuleIndex != -1)
			bRet = true;
	}
	return bRet;
}


void Tools_Mode_ModuleLinking::AddModuleLinks()
{
	
	if(!AutoSetBaseModule)
	{
		Tools_Timed_Message_System::AddMessage(0xff0000,"'Add Module Links' can only be used if 'Auto Set Base Module' is on");
		AddingLinks = false;
		return;
		
	}
	if(BaseModuleIndex==-1) return;
	
	int vm = -1;
	Asura_Hash_ID uZoneHash;
	#ifdef ASURA_USE_STREAMING
	const Asura_Hash_ID uMainZoneHash = Tools_Streaming_System::GetStreamingZoneForThisLevel()->GetHashID();
	#else
	const Asura_Hash_ID uMainZoneHash = 0;
	#endif
	float fDistance = ASURA_LOS_MAX_DISTANCE;
	
	const Asura_Vector_3* cameraPosition = Asura_Camera::GetPosition();
	const Asura_Matrix_3x3* cameraOrientation = Asura_Camera::GetOrientation();

	Tools_LOS::SetIgnoreModuleLinking(true);

	//try straight in front of the camera
	Asura_Vector_3 beta = Asura_Vector_3(cameraOrientation->e13,cameraOrientation->e23,cameraOrientation->e33);
	Asura_Vector_3 alpha = *cameraPosition;
	int iJunk;
	if(Tools_LOS::FindPolyInLOS(alpha, beta,iJunk,vm,fDistance,false,&uZoneHash))
	{
		SetModuleLink(vm,uZoneHash,true,s_iLinkDirection);
	}
	if (s_bIncludeStaticObjects &&
		TestAgainstStaticObjects(alpha,beta,vm,fDistance))
	{
		SetModuleLink(vm,uMainZoneHash,true,s_iLinkDirection);
	}
	fDistance = ASURA_LOS_MAX_DISTANCE;

	//try under the cursor
	Asura_Cursor::GetLOSDirection(beta);
	beta = beta * (*cameraOrientation);
	if(Tools_LOS::FindPolyInLOS(alpha, beta, iJunk, vm, fDistance, false,&uZoneHash))
	{
		SetModuleLink(vm,uZoneHash,true,s_iLinkDirection);
	}
	if (s_bIncludeStaticObjects &&
		TestAgainstStaticObjects(alpha,beta,vm,fDistance))
	{
		SetModuleLink(vm,uMainZoneHash,true,s_iLinkDirection);
	}
	fDistance = ASURA_LOS_MAX_DISTANCE;

	//try a random direction
	beta.x = Asura_Random::GetFloat()*2.0f - 1.0f;
	beta.y = 0.0f;
	beta.z = Asura_Random::GetFloat()*2.0f - 1.0f;
	beta.Normalise();
	if(Tools_LOS::FindPolyInLOS(alpha, beta, iJunk, vm, fDistance, false,&uZoneHash))
	{
		SetModuleLink(vm,uZoneHash,true,s_iLinkDirection);
	}
	if (s_bIncludeStaticObjects &&
		TestAgainstStaticObjects(alpha,beta,vm,fDistance))
	{
		SetModuleLink(vm,uMainZoneHash,true,s_iLinkDirection);
	}
	fDistance = ASURA_LOS_MAX_DISTANCE;
	

	
	//try a number of other point on the screen
	static int counter = 0;
	for(int i=0;i<50;i++)
	{
		int num = (i + counter)*613;
		unsigned x = (num%400);
		unsigned y = i*8+((num/400)%8);

		beta.x = (x/400.0f)-0.5f;
		beta.y = ((y/400.0f)-0.5f);
		beta.z = 0.5f;
		beta.Normalise();
		beta = beta * (*cameraOrientation);

		if(Tools_LOS::FindPolyInLOS(alpha, beta, iJunk, vm, fDistance, false,&uZoneHash))
		{
			SetModuleLink(vm,uZoneHash,true,s_iLinkDirection);
		}
		if (s_bIncludeStaticObjects &&
			TestAgainstStaticObjects(alpha,beta,vm,fDistance))
		{
			SetModuleLink(vm,uMainZoneHash,true,s_iLinkDirection);
		}
		fDistance = ASURA_LOS_MAX_DISTANCE;
	}
	counter++;

	Tools_LOS::SetIgnoreModuleLinking(false);

	Tools_SaveAsura::SetModuleLinkingModified();
}


float Tools_Mode_ModuleLinking::GetDistanceBetweenModules(int iModule1,int iModule2,Asura_Hash_ID uZoneHash2)
{
	//get each modules bounding box
	Asura_Vector_3 xPos1 = Asura_Module_System::GetPositionOfModule(iModule1);
	Asura_Mesh* pxMesh1 = Asura_Module_System::GetMeshForModule(iModule1);
	if(!pxMesh1->BoundingBoxDataValid) pxMesh1->CalculateBoundingBox();
	Asura_Bounding_Box xBox1 = pxMesh1->BoundingBox;
	xBox1.Displace(xPos1);

	#ifdef ASURA_USE_STREAMING
	Asura_Module_Handler* pxHandler = Asura_Module_System::GetHandler(uZoneHash2);
	if(!pxHandler)
	{
		Asura_Assert(0,"Failed to find module system handler for second module");
		return 0.0f;
	}
	#else
	Asura_Module_Handler* pxHandler = &Asura_Module_System::GetCurrentHandler();
	#endif
	
	Asura_Vector_3 xPos2 = pxHandler->GetPositionOfModule(iModule2);
	Asura_Mesh* pxMesh2 = pxHandler->GetMeshForModule(iModule2);
	if(!pxMesh2->BoundingBoxDataValid) pxMesh2->CalculateBoundingBox();
	Asura_Bounding_Box xBox2 = pxMesh2->BoundingBox;
	xBox2.Displace(xPos2);

	Asura_Vector_3 xOffset;

	//get the distance between the boxes
	xOffset.x = Asura_Maths::Max(xBox1.MinX,xBox2.MinX) - Asura_Maths::Min(xBox1.MaxX,xBox2.MaxX);
	if(xOffset.x<0) xOffset.x = 0;

	xOffset.y = Asura_Maths::Max(xBox1.MinY,xBox2.MinY) - Asura_Maths::Min(xBox1.MaxY,xBox2.MaxY);
	if(xOffset.y<0) xOffset.y = 0;
	
	xOffset.z = Asura_Maths::Max(xBox1.MinZ,xBox2.MinZ) - Asura_Maths::Min(xBox1.MaxZ,xBox2.MaxZ);
	if(xOffset.z<0) xOffset.z = 0;

	return xOffset.Magnitude();
}

void Tools_Mode_ModuleLinking::SetModuleLink(int iLinkModuleIndex,Asura_Hash_ID uLinkZoneHash,bool bLink,int iLinkDirection /*= TOOLS_MODULELINK_NORMAL*/)
{
	if(BaseModuleIndex==-1)
	{
		return;
	}

	if(bLink && s_bUseMaxLinkRange)
	{
		//make sure the module is in range
		if(GetDistanceBetweenModules(BaseModuleIndex,iLinkModuleIndex,uLinkZoneHash)>s_fMaxLinkRange) return;
	}
	
	Asura_Environment* pxEnv = Asura_Environment_System::GetEnvironment();
	
	if(iLinkDirection == TOOLS_MODULELINK_NORMAL || iLinkDirection == TOOLS_MODULELINK_2WAY)
	{
		//don't change link if this is attempting to add a link to a scenery module
		if(bLink==false || !(pxEnv->IsSceneryModule(BaseModuleIndex) || pxEnv->IsNeverCameraModule(BaseModuleIndex)))
		{
			Asura_Module_System::SetModuleLink(BaseModuleIndex,iLinkModuleIndex,uLinkZoneHash,bLink);
		}
	}

	#ifdef ASURA_USE_STREAMING
	//reverse links can only be setup within the same zone
	if(uLinkZoneHash == Tools_Streaming_System::GetHashForThisLevel())
	#endif
	{
		if(iLinkDirection == TOOLS_MODULELINK_REVERSED || iLinkDirection == TOOLS_MODULELINK_2WAY)
		{
			//don't change link if this is attempting to add a link to a scenery module
			if(bLink==false || !(pxEnv->IsSceneryModule(iLinkModuleIndex) || pxEnv->IsNeverCameraModule(iLinkModuleIndex)))
			{
				Asura_Module_System::SetModuleLink(iLinkModuleIndex,BaseModuleIndex,uLinkZoneHash,bLink);
			}
		}
	}
		
	Tools_SaveAsura::SetModuleLinkingModified();
}

void Tools_Mode_ModuleLinking::ClearLinksByRange()
{
	if(!ShowYesNoOverlay("Remove all module links for modules more than %d metres apart?",(int)s_fMaxLinkRange))
	{
		return;
	}

	for(Tools_Streaming_Iterator xIt(false);!xIt.Done();xIt.Next())
	{
		Asura_Hash_ID uLinkZoneHash = xIt.GetHash();
		#ifdef ASURA_USE_STREAMING
		Asura_Module_Handler* pxLinkHandler = Asura_Module_System::GetHandler(uLinkZoneHash);
		if(!pxLinkHandler) continue;
		#else
		Asura_Module_Handler* pxLinkHandler = &Asura_Module_System::GetCurrentHandler();
		#endif

		for(int i=0;i<Asura_Module_System::GetNumberOfModules();i++)
		{

			for(int j=0;j<pxLinkHandler->GetNumberOfModules();j++)
			{
				if(!Asura_Module_System::IsModuleLinked(i,j,uLinkZoneHash)) continue;
				if(GetDistanceBetweenModules(i,j,uLinkZoneHash)<=s_fMaxLinkRange) continue;

				//this link is too far , so remove it
				Asura_Module_System::SetModuleLink(i,j,uLinkZoneHash,false);
			}
		}
	}
}

void Tools_Mode_ModuleLinking::ClearLinks()
{
	if(BaseModuleIndex==-1)
	{
		return;
	}
	
	Asura_Module_System::ClearModuleLinks(BaseModuleIndex);
		
	Tools_SaveAsura::SetModuleLinkingModified();
}

void Tools_Mode_ModuleLinking::ClearAllLinks()
{
	YesNo_Overlay xDlg("This will remove all module linking data. Continue?");
	if(!xDlg.DoModal()) return;
	for (int vm=0; vm<Asura_Module_System::GetNumberOfModules(); vm++)
	{
		Asura_Module_System::ClearModuleLinks(vm);
	}
	Tools_SaveAsura::SetModuleLinkingModified();
}



static enum
{
	MODULESORT_NAME_UP,
	MODULESORT_NAME_DOWN,
	MODULESORT_POLY_UP,
	MODULESORT_POLY_DOWN,
	MODULESORT_TRIANGLE_UP,
	MODULESORT_TRIANGLE_DOWN,
	MODULESORT_MODULE_UP,
	MODULESORT_MODULE_DOWN,
} g_eModuleSortMode = MODULESORT_POLY_DOWN;

struct ToolsModuleInfo
{
	int m_iIndex;
	Asura_Mesh* m_pxMesh;
	int m_iNumLinkedPolygons;
	int m_iNumLinkedTriangles;
	int m_iNumLinkedModules;
};

int ModuleLinkSort(const void* m1,const void* m2)
{
	ToolsModuleInfo* pxModule1 = (ToolsModuleInfo*)m1;
	ToolsModuleInfo* pxModule2 = (ToolsModuleInfo*)m2;

	switch(g_eModuleSortMode)
	{
		case MODULESORT_NAME_UP:
			return pxModule1->m_pxMesh->Name.Compare(pxModule2->m_pxMesh->Name);
			break;
		case MODULESORT_NAME_DOWN:
			return -pxModule1->m_pxMesh->Name.Compare(pxModule2->m_pxMesh->Name);
			break;
		case MODULESORT_POLY_UP:
			return pxModule1->m_iNumLinkedPolygons - pxModule2->m_iNumLinkedPolygons;
			break;
		case MODULESORT_POLY_DOWN:
			return pxModule2->m_iNumLinkedPolygons - pxModule1->m_iNumLinkedPolygons;
			break;
		case MODULESORT_TRIANGLE_UP:
			return pxModule1->m_iNumLinkedTriangles - pxModule2->m_iNumLinkedTriangles;
			break;
		case MODULESORT_TRIANGLE_DOWN:
			return pxModule2->m_iNumLinkedTriangles - pxModule1->m_iNumLinkedTriangles;
			break;
		case MODULESORT_MODULE_UP:
			return pxModule1->m_iNumLinkedModules - pxModule2->m_iNumLinkedModules;
			break;
		case MODULESORT_MODULE_DOWN:
			return pxModule2->m_iNumLinkedModules - pxModule1->m_iNumLinkedModules;
			break;
	}

	return 0;
}

void Tools_Mode_ModuleLinking::ShowModulePolygonCounts()
{
	int FirstShownModule = 0;
	int MouseOverIndex = -1;
	
	
	float NameOffsetX = 10;
	float PolygonOffsetX = 25;
	float TriangleOffsetX = 35;
	float ModuleOffsetX = 45;
	float SecondColumnX = 50;

	//work out how many columns and rows we can display
	//get the screen size , and convert to print coordinates
	float print_width = (float)Asura_Viewport::GetWidth();
	float print_height = (float)Asura_Viewport::GetHeight();

	Asura_Text_Output::ConvertPixelsToPrintCoords(print_width,print_height);

	int NumColumns = (int)(print_width/SecondColumnX);
	int NumModulesPerColumn = (int)(print_height - 10);

	
	int iNumModules = Asura_Module_System::GetNumberOfModules();

	//build an array of module iinfo , to be sorted by name or poly count
	ToolsModuleInfo* pxModuleInfo = new ToolsModuleInfo[iNumModules];
	for(int i=0;i<iNumModules;i++)
	{
		pxModuleInfo[i].m_iIndex = i;
		pxModuleInfo[i].m_pxMesh = Asura_Module_System::GetMeshForModule(i);

		CountLinkedPolygonsAndModules(i,pxModuleInfo[i].m_iNumLinkedPolygons,pxModuleInfo[i].m_iNumLinkedTriangles,pxModuleInfo[i].m_iNumLinkedModules);

	}
	
	qsort(pxModuleInfo,iNumModules,sizeof(ToolsModuleInfo),ModuleLinkSort);

	
	do
	{
		Asura_Windows::CheckForWindowsMessages();
		Asura_UserInput::Update();
		Asura_Cursor::OncePerFrameUpdate();

		//get the current mouse position
		int mx,my;
		Asura_Cursor::GetScreenSpaceCoords(mx,my);
		float fPrintMX = (float)mx;
		float fPrintMY = (float)my;
		Asura_Text_Output::ConvertPixelsToPrintCoords(fPrintMX,fPrintMY);

		//determine which module entry the mouse is over
		MouseOverIndex = -1;
		if(fPrintMY>=1 && fPrintMY<1+NumModulesPerColumn)
		{
			MouseOverIndex = (int)(fPrintMY-1);
			if(fPrintMX>=SecondColumnX) MouseOverIndex+=NumModulesPerColumn;
		}

		//check user input
		if(Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_MOUSEBUTTON1))
		{
			if(MouseOverIndex>=0)
			{
				//goto the module the mouse is over
				int iIndex = MouseOverIndex + FirstShownModule;
				if(iIndex<Asura_Module_System::GetNumberOfModules())
				{
					int iModule = pxModuleInfo[iIndex].m_iIndex;
					
					//get the position of the module
					Asura_Vector_3 position;
					Asura_Module_System::GetPositionOfModule(iModule,position);

					//and put the player there
					//extern Asura_Dynamic_Object* PlayerDynamicObject;
					//PlayerDynamicObject->SetPosition(position);
					Asura_Camera::SetPosition(position);
					break;
				}
			}
			else if(fPrintMY < 1.0f)
			{
				//determine column clicked on
				float fX = (float)((int)fPrintMX % (int) SecondColumnX);
				
					
				if(fX>NameOffsetX-4.0f && fX<NameOffsetX+4.0f)
				{
					//clicked on name column
					g_eModuleSortMode = (g_eModuleSortMode==MODULESORT_NAME_UP) ? MODULESORT_NAME_DOWN : MODULESORT_NAME_UP;
				}
				else if(fX>PolygonOffsetX-4.0f && fX<PolygonOffsetX+4.0f)
				{
					//clicked on poly column
					g_eModuleSortMode = (g_eModuleSortMode==MODULESORT_POLY_DOWN) ? MODULESORT_POLY_UP : MODULESORT_POLY_DOWN;
				}
				else if(fX>TriangleOffsetX-4.0f && fX<TriangleOffsetX+4.0f)
				{
					//clicked on triangle column
					g_eModuleSortMode = (g_eModuleSortMode==MODULESORT_TRIANGLE_DOWN) ? MODULESORT_TRIANGLE_UP : MODULESORT_TRIANGLE_DOWN;
				}
				else if(fX>ModuleOffsetX-4.0f && fX<ModuleOffsetX+4.0f)
				{
					//clicked on module column
					g_eModuleSortMode = (g_eModuleSortMode==MODULESORT_MODULE_DOWN) ? MODULESORT_MODULE_UP : MODULESORT_MODULE_DOWN;
				}
				
				qsort(pxModuleInfo,iNumModules,sizeof(ToolsModuleInfo),ModuleLinkSort);
			}
		}

		if(Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_LEFT))
		{
			//show previous modules
			FirstShownModule -= NumModulesPerColumn*NumColumns;
			if(FirstShownModule<0)
			{
				FirstShownModule = ((Asura_Module_System::GetNumberOfModules()-1)/(NumModulesPerColumn*NumColumns))*(NumModulesPerColumn*NumColumns);
			}
		}
		
		if(Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_RIGHT))
		{
			//show next modules
			FirstShownModule += NumModulesPerColumn*NumColumns;
			if(FirstShownModule>Asura_Module_System::GetNumberOfModules())
			{
				FirstShownModule = 0;
			}
		}
		
		if(Asura_UserInput::DebouncedKeyPressed('f'))
		{
			WriteModuleLinkTextFile();
			break;
		}


		//begin rendering
		Asura_Render::BeginFrame();

		//first draw a big black poly over everything that has taken place so far...
		float width = (float)Asura_Viewport::GetWidth();
		float height = (float)Asura_Viewport::GetHeight();
		Asura_Render::Overlay.SetRectXY(0,0,width,height);
		Asura_Render::Overlay.SetRectUV(0,0,1,1);
		Asura_Render::Overlay.TranslucencyMode = ASURA_TRANSLUCENCY_OFF;
		Asura_Render::Overlay.TextureIndex = ASURA_TEXTURE_NONE;
		Asura_Render::Overlay.Colour=0xff000000;
		Asura_Render::Overlay.Render();
		Asura_Render::FlushDisplay();

		
		//display the column headings
		int i;
		for(i=0;i<NumColumns;i++)
		{
			float x = (float)(SecondColumnX*i);
			Asura_Text_Output::Print(x+PolygonOffsetX,0,0xff00ff00,ASURA_TEXT_FORMAT_CENTREJUSTIFIED,"Polygons");
			Asura_Text_Output::Print(x+TriangleOffsetX,0,0xff00ff00,ASURA_TEXT_FORMAT_CENTREJUSTIFIED,"Triangles");
			Asura_Text_Output::Print(x+ModuleOffsetX,0,0xff00ff00,ASURA_TEXT_FORMAT_CENTREJUSTIFIED,"Modules");
			Asura_Text_Output::Print(x+NameOffsetX,0,0xff00ff00,ASURA_TEXT_FORMAT_CENTREJUSTIFIED,"Name");
		}

		
		//display the polygon counts
		for(i=0;i<NumModulesPerColumn*NumColumns;i++)
		{
			int iIndex = i+FirstShownModule;
			if(iIndex>=Asura_Module_System::GetNumberOfModules()) break;

			const ToolsModuleInfo* pxModule = &pxModuleInfo[iIndex];
			
			float x = (i/NumModulesPerColumn)*SecondColumnX;
			float y = (float)(1+i%NumModulesPerColumn);
			int colour = (i == MouseOverIndex) ? 0xffff0000 : 0xffffffff;

			
			Asura_Text_Output::Print(x+NameOffsetX,y,colour,ASURA_TEXT_FORMAT_CENTREJUSTIFIED,pxModule->m_pxMesh->Name.GetString());
			if(Asura_Environment_System::GetEnvironment()->IsSceneryModule(pxModule->m_iIndex))
			{
				//scenery modules don't need linked in counts
				Asura_Text_Output::Print(x+TriangleOffsetX,y,colour,ASURA_TEXT_FORMAT_CENTREJUSTIFIED,"(Scenery)");
			}
			else if(Asura_Environment_System::GetEnvironment()->IsNeverCameraModule(pxModule->m_iIndex))
			{
				//nor do ones flagged as 'never camera module'
				Asura_Text_Output::Print(x+TriangleOffsetX,y,colour,ASURA_TEXT_FORMAT_CENTREJUSTIFIED,"(Never Camera Module)");
			}
			else
			{
				Asura_Text_Output::Print(x+PolygonOffsetX,y,colour,ASURA_TEXT_FORMAT_CENTREJUSTIFIED,"%d",pxModule->m_iNumLinkedPolygons);
				Asura_Text_Output::Print(x+TriangleOffsetX,y,colour,ASURA_TEXT_FORMAT_CENTREJUSTIFIED,"%d",pxModule->m_iNumLinkedTriangles);
				Asura_Text_Output::Print(x+ModuleOffsetX,y,colour,ASURA_TEXT_FORMAT_CENTREJUSTIFIED,"%d",pxModule->m_iNumLinkedModules);
			}
		}

		//show some instructions
		Asura_Text_Output::Print(1,print_height-7,0xffffffff,0,"Click on module to go to it");
		Asura_Text_Output::Print(1,print_height-6,0xffffffff,0,"Click on heading to change sorting");
		Asura_Text_Output::Print(1,print_height-5,0xffffffff,0,"Escape : Exit");
		Asura_Text_Output::Print(1,print_height-4,0xffffffff,0,"Left Arrow : Show previous modules");
		Asura_Text_Output::Print(1,print_height-3,0xffffffff,0,"Right Arrow : Show next modules");
		Asura_Text_Output::Print(1,print_height-2,0xffffffff,0,"F : Output to text file");

		//render cursor and finish
		Asura_Cursor::SetType(ASURA_CURSOR_CROSSHAIR);
		Asura_Cursor::Render();
		Asura_Render::EndFrame();
  
	}while(!Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_ESCAPE));

	delete [] pxModuleInfo;
}

struct ModuleLinkCount
{
	int LinkedModules;
	int LinkedPolygons;
	int LinkedTriangles;
	const char* Name;
};

int ModuleLinkCountSort(const void* m1,const void* m2)
{
	return ((ModuleLinkCount*)m2)->LinkedTriangles-((ModuleLinkCount*)m1)->LinkedTriangles;
};

void Tools_Mode_ModuleLinking::WriteModuleLinkTextFile()
{
	//build the filename for the file
	extern Asura_String EnvironmentFilename;
	char filename[MAX_PATH];
	strcpy(filename,EnvironmentFilename.GetString());
	char* dotpos = strrchr(filename,'.');
	Asura_Assert(dotpos,"");
	strcpy(dotpos,"_ModuleLinkCounts.txt");

	
	FILE* file = fopen(filename,"wt");
	if(!file)
	{
		Tools_Timed_Message_System::AddMessage(0xff0000,"Failed to write %s",filename);
		return;
	}


	ModuleLinkCount* linkCountArray = new ModuleLinkCount[Asura_Module_System::GetNumberOfModules()];
	
	for(int i=0;i<Asura_Module_System::GetNumberOfModules();i++)
	{
		//count the linked in polygons and modules
		linkCountArray[i].LinkedModules = 0;
		linkCountArray[i].LinkedPolygons = 0;
		linkCountArray[i].LinkedTriangles = 0;

		CountLinkedPolygonsAndModules(i,linkCountArray[i].LinkedPolygons,linkCountArray[i].LinkedTriangles,linkCountArray[i].LinkedModules);

		//get the mesh for the name
		Asura_Mesh* mesh = Asura_Module_System::GetMeshForModule(i);
		linkCountArray[i].Name = mesh->Name.GetString();
	}

	//sort the array
	qsort(linkCountArray,Asura_Module_System::GetNumberOfModules(),sizeof(ModuleLinkCount),ModuleLinkCountSort);

	//write the file
	fprintf(file,"%-20s %10s %10s %10s\n\n","Name","Polygons","Triangles","Modules");

	for(int i=0;i<Asura_Module_System::GetNumberOfModules();i++)
	{
		fprintf(file,"%-20s %10d %10d %10d\n",linkCountArray[i].Name,linkCountArray[i].LinkedPolygons,linkCountArray[i].LinkedTriangles,linkCountArray[i].LinkedModules);
	}
  	fclose(file);

	delete [] linkCountArray;

	Tools_Timed_Message_System::AddMessage(0x00ff00,"%s written",filename);


}

void Tools_Mode_ModuleLinking::CountLinkedPolygonsAndModules(int moduleIndex,int& polyCount,int& triangleCount,int& moduleCount)
{
	polyCount = 0;
	moduleCount = 0;
	triangleCount = 0;

	if(Asura_Environment_System::GetEnvironment()->IsSceneryModule(moduleIndex) || Asura_Environment_System::GetEnvironment()->IsNeverCameraModule(moduleIndex))
	{
		//don't bother counting linked polygons for scenery modules
		return;
	}

	for(Tools_Streaming_Iterator xIt(false);!xIt.Done();xIt.Next())
	{
		Asura_Hash_ID uLinkZoneHash = xIt.GetHash();
		#ifdef ASURA_USE_STREAMING
		Asura_Module_Handler* pxLinkHandler = Asura_Module_System::GetHandler(uLinkZoneHash);
		if(!pxLinkHandler) continue;
		#else
		Asura_Module_Handler* pxLinkHandler = &Asura_Module_System::GetCurrentHandler();
		#endif
	
		for(int i=0;i<pxLinkHandler->GetNumberOfModules();i++)
		{
			if(Asura_Module_System::IsModuleLinked(moduleIndex,i,uLinkZoneHash))
			{
				//add on this module to the count
				moduleCount++;
				polyCount += pxLinkHandler->GetMeshForModule(i)->NumberOfPolygons;
				triangleCount += pxLinkHandler->GetMeshForModule(i)->NumberOfTriangles;
			}
		}
	}
}


void Tools_Mode_ModuleLinking::SetBaseModule()
{
	//find the module under the cursor
	int iModuleIndex = -1;
	Tools_LOS::FindModuleUnderCursor(iModuleIndex);

	SetBaseModule(iModuleIndex);
}

void Tools_Mode_ModuleLinking::SetBaseModuleFromList()
{
	Tools_ModuleSelect_Overlay xOverlay("Set Base Module",BaseModuleIndex);
	if(xOverlay.DoModal())
	{
		SetBaseModule(xOverlay.GetSelectionData());
	}
}

void Tools_Mode_ModuleLinking::SetBaseModule(int iModuleIndex)
{
	//switch off automatic updating of base module
	AutoSetBaseModule = false;

	bool bModuleSet = false;
	BaseModuleIndex = iModuleIndex;
	if(BaseModuleIndex!=-1)
	{
		if(Asura_Environment_System::GetEnvironment()->IsSceneryModule(BaseModuleIndex))
		{
			Asura_Mesh* pxMesh = Asura_Module_System::GetMeshForModule(BaseModuleIndex);
			Tools_Timed_Message_System::AddMessage(0xff0000,"%s is a scenery module",pxMesh->Name.GetString());
		}
		else if(Asura_Environment_System::GetEnvironment()->IsNeverCameraModule(BaseModuleIndex))
		{
			Asura_Mesh* pxMesh = Asura_Module_System::GetMeshForModule(BaseModuleIndex);
			Tools_Timed_Message_System::AddMessage(0xff0000,"%s is flagged as 'never camera module'",pxMesh->Name.GetString());
		}
		else
		{
 			Tools_Render::SetModuleForModuleLinking(BaseModuleIndex);
			bModuleSet = true;
		}
	}
	
	if(!bModuleSet)
	{
		BaseModuleIndex = -1;
		Tools_Render::UseCameraForModuleLinking();
	}	
}

void Tools_Mode_ModuleLinking::PreSave( void )
{
    if( IsModified() )
    {
        Tools_Mode_Regions::DealWithModifiedModuleLinks();
    }
}


void Tools_Mode_ModuleLinking::UpdateStaticObjects()
{
	//deal with existence of static objects
	bool bNeedStaticObjects = s_bIncludeStaticObjects || s_bViewStaticObjects;

	if(bNeedStaticObjects != s_bStaticObjectsCreated)
	{
		if(bNeedStaticObjects)
		{
			//create all the objects  , and start them off as invisible
			Tools_Entity_Object::CreateAllObjects();
			for(LIF<Tools_Object*> xObjectLIF(&Tools_Object_System::ObjectList);!xObjectLIF.done();xObjectLIF.next())
			{
				xObjectLIF()->HiddenFlags |= ObjectHideFlag_Mode;
			}
			s_bStaticObjectsVisible = false;
		}
		else
		{
			Tools_Object_System::DeleteAllModeSpecific();
		}
		s_bStaticObjectsCreated = bNeedStaticObjects;
	}
	
	if(!s_bStaticObjectsCreated) return;

	//now update visibility
	if(s_bViewStaticObjects != s_bStaticObjectsVisible)
	{
		for(LIF<Tools_Object*> xObjectLIF(&Tools_Object_System::ObjectList);!xObjectLIF.done();xObjectLIF.next())
		{
			if(xObjectLIF()->Type == TOT_Entity_PhysicsObject)
			{
				if(s_bViewStaticObjects)
				{
					xObjectLIF()->HiddenFlags &=~ ObjectHideFlag_Mode;
				}
				else
				{
					xObjectLIF()->HiddenFlags |= ObjectHideFlag_Mode;
				}
			}
		}
		s_bStaticObjectsVisible = s_bViewStaticObjects;		
	}
}




void Tools_Mode_ModuleLinking::AutoModLink()
{
	//count up the number of waypoint links to determine how to split the time
	int iTotalLinks = 0;
	for(int i=0;i<Asura_AI_Navigation_System::GetNumberOfWaypoints();i++)
	{
		iTotalLinks += Asura_AI_Navigation_System::GetWaypoint(i)->m_ucNumConnections;
	}
	if(!iTotalLinks)
	{
		ShowMessageOverlay("Waypoints not generated yet.");
		return;
	}

	//ask how long to take
	GetNumber_Overlay xOverlay("Take how long module linking? (in minutes)",1.0f,0.1f,1000.0f,"%.1f");
	if(!xOverlay.DoModal()) return;
	float fDesiredTime = xOverlay.GetNumber()*60.0f;

	int iDesiredMin = Asura_Maths::FloatToInt_Chop(fDesiredTime)/60;
	int iDesiredSec = Asura_Maths::FloatToInt_Chop(fDesiredTime)%60;

	Asura_Timers::SetMaxFrameRate(0.0f);
	Asura_Timers::HandlePrimaryTimer();
	
	float fTimePerLink = fDesiredTime / (float) iTotalLinks;
	float fTimeForNextLink = 0.0f;
	float fTotalTime = 0.0f;
	float fFrameStartTime = 0.0f;
	float fStartTime = Asura_Timers::GetUncappedPhase();
	
	int iLinked = 0;
		
	Tools_LOS::SetIgnoreModuleLinking(true);

	for(int i=0;i<Asura_AI_Navigation_System::GetNumberOfWaypoints();i++)
	{
		Asura_AI_NavVertex* pxWaypoint = Asura_AI_Navigation_System::GetWaypoint(i);
		
		float fTimeForNextWP = fTimeForNextLink + fTimePerLink*pxWaypoint->m_ucNumConnections;
		if(fTotalTime > fTimeForNextWP) continue;
		
		//get position for tests
		Asura_Vector_3 xPosition = pxWaypoint->m_xPosition;
		xPosition.y -= 1.8f;

		//determine module for waypoint
		{
			//Do a line of sight test down to find the module , and show the module linking from there
			Asura_Vector_3 xDirection(0,1,0);
			int iModuleIndex;
			int iJunk;
			float fJunk;
			if(Tools_LOS::FindPolyInLOS(xPosition,xDirection,iJunk,iModuleIndex,fJunk,false) &&
			   !Asura_Environment_System::GetEnvironment()->IsSceneryModule(iModuleIndex) &&
			   !Asura_Environment_System::GetEnvironment()->IsNeverCameraModule(iModuleIndex))

			{
				BaseModuleIndex = iModuleIndex;
				
			}
			else
			{
				BaseModuleIndex = Asura_Module_System::GetCameraModuleIndexFromPosition(xPosition,-1);
			
			}
		}
		if(BaseModuleIndex==-1)
		{
			fTimeForNextLink = fTimeForNextWP;
			continue;
		}
		
		bool bQuit = false;
		
		for(int j=0;j<pxWaypoint->m_ucNumConnections && !bQuit;j++)
		{
			//add on time allowed for this link
			fTimeForNextLink += fTimePerLink;
			if(fTotalTime>fTimeForNextLink) continue;
			
		
			//get normal along connection
			Asura_AI_NavVertex* pxWaypoint2 = Asura_AI_Navigation_System::GetWaypoint(pxWaypoint->m_pxConnections[j].m_usIndex);
			Asura_Vector_3 xNormal = pxWaypoint2->m_xPosition-pxWaypoint->m_xPosition;
			xNormal.Normalise();

			Asura_Matrix_3x3 xMat;
			Asura_Maths::OrientationFromDirection(xMat,xNormal);
						
			do
			{
				//do a random LOS test roughly in this direction
				Asura_Vector_3 xBeta;
				xBeta.x = Asura_Random::GetFloat(-.5f,0.5f); 
				xBeta.y = Asura_Random::GetFloat(-.5f,0.5f); 
				xBeta.z = 1.0f;
				xBeta.Normalise();
				xBeta = xBeta * xMat;
				
				int iModule;
				int iJunk;
				float fDistance = ASURA_LOS_MAX_DISTANCE;
				Asura_Hash_ID uZoneHash;
				if(Tools_LOS::FindPolyInLOS(xPosition, xBeta, iJunk, iModule, fDistance, false, &uZoneHash))
				{
					if(!Asura_Module_System::IsModuleLinked(BaseModuleIndex,iModule,uZoneHash))
					{
						SetModuleLink(iModule,uZoneHash,true,TOOLS_MODULELINK_NORMAL);
						if(Asura_Module_System::IsModuleLinked(BaseModuleIndex,iModule,uZoneHash)) iLinked++;
					}
				}

				//update timers
				float fTimeInFrame = Asura_Timers::GetTimeInFrame();
				fTotalTime = fFrameStartTime + fTimeInFrame;

				//Do a new frame if needed
				if(fTimeInFrame > 0.5f)
				{

					Asura_Windows::CheckForWindowsMessages();
					Asura_UserInput::Update();
					Asura_Cursor::OncePerFrameUpdate();

					Asura_Render::BeginFrame();
					//black out the screen
					float fWidth = (float)Asura_Viewport::GetWidth();
					float fHeight = (float)Asura_Viewport::GetHeight();
					Asura_Render::Overlay.SetRectXY(0,0,fWidth,fHeight);
					Asura_Render::Overlay.SetRectUV(0,0,1,1);
					Asura_Render::Overlay.TranslucencyMode = ASURA_TRANSLUCENCY_OFF;
					Asura_Render::Overlay.TextureIndex = ASURA_TEXTURE_NONE;
					Asura_Render::Overlay.Colour=0xff000000;
					Asura_Render::Overlay.Render();
					Asura_Render::FlushDisplay();

					//show some text
					float fPrintX = fWidth/2;
					float fPrintY = fHeight/2;
					Asura_Text_Output::ConvertPixelsToPrintCoords(fPrintX,fPrintY);

					fPrintY -= 2.0f;
					Asura_Text_Output::Print(fPrintX,fPrintY,0xffffffff,ASURA_TEXT_FORMAT_CENTREJUSTIFIED,"Modulelinking");

					fPrintY += 2.0f;
					Asura_Text_Output::Print(fPrintX,fPrintY,0xffff0000,ASURA_TEXT_FORMAT_CENTREJUSTIFIED,"Hold escape to abort");

					int iTotalMin = Asura_Maths::FloatToInt_Chop(fTotalTime)/60;
					int iTotalSec = Asura_Maths::FloatToInt_Chop(fTotalTime)%60;
					
					fPrintY += 2.0f;
					Asura_Text_Output::Print(fPrintX,fPrintY,0xffffffff,ASURA_TEXT_FORMAT_CENTREJUSTIFIED,"Time taken : %0d:%02d",iTotalMin,iTotalSec);
					
					fPrintY += 2.0f;
					Asura_Text_Output::Print(fPrintX,fPrintY,0xffffffff,ASURA_TEXT_FORMAT_CENTREJUSTIFIED,"Desired Time : %0d:%02d",iDesiredMin,iDesiredSec);

					fPrintY += 2.0f;
					Asura_Text_Output::Print(fPrintX,fPrintY,0xffffffff,ASURA_TEXT_FORMAT_CENTREJUSTIFIED,"Links Added : %d",iLinked);
															

					Tools_Timed_Message_System::Render();
					Asura_Cursor::Render();

					
					Asura_Render::EndFrame();

					Asura_Timers::HandlePrimaryTimer();

					if (Asura_UserInput::KeyPressed(ASURA_KEYID_ESCAPE))
					{
						bQuit = true;
						if(Tools_BatchProcessing::IsBatch())
						{
							Tools_BatchProcessing::Abort();
						}
					}

					fFrameStartTime = Asura_Timers::GetUncappedPhase()-fStartTime;
					
				}
	
			
			}while(fTotalTime<fTimeForNextLink && !bQuit);
		}
	}
	Tools_LOS::SetIgnoreModuleLinking(false);

}
