//////////////////////////////////////////////////////////////////////////////
//
// Tools_Object_Axon_Interface.cpp
//
// Copyright (C) 2008 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"

#ifdef ASURA_USE_AXON

// header
#include "Objects/Axon/Tools_Object_Axon_Interface.h"

// asura
#include "Entity/Locations/OrientedNode/GamesceneNode/Asura_ServerNode_GS2Node.h"
#include "Entity/Locations/OrientedNode/GamesceneNode/AttractorNode/Asura_ServerNode_GS2Attractor.h"
#include "Entity/Locations/Spline/GamesceneSpline/Asura_ServerNode_GS2Spline.h"

// axon
#include "AnimTemplates/Axon_Gamescene_AnimTemplate_System.h"
#include "Behaviour/Axon_BehaviourParams.h"
#include "Behaviour/BehaviourTypes/Axon_Behaviour_Types.h"

// tools
#include "AnimTemplates/Tools_Mode_AnimTemplates.h"
#include "Entity/Tools_Entity_Object.h"
#include "Entity/Tools_SecondaryEntityFile_System.h"
#include "Entity/Locations/OrientedNode/GamesceneNode/Tools_EntityNode_GS2Node.h"
#include "Entity/Locations/OrientedNode/GamesceneNode/AtttactorNode/Tools_EntityNode_GS2Attractor.h"
#include "Entity/Locations/Spline/GamesceneSpline/Tools_EntityNode_GS2Spline.h"
#include "Load&Save/Tools_ModeDataFile.h"

//////////////////////////////////////////////////////////////////////////////
//
// Tools_Object_Axon_Interface
//
//////////////////////////////////////////////////////////////////////////////

bool       Tools_Object_Axon_Interface::s_bPreviewInProgress = false;
bool       Tools_Object_Axon_Interface::s_bPreviewStopping   = false;
int        Tools_Object_Axon_Interface::s_iNumPreviewing     = 0;
bool	   Tools_Object_Axon_Interface::s_bRefreshPreviewSettingsOverlay = false;
Asura_Vector_2 Tools_Object_Axon_Interface::s_xPreviewSettingsOverlayPosition( 5.0f, 5.0f );
bool       Tools_Object_Axon_Interface::s_bPreviewPropagate  = false;
bool       Tools_Object_Axon_Interface::s_bPreviewLoop       = false;
Asura_LinkList< Asura_Guid > Tools_Object_Axon_Interface::s_xPreviewInitialObjects;

Tools_Object_Axon_Interface* Tools_Object_Axon_Interface::s_pxThis = NULL;

void Tools_Object_Axon_Interface::Initialise()
{
	s_pxThis = Project_CreateAxonInterface();
	s_pxThis->InitialiseMode();
}

void Tools_Object_Axon_Interface::Shutdown()
{
	s_pxThis->ShutdownMode();
	delete s_pxThis;
	s_pxThis = NULL;
}

void Tools_Object_Axon_Interface::TransferModeData( Tools_ModeDataFile& xFile )
{
	int iVersion = CURRENT_TRANSFER_VERSION;
	xFile.Transfer( iVersion );

	if ( iVersion == CURRENT_TRANSFER_VERSION )
	{
		Tools_EntityNode_GS2Attractor::TransferToolsModeData( xFile );
		Tools_EntityNode_GS2Node::TransferToolsModeData( xFile );
		Tools_EntityNode_GS2Spline::TransferToolsModeData( xFile );

		xFile.Transfer( s_xPreviewSettingsOverlayPosition );
		xFile.Transfer( s_bPreviewPropagate );
		xFile.Transfer( s_bPreviewLoop );
	}
	else if ( iVersion == 4 )
	{
		Tools_EntityNode_GS2Attractor::TransferToolsModeData( xFile );
		Tools_EntityNode_GS2Node::TransferToolsModeData( xFile );
		Tools_EntityNode_GS2Spline::TransferToolsModeData( xFile );
	}
	else if ( iVersion == 3 )
	{
		Tools_EntityNode_GS2Attractor::TransferToolsModeData( xFile );
		Tools_EntityNode_GS2Node::TransferToolsModeData( xFile );
		Tools_EntityNode_GS2Spline::TransferToolsModeData( xFile );

		bool bDummy;
		xFile.Transfer( bDummy );
		TransferOldOverlayData( xFile );
	}
	else if ( iVersion == 2 )
	{
		Tools_EntityNode_GS2Attractor::TransferToolsModeData( xFile );
		Tools_EntityNode_GS2Node::TransferToolsModeData( xFile );

		bool bDummy;
		xFile.Transfer( bDummy );
		TransferOldOverlayData( xFile );
	}
	else if ( iVersion == 1 )
	{
		Tools_EntityNode_GS2Node::TransferToolsModeData( xFile );

		bool bDummy;
		xFile.Transfer( bDummy );
		TransferOldOverlayData( xFile );
	}
	else if ( iVersion == 0 )
	{
		bool bDummy;
		xFile.Transfer( bDummy );
		TransferOldOverlayData( xFile );
	}
}

void Tools_Object_Axon_Interface::Update()
{
	if ( s_bRefreshPreviewSettingsOverlay )
	{
		s_bRefreshPreviewSettingsOverlay = false;

		if ( PreviewSettingsOverlayOpen() )
		{
			ClosePreviewSettingsOverlay();
			OpenPreviewSettingsOverlay();
		}
	}
}

void Tools_Object_Axon_Interface::ShutdownMode()
{
	if ( PreviewSettingsOverlayOpen() )
	{
		ClosePreviewSettingsOverlay();
	}
}

void Tools_Object_Axon_Interface::BuildGS2StructureList( ListSelection_Overlay& xList, const Tools_EntityNode_GS2Node* pxGS2Node /*= NULL*/ )
{
	static struct StringAndValue
	{
		const char* m_pszString;
		int			m_iValue;
	} s_axStructures[] =
	{
		{ "*none*", AXON_GS2_ST_NONE },
		{ "", -1 } // terminator
	};

	StringAndValue* pxEntry = s_axStructures;
	while ( pxEntry->m_iValue != -1 )
	{
		xList.AddItem( pxEntry->m_pszString, pxEntry->m_iValue );
		++pxEntry;
	}
}

void Tools_Object_Axon_Interface::BuildGS2NodeBehaviourSetList( ListSelection_Overlay& xList, const Tools_EntityNode_GS2Node* pxGS2Node /*= NULL*/ )
{
	static struct StringAndValue
	{
		const char* m_pszString;
		int			m_iValue;
	} s_axNodeBehaviours[] =
	{
		{ "*none*", AXON_BEHAVIOUR_TYPE_UNSET },
		{ "Core Node Behaviour", AXON_BS_ID_CORE_NODE },
		{ "", -1 } // terminator
	};

	StringAndValue* pxEntry = s_axNodeBehaviours;
	while ( pxEntry->m_iValue != -1 )
	{
		xList.AddItem( pxEntry->m_pszString, pxEntry->m_iValue );
		++pxEntry;
	}
}

void Tools_Object_Axon_Interface::BuildGS2SplineBehaviourSetList( ListSelection_Overlay& xList,
																  const Tools_EntityNode_GS2Spline* pxGS2Spline /*= NULL*/ )
{
	static struct StringAndValue
	{
		const char* m_pszString;
		int			m_iValue;
	} s_axSplineBehaviours[] =
	{
		{ "*none*", AXON_BEHAVIOUR_TYPE_UNSET },
		{ "Core Spline Behaviour", AXON_BS_ID_CORE_SPLINE },
		{ "", -1 } // terminator
	};

	StringAndValue* pxEntry = s_axSplineBehaviours;
	while ( pxEntry->m_iValue != -1 )
	{
		xList.AddItem( pxEntry->m_pszString, pxEntry->m_iValue );
		++pxEntry;
	}
}

void Tools_Object_Axon_Interface::BuildGS2AttractorAttractTypesList( ListSelection_Overlay& xList,
																	 const Tools_EntityNode_GS2Attractor* pxGS2Attractor /*= NULL*/ )
{
	static struct StringAndValue
	{
		const char* m_pszString;
		int			m_iValue;
	} s_axAttractTypes[] =
	{
		{ "Anything", AXON_GS2_AAT_ANYTHING },
		{ "Specific Entity", AXON_GS2_AAT_SPECIFIC_GUID },
		{ "", -1 } // terminator
	};

	StringAndValue* pxEntry = s_axAttractTypes;
	while ( pxEntry->m_iValue != -1 )
	{
		xList.AddItem( pxEntry->m_pszString, pxEntry->m_iValue );
		++pxEntry;
	}
}

void Tools_Object_Axon_Interface::BuildGS2AttractorEnRouteBehaviourSetList( ListSelection_Overlay& xList,
																			const Tools_EntityNode_GS2Attractor* pxGS2Attractor /*= NULL*/ )
{
	static struct StringAndValue
	{
		const char* m_pszString;
		int			m_iValue;
	} s_axEnRouteBehaviours[] =
	{
		{ "*none*", AXON_BEHAVIOUR_TYPE_UNSET },
		{ "Core En-Route Behaviour", AXON_BS_ID_CORE_ATTRACTOR_EN_ROUTE },
		{ "", -1 } // terminator
	};

	StringAndValue* pxEntry = s_axEnRouteBehaviours;
	while ( pxEntry->m_iValue != -1 )
	{
		xList.AddItem( pxEntry->m_pszString, pxEntry->m_iValue );
		++pxEntry;
	}
}

bool Tools_Object_Axon_Interface::GetAnimSetTotalMovementOffset( const u_int uAnimSet, const Asura_String xSkinName, Asura_Vector_3& xPositionOffset, Asura_Quat& xQOrientOffset )
{
	const Axon_Gamescene_AnimTemplate* pxAnimationTemplate = Axon_Gamescene_AnimTemplate_System::GetAnimTemplateByID( uAnimSet );
	if ( !pxAnimationTemplate ) { return false; }

	// Make sure the anims are loaded in.
	pxAnimationTemplate->LoadAnims();

	Asura_Matrix_3x3 xOrientOffset;
	pxAnimationTemplate->GetMovementOffset( Asura_Maths::LargeFloat, NULL, xPositionOffset, xOrientOffset );
	xQOrientOffset.FromMatrix( xOrientOffset );
	return true;
}

void Tools_Object_Axon_Interface::EstablishGS2ObjectOwnerData()
{
	// Reset things first.
	// In order to try and preserve the skin names of anonymous GS2 objects that the user has set previously we only clear
	// the owner guids and skins of objects that have an owner. Objects that are owned will have their data established again,
	// newly-anonymous objects will remain blank and previously anonymous objects with skins set will remain unaffected.
	for ( Asura_Hashed_List_It< Tools_Entity_Node > xNodeIter( Tools_Entity_Node::GetList() ); !xNodeIter.Done(); xNodeIter.Next() )
	{
		Tools_Entity_Node* pxNode = xNodeIter.GetCurrent();
		if ( pxNode->IsBaseOrDerivativeOf( TOT_Entity_GS2Node ) )
		{
			Tools_EntityNode_GS2Node* pxGS2Node = static_cast< Tools_EntityNode_GS2Node* >( pxNode );
			if ( pxGS2Node->GetOwnerGuid() != ASURA_GUID_UNREGISTERED )
			{
				pxGS2Node->SetOwnerGuid( ASURA_GUID_UNREGISTERED );
				pxGS2Node->SetOwnerSkinName( "" );
			}
		}
		else if ( pxNode->IsBaseOrDerivativeOf( TOT_Entity_GS2Spline ) )
		{
			Tools_EntityNode_GS2Spline* pxGS2Spline = static_cast< Tools_EntityNode_GS2Spline* >( pxNode );
			if ( pxGS2Spline->GetOwnerGuid() != ASURA_GUID_UNREGISTERED )
			{
				pxGS2Spline->SetOwnerGuid( ASURA_GUID_UNREGISTERED );
				pxGS2Spline->SetOwnerSkinName( "" );
			}
		}
	}

	// Starting from nodes that are owned by actors, traverse the trees and set owner guids.
	ForEachGS2StartNode( SetGS2StructureOwnerData );

	// Traverse the list of attractors also and see if we can determine the owner guid from the attractor settings.
	for ( Asura_Hashed_List_It< Tools_EntityNode_GS2Attractor > xAttrIter( Tools_EntityNode_GS2Attractor::GetList() ); !xAttrIter.Done(); xAttrIter.Next() )
	{
		Tools_EntityNode_GS2Attractor* pxAttr = xAttrIter.GetCurrent();
		if ( pxAttr->GetOwnerGuid() == ASURA_GUID_UNREGISTERED )
		{
			SetGS2StructureOwnerData( Asura_ServerEntity_Node::FindEntity( pxAttr->GetGuid() ), pxAttr->GetOwnerGuidFromSettings() );
		}
	}
}

void Tools_Object_Axon_Interface::UpdateBehaviourTrees()
{
	for ( Asura_Hashed_List_It< Tools_EntityNode_GS2Node > xNodeIter( Tools_EntityNode_GS2Node::GetList() ); !xNodeIter.Done(); xNodeIter.Next() )
	{
		Tools_EntityNode_GS2Node* pxNode = xNodeIter.GetCurrent();
		pxNode->UpdateBehaviourTree();
	}
	for ( Asura_Hashed_List_It< Tools_EntityNode_GS2Spline > xSplineIter( Tools_EntityNode_GS2Spline::GetList() ); !xSplineIter.Done(); xSplineIter.Next() )
	{
		Tools_EntityNode_GS2Spline* pxSpline = xSplineIter.GetCurrent();
		pxSpline->UpdateBehaviourTree();
	}
}

bool Tools_Object_Axon_Interface::CanCreateGS2LinkBetweenNodes()
{
	if ( Tools_Object_System::SelectedObjectList.size() != 2 ) { return false; }

	const Asura_Guid uSourceGuid = Tools_Object_System::SelectedObjectList.first_entry()->GetGuid();
	const Asura_Guid uTargetGuid = Tools_Object_System::SelectedObjectList.last_entry()->GetGuid();

	const Tools_EntityNode_GS2Node* pxSourceGS2Node = Tools_EntityNode_GS2Node::FindEntity( uSourceGuid );
	if ( !pxSourceGS2Node ) { return false; }
	if ( !Tools_EntityNode_GS2Node::FindEntity( uTargetGuid ) ) { return false; }

	for ( u_int uLink = 0; uLink < pxSourceGS2Node->GetGameNode()->GetNumberLinks(); ++uLink )
	{
		const Tools_Entity_Node* pxLinkNode = Tools_Entity_Node::FindEntity( pxSourceGS2Node->GetGameNode()->GetLink( uLink ) );
		if ( !pxLinkNode || !pxLinkNode->IsBaseOrDerivativeOf( TOT_Entity_GS2Spline ) ) { return false; }

		const Tools_EntityNode_GS2Spline* pxLink = static_cast< const Tools_EntityNode_GS2Spline* >( pxLinkNode );
		if ( pxLink->GetGameSpline()->GetTargetGuid() == uTargetGuid ) { return false; }
	}

	return true;
}

bool Tools_Object_Axon_Interface::CanCreateGS2LinkedNode()
{
	if ( Tools_Object_System::SelectedObjectList.size() == 1 )
	{
		return ( Tools_EntityNode_GS2Node::FindEntity( Tools_Object_System::SelectedObjectList.first_entry()->GetGuid() ) != NULL );
	}
	return false;
}

Tools_EntityNode_GS2Node* Tools_Object_Axon_Interface::CreateGS2Node()
{
	Tools_SecondaryEntityFile_System::PreCreateObject();
	Tools_EntityNode_GS2Node* pxGS2Node = Tools_EntityNode_GS2Node::CreateAndReturn();
	Tools_SecondaryEntityFile_System::PostCreateObject();
	return pxGS2Node;
}

Tools_EntityNode_GS2Spline* Tools_Object_Axon_Interface::CreateGS2Link()
{
	Tools_SecondaryEntityFile_System::PreCreateObject();
	Tools_EntityNode_GS2Spline* pxGS2Spline = Tools_EntityNode_GS2Spline::Create();
	Tools_SecondaryEntityFile_System::PostCreateObject();
	return pxGS2Spline;
}

void Tools_Object_Axon_Interface::CreateGS2LinkBetweenNodes()
{
	Tools_EntityNode_GS2Node* pxSourceNode = static_cast< Tools_EntityNode_GS2Node* >( Tools_Object_System::SelectedObjectList.first_entry() );
	Tools_EntityNode_GS2Node* pxTargetNode = static_cast< Tools_EntityNode_GS2Node* >( Tools_Object_System::SelectedObjectList.last_entry() );

	Tools_EntityNode_GS2Spline* pxGS2Spline = CreateGS2Link();

	// Give the newly create spline the same colour as the source node.
	if ( pxGS2Spline )
	{
		pxGS2Spline->SetColour( pxSourceNode->GetColour() );
	}

	// Keep the owner data up-to-date if possible.
	if ( pxSourceNode->GetOwnerGuid() != ASURA_GUID_UNREGISTERED )
	{
		SetGS2StructureOwnerData( pxSourceNode->GetGameNode(), pxSourceNode->GetOwnerGuid() );
	}
	else if ( pxTargetNode->GetOwnerGuid() != ASURA_GUID_UNREGISTERED )
	{
		SetGS2StructureOwnerData( pxTargetNode->GetGameNode(), pxTargetNode->GetOwnerGuid() );
	}
}

void Tools_Object_Axon_Interface::CreateGS2LinkedNode()
{
	Tools_EntityNode_GS2Node* pxSourceNode = static_cast< Tools_EntityNode_GS2Node* >( Tools_Object_System::SelectedObjectList.first_entry() );
	CreateGS2Node();
	Tools_EntityNode_GS2Node* pxTargetNode = static_cast< Tools_EntityNode_GS2Node* >( Tools_Object_System::SelectedObjectList.first_entry() );

	Tools_Object_System::SelectObject( pxSourceNode );
	Tools_Object_System::SelectObject( pxTargetNode, true );
	CreateGS2Link();
	Tools_Object_System::SelectObject( pxTargetNode );

	// Update splines in case any should 'smooth over node'.
	pxSourceNode->UpdateAdjacentSplines();
	pxTargetNode->UpdateAdjacentSplines();

	// Give the newly created node the same colour as the source node (the spline will already have been coloured).
	pxTargetNode->SetColour( pxSourceNode->GetColour() );

	// Keep the owner data up-to-date if possible.
	if ( pxSourceNode->GetOwnerGuid() != ASURA_GUID_UNREGISTERED )
	{
		SetGS2StructureOwnerData( pxSourceNode->GetGameNode(), pxSourceNode->GetOwnerGuid() );
	}
	else if ( pxTargetNode->GetOwnerGuid() != ASURA_GUID_UNREGISTERED )
	{
		SetGS2StructureOwnerData( pxTargetNode->GetGameNode(), pxTargetNode->GetOwnerGuid() );
	}
}

void Tools_Object_Axon_Interface::AddPreviewControls( Asura_Overlay* pxNewOverlay, Asura_Vector_2& xOffset )
{
	Asura_Dialog* pxDialog = new Asura_Dialog( "Preview:", xOffset );
	pxNewOverlay->AddDialog( pxDialog );
	xOffset.x += 6.0f;

	Asura_Function_Dialog* pxFDialog = new Asura_Function_Dialog( "Start", xOffset, StartPreview, NULL, CanStartPreview );
	pxFDialog->SetUseSharedDialogColour( true );
	pxNewOverlay->AddDialog( pxFDialog );
	xOffset.x += 6.0f;

	pxDialog = new Asura_Function_Dialog( "Stop", xOffset, StopPreview, NULL, CanStopPreview );
	pxNewOverlay->AddDialog( pxDialog );
	xOffset.x += 6.0f;

	pxDialog = new Asura_Function_Dialog( "Settings", xOffset, OpenPreviewSettingsOverlay, NULL, PreviewSettingsOverlayNotOpen );
	pxNewOverlay->AddDialog( pxDialog );
	xOffset.x -= 18.0f;
	xOffset.y += Asura_Dialog::DefaultSpacing();
}

void Tools_Object_Axon_Interface::SetGS2StructureOwnerData( Asura_ServerEntity_Node* pxStartNode, const Asura_Guid uOwnerGuid )
{
	GS2OwnerData xOwnerData;
	xOwnerData.m_uGuid     = uOwnerGuid;
	xOwnerData.m_xSkinName = GetInstance()->GetSkinNameFromGuid( uOwnerGuid );

	if ( pxStartNode )
	{
		Asura_ServerEntity_Node::TraverseStructure( pxStartNode->GetGuid(), SetGS2NodeOwnerData, reinterpret_cast< void* >( &xOwnerData ) );
	}
}

void Tools_Object_Axon_Interface::SetGS2NodeOwnerData( Asura_ServerEntity_Node* pxNode, void* pUserData )
{
	const GS2OwnerData* pxOwnerData = reinterpret_cast< const GS2OwnerData* >( pUserData );

	Tools_Entity_Node* pxToolsNode = Tools_Entity_Node::FindEntity( pxNode->GetGuid() );
	if ( !pxToolsNode ) { return; }

	if ( pxToolsNode->IsBaseOrDerivativeOf( TOT_Entity_GS2Node ) )
	{
		Tools_EntityNode_GS2Node* pxGS2Node = static_cast< Tools_EntityNode_GS2Node* >( pxToolsNode );
		if ( pxGS2Node->GetOwnerGuid() != pxOwnerData->m_uGuid )
		{
			pxGS2Node->SetOwnerGuid( pxOwnerData->m_uGuid );
			pxGS2Node->SetOwnerSkinName( pxOwnerData->m_xSkinName );
		}
	}
	else if ( pxToolsNode->IsBaseOrDerivativeOf( TOT_Entity_GS2Spline ) )
	{
		Tools_EntityNode_GS2Spline* pxGS2Spline = static_cast< Tools_EntityNode_GS2Spline* >( pxToolsNode );
		if ( pxGS2Spline->GetOwnerGuid() != pxOwnerData->m_uGuid )
		{
			pxGS2Spline->SetOwnerGuid( pxOwnerData->m_uGuid );
			pxGS2Spline->SetOwnerSkinName( pxOwnerData->m_xSkinName );
		}
	}
}

void Tools_Object_Axon_Interface::SetGS2NodeOwnerSkinName( Asura_ServerEntity_Node* pxNode, void* pUserData )
{
	Tools_Entity_Node* pxToolsNode = Tools_Entity_Node::FindEntity( pxNode->GetGuid() );
	if ( !pxToolsNode ) { return; }

	if ( pxToolsNode->IsBaseOrDerivativeOf( TOT_Entity_GS2Node ) )
	{
		Tools_EntityNode_GS2Node* pxGS2Node = static_cast< Tools_EntityNode_GS2Node* >( pxToolsNode );
		pxGS2Node->SetOwnerSkinName( *reinterpret_cast< Asura_String* >( pUserData ) );
	}
	else if ( pxToolsNode->IsBaseOrDerivativeOf( TOT_Entity_GS2Spline ) )
	{
		Tools_EntityNode_GS2Spline* pxGS2Spline = static_cast< Tools_EntityNode_GS2Spline* >( pxToolsNode );
		pxGS2Spline->SetOwnerSkinName( *reinterpret_cast< Asura_String* >( pUserData ) );
	}
}

bool Tools_Object_Axon_Interface::CanStartPreview()
{
	if ( !Tools_Object_Axon_Interface::GetInstance() )
	{
		return false;
	}

	if ( s_bPreviewInProgress )
	{
		return false;
	}

	if ( !Tools_Object_System::SelectedObject || Tools_Object_System::SelectedObjectList.size() == 0 )
	{
		return false;
	}

	return true;
}

bool Tools_Object_Axon_Interface::CanStopPreview()
{
	return s_bPreviewInProgress;
}

void Tools_Object_Axon_Interface::StartPreview()
{
	// Establish the owner guids and skin names, so anims can preview correctly.
	if ( Tools_Object_Axon_Interface::GetInstance() )
	{
		Tools_Object_Axon_Interface::GetInstance()->EstablishGS2ObjectOwnerData();
	}

	// If the skin name is still not set then prompt the user for it and propagate it over the entire structure.
	bool bCancelled = false;
	for ( CLIF< Tools_Object* > xSelectedIter( &Tools_Object_System::SelectedObjectList ); !xSelectedIter.done() && !bCancelled; xSelectedIter.next() )
	{
		Tools_EntityNode_GS2Node* pxGS2Node = Tools_EntityNode_GS2Node::FindEntity( xSelectedIter()->GetGuid() );
		if ( pxGS2Node )
		{
			if ( pxGS2Node->GetOwnerSkinName().IsBlank() )
			{
				if ( pxGS2Node->PromptForOwnerSkinName() )
				{
					Asura_ServerEntity_Node::TraverseStructure( pxGS2Node->GetGuid(),
						SetGS2NodeOwnerSkinName,
						reinterpret_cast< void* >( const_cast< Asura_String* >( &pxGS2Node->GetOwnerSkinName() ) ) );
				}
				else
				{
					bCancelled = true;
				}
			}

			continue;
		}

		Tools_EntityNode_GS2Spline* pxGS2Spline = Tools_EntityNode_GS2Spline::FindEntity( xSelectedIter()->GetGuid() );
		if ( pxGS2Spline )
		{
			if ( pxGS2Spline->GetOwnerSkinName().IsBlank() )
			{
				if ( pxGS2Spline->PromptForOwnerSkinName() )
				{
					Asura_ServerEntity_Node::TraverseStructure( pxGS2Spline->GetGuid(),
						SetGS2NodeOwnerSkinName,
						reinterpret_cast< void* >( const_cast< Asura_String* >( &pxGS2Spline->GetOwnerSkinName() ) ) );
				}
				else
				{
					bCancelled = true;
				}
			}
		}
	}

	// Start the preview on all of the selected objects.
	for ( CLIF< Tools_Object* > xSelectedIter( &Tools_Object_System::SelectedObjectList ); !xSelectedIter.done() && !bCancelled; xSelectedIter.next() )
	{
		Tools_EntityNode_GS2Node* pxGS2Node = Tools_EntityNode_GS2Node::FindEntity( xSelectedIter()->GetGuid() );
		if ( pxGS2Node )
		{
			pxGS2Node->StartPreview();
			s_xPreviewInitialObjects.Push_Back( pxGS2Node->GetGuid() );
			continue;
		}

		Tools_EntityNode_GS2Spline* pxGS2Spline = Tools_EntityNode_GS2Spline::FindEntity( xSelectedIter()->GetGuid() );
		if ( pxGS2Spline )
		{
			pxGS2Spline->StartPreview();
			s_xPreviewInitialObjects.Push_Back( pxGS2Spline->GetGuid() );
		}
	}

	s_bPreviewInProgress = ( s_xPreviewInitialObjects.Size() > 0 );
}

void Tools_Object_Axon_Interface::StopPreview()
{
	s_bPreviewStopping = true;
	for ( Asura_Hashed_List_It< Tools_EntityNode_GS2Node > xNodeIter( Tools_EntityNode_GS2Node::GetList() ); !xNodeIter.Done(); xNodeIter.Next() )
	{
		xNodeIter.GetCurrent()->StopPreview();
	}
	for ( Asura_Hashed_List_It< Tools_EntityNode_GS2Spline > xSplineIter( Tools_EntityNode_GS2Spline::GetList() ); !xSplineIter.Done(); xSplineIter.Next() )
	{
		xSplineIter.GetCurrent()->StopPreview();
	}
	s_bPreviewStopping = false;
}

void Tools_Object_Axon_Interface::DecrementNumPreviewing()
{
	--s_iNumPreviewing;
	if ( s_iNumPreviewing < 0 )
	{
		Asura_Assert( false, "Tools_Object_Axon_Interface::DecrementNumPreviewing - under-ran num previewing" );
		s_iNumPreviewing = 0;
	}
	if ( s_iNumPreviewing == 0 )
	{
		PreviewFinished();
	}
}

void Tools_Object_Axon_Interface::PreviewFinished()
{
	if ( s_bPreviewLoop && !s_bPreviewStopping )
	{
		for ( Asura_LinkList_ConstIt< Asura_Guid > xInitialIter( s_xPreviewInitialObjects ); !xInitialIter.Done(); xInitialIter.Next() )
		{
			Tools_EntityNode_GS2Node* pxGS2Node = Tools_EntityNode_GS2Node::FindEntity( *xInitialIter.GetCurrent() );
			if ( pxGS2Node )
			{
				pxGS2Node->StartPreview();
				continue;
			}

			Tools_EntityNode_GS2Spline* pxGS2Spline = Tools_EntityNode_GS2Spline::FindEntity( *xInitialIter.GetCurrent() );
			if ( pxGS2Spline )
			{
				pxGS2Spline->StartPreview();
			}
		}
	}
	else
	{
        s_bPreviewInProgress = false;
		s_xPreviewInitialObjects.Clear();
	}
}

bool Tools_Object_Axon_Interface::PreviewSettingsOverlayOpen()
{
	return Asura_Overlay_System::DoesOverlayExist( GetPreviewSettingsOverlayID() );
}

void Tools_Object_Axon_Interface::OpenPreviewSettingsOverlay()
{
	Asura_Overlay* pxOverlay = new Asura_Overlay( "GS2 Preview Settings", false, &s_xPreviewSettingsOverlayPosition );
	Asura_Vector_2 xOffset = Asura_Vector_2( 1.0f, 3.0f );
	s_pxThis->AddPreviewSettingsControls( pxOverlay, xOffset );

	pxOverlay->m_uID = GetPreviewSettingsOverlayID();
	Asura_Overlay_System::SubmitNewOverlay( pxOverlay, true );
}

void Tools_Object_Axon_Interface::ClosePreviewSettingsOverlay()
{
	Asura_Overlay_System::CloseOverlays( GetPreviewSettingsOverlayID() );
}

void Tools_Object_Axon_Interface::AddPreviewSettingsControls( Asura_Overlay* pxNewOverlay, Asura_Vector_2& xOffset )
{
	Asura_Dialog* pxDialog = new Asura_Tickbox_Dialog( "Loop", xOffset, &s_bPreviewLoop );
	pxDialog->SetIsEnabledFunction( NoPreviewInProgress );
	pxDialog->SetHelpText( "Whether the preview should loop when it finishes." );
	pxNewOverlay->AddDialog( pxDialog );
	xOffset.x += 8.0f;

	pxDialog = new Asura_Tickbox_Dialog( "Propagate", xOffset, &s_bPreviewPropagate );
	pxDialog->SetIsEnabledFunction( NoPreviewInProgress );
	pxDialog->SetHelpText( "Whether the preview should propagate along links." );
	pxNewOverlay->AddDialog( pxDialog );
	xOffset.x -= 8.0f;
	xOffset.y += Asura_Dialog::DefaultSpacing();
}

void Tools_Object_Axon_Interface::TransferOldOverlayData( Tools_ModeDataFile& xFile )
{
	int iVersion = 4;
	xFile.Transfer( iVersion );

	if ( iVersion == 4 )
	{
		Asura_Vector_2 xDummy;
		bool bDummy;
		u_int uDummy;
		int iDummy;
		xFile.Transfer(xDummy);
		xFile.Transfer(bDummy);
		xFile.Transfer(bDummy);
		xFile.Transfer(bDummy);
		xFile.Transfer(bDummy);
		xFile.Transfer(uDummy);
		xFile.Transfer(uDummy);
		xFile.Transfer(bDummy);
		xFile.Transfer(bDummy);
		xFile.Transfer(bDummy);
		xFile.Transfer(bDummy);
		xFile.Transfer(bDummy);
		xFile.Transfer(bDummy);
		xFile.Transfer(bDummy);
		xFile.Transfer(bDummy);
		xFile.Transfer(bDummy);
		xFile.Transfer(bDummy);
		xFile.Transfer(bDummy);
		xFile.Transfer(iDummy);
		xFile.Transfer(bDummy);
	}
	else if ( iVersion == 3 )
	{
		Asura_Vector_2 xDummy;
		bool bDummy;
		u_int uDummy;
		int iDummy;
		xFile.Transfer(xDummy);
		xFile.Transfer(bDummy);
		xFile.Transfer(bDummy);
		xFile.Transfer(bDummy);
		xFile.Transfer(bDummy);
		xFile.Transfer(bDummy);
		xFile.Transfer(uDummy);
		xFile.Transfer(uDummy);
		xFile.Transfer(bDummy);
		xFile.Transfer(bDummy);
		xFile.Transfer(bDummy);
		xFile.Transfer(bDummy);
		xFile.Transfer(bDummy);
		xFile.Transfer(bDummy);
		xFile.Transfer(bDummy);
		xFile.Transfer(bDummy);
		xFile.Transfer(bDummy);
		xFile.Transfer(bDummy);
		xFile.Transfer(bDummy);
		xFile.Transfer(iDummy);
		xFile.Transfer(bDummy);
	}
	else if ( iVersion == 2 )
	{
		Asura_Vector_2 xDummy;
		bool bDummy;
		u_int uDummy;
		int iDummy;
		xFile.Transfer(xDummy);
		xFile.Transfer(bDummy);
		xFile.Transfer(bDummy);
		xFile.Transfer(bDummy);
		xFile.Transfer(bDummy);
		xFile.Transfer(bDummy);
		xFile.Transfer(uDummy);
		xFile.Transfer(uDummy);
		xFile.Transfer(bDummy);
		xFile.Transfer(bDummy);
		xFile.Transfer(bDummy);
		xFile.Transfer(bDummy);
		xFile.Transfer(bDummy);
		xFile.Transfer(bDummy);
		xFile.Transfer(bDummy);
		xFile.Transfer(bDummy);
		xFile.Transfer(bDummy);
		xFile.Transfer(bDummy);
		xFile.Transfer(bDummy);
		xFile.Transfer(bDummy);
		xFile.Transfer(bDummy);
		xFile.Transfer(bDummy);
		xFile.Transfer(iDummy);
		xFile.Transfer(bDummy);
	}
	else if ( iVersion == 1 )
	{
		Asura_Vector_2 xDummy;
		bool bDummy;
		u_int uDummy;
		int iDummy;
		xFile.Transfer(xDummy);
		xFile.Transfer(bDummy);
		xFile.Transfer(bDummy);
		xFile.Transfer(bDummy);
		xFile.Transfer(bDummy);
		xFile.Transfer(bDummy);
		xFile.Transfer(bDummy);
		xFile.Transfer(uDummy);
		xFile.Transfer(uDummy);
		xFile.Transfer(bDummy);
		xFile.Transfer(bDummy);
		xFile.Transfer(bDummy);
		xFile.Transfer(bDummy);
		xFile.Transfer(bDummy);
		xFile.Transfer(bDummy);
		xFile.Transfer(bDummy);
		xFile.Transfer(bDummy);
		xFile.Transfer(bDummy);
		xFile.Transfer(bDummy);
		xFile.Transfer(bDummy);
		xFile.Transfer(bDummy);
		xFile.Transfer(bDummy);
		xFile.Transfer(bDummy);
		xFile.Transfer(iDummy);
		xFile.Transfer(bDummy);
	}
	else if ( iVersion == 0 )
	{
		Asura_Vector_2 xDummy;
		bool bDummy;
		u_int uDummy;
		int iDummy;
		xFile.Transfer(xDummy);
		xFile.Transfer(bDummy);
		xFile.Transfer(bDummy);
		xFile.Transfer(bDummy);
		xFile.Transfer(bDummy);
		xFile.Transfer(bDummy);
		xFile.Transfer(bDummy);
		xFile.Transfer(bDummy);
		xFile.Transfer(uDummy);
		xFile.Transfer(uDummy);
		xFile.Transfer(bDummy);
		xFile.Transfer(bDummy);
		xFile.Transfer(bDummy);
		xFile.Transfer(bDummy);
		xFile.Transfer(bDummy);
		xFile.Transfer(bDummy);
		xFile.Transfer(bDummy);
		xFile.Transfer(bDummy);
		xFile.Transfer(bDummy);
		xFile.Transfer(bDummy);
		xFile.Transfer(bDummy);
		xFile.Transfer(bDummy);
		xFile.Transfer(bDummy);
		xFile.Transfer(bDummy);
		xFile.Transfer(iDummy);
	}
}

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

#endif // ASURA_USE_AXON
